/**! 
 * @wcj/markdown-to-html v1.0.0 
 * Converts markdown text to HTML. 
 * 
 * Copyright (c) 2022 kenny wang <wowohoo@qq.com> (https://github.com/jaywcjlove) 
 * https://github.com/jaywcjlove/markdown-to-html 
 * Licensed under the MIT license 
 */

 (function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.markdown = {}));
})(this, (function (exports) { 'use strict';

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation.

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */

  var __assign = function() {
      __assign = Object.assign || function __assign(t) {
          for (var s, i = 1, n = arguments.length; i < n; i++) {
              s = arguments[i];
              for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
          }
          return t;
      };
      return __assign.apply(this, arguments);
  };

  var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

  function getDefaultExportFromCjs (x) {
    return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
  }

  /*!
   * Determine if an object is a Buffer
   *
   * @author   Feross Aboukhadijeh <https://feross.org>
   * @license  MIT
   */

  var isBuffer$1 = function isBuffer (obj) {
    return obj != null && obj.constructor != null &&
      typeof obj.constructor.isBuffer === 'function' && obj.constructor.isBuffer(obj)
  };

  /**
   * @typedef {import('unist').Point} Point
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Position} Position
   * @typedef {object & {type: string, position?: Position|undefined}} NodeLike
   */

  /**
   * Stringify one point, a position (start and end points), or a node’s
   * positional information.
   *
   * @param {Node|NodeLike|Position|Point|null} [value]
   * @returns {string}
   */
  function stringifyPosition$1(value) {
    // Nothing.
    if (!value || typeof value !== 'object') {
      return ''
    }

    // Node.
    if ('position' in value || 'type' in value) {
      return position$3(value.position)
    }

    // Position.
    if ('start' in value || 'end' in value) {
      return position$3(value)
    }

    // Point.
    if ('line' in value || 'column' in value) {
      return point$4(value)
    }

    // ?
    return ''
  }

  /**
   * @param {Point|undefined} point
   * @returns {string}
   */
  function point$4(point) {
    return index$1(point && point.line) + ':' + index$1(point && point.column)
  }

  /**
   * @param {Position|undefined} pos
   * @returns {string}
   */
  function position$3(pos) {
    return point$4(pos && pos.start) + '-' + point$4(pos && pos.end)
  }

  /**
   * @param {number|undefined} value
   * @returns {number}
   */
  function index$1(value) {
    return value && typeof value === 'number' ? value : 1
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Position} Position
   * @typedef {import('unist').Point} Point
   * @typedef {object & {type: string, position?: Position|undefined}} NodeLike
   */

  class VFileMessage$1 extends Error {
    /**
     * Constructor of a message for `reason` at `place` from `origin`.
     * When an error is passed in as `reason`, copies the `stack`.
     *
     * @param {string|Error} reason Reason for message (`string` or `Error`). Uses the stack and message of the error if given.
     * @param {Node|NodeLike|Position|Point} [place] Place at which the message occurred in a file (`Node`, `Position`, or `Point`, optional).
     * @param {string} [origin] Place in code the message originates from (`string`, optional).
     */
    constructor(reason, place, origin) {
      /** @type {[string|null, string|null]} */
      const parts = [null, null];
      /** @type {Position} */
      let position = {
        // @ts-expect-error: we always follows the structure of `position`.
        start: {line: null, column: null},
        // @ts-expect-error: "
        end: {line: null, column: null}
      };

      super();

      if (typeof place === 'string') {
        origin = place;
        place = undefined;
      }

      if (typeof origin === 'string') {
        const index = origin.indexOf(':');

        if (index === -1) {
          parts[1] = origin;
        } else {
          parts[0] = origin.slice(0, index);
          parts[1] = origin.slice(index + 1);
        }
      }

      if (place) {
        // Node.
        if ('type' in place || 'position' in place) {
          if (place.position) {
            position = place.position;
          }
        }
        // Position.
        else if ('start' in place || 'end' in place) {
          position = place;
        }
        // Point.
        else if ('line' in place || 'column' in place) {
          position.start = place;
        }
      }

      // Fields from `Error`
      this.name = stringifyPosition$1(place) || '1:1';
      this.message = typeof reason === 'object' ? reason.message : reason;
      this.stack = typeof reason === 'object' ? reason.stack : '';

      /**
       * Reason for message.
       * @type {string}
       */
      this.reason = this.message;
      /**
       * If true, marks associated file as no longer processable.
       * @type {boolean?}
       */
      // eslint-disable-next-line no-unused-expressions
      this.fatal;
      /**
       * Starting line of error.
       * @type {number?}
       */
      this.line = position.start.line;
      /**
       * Starting column of error.
       * @type {number?}
       */
      this.column = position.start.column;
      /**
       * Namespace of warning.
       * @type {string?}
       */
      this.source = parts[0];
      /**
       * Category of message.
       * @type {string?}
       */
      this.ruleId = parts[1];
      /**
       * Full range information, when available.
       * Has start and end properties, both set to an object with line and column, set to number?.
       * @type {Position?}
       */
      this.position = position;

      // The following fields are “well known”.
      // Not standard.
      // Feel free to add other non-standard fields to your messages.

      /* eslint-disable no-unused-expressions */
      /**
       * You can use this to specify the source value that’s being reported, which
       * is deemed incorrect.
       * @type {string?}
       */
      this.actual;
      /**
       * You can use this to suggest values that should be used instead of
       * `actual`, one or more values that are deemed as acceptable.
       * @type {Array<string>?}
       */
      this.expected;
      /**
       * You may add a file property with a path of a file (used throughout the VFile ecosystem).
       * @type {string?}
       */
      this.file;
      /**
       * You may add a url property with a link to documentation for the message.
       * @type {string?}
       */
      this.url;
      /**
       * You may add a note property with a long form description of the message (supported by vfile-reporter).
       * @type {string?}
       */
      this.note;
      /* eslint-enable no-unused-expressions */
    }
  }

  VFileMessage$1.prototype.file = '';
  VFileMessage$1.prototype.name = '';
  VFileMessage$1.prototype.reason = '';
  VFileMessage$1.prototype.message = '';
  VFileMessage$1.prototype.stack = '';
  VFileMessage$1.prototype.fatal = null;
  VFileMessage$1.prototype.column = null;
  VFileMessage$1.prototype.line = null;
  VFileMessage$1.prototype.source = null;
  VFileMessage$1.prototype.ruleId = null;
  VFileMessage$1.prototype.position = null;

  // A derivative work based on:
  // <https://github.com/browserify/path-browserify>.
  // Which is licensed:
  //
  // MIT License
  //
  // Copyright (c) 2013 James Halliday
  //
  // Permission is hereby granted, free of charge, to any person obtaining a copy of
  // this software and associated documentation files (the "Software"), to deal in
  // the Software without restriction, including without limitation the rights to
  // use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
  // the Software, and to permit persons to whom the Software is furnished to do so,
  // subject to the following conditions:
  //
  // The above copyright notice and this permission notice shall be included in all
  // copies or substantial portions of the Software.
  //
  // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
  // FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
  // COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
  // IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  // CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  // A derivative work based on:
  //
  // Parts of that are extracted from Node’s internal `path` module:
  // <https://github.com/nodejs/node/blob/master/lib/path.js>.
  // Which is licensed:
  //
  // Copyright Joyent, Inc. and other Node contributors.
  //
  // Permission is hereby granted, free of charge, to any person obtaining a
  // copy of this software and associated documentation files (the
  // "Software"), to deal in the Software without restriction, including
  // without limitation the rights to use, copy, modify, merge, publish,
  // distribute, sublicense, and/or sell copies of the Software, and to permit
  // persons to whom the Software is furnished to do so, subject to the
  // following conditions:
  //
  // The above copyright notice and this permission notice shall be included
  // in all copies or substantial portions of the Software.
  //
  // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  // OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
  // NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
  // DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
  // OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
  // USE OR OTHER DEALINGS IN THE SOFTWARE.

  const path$3 = {basename: basename$1, dirname: dirname$1, extname: extname$1, join: join$1, sep: '/'};

  /* eslint-disable max-depth, complexity */

  /**
   * @param {string} path
   * @param {string} [ext]
   * @returns {string}
   */
  function basename$1(path, ext) {
    if (ext !== undefined && typeof ext !== 'string') {
      throw new TypeError('"ext" argument must be a string')
    }

    assertPath$3(path);
    let start = 0;
    let end = -1;
    let index = path.length;
    /** @type {boolean|undefined} */
    let seenNonSlash;

    if (ext === undefined || ext.length === 0 || ext.length > path.length) {
      while (index--) {
        if (path.charCodeAt(index) === 47 /* `/` */) {
          // If we reached a path separator that was not part of a set of path
          // separators at the end of the string, stop now.
          if (seenNonSlash) {
            start = index + 1;
            break
          }
        } else if (end < 0) {
          // We saw the first non-path separator, mark this as the end of our
          // path component.
          seenNonSlash = true;
          end = index + 1;
        }
      }

      return end < 0 ? '' : path.slice(start, end)
    }

    if (ext === path) {
      return ''
    }

    let firstNonSlashEnd = -1;
    let extIndex = ext.length - 1;

    while (index--) {
      if (path.charCodeAt(index) === 47 /* `/` */) {
        // If we reached a path separator that was not part of a set of path
        // separators at the end of the string, stop now.
        if (seenNonSlash) {
          start = index + 1;
          break
        }
      } else {
        if (firstNonSlashEnd < 0) {
          // We saw the first non-path separator, remember this index in case
          // we need it if the extension ends up not matching.
          seenNonSlash = true;
          firstNonSlashEnd = index + 1;
        }

        if (extIndex > -1) {
          // Try to match the explicit extension.
          if (path.charCodeAt(index) === ext.charCodeAt(extIndex--)) {
            if (extIndex < 0) {
              // We matched the extension, so mark this as the end of our path
              // component
              end = index;
            }
          } else {
            // Extension does not match, so our result is the entire path
            // component
            extIndex = -1;
            end = firstNonSlashEnd;
          }
        }
      }
    }

    if (start === end) {
      end = firstNonSlashEnd;
    } else if (end < 0) {
      end = path.length;
    }

    return path.slice(start, end)
  }

  /**
   * @param {string} path
   * @returns {string}
   */
  function dirname$1(path) {
    assertPath$3(path);

    if (path.length === 0) {
      return '.'
    }

    let end = -1;
    let index = path.length;
    /** @type {boolean|undefined} */
    let unmatchedSlash;

    // Prefix `--` is important to not run on `0`.
    while (--index) {
      if (path.charCodeAt(index) === 47 /* `/` */) {
        if (unmatchedSlash) {
          end = index;
          break
        }
      } else if (!unmatchedSlash) {
        // We saw the first non-path separator
        unmatchedSlash = true;
      }
    }

    return end < 0
      ? path.charCodeAt(0) === 47 /* `/` */
        ? '/'
        : '.'
      : end === 1 && path.charCodeAt(0) === 47 /* `/` */
      ? '//'
      : path.slice(0, end)
  }

  /**
   * @param {string} path
   * @returns {string}
   */
  function extname$1(path) {
    assertPath$3(path);

    let index = path.length;

    let end = -1;
    let startPart = 0;
    let startDot = -1;
    // Track the state of characters (if any) we see before our first dot and
    // after any path separator we find.
    let preDotState = 0;
    /** @type {boolean|undefined} */
    let unmatchedSlash;

    while (index--) {
      const code = path.charCodeAt(index);

      if (code === 47 /* `/` */) {
        // If we reached a path separator that was not part of a set of path
        // separators at the end of the string, stop now.
        if (unmatchedSlash) {
          startPart = index + 1;
          break
        }

        continue
      }

      if (end < 0) {
        // We saw the first non-path separator, mark this as the end of our
        // extension.
        unmatchedSlash = true;
        end = index + 1;
      }

      if (code === 46 /* `.` */) {
        // If this is our first dot, mark it as the start of our extension.
        if (startDot < 0) {
          startDot = index;
        } else if (preDotState !== 1) {
          preDotState = 1;
        }
      } else if (startDot > -1) {
        // We saw a non-dot and non-path separator before our dot, so we should
        // have a good chance at having a non-empty extension.
        preDotState = -1;
      }
    }

    if (
      startDot < 0 ||
      end < 0 ||
      // We saw a non-dot character immediately before the dot.
      preDotState === 0 ||
      // The (right-most) trimmed path component is exactly `..`.
      (preDotState === 1 && startDot === end - 1 && startDot === startPart + 1)
    ) {
      return ''
    }

    return path.slice(startDot, end)
  }

  /**
   * @param {Array<string>} segments
   * @returns {string}
   */
  function join$1(...segments) {
    let index = -1;
    /** @type {string|undefined} */
    let joined;

    while (++index < segments.length) {
      assertPath$3(segments[index]);

      if (segments[index]) {
        joined =
          joined === undefined ? segments[index] : joined + '/' + segments[index];
      }
    }

    return joined === undefined ? '.' : normalize$8(joined)
  }

  /**
   * Note: `normalize` is not exposed as `path.normalize`, so some code is
   * manually removed from it.
   *
   * @param {string} path
   * @returns {string}
   */
  function normalize$8(path) {
    assertPath$3(path);

    const absolute = path.charCodeAt(0) === 47; /* `/` */

    // Normalize the path according to POSIX rules.
    let value = normalizeString$1(path, !absolute);

    if (value.length === 0 && !absolute) {
      value = '.';
    }

    if (value.length > 0 && path.charCodeAt(path.length - 1) === 47 /* / */) {
      value += '/';
    }

    return absolute ? '/' + value : value
  }

  /**
   * Resolve `.` and `..` elements in a path with directory names.
   *
   * @param {string} path
   * @param {boolean} allowAboveRoot
   * @returns {string}
   */
  function normalizeString$1(path, allowAboveRoot) {
    let result = '';
    let lastSegmentLength = 0;
    let lastSlash = -1;
    let dots = 0;
    let index = -1;
    /** @type {number|undefined} */
    let code;
    /** @type {number} */
    let lastSlashIndex;

    while (++index <= path.length) {
      if (index < path.length) {
        code = path.charCodeAt(index);
      } else if (code === 47 /* `/` */) {
        break
      } else {
        code = 47; /* `/` */
      }

      if (code === 47 /* `/` */) {
        if (lastSlash === index - 1 || dots === 1) ; else if (lastSlash !== index - 1 && dots === 2) {
          if (
            result.length < 2 ||
            lastSegmentLength !== 2 ||
            result.charCodeAt(result.length - 1) !== 46 /* `.` */ ||
            result.charCodeAt(result.length - 2) !== 46 /* `.` */
          ) {
            if (result.length > 2) {
              lastSlashIndex = result.lastIndexOf('/');

              if (lastSlashIndex !== result.length - 1) {
                if (lastSlashIndex < 0) {
                  result = '';
                  lastSegmentLength = 0;
                } else {
                  result = result.slice(0, lastSlashIndex);
                  lastSegmentLength = result.length - 1 - result.lastIndexOf('/');
                }

                lastSlash = index;
                dots = 0;
                continue
              }
            } else if (result.length > 0) {
              result = '';
              lastSegmentLength = 0;
              lastSlash = index;
              dots = 0;
              continue
            }
          }

          if (allowAboveRoot) {
            result = result.length > 0 ? result + '/..' : '..';
            lastSegmentLength = 2;
          }
        } else {
          if (result.length > 0) {
            result += '/' + path.slice(lastSlash + 1, index);
          } else {
            result = path.slice(lastSlash + 1, index);
          }

          lastSegmentLength = index - lastSlash - 1;
        }

        lastSlash = index;
        dots = 0;
      } else if (code === 46 /* `.` */ && dots > -1) {
        dots++;
      } else {
        dots = -1;
      }
    }

    return result
  }

  /**
   * @param {string} path
   */
  function assertPath$3(path) {
    if (typeof path !== 'string') {
      throw new TypeError(
        'Path must be a string. Received ' + JSON.stringify(path)
      )
    }
  }

  /* eslint-enable max-depth, complexity */

  // Somewhat based on:
  // <https://github.com/defunctzombie/node-process/blob/master/browser.js>.
  // But I don’t think one tiny line of code can be copyrighted. 😅
  const proc$1 = {cwd: cwd$1};

  function cwd$1() {
    return '/'
  }

  /**
   * @typedef URL
   * @property {string} hash
   * @property {string} host
   * @property {string} hostname
   * @property {string} href
   * @property {string} origin
   * @property {string} password
   * @property {string} pathname
   * @property {string} port
   * @property {string} protocol
   * @property {string} search
   * @property {any} searchParams
   * @property {string} username
   * @property {() => string} toString
   * @property {() => string} toJSON
   */

  /**
   * @param {unknown} fileURLOrPath
   * @returns {fileURLOrPath is URL}
   */
  // From: <https://github.com/nodejs/node/blob/fcf8ba4/lib/internal/url.js#L1501>
  function isUrl$1(fileURLOrPath) {
    return (
      fileURLOrPath !== null &&
      typeof fileURLOrPath === 'object' &&
      // @ts-expect-error: indexable.
      fileURLOrPath.href &&
      // @ts-expect-error: indexable.
      fileURLOrPath.origin
    )
  }

  /// <reference lib="dom" />

  // See: <https://github.com/nodejs/node/blob/fcf8ba4/lib/internal/url.js>

  /**
   * @param {string|URL} path
   */
  function urlToPath$1(path) {
    if (typeof path === 'string') {
      path = new URL(path);
    } else if (!isUrl$1(path)) {
      /** @type {NodeJS.ErrnoException} */
      const error = new TypeError(
        'The "path" argument must be of type string or an instance of URL. Received `' +
          path +
          '`'
      );
      error.code = 'ERR_INVALID_ARG_TYPE';
      throw error
    }

    if (path.protocol !== 'file:') {
      /** @type {NodeJS.ErrnoException} */
      const error = new TypeError('The URL must be of scheme file');
      error.code = 'ERR_INVALID_URL_SCHEME';
      throw error
    }

    return getPathFromURLPosix$1(path)
  }

  /**
   * @param {URL} url
   */
  function getPathFromURLPosix$1(url) {
    if (url.hostname !== '') {
      /** @type {NodeJS.ErrnoException} */
      const error = new TypeError(
        'File URL host must be "localhost" or empty on darwin'
      );
      error.code = 'ERR_INVALID_FILE_URL_HOST';
      throw error
    }

    const pathname = url.pathname;
    let index = -1;

    while (++index < pathname.length) {
      if (
        pathname.charCodeAt(index) === 37 /* `%` */ &&
        pathname.charCodeAt(index + 1) === 50 /* `2` */
      ) {
        const third = pathname.charCodeAt(index + 2);
        if (third === 70 /* `F` */ || third === 102 /* `f` */) {
          /** @type {NodeJS.ErrnoException} */
          const error = new TypeError(
            'File URL path must not include encoded / characters'
          );
          error.code = 'ERR_INVALID_FILE_URL_PATH';
          throw error
        }
      }
    }

    return decodeURIComponent(pathname)
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Position} Position
   * @typedef {import('unist').Point} Point
   * @typedef {Record<string, unknown> & {type: string, position?: Position|undefined}} NodeLike
   * @typedef {import('./minurl.shared.js').URL} URL
   * @typedef {import('..').VFileData} VFileData
   * @typedef {import('..').VFileValue} VFileValue
   *
   * @typedef {'ascii'|'utf8'|'utf-8'|'utf16le'|'ucs2'|'ucs-2'|'base64'|'base64url'|'latin1'|'binary'|'hex'} BufferEncoding
   *   Encodings supported by the buffer class.
   *   This is a copy of the typing from Node, copied to prevent Node globals from
   *   being needed.
   *   Copied from: <https://github.com/DefinitelyTyped/DefinitelyTyped/blob/90a4ec8/types/node/buffer.d.ts#L170>
   *
   * @typedef {VFileValue|VFileOptions|VFile|URL} VFileCompatible
   *   Things that can be passed to the constructor.
   *
   * @typedef VFileCoreOptions
   * @property {VFileValue} [value]
   * @property {string} [cwd]
   * @property {Array<string>} [history]
   * @property {string|URL} [path]
   * @property {string} [basename]
   * @property {string} [stem]
   * @property {string} [extname]
   * @property {string} [dirname]
   * @property {VFileData} [data]
   *
   * @typedef Map
   *   Raw source map, see:
   *   <https://github.com/mozilla/source-map/blob/58819f0/source-map.d.ts#L15-L23>.
   * @property {number} version
   * @property {Array<string>} sources
   * @property {Array<string>} names
   * @property {string|undefined} [sourceRoot]
   * @property {Array<string>|undefined} [sourcesContent]
   * @property {string} mappings
   * @property {string} file
   *
   * @typedef {{[key: string]: unknown} & VFileCoreOptions} VFileOptions
   *   Configuration: a bunch of keys that will be shallow copied over to the new
   *   file.
   *
   * @typedef {Record<string, unknown>} VFileReporterSettings
   * @typedef {<T = VFileReporterSettings>(files: Array<VFile>, options: T) => string} VFileReporter
   */

  // Order of setting (least specific to most), we need this because otherwise
  // `{stem: 'a', path: '~/b.js'}` would throw, as a path is needed before a
  // stem can be set.
  const order$1 = ['history', 'path', 'basename', 'stem', 'extname', 'dirname'];

  class VFile$1 {
    /**
     * Create a new virtual file.
     *
     * If `options` is `string` or `Buffer`, treats it as `{value: options}`.
     * If `options` is a `VFile`, shallow copies its data over to the new file.
     * All other given fields are set on the newly created `VFile`.
     *
     * Path related properties are set in the following order (least specific to
     * most specific): `history`, `path`, `basename`, `stem`, `extname`,
     * `dirname`.
     *
     * It’s not possible to set either `dirname` or `extname` without setting
     * either `history`, `path`, `basename`, or `stem` as well.
     *
     * @param {VFileCompatible} [value]
     */
    constructor(value) {
      /** @type {VFileOptions} */
      let options;

      if (!value) {
        options = {};
      } else if (typeof value === 'string' || isBuffer$1(value)) {
        // @ts-expect-error Looks like a buffer.
        options = {value};
      } else if (isUrl$1(value)) {
        options = {path: value};
      } else {
        // @ts-expect-error Looks like file or options.
        options = value;
      }

      /**
       * Place to store custom information.
       * It’s OK to store custom data directly on the file, moving it to `data`
       * gives a little more privacy.
       * @type {VFileData}
       */
      this.data = {};

      /**
       * List of messages associated with the file.
       * @type {Array<VFileMessage>}
       */
      this.messages = [];

      /**
       * List of file paths the file moved between.
       * @type {Array<string>}
       */
      this.history = [];

      /**
       * Base of `path`.
       * Defaults to `process.cwd()` (`/` in browsers).
       * @type {string}
       */
      this.cwd = proc$1.cwd();

      /* eslint-disable no-unused-expressions */
      /**
       * Raw value.
       * @type {VFileValue}
       */
      this.value;

      // The below are non-standard, they are “well-known”.
      // As in, used in several tools.

      /**
       * Whether a file was saved to disk.
       * This is used by vfile reporters.
       * @type {boolean}
       */
      this.stored;

      /**
       * Sometimes files have a non-string representation.
       * This can be stored in the `result` field.
       * One example is when turning markdown into React nodes.
       * This is used by unified to store non-string results.
       * @type {unknown}
       */
      this.result;

      /**
       * Sometimes files have a source map associated with them.
       * This can be stored in the `map` field.
       * This should be a `RawSourceMap` type from the `source-map` module.
       * @type {Map|undefined}
       */
      this.map;
      /* eslint-enable no-unused-expressions */

      // Set path related properties in the correct order.
      let index = -1;

      while (++index < order$1.length) {
        const prop = order$1[index];

        // Note: we specifically use `in` instead of `hasOwnProperty` to accept
        // `vfile`s too.
        if (prop in options && options[prop] !== undefined) {
          // @ts-expect-error: TS is confused by the different types for `history`.
          this[prop] = prop === 'history' ? [...options[prop]] : options[prop];
        }
      }

      /** @type {string} */
      let prop;

      // Set non-path related properties.
      for (prop in options) {
        // @ts-expect-error: fine to set other things.
        if (!order$1.includes(prop)) this[prop] = options[prop];
      }
    }

    /**
     * Access full path (`~/index.min.js`).
     *
     * @returns {string}
     */
    get path() {
      return this.history[this.history.length - 1]
    }

    /**
     * Set full path (`~/index.min.js`).
     * Cannot be nullified.
     *
     * @param {string|URL} path
     */
    set path(path) {
      if (isUrl$1(path)) {
        path = urlToPath$1(path);
      }

      assertNonEmpty$1(path, 'path');

      if (this.path !== path) {
        this.history.push(path);
      }
    }

    /**
     * Access parent path (`~`).
     */
    get dirname() {
      return typeof this.path === 'string' ? path$3.dirname(this.path) : undefined
    }

    /**
     * Set parent path (`~`).
     * Cannot be set if there's no `path` yet.
     */
    set dirname(dirname) {
      assertPath$2(this.basename, 'dirname');
      this.path = path$3.join(dirname || '', this.basename);
    }

    /**
     * Access basename (including extname) (`index.min.js`).
     */
    get basename() {
      return typeof this.path === 'string' ? path$3.basename(this.path) : undefined
    }

    /**
     * Set basename (`index.min.js`).
     * Cannot contain path separators.
     * Cannot be nullified either (use `file.path = file.dirname` instead).
     */
    set basename(basename) {
      assertNonEmpty$1(basename, 'basename');
      assertPart$1(basename, 'basename');
      this.path = path$3.join(this.dirname || '', basename);
    }

    /**
     * Access extname (including dot) (`.js`).
     */
    get extname() {
      return typeof this.path === 'string' ? path$3.extname(this.path) : undefined
    }

    /**
     * Set extname (including dot) (`.js`).
     * Cannot be set if there's no `path` yet and cannot contain path separators.
     */
    set extname(extname) {
      assertPart$1(extname, 'extname');
      assertPath$2(this.dirname, 'extname');

      if (extname) {
        if (extname.charCodeAt(0) !== 46 /* `.` */) {
          throw new Error('`extname` must start with `.`')
        }

        if (extname.includes('.', 1)) {
          throw new Error('`extname` cannot contain multiple dots')
        }
      }

      this.path = path$3.join(this.dirname, this.stem + (extname || ''));
    }

    /**
     * Access stem (w/o extname) (`index.min`).
     */
    get stem() {
      return typeof this.path === 'string'
        ? path$3.basename(this.path, this.extname)
        : undefined
    }

    /**
     * Set stem (w/o extname) (`index.min`).
     * Cannot be nullified, and cannot contain path separators.
     */
    set stem(stem) {
      assertNonEmpty$1(stem, 'stem');
      assertPart$1(stem, 'stem');
      this.path = path$3.join(this.dirname || '', stem + (this.extname || ''));
    }

    /**
     * Serialize the file.
     *
     * @param {BufferEncoding} [encoding='utf8'] If `file.value` is a buffer, `encoding` is used to serialize buffers.
     * @returns {string}
     */
    toString(encoding) {
      return (this.value || '').toString(encoding)
    }

    /**
     * Create a message and associates it w/ the file.
     *
     * @param {string|Error} reason Reason for message (`string` or `Error`). Uses the stack and message of the error if given.
     * @param {Node|NodeLike|Position|Point} [place] Place at which the message occurred in a file (`Node`, `Position`, or `Point`, optional).
     * @param {string} [origin] Place in code the message originates from (`string`, optional).
     * @returns {VFileMessage}
     */
    message(reason, place, origin) {
      const message = new VFileMessage$1(reason, place, origin);

      if (this.path) {
        message.name = this.path + ':' + message.name;
        message.file = this.path;
      }

      message.fatal = false;

      this.messages.push(message);

      return message
    }

    /**
     * Info: create a message, associate it with the file, and mark the fatality
     * as `null`.
     * Calls `message()` internally.
     *
     * @param {string|Error} reason Reason for message (`string` or `Error`). Uses the stack and message of the error if given.
     * @param {Node|NodeLike|Position|Point} [place] Place at which the message occurred in a file (`Node`, `Position`, or `Point`, optional).
     * @param {string} [origin] Place in code the message originates from (`string`, optional).
     * @returns {VFileMessage}
     */
    info(reason, place, origin) {
      const message = this.message(reason, place, origin);

      message.fatal = null;

      return message
    }

    /**
     * Fail: create a message, associate it with the file, mark the fatality as
     * `true`.
     * Note: fatal errors mean a file is no longer processable.
     * Calls `message()` internally.
     *
     * @param {string|Error} reason Reason for message (`string` or `Error`). Uses the stack and message of the error if given.
     * @param {Node|NodeLike|Position|Point} [place] Place at which the message occurred in a file (`Node`, `Position`, or `Point`, optional).
     * @param {string} [origin] Place in code the message originates from (`string`, optional).
     * @returns {never}
     */
    fail(reason, place, origin) {
      const message = this.message(reason, place, origin);

      message.fatal = true;

      throw message
    }
  }

  /**
   * Assert that `part` is not a path (as in, does not contain `path.sep`).
   *
   * @param {string|undefined} part
   * @param {string} name
   * @returns {void}
   */
  function assertPart$1(part, name) {
    if (part && part.includes(path$3.sep)) {
      throw new Error(
        '`' + name + '` cannot be a path: did not expect `' + path$3.sep + '`'
      )
    }
  }

  /**
   * Assert that `part` is not empty.
   *
   * @param {string|undefined} part
   * @param {string} name
   * @returns {asserts part is string}
   */
  function assertNonEmpty$1(part, name) {
    if (!part) {
      throw new Error('`' + name + '` cannot be empty')
    }
  }

  /**
   * Assert `path` exists.
   *
   * @param {string|undefined} path
   * @param {string} name
   * @returns {asserts path is string}
   */
  function assertPath$2(path, name) {
    if (!path) {
      throw new Error('Setting `' + name + '` requires `path` to be set too')
    }
  }

  /**
   * Throw a given error.
   *
   * @param {Error|null|undefined} [error]
   *   Maybe error.
   * @returns {asserts error is null|undefined}
   */
  function bail$1(error) {
    if (error) {
      throw error
    }
  }

  var hasOwn$1 = Object.prototype.hasOwnProperty;
  var toStr$1 = Object.prototype.toString;
  var defineProperty$1 = Object.defineProperty;
  var gOPD$1 = Object.getOwnPropertyDescriptor;

  var isArray$1 = function isArray(arr) {
    if (typeof Array.isArray === 'function') {
      return Array.isArray(arr);
    }

    return toStr$1.call(arr) === '[object Array]';
  };

  var isPlainObject$3 = function isPlainObject(obj) {
    if (!obj || toStr$1.call(obj) !== '[object Object]') {
      return false;
    }

    var hasOwnConstructor = hasOwn$1.call(obj, 'constructor');
    var hasIsPrototypeOf = obj.constructor && obj.constructor.prototype && hasOwn$1.call(obj.constructor.prototype, 'isPrototypeOf');
    // Not own constructor property must be Object
    if (obj.constructor && !hasOwnConstructor && !hasIsPrototypeOf) {
      return false;
    }

    // Own properties are enumerated firstly, so to speed up,
    // if last one is own, then all properties are own.
    var key;
    for (key in obj) { /**/ }

    return typeof key === 'undefined' || hasOwn$1.call(obj, key);
  };

  // If name is '__proto__', and Object.defineProperty is available, define __proto__ as an own property on target
  var setProperty$1 = function setProperty(target, options) {
    if (defineProperty$1 && options.name === '__proto__') {
      defineProperty$1(target, options.name, {
        enumerable: true,
        configurable: true,
        value: options.newValue,
        writable: true
      });
    } else {
      target[options.name] = options.newValue;
    }
  };

  // Return undefined instead of __proto__ if '__proto__' is not an own property
  var getProperty$1 = function getProperty(obj, name) {
    if (name === '__proto__') {
      if (!hasOwn$1.call(obj, name)) {
        return void 0;
      } else if (gOPD$1) {
        // In early versions of node, obj['__proto__'] is buggy when obj has
        // __proto__ as an own property. Object.getOwnPropertyDescriptor() works.
        return gOPD$1(obj, name).value;
      }
    }

    return obj[name];
  };

  var extend$1 = function extend() {
    var options, name, src, copy, copyIsArray, clone;
    var target = arguments[0];
    var i = 1;
    var length = arguments.length;
    var deep = false;

    // Handle a deep copy situation
    if (typeof target === 'boolean') {
      deep = target;
      target = arguments[1] || {};
      // skip the boolean and the target
      i = 2;
    }
    if (target == null || (typeof target !== 'object' && typeof target !== 'function')) {
      target = {};
    }

    for (; i < length; ++i) {
      options = arguments[i];
      // Only deal with non-null/undefined values
      if (options != null) {
        // Extend the base object
        for (name in options) {
          src = getProperty$1(target, name);
          copy = getProperty$1(options, name);

          // Prevent never-ending loop
          if (target !== copy) {
            // Recurse if we're merging plain objects or arrays
            if (deep && copy && (isPlainObject$3(copy) || (copyIsArray = isArray$1(copy)))) {
              if (copyIsArray) {
                copyIsArray = false;
                clone = src && isArray$1(src) ? src : [];
              } else {
                clone = src && isPlainObject$3(src) ? src : {};
              }

              // Never move original objects, clone them
              setProperty$1(target, { name: name, newValue: extend(deep, clone, copy) });

            // Don't bring in undefined values
            } else if (typeof copy !== 'undefined') {
              setProperty$1(target, { name: name, newValue: copy });
            }
          }
        }
      }
    }

    // Return the modified object
    return target;
  };

  function isPlainObject$2(value) {
    if (Object.prototype.toString.call(value) !== '[object Object]') {
      return false;
    }

    const prototype = Object.getPrototypeOf(value);
    return prototype === null || prototype === Object.prototype;
  }

  /**
   * @typedef {(error?: Error|null|undefined, ...output: Array<any>) => void} Callback
   * @typedef {(...input: Array<any>) => any} Middleware
   *
   * @typedef {(...input: Array<any>) => void} Run
   *   Call all middleware.
   * @typedef {(fn: Middleware) => Pipeline} Use
   *   Add `fn` (middleware) to the list.
   * @typedef {{run: Run, use: Use}} Pipeline
   *   Middleware.
   */

  /**
   * Create new middleware.
   *
   * @returns {Pipeline}
   */
  function trough$1() {
    /** @type {Array<Middleware>} */
    const fns = [];
    /** @type {Pipeline} */
    const pipeline = {run, use};

    return pipeline

    /** @type {Run} */
    function run(...values) {
      let middlewareIndex = -1;
      /** @type {Callback} */
      const callback = values.pop();

      if (typeof callback !== 'function') {
        throw new TypeError('Expected function as last argument, not ' + callback)
      }

      next(null, ...values);

      /**
       * Run the next `fn`, or we’re done.
       *
       * @param {Error|null|undefined} error
       * @param {Array<any>} output
       */
      function next(error, ...output) {
        const fn = fns[++middlewareIndex];
        let index = -1;

        if (error) {
          callback(error);
          return
        }

        // Copy non-nullish input into values.
        while (++index < values.length) {
          if (output[index] === null || output[index] === undefined) {
            output[index] = values[index];
          }
        }

        // Save the newly created `output` for the next call.
        values = output;

        // Next or done.
        if (fn) {
          wrap$2(fn, next)(...output);
        } else {
          callback(null, ...output);
        }
      }
    }

    /** @type {Use} */
    function use(middelware) {
      if (typeof middelware !== 'function') {
        throw new TypeError(
          'Expected `middelware` to be a function, not ' + middelware
        )
      }

      fns.push(middelware);
      return pipeline
    }
  }

  /**
   * Wrap `middleware`.
   * Can be sync or async; return a promise, receive a callback, or return new
   * values and errors.
   *
   * @param {Middleware} middleware
   * @param {Callback} callback
   */
  function wrap$2(middleware, callback) {
    /** @type {boolean} */
    let called;

    return wrapped

    /**
     * Call `middleware`.
     * @this {any}
     * @param {Array<any>} parameters
     * @returns {void}
     */
    function wrapped(...parameters) {
      const fnExpectsCallback = middleware.length > parameters.length;
      /** @type {any} */
      let result;

      if (fnExpectsCallback) {
        parameters.push(done);
      }

      try {
        result = middleware.apply(this, parameters);
      } catch (error) {
        const exception = /** @type {Error} */ (error);

        // Well, this is quite the pickle.
        // `middleware` received a callback and called it synchronously, but that
        // threw an error.
        // The only thing left to do is to throw the thing instead.
        if (fnExpectsCallback && called) {
          throw exception
        }

        return done(exception)
      }

      if (!fnExpectsCallback) {
        if (result instanceof Promise) {
          result.then(then, done);
        } else if (result instanceof Error) {
          done(result);
        } else {
          then(result);
        }
      }
    }

    /**
     * Call `callback`, only once.
     * @type {Callback}
     */
    function done(error, ...output) {
      if (!called) {
        called = true;
        callback(error, ...output);
      }
    }

    /**
     * Call `done` with one value.
     *
     * @param {any} [value]
     */
    function then(value) {
      done(null, value);
    }
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('vfile').VFileCompatible} VFileCompatible
   * @typedef {import('vfile').VFileValue} VFileValue
   * @typedef {import('..').Processor} Processor
   * @typedef {import('..').Plugin} Plugin
   * @typedef {import('..').Preset} Preset
   * @typedef {import('..').Pluggable} Pluggable
   * @typedef {import('..').PluggableList} PluggableList
   * @typedef {import('..').Transformer} Transformer
   * @typedef {import('..').Parser} Parser
   * @typedef {import('..').Compiler} Compiler
   * @typedef {import('..').RunCallback} RunCallback
   * @typedef {import('..').ProcessCallback} ProcessCallback
   *
   * @typedef Context
   * @property {Node} tree
   * @property {VFile} file
   */

  // Expose a frozen processor.
  const unified$1 = base$2().freeze();

  const own$s = {}.hasOwnProperty;

  // Function to create the first processor.
  /**
   * @returns {Processor}
   */
  function base$2() {
    const transformers = trough$1();
    /** @type {Processor['attachers']} */
    const attachers = [];
    /** @type {Record<string, unknown>} */
    let namespace = {};
    /** @type {boolean|undefined} */
    let frozen;
    let freezeIndex = -1;

    // Data management.
    // @ts-expect-error: overloads are handled.
    processor.data = data;
    processor.Parser = undefined;
    processor.Compiler = undefined;

    // Lock.
    processor.freeze = freeze;

    // Plugins.
    processor.attachers = attachers;
    // @ts-expect-error: overloads are handled.
    processor.use = use;

    // API.
    processor.parse = parse;
    processor.stringify = stringify;
    // @ts-expect-error: overloads are handled.
    processor.run = run;
    processor.runSync = runSync;
    // @ts-expect-error: overloads are handled.
    processor.process = process;
    processor.processSync = processSync;

    // Expose.
    return processor

    // Create a new processor based on the processor in the current scope.
    /** @type {Processor} */
    function processor() {
      const destination = base$2();
      let index = -1;

      while (++index < attachers.length) {
        destination.use(...attachers[index]);
      }

      destination.data(extend$1(true, {}, namespace));

      return destination
    }

    /**
     * @param {string|Record<string, unknown>} [key]
     * @param {unknown} [value]
     * @returns {unknown}
     */
    function data(key, value) {
      if (typeof key === 'string') {
        // Set `key`.
        if (arguments.length === 2) {
          assertUnfrozen$1('data', frozen);
          namespace[key] = value;
          return processor
        }

        // Get `key`.
        return (own$s.call(namespace, key) && namespace[key]) || null
      }

      // Set space.
      if (key) {
        assertUnfrozen$1('data', frozen);
        namespace = key;
        return processor
      }

      // Get space.
      return namespace
    }

    /** @type {Processor['freeze']} */
    function freeze() {
      if (frozen) {
        return processor
      }

      while (++freezeIndex < attachers.length) {
        const [attacher, ...options] = attachers[freezeIndex];

        if (options[0] === false) {
          continue
        }

        if (options[0] === true) {
          options[0] = undefined;
        }

        /** @type {Transformer|void} */
        const transformer = attacher.call(processor, ...options);

        if (typeof transformer === 'function') {
          transformers.use(transformer);
        }
      }

      frozen = true;
      freezeIndex = Number.POSITIVE_INFINITY;

      return processor
    }

    /**
     * @param {Pluggable|null|undefined} [value]
     * @param {...unknown} options
     * @returns {Processor}
     */
    function use(value, ...options) {
      /** @type {Record<string, unknown>|undefined} */
      let settings;

      assertUnfrozen$1('use', frozen);

      if (value === null || value === undefined) ; else if (typeof value === 'function') {
        addPlugin(value, ...options);
      } else if (typeof value === 'object') {
        if (Array.isArray(value)) {
          addList(value);
        } else {
          addPreset(value);
        }
      } else {
        throw new TypeError('Expected usable value, not `' + value + '`')
      }

      if (settings) {
        namespace.settings = Object.assign(namespace.settings || {}, settings);
      }

      return processor

      /**
       * @param {import('..').Pluggable<unknown[]>} value
       * @returns {void}
       */
      function add(value) {
        if (typeof value === 'function') {
          addPlugin(value);
        } else if (typeof value === 'object') {
          if (Array.isArray(value)) {
            const [plugin, ...options] = value;
            addPlugin(plugin, ...options);
          } else {
            addPreset(value);
          }
        } else {
          throw new TypeError('Expected usable value, not `' + value + '`')
        }
      }

      /**
       * @param {Preset} result
       * @returns {void}
       */
      function addPreset(result) {
        addList(result.plugins);

        if (result.settings) {
          settings = Object.assign(settings || {}, result.settings);
        }
      }

      /**
       * @param {PluggableList|null|undefined} [plugins]
       * @returns {void}
       */
      function addList(plugins) {
        let index = -1;

        if (plugins === null || plugins === undefined) ; else if (Array.isArray(plugins)) {
          while (++index < plugins.length) {
            const thing = plugins[index];
            add(thing);
          }
        } else {
          throw new TypeError('Expected a list of plugins, not `' + plugins + '`')
        }
      }

      /**
       * @param {Plugin} plugin
       * @param {...unknown} [value]
       * @returns {void}
       */
      function addPlugin(plugin, value) {
        let index = -1;
        /** @type {Processor['attachers'][number]|undefined} */
        let entry;

        while (++index < attachers.length) {
          if (attachers[index][0] === plugin) {
            entry = attachers[index];
            break
          }
        }

        if (entry) {
          if (isPlainObject$2(entry[1]) && isPlainObject$2(value)) {
            value = extend$1(true, entry[1], value);
          }

          entry[1] = value;
        } else {
          // @ts-expect-error: fine.
          attachers.push([...arguments]);
        }
      }
    }

    /** @type {Processor['parse']} */
    function parse(doc) {
      processor.freeze();
      const file = vfile$1(doc);
      const Parser = processor.Parser;
      assertParser$1('parse', Parser);

      if (newable$1(Parser, 'parse')) {
        // @ts-expect-error: `newable` checks this.
        return new Parser(String(file), file).parse()
      }

      // @ts-expect-error: `newable` checks this.
      return Parser(String(file), file) // eslint-disable-line new-cap
    }

    /** @type {Processor['stringify']} */
    function stringify(node, doc) {
      processor.freeze();
      const file = vfile$1(doc);
      const Compiler = processor.Compiler;
      assertCompiler$1('stringify', Compiler);
      assertNode$1(node);

      if (newable$1(Compiler, 'compile')) {
        // @ts-expect-error: `newable` checks this.
        return new Compiler(node, file).compile()
      }

      // @ts-expect-error: `newable` checks this.
      return Compiler(node, file) // eslint-disable-line new-cap
    }

    /**
     * @param {Node} node
     * @param {VFileCompatible|RunCallback} [doc]
     * @param {RunCallback} [callback]
     * @returns {Promise<Node>|void}
     */
    function run(node, doc, callback) {
      assertNode$1(node);
      processor.freeze();

      if (!callback && typeof doc === 'function') {
        callback = doc;
        doc = undefined;
      }

      if (!callback) {
        return new Promise(executor)
      }

      executor(null, callback);

      /**
       * @param {null|((node: Node) => void)} resolve
       * @param {(error: Error) => void} reject
       * @returns {void}
       */
      function executor(resolve, reject) {
        // @ts-expect-error: `doc` can’t be a callback anymore, we checked.
        transformers.run(node, vfile$1(doc), done);

        /**
         * @param {Error|null} error
         * @param {Node} tree
         * @param {VFile} file
         * @returns {void}
         */
        function done(error, tree, file) {
          tree = tree || node;
          if (error) {
            reject(error);
          } else if (resolve) {
            resolve(tree);
          } else {
            // @ts-expect-error: `callback` is defined if `resolve` is not.
            callback(null, tree, file);
          }
        }
      }
    }

    /** @type {Processor['runSync']} */
    function runSync(node, file) {
      /** @type {Node|undefined} */
      let result;
      /** @type {boolean|undefined} */
      let complete;

      processor.run(node, file, done);

      assertDone$1('runSync', 'run', complete);

      // @ts-expect-error: we either bailed on an error or have a tree.
      return result

      /**
       * @param {Error|null} [error]
       * @param {Node} [tree]
       * @returns {void}
       */
      function done(error, tree) {
        bail$1(error);
        result = tree;
        complete = true;
      }
    }

    /**
     * @param {VFileCompatible} doc
     * @param {ProcessCallback} [callback]
     * @returns {Promise<VFile>|undefined}
     */
    function process(doc, callback) {
      processor.freeze();
      assertParser$1('process', processor.Parser);
      assertCompiler$1('process', processor.Compiler);

      if (!callback) {
        return new Promise(executor)
      }

      executor(null, callback);

      /**
       * @param {null|((file: VFile) => void)} resolve
       * @param {(error?: Error|null|undefined) => void} reject
       * @returns {void}
       */
      function executor(resolve, reject) {
        const file = vfile$1(doc);

        processor.run(processor.parse(file), file, (error, tree, file) => {
          if (error || !tree || !file) {
            done(error);
          } else {
            /** @type {unknown} */
            const result = processor.stringify(tree, file);

            if (result === undefined || result === null) ; else if (looksLikeAVFileValue$1(result)) {
              file.value = result;
            } else {
              file.result = result;
            }

            done(error, file);
          }
        });

        /**
         * @param {Error|null|undefined} [error]
         * @param {VFile|undefined} [file]
         * @returns {void}
         */
        function done(error, file) {
          if (error || !file) {
            reject(error);
          } else if (resolve) {
            resolve(file);
          } else {
            // @ts-expect-error: `callback` is defined if `resolve` is not.
            callback(null, file);
          }
        }
      }
    }

    /** @type {Processor['processSync']} */
    function processSync(doc) {
      /** @type {boolean|undefined} */
      let complete;

      processor.freeze();
      assertParser$1('processSync', processor.Parser);
      assertCompiler$1('processSync', processor.Compiler);

      const file = vfile$1(doc);

      processor.process(file, done);

      assertDone$1('processSync', 'process', complete);

      return file

      /**
       * @param {Error|null|undefined} [error]
       * @returns {void}
       */
      function done(error) {
        complete = true;
        bail$1(error);
      }
    }
  }

  /**
   * Check if `value` is a constructor.
   *
   * @param {unknown} value
   * @param {string} name
   * @returns {boolean}
   */
  function newable$1(value, name) {
    return (
      typeof value === 'function' &&
      // Prototypes do exist.
      // type-coverage:ignore-next-line
      value.prototype &&
      // A function with keys in its prototype is probably a constructor.
      // Classes’ prototype methods are not enumerable, so we check if some value
      // exists in the prototype.
      // type-coverage:ignore-next-line
      (keys$1(value.prototype) || name in value.prototype)
    )
  }

  /**
   * Check if `value` is an object with keys.
   *
   * @param {Record<string, unknown>} value
   * @returns {boolean}
   */
  function keys$1(value) {
    /** @type {string} */
    let key;

    for (key in value) {
      if (own$s.call(value, key)) {
        return true
      }
    }

    return false
  }

  /**
   * Assert a parser is available.
   *
   * @param {string} name
   * @param {unknown} value
   * @returns {asserts value is Parser}
   */
  function assertParser$1(name, value) {
    if (typeof value !== 'function') {
      throw new TypeError('Cannot `' + name + '` without `Parser`')
    }
  }

  /**
   * Assert a compiler is available.
   *
   * @param {string} name
   * @param {unknown} value
   * @returns {asserts value is Compiler}
   */
  function assertCompiler$1(name, value) {
    if (typeof value !== 'function') {
      throw new TypeError('Cannot `' + name + '` without `Compiler`')
    }
  }

  /**
   * Assert the processor is not frozen.
   *
   * @param {string} name
   * @param {unknown} frozen
   * @returns {asserts frozen is false}
   */
  function assertUnfrozen$1(name, frozen) {
    if (frozen) {
      throw new Error(
        'Cannot call `' +
          name +
          '` on a frozen processor.\nCreate a new processor first, by calling it: use `processor()` instead of `processor`.'
      )
    }
  }

  /**
   * Assert `node` is a unist node.
   *
   * @param {unknown} node
   * @returns {asserts node is Node}
   */
  function assertNode$1(node) {
    // `isPlainObj` unfortunately uses `any` instead of `unknown`.
    // type-coverage:ignore-next-line
    if (!isPlainObject$2(node) || typeof node.type !== 'string') {
      throw new TypeError('Expected node, got `' + node + '`')
      // Fine.
    }
  }

  /**
   * Assert that `complete` is `true`.
   *
   * @param {string} name
   * @param {string} asyncName
   * @param {unknown} complete
   * @returns {asserts complete is true}
   */
  function assertDone$1(name, asyncName, complete) {
    if (!complete) {
      throw new Error(
        '`' + name + '` finished async. Use `' + asyncName + '` instead'
      )
    }
  }

  /**
   * @param {VFileCompatible} [value]
   * @returns {VFile}
   */
  function vfile$1(value) {
    return looksLikeAVFile$1(value) ? value : new VFile$1(value)
  }

  /**
   * @param {VFileCompatible} [value]
   * @returns {value is VFile}
   */
  function looksLikeAVFile$1(value) {
    return Boolean(
      value &&
        typeof value === 'object' &&
        'message' in value &&
        'messages' in value
    )
  }

  /**
   * @param {unknown} [value]
   * @returns {value is VFileValue}
   */
  function looksLikeAVFileValue$1(value) {
    return typeof value === 'string' || isBuffer$1(value)
  }

  /**
   * Like `Array#splice`, but smarter for giant arrays.
   *
   * `Array#splice` takes all items to be inserted as individual argument which
   * causes a stack overflow in V8 when trying to insert 100k items for instance.
   *
   * Otherwise, this does not return the removed items, and takes `items` as an
   * array instead of rest parameters.
   *
   * @template {unknown} T
   * @param {T[]} list
   * @param {number} start
   * @param {number} remove
   * @param {T[]} items
   * @returns {void}
   */
  function splice(list, start, remove, items) {
    const end = list.length;
    let chunkStart = 0;
    /** @type {unknown[]} */

    let parameters; // Make start between zero and `end` (included).

    if (start < 0) {
      start = -start > end ? 0 : end + start;
    } else {
      start = start > end ? end : start;
    }

    remove = remove > 0 ? remove : 0; // No need to chunk the items if there’s only a couple (10k) items.

    if (items.length < 10000) {
      parameters = Array.from(items);
      parameters.unshift(start, remove) // @ts-expect-error Hush, it’s fine.
      ;[].splice.apply(list, parameters);
    } else {
      // Delete `remove` items starting from `start`
      if (remove) [].splice.apply(list, [start, remove]); // Insert the items in chunks to not cause stack overflows.

      while (chunkStart < items.length) {
        parameters = items.slice(chunkStart, chunkStart + 10000);
        parameters.unshift(start, 0) // @ts-expect-error Hush, it’s fine.
        ;[].splice.apply(list, parameters);
        chunkStart += 10000;
        start += 10000;
      }
    }
  }
  /**
   * Append `items` (an array) at the end of `list` (another array).
   * When `list` was empty, returns `items` instead.
   *
   * This prevents a potentially expensive operation when `list` is empty,
   * and adds items in batches to prevent V8 from hanging.
   *
   * @template {unknown} T
   * @param {T[]} list
   * @param {T[]} items
   * @returns {T[]}
   */

  function push(list, items) {
    if (list.length > 0) {
      splice(list, list.length, 0, items);
      return list
    }

    return items
  }

  /**
   * @typedef {import('micromark-util-types').NormalizedExtension} NormalizedExtension
   * @typedef {import('micromark-util-types').Extension} Extension
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').HtmlExtension} HtmlExtension
   */

  const hasOwnProperty = {}.hasOwnProperty;

  /**
   * Combine several syntax extensions into one.
   *
   * @param {Extension[]} extensions List of syntax extensions.
   * @returns {NormalizedExtension} A single combined extension.
   */
  function combineExtensions(extensions) {
    /** @type {NormalizedExtension} */
    const all = {};
    let index = -1;

    while (++index < extensions.length) {
      syntaxExtension(all, extensions[index]);
    }

    return all
  }

  /**
   * Merge `extension` into `all`.
   *
   * @param {NormalizedExtension} all Extension to merge into.
   * @param {Extension} extension Extension to merge.
   * @returns {void}
   */
  function syntaxExtension(all, extension) {
    /** @type {string} */
    let hook;

    for (hook in extension) {
      const maybe = hasOwnProperty.call(all, hook) ? all[hook] : undefined;
      const left = maybe || (all[hook] = {});
      const right = extension[hook];
      /** @type {string} */
      let code;

      for (code in right) {
        if (!hasOwnProperty.call(left, code)) left[code] = [];
        const value = right[code];
        constructs(
          // @ts-expect-error Looks like a list.
          left[code],
          Array.isArray(value) ? value : value ? [value] : []
        );
      }
    }
  }

  /**
   * Merge `list` into `existing` (both lists of constructs).
   * Mutates `existing`.
   *
   * @param {unknown[]} existing
   * @param {unknown[]} list
   * @returns {void}
   */
  function constructs(existing, list) {
    let index = -1;
    /** @type {unknown[]} */
    const before = [];

    while (++index < list.length) {
  (list[index].add === 'after' ? existing : before).push(list[index]);
    }

    splice(existing, 0, 0, before);
  }

  // This module is generated by `script/`.
  //
  // CommonMark handles attention (emphasis, strong) markers based on what comes
  // before or after them.
  // One such difference is if those characters are Unicode punctuation.
  // This script is generated from the Unicode data.
  const unicodePunctuationRegex =
    /[!-/:-@[-`{-~\u00A1\u00A7\u00AB\u00B6\u00B7\u00BB\u00BF\u037E\u0387\u055A-\u055F\u0589\u058A\u05BE\u05C0\u05C3\u05C6\u05F3\u05F4\u0609\u060A\u060C\u060D\u061B\u061E\u061F\u066A-\u066D\u06D4\u0700-\u070D\u07F7-\u07F9\u0830-\u083E\u085E\u0964\u0965\u0970\u09FD\u0A76\u0AF0\u0C77\u0C84\u0DF4\u0E4F\u0E5A\u0E5B\u0F04-\u0F12\u0F14\u0F3A-\u0F3D\u0F85\u0FD0-\u0FD4\u0FD9\u0FDA\u104A-\u104F\u10FB\u1360-\u1368\u1400\u166E\u169B\u169C\u16EB-\u16ED\u1735\u1736\u17D4-\u17D6\u17D8-\u17DA\u1800-\u180A\u1944\u1945\u1A1E\u1A1F\u1AA0-\u1AA6\u1AA8-\u1AAD\u1B5A-\u1B60\u1BFC-\u1BFF\u1C3B-\u1C3F\u1C7E\u1C7F\u1CC0-\u1CC7\u1CD3\u2010-\u2027\u2030-\u2043\u2045-\u2051\u2053-\u205E\u207D\u207E\u208D\u208E\u2308-\u230B\u2329\u232A\u2768-\u2775\u27C5\u27C6\u27E6-\u27EF\u2983-\u2998\u29D8-\u29DB\u29FC\u29FD\u2CF9-\u2CFC\u2CFE\u2CFF\u2D70\u2E00-\u2E2E\u2E30-\u2E4F\u2E52\u3001-\u3003\u3008-\u3011\u3014-\u301F\u3030\u303D\u30A0\u30FB\uA4FE\uA4FF\uA60D-\uA60F\uA673\uA67E\uA6F2-\uA6F7\uA874-\uA877\uA8CE\uA8CF\uA8F8-\uA8FA\uA8FC\uA92E\uA92F\uA95F\uA9C1-\uA9CD\uA9DE\uA9DF\uAA5C-\uAA5F\uAADE\uAADF\uAAF0\uAAF1\uABEB\uFD3E\uFD3F\uFE10-\uFE19\uFE30-\uFE52\uFE54-\uFE61\uFE63\uFE68\uFE6A\uFE6B\uFF01-\uFF03\uFF05-\uFF0A\uFF0C-\uFF0F\uFF1A\uFF1B\uFF1F\uFF20\uFF3B-\uFF3D\uFF3F\uFF5B\uFF5D\uFF5F-\uFF65]/;

  /**
   * @typedef {import('micromark-util-types').Code} Code
   */
  /**
   * Check whether the character code represents an ASCII alpha (`a` through `z`,
   * case insensitive).
   *
   * An **ASCII alpha** is an ASCII upper alpha or ASCII lower alpha.
   *
   * An **ASCII upper alpha** is a character in the inclusive range U+0041 (`A`)
   * to U+005A (`Z`).
   *
   * An **ASCII lower alpha** is a character in the inclusive range U+0061 (`a`)
   * to U+007A (`z`).
   */

  const asciiAlpha = regexCheck(/[A-Za-z]/);
  /**
   * Check whether the character code represents an ASCII digit (`0` through `9`).
   *
   * An **ASCII digit** is a character in the inclusive range U+0030 (`0`) to
   * U+0039 (`9`).
   */

  const asciiDigit = regexCheck(/\d/);
  /**
   * Check whether the character code represents an ASCII hex digit (`a` through
   * `f`, case insensitive, or `0` through `9`).
   *
   * An **ASCII hex digit** is an ASCII digit (see `asciiDigit`), ASCII upper hex
   * digit, or an ASCII lower hex digit.
   *
   * An **ASCII upper hex digit** is a character in the inclusive range U+0041
   * (`A`) to U+0046 (`F`).
   *
   * An **ASCII lower hex digit** is a character in the inclusive range U+0061
   * (`a`) to U+0066 (`f`).
   */

  const asciiHexDigit = regexCheck(/[\dA-Fa-f]/);
  /**
   * Check whether the character code represents an ASCII alphanumeric (`a`
   * through `z`, case insensitive, or `0` through `9`).
   *
   * An **ASCII alphanumeric** is an ASCII digit (see `asciiDigit`) or ASCII alpha
   * (see `asciiAlpha`).
   */

  const asciiAlphanumeric = regexCheck(/[\dA-Za-z]/);
  /**
   * Check whether the character code represents ASCII punctuation.
   *
   * An **ASCII punctuation** is a character in the inclusive ranges U+0021
   * EXCLAMATION MARK (`!`) to U+002F SLASH (`/`), U+003A COLON (`:`) to U+0040 AT
   * SIGN (`@`), U+005B LEFT SQUARE BRACKET (`[`) to U+0060 GRAVE ACCENT
   * (`` ` ``), or U+007B LEFT CURLY BRACE (`{`) to U+007E TILDE (`~`).
   */

  const asciiPunctuation = regexCheck(/[!-/:-@[-`{-~]/);
  /**
   * Check whether the character code represents an ASCII atext.
   *
   * atext is an ASCII alphanumeric (see `asciiAlphanumeric`), or a character in
   * the inclusive ranges U+0023 NUMBER SIGN (`#`) to U+0027 APOSTROPHE (`'`),
   * U+002A ASTERISK (`*`), U+002B PLUS SIGN (`+`), U+002D DASH (`-`), U+002F
   * SLASH (`/`), U+003D EQUALS TO (`=`), U+003F QUESTION MARK (`?`), U+005E
   * CARET (`^`) to U+0060 GRAVE ACCENT (`` ` ``), or U+007B LEFT CURLY BRACE
   * (`{`) to U+007E TILDE (`~`).
   *
   * See:
   * **\[RFC5322]**:
   * [Internet Message Format](https://tools.ietf.org/html/rfc5322).
   * P. Resnick.
   * IETF.
   */

  const asciiAtext = regexCheck(/[#-'*+\--9=?A-Z^-~]/);
  /**
   * Check whether a character code is an ASCII control character.
   *
   * An **ASCII control** is a character in the inclusive range U+0000 NULL (NUL)
   * to U+001F (US), or U+007F (DEL).
   *
   * @param {Code} code
   * @returns {code is number}
   */

  function asciiControl(code) {
    return (
      // Special whitespace codes (which have negative values), C0 and Control
      // character DEL
      code !== null && (code < 32 || code === 127)
    )
  }
  /**
   * Check whether a character code is a markdown line ending (see
   * `markdownLineEnding`) or markdown space (see `markdownSpace`).
   *
   * @param {Code} code
   * @returns {code is number}
   */

  function markdownLineEndingOrSpace(code) {
    return code !== null && (code < 0 || code === 32)
  }
  /**
   * Check whether a character code is a markdown line ending.
   *
   * A **markdown line ending** is the virtual characters M-0003 CARRIAGE RETURN
   * LINE FEED (CRLF), M-0004 LINE FEED (LF) and M-0005 CARRIAGE RETURN (CR).
   *
   * In micromark, the actual character U+000A LINE FEED (LF) and U+000D CARRIAGE
   * RETURN (CR) are replaced by these virtual characters depending on whether
   * they occurred together.
   *
   * @param {Code} code
   * @returns {code is number}
   */

  function markdownLineEnding(code) {
    return code !== null && code < -2
  }
  /**
   * Check whether a character code is a markdown space.
   *
   * A **markdown space** is the concrete character U+0020 SPACE (SP) and the
   * virtual characters M-0001 VIRTUAL SPACE (VS) and M-0002 HORIZONTAL TAB (HT).
   *
   * In micromark, the actual character U+0009 CHARACTER TABULATION (HT) is
   * replaced by one M-0002 HORIZONTAL TAB (HT) and between 0 and 3 M-0001 VIRTUAL
   * SPACE (VS) characters, depending on the column at which the tab occurred.
   *
   * @param {Code} code
   * @returns {code is number}
   */

  function markdownSpace(code) {
    return code === -2 || code === -1 || code === 32
  }
  /**
   * Check whether the character code represents Unicode whitespace.
   *
   * Note that this does handle micromark specific markdown whitespace characters.
   * See `markdownLineEndingOrSpace` to check that.
   *
   * A **Unicode whitespace** is a character in the Unicode `Zs` (Separator,
   * Space) category, or U+0009 CHARACTER TABULATION (HT), U+000A LINE FEED (LF),
   * U+000C (FF), or U+000D CARRIAGE RETURN (CR) (**\[UNICODE]**).
   *
   * See:
   * **\[UNICODE]**:
   * [The Unicode Standard](https://www.unicode.org/versions/).
   * Unicode Consortium.
   */

  const unicodeWhitespace = regexCheck(/\s/);
  /**
   * Check whether the character code represents Unicode punctuation.
   *
   * A **Unicode punctuation** is a character in the Unicode `Pc` (Punctuation,
   * Connector), `Pd` (Punctuation, Dash), `Pe` (Punctuation, Close), `Pf`
   * (Punctuation, Final quote), `Pi` (Punctuation, Initial quote), `Po`
   * (Punctuation, Other), or `Ps` (Punctuation, Open) categories, or an ASCII
   * punctuation (see `asciiPunctuation`).
   *
   * See:
   * **\[UNICODE]**:
   * [The Unicode Standard](https://www.unicode.org/versions/).
   * Unicode Consortium.
   */
  // Size note: removing ASCII from the regex and using `asciiPunctuation` here
  // In fact adds to the bundle size.

  const unicodePunctuation = regexCheck(unicodePunctuationRegex);
  /**
   * Create a code check from a regex.
   *
   * @param {RegExp} regex
   * @returns {(code: Code) => code is number}
   */

  function regexCheck(regex) {
    return check
    /**
     * Check whether a code matches the bound regex.
     *
     * @param {Code} code Character code
     * @returns {code is number} Whether the character code matches the bound regex
     */

    function check(code) {
      return code !== null && regex.test(String.fromCharCode(code))
    }
  }

  /**
   * @typedef {import('micromark-util-types').Extension} Extension
   * @typedef {import('micromark-util-types').ConstructRecord} ConstructRecord
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').Previous} Previous
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Event} Event
   * @typedef {import('micromark-util-types').Code} Code
   */
  const www = {
    tokenize: tokenizeWww,
    partial: true
  };
  const domain = {
    tokenize: tokenizeDomain,
    partial: true
  };
  const path$2 = {
    tokenize: tokenizePath,
    partial: true
  };
  const punctuation = {
    tokenize: tokenizePunctuation,
    partial: true
  };
  const namedCharacterReference = {
    tokenize: tokenizeNamedCharacterReference,
    partial: true
  };
  const wwwAutolink = {
    tokenize: tokenizeWwwAutolink,
    previous: previousWww
  };
  const httpAutolink = {
    tokenize: tokenizeHttpAutolink,
    previous: previousHttp
  };
  const emailAutolink = {
    tokenize: tokenizeEmailAutolink,
    previous: previousEmail
  };
  /** @type {ConstructRecord} */

  const text$9 = {};
  /** @type {Extension} */

  const gfmAutolinkLiteral = {
    text: text$9
  };
  let code$1 = 48; // Add alphanumerics.

  while (code$1 < 123) {
    text$9[code$1] = emailAutolink;
    code$1++;
    if (code$1 === 58) code$1 = 65;
    else if (code$1 === 91) code$1 = 97;
  }

  text$9[43] = emailAutolink;
  text$9[45] = emailAutolink;
  text$9[46] = emailAutolink;
  text$9[95] = emailAutolink;
  text$9[72] = [emailAutolink, httpAutolink];
  text$9[104] = [emailAutolink, httpAutolink];
  text$9[87] = [emailAutolink, wwwAutolink];
  text$9[119] = [emailAutolink, wwwAutolink];
  /** @type {Tokenizer} */

  function tokenizeEmailAutolink(effects, ok, nok) {
    const self = this;
    /** @type {boolean} */

    let hasDot;
    /** @type {boolean|undefined} */

    let hasDigitInLastSegment;
    return start
    /** @type {State} */

    function start(code) {
      if (
        !gfmAtext(code) ||
        !previousEmail(self.previous) ||
        previousUnbalanced(self.events)
      ) {
        return nok(code)
      }

      effects.enter('literalAutolink');
      effects.enter('literalAutolinkEmail');
      return atext(code)
    }
    /** @type {State} */

    function atext(code) {
      if (gfmAtext(code)) {
        effects.consume(code);
        return atext
      }

      if (code === 64) {
        effects.consume(code);
        return label
      }

      return nok(code)
    }
    /** @type {State} */

    function label(code) {
      if (code === 46) {
        return effects.check(punctuation, done, dotContinuation)(code)
      }

      if (code === 45 || code === 95) {
        return effects.check(punctuation, nok, dashOrUnderscoreContinuation)(code)
      }

      if (asciiAlphanumeric(code)) {
        if (!hasDigitInLastSegment && asciiDigit(code)) {
          hasDigitInLastSegment = true;
        }

        effects.consume(code);
        return label
      }

      return done(code)
    }
    /** @type {State} */

    function dotContinuation(code) {
      effects.consume(code);
      hasDot = true;
      hasDigitInLastSegment = undefined;
      return label
    }
    /** @type {State} */

    function dashOrUnderscoreContinuation(code) {
      effects.consume(code);
      return afterDashOrUnderscore
    }
    /** @type {State} */

    function afterDashOrUnderscore(code) {
      if (code === 46) {
        return effects.check(punctuation, nok, dotContinuation)(code)
      }

      return label(code)
    }
    /** @type {State} */

    function done(code) {
      if (hasDot && !hasDigitInLastSegment) {
        effects.exit('literalAutolinkEmail');
        effects.exit('literalAutolink');
        return ok(code)
      }

      return nok(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizeWwwAutolink(effects, ok, nok) {
    const self = this;
    return start
    /** @type {State} */

    function start(code) {
      if (
        (code !== 87 && code !== 119) ||
        !previousWww(self.previous) ||
        previousUnbalanced(self.events)
      ) {
        return nok(code)
      }

      effects.enter('literalAutolink');
      effects.enter('literalAutolinkWww'); // For `www.` we check instead of attempt, because when it matches, GH
      // treats it as part of a domain (yes, it says a valid domain must come
      // after `www.`, but that’s not how it’s implemented by them).

      return effects.check(
        www,
        effects.attempt(domain, effects.attempt(path$2, done), nok),
        nok
      )(code)
    }
    /** @type {State} */

    function done(code) {
      effects.exit('literalAutolinkWww');
      effects.exit('literalAutolink');
      return ok(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizeHttpAutolink(effects, ok, nok) {
    const self = this;
    return start
    /** @type {State} */

    function start(code) {
      if (
        (code !== 72 && code !== 104) ||
        !previousHttp(self.previous) ||
        previousUnbalanced(self.events)
      ) {
        return nok(code)
      }

      effects.enter('literalAutolink');
      effects.enter('literalAutolinkHttp');
      effects.consume(code);
      return t1
    }
    /** @type {State} */

    function t1(code) {
      if (code === 84 || code === 116) {
        effects.consume(code);
        return t2
      }

      return nok(code)
    }
    /** @type {State} */

    function t2(code) {
      if (code === 84 || code === 116) {
        effects.consume(code);
        return p
      }

      return nok(code)
    }
    /** @type {State} */

    function p(code) {
      if (code === 80 || code === 112) {
        effects.consume(code);
        return s
      }

      return nok(code)
    }
    /** @type {State} */

    function s(code) {
      if (code === 83 || code === 115) {
        effects.consume(code);
        return colon
      }

      return colon(code)
    }
    /** @type {State} */

    function colon(code) {
      if (code === 58) {
        effects.consume(code);
        return slash1
      }

      return nok(code)
    }
    /** @type {State} */

    function slash1(code) {
      if (code === 47) {
        effects.consume(code);
        return slash2
      }

      return nok(code)
    }
    /** @type {State} */

    function slash2(code) {
      if (code === 47) {
        effects.consume(code);
        return after
      }

      return nok(code)
    }
    /** @type {State} */

    function after(code) {
      return code === null ||
        asciiControl(code) ||
        unicodeWhitespace(code) ||
        unicodePunctuation(code)
        ? nok(code)
        : effects.attempt(domain, effects.attempt(path$2, done), nok)(code)
    }
    /** @type {State} */

    function done(code) {
      effects.exit('literalAutolinkHttp');
      effects.exit('literalAutolink');
      return ok(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizeWww(effects, ok, nok) {
    return start
    /** @type {State} */

    function start(code) {
      effects.consume(code);
      return w2
    }
    /** @type {State} */

    function w2(code) {
      if (code === 87 || code === 119) {
        effects.consume(code);
        return w3
      }

      return nok(code)
    }
    /** @type {State} */

    function w3(code) {
      if (code === 87 || code === 119) {
        effects.consume(code);
        return dot
      }

      return nok(code)
    }
    /** @type {State} */

    function dot(code) {
      if (code === 46) {
        effects.consume(code);
        return after
      }

      return nok(code)
    }
    /** @type {State} */

    function after(code) {
      return code === null || markdownLineEnding(code) ? nok(code) : ok(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizeDomain(effects, ok, nok) {
    /** @type {boolean|undefined} */
    let hasUnderscoreInLastSegment;
    /** @type {boolean|undefined} */

    let hasUnderscoreInLastLastSegment;
    return domain
    /** @type {State} */

    function domain(code) {
      if (code === 38) {
        return effects.check(
          namedCharacterReference,
          done,
          punctuationContinuation
        )(code)
      }

      if (code === 46 || code === 95) {
        return effects.check(punctuation, done, punctuationContinuation)(code)
      } // GH documents that only alphanumerics (other than `-`, `.`, and `_`) can
      // occur, which sounds like ASCII only, but they also support `www.點看.com`,
      // so that’s Unicode.
      // Instead of some new production for Unicode alphanumerics, markdown
      // already has that for Unicode punctuation and whitespace, so use those.

      if (
        code === null ||
        asciiControl(code) ||
        unicodeWhitespace(code) ||
        (code !== 45 && unicodePunctuation(code))
      ) {
        return done(code)
      }

      effects.consume(code);
      return domain
    }
    /** @type {State} */

    function punctuationContinuation(code) {
      if (code === 46) {
        hasUnderscoreInLastLastSegment = hasUnderscoreInLastSegment;
        hasUnderscoreInLastSegment = undefined;
        effects.consume(code);
        return domain
      }

      if (code === 95) hasUnderscoreInLastSegment = true;
      effects.consume(code);
      return domain
    }
    /** @type {State} */

    function done(code) {
      if (!hasUnderscoreInLastLastSegment && !hasUnderscoreInLastSegment) {
        return ok(code)
      }

      return nok(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizePath(effects, ok) {
    let balance = 0;
    return inPath
    /** @type {State} */

    function inPath(code) {
      if (code === 38) {
        return effects.check(
          namedCharacterReference,
          ok,
          continuedPunctuation
        )(code)
      }

      if (code === 40) {
        balance++;
      }

      if (code === 41) {
        return effects.check(
          punctuation,
          parenAtPathEnd,
          continuedPunctuation
        )(code)
      }

      if (pathEnd(code)) {
        return ok(code)
      }

      if (trailingPunctuation(code)) {
        return effects.check(punctuation, ok, continuedPunctuation)(code)
      }

      effects.consume(code);
      return inPath
    }
    /** @type {State} */

    function continuedPunctuation(code) {
      effects.consume(code);
      return inPath
    }
    /** @type {State} */

    function parenAtPathEnd(code) {
      balance--;
      return balance < 0 ? ok(code) : continuedPunctuation(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizeNamedCharacterReference(effects, ok, nok) {
    return start
    /** @type {State} */

    function start(code) {
      effects.consume(code);
      return inside
    }
    /** @type {State} */

    function inside(code) {
      if (asciiAlpha(code)) {
        effects.consume(code);
        return inside
      }

      if (code === 59) {
        effects.consume(code);
        return after
      }

      return nok(code)
    }
    /** @type {State} */

    function after(code) {
      // If the named character reference is followed by the end of the path, it’s
      // not continued punctuation.
      return pathEnd(code) ? ok(code) : nok(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizePunctuation(effects, ok, nok) {
    return start
    /** @type {State} */

    function start(code) {
      effects.consume(code);
      return after
    }
    /** @type {State} */

    function after(code) {
      // Check the next.
      if (trailingPunctuation(code)) {
        effects.consume(code);
        return after
      } // If the punctuation marker is followed by the end of the path, it’s not
      // continued punctuation.

      return pathEnd(code) ? ok(code) : nok(code)
    }
  }
  /**
   * @param {Code} code
   * @returns {boolean}
   */

  function trailingPunctuation(code) {
    return (
      code === 33 ||
      code === 34 ||
      code === 39 ||
      code === 41 ||
      code === 42 ||
      code === 44 ||
      code === 46 ||
      code === 58 ||
      code === 59 ||
      code === 60 ||
      code === 63 ||
      code === 95 ||
      code === 126
    )
  }
  /**
   * @param {Code} code
   * @returns {boolean}
   */

  function pathEnd(code) {
    return code === null || code === 60 || markdownLineEndingOrSpace(code)
  }
  /**
   * @param {Code} code
   * @returns {boolean}
   */

  function gfmAtext(code) {
    return (
      code === 43 ||
      code === 45 ||
      code === 46 ||
      code === 95 ||
      asciiAlphanumeric(code)
    )
  }
  /** @type {Previous} */

  function previousWww(code) {
    return (
      code === null ||
      code === 40 ||
      code === 42 ||
      code === 95 ||
      code === 126 ||
      markdownLineEndingOrSpace(code)
    )
  }
  /** @type {Previous} */

  function previousHttp(code) {
    return code === null || !asciiAlpha(code)
  }
  /** @type {Previous} */

  function previousEmail(code) {
    return code !== 47 && previousHttp(code)
  }
  /**
   * @param {Array<Event>} events
   * @returns {boolean}
   */

  function previousUnbalanced(events) {
    let index = events.length;
    let result = false;

    while (index--) {
      const token = events[index][1];

      if (
        (token.type === 'labelLink' || token.type === 'labelImage') &&
        !token._balanced
      ) {
        result = true;
        break
      } // @ts-expect-error If we’ve seen this token, and it was marked as not
      // having any unbalanced bracket before it, we can exit.

      if (token._gfmAutolinkLiteralWalkedInto) {
        result = false;
        break
      }
    }

    if (events.length > 0 && !result) {
      // @ts-expect-error Mark the last token as “walked into” w/o finding
      // anything.
      events[events.length - 1][1]._gfmAutolinkLiteralWalkedInto = true;
    }

    return result
  }

  const characterReferences = {'"': 'quot', '&': 'amp', '<': 'lt', '>': 'gt'};

  /**
   * Encode only the dangerous HTML characters.
   *
   * This ensures that certain characters which have special meaning in HTML are
   * dealt with.
   * Technically, we can skip `>` and `"` in many cases, but CM includes them.
   *
   * @param {string} value
   * @returns {string}
   */
  function encode$2(value) {
    return value.replace(/["&<>]/g, replace)

    /**
     * @param {string} value
     * @returns {string}
     */
    function replace(value) {
      // @ts-expect-error Hush, it’s fine.
      return '&' + characterReferences[value] + ';'
    }
  }

  /**
   * Make a value safe for injection as a URL.
   *
   * This encodes unsafe characters with percent-encoding and skips already
   * encoded sequences (see `normalizeUri` below).
   * Further unsafe characters are encoded as character references (see
   * `micromark-util-encode`).
   *
   * Then, a regex of allowed protocols can be given, in which case the URL is
   * sanitized.
   * For example, `/^(https?|ircs?|mailto|xmpp)$/i` can be used for `a[href]`,
   * or `/^https?$/i` for `img[src]`.
   * If the URL includes an unknown protocol (one not matched by `protocol`, such
   * as a dangerous example, `javascript:`), the value is ignored.
   *
   * @param {string|undefined} url
   * @param {RegExp} [protocol]
   * @returns {string}
   */
  function sanitizeUri(url, protocol) {
    const value = encode$2(normalizeUri(url || ''));

    if (!protocol) {
      return value
    }

    const colon = value.indexOf(':');
    const questionMark = value.indexOf('?');
    const numberSign = value.indexOf('#');
    const slash = value.indexOf('/');

    if (
      // If there is no protocol, it’s relative.
      colon < 0 || // If the first colon is after a `?`, `#`, or `/`, it’s not a protocol.
      (slash > -1 && colon > slash) ||
      (questionMark > -1 && colon > questionMark) ||
      (numberSign > -1 && colon > numberSign) || // It is a protocol, it should be allowed.
      protocol.test(value.slice(0, colon))
    ) {
      return value
    }

    return ''
  }
  /**
   * Normalize a URL (such as used in definitions).
   *
   * Encode unsafe characters with percent-encoding, skipping already encoded
   * sequences.
   *
   * @param {string} value
   * @returns {string}
   */

  function normalizeUri(value) {
    /** @type {string[]} */
    const result = [];
    let index = -1;
    let start = 0;
    let skip = 0;

    while (++index < value.length) {
      const code = value.charCodeAt(index);
      /** @type {string} */

      let replace = ''; // A correct percent encoded value.

      if (
        code === 37 &&
        asciiAlphanumeric(value.charCodeAt(index + 1)) &&
        asciiAlphanumeric(value.charCodeAt(index + 2))
      ) {
        skip = 2;
      } // ASCII.
      else if (code < 128) {
        if (!/[!#$&-;=?-Z_a-z~]/.test(String.fromCharCode(code))) {
          replace = String.fromCharCode(code);
        }
      } // Astral.
      else if (code > 55295 && code < 57344) {
        const next = value.charCodeAt(index + 1); // A correct surrogate pair.

        if (code < 56320 && next > 56319 && next < 57344) {
          replace = String.fromCharCode(code, next);
          skip = 1;
        } // Lone surrogate.
        else {
          replace = '\uFFFD';
        }
      } // Unicode.
      else {
        replace = String.fromCharCode(code);
      }

      if (replace) {
        result.push(value.slice(start, index), encodeURIComponent(replace));
        start = index + skip + 1;
        replace = '';
      }

      if (skip) {
        index += skip;
        skip = 0;
      }
    }

    return result.join('') + value.slice(start)
  }

  /**
   * @typedef {import('micromark-util-types').Code} Code
   */

  /**
   * Classify whether a character code represents whitespace, punctuation, or
   * something else.
   *
   * Used for attention (emphasis, strong), whose sequences can open or close
   * based on the class of surrounding characters.
   *
   * Note that eof (`null`) is seen as whitespace.
   *
   * @param {Code} code
   * @returns {number|undefined}
   */
  function classifyCharacter(code) {
    if (
      code === null ||
      markdownLineEndingOrSpace(code) ||
      unicodeWhitespace(code)
    ) {
      return 1
    }

    if (unicodePunctuation(code)) {
      return 2
    }
  }

  /**
   * @typedef {import('micromark-util-types').TokenizeContext} TokenizeContext
   * @typedef {import('micromark-util-types').Event} Event
   * @typedef {import('micromark-util-types').Resolver} Resolver
   */

  /**
   * Call all `resolveAll`s.
   *
   * @param {{resolveAll?: Resolver}[]} constructs
   * @param {Event[]} events
   * @param {TokenizeContext} context
   * @returns {Event[]}
   */
  function resolveAll(constructs, events, context) {
    /** @type {Resolver[]} */
    const called = [];
    let index = -1;

    while (++index < constructs.length) {
      const resolve = constructs[index].resolveAll;

      if (resolve && !called.includes(resolve)) {
        events = resolve(events, context);
        called.push(resolve);
      }
    }

    return events
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').Resolver} Resolver
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Token} Token
   * @typedef {import('micromark-util-types').Event} Event
   * @typedef {import('micromark-util-types').Code} Code
   * @typedef {import('micromark-util-types').Point} Point
   */

  /** @type {Construct} */
  const attention = {
    name: 'attention',
    tokenize: tokenizeAttention,
    resolveAll: resolveAllAttention
  };
  /**
   * Take all events and resolve attention to emphasis or strong.
   *
   * @type {Resolver}
   */

  function resolveAllAttention(events, context) {
    let index = -1;
    /** @type {number} */

    let open;
    /** @type {Token} */

    let group;
    /** @type {Token} */

    let text;
    /** @type {Token} */

    let openingSequence;
    /** @type {Token} */

    let closingSequence;
    /** @type {number} */

    let use;
    /** @type {Event[]} */

    let nextEvents;
    /** @type {number} */

    let offset; // Walk through all events.
    //
    // Note: performance of this is fine on an mb of normal markdown, but it’s
    // a bottleneck for malicious stuff.

    while (++index < events.length) {
      // Find a token that can close.
      if (
        events[index][0] === 'enter' &&
        events[index][1].type === 'attentionSequence' &&
        events[index][1]._close
      ) {
        open = index; // Now walk back to find an opener.

        while (open--) {
          // Find a token that can open the closer.
          if (
            events[open][0] === 'exit' &&
            events[open][1].type === 'attentionSequence' &&
            events[open][1]._open && // If the markers are the same:
            context.sliceSerialize(events[open][1]).charCodeAt(0) ===
              context.sliceSerialize(events[index][1]).charCodeAt(0)
          ) {
            // If the opening can close or the closing can open,
            // and the close size *is not* a multiple of three,
            // but the sum of the opening and closing size *is* multiple of three,
            // then don’t match.
            if (
              (events[open][1]._close || events[index][1]._open) &&
              (events[index][1].end.offset - events[index][1].start.offset) % 3 &&
              !(
                (events[open][1].end.offset -
                  events[open][1].start.offset +
                  events[index][1].end.offset -
                  events[index][1].start.offset) %
                3
              )
            ) {
              continue
            } // Number of markers to use from the sequence.

            use =
              events[open][1].end.offset - events[open][1].start.offset > 1 &&
              events[index][1].end.offset - events[index][1].start.offset > 1
                ? 2
                : 1;
            const start = Object.assign({}, events[open][1].end);
            const end = Object.assign({}, events[index][1].start);
            movePoint(start, -use);
            movePoint(end, use);
            openingSequence = {
              type: use > 1 ? 'strongSequence' : 'emphasisSequence',
              start,
              end: Object.assign({}, events[open][1].end)
            };
            closingSequence = {
              type: use > 1 ? 'strongSequence' : 'emphasisSequence',
              start: Object.assign({}, events[index][1].start),
              end
            };
            text = {
              type: use > 1 ? 'strongText' : 'emphasisText',
              start: Object.assign({}, events[open][1].end),
              end: Object.assign({}, events[index][1].start)
            };
            group = {
              type: use > 1 ? 'strong' : 'emphasis',
              start: Object.assign({}, openingSequence.start),
              end: Object.assign({}, closingSequence.end)
            };
            events[open][1].end = Object.assign({}, openingSequence.start);
            events[index][1].start = Object.assign({}, closingSequence.end);
            nextEvents = []; // If there are more markers in the opening, add them before.

            if (events[open][1].end.offset - events[open][1].start.offset) {
              nextEvents = push(nextEvents, [
                ['enter', events[open][1], context],
                ['exit', events[open][1], context]
              ]);
            } // Opening.

            nextEvents = push(nextEvents, [
              ['enter', group, context],
              ['enter', openingSequence, context],
              ['exit', openingSequence, context],
              ['enter', text, context]
            ]); // Between.

            nextEvents = push(
              nextEvents,
              resolveAll(
                context.parser.constructs.insideSpan.null,
                events.slice(open + 1, index),
                context
              )
            ); // Closing.

            nextEvents = push(nextEvents, [
              ['exit', text, context],
              ['enter', closingSequence, context],
              ['exit', closingSequence, context],
              ['exit', group, context]
            ]); // If there are more markers in the closing, add them after.

            if (events[index][1].end.offset - events[index][1].start.offset) {
              offset = 2;
              nextEvents = push(nextEvents, [
                ['enter', events[index][1], context],
                ['exit', events[index][1], context]
              ]);
            } else {
              offset = 0;
            }

            splice(events, open - 1, index - open + 3, nextEvents);
            index = open + nextEvents.length - offset - 2;
            break
          }
        }
      }
    } // Remove remaining sequences.

    index = -1;

    while (++index < events.length) {
      if (events[index][1].type === 'attentionSequence') {
        events[index][1].type = 'data';
      }
    }

    return events
  }
  /** @type {Tokenizer} */

  function tokenizeAttention(effects, ok) {
    const attentionMarkers = this.parser.constructs.attentionMarkers.null;
    const previous = this.previous;
    const before = classifyCharacter(previous);
    /** @type {NonNullable<Code>} */

    let marker;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('attentionSequence');
      marker = code;
      return sequence(code)
    }
    /** @type {State} */

    function sequence(code) {
      if (code === marker) {
        effects.consume(code);
        return sequence
      }

      const token = effects.exit('attentionSequence');
      const after = classifyCharacter(code);
      const open =
        !after || (after === 2 && before) || attentionMarkers.includes(code);
      const close =
        !before || (before === 2 && after) || attentionMarkers.includes(previous);
      token._open = Boolean(marker === 42 ? open : open && (before || !close));
      token._close = Boolean(marker === 42 ? close : close && (after || !open));
      return ok(code)
    }
  }
  /**
   * Move a point a bit.
   *
   * Note: `move` only works inside lines! It’s not possible to move past other
   * chunks (replacement characters, tabs, or line endings).
   *
   * @param {Point} point
   * @param {number} offset
   * @returns {void}
   */

  function movePoint(point, offset) {
    point.column += offset;
    point.offset += offset;
    point._bufferIndex += offset;
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   */

  /** @type {Construct} */
  const autolink = {
    name: 'autolink',
    tokenize: tokenizeAutolink
  };
  /** @type {Tokenizer} */

  function tokenizeAutolink(effects, ok, nok) {
    let size = 1;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('autolink');
      effects.enter('autolinkMarker');
      effects.consume(code);
      effects.exit('autolinkMarker');
      effects.enter('autolinkProtocol');
      return open
    }
    /** @type {State} */

    function open(code) {
      if (asciiAlpha(code)) {
        effects.consume(code);
        return schemeOrEmailAtext
      }

      return asciiAtext(code) ? emailAtext(code) : nok(code)
    }
    /** @type {State} */

    function schemeOrEmailAtext(code) {
      return code === 43 || code === 45 || code === 46 || asciiAlphanumeric(code)
        ? schemeInsideOrEmailAtext(code)
        : emailAtext(code)
    }
    /** @type {State} */

    function schemeInsideOrEmailAtext(code) {
      if (code === 58) {
        effects.consume(code);
        return urlInside
      }

      if (
        (code === 43 || code === 45 || code === 46 || asciiAlphanumeric(code)) &&
        size++ < 32
      ) {
        effects.consume(code);
        return schemeInsideOrEmailAtext
      }

      return emailAtext(code)
    }
    /** @type {State} */

    function urlInside(code) {
      if (code === 62) {
        effects.exit('autolinkProtocol');
        return end(code)
      }

      if (code === null || code === 32 || code === 60 || asciiControl(code)) {
        return nok(code)
      }

      effects.consume(code);
      return urlInside
    }
    /** @type {State} */

    function emailAtext(code) {
      if (code === 64) {
        effects.consume(code);
        size = 0;
        return emailAtSignOrDot
      }

      if (asciiAtext(code)) {
        effects.consume(code);
        return emailAtext
      }

      return nok(code)
    }
    /** @type {State} */

    function emailAtSignOrDot(code) {
      return asciiAlphanumeric(code) ? emailLabel(code) : nok(code)
    }
    /** @type {State} */

    function emailLabel(code) {
      if (code === 46) {
        effects.consume(code);
        size = 0;
        return emailAtSignOrDot
      }

      if (code === 62) {
        // Exit, then change the type.
        effects.exit('autolinkProtocol').type = 'autolinkEmail';
        return end(code)
      }

      return emailValue(code)
    }
    /** @type {State} */

    function emailValue(code) {
      if ((code === 45 || asciiAlphanumeric(code)) && size++ < 63) {
        effects.consume(code);
        return code === 45 ? emailValue : emailLabel
      }

      return nok(code)
    }
    /** @type {State} */

    function end(code) {
      effects.enter('autolinkMarker');
      effects.consume(code);
      effects.exit('autolinkMarker');
      effects.exit('autolink');
      return ok
    }
  }

  /**
   * @typedef {import('micromark-util-types').Effects} Effects
   * @typedef {import('micromark-util-types').State} State
   */
  /**
   * @param {Effects} effects
   * @param {State} ok
   * @param {string} type
   * @param {number} [max=Infinity]
   * @returns {State}
   */

  function factorySpace(effects, ok, type, max) {
    const limit = max ? max - 1 : Number.POSITIVE_INFINITY;
    let size = 0;
    return start
    /** @type {State} */

    function start(code) {
      if (markdownSpace(code)) {
        effects.enter(type);
        return prefix(code)
      }

      return ok(code)
    }
    /** @type {State} */

    function prefix(code) {
      if (markdownSpace(code) && size++ < limit) {
        effects.consume(code);
        return prefix
      }

      effects.exit(type);
      return ok(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   */

  /** @type {Construct} */
  const blankLine = {
    tokenize: tokenizeBlankLine,
    partial: true
  };
  /** @type {Tokenizer} */

  function tokenizeBlankLine(effects, ok, nok) {
    return factorySpace(effects, afterWhitespace, 'linePrefix')
    /** @type {State} */

    function afterWhitespace(code) {
      return code === null || markdownLineEnding(code) ? ok(code) : nok(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').Exiter} Exiter
   * @typedef {import('micromark-util-types').State} State
   */

  /** @type {Construct} */
  const blockQuote = {
    name: 'blockQuote',
    tokenize: tokenizeBlockQuoteStart,
    continuation: {
      tokenize: tokenizeBlockQuoteContinuation
    },
    exit: exit$1
  };
  /** @type {Tokenizer} */

  function tokenizeBlockQuoteStart(effects, ok, nok) {
    const self = this;
    return start
    /** @type {State} */

    function start(code) {
      if (code === 62) {
        const state = self.containerState;

        if (!state.open) {
          effects.enter('blockQuote', {
            _container: true
          });
          state.open = true;
        }

        effects.enter('blockQuotePrefix');
        effects.enter('blockQuoteMarker');
        effects.consume(code);
        effects.exit('blockQuoteMarker');
        return after
      }

      return nok(code)
    }
    /** @type {State} */

    function after(code) {
      if (markdownSpace(code)) {
        effects.enter('blockQuotePrefixWhitespace');
        effects.consume(code);
        effects.exit('blockQuotePrefixWhitespace');
        effects.exit('blockQuotePrefix');
        return ok
      }

      effects.exit('blockQuotePrefix');
      return ok(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizeBlockQuoteContinuation(effects, ok, nok) {
    return factorySpace(
      effects,
      effects.attempt(blockQuote, ok, nok),
      'linePrefix',
      this.parser.constructs.disable.null.includes('codeIndented') ? undefined : 4
    )
  }
  /** @type {Exiter} */

  function exit$1(effects) {
    effects.exit('blockQuote');
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   */

  /** @type {Construct} */
  const characterEscape = {
    name: 'characterEscape',
    tokenize: tokenizeCharacterEscape
  };
  /** @type {Tokenizer} */

  function tokenizeCharacterEscape(effects, ok, nok) {
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('characterEscape');
      effects.enter('escapeMarker');
      effects.consume(code);
      effects.exit('escapeMarker');
      return open
    }
    /** @type {State} */

    function open(code) {
      if (asciiPunctuation(code)) {
        effects.enter('characterEscapeValue');
        effects.consume(code);
        effects.exit('characterEscapeValue');
        effects.exit('characterEscape');
        return ok
      }

      return nok(code)
    }
  }

  /// <reference lib="dom" />

  /* eslint-env browser */

  const element$9 = document.createElement('i');

  /**
   * @param {string} value
   * @returns {string|false}
   */
  function decodeNamedCharacterReference(value) {
    const characterReference = '&' + value + ';';
    element$9.innerHTML = characterReference;
    const char = element$9.textContent;

    // Some named character references do not require the closing semicolon
    // (`&not`, for instance), which leads to situations where parsing the assumed
    // named reference of `&notit;` will result in the string `¬it;`.
    // When we encounter a trailing semicolon after parsing, and the character
    // reference to decode was not a semicolon (`&semi;`), we can assume that the
    // matching was not complete.
    // @ts-expect-error: TypeScript is wrong that `textContent` on elements can
    // yield `null`.
    if (char.charCodeAt(char.length - 1) === 59 /* `;` */ && value !== 'semi') {
      return false
    }

    // If the decoded string is equal to the input, the character reference was
    // not valid.
    // @ts-expect-error: TypeScript is wrong that `textContent` on elements can
    // yield `null`.
    return char === characterReference ? false : char
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').Token} Token
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Code} Code
   */

  /** @type {Construct} */
  const characterReference = {
    name: 'characterReference',
    tokenize: tokenizeCharacterReference
  };
  /** @type {Tokenizer} */

  function tokenizeCharacterReference(effects, ok, nok) {
    const self = this;
    let size = 0;
    /** @type {number} */

    let max;
    /** @type {(code: Code) => code is number} */

    let test;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('characterReference');
      effects.enter('characterReferenceMarker');
      effects.consume(code);
      effects.exit('characterReferenceMarker');
      return open
    }
    /** @type {State} */

    function open(code) {
      if (code === 35) {
        effects.enter('characterReferenceMarkerNumeric');
        effects.consume(code);
        effects.exit('characterReferenceMarkerNumeric');
        return numeric
      }

      effects.enter('characterReferenceValue');
      max = 31;
      test = asciiAlphanumeric;
      return value(code)
    }
    /** @type {State} */

    function numeric(code) {
      if (code === 88 || code === 120) {
        effects.enter('characterReferenceMarkerHexadecimal');
        effects.consume(code);
        effects.exit('characterReferenceMarkerHexadecimal');
        effects.enter('characterReferenceValue');
        max = 6;
        test = asciiHexDigit;
        return value
      }

      effects.enter('characterReferenceValue');
      max = 7;
      test = asciiDigit;
      return value(code)
    }
    /** @type {State} */

    function value(code) {
      /** @type {Token} */
      let token;

      if (code === 59 && size) {
        token = effects.exit('characterReferenceValue');

        if (
          test === asciiAlphanumeric &&
          !decodeNamedCharacterReference(self.sliceSerialize(token))
        ) {
          return nok(code)
        }

        effects.enter('characterReferenceMarker');
        effects.consume(code);
        effects.exit('characterReferenceMarker');
        effects.exit('characterReference');
        return ok
      }

      if (test(code) && size++ < max) {
        effects.consume(code);
        return value
      }

      return nok(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Code} Code
   */

  /** @type {Construct} */
  const codeFenced = {
    name: 'codeFenced',
    tokenize: tokenizeCodeFenced,
    concrete: true
  };
  /** @type {Tokenizer} */

  function tokenizeCodeFenced(effects, ok, nok) {
    const self = this;
    /** @type {Construct} */

    const closingFenceConstruct = {
      tokenize: tokenizeClosingFence,
      partial: true
    };
    /** @type {Construct} */

    const nonLazyLine = {
      tokenize: tokenizeNonLazyLine,
      partial: true
    };
    const tail = this.events[this.events.length - 1];
    const initialPrefix =
      tail && tail[1].type === 'linePrefix'
        ? tail[2].sliceSerialize(tail[1], true).length
        : 0;
    let sizeOpen = 0;
    /** @type {NonNullable<Code>} */

    let marker;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('codeFenced');
      effects.enter('codeFencedFence');
      effects.enter('codeFencedFenceSequence');
      marker = code;
      return sequenceOpen(code)
    }
    /** @type {State} */

    function sequenceOpen(code) {
      if (code === marker) {
        effects.consume(code);
        sizeOpen++;
        return sequenceOpen
      }

      effects.exit('codeFencedFenceSequence');
      return sizeOpen < 3
        ? nok(code)
        : factorySpace(effects, infoOpen, 'whitespace')(code)
    }
    /** @type {State} */

    function infoOpen(code) {
      if (code === null || markdownLineEnding(code)) {
        return openAfter(code)
      }

      effects.enter('codeFencedFenceInfo');
      effects.enter('chunkString', {
        contentType: 'string'
      });
      return info(code)
    }
    /** @type {State} */

    function info(code) {
      if (code === null || markdownLineEndingOrSpace(code)) {
        effects.exit('chunkString');
        effects.exit('codeFencedFenceInfo');
        return factorySpace(effects, infoAfter, 'whitespace')(code)
      }

      if (code === 96 && code === marker) return nok(code)
      effects.consume(code);
      return info
    }
    /** @type {State} */

    function infoAfter(code) {
      if (code === null || markdownLineEnding(code)) {
        return openAfter(code)
      }

      effects.enter('codeFencedFenceMeta');
      effects.enter('chunkString', {
        contentType: 'string'
      });
      return meta(code)
    }
    /** @type {State} */

    function meta(code) {
      if (code === null || markdownLineEnding(code)) {
        effects.exit('chunkString');
        effects.exit('codeFencedFenceMeta');
        return openAfter(code)
      }

      if (code === 96 && code === marker) return nok(code)
      effects.consume(code);
      return meta
    }
    /** @type {State} */

    function openAfter(code) {
      effects.exit('codeFencedFence');
      return self.interrupt ? ok(code) : contentStart(code)
    }
    /** @type {State} */

    function contentStart(code) {
      if (code === null) {
        return after(code)
      }

      if (markdownLineEnding(code)) {
        return effects.attempt(
          nonLazyLine,
          effects.attempt(
            closingFenceConstruct,
            after,
            initialPrefix
              ? factorySpace(
                  effects,
                  contentStart,
                  'linePrefix',
                  initialPrefix + 1
                )
              : contentStart
          ),
          after
        )(code)
      }

      effects.enter('codeFlowValue');
      return contentContinue(code)
    }
    /** @type {State} */

    function contentContinue(code) {
      if (code === null || markdownLineEnding(code)) {
        effects.exit('codeFlowValue');
        return contentStart(code)
      }

      effects.consume(code);
      return contentContinue
    }
    /** @type {State} */

    function after(code) {
      effects.exit('codeFenced');
      return ok(code)
    }
    /** @type {Tokenizer} */

    function tokenizeNonLazyLine(effects, ok, nok) {
      const self = this;
      return start
      /** @type {State} */

      function start(code) {
        effects.enter('lineEnding');
        effects.consume(code);
        effects.exit('lineEnding');
        return lineStart
      }
      /** @type {State} */

      function lineStart(code) {
        return self.parser.lazy[self.now().line] ? nok(code) : ok(code)
      }
    }
    /** @type {Tokenizer} */

    function tokenizeClosingFence(effects, ok, nok) {
      let size = 0;
      return factorySpace(
        effects,
        closingSequenceStart,
        'linePrefix',
        this.parser.constructs.disable.null.includes('codeIndented')
          ? undefined
          : 4
      )
      /** @type {State} */

      function closingSequenceStart(code) {
        effects.enter('codeFencedFence');
        effects.enter('codeFencedFenceSequence');
        return closingSequence(code)
      }
      /** @type {State} */

      function closingSequence(code) {
        if (code === marker) {
          effects.consume(code);
          size++;
          return closingSequence
        }

        if (size < sizeOpen) return nok(code)
        effects.exit('codeFencedFenceSequence');
        return factorySpace(effects, closingSequenceEnd, 'whitespace')(code)
      }
      /** @type {State} */

      function closingSequenceEnd(code) {
        if (code === null || markdownLineEnding(code)) {
          effects.exit('codeFencedFence');
          return ok(code)
        }

        return nok(code)
      }
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').Resolver} Resolver
   * @typedef {import('micromark-util-types').Token} Token
   * @typedef {import('micromark-util-types').State} State
   */

  /** @type {Construct} */
  const codeIndented = {
    name: 'codeIndented',
    tokenize: tokenizeCodeIndented
  };
  /** @type {Construct} */

  const indentedContent = {
    tokenize: tokenizeIndentedContent,
    partial: true
  };
  /** @type {Tokenizer} */

  function tokenizeCodeIndented(effects, ok, nok) {
    const self = this;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('codeIndented');
      return factorySpace(effects, afterStartPrefix, 'linePrefix', 4 + 1)(code)
    }
    /** @type {State} */

    function afterStartPrefix(code) {
      const tail = self.events[self.events.length - 1];
      return tail &&
        tail[1].type === 'linePrefix' &&
        tail[2].sliceSerialize(tail[1], true).length >= 4
        ? afterPrefix(code)
        : nok(code)
    }
    /** @type {State} */

    function afterPrefix(code) {
      if (code === null) {
        return after(code)
      }

      if (markdownLineEnding(code)) {
        return effects.attempt(indentedContent, afterPrefix, after)(code)
      }

      effects.enter('codeFlowValue');
      return content(code)
    }
    /** @type {State} */

    function content(code) {
      if (code === null || markdownLineEnding(code)) {
        effects.exit('codeFlowValue');
        return afterPrefix(code)
      }

      effects.consume(code);
      return content
    }
    /** @type {State} */

    function after(code) {
      effects.exit('codeIndented');
      return ok(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizeIndentedContent(effects, ok, nok) {
    const self = this;
    return start
    /** @type {State} */

    function start(code) {
      // If this is a lazy line, it can’t be code.
      if (self.parser.lazy[self.now().line]) {
        return nok(code)
      }

      if (markdownLineEnding(code)) {
        effects.enter('lineEnding');
        effects.consume(code);
        effects.exit('lineEnding');
        return start
      }

      return factorySpace(effects, afterPrefix, 'linePrefix', 4 + 1)(code)
    }
    /** @type {State} */

    function afterPrefix(code) {
      const tail = self.events[self.events.length - 1];
      return tail &&
        tail[1].type === 'linePrefix' &&
        tail[2].sliceSerialize(tail[1], true).length >= 4
        ? ok(code)
        : markdownLineEnding(code)
        ? start(code)
        : nok(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Resolver} Resolver
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').Previous} Previous
   * @typedef {import('micromark-util-types').Token} Token
   * @typedef {import('micromark-util-types').State} State
   */

  /** @type {Construct} */
  const codeText = {
    name: 'codeText',
    tokenize: tokenizeCodeText,
    resolve: resolveCodeText,
    previous: previous$1
  };
  /** @type {Resolver} */

  function resolveCodeText(events) {
    let tailExitIndex = events.length - 4;
    let headEnterIndex = 3;
    /** @type {number} */

    let index;
    /** @type {number|undefined} */

    let enter; // If we start and end with an EOL or a space.

    if (
      (events[headEnterIndex][1].type === 'lineEnding' ||
        events[headEnterIndex][1].type === 'space') &&
      (events[tailExitIndex][1].type === 'lineEnding' ||
        events[tailExitIndex][1].type === 'space')
    ) {
      index = headEnterIndex; // And we have data.

      while (++index < tailExitIndex) {
        if (events[index][1].type === 'codeTextData') {
          // Then we have padding.
          events[headEnterIndex][1].type = 'codeTextPadding';
          events[tailExitIndex][1].type = 'codeTextPadding';
          headEnterIndex += 2;
          tailExitIndex -= 2;
          break
        }
      }
    } // Merge adjacent spaces and data.

    index = headEnterIndex - 1;
    tailExitIndex++;

    while (++index <= tailExitIndex) {
      if (enter === undefined) {
        if (index !== tailExitIndex && events[index][1].type !== 'lineEnding') {
          enter = index;
        }
      } else if (
        index === tailExitIndex ||
        events[index][1].type === 'lineEnding'
      ) {
        events[enter][1].type = 'codeTextData';

        if (index !== enter + 2) {
          events[enter][1].end = events[index - 1][1].end;
          events.splice(enter + 2, index - enter - 2);
          tailExitIndex -= index - enter - 2;
          index = enter + 2;
        }

        enter = undefined;
      }
    }

    return events
  }
  /** @type {Previous} */

  function previous$1(code) {
    // If there is a previous code, there will always be a tail.
    return (
      code !== 96 ||
      this.events[this.events.length - 1][1].type === 'characterEscape'
    )
  }
  /** @type {Tokenizer} */

  function tokenizeCodeText(effects, ok, nok) {
    let sizeOpen = 0;
    /** @type {number} */

    let size;
    /** @type {Token} */

    let token;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('codeText');
      effects.enter('codeTextSequence');
      return openingSequence(code)
    }
    /** @type {State} */

    function openingSequence(code) {
      if (code === 96) {
        effects.consume(code);
        sizeOpen++;
        return openingSequence
      }

      effects.exit('codeTextSequence');
      return gap(code)
    }
    /** @type {State} */

    function gap(code) {
      // EOF.
      if (code === null) {
        return nok(code)
      } // Closing fence?
      // Could also be data.

      if (code === 96) {
        token = effects.enter('codeTextSequence');
        size = 0;
        return closingSequence(code)
      } // Tabs don’t work, and virtual spaces don’t make sense.

      if (code === 32) {
        effects.enter('space');
        effects.consume(code);
        effects.exit('space');
        return gap
      }

      if (markdownLineEnding(code)) {
        effects.enter('lineEnding');
        effects.consume(code);
        effects.exit('lineEnding');
        return gap
      } // Data.

      effects.enter('codeTextData');
      return data(code)
    } // In code.

    /** @type {State} */

    function data(code) {
      if (
        code === null ||
        code === 32 ||
        code === 96 ||
        markdownLineEnding(code)
      ) {
        effects.exit('codeTextData');
        return gap(code)
      }

      effects.consume(code);
      return data
    } // Closing fence.

    /** @type {State} */

    function closingSequence(code) {
      // More.
      if (code === 96) {
        effects.consume(code);
        size++;
        return closingSequence
      } // Done!

      if (size === sizeOpen) {
        effects.exit('codeTextSequence');
        effects.exit('codeText');
        return ok(code)
      } // More or less accents: mark as data.

      token.type = 'codeTextData';
      return data(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Token} Token
   * @typedef {import('micromark-util-types').Chunk} Chunk
   * @typedef {import('micromark-util-types').Event} Event
   */

  /**
   * Tokenize subcontent.
   *
   * @param {Event[]} events
   * @returns {boolean}
   */
  function subtokenize(events) {
    /** @type {Record<string, number>} */
    const jumps = {};
    let index = -1;
    /** @type {Event} */

    let event;
    /** @type {number|undefined} */

    let lineIndex;
    /** @type {number} */

    let otherIndex;
    /** @type {Event} */

    let otherEvent;
    /** @type {Event[]} */

    let parameters;
    /** @type {Event[]} */

    let subevents;
    /** @type {boolean|undefined} */

    let more;

    while (++index < events.length) {
      while (index in jumps) {
        index = jumps[index];
      }

      event = events[index]; // Add a hook for the GFM tasklist extension, which needs to know if text
      // is in the first content of a list item.

      if (
        index &&
        event[1].type === 'chunkFlow' &&
        events[index - 1][1].type === 'listItemPrefix'
      ) {
        subevents = event[1]._tokenizer.events;
        otherIndex = 0;

        if (
          otherIndex < subevents.length &&
          subevents[otherIndex][1].type === 'lineEndingBlank'
        ) {
          otherIndex += 2;
        }

        if (
          otherIndex < subevents.length &&
          subevents[otherIndex][1].type === 'content'
        ) {
          while (++otherIndex < subevents.length) {
            if (subevents[otherIndex][1].type === 'content') {
              break
            }

            if (subevents[otherIndex][1].type === 'chunkText') {
              subevents[otherIndex][1]._isInFirstContentOfListItem = true;
              otherIndex++;
            }
          }
        }
      } // Enter.

      if (event[0] === 'enter') {
        if (event[1].contentType) {
          Object.assign(jumps, subcontent(events, index));
          index = jumps[index];
          more = true;
        }
      } // Exit.
      else if (event[1]._container) {
        otherIndex = index;
        lineIndex = undefined;

        while (otherIndex--) {
          otherEvent = events[otherIndex];

          if (
            otherEvent[1].type === 'lineEnding' ||
            otherEvent[1].type === 'lineEndingBlank'
          ) {
            if (otherEvent[0] === 'enter') {
              if (lineIndex) {
                events[lineIndex][1].type = 'lineEndingBlank';
              }

              otherEvent[1].type = 'lineEnding';
              lineIndex = otherIndex;
            }
          } else {
            break
          }
        }

        if (lineIndex) {
          // Fix position.
          event[1].end = Object.assign({}, events[lineIndex][1].start); // Switch container exit w/ line endings.

          parameters = events.slice(lineIndex, index);
          parameters.unshift(event);
          splice(events, lineIndex, index - lineIndex + 1, parameters);
        }
      }
    }

    return !more
  }
  /**
   * Tokenize embedded tokens.
   *
   * @param {Event[]} events
   * @param {number} eventIndex
   * @returns {Record<string, number>}
   */

  function subcontent(events, eventIndex) {
    const token = events[eventIndex][1];
    const context = events[eventIndex][2];
    let startPosition = eventIndex - 1;
    /** @type {number[]} */

    const startPositions = [];
    const tokenizer =
      token._tokenizer || context.parser[token.contentType](token.start);
    const childEvents = tokenizer.events;
    /** @type {[number, number][]} */

    const jumps = [];
    /** @type {Record<string, number>} */

    const gaps = {};
    /** @type {Chunk[]} */

    let stream;
    /** @type {Token|undefined} */

    let previous;
    let index = -1;
    /** @type {Token|undefined} */

    let current = token;
    let adjust = 0;
    let start = 0;
    const breaks = [start]; // Loop forward through the linked tokens to pass them in order to the
    // subtokenizer.

    while (current) {
      // Find the position of the event for this token.
      while (events[++startPosition][1] !== current) {
        // Empty.
      }

      startPositions.push(startPosition);

      if (!current._tokenizer) {
        stream = context.sliceStream(current);

        if (!current.next) {
          stream.push(null);
        }

        if (previous) {
          tokenizer.defineSkip(current.start);
        }

        if (current._isInFirstContentOfListItem) {
          tokenizer._gfmTasklistFirstContentOfListItem = true;
        }

        tokenizer.write(stream);

        if (current._isInFirstContentOfListItem) {
          tokenizer._gfmTasklistFirstContentOfListItem = undefined;
        }
      } // Unravel the next token.

      previous = current;
      current = current.next;
    } // Now, loop back through all events (and linked tokens), to figure out which
    // parts belong where.

    current = token;

    while (++index < childEvents.length) {
      if (
        // Find a void token that includes a break.
        childEvents[index][0] === 'exit' &&
        childEvents[index - 1][0] === 'enter' &&
        childEvents[index][1].type === childEvents[index - 1][1].type &&
        childEvents[index][1].start.line !== childEvents[index][1].end.line
      ) {
        start = index + 1;
        breaks.push(start); // Help GC.

        current._tokenizer = undefined;
        current.previous = undefined;
        current = current.next;
      }
    } // Help GC.

    tokenizer.events = []; // If there’s one more token (which is the cases for lines that end in an
    // EOF), that’s perfect: the last point we found starts it.
    // If there isn’t then make sure any remaining content is added to it.

    if (current) {
      // Help GC.
      current._tokenizer = undefined;
      current.previous = undefined;
    } else {
      breaks.pop();
    } // Now splice the events from the subtokenizer into the current events,
    // moving back to front so that splice indices aren’t affected.

    index = breaks.length;

    while (index--) {
      const slice = childEvents.slice(breaks[index], breaks[index + 1]);
      const start = startPositions.pop();
      jumps.unshift([start, start + slice.length - 1]);
      splice(events, start, 2, slice);
    }

    index = -1;

    while (++index < jumps.length) {
      gaps[adjust + jumps[index][0]] = adjust + jumps[index][1];
      adjust += jumps[index][1] - jumps[index][0] - 1;
    }

    return gaps
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Resolver} Resolver
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').Token} Token
   * @typedef {import('micromark-util-types').State} State
   */

  /**
   * No name because it must not be turned off.
   * @type {Construct}
   */
  const content$1 = {
    tokenize: tokenizeContent,
    resolve: resolveContent
  };
  /** @type {Construct} */

  const continuationConstruct = {
    tokenize: tokenizeContinuation,
    partial: true
  };
  /**
   * Content is transparent: it’s parsed right now. That way, definitions are also
   * parsed right now: before text in paragraphs (specifically, media) are parsed.
   *
   * @type {Resolver}
   */

  function resolveContent(events) {
    subtokenize(events);
    return events
  }
  /** @type {Tokenizer} */

  function tokenizeContent(effects, ok) {
    /** @type {Token} */
    let previous;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('content');
      previous = effects.enter('chunkContent', {
        contentType: 'content'
      });
      return data(code)
    }
    /** @type {State} */

    function data(code) {
      if (code === null) {
        return contentEnd(code)
      }

      if (markdownLineEnding(code)) {
        return effects.check(
          continuationConstruct,
          contentContinue,
          contentEnd
        )(code)
      } // Data.

      effects.consume(code);
      return data
    }
    /** @type {State} */

    function contentEnd(code) {
      effects.exit('chunkContent');
      effects.exit('content');
      return ok(code)
    }
    /** @type {State} */

    function contentContinue(code) {
      effects.consume(code);
      effects.exit('chunkContent');
      previous.next = effects.enter('chunkContent', {
        contentType: 'content',
        previous
      });
      previous = previous.next;
      return data
    }
  }
  /** @type {Tokenizer} */

  function tokenizeContinuation(effects, ok, nok) {
    const self = this;
    return startLookahead
    /** @type {State} */

    function startLookahead(code) {
      effects.exit('chunkContent');
      effects.enter('lineEnding');
      effects.consume(code);
      effects.exit('lineEnding');
      return factorySpace(effects, prefixed, 'linePrefix')
    }
    /** @type {State} */

    function prefixed(code) {
      if (code === null || markdownLineEnding(code)) {
        return nok(code)
      }

      const tail = self.events[self.events.length - 1];

      if (
        !self.parser.constructs.disable.null.includes('codeIndented') &&
        tail &&
        tail[1].type === 'linePrefix' &&
        tail[2].sliceSerialize(tail[1], true).length >= 4
      ) {
        return ok(code)
      }

      return effects.interrupt(self.parser.constructs.flow, nok, ok)(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Effects} Effects
   * @typedef {import('micromark-util-types').State} State
   */

  /**
   * @param {Effects} effects
   * @param {State} ok
   * @param {State} nok
   * @param {string} type
   * @param {string} literalType
   * @param {string} literalMarkerType
   * @param {string} rawType
   * @param {string} stringType
   * @param {number} [max=Infinity]
   * @returns {State}
   */
  // eslint-disable-next-line max-params
  function factoryDestination(
    effects,
    ok,
    nok,
    type,
    literalType,
    literalMarkerType,
    rawType,
    stringType,
    max
  ) {
    const limit = max || Number.POSITIVE_INFINITY;
    let balance = 0;
    return start
    /** @type {State} */

    function start(code) {
      if (code === 60) {
        effects.enter(type);
        effects.enter(literalType);
        effects.enter(literalMarkerType);
        effects.consume(code);
        effects.exit(literalMarkerType);
        return destinationEnclosedBefore
      }

      if (code === null || code === 41 || asciiControl(code)) {
        return nok(code)
      }

      effects.enter(type);
      effects.enter(rawType);
      effects.enter(stringType);
      effects.enter('chunkString', {
        contentType: 'string'
      });
      return destinationRaw(code)
    }
    /** @type {State} */

    function destinationEnclosedBefore(code) {
      if (code === 62) {
        effects.enter(literalMarkerType);
        effects.consume(code);
        effects.exit(literalMarkerType);
        effects.exit(literalType);
        effects.exit(type);
        return ok
      }

      effects.enter(stringType);
      effects.enter('chunkString', {
        contentType: 'string'
      });
      return destinationEnclosed(code)
    }
    /** @type {State} */

    function destinationEnclosed(code) {
      if (code === 62) {
        effects.exit('chunkString');
        effects.exit(stringType);
        return destinationEnclosedBefore(code)
      }

      if (code === null || code === 60 || markdownLineEnding(code)) {
        return nok(code)
      }

      effects.consume(code);
      return code === 92 ? destinationEnclosedEscape : destinationEnclosed
    }
    /** @type {State} */

    function destinationEnclosedEscape(code) {
      if (code === 60 || code === 62 || code === 92) {
        effects.consume(code);
        return destinationEnclosed
      }

      return destinationEnclosed(code)
    }
    /** @type {State} */

    function destinationRaw(code) {
      if (code === 40) {
        if (++balance > limit) return nok(code)
        effects.consume(code);
        return destinationRaw
      }

      if (code === 41) {
        if (!balance--) {
          effects.exit('chunkString');
          effects.exit(stringType);
          effects.exit(rawType);
          effects.exit(type);
          return ok(code)
        }

        effects.consume(code);
        return destinationRaw
      }

      if (code === null || markdownLineEndingOrSpace(code)) {
        if (balance) return nok(code)
        effects.exit('chunkString');
        effects.exit(stringType);
        effects.exit(rawType);
        effects.exit(type);
        return ok(code)
      }

      if (asciiControl(code)) return nok(code)
      effects.consume(code);
      return code === 92 ? destinationRawEscape : destinationRaw
    }
    /** @type {State} */

    function destinationRawEscape(code) {
      if (code === 40 || code === 41 || code === 92) {
        effects.consume(code);
        return destinationRaw
      }

      return destinationRaw(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Effects} Effects
   * @typedef {import('micromark-util-types').TokenizeContext} TokenizeContext
   * @typedef {import('micromark-util-types').State} State
   */

  /**
   * @this {TokenizeContext}
   * @param {Effects} effects
   * @param {State} ok
   * @param {State} nok
   * @param {string} type
   * @param {string} markerType
   * @param {string} stringType
   * @returns {State}
   */
  // eslint-disable-next-line max-params
  function factoryLabel(effects, ok, nok, type, markerType, stringType) {
    const self = this;
    let size = 0;
    /** @type {boolean} */

    let data;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter(type);
      effects.enter(markerType);
      effects.consume(code);
      effects.exit(markerType);
      effects.enter(stringType);
      return atBreak
    }
    /** @type {State} */

    function atBreak(code) {
      if (
        code === null ||
        code === 91 ||
        (code === 93 && !data) ||
        /* To do: remove in the future once we’ve switched from
         * `micromark-extension-footnote` to `micromark-extension-gfm-footnote`,
         * which doesn’t need this */

        /* Hidden footnotes hook */

        /* c8 ignore next 3 */
        (code === 94 &&
          !size &&
          '_hiddenFootnoteSupport' in self.parser.constructs) ||
        size > 999
      ) {
        return nok(code)
      }

      if (code === 93) {
        effects.exit(stringType);
        effects.enter(markerType);
        effects.consume(code);
        effects.exit(markerType);
        effects.exit(type);
        return ok
      }

      if (markdownLineEnding(code)) {
        effects.enter('lineEnding');
        effects.consume(code);
        effects.exit('lineEnding');
        return atBreak
      }

      effects.enter('chunkString', {
        contentType: 'string'
      });
      return label(code)
    }
    /** @type {State} */

    function label(code) {
      if (
        code === null ||
        code === 91 ||
        code === 93 ||
        markdownLineEnding(code) ||
        size++ > 999
      ) {
        effects.exit('chunkString');
        return atBreak(code)
      }

      effects.consume(code);
      data = data || !markdownSpace(code);
      return code === 92 ? labelEscape : label
    }
    /** @type {State} */

    function labelEscape(code) {
      if (code === 91 || code === 92 || code === 93) {
        effects.consume(code);
        size++;
        return label
      }

      return label(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Effects} Effects
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Code} Code
   */

  /**
   * @param {Effects} effects
   * @param {State} ok
   * @param {State} nok
   * @param {string} type
   * @param {string} markerType
   * @param {string} stringType
   * @returns {State}
   */
  // eslint-disable-next-line max-params
  function factoryTitle(effects, ok, nok, type, markerType, stringType) {
    /** @type {NonNullable<Code>} */
    let marker;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter(type);
      effects.enter(markerType);
      effects.consume(code);
      effects.exit(markerType);
      marker = code === 40 ? 41 : code;
      return atFirstTitleBreak
    }
    /** @type {State} */

    function atFirstTitleBreak(code) {
      if (code === marker) {
        effects.enter(markerType);
        effects.consume(code);
        effects.exit(markerType);
        effects.exit(type);
        return ok
      }

      effects.enter(stringType);
      return atTitleBreak(code)
    }
    /** @type {State} */

    function atTitleBreak(code) {
      if (code === marker) {
        effects.exit(stringType);
        return atFirstTitleBreak(marker)
      }

      if (code === null) {
        return nok(code)
      } // Note: blank lines can’t exist in content.

      if (markdownLineEnding(code)) {
        effects.enter('lineEnding');
        effects.consume(code);
        effects.exit('lineEnding');
        return factorySpace(effects, atTitleBreak, 'linePrefix')
      }

      effects.enter('chunkString', {
        contentType: 'string'
      });
      return title(code)
    }
    /** @type {State} */

    function title(code) {
      if (code === marker || code === null || markdownLineEnding(code)) {
        effects.exit('chunkString');
        return atTitleBreak(code)
      }

      effects.consume(code);
      return code === 92 ? titleEscape : title
    }
    /** @type {State} */

    function titleEscape(code) {
      if (code === marker || code === 92) {
        effects.consume(code);
        return title
      }

      return title(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Effects} Effects
   * @typedef {import('micromark-util-types').State} State
   */

  /**
   * @param {Effects} effects
   * @param {State} ok
   */
  function factoryWhitespace(effects, ok) {
    /** @type {boolean} */
    let seen;
    return start
    /** @type {State} */

    function start(code) {
      if (markdownLineEnding(code)) {
        effects.enter('lineEnding');
        effects.consume(code);
        effects.exit('lineEnding');
        seen = true;
        return start
      }

      if (markdownSpace(code)) {
        return factorySpace(
          effects,
          start,
          seen ? 'linePrefix' : 'lineSuffix'
        )(code)
      }

      return ok(code)
    }
  }

  /**
   * Normalize an identifier (such as used in definitions).
   *
   * @param {string} value
   * @returns {string}
   */
  function normalizeIdentifier(value) {
    return (
      value // Collapse Markdown whitespace.
        .replace(/[\t\n\r ]+/g, ' ') // Trim.
        .replace(/^ | $/g, '') // Some characters are considered “uppercase”, but if their lowercase
        // counterpart is uppercased will result in a different uppercase
        // character.
        // Hence, to get that form, we perform both lower- and uppercase.
        // Upper case makes sure keys will not interact with default prototypal
        // methods: no method is uppercase.
        .toLowerCase()
        .toUpperCase()
    )
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   */

  /** @type {Construct} */
  const definition = {
    name: 'definition',
    tokenize: tokenizeDefinition
  };
  /** @type {Construct} */

  const titleConstruct = {
    tokenize: tokenizeTitle,
    partial: true
  };
  /** @type {Tokenizer} */

  function tokenizeDefinition(effects, ok, nok) {
    const self = this;
    /** @type {string} */

    let identifier;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('definition');
      return factoryLabel.call(
        self,
        effects,
        labelAfter,
        nok,
        'definitionLabel',
        'definitionLabelMarker',
        'definitionLabelString'
      )(code)
    }
    /** @type {State} */

    function labelAfter(code) {
      identifier = normalizeIdentifier(
        self.sliceSerialize(self.events[self.events.length - 1][1]).slice(1, -1)
      );

      if (code === 58) {
        effects.enter('definitionMarker');
        effects.consume(code);
        effects.exit('definitionMarker'); // Note: blank lines can’t exist in content.

        return factoryWhitespace(
          effects,
          factoryDestination(
            effects,
            effects.attempt(
              titleConstruct,
              factorySpace(effects, after, 'whitespace'),
              factorySpace(effects, after, 'whitespace')
            ),
            nok,
            'definitionDestination',
            'definitionDestinationLiteral',
            'definitionDestinationLiteralMarker',
            'definitionDestinationRaw',
            'definitionDestinationString'
          )
        )
      }

      return nok(code)
    }
    /** @type {State} */

    function after(code) {
      if (code === null || markdownLineEnding(code)) {
        effects.exit('definition');

        if (!self.parser.defined.includes(identifier)) {
          self.parser.defined.push(identifier);
        }

        return ok(code)
      }

      return nok(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizeTitle(effects, ok, nok) {
    return start
    /** @type {State} */

    function start(code) {
      return markdownLineEndingOrSpace(code)
        ? factoryWhitespace(effects, before)(code)
        : nok(code)
    }
    /** @type {State} */

    function before(code) {
      if (code === 34 || code === 39 || code === 40) {
        return factoryTitle(
          effects,
          factorySpace(effects, after, 'whitespace'),
          nok,
          'definitionTitle',
          'definitionTitleMarker',
          'definitionTitleString'
        )(code)
      }

      return nok(code)
    }
    /** @type {State} */

    function after(code) {
      return code === null || markdownLineEnding(code) ? ok(code) : nok(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   */

  /** @type {Construct} */
  const hardBreakEscape = {
    name: 'hardBreakEscape',
    tokenize: tokenizeHardBreakEscape
  };
  /** @type {Tokenizer} */

  function tokenizeHardBreakEscape(effects, ok, nok) {
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('hardBreakEscape');
      effects.enter('escapeMarker');
      effects.consume(code);
      return open
    }
    /** @type {State} */

    function open(code) {
      if (markdownLineEnding(code)) {
        effects.exit('escapeMarker');
        effects.exit('hardBreakEscape');
        return ok(code)
      }

      return nok(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Resolver} Resolver
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').Token} Token
   * @typedef {import('micromark-util-types').State} State
   */

  /** @type {Construct} */
  const headingAtx = {
    name: 'headingAtx',
    tokenize: tokenizeHeadingAtx,
    resolve: resolveHeadingAtx
  };
  /** @type {Resolver} */

  function resolveHeadingAtx(events, context) {
    let contentEnd = events.length - 2;
    let contentStart = 3;
    /** @type {Token} */

    let content;
    /** @type {Token} */

    let text; // Prefix whitespace, part of the opening.

    if (events[contentStart][1].type === 'whitespace') {
      contentStart += 2;
    } // Suffix whitespace, part of the closing.

    if (
      contentEnd - 2 > contentStart &&
      events[contentEnd][1].type === 'whitespace'
    ) {
      contentEnd -= 2;
    }

    if (
      events[contentEnd][1].type === 'atxHeadingSequence' &&
      (contentStart === contentEnd - 1 ||
        (contentEnd - 4 > contentStart &&
          events[contentEnd - 2][1].type === 'whitespace'))
    ) {
      contentEnd -= contentStart + 1 === contentEnd ? 2 : 4;
    }

    if (contentEnd > contentStart) {
      content = {
        type: 'atxHeadingText',
        start: events[contentStart][1].start,
        end: events[contentEnd][1].end
      };
      text = {
        type: 'chunkText',
        start: events[contentStart][1].start,
        end: events[contentEnd][1].end,
        // @ts-expect-error Constants are fine to assign.
        contentType: 'text'
      };
      splice(events, contentStart, contentEnd - contentStart + 1, [
        ['enter', content, context],
        ['enter', text, context],
        ['exit', text, context],
        ['exit', content, context]
      ]);
    }

    return events
  }
  /** @type {Tokenizer} */

  function tokenizeHeadingAtx(effects, ok, nok) {
    const self = this;
    let size = 0;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('atxHeading');
      effects.enter('atxHeadingSequence');
      return fenceOpenInside(code)
    }
    /** @type {State} */

    function fenceOpenInside(code) {
      if (code === 35 && size++ < 6) {
        effects.consume(code);
        return fenceOpenInside
      }

      if (code === null || markdownLineEndingOrSpace(code)) {
        effects.exit('atxHeadingSequence');
        return self.interrupt ? ok(code) : headingBreak(code)
      }

      return nok(code)
    }
    /** @type {State} */

    function headingBreak(code) {
      if (code === 35) {
        effects.enter('atxHeadingSequence');
        return sequence(code)
      }

      if (code === null || markdownLineEnding(code)) {
        effects.exit('atxHeading');
        return ok(code)
      }

      if (markdownSpace(code)) {
        return factorySpace(effects, headingBreak, 'whitespace')(code)
      }

      effects.enter('atxHeadingText');
      return data(code)
    }
    /** @type {State} */

    function sequence(code) {
      if (code === 35) {
        effects.consume(code);
        return sequence
      }

      effects.exit('atxHeadingSequence');
      return headingBreak(code)
    }
    /** @type {State} */

    function data(code) {
      if (code === null || code === 35 || markdownLineEndingOrSpace(code)) {
        effects.exit('atxHeadingText');
        return headingBreak(code)
      }

      effects.consume(code);
      return data
    }
  }

  /**
   * List of lowercase HTML tag names which when parsing HTML (flow), result
   * in more relaxed rules (condition 6): because they are known blocks, the
   * HTML-like syntax doesn’t have to be strictly parsed.
   * For tag names not in this list, a more strict algorithm (condition 7) is used
   * to detect whether the HTML-like syntax is seen as HTML (flow) or not.
   *
   * This is copied from:
   * <https://spec.commonmark.org/0.29/#html-blocks>.
   */
  const htmlBlockNames = [
    'address',
    'article',
    'aside',
    'base',
    'basefont',
    'blockquote',
    'body',
    'caption',
    'center',
    'col',
    'colgroup',
    'dd',
    'details',
    'dialog',
    'dir',
    'div',
    'dl',
    'dt',
    'fieldset',
    'figcaption',
    'figure',
    'footer',
    'form',
    'frame',
    'frameset',
    'h1',
    'h2',
    'h3',
    'h4',
    'h5',
    'h6',
    'head',
    'header',
    'hr',
    'html',
    'iframe',
    'legend',
    'li',
    'link',
    'main',
    'menu',
    'menuitem',
    'nav',
    'noframes',
    'ol',
    'optgroup',
    'option',
    'p',
    'param',
    'section',
    'source',
    'summary',
    'table',
    'tbody',
    'td',
    'tfoot',
    'th',
    'thead',
    'title',
    'tr',
    'track',
    'ul'
  ];

  /**
   * List of lowercase HTML tag names which when parsing HTML (flow), result in
   * HTML that can include lines w/o exiting, until a closing tag also in this
   * list is found (condition 1).
   *
   * This module is copied from:
   * <https://spec.commonmark.org/0.29/#html-blocks>.
   *
   * Note that `textarea` is not available in `CommonMark@0.29` but has been
   * merged to the primary branch and is slated to be released in the next release
   * of CommonMark.
   */
  const htmlRawNames = ['pre', 'script', 'style', 'textarea'];

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Resolver} Resolver
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Code} Code
   */
  /** @type {Construct} */

  const htmlFlow = {
    name: 'htmlFlow',
    tokenize: tokenizeHtmlFlow,
    resolveTo: resolveToHtmlFlow,
    concrete: true
  };
  /** @type {Construct} */

  const nextBlankConstruct = {
    tokenize: tokenizeNextBlank,
    partial: true
  };
  /** @type {Resolver} */

  function resolveToHtmlFlow(events) {
    let index = events.length;

    while (index--) {
      if (events[index][0] === 'enter' && events[index][1].type === 'htmlFlow') {
        break
      }
    }

    if (index > 1 && events[index - 2][1].type === 'linePrefix') {
      // Add the prefix start to the HTML token.
      events[index][1].start = events[index - 2][1].start; // Add the prefix start to the HTML line token.

      events[index + 1][1].start = events[index - 2][1].start; // Remove the line prefix.

      events.splice(index - 2, 2);
    }

    return events
  }
  /** @type {Tokenizer} */

  function tokenizeHtmlFlow(effects, ok, nok) {
    const self = this;
    /** @type {number} */

    let kind;
    /** @type {boolean} */

    let startTag;
    /** @type {string} */

    let buffer;
    /** @type {number} */

    let index;
    /** @type {Code} */

    let marker;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('htmlFlow');
      effects.enter('htmlFlowData');
      effects.consume(code);
      return open
    }
    /** @type {State} */

    function open(code) {
      if (code === 33) {
        effects.consume(code);
        return declarationStart
      }

      if (code === 47) {
        effects.consume(code);
        return tagCloseStart
      }

      if (code === 63) {
        effects.consume(code);
        kind = 3; // While we’re in an instruction instead of a declaration, we’re on a `?`
        // right now, so we do need to search for `>`, similar to declarations.

        return self.interrupt ? ok : continuationDeclarationInside
      }

      if (asciiAlpha(code)) {
        effects.consume(code);
        buffer = String.fromCharCode(code);
        startTag = true;
        return tagName
      }

      return nok(code)
    }
    /** @type {State} */

    function declarationStart(code) {
      if (code === 45) {
        effects.consume(code);
        kind = 2;
        return commentOpenInside
      }

      if (code === 91) {
        effects.consume(code);
        kind = 5;
        buffer = 'CDATA[';
        index = 0;
        return cdataOpenInside
      }

      if (asciiAlpha(code)) {
        effects.consume(code);
        kind = 4;
        return self.interrupt ? ok : continuationDeclarationInside
      }

      return nok(code)
    }
    /** @type {State} */

    function commentOpenInside(code) {
      if (code === 45) {
        effects.consume(code);
        return self.interrupt ? ok : continuationDeclarationInside
      }

      return nok(code)
    }
    /** @type {State} */

    function cdataOpenInside(code) {
      if (code === buffer.charCodeAt(index++)) {
        effects.consume(code);
        return index === buffer.length
          ? self.interrupt
            ? ok
            : continuation
          : cdataOpenInside
      }

      return nok(code)
    }
    /** @type {State} */

    function tagCloseStart(code) {
      if (asciiAlpha(code)) {
        effects.consume(code);
        buffer = String.fromCharCode(code);
        return tagName
      }

      return nok(code)
    }
    /** @type {State} */

    function tagName(code) {
      if (
        code === null ||
        code === 47 ||
        code === 62 ||
        markdownLineEndingOrSpace(code)
      ) {
        if (
          code !== 47 &&
          startTag &&
          htmlRawNames.includes(buffer.toLowerCase())
        ) {
          kind = 1;
          return self.interrupt ? ok(code) : continuation(code)
        }

        if (htmlBlockNames.includes(buffer.toLowerCase())) {
          kind = 6;

          if (code === 47) {
            effects.consume(code);
            return basicSelfClosing
          }

          return self.interrupt ? ok(code) : continuation(code)
        }

        kind = 7; // Do not support complete HTML when interrupting

        return self.interrupt && !self.parser.lazy[self.now().line]
          ? nok(code)
          : startTag
          ? completeAttributeNameBefore(code)
          : completeClosingTagAfter(code)
      }

      if (code === 45 || asciiAlphanumeric(code)) {
        effects.consume(code);
        buffer += String.fromCharCode(code);
        return tagName
      }

      return nok(code)
    }
    /** @type {State} */

    function basicSelfClosing(code) {
      if (code === 62) {
        effects.consume(code);
        return self.interrupt ? ok : continuation
      }

      return nok(code)
    }
    /** @type {State} */

    function completeClosingTagAfter(code) {
      if (markdownSpace(code)) {
        effects.consume(code);
        return completeClosingTagAfter
      }

      return completeEnd(code)
    }
    /** @type {State} */

    function completeAttributeNameBefore(code) {
      if (code === 47) {
        effects.consume(code);
        return completeEnd
      }

      if (code === 58 || code === 95 || asciiAlpha(code)) {
        effects.consume(code);
        return completeAttributeName
      }

      if (markdownSpace(code)) {
        effects.consume(code);
        return completeAttributeNameBefore
      }

      return completeEnd(code)
    }
    /** @type {State} */

    function completeAttributeName(code) {
      if (
        code === 45 ||
        code === 46 ||
        code === 58 ||
        code === 95 ||
        asciiAlphanumeric(code)
      ) {
        effects.consume(code);
        return completeAttributeName
      }

      return completeAttributeNameAfter(code)
    }
    /** @type {State} */

    function completeAttributeNameAfter(code) {
      if (code === 61) {
        effects.consume(code);
        return completeAttributeValueBefore
      }

      if (markdownSpace(code)) {
        effects.consume(code);
        return completeAttributeNameAfter
      }

      return completeAttributeNameBefore(code)
    }
    /** @type {State} */

    function completeAttributeValueBefore(code) {
      if (
        code === null ||
        code === 60 ||
        code === 61 ||
        code === 62 ||
        code === 96
      ) {
        return nok(code)
      }

      if (code === 34 || code === 39) {
        effects.consume(code);
        marker = code;
        return completeAttributeValueQuoted
      }

      if (markdownSpace(code)) {
        effects.consume(code);
        return completeAttributeValueBefore
      }

      marker = null;
      return completeAttributeValueUnquoted(code)
    }
    /** @type {State} */

    function completeAttributeValueQuoted(code) {
      if (code === null || markdownLineEnding(code)) {
        return nok(code)
      }

      if (code === marker) {
        effects.consume(code);
        return completeAttributeValueQuotedAfter
      }

      effects.consume(code);
      return completeAttributeValueQuoted
    }
    /** @type {State} */

    function completeAttributeValueUnquoted(code) {
      if (
        code === null ||
        code === 34 ||
        code === 39 ||
        code === 60 ||
        code === 61 ||
        code === 62 ||
        code === 96 ||
        markdownLineEndingOrSpace(code)
      ) {
        return completeAttributeNameAfter(code)
      }

      effects.consume(code);
      return completeAttributeValueUnquoted
    }
    /** @type {State} */

    function completeAttributeValueQuotedAfter(code) {
      if (code === 47 || code === 62 || markdownSpace(code)) {
        return completeAttributeNameBefore(code)
      }

      return nok(code)
    }
    /** @type {State} */

    function completeEnd(code) {
      if (code === 62) {
        effects.consume(code);
        return completeAfter
      }

      return nok(code)
    }
    /** @type {State} */

    function completeAfter(code) {
      if (markdownSpace(code)) {
        effects.consume(code);
        return completeAfter
      }

      return code === null || markdownLineEnding(code)
        ? continuation(code)
        : nok(code)
    }
    /** @type {State} */

    function continuation(code) {
      if (code === 45 && kind === 2) {
        effects.consume(code);
        return continuationCommentInside
      }

      if (code === 60 && kind === 1) {
        effects.consume(code);
        return continuationRawTagOpen
      }

      if (code === 62 && kind === 4) {
        effects.consume(code);
        return continuationClose
      }

      if (code === 63 && kind === 3) {
        effects.consume(code);
        return continuationDeclarationInside
      }

      if (code === 93 && kind === 5) {
        effects.consume(code);
        return continuationCharacterDataInside
      }

      if (markdownLineEnding(code) && (kind === 6 || kind === 7)) {
        return effects.check(
          nextBlankConstruct,
          continuationClose,
          continuationAtLineEnding
        )(code)
      }

      if (code === null || markdownLineEnding(code)) {
        return continuationAtLineEnding(code)
      }

      effects.consume(code);
      return continuation
    }
    /** @type {State} */

    function continuationAtLineEnding(code) {
      effects.exit('htmlFlowData');
      return htmlContinueStart(code)
    }
    /** @type {State} */

    function htmlContinueStart(code) {
      if (code === null) {
        return done(code)
      }

      if (markdownLineEnding(code)) {
        return effects.attempt(
          {
            tokenize: htmlLineEnd,
            partial: true
          },
          htmlContinueStart,
          done
        )(code)
      }

      effects.enter('htmlFlowData');
      return continuation(code)
    }
    /** @type {Tokenizer} */

    function htmlLineEnd(effects, ok, nok) {
      return start
      /** @type {State} */

      function start(code) {
        effects.enter('lineEnding');
        effects.consume(code);
        effects.exit('lineEnding');
        return lineStart
      }
      /** @type {State} */

      function lineStart(code) {
        return self.parser.lazy[self.now().line] ? nok(code) : ok(code)
      }
    }
    /** @type {State} */

    function continuationCommentInside(code) {
      if (code === 45) {
        effects.consume(code);
        return continuationDeclarationInside
      }

      return continuation(code)
    }
    /** @type {State} */

    function continuationRawTagOpen(code) {
      if (code === 47) {
        effects.consume(code);
        buffer = '';
        return continuationRawEndTag
      }

      return continuation(code)
    }
    /** @type {State} */

    function continuationRawEndTag(code) {
      if (code === 62 && htmlRawNames.includes(buffer.toLowerCase())) {
        effects.consume(code);
        return continuationClose
      }

      if (asciiAlpha(code) && buffer.length < 8) {
        effects.consume(code);
        buffer += String.fromCharCode(code);
        return continuationRawEndTag
      }

      return continuation(code)
    }
    /** @type {State} */

    function continuationCharacterDataInside(code) {
      if (code === 93) {
        effects.consume(code);
        return continuationDeclarationInside
      }

      return continuation(code)
    }
    /** @type {State} */

    function continuationDeclarationInside(code) {
      if (code === 62) {
        effects.consume(code);
        return continuationClose
      } // More dashes.

      if (code === 45 && kind === 2) {
        effects.consume(code);
        return continuationDeclarationInside
      }

      return continuation(code)
    }
    /** @type {State} */

    function continuationClose(code) {
      if (code === null || markdownLineEnding(code)) {
        effects.exit('htmlFlowData');
        return done(code)
      }

      effects.consume(code);
      return continuationClose
    }
    /** @type {State} */

    function done(code) {
      effects.exit('htmlFlow');
      return ok(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizeNextBlank(effects, ok, nok) {
    return start
    /** @type {State} */

    function start(code) {
      effects.exit('htmlFlowData');
      effects.enter('lineEndingBlank');
      effects.consume(code);
      effects.exit('lineEndingBlank');
      return effects.attempt(blankLine, ok, nok)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Code} Code
   */

  /** @type {Construct} */
  const htmlText = {
    name: 'htmlText',
    tokenize: tokenizeHtmlText
  };
  /** @type {Tokenizer} */

  function tokenizeHtmlText(effects, ok, nok) {
    const self = this;
    /** @type {NonNullable<Code>|undefined} */

    let marker;
    /** @type {string} */

    let buffer;
    /** @type {number} */

    let index;
    /** @type {State} */

    let returnState;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('htmlText');
      effects.enter('htmlTextData');
      effects.consume(code);
      return open
    }
    /** @type {State} */

    function open(code) {
      if (code === 33) {
        effects.consume(code);
        return declarationOpen
      }

      if (code === 47) {
        effects.consume(code);
        return tagCloseStart
      }

      if (code === 63) {
        effects.consume(code);
        return instruction
      }

      if (asciiAlpha(code)) {
        effects.consume(code);
        return tagOpen
      }

      return nok(code)
    }
    /** @type {State} */

    function declarationOpen(code) {
      if (code === 45) {
        effects.consume(code);
        return commentOpen
      }

      if (code === 91) {
        effects.consume(code);
        buffer = 'CDATA[';
        index = 0;
        return cdataOpen
      }

      if (asciiAlpha(code)) {
        effects.consume(code);
        return declaration
      }

      return nok(code)
    }
    /** @type {State} */

    function commentOpen(code) {
      if (code === 45) {
        effects.consume(code);
        return commentStart
      }

      return nok(code)
    }
    /** @type {State} */

    function commentStart(code) {
      if (code === null || code === 62) {
        return nok(code)
      }

      if (code === 45) {
        effects.consume(code);
        return commentStartDash
      }

      return comment(code)
    }
    /** @type {State} */

    function commentStartDash(code) {
      if (code === null || code === 62) {
        return nok(code)
      }

      return comment(code)
    }
    /** @type {State} */

    function comment(code) {
      if (code === null) {
        return nok(code)
      }

      if (code === 45) {
        effects.consume(code);
        return commentClose
      }

      if (markdownLineEnding(code)) {
        returnState = comment;
        return atLineEnding(code)
      }

      effects.consume(code);
      return comment
    }
    /** @type {State} */

    function commentClose(code) {
      if (code === 45) {
        effects.consume(code);
        return end
      }

      return comment(code)
    }
    /** @type {State} */

    function cdataOpen(code) {
      if (code === buffer.charCodeAt(index++)) {
        effects.consume(code);
        return index === buffer.length ? cdata : cdataOpen
      }

      return nok(code)
    }
    /** @type {State} */

    function cdata(code) {
      if (code === null) {
        return nok(code)
      }

      if (code === 93) {
        effects.consume(code);
        return cdataClose
      }

      if (markdownLineEnding(code)) {
        returnState = cdata;
        return atLineEnding(code)
      }

      effects.consume(code);
      return cdata
    }
    /** @type {State} */

    function cdataClose(code) {
      if (code === 93) {
        effects.consume(code);
        return cdataEnd
      }

      return cdata(code)
    }
    /** @type {State} */

    function cdataEnd(code) {
      if (code === 62) {
        return end(code)
      }

      if (code === 93) {
        effects.consume(code);
        return cdataEnd
      }

      return cdata(code)
    }
    /** @type {State} */

    function declaration(code) {
      if (code === null || code === 62) {
        return end(code)
      }

      if (markdownLineEnding(code)) {
        returnState = declaration;
        return atLineEnding(code)
      }

      effects.consume(code);
      return declaration
    }
    /** @type {State} */

    function instruction(code) {
      if (code === null) {
        return nok(code)
      }

      if (code === 63) {
        effects.consume(code);
        return instructionClose
      }

      if (markdownLineEnding(code)) {
        returnState = instruction;
        return atLineEnding(code)
      }

      effects.consume(code);
      return instruction
    }
    /** @type {State} */

    function instructionClose(code) {
      return code === 62 ? end(code) : instruction(code)
    }
    /** @type {State} */

    function tagCloseStart(code) {
      if (asciiAlpha(code)) {
        effects.consume(code);
        return tagClose
      }

      return nok(code)
    }
    /** @type {State} */

    function tagClose(code) {
      if (code === 45 || asciiAlphanumeric(code)) {
        effects.consume(code);
        return tagClose
      }

      return tagCloseBetween(code)
    }
    /** @type {State} */

    function tagCloseBetween(code) {
      if (markdownLineEnding(code)) {
        returnState = tagCloseBetween;
        return atLineEnding(code)
      }

      if (markdownSpace(code)) {
        effects.consume(code);
        return tagCloseBetween
      }

      return end(code)
    }
    /** @type {State} */

    function tagOpen(code) {
      if (code === 45 || asciiAlphanumeric(code)) {
        effects.consume(code);
        return tagOpen
      }

      if (code === 47 || code === 62 || markdownLineEndingOrSpace(code)) {
        return tagOpenBetween(code)
      }

      return nok(code)
    }
    /** @type {State} */

    function tagOpenBetween(code) {
      if (code === 47) {
        effects.consume(code);
        return end
      }

      if (code === 58 || code === 95 || asciiAlpha(code)) {
        effects.consume(code);
        return tagOpenAttributeName
      }

      if (markdownLineEnding(code)) {
        returnState = tagOpenBetween;
        return atLineEnding(code)
      }

      if (markdownSpace(code)) {
        effects.consume(code);
        return tagOpenBetween
      }

      return end(code)
    }
    /** @type {State} */

    function tagOpenAttributeName(code) {
      if (
        code === 45 ||
        code === 46 ||
        code === 58 ||
        code === 95 ||
        asciiAlphanumeric(code)
      ) {
        effects.consume(code);
        return tagOpenAttributeName
      }

      return tagOpenAttributeNameAfter(code)
    }
    /** @type {State} */

    function tagOpenAttributeNameAfter(code) {
      if (code === 61) {
        effects.consume(code);
        return tagOpenAttributeValueBefore
      }

      if (markdownLineEnding(code)) {
        returnState = tagOpenAttributeNameAfter;
        return atLineEnding(code)
      }

      if (markdownSpace(code)) {
        effects.consume(code);
        return tagOpenAttributeNameAfter
      }

      return tagOpenBetween(code)
    }
    /** @type {State} */

    function tagOpenAttributeValueBefore(code) {
      if (
        code === null ||
        code === 60 ||
        code === 61 ||
        code === 62 ||
        code === 96
      ) {
        return nok(code)
      }

      if (code === 34 || code === 39) {
        effects.consume(code);
        marker = code;
        return tagOpenAttributeValueQuoted
      }

      if (markdownLineEnding(code)) {
        returnState = tagOpenAttributeValueBefore;
        return atLineEnding(code)
      }

      if (markdownSpace(code)) {
        effects.consume(code);
        return tagOpenAttributeValueBefore
      }

      effects.consume(code);
      marker = undefined;
      return tagOpenAttributeValueUnquoted
    }
    /** @type {State} */

    function tagOpenAttributeValueQuoted(code) {
      if (code === marker) {
        effects.consume(code);
        return tagOpenAttributeValueQuotedAfter
      }

      if (code === null) {
        return nok(code)
      }

      if (markdownLineEnding(code)) {
        returnState = tagOpenAttributeValueQuoted;
        return atLineEnding(code)
      }

      effects.consume(code);
      return tagOpenAttributeValueQuoted
    }
    /** @type {State} */

    function tagOpenAttributeValueQuotedAfter(code) {
      if (code === 62 || code === 47 || markdownLineEndingOrSpace(code)) {
        return tagOpenBetween(code)
      }

      return nok(code)
    }
    /** @type {State} */

    function tagOpenAttributeValueUnquoted(code) {
      if (
        code === null ||
        code === 34 ||
        code === 39 ||
        code === 60 ||
        code === 61 ||
        code === 96
      ) {
        return nok(code)
      }

      if (code === 62 || markdownLineEndingOrSpace(code)) {
        return tagOpenBetween(code)
      }

      effects.consume(code);
      return tagOpenAttributeValueUnquoted
    } // We can’t have blank lines in content, so no need to worry about empty
    // tokens.

    /** @type {State} */

    function atLineEnding(code) {
      effects.exit('htmlTextData');
      effects.enter('lineEnding');
      effects.consume(code);
      effects.exit('lineEnding');
      return factorySpace(
        effects,
        afterPrefix,
        'linePrefix',
        self.parser.constructs.disable.null.includes('codeIndented')
          ? undefined
          : 4
      )
    }
    /** @type {State} */

    function afterPrefix(code) {
      effects.enter('htmlTextData');
      return returnState(code)
    }
    /** @type {State} */

    function end(code) {
      if (code === 62) {
        effects.consume(code);
        effects.exit('htmlTextData');
        effects.exit('htmlText');
        return ok
      }

      return nok(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Resolver} Resolver
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').Event} Event
   * @typedef {import('micromark-util-types').Token} Token
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Code} Code
   */

  /** @type {Construct} */
  const labelEnd = {
    name: 'labelEnd',
    tokenize: tokenizeLabelEnd,
    resolveTo: resolveToLabelEnd,
    resolveAll: resolveAllLabelEnd
  };
  /** @type {Construct} */

  const resourceConstruct = {
    tokenize: tokenizeResource
  };
  /** @type {Construct} */

  const fullReferenceConstruct = {
    tokenize: tokenizeFullReference
  };
  /** @type {Construct} */

  const collapsedReferenceConstruct = {
    tokenize: tokenizeCollapsedReference
  };
  /** @type {Resolver} */

  function resolveAllLabelEnd(events) {
    let index = -1;
    /** @type {Token} */

    let token;

    while (++index < events.length) {
      token = events[index][1];

      if (
        token.type === 'labelImage' ||
        token.type === 'labelLink' ||
        token.type === 'labelEnd'
      ) {
        // Remove the marker.
        events.splice(index + 1, token.type === 'labelImage' ? 4 : 2);
        token.type = 'data';
        index++;
      }
    }

    return events
  }
  /** @type {Resolver} */

  function resolveToLabelEnd(events, context) {
    let index = events.length;
    let offset = 0;
    /** @type {Token} */

    let token;
    /** @type {number|undefined} */

    let open;
    /** @type {number|undefined} */

    let close;
    /** @type {Event[]} */

    let media; // Find an opening.

    while (index--) {
      token = events[index][1];

      if (open) {
        // If we see another link, or inactive link label, we’ve been here before.
        if (
          token.type === 'link' ||
          (token.type === 'labelLink' && token._inactive)
        ) {
          break
        } // Mark other link openings as inactive, as we can’t have links in
        // links.

        if (events[index][0] === 'enter' && token.type === 'labelLink') {
          token._inactive = true;
        }
      } else if (close) {
        if (
          events[index][0] === 'enter' &&
          (token.type === 'labelImage' || token.type === 'labelLink') &&
          !token._balanced
        ) {
          open = index;

          if (token.type !== 'labelLink') {
            offset = 2;
            break
          }
        }
      } else if (token.type === 'labelEnd') {
        close = index;
      }
    }

    const group = {
      type: events[open][1].type === 'labelLink' ? 'link' : 'image',
      start: Object.assign({}, events[open][1].start),
      end: Object.assign({}, events[events.length - 1][1].end)
    };
    const label = {
      type: 'label',
      start: Object.assign({}, events[open][1].start),
      end: Object.assign({}, events[close][1].end)
    };
    const text = {
      type: 'labelText',
      start: Object.assign({}, events[open + offset + 2][1].end),
      end: Object.assign({}, events[close - 2][1].start)
    };
    media = [
      ['enter', group, context],
      ['enter', label, context]
    ]; // Opening marker.

    media = push(media, events.slice(open + 1, open + offset + 3)); // Text open.

    media = push(media, [['enter', text, context]]); // Between.

    media = push(
      media,
      resolveAll(
        context.parser.constructs.insideSpan.null,
        events.slice(open + offset + 4, close - 3),
        context
      )
    ); // Text close, marker close, label close.

    media = push(media, [
      ['exit', text, context],
      events[close - 2],
      events[close - 1],
      ['exit', label, context]
    ]); // Reference, resource, or so.

    media = push(media, events.slice(close + 1)); // Media close.

    media = push(media, [['exit', group, context]]);
    splice(events, open, events.length, media);
    return events
  }
  /** @type {Tokenizer} */

  function tokenizeLabelEnd(effects, ok, nok) {
    const self = this;
    let index = self.events.length;
    /** @type {Token} */

    let labelStart;
    /** @type {boolean} */

    let defined; // Find an opening.

    while (index--) {
      if (
        (self.events[index][1].type === 'labelImage' ||
          self.events[index][1].type === 'labelLink') &&
        !self.events[index][1]._balanced
      ) {
        labelStart = self.events[index][1];
        break
      }
    }

    return start
    /** @type {State} */

    function start(code) {
      if (!labelStart) {
        return nok(code)
      } // It’s a balanced bracket, but contains a link.

      if (labelStart._inactive) return balanced(code)
      defined = self.parser.defined.includes(
        normalizeIdentifier(
          self.sliceSerialize({
            start: labelStart.end,
            end: self.now()
          })
        )
      );
      effects.enter('labelEnd');
      effects.enter('labelMarker');
      effects.consume(code);
      effects.exit('labelMarker');
      effects.exit('labelEnd');
      return afterLabelEnd
    }
    /** @type {State} */

    function afterLabelEnd(code) {
      // Resource: `[asd](fgh)`.
      if (code === 40) {
        return effects.attempt(
          resourceConstruct,
          ok,
          defined ? ok : balanced
        )(code)
      } // Collapsed (`[asd][]`) or full (`[asd][fgh]`) reference?

      if (code === 91) {
        return effects.attempt(
          fullReferenceConstruct,
          ok,
          defined
            ? effects.attempt(collapsedReferenceConstruct, ok, balanced)
            : balanced
        )(code)
      } // Shortcut reference: `[asd]`?

      return defined ? ok(code) : balanced(code)
    }
    /** @type {State} */

    function balanced(code) {
      labelStart._balanced = true;
      return nok(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizeResource(effects, ok, nok) {
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('resource');
      effects.enter('resourceMarker');
      effects.consume(code);
      effects.exit('resourceMarker');
      return factoryWhitespace(effects, open)
    }
    /** @type {State} */

    function open(code) {
      if (code === 41) {
        return end(code)
      }

      return factoryDestination(
        effects,
        destinationAfter,
        nok,
        'resourceDestination',
        'resourceDestinationLiteral',
        'resourceDestinationLiteralMarker',
        'resourceDestinationRaw',
        'resourceDestinationString',
        32
      )(code)
    }
    /** @type {State} */

    function destinationAfter(code) {
      return markdownLineEndingOrSpace(code)
        ? factoryWhitespace(effects, between)(code)
        : end(code)
    }
    /** @type {State} */

    function between(code) {
      if (code === 34 || code === 39 || code === 40) {
        return factoryTitle(
          effects,
          factoryWhitespace(effects, end),
          nok,
          'resourceTitle',
          'resourceTitleMarker',
          'resourceTitleString'
        )(code)
      }

      return end(code)
    }
    /** @type {State} */

    function end(code) {
      if (code === 41) {
        effects.enter('resourceMarker');
        effects.consume(code);
        effects.exit('resourceMarker');
        effects.exit('resource');
        return ok
      }

      return nok(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizeFullReference(effects, ok, nok) {
    const self = this;
    return start
    /** @type {State} */

    function start(code) {
      return factoryLabel.call(
        self,
        effects,
        afterLabel,
        nok,
        'reference',
        'referenceMarker',
        'referenceString'
      )(code)
    }
    /** @type {State} */

    function afterLabel(code) {
      return self.parser.defined.includes(
        normalizeIdentifier(
          self.sliceSerialize(self.events[self.events.length - 1][1]).slice(1, -1)
        )
      )
        ? ok(code)
        : nok(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizeCollapsedReference(effects, ok, nok) {
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('reference');
      effects.enter('referenceMarker');
      effects.consume(code);
      effects.exit('referenceMarker');
      return open
    }
    /** @type {State} */

    function open(code) {
      if (code === 93) {
        effects.enter('referenceMarker');
        effects.consume(code);
        effects.exit('referenceMarker');
        effects.exit('reference');
        return ok
      }

      return nok(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   */
  /** @type {Construct} */

  const labelStartImage = {
    name: 'labelStartImage',
    tokenize: tokenizeLabelStartImage,
    resolveAll: labelEnd.resolveAll
  };
  /** @type {Tokenizer} */

  function tokenizeLabelStartImage(effects, ok, nok) {
    const self = this;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('labelImage');
      effects.enter('labelImageMarker');
      effects.consume(code);
      effects.exit('labelImageMarker');
      return open
    }
    /** @type {State} */

    function open(code) {
      if (code === 91) {
        effects.enter('labelMarker');
        effects.consume(code);
        effects.exit('labelMarker');
        effects.exit('labelImage');
        return after
      }

      return nok(code)
    }
    /** @type {State} */

    function after(code) {
      /* To do: remove in the future once we’ve switched from
       * `micromark-extension-footnote` to `micromark-extension-gfm-footnote`,
       * which doesn’t need this */

      /* Hidden footnotes hook */

      /* c8 ignore next 3 */
      return code === 94 && '_hiddenFootnoteSupport' in self.parser.constructs
        ? nok(code)
        : ok(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   */
  /** @type {Construct} */

  const labelStartLink = {
    name: 'labelStartLink',
    tokenize: tokenizeLabelStartLink,
    resolveAll: labelEnd.resolveAll
  };
  /** @type {Tokenizer} */

  function tokenizeLabelStartLink(effects, ok, nok) {
    const self = this;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('labelLink');
      effects.enter('labelMarker');
      effects.consume(code);
      effects.exit('labelMarker');
      effects.exit('labelLink');
      return after
    }
    /** @type {State} */

    function after(code) {
      /* To do: remove in the future once we’ve switched from
       * `micromark-extension-footnote` to `micromark-extension-gfm-footnote`,
       * which doesn’t need this */

      /* Hidden footnotes hook. */

      /* c8 ignore next 3 */
      return code === 94 && '_hiddenFootnoteSupport' in self.parser.constructs
        ? nok(code)
        : ok(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   */

  /** @type {Construct} */
  const lineEnding = {
    name: 'lineEnding',
    tokenize: tokenizeLineEnding
  };
  /** @type {Tokenizer} */

  function tokenizeLineEnding(effects, ok) {
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('lineEnding');
      effects.consume(code);
      effects.exit('lineEnding');
      return factorySpace(effects, ok, 'linePrefix')
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Code} Code
   */

  /** @type {Construct} */
  const thematicBreak$1 = {
    name: 'thematicBreak',
    tokenize: tokenizeThematicBreak
  };
  /** @type {Tokenizer} */

  function tokenizeThematicBreak(effects, ok, nok) {
    let size = 0;
    /** @type {NonNullable<Code>} */

    let marker;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('thematicBreak');
      marker = code;
      return atBreak(code)
    }
    /** @type {State} */

    function atBreak(code) {
      if (code === marker) {
        effects.enter('thematicBreakSequence');
        return sequence(code)
      }

      if (markdownSpace(code)) {
        return factorySpace(effects, atBreak, 'whitespace')(code)
      }

      if (size < 3 || (code !== null && !markdownLineEnding(code))) {
        return nok(code)
      }

      effects.exit('thematicBreak');
      return ok(code)
    }
    /** @type {State} */

    function sequence(code) {
      if (code === marker) {
        effects.consume(code);
        size++;
        return sequence
      }

      effects.exit('thematicBreakSequence');
      return atBreak(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').TokenizeContext} TokenizeContext
   * @typedef {import('micromark-util-types').Exiter} Exiter
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Code} Code
   */
  /** @type {Construct} */

  const list$1 = {
    name: 'list',
    tokenize: tokenizeListStart,
    continuation: {
      tokenize: tokenizeListContinuation
    },
    exit: tokenizeListEnd
  };
  /** @type {Construct} */

  const listItemPrefixWhitespaceConstruct = {
    tokenize: tokenizeListItemPrefixWhitespace,
    partial: true
  };
  /** @type {Construct} */

  const indentConstruct = {
    tokenize: tokenizeIndent$1,
    partial: true
  };
  /**
   * @type {Tokenizer}
   * @this {TokenizeContextWithState}
   */

  function tokenizeListStart(effects, ok, nok) {
    const self = this;
    const tail = self.events[self.events.length - 1];
    let initialSize =
      tail && tail[1].type === 'linePrefix'
        ? tail[2].sliceSerialize(tail[1], true).length
        : 0;
    let size = 0;
    return start
    /** @type {State} */

    function start(code) {
      const kind =
        self.containerState.type ||
        (code === 42 || code === 43 || code === 45
          ? 'listUnordered'
          : 'listOrdered');

      if (
        kind === 'listUnordered'
          ? !self.containerState.marker || code === self.containerState.marker
          : asciiDigit(code)
      ) {
        if (!self.containerState.type) {
          self.containerState.type = kind;
          effects.enter(kind, {
            _container: true
          });
        }

        if (kind === 'listUnordered') {
          effects.enter('listItemPrefix');
          return code === 42 || code === 45
            ? effects.check(thematicBreak$1, nok, atMarker)(code)
            : atMarker(code)
        }

        if (!self.interrupt || code === 49) {
          effects.enter('listItemPrefix');
          effects.enter('listItemValue');
          return inside(code)
        }
      }

      return nok(code)
    }
    /** @type {State} */

    function inside(code) {
      if (asciiDigit(code) && ++size < 10) {
        effects.consume(code);
        return inside
      }

      if (
        (!self.interrupt || size < 2) &&
        (self.containerState.marker
          ? code === self.containerState.marker
          : code === 41 || code === 46)
      ) {
        effects.exit('listItemValue');
        return atMarker(code)
      }

      return nok(code)
    }
    /**
     * @type {State}
     **/

    function atMarker(code) {
      effects.enter('listItemMarker');
      effects.consume(code);
      effects.exit('listItemMarker');
      self.containerState.marker = self.containerState.marker || code;
      return effects.check(
        blankLine, // Can’t be empty when interrupting.
        self.interrupt ? nok : onBlank,
        effects.attempt(
          listItemPrefixWhitespaceConstruct,
          endOfPrefix,
          otherPrefix
        )
      )
    }
    /** @type {State} */

    function onBlank(code) {
      self.containerState.initialBlankLine = true;
      initialSize++;
      return endOfPrefix(code)
    }
    /** @type {State} */

    function otherPrefix(code) {
      if (markdownSpace(code)) {
        effects.enter('listItemPrefixWhitespace');
        effects.consume(code);
        effects.exit('listItemPrefixWhitespace');
        return endOfPrefix
      }

      return nok(code)
    }
    /** @type {State} */

    function endOfPrefix(code) {
      self.containerState.size =
        initialSize +
        self.sliceSerialize(effects.exit('listItemPrefix'), true).length;
      return ok(code)
    }
  }
  /**
   * @type {Tokenizer}
   * @this {TokenizeContextWithState}
   */

  function tokenizeListContinuation(effects, ok, nok) {
    const self = this;
    self.containerState._closeFlow = undefined;
    return effects.check(blankLine, onBlank, notBlank)
    /** @type {State} */

    function onBlank(code) {
      self.containerState.furtherBlankLines =
        self.containerState.furtherBlankLines ||
        self.containerState.initialBlankLine; // We have a blank line.
      // Still, try to consume at most the items size.

      return factorySpace(
        effects,
        ok,
        'listItemIndent',
        self.containerState.size + 1
      )(code)
    }
    /** @type {State} */

    function notBlank(code) {
      if (self.containerState.furtherBlankLines || !markdownSpace(code)) {
        self.containerState.furtherBlankLines = undefined;
        self.containerState.initialBlankLine = undefined;
        return notInCurrentItem(code)
      }

      self.containerState.furtherBlankLines = undefined;
      self.containerState.initialBlankLine = undefined;
      return effects.attempt(indentConstruct, ok, notInCurrentItem)(code)
    }
    /** @type {State} */

    function notInCurrentItem(code) {
      // While we do continue, we signal that the flow should be closed.
      self.containerState._closeFlow = true; // As we’re closing flow, we’re no longer interrupting.

      self.interrupt = undefined;
      return factorySpace(
        effects,
        effects.attempt(list$1, ok, nok),
        'linePrefix',
        self.parser.constructs.disable.null.includes('codeIndented')
          ? undefined
          : 4
      )(code)
    }
  }
  /**
   * @type {Tokenizer}
   * @this {TokenizeContextWithState}
   */

  function tokenizeIndent$1(effects, ok, nok) {
    const self = this;
    return factorySpace(
      effects,
      afterPrefix,
      'listItemIndent',
      self.containerState.size + 1
    )
    /** @type {State} */

    function afterPrefix(code) {
      const tail = self.events[self.events.length - 1];
      return tail &&
        tail[1].type === 'listItemIndent' &&
        tail[2].sliceSerialize(tail[1], true).length === self.containerState.size
        ? ok(code)
        : nok(code)
    }
  }
  /**
   * @type {Exiter}
   * @this {TokenizeContextWithState}
   */

  function tokenizeListEnd(effects) {
    effects.exit(this.containerState.type);
  }
  /**
   * @type {Tokenizer}
   * @this {TokenizeContextWithState}
   */

  function tokenizeListItemPrefixWhitespace(effects, ok, nok) {
    const self = this;
    return factorySpace(
      effects,
      afterPrefix,
      'listItemPrefixWhitespace',
      self.parser.constructs.disable.null.includes('codeIndented')
        ? undefined
        : 4 + 1
    )
    /** @type {State} */

    function afterPrefix(code) {
      const tail = self.events[self.events.length - 1];
      return !markdownSpace(code) &&
        tail &&
        tail[1].type === 'listItemPrefixWhitespace'
        ? ok(code)
        : nok(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').Resolver} Resolver
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Code} Code
   */

  /** @type {Construct} */
  const setextUnderline = {
    name: 'setextUnderline',
    tokenize: tokenizeSetextUnderline,
    resolveTo: resolveToSetextUnderline
  };
  /** @type {Resolver} */

  function resolveToSetextUnderline(events, context) {
    let index = events.length;
    /** @type {number|undefined} */

    let content;
    /** @type {number|undefined} */

    let text;
    /** @type {number|undefined} */

    let definition; // Find the opening of the content.
    // It’ll always exist: we don’t tokenize if it isn’t there.

    while (index--) {
      if (events[index][0] === 'enter') {
        if (events[index][1].type === 'content') {
          content = index;
          break
        }

        if (events[index][1].type === 'paragraph') {
          text = index;
        }
      } // Exit
      else {
        if (events[index][1].type === 'content') {
          // Remove the content end (if needed we’ll add it later)
          events.splice(index, 1);
        }

        if (!definition && events[index][1].type === 'definition') {
          definition = index;
        }
      }
    }

    const heading = {
      type: 'setextHeading',
      start: Object.assign({}, events[text][1].start),
      end: Object.assign({}, events[events.length - 1][1].end)
    }; // Change the paragraph to setext heading text.

    events[text][1].type = 'setextHeadingText'; // If we have definitions in the content, we’ll keep on having content,
    // but we need move it.

    if (definition) {
      events.splice(text, 0, ['enter', heading, context]);
      events.splice(definition + 1, 0, ['exit', events[content][1], context]);
      events[content][1].end = Object.assign({}, events[definition][1].end);
    } else {
      events[content][1] = heading;
    } // Add the heading exit at the end.

    events.push(['exit', heading, context]);
    return events
  }
  /** @type {Tokenizer} */

  function tokenizeSetextUnderline(effects, ok, nok) {
    const self = this;
    let index = self.events.length;
    /** @type {NonNullable<Code>} */

    let marker;
    /** @type {boolean} */

    let paragraph; // Find an opening.

    while (index--) {
      // Skip enter/exit of line ending, line prefix, and content.
      // We can now either have a definition or a paragraph.
      if (
        self.events[index][1].type !== 'lineEnding' &&
        self.events[index][1].type !== 'linePrefix' &&
        self.events[index][1].type !== 'content'
      ) {
        paragraph = self.events[index][1].type === 'paragraph';
        break
      }
    }

    return start
    /** @type {State} */

    function start(code) {
      if (!self.parser.lazy[self.now().line] && (self.interrupt || paragraph)) {
        effects.enter('setextHeadingLine');
        effects.enter('setextHeadingLineSequence');
        marker = code;
        return closingSequence(code)
      }

      return nok(code)
    }
    /** @type {State} */

    function closingSequence(code) {
      if (code === marker) {
        effects.consume(code);
        return closingSequence
      }

      effects.exit('setextHeadingLineSequence');
      return factorySpace(effects, closingSequenceEnd, 'lineSuffix')(code)
    }
    /** @type {State} */

    function closingSequenceEnd(code) {
      if (code === null || markdownLineEnding(code)) {
        effects.exit('setextHeadingLine');
        return ok(code)
      }

      return nok(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Extension} Extension
   * @typedef {import('micromark-util-types').Resolver} Resolver
   * @typedef {import('micromark-util-types').Token} Token
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').Exiter} Exiter
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Event} Event
   */
  const indent = {
    tokenize: tokenizeIndent,
    partial: true
  };
  /**
   * @returns {Extension}
   */

  function gfmFootnote() {
    /** @type {Extension} */
    return {
      document: {
        [91]: {
          tokenize: tokenizeDefinitionStart,
          continuation: {
            tokenize: tokenizeDefinitionContinuation
          },
          exit: gfmFootnoteDefinitionEnd
        }
      },
      text: {
        [91]: {
          tokenize: tokenizeGfmFootnoteCall
        },
        [93]: {
          add: 'after',
          tokenize: tokenizePotentialGfmFootnoteCall,
          resolveTo: resolveToPotentialGfmFootnoteCall
        }
      }
    }
  }
  /** @type {Tokenizer} */

  function tokenizePotentialGfmFootnoteCall(effects, ok, nok) {
    const self = this;
    let index = self.events.length;
    /** @type {Array<string>} */
    // @ts-expect-error It’s fine!

    const defined = self.parser.gfmFootnotes || (self.parser.gfmFootnotes = []);
    /** @type {Token} */

    let labelStart; // Find an opening.

    while (index--) {
      const token = self.events[index][1];

      if (token.type === 'labelImage') {
        labelStart = token;
        break
      } // Exit if we’ve walked far enough.

      if (
        token.type === 'gfmFootnoteCall' ||
        token.type === 'labelLink' ||
        token.type === 'label' ||
        token.type === 'image' ||
        token.type === 'link'
      ) {
        break
      }
    }

    return start
    /** @type {State} */

    function start(code) {
      if (!labelStart || !labelStart._balanced) {
        return nok(code)
      }

      const id = normalizeIdentifier(
        self.sliceSerialize({
          start: labelStart.end,
          end: self.now()
        })
      );

      if (id.charCodeAt(0) !== 94 || !defined.includes(id.slice(1))) {
        return nok(code)
      }

      effects.enter('gfmFootnoteCallLabelMarker');
      effects.consume(code);
      effects.exit('gfmFootnoteCallLabelMarker');
      return ok(code)
    }
  }
  /** @type {Resolver} */

  function resolveToPotentialGfmFootnoteCall(events, context) {
    let index = events.length;

    while (index--) {
      if (
        events[index][1].type === 'labelImage' &&
        events[index][0] === 'enter'
      ) {
        events[index][1];
        break
      }
    }

    // Change the `labelImageMarker` to a `data`.
    events[index + 1][1].type = 'data';
    events[index + 3][1].type = 'gfmFootnoteCallLabelMarker'; // The whole (without `!`):

    const call = {
      type: 'gfmFootnoteCall',
      start: Object.assign({}, events[index + 3][1].start),
      end: Object.assign({}, events[events.length - 1][1].end)
    }; // The `^` marker

    const marker = {
      type: 'gfmFootnoteCallMarker',
      start: Object.assign({}, events[index + 3][1].end),
      end: Object.assign({}, events[index + 3][1].end)
    }; // Increment the end 1 character.

    marker.end.column++;
    marker.end.offset++;
    marker.end._bufferIndex++;
    const string = {
      type: 'gfmFootnoteCallString',
      start: Object.assign({}, marker.end),
      end: Object.assign({}, events[events.length - 1][1].start)
    };
    const chunk = {
      type: 'chunkString',
      contentType: 'string',
      start: Object.assign({}, string.start),
      end: Object.assign({}, string.end)
    };
    /** @type {Array<Event>} */

    const replacement = [
      // Take the `labelImageMarker` (now `data`, the `!`)
      events[index + 1],
      events[index + 2],
      ['enter', call, context], // The `[`
      events[index + 3],
      events[index + 4], // The `^`.
      ['enter', marker, context],
      ['exit', marker, context], // Everything in between.
      ['enter', string, context],
      ['enter', chunk, context],
      ['exit', chunk, context],
      ['exit', string, context], // The ending (`]`, properly parsed and labelled).
      events[events.length - 2],
      events[events.length - 1],
      ['exit', call, context]
    ];
    events.splice(index, events.length - index + 1, ...replacement);
    return events
  }
  /** @type {Tokenizer} */

  function tokenizeGfmFootnoteCall(effects, ok, nok) {
    const self = this;
    /** @type {Array<string>} */
    // @ts-expect-error It’s fine!

    const defined = self.parser.gfmFootnotes || (self.parser.gfmFootnotes = []);
    let size = 0;
    /** @type {boolean} */

    let data;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('gfmFootnoteCall');
      effects.enter('gfmFootnoteCallLabelMarker');
      effects.consume(code);
      effects.exit('gfmFootnoteCallLabelMarker');
      return callStart
    }
    /** @type {State} */

    function callStart(code) {
      if (code !== 94) return nok(code)
      effects.enter('gfmFootnoteCallMarker');
      effects.consume(code);
      effects.exit('gfmFootnoteCallMarker');
      effects.enter('gfmFootnoteCallString');
      effects.enter('chunkString').contentType = 'string';
      return callData
    }
    /** @type {State} */

    function callData(code) {
      /** @type {Token} */
      let token;

      if (code === null || code === 91 || size++ > 999) {
        return nok(code)
      }

      if (code === 93) {
        if (!data) {
          return nok(code)
        }

        effects.exit('chunkString');
        token = effects.exit('gfmFootnoteCallString');
        return defined.includes(normalizeIdentifier(self.sliceSerialize(token)))
          ? end(code)
          : nok(code)
      }

      effects.consume(code);

      if (!markdownLineEndingOrSpace(code)) {
        data = true;
      }

      return code === 92 ? callEscape : callData
    }
    /** @type {State} */

    function callEscape(code) {
      if (code === 91 || code === 92 || code === 93) {
        effects.consume(code);
        size++;
        return callData
      }

      return callData(code)
    }
    /** @type {State} */

    function end(code) {
      effects.enter('gfmFootnoteCallLabelMarker');
      effects.consume(code);
      effects.exit('gfmFootnoteCallLabelMarker');
      effects.exit('gfmFootnoteCall');
      return ok
    }
  }
  /** @type {Tokenizer} */

  function tokenizeDefinitionStart(effects, ok, nok) {
    const self = this;
    /** @type {Array<string>} */
    // @ts-expect-error It’s fine!

    const defined = self.parser.gfmFootnotes || (self.parser.gfmFootnotes = []);
    /** @type {string} */

    let identifier;
    let size = 0;
    /** @type {boolean|undefined} */

    let data;
    return start
    /** @type {State} */

    function start(code) {
      effects.enter('gfmFootnoteDefinition')._container = true;
      effects.enter('gfmFootnoteDefinitionLabel');
      effects.enter('gfmFootnoteDefinitionLabelMarker');
      effects.consume(code);
      effects.exit('gfmFootnoteDefinitionLabelMarker');
      return labelStart
    }
    /** @type {State} */

    function labelStart(code) {
      if (code === 94) {
        effects.enter('gfmFootnoteDefinitionMarker');
        effects.consume(code);
        effects.exit('gfmFootnoteDefinitionMarker');
        effects.enter('gfmFootnoteDefinitionLabelString');
        return atBreak
      }

      return nok(code)
    }
    /** @type {State} */

    function atBreak(code) {
      /** @type {Token} */
      let token;

      if (code === null || code === 91 || size > 999) {
        return nok(code)
      }

      if (code === 93) {
        if (!data) {
          return nok(code)
        }

        token = effects.exit('gfmFootnoteDefinitionLabelString');
        identifier = normalizeIdentifier(self.sliceSerialize(token));
        effects.enter('gfmFootnoteDefinitionLabelMarker');
        effects.consume(code);
        effects.exit('gfmFootnoteDefinitionLabelMarker');
        effects.exit('gfmFootnoteDefinitionLabel');
        return labelAfter
      }

      if (markdownLineEnding(code)) {
        effects.enter('lineEnding');
        effects.consume(code);
        effects.exit('lineEnding');
        size++;
        return atBreak
      }

      effects.enter('chunkString').contentType = 'string';
      return label(code)
    }
    /** @type {State} */

    function label(code) {
      if (
        code === null ||
        markdownLineEnding(code) ||
        code === 91 ||
        code === 93 ||
        size > 999
      ) {
        effects.exit('chunkString');
        return atBreak(code)
      }

      if (!markdownLineEndingOrSpace(code)) {
        data = true;
      }

      size++;
      effects.consume(code);
      return code === 92 ? labelEscape : label
    }
    /** @type {State} */

    function labelEscape(code) {
      if (code === 91 || code === 92 || code === 93) {
        effects.consume(code);
        size++;
        return label
      }

      return label(code)
    }
    /** @type {State} */

    function labelAfter(code) {
      if (code === 58) {
        effects.enter('definitionMarker');
        effects.consume(code);
        effects.exit('definitionMarker'); // Any whitespace after the marker is eaten, forming indented code
        // is not possible.
        // No space is also fine, just like a block quote marker.

        return factorySpace(effects, done, 'gfmFootnoteDefinitionWhitespace')
      }

      return nok(code)
    }
    /** @type {State} */

    function done(code) {
      if (!defined.includes(identifier)) {
        defined.push(identifier);
      }

      return ok(code)
    }
  }
  /** @type {Tokenizer} */

  function tokenizeDefinitionContinuation(effects, ok, nok) {
    // Either a blank line, which is okay, or an indented thing.
    return effects.check(blankLine, ok, effects.attempt(indent, ok, nok))
  }
  /** @type {Exiter} */

  function gfmFootnoteDefinitionEnd(effects) {
    effects.exit('gfmFootnoteDefinition');
  }
  /** @type {Tokenizer} */

  function tokenizeIndent(effects, ok, nok) {
    const self = this;
    return factorySpace(
      effects,
      afterPrefix,
      'gfmFootnoteDefinitionIndent',
      4 + 1
    )
    /** @type {State} */

    function afterPrefix(code) {
      const tail = self.events[self.events.length - 1];
      return tail &&
        tail[1].type === 'gfmFootnoteDefinitionIndent' &&
        tail[2].sliceSerialize(tail[1], true).length === 4
        ? ok(code)
        : nok(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Extension} Extension
   * @typedef {import('micromark-util-types').Resolver} Resolver
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Token} Token
   * @typedef {import('micromark-util-types').Event} Event
   */

  /**
   * @param {Options} [options]
   * @returns {Extension}
   */
  function gfmStrikethrough(options = {}) {
    let single = options.singleTilde;
    const tokenizer = {
      tokenize: tokenizeStrikethrough,
      resolveAll: resolveAllStrikethrough
    };

    if (single === null || single === undefined) {
      single = true;
    }

    return {
      text: {
        [126]: tokenizer
      },
      insideSpan: {
        null: [tokenizer]
      },
      attentionMarkers: {
        null: [126]
      }
    }
    /**
     * Take events and resolve strikethrough.
     *
     * @type {Resolver}
     */

    function resolveAllStrikethrough(events, context) {
      let index = -1; // Walk through all events.

      while (++index < events.length) {
        // Find a token that can close.
        if (
          events[index][0] === 'enter' &&
          events[index][1].type === 'strikethroughSequenceTemporary' &&
          events[index][1]._close
        ) {
          let open = index; // Now walk back to find an opener.

          while (open--) {
            // Find a token that can open the closer.
            if (
              events[open][0] === 'exit' &&
              events[open][1].type === 'strikethroughSequenceTemporary' &&
              events[open][1]._open && // If the sizes are the same:
              events[index][1].end.offset - events[index][1].start.offset ===
                events[open][1].end.offset - events[open][1].start.offset
            ) {
              events[index][1].type = 'strikethroughSequence';
              events[open][1].type = 'strikethroughSequence';
              const strikethrough = {
                type: 'strikethrough',
                start: Object.assign({}, events[open][1].start),
                end: Object.assign({}, events[index][1].end)
              };
              const text = {
                type: 'strikethroughText',
                start: Object.assign({}, events[open][1].end),
                end: Object.assign({}, events[index][1].start)
              }; // Opening.

              const nextEvents = [
                ['enter', strikethrough, context],
                ['enter', events[open][1], context],
                ['exit', events[open][1], context],
                ['enter', text, context]
              ]; // Between.

              splice(
                nextEvents,
                nextEvents.length,
                0,
                resolveAll(
                  context.parser.constructs.insideSpan.null,
                  events.slice(open + 1, index),
                  context
                )
              ); // Closing.

              splice(nextEvents, nextEvents.length, 0, [
                ['exit', text, context],
                ['enter', events[index][1], context],
                ['exit', events[index][1], context],
                ['exit', strikethrough, context]
              ]);
              splice(events, open - 1, index - open + 3, nextEvents);
              index = open + nextEvents.length - 2;
              break
            }
          }
        }
      }

      index = -1;

      while (++index < events.length) {
        if (events[index][1].type === 'strikethroughSequenceTemporary') {
          events[index][1].type = 'data';
        }
      }

      return events
    }
    /** @type {Tokenizer} */

    function tokenizeStrikethrough(effects, ok, nok) {
      const previous = this.previous;
      const events = this.events;
      let size = 0;
      return start
      /** @type {State} */

      function start(code) {
        if (
          previous === 126 &&
          events[events.length - 1][1].type !== 'characterEscape'
        ) {
          return nok(code)
        }

        effects.enter('strikethroughSequenceTemporary');
        return more(code)
      }
      /** @type {State} */

      function more(code) {
        const before = classifyCharacter(previous);

        if (code === 126) {
          // If this is the third marker, exit.
          if (size > 1) return nok(code)
          effects.consume(code);
          size++;
          return more
        }

        if (size < 2 && !single) return nok(code)
        const token = effects.exit('strikethroughSequenceTemporary');
        const after = classifyCharacter(code);
        token._open = !after || (after === 2 && Boolean(before));
        token._close = !before || (before === 2 && Boolean(after));
        return ok(code)
      }
    }
  }

  /**
   * @typedef {import('micromark-util-types').Extension} Extension
   * @typedef {import('micromark-util-types').Resolver} Resolver
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Token} Token
   */

  /** @type {Extension} */
  const gfmTable = {
    flow: {
      null: {
        tokenize: tokenizeTable,
        resolve: resolveTable
      }
    }
  };
  const nextPrefixedOrBlank = {
    tokenize: tokenizeNextPrefixedOrBlank,
    partial: true
  };
  /** @type {Resolver} */

  function resolveTable(events, context) {
    let index = -1;
    /** @type {boolean|undefined} */

    let inHead;
    /** @type {boolean|undefined} */

    let inDelimiterRow;
    /** @type {boolean|undefined} */

    let inRow;
    /** @type {number|undefined} */

    let contentStart;
    /** @type {number|undefined} */

    let contentEnd;
    /** @type {number|undefined} */

    let cellStart;
    /** @type {boolean|undefined} */

    let seenCellInRow;

    while (++index < events.length) {
      const token = events[index][1];

      if (inRow) {
        if (token.type === 'temporaryTableCellContent') {
          contentStart = contentStart || index;
          contentEnd = index;
        }

        if (
          // Combine separate content parts into one.
          (token.type === 'tableCellDivider' || token.type === 'tableRow') &&
          contentEnd
        ) {
          const content = {
            type: 'tableContent',
            start: events[contentStart][1].start,
            end: events[contentEnd][1].end
          };
          /** @type {Token} */

          const text = {
            type: 'chunkText',
            start: content.start,
            end: content.end,
            // @ts-expect-error It’s fine.
            contentType: 'text'
          };
          events.splice(
            contentStart,
            contentEnd - contentStart + 1,
            ['enter', content, context],
            ['enter', text, context],
            ['exit', text, context],
            ['exit', content, context]
          );
          index -= contentEnd - contentStart - 3;
          contentStart = undefined;
          contentEnd = undefined;
        }
      }

      if (
        events[index][0] === 'exit' &&
        cellStart !== undefined &&
        cellStart + (seenCellInRow ? 0 : 1) < index &&
        (token.type === 'tableCellDivider' ||
          (token.type === 'tableRow' &&
            (cellStart + 3 < index ||
              events[cellStart][1].type !== 'whitespace')))
      ) {
        const cell = {
          type: inDelimiterRow
            ? 'tableDelimiter'
            : inHead
            ? 'tableHeader'
            : 'tableData',
          start: events[cellStart][1].start,
          end: events[index][1].end
        };
        events.splice(index + (token.type === 'tableCellDivider' ? 1 : 0), 0, [
          'exit',
          cell,
          context
        ]);
        events.splice(cellStart, 0, ['enter', cell, context]);
        index += 2;
        cellStart = index + 1;
        seenCellInRow = true;
      }

      if (token.type === 'tableRow') {
        inRow = events[index][0] === 'enter';

        if (inRow) {
          cellStart = index + 1;
          seenCellInRow = false;
        }
      }

      if (token.type === 'tableDelimiterRow') {
        inDelimiterRow = events[index][0] === 'enter';

        if (inDelimiterRow) {
          cellStart = index + 1;
          seenCellInRow = false;
        }
      }

      if (token.type === 'tableHead') {
        inHead = events[index][0] === 'enter';
      }
    }

    return events
  }
  /** @type {Tokenizer} */

  function tokenizeTable(effects, ok, nok) {
    const self = this;
    /** @type {Array<Align>} */

    const align = [];
    let tableHeaderCount = 0;
    /** @type {boolean|undefined} */

    let seenDelimiter;
    /** @type {boolean|undefined} */

    let hasDash;
    return start
    /** @type {State} */

    function start(code) {
      // @ts-expect-error Custom.
      effects.enter('table')._align = align;
      effects.enter('tableHead');
      effects.enter('tableRow'); // If we start with a pipe, we open a cell marker.

      if (code === 124) {
        return cellDividerHead(code)
      }

      tableHeaderCount++;
      effects.enter('temporaryTableCellContent'); // Can’t be space or eols at the start of a construct, so we’re in a cell.

      return inCellContentHead(code)
    }
    /** @type {State} */

    function cellDividerHead(code) {
      effects.enter('tableCellDivider');
      effects.consume(code);
      effects.exit('tableCellDivider');
      seenDelimiter = true;
      return cellBreakHead
    }
    /** @type {State} */

    function cellBreakHead(code) {
      if (code === null || markdownLineEnding(code)) {
        return atRowEndHead(code)
      }

      if (markdownSpace(code)) {
        effects.enter('whitespace');
        effects.consume(code);
        return inWhitespaceHead
      }

      if (seenDelimiter) {
        seenDelimiter = undefined;
        tableHeaderCount++;
      }

      if (code === 124) {
        return cellDividerHead(code)
      } // Anything else is cell content.

      effects.enter('temporaryTableCellContent');
      return inCellContentHead(code)
    }
    /** @type {State} */

    function inWhitespaceHead(code) {
      if (markdownSpace(code)) {
        effects.consume(code);
        return inWhitespaceHead
      }

      effects.exit('whitespace');
      return cellBreakHead(code)
    }
    /** @type {State} */

    function inCellContentHead(code) {
      // EOF, whitespace, pipe
      if (code === null || code === 124 || markdownLineEndingOrSpace(code)) {
        effects.exit('temporaryTableCellContent');
        return cellBreakHead(code)
      }

      effects.consume(code);
      return code === 92 ? inCellContentEscapeHead : inCellContentHead
    }
    /** @type {State} */

    function inCellContentEscapeHead(code) {
      if (code === 92 || code === 124) {
        effects.consume(code);
        return inCellContentHead
      } // Anything else.

      return inCellContentHead(code)
    }
    /** @type {State} */

    function atRowEndHead(code) {
      if (code === null) {
        return nok(code)
      }

      effects.exit('tableRow');
      effects.exit('tableHead');
      const originalInterrupt = self.interrupt;
      self.interrupt = true;
      return effects.attempt(
        {
          tokenize: tokenizeRowEnd,
          partial: true
        },
        function (code) {
          self.interrupt = originalInterrupt;
          effects.enter('tableDelimiterRow');
          return atDelimiterRowBreak(code)
        },
        function (code) {
          self.interrupt = originalInterrupt;
          return nok(code)
        }
      )(code)
    }
    /** @type {State} */

    function atDelimiterRowBreak(code) {
      if (code === null || markdownLineEnding(code)) {
        return rowEndDelimiter(code)
      }

      if (markdownSpace(code)) {
        effects.enter('whitespace');
        effects.consume(code);
        return inWhitespaceDelimiter
      }

      if (code === 45) {
        effects.enter('tableDelimiterFiller');
        effects.consume(code);
        hasDash = true;
        align.push('none');
        return inFillerDelimiter
      }

      if (code === 58) {
        effects.enter('tableDelimiterAlignment');
        effects.consume(code);
        effects.exit('tableDelimiterAlignment');
        align.push('left');
        return afterLeftAlignment
      } // If we start with a pipe, we open a cell marker.

      if (code === 124) {
        effects.enter('tableCellDivider');
        effects.consume(code);
        effects.exit('tableCellDivider');
        return atDelimiterRowBreak
      }

      return nok(code)
    }
    /** @type {State} */

    function inWhitespaceDelimiter(code) {
      if (markdownSpace(code)) {
        effects.consume(code);
        return inWhitespaceDelimiter
      }

      effects.exit('whitespace');
      return atDelimiterRowBreak(code)
    }
    /** @type {State} */

    function inFillerDelimiter(code) {
      if (code === 45) {
        effects.consume(code);
        return inFillerDelimiter
      }

      effects.exit('tableDelimiterFiller');

      if (code === 58) {
        effects.enter('tableDelimiterAlignment');
        effects.consume(code);
        effects.exit('tableDelimiterAlignment');
        align[align.length - 1] =
          align[align.length - 1] === 'left' ? 'center' : 'right';
        return afterRightAlignment
      }

      return atDelimiterRowBreak(code)
    }
    /** @type {State} */

    function afterLeftAlignment(code) {
      if (code === 45) {
        effects.enter('tableDelimiterFiller');
        effects.consume(code);
        hasDash = true;
        return inFillerDelimiter
      } // Anything else is not ok.

      return nok(code)
    }
    /** @type {State} */

    function afterRightAlignment(code) {
      if (code === null || markdownLineEnding(code)) {
        return rowEndDelimiter(code)
      }

      if (markdownSpace(code)) {
        effects.enter('whitespace');
        effects.consume(code);
        return inWhitespaceDelimiter
      } // `|`

      if (code === 124) {
        effects.enter('tableCellDivider');
        effects.consume(code);
        effects.exit('tableCellDivider');
        return atDelimiterRowBreak
      }

      return nok(code)
    }
    /** @type {State} */

    function rowEndDelimiter(code) {
      effects.exit('tableDelimiterRow'); // Exit if there was no dash at all, or if the header cell count is not the
      // delimiter cell count.

      if (!hasDash || tableHeaderCount !== align.length) {
        return nok(code)
      }

      if (code === null) {
        return tableClose(code)
      }

      return effects.check(
        nextPrefixedOrBlank,
        tableClose,
        effects.attempt(
          {
            tokenize: tokenizeRowEnd,
            partial: true
          },
          factorySpace(effects, bodyStart, 'linePrefix', 4),
          tableClose
        )
      )(code)
    }
    /** @type {State} */

    function tableClose(code) {
      effects.exit('table');
      return ok(code)
    }
    /** @type {State} */

    function bodyStart(code) {
      effects.enter('tableBody');
      return rowStartBody(code)
    }
    /** @type {State} */

    function rowStartBody(code) {
      effects.enter('tableRow'); // If we start with a pipe, we open a cell marker.

      if (code === 124) {
        return cellDividerBody(code)
      }

      effects.enter('temporaryTableCellContent'); // Can’t be space or eols at the start of a construct, so we’re in a cell.

      return inCellContentBody(code)
    }
    /** @type {State} */

    function cellDividerBody(code) {
      effects.enter('tableCellDivider');
      effects.consume(code);
      effects.exit('tableCellDivider');
      return cellBreakBody
    }
    /** @type {State} */

    function cellBreakBody(code) {
      if (code === null || markdownLineEnding(code)) {
        return atRowEndBody(code)
      }

      if (markdownSpace(code)) {
        effects.enter('whitespace');
        effects.consume(code);
        return inWhitespaceBody
      } // `|`

      if (code === 124) {
        return cellDividerBody(code)
      } // Anything else is cell content.

      effects.enter('temporaryTableCellContent');
      return inCellContentBody(code)
    }
    /** @type {State} */

    function inWhitespaceBody(code) {
      if (markdownSpace(code)) {
        effects.consume(code);
        return inWhitespaceBody
      }

      effects.exit('whitespace');
      return cellBreakBody(code)
    }
    /** @type {State} */

    function inCellContentBody(code) {
      // EOF, whitespace, pipe
      if (code === null || code === 124 || markdownLineEndingOrSpace(code)) {
        effects.exit('temporaryTableCellContent');
        return cellBreakBody(code)
      }

      effects.consume(code);
      return code === 92 ? inCellContentEscapeBody : inCellContentBody
    }
    /** @type {State} */

    function inCellContentEscapeBody(code) {
      if (code === 92 || code === 124) {
        effects.consume(code);
        return inCellContentBody
      } // Anything else.

      return inCellContentBody(code)
    }
    /** @type {State} */

    function atRowEndBody(code) {
      effects.exit('tableRow');

      if (code === null) {
        return tableBodyClose(code)
      }

      return effects.check(
        nextPrefixedOrBlank,
        tableBodyClose,
        effects.attempt(
          {
            tokenize: tokenizeRowEnd,
            partial: true
          },
          factorySpace(effects, rowStartBody, 'linePrefix', 4),
          tableBodyClose
        )
      )(code)
    }
    /** @type {State} */

    function tableBodyClose(code) {
      effects.exit('tableBody');
      return tableClose(code)
    }
    /** @type {Tokenizer} */

    function tokenizeRowEnd(effects, ok, nok) {
      return start
      /** @type {State} */

      function start(code) {
        effects.enter('lineEnding');
        effects.consume(code);
        effects.exit('lineEnding');
        return factorySpace(effects, prefixed, 'linePrefix')
      }
      /** @type {State} */

      function prefixed(code) {
        // Blank or interrupting line.
        if (
          self.parser.lazy[self.now().line] ||
          code === null ||
          markdownLineEnding(code)
        ) {
          return nok(code)
        }

        const tail = self.events[self.events.length - 1]; // Indented code can interrupt delimiter and body rows.

        if (
          !self.parser.constructs.disable.null.includes('codeIndented') &&
          tail &&
          tail[1].type === 'linePrefix' &&
          tail[2].sliceSerialize(tail[1], true).length >= 4
        ) {
          return nok(code)
        }

        self._gfmTableDynamicInterruptHack = true;
        return effects.check(
          self.parser.constructs.flow,
          function (code) {
            self._gfmTableDynamicInterruptHack = false;
            return nok(code)
          },
          function (code) {
            self._gfmTableDynamicInterruptHack = false;
            return ok(code)
          }
        )(code)
      }
    }
  }
  /** @type {Tokenizer} */

  function tokenizeNextPrefixedOrBlank(effects, ok, nok) {
    let size = 0;
    return start
    /** @type {State} */

    function start(code) {
      // This is a check, so we don’t care about tokens, but we open a bogus one
      // so we’re valid.
      effects.enter('check'); // EOL.

      effects.consume(code);
      return whitespace
    }
    /** @type {State} */

    function whitespace(code) {
      if (code === -1 || code === 32) {
        effects.consume(code);
        size++;
        return size === 4 ? ok : whitespace
      } // EOF or whitespace

      if (code === null || markdownLineEndingOrSpace(code)) {
        return ok(code)
      } // Anything else.

      return nok(code)
    }
  }

  /**
   * @typedef {import('micromark-util-types').Extension} Extension
   * @typedef {import('micromark-util-types').ConstructRecord} ConstructRecord
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').Previous} Previous
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Event} Event
   * @typedef {import('micromark-util-types').Code} Code
   */
  const tasklistCheck = {
    tokenize: tokenizeTasklistCheck
  };
  const gfmTaskListItem = {
    text: {
      [91]: tasklistCheck
    }
  };
  /** @type {Tokenizer} */

  function tokenizeTasklistCheck(effects, ok, nok) {
    const self = this;
    return open
    /** @type {State} */

    function open(code) {
      if (
        // Exit if there’s stuff before.
        self.previous !== null || // Exit if not in the first content that is the first child of a list
        // item.
        !self._gfmTasklistFirstContentOfListItem
      ) {
        return nok(code)
      }

      effects.enter('taskListCheck');
      effects.enter('taskListCheckMarker');
      effects.consume(code);
      effects.exit('taskListCheckMarker');
      return inside
    }
    /** @type {State} */

    function inside(code) {
      // To match how GH works in comments, use `markdownSpace` (`[ \t]`) instead
      // of `markdownLineEndingOrSpace` (`[ \t\r\n]`).
      if (markdownLineEndingOrSpace(code)) {
        effects.enter('taskListCheckValueUnchecked');
        effects.consume(code);
        effects.exit('taskListCheckValueUnchecked');
        return close
      }

      if (code === 88 || code === 120) {
        effects.enter('taskListCheckValueChecked');
        effects.consume(code);
        effects.exit('taskListCheckValueChecked');
        return close
      }

      return nok(code)
    }
    /** @type {State} */

    function close(code) {
      if (code === 93) {
        effects.enter('taskListCheckMarker');
        effects.consume(code);
        effects.exit('taskListCheckMarker');
        effects.exit('taskListCheck');
        return effects.check(
          {
            tokenize: spaceThenNonSpace
          },
          ok,
          nok
        )
      }

      return nok(code)
    }
  }
  /** @type {Tokenizer} */

  function spaceThenNonSpace(effects, ok, nok) {
    const self = this;
    return factorySpace(effects, after, 'whitespace')
    /** @type {State} */

    function after(code) {
      const tail = self.events[self.events.length - 1];
      return (
        // We either found spaces…
        ((tail && tail[1].type === 'whitespace') || // …or it was followed by a line ending, in which case, there has to be
          // non-whitespace after that line ending, because otherwise we’d get an
          // EOF as the content is closed with blank lines.
          markdownLineEnding(code)) &&
          code !== null
          ? ok(code)
          : nok(code)
      )
    }
  }

  /**
   * @typedef {import('micromark-util-types').Extension} Extension
   * @typedef {import('micromark-util-types').HtmlExtension} HtmlExtension
   * @typedef {import('micromark-extension-gfm-strikethrough').Options} Options
   * @typedef {import('micromark-extension-gfm-footnote').HtmlOptions} HtmlOptions
   */

  /**
   * Support GFM or markdown on github.com.
   *
   * @param {Options} [options]
   * @returns {Extension}
   */
  function gfm(options) {
    return combineExtensions([
      gfmAutolinkLiteral,
      gfmFootnote(),
      gfmStrikethrough(options),
      gfmTable,
      gfmTaskListItem
    ])
  }

  /**
   * Count how often a character (or substring) is used in a string.
   *
   * @param {string} value
   *   Value to search in.
   * @param {string} character
   *   Character (or substring) to look for.
   * @return {number}
   *   Number of times `character` occurred in `value`.
   */
  function ccount$1(value, character) {
    const source = String(value);

    if (typeof character !== 'string') {
      throw new TypeError('Expected character')
    }

    let count = 0;
    let index = source.indexOf(character);

    while (index !== -1) {
      count++;
      index = source.indexOf(character, index + character.length);
    }

    return count
  }

  function escapeStringRegexp(string) {
    if (typeof string !== 'string') {
      throw new TypeError('Expected a string');
    }

    // Escape characters with special meaning either inside or outside character sets.
    // Use a simple backslash escape when it’s always valid, and a `\xnn` escape when the simpler form would be disallowed by Unicode patterns’ stricter grammar.
    return string
      .replace(/[|\\{}()[\]^$+*?.]/g, '\\$&')
      .replace(/-/g, '\\x2d');
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   *
   * @typedef {string} Type
   * @typedef {Object<string, unknown>} Props
   *
   * @typedef {null|undefined|Type|Props|TestFunctionAnything|Array.<Type|Props|TestFunctionAnything>} Test
   */

  const convert$1 =
    /**
     * @type {(
     *   (<T extends Node>(test: T['type']|Partial<T>|TestFunctionPredicate<T>) => AssertPredicate<T>) &
     *   ((test?: Test) => AssertAnything)
     * )}
     */
    (
      /**
       * Generate an assertion from a check.
       * @param {Test} [test]
       * When nullish, checks if `node` is a `Node`.
       * When `string`, works like passing `function (node) {return node.type === test}`.
       * When `function` checks if function passed the node is true.
       * When `object`, checks that all keys in test are in node, and that they have (strictly) equal values.
       * When `array`, checks any one of the subtests pass.
       * @returns {AssertAnything}
       */
      function (test) {
        if (test === undefined || test === null) {
          return ok$1
        }

        if (typeof test === 'string') {
          return typeFactory$1(test)
        }

        if (typeof test === 'object') {
          return Array.isArray(test) ? anyFactory$4(test) : propsFactory$1(test)
        }

        if (typeof test === 'function') {
          return castFactory$4(test)
        }

        throw new Error('Expected function, string, or object as test')
      }
    );
  /**
   * @param {Array.<Type|Props|TestFunctionAnything>} tests
   * @returns {AssertAnything}
   */
  function anyFactory$4(tests) {
    /** @type {Array.<AssertAnything>} */
    const checks = [];
    let index = -1;

    while (++index < tests.length) {
      checks[index] = convert$1(tests[index]);
    }

    return castFactory$4(any)

    /**
     * @this {unknown}
     * @param {unknown[]} parameters
     * @returns {boolean}
     */
    function any(...parameters) {
      let index = -1;

      while (++index < checks.length) {
        if (checks[index].call(this, ...parameters)) return true
      }

      return false
    }
  }

  /**
   * Utility to assert each property in `test` is represented in `node`, and each
   * values are strictly equal.
   *
   * @param {Props} check
   * @returns {AssertAnything}
   */
  function propsFactory$1(check) {
    return castFactory$4(all)

    /**
     * @param {Node} node
     * @returns {boolean}
     */
    function all(node) {
      /** @type {string} */
      let key;

      for (key in check) {
        // @ts-expect-error: hush, it sure works as an index.
        if (node[key] !== check[key]) return false
      }

      return true
    }
  }

  /**
   * Utility to convert a string into a function which checks a given node’s type
   * for said string.
   *
   * @param {Type} check
   * @returns {AssertAnything}
   */
  function typeFactory$1(check) {
    return castFactory$4(type)

    /**
     * @param {Node} node
     */
    function type(node) {
      return node && node.type === check
    }
  }

  /**
   * Utility to convert a string into a function which checks a given node’s type
   * for said string.
   * @param {TestFunctionAnything} check
   * @returns {AssertAnything}
   */
  function castFactory$4(check) {
    return assertion

    /**
     * @this {unknown}
     * @param {Array.<unknown>} parameters
     * @returns {boolean}
     */
    function assertion(...parameters) {
      // @ts-expect-error: spreading is fine.
      return Boolean(check.call(this, ...parameters))
    }
  }

  // Utility to return true.
  function ok$1() {
    return true
  }

  /**
   * @param {string} d
   * @returns {string}
   */
  function color$3(d) {
    return d
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   * @typedef {import('unist-util-is').Test} Test
   */

  /**
   * Continue traversing as normal
   */
  const CONTINUE$3 = true;
  /**
   * Do not traverse this node’s children
   */
  const SKIP$3 = 'skip';
  /**
   * Stop traversing immediately
   */
  const EXIT$3 = false;

  const visitParents$3 =
    /**
     * @type {(
     *   (<T extends Node>(tree: Node, test: T['type']|Partial<T>|import('unist-util-is').TestFunctionPredicate<T>|Array.<T['type']|Partial<T>|import('unist-util-is').TestFunctionPredicate<T>>, visitor: Visitor<T>, reverse?: boolean) => void) &
     *   ((tree: Node, test: Test, visitor: Visitor<Node>, reverse?: boolean) => void) &
     *   ((tree: Node, visitor: Visitor<Node>, reverse?: boolean) => void)
     * )}
     */
    (
      /**
       * Visit children of tree which pass a test
       *
       * @param {Node} tree Abstract syntax tree to walk
       * @param {Test} test test Test node
       * @param {Visitor<Node>} visitor Function to run for each node
       * @param {boolean} [reverse] Fisit the tree in reverse, defaults to false
       */
      function (tree, test, visitor, reverse) {
        if (typeof test === 'function' && typeof visitor !== 'function') {
          reverse = visitor;
          // @ts-ignore no visitor given, so `visitor` is test.
          visitor = test;
          test = null;
        }

        var is = convert$1(test);
        var step = reverse ? -1 : 1;

        factory(tree, null, [])();

        /**
         * @param {Node} node
         * @param {number?} index
         * @param {Array.<Parent>} parents
         */
        function factory(node, index, parents) {
          /** @type {Object.<string, unknown>} */
          var value = typeof node === 'object' && node !== null ? node : {};
          /** @type {string} */
          var name;

          if (typeof value.type === 'string') {
            name =
              typeof value.tagName === 'string'
                ? value.tagName
                : typeof value.name === 'string'
                ? value.name
                : undefined;

            Object.defineProperty(visit, 'name', {
              value:
                'node (' +
                color$3(value.type + (name ? '<' + name + '>' : '')) +
                ')'
            });
          }

          return visit

          function visit() {
            /** @type {ActionTuple} */
            var result = [];
            /** @type {ActionTuple} */
            var subresult;
            /** @type {number} */
            var offset;
            /** @type {Array.<Parent>} */
            var grandparents;

            if (!test || is(node, index, parents[parents.length - 1] || null)) {
              result = toResult$3(visitor(node, parents));

              if (result[0] === EXIT$3) {
                return result
              }
            }

            if (node.children && result[0] !== SKIP$3) {
              // @ts-ignore looks like a parent.
              offset = (reverse ? node.children.length : -1) + step;
              // @ts-ignore looks like a parent.
              grandparents = parents.concat(node);

              // @ts-ignore looks like a parent.
              while (offset > -1 && offset < node.children.length) {
                subresult = factory(node.children[offset], offset, grandparents)();

                if (subresult[0] === EXIT$3) {
                  return subresult
                }

                offset =
                  typeof subresult[1] === 'number' ? subresult[1] : offset + step;
              }
            }

            return result
          }
        }
      }
    );

  /**
   * @param {VisitorResult} value
   * @returns {ActionTuple}
   */
  function toResult$3(value) {
    if (Array.isArray(value)) {
      return value
    }

    if (typeof value === 'number') {
      return [CONTINUE$3, value]
    }

    return [value]
  }

  /**
   * @typedef Options Configuration.
   * @property {Test} [ignore] `unist-util-is` test used to assert parents
   *
   * @typedef {import('mdast').Root} Root
   * @typedef {import('mdast').Content} Content
   * @typedef {import('mdast').PhrasingContent} PhrasingContent
   * @typedef {import('mdast').Text} Text
   * @typedef {Content|Root} Node
   * @typedef {Extract<Node, import('mdast').Parent>} Parent
   *
   * @typedef {import('unist-util-visit-parents').Test} Test
   * @typedef {import('unist-util-visit-parents').VisitorResult} VisitorResult
   *
   * @typedef RegExpMatchObject
   * @property {number} index
   * @property {string} input
   *
   * @typedef {string|RegExp} Find
   * @typedef {string|ReplaceFunction} Replace
   *
   * @typedef {[Find, Replace]} FindAndReplaceTuple
   * @typedef {Object.<string, Replace>} FindAndReplaceSchema
   * @typedef {Array.<FindAndReplaceTuple>} FindAndReplaceList
   *
   * @typedef {[RegExp, ReplaceFunction]} Pair
   * @typedef {Array.<Pair>} Pairs
   */

  const own$r = {}.hasOwnProperty;

  /**
   * @param tree mdast tree
   * @param find Value to find and remove. When `string`, escaped and made into a global `RegExp`
   * @param [replace] Value to insert.
   *   * When `string`, turned into a Text node.
   *   * When `Function`, called with the results of calling `RegExp.exec` as
   *     arguments, in which case it can return a single or a list of `Node`,
   *     a `string` (which is wrapped in a `Text` node), or `false` to not replace
   * @param [options] Configuration.
   */
  const findAndReplace =
    /**
     * @type {(
     *   ((tree: Node, find: Find, replace?: Replace, options?: Options) => Node) &
     *   ((tree: Node, schema: FindAndReplaceSchema|FindAndReplaceList, options?: Options) => Node)
     * )}
     **/
    (
      /**
       * @param {Node} tree
       * @param {Find|FindAndReplaceSchema|FindAndReplaceList} find
       * @param {Replace|Options} [replace]
       * @param {Options} [options]
       */
      function (tree, find, replace, options) {
        /** @type {Options|undefined} */
        let settings;
        /** @type {FindAndReplaceSchema|FindAndReplaceList} */
        let schema;

        if (typeof find === 'string' || find instanceof RegExp) {
          // @ts-expect-error don’t expect options twice.
          schema = [[find, replace]];
          settings = options;
        } else {
          schema = find;
          // @ts-expect-error don’t expect replace twice.
          settings = replace;
        }

        if (!settings) {
          settings = {};
        }

        const ignored = convert$1(settings.ignore || []);
        const pairs = toPairs(schema);
        let pairIndex = -1;

        while (++pairIndex < pairs.length) {
          visitParents$3(tree, 'text', visitor);
        }

        return tree

        /** @type {import('unist-util-visit-parents').Visitor<Text>} */
        function visitor(node, parents) {
          let index = -1;
          /** @type {Parent|undefined} */
          let grandparent;

          while (++index < parents.length) {
            const parent = /** @type {Parent} */ (parents[index]);

            if (
              ignored(
                parent,
                // @ts-expect-error mdast vs. unist parent.
                grandparent ? grandparent.children.indexOf(parent) : undefined,
                grandparent
              )
            ) {
              return
            }

            grandparent = parent;
          }

          if (grandparent) {
            return handler(node, grandparent)
          }
        }

        /**
         * @param {Text} node
         * @param {Parent} parent
         * @returns {VisitorResult}
         */
        function handler(node, parent) {
          const find = pairs[pairIndex][0];
          const replace = pairs[pairIndex][1];
          let start = 0;
          // @ts-expect-error: TS is wrong, some of these children can be text.
          let index = parent.children.indexOf(node);
          /** @type {Array.<PhrasingContent>} */
          let nodes = [];
          /** @type {number|undefined} */
          let position;

          find.lastIndex = 0;

          let match = find.exec(node.value);

          while (match) {
            position = match.index;
            // @ts-expect-error this is perfectly fine, typescript.
            let value = replace(...match, {
              index: match.index,
              input: match.input
            });

            if (typeof value === 'string') {
              value = value.length > 0 ? {type: 'text', value} : undefined;
            }

            if (value !== false) {
              if (start !== position) {
                nodes.push({
                  type: 'text',
                  value: node.value.slice(start, position)
                });
              }

              if (Array.isArray(value)) {
                nodes.push(...value);
              } else if (value) {
                nodes.push(value);
              }

              start = position + match[0].length;
            }

            if (!find.global) {
              break
            }

            match = find.exec(node.value);
          }

          if (position === undefined) {
            nodes = [node];
            index--;
          } else {
            if (start < node.value.length) {
              nodes.push({type: 'text', value: node.value.slice(start)});
            }

            parent.children.splice(index, 1, ...nodes);
          }

          return index + nodes.length + 1
        }
      }
    );

  /**
   * @param {FindAndReplaceSchema|FindAndReplaceList} schema
   * @returns {Pairs}
   */
  function toPairs(schema) {
    /** @type {Pairs} */
    const result = [];

    if (typeof schema !== 'object') {
      throw new TypeError('Expected array or object as schema')
    }

    if (Array.isArray(schema)) {
      let index = -1;

      while (++index < schema.length) {
        result.push([
          toExpression(schema[index][0]),
          toFunction(schema[index][1])
        ]);
      }
    } else {
      /** @type {string} */
      let key;

      for (key in schema) {
        if (own$r.call(schema, key)) {
          result.push([toExpression(key), toFunction(schema[key])]);
        }
      }
    }

    return result
  }

  /**
   * @param {Find} find
   * @returns {RegExp}
   */
  function toExpression(find) {
    return typeof find === 'string' ? new RegExp(escapeStringRegexp(find), 'g') : find
  }

  /**
   * @param {Replace} replace
   * @returns {ReplaceFunction}
   */
  function toFunction(replace) {
    return typeof replace === 'function' ? replace : () => replace
  }

  /**
   * @typedef {import('mdast').Link} Link
   * @typedef {import('mdast-util-from-markdown').Extension} FromMarkdownExtension
   * @typedef {import('mdast-util-from-markdown').Transform} FromMarkdownTransform
   * @typedef {import('mdast-util-from-markdown').Handle} FromMarkdownHandle
   * @typedef {import('mdast-util-to-markdown/lib/types.js').Options} ToMarkdownExtension
   * @typedef {import('mdast-util-find-and-replace').ReplaceFunction} ReplaceFunction
   * @typedef {import('mdast-util-find-and-replace').RegExpMatchObject} RegExpMatchObject
   * @typedef {import('mdast-util-find-and-replace').PhrasingContent} PhrasingContent
   */

  const inConstruct = 'phrasing';
  const notInConstruct = ['autolink', 'link', 'image', 'label'];

  /** @type {FromMarkdownExtension} */
  const gfmAutolinkLiteralFromMarkdown = {
    transforms: [transformGfmAutolinkLiterals],
    enter: {
      literalAutolink: enterLiteralAutolink,
      literalAutolinkEmail: enterLiteralAutolinkValue,
      literalAutolinkHttp: enterLiteralAutolinkValue,
      literalAutolinkWww: enterLiteralAutolinkValue
    },
    exit: {
      literalAutolink: exitLiteralAutolink,
      literalAutolinkEmail: exitLiteralAutolinkEmail,
      literalAutolinkHttp: exitLiteralAutolinkHttp,
      literalAutolinkWww: exitLiteralAutolinkWww
    }
  };

  /** @type {ToMarkdownExtension} */
  const gfmAutolinkLiteralToMarkdown = {
    unsafe: [
      {
        character: '@',
        before: '[+\\-.\\w]',
        after: '[\\-.\\w]',
        inConstruct,
        notInConstruct
      },
      {
        character: '.',
        before: '[Ww]',
        after: '[\\-.\\w]',
        inConstruct,
        notInConstruct
      },
      {character: ':', before: '[ps]', after: '\\/', inConstruct, notInConstruct}
    ]
  };

  /** @type {FromMarkdownHandle} */
  function enterLiteralAutolink(token) {
    this.enter({type: 'link', title: null, url: '', children: []}, token);
  }

  /** @type {FromMarkdownHandle} */
  function enterLiteralAutolinkValue(token) {
    this.config.enter.autolinkProtocol.call(this, token);
  }

  /** @type {FromMarkdownHandle} */
  function exitLiteralAutolinkHttp(token) {
    this.config.exit.autolinkProtocol.call(this, token);
  }

  /** @type {FromMarkdownHandle} */
  function exitLiteralAutolinkWww(token) {
    this.config.exit.data.call(this, token);
    const node = /** @type {Link} */ (this.stack[this.stack.length - 1]);
    node.url = 'http://' + this.sliceSerialize(token);
  }

  /** @type {FromMarkdownHandle} */
  function exitLiteralAutolinkEmail(token) {
    this.config.exit.autolinkEmail.call(this, token);
  }

  /** @type {FromMarkdownHandle} */
  function exitLiteralAutolink(token) {
    this.exit(token);
  }

  /** @type {FromMarkdownTransform} */
  function transformGfmAutolinkLiterals(tree) {
    findAndReplace(
      tree,
      [
        [/(https?:\/\/|www(?=\.))([-.\w]+)([^ \t\r\n]*)/gi, findUrl],
        [/([-.\w+]+)@([-\w]+(?:\.[-\w]+)+)/g, findEmail]
      ],
      {ignore: ['link', 'linkReference']}
    );
  }

  /**
   * @type {ReplaceFunction}
   * @param {string} _
   * @param {string} protocol
   * @param {string} domain
   * @param {string} path
   * @param {RegExpMatchObject} match
   */
  // eslint-disable-next-line max-params
  function findUrl(_, protocol, domain, path, match) {
    let prefix = '';

    // Not an expected previous character.
    if (!previous(match)) {
      return false
    }

    // Treat `www` as part of the domain.
    if (/^w/i.test(protocol)) {
      domain = protocol + domain;
      protocol = '';
      prefix = 'http://';
    }

    if (!isCorrectDomain(domain)) {
      return false
    }

    const parts = splitUrl(domain + path);

    if (!parts[0]) return false

    /** @type {PhrasingContent} */
    const result = {
      type: 'link',
      title: null,
      url: prefix + protocol + parts[0],
      children: [{type: 'text', value: protocol + parts[0]}]
    };

    if (parts[1]) {
      return [result, {type: 'text', value: parts[1]}]
    }

    return result
  }

  /**
   * @type {ReplaceFunction}
   * @param {string} _
   * @param {string} atext
   * @param {string} label
   * @param {RegExpMatchObject} match
   */
  function findEmail(_, atext, label, match) {
    if (
      // Not an expected previous character.
      !previous(match, true) ||
      // Label ends in not allowed character.
      /[_-\d]$/.test(label)
    ) {
      return false
    }

    return {
      type: 'link',
      title: null,
      url: 'mailto:' + atext + '@' + label,
      children: [{type: 'text', value: atext + '@' + label}]
    }
  }

  /**
   * @param {string} domain
   * @returns {boolean}
   */
  function isCorrectDomain(domain) {
    const parts = domain.split('.');

    if (
      parts.length < 2 ||
      (parts[parts.length - 1] &&
        (/_/.test(parts[parts.length - 1]) ||
          !/[a-zA-Z\d]/.test(parts[parts.length - 1]))) ||
      (parts[parts.length - 2] &&
        (/_/.test(parts[parts.length - 2]) ||
          !/[a-zA-Z\d]/.test(parts[parts.length - 2])))
    ) {
      return false
    }

    return true
  }

  /**
   * @param {string} url
   * @returns {[string, string|undefined]}
   */
  function splitUrl(url) {
    const trailExec = /[!"&'),.:;<>?\]}]+$/.exec(url);
    /** @type {number} */
    let closingParenIndex;
    /** @type {number} */
    let openingParens;
    /** @type {number} */
    let closingParens;
    /** @type {string|undefined} */
    let trail;

    if (trailExec) {
      url = url.slice(0, trailExec.index);
      trail = trailExec[0];
      closingParenIndex = trail.indexOf(')');
      openingParens = ccount$1(url, '(');
      closingParens = ccount$1(url, ')');

      while (closingParenIndex !== -1 && openingParens > closingParens) {
        url += trail.slice(0, closingParenIndex + 1);
        trail = trail.slice(closingParenIndex + 1);
        closingParenIndex = trail.indexOf(')');
        closingParens++;
      }
    }

    return [url, trail]
  }

  /**
   * @param {RegExpMatchObject} match
   * @param {boolean} [email=false]
   * @returns {boolean}
   */
  function previous(match, email) {
    const code = match.input.charCodeAt(match.index - 1);

    return (
      (match.index === 0 ||
        unicodeWhitespace(code) ||
        unicodePunctuation(code)) &&
      (!email || code !== 47)
    )
  }

  /**
   * Turn the number (in string form as either hexa- or plain decimal) coming from
   * a numeric character reference into a character.
   *
   * @param {string} value
   *   Value to decode.
   * @param {number} base
   *   Numeric base.
   * @returns {string}
   */
  function decodeNumericCharacterReference(value, base) {
    const code = Number.parseInt(value, base);

    if (
      // C0 except for HT, LF, FF, CR, space
      code < 9 ||
      code === 11 ||
      (code > 13 && code < 32) || // Control character (DEL) of the basic block and C1 controls.
      (code > 126 && code < 160) || // Lone high surrogates and low surrogates.
      (code > 55295 && code < 57344) || // Noncharacters.
      (code > 64975 && code < 65008) ||
      (code & 65535) === 65535 ||
      (code & 65535) === 65534 || // Out of range
      code > 1114111
    ) {
      return '\uFFFD'
    }

    return String.fromCharCode(code)
  }

  const characterEscapeOrReference =
    /\\([!-/:-@[-`{-~])|&(#(?:\d{1,7}|x[\da-f]{1,6})|[\da-z]{1,31});/gi;
  /**
   * Utility to decode markdown strings (which occur in places such as fenced
   * code info strings, destinations, labels, and titles).
   * The “string” content type allows character escapes and -references.
   * This decodes those.
   *
   * @param {string} value
   * @returns {string}
   */

  function decodeString(value) {
    return value.replace(characterEscapeOrReference, decode)
  }
  /**
   * @param {string} $0
   * @param {string} $1
   * @param {string} $2
   * @returns {string}
   */

  function decode($0, $1, $2) {
    if ($1) {
      // Escape.
      return $1
    } // Reference.

    const head = $2.charCodeAt(0);

    if (head === 35) {
      const head = $2.charCodeAt(1);
      const hex = head === 120 || head === 88;
      return decodeNumericCharacterReference($2.slice(hex ? 2 : 1), hex ? 16 : 10)
    }

    return decodeNamedCharacterReference($2) || $0
  }

  /**
   * @typedef {import('mdast').Association} Association
   */

  /**
   * The `label` of an association is the string value: character escapes and
   * references work, and casing is intact.
   * The `identifier` is used to match one association to another: controversially,
   * character escapes and references don’t work in this matching: `&copy;` does
   * not match `©`, and `\+` does not match `+`.
   * But casing is ignored (and whitespace) is trimmed and collapsed: ` A\nb`
   * matches `a b`.
   * So, we do prefer the label when figuring out how we’re going to serialize:
   * it has whitespace, casing, and we can ignore most useless character escapes
   * and all character references.
   *
   * @param {Association} node
   * @returns {string}
   */
  function association(node) {
    if (node.label || !node.identifier) {
      return node.label || ''
    }

    return decodeString(node.identifier)
  }

  /**
   * @typedef {import('unist').Point} Point
   * @typedef {import('../types.js').TrackFields} TrackFields
   */

  /**
   * Functions to track output positions.
   * This info isn’t used yet but suchs functionality allows line wrapping,
   * and theoretically source maps (though, is there practical use in that?).
   *
   * @param {TrackFields} options_
   */
  function track(options_) {
    // Defaults are used to prevent crashes when older utilities somehow activate
    // this code.
    /* c8 ignore next 5 */
    const options = options_ || {};
    const now = options.now || {};
    let lineShift = options.lineShift || 0;
    let line = now.line || 1;
    let column = now.column || 1;

    return {move, current, shift}

    /**
     * Get the current tracked info.
     *
     * @returns {{now: Point, lineShift: number}}
     */
    function current() {
      return {now: {line, column}, lineShift}
    }

    /**
     * Define an increased line shift (the typical indent for lines).
     *
     * @param {number} value
     */
    function shift(value) {
      lineShift += value;
    }

    /**
     * Move past a string.
     *
     * @param {string} value
     * @returns {string}
     */
    function move(value = '') {
      const chunks = value.split(/\r?\n|\r/g);
      const tail = chunks[chunks.length - 1];
      line += chunks.length - 1;
      column =
        chunks.length === 1 ? column + tail.length : 1 + tail.length + lineShift;
      return value
    }
  }

  /**
   * @typedef {import('../types.js').Node} Node
   * @typedef {import('../types.js').Parent} Parent
   * @typedef {import('../types.js').Join} Join
   * @typedef {import('../types.js').Context} Context
   * @typedef {import('../types.js').TrackFields} TrackFields
   */

  /**
   * @param {Parent} parent
   * @param {Context} context
   * @param {TrackFields} safeOptions
   * @returns {string}
   */
  function containerFlow(parent, context, safeOptions) {
    const indexStack = context.indexStack;
    const children = parent.children || [];
    const tracker = track(safeOptions);
    /** @type {Array<string>} */
    const results = [];
    let index = -1;

    indexStack.push(-1);

    while (++index < children.length) {
      const child = children[index];

      indexStack[indexStack.length - 1] = index;

      results.push(
        tracker.move(
          context.handle(child, parent, context, {
            before: '\n',
            after: '\n',
            ...tracker.current()
          })
        )
      );

      if (child.type !== 'list') {
        context.bulletLastUsed = undefined;
      }

      if (index < children.length - 1) {
        results.push(tracker.move(between(child, children[index + 1])));
      }
    }

    indexStack.pop();

    return results.join('')

    /**
     * @param {Node} left
     * @param {Node} right
     * @returns {string}
     */
    function between(left, right) {
      let index = context.join.length;

      while (index--) {
        const result = context.join[index](left, right, parent, context);

        if (result === true || result === 1) {
          break
        }

        if (typeof result === 'number') {
          return '\n'.repeat(1 + result)
        }

        if (result === false) {
          return '\n\n<!---->\n\n'
        }
      }

      return '\n\n'
    }
  }

  /**
   * @callback Map
   * @param {string} value
   * @param {number} line
   * @param {boolean} blank
   * @returns {string}
   */

  const eol = /\r?\n|\r/g;

  /**
   * @param {string} value
   * @param {Map} map
   * @returns {string}
   */
  function indentLines(value, map) {
    /** @type {Array<string>} */
    const result = [];
    let start = 0;
    let line = 0;
    /** @type {RegExpExecArray|null} */
    let match;

    while ((match = eol.exec(value))) {
      one(value.slice(start, match.index));
      result.push(match[0]);
      start = match.index + match[0].length;
      line++;
    }

    one(value.slice(start));

    return result.join('')

    /**
     * @param {string} value
     */
    function one(value) {
      result.push(map(value, line, !value));
    }
  }

  /**
   * @typedef {import('../types.js').Unsafe} Unsafe
   */

  /**
   * @param {Unsafe} pattern
   * @returns {RegExp}
   */
  function patternCompile(pattern) {
    if (!pattern._compiled) {
      const before =
        (pattern.atBreak ? '[\\r\\n][\\t ]*' : '') +
        (pattern.before ? '(?:' + pattern.before + ')' : '');

      pattern._compiled = new RegExp(
        (before ? '(' + before + ')' : '') +
          (/[|\\{}()[\]^$+*?.-]/.test(pattern.character) ? '\\' : '') +
          pattern.character +
          (pattern.after ? '(?:' + pattern.after + ')' : ''),
        'g'
      );
    }

    return pattern._compiled
  }

  /**
   * @typedef {import('../types.js').Unsafe} Unsafe
   */

  /**
   * @param {Array<string>} stack
   * @param {Unsafe} pattern
   * @returns {boolean}
   */
  function patternInScope(stack, pattern) {
    return (
      listInScope(stack, pattern.inConstruct, true) &&
      !listInScope(stack, pattern.notInConstruct, false)
    )
  }

  /**
   * @param {Array<string>} stack
   * @param {Unsafe['inConstruct']} list
   * @param {boolean} none
   * @returns {boolean}
   */
  function listInScope(stack, list, none) {
    if (!list) {
      return none
    }

    if (typeof list === 'string') {
      list = [list];
    }

    let index = -1;

    while (++index < list.length) {
      if (stack.includes(list[index])) {
        return true
      }
    }

    return false
  }

  /**
   * @typedef {import('../types.js').Context} Context
   * @typedef {import('../types.js').SafeOptions} SafeOptions
   */

  /**
   * @param {Context} context
   * @param {string|null|undefined} input
   * @param {SafeOptions & {encode?: Array<string>}} config
   * @returns {string}
   */
  function safe(context, input, config) {
    const value = (config.before || '') + (input || '') + (config.after || '');
    /** @type {Array<number>} */
    const positions = [];
    /** @type {Array<string>} */
    const result = [];
    /** @type {Record<number, {before: boolean, after: boolean}>} */
    const infos = {};
    let index = -1;

    while (++index < context.unsafe.length) {
      const pattern = context.unsafe[index];

      if (!patternInScope(context.stack, pattern)) {
        continue
      }

      const expression = patternCompile(pattern);
      /** @type {RegExpExecArray|null} */
      let match;

      while ((match = expression.exec(value))) {
        const before = 'before' in pattern || Boolean(pattern.atBreak);
        const after = 'after' in pattern;
        const position = match.index + (before ? match[1].length : 0);

        if (positions.includes(position)) {
          if (infos[position].before && !before) {
            infos[position].before = false;
          }

          if (infos[position].after && !after) {
            infos[position].after = false;
          }
        } else {
          positions.push(position);
          infos[position] = {before, after};
        }
      }
    }

    positions.sort(numerical);

    let start = config.before ? config.before.length : 0;
    const end = value.length - (config.after ? config.after.length : 0);
    index = -1;

    while (++index < positions.length) {
      const position = positions[index];

      // Character before or after matched:
      if (position < start || position >= end) {
        continue
      }

      // If this character is supposed to be escaped because it has a condition on
      // the next character, and the next character is definitly being escaped,
      // then skip this escape.
      if (
        (position + 1 < end &&
          positions[index + 1] === position + 1 &&
          infos[position].after &&
          !infos[position + 1].before &&
          !infos[position + 1].after) ||
        (positions[index - 1] === position - 1 &&
          infos[position].before &&
          !infos[position - 1].before &&
          !infos[position - 1].after)
      ) {
        continue
      }

      if (start !== position) {
        // If we have to use a character reference, an ampersand would be more
        // correct, but as backslashes only care about punctuation, either will
        // do the trick
        result.push(escapeBackslashes(value.slice(start, position), '\\'));
      }

      start = position;

      if (
        /[!-/:-@[-`{-~]/.test(value.charAt(position)) &&
        (!config.encode || !config.encode.includes(value.charAt(position)))
      ) {
        // Character escape.
        result.push('\\');
      } else {
        // Character reference.
        result.push(
          '&#x' + value.charCodeAt(position).toString(16).toUpperCase() + ';'
        );
        start++;
      }
    }

    result.push(escapeBackslashes(value.slice(start, end), config.after));

    return result.join('')
  }

  /**
   * @param {number} a
   * @param {number} b
   * @returns {number}
   */
  function numerical(a, b) {
    return a - b
  }

  /**
   * @param {string} value
   * @param {string} after
   * @returns {string}
   */
  function escapeBackslashes(value, after) {
    const expression = /\\(?=[!-/:-@[-`{-~])/g;
    /** @type {Array<number>} */
    const positions = [];
    /** @type {Array<string>} */
    const results = [];
    const whole = value + after;
    let index = -1;
    let start = 0;
    /** @type {RegExpExecArray|null} */
    let match;

    while ((match = expression.exec(whole))) {
      positions.push(match.index);
    }

    while (++index < positions.length) {
      if (start !== positions[index]) {
        results.push(value.slice(start, positions[index]));
      }

      results.push('\\');
      start = positions[index];
    }

    results.push(value.slice(start));

    return results.join('')
  }

  /**
   * @typedef {import('mdast').FootnoteReference} FootnoteReference
   * @typedef {import('mdast').FootnoteDefinition} FootnoteDefinition
   * @typedef {import('mdast-util-from-markdown').Extension} FromMarkdownExtension
   * @typedef {import('mdast-util-from-markdown').Handle} FromMarkdownHandle
   * @typedef {import('mdast-util-to-markdown').Options} ToMarkdownExtension
   * @typedef {import('mdast-util-to-markdown').Handle} ToMarkdownHandle
   * @typedef {import('mdast-util-to-markdown').Map} Map
   */

  /**
   * @returns {FromMarkdownExtension}
   */
  function gfmFootnoteFromMarkdown() {
    return {
      enter: {
        gfmFootnoteDefinition: enterFootnoteDefinition,
        gfmFootnoteDefinitionLabelString: enterFootnoteDefinitionLabelString,
        gfmFootnoteCall: enterFootnoteCall,
        gfmFootnoteCallString: enterFootnoteCallString
      },
      exit: {
        gfmFootnoteDefinition: exitFootnoteDefinition,
        gfmFootnoteDefinitionLabelString: exitFootnoteDefinitionLabelString,
        gfmFootnoteCall: exitFootnoteCall,
        gfmFootnoteCallString: exitFootnoteCallString
      }
    }

    /** @type {FromMarkdownHandle} */
    function enterFootnoteDefinition(token) {
      this.enter(
        {type: 'footnoteDefinition', identifier: '', label: '', children: []},
        token
      );
    }

    /** @type {FromMarkdownHandle} */
    function enterFootnoteDefinitionLabelString() {
      this.buffer();
    }

    /** @type {FromMarkdownHandle} */
    function exitFootnoteDefinitionLabelString(token) {
      const label = this.resume();
      const node = /** @type {FootnoteDefinition} */ (
        this.stack[this.stack.length - 1]
      );
      node.label = label;
      node.identifier = normalizeIdentifier(
        this.sliceSerialize(token)
      ).toLowerCase();
    }

    /** @type {FromMarkdownHandle} */
    function exitFootnoteDefinition(token) {
      this.exit(token);
    }

    /** @type {FromMarkdownHandle} */
    function enterFootnoteCall(token) {
      this.enter({type: 'footnoteReference', identifier: '', label: ''}, token);
    }

    /** @type {FromMarkdownHandle} */
    function enterFootnoteCallString() {
      this.buffer();
    }

    /** @type {FromMarkdownHandle} */
    function exitFootnoteCallString(token) {
      const label = this.resume();
      const node = /** @type {FootnoteDefinition} */ (
        this.stack[this.stack.length - 1]
      );
      node.label = label;
      node.identifier = normalizeIdentifier(
        this.sliceSerialize(token)
      ).toLowerCase();
    }

    /** @type {FromMarkdownHandle} */
    function exitFootnoteCall(token) {
      this.exit(token);
    }
  }

  /**
   * @returns {ToMarkdownExtension}
   */
  function gfmFootnoteToMarkdown() {
    footnoteReference.peek = footnoteReferencePeek;

    return {
      // This is on by default already.
      unsafe: [{character: '[', inConstruct: ['phrasing', 'label', 'reference']}],
      handlers: {footnoteDefinition, footnoteReference}
    }

    /**
     * @type {ToMarkdownHandle}
     * @param {FootnoteReference} node
     */
    function footnoteReference(node, _, context, safeOptions) {
      const tracker = track(safeOptions);
      let value = tracker.move('[^');
      const exit = context.enter('footnoteReference');
      const subexit = context.enter('reference');
      value += tracker.move(
        safe(context, association(node), {
          ...tracker.current(),
          before: value,
          after: ']'
        })
      );
      subexit();
      exit();
      value += tracker.move(']');
      return value
    }

    /** @type {ToMarkdownHandle} */
    function footnoteReferencePeek() {
      return '['
    }

    /**
     * @type {ToMarkdownHandle}
     * @param {FootnoteDefinition} node
     */
    function footnoteDefinition(node, _, context, safeOptions) {
      const tracker = track(safeOptions);
      let value = tracker.move('[^');
      const exit = context.enter('footnoteDefinition');
      const subexit = context.enter('label');
      value += tracker.move(
        safe(context, association(node), {
          ...tracker.current(),
          before: value,
          after: ']'
        })
      );
      subexit();
      value += tracker.move(
        ']:' + (node.children && node.children.length > 0 ? ' ' : '')
      );
      tracker.shift(4);
      value += tracker.move(
        indentLines(containerFlow(node, context, tracker.current()), map)
      );
      exit();

      return value

      /** @type {Map} */
      function map(line, index, blank) {
        if (index) {
          return (blank ? '' : '    ') + line
        }

        return line
      }
    }
  }

  /**
   * @typedef {import('../types.js').Node} Node
   * @typedef {import('../types.js').Parent} Parent
   * @typedef {import('../types.js').SafeOptions} SafeOptions
   * @typedef {import('../types.js').Context} Context
   */

  /**
   * @param {Parent} parent
   * @param {Context} context
   * @param {SafeOptions} safeOptions
   * @returns {string}
   */
  function containerPhrasing(parent, context, safeOptions) {
    const indexStack = context.indexStack;
    const children = parent.children || [];
    /** @type {Array<string>} */
    const results = [];
    let index = -1;
    let before = safeOptions.before;

    indexStack.push(-1);
    let tracker = track(safeOptions);

    while (++index < children.length) {
      const child = children[index];
      /** @type {string} */
      let after;

      indexStack[indexStack.length - 1] = index;

      if (index + 1 < children.length) {
        // @ts-expect-error: hush, it’s actually a `zwitch`.
        let handle = context.handle.handlers[children[index + 1].type];
        if (handle && handle.peek) handle = handle.peek;
        after = handle
          ? handle(children[index + 1], parent, context, {
              before: '',
              after: '',
              ...tracker.current()
            }).charAt(0)
          : '';
      } else {
        after = safeOptions.after;
      }

      // In some cases, html (text) can be found in phrasing right after an eol.
      // When we’d serialize that, in most cases that would be seen as html
      // (flow).
      // As we can’t escape or so to prevent it from happening, we take a somewhat
      // reasonable approach: replace that eol with a space.
      // See: <https://github.com/syntax-tree/mdast-util-to-markdown/issues/15>
      if (
        results.length > 0 &&
        (before === '\r' || before === '\n') &&
        child.type === 'html'
      ) {
        results[results.length - 1] = results[results.length - 1].replace(
          /(\r?\n|\r)$/,
          ' '
        );
        before = ' ';

        // To do: does this work to reset tracker?
        tracker = track(safeOptions);
        tracker.move(results.join(''));
      }

      results.push(
        tracker.move(
          context.handle(child, parent, context, {
            ...tracker.current(),
            before,
            after
          })
        )
      );

      before = results[results.length - 1].slice(-1);
    }

    indexStack.pop();

    return results.join('')
  }

  /**
   * @typedef {import('mdast').Delete} Delete
   * @typedef {import('mdast-util-from-markdown').Extension} FromMarkdownExtension
   * @typedef {import('mdast-util-from-markdown').Handle} FromMarkdownHandle
   * @typedef {import('mdast-util-to-markdown').Options} ToMarkdownExtension
   * @typedef {import('mdast-util-to-markdown').Handle} ToMarkdownHandle
   */

  /** @type {FromMarkdownExtension} */
  const gfmStrikethroughFromMarkdown = {
    canContainEols: ['delete'],
    enter: {strikethrough: enterStrikethrough},
    exit: {strikethrough: exitStrikethrough}
  };

  /** @type {ToMarkdownExtension} */
  const gfmStrikethroughToMarkdown = {
    unsafe: [{character: '~', inConstruct: 'phrasing'}],
    handlers: {delete: handleDelete}
  };

  handleDelete.peek = peekDelete;

  /** @type {FromMarkdownHandle} */
  function enterStrikethrough(token) {
    this.enter({type: 'delete', children: []}, token);
  }

  /** @type {FromMarkdownHandle} */
  function exitStrikethrough(token) {
    this.exit(token);
  }

  /**
   * @type {ToMarkdownHandle}
   * @param {Delete} node
   */
  function handleDelete(node, _, context, safeOptions) {
    const tracker = track(safeOptions);
    const exit = context.enter('emphasis');
    let value = tracker.move('~~');
    value += containerPhrasing(node, context, {
      ...tracker.current(),
      before: value,
      after: '~'
    });
    value += tracker.move('~~');
    exit();
    return value
  }

  /** @type {ToMarkdownHandle} */
  function peekDelete() {
    return '~'
  }

  /**
   * @typedef {import('mdast').InlineCode} InlineCode
   * @typedef {import('../types.js').Handle} Handle
   */

  inlineCode$1.peek = inlineCodePeek;

  /**
   * @type {Handle}
   * @param {InlineCode} node
   */
  function inlineCode$1(node, _, context) {
    let value = node.value || '';
    let sequence = '`';
    let index = -1;

    // If there is a single grave accent on its own in the code, use a fence of
    // two.
    // If there are two in a row, use one.
    while (new RegExp('(^|[^`])' + sequence + '([^`]|$)').test(value)) {
      sequence += '`';
    }

    // If this is not just spaces or eols (tabs don’t count), and either the
    // first or last character are a space, eol, or tick, then pad with spaces.
    if (
      /[^ \r\n]/.test(value) &&
      ((/^[ \r\n]/.test(value) && /[ \r\n]$/.test(value)) || /^`|`$/.test(value))
    ) {
      value = ' ' + value + ' ';
    }

    // We have a potential problem: certain characters after eols could result in
    // blocks being seen.
    // For example, if someone injected the string `'\n# b'`, then that would
    // result in an ATX heading.
    // We can’t escape characters in `inlineCode`, but because eols are
    // transformed to spaces when going from markdown to HTML anyway, we can swap
    // them out.
    while (++index < context.unsafe.length) {
      const pattern = context.unsafe[index];
      const expression = patternCompile(pattern);
      /** @type {RegExpExecArray|null} */
      let match;

      // Only look for `atBreak`s.
      // Btw: note that `atBreak` patterns will always start the regex at LF or
      // CR.
      if (!pattern.atBreak) continue

      while ((match = expression.exec(value))) {
        let position = match.index;

        // Support CRLF (patterns only look for one of the characters).
        if (
          value.charCodeAt(position) === 10 /* `\n` */ &&
          value.charCodeAt(position - 1) === 13 /* `\r` */
        ) {
          position--;
        }

        value = value.slice(0, position) + ' ' + value.slice(match.index + 1);
      }
    }

    return sequence + value + sequence
  }

  /**
   * @type {Handle}
   */
  function inlineCodePeek() {
    return '`'
  }

  /**
   * @typedef Options
   *   Configuration (optional).
   * @property {string|null|Array<string|null|undefined>} [align]
   *   One style for all columns, or styles for their respective columns.
   *   Each style is either `'l'` (left), `'r'` (right), or `'c'` (center).
   *   Other values are treated as `''`, which doesn’t place the colon in the
   *   alignment row but does align left.
   *   *Only the lowercased first character is used, so `Right` is fine.*
   * @property {boolean} [padding=true]
   *   Whether to add a space of padding between delimiters and cells.
   *
   *   When `true`, there is padding:
   *
   *   ```markdown
   *   | Alpha | B     |
   *   | ----- | ----- |
   *   | C     | Delta |
   *   ```
   *
   *   When `false`, there is no padding:
   *
   *   ```markdown
   *   |Alpha|B    |
   *   |-----|-----|
   *   |C    |Delta|
   *   ```
   * @property {boolean} [delimiterStart=true]
   *   Whether to begin each row with the delimiter.
   *
   *   > 👉 **Note**: please don’t use this: it could create fragile structures
   *   > that aren’t understandable to some markdown parsers.
   *
   *   When `true`, there are starting delimiters:
   *
   *   ```markdown
   *   | Alpha | B     |
   *   | ----- | ----- |
   *   | C     | Delta |
   *   ```
   *
   *   When `false`, there are no starting delimiters:
   *
   *   ```markdown
   *   Alpha | B     |
   *   ----- | ----- |
   *   C     | Delta |
   *   ```
   * @property {boolean} [delimiterEnd=true]
   *   Whether to end each row with the delimiter.
   *
   *   > 👉 **Note**: please don’t use this: it could create fragile structures
   *   > that aren’t understandable to some markdown parsers.
   *
   *   When `true`, there are ending delimiters:
   *
   *   ```markdown
   *   | Alpha | B     |
   *   | ----- | ----- |
   *   | C     | Delta |
   *   ```
   *
   *   When `false`, there are no ending delimiters:
   *
   *   ```markdown
   *   | Alpha | B
   *   | ----- | -----
   *   | C     | Delta
   *   ```
   * @property {boolean} [alignDelimiters=true]
   *   Whether to align the delimiters.
   *   By default, they are aligned:
   *
   *   ```markdown
   *   | Alpha | B     |
   *   | ----- | ----- |
   *   | C     | Delta |
   *   ```
   *
   *   Pass `false` to make them staggered:
   *
   *   ```markdown
   *   | Alpha | B |
   *   | - | - |
   *   | C | Delta |
   *   ```
   * @property {(value: string) => number} [stringLength]
   *   Function to detect the length of table cell content.
   *   This is used when aligning the delimiters (`|`) between table cells.
   *   Full-width characters and emoji mess up delimiter alignment when viewing
   *   the markdown source.
   *   To fix this, you can pass this function, which receives the cell content
   *   and returns its “visible” size.
   *   Note that what is and isn’t visible depends on where the text is displayed.
   *
   *   Without such a function, the following:
   *
   *   ```js
   *   markdownTable([
   *     ['Alpha', 'Bravo'],
   *     ['中文', 'Charlie'],
   *     ['👩‍❤️‍👩', 'Delta']
   *   ])
   *   ```
   *
   *   Yields:
   *
   *   ```markdown
   *   | Alpha | Bravo |
   *   | - | - |
   *   | 中文 | Charlie |
   *   | 👩‍❤️‍👩 | Delta |
   *   ```
   *
   *   With [`string-width`](https://github.com/sindresorhus/string-width):
   *
   *   ```js
   *   import stringWidth from 'string-width'
   *
   *   markdownTable(
   *     [
   *       ['Alpha', 'Bravo'],
   *       ['中文', 'Charlie'],
   *       ['👩‍❤️‍👩', 'Delta']
   *     ],
   *     {stringLength: stringWidth}
   *   )
   *   ```
   *
   *   Yields:
   *
   *   ```markdown
   *   | Alpha | Bravo   |
   *   | ----- | ------- |
   *   | 中文  | Charlie |
   *   | 👩‍❤️‍👩    | Delta   |
   *   ```
   */

  /**
   * @typedef {Options} MarkdownTableOptions
   * @todo
   *   Remove next major.
   */

  /**
   * Generate a markdown ([GFM](https://docs.github.com/en/github/writing-on-github/working-with-advanced-formatting/organizing-information-with-tables)) table..
   *
   * @param {Array<Array<string|null|undefined>>} table
   *   Table data (matrix of strings).
   * @param {Options} [options]
   *   Configuration (optional).
   * @returns {string}
   */
  function markdownTable(table, options = {}) {
    const align = (options.align || []).concat();
    const stringLength = options.stringLength || defaultStringLength;
    /** @type {Array<number>} Character codes as symbols for alignment per column. */
    const alignments = [];
    /** @type {Array<Array<string>>} Cells per row. */
    const cellMatrix = [];
    /** @type {Array<Array<number>>} Sizes of each cell per row. */
    const sizeMatrix = [];
    /** @type {Array<number>} */
    const longestCellByColumn = [];
    let mostCellsPerRow = 0;
    let rowIndex = -1;

    // This is a superfluous loop if we don’t align delimiters, but otherwise we’d
    // do superfluous work when aligning, so optimize for aligning.
    while (++rowIndex < table.length) {
      /** @type {Array<string>} */
      const row = [];
      /** @type {Array<number>} */
      const sizes = [];
      let columnIndex = -1;

      if (table[rowIndex].length > mostCellsPerRow) {
        mostCellsPerRow = table[rowIndex].length;
      }

      while (++columnIndex < table[rowIndex].length) {
        const cell = serialize(table[rowIndex][columnIndex]);

        if (options.alignDelimiters !== false) {
          const size = stringLength(cell);
          sizes[columnIndex] = size;

          if (
            longestCellByColumn[columnIndex] === undefined ||
            size > longestCellByColumn[columnIndex]
          ) {
            longestCellByColumn[columnIndex] = size;
          }
        }

        row.push(cell);
      }

      cellMatrix[rowIndex] = row;
      sizeMatrix[rowIndex] = sizes;
    }

    // Figure out which alignments to use.
    let columnIndex = -1;

    if (typeof align === 'object' && 'length' in align) {
      while (++columnIndex < mostCellsPerRow) {
        alignments[columnIndex] = toAlignment(align[columnIndex]);
      }
    } else {
      const code = toAlignment(align);

      while (++columnIndex < mostCellsPerRow) {
        alignments[columnIndex] = code;
      }
    }

    // Inject the alignment row.
    columnIndex = -1;
    /** @type {Array<string>} */
    const row = [];
    /** @type {Array<number>} */
    const sizes = [];

    while (++columnIndex < mostCellsPerRow) {
      const code = alignments[columnIndex];
      let before = '';
      let after = '';

      if (code === 99 /* `c` */) {
        before = ':';
        after = ':';
      } else if (code === 108 /* `l` */) {
        before = ':';
      } else if (code === 114 /* `r` */) {
        after = ':';
      }

      // There *must* be at least one hyphen-minus in each alignment cell.
      let size =
        options.alignDelimiters === false
          ? 1
          : Math.max(
              1,
              longestCellByColumn[columnIndex] - before.length - after.length
            );

      const cell = before + '-'.repeat(size) + after;

      if (options.alignDelimiters !== false) {
        size = before.length + size + after.length;

        if (size > longestCellByColumn[columnIndex]) {
          longestCellByColumn[columnIndex] = size;
        }

        sizes[columnIndex] = size;
      }

      row[columnIndex] = cell;
    }

    // Inject the alignment row.
    cellMatrix.splice(1, 0, row);
    sizeMatrix.splice(1, 0, sizes);

    rowIndex = -1;
    /** @type {Array<string>} */
    const lines = [];

    while (++rowIndex < cellMatrix.length) {
      const row = cellMatrix[rowIndex];
      const sizes = sizeMatrix[rowIndex];
      columnIndex = -1;
      /** @type {Array<string>} */
      const line = [];

      while (++columnIndex < mostCellsPerRow) {
        const cell = row[columnIndex] || '';
        let before = '';
        let after = '';

        if (options.alignDelimiters !== false) {
          const size =
            longestCellByColumn[columnIndex] - (sizes[columnIndex] || 0);
          const code = alignments[columnIndex];

          if (code === 114 /* `r` */) {
            before = ' '.repeat(size);
          } else if (code === 99 /* `c` */) {
            if (size % 2) {
              before = ' '.repeat(size / 2 + 0.5);
              after = ' '.repeat(size / 2 - 0.5);
            } else {
              before = ' '.repeat(size / 2);
              after = before;
            }
          } else {
            after = ' '.repeat(size);
          }
        }

        if (options.delimiterStart !== false && !columnIndex) {
          line.push('|');
        }

        if (
          options.padding !== false &&
          // Don’t add the opening space if we’re not aligning and the cell is
          // empty: there will be a closing space.
          !(options.alignDelimiters === false && cell === '') &&
          (options.delimiterStart !== false || columnIndex)
        ) {
          line.push(' ');
        }

        if (options.alignDelimiters !== false) {
          line.push(before);
        }

        line.push(cell);

        if (options.alignDelimiters !== false) {
          line.push(after);
        }

        if (options.padding !== false) {
          line.push(' ');
        }

        if (
          options.delimiterEnd !== false ||
          columnIndex !== mostCellsPerRow - 1
        ) {
          line.push('|');
        }
      }

      lines.push(
        options.delimiterEnd === false
          ? line.join('').replace(/ +$/, '')
          : line.join('')
      );
    }

    return lines.join('\n')
  }

  /**
   * @param {string|null|undefined} [value]
   * @returns {string}
   */
  function serialize(value) {
    return value === null || value === undefined ? '' : String(value)
  }

  /**
   * @param {string} value
   * @returns {number}
   */
  function defaultStringLength(value) {
    return value.length
  }

  /**
   * @param {string|null|undefined} value
   * @returns {number}
   */
  function toAlignment(value) {
    const code = typeof value === 'string' ? value.codePointAt(0) : 0;

    return code === 67 /* `C` */ || code === 99 /* `c` */
      ? 99 /* `c` */
      : code === 76 /* `L` */ || code === 108 /* `l` */
      ? 108 /* `l` */
      : code === 82 /* `R` */ || code === 114 /* `r` */
      ? 114 /* `r` */
      : 0
  }

  /**
   * @typedef {import('mdast').AlignType} AlignType
   * @typedef {import('mdast').Table} Table
   * @typedef {import('mdast').TableRow} TableRow
   * @typedef {import('mdast').TableCell} TableCell
   * @typedef {import('mdast').InlineCode} InlineCode
   * @typedef {import('markdown-table').MarkdownTableOptions} MarkdownTableOptions
   * @typedef {import('mdast-util-from-markdown').Extension} FromMarkdownExtension
   * @typedef {import('mdast-util-from-markdown').Handle} FromMarkdownHandle
   * @typedef {import('mdast-util-to-markdown').Options} ToMarkdownExtension
   * @typedef {import('mdast-util-to-markdown').Handle} ToMarkdownHandle
   * @typedef {import('mdast-util-to-markdown').Context} ToMarkdownContext
   * @typedef {import('mdast-util-to-markdown').SafeOptions} SafeOptions
   *
   * @typedef Options
   * @property {boolean} [tableCellPadding=true]
   * @property {boolean} [tablePipeAlign=true]
   * @property {MarkdownTableOptions['stringLength']} [stringLength]
   */

  /** @type {FromMarkdownExtension} */
  const gfmTableFromMarkdown = {
    enter: {
      table: enterTable,
      tableData: enterCell,
      tableHeader: enterCell,
      tableRow: enterRow
    },
    exit: {
      codeText: exitCodeText,
      table: exitTable,
      tableData: exit,
      tableHeader: exit,
      tableRow: exit
    }
  };

  /** @type {FromMarkdownHandle} */
  function enterTable(token) {
    /** @type {Array<'left'|'right'|'center'|'none'>} */
    // @ts-expect-error: `align` is custom.
    const align = token._align;
    this.enter(
      {
        type: 'table',
        align: align.map((d) => (d === 'none' ? null : d)),
        children: []
      },
      token
    );
    this.setData('inTable', true);
  }

  /** @type {FromMarkdownHandle} */
  function exitTable(token) {
    this.exit(token);
    this.setData('inTable');
  }

  /** @type {FromMarkdownHandle} */
  function enterRow(token) {
    this.enter({type: 'tableRow', children: []}, token);
  }

  /** @type {FromMarkdownHandle} */
  function exit(token) {
    this.exit(token);
  }

  /** @type {FromMarkdownHandle} */
  function enterCell(token) {
    this.enter({type: 'tableCell', children: []}, token);
  }

  // Overwrite the default code text data handler to unescape escaped pipes when
  // they are in tables.
  /** @type {FromMarkdownHandle} */
  function exitCodeText(token) {
    let value = this.resume();

    if (this.getData('inTable')) {
      value = value.replace(/\\([\\|])/g, replace);
    }

    const node = /** @type {InlineCode} */ (this.stack[this.stack.length - 1]);
    node.value = value;
    this.exit(token);
  }

  /**
   * @param {string} $0
   * @param {string} $1
   * @returns {string}
   */
  function replace($0, $1) {
    // Pipes work, backslashes don’t (but can’t escape pipes).
    return $1 === '|' ? $1 : $0
  }

  /**
   * @param {Options} [options]
   * @returns {ToMarkdownExtension}
   */
  function gfmTableToMarkdown(options) {
    const settings = options || {};
    const padding = settings.tableCellPadding;
    const alignDelimiters = settings.tablePipeAlign;
    const stringLength = settings.stringLength;
    const around = padding ? ' ' : '|';

    return {
      unsafe: [
        {character: '\r', inConstruct: 'tableCell'},
        {character: '\n', inConstruct: 'tableCell'},
        // A pipe, when followed by a tab or space (padding), or a dash or colon
        // (unpadded delimiter row), could result in a table.
        {atBreak: true, character: '|', after: '[\t :-]'},
        // A pipe in a cell must be encoded.
        {character: '|', inConstruct: 'tableCell'},
        // A colon must be followed by a dash, in which case it could start a
        // delimiter row.
        {atBreak: true, character: ':', after: '-'},
        // A delimiter row can also start with a dash, when followed by more
        // dashes, a colon, or a pipe.
        // This is a stricter version than the built in check for lists, thematic
        // breaks, and setex heading underlines though:
        // <https://github.com/syntax-tree/mdast-util-to-markdown/blob/51a2038/lib/unsafe.js#L57>
        {atBreak: true, character: '-', after: '[:|-]'}
      ],
      handlers: {
        table: handleTable,
        tableRow: handleTableRow,
        tableCell: handleTableCell,
        inlineCode: inlineCodeWithTable
      }
    }

    /**
     * @type {ToMarkdownHandle}
     * @param {Table} node
     */
    function handleTable(node, _, context, safeOptions) {
      return serializeData(
        handleTableAsData(node, context, safeOptions),
        // @ts-expect-error: fixed in `markdown-table@3.0.1`.
        node.align
      )
    }

    /**
     * This function isn’t really used normally, because we handle rows at the
     * table level.
     * But, if someone passes in a table row, this ensures we make somewhat sense.
     *
     * @type {ToMarkdownHandle}
     * @param {TableRow} node
     */
    function handleTableRow(node, _, context, safeOptions) {
      const row = handleTableRowAsData(node, context, safeOptions);
      // `markdown-table` will always add an align row
      const value = serializeData([row]);
      return value.slice(0, value.indexOf('\n'))
    }

    /**
     * @type {ToMarkdownHandle}
     * @param {TableCell} node
     */
    function handleTableCell(node, _, context, safeOptions) {
      const exit = context.enter('tableCell');
      const subexit = context.enter('phrasing');
      const value = containerPhrasing(node, context, {
        ...safeOptions,
        before: around,
        after: around
      });
      subexit();
      exit();
      return value
    }

    /**
     * @param {Array<Array<string>>} matrix
     * @param {Array<string>} [align]
     */
    function serializeData(matrix, align) {
      return markdownTable(matrix, {
        align,
        alignDelimiters,
        padding,
        stringLength
      })
    }

    /**
     * @param {Table} node
     * @param {ToMarkdownContext} context
     * @param {SafeOptions} safeOptions
     */
    function handleTableAsData(node, context, safeOptions) {
      const children = node.children;
      let index = -1;
      /** @type {Array<Array<string>>} */
      const result = [];
      const subexit = context.enter('table');

      while (++index < children.length) {
        result[index] = handleTableRowAsData(
          children[index],
          context,
          safeOptions
        );
      }

      subexit();

      return result
    }

    /**
     * @param {TableRow} node
     * @param {ToMarkdownContext} context
     * @param {SafeOptions} safeOptions
     */
    function handleTableRowAsData(node, context, safeOptions) {
      const children = node.children;
      let index = -1;
      /** @type {Array<string>} */
      const result = [];
      const subexit = context.enter('tableRow');

      while (++index < children.length) {
        // Note: the positional info as used here is incorrect.
        // Making it correct would be impossible due to aligning cells?
        // And it would need copy/pasting `markdown-table` into this project.
        result[index] = handleTableCell(
          children[index],
          node,
          context,
          safeOptions
        );
      }

      subexit();

      return result
    }

    /**
     * @type {ToMarkdownHandle}
     * @param {InlineCode} node
     */
    function inlineCodeWithTable(node, parent, context) {
      let value = inlineCode$1(node, parent, context);

      if (context.stack.includes('tableCell')) {
        value = value.replace(/\|/g, '\\$&');
      }

      return value
    }
  }

  /**
   * @typedef {import('../types.js').Context} Context
   * @typedef {import('../types.js').Options} Options
   */

  /**
   * @param {Context} context
   * @returns {Exclude<Options['bullet'], undefined>}
   */
  function checkBullet(context) {
    const marker = context.options.bullet || '*';

    if (marker !== '*' && marker !== '+' && marker !== '-') {
      throw new Error(
        'Cannot serialize items with `' +
          marker +
          '` for `options.bullet`, expected `*`, `+`, or `-`'
      )
    }

    return marker
  }

  /**
   * @typedef {import('../types.js').Context} Context
   * @typedef {import('../types.js').Options} Options
   */

  /**
   * @param {Context} context
   * @returns {Exclude<Options['listItemIndent'], undefined>}
   */
  function checkListItemIndent(context) {
    const style = context.options.listItemIndent || 'tab';

    // To do: remove in a major.
    // @ts-expect-error: deprecated.
    if (style === 1 || style === '1') {
      return 'one'
    }

    if (style !== 'tab' && style !== 'one' && style !== 'mixed') {
      throw new Error(
        'Cannot serialize items with `' +
          style +
          '` for `options.listItemIndent`, expected `tab`, `one`, or `mixed`'
      )
    }

    return style
  }

  /**
   * @typedef {import('mdast').ListItem} ListItem
   * @typedef {import('mdast').List} List
   * @typedef {import('../util/indent-lines.js').Map} Map
   * @typedef {import('../types.js').Options} Options
   * @typedef {import('../types.js').Handle} Handle
   */

  /**
   * @type {Handle}
   * @param {ListItem} node
   */
  function listItem$1(node, parent, context, safeOptions) {
    const listItemIndent = checkListItemIndent(context);
    let bullet = context.bulletCurrent || checkBullet(context);

    // Add the marker value for ordered lists.
    if (parent && parent.type === 'list' && parent.ordered) {
      bullet =
        (typeof parent.start === 'number' && parent.start > -1
          ? parent.start
          : 1) +
        (context.options.incrementListMarker === false
          ? 0
          : parent.children.indexOf(node)) +
        bullet;
    }

    let size = bullet.length + 1;

    if (
      listItemIndent === 'tab' ||
      (listItemIndent === 'mixed' &&
        ((parent && parent.type === 'list' && parent.spread) || node.spread))
    ) {
      size = Math.ceil(size / 4) * 4;
    }

    const tracker = track(safeOptions);
    tracker.move(bullet + ' '.repeat(size - bullet.length));
    tracker.shift(size);
    const exit = context.enter('listItem');
    const value = indentLines(
      containerFlow(node, context, tracker.current()),
      map
    );
    exit();

    return value

    /** @type {Map} */
    function map(line, index, blank) {
      if (index) {
        return (blank ? '' : ' '.repeat(size)) + line
      }

      return (blank ? bullet : bullet + ' '.repeat(size - bullet.length)) + line
    }
  }

  /**
   * @typedef {Extract<import('mdast').Root|import('mdast').Content, import('unist').Parent>} Parent
   * @typedef {import('mdast').ListItem} ListItem
   * @typedef {import('mdast').Paragraph} Paragraph
   * @typedef {import('mdast').BlockContent} BlockContent
   * @typedef {import('mdast-util-from-markdown').Extension} FromMarkdownExtension
   * @typedef {import('mdast-util-from-markdown').Handle} FromMarkdownHandle
   * @typedef {import('mdast-util-to-markdown').Options} ToMarkdownExtension
   * @typedef {import('mdast-util-to-markdown').Handle} ToMarkdownHandle
   */

  /** @type {FromMarkdownExtension} */
  const gfmTaskListItemFromMarkdown = {
    exit: {
      taskListCheckValueChecked: exitCheck,
      taskListCheckValueUnchecked: exitCheck,
      paragraph: exitParagraphWithTaskListItem
    }
  };

  /** @type {ToMarkdownExtension} */
  const gfmTaskListItemToMarkdown = {
    unsafe: [{atBreak: true, character: '-', after: '[:|-]'}],
    handlers: {listItem: listItemWithTaskListItem}
  };

  /** @type {FromMarkdownHandle} */
  function exitCheck(token) {
    const node = /** @type {ListItem} */ (this.stack[this.stack.length - 2]);
    // We’re always in a paragraph, in a list item.
    node.checked = token.type === 'taskListCheckValueChecked';
  }

  /** @type {FromMarkdownHandle} */
  function exitParagraphWithTaskListItem(token) {
    const parent = /** @type {Parent} */ (this.stack[this.stack.length - 2]);
    const node = /** @type {Paragraph} */ (this.stack[this.stack.length - 1]);
    const siblings = parent.children;
    const head = node.children[0];
    let index = -1;
    /** @type {Paragraph|undefined} */
    let firstParaghraph;

    if (
      parent &&
      parent.type === 'listItem' &&
      typeof parent.checked === 'boolean' &&
      head &&
      head.type === 'text'
    ) {
      while (++index < siblings.length) {
        const sibling = siblings[index];
        if (sibling.type === 'paragraph') {
          firstParaghraph = sibling;
          break
        }
      }

      if (firstParaghraph === node) {
        // Must start with a space or a tab.
        head.value = head.value.slice(1);

        if (head.value.length === 0) {
          node.children.shift();
        } else if (
          node.position &&
          head.position &&
          typeof head.position.start.offset === 'number'
        ) {
          head.position.start.column++;
          head.position.start.offset++;
          node.position.start = Object.assign({}, head.position.start);
        }
      }
    }

    this.exit(token);
  }

  /**
   * @type {ToMarkdownHandle}
   * @param {ListItem} node
   */
  function listItemWithTaskListItem(node, parent, context, safeOptions) {
    const head = node.children[0];
    const checkable =
      typeof node.checked === 'boolean' && head && head.type === 'paragraph';
    const checkbox = '[' + (node.checked ? 'x' : ' ') + '] ';
    const tracker = track(safeOptions);

    if (checkable) {
      tracker.move(checkbox);
    }

    let value = listItem$1(node, parent, context, {
      ...safeOptions,
      ...tracker.current()
    });

    if (checkable) {
      value = value.replace(/^(?:[*+-]|\d+\.)([\r\n]| {1,3})/, check);
    }

    return value

    /**
     * @param {string} $0
     * @returns {string}
     */
    function check($0) {
      return $0 + checkbox
    }
  }

  /**
   * @typedef {import('mdast-util-from-markdown').Extension} FromMarkdownExtension
   * @typedef {import('mdast-util-to-markdown').Options} ToMarkdownExtension
   *
   * @typedef {import('mdast-util-gfm-table').Options} Options
   */

  /**
   * @returns {Array.<FromMarkdownExtension>}
   */
  function gfmFromMarkdown() {
    return [
      gfmAutolinkLiteralFromMarkdown,
      gfmFootnoteFromMarkdown(),
      gfmStrikethroughFromMarkdown,
      gfmTableFromMarkdown,
      gfmTaskListItemFromMarkdown
    ]
  }

  /**
   * @param {Options} [options]
   * @returns {ToMarkdownExtension}
   */
  function gfmToMarkdown(options) {
    return {
      extensions: [
        gfmAutolinkLiteralToMarkdown,
        gfmFootnoteToMarkdown(),
        gfmStrikethroughToMarkdown,
        gfmTableToMarkdown(options),
        gfmTaskListItemToMarkdown
      ]
    }
  }

  /**
   * @typedef {import('mdast').Root} Root
   * @typedef {import('micromark-extension-gfm').Options & import('mdast-util-gfm').Options} Options
   */

  /**
   * Plugin to support GFM (autolink literals, footnotes, strikethrough, tables, tasklists).
   *
   * @type {import('unified').Plugin<[Options?]|void[], Root>}
   */
  function remarkGfm(options = {}) {
    const data = this.data();

    add('micromarkExtensions', gfm(options));
    add('fromMarkdownExtensions', gfmFromMarkdown());
    add('toMarkdownExtensions', gfmToMarkdown(options));

    /**
     * @param {string} field
     * @param {unknown} value
     */
    function add(field, value) {
      const list = /** @type {unknown[]} */ (
        // Other extensions
        /* c8 ignore next 2 */
        data[field] ? data[field] : (data[field] = [])
      );

      list.push(value);
    }
  }

  /**
   * @param {string} d
   * @returns {string}
   */
  function color$2(d) {
    return d
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   * @typedef {import('unist-util-is').Test} Test
   * @typedef {import('./complex-types').Action} Action
   * @typedef {import('./complex-types').Index} Index
   * @typedef {import('./complex-types').ActionTuple} ActionTuple
   * @typedef {import('./complex-types').VisitorResult} VisitorResult
   * @typedef {import('./complex-types').Visitor} Visitor
   */

  /**
   * Continue traversing as normal
   */
  const CONTINUE$2 = true;
  /**
   * Do not traverse this node’s children
   */
  const SKIP$2 = 'skip';
  /**
   * Stop traversing immediately
   */
  const EXIT$2 = false;

  /**
   * Visit children of tree which pass a test
   *
   * @param tree Abstract syntax tree to walk
   * @param test Test node, optional
   * @param visitor Function to run for each node
   * @param reverse Visit the tree in reverse order, defaults to false
   */
  const visitParents$2 =
    /**
     * @type {(
     *   (<Tree extends Node, Check extends Test>(tree: Tree, test: Check, visitor: import('./complex-types').BuildVisitor<Tree, Check>, reverse?: boolean) => void) &
     *   (<Tree extends Node>(tree: Tree, visitor: import('./complex-types').BuildVisitor<Tree>, reverse?: boolean) => void)
     * )}
     */
    (
      /**
       * @param {Node} tree
       * @param {Test} test
       * @param {import('./complex-types').Visitor<Node>} visitor
       * @param {boolean} [reverse]
       */
      function (tree, test, visitor, reverse) {
        if (typeof test === 'function' && typeof visitor !== 'function') {
          reverse = visitor;
          // @ts-expect-error no visitor given, so `visitor` is test.
          visitor = test;
          test = null;
        }

        const is = convert$1(test);
        const step = reverse ? -1 : 1;

        factory(tree, null, [])();

        /**
         * @param {Node} node
         * @param {number?} index
         * @param {Array.<Parent>} parents
         */
        function factory(node, index, parents) {
          /** @type {Object.<string, unknown>} */
          // @ts-expect-error: hush
          const value = typeof node === 'object' && node !== null ? node : {};
          /** @type {string|undefined} */
          let name;

          if (typeof value.type === 'string') {
            name =
              typeof value.tagName === 'string'
                ? value.tagName
                : typeof value.name === 'string'
                ? value.name
                : undefined;

            Object.defineProperty(visit, 'name', {
              value:
                'node (' +
                color$2(value.type + (name ? '<' + name + '>' : '')) +
                ')'
            });
          }

          return visit

          function visit() {
            /** @type {ActionTuple} */
            let result = [];
            /** @type {ActionTuple} */
            let subresult;
            /** @type {number} */
            let offset;
            /** @type {Array.<Parent>} */
            let grandparents;

            if (!test || is(node, index, parents[parents.length - 1] || null)) {
              result = toResult$2(visitor(node, parents));

              if (result[0] === EXIT$2) {
                return result
              }
            }

            // @ts-expect-error looks like a parent.
            if (node.children && result[0] !== SKIP$2) {
              // @ts-expect-error looks like a parent.
              offset = (reverse ? node.children.length : -1) + step;
              // @ts-expect-error looks like a parent.
              grandparents = parents.concat(node);

              // @ts-expect-error looks like a parent.
              while (offset > -1 && offset < node.children.length) {
                // @ts-expect-error looks like a parent.
                subresult = factory(node.children[offset], offset, grandparents)();

                if (subresult[0] === EXIT$2) {
                  return subresult
                }

                offset =
                  typeof subresult[1] === 'number' ? subresult[1] : offset + step;
              }
            }

            return result
          }
        }
      }
    );

  /**
   * @param {VisitorResult} value
   * @returns {ActionTuple}
   */
  function toResult$2(value) {
    if (Array.isArray(value)) {
      return value
    }

    if (typeof value === 'number') {
      return [CONTINUE$2, value]
    }

    return [value]
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   * @typedef {import('unist-util-is').Test} Test
   * @typedef {import('unist-util-visit-parents').VisitorResult} VisitorResult
   * @typedef {import('./complex-types').Visitor} Visitor
   */

  /**
   * Visit children of tree which pass a test
   *
   * @param tree Abstract syntax tree to walk
   * @param test Test, optional
   * @param visitor Function to run for each node
   * @param reverse Fisit the tree in reverse, defaults to false
   */
  const visit$2 =
    /**
     * @type {(
     *   (<Tree extends Node, Check extends Test>(tree: Tree, test: Check, visitor: import('./complex-types').BuildVisitor<Tree, Check>, reverse?: boolean) => void) &
     *   (<Tree extends Node>(tree: Tree, visitor: import('./complex-types').BuildVisitor<Tree>, reverse?: boolean) => void)
     * )}
     */
    (
      /**
       * @param {Node} tree
       * @param {Test} test
       * @param {import('./complex-types').Visitor} visitor
       * @param {boolean} [reverse]
       */
      function (tree, test, visitor, reverse) {
        if (typeof test === 'function' && typeof visitor !== 'function') {
          reverse = visitor;
          visitor = test;
          test = null;
        }

        visitParents$2(tree, test, overload, reverse);

        /**
         * @param {Node} node
         * @param {Array.<Parent>} parents
         */
        function overload(node, parents) {
          const parent = parents[parents.length - 1];
          return visitor(
            node,
            parent ? parent.children.indexOf(node) : null,
            parent
          )
        }
      }
    );

  const getURLParameters = (url) => (url.match(/([^?=&]+)(=([^&]*))/g) || []).reduce((a, v) => ((a[v.slice(0, v.indexOf('='))] = v.slice(v.indexOf('=') + 1)), a), {});
  const prevChild = (data = [], index) => {
      let i = index;
      while (i > -1) {
          i--;
          if (!data[i])
              return;
          if ((data[i] && data[i].value && data[i].value.replace(/(\n|\s)/g, '') !== '') || data[i].type !== 'text') {
              if (!/^rehype:/.test(data[i].value) || data[i].type !== 'comment')
                  return;
              return data[i];
          }
      }
      return;
  };
  const nextChild = (data = [], index, tagName) => {
      let i = index;
      while (i < data.length) {
          i++;
          if (tagName) {
              const element = data[i];
              if (element && element.value && element.value.replace(/(\n|\s)/g, '') !== '' || data[i] && data[i].type === 'element') {
                  return element.tagName === tagName ? element : undefined;
              }
          }
          else {
              const element = data[i];
              if (!element || (element.type !== 'text' && element.type !== 'comment') || (element.type === 'text' && element.value.replace(/(\n|\s)/g, '') !== ''))
                  return;
              if (element.type === 'comment') {
                  if (!/^rehype:/.test(element.value))
                      return;
                  const nextNode = nextChild(data, i, 'pre');
                  if (nextNode)
                      return;
                  return element;
              }
          }
      }
      return;
  };
  /**
   * 获取代码注视的位置
   * @param data 数据
   * @param index 当前数据所在的位置
   * @returns 返回 当前参数数据 Object，`{}`
   */
  const getCommentObject = ({ value = '' }) => {
      const param = getURLParameters(value.replace(/^rehype:/, ''));
      Object.keys(param).forEach((keyName) => {
          if (param[keyName] === 'true') {
              param[keyName] = true;
          }
          if (param[keyName] === 'false') {
              param[keyName] = false;
          }
          if (typeof param[keyName] === 'string' && !/^0/.test(param[keyName]) && !isNaN(+param[keyName])) {
              param[keyName] = +param[keyName];
          }
      });
      return param;
  };
  const propertiesHandle = (defaultAttrs, attrs, type) => {
      if (type === 'string') {
          return { ...defaultAttrs, 'data-config': JSON.stringify({ ...attrs, rehyp: true }) };
      }
      else if (type === 'attr') {
          return { ...defaultAttrs, ...attrs };
      }
      return { ...defaultAttrs, 'data-config': { ...attrs, rehyp: true } };
  };

  const defaultOptions = {
      properties: 'data',
  };
  const rehypeAttrs = (options) => {
      const opts = { ...defaultOptions, ...options };
      return (tree) => {
          visit$2(tree, 'element', (node, index, parent) => {
              if (node.tagName === 'pre' && node && Array.isArray(node.children) && parent && Array.isArray(parent.children) && parent.children.length > 1) {
                  const firstChild = node.children[0];
                  if (firstChild && firstChild.tagName === 'code' && typeof index === 'number') {
                      const child = prevChild(parent.children, index);
                      if (child) {
                          const attr = getCommentObject(child);
                          if (Object.keys(attr).length > 0) {
                              node.properties = { ...node.properties, ...{ 'data-type': 'rehyp' } };
                              firstChild.properties = propertiesHandle(firstChild.properties, attr, opts.properties);
                          }
                      }
                  }
              }
              if (/^(em|strong|b|a|i|p|pre|kbd|blockquote|h(1|2|3|4|5|6)|code|table|img|del|ul|ol)$/.test(node.tagName) && parent && Array.isArray(parent.children) && typeof index === 'number') {
                  const child = nextChild(parent.children, index);
                  if (child) {
                      const attr = getCommentObject(child);
                      if (Object.keys(attr).length > 0) {
                          node.properties = propertiesHandle(node.properties, attr, opts.properties);
                      }
                  }
              }
          });
      };
  };
  var rehypeAttrs$1 = rehypeAttrs;

  /**
   * @typedef Options
   * @property {boolean} [includeImageAlt=true]
   */

  /**
   * Get the text content of a node.
   * Prefer the node’s plain-text fields, otherwise serialize its children,
   * and if the given value is an array, serialize the nodes in it.
   *
   * @param {unknown} node
   * @param {Options} [options]
   * @returns {string}
   */
  function toString$1(node, options) {
    var {includeImageAlt = true} = options || {};
    return one$4(node, includeImageAlt)
  }

  /**
   * @param {unknown} node
   * @param {boolean} includeImageAlt
   * @returns {string}
   */
  function one$4(node, includeImageAlt) {
    return (
      (node &&
        typeof node === 'object' &&
        // @ts-ignore looks like a literal.
        (node.value ||
          // @ts-ignore looks like an image.
          (includeImageAlt ? node.alt : '') ||
          // @ts-ignore looks like a parent.
          ('children' in node && all$4(node.children, includeImageAlt)) ||
          (Array.isArray(node) && all$4(node, includeImageAlt)))) ||
      ''
    )
  }

  /**
   * @param {Array.<unknown>} values
   * @param {boolean} includeImageAlt
   * @returns {string}
   */
  function all$4(values, includeImageAlt) {
    /** @type {Array.<string>} */
    var result = [];
    var index = -1;

    while (++index < values.length) {
      result[index] = one$4(values[index], includeImageAlt);
    }

    return result.join('')
  }

  /**
   * @typedef {import('micromark-util-types').InitialConstruct} InitialConstruct
   * @typedef {import('micromark-util-types').Initializer} Initializer
   * @typedef {import('micromark-util-types').Token} Token
   * @typedef {import('micromark-util-types').State} State
   */

  /** @type {InitialConstruct} */
  const content = {
    tokenize: initializeContent
  };
  /** @type {Initializer} */

  function initializeContent(effects) {
    const contentStart = effects.attempt(
      this.parser.constructs.contentInitial,
      afterContentStartConstruct,
      paragraphInitial
    );
    /** @type {Token} */

    let previous;
    return contentStart
    /** @type {State} */

    function afterContentStartConstruct(code) {
      if (code === null) {
        effects.consume(code);
        return
      }

      effects.enter('lineEnding');
      effects.consume(code);
      effects.exit('lineEnding');
      return factorySpace(effects, contentStart, 'linePrefix')
    }
    /** @type {State} */

    function paragraphInitial(code) {
      effects.enter('paragraph');
      return lineStart(code)
    }
    /** @type {State} */

    function lineStart(code) {
      const token = effects.enter('chunkText', {
        contentType: 'text',
        previous
      });

      if (previous) {
        previous.next = token;
      }

      previous = token;
      return data(code)
    }
    /** @type {State} */

    function data(code) {
      if (code === null) {
        effects.exit('chunkText');
        effects.exit('paragraph');
        effects.consume(code);
        return
      }

      if (markdownLineEnding(code)) {
        effects.consume(code);
        effects.exit('chunkText');
        return lineStart
      } // Data.

      effects.consume(code);
      return data
    }
  }

  /**
   * @typedef {import('micromark-util-types').InitialConstruct} InitialConstruct
   * @typedef {import('micromark-util-types').Initializer} Initializer
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').TokenizeContext} TokenizeContext
   * @typedef {import('micromark-util-types').Tokenizer} Tokenizer
   * @typedef {import('micromark-util-types').Token} Token
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Point} Point
   */
  /** @type {InitialConstruct} */

  const document$2 = {
    tokenize: initializeDocument
  };
  /** @type {Construct} */

  const containerConstruct = {
    tokenize: tokenizeContainer
  };
  /** @type {Initializer} */

  function initializeDocument(effects) {
    const self = this;
    /** @type {StackItem[]} */

    const stack = [];
    let continued = 0;
    /** @type {TokenizeContext|undefined} */

    let childFlow;
    /** @type {Token|undefined} */

    let childToken;
    /** @type {number} */

    let lineStartOffset;
    return start
    /** @type {State} */

    function start(code) {
      // First we iterate through the open blocks, starting with the root
      // document, and descending through last children down to the last open
      // block.
      // Each block imposes a condition that the line must satisfy if the block is
      // to remain open.
      // For example, a block quote requires a `>` character.
      // A paragraph requires a non-blank line.
      // In this phase we may match all or just some of the open blocks.
      // But we cannot close unmatched blocks yet, because we may have a lazy
      // continuation line.
      if (continued < stack.length) {
        const item = stack[continued];
        self.containerState = item[1];
        return effects.attempt(
          item[0].continuation,
          documentContinue,
          checkNewContainers
        )(code)
      } // Done.

      return checkNewContainers(code)
    }
    /** @type {State} */

    function documentContinue(code) {
      continued++; // Note: this field is called `_closeFlow` but it also closes containers.
      // Perhaps a good idea to rename it but it’s already used in the wild by
      // extensions.

      if (self.containerState._closeFlow) {
        self.containerState._closeFlow = undefined;

        if (childFlow) {
          closeFlow();
        } // Note: this algorithm for moving events around is similar to the
        // algorithm when dealing with lazy lines in `writeToChild`.

        const indexBeforeExits = self.events.length;
        let indexBeforeFlow = indexBeforeExits;
        /** @type {Point|undefined} */

        let point; // Find the flow chunk.

        while (indexBeforeFlow--) {
          if (
            self.events[indexBeforeFlow][0] === 'exit' &&
            self.events[indexBeforeFlow][1].type === 'chunkFlow'
          ) {
            point = self.events[indexBeforeFlow][1].end;
            break
          }
        }

        exitContainers(continued); // Fix positions.

        let index = indexBeforeExits;

        while (index < self.events.length) {
          self.events[index][1].end = Object.assign({}, point);
          index++;
        } // Inject the exits earlier (they’re still also at the end).

        splice(
          self.events,
          indexBeforeFlow + 1,
          0,
          self.events.slice(indexBeforeExits)
        ); // Discard the duplicate exits.

        self.events.length = index;
        return checkNewContainers(code)
      }

      return start(code)
    }
    /** @type {State} */

    function checkNewContainers(code) {
      // Next, after consuming the continuation markers for existing blocks, we
      // look for new block starts (e.g. `>` for a block quote).
      // If we encounter a new block start, we close any blocks unmatched in
      // step 1 before creating the new block as a child of the last matched
      // block.
      if (continued === stack.length) {
        // No need to `check` whether there’s a container, of `exitContainers`
        // would be moot.
        // We can instead immediately `attempt` to parse one.
        if (!childFlow) {
          return documentContinued(code)
        } // If we have concrete content, such as block HTML or fenced code,
        // we can’t have containers “pierce” into them, so we can immediately
        // start.

        if (childFlow.currentConstruct && childFlow.currentConstruct.concrete) {
          return flowStart(code)
        } // If we do have flow, it could still be a blank line,
        // but we’d be interrupting it w/ a new container if there’s a current
        // construct.

        self.interrupt = Boolean(
          childFlow.currentConstruct && !childFlow._gfmTableDynamicInterruptHack
        );
      } // Check if there is a new container.

      self.containerState = {};
      return effects.check(
        containerConstruct,
        thereIsANewContainer,
        thereIsNoNewContainer
      )(code)
    }
    /** @type {State} */

    function thereIsANewContainer(code) {
      if (childFlow) closeFlow();
      exitContainers(continued);
      return documentContinued(code)
    }
    /** @type {State} */

    function thereIsNoNewContainer(code) {
      self.parser.lazy[self.now().line] = continued !== stack.length;
      lineStartOffset = self.now().offset;
      return flowStart(code)
    }
    /** @type {State} */

    function documentContinued(code) {
      // Try new containers.
      self.containerState = {};
      return effects.attempt(
        containerConstruct,
        containerContinue,
        flowStart
      )(code)
    }
    /** @type {State} */

    function containerContinue(code) {
      continued++;
      stack.push([self.currentConstruct, self.containerState]); // Try another.

      return documentContinued(code)
    }
    /** @type {State} */

    function flowStart(code) {
      if (code === null) {
        if (childFlow) closeFlow();
        exitContainers(0);
        effects.consume(code);
        return
      }

      childFlow = childFlow || self.parser.flow(self.now());
      effects.enter('chunkFlow', {
        contentType: 'flow',
        previous: childToken,
        _tokenizer: childFlow
      });
      return flowContinue(code)
    }
    /** @type {State} */

    function flowContinue(code) {
      if (code === null) {
        writeToChild(effects.exit('chunkFlow'), true);
        exitContainers(0);
        effects.consume(code);
        return
      }

      if (markdownLineEnding(code)) {
        effects.consume(code);
        writeToChild(effects.exit('chunkFlow')); // Get ready for the next line.

        continued = 0;
        self.interrupt = undefined;
        return start
      }

      effects.consume(code);
      return flowContinue
    }
    /**
     * @param {Token} token
     * @param {boolean} [eof]
     * @returns {void}
     */

    function writeToChild(token, eof) {
      const stream = self.sliceStream(token);
      if (eof) stream.push(null);
      token.previous = childToken;
      if (childToken) childToken.next = token;
      childToken = token;
      childFlow.defineSkip(token.start);
      childFlow.write(stream); // Alright, so we just added a lazy line:
      //
      // ```markdown
      // > a
      // b.
      //
      // Or:
      //
      // > ~~~c
      // d
      //
      // Or:
      //
      // > | e |
      // f
      // ```
      //
      // The construct in the second example (fenced code) does not accept lazy
      // lines, so it marked itself as done at the end of its first line, and
      // then the content construct parses `d`.
      // Most constructs in markdown match on the first line: if the first line
      // forms a construct, a non-lazy line can’t “unmake” it.
      //
      // The construct in the third example is potentially a GFM table, and
      // those are *weird*.
      // It *could* be a table, from the first line, if the following line
      // matches a condition.
      // In this case, that second line is lazy, which “unmakes” the first line
      // and turns the whole into one content block.
      //
      // We’ve now parsed the non-lazy and the lazy line, and can figure out
      // whether the lazy line started a new flow block.
      // If it did, we exit the current containers between the two flow blocks.

      if (self.parser.lazy[token.start.line]) {
        let index = childFlow.events.length;

        while (index--) {
          if (
            // The token starts before the line ending…
            childFlow.events[index][1].start.offset < lineStartOffset && // …and either is not ended yet…
            (!childFlow.events[index][1].end || // …or ends after it.
              childFlow.events[index][1].end.offset > lineStartOffset)
          ) {
            // Exit: there’s still something open, which means it’s a lazy line
            // part of something.
            return
          }
        } // Note: this algorithm for moving events around is similar to the
        // algorithm when closing flow in `documentContinue`.

        const indexBeforeExits = self.events.length;
        let indexBeforeFlow = indexBeforeExits;
        /** @type {boolean|undefined} */

        let seen;
        /** @type {Point|undefined} */

        let point; // Find the previous chunk (the one before the lazy line).

        while (indexBeforeFlow--) {
          if (
            self.events[indexBeforeFlow][0] === 'exit' &&
            self.events[indexBeforeFlow][1].type === 'chunkFlow'
          ) {
            if (seen) {
              point = self.events[indexBeforeFlow][1].end;
              break
            }

            seen = true;
          }
        }

        exitContainers(continued); // Fix positions.

        index = indexBeforeExits;

        while (index < self.events.length) {
          self.events[index][1].end = Object.assign({}, point);
          index++;
        } // Inject the exits earlier (they’re still also at the end).

        splice(
          self.events,
          indexBeforeFlow + 1,
          0,
          self.events.slice(indexBeforeExits)
        ); // Discard the duplicate exits.

        self.events.length = index;
      }
    }
    /**
     * @param {number} size
     * @returns {void}
     */

    function exitContainers(size) {
      let index = stack.length; // Exit open containers.

      while (index-- > size) {
        const entry = stack[index];
        self.containerState = entry[1];
        entry[0].exit.call(self, effects);
      }

      stack.length = size;
    }

    function closeFlow() {
      childFlow.write([null]);
      childToken = undefined;
      childFlow = undefined;
      self.containerState._closeFlow = undefined;
    }
  }
  /** @type {Tokenizer} */

  function tokenizeContainer(effects, ok, nok) {
    return factorySpace(
      effects,
      effects.attempt(this.parser.constructs.document, ok, nok),
      'linePrefix',
      this.parser.constructs.disable.null.includes('codeIndented') ? undefined : 4
    )
  }

  /**
   * @typedef {import('micromark-util-types').InitialConstruct} InitialConstruct
   * @typedef {import('micromark-util-types').Initializer} Initializer
   * @typedef {import('micromark-util-types').State} State
   */

  /** @type {InitialConstruct} */
  const flow$2 = {
    tokenize: initializeFlow
  };
  /** @type {Initializer} */

  function initializeFlow(effects) {
    const self = this;
    const initial = effects.attempt(
      // Try to parse a blank line.
      blankLine,
      atBlankEnding, // Try to parse initial flow (essentially, only code).
      effects.attempt(
        this.parser.constructs.flowInitial,
        afterConstruct,
        factorySpace(
          effects,
          effects.attempt(
            this.parser.constructs.flow,
            afterConstruct,
            effects.attempt(content$1, afterConstruct)
          ),
          'linePrefix'
        )
      )
    );
    return initial
    /** @type {State} */

    function atBlankEnding(code) {
      if (code === null) {
        effects.consume(code);
        return
      }

      effects.enter('lineEndingBlank');
      effects.consume(code);
      effects.exit('lineEndingBlank');
      self.currentConstruct = undefined;
      return initial
    }
    /** @type {State} */

    function afterConstruct(code) {
      if (code === null) {
        effects.consume(code);
        return
      }

      effects.enter('lineEnding');
      effects.consume(code);
      effects.exit('lineEnding');
      self.currentConstruct = undefined;
      return initial
    }
  }

  /**
   * @typedef {import('micromark-util-types').Resolver} Resolver
   * @typedef {import('micromark-util-types').Initializer} Initializer
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').InitialConstruct} InitialConstruct
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Code} Code
   */
  const resolver = {
    resolveAll: createResolver()
  };
  const string$1 = initializeFactory('string');
  const text$8 = initializeFactory('text');
  /**
   * @param {'string'|'text'} field
   * @returns {InitialConstruct}
   */

  function initializeFactory(field) {
    return {
      tokenize: initializeText,
      resolveAll: createResolver(
        field === 'text' ? resolveAllLineSuffixes : undefined
      )
    }
    /** @type {Initializer} */

    function initializeText(effects) {
      const self = this;
      const constructs = this.parser.constructs[field];
      const text = effects.attempt(constructs, start, notText);
      return start
      /** @type {State} */

      function start(code) {
        return atBreak(code) ? text(code) : notText(code)
      }
      /** @type {State} */

      function notText(code) {
        if (code === null) {
          effects.consume(code);
          return
        }

        effects.enter('data');
        effects.consume(code);
        return data
      }
      /** @type {State} */

      function data(code) {
        if (atBreak(code)) {
          effects.exit('data');
          return text(code)
        } // Data.

        effects.consume(code);
        return data
      }
      /**
       * @param {Code} code
       * @returns {boolean}
       */

      function atBreak(code) {
        if (code === null) {
          return true
        }

        const list = constructs[code];
        let index = -1;

        if (list) {
          while (++index < list.length) {
            const item = list[index];

            if (!item.previous || item.previous.call(self, self.previous)) {
              return true
            }
          }
        }

        return false
      }
    }
  }
  /**
   * @param {Resolver} [extraResolver]
   * @returns {Resolver}
   */

  function createResolver(extraResolver) {
    return resolveAllText
    /** @type {Resolver} */

    function resolveAllText(events, context) {
      let index = -1;
      /** @type {number|undefined} */

      let enter; // A rather boring computation (to merge adjacent `data` events) which
      // improves mm performance by 29%.

      while (++index <= events.length) {
        if (enter === undefined) {
          if (events[index] && events[index][1].type === 'data') {
            enter = index;
            index++;
          }
        } else if (!events[index] || events[index][1].type !== 'data') {
          // Don’t do anything if there is one data token.
          if (index !== enter + 2) {
            events[enter][1].end = events[index - 1][1].end;
            events.splice(enter + 2, index - enter - 2);
            index = enter + 2;
          }

          enter = undefined;
        }
      }

      return extraResolver ? extraResolver(events, context) : events
    }
  }
  /**
   * A rather ugly set of instructions which again looks at chunks in the input
   * stream.
   * The reason to do this here is that it is *much* faster to parse in reverse.
   * And that we can’t hook into `null` to split the line suffix before an EOF.
   * To do: figure out if we can make this into a clean utility, or even in core.
   * As it will be useful for GFMs literal autolink extension (and maybe even
   * tables?)
   *
   * @type {Resolver}
   */

  function resolveAllLineSuffixes(events, context) {
    let eventIndex = 0; // Skip first.

    while (++eventIndex <= events.length) {
      if (
        (eventIndex === events.length ||
          events[eventIndex][1].type === 'lineEnding') &&
        events[eventIndex - 1][1].type === 'data'
      ) {
        const data = events[eventIndex - 1][1];
        const chunks = context.sliceStream(data);
        let index = chunks.length;
        let bufferIndex = -1;
        let size = 0;
        /** @type {boolean|undefined} */

        let tabs;

        while (index--) {
          const chunk = chunks[index];

          if (typeof chunk === 'string') {
            bufferIndex = chunk.length;

            while (chunk.charCodeAt(bufferIndex - 1) === 32) {
              size++;
              bufferIndex--;
            }

            if (bufferIndex) break
            bufferIndex = -1;
          } // Number
          else if (chunk === -2) {
            tabs = true;
            size++;
          } else if (chunk === -1) ; else {
            // Replacement character, exit.
            index++;
            break
          }
        }

        if (size) {
          const token = {
            type:
              eventIndex === events.length || tabs || size < 2
                ? 'lineSuffix'
                : 'hardBreakTrailing',
            start: {
              line: data.end.line,
              column: data.end.column - size,
              offset: data.end.offset - size,
              _index: data.start._index + index,
              _bufferIndex: index
                ? bufferIndex
                : data.start._bufferIndex + bufferIndex
            },
            end: Object.assign({}, data.end)
          };
          data.end = Object.assign({}, token.start);

          if (data.start.offset === data.end.offset) {
            Object.assign(data, token);
          } else {
            events.splice(
              eventIndex,
              0,
              ['enter', token, context],
              ['exit', token, context]
            );
            eventIndex += 2;
          }
        }

        eventIndex++;
      }
    }

    return events
  }

  /**
   * @typedef {import('micromark-util-types').Code} Code
   * @typedef {import('micromark-util-types').Chunk} Chunk
   * @typedef {import('micromark-util-types').Point} Point
   * @typedef {import('micromark-util-types').Token} Token
   * @typedef {import('micromark-util-types').Effects} Effects
   * @typedef {import('micromark-util-types').State} State
   * @typedef {import('micromark-util-types').Construct} Construct
   * @typedef {import('micromark-util-types').InitialConstruct} InitialConstruct
   * @typedef {import('micromark-util-types').ConstructRecord} ConstructRecord
   * @typedef {import('micromark-util-types').TokenizeContext} TokenizeContext
   * @typedef {import('micromark-util-types').ParseContext} ParseContext
   */

  /**
   * Create a tokenizer.
   * Tokenizers deal with one type of data (e.g., containers, flow, text).
   * The parser is the object dealing with it all.
   * `initialize` works like other constructs, except that only its `tokenize`
   * function is used, in which case it doesn’t receive an `ok` or `nok`.
   * `from` can be given to set the point before the first character, although
   * when further lines are indented, they must be set with `defineSkip`.
   *
   * @param {ParseContext} parser
   * @param {InitialConstruct} initialize
   * @param {Omit<Point, '_index'|'_bufferIndex'>} [from]
   * @returns {TokenizeContext}
   */
  function createTokenizer(parser, initialize, from) {
    /** @type {Point} */
    let point = Object.assign(
      from
        ? Object.assign({}, from)
        : {
            line: 1,
            column: 1,
            offset: 0
          },
      {
        _index: 0,
        _bufferIndex: -1
      }
    );
    /** @type {Record<string, number>} */

    const columnStart = {};
    /** @type {Construct[]} */

    const resolveAllConstructs = [];
    /** @type {Chunk[]} */

    let chunks = [];
    /** @type {Token[]} */

    let stack = [];
    /**
     * Tools used for tokenizing.
     *
     * @type {Effects}
     */

    const effects = {
      consume,
      enter,
      exit,
      attempt: constructFactory(onsuccessfulconstruct),
      check: constructFactory(onsuccessfulcheck),
      interrupt: constructFactory(onsuccessfulcheck, {
        interrupt: true
      })
    };
    /**
     * State and tools for resolving and serializing.
     *
     * @type {TokenizeContext}
     */

    const context = {
      previous: null,
      code: null,
      containerState: {},
      events: [],
      parser,
      sliceStream,
      sliceSerialize,
      now,
      defineSkip,
      write
    };
    /**
     * The state function.
     *
     * @type {State|void}
     */

    let state = initialize.tokenize.call(context, effects);

    if (initialize.resolveAll) {
      resolveAllConstructs.push(initialize);
    }

    return context
    /** @type {TokenizeContext['write']} */

    function write(slice) {
      chunks = push(chunks, slice);
      main(); // Exit if we’re not done, resolve might change stuff.

      if (chunks[chunks.length - 1] !== null) {
        return []
      }

      addResult(initialize, 0); // Otherwise, resolve, and exit.

      context.events = resolveAll(resolveAllConstructs, context.events, context);
      return context.events
    } //
    // Tools.
    //

    /** @type {TokenizeContext['sliceSerialize']} */

    function sliceSerialize(token, expandTabs) {
      return serializeChunks(sliceStream(token), expandTabs)
    }
    /** @type {TokenizeContext['sliceStream']} */

    function sliceStream(token) {
      return sliceChunks(chunks, token)
    }
    /** @type {TokenizeContext['now']} */

    function now() {
      return Object.assign({}, point)
    }
    /** @type {TokenizeContext['defineSkip']} */

    function defineSkip(value) {
      columnStart[value.line] = value.column;
      accountForPotentialSkip();
    } //
    // State management.
    //

    /**
     * Main loop (note that `_index` and `_bufferIndex` in `point` are modified by
     * `consume`).
     * Here is where we walk through the chunks, which either include strings of
     * several characters, or numerical character codes.
     * The reason to do this in a loop instead of a call is so the stack can
     * drain.
     *
     * @returns {void}
     */

    function main() {
      /** @type {number} */
      let chunkIndex;

      while (point._index < chunks.length) {
        const chunk = chunks[point._index]; // If we’re in a buffer chunk, loop through it.

        if (typeof chunk === 'string') {
          chunkIndex = point._index;

          if (point._bufferIndex < 0) {
            point._bufferIndex = 0;
          }

          while (
            point._index === chunkIndex &&
            point._bufferIndex < chunk.length
          ) {
            go(chunk.charCodeAt(point._bufferIndex));
          }
        } else {
          go(chunk);
        }
      }
    }
    /**
     * Deal with one code.
     *
     * @param {Code} code
     * @returns {void}
     */

    function go(code) {
      state = state(code);
    }
    /** @type {Effects['consume']} */

    function consume(code) {
      if (markdownLineEnding(code)) {
        point.line++;
        point.column = 1;
        point.offset += code === -3 ? 2 : 1;
        accountForPotentialSkip();
      } else if (code !== -1) {
        point.column++;
        point.offset++;
      } // Not in a string chunk.

      if (point._bufferIndex < 0) {
        point._index++;
      } else {
        point._bufferIndex++; // At end of string chunk.
        // @ts-expect-error Points w/ non-negative `_bufferIndex` reference
        // strings.

        if (point._bufferIndex === chunks[point._index].length) {
          point._bufferIndex = -1;
          point._index++;
        }
      } // Expose the previous character.

      context.previous = code; // Mark as consumed.
    }
    /** @type {Effects['enter']} */

    function enter(type, fields) {
      /** @type {Token} */
      // @ts-expect-error Patch instead of assign required fields to help GC.
      const token = fields || {};
      token.type = type;
      token.start = now();
      context.events.push(['enter', token, context]);
      stack.push(token);
      return token
    }
    /** @type {Effects['exit']} */

    function exit(type) {
      const token = stack.pop();
      token.end = now();
      context.events.push(['exit', token, context]);
      return token
    }
    /**
     * Use results.
     *
     * @type {ReturnHandle}
     */

    function onsuccessfulconstruct(construct, info) {
      addResult(construct, info.from);
    }
    /**
     * Discard results.
     *
     * @type {ReturnHandle}
     */

    function onsuccessfulcheck(_, info) {
      info.restore();
    }
    /**
     * Factory to attempt/check/interrupt.
     *
     * @param {ReturnHandle} onreturn
     * @param {Record<string, unknown>} [fields]
     */

    function constructFactory(onreturn, fields) {
      return hook
      /**
       * Handle either an object mapping codes to constructs, a list of
       * constructs, or a single construct.
       *
       * @param {Construct|Construct[]|ConstructRecord} constructs
       * @param {State} returnState
       * @param {State} [bogusState]
       * @returns {State}
       */

      function hook(constructs, returnState, bogusState) {
        /** @type {Construct[]} */
        let listOfConstructs;
        /** @type {number} */

        let constructIndex;
        /** @type {Construct} */

        let currentConstruct;
        /** @type {Info} */

        let info;
        return Array.isArray(constructs)
          ? /* c8 ignore next 1 */
            handleListOfConstructs(constructs)
          : 'tokenize' in constructs // @ts-expect-error Looks like a construct.
          ? handleListOfConstructs([constructs])
          : handleMapOfConstructs(constructs)
        /**
         * Handle a list of construct.
         *
         * @param {ConstructRecord} map
         * @returns {State}
         */

        function handleMapOfConstructs(map) {
          return start
          /** @type {State} */

          function start(code) {
            const def = code !== null && map[code];
            const all = code !== null && map.null;
            const list = [
              // To do: add more extension tests.

              /* c8 ignore next 2 */
              ...(Array.isArray(def) ? def : def ? [def] : []),
              ...(Array.isArray(all) ? all : all ? [all] : [])
            ];
            return handleListOfConstructs(list)(code)
          }
        }
        /**
         * Handle a list of construct.
         *
         * @param {Construct[]} list
         * @returns {State}
         */

        function handleListOfConstructs(list) {
          listOfConstructs = list;
          constructIndex = 0;

          if (list.length === 0) {
            return bogusState
          }

          return handleConstruct(list[constructIndex])
        }
        /**
         * Handle a single construct.
         *
         * @param {Construct} construct
         * @returns {State}
         */

        function handleConstruct(construct) {
          return start
          /** @type {State} */

          function start(code) {
            // To do: not needed to store if there is no bogus state, probably?
            // Currently doesn’t work because `inspect` in document does a check
            // w/o a bogus, which doesn’t make sense. But it does seem to help perf
            // by not storing.
            info = store();
            currentConstruct = construct;

            if (!construct.partial) {
              context.currentConstruct = construct;
            }

            if (
              construct.name &&
              context.parser.constructs.disable.null.includes(construct.name)
            ) {
              return nok()
            }

            return construct.tokenize.call(
              // If we do have fields, create an object w/ `context` as its
              // prototype.
              // This allows a “live binding”, which is needed for `interrupt`.
              fields ? Object.assign(Object.create(context), fields) : context,
              effects,
              ok,
              nok
            )(code)
          }
        }
        /** @type {State} */

        function ok(code) {
          onreturn(currentConstruct, info);
          return returnState
        }
        /** @type {State} */

        function nok(code) {
          info.restore();

          if (++constructIndex < listOfConstructs.length) {
            return handleConstruct(listOfConstructs[constructIndex])
          }

          return bogusState
        }
      }
    }
    /**
     * @param {Construct} construct
     * @param {number} from
     * @returns {void}
     */

    function addResult(construct, from) {
      if (construct.resolveAll && !resolveAllConstructs.includes(construct)) {
        resolveAllConstructs.push(construct);
      }

      if (construct.resolve) {
        splice(
          context.events,
          from,
          context.events.length - from,
          construct.resolve(context.events.slice(from), context)
        );
      }

      if (construct.resolveTo) {
        context.events = construct.resolveTo(context.events, context);
      }
    }
    /**
     * Store state.
     *
     * @returns {Info}
     */

    function store() {
      const startPoint = now();
      const startPrevious = context.previous;
      const startCurrentConstruct = context.currentConstruct;
      const startEventsIndex = context.events.length;
      const startStack = Array.from(stack);
      return {
        restore,
        from: startEventsIndex
      }
      /**
       * Restore state.
       *
       * @returns {void}
       */

      function restore() {
        point = startPoint;
        context.previous = startPrevious;
        context.currentConstruct = startCurrentConstruct;
        context.events.length = startEventsIndex;
        stack = startStack;
        accountForPotentialSkip();
      }
    }
    /**
     * Move the current point a bit forward in the line when it’s on a column
     * skip.
     *
     * @returns {void}
     */

    function accountForPotentialSkip() {
      if (point.line in columnStart && point.column < 2) {
        point.column = columnStart[point.line];
        point.offset += columnStart[point.line] - 1;
      }
    }
  }
  /**
   * Get the chunks from a slice of chunks in the range of a token.
   *
   * @param {Chunk[]} chunks
   * @param {Pick<Token, 'start'|'end'>} token
   * @returns {Chunk[]}
   */

  function sliceChunks(chunks, token) {
    const startIndex = token.start._index;
    const startBufferIndex = token.start._bufferIndex;
    const endIndex = token.end._index;
    const endBufferIndex = token.end._bufferIndex;
    /** @type {Chunk[]} */

    let view;

    if (startIndex === endIndex) {
      // @ts-expect-error `_bufferIndex` is used on string chunks.
      view = [chunks[startIndex].slice(startBufferIndex, endBufferIndex)];
    } else {
      view = chunks.slice(startIndex, endIndex);

      if (startBufferIndex > -1) {
        // @ts-expect-error `_bufferIndex` is used on string chunks.
        view[0] = view[0].slice(startBufferIndex);
      }

      if (endBufferIndex > 0) {
        // @ts-expect-error `_bufferIndex` is used on string chunks.
        view.push(chunks[endIndex].slice(0, endBufferIndex));
      }
    }

    return view
  }
  /**
   * Get the string value of a slice of chunks.
   *
   * @param {Chunk[]} chunks
   * @param {boolean} [expandTabs=false]
   * @returns {string}
   */

  function serializeChunks(chunks, expandTabs) {
    let index = -1;
    /** @type {string[]} */

    const result = [];
    /** @type {boolean|undefined} */

    let atTab;

    while (++index < chunks.length) {
      const chunk = chunks[index];
      /** @type {string} */

      let value;

      if (typeof chunk === 'string') {
        value = chunk;
      } else
        switch (chunk) {
          case -5: {
            value = '\r';
            break
          }

          case -4: {
            value = '\n';
            break
          }

          case -3: {
            value = '\r' + '\n';
            break
          }

          case -2: {
            value = expandTabs ? ' ' : '\t';
            break
          }

          case -1: {
            if (!expandTabs && atTab) continue
            value = ' ';
            break
          }

          default: {
            // Currently only replacement character.
            value = String.fromCharCode(chunk);
          }
        }

      atTab = chunk === -2;
      result.push(value);
    }

    return result.join('')
  }

  /**
   * @typedef {import('micromark-util-types').Extension} Extension
   */
  /** @type {Extension['document']} */

  const document$1 = {
    [42]: list$1,
    [43]: list$1,
    [45]: list$1,
    [48]: list$1,
    [49]: list$1,
    [50]: list$1,
    [51]: list$1,
    [52]: list$1,
    [53]: list$1,
    [54]: list$1,
    [55]: list$1,
    [56]: list$1,
    [57]: list$1,
    [62]: blockQuote
  };
  /** @type {Extension['contentInitial']} */

  const contentInitial = {
    [91]: definition
  };
  /** @type {Extension['flowInitial']} */

  const flowInitial = {
    [-2]: codeIndented,
    [-1]: codeIndented,
    [32]: codeIndented
  };
  /** @type {Extension['flow']} */

  const flow$1 = {
    [35]: headingAtx,
    [42]: thematicBreak$1,
    [45]: [setextUnderline, thematicBreak$1],
    [60]: htmlFlow,
    [61]: setextUnderline,
    [95]: thematicBreak$1,
    [96]: codeFenced,
    [126]: codeFenced
  };
  /** @type {Extension['string']} */

  const string = {
    [38]: characterReference,
    [92]: characterEscape
  };
  /** @type {Extension['text']} */

  const text$7 = {
    [-5]: lineEnding,
    [-4]: lineEnding,
    [-3]: lineEnding,
    [33]: labelStartImage,
    [38]: characterReference,
    [42]: attention,
    [60]: [autolink, htmlText],
    [91]: labelStartLink,
    [92]: [hardBreakEscape, characterEscape],
    [93]: labelEnd,
    [95]: attention,
    [96]: codeText
  };
  /** @type {Extension['insideSpan']} */

  const insideSpan = {
    null: [attention, resolver]
  };
  /** @type {Extension['attentionMarkers']} */

  const attentionMarkers = {
    null: [42, 95]
  };
  /** @type {Extension['disable']} */

  const disable = {
    null: []
  };

  var defaultConstructs = /*#__PURE__*/Object.freeze({
      __proto__: null,
      document: document$1,
      contentInitial: contentInitial,
      flowInitial: flowInitial,
      flow: flow$1,
      string: string,
      text: text$7,
      insideSpan: insideSpan,
      attentionMarkers: attentionMarkers,
      disable: disable
  });

  /**
   * @typedef {import('micromark-util-types').InitialConstruct} InitialConstruct
   * @typedef {import('micromark-util-types').FullNormalizedExtension} FullNormalizedExtension
   * @typedef {import('micromark-util-types').ParseOptions} ParseOptions
   * @typedef {import('micromark-util-types').ParseContext} ParseContext
   * @typedef {import('micromark-util-types').Create} Create
   */
  /**
   * @param {ParseOptions} [options]
   * @returns {ParseContext}
   */

  function parse$b(options = {}) {
    /** @type {FullNormalizedExtension} */
    // @ts-expect-error `defaultConstructs` is full, so the result will be too.
    const constructs = combineExtensions(
      // @ts-expect-error Same as above.
      [defaultConstructs].concat(options.extensions || [])
    );
    /** @type {ParseContext} */

    const parser = {
      defined: [],
      lazy: {},
      constructs,
      content: create(content),
      document: create(document$2),
      flow: create(flow$2),
      string: create(string$1),
      text: create(text$8)
    };
    return parser
    /**
     * @param {InitialConstruct} initial
     */

    function create(initial) {
      return creator
      /** @type {Create} */

      function creator(from) {
        return createTokenizer(parser, initial, from)
      }
    }
  }

  /**
   * @typedef {import('micromark-util-types').Encoding} Encoding
   * @typedef {import('micromark-util-types').Value} Value
   * @typedef {import('micromark-util-types').Chunk} Chunk
   * @typedef {import('micromark-util-types').Code} Code
   */

  /**
   * @callback Preprocessor
   * @param {Value} value
   * @param {Encoding} [encoding]
   * @param {boolean} [end=false]
   * @returns {Chunk[]}
   */
  const search$3 = /[\0\t\n\r]/g;
  /**
   * @returns {Preprocessor}
   */

  function preprocess() {
    let column = 1;
    let buffer = '';
    /** @type {boolean|undefined} */

    let start = true;
    /** @type {boolean|undefined} */

    let atCarriageReturn;
    return preprocessor
    /** @type {Preprocessor} */

    function preprocessor(value, encoding, end) {
      /** @type {Chunk[]} */
      const chunks = [];
      /** @type {RegExpMatchArray|null} */

      let match;
      /** @type {number} */

      let next;
      /** @type {number} */

      let startPosition;
      /** @type {number} */

      let endPosition;
      /** @type {Code} */

      let code; // @ts-expect-error `Buffer` does allow an encoding.

      value = buffer + value.toString(encoding);
      startPosition = 0;
      buffer = '';

      if (start) {
        if (value.charCodeAt(0) === 65279) {
          startPosition++;
        }

        start = undefined;
      }

      while (startPosition < value.length) {
        search$3.lastIndex = startPosition;
        match = search$3.exec(value);
        endPosition =
          match && match.index !== undefined ? match.index : value.length;
        code = value.charCodeAt(endPosition);

        if (!match) {
          buffer = value.slice(startPosition);
          break
        }

        if (code === 10 && startPosition === endPosition && atCarriageReturn) {
          chunks.push(-3);
          atCarriageReturn = undefined;
        } else {
          if (atCarriageReturn) {
            chunks.push(-5);
            atCarriageReturn = undefined;
          }

          if (startPosition < endPosition) {
            chunks.push(value.slice(startPosition, endPosition));
            column += endPosition - startPosition;
          }

          switch (code) {
            case 0: {
              chunks.push(65533);
              column++;
              break
            }

            case 9: {
              next = Math.ceil(column / 4) * 4;
              chunks.push(-2);

              while (column++ < next) chunks.push(-1);

              break
            }

            case 10: {
              chunks.push(-4);
              column = 1;
              break
            }

            default: {
              atCarriageReturn = true;
              column = 1;
            }
          }
        }

        startPosition = endPosition + 1;
      }

      if (end) {
        if (atCarriageReturn) chunks.push(-5);
        if (buffer) chunks.push(buffer);
        chunks.push(null);
      }

      return chunks
    }
  }

  /**
   * @typedef {import('micromark-util-types').Event} Event
   */
  /**
   * @param {Event[]} events
   * @returns {Event[]}
   */

  function postprocess(events) {
    while (!subtokenize(events)) {
      // Empty
    }

    return events
  }

  /**
   * @typedef {import('micromark-util-types').Encoding} Encoding
   * @typedef {import('micromark-util-types').Event} Event
   * @typedef {import('micromark-util-types').ParseOptions} ParseOptions
   * @typedef {import('micromark-util-types').Token} Token
   * @typedef {import('micromark-util-types').TokenizeContext} TokenizeContext
   * @typedef {import('micromark-util-types').Value} Value
   * @typedef {import('unist').Parent} UnistParent
   * @typedef {import('unist').Point} Point
   * @typedef {import('mdast').PhrasingContent} PhrasingContent
   * @typedef {import('mdast').Content} Content
   * @typedef {Root|Content} Node
   * @typedef {Extract<Node, UnistParent>} Parent
   * @typedef {import('mdast').Break} Break
   * @typedef {import('mdast').Blockquote} Blockquote
   * @typedef {import('mdast').Code} Code
   * @typedef {import('mdast').Definition} Definition
   * @typedef {import('mdast').Emphasis} Emphasis
   * @typedef {import('mdast').Heading} Heading
   * @typedef {import('mdast').HTML} HTML
   * @typedef {import('mdast').Image} Image
   * @typedef {import('mdast').ImageReference} ImageReference
   * @typedef {import('mdast').InlineCode} InlineCode
   * @typedef {import('mdast').Link} Link
   * @typedef {import('mdast').LinkReference} LinkReference
   * @typedef {import('mdast').List} List
   * @typedef {import('mdast').ListItem} ListItem
   * @typedef {import('mdast').Paragraph} Paragraph
   * @typedef {import('mdast').Root} Root
   * @typedef {import('mdast').Strong} Strong
   * @typedef {import('mdast').Text} Text
   * @typedef {import('mdast').ThematicBreak} ThematicBreak
   *
   * @typedef {UnistParent & {type: 'fragment', children: Array<PhrasingContent>}} Fragment
   */
  const own$q = {}.hasOwnProperty;
  /**
   * @param value Markdown to parse (`string` or `Buffer`).
   * @param [encoding] Character encoding to understand `value` as when it’s a `Buffer` (`string`, default: `'utf8'`).
   * @param [options] Configuration
   */

  const fromMarkdown =
    /**
     * @type {(
     *   ((value: Value, encoding: Encoding, options?: Options) => Root) &
     *   ((value: Value, options?: Options) => Root)
     * )}
     */

    /**
     * @param {Value} value
     * @param {Encoding} [encoding]
     * @param {Options} [options]
     * @returns {Root}
     */
    function (value, encoding, options) {
      if (typeof encoding !== 'string') {
        options = encoding;
        encoding = undefined;
      }

      return compiler(options)(
        postprocess(
          parse$b(options).document().write(preprocess()(value, encoding, true))
        )
      )
    };
  /**
   * Note this compiler only understand complete buffering, not streaming.
   *
   * @param {Options} [options]
   */

  function compiler(options = {}) {
    /** @type {NormalizedExtension} */
    // @ts-expect-error: our base has all required fields, so the result will too.
    const config = configure(
      {
        transforms: [],
        canContainEols: [
          'emphasis',
          'fragment',
          'heading',
          'paragraph',
          'strong'
        ],
        enter: {
          autolink: opener(link),
          autolinkProtocol: onenterdata,
          autolinkEmail: onenterdata,
          atxHeading: opener(heading),
          blockQuote: opener(blockQuote),
          characterEscape: onenterdata,
          characterReference: onenterdata,
          codeFenced: opener(codeFlow),
          codeFencedFenceInfo: buffer,
          codeFencedFenceMeta: buffer,
          codeIndented: opener(codeFlow, buffer),
          codeText: opener(codeText, buffer),
          codeTextData: onenterdata,
          data: onenterdata,
          codeFlowValue: onenterdata,
          definition: opener(definition),
          definitionDestinationString: buffer,
          definitionLabelString: buffer,
          definitionTitleString: buffer,
          emphasis: opener(emphasis),
          hardBreakEscape: opener(hardBreak),
          hardBreakTrailing: opener(hardBreak),
          htmlFlow: opener(html, buffer),
          htmlFlowData: onenterdata,
          htmlText: opener(html, buffer),
          htmlTextData: onenterdata,
          image: opener(image),
          label: buffer,
          link: opener(link),
          listItem: opener(listItem),
          listItemValue: onenterlistitemvalue,
          listOrdered: opener(list, onenterlistordered),
          listUnordered: opener(list),
          paragraph: opener(paragraph),
          reference: onenterreference,
          referenceString: buffer,
          resourceDestinationString: buffer,
          resourceTitleString: buffer,
          setextHeading: opener(heading),
          strong: opener(strong),
          thematicBreak: opener(thematicBreak)
        },
        exit: {
          atxHeading: closer(),
          atxHeadingSequence: onexitatxheadingsequence,
          autolink: closer(),
          autolinkEmail: onexitautolinkemail,
          autolinkProtocol: onexitautolinkprotocol,
          blockQuote: closer(),
          characterEscapeValue: onexitdata,
          characterReferenceMarkerHexadecimal: onexitcharacterreferencemarker,
          characterReferenceMarkerNumeric: onexitcharacterreferencemarker,
          characterReferenceValue: onexitcharacterreferencevalue,
          codeFenced: closer(onexitcodefenced),
          codeFencedFence: onexitcodefencedfence,
          codeFencedFenceInfo: onexitcodefencedfenceinfo,
          codeFencedFenceMeta: onexitcodefencedfencemeta,
          codeFlowValue: onexitdata,
          codeIndented: closer(onexitcodeindented),
          codeText: closer(onexitcodetext),
          codeTextData: onexitdata,
          data: onexitdata,
          definition: closer(),
          definitionDestinationString: onexitdefinitiondestinationstring,
          definitionLabelString: onexitdefinitionlabelstring,
          definitionTitleString: onexitdefinitiontitlestring,
          emphasis: closer(),
          hardBreakEscape: closer(onexithardbreak),
          hardBreakTrailing: closer(onexithardbreak),
          htmlFlow: closer(onexithtmlflow),
          htmlFlowData: onexitdata,
          htmlText: closer(onexithtmltext),
          htmlTextData: onexitdata,
          image: closer(onexitimage),
          label: onexitlabel,
          labelText: onexitlabeltext,
          lineEnding: onexitlineending,
          link: closer(onexitlink),
          listItem: closer(),
          listOrdered: closer(),
          listUnordered: closer(),
          paragraph: closer(),
          referenceString: onexitreferencestring,
          resourceDestinationString: onexitresourcedestinationstring,
          resourceTitleString: onexitresourcetitlestring,
          resource: onexitresource,
          setextHeading: closer(onexitsetextheading),
          setextHeadingLineSequence: onexitsetextheadinglinesequence,
          setextHeadingText: onexitsetextheadingtext,
          strong: closer(),
          thematicBreak: closer()
        }
      },
      options.mdastExtensions || []
    );
    /** @type {CompileData} */

    const data = {};
    return compile
    /**
     * @param {Array<Event>} events
     * @returns {Root}
     */

    function compile(events) {
      /** @type {Root} */
      let tree = {
        type: 'root',
        children: []
      };
      /** @type {CompileContext['stack']} */

      const stack = [tree];
      /** @type {CompileContext['tokenStack']} */

      const tokenStack = [];
      /** @type {Array<number>} */

      const listStack = [];
      /** @type {Omit<CompileContext, 'sliceSerialize'>} */

      const context = {
        stack,
        tokenStack,
        config,
        enter,
        exit,
        buffer,
        resume,
        setData,
        getData
      };
      let index = -1;

      while (++index < events.length) {
        // We preprocess lists to add `listItem` tokens, and to infer whether
        // items the list itself are spread out.
        if (
          events[index][1].type === 'listOrdered' ||
          events[index][1].type === 'listUnordered'
        ) {
          if (events[index][0] === 'enter') {
            listStack.push(index);
          } else {
            const tail = listStack.pop();
            index = prepareList(events, tail, index);
          }
        }
      }

      index = -1;

      while (++index < events.length) {
        const handler = config[events[index][0]];

        if (own$q.call(handler, events[index][1].type)) {
          handler[events[index][1].type].call(
            Object.assign(
              {
                sliceSerialize: events[index][2].sliceSerialize
              },
              context
            ),
            events[index][1]
          );
        }
      }

      if (tokenStack.length > 0) {
        const tail = tokenStack[tokenStack.length - 1];
        const handler = tail[1] || defaultOnError;
        handler.call(context, undefined, tail[0]);
      } // Figure out `root` position.

      tree.position = {
        start: point(
          events.length > 0
            ? events[0][1].start
            : {
                line: 1,
                column: 1,
                offset: 0
              }
        ),
        end: point(
          events.length > 0
            ? events[events.length - 2][1].end
            : {
                line: 1,
                column: 1,
                offset: 0
              }
        )
      };
      index = -1;

      while (++index < config.transforms.length) {
        tree = config.transforms[index](tree) || tree;
      }

      return tree
    }
    /**
     * @param {Array<Event>} events
     * @param {number} start
     * @param {number} length
     * @returns {number}
     */

    function prepareList(events, start, length) {
      let index = start - 1;
      let containerBalance = -1;
      let listSpread = false;
      /** @type {Token|undefined} */

      let listItem;
      /** @type {number|undefined} */

      let lineIndex;
      /** @type {number|undefined} */

      let firstBlankLineIndex;
      /** @type {boolean|undefined} */

      let atMarker;

      while (++index <= length) {
        const event = events[index];

        if (
          event[1].type === 'listUnordered' ||
          event[1].type === 'listOrdered' ||
          event[1].type === 'blockQuote'
        ) {
          if (event[0] === 'enter') {
            containerBalance++;
          } else {
            containerBalance--;
          }

          atMarker = undefined;
        } else if (event[1].type === 'lineEndingBlank') {
          if (event[0] === 'enter') {
            if (
              listItem &&
              !atMarker &&
              !containerBalance &&
              !firstBlankLineIndex
            ) {
              firstBlankLineIndex = index;
            }

            atMarker = undefined;
          }
        } else if (
          event[1].type === 'linePrefix' ||
          event[1].type === 'listItemValue' ||
          event[1].type === 'listItemMarker' ||
          event[1].type === 'listItemPrefix' ||
          event[1].type === 'listItemPrefixWhitespace'
        ) ; else {
          atMarker = undefined;
        }

        if (
          (!containerBalance &&
            event[0] === 'enter' &&
            event[1].type === 'listItemPrefix') ||
          (containerBalance === -1 &&
            event[0] === 'exit' &&
            (event[1].type === 'listUnordered' ||
              event[1].type === 'listOrdered'))
        ) {
          if (listItem) {
            let tailIndex = index;
            lineIndex = undefined;

            while (tailIndex--) {
              const tailEvent = events[tailIndex];

              if (
                tailEvent[1].type === 'lineEnding' ||
                tailEvent[1].type === 'lineEndingBlank'
              ) {
                if (tailEvent[0] === 'exit') continue

                if (lineIndex) {
                  events[lineIndex][1].type = 'lineEndingBlank';
                  listSpread = true;
                }

                tailEvent[1].type = 'lineEnding';
                lineIndex = tailIndex;
              } else if (
                tailEvent[1].type === 'linePrefix' ||
                tailEvent[1].type === 'blockQuotePrefix' ||
                tailEvent[1].type === 'blockQuotePrefixWhitespace' ||
                tailEvent[1].type === 'blockQuoteMarker' ||
                tailEvent[1].type === 'listItemIndent'
              ) ; else {
                break
              }
            }

            if (
              firstBlankLineIndex &&
              (!lineIndex || firstBlankLineIndex < lineIndex)
            ) {
              // @ts-expect-error Patched.
              listItem._spread = true;
            } // Fix position.

            listItem.end = Object.assign(
              {},
              lineIndex ? events[lineIndex][1].start : event[1].end
            );
            events.splice(lineIndex || index, 0, ['exit', listItem, event[2]]);
            index++;
            length++;
          } // Create a new list item.

          if (event[1].type === 'listItemPrefix') {
            listItem = {
              type: 'listItem',
              // @ts-expect-error Patched
              _spread: false,
              start: Object.assign({}, event[1].start)
            }; // @ts-expect-error: `listItem` is most definitely defined, TS...

            events.splice(index, 0, ['enter', listItem, event[2]]);
            index++;
            length++;
            firstBlankLineIndex = undefined;
            atMarker = true;
          }
        }
      } // @ts-expect-error Patched.

      events[start][1]._spread = listSpread;
      return length
    }
    /**
     * @type {CompileContext['setData']}
     * @param [value]
     */

    function setData(key, value) {
      data[key] = value;
    }
    /**
     * @type {CompileContext['getData']}
     * @template {string} K
     * @param {K} key
     * @returns {CompileData[K]}
     */

    function getData(key) {
      return data[key]
    }
    /**
     * @param {Point} d
     * @returns {Point}
     */

    function point(d) {
      return {
        line: d.line,
        column: d.column,
        offset: d.offset
      }
    }
    /**
     * @param {(token: Token) => Node} create
     * @param {Handle} [and]
     * @returns {Handle}
     */

    function opener(create, and) {
      return open
      /**
       * @this {CompileContext}
       * @param {Token} token
       * @returns {void}
       */

      function open(token) {
        enter.call(this, create(token), token);
        if (and) and.call(this, token);
      }
    }
    /** @type {CompileContext['buffer']} */

    function buffer() {
      this.stack.push({
        type: 'fragment',
        children: []
      });
    }
    /**
     * @type {CompileContext['enter']}
     * @template {Node} N
     * @this {CompileContext}
     * @param {N} node
     * @param {Token} token
     * @param {OnEnterError} [errorHandler]
     * @returns {N}
     */

    function enter(node, token, errorHandler) {
      const parent = this.stack[this.stack.length - 1];
      // @ts-expect-error: Assume `Node` can exist as a child of `parent`.
      parent.children.push(node);
      this.stack.push(node);
      this.tokenStack.push([token, errorHandler]); // @ts-expect-error: `end` will be patched later.

      node.position = {
        start: point(token.start)
      };
      return node
    }
    /**
     * @param {Handle} [and]
     * @returns {Handle}
     */

    function closer(and) {
      return close
      /**
       * @this {CompileContext}
       * @param {Token} token
       * @returns {void}
       */

      function close(token) {
        if (and) and.call(this, token);
        exit.call(this, token);
      }
    }
    /**
     * @type {CompileContext['exit']}
     * @this {CompileContext}
     * @param {Token} token
     * @param {OnExitError} [onExitError]
     * @returns {Node}
     */

    function exit(token, onExitError) {
      const node = this.stack.pop();
      const open = this.tokenStack.pop();

      if (!open) {
        throw new Error(
          'Cannot close `' +
            token.type +
            '` (' +
            stringifyPosition$1({
              start: token.start,
              end: token.end
            }) +
            '): it’s not open'
        )
      } else if (open[0].type !== token.type) {
        if (onExitError) {
          onExitError.call(this, token, open[0]);
        } else {
          const handler = open[1] || defaultOnError;
          handler.call(this, token, open[0]);
        }
      }

      node.position.end = point(token.end);
      return node
    }
    /**
     * @this {CompileContext}
     * @returns {string}
     */

    function resume() {
      return toString$1(this.stack.pop())
    } //
    // Handlers.
    //

    /** @type {Handle} */

    function onenterlistordered() {
      setData('expectingFirstListItemValue', true);
    }
    /** @type {Handle} */

    function onenterlistitemvalue(token) {
      if (getData('expectingFirstListItemValue')) {
        const ancestor =
          /** @type {List} */
          this.stack[this.stack.length - 2];
        ancestor.start = Number.parseInt(this.sliceSerialize(token), 10);
        setData('expectingFirstListItemValue');
      }
    }
    /** @type {Handle} */

    function onexitcodefencedfenceinfo() {
      const data = this.resume();
      const node =
        /** @type {Code} */
        this.stack[this.stack.length - 1];
      node.lang = data;
    }
    /** @type {Handle} */

    function onexitcodefencedfencemeta() {
      const data = this.resume();
      const node =
        /** @type {Code} */
        this.stack[this.stack.length - 1];
      node.meta = data;
    }
    /** @type {Handle} */

    function onexitcodefencedfence() {
      // Exit if this is the closing fence.
      if (getData('flowCodeInside')) return
      this.buffer();
      setData('flowCodeInside', true);
    }
    /** @type {Handle} */

    function onexitcodefenced() {
      const data = this.resume();
      const node =
        /** @type {Code} */
        this.stack[this.stack.length - 1];
      node.value = data.replace(/^(\r?\n|\r)|(\r?\n|\r)$/g, '');
      setData('flowCodeInside');
    }
    /** @type {Handle} */

    function onexitcodeindented() {
      const data = this.resume();
      const node =
        /** @type {Code} */
        this.stack[this.stack.length - 1];
      node.value = data.replace(/(\r?\n|\r)$/g, '');
    }
    /** @type {Handle} */

    function onexitdefinitionlabelstring(token) {
      // Discard label, use the source content instead.
      const label = this.resume();
      const node =
        /** @type {Definition} */
        this.stack[this.stack.length - 1];
      node.label = label;
      node.identifier = normalizeIdentifier(
        this.sliceSerialize(token)
      ).toLowerCase();
    }
    /** @type {Handle} */

    function onexitdefinitiontitlestring() {
      const data = this.resume();
      const node =
        /** @type {Definition} */
        this.stack[this.stack.length - 1];
      node.title = data;
    }
    /** @type {Handle} */

    function onexitdefinitiondestinationstring() {
      const data = this.resume();
      const node =
        /** @type {Definition} */
        this.stack[this.stack.length - 1];
      node.url = data;
    }
    /** @type {Handle} */

    function onexitatxheadingsequence(token) {
      const node =
        /** @type {Heading} */
        this.stack[this.stack.length - 1];

      if (!node.depth) {
        const depth = this.sliceSerialize(token).length;
        node.depth = depth;
      }
    }
    /** @type {Handle} */

    function onexitsetextheadingtext() {
      setData('setextHeadingSlurpLineEnding', true);
    }
    /** @type {Handle} */

    function onexitsetextheadinglinesequence(token) {
      const node =
        /** @type {Heading} */
        this.stack[this.stack.length - 1];
      node.depth = this.sliceSerialize(token).charCodeAt(0) === 61 ? 1 : 2;
    }
    /** @type {Handle} */

    function onexitsetextheading() {
      setData('setextHeadingSlurpLineEnding');
    }
    /** @type {Handle} */

    function onenterdata(token) {
      const parent =
        /** @type {Parent} */
        this.stack[this.stack.length - 1];
      /** @type {Node} */

      let tail = parent.children[parent.children.length - 1];

      if (!tail || tail.type !== 'text') {
        // Add a new text node.
        tail = text(); // @ts-expect-error: we’ll add `end` later.

        tail.position = {
          start: point(token.start)
        }; // @ts-expect-error: Assume `parent` accepts `text`.

        parent.children.push(tail);
      }

      this.stack.push(tail);
    }
    /** @type {Handle} */

    function onexitdata(token) {
      const tail = this.stack.pop();
      tail.value += this.sliceSerialize(token);
      tail.position.end = point(token.end);
    }
    /** @type {Handle} */

    function onexitlineending(token) {
      const context = this.stack[this.stack.length - 1];

      // If we’re at a hard break, include the line ending in there.
      if (getData('atHardBreak')) {
        const tail = context.children[context.children.length - 1];
        tail.position.end = point(token.end);
        setData('atHardBreak');
        return
      }

      if (
        !getData('setextHeadingSlurpLineEnding') &&
        config.canContainEols.includes(context.type)
      ) {
        onenterdata.call(this, token);
        onexitdata.call(this, token);
      }
    }
    /** @type {Handle} */

    function onexithardbreak() {
      setData('atHardBreak', true);
    }
    /** @type {Handle} */

    function onexithtmlflow() {
      const data = this.resume();
      const node =
        /** @type {HTML} */
        this.stack[this.stack.length - 1];
      node.value = data;
    }
    /** @type {Handle} */

    function onexithtmltext() {
      const data = this.resume();
      const node =
        /** @type {HTML} */
        this.stack[this.stack.length - 1];
      node.value = data;
    }
    /** @type {Handle} */

    function onexitcodetext() {
      const data = this.resume();
      const node =
        /** @type {InlineCode} */
        this.stack[this.stack.length - 1];
      node.value = data;
    }
    /** @type {Handle} */

    function onexitlink() {
      const context =
        /** @type {Link & {identifier: string, label: string}} */
        this.stack[this.stack.length - 1]; // To do: clean.

      if (getData('inReference')) {
        context.type += 'Reference'; // @ts-expect-error: mutate.

        context.referenceType = getData('referenceType') || 'shortcut'; // @ts-expect-error: mutate.

        delete context.url;
        delete context.title;
      } else {
        // @ts-expect-error: mutate.
        delete context.identifier; // @ts-expect-error: mutate.

        delete context.label;
      }

      setData('referenceType');
    }
    /** @type {Handle} */

    function onexitimage() {
      const context =
        /** @type {Image & {identifier: string, label: string}} */
        this.stack[this.stack.length - 1]; // To do: clean.

      if (getData('inReference')) {
        context.type += 'Reference'; // @ts-expect-error: mutate.

        context.referenceType = getData('referenceType') || 'shortcut'; // @ts-expect-error: mutate.

        delete context.url;
        delete context.title;
      } else {
        // @ts-expect-error: mutate.
        delete context.identifier; // @ts-expect-error: mutate.

        delete context.label;
      }

      setData('referenceType');
    }
    /** @type {Handle} */

    function onexitlabeltext(token) {
      const ancestor =
        /** @type {(Link|Image) & {identifier: string, label: string}} */
        this.stack[this.stack.length - 2];
      const string = this.sliceSerialize(token);
      ancestor.label = decodeString(string);
      ancestor.identifier = normalizeIdentifier(string).toLowerCase();
    }
    /** @type {Handle} */

    function onexitlabel() {
      const fragment =
        /** @type {Fragment} */
        this.stack[this.stack.length - 1];
      const value = this.resume();
      const node =
        /** @type {(Link|Image) & {identifier: string, label: string}} */
        this.stack[this.stack.length - 1]; // Assume a reference.

      setData('inReference', true);

      if (node.type === 'link') {
        // @ts-expect-error: Assume static phrasing content.
        node.children = fragment.children;
      } else {
        node.alt = value;
      }
    }
    /** @type {Handle} */

    function onexitresourcedestinationstring() {
      const data = this.resume();
      const node =
        /** @type {Link|Image} */
        this.stack[this.stack.length - 1];
      node.url = data;
    }
    /** @type {Handle} */

    function onexitresourcetitlestring() {
      const data = this.resume();
      const node =
        /** @type {Link|Image} */
        this.stack[this.stack.length - 1];
      node.title = data;
    }
    /** @type {Handle} */

    function onexitresource() {
      setData('inReference');
    }
    /** @type {Handle} */

    function onenterreference() {
      setData('referenceType', 'collapsed');
    }
    /** @type {Handle} */

    function onexitreferencestring(token) {
      const label = this.resume();
      const node =
        /** @type {LinkReference|ImageReference} */
        this.stack[this.stack.length - 1];
      node.label = label;
      node.identifier = normalizeIdentifier(
        this.sliceSerialize(token)
      ).toLowerCase();
      setData('referenceType', 'full');
    }
    /** @type {Handle} */

    function onexitcharacterreferencemarker(token) {
      setData('characterReferenceType', token.type);
    }
    /** @type {Handle} */

    function onexitcharacterreferencevalue(token) {
      const data = this.sliceSerialize(token);
      const type = getData('characterReferenceType');
      /** @type {string} */

      let value;

      if (type) {
        value = decodeNumericCharacterReference(
          data,
          type === 'characterReferenceMarkerNumeric' ? 10 : 16
        );
        setData('characterReferenceType');
      } else {
        // @ts-expect-error `decodeNamedCharacterReference` can return false for
        // invalid named character references, but everything we’ve tokenized is
        // valid.
        value = decodeNamedCharacterReference(data);
      }

      const tail = this.stack.pop();
      tail.value += value;
      tail.position.end = point(token.end);
    }
    /** @type {Handle} */

    function onexitautolinkprotocol(token) {
      onexitdata.call(this, token);
      const node =
        /** @type {Link} */
        this.stack[this.stack.length - 1];
      node.url = this.sliceSerialize(token);
    }
    /** @type {Handle} */

    function onexitautolinkemail(token) {
      onexitdata.call(this, token);
      const node =
        /** @type {Link} */
        this.stack[this.stack.length - 1];
      node.url = 'mailto:' + this.sliceSerialize(token);
    } //
    // Creaters.
    //

    /** @returns {Blockquote} */

    function blockQuote() {
      return {
        type: 'blockquote',
        children: []
      }
    }
    /** @returns {Code} */

    function codeFlow() {
      return {
        type: 'code',
        lang: null,
        meta: null,
        value: ''
      }
    }
    /** @returns {InlineCode} */

    function codeText() {
      return {
        type: 'inlineCode',
        value: ''
      }
    }
    /** @returns {Definition} */

    function definition() {
      return {
        type: 'definition',
        identifier: '',
        label: null,
        title: null,
        url: ''
      }
    }
    /** @returns {Emphasis} */

    function emphasis() {
      return {
        type: 'emphasis',
        children: []
      }
    }
    /** @returns {Heading} */

    function heading() {
      // @ts-expect-error `depth` will be set later.
      return {
        type: 'heading',
        depth: undefined,
        children: []
      }
    }
    /** @returns {Break} */

    function hardBreak() {
      return {
        type: 'break'
      }
    }
    /** @returns {HTML} */

    function html() {
      return {
        type: 'html',
        value: ''
      }
    }
    /** @returns {Image} */

    function image() {
      return {
        type: 'image',
        title: null,
        url: '',
        alt: null
      }
    }
    /** @returns {Link} */

    function link() {
      return {
        type: 'link',
        title: null,
        url: '',
        children: []
      }
    }
    /**
     * @param {Token} token
     * @returns {List}
     */

    function list(token) {
      return {
        type: 'list',
        ordered: token.type === 'listOrdered',
        start: null,
        // @ts-expect-error Patched.
        spread: token._spread,
        children: []
      }
    }
    /**
     * @param {Token} token
     * @returns {ListItem}
     */

    function listItem(token) {
      return {
        type: 'listItem',
        // @ts-expect-error Patched.
        spread: token._spread,
        checked: null,
        children: []
      }
    }
    /** @returns {Paragraph} */

    function paragraph() {
      return {
        type: 'paragraph',
        children: []
      }
    }
    /** @returns {Strong} */

    function strong() {
      return {
        type: 'strong',
        children: []
      }
    }
    /** @returns {Text} */

    function text() {
      return {
        type: 'text',
        value: ''
      }
    }
    /** @returns {ThematicBreak} */

    function thematicBreak() {
      return {
        type: 'thematicBreak'
      }
    }
  }
  /**
   * @param {Extension} combined
   * @param {Array<Extension|Array<Extension>>} extensions
   * @returns {Extension}
   */

  function configure(combined, extensions) {
    let index = -1;

    while (++index < extensions.length) {
      const value = extensions[index];

      if (Array.isArray(value)) {
        configure(combined, value);
      } else {
        extension(combined, value);
      }
    }

    return combined
  }
  /**
   * @param {Extension} combined
   * @param {Extension} extension
   * @returns {void}
   */

  function extension(combined, extension) {
    /** @type {string} */
    let key;

    for (key in extension) {
      if (own$q.call(extension, key)) {
        const list = key === 'canContainEols' || key === 'transforms';
        const maybe = own$q.call(combined, key) ? combined[key] : undefined;
        /* c8 ignore next */

        const left = maybe || (combined[key] = list ? [] : {});
        const right = extension[key];

        if (right) {
          if (list) {
            // @ts-expect-error: `left` is an array.
            combined[key] = [...left, ...right];
          } else {
            Object.assign(left, right);
          }
        }
      }
    }
  }
  /** @type {OnEnterError} */

  function defaultOnError(left, right) {
    if (left) {
      throw new Error(
        'Cannot close `' +
          left.type +
          '` (' +
          stringifyPosition$1({
            start: left.start,
            end: left.end
          }) +
          '): a different token (`' +
          right.type +
          '`, ' +
          stringifyPosition$1({
            start: right.start,
            end: right.end
          }) +
          ') is open'
      )
    } else {
      throw new Error(
        'Cannot close document, a token (`' +
          right.type +
          '`, ' +
          stringifyPosition$1({
            start: right.start,
            end: right.end
          }) +
          ') is still open'
      )
    }
  }

  /**
   * @typedef {import('mdast').Root} Root
   * @typedef {import('mdast-util-from-markdown').Options} Options
   */

  /** @type {import('unified').Plugin<[Options?] | void[], string, Root>} */
  function remarkParse(options) {
    /** @type {import('unified').ParserFunction<Root>} */
    const parser = (doc) => {
      // Assume options.
      const settings = /** @type {Options} */ (this.data('settings'));

      return fromMarkdown(
        doc,
        Object.assign({}, settings, options, {
          // Note: these options are not in the readme.
          // The goal is for them to be set by plugins on `data` instead of being
          // passed by users.
          extensions: this.data('micromarkExtensions') || [],
          mdastExtensions: this.data('fromMarkdownExtensions') || []
        })
      )
    };

    Object.assign(this, {Parser: parser});
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   * @typedef {import('unist').Literal} Literal
   * @typedef {Object.<string, unknown>} Props
   * @typedef {Array.<Node>|string} ChildrenOrValue
   *
   * @typedef {(<T extends string, P extends Record<string, unknown>, C extends Node[]>(type: T, props: P, children: C) => {type: T, children: C} & P)} BuildParentWithProps
   * @typedef {(<T extends string, P extends Record<string, unknown>>(type: T, props: P, value: string) => {type: T, value: string} & P)} BuildLiteralWithProps
   * @typedef {(<T extends string, P extends Record<string, unknown>>(type: T, props: P) => {type: T} & P)} BuildVoidWithProps
   * @typedef {(<T extends string, C extends Node[]>(type: T, children: C) => {type: T, children: C})} BuildParent
   * @typedef {(<T extends string>(type: T, value: string) => {type: T, value: string})} BuildLiteral
   * @typedef {(<T extends string>(type: T) => {type: T})} BuildVoid
   */

  var u$1 = /**
   * @type {BuildVoid & BuildVoidWithProps & BuildLiteral & BuildLiteralWithProps & BuildParent & BuildParentWithProps}
   */ (
    /**
     * @param {string} type Type of node
     * @param {Props|ChildrenOrValue} [props] Additional properties for node (or `children` or `value`)
     * @param {ChildrenOrValue} [value] `children` or `value` of node
     * @returns {Node}
     */
    function (type, props, value) {
      /** @type {Node} */
      var node = {type: String(type)};

      if (
        (value === undefined || value === null) &&
        (typeof props === 'string' || Array.isArray(props))
      ) {
        value = props;
      } else {
        Object.assign(node, props);
      }

      if (Array.isArray(value)) {
        node.children = value;
      } else if (value !== undefined && value !== null) {
        node.value = String(value);
      }

      return node
    }
  );

  /**
   * @typedef {import('mdast').Root|import('mdast').Parent['children'][number]} MdastNode
   * @typedef {import('./index.js').H} H
   * @typedef {import('./index.js').Handler} Handler
   * @typedef {import('./index.js').Content} Content
   */

  const own$p = {}.hasOwnProperty;

  /**
   * Transform an unknown node.
   * @type {Handler}
   * @param {MdastNode} node
   */
  function unknown$1(h, node) {
    const data = node.data || {};

    if (
      'value' in node &&
      !(
        own$p.call(data, 'hName') ||
        own$p.call(data, 'hProperties') ||
        own$p.call(data, 'hChildren')
      )
    ) {
      return h.augment(node, u$1('text', node.value))
    }

    return h(node, 'div', all$3(h, node))
  }

  /**
   * @type {Handler}
   * @param {MdastNode} node
   */
  function one$3(h, node, parent) {
    const type = node && node.type;
    /** @type {Handler} */
    let fn;

    // Fail on non-nodes.
    if (!type) {
      throw new Error('Expected node, got `' + node + '`')
    }

    if (own$p.call(h.handlers, type)) {
      fn = h.handlers[type];
    } else if (h.passThrough && h.passThrough.includes(type)) {
      fn = returnNode;
    } else {
      fn = h.unknownHandler;
    }

    return (typeof fn === 'function' ? fn : unknown$1)(h, node, parent)
  }

  /**
   * @type {Handler}
   * @param {MdastNode} node
   */
  function returnNode(h, node) {
    // @ts-expect-error: Pass through custom node.
    return 'children' in node ? {...node, children: all$3(h, node)} : node
  }

  /**
   * @param {H} h
   * @param {MdastNode} parent
   */
  function all$3(h, parent) {
    /** @type {Array<Content>} */
    const values = [];

    if ('children' in parent) {
      const nodes = parent.children;
      let index = -1;

      while (++index < nodes.length) {
        const result = one$3(h, nodes[index], parent);

        if (result) {
          if (index && nodes[index - 1].type === 'break') {
            if (!Array.isArray(result) && result.type === 'text') {
              result.value = result.value.replace(/^\s+/, '');
            }

            if (!Array.isArray(result) && result.type === 'element') {
              const head = result.children[0];

              if (head && head.type === 'text') {
                head.value = head.value.replace(/^\s+/, '');
              }
            }
          }

          if (Array.isArray(result)) {
            values.push(...result);
          } else {
            values.push(result);
          }
        }
      }
    }

    return values
  }

  /**
   * @typedef {import('unist').Position} Position
   * @typedef {import('unist').Point} Point
   *
   * @typedef {Partial<Point>} PointLike
   *
   * @typedef {Object} PositionLike
   * @property {PointLike} [start]
   * @property {PointLike} [end]
   *
   * @typedef {Object} NodeLike
   * @property {PositionLike} [position]
   */

  var pointStart = point$3('start');
  var pointEnd = point$3('end');

  /**
   * Get the positional info of `node`.
   *
   * @param {'start'|'end'} type
   */
  function point$3(type) {
    return point

    /**
     * Get the positional info of `node`.
     *
     * @param {NodeLike} [node]
     * @returns {Point}
     */
    function point(node) {
      /** @type {Point} */
      // @ts-ignore looks like a point
      var point = (node && node.position && node.position[type]) || {};

      return {
        line: point.line || null,
        column: point.column || null,
        offset: point.offset > -1 ? point.offset : null
      }
    }
  }

  /**
   * @typedef {Object} PointLike
   * @property {number} [line]
   * @property {number} [column]
   * @property {number} [offset]
   *
   * @typedef {Object} PositionLike
   * @property {PointLike} [start]
   * @property {PointLike} [end]
   *
   * @typedef {Object} NodeLike
   * @property {PositionLike} [position]
   */

  /**
   * Check if `node` is *generated*.
   *
   * @param {NodeLike} [node]
   * @returns {boolean}
   */
  function generated(node) {
    return (
      !node ||
      !node.position ||
      !node.position.start ||
      !node.position.start.line ||
      !node.position.start.column ||
      !node.position.end ||
      !node.position.end.line ||
      !node.position.end.column
    )
  }

  /**
   * @param {string} d
   * @returns {string}
   */
  function color$1(d) {
    return d
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   * @typedef {import('unist-util-is').Test} Test
   */

  /**
   * Continue traversing as normal
   */
  const CONTINUE$1 = true;
  /**
   * Do not traverse this node’s children
   */
  const SKIP$1 = 'skip';
  /**
   * Stop traversing immediately
   */
  const EXIT$1 = false;

  const visitParents$1 =
    /**
     * @type {(
     *   (<T extends Node>(tree: Node, test: T['type']|Partial<T>|import('unist-util-is').TestFunctionPredicate<T>|Array.<T['type']|Partial<T>|import('unist-util-is').TestFunctionPredicate<T>>, visitor: Visitor<T>, reverse?: boolean) => void) &
     *   ((tree: Node, test: Test, visitor: Visitor<Node>, reverse?: boolean) => void) &
     *   ((tree: Node, visitor: Visitor<Node>, reverse?: boolean) => void)
     * )}
     */
    (
      /**
       * Visit children of tree which pass a test
       *
       * @param {Node} tree Abstract syntax tree to walk
       * @param {Test} test test Test node
       * @param {Visitor<Node>} visitor Function to run for each node
       * @param {boolean} [reverse] Fisit the tree in reverse, defaults to false
       */
      function (tree, test, visitor, reverse) {
        if (typeof test === 'function' && typeof visitor !== 'function') {
          reverse = visitor;
          // @ts-ignore no visitor given, so `visitor` is test.
          visitor = test;
          test = null;
        }

        var is = convert$1(test);
        var step = reverse ? -1 : 1;

        factory(tree, null, [])();

        /**
         * @param {Node} node
         * @param {number?} index
         * @param {Array.<Parent>} parents
         */
        function factory(node, index, parents) {
          /** @type {Object.<string, unknown>} */
          var value = typeof node === 'object' && node !== null ? node : {};
          /** @type {string} */
          var name;

          if (typeof value.type === 'string') {
            name =
              typeof value.tagName === 'string'
                ? value.tagName
                : typeof value.name === 'string'
                ? value.name
                : undefined;

            Object.defineProperty(visit, 'name', {
              value:
                'node (' +
                color$1(value.type + (name ? '<' + name + '>' : '')) +
                ')'
            });
          }

          return visit

          function visit() {
            /** @type {ActionTuple} */
            var result = [];
            /** @type {ActionTuple} */
            var subresult;
            /** @type {number} */
            var offset;
            /** @type {Array.<Parent>} */
            var grandparents;

            if (!test || is(node, index, parents[parents.length - 1] || null)) {
              result = toResult$1(visitor(node, parents));

              if (result[0] === EXIT$1) {
                return result
              }
            }

            if (node.children && result[0] !== SKIP$1) {
              // @ts-ignore looks like a parent.
              offset = (reverse ? node.children.length : -1) + step;
              // @ts-ignore looks like a parent.
              grandparents = parents.concat(node);

              // @ts-ignore looks like a parent.
              while (offset > -1 && offset < node.children.length) {
                subresult = factory(node.children[offset], offset, grandparents)();

                if (subresult[0] === EXIT$1) {
                  return subresult
                }

                offset =
                  typeof subresult[1] === 'number' ? subresult[1] : offset + step;
              }
            }

            return result
          }
        }
      }
    );

  /**
   * @param {VisitorResult} value
   * @returns {ActionTuple}
   */
  function toResult$1(value) {
    if (Array.isArray(value)) {
      return value
    }

    if (typeof value === 'number') {
      return [CONTINUE$1, value]
    }

    return [value]
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   * @typedef {import('unist-util-is').Test} Test
   * @typedef {import('unist-util-visit-parents').VisitorResult} VisitorResult
   */

  const visit$1 =
    /**
     * @type {(
     *   (<T extends Node>(tree: Node, test: T['type']|Partial<T>|import('unist-util-is').TestFunctionPredicate<T>|Array.<T['type']|Partial<T>|import('unist-util-is').TestFunctionPredicate<T>>, visitor: Visitor<T>, reverse?: boolean) => void) &
     *   ((tree: Node, test: Test, visitor: Visitor<Node>, reverse?: boolean) => void) &
     *   ((tree: Node, visitor: Visitor<Node>, reverse?: boolean) => void)
     * )}
     */
    (
      /**
       * Visit children of tree which pass a test
       *
       * @param {Node} tree Abstract syntax tree to walk
       * @param {Test} test test Test node
       * @param {Visitor<Node>} visitor Function to run for each node
       * @param {boolean} [reverse] Fisit the tree in reverse, defaults to false
       */
      function (tree, test, visitor, reverse) {
        if (typeof test === 'function' && typeof visitor !== 'function') {
          reverse = visitor;
          visitor = test;
          test = null;
        }

        visitParents$1(tree, test, overload, reverse);

        /**
         * @param {Node} node
         * @param {Array.<Parent>} parents
         */
        function overload(node, parents) {
          var parent = parents[parents.length - 1];
          return visitor(
            node,
            parent ? parent.children.indexOf(node) : null,
            parent
          )
        }
      }
    );

  /**
   * @typedef {import('mdast').Root|import('mdast').Content} Node
   * @typedef {import('mdast').Definition} Definition
   * @typedef {import('unist-util-visit').Visitor<Definition>} DefinitionVisitor
   */

  const own$o = {}.hasOwnProperty;

  /**
   *
   * @param {Node} node
   */
  function definitions(node) {
    /** @type {Object.<string, Definition>} */
    const cache = Object.create(null);

    if (!node || !node.type) {
      throw new Error('mdast-util-definitions expected node')
    }

    visit$1(node, 'definition', ondefinition);

    return getDefinition

    /** @type {DefinitionVisitor} */
    function ondefinition(definition) {
      const id = clean(definition.identifier);
      if (id && !own$o.call(cache, id)) {
        cache[id] = definition;
      }
    }

    /**
     * Get a node from the bound definition-cache.
     *
     * @param {string} identifier
     * @returns {Definition|null}
     */
    function getDefinition(identifier) {
      const id = clean(identifier);
      return id && own$o.call(cache, id) ? cache[id] : null
    }
  }

  /**
   * @param {string} [value]
   * @returns {string}
   */
  function clean(value) {
    return String(value || '').toUpperCase()
  }

  /**
   * @typedef {import('./index.js').Content} Content
   */

  /**
   * Wrap `nodes` with line feeds between each entry.
   * Optionally adds line feeds at the start and end.
   *
   * @param {Array<Content>} nodes
   * @param {boolean} [loose=false]
   * @returns {Array<Content>}
   */
  function wrap$1(nodes, loose) {
    /** @type {Array<Content>} */
    const result = [];
    let index = -1;

    if (loose) {
      result.push(u$1('text', '\n'));
    }

    while (++index < nodes.length) {
      if (index) result.push(u$1('text', '\n'));
      result.push(nodes[index]);
    }

    if (loose && nodes.length > 0) {
      result.push(u$1('text', '\n'));
    }

    return result
  }

  /**
   * @typedef {import('mdast').BlockContent} BlockContent
   * @typedef {import('mdast').FootnoteDefinition} FootnoteDefinition
   * @typedef {import('hast').Element} Element
   * @typedef {import('hast').ElementContent} ElementContent
   * @typedef {import('./index.js').H} H
   */

  /**
   * @param {H} h
   */
  function footer(h) {
    let index = -1;
    /** @type {Array<ElementContent>} */
    const listItems = [];

    while (++index < h.footnoteOrder.length) {
      const def = h.footnoteById[h.footnoteOrder[index].toUpperCase()];

      if (!def) {
        continue
      }

      const content = all$3(h, def);
      const id = String(def.identifier);
      const safeId = sanitizeUri(id.toLowerCase());
      let referenceIndex = 0;
      /** @type {Array<ElementContent>} */
      const backReferences = [];

      while (++referenceIndex <= h.footnoteCounts[id]) {
        /** @type {Element} */
        const backReference = {
          type: 'element',
          tagName: 'a',
          properties: {
            href:
              '#' +
              h.clobberPrefix +
              'fnref-' +
              safeId +
              (referenceIndex > 1 ? '-' + referenceIndex : ''),
            dataFootnoteBackref: true,
            className: ['data-footnote-backref'],
            ariaLabel: h.footnoteBackLabel
          },
          children: [{type: 'text', value: '↩'}]
        };

        if (referenceIndex > 1) {
          backReference.children.push({
            type: 'element',
            tagName: 'sup',
            children: [{type: 'text', value: String(referenceIndex)}]
          });
        }

        if (backReferences.length > 0) {
          backReferences.push({type: 'text', value: ' '});
        }

        backReferences.push(backReference);
      }

      const tail = content[content.length - 1];

      if (tail && tail.type === 'element' && tail.tagName === 'p') {
        const tailTail = tail.children[tail.children.length - 1];
        if (tailTail && tailTail.type === 'text') {
          tailTail.value += ' ';
        } else {
          tail.children.push({type: 'text', value: ' '});
        }

        tail.children.push(...backReferences);
      } else {
        content.push(...backReferences);
      }

      /** @type {Element} */
      const listItem = {
        type: 'element',
        tagName: 'li',
        properties: {id: h.clobberPrefix + 'fn-' + safeId},
        children: wrap$1(content, true)
      };

      if (def.position) {
        listItem.position = def.position;
      }

      listItems.push(listItem);
    }

    if (listItems.length === 0) {
      return null
    }

    return {
      type: 'element',
      tagName: 'section',
      properties: {dataFootnotes: true, className: ['footnotes']},
      children: [
        {
          type: 'element',
          tagName: 'h2',
          properties: {id: 'footnote-label', className: ['sr-only']},
          children: [u$1('text', h.footnoteLabel)]
        },
        {type: 'text', value: '\n'},
        {
          type: 'element',
          tagName: 'ol',
          properties: {},
          children: wrap$1(listItems, true)
        },
        {type: 'text', value: '\n'}
      ]
    }
  }

  /**
   * @typedef {import('mdast').Blockquote} Blockquote
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {Blockquote} node
   */
  function blockquote(h, node) {
    return h(node, 'blockquote', wrap$1(all$3(h, node), true))
  }

  /**
   * @typedef {import('hast').Element} Element
   * @typedef {import('hast').Text} Text
   * @typedef {import('mdast').Break} Break
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {Break} node
   * @returns {Array<Element|Text>}
   */
  function hardBreak(h, node) {
    return [h(node, 'br'), u$1('text', '\n')]
  }

  /**
   * @typedef {import('mdast').Code} Code
   * @typedef {import('hast').Element} Element
   * @typedef {import('hast').Properties} Properties
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {Code} node
   */
  function code(h, node) {
    const value = node.value ? node.value + '\n' : '';
    // To do: next major, use `node.lang` w/o regex, the splitting’s been going
    // on for years in remark now.
    const lang = node.lang && node.lang.match(/^[^ \t]+(?=[ \t]|$)/);
    /** @type {Properties} */
    const props = {};

    if (lang) {
      props.className = ['language-' + lang];
    }

    const code = h(node, 'code', props, [u$1('text', value)]);

    if (node.meta) {
      code.data = {meta: node.meta};
    }

    return h(node.position, 'pre', [code])
  }

  /**
   * @typedef {import('mdast').Delete} Delete
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {Delete} node
   */
  function strikethrough(h, node) {
    return h(node, 'del', all$3(h, node))
  }

  /**
   * @typedef {import('mdast').Emphasis} Emphasis
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {Emphasis} node
   */
  function emphasis(h, node) {
    return h(node, 'em', all$3(h, node))
  }

  /**
   * @typedef {import('mdast').FootnoteReference} FootnoteReference
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {FootnoteReference} node
   */
  function footnoteReference(h, node) {
    const id = String(node.identifier);
    const safeId = sanitizeUri(id.toLowerCase());
    const index = h.footnoteOrder.indexOf(id);
    /** @type {number} */
    let counter;

    if (index === -1) {
      h.footnoteOrder.push(id);
      h.footnoteCounts[id] = 1;
      counter = h.footnoteOrder.length;
    } else {
      h.footnoteCounts[id]++;
      counter = index + 1;
    }

    const reuseCounter = h.footnoteCounts[id];

    return h(node, 'sup', [
      h(
        node.position,
        'a',
        {
          href: '#' + h.clobberPrefix + 'fn-' + safeId,
          id:
            h.clobberPrefix +
            'fnref-' +
            safeId +
            (reuseCounter > 1 ? '-' + reuseCounter : ''),
          dataFootnoteRef: true,
          ariaDescribedBy: 'footnote-label'
        },
        [u$1('text', String(counter))]
      )
    ])
  }

  /**
   * @typedef {import('mdast').Footnote} Footnote
   * @typedef {import('../index.js').Handler} Handler
   *
   * @todo
   *   `footnote` (or “inline note”) are a pandoc footnotes feature (`^[a note]`)
   *   that does not exist in GFM.
   *   We still have support for it, so that things remain working with
   *   `micromark-extension-footnote` and `mdast-util-footnote`, but in the future
   *   we might be able to remove it?
   */

  /**
   * @type {Handler}
   * @param {Footnote} node
   */
  function footnote(h, node) {
    const footnoteById = h.footnoteById;
    let no = 1;

    while (no in footnoteById) no++;

    const identifier = String(no);

    footnoteById[identifier] = {
      type: 'footnoteDefinition',
      identifier,
      children: [{type: 'paragraph', children: node.children}],
      position: node.position
    };

    return footnoteReference(h, {
      type: 'footnoteReference',
      identifier,
      position: node.position
    })
  }

  /**
   * @typedef {import('mdast').Heading} Heading
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {Heading} node
   */
  function heading(h, node) {
    return h(node, 'h' + node.depth, all$3(h, node))
  }

  /**
   * @typedef {import('mdast').HTML} HTML
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * Return either a `raw` node in dangerous mode, otherwise nothing.
   *
   * @type {Handler}
   * @param {HTML} node
   */
  function html$i(h, node) {
    return h.dangerous ? h.augment(node, u$1('raw', node.value)) : null
  }

  var encodeCache = {};


  // Create a lookup array where anything but characters in `chars` string
  // and alphanumeric chars is percent-encoded.
  //
  function getEncodeCache(exclude) {
    var i, ch, cache = encodeCache[exclude];
    if (cache) { return cache; }

    cache = encodeCache[exclude] = [];

    for (i = 0; i < 128; i++) {
      ch = String.fromCharCode(i);

      if (/^[0-9a-z]$/i.test(ch)) {
        // always allow unencoded alphanumeric characters
        cache.push(ch);
      } else {
        cache.push('%' + ('0' + i.toString(16).toUpperCase()).slice(-2));
      }
    }

    for (i = 0; i < exclude.length; i++) {
      cache[exclude.charCodeAt(i)] = exclude[i];
    }

    return cache;
  }


  // Encode unsafe characters with percent-encoding, skipping already
  // encoded sequences.
  //
  //  - string       - string to encode
  //  - exclude      - list of characters to ignore (in addition to a-zA-Z0-9)
  //  - keepEscaped  - don't encode '%' in a correct escape sequence (default: true)
  //
  function encode$1(string, exclude, keepEscaped) {
    var i, l, code, nextCode, cache,
        result = '';

    if (typeof exclude !== 'string') {
      // encode(string, keepEscaped)
      keepEscaped  = exclude;
      exclude = encode$1.defaultChars;
    }

    if (typeof keepEscaped === 'undefined') {
      keepEscaped = true;
    }

    cache = getEncodeCache(exclude);

    for (i = 0, l = string.length; i < l; i++) {
      code = string.charCodeAt(i);

      if (keepEscaped && code === 0x25 /* % */ && i + 2 < l) {
        if (/^[0-9a-f]{2}$/i.test(string.slice(i + 1, i + 3))) {
          result += string.slice(i, i + 3);
          i += 2;
          continue;
        }
      }

      if (code < 128) {
        result += cache[code];
        continue;
      }

      if (code >= 0xD800 && code <= 0xDFFF) {
        if (code >= 0xD800 && code <= 0xDBFF && i + 1 < l) {
          nextCode = string.charCodeAt(i + 1);
          if (nextCode >= 0xDC00 && nextCode <= 0xDFFF) {
            result += encodeURIComponent(string[i] + string[i + 1]);
            i++;
            continue;
          }
        }
        result += '%EF%BF%BD';
        continue;
      }

      result += encodeURIComponent(string[i]);
    }

    return result;
  }

  encode$1.defaultChars   = ";/?:@&=+$,-_.!~*'()#";
  encode$1.componentChars = "-_.!~*'()";


  var encode_1 = encode$1;

  /**
   * @typedef {import('mdast').LinkReference} LinkReference
   * @typedef {import('mdast').ImageReference} ImageReference
   * @typedef {import('./index.js').Handler} Handler
   * @typedef {import('./index.js').Content} Content
   */

  /**
   * Return the content of a reference without definition as plain text.
   *
   * @type {Handler}
   * @param {ImageReference|LinkReference} node
   * @returns {Content|Array<Content>}
   */
  function revert(h, node) {
    const subtype = node.referenceType;
    let suffix = ']';

    if (subtype === 'collapsed') {
      suffix += '[]';
    } else if (subtype === 'full') {
      suffix += '[' + (node.label || node.identifier) + ']';
    }

    if (node.type === 'imageReference') {
      return u$1('text', '![' + node.alt + suffix)
    }

    const contents = all$3(h, node);
    const head = contents[0];

    if (head && head.type === 'text') {
      head.value = '[' + head.value;
    } else {
      contents.unshift(u$1('text', '['));
    }

    const tail = contents[contents.length - 1];

    if (tail && tail.type === 'text') {
      tail.value += suffix;
    } else {
      contents.push(u$1('text', suffix));
    }

    return contents
  }

  /**
   * @typedef {import('mdast').ImageReference} ImageReference
   * @typedef {import('hast').Properties} Properties
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {ImageReference} node
   */
  function imageReference(h, node) {
    const def = h.definition(node.identifier);

    if (!def) {
      return revert(h, node)
    }

    /** @type {Properties} */
    const props = {src: encode_1(def.url || ''), alt: node.alt};

    if (def.title !== null && def.title !== undefined) {
      props.title = def.title;
    }

    return h(node, 'img', props)
  }

  /**
   * @typedef {import('mdast').Image} Image
   * @typedef {import('hast').Properties} Properties
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {Image} node
   */
  function image(h, node) {
    /** @type {Properties} */
    const props = {src: encode_1(node.url), alt: node.alt};

    if (node.title !== null && node.title !== undefined) {
      props.title = node.title;
    }

    return h(node, 'img', props)
  }

  /**
   * @typedef {import('mdast').InlineCode} InlineCode
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {InlineCode} node
   */
  function inlineCode(h, node) {
    return h(node, 'code', [u$1('text', node.value.replace(/\r?\n|\r/g, ' '))])
  }

  /**
   * @typedef {import('mdast').LinkReference} LinkReference
   * @typedef {import('hast').Properties} Properties
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {LinkReference} node
   */
  function linkReference(h, node) {
    const def = h.definition(node.identifier);

    if (!def) {
      return revert(h, node)
    }

    /** @type {Properties} */
    const props = {href: encode_1(def.url || '')};

    if (def.title !== null && def.title !== undefined) {
      props.title = def.title;
    }

    return h(node, 'a', props, all$3(h, node))
  }

  /**
   * @typedef {import('mdast').Link} Link
   * @typedef {import('hast').Properties} Properties
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {Link} node
   */
  function link(h, node) {
    /** @type {Properties} */
    const props = {href: encode_1(node.url)};

    if (node.title !== null && node.title !== undefined) {
      props.title = node.title;
    }

    return h(node, 'a', props, all$3(h, node))
  }

  /**
   * @typedef {import('mdast').ListItem} ListItem
   * @typedef {import('mdast').List} List
   * @typedef {import('hast').Properties} Properties
   * @typedef {import('hast').Element} Element
   * @typedef {import('../index.js').Handler} Handler
   * @typedef {import('../index.js').Content} Content
   */

  /**
   * @type {Handler}
   * @param {ListItem} node
   * @param {List} parent
   */
  function listItem(h, node, parent) {
    const result = all$3(h, node);
    const loose = parent ? listLoose(parent) : listItemLoose(node);
    /** @type {Properties} */
    const props = {};
    /** @type {Array<Content>} */
    const wrapped = [];

    if (typeof node.checked === 'boolean') {
      /** @type {Element} */
      let paragraph;

      if (
        result[0] &&
        result[0].type === 'element' &&
        result[0].tagName === 'p'
      ) {
        paragraph = result[0];
      } else {
        paragraph = h(null, 'p', []);
        result.unshift(paragraph);
      }

      if (paragraph.children.length > 0) {
        paragraph.children.unshift(u$1('text', ' '));
      }

      paragraph.children.unshift(
        h(null, 'input', {
          type: 'checkbox',
          checked: node.checked,
          disabled: true
        })
      );

      // According to github-markdown-css, this class hides bullet.
      // See: <https://github.com/sindresorhus/github-markdown-css>.
      props.className = ['task-list-item'];
    }

    let index = -1;

    while (++index < result.length) {
      const child = result[index];

      // Add eols before nodes, except if this is a loose, first paragraph.
      if (
        loose ||
        index !== 0 ||
        child.type !== 'element' ||
        child.tagName !== 'p'
      ) {
        wrapped.push(u$1('text', '\n'));
      }

      if (child.type === 'element' && child.tagName === 'p' && !loose) {
        wrapped.push(...child.children);
      } else {
        wrapped.push(child);
      }
    }

    const tail = result[result.length - 1];

    // Add a final eol.
    if (tail && (loose || !('tagName' in tail) || tail.tagName !== 'p')) {
      wrapped.push(u$1('text', '\n'));
    }

    return h(node, 'li', props, wrapped)
  }

  /**
   * @param {List} node
   * @return {Boolean}
   */
  function listLoose(node) {
    let loose = node.spread;
    const children = node.children;
    let index = -1;

    while (!loose && ++index < children.length) {
      loose = listItemLoose(children[index]);
    }

    return Boolean(loose)
  }

  /**
   * @param {ListItem} node
   * @return {Boolean}
   */
  function listItemLoose(node) {
    const spread = node.spread;

    return spread === undefined || spread === null
      ? node.children.length > 1
      : spread
  }

  /**
   * @typedef {import('mdast').List} List
   * @typedef {import('hast').Element} Element
   * @typedef {import('hast').Properties} Properties
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {List} node
   * @returns {Element}
   */
  function list(h, node) {
    /** @type {Properties} */
    const props = {};
    const name = node.ordered ? 'ol' : 'ul';
    const items = all$3(h, node);
    let index = -1;

    if (typeof node.start === 'number' && node.start !== 1) {
      props.start = node.start;
    }

    // Like GitHub, add a class for custom styling.
    while (++index < items.length) {
      const item = items[index];

      if (
        item.type === 'element' &&
        item.tagName === 'li' &&
        item.properties &&
        Array.isArray(item.properties.className) &&
        item.properties.className.includes('task-list-item')
      ) {
        props.className = ['contains-task-list'];
        break
      }
    }

    return h(node, name, props, wrap$1(items, true))
  }

  /**
   * @typedef {import('mdast').Paragraph} Paragraph
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {Paragraph} node
   */
  function paragraph(h, node) {
    return h(node, 'p', all$3(h, node))
  }

  /**
   * @typedef {import('mdast').Root} Root
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {Root} node
   */
  function root$5(h, node) {
    // @ts-expect-error `root`s are also fine.
    return h.augment(node, u$1('root', wrap$1(all$3(h, node))))
  }

  /**
   * @typedef {import('mdast').Strong} Strong
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {Strong} node
   */
  function strong(h, node) {
    return h(node, 'strong', all$3(h, node))
  }

  /**
   * @typedef {import('mdast').Table} Table
   * @typedef {import('mdast').TableCell} TableCell
   * @typedef {import('hast').Element} Element
   * @typedef {import('../index.js').Handler} Handler
   * @typedef {import('../index.js').Content} Content
   */

  /**
   * @type {Handler}
   * @param {Table} node
   */
  function table(h, node) {
    const rows = node.children;
    let index = -1;
    const align = node.align || [];
    /** @type {Array<Element>} */
    const result = [];

    while (++index < rows.length) {
      const row = rows[index].children;
      const name = index === 0 ? 'th' : 'td';
      /** @type {Array<Content>} */
      const out = [];
      let cellIndex = -1;
      const length = node.align ? align.length : row.length;

      while (++cellIndex < length) {
        const cell = row[cellIndex];
        out.push(
          h(cell, name, {align: align[cellIndex]}, cell ? all$3(h, cell) : [])
        );
      }

      result[index] = h(rows[index], 'tr', wrap$1(out, true));
    }

    return h(
      node,
      'table',
      wrap$1(
        [h(result[0].position, 'thead', wrap$1([result[0]], true))].concat(
          result[1]
            ? h(
                {
                  start: pointStart(result[1]),
                  end: pointEnd(result[result.length - 1])
                },
                'tbody',
                wrap$1(result.slice(1), true)
              )
            : []
        ),
        true
      )
    )
  }

  /**
   * @typedef {import('mdast').Text} Text
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {Text} node
   */
  function text$6(h, node) {
    return h.augment(
      node,
      u$1('text', String(node.value).replace(/[ \t]*(\r?\n|\r)[ \t]*/g, '$1'))
    )
  }

  /**
   * @typedef {import('mdast').ThematicBreak} ThematicBreak
   * @typedef {import('hast').Element} Element
   * @typedef {import('../index.js').Handler} Handler
   */

  /**
   * @type {Handler}
   * @param {ThematicBreak} [node]
   * @returns {Element}
   */
  function thematicBreak(h, node) {
    return h(node, 'hr')
  }

  const handlers$1 = {
    blockquote,
    break: hardBreak,
    code,
    delete: strikethrough,
    emphasis,
    footnoteReference,
    footnote,
    heading,
    html: html$i,
    imageReference,
    image,
    inlineCode,
    linkReference,
    link,
    listItem,
    list,
    paragraph,
    root: root$5,
    strong,
    table,
    text: text$6,
    thematicBreak,
    toml: ignore$1,
    yaml: ignore$1,
    definition: ignore$1,
    footnoteDefinition: ignore$1
  };

  // Return nothing for nodes that are ignored.
  function ignore$1() {
    return null
  }

  /**
   * @typedef {import('mdast').Root|import('mdast').Parent['children'][number]} MdastNode
   * @typedef {import('hast').Root|import('hast').Parent['children'][number]} HastNode
   * @typedef {import('mdast').Parent} Parent
   * @typedef {import('mdast').Definition} Definition
   * @typedef {import('mdast').FootnoteDefinition} FootnoteDefinition
   * @typedef {import('hast').Properties} Properties
   * @typedef {import('hast').Text} Text
   * @typedef {import('hast').Comment} Comment
   * @typedef {import('hast').Element} Element
   * @typedef {import('hast').Root} Root
   * @typedef {import('hast').ElementContent} Content
   * @typedef {import('unist-util-position').PositionLike} PositionLike
   *
   * @typedef EmbeddedHastFields
   * @property {string} [hName] Defines the tag name of an element
   * @property {Properties} [hProperties] Defines the properties of an element
   * @property {Array<Content>} [hChildren] Defines the (hast) children of an element
   *
   * @typedef {Record<string, unknown> & EmbeddedHastFields} Data unist data with embedded hast fields
   *
   * @typedef {MdastNode & {data?: Data}} NodeWithData unist node with embedded hast data
   *
   * @callback Handler
   * @param {H} h Handle context
   * @param {any} node mdast node to handle
   * @param {Parent|null} parent Parent of `node`
   * @returns {Content|Array<Content>|null|undefined} hast node
   *
   * @callback HFunctionProps
   * @param {MdastNode|PositionLike|null|undefined} node mdast node or unist position
   * @param {string} tagName HTML tag name
   * @param {Properties} props Properties
   * @param {Array<Content>?} [children] hast content
   * @returns {Element}
   *
   * @callback HFunctionNoProps
   * @param {MdastNode|PositionLike|null|undefined} node mdast node or unist position
   * @param {string} tagName HTML tag name
   * @param {Array<Content>?} [children] hast content
   * @returns {Element}
   *
   * @typedef HFields
   * @property {boolean} dangerous Whether HTML is allowed
   * @property {string} clobberPrefix Prefix to use to prevent DOM clobbering
   * @property {string} footnoteLabel Label to use to introduce the footnote section
   * @property {string} footnoteBackLabel Label to use to go back to a footnote call from the footnote section
   * @property {(identifier: string) => Definition|null} definition Definition cache
   * @property {Record<string, FootnoteDefinition>} footnoteById Footnote cache
   * @property {Array<string>} footnoteOrder Order in which footnotes occur
   * @property {Record<string, number>} footnoteCounts Counts the same footnote was used
   * @property {Handlers} handlers Applied handlers
   * @property {Handler} unknownHandler Handler for any none not in `passThrough` or otherwise handled
   * @property {(left: NodeWithData|PositionLike|null|undefined, right: Content) => Content} augment Like `h` but lower-level and usable on non-elements.
   * @property {Array<string>} passThrough List of node types to pass through untouched (except for their children).
   *
   * @typedef Options
   * @property {boolean} [allowDangerousHtml=false]
   *   Whether to allow `html` nodes and inject them as `raw` HTML
   * @property {string} [clobberPrefix='user-content-']
   *   Prefix to use before the `id` attribute to prevent it from *clobbering*.
   *   attributes.
   *   DOM clobbering is this:
   *
   *   ```html
   *   <p id=x></p>
   *   <script>alert(x)</script>
   *   ```
   *
   *   Elements by their ID are made available in browsers on the `window` object.
   *   Using a prefix prevents this from being a problem.
   * @property {string} [footnoteLabel='Footnotes']
   *   Label to use for the footnotes section.
   *   Affects screen reader users.
   *   Change it if you’re authoring in a different language.
   * @property {string} [footnoteBackLabel='Back to content']
   *   Label to use from backreferences back to their footnote call.
   *   Affects screen reader users.
   *   Change it if you’re authoring in a different language.
   * @property {Handlers} [handlers]
   *   Object mapping mdast nodes to functions handling them
   * @property {Array<string>} [passThrough]
   *   List of custom mdast node types to pass through (keep) in hast
   * @property {Handler} [unknownHandler]
   *   Handler for all unknown nodes.
   *
   * @typedef {Record<string, Handler>} Handlers
   *   Map of node types to handlers
   * @typedef {HFunctionProps & HFunctionNoProps & HFields} H
   *   Handle context
   */

  const own$n = {}.hasOwnProperty;

  /**
   * Factory to transform.
   * @param {MdastNode} tree mdast node
   * @param {Options} [options] Configuration
   * @returns {H} `h` function
   */
  function factory$1(tree, options) {
    const settings = options || {};
    const dangerous = settings.allowDangerousHtml || false;
    /** @type {Record<string, FootnoteDefinition>} */
    const footnoteById = {};

    h.dangerous = dangerous;
    h.clobberPrefix =
      settings.clobberPrefix === undefined || settings.clobberPrefix === null
        ? 'user-content-'
        : settings.clobberPrefix;
    h.footnoteLabel = settings.footnoteLabel || 'Footnotes';
    h.footnoteBackLabel = settings.footnoteBackLabel || 'Back to content';
    h.definition = definitions(tree);
    h.footnoteById = footnoteById;
    /** @type {Array<string>} */
    h.footnoteOrder = [];
    /** @type {Record<string, number>} */
    h.footnoteCounts = {};
    h.augment = augment;
    h.handlers = {...handlers$1, ...settings.handlers};
    h.unknownHandler = settings.unknownHandler;
    h.passThrough = settings.passThrough;

    visit$2(tree, 'footnoteDefinition', (definition) => {
      const id = String(definition.identifier).toUpperCase();

      // Mimick CM behavior of link definitions.
      // See: <https://github.com/syntax-tree/mdast-util-definitions/blob/8290999/index.js#L26>.
      if (!own$n.call(footnoteById, id)) {
        footnoteById[id] = definition;
      }
    });

    // @ts-expect-error Hush, it’s fine!
    return h

    /**
     * Finalise the created `right`, a hast node, from `left`, an mdast node.
     * @param {(NodeWithData|PositionLike)?} left
     * @param {Content} right
     * @returns {Content}
     */
    function augment(left, right) {
      // Handle `data.hName`, `data.hProperties, `data.hChildren`.
      if (left && 'data' in left && left.data) {
        /** @type {Data} */
        const data = left.data;

        if (data.hName) {
          if (right.type !== 'element') {
            right = {
              type: 'element',
              tagName: '',
              properties: {},
              children: []
            };
          }

          right.tagName = data.hName;
        }

        if (right.type === 'element' && data.hProperties) {
          right.properties = {...right.properties, ...data.hProperties};
        }

        if ('children' in right && right.children && data.hChildren) {
          right.children = data.hChildren;
        }
      }

      if (left) {
        const ctx = 'type' in left ? left : {position: left};

        if (!generated(ctx)) {
          right.position = {start: pointStart(ctx), end: pointEnd(ctx)};
        }
      }

      return right
    }

    /**
     * Create an element for `node`.
     *
     * @type {HFunctionProps}
     */
    function h(node, tagName, props, children) {
      if (Array.isArray(props)) {
        children = props;
        props = {};
      }

      // @ts-expect-error augmenting an element yields an element.
      return augment(node, {
        type: 'element',
        tagName,
        properties: props || {},
        children: children || []
      })
    }
  }

  /**
   * Transform `tree` (an mdast node) to a hast node.
   *
   * @param {MdastNode} tree mdast node
   * @param {Options} [options] Configuration
   * @returns {HastNode|null|undefined} hast node
   */
  function toHast(tree, options) {
    const h = factory$1(tree, options);
    const node = one$3(h, tree, null);
    const foot = footer(h);

    if (foot) {
      // @ts-expect-error If there’s a footer, there were definitions, meaning block
      // content.
      // So assume `node` is a parent node.
      node.children.push(u$1('text', '\n'), foot);
    }

    return Array.isArray(node) ? {type: 'root', children: node} : node
  }

  /**
   * @typedef {import('hast').Root} HastRoot
   * @typedef {import('mdast').Root} MdastRoot
   * @typedef {import('mdast-util-to-hast').Options} Options
   * @typedef {import('unified').Processor<any, any, any, any>} Processor
   *
   * @typedef {import('mdast-util-to-hast')} DoNotTouchAsThisImportIncludesRawInTree
   */

  // Note: the `<MdastRoot, HastRoot>` overload doesn’t seem to work :'(

  /**
   * Plugin that turns markdown into HTML to support rehype.
   *
   * *   If a destination processor is given, that processor runs with a new HTML
   *     (hast) tree (bridge-mode).
   *     As the given processor runs with a hast tree, and rehype plugins support
   *     hast, that means rehype plugins can be used with the given processor.
   *     The hast tree is discarded in the end.
   *     It’s highly unlikely that you want to do this.
   * *   The common case is to not pass a destination processor, in which case the
   *     current processor continues running with a new HTML (hast) tree
   *     (mutate-mode).
   *     As the current processor continues with a hast tree, and rehype plugins
   *     support hast, that means rehype plugins can be used after
   *     `remark-rehype`.
   *     It’s likely that this is what you want to do.
   *
   * @param destination
   *   Optional unified processor.
   * @param options
   *   Options passed to `mdast-util-to-hast`.
   */
  const remarkRehype =
    /** @type {(import('unified').Plugin<[Processor, Options?]|[null|undefined, Options?]|[Options]|[], MdastRoot>)} */
    (
      function (destination, options) {
        return destination && 'run' in destination
          ? bridge(destination, options)
          : mutate(destination || options)
      }
    );

  var remarkRehype$1 = remarkRehype;

  /**
   * Bridge-mode.
   * Runs the destination with the new hast tree.
   *
   * @type {import('unified').Plugin<[Processor, Options?], MdastRoot>}
   */
  function bridge(destination, options) {
    return (node, file, next) => {
      destination.run(toHast(node, options), file, (error) => {
        next(error);
      });
    }
  }

  /**
   * Mutate-mode.
   * Further plugins run on the hast tree.
   *
   * @type {import('unified').Plugin<[Options?]|void[], MdastRoot, HastRoot>}
   */
  function mutate(options) {
    // @ts-expect-error: assume a corresponding node is returned by `toHast`.
    return (node) => toHast(node, options)
  }

  function detailsNode(title) {
      return {
          type: 'element',
          tagName: 'details',
          properties: { open: true, className: 'octicon octicon-video' },
          children: [
              {
                  type: 'element',
                  tagName: 'summary',
                  children: [
                      {
                          type: 'element',
                          tagName: 'svg',
                          properties: {
                              'aria-hidden': true, height: 16, width: 16, viewBox: '0 0 16 16', version: '1.1', 'data-view-component': true,
                              className: 'octicon octicon-device-camera-video'
                          },
                          children: [
                              {
                                  type: 'element',
                                  tagName: 'path',
                                  properties: {
                                      'fill-rule': 'evenodd',
                                      d: 'M16 3.75a.75.75 0 00-1.136-.643L11 5.425V4.75A1.75 1.75 0 009.25 3h-7.5A1.75 1.75 0 000 4.75v6.5C0 12.216.784 13 1.75 13h7.5A1.75 1.75 0 0011 11.25v-.675l3.864 2.318A.75.75 0 0016 12.25v-8.5zm-5 5.075l3.5 2.1v-5.85l-3.5 2.1v1.65zM9.5 6.75v-2a.25.25 0 00-.25-.25h-7.5a.25.25 0 00-.25.25v6.5c0 .138.112.25.25.25h7.5a.25.25 0 00.25-.25v-4.5z'
                                  },
                                  children: [],
                              }
                          ]
                      },
                      {
                          type: 'element',
                          tagName: 'span',
                          properties: {
                              'aria-label': `Video description ${title || ''}`
                          },
                          children: [
                              {
                                  type: 'text',
                                  value: title || ''
                              }
                          ]
                      },
                      {
                          type: 'element',
                          tagName: 'span',
                          properties: {
                              className: 'dropdown-caret'
                          },
                          children: []
                      }
                  ]
              }
          ]
      };
  }

  const properties$1 = { muted: 'muted', controls: 'controls', style: 'max-height:640px;' };
  function reElement(node, details, href) {
      const filename = href.split('/').pop();
      node.properties = { ...properties$1, src: href };
      node.tagName = 'video';
      node.children = [];
      if (details) {
          const reNode = detailsNode(filename);
          reNode.children.push({ ...node });
          node.children = reNode.children;
          node.tagName = reNode.tagName;
          node.properties = reNode.properties;
      }
  }
  const RehypeVideo = (options) => {
      const { test = /\/(.*)(.mp4|.mov)$/, details = true } = options || {};
      return (tree) => {
          visit$2(tree, 'element', (node, index, parent) => {
              const isChecked = (str) => test.test(str);
              const child = node.children[0];
              const delimiter = /((?:https?:\/\/)(?:(?:[a-z0-9]?(?:[a-z0-9\-]{1,61}[a-z0-9])?\.[^\.|\s])+[a-z\.]*[a-z]+|(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(?:\.(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)){3})(?::\d{1,5})*[a-z0-9.,_\/~#&=;%+?\-\\(\\)]*)/g;
              // const delimiter = /((?:https?:\/\/)?(?:(?:[a-z0-9]?(?:[a-z0-9\-]{1,61}[a-z0-9])?\.[^\.|\s])+[a-z\.]*[a-z]+|(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(?:\.(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)){3})(?::\d{1,5})*[a-z0-9.,_\/~#&=;%+?\-\\(\\)]*)/g;
              if (node.tagName === 'p' && node.children.length === 1) {
                  if (child.type === 'text' && delimiter.test(child.value) && isChecked(child.value)) {
                      reElement(node, details, child.value);
                  }
                  if (child.type === 'element' && child.tagName === 'a' && child.properties && typeof child.properties.href === 'string' && isChecked(child.properties.href)) {
                      reElement(node, details, child.properties.href);
                  }
              }
          });
      };
  };
  var rehypeVideo = RehypeVideo;

  /**
   * Lexing or parsing positional information for error reporting.
   * This object is immutable.
   */
  class SourceLocation {
    // The + prefix indicates that these fields aren't writeable
    // Lexer holding the input string.
    // Start offset, zero-based inclusive.
    // End offset, zero-based exclusive.
    constructor(lexer, start, end) {
      this.lexer = void 0;
      this.start = void 0;
      this.end = void 0;
      this.lexer = lexer;
      this.start = start;
      this.end = end;
    }
    /**
     * Merges two `SourceLocation`s from location providers, given they are
     * provided in order of appearance.
     * - Returns the first one's location if only the first is provided.
     * - Returns a merged range of the first and the last if both are provided
     *   and their lexers match.
     * - Otherwise, returns null.
     */


    static range(first, second) {
      if (!second) {
        return first && first.loc;
      } else if (!first || !first.loc || !second.loc || first.loc.lexer !== second.loc.lexer) {
        return null;
      } else {
        return new SourceLocation(first.loc.lexer, first.loc.start, second.loc.end);
      }
    }

  }

  /**
   * Interface required to break circular dependency between Token, Lexer, and
   * ParseError.
   */

  /**
   * The resulting token returned from `lex`.
   *
   * It consists of the token text plus some position information.
   * The position information is essentially a range in an input string,
   * but instead of referencing the bare input string, we refer to the lexer.
   * That way it is possible to attach extra metadata to the input string,
   * like for example a file name or similar.
   *
   * The position information is optional, so it is OK to construct synthetic
   * tokens if appropriate. Not providing available position information may
   * lead to degraded error reporting, though.
   */
  class Token {
    // don't expand the token
    // used in \noexpand
    constructor(text, // the text of this token
    loc) {
      this.text = void 0;
      this.loc = void 0;
      this.noexpand = void 0;
      this.treatAsRelax = void 0;
      this.text = text;
      this.loc = loc;
    }
    /**
     * Given a pair of tokens (this and endToken), compute a `Token` encompassing
     * the whole input range enclosed by these two.
     */


    range(endToken, // last token of the range, inclusive
    text // the text of the newly constructed token
    ) {
      return new Token(text, SourceLocation.range(this, endToken));
    }

  }

  /**
   * This is the ParseError class, which is the main error thrown by KaTeX
   * functions when something has gone wrong. This is used to distinguish internal
   * errors from errors in the expression that the user provided.
   *
   * If possible, a caller should provide a Token or ParseNode with information
   * about where in the source string the problem occurred.
   */
  class ParseError {
    // Error position based on passed-in Token or ParseNode.
    constructor(message, // The error message
    token // An object providing position information
    ) {
      this.position = void 0;
      var error = "KaTeX parse error: " + message;
      var start;
      var loc = token && token.loc;

      if (loc && loc.start <= loc.end) {
        // If we have the input and a position, make the error a bit fancier
        // Get the input
        var input = loc.lexer.input; // Prepend some information

        start = loc.start;
        var end = loc.end;

        if (start === input.length) {
          error += " at end of input: ";
        } else {
          error += " at position " + (start + 1) + ": ";
        } // Underline token in question using combining underscores


        var underlined = input.slice(start, end).replace(/[^]/g, "$&\u0332"); // Extract some context from the input and add it to the error

        var left;

        if (start > 15) {
          left = "…" + input.slice(start - 15, start);
        } else {
          left = input.slice(0, start);
        }

        var right;

        if (end + 15 < input.length) {
          right = input.slice(end, end + 15) + "…";
        } else {
          right = input.slice(end);
        }

        error += left + underlined + right;
      } // Some hackery to make ParseError a prototype of Error
      // See http://stackoverflow.com/a/8460753


      var self = new Error(error);
      self.name = "ParseError"; // $FlowFixMe

      self.__proto__ = ParseError.prototype; // $FlowFixMe

      self.position = start;
      return self;
    }

  } // $FlowFixMe More hackery


  ParseError.prototype.__proto__ = Error.prototype;

  /**
   * This file contains a list of utility functions which are useful in other
   * files.
   */

  /**
   * Return whether an element is contained in a list
   */
  var contains$1 = function contains(list, elem) {
    return list.indexOf(elem) !== -1;
  };
  /**
   * Provide a default value if a setting is undefined
   * NOTE: Couldn't use `T` as the output type due to facebook/flow#5022.
   */


  var deflt = function deflt(setting, defaultIfUndefined) {
    return setting === undefined ? defaultIfUndefined : setting;
  }; // hyphenate and escape adapted from Facebook's React under Apache 2 license


  var uppercase = /([A-Z])/g;

  var hyphenate = function hyphenate(str) {
    return str.replace(uppercase, "-$1").toLowerCase();
  };

  var ESCAPE_LOOKUP = {
    "&": "&amp;",
    ">": "&gt;",
    "<": "&lt;",
    "\"": "&quot;",
    "'": "&#x27;"
  };
  var ESCAPE_REGEX = /[&><"']/g;
  /**
   * Escapes text to prevent scripting attacks.
   */

  function escape(text) {
    return String(text).replace(ESCAPE_REGEX, match => ESCAPE_LOOKUP[match]);
  }
  /**
   * Sometimes we want to pull out the innermost element of a group. In most
   * cases, this will just be the group itself, but when ordgroups and colors have
   * a single element, we want to pull that out.
   */


  var getBaseElem = function getBaseElem(group) {
    if (group.type === "ordgroup") {
      if (group.body.length === 1) {
        return getBaseElem(group.body[0]);
      } else {
        return group;
      }
    } else if (group.type === "color") {
      if (group.body.length === 1) {
        return getBaseElem(group.body[0]);
      } else {
        return group;
      }
    } else if (group.type === "font") {
      return getBaseElem(group.body);
    } else {
      return group;
    }
  };
  /**
   * TeXbook algorithms often reference "character boxes", which are simply groups
   * with a single character in them. To decide if something is a character box,
   * we find its innermost group, and see if it is a single character.
   */


  var isCharacterBox = function isCharacterBox(group) {
    var baseElem = getBaseElem(group); // These are all they types of groups which hold single characters

    return baseElem.type === "mathord" || baseElem.type === "textord" || baseElem.type === "atom";
  };

  var assert = function assert(value) {
    if (!value) {
      throw new Error('Expected non-null, but got ' + String(value));
    }

    return value;
  };
  /**
   * Return the protocol of a URL, or "_relative" if the URL does not specify a
   * protocol (and thus is relative).
   */

  var protocolFromUrl = function protocolFromUrl(url) {
    var protocol = /^\s*([^\\/#]*?)(?::|&#0*58|&#x0*3a)/i.exec(url);
    return protocol != null ? protocol[1] : "_relative";
  };
  var utils$1 = {
    contains: contains$1,
    deflt,
    escape,
    hyphenate,
    getBaseElem,
    isCharacterBox,
    protocolFromUrl
  };

  /* eslint no-console:0 */
  // TODO: automatically generate documentation
  // TODO: check all properties on Settings exist
  // TODO: check the type of a property on Settings matches
  var SETTINGS_SCHEMA = {
    displayMode: {
      type: "boolean",
      description: "Render math in display mode, which puts the math in " + "display style (so \\int and \\sum are large, for example), and " + "centers the math on the page on its own line.",
      cli: "-d, --display-mode"
    },
    output: {
      type: {
        enum: ["htmlAndMathml", "html", "mathml"]
      },
      description: "Determines the markup language of the output.",
      cli: "-F, --format <type>"
    },
    leqno: {
      type: "boolean",
      description: "Render display math in leqno style (left-justified tags)."
    },
    fleqn: {
      type: "boolean",
      description: "Render display math flush left."
    },
    throwOnError: {
      type: "boolean",
      default: true,
      cli: "-t, --no-throw-on-error",
      cliDescription: "Render errors (in the color given by --error-color) ins" + "tead of throwing a ParseError exception when encountering an error."
    },
    errorColor: {
      type: "string",
      default: "#cc0000",
      cli: "-c, --error-color <color>",
      cliDescription: "A color string given in the format 'rgb' or 'rrggbb' " + "(no #). This option determines the color of errors rendered by the " + "-t option.",
      cliProcessor: color => "#" + color
    },
    macros: {
      type: "object",
      cli: "-m, --macro <def>",
      cliDescription: "Define custom macro of the form '\\foo:expansion' (use " + "multiple -m arguments for multiple macros).",
      cliDefault: [],
      cliProcessor: (def, defs) => {
        defs.push(def);
        return defs;
      }
    },
    minRuleThickness: {
      type: "number",
      description: "Specifies a minimum thickness, in ems, for fraction lines," + " `\\sqrt` top lines, `{array}` vertical lines, `\\hline`, " + "`\\hdashline`, `\\underline`, `\\overline`, and the borders of " + "`\\fbox`, `\\boxed`, and `\\fcolorbox`.",
      processor: t => Math.max(0, t),
      cli: "--min-rule-thickness <size>",
      cliProcessor: parseFloat
    },
    colorIsTextColor: {
      type: "boolean",
      description: "Makes \\color behave like LaTeX's 2-argument \\textcolor, " + "instead of LaTeX's one-argument \\color mode change.",
      cli: "-b, --color-is-text-color"
    },
    strict: {
      type: [{
        enum: ["warn", "ignore", "error"]
      }, "boolean", "function"],
      description: "Turn on strict / LaTeX faithfulness mode, which throws an " + "error if the input uses features that are not supported by LaTeX.",
      cli: "-S, --strict",
      cliDefault: false
    },
    trust: {
      type: ["boolean", "function"],
      description: "Trust the input, enabling all HTML features such as \\url.",
      cli: "-T, --trust"
    },
    maxSize: {
      type: "number",
      default: Infinity,
      description: "If non-zero, all user-specified sizes, e.g. in " + "\\rule{500em}{500em}, will be capped to maxSize ems. Otherwise, " + "elements and spaces can be arbitrarily large",
      processor: s => Math.max(0, s),
      cli: "-s, --max-size <n>",
      cliProcessor: parseInt
    },
    maxExpand: {
      type: "number",
      default: 1000,
      description: "Limit the number of macro expansions to the specified " + "number, to prevent e.g. infinite macro loops. If set to Infinity, " + "the macro expander will try to fully expand as in LaTeX.",
      processor: n => Math.max(0, n),
      cli: "-e, --max-expand <n>",
      cliProcessor: n => n === "Infinity" ? Infinity : parseInt(n)
    },
    globalGroup: {
      type: "boolean",
      cli: false
    }
  };

  function getDefaultValue(schema) {
    if (schema.default) {
      return schema.default;
    }

    var type = schema.type;
    var defaultType = Array.isArray(type) ? type[0] : type;

    if (typeof defaultType !== 'string') {
      return defaultType.enum[0];
    }

    switch (defaultType) {
      case 'boolean':
        return false;

      case 'string':
        return '';

      case 'number':
        return 0;

      case 'object':
        return {};
    }
  }
  /**
   * The main Settings object
   *
   * The current options stored are:
   *  - displayMode: Whether the expression should be typeset as inline math
   *                 (false, the default), meaning that the math starts in
   *                 \textstyle and is placed in an inline-block); or as display
   *                 math (true), meaning that the math starts in \displaystyle
   *                 and is placed in a block with vertical margin.
   */


  class Settings {
    constructor(options) {
      this.displayMode = void 0;
      this.output = void 0;
      this.leqno = void 0;
      this.fleqn = void 0;
      this.throwOnError = void 0;
      this.errorColor = void 0;
      this.macros = void 0;
      this.minRuleThickness = void 0;
      this.colorIsTextColor = void 0;
      this.strict = void 0;
      this.trust = void 0;
      this.maxSize = void 0;
      this.maxExpand = void 0;
      this.globalGroup = void 0;
      // allow null options
      options = options || {};

      for (var prop in SETTINGS_SCHEMA) {
        if (SETTINGS_SCHEMA.hasOwnProperty(prop)) {
          // $FlowFixMe
          var schema = SETTINGS_SCHEMA[prop]; // TODO: validate options
          // $FlowFixMe

          this[prop] = options[prop] !== undefined ? schema.processor ? schema.processor(options[prop]) : options[prop] : getDefaultValue(schema);
        }
      }
    }
    /**
     * Report nonstrict (non-LaTeX-compatible) input.
     * Can safely not be called if `this.strict` is false in JavaScript.
     */


    reportNonstrict(errorCode, errorMsg, token) {
      var strict = this.strict;

      if (typeof strict === "function") {
        // Allow return value of strict function to be boolean or string
        // (or null/undefined, meaning no further processing).
        strict = strict(errorCode, errorMsg, token);
      }

      if (!strict || strict === "ignore") {
        return;
      } else if (strict === true || strict === "error") {
        throw new ParseError("LaTeX-incompatible input and strict mode is set to 'error': " + (errorMsg + " [" + errorCode + "]"), token);
      } else if (strict === "warn") {
        typeof console !== "undefined" && console.warn("LaTeX-incompatible input and strict mode is set to 'warn': " + (errorMsg + " [" + errorCode + "]"));
      } else {
        // won't happen in type-safe code
        typeof console !== "undefined" && console.warn("LaTeX-incompatible input and strict mode is set to " + ("unrecognized '" + strict + "': " + errorMsg + " [" + errorCode + "]"));
      }
    }
    /**
     * Check whether to apply strict (LaTeX-adhering) behavior for unusual
     * input (like `\\`).  Unlike `nonstrict`, will not throw an error;
     * instead, "error" translates to a return value of `true`, while "ignore"
     * translates to a return value of `false`.  May still print a warning:
     * "warn" prints a warning and returns `false`.
     * This is for the second category of `errorCode`s listed in the README.
     */


    useStrictBehavior(errorCode, errorMsg, token) {
      var strict = this.strict;

      if (typeof strict === "function") {
        // Allow return value of strict function to be boolean or string
        // (or null/undefined, meaning no further processing).
        // But catch any exceptions thrown by function, treating them
        // like "error".
        try {
          strict = strict(errorCode, errorMsg, token);
        } catch (error) {
          strict = "error";
        }
      }

      if (!strict || strict === "ignore") {
        return false;
      } else if (strict === true || strict === "error") {
        return true;
      } else if (strict === "warn") {
        typeof console !== "undefined" && console.warn("LaTeX-incompatible input and strict mode is set to 'warn': " + (errorMsg + " [" + errorCode + "]"));
        return false;
      } else {
        // won't happen in type-safe code
        typeof console !== "undefined" && console.warn("LaTeX-incompatible input and strict mode is set to " + ("unrecognized '" + strict + "': " + errorMsg + " [" + errorCode + "]"));
        return false;
      }
    }
    /**
     * Check whether to test potentially dangerous input, and return
     * `true` (trusted) or `false` (untrusted).  The sole argument `context`
     * should be an object with `command` field specifying the relevant LaTeX
     * command (as a string starting with `\`), and any other arguments, etc.
     * If `context` has a `url` field, a `protocol` field will automatically
     * get added by this function (changing the specified object).
     */


    isTrusted(context) {
      if (context.url && !context.protocol) {
        context.protocol = utils$1.protocolFromUrl(context.url);
      }

      var trust = typeof this.trust === "function" ? this.trust(context) : this.trust;
      return Boolean(trust);
    }

  }

  /**
   * This file contains information and classes for the various kinds of styles
   * used in TeX. It provides a generic `Style` class, which holds information
   * about a specific style. It then provides instances of all the different kinds
   * of styles possible, and provides functions to move between them and get
   * information about them.
   */

  /**
   * The main style class. Contains a unique id for the style, a size (which is
   * the same for cramped and uncramped version of a style), and a cramped flag.
   */
  class Style {
    constructor(id, size, cramped) {
      this.id = void 0;
      this.size = void 0;
      this.cramped = void 0;
      this.id = id;
      this.size = size;
      this.cramped = cramped;
    }
    /**
     * Get the style of a superscript given a base in the current style.
     */


    sup() {
      return styles[sup[this.id]];
    }
    /**
     * Get the style of a subscript given a base in the current style.
     */


    sub() {
      return styles[sub[this.id]];
    }
    /**
     * Get the style of a fraction numerator given the fraction in the current
     * style.
     */


    fracNum() {
      return styles[fracNum[this.id]];
    }
    /**
     * Get the style of a fraction denominator given the fraction in the current
     * style.
     */


    fracDen() {
      return styles[fracDen[this.id]];
    }
    /**
     * Get the cramped version of a style (in particular, cramping a cramped style
     * doesn't change the style).
     */


    cramp() {
      return styles[cramp[this.id]];
    }
    /**
     * Get a text or display version of this style.
     */


    text() {
      return styles[text$1$1[this.id]];
    }
    /**
     * Return true if this style is tightly spaced (scriptstyle/scriptscriptstyle)
     */


    isTight() {
      return this.size >= 2;
    }

  } // Export an interface for type checking, but don't expose the implementation.
  // This way, no more styles can be generated.


  // IDs of the different styles
  var D = 0;
  var Dc = 1;
  var T = 2;
  var Tc = 3;
  var S = 4;
  var Sc = 5;
  var SS = 6;
  var SSc = 7; // Instances of the different styles

  var styles = [new Style(D, 0, false), new Style(Dc, 0, true), new Style(T, 1, false), new Style(Tc, 1, true), new Style(S, 2, false), new Style(Sc, 2, true), new Style(SS, 3, false), new Style(SSc, 3, true)]; // Lookup tables for switching from one style to another

  var sup = [S, Sc, S, Sc, SS, SSc, SS, SSc];
  var sub = [Sc, Sc, Sc, Sc, SSc, SSc, SSc, SSc];
  var fracNum = [T, Tc, S, Sc, SS, SSc, SS, SSc];
  var fracDen = [Tc, Tc, Sc, Sc, SSc, SSc, SSc, SSc];
  var cramp = [Dc, Dc, Tc, Tc, Sc, Sc, SSc, SSc];
  var text$1$1 = [D, Dc, T, Tc, T, Tc, T, Tc]; // We only export some of the styles.

  var Style$1 = {
    DISPLAY: styles[D],
    TEXT: styles[T],
    SCRIPT: styles[S],
    SCRIPTSCRIPT: styles[SS]
  };

  /*
   * This file defines the Unicode scripts and script families that we
   * support. To add new scripts or families, just add a new entry to the
   * scriptData array below. Adding scripts to the scriptData array allows
   * characters from that script to appear in \text{} environments.
   */

  /**
   * Each script or script family has a name and an array of blocks.
   * Each block is an array of two numbers which specify the start and
   * end points (inclusive) of a block of Unicode codepoints.
   */

  /**
   * Unicode block data for the families of scripts we support in \text{}.
   * Scripts only need to appear here if they do not have font metrics.
   */
  var scriptData = [{
    // Latin characters beyond the Latin-1 characters we have metrics for.
    // Needed for Czech, Hungarian and Turkish text, for example.
    name: 'latin',
    blocks: [[0x0100, 0x024f], // Latin Extended-A and Latin Extended-B
    [0x0300, 0x036f] // Combining Diacritical marks
    ]
  }, {
    // The Cyrillic script used by Russian and related languages.
    // A Cyrillic subset used to be supported as explicitly defined
    // symbols in symbols.js
    name: 'cyrillic',
    blocks: [[0x0400, 0x04ff]]
  }, {
    // Armenian
    name: 'armenian',
    blocks: [[0x0530, 0x058F]]
  }, {
    // The Brahmic scripts of South and Southeast Asia
    // Devanagari (0900–097F)
    // Bengali (0980–09FF)
    // Gurmukhi (0A00–0A7F)
    // Gujarati (0A80–0AFF)
    // Oriya (0B00–0B7F)
    // Tamil (0B80–0BFF)
    // Telugu (0C00–0C7F)
    // Kannada (0C80–0CFF)
    // Malayalam (0D00–0D7F)
    // Sinhala (0D80–0DFF)
    // Thai (0E00–0E7F)
    // Lao (0E80–0EFF)
    // Tibetan (0F00–0FFF)
    // Myanmar (1000–109F)
    name: 'brahmic',
    blocks: [[0x0900, 0x109F]]
  }, {
    name: 'georgian',
    blocks: [[0x10A0, 0x10ff]]
  }, {
    // Chinese and Japanese.
    // The "k" in cjk is for Korean, but we've separated Korean out
    name: "cjk",
    blocks: [[0x3000, 0x30FF], // CJK symbols and punctuation, Hiragana, Katakana
    [0x4E00, 0x9FAF], // CJK ideograms
    [0xFF00, 0xFF60] // Fullwidth punctuation
    // TODO: add halfwidth Katakana and Romanji glyphs
    ]
  }, {
    // Korean
    name: 'hangul',
    blocks: [[0xAC00, 0xD7AF]]
  }];
  /**
   * Given a codepoint, return the name of the script or script family
   * it is from, or null if it is not part of a known block
   */

  function scriptFromCodepoint(codepoint) {
    for (var i = 0; i < scriptData.length; i++) {
      var script = scriptData[i];

      for (var _i = 0; _i < script.blocks.length; _i++) {
        var block = script.blocks[_i];

        if (codepoint >= block[0] && codepoint <= block[1]) {
          return script.name;
        }
      }
    }

    return null;
  }
  /**
   * A flattened version of all the supported blocks in a single array.
   * This is an optimization to make supportedCodepoint() fast.
   */

  var allBlocks = [];
  scriptData.forEach(s => s.blocks.forEach(b => allBlocks.push(...b)));
  /**
   * Given a codepoint, return true if it falls within one of the
   * scripts or script families defined above and false otherwise.
   *
   * Micro benchmarks shows that this is faster than
   * /[\u3000-\u30FF\u4E00-\u9FAF\uFF00-\uFF60\uAC00-\uD7AF\u0900-\u109F]/.test()
   * in Firefox, Chrome and Node.
   */

  function supportedCodepoint(codepoint) {
    for (var i = 0; i < allBlocks.length; i += 2) {
      if (codepoint >= allBlocks[i] && codepoint <= allBlocks[i + 1]) {
        return true;
      }
    }

    return false;
  }

  /**
   * This file provides support to domTree.js and delimiter.js.
   * It's a storehouse of path geometry for SVG images.
   */
  // In all paths below, the viewBox-to-em scale is 1000:1.
  var hLinePad = 80; // padding above a sqrt viniculum. Prevents image cropping.
  // The viniculum of a \sqrt can be made thicker by a KaTeX rendering option.
  // Think of variable extraViniculum as two detours in the SVG path.
  // The detour begins at the lower left of the area labeled extraViniculum below.
  // The detour proceeds one extraViniculum distance up and slightly to the right,
  // displacing the radiused corner between surd and viniculum. The radius is
  // traversed as usual, then the detour resumes. It goes right, to the end of
  // the very long viniculumn, then down one extraViniculum distance,
  // after which it resumes regular path geometry for the radical.

  /*                                                  viniculum
                                                     /
           /▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒←extraViniculum
          / █████████████████████←0.04em (40 unit) std viniculum thickness
         / /
        / /
       / /\
      / / surd
  */

  var sqrtMain = function sqrtMain(extraViniculum, hLinePad) {
    // sqrtMain path geometry is from glyph U221A in the font KaTeX Main
    return "M95," + (622 + extraViniculum + hLinePad) + "\nc-2.7,0,-7.17,-2.7,-13.5,-8c-5.8,-5.3,-9.5,-10,-9.5,-14\nc0,-2,0.3,-3.3,1,-4c1.3,-2.7,23.83,-20.7,67.5,-54\nc44.2,-33.3,65.8,-50.3,66.5,-51c1.3,-1.3,3,-2,5,-2c4.7,0,8.7,3.3,12,10\ns173,378,173,378c0.7,0,35.3,-71,104,-213c68.7,-142,137.5,-285,206.5,-429\nc69,-144,104.5,-217.7,106.5,-221\nl" + extraViniculum / 2.075 + " -" + extraViniculum + "\nc5.3,-9.3,12,-14,20,-14\nH400000v" + (40 + extraViniculum) + "H845.2724\ns-225.272,467,-225.272,467s-235,486,-235,486c-2.7,4.7,-9,7,-19,7\nc-6,0,-10,-1,-12,-3s-194,-422,-194,-422s-65,47,-65,47z\nM" + (834 + extraViniculum) + " " + hLinePad + "h400000v" + (40 + extraViniculum) + "h-400000z";
  };

  var sqrtSize1 = function sqrtSize1(extraViniculum, hLinePad) {
    // size1 is from glyph U221A in the font KaTeX_Size1-Regular
    return "M263," + (601 + extraViniculum + hLinePad) + "c0.7,0,18,39.7,52,119\nc34,79.3,68.167,158.7,102.5,238c34.3,79.3,51.8,119.3,52.5,120\nc340,-704.7,510.7,-1060.3,512,-1067\nl" + extraViniculum / 2.084 + " -" + extraViniculum + "\nc4.7,-7.3,11,-11,19,-11\nH40000v" + (40 + extraViniculum) + "H1012.3\ns-271.3,567,-271.3,567c-38.7,80.7,-84,175,-136,283c-52,108,-89.167,185.3,-111.5,232\nc-22.3,46.7,-33.8,70.3,-34.5,71c-4.7,4.7,-12.3,7,-23,7s-12,-1,-12,-1\ns-109,-253,-109,-253c-72.7,-168,-109.3,-252,-110,-252c-10.7,8,-22,16.7,-34,26\nc-22,17.3,-33.3,26,-34,26s-26,-26,-26,-26s76,-59,76,-59s76,-60,76,-60z\nM" + (1001 + extraViniculum) + " " + hLinePad + "h400000v" + (40 + extraViniculum) + "h-400000z";
  };

  var sqrtSize2 = function sqrtSize2(extraViniculum, hLinePad) {
    // size2 is from glyph U221A in the font KaTeX_Size2-Regular
    return "M983 " + (10 + extraViniculum + hLinePad) + "\nl" + extraViniculum / 3.13 + " -" + extraViniculum + "\nc4,-6.7,10,-10,18,-10 H400000v" + (40 + extraViniculum) + "\nH1013.1s-83.4,268,-264.1,840c-180.7,572,-277,876.3,-289,913c-4.7,4.7,-12.7,7,-24,7\ns-12,0,-12,0c-1.3,-3.3,-3.7,-11.7,-7,-25c-35.3,-125.3,-106.7,-373.3,-214,-744\nc-10,12,-21,25,-33,39s-32,39,-32,39c-6,-5.3,-15,-14,-27,-26s25,-30,25,-30\nc26.7,-32.7,52,-63,76,-91s52,-60,52,-60s208,722,208,722\nc56,-175.3,126.3,-397.3,211,-666c84.7,-268.7,153.8,-488.2,207.5,-658.5\nc53.7,-170.3,84.5,-266.8,92.5,-289.5z\nM" + (1001 + extraViniculum) + " " + hLinePad + "h400000v" + (40 + extraViniculum) + "h-400000z";
  };

  var sqrtSize3 = function sqrtSize3(extraViniculum, hLinePad) {
    // size3 is from glyph U221A in the font KaTeX_Size3-Regular
    return "M424," + (2398 + extraViniculum + hLinePad) + "\nc-1.3,-0.7,-38.5,-172,-111.5,-514c-73,-342,-109.8,-513.3,-110.5,-514\nc0,-2,-10.7,14.3,-32,49c-4.7,7.3,-9.8,15.7,-15.5,25c-5.7,9.3,-9.8,16,-12.5,20\ns-5,7,-5,7c-4,-3.3,-8.3,-7.7,-13,-13s-13,-13,-13,-13s76,-122,76,-122s77,-121,77,-121\ns209,968,209,968c0,-2,84.7,-361.7,254,-1079c169.3,-717.3,254.7,-1077.7,256,-1081\nl" + extraViniculum / 4.223 + " -" + extraViniculum + "c4,-6.7,10,-10,18,-10 H400000\nv" + (40 + extraViniculum) + "H1014.6\ns-87.3,378.7,-272.6,1166c-185.3,787.3,-279.3,1182.3,-282,1185\nc-2,6,-10,9,-24,9\nc-8,0,-12,-0.7,-12,-2z M" + (1001 + extraViniculum) + " " + hLinePad + "\nh400000v" + (40 + extraViniculum) + "h-400000z";
  };

  var sqrtSize4 = function sqrtSize4(extraViniculum, hLinePad) {
    // size4 is from glyph U221A in the font KaTeX_Size4-Regular
    return "M473," + (2713 + extraViniculum + hLinePad) + "\nc339.3,-1799.3,509.3,-2700,510,-2702 l" + extraViniculum / 5.298 + " -" + extraViniculum + "\nc3.3,-7.3,9.3,-11,18,-11 H400000v" + (40 + extraViniculum) + "H1017.7\ns-90.5,478,-276.2,1466c-185.7,988,-279.5,1483,-281.5,1485c-2,6,-10,9,-24,9\nc-8,0,-12,-0.7,-12,-2c0,-1.3,-5.3,-32,-16,-92c-50.7,-293.3,-119.7,-693.3,-207,-1200\nc0,-1.3,-5.3,8.7,-16,30c-10.7,21.3,-21.3,42.7,-32,64s-16,33,-16,33s-26,-26,-26,-26\ns76,-153,76,-153s77,-151,77,-151c0.7,0.7,35.7,202,105,604c67.3,400.7,102,602.7,104,\n606zM" + (1001 + extraViniculum) + " " + hLinePad + "h400000v" + (40 + extraViniculum) + "H1017.7z";
  };

  var phasePath = function phasePath(y) {
    var x = y / 2; // x coordinate at top of angle

    return "M400000 " + y + " H0 L" + x + " 0 l65 45 L145 " + (y - 80) + " H400000z";
  };

  var sqrtTall = function sqrtTall(extraViniculum, hLinePad, viewBoxHeight) {
    // sqrtTall is from glyph U23B7 in the font KaTeX_Size4-Regular
    // One path edge has a variable length. It runs vertically from the viniculumn
    // to a point near (14 units) the bottom of the surd. The viniculum
    // is normally 40 units thick. So the length of the line in question is:
    var vertSegment = viewBoxHeight - 54 - hLinePad - extraViniculum;
    return "M702 " + (extraViniculum + hLinePad) + "H400000" + (40 + extraViniculum) + "\nH742v" + vertSegment + "l-4 4-4 4c-.667.7 -2 1.5-4 2.5s-4.167 1.833-6.5 2.5-5.5 1-9.5 1\nh-12l-28-84c-16.667-52-96.667 -294.333-240-727l-212 -643 -85 170\nc-4-3.333-8.333-7.667-13 -13l-13-13l77-155 77-156c66 199.333 139 419.667\n219 661 l218 661zM702 " + hLinePad + "H400000v" + (40 + extraViniculum) + "H742z";
  };

  var sqrtPath = function sqrtPath(size, extraViniculum, viewBoxHeight) {
    extraViniculum = 1000 * extraViniculum; // Convert from document ems to viewBox.

    var path = "";

    switch (size) {
      case "sqrtMain":
        path = sqrtMain(extraViniculum, hLinePad);
        break;

      case "sqrtSize1":
        path = sqrtSize1(extraViniculum, hLinePad);
        break;

      case "sqrtSize2":
        path = sqrtSize2(extraViniculum, hLinePad);
        break;

      case "sqrtSize3":
        path = sqrtSize3(extraViniculum, hLinePad);
        break;

      case "sqrtSize4":
        path = sqrtSize4(extraViniculum, hLinePad);
        break;

      case "sqrtTall":
        path = sqrtTall(extraViniculum, hLinePad, viewBoxHeight);
    }

    return path;
  };
  var innerPath = function innerPath(name, height) {
    // The inner part of stretchy tall delimiters
    switch (name) {
      case "\u239c":
        return "M291 0 H417 V" + height + " H291z M291 0 H417 V" + height + " H291z";

      case "\u2223":
        return "M145 0 H188 V" + height + " H145z M145 0 H188 V" + height + " H145z";

      case "\u2225":
        return "M145 0 H188 V" + height + " H145z M145 0 H188 V" + height + " H145z" + ("M367 0 H410 V" + height + " H367z M367 0 H410 V" + height + " H367z");

      case "\u239f":
        return "M457 0 H583 V" + height + " H457z M457 0 H583 V" + height + " H457z";

      case "\u23a2":
        return "M319 0 H403 V" + height + " H319z M319 0 H403 V" + height + " H319z";

      case "\u23a5":
        return "M263 0 H347 V" + height + " H263z M263 0 H347 V" + height + " H263z";

      case "\u23aa":
        return "M384 0 H504 V" + height + " H384z M384 0 H504 V" + height + " H384z";

      case "\u23d0":
        return "M312 0 H355 V" + height + " H312z M312 0 H355 V" + height + " H312z";

      case "\u2016":
        return "M257 0 H300 V" + height + " H257z M257 0 H300 V" + height + " H257z" + ("M478 0 H521 V" + height + " H478z M478 0 H521 V" + height + " H478z");

      default:
        return "";
    }
  };
  var path$1 = {
    // The doubleleftarrow geometry is from glyph U+21D0 in the font KaTeX Main
    doubleleftarrow: "M262 157\nl10-10c34-36 62.7-77 86-123 3.3-8 5-13.3 5-16 0-5.3-6.7-8-20-8-7.3\n 0-12.2.5-14.5 1.5-2.3 1-4.8 4.5-7.5 10.5-49.3 97.3-121.7 169.3-217 216-28\n 14-57.3 25-88 33-6.7 2-11 3.8-13 5.5-2 1.7-3 4.2-3 7.5s1 5.8 3 7.5\nc2 1.7 6.3 3.5 13 5.5 68 17.3 128.2 47.8 180.5 91.5 52.3 43.7 93.8 96.2 124.5\n 157.5 9.3 8 15.3 12.3 18 13h6c12-.7 18-4 18-10 0-2-1.7-7-5-15-23.3-46-52-87\n-86-123l-10-10h399738v-40H218c328 0 0 0 0 0l-10-8c-26.7-20-65.7-43-117-69 2.7\n-2 6-3.7 10-5 36.7-16 72.3-37.3 107-64l10-8h399782v-40z\nm8 0v40h399730v-40zm0 194v40h399730v-40z",
    // doublerightarrow is from glyph U+21D2 in font KaTeX Main
    doublerightarrow: "M399738 392l\n-10 10c-34 36-62.7 77-86 123-3.3 8-5 13.3-5 16 0 5.3 6.7 8 20 8 7.3 0 12.2-.5\n 14.5-1.5 2.3-1 4.8-4.5 7.5-10.5 49.3-97.3 121.7-169.3 217-216 28-14 57.3-25 88\n-33 6.7-2 11-3.8 13-5.5 2-1.7 3-4.2 3-7.5s-1-5.8-3-7.5c-2-1.7-6.3-3.5-13-5.5-68\n-17.3-128.2-47.8-180.5-91.5-52.3-43.7-93.8-96.2-124.5-157.5-9.3-8-15.3-12.3-18\n-13h-6c-12 .7-18 4-18 10 0 2 1.7 7 5 15 23.3 46 52 87 86 123l10 10H0v40h399782\nc-328 0 0 0 0 0l10 8c26.7 20 65.7 43 117 69-2.7 2-6 3.7-10 5-36.7 16-72.3 37.3\n-107 64l-10 8H0v40zM0 157v40h399730v-40zm0 194v40h399730v-40z",
    // leftarrow is from glyph U+2190 in font KaTeX Main
    leftarrow: "M400000 241H110l3-3c68.7-52.7 113.7-120\n 135-202 4-14.7 6-23 6-25 0-7.3-7-11-21-11-8 0-13.2.8-15.5 2.5-2.3 1.7-4.2 5.8\n-5.5 12.5-1.3 4.7-2.7 10.3-4 17-12 48.7-34.8 92-68.5 130S65.3 228.3 18 247\nc-10 4-16 7.7-18 11 0 8.7 6 14.3 18 17 47.3 18.7 87.8 47 121.5 85S196 441.3 208\n 490c.7 2 1.3 5 2 9s1.2 6.7 1.5 8c.3 1.3 1 3.3 2 6s2.2 4.5 3.5 5.5c1.3 1 3.3\n 1.8 6 2.5s6 1 10 1c14 0 21-3.7 21-11 0-2-2-10.3-6-25-20-79.3-65-146.7-135-202\n l-3-3h399890zM100 241v40h399900v-40z",
    // overbrace is from glyphs U+23A9/23A8/23A7 in font KaTeX_Size4-Regular
    leftbrace: "M6 548l-6-6v-35l6-11c56-104 135.3-181.3 238-232 57.3-28.7 117\n-45 179-50h399577v120H403c-43.3 7-81 15-113 26-100.7 33-179.7 91-237 174-2.7\n 5-6 9-10 13-.7 1-7.3 1-20 1H6z",
    leftbraceunder: "M0 6l6-6h17c12.688 0 19.313.3 20 1 4 4 7.313 8.3 10 13\n 35.313 51.3 80.813 93.8 136.5 127.5 55.688 33.7 117.188 55.8 184.5 66.5.688\n 0 2 .3 4 1 18.688 2.7 76 4.3 172 5h399450v120H429l-6-1c-124.688-8-235-61.7\n-331-161C60.687 138.7 32.312 99.3 7 54L0 41V6z",
    // overgroup is from the MnSymbol package (public domain)
    leftgroup: "M400000 80\nH435C64 80 168.3 229.4 21 260c-5.9 1.2-18 0-18 0-2 0-3-1-3-3v-38C76 61 257 0\n 435 0h399565z",
    leftgroupunder: "M400000 262\nH435C64 262 168.3 112.6 21 82c-5.9-1.2-18 0-18 0-2 0-3 1-3 3v38c76 158 257 219\n 435 219h399565z",
    // Harpoons are from glyph U+21BD in font KaTeX Main
    leftharpoon: "M0 267c.7 5.3 3 10 7 14h399993v-40H93c3.3\n-3.3 10.2-9.5 20.5-18.5s17.8-15.8 22.5-20.5c50.7-52 88-110.3 112-175 4-11.3 5\n-18.3 3-21-1.3-4-7.3-6-18-6-8 0-13 .7-15 2s-4.7 6.7-8 16c-42 98.7-107.3 174.7\n-196 228-6.7 4.7-10.7 8-12 10-1.3 2-2 5.7-2 11zm100-26v40h399900v-40z",
    leftharpoonplus: "M0 267c.7 5.3 3 10 7 14h399993v-40H93c3.3-3.3 10.2-9.5\n 20.5-18.5s17.8-15.8 22.5-20.5c50.7-52 88-110.3 112-175 4-11.3 5-18.3 3-21-1.3\n-4-7.3-6-18-6-8 0-13 .7-15 2s-4.7 6.7-8 16c-42 98.7-107.3 174.7-196 228-6.7 4.7\n-10.7 8-12 10-1.3 2-2 5.7-2 11zm100-26v40h399900v-40zM0 435v40h400000v-40z\nm0 0v40h400000v-40z",
    leftharpoondown: "M7 241c-4 4-6.333 8.667-7 14 0 5.333.667 9 2 11s5.333\n 5.333 12 10c90.667 54 156 130 196 228 3.333 10.667 6.333 16.333 9 17 2 .667 5\n 1 9 1h5c10.667 0 16.667-2 18-6 2-2.667 1-9.667-3-21-32-87.333-82.667-157.667\n-152-211l-3-3h399907v-40zM93 281 H400000 v-40L7 241z",
    leftharpoondownplus: "M7 435c-4 4-6.3 8.7-7 14 0 5.3.7 9 2 11s5.3 5.3 12\n 10c90.7 54 156 130 196 228 3.3 10.7 6.3 16.3 9 17 2 .7 5 1 9 1h5c10.7 0 16.7\n-2 18-6 2-2.7 1-9.7-3-21-32-87.3-82.7-157.7-152-211l-3-3h399907v-40H7zm93 0\nv40h399900v-40zM0 241v40h399900v-40zm0 0v40h399900v-40z",
    // hook is from glyph U+21A9 in font KaTeX Main
    lefthook: "M400000 281 H103s-33-11.2-61-33.5S0 197.3 0 164s14.2-61.2 42.5\n-83.5C70.8 58.2 104 47 142 47 c16.7 0 25 6.7 25 20 0 12-8.7 18.7-26 20-40 3.3\n-68.7 15.7-86 37-10 12-15 25.3-15 40 0 22.7 9.8 40.7 29.5 54 19.7 13.3 43.5 21\n 71.5 23h399859zM103 281v-40h399897v40z",
    leftlinesegment: "M40 281 V428 H0 V94 H40 V241 H400000 v40z\nM40 281 V428 H0 V94 H40 V241 H400000 v40z",
    leftmapsto: "M40 281 V448H0V74H40V241H400000v40z\nM40 281 V448H0V74H40V241H400000v40z",
    // tofrom is from glyph U+21C4 in font KaTeX AMS Regular
    leftToFrom: "M0 147h400000v40H0zm0 214c68 40 115.7 95.7 143 167h22c15.3 0 23\n-.3 23-1 0-1.3-5.3-13.7-16-37-18-35.3-41.3-69-70-101l-7-8h399905v-40H95l7-8\nc28.7-32 52-65.7 70-101 10.7-23.3 16-35.7 16-37 0-.7-7.7-1-23-1h-22C115.7 265.3\n 68 321 0 361zm0-174v-40h399900v40zm100 154v40h399900v-40z",
    longequal: "M0 50 h400000 v40H0z m0 194h40000v40H0z\nM0 50 h400000 v40H0z m0 194h40000v40H0z",
    midbrace: "M200428 334\nc-100.7-8.3-195.3-44-280-108-55.3-42-101.7-93-139-153l-9-14c-2.7 4-5.7 8.7-9 14\n-53.3 86.7-123.7 153-211 199-66.7 36-137.3 56.3-212 62H0V214h199568c178.3-11.7\n 311.7-78.3 403-201 6-8 9.7-12 11-12 .7-.7 6.7-1 18-1s17.3.3 18 1c1.3 0 5 4 11\n 12 44.7 59.3 101.3 106.3 170 141s145.3 54.3 229 60h199572v120z",
    midbraceunder: "M199572 214\nc100.7 8.3 195.3 44 280 108 55.3 42 101.7 93 139 153l9 14c2.7-4 5.7-8.7 9-14\n 53.3-86.7 123.7-153 211-199 66.7-36 137.3-56.3 212-62h199568v120H200432c-178.3\n 11.7-311.7 78.3-403 201-6 8-9.7 12-11 12-.7.7-6.7 1-18 1s-17.3-.3-18-1c-1.3 0\n-5-4-11-12-44.7-59.3-101.3-106.3-170-141s-145.3-54.3-229-60H0V214z",
    oiintSize1: "M512.6 71.6c272.6 0 320.3 106.8 320.3 178.2 0 70.8-47.7 177.6\n-320.3 177.6S193.1 320.6 193.1 249.8c0-71.4 46.9-178.2 319.5-178.2z\nm368.1 178.2c0-86.4-60.9-215.4-368.1-215.4-306.4 0-367.3 129-367.3 215.4 0 85.8\n60.9 214.8 367.3 214.8 307.2 0 368.1-129 368.1-214.8z",
    oiintSize2: "M757.8 100.1c384.7 0 451.1 137.6 451.1 230 0 91.3-66.4 228.8\n-451.1 228.8-386.3 0-452.7-137.5-452.7-228.8 0-92.4 66.4-230 452.7-230z\nm502.4 230c0-111.2-82.4-277.2-502.4-277.2s-504 166-504 277.2\nc0 110 84 276 504 276s502.4-166 502.4-276z",
    oiiintSize1: "M681.4 71.6c408.9 0 480.5 106.8 480.5 178.2 0 70.8-71.6 177.6\n-480.5 177.6S202.1 320.6 202.1 249.8c0-71.4 70.5-178.2 479.3-178.2z\nm525.8 178.2c0-86.4-86.8-215.4-525.7-215.4-437.9 0-524.7 129-524.7 215.4 0\n85.8 86.8 214.8 524.7 214.8 438.9 0 525.7-129 525.7-214.8z",
    oiiintSize2: "M1021.2 53c603.6 0 707.8 165.8 707.8 277.2 0 110-104.2 275.8\n-707.8 275.8-606 0-710.2-165.8-710.2-275.8C311 218.8 415.2 53 1021.2 53z\nm770.4 277.1c0-131.2-126.4-327.6-770.5-327.6S248.4 198.9 248.4 330.1\nc0 130 128.8 326.4 772.7 326.4s770.5-196.4 770.5-326.4z",
    rightarrow: "M0 241v40h399891c-47.3 35.3-84 78-110 128\n-16.7 32-27.7 63.7-33 95 0 1.3-.2 2.7-.5 4-.3 1.3-.5 2.3-.5 3 0 7.3 6.7 11 20\n 11 8 0 13.2-.8 15.5-2.5 2.3-1.7 4.2-5.5 5.5-11.5 2-13.3 5.7-27 11-41 14.7-44.7\n 39-84.5 73-119.5s73.7-60.2 119-75.5c6-2 9-5.7 9-11s-3-9-9-11c-45.3-15.3-85\n-40.5-119-75.5s-58.3-74.8-73-119.5c-4.7-14-8.3-27.3-11-40-1.3-6.7-3.2-10.8-5.5\n-12.5-2.3-1.7-7.5-2.5-15.5-2.5-14 0-21 3.7-21 11 0 2 2 10.3 6 25 20.7 83.3 67\n 151.7 139 205zm0 0v40h399900v-40z",
    rightbrace: "M400000 542l\n-6 6h-17c-12.7 0-19.3-.3-20-1-4-4-7.3-8.3-10-13-35.3-51.3-80.8-93.8-136.5-127.5\ns-117.2-55.8-184.5-66.5c-.7 0-2-.3-4-1-18.7-2.7-76-4.3-172-5H0V214h399571l6 1\nc124.7 8 235 61.7 331 161 31.3 33.3 59.7 72.7 85 118l7 13v35z",
    rightbraceunder: "M399994 0l6 6v35l-6 11c-56 104-135.3 181.3-238 232-57.3\n 28.7-117 45-179 50H-300V214h399897c43.3-7 81-15 113-26 100.7-33 179.7-91 237\n-174 2.7-5 6-9 10-13 .7-1 7.3-1 20-1h17z",
    rightgroup: "M0 80h399565c371 0 266.7 149.4 414 180 5.9 1.2 18 0 18 0 2 0\n 3-1 3-3v-38c-76-158-257-219-435-219H0z",
    rightgroupunder: "M0 262h399565c371 0 266.7-149.4 414-180 5.9-1.2 18 0 18\n 0 2 0 3 1 3 3v38c-76 158-257 219-435 219H0z",
    rightharpoon: "M0 241v40h399993c4.7-4.7 7-9.3 7-14 0-9.3\n-3.7-15.3-11-18-92.7-56.7-159-133.7-199-231-3.3-9.3-6-14.7-8-16-2-1.3-7-2-15-2\n-10.7 0-16.7 2-18 6-2 2.7-1 9.7 3 21 15.3 42 36.7 81.8 64 119.5 27.3 37.7 58\n 69.2 92 94.5zm0 0v40h399900v-40z",
    rightharpoonplus: "M0 241v40h399993c4.7-4.7 7-9.3 7-14 0-9.3-3.7-15.3-11\n-18-92.7-56.7-159-133.7-199-231-3.3-9.3-6-14.7-8-16-2-1.3-7-2-15-2-10.7 0-16.7\n 2-18 6-2 2.7-1 9.7 3 21 15.3 42 36.7 81.8 64 119.5 27.3 37.7 58 69.2 92 94.5z\nm0 0v40h399900v-40z m100 194v40h399900v-40zm0 0v40h399900v-40z",
    rightharpoondown: "M399747 511c0 7.3 6.7 11 20 11 8 0 13-.8 15-2.5s4.7-6.8\n 8-15.5c40-94 99.3-166.3 178-217 13.3-8 20.3-12.3 21-13 5.3-3.3 8.5-5.8 9.5\n-7.5 1-1.7 1.5-5.2 1.5-10.5s-2.3-10.3-7-15H0v40h399908c-34 25.3-64.7 57-92 95\n-27.3 38-48.7 77.7-64 119-3.3 8.7-5 14-5 16zM0 241v40h399900v-40z",
    rightharpoondownplus: "M399747 705c0 7.3 6.7 11 20 11 8 0 13-.8\n 15-2.5s4.7-6.8 8-15.5c40-94 99.3-166.3 178-217 13.3-8 20.3-12.3 21-13 5.3-3.3\n 8.5-5.8 9.5-7.5 1-1.7 1.5-5.2 1.5-10.5s-2.3-10.3-7-15H0v40h399908c-34 25.3\n-64.7 57-92 95-27.3 38-48.7 77.7-64 119-3.3 8.7-5 14-5 16zM0 435v40h399900v-40z\nm0-194v40h400000v-40zm0 0v40h400000v-40z",
    righthook: "M399859 241c-764 0 0 0 0 0 40-3.3 68.7-15.7 86-37 10-12 15-25.3\n 15-40 0-22.7-9.8-40.7-29.5-54-19.7-13.3-43.5-21-71.5-23-17.3-1.3-26-8-26-20 0\n-13.3 8.7-20 26-20 38 0 71 11.2 99 33.5 0 0 7 5.6 21 16.7 14 11.2 21 33.5 21\n 66.8s-14 61.2-42 83.5c-28 22.3-61 33.5-99 33.5L0 241z M0 281v-40h399859v40z",
    rightlinesegment: "M399960 241 V94 h40 V428 h-40 V281 H0 v-40z\nM399960 241 V94 h40 V428 h-40 V281 H0 v-40z",
    rightToFrom: "M400000 167c-70.7-42-118-97.7-142-167h-23c-15.3 0-23 .3-23\n 1 0 1.3 5.3 13.7 16 37 18 35.3 41.3 69 70 101l7 8H0v40h399905l-7 8c-28.7 32\n-52 65.7-70 101-10.7 23.3-16 35.7-16 37 0 .7 7.7 1 23 1h23c24-69.3 71.3-125 142\n-167z M100 147v40h399900v-40zM0 341v40h399900v-40z",
    // twoheadleftarrow is from glyph U+219E in font KaTeX AMS Regular
    twoheadleftarrow: "M0 167c68 40\n 115.7 95.7 143 167h22c15.3 0 23-.3 23-1 0-1.3-5.3-13.7-16-37-18-35.3-41.3-69\n-70-101l-7-8h125l9 7c50.7 39.3 85 86 103 140h46c0-4.7-6.3-18.7-19-42-18-35.3\n-40-67.3-66-96l-9-9h399716v-40H284l9-9c26-28.7 48-60.7 66-96 12.7-23.333 19\n-37.333 19-42h-46c-18 54-52.3 100.7-103 140l-9 7H95l7-8c28.7-32 52-65.7 70-101\n 10.7-23.333 16-35.7 16-37 0-.7-7.7-1-23-1h-22C115.7 71.3 68 127 0 167z",
    twoheadrightarrow: "M400000 167\nc-68-40-115.7-95.7-143-167h-22c-15.3 0-23 .3-23 1 0 1.3 5.3 13.7 16 37 18 35.3\n 41.3 69 70 101l7 8h-125l-9-7c-50.7-39.3-85-86-103-140h-46c0 4.7 6.3 18.7 19 42\n 18 35.3 40 67.3 66 96l9 9H0v40h399716l-9 9c-26 28.7-48 60.7-66 96-12.7 23.333\n-19 37.333-19 42h46c18-54 52.3-100.7 103-140l9-7h125l-7 8c-28.7 32-52 65.7-70\n 101-10.7 23.333-16 35.7-16 37 0 .7 7.7 1 23 1h22c27.3-71.3 75-127 143-167z",
    // tilde1 is a modified version of a glyph from the MnSymbol package
    tilde1: "M200 55.538c-77 0-168 73.953-177 73.953-3 0-7\n-2.175-9-5.437L2 97c-1-2-2-4-2-6 0-4 2-7 5-9l20-12C116 12 171 0 207 0c86 0\n 114 68 191 68 78 0 168-68 177-68 4 0 7 2 9 5l12 19c1 2.175 2 4.35 2 6.525 0\n 4.35-2 7.613-5 9.788l-19 13.05c-92 63.077-116.937 75.308-183 76.128\n-68.267.847-113-73.952-191-73.952z",
    // ditto tilde2, tilde3, & tilde4
    tilde2: "M344 55.266c-142 0-300.638 81.316-311.5 86.418\n-8.01 3.762-22.5 10.91-23.5 5.562L1 120c-1-2-1-3-1-4 0-5 3-9 8-10l18.4-9C160.9\n 31.9 283 0 358 0c148 0 188 122 331 122s314-97 326-97c4 0 8 2 10 7l7 21.114\nc1 2.14 1 3.21 1 4.28 0 5.347-3 9.626-7 10.696l-22.3 12.622C852.6 158.372 751\n 181.476 676 181.476c-149 0-189-126.21-332-126.21z",
    tilde3: "M786 59C457 59 32 175.242 13 175.242c-6 0-10-3.457\n-11-10.37L.15 138c-1-7 3-12 10-13l19.2-6.4C378.4 40.7 634.3 0 804.3 0c337 0\n 411.8 157 746.8 157 328 0 754-112 773-112 5 0 10 3 11 9l1 14.075c1 8.066-.697\n 16.595-6.697 17.492l-21.052 7.31c-367.9 98.146-609.15 122.696-778.15 122.696\n -338 0-409-156.573-744-156.573z",
    tilde4: "M786 58C457 58 32 177.487 13 177.487c-6 0-10-3.345\n-11-10.035L.15 143c-1-7 3-12 10-13l22-6.7C381.2 35 637.15 0 807.15 0c337 0 409\n 177 744 177 328 0 754-127 773-127 5 0 10 3 11 9l1 14.794c1 7.805-3 13.38-9\n 14.495l-20.7 5.574c-366.85 99.79-607.3 139.372-776.3 139.372-338 0-409\n -175.236-744-175.236z",
    // vec is from glyph U+20D7 in font KaTeX Main
    vec: "M377 20c0-5.333 1.833-10 5.5-14S391 0 397 0c4.667 0 8.667 1.667 12 5\n3.333 2.667 6.667 9 10 19 6.667 24.667 20.333 43.667 41 57 7.333 4.667 11\n10.667 11 18 0 6-1 10-3 12s-6.667 5-14 9c-28.667 14.667-53.667 35.667-75 63\n-1.333 1.333-3.167 3.5-5.5 6.5s-4 4.833-5 5.5c-1 .667-2.5 1.333-4.5 2s-4.333 1\n-7 1c-4.667 0-9.167-1.833-13.5-5.5S337 184 337 178c0-12.667 15.667-32.333 47-59\nH213l-171-1c-8.667-6-13-12.333-13-19 0-4.667 4.333-11.333 13-20h359\nc-16-25.333-24-45-24-59z",
    // widehat1 is a modified version of a glyph from the MnSymbol package
    widehat1: "M529 0h5l519 115c5 1 9 5 9 10 0 1-1 2-1 3l-4 22\nc-1 5-5 9-11 9h-2L532 67 19 159h-2c-5 0-9-4-11-9l-5-22c-1-6 2-12 8-13z",
    // ditto widehat2, widehat3, & widehat4
    widehat2: "M1181 0h2l1171 176c6 0 10 5 10 11l-2 23c-1 6-5 10\n-11 10h-1L1182 67 15 220h-1c-6 0-10-4-11-10l-2-23c-1-6 4-11 10-11z",
    widehat3: "M1181 0h2l1171 236c6 0 10 5 10 11l-2 23c-1 6-5 10\n-11 10h-1L1182 67 15 280h-1c-6 0-10-4-11-10l-2-23c-1-6 4-11 10-11z",
    widehat4: "M1181 0h2l1171 296c6 0 10 5 10 11l-2 23c-1 6-5 10\n-11 10h-1L1182 67 15 340h-1c-6 0-10-4-11-10l-2-23c-1-6 4-11 10-11z",
    // widecheck paths are all inverted versions of widehat
    widecheck1: "M529,159h5l519,-115c5,-1,9,-5,9,-10c0,-1,-1,-2,-1,-3l-4,-22c-1,\n-5,-5,-9,-11,-9h-2l-512,92l-513,-92h-2c-5,0,-9,4,-11,9l-5,22c-1,6,2,12,8,13z",
    widecheck2: "M1181,220h2l1171,-176c6,0,10,-5,10,-11l-2,-23c-1,-6,-5,-10,\n-11,-10h-1l-1168,153l-1167,-153h-1c-6,0,-10,4,-11,10l-2,23c-1,6,4,11,10,11z",
    widecheck3: "M1181,280h2l1171,-236c6,0,10,-5,10,-11l-2,-23c-1,-6,-5,-10,\n-11,-10h-1l-1168,213l-1167,-213h-1c-6,0,-10,4,-11,10l-2,23c-1,6,4,11,10,11z",
    widecheck4: "M1181,340h2l1171,-296c6,0,10,-5,10,-11l-2,-23c-1,-6,-5,-10,\n-11,-10h-1l-1168,273l-1167,-273h-1c-6,0,-10,4,-11,10l-2,23c-1,6,4,11,10,11z",
    // The next ten paths support reaction arrows from the mhchem package.
    // Arrows for \ce{<-->} are offset from xAxis by 0.22ex, per mhchem in LaTeX
    // baraboveleftarrow is mostly from from glyph U+2190 in font KaTeX Main
    baraboveleftarrow: "M400000 620h-399890l3 -3c68.7 -52.7 113.7 -120 135 -202\nc4 -14.7 6 -23 6 -25c0 -7.3 -7 -11 -21 -11c-8 0 -13.2 0.8 -15.5 2.5\nc-2.3 1.7 -4.2 5.8 -5.5 12.5c-1.3 4.7 -2.7 10.3 -4 17c-12 48.7 -34.8 92 -68.5 130\ns-74.2 66.3 -121.5 85c-10 4 -16 7.7 -18 11c0 8.7 6 14.3 18 17c47.3 18.7 87.8 47\n121.5 85s56.5 81.3 68.5 130c0.7 2 1.3 5 2 9s1.2 6.7 1.5 8c0.3 1.3 1 3.3 2 6\ns2.2 4.5 3.5 5.5c1.3 1 3.3 1.8 6 2.5s6 1 10 1c14 0 21 -3.7 21 -11\nc0 -2 -2 -10.3 -6 -25c-20 -79.3 -65 -146.7 -135 -202l-3 -3h399890z\nM100 620v40h399900v-40z M0 241v40h399900v-40zM0 241v40h399900v-40z",
    // rightarrowabovebar is mostly from glyph U+2192, KaTeX Main
    rightarrowabovebar: "M0 241v40h399891c-47.3 35.3-84 78-110 128-16.7 32\n-27.7 63.7-33 95 0 1.3-.2 2.7-.5 4-.3 1.3-.5 2.3-.5 3 0 7.3 6.7 11 20 11 8 0\n13.2-.8 15.5-2.5 2.3-1.7 4.2-5.5 5.5-11.5 2-13.3 5.7-27 11-41 14.7-44.7 39\n-84.5 73-119.5s73.7-60.2 119-75.5c6-2 9-5.7 9-11s-3-9-9-11c-45.3-15.3-85-40.5\n-119-75.5s-58.3-74.8-73-119.5c-4.7-14-8.3-27.3-11-40-1.3-6.7-3.2-10.8-5.5\n-12.5-2.3-1.7-7.5-2.5-15.5-2.5-14 0-21 3.7-21 11 0 2 2 10.3 6 25 20.7 83.3 67\n151.7 139 205zm96 379h399894v40H0zm0 0h399904v40H0z",
    // The short left harpoon has 0.5em (i.e. 500 units) kern on the left end.
    // Ref from mhchem.sty: \rlap{\raisebox{-.22ex}{$\kern0.5em
    baraboveshortleftharpoon: "M507,435c-4,4,-6.3,8.7,-7,14c0,5.3,0.7,9,2,11\nc1.3,2,5.3,5.3,12,10c90.7,54,156,130,196,228c3.3,10.7,6.3,16.3,9,17\nc2,0.7,5,1,9,1c0,0,5,0,5,0c10.7,0,16.7,-2,18,-6c2,-2.7,1,-9.7,-3,-21\nc-32,-87.3,-82.7,-157.7,-152,-211c0,0,-3,-3,-3,-3l399351,0l0,-40\nc-398570,0,-399437,0,-399437,0z M593 435 v40 H399500 v-40z\nM0 281 v-40 H399908 v40z M0 281 v-40 H399908 v40z",
    rightharpoonaboveshortbar: "M0,241 l0,40c399126,0,399993,0,399993,0\nc4.7,-4.7,7,-9.3,7,-14c0,-9.3,-3.7,-15.3,-11,-18c-92.7,-56.7,-159,-133.7,-199,\n-231c-3.3,-9.3,-6,-14.7,-8,-16c-2,-1.3,-7,-2,-15,-2c-10.7,0,-16.7,2,-18,6\nc-2,2.7,-1,9.7,3,21c15.3,42,36.7,81.8,64,119.5c27.3,37.7,58,69.2,92,94.5z\nM0 241 v40 H399908 v-40z M0 475 v-40 H399500 v40z M0 475 v-40 H399500 v40z",
    shortbaraboveleftharpoon: "M7,435c-4,4,-6.3,8.7,-7,14c0,5.3,0.7,9,2,11\nc1.3,2,5.3,5.3,12,10c90.7,54,156,130,196,228c3.3,10.7,6.3,16.3,9,17c2,0.7,5,1,9,\n1c0,0,5,0,5,0c10.7,0,16.7,-2,18,-6c2,-2.7,1,-9.7,-3,-21c-32,-87.3,-82.7,-157.7,\n-152,-211c0,0,-3,-3,-3,-3l399907,0l0,-40c-399126,0,-399993,0,-399993,0z\nM93 435 v40 H400000 v-40z M500 241 v40 H400000 v-40z M500 241 v40 H400000 v-40z",
    shortrightharpoonabovebar: "M53,241l0,40c398570,0,399437,0,399437,0\nc4.7,-4.7,7,-9.3,7,-14c0,-9.3,-3.7,-15.3,-11,-18c-92.7,-56.7,-159,-133.7,-199,\n-231c-3.3,-9.3,-6,-14.7,-8,-16c-2,-1.3,-7,-2,-15,-2c-10.7,0,-16.7,2,-18,6\nc-2,2.7,-1,9.7,3,21c15.3,42,36.7,81.8,64,119.5c27.3,37.7,58,69.2,92,94.5z\nM500 241 v40 H399408 v-40z M500 435 v40 H400000 v-40z"
  };

  /**
   * This node represents a document fragment, which contains elements, but when
   * placed into the DOM doesn't have any representation itself. It only contains
   * children and doesn't have any DOM node properties.
   */
  class DocumentFragment {
    // HtmlDomNode
    // Never used; needed for satisfying interface.
    constructor(children) {
      this.children = void 0;
      this.classes = void 0;
      this.height = void 0;
      this.depth = void 0;
      this.maxFontSize = void 0;
      this.style = void 0;
      this.children = children;
      this.classes = [];
      this.height = 0;
      this.depth = 0;
      this.maxFontSize = 0;
      this.style = {};
    }

    hasClass(className) {
      return utils$1.contains(this.classes, className);
    }
    /** Convert the fragment into a node. */


    toNode() {
      var frag = document.createDocumentFragment();

      for (var i = 0; i < this.children.length; i++) {
        frag.appendChild(this.children[i].toNode());
      }

      return frag;
    }
    /** Convert the fragment into HTML markup. */


    toMarkup() {
      var markup = ""; // Simply concatenate the markup for the children together.

      for (var i = 0; i < this.children.length; i++) {
        markup += this.children[i].toMarkup();
      }

      return markup;
    }
    /**
     * Converts the math node into a string, similar to innerText. Applies to
     * MathDomNode's only.
     */


    toText() {
      // To avoid this, we would subclass documentFragment separately for
      // MathML, but polyfills for subclassing is expensive per PR 1469.
      // $FlowFixMe: Only works for ChildType = MathDomNode.
      var toText = child => child.toText();

      return this.children.map(toText).join("");
    }

  }

  // This file is GENERATED by buildMetrics.sh. DO NOT MODIFY.
  var fontMetricsData = {
    "AMS-Regular": {
      "32": [0, 0, 0, 0, 0.25],
      "65": [0, 0.68889, 0, 0, 0.72222],
      "66": [0, 0.68889, 0, 0, 0.66667],
      "67": [0, 0.68889, 0, 0, 0.72222],
      "68": [0, 0.68889, 0, 0, 0.72222],
      "69": [0, 0.68889, 0, 0, 0.66667],
      "70": [0, 0.68889, 0, 0, 0.61111],
      "71": [0, 0.68889, 0, 0, 0.77778],
      "72": [0, 0.68889, 0, 0, 0.77778],
      "73": [0, 0.68889, 0, 0, 0.38889],
      "74": [0.16667, 0.68889, 0, 0, 0.5],
      "75": [0, 0.68889, 0, 0, 0.77778],
      "76": [0, 0.68889, 0, 0, 0.66667],
      "77": [0, 0.68889, 0, 0, 0.94445],
      "78": [0, 0.68889, 0, 0, 0.72222],
      "79": [0.16667, 0.68889, 0, 0, 0.77778],
      "80": [0, 0.68889, 0, 0, 0.61111],
      "81": [0.16667, 0.68889, 0, 0, 0.77778],
      "82": [0, 0.68889, 0, 0, 0.72222],
      "83": [0, 0.68889, 0, 0, 0.55556],
      "84": [0, 0.68889, 0, 0, 0.66667],
      "85": [0, 0.68889, 0, 0, 0.72222],
      "86": [0, 0.68889, 0, 0, 0.72222],
      "87": [0, 0.68889, 0, 0, 1.0],
      "88": [0, 0.68889, 0, 0, 0.72222],
      "89": [0, 0.68889, 0, 0, 0.72222],
      "90": [0, 0.68889, 0, 0, 0.66667],
      "107": [0, 0.68889, 0, 0, 0.55556],
      "160": [0, 0, 0, 0, 0.25],
      "165": [0, 0.675, 0.025, 0, 0.75],
      "174": [0.15559, 0.69224, 0, 0, 0.94666],
      "240": [0, 0.68889, 0, 0, 0.55556],
      "295": [0, 0.68889, 0, 0, 0.54028],
      "710": [0, 0.825, 0, 0, 2.33334],
      "732": [0, 0.9, 0, 0, 2.33334],
      "770": [0, 0.825, 0, 0, 2.33334],
      "771": [0, 0.9, 0, 0, 2.33334],
      "989": [0.08167, 0.58167, 0, 0, 0.77778],
      "1008": [0, 0.43056, 0.04028, 0, 0.66667],
      "8245": [0, 0.54986, 0, 0, 0.275],
      "8463": [0, 0.68889, 0, 0, 0.54028],
      "8487": [0, 0.68889, 0, 0, 0.72222],
      "8498": [0, 0.68889, 0, 0, 0.55556],
      "8502": [0, 0.68889, 0, 0, 0.66667],
      "8503": [0, 0.68889, 0, 0, 0.44445],
      "8504": [0, 0.68889, 0, 0, 0.66667],
      "8513": [0, 0.68889, 0, 0, 0.63889],
      "8592": [-0.03598, 0.46402, 0, 0, 0.5],
      "8594": [-0.03598, 0.46402, 0, 0, 0.5],
      "8602": [-0.13313, 0.36687, 0, 0, 1.0],
      "8603": [-0.13313, 0.36687, 0, 0, 1.0],
      "8606": [0.01354, 0.52239, 0, 0, 1.0],
      "8608": [0.01354, 0.52239, 0, 0, 1.0],
      "8610": [0.01354, 0.52239, 0, 0, 1.11111],
      "8611": [0.01354, 0.52239, 0, 0, 1.11111],
      "8619": [0, 0.54986, 0, 0, 1.0],
      "8620": [0, 0.54986, 0, 0, 1.0],
      "8621": [-0.13313, 0.37788, 0, 0, 1.38889],
      "8622": [-0.13313, 0.36687, 0, 0, 1.0],
      "8624": [0, 0.69224, 0, 0, 0.5],
      "8625": [0, 0.69224, 0, 0, 0.5],
      "8630": [0, 0.43056, 0, 0, 1.0],
      "8631": [0, 0.43056, 0, 0, 1.0],
      "8634": [0.08198, 0.58198, 0, 0, 0.77778],
      "8635": [0.08198, 0.58198, 0, 0, 0.77778],
      "8638": [0.19444, 0.69224, 0, 0, 0.41667],
      "8639": [0.19444, 0.69224, 0, 0, 0.41667],
      "8642": [0.19444, 0.69224, 0, 0, 0.41667],
      "8643": [0.19444, 0.69224, 0, 0, 0.41667],
      "8644": [0.1808, 0.675, 0, 0, 1.0],
      "8646": [0.1808, 0.675, 0, 0, 1.0],
      "8647": [0.1808, 0.675, 0, 0, 1.0],
      "8648": [0.19444, 0.69224, 0, 0, 0.83334],
      "8649": [0.1808, 0.675, 0, 0, 1.0],
      "8650": [0.19444, 0.69224, 0, 0, 0.83334],
      "8651": [0.01354, 0.52239, 0, 0, 1.0],
      "8652": [0.01354, 0.52239, 0, 0, 1.0],
      "8653": [-0.13313, 0.36687, 0, 0, 1.0],
      "8654": [-0.13313, 0.36687, 0, 0, 1.0],
      "8655": [-0.13313, 0.36687, 0, 0, 1.0],
      "8666": [0.13667, 0.63667, 0, 0, 1.0],
      "8667": [0.13667, 0.63667, 0, 0, 1.0],
      "8669": [-0.13313, 0.37788, 0, 0, 1.0],
      "8672": [-0.064, 0.437, 0, 0, 1.334],
      "8674": [-0.064, 0.437, 0, 0, 1.334],
      "8705": [0, 0.825, 0, 0, 0.5],
      "8708": [0, 0.68889, 0, 0, 0.55556],
      "8709": [0.08167, 0.58167, 0, 0, 0.77778],
      "8717": [0, 0.43056, 0, 0, 0.42917],
      "8722": [-0.03598, 0.46402, 0, 0, 0.5],
      "8724": [0.08198, 0.69224, 0, 0, 0.77778],
      "8726": [0.08167, 0.58167, 0, 0, 0.77778],
      "8733": [0, 0.69224, 0, 0, 0.77778],
      "8736": [0, 0.69224, 0, 0, 0.72222],
      "8737": [0, 0.69224, 0, 0, 0.72222],
      "8738": [0.03517, 0.52239, 0, 0, 0.72222],
      "8739": [0.08167, 0.58167, 0, 0, 0.22222],
      "8740": [0.25142, 0.74111, 0, 0, 0.27778],
      "8741": [0.08167, 0.58167, 0, 0, 0.38889],
      "8742": [0.25142, 0.74111, 0, 0, 0.5],
      "8756": [0, 0.69224, 0, 0, 0.66667],
      "8757": [0, 0.69224, 0, 0, 0.66667],
      "8764": [-0.13313, 0.36687, 0, 0, 0.77778],
      "8765": [-0.13313, 0.37788, 0, 0, 0.77778],
      "8769": [-0.13313, 0.36687, 0, 0, 0.77778],
      "8770": [-0.03625, 0.46375, 0, 0, 0.77778],
      "8774": [0.30274, 0.79383, 0, 0, 0.77778],
      "8776": [-0.01688, 0.48312, 0, 0, 0.77778],
      "8778": [0.08167, 0.58167, 0, 0, 0.77778],
      "8782": [0.06062, 0.54986, 0, 0, 0.77778],
      "8783": [0.06062, 0.54986, 0, 0, 0.77778],
      "8785": [0.08198, 0.58198, 0, 0, 0.77778],
      "8786": [0.08198, 0.58198, 0, 0, 0.77778],
      "8787": [0.08198, 0.58198, 0, 0, 0.77778],
      "8790": [0, 0.69224, 0, 0, 0.77778],
      "8791": [0.22958, 0.72958, 0, 0, 0.77778],
      "8796": [0.08198, 0.91667, 0, 0, 0.77778],
      "8806": [0.25583, 0.75583, 0, 0, 0.77778],
      "8807": [0.25583, 0.75583, 0, 0, 0.77778],
      "8808": [0.25142, 0.75726, 0, 0, 0.77778],
      "8809": [0.25142, 0.75726, 0, 0, 0.77778],
      "8812": [0.25583, 0.75583, 0, 0, 0.5],
      "8814": [0.20576, 0.70576, 0, 0, 0.77778],
      "8815": [0.20576, 0.70576, 0, 0, 0.77778],
      "8816": [0.30274, 0.79383, 0, 0, 0.77778],
      "8817": [0.30274, 0.79383, 0, 0, 0.77778],
      "8818": [0.22958, 0.72958, 0, 0, 0.77778],
      "8819": [0.22958, 0.72958, 0, 0, 0.77778],
      "8822": [0.1808, 0.675, 0, 0, 0.77778],
      "8823": [0.1808, 0.675, 0, 0, 0.77778],
      "8828": [0.13667, 0.63667, 0, 0, 0.77778],
      "8829": [0.13667, 0.63667, 0, 0, 0.77778],
      "8830": [0.22958, 0.72958, 0, 0, 0.77778],
      "8831": [0.22958, 0.72958, 0, 0, 0.77778],
      "8832": [0.20576, 0.70576, 0, 0, 0.77778],
      "8833": [0.20576, 0.70576, 0, 0, 0.77778],
      "8840": [0.30274, 0.79383, 0, 0, 0.77778],
      "8841": [0.30274, 0.79383, 0, 0, 0.77778],
      "8842": [0.13597, 0.63597, 0, 0, 0.77778],
      "8843": [0.13597, 0.63597, 0, 0, 0.77778],
      "8847": [0.03517, 0.54986, 0, 0, 0.77778],
      "8848": [0.03517, 0.54986, 0, 0, 0.77778],
      "8858": [0.08198, 0.58198, 0, 0, 0.77778],
      "8859": [0.08198, 0.58198, 0, 0, 0.77778],
      "8861": [0.08198, 0.58198, 0, 0, 0.77778],
      "8862": [0, 0.675, 0, 0, 0.77778],
      "8863": [0, 0.675, 0, 0, 0.77778],
      "8864": [0, 0.675, 0, 0, 0.77778],
      "8865": [0, 0.675, 0, 0, 0.77778],
      "8872": [0, 0.69224, 0, 0, 0.61111],
      "8873": [0, 0.69224, 0, 0, 0.72222],
      "8874": [0, 0.69224, 0, 0, 0.88889],
      "8876": [0, 0.68889, 0, 0, 0.61111],
      "8877": [0, 0.68889, 0, 0, 0.61111],
      "8878": [0, 0.68889, 0, 0, 0.72222],
      "8879": [0, 0.68889, 0, 0, 0.72222],
      "8882": [0.03517, 0.54986, 0, 0, 0.77778],
      "8883": [0.03517, 0.54986, 0, 0, 0.77778],
      "8884": [0.13667, 0.63667, 0, 0, 0.77778],
      "8885": [0.13667, 0.63667, 0, 0, 0.77778],
      "8888": [0, 0.54986, 0, 0, 1.11111],
      "8890": [0.19444, 0.43056, 0, 0, 0.55556],
      "8891": [0.19444, 0.69224, 0, 0, 0.61111],
      "8892": [0.19444, 0.69224, 0, 0, 0.61111],
      "8901": [0, 0.54986, 0, 0, 0.27778],
      "8903": [0.08167, 0.58167, 0, 0, 0.77778],
      "8905": [0.08167, 0.58167, 0, 0, 0.77778],
      "8906": [0.08167, 0.58167, 0, 0, 0.77778],
      "8907": [0, 0.69224, 0, 0, 0.77778],
      "8908": [0, 0.69224, 0, 0, 0.77778],
      "8909": [-0.03598, 0.46402, 0, 0, 0.77778],
      "8910": [0, 0.54986, 0, 0, 0.76042],
      "8911": [0, 0.54986, 0, 0, 0.76042],
      "8912": [0.03517, 0.54986, 0, 0, 0.77778],
      "8913": [0.03517, 0.54986, 0, 0, 0.77778],
      "8914": [0, 0.54986, 0, 0, 0.66667],
      "8915": [0, 0.54986, 0, 0, 0.66667],
      "8916": [0, 0.69224, 0, 0, 0.66667],
      "8918": [0.0391, 0.5391, 0, 0, 0.77778],
      "8919": [0.0391, 0.5391, 0, 0, 0.77778],
      "8920": [0.03517, 0.54986, 0, 0, 1.33334],
      "8921": [0.03517, 0.54986, 0, 0, 1.33334],
      "8922": [0.38569, 0.88569, 0, 0, 0.77778],
      "8923": [0.38569, 0.88569, 0, 0, 0.77778],
      "8926": [0.13667, 0.63667, 0, 0, 0.77778],
      "8927": [0.13667, 0.63667, 0, 0, 0.77778],
      "8928": [0.30274, 0.79383, 0, 0, 0.77778],
      "8929": [0.30274, 0.79383, 0, 0, 0.77778],
      "8934": [0.23222, 0.74111, 0, 0, 0.77778],
      "8935": [0.23222, 0.74111, 0, 0, 0.77778],
      "8936": [0.23222, 0.74111, 0, 0, 0.77778],
      "8937": [0.23222, 0.74111, 0, 0, 0.77778],
      "8938": [0.20576, 0.70576, 0, 0, 0.77778],
      "8939": [0.20576, 0.70576, 0, 0, 0.77778],
      "8940": [0.30274, 0.79383, 0, 0, 0.77778],
      "8941": [0.30274, 0.79383, 0, 0, 0.77778],
      "8994": [0.19444, 0.69224, 0, 0, 0.77778],
      "8995": [0.19444, 0.69224, 0, 0, 0.77778],
      "9416": [0.15559, 0.69224, 0, 0, 0.90222],
      "9484": [0, 0.69224, 0, 0, 0.5],
      "9488": [0, 0.69224, 0, 0, 0.5],
      "9492": [0, 0.37788, 0, 0, 0.5],
      "9496": [0, 0.37788, 0, 0, 0.5],
      "9585": [0.19444, 0.68889, 0, 0, 0.88889],
      "9586": [0.19444, 0.74111, 0, 0, 0.88889],
      "9632": [0, 0.675, 0, 0, 0.77778],
      "9633": [0, 0.675, 0, 0, 0.77778],
      "9650": [0, 0.54986, 0, 0, 0.72222],
      "9651": [0, 0.54986, 0, 0, 0.72222],
      "9654": [0.03517, 0.54986, 0, 0, 0.77778],
      "9660": [0, 0.54986, 0, 0, 0.72222],
      "9661": [0, 0.54986, 0, 0, 0.72222],
      "9664": [0.03517, 0.54986, 0, 0, 0.77778],
      "9674": [0.11111, 0.69224, 0, 0, 0.66667],
      "9733": [0.19444, 0.69224, 0, 0, 0.94445],
      "10003": [0, 0.69224, 0, 0, 0.83334],
      "10016": [0, 0.69224, 0, 0, 0.83334],
      "10731": [0.11111, 0.69224, 0, 0, 0.66667],
      "10846": [0.19444, 0.75583, 0, 0, 0.61111],
      "10877": [0.13667, 0.63667, 0, 0, 0.77778],
      "10878": [0.13667, 0.63667, 0, 0, 0.77778],
      "10885": [0.25583, 0.75583, 0, 0, 0.77778],
      "10886": [0.25583, 0.75583, 0, 0, 0.77778],
      "10887": [0.13597, 0.63597, 0, 0, 0.77778],
      "10888": [0.13597, 0.63597, 0, 0, 0.77778],
      "10889": [0.26167, 0.75726, 0, 0, 0.77778],
      "10890": [0.26167, 0.75726, 0, 0, 0.77778],
      "10891": [0.48256, 0.98256, 0, 0, 0.77778],
      "10892": [0.48256, 0.98256, 0, 0, 0.77778],
      "10901": [0.13667, 0.63667, 0, 0, 0.77778],
      "10902": [0.13667, 0.63667, 0, 0, 0.77778],
      "10933": [0.25142, 0.75726, 0, 0, 0.77778],
      "10934": [0.25142, 0.75726, 0, 0, 0.77778],
      "10935": [0.26167, 0.75726, 0, 0, 0.77778],
      "10936": [0.26167, 0.75726, 0, 0, 0.77778],
      "10937": [0.26167, 0.75726, 0, 0, 0.77778],
      "10938": [0.26167, 0.75726, 0, 0, 0.77778],
      "10949": [0.25583, 0.75583, 0, 0, 0.77778],
      "10950": [0.25583, 0.75583, 0, 0, 0.77778],
      "10955": [0.28481, 0.79383, 0, 0, 0.77778],
      "10956": [0.28481, 0.79383, 0, 0, 0.77778],
      "57350": [0.08167, 0.58167, 0, 0, 0.22222],
      "57351": [0.08167, 0.58167, 0, 0, 0.38889],
      "57352": [0.08167, 0.58167, 0, 0, 0.77778],
      "57353": [0, 0.43056, 0.04028, 0, 0.66667],
      "57356": [0.25142, 0.75726, 0, 0, 0.77778],
      "57357": [0.25142, 0.75726, 0, 0, 0.77778],
      "57358": [0.41951, 0.91951, 0, 0, 0.77778],
      "57359": [0.30274, 0.79383, 0, 0, 0.77778],
      "57360": [0.30274, 0.79383, 0, 0, 0.77778],
      "57361": [0.41951, 0.91951, 0, 0, 0.77778],
      "57366": [0.25142, 0.75726, 0, 0, 0.77778],
      "57367": [0.25142, 0.75726, 0, 0, 0.77778],
      "57368": [0.25142, 0.75726, 0, 0, 0.77778],
      "57369": [0.25142, 0.75726, 0, 0, 0.77778],
      "57370": [0.13597, 0.63597, 0, 0, 0.77778],
      "57371": [0.13597, 0.63597, 0, 0, 0.77778]
    },
    "Caligraphic-Regular": {
      "32": [0, 0, 0, 0, 0.25],
      "65": [0, 0.68333, 0, 0.19445, 0.79847],
      "66": [0, 0.68333, 0.03041, 0.13889, 0.65681],
      "67": [0, 0.68333, 0.05834, 0.13889, 0.52653],
      "68": [0, 0.68333, 0.02778, 0.08334, 0.77139],
      "69": [0, 0.68333, 0.08944, 0.11111, 0.52778],
      "70": [0, 0.68333, 0.09931, 0.11111, 0.71875],
      "71": [0.09722, 0.68333, 0.0593, 0.11111, 0.59487],
      "72": [0, 0.68333, 0.00965, 0.11111, 0.84452],
      "73": [0, 0.68333, 0.07382, 0, 0.54452],
      "74": [0.09722, 0.68333, 0.18472, 0.16667, 0.67778],
      "75": [0, 0.68333, 0.01445, 0.05556, 0.76195],
      "76": [0, 0.68333, 0, 0.13889, 0.68972],
      "77": [0, 0.68333, 0, 0.13889, 1.2009],
      "78": [0, 0.68333, 0.14736, 0.08334, 0.82049],
      "79": [0, 0.68333, 0.02778, 0.11111, 0.79611],
      "80": [0, 0.68333, 0.08222, 0.08334, 0.69556],
      "81": [0.09722, 0.68333, 0, 0.11111, 0.81667],
      "82": [0, 0.68333, 0, 0.08334, 0.8475],
      "83": [0, 0.68333, 0.075, 0.13889, 0.60556],
      "84": [0, 0.68333, 0.25417, 0, 0.54464],
      "85": [0, 0.68333, 0.09931, 0.08334, 0.62583],
      "86": [0, 0.68333, 0.08222, 0, 0.61278],
      "87": [0, 0.68333, 0.08222, 0.08334, 0.98778],
      "88": [0, 0.68333, 0.14643, 0.13889, 0.7133],
      "89": [0.09722, 0.68333, 0.08222, 0.08334, 0.66834],
      "90": [0, 0.68333, 0.07944, 0.13889, 0.72473],
      "160": [0, 0, 0, 0, 0.25]
    },
    "Fraktur-Regular": {
      "32": [0, 0, 0, 0, 0.25],
      "33": [0, 0.69141, 0, 0, 0.29574],
      "34": [0, 0.69141, 0, 0, 0.21471],
      "38": [0, 0.69141, 0, 0, 0.73786],
      "39": [0, 0.69141, 0, 0, 0.21201],
      "40": [0.24982, 0.74947, 0, 0, 0.38865],
      "41": [0.24982, 0.74947, 0, 0, 0.38865],
      "42": [0, 0.62119, 0, 0, 0.27764],
      "43": [0.08319, 0.58283, 0, 0, 0.75623],
      "44": [0, 0.10803, 0, 0, 0.27764],
      "45": [0.08319, 0.58283, 0, 0, 0.75623],
      "46": [0, 0.10803, 0, 0, 0.27764],
      "47": [0.24982, 0.74947, 0, 0, 0.50181],
      "48": [0, 0.47534, 0, 0, 0.50181],
      "49": [0, 0.47534, 0, 0, 0.50181],
      "50": [0, 0.47534, 0, 0, 0.50181],
      "51": [0.18906, 0.47534, 0, 0, 0.50181],
      "52": [0.18906, 0.47534, 0, 0, 0.50181],
      "53": [0.18906, 0.47534, 0, 0, 0.50181],
      "54": [0, 0.69141, 0, 0, 0.50181],
      "55": [0.18906, 0.47534, 0, 0, 0.50181],
      "56": [0, 0.69141, 0, 0, 0.50181],
      "57": [0.18906, 0.47534, 0, 0, 0.50181],
      "58": [0, 0.47534, 0, 0, 0.21606],
      "59": [0.12604, 0.47534, 0, 0, 0.21606],
      "61": [-0.13099, 0.36866, 0, 0, 0.75623],
      "63": [0, 0.69141, 0, 0, 0.36245],
      "65": [0, 0.69141, 0, 0, 0.7176],
      "66": [0, 0.69141, 0, 0, 0.88397],
      "67": [0, 0.69141, 0, 0, 0.61254],
      "68": [0, 0.69141, 0, 0, 0.83158],
      "69": [0, 0.69141, 0, 0, 0.66278],
      "70": [0.12604, 0.69141, 0, 0, 0.61119],
      "71": [0, 0.69141, 0, 0, 0.78539],
      "72": [0.06302, 0.69141, 0, 0, 0.7203],
      "73": [0, 0.69141, 0, 0, 0.55448],
      "74": [0.12604, 0.69141, 0, 0, 0.55231],
      "75": [0, 0.69141, 0, 0, 0.66845],
      "76": [0, 0.69141, 0, 0, 0.66602],
      "77": [0, 0.69141, 0, 0, 1.04953],
      "78": [0, 0.69141, 0, 0, 0.83212],
      "79": [0, 0.69141, 0, 0, 0.82699],
      "80": [0.18906, 0.69141, 0, 0, 0.82753],
      "81": [0.03781, 0.69141, 0, 0, 0.82699],
      "82": [0, 0.69141, 0, 0, 0.82807],
      "83": [0, 0.69141, 0, 0, 0.82861],
      "84": [0, 0.69141, 0, 0, 0.66899],
      "85": [0, 0.69141, 0, 0, 0.64576],
      "86": [0, 0.69141, 0, 0, 0.83131],
      "87": [0, 0.69141, 0, 0, 1.04602],
      "88": [0, 0.69141, 0, 0, 0.71922],
      "89": [0.18906, 0.69141, 0, 0, 0.83293],
      "90": [0.12604, 0.69141, 0, 0, 0.60201],
      "91": [0.24982, 0.74947, 0, 0, 0.27764],
      "93": [0.24982, 0.74947, 0, 0, 0.27764],
      "94": [0, 0.69141, 0, 0, 0.49965],
      "97": [0, 0.47534, 0, 0, 0.50046],
      "98": [0, 0.69141, 0, 0, 0.51315],
      "99": [0, 0.47534, 0, 0, 0.38946],
      "100": [0, 0.62119, 0, 0, 0.49857],
      "101": [0, 0.47534, 0, 0, 0.40053],
      "102": [0.18906, 0.69141, 0, 0, 0.32626],
      "103": [0.18906, 0.47534, 0, 0, 0.5037],
      "104": [0.18906, 0.69141, 0, 0, 0.52126],
      "105": [0, 0.69141, 0, 0, 0.27899],
      "106": [0, 0.69141, 0, 0, 0.28088],
      "107": [0, 0.69141, 0, 0, 0.38946],
      "108": [0, 0.69141, 0, 0, 0.27953],
      "109": [0, 0.47534, 0, 0, 0.76676],
      "110": [0, 0.47534, 0, 0, 0.52666],
      "111": [0, 0.47534, 0, 0, 0.48885],
      "112": [0.18906, 0.52396, 0, 0, 0.50046],
      "113": [0.18906, 0.47534, 0, 0, 0.48912],
      "114": [0, 0.47534, 0, 0, 0.38919],
      "115": [0, 0.47534, 0, 0, 0.44266],
      "116": [0, 0.62119, 0, 0, 0.33301],
      "117": [0, 0.47534, 0, 0, 0.5172],
      "118": [0, 0.52396, 0, 0, 0.5118],
      "119": [0, 0.52396, 0, 0, 0.77351],
      "120": [0.18906, 0.47534, 0, 0, 0.38865],
      "121": [0.18906, 0.47534, 0, 0, 0.49884],
      "122": [0.18906, 0.47534, 0, 0, 0.39054],
      "160": [0, 0, 0, 0, 0.25],
      "8216": [0, 0.69141, 0, 0, 0.21471],
      "8217": [0, 0.69141, 0, 0, 0.21471],
      "58112": [0, 0.62119, 0, 0, 0.49749],
      "58113": [0, 0.62119, 0, 0, 0.4983],
      "58114": [0.18906, 0.69141, 0, 0, 0.33328],
      "58115": [0.18906, 0.69141, 0, 0, 0.32923],
      "58116": [0.18906, 0.47534, 0, 0, 0.50343],
      "58117": [0, 0.69141, 0, 0, 0.33301],
      "58118": [0, 0.62119, 0, 0, 0.33409],
      "58119": [0, 0.47534, 0, 0, 0.50073]
    },
    "Main-Bold": {
      "32": [0, 0, 0, 0, 0.25],
      "33": [0, 0.69444, 0, 0, 0.35],
      "34": [0, 0.69444, 0, 0, 0.60278],
      "35": [0.19444, 0.69444, 0, 0, 0.95833],
      "36": [0.05556, 0.75, 0, 0, 0.575],
      "37": [0.05556, 0.75, 0, 0, 0.95833],
      "38": [0, 0.69444, 0, 0, 0.89444],
      "39": [0, 0.69444, 0, 0, 0.31944],
      "40": [0.25, 0.75, 0, 0, 0.44722],
      "41": [0.25, 0.75, 0, 0, 0.44722],
      "42": [0, 0.75, 0, 0, 0.575],
      "43": [0.13333, 0.63333, 0, 0, 0.89444],
      "44": [0.19444, 0.15556, 0, 0, 0.31944],
      "45": [0, 0.44444, 0, 0, 0.38333],
      "46": [0, 0.15556, 0, 0, 0.31944],
      "47": [0.25, 0.75, 0, 0, 0.575],
      "48": [0, 0.64444, 0, 0, 0.575],
      "49": [0, 0.64444, 0, 0, 0.575],
      "50": [0, 0.64444, 0, 0, 0.575],
      "51": [0, 0.64444, 0, 0, 0.575],
      "52": [0, 0.64444, 0, 0, 0.575],
      "53": [0, 0.64444, 0, 0, 0.575],
      "54": [0, 0.64444, 0, 0, 0.575],
      "55": [0, 0.64444, 0, 0, 0.575],
      "56": [0, 0.64444, 0, 0, 0.575],
      "57": [0, 0.64444, 0, 0, 0.575],
      "58": [0, 0.44444, 0, 0, 0.31944],
      "59": [0.19444, 0.44444, 0, 0, 0.31944],
      "60": [0.08556, 0.58556, 0, 0, 0.89444],
      "61": [-0.10889, 0.39111, 0, 0, 0.89444],
      "62": [0.08556, 0.58556, 0, 0, 0.89444],
      "63": [0, 0.69444, 0, 0, 0.54305],
      "64": [0, 0.69444, 0, 0, 0.89444],
      "65": [0, 0.68611, 0, 0, 0.86944],
      "66": [0, 0.68611, 0, 0, 0.81805],
      "67": [0, 0.68611, 0, 0, 0.83055],
      "68": [0, 0.68611, 0, 0, 0.88194],
      "69": [0, 0.68611, 0, 0, 0.75555],
      "70": [0, 0.68611, 0, 0, 0.72361],
      "71": [0, 0.68611, 0, 0, 0.90416],
      "72": [0, 0.68611, 0, 0, 0.9],
      "73": [0, 0.68611, 0, 0, 0.43611],
      "74": [0, 0.68611, 0, 0, 0.59444],
      "75": [0, 0.68611, 0, 0, 0.90138],
      "76": [0, 0.68611, 0, 0, 0.69166],
      "77": [0, 0.68611, 0, 0, 1.09166],
      "78": [0, 0.68611, 0, 0, 0.9],
      "79": [0, 0.68611, 0, 0, 0.86388],
      "80": [0, 0.68611, 0, 0, 0.78611],
      "81": [0.19444, 0.68611, 0, 0, 0.86388],
      "82": [0, 0.68611, 0, 0, 0.8625],
      "83": [0, 0.68611, 0, 0, 0.63889],
      "84": [0, 0.68611, 0, 0, 0.8],
      "85": [0, 0.68611, 0, 0, 0.88472],
      "86": [0, 0.68611, 0.01597, 0, 0.86944],
      "87": [0, 0.68611, 0.01597, 0, 1.18888],
      "88": [0, 0.68611, 0, 0, 0.86944],
      "89": [0, 0.68611, 0.02875, 0, 0.86944],
      "90": [0, 0.68611, 0, 0, 0.70277],
      "91": [0.25, 0.75, 0, 0, 0.31944],
      "92": [0.25, 0.75, 0, 0, 0.575],
      "93": [0.25, 0.75, 0, 0, 0.31944],
      "94": [0, 0.69444, 0, 0, 0.575],
      "95": [0.31, 0.13444, 0.03194, 0, 0.575],
      "97": [0, 0.44444, 0, 0, 0.55902],
      "98": [0, 0.69444, 0, 0, 0.63889],
      "99": [0, 0.44444, 0, 0, 0.51111],
      "100": [0, 0.69444, 0, 0, 0.63889],
      "101": [0, 0.44444, 0, 0, 0.52708],
      "102": [0, 0.69444, 0.10903, 0, 0.35139],
      "103": [0.19444, 0.44444, 0.01597, 0, 0.575],
      "104": [0, 0.69444, 0, 0, 0.63889],
      "105": [0, 0.69444, 0, 0, 0.31944],
      "106": [0.19444, 0.69444, 0, 0, 0.35139],
      "107": [0, 0.69444, 0, 0, 0.60694],
      "108": [0, 0.69444, 0, 0, 0.31944],
      "109": [0, 0.44444, 0, 0, 0.95833],
      "110": [0, 0.44444, 0, 0, 0.63889],
      "111": [0, 0.44444, 0, 0, 0.575],
      "112": [0.19444, 0.44444, 0, 0, 0.63889],
      "113": [0.19444, 0.44444, 0, 0, 0.60694],
      "114": [0, 0.44444, 0, 0, 0.47361],
      "115": [0, 0.44444, 0, 0, 0.45361],
      "116": [0, 0.63492, 0, 0, 0.44722],
      "117": [0, 0.44444, 0, 0, 0.63889],
      "118": [0, 0.44444, 0.01597, 0, 0.60694],
      "119": [0, 0.44444, 0.01597, 0, 0.83055],
      "120": [0, 0.44444, 0, 0, 0.60694],
      "121": [0.19444, 0.44444, 0.01597, 0, 0.60694],
      "122": [0, 0.44444, 0, 0, 0.51111],
      "123": [0.25, 0.75, 0, 0, 0.575],
      "124": [0.25, 0.75, 0, 0, 0.31944],
      "125": [0.25, 0.75, 0, 0, 0.575],
      "126": [0.35, 0.34444, 0, 0, 0.575],
      "160": [0, 0, 0, 0, 0.25],
      "163": [0, 0.69444, 0, 0, 0.86853],
      "168": [0, 0.69444, 0, 0, 0.575],
      "172": [0, 0.44444, 0, 0, 0.76666],
      "176": [0, 0.69444, 0, 0, 0.86944],
      "177": [0.13333, 0.63333, 0, 0, 0.89444],
      "184": [0.17014, 0, 0, 0, 0.51111],
      "198": [0, 0.68611, 0, 0, 1.04166],
      "215": [0.13333, 0.63333, 0, 0, 0.89444],
      "216": [0.04861, 0.73472, 0, 0, 0.89444],
      "223": [0, 0.69444, 0, 0, 0.59722],
      "230": [0, 0.44444, 0, 0, 0.83055],
      "247": [0.13333, 0.63333, 0, 0, 0.89444],
      "248": [0.09722, 0.54167, 0, 0, 0.575],
      "305": [0, 0.44444, 0, 0, 0.31944],
      "338": [0, 0.68611, 0, 0, 1.16944],
      "339": [0, 0.44444, 0, 0, 0.89444],
      "567": [0.19444, 0.44444, 0, 0, 0.35139],
      "710": [0, 0.69444, 0, 0, 0.575],
      "711": [0, 0.63194, 0, 0, 0.575],
      "713": [0, 0.59611, 0, 0, 0.575],
      "714": [0, 0.69444, 0, 0, 0.575],
      "715": [0, 0.69444, 0, 0, 0.575],
      "728": [0, 0.69444, 0, 0, 0.575],
      "729": [0, 0.69444, 0, 0, 0.31944],
      "730": [0, 0.69444, 0, 0, 0.86944],
      "732": [0, 0.69444, 0, 0, 0.575],
      "733": [0, 0.69444, 0, 0, 0.575],
      "915": [0, 0.68611, 0, 0, 0.69166],
      "916": [0, 0.68611, 0, 0, 0.95833],
      "920": [0, 0.68611, 0, 0, 0.89444],
      "923": [0, 0.68611, 0, 0, 0.80555],
      "926": [0, 0.68611, 0, 0, 0.76666],
      "928": [0, 0.68611, 0, 0, 0.9],
      "931": [0, 0.68611, 0, 0, 0.83055],
      "933": [0, 0.68611, 0, 0, 0.89444],
      "934": [0, 0.68611, 0, 0, 0.83055],
      "936": [0, 0.68611, 0, 0, 0.89444],
      "937": [0, 0.68611, 0, 0, 0.83055],
      "8211": [0, 0.44444, 0.03194, 0, 0.575],
      "8212": [0, 0.44444, 0.03194, 0, 1.14999],
      "8216": [0, 0.69444, 0, 0, 0.31944],
      "8217": [0, 0.69444, 0, 0, 0.31944],
      "8220": [0, 0.69444, 0, 0, 0.60278],
      "8221": [0, 0.69444, 0, 0, 0.60278],
      "8224": [0.19444, 0.69444, 0, 0, 0.51111],
      "8225": [0.19444, 0.69444, 0, 0, 0.51111],
      "8242": [0, 0.55556, 0, 0, 0.34444],
      "8407": [0, 0.72444, 0.15486, 0, 0.575],
      "8463": [0, 0.69444, 0, 0, 0.66759],
      "8465": [0, 0.69444, 0, 0, 0.83055],
      "8467": [0, 0.69444, 0, 0, 0.47361],
      "8472": [0.19444, 0.44444, 0, 0, 0.74027],
      "8476": [0, 0.69444, 0, 0, 0.83055],
      "8501": [0, 0.69444, 0, 0, 0.70277],
      "8592": [-0.10889, 0.39111, 0, 0, 1.14999],
      "8593": [0.19444, 0.69444, 0, 0, 0.575],
      "8594": [-0.10889, 0.39111, 0, 0, 1.14999],
      "8595": [0.19444, 0.69444, 0, 0, 0.575],
      "8596": [-0.10889, 0.39111, 0, 0, 1.14999],
      "8597": [0.25, 0.75, 0, 0, 0.575],
      "8598": [0.19444, 0.69444, 0, 0, 1.14999],
      "8599": [0.19444, 0.69444, 0, 0, 1.14999],
      "8600": [0.19444, 0.69444, 0, 0, 1.14999],
      "8601": [0.19444, 0.69444, 0, 0, 1.14999],
      "8636": [-0.10889, 0.39111, 0, 0, 1.14999],
      "8637": [-0.10889, 0.39111, 0, 0, 1.14999],
      "8640": [-0.10889, 0.39111, 0, 0, 1.14999],
      "8641": [-0.10889, 0.39111, 0, 0, 1.14999],
      "8656": [-0.10889, 0.39111, 0, 0, 1.14999],
      "8657": [0.19444, 0.69444, 0, 0, 0.70277],
      "8658": [-0.10889, 0.39111, 0, 0, 1.14999],
      "8659": [0.19444, 0.69444, 0, 0, 0.70277],
      "8660": [-0.10889, 0.39111, 0, 0, 1.14999],
      "8661": [0.25, 0.75, 0, 0, 0.70277],
      "8704": [0, 0.69444, 0, 0, 0.63889],
      "8706": [0, 0.69444, 0.06389, 0, 0.62847],
      "8707": [0, 0.69444, 0, 0, 0.63889],
      "8709": [0.05556, 0.75, 0, 0, 0.575],
      "8711": [0, 0.68611, 0, 0, 0.95833],
      "8712": [0.08556, 0.58556, 0, 0, 0.76666],
      "8715": [0.08556, 0.58556, 0, 0, 0.76666],
      "8722": [0.13333, 0.63333, 0, 0, 0.89444],
      "8723": [0.13333, 0.63333, 0, 0, 0.89444],
      "8725": [0.25, 0.75, 0, 0, 0.575],
      "8726": [0.25, 0.75, 0, 0, 0.575],
      "8727": [-0.02778, 0.47222, 0, 0, 0.575],
      "8728": [-0.02639, 0.47361, 0, 0, 0.575],
      "8729": [-0.02639, 0.47361, 0, 0, 0.575],
      "8730": [0.18, 0.82, 0, 0, 0.95833],
      "8733": [0, 0.44444, 0, 0, 0.89444],
      "8734": [0, 0.44444, 0, 0, 1.14999],
      "8736": [0, 0.69224, 0, 0, 0.72222],
      "8739": [0.25, 0.75, 0, 0, 0.31944],
      "8741": [0.25, 0.75, 0, 0, 0.575],
      "8743": [0, 0.55556, 0, 0, 0.76666],
      "8744": [0, 0.55556, 0, 0, 0.76666],
      "8745": [0, 0.55556, 0, 0, 0.76666],
      "8746": [0, 0.55556, 0, 0, 0.76666],
      "8747": [0.19444, 0.69444, 0.12778, 0, 0.56875],
      "8764": [-0.10889, 0.39111, 0, 0, 0.89444],
      "8768": [0.19444, 0.69444, 0, 0, 0.31944],
      "8771": [0.00222, 0.50222, 0, 0, 0.89444],
      "8773": [0.027, 0.638, 0, 0, 0.894],
      "8776": [0.02444, 0.52444, 0, 0, 0.89444],
      "8781": [0.00222, 0.50222, 0, 0, 0.89444],
      "8801": [0.00222, 0.50222, 0, 0, 0.89444],
      "8804": [0.19667, 0.69667, 0, 0, 0.89444],
      "8805": [0.19667, 0.69667, 0, 0, 0.89444],
      "8810": [0.08556, 0.58556, 0, 0, 1.14999],
      "8811": [0.08556, 0.58556, 0, 0, 1.14999],
      "8826": [0.08556, 0.58556, 0, 0, 0.89444],
      "8827": [0.08556, 0.58556, 0, 0, 0.89444],
      "8834": [0.08556, 0.58556, 0, 0, 0.89444],
      "8835": [0.08556, 0.58556, 0, 0, 0.89444],
      "8838": [0.19667, 0.69667, 0, 0, 0.89444],
      "8839": [0.19667, 0.69667, 0, 0, 0.89444],
      "8846": [0, 0.55556, 0, 0, 0.76666],
      "8849": [0.19667, 0.69667, 0, 0, 0.89444],
      "8850": [0.19667, 0.69667, 0, 0, 0.89444],
      "8851": [0, 0.55556, 0, 0, 0.76666],
      "8852": [0, 0.55556, 0, 0, 0.76666],
      "8853": [0.13333, 0.63333, 0, 0, 0.89444],
      "8854": [0.13333, 0.63333, 0, 0, 0.89444],
      "8855": [0.13333, 0.63333, 0, 0, 0.89444],
      "8856": [0.13333, 0.63333, 0, 0, 0.89444],
      "8857": [0.13333, 0.63333, 0, 0, 0.89444],
      "8866": [0, 0.69444, 0, 0, 0.70277],
      "8867": [0, 0.69444, 0, 0, 0.70277],
      "8868": [0, 0.69444, 0, 0, 0.89444],
      "8869": [0, 0.69444, 0, 0, 0.89444],
      "8900": [-0.02639, 0.47361, 0, 0, 0.575],
      "8901": [-0.02639, 0.47361, 0, 0, 0.31944],
      "8902": [-0.02778, 0.47222, 0, 0, 0.575],
      "8968": [0.25, 0.75, 0, 0, 0.51111],
      "8969": [0.25, 0.75, 0, 0, 0.51111],
      "8970": [0.25, 0.75, 0, 0, 0.51111],
      "8971": [0.25, 0.75, 0, 0, 0.51111],
      "8994": [-0.13889, 0.36111, 0, 0, 1.14999],
      "8995": [-0.13889, 0.36111, 0, 0, 1.14999],
      "9651": [0.19444, 0.69444, 0, 0, 1.02222],
      "9657": [-0.02778, 0.47222, 0, 0, 0.575],
      "9661": [0.19444, 0.69444, 0, 0, 1.02222],
      "9667": [-0.02778, 0.47222, 0, 0, 0.575],
      "9711": [0.19444, 0.69444, 0, 0, 1.14999],
      "9824": [0.12963, 0.69444, 0, 0, 0.89444],
      "9825": [0.12963, 0.69444, 0, 0, 0.89444],
      "9826": [0.12963, 0.69444, 0, 0, 0.89444],
      "9827": [0.12963, 0.69444, 0, 0, 0.89444],
      "9837": [0, 0.75, 0, 0, 0.44722],
      "9838": [0.19444, 0.69444, 0, 0, 0.44722],
      "9839": [0.19444, 0.69444, 0, 0, 0.44722],
      "10216": [0.25, 0.75, 0, 0, 0.44722],
      "10217": [0.25, 0.75, 0, 0, 0.44722],
      "10815": [0, 0.68611, 0, 0, 0.9],
      "10927": [0.19667, 0.69667, 0, 0, 0.89444],
      "10928": [0.19667, 0.69667, 0, 0, 0.89444],
      "57376": [0.19444, 0.69444, 0, 0, 0]
    },
    "Main-BoldItalic": {
      "32": [0, 0, 0, 0, 0.25],
      "33": [0, 0.69444, 0.11417, 0, 0.38611],
      "34": [0, 0.69444, 0.07939, 0, 0.62055],
      "35": [0.19444, 0.69444, 0.06833, 0, 0.94444],
      "37": [0.05556, 0.75, 0.12861, 0, 0.94444],
      "38": [0, 0.69444, 0.08528, 0, 0.88555],
      "39": [0, 0.69444, 0.12945, 0, 0.35555],
      "40": [0.25, 0.75, 0.15806, 0, 0.47333],
      "41": [0.25, 0.75, 0.03306, 0, 0.47333],
      "42": [0, 0.75, 0.14333, 0, 0.59111],
      "43": [0.10333, 0.60333, 0.03306, 0, 0.88555],
      "44": [0.19444, 0.14722, 0, 0, 0.35555],
      "45": [0, 0.44444, 0.02611, 0, 0.41444],
      "46": [0, 0.14722, 0, 0, 0.35555],
      "47": [0.25, 0.75, 0.15806, 0, 0.59111],
      "48": [0, 0.64444, 0.13167, 0, 0.59111],
      "49": [0, 0.64444, 0.13167, 0, 0.59111],
      "50": [0, 0.64444, 0.13167, 0, 0.59111],
      "51": [0, 0.64444, 0.13167, 0, 0.59111],
      "52": [0.19444, 0.64444, 0.13167, 0, 0.59111],
      "53": [0, 0.64444, 0.13167, 0, 0.59111],
      "54": [0, 0.64444, 0.13167, 0, 0.59111],
      "55": [0.19444, 0.64444, 0.13167, 0, 0.59111],
      "56": [0, 0.64444, 0.13167, 0, 0.59111],
      "57": [0, 0.64444, 0.13167, 0, 0.59111],
      "58": [0, 0.44444, 0.06695, 0, 0.35555],
      "59": [0.19444, 0.44444, 0.06695, 0, 0.35555],
      "61": [-0.10889, 0.39111, 0.06833, 0, 0.88555],
      "63": [0, 0.69444, 0.11472, 0, 0.59111],
      "64": [0, 0.69444, 0.09208, 0, 0.88555],
      "65": [0, 0.68611, 0, 0, 0.86555],
      "66": [0, 0.68611, 0.0992, 0, 0.81666],
      "67": [0, 0.68611, 0.14208, 0, 0.82666],
      "68": [0, 0.68611, 0.09062, 0, 0.87555],
      "69": [0, 0.68611, 0.11431, 0, 0.75666],
      "70": [0, 0.68611, 0.12903, 0, 0.72722],
      "71": [0, 0.68611, 0.07347, 0, 0.89527],
      "72": [0, 0.68611, 0.17208, 0, 0.8961],
      "73": [0, 0.68611, 0.15681, 0, 0.47166],
      "74": [0, 0.68611, 0.145, 0, 0.61055],
      "75": [0, 0.68611, 0.14208, 0, 0.89499],
      "76": [0, 0.68611, 0, 0, 0.69777],
      "77": [0, 0.68611, 0.17208, 0, 1.07277],
      "78": [0, 0.68611, 0.17208, 0, 0.8961],
      "79": [0, 0.68611, 0.09062, 0, 0.85499],
      "80": [0, 0.68611, 0.0992, 0, 0.78721],
      "81": [0.19444, 0.68611, 0.09062, 0, 0.85499],
      "82": [0, 0.68611, 0.02559, 0, 0.85944],
      "83": [0, 0.68611, 0.11264, 0, 0.64999],
      "84": [0, 0.68611, 0.12903, 0, 0.7961],
      "85": [0, 0.68611, 0.17208, 0, 0.88083],
      "86": [0, 0.68611, 0.18625, 0, 0.86555],
      "87": [0, 0.68611, 0.18625, 0, 1.15999],
      "88": [0, 0.68611, 0.15681, 0, 0.86555],
      "89": [0, 0.68611, 0.19803, 0, 0.86555],
      "90": [0, 0.68611, 0.14208, 0, 0.70888],
      "91": [0.25, 0.75, 0.1875, 0, 0.35611],
      "93": [0.25, 0.75, 0.09972, 0, 0.35611],
      "94": [0, 0.69444, 0.06709, 0, 0.59111],
      "95": [0.31, 0.13444, 0.09811, 0, 0.59111],
      "97": [0, 0.44444, 0.09426, 0, 0.59111],
      "98": [0, 0.69444, 0.07861, 0, 0.53222],
      "99": [0, 0.44444, 0.05222, 0, 0.53222],
      "100": [0, 0.69444, 0.10861, 0, 0.59111],
      "101": [0, 0.44444, 0.085, 0, 0.53222],
      "102": [0.19444, 0.69444, 0.21778, 0, 0.4],
      "103": [0.19444, 0.44444, 0.105, 0, 0.53222],
      "104": [0, 0.69444, 0.09426, 0, 0.59111],
      "105": [0, 0.69326, 0.11387, 0, 0.35555],
      "106": [0.19444, 0.69326, 0.1672, 0, 0.35555],
      "107": [0, 0.69444, 0.11111, 0, 0.53222],
      "108": [0, 0.69444, 0.10861, 0, 0.29666],
      "109": [0, 0.44444, 0.09426, 0, 0.94444],
      "110": [0, 0.44444, 0.09426, 0, 0.64999],
      "111": [0, 0.44444, 0.07861, 0, 0.59111],
      "112": [0.19444, 0.44444, 0.07861, 0, 0.59111],
      "113": [0.19444, 0.44444, 0.105, 0, 0.53222],
      "114": [0, 0.44444, 0.11111, 0, 0.50167],
      "115": [0, 0.44444, 0.08167, 0, 0.48694],
      "116": [0, 0.63492, 0.09639, 0, 0.385],
      "117": [0, 0.44444, 0.09426, 0, 0.62055],
      "118": [0, 0.44444, 0.11111, 0, 0.53222],
      "119": [0, 0.44444, 0.11111, 0, 0.76777],
      "120": [0, 0.44444, 0.12583, 0, 0.56055],
      "121": [0.19444, 0.44444, 0.105, 0, 0.56166],
      "122": [0, 0.44444, 0.13889, 0, 0.49055],
      "126": [0.35, 0.34444, 0.11472, 0, 0.59111],
      "160": [0, 0, 0, 0, 0.25],
      "168": [0, 0.69444, 0.11473, 0, 0.59111],
      "176": [0, 0.69444, 0, 0, 0.94888],
      "184": [0.17014, 0, 0, 0, 0.53222],
      "198": [0, 0.68611, 0.11431, 0, 1.02277],
      "216": [0.04861, 0.73472, 0.09062, 0, 0.88555],
      "223": [0.19444, 0.69444, 0.09736, 0, 0.665],
      "230": [0, 0.44444, 0.085, 0, 0.82666],
      "248": [0.09722, 0.54167, 0.09458, 0, 0.59111],
      "305": [0, 0.44444, 0.09426, 0, 0.35555],
      "338": [0, 0.68611, 0.11431, 0, 1.14054],
      "339": [0, 0.44444, 0.085, 0, 0.82666],
      "567": [0.19444, 0.44444, 0.04611, 0, 0.385],
      "710": [0, 0.69444, 0.06709, 0, 0.59111],
      "711": [0, 0.63194, 0.08271, 0, 0.59111],
      "713": [0, 0.59444, 0.10444, 0, 0.59111],
      "714": [0, 0.69444, 0.08528, 0, 0.59111],
      "715": [0, 0.69444, 0, 0, 0.59111],
      "728": [0, 0.69444, 0.10333, 0, 0.59111],
      "729": [0, 0.69444, 0.12945, 0, 0.35555],
      "730": [0, 0.69444, 0, 0, 0.94888],
      "732": [0, 0.69444, 0.11472, 0, 0.59111],
      "733": [0, 0.69444, 0.11472, 0, 0.59111],
      "915": [0, 0.68611, 0.12903, 0, 0.69777],
      "916": [0, 0.68611, 0, 0, 0.94444],
      "920": [0, 0.68611, 0.09062, 0, 0.88555],
      "923": [0, 0.68611, 0, 0, 0.80666],
      "926": [0, 0.68611, 0.15092, 0, 0.76777],
      "928": [0, 0.68611, 0.17208, 0, 0.8961],
      "931": [0, 0.68611, 0.11431, 0, 0.82666],
      "933": [0, 0.68611, 0.10778, 0, 0.88555],
      "934": [0, 0.68611, 0.05632, 0, 0.82666],
      "936": [0, 0.68611, 0.10778, 0, 0.88555],
      "937": [0, 0.68611, 0.0992, 0, 0.82666],
      "8211": [0, 0.44444, 0.09811, 0, 0.59111],
      "8212": [0, 0.44444, 0.09811, 0, 1.18221],
      "8216": [0, 0.69444, 0.12945, 0, 0.35555],
      "8217": [0, 0.69444, 0.12945, 0, 0.35555],
      "8220": [0, 0.69444, 0.16772, 0, 0.62055],
      "8221": [0, 0.69444, 0.07939, 0, 0.62055]
    },
    "Main-Italic": {
      "32": [0, 0, 0, 0, 0.25],
      "33": [0, 0.69444, 0.12417, 0, 0.30667],
      "34": [0, 0.69444, 0.06961, 0, 0.51444],
      "35": [0.19444, 0.69444, 0.06616, 0, 0.81777],
      "37": [0.05556, 0.75, 0.13639, 0, 0.81777],
      "38": [0, 0.69444, 0.09694, 0, 0.76666],
      "39": [0, 0.69444, 0.12417, 0, 0.30667],
      "40": [0.25, 0.75, 0.16194, 0, 0.40889],
      "41": [0.25, 0.75, 0.03694, 0, 0.40889],
      "42": [0, 0.75, 0.14917, 0, 0.51111],
      "43": [0.05667, 0.56167, 0.03694, 0, 0.76666],
      "44": [0.19444, 0.10556, 0, 0, 0.30667],
      "45": [0, 0.43056, 0.02826, 0, 0.35778],
      "46": [0, 0.10556, 0, 0, 0.30667],
      "47": [0.25, 0.75, 0.16194, 0, 0.51111],
      "48": [0, 0.64444, 0.13556, 0, 0.51111],
      "49": [0, 0.64444, 0.13556, 0, 0.51111],
      "50": [0, 0.64444, 0.13556, 0, 0.51111],
      "51": [0, 0.64444, 0.13556, 0, 0.51111],
      "52": [0.19444, 0.64444, 0.13556, 0, 0.51111],
      "53": [0, 0.64444, 0.13556, 0, 0.51111],
      "54": [0, 0.64444, 0.13556, 0, 0.51111],
      "55": [0.19444, 0.64444, 0.13556, 0, 0.51111],
      "56": [0, 0.64444, 0.13556, 0, 0.51111],
      "57": [0, 0.64444, 0.13556, 0, 0.51111],
      "58": [0, 0.43056, 0.0582, 0, 0.30667],
      "59": [0.19444, 0.43056, 0.0582, 0, 0.30667],
      "61": [-0.13313, 0.36687, 0.06616, 0, 0.76666],
      "63": [0, 0.69444, 0.1225, 0, 0.51111],
      "64": [0, 0.69444, 0.09597, 0, 0.76666],
      "65": [0, 0.68333, 0, 0, 0.74333],
      "66": [0, 0.68333, 0.10257, 0, 0.70389],
      "67": [0, 0.68333, 0.14528, 0, 0.71555],
      "68": [0, 0.68333, 0.09403, 0, 0.755],
      "69": [0, 0.68333, 0.12028, 0, 0.67833],
      "70": [0, 0.68333, 0.13305, 0, 0.65277],
      "71": [0, 0.68333, 0.08722, 0, 0.77361],
      "72": [0, 0.68333, 0.16389, 0, 0.74333],
      "73": [0, 0.68333, 0.15806, 0, 0.38555],
      "74": [0, 0.68333, 0.14028, 0, 0.525],
      "75": [0, 0.68333, 0.14528, 0, 0.76888],
      "76": [0, 0.68333, 0, 0, 0.62722],
      "77": [0, 0.68333, 0.16389, 0, 0.89666],
      "78": [0, 0.68333, 0.16389, 0, 0.74333],
      "79": [0, 0.68333, 0.09403, 0, 0.76666],
      "80": [0, 0.68333, 0.10257, 0, 0.67833],
      "81": [0.19444, 0.68333, 0.09403, 0, 0.76666],
      "82": [0, 0.68333, 0.03868, 0, 0.72944],
      "83": [0, 0.68333, 0.11972, 0, 0.56222],
      "84": [0, 0.68333, 0.13305, 0, 0.71555],
      "85": [0, 0.68333, 0.16389, 0, 0.74333],
      "86": [0, 0.68333, 0.18361, 0, 0.74333],
      "87": [0, 0.68333, 0.18361, 0, 0.99888],
      "88": [0, 0.68333, 0.15806, 0, 0.74333],
      "89": [0, 0.68333, 0.19383, 0, 0.74333],
      "90": [0, 0.68333, 0.14528, 0, 0.61333],
      "91": [0.25, 0.75, 0.1875, 0, 0.30667],
      "93": [0.25, 0.75, 0.10528, 0, 0.30667],
      "94": [0, 0.69444, 0.06646, 0, 0.51111],
      "95": [0.31, 0.12056, 0.09208, 0, 0.51111],
      "97": [0, 0.43056, 0.07671, 0, 0.51111],
      "98": [0, 0.69444, 0.06312, 0, 0.46],
      "99": [0, 0.43056, 0.05653, 0, 0.46],
      "100": [0, 0.69444, 0.10333, 0, 0.51111],
      "101": [0, 0.43056, 0.07514, 0, 0.46],
      "102": [0.19444, 0.69444, 0.21194, 0, 0.30667],
      "103": [0.19444, 0.43056, 0.08847, 0, 0.46],
      "104": [0, 0.69444, 0.07671, 0, 0.51111],
      "105": [0, 0.65536, 0.1019, 0, 0.30667],
      "106": [0.19444, 0.65536, 0.14467, 0, 0.30667],
      "107": [0, 0.69444, 0.10764, 0, 0.46],
      "108": [0, 0.69444, 0.10333, 0, 0.25555],
      "109": [0, 0.43056, 0.07671, 0, 0.81777],
      "110": [0, 0.43056, 0.07671, 0, 0.56222],
      "111": [0, 0.43056, 0.06312, 0, 0.51111],
      "112": [0.19444, 0.43056, 0.06312, 0, 0.51111],
      "113": [0.19444, 0.43056, 0.08847, 0, 0.46],
      "114": [0, 0.43056, 0.10764, 0, 0.42166],
      "115": [0, 0.43056, 0.08208, 0, 0.40889],
      "116": [0, 0.61508, 0.09486, 0, 0.33222],
      "117": [0, 0.43056, 0.07671, 0, 0.53666],
      "118": [0, 0.43056, 0.10764, 0, 0.46],
      "119": [0, 0.43056, 0.10764, 0, 0.66444],
      "120": [0, 0.43056, 0.12042, 0, 0.46389],
      "121": [0.19444, 0.43056, 0.08847, 0, 0.48555],
      "122": [0, 0.43056, 0.12292, 0, 0.40889],
      "126": [0.35, 0.31786, 0.11585, 0, 0.51111],
      "160": [0, 0, 0, 0, 0.25],
      "168": [0, 0.66786, 0.10474, 0, 0.51111],
      "176": [0, 0.69444, 0, 0, 0.83129],
      "184": [0.17014, 0, 0, 0, 0.46],
      "198": [0, 0.68333, 0.12028, 0, 0.88277],
      "216": [0.04861, 0.73194, 0.09403, 0, 0.76666],
      "223": [0.19444, 0.69444, 0.10514, 0, 0.53666],
      "230": [0, 0.43056, 0.07514, 0, 0.71555],
      "248": [0.09722, 0.52778, 0.09194, 0, 0.51111],
      "338": [0, 0.68333, 0.12028, 0, 0.98499],
      "339": [0, 0.43056, 0.07514, 0, 0.71555],
      "710": [0, 0.69444, 0.06646, 0, 0.51111],
      "711": [0, 0.62847, 0.08295, 0, 0.51111],
      "713": [0, 0.56167, 0.10333, 0, 0.51111],
      "714": [0, 0.69444, 0.09694, 0, 0.51111],
      "715": [0, 0.69444, 0, 0, 0.51111],
      "728": [0, 0.69444, 0.10806, 0, 0.51111],
      "729": [0, 0.66786, 0.11752, 0, 0.30667],
      "730": [0, 0.69444, 0, 0, 0.83129],
      "732": [0, 0.66786, 0.11585, 0, 0.51111],
      "733": [0, 0.69444, 0.1225, 0, 0.51111],
      "915": [0, 0.68333, 0.13305, 0, 0.62722],
      "916": [0, 0.68333, 0, 0, 0.81777],
      "920": [0, 0.68333, 0.09403, 0, 0.76666],
      "923": [0, 0.68333, 0, 0, 0.69222],
      "926": [0, 0.68333, 0.15294, 0, 0.66444],
      "928": [0, 0.68333, 0.16389, 0, 0.74333],
      "931": [0, 0.68333, 0.12028, 0, 0.71555],
      "933": [0, 0.68333, 0.11111, 0, 0.76666],
      "934": [0, 0.68333, 0.05986, 0, 0.71555],
      "936": [0, 0.68333, 0.11111, 0, 0.76666],
      "937": [0, 0.68333, 0.10257, 0, 0.71555],
      "8211": [0, 0.43056, 0.09208, 0, 0.51111],
      "8212": [0, 0.43056, 0.09208, 0, 1.02222],
      "8216": [0, 0.69444, 0.12417, 0, 0.30667],
      "8217": [0, 0.69444, 0.12417, 0, 0.30667],
      "8220": [0, 0.69444, 0.1685, 0, 0.51444],
      "8221": [0, 0.69444, 0.06961, 0, 0.51444],
      "8463": [0, 0.68889, 0, 0, 0.54028]
    },
    "Main-Regular": {
      "32": [0, 0, 0, 0, 0.25],
      "33": [0, 0.69444, 0, 0, 0.27778],
      "34": [0, 0.69444, 0, 0, 0.5],
      "35": [0.19444, 0.69444, 0, 0, 0.83334],
      "36": [0.05556, 0.75, 0, 0, 0.5],
      "37": [0.05556, 0.75, 0, 0, 0.83334],
      "38": [0, 0.69444, 0, 0, 0.77778],
      "39": [0, 0.69444, 0, 0, 0.27778],
      "40": [0.25, 0.75, 0, 0, 0.38889],
      "41": [0.25, 0.75, 0, 0, 0.38889],
      "42": [0, 0.75, 0, 0, 0.5],
      "43": [0.08333, 0.58333, 0, 0, 0.77778],
      "44": [0.19444, 0.10556, 0, 0, 0.27778],
      "45": [0, 0.43056, 0, 0, 0.33333],
      "46": [0, 0.10556, 0, 0, 0.27778],
      "47": [0.25, 0.75, 0, 0, 0.5],
      "48": [0, 0.64444, 0, 0, 0.5],
      "49": [0, 0.64444, 0, 0, 0.5],
      "50": [0, 0.64444, 0, 0, 0.5],
      "51": [0, 0.64444, 0, 0, 0.5],
      "52": [0, 0.64444, 0, 0, 0.5],
      "53": [0, 0.64444, 0, 0, 0.5],
      "54": [0, 0.64444, 0, 0, 0.5],
      "55": [0, 0.64444, 0, 0, 0.5],
      "56": [0, 0.64444, 0, 0, 0.5],
      "57": [0, 0.64444, 0, 0, 0.5],
      "58": [0, 0.43056, 0, 0, 0.27778],
      "59": [0.19444, 0.43056, 0, 0, 0.27778],
      "60": [0.0391, 0.5391, 0, 0, 0.77778],
      "61": [-0.13313, 0.36687, 0, 0, 0.77778],
      "62": [0.0391, 0.5391, 0, 0, 0.77778],
      "63": [0, 0.69444, 0, 0, 0.47222],
      "64": [0, 0.69444, 0, 0, 0.77778],
      "65": [0, 0.68333, 0, 0, 0.75],
      "66": [0, 0.68333, 0, 0, 0.70834],
      "67": [0, 0.68333, 0, 0, 0.72222],
      "68": [0, 0.68333, 0, 0, 0.76389],
      "69": [0, 0.68333, 0, 0, 0.68056],
      "70": [0, 0.68333, 0, 0, 0.65278],
      "71": [0, 0.68333, 0, 0, 0.78472],
      "72": [0, 0.68333, 0, 0, 0.75],
      "73": [0, 0.68333, 0, 0, 0.36111],
      "74": [0, 0.68333, 0, 0, 0.51389],
      "75": [0, 0.68333, 0, 0, 0.77778],
      "76": [0, 0.68333, 0, 0, 0.625],
      "77": [0, 0.68333, 0, 0, 0.91667],
      "78": [0, 0.68333, 0, 0, 0.75],
      "79": [0, 0.68333, 0, 0, 0.77778],
      "80": [0, 0.68333, 0, 0, 0.68056],
      "81": [0.19444, 0.68333, 0, 0, 0.77778],
      "82": [0, 0.68333, 0, 0, 0.73611],
      "83": [0, 0.68333, 0, 0, 0.55556],
      "84": [0, 0.68333, 0, 0, 0.72222],
      "85": [0, 0.68333, 0, 0, 0.75],
      "86": [0, 0.68333, 0.01389, 0, 0.75],
      "87": [0, 0.68333, 0.01389, 0, 1.02778],
      "88": [0, 0.68333, 0, 0, 0.75],
      "89": [0, 0.68333, 0.025, 0, 0.75],
      "90": [0, 0.68333, 0, 0, 0.61111],
      "91": [0.25, 0.75, 0, 0, 0.27778],
      "92": [0.25, 0.75, 0, 0, 0.5],
      "93": [0.25, 0.75, 0, 0, 0.27778],
      "94": [0, 0.69444, 0, 0, 0.5],
      "95": [0.31, 0.12056, 0.02778, 0, 0.5],
      "97": [0, 0.43056, 0, 0, 0.5],
      "98": [0, 0.69444, 0, 0, 0.55556],
      "99": [0, 0.43056, 0, 0, 0.44445],
      "100": [0, 0.69444, 0, 0, 0.55556],
      "101": [0, 0.43056, 0, 0, 0.44445],
      "102": [0, 0.69444, 0.07778, 0, 0.30556],
      "103": [0.19444, 0.43056, 0.01389, 0, 0.5],
      "104": [0, 0.69444, 0, 0, 0.55556],
      "105": [0, 0.66786, 0, 0, 0.27778],
      "106": [0.19444, 0.66786, 0, 0, 0.30556],
      "107": [0, 0.69444, 0, 0, 0.52778],
      "108": [0, 0.69444, 0, 0, 0.27778],
      "109": [0, 0.43056, 0, 0, 0.83334],
      "110": [0, 0.43056, 0, 0, 0.55556],
      "111": [0, 0.43056, 0, 0, 0.5],
      "112": [0.19444, 0.43056, 0, 0, 0.55556],
      "113": [0.19444, 0.43056, 0, 0, 0.52778],
      "114": [0, 0.43056, 0, 0, 0.39167],
      "115": [0, 0.43056, 0, 0, 0.39445],
      "116": [0, 0.61508, 0, 0, 0.38889],
      "117": [0, 0.43056, 0, 0, 0.55556],
      "118": [0, 0.43056, 0.01389, 0, 0.52778],
      "119": [0, 0.43056, 0.01389, 0, 0.72222],
      "120": [0, 0.43056, 0, 0, 0.52778],
      "121": [0.19444, 0.43056, 0.01389, 0, 0.52778],
      "122": [0, 0.43056, 0, 0, 0.44445],
      "123": [0.25, 0.75, 0, 0, 0.5],
      "124": [0.25, 0.75, 0, 0, 0.27778],
      "125": [0.25, 0.75, 0, 0, 0.5],
      "126": [0.35, 0.31786, 0, 0, 0.5],
      "160": [0, 0, 0, 0, 0.25],
      "163": [0, 0.69444, 0, 0, 0.76909],
      "167": [0.19444, 0.69444, 0, 0, 0.44445],
      "168": [0, 0.66786, 0, 0, 0.5],
      "172": [0, 0.43056, 0, 0, 0.66667],
      "176": [0, 0.69444, 0, 0, 0.75],
      "177": [0.08333, 0.58333, 0, 0, 0.77778],
      "182": [0.19444, 0.69444, 0, 0, 0.61111],
      "184": [0.17014, 0, 0, 0, 0.44445],
      "198": [0, 0.68333, 0, 0, 0.90278],
      "215": [0.08333, 0.58333, 0, 0, 0.77778],
      "216": [0.04861, 0.73194, 0, 0, 0.77778],
      "223": [0, 0.69444, 0, 0, 0.5],
      "230": [0, 0.43056, 0, 0, 0.72222],
      "247": [0.08333, 0.58333, 0, 0, 0.77778],
      "248": [0.09722, 0.52778, 0, 0, 0.5],
      "305": [0, 0.43056, 0, 0, 0.27778],
      "338": [0, 0.68333, 0, 0, 1.01389],
      "339": [0, 0.43056, 0, 0, 0.77778],
      "567": [0.19444, 0.43056, 0, 0, 0.30556],
      "710": [0, 0.69444, 0, 0, 0.5],
      "711": [0, 0.62847, 0, 0, 0.5],
      "713": [0, 0.56778, 0, 0, 0.5],
      "714": [0, 0.69444, 0, 0, 0.5],
      "715": [0, 0.69444, 0, 0, 0.5],
      "728": [0, 0.69444, 0, 0, 0.5],
      "729": [0, 0.66786, 0, 0, 0.27778],
      "730": [0, 0.69444, 0, 0, 0.75],
      "732": [0, 0.66786, 0, 0, 0.5],
      "733": [0, 0.69444, 0, 0, 0.5],
      "915": [0, 0.68333, 0, 0, 0.625],
      "916": [0, 0.68333, 0, 0, 0.83334],
      "920": [0, 0.68333, 0, 0, 0.77778],
      "923": [0, 0.68333, 0, 0, 0.69445],
      "926": [0, 0.68333, 0, 0, 0.66667],
      "928": [0, 0.68333, 0, 0, 0.75],
      "931": [0, 0.68333, 0, 0, 0.72222],
      "933": [0, 0.68333, 0, 0, 0.77778],
      "934": [0, 0.68333, 0, 0, 0.72222],
      "936": [0, 0.68333, 0, 0, 0.77778],
      "937": [0, 0.68333, 0, 0, 0.72222],
      "8211": [0, 0.43056, 0.02778, 0, 0.5],
      "8212": [0, 0.43056, 0.02778, 0, 1.0],
      "8216": [0, 0.69444, 0, 0, 0.27778],
      "8217": [0, 0.69444, 0, 0, 0.27778],
      "8220": [0, 0.69444, 0, 0, 0.5],
      "8221": [0, 0.69444, 0, 0, 0.5],
      "8224": [0.19444, 0.69444, 0, 0, 0.44445],
      "8225": [0.19444, 0.69444, 0, 0, 0.44445],
      "8230": [0, 0.123, 0, 0, 1.172],
      "8242": [0, 0.55556, 0, 0, 0.275],
      "8407": [0, 0.71444, 0.15382, 0, 0.5],
      "8463": [0, 0.68889, 0, 0, 0.54028],
      "8465": [0, 0.69444, 0, 0, 0.72222],
      "8467": [0, 0.69444, 0, 0.11111, 0.41667],
      "8472": [0.19444, 0.43056, 0, 0.11111, 0.63646],
      "8476": [0, 0.69444, 0, 0, 0.72222],
      "8501": [0, 0.69444, 0, 0, 0.61111],
      "8592": [-0.13313, 0.36687, 0, 0, 1.0],
      "8593": [0.19444, 0.69444, 0, 0, 0.5],
      "8594": [-0.13313, 0.36687, 0, 0, 1.0],
      "8595": [0.19444, 0.69444, 0, 0, 0.5],
      "8596": [-0.13313, 0.36687, 0, 0, 1.0],
      "8597": [0.25, 0.75, 0, 0, 0.5],
      "8598": [0.19444, 0.69444, 0, 0, 1.0],
      "8599": [0.19444, 0.69444, 0, 0, 1.0],
      "8600": [0.19444, 0.69444, 0, 0, 1.0],
      "8601": [0.19444, 0.69444, 0, 0, 1.0],
      "8614": [0.011, 0.511, 0, 0, 1.0],
      "8617": [0.011, 0.511, 0, 0, 1.126],
      "8618": [0.011, 0.511, 0, 0, 1.126],
      "8636": [-0.13313, 0.36687, 0, 0, 1.0],
      "8637": [-0.13313, 0.36687, 0, 0, 1.0],
      "8640": [-0.13313, 0.36687, 0, 0, 1.0],
      "8641": [-0.13313, 0.36687, 0, 0, 1.0],
      "8652": [0.011, 0.671, 0, 0, 1.0],
      "8656": [-0.13313, 0.36687, 0, 0, 1.0],
      "8657": [0.19444, 0.69444, 0, 0, 0.61111],
      "8658": [-0.13313, 0.36687, 0, 0, 1.0],
      "8659": [0.19444, 0.69444, 0, 0, 0.61111],
      "8660": [-0.13313, 0.36687, 0, 0, 1.0],
      "8661": [0.25, 0.75, 0, 0, 0.61111],
      "8704": [0, 0.69444, 0, 0, 0.55556],
      "8706": [0, 0.69444, 0.05556, 0.08334, 0.5309],
      "8707": [0, 0.69444, 0, 0, 0.55556],
      "8709": [0.05556, 0.75, 0, 0, 0.5],
      "8711": [0, 0.68333, 0, 0, 0.83334],
      "8712": [0.0391, 0.5391, 0, 0, 0.66667],
      "8715": [0.0391, 0.5391, 0, 0, 0.66667],
      "8722": [0.08333, 0.58333, 0, 0, 0.77778],
      "8723": [0.08333, 0.58333, 0, 0, 0.77778],
      "8725": [0.25, 0.75, 0, 0, 0.5],
      "8726": [0.25, 0.75, 0, 0, 0.5],
      "8727": [-0.03472, 0.46528, 0, 0, 0.5],
      "8728": [-0.05555, 0.44445, 0, 0, 0.5],
      "8729": [-0.05555, 0.44445, 0, 0, 0.5],
      "8730": [0.2, 0.8, 0, 0, 0.83334],
      "8733": [0, 0.43056, 0, 0, 0.77778],
      "8734": [0, 0.43056, 0, 0, 1.0],
      "8736": [0, 0.69224, 0, 0, 0.72222],
      "8739": [0.25, 0.75, 0, 0, 0.27778],
      "8741": [0.25, 0.75, 0, 0, 0.5],
      "8743": [0, 0.55556, 0, 0, 0.66667],
      "8744": [0, 0.55556, 0, 0, 0.66667],
      "8745": [0, 0.55556, 0, 0, 0.66667],
      "8746": [0, 0.55556, 0, 0, 0.66667],
      "8747": [0.19444, 0.69444, 0.11111, 0, 0.41667],
      "8764": [-0.13313, 0.36687, 0, 0, 0.77778],
      "8768": [0.19444, 0.69444, 0, 0, 0.27778],
      "8771": [-0.03625, 0.46375, 0, 0, 0.77778],
      "8773": [-0.022, 0.589, 0, 0, 0.778],
      "8776": [-0.01688, 0.48312, 0, 0, 0.77778],
      "8781": [-0.03625, 0.46375, 0, 0, 0.77778],
      "8784": [-0.133, 0.673, 0, 0, 0.778],
      "8801": [-0.03625, 0.46375, 0, 0, 0.77778],
      "8804": [0.13597, 0.63597, 0, 0, 0.77778],
      "8805": [0.13597, 0.63597, 0, 0, 0.77778],
      "8810": [0.0391, 0.5391, 0, 0, 1.0],
      "8811": [0.0391, 0.5391, 0, 0, 1.0],
      "8826": [0.0391, 0.5391, 0, 0, 0.77778],
      "8827": [0.0391, 0.5391, 0, 0, 0.77778],
      "8834": [0.0391, 0.5391, 0, 0, 0.77778],
      "8835": [0.0391, 0.5391, 0, 0, 0.77778],
      "8838": [0.13597, 0.63597, 0, 0, 0.77778],
      "8839": [0.13597, 0.63597, 0, 0, 0.77778],
      "8846": [0, 0.55556, 0, 0, 0.66667],
      "8849": [0.13597, 0.63597, 0, 0, 0.77778],
      "8850": [0.13597, 0.63597, 0, 0, 0.77778],
      "8851": [0, 0.55556, 0, 0, 0.66667],
      "8852": [0, 0.55556, 0, 0, 0.66667],
      "8853": [0.08333, 0.58333, 0, 0, 0.77778],
      "8854": [0.08333, 0.58333, 0, 0, 0.77778],
      "8855": [0.08333, 0.58333, 0, 0, 0.77778],
      "8856": [0.08333, 0.58333, 0, 0, 0.77778],
      "8857": [0.08333, 0.58333, 0, 0, 0.77778],
      "8866": [0, 0.69444, 0, 0, 0.61111],
      "8867": [0, 0.69444, 0, 0, 0.61111],
      "8868": [0, 0.69444, 0, 0, 0.77778],
      "8869": [0, 0.69444, 0, 0, 0.77778],
      "8872": [0.249, 0.75, 0, 0, 0.867],
      "8900": [-0.05555, 0.44445, 0, 0, 0.5],
      "8901": [-0.05555, 0.44445, 0, 0, 0.27778],
      "8902": [-0.03472, 0.46528, 0, 0, 0.5],
      "8904": [0.005, 0.505, 0, 0, 0.9],
      "8942": [0.03, 0.903, 0, 0, 0.278],
      "8943": [-0.19, 0.313, 0, 0, 1.172],
      "8945": [-0.1, 0.823, 0, 0, 1.282],
      "8968": [0.25, 0.75, 0, 0, 0.44445],
      "8969": [0.25, 0.75, 0, 0, 0.44445],
      "8970": [0.25, 0.75, 0, 0, 0.44445],
      "8971": [0.25, 0.75, 0, 0, 0.44445],
      "8994": [-0.14236, 0.35764, 0, 0, 1.0],
      "8995": [-0.14236, 0.35764, 0, 0, 1.0],
      "9136": [0.244, 0.744, 0, 0, 0.412],
      "9137": [0.244, 0.745, 0, 0, 0.412],
      "9651": [0.19444, 0.69444, 0, 0, 0.88889],
      "9657": [-0.03472, 0.46528, 0, 0, 0.5],
      "9661": [0.19444, 0.69444, 0, 0, 0.88889],
      "9667": [-0.03472, 0.46528, 0, 0, 0.5],
      "9711": [0.19444, 0.69444, 0, 0, 1.0],
      "9824": [0.12963, 0.69444, 0, 0, 0.77778],
      "9825": [0.12963, 0.69444, 0, 0, 0.77778],
      "9826": [0.12963, 0.69444, 0, 0, 0.77778],
      "9827": [0.12963, 0.69444, 0, 0, 0.77778],
      "9837": [0, 0.75, 0, 0, 0.38889],
      "9838": [0.19444, 0.69444, 0, 0, 0.38889],
      "9839": [0.19444, 0.69444, 0, 0, 0.38889],
      "10216": [0.25, 0.75, 0, 0, 0.38889],
      "10217": [0.25, 0.75, 0, 0, 0.38889],
      "10222": [0.244, 0.744, 0, 0, 0.412],
      "10223": [0.244, 0.745, 0, 0, 0.412],
      "10229": [0.011, 0.511, 0, 0, 1.609],
      "10230": [0.011, 0.511, 0, 0, 1.638],
      "10231": [0.011, 0.511, 0, 0, 1.859],
      "10232": [0.024, 0.525, 0, 0, 1.609],
      "10233": [0.024, 0.525, 0, 0, 1.638],
      "10234": [0.024, 0.525, 0, 0, 1.858],
      "10236": [0.011, 0.511, 0, 0, 1.638],
      "10815": [0, 0.68333, 0, 0, 0.75],
      "10927": [0.13597, 0.63597, 0, 0, 0.77778],
      "10928": [0.13597, 0.63597, 0, 0, 0.77778],
      "57376": [0.19444, 0.69444, 0, 0, 0]
    },
    "Math-BoldItalic": {
      "32": [0, 0, 0, 0, 0.25],
      "48": [0, 0.44444, 0, 0, 0.575],
      "49": [0, 0.44444, 0, 0, 0.575],
      "50": [0, 0.44444, 0, 0, 0.575],
      "51": [0.19444, 0.44444, 0, 0, 0.575],
      "52": [0.19444, 0.44444, 0, 0, 0.575],
      "53": [0.19444, 0.44444, 0, 0, 0.575],
      "54": [0, 0.64444, 0, 0, 0.575],
      "55": [0.19444, 0.44444, 0, 0, 0.575],
      "56": [0, 0.64444, 0, 0, 0.575],
      "57": [0.19444, 0.44444, 0, 0, 0.575],
      "65": [0, 0.68611, 0, 0, 0.86944],
      "66": [0, 0.68611, 0.04835, 0, 0.8664],
      "67": [0, 0.68611, 0.06979, 0, 0.81694],
      "68": [0, 0.68611, 0.03194, 0, 0.93812],
      "69": [0, 0.68611, 0.05451, 0, 0.81007],
      "70": [0, 0.68611, 0.15972, 0, 0.68889],
      "71": [0, 0.68611, 0, 0, 0.88673],
      "72": [0, 0.68611, 0.08229, 0, 0.98229],
      "73": [0, 0.68611, 0.07778, 0, 0.51111],
      "74": [0, 0.68611, 0.10069, 0, 0.63125],
      "75": [0, 0.68611, 0.06979, 0, 0.97118],
      "76": [0, 0.68611, 0, 0, 0.75555],
      "77": [0, 0.68611, 0.11424, 0, 1.14201],
      "78": [0, 0.68611, 0.11424, 0, 0.95034],
      "79": [0, 0.68611, 0.03194, 0, 0.83666],
      "80": [0, 0.68611, 0.15972, 0, 0.72309],
      "81": [0.19444, 0.68611, 0, 0, 0.86861],
      "82": [0, 0.68611, 0.00421, 0, 0.87235],
      "83": [0, 0.68611, 0.05382, 0, 0.69271],
      "84": [0, 0.68611, 0.15972, 0, 0.63663],
      "85": [0, 0.68611, 0.11424, 0, 0.80027],
      "86": [0, 0.68611, 0.25555, 0, 0.67778],
      "87": [0, 0.68611, 0.15972, 0, 1.09305],
      "88": [0, 0.68611, 0.07778, 0, 0.94722],
      "89": [0, 0.68611, 0.25555, 0, 0.67458],
      "90": [0, 0.68611, 0.06979, 0, 0.77257],
      "97": [0, 0.44444, 0, 0, 0.63287],
      "98": [0, 0.69444, 0, 0, 0.52083],
      "99": [0, 0.44444, 0, 0, 0.51342],
      "100": [0, 0.69444, 0, 0, 0.60972],
      "101": [0, 0.44444, 0, 0, 0.55361],
      "102": [0.19444, 0.69444, 0.11042, 0, 0.56806],
      "103": [0.19444, 0.44444, 0.03704, 0, 0.5449],
      "104": [0, 0.69444, 0, 0, 0.66759],
      "105": [0, 0.69326, 0, 0, 0.4048],
      "106": [0.19444, 0.69326, 0.0622, 0, 0.47083],
      "107": [0, 0.69444, 0.01852, 0, 0.6037],
      "108": [0, 0.69444, 0.0088, 0, 0.34815],
      "109": [0, 0.44444, 0, 0, 1.0324],
      "110": [0, 0.44444, 0, 0, 0.71296],
      "111": [0, 0.44444, 0, 0, 0.58472],
      "112": [0.19444, 0.44444, 0, 0, 0.60092],
      "113": [0.19444, 0.44444, 0.03704, 0, 0.54213],
      "114": [0, 0.44444, 0.03194, 0, 0.5287],
      "115": [0, 0.44444, 0, 0, 0.53125],
      "116": [0, 0.63492, 0, 0, 0.41528],
      "117": [0, 0.44444, 0, 0, 0.68102],
      "118": [0, 0.44444, 0.03704, 0, 0.56666],
      "119": [0, 0.44444, 0.02778, 0, 0.83148],
      "120": [0, 0.44444, 0, 0, 0.65903],
      "121": [0.19444, 0.44444, 0.03704, 0, 0.59028],
      "122": [0, 0.44444, 0.04213, 0, 0.55509],
      "160": [0, 0, 0, 0, 0.25],
      "915": [0, 0.68611, 0.15972, 0, 0.65694],
      "916": [0, 0.68611, 0, 0, 0.95833],
      "920": [0, 0.68611, 0.03194, 0, 0.86722],
      "923": [0, 0.68611, 0, 0, 0.80555],
      "926": [0, 0.68611, 0.07458, 0, 0.84125],
      "928": [0, 0.68611, 0.08229, 0, 0.98229],
      "931": [0, 0.68611, 0.05451, 0, 0.88507],
      "933": [0, 0.68611, 0.15972, 0, 0.67083],
      "934": [0, 0.68611, 0, 0, 0.76666],
      "936": [0, 0.68611, 0.11653, 0, 0.71402],
      "937": [0, 0.68611, 0.04835, 0, 0.8789],
      "945": [0, 0.44444, 0, 0, 0.76064],
      "946": [0.19444, 0.69444, 0.03403, 0, 0.65972],
      "947": [0.19444, 0.44444, 0.06389, 0, 0.59003],
      "948": [0, 0.69444, 0.03819, 0, 0.52222],
      "949": [0, 0.44444, 0, 0, 0.52882],
      "950": [0.19444, 0.69444, 0.06215, 0, 0.50833],
      "951": [0.19444, 0.44444, 0.03704, 0, 0.6],
      "952": [0, 0.69444, 0.03194, 0, 0.5618],
      "953": [0, 0.44444, 0, 0, 0.41204],
      "954": [0, 0.44444, 0, 0, 0.66759],
      "955": [0, 0.69444, 0, 0, 0.67083],
      "956": [0.19444, 0.44444, 0, 0, 0.70787],
      "957": [0, 0.44444, 0.06898, 0, 0.57685],
      "958": [0.19444, 0.69444, 0.03021, 0, 0.50833],
      "959": [0, 0.44444, 0, 0, 0.58472],
      "960": [0, 0.44444, 0.03704, 0, 0.68241],
      "961": [0.19444, 0.44444, 0, 0, 0.6118],
      "962": [0.09722, 0.44444, 0.07917, 0, 0.42361],
      "963": [0, 0.44444, 0.03704, 0, 0.68588],
      "964": [0, 0.44444, 0.13472, 0, 0.52083],
      "965": [0, 0.44444, 0.03704, 0, 0.63055],
      "966": [0.19444, 0.44444, 0, 0, 0.74722],
      "967": [0.19444, 0.44444, 0, 0, 0.71805],
      "968": [0.19444, 0.69444, 0.03704, 0, 0.75833],
      "969": [0, 0.44444, 0.03704, 0, 0.71782],
      "977": [0, 0.69444, 0, 0, 0.69155],
      "981": [0.19444, 0.69444, 0, 0, 0.7125],
      "982": [0, 0.44444, 0.03194, 0, 0.975],
      "1009": [0.19444, 0.44444, 0, 0, 0.6118],
      "1013": [0, 0.44444, 0, 0, 0.48333],
      "57649": [0, 0.44444, 0, 0, 0.39352],
      "57911": [0.19444, 0.44444, 0, 0, 0.43889]
    },
    "Math-Italic": {
      "32": [0, 0, 0, 0, 0.25],
      "48": [0, 0.43056, 0, 0, 0.5],
      "49": [0, 0.43056, 0, 0, 0.5],
      "50": [0, 0.43056, 0, 0, 0.5],
      "51": [0.19444, 0.43056, 0, 0, 0.5],
      "52": [0.19444, 0.43056, 0, 0, 0.5],
      "53": [0.19444, 0.43056, 0, 0, 0.5],
      "54": [0, 0.64444, 0, 0, 0.5],
      "55": [0.19444, 0.43056, 0, 0, 0.5],
      "56": [0, 0.64444, 0, 0, 0.5],
      "57": [0.19444, 0.43056, 0, 0, 0.5],
      "65": [0, 0.68333, 0, 0.13889, 0.75],
      "66": [0, 0.68333, 0.05017, 0.08334, 0.75851],
      "67": [0, 0.68333, 0.07153, 0.08334, 0.71472],
      "68": [0, 0.68333, 0.02778, 0.05556, 0.82792],
      "69": [0, 0.68333, 0.05764, 0.08334, 0.7382],
      "70": [0, 0.68333, 0.13889, 0.08334, 0.64306],
      "71": [0, 0.68333, 0, 0.08334, 0.78625],
      "72": [0, 0.68333, 0.08125, 0.05556, 0.83125],
      "73": [0, 0.68333, 0.07847, 0.11111, 0.43958],
      "74": [0, 0.68333, 0.09618, 0.16667, 0.55451],
      "75": [0, 0.68333, 0.07153, 0.05556, 0.84931],
      "76": [0, 0.68333, 0, 0.02778, 0.68056],
      "77": [0, 0.68333, 0.10903, 0.08334, 0.97014],
      "78": [0, 0.68333, 0.10903, 0.08334, 0.80347],
      "79": [0, 0.68333, 0.02778, 0.08334, 0.76278],
      "80": [0, 0.68333, 0.13889, 0.08334, 0.64201],
      "81": [0.19444, 0.68333, 0, 0.08334, 0.79056],
      "82": [0, 0.68333, 0.00773, 0.08334, 0.75929],
      "83": [0, 0.68333, 0.05764, 0.08334, 0.6132],
      "84": [0, 0.68333, 0.13889, 0.08334, 0.58438],
      "85": [0, 0.68333, 0.10903, 0.02778, 0.68278],
      "86": [0, 0.68333, 0.22222, 0, 0.58333],
      "87": [0, 0.68333, 0.13889, 0, 0.94445],
      "88": [0, 0.68333, 0.07847, 0.08334, 0.82847],
      "89": [0, 0.68333, 0.22222, 0, 0.58056],
      "90": [0, 0.68333, 0.07153, 0.08334, 0.68264],
      "97": [0, 0.43056, 0, 0, 0.52859],
      "98": [0, 0.69444, 0, 0, 0.42917],
      "99": [0, 0.43056, 0, 0.05556, 0.43276],
      "100": [0, 0.69444, 0, 0.16667, 0.52049],
      "101": [0, 0.43056, 0, 0.05556, 0.46563],
      "102": [0.19444, 0.69444, 0.10764, 0.16667, 0.48959],
      "103": [0.19444, 0.43056, 0.03588, 0.02778, 0.47697],
      "104": [0, 0.69444, 0, 0, 0.57616],
      "105": [0, 0.65952, 0, 0, 0.34451],
      "106": [0.19444, 0.65952, 0.05724, 0, 0.41181],
      "107": [0, 0.69444, 0.03148, 0, 0.5206],
      "108": [0, 0.69444, 0.01968, 0.08334, 0.29838],
      "109": [0, 0.43056, 0, 0, 0.87801],
      "110": [0, 0.43056, 0, 0, 0.60023],
      "111": [0, 0.43056, 0, 0.05556, 0.48472],
      "112": [0.19444, 0.43056, 0, 0.08334, 0.50313],
      "113": [0.19444, 0.43056, 0.03588, 0.08334, 0.44641],
      "114": [0, 0.43056, 0.02778, 0.05556, 0.45116],
      "115": [0, 0.43056, 0, 0.05556, 0.46875],
      "116": [0, 0.61508, 0, 0.08334, 0.36111],
      "117": [0, 0.43056, 0, 0.02778, 0.57246],
      "118": [0, 0.43056, 0.03588, 0.02778, 0.48472],
      "119": [0, 0.43056, 0.02691, 0.08334, 0.71592],
      "120": [0, 0.43056, 0, 0.02778, 0.57153],
      "121": [0.19444, 0.43056, 0.03588, 0.05556, 0.49028],
      "122": [0, 0.43056, 0.04398, 0.05556, 0.46505],
      "160": [0, 0, 0, 0, 0.25],
      "915": [0, 0.68333, 0.13889, 0.08334, 0.61528],
      "916": [0, 0.68333, 0, 0.16667, 0.83334],
      "920": [0, 0.68333, 0.02778, 0.08334, 0.76278],
      "923": [0, 0.68333, 0, 0.16667, 0.69445],
      "926": [0, 0.68333, 0.07569, 0.08334, 0.74236],
      "928": [0, 0.68333, 0.08125, 0.05556, 0.83125],
      "931": [0, 0.68333, 0.05764, 0.08334, 0.77986],
      "933": [0, 0.68333, 0.13889, 0.05556, 0.58333],
      "934": [0, 0.68333, 0, 0.08334, 0.66667],
      "936": [0, 0.68333, 0.11, 0.05556, 0.61222],
      "937": [0, 0.68333, 0.05017, 0.08334, 0.7724],
      "945": [0, 0.43056, 0.0037, 0.02778, 0.6397],
      "946": [0.19444, 0.69444, 0.05278, 0.08334, 0.56563],
      "947": [0.19444, 0.43056, 0.05556, 0, 0.51773],
      "948": [0, 0.69444, 0.03785, 0.05556, 0.44444],
      "949": [0, 0.43056, 0, 0.08334, 0.46632],
      "950": [0.19444, 0.69444, 0.07378, 0.08334, 0.4375],
      "951": [0.19444, 0.43056, 0.03588, 0.05556, 0.49653],
      "952": [0, 0.69444, 0.02778, 0.08334, 0.46944],
      "953": [0, 0.43056, 0, 0.05556, 0.35394],
      "954": [0, 0.43056, 0, 0, 0.57616],
      "955": [0, 0.69444, 0, 0, 0.58334],
      "956": [0.19444, 0.43056, 0, 0.02778, 0.60255],
      "957": [0, 0.43056, 0.06366, 0.02778, 0.49398],
      "958": [0.19444, 0.69444, 0.04601, 0.11111, 0.4375],
      "959": [0, 0.43056, 0, 0.05556, 0.48472],
      "960": [0, 0.43056, 0.03588, 0, 0.57003],
      "961": [0.19444, 0.43056, 0, 0.08334, 0.51702],
      "962": [0.09722, 0.43056, 0.07986, 0.08334, 0.36285],
      "963": [0, 0.43056, 0.03588, 0, 0.57141],
      "964": [0, 0.43056, 0.1132, 0.02778, 0.43715],
      "965": [0, 0.43056, 0.03588, 0.02778, 0.54028],
      "966": [0.19444, 0.43056, 0, 0.08334, 0.65417],
      "967": [0.19444, 0.43056, 0, 0.05556, 0.62569],
      "968": [0.19444, 0.69444, 0.03588, 0.11111, 0.65139],
      "969": [0, 0.43056, 0.03588, 0, 0.62245],
      "977": [0, 0.69444, 0, 0.08334, 0.59144],
      "981": [0.19444, 0.69444, 0, 0.08334, 0.59583],
      "982": [0, 0.43056, 0.02778, 0, 0.82813],
      "1009": [0.19444, 0.43056, 0, 0.08334, 0.51702],
      "1013": [0, 0.43056, 0, 0.05556, 0.4059],
      "57649": [0, 0.43056, 0, 0.02778, 0.32246],
      "57911": [0.19444, 0.43056, 0, 0.08334, 0.38403]
    },
    "SansSerif-Bold": {
      "32": [0, 0, 0, 0, 0.25],
      "33": [0, 0.69444, 0, 0, 0.36667],
      "34": [0, 0.69444, 0, 0, 0.55834],
      "35": [0.19444, 0.69444, 0, 0, 0.91667],
      "36": [0.05556, 0.75, 0, 0, 0.55],
      "37": [0.05556, 0.75, 0, 0, 1.02912],
      "38": [0, 0.69444, 0, 0, 0.83056],
      "39": [0, 0.69444, 0, 0, 0.30556],
      "40": [0.25, 0.75, 0, 0, 0.42778],
      "41": [0.25, 0.75, 0, 0, 0.42778],
      "42": [0, 0.75, 0, 0, 0.55],
      "43": [0.11667, 0.61667, 0, 0, 0.85556],
      "44": [0.10556, 0.13056, 0, 0, 0.30556],
      "45": [0, 0.45833, 0, 0, 0.36667],
      "46": [0, 0.13056, 0, 0, 0.30556],
      "47": [0.25, 0.75, 0, 0, 0.55],
      "48": [0, 0.69444, 0, 0, 0.55],
      "49": [0, 0.69444, 0, 0, 0.55],
      "50": [0, 0.69444, 0, 0, 0.55],
      "51": [0, 0.69444, 0, 0, 0.55],
      "52": [0, 0.69444, 0, 0, 0.55],
      "53": [0, 0.69444, 0, 0, 0.55],
      "54": [0, 0.69444, 0, 0, 0.55],
      "55": [0, 0.69444, 0, 0, 0.55],
      "56": [0, 0.69444, 0, 0, 0.55],
      "57": [0, 0.69444, 0, 0, 0.55],
      "58": [0, 0.45833, 0, 0, 0.30556],
      "59": [0.10556, 0.45833, 0, 0, 0.30556],
      "61": [-0.09375, 0.40625, 0, 0, 0.85556],
      "63": [0, 0.69444, 0, 0, 0.51945],
      "64": [0, 0.69444, 0, 0, 0.73334],
      "65": [0, 0.69444, 0, 0, 0.73334],
      "66": [0, 0.69444, 0, 0, 0.73334],
      "67": [0, 0.69444, 0, 0, 0.70278],
      "68": [0, 0.69444, 0, 0, 0.79445],
      "69": [0, 0.69444, 0, 0, 0.64167],
      "70": [0, 0.69444, 0, 0, 0.61111],
      "71": [0, 0.69444, 0, 0, 0.73334],
      "72": [0, 0.69444, 0, 0, 0.79445],
      "73": [0, 0.69444, 0, 0, 0.33056],
      "74": [0, 0.69444, 0, 0, 0.51945],
      "75": [0, 0.69444, 0, 0, 0.76389],
      "76": [0, 0.69444, 0, 0, 0.58056],
      "77": [0, 0.69444, 0, 0, 0.97778],
      "78": [0, 0.69444, 0, 0, 0.79445],
      "79": [0, 0.69444, 0, 0, 0.79445],
      "80": [0, 0.69444, 0, 0, 0.70278],
      "81": [0.10556, 0.69444, 0, 0, 0.79445],
      "82": [0, 0.69444, 0, 0, 0.70278],
      "83": [0, 0.69444, 0, 0, 0.61111],
      "84": [0, 0.69444, 0, 0, 0.73334],
      "85": [0, 0.69444, 0, 0, 0.76389],
      "86": [0, 0.69444, 0.01528, 0, 0.73334],
      "87": [0, 0.69444, 0.01528, 0, 1.03889],
      "88": [0, 0.69444, 0, 0, 0.73334],
      "89": [0, 0.69444, 0.0275, 0, 0.73334],
      "90": [0, 0.69444, 0, 0, 0.67223],
      "91": [0.25, 0.75, 0, 0, 0.34306],
      "93": [0.25, 0.75, 0, 0, 0.34306],
      "94": [0, 0.69444, 0, 0, 0.55],
      "95": [0.35, 0.10833, 0.03056, 0, 0.55],
      "97": [0, 0.45833, 0, 0, 0.525],
      "98": [0, 0.69444, 0, 0, 0.56111],
      "99": [0, 0.45833, 0, 0, 0.48889],
      "100": [0, 0.69444, 0, 0, 0.56111],
      "101": [0, 0.45833, 0, 0, 0.51111],
      "102": [0, 0.69444, 0.07639, 0, 0.33611],
      "103": [0.19444, 0.45833, 0.01528, 0, 0.55],
      "104": [0, 0.69444, 0, 0, 0.56111],
      "105": [0, 0.69444, 0, 0, 0.25556],
      "106": [0.19444, 0.69444, 0, 0, 0.28611],
      "107": [0, 0.69444, 0, 0, 0.53056],
      "108": [0, 0.69444, 0, 0, 0.25556],
      "109": [0, 0.45833, 0, 0, 0.86667],
      "110": [0, 0.45833, 0, 0, 0.56111],
      "111": [0, 0.45833, 0, 0, 0.55],
      "112": [0.19444, 0.45833, 0, 0, 0.56111],
      "113": [0.19444, 0.45833, 0, 0, 0.56111],
      "114": [0, 0.45833, 0.01528, 0, 0.37222],
      "115": [0, 0.45833, 0, 0, 0.42167],
      "116": [0, 0.58929, 0, 0, 0.40417],
      "117": [0, 0.45833, 0, 0, 0.56111],
      "118": [0, 0.45833, 0.01528, 0, 0.5],
      "119": [0, 0.45833, 0.01528, 0, 0.74445],
      "120": [0, 0.45833, 0, 0, 0.5],
      "121": [0.19444, 0.45833, 0.01528, 0, 0.5],
      "122": [0, 0.45833, 0, 0, 0.47639],
      "126": [0.35, 0.34444, 0, 0, 0.55],
      "160": [0, 0, 0, 0, 0.25],
      "168": [0, 0.69444, 0, 0, 0.55],
      "176": [0, 0.69444, 0, 0, 0.73334],
      "180": [0, 0.69444, 0, 0, 0.55],
      "184": [0.17014, 0, 0, 0, 0.48889],
      "305": [0, 0.45833, 0, 0, 0.25556],
      "567": [0.19444, 0.45833, 0, 0, 0.28611],
      "710": [0, 0.69444, 0, 0, 0.55],
      "711": [0, 0.63542, 0, 0, 0.55],
      "713": [0, 0.63778, 0, 0, 0.55],
      "728": [0, 0.69444, 0, 0, 0.55],
      "729": [0, 0.69444, 0, 0, 0.30556],
      "730": [0, 0.69444, 0, 0, 0.73334],
      "732": [0, 0.69444, 0, 0, 0.55],
      "733": [0, 0.69444, 0, 0, 0.55],
      "915": [0, 0.69444, 0, 0, 0.58056],
      "916": [0, 0.69444, 0, 0, 0.91667],
      "920": [0, 0.69444, 0, 0, 0.85556],
      "923": [0, 0.69444, 0, 0, 0.67223],
      "926": [0, 0.69444, 0, 0, 0.73334],
      "928": [0, 0.69444, 0, 0, 0.79445],
      "931": [0, 0.69444, 0, 0, 0.79445],
      "933": [0, 0.69444, 0, 0, 0.85556],
      "934": [0, 0.69444, 0, 0, 0.79445],
      "936": [0, 0.69444, 0, 0, 0.85556],
      "937": [0, 0.69444, 0, 0, 0.79445],
      "8211": [0, 0.45833, 0.03056, 0, 0.55],
      "8212": [0, 0.45833, 0.03056, 0, 1.10001],
      "8216": [0, 0.69444, 0, 0, 0.30556],
      "8217": [0, 0.69444, 0, 0, 0.30556],
      "8220": [0, 0.69444, 0, 0, 0.55834],
      "8221": [0, 0.69444, 0, 0, 0.55834]
    },
    "SansSerif-Italic": {
      "32": [0, 0, 0, 0, 0.25],
      "33": [0, 0.69444, 0.05733, 0, 0.31945],
      "34": [0, 0.69444, 0.00316, 0, 0.5],
      "35": [0.19444, 0.69444, 0.05087, 0, 0.83334],
      "36": [0.05556, 0.75, 0.11156, 0, 0.5],
      "37": [0.05556, 0.75, 0.03126, 0, 0.83334],
      "38": [0, 0.69444, 0.03058, 0, 0.75834],
      "39": [0, 0.69444, 0.07816, 0, 0.27778],
      "40": [0.25, 0.75, 0.13164, 0, 0.38889],
      "41": [0.25, 0.75, 0.02536, 0, 0.38889],
      "42": [0, 0.75, 0.11775, 0, 0.5],
      "43": [0.08333, 0.58333, 0.02536, 0, 0.77778],
      "44": [0.125, 0.08333, 0, 0, 0.27778],
      "45": [0, 0.44444, 0.01946, 0, 0.33333],
      "46": [0, 0.08333, 0, 0, 0.27778],
      "47": [0.25, 0.75, 0.13164, 0, 0.5],
      "48": [0, 0.65556, 0.11156, 0, 0.5],
      "49": [0, 0.65556, 0.11156, 0, 0.5],
      "50": [0, 0.65556, 0.11156, 0, 0.5],
      "51": [0, 0.65556, 0.11156, 0, 0.5],
      "52": [0, 0.65556, 0.11156, 0, 0.5],
      "53": [0, 0.65556, 0.11156, 0, 0.5],
      "54": [0, 0.65556, 0.11156, 0, 0.5],
      "55": [0, 0.65556, 0.11156, 0, 0.5],
      "56": [0, 0.65556, 0.11156, 0, 0.5],
      "57": [0, 0.65556, 0.11156, 0, 0.5],
      "58": [0, 0.44444, 0.02502, 0, 0.27778],
      "59": [0.125, 0.44444, 0.02502, 0, 0.27778],
      "61": [-0.13, 0.37, 0.05087, 0, 0.77778],
      "63": [0, 0.69444, 0.11809, 0, 0.47222],
      "64": [0, 0.69444, 0.07555, 0, 0.66667],
      "65": [0, 0.69444, 0, 0, 0.66667],
      "66": [0, 0.69444, 0.08293, 0, 0.66667],
      "67": [0, 0.69444, 0.11983, 0, 0.63889],
      "68": [0, 0.69444, 0.07555, 0, 0.72223],
      "69": [0, 0.69444, 0.11983, 0, 0.59722],
      "70": [0, 0.69444, 0.13372, 0, 0.56945],
      "71": [0, 0.69444, 0.11983, 0, 0.66667],
      "72": [0, 0.69444, 0.08094, 0, 0.70834],
      "73": [0, 0.69444, 0.13372, 0, 0.27778],
      "74": [0, 0.69444, 0.08094, 0, 0.47222],
      "75": [0, 0.69444, 0.11983, 0, 0.69445],
      "76": [0, 0.69444, 0, 0, 0.54167],
      "77": [0, 0.69444, 0.08094, 0, 0.875],
      "78": [0, 0.69444, 0.08094, 0, 0.70834],
      "79": [0, 0.69444, 0.07555, 0, 0.73611],
      "80": [0, 0.69444, 0.08293, 0, 0.63889],
      "81": [0.125, 0.69444, 0.07555, 0, 0.73611],
      "82": [0, 0.69444, 0.08293, 0, 0.64584],
      "83": [0, 0.69444, 0.09205, 0, 0.55556],
      "84": [0, 0.69444, 0.13372, 0, 0.68056],
      "85": [0, 0.69444, 0.08094, 0, 0.6875],
      "86": [0, 0.69444, 0.1615, 0, 0.66667],
      "87": [0, 0.69444, 0.1615, 0, 0.94445],
      "88": [0, 0.69444, 0.13372, 0, 0.66667],
      "89": [0, 0.69444, 0.17261, 0, 0.66667],
      "90": [0, 0.69444, 0.11983, 0, 0.61111],
      "91": [0.25, 0.75, 0.15942, 0, 0.28889],
      "93": [0.25, 0.75, 0.08719, 0, 0.28889],
      "94": [0, 0.69444, 0.0799, 0, 0.5],
      "95": [0.35, 0.09444, 0.08616, 0, 0.5],
      "97": [0, 0.44444, 0.00981, 0, 0.48056],
      "98": [0, 0.69444, 0.03057, 0, 0.51667],
      "99": [0, 0.44444, 0.08336, 0, 0.44445],
      "100": [0, 0.69444, 0.09483, 0, 0.51667],
      "101": [0, 0.44444, 0.06778, 0, 0.44445],
      "102": [0, 0.69444, 0.21705, 0, 0.30556],
      "103": [0.19444, 0.44444, 0.10836, 0, 0.5],
      "104": [0, 0.69444, 0.01778, 0, 0.51667],
      "105": [0, 0.67937, 0.09718, 0, 0.23889],
      "106": [0.19444, 0.67937, 0.09162, 0, 0.26667],
      "107": [0, 0.69444, 0.08336, 0, 0.48889],
      "108": [0, 0.69444, 0.09483, 0, 0.23889],
      "109": [0, 0.44444, 0.01778, 0, 0.79445],
      "110": [0, 0.44444, 0.01778, 0, 0.51667],
      "111": [0, 0.44444, 0.06613, 0, 0.5],
      "112": [0.19444, 0.44444, 0.0389, 0, 0.51667],
      "113": [0.19444, 0.44444, 0.04169, 0, 0.51667],
      "114": [0, 0.44444, 0.10836, 0, 0.34167],
      "115": [0, 0.44444, 0.0778, 0, 0.38333],
      "116": [0, 0.57143, 0.07225, 0, 0.36111],
      "117": [0, 0.44444, 0.04169, 0, 0.51667],
      "118": [0, 0.44444, 0.10836, 0, 0.46111],
      "119": [0, 0.44444, 0.10836, 0, 0.68334],
      "120": [0, 0.44444, 0.09169, 0, 0.46111],
      "121": [0.19444, 0.44444, 0.10836, 0, 0.46111],
      "122": [0, 0.44444, 0.08752, 0, 0.43472],
      "126": [0.35, 0.32659, 0.08826, 0, 0.5],
      "160": [0, 0, 0, 0, 0.25],
      "168": [0, 0.67937, 0.06385, 0, 0.5],
      "176": [0, 0.69444, 0, 0, 0.73752],
      "184": [0.17014, 0, 0, 0, 0.44445],
      "305": [0, 0.44444, 0.04169, 0, 0.23889],
      "567": [0.19444, 0.44444, 0.04169, 0, 0.26667],
      "710": [0, 0.69444, 0.0799, 0, 0.5],
      "711": [0, 0.63194, 0.08432, 0, 0.5],
      "713": [0, 0.60889, 0.08776, 0, 0.5],
      "714": [0, 0.69444, 0.09205, 0, 0.5],
      "715": [0, 0.69444, 0, 0, 0.5],
      "728": [0, 0.69444, 0.09483, 0, 0.5],
      "729": [0, 0.67937, 0.07774, 0, 0.27778],
      "730": [0, 0.69444, 0, 0, 0.73752],
      "732": [0, 0.67659, 0.08826, 0, 0.5],
      "733": [0, 0.69444, 0.09205, 0, 0.5],
      "915": [0, 0.69444, 0.13372, 0, 0.54167],
      "916": [0, 0.69444, 0, 0, 0.83334],
      "920": [0, 0.69444, 0.07555, 0, 0.77778],
      "923": [0, 0.69444, 0, 0, 0.61111],
      "926": [0, 0.69444, 0.12816, 0, 0.66667],
      "928": [0, 0.69444, 0.08094, 0, 0.70834],
      "931": [0, 0.69444, 0.11983, 0, 0.72222],
      "933": [0, 0.69444, 0.09031, 0, 0.77778],
      "934": [0, 0.69444, 0.04603, 0, 0.72222],
      "936": [0, 0.69444, 0.09031, 0, 0.77778],
      "937": [0, 0.69444, 0.08293, 0, 0.72222],
      "8211": [0, 0.44444, 0.08616, 0, 0.5],
      "8212": [0, 0.44444, 0.08616, 0, 1.0],
      "8216": [0, 0.69444, 0.07816, 0, 0.27778],
      "8217": [0, 0.69444, 0.07816, 0, 0.27778],
      "8220": [0, 0.69444, 0.14205, 0, 0.5],
      "8221": [0, 0.69444, 0.00316, 0, 0.5]
    },
    "SansSerif-Regular": {
      "32": [0, 0, 0, 0, 0.25],
      "33": [0, 0.69444, 0, 0, 0.31945],
      "34": [0, 0.69444, 0, 0, 0.5],
      "35": [0.19444, 0.69444, 0, 0, 0.83334],
      "36": [0.05556, 0.75, 0, 0, 0.5],
      "37": [0.05556, 0.75, 0, 0, 0.83334],
      "38": [0, 0.69444, 0, 0, 0.75834],
      "39": [0, 0.69444, 0, 0, 0.27778],
      "40": [0.25, 0.75, 0, 0, 0.38889],
      "41": [0.25, 0.75, 0, 0, 0.38889],
      "42": [0, 0.75, 0, 0, 0.5],
      "43": [0.08333, 0.58333, 0, 0, 0.77778],
      "44": [0.125, 0.08333, 0, 0, 0.27778],
      "45": [0, 0.44444, 0, 0, 0.33333],
      "46": [0, 0.08333, 0, 0, 0.27778],
      "47": [0.25, 0.75, 0, 0, 0.5],
      "48": [0, 0.65556, 0, 0, 0.5],
      "49": [0, 0.65556, 0, 0, 0.5],
      "50": [0, 0.65556, 0, 0, 0.5],
      "51": [0, 0.65556, 0, 0, 0.5],
      "52": [0, 0.65556, 0, 0, 0.5],
      "53": [0, 0.65556, 0, 0, 0.5],
      "54": [0, 0.65556, 0, 0, 0.5],
      "55": [0, 0.65556, 0, 0, 0.5],
      "56": [0, 0.65556, 0, 0, 0.5],
      "57": [0, 0.65556, 0, 0, 0.5],
      "58": [0, 0.44444, 0, 0, 0.27778],
      "59": [0.125, 0.44444, 0, 0, 0.27778],
      "61": [-0.13, 0.37, 0, 0, 0.77778],
      "63": [0, 0.69444, 0, 0, 0.47222],
      "64": [0, 0.69444, 0, 0, 0.66667],
      "65": [0, 0.69444, 0, 0, 0.66667],
      "66": [0, 0.69444, 0, 0, 0.66667],
      "67": [0, 0.69444, 0, 0, 0.63889],
      "68": [0, 0.69444, 0, 0, 0.72223],
      "69": [0, 0.69444, 0, 0, 0.59722],
      "70": [0, 0.69444, 0, 0, 0.56945],
      "71": [0, 0.69444, 0, 0, 0.66667],
      "72": [0, 0.69444, 0, 0, 0.70834],
      "73": [0, 0.69444, 0, 0, 0.27778],
      "74": [0, 0.69444, 0, 0, 0.47222],
      "75": [0, 0.69444, 0, 0, 0.69445],
      "76": [0, 0.69444, 0, 0, 0.54167],
      "77": [0, 0.69444, 0, 0, 0.875],
      "78": [0, 0.69444, 0, 0, 0.70834],
      "79": [0, 0.69444, 0, 0, 0.73611],
      "80": [0, 0.69444, 0, 0, 0.63889],
      "81": [0.125, 0.69444, 0, 0, 0.73611],
      "82": [0, 0.69444, 0, 0, 0.64584],
      "83": [0, 0.69444, 0, 0, 0.55556],
      "84": [0, 0.69444, 0, 0, 0.68056],
      "85": [0, 0.69444, 0, 0, 0.6875],
      "86": [0, 0.69444, 0.01389, 0, 0.66667],
      "87": [0, 0.69444, 0.01389, 0, 0.94445],
      "88": [0, 0.69444, 0, 0, 0.66667],
      "89": [0, 0.69444, 0.025, 0, 0.66667],
      "90": [0, 0.69444, 0, 0, 0.61111],
      "91": [0.25, 0.75, 0, 0, 0.28889],
      "93": [0.25, 0.75, 0, 0, 0.28889],
      "94": [0, 0.69444, 0, 0, 0.5],
      "95": [0.35, 0.09444, 0.02778, 0, 0.5],
      "97": [0, 0.44444, 0, 0, 0.48056],
      "98": [0, 0.69444, 0, 0, 0.51667],
      "99": [0, 0.44444, 0, 0, 0.44445],
      "100": [0, 0.69444, 0, 0, 0.51667],
      "101": [0, 0.44444, 0, 0, 0.44445],
      "102": [0, 0.69444, 0.06944, 0, 0.30556],
      "103": [0.19444, 0.44444, 0.01389, 0, 0.5],
      "104": [0, 0.69444, 0, 0, 0.51667],
      "105": [0, 0.67937, 0, 0, 0.23889],
      "106": [0.19444, 0.67937, 0, 0, 0.26667],
      "107": [0, 0.69444, 0, 0, 0.48889],
      "108": [0, 0.69444, 0, 0, 0.23889],
      "109": [0, 0.44444, 0, 0, 0.79445],
      "110": [0, 0.44444, 0, 0, 0.51667],
      "111": [0, 0.44444, 0, 0, 0.5],
      "112": [0.19444, 0.44444, 0, 0, 0.51667],
      "113": [0.19444, 0.44444, 0, 0, 0.51667],
      "114": [0, 0.44444, 0.01389, 0, 0.34167],
      "115": [0, 0.44444, 0, 0, 0.38333],
      "116": [0, 0.57143, 0, 0, 0.36111],
      "117": [0, 0.44444, 0, 0, 0.51667],
      "118": [0, 0.44444, 0.01389, 0, 0.46111],
      "119": [0, 0.44444, 0.01389, 0, 0.68334],
      "120": [0, 0.44444, 0, 0, 0.46111],
      "121": [0.19444, 0.44444, 0.01389, 0, 0.46111],
      "122": [0, 0.44444, 0, 0, 0.43472],
      "126": [0.35, 0.32659, 0, 0, 0.5],
      "160": [0, 0, 0, 0, 0.25],
      "168": [0, 0.67937, 0, 0, 0.5],
      "176": [0, 0.69444, 0, 0, 0.66667],
      "184": [0.17014, 0, 0, 0, 0.44445],
      "305": [0, 0.44444, 0, 0, 0.23889],
      "567": [0.19444, 0.44444, 0, 0, 0.26667],
      "710": [0, 0.69444, 0, 0, 0.5],
      "711": [0, 0.63194, 0, 0, 0.5],
      "713": [0, 0.60889, 0, 0, 0.5],
      "714": [0, 0.69444, 0, 0, 0.5],
      "715": [0, 0.69444, 0, 0, 0.5],
      "728": [0, 0.69444, 0, 0, 0.5],
      "729": [0, 0.67937, 0, 0, 0.27778],
      "730": [0, 0.69444, 0, 0, 0.66667],
      "732": [0, 0.67659, 0, 0, 0.5],
      "733": [0, 0.69444, 0, 0, 0.5],
      "915": [0, 0.69444, 0, 0, 0.54167],
      "916": [0, 0.69444, 0, 0, 0.83334],
      "920": [0, 0.69444, 0, 0, 0.77778],
      "923": [0, 0.69444, 0, 0, 0.61111],
      "926": [0, 0.69444, 0, 0, 0.66667],
      "928": [0, 0.69444, 0, 0, 0.70834],
      "931": [0, 0.69444, 0, 0, 0.72222],
      "933": [0, 0.69444, 0, 0, 0.77778],
      "934": [0, 0.69444, 0, 0, 0.72222],
      "936": [0, 0.69444, 0, 0, 0.77778],
      "937": [0, 0.69444, 0, 0, 0.72222],
      "8211": [0, 0.44444, 0.02778, 0, 0.5],
      "8212": [0, 0.44444, 0.02778, 0, 1.0],
      "8216": [0, 0.69444, 0, 0, 0.27778],
      "8217": [0, 0.69444, 0, 0, 0.27778],
      "8220": [0, 0.69444, 0, 0, 0.5],
      "8221": [0, 0.69444, 0, 0, 0.5]
    },
    "Script-Regular": {
      "32": [0, 0, 0, 0, 0.25],
      "65": [0, 0.7, 0.22925, 0, 0.80253],
      "66": [0, 0.7, 0.04087, 0, 0.90757],
      "67": [0, 0.7, 0.1689, 0, 0.66619],
      "68": [0, 0.7, 0.09371, 0, 0.77443],
      "69": [0, 0.7, 0.18583, 0, 0.56162],
      "70": [0, 0.7, 0.13634, 0, 0.89544],
      "71": [0, 0.7, 0.17322, 0, 0.60961],
      "72": [0, 0.7, 0.29694, 0, 0.96919],
      "73": [0, 0.7, 0.19189, 0, 0.80907],
      "74": [0.27778, 0.7, 0.19189, 0, 1.05159],
      "75": [0, 0.7, 0.31259, 0, 0.91364],
      "76": [0, 0.7, 0.19189, 0, 0.87373],
      "77": [0, 0.7, 0.15981, 0, 1.08031],
      "78": [0, 0.7, 0.3525, 0, 0.9015],
      "79": [0, 0.7, 0.08078, 0, 0.73787],
      "80": [0, 0.7, 0.08078, 0, 1.01262],
      "81": [0, 0.7, 0.03305, 0, 0.88282],
      "82": [0, 0.7, 0.06259, 0, 0.85],
      "83": [0, 0.7, 0.19189, 0, 0.86767],
      "84": [0, 0.7, 0.29087, 0, 0.74697],
      "85": [0, 0.7, 0.25815, 0, 0.79996],
      "86": [0, 0.7, 0.27523, 0, 0.62204],
      "87": [0, 0.7, 0.27523, 0, 0.80532],
      "88": [0, 0.7, 0.26006, 0, 0.94445],
      "89": [0, 0.7, 0.2939, 0, 0.70961],
      "90": [0, 0.7, 0.24037, 0, 0.8212],
      "160": [0, 0, 0, 0, 0.25]
    },
    "Size1-Regular": {
      "32": [0, 0, 0, 0, 0.25],
      "40": [0.35001, 0.85, 0, 0, 0.45834],
      "41": [0.35001, 0.85, 0, 0, 0.45834],
      "47": [0.35001, 0.85, 0, 0, 0.57778],
      "91": [0.35001, 0.85, 0, 0, 0.41667],
      "92": [0.35001, 0.85, 0, 0, 0.57778],
      "93": [0.35001, 0.85, 0, 0, 0.41667],
      "123": [0.35001, 0.85, 0, 0, 0.58334],
      "125": [0.35001, 0.85, 0, 0, 0.58334],
      "160": [0, 0, 0, 0, 0.25],
      "710": [0, 0.72222, 0, 0, 0.55556],
      "732": [0, 0.72222, 0, 0, 0.55556],
      "770": [0, 0.72222, 0, 0, 0.55556],
      "771": [0, 0.72222, 0, 0, 0.55556],
      "8214": [-0.00099, 0.601, 0, 0, 0.77778],
      "8593": [1e-05, 0.6, 0, 0, 0.66667],
      "8595": [1e-05, 0.6, 0, 0, 0.66667],
      "8657": [1e-05, 0.6, 0, 0, 0.77778],
      "8659": [1e-05, 0.6, 0, 0, 0.77778],
      "8719": [0.25001, 0.75, 0, 0, 0.94445],
      "8720": [0.25001, 0.75, 0, 0, 0.94445],
      "8721": [0.25001, 0.75, 0, 0, 1.05556],
      "8730": [0.35001, 0.85, 0, 0, 1.0],
      "8739": [-0.00599, 0.606, 0, 0, 0.33333],
      "8741": [-0.00599, 0.606, 0, 0, 0.55556],
      "8747": [0.30612, 0.805, 0.19445, 0, 0.47222],
      "8748": [0.306, 0.805, 0.19445, 0, 0.47222],
      "8749": [0.306, 0.805, 0.19445, 0, 0.47222],
      "8750": [0.30612, 0.805, 0.19445, 0, 0.47222],
      "8896": [0.25001, 0.75, 0, 0, 0.83334],
      "8897": [0.25001, 0.75, 0, 0, 0.83334],
      "8898": [0.25001, 0.75, 0, 0, 0.83334],
      "8899": [0.25001, 0.75, 0, 0, 0.83334],
      "8968": [0.35001, 0.85, 0, 0, 0.47222],
      "8969": [0.35001, 0.85, 0, 0, 0.47222],
      "8970": [0.35001, 0.85, 0, 0, 0.47222],
      "8971": [0.35001, 0.85, 0, 0, 0.47222],
      "9168": [-0.00099, 0.601, 0, 0, 0.66667],
      "10216": [0.35001, 0.85, 0, 0, 0.47222],
      "10217": [0.35001, 0.85, 0, 0, 0.47222],
      "10752": [0.25001, 0.75, 0, 0, 1.11111],
      "10753": [0.25001, 0.75, 0, 0, 1.11111],
      "10754": [0.25001, 0.75, 0, 0, 1.11111],
      "10756": [0.25001, 0.75, 0, 0, 0.83334],
      "10758": [0.25001, 0.75, 0, 0, 0.83334]
    },
    "Size2-Regular": {
      "32": [0, 0, 0, 0, 0.25],
      "40": [0.65002, 1.15, 0, 0, 0.59722],
      "41": [0.65002, 1.15, 0, 0, 0.59722],
      "47": [0.65002, 1.15, 0, 0, 0.81111],
      "91": [0.65002, 1.15, 0, 0, 0.47222],
      "92": [0.65002, 1.15, 0, 0, 0.81111],
      "93": [0.65002, 1.15, 0, 0, 0.47222],
      "123": [0.65002, 1.15, 0, 0, 0.66667],
      "125": [0.65002, 1.15, 0, 0, 0.66667],
      "160": [0, 0, 0, 0, 0.25],
      "710": [0, 0.75, 0, 0, 1.0],
      "732": [0, 0.75, 0, 0, 1.0],
      "770": [0, 0.75, 0, 0, 1.0],
      "771": [0, 0.75, 0, 0, 1.0],
      "8719": [0.55001, 1.05, 0, 0, 1.27778],
      "8720": [0.55001, 1.05, 0, 0, 1.27778],
      "8721": [0.55001, 1.05, 0, 0, 1.44445],
      "8730": [0.65002, 1.15, 0, 0, 1.0],
      "8747": [0.86225, 1.36, 0.44445, 0, 0.55556],
      "8748": [0.862, 1.36, 0.44445, 0, 0.55556],
      "8749": [0.862, 1.36, 0.44445, 0, 0.55556],
      "8750": [0.86225, 1.36, 0.44445, 0, 0.55556],
      "8896": [0.55001, 1.05, 0, 0, 1.11111],
      "8897": [0.55001, 1.05, 0, 0, 1.11111],
      "8898": [0.55001, 1.05, 0, 0, 1.11111],
      "8899": [0.55001, 1.05, 0, 0, 1.11111],
      "8968": [0.65002, 1.15, 0, 0, 0.52778],
      "8969": [0.65002, 1.15, 0, 0, 0.52778],
      "8970": [0.65002, 1.15, 0, 0, 0.52778],
      "8971": [0.65002, 1.15, 0, 0, 0.52778],
      "10216": [0.65002, 1.15, 0, 0, 0.61111],
      "10217": [0.65002, 1.15, 0, 0, 0.61111],
      "10752": [0.55001, 1.05, 0, 0, 1.51112],
      "10753": [0.55001, 1.05, 0, 0, 1.51112],
      "10754": [0.55001, 1.05, 0, 0, 1.51112],
      "10756": [0.55001, 1.05, 0, 0, 1.11111],
      "10758": [0.55001, 1.05, 0, 0, 1.11111]
    },
    "Size3-Regular": {
      "32": [0, 0, 0, 0, 0.25],
      "40": [0.95003, 1.45, 0, 0, 0.73611],
      "41": [0.95003, 1.45, 0, 0, 0.73611],
      "47": [0.95003, 1.45, 0, 0, 1.04445],
      "91": [0.95003, 1.45, 0, 0, 0.52778],
      "92": [0.95003, 1.45, 0, 0, 1.04445],
      "93": [0.95003, 1.45, 0, 0, 0.52778],
      "123": [0.95003, 1.45, 0, 0, 0.75],
      "125": [0.95003, 1.45, 0, 0, 0.75],
      "160": [0, 0, 0, 0, 0.25],
      "710": [0, 0.75, 0, 0, 1.44445],
      "732": [0, 0.75, 0, 0, 1.44445],
      "770": [0, 0.75, 0, 0, 1.44445],
      "771": [0, 0.75, 0, 0, 1.44445],
      "8730": [0.95003, 1.45, 0, 0, 1.0],
      "8968": [0.95003, 1.45, 0, 0, 0.58334],
      "8969": [0.95003, 1.45, 0, 0, 0.58334],
      "8970": [0.95003, 1.45, 0, 0, 0.58334],
      "8971": [0.95003, 1.45, 0, 0, 0.58334],
      "10216": [0.95003, 1.45, 0, 0, 0.75],
      "10217": [0.95003, 1.45, 0, 0, 0.75]
    },
    "Size4-Regular": {
      "32": [0, 0, 0, 0, 0.25],
      "40": [1.25003, 1.75, 0, 0, 0.79167],
      "41": [1.25003, 1.75, 0, 0, 0.79167],
      "47": [1.25003, 1.75, 0, 0, 1.27778],
      "91": [1.25003, 1.75, 0, 0, 0.58334],
      "92": [1.25003, 1.75, 0, 0, 1.27778],
      "93": [1.25003, 1.75, 0, 0, 0.58334],
      "123": [1.25003, 1.75, 0, 0, 0.80556],
      "125": [1.25003, 1.75, 0, 0, 0.80556],
      "160": [0, 0, 0, 0, 0.25],
      "710": [0, 0.825, 0, 0, 1.8889],
      "732": [0, 0.825, 0, 0, 1.8889],
      "770": [0, 0.825, 0, 0, 1.8889],
      "771": [0, 0.825, 0, 0, 1.8889],
      "8730": [1.25003, 1.75, 0, 0, 1.0],
      "8968": [1.25003, 1.75, 0, 0, 0.63889],
      "8969": [1.25003, 1.75, 0, 0, 0.63889],
      "8970": [1.25003, 1.75, 0, 0, 0.63889],
      "8971": [1.25003, 1.75, 0, 0, 0.63889],
      "9115": [0.64502, 1.155, 0, 0, 0.875],
      "9116": [1e-05, 0.6, 0, 0, 0.875],
      "9117": [0.64502, 1.155, 0, 0, 0.875],
      "9118": [0.64502, 1.155, 0, 0, 0.875],
      "9119": [1e-05, 0.6, 0, 0, 0.875],
      "9120": [0.64502, 1.155, 0, 0, 0.875],
      "9121": [0.64502, 1.155, 0, 0, 0.66667],
      "9122": [-0.00099, 0.601, 0, 0, 0.66667],
      "9123": [0.64502, 1.155, 0, 0, 0.66667],
      "9124": [0.64502, 1.155, 0, 0, 0.66667],
      "9125": [-0.00099, 0.601, 0, 0, 0.66667],
      "9126": [0.64502, 1.155, 0, 0, 0.66667],
      "9127": [1e-05, 0.9, 0, 0, 0.88889],
      "9128": [0.65002, 1.15, 0, 0, 0.88889],
      "9129": [0.90001, 0, 0, 0, 0.88889],
      "9130": [0, 0.3, 0, 0, 0.88889],
      "9131": [1e-05, 0.9, 0, 0, 0.88889],
      "9132": [0.65002, 1.15, 0, 0, 0.88889],
      "9133": [0.90001, 0, 0, 0, 0.88889],
      "9143": [0.88502, 0.915, 0, 0, 1.05556],
      "10216": [1.25003, 1.75, 0, 0, 0.80556],
      "10217": [1.25003, 1.75, 0, 0, 0.80556],
      "57344": [-0.00499, 0.605, 0, 0, 1.05556],
      "57345": [-0.00499, 0.605, 0, 0, 1.05556],
      "57680": [0, 0.12, 0, 0, 0.45],
      "57681": [0, 0.12, 0, 0, 0.45],
      "57682": [0, 0.12, 0, 0, 0.45],
      "57683": [0, 0.12, 0, 0, 0.45]
    },
    "Typewriter-Regular": {
      "32": [0, 0, 0, 0, 0.525],
      "33": [0, 0.61111, 0, 0, 0.525],
      "34": [0, 0.61111, 0, 0, 0.525],
      "35": [0, 0.61111, 0, 0, 0.525],
      "36": [0.08333, 0.69444, 0, 0, 0.525],
      "37": [0.08333, 0.69444, 0, 0, 0.525],
      "38": [0, 0.61111, 0, 0, 0.525],
      "39": [0, 0.61111, 0, 0, 0.525],
      "40": [0.08333, 0.69444, 0, 0, 0.525],
      "41": [0.08333, 0.69444, 0, 0, 0.525],
      "42": [0, 0.52083, 0, 0, 0.525],
      "43": [-0.08056, 0.53055, 0, 0, 0.525],
      "44": [0.13889, 0.125, 0, 0, 0.525],
      "45": [-0.08056, 0.53055, 0, 0, 0.525],
      "46": [0, 0.125, 0, 0, 0.525],
      "47": [0.08333, 0.69444, 0, 0, 0.525],
      "48": [0, 0.61111, 0, 0, 0.525],
      "49": [0, 0.61111, 0, 0, 0.525],
      "50": [0, 0.61111, 0, 0, 0.525],
      "51": [0, 0.61111, 0, 0, 0.525],
      "52": [0, 0.61111, 0, 0, 0.525],
      "53": [0, 0.61111, 0, 0, 0.525],
      "54": [0, 0.61111, 0, 0, 0.525],
      "55": [0, 0.61111, 0, 0, 0.525],
      "56": [0, 0.61111, 0, 0, 0.525],
      "57": [0, 0.61111, 0, 0, 0.525],
      "58": [0, 0.43056, 0, 0, 0.525],
      "59": [0.13889, 0.43056, 0, 0, 0.525],
      "60": [-0.05556, 0.55556, 0, 0, 0.525],
      "61": [-0.19549, 0.41562, 0, 0, 0.525],
      "62": [-0.05556, 0.55556, 0, 0, 0.525],
      "63": [0, 0.61111, 0, 0, 0.525],
      "64": [0, 0.61111, 0, 0, 0.525],
      "65": [0, 0.61111, 0, 0, 0.525],
      "66": [0, 0.61111, 0, 0, 0.525],
      "67": [0, 0.61111, 0, 0, 0.525],
      "68": [0, 0.61111, 0, 0, 0.525],
      "69": [0, 0.61111, 0, 0, 0.525],
      "70": [0, 0.61111, 0, 0, 0.525],
      "71": [0, 0.61111, 0, 0, 0.525],
      "72": [0, 0.61111, 0, 0, 0.525],
      "73": [0, 0.61111, 0, 0, 0.525],
      "74": [0, 0.61111, 0, 0, 0.525],
      "75": [0, 0.61111, 0, 0, 0.525],
      "76": [0, 0.61111, 0, 0, 0.525],
      "77": [0, 0.61111, 0, 0, 0.525],
      "78": [0, 0.61111, 0, 0, 0.525],
      "79": [0, 0.61111, 0, 0, 0.525],
      "80": [0, 0.61111, 0, 0, 0.525],
      "81": [0.13889, 0.61111, 0, 0, 0.525],
      "82": [0, 0.61111, 0, 0, 0.525],
      "83": [0, 0.61111, 0, 0, 0.525],
      "84": [0, 0.61111, 0, 0, 0.525],
      "85": [0, 0.61111, 0, 0, 0.525],
      "86": [0, 0.61111, 0, 0, 0.525],
      "87": [0, 0.61111, 0, 0, 0.525],
      "88": [0, 0.61111, 0, 0, 0.525],
      "89": [0, 0.61111, 0, 0, 0.525],
      "90": [0, 0.61111, 0, 0, 0.525],
      "91": [0.08333, 0.69444, 0, 0, 0.525],
      "92": [0.08333, 0.69444, 0, 0, 0.525],
      "93": [0.08333, 0.69444, 0, 0, 0.525],
      "94": [0, 0.61111, 0, 0, 0.525],
      "95": [0.09514, 0, 0, 0, 0.525],
      "96": [0, 0.61111, 0, 0, 0.525],
      "97": [0, 0.43056, 0, 0, 0.525],
      "98": [0, 0.61111, 0, 0, 0.525],
      "99": [0, 0.43056, 0, 0, 0.525],
      "100": [0, 0.61111, 0, 0, 0.525],
      "101": [0, 0.43056, 0, 0, 0.525],
      "102": [0, 0.61111, 0, 0, 0.525],
      "103": [0.22222, 0.43056, 0, 0, 0.525],
      "104": [0, 0.61111, 0, 0, 0.525],
      "105": [0, 0.61111, 0, 0, 0.525],
      "106": [0.22222, 0.61111, 0, 0, 0.525],
      "107": [0, 0.61111, 0, 0, 0.525],
      "108": [0, 0.61111, 0, 0, 0.525],
      "109": [0, 0.43056, 0, 0, 0.525],
      "110": [0, 0.43056, 0, 0, 0.525],
      "111": [0, 0.43056, 0, 0, 0.525],
      "112": [0.22222, 0.43056, 0, 0, 0.525],
      "113": [0.22222, 0.43056, 0, 0, 0.525],
      "114": [0, 0.43056, 0, 0, 0.525],
      "115": [0, 0.43056, 0, 0, 0.525],
      "116": [0, 0.55358, 0, 0, 0.525],
      "117": [0, 0.43056, 0, 0, 0.525],
      "118": [0, 0.43056, 0, 0, 0.525],
      "119": [0, 0.43056, 0, 0, 0.525],
      "120": [0, 0.43056, 0, 0, 0.525],
      "121": [0.22222, 0.43056, 0, 0, 0.525],
      "122": [0, 0.43056, 0, 0, 0.525],
      "123": [0.08333, 0.69444, 0, 0, 0.525],
      "124": [0.08333, 0.69444, 0, 0, 0.525],
      "125": [0.08333, 0.69444, 0, 0, 0.525],
      "126": [0, 0.61111, 0, 0, 0.525],
      "127": [0, 0.61111, 0, 0, 0.525],
      "160": [0, 0, 0, 0, 0.525],
      "176": [0, 0.61111, 0, 0, 0.525],
      "184": [0.19445, 0, 0, 0, 0.525],
      "305": [0, 0.43056, 0, 0, 0.525],
      "567": [0.22222, 0.43056, 0, 0, 0.525],
      "711": [0, 0.56597, 0, 0, 0.525],
      "713": [0, 0.56555, 0, 0, 0.525],
      "714": [0, 0.61111, 0, 0, 0.525],
      "715": [0, 0.61111, 0, 0, 0.525],
      "728": [0, 0.61111, 0, 0, 0.525],
      "730": [0, 0.61111, 0, 0, 0.525],
      "770": [0, 0.61111, 0, 0, 0.525],
      "771": [0, 0.61111, 0, 0, 0.525],
      "776": [0, 0.61111, 0, 0, 0.525],
      "915": [0, 0.61111, 0, 0, 0.525],
      "916": [0, 0.61111, 0, 0, 0.525],
      "920": [0, 0.61111, 0, 0, 0.525],
      "923": [0, 0.61111, 0, 0, 0.525],
      "926": [0, 0.61111, 0, 0, 0.525],
      "928": [0, 0.61111, 0, 0, 0.525],
      "931": [0, 0.61111, 0, 0, 0.525],
      "933": [0, 0.61111, 0, 0, 0.525],
      "934": [0, 0.61111, 0, 0, 0.525],
      "936": [0, 0.61111, 0, 0, 0.525],
      "937": [0, 0.61111, 0, 0, 0.525],
      "8216": [0, 0.61111, 0, 0, 0.525],
      "8217": [0, 0.61111, 0, 0, 0.525],
      "8242": [0, 0.61111, 0, 0, 0.525],
      "9251": [0.11111, 0.21944, 0, 0, 0.525]
    }
  };

  /**
   * This file contains metrics regarding fonts and individual symbols. The sigma
   * and xi variables, as well as the metricMap map contain data extracted from
   * TeX, TeX font metrics, and the TTF files. These data are then exposed via the
   * `metrics` variable and the getCharacterMetrics function.
   */
  // In TeX, there are actually three sets of dimensions, one for each of
  // textstyle (size index 5 and higher: >=9pt), scriptstyle (size index 3 and 4:
  // 7-8pt), and scriptscriptstyle (size index 1 and 2: 5-6pt).  These are
  // provided in the the arrays below, in that order.
  //
  // The font metrics are stored in fonts cmsy10, cmsy7, and cmsy5 respsectively.
  // This was determined by running the following script:
  //
  //     latex -interaction=nonstopmode \
  //     '\documentclass{article}\usepackage{amsmath}\begin{document}' \
  //     '$a$ \expandafter\show\the\textfont2' \
  //     '\expandafter\show\the\scriptfont2' \
  //     '\expandafter\show\the\scriptscriptfont2' \
  //     '\stop'
  //
  // The metrics themselves were retreived using the following commands:
  //
  //     tftopl cmsy10
  //     tftopl cmsy7
  //     tftopl cmsy5
  //
  // The output of each of these commands is quite lengthy.  The only part we
  // care about is the FONTDIMEN section. Each value is measured in EMs.
  var sigmasAndXis = {
    slant: [0.250, 0.250, 0.250],
    // sigma1
    space: [0.000, 0.000, 0.000],
    // sigma2
    stretch: [0.000, 0.000, 0.000],
    // sigma3
    shrink: [0.000, 0.000, 0.000],
    // sigma4
    xHeight: [0.431, 0.431, 0.431],
    // sigma5
    quad: [1.000, 1.171, 1.472],
    // sigma6
    extraSpace: [0.000, 0.000, 0.000],
    // sigma7
    num1: [0.677, 0.732, 0.925],
    // sigma8
    num2: [0.394, 0.384, 0.387],
    // sigma9
    num3: [0.444, 0.471, 0.504],
    // sigma10
    denom1: [0.686, 0.752, 1.025],
    // sigma11
    denom2: [0.345, 0.344, 0.532],
    // sigma12
    sup1: [0.413, 0.503, 0.504],
    // sigma13
    sup2: [0.363, 0.431, 0.404],
    // sigma14
    sup3: [0.289, 0.286, 0.294],
    // sigma15
    sub1: [0.150, 0.143, 0.200],
    // sigma16
    sub2: [0.247, 0.286, 0.400],
    // sigma17
    supDrop: [0.386, 0.353, 0.494],
    // sigma18
    subDrop: [0.050, 0.071, 0.100],
    // sigma19
    delim1: [2.390, 1.700, 1.980],
    // sigma20
    delim2: [1.010, 1.157, 1.420],
    // sigma21
    axisHeight: [0.250, 0.250, 0.250],
    // sigma22
    // These font metrics are extracted from TeX by using tftopl on cmex10.tfm;
    // they correspond to the font parameters of the extension fonts (family 3).
    // See the TeXbook, page 441. In AMSTeX, the extension fonts scale; to
    // match cmex7, we'd use cmex7.tfm values for script and scriptscript
    // values.
    defaultRuleThickness: [0.04, 0.049, 0.049],
    // xi8; cmex7: 0.049
    bigOpSpacing1: [0.111, 0.111, 0.111],
    // xi9
    bigOpSpacing2: [0.166, 0.166, 0.166],
    // xi10
    bigOpSpacing3: [0.2, 0.2, 0.2],
    // xi11
    bigOpSpacing4: [0.6, 0.611, 0.611],
    // xi12; cmex7: 0.611
    bigOpSpacing5: [0.1, 0.143, 0.143],
    // xi13; cmex7: 0.143
    // The \sqrt rule width is taken from the height of the surd character.
    // Since we use the same font at all sizes, this thickness doesn't scale.
    sqrtRuleThickness: [0.04, 0.04, 0.04],
    // This value determines how large a pt is, for metrics which are defined
    // in terms of pts.
    // This value is also used in katex.less; if you change it make sure the
    // values match.
    ptPerEm: [10.0, 10.0, 10.0],
    // The space between adjacent `|` columns in an array definition. From
    // `\showthe\doublerulesep` in LaTeX. Equals 2.0 / ptPerEm.
    doubleRuleSep: [0.2, 0.2, 0.2],
    // The width of separator lines in {array} environments. From
    // `\showthe\arrayrulewidth` in LaTeX. Equals 0.4 / ptPerEm.
    arrayRuleWidth: [0.04, 0.04, 0.04],
    // Two values from LaTeX source2e:
    fboxsep: [0.3, 0.3, 0.3],
    //        3 pt / ptPerEm
    fboxrule: [0.04, 0.04, 0.04] // 0.4 pt / ptPerEm

  }; // This map contains a mapping from font name and character code to character
  // should have Latin-1 and Cyrillic characters, but may not depending on the
  // operating system.  The metrics do not account for extra height from the
  // accents.  In the case of Cyrillic characters which have both ascenders and
  // descenders we prefer approximations with ascenders, primarily to prevent
  // the fraction bar or root line from intersecting the glyph.
  // TODO(kevinb) allow union of multiple glyph metrics for better accuracy.

  var extraCharacterMap = {
    // Latin-1
    'Å': 'A',
    'Ð': 'D',
    'Þ': 'o',
    'å': 'a',
    'ð': 'd',
    'þ': 'o',
    // Cyrillic
    'А': 'A',
    'Б': 'B',
    'В': 'B',
    'Г': 'F',
    'Д': 'A',
    'Е': 'E',
    'Ж': 'K',
    'З': '3',
    'И': 'N',
    'Й': 'N',
    'К': 'K',
    'Л': 'N',
    'М': 'M',
    'Н': 'H',
    'О': 'O',
    'П': 'N',
    'Р': 'P',
    'С': 'C',
    'Т': 'T',
    'У': 'y',
    'Ф': 'O',
    'Х': 'X',
    'Ц': 'U',
    'Ч': 'h',
    'Ш': 'W',
    'Щ': 'W',
    'Ъ': 'B',
    'Ы': 'X',
    'Ь': 'B',
    'Э': '3',
    'Ю': 'X',
    'Я': 'R',
    'а': 'a',
    'б': 'b',
    'в': 'a',
    'г': 'r',
    'д': 'y',
    'е': 'e',
    'ж': 'm',
    'з': 'e',
    'и': 'n',
    'й': 'n',
    'к': 'n',
    'л': 'n',
    'м': 'm',
    'н': 'n',
    'о': 'o',
    'п': 'n',
    'р': 'p',
    'с': 'c',
    'т': 'o',
    'у': 'y',
    'ф': 'b',
    'х': 'x',
    'ц': 'n',
    'ч': 'n',
    'ш': 'w',
    'щ': 'w',
    'ъ': 'a',
    'ы': 'm',
    'ь': 'a',
    'э': 'e',
    'ю': 'm',
    'я': 'r'
  };

  /**
   * This function adds new font metrics to default metricMap
   * It can also override existing metrics
   */
  function setFontMetrics(fontName, metrics) {
    fontMetricsData[fontName] = metrics;
  }
  /**
   * This function is a convenience function for looking up information in the
   * metricMap table. It takes a character as a string, and a font.
   *
   * Note: the `width` property may be undefined if fontMetricsData.js wasn't
   * built using `Make extended_metrics`.
   */

  function getCharacterMetrics(character, font, mode) {
    if (!fontMetricsData[font]) {
      throw new Error("Font metrics not found for font: " + font + ".");
    }

    var ch = character.charCodeAt(0);
    var metrics = fontMetricsData[font][ch];

    if (!metrics && character[0] in extraCharacterMap) {
      ch = extraCharacterMap[character[0]].charCodeAt(0);
      metrics = fontMetricsData[font][ch];
    }

    if (!metrics && mode === 'text') {
      // We don't typically have font metrics for Asian scripts.
      // But since we support them in text mode, we need to return
      // some sort of metrics.
      // So if the character is in a script we support but we
      // don't have metrics for it, just use the metrics for
      // the Latin capital letter M. This is close enough because
      // we (currently) only care about the height of the glpyh
      // not its width.
      if (supportedCodepoint(ch)) {
        metrics = fontMetricsData[font][77]; // 77 is the charcode for 'M'
      }
    }

    if (metrics) {
      return {
        depth: metrics[0],
        height: metrics[1],
        italic: metrics[2],
        skew: metrics[3],
        width: metrics[4]
      };
    }
  }
  var fontMetricsBySizeIndex = {};
  /**
   * Get the font metrics for a given size.
   */

  function getGlobalMetrics(size) {
    var sizeIndex;

    if (size >= 5) {
      sizeIndex = 0;
    } else if (size >= 3) {
      sizeIndex = 1;
    } else {
      sizeIndex = 2;
    }

    if (!fontMetricsBySizeIndex[sizeIndex]) {
      var metrics = fontMetricsBySizeIndex[sizeIndex] = {
        cssEmPerMu: sigmasAndXis.quad[sizeIndex] / 18
      };

      for (var key in sigmasAndXis) {
        if (sigmasAndXis.hasOwnProperty(key)) {
          metrics[key] = sigmasAndXis[key][sizeIndex];
        }
      }
    }

    return fontMetricsBySizeIndex[sizeIndex];
  }

  /**
   * This file contains information about the options that the Parser carries
   * around with it while parsing. Data is held in an `Options` object, and when
   * recursing, a new `Options` object can be created with the `.with*` and
   * `.reset` functions.
   */
  var sizeStyleMap = [// Each element contains [textsize, scriptsize, scriptscriptsize].
  // The size mappings are taken from TeX with \normalsize=10pt.
  [1, 1, 1], // size1: [5, 5, 5]              \tiny
  [2, 1, 1], // size2: [6, 5, 5]
  [3, 1, 1], // size3: [7, 5, 5]              \scriptsize
  [4, 2, 1], // size4: [8, 6, 5]              \footnotesize
  [5, 2, 1], // size5: [9, 6, 5]              \small
  [6, 3, 1], // size6: [10, 7, 5]             \normalsize
  [7, 4, 2], // size7: [12, 8, 6]             \large
  [8, 6, 3], // size8: [14.4, 10, 7]          \Large
  [9, 7, 6], // size9: [17.28, 12, 10]        \LARGE
  [10, 8, 7], // size10: [20.74, 14.4, 12]     \huge
  [11, 10, 9] // size11: [24.88, 20.74, 17.28] \HUGE
  ];
  var sizeMultipliers = [// fontMetrics.js:getGlobalMetrics also uses size indexes, so if
  // you change size indexes, change that function.
  0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 1.2, 1.44, 1.728, 2.074, 2.488];

  var sizeAtStyle = function sizeAtStyle(size, style) {
    return style.size < 2 ? size : sizeStyleMap[size - 1][style.size - 1];
  }; // In these types, "" (empty string) means "no change".


  /**
   * This is the main options class. It contains the current style, size, color,
   * and font.
   *
   * Options objects should not be modified. To create a new Options with
   * different properties, call a `.having*` method.
   */
  class Options {
    // A font family applies to a group of fonts (i.e. SansSerif), while a font
    // represents a specific font (i.e. SansSerif Bold).
    // See: https://tex.stackexchange.com/questions/22350/difference-between-textrm-and-mathrm

    /**
     * The base size index.
     */
    constructor(data) {
      this.style = void 0;
      this.color = void 0;
      this.size = void 0;
      this.textSize = void 0;
      this.phantom = void 0;
      this.font = void 0;
      this.fontFamily = void 0;
      this.fontWeight = void 0;
      this.fontShape = void 0;
      this.sizeMultiplier = void 0;
      this.maxSize = void 0;
      this.minRuleThickness = void 0;
      this._fontMetrics = void 0;
      this.style = data.style;
      this.color = data.color;
      this.size = data.size || Options.BASESIZE;
      this.textSize = data.textSize || this.size;
      this.phantom = !!data.phantom;
      this.font = data.font || "";
      this.fontFamily = data.fontFamily || "";
      this.fontWeight = data.fontWeight || '';
      this.fontShape = data.fontShape || '';
      this.sizeMultiplier = sizeMultipliers[this.size - 1];
      this.maxSize = data.maxSize;
      this.minRuleThickness = data.minRuleThickness;
      this._fontMetrics = undefined;
    }
    /**
     * Returns a new options object with the same properties as "this".  Properties
     * from "extension" will be copied to the new options object.
     */


    extend(extension) {
      var data = {
        style: this.style,
        size: this.size,
        textSize: this.textSize,
        color: this.color,
        phantom: this.phantom,
        font: this.font,
        fontFamily: this.fontFamily,
        fontWeight: this.fontWeight,
        fontShape: this.fontShape,
        maxSize: this.maxSize,
        minRuleThickness: this.minRuleThickness
      };

      for (var key in extension) {
        if (extension.hasOwnProperty(key)) {
          data[key] = extension[key];
        }
      }

      return new Options(data);
    }
    /**
     * Return an options object with the given style. If `this.style === style`,
     * returns `this`.
     */


    havingStyle(style) {
      if (this.style === style) {
        return this;
      } else {
        return this.extend({
          style: style,
          size: sizeAtStyle(this.textSize, style)
        });
      }
    }
    /**
     * Return an options object with a cramped version of the current style. If
     * the current style is cramped, returns `this`.
     */


    havingCrampedStyle() {
      return this.havingStyle(this.style.cramp());
    }
    /**
     * Return an options object with the given size and in at least `\textstyle`.
     * Returns `this` if appropriate.
     */


    havingSize(size) {
      if (this.size === size && this.textSize === size) {
        return this;
      } else {
        return this.extend({
          style: this.style.text(),
          size: size,
          textSize: size,
          sizeMultiplier: sizeMultipliers[size - 1]
        });
      }
    }
    /**
     * Like `this.havingSize(BASESIZE).havingStyle(style)`. If `style` is omitted,
     * changes to at least `\textstyle`.
     */


    havingBaseStyle(style) {
      style = style || this.style.text();
      var wantSize = sizeAtStyle(Options.BASESIZE, style);

      if (this.size === wantSize && this.textSize === Options.BASESIZE && this.style === style) {
        return this;
      } else {
        return this.extend({
          style: style,
          size: wantSize
        });
      }
    }
    /**
     * Remove the effect of sizing changes such as \Huge.
     * Keep the effect of the current style, such as \scriptstyle.
     */


    havingBaseSizing() {
      var size;

      switch (this.style.id) {
        case 4:
        case 5:
          size = 3; // normalsize in scriptstyle

          break;

        case 6:
        case 7:
          size = 1; // normalsize in scriptscriptstyle

          break;

        default:
          size = 6;
        // normalsize in textstyle or displaystyle
      }

      return this.extend({
        style: this.style.text(),
        size: size
      });
    }
    /**
     * Create a new options object with the given color.
     */


    withColor(color) {
      return this.extend({
        color: color
      });
    }
    /**
     * Create a new options object with "phantom" set to true.
     */


    withPhantom() {
      return this.extend({
        phantom: true
      });
    }
    /**
     * Creates a new options object with the given math font or old text font.
     * @type {[type]}
     */


    withFont(font) {
      return this.extend({
        font
      });
    }
    /**
     * Create a new options objects with the given fontFamily.
     */


    withTextFontFamily(fontFamily) {
      return this.extend({
        fontFamily,
        font: ""
      });
    }
    /**
     * Creates a new options object with the given font weight
     */


    withTextFontWeight(fontWeight) {
      return this.extend({
        fontWeight,
        font: ""
      });
    }
    /**
     * Creates a new options object with the given font weight
     */


    withTextFontShape(fontShape) {
      return this.extend({
        fontShape,
        font: ""
      });
    }
    /**
     * Return the CSS sizing classes required to switch from enclosing options
     * `oldOptions` to `this`. Returns an array of classes.
     */


    sizingClasses(oldOptions) {
      if (oldOptions.size !== this.size) {
        return ["sizing", "reset-size" + oldOptions.size, "size" + this.size];
      } else {
        return [];
      }
    }
    /**
     * Return the CSS sizing classes required to switch to the base size. Like
     * `this.havingSize(BASESIZE).sizingClasses(this)`.
     */


    baseSizingClasses() {
      if (this.size !== Options.BASESIZE) {
        return ["sizing", "reset-size" + this.size, "size" + Options.BASESIZE];
      } else {
        return [];
      }
    }
    /**
     * Return the font metrics for this size.
     */


    fontMetrics() {
      if (!this._fontMetrics) {
        this._fontMetrics = getGlobalMetrics(this.size);
      }

      return this._fontMetrics;
    }
    /**
     * Gets the CSS color of the current options object
     */


    getColor() {
      if (this.phantom) {
        return "transparent";
      } else {
        return this.color;
      }
    }

  }

  Options.BASESIZE = 6;

  /**
   * This file does conversion between units.  In particular, it provides
   * calculateSize to convert other units into ems.
   */
  // Thus, multiplying a length by this number converts the length from units
  // into pts.  Dividing the result by ptPerEm gives the number of ems
  // *assuming* a font size of ptPerEm (normal size, normal style).

  var ptPerUnit = {
    // https://en.wikibooks.org/wiki/LaTeX/Lengths and
    // https://tex.stackexchange.com/a/8263
    "pt": 1,
    // TeX point
    "mm": 7227 / 2540,
    // millimeter
    "cm": 7227 / 254,
    // centimeter
    "in": 72.27,
    // inch
    "bp": 803 / 800,
    // big (PostScript) points
    "pc": 12,
    // pica
    "dd": 1238 / 1157,
    // didot
    "cc": 14856 / 1157,
    // cicero (12 didot)
    "nd": 685 / 642,
    // new didot
    "nc": 1370 / 107,
    // new cicero (12 new didot)
    "sp": 1 / 65536,
    // scaled point (TeX's internal smallest unit)
    // https://tex.stackexchange.com/a/41371
    "px": 803 / 800 // \pdfpxdimen defaults to 1 bp in pdfTeX and LuaTeX

  }; // Dictionary of relative units, for fast validity testing.

  var relativeUnit = {
    "ex": true,
    "em": true,
    "mu": true
  };

  /**
   * Determine whether the specified unit (either a string defining the unit
   * or a "size" parse node containing a unit field) is valid.
   */
  var validUnit = function validUnit(unit) {
    if (typeof unit !== "string") {
      unit = unit.unit;
    }

    return unit in ptPerUnit || unit in relativeUnit || unit === "ex";
  };
  /*
   * Convert a "size" parse node (with numeric "number" and string "unit" fields,
   * as parsed by functions.js argType "size") into a CSS em value for the
   * current style/scale.  `options` gives the current options.
   */

  var calculateSize = function calculateSize(sizeValue, options) {
    var scale;

    if (sizeValue.unit in ptPerUnit) {
      // Absolute units
      scale = ptPerUnit[sizeValue.unit] // Convert unit to pt
      / options.fontMetrics().ptPerEm // Convert pt to CSS em
      / options.sizeMultiplier; // Unscale to make absolute units
    } else if (sizeValue.unit === "mu") {
      // `mu` units scale with scriptstyle/scriptscriptstyle.
      scale = options.fontMetrics().cssEmPerMu;
    } else {
      // Other relative units always refer to the *textstyle* font
      // in the current size.
      var unitOptions;

      if (options.style.isTight()) {
        // isTight() means current style is script/scriptscript.
        unitOptions = options.havingStyle(options.style.text());
      } else {
        unitOptions = options;
      } // TODO: In TeX these units are relative to the quad of the current
      // *text* font, e.g. cmr10. KaTeX instead uses values from the
      // comparably-sized *Computer Modern symbol* font. At 10pt, these
      // match. At 7pt and 5pt, they differ: cmr7=1.138894, cmsy7=1.170641;
      // cmr5=1.361133, cmsy5=1.472241. Consider $\scriptsize a\kern1emb$.
      // TeX \showlists shows a kern of 1.13889 * fontsize;
      // KaTeX shows a kern of 1.171 * fontsize.


      if (sizeValue.unit === "ex") {
        scale = unitOptions.fontMetrics().xHeight;
      } else if (sizeValue.unit === "em") {
        scale = unitOptions.fontMetrics().quad;
      } else {
        throw new ParseError("Invalid unit: '" + sizeValue.unit + "'");
      }

      if (unitOptions !== options) {
        scale *= unitOptions.sizeMultiplier / options.sizeMultiplier;
      }
    }

    return Math.min(sizeValue.number * scale, options.maxSize);
  };
  /**
   * Round `n` to 4 decimal places, or to the nearest 1/10,000th em. See
   * https://github.com/KaTeX/KaTeX/pull/2460.
   */

  var makeEm = function makeEm(n) {
    return +n.toFixed(4) + "em";
  };

  /**
   * These objects store the data about the DOM nodes we create, as well as some
   * extra data. They can then be transformed into real DOM nodes with the
   * `toNode` function or HTML markup using `toMarkup`. They are useful for both
   * storing extra properties on the nodes, as well as providing a way to easily
   * work with the DOM.
   *
   * Similar functions for working with MathML nodes exist in mathMLTree.js.
   *
   * TODO: refactor `span` and `anchor` into common superclass when
   * target environments support class inheritance
   */

  /**
   * Create an HTML className based on a list of classes. In addition to joining
   * with spaces, we also remove empty classes.
   */
  var createClass = function createClass(classes) {
    return classes.filter(cls => cls).join(" ");
  };

  var initNode = function initNode(classes, options, style) {
    this.classes = classes || [];
    this.attributes = {};
    this.height = 0;
    this.depth = 0;
    this.maxFontSize = 0;
    this.style = style || {};

    if (options) {
      if (options.style.isTight()) {
        this.classes.push("mtight");
      }

      var color = options.getColor();

      if (color) {
        this.style.color = color;
      }
    }
  };
  /**
   * Convert into an HTML node
   */


  var toNode = function toNode(tagName) {
    var node = document.createElement(tagName); // Apply the class

    node.className = createClass(this.classes); // Apply inline styles

    for (var style in this.style) {
      if (this.style.hasOwnProperty(style)) {
        // $FlowFixMe Flow doesn't seem to understand span.style's type.
        node.style[style] = this.style[style];
      }
    } // Apply attributes


    for (var attr in this.attributes) {
      if (this.attributes.hasOwnProperty(attr)) {
        node.setAttribute(attr, this.attributes[attr]);
      }
    } // Append the children, also as HTML nodes


    for (var i = 0; i < this.children.length; i++) {
      node.appendChild(this.children[i].toNode());
    }

    return node;
  };
  /**
   * Convert into an HTML markup string
   */


  var toMarkup = function toMarkup(tagName) {
    var markup = "<" + tagName; // Add the class

    if (this.classes.length) {
      markup += " class=\"" + utils$1.escape(createClass(this.classes)) + "\"";
    }

    var styles = ""; // Add the styles, after hyphenation

    for (var style in this.style) {
      if (this.style.hasOwnProperty(style)) {
        styles += utils$1.hyphenate(style) + ":" + this.style[style] + ";";
      }
    }

    if (styles) {
      markup += " style=\"" + utils$1.escape(styles) + "\"";
    } // Add the attributes


    for (var attr in this.attributes) {
      if (this.attributes.hasOwnProperty(attr)) {
        markup += " " + attr + "=\"" + utils$1.escape(this.attributes[attr]) + "\"";
      }
    }

    markup += ">"; // Add the markup of the children, also as markup

    for (var i = 0; i < this.children.length; i++) {
      markup += this.children[i].toMarkup();
    }

    markup += "</" + tagName + ">";
    return markup;
  }; // Making the type below exact with all optional fields doesn't work due to
  // - https://github.com/facebook/flow/issues/4582
  // - https://github.com/facebook/flow/issues/5688
  // However, since *all* fields are optional, $Shape<> works as suggested in 5688
  // above.
  // This type does not include all CSS properties. Additional properties should
  // be added as needed.


  /**
   * This node represents a span node, with a className, a list of children, and
   * an inline style. It also contains information about its height, depth, and
   * maxFontSize.
   *
   * Represents two types with different uses: SvgSpan to wrap an SVG and DomSpan
   * otherwise. This typesafety is important when HTML builders access a span's
   * children.
   */
  class Span {
    constructor(classes, children, options, style) {
      this.children = void 0;
      this.attributes = void 0;
      this.classes = void 0;
      this.height = void 0;
      this.depth = void 0;
      this.width = void 0;
      this.maxFontSize = void 0;
      this.style = void 0;
      initNode.call(this, classes, options, style);
      this.children = children || [];
    }
    /**
     * Sets an arbitrary attribute on the span. Warning: use this wisely. Not
     * all browsers support attributes the same, and having too many custom
     * attributes is probably bad.
     */


    setAttribute(attribute, value) {
      this.attributes[attribute] = value;
    }

    hasClass(className) {
      return utils$1.contains(this.classes, className);
    }

    toNode() {
      return toNode.call(this, "span");
    }

    toMarkup() {
      return toMarkup.call(this, "span");
    }

  }
  /**
   * This node represents an anchor (<a>) element with a hyperlink.  See `span`
   * for further details.
   */

  class Anchor {
    constructor(href, classes, children, options) {
      this.children = void 0;
      this.attributes = void 0;
      this.classes = void 0;
      this.height = void 0;
      this.depth = void 0;
      this.maxFontSize = void 0;
      this.style = void 0;
      initNode.call(this, classes, options);
      this.children = children || [];
      this.setAttribute('href', href);
    }

    setAttribute(attribute, value) {
      this.attributes[attribute] = value;
    }

    hasClass(className) {
      return utils$1.contains(this.classes, className);
    }

    toNode() {
      return toNode.call(this, "a");
    }

    toMarkup() {
      return toMarkup.call(this, "a");
    }

  }
  /**
   * This node represents an image embed (<img>) element.
   */

  class Img {
    constructor(src, alt, style) {
      this.src = void 0;
      this.alt = void 0;
      this.classes = void 0;
      this.height = void 0;
      this.depth = void 0;
      this.maxFontSize = void 0;
      this.style = void 0;
      this.alt = alt;
      this.src = src;
      this.classes = ["mord"];
      this.style = style;
    }

    hasClass(className) {
      return utils$1.contains(this.classes, className);
    }

    toNode() {
      var node = document.createElement("img");
      node.src = this.src;
      node.alt = this.alt;
      node.className = "mord"; // Apply inline styles

      for (var style in this.style) {
        if (this.style.hasOwnProperty(style)) {
          // $FlowFixMe
          node.style[style] = this.style[style];
        }
      }

      return node;
    }

    toMarkup() {
      var markup = "<img  src='" + this.src + " 'alt='" + this.alt + "' "; // Add the styles, after hyphenation

      var styles = "";

      for (var style in this.style) {
        if (this.style.hasOwnProperty(style)) {
          styles += utils$1.hyphenate(style) + ":" + this.style[style] + ";";
        }
      }

      if (styles) {
        markup += " style=\"" + utils$1.escape(styles) + "\"";
      }

      markup += "'/>";
      return markup;
    }

  }
  var iCombinations = {
    'î': '\u0131\u0302',
    'ï': '\u0131\u0308',
    'í': '\u0131\u0301',
    // 'ī': '\u0131\u0304', // enable when we add Extended Latin
    'ì': '\u0131\u0300'
  };
  /**
   * A symbol node contains information about a single symbol. It either renders
   * to a single text node, or a span with a single text node in it, depending on
   * whether it has CSS classes, styles, or needs italic correction.
   */

  class SymbolNode {
    constructor(text, height, depth, italic, skew, width, classes, style) {
      this.text = void 0;
      this.height = void 0;
      this.depth = void 0;
      this.italic = void 0;
      this.skew = void 0;
      this.width = void 0;
      this.maxFontSize = void 0;
      this.classes = void 0;
      this.style = void 0;
      this.text = text;
      this.height = height || 0;
      this.depth = depth || 0;
      this.italic = italic || 0;
      this.skew = skew || 0;
      this.width = width || 0;
      this.classes = classes || [];
      this.style = style || {};
      this.maxFontSize = 0; // Mark text from non-Latin scripts with specific classes so that we
      // can specify which fonts to use.  This allows us to render these
      // characters with a serif font in situations where the browser would
      // either default to a sans serif or render a placeholder character.
      // We use CSS class names like cjk_fallback, hangul_fallback and
      // brahmic_fallback. See ./unicodeScripts.js for the set of possible
      // script names

      var script = scriptFromCodepoint(this.text.charCodeAt(0));

      if (script) {
        this.classes.push(script + "_fallback");
      }

      if (/[îïíì]/.test(this.text)) {
        // add ī when we add Extended Latin
        this.text = iCombinations[this.text];
      }
    }

    hasClass(className) {
      return utils$1.contains(this.classes, className);
    }
    /**
     * Creates a text node or span from a symbol node. Note that a span is only
     * created if it is needed.
     */


    toNode() {
      var node = document.createTextNode(this.text);
      var span = null;

      if (this.italic > 0) {
        span = document.createElement("span");
        span.style.marginRight = makeEm(this.italic);
      }

      if (this.classes.length > 0) {
        span = span || document.createElement("span");
        span.className = createClass(this.classes);
      }

      for (var style in this.style) {
        if (this.style.hasOwnProperty(style)) {
          span = span || document.createElement("span"); // $FlowFixMe Flow doesn't seem to understand span.style's type.

          span.style[style] = this.style[style];
        }
      }

      if (span) {
        span.appendChild(node);
        return span;
      } else {
        return node;
      }
    }
    /**
     * Creates markup for a symbol node.
     */


    toMarkup() {
      // TODO(alpert): More duplication than I'd like from
      // span.prototype.toMarkup and symbolNode.prototype.toNode...
      var needsSpan = false;
      var markup = "<span";

      if (this.classes.length) {
        needsSpan = true;
        markup += " class=\"";
        markup += utils$1.escape(createClass(this.classes));
        markup += "\"";
      }

      var styles = "";

      if (this.italic > 0) {
        styles += "margin-right:" + this.italic + "em;";
      }

      for (var style in this.style) {
        if (this.style.hasOwnProperty(style)) {
          styles += utils$1.hyphenate(style) + ":" + this.style[style] + ";";
        }
      }

      if (styles) {
        needsSpan = true;
        markup += " style=\"" + utils$1.escape(styles) + "\"";
      }

      var escaped = utils$1.escape(this.text);

      if (needsSpan) {
        markup += ">";
        markup += escaped;
        markup += "</span>";
        return markup;
      } else {
        return escaped;
      }
    }

  }
  /**
   * SVG nodes are used to render stretchy wide elements.
   */

  class SvgNode {
    constructor(children, attributes) {
      this.children = void 0;
      this.attributes = void 0;
      this.children = children || [];
      this.attributes = attributes || {};
    }

    toNode() {
      var svgNS = "http://www.w3.org/2000/svg";
      var node = document.createElementNS(svgNS, "svg"); // Apply attributes

      for (var attr in this.attributes) {
        if (Object.prototype.hasOwnProperty.call(this.attributes, attr)) {
          node.setAttribute(attr, this.attributes[attr]);
        }
      }

      for (var i = 0; i < this.children.length; i++) {
        node.appendChild(this.children[i].toNode());
      }

      return node;
    }

    toMarkup() {
      var markup = "<svg xmlns=\"http://www.w3.org/2000/svg\""; // Apply attributes

      for (var attr in this.attributes) {
        if (Object.prototype.hasOwnProperty.call(this.attributes, attr)) {
          markup += " " + attr + "='" + this.attributes[attr] + "'";
        }
      }

      markup += ">";

      for (var i = 0; i < this.children.length; i++) {
        markup += this.children[i].toMarkup();
      }

      markup += "</svg>";
      return markup;
    }

  }
  class PathNode {
    constructor(pathName, alternate) {
      this.pathName = void 0;
      this.alternate = void 0;
      this.pathName = pathName;
      this.alternate = alternate; // Used only for \sqrt, \phase, & tall delims
    }

    toNode() {
      var svgNS = "http://www.w3.org/2000/svg";
      var node = document.createElementNS(svgNS, "path");

      if (this.alternate) {
        node.setAttribute("d", this.alternate);
      } else {
        node.setAttribute("d", path$1[this.pathName]);
      }

      return node;
    }

    toMarkup() {
      if (this.alternate) {
        return "<path d='" + this.alternate + "'/>";
      } else {
        return "<path d='" + path$1[this.pathName] + "'/>";
      }
    }

  }
  class LineNode {
    constructor(attributes) {
      this.attributes = void 0;
      this.attributes = attributes || {};
    }

    toNode() {
      var svgNS = "http://www.w3.org/2000/svg";
      var node = document.createElementNS(svgNS, "line"); // Apply attributes

      for (var attr in this.attributes) {
        if (Object.prototype.hasOwnProperty.call(this.attributes, attr)) {
          node.setAttribute(attr, this.attributes[attr]);
        }
      }

      return node;
    }

    toMarkup() {
      var markup = "<line";

      for (var attr in this.attributes) {
        if (Object.prototype.hasOwnProperty.call(this.attributes, attr)) {
          markup += " " + attr + "='" + this.attributes[attr] + "'";
        }
      }

      markup += "/>";
      return markup;
    }

  }
  function assertSymbolDomNode(group) {
    if (group instanceof SymbolNode) {
      return group;
    } else {
      throw new Error("Expected symbolNode but got " + String(group) + ".");
    }
  }
  function assertSpan(group) {
    if (group instanceof Span) {
      return group;
    } else {
      throw new Error("Expected span<HtmlDomNode> but got " + String(group) + ".");
    }
  }

  /**
   * This file holds a list of all no-argument functions and single-character
   * symbols (like 'a' or ';').
   *
   * For each of the symbols, there are three properties they can have:
   * - font (required): the font to be used for this symbol. Either "main" (the
       normal font), or "ams" (the ams fonts).
   * - group (required): the ParseNode group type the symbol should have (i.e.
       "textord", "mathord", etc).
       See https://github.com/KaTeX/KaTeX/wiki/Examining-TeX#group-types
   * - replace: the character that this symbol or function should be
   *   replaced with (i.e. "\phi" has a replace value of "\u03d5", the phi
   *   character in the main font).
   *
   * The outermost map in the table indicates what mode the symbols should be
   * accepted in (e.g. "math" or "text").
   */
  // Some of these have a "-token" suffix since these are also used as `ParseNode`
  // types for raw text tokens, and we want to avoid conflicts with higher-level
  // `ParseNode` types. These `ParseNode`s are constructed within `Parser` by
  // looking up the `symbols` map.
  var ATOMS = {
    "bin": 1,
    "close": 1,
    "inner": 1,
    "open": 1,
    "punct": 1,
    "rel": 1
  };
  var NON_ATOMS = {
    "accent-token": 1,
    "mathord": 1,
    "op-token": 1,
    "spacing": 1,
    "textord": 1
  };
  var symbols = {
    "math": {},
    "text": {}
  };
  /** `acceptUnicodeChar = true` is only applicable if `replace` is set. */

  function defineSymbol(mode, font, group, replace, name, acceptUnicodeChar) {
    symbols[mode][name] = {
      font,
      group,
      replace
    };

    if (acceptUnicodeChar && replace) {
      symbols[mode][replace] = symbols[mode][name];
    }
  } // Some abbreviations for commonly used strings.
  // This helps minify the code, and also spotting typos using jshint.
  // modes:

  var math = "math";
  var text$5 = "text"; // fonts:

  var main = "main";
  var ams = "ams"; // groups:

  var accent = "accent-token";
  var bin = "bin";
  var close = "close";
  var inner = "inner";
  var mathord = "mathord";
  var op = "op-token";
  var open = "open";
  var punct = "punct";
  var rel = "rel";
  var spacing = "spacing";
  var textord = "textord"; // Now comes the symbol table
  // Relation Symbols

  defineSymbol(math, main, rel, "\u2261", "\\equiv", true);
  defineSymbol(math, main, rel, "\u227a", "\\prec", true);
  defineSymbol(math, main, rel, "\u227b", "\\succ", true);
  defineSymbol(math, main, rel, "\u223c", "\\sim", true);
  defineSymbol(math, main, rel, "\u22a5", "\\perp");
  defineSymbol(math, main, rel, "\u2aaf", "\\preceq", true);
  defineSymbol(math, main, rel, "\u2ab0", "\\succeq", true);
  defineSymbol(math, main, rel, "\u2243", "\\simeq", true);
  defineSymbol(math, main, rel, "\u2223", "\\mid", true);
  defineSymbol(math, main, rel, "\u226a", "\\ll", true);
  defineSymbol(math, main, rel, "\u226b", "\\gg", true);
  defineSymbol(math, main, rel, "\u224d", "\\asymp", true);
  defineSymbol(math, main, rel, "\u2225", "\\parallel");
  defineSymbol(math, main, rel, "\u22c8", "\\bowtie", true);
  defineSymbol(math, main, rel, "\u2323", "\\smile", true);
  defineSymbol(math, main, rel, "\u2291", "\\sqsubseteq", true);
  defineSymbol(math, main, rel, "\u2292", "\\sqsupseteq", true);
  defineSymbol(math, main, rel, "\u2250", "\\doteq", true);
  defineSymbol(math, main, rel, "\u2322", "\\frown", true);
  defineSymbol(math, main, rel, "\u220b", "\\ni", true);
  defineSymbol(math, main, rel, "\u221d", "\\propto", true);
  defineSymbol(math, main, rel, "\u22a2", "\\vdash", true);
  defineSymbol(math, main, rel, "\u22a3", "\\dashv", true);
  defineSymbol(math, main, rel, "\u220b", "\\owns"); // Punctuation

  defineSymbol(math, main, punct, "\u002e", "\\ldotp");
  defineSymbol(math, main, punct, "\u22c5", "\\cdotp"); // Misc Symbols

  defineSymbol(math, main, textord, "\u0023", "\\#");
  defineSymbol(text$5, main, textord, "\u0023", "\\#");
  defineSymbol(math, main, textord, "\u0026", "\\&");
  defineSymbol(text$5, main, textord, "\u0026", "\\&");
  defineSymbol(math, main, textord, "\u2135", "\\aleph", true);
  defineSymbol(math, main, textord, "\u2200", "\\forall", true);
  defineSymbol(math, main, textord, "\u210f", "\\hbar", true);
  defineSymbol(math, main, textord, "\u2203", "\\exists", true);
  defineSymbol(math, main, textord, "\u2207", "\\nabla", true);
  defineSymbol(math, main, textord, "\u266d", "\\flat", true);
  defineSymbol(math, main, textord, "\u2113", "\\ell", true);
  defineSymbol(math, main, textord, "\u266e", "\\natural", true);
  defineSymbol(math, main, textord, "\u2663", "\\clubsuit", true);
  defineSymbol(math, main, textord, "\u2118", "\\wp", true);
  defineSymbol(math, main, textord, "\u266f", "\\sharp", true);
  defineSymbol(math, main, textord, "\u2662", "\\diamondsuit", true);
  defineSymbol(math, main, textord, "\u211c", "\\Re", true);
  defineSymbol(math, main, textord, "\u2661", "\\heartsuit", true);
  defineSymbol(math, main, textord, "\u2111", "\\Im", true);
  defineSymbol(math, main, textord, "\u2660", "\\spadesuit", true);
  defineSymbol(math, main, textord, "\u00a7", "\\S", true);
  defineSymbol(text$5, main, textord, "\u00a7", "\\S");
  defineSymbol(math, main, textord, "\u00b6", "\\P", true);
  defineSymbol(text$5, main, textord, "\u00b6", "\\P"); // Math and Text

  defineSymbol(math, main, textord, "\u2020", "\\dag");
  defineSymbol(text$5, main, textord, "\u2020", "\\dag");
  defineSymbol(text$5, main, textord, "\u2020", "\\textdagger");
  defineSymbol(math, main, textord, "\u2021", "\\ddag");
  defineSymbol(text$5, main, textord, "\u2021", "\\ddag");
  defineSymbol(text$5, main, textord, "\u2021", "\\textdaggerdbl"); // Large Delimiters

  defineSymbol(math, main, close, "\u23b1", "\\rmoustache", true);
  defineSymbol(math, main, open, "\u23b0", "\\lmoustache", true);
  defineSymbol(math, main, close, "\u27ef", "\\rgroup", true);
  defineSymbol(math, main, open, "\u27ee", "\\lgroup", true); // Binary Operators

  defineSymbol(math, main, bin, "\u2213", "\\mp", true);
  defineSymbol(math, main, bin, "\u2296", "\\ominus", true);
  defineSymbol(math, main, bin, "\u228e", "\\uplus", true);
  defineSymbol(math, main, bin, "\u2293", "\\sqcap", true);
  defineSymbol(math, main, bin, "\u2217", "\\ast");
  defineSymbol(math, main, bin, "\u2294", "\\sqcup", true);
  defineSymbol(math, main, bin, "\u25ef", "\\bigcirc", true);
  defineSymbol(math, main, bin, "\u2219", "\\bullet");
  defineSymbol(math, main, bin, "\u2021", "\\ddagger");
  defineSymbol(math, main, bin, "\u2240", "\\wr", true);
  defineSymbol(math, main, bin, "\u2a3f", "\\amalg");
  defineSymbol(math, main, bin, "\u0026", "\\And"); // from amsmath
  // Arrow Symbols

  defineSymbol(math, main, rel, "\u27f5", "\\longleftarrow", true);
  defineSymbol(math, main, rel, "\u21d0", "\\Leftarrow", true);
  defineSymbol(math, main, rel, "\u27f8", "\\Longleftarrow", true);
  defineSymbol(math, main, rel, "\u27f6", "\\longrightarrow", true);
  defineSymbol(math, main, rel, "\u21d2", "\\Rightarrow", true);
  defineSymbol(math, main, rel, "\u27f9", "\\Longrightarrow", true);
  defineSymbol(math, main, rel, "\u2194", "\\leftrightarrow", true);
  defineSymbol(math, main, rel, "\u27f7", "\\longleftrightarrow", true);
  defineSymbol(math, main, rel, "\u21d4", "\\Leftrightarrow", true);
  defineSymbol(math, main, rel, "\u27fa", "\\Longleftrightarrow", true);
  defineSymbol(math, main, rel, "\u21a6", "\\mapsto", true);
  defineSymbol(math, main, rel, "\u27fc", "\\longmapsto", true);
  defineSymbol(math, main, rel, "\u2197", "\\nearrow", true);
  defineSymbol(math, main, rel, "\u21a9", "\\hookleftarrow", true);
  defineSymbol(math, main, rel, "\u21aa", "\\hookrightarrow", true);
  defineSymbol(math, main, rel, "\u2198", "\\searrow", true);
  defineSymbol(math, main, rel, "\u21bc", "\\leftharpoonup", true);
  defineSymbol(math, main, rel, "\u21c0", "\\rightharpoonup", true);
  defineSymbol(math, main, rel, "\u2199", "\\swarrow", true);
  defineSymbol(math, main, rel, "\u21bd", "\\leftharpoondown", true);
  defineSymbol(math, main, rel, "\u21c1", "\\rightharpoondown", true);
  defineSymbol(math, main, rel, "\u2196", "\\nwarrow", true);
  defineSymbol(math, main, rel, "\u21cc", "\\rightleftharpoons", true); // AMS Negated Binary Relations

  defineSymbol(math, ams, rel, "\u226e", "\\nless", true); // Symbol names preceeded by "@" each have a corresponding macro.

  defineSymbol(math, ams, rel, "\ue010", "\\@nleqslant");
  defineSymbol(math, ams, rel, "\ue011", "\\@nleqq");
  defineSymbol(math, ams, rel, "\u2a87", "\\lneq", true);
  defineSymbol(math, ams, rel, "\u2268", "\\lneqq", true);
  defineSymbol(math, ams, rel, "\ue00c", "\\@lvertneqq");
  defineSymbol(math, ams, rel, "\u22e6", "\\lnsim", true);
  defineSymbol(math, ams, rel, "\u2a89", "\\lnapprox", true);
  defineSymbol(math, ams, rel, "\u2280", "\\nprec", true); // unicode-math maps \u22e0 to \npreccurlyeq. We'll use the AMS synonym.

  defineSymbol(math, ams, rel, "\u22e0", "\\npreceq", true);
  defineSymbol(math, ams, rel, "\u22e8", "\\precnsim", true);
  defineSymbol(math, ams, rel, "\u2ab9", "\\precnapprox", true);
  defineSymbol(math, ams, rel, "\u2241", "\\nsim", true);
  defineSymbol(math, ams, rel, "\ue006", "\\@nshortmid");
  defineSymbol(math, ams, rel, "\u2224", "\\nmid", true);
  defineSymbol(math, ams, rel, "\u22ac", "\\nvdash", true);
  defineSymbol(math, ams, rel, "\u22ad", "\\nvDash", true);
  defineSymbol(math, ams, rel, "\u22ea", "\\ntriangleleft");
  defineSymbol(math, ams, rel, "\u22ec", "\\ntrianglelefteq", true);
  defineSymbol(math, ams, rel, "\u228a", "\\subsetneq", true);
  defineSymbol(math, ams, rel, "\ue01a", "\\@varsubsetneq");
  defineSymbol(math, ams, rel, "\u2acb", "\\subsetneqq", true);
  defineSymbol(math, ams, rel, "\ue017", "\\@varsubsetneqq");
  defineSymbol(math, ams, rel, "\u226f", "\\ngtr", true);
  defineSymbol(math, ams, rel, "\ue00f", "\\@ngeqslant");
  defineSymbol(math, ams, rel, "\ue00e", "\\@ngeqq");
  defineSymbol(math, ams, rel, "\u2a88", "\\gneq", true);
  defineSymbol(math, ams, rel, "\u2269", "\\gneqq", true);
  defineSymbol(math, ams, rel, "\ue00d", "\\@gvertneqq");
  defineSymbol(math, ams, rel, "\u22e7", "\\gnsim", true);
  defineSymbol(math, ams, rel, "\u2a8a", "\\gnapprox", true);
  defineSymbol(math, ams, rel, "\u2281", "\\nsucc", true); // unicode-math maps \u22e1 to \nsucccurlyeq. We'll use the AMS synonym.

  defineSymbol(math, ams, rel, "\u22e1", "\\nsucceq", true);
  defineSymbol(math, ams, rel, "\u22e9", "\\succnsim", true);
  defineSymbol(math, ams, rel, "\u2aba", "\\succnapprox", true); // unicode-math maps \u2246 to \simneqq. We'll use the AMS synonym.

  defineSymbol(math, ams, rel, "\u2246", "\\ncong", true);
  defineSymbol(math, ams, rel, "\ue007", "\\@nshortparallel");
  defineSymbol(math, ams, rel, "\u2226", "\\nparallel", true);
  defineSymbol(math, ams, rel, "\u22af", "\\nVDash", true);
  defineSymbol(math, ams, rel, "\u22eb", "\\ntriangleright");
  defineSymbol(math, ams, rel, "\u22ed", "\\ntrianglerighteq", true);
  defineSymbol(math, ams, rel, "\ue018", "\\@nsupseteqq");
  defineSymbol(math, ams, rel, "\u228b", "\\supsetneq", true);
  defineSymbol(math, ams, rel, "\ue01b", "\\@varsupsetneq");
  defineSymbol(math, ams, rel, "\u2acc", "\\supsetneqq", true);
  defineSymbol(math, ams, rel, "\ue019", "\\@varsupsetneqq");
  defineSymbol(math, ams, rel, "\u22ae", "\\nVdash", true);
  defineSymbol(math, ams, rel, "\u2ab5", "\\precneqq", true);
  defineSymbol(math, ams, rel, "\u2ab6", "\\succneqq", true);
  defineSymbol(math, ams, rel, "\ue016", "\\@nsubseteqq");
  defineSymbol(math, ams, bin, "\u22b4", "\\unlhd");
  defineSymbol(math, ams, bin, "\u22b5", "\\unrhd"); // AMS Negated Arrows

  defineSymbol(math, ams, rel, "\u219a", "\\nleftarrow", true);
  defineSymbol(math, ams, rel, "\u219b", "\\nrightarrow", true);
  defineSymbol(math, ams, rel, "\u21cd", "\\nLeftarrow", true);
  defineSymbol(math, ams, rel, "\u21cf", "\\nRightarrow", true);
  defineSymbol(math, ams, rel, "\u21ae", "\\nleftrightarrow", true);
  defineSymbol(math, ams, rel, "\u21ce", "\\nLeftrightarrow", true); // AMS Misc

  defineSymbol(math, ams, rel, "\u25b3", "\\vartriangle");
  defineSymbol(math, ams, textord, "\u210f", "\\hslash");
  defineSymbol(math, ams, textord, "\u25bd", "\\triangledown");
  defineSymbol(math, ams, textord, "\u25ca", "\\lozenge");
  defineSymbol(math, ams, textord, "\u24c8", "\\circledS");
  defineSymbol(math, ams, textord, "\u00ae", "\\circledR");
  defineSymbol(text$5, ams, textord, "\u00ae", "\\circledR");
  defineSymbol(math, ams, textord, "\u2221", "\\measuredangle", true);
  defineSymbol(math, ams, textord, "\u2204", "\\nexists");
  defineSymbol(math, ams, textord, "\u2127", "\\mho");
  defineSymbol(math, ams, textord, "\u2132", "\\Finv", true);
  defineSymbol(math, ams, textord, "\u2141", "\\Game", true);
  defineSymbol(math, ams, textord, "\u2035", "\\backprime");
  defineSymbol(math, ams, textord, "\u25b2", "\\blacktriangle");
  defineSymbol(math, ams, textord, "\u25bc", "\\blacktriangledown");
  defineSymbol(math, ams, textord, "\u25a0", "\\blacksquare");
  defineSymbol(math, ams, textord, "\u29eb", "\\blacklozenge");
  defineSymbol(math, ams, textord, "\u2605", "\\bigstar");
  defineSymbol(math, ams, textord, "\u2222", "\\sphericalangle", true);
  defineSymbol(math, ams, textord, "\u2201", "\\complement", true); // unicode-math maps U+F0 to \matheth. We map to AMS function \eth

  defineSymbol(math, ams, textord, "\u00f0", "\\eth", true);
  defineSymbol(text$5, main, textord, "\u00f0", "\u00f0");
  defineSymbol(math, ams, textord, "\u2571", "\\diagup");
  defineSymbol(math, ams, textord, "\u2572", "\\diagdown");
  defineSymbol(math, ams, textord, "\u25a1", "\\square");
  defineSymbol(math, ams, textord, "\u25a1", "\\Box");
  defineSymbol(math, ams, textord, "\u25ca", "\\Diamond"); // unicode-math maps U+A5 to \mathyen. We map to AMS function \yen

  defineSymbol(math, ams, textord, "\u00a5", "\\yen", true);
  defineSymbol(text$5, ams, textord, "\u00a5", "\\yen", true);
  defineSymbol(math, ams, textord, "\u2713", "\\checkmark", true);
  defineSymbol(text$5, ams, textord, "\u2713", "\\checkmark"); // AMS Hebrew

  defineSymbol(math, ams, textord, "\u2136", "\\beth", true);
  defineSymbol(math, ams, textord, "\u2138", "\\daleth", true);
  defineSymbol(math, ams, textord, "\u2137", "\\gimel", true); // AMS Greek

  defineSymbol(math, ams, textord, "\u03dd", "\\digamma", true);
  defineSymbol(math, ams, textord, "\u03f0", "\\varkappa"); // AMS Delimiters

  defineSymbol(math, ams, open, "\u250c", "\\@ulcorner", true);
  defineSymbol(math, ams, close, "\u2510", "\\@urcorner", true);
  defineSymbol(math, ams, open, "\u2514", "\\@llcorner", true);
  defineSymbol(math, ams, close, "\u2518", "\\@lrcorner", true); // AMS Binary Relations

  defineSymbol(math, ams, rel, "\u2266", "\\leqq", true);
  defineSymbol(math, ams, rel, "\u2a7d", "\\leqslant", true);
  defineSymbol(math, ams, rel, "\u2a95", "\\eqslantless", true);
  defineSymbol(math, ams, rel, "\u2272", "\\lesssim", true);
  defineSymbol(math, ams, rel, "\u2a85", "\\lessapprox", true);
  defineSymbol(math, ams, rel, "\u224a", "\\approxeq", true);
  defineSymbol(math, ams, bin, "\u22d6", "\\lessdot");
  defineSymbol(math, ams, rel, "\u22d8", "\\lll", true);
  defineSymbol(math, ams, rel, "\u2276", "\\lessgtr", true);
  defineSymbol(math, ams, rel, "\u22da", "\\lesseqgtr", true);
  defineSymbol(math, ams, rel, "\u2a8b", "\\lesseqqgtr", true);
  defineSymbol(math, ams, rel, "\u2251", "\\doteqdot");
  defineSymbol(math, ams, rel, "\u2253", "\\risingdotseq", true);
  defineSymbol(math, ams, rel, "\u2252", "\\fallingdotseq", true);
  defineSymbol(math, ams, rel, "\u223d", "\\backsim", true);
  defineSymbol(math, ams, rel, "\u22cd", "\\backsimeq", true);
  defineSymbol(math, ams, rel, "\u2ac5", "\\subseteqq", true);
  defineSymbol(math, ams, rel, "\u22d0", "\\Subset", true);
  defineSymbol(math, ams, rel, "\u228f", "\\sqsubset", true);
  defineSymbol(math, ams, rel, "\u227c", "\\preccurlyeq", true);
  defineSymbol(math, ams, rel, "\u22de", "\\curlyeqprec", true);
  defineSymbol(math, ams, rel, "\u227e", "\\precsim", true);
  defineSymbol(math, ams, rel, "\u2ab7", "\\precapprox", true);
  defineSymbol(math, ams, rel, "\u22b2", "\\vartriangleleft");
  defineSymbol(math, ams, rel, "\u22b4", "\\trianglelefteq");
  defineSymbol(math, ams, rel, "\u22a8", "\\vDash", true);
  defineSymbol(math, ams, rel, "\u22aa", "\\Vvdash", true);
  defineSymbol(math, ams, rel, "\u2323", "\\smallsmile");
  defineSymbol(math, ams, rel, "\u2322", "\\smallfrown");
  defineSymbol(math, ams, rel, "\u224f", "\\bumpeq", true);
  defineSymbol(math, ams, rel, "\u224e", "\\Bumpeq", true);
  defineSymbol(math, ams, rel, "\u2267", "\\geqq", true);
  defineSymbol(math, ams, rel, "\u2a7e", "\\geqslant", true);
  defineSymbol(math, ams, rel, "\u2a96", "\\eqslantgtr", true);
  defineSymbol(math, ams, rel, "\u2273", "\\gtrsim", true);
  defineSymbol(math, ams, rel, "\u2a86", "\\gtrapprox", true);
  defineSymbol(math, ams, bin, "\u22d7", "\\gtrdot");
  defineSymbol(math, ams, rel, "\u22d9", "\\ggg", true);
  defineSymbol(math, ams, rel, "\u2277", "\\gtrless", true);
  defineSymbol(math, ams, rel, "\u22db", "\\gtreqless", true);
  defineSymbol(math, ams, rel, "\u2a8c", "\\gtreqqless", true);
  defineSymbol(math, ams, rel, "\u2256", "\\eqcirc", true);
  defineSymbol(math, ams, rel, "\u2257", "\\circeq", true);
  defineSymbol(math, ams, rel, "\u225c", "\\triangleq", true);
  defineSymbol(math, ams, rel, "\u223c", "\\thicksim");
  defineSymbol(math, ams, rel, "\u2248", "\\thickapprox");
  defineSymbol(math, ams, rel, "\u2ac6", "\\supseteqq", true);
  defineSymbol(math, ams, rel, "\u22d1", "\\Supset", true);
  defineSymbol(math, ams, rel, "\u2290", "\\sqsupset", true);
  defineSymbol(math, ams, rel, "\u227d", "\\succcurlyeq", true);
  defineSymbol(math, ams, rel, "\u22df", "\\curlyeqsucc", true);
  defineSymbol(math, ams, rel, "\u227f", "\\succsim", true);
  defineSymbol(math, ams, rel, "\u2ab8", "\\succapprox", true);
  defineSymbol(math, ams, rel, "\u22b3", "\\vartriangleright");
  defineSymbol(math, ams, rel, "\u22b5", "\\trianglerighteq");
  defineSymbol(math, ams, rel, "\u22a9", "\\Vdash", true);
  defineSymbol(math, ams, rel, "\u2223", "\\shortmid");
  defineSymbol(math, ams, rel, "\u2225", "\\shortparallel");
  defineSymbol(math, ams, rel, "\u226c", "\\between", true);
  defineSymbol(math, ams, rel, "\u22d4", "\\pitchfork", true);
  defineSymbol(math, ams, rel, "\u221d", "\\varpropto");
  defineSymbol(math, ams, rel, "\u25c0", "\\blacktriangleleft"); // unicode-math says that \therefore is a mathord atom.
  // We kept the amssymb atom type, which is rel.

  defineSymbol(math, ams, rel, "\u2234", "\\therefore", true);
  defineSymbol(math, ams, rel, "\u220d", "\\backepsilon");
  defineSymbol(math, ams, rel, "\u25b6", "\\blacktriangleright"); // unicode-math says that \because is a mathord atom.
  // We kept the amssymb atom type, which is rel.

  defineSymbol(math, ams, rel, "\u2235", "\\because", true);
  defineSymbol(math, ams, rel, "\u22d8", "\\llless");
  defineSymbol(math, ams, rel, "\u22d9", "\\gggtr");
  defineSymbol(math, ams, bin, "\u22b2", "\\lhd");
  defineSymbol(math, ams, bin, "\u22b3", "\\rhd");
  defineSymbol(math, ams, rel, "\u2242", "\\eqsim", true);
  defineSymbol(math, main, rel, "\u22c8", "\\Join");
  defineSymbol(math, ams, rel, "\u2251", "\\Doteq", true); // AMS Binary Operators

  defineSymbol(math, ams, bin, "\u2214", "\\dotplus", true);
  defineSymbol(math, ams, bin, "\u2216", "\\smallsetminus");
  defineSymbol(math, ams, bin, "\u22d2", "\\Cap", true);
  defineSymbol(math, ams, bin, "\u22d3", "\\Cup", true);
  defineSymbol(math, ams, bin, "\u2a5e", "\\doublebarwedge", true);
  defineSymbol(math, ams, bin, "\u229f", "\\boxminus", true);
  defineSymbol(math, ams, bin, "\u229e", "\\boxplus", true);
  defineSymbol(math, ams, bin, "\u22c7", "\\divideontimes", true);
  defineSymbol(math, ams, bin, "\u22c9", "\\ltimes", true);
  defineSymbol(math, ams, bin, "\u22ca", "\\rtimes", true);
  defineSymbol(math, ams, bin, "\u22cb", "\\leftthreetimes", true);
  defineSymbol(math, ams, bin, "\u22cc", "\\rightthreetimes", true);
  defineSymbol(math, ams, bin, "\u22cf", "\\curlywedge", true);
  defineSymbol(math, ams, bin, "\u22ce", "\\curlyvee", true);
  defineSymbol(math, ams, bin, "\u229d", "\\circleddash", true);
  defineSymbol(math, ams, bin, "\u229b", "\\circledast", true);
  defineSymbol(math, ams, bin, "\u22c5", "\\centerdot");
  defineSymbol(math, ams, bin, "\u22ba", "\\intercal", true);
  defineSymbol(math, ams, bin, "\u22d2", "\\doublecap");
  defineSymbol(math, ams, bin, "\u22d3", "\\doublecup");
  defineSymbol(math, ams, bin, "\u22a0", "\\boxtimes", true); // AMS Arrows
  // Note: unicode-math maps \u21e2 to their own function \rightdasharrow.
  // We'll map it to AMS function \dashrightarrow. It produces the same atom.

  defineSymbol(math, ams, rel, "\u21e2", "\\dashrightarrow", true); // unicode-math maps \u21e0 to \leftdasharrow. We'll use the AMS synonym.

  defineSymbol(math, ams, rel, "\u21e0", "\\dashleftarrow", true);
  defineSymbol(math, ams, rel, "\u21c7", "\\leftleftarrows", true);
  defineSymbol(math, ams, rel, "\u21c6", "\\leftrightarrows", true);
  defineSymbol(math, ams, rel, "\u21da", "\\Lleftarrow", true);
  defineSymbol(math, ams, rel, "\u219e", "\\twoheadleftarrow", true);
  defineSymbol(math, ams, rel, "\u21a2", "\\leftarrowtail", true);
  defineSymbol(math, ams, rel, "\u21ab", "\\looparrowleft", true);
  defineSymbol(math, ams, rel, "\u21cb", "\\leftrightharpoons", true);
  defineSymbol(math, ams, rel, "\u21b6", "\\curvearrowleft", true); // unicode-math maps \u21ba to \acwopencirclearrow. We'll use the AMS synonym.

  defineSymbol(math, ams, rel, "\u21ba", "\\circlearrowleft", true);
  defineSymbol(math, ams, rel, "\u21b0", "\\Lsh", true);
  defineSymbol(math, ams, rel, "\u21c8", "\\upuparrows", true);
  defineSymbol(math, ams, rel, "\u21bf", "\\upharpoonleft", true);
  defineSymbol(math, ams, rel, "\u21c3", "\\downharpoonleft", true);
  defineSymbol(math, main, rel, "\u22b6", "\\origof", true); // not in font

  defineSymbol(math, main, rel, "\u22b7", "\\imageof", true); // not in font

  defineSymbol(math, ams, rel, "\u22b8", "\\multimap", true);
  defineSymbol(math, ams, rel, "\u21ad", "\\leftrightsquigarrow", true);
  defineSymbol(math, ams, rel, "\u21c9", "\\rightrightarrows", true);
  defineSymbol(math, ams, rel, "\u21c4", "\\rightleftarrows", true);
  defineSymbol(math, ams, rel, "\u21a0", "\\twoheadrightarrow", true);
  defineSymbol(math, ams, rel, "\u21a3", "\\rightarrowtail", true);
  defineSymbol(math, ams, rel, "\u21ac", "\\looparrowright", true);
  defineSymbol(math, ams, rel, "\u21b7", "\\curvearrowright", true); // unicode-math maps \u21bb to \cwopencirclearrow. We'll use the AMS synonym.

  defineSymbol(math, ams, rel, "\u21bb", "\\circlearrowright", true);
  defineSymbol(math, ams, rel, "\u21b1", "\\Rsh", true);
  defineSymbol(math, ams, rel, "\u21ca", "\\downdownarrows", true);
  defineSymbol(math, ams, rel, "\u21be", "\\upharpoonright", true);
  defineSymbol(math, ams, rel, "\u21c2", "\\downharpoonright", true);
  defineSymbol(math, ams, rel, "\u21dd", "\\rightsquigarrow", true);
  defineSymbol(math, ams, rel, "\u21dd", "\\leadsto");
  defineSymbol(math, ams, rel, "\u21db", "\\Rrightarrow", true);
  defineSymbol(math, ams, rel, "\u21be", "\\restriction");
  defineSymbol(math, main, textord, "\u2018", "`");
  defineSymbol(math, main, textord, "$", "\\$");
  defineSymbol(text$5, main, textord, "$", "\\$");
  defineSymbol(text$5, main, textord, "$", "\\textdollar");
  defineSymbol(math, main, textord, "%", "\\%");
  defineSymbol(text$5, main, textord, "%", "\\%");
  defineSymbol(math, main, textord, "_", "\\_");
  defineSymbol(text$5, main, textord, "_", "\\_");
  defineSymbol(text$5, main, textord, "_", "\\textunderscore");
  defineSymbol(math, main, textord, "\u2220", "\\angle", true);
  defineSymbol(math, main, textord, "\u221e", "\\infty", true);
  defineSymbol(math, main, textord, "\u2032", "\\prime");
  defineSymbol(math, main, textord, "\u25b3", "\\triangle");
  defineSymbol(math, main, textord, "\u0393", "\\Gamma", true);
  defineSymbol(math, main, textord, "\u0394", "\\Delta", true);
  defineSymbol(math, main, textord, "\u0398", "\\Theta", true);
  defineSymbol(math, main, textord, "\u039b", "\\Lambda", true);
  defineSymbol(math, main, textord, "\u039e", "\\Xi", true);
  defineSymbol(math, main, textord, "\u03a0", "\\Pi", true);
  defineSymbol(math, main, textord, "\u03a3", "\\Sigma", true);
  defineSymbol(math, main, textord, "\u03a5", "\\Upsilon", true);
  defineSymbol(math, main, textord, "\u03a6", "\\Phi", true);
  defineSymbol(math, main, textord, "\u03a8", "\\Psi", true);
  defineSymbol(math, main, textord, "\u03a9", "\\Omega", true);
  defineSymbol(math, main, textord, "A", "\u0391");
  defineSymbol(math, main, textord, "B", "\u0392");
  defineSymbol(math, main, textord, "E", "\u0395");
  defineSymbol(math, main, textord, "Z", "\u0396");
  defineSymbol(math, main, textord, "H", "\u0397");
  defineSymbol(math, main, textord, "I", "\u0399");
  defineSymbol(math, main, textord, "K", "\u039A");
  defineSymbol(math, main, textord, "M", "\u039C");
  defineSymbol(math, main, textord, "N", "\u039D");
  defineSymbol(math, main, textord, "O", "\u039F");
  defineSymbol(math, main, textord, "P", "\u03A1");
  defineSymbol(math, main, textord, "T", "\u03A4");
  defineSymbol(math, main, textord, "X", "\u03A7");
  defineSymbol(math, main, textord, "\u00ac", "\\neg", true);
  defineSymbol(math, main, textord, "\u00ac", "\\lnot");
  defineSymbol(math, main, textord, "\u22a4", "\\top");
  defineSymbol(math, main, textord, "\u22a5", "\\bot");
  defineSymbol(math, main, textord, "\u2205", "\\emptyset");
  defineSymbol(math, ams, textord, "\u2205", "\\varnothing");
  defineSymbol(math, main, mathord, "\u03b1", "\\alpha", true);
  defineSymbol(math, main, mathord, "\u03b2", "\\beta", true);
  defineSymbol(math, main, mathord, "\u03b3", "\\gamma", true);
  defineSymbol(math, main, mathord, "\u03b4", "\\delta", true);
  defineSymbol(math, main, mathord, "\u03f5", "\\epsilon", true);
  defineSymbol(math, main, mathord, "\u03b6", "\\zeta", true);
  defineSymbol(math, main, mathord, "\u03b7", "\\eta", true);
  defineSymbol(math, main, mathord, "\u03b8", "\\theta", true);
  defineSymbol(math, main, mathord, "\u03b9", "\\iota", true);
  defineSymbol(math, main, mathord, "\u03ba", "\\kappa", true);
  defineSymbol(math, main, mathord, "\u03bb", "\\lambda", true);
  defineSymbol(math, main, mathord, "\u03bc", "\\mu", true);
  defineSymbol(math, main, mathord, "\u03bd", "\\nu", true);
  defineSymbol(math, main, mathord, "\u03be", "\\xi", true);
  defineSymbol(math, main, mathord, "\u03bf", "\\omicron", true);
  defineSymbol(math, main, mathord, "\u03c0", "\\pi", true);
  defineSymbol(math, main, mathord, "\u03c1", "\\rho", true);
  defineSymbol(math, main, mathord, "\u03c3", "\\sigma", true);
  defineSymbol(math, main, mathord, "\u03c4", "\\tau", true);
  defineSymbol(math, main, mathord, "\u03c5", "\\upsilon", true);
  defineSymbol(math, main, mathord, "\u03d5", "\\phi", true);
  defineSymbol(math, main, mathord, "\u03c7", "\\chi", true);
  defineSymbol(math, main, mathord, "\u03c8", "\\psi", true);
  defineSymbol(math, main, mathord, "\u03c9", "\\omega", true);
  defineSymbol(math, main, mathord, "\u03b5", "\\varepsilon", true);
  defineSymbol(math, main, mathord, "\u03d1", "\\vartheta", true);
  defineSymbol(math, main, mathord, "\u03d6", "\\varpi", true);
  defineSymbol(math, main, mathord, "\u03f1", "\\varrho", true);
  defineSymbol(math, main, mathord, "\u03c2", "\\varsigma", true);
  defineSymbol(math, main, mathord, "\u03c6", "\\varphi", true);
  defineSymbol(math, main, bin, "\u2217", "*", true);
  defineSymbol(math, main, bin, "+", "+");
  defineSymbol(math, main, bin, "\u2212", "-", true);
  defineSymbol(math, main, bin, "\u22c5", "\\cdot", true);
  defineSymbol(math, main, bin, "\u2218", "\\circ");
  defineSymbol(math, main, bin, "\u00f7", "\\div", true);
  defineSymbol(math, main, bin, "\u00b1", "\\pm", true);
  defineSymbol(math, main, bin, "\u00d7", "\\times", true);
  defineSymbol(math, main, bin, "\u2229", "\\cap", true);
  defineSymbol(math, main, bin, "\u222a", "\\cup", true);
  defineSymbol(math, main, bin, "\u2216", "\\setminus");
  defineSymbol(math, main, bin, "\u2227", "\\land");
  defineSymbol(math, main, bin, "\u2228", "\\lor");
  defineSymbol(math, main, bin, "\u2227", "\\wedge", true);
  defineSymbol(math, main, bin, "\u2228", "\\vee", true);
  defineSymbol(math, main, textord, "\u221a", "\\surd");
  defineSymbol(math, main, open, "\u27e8", "\\langle", true);
  defineSymbol(math, main, open, "\u2223", "\\lvert");
  defineSymbol(math, main, open, "\u2225", "\\lVert");
  defineSymbol(math, main, close, "?", "?");
  defineSymbol(math, main, close, "!", "!");
  defineSymbol(math, main, close, "\u27e9", "\\rangle", true);
  defineSymbol(math, main, close, "\u2223", "\\rvert");
  defineSymbol(math, main, close, "\u2225", "\\rVert");
  defineSymbol(math, main, rel, "=", "=");
  defineSymbol(math, main, rel, ":", ":");
  defineSymbol(math, main, rel, "\u2248", "\\approx", true);
  defineSymbol(math, main, rel, "\u2245", "\\cong", true);
  defineSymbol(math, main, rel, "\u2265", "\\ge");
  defineSymbol(math, main, rel, "\u2265", "\\geq", true);
  defineSymbol(math, main, rel, "\u2190", "\\gets");
  defineSymbol(math, main, rel, ">", "\\gt", true);
  defineSymbol(math, main, rel, "\u2208", "\\in", true);
  defineSymbol(math, main, rel, "\ue020", "\\@not");
  defineSymbol(math, main, rel, "\u2282", "\\subset", true);
  defineSymbol(math, main, rel, "\u2283", "\\supset", true);
  defineSymbol(math, main, rel, "\u2286", "\\subseteq", true);
  defineSymbol(math, main, rel, "\u2287", "\\supseteq", true);
  defineSymbol(math, ams, rel, "\u2288", "\\nsubseteq", true);
  defineSymbol(math, ams, rel, "\u2289", "\\nsupseteq", true);
  defineSymbol(math, main, rel, "\u22a8", "\\models");
  defineSymbol(math, main, rel, "\u2190", "\\leftarrow", true);
  defineSymbol(math, main, rel, "\u2264", "\\le");
  defineSymbol(math, main, rel, "\u2264", "\\leq", true);
  defineSymbol(math, main, rel, "<", "\\lt", true);
  defineSymbol(math, main, rel, "\u2192", "\\rightarrow", true);
  defineSymbol(math, main, rel, "\u2192", "\\to");
  defineSymbol(math, ams, rel, "\u2271", "\\ngeq", true);
  defineSymbol(math, ams, rel, "\u2270", "\\nleq", true);
  defineSymbol(math, main, spacing, "\u00a0", "\\ ");
  defineSymbol(math, main, spacing, "\u00a0", "\\space"); // Ref: LaTeX Source 2e: \DeclareRobustCommand{\nobreakspace}{%

  defineSymbol(math, main, spacing, "\u00a0", "\\nobreakspace");
  defineSymbol(text$5, main, spacing, "\u00a0", "\\ ");
  defineSymbol(text$5, main, spacing, "\u00a0", " ");
  defineSymbol(text$5, main, spacing, "\u00a0", "\\space");
  defineSymbol(text$5, main, spacing, "\u00a0", "\\nobreakspace");
  defineSymbol(math, main, spacing, null, "\\nobreak");
  defineSymbol(math, main, spacing, null, "\\allowbreak");
  defineSymbol(math, main, punct, ",", ",");
  defineSymbol(math, main, punct, ";", ";");
  defineSymbol(math, ams, bin, "\u22bc", "\\barwedge", true);
  defineSymbol(math, ams, bin, "\u22bb", "\\veebar", true);
  defineSymbol(math, main, bin, "\u2299", "\\odot", true);
  defineSymbol(math, main, bin, "\u2295", "\\oplus", true);
  defineSymbol(math, main, bin, "\u2297", "\\otimes", true);
  defineSymbol(math, main, textord, "\u2202", "\\partial", true);
  defineSymbol(math, main, bin, "\u2298", "\\oslash", true);
  defineSymbol(math, ams, bin, "\u229a", "\\circledcirc", true);
  defineSymbol(math, ams, bin, "\u22a1", "\\boxdot", true);
  defineSymbol(math, main, bin, "\u25b3", "\\bigtriangleup");
  defineSymbol(math, main, bin, "\u25bd", "\\bigtriangledown");
  defineSymbol(math, main, bin, "\u2020", "\\dagger");
  defineSymbol(math, main, bin, "\u22c4", "\\diamond");
  defineSymbol(math, main, bin, "\u22c6", "\\star");
  defineSymbol(math, main, bin, "\u25c3", "\\triangleleft");
  defineSymbol(math, main, bin, "\u25b9", "\\triangleright");
  defineSymbol(math, main, open, "{", "\\{");
  defineSymbol(text$5, main, textord, "{", "\\{");
  defineSymbol(text$5, main, textord, "{", "\\textbraceleft");
  defineSymbol(math, main, close, "}", "\\}");
  defineSymbol(text$5, main, textord, "}", "\\}");
  defineSymbol(text$5, main, textord, "}", "\\textbraceright");
  defineSymbol(math, main, open, "{", "\\lbrace");
  defineSymbol(math, main, close, "}", "\\rbrace");
  defineSymbol(math, main, open, "[", "\\lbrack", true);
  defineSymbol(text$5, main, textord, "[", "\\lbrack", true);
  defineSymbol(math, main, close, "]", "\\rbrack", true);
  defineSymbol(text$5, main, textord, "]", "\\rbrack", true);
  defineSymbol(math, main, open, "(", "\\lparen", true);
  defineSymbol(math, main, close, ")", "\\rparen", true);
  defineSymbol(text$5, main, textord, "<", "\\textless", true); // in T1 fontenc

  defineSymbol(text$5, main, textord, ">", "\\textgreater", true); // in T1 fontenc

  defineSymbol(math, main, open, "\u230a", "\\lfloor", true);
  defineSymbol(math, main, close, "\u230b", "\\rfloor", true);
  defineSymbol(math, main, open, "\u2308", "\\lceil", true);
  defineSymbol(math, main, close, "\u2309", "\\rceil", true);
  defineSymbol(math, main, textord, "\\", "\\backslash");
  defineSymbol(math, main, textord, "\u2223", "|");
  defineSymbol(math, main, textord, "\u2223", "\\vert");
  defineSymbol(text$5, main, textord, "|", "\\textbar", true); // in T1 fontenc

  defineSymbol(math, main, textord, "\u2225", "\\|");
  defineSymbol(math, main, textord, "\u2225", "\\Vert");
  defineSymbol(text$5, main, textord, "\u2225", "\\textbardbl");
  defineSymbol(text$5, main, textord, "~", "\\textasciitilde");
  defineSymbol(text$5, main, textord, "\\", "\\textbackslash");
  defineSymbol(text$5, main, textord, "^", "\\textasciicircum");
  defineSymbol(math, main, rel, "\u2191", "\\uparrow", true);
  defineSymbol(math, main, rel, "\u21d1", "\\Uparrow", true);
  defineSymbol(math, main, rel, "\u2193", "\\downarrow", true);
  defineSymbol(math, main, rel, "\u21d3", "\\Downarrow", true);
  defineSymbol(math, main, rel, "\u2195", "\\updownarrow", true);
  defineSymbol(math, main, rel, "\u21d5", "\\Updownarrow", true);
  defineSymbol(math, main, op, "\u2210", "\\coprod");
  defineSymbol(math, main, op, "\u22c1", "\\bigvee");
  defineSymbol(math, main, op, "\u22c0", "\\bigwedge");
  defineSymbol(math, main, op, "\u2a04", "\\biguplus");
  defineSymbol(math, main, op, "\u22c2", "\\bigcap");
  defineSymbol(math, main, op, "\u22c3", "\\bigcup");
  defineSymbol(math, main, op, "\u222b", "\\int");
  defineSymbol(math, main, op, "\u222b", "\\intop");
  defineSymbol(math, main, op, "\u222c", "\\iint");
  defineSymbol(math, main, op, "\u222d", "\\iiint");
  defineSymbol(math, main, op, "\u220f", "\\prod");
  defineSymbol(math, main, op, "\u2211", "\\sum");
  defineSymbol(math, main, op, "\u2a02", "\\bigotimes");
  defineSymbol(math, main, op, "\u2a01", "\\bigoplus");
  defineSymbol(math, main, op, "\u2a00", "\\bigodot");
  defineSymbol(math, main, op, "\u222e", "\\oint");
  defineSymbol(math, main, op, "\u222f", "\\oiint");
  defineSymbol(math, main, op, "\u2230", "\\oiiint");
  defineSymbol(math, main, op, "\u2a06", "\\bigsqcup");
  defineSymbol(math, main, op, "\u222b", "\\smallint");
  defineSymbol(text$5, main, inner, "\u2026", "\\textellipsis");
  defineSymbol(math, main, inner, "\u2026", "\\mathellipsis");
  defineSymbol(text$5, main, inner, "\u2026", "\\ldots", true);
  defineSymbol(math, main, inner, "\u2026", "\\ldots", true);
  defineSymbol(math, main, inner, "\u22ef", "\\@cdots", true);
  defineSymbol(math, main, inner, "\u22f1", "\\ddots", true);
  defineSymbol(math, main, textord, "\u22ee", "\\varvdots"); // \vdots is a macro

  defineSymbol(math, main, accent, "\u02ca", "\\acute");
  defineSymbol(math, main, accent, "\u02cb", "\\grave");
  defineSymbol(math, main, accent, "\u00a8", "\\ddot");
  defineSymbol(math, main, accent, "\u007e", "\\tilde");
  defineSymbol(math, main, accent, "\u02c9", "\\bar");
  defineSymbol(math, main, accent, "\u02d8", "\\breve");
  defineSymbol(math, main, accent, "\u02c7", "\\check");
  defineSymbol(math, main, accent, "\u005e", "\\hat");
  defineSymbol(math, main, accent, "\u20d7", "\\vec");
  defineSymbol(math, main, accent, "\u02d9", "\\dot");
  defineSymbol(math, main, accent, "\u02da", "\\mathring"); // \imath and \jmath should be invariant to \mathrm, \mathbf, etc., so use PUA

  defineSymbol(math, main, mathord, "\ue131", "\\@imath");
  defineSymbol(math, main, mathord, "\ue237", "\\@jmath");
  defineSymbol(math, main, textord, "\u0131", "\u0131");
  defineSymbol(math, main, textord, "\u0237", "\u0237");
  defineSymbol(text$5, main, textord, "\u0131", "\\i", true);
  defineSymbol(text$5, main, textord, "\u0237", "\\j", true);
  defineSymbol(text$5, main, textord, "\u00df", "\\ss", true);
  defineSymbol(text$5, main, textord, "\u00e6", "\\ae", true);
  defineSymbol(text$5, main, textord, "\u0153", "\\oe", true);
  defineSymbol(text$5, main, textord, "\u00f8", "\\o", true);
  defineSymbol(text$5, main, textord, "\u00c6", "\\AE", true);
  defineSymbol(text$5, main, textord, "\u0152", "\\OE", true);
  defineSymbol(text$5, main, textord, "\u00d8", "\\O", true);
  defineSymbol(text$5, main, accent, "\u02ca", "\\'"); // acute

  defineSymbol(text$5, main, accent, "\u02cb", "\\`"); // grave

  defineSymbol(text$5, main, accent, "\u02c6", "\\^"); // circumflex

  defineSymbol(text$5, main, accent, "\u02dc", "\\~"); // tilde

  defineSymbol(text$5, main, accent, "\u02c9", "\\="); // macron

  defineSymbol(text$5, main, accent, "\u02d8", "\\u"); // breve

  defineSymbol(text$5, main, accent, "\u02d9", "\\."); // dot above

  defineSymbol(text$5, main, accent, "\u00b8", "\\c"); // cedilla

  defineSymbol(text$5, main, accent, "\u02da", "\\r"); // ring above

  defineSymbol(text$5, main, accent, "\u02c7", "\\v"); // caron

  defineSymbol(text$5, main, accent, "\u00a8", '\\"'); // diaresis

  defineSymbol(text$5, main, accent, "\u02dd", "\\H"); // double acute

  defineSymbol(text$5, main, accent, "\u25ef", "\\textcircled"); // \bigcirc glyph
  // These ligatures are detected and created in Parser.js's `formLigatures`.

  var ligatures = {
    "--": true,
    "---": true,
    "``": true,
    "''": true
  };
  defineSymbol(text$5, main, textord, "\u2013", "--", true);
  defineSymbol(text$5, main, textord, "\u2013", "\\textendash");
  defineSymbol(text$5, main, textord, "\u2014", "---", true);
  defineSymbol(text$5, main, textord, "\u2014", "\\textemdash");
  defineSymbol(text$5, main, textord, "\u2018", "`", true);
  defineSymbol(text$5, main, textord, "\u2018", "\\textquoteleft");
  defineSymbol(text$5, main, textord, "\u2019", "'", true);
  defineSymbol(text$5, main, textord, "\u2019", "\\textquoteright");
  defineSymbol(text$5, main, textord, "\u201c", "``", true);
  defineSymbol(text$5, main, textord, "\u201c", "\\textquotedblleft");
  defineSymbol(text$5, main, textord, "\u201d", "''", true);
  defineSymbol(text$5, main, textord, "\u201d", "\\textquotedblright"); //  \degree from gensymb package

  defineSymbol(math, main, textord, "\u00b0", "\\degree", true);
  defineSymbol(text$5, main, textord, "\u00b0", "\\degree"); // \textdegree from inputenc package

  defineSymbol(text$5, main, textord, "\u00b0", "\\textdegree", true); // TODO: In LaTeX, \pounds can generate a different character in text and math
  // mode, but among our fonts, only Main-Regular defines this character "163".

  defineSymbol(math, main, textord, "\u00a3", "\\pounds");
  defineSymbol(math, main, textord, "\u00a3", "\\mathsterling", true);
  defineSymbol(text$5, main, textord, "\u00a3", "\\pounds");
  defineSymbol(text$5, main, textord, "\u00a3", "\\textsterling", true);
  defineSymbol(math, ams, textord, "\u2720", "\\maltese");
  defineSymbol(text$5, ams, textord, "\u2720", "\\maltese"); // There are lots of symbols which are the same, so we add them in afterwards.
  // All of these are textords in math mode

  var mathTextSymbols = "0123456789/@.\"";

  for (var i = 0; i < mathTextSymbols.length; i++) {
    var ch = mathTextSymbols.charAt(i);
    defineSymbol(math, main, textord, ch, ch);
  } // All of these are textords in text mode


  var textSymbols = "0123456789!@*()-=+\";:?/.,";

  for (var _i = 0; _i < textSymbols.length; _i++) {
    var _ch = textSymbols.charAt(_i);

    defineSymbol(text$5, main, textord, _ch, _ch);
  } // All of these are textords in text mode, and mathords in math mode


  var letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

  for (var _i2 = 0; _i2 < letters.length; _i2++) {
    var _ch2 = letters.charAt(_i2);

    defineSymbol(math, main, mathord, _ch2, _ch2);
    defineSymbol(text$5, main, textord, _ch2, _ch2);
  } // Blackboard bold and script letters in Unicode range


  defineSymbol(math, ams, textord, "C", "\u2102"); // blackboard bold

  defineSymbol(text$5, ams, textord, "C", "\u2102");
  defineSymbol(math, ams, textord, "H", "\u210D");
  defineSymbol(text$5, ams, textord, "H", "\u210D");
  defineSymbol(math, ams, textord, "N", "\u2115");
  defineSymbol(text$5, ams, textord, "N", "\u2115");
  defineSymbol(math, ams, textord, "P", "\u2119");
  defineSymbol(text$5, ams, textord, "P", "\u2119");
  defineSymbol(math, ams, textord, "Q", "\u211A");
  defineSymbol(text$5, ams, textord, "Q", "\u211A");
  defineSymbol(math, ams, textord, "R", "\u211D");
  defineSymbol(text$5, ams, textord, "R", "\u211D");
  defineSymbol(math, ams, textord, "Z", "\u2124");
  defineSymbol(text$5, ams, textord, "Z", "\u2124");
  defineSymbol(math, main, mathord, "h", "\u210E"); // italic h, Planck constant

  defineSymbol(text$5, main, mathord, "h", "\u210E"); // The next loop loads wide (surrogate pair) characters.
  // We support some letters in the Unicode range U+1D400 to U+1D7FF,
  // Mathematical Alphanumeric Symbols.
  // Some editors do not deal well with wide characters. So don't write the
  // string into this file. Instead, create the string from the surrogate pair.

  var wideChar = "";

  for (var _i3 = 0; _i3 < letters.length; _i3++) {
    var _ch3 = letters.charAt(_i3); // The hex numbers in the next line are a surrogate pair.
    // 0xD835 is the high surrogate for all letters in the range we support.
    // 0xDC00 is the low surrogate for bold A.


    wideChar = String.fromCharCode(0xD835, 0xDC00 + _i3); // A-Z a-z bold

    defineSymbol(math, main, mathord, _ch3, wideChar);
    defineSymbol(text$5, main, textord, _ch3, wideChar);
    wideChar = String.fromCharCode(0xD835, 0xDC34 + _i3); // A-Z a-z italic

    defineSymbol(math, main, mathord, _ch3, wideChar);
    defineSymbol(text$5, main, textord, _ch3, wideChar);
    wideChar = String.fromCharCode(0xD835, 0xDC68 + _i3); // A-Z a-z bold italic

    defineSymbol(math, main, mathord, _ch3, wideChar);
    defineSymbol(text$5, main, textord, _ch3, wideChar);
    wideChar = String.fromCharCode(0xD835, 0xDD04 + _i3); // A-Z a-z Fractur

    defineSymbol(math, main, mathord, _ch3, wideChar);
    defineSymbol(text$5, main, textord, _ch3, wideChar);
    wideChar = String.fromCharCode(0xD835, 0xDDA0 + _i3); // A-Z a-z sans-serif

    defineSymbol(math, main, mathord, _ch3, wideChar);
    defineSymbol(text$5, main, textord, _ch3, wideChar);
    wideChar = String.fromCharCode(0xD835, 0xDDD4 + _i3); // A-Z a-z sans bold

    defineSymbol(math, main, mathord, _ch3, wideChar);
    defineSymbol(text$5, main, textord, _ch3, wideChar);
    wideChar = String.fromCharCode(0xD835, 0xDE08 + _i3); // A-Z a-z sans italic

    defineSymbol(math, main, mathord, _ch3, wideChar);
    defineSymbol(text$5, main, textord, _ch3, wideChar);
    wideChar = String.fromCharCode(0xD835, 0xDE70 + _i3); // A-Z a-z monospace

    defineSymbol(math, main, mathord, _ch3, wideChar);
    defineSymbol(text$5, main, textord, _ch3, wideChar);

    if (_i3 < 26) {
      // KaTeX fonts have only capital letters for blackboard bold and script.
      // See exception for k below.
      wideChar = String.fromCharCode(0xD835, 0xDD38 + _i3); // A-Z double struck

      defineSymbol(math, main, mathord, _ch3, wideChar);
      defineSymbol(text$5, main, textord, _ch3, wideChar);
      wideChar = String.fromCharCode(0xD835, 0xDC9C + _i3); // A-Z script

      defineSymbol(math, main, mathord, _ch3, wideChar);
      defineSymbol(text$5, main, textord, _ch3, wideChar);
    } // TODO: Add bold script when it is supported by a KaTeX font.

  } // "k" is the only double struck lower case letter in the KaTeX fonts.


  wideChar = String.fromCharCode(0xD835, 0xDD5C); // k double struck

  defineSymbol(math, main, mathord, "k", wideChar);
  defineSymbol(text$5, main, textord, "k", wideChar); // Next, some wide character numerals

  for (var _i4 = 0; _i4 < 10; _i4++) {
    var _ch4 = _i4.toString();

    wideChar = String.fromCharCode(0xD835, 0xDFCE + _i4); // 0-9 bold

    defineSymbol(math, main, mathord, _ch4, wideChar);
    defineSymbol(text$5, main, textord, _ch4, wideChar);
    wideChar = String.fromCharCode(0xD835, 0xDFE2 + _i4); // 0-9 sans serif

    defineSymbol(math, main, mathord, _ch4, wideChar);
    defineSymbol(text$5, main, textord, _ch4, wideChar);
    wideChar = String.fromCharCode(0xD835, 0xDFEC + _i4); // 0-9 bold sans

    defineSymbol(math, main, mathord, _ch4, wideChar);
    defineSymbol(text$5, main, textord, _ch4, wideChar);
    wideChar = String.fromCharCode(0xD835, 0xDFF6 + _i4); // 0-9 monospace

    defineSymbol(math, main, mathord, _ch4, wideChar);
    defineSymbol(text$5, main, textord, _ch4, wideChar);
  } // We add these Latin-1 letters as symbols for backwards-compatibility,
  // but they are not actually in the font, nor are they supported by the
  // Unicode accent mechanism, so they fall back to Times font and look ugly.
  // TODO(edemaine): Fix this.


  var extraLatin = "\u00d0\u00de\u00fe";

  for (var _i5 = 0; _i5 < extraLatin.length; _i5++) {
    var _ch5 = extraLatin.charAt(_i5);

    defineSymbol(math, main, mathord, _ch5, _ch5);
    defineSymbol(text$5, main, textord, _ch5, _ch5);
  }

  /**
   * This file provides support for Unicode range U+1D400 to U+1D7FF,
   * Mathematical Alphanumeric Symbols.
   *
   * Function wideCharacterFont takes a wide character as input and returns
   * the font information necessary to render it properly.
   */
  /**
   * Data below is from https://www.unicode.org/charts/PDF/U1D400.pdf
   * That document sorts characters into groups by font type, say bold or italic.
   *
   * In the arrays below, each subarray consists three elements:
   *      * The CSS class of that group when in math mode.
   *      * The CSS class of that group when in text mode.
   *      * The font name, so that KaTeX can get font metrics.
   */

  var wideLatinLetterData = [["mathbf", "textbf", "Main-Bold"], // A-Z bold upright
  ["mathbf", "textbf", "Main-Bold"], // a-z bold upright
  ["mathnormal", "textit", "Math-Italic"], // A-Z italic
  ["mathnormal", "textit", "Math-Italic"], // a-z italic
  ["boldsymbol", "boldsymbol", "Main-BoldItalic"], // A-Z bold italic
  ["boldsymbol", "boldsymbol", "Main-BoldItalic"], // a-z bold italic
  // Map fancy A-Z letters to script, not calligraphic.
  // This aligns with unicode-math and math fonts (except Cambria Math).
  ["mathscr", "textscr", "Script-Regular"], // A-Z script
  ["", "", ""], // a-z script.  No font
  ["", "", ""], // A-Z bold script. No font
  ["", "", ""], // a-z bold script. No font
  ["mathfrak", "textfrak", "Fraktur-Regular"], // A-Z Fraktur
  ["mathfrak", "textfrak", "Fraktur-Regular"], // a-z Fraktur
  ["mathbb", "textbb", "AMS-Regular"], // A-Z double-struck
  ["mathbb", "textbb", "AMS-Regular"], // k double-struck
  ["", "", ""], // A-Z bold Fraktur No font metrics
  ["", "", ""], // a-z bold Fraktur.   No font.
  ["mathsf", "textsf", "SansSerif-Regular"], // A-Z sans-serif
  ["mathsf", "textsf", "SansSerif-Regular"], // a-z sans-serif
  ["mathboldsf", "textboldsf", "SansSerif-Bold"], // A-Z bold sans-serif
  ["mathboldsf", "textboldsf", "SansSerif-Bold"], // a-z bold sans-serif
  ["mathitsf", "textitsf", "SansSerif-Italic"], // A-Z italic sans-serif
  ["mathitsf", "textitsf", "SansSerif-Italic"], // a-z italic sans-serif
  ["", "", ""], // A-Z bold italic sans. No font
  ["", "", ""], // a-z bold italic sans. No font
  ["mathtt", "texttt", "Typewriter-Regular"], // A-Z monospace
  ["mathtt", "texttt", "Typewriter-Regular"] // a-z monospace
  ];
  var wideNumeralData = [["mathbf", "textbf", "Main-Bold"], // 0-9 bold
  ["", "", ""], // 0-9 double-struck. No KaTeX font.
  ["mathsf", "textsf", "SansSerif-Regular"], // 0-9 sans-serif
  ["mathboldsf", "textboldsf", "SansSerif-Bold"], // 0-9 bold sans-serif
  ["mathtt", "texttt", "Typewriter-Regular"] // 0-9 monospace
  ];
  var wideCharacterFont = function wideCharacterFont(wideChar, mode) {
    // IE doesn't support codePointAt(). So work with the surrogate pair.
    var H = wideChar.charCodeAt(0); // high surrogate

    var L = wideChar.charCodeAt(1); // low surrogate

    var codePoint = (H - 0xD800) * 0x400 + (L - 0xDC00) + 0x10000;
    var j = mode === "math" ? 0 : 1; // column index for CSS class.

    if (0x1D400 <= codePoint && codePoint < 0x1D6A4) {
      // wideLatinLetterData contains exactly 26 chars on each row.
      // So we can calculate the relevant row. No traverse necessary.
      var i = Math.floor((codePoint - 0x1D400) / 26);
      return [wideLatinLetterData[i][2], wideLatinLetterData[i][j]];
    } else if (0x1D7CE <= codePoint && codePoint <= 0x1D7FF) {
      // Numerals, ten per row.
      var _i = Math.floor((codePoint - 0x1D7CE) / 10);

      return [wideNumeralData[_i][2], wideNumeralData[_i][j]];
    } else if (codePoint === 0x1D6A5 || codePoint === 0x1D6A6) {
      // dotless i or j
      return [wideLatinLetterData[0][2], wideLatinLetterData[0][j]];
    } else if (0x1D6A6 < codePoint && codePoint < 0x1D7CE) {
      // Greek letters. Not supported, yet.
      return ["", ""];
    } else {
      // We don't support any wide characters outside 1D400–1D7FF.
      throw new ParseError("Unsupported character: " + wideChar);
    }
  };

  /* eslint no-console:0 */

  /**
   * Looks up the given symbol in fontMetrics, after applying any symbol
   * replacements defined in symbol.js
   */
  var lookupSymbol = function lookupSymbol(value, // TODO(#963): Use a union type for this.
  fontName, mode) {
    // Replace the value with its replaced value from symbol.js
    if (symbols[mode][value] && symbols[mode][value].replace) {
      value = symbols[mode][value].replace;
    }

    return {
      value: value,
      metrics: getCharacterMetrics(value, fontName, mode)
    };
  };
  /**
   * Makes a symbolNode after translation via the list of symbols in symbols.js.
   * Correctly pulls out metrics for the character, and optionally takes a list of
   * classes to be attached to the node.
   *
   * TODO: make argument order closer to makeSpan
   * TODO: add a separate argument for math class (e.g. `mop`, `mbin`), which
   * should if present come first in `classes`.
   * TODO(#953): Make `options` mandatory and always pass it in.
   */


  var makeSymbol = function makeSymbol(value, fontName, mode, options, classes) {
    var lookup = lookupSymbol(value, fontName, mode);
    var metrics = lookup.metrics;
    value = lookup.value;
    var symbolNode;

    if (metrics) {
      var italic = metrics.italic;

      if (mode === "text" || options && options.font === "mathit") {
        italic = 0;
      }

      symbolNode = new SymbolNode(value, metrics.height, metrics.depth, italic, metrics.skew, metrics.width, classes);
    } else {
      // TODO(emily): Figure out a good way to only print this in development
      typeof console !== "undefined" && console.warn("No character metrics " + ("for '" + value + "' in style '" + fontName + "' and mode '" + mode + "'"));
      symbolNode = new SymbolNode(value, 0, 0, 0, 0, 0, classes);
    }

    if (options) {
      symbolNode.maxFontSize = options.sizeMultiplier;

      if (options.style.isTight()) {
        symbolNode.classes.push("mtight");
      }

      var color = options.getColor();

      if (color) {
        symbolNode.style.color = color;
      }
    }

    return symbolNode;
  };
  /**
   * Makes a symbol in Main-Regular or AMS-Regular.
   * Used for rel, bin, open, close, inner, and punct.
   */


  var mathsym = function mathsym(value, mode, options, classes) {
    if (classes === void 0) {
      classes = [];
    }

    // Decide what font to render the symbol in by its entry in the symbols
    // table.
    // Have a special case for when the value = \ because the \ is used as a
    // textord in unsupported command errors but cannot be parsed as a regular
    // text ordinal and is therefore not present as a symbol in the symbols
    // table for text, as well as a special case for boldsymbol because it
    // can be used for bold + and -
    if (options.font === "boldsymbol" && lookupSymbol(value, "Main-Bold", mode).metrics) {
      return makeSymbol(value, "Main-Bold", mode, options, classes.concat(["mathbf"]));
    } else if (value === "\\" || symbols[mode][value].font === "main") {
      return makeSymbol(value, "Main-Regular", mode, options, classes);
    } else {
      return makeSymbol(value, "AMS-Regular", mode, options, classes.concat(["amsrm"]));
    }
  };
  /**
   * Determines which of the two font names (Main-Bold and Math-BoldItalic) and
   * corresponding style tags (mathbf or boldsymbol) to use for font "boldsymbol",
   * depending on the symbol.  Use this function instead of fontMap for font
   * "boldsymbol".
   */


  var boldsymbol = function boldsymbol(value, mode, options, classes, type) {
    if (type !== "textord" && lookupSymbol(value, "Math-BoldItalic", mode).metrics) {
      return {
        fontName: "Math-BoldItalic",
        fontClass: "boldsymbol"
      };
    } else {
      // Some glyphs do not exist in Math-BoldItalic so we need to use
      // Main-Bold instead.
      return {
        fontName: "Main-Bold",
        fontClass: "mathbf"
      };
    }
  };
  /**
   * Makes either a mathord or textord in the correct font and color.
   */


  var makeOrd = function makeOrd(group, options, type) {
    var mode = group.mode;
    var text = group.text;
    var classes = ["mord"]; // Math mode or Old font (i.e. \rm)

    var isFont = mode === "math" || mode === "text" && options.font;
    var fontOrFamily = isFont ? options.font : options.fontFamily;

    if (text.charCodeAt(0) === 0xD835) {
      // surrogate pairs get special treatment
      var [wideFontName, wideFontClass] = wideCharacterFont(text, mode);
      return makeSymbol(text, wideFontName, mode, options, classes.concat(wideFontClass));
    } else if (fontOrFamily) {
      var fontName;
      var fontClasses;

      if (fontOrFamily === "boldsymbol") {
        var fontData = boldsymbol(text, mode, options, classes, type);
        fontName = fontData.fontName;
        fontClasses = [fontData.fontClass];
      } else if (isFont) {
        fontName = fontMap[fontOrFamily].fontName;
        fontClasses = [fontOrFamily];
      } else {
        fontName = retrieveTextFontName(fontOrFamily, options.fontWeight, options.fontShape);
        fontClasses = [fontOrFamily, options.fontWeight, options.fontShape];
      }

      if (lookupSymbol(text, fontName, mode).metrics) {
        return makeSymbol(text, fontName, mode, options, classes.concat(fontClasses));
      } else if (ligatures.hasOwnProperty(text) && fontName.substr(0, 10) === "Typewriter") {
        // Deconstruct ligatures in monospace fonts (\texttt, \tt).
        var parts = [];

        for (var i = 0; i < text.length; i++) {
          parts.push(makeSymbol(text[i], fontName, mode, options, classes.concat(fontClasses)));
        }

        return makeFragment(parts);
      }
    } // Makes a symbol in the default font for mathords and textords.


    if (type === "mathord") {
      return makeSymbol(text, "Math-Italic", mode, options, classes.concat(["mathnormal"]));
    } else if (type === "textord") {
      var font = symbols[mode][text] && symbols[mode][text].font;

      if (font === "ams") {
        var _fontName = retrieveTextFontName("amsrm", options.fontWeight, options.fontShape);

        return makeSymbol(text, _fontName, mode, options, classes.concat("amsrm", options.fontWeight, options.fontShape));
      } else if (font === "main" || !font) {
        var _fontName2 = retrieveTextFontName("textrm", options.fontWeight, options.fontShape);

        return makeSymbol(text, _fontName2, mode, options, classes.concat(options.fontWeight, options.fontShape));
      } else {
        // fonts added by plugins
        var _fontName3 = retrieveTextFontName(font, options.fontWeight, options.fontShape); // We add font name as a css class


        return makeSymbol(text, _fontName3, mode, options, classes.concat(_fontName3, options.fontWeight, options.fontShape));
      }
    } else {
      throw new Error("unexpected type: " + type + " in makeOrd");
    }
  };
  /**
   * Returns true if subsequent symbolNodes have the same classes, skew, maxFont,
   * and styles.
   */


  var canCombine = (prev, next) => {
    if (createClass(prev.classes) !== createClass(next.classes) || prev.skew !== next.skew || prev.maxFontSize !== next.maxFontSize) {
      return false;
    } // If prev and next both are just "mbin"s or "mord"s we don't combine them
    // so that the proper spacing can be preserved.


    if (prev.classes.length === 1) {
      var cls = prev.classes[0];

      if (cls === "mbin" || cls === "mord") {
        return false;
      }
    }

    for (var style in prev.style) {
      if (prev.style.hasOwnProperty(style) && prev.style[style] !== next.style[style]) {
        return false;
      }
    }

    for (var _style in next.style) {
      if (next.style.hasOwnProperty(_style) && prev.style[_style] !== next.style[_style]) {
        return false;
      }
    }

    return true;
  };
  /**
   * Combine consecutive domTree.symbolNodes into a single symbolNode.
   * Note: this function mutates the argument.
   */


  var tryCombineChars = chars => {
    for (var i = 0; i < chars.length - 1; i++) {
      var prev = chars[i];
      var next = chars[i + 1];

      if (prev instanceof SymbolNode && next instanceof SymbolNode && canCombine(prev, next)) {
        prev.text += next.text;
        prev.height = Math.max(prev.height, next.height);
        prev.depth = Math.max(prev.depth, next.depth); // Use the last character's italic correction since we use
        // it to add padding to the right of the span created from
        // the combined characters.

        prev.italic = next.italic;
        chars.splice(i + 1, 1);
        i--;
      }
    }

    return chars;
  };
  /**
   * Calculate the height, depth, and maxFontSize of an element based on its
   * children.
   */


  var sizeElementFromChildren = function sizeElementFromChildren(elem) {
    var height = 0;
    var depth = 0;
    var maxFontSize = 0;

    for (var i = 0; i < elem.children.length; i++) {
      var child = elem.children[i];

      if (child.height > height) {
        height = child.height;
      }

      if (child.depth > depth) {
        depth = child.depth;
      }

      if (child.maxFontSize > maxFontSize) {
        maxFontSize = child.maxFontSize;
      }
    }

    elem.height = height;
    elem.depth = depth;
    elem.maxFontSize = maxFontSize;
  };
  /**
   * Makes a span with the given list of classes, list of children, and options.
   *
   * TODO(#953): Ensure that `options` is always provided (currently some call
   * sites don't pass it) and make the type below mandatory.
   * TODO: add a separate argument for math class (e.g. `mop`, `mbin`), which
   * should if present come first in `classes`.
   */


  var makeSpan$2 = function makeSpan(classes, children, options, style) {
    var span = new Span(classes, children, options, style);
    sizeElementFromChildren(span);
    return span;
  }; // SVG one is simpler -- doesn't require height, depth, max-font setting.
  // This is also a separate method for typesafety.


  var makeSvgSpan = (classes, children, options, style) => new Span(classes, children, options, style);

  var makeLineSpan = function makeLineSpan(className, options, thickness) {
    var line = makeSpan$2([className], [], options);
    line.height = Math.max(thickness || options.fontMetrics().defaultRuleThickness, options.minRuleThickness);
    line.style.borderBottomWidth = makeEm(line.height);
    line.maxFontSize = 1.0;
    return line;
  };
  /**
   * Makes an anchor with the given href, list of classes, list of children,
   * and options.
   */


  var makeAnchor = function makeAnchor(href, classes, children, options) {
    var anchor = new Anchor(href, classes, children, options);
    sizeElementFromChildren(anchor);
    return anchor;
  };
  /**
   * Makes a document fragment with the given list of children.
   */


  var makeFragment = function makeFragment(children) {
    var fragment = new DocumentFragment(children);
    sizeElementFromChildren(fragment);
    return fragment;
  };
  /**
   * Wraps group in a span if it's a document fragment, allowing to apply classes
   * and styles
   */


  var wrapFragment = function wrapFragment(group, options) {
    if (group instanceof DocumentFragment) {
      return makeSpan$2([], [group], options);
    }

    return group;
  }; // These are exact object types to catch typos in the names of the optional fields.


  // Computes the updated `children` list and the overall depth.
  //
  // This helper function for makeVList makes it easier to enforce type safety by
  // allowing early exits (returns) in the logic.
  var getVListChildrenAndDepth = function getVListChildrenAndDepth(params) {
    if (params.positionType === "individualShift") {
      var oldChildren = params.children;
      var children = [oldChildren[0]]; // Add in kerns to the list of params.children to get each element to be
      // shifted to the correct specified shift

      var _depth = -oldChildren[0].shift - oldChildren[0].elem.depth;

      var currPos = _depth;

      for (var i = 1; i < oldChildren.length; i++) {
        var diff = -oldChildren[i].shift - currPos - oldChildren[i].elem.depth;
        var size = diff - (oldChildren[i - 1].elem.height + oldChildren[i - 1].elem.depth);
        currPos = currPos + diff;
        children.push({
          type: "kern",
          size
        });
        children.push(oldChildren[i]);
      }

      return {
        children,
        depth: _depth
      };
    }

    var depth;

    if (params.positionType === "top") {
      // We always start at the bottom, so calculate the bottom by adding up
      // all the sizes
      var bottom = params.positionData;

      for (var _i = 0; _i < params.children.length; _i++) {
        var child = params.children[_i];
        bottom -= child.type === "kern" ? child.size : child.elem.height + child.elem.depth;
      }

      depth = bottom;
    } else if (params.positionType === "bottom") {
      depth = -params.positionData;
    } else {
      var firstChild = params.children[0];

      if (firstChild.type !== "elem") {
        throw new Error('First child must have type "elem".');
      }

      if (params.positionType === "shift") {
        depth = -firstChild.elem.depth - params.positionData;
      } else if (params.positionType === "firstBaseline") {
        depth = -firstChild.elem.depth;
      } else {
        throw new Error("Invalid positionType " + params.positionType + ".");
      }
    }

    return {
      children: params.children,
      depth
    };
  };
  /**
   * Makes a vertical list by stacking elements and kerns on top of each other.
   * Allows for many different ways of specifying the positioning method.
   *
   * See VListParam documentation above.
   */


  var makeVList = function makeVList(params, options) {
    var {
      children,
      depth
    } = getVListChildrenAndDepth(params); // Create a strut that is taller than any list item. The strut is added to
    // each item, where it will determine the item's baseline. Since it has
    // `overflow:hidden`, the strut's top edge will sit on the item's line box's
    // top edge and the strut's bottom edge will sit on the item's baseline,
    // with no additional line-height spacing. This allows the item baseline to
    // be positioned precisely without worrying about font ascent and
    // line-height.

    var pstrutSize = 0;

    for (var i = 0; i < children.length; i++) {
      var child = children[i];

      if (child.type === "elem") {
        var elem = child.elem;
        pstrutSize = Math.max(pstrutSize, elem.maxFontSize, elem.height);
      }
    }

    pstrutSize += 2;
    var pstrut = makeSpan$2(["pstrut"], []);
    pstrut.style.height = makeEm(pstrutSize); // Create a new list of actual children at the correct offsets

    var realChildren = [];
    var minPos = depth;
    var maxPos = depth;
    var currPos = depth;

    for (var _i2 = 0; _i2 < children.length; _i2++) {
      var _child = children[_i2];

      if (_child.type === "kern") {
        currPos += _child.size;
      } else {
        var _elem = _child.elem;
        var classes = _child.wrapperClasses || [];
        var style = _child.wrapperStyle || {};
        var childWrap = makeSpan$2(classes, [pstrut, _elem], undefined, style);
        childWrap.style.top = makeEm(-pstrutSize - currPos - _elem.depth);

        if (_child.marginLeft) {
          childWrap.style.marginLeft = _child.marginLeft;
        }

        if (_child.marginRight) {
          childWrap.style.marginRight = _child.marginRight;
        }

        realChildren.push(childWrap);
        currPos += _elem.height + _elem.depth;
      }

      minPos = Math.min(minPos, currPos);
      maxPos = Math.max(maxPos, currPos);
    } // The vlist contents go in a table-cell with `vertical-align:bottom`.
    // This cell's bottom edge will determine the containing table's baseline
    // without overly expanding the containing line-box.


    var vlist = makeSpan$2(["vlist"], realChildren);
    vlist.style.height = makeEm(maxPos); // A second row is used if necessary to represent the vlist's depth.

    var rows;

    if (minPos < 0) {
      // We will define depth in an empty span with display: table-cell.
      // It should render with the height that we define. But Chrome, in
      // contenteditable mode only, treats that span as if it contains some
      // text content. And that min-height over-rides our desired height.
      // So we put another empty span inside the depth strut span.
      var emptySpan = makeSpan$2([], []);
      var depthStrut = makeSpan$2(["vlist"], [emptySpan]);
      depthStrut.style.height = makeEm(-minPos); // Safari wants the first row to have inline content; otherwise it
      // puts the bottom of the *second* row on the baseline.

      var topStrut = makeSpan$2(["vlist-s"], [new SymbolNode("\u200b")]);
      rows = [makeSpan$2(["vlist-r"], [vlist, topStrut]), makeSpan$2(["vlist-r"], [depthStrut])];
    } else {
      rows = [makeSpan$2(["vlist-r"], [vlist])];
    }

    var vtable = makeSpan$2(["vlist-t"], rows);

    if (rows.length === 2) {
      vtable.classes.push("vlist-t2");
    }

    vtable.height = maxPos;
    vtable.depth = -minPos;
    return vtable;
  }; // Glue is a concept from TeX which is a flexible space between elements in
  // either a vertical or horizontal list. In KaTeX, at least for now, it's
  // static space between elements in a horizontal layout.


  var makeGlue = (measurement, options) => {
    // Make an empty span for the space
    var rule = makeSpan$2(["mspace"], [], options);
    var size = calculateSize(measurement, options);
    rule.style.marginRight = makeEm(size);
    return rule;
  }; // Takes font options, and returns the appropriate fontLookup name


  var retrieveTextFontName = function retrieveTextFontName(fontFamily, fontWeight, fontShape) {
    var baseFontName = "";

    switch (fontFamily) {
      case "amsrm":
        baseFontName = "AMS";
        break;

      case "textrm":
        baseFontName = "Main";
        break;

      case "textsf":
        baseFontName = "SansSerif";
        break;

      case "texttt":
        baseFontName = "Typewriter";
        break;

      default:
        baseFontName = fontFamily;
      // use fonts added by a plugin
    }

    var fontStylesName;

    if (fontWeight === "textbf" && fontShape === "textit") {
      fontStylesName = "BoldItalic";
    } else if (fontWeight === "textbf") {
      fontStylesName = "Bold";
    } else if (fontWeight === "textit") {
      fontStylesName = "Italic";
    } else {
      fontStylesName = "Regular";
    }

    return baseFontName + "-" + fontStylesName;
  };
  /**
   * Maps TeX font commands to objects containing:
   * - variant: string used for "mathvariant" attribute in buildMathML.js
   * - fontName: the "style" parameter to fontMetrics.getCharacterMetrics
   */
  // A map between tex font commands an MathML mathvariant attribute values


  var fontMap = {
    // styles
    "mathbf": {
      variant: "bold",
      fontName: "Main-Bold"
    },
    "mathrm": {
      variant: "normal",
      fontName: "Main-Regular"
    },
    "textit": {
      variant: "italic",
      fontName: "Main-Italic"
    },
    "mathit": {
      variant: "italic",
      fontName: "Main-Italic"
    },
    "mathnormal": {
      variant: "italic",
      fontName: "Math-Italic"
    },
    // "boldsymbol" is missing because they require the use of multiple fonts:
    // Math-BoldItalic and Main-Bold.  This is handled by a special case in
    // makeOrd which ends up calling boldsymbol.
    // families
    "mathbb": {
      variant: "double-struck",
      fontName: "AMS-Regular"
    },
    "mathcal": {
      variant: "script",
      fontName: "Caligraphic-Regular"
    },
    "mathfrak": {
      variant: "fraktur",
      fontName: "Fraktur-Regular"
    },
    "mathscr": {
      variant: "script",
      fontName: "Script-Regular"
    },
    "mathsf": {
      variant: "sans-serif",
      fontName: "SansSerif-Regular"
    },
    "mathtt": {
      variant: "monospace",
      fontName: "Typewriter-Regular"
    }
  };
  var svgData = {
    //   path, width, height
    vec: ["vec", 0.471, 0.714],
    // values from the font glyph
    oiintSize1: ["oiintSize1", 0.957, 0.499],
    // oval to overlay the integrand
    oiintSize2: ["oiintSize2", 1.472, 0.659],
    oiiintSize1: ["oiiintSize1", 1.304, 0.499],
    oiiintSize2: ["oiiintSize2", 1.98, 0.659]
  };

  var staticSvg = function staticSvg(value, options) {
    // Create a span with inline SVG for the element.
    var [pathName, width, height] = svgData[value];
    var path = new PathNode(pathName);
    var svgNode = new SvgNode([path], {
      "width": makeEm(width),
      "height": makeEm(height),
      // Override CSS rule `.katex svg { width: 100% }`
      "style": "width:" + makeEm(width),
      "viewBox": "0 0 " + 1000 * width + " " + 1000 * height,
      "preserveAspectRatio": "xMinYMin"
    });
    var span = makeSvgSpan(["overlay"], [svgNode], options);
    span.height = height;
    span.style.height = makeEm(height);
    span.style.width = makeEm(width);
    return span;
  };

  var buildCommon = {
    fontMap,
    makeSymbol,
    mathsym,
    makeSpan: makeSpan$2,
    makeSvgSpan,
    makeLineSpan,
    makeAnchor,
    makeFragment,
    wrapFragment,
    makeVList,
    makeOrd,
    makeGlue,
    staticSvg,
    svgData,
    tryCombineChars
  };

  /**
   * Describes spaces between different classes of atoms.
   */
  var thinspace = {
    number: 3,
    unit: "mu"
  };
  var mediumspace = {
    number: 4,
    unit: "mu"
  };
  var thickspace = {
    number: 5,
    unit: "mu"
  }; // Making the type below exact with all optional fields doesn't work due to
  // - https://github.com/facebook/flow/issues/4582
  // - https://github.com/facebook/flow/issues/5688
  // However, since *all* fields are optional, $Shape<> works as suggested in 5688
  // above.

  // Spacing relationships for display and text styles
  var spacings = {
    mord: {
      mop: thinspace,
      mbin: mediumspace,
      mrel: thickspace,
      minner: thinspace
    },
    mop: {
      mord: thinspace,
      mop: thinspace,
      mrel: thickspace,
      minner: thinspace
    },
    mbin: {
      mord: mediumspace,
      mop: mediumspace,
      mopen: mediumspace,
      minner: mediumspace
    },
    mrel: {
      mord: thickspace,
      mop: thickspace,
      mopen: thickspace,
      minner: thickspace
    },
    mopen: {},
    mclose: {
      mop: thinspace,
      mbin: mediumspace,
      mrel: thickspace,
      minner: thinspace
    },
    mpunct: {
      mord: thinspace,
      mop: thinspace,
      mrel: thickspace,
      mopen: thinspace,
      mclose: thinspace,
      mpunct: thinspace,
      minner: thinspace
    },
    minner: {
      mord: thinspace,
      mop: thinspace,
      mbin: mediumspace,
      mrel: thickspace,
      mopen: thinspace,
      mpunct: thinspace,
      minner: thinspace
    }
  }; // Spacing relationships for script and scriptscript styles

  var tightSpacings = {
    mord: {
      mop: thinspace
    },
    mop: {
      mord: thinspace,
      mop: thinspace
    },
    mbin: {},
    mrel: {},
    mopen: {},
    mclose: {
      mop: thinspace
    },
    mpunct: {},
    minner: {
      mop: thinspace
    }
  };

  /** Context provided to function handlers for error messages. */
  // Note: reverse the order of the return type union will cause a flow error.
  // See https://github.com/facebook/flow/issues/3663.
  // More general version of `HtmlBuilder` for nodes (e.g. \sum, accent types)
  // whose presence impacts super/subscripting. In this case, ParseNode<"supsub">
  // delegates its HTML building to the HtmlBuilder corresponding to these nodes.

  /**
   * Final function spec for use at parse time.
   * This is almost identical to `FunctionPropSpec`, except it
   * 1. includes the function handler, and
   * 2. requires all arguments except argTypes.
   * It is generated by `defineFunction()` below.
   */

  /**
   * All registered functions.
   * `functions.js` just exports this same dictionary again and makes it public.
   * `Parser.js` requires this dictionary.
   */
  var _functions = {};
  /**
   * All HTML builders. Should be only used in the `define*` and the `build*ML`
   * functions.
   */

  var _htmlGroupBuilders = {};
  /**
   * All MathML builders. Should be only used in the `define*` and the `build*ML`
   * functions.
   */

  var _mathmlGroupBuilders = {};
  function defineFunction(_ref) {
    var {
      type,
      names,
      props,
      handler,
      htmlBuilder,
      mathmlBuilder
    } = _ref;
    // Set default values of functions
    var data = {
      type,
      numArgs: props.numArgs,
      argTypes: props.argTypes,
      allowedInArgument: !!props.allowedInArgument,
      allowedInText: !!props.allowedInText,
      allowedInMath: props.allowedInMath === undefined ? true : props.allowedInMath,
      numOptionalArgs: props.numOptionalArgs || 0,
      infix: !!props.infix,
      primitive: !!props.primitive,
      handler: handler
    };

    for (var i = 0; i < names.length; ++i) {
      _functions[names[i]] = data;
    }

    if (type) {
      if (htmlBuilder) {
        _htmlGroupBuilders[type] = htmlBuilder;
      }

      if (mathmlBuilder) {
        _mathmlGroupBuilders[type] = mathmlBuilder;
      }
    }
  }
  /**
   * Use this to register only the HTML and MathML builders for a function (e.g.
   * if the function's ParseNode is generated in Parser.js rather than via a
   * stand-alone handler provided to `defineFunction`).
   */

  function defineFunctionBuilders(_ref2) {
    var {
      type,
      htmlBuilder,
      mathmlBuilder
    } = _ref2;
    defineFunction({
      type,
      names: [],
      props: {
        numArgs: 0
      },

      handler() {
        throw new Error('Should never be called.');
      },

      htmlBuilder,
      mathmlBuilder
    });
  }
  var normalizeArgument = function normalizeArgument(arg) {
    return arg.type === "ordgroup" && arg.body.length === 1 ? arg.body[0] : arg;
  }; // Since the corresponding buildHTML/buildMathML function expects a
  // list of elements, we normalize for different kinds of arguments

  var ordargument = function ordargument(arg) {
    return arg.type === "ordgroup" ? arg.body : [arg];
  };

  /**
   * This file does the main work of building a domTree structure from a parse
   * tree. The entry point is the `buildHTML` function, which takes a parse tree.
   * Then, the buildExpression, buildGroup, and various groupBuilders functions
   * are called, to produce a final HTML tree.
   */
  var makeSpan$1 = buildCommon.makeSpan; // Binary atoms (first class `mbin`) change into ordinary atoms (`mord`)
  // depending on their surroundings. See TeXbook pg. 442-446, Rules 5 and 6,
  // and the text before Rule 19.

  var binLeftCanceller = ["leftmost", "mbin", "mopen", "mrel", "mop", "mpunct"];
  var binRightCanceller = ["rightmost", "mrel", "mclose", "mpunct"];
  var styleMap$1 = {
    "display": Style$1.DISPLAY,
    "text": Style$1.TEXT,
    "script": Style$1.SCRIPT,
    "scriptscript": Style$1.SCRIPTSCRIPT
  };
  var DomEnum = {
    mord: "mord",
    mop: "mop",
    mbin: "mbin",
    mrel: "mrel",
    mopen: "mopen",
    mclose: "mclose",
    mpunct: "mpunct",
    minner: "minner"
  };

  /**
   * Take a list of nodes, build them in order, and return a list of the built
   * nodes. documentFragments are flattened into their contents, so the
   * returned list contains no fragments. `isRealGroup` is true if `expression`
   * is a real group (no atoms will be added on either side), as opposed to
   * a partial group (e.g. one created by \color). `surrounding` is an array
   * consisting type of nodes that will be added to the left and right.
   */
  var buildExpression$1 = function buildExpression(expression, options, isRealGroup, surrounding) {
    if (surrounding === void 0) {
      surrounding = [null, null];
    }

    // Parse expressions into `groups`.
    var groups = [];

    for (var i = 0; i < expression.length; i++) {
      var output = buildGroup$1(expression[i], options);

      if (output instanceof DocumentFragment) {
        var children = output.children;
        groups.push(...children);
      } else {
        groups.push(output);
      }
    } // Combine consecutive domTree.symbolNodes into a single symbolNode.


    buildCommon.tryCombineChars(groups); // If `expression` is a partial group, let the parent handle spacings
    // to avoid processing groups multiple times.

    if (!isRealGroup) {
      return groups;
    }

    var glueOptions = options;

    if (expression.length === 1) {
      var node = expression[0];

      if (node.type === "sizing") {
        glueOptions = options.havingSize(node.size);
      } else if (node.type === "styling") {
        glueOptions = options.havingStyle(styleMap$1[node.style]);
      }
    } // Dummy spans for determining spacings between surrounding atoms.
    // If `expression` has no atoms on the left or right, class "leftmost"
    // or "rightmost", respectively, is used to indicate it.


    var dummyPrev = makeSpan$1([surrounding[0] || "leftmost"], [], options);
    var dummyNext = makeSpan$1([surrounding[1] || "rightmost"], [], options); // TODO: These code assumes that a node's math class is the first element
    // of its `classes` array. A later cleanup should ensure this, for
    // instance by changing the signature of `makeSpan`.
    // Before determining what spaces to insert, perform bin cancellation.
    // Binary operators change to ordinary symbols in some contexts.

    var isRoot = isRealGroup === "root";
    traverseNonSpaceNodes(groups, (node, prev) => {
      var prevType = prev.classes[0];
      var type = node.classes[0];

      if (prevType === "mbin" && utils$1.contains(binRightCanceller, type)) {
        prev.classes[0] = "mord";
      } else if (type === "mbin" && utils$1.contains(binLeftCanceller, prevType)) {
        node.classes[0] = "mord";
      }
    }, {
      node: dummyPrev
    }, dummyNext, isRoot);
    traverseNonSpaceNodes(groups, (node, prev) => {
      var prevType = getTypeOfDomTree(prev);
      var type = getTypeOfDomTree(node); // 'mtight' indicates that the node is script or scriptscript style.

      var space = prevType && type ? node.hasClass("mtight") ? tightSpacings[prevType][type] : spacings[prevType][type] : null;

      if (space) {
        // Insert glue (spacing) after the `prev`.
        return buildCommon.makeGlue(space, glueOptions);
      }
    }, {
      node: dummyPrev
    }, dummyNext, isRoot);
    return groups;
  }; // Depth-first traverse non-space `nodes`, calling `callback` with the current and
  // previous node as arguments, optionally returning a node to insert after the
  // previous node. `prev` is an object with the previous node and `insertAfter`
  // function to insert after it. `next` is a node that will be added to the right.
  // Used for bin cancellation and inserting spacings.

  var traverseNonSpaceNodes = function traverseNonSpaceNodes(nodes, callback, prev, next, isRoot) {
    if (next) {
      // temporarily append the right node, if exists
      nodes.push(next);
    }

    var i = 0;

    for (; i < nodes.length; i++) {
      var node = nodes[i];
      var partialGroup = checkPartialGroup(node);

      if (partialGroup) {
        // Recursive DFS
        // $FlowFixMe: make nodes a $ReadOnlyArray by returning a new array
        traverseNonSpaceNodes(partialGroup.children, callback, prev, null, isRoot);
        continue;
      } // Ignore explicit spaces (e.g., \;, \,) when determining what implicit
      // spacing should go between atoms of different classes


      var nonspace = !node.hasClass("mspace");

      if (nonspace) {
        var result = callback(node, prev.node);

        if (result) {
          if (prev.insertAfter) {
            prev.insertAfter(result);
          } else {
            // insert at front
            nodes.unshift(result);
            i++;
          }
        }
      }

      if (nonspace) {
        prev.node = node;
      } else if (isRoot && node.hasClass("newline")) {
        prev.node = makeSpan$1(["leftmost"]); // treat like beginning of line
      }

      prev.insertAfter = (index => n => {
        nodes.splice(index + 1, 0, n);
        i++;
      })(i);
    }

    if (next) {
      nodes.pop();
    }
  }; // Check if given node is a partial group, i.e., does not affect spacing around.


  var checkPartialGroup = function checkPartialGroup(node) {
    if (node instanceof DocumentFragment || node instanceof Anchor || node instanceof Span && node.hasClass("enclosing")) {
      return node;
    }

    return null;
  }; // Return the outermost node of a domTree.


  var getOutermostNode = function getOutermostNode(node, side) {
    var partialGroup = checkPartialGroup(node);

    if (partialGroup) {
      var children = partialGroup.children;

      if (children.length) {
        if (side === "right") {
          return getOutermostNode(children[children.length - 1], "right");
        } else if (side === "left") {
          return getOutermostNode(children[0], "left");
        }
      }
    }

    return node;
  }; // Return math atom class (mclass) of a domTree.
  // If `side` is given, it will get the type of the outermost node at given side.


  var getTypeOfDomTree = function getTypeOfDomTree(node, side) {
    if (!node) {
      return null;
    }

    if (side) {
      node = getOutermostNode(node, side);
    } // This makes a lot of assumptions as to where the type of atom
    // appears.  We should do a better job of enforcing this.


    return DomEnum[node.classes[0]] || null;
  };
  var makeNullDelimiter = function makeNullDelimiter(options, classes) {
    var moreClasses = ["nulldelimiter"].concat(options.baseSizingClasses());
    return makeSpan$1(classes.concat(moreClasses));
  };
  /**
   * buildGroup is the function that takes a group and calls the correct groupType
   * function for it. It also handles the interaction of size and style changes
   * between parents and children.
   */

  var buildGroup$1 = function buildGroup(group, options, baseOptions) {
    if (!group) {
      return makeSpan$1();
    }

    if (_htmlGroupBuilders[group.type]) {
      // Call the groupBuilders function
      // $FlowFixMe
      var groupNode = _htmlGroupBuilders[group.type](group, options); // If the size changed between the parent and the current group, account
      // for that size difference.

      if (baseOptions && options.size !== baseOptions.size) {
        groupNode = makeSpan$1(options.sizingClasses(baseOptions), [groupNode], options);
        var multiplier = options.sizeMultiplier / baseOptions.sizeMultiplier;
        groupNode.height *= multiplier;
        groupNode.depth *= multiplier;
      }

      return groupNode;
    } else {
      throw new ParseError("Got group of unknown type: '" + group.type + "'");
    }
  };
  /**
   * Combine an array of HTML DOM nodes (e.g., the output of `buildExpression`)
   * into an unbreakable HTML node of class .base, with proper struts to
   * guarantee correct vertical extent.  `buildHTML` calls this repeatedly to
   * make up the entire expression as a sequence of unbreakable units.
   */

  function buildHTMLUnbreakable(children, options) {
    // Compute height and depth of this chunk.
    var body = makeSpan$1(["base"], children, options); // Add strut, which ensures that the top of the HTML element falls at
    // the height of the expression, and the bottom of the HTML element
    // falls at the depth of the expression.

    var strut = makeSpan$1(["strut"]);
    strut.style.height = makeEm(body.height + body.depth);

    if (body.depth) {
      strut.style.verticalAlign = makeEm(-body.depth);
    }

    body.children.unshift(strut);
    return body;
  }
  /**
   * Take an entire parse tree, and build it into an appropriate set of HTML
   * nodes.
   */


  function buildHTML(tree, options) {
    // Strip off outer tag wrapper for processing below.
    var tag = null;

    if (tree.length === 1 && tree[0].type === "tag") {
      tag = tree[0].tag;
      tree = tree[0].body;
    } // Build the expression contained in the tree


    var expression = buildExpression$1(tree, options, "root");
    var eqnNum;

    if (expression.length === 2 && expression[1].hasClass("tag")) {
      // An environment with automatic equation numbers, e.g. {gather}.
      eqnNum = expression.pop();
    }

    var children = []; // Create one base node for each chunk between potential line breaks.
    // The TeXBook [p.173] says "A formula will be broken only after a
    // relation symbol like $=$ or $<$ or $\rightarrow$, or after a binary
    // operation symbol like $+$ or $-$ or $\times$, where the relation or
    // binary operation is on the ``outer level'' of the formula (i.e., not
    // enclosed in {...} and not part of an \over construction)."

    var parts = [];

    for (var i = 0; i < expression.length; i++) {
      parts.push(expression[i]);

      if (expression[i].hasClass("mbin") || expression[i].hasClass("mrel") || expression[i].hasClass("allowbreak")) {
        // Put any post-operator glue on same line as operator.
        // Watch for \nobreak along the way, and stop at \newline.
        var nobreak = false;

        while (i < expression.length - 1 && expression[i + 1].hasClass("mspace") && !expression[i + 1].hasClass("newline")) {
          i++;
          parts.push(expression[i]);

          if (expression[i].hasClass("nobreak")) {
            nobreak = true;
          }
        } // Don't allow break if \nobreak among the post-operator glue.


        if (!nobreak) {
          children.push(buildHTMLUnbreakable(parts, options));
          parts = [];
        }
      } else if (expression[i].hasClass("newline")) {
        // Write the line except the newline
        parts.pop();

        if (parts.length > 0) {
          children.push(buildHTMLUnbreakable(parts, options));
          parts = [];
        } // Put the newline at the top level


        children.push(expression[i]);
      }
    }

    if (parts.length > 0) {
      children.push(buildHTMLUnbreakable(parts, options));
    } // Now, if there was a tag, build it too and append it as a final child.


    var tagChild;

    if (tag) {
      tagChild = buildHTMLUnbreakable(buildExpression$1(tag, options, true));
      tagChild.classes = ["tag"];
      children.push(tagChild);
    } else if (eqnNum) {
      children.push(eqnNum);
    }

    var htmlNode = makeSpan$1(["katex-html"], children);
    htmlNode.setAttribute("aria-hidden", "true"); // Adjust the strut of the tag to be the maximum height of all children
    // (the height of the enclosing htmlNode) for proper vertical alignment.

    if (tagChild) {
      var strut = tagChild.children[0];
      strut.style.height = makeEm(htmlNode.height + htmlNode.depth);

      if (htmlNode.depth) {
        strut.style.verticalAlign = makeEm(-htmlNode.depth);
      }
    }

    return htmlNode;
  }

  /**
   * These objects store data about MathML nodes. This is the MathML equivalent
   * of the types in domTree.js. Since MathML handles its own rendering, and
   * since we're mainly using MathML to improve accessibility, we don't manage
   * any of the styling state that the plain DOM nodes do.
   *
   * The `toNode` and `toMarkup` functions work simlarly to how they do in
   * domTree.js, creating namespaced DOM nodes and HTML text markup respectively.
   */
  function newDocumentFragment(children) {
    return new DocumentFragment(children);
  }
  /**
   * This node represents a general purpose MathML node of any type. The
   * constructor requires the type of node to create (for example, `"mo"` or
   * `"mspace"`, corresponding to `<mo>` and `<mspace>` tags).
   */

  class MathNode {
    constructor(type, children, classes) {
      this.type = void 0;
      this.attributes = void 0;
      this.children = void 0;
      this.classes = void 0;
      this.type = type;
      this.attributes = {};
      this.children = children || [];
      this.classes = classes || [];
    }
    /**
     * Sets an attribute on a MathML node. MathML depends on attributes to convey a
     * semantic content, so this is used heavily.
     */


    setAttribute(name, value) {
      this.attributes[name] = value;
    }
    /**
     * Gets an attribute on a MathML node.
     */


    getAttribute(name) {
      return this.attributes[name];
    }
    /**
     * Converts the math node into a MathML-namespaced DOM element.
     */


    toNode() {
      var node = document.createElementNS("http://www.w3.org/1998/Math/MathML", this.type);

      for (var attr in this.attributes) {
        if (Object.prototype.hasOwnProperty.call(this.attributes, attr)) {
          node.setAttribute(attr, this.attributes[attr]);
        }
      }

      if (this.classes.length > 0) {
        node.className = createClass(this.classes);
      }

      for (var i = 0; i < this.children.length; i++) {
        node.appendChild(this.children[i].toNode());
      }

      return node;
    }
    /**
     * Converts the math node into an HTML markup string.
     */


    toMarkup() {
      var markup = "<" + this.type; // Add the attributes

      for (var attr in this.attributes) {
        if (Object.prototype.hasOwnProperty.call(this.attributes, attr)) {
          markup += " " + attr + "=\"";
          markup += utils$1.escape(this.attributes[attr]);
          markup += "\"";
        }
      }

      if (this.classes.length > 0) {
        markup += " class =\"" + utils$1.escape(createClass(this.classes)) + "\"";
      }

      markup += ">";

      for (var i = 0; i < this.children.length; i++) {
        markup += this.children[i].toMarkup();
      }

      markup += "</" + this.type + ">";
      return markup;
    }
    /**
     * Converts the math node into a string, similar to innerText, but escaped.
     */


    toText() {
      return this.children.map(child => child.toText()).join("");
    }

  }
  /**
   * This node represents a piece of text.
   */

  class TextNode {
    constructor(text) {
      this.text = void 0;
      this.text = text;
    }
    /**
     * Converts the text node into a DOM text node.
     */


    toNode() {
      return document.createTextNode(this.text);
    }
    /**
     * Converts the text node into escaped HTML markup
     * (representing the text itself).
     */


    toMarkup() {
      return utils$1.escape(this.toText());
    }
    /**
     * Converts the text node into a string
     * (representing the text iteself).
     */


    toText() {
      return this.text;
    }

  }
  /**
   * This node represents a space, but may render as <mspace.../> or as text,
   * depending on the width.
   */

  class SpaceNode {
    /**
     * Create a Space node with width given in CSS ems.
     */
    constructor(width) {
      this.width = void 0;
      this.character = void 0;
      this.width = width; // See https://www.w3.org/TR/2000/WD-MathML2-20000328/chapter6.html
      // for a table of space-like characters.  We use Unicode
      // representations instead of &LongNames; as it's not clear how to
      // make the latter via document.createTextNode.

      if (width >= 0.05555 && width <= 0.05556) {
        this.character = "\u200a"; // &VeryThinSpace;
      } else if (width >= 0.1666 && width <= 0.1667) {
        this.character = "\u2009"; // &ThinSpace;
      } else if (width >= 0.2222 && width <= 0.2223) {
        this.character = "\u2005"; // &MediumSpace;
      } else if (width >= 0.2777 && width <= 0.2778) {
        this.character = "\u2005\u200a"; // &ThickSpace;
      } else if (width >= -0.05556 && width <= -0.05555) {
        this.character = "\u200a\u2063"; // &NegativeVeryThinSpace;
      } else if (width >= -0.1667 && width <= -0.1666) {
        this.character = "\u2009\u2063"; // &NegativeThinSpace;
      } else if (width >= -0.2223 && width <= -0.2222) {
        this.character = "\u205f\u2063"; // &NegativeMediumSpace;
      } else if (width >= -0.2778 && width <= -0.2777) {
        this.character = "\u2005\u2063"; // &NegativeThickSpace;
      } else {
        this.character = null;
      }
    }
    /**
     * Converts the math node into a MathML-namespaced DOM element.
     */


    toNode() {
      if (this.character) {
        return document.createTextNode(this.character);
      } else {
        var node = document.createElementNS("http://www.w3.org/1998/Math/MathML", "mspace");
        node.setAttribute("width", makeEm(this.width));
        return node;
      }
    }
    /**
     * Converts the math node into an HTML markup string.
     */


    toMarkup() {
      if (this.character) {
        return "<mtext>" + this.character + "</mtext>";
      } else {
        return "<mspace width=\"" + makeEm(this.width) + "\"/>";
      }
    }
    /**
     * Converts the math node into a string, similar to innerText.
     */


    toText() {
      if (this.character) {
        return this.character;
      } else {
        return " ";
      }
    }

  }

  var mathMLTree = {
    MathNode,
    TextNode,
    SpaceNode,
    newDocumentFragment
  };

  /**
   * This file converts a parse tree into a cooresponding MathML tree. The main
   * entry point is the `buildMathML` function, which takes a parse tree from the
   * parser.
   */

  /**
   * Takes a symbol and converts it into a MathML text node after performing
   * optional replacement from symbols.js.
   */
  var makeText = function makeText(text, mode, options) {
    if (symbols[mode][text] && symbols[mode][text].replace && text.charCodeAt(0) !== 0xD835 && !(ligatures.hasOwnProperty(text) && options && (options.fontFamily && options.fontFamily.substr(4, 2) === "tt" || options.font && options.font.substr(4, 2) === "tt"))) {
      text = symbols[mode][text].replace;
    }

    return new mathMLTree.TextNode(text);
  };
  /**
   * Wrap the given array of nodes in an <mrow> node if needed, i.e.,
   * unless the array has length 1.  Always returns a single node.
   */

  var makeRow = function makeRow(body) {
    if (body.length === 1) {
      return body[0];
    } else {
      return new mathMLTree.MathNode("mrow", body);
    }
  };
  /**
   * Returns the math variant as a string or null if none is required.
   */

  var getVariant = function getVariant(group, options) {
    // Handle \text... font specifiers as best we can.
    // MathML has a limited list of allowable mathvariant specifiers; see
    // https://www.w3.org/TR/MathML3/chapter3.html#presm.commatt
    if (options.fontFamily === "texttt") {
      return "monospace";
    } else if (options.fontFamily === "textsf") {
      if (options.fontShape === "textit" && options.fontWeight === "textbf") {
        return "sans-serif-bold-italic";
      } else if (options.fontShape === "textit") {
        return "sans-serif-italic";
      } else if (options.fontWeight === "textbf") {
        return "bold-sans-serif";
      } else {
        return "sans-serif";
      }
    } else if (options.fontShape === "textit" && options.fontWeight === "textbf") {
      return "bold-italic";
    } else if (options.fontShape === "textit") {
      return "italic";
    } else if (options.fontWeight === "textbf") {
      return "bold";
    }

    var font = options.font;

    if (!font || font === "mathnormal") {
      return null;
    }

    var mode = group.mode;

    if (font === "mathit") {
      return "italic";
    } else if (font === "boldsymbol") {
      return group.type === "textord" ? "bold" : "bold-italic";
    } else if (font === "mathbf") {
      return "bold";
    } else if (font === "mathbb") {
      return "double-struck";
    } else if (font === "mathfrak") {
      return "fraktur";
    } else if (font === "mathscr" || font === "mathcal") {
      // MathML makes no distinction between script and caligrahpic
      return "script";
    } else if (font === "mathsf") {
      return "sans-serif";
    } else if (font === "mathtt") {
      return "monospace";
    }

    var text = group.text;

    if (utils$1.contains(["\\imath", "\\jmath"], text)) {
      return null;
    }

    if (symbols[mode][text] && symbols[mode][text].replace) {
      text = symbols[mode][text].replace;
    }

    var fontName = buildCommon.fontMap[font].fontName;

    if (getCharacterMetrics(text, fontName, mode)) {
      return buildCommon.fontMap[font].variant;
    }

    return null;
  };
  /**
   * Takes a list of nodes, builds them, and returns a list of the generated
   * MathML nodes.  Also combine consecutive <mtext> outputs into a single
   * <mtext> tag.
   */

  var buildExpression = function buildExpression(expression, options, isOrdgroup) {
    if (expression.length === 1) {
      var group = buildGroup(expression[0], options);

      if (isOrdgroup && group instanceof MathNode && group.type === "mo") {
        // When TeX writers want to suppress spacing on an operator,
        // they often put the operator by itself inside braces.
        group.setAttribute("lspace", "0em");
        group.setAttribute("rspace", "0em");
      }

      return [group];
    }

    var groups = [];
    var lastGroup;

    for (var i = 0; i < expression.length; i++) {
      var _group = buildGroup(expression[i], options);

      if (_group instanceof MathNode && lastGroup instanceof MathNode) {
        // Concatenate adjacent <mtext>s
        if (_group.type === 'mtext' && lastGroup.type === 'mtext' && _group.getAttribute('mathvariant') === lastGroup.getAttribute('mathvariant')) {
          lastGroup.children.push(..._group.children);
          continue; // Concatenate adjacent <mn>s
        } else if (_group.type === 'mn' && lastGroup.type === 'mn') {
          lastGroup.children.push(..._group.children);
          continue; // Concatenate <mn>...</mn> followed by <mi>.</mi>
        } else if (_group.type === 'mi' && _group.children.length === 1 && lastGroup.type === 'mn') {
          var child = _group.children[0];

          if (child instanceof TextNode && child.text === '.') {
            lastGroup.children.push(..._group.children);
            continue;
          }
        } else if (lastGroup.type === 'mi' && lastGroup.children.length === 1) {
          var lastChild = lastGroup.children[0];

          if (lastChild instanceof TextNode && lastChild.text === '\u0338' && (_group.type === 'mo' || _group.type === 'mi' || _group.type === 'mn')) {
            var _child = _group.children[0];

            if (_child instanceof TextNode && _child.text.length > 0) {
              // Overlay with combining character long solidus
              _child.text = _child.text.slice(0, 1) + "\u0338" + _child.text.slice(1);
              groups.pop();
            }
          }
        }
      }

      groups.push(_group);
      lastGroup = _group;
    }

    return groups;
  };
  /**
   * Equivalent to buildExpression, but wraps the elements in an <mrow>
   * if there's more than one.  Returns a single node instead of an array.
   */

  var buildExpressionRow = function buildExpressionRow(expression, options, isOrdgroup) {
    return makeRow(buildExpression(expression, options, isOrdgroup));
  };
  /**
   * Takes a group from the parser and calls the appropriate groupBuilders function
   * on it to produce a MathML node.
   */

  var buildGroup = function buildGroup(group, options) {
    if (!group) {
      return new mathMLTree.MathNode("mrow");
    }

    if (_mathmlGroupBuilders[group.type]) {
      // Call the groupBuilders function
      // $FlowFixMe
      var result = _mathmlGroupBuilders[group.type](group, options); // $FlowFixMe

      return result;
    } else {
      throw new ParseError("Got group of unknown type: '" + group.type + "'");
    }
  };
  /**
   * Takes a full parse tree and settings and builds a MathML representation of
   * it. In particular, we put the elements from building the parse tree into a
   * <semantics> tag so we can also include that TeX source as an annotation.
   *
   * Note that we actually return a domTree element with a `<math>` inside it so
   * we can do appropriate styling.
   */

  function buildMathML(tree, texExpression, options, isDisplayMode, forMathmlOnly) {
    var expression = buildExpression(tree, options); // TODO: Make a pass thru the MathML similar to buildHTML.traverseNonSpaceNodes
    // and add spacing nodes. This is necessary only adjacent to math operators
    // like \sin or \lim or to subsup elements that contain math operators.
    // MathML takes care of the other spacing issues.
    // Wrap up the expression in an mrow so it is presented in the semantics
    // tag correctly, unless it's a single <mrow> or <mtable>.

    var wrapper;

    if (expression.length === 1 && expression[0] instanceof MathNode && utils$1.contains(["mrow", "mtable"], expression[0].type)) {
      wrapper = expression[0];
    } else {
      wrapper = new mathMLTree.MathNode("mrow", expression);
    } // Build a TeX annotation of the source


    var annotation = new mathMLTree.MathNode("annotation", [new mathMLTree.TextNode(texExpression)]);
    annotation.setAttribute("encoding", "application/x-tex");
    var semantics = new mathMLTree.MathNode("semantics", [wrapper, annotation]);
    var math = new mathMLTree.MathNode("math", [semantics]);
    math.setAttribute("xmlns", "http://www.w3.org/1998/Math/MathML");

    if (isDisplayMode) {
      math.setAttribute("display", "block");
    } // You can't style <math> nodes, so we wrap the node in a span.
    // NOTE: The span class is not typed to have <math> nodes as children, and
    // we don't want to make the children type more generic since the children
    // of span are expected to have more fields in `buildHtml` contexts.


    var wrapperClass = forMathmlOnly ? "katex" : "katex-mathml"; // $FlowFixMe

    return buildCommon.makeSpan([wrapperClass], [math]);
  }

  var optionsFromSettings = function optionsFromSettings(settings) {
    return new Options({
      style: settings.displayMode ? Style$1.DISPLAY : Style$1.TEXT,
      maxSize: settings.maxSize,
      minRuleThickness: settings.minRuleThickness
    });
  };

  var displayWrap = function displayWrap(node, settings) {
    if (settings.displayMode) {
      var classes = ["katex-display"];

      if (settings.leqno) {
        classes.push("leqno");
      }

      if (settings.fleqn) {
        classes.push("fleqn");
      }

      node = buildCommon.makeSpan(classes, [node]);
    }

    return node;
  };

  var buildTree = function buildTree(tree, expression, settings) {
    var options = optionsFromSettings(settings);
    var katexNode;

    if (settings.output === "mathml") {
      return buildMathML(tree, expression, options, settings.displayMode, true);
    } else if (settings.output === "html") {
      var htmlNode = buildHTML(tree, options);
      katexNode = buildCommon.makeSpan(["katex"], [htmlNode]);
    } else {
      var mathMLNode = buildMathML(tree, expression, options, settings.displayMode, false);

      var _htmlNode = buildHTML(tree, options);

      katexNode = buildCommon.makeSpan(["katex"], [mathMLNode, _htmlNode]);
    }

    return displayWrap(katexNode, settings);
  };
  var buildHTMLTree = function buildHTMLTree(tree, expression, settings) {
    var options = optionsFromSettings(settings);
    var htmlNode = buildHTML(tree, options);
    var katexNode = buildCommon.makeSpan(["katex"], [htmlNode]);
    return displayWrap(katexNode, settings);
  };

  /**
   * This file provides support to buildMathML.js and buildHTML.js
   * for stretchy wide elements rendered from SVG files
   * and other CSS trickery.
   */
  var stretchyCodePoint = {
    widehat: "^",
    widecheck: "ˇ",
    widetilde: "~",
    utilde: "~",
    overleftarrow: "\u2190",
    underleftarrow: "\u2190",
    xleftarrow: "\u2190",
    overrightarrow: "\u2192",
    underrightarrow: "\u2192",
    xrightarrow: "\u2192",
    underbrace: "\u23df",
    overbrace: "\u23de",
    overgroup: "\u23e0",
    undergroup: "\u23e1",
    overleftrightarrow: "\u2194",
    underleftrightarrow: "\u2194",
    xleftrightarrow: "\u2194",
    Overrightarrow: "\u21d2",
    xRightarrow: "\u21d2",
    overleftharpoon: "\u21bc",
    xleftharpoonup: "\u21bc",
    overrightharpoon: "\u21c0",
    xrightharpoonup: "\u21c0",
    xLeftarrow: "\u21d0",
    xLeftrightarrow: "\u21d4",
    xhookleftarrow: "\u21a9",
    xhookrightarrow: "\u21aa",
    xmapsto: "\u21a6",
    xrightharpoondown: "\u21c1",
    xleftharpoondown: "\u21bd",
    xrightleftharpoons: "\u21cc",
    xleftrightharpoons: "\u21cb",
    xtwoheadleftarrow: "\u219e",
    xtwoheadrightarrow: "\u21a0",
    xlongequal: "=",
    xtofrom: "\u21c4",
    xrightleftarrows: "\u21c4",
    xrightequilibrium: "\u21cc",
    // Not a perfect match.
    xleftequilibrium: "\u21cb",
    // None better available.
    "\\cdrightarrow": "\u2192",
    "\\cdleftarrow": "\u2190",
    "\\cdlongequal": "="
  };

  var mathMLnode = function mathMLnode(label) {
    var node = new mathMLTree.MathNode("mo", [new mathMLTree.TextNode(stretchyCodePoint[label.replace(/^\\/, '')])]);
    node.setAttribute("stretchy", "true");
    return node;
  }; // Many of the KaTeX SVG images have been adapted from glyphs in KaTeX fonts.
  // Copyright (c) 2009-2010, Design Science, Inc. (<www.mathjax.org>)
  // Copyright (c) 2014-2017 Khan Academy (<www.khanacademy.org>)
  // Licensed under the SIL Open Font License, Version 1.1.
  // See \nhttp://scripts.sil.org/OFL
  // Very Long SVGs
  //    Many of the KaTeX stretchy wide elements use a long SVG image and an
  //    overflow: hidden tactic to achieve a stretchy image while avoiding
  //    distortion of arrowheads or brace corners.
  //    The SVG typically contains a very long (400 em) arrow.
  //    The SVG is in a container span that has overflow: hidden, so the span
  //    acts like a window that exposes only part of the  SVG.
  //    The SVG always has a longer, thinner aspect ratio than the container span.
  //    After the SVG fills 100% of the height of the container span,
  //    there is a long arrow shaft left over. That left-over shaft is not shown.
  //    Instead, it is sliced off because the span's CSS has overflow: hidden.
  //    Thus, the reader sees an arrow that matches the subject matter width
  //    without distortion.
  //    Some functions, such as \cancel, need to vary their aspect ratio. These
  //    functions do not get the overflow SVG treatment.
  // Second Brush Stroke
  //    Low resolution monitors struggle to display images in fine detail.
  //    So browsers apply anti-aliasing. A long straight arrow shaft therefore
  //    will sometimes appear as if it has a blurred edge.
  //    To mitigate this, these SVG files contain a second "brush-stroke" on the
  //    arrow shafts. That is, a second long thin rectangular SVG path has been
  //    written directly on top of each arrow shaft. This reinforcement causes
  //    some of the screen pixels to display as black instead of the anti-aliased
  //    gray pixel that a  single path would generate. So we get arrow shafts
  //    whose edges appear to be sharper.
  // In the katexImagesData object just below, the dimensions all
  // correspond to path geometry inside the relevant SVG.
  // For example, \overrightarrow uses the same arrowhead as glyph U+2192
  // from the KaTeX Main font. The scaling factor is 1000.
  // That is, inside the font, that arrowhead is 522 units tall, which
  // corresponds to 0.522 em inside the document.


  var katexImagesData = {
    //   path(s), minWidth, height, align
    overrightarrow: [["rightarrow"], 0.888, 522, "xMaxYMin"],
    overleftarrow: [["leftarrow"], 0.888, 522, "xMinYMin"],
    underrightarrow: [["rightarrow"], 0.888, 522, "xMaxYMin"],
    underleftarrow: [["leftarrow"], 0.888, 522, "xMinYMin"],
    xrightarrow: [["rightarrow"], 1.469, 522, "xMaxYMin"],
    "\\cdrightarrow": [["rightarrow"], 3.0, 522, "xMaxYMin"],
    // CD minwwidth2.5pc
    xleftarrow: [["leftarrow"], 1.469, 522, "xMinYMin"],
    "\\cdleftarrow": [["leftarrow"], 3.0, 522, "xMinYMin"],
    Overrightarrow: [["doublerightarrow"], 0.888, 560, "xMaxYMin"],
    xRightarrow: [["doublerightarrow"], 1.526, 560, "xMaxYMin"],
    xLeftarrow: [["doubleleftarrow"], 1.526, 560, "xMinYMin"],
    overleftharpoon: [["leftharpoon"], 0.888, 522, "xMinYMin"],
    xleftharpoonup: [["leftharpoon"], 0.888, 522, "xMinYMin"],
    xleftharpoondown: [["leftharpoondown"], 0.888, 522, "xMinYMin"],
    overrightharpoon: [["rightharpoon"], 0.888, 522, "xMaxYMin"],
    xrightharpoonup: [["rightharpoon"], 0.888, 522, "xMaxYMin"],
    xrightharpoondown: [["rightharpoondown"], 0.888, 522, "xMaxYMin"],
    xlongequal: [["longequal"], 0.888, 334, "xMinYMin"],
    "\\cdlongequal": [["longequal"], 3.0, 334, "xMinYMin"],
    xtwoheadleftarrow: [["twoheadleftarrow"], 0.888, 334, "xMinYMin"],
    xtwoheadrightarrow: [["twoheadrightarrow"], 0.888, 334, "xMaxYMin"],
    overleftrightarrow: [["leftarrow", "rightarrow"], 0.888, 522],
    overbrace: [["leftbrace", "midbrace", "rightbrace"], 1.6, 548],
    underbrace: [["leftbraceunder", "midbraceunder", "rightbraceunder"], 1.6, 548],
    underleftrightarrow: [["leftarrow", "rightarrow"], 0.888, 522],
    xleftrightarrow: [["leftarrow", "rightarrow"], 1.75, 522],
    xLeftrightarrow: [["doubleleftarrow", "doublerightarrow"], 1.75, 560],
    xrightleftharpoons: [["leftharpoondownplus", "rightharpoonplus"], 1.75, 716],
    xleftrightharpoons: [["leftharpoonplus", "rightharpoondownplus"], 1.75, 716],
    xhookleftarrow: [["leftarrow", "righthook"], 1.08, 522],
    xhookrightarrow: [["lefthook", "rightarrow"], 1.08, 522],
    overlinesegment: [["leftlinesegment", "rightlinesegment"], 0.888, 522],
    underlinesegment: [["leftlinesegment", "rightlinesegment"], 0.888, 522],
    overgroup: [["leftgroup", "rightgroup"], 0.888, 342],
    undergroup: [["leftgroupunder", "rightgroupunder"], 0.888, 342],
    xmapsto: [["leftmapsto", "rightarrow"], 1.5, 522],
    xtofrom: [["leftToFrom", "rightToFrom"], 1.75, 528],
    // The next three arrows are from the mhchem package.
    // In mhchem.sty, min-length is 2.0em. But these arrows might appear in the
    // document as \xrightarrow or \xrightleftharpoons. Those have
    // min-length = 1.75em, so we set min-length on these next three to match.
    xrightleftarrows: [["baraboveleftarrow", "rightarrowabovebar"], 1.75, 901],
    xrightequilibrium: [["baraboveshortleftharpoon", "rightharpoonaboveshortbar"], 1.75, 716],
    xleftequilibrium: [["shortbaraboveleftharpoon", "shortrightharpoonabovebar"], 1.75, 716]
  };

  var groupLength = function groupLength(arg) {
    if (arg.type === "ordgroup") {
      return arg.body.length;
    } else {
      return 1;
    }
  };

  var svgSpan = function svgSpan(group, options) {
    // Create a span with inline SVG for the element.
    function buildSvgSpan_() {
      var viewBoxWidth = 400000; // default

      var label = group.label.substr(1);

      if (utils$1.contains(["widehat", "widecheck", "widetilde", "utilde"], label)) {
        // Each type in the `if` statement corresponds to one of the ParseNode
        // types below. This narrowing is required to access `grp.base`.
        // $FlowFixMe
        var grp = group; // There are four SVG images available for each function.
        // Choose a taller image when there are more characters.

        var numChars = groupLength(grp.base);
        var viewBoxHeight;
        var pathName;

        var _height;

        if (numChars > 5) {
          if (label === "widehat" || label === "widecheck") {
            viewBoxHeight = 420;
            viewBoxWidth = 2364;
            _height = 0.42;
            pathName = label + "4";
          } else {
            viewBoxHeight = 312;
            viewBoxWidth = 2340;
            _height = 0.34;
            pathName = "tilde4";
          }
        } else {
          var imgIndex = [1, 1, 2, 2, 3, 3][numChars];

          if (label === "widehat" || label === "widecheck") {
            viewBoxWidth = [0, 1062, 2364, 2364, 2364][imgIndex];
            viewBoxHeight = [0, 239, 300, 360, 420][imgIndex];
            _height = [0, 0.24, 0.3, 0.3, 0.36, 0.42][imgIndex];
            pathName = label + imgIndex;
          } else {
            viewBoxWidth = [0, 600, 1033, 2339, 2340][imgIndex];
            viewBoxHeight = [0, 260, 286, 306, 312][imgIndex];
            _height = [0, 0.26, 0.286, 0.3, 0.306, 0.34][imgIndex];
            pathName = "tilde" + imgIndex;
          }
        }

        var path = new PathNode(pathName);
        var svgNode = new SvgNode([path], {
          "width": "100%",
          "height": makeEm(_height),
          "viewBox": "0 0 " + viewBoxWidth + " " + viewBoxHeight,
          "preserveAspectRatio": "none"
        });
        return {
          span: buildCommon.makeSvgSpan([], [svgNode], options),
          minWidth: 0,
          height: _height
        };
      } else {
        var spans = [];
        var data = katexImagesData[label];
        var [paths, _minWidth, _viewBoxHeight] = data;

        var _height2 = _viewBoxHeight / 1000;

        var numSvgChildren = paths.length;
        var widthClasses;
        var aligns;

        if (numSvgChildren === 1) {
          // $FlowFixMe: All these cases must be of the 4-tuple type.
          var align1 = data[3];
          widthClasses = ["hide-tail"];
          aligns = [align1];
        } else if (numSvgChildren === 2) {
          widthClasses = ["halfarrow-left", "halfarrow-right"];
          aligns = ["xMinYMin", "xMaxYMin"];
        } else if (numSvgChildren === 3) {
          widthClasses = ["brace-left", "brace-center", "brace-right"];
          aligns = ["xMinYMin", "xMidYMin", "xMaxYMin"];
        } else {
          throw new Error("Correct katexImagesData or update code here to support\n                    " + numSvgChildren + " children.");
        }

        for (var i = 0; i < numSvgChildren; i++) {
          var _path = new PathNode(paths[i]);

          var _svgNode = new SvgNode([_path], {
            "width": "400em",
            "height": makeEm(_height2),
            "viewBox": "0 0 " + viewBoxWidth + " " + _viewBoxHeight,
            "preserveAspectRatio": aligns[i] + " slice"
          });

          var _span = buildCommon.makeSvgSpan([widthClasses[i]], [_svgNode], options);

          if (numSvgChildren === 1) {
            return {
              span: _span,
              minWidth: _minWidth,
              height: _height2
            };
          } else {
            _span.style.height = makeEm(_height2);
            spans.push(_span);
          }
        }

        return {
          span: buildCommon.makeSpan(["stretchy"], spans, options),
          minWidth: _minWidth,
          height: _height2
        };
      }
    } // buildSvgSpan_()


    var {
      span,
      minWidth,
      height
    } = buildSvgSpan_(); // Note that we are returning span.depth = 0.
    // Any adjustments relative to the baseline must be done in buildHTML.

    span.height = height;
    span.style.height = makeEm(height);

    if (minWidth > 0) {
      span.style.minWidth = makeEm(minWidth);
    }

    return span;
  };

  var encloseSpan = function encloseSpan(inner, label, topPad, bottomPad, options) {
    // Return an image span for \cancel, \bcancel, \xcancel, \fbox, or \angl
    var img;
    var totalHeight = inner.height + inner.depth + topPad + bottomPad;

    if (/fbox|color|angl/.test(label)) {
      img = buildCommon.makeSpan(["stretchy", label], [], options);

      if (label === "fbox") {
        var color = options.color && options.getColor();

        if (color) {
          img.style.borderColor = color;
        }
      }
    } else {
      // \cancel, \bcancel, or \xcancel
      // Since \cancel's SVG is inline and it omits the viewBox attribute,
      // its stroke-width will not vary with span area.
      var lines = [];

      if (/^[bx]cancel$/.test(label)) {
        lines.push(new LineNode({
          "x1": "0",
          "y1": "0",
          "x2": "100%",
          "y2": "100%",
          "stroke-width": "0.046em"
        }));
      }

      if (/^x?cancel$/.test(label)) {
        lines.push(new LineNode({
          "x1": "0",
          "y1": "100%",
          "x2": "100%",
          "y2": "0",
          "stroke-width": "0.046em"
        }));
      }

      var svgNode = new SvgNode(lines, {
        "width": "100%",
        "height": makeEm(totalHeight)
      });
      img = buildCommon.makeSvgSpan([], [svgNode], options);
    }

    img.height = totalHeight;
    img.style.height = makeEm(totalHeight);
    return img;
  };

  var stretchy = {
    encloseSpan,
    mathMLnode,
    svgSpan
  };

  /**
   * Asserts that the node is of the given type and returns it with stricter
   * typing. Throws if the node's type does not match.
   */
  function assertNodeType(node, type) {
    if (!node || node.type !== type) {
      throw new Error("Expected node of type " + type + ", but got " + (node ? "node of type " + node.type : String(node)));
    } // $FlowFixMe, >=0.125


    return node;
  }
  /**
   * Returns the node more strictly typed iff it is of the given type. Otherwise,
   * returns null.
   */

  function assertSymbolNodeType(node) {
    var typedNode = checkSymbolNodeType(node);

    if (!typedNode) {
      throw new Error("Expected node of symbol group type, but got " + (node ? "node of type " + node.type : String(node)));
    }

    return typedNode;
  }
  /**
   * Returns the node more strictly typed iff it is of the given type. Otherwise,
   * returns null.
   */

  function checkSymbolNodeType(node) {
    if (node && (node.type === "atom" || NON_ATOMS.hasOwnProperty(node.type))) {
      // $FlowFixMe
      return node;
    }

    return null;
  }

  // NOTE: Unlike most `htmlBuilder`s, this one handles not only "accent", but
  // also "supsub" since an accent can affect super/subscripting.
  var htmlBuilder$a = (grp, options) => {
    // Accents are handled in the TeXbook pg. 443, rule 12.
    var base;
    var group;
    var supSubGroup;

    if (grp && grp.type === "supsub") {
      // If our base is a character box, and we have superscripts and
      // subscripts, the supsub will defer to us. In particular, we want
      // to attach the superscripts and subscripts to the inner body (so
      // that the position of the superscripts and subscripts won't be
      // affected by the height of the accent). We accomplish this by
      // sticking the base of the accent into the base of the supsub, and
      // rendering that, while keeping track of where the accent is.
      // The real accent group is the base of the supsub group
      group = assertNodeType(grp.base, "accent"); // The character box is the base of the accent group

      base = group.base; // Stick the character box into the base of the supsub group

      grp.base = base; // Rerender the supsub group with its new base, and store that
      // result.

      supSubGroup = assertSpan(buildGroup$1(grp, options)); // reset original base

      grp.base = group;
    } else {
      group = assertNodeType(grp, "accent");
      base = group.base;
    } // Build the base group


    var body = buildGroup$1(base, options.havingCrampedStyle()); // Does the accent need to shift for the skew of a character?

    var mustShift = group.isShifty && utils$1.isCharacterBox(base); // Calculate the skew of the accent. This is based on the line "If the
    // nucleus is not a single character, let s = 0; otherwise set s to the
    // kern amount for the nucleus followed by the \skewchar of its font."
    // Note that our skew metrics are just the kern between each character
    // and the skewchar.

    var skew = 0;

    if (mustShift) {
      // If the base is a character box, then we want the skew of the
      // innermost character. To do that, we find the innermost character:
      var baseChar = utils$1.getBaseElem(base); // Then, we render its group to get the symbol inside it

      var baseGroup = buildGroup$1(baseChar, options.havingCrampedStyle()); // Finally, we pull the skew off of the symbol.

      skew = assertSymbolDomNode(baseGroup).skew; // Note that we now throw away baseGroup, because the layers we
      // removed with getBaseElem might contain things like \color which
      // we can't get rid of.
      // TODO(emily): Find a better way to get the skew
    }

    var accentBelow = group.label === "\\c"; // calculate the amount of space between the body and the accent

    var clearance = accentBelow ? body.height + body.depth : Math.min(body.height, options.fontMetrics().xHeight); // Build the accent

    var accentBody;

    if (!group.isStretchy) {
      var accent;
      var width;

      if (group.label === "\\vec") {
        // Before version 0.9, \vec used the combining font glyph U+20D7.
        // But browsers, especially Safari, are not consistent in how they
        // render combining characters when not preceded by a character.
        // So now we use an SVG.
        // If Safari reforms, we should consider reverting to the glyph.
        accent = buildCommon.staticSvg("vec", options);
        width = buildCommon.svgData.vec[1];
      } else {
        accent = buildCommon.makeOrd({
          mode: group.mode,
          text: group.label
        }, options, "textord");
        accent = assertSymbolDomNode(accent); // Remove the italic correction of the accent, because it only serves to
        // shift the accent over to a place we don't want.

        accent.italic = 0;
        width = accent.width;

        if (accentBelow) {
          clearance += accent.depth;
        }
      }

      accentBody = buildCommon.makeSpan(["accent-body"], [accent]); // "Full" accents expand the width of the resulting symbol to be
      // at least the width of the accent, and overlap directly onto the
      // character without any vertical offset.

      var accentFull = group.label === "\\textcircled";

      if (accentFull) {
        accentBody.classes.push('accent-full');
        clearance = body.height;
      } // Shift the accent over by the skew.


      var left = skew; // CSS defines `.katex .accent .accent-body:not(.accent-full) { width: 0 }`
      // so that the accent doesn't contribute to the bounding box.
      // We need to shift the character by its width (effectively half
      // its width) to compensate.

      if (!accentFull) {
        left -= width / 2;
      }

      accentBody.style.left = makeEm(left); // \textcircled uses the \bigcirc glyph, so it needs some
      // vertical adjustment to match LaTeX.

      if (group.label === "\\textcircled") {
        accentBody.style.top = ".2em";
      }

      accentBody = buildCommon.makeVList({
        positionType: "firstBaseline",
        children: [{
          type: "elem",
          elem: body
        }, {
          type: "kern",
          size: -clearance
        }, {
          type: "elem",
          elem: accentBody
        }]
      }, options);
    } else {
      accentBody = stretchy.svgSpan(group, options);
      accentBody = buildCommon.makeVList({
        positionType: "firstBaseline",
        children: [{
          type: "elem",
          elem: body
        }, {
          type: "elem",
          elem: accentBody,
          wrapperClasses: ["svg-align"],
          wrapperStyle: skew > 0 ? {
            width: "calc(100% - " + makeEm(2 * skew) + ")",
            marginLeft: makeEm(2 * skew)
          } : undefined
        }]
      }, options);
    }

    var accentWrap = buildCommon.makeSpan(["mord", "accent"], [accentBody], options);

    if (supSubGroup) {
      // Here, we replace the "base" child of the supsub with our newly
      // generated accent.
      supSubGroup.children[0] = accentWrap; // Since we don't rerun the height calculation after replacing the
      // accent, we manually recalculate height.

      supSubGroup.height = Math.max(accentWrap.height, supSubGroup.height); // Accents should always be ords, even when their innards are not.

      supSubGroup.classes[0] = "mord";
      return supSubGroup;
    } else {
      return accentWrap;
    }
  };

  var mathmlBuilder$9 = (group, options) => {
    var accentNode = group.isStretchy ? stretchy.mathMLnode(group.label) : new mathMLTree.MathNode("mo", [makeText(group.label, group.mode)]);
    var node = new mathMLTree.MathNode("mover", [buildGroup(group.base, options), accentNode]);
    node.setAttribute("accent", "true");
    return node;
  };

  var NON_STRETCHY_ACCENT_REGEX = new RegExp(["\\acute", "\\grave", "\\ddot", "\\tilde", "\\bar", "\\breve", "\\check", "\\hat", "\\vec", "\\dot", "\\mathring"].map(accent => "\\" + accent).join("|")); // Accents

  defineFunction({
    type: "accent",
    names: ["\\acute", "\\grave", "\\ddot", "\\tilde", "\\bar", "\\breve", "\\check", "\\hat", "\\vec", "\\dot", "\\mathring", "\\widecheck", "\\widehat", "\\widetilde", "\\overrightarrow", "\\overleftarrow", "\\Overrightarrow", "\\overleftrightarrow", "\\overgroup", "\\overlinesegment", "\\overleftharpoon", "\\overrightharpoon"],
    props: {
      numArgs: 1
    },
    handler: (context, args) => {
      var base = normalizeArgument(args[0]);
      var isStretchy = !NON_STRETCHY_ACCENT_REGEX.test(context.funcName);
      var isShifty = !isStretchy || context.funcName === "\\widehat" || context.funcName === "\\widetilde" || context.funcName === "\\widecheck";
      return {
        type: "accent",
        mode: context.parser.mode,
        label: context.funcName,
        isStretchy: isStretchy,
        isShifty: isShifty,
        base: base
      };
    },
    htmlBuilder: htmlBuilder$a,
    mathmlBuilder: mathmlBuilder$9
  }); // Text-mode accents

  defineFunction({
    type: "accent",
    names: ["\\'", "\\`", "\\^", "\\~", "\\=", "\\u", "\\.", '\\"', "\\c", "\\r", "\\H", "\\v", "\\textcircled"],
    props: {
      numArgs: 1,
      allowedInText: true,
      allowedInMath: true,
      // unless in strict mode
      argTypes: ["primitive"]
    },
    handler: (context, args) => {
      var base = args[0];
      var mode = context.parser.mode;

      if (mode === "math") {
        context.parser.settings.reportNonstrict("mathVsTextAccents", "LaTeX's accent " + context.funcName + " works only in text mode");
        mode = "text";
      }

      return {
        type: "accent",
        mode: mode,
        label: context.funcName,
        isStretchy: false,
        isShifty: true,
        base: base
      };
    },
    htmlBuilder: htmlBuilder$a,
    mathmlBuilder: mathmlBuilder$9
  });

  // Horizontal overlap functions
  defineFunction({
    type: "accentUnder",
    names: ["\\underleftarrow", "\\underrightarrow", "\\underleftrightarrow", "\\undergroup", "\\underlinesegment", "\\utilde"],
    props: {
      numArgs: 1
    },
    handler: (_ref, args) => {
      var {
        parser,
        funcName
      } = _ref;
      var base = args[0];
      return {
        type: "accentUnder",
        mode: parser.mode,
        label: funcName,
        base: base
      };
    },
    htmlBuilder: (group, options) => {
      // Treat under accents much like underlines.
      var innerGroup = buildGroup$1(group.base, options);
      var accentBody = stretchy.svgSpan(group, options);
      var kern = group.label === "\\utilde" ? 0.12 : 0; // Generate the vlist, with the appropriate kerns

      var vlist = buildCommon.makeVList({
        positionType: "top",
        positionData: innerGroup.height,
        children: [{
          type: "elem",
          elem: accentBody,
          wrapperClasses: ["svg-align"]
        }, {
          type: "kern",
          size: kern
        }, {
          type: "elem",
          elem: innerGroup
        }]
      }, options);
      return buildCommon.makeSpan(["mord", "accentunder"], [vlist], options);
    },
    mathmlBuilder: (group, options) => {
      var accentNode = stretchy.mathMLnode(group.label);
      var node = new mathMLTree.MathNode("munder", [buildGroup(group.base, options), accentNode]);
      node.setAttribute("accentunder", "true");
      return node;
    }
  });

  // Helper function
  var paddedNode = group => {
    var node = new mathMLTree.MathNode("mpadded", group ? [group] : []);
    node.setAttribute("width", "+0.6em");
    node.setAttribute("lspace", "0.3em");
    return node;
  }; // Stretchy arrows with an optional argument


  defineFunction({
    type: "xArrow",
    names: ["\\xleftarrow", "\\xrightarrow", "\\xLeftarrow", "\\xRightarrow", "\\xleftrightarrow", "\\xLeftrightarrow", "\\xhookleftarrow", "\\xhookrightarrow", "\\xmapsto", "\\xrightharpoondown", "\\xrightharpoonup", "\\xleftharpoondown", "\\xleftharpoonup", "\\xrightleftharpoons", "\\xleftrightharpoons", "\\xlongequal", "\\xtwoheadrightarrow", "\\xtwoheadleftarrow", "\\xtofrom", // The next 3 functions are here to support the mhchem extension.
    // Direct use of these functions is discouraged and may break someday.
    "\\xrightleftarrows", "\\xrightequilibrium", "\\xleftequilibrium", // The next 3 functions are here only to support the {CD} environment.
    "\\\\cdrightarrow", "\\\\cdleftarrow", "\\\\cdlongequal"],
    props: {
      numArgs: 1,
      numOptionalArgs: 1
    },

    handler(_ref, args, optArgs) {
      var {
        parser,
        funcName
      } = _ref;
      return {
        type: "xArrow",
        mode: parser.mode,
        label: funcName,
        body: args[0],
        below: optArgs[0]
      };
    },

    // Flow is unable to correctly infer the type of `group`, even though it's
    // unamibiguously determined from the passed-in `type` above.
    htmlBuilder(group, options) {
      var style = options.style; // Build the argument groups in the appropriate style.
      // Ref: amsmath.dtx:   \hbox{$\scriptstyle\mkern#3mu{#6}\mkern#4mu$}%
      // Some groups can return document fragments.  Handle those by wrapping
      // them in a span.

      var newOptions = options.havingStyle(style.sup());
      var upperGroup = buildCommon.wrapFragment(buildGroup$1(group.body, newOptions, options), options);
      var arrowPrefix = group.label.slice(0, 2) === "\\x" ? "x" : "cd";
      upperGroup.classes.push(arrowPrefix + "-arrow-pad");
      var lowerGroup;

      if (group.below) {
        // Build the lower group
        newOptions = options.havingStyle(style.sub());
        lowerGroup = buildCommon.wrapFragment(buildGroup$1(group.below, newOptions, options), options);
        lowerGroup.classes.push(arrowPrefix + "-arrow-pad");
      }

      var arrowBody = stretchy.svgSpan(group, options); // Re shift: Note that stretchy.svgSpan returned arrowBody.depth = 0.
      // The point we want on the math axis is at 0.5 * arrowBody.height.

      var arrowShift = -options.fontMetrics().axisHeight + 0.5 * arrowBody.height; // 2 mu kern. Ref: amsmath.dtx: #7\if0#2\else\mkern#2mu\fi

      var upperShift = -options.fontMetrics().axisHeight - 0.5 * arrowBody.height - 0.111; // 0.111 em = 2 mu

      if (upperGroup.depth > 0.25 || group.label === "\\xleftequilibrium") {
        upperShift -= upperGroup.depth; // shift up if depth encroaches
      } // Generate the vlist


      var vlist;

      if (lowerGroup) {
        var lowerShift = -options.fontMetrics().axisHeight + lowerGroup.height + 0.5 * arrowBody.height + 0.111;
        vlist = buildCommon.makeVList({
          positionType: "individualShift",
          children: [{
            type: "elem",
            elem: upperGroup,
            shift: upperShift
          }, {
            type: "elem",
            elem: arrowBody,
            shift: arrowShift
          }, {
            type: "elem",
            elem: lowerGroup,
            shift: lowerShift
          }]
        }, options);
      } else {
        vlist = buildCommon.makeVList({
          positionType: "individualShift",
          children: [{
            type: "elem",
            elem: upperGroup,
            shift: upperShift
          }, {
            type: "elem",
            elem: arrowBody,
            shift: arrowShift
          }]
        }, options);
      } // $FlowFixMe: Replace this with passing "svg-align" into makeVList.


      vlist.children[0].children[0].children[1].classes.push("svg-align");
      return buildCommon.makeSpan(["mrel", "x-arrow"], [vlist], options);
    },

    mathmlBuilder(group, options) {
      var arrowNode = stretchy.mathMLnode(group.label);
      arrowNode.setAttribute("minsize", group.label.charAt(0) === "x" ? "1.75em" : "3.0em");
      var node;

      if (group.body) {
        var upperNode = paddedNode(buildGroup(group.body, options));

        if (group.below) {
          var lowerNode = paddedNode(buildGroup(group.below, options));
          node = new mathMLTree.MathNode("munderover", [arrowNode, lowerNode, upperNode]);
        } else {
          node = new mathMLTree.MathNode("mover", [arrowNode, upperNode]);
        }
      } else if (group.below) {
        var _lowerNode = paddedNode(buildGroup(group.below, options));

        node = new mathMLTree.MathNode("munder", [arrowNode, _lowerNode]);
      } else {
        // This should never happen.
        // Parser.js throws an error if there is no argument.
        node = paddedNode();
        node = new mathMLTree.MathNode("mover", [arrowNode, node]);
      }

      return node;
    }

  });

  var cdArrowFunctionName = {
    ">": "\\\\cdrightarrow",
    "<": "\\\\cdleftarrow",
    "=": "\\\\cdlongequal",
    "A": "\\uparrow",
    "V": "\\downarrow",
    "|": "\\Vert",
    ".": "no arrow"
  };

  var newCell = () => {
    // Create an empty cell, to be filled below with parse nodes.
    // The parseTree from this module must be constructed like the
    // one created by parseArray(), so an empty CD cell must
    // be a ParseNode<"styling">. And CD is always displaystyle.
    // So these values are fixed and flow can do implicit typing.
    return {
      type: "styling",
      body: [],
      mode: "math",
      style: "display"
    };
  };

  var isStartOfArrow = node => {
    return node.type === "textord" && node.text === "@";
  };

  var isLabelEnd = (node, endChar) => {
    return (node.type === "mathord" || node.type === "atom") && node.text === endChar;
  };

  function cdArrow(arrowChar, labels, parser) {
    // Return a parse tree of an arrow and its labels.
    // This acts in a way similar to a macro expansion.
    var funcName = cdArrowFunctionName[arrowChar];

    switch (funcName) {
      case "\\\\cdrightarrow":
      case "\\\\cdleftarrow":
        return parser.callFunction(funcName, [labels[0]], [labels[1]]);

      case "\\uparrow":
      case "\\downarrow":
        {
          var leftLabel = parser.callFunction("\\\\cdleft", [labels[0]], []);
          var bareArrow = {
            type: "atom",
            text: funcName,
            mode: "math",
            family: "rel"
          };
          var sizedArrow = parser.callFunction("\\Big", [bareArrow], []);
          var rightLabel = parser.callFunction("\\\\cdright", [labels[1]], []);
          var arrowGroup = {
            type: "ordgroup",
            mode: "math",
            body: [leftLabel, sizedArrow, rightLabel]
          };
          return parser.callFunction("\\\\cdparent", [arrowGroup], []);
        }

      case "\\\\cdlongequal":
        return parser.callFunction("\\\\cdlongequal", [], []);

      case "\\Vert":
        {
          var arrow = {
            type: "textord",
            text: "\\Vert",
            mode: "math"
          };
          return parser.callFunction("\\Big", [arrow], []);
        }

      default:
        return {
          type: "textord",
          text: " ",
          mode: "math"
        };
    }
  }

  function parseCD(parser) {
    // Get the array's parse nodes with \\ temporarily mapped to \cr.
    var parsedRows = [];
    parser.gullet.beginGroup();
    parser.gullet.macros.set("\\cr", "\\\\\\relax");
    parser.gullet.beginGroup();

    while (true) {
      // eslint-disable-line no-constant-condition
      // Get the parse nodes for the next row.
      parsedRows.push(parser.parseExpression(false, "\\\\"));
      parser.gullet.endGroup();
      parser.gullet.beginGroup();
      var next = parser.fetch().text;

      if (next === "&" || next === "\\\\") {
        parser.consume();
      } else if (next === "\\end") {
        if (parsedRows[parsedRows.length - 1].length === 0) {
          parsedRows.pop(); // final row ended in \\
        }

        break;
      } else {
        throw new ParseError("Expected \\\\ or \\cr or \\end", parser.nextToken);
      }
    }

    var row = [];
    var body = [row]; // Loop thru the parse nodes. Collect them into cells and arrows.

    for (var i = 0; i < parsedRows.length; i++) {
      // Start a new row.
      var rowNodes = parsedRows[i]; // Create the first cell.

      var cell = newCell();

      for (var j = 0; j < rowNodes.length; j++) {
        if (!isStartOfArrow(rowNodes[j])) {
          // If a parseNode is not an arrow, it goes into a cell.
          cell.body.push(rowNodes[j]);
        } else {
          // Parse node j is an "@", the start of an arrow.
          // Before starting on the arrow, push the cell into `row`.
          row.push(cell); // Now collect parseNodes into an arrow.
          // The character after "@" defines the arrow type.

          j += 1;
          var arrowChar = assertSymbolNodeType(rowNodes[j]).text; // Create two empty label nodes. We may or may not use them.

          var labels = new Array(2);
          labels[0] = {
            type: "ordgroup",
            mode: "math",
            body: []
          };
          labels[1] = {
            type: "ordgroup",
            mode: "math",
            body: []
          }; // Process the arrow.

          if ("=|.".indexOf(arrowChar) > -1) ; else if ("<>AV".indexOf(arrowChar) > -1) {
            // Four arrows, `@>>>`, `@<<<`, `@AAA`, and `@VVV`, each take
            // two optional labels. E.g. the right-point arrow syntax is
            // really:  @>{optional label}>{optional label}>
            // Collect parseNodes into labels.
            for (var labelNum = 0; labelNum < 2; labelNum++) {
              var inLabel = true;

              for (var k = j + 1; k < rowNodes.length; k++) {
                if (isLabelEnd(rowNodes[k], arrowChar)) {
                  inLabel = false;
                  j = k;
                  break;
                }

                if (isStartOfArrow(rowNodes[k])) {
                  throw new ParseError("Missing a " + arrowChar + " character to complete a CD arrow.", rowNodes[k]);
                }

                labels[labelNum].body.push(rowNodes[k]);
              }

              if (inLabel) {
                // isLabelEnd never returned a true.
                throw new ParseError("Missing a " + arrowChar + " character to complete a CD arrow.", rowNodes[j]);
              }
            }
          } else {
            throw new ParseError("Expected one of \"<>AV=|.\" after @", rowNodes[j]);
          } // Now join the arrow to its labels.


          var arrow = cdArrow(arrowChar, labels, parser); // Wrap the arrow in  ParseNode<"styling">.
          // This is done to match parseArray() behavior.

          var wrappedArrow = {
            type: "styling",
            body: [arrow],
            mode: "math",
            style: "display" // CD is always displaystyle.

          };
          row.push(wrappedArrow); // In CD's syntax, cells are implicit. That is, everything that
          // is not an arrow gets collected into a cell. So create an empty
          // cell now. It will collect upcoming parseNodes.

          cell = newCell();
        }
      }

      if (i % 2 === 0) {
        // Even-numbered rows consist of: cell, arrow, cell, arrow, ... cell
        // The last cell is not yet pushed into `row`, so:
        row.push(cell);
      } else {
        // Odd-numbered rows consist of: vert arrow, empty cell, ... vert arrow
        // Remove the empty cell that was placed at the beginning of `row`.
        row.shift();
      }

      row = [];
      body.push(row);
    } // End row group


    parser.gullet.endGroup(); // End array group defining \\

    parser.gullet.endGroup(); // define column separation.

    var cols = new Array(body[0].length).fill({
      type: "align",
      align: "c",
      pregap: 0.25,
      // CD package sets \enskip between columns.
      postgap: 0.25 // So pre and post each get half an \enskip, i.e. 0.25em.

    });
    return {
      type: "array",
      mode: "math",
      body,
      arraystretch: 1,
      addJot: true,
      rowGaps: [null],
      cols,
      colSeparationType: "CD",
      hLinesBeforeRow: new Array(body.length + 1).fill([])
    };
  } // The functions below are not available for general use.
  // They are here only for internal use by the {CD} environment in placing labels
  // next to vertical arrows.
  // We don't need any such functions for horizontal arrows because we can reuse
  // the functionality that already exists for extensible arrows.

  defineFunction({
    type: "cdlabel",
    names: ["\\\\cdleft", "\\\\cdright"],
    props: {
      numArgs: 1
    },

    handler(_ref, args) {
      var {
        parser,
        funcName
      } = _ref;
      return {
        type: "cdlabel",
        mode: parser.mode,
        side: funcName.slice(4),
        label: args[0]
      };
    },

    htmlBuilder(group, options) {
      var newOptions = options.havingStyle(options.style.sup());
      var label = buildCommon.wrapFragment(buildGroup$1(group.label, newOptions, options), options);
      label.classes.push("cd-label-" + group.side);
      label.style.bottom = makeEm(0.8 - label.depth); // Zero out label height & depth, so vertical align of arrow is set
      // by the arrow height, not by the label.

      label.height = 0;
      label.depth = 0;
      return label;
    },

    mathmlBuilder(group, options) {
      var label = new mathMLTree.MathNode("mrow", [buildGroup(group.label, options)]);
      label = new mathMLTree.MathNode("mpadded", [label]);
      label.setAttribute("width", "0");

      if (group.side === "left") {
        label.setAttribute("lspace", "-1width");
      } // We have to guess at vertical alignment. We know the arrow is 1.8em tall,
      // But we don't know the height or depth of the label.


      label.setAttribute("voffset", "0.7em");
      label = new mathMLTree.MathNode("mstyle", [label]);
      label.setAttribute("displaystyle", "false");
      label.setAttribute("scriptlevel", "1");
      return label;
    }

  });
  defineFunction({
    type: "cdlabelparent",
    names: ["\\\\cdparent"],
    props: {
      numArgs: 1
    },

    handler(_ref2, args) {
      var {
        parser
      } = _ref2;
      return {
        type: "cdlabelparent",
        mode: parser.mode,
        fragment: args[0]
      };
    },

    htmlBuilder(group, options) {
      // Wrap the vertical arrow and its labels.
      // The parent gets position: relative. The child gets position: absolute.
      // So CSS can locate the label correctly.
      var parent = buildCommon.wrapFragment(buildGroup$1(group.fragment, options), options);
      parent.classes.push("cd-vert-arrow");
      return parent;
    },

    mathmlBuilder(group, options) {
      return new mathMLTree.MathNode("mrow", [buildGroup(group.fragment, options)]);
    }

  });

  // {123} and converts into symbol with code 123.  It is used by the *macro*
  // \char defined in macros.js.

  defineFunction({
    type: "textord",
    names: ["\\@char"],
    props: {
      numArgs: 1,
      allowedInText: true
    },

    handler(_ref, args) {
      var {
        parser
      } = _ref;
      var arg = assertNodeType(args[0], "ordgroup");
      var group = arg.body;
      var number = "";

      for (var i = 0; i < group.length; i++) {
        var node = assertNodeType(group[i], "textord");
        number += node.text;
      }

      var code = parseInt(number);
      var text;

      if (isNaN(code)) {
        throw new ParseError("\\@char has non-numeric argument " + number); // If we drop IE support, the following code could be replaced with
        // text = String.fromCodePoint(code)
      } else if (code < 0 || code >= 0x10ffff) {
        throw new ParseError("\\@char with invalid code point " + number);
      } else if (code <= 0xffff) {
        text = String.fromCharCode(code);
      } else {
        // Astral code point; split into surrogate halves
        code -= 0x10000;
        text = String.fromCharCode((code >> 10) + 0xd800, (code & 0x3ff) + 0xdc00);
      }

      return {
        type: "textord",
        mode: parser.mode,
        text: text
      };
    }

  });

  var htmlBuilder$9 = (group, options) => {
    var elements = buildExpression$1(group.body, options.withColor(group.color), false); // \color isn't supposed to affect the type of the elements it contains.
    // To accomplish this, we wrap the results in a fragment, so the inner
    // elements will be able to directly interact with their neighbors. For
    // example, `\color{red}{2 +} 3` has the same spacing as `2 + 3`

    return buildCommon.makeFragment(elements);
  };

  var mathmlBuilder$8 = (group, options) => {
    var inner = buildExpression(group.body, options.withColor(group.color));
    var node = new mathMLTree.MathNode("mstyle", inner);
    node.setAttribute("mathcolor", group.color);
    return node;
  };

  defineFunction({
    type: "color",
    names: ["\\textcolor"],
    props: {
      numArgs: 2,
      allowedInText: true,
      argTypes: ["color", "original"]
    },

    handler(_ref, args) {
      var {
        parser
      } = _ref;
      var color = assertNodeType(args[0], "color-token").color;
      var body = args[1];
      return {
        type: "color",
        mode: parser.mode,
        color,
        body: ordargument(body)
      };
    },

    htmlBuilder: htmlBuilder$9,
    mathmlBuilder: mathmlBuilder$8
  });
  defineFunction({
    type: "color",
    names: ["\\color"],
    props: {
      numArgs: 1,
      allowedInText: true,
      argTypes: ["color"]
    },

    handler(_ref2, args) {
      var {
        parser,
        breakOnTokenText
      } = _ref2;
      var color = assertNodeType(args[0], "color-token").color; // Set macro \current@color in current namespace to store the current
      // color, mimicking the behavior of color.sty.
      // This is currently used just to correctly color a \right
      // that follows a \color command.

      parser.gullet.macros.set("\\current@color", color); // Parse out the implicit body that should be colored.

      var body = parser.parseExpression(true, breakOnTokenText);
      return {
        type: "color",
        mode: parser.mode,
        color,
        body
      };
    },

    htmlBuilder: htmlBuilder$9,
    mathmlBuilder: mathmlBuilder$8
  });

  // Row breaks within tabular environments, and line breaks at top level

  defineFunction({
    type: "cr",
    names: ["\\\\"],
    props: {
      numArgs: 0,
      numOptionalArgs: 1,
      argTypes: ["size"],
      allowedInText: true
    },

    handler(_ref, args, optArgs) {
      var {
        parser
      } = _ref;
      var size = optArgs[0];
      var newLine = !parser.settings.displayMode || !parser.settings.useStrictBehavior("newLineInDisplayMode", "In LaTeX, \\\\ or \\newline " + "does nothing in display mode");
      return {
        type: "cr",
        mode: parser.mode,
        newLine,
        size: size && assertNodeType(size, "size").value
      };
    },

    // The following builders are called only at the top level,
    // not within tabular/array environments.
    htmlBuilder(group, options) {
      var span = buildCommon.makeSpan(["mspace"], [], options);

      if (group.newLine) {
        span.classes.push("newline");

        if (group.size) {
          span.style.marginTop = makeEm(calculateSize(group.size, options));
        }
      }

      return span;
    },

    mathmlBuilder(group, options) {
      var node = new mathMLTree.MathNode("mspace");

      if (group.newLine) {
        node.setAttribute("linebreak", "newline");

        if (group.size) {
          node.setAttribute("height", makeEm(calculateSize(group.size, options)));
        }
      }

      return node;
    }

  });

  var globalMap = {
    "\\global": "\\global",
    "\\long": "\\\\globallong",
    "\\\\globallong": "\\\\globallong",
    "\\def": "\\gdef",
    "\\gdef": "\\gdef",
    "\\edef": "\\xdef",
    "\\xdef": "\\xdef",
    "\\let": "\\\\globallet",
    "\\futurelet": "\\\\globalfuture"
  };

  var checkControlSequence = tok => {
    var name = tok.text;

    if (/^(?:[\\{}$&#^_]|EOF)$/.test(name)) {
      throw new ParseError("Expected a control sequence", tok);
    }

    return name;
  };

  var getRHS = parser => {
    var tok = parser.gullet.popToken();

    if (tok.text === "=") {
      // consume optional equals
      tok = parser.gullet.popToken();

      if (tok.text === " ") {
        // consume one optional space
        tok = parser.gullet.popToken();
      }
    }

    return tok;
  };

  var letCommand = (parser, name, tok, global) => {
    var macro = parser.gullet.macros.get(tok.text);

    if (macro == null) {
      // don't expand it later even if a macro with the same name is defined
      // e.g., \let\foo=\frac \def\frac{\relax} \frac12
      tok.noexpand = true;
      macro = {
        tokens: [tok],
        numArgs: 0,
        // reproduce the same behavior in expansion
        unexpandable: !parser.gullet.isExpandable(tok.text)
      };
    }

    parser.gullet.macros.set(name, macro, global);
  }; // <assignment> -> <non-macro assignment>|<macro assignment>
  // <non-macro assignment> -> <simple assignment>|\global<non-macro assignment>
  // <macro assignment> -> <definition>|<prefix><macro assignment>
  // <prefix> -> \global|\long|\outer


  defineFunction({
    type: "internal",
    names: ["\\global", "\\long", "\\\\globallong" // can’t be entered directly
    ],
    props: {
      numArgs: 0,
      allowedInText: true
    },

    handler(_ref) {
      var {
        parser,
        funcName
      } = _ref;
      parser.consumeSpaces();
      var token = parser.fetch();

      if (globalMap[token.text]) {
        // KaTeX doesn't have \par, so ignore \long
        if (funcName === "\\global" || funcName === "\\\\globallong") {
          token.text = globalMap[token.text];
        }

        return assertNodeType(parser.parseFunction(), "internal");
      }

      throw new ParseError("Invalid token after macro prefix", token);
    }

  }); // Basic support for macro definitions: \def, \gdef, \edef, \xdef
  // <definition> -> <def><control sequence><definition text>
  // <def> -> \def|\gdef|\edef|\xdef
  // <definition text> -> <parameter text><left brace><balanced text><right brace>

  defineFunction({
    type: "internal",
    names: ["\\def", "\\gdef", "\\edef", "\\xdef"],
    props: {
      numArgs: 0,
      allowedInText: true,
      primitive: true
    },

    handler(_ref2) {
      var {
        parser,
        funcName
      } = _ref2;
      var tok = parser.gullet.popToken();
      var name = tok.text;

      if (/^(?:[\\{}$&#^_]|EOF)$/.test(name)) {
        throw new ParseError("Expected a control sequence", tok);
      }

      var numArgs = 0;
      var insert;
      var delimiters = [[]]; // <parameter text> contains no braces

      while (parser.gullet.future().text !== "{") {
        tok = parser.gullet.popToken();

        if (tok.text === "#") {
          // If the very last character of the <parameter text> is #, so that
          // this # is immediately followed by {, TeX will behave as if the {
          // had been inserted at the right end of both the parameter text
          // and the replacement text.
          if (parser.gullet.future().text === "{") {
            insert = parser.gullet.future();
            delimiters[numArgs].push("{");
            break;
          } // A parameter, the first appearance of # must be followed by 1,
          // the next by 2, and so on; up to nine #’s are allowed


          tok = parser.gullet.popToken();

          if (!/^[1-9]$/.test(tok.text)) {
            throw new ParseError("Invalid argument number \"" + tok.text + "\"");
          }

          if (parseInt(tok.text) !== numArgs + 1) {
            throw new ParseError("Argument number \"" + tok.text + "\" out of order");
          }

          numArgs++;
          delimiters.push([]);
        } else if (tok.text === "EOF") {
          throw new ParseError("Expected a macro definition");
        } else {
          delimiters[numArgs].push(tok.text);
        }
      } // replacement text, enclosed in '{' and '}' and properly nested


      var {
        tokens
      } = parser.gullet.consumeArg();

      if (insert) {
        tokens.unshift(insert);
      }

      if (funcName === "\\edef" || funcName === "\\xdef") {
        tokens = parser.gullet.expandTokens(tokens);
        tokens.reverse(); // to fit in with stack order
      } // Final arg is the expansion of the macro


      parser.gullet.macros.set(name, {
        tokens,
        numArgs,
        delimiters
      }, funcName === globalMap[funcName]);
      return {
        type: "internal",
        mode: parser.mode
      };
    }

  }); // <simple assignment> -> <let assignment>
  // <let assignment> -> \futurelet<control sequence><token><token>
  //     | \let<control sequence><equals><one optional space><token>
  // <equals> -> <optional spaces>|<optional spaces>=

  defineFunction({
    type: "internal",
    names: ["\\let", "\\\\globallet" // can’t be entered directly
    ],
    props: {
      numArgs: 0,
      allowedInText: true,
      primitive: true
    },

    handler(_ref3) {
      var {
        parser,
        funcName
      } = _ref3;
      var name = checkControlSequence(parser.gullet.popToken());
      parser.gullet.consumeSpaces();
      var tok = getRHS(parser);
      letCommand(parser, name, tok, funcName === "\\\\globallet");
      return {
        type: "internal",
        mode: parser.mode
      };
    }

  }); // ref: https://www.tug.org/TUGboat/tb09-3/tb22bechtolsheim.pdf

  defineFunction({
    type: "internal",
    names: ["\\futurelet", "\\\\globalfuture" // can’t be entered directly
    ],
    props: {
      numArgs: 0,
      allowedInText: true,
      primitive: true
    },

    handler(_ref4) {
      var {
        parser,
        funcName
      } = _ref4;
      var name = checkControlSequence(parser.gullet.popToken());
      var middle = parser.gullet.popToken();
      var tok = parser.gullet.popToken();
      letCommand(parser, name, tok, funcName === "\\\\globalfuture");
      parser.gullet.pushToken(tok);
      parser.gullet.pushToken(middle);
      return {
        type: "internal",
        mode: parser.mode
      };
    }

  });

  /**
   * This file deals with creating delimiters of various sizes. The TeXbook
   * discusses these routines on page 441-442, in the "Another subroutine sets box
   * x to a specified variable delimiter" paragraph.
   *
   * There are three main routines here. `makeSmallDelim` makes a delimiter in the
   * normal font, but in either text, script, or scriptscript style.
   * `makeLargeDelim` makes a delimiter in textstyle, but in one of the Size1,
   * Size2, Size3, or Size4 fonts. `makeStackedDelim` makes a delimiter out of
   * smaller pieces that are stacked on top of one another.
   *
   * The functions take a parameter `center`, which determines if the delimiter
   * should be centered around the axis.
   *
   * Then, there are three exposed functions. `sizedDelim` makes a delimiter in
   * one of the given sizes. This is used for things like `\bigl`.
   * `customSizedDelim` makes a delimiter with a given total height+depth. It is
   * called in places like `\sqrt`. `leftRightDelim` makes an appropriate
   * delimiter which surrounds an expression of a given height an depth. It is
   * used in `\left` and `\right`.
   */

  /**
   * Get the metrics for a given symbol and font, after transformation (i.e.
   * after following replacement from symbols.js)
   */
  var getMetrics = function getMetrics(symbol, font, mode) {
    var replace = symbols.math[symbol] && symbols.math[symbol].replace;
    var metrics = getCharacterMetrics(replace || symbol, font, mode);

    if (!metrics) {
      throw new Error("Unsupported symbol " + symbol + " and font size " + font + ".");
    }

    return metrics;
  };
  /**
   * Puts a delimiter span in a given style, and adds appropriate height, depth,
   * and maxFontSizes.
   */


  var styleWrap = function styleWrap(delim, toStyle, options, classes) {
    var newOptions = options.havingBaseStyle(toStyle);
    var span = buildCommon.makeSpan(classes.concat(newOptions.sizingClasses(options)), [delim], options);
    var delimSizeMultiplier = newOptions.sizeMultiplier / options.sizeMultiplier;
    span.height *= delimSizeMultiplier;
    span.depth *= delimSizeMultiplier;
    span.maxFontSize = newOptions.sizeMultiplier;
    return span;
  };

  var centerSpan = function centerSpan(span, options, style) {
    var newOptions = options.havingBaseStyle(style);
    var shift = (1 - options.sizeMultiplier / newOptions.sizeMultiplier) * options.fontMetrics().axisHeight;
    span.classes.push("delimcenter");
    span.style.top = makeEm(shift);
    span.height -= shift;
    span.depth += shift;
  };
  /**
   * Makes a small delimiter. This is a delimiter that comes in the Main-Regular
   * font, but is restyled to either be in textstyle, scriptstyle, or
   * scriptscriptstyle.
   */


  var makeSmallDelim = function makeSmallDelim(delim, style, center, options, mode, classes) {
    var text = buildCommon.makeSymbol(delim, "Main-Regular", mode, options);
    var span = styleWrap(text, style, options, classes);

    if (center) {
      centerSpan(span, options, style);
    }

    return span;
  };
  /**
   * Builds a symbol in the given font size (note size is an integer)
   */


  var mathrmSize = function mathrmSize(value, size, mode, options) {
    return buildCommon.makeSymbol(value, "Size" + size + "-Regular", mode, options);
  };
  /**
   * Makes a large delimiter. This is a delimiter that comes in the Size1, Size2,
   * Size3, or Size4 fonts. It is always rendered in textstyle.
   */


  var makeLargeDelim = function makeLargeDelim(delim, size, center, options, mode, classes) {
    var inner = mathrmSize(delim, size, mode, options);
    var span = styleWrap(buildCommon.makeSpan(["delimsizing", "size" + size], [inner], options), Style$1.TEXT, options, classes);

    if (center) {
      centerSpan(span, options, Style$1.TEXT);
    }

    return span;
  };
  /**
   * Make a span from a font glyph with the given offset and in the given font.
   * This is used in makeStackedDelim to make the stacking pieces for the delimiter.
   */


  var makeGlyphSpan = function makeGlyphSpan(symbol, font, mode) {
    var sizeClass; // Apply the correct CSS class to choose the right font.

    if (font === "Size1-Regular") {
      sizeClass = "delim-size1";
    } else
      /* if (font === "Size4-Regular") */
      {
        sizeClass = "delim-size4";
      }

    var corner = buildCommon.makeSpan(["delimsizinginner", sizeClass], [buildCommon.makeSpan([], [buildCommon.makeSymbol(symbol, font, mode)])]); // Since this will be passed into `makeVList` in the end, wrap the element
    // in the appropriate tag that VList uses.

    return {
      type: "elem",
      elem: corner
    };
  };

  var makeInner = function makeInner(ch, height, options) {
    // Create a span with inline SVG for the inner part of a tall stacked delimiter.
    var width = fontMetricsData['Size4-Regular'][ch.charCodeAt(0)] ? fontMetricsData['Size4-Regular'][ch.charCodeAt(0)][4] : fontMetricsData['Size1-Regular'][ch.charCodeAt(0)][4];
    var path = new PathNode("inner", innerPath(ch, Math.round(1000 * height)));
    var svgNode = new SvgNode([path], {
      "width": makeEm(width),
      "height": makeEm(height),
      // Override CSS rule `.katex svg { width: 100% }`
      "style": "width:" + makeEm(width),
      "viewBox": "0 0 " + 1000 * width + " " + Math.round(1000 * height),
      "preserveAspectRatio": "xMinYMin"
    });
    var span = buildCommon.makeSvgSpan([], [svgNode], options);
    span.height = height;
    span.style.height = makeEm(height);
    span.style.width = makeEm(width);
    return {
      type: "elem",
      elem: span
    };
  }; // Helpers for makeStackedDelim


  var lapInEms = 0.008;
  var lap = {
    type: "kern",
    size: -1 * lapInEms
  };
  var verts = ["|", "\\lvert", "\\rvert", "\\vert"];
  var doubleVerts = ["\\|", "\\lVert", "\\rVert", "\\Vert"];
  /**
   * Make a stacked delimiter out of a given delimiter, with the total height at
   * least `heightTotal`. This routine is mentioned on page 442 of the TeXbook.
   */

  var makeStackedDelim = function makeStackedDelim(delim, heightTotal, center, options, mode, classes) {
    // There are four parts, the top, an optional middle, a repeated part, and a
    // bottom.
    var top;
    var middle;
    var repeat;
    var bottom;
    top = repeat = bottom = delim;
    middle = null; // Also keep track of what font the delimiters are in

    var font = "Size1-Regular"; // We set the parts and font based on the symbol. Note that we use
    // '\u23d0' instead of '|' and '\u2016' instead of '\\|' for the
    // repeats of the arrows

    if (delim === "\\uparrow") {
      repeat = bottom = "\u23d0";
    } else if (delim === "\\Uparrow") {
      repeat = bottom = "\u2016";
    } else if (delim === "\\downarrow") {
      top = repeat = "\u23d0";
    } else if (delim === "\\Downarrow") {
      top = repeat = "\u2016";
    } else if (delim === "\\updownarrow") {
      top = "\\uparrow";
      repeat = "\u23d0";
      bottom = "\\downarrow";
    } else if (delim === "\\Updownarrow") {
      top = "\\Uparrow";
      repeat = "\u2016";
      bottom = "\\Downarrow";
    } else if (utils$1.contains(verts, delim)) {
      repeat = "\u2223";
    } else if (utils$1.contains(doubleVerts, delim)) {
      repeat = "\u2225";
    } else if (delim === "[" || delim === "\\lbrack") {
      top = "\u23a1";
      repeat = "\u23a2";
      bottom = "\u23a3";
      font = "Size4-Regular";
    } else if (delim === "]" || delim === "\\rbrack") {
      top = "\u23a4";
      repeat = "\u23a5";
      bottom = "\u23a6";
      font = "Size4-Regular";
    } else if (delim === "\\lfloor" || delim === "\u230a") {
      repeat = top = "\u23a2";
      bottom = "\u23a3";
      font = "Size4-Regular";
    } else if (delim === "\\lceil" || delim === "\u2308") {
      top = "\u23a1";
      repeat = bottom = "\u23a2";
      font = "Size4-Regular";
    } else if (delim === "\\rfloor" || delim === "\u230b") {
      repeat = top = "\u23a5";
      bottom = "\u23a6";
      font = "Size4-Regular";
    } else if (delim === "\\rceil" || delim === "\u2309") {
      top = "\u23a4";
      repeat = bottom = "\u23a5";
      font = "Size4-Regular";
    } else if (delim === "(" || delim === "\\lparen") {
      top = "\u239b";
      repeat = "\u239c";
      bottom = "\u239d";
      font = "Size4-Regular";
    } else if (delim === ")" || delim === "\\rparen") {
      top = "\u239e";
      repeat = "\u239f";
      bottom = "\u23a0";
      font = "Size4-Regular";
    } else if (delim === "\\{" || delim === "\\lbrace") {
      top = "\u23a7";
      middle = "\u23a8";
      bottom = "\u23a9";
      repeat = "\u23aa";
      font = "Size4-Regular";
    } else if (delim === "\\}" || delim === "\\rbrace") {
      top = "\u23ab";
      middle = "\u23ac";
      bottom = "\u23ad";
      repeat = "\u23aa";
      font = "Size4-Regular";
    } else if (delim === "\\lgroup" || delim === "\u27ee") {
      top = "\u23a7";
      bottom = "\u23a9";
      repeat = "\u23aa";
      font = "Size4-Regular";
    } else if (delim === "\\rgroup" || delim === "\u27ef") {
      top = "\u23ab";
      bottom = "\u23ad";
      repeat = "\u23aa";
      font = "Size4-Regular";
    } else if (delim === "\\lmoustache" || delim === "\u23b0") {
      top = "\u23a7";
      bottom = "\u23ad";
      repeat = "\u23aa";
      font = "Size4-Regular";
    } else if (delim === "\\rmoustache" || delim === "\u23b1") {
      top = "\u23ab";
      bottom = "\u23a9";
      repeat = "\u23aa";
      font = "Size4-Regular";
    } // Get the metrics of the four sections


    var topMetrics = getMetrics(top, font, mode);
    var topHeightTotal = topMetrics.height + topMetrics.depth;
    var repeatMetrics = getMetrics(repeat, font, mode);
    var repeatHeightTotal = repeatMetrics.height + repeatMetrics.depth;
    var bottomMetrics = getMetrics(bottom, font, mode);
    var bottomHeightTotal = bottomMetrics.height + bottomMetrics.depth;
    var middleHeightTotal = 0;
    var middleFactor = 1;

    if (middle !== null) {
      var middleMetrics = getMetrics(middle, font, mode);
      middleHeightTotal = middleMetrics.height + middleMetrics.depth;
      middleFactor = 2; // repeat symmetrically above and below middle
    } // Calcuate the minimal height that the delimiter can have.
    // It is at least the size of the top, bottom, and optional middle combined.


    var minHeight = topHeightTotal + bottomHeightTotal + middleHeightTotal; // Compute the number of copies of the repeat symbol we will need

    var repeatCount = Math.max(0, Math.ceil((heightTotal - minHeight) / (middleFactor * repeatHeightTotal))); // Compute the total height of the delimiter including all the symbols

    var realHeightTotal = minHeight + repeatCount * middleFactor * repeatHeightTotal; // The center of the delimiter is placed at the center of the axis. Note
    // that in this context, "center" means that the delimiter should be
    // centered around the axis in the current style, while normally it is
    // centered around the axis in textstyle.

    var axisHeight = options.fontMetrics().axisHeight;

    if (center) {
      axisHeight *= options.sizeMultiplier;
    } // Calculate the depth


    var depth = realHeightTotal / 2 - axisHeight; // Now, we start building the pieces that will go into the vlist
    // Keep a list of the pieces of the stacked delimiter

    var stack = []; // Add the bottom symbol

    stack.push(makeGlyphSpan(bottom, font, mode));
    stack.push(lap); // overlap

    if (middle === null) {
      // The middle section will be an SVG. Make it an extra 0.016em tall.
      // We'll overlap by 0.008em at top and bottom.
      var innerHeight = realHeightTotal - topHeightTotal - bottomHeightTotal + 2 * lapInEms;
      stack.push(makeInner(repeat, innerHeight, options));
    } else {
      // When there is a middle bit, we need the middle part and two repeated
      // sections
      var _innerHeight = (realHeightTotal - topHeightTotal - bottomHeightTotal - middleHeightTotal) / 2 + 2 * lapInEms;

      stack.push(makeInner(repeat, _innerHeight, options)); // Now insert the middle of the brace.

      stack.push(lap);
      stack.push(makeGlyphSpan(middle, font, mode));
      stack.push(lap);
      stack.push(makeInner(repeat, _innerHeight, options));
    } // Add the top symbol


    stack.push(lap);
    stack.push(makeGlyphSpan(top, font, mode)); // Finally, build the vlist

    var newOptions = options.havingBaseStyle(Style$1.TEXT);
    var inner = buildCommon.makeVList({
      positionType: "bottom",
      positionData: depth,
      children: stack
    }, newOptions);
    return styleWrap(buildCommon.makeSpan(["delimsizing", "mult"], [inner], newOptions), Style$1.TEXT, options, classes);
  }; // All surds have 0.08em padding above the viniculum inside the SVG.
  // That keeps browser span height rounding error from pinching the line.


  var vbPad = 80; // padding above the surd, measured inside the viewBox.

  var emPad = 0.08; // padding, in ems, measured in the document.

  var sqrtSvg = function sqrtSvg(sqrtName, height, viewBoxHeight, extraViniculum, options) {
    var path = sqrtPath(sqrtName, extraViniculum, viewBoxHeight);
    var pathNode = new PathNode(sqrtName, path);
    var svg = new SvgNode([pathNode], {
      // Note: 1000:1 ratio of viewBox to document em width.
      "width": "400em",
      "height": makeEm(height),
      "viewBox": "0 0 400000 " + viewBoxHeight,
      "preserveAspectRatio": "xMinYMin slice"
    });
    return buildCommon.makeSvgSpan(["hide-tail"], [svg], options);
  };
  /**
   * Make a sqrt image of the given height,
   */


  var makeSqrtImage = function makeSqrtImage(height, options) {
    // Define a newOptions that removes the effect of size changes such as \Huge.
    // We don't pick different a height surd for \Huge. For it, we scale up.
    var newOptions = options.havingBaseSizing(); // Pick the desired surd glyph from a sequence of surds.

    var delim = traverseSequence("\\surd", height * newOptions.sizeMultiplier, stackLargeDelimiterSequence, newOptions);
    var sizeMultiplier = newOptions.sizeMultiplier; // default
    // The standard sqrt SVGs each have a 0.04em thick viniculum.
    // If Settings.minRuleThickness is larger than that, we add extraViniculum.

    var extraViniculum = Math.max(0, options.minRuleThickness - options.fontMetrics().sqrtRuleThickness); // Create a span containing an SVG image of a sqrt symbol.

    var span;
    var spanHeight = 0;
    var texHeight = 0;
    var viewBoxHeight = 0;
    var advanceWidth; // We create viewBoxes with 80 units of "padding" above each surd.
    // Then browser rounding error on the parent span height will not
    // encroach on the ink of the viniculum. But that padding is not
    // included in the TeX-like `height` used for calculation of
    // vertical alignment. So texHeight = span.height < span.style.height.

    if (delim.type === "small") {
      // Get an SVG that is derived from glyph U+221A in font KaTeX-Main.
      // 1000 unit normal glyph height.
      viewBoxHeight = 1000 + 1000 * extraViniculum + vbPad;

      if (height < 1.0) {
        sizeMultiplier = 1.0; // mimic a \textfont radical
      } else if (height < 1.4) {
        sizeMultiplier = 0.7; // mimic a \scriptfont radical
      }

      spanHeight = (1.0 + extraViniculum + emPad) / sizeMultiplier;
      texHeight = (1.00 + extraViniculum) / sizeMultiplier;
      span = sqrtSvg("sqrtMain", spanHeight, viewBoxHeight, extraViniculum, options);
      span.style.minWidth = "0.853em";
      advanceWidth = 0.833 / sizeMultiplier; // from the font.
    } else if (delim.type === "large") {
      // These SVGs come from fonts: KaTeX_Size1, _Size2, etc.
      viewBoxHeight = (1000 + vbPad) * sizeToMaxHeight[delim.size];
      texHeight = (sizeToMaxHeight[delim.size] + extraViniculum) / sizeMultiplier;
      spanHeight = (sizeToMaxHeight[delim.size] + extraViniculum + emPad) / sizeMultiplier;
      span = sqrtSvg("sqrtSize" + delim.size, spanHeight, viewBoxHeight, extraViniculum, options);
      span.style.minWidth = "1.02em";
      advanceWidth = 1.0 / sizeMultiplier; // 1.0 from the font.
    } else {
      // Tall sqrt. In TeX, this would be stacked using multiple glyphs.
      // We'll use a single SVG to accomplish the same thing.
      spanHeight = height + extraViniculum + emPad;
      texHeight = height + extraViniculum;
      viewBoxHeight = Math.floor(1000 * height + extraViniculum) + vbPad;
      span = sqrtSvg("sqrtTall", spanHeight, viewBoxHeight, extraViniculum, options);
      span.style.minWidth = "0.742em";
      advanceWidth = 1.056;
    }

    span.height = texHeight;
    span.style.height = makeEm(spanHeight);
    return {
      span,
      advanceWidth,
      // Calculate the actual line width.
      // This actually should depend on the chosen font -- e.g. \boldmath
      // should use the thicker surd symbols from e.g. KaTeX_Main-Bold, and
      // have thicker rules.
      ruleWidth: (options.fontMetrics().sqrtRuleThickness + extraViniculum) * sizeMultiplier
    };
  }; // There are three kinds of delimiters, delimiters that stack when they become
  // too large


  var stackLargeDelimiters = ["(", "\\lparen", ")", "\\rparen", "[", "\\lbrack", "]", "\\rbrack", "\\{", "\\lbrace", "\\}", "\\rbrace", "\\lfloor", "\\rfloor", "\u230a", "\u230b", "\\lceil", "\\rceil", "\u2308", "\u2309", "\\surd"]; // delimiters that always stack

  var stackAlwaysDelimiters = ["\\uparrow", "\\downarrow", "\\updownarrow", "\\Uparrow", "\\Downarrow", "\\Updownarrow", "|", "\\|", "\\vert", "\\Vert", "\\lvert", "\\rvert", "\\lVert", "\\rVert", "\\lgroup", "\\rgroup", "\u27ee", "\u27ef", "\\lmoustache", "\\rmoustache", "\u23b0", "\u23b1"]; // and delimiters that never stack

  var stackNeverDelimiters = ["<", ">", "\\langle", "\\rangle", "/", "\\backslash", "\\lt", "\\gt"]; // Metrics of the different sizes. Found by looking at TeX's output of
  // $\bigl| // \Bigl| \biggl| \Biggl| \showlists$
  // Used to create stacked delimiters of appropriate sizes in makeSizedDelim.

  var sizeToMaxHeight = [0, 1.2, 1.8, 2.4, 3.0];
  /**
   * Used to create a delimiter of a specific size, where `size` is 1, 2, 3, or 4.
   */

  var makeSizedDelim = function makeSizedDelim(delim, size, options, mode, classes) {
    // < and > turn into \langle and \rangle in delimiters
    if (delim === "<" || delim === "\\lt" || delim === "\u27e8") {
      delim = "\\langle";
    } else if (delim === ">" || delim === "\\gt" || delim === "\u27e9") {
      delim = "\\rangle";
    } // Sized delimiters are never centered.


    if (utils$1.contains(stackLargeDelimiters, delim) || utils$1.contains(stackNeverDelimiters, delim)) {
      return makeLargeDelim(delim, size, false, options, mode, classes);
    } else if (utils$1.contains(stackAlwaysDelimiters, delim)) {
      return makeStackedDelim(delim, sizeToMaxHeight[size], false, options, mode, classes);
    } else {
      throw new ParseError("Illegal delimiter: '" + delim + "'");
    }
  };
  /**
   * There are three different sequences of delimiter sizes that the delimiters
   * follow depending on the kind of delimiter. This is used when creating custom
   * sized delimiters to decide whether to create a small, large, or stacked
   * delimiter.
   *
   * In real TeX, these sequences aren't explicitly defined, but are instead
   * defined inside the font metrics. Since there are only three sequences that
   * are possible for the delimiters that TeX defines, it is easier to just encode
   * them explicitly here.
   */


  // Delimiters that never stack try small delimiters and large delimiters only
  var stackNeverDelimiterSequence = [{
    type: "small",
    style: Style$1.SCRIPTSCRIPT
  }, {
    type: "small",
    style: Style$1.SCRIPT
  }, {
    type: "small",
    style: Style$1.TEXT
  }, {
    type: "large",
    size: 1
  }, {
    type: "large",
    size: 2
  }, {
    type: "large",
    size: 3
  }, {
    type: "large",
    size: 4
  }]; // Delimiters that always stack try the small delimiters first, then stack

  var stackAlwaysDelimiterSequence = [{
    type: "small",
    style: Style$1.SCRIPTSCRIPT
  }, {
    type: "small",
    style: Style$1.SCRIPT
  }, {
    type: "small",
    style: Style$1.TEXT
  }, {
    type: "stack"
  }]; // Delimiters that stack when large try the small and then large delimiters, and
  // stack afterwards

  var stackLargeDelimiterSequence = [{
    type: "small",
    style: Style$1.SCRIPTSCRIPT
  }, {
    type: "small",
    style: Style$1.SCRIPT
  }, {
    type: "small",
    style: Style$1.TEXT
  }, {
    type: "large",
    size: 1
  }, {
    type: "large",
    size: 2
  }, {
    type: "large",
    size: 3
  }, {
    type: "large",
    size: 4
  }, {
    type: "stack"
  }];
  /**
   * Get the font used in a delimiter based on what kind of delimiter it is.
   * TODO(#963) Use more specific font family return type once that is introduced.
   */

  var delimTypeToFont = function delimTypeToFont(type) {
    if (type.type === "small") {
      return "Main-Regular";
    } else if (type.type === "large") {
      return "Size" + type.size + "-Regular";
    } else if (type.type === "stack") {
      return "Size4-Regular";
    } else {
      throw new Error("Add support for delim type '" + type.type + "' here.");
    }
  };
  /**
   * Traverse a sequence of types of delimiters to decide what kind of delimiter
   * should be used to create a delimiter of the given height+depth.
   */


  var traverseSequence = function traverseSequence(delim, height, sequence, options) {
    // Here, we choose the index we should start at in the sequences. In smaller
    // sizes (which correspond to larger numbers in style.size) we start earlier
    // in the sequence. Thus, scriptscript starts at index 3-3=0, script starts
    // at index 3-2=1, text starts at 3-1=2, and display starts at min(2,3-0)=2
    var start = Math.min(2, 3 - options.style.size);

    for (var i = start; i < sequence.length; i++) {
      if (sequence[i].type === "stack") {
        // This is always the last delimiter, so we just break the loop now.
        break;
      }

      var metrics = getMetrics(delim, delimTypeToFont(sequence[i]), "math");
      var heightDepth = metrics.height + metrics.depth; // Small delimiters are scaled down versions of the same font, so we
      // account for the style change size.

      if (sequence[i].type === "small") {
        var newOptions = options.havingBaseStyle(sequence[i].style);
        heightDepth *= newOptions.sizeMultiplier;
      } // Check if the delimiter at this size works for the given height.


      if (heightDepth > height) {
        return sequence[i];
      }
    } // If we reached the end of the sequence, return the last sequence element.


    return sequence[sequence.length - 1];
  };
  /**
   * Make a delimiter of a given height+depth, with optional centering. Here, we
   * traverse the sequences, and create a delimiter that the sequence tells us to.
   */


  var makeCustomSizedDelim = function makeCustomSizedDelim(delim, height, center, options, mode, classes) {
    if (delim === "<" || delim === "\\lt" || delim === "\u27e8") {
      delim = "\\langle";
    } else if (delim === ">" || delim === "\\gt" || delim === "\u27e9") {
      delim = "\\rangle";
    } // Decide what sequence to use


    var sequence;

    if (utils$1.contains(stackNeverDelimiters, delim)) {
      sequence = stackNeverDelimiterSequence;
    } else if (utils$1.contains(stackLargeDelimiters, delim)) {
      sequence = stackLargeDelimiterSequence;
    } else {
      sequence = stackAlwaysDelimiterSequence;
    } // Look through the sequence


    var delimType = traverseSequence(delim, height, sequence, options); // Get the delimiter from font glyphs.
    // Depending on the sequence element we decided on, call the
    // appropriate function.

    if (delimType.type === "small") {
      return makeSmallDelim(delim, delimType.style, center, options, mode, classes);
    } else if (delimType.type === "large") {
      return makeLargeDelim(delim, delimType.size, center, options, mode, classes);
    } else
      /* if (delimType.type === "stack") */
      {
        return makeStackedDelim(delim, height, center, options, mode, classes);
      }
  };
  /**
   * Make a delimiter for use with `\left` and `\right`, given a height and depth
   * of an expression that the delimiters surround.
   */


  var makeLeftRightDelim = function makeLeftRightDelim(delim, height, depth, options, mode, classes) {
    // We always center \left/\right delimiters, so the axis is always shifted
    var axisHeight = options.fontMetrics().axisHeight * options.sizeMultiplier; // Taken from TeX source, tex.web, function make_left_right

    var delimiterFactor = 901;
    var delimiterExtend = 5.0 / options.fontMetrics().ptPerEm;
    var maxDistFromAxis = Math.max(height - axisHeight, depth + axisHeight);
    var totalHeight = Math.max( // In real TeX, calculations are done using integral values which are
    // 65536 per pt, or 655360 per em. So, the division here truncates in
    // TeX but doesn't here, producing different results. If we wanted to
    // exactly match TeX's calculation, we could do
    //   Math.floor(655360 * maxDistFromAxis / 500) *
    //    delimiterFactor / 655360
    // (To see the difference, compare
    //    x^{x^{\left(\rule{0.1em}{0.68em}\right)}}
    // in TeX and KaTeX)
    maxDistFromAxis / 500 * delimiterFactor, 2 * maxDistFromAxis - delimiterExtend); // Finally, we defer to `makeCustomSizedDelim` with our calculated total
    // height

    return makeCustomSizedDelim(delim, totalHeight, true, options, mode, classes);
  };

  var delimiter = {
    sqrtImage: makeSqrtImage,
    sizedDelim: makeSizedDelim,
    sizeToMaxHeight: sizeToMaxHeight,
    customSizedDelim: makeCustomSizedDelim,
    leftRightDelim: makeLeftRightDelim
  };

  // Extra data needed for the delimiter handler down below
  var delimiterSizes = {
    "\\bigl": {
      mclass: "mopen",
      size: 1
    },
    "\\Bigl": {
      mclass: "mopen",
      size: 2
    },
    "\\biggl": {
      mclass: "mopen",
      size: 3
    },
    "\\Biggl": {
      mclass: "mopen",
      size: 4
    },
    "\\bigr": {
      mclass: "mclose",
      size: 1
    },
    "\\Bigr": {
      mclass: "mclose",
      size: 2
    },
    "\\biggr": {
      mclass: "mclose",
      size: 3
    },
    "\\Biggr": {
      mclass: "mclose",
      size: 4
    },
    "\\bigm": {
      mclass: "mrel",
      size: 1
    },
    "\\Bigm": {
      mclass: "mrel",
      size: 2
    },
    "\\biggm": {
      mclass: "mrel",
      size: 3
    },
    "\\Biggm": {
      mclass: "mrel",
      size: 4
    },
    "\\big": {
      mclass: "mord",
      size: 1
    },
    "\\Big": {
      mclass: "mord",
      size: 2
    },
    "\\bigg": {
      mclass: "mord",
      size: 3
    },
    "\\Bigg": {
      mclass: "mord",
      size: 4
    }
  };
  var delimiters = ["(", "\\lparen", ")", "\\rparen", "[", "\\lbrack", "]", "\\rbrack", "\\{", "\\lbrace", "\\}", "\\rbrace", "\\lfloor", "\\rfloor", "\u230a", "\u230b", "\\lceil", "\\rceil", "\u2308", "\u2309", "<", ">", "\\langle", "\u27e8", "\\rangle", "\u27e9", "\\lt", "\\gt", "\\lvert", "\\rvert", "\\lVert", "\\rVert", "\\lgroup", "\\rgroup", "\u27ee", "\u27ef", "\\lmoustache", "\\rmoustache", "\u23b0", "\u23b1", "/", "\\backslash", "|", "\\vert", "\\|", "\\Vert", "\\uparrow", "\\Uparrow", "\\downarrow", "\\Downarrow", "\\updownarrow", "\\Updownarrow", "."];

  // Delimiter functions
  function checkDelimiter(delim, context) {
    var symDelim = checkSymbolNodeType(delim);

    if (symDelim && utils$1.contains(delimiters, symDelim.text)) {
      return symDelim;
    } else if (symDelim) {
      throw new ParseError("Invalid delimiter '" + symDelim.text + "' after '" + context.funcName + "'", delim);
    } else {
      throw new ParseError("Invalid delimiter type '" + delim.type + "'", delim);
    }
  }

  defineFunction({
    type: "delimsizing",
    names: ["\\bigl", "\\Bigl", "\\biggl", "\\Biggl", "\\bigr", "\\Bigr", "\\biggr", "\\Biggr", "\\bigm", "\\Bigm", "\\biggm", "\\Biggm", "\\big", "\\Big", "\\bigg", "\\Bigg"],
    props: {
      numArgs: 1,
      argTypes: ["primitive"]
    },
    handler: (context, args) => {
      var delim = checkDelimiter(args[0], context);
      return {
        type: "delimsizing",
        mode: context.parser.mode,
        size: delimiterSizes[context.funcName].size,
        mclass: delimiterSizes[context.funcName].mclass,
        delim: delim.text
      };
    },
    htmlBuilder: (group, options) => {
      if (group.delim === ".") {
        // Empty delimiters still count as elements, even though they don't
        // show anything.
        return buildCommon.makeSpan([group.mclass]);
      } // Use delimiter.sizedDelim to generate the delimiter.


      return delimiter.sizedDelim(group.delim, group.size, options, group.mode, [group.mclass]);
    },
    mathmlBuilder: group => {
      var children = [];

      if (group.delim !== ".") {
        children.push(makeText(group.delim, group.mode));
      }

      var node = new mathMLTree.MathNode("mo", children);

      if (group.mclass === "mopen" || group.mclass === "mclose") {
        // Only some of the delimsizing functions act as fences, and they
        // return "mopen" or "mclose" mclass.
        node.setAttribute("fence", "true");
      } else {
        // Explicitly disable fencing if it's not a fence, to override the
        // defaults.
        node.setAttribute("fence", "false");
      }

      node.setAttribute("stretchy", "true");
      var size = makeEm(delimiter.sizeToMaxHeight[group.size]);
      node.setAttribute("minsize", size);
      node.setAttribute("maxsize", size);
      return node;
    }
  });

  function assertParsed(group) {
    if (!group.body) {
      throw new Error("Bug: The leftright ParseNode wasn't fully parsed.");
    }
  }

  defineFunction({
    type: "leftright-right",
    names: ["\\right"],
    props: {
      numArgs: 1,
      primitive: true
    },
    handler: (context, args) => {
      // \left case below triggers parsing of \right in
      //   `const right = parser.parseFunction();`
      // uses this return value.
      var color = context.parser.gullet.macros.get("\\current@color");

      if (color && typeof color !== "string") {
        throw new ParseError("\\current@color set to non-string in \\right");
      }

      return {
        type: "leftright-right",
        mode: context.parser.mode,
        delim: checkDelimiter(args[0], context).text,
        color // undefined if not set via \color

      };
    }
  });
  defineFunction({
    type: "leftright",
    names: ["\\left"],
    props: {
      numArgs: 1,
      primitive: true
    },
    handler: (context, args) => {
      var delim = checkDelimiter(args[0], context);
      var parser = context.parser; // Parse out the implicit body

      ++parser.leftrightDepth; // parseExpression stops before '\\right'

      var body = parser.parseExpression(false);
      --parser.leftrightDepth; // Check the next token

      parser.expect("\\right", false);
      var right = assertNodeType(parser.parseFunction(), "leftright-right");
      return {
        type: "leftright",
        mode: parser.mode,
        body,
        left: delim.text,
        right: right.delim,
        rightColor: right.color
      };
    },
    htmlBuilder: (group, options) => {
      assertParsed(group); // Build the inner expression

      var inner = buildExpression$1(group.body, options, true, ["mopen", "mclose"]);
      var innerHeight = 0;
      var innerDepth = 0;
      var hadMiddle = false; // Calculate its height and depth

      for (var i = 0; i < inner.length; i++) {
        // Property `isMiddle` not defined on `span`. See comment in
        // "middle"'s htmlBuilder.
        // $FlowFixMe
        if (inner[i].isMiddle) {
          hadMiddle = true;
        } else {
          innerHeight = Math.max(inner[i].height, innerHeight);
          innerDepth = Math.max(inner[i].depth, innerDepth);
        }
      } // The size of delimiters is the same, regardless of what style we are
      // in. Thus, to correctly calculate the size of delimiter we need around
      // a group, we scale down the inner size based on the size.


      innerHeight *= options.sizeMultiplier;
      innerDepth *= options.sizeMultiplier;
      var leftDelim;

      if (group.left === ".") {
        // Empty delimiters in \left and \right make null delimiter spaces.
        leftDelim = makeNullDelimiter(options, ["mopen"]);
      } else {
        // Otherwise, use leftRightDelim to generate the correct sized
        // delimiter.
        leftDelim = delimiter.leftRightDelim(group.left, innerHeight, innerDepth, options, group.mode, ["mopen"]);
      } // Add it to the beginning of the expression


      inner.unshift(leftDelim); // Handle middle delimiters

      if (hadMiddle) {
        for (var _i = 1; _i < inner.length; _i++) {
          var middleDelim = inner[_i]; // Property `isMiddle` not defined on `span`. See comment in
          // "middle"'s htmlBuilder.
          // $FlowFixMe

          var isMiddle = middleDelim.isMiddle;

          if (isMiddle) {
            // Apply the options that were active when \middle was called
            inner[_i] = delimiter.leftRightDelim(isMiddle.delim, innerHeight, innerDepth, isMiddle.options, group.mode, []);
          }
        }
      }

      var rightDelim; // Same for the right delimiter, but using color specified by \color

      if (group.right === ".") {
        rightDelim = makeNullDelimiter(options, ["mclose"]);
      } else {
        var colorOptions = group.rightColor ? options.withColor(group.rightColor) : options;
        rightDelim = delimiter.leftRightDelim(group.right, innerHeight, innerDepth, colorOptions, group.mode, ["mclose"]);
      } // Add it to the end of the expression.


      inner.push(rightDelim);
      return buildCommon.makeSpan(["minner"], inner, options);
    },
    mathmlBuilder: (group, options) => {
      assertParsed(group);
      var inner = buildExpression(group.body, options);

      if (group.left !== ".") {
        var leftNode = new mathMLTree.MathNode("mo", [makeText(group.left, group.mode)]);
        leftNode.setAttribute("fence", "true");
        inner.unshift(leftNode);
      }

      if (group.right !== ".") {
        var rightNode = new mathMLTree.MathNode("mo", [makeText(group.right, group.mode)]);
        rightNode.setAttribute("fence", "true");

        if (group.rightColor) {
          rightNode.setAttribute("mathcolor", group.rightColor);
        }

        inner.push(rightNode);
      }

      return makeRow(inner);
    }
  });
  defineFunction({
    type: "middle",
    names: ["\\middle"],
    props: {
      numArgs: 1,
      primitive: true
    },
    handler: (context, args) => {
      var delim = checkDelimiter(args[0], context);

      if (!context.parser.leftrightDepth) {
        throw new ParseError("\\middle without preceding \\left", delim);
      }

      return {
        type: "middle",
        mode: context.parser.mode,
        delim: delim.text
      };
    },
    htmlBuilder: (group, options) => {
      var middleDelim;

      if (group.delim === ".") {
        middleDelim = makeNullDelimiter(options, []);
      } else {
        middleDelim = delimiter.sizedDelim(group.delim, 1, options, group.mode, []);
        var isMiddle = {
          delim: group.delim,
          options
        }; // Property `isMiddle` not defined on `span`. It is only used in
        // this file above.
        // TODO: Fix this violation of the `span` type and possibly rename
        // things since `isMiddle` sounds like a boolean, but is a struct.
        // $FlowFixMe

        middleDelim.isMiddle = isMiddle;
      }

      return middleDelim;
    },
    mathmlBuilder: (group, options) => {
      // A Firefox \middle will strech a character vertically only if it
      // is in the fence part of the operator dictionary at:
      // https://www.w3.org/TR/MathML3/appendixc.html.
      // So we need to avoid U+2223 and use plain "|" instead.
      var textNode = group.delim === "\\vert" || group.delim === "|" ? makeText("|", "text") : makeText(group.delim, group.mode);
      var middleNode = new mathMLTree.MathNode("mo", [textNode]);
      middleNode.setAttribute("fence", "true"); // MathML gives 5/18em spacing to each <mo> element.
      // \middle should get delimiter spacing instead.

      middleNode.setAttribute("lspace", "0.05em");
      middleNode.setAttribute("rspace", "0.05em");
      return middleNode;
    }
  });

  var htmlBuilder$8 = (group, options) => {
    // \cancel, \bcancel, \xcancel, \sout, \fbox, \colorbox, \fcolorbox, \phase
    // Some groups can return document fragments.  Handle those by wrapping
    // them in a span.
    var inner = buildCommon.wrapFragment(buildGroup$1(group.body, options), options);
    var label = group.label.substr(1);
    var scale = options.sizeMultiplier;
    var img;
    var imgShift = 0; // In the LaTeX cancel package, line geometry is slightly different
    // depending on whether the subject is wider than it is tall, or vice versa.
    // We don't know the width of a group, so as a proxy, we test if
    // the subject is a single character. This captures most of the
    // subjects that should get the "tall" treatment.

    var isSingleChar = utils$1.isCharacterBox(group.body);

    if (label === "sout") {
      img = buildCommon.makeSpan(["stretchy", "sout"]);
      img.height = options.fontMetrics().defaultRuleThickness / scale;
      imgShift = -0.5 * options.fontMetrics().xHeight;
    } else if (label === "phase") {
      // Set a couple of dimensions from the steinmetz package.
      var lineWeight = calculateSize({
        number: 0.6,
        unit: "pt"
      }, options);
      var clearance = calculateSize({
        number: 0.35,
        unit: "ex"
      }, options); // Prevent size changes like \Huge from affecting line thickness

      var newOptions = options.havingBaseSizing();
      scale = scale / newOptions.sizeMultiplier;
      var angleHeight = inner.height + inner.depth + lineWeight + clearance; // Reserve a left pad for the angle.

      inner.style.paddingLeft = makeEm(angleHeight / 2 + lineWeight); // Create an SVG

      var viewBoxHeight = Math.floor(1000 * angleHeight * scale);
      var path = phasePath(viewBoxHeight);
      var svgNode = new SvgNode([new PathNode("phase", path)], {
        "width": "400em",
        "height": makeEm(viewBoxHeight / 1000),
        "viewBox": "0 0 400000 " + viewBoxHeight,
        "preserveAspectRatio": "xMinYMin slice"
      }); // Wrap it in a span with overflow: hidden.

      img = buildCommon.makeSvgSpan(["hide-tail"], [svgNode], options);
      img.style.height = makeEm(angleHeight);
      imgShift = inner.depth + lineWeight + clearance;
    } else {
      // Add horizontal padding
      if (/cancel/.test(label)) {
        if (!isSingleChar) {
          inner.classes.push("cancel-pad");
        }
      } else if (label === "angl") {
        inner.classes.push("anglpad");
      } else {
        inner.classes.push("boxpad");
      } // Add vertical padding


      var topPad = 0;
      var bottomPad = 0;
      var ruleThickness = 0; // ref: cancel package: \advance\totalheight2\p@ % "+2"

      if (/box/.test(label)) {
        ruleThickness = Math.max(options.fontMetrics().fboxrule, // default
        options.minRuleThickness // User override.
        );
        topPad = options.fontMetrics().fboxsep + (label === "colorbox" ? 0 : ruleThickness);
        bottomPad = topPad;
      } else if (label === "angl") {
        ruleThickness = Math.max(options.fontMetrics().defaultRuleThickness, options.minRuleThickness);
        topPad = 4 * ruleThickness; // gap = 3 × line, plus the line itself.

        bottomPad = Math.max(0, 0.25 - inner.depth);
      } else {
        topPad = isSingleChar ? 0.2 : 0;
        bottomPad = topPad;
      }

      img = stretchy.encloseSpan(inner, label, topPad, bottomPad, options);

      if (/fbox|boxed|fcolorbox/.test(label)) {
        img.style.borderStyle = "solid";
        img.style.borderWidth = makeEm(ruleThickness);
      } else if (label === "angl" && ruleThickness !== 0.049) {
        img.style.borderTopWidth = makeEm(ruleThickness);
        img.style.borderRightWidth = makeEm(ruleThickness);
      }

      imgShift = inner.depth + bottomPad;

      if (group.backgroundColor) {
        img.style.backgroundColor = group.backgroundColor;

        if (group.borderColor) {
          img.style.borderColor = group.borderColor;
        }
      }
    }

    var vlist;

    if (group.backgroundColor) {
      vlist = buildCommon.makeVList({
        positionType: "individualShift",
        children: [// Put the color background behind inner;
        {
          type: "elem",
          elem: img,
          shift: imgShift
        }, {
          type: "elem",
          elem: inner,
          shift: 0
        }]
      }, options);
    } else {
      var classes = /cancel|phase/.test(label) ? ["svg-align"] : [];
      vlist = buildCommon.makeVList({
        positionType: "individualShift",
        children: [// Write the \cancel stroke on top of inner.
        {
          type: "elem",
          elem: inner,
          shift: 0
        }, {
          type: "elem",
          elem: img,
          shift: imgShift,
          wrapperClasses: classes
        }]
      }, options);
    }

    if (/cancel/.test(label)) {
      // The cancel package documentation says that cancel lines add their height
      // to the expression, but tests show that isn't how it actually works.
      vlist.height = inner.height;
      vlist.depth = inner.depth;
    }

    if (/cancel/.test(label) && !isSingleChar) {
      // cancel does not create horiz space for its line extension.
      return buildCommon.makeSpan(["mord", "cancel-lap"], [vlist], options);
    } else {
      return buildCommon.makeSpan(["mord"], [vlist], options);
    }
  };

  var mathmlBuilder$7 = (group, options) => {
    var fboxsep = 0;
    var node = new mathMLTree.MathNode(group.label.indexOf("colorbox") > -1 ? "mpadded" : "menclose", [buildGroup(group.body, options)]);

    switch (group.label) {
      case "\\cancel":
        node.setAttribute("notation", "updiagonalstrike");
        break;

      case "\\bcancel":
        node.setAttribute("notation", "downdiagonalstrike");
        break;

      case "\\phase":
        node.setAttribute("notation", "phasorangle");
        break;

      case "\\sout":
        node.setAttribute("notation", "horizontalstrike");
        break;

      case "\\fbox":
        node.setAttribute("notation", "box");
        break;

      case "\\angl":
        node.setAttribute("notation", "actuarial");
        break;

      case "\\fcolorbox":
      case "\\colorbox":
        // <menclose> doesn't have a good notation option. So use <mpadded>
        // instead. Set some attributes that come included with <menclose>.
        fboxsep = options.fontMetrics().fboxsep * options.fontMetrics().ptPerEm;
        node.setAttribute("width", "+" + 2 * fboxsep + "pt");
        node.setAttribute("height", "+" + 2 * fboxsep + "pt");
        node.setAttribute("lspace", fboxsep + "pt"); //

        node.setAttribute("voffset", fboxsep + "pt");

        if (group.label === "\\fcolorbox") {
          var thk = Math.max(options.fontMetrics().fboxrule, // default
          options.minRuleThickness // user override
          );
          node.setAttribute("style", "border: " + thk + "em solid " + String(group.borderColor));
        }

        break;

      case "\\xcancel":
        node.setAttribute("notation", "updiagonalstrike downdiagonalstrike");
        break;
    }

    if (group.backgroundColor) {
      node.setAttribute("mathbackground", group.backgroundColor);
    }

    return node;
  };

  defineFunction({
    type: "enclose",
    names: ["\\colorbox"],
    props: {
      numArgs: 2,
      allowedInText: true,
      argTypes: ["color", "text"]
    },

    handler(_ref, args, optArgs) {
      var {
        parser,
        funcName
      } = _ref;
      var color = assertNodeType(args[0], "color-token").color;
      var body = args[1];
      return {
        type: "enclose",
        mode: parser.mode,
        label: funcName,
        backgroundColor: color,
        body
      };
    },

    htmlBuilder: htmlBuilder$8,
    mathmlBuilder: mathmlBuilder$7
  });
  defineFunction({
    type: "enclose",
    names: ["\\fcolorbox"],
    props: {
      numArgs: 3,
      allowedInText: true,
      argTypes: ["color", "color", "text"]
    },

    handler(_ref2, args, optArgs) {
      var {
        parser,
        funcName
      } = _ref2;
      var borderColor = assertNodeType(args[0], "color-token").color;
      var backgroundColor = assertNodeType(args[1], "color-token").color;
      var body = args[2];
      return {
        type: "enclose",
        mode: parser.mode,
        label: funcName,
        backgroundColor,
        borderColor,
        body
      };
    },

    htmlBuilder: htmlBuilder$8,
    mathmlBuilder: mathmlBuilder$7
  });
  defineFunction({
    type: "enclose",
    names: ["\\fbox"],
    props: {
      numArgs: 1,
      argTypes: ["hbox"],
      allowedInText: true
    },

    handler(_ref3, args) {
      var {
        parser
      } = _ref3;
      return {
        type: "enclose",
        mode: parser.mode,
        label: "\\fbox",
        body: args[0]
      };
    }

  });
  defineFunction({
    type: "enclose",
    names: ["\\cancel", "\\bcancel", "\\xcancel", "\\sout", "\\phase"],
    props: {
      numArgs: 1
    },

    handler(_ref4, args) {
      var {
        parser,
        funcName
      } = _ref4;
      var body = args[0];
      return {
        type: "enclose",
        mode: parser.mode,
        label: funcName,
        body
      };
    },

    htmlBuilder: htmlBuilder$8,
    mathmlBuilder: mathmlBuilder$7
  });
  defineFunction({
    type: "enclose",
    names: ["\\angl"],
    props: {
      numArgs: 1,
      argTypes: ["hbox"],
      allowedInText: false
    },

    handler(_ref5, args) {
      var {
        parser
      } = _ref5;
      return {
        type: "enclose",
        mode: parser.mode,
        label: "\\angl",
        body: args[0]
      };
    }

  });

  /**
   * All registered environments.
   * `environments.js` exports this same dictionary again and makes it public.
   * `Parser.js` requires this dictionary via `environments.js`.
   */
  var _environments = {};
  function defineEnvironment(_ref) {
    var {
      type,
      names,
      props,
      handler,
      htmlBuilder,
      mathmlBuilder
    } = _ref;
    // Set default values of environments.
    var data = {
      type,
      numArgs: props.numArgs || 0,
      allowedInText: false,
      numOptionalArgs: 0,
      handler
    };

    for (var i = 0; i < names.length; ++i) {
      // TODO: The value type of _environments should be a type union of all
      // possible `EnvSpec<>` possibilities instead of `EnvSpec<*>`, which is
      // an existential type.
      _environments[names[i]] = data;
    }

    if (htmlBuilder) {
      _htmlGroupBuilders[type] = htmlBuilder;
    }

    if (mathmlBuilder) {
      _mathmlGroupBuilders[type] = mathmlBuilder;
    }
  }

  /**
   * All registered global/built-in macros.
   * `macros.js` exports this same dictionary again and makes it public.
   * `Parser.js` requires this dictionary via `macros.js`.
   */
  var _macros = {}; // This function might one day accept an additional argument and do more things.

  function defineMacro(name, body) {
    _macros[name] = body;
  }

  // Helper functions
  function getHLines(parser) {
    // Return an array. The array length = number of hlines.
    // Each element in the array tells if the line is dashed.
    var hlineInfo = [];
    parser.consumeSpaces();
    var nxt = parser.fetch().text;

    while (nxt === "\\hline" || nxt === "\\hdashline") {
      parser.consume();
      hlineInfo.push(nxt === "\\hdashline");
      parser.consumeSpaces();
      nxt = parser.fetch().text;
    }

    return hlineInfo;
  }

  var validateAmsEnvironmentContext = context => {
    var settings = context.parser.settings;

    if (!settings.displayMode) {
      throw new ParseError("{" + context.envName + "} can be used only in" + " display mode.");
    }
  }; // autoTag (an argument to parseArray) can be one of three values:
  // * undefined: Regular (not-top-level) array; no tags on each row
  // * true: Automatic equation numbering, overridable by \tag
  // * false: Tags allowed on each row, but no automatic numbering
  // This function *doesn't* work with the "split" environment name.


  function getAutoTag(name) {
    if (name.indexOf("ed") === -1) {
      return name.indexOf("*") === -1;
    } // return undefined;

  }
  /**
   * Parse the body of the environment, with rows delimited by \\ and
   * columns delimited by &, and create a nested list in row-major order
   * with one group per cell.  If given an optional argument style
   * ("text", "display", etc.), then each cell is cast into that style.
   */


  function parseArray(parser, _ref, style) {
    var {
      hskipBeforeAndAfter,
      addJot,
      cols,
      arraystretch,
      colSeparationType,
      autoTag,
      singleRow,
      emptySingleRow,
      maxNumCols,
      leqno
    } = _ref;
    parser.gullet.beginGroup();

    if (!singleRow) {
      // \cr is equivalent to \\ without the optional size argument (see below)
      // TODO: provide helpful error when \cr is used outside array environment
      parser.gullet.macros.set("\\cr", "\\\\\\relax");
    } // Get current arraystretch if it's not set by the environment


    if (!arraystretch) {
      var stretch = parser.gullet.expandMacroAsText("\\arraystretch");

      if (stretch == null) {
        // Default \arraystretch from lttab.dtx
        arraystretch = 1;
      } else {
        arraystretch = parseFloat(stretch);

        if (!arraystretch || arraystretch < 0) {
          throw new ParseError("Invalid \\arraystretch: " + stretch);
        }
      }
    } // Start group for first cell


    parser.gullet.beginGroup();
    var row = [];
    var body = [row];
    var rowGaps = [];
    var hLinesBeforeRow = [];
    var tags = autoTag != null ? [] : undefined; // amsmath uses \global\@eqnswtrue and \global\@eqnswfalse to represent
    // whether this row should have an equation number.  Simulate this with
    // a \@eqnsw macro set to 1 or 0.

    function beginRow() {
      if (autoTag) {
        parser.gullet.macros.set("\\@eqnsw", "1", true);
      }
    }

    function endRow() {
      if (tags) {
        if (parser.gullet.macros.get("\\df@tag")) {
          tags.push(parser.subparse([new Token("\\df@tag")]));
          parser.gullet.macros.set("\\df@tag", undefined, true);
        } else {
          tags.push(Boolean(autoTag) && parser.gullet.macros.get("\\@eqnsw") === "1");
        }
      }
    }

    beginRow(); // Test for \hline at the top of the array.

    hLinesBeforeRow.push(getHLines(parser));

    while (true) {
      // eslint-disable-line no-constant-condition
      // Parse each cell in its own group (namespace)
      var cell = parser.parseExpression(false, singleRow ? "\\end" : "\\\\");
      parser.gullet.endGroup();
      parser.gullet.beginGroup();
      cell = {
        type: "ordgroup",
        mode: parser.mode,
        body: cell
      };

      if (style) {
        cell = {
          type: "styling",
          mode: parser.mode,
          style,
          body: [cell]
        };
      }

      row.push(cell);
      var next = parser.fetch().text;

      if (next === "&") {
        if (maxNumCols && row.length === maxNumCols) {
          if (singleRow || colSeparationType) {
            // {equation} or {split}
            throw new ParseError("Too many tab characters: &", parser.nextToken);
          } else {
            // {array} environment
            parser.settings.reportNonstrict("textEnv", "Too few columns " + "specified in the {array} column argument.");
          }
        }

        parser.consume();
      } else if (next === "\\end") {
        endRow(); // Arrays terminate newlines with `\crcr` which consumes a `\cr` if
        // the last line is empty.  However, AMS environments keep the
        // empty row if it's the only one.
        // NOTE: Currently, `cell` is the last item added into `row`.

        if (row.length === 1 && cell.type === "styling" && cell.body[0].body.length === 0 && (body.length > 1 || !emptySingleRow)) {
          body.pop();
        }

        if (hLinesBeforeRow.length < body.length + 1) {
          hLinesBeforeRow.push([]);
        }

        break;
      } else if (next === "\\\\") {
        parser.consume();
        var size = void 0; // \def\Let@{\let\\\math@cr}
        // \def\math@cr{...\math@cr@}
        // \def\math@cr@{\new@ifnextchar[\math@cr@@{\math@cr@@[\z@]}}
        // \def\math@cr@@[#1]{...\math@cr@@@...}
        // \def\math@cr@@@{\cr}

        if (parser.gullet.future().text !== " ") {
          size = parser.parseSizeGroup(true);
        }

        rowGaps.push(size ? size.value : null);
        endRow(); // check for \hline(s) following the row separator

        hLinesBeforeRow.push(getHLines(parser));
        row = [];
        body.push(row);
        beginRow();
      } else {
        throw new ParseError("Expected & or \\\\ or \\cr or \\end", parser.nextToken);
      }
    } // End cell group


    parser.gullet.endGroup(); // End array group defining \cr

    parser.gullet.endGroup();
    return {
      type: "array",
      mode: parser.mode,
      addJot,
      arraystretch,
      body,
      cols,
      rowGaps,
      hskipBeforeAndAfter,
      hLinesBeforeRow,
      colSeparationType,
      tags,
      leqno
    };
  } // Decides on a style for cells in an array according to whether the given
  // environment name starts with the letter 'd'.


  function dCellStyle(envName) {
    if (envName.substr(0, 1) === "d") {
      return "display";
    } else {
      return "text";
    }
  }

  var htmlBuilder$7 = function htmlBuilder(group, options) {
    var r;
    var c;
    var nr = group.body.length;
    var hLinesBeforeRow = group.hLinesBeforeRow;
    var nc = 0;
    var body = new Array(nr);
    var hlines = [];
    var ruleThickness = Math.max( // From LaTeX \showthe\arrayrulewidth. Equals 0.04 em.
    options.fontMetrics().arrayRuleWidth, options.minRuleThickness // User override.
    ); // Horizontal spacing

    var pt = 1 / options.fontMetrics().ptPerEm;
    var arraycolsep = 5 * pt; // default value, i.e. \arraycolsep in article.cls

    if (group.colSeparationType && group.colSeparationType === "small") {
      // We're in a {smallmatrix}. Default column space is \thickspace,
      // i.e. 5/18em = 0.2778em, per amsmath.dtx for {smallmatrix}.
      // But that needs adjustment because LaTeX applies \scriptstyle to the
      // entire array, including the colspace, but this function applies
      // \scriptstyle only inside each element.
      var localMultiplier = options.havingStyle(Style$1.SCRIPT).sizeMultiplier;
      arraycolsep = 0.2778 * (localMultiplier / options.sizeMultiplier);
    } // Vertical spacing


    var baselineskip = group.colSeparationType === "CD" ? calculateSize({
      number: 3,
      unit: "ex"
    }, options) : 12 * pt; // see size10.clo
    // Default \jot from ltmath.dtx
    // TODO(edemaine): allow overriding \jot via \setlength (#687)

    var jot = 3 * pt;
    var arrayskip = group.arraystretch * baselineskip;
    var arstrutHeight = 0.7 * arrayskip; // \strutbox in ltfsstrc.dtx and

    var arstrutDepth = 0.3 * arrayskip; // \@arstrutbox in lttab.dtx

    var totalHeight = 0; // Set a position for \hline(s) at the top of the array, if any.

    function setHLinePos(hlinesInGap) {
      for (var i = 0; i < hlinesInGap.length; ++i) {
        if (i > 0) {
          totalHeight += 0.25;
        }

        hlines.push({
          pos: totalHeight,
          isDashed: hlinesInGap[i]
        });
      }
    }

    setHLinePos(hLinesBeforeRow[0]);

    for (r = 0; r < group.body.length; ++r) {
      var inrow = group.body[r];
      var height = arstrutHeight; // \@array adds an \@arstrut

      var depth = arstrutDepth; // to each tow (via the template)

      if (nc < inrow.length) {
        nc = inrow.length;
      }

      var outrow = new Array(inrow.length);

      for (c = 0; c < inrow.length; ++c) {
        var elt = buildGroup$1(inrow[c], options);

        if (depth < elt.depth) {
          depth = elt.depth;
        }

        if (height < elt.height) {
          height = elt.height;
        }

        outrow[c] = elt;
      }

      var rowGap = group.rowGaps[r];
      var gap = 0;

      if (rowGap) {
        gap = calculateSize(rowGap, options);

        if (gap > 0) {
          // \@argarraycr
          gap += arstrutDepth;

          if (depth < gap) {
            depth = gap; // \@xargarraycr
          }

          gap = 0;
        }
      } // In AMS multiline environments such as aligned and gathered, rows
      // correspond to lines that have additional \jot added to the
      // \baselineskip via \openup.


      if (group.addJot) {
        depth += jot;
      }

      outrow.height = height;
      outrow.depth = depth;
      totalHeight += height;
      outrow.pos = totalHeight;
      totalHeight += depth + gap; // \@yargarraycr

      body[r] = outrow; // Set a position for \hline(s), if any.

      setHLinePos(hLinesBeforeRow[r + 1]);
    }

    var offset = totalHeight / 2 + options.fontMetrics().axisHeight;
    var colDescriptions = group.cols || [];
    var cols = [];
    var colSep;
    var colDescrNum;
    var tagSpans = [];

    if (group.tags && group.tags.some(tag => tag)) {
      // An environment with manual tags and/or automatic equation numbers.
      // Create node(s), the latter of which trigger CSS counter increment.
      for (r = 0; r < nr; ++r) {
        var rw = body[r];
        var shift = rw.pos - offset;
        var tag = group.tags[r];
        var tagSpan = void 0;

        if (tag === true) {
          // automatic numbering
          tagSpan = buildCommon.makeSpan(["eqn-num"], [], options);
        } else if (tag === false) {
          // \nonumber/\notag or starred environment
          tagSpan = buildCommon.makeSpan([], [], options);
        } else {
          // manual \tag
          tagSpan = buildCommon.makeSpan([], buildExpression$1(tag, options, true), options);
        }

        tagSpan.depth = rw.depth;
        tagSpan.height = rw.height;
        tagSpans.push({
          type: "elem",
          elem: tagSpan,
          shift
        });
      }
    }

    for (c = 0, colDescrNum = 0; // Continue while either there are more columns or more column
    // descriptions, so trailing separators don't get lost.
    c < nc || colDescrNum < colDescriptions.length; ++c, ++colDescrNum) {
      var colDescr = colDescriptions[colDescrNum] || {};
      var firstSeparator = true;

      while (colDescr.type === "separator") {
        // If there is more than one separator in a row, add a space
        // between them.
        if (!firstSeparator) {
          colSep = buildCommon.makeSpan(["arraycolsep"], []);
          colSep.style.width = makeEm(options.fontMetrics().doubleRuleSep);
          cols.push(colSep);
        }

        if (colDescr.separator === "|" || colDescr.separator === ":") {
          var lineType = colDescr.separator === "|" ? "solid" : "dashed";
          var separator = buildCommon.makeSpan(["vertical-separator"], [], options);
          separator.style.height = makeEm(totalHeight);
          separator.style.borderRightWidth = makeEm(ruleThickness);
          separator.style.borderRightStyle = lineType;
          separator.style.margin = "0 " + makeEm(-ruleThickness / 2);

          var _shift = totalHeight - offset;

          if (_shift) {
            separator.style.verticalAlign = makeEm(-_shift);
          }

          cols.push(separator);
        } else {
          throw new ParseError("Invalid separator type: " + colDescr.separator);
        }

        colDescrNum++;
        colDescr = colDescriptions[colDescrNum] || {};
        firstSeparator = false;
      }

      if (c >= nc) {
        continue;
      }

      var sepwidth = void 0;

      if (c > 0 || group.hskipBeforeAndAfter) {
        sepwidth = utils$1.deflt(colDescr.pregap, arraycolsep);

        if (sepwidth !== 0) {
          colSep = buildCommon.makeSpan(["arraycolsep"], []);
          colSep.style.width = makeEm(sepwidth);
          cols.push(colSep);
        }
      }

      var col = [];

      for (r = 0; r < nr; ++r) {
        var row = body[r];
        var elem = row[c];

        if (!elem) {
          continue;
        }

        var _shift2 = row.pos - offset;

        elem.depth = row.depth;
        elem.height = row.height;
        col.push({
          type: "elem",
          elem: elem,
          shift: _shift2
        });
      }

      col = buildCommon.makeVList({
        positionType: "individualShift",
        children: col
      }, options);
      col = buildCommon.makeSpan(["col-align-" + (colDescr.align || "c")], [col]);
      cols.push(col);

      if (c < nc - 1 || group.hskipBeforeAndAfter) {
        sepwidth = utils$1.deflt(colDescr.postgap, arraycolsep);

        if (sepwidth !== 0) {
          colSep = buildCommon.makeSpan(["arraycolsep"], []);
          colSep.style.width = makeEm(sepwidth);
          cols.push(colSep);
        }
      }
    }

    body = buildCommon.makeSpan(["mtable"], cols); // Add \hline(s), if any.

    if (hlines.length > 0) {
      var line = buildCommon.makeLineSpan("hline", options, ruleThickness);
      var dashes = buildCommon.makeLineSpan("hdashline", options, ruleThickness);
      var vListElems = [{
        type: "elem",
        elem: body,
        shift: 0
      }];

      while (hlines.length > 0) {
        var hline = hlines.pop();
        var lineShift = hline.pos - offset;

        if (hline.isDashed) {
          vListElems.push({
            type: "elem",
            elem: dashes,
            shift: lineShift
          });
        } else {
          vListElems.push({
            type: "elem",
            elem: line,
            shift: lineShift
          });
        }
      }

      body = buildCommon.makeVList({
        positionType: "individualShift",
        children: vListElems
      }, options);
    }

    if (tagSpans.length === 0) {
      return buildCommon.makeSpan(["mord"], [body], options);
    } else {
      var eqnNumCol = buildCommon.makeVList({
        positionType: "individualShift",
        children: tagSpans
      }, options);
      eqnNumCol = buildCommon.makeSpan(["tag"], [eqnNumCol], options);
      return buildCommon.makeFragment([body, eqnNumCol]);
    }
  };

  var alignMap = {
    c: "center ",
    l: "left ",
    r: "right "
  };

  var mathmlBuilder$6 = function mathmlBuilder(group, options) {
    var tbl = [];
    var glue = new mathMLTree.MathNode("mtd", [], ["mtr-glue"]);
    var tag = new mathMLTree.MathNode("mtd", [], ["mml-eqn-num"]);

    for (var i = 0; i < group.body.length; i++) {
      var rw = group.body[i];
      var row = [];

      for (var j = 0; j < rw.length; j++) {
        row.push(new mathMLTree.MathNode("mtd", [buildGroup(rw[j], options)]));
      }

      if (group.tags && group.tags[i]) {
        row.unshift(glue);
        row.push(glue);

        if (group.leqno) {
          row.unshift(tag);
        } else {
          row.push(tag);
        }
      }

      tbl.push(new mathMLTree.MathNode("mtr", row));
    }

    var table = new mathMLTree.MathNode("mtable", tbl); // Set column alignment, row spacing, column spacing, and
    // array lines by setting attributes on the table element.
    // Set the row spacing. In MathML, we specify a gap distance.
    // We do not use rowGap[] because MathML automatically increases
    // cell height with the height/depth of the element content.
    // LaTeX \arraystretch multiplies the row baseline-to-baseline distance.
    // We simulate this by adding (arraystretch - 1)em to the gap. This
    // does a reasonable job of adjusting arrays containing 1 em tall content.
    // The 0.16 and 0.09 values are found emprically. They produce an array
    // similar to LaTeX and in which content does not interfere with \hines.

    var gap = group.arraystretch === 0.5 ? 0.1 // {smallmatrix}, {subarray}
    : 0.16 + group.arraystretch - 1 + (group.addJot ? 0.09 : 0);
    table.setAttribute("rowspacing", makeEm(gap)); // MathML table lines go only between cells.
    // To place a line on an edge we'll use <menclose>, if necessary.

    var menclose = "";
    var align = "";

    if (group.cols && group.cols.length > 0) {
      // Find column alignment, column spacing, and  vertical lines.
      var cols = group.cols;
      var columnLines = "";
      var prevTypeWasAlign = false;
      var iStart = 0;
      var iEnd = cols.length;

      if (cols[0].type === "separator") {
        menclose += "top ";
        iStart = 1;
      }

      if (cols[cols.length - 1].type === "separator") {
        menclose += "bottom ";
        iEnd -= 1;
      }

      for (var _i = iStart; _i < iEnd; _i++) {
        if (cols[_i].type === "align") {
          align += alignMap[cols[_i].align];

          if (prevTypeWasAlign) {
            columnLines += "none ";
          }

          prevTypeWasAlign = true;
        } else if (cols[_i].type === "separator") {
          // MathML accepts only single lines between cells.
          // So we read only the first of consecutive separators.
          if (prevTypeWasAlign) {
            columnLines += cols[_i].separator === "|" ? "solid " : "dashed ";
            prevTypeWasAlign = false;
          }
        }
      }

      table.setAttribute("columnalign", align.trim());

      if (/[sd]/.test(columnLines)) {
        table.setAttribute("columnlines", columnLines.trim());
      }
    } // Set column spacing.


    if (group.colSeparationType === "align") {
      var _cols = group.cols || [];

      var spacing = "";

      for (var _i2 = 1; _i2 < _cols.length; _i2++) {
        spacing += _i2 % 2 ? "0em " : "1em ";
      }

      table.setAttribute("columnspacing", spacing.trim());
    } else if (group.colSeparationType === "alignat" || group.colSeparationType === "gather") {
      table.setAttribute("columnspacing", "0em");
    } else if (group.colSeparationType === "small") {
      table.setAttribute("columnspacing", "0.2778em");
    } else if (group.colSeparationType === "CD") {
      table.setAttribute("columnspacing", "0.5em");
    } else {
      table.setAttribute("columnspacing", "1em");
    } // Address \hline and \hdashline


    var rowLines = "";
    var hlines = group.hLinesBeforeRow;
    menclose += hlines[0].length > 0 ? "left " : "";
    menclose += hlines[hlines.length - 1].length > 0 ? "right " : "";

    for (var _i3 = 1; _i3 < hlines.length - 1; _i3++) {
      rowLines += hlines[_i3].length === 0 ? "none " // MathML accepts only a single line between rows. Read one element.
      : hlines[_i3][0] ? "dashed " : "solid ";
    }

    if (/[sd]/.test(rowLines)) {
      table.setAttribute("rowlines", rowLines.trim());
    }

    if (menclose !== "") {
      table = new mathMLTree.MathNode("menclose", [table]);
      table.setAttribute("notation", menclose.trim());
    }

    if (group.arraystretch && group.arraystretch < 1) {
      // A small array. Wrap in scriptstyle so row gap is not too large.
      table = new mathMLTree.MathNode("mstyle", [table]);
      table.setAttribute("scriptlevel", "1");
    }

    return table;
  }; // Convenience function for align, align*, aligned, alignat, alignat*, alignedat.


  var alignedHandler = function alignedHandler(context, args) {
    if (context.envName.indexOf("ed") === -1) {
      validateAmsEnvironmentContext(context);
    }

    var cols = [];
    var separationType = context.envName.indexOf("at") > -1 ? "alignat" : "align";
    var isSplit = context.envName === "split";
    var res = parseArray(context.parser, {
      cols,
      addJot: true,
      autoTag: isSplit ? undefined : getAutoTag(context.envName),
      emptySingleRow: true,
      colSeparationType: separationType,
      maxNumCols: isSplit ? 2 : undefined,
      leqno: context.parser.settings.leqno
    }, "display"); // Determining number of columns.
    // 1. If the first argument is given, we use it as a number of columns,
    //    and makes sure that each row doesn't exceed that number.
    // 2. Otherwise, just count number of columns = maximum number
    //    of cells in each row ("aligned" mode -- isAligned will be true).
    //
    // At the same time, prepend empty group {} at beginning of every second
    // cell in each row (starting with second cell) so that operators become
    // binary.  This behavior is implemented in amsmath's \start@aligned.

    var numMaths;
    var numCols = 0;
    var emptyGroup = {
      type: "ordgroup",
      mode: context.mode,
      body: []
    };

    if (args[0] && args[0].type === "ordgroup") {
      var arg0 = "";

      for (var i = 0; i < args[0].body.length; i++) {
        var textord = assertNodeType(args[0].body[i], "textord");
        arg0 += textord.text;
      }

      numMaths = Number(arg0);
      numCols = numMaths * 2;
    }

    var isAligned = !numCols;
    res.body.forEach(function (row) {
      for (var _i4 = 1; _i4 < row.length; _i4 += 2) {
        // Modify ordgroup node within styling node
        var styling = assertNodeType(row[_i4], "styling");
        var ordgroup = assertNodeType(styling.body[0], "ordgroup");
        ordgroup.body.unshift(emptyGroup);
      }

      if (!isAligned) {
        // Case 1
        var curMaths = row.length / 2;

        if (numMaths < curMaths) {
          throw new ParseError("Too many math in a row: " + ("expected " + numMaths + ", but got " + curMaths), row[0]);
        }
      } else if (numCols < row.length) {
        // Case 2
        numCols = row.length;
      }
    }); // Adjusting alignment.
    // In aligned mode, we add one \qquad between columns;
    // otherwise we add nothing.

    for (var _i5 = 0; _i5 < numCols; ++_i5) {
      var align = "r";
      var pregap = 0;

      if (_i5 % 2 === 1) {
        align = "l";
      } else if (_i5 > 0 && isAligned) {
        // "aligned" mode.
        pregap = 1; // add one \quad
      }

      cols[_i5] = {
        type: "align",
        align: align,
        pregap: pregap,
        postgap: 0
      };
    }

    res.colSeparationType = isAligned ? "align" : "alignat";
    return res;
  }; // Arrays are part of LaTeX, defined in lttab.dtx so its documentation
  // is part of the source2e.pdf file of LaTeX2e source documentation.
  // {darray} is an {array} environment where cells are set in \displaystyle,
  // as defined in nccmath.sty.


  defineEnvironment({
    type: "array",
    names: ["array", "darray"],
    props: {
      numArgs: 1
    },

    handler(context, args) {
      // Since no types are specified above, the two possibilities are
      // - The argument is wrapped in {} or [], in which case Parser's
      //   parseGroup() returns an "ordgroup" wrapping some symbol node.
      // - The argument is a bare symbol node.
      var symNode = checkSymbolNodeType(args[0]);
      var colalign = symNode ? [args[0]] : assertNodeType(args[0], "ordgroup").body;
      var cols = colalign.map(function (nde) {
        var node = assertSymbolNodeType(nde);
        var ca = node.text;

        if ("lcr".indexOf(ca) !== -1) {
          return {
            type: "align",
            align: ca
          };
        } else if (ca === "|") {
          return {
            type: "separator",
            separator: "|"
          };
        } else if (ca === ":") {
          return {
            type: "separator",
            separator: ":"
          };
        }

        throw new ParseError("Unknown column alignment: " + ca, nde);
      });
      var res = {
        cols,
        hskipBeforeAndAfter: true,
        // \@preamble in lttab.dtx
        maxNumCols: cols.length
      };
      return parseArray(context.parser, res, dCellStyle(context.envName));
    },

    htmlBuilder: htmlBuilder$7,
    mathmlBuilder: mathmlBuilder$6
  }); // The matrix environments of amsmath builds on the array environment
  // of LaTeX, which is discussed above.
  // The mathtools package adds starred versions of the same environments.
  // These have an optional argument to choose left|center|right justification.

  defineEnvironment({
    type: "array",
    names: ["matrix", "pmatrix", "bmatrix", "Bmatrix", "vmatrix", "Vmatrix", "matrix*", "pmatrix*", "bmatrix*", "Bmatrix*", "vmatrix*", "Vmatrix*"],
    props: {
      numArgs: 0
    },

    handler(context) {
      var delimiters = {
        "matrix": null,
        "pmatrix": ["(", ")"],
        "bmatrix": ["[", "]"],
        "Bmatrix": ["\\{", "\\}"],
        "vmatrix": ["|", "|"],
        "Vmatrix": ["\\Vert", "\\Vert"]
      }[context.envName.replace("*", "")]; // \hskip -\arraycolsep in amsmath

      var colAlign = "c";
      var payload = {
        hskipBeforeAndAfter: false,
        cols: [{
          type: "align",
          align: colAlign
        }]
      };

      if (context.envName.charAt(context.envName.length - 1) === "*") {
        // It's one of the mathtools starred functions.
        // Parse the optional alignment argument.
        var parser = context.parser;
        parser.consumeSpaces();

        if (parser.fetch().text === "[") {
          parser.consume();
          parser.consumeSpaces();
          colAlign = parser.fetch().text;

          if ("lcr".indexOf(colAlign) === -1) {
            throw new ParseError("Expected l or c or r", parser.nextToken);
          }

          parser.consume();
          parser.consumeSpaces();
          parser.expect("]");
          parser.consume();
          payload.cols = [{
            type: "align",
            align: colAlign
          }];
        }
      }

      var res = parseArray(context.parser, payload, dCellStyle(context.envName)); // Populate cols with the correct number of column alignment specs.

      var numCols = Math.max(0, ...res.body.map(row => row.length));
      res.cols = new Array(numCols).fill({
        type: "align",
        align: colAlign
      });
      return delimiters ? {
        type: "leftright",
        mode: context.mode,
        body: [res],
        left: delimiters[0],
        right: delimiters[1],
        rightColor: undefined // \right uninfluenced by \color in array

      } : res;
    },

    htmlBuilder: htmlBuilder$7,
    mathmlBuilder: mathmlBuilder$6
  });
  defineEnvironment({
    type: "array",
    names: ["smallmatrix"],
    props: {
      numArgs: 0
    },

    handler(context) {
      var payload = {
        arraystretch: 0.5
      };
      var res = parseArray(context.parser, payload, "script");
      res.colSeparationType = "small";
      return res;
    },

    htmlBuilder: htmlBuilder$7,
    mathmlBuilder: mathmlBuilder$6
  });
  defineEnvironment({
    type: "array",
    names: ["subarray"],
    props: {
      numArgs: 1
    },

    handler(context, args) {
      // Parsing of {subarray} is similar to {array}
      var symNode = checkSymbolNodeType(args[0]);
      var colalign = symNode ? [args[0]] : assertNodeType(args[0], "ordgroup").body;
      var cols = colalign.map(function (nde) {
        var node = assertSymbolNodeType(nde);
        var ca = node.text; // {subarray} only recognizes "l" & "c"

        if ("lc".indexOf(ca) !== -1) {
          return {
            type: "align",
            align: ca
          };
        }

        throw new ParseError("Unknown column alignment: " + ca, nde);
      });

      if (cols.length > 1) {
        throw new ParseError("{subarray} can contain only one column");
      }

      var res = {
        cols,
        hskipBeforeAndAfter: false,
        arraystretch: 0.5
      };
      res = parseArray(context.parser, res, "script");

      if (res.body.length > 0 && res.body[0].length > 1) {
        throw new ParseError("{subarray} can contain only one column");
      }

      return res;
    },

    htmlBuilder: htmlBuilder$7,
    mathmlBuilder: mathmlBuilder$6
  }); // A cases environment (in amsmath.sty) is almost equivalent to
  // \def\arraystretch{1.2}%
  // \left\{\begin{array}{@{}l@{\quad}l@{}} … \end{array}\right.
  // {dcases} is a {cases} environment where cells are set in \displaystyle,
  // as defined in mathtools.sty.
  // {rcases} is another mathtools environment. It's brace is on the right side.

  defineEnvironment({
    type: "array",
    names: ["cases", "dcases", "rcases", "drcases"],
    props: {
      numArgs: 0
    },

    handler(context) {
      var payload = {
        arraystretch: 1.2,
        cols: [{
          type: "align",
          align: "l",
          pregap: 0,
          // TODO(kevinb) get the current style.
          // For now we use the metrics for TEXT style which is what we were
          // doing before.  Before attempting to get the current style we
          // should look at TeX's behavior especially for \over and matrices.
          postgap: 1.0
          /* 1em quad */

        }, {
          type: "align",
          align: "l",
          pregap: 0,
          postgap: 0
        }]
      };
      var res = parseArray(context.parser, payload, dCellStyle(context.envName));
      return {
        type: "leftright",
        mode: context.mode,
        body: [res],
        left: context.envName.indexOf("r") > -1 ? "." : "\\{",
        right: context.envName.indexOf("r") > -1 ? "\\}" : ".",
        rightColor: undefined
      };
    },

    htmlBuilder: htmlBuilder$7,
    mathmlBuilder: mathmlBuilder$6
  }); // In the align environment, one uses ampersands, &, to specify number of
  // columns in each row, and to locate spacing between each column.
  // align gets automatic numbering. align* and aligned do not.
  // The alignedat environment can be used in math mode.
  // Note that we assume \nomallineskiplimit to be zero,
  // so that \strut@ is the same as \strut.

  defineEnvironment({
    type: "array",
    names: ["align", "align*", "aligned", "split"],
    props: {
      numArgs: 0
    },
    handler: alignedHandler,
    htmlBuilder: htmlBuilder$7,
    mathmlBuilder: mathmlBuilder$6
  }); // A gathered environment is like an array environment with one centered
  // column, but where rows are considered lines so get \jot line spacing
  // and contents are set in \displaystyle.

  defineEnvironment({
    type: "array",
    names: ["gathered", "gather", "gather*"],
    props: {
      numArgs: 0
    },

    handler(context) {
      if (utils$1.contains(["gather", "gather*"], context.envName)) {
        validateAmsEnvironmentContext(context);
      }

      var res = {
        cols: [{
          type: "align",
          align: "c"
        }],
        addJot: true,
        colSeparationType: "gather",
        autoTag: getAutoTag(context.envName),
        emptySingleRow: true,
        leqno: context.parser.settings.leqno
      };
      return parseArray(context.parser, res, "display");
    },

    htmlBuilder: htmlBuilder$7,
    mathmlBuilder: mathmlBuilder$6
  }); // alignat environment is like an align environment, but one must explicitly
  // specify maximum number of columns in each row, and can adjust spacing between
  // each columns.

  defineEnvironment({
    type: "array",
    names: ["alignat", "alignat*", "alignedat"],
    props: {
      numArgs: 1
    },
    handler: alignedHandler,
    htmlBuilder: htmlBuilder$7,
    mathmlBuilder: mathmlBuilder$6
  });
  defineEnvironment({
    type: "array",
    names: ["equation", "equation*"],
    props: {
      numArgs: 0
    },

    handler(context) {
      validateAmsEnvironmentContext(context);
      var res = {
        autoTag: getAutoTag(context.envName),
        emptySingleRow: true,
        singleRow: true,
        maxNumCols: 1,
        leqno: context.parser.settings.leqno
      };
      return parseArray(context.parser, res, "display");
    },

    htmlBuilder: htmlBuilder$7,
    mathmlBuilder: mathmlBuilder$6
  });
  defineEnvironment({
    type: "array",
    names: ["CD"],
    props: {
      numArgs: 0
    },

    handler(context) {
      validateAmsEnvironmentContext(context);
      return parseCD(context.parser);
    },

    htmlBuilder: htmlBuilder$7,
    mathmlBuilder: mathmlBuilder$6
  });
  defineMacro("\\nonumber", "\\gdef\\@eqnsw{0}");
  defineMacro("\\notag", "\\nonumber"); // Catch \hline outside array environment

  defineFunction({
    type: "text",
    // Doesn't matter what this is.
    names: ["\\hline", "\\hdashline"],
    props: {
      numArgs: 0,
      allowedInText: true,
      allowedInMath: true
    },

    handler(context, args) {
      throw new ParseError(context.funcName + " valid only within array environment");
    }

  });

  var environments = _environments;

  // defineEnvironment definitions.

  defineFunction({
    type: "environment",
    names: ["\\begin", "\\end"],
    props: {
      numArgs: 1,
      argTypes: ["text"]
    },

    handler(_ref, args) {
      var {
        parser,
        funcName
      } = _ref;
      var nameGroup = args[0];

      if (nameGroup.type !== "ordgroup") {
        throw new ParseError("Invalid environment name", nameGroup);
      }

      var envName = "";

      for (var i = 0; i < nameGroup.body.length; ++i) {
        envName += assertNodeType(nameGroup.body[i], "textord").text;
      }

      if (funcName === "\\begin") {
        // begin...end is similar to left...right
        if (!environments.hasOwnProperty(envName)) {
          throw new ParseError("No such environment: " + envName, nameGroup);
        } // Build the environment object. Arguments and other information will
        // be made available to the begin and end methods using properties.


        var env = environments[envName];
        var {
          args: _args,
          optArgs
        } = parser.parseArguments("\\begin{" + envName + "}", env);
        var context = {
          mode: parser.mode,
          envName,
          parser
        };
        var result = env.handler(context, _args, optArgs);
        parser.expect("\\end", false);
        var endNameToken = parser.nextToken;
        var end = assertNodeType(parser.parseFunction(), "environment");

        if (end.name !== envName) {
          throw new ParseError("Mismatch: \\begin{" + envName + "} matched by \\end{" + end.name + "}", endNameToken);
        } // $FlowFixMe, "environment" handler returns an environment ParseNode


        return result;
      }

      return {
        type: "environment",
        mode: parser.mode,
        name: envName,
        nameGroup
      };
    }

  });

  var makeSpan = buildCommon.makeSpan;

  function htmlBuilder$6(group, options) {
    var elements = buildExpression$1(group.body, options, true);
    return makeSpan([group.mclass], elements, options);
  }

  function mathmlBuilder$5(group, options) {
    var node;
    var inner = buildExpression(group.body, options);

    if (group.mclass === "minner") {
      node = new mathMLTree.MathNode("mpadded", inner);
    } else if (group.mclass === "mord") {
      if (group.isCharacterBox) {
        node = inner[0];
        node.type = "mi";
      } else {
        node = new mathMLTree.MathNode("mi", inner);
      }
    } else {
      if (group.isCharacterBox) {
        node = inner[0];
        node.type = "mo";
      } else {
        node = new mathMLTree.MathNode("mo", inner);
      } // Set spacing based on what is the most likely adjacent atom type.
      // See TeXbook p170.


      if (group.mclass === "mbin") {
        node.attributes.lspace = "0.22em"; // medium space

        node.attributes.rspace = "0.22em";
      } else if (group.mclass === "mpunct") {
        node.attributes.lspace = "0em";
        node.attributes.rspace = "0.17em"; // thinspace
      } else if (group.mclass === "mopen" || group.mclass === "mclose") {
        node.attributes.lspace = "0em";
        node.attributes.rspace = "0em";
      } else if (group.mclass === "minner") {
        node.attributes.lspace = "0.0556em"; // 1 mu is the most likely option

        node.attributes.width = "+0.1111em";
      } // MathML <mo> default space is 5/18 em, so <mrel> needs no action.
      // Ref: https://developer.mozilla.org/en-US/docs/Web/MathML/Element/mo

    }

    return node;
  } // Math class commands except \mathop


  defineFunction({
    type: "mclass",
    names: ["\\mathord", "\\mathbin", "\\mathrel", "\\mathopen", "\\mathclose", "\\mathpunct", "\\mathinner"],
    props: {
      numArgs: 1,
      primitive: true
    },

    handler(_ref, args) {
      var {
        parser,
        funcName
      } = _ref;
      var body = args[0];
      return {
        type: "mclass",
        mode: parser.mode,
        mclass: "m" + funcName.substr(5),
        // TODO(kevinb): don't prefix with 'm'
        body: ordargument(body),
        isCharacterBox: utils$1.isCharacterBox(body)
      };
    },

    htmlBuilder: htmlBuilder$6,
    mathmlBuilder: mathmlBuilder$5
  });
  var binrelClass = arg => {
    // \binrel@ spacing varies with (bin|rel|ord) of the atom in the argument.
    // (by rendering separately and with {}s before and after, and measuring
    // the change in spacing).  We'll do roughly the same by detecting the
    // atom type directly.
    var atom = arg.type === "ordgroup" && arg.body.length ? arg.body[0] : arg;

    if (atom.type === "atom" && (atom.family === "bin" || atom.family === "rel")) {
      return "m" + atom.family;
    } else {
      return "mord";
    }
  }; // \@binrel{x}{y} renders like y but as mbin/mrel/mord if x is mbin/mrel/mord.
  // This is equivalent to \binrel@{x}\binrel@@{y} in AMSTeX.

  defineFunction({
    type: "mclass",
    names: ["\\@binrel"],
    props: {
      numArgs: 2
    },

    handler(_ref2, args) {
      var {
        parser
      } = _ref2;
      return {
        type: "mclass",
        mode: parser.mode,
        mclass: binrelClass(args[0]),
        body: ordargument(args[1]),
        isCharacterBox: utils$1.isCharacterBox(args[1])
      };
    }

  }); // Build a relation or stacked op by placing one symbol on top of another

  defineFunction({
    type: "mclass",
    names: ["\\stackrel", "\\overset", "\\underset"],
    props: {
      numArgs: 2
    },

    handler(_ref3, args) {
      var {
        parser,
        funcName
      } = _ref3;
      var baseArg = args[1];
      var shiftedArg = args[0];
      var mclass;

      if (funcName !== "\\stackrel") {
        // LaTeX applies \binrel spacing to \overset and \underset.
        mclass = binrelClass(baseArg);
      } else {
        mclass = "mrel"; // for \stackrel
      }

      var baseOp = {
        type: "op",
        mode: baseArg.mode,
        limits: true,
        alwaysHandleSupSub: true,
        parentIsSupSub: false,
        symbol: false,
        suppressBaseShift: funcName !== "\\stackrel",
        body: ordargument(baseArg)
      };
      var supsub = {
        type: "supsub",
        mode: shiftedArg.mode,
        base: baseOp,
        sup: funcName === "\\underset" ? null : shiftedArg,
        sub: funcName === "\\underset" ? shiftedArg : null
      };
      return {
        type: "mclass",
        mode: parser.mode,
        mclass,
        body: [supsub],
        isCharacterBox: utils$1.isCharacterBox(supsub)
      };
    },

    htmlBuilder: htmlBuilder$6,
    mathmlBuilder: mathmlBuilder$5
  });

  // TODO(kevinb): implement \\sl and \\sc

  var htmlBuilder$5 = (group, options) => {
    var font = group.font;
    var newOptions = options.withFont(font);
    return buildGroup$1(group.body, newOptions);
  };

  var mathmlBuilder$4 = (group, options) => {
    var font = group.font;
    var newOptions = options.withFont(font);
    return buildGroup(group.body, newOptions);
  };

  var fontAliases = {
    "\\Bbb": "\\mathbb",
    "\\bold": "\\mathbf",
    "\\frak": "\\mathfrak",
    "\\bm": "\\boldsymbol"
  };
  defineFunction({
    type: "font",
    names: [// styles, except \boldsymbol defined below
    "\\mathrm", "\\mathit", "\\mathbf", "\\mathnormal", // families
    "\\mathbb", "\\mathcal", "\\mathfrak", "\\mathscr", "\\mathsf", "\\mathtt", // aliases, except \bm defined below
    "\\Bbb", "\\bold", "\\frak"],
    props: {
      numArgs: 1,
      allowedInArgument: true
    },
    handler: (_ref, args) => {
      var {
        parser,
        funcName
      } = _ref;
      var body = normalizeArgument(args[0]);
      var func = funcName;

      if (func in fontAliases) {
        func = fontAliases[func];
      }

      return {
        type: "font",
        mode: parser.mode,
        font: func.slice(1),
        body
      };
    },
    htmlBuilder: htmlBuilder$5,
    mathmlBuilder: mathmlBuilder$4
  });
  defineFunction({
    type: "mclass",
    names: ["\\boldsymbol", "\\bm"],
    props: {
      numArgs: 1
    },
    handler: (_ref2, args) => {
      var {
        parser
      } = _ref2;
      var body = args[0];
      var isCharacterBox = utils$1.isCharacterBox(body); // amsbsy.sty's \boldsymbol uses \binrel spacing to inherit the
      // argument's bin|rel|ord status

      return {
        type: "mclass",
        mode: parser.mode,
        mclass: binrelClass(body),
        body: [{
          type: "font",
          mode: parser.mode,
          font: "boldsymbol",
          body
        }],
        isCharacterBox: isCharacterBox
      };
    }
  }); // Old font changing functions

  defineFunction({
    type: "font",
    names: ["\\rm", "\\sf", "\\tt", "\\bf", "\\it", "\\cal"],
    props: {
      numArgs: 0,
      allowedInText: true
    },
    handler: (_ref3, args) => {
      var {
        parser,
        funcName,
        breakOnTokenText
      } = _ref3;
      var {
        mode
      } = parser;
      var body = parser.parseExpression(true, breakOnTokenText);
      var style = "math" + funcName.slice(1);
      return {
        type: "font",
        mode: mode,
        font: style,
        body: {
          type: "ordgroup",
          mode: parser.mode,
          body
        }
      };
    },
    htmlBuilder: htmlBuilder$5,
    mathmlBuilder: mathmlBuilder$4
  });

  var adjustStyle = (size, originalStyle) => {
    // Figure out what style this fraction should be in based on the
    // function used
    var style = originalStyle;

    if (size === "display") {
      // Get display style as a default.
      // If incoming style is sub/sup, use style.text() to get correct size.
      style = style.id >= Style$1.SCRIPT.id ? style.text() : Style$1.DISPLAY;
    } else if (size === "text" && style.size === Style$1.DISPLAY.size) {
      // We're in a \tfrac but incoming style is displaystyle, so:
      style = Style$1.TEXT;
    } else if (size === "script") {
      style = Style$1.SCRIPT;
    } else if (size === "scriptscript") {
      style = Style$1.SCRIPTSCRIPT;
    }

    return style;
  };

  var htmlBuilder$4 = (group, options) => {
    // Fractions are handled in the TeXbook on pages 444-445, rules 15(a-e).
    var style = adjustStyle(group.size, options.style);
    var nstyle = style.fracNum();
    var dstyle = style.fracDen();
    var newOptions;
    newOptions = options.havingStyle(nstyle);
    var numerm = buildGroup$1(group.numer, newOptions, options);

    if (group.continued) {
      // \cfrac inserts a \strut into the numerator.
      // Get \strut dimensions from TeXbook page 353.
      var hStrut = 8.5 / options.fontMetrics().ptPerEm;
      var dStrut = 3.5 / options.fontMetrics().ptPerEm;
      numerm.height = numerm.height < hStrut ? hStrut : numerm.height;
      numerm.depth = numerm.depth < dStrut ? dStrut : numerm.depth;
    }

    newOptions = options.havingStyle(dstyle);
    var denomm = buildGroup$1(group.denom, newOptions, options);
    var rule;
    var ruleWidth;
    var ruleSpacing;

    if (group.hasBarLine) {
      if (group.barSize) {
        ruleWidth = calculateSize(group.barSize, options);
        rule = buildCommon.makeLineSpan("frac-line", options, ruleWidth);
      } else {
        rule = buildCommon.makeLineSpan("frac-line", options);
      }

      ruleWidth = rule.height;
      ruleSpacing = rule.height;
    } else {
      rule = null;
      ruleWidth = 0;
      ruleSpacing = options.fontMetrics().defaultRuleThickness;
    } // Rule 15b


    var numShift;
    var clearance;
    var denomShift;

    if (style.size === Style$1.DISPLAY.size || group.size === "display") {
      numShift = options.fontMetrics().num1;

      if (ruleWidth > 0) {
        clearance = 3 * ruleSpacing;
      } else {
        clearance = 7 * ruleSpacing;
      }

      denomShift = options.fontMetrics().denom1;
    } else {
      if (ruleWidth > 0) {
        numShift = options.fontMetrics().num2;
        clearance = ruleSpacing;
      } else {
        numShift = options.fontMetrics().num3;
        clearance = 3 * ruleSpacing;
      }

      denomShift = options.fontMetrics().denom2;
    }

    var frac;

    if (!rule) {
      // Rule 15c
      var candidateClearance = numShift - numerm.depth - (denomm.height - denomShift);

      if (candidateClearance < clearance) {
        numShift += 0.5 * (clearance - candidateClearance);
        denomShift += 0.5 * (clearance - candidateClearance);
      }

      frac = buildCommon.makeVList({
        positionType: "individualShift",
        children: [{
          type: "elem",
          elem: denomm,
          shift: denomShift
        }, {
          type: "elem",
          elem: numerm,
          shift: -numShift
        }]
      }, options);
    } else {
      // Rule 15d
      var axisHeight = options.fontMetrics().axisHeight;

      if (numShift - numerm.depth - (axisHeight + 0.5 * ruleWidth) < clearance) {
        numShift += clearance - (numShift - numerm.depth - (axisHeight + 0.5 * ruleWidth));
      }

      if (axisHeight - 0.5 * ruleWidth - (denomm.height - denomShift) < clearance) {
        denomShift += clearance - (axisHeight - 0.5 * ruleWidth - (denomm.height - denomShift));
      }

      var midShift = -(axisHeight - 0.5 * ruleWidth);
      frac = buildCommon.makeVList({
        positionType: "individualShift",
        children: [{
          type: "elem",
          elem: denomm,
          shift: denomShift
        }, {
          type: "elem",
          elem: rule,
          shift: midShift
        }, {
          type: "elem",
          elem: numerm,
          shift: -numShift
        }]
      }, options);
    } // Since we manually change the style sometimes (with \dfrac or \tfrac),
    // account for the possible size change here.


    newOptions = options.havingStyle(style);
    frac.height *= newOptions.sizeMultiplier / options.sizeMultiplier;
    frac.depth *= newOptions.sizeMultiplier / options.sizeMultiplier; // Rule 15e

    var delimSize;

    if (style.size === Style$1.DISPLAY.size) {
      delimSize = options.fontMetrics().delim1;
    } else if (style.size === Style$1.SCRIPTSCRIPT.size) {
      delimSize = options.havingStyle(Style$1.SCRIPT).fontMetrics().delim2;
    } else {
      delimSize = options.fontMetrics().delim2;
    }

    var leftDelim;
    var rightDelim;

    if (group.leftDelim == null) {
      leftDelim = makeNullDelimiter(options, ["mopen"]);
    } else {
      leftDelim = delimiter.customSizedDelim(group.leftDelim, delimSize, true, options.havingStyle(style), group.mode, ["mopen"]);
    }

    if (group.continued) {
      rightDelim = buildCommon.makeSpan([]); // zero width for \cfrac
    } else if (group.rightDelim == null) {
      rightDelim = makeNullDelimiter(options, ["mclose"]);
    } else {
      rightDelim = delimiter.customSizedDelim(group.rightDelim, delimSize, true, options.havingStyle(style), group.mode, ["mclose"]);
    }

    return buildCommon.makeSpan(["mord"].concat(newOptions.sizingClasses(options)), [leftDelim, buildCommon.makeSpan(["mfrac"], [frac]), rightDelim], options);
  };

  var mathmlBuilder$3 = (group, options) => {
    var node = new mathMLTree.MathNode("mfrac", [buildGroup(group.numer, options), buildGroup(group.denom, options)]);

    if (!group.hasBarLine) {
      node.setAttribute("linethickness", "0px");
    } else if (group.barSize) {
      var ruleWidth = calculateSize(group.barSize, options);
      node.setAttribute("linethickness", makeEm(ruleWidth));
    }

    var style = adjustStyle(group.size, options.style);

    if (style.size !== options.style.size) {
      node = new mathMLTree.MathNode("mstyle", [node]);
      var isDisplay = style.size === Style$1.DISPLAY.size ? "true" : "false";
      node.setAttribute("displaystyle", isDisplay);
      node.setAttribute("scriptlevel", "0");
    }

    if (group.leftDelim != null || group.rightDelim != null) {
      var withDelims = [];

      if (group.leftDelim != null) {
        var leftOp = new mathMLTree.MathNode("mo", [new mathMLTree.TextNode(group.leftDelim.replace("\\", ""))]);
        leftOp.setAttribute("fence", "true");
        withDelims.push(leftOp);
      }

      withDelims.push(node);

      if (group.rightDelim != null) {
        var rightOp = new mathMLTree.MathNode("mo", [new mathMLTree.TextNode(group.rightDelim.replace("\\", ""))]);
        rightOp.setAttribute("fence", "true");
        withDelims.push(rightOp);
      }

      return makeRow(withDelims);
    }

    return node;
  };

  defineFunction({
    type: "genfrac",
    names: ["\\dfrac", "\\frac", "\\tfrac", "\\dbinom", "\\binom", "\\tbinom", "\\\\atopfrac", // can’t be entered directly
    "\\\\bracefrac", "\\\\brackfrac" // ditto
    ],
    props: {
      numArgs: 2,
      allowedInArgument: true
    },
    handler: (_ref, args) => {
      var {
        parser,
        funcName
      } = _ref;
      var numer = args[0];
      var denom = args[1];
      var hasBarLine;
      var leftDelim = null;
      var rightDelim = null;
      var size = "auto";

      switch (funcName) {
        case "\\dfrac":
        case "\\frac":
        case "\\tfrac":
          hasBarLine = true;
          break;

        case "\\\\atopfrac":
          hasBarLine = false;
          break;

        case "\\dbinom":
        case "\\binom":
        case "\\tbinom":
          hasBarLine = false;
          leftDelim = "(";
          rightDelim = ")";
          break;

        case "\\\\bracefrac":
          hasBarLine = false;
          leftDelim = "\\{";
          rightDelim = "\\}";
          break;

        case "\\\\brackfrac":
          hasBarLine = false;
          leftDelim = "[";
          rightDelim = "]";
          break;

        default:
          throw new Error("Unrecognized genfrac command");
      }

      switch (funcName) {
        case "\\dfrac":
        case "\\dbinom":
          size = "display";
          break;

        case "\\tfrac":
        case "\\tbinom":
          size = "text";
          break;
      }

      return {
        type: "genfrac",
        mode: parser.mode,
        continued: false,
        numer,
        denom,
        hasBarLine,
        leftDelim,
        rightDelim,
        size,
        barSize: null
      };
    },
    htmlBuilder: htmlBuilder$4,
    mathmlBuilder: mathmlBuilder$3
  });
  defineFunction({
    type: "genfrac",
    names: ["\\cfrac"],
    props: {
      numArgs: 2
    },
    handler: (_ref2, args) => {
      var {
        parser,
        funcName
      } = _ref2;
      var numer = args[0];
      var denom = args[1];
      return {
        type: "genfrac",
        mode: parser.mode,
        continued: true,
        numer,
        denom,
        hasBarLine: true,
        leftDelim: null,
        rightDelim: null,
        size: "display",
        barSize: null
      };
    }
  }); // Infix generalized fractions -- these are not rendered directly, but replaced
  // immediately by one of the variants above.

  defineFunction({
    type: "infix",
    names: ["\\over", "\\choose", "\\atop", "\\brace", "\\brack"],
    props: {
      numArgs: 0,
      infix: true
    },

    handler(_ref3) {
      var {
        parser,
        funcName,
        token
      } = _ref3;
      var replaceWith;

      switch (funcName) {
        case "\\over":
          replaceWith = "\\frac";
          break;

        case "\\choose":
          replaceWith = "\\binom";
          break;

        case "\\atop":
          replaceWith = "\\\\atopfrac";
          break;

        case "\\brace":
          replaceWith = "\\\\bracefrac";
          break;

        case "\\brack":
          replaceWith = "\\\\brackfrac";
          break;

        default:
          throw new Error("Unrecognized infix genfrac command");
      }

      return {
        type: "infix",
        mode: parser.mode,
        replaceWith,
        token
      };
    }

  });
  var stylArray = ["display", "text", "script", "scriptscript"];

  var delimFromValue = function delimFromValue(delimString) {
    var delim = null;

    if (delimString.length > 0) {
      delim = delimString;
      delim = delim === "." ? null : delim;
    }

    return delim;
  };

  defineFunction({
    type: "genfrac",
    names: ["\\genfrac"],
    props: {
      numArgs: 6,
      allowedInArgument: true,
      argTypes: ["math", "math", "size", "text", "math", "math"]
    },

    handler(_ref4, args) {
      var {
        parser
      } = _ref4;
      var numer = args[4];
      var denom = args[5]; // Look into the parse nodes to get the desired delimiters.

      var leftNode = normalizeArgument(args[0]);
      var leftDelim = leftNode.type === "atom" && leftNode.family === "open" ? delimFromValue(leftNode.text) : null;
      var rightNode = normalizeArgument(args[1]);
      var rightDelim = rightNode.type === "atom" && rightNode.family === "close" ? delimFromValue(rightNode.text) : null;
      var barNode = assertNodeType(args[2], "size");
      var hasBarLine;
      var barSize = null;

      if (barNode.isBlank) {
        // \genfrac acts differently than \above.
        // \genfrac treats an empty size group as a signal to use a
        // standard bar size. \above would see size = 0 and omit the bar.
        hasBarLine = true;
      } else {
        barSize = barNode.value;
        hasBarLine = barSize.number > 0;
      } // Find out if we want displaystyle, textstyle, etc.


      var size = "auto";
      var styl = args[3];

      if (styl.type === "ordgroup") {
        if (styl.body.length > 0) {
          var textOrd = assertNodeType(styl.body[0], "textord");
          size = stylArray[Number(textOrd.text)];
        }
      } else {
        styl = assertNodeType(styl, "textord");
        size = stylArray[Number(styl.text)];
      }

      return {
        type: "genfrac",
        mode: parser.mode,
        numer,
        denom,
        continued: false,
        hasBarLine,
        barSize,
        leftDelim,
        rightDelim,
        size
      };
    },

    htmlBuilder: htmlBuilder$4,
    mathmlBuilder: mathmlBuilder$3
  }); // \above is an infix fraction that also defines a fraction bar size.

  defineFunction({
    type: "infix",
    names: ["\\above"],
    props: {
      numArgs: 1,
      argTypes: ["size"],
      infix: true
    },

    handler(_ref5, args) {
      var {
        parser,
        funcName,
        token
      } = _ref5;
      return {
        type: "infix",
        mode: parser.mode,
        replaceWith: "\\\\abovefrac",
        size: assertNodeType(args[0], "size").value,
        token
      };
    }

  });
  defineFunction({
    type: "genfrac",
    names: ["\\\\abovefrac"],
    props: {
      numArgs: 3,
      argTypes: ["math", "size", "math"]
    },
    handler: (_ref6, args) => {
      var {
        parser,
        funcName
      } = _ref6;
      var numer = args[0];
      var barSize = assert(assertNodeType(args[1], "infix").size);
      var denom = args[2];
      var hasBarLine = barSize.number > 0;
      return {
        type: "genfrac",
        mode: parser.mode,
        numer,
        denom,
        continued: false,
        hasBarLine,
        barSize,
        leftDelim: null,
        rightDelim: null,
        size: "auto"
      };
    },
    htmlBuilder: htmlBuilder$4,
    mathmlBuilder: mathmlBuilder$3
  });

  // NOTE: Unlike most `htmlBuilder`s, this one handles not only "horizBrace", but
  // also "supsub" since an over/underbrace can affect super/subscripting.
  var htmlBuilder$3 = (grp, options) => {
    var style = options.style; // Pull out the `ParseNode<"horizBrace">` if `grp` is a "supsub" node.

    var supSubGroup;
    var group;

    if (grp.type === "supsub") {
      // Ref: LaTeX source2e: }}}}\limits}
      // i.e. LaTeX treats the brace similar to an op and passes it
      // with \limits, so we need to assign supsub style.
      supSubGroup = grp.sup ? buildGroup$1(grp.sup, options.havingStyle(style.sup()), options) : buildGroup$1(grp.sub, options.havingStyle(style.sub()), options);
      group = assertNodeType(grp.base, "horizBrace");
    } else {
      group = assertNodeType(grp, "horizBrace");
    } // Build the base group


    var body = buildGroup$1(group.base, options.havingBaseStyle(Style$1.DISPLAY)); // Create the stretchy element

    var braceBody = stretchy.svgSpan(group, options); // Generate the vlist, with the appropriate kerns        ┏━━━━━━━━┓
    // This first vlist contains the content and the brace:   equation

    var vlist;

    if (group.isOver) {
      vlist = buildCommon.makeVList({
        positionType: "firstBaseline",
        children: [{
          type: "elem",
          elem: body
        }, {
          type: "kern",
          size: 0.1
        }, {
          type: "elem",
          elem: braceBody
        }]
      }, options); // $FlowFixMe: Replace this with passing "svg-align" into makeVList.

      vlist.children[0].children[0].children[1].classes.push("svg-align");
    } else {
      vlist = buildCommon.makeVList({
        positionType: "bottom",
        positionData: body.depth + 0.1 + braceBody.height,
        children: [{
          type: "elem",
          elem: braceBody
        }, {
          type: "kern",
          size: 0.1
        }, {
          type: "elem",
          elem: body
        }]
      }, options); // $FlowFixMe: Replace this with passing "svg-align" into makeVList.

      vlist.children[0].children[0].children[0].classes.push("svg-align");
    }

    if (supSubGroup) {
      // To write the supsub, wrap the first vlist in another vlist:
      // They can't all go in the same vlist, because the note might be
      // wider than the equation. We want the equation to control the
      // brace width.
      //      note          long note           long note
      //   ┏━━━━━━━━┓   or    ┏━━━┓     not    ┏━━━━━━━━━┓
      //    equation           eqn                 eqn
      var vSpan = buildCommon.makeSpan(["mord", group.isOver ? "mover" : "munder"], [vlist], options);

      if (group.isOver) {
        vlist = buildCommon.makeVList({
          positionType: "firstBaseline",
          children: [{
            type: "elem",
            elem: vSpan
          }, {
            type: "kern",
            size: 0.2
          }, {
            type: "elem",
            elem: supSubGroup
          }]
        }, options);
      } else {
        vlist = buildCommon.makeVList({
          positionType: "bottom",
          positionData: vSpan.depth + 0.2 + supSubGroup.height + supSubGroup.depth,
          children: [{
            type: "elem",
            elem: supSubGroup
          }, {
            type: "kern",
            size: 0.2
          }, {
            type: "elem",
            elem: vSpan
          }]
        }, options);
      }
    }

    return buildCommon.makeSpan(["mord", group.isOver ? "mover" : "munder"], [vlist], options);
  };

  var mathmlBuilder$2 = (group, options) => {
    var accentNode = stretchy.mathMLnode(group.label);
    return new mathMLTree.MathNode(group.isOver ? "mover" : "munder", [buildGroup(group.base, options), accentNode]);
  }; // Horizontal stretchy braces


  defineFunction({
    type: "horizBrace",
    names: ["\\overbrace", "\\underbrace"],
    props: {
      numArgs: 1
    },

    handler(_ref, args) {
      var {
        parser,
        funcName
      } = _ref;
      return {
        type: "horizBrace",
        mode: parser.mode,
        label: funcName,
        isOver: /^\\over/.test(funcName),
        base: args[0]
      };
    },

    htmlBuilder: htmlBuilder$3,
    mathmlBuilder: mathmlBuilder$2
  });

  defineFunction({
    type: "href",
    names: ["\\href"],
    props: {
      numArgs: 2,
      argTypes: ["url", "original"],
      allowedInText: true
    },
    handler: (_ref, args) => {
      var {
        parser
      } = _ref;
      var body = args[1];
      var href = assertNodeType(args[0], "url").url;

      if (!parser.settings.isTrusted({
        command: "\\href",
        url: href
      })) {
        return parser.formatUnsupportedCmd("\\href");
      }

      return {
        type: "href",
        mode: parser.mode,
        href,
        body: ordargument(body)
      };
    },
    htmlBuilder: (group, options) => {
      var elements = buildExpression$1(group.body, options, false);
      return buildCommon.makeAnchor(group.href, [], elements, options);
    },
    mathmlBuilder: (group, options) => {
      var math = buildExpressionRow(group.body, options);

      if (!(math instanceof MathNode)) {
        math = new MathNode("mrow", [math]);
      }

      math.setAttribute("href", group.href);
      return math;
    }
  });
  defineFunction({
    type: "href",
    names: ["\\url"],
    props: {
      numArgs: 1,
      argTypes: ["url"],
      allowedInText: true
    },
    handler: (_ref2, args) => {
      var {
        parser
      } = _ref2;
      var href = assertNodeType(args[0], "url").url;

      if (!parser.settings.isTrusted({
        command: "\\url",
        url: href
      })) {
        return parser.formatUnsupportedCmd("\\url");
      }

      var chars = [];

      for (var i = 0; i < href.length; i++) {
        var c = href[i];

        if (c === "~") {
          c = "\\textasciitilde";
        }

        chars.push({
          type: "textord",
          mode: "text",
          text: c
        });
      }

      var body = {
        type: "text",
        mode: parser.mode,
        font: "\\texttt",
        body: chars
      };
      return {
        type: "href",
        mode: parser.mode,
        href,
        body: ordargument(body)
      };
    }
  });

  // In LaTeX, \vcenter can act only on a box, as in
  // \vcenter{\hbox{$\frac{a+b}{\dfrac{c}{d}}$}}
  // This function by itself doesn't do anything but prevent a soft line break.

  defineFunction({
    type: "hbox",
    names: ["\\hbox"],
    props: {
      numArgs: 1,
      argTypes: ["text"],
      allowedInText: true,
      primitive: true
    },

    handler(_ref, args) {
      var {
        parser
      } = _ref;
      return {
        type: "hbox",
        mode: parser.mode,
        body: ordargument(args[0])
      };
    },

    htmlBuilder(group, options) {
      var elements = buildExpression$1(group.body, options, false);
      return buildCommon.makeFragment(elements);
    },

    mathmlBuilder(group, options) {
      return new mathMLTree.MathNode("mrow", buildExpression(group.body, options));
    }

  });

  defineFunction({
    type: "html",
    names: ["\\htmlClass", "\\htmlId", "\\htmlStyle", "\\htmlData"],
    props: {
      numArgs: 2,
      argTypes: ["raw", "original"],
      allowedInText: true
    },
    handler: (_ref, args) => {
      var {
        parser,
        funcName,
        token
      } = _ref;
      var value = assertNodeType(args[0], "raw").string;
      var body = args[1];

      if (parser.settings.strict) {
        parser.settings.reportNonstrict("htmlExtension", "HTML extension is disabled on strict mode");
      }

      var trustContext;
      var attributes = {};

      switch (funcName) {
        case "\\htmlClass":
          attributes.class = value;
          trustContext = {
            command: "\\htmlClass",
            class: value
          };
          break;

        case "\\htmlId":
          attributes.id = value;
          trustContext = {
            command: "\\htmlId",
            id: value
          };
          break;

        case "\\htmlStyle":
          attributes.style = value;
          trustContext = {
            command: "\\htmlStyle",
            style: value
          };
          break;

        case "\\htmlData":
          {
            var data = value.split(",");

            for (var i = 0; i < data.length; i++) {
              var keyVal = data[i].split("=");

              if (keyVal.length !== 2) {
                throw new ParseError("Error parsing key-value for \\htmlData");
              }

              attributes["data-" + keyVal[0].trim()] = keyVal[1].trim();
            }

            trustContext = {
              command: "\\htmlData",
              attributes
            };
            break;
          }

        default:
          throw new Error("Unrecognized html command");
      }

      if (!parser.settings.isTrusted(trustContext)) {
        return parser.formatUnsupportedCmd(funcName);
      }

      return {
        type: "html",
        mode: parser.mode,
        attributes,
        body: ordargument(body)
      };
    },
    htmlBuilder: (group, options) => {
      var elements = buildExpression$1(group.body, options, false);
      var classes = ["enclosing"];

      if (group.attributes.class) {
        classes.push(...group.attributes.class.trim().split(/\s+/));
      }

      var span = buildCommon.makeSpan(classes, elements, options);

      for (var attr in group.attributes) {
        if (attr !== "class" && group.attributes.hasOwnProperty(attr)) {
          span.setAttribute(attr, group.attributes[attr]);
        }
      }

      return span;
    },
    mathmlBuilder: (group, options) => {
      return buildExpressionRow(group.body, options);
    }
  });

  defineFunction({
    type: "htmlmathml",
    names: ["\\html@mathml"],
    props: {
      numArgs: 2,
      allowedInText: true
    },
    handler: (_ref, args) => {
      var {
        parser
      } = _ref;
      return {
        type: "htmlmathml",
        mode: parser.mode,
        html: ordargument(args[0]),
        mathml: ordargument(args[1])
      };
    },
    htmlBuilder: (group, options) => {
      var elements = buildExpression$1(group.html, options, false);
      return buildCommon.makeFragment(elements);
    },
    mathmlBuilder: (group, options) => {
      return buildExpressionRow(group.mathml, options);
    }
  });

  var sizeData = function sizeData(str) {
    if (/^[-+]? *(\d+(\.\d*)?|\.\d+)$/.test(str)) {
      // str is a number with no unit specified.
      // default unit is bp, per graphix package.
      return {
        number: +str,
        unit: "bp"
      };
    } else {
      var match = /([-+]?) *(\d+(?:\.\d*)?|\.\d+) *([a-z]{2})/.exec(str);

      if (!match) {
        throw new ParseError("Invalid size: '" + str + "' in \\includegraphics");
      }

      var data = {
        number: +(match[1] + match[2]),
        // sign + magnitude, cast to number
        unit: match[3]
      };

      if (!validUnit(data)) {
        throw new ParseError("Invalid unit: '" + data.unit + "' in \\includegraphics.");
      }

      return data;
    }
  };

  defineFunction({
    type: "includegraphics",
    names: ["\\includegraphics"],
    props: {
      numArgs: 1,
      numOptionalArgs: 1,
      argTypes: ["raw", "url"],
      allowedInText: false
    },
    handler: (_ref, args, optArgs) => {
      var {
        parser
      } = _ref;
      var width = {
        number: 0,
        unit: "em"
      };
      var height = {
        number: 0.9,
        unit: "em"
      }; // sorta character sized.

      var totalheight = {
        number: 0,
        unit: "em"
      };
      var alt = "";

      if (optArgs[0]) {
        var attributeStr = assertNodeType(optArgs[0], "raw").string; // Parser.js does not parse key/value pairs. We get a string.

        var attributes = attributeStr.split(",");

        for (var i = 0; i < attributes.length; i++) {
          var keyVal = attributes[i].split("=");

          if (keyVal.length === 2) {
            var str = keyVal[1].trim();

            switch (keyVal[0].trim()) {
              case "alt":
                alt = str;
                break;

              case "width":
                width = sizeData(str);
                break;

              case "height":
                height = sizeData(str);
                break;

              case "totalheight":
                totalheight = sizeData(str);
                break;

              default:
                throw new ParseError("Invalid key: '" + keyVal[0] + "' in \\includegraphics.");
            }
          }
        }
      }

      var src = assertNodeType(args[0], "url").url;

      if (alt === "") {
        // No alt given. Use the file name. Strip away the path.
        alt = src;
        alt = alt.replace(/^.*[\\/]/, '');
        alt = alt.substring(0, alt.lastIndexOf('.'));
      }

      if (!parser.settings.isTrusted({
        command: "\\includegraphics",
        url: src
      })) {
        return parser.formatUnsupportedCmd("\\includegraphics");
      }

      return {
        type: "includegraphics",
        mode: parser.mode,
        alt: alt,
        width: width,
        height: height,
        totalheight: totalheight,
        src: src
      };
    },
    htmlBuilder: (group, options) => {
      var height = calculateSize(group.height, options);
      var depth = 0;

      if (group.totalheight.number > 0) {
        depth = calculateSize(group.totalheight, options) - height;
      }

      var width = 0;

      if (group.width.number > 0) {
        width = calculateSize(group.width, options);
      }

      var style = {
        height: makeEm(height + depth)
      };

      if (width > 0) {
        style.width = makeEm(width);
      }

      if (depth > 0) {
        style.verticalAlign = makeEm(-depth);
      }

      var node = new Img(group.src, group.alt, style);
      node.height = height;
      node.depth = depth;
      return node;
    },
    mathmlBuilder: (group, options) => {
      var node = new mathMLTree.MathNode("mglyph", []);
      node.setAttribute("alt", group.alt);
      var height = calculateSize(group.height, options);
      var depth = 0;

      if (group.totalheight.number > 0) {
        depth = calculateSize(group.totalheight, options) - height;
        node.setAttribute("valign", makeEm(-depth));
      }

      node.setAttribute("height", makeEm(height + depth));

      if (group.width.number > 0) {
        var width = calculateSize(group.width, options);
        node.setAttribute("width", makeEm(width));
      }

      node.setAttribute("src", group.src);
      return node;
    }
  });

  // Horizontal spacing commands

  defineFunction({
    type: "kern",
    names: ["\\kern", "\\mkern", "\\hskip", "\\mskip"],
    props: {
      numArgs: 1,
      argTypes: ["size"],
      primitive: true,
      allowedInText: true
    },

    handler(_ref, args) {
      var {
        parser,
        funcName
      } = _ref;
      var size = assertNodeType(args[0], "size");

      if (parser.settings.strict) {
        var mathFunction = funcName[1] === 'm'; // \mkern, \mskip

        var muUnit = size.value.unit === 'mu';

        if (mathFunction) {
          if (!muUnit) {
            parser.settings.reportNonstrict("mathVsTextUnits", "LaTeX's " + funcName + " supports only mu units, " + ("not " + size.value.unit + " units"));
          }

          if (parser.mode !== "math") {
            parser.settings.reportNonstrict("mathVsTextUnits", "LaTeX's " + funcName + " works only in math mode");
          }
        } else {
          // !mathFunction
          if (muUnit) {
            parser.settings.reportNonstrict("mathVsTextUnits", "LaTeX's " + funcName + " doesn't support mu units");
          }
        }
      }

      return {
        type: "kern",
        mode: parser.mode,
        dimension: size.value
      };
    },

    htmlBuilder(group, options) {
      return buildCommon.makeGlue(group.dimension, options);
    },

    mathmlBuilder(group, options) {
      var dimension = calculateSize(group.dimension, options);
      return new mathMLTree.SpaceNode(dimension);
    }

  });

  // Horizontal overlap functions
  defineFunction({
    type: "lap",
    names: ["\\mathllap", "\\mathrlap", "\\mathclap"],
    props: {
      numArgs: 1,
      allowedInText: true
    },
    handler: (_ref, args) => {
      var {
        parser,
        funcName
      } = _ref;
      var body = args[0];
      return {
        type: "lap",
        mode: parser.mode,
        alignment: funcName.slice(5),
        body
      };
    },
    htmlBuilder: (group, options) => {
      // mathllap, mathrlap, mathclap
      var inner;

      if (group.alignment === "clap") {
        // ref: https://www.math.lsu.edu/~aperlis/publications/mathclap/
        inner = buildCommon.makeSpan([], [buildGroup$1(group.body, options)]); // wrap, since CSS will center a .clap > .inner > span

        inner = buildCommon.makeSpan(["inner"], [inner], options);
      } else {
        inner = buildCommon.makeSpan(["inner"], [buildGroup$1(group.body, options)]);
      }

      var fix = buildCommon.makeSpan(["fix"], []);
      var node = buildCommon.makeSpan([group.alignment], [inner, fix], options); // At this point, we have correctly set horizontal alignment of the
      // two items involved in the lap.
      // Next, use a strut to set the height of the HTML bounding box.
      // Otherwise, a tall argument may be misplaced.
      // This code resolved issue #1153

      var strut = buildCommon.makeSpan(["strut"]);
      strut.style.height = makeEm(node.height + node.depth);

      if (node.depth) {
        strut.style.verticalAlign = makeEm(-node.depth);
      }

      node.children.unshift(strut); // Next, prevent vertical misplacement when next to something tall.
      // This code resolves issue #1234

      node = buildCommon.makeSpan(["thinbox"], [node], options);
      return buildCommon.makeSpan(["mord", "vbox"], [node], options);
    },
    mathmlBuilder: (group, options) => {
      // mathllap, mathrlap, mathclap
      var node = new mathMLTree.MathNode("mpadded", [buildGroup(group.body, options)]);

      if (group.alignment !== "rlap") {
        var offset = group.alignment === "llap" ? "-1" : "-0.5";
        node.setAttribute("lspace", offset + "width");
      }

      node.setAttribute("width", "0px");
      return node;
    }
  });

  defineFunction({
    type: "styling",
    names: ["\\(", "$"],
    props: {
      numArgs: 0,
      allowedInText: true,
      allowedInMath: false
    },

    handler(_ref, args) {
      var {
        funcName,
        parser
      } = _ref;
      var outerMode = parser.mode;
      parser.switchMode("math");
      var close = funcName === "\\(" ? "\\)" : "$";
      var body = parser.parseExpression(false, close);
      parser.expect(close);
      parser.switchMode(outerMode);
      return {
        type: "styling",
        mode: parser.mode,
        style: "text",
        body
      };
    }

  }); // Check for extra closing math delimiters

  defineFunction({
    type: "text",
    // Doesn't matter what this is.
    names: ["\\)", "\\]"],
    props: {
      numArgs: 0,
      allowedInText: true,
      allowedInMath: false
    },

    handler(context, args) {
      throw new ParseError("Mismatched " + context.funcName);
    }

  });

  var chooseMathStyle = (group, options) => {
    switch (options.style.size) {
      case Style$1.DISPLAY.size:
        return group.display;

      case Style$1.TEXT.size:
        return group.text;

      case Style$1.SCRIPT.size:
        return group.script;

      case Style$1.SCRIPTSCRIPT.size:
        return group.scriptscript;

      default:
        return group.text;
    }
  };

  defineFunction({
    type: "mathchoice",
    names: ["\\mathchoice"],
    props: {
      numArgs: 4,
      primitive: true
    },
    handler: (_ref, args) => {
      var {
        parser
      } = _ref;
      return {
        type: "mathchoice",
        mode: parser.mode,
        display: ordargument(args[0]),
        text: ordargument(args[1]),
        script: ordargument(args[2]),
        scriptscript: ordargument(args[3])
      };
    },
    htmlBuilder: (group, options) => {
      var body = chooseMathStyle(group, options);
      var elements = buildExpression$1(body, options, false);
      return buildCommon.makeFragment(elements);
    },
    mathmlBuilder: (group, options) => {
      var body = chooseMathStyle(group, options);
      return buildExpressionRow(body, options);
    }
  });

  var assembleSupSub = (base, supGroup, subGroup, options, style, slant, baseShift) => {
    base = buildCommon.makeSpan([], [base]);
    var subIsSingleCharacter = subGroup && utils$1.isCharacterBox(subGroup);
    var sub;
    var sup; // We manually have to handle the superscripts and subscripts. This,
    // aside from the kern calculations, is copied from supsub.

    if (supGroup) {
      var elem = buildGroup$1(supGroup, options.havingStyle(style.sup()), options);
      sup = {
        elem,
        kern: Math.max(options.fontMetrics().bigOpSpacing1, options.fontMetrics().bigOpSpacing3 - elem.depth)
      };
    }

    if (subGroup) {
      var _elem = buildGroup$1(subGroup, options.havingStyle(style.sub()), options);

      sub = {
        elem: _elem,
        kern: Math.max(options.fontMetrics().bigOpSpacing2, options.fontMetrics().bigOpSpacing4 - _elem.height)
      };
    } // Build the final group as a vlist of the possible subscript, base,
    // and possible superscript.


    var finalGroup;

    if (sup && sub) {
      var bottom = options.fontMetrics().bigOpSpacing5 + sub.elem.height + sub.elem.depth + sub.kern + base.depth + baseShift;
      finalGroup = buildCommon.makeVList({
        positionType: "bottom",
        positionData: bottom,
        children: [{
          type: "kern",
          size: options.fontMetrics().bigOpSpacing5
        }, {
          type: "elem",
          elem: sub.elem,
          marginLeft: makeEm(-slant)
        }, {
          type: "kern",
          size: sub.kern
        }, {
          type: "elem",
          elem: base
        }, {
          type: "kern",
          size: sup.kern
        }, {
          type: "elem",
          elem: sup.elem,
          marginLeft: makeEm(slant)
        }, {
          type: "kern",
          size: options.fontMetrics().bigOpSpacing5
        }]
      }, options);
    } else if (sub) {
      var top = base.height - baseShift; // Shift the limits by the slant of the symbol. Note
      // that we are supposed to shift the limits by 1/2 of the slant,
      // but since we are centering the limits adding a full slant of
      // margin will shift by 1/2 that.

      finalGroup = buildCommon.makeVList({
        positionType: "top",
        positionData: top,
        children: [{
          type: "kern",
          size: options.fontMetrics().bigOpSpacing5
        }, {
          type: "elem",
          elem: sub.elem,
          marginLeft: makeEm(-slant)
        }, {
          type: "kern",
          size: sub.kern
        }, {
          type: "elem",
          elem: base
        }]
      }, options);
    } else if (sup) {
      var _bottom = base.depth + baseShift;

      finalGroup = buildCommon.makeVList({
        positionType: "bottom",
        positionData: _bottom,
        children: [{
          type: "elem",
          elem: base
        }, {
          type: "kern",
          size: sup.kern
        }, {
          type: "elem",
          elem: sup.elem,
          marginLeft: makeEm(slant)
        }, {
          type: "kern",
          size: options.fontMetrics().bigOpSpacing5
        }]
      }, options);
    } else {
      // This case probably shouldn't occur (this would mean the
      // supsub was sending us a group with no superscript or
      // subscript) but be safe.
      return base;
    }

    var parts = [finalGroup];

    if (sub && slant !== 0 && !subIsSingleCharacter) {
      // A negative margin-left was applied to the lower limit.
      // Avoid an overlap by placing a spacer on the left on the group.
      var spacer = buildCommon.makeSpan(["mspace"], [], options);
      spacer.style.marginRight = makeEm(slant);
      parts.unshift(spacer);
    }

    return buildCommon.makeSpan(["mop", "op-limits"], parts, options);
  };

  // Limits, symbols
  // Most operators have a large successor symbol, but these don't.
  var noSuccessor = ["\\smallint"]; // NOTE: Unlike most `htmlBuilder`s, this one handles not only "op", but also
  // "supsub" since some of them (like \int) can affect super/subscripting.

  var htmlBuilder$2 = (grp, options) => {
    // Operators are handled in the TeXbook pg. 443-444, rule 13(a).
    var supGroup;
    var subGroup;
    var hasLimits = false;
    var group;

    if (grp.type === "supsub") {
      // If we have limits, supsub will pass us its group to handle. Pull
      // out the superscript and subscript and set the group to the op in
      // its base.
      supGroup = grp.sup;
      subGroup = grp.sub;
      group = assertNodeType(grp.base, "op");
      hasLimits = true;
    } else {
      group = assertNodeType(grp, "op");
    }

    var style = options.style;
    var large = false;

    if (style.size === Style$1.DISPLAY.size && group.symbol && !utils$1.contains(noSuccessor, group.name)) {
      // Most symbol operators get larger in displaystyle (rule 13)
      large = true;
    }

    var base;

    if (group.symbol) {
      // If this is a symbol, create the symbol.
      var fontName = large ? "Size2-Regular" : "Size1-Regular";
      var stash = "";

      if (group.name === "\\oiint" || group.name === "\\oiiint") {
        // No font glyphs yet, so use a glyph w/o the oval.
        // TODO: When font glyphs are available, delete this code.
        stash = group.name.substr(1);
        group.name = stash === "oiint" ? "\\iint" : "\\iiint";
      }

      base = buildCommon.makeSymbol(group.name, fontName, "math", options, ["mop", "op-symbol", large ? "large-op" : "small-op"]);

      if (stash.length > 0) {
        // We're in \oiint or \oiiint. Overlay the oval.
        // TODO: When font glyphs are available, delete this code.
        var italic = base.italic;
        var oval = buildCommon.staticSvg(stash + "Size" + (large ? "2" : "1"), options);
        base = buildCommon.makeVList({
          positionType: "individualShift",
          children: [{
            type: "elem",
            elem: base,
            shift: 0
          }, {
            type: "elem",
            elem: oval,
            shift: large ? 0.08 : 0
          }]
        }, options);
        group.name = "\\" + stash;
        base.classes.unshift("mop"); // $FlowFixMe

        base.italic = italic;
      }
    } else if (group.body) {
      // If this is a list, compose that list.
      var inner = buildExpression$1(group.body, options, true);

      if (inner.length === 1 && inner[0] instanceof SymbolNode) {
        base = inner[0];
        base.classes[0] = "mop"; // replace old mclass
      } else {
        base = buildCommon.makeSpan(["mop"], inner, options);
      }
    } else {
      // Otherwise, this is a text operator. Build the text from the
      // operator's name.
      var output = [];

      for (var i = 1; i < group.name.length; i++) {
        output.push(buildCommon.mathsym(group.name[i], group.mode, options));
      }

      base = buildCommon.makeSpan(["mop"], output, options);
    } // If content of op is a single symbol, shift it vertically.


    var baseShift = 0;
    var slant = 0;

    if ((base instanceof SymbolNode || group.name === "\\oiint" || group.name === "\\oiiint") && !group.suppressBaseShift) {
      // We suppress the shift of the base of \overset and \underset. Otherwise,
      // shift the symbol so its center lies on the axis (rule 13). It
      // appears that our fonts have the centers of the symbols already
      // almost on the axis, so these numbers are very small. Note we
      // don't actually apply this here, but instead it is used either in
      // the vlist creation or separately when there are no limits.
      baseShift = (base.height - base.depth) / 2 - options.fontMetrics().axisHeight; // The slant of the symbol is just its italic correction.
      // $FlowFixMe

      slant = base.italic;
    }

    if (hasLimits) {
      return assembleSupSub(base, supGroup, subGroup, options, style, slant, baseShift);
    } else {
      if (baseShift) {
        base.style.position = "relative";
        base.style.top = makeEm(baseShift);
      }

      return base;
    }
  };

  var mathmlBuilder$1 = (group, options) => {
    var node;

    if (group.symbol) {
      // This is a symbol. Just add the symbol.
      node = new MathNode("mo", [makeText(group.name, group.mode)]);

      if (utils$1.contains(noSuccessor, group.name)) {
        node.setAttribute("largeop", "false");
      }
    } else if (group.body) {
      // This is an operator with children. Add them.
      node = new MathNode("mo", buildExpression(group.body, options));
    } else {
      // This is a text operator. Add all of the characters from the
      // operator's name.
      node = new MathNode("mi", [new TextNode(group.name.slice(1))]); // Append an <mo>&ApplyFunction;</mo>.
      // ref: https://www.w3.org/TR/REC-MathML/chap3_2.html#sec3.2.4

      var operator = new MathNode("mo", [makeText("\u2061", "text")]);

      if (group.parentIsSupSub) {
        node = new MathNode("mrow", [node, operator]);
      } else {
        node = newDocumentFragment([node, operator]);
      }
    }

    return node;
  };

  var singleCharBigOps = {
    "\u220F": "\\prod",
    "\u2210": "\\coprod",
    "\u2211": "\\sum",
    "\u22c0": "\\bigwedge",
    "\u22c1": "\\bigvee",
    "\u22c2": "\\bigcap",
    "\u22c3": "\\bigcup",
    "\u2a00": "\\bigodot",
    "\u2a01": "\\bigoplus",
    "\u2a02": "\\bigotimes",
    "\u2a04": "\\biguplus",
    "\u2a06": "\\bigsqcup"
  };
  defineFunction({
    type: "op",
    names: ["\\coprod", "\\bigvee", "\\bigwedge", "\\biguplus", "\\bigcap", "\\bigcup", "\\intop", "\\prod", "\\sum", "\\bigotimes", "\\bigoplus", "\\bigodot", "\\bigsqcup", "\\smallint", "\u220F", "\u2210", "\u2211", "\u22c0", "\u22c1", "\u22c2", "\u22c3", "\u2a00", "\u2a01", "\u2a02", "\u2a04", "\u2a06"],
    props: {
      numArgs: 0
    },
    handler: (_ref, args) => {
      var {
        parser,
        funcName
      } = _ref;
      var fName = funcName;

      if (fName.length === 1) {
        fName = singleCharBigOps[fName];
      }

      return {
        type: "op",
        mode: parser.mode,
        limits: true,
        parentIsSupSub: false,
        symbol: true,
        name: fName
      };
    },
    htmlBuilder: htmlBuilder$2,
    mathmlBuilder: mathmlBuilder$1
  }); // Note: calling defineFunction with a type that's already been defined only
  // works because the same htmlBuilder and mathmlBuilder are being used.

  defineFunction({
    type: "op",
    names: ["\\mathop"],
    props: {
      numArgs: 1,
      primitive: true
    },
    handler: (_ref2, args) => {
      var {
        parser
      } = _ref2;
      var body = args[0];
      return {
        type: "op",
        mode: parser.mode,
        limits: false,
        parentIsSupSub: false,
        symbol: false,
        body: ordargument(body)
      };
    },
    htmlBuilder: htmlBuilder$2,
    mathmlBuilder: mathmlBuilder$1
  }); // There are 2 flags for operators; whether they produce limits in
  // displaystyle, and whether they are symbols and should grow in
  // displaystyle. These four groups cover the four possible choices.

  var singleCharIntegrals = {
    "\u222b": "\\int",
    "\u222c": "\\iint",
    "\u222d": "\\iiint",
    "\u222e": "\\oint",
    "\u222f": "\\oiint",
    "\u2230": "\\oiiint"
  }; // No limits, not symbols

  defineFunction({
    type: "op",
    names: ["\\arcsin", "\\arccos", "\\arctan", "\\arctg", "\\arcctg", "\\arg", "\\ch", "\\cos", "\\cosec", "\\cosh", "\\cot", "\\cotg", "\\coth", "\\csc", "\\ctg", "\\cth", "\\deg", "\\dim", "\\exp", "\\hom", "\\ker", "\\lg", "\\ln", "\\log", "\\sec", "\\sin", "\\sinh", "\\sh", "\\tan", "\\tanh", "\\tg", "\\th"],
    props: {
      numArgs: 0
    },

    handler(_ref3) {
      var {
        parser,
        funcName
      } = _ref3;
      return {
        type: "op",
        mode: parser.mode,
        limits: false,
        parentIsSupSub: false,
        symbol: false,
        name: funcName
      };
    },

    htmlBuilder: htmlBuilder$2,
    mathmlBuilder: mathmlBuilder$1
  }); // Limits, not symbols

  defineFunction({
    type: "op",
    names: ["\\det", "\\gcd", "\\inf", "\\lim", "\\max", "\\min", "\\Pr", "\\sup"],
    props: {
      numArgs: 0
    },

    handler(_ref4) {
      var {
        parser,
        funcName
      } = _ref4;
      return {
        type: "op",
        mode: parser.mode,
        limits: true,
        parentIsSupSub: false,
        symbol: false,
        name: funcName
      };
    },

    htmlBuilder: htmlBuilder$2,
    mathmlBuilder: mathmlBuilder$1
  }); // No limits, symbols

  defineFunction({
    type: "op",
    names: ["\\int", "\\iint", "\\iiint", "\\oint", "\\oiint", "\\oiiint", "\u222b", "\u222c", "\u222d", "\u222e", "\u222f", "\u2230"],
    props: {
      numArgs: 0
    },

    handler(_ref5) {
      var {
        parser,
        funcName
      } = _ref5;
      var fName = funcName;

      if (fName.length === 1) {
        fName = singleCharIntegrals[fName];
      }

      return {
        type: "op",
        mode: parser.mode,
        limits: false,
        parentIsSupSub: false,
        symbol: true,
        name: fName
      };
    },

    htmlBuilder: htmlBuilder$2,
    mathmlBuilder: mathmlBuilder$1
  });

  // NOTE: Unlike most `htmlBuilder`s, this one handles not only
  // "operatorname", but also  "supsub" since \operatorname* can
  // affect super/subscripting.
  var htmlBuilder$1 = (grp, options) => {
    // Operators are handled in the TeXbook pg. 443-444, rule 13(a).
    var supGroup;
    var subGroup;
    var hasLimits = false;
    var group;

    if (grp.type === "supsub") {
      // If we have limits, supsub will pass us its group to handle. Pull
      // out the superscript and subscript and set the group to the op in
      // its base.
      supGroup = grp.sup;
      subGroup = grp.sub;
      group = assertNodeType(grp.base, "operatorname");
      hasLimits = true;
    } else {
      group = assertNodeType(grp, "operatorname");
    }

    var base;

    if (group.body.length > 0) {
      var body = group.body.map(child => {
        // $FlowFixMe: Check if the node has a string `text` property.
        var childText = child.text;

        if (typeof childText === "string") {
          return {
            type: "textord",
            mode: child.mode,
            text: childText
          };
        } else {
          return child;
        }
      }); // Consolidate function names into symbol characters.

      var expression = buildExpression$1(body, options.withFont("mathrm"), true);

      for (var i = 0; i < expression.length; i++) {
        var child = expression[i];

        if (child instanceof SymbolNode) {
          // Per amsopn package,
          // change minus to hyphen and \ast to asterisk
          child.text = child.text.replace(/\u2212/, "-").replace(/\u2217/, "*");
        }
      }

      base = buildCommon.makeSpan(["mop"], expression, options);
    } else {
      base = buildCommon.makeSpan(["mop"], [], options);
    }

    if (hasLimits) {
      return assembleSupSub(base, supGroup, subGroup, options, options.style, 0, 0);
    } else {
      return base;
    }
  };

  var mathmlBuilder = (group, options) => {
    // The steps taken here are similar to the html version.
    var expression = buildExpression(group.body, options.withFont("mathrm")); // Is expression a string or has it something like a fraction?

    var isAllString = true; // default

    for (var i = 0; i < expression.length; i++) {
      var node = expression[i];

      if (node instanceof mathMLTree.SpaceNode) ; else if (node instanceof mathMLTree.MathNode) {
        switch (node.type) {
          case "mi":
          case "mn":
          case "ms":
          case "mspace":
          case "mtext":
            break;
          // Do nothing yet.

          case "mo":
            {
              var child = node.children[0];

              if (node.children.length === 1 && child instanceof mathMLTree.TextNode) {
                child.text = child.text.replace(/\u2212/, "-").replace(/\u2217/, "*");
              } else {
                isAllString = false;
              }

              break;
            }

          default:
            isAllString = false;
        }
      } else {
        isAllString = false;
      }
    }

    if (isAllString) {
      // Write a single TextNode instead of multiple nested tags.
      var word = expression.map(node => node.toText()).join("");
      expression = [new mathMLTree.TextNode(word)];
    }

    var identifier = new mathMLTree.MathNode("mi", expression);
    identifier.setAttribute("mathvariant", "normal"); // \u2061 is the same as &ApplyFunction;
    // ref: https://www.w3schools.com/charsets/ref_html_entities_a.asp

    var operator = new mathMLTree.MathNode("mo", [makeText("\u2061", "text")]);

    if (group.parentIsSupSub) {
      return new mathMLTree.MathNode("mrow", [identifier, operator]);
    } else {
      return mathMLTree.newDocumentFragment([identifier, operator]);
    }
  }; // \operatorname
  // amsopn.dtx: \mathop{#1\kern\z@\operator@font#3}\newmcodes@


  defineFunction({
    type: "operatorname",
    names: ["\\operatorname@", "\\operatornamewithlimits"],
    props: {
      numArgs: 1
    },
    handler: (_ref, args) => {
      var {
        parser,
        funcName
      } = _ref;
      var body = args[0];
      return {
        type: "operatorname",
        mode: parser.mode,
        body: ordargument(body),
        alwaysHandleSupSub: funcName === "\\operatornamewithlimits",
        limits: false,
        parentIsSupSub: false
      };
    },
    htmlBuilder: htmlBuilder$1,
    mathmlBuilder
  });
  defineMacro("\\operatorname", "\\@ifstar\\operatornamewithlimits\\operatorname@");

  defineFunctionBuilders({
    type: "ordgroup",

    htmlBuilder(group, options) {
      if (group.semisimple) {
        return buildCommon.makeFragment(buildExpression$1(group.body, options, false));
      }

      return buildCommon.makeSpan(["mord"], buildExpression$1(group.body, options, true), options);
    },

    mathmlBuilder(group, options) {
      return buildExpressionRow(group.body, options, true);
    }

  });

  defineFunction({
    type: "overline",
    names: ["\\overline"],
    props: {
      numArgs: 1
    },

    handler(_ref, args) {
      var {
        parser
      } = _ref;
      var body = args[0];
      return {
        type: "overline",
        mode: parser.mode,
        body
      };
    },

    htmlBuilder(group, options) {
      // Overlines are handled in the TeXbook pg 443, Rule 9.
      // Build the inner group in the cramped style.
      var innerGroup = buildGroup$1(group.body, options.havingCrampedStyle()); // Create the line above the body

      var line = buildCommon.makeLineSpan("overline-line", options); // Generate the vlist, with the appropriate kerns

      var defaultRuleThickness = options.fontMetrics().defaultRuleThickness;
      var vlist = buildCommon.makeVList({
        positionType: "firstBaseline",
        children: [{
          type: "elem",
          elem: innerGroup
        }, {
          type: "kern",
          size: 3 * defaultRuleThickness
        }, {
          type: "elem",
          elem: line
        }, {
          type: "kern",
          size: defaultRuleThickness
        }]
      }, options);
      return buildCommon.makeSpan(["mord", "overline"], [vlist], options);
    },

    mathmlBuilder(group, options) {
      var operator = new mathMLTree.MathNode("mo", [new mathMLTree.TextNode("\u203e")]);
      operator.setAttribute("stretchy", "true");
      var node = new mathMLTree.MathNode("mover", [buildGroup(group.body, options), operator]);
      node.setAttribute("accent", "true");
      return node;
    }

  });

  defineFunction({
    type: "phantom",
    names: ["\\phantom"],
    props: {
      numArgs: 1,
      allowedInText: true
    },
    handler: (_ref, args) => {
      var {
        parser
      } = _ref;
      var body = args[0];
      return {
        type: "phantom",
        mode: parser.mode,
        body: ordargument(body)
      };
    },
    htmlBuilder: (group, options) => {
      var elements = buildExpression$1(group.body, options.withPhantom(), false); // \phantom isn't supposed to affect the elements it contains.
      // See "color" for more details.

      return buildCommon.makeFragment(elements);
    },
    mathmlBuilder: (group, options) => {
      var inner = buildExpression(group.body, options);
      return new mathMLTree.MathNode("mphantom", inner);
    }
  });
  defineFunction({
    type: "hphantom",
    names: ["\\hphantom"],
    props: {
      numArgs: 1,
      allowedInText: true
    },
    handler: (_ref2, args) => {
      var {
        parser
      } = _ref2;
      var body = args[0];
      return {
        type: "hphantom",
        mode: parser.mode,
        body
      };
    },
    htmlBuilder: (group, options) => {
      var node = buildCommon.makeSpan([], [buildGroup$1(group.body, options.withPhantom())]);
      node.height = 0;
      node.depth = 0;

      if (node.children) {
        for (var i = 0; i < node.children.length; i++) {
          node.children[i].height = 0;
          node.children[i].depth = 0;
        }
      } // See smash for comment re: use of makeVList


      node = buildCommon.makeVList({
        positionType: "firstBaseline",
        children: [{
          type: "elem",
          elem: node
        }]
      }, options); // For spacing, TeX treats \smash as a math group (same spacing as ord).

      return buildCommon.makeSpan(["mord"], [node], options);
    },
    mathmlBuilder: (group, options) => {
      var inner = buildExpression(ordargument(group.body), options);
      var phantom = new mathMLTree.MathNode("mphantom", inner);
      var node = new mathMLTree.MathNode("mpadded", [phantom]);
      node.setAttribute("height", "0px");
      node.setAttribute("depth", "0px");
      return node;
    }
  });
  defineFunction({
    type: "vphantom",
    names: ["\\vphantom"],
    props: {
      numArgs: 1,
      allowedInText: true
    },
    handler: (_ref3, args) => {
      var {
        parser
      } = _ref3;
      var body = args[0];
      return {
        type: "vphantom",
        mode: parser.mode,
        body
      };
    },
    htmlBuilder: (group, options) => {
      var inner = buildCommon.makeSpan(["inner"], [buildGroup$1(group.body, options.withPhantom())]);
      var fix = buildCommon.makeSpan(["fix"], []);
      return buildCommon.makeSpan(["mord", "rlap"], [inner, fix], options);
    },
    mathmlBuilder: (group, options) => {
      var inner = buildExpression(ordargument(group.body), options);
      var phantom = new mathMLTree.MathNode("mphantom", inner);
      var node = new mathMLTree.MathNode("mpadded", [phantom]);
      node.setAttribute("width", "0px");
      return node;
    }
  });

  defineFunction({
    type: "raisebox",
    names: ["\\raisebox"],
    props: {
      numArgs: 2,
      argTypes: ["size", "hbox"],
      allowedInText: true
    },

    handler(_ref, args) {
      var {
        parser
      } = _ref;
      var amount = assertNodeType(args[0], "size").value;
      var body = args[1];
      return {
        type: "raisebox",
        mode: parser.mode,
        dy: amount,
        body
      };
    },

    htmlBuilder(group, options) {
      var body = buildGroup$1(group.body, options);
      var dy = calculateSize(group.dy, options);
      return buildCommon.makeVList({
        positionType: "shift",
        positionData: -dy,
        children: [{
          type: "elem",
          elem: body
        }]
      }, options);
    },

    mathmlBuilder(group, options) {
      var node = new mathMLTree.MathNode("mpadded", [buildGroup(group.body, options)]);
      var dy = group.dy.number + group.dy.unit;
      node.setAttribute("voffset", dy);
      return node;
    }

  });

  defineFunction({
    type: "internal",
    names: ["\\relax"],
    props: {
      numArgs: 0,
      allowedInText: true
    },

    handler(_ref) {
      var {
        parser
      } = _ref;
      return {
        type: "internal",
        mode: parser.mode
      };
    }

  });

  defineFunction({
    type: "rule",
    names: ["\\rule"],
    props: {
      numArgs: 2,
      numOptionalArgs: 1,
      argTypes: ["size", "size", "size"]
    },

    handler(_ref, args, optArgs) {
      var {
        parser
      } = _ref;
      var shift = optArgs[0];
      var width = assertNodeType(args[0], "size");
      var height = assertNodeType(args[1], "size");
      return {
        type: "rule",
        mode: parser.mode,
        shift: shift && assertNodeType(shift, "size").value,
        width: width.value,
        height: height.value
      };
    },

    htmlBuilder(group, options) {
      // Make an empty span for the rule
      var rule = buildCommon.makeSpan(["mord", "rule"], [], options); // Calculate the shift, width, and height of the rule, and account for units

      var width = calculateSize(group.width, options);
      var height = calculateSize(group.height, options);
      var shift = group.shift ? calculateSize(group.shift, options) : 0; // Style the rule to the right size

      rule.style.borderRightWidth = makeEm(width);
      rule.style.borderTopWidth = makeEm(height);
      rule.style.bottom = makeEm(shift); // Record the height and width

      rule.width = width;
      rule.height = height + shift;
      rule.depth = -shift; // Font size is the number large enough that the browser will
      // reserve at least `absHeight` space above the baseline.
      // The 1.125 factor was empirically determined

      rule.maxFontSize = height * 1.125 * options.sizeMultiplier;
      return rule;
    },

    mathmlBuilder(group, options) {
      var width = calculateSize(group.width, options);
      var height = calculateSize(group.height, options);
      var shift = group.shift ? calculateSize(group.shift, options) : 0;
      var color = options.color && options.getColor() || "black";
      var rule = new mathMLTree.MathNode("mspace");
      rule.setAttribute("mathbackground", color);
      rule.setAttribute("width", makeEm(width));
      rule.setAttribute("height", makeEm(height));
      var wrapper = new mathMLTree.MathNode("mpadded", [rule]);

      if (shift >= 0) {
        wrapper.setAttribute("height", makeEm(shift));
      } else {
        wrapper.setAttribute("height", makeEm(shift));
        wrapper.setAttribute("depth", makeEm(-shift));
      }

      wrapper.setAttribute("voffset", makeEm(shift));
      return wrapper;
    }

  });

  function sizingGroup(value, options, baseOptions) {
    var inner = buildExpression$1(value, options, false);
    var multiplier = options.sizeMultiplier / baseOptions.sizeMultiplier; // Add size-resetting classes to the inner list and set maxFontSize
    // manually. Handle nested size changes.

    for (var i = 0; i < inner.length; i++) {
      var pos = inner[i].classes.indexOf("sizing");

      if (pos < 0) {
        Array.prototype.push.apply(inner[i].classes, options.sizingClasses(baseOptions));
      } else if (inner[i].classes[pos + 1] === "reset-size" + options.size) {
        // This is a nested size change: e.g., inner[i] is the "b" in
        // `\Huge a \small b`. Override the old size (the `reset-` class)
        // but not the new size.
        inner[i].classes[pos + 1] = "reset-size" + baseOptions.size;
      }

      inner[i].height *= multiplier;
      inner[i].depth *= multiplier;
    }

    return buildCommon.makeFragment(inner);
  }
  var sizeFuncs = ["\\tiny", "\\sixptsize", "\\scriptsize", "\\footnotesize", "\\small", "\\normalsize", "\\large", "\\Large", "\\LARGE", "\\huge", "\\Huge"];
  var htmlBuilder = (group, options) => {
    // Handle sizing operators like \Huge. Real TeX doesn't actually allow
    // these functions inside of math expressions, so we do some special
    // handling.
    var newOptions = options.havingSize(group.size);
    return sizingGroup(group.body, newOptions, options);
  };
  defineFunction({
    type: "sizing",
    names: sizeFuncs,
    props: {
      numArgs: 0,
      allowedInText: true
    },
    handler: (_ref, args) => {
      var {
        breakOnTokenText,
        funcName,
        parser
      } = _ref;
      var body = parser.parseExpression(false, breakOnTokenText);
      return {
        type: "sizing",
        mode: parser.mode,
        // Figure out what size to use based on the list of functions above
        size: sizeFuncs.indexOf(funcName) + 1,
        body
      };
    },
    htmlBuilder,
    mathmlBuilder: (group, options) => {
      var newOptions = options.havingSize(group.size);
      var inner = buildExpression(group.body, newOptions);
      var node = new mathMLTree.MathNode("mstyle", inner); // TODO(emily): This doesn't produce the correct size for nested size
      // changes, because we don't keep state of what style we're currently
      // in, so we can't reset the size to normal before changing it.  Now
      // that we're passing an options parameter we should be able to fix
      // this.

      node.setAttribute("mathsize", makeEm(newOptions.sizeMultiplier));
      return node;
    }
  });

  // smash, with optional [tb], as in AMS
  defineFunction({
    type: "smash",
    names: ["\\smash"],
    props: {
      numArgs: 1,
      numOptionalArgs: 1,
      allowedInText: true
    },
    handler: (_ref, args, optArgs) => {
      var {
        parser
      } = _ref;
      var smashHeight = false;
      var smashDepth = false;
      var tbArg = optArgs[0] && assertNodeType(optArgs[0], "ordgroup");

      if (tbArg) {
        // Optional [tb] argument is engaged.
        // ref: amsmath: \renewcommand{\smash}[1][tb]{%
        //               def\mb@t{\ht}\def\mb@b{\dp}\def\mb@tb{\ht\z@\z@\dp}%
        var letter = "";

        for (var i = 0; i < tbArg.body.length; ++i) {
          var node = tbArg.body[i]; // $FlowFixMe: Not every node type has a `text` property.

          letter = node.text;

          if (letter === "t") {
            smashHeight = true;
          } else if (letter === "b") {
            smashDepth = true;
          } else {
            smashHeight = false;
            smashDepth = false;
            break;
          }
        }
      } else {
        smashHeight = true;
        smashDepth = true;
      }

      var body = args[0];
      return {
        type: "smash",
        mode: parser.mode,
        body,
        smashHeight,
        smashDepth
      };
    },
    htmlBuilder: (group, options) => {
      var node = buildCommon.makeSpan([], [buildGroup$1(group.body, options)]);

      if (!group.smashHeight && !group.smashDepth) {
        return node;
      }

      if (group.smashHeight) {
        node.height = 0; // In order to influence makeVList, we have to reset the children.

        if (node.children) {
          for (var i = 0; i < node.children.length; i++) {
            node.children[i].height = 0;
          }
        }
      }

      if (group.smashDepth) {
        node.depth = 0;

        if (node.children) {
          for (var _i = 0; _i < node.children.length; _i++) {
            node.children[_i].depth = 0;
          }
        }
      } // At this point, we've reset the TeX-like height and depth values.
      // But the span still has an HTML line height.
      // makeVList applies "display: table-cell", which prevents the browser
      // from acting on that line height. So we'll call makeVList now.


      var smashedNode = buildCommon.makeVList({
        positionType: "firstBaseline",
        children: [{
          type: "elem",
          elem: node
        }]
      }, options); // For spacing, TeX treats \hphantom as a math group (same spacing as ord).

      return buildCommon.makeSpan(["mord"], [smashedNode], options);
    },
    mathmlBuilder: (group, options) => {
      var node = new mathMLTree.MathNode("mpadded", [buildGroup(group.body, options)]);

      if (group.smashHeight) {
        node.setAttribute("height", "0px");
      }

      if (group.smashDepth) {
        node.setAttribute("depth", "0px");
      }

      return node;
    }
  });

  defineFunction({
    type: "sqrt",
    names: ["\\sqrt"],
    props: {
      numArgs: 1,
      numOptionalArgs: 1
    },

    handler(_ref, args, optArgs) {
      var {
        parser
      } = _ref;
      var index = optArgs[0];
      var body = args[0];
      return {
        type: "sqrt",
        mode: parser.mode,
        body,
        index
      };
    },

    htmlBuilder(group, options) {
      // Square roots are handled in the TeXbook pg. 443, Rule 11.
      // First, we do the same steps as in overline to build the inner group
      // and line
      var inner = buildGroup$1(group.body, options.havingCrampedStyle());

      if (inner.height === 0) {
        // Render a small surd.
        inner.height = options.fontMetrics().xHeight;
      } // Some groups can return document fragments.  Handle those by wrapping
      // them in a span.


      inner = buildCommon.wrapFragment(inner, options); // Calculate the minimum size for the \surd delimiter

      var metrics = options.fontMetrics();
      var theta = metrics.defaultRuleThickness;
      var phi = theta;

      if (options.style.id < Style$1.TEXT.id) {
        phi = options.fontMetrics().xHeight;
      } // Calculate the clearance between the body and line


      var lineClearance = theta + phi / 4;
      var minDelimiterHeight = inner.height + inner.depth + lineClearance + theta; // Create a sqrt SVG of the required minimum size

      var {
        span: img,
        ruleWidth,
        advanceWidth
      } = delimiter.sqrtImage(minDelimiterHeight, options);
      var delimDepth = img.height - ruleWidth; // Adjust the clearance based on the delimiter size

      if (delimDepth > inner.height + inner.depth + lineClearance) {
        lineClearance = (lineClearance + delimDepth - inner.height - inner.depth) / 2;
      } // Shift the sqrt image


      var imgShift = img.height - inner.height - lineClearance - ruleWidth;
      inner.style.paddingLeft = makeEm(advanceWidth); // Overlay the image and the argument.

      var body = buildCommon.makeVList({
        positionType: "firstBaseline",
        children: [{
          type: "elem",
          elem: inner,
          wrapperClasses: ["svg-align"]
        }, {
          type: "kern",
          size: -(inner.height + imgShift)
        }, {
          type: "elem",
          elem: img
        }, {
          type: "kern",
          size: ruleWidth
        }]
      }, options);

      if (!group.index) {
        return buildCommon.makeSpan(["mord", "sqrt"], [body], options);
      } else {
        // Handle the optional root index
        // The index is always in scriptscript style
        var newOptions = options.havingStyle(Style$1.SCRIPTSCRIPT);
        var rootm = buildGroup$1(group.index, newOptions, options); // The amount the index is shifted by. This is taken from the TeX
        // source, in the definition of `\r@@t`.

        var toShift = 0.6 * (body.height - body.depth); // Build a VList with the superscript shifted up correctly

        var rootVList = buildCommon.makeVList({
          positionType: "shift",
          positionData: -toShift,
          children: [{
            type: "elem",
            elem: rootm
          }]
        }, options); // Add a class surrounding it so we can add on the appropriate
        // kerning

        var rootVListWrap = buildCommon.makeSpan(["root"], [rootVList]);
        return buildCommon.makeSpan(["mord", "sqrt"], [rootVListWrap, body], options);
      }
    },

    mathmlBuilder(group, options) {
      var {
        body,
        index
      } = group;
      return index ? new mathMLTree.MathNode("mroot", [buildGroup(body, options), buildGroup(index, options)]) : new mathMLTree.MathNode("msqrt", [buildGroup(body, options)]);
    }

  });

  var styleMap = {
    "display": Style$1.DISPLAY,
    "text": Style$1.TEXT,
    "script": Style$1.SCRIPT,
    "scriptscript": Style$1.SCRIPTSCRIPT
  };
  defineFunction({
    type: "styling",
    names: ["\\displaystyle", "\\textstyle", "\\scriptstyle", "\\scriptscriptstyle"],
    props: {
      numArgs: 0,
      allowedInText: true,
      primitive: true
    },

    handler(_ref, args) {
      var {
        breakOnTokenText,
        funcName,
        parser
      } = _ref;
      // parse out the implicit body
      var body = parser.parseExpression(true, breakOnTokenText); // TODO: Refactor to avoid duplicating styleMap in multiple places (e.g.
      // here and in buildHTML and de-dupe the enumeration of all the styles).
      // $FlowFixMe: The names above exactly match the styles.

      var style = funcName.slice(1, funcName.length - 5);
      return {
        type: "styling",
        mode: parser.mode,
        // Figure out what style to use by pulling out the style from
        // the function name
        style,
        body
      };
    },

    htmlBuilder(group, options) {
      // Style changes are handled in the TeXbook on pg. 442, Rule 3.
      var newStyle = styleMap[group.style];
      var newOptions = options.havingStyle(newStyle).withFont('');
      return sizingGroup(group.body, newOptions, options);
    },

    mathmlBuilder(group, options) {
      // Figure out what style we're changing to.
      var newStyle = styleMap[group.style];
      var newOptions = options.havingStyle(newStyle);
      var inner = buildExpression(group.body, newOptions);
      var node = new mathMLTree.MathNode("mstyle", inner);
      var styleAttributes = {
        "display": ["0", "true"],
        "text": ["0", "false"],
        "script": ["1", "false"],
        "scriptscript": ["2", "false"]
      };
      var attr = styleAttributes[group.style];
      node.setAttribute("scriptlevel", attr[0]);
      node.setAttribute("displaystyle", attr[1]);
      return node;
    }

  });

  /**
   * Sometimes, groups perform special rules when they have superscripts or
   * subscripts attached to them. This function lets the `supsub` group know that
   * Sometimes, groups perform special rules when they have superscripts or
   * its inner element should handle the superscripts and subscripts instead of
   * handling them itself.
   */
  var htmlBuilderDelegate = function htmlBuilderDelegate(group, options) {
    var base = group.base;

    if (!base) {
      return null;
    } else if (base.type === "op") {
      // Operators handle supsubs differently when they have limits
      // (e.g. `\displaystyle\sum_2^3`)
      var delegate = base.limits && (options.style.size === Style$1.DISPLAY.size || base.alwaysHandleSupSub);
      return delegate ? htmlBuilder$2 : null;
    } else if (base.type === "operatorname") {
      var _delegate = base.alwaysHandleSupSub && (options.style.size === Style$1.DISPLAY.size || base.limits);

      return _delegate ? htmlBuilder$1 : null;
    } else if (base.type === "accent") {
      return utils$1.isCharacterBox(base.base) ? htmlBuilder$a : null;
    } else if (base.type === "horizBrace") {
      var isSup = !group.sub;
      return isSup === base.isOver ? htmlBuilder$3 : null;
    } else {
      return null;
    }
  }; // Super scripts and subscripts, whose precise placement can depend on other
  // functions that precede them.


  defineFunctionBuilders({
    type: "supsub",

    htmlBuilder(group, options) {
      // Superscript and subscripts are handled in the TeXbook on page
      // 445-446, rules 18(a-f).
      // Here is where we defer to the inner group if it should handle
      // superscripts and subscripts itself.
      var builderDelegate = htmlBuilderDelegate(group, options);

      if (builderDelegate) {
        return builderDelegate(group, options);
      }

      var {
        base: valueBase,
        sup: valueSup,
        sub: valueSub
      } = group;
      var base = buildGroup$1(valueBase, options);
      var supm;
      var subm;
      var metrics = options.fontMetrics(); // Rule 18a

      var supShift = 0;
      var subShift = 0;
      var isCharacterBox = valueBase && utils$1.isCharacterBox(valueBase);

      if (valueSup) {
        var newOptions = options.havingStyle(options.style.sup());
        supm = buildGroup$1(valueSup, newOptions, options);

        if (!isCharacterBox) {
          supShift = base.height - newOptions.fontMetrics().supDrop * newOptions.sizeMultiplier / options.sizeMultiplier;
        }
      }

      if (valueSub) {
        var _newOptions = options.havingStyle(options.style.sub());

        subm = buildGroup$1(valueSub, _newOptions, options);

        if (!isCharacterBox) {
          subShift = base.depth + _newOptions.fontMetrics().subDrop * _newOptions.sizeMultiplier / options.sizeMultiplier;
        }
      } // Rule 18c


      var minSupShift;

      if (options.style === Style$1.DISPLAY) {
        minSupShift = metrics.sup1;
      } else if (options.style.cramped) {
        minSupShift = metrics.sup3;
      } else {
        minSupShift = metrics.sup2;
      } // scriptspace is a font-size-independent size, so scale it
      // appropriately for use as the marginRight.


      var multiplier = options.sizeMultiplier;
      var marginRight = makeEm(0.5 / metrics.ptPerEm / multiplier);
      var marginLeft = null;

      if (subm) {
        // Subscripts shouldn't be shifted by the base's italic correction.
        // Account for that by shifting the subscript back the appropriate
        // amount. Note we only do this when the base is a single symbol.
        var isOiint = group.base && group.base.type === "op" && group.base.name && (group.base.name === "\\oiint" || group.base.name === "\\oiiint");

        if (base instanceof SymbolNode || isOiint) {
          // $FlowFixMe
          marginLeft = makeEm(-base.italic);
        }
      }

      var supsub;

      if (supm && subm) {
        supShift = Math.max(supShift, minSupShift, supm.depth + 0.25 * metrics.xHeight);
        subShift = Math.max(subShift, metrics.sub2);
        var ruleWidth = metrics.defaultRuleThickness; // Rule 18e

        var maxWidth = 4 * ruleWidth;

        if (supShift - supm.depth - (subm.height - subShift) < maxWidth) {
          subShift = maxWidth - (supShift - supm.depth) + subm.height;
          var psi = 0.8 * metrics.xHeight - (supShift - supm.depth);

          if (psi > 0) {
            supShift += psi;
            subShift -= psi;
          }
        }

        var vlistElem = [{
          type: "elem",
          elem: subm,
          shift: subShift,
          marginRight,
          marginLeft
        }, {
          type: "elem",
          elem: supm,
          shift: -supShift,
          marginRight
        }];
        supsub = buildCommon.makeVList({
          positionType: "individualShift",
          children: vlistElem
        }, options);
      } else if (subm) {
        // Rule 18b
        subShift = Math.max(subShift, metrics.sub1, subm.height - 0.8 * metrics.xHeight);
        var _vlistElem = [{
          type: "elem",
          elem: subm,
          marginLeft,
          marginRight
        }];
        supsub = buildCommon.makeVList({
          positionType: "shift",
          positionData: subShift,
          children: _vlistElem
        }, options);
      } else if (supm) {
        // Rule 18c, d
        supShift = Math.max(supShift, minSupShift, supm.depth + 0.25 * metrics.xHeight);
        supsub = buildCommon.makeVList({
          positionType: "shift",
          positionData: -supShift,
          children: [{
            type: "elem",
            elem: supm,
            marginRight
          }]
        }, options);
      } else {
        throw new Error("supsub must have either sup or sub.");
      } // Wrap the supsub vlist in a span.msupsub to reset text-align.


      var mclass = getTypeOfDomTree(base, "right") || "mord";
      return buildCommon.makeSpan([mclass], [base, buildCommon.makeSpan(["msupsub"], [supsub])], options);
    },

    mathmlBuilder(group, options) {
      // Is the inner group a relevant horizonal brace?
      var isBrace = false;
      var isOver;
      var isSup;

      if (group.base && group.base.type === "horizBrace") {
        isSup = !!group.sup;

        if (isSup === group.base.isOver) {
          isBrace = true;
          isOver = group.base.isOver;
        }
      }

      if (group.base && (group.base.type === "op" || group.base.type === "operatorname")) {
        group.base.parentIsSupSub = true;
      }

      var children = [buildGroup(group.base, options)];

      if (group.sub) {
        children.push(buildGroup(group.sub, options));
      }

      if (group.sup) {
        children.push(buildGroup(group.sup, options));
      }

      var nodeType;

      if (isBrace) {
        nodeType = isOver ? "mover" : "munder";
      } else if (!group.sub) {
        var base = group.base;

        if (base && base.type === "op" && base.limits && (options.style === Style$1.DISPLAY || base.alwaysHandleSupSub)) {
          nodeType = "mover";
        } else if (base && base.type === "operatorname" && base.alwaysHandleSupSub && (base.limits || options.style === Style$1.DISPLAY)) {
          nodeType = "mover";
        } else {
          nodeType = "msup";
        }
      } else if (!group.sup) {
        var _base = group.base;

        if (_base && _base.type === "op" && _base.limits && (options.style === Style$1.DISPLAY || _base.alwaysHandleSupSub)) {
          nodeType = "munder";
        } else if (_base && _base.type === "operatorname" && _base.alwaysHandleSupSub && (_base.limits || options.style === Style$1.DISPLAY)) {
          nodeType = "munder";
        } else {
          nodeType = "msub";
        }
      } else {
        var _base2 = group.base;

        if (_base2 && _base2.type === "op" && _base2.limits && options.style === Style$1.DISPLAY) {
          nodeType = "munderover";
        } else if (_base2 && _base2.type === "operatorname" && _base2.alwaysHandleSupSub && (options.style === Style$1.DISPLAY || _base2.limits)) {
          nodeType = "munderover";
        } else {
          nodeType = "msubsup";
        }
      }

      return new mathMLTree.MathNode(nodeType, children);
    }

  });

  defineFunctionBuilders({
    type: "atom",

    htmlBuilder(group, options) {
      return buildCommon.mathsym(group.text, group.mode, options, ["m" + group.family]);
    },

    mathmlBuilder(group, options) {
      var node = new mathMLTree.MathNode("mo", [makeText(group.text, group.mode)]);

      if (group.family === "bin") {
        var variant = getVariant(group, options);

        if (variant === "bold-italic") {
          node.setAttribute("mathvariant", variant);
        }
      } else if (group.family === "punct") {
        node.setAttribute("separator", "true");
      } else if (group.family === "open" || group.family === "close") {
        // Delims built here should not stretch vertically.
        // See delimsizing.js for stretchy delims.
        node.setAttribute("stretchy", "false");
      }

      return node;
    }

  });

  // "mathord" and "textord" ParseNodes created in Parser.js from symbol Groups in
  // src/symbols.js.
  var defaultVariant = {
    "mi": "italic",
    "mn": "normal",
    "mtext": "normal"
  };
  defineFunctionBuilders({
    type: "mathord",

    htmlBuilder(group, options) {
      return buildCommon.makeOrd(group, options, "mathord");
    },

    mathmlBuilder(group, options) {
      var node = new mathMLTree.MathNode("mi", [makeText(group.text, group.mode, options)]);
      var variant = getVariant(group, options) || "italic";

      if (variant !== defaultVariant[node.type]) {
        node.setAttribute("mathvariant", variant);
      }

      return node;
    }

  });
  defineFunctionBuilders({
    type: "textord",

    htmlBuilder(group, options) {
      return buildCommon.makeOrd(group, options, "textord");
    },

    mathmlBuilder(group, options) {
      var text = makeText(group.text, group.mode, options);
      var variant = getVariant(group, options) || "normal";
      var node;

      if (group.mode === 'text') {
        node = new mathMLTree.MathNode("mtext", [text]);
      } else if (/[0-9]/.test(group.text)) {
        node = new mathMLTree.MathNode("mn", [text]);
      } else if (group.text === "\\prime") {
        node = new mathMLTree.MathNode("mo", [text]);
      } else {
        node = new mathMLTree.MathNode("mi", [text]);
      }

      if (variant !== defaultVariant[node.type]) {
        node.setAttribute("mathvariant", variant);
      }

      return node;
    }

  });

  var cssSpace = {
    "\\nobreak": "nobreak",
    "\\allowbreak": "allowbreak"
  }; // A lookup table to determine whether a spacing function/symbol should be
  // treated like a regular space character.  If a symbol or command is a key
  // in this table, then it should be a regular space character.  Furthermore,
  // the associated value may have a `className` specifying an extra CSS class
  // to add to the created `span`.

  var regularSpace = {
    " ": {},
    "\\ ": {},
    "~": {
      className: "nobreak"
    },
    "\\space": {},
    "\\nobreakspace": {
      className: "nobreak"
    }
  }; // ParseNode<"spacing"> created in Parser.js from the "spacing" symbol Groups in
  // src/symbols.js.

  defineFunctionBuilders({
    type: "spacing",

    htmlBuilder(group, options) {
      if (regularSpace.hasOwnProperty(group.text)) {
        var className = regularSpace[group.text].className || ""; // Spaces are generated by adding an actual space. Each of these
        // things has an entry in the symbols table, so these will be turned
        // into appropriate outputs.

        if (group.mode === "text") {
          var ord = buildCommon.makeOrd(group, options, "textord");
          ord.classes.push(className);
          return ord;
        } else {
          return buildCommon.makeSpan(["mspace", className], [buildCommon.mathsym(group.text, group.mode, options)], options);
        }
      } else if (cssSpace.hasOwnProperty(group.text)) {
        // Spaces based on just a CSS class.
        return buildCommon.makeSpan(["mspace", cssSpace[group.text]], [], options);
      } else {
        throw new ParseError("Unknown type of space \"" + group.text + "\"");
      }
    },

    mathmlBuilder(group, options) {
      var node;

      if (regularSpace.hasOwnProperty(group.text)) {
        node = new mathMLTree.MathNode("mtext", [new mathMLTree.TextNode("\u00a0")]);
      } else if (cssSpace.hasOwnProperty(group.text)) {
        // CSS-based MathML spaces (\nobreak, \allowbreak) are ignored
        return new mathMLTree.MathNode("mspace");
      } else {
        throw new ParseError("Unknown type of space \"" + group.text + "\"");
      }

      return node;
    }

  });

  var pad = () => {
    var padNode = new mathMLTree.MathNode("mtd", []);
    padNode.setAttribute("width", "50%");
    return padNode;
  };

  defineFunctionBuilders({
    type: "tag",

    mathmlBuilder(group, options) {
      var table = new mathMLTree.MathNode("mtable", [new mathMLTree.MathNode("mtr", [pad(), new mathMLTree.MathNode("mtd", [buildExpressionRow(group.body, options)]), pad(), new mathMLTree.MathNode("mtd", [buildExpressionRow(group.tag, options)])])]);
      table.setAttribute("width", "100%");
      return table; // TODO: Left-aligned tags.
      // Currently, the group and options passed here do not contain
      // enough info to set tag alignment. `leqno` is in Settings but it is
      // not passed to Options. On the HTML side, leqno is
      // set by a CSS class applied in buildTree.js. That would have worked
      // in MathML if browsers supported <mlabeledtr>. Since they don't, we
      // need to rewrite the way this function is called.
    }

  });

  var textFontFamilies = {
    "\\text": undefined,
    "\\textrm": "textrm",
    "\\textsf": "textsf",
    "\\texttt": "texttt",
    "\\textnormal": "textrm"
  };
  var textFontWeights = {
    "\\textbf": "textbf",
    "\\textmd": "textmd"
  };
  var textFontShapes = {
    "\\textit": "textit",
    "\\textup": "textup"
  };

  var optionsWithFont = (group, options) => {
    var font = group.font; // Checks if the argument is a font family or a font style.

    if (!font) {
      return options;
    } else if (textFontFamilies[font]) {
      return options.withTextFontFamily(textFontFamilies[font]);
    } else if (textFontWeights[font]) {
      return options.withTextFontWeight(textFontWeights[font]);
    } else {
      return options.withTextFontShape(textFontShapes[font]);
    }
  };

  defineFunction({
    type: "text",
    names: [// Font families
    "\\text", "\\textrm", "\\textsf", "\\texttt", "\\textnormal", // Font weights
    "\\textbf", "\\textmd", // Font Shapes
    "\\textit", "\\textup"],
    props: {
      numArgs: 1,
      argTypes: ["text"],
      allowedInArgument: true,
      allowedInText: true
    },

    handler(_ref, args) {
      var {
        parser,
        funcName
      } = _ref;
      var body = args[0];
      return {
        type: "text",
        mode: parser.mode,
        body: ordargument(body),
        font: funcName
      };
    },

    htmlBuilder(group, options) {
      var newOptions = optionsWithFont(group, options);
      var inner = buildExpression$1(group.body, newOptions, true);
      return buildCommon.makeSpan(["mord", "text"], inner, newOptions);
    },

    mathmlBuilder(group, options) {
      var newOptions = optionsWithFont(group, options);
      return buildExpressionRow(group.body, newOptions);
    }

  });

  defineFunction({
    type: "underline",
    names: ["\\underline"],
    props: {
      numArgs: 1,
      allowedInText: true
    },

    handler(_ref, args) {
      var {
        parser
      } = _ref;
      return {
        type: "underline",
        mode: parser.mode,
        body: args[0]
      };
    },

    htmlBuilder(group, options) {
      // Underlines are handled in the TeXbook pg 443, Rule 10.
      // Build the inner group.
      var innerGroup = buildGroup$1(group.body, options); // Create the line to go below the body

      var line = buildCommon.makeLineSpan("underline-line", options); // Generate the vlist, with the appropriate kerns

      var defaultRuleThickness = options.fontMetrics().defaultRuleThickness;
      var vlist = buildCommon.makeVList({
        positionType: "top",
        positionData: innerGroup.height,
        children: [{
          type: "kern",
          size: defaultRuleThickness
        }, {
          type: "elem",
          elem: line
        }, {
          type: "kern",
          size: 3 * defaultRuleThickness
        }, {
          type: "elem",
          elem: innerGroup
        }]
      }, options);
      return buildCommon.makeSpan(["mord", "underline"], [vlist], options);
    },

    mathmlBuilder(group, options) {
      var operator = new mathMLTree.MathNode("mo", [new mathMLTree.TextNode("\u203e")]);
      operator.setAttribute("stretchy", "true");
      var node = new mathMLTree.MathNode("munder", [buildGroup(group.body, options), operator]);
      node.setAttribute("accentunder", "true");
      return node;
    }

  });

  defineFunction({
    type: "vcenter",
    names: ["\\vcenter"],
    props: {
      numArgs: 1,
      argTypes: ["original"],
      // In LaTeX, \vcenter can act only on a box.
      allowedInText: false
    },

    handler(_ref, args) {
      var {
        parser
      } = _ref;
      return {
        type: "vcenter",
        mode: parser.mode,
        body: args[0]
      };
    },

    htmlBuilder(group, options) {
      var body = buildGroup$1(group.body, options);
      var axisHeight = options.fontMetrics().axisHeight;
      var dy = 0.5 * (body.height - axisHeight - (body.depth + axisHeight));
      return buildCommon.makeVList({
        positionType: "shift",
        positionData: dy,
        children: [{
          type: "elem",
          elem: body
        }]
      }, options);
    },

    mathmlBuilder(group, options) {
      // There is no way to do this in MathML.
      // Write a class as a breadcrumb in case some post-processor wants
      // to perform a vcenter adjustment.
      return new mathMLTree.MathNode("mpadded", [buildGroup(group.body, options)], ["vcenter"]);
    }

  });

  defineFunction({
    type: "verb",
    names: ["\\verb"],
    props: {
      numArgs: 0,
      allowedInText: true
    },

    handler(context, args, optArgs) {
      // \verb and \verb* are dealt with directly in Parser.js.
      // If we end up here, it's because of a failure to match the two delimiters
      // in the regex in Lexer.js.  LaTeX raises the following error when \verb is
      // terminated by end of line (or file).
      throw new ParseError("\\verb ended by end of line instead of matching delimiter");
    },

    htmlBuilder(group, options) {
      var text = makeVerb(group);
      var body = []; // \verb enters text mode and therefore is sized like \textstyle

      var newOptions = options.havingStyle(options.style.text());

      for (var i = 0; i < text.length; i++) {
        var c = text[i];

        if (c === '~') {
          c = '\\textasciitilde';
        }

        body.push(buildCommon.makeSymbol(c, "Typewriter-Regular", group.mode, newOptions, ["mord", "texttt"]));
      }

      return buildCommon.makeSpan(["mord", "text"].concat(newOptions.sizingClasses(options)), buildCommon.tryCombineChars(body), newOptions);
    },

    mathmlBuilder(group, options) {
      var text = new mathMLTree.TextNode(makeVerb(group));
      var node = new mathMLTree.MathNode("mtext", [text]);
      node.setAttribute("mathvariant", "monospace");
      return node;
    }

  });
  /**
   * Converts verb group into body string.
   *
   * \verb* replaces each space with an open box \u2423
   * \verb replaces each space with a no-break space \xA0
   */

  var makeVerb = group => group.body.replace(/ /g, group.star ? '\u2423' : '\xA0');

  /** Include this to ensure that all functions are defined. */
  var functions = _functions;

  /**
   * The Lexer class handles tokenizing the input in various ways. Since our
   * parser expects us to be able to backtrack, the lexer allows lexing from any
   * given starting point.
   *
   * Its main exposed function is the `lex` function, which takes a position to
   * lex from and a type of token to lex. It defers to the appropriate `_innerLex`
   * function.
   *
   * The various `_innerLex` functions perform the actual lexing of different
   * kinds.
   */

  /* The following tokenRegex
   * - matches typical whitespace (but not NBSP etc.) using its first group
   * - does not match any control character \x00-\x1f except whitespace
   * - does not match a bare backslash
   * - matches any ASCII character except those just mentioned
   * - does not match the BMP private use area \uE000-\uF8FF
   * - does not match bare surrogate code units
   * - matches any BMP character except for those just described
   * - matches any valid Unicode surrogate pair
   * - matches a backslash followed by one or more whitespace characters
   * - matches a backslash followed by one or more letters then whitespace
   * - matches a backslash followed by any BMP character
   * Capturing groups:
   *   [1] regular whitespace
   *   [2] backslash followed by whitespace
   *   [3] anything else, which may include:
   *     [4] left character of \verb*
   *     [5] left character of \verb
   *     [6] backslash followed by word, excluding any trailing whitespace
   * Just because the Lexer matches something doesn't mean it's valid input:
   * If there is no matching function or symbol definition, the Parser will
   * still reject the input.
   */
  var spaceRegexString = "[ \r\n\t]";
  var controlWordRegexString = "\\\\[a-zA-Z@]+";
  var controlSymbolRegexString = "\\\\[^\uD800-\uDFFF]";
  var controlWordWhitespaceRegexString = "(" + controlWordRegexString + ")" + spaceRegexString + "*";
  var controlSpaceRegexString = "\\\\(\n|[ \r\t]+\n?)[ \r\t]*";
  var combiningDiacriticalMarkString = "[\u0300-\u036f]";
  var combiningDiacriticalMarksEndRegex = new RegExp(combiningDiacriticalMarkString + "+$");
  var tokenRegexString = "(" + spaceRegexString + "+)|" + ( // whitespace
  controlSpaceRegexString + "|") + // \whitespace
  "([!-\\[\\]-\u2027\u202A-\uD7FF\uF900-\uFFFF]" + ( // single codepoint
  combiningDiacriticalMarkString + "*") + // ...plus accents
  "|[\uD800-\uDBFF][\uDC00-\uDFFF]" + ( // surrogate pair
  combiningDiacriticalMarkString + "*") + // ...plus accents
  "|\\\\verb\\*([^]).*?\\4" + // \verb*
  "|\\\\verb([^*a-zA-Z]).*?\\5" + ( // \verb unstarred
  "|" + controlWordWhitespaceRegexString) + ( // \macroName + spaces
  "|" + controlSymbolRegexString + ")"); // \\, \', etc.

  /** Main Lexer class */

  class Lexer {
    // Category codes. The lexer only supports comment characters (14) for now.
    // MacroExpander additionally distinguishes active (13).
    constructor(input, settings) {
      this.input = void 0;
      this.settings = void 0;
      this.tokenRegex = void 0;
      this.catcodes = void 0;
      // Separate accents from characters
      this.input = input;
      this.settings = settings;
      this.tokenRegex = new RegExp(tokenRegexString, 'g');
      this.catcodes = {
        "%": 14,
        // comment character
        "~": 13 // active character

      };
    }

    setCatcode(char, code) {
      this.catcodes[char] = code;
    }
    /**
     * This function lexes a single token.
     */


    lex() {
      var input = this.input;
      var pos = this.tokenRegex.lastIndex;

      if (pos === input.length) {
        return new Token("EOF", new SourceLocation(this, pos, pos));
      }

      var match = this.tokenRegex.exec(input);

      if (match === null || match.index !== pos) {
        throw new ParseError("Unexpected character: '" + input[pos] + "'", new Token(input[pos], new SourceLocation(this, pos, pos + 1)));
      }

      var text = match[6] || match[3] || (match[2] ? "\\ " : " ");

      if (this.catcodes[text] === 14) {
        // comment character
        var nlIndex = input.indexOf('\n', this.tokenRegex.lastIndex);

        if (nlIndex === -1) {
          this.tokenRegex.lastIndex = input.length; // EOF

          this.settings.reportNonstrict("commentAtEnd", "% comment has no terminating newline; LaTeX would " + "fail because of commenting the end of math mode (e.g. $)");
        } else {
          this.tokenRegex.lastIndex = nlIndex + 1;
        }

        return this.lex();
      }

      return new Token(text, new SourceLocation(this, pos, this.tokenRegex.lastIndex));
    }

  }

  /**
   * A `Namespace` refers to a space of nameable things like macros or lengths,
   * which can be `set` either globally or local to a nested group, using an
   * undo stack similar to how TeX implements this functionality.
   * Performance-wise, `get` and local `set` take constant time, while global
   * `set` takes time proportional to the depth of group nesting.
   */
  class Namespace {
    /**
     * Both arguments are optional.  The first argument is an object of
     * built-in mappings which never change.  The second argument is an object
     * of initial (global-level) mappings, which will constantly change
     * according to any global/top-level `set`s done.
     */
    constructor(builtins, globalMacros) {
      if (builtins === void 0) {
        builtins = {};
      }

      if (globalMacros === void 0) {
        globalMacros = {};
      }

      this.current = void 0;
      this.builtins = void 0;
      this.undefStack = void 0;
      this.current = globalMacros;
      this.builtins = builtins;
      this.undefStack = [];
    }
    /**
     * Start a new nested group, affecting future local `set`s.
     */


    beginGroup() {
      this.undefStack.push({});
    }
    /**
     * End current nested group, restoring values before the group began.
     */


    endGroup() {
      if (this.undefStack.length === 0) {
        throw new ParseError("Unbalanced namespace destruction: attempt " + "to pop global namespace; please report this as a bug");
      }

      var undefs = this.undefStack.pop();

      for (var undef in undefs) {
        if (undefs.hasOwnProperty(undef)) {
          if (undefs[undef] == null) {
            delete this.current[undef];
          } else {
            this.current[undef] = undefs[undef];
          }
        }
      }
    }
    /**
     * Ends all currently nested groups (if any), restoring values before the
     * groups began.  Useful in case of an error in the middle of parsing.
     */


    endGroups() {
      while (this.undefStack.length > 0) {
        this.endGroup();
      }
    }
    /**
     * Detect whether `name` has a definition.  Equivalent to
     * `get(name) != null`.
     */


    has(name) {
      return this.current.hasOwnProperty(name) || this.builtins.hasOwnProperty(name);
    }
    /**
     * Get the current value of a name, or `undefined` if there is no value.
     *
     * Note: Do not use `if (namespace.get(...))` to detect whether a macro
     * is defined, as the definition may be the empty string which evaluates
     * to `false` in JavaScript.  Use `if (namespace.get(...) != null)` or
     * `if (namespace.has(...))`.
     */


    get(name) {
      if (this.current.hasOwnProperty(name)) {
        return this.current[name];
      } else {
        return this.builtins[name];
      }
    }
    /**
     * Set the current value of a name, and optionally set it globally too.
     * Local set() sets the current value and (when appropriate) adds an undo
     * operation to the undo stack.  Global set() may change the undo
     * operation at every level, so takes time linear in their number.
     * A value of undefined means to delete existing definitions.
     */


    set(name, value, global) {
      if (global === void 0) {
        global = false;
      }

      if (global) {
        // Global set is equivalent to setting in all groups.  Simulate this
        // by destroying any undos currently scheduled for this name,
        // and adding an undo with the *new* value (in case it later gets
        // locally reset within this environment).
        for (var i = 0; i < this.undefStack.length; i++) {
          delete this.undefStack[i][name];
        }

        if (this.undefStack.length > 0) {
          this.undefStack[this.undefStack.length - 1][name] = value;
        }
      } else {
        // Undo this set at end of this group (possibly to `undefined`),
        // unless an undo is already in place, in which case that older
        // value is the correct one.
        var top = this.undefStack[this.undefStack.length - 1];

        if (top && !top.hasOwnProperty(name)) {
          top[name] = this.current[name];
        }
      }

      if (value == null) {
        delete this.current[name];
      } else {
        this.current[name] = value;
      }
    }

  }

  /**
   * Predefined macros for KaTeX.
   * This can be used to define some commands in terms of others.
   */
  var macros = _macros;
  // macro tools

  defineMacro("\\noexpand", function (context) {
    // The expansion is the token itself; but that token is interpreted
    // as if its meaning were ‘\relax’ if it is a control sequence that
    // would ordinarily be expanded by TeX’s expansion rules.
    var t = context.popToken();

    if (context.isExpandable(t.text)) {
      t.noexpand = true;
      t.treatAsRelax = true;
    }

    return {
      tokens: [t],
      numArgs: 0
    };
  });
  defineMacro("\\expandafter", function (context) {
    // TeX first reads the token that comes immediately after \expandafter,
    // without expanding it; let’s call this token t. Then TeX reads the
    // token that comes after t (and possibly more tokens, if that token
    // has an argument), replacing it by its expansion. Finally TeX puts
    // t back in front of that expansion.
    var t = context.popToken();
    context.expandOnce(true); // expand only an expandable token

    return {
      tokens: [t],
      numArgs: 0
    };
  }); // LaTeX's \@firstoftwo{#1}{#2} expands to #1, skipping #2
  // TeX source: \long\def\@firstoftwo#1#2{#1}

  defineMacro("\\@firstoftwo", function (context) {
    var args = context.consumeArgs(2);
    return {
      tokens: args[0],
      numArgs: 0
    };
  }); // LaTeX's \@secondoftwo{#1}{#2} expands to #2, skipping #1
  // TeX source: \long\def\@secondoftwo#1#2{#2}

  defineMacro("\\@secondoftwo", function (context) {
    var args = context.consumeArgs(2);
    return {
      tokens: args[1],
      numArgs: 0
    };
  }); // LaTeX's \@ifnextchar{#1}{#2}{#3} looks ahead to the next (unexpanded)
  // symbol that isn't a space, consuming any spaces but not consuming the
  // first nonspace character.  If that nonspace character matches #1, then
  // the macro expands to #2; otherwise, it expands to #3.

  defineMacro("\\@ifnextchar", function (context) {
    var args = context.consumeArgs(3); // symbol, if, else

    context.consumeSpaces();
    var nextToken = context.future();

    if (args[0].length === 1 && args[0][0].text === nextToken.text) {
      return {
        tokens: args[1],
        numArgs: 0
      };
    } else {
      return {
        tokens: args[2],
        numArgs: 0
      };
    }
  }); // LaTeX's \@ifstar{#1}{#2} looks ahead to the next (unexpanded) symbol.
  // If it is `*`, then it consumes the symbol, and the macro expands to #1;
  // otherwise, the macro expands to #2 (without consuming the symbol).
  // TeX source: \def\@ifstar#1{\@ifnextchar *{\@firstoftwo{#1}}}

  defineMacro("\\@ifstar", "\\@ifnextchar *{\\@firstoftwo{#1}}"); // LaTeX's \TextOrMath{#1}{#2} expands to #1 in text mode, #2 in math mode

  defineMacro("\\TextOrMath", function (context) {
    var args = context.consumeArgs(2);

    if (context.mode === 'text') {
      return {
        tokens: args[0],
        numArgs: 0
      };
    } else {
      return {
        tokens: args[1],
        numArgs: 0
      };
    }
  }); // Lookup table for parsing numbers in base 8 through 16

  var digitToNumber = {
    "0": 0,
    "1": 1,
    "2": 2,
    "3": 3,
    "4": 4,
    "5": 5,
    "6": 6,
    "7": 7,
    "8": 8,
    "9": 9,
    "a": 10,
    "A": 10,
    "b": 11,
    "B": 11,
    "c": 12,
    "C": 12,
    "d": 13,
    "D": 13,
    "e": 14,
    "E": 14,
    "f": 15,
    "F": 15
  }; // TeX \char makes a literal character (catcode 12) using the following forms:
  // (see The TeXBook, p. 43)
  //   \char123  -- decimal
  //   \char'123 -- octal
  //   \char"123 -- hex
  //   \char`x   -- character that can be written (i.e. isn't active)
  //   \char`\x  -- character that cannot be written (e.g. %)
  // These all refer to characters from the font, so we turn them into special
  // calls to a function \@char dealt with in the Parser.

  defineMacro("\\char", function (context) {
    var token = context.popToken();
    var base;
    var number = '';

    if (token.text === "'") {
      base = 8;
      token = context.popToken();
    } else if (token.text === '"') {
      base = 16;
      token = context.popToken();
    } else if (token.text === "`") {
      token = context.popToken();

      if (token.text[0] === "\\") {
        number = token.text.charCodeAt(1);
      } else if (token.text === "EOF") {
        throw new ParseError("\\char` missing argument");
      } else {
        number = token.text.charCodeAt(0);
      }
    } else {
      base = 10;
    }

    if (base) {
      // Parse a number in the given base, starting with first `token`.
      number = digitToNumber[token.text];

      if (number == null || number >= base) {
        throw new ParseError("Invalid base-" + base + " digit " + token.text);
      }

      var digit;

      while ((digit = digitToNumber[context.future().text]) != null && digit < base) {
        number *= base;
        number += digit;
        context.popToken();
      }
    }

    return "\\@char{" + number + "}";
  }); // \newcommand{\macro}[args]{definition}
  // \renewcommand{\macro}[args]{definition}
  // TODO: Optional arguments: \newcommand{\macro}[args][default]{definition}

  var newcommand = (context, existsOK, nonexistsOK) => {
    var arg = context.consumeArg().tokens;

    if (arg.length !== 1) {
      throw new ParseError("\\newcommand's first argument must be a macro name");
    }

    var name = arg[0].text;
    var exists = context.isDefined(name);

    if (exists && !existsOK) {
      throw new ParseError("\\newcommand{" + name + "} attempting to redefine " + (name + "; use \\renewcommand"));
    }

    if (!exists && !nonexistsOK) {
      throw new ParseError("\\renewcommand{" + name + "} when command " + name + " " + "does not yet exist; use \\newcommand");
    }

    var numArgs = 0;
    arg = context.consumeArg().tokens;

    if (arg.length === 1 && arg[0].text === "[") {
      var argText = '';
      var token = context.expandNextToken();

      while (token.text !== "]" && token.text !== "EOF") {
        // TODO: Should properly expand arg, e.g., ignore {}s
        argText += token.text;
        token = context.expandNextToken();
      }

      if (!argText.match(/^\s*[0-9]+\s*$/)) {
        throw new ParseError("Invalid number of arguments: " + argText);
      }

      numArgs = parseInt(argText);
      arg = context.consumeArg().tokens;
    } // Final arg is the expansion of the macro


    context.macros.set(name, {
      tokens: arg,
      numArgs
    });
    return '';
  };

  defineMacro("\\newcommand", context => newcommand(context, false, true));
  defineMacro("\\renewcommand", context => newcommand(context, true, false));
  defineMacro("\\providecommand", context => newcommand(context, true, true)); // terminal (console) tools

  defineMacro("\\message", context => {
    var arg = context.consumeArgs(1)[0]; // eslint-disable-next-line no-console

    console.log(arg.reverse().map(token => token.text).join(""));
    return '';
  });
  defineMacro("\\errmessage", context => {
    var arg = context.consumeArgs(1)[0]; // eslint-disable-next-line no-console

    console.error(arg.reverse().map(token => token.text).join(""));
    return '';
  });
  defineMacro("\\show", context => {
    var tok = context.popToken();
    var name = tok.text; // eslint-disable-next-line no-console

    console.log(tok, context.macros.get(name), functions[name], symbols.math[name], symbols.text[name]);
    return '';
  }); //////////////////////////////////////////////////////////////////////
  // Grouping
  // \let\bgroup={ \let\egroup=}

  defineMacro("\\bgroup", "{");
  defineMacro("\\egroup", "}"); // Symbols from latex.ltx:
  // \def~{\nobreakspace{}}
  // \def\lq{`}
  // \def\rq{'}
  // \def \aa {\r a}
  // \def \AA {\r A}

  defineMacro("~", "\\nobreakspace");
  defineMacro("\\lq", "`");
  defineMacro("\\rq", "'");
  defineMacro("\\aa", "\\r a");
  defineMacro("\\AA", "\\r A"); // Copyright (C) and registered (R) symbols. Use raw symbol in MathML.
  // \DeclareTextCommandDefault{\textcopyright}{\textcircled{c}}
  // \DeclareTextCommandDefault{\textregistered}{\textcircled{%
  //      \check@mathfonts\fontsize\sf@size\z@\math@fontsfalse\selectfont R}}
  // \DeclareRobustCommand{\copyright}{%
  //    \ifmmode{\nfss@text{\textcopyright}}\else\textcopyright\fi}

  defineMacro("\\textcopyright", "\\html@mathml{\\textcircled{c}}{\\char`©}");
  defineMacro("\\copyright", "\\TextOrMath{\\textcopyright}{\\text{\\textcopyright}}");
  defineMacro("\\textregistered", "\\html@mathml{\\textcircled{\\scriptsize R}}{\\char`®}"); // Characters omitted from Unicode range 1D400–1D7FF

  defineMacro("\u212C", "\\mathscr{B}"); // script

  defineMacro("\u2130", "\\mathscr{E}");
  defineMacro("\u2131", "\\mathscr{F}");
  defineMacro("\u210B", "\\mathscr{H}");
  defineMacro("\u2110", "\\mathscr{I}");
  defineMacro("\u2112", "\\mathscr{L}");
  defineMacro("\u2133", "\\mathscr{M}");
  defineMacro("\u211B", "\\mathscr{R}");
  defineMacro("\u212D", "\\mathfrak{C}"); // Fraktur

  defineMacro("\u210C", "\\mathfrak{H}");
  defineMacro("\u2128", "\\mathfrak{Z}"); // Define \Bbbk with a macro that works in both HTML and MathML.

  defineMacro("\\Bbbk", "\\Bbb{k}"); // Unicode middle dot
  // The KaTeX fonts do not contain U+00B7. Instead, \cdotp displays
  // the dot at U+22C5 and gives it punct spacing.

  defineMacro("\u00b7", "\\cdotp"); // \llap and \rlap render their contents in text mode

  defineMacro("\\llap", "\\mathllap{\\textrm{#1}}");
  defineMacro("\\rlap", "\\mathrlap{\\textrm{#1}}");
  defineMacro("\\clap", "\\mathclap{\\textrm{#1}}"); // \mathstrut from the TeXbook, p 360

  defineMacro("\\mathstrut", "\\vphantom{(}"); // \underbar from TeXbook p 353

  defineMacro("\\underbar", "\\underline{\\text{#1}}"); // \not is defined by base/fontmath.ltx via
  // \DeclareMathSymbol{\not}{\mathrel}{symbols}{"36}
  // It's thus treated like a \mathrel, but defined by a symbol that has zero
  // width but extends to the right.  We use \rlap to get that spacing.
  // For MathML we write U+0338 here. buildMathML.js will then do the overlay.

  defineMacro("\\not", '\\html@mathml{\\mathrel{\\mathrlap\\@not}}{\\char"338}'); // Negated symbols from base/fontmath.ltx:
  // \def\neq{\not=} \let\ne=\neq
  // \DeclareRobustCommand
  //   \notin{\mathrel{\m@th\mathpalette\c@ncel\in}}
  // \def\c@ncel#1#2{\m@th\ooalign{$\hfil#1\mkern1mu/\hfil$\crcr$#1#2$}}

  defineMacro("\\neq", "\\html@mathml{\\mathrel{\\not=}}{\\mathrel{\\char`≠}}");
  defineMacro("\\ne", "\\neq");
  defineMacro("\u2260", "\\neq");
  defineMacro("\\notin", "\\html@mathml{\\mathrel{{\\in}\\mathllap{/\\mskip1mu}}}" + "{\\mathrel{\\char`∉}}");
  defineMacro("\u2209", "\\notin"); // Unicode stacked relations

  defineMacro("\u2258", "\\html@mathml{" + "\\mathrel{=\\kern{-1em}\\raisebox{0.4em}{$\\scriptsize\\frown$}}" + "}{\\mathrel{\\char`\u2258}}");
  defineMacro("\u2259", "\\html@mathml{\\stackrel{\\tiny\\wedge}{=}}{\\mathrel{\\char`\u2258}}");
  defineMacro("\u225A", "\\html@mathml{\\stackrel{\\tiny\\vee}{=}}{\\mathrel{\\char`\u225A}}");
  defineMacro("\u225B", "\\html@mathml{\\stackrel{\\scriptsize\\star}{=}}" + "{\\mathrel{\\char`\u225B}}");
  defineMacro("\u225D", "\\html@mathml{\\stackrel{\\tiny\\mathrm{def}}{=}}" + "{\\mathrel{\\char`\u225D}}");
  defineMacro("\u225E", "\\html@mathml{\\stackrel{\\tiny\\mathrm{m}}{=}}" + "{\\mathrel{\\char`\u225E}}");
  defineMacro("\u225F", "\\html@mathml{\\stackrel{\\tiny?}{=}}{\\mathrel{\\char`\u225F}}"); // Misc Unicode

  defineMacro("\u27C2", "\\perp");
  defineMacro("\u203C", "\\mathclose{!\\mkern-0.8mu!}");
  defineMacro("\u220C", "\\notni");
  defineMacro("\u231C", "\\ulcorner");
  defineMacro("\u231D", "\\urcorner");
  defineMacro("\u231E", "\\llcorner");
  defineMacro("\u231F", "\\lrcorner");
  defineMacro("\u00A9", "\\copyright");
  defineMacro("\u00AE", "\\textregistered");
  defineMacro("\uFE0F", "\\textregistered"); // The KaTeX fonts have corners at codepoints that don't match Unicode.
  // For MathML purposes, use the Unicode code point.

  defineMacro("\\ulcorner", "\\html@mathml{\\@ulcorner}{\\mathop{\\char\"231c}}");
  defineMacro("\\urcorner", "\\html@mathml{\\@urcorner}{\\mathop{\\char\"231d}}");
  defineMacro("\\llcorner", "\\html@mathml{\\@llcorner}{\\mathop{\\char\"231e}}");
  defineMacro("\\lrcorner", "\\html@mathml{\\@lrcorner}{\\mathop{\\char\"231f}}"); //////////////////////////////////////////////////////////////////////
  // LaTeX_2ε
  // \vdots{\vbox{\baselineskip4\p@  \lineskiplimit\z@
  // \kern6\p@\hbox{.}\hbox{.}\hbox{.}}}
  // We'll call \varvdots, which gets a glyph from symbols.js.
  // The zero-width rule gets us an equivalent to the vertical 6pt kern.

  defineMacro("\\vdots", "\\mathord{\\varvdots\\rule{0pt}{15pt}}");
  defineMacro("\u22ee", "\\vdots"); //////////////////////////////////////////////////////////////////////
  // amsmath.sty
  // http://mirrors.concertpass.com/tex-archive/macros/latex/required/amsmath/amsmath.pdf
  // Italic Greek capital letters.  AMS defines these with \DeclareMathSymbol,
  // but they are equivalent to \mathit{\Letter}.

  defineMacro("\\varGamma", "\\mathit{\\Gamma}");
  defineMacro("\\varDelta", "\\mathit{\\Delta}");
  defineMacro("\\varTheta", "\\mathit{\\Theta}");
  defineMacro("\\varLambda", "\\mathit{\\Lambda}");
  defineMacro("\\varXi", "\\mathit{\\Xi}");
  defineMacro("\\varPi", "\\mathit{\\Pi}");
  defineMacro("\\varSigma", "\\mathit{\\Sigma}");
  defineMacro("\\varUpsilon", "\\mathit{\\Upsilon}");
  defineMacro("\\varPhi", "\\mathit{\\Phi}");
  defineMacro("\\varPsi", "\\mathit{\\Psi}");
  defineMacro("\\varOmega", "\\mathit{\\Omega}"); //\newcommand{\substack}[1]{\subarray{c}#1\endsubarray}

  defineMacro("\\substack", "\\begin{subarray}{c}#1\\end{subarray}"); // \renewcommand{\colon}{\nobreak\mskip2mu\mathpunct{}\nonscript
  // \mkern-\thinmuskip{:}\mskip6muplus1mu\relax}

  defineMacro("\\colon", "\\nobreak\\mskip2mu\\mathpunct{}" + "\\mathchoice{\\mkern-3mu}{\\mkern-3mu}{}{}{:}\\mskip6mu\\relax"); // \newcommand{\boxed}[1]{\fbox{\m@th$\displaystyle#1$}}

  defineMacro("\\boxed", "\\fbox{$\\displaystyle{#1}$}"); // \def\iff{\DOTSB\;\Longleftrightarrow\;}
  // \def\implies{\DOTSB\;\Longrightarrow\;}
  // \def\impliedby{\DOTSB\;\Longleftarrow\;}

  defineMacro("\\iff", "\\DOTSB\\;\\Longleftrightarrow\\;");
  defineMacro("\\implies", "\\DOTSB\\;\\Longrightarrow\\;");
  defineMacro("\\impliedby", "\\DOTSB\\;\\Longleftarrow\\;"); // AMSMath's automatic \dots, based on \mdots@@ macro.

  var dotsByToken = {
    ',': '\\dotsc',
    '\\not': '\\dotsb',
    // \keybin@ checks for the following:
    '+': '\\dotsb',
    '=': '\\dotsb',
    '<': '\\dotsb',
    '>': '\\dotsb',
    '-': '\\dotsb',
    '*': '\\dotsb',
    ':': '\\dotsb',
    // Symbols whose definition starts with \DOTSB:
    '\\DOTSB': '\\dotsb',
    '\\coprod': '\\dotsb',
    '\\bigvee': '\\dotsb',
    '\\bigwedge': '\\dotsb',
    '\\biguplus': '\\dotsb',
    '\\bigcap': '\\dotsb',
    '\\bigcup': '\\dotsb',
    '\\prod': '\\dotsb',
    '\\sum': '\\dotsb',
    '\\bigotimes': '\\dotsb',
    '\\bigoplus': '\\dotsb',
    '\\bigodot': '\\dotsb',
    '\\bigsqcup': '\\dotsb',
    '\\And': '\\dotsb',
    '\\longrightarrow': '\\dotsb',
    '\\Longrightarrow': '\\dotsb',
    '\\longleftarrow': '\\dotsb',
    '\\Longleftarrow': '\\dotsb',
    '\\longleftrightarrow': '\\dotsb',
    '\\Longleftrightarrow': '\\dotsb',
    '\\mapsto': '\\dotsb',
    '\\longmapsto': '\\dotsb',
    '\\hookrightarrow': '\\dotsb',
    '\\doteq': '\\dotsb',
    // Symbols whose definition starts with \mathbin:
    '\\mathbin': '\\dotsb',
    // Symbols whose definition starts with \mathrel:
    '\\mathrel': '\\dotsb',
    '\\relbar': '\\dotsb',
    '\\Relbar': '\\dotsb',
    '\\xrightarrow': '\\dotsb',
    '\\xleftarrow': '\\dotsb',
    // Symbols whose definition starts with \DOTSI:
    '\\DOTSI': '\\dotsi',
    '\\int': '\\dotsi',
    '\\oint': '\\dotsi',
    '\\iint': '\\dotsi',
    '\\iiint': '\\dotsi',
    '\\iiiint': '\\dotsi',
    '\\idotsint': '\\dotsi',
    // Symbols whose definition starts with \DOTSX:
    '\\DOTSX': '\\dotsx'
  };
  defineMacro("\\dots", function (context) {
    // TODO: If used in text mode, should expand to \textellipsis.
    // However, in KaTeX, \textellipsis and \ldots behave the same
    // (in text mode), and it's unlikely we'd see any of the math commands
    // that affect the behavior of \dots when in text mode.  So fine for now
    // (until we support \ifmmode ... \else ... \fi).
    var thedots = '\\dotso';
    var next = context.expandAfterFuture().text;

    if (next in dotsByToken) {
      thedots = dotsByToken[next];
    } else if (next.substr(0, 4) === '\\not') {
      thedots = '\\dotsb';
    } else if (next in symbols.math) {
      if (utils$1.contains(['bin', 'rel'], symbols.math[next].group)) {
        thedots = '\\dotsb';
      }
    }

    return thedots;
  });
  var spaceAfterDots = {
    // \rightdelim@ checks for the following:
    ')': true,
    ']': true,
    '\\rbrack': true,
    '\\}': true,
    '\\rbrace': true,
    '\\rangle': true,
    '\\rceil': true,
    '\\rfloor': true,
    '\\rgroup': true,
    '\\rmoustache': true,
    '\\right': true,
    '\\bigr': true,
    '\\biggr': true,
    '\\Bigr': true,
    '\\Biggr': true,
    // \extra@ also tests for the following:
    '$': true,
    // \extrap@ checks for the following:
    ';': true,
    '.': true,
    ',': true
  };
  defineMacro("\\dotso", function (context) {
    var next = context.future().text;

    if (next in spaceAfterDots) {
      return "\\ldots\\,";
    } else {
      return "\\ldots";
    }
  });
  defineMacro("\\dotsc", function (context) {
    var next = context.future().text; // \dotsc uses \extra@ but not \extrap@, instead specially checking for
    // ';' and '.', but doesn't check for ','.

    if (next in spaceAfterDots && next !== ',') {
      return "\\ldots\\,";
    } else {
      return "\\ldots";
    }
  });
  defineMacro("\\cdots", function (context) {
    var next = context.future().text;

    if (next in spaceAfterDots) {
      return "\\@cdots\\,";
    } else {
      return "\\@cdots";
    }
  });
  defineMacro("\\dotsb", "\\cdots");
  defineMacro("\\dotsm", "\\cdots");
  defineMacro("\\dotsi", "\\!\\cdots"); // amsmath doesn't actually define \dotsx, but \dots followed by a macro
  // starting with \DOTSX implies \dotso, and then \extra@ detects this case
  // and forces the added `\,`.

  defineMacro("\\dotsx", "\\ldots\\,"); // \let\DOTSI\relax
  // \let\DOTSB\relax
  // \let\DOTSX\relax

  defineMacro("\\DOTSI", "\\relax");
  defineMacro("\\DOTSB", "\\relax");
  defineMacro("\\DOTSX", "\\relax"); // Spacing, based on amsmath.sty's override of LaTeX defaults
  // \DeclareRobustCommand{\tmspace}[3]{%
  //   \ifmmode\mskip#1#2\else\kern#1#3\fi\relax}

  defineMacro("\\tmspace", "\\TextOrMath{\\kern#1#3}{\\mskip#1#2}\\relax"); // \renewcommand{\,}{\tmspace+\thinmuskip{.1667em}}
  // TODO: math mode should use \thinmuskip

  defineMacro("\\,", "\\tmspace+{3mu}{.1667em}"); // \let\thinspace\,

  defineMacro("\\thinspace", "\\,"); // \def\>{\mskip\medmuskip}
  // \renewcommand{\:}{\tmspace+\medmuskip{.2222em}}
  // TODO: \> and math mode of \: should use \medmuskip = 4mu plus 2mu minus 4mu

  defineMacro("\\>", "\\mskip{4mu}");
  defineMacro("\\:", "\\tmspace+{4mu}{.2222em}"); // \let\medspace\:

  defineMacro("\\medspace", "\\:"); // \renewcommand{\;}{\tmspace+\thickmuskip{.2777em}}
  // TODO: math mode should use \thickmuskip = 5mu plus 5mu

  defineMacro("\\;", "\\tmspace+{5mu}{.2777em}"); // \let\thickspace\;

  defineMacro("\\thickspace", "\\;"); // \renewcommand{\!}{\tmspace-\thinmuskip{.1667em}}
  // TODO: math mode should use \thinmuskip

  defineMacro("\\!", "\\tmspace-{3mu}{.1667em}"); // \let\negthinspace\!

  defineMacro("\\negthinspace", "\\!"); // \newcommand{\negmedspace}{\tmspace-\medmuskip{.2222em}}
  // TODO: math mode should use \medmuskip

  defineMacro("\\negmedspace", "\\tmspace-{4mu}{.2222em}"); // \newcommand{\negthickspace}{\tmspace-\thickmuskip{.2777em}}
  // TODO: math mode should use \thickmuskip

  defineMacro("\\negthickspace", "\\tmspace-{5mu}{.277em}"); // \def\enspace{\kern.5em }

  defineMacro("\\enspace", "\\kern.5em "); // \def\enskip{\hskip.5em\relax}

  defineMacro("\\enskip", "\\hskip.5em\\relax"); // \def\quad{\hskip1em\relax}

  defineMacro("\\quad", "\\hskip1em\\relax"); // \def\qquad{\hskip2em\relax}

  defineMacro("\\qquad", "\\hskip2em\\relax"); // \tag@in@display form of \tag

  defineMacro("\\tag", "\\@ifstar\\tag@literal\\tag@paren");
  defineMacro("\\tag@paren", "\\tag@literal{({#1})}");
  defineMacro("\\tag@literal", context => {
    if (context.macros.get("\\df@tag")) {
      throw new ParseError("Multiple \\tag");
    }

    return "\\gdef\\df@tag{\\text{#1}}";
  }); // \renewcommand{\bmod}{\nonscript\mskip-\medmuskip\mkern5mu\mathbin
  //   {\operator@font mod}\penalty900
  //   \mkern5mu\nonscript\mskip-\medmuskip}
  // \newcommand{\pod}[1]{\allowbreak
  //   \if@display\mkern18mu\else\mkern8mu\fi(#1)}
  // \renewcommand{\pmod}[1]{\pod{{\operator@font mod}\mkern6mu#1}}
  // \newcommand{\mod}[1]{\allowbreak\if@display\mkern18mu
  //   \else\mkern12mu\fi{\operator@font mod}\,\,#1}
  // TODO: math mode should use \medmuskip = 4mu plus 2mu minus 4mu

  defineMacro("\\bmod", "\\mathchoice{\\mskip1mu}{\\mskip1mu}{\\mskip5mu}{\\mskip5mu}" + "\\mathbin{\\rm mod}" + "\\mathchoice{\\mskip1mu}{\\mskip1mu}{\\mskip5mu}{\\mskip5mu}");
  defineMacro("\\pod", "\\allowbreak" + "\\mathchoice{\\mkern18mu}{\\mkern8mu}{\\mkern8mu}{\\mkern8mu}(#1)");
  defineMacro("\\pmod", "\\pod{{\\rm mod}\\mkern6mu#1}");
  defineMacro("\\mod", "\\allowbreak" + "\\mathchoice{\\mkern18mu}{\\mkern12mu}{\\mkern12mu}{\\mkern12mu}" + "{\\rm mod}\\,\\,#1"); // \pmb    --   A simulation of bold.
  // The version in ambsy.sty works by typesetting three copies of the argument
  // with small offsets. We use two copies. We omit the vertical offset because
  // of rendering problems that makeVList encounters in Safari.

  defineMacro("\\pmb", "\\html@mathml{" + "\\@binrel{#1}{\\mathrlap{#1}\\kern0.5px#1}}" + "{\\mathbf{#1}}"); //////////////////////////////////////////////////////////////////////
  // LaTeX source2e
  // \expandafter\let\expandafter\@normalcr
  //     \csname\expandafter\@gobble\string\\ \endcsname
  // \DeclareRobustCommand\newline{\@normalcr\relax}

  defineMacro("\\newline", "\\\\\\relax"); // \def\TeX{T\kern-.1667em\lower.5ex\hbox{E}\kern-.125emX\@}
  // TODO: Doesn't normally work in math mode because \@ fails.  KaTeX doesn't
  // support \@ yet, so that's omitted, and we add \text so that the result
  // doesn't look funny in math mode.

  defineMacro("\\TeX", "\\textrm{\\html@mathml{" + "T\\kern-.1667em\\raisebox{-.5ex}{E}\\kern-.125emX" + "}{TeX}}"); // \DeclareRobustCommand{\LaTeX}{L\kern-.36em%
  //         {\sbox\z@ T%
  //          \vbox to\ht\z@{\hbox{\check@mathfonts
  //                               \fontsize\sf@size\z@
  //                               \math@fontsfalse\selectfont
  //                               A}%
  //                         \vss}%
  //         }%
  //         \kern-.15em%
  //         \TeX}
  // This code aligns the top of the A with the T (from the perspective of TeX's
  // boxes, though visually the A appears to extend above slightly).
  // We compute the corresponding \raisebox when A is rendered in \normalsize
  // \scriptstyle, which has a scale factor of 0.7 (see Options.js).

  var latexRaiseA = makeEm(fontMetricsData['Main-Regular']["T".charCodeAt(0)][1] - 0.7 * fontMetricsData['Main-Regular']["A".charCodeAt(0)][1]);
  defineMacro("\\LaTeX", "\\textrm{\\html@mathml{" + ("L\\kern-.36em\\raisebox{" + latexRaiseA + "}{\\scriptstyle A}") + "\\kern-.15em\\TeX}{LaTeX}}"); // New KaTeX logo based on tweaking LaTeX logo

  defineMacro("\\KaTeX", "\\textrm{\\html@mathml{" + ("K\\kern-.17em\\raisebox{" + latexRaiseA + "}{\\scriptstyle A}") + "\\kern-.15em\\TeX}{KaTeX}}"); // \DeclareRobustCommand\hspace{\@ifstar\@hspacer\@hspace}
  // \def\@hspace#1{\hskip  #1\relax}
  // \def\@hspacer#1{\vrule \@width\z@\nobreak
  //                 \hskip #1\hskip \z@skip}

  defineMacro("\\hspace", "\\@ifstar\\@hspacer\\@hspace");
  defineMacro("\\@hspace", "\\hskip #1\\relax");
  defineMacro("\\@hspacer", "\\rule{0pt}{0pt}\\hskip #1\\relax"); //////////////////////////////////////////////////////////////////////
  // mathtools.sty
  //\providecommand\ordinarycolon{:}

  defineMacro("\\ordinarycolon", ":"); //\def\vcentcolon{\mathrel{\mathop\ordinarycolon}}
  //TODO(edemaine): Not yet centered. Fix via \raisebox or #726

  defineMacro("\\vcentcolon", "\\mathrel{\\mathop\\ordinarycolon}"); // \providecommand*\dblcolon{\vcentcolon\mathrel{\mkern-.9mu}\vcentcolon}

  defineMacro("\\dblcolon", "\\html@mathml{" + "\\mathrel{\\vcentcolon\\mathrel{\\mkern-.9mu}\\vcentcolon}}" + "{\\mathop{\\char\"2237}}"); // \providecommand*\coloneqq{\vcentcolon\mathrel{\mkern-1.2mu}=}

  defineMacro("\\coloneqq", "\\html@mathml{" + "\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}=}}" + "{\\mathop{\\char\"2254}}"); // ≔
  // \providecommand*\Coloneqq{\dblcolon\mathrel{\mkern-1.2mu}=}

  defineMacro("\\Coloneqq", "\\html@mathml{" + "\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}=}}" + "{\\mathop{\\char\"2237\\char\"3d}}"); // \providecommand*\coloneq{\vcentcolon\mathrel{\mkern-1.2mu}\mathrel{-}}

  defineMacro("\\coloneq", "\\html@mathml{" + "\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}\\mathrel{-}}}" + "{\\mathop{\\char\"3a\\char\"2212}}"); // \providecommand*\Coloneq{\dblcolon\mathrel{\mkern-1.2mu}\mathrel{-}}

  defineMacro("\\Coloneq", "\\html@mathml{" + "\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}\\mathrel{-}}}" + "{\\mathop{\\char\"2237\\char\"2212}}"); // \providecommand*\eqqcolon{=\mathrel{\mkern-1.2mu}\vcentcolon}

  defineMacro("\\eqqcolon", "\\html@mathml{" + "\\mathrel{=\\mathrel{\\mkern-1.2mu}\\vcentcolon}}" + "{\\mathop{\\char\"2255}}"); // ≕
  // \providecommand*\Eqqcolon{=\mathrel{\mkern-1.2mu}\dblcolon}

  defineMacro("\\Eqqcolon", "\\html@mathml{" + "\\mathrel{=\\mathrel{\\mkern-1.2mu}\\dblcolon}}" + "{\\mathop{\\char\"3d\\char\"2237}}"); // \providecommand*\eqcolon{\mathrel{-}\mathrel{\mkern-1.2mu}\vcentcolon}

  defineMacro("\\eqcolon", "\\html@mathml{" + "\\mathrel{\\mathrel{-}\\mathrel{\\mkern-1.2mu}\\vcentcolon}}" + "{\\mathop{\\char\"2239}}"); // \providecommand*\Eqcolon{\mathrel{-}\mathrel{\mkern-1.2mu}\dblcolon}

  defineMacro("\\Eqcolon", "\\html@mathml{" + "\\mathrel{\\mathrel{-}\\mathrel{\\mkern-1.2mu}\\dblcolon}}" + "{\\mathop{\\char\"2212\\char\"2237}}"); // \providecommand*\colonapprox{\vcentcolon\mathrel{\mkern-1.2mu}\approx}

  defineMacro("\\colonapprox", "\\html@mathml{" + "\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}\\approx}}" + "{\\mathop{\\char\"3a\\char\"2248}}"); // \providecommand*\Colonapprox{\dblcolon\mathrel{\mkern-1.2mu}\approx}

  defineMacro("\\Colonapprox", "\\html@mathml{" + "\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}\\approx}}" + "{\\mathop{\\char\"2237\\char\"2248}}"); // \providecommand*\colonsim{\vcentcolon\mathrel{\mkern-1.2mu}\sim}

  defineMacro("\\colonsim", "\\html@mathml{" + "\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}\\sim}}" + "{\\mathop{\\char\"3a\\char\"223c}}"); // \providecommand*\Colonsim{\dblcolon\mathrel{\mkern-1.2mu}\sim}

  defineMacro("\\Colonsim", "\\html@mathml{" + "\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}\\sim}}" + "{\\mathop{\\char\"2237\\char\"223c}}"); // Some Unicode characters are implemented with macros to mathtools functions.

  defineMacro("\u2237", "\\dblcolon"); // ::

  defineMacro("\u2239", "\\eqcolon"); // -:

  defineMacro("\u2254", "\\coloneqq"); // :=

  defineMacro("\u2255", "\\eqqcolon"); // =:

  defineMacro("\u2A74", "\\Coloneqq"); // ::=
  //////////////////////////////////////////////////////////////////////
  // colonequals.sty
  // Alternate names for mathtools's macros:

  defineMacro("\\ratio", "\\vcentcolon");
  defineMacro("\\coloncolon", "\\dblcolon");
  defineMacro("\\colonequals", "\\coloneqq");
  defineMacro("\\coloncolonequals", "\\Coloneqq");
  defineMacro("\\equalscolon", "\\eqqcolon");
  defineMacro("\\equalscoloncolon", "\\Eqqcolon");
  defineMacro("\\colonminus", "\\coloneq");
  defineMacro("\\coloncolonminus", "\\Coloneq");
  defineMacro("\\minuscolon", "\\eqcolon");
  defineMacro("\\minuscoloncolon", "\\Eqcolon"); // \colonapprox name is same in mathtools and colonequals.

  defineMacro("\\coloncolonapprox", "\\Colonapprox"); // \colonsim name is same in mathtools and colonequals.

  defineMacro("\\coloncolonsim", "\\Colonsim"); // Additional macros, implemented by analogy with mathtools definitions:

  defineMacro("\\simcolon", "\\mathrel{\\sim\\mathrel{\\mkern-1.2mu}\\vcentcolon}");
  defineMacro("\\simcoloncolon", "\\mathrel{\\sim\\mathrel{\\mkern-1.2mu}\\dblcolon}");
  defineMacro("\\approxcolon", "\\mathrel{\\approx\\mathrel{\\mkern-1.2mu}\\vcentcolon}");
  defineMacro("\\approxcoloncolon", "\\mathrel{\\approx\\mathrel{\\mkern-1.2mu}\\dblcolon}"); // Present in newtxmath, pxfonts and txfonts

  defineMacro("\\notni", "\\html@mathml{\\not\\ni}{\\mathrel{\\char`\u220C}}");
  defineMacro("\\limsup", "\\DOTSB\\operatorname*{lim\\,sup}");
  defineMacro("\\liminf", "\\DOTSB\\operatorname*{lim\\,inf}"); //////////////////////////////////////////////////////////////////////
  // From amsopn.sty

  defineMacro("\\injlim", "\\DOTSB\\operatorname*{inj\\,lim}");
  defineMacro("\\projlim", "\\DOTSB\\operatorname*{proj\\,lim}");
  defineMacro("\\varlimsup", "\\DOTSB\\operatorname*{\\overline{lim}}");
  defineMacro("\\varliminf", "\\DOTSB\\operatorname*{\\underline{lim}}");
  defineMacro("\\varinjlim", "\\DOTSB\\operatorname*{\\underrightarrow{lim}}");
  defineMacro("\\varprojlim", "\\DOTSB\\operatorname*{\\underleftarrow{lim}}"); //////////////////////////////////////////////////////////////////////
  // MathML alternates for KaTeX glyphs in the Unicode private area

  defineMacro("\\gvertneqq", "\\html@mathml{\\@gvertneqq}{\u2269}");
  defineMacro("\\lvertneqq", "\\html@mathml{\\@lvertneqq}{\u2268}");
  defineMacro("\\ngeqq", "\\html@mathml{\\@ngeqq}{\u2271}");
  defineMacro("\\ngeqslant", "\\html@mathml{\\@ngeqslant}{\u2271}");
  defineMacro("\\nleqq", "\\html@mathml{\\@nleqq}{\u2270}");
  defineMacro("\\nleqslant", "\\html@mathml{\\@nleqslant}{\u2270}");
  defineMacro("\\nshortmid", "\\html@mathml{\\@nshortmid}{∤}");
  defineMacro("\\nshortparallel", "\\html@mathml{\\@nshortparallel}{∦}");
  defineMacro("\\nsubseteqq", "\\html@mathml{\\@nsubseteqq}{\u2288}");
  defineMacro("\\nsupseteqq", "\\html@mathml{\\@nsupseteqq}{\u2289}");
  defineMacro("\\varsubsetneq", "\\html@mathml{\\@varsubsetneq}{⊊}");
  defineMacro("\\varsubsetneqq", "\\html@mathml{\\@varsubsetneqq}{⫋}");
  defineMacro("\\varsupsetneq", "\\html@mathml{\\@varsupsetneq}{⊋}");
  defineMacro("\\varsupsetneqq", "\\html@mathml{\\@varsupsetneqq}{⫌}");
  defineMacro("\\imath", "\\html@mathml{\\@imath}{\u0131}");
  defineMacro("\\jmath", "\\html@mathml{\\@jmath}{\u0237}"); //////////////////////////////////////////////////////////////////////
  // stmaryrd and semantic
  // The stmaryrd and semantic packages render the next four items by calling a
  // glyph. Those glyphs do not exist in the KaTeX fonts. Hence the macros.

  defineMacro("\\llbracket", "\\html@mathml{" + "\\mathopen{[\\mkern-3.2mu[}}" + "{\\mathopen{\\char`\u27e6}}");
  defineMacro("\\rrbracket", "\\html@mathml{" + "\\mathclose{]\\mkern-3.2mu]}}" + "{\\mathclose{\\char`\u27e7}}");
  defineMacro("\u27e6", "\\llbracket"); // blackboard bold [

  defineMacro("\u27e7", "\\rrbracket"); // blackboard bold ]

  defineMacro("\\lBrace", "\\html@mathml{" + "\\mathopen{\\{\\mkern-3.2mu[}}" + "{\\mathopen{\\char`\u2983}}");
  defineMacro("\\rBrace", "\\html@mathml{" + "\\mathclose{]\\mkern-3.2mu\\}}}" + "{\\mathclose{\\char`\u2984}}");
  defineMacro("\u2983", "\\lBrace"); // blackboard bold {

  defineMacro("\u2984", "\\rBrace"); // blackboard bold }
  // TODO: Create variable sized versions of the last two items. I believe that
  // will require new font glyphs.
  // The stmaryrd function `\minuso` provides a "Plimsoll" symbol that
  // superimposes the characters \circ and \mathminus. Used in chemistry.

  defineMacro("\\minuso", "\\mathbin{\\html@mathml{" + "{\\mathrlap{\\mathchoice{\\kern{0.145em}}{\\kern{0.145em}}" + "{\\kern{0.1015em}}{\\kern{0.0725em}}\\circ}{-}}}" + "{\\char`⦵}}");
  defineMacro("⦵", "\\minuso"); //////////////////////////////////////////////////////////////////////
  // texvc.sty
  // The texvc package contains macros available in mediawiki pages.
  // We omit the functions deprecated at
  // https://en.wikipedia.org/wiki/Help:Displaying_a_formula#Deprecated_syntax
  // We also omit texvc's \O, which conflicts with \text{\O}

  defineMacro("\\darr", "\\downarrow");
  defineMacro("\\dArr", "\\Downarrow");
  defineMacro("\\Darr", "\\Downarrow");
  defineMacro("\\lang", "\\langle");
  defineMacro("\\rang", "\\rangle");
  defineMacro("\\uarr", "\\uparrow");
  defineMacro("\\uArr", "\\Uparrow");
  defineMacro("\\Uarr", "\\Uparrow");
  defineMacro("\\N", "\\mathbb{N}");
  defineMacro("\\R", "\\mathbb{R}");
  defineMacro("\\Z", "\\mathbb{Z}");
  defineMacro("\\alef", "\\aleph");
  defineMacro("\\alefsym", "\\aleph");
  defineMacro("\\Alpha", "\\mathrm{A}");
  defineMacro("\\Beta", "\\mathrm{B}");
  defineMacro("\\bull", "\\bullet");
  defineMacro("\\Chi", "\\mathrm{X}");
  defineMacro("\\clubs", "\\clubsuit");
  defineMacro("\\cnums", "\\mathbb{C}");
  defineMacro("\\Complex", "\\mathbb{C}");
  defineMacro("\\Dagger", "\\ddagger");
  defineMacro("\\diamonds", "\\diamondsuit");
  defineMacro("\\empty", "\\emptyset");
  defineMacro("\\Epsilon", "\\mathrm{E}");
  defineMacro("\\Eta", "\\mathrm{H}");
  defineMacro("\\exist", "\\exists");
  defineMacro("\\harr", "\\leftrightarrow");
  defineMacro("\\hArr", "\\Leftrightarrow");
  defineMacro("\\Harr", "\\Leftrightarrow");
  defineMacro("\\hearts", "\\heartsuit");
  defineMacro("\\image", "\\Im");
  defineMacro("\\infin", "\\infty");
  defineMacro("\\Iota", "\\mathrm{I}");
  defineMacro("\\isin", "\\in");
  defineMacro("\\Kappa", "\\mathrm{K}");
  defineMacro("\\larr", "\\leftarrow");
  defineMacro("\\lArr", "\\Leftarrow");
  defineMacro("\\Larr", "\\Leftarrow");
  defineMacro("\\lrarr", "\\leftrightarrow");
  defineMacro("\\lrArr", "\\Leftrightarrow");
  defineMacro("\\Lrarr", "\\Leftrightarrow");
  defineMacro("\\Mu", "\\mathrm{M}");
  defineMacro("\\natnums", "\\mathbb{N}");
  defineMacro("\\Nu", "\\mathrm{N}");
  defineMacro("\\Omicron", "\\mathrm{O}");
  defineMacro("\\plusmn", "\\pm");
  defineMacro("\\rarr", "\\rightarrow");
  defineMacro("\\rArr", "\\Rightarrow");
  defineMacro("\\Rarr", "\\Rightarrow");
  defineMacro("\\real", "\\Re");
  defineMacro("\\reals", "\\mathbb{R}");
  defineMacro("\\Reals", "\\mathbb{R}");
  defineMacro("\\Rho", "\\mathrm{P}");
  defineMacro("\\sdot", "\\cdot");
  defineMacro("\\sect", "\\S");
  defineMacro("\\spades", "\\spadesuit");
  defineMacro("\\sub", "\\subset");
  defineMacro("\\sube", "\\subseteq");
  defineMacro("\\supe", "\\supseteq");
  defineMacro("\\Tau", "\\mathrm{T}");
  defineMacro("\\thetasym", "\\vartheta"); // TODO: defineMacro("\\varcoppa", "\\\mbox{\\coppa}");

  defineMacro("\\weierp", "\\wp");
  defineMacro("\\Zeta", "\\mathrm{Z}"); //////////////////////////////////////////////////////////////////////
  // statmath.sty
  // https://ctan.math.illinois.edu/macros/latex/contrib/statmath/statmath.pdf

  defineMacro("\\argmin", "\\DOTSB\\operatorname*{arg\\,min}");
  defineMacro("\\argmax", "\\DOTSB\\operatorname*{arg\\,max}");
  defineMacro("\\plim", "\\DOTSB\\mathop{\\operatorname{plim}}\\limits"); //////////////////////////////////////////////////////////////////////
  // braket.sty
  // http://ctan.math.washington.edu/tex-archive/macros/latex/contrib/braket/braket.pdf

  defineMacro("\\bra", "\\mathinner{\\langle{#1}|}");
  defineMacro("\\ket", "\\mathinner{|{#1}\\rangle}");
  defineMacro("\\braket", "\\mathinner{\\langle{#1}\\rangle}");
  defineMacro("\\Bra", "\\left\\langle#1\\right|");
  defineMacro("\\Ket", "\\left|#1\\right\\rangle"); //////////////////////////////////////////////////////////////////////
  // actuarialangle.dtx

  defineMacro("\\angln", "{\\angl n}"); // Custom Khan Academy colors, should be moved to an optional package

  defineMacro("\\blue", "\\textcolor{##6495ed}{#1}");
  defineMacro("\\orange", "\\textcolor{##ffa500}{#1}");
  defineMacro("\\pink", "\\textcolor{##ff00af}{#1}");
  defineMacro("\\red", "\\textcolor{##df0030}{#1}");
  defineMacro("\\green", "\\textcolor{##28ae7b}{#1}");
  defineMacro("\\gray", "\\textcolor{gray}{#1}");
  defineMacro("\\purple", "\\textcolor{##9d38bd}{#1}");
  defineMacro("\\blueA", "\\textcolor{##ccfaff}{#1}");
  defineMacro("\\blueB", "\\textcolor{##80f6ff}{#1}");
  defineMacro("\\blueC", "\\textcolor{##63d9ea}{#1}");
  defineMacro("\\blueD", "\\textcolor{##11accd}{#1}");
  defineMacro("\\blueE", "\\textcolor{##0c7f99}{#1}");
  defineMacro("\\tealA", "\\textcolor{##94fff5}{#1}");
  defineMacro("\\tealB", "\\textcolor{##26edd5}{#1}");
  defineMacro("\\tealC", "\\textcolor{##01d1c1}{#1}");
  defineMacro("\\tealD", "\\textcolor{##01a995}{#1}");
  defineMacro("\\tealE", "\\textcolor{##208170}{#1}");
  defineMacro("\\greenA", "\\textcolor{##b6ffb0}{#1}");
  defineMacro("\\greenB", "\\textcolor{##8af281}{#1}");
  defineMacro("\\greenC", "\\textcolor{##74cf70}{#1}");
  defineMacro("\\greenD", "\\textcolor{##1fab54}{#1}");
  defineMacro("\\greenE", "\\textcolor{##0d923f}{#1}");
  defineMacro("\\goldA", "\\textcolor{##ffd0a9}{#1}");
  defineMacro("\\goldB", "\\textcolor{##ffbb71}{#1}");
  defineMacro("\\goldC", "\\textcolor{##ff9c39}{#1}");
  defineMacro("\\goldD", "\\textcolor{##e07d10}{#1}");
  defineMacro("\\goldE", "\\textcolor{##a75a05}{#1}");
  defineMacro("\\redA", "\\textcolor{##fca9a9}{#1}");
  defineMacro("\\redB", "\\textcolor{##ff8482}{#1}");
  defineMacro("\\redC", "\\textcolor{##f9685d}{#1}");
  defineMacro("\\redD", "\\textcolor{##e84d39}{#1}");
  defineMacro("\\redE", "\\textcolor{##bc2612}{#1}");
  defineMacro("\\maroonA", "\\textcolor{##ffbde0}{#1}");
  defineMacro("\\maroonB", "\\textcolor{##ff92c6}{#1}");
  defineMacro("\\maroonC", "\\textcolor{##ed5fa6}{#1}");
  defineMacro("\\maroonD", "\\textcolor{##ca337c}{#1}");
  defineMacro("\\maroonE", "\\textcolor{##9e034e}{#1}");
  defineMacro("\\purpleA", "\\textcolor{##ddd7ff}{#1}");
  defineMacro("\\purpleB", "\\textcolor{##c6b9fc}{#1}");
  defineMacro("\\purpleC", "\\textcolor{##aa87ff}{#1}");
  defineMacro("\\purpleD", "\\textcolor{##7854ab}{#1}");
  defineMacro("\\purpleE", "\\textcolor{##543b78}{#1}");
  defineMacro("\\mintA", "\\textcolor{##f5f9e8}{#1}");
  defineMacro("\\mintB", "\\textcolor{##edf2df}{#1}");
  defineMacro("\\mintC", "\\textcolor{##e0e5cc}{#1}");
  defineMacro("\\grayA", "\\textcolor{##f6f7f7}{#1}");
  defineMacro("\\grayB", "\\textcolor{##f0f1f2}{#1}");
  defineMacro("\\grayC", "\\textcolor{##e3e5e6}{#1}");
  defineMacro("\\grayD", "\\textcolor{##d6d8da}{#1}");
  defineMacro("\\grayE", "\\textcolor{##babec2}{#1}");
  defineMacro("\\grayF", "\\textcolor{##888d93}{#1}");
  defineMacro("\\grayG", "\\textcolor{##626569}{#1}");
  defineMacro("\\grayH", "\\textcolor{##3b3e40}{#1}");
  defineMacro("\\grayI", "\\textcolor{##21242c}{#1}");
  defineMacro("\\kaBlue", "\\textcolor{##314453}{#1}");
  defineMacro("\\kaGreen", "\\textcolor{##71B307}{#1}");

  /**
   * This file contains the “gullet” where macros are expanded
   * until only non-macro tokens remain.
   */
  // List of commands that act like macros but aren't defined as a macro,
  // function, or symbol.  Used in `isDefined`.
  var implicitCommands = {
    "^": true,
    // Parser.js
    "_": true,
    // Parser.js
    "\\limits": true,
    // Parser.js
    "\\nolimits": true // Parser.js

  };
  class MacroExpander {
    constructor(input, settings, mode) {
      this.settings = void 0;
      this.expansionCount = void 0;
      this.lexer = void 0;
      this.macros = void 0;
      this.stack = void 0;
      this.mode = void 0;
      this.settings = settings;
      this.expansionCount = 0;
      this.feed(input); // Make new global namespace

      this.macros = new Namespace(macros, settings.macros);
      this.mode = mode;
      this.stack = []; // contains tokens in REVERSE order
    }
    /**
     * Feed a new input string to the same MacroExpander
     * (with existing macros etc.).
     */


    feed(input) {
      this.lexer = new Lexer(input, this.settings);
    }
    /**
     * Switches between "text" and "math" modes.
     */


    switchMode(newMode) {
      this.mode = newMode;
    }
    /**
     * Start a new group nesting within all namespaces.
     */


    beginGroup() {
      this.macros.beginGroup();
    }
    /**
     * End current group nesting within all namespaces.
     */


    endGroup() {
      this.macros.endGroup();
    }
    /**
     * Ends all currently nested groups (if any), restoring values before the
     * groups began.  Useful in case of an error in the middle of parsing.
     */


    endGroups() {
      this.macros.endGroups();
    }
    /**
     * Returns the topmost token on the stack, without expanding it.
     * Similar in behavior to TeX's `\futurelet`.
     */


    future() {
      if (this.stack.length === 0) {
        this.pushToken(this.lexer.lex());
      }

      return this.stack[this.stack.length - 1];
    }
    /**
     * Remove and return the next unexpanded token.
     */


    popToken() {
      this.future(); // ensure non-empty stack

      return this.stack.pop();
    }
    /**
     * Add a given token to the token stack.  In particular, this get be used
     * to put back a token returned from one of the other methods.
     */


    pushToken(token) {
      this.stack.push(token);
    }
    /**
     * Append an array of tokens to the token stack.
     */


    pushTokens(tokens) {
      this.stack.push(...tokens);
    }
    /**
     * Find an macro argument without expanding tokens and append the array of
     * tokens to the token stack. Uses Token as a container for the result.
     */


    scanArgument(isOptional) {
      var start;
      var end;
      var tokens;

      if (isOptional) {
        this.consumeSpaces(); // \@ifnextchar gobbles any space following it

        if (this.future().text !== "[") {
          return null;
        }

        start = this.popToken(); // don't include [ in tokens

        ({
          tokens,
          end
        } = this.consumeArg(["]"]));
      } else {
        ({
          tokens,
          start,
          end
        } = this.consumeArg());
      } // indicate the end of an argument


      this.pushToken(new Token("EOF", end.loc));
      this.pushTokens(tokens);
      return start.range(end, "");
    }
    /**
     * Consume all following space tokens, without expansion.
     */


    consumeSpaces() {
      for (;;) {
        var token = this.future();

        if (token.text === " ") {
          this.stack.pop();
        } else {
          break;
        }
      }
    }
    /**
     * Consume an argument from the token stream, and return the resulting array
     * of tokens and start/end token.
     */


    consumeArg(delims) {
      // The argument for a delimited parameter is the shortest (possibly
      // empty) sequence of tokens with properly nested {...} groups that is
      // followed ... by this particular list of non-parameter tokens.
      // The argument for an undelimited parameter is the next nonblank
      // token, unless that token is ‘{’, when the argument will be the
      // entire {...} group that follows.
      var tokens = [];
      var isDelimited = delims && delims.length > 0;

      if (!isDelimited) {
        // Ignore spaces between arguments.  As the TeXbook says:
        // "After you have said ‘\def\row#1#2{...}’, you are allowed to
        //  put spaces between the arguments (e.g., ‘\row x n’), because
        //  TeX doesn’t use single spaces as undelimited arguments."
        this.consumeSpaces();
      }

      var start = this.future();
      var tok;
      var depth = 0;
      var match = 0;

      do {
        tok = this.popToken();
        tokens.push(tok);

        if (tok.text === "{") {
          ++depth;
        } else if (tok.text === "}") {
          --depth;

          if (depth === -1) {
            throw new ParseError("Extra }", tok);
          }
        } else if (tok.text === "EOF") {
          throw new ParseError("Unexpected end of input in a macro argument" + ", expected '" + (delims && isDelimited ? delims[match] : "}") + "'", tok);
        }

        if (delims && isDelimited) {
          if ((depth === 0 || depth === 1 && delims[match] === "{") && tok.text === delims[match]) {
            ++match;

            if (match === delims.length) {
              // don't include delims in tokens
              tokens.splice(-match, match);
              break;
            }
          } else {
            match = 0;
          }
        }
      } while (depth !== 0 || isDelimited); // If the argument found ... has the form ‘{<nested tokens>}’,
      // ... the outermost braces enclosing the argument are removed


      if (start.text === "{" && tokens[tokens.length - 1].text === "}") {
        tokens.pop();
        tokens.shift();
      }

      tokens.reverse(); // to fit in with stack order

      return {
        tokens,
        start,
        end: tok
      };
    }
    /**
     * Consume the specified number of (delimited) arguments from the token
     * stream and return the resulting array of arguments.
     */


    consumeArgs(numArgs, delimiters) {
      if (delimiters) {
        if (delimiters.length !== numArgs + 1) {
          throw new ParseError("The length of delimiters doesn't match the number of args!");
        }

        var delims = delimiters[0];

        for (var i = 0; i < delims.length; i++) {
          var tok = this.popToken();

          if (delims[i] !== tok.text) {
            throw new ParseError("Use of the macro doesn't match its definition", tok);
          }
        }
      }

      var args = [];

      for (var _i = 0; _i < numArgs; _i++) {
        args.push(this.consumeArg(delimiters && delimiters[_i + 1]).tokens);
      }

      return args;
    }
    /**
     * Expand the next token only once if possible.
     *
     * If the token is expanded, the resulting tokens will be pushed onto
     * the stack in reverse order and will be returned as an array,
     * also in reverse order.
     *
     * If not, the next token will be returned without removing it
     * from the stack.  This case can be detected by a `Token` return value
     * instead of an `Array` return value.
     *
     * In either case, the next token will be on the top of the stack,
     * or the stack will be empty.
     *
     * Used to implement `expandAfterFuture` and `expandNextToken`.
     *
     * If expandableOnly, only expandable tokens are expanded and
     * an undefined control sequence results in an error.
     */


    expandOnce(expandableOnly) {
      var topToken = this.popToken();
      var name = topToken.text;
      var expansion = !topToken.noexpand ? this._getExpansion(name) : null;

      if (expansion == null || expandableOnly && expansion.unexpandable) {
        if (expandableOnly && expansion == null && name[0] === "\\" && !this.isDefined(name)) {
          throw new ParseError("Undefined control sequence: " + name);
        }

        this.pushToken(topToken);
        return topToken;
      }

      this.expansionCount++;

      if (this.expansionCount > this.settings.maxExpand) {
        throw new ParseError("Too many expansions: infinite loop or " + "need to increase maxExpand setting");
      }

      var tokens = expansion.tokens;
      var args = this.consumeArgs(expansion.numArgs, expansion.delimiters);

      if (expansion.numArgs) {
        // paste arguments in place of the placeholders
        tokens = tokens.slice(); // make a shallow copy

        for (var i = tokens.length - 1; i >= 0; --i) {
          var tok = tokens[i];

          if (tok.text === "#") {
            if (i === 0) {
              throw new ParseError("Incomplete placeholder at end of macro body", tok);
            }

            tok = tokens[--i]; // next token on stack

            if (tok.text === "#") {
              // ## → #
              tokens.splice(i + 1, 1); // drop first #
            } else if (/^[1-9]$/.test(tok.text)) {
              // replace the placeholder with the indicated argument
              tokens.splice(i, 2, ...args[+tok.text - 1]);
            } else {
              throw new ParseError("Not a valid argument number", tok);
            }
          }
        }
      } // Concatenate expansion onto top of stack.


      this.pushTokens(tokens);
      return tokens;
    }
    /**
     * Expand the next token only once (if possible), and return the resulting
     * top token on the stack (without removing anything from the stack).
     * Similar in behavior to TeX's `\expandafter\futurelet`.
     * Equivalent to expandOnce() followed by future().
     */


    expandAfterFuture() {
      this.expandOnce();
      return this.future();
    }
    /**
     * Recursively expand first token, then return first non-expandable token.
     */


    expandNextToken() {
      for (;;) {
        var expanded = this.expandOnce(); // expandOnce returns Token if and only if it's fully expanded.

        if (expanded instanceof Token) {
          // the token after \noexpand is interpreted as if its meaning
          // were ‘\relax’
          if (expanded.treatAsRelax) {
            expanded.text = "\\relax";
          }

          return this.stack.pop(); // === expanded
        }
      } // Flow unable to figure out that this pathway is impossible.
      // https://github.com/facebook/flow/issues/4808


      throw new Error(); // eslint-disable-line no-unreachable
    }
    /**
     * Fully expand the given macro name and return the resulting list of
     * tokens, or return `undefined` if no such macro is defined.
     */


    expandMacro(name) {
      return this.macros.has(name) ? this.expandTokens([new Token(name)]) : undefined;
    }
    /**
     * Fully expand the given token stream and return the resulting list of tokens
     */


    expandTokens(tokens) {
      var output = [];
      var oldStackLength = this.stack.length;
      this.pushTokens(tokens);

      while (this.stack.length > oldStackLength) {
        var expanded = this.expandOnce(true); // expand only expandable tokens
        // expandOnce returns Token if and only if it's fully expanded.

        if (expanded instanceof Token) {
          if (expanded.treatAsRelax) {
            // the expansion of \noexpand is the token itself
            expanded.noexpand = false;
            expanded.treatAsRelax = false;
          }

          output.push(this.stack.pop());
        }
      }

      return output;
    }
    /**
     * Fully expand the given macro name and return the result as a string,
     * or return `undefined` if no such macro is defined.
     */


    expandMacroAsText(name) {
      var tokens = this.expandMacro(name);

      if (tokens) {
        return tokens.map(token => token.text).join("");
      } else {
        return tokens;
      }
    }
    /**
     * Returns the expanded macro as a reversed array of tokens and a macro
     * argument count.  Or returns `null` if no such macro.
     */


    _getExpansion(name) {
      var definition = this.macros.get(name);

      if (definition == null) {
        // mainly checking for undefined here
        return definition;
      } // If a single character has an associated catcode other than 13
      // (active character), then don't expand it.


      if (name.length === 1) {
        var catcode = this.lexer.catcodes[name];

        if (catcode != null && catcode !== 13) {
          return;
        }
      }

      var expansion = typeof definition === "function" ? definition(this) : definition;

      if (typeof expansion === "string") {
        var numArgs = 0;

        if (expansion.indexOf("#") !== -1) {
          var stripped = expansion.replace(/##/g, "");

          while (stripped.indexOf("#" + (numArgs + 1)) !== -1) {
            ++numArgs;
          }
        }

        var bodyLexer = new Lexer(expansion, this.settings);
        var tokens = [];
        var tok = bodyLexer.lex();

        while (tok.text !== "EOF") {
          tokens.push(tok);
          tok = bodyLexer.lex();
        }

        tokens.reverse(); // to fit in with stack using push and pop

        var expanded = {
          tokens,
          numArgs
        };
        return expanded;
      }

      return expansion;
    }
    /**
     * Determine whether a command is currently "defined" (has some
     * functionality), meaning that it's a macro (in the current group),
     * a function, a symbol, or one of the special commands listed in
     * `implicitCommands`.
     */


    isDefined(name) {
      return this.macros.has(name) || functions.hasOwnProperty(name) || symbols.math.hasOwnProperty(name) || symbols.text.hasOwnProperty(name) || implicitCommands.hasOwnProperty(name);
    }
    /**
     * Determine whether a command is expandable.
     */


    isExpandable(name) {
      var macro = this.macros.get(name);
      return macro != null ? typeof macro === "string" || typeof macro === "function" || !macro.unexpandable : functions.hasOwnProperty(name) && !functions[name].primitive;
    }

  }

  /* eslint no-constant-condition:0 */

  var unicodeAccents = {
    "́": {
      "text": "\\'",
      "math": "\\acute"
    },
    "̀": {
      "text": "\\`",
      "math": "\\grave"
    },
    "̈": {
      "text": "\\\"",
      "math": "\\ddot"
    },
    "̃": {
      "text": "\\~",
      "math": "\\tilde"
    },
    "̄": {
      "text": "\\=",
      "math": "\\bar"
    },
    "̆": {
      "text": "\\u",
      "math": "\\breve"
    },
    "̌": {
      "text": "\\v",
      "math": "\\check"
    },
    "̂": {
      "text": "\\^",
      "math": "\\hat"
    },
    "̇": {
      "text": "\\.",
      "math": "\\dot"
    },
    "̊": {
      "text": "\\r",
      "math": "\\mathring"
    },
    "̋": {
      "text": "\\H"
    },
    "̧": {
      "text": "\\c"
    }
  };
  var unicodeSymbols = {
    "á": "á",
    "à": "à",
    "ä": "ä",
    "ǟ": "ǟ",
    "ã": "ã",
    "ā": "ā",
    "ă": "ă",
    "ắ": "ắ",
    "ằ": "ằ",
    "ẵ": "ẵ",
    "ǎ": "ǎ",
    "â": "â",
    "ấ": "ấ",
    "ầ": "ầ",
    "ẫ": "ẫ",
    "ȧ": "ȧ",
    "ǡ": "ǡ",
    "å": "å",
    "ǻ": "ǻ",
    "ḃ": "ḃ",
    "ć": "ć",
    "ḉ": "ḉ",
    "č": "č",
    "ĉ": "ĉ",
    "ċ": "ċ",
    "ç": "ç",
    "ď": "ď",
    "ḋ": "ḋ",
    "ḑ": "ḑ",
    "é": "é",
    "è": "è",
    "ë": "ë",
    "ẽ": "ẽ",
    "ē": "ē",
    "ḗ": "ḗ",
    "ḕ": "ḕ",
    "ĕ": "ĕ",
    "ḝ": "ḝ",
    "ě": "ě",
    "ê": "ê",
    "ế": "ế",
    "ề": "ề",
    "ễ": "ễ",
    "ė": "ė",
    "ȩ": "ȩ",
    "ḟ": "ḟ",
    "ǵ": "ǵ",
    "ḡ": "ḡ",
    "ğ": "ğ",
    "ǧ": "ǧ",
    "ĝ": "ĝ",
    "ġ": "ġ",
    "ģ": "ģ",
    "ḧ": "ḧ",
    "ȟ": "ȟ",
    "ĥ": "ĥ",
    "ḣ": "ḣ",
    "ḩ": "ḩ",
    "í": "í",
    "ì": "ì",
    "ï": "ï",
    "ḯ": "ḯ",
    "ĩ": "ĩ",
    "ī": "ī",
    "ĭ": "ĭ",
    "ǐ": "ǐ",
    "î": "î",
    "ǰ": "ǰ",
    "ĵ": "ĵ",
    "ḱ": "ḱ",
    "ǩ": "ǩ",
    "ķ": "ķ",
    "ĺ": "ĺ",
    "ľ": "ľ",
    "ļ": "ļ",
    "ḿ": "ḿ",
    "ṁ": "ṁ",
    "ń": "ń",
    "ǹ": "ǹ",
    "ñ": "ñ",
    "ň": "ň",
    "ṅ": "ṅ",
    "ņ": "ņ",
    "ó": "ó",
    "ò": "ò",
    "ö": "ö",
    "ȫ": "ȫ",
    "õ": "õ",
    "ṍ": "ṍ",
    "ṏ": "ṏ",
    "ȭ": "ȭ",
    "ō": "ō",
    "ṓ": "ṓ",
    "ṑ": "ṑ",
    "ŏ": "ŏ",
    "ǒ": "ǒ",
    "ô": "ô",
    "ố": "ố",
    "ồ": "ồ",
    "ỗ": "ỗ",
    "ȯ": "ȯ",
    "ȱ": "ȱ",
    "ő": "ő",
    "ṕ": "ṕ",
    "ṗ": "ṗ",
    "ŕ": "ŕ",
    "ř": "ř",
    "ṙ": "ṙ",
    "ŗ": "ŗ",
    "ś": "ś",
    "ṥ": "ṥ",
    "š": "š",
    "ṧ": "ṧ",
    "ŝ": "ŝ",
    "ṡ": "ṡ",
    "ş": "ş",
    "ẗ": "ẗ",
    "ť": "ť",
    "ṫ": "ṫ",
    "ţ": "ţ",
    "ú": "ú",
    "ù": "ù",
    "ü": "ü",
    "ǘ": "ǘ",
    "ǜ": "ǜ",
    "ǖ": "ǖ",
    "ǚ": "ǚ",
    "ũ": "ũ",
    "ṹ": "ṹ",
    "ū": "ū",
    "ṻ": "ṻ",
    "ŭ": "ŭ",
    "ǔ": "ǔ",
    "û": "û",
    "ů": "ů",
    "ű": "ű",
    "ṽ": "ṽ",
    "ẃ": "ẃ",
    "ẁ": "ẁ",
    "ẅ": "ẅ",
    "ŵ": "ŵ",
    "ẇ": "ẇ",
    "ẘ": "ẘ",
    "ẍ": "ẍ",
    "ẋ": "ẋ",
    "ý": "ý",
    "ỳ": "ỳ",
    "ÿ": "ÿ",
    "ỹ": "ỹ",
    "ȳ": "ȳ",
    "ŷ": "ŷ",
    "ẏ": "ẏ",
    "ẙ": "ẙ",
    "ź": "ź",
    "ž": "ž",
    "ẑ": "ẑ",
    "ż": "ż",
    "Á": "Á",
    "À": "À",
    "Ä": "Ä",
    "Ǟ": "Ǟ",
    "Ã": "Ã",
    "Ā": "Ā",
    "Ă": "Ă",
    "Ắ": "Ắ",
    "Ằ": "Ằ",
    "Ẵ": "Ẵ",
    "Ǎ": "Ǎ",
    "Â": "Â",
    "Ấ": "Ấ",
    "Ầ": "Ầ",
    "Ẫ": "Ẫ",
    "Ȧ": "Ȧ",
    "Ǡ": "Ǡ",
    "Å": "Å",
    "Ǻ": "Ǻ",
    "Ḃ": "Ḃ",
    "Ć": "Ć",
    "Ḉ": "Ḉ",
    "Č": "Č",
    "Ĉ": "Ĉ",
    "Ċ": "Ċ",
    "Ç": "Ç",
    "Ď": "Ď",
    "Ḋ": "Ḋ",
    "Ḑ": "Ḑ",
    "É": "É",
    "È": "È",
    "Ë": "Ë",
    "Ẽ": "Ẽ",
    "Ē": "Ē",
    "Ḗ": "Ḗ",
    "Ḕ": "Ḕ",
    "Ĕ": "Ĕ",
    "Ḝ": "Ḝ",
    "Ě": "Ě",
    "Ê": "Ê",
    "Ế": "Ế",
    "Ề": "Ề",
    "Ễ": "Ễ",
    "Ė": "Ė",
    "Ȩ": "Ȩ",
    "Ḟ": "Ḟ",
    "Ǵ": "Ǵ",
    "Ḡ": "Ḡ",
    "Ğ": "Ğ",
    "Ǧ": "Ǧ",
    "Ĝ": "Ĝ",
    "Ġ": "Ġ",
    "Ģ": "Ģ",
    "Ḧ": "Ḧ",
    "Ȟ": "Ȟ",
    "Ĥ": "Ĥ",
    "Ḣ": "Ḣ",
    "Ḩ": "Ḩ",
    "Í": "Í",
    "Ì": "Ì",
    "Ï": "Ï",
    "Ḯ": "Ḯ",
    "Ĩ": "Ĩ",
    "Ī": "Ī",
    "Ĭ": "Ĭ",
    "Ǐ": "Ǐ",
    "Î": "Î",
    "İ": "İ",
    "Ĵ": "Ĵ",
    "Ḱ": "Ḱ",
    "Ǩ": "Ǩ",
    "Ķ": "Ķ",
    "Ĺ": "Ĺ",
    "Ľ": "Ľ",
    "Ļ": "Ļ",
    "Ḿ": "Ḿ",
    "Ṁ": "Ṁ",
    "Ń": "Ń",
    "Ǹ": "Ǹ",
    "Ñ": "Ñ",
    "Ň": "Ň",
    "Ṅ": "Ṅ",
    "Ņ": "Ņ",
    "Ó": "Ó",
    "Ò": "Ò",
    "Ö": "Ö",
    "Ȫ": "Ȫ",
    "Õ": "Õ",
    "Ṍ": "Ṍ",
    "Ṏ": "Ṏ",
    "Ȭ": "Ȭ",
    "Ō": "Ō",
    "Ṓ": "Ṓ",
    "Ṑ": "Ṑ",
    "Ŏ": "Ŏ",
    "Ǒ": "Ǒ",
    "Ô": "Ô",
    "Ố": "Ố",
    "Ồ": "Ồ",
    "Ỗ": "Ỗ",
    "Ȯ": "Ȯ",
    "Ȱ": "Ȱ",
    "Ő": "Ő",
    "Ṕ": "Ṕ",
    "Ṗ": "Ṗ",
    "Ŕ": "Ŕ",
    "Ř": "Ř",
    "Ṙ": "Ṙ",
    "Ŗ": "Ŗ",
    "Ś": "Ś",
    "Ṥ": "Ṥ",
    "Š": "Š",
    "Ṧ": "Ṧ",
    "Ŝ": "Ŝ",
    "Ṡ": "Ṡ",
    "Ş": "Ş",
    "Ť": "Ť",
    "Ṫ": "Ṫ",
    "Ţ": "Ţ",
    "Ú": "Ú",
    "Ù": "Ù",
    "Ü": "Ü",
    "Ǘ": "Ǘ",
    "Ǜ": "Ǜ",
    "Ǖ": "Ǖ",
    "Ǚ": "Ǚ",
    "Ũ": "Ũ",
    "Ṹ": "Ṹ",
    "Ū": "Ū",
    "Ṻ": "Ṻ",
    "Ŭ": "Ŭ",
    "Ǔ": "Ǔ",
    "Û": "Û",
    "Ů": "Ů",
    "Ű": "Ű",
    "Ṽ": "Ṽ",
    "Ẃ": "Ẃ",
    "Ẁ": "Ẁ",
    "Ẅ": "Ẅ",
    "Ŵ": "Ŵ",
    "Ẇ": "Ẇ",
    "Ẍ": "Ẍ",
    "Ẋ": "Ẋ",
    "Ý": "Ý",
    "Ỳ": "Ỳ",
    "Ÿ": "Ÿ",
    "Ỹ": "Ỹ",
    "Ȳ": "Ȳ",
    "Ŷ": "Ŷ",
    "Ẏ": "Ẏ",
    "Ź": "Ź",
    "Ž": "Ž",
    "Ẑ": "Ẑ",
    "Ż": "Ż",
    "ά": "ά",
    "ὰ": "ὰ",
    "ᾱ": "ᾱ",
    "ᾰ": "ᾰ",
    "έ": "έ",
    "ὲ": "ὲ",
    "ή": "ή",
    "ὴ": "ὴ",
    "ί": "ί",
    "ὶ": "ὶ",
    "ϊ": "ϊ",
    "ΐ": "ΐ",
    "ῒ": "ῒ",
    "ῑ": "ῑ",
    "ῐ": "ῐ",
    "ό": "ό",
    "ὸ": "ὸ",
    "ύ": "ύ",
    "ὺ": "ὺ",
    "ϋ": "ϋ",
    "ΰ": "ΰ",
    "ῢ": "ῢ",
    "ῡ": "ῡ",
    "ῠ": "ῠ",
    "ώ": "ώ",
    "ὼ": "ὼ",
    "Ύ": "Ύ",
    "Ὺ": "Ὺ",
    "Ϋ": "Ϋ",
    "Ῡ": "Ῡ",
    "Ῠ": "Ῠ",
    "Ώ": "Ώ",
    "Ὼ": "Ὼ"
  };

  /**
   * This file contains the parser used to parse out a TeX expression from the
   * input. Since TeX isn't context-free, standard parsers don't work particularly
   * well.
   *
   * The strategy of this parser is as such:
   *
   * The main functions (the `.parse...` ones) take a position in the current
   * parse string to parse tokens from. The lexer (found in Lexer.js, stored at
   * this.gullet.lexer) also supports pulling out tokens at arbitrary places. When
   * individual tokens are needed at a position, the lexer is called to pull out a
   * token, which is then used.
   *
   * The parser has a property called "mode" indicating the mode that
   * the parser is currently in. Currently it has to be one of "math" or
   * "text", which denotes whether the current environment is a math-y
   * one or a text-y one (e.g. inside \text). Currently, this serves to
   * limit the functions which can be used in text mode.
   *
   * The main functions then return an object which contains the useful data that
   * was parsed at its given point, and a new position at the end of the parsed
   * data. The main functions can call each other and continue the parsing by
   * using the returned position as a new starting point.
   *
   * There are also extra `.handle...` functions, which pull out some reused
   * functionality into self-contained functions.
   *
   * The functions return ParseNodes.
   */
  class Parser$2 {
    constructor(input, settings) {
      this.mode = void 0;
      this.gullet = void 0;
      this.settings = void 0;
      this.leftrightDepth = void 0;
      this.nextToken = void 0;
      // Start in math mode
      this.mode = "math"; // Create a new macro expander (gullet) and (indirectly via that) also a
      // new lexer (mouth) for this parser (stomach, in the language of TeX)

      this.gullet = new MacroExpander(input, settings, this.mode); // Store the settings for use in parsing

      this.settings = settings; // Count leftright depth (for \middle errors)

      this.leftrightDepth = 0;
    }
    /**
     * Checks a result to make sure it has the right type, and throws an
     * appropriate error otherwise.
     */


    expect(text, consume) {
      if (consume === void 0) {
        consume = true;
      }

      if (this.fetch().text !== text) {
        throw new ParseError("Expected '" + text + "', got '" + this.fetch().text + "'", this.fetch());
      }

      if (consume) {
        this.consume();
      }
    }
    /**
     * Discards the current lookahead token, considering it consumed.
     */


    consume() {
      this.nextToken = null;
    }
    /**
     * Return the current lookahead token, or if there isn't one (at the
     * beginning, or if the previous lookahead token was consume()d),
     * fetch the next token as the new lookahead token and return it.
     */


    fetch() {
      if (this.nextToken == null) {
        this.nextToken = this.gullet.expandNextToken();
      }

      return this.nextToken;
    }
    /**
     * Switches between "text" and "math" modes.
     */


    switchMode(newMode) {
      this.mode = newMode;
      this.gullet.switchMode(newMode);
    }
    /**
     * Main parsing function, which parses an entire input.
     */


    parse() {
      if (!this.settings.globalGroup) {
        // Create a group namespace for the math expression.
        // (LaTeX creates a new group for every $...$, $$...$$, \[...\].)
        this.gullet.beginGroup();
      } // Use old \color behavior (same as LaTeX's \textcolor) if requested.
      // We do this within the group for the math expression, so it doesn't
      // pollute settings.macros.


      if (this.settings.colorIsTextColor) {
        this.gullet.macros.set("\\color", "\\textcolor");
      }

      try {
        // Try to parse the input
        var parse = this.parseExpression(false); // If we succeeded, make sure there's an EOF at the end

        this.expect("EOF"); // End the group namespace for the expression

        if (!this.settings.globalGroup) {
          this.gullet.endGroup();
        }

        return parse; // Close any leftover groups in case of a parse error.
      } finally {
        this.gullet.endGroups();
      }
    }
    /**
     * Fully parse a separate sequence of tokens as a separate job.
     * Tokens should be specified in reverse order, as in a MacroDefinition.
     */


    subparse(tokens) {
      // Save the next token from the current job.
      var oldToken = this.nextToken;
      this.consume(); // Run the new job, terminating it with an excess '}'

      this.gullet.pushToken(new Token("}"));
      this.gullet.pushTokens(tokens);
      var parse = this.parseExpression(false);
      this.expect("}"); // Restore the next token from the current job.

      this.nextToken = oldToken;
      return parse;
    }

    /**
     * Parses an "expression", which is a list of atoms.
     *
     * `breakOnInfix`: Should the parsing stop when we hit infix nodes? This
     *                 happens when functions have higher precendence han infix
     *                 nodes in implicit parses.
     *
     * `breakOnTokenText`: The text of the token that the expression should end
     *                     with, or `null` if something else should end the
     *                     expression.
     */
    parseExpression(breakOnInfix, breakOnTokenText) {
      var body = []; // Keep adding atoms to the body until we can't parse any more atoms (either
      // we reached the end, a }, or a \right)

      while (true) {
        // Ignore spaces in math mode
        if (this.mode === "math") {
          this.consumeSpaces();
        }

        var lex = this.fetch();

        if (Parser$2.endOfExpression.indexOf(lex.text) !== -1) {
          break;
        }

        if (breakOnTokenText && lex.text === breakOnTokenText) {
          break;
        }

        if (breakOnInfix && functions[lex.text] && functions[lex.text].infix) {
          break;
        }

        var atom = this.parseAtom(breakOnTokenText);

        if (!atom) {
          break;
        } else if (atom.type === "internal") {
          continue;
        }

        body.push(atom);
      }

      if (this.mode === "text") {
        this.formLigatures(body);
      }

      return this.handleInfixNodes(body);
    }
    /**
     * Rewrites infix operators such as \over with corresponding commands such
     * as \frac.
     *
     * There can only be one infix operator per group.  If there's more than one
     * then the expression is ambiguous.  This can be resolved by adding {}.
     */


    handleInfixNodes(body) {
      var overIndex = -1;
      var funcName;

      for (var i = 0; i < body.length; i++) {
        if (body[i].type === "infix") {
          if (overIndex !== -1) {
            throw new ParseError("only one infix operator per group", body[i].token);
          }

          overIndex = i;
          funcName = body[i].replaceWith;
        }
      }

      if (overIndex !== -1 && funcName) {
        var numerNode;
        var denomNode;
        var numerBody = body.slice(0, overIndex);
        var denomBody = body.slice(overIndex + 1);

        if (numerBody.length === 1 && numerBody[0].type === "ordgroup") {
          numerNode = numerBody[0];
        } else {
          numerNode = {
            type: "ordgroup",
            mode: this.mode,
            body: numerBody
          };
        }

        if (denomBody.length === 1 && denomBody[0].type === "ordgroup") {
          denomNode = denomBody[0];
        } else {
          denomNode = {
            type: "ordgroup",
            mode: this.mode,
            body: denomBody
          };
        }

        var node;

        if (funcName === "\\\\abovefrac") {
          node = this.callFunction(funcName, [numerNode, body[overIndex], denomNode], []);
        } else {
          node = this.callFunction(funcName, [numerNode, denomNode], []);
        }

        return [node];
      } else {
        return body;
      }
    }
    /**
     * Handle a subscript or superscript with nice errors.
     */


    handleSupSubscript(name // For error reporting.
    ) {
      var symbolToken = this.fetch();
      var symbol = symbolToken.text;
      this.consume();
      this.consumeSpaces(); // ignore spaces before sup/subscript argument

      var group = this.parseGroup(name);

      if (!group) {
        throw new ParseError("Expected group after '" + symbol + "'", symbolToken);
      }

      return group;
    }
    /**
     * Converts the textual input of an unsupported command into a text node
     * contained within a color node whose color is determined by errorColor
     */


    formatUnsupportedCmd(text) {
      var textordArray = [];

      for (var i = 0; i < text.length; i++) {
        textordArray.push({
          type: "textord",
          mode: "text",
          text: text[i]
        });
      }

      var textNode = {
        type: "text",
        mode: this.mode,
        body: textordArray
      };
      var colorNode = {
        type: "color",
        mode: this.mode,
        color: this.settings.errorColor,
        body: [textNode]
      };
      return colorNode;
    }
    /**
     * Parses a group with optional super/subscripts.
     */


    parseAtom(breakOnTokenText) {
      // The body of an atom is an implicit group, so that things like
      // \left(x\right)^2 work correctly.
      var base = this.parseGroup("atom", breakOnTokenText); // In text mode, we don't have superscripts or subscripts

      if (this.mode === "text") {
        return base;
      } // Note that base may be empty (i.e. null) at this point.


      var superscript;
      var subscript;

      while (true) {
        // Guaranteed in math mode, so eat any spaces first.
        this.consumeSpaces(); // Lex the first token

        var lex = this.fetch();

        if (lex.text === "\\limits" || lex.text === "\\nolimits") {
          // We got a limit control
          if (base && base.type === "op") {
            var limits = lex.text === "\\limits";
            base.limits = limits;
            base.alwaysHandleSupSub = true;
          } else if (base && base.type === "operatorname") {
            if (base.alwaysHandleSupSub) {
              base.limits = lex.text === "\\limits";
            }
          } else {
            throw new ParseError("Limit controls must follow a math operator", lex);
          }

          this.consume();
        } else if (lex.text === "^") {
          // We got a superscript start
          if (superscript) {
            throw new ParseError("Double superscript", lex);
          }

          superscript = this.handleSupSubscript("superscript");
        } else if (lex.text === "_") {
          // We got a subscript start
          if (subscript) {
            throw new ParseError("Double subscript", lex);
          }

          subscript = this.handleSupSubscript("subscript");
        } else if (lex.text === "'") {
          // We got a prime
          if (superscript) {
            throw new ParseError("Double superscript", lex);
          }

          var prime = {
            type: "textord",
            mode: this.mode,
            text: "\\prime"
          }; // Many primes can be grouped together, so we handle this here

          var primes = [prime];
          this.consume(); // Keep lexing tokens until we get something that's not a prime

          while (this.fetch().text === "'") {
            // For each one, add another prime to the list
            primes.push(prime);
            this.consume();
          } // If there's a superscript following the primes, combine that
          // superscript in with the primes.


          if (this.fetch().text === "^") {
            primes.push(this.handleSupSubscript("superscript"));
          } // Put everything into an ordgroup as the superscript


          superscript = {
            type: "ordgroup",
            mode: this.mode,
            body: primes
          };
        } else {
          // If it wasn't ^, _, or ', stop parsing super/subscripts
          break;
        }
      } // Base must be set if superscript or subscript are set per logic above,
      // but need to check here for type check to pass.


      if (superscript || subscript) {
        // If we got either a superscript or subscript, create a supsub
        return {
          type: "supsub",
          mode: this.mode,
          base: base,
          sup: superscript,
          sub: subscript
        };
      } else {
        // Otherwise return the original body
        return base;
      }
    }
    /**
     * Parses an entire function, including its base and all of its arguments.
     */


    parseFunction(breakOnTokenText, name // For determining its context
    ) {
      var token = this.fetch();
      var func = token.text;
      var funcData = functions[func];

      if (!funcData) {
        return null;
      }

      this.consume(); // consume command token

      if (name && name !== "atom" && !funcData.allowedInArgument) {
        throw new ParseError("Got function '" + func + "' with no arguments" + (name ? " as " + name : ""), token);
      } else if (this.mode === "text" && !funcData.allowedInText) {
        throw new ParseError("Can't use function '" + func + "' in text mode", token);
      } else if (this.mode === "math" && funcData.allowedInMath === false) {
        throw new ParseError("Can't use function '" + func + "' in math mode", token);
      }

      var {
        args,
        optArgs
      } = this.parseArguments(func, funcData);
      return this.callFunction(func, args, optArgs, token, breakOnTokenText);
    }
    /**
     * Call a function handler with a suitable context and arguments.
     */


    callFunction(name, args, optArgs, token, breakOnTokenText) {
      var context = {
        funcName: name,
        parser: this,
        token,
        breakOnTokenText
      };
      var func = functions[name];

      if (func && func.handler) {
        return func.handler(context, args, optArgs);
      } else {
        throw new ParseError("No function handler for " + name);
      }
    }
    /**
     * Parses the arguments of a function or environment
     */


    parseArguments(func, // Should look like "\name" or "\begin{name}".
    funcData) {
      var totalArgs = funcData.numArgs + funcData.numOptionalArgs;

      if (totalArgs === 0) {
        return {
          args: [],
          optArgs: []
        };
      }

      var args = [];
      var optArgs = [];

      for (var i = 0; i < totalArgs; i++) {
        var argType = funcData.argTypes && funcData.argTypes[i];
        var isOptional = i < funcData.numOptionalArgs;

        if (funcData.primitive && argType == null || // \sqrt expands into primitive if optional argument doesn't exist
        funcData.type === "sqrt" && i === 1 && optArgs[0] == null) {
          argType = "primitive";
        }

        var arg = this.parseGroupOfType("argument to '" + func + "'", argType, isOptional);

        if (isOptional) {
          optArgs.push(arg);
        } else if (arg != null) {
          args.push(arg);
        } else {
          // should be unreachable
          throw new ParseError("Null argument, please report this as a bug");
        }
      }

      return {
        args,
        optArgs
      };
    }
    /**
     * Parses a group when the mode is changing.
     */


    parseGroupOfType(name, type, optional) {
      switch (type) {
        case "color":
          return this.parseColorGroup(optional);

        case "size":
          return this.parseSizeGroup(optional);

        case "url":
          return this.parseUrlGroup(optional);

        case "math":
        case "text":
          return this.parseArgumentGroup(optional, type);

        case "hbox":
          {
            // hbox argument type wraps the argument in the equivalent of
            // \hbox, which is like \text but switching to \textstyle size.
            var group = this.parseArgumentGroup(optional, "text");
            return group != null ? {
              type: "styling",
              mode: group.mode,
              body: [group],
              style: "text" // simulate \textstyle

            } : null;
          }

        case "raw":
          {
            var token = this.parseStringGroup("raw", optional);
            return token != null ? {
              type: "raw",
              mode: "text",
              string: token.text
            } : null;
          }

        case "primitive":
          {
            if (optional) {
              throw new ParseError("A primitive argument cannot be optional");
            }

            var _group = this.parseGroup(name);

            if (_group == null) {
              throw new ParseError("Expected group as " + name, this.fetch());
            }

            return _group;
          }

        case "original":
        case null:
        case undefined:
          return this.parseArgumentGroup(optional);

        default:
          throw new ParseError("Unknown group type as " + name, this.fetch());
      }
    }
    /**
     * Discard any space tokens, fetching the next non-space token.
     */


    consumeSpaces() {
      while (this.fetch().text === " ") {
        this.consume();
      }
    }
    /**
     * Parses a group, essentially returning the string formed by the
     * brace-enclosed tokens plus some position information.
     */


    parseStringGroup(modeName, // Used to describe the mode in error messages.
    optional) {
      var argToken = this.gullet.scanArgument(optional);

      if (argToken == null) {
        return null;
      }

      var str = "";
      var nextToken;

      while ((nextToken = this.fetch()).text !== "EOF") {
        str += nextToken.text;
        this.consume();
      }

      this.consume(); // consume the end of the argument

      argToken.text = str;
      return argToken;
    }
    /**
     * Parses a regex-delimited group: the largest sequence of tokens
     * whose concatenated strings match `regex`. Returns the string
     * formed by the tokens plus some position information.
     */


    parseRegexGroup(regex, modeName // Used to describe the mode in error messages.
    ) {
      var firstToken = this.fetch();
      var lastToken = firstToken;
      var str = "";
      var nextToken;

      while ((nextToken = this.fetch()).text !== "EOF" && regex.test(str + nextToken.text)) {
        lastToken = nextToken;
        str += lastToken.text;
        this.consume();
      }

      if (str === "") {
        throw new ParseError("Invalid " + modeName + ": '" + firstToken.text + "'", firstToken);
      }

      return firstToken.range(lastToken, str);
    }
    /**
     * Parses a color description.
     */


    parseColorGroup(optional) {
      var res = this.parseStringGroup("color", optional);

      if (res == null) {
        return null;
      }

      var match = /^(#[a-f0-9]{3}|#?[a-f0-9]{6}|[a-z]+)$/i.exec(res.text);

      if (!match) {
        throw new ParseError("Invalid color: '" + res.text + "'", res);
      }

      var color = match[0];

      if (/^[0-9a-f]{6}$/i.test(color)) {
        // We allow a 6-digit HTML color spec without a leading "#".
        // This follows the xcolor package's HTML color model.
        // Predefined color names are all missed by this RegEx pattern.
        color = "#" + color;
      }

      return {
        type: "color-token",
        mode: this.mode,
        color
      };
    }
    /**
     * Parses a size specification, consisting of magnitude and unit.
     */


    parseSizeGroup(optional) {
      var res;
      var isBlank = false; // don't expand before parseStringGroup

      this.gullet.consumeSpaces();

      if (!optional && this.gullet.future().text !== "{") {
        res = this.parseRegexGroup(/^[-+]? *(?:$|\d+|\d+\.\d*|\.\d*) *[a-z]{0,2} *$/, "size");
      } else {
        res = this.parseStringGroup("size", optional);
      }

      if (!res) {
        return null;
      }

      if (!optional && res.text.length === 0) {
        // Because we've tested for what is !optional, this block won't
        // affect \kern, \hspace, etc. It will capture the mandatory arguments
        // to \genfrac and \above.
        res.text = "0pt"; // Enable \above{}

        isBlank = true; // This is here specifically for \genfrac
      }

      var match = /([-+]?) *(\d+(?:\.\d*)?|\.\d+) *([a-z]{2})/.exec(res.text);

      if (!match) {
        throw new ParseError("Invalid size: '" + res.text + "'", res);
      }

      var data = {
        number: +(match[1] + match[2]),
        // sign + magnitude, cast to number
        unit: match[3]
      };

      if (!validUnit(data)) {
        throw new ParseError("Invalid unit: '" + data.unit + "'", res);
      }

      return {
        type: "size",
        mode: this.mode,
        value: data,
        isBlank
      };
    }
    /**
     * Parses an URL, checking escaped letters and allowed protocols,
     * and setting the catcode of % as an active character (as in \hyperref).
     */


    parseUrlGroup(optional) {
      this.gullet.lexer.setCatcode("%", 13); // active character

      this.gullet.lexer.setCatcode("~", 12); // other character

      var res = this.parseStringGroup("url", optional);
      this.gullet.lexer.setCatcode("%", 14); // comment character

      this.gullet.lexer.setCatcode("~", 13); // active character

      if (res == null) {
        return null;
      } // hyperref package allows backslashes alone in href, but doesn't
      // generate valid links in such cases; we interpret this as
      // "undefined" behaviour, and keep them as-is. Some browser will
      // replace backslashes with forward slashes.


      var url = res.text.replace(/\\([#$%&~_^{}])/g, '$1');
      return {
        type: "url",
        mode: this.mode,
        url
      };
    }
    /**
     * Parses an argument with the mode specified.
     */


    parseArgumentGroup(optional, mode) {
      var argToken = this.gullet.scanArgument(optional);

      if (argToken == null) {
        return null;
      }

      var outerMode = this.mode;

      if (mode) {
        // Switch to specified mode
        this.switchMode(mode);
      }

      this.gullet.beginGroup();
      var expression = this.parseExpression(false, "EOF"); // TODO: find an alternative way to denote the end

      this.expect("EOF"); // expect the end of the argument

      this.gullet.endGroup();
      var result = {
        type: "ordgroup",
        mode: this.mode,
        loc: argToken.loc,
        body: expression
      };

      if (mode) {
        // Switch mode back
        this.switchMode(outerMode);
      }

      return result;
    }
    /**
     * Parses an ordinary group, which is either a single nucleus (like "x")
     * or an expression in braces (like "{x+y}") or an implicit group, a group
     * that starts at the current position, and ends right before a higher explicit
     * group ends, or at EOF.
     */


    parseGroup(name, // For error reporting.
    breakOnTokenText) {
      var firstToken = this.fetch();
      var text = firstToken.text;
      var result; // Try to parse an open brace or \begingroup

      if (text === "{" || text === "\\begingroup") {
        this.consume();
        var groupEnd = text === "{" ? "}" : "\\endgroup";
        this.gullet.beginGroup(); // If we get a brace, parse an expression

        var expression = this.parseExpression(false, groupEnd);
        var lastToken = this.fetch();
        this.expect(groupEnd); // Check that we got a matching closing brace

        this.gullet.endGroup();
        result = {
          type: "ordgroup",
          mode: this.mode,
          loc: SourceLocation.range(firstToken, lastToken),
          body: expression,
          // A group formed by \begingroup...\endgroup is a semi-simple group
          // which doesn't affect spacing in math mode, i.e., is transparent.
          // https://tex.stackexchange.com/questions/1930/when-should-one-
          // use-begingroup-instead-of-bgroup
          semisimple: text === "\\begingroup" || undefined
        };
      } else {
        // If there exists a function with this name, parse the function.
        // Otherwise, just return a nucleus
        result = this.parseFunction(breakOnTokenText, name) || this.parseSymbol();

        if (result == null && text[0] === "\\" && !implicitCommands.hasOwnProperty(text)) {
          if (this.settings.throwOnError) {
            throw new ParseError("Undefined control sequence: " + text, firstToken);
          }

          result = this.formatUnsupportedCmd(text);
          this.consume();
        }
      }

      return result;
    }
    /**
     * Form ligature-like combinations of characters for text mode.
     * This includes inputs like "--", "---", "``" and "''".
     * The result will simply replace multiple textord nodes with a single
     * character in each value by a single textord node having multiple
     * characters in its value.  The representation is still ASCII source.
     * The group will be modified in place.
     */


    formLigatures(group) {
      var n = group.length - 1;

      for (var i = 0; i < n; ++i) {
        var a = group[i]; // $FlowFixMe: Not every node type has a `text` property.

        var v = a.text;

        if (v === "-" && group[i + 1].text === "-") {
          if (i + 1 < n && group[i + 2].text === "-") {
            group.splice(i, 3, {
              type: "textord",
              mode: "text",
              loc: SourceLocation.range(a, group[i + 2]),
              text: "---"
            });
            n -= 2;
          } else {
            group.splice(i, 2, {
              type: "textord",
              mode: "text",
              loc: SourceLocation.range(a, group[i + 1]),
              text: "--"
            });
            n -= 1;
          }
        }

        if ((v === "'" || v === "`") && group[i + 1].text === v) {
          group.splice(i, 2, {
            type: "textord",
            mode: "text",
            loc: SourceLocation.range(a, group[i + 1]),
            text: v + v
          });
          n -= 1;
        }
      }
    }
    /**
     * Parse a single symbol out of the string. Here, we handle single character
     * symbols and special functions like \verb.
     */


    parseSymbol() {
      var nucleus = this.fetch();
      var text = nucleus.text;

      if (/^\\verb[^a-zA-Z]/.test(text)) {
        this.consume();
        var arg = text.slice(5);
        var star = arg.charAt(0) === "*";

        if (star) {
          arg = arg.slice(1);
        } // Lexer's tokenRegex is constructed to always have matching
        // first/last characters.


        if (arg.length < 2 || arg.charAt(0) !== arg.slice(-1)) {
          throw new ParseError("\\verb assertion failed --\n                    please report what input caused this bug");
        }

        arg = arg.slice(1, -1); // remove first and last char

        return {
          type: "verb",
          mode: "text",
          body: arg,
          star
        };
      } // At this point, we should have a symbol, possibly with accents.
      // First expand any accented base symbol according to unicodeSymbols.


      if (unicodeSymbols.hasOwnProperty(text[0]) && !symbols[this.mode][text[0]]) {
        // This behavior is not strict (XeTeX-compatible) in math mode.
        if (this.settings.strict && this.mode === "math") {
          this.settings.reportNonstrict("unicodeTextInMathMode", "Accented Unicode text character \"" + text[0] + "\" used in " + "math mode", nucleus);
        }

        text = unicodeSymbols[text[0]] + text.substr(1);
      } // Strip off any combining characters


      var match = combiningDiacriticalMarksEndRegex.exec(text);

      if (match) {
        text = text.substring(0, match.index);

        if (text === 'i') {
          text = '\u0131'; // dotless i, in math and text mode
        } else if (text === 'j') {
          text = '\u0237'; // dotless j, in math and text mode
        }
      } // Recognize base symbol


      var symbol;

      if (symbols[this.mode][text]) {
        if (this.settings.strict && this.mode === 'math' && extraLatin.indexOf(text) >= 0) {
          this.settings.reportNonstrict("unicodeTextInMathMode", "Latin-1/Unicode text character \"" + text[0] + "\" used in " + "math mode", nucleus);
        }

        var group = symbols[this.mode][text].group;
        var loc = SourceLocation.range(nucleus);
        var s;

        if (ATOMS.hasOwnProperty(group)) {
          // $FlowFixMe
          var family = group;
          s = {
            type: "atom",
            mode: this.mode,
            family,
            loc,
            text
          };
        } else {
          // $FlowFixMe
          s = {
            type: group,
            mode: this.mode,
            loc,
            text
          };
        } // $FlowFixMe


        symbol = s;
      } else if (text.charCodeAt(0) >= 0x80) {
        // no symbol for e.g. ^
        if (this.settings.strict) {
          if (!supportedCodepoint(text.charCodeAt(0))) {
            this.settings.reportNonstrict("unknownSymbol", "Unrecognized Unicode character \"" + text[0] + "\"" + (" (" + text.charCodeAt(0) + ")"), nucleus);
          } else if (this.mode === "math") {
            this.settings.reportNonstrict("unicodeTextInMathMode", "Unicode text character \"" + text[0] + "\" used in math mode", nucleus);
          }
        } // All nonmathematical Unicode characters are rendered as if they
        // are in text mode (wrapped in \text) because that's what it
        // takes to render them in LaTeX.  Setting `mode: this.mode` is
        // another natural choice (the user requested math mode), but
        // this makes it more difficult for getCharacterMetrics() to
        // distinguish Unicode characters without metrics and those for
        // which we want to simulate the letter M.


        symbol = {
          type: "textord",
          mode: "text",
          loc: SourceLocation.range(nucleus),
          text
        };
      } else {
        return null; // EOF, ^, _, {, }, etc.
      }

      this.consume(); // Transform combining characters into accents

      if (match) {
        for (var i = 0; i < match[0].length; i++) {
          var accent = match[0][i];

          if (!unicodeAccents[accent]) {
            throw new ParseError("Unknown accent ' " + accent + "'", nucleus);
          }

          var command = unicodeAccents[accent][this.mode] || unicodeAccents[accent].text;

          if (!command) {
            throw new ParseError("Accent " + accent + " unsupported in " + this.mode + " mode", nucleus);
          }

          symbol = {
            type: "accent",
            mode: this.mode,
            loc: SourceLocation.range(nucleus),
            label: command,
            isStretchy: false,
            isShifty: true,
            // $FlowFixMe
            base: symbol
          };
        }
      } // $FlowFixMe


      return symbol;
    }

  }
  Parser$2.endOfExpression = ["}", "\\endgroup", "\\end", "\\right", "&"];

  /**
   * Provides a single function for parsing an expression using a Parser
   * TODO(emily): Remove this
   */

  /**
   * Parses an expression using a Parser, then returns the parsed result.
   */
  var parseTree = function parseTree(toParse, settings) {
    if (!(typeof toParse === 'string' || toParse instanceof String)) {
      throw new TypeError('KaTeX can only parse string typed expression');
    }

    var parser = new Parser$2(toParse, settings); // Blank out any \df@tag to avoid spurious "Duplicate \tag" errors

    delete parser.gullet.macros.current["\\df@tag"];
    var tree = parser.parse(); // Prevent a color definition from persisting between calls to katex.render().

    delete parser.gullet.macros.current["\\current@color"];
    delete parser.gullet.macros.current["\\color"]; // If the input used \tag, it will set the \df@tag macro to the tag.
    // In this case, we separately parse the tag and wrap the tree.

    if (parser.gullet.macros.get("\\df@tag")) {
      if (!settings.displayMode) {
        throw new ParseError("\\tag works only in display equations");
      }

      tree = [{
        type: "tag",
        mode: "text",
        body: tree,
        tag: parser.subparse([new Token("\\df@tag")])
      }];
    }

    return tree;
  };

  /* eslint no-console:0 */

  /**
   * Parse and build an expression, and place that expression in the DOM node
   * given.
   */
  var render$1 = function render(expression, baseNode, options) {
    baseNode.textContent = "";
    var node = renderToDomTree(expression, options).toNode();
    baseNode.appendChild(node);
  }; // KaTeX's styles don't work properly in quirks mode. Print out an error, and
  // disable rendering.


  if (typeof document !== "undefined") {
    if (document.compatMode !== "CSS1Compat") {
      typeof console !== "undefined" && console.warn("Warning: KaTeX doesn't work in quirks mode. Make sure your " + "website has a suitable doctype.");

      render$1 = function render() {
        throw new ParseError("KaTeX doesn't work in quirks mode.");
      };
    }
  }
  /**
   * Parse and build an expression, and return the markup for that.
   */


  var renderToString = function renderToString(expression, options) {
    var markup = renderToDomTree(expression, options).toMarkup();
    return markup;
  };
  /**
   * Parse an expression and return the parse tree.
   */


  var generateParseTree = function generateParseTree(expression, options) {
    var settings = new Settings(options);
    return parseTree(expression, settings);
  };
  /**
   * If the given error is a KaTeX ParseError and options.throwOnError is false,
   * renders the invalid LaTeX as a span with hover title giving the KaTeX
   * error message.  Otherwise, simply throws the error.
   */


  var renderError = function renderError(error, expression, options) {
    if (options.throwOnError || !(error instanceof ParseError)) {
      throw error;
    }

    var node = buildCommon.makeSpan(["katex-error"], [new SymbolNode(expression)]);
    node.setAttribute("title", error.toString());
    node.setAttribute("style", "color:" + options.errorColor);
    return node;
  };
  /**
   * Generates and returns the katex build tree. This is used for advanced
   * use cases (like rendering to custom output).
   */


  var renderToDomTree = function renderToDomTree(expression, options) {
    var settings = new Settings(options);

    try {
      var tree = parseTree(expression, settings);
      return buildTree(tree, expression, settings);
    } catch (error) {
      return renderError(error, expression, settings);
    }
  };
  /**
   * Generates and returns the katex build tree, with just HTML (no MathML).
   * This is used for advanced use cases (like rendering to custom output).
   */


  var renderToHTMLTree = function renderToHTMLTree(expression, options) {
    var settings = new Settings(options);

    try {
      var tree = parseTree(expression, settings);
      return buildHTMLTree(tree, expression, settings);
    } catch (error) {
      return renderError(error, expression, settings);
    }
  };

  var katex = {
    /**
     * Current KaTeX version
     */
    version: "0.15.2",

    /**
     * Renders the given LaTeX into an HTML+MathML combination, and adds
     * it as a child to the specified DOM node.
     */
    render: render$1,

    /**
     * Renders the given LaTeX into an HTML+MathML combination string,
     * for sending to the client.
     */
    renderToString,

    /**
     * KaTeX error, usually during parsing.
     */
    ParseError,

    /**
     * The shema of Settings
     */
    SETTINGS_SCHEMA,

    /**
     * Parses the given LaTeX into KaTeX's internal parse tree structure,
     * without rendering to HTML or MathML.
     *
     * NOTE: This method is not currently recommended for public use.
     * The internal tree representation is unstable and is very likely
     * to change. Use at your own risk.
     */
    __parse: generateParseTree,

    /**
     * Renders the given LaTeX into an HTML+MathML internal DOM tree
     * representation, without flattening that representation to a string.
     *
     * NOTE: This method is not currently recommended for public use.
     * The internal tree representation is unstable and is very likely
     * to change. Use at your own risk.
     */
    __renderToDomTree: renderToDomTree,

    /**
     * Renders the given LaTeX into an HTML internal DOM tree representation,
     * without MathML and without flattening that representation to a string.
     *
     * NOTE: This method is not currently recommended for public use.
     * The internal tree representation is unstable and is very likely
     * to change. Use at your own risk.
     */
    __renderToHTMLTree: renderToHTMLTree,

    /**
     * extends internal font metrics object with a new object
     * each key in the new object represents a font name
    */
    __setFontMetrics: setFontMetrics,

    /**
     * adds a new symbol to builtin symbols table
     */
    __defineSymbol: defineSymbol,

    /**
     * adds a new macro to builtin macro list
     */
    __defineMacro: defineMacro,

    /**
     * Expose the dom tree node types, which can be useful for type checking nodes.
     *
     * NOTE: This method is not currently recommended for public use.
     * The internal tree representation is unstable and is very likely
     * to change. Use at your own risk.
     */
    __domTree: {
      Span,
      Anchor,
      SymbolNode,
      SvgNode,
      PathNode,
      LineNode
    }
  };

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   *
   * @typedef {string} Type
   * @typedef {Object<string, unknown>} Props
   *
   * @typedef {null|undefined|Type|Props|TestFunctionAnything|Array.<Type|Props|TestFunctionAnything>} Test
   */

  const convert =
    /**
     * @type {(
     *   (<T extends Node>(test: T['type']|Partial<T>|TestFunctionPredicate<T>) => AssertPredicate<T>) &
     *   ((test?: Test) => AssertAnything)
     * )}
     */
    (
      /**
       * Generate an assertion from a check.
       * @param {Test} [test]
       * When nullish, checks if `node` is a `Node`.
       * When `string`, works like passing `function (node) {return node.type === test}`.
       * When `function` checks if function passed the node is true.
       * When `object`, checks that all keys in test are in node, and that they have (strictly) equal values.
       * When `array`, checks any one of the subtests pass.
       * @returns {AssertAnything}
       */
      function (test) {
        if (test === undefined || test === null) {
          return ok
        }

        if (typeof test === 'string') {
          return typeFactory(test)
        }

        if (typeof test === 'object') {
          return Array.isArray(test) ? anyFactory$3(test) : propsFactory(test)
        }

        if (typeof test === 'function') {
          return castFactory$3(test)
        }

        throw new Error('Expected function, string, or object as test')
      }
    );
  /**
   * @param {Array.<Type|Props|TestFunctionAnything>} tests
   * @returns {AssertAnything}
   */
  function anyFactory$3(tests) {
    /** @type {Array.<AssertAnything>} */
    const checks = [];
    let index = -1;

    while (++index < tests.length) {
      checks[index] = convert(tests[index]);
    }

    return castFactory$3(any)

    /**
     * @this {unknown}
     * @param {unknown[]} parameters
     * @returns {boolean}
     */
    function any(...parameters) {
      let index = -1;

      while (++index < checks.length) {
        if (checks[index].call(this, ...parameters)) return true
      }

      return false
    }
  }

  /**
   * Utility to assert each property in `test` is represented in `node`, and each
   * values are strictly equal.
   *
   * @param {Props} check
   * @returns {AssertAnything}
   */
  function propsFactory(check) {
    return castFactory$3(all)

    /**
     * @param {Node} node
     * @returns {boolean}
     */
    function all(node) {
      /** @type {string} */
      let key;

      for (key in check) {
        // @ts-expect-error: hush, it sure works as an index.
        if (node[key] !== check[key]) return false
      }

      return true
    }
  }

  /**
   * Utility to convert a string into a function which checks a given node’s type
   * for said string.
   *
   * @param {Type} check
   * @returns {AssertAnything}
   */
  function typeFactory(check) {
    return castFactory$3(type)

    /**
     * @param {Node} node
     */
    function type(node) {
      return node && node.type === check
    }
  }

  /**
   * Utility to convert a string into a function which checks a given node’s type
   * for said string.
   * @param {TestFunctionAnything} check
   * @returns {AssertAnything}
   */
  function castFactory$3(check) {
    return assertion

    /**
     * @this {unknown}
     * @param {Array.<unknown>} parameters
     * @returns {boolean}
     */
    function assertion(...parameters) {
      // @ts-expect-error: spreading is fine.
      return Boolean(check.call(this, ...parameters))
    }
  }

  // Utility to return true.
  function ok() {
    return true
  }

  /**
   * @param {string} d
   * @returns {string}
   */
  function color(d) {
    return d
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   * @typedef {import('unist-util-is').Test} Test
   * @typedef {import('./complex-types').Action} Action
   * @typedef {import('./complex-types').Index} Index
   * @typedef {import('./complex-types').ActionTuple} ActionTuple
   * @typedef {import('./complex-types').VisitorResult} VisitorResult
   * @typedef {import('./complex-types').Visitor} Visitor
   */

  /**
   * Continue traversing as normal
   */
  const CONTINUE = true;
  /**
   * Do not traverse this node’s children
   */
  const SKIP = 'skip';
  /**
   * Stop traversing immediately
   */
  const EXIT = false;

  /**
   * Visit children of tree which pass a test
   *
   * @param tree Abstract syntax tree to walk
   * @param test Test node, optional
   * @param visitor Function to run for each node
   * @param reverse Visit the tree in reverse order, defaults to false
   */
  const visitParents =
    /**
     * @type {(
     *   (<Tree extends Node, Check extends Test>(tree: Tree, test: Check, visitor: import('./complex-types').BuildVisitor<Tree, Check>, reverse?: boolean) => void) &
     *   (<Tree extends Node>(tree: Tree, visitor: import('./complex-types').BuildVisitor<Tree>, reverse?: boolean) => void)
     * )}
     */
    (
      /**
       * @param {Node} tree
       * @param {Test} test
       * @param {import('./complex-types').Visitor<Node>} visitor
       * @param {boolean} [reverse]
       */
      function (tree, test, visitor, reverse) {
        if (typeof test === 'function' && typeof visitor !== 'function') {
          reverse = visitor;
          // @ts-expect-error no visitor given, so `visitor` is test.
          visitor = test;
          test = null;
        }

        const is = convert(test);
        const step = reverse ? -1 : 1;

        factory(tree, null, [])();

        /**
         * @param {Node} node
         * @param {number?} index
         * @param {Array.<Parent>} parents
         */
        function factory(node, index, parents) {
          /** @type {Object.<string, unknown>} */
          // @ts-expect-error: hush
          const value = typeof node === 'object' && node !== null ? node : {};
          /** @type {string|undefined} */
          let name;

          if (typeof value.type === 'string') {
            name =
              typeof value.tagName === 'string'
                ? value.tagName
                : typeof value.name === 'string'
                ? value.name
                : undefined;

            Object.defineProperty(visit, 'name', {
              value:
                'node (' +
                color(value.type + (name ? '<' + name + '>' : '')) +
                ')'
            });
          }

          return visit

          function visit() {
            /** @type {ActionTuple} */
            let result = [];
            /** @type {ActionTuple} */
            let subresult;
            /** @type {number} */
            let offset;
            /** @type {Array.<Parent>} */
            let grandparents;

            if (!test || is(node, index, parents[parents.length - 1] || null)) {
              result = toResult(visitor(node, parents));

              if (result[0] === EXIT) {
                return result
              }
            }

            // @ts-expect-error looks like a parent.
            if (node.children && result[0] !== SKIP) {
              // @ts-expect-error looks like a parent.
              offset = (reverse ? node.children.length : -1) + step;
              // @ts-expect-error looks like a parent.
              grandparents = parents.concat(node);

              // @ts-expect-error looks like a parent.
              while (offset > -1 && offset < node.children.length) {
                // @ts-expect-error looks like a parent.
                subresult = factory(node.children[offset], offset, grandparents)();

                if (subresult[0] === EXIT) {
                  return subresult
                }

                offset =
                  typeof subresult[1] === 'number' ? subresult[1] : offset + step;
              }
            }

            return result
          }
        }
      }
    );

  /**
   * @param {VisitorResult} value
   * @returns {ActionTuple}
   */
  function toResult(value) {
    if (Array.isArray(value)) {
      return value
    }

    if (typeof value === 'number') {
      return [CONTINUE, value]
    }

    return [value]
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   * @typedef {import('unist-util-is').Test} Test
   * @typedef {import('unist-util-visit-parents').VisitorResult} VisitorResult
   * @typedef {import('./complex-types').Visitor} Visitor
   */

  /**
   * Visit children of tree which pass a test
   *
   * @param tree Abstract syntax tree to walk
   * @param test Test, optional
   * @param visitor Function to run for each node
   * @param reverse Fisit the tree in reverse, defaults to false
   */
  const visit =
    /**
     * @type {(
     *   (<Tree extends Node, Check extends Test>(tree: Tree, test: Check, visitor: import('./complex-types').BuildVisitor<Tree, Check>, reverse?: boolean) => void) &
     *   (<Tree extends Node>(tree: Tree, visitor: import('./complex-types').BuildVisitor<Tree>, reverse?: boolean) => void)
     * )}
     */
    (
      /**
       * @param {Node} tree
       * @param {Test} test
       * @param {import('./complex-types').Visitor} visitor
       * @param {boolean} [reverse]
       */
      function (tree, test, visitor, reverse) {
        if (typeof test === 'function' && typeof visitor !== 'function') {
          reverse = visitor;
          visitor = test;
          test = null;
        }

        visitParents(tree, test, overload, reverse);

        /**
         * @param {Node} node
         * @param {Array.<Parent>} parents
         */
        function overload(node, parents) {
          const parent = parents[parents.length - 1];
          return visitor(
            node,
            parent ? parent.children.indexOf(node) : null,
            parent
          )
        }
      }
    );

  /**
   * @typedef {import('unist').Node} Node
   */

  /**
   * Utility to remove positions from a tree
   *
   * @param node The unist tree
   * @param force if `force` is given, uses `delete`, otherwise, sets positions to `undefined`.
   * @returns The same node, but either with `position: undefined` or w/o `position` fields
   */
  const removePosition =
    /**
     * @type {(
     *   (<Tree extends Node>(tree: Tree, force?: false) => Tree) &
     *   (<Tree extends Node>(tree: Tree, force: true) => Tree)
     * )}
     */
    (
      /**
       * @template {Node} Tree
       * @param {Tree} node
       * @param {boolean} [force=false]
       * @returns {Tree}
       */
      function (node, force) {
        visit(node, remove);

        // @ts-ignore hush TS, we know what we’re doing.
        return node

        /**
         * @param {Node} node the unist tree
         */
        function remove(node) {
          if (force) {
            delete node.position;
          } else {
            node.position = undefined;
          }
        }
      }
    );

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   * @typedef {import('hast').Element} Element
   *
   * @typedef {string} TagName
   * @typedef {null|undefined|TagName|TestFunctionAnything|Array.<TagName|TestFunctionAnything>} Test
   */

  const convertElement$2 =
    /**
     * @type {(
     *   (<T extends Element>(test: T['tagName']|TestFunctionPredicate<T>) => AssertPredicate<T>) &
     *   ((test?: Test) => AssertAnything)
     * )}
     */
    (
      /**
       * Generate an assertion from a check.
       * @param {Test} [test]
       * When nullish, checks if `node` is a `Node`.
       * When `string`, works like passing `function (node) {return node.type === test}`.
       * When `function` checks if function passed the node is true.
       * When `object`, checks that all keys in test are in node, and that they have (strictly) equal values.
       * When `array`, checks any one of the subtests pass.
       * @returns {AssertAnything}
       */
      function (test) {
        if (test === undefined || test === null) {
          return element$8
        }

        if (typeof test === 'string') {
          return tagNameFactory$2(test)
        }

        if (typeof test === 'object') {
          return anyFactory$2(test)
        }

        if (typeof test === 'function') {
          return castFactory$2(test)
        }

        throw new Error('Expected function, string, or array as test')
      }
    );

  /**
   * @param {Array.<TagName|TestFunctionAnything>} tests
   * @returns {AssertAnything}
   */
  function anyFactory$2(tests) {
    /** @type {Array.<AssertAnything>} */
    const checks = [];
    let index = -1;

    while (++index < tests.length) {
      checks[index] = convertElement$2(tests[index]);
    }

    return castFactory$2(any)

    /**
     * @this {unknown}
     * @param {unknown[]} parameters
     * @returns {boolean}
     */
    function any(...parameters) {
      let index = -1;

      while (++index < checks.length) {
        if (checks[index].call(this, ...parameters)) {
          return true
        }
      }

      return false
    }
  }

  /**
   * Utility to convert a string into a function which checks a given node’s tag
   * name for said string.
   *
   * @param {TagName} check
   * @returns {AssertAnything}
   */
  function tagNameFactory$2(check) {
    return tagName

    /**
     * @param {unknown} node
     * @returns {boolean}
     */
    function tagName(node) {
      return element$8(node) && node.tagName === check
    }
  }

  /**
   * @param {TestFunctionAnything} check
   * @returns {AssertAnything}
   */
  function castFactory$2(check) {
    return assertion

    /**
     * @this {unknown}
     * @param {unknown} node
     * @param {Array.<unknown>} parameters
     * @returns {boolean}
     */
    function assertion(node, ...parameters) {
      // @ts-expect-error: fine.
      return element$8(node) && Boolean(check.call(this, node, ...parameters))
    }
  }

  /**
   * Utility to return true if this is an element.
   * @param {unknown} node
   * @returns {node is Element}
   */
  function element$8(node) {
    return Boolean(
      node &&
        typeof node === 'object' &&
        // @ts-expect-error Looks like a node.
        node.type === 'element' &&
        // @ts-expect-error Looks like an element.
        typeof node.tagName === 'string'
    )
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   *
   * @typedef {import('unist-util-is').Type} Type
   * @typedef {import('unist-util-is').Props} Props
   * @typedef {import('unist-util-is').TestFunctionAnything} TestFunctionAnything
   */

  var findAfter =
    /**
     * @type {(
     *  (<T extends Node>(node: Parent, index: Node|number, test: T['type']|Partial<T>|import('unist-util-is').TestFunctionPredicate<T>|Array.<T['type']|Partial<T>|import('unist-util-is').TestFunctionPredicate<T>>) => T|null) &
     *  ((node: Parent, index: Node|number, test?: null|undefined|Type|Props|TestFunctionAnything|Array<Type|Props|TestFunctionAnything>) => Node|null)
     * )}
     */
    (
      /**
       * @param {Parent} parent Parent node
       * @param {Node|number} index Child of `parent`, or it’s index
       * @param {null|undefined|Type|Props|TestFunctionAnything|Array<Type|Props|TestFunctionAnything>} [test] is-compatible test (such as a type)
       * @returns {Node|null}
       */
      function (parent, index, test) {
        var is = convert(test);

        if (!parent || !parent.type || !parent.children) {
          throw new Error('Expected parent node')
        }

        if (typeof index === 'number') {
          if (index < 0 || index === Number.POSITIVE_INFINITY) {
            throw new Error('Expected positive finite number as index')
          }
        } else {
          index = parent.children.indexOf(index);

          if (index < 0) {
            throw new Error('Expected child node or index')
          }
        }

        while (++index < parent.children.length) {
          if (is(parent.children[index], index, parent)) {
            return parent.children[index]
          }
        }

        return null
      }
    );

  /**
   * @typedef {import('hast-util-is-element').TestFunctionAnything} TestFunctionAnything
   * @typedef {import('hast').Parent['children'][number]} HastChild
   * @typedef {import('hast').Text} HastText
   * @typedef {import('hast').Comment} HastComment
   * @typedef {import('hast').Root} HastRoot
   * @typedef {import('hast').Element} HastElement
   * @typedef {import('hast').Properties} HastProperties
   * @typedef {HastChild|HastRoot} HastNode
   * @typedef {HastRoot|HastElement} HastParent
   *
   * @typedef {'normal'|'pre'|'nowrap'|'pre-wrap'} Whitespace
   * @typedef {boolean} BreakValue
   * @typedef {1|2} BreakNumber
   * @typedef {'\n'} BreakForce
   * @typedef {BreakValue|BreakNumber|undefined} BreakBefore
   * @typedef {BreakValue|BreakNumber|BreakForce|undefined} BreakAfter
   *
   * @typedef CollectionOptions
   * @property {Whitespace} whitespace
   * @property {BreakBefore} breakBefore
   * @property {BreakAfter} breakAfter
   *
   * @typedef Options
   *   Configuration.
   * @property {Whitespace} [whitespace='normal']
   *   Initial CSS whitespace setting to use.
   */

  const searchLineFeeds = /\n/g;
  const searchTabOrSpaces = /[\t ]+/g;

  const br = convertElement$2('br');
  const p$2 = convertElement$2('p');
  const cell = convertElement$2(['th', 'td']);
  const row = convertElement$2('tr');

  // Note that we don’t need to include void elements here as they don’t have text.
  // See: <https://github.com/wooorm/html-void-elements>
  const notRendered = convertElement$2([
    // List from: <https://html.spec.whatwg.org/#hidden-elements>
    'datalist',
    'head',
    'noembed',
    'noframes',
    'noscript', // Act as if we support scripting.
    'rp',
    'script',
    'style',
    'template',
    'title',
    // Hidden attribute.
    hidden,
    // From: <https://html.spec.whatwg.org/#flow-content-3>
    closedDialog
  ]);

  // See: <https://html.spec.whatwg.org/#the-css-user-agent-style-sheet-and-presentational-hints>
  const blockOrCaption = convertElement$2([
    'address', // Flow content
    'article', // Sections and headings
    'aside', // Sections and headings
    'blockquote', // Flow content
    'body', // Page
    'caption', // `table-caption`
    'center', // Flow content (legacy)
    'dd', // Lists
    'dialog', // Flow content
    'dir', // Lists (legacy)
    'dl', // Lists
    'dt', // Lists
    'div', // Flow content
    'figure', // Flow content
    'figcaption', // Flow content
    'footer', // Flow content
    'form,', // Flow content
    'h1', // Sections and headings
    'h2', // Sections and headings
    'h3', // Sections and headings
    'h4', // Sections and headings
    'h5', // Sections and headings
    'h6', // Sections and headings
    'header', // Flow content
    'hgroup', // Sections and headings
    'hr', // Flow content
    'html', // Page
    'legend', // Flow content
    'listing', // Flow content (legacy)
    'main', // Flow content
    'menu', // Lists
    'nav', // Sections and headings
    'ol', // Lists
    'p', // Flow content
    'plaintext', // Flow content (legacy)
    'pre', // Flow content
    'section', // Sections and headings
    'ul', // Lists
    'xmp' // Flow content (legacy)
  ]);

  /**
   * Implementation of the `innerText` getter:
   * <https://html.spec.whatwg.org/#the-innertext-idl-attribute>
   * Note that we act as if `node` is being rendered, and as if we’re a
   * CSS-supporting user agent.
   *
   * @param {HastNode} node
   * @param {Options} [options={}]
   * @returns {string}
   */
  function toText(node, options = {}) {
    /** @type {Array.<HastChild>} */
    // @ts-ignore looks like a parent.
    const children = node.children || [];
    const block = blockOrCaption(node);
    const whitespace = inferWhitespace(node, {
      whitespace: options.whitespace || 'normal',
      breakBefore: false,
      breakAfter: false
    });
    let index = -1;
    /** @type {Array.<string|BreakNumber>} */
    let results;
    /** @type {string|BreakNumber} */
    let value;
    /** @type {number|undefined} */
    let count;

    // Treat `text` and `comment` as having normal white-space.
    // This deviates from the spec as in the DOM the node’s `.data` has to be
    // returned.
    // If you want that behavior use `hast-util-to-string`.
    // All other nodes are later handled as if they are `element`s (so the
    // algorithm also works on a `root`).
    // Nodes without children are treated as a void element, so `doctype` is thus
    // ignored.
    if (node.type === 'text' || node.type === 'comment') {
      return collectText(node, {whitespace, breakBefore: true, breakAfter: true})
    }

    // 1.  If this element is not being rendered, or if the user agent is a
    //     non-CSS user agent, then return the same value as the textContent IDL
    //     attribute on this element.
    //
    //     Note: we’re not supporting stylesheets so we’re acting as if the node
    //     is rendered.
    //
    //     If you want that behavior use `hast-util-to-string`.
    //     Important: we’ll have to account for this later though.

    // 2.  Let results be a new empty list.
    results = [];

    // 3.  For each child node node of this element:
    while (++index < children.length) {
      // 3.1. Let current be the list resulting in running the inner text
      //      collection steps with node.
      //      Each item in results will either be a JavaScript string or a
      //      positive integer (a required line break count).
      // 3.2. For each item item in current, append item to results.
      results = results.concat(
        // @ts-ignore Looks like a parent.
        innerTextCollection(children[index], node, {
          whitespace,
          breakBefore: index ? null : block,
          breakAfter:
            index < children.length - 1 ? br(children[index + 1]) : block
        })
      );
    }

    // 4.  Remove any items from results that are the empty string.
    // 5.  Remove any runs of consecutive required line break count items at the
    //     start or end of results.
    // 6.  Replace each remaining run of consecutive required line break count
    //     items with a string consisting of as many U+000A LINE FEED (LF)
    //     characters as the maximum of the values in the required line break
    //     count items.
    index = -1;
    /** @type {Array.<string>} */
    const result = [];

    while (++index < results.length) {
      value = results[index];

      if (typeof value === 'number') {
        if (count !== undefined && value > count) count = value;
      } else if (value) {
        if (count) result.push('\n'.repeat(count));
        count = 0;
        result.push(value);
      }
    }

    // 7.  Return the concatenation of the string items in results.
    return result.join('')
  }

  /**
   * <https://html.spec.whatwg.org/#inner-text-collection-steps>
   *
   * @param {HastNode} node
   * @param {HastParent} parent
   * @param {CollectionOptions} options
   * @returns {Array.<string|BreakNumber>}
   */
  function innerTextCollection(node, parent, options) {
    if (node.type === 'element') {
      return collectElement(node, parent, options)
    }

    if (node.type === 'text') {
      return [
        options.whitespace === 'normal'
          ? collectText(node, options)
          : collectPreText(node)
      ]
    }

    return []
  }

  /**
   * Collect an element.
   *
   * @param {HastElement} node
   * @param {HastParent} parent
   * @param {CollectionOptions} options
   */
  function collectElement(node, parent, options) {
    // First we infer the `white-space` property.
    const whitespace = inferWhitespace(node, options);
    const children = node.children || [];
    let index = -1;
    /** @type {Array.<string|BreakNumber>} */
    let items = [];
    /** @type {BreakNumber|undefined} */
    let prefix;
    /** @type {BreakNumber|BreakForce|undefined} */
    let suffix;

    // We’re ignoring point 3, and exiting without any content here, because we
    // deviated from the spec in `toText` at step 3.
    if (notRendered(node)) {
      return items
    }

    // Note: we first detect if there is going to be a break before or after the
    // contents, as that changes the white-space handling.

    // 2.  If node’s computed value of `visibility` is not `visible`, then return
    //     items.
    //
    //     Note: Ignored, as everything is visible by default user agent styles.

    // 3.  If node is not being rendered, then return items. [...]
    //
    //     Note: We already did this above.

    // See `collectText` for step 4.

    // 5.  If node is a `<br>` element, then append a string containing a single
    //     U+000A LINE FEED (LF) character to items.
    if (br(node)) {
      suffix = '\n';
    }

    // 7.  If node’s computed value of `display` is `table-row`, and node’s CSS
    //     box is not the last `table-row` box of the nearest ancestor `table`
    //     box, then append a string containing a single U+000A LINE FEED (LF)
    //     character to items.
    //
    //     See: <https://html.spec.whatwg.org/#tables-2>
    //     Note: needs further investigation as this does not account for implicit
    //     rows.
    else if (row(node) && findAfter(parent, node, row)) {
      suffix = '\n';
    }

    // 8.  If node is a `<p>` element, then append 2 (a required line break count)
    //     at the beginning and end of items.
    else if (p$2(node)) {
      prefix = 2;
      suffix = 2;
    }

    // 9.  If node’s used value of `display` is block-level or `table-caption`,
    //     then append 1 (a required line break count) at the beginning and end of
    //     items.
    else if (blockOrCaption(node)) {
      prefix = 1;
      suffix = 1;
    }

    // 1.  Let items be the result of running the inner text collection steps with
    //     each child node of node in tree order, and then concatenating the
    //     results to a single list.
    while (++index < children.length) {
      items = items.concat(
        innerTextCollection(children[index], node, {
          whitespace,
          breakBefore: index ? undefined : prefix,
          breakAfter:
            index < children.length - 1 ? br(children[index + 1]) : suffix
        })
      );
    }

    // 6.  If node’s computed value of `display` is `table-cell`, and node’s CSS
    //     box is not the last `table-cell` box of its enclosing `table-row` box,
    //     then append a string containing a single U+0009 CHARACTER TABULATION
    //     (tab) character to items.
    //
    //     See: <https://html.spec.whatwg.org/#tables-2>
    if (cell(node) && findAfter(parent, node, cell)) {
      items.push('\t');
    }

    // Add the pre- and suffix.
    if (prefix) items.unshift(prefix);
    if (suffix) items.push(suffix);

    return items
  }

  /**
   * 4.  If node is a Text node, then for each CSS text box produced by node,
   *     in content order, compute the text of the box after application of the
   *     CSS `white-space` processing rules and `text-transform` rules, set
   *     items to the list of the resulting strings, and return items.
   *     The CSS `white-space` processing rules are slightly modified:
   *     collapsible spaces at the end of lines are always collapsed, but they
   *     are only removed if the line is the last line of the block, or it ends
   *     with a br element.
   *     Soft hyphens should be preserved.
   *
   *     Note: See `collectText` and `collectPreText`.
   *     Note: we don’t deal with `text-transform`, no element has that by
   *     default.
   *
   * See: <https://drafts.csswg.org/css-text/#white-space-phase-1>
   *
   * @param {HastText|HastComment} node
   * @param {CollectionOptions} options
   * @returns {string}
   */
  function collectText(node, options) {
    const value = String(node.value);
    /** @type {Array.<string>} */
    const lines = [];
    /** @type {Array.<string>} */
    const result = [];
    let start = 0;
    let index = -1;
    /** @type {RegExpMatchArray|null} */
    let match;
    /** @type {number} */
    let end;
    /** @type {string|undefined} */
    let join;

    while (start < value.length) {
      searchLineFeeds.lastIndex = start;
      match = searchLineFeeds.exec(value);
      // @ts-expect-error: `index` is set.
      end = match ? match.index : value.length;

      lines.push(
        // Any sequence of collapsible spaces and tabs immediately preceding or
        // following a segment break is removed.
        trimAndCollapseSpacesAndTabs(
          // [...] ignoring bidi formatting characters (characters with the
          // Bidi_Control property [UAX9]: ALM, LTR, RTL, LRE-RLO, LRI-PDI) as if
          // they were not there.
          value
            .slice(start, end)
            .replace(/[\u061C\u200E\u200F\u202A-\u202E\u2066-\u2069]/g, ''),
          options.breakBefore,
          options.breakAfter
        )
      );

      start = end + 1;
    }

    // Collapsible segment breaks are transformed for rendering according to the
    // segment break transformation rules.
    // So here we jump to 4.1.2 of [CSSTEXT]:
    // Any collapsible segment break immediately following another collapsible
    // segment break is removed
    while (++index < lines.length) {
      // *   If the character immediately before or immediately after the segment
      //     break is the zero-width space character (U+200B), then the break is
      //     removed, leaving behind the zero-width space.
      if (
        lines[index].charCodeAt(lines[index].length - 1) === 0x200b /* ZWSP */ ||
        (index < lines.length - 1 &&
          lines[index + 1].charCodeAt(0) === 0x200b) /* ZWSP */
      ) {
        result.push(lines[index]);
        join = '';
      }

      // *   Otherwise, if the East Asian Width property [UAX11] of both the
      //     character before and after the segment break is Fullwidth, Wide, or
      //     Halfwidth (not Ambiguous), and neither side is Hangul, then the
      //     segment break is removed.
      //
      //     Note: ignored.
      // *   Otherwise, if the writing system of the segment break is Chinese,
      //     Japanese, or Yi, and the character before or after the segment break
      //     is punctuation or a symbol (Unicode general category P* or S*) and
      //     has an East Asian Width property of Ambiguous, and the character on
      //     the other side of the segment break is Fullwidth, Wide, or Halfwidth,
      //     and not Hangul, then the segment break is removed.
      //
      //     Note: ignored.

      // *   Otherwise, the segment break is converted to a space (U+0020).
      else if (lines[index]) {
        if (join) result.push(join);
        result.push(lines[index]);
        join = ' ';
      }
    }

    return result.join('')
  }

  /**
   * @param {HastText|HastComment} node
   * @returns {string}
   */
  function collectPreText(node) {
    return String(node.value)
  }

  /**
   * 3.  Every collapsible tab is converted to a collapsible space (U+0020).
   * 4.  Any collapsible space immediately following another collapsible
   *     space—even one outside the boundary of the inline containing that
   *     space, provided both spaces are within the same inline formatting
   *     context—is collapsed to have zero advance width. (It is invisible,
   *     but retains its soft wrap opportunity, if any.)
   *
   * @param {string} value
   * @param {BreakBefore} breakBefore
   * @param {BreakAfter} breakAfter
   * @returns {string}
   */
  function trimAndCollapseSpacesAndTabs(value, breakBefore, breakAfter) {
    /** @type {Array.<string>} */
    const result = [];
    let start = 0;
    /** @type {RegExpMatchArray|null} */
    let match;
    /** @type {number} */
    let end;

    while (start < value.length) {
      searchTabOrSpaces.lastIndex = start;
      match = searchTabOrSpaces.exec(value);
      // @ts-expect-error: `index` is set.
      end = match ? match.index : value.length;

      // If we’re not directly after a segment break, but there was white space,
      // add an empty value that will be turned into a space.
      if (!start && !end && match && !breakBefore) {
        result.push('');
      }

      if (start !== end) {
        result.push(value.slice(start, end));
      }

      start = match ? end + match[0].length : end;
    }

    // If we reached the end, there was trailing white space, and there’s no
    // segment break after this node, add an empty value that will be turned
    // into a space.
    // @ts-expect-error: `end` is defined.
    if (start !== end && !breakAfter) {
      result.push('');
    }

    return result.join(' ')
  }

  /**
   * We don’t support void elements here (so `nobr wbr` -> `normal` is ignored).
   *
   * @param {HastNode} node
   * @param {CollectionOptions} options
   * @returns {Whitespace}
   */
  function inferWhitespace(node, options) {
    /** @type {HastProperties} */
    let props;

    if (node.type === 'element') {
      props = node.properties || {};
      switch (node.tagName) {
        case 'listing':
        case 'plaintext':
        case 'xmp':
          return 'pre'
        case 'nobr':
          return 'nowrap'
        case 'pre':
          return props.wrap ? 'pre-wrap' : 'pre'
        case 'td':
        case 'th':
          return props.noWrap ? 'nowrap' : options.whitespace
        case 'textarea':
          return 'pre-wrap'
      }
    }

    return options.whitespace
  }

  /** @type {TestFunctionAnything} */
  function hidden(node) {
    return Boolean((node.properties || {}).hidden)
  }

  /** @type {TestFunctionAnything} */
  function closedDialog(node) {
    return node.tagName === 'dialog' && !(node.properties || {}).open
  }

  /**
   * Throw a given error.
   *
   * @param {Error|null|undefined} [error]
   *   Maybe error.
   * @returns {asserts error is null|undefined}
   */
  function bail(error) {
    if (error) {
      throw error
    }
  }

  /*!
   * Determine if an object is a Buffer
   *
   * @author   Feross Aboukhadijeh <https://feross.org>
   * @license  MIT
   */

  var isBuffer = function isBuffer (obj) {
    return obj != null && obj.constructor != null &&
      typeof obj.constructor.isBuffer === 'function' && obj.constructor.isBuffer(obj)
  };

  var hasOwn = Object.prototype.hasOwnProperty;
  var toStr = Object.prototype.toString;
  var defineProperty = Object.defineProperty;
  var gOPD = Object.getOwnPropertyDescriptor;

  var isArray = function isArray(arr) {
    if (typeof Array.isArray === 'function') {
      return Array.isArray(arr);
    }

    return toStr.call(arr) === '[object Array]';
  };

  var isPlainObject$1 = function isPlainObject(obj) {
    if (!obj || toStr.call(obj) !== '[object Object]') {
      return false;
    }

    var hasOwnConstructor = hasOwn.call(obj, 'constructor');
    var hasIsPrototypeOf = obj.constructor && obj.constructor.prototype && hasOwn.call(obj.constructor.prototype, 'isPrototypeOf');
    // Not own constructor property must be Object
    if (obj.constructor && !hasOwnConstructor && !hasIsPrototypeOf) {
      return false;
    }

    // Own properties are enumerated firstly, so to speed up,
    // if last one is own, then all properties are own.
    var key;
    for (key in obj) { /**/ }

    return typeof key === 'undefined' || hasOwn.call(obj, key);
  };

  // If name is '__proto__', and Object.defineProperty is available, define __proto__ as an own property on target
  var setProperty = function setProperty(target, options) {
    if (defineProperty && options.name === '__proto__') {
      defineProperty(target, options.name, {
        enumerable: true,
        configurable: true,
        value: options.newValue,
        writable: true
      });
    } else {
      target[options.name] = options.newValue;
    }
  };

  // Return undefined instead of __proto__ if '__proto__' is not an own property
  var getProperty = function getProperty(obj, name) {
    if (name === '__proto__') {
      if (!hasOwn.call(obj, name)) {
        return void 0;
      } else if (gOPD) {
        // In early versions of node, obj['__proto__'] is buggy when obj has
        // __proto__ as an own property. Object.getOwnPropertyDescriptor() works.
        return gOPD(obj, name).value;
      }
    }

    return obj[name];
  };

  var extend = function extend() {
    var options, name, src, copy, copyIsArray, clone;
    var target = arguments[0];
    var i = 1;
    var length = arguments.length;
    var deep = false;

    // Handle a deep copy situation
    if (typeof target === 'boolean') {
      deep = target;
      target = arguments[1] || {};
      // skip the boolean and the target
      i = 2;
    }
    if (target == null || (typeof target !== 'object' && typeof target !== 'function')) {
      target = {};
    }

    for (; i < length; ++i) {
      options = arguments[i];
      // Only deal with non-null/undefined values
      if (options != null) {
        // Extend the base object
        for (name in options) {
          src = getProperty(target, name);
          copy = getProperty(options, name);

          // Prevent never-ending loop
          if (target !== copy) {
            // Recurse if we're merging plain objects or arrays
            if (deep && copy && (isPlainObject$1(copy) || (copyIsArray = isArray(copy)))) {
              if (copyIsArray) {
                copyIsArray = false;
                clone = src && isArray(src) ? src : [];
              } else {
                clone = src && isPlainObject$1(src) ? src : {};
              }

              // Never move original objects, clone them
              setProperty(target, { name: name, newValue: extend(deep, clone, copy) });

            // Don't bring in undefined values
            } else if (typeof copy !== 'undefined') {
              setProperty(target, { name: name, newValue: copy });
            }
          }
        }
      }
    }

    // Return the modified object
    return target;
  };

  function isPlainObject(value) {
    if (Object.prototype.toString.call(value) !== '[object Object]') {
      return false;
    }

    const prototype = Object.getPrototypeOf(value);
    return prototype === null || prototype === Object.prototype;
  }

  /**
   * @typedef {(error?: Error|null|undefined, ...output: Array<any>) => void} Callback
   * @typedef {(...input: Array<any>) => any} Middleware
   *
   * @typedef {(...input: Array<any>) => void} Run
   *   Call all middleware.
   * @typedef {(fn: Middleware) => Pipeline} Use
   *   Add `fn` (middleware) to the list.
   * @typedef {{run: Run, use: Use}} Pipeline
   *   Middleware.
   */

  /**
   * Create new middleware.
   *
   * @returns {Pipeline}
   */
  function trough() {
    /** @type {Array<Middleware>} */
    const fns = [];
    /** @type {Pipeline} */
    const pipeline = {run, use};

    return pipeline

    /** @type {Run} */
    function run(...values) {
      let middlewareIndex = -1;
      /** @type {Callback} */
      const callback = values.pop();

      if (typeof callback !== 'function') {
        throw new TypeError('Expected function as last argument, not ' + callback)
      }

      next(null, ...values);

      /**
       * Run the next `fn`, or we’re done.
       *
       * @param {Error|null|undefined} error
       * @param {Array<any>} output
       */
      function next(error, ...output) {
        const fn = fns[++middlewareIndex];
        let index = -1;

        if (error) {
          callback(error);
          return
        }

        // Copy non-nullish input into values.
        while (++index < values.length) {
          if (output[index] === null || output[index] === undefined) {
            output[index] = values[index];
          }
        }

        // Save the newly created `output` for the next call.
        values = output;

        // Next or done.
        if (fn) {
          wrap(fn, next)(...output);
        } else {
          callback(null, ...output);
        }
      }
    }

    /** @type {Use} */
    function use(middelware) {
      if (typeof middelware !== 'function') {
        throw new TypeError(
          'Expected `middelware` to be a function, not ' + middelware
        )
      }

      fns.push(middelware);
      return pipeline
    }
  }

  /**
   * Wrap `middleware`.
   * Can be sync or async; return a promise, receive a callback, or return new
   * values and errors.
   *
   * @param {Middleware} middleware
   * @param {Callback} callback
   */
  function wrap(middleware, callback) {
    /** @type {boolean} */
    let called;

    return wrapped

    /**
     * Call `middleware`.
     * @this {any}
     * @param {Array<any>} parameters
     * @returns {void}
     */
    function wrapped(...parameters) {
      const fnExpectsCallback = middleware.length > parameters.length;
      /** @type {any} */
      let result;

      if (fnExpectsCallback) {
        parameters.push(done);
      }

      try {
        result = middleware.apply(this, parameters);
      } catch (error) {
        const exception = /** @type {Error} */ (error);

        // Well, this is quite the pickle.
        // `middleware` received a callback and called it synchronously, but that
        // threw an error.
        // The only thing left to do is to throw the thing instead.
        if (fnExpectsCallback && called) {
          throw exception
        }

        return done(exception)
      }

      if (!fnExpectsCallback) {
        if (result instanceof Promise) {
          result.then(then, done);
        } else if (result instanceof Error) {
          done(result);
        } else {
          then(result);
        }
      }
    }

    /**
     * Call `callback`, only once.
     * @type {Callback}
     */
    function done(error, ...output) {
      if (!called) {
        called = true;
        callback(error, ...output);
      }
    }

    /**
     * Call `done` with one value.
     *
     * @param {any} [value]
     */
    function then(value) {
      done(null, value);
    }
  }

  /**
   * @typedef {import('unist').Point} Point
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Position} Position
   * @typedef {object & {type: string, position?: Position|undefined}} NodeLike
   */

  /**
   * Stringify one point, a position (start and end points), or a node’s
   * positional information.
   *
   * @param {Node|NodeLike|Position|Point|null} [value]
   * @returns {string}
   */
  function stringifyPosition(value) {
    // Nothing.
    if (!value || typeof value !== 'object') {
      return ''
    }

    // Node.
    if ('position' in value || 'type' in value) {
      return position$2(value.position)
    }

    // Position.
    if ('start' in value || 'end' in value) {
      return position$2(value)
    }

    // Point.
    if ('line' in value || 'column' in value) {
      return point$2(value)
    }

    // ?
    return ''
  }

  /**
   * @param {Point|undefined} point
   * @returns {string}
   */
  function point$2(point) {
    return index(point && point.line) + ':' + index(point && point.column)
  }

  /**
   * @param {Position|undefined} pos
   * @returns {string}
   */
  function position$2(pos) {
    return point$2(pos && pos.start) + '-' + point$2(pos && pos.end)
  }

  /**
   * @param {number|undefined} value
   * @returns {number}
   */
  function index(value) {
    return value && typeof value === 'number' ? value : 1
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Position} Position
   * @typedef {import('unist').Point} Point
   * @typedef {object & {type: string, position?: Position|undefined}} NodeLike
   */

  class VFileMessage extends Error {
    /**
     * Constructor of a message for `reason` at `place` from `origin`.
     * When an error is passed in as `reason`, copies the `stack`.
     *
     * @param {string|Error} reason Reason for message (`string` or `Error`). Uses the stack and message of the error if given.
     * @param {Node|NodeLike|Position|Point} [place] Place at which the message occurred in a file (`Node`, `Position`, or `Point`, optional).
     * @param {string} [origin] Place in code the message originates from (`string`, optional).
     */
    constructor(reason, place, origin) {
      /** @type {[string|null, string|null]} */
      const parts = [null, null];
      /** @type {Position} */
      let position = {
        // @ts-expect-error: we always follows the structure of `position`.
        start: {line: null, column: null},
        // @ts-expect-error: "
        end: {line: null, column: null}
      };

      super();

      if (typeof place === 'string') {
        origin = place;
        place = undefined;
      }

      if (typeof origin === 'string') {
        const index = origin.indexOf(':');

        if (index === -1) {
          parts[1] = origin;
        } else {
          parts[0] = origin.slice(0, index);
          parts[1] = origin.slice(index + 1);
        }
      }

      if (place) {
        // Node.
        if ('type' in place || 'position' in place) {
          if (place.position) {
            position = place.position;
          }
        }
        // Position.
        else if ('start' in place || 'end' in place) {
          position = place;
        }
        // Point.
        else if ('line' in place || 'column' in place) {
          position.start = place;
        }
      }

      // Fields from `Error`
      this.name = stringifyPosition(place) || '1:1';
      this.message = typeof reason === 'object' ? reason.message : reason;
      this.stack = typeof reason === 'object' ? reason.stack : '';

      /**
       * Reason for message.
       * @type {string}
       */
      this.reason = this.message;
      /**
       * If true, marks associated file as no longer processable.
       * @type {boolean?}
       */
      // eslint-disable-next-line no-unused-expressions
      this.fatal;
      /**
       * Starting line of error.
       * @type {number?}
       */
      this.line = position.start.line;
      /**
       * Starting column of error.
       * @type {number?}
       */
      this.column = position.start.column;
      /**
       * Namespace of warning.
       * @type {string?}
       */
      this.source = parts[0];
      /**
       * Category of message.
       * @type {string?}
       */
      this.ruleId = parts[1];
      /**
       * Full range information, when available.
       * Has start and end properties, both set to an object with line and column, set to number?.
       * @type {Position?}
       */
      this.position = position;

      // The following fields are “well known”.
      // Not standard.
      // Feel free to add other non-standard fields to your messages.

      /* eslint-disable no-unused-expressions */
      /**
       * You can use this to specify the source value that’s being reported, which
       * is deemed incorrect.
       * @type {string?}
       */
      this.actual;
      /**
       * You can use this to suggest values that should be used instead of
       * `actual`, one or more values that are deemed as acceptable.
       * @type {Array<string>?}
       */
      this.expected;
      /**
       * You may add a file property with a path of a file (used throughout the VFile ecosystem).
       * @type {string?}
       */
      this.file;
      /**
       * You may add a url property with a link to documentation for the message.
       * @type {string?}
       */
      this.url;
      /**
       * You may add a note property with a long form description of the message (supported by vfile-reporter).
       * @type {string?}
       */
      this.note;
      /* eslint-enable no-unused-expressions */
    }
  }

  VFileMessage.prototype.file = '';
  VFileMessage.prototype.name = '';
  VFileMessage.prototype.reason = '';
  VFileMessage.prototype.message = '';
  VFileMessage.prototype.stack = '';
  VFileMessage.prototype.fatal = null;
  VFileMessage.prototype.column = null;
  VFileMessage.prototype.line = null;
  VFileMessage.prototype.source = null;
  VFileMessage.prototype.ruleId = null;
  VFileMessage.prototype.position = null;

  // A derivative work based on:
  // <https://github.com/browserify/path-browserify>.
  // Which is licensed:
  //
  // MIT License
  //
  // Copyright (c) 2013 James Halliday
  //
  // Permission is hereby granted, free of charge, to any person obtaining a copy of
  // this software and associated documentation files (the "Software"), to deal in
  // the Software without restriction, including without limitation the rights to
  // use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
  // the Software, and to permit persons to whom the Software is furnished to do so,
  // subject to the following conditions:
  //
  // The above copyright notice and this permission notice shall be included in all
  // copies or substantial portions of the Software.
  //
  // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
  // FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
  // COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
  // IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  // CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  // A derivative work based on:
  //
  // Parts of that are extracted from Node’s internal `path` module:
  // <https://github.com/nodejs/node/blob/master/lib/path.js>.
  // Which is licensed:
  //
  // Copyright Joyent, Inc. and other Node contributors.
  //
  // Permission is hereby granted, free of charge, to any person obtaining a
  // copy of this software and associated documentation files (the
  // "Software"), to deal in the Software without restriction, including
  // without limitation the rights to use, copy, modify, merge, publish,
  // distribute, sublicense, and/or sell copies of the Software, and to permit
  // persons to whom the Software is furnished to do so, subject to the
  // following conditions:
  //
  // The above copyright notice and this permission notice shall be included
  // in all copies or substantial portions of the Software.
  //
  // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  // OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
  // NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
  // DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
  // OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
  // USE OR OTHER DEALINGS IN THE SOFTWARE.

  const path = {basename, dirname, extname, join, sep: '/'};

  /* eslint-disable max-depth, complexity */

  /**
   * @param {string} path
   * @param {string} [ext]
   * @returns {string}
   */
  function basename(path, ext) {
    if (ext !== undefined && typeof ext !== 'string') {
      throw new TypeError('"ext" argument must be a string')
    }

    assertPath$1(path);
    let start = 0;
    let end = -1;
    let index = path.length;
    /** @type {boolean|undefined} */
    let seenNonSlash;

    if (ext === undefined || ext.length === 0 || ext.length > path.length) {
      while (index--) {
        if (path.charCodeAt(index) === 47 /* `/` */) {
          // If we reached a path separator that was not part of a set of path
          // separators at the end of the string, stop now.
          if (seenNonSlash) {
            start = index + 1;
            break
          }
        } else if (end < 0) {
          // We saw the first non-path separator, mark this as the end of our
          // path component.
          seenNonSlash = true;
          end = index + 1;
        }
      }

      return end < 0 ? '' : path.slice(start, end)
    }

    if (ext === path) {
      return ''
    }

    let firstNonSlashEnd = -1;
    let extIndex = ext.length - 1;

    while (index--) {
      if (path.charCodeAt(index) === 47 /* `/` */) {
        // If we reached a path separator that was not part of a set of path
        // separators at the end of the string, stop now.
        if (seenNonSlash) {
          start = index + 1;
          break
        }
      } else {
        if (firstNonSlashEnd < 0) {
          // We saw the first non-path separator, remember this index in case
          // we need it if the extension ends up not matching.
          seenNonSlash = true;
          firstNonSlashEnd = index + 1;
        }

        if (extIndex > -1) {
          // Try to match the explicit extension.
          if (path.charCodeAt(index) === ext.charCodeAt(extIndex--)) {
            if (extIndex < 0) {
              // We matched the extension, so mark this as the end of our path
              // component
              end = index;
            }
          } else {
            // Extension does not match, so our result is the entire path
            // component
            extIndex = -1;
            end = firstNonSlashEnd;
          }
        }
      }
    }

    if (start === end) {
      end = firstNonSlashEnd;
    } else if (end < 0) {
      end = path.length;
    }

    return path.slice(start, end)
  }

  /**
   * @param {string} path
   * @returns {string}
   */
  function dirname(path) {
    assertPath$1(path);

    if (path.length === 0) {
      return '.'
    }

    let end = -1;
    let index = path.length;
    /** @type {boolean|undefined} */
    let unmatchedSlash;

    // Prefix `--` is important to not run on `0`.
    while (--index) {
      if (path.charCodeAt(index) === 47 /* `/` */) {
        if (unmatchedSlash) {
          end = index;
          break
        }
      } else if (!unmatchedSlash) {
        // We saw the first non-path separator
        unmatchedSlash = true;
      }
    }

    return end < 0
      ? path.charCodeAt(0) === 47 /* `/` */
        ? '/'
        : '.'
      : end === 1 && path.charCodeAt(0) === 47 /* `/` */
      ? '//'
      : path.slice(0, end)
  }

  /**
   * @param {string} path
   * @returns {string}
   */
  function extname(path) {
    assertPath$1(path);

    let index = path.length;

    let end = -1;
    let startPart = 0;
    let startDot = -1;
    // Track the state of characters (if any) we see before our first dot and
    // after any path separator we find.
    let preDotState = 0;
    /** @type {boolean|undefined} */
    let unmatchedSlash;

    while (index--) {
      const code = path.charCodeAt(index);

      if (code === 47 /* `/` */) {
        // If we reached a path separator that was not part of a set of path
        // separators at the end of the string, stop now.
        if (unmatchedSlash) {
          startPart = index + 1;
          break
        }

        continue
      }

      if (end < 0) {
        // We saw the first non-path separator, mark this as the end of our
        // extension.
        unmatchedSlash = true;
        end = index + 1;
      }

      if (code === 46 /* `.` */) {
        // If this is our first dot, mark it as the start of our extension.
        if (startDot < 0) {
          startDot = index;
        } else if (preDotState !== 1) {
          preDotState = 1;
        }
      } else if (startDot > -1) {
        // We saw a non-dot and non-path separator before our dot, so we should
        // have a good chance at having a non-empty extension.
        preDotState = -1;
      }
    }

    if (
      startDot < 0 ||
      end < 0 ||
      // We saw a non-dot character immediately before the dot.
      preDotState === 0 ||
      // The (right-most) trimmed path component is exactly `..`.
      (preDotState === 1 && startDot === end - 1 && startDot === startPart + 1)
    ) {
      return ''
    }

    return path.slice(startDot, end)
  }

  /**
   * @param {Array<string>} segments
   * @returns {string}
   */
  function join(...segments) {
    let index = -1;
    /** @type {string|undefined} */
    let joined;

    while (++index < segments.length) {
      assertPath$1(segments[index]);

      if (segments[index]) {
        joined =
          joined === undefined ? segments[index] : joined + '/' + segments[index];
      }
    }

    return joined === undefined ? '.' : normalize$7(joined)
  }

  /**
   * Note: `normalize` is not exposed as `path.normalize`, so some code is
   * manually removed from it.
   *
   * @param {string} path
   * @returns {string}
   */
  function normalize$7(path) {
    assertPath$1(path);

    const absolute = path.charCodeAt(0) === 47; /* `/` */

    // Normalize the path according to POSIX rules.
    let value = normalizeString(path, !absolute);

    if (value.length === 0 && !absolute) {
      value = '.';
    }

    if (value.length > 0 && path.charCodeAt(path.length - 1) === 47 /* / */) {
      value += '/';
    }

    return absolute ? '/' + value : value
  }

  /**
   * Resolve `.` and `..` elements in a path with directory names.
   *
   * @param {string} path
   * @param {boolean} allowAboveRoot
   * @returns {string}
   */
  function normalizeString(path, allowAboveRoot) {
    let result = '';
    let lastSegmentLength = 0;
    let lastSlash = -1;
    let dots = 0;
    let index = -1;
    /** @type {number|undefined} */
    let code;
    /** @type {number} */
    let lastSlashIndex;

    while (++index <= path.length) {
      if (index < path.length) {
        code = path.charCodeAt(index);
      } else if (code === 47 /* `/` */) {
        break
      } else {
        code = 47; /* `/` */
      }

      if (code === 47 /* `/` */) {
        if (lastSlash === index - 1 || dots === 1) ; else if (lastSlash !== index - 1 && dots === 2) {
          if (
            result.length < 2 ||
            lastSegmentLength !== 2 ||
            result.charCodeAt(result.length - 1) !== 46 /* `.` */ ||
            result.charCodeAt(result.length - 2) !== 46 /* `.` */
          ) {
            if (result.length > 2) {
              lastSlashIndex = result.lastIndexOf('/');

              if (lastSlashIndex !== result.length - 1) {
                if (lastSlashIndex < 0) {
                  result = '';
                  lastSegmentLength = 0;
                } else {
                  result = result.slice(0, lastSlashIndex);
                  lastSegmentLength = result.length - 1 - result.lastIndexOf('/');
                }

                lastSlash = index;
                dots = 0;
                continue
              }
            } else if (result.length > 0) {
              result = '';
              lastSegmentLength = 0;
              lastSlash = index;
              dots = 0;
              continue
            }
          }

          if (allowAboveRoot) {
            result = result.length > 0 ? result + '/..' : '..';
            lastSegmentLength = 2;
          }
        } else {
          if (result.length > 0) {
            result += '/' + path.slice(lastSlash + 1, index);
          } else {
            result = path.slice(lastSlash + 1, index);
          }

          lastSegmentLength = index - lastSlash - 1;
        }

        lastSlash = index;
        dots = 0;
      } else if (code === 46 /* `.` */ && dots > -1) {
        dots++;
      } else {
        dots = -1;
      }
    }

    return result
  }

  /**
   * @param {string} path
   */
  function assertPath$1(path) {
    if (typeof path !== 'string') {
      throw new TypeError(
        'Path must be a string. Received ' + JSON.stringify(path)
      )
    }
  }

  /* eslint-enable max-depth, complexity */

  // Somewhat based on:
  // <https://github.com/defunctzombie/node-process/blob/master/browser.js>.
  // But I don’t think one tiny line of code can be copyrighted. 😅
  const proc = {cwd};

  function cwd() {
    return '/'
  }

  /**
   * @typedef URL
   * @property {string} hash
   * @property {string} host
   * @property {string} hostname
   * @property {string} href
   * @property {string} origin
   * @property {string} password
   * @property {string} pathname
   * @property {string} port
   * @property {string} protocol
   * @property {string} search
   * @property {any} searchParams
   * @property {string} username
   * @property {() => string} toString
   * @property {() => string} toJSON
   */

  /**
   * @param {unknown} fileURLOrPath
   * @returns {fileURLOrPath is URL}
   */
  // From: <https://github.com/nodejs/node/blob/fcf8ba4/lib/internal/url.js#L1501>
  function isUrl(fileURLOrPath) {
    return (
      fileURLOrPath !== null &&
      typeof fileURLOrPath === 'object' &&
      // @ts-expect-error: indexable.
      fileURLOrPath.href &&
      // @ts-expect-error: indexable.
      fileURLOrPath.origin
    )
  }

  /// <reference lib="dom" />

  // See: <https://github.com/nodejs/node/blob/fcf8ba4/lib/internal/url.js>

  /**
   * @param {string|URL} path
   */
  function urlToPath(path) {
    if (typeof path === 'string') {
      path = new URL(path);
    } else if (!isUrl(path)) {
      /** @type {NodeJS.ErrnoException} */
      const error = new TypeError(
        'The "path" argument must be of type string or an instance of URL. Received `' +
          path +
          '`'
      );
      error.code = 'ERR_INVALID_ARG_TYPE';
      throw error
    }

    if (path.protocol !== 'file:') {
      /** @type {NodeJS.ErrnoException} */
      const error = new TypeError('The URL must be of scheme file');
      error.code = 'ERR_INVALID_URL_SCHEME';
      throw error
    }

    return getPathFromURLPosix(path)
  }

  /**
   * @param {URL} url
   */
  function getPathFromURLPosix(url) {
    if (url.hostname !== '') {
      /** @type {NodeJS.ErrnoException} */
      const error = new TypeError(
        'File URL host must be "localhost" or empty on darwin'
      );
      error.code = 'ERR_INVALID_FILE_URL_HOST';
      throw error
    }

    const pathname = url.pathname;
    let index = -1;

    while (++index < pathname.length) {
      if (
        pathname.charCodeAt(index) === 37 /* `%` */ &&
        pathname.charCodeAt(index + 1) === 50 /* `2` */
      ) {
        const third = pathname.charCodeAt(index + 2);
        if (third === 70 /* `F` */ || third === 102 /* `f` */) {
          /** @type {NodeJS.ErrnoException} */
          const error = new TypeError(
            'File URL path must not include encoded / characters'
          );
          error.code = 'ERR_INVALID_FILE_URL_PATH';
          throw error
        }
      }
    }

    return decodeURIComponent(pathname)
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Position} Position
   * @typedef {import('unist').Point} Point
   * @typedef {Record<string, unknown> & {type: string, position?: Position|undefined}} NodeLike
   * @typedef {import('./minurl.shared.js').URL} URL
   * @typedef {import('..').VFileData} VFileData
   * @typedef {import('..').VFileValue} VFileValue
   *
   * @typedef {'ascii'|'utf8'|'utf-8'|'utf16le'|'ucs2'|'ucs-2'|'base64'|'base64url'|'latin1'|'binary'|'hex'} BufferEncoding
   *   Encodings supported by the buffer class.
   *   This is a copy of the typing from Node, copied to prevent Node globals from
   *   being needed.
   *   Copied from: <https://github.com/DefinitelyTyped/DefinitelyTyped/blob/90a4ec8/types/node/buffer.d.ts#L170>
   *
   * @typedef {VFileValue|VFileOptions|VFile|URL} VFileCompatible
   *   Things that can be passed to the constructor.
   *
   * @typedef VFileCoreOptions
   * @property {VFileValue} [value]
   * @property {string} [cwd]
   * @property {Array<string>} [history]
   * @property {string|URL} [path]
   * @property {string} [basename]
   * @property {string} [stem]
   * @property {string} [extname]
   * @property {string} [dirname]
   * @property {VFileData} [data]
   *
   * @typedef Map
   *   Raw source map, see:
   *   <https://github.com/mozilla/source-map/blob/58819f0/source-map.d.ts#L15-L23>.
   * @property {number} version
   * @property {Array<string>} sources
   * @property {Array<string>} names
   * @property {string|undefined} [sourceRoot]
   * @property {Array<string>|undefined} [sourcesContent]
   * @property {string} mappings
   * @property {string} file
   *
   * @typedef {{[key: string]: unknown} & VFileCoreOptions} VFileOptions
   *   Configuration: a bunch of keys that will be shallow copied over to the new
   *   file.
   *
   * @typedef {Record<string, unknown>} VFileReporterSettings
   * @typedef {<T = VFileReporterSettings>(files: Array<VFile>, options: T) => string} VFileReporter
   */

  // Order of setting (least specific to most), we need this because otherwise
  // `{stem: 'a', path: '~/b.js'}` would throw, as a path is needed before a
  // stem can be set.
  const order = ['history', 'path', 'basename', 'stem', 'extname', 'dirname'];

  class VFile {
    /**
     * Create a new virtual file.
     *
     * If `options` is `string` or `Buffer`, treats it as `{value: options}`.
     * If `options` is a `VFile`, shallow copies its data over to the new file.
     * All other given fields are set on the newly created `VFile`.
     *
     * Path related properties are set in the following order (least specific to
     * most specific): `history`, `path`, `basename`, `stem`, `extname`,
     * `dirname`.
     *
     * It’s not possible to set either `dirname` or `extname` without setting
     * either `history`, `path`, `basename`, or `stem` as well.
     *
     * @param {VFileCompatible} [value]
     */
    constructor(value) {
      /** @type {VFileOptions} */
      let options;

      if (!value) {
        options = {};
      } else if (typeof value === 'string' || isBuffer(value)) {
        // @ts-expect-error Looks like a buffer.
        options = {value};
      } else if (isUrl(value)) {
        options = {path: value};
      } else {
        // @ts-expect-error Looks like file or options.
        options = value;
      }

      /**
       * Place to store custom information.
       * It’s OK to store custom data directly on the file, moving it to `data`
       * gives a little more privacy.
       * @type {VFileData}
       */
      this.data = {};

      /**
       * List of messages associated with the file.
       * @type {Array<VFileMessage>}
       */
      this.messages = [];

      /**
       * List of file paths the file moved between.
       * @type {Array<string>}
       */
      this.history = [];

      /**
       * Base of `path`.
       * Defaults to `process.cwd()` (`/` in browsers).
       * @type {string}
       */
      this.cwd = proc.cwd();

      /* eslint-disable no-unused-expressions */
      /**
       * Raw value.
       * @type {VFileValue}
       */
      this.value;

      // The below are non-standard, they are “well-known”.
      // As in, used in several tools.

      /**
       * Whether a file was saved to disk.
       * This is used by vfile reporters.
       * @type {boolean}
       */
      this.stored;

      /**
       * Sometimes files have a non-string representation.
       * This can be stored in the `result` field.
       * One example is when turning markdown into React nodes.
       * This is used by unified to store non-string results.
       * @type {unknown}
       */
      this.result;

      /**
       * Sometimes files have a source map associated with them.
       * This can be stored in the `map` field.
       * This should be a `RawSourceMap` type from the `source-map` module.
       * @type {Map|undefined}
       */
      this.map;
      /* eslint-enable no-unused-expressions */

      // Set path related properties in the correct order.
      let index = -1;

      while (++index < order.length) {
        const prop = order[index];

        // Note: we specifically use `in` instead of `hasOwnProperty` to accept
        // `vfile`s too.
        if (prop in options && options[prop] !== undefined) {
          // @ts-expect-error: TS is confused by the different types for `history`.
          this[prop] = prop === 'history' ? [...options[prop]] : options[prop];
        }
      }

      /** @type {string} */
      let prop;

      // Set non-path related properties.
      for (prop in options) {
        // @ts-expect-error: fine to set other things.
        if (!order.includes(prop)) this[prop] = options[prop];
      }
    }

    /**
     * Access full path (`~/index.min.js`).
     *
     * @returns {string}
     */
    get path() {
      return this.history[this.history.length - 1]
    }

    /**
     * Set full path (`~/index.min.js`).
     * Cannot be nullified.
     *
     * @param {string|URL} path
     */
    set path(path) {
      if (isUrl(path)) {
        path = urlToPath(path);
      }

      assertNonEmpty(path, 'path');

      if (this.path !== path) {
        this.history.push(path);
      }
    }

    /**
     * Access parent path (`~`).
     */
    get dirname() {
      return typeof this.path === 'string' ? path.dirname(this.path) : undefined
    }

    /**
     * Set parent path (`~`).
     * Cannot be set if there's no `path` yet.
     */
    set dirname(dirname) {
      assertPath(this.basename, 'dirname');
      this.path = path.join(dirname || '', this.basename);
    }

    /**
     * Access basename (including extname) (`index.min.js`).
     */
    get basename() {
      return typeof this.path === 'string' ? path.basename(this.path) : undefined
    }

    /**
     * Set basename (`index.min.js`).
     * Cannot contain path separators.
     * Cannot be nullified either (use `file.path = file.dirname` instead).
     */
    set basename(basename) {
      assertNonEmpty(basename, 'basename');
      assertPart(basename, 'basename');
      this.path = path.join(this.dirname || '', basename);
    }

    /**
     * Access extname (including dot) (`.js`).
     */
    get extname() {
      return typeof this.path === 'string' ? path.extname(this.path) : undefined
    }

    /**
     * Set extname (including dot) (`.js`).
     * Cannot be set if there's no `path` yet and cannot contain path separators.
     */
    set extname(extname) {
      assertPart(extname, 'extname');
      assertPath(this.dirname, 'extname');

      if (extname) {
        if (extname.charCodeAt(0) !== 46 /* `.` */) {
          throw new Error('`extname` must start with `.`')
        }

        if (extname.includes('.', 1)) {
          throw new Error('`extname` cannot contain multiple dots')
        }
      }

      this.path = path.join(this.dirname, this.stem + (extname || ''));
    }

    /**
     * Access stem (w/o extname) (`index.min`).
     */
    get stem() {
      return typeof this.path === 'string'
        ? path.basename(this.path, this.extname)
        : undefined
    }

    /**
     * Set stem (w/o extname) (`index.min`).
     * Cannot be nullified, and cannot contain path separators.
     */
    set stem(stem) {
      assertNonEmpty(stem, 'stem');
      assertPart(stem, 'stem');
      this.path = path.join(this.dirname || '', stem + (this.extname || ''));
    }

    /**
     * Serialize the file.
     *
     * @param {BufferEncoding} [encoding='utf8'] If `file.value` is a buffer, `encoding` is used to serialize buffers.
     * @returns {string}
     */
    toString(encoding) {
      return (this.value || '').toString(encoding)
    }

    /**
     * Create a message and associates it w/ the file.
     *
     * @param {string|Error} reason Reason for message (`string` or `Error`). Uses the stack and message of the error if given.
     * @param {Node|NodeLike|Position|Point} [place] Place at which the message occurred in a file (`Node`, `Position`, or `Point`, optional).
     * @param {string} [origin] Place in code the message originates from (`string`, optional).
     * @returns {VFileMessage}
     */
    message(reason, place, origin) {
      const message = new VFileMessage(reason, place, origin);

      if (this.path) {
        message.name = this.path + ':' + message.name;
        message.file = this.path;
      }

      message.fatal = false;

      this.messages.push(message);

      return message
    }

    /**
     * Info: create a message, associate it with the file, and mark the fatality
     * as `null`.
     * Calls `message()` internally.
     *
     * @param {string|Error} reason Reason for message (`string` or `Error`). Uses the stack and message of the error if given.
     * @param {Node|NodeLike|Position|Point} [place] Place at which the message occurred in a file (`Node`, `Position`, or `Point`, optional).
     * @param {string} [origin] Place in code the message originates from (`string`, optional).
     * @returns {VFileMessage}
     */
    info(reason, place, origin) {
      const message = this.message(reason, place, origin);

      message.fatal = null;

      return message
    }

    /**
     * Fail: create a message, associate it with the file, mark the fatality as
     * `true`.
     * Note: fatal errors mean a file is no longer processable.
     * Calls `message()` internally.
     *
     * @param {string|Error} reason Reason for message (`string` or `Error`). Uses the stack and message of the error if given.
     * @param {Node|NodeLike|Position|Point} [place] Place at which the message occurred in a file (`Node`, `Position`, or `Point`, optional).
     * @param {string} [origin] Place in code the message originates from (`string`, optional).
     * @returns {never}
     */
    fail(reason, place, origin) {
      const message = this.message(reason, place, origin);

      message.fatal = true;

      throw message
    }
  }

  /**
   * Assert that `part` is not a path (as in, does not contain `path.sep`).
   *
   * @param {string|undefined} part
   * @param {string} name
   * @returns {void}
   */
  function assertPart(part, name) {
    if (part && part.includes(path.sep)) {
      throw new Error(
        '`' + name + '` cannot be a path: did not expect `' + path.sep + '`'
      )
    }
  }

  /**
   * Assert that `part` is not empty.
   *
   * @param {string|undefined} part
   * @param {string} name
   * @returns {asserts part is string}
   */
  function assertNonEmpty(part, name) {
    if (!part) {
      throw new Error('`' + name + '` cannot be empty')
    }
  }

  /**
   * Assert `path` exists.
   *
   * @param {string|undefined} path
   * @param {string} name
   * @returns {asserts path is string}
   */
  function assertPath(path, name) {
    if (!path) {
      throw new Error('Setting `' + name + '` requires `path` to be set too')
    }
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('vfile').VFileCompatible} VFileCompatible
   * @typedef {import('vfile').VFileValue} VFileValue
   * @typedef {import('..').Processor} Processor
   * @typedef {import('..').Plugin} Plugin
   * @typedef {import('..').Preset} Preset
   * @typedef {import('..').Pluggable} Pluggable
   * @typedef {import('..').PluggableList} PluggableList
   * @typedef {import('..').Transformer} Transformer
   * @typedef {import('..').Parser} Parser
   * @typedef {import('..').Compiler} Compiler
   * @typedef {import('..').RunCallback} RunCallback
   * @typedef {import('..').ProcessCallback} ProcessCallback
   *
   * @typedef Context
   * @property {Node} tree
   * @property {VFile} file
   */

  // Expose a frozen processor.
  const unified = base$1().freeze();

  const own$m = {}.hasOwnProperty;

  // Function to create the first processor.
  /**
   * @returns {Processor}
   */
  function base$1() {
    const transformers = trough();
    /** @type {Processor['attachers']} */
    const attachers = [];
    /** @type {Record<string, unknown>} */
    let namespace = {};
    /** @type {boolean|undefined} */
    let frozen;
    let freezeIndex = -1;

    // Data management.
    // @ts-expect-error: overloads are handled.
    processor.data = data;
    processor.Parser = undefined;
    processor.Compiler = undefined;

    // Lock.
    processor.freeze = freeze;

    // Plugins.
    processor.attachers = attachers;
    // @ts-expect-error: overloads are handled.
    processor.use = use;

    // API.
    processor.parse = parse;
    processor.stringify = stringify;
    // @ts-expect-error: overloads are handled.
    processor.run = run;
    processor.runSync = runSync;
    // @ts-expect-error: overloads are handled.
    processor.process = process;
    processor.processSync = processSync;

    // Expose.
    return processor

    // Create a new processor based on the processor in the current scope.
    /** @type {Processor} */
    function processor() {
      const destination = base$1();
      let index = -1;

      while (++index < attachers.length) {
        destination.use(...attachers[index]);
      }

      destination.data(extend(true, {}, namespace));

      return destination
    }

    /**
     * @param {string|Record<string, unknown>} [key]
     * @param {unknown} [value]
     * @returns {unknown}
     */
    function data(key, value) {
      if (typeof key === 'string') {
        // Set `key`.
        if (arguments.length === 2) {
          assertUnfrozen('data', frozen);
          namespace[key] = value;
          return processor
        }

        // Get `key`.
        return (own$m.call(namespace, key) && namespace[key]) || null
      }

      // Set space.
      if (key) {
        assertUnfrozen('data', frozen);
        namespace = key;
        return processor
      }

      // Get space.
      return namespace
    }

    /** @type {Processor['freeze']} */
    function freeze() {
      if (frozen) {
        return processor
      }

      while (++freezeIndex < attachers.length) {
        const [attacher, ...options] = attachers[freezeIndex];

        if (options[0] === false) {
          continue
        }

        if (options[0] === true) {
          options[0] = undefined;
        }

        /** @type {Transformer|void} */
        const transformer = attacher.call(processor, ...options);

        if (typeof transformer === 'function') {
          transformers.use(transformer);
        }
      }

      frozen = true;
      freezeIndex = Number.POSITIVE_INFINITY;

      return processor
    }

    /**
     * @param {Pluggable|null|undefined} [value]
     * @param {...unknown} options
     * @returns {Processor}
     */
    function use(value, ...options) {
      /** @type {Record<string, unknown>|undefined} */
      let settings;

      assertUnfrozen('use', frozen);

      if (value === null || value === undefined) ; else if (typeof value === 'function') {
        addPlugin(value, ...options);
      } else if (typeof value === 'object') {
        if (Array.isArray(value)) {
          addList(value);
        } else {
          addPreset(value);
        }
      } else {
        throw new TypeError('Expected usable value, not `' + value + '`')
      }

      if (settings) {
        namespace.settings = Object.assign(namespace.settings || {}, settings);
      }

      return processor

      /**
       * @param {import('..').Pluggable<unknown[]>} value
       * @returns {void}
       */
      function add(value) {
        if (typeof value === 'function') {
          addPlugin(value);
        } else if (typeof value === 'object') {
          if (Array.isArray(value)) {
            const [plugin, ...options] = value;
            addPlugin(plugin, ...options);
          } else {
            addPreset(value);
          }
        } else {
          throw new TypeError('Expected usable value, not `' + value + '`')
        }
      }

      /**
       * @param {Preset} result
       * @returns {void}
       */
      function addPreset(result) {
        addList(result.plugins);

        if (result.settings) {
          settings = Object.assign(settings || {}, result.settings);
        }
      }

      /**
       * @param {PluggableList|null|undefined} [plugins]
       * @returns {void}
       */
      function addList(plugins) {
        let index = -1;

        if (plugins === null || plugins === undefined) ; else if (Array.isArray(plugins)) {
          while (++index < plugins.length) {
            const thing = plugins[index];
            add(thing);
          }
        } else {
          throw new TypeError('Expected a list of plugins, not `' + plugins + '`')
        }
      }

      /**
       * @param {Plugin} plugin
       * @param {...unknown} [value]
       * @returns {void}
       */
      function addPlugin(plugin, value) {
        let index = -1;
        /** @type {Processor['attachers'][number]|undefined} */
        let entry;

        while (++index < attachers.length) {
          if (attachers[index][0] === plugin) {
            entry = attachers[index];
            break
          }
        }

        if (entry) {
          if (isPlainObject(entry[1]) && isPlainObject(value)) {
            value = extend(true, entry[1], value);
          }

          entry[1] = value;
        } else {
          // @ts-expect-error: fine.
          attachers.push([...arguments]);
        }
      }
    }

    /** @type {Processor['parse']} */
    function parse(doc) {
      processor.freeze();
      const file = vfile(doc);
      const Parser = processor.Parser;
      assertParser('parse', Parser);

      if (newable(Parser, 'parse')) {
        // @ts-expect-error: `newable` checks this.
        return new Parser(String(file), file).parse()
      }

      // @ts-expect-error: `newable` checks this.
      return Parser(String(file), file) // eslint-disable-line new-cap
    }

    /** @type {Processor['stringify']} */
    function stringify(node, doc) {
      processor.freeze();
      const file = vfile(doc);
      const Compiler = processor.Compiler;
      assertCompiler('stringify', Compiler);
      assertNode(node);

      if (newable(Compiler, 'compile')) {
        // @ts-expect-error: `newable` checks this.
        return new Compiler(node, file).compile()
      }

      // @ts-expect-error: `newable` checks this.
      return Compiler(node, file) // eslint-disable-line new-cap
    }

    /**
     * @param {Node} node
     * @param {VFileCompatible|RunCallback} [doc]
     * @param {RunCallback} [callback]
     * @returns {Promise<Node>|void}
     */
    function run(node, doc, callback) {
      assertNode(node);
      processor.freeze();

      if (!callback && typeof doc === 'function') {
        callback = doc;
        doc = undefined;
      }

      if (!callback) {
        return new Promise(executor)
      }

      executor(null, callback);

      /**
       * @param {null|((node: Node) => void)} resolve
       * @param {(error: Error) => void} reject
       * @returns {void}
       */
      function executor(resolve, reject) {
        // @ts-expect-error: `doc` can’t be a callback anymore, we checked.
        transformers.run(node, vfile(doc), done);

        /**
         * @param {Error|null} error
         * @param {Node} tree
         * @param {VFile} file
         * @returns {void}
         */
        function done(error, tree, file) {
          tree = tree || node;
          if (error) {
            reject(error);
          } else if (resolve) {
            resolve(tree);
          } else {
            // @ts-expect-error: `callback` is defined if `resolve` is not.
            callback(null, tree, file);
          }
        }
      }
    }

    /** @type {Processor['runSync']} */
    function runSync(node, file) {
      /** @type {Node|undefined} */
      let result;
      /** @type {boolean|undefined} */
      let complete;

      processor.run(node, file, done);

      assertDone('runSync', 'run', complete);

      // @ts-expect-error: we either bailed on an error or have a tree.
      return result

      /**
       * @param {Error|null} [error]
       * @param {Node} [tree]
       * @returns {void}
       */
      function done(error, tree) {
        bail(error);
        result = tree;
        complete = true;
      }
    }

    /**
     * @param {VFileCompatible} doc
     * @param {ProcessCallback} [callback]
     * @returns {Promise<VFile>|undefined}
     */
    function process(doc, callback) {
      processor.freeze();
      assertParser('process', processor.Parser);
      assertCompiler('process', processor.Compiler);

      if (!callback) {
        return new Promise(executor)
      }

      executor(null, callback);

      /**
       * @param {null|((file: VFile) => void)} resolve
       * @param {(error?: Error|null|undefined) => void} reject
       * @returns {void}
       */
      function executor(resolve, reject) {
        const file = vfile(doc);

        processor.run(processor.parse(file), file, (error, tree, file) => {
          if (error || !tree || !file) {
            done(error);
          } else {
            /** @type {unknown} */
            const result = processor.stringify(tree, file);

            if (result === undefined || result === null) ; else if (looksLikeAVFileValue(result)) {
              file.value = result;
            } else {
              file.result = result;
            }

            done(error, file);
          }
        });

        /**
         * @param {Error|null|undefined} [error]
         * @param {VFile|undefined} [file]
         * @returns {void}
         */
        function done(error, file) {
          if (error || !file) {
            reject(error);
          } else if (resolve) {
            resolve(file);
          } else {
            // @ts-expect-error: `callback` is defined if `resolve` is not.
            callback(null, file);
          }
        }
      }
    }

    /** @type {Processor['processSync']} */
    function processSync(doc) {
      /** @type {boolean|undefined} */
      let complete;

      processor.freeze();
      assertParser('processSync', processor.Parser);
      assertCompiler('processSync', processor.Compiler);

      const file = vfile(doc);

      processor.process(file, done);

      assertDone('processSync', 'process', complete);

      return file

      /**
       * @param {Error|null|undefined} [error]
       * @returns {void}
       */
      function done(error) {
        complete = true;
        bail(error);
      }
    }
  }

  /**
   * Check if `value` is a constructor.
   *
   * @param {unknown} value
   * @param {string} name
   * @returns {boolean}
   */
  function newable(value, name) {
    return (
      typeof value === 'function' &&
      // Prototypes do exist.
      // type-coverage:ignore-next-line
      value.prototype &&
      // A function with keys in its prototype is probably a constructor.
      // Classes’ prototype methods are not enumerable, so we check if some value
      // exists in the prototype.
      // type-coverage:ignore-next-line
      (keys(value.prototype) || name in value.prototype)
    )
  }

  /**
   * Check if `value` is an object with keys.
   *
   * @param {Record<string, unknown>} value
   * @returns {boolean}
   */
  function keys(value) {
    /** @type {string} */
    let key;

    for (key in value) {
      if (own$m.call(value, key)) {
        return true
      }
    }

    return false
  }

  /**
   * Assert a parser is available.
   *
   * @param {string} name
   * @param {unknown} value
   * @returns {asserts value is Parser}
   */
  function assertParser(name, value) {
    if (typeof value !== 'function') {
      throw new TypeError('Cannot `' + name + '` without `Parser`')
    }
  }

  /**
   * Assert a compiler is available.
   *
   * @param {string} name
   * @param {unknown} value
   * @returns {asserts value is Compiler}
   */
  function assertCompiler(name, value) {
    if (typeof value !== 'function') {
      throw new TypeError('Cannot `' + name + '` without `Compiler`')
    }
  }

  /**
   * Assert the processor is not frozen.
   *
   * @param {string} name
   * @param {unknown} frozen
   * @returns {asserts frozen is false}
   */
  function assertUnfrozen(name, frozen) {
    if (frozen) {
      throw new Error(
        'Cannot call `' +
          name +
          '` on a frozen processor.\nCreate a new processor first, by calling it: use `processor()` instead of `processor`.'
      )
    }
  }

  /**
   * Assert `node` is a unist node.
   *
   * @param {unknown} node
   * @returns {asserts node is Node}
   */
  function assertNode(node) {
    // `isPlainObj` unfortunately uses `any` instead of `unknown`.
    // type-coverage:ignore-next-line
    if (!isPlainObject(node) || typeof node.type !== 'string') {
      throw new TypeError('Expected node, got `' + node + '`')
      // Fine.
    }
  }

  /**
   * Assert that `complete` is `true`.
   *
   * @param {string} name
   * @param {string} asyncName
   * @param {unknown} complete
   * @returns {asserts complete is true}
   */
  function assertDone(name, asyncName, complete) {
    if (!complete) {
      throw new Error(
        '`' + name + '` finished async. Use `' + asyncName + '` instead'
      )
    }
  }

  /**
   * @param {VFileCompatible} [value]
   * @returns {VFile}
   */
  function vfile(value) {
    return looksLikeAVFile(value) ? value : new VFile(value)
  }

  /**
   * @param {VFileCompatible} [value]
   * @returns {value is VFile}
   */
  function looksLikeAVFile(value) {
    return Boolean(
      value &&
        typeof value === 'object' &&
        'message' in value &&
        'messages' in value
    )
  }

  /**
   * @param {unknown} [value]
   * @returns {value is VFileValue}
   */
  function looksLikeAVFileValue(value) {
    return typeof value === 'string' || isBuffer(value)
  }

  var unicode$7 = {};

  const UNDEFINED_CODE_POINTS$1 = [
      0xfffe,
      0xffff,
      0x1fffe,
      0x1ffff,
      0x2fffe,
      0x2ffff,
      0x3fffe,
      0x3ffff,
      0x4fffe,
      0x4ffff,
      0x5fffe,
      0x5ffff,
      0x6fffe,
      0x6ffff,
      0x7fffe,
      0x7ffff,
      0x8fffe,
      0x8ffff,
      0x9fffe,
      0x9ffff,
      0xafffe,
      0xaffff,
      0xbfffe,
      0xbffff,
      0xcfffe,
      0xcffff,
      0xdfffe,
      0xdffff,
      0xefffe,
      0xeffff,
      0xffffe,
      0xfffff,
      0x10fffe,
      0x10ffff
  ];

  unicode$7.REPLACEMENT_CHARACTER = '\uFFFD';

  unicode$7.CODE_POINTS = {
      EOF: -1,
      NULL: 0x00,
      TABULATION: 0x09,
      CARRIAGE_RETURN: 0x0d,
      LINE_FEED: 0x0a,
      FORM_FEED: 0x0c,
      SPACE: 0x20,
      EXCLAMATION_MARK: 0x21,
      QUOTATION_MARK: 0x22,
      NUMBER_SIGN: 0x23,
      AMPERSAND: 0x26,
      APOSTROPHE: 0x27,
      HYPHEN_MINUS: 0x2d,
      SOLIDUS: 0x2f,
      DIGIT_0: 0x30,
      DIGIT_9: 0x39,
      SEMICOLON: 0x3b,
      LESS_THAN_SIGN: 0x3c,
      EQUALS_SIGN: 0x3d,
      GREATER_THAN_SIGN: 0x3e,
      QUESTION_MARK: 0x3f,
      LATIN_CAPITAL_A: 0x41,
      LATIN_CAPITAL_F: 0x46,
      LATIN_CAPITAL_X: 0x58,
      LATIN_CAPITAL_Z: 0x5a,
      RIGHT_SQUARE_BRACKET: 0x5d,
      GRAVE_ACCENT: 0x60,
      LATIN_SMALL_A: 0x61,
      LATIN_SMALL_F: 0x66,
      LATIN_SMALL_X: 0x78,
      LATIN_SMALL_Z: 0x7a,
      REPLACEMENT_CHARACTER: 0xfffd
  };

  unicode$7.CODE_POINT_SEQUENCES = {
      DASH_DASH_STRING: [0x2d, 0x2d], //--
      DOCTYPE_STRING: [0x44, 0x4f, 0x43, 0x54, 0x59, 0x50, 0x45], //DOCTYPE
      CDATA_START_STRING: [0x5b, 0x43, 0x44, 0x41, 0x54, 0x41, 0x5b], //[CDATA[
      SCRIPT_STRING: [0x73, 0x63, 0x72, 0x69, 0x70, 0x74], //script
      PUBLIC_STRING: [0x50, 0x55, 0x42, 0x4c, 0x49, 0x43], //PUBLIC
      SYSTEM_STRING: [0x53, 0x59, 0x53, 0x54, 0x45, 0x4d] //SYSTEM
  };

  //Surrogates
  unicode$7.isSurrogate = function(cp) {
      return cp >= 0xd800 && cp <= 0xdfff;
  };

  unicode$7.isSurrogatePair = function(cp) {
      return cp >= 0xdc00 && cp <= 0xdfff;
  };

  unicode$7.getSurrogatePairCodePoint = function(cp1, cp2) {
      return (cp1 - 0xd800) * 0x400 + 0x2400 + cp2;
  };

  //NOTE: excluding NULL and ASCII whitespace
  unicode$7.isControlCodePoint = function(cp) {
      return (
          (cp !== 0x20 && cp !== 0x0a && cp !== 0x0d && cp !== 0x09 && cp !== 0x0c && cp >= 0x01 && cp <= 0x1f) ||
          (cp >= 0x7f && cp <= 0x9f)
      );
  };

  unicode$7.isUndefinedCodePoint = function(cp) {
      return (cp >= 0xfdd0 && cp <= 0xfdef) || UNDEFINED_CODE_POINTS$1.indexOf(cp) > -1;
  };

  var errorCodes$1 = {
      controlCharacterInInputStream: 'control-character-in-input-stream',
      noncharacterInInputStream: 'noncharacter-in-input-stream',
      surrogateInInputStream: 'surrogate-in-input-stream',
      nonVoidHtmlElementStartTagWithTrailingSolidus: 'non-void-html-element-start-tag-with-trailing-solidus',
      endTagWithAttributes: 'end-tag-with-attributes',
      endTagWithTrailingSolidus: 'end-tag-with-trailing-solidus',
      unexpectedSolidusInTag: 'unexpected-solidus-in-tag',
      unexpectedNullCharacter: 'unexpected-null-character',
      unexpectedQuestionMarkInsteadOfTagName: 'unexpected-question-mark-instead-of-tag-name',
      invalidFirstCharacterOfTagName: 'invalid-first-character-of-tag-name',
      unexpectedEqualsSignBeforeAttributeName: 'unexpected-equals-sign-before-attribute-name',
      missingEndTagName: 'missing-end-tag-name',
      unexpectedCharacterInAttributeName: 'unexpected-character-in-attribute-name',
      unknownNamedCharacterReference: 'unknown-named-character-reference',
      missingSemicolonAfterCharacterReference: 'missing-semicolon-after-character-reference',
      unexpectedCharacterAfterDoctypeSystemIdentifier: 'unexpected-character-after-doctype-system-identifier',
      unexpectedCharacterInUnquotedAttributeValue: 'unexpected-character-in-unquoted-attribute-value',
      eofBeforeTagName: 'eof-before-tag-name',
      eofInTag: 'eof-in-tag',
      missingAttributeValue: 'missing-attribute-value',
      missingWhitespaceBetweenAttributes: 'missing-whitespace-between-attributes',
      missingWhitespaceAfterDoctypePublicKeyword: 'missing-whitespace-after-doctype-public-keyword',
      missingWhitespaceBetweenDoctypePublicAndSystemIdentifiers:
          'missing-whitespace-between-doctype-public-and-system-identifiers',
      missingWhitespaceAfterDoctypeSystemKeyword: 'missing-whitespace-after-doctype-system-keyword',
      missingQuoteBeforeDoctypePublicIdentifier: 'missing-quote-before-doctype-public-identifier',
      missingQuoteBeforeDoctypeSystemIdentifier: 'missing-quote-before-doctype-system-identifier',
      missingDoctypePublicIdentifier: 'missing-doctype-public-identifier',
      missingDoctypeSystemIdentifier: 'missing-doctype-system-identifier',
      abruptDoctypePublicIdentifier: 'abrupt-doctype-public-identifier',
      abruptDoctypeSystemIdentifier: 'abrupt-doctype-system-identifier',
      cdataInHtmlContent: 'cdata-in-html-content',
      incorrectlyOpenedComment: 'incorrectly-opened-comment',
      eofInScriptHtmlCommentLikeText: 'eof-in-script-html-comment-like-text',
      eofInDoctype: 'eof-in-doctype',
      nestedComment: 'nested-comment',
      abruptClosingOfEmptyComment: 'abrupt-closing-of-empty-comment',
      eofInComment: 'eof-in-comment',
      incorrectlyClosedComment: 'incorrectly-closed-comment',
      eofInCdata: 'eof-in-cdata',
      absenceOfDigitsInNumericCharacterReference: 'absence-of-digits-in-numeric-character-reference',
      nullCharacterReference: 'null-character-reference',
      surrogateCharacterReference: 'surrogate-character-reference',
      characterReferenceOutsideUnicodeRange: 'character-reference-outside-unicode-range',
      controlCharacterReference: 'control-character-reference',
      noncharacterCharacterReference: 'noncharacter-character-reference',
      missingWhitespaceBeforeDoctypeName: 'missing-whitespace-before-doctype-name',
      missingDoctypeName: 'missing-doctype-name',
      invalidCharacterSequenceAfterDoctypeName: 'invalid-character-sequence-after-doctype-name',
      duplicateAttribute: 'duplicate-attribute',
      nonConformingDoctype: 'non-conforming-doctype',
      missingDoctype: 'missing-doctype',
      misplacedDoctype: 'misplaced-doctype',
      endTagWithoutMatchingOpenElement: 'end-tag-without-matching-open-element',
      closingOfElementWithOpenChildElements: 'closing-of-element-with-open-child-elements',
      disallowedContentInNoscriptInHead: 'disallowed-content-in-noscript-in-head',
      openElementsLeftAfterEof: 'open-elements-left-after-eof',
      abandonedHeadElementChild: 'abandoned-head-element-child',
      misplacedStartTagForHeadElement: 'misplaced-start-tag-for-head-element',
      nestedNoscriptInHead: 'nested-noscript-in-head',
      eofInElementThatCanContainOnlyText: 'eof-in-element-that-can-contain-only-text'
  };

  const unicode$6 = unicode$7;
  const ERR$5 = errorCodes$1;

  //Aliases
  const $$d = unicode$6.CODE_POINTS;

  //Const
  const DEFAULT_BUFFER_WATERLINE$1 = 1 << 16;

  //Preprocessor
  //NOTE: HTML input preprocessing
  //(see: http://www.whatwg.org/specs/web-apps/current-work/multipage/parsing.html#preprocessing-the-input-stream)
  class Preprocessor$3 {
      constructor() {
          this.html = null;

          this.pos = -1;
          this.lastGapPos = -1;
          this.lastCharPos = -1;

          this.gapStack = [];

          this.skipNextNewLine = false;

          this.lastChunkWritten = false;
          this.endOfChunkHit = false;
          this.bufferWaterline = DEFAULT_BUFFER_WATERLINE$1;
      }

      _err() {
          // NOTE: err reporting is noop by default. Enabled by mixin.
      }

      _addGap() {
          this.gapStack.push(this.lastGapPos);
          this.lastGapPos = this.pos;
      }

      _processSurrogate(cp) {
          //NOTE: try to peek a surrogate pair
          if (this.pos !== this.lastCharPos) {
              const nextCp = this.html.charCodeAt(this.pos + 1);

              if (unicode$6.isSurrogatePair(nextCp)) {
                  //NOTE: we have a surrogate pair. Peek pair character and recalculate code point.
                  this.pos++;

                  //NOTE: add gap that should be avoided during retreat
                  this._addGap();

                  return unicode$6.getSurrogatePairCodePoint(cp, nextCp);
              }
          }

          //NOTE: we are at the end of a chunk, therefore we can't infer surrogate pair yet.
          else if (!this.lastChunkWritten) {
              this.endOfChunkHit = true;
              return $$d.EOF;
          }

          //NOTE: isolated surrogate
          this._err(ERR$5.surrogateInInputStream);

          return cp;
      }

      dropParsedChunk() {
          if (this.pos > this.bufferWaterline) {
              this.lastCharPos -= this.pos;
              this.html = this.html.substring(this.pos);
              this.pos = 0;
              this.lastGapPos = -1;
              this.gapStack = [];
          }
      }

      write(chunk, isLastChunk) {
          if (this.html) {
              this.html += chunk;
          } else {
              this.html = chunk;
          }

          this.lastCharPos = this.html.length - 1;
          this.endOfChunkHit = false;
          this.lastChunkWritten = isLastChunk;
      }

      insertHtmlAtCurrentPos(chunk) {
          this.html = this.html.substring(0, this.pos + 1) + chunk + this.html.substring(this.pos + 1, this.html.length);

          this.lastCharPos = this.html.length - 1;
          this.endOfChunkHit = false;
      }

      advance() {
          this.pos++;

          if (this.pos > this.lastCharPos) {
              this.endOfChunkHit = !this.lastChunkWritten;
              return $$d.EOF;
          }

          let cp = this.html.charCodeAt(this.pos);

          //NOTE: any U+000A LINE FEED (LF) characters that immediately follow a U+000D CARRIAGE RETURN (CR) character
          //must be ignored.
          if (this.skipNextNewLine && cp === $$d.LINE_FEED) {
              this.skipNextNewLine = false;
              this._addGap();
              return this.advance();
          }

          //NOTE: all U+000D CARRIAGE RETURN (CR) characters must be converted to U+000A LINE FEED (LF) characters
          if (cp === $$d.CARRIAGE_RETURN) {
              this.skipNextNewLine = true;
              return $$d.LINE_FEED;
          }

          this.skipNextNewLine = false;

          if (unicode$6.isSurrogate(cp)) {
              cp = this._processSurrogate(cp);
          }

          //OPTIMIZATION: first check if code point is in the common allowed
          //range (ASCII alphanumeric, whitespaces, big chunk of BMP)
          //before going into detailed performance cost validation.
          const isCommonValidRange =
              (cp > 0x1f && cp < 0x7f) || cp === $$d.LINE_FEED || cp === $$d.CARRIAGE_RETURN || (cp > 0x9f && cp < 0xfdd0);

          if (!isCommonValidRange) {
              this._checkForProblematicCharacters(cp);
          }

          return cp;
      }

      _checkForProblematicCharacters(cp) {
          if (unicode$6.isControlCodePoint(cp)) {
              this._err(ERR$5.controlCharacterInInputStream);
          } else if (unicode$6.isUndefinedCodePoint(cp)) {
              this._err(ERR$5.noncharacterInInputStream);
          }
      }

      retreat() {
          if (this.pos === this.lastGapPos) {
              this.lastGapPos = this.gapStack.pop();
              this.pos--;
          }

          this.pos--;
      }
  }

  var preprocessor$1 = Preprocessor$3;

  //NOTE: this file contains auto-generated array mapped radix tree that is used for the named entity references consumption
  //(details: https://github.com/inikulin/parse5/tree/master/scripts/generate-named-entity-data/README.md)
  var namedEntityData$1 = new Uint16Array([4,52,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,106,303,412,810,1432,1701,1796,1987,2114,2360,2420,2484,3170,3251,4140,4393,4575,4610,5106,5512,5728,6117,6274,6315,6345,6427,6516,7002,7910,8733,9323,9870,10170,10631,10893,11318,11386,11467,12773,13092,14474,14922,15448,15542,16419,17666,18166,18611,19004,19095,19298,19397,4,16,69,77,97,98,99,102,103,108,109,110,111,112,114,115,116,117,140,150,158,169,176,194,199,210,216,222,226,242,256,266,283,294,108,105,103,5,198,1,59,148,1,198,80,5,38,1,59,156,1,38,99,117,116,101,5,193,1,59,167,1,193,114,101,118,101,59,1,258,4,2,105,121,182,191,114,99,5,194,1,59,189,1,194,59,1,1040,114,59,3,55349,56580,114,97,118,101,5,192,1,59,208,1,192,112,104,97,59,1,913,97,99,114,59,1,256,100,59,1,10835,4,2,103,112,232,237,111,110,59,1,260,102,59,3,55349,56632,112,108,121,70,117,110,99,116,105,111,110,59,1,8289,105,110,103,5,197,1,59,264,1,197,4,2,99,115,272,277,114,59,3,55349,56476,105,103,110,59,1,8788,105,108,100,101,5,195,1,59,292,1,195,109,108,5,196,1,59,301,1,196,4,8,97,99,101,102,111,114,115,117,321,350,354,383,388,394,400,405,4,2,99,114,327,336,107,115,108,97,115,104,59,1,8726,4,2,118,119,342,345,59,1,10983,101,100,59,1,8966,121,59,1,1041,4,3,99,114,116,362,369,379,97,117,115,101,59,1,8757,110,111,117,108,108,105,115,59,1,8492,97,59,1,914,114,59,3,55349,56581,112,102,59,3,55349,56633,101,118,101,59,1,728,99,114,59,1,8492,109,112,101,113,59,1,8782,4,14,72,79,97,99,100,101,102,104,105,108,111,114,115,117,442,447,456,504,542,547,569,573,577,616,678,784,790,796,99,121,59,1,1063,80,89,5,169,1,59,454,1,169,4,3,99,112,121,464,470,497,117,116,101,59,1,262,4,2,59,105,476,478,1,8914,116,97,108,68,105,102,102,101,114,101,110,116,105,97,108,68,59,1,8517,108,101,121,115,59,1,8493,4,4,97,101,105,111,514,520,530,535,114,111,110,59,1,268,100,105,108,5,199,1,59,528,1,199,114,99,59,1,264,110,105,110,116,59,1,8752,111,116,59,1,266,4,2,100,110,553,560,105,108,108,97,59,1,184,116,101,114,68,111,116,59,1,183,114,59,1,8493,105,59,1,935,114,99,108,101,4,4,68,77,80,84,591,596,603,609,111,116,59,1,8857,105,110,117,115,59,1,8854,108,117,115,59,1,8853,105,109,101,115,59,1,8855,111,4,2,99,115,623,646,107,119,105,115,101,67,111,110,116,111,117,114,73,110,116,101,103,114,97,108,59,1,8754,101,67,117,114,108,121,4,2,68,81,658,671,111,117,98,108,101,81,117,111,116,101,59,1,8221,117,111,116,101,59,1,8217,4,4,108,110,112,117,688,701,736,753,111,110,4,2,59,101,696,698,1,8759,59,1,10868,4,3,103,105,116,709,717,722,114,117,101,110,116,59,1,8801,110,116,59,1,8751,111,117,114,73,110,116,101,103,114,97,108,59,1,8750,4,2,102,114,742,745,59,1,8450,111,100,117,99,116,59,1,8720,110,116,101,114,67,108,111,99,107,119,105,115,101,67,111,110,116,111,117,114,73,110,116,101,103,114,97,108,59,1,8755,111,115,115,59,1,10799,99,114,59,3,55349,56478,112,4,2,59,67,803,805,1,8915,97,112,59,1,8781,4,11,68,74,83,90,97,99,101,102,105,111,115,834,850,855,860,865,888,903,916,921,1011,1415,4,2,59,111,840,842,1,8517,116,114,97,104,100,59,1,10513,99,121,59,1,1026,99,121,59,1,1029,99,121,59,1,1039,4,3,103,114,115,873,879,883,103,101,114,59,1,8225,114,59,1,8609,104,118,59,1,10980,4,2,97,121,894,900,114,111,110,59,1,270,59,1,1044,108,4,2,59,116,910,912,1,8711,97,59,1,916,114,59,3,55349,56583,4,2,97,102,927,998,4,2,99,109,933,992,114,105,116,105,99,97,108,4,4,65,68,71,84,950,957,978,985,99,117,116,101,59,1,180,111,4,2,116,117,964,967,59,1,729,98,108,101,65,99,117,116,101,59,1,733,114,97,118,101,59,1,96,105,108,100,101,59,1,732,111,110,100,59,1,8900,102,101,114,101,110,116,105,97,108,68,59,1,8518,4,4,112,116,117,119,1021,1026,1048,1249,102,59,3,55349,56635,4,3,59,68,69,1034,1036,1041,1,168,111,116,59,1,8412,113,117,97,108,59,1,8784,98,108,101,4,6,67,68,76,82,85,86,1065,1082,1101,1189,1211,1236,111,110,116,111,117,114,73,110,116,101,103,114,97,108,59,1,8751,111,4,2,116,119,1089,1092,59,1,168,110,65,114,114,111,119,59,1,8659,4,2,101,111,1107,1141,102,116,4,3,65,82,84,1117,1124,1136,114,114,111,119,59,1,8656,105,103,104,116,65,114,114,111,119,59,1,8660,101,101,59,1,10980,110,103,4,2,76,82,1149,1177,101,102,116,4,2,65,82,1158,1165,114,114,111,119,59,1,10232,105,103,104,116,65,114,114,111,119,59,1,10234,105,103,104,116,65,114,114,111,119,59,1,10233,105,103,104,116,4,2,65,84,1199,1206,114,114,111,119,59,1,8658,101,101,59,1,8872,112,4,2,65,68,1218,1225,114,114,111,119,59,1,8657,111,119,110,65,114,114,111,119,59,1,8661,101,114,116,105,99,97,108,66,97,114,59,1,8741,110,4,6,65,66,76,82,84,97,1264,1292,1299,1352,1391,1408,114,114,111,119,4,3,59,66,85,1276,1278,1283,1,8595,97,114,59,1,10515,112,65,114,114,111,119,59,1,8693,114,101,118,101,59,1,785,101,102,116,4,3,82,84,86,1310,1323,1334,105,103,104,116,86,101,99,116,111,114,59,1,10576,101,101,86,101,99,116,111,114,59,1,10590,101,99,116,111,114,4,2,59,66,1345,1347,1,8637,97,114,59,1,10582,105,103,104,116,4,2,84,86,1362,1373,101,101,86,101,99,116,111,114,59,1,10591,101,99,116,111,114,4,2,59,66,1384,1386,1,8641,97,114,59,1,10583,101,101,4,2,59,65,1399,1401,1,8868,114,114,111,119,59,1,8615,114,114,111,119,59,1,8659,4,2,99,116,1421,1426,114,59,3,55349,56479,114,111,107,59,1,272,4,16,78,84,97,99,100,102,103,108,109,111,112,113,115,116,117,120,1466,1470,1478,1489,1515,1520,1525,1536,1544,1593,1609,1617,1650,1664,1668,1677,71,59,1,330,72,5,208,1,59,1476,1,208,99,117,116,101,5,201,1,59,1487,1,201,4,3,97,105,121,1497,1503,1512,114,111,110,59,1,282,114,99,5,202,1,59,1510,1,202,59,1,1069,111,116,59,1,278,114,59,3,55349,56584,114,97,118,101,5,200,1,59,1534,1,200,101,109,101,110,116,59,1,8712,4,2,97,112,1550,1555,99,114,59,1,274,116,121,4,2,83,86,1563,1576,109,97,108,108,83,113,117,97,114,101,59,1,9723,101,114,121,83,109,97,108,108,83,113,117,97,114,101,59,1,9643,4,2,103,112,1599,1604,111,110,59,1,280,102,59,3,55349,56636,115,105,108,111,110,59,1,917,117,4,2,97,105,1624,1640,108,4,2,59,84,1631,1633,1,10869,105,108,100,101,59,1,8770,108,105,98,114,105,117,109,59,1,8652,4,2,99,105,1656,1660,114,59,1,8496,109,59,1,10867,97,59,1,919,109,108,5,203,1,59,1675,1,203,4,2,105,112,1683,1689,115,116,115,59,1,8707,111,110,101,110,116,105,97,108,69,59,1,8519,4,5,99,102,105,111,115,1713,1717,1722,1762,1791,121,59,1,1060,114,59,3,55349,56585,108,108,101,100,4,2,83,86,1732,1745,109,97,108,108,83,113,117,97,114,101,59,1,9724,101,114,121,83,109,97,108,108,83,113,117,97,114,101,59,1,9642,4,3,112,114,117,1770,1775,1781,102,59,3,55349,56637,65,108,108,59,1,8704,114,105,101,114,116,114,102,59,1,8497,99,114,59,1,8497,4,12,74,84,97,98,99,100,102,103,111,114,115,116,1822,1827,1834,1848,1855,1877,1882,1887,1890,1896,1978,1984,99,121,59,1,1027,5,62,1,59,1832,1,62,109,109,97,4,2,59,100,1843,1845,1,915,59,1,988,114,101,118,101,59,1,286,4,3,101,105,121,1863,1869,1874,100,105,108,59,1,290,114,99,59,1,284,59,1,1043,111,116,59,1,288,114,59,3,55349,56586,59,1,8921,112,102,59,3,55349,56638,101,97,116,101,114,4,6,69,70,71,76,83,84,1915,1933,1944,1953,1959,1971,113,117,97,108,4,2,59,76,1925,1927,1,8805,101,115,115,59,1,8923,117,108,108,69,113,117,97,108,59,1,8807,114,101,97,116,101,114,59,1,10914,101,115,115,59,1,8823,108,97,110,116,69,113,117,97,108,59,1,10878,105,108,100,101,59,1,8819,99,114,59,3,55349,56482,59,1,8811,4,8,65,97,99,102,105,111,115,117,2005,2012,2026,2032,2036,2049,2073,2089,82,68,99,121,59,1,1066,4,2,99,116,2018,2023,101,107,59,1,711,59,1,94,105,114,99,59,1,292,114,59,1,8460,108,98,101,114,116,83,112,97,99,101,59,1,8459,4,2,112,114,2055,2059,102,59,1,8461,105,122,111,110,116,97,108,76,105,110,101,59,1,9472,4,2,99,116,2079,2083,114,59,1,8459,114,111,107,59,1,294,109,112,4,2,68,69,2097,2107,111,119,110,72,117,109,112,59,1,8782,113,117,97,108,59,1,8783,4,14,69,74,79,97,99,100,102,103,109,110,111,115,116,117,2144,2149,2155,2160,2171,2189,2194,2198,2209,2245,2307,2329,2334,2341,99,121,59,1,1045,108,105,103,59,1,306,99,121,59,1,1025,99,117,116,101,5,205,1,59,2169,1,205,4,2,105,121,2177,2186,114,99,5,206,1,59,2184,1,206,59,1,1048,111,116,59,1,304,114,59,1,8465,114,97,118,101,5,204,1,59,2207,1,204,4,3,59,97,112,2217,2219,2238,1,8465,4,2,99,103,2225,2229,114,59,1,298,105,110,97,114,121,73,59,1,8520,108,105,101,115,59,1,8658,4,2,116,118,2251,2281,4,2,59,101,2257,2259,1,8748,4,2,103,114,2265,2271,114,97,108,59,1,8747,115,101,99,116,105,111,110,59,1,8898,105,115,105,98,108,101,4,2,67,84,2293,2300,111,109,109,97,59,1,8291,105,109,101,115,59,1,8290,4,3,103,112,116,2315,2320,2325,111,110,59,1,302,102,59,3,55349,56640,97,59,1,921,99,114,59,1,8464,105,108,100,101,59,1,296,4,2,107,109,2347,2352,99,121,59,1,1030,108,5,207,1,59,2358,1,207,4,5,99,102,111,115,117,2372,2386,2391,2397,2414,4,2,105,121,2378,2383,114,99,59,1,308,59,1,1049,114,59,3,55349,56589,112,102,59,3,55349,56641,4,2,99,101,2403,2408,114,59,3,55349,56485,114,99,121,59,1,1032,107,99,121,59,1,1028,4,7,72,74,97,99,102,111,115,2436,2441,2446,2452,2467,2472,2478,99,121,59,1,1061,99,121,59,1,1036,112,112,97,59,1,922,4,2,101,121,2458,2464,100,105,108,59,1,310,59,1,1050,114,59,3,55349,56590,112,102,59,3,55349,56642,99,114,59,3,55349,56486,4,11,74,84,97,99,101,102,108,109,111,115,116,2508,2513,2520,2562,2585,2981,2986,3004,3011,3146,3167,99,121,59,1,1033,5,60,1,59,2518,1,60,4,5,99,109,110,112,114,2532,2538,2544,2548,2558,117,116,101,59,1,313,98,100,97,59,1,923,103,59,1,10218,108,97,99,101,116,114,102,59,1,8466,114,59,1,8606,4,3,97,101,121,2570,2576,2582,114,111,110,59,1,317,100,105,108,59,1,315,59,1,1051,4,2,102,115,2591,2907,116,4,10,65,67,68,70,82,84,85,86,97,114,2614,2663,2672,2728,2735,2760,2820,2870,2888,2895,4,2,110,114,2620,2633,103,108,101,66,114,97,99,107,101,116,59,1,10216,114,111,119,4,3,59,66,82,2644,2646,2651,1,8592,97,114,59,1,8676,105,103,104,116,65,114,114,111,119,59,1,8646,101,105,108,105,110,103,59,1,8968,111,4,2,117,119,2679,2692,98,108,101,66,114,97,99,107,101,116,59,1,10214,110,4,2,84,86,2699,2710,101,101,86,101,99,116,111,114,59,1,10593,101,99,116,111,114,4,2,59,66,2721,2723,1,8643,97,114,59,1,10585,108,111,111,114,59,1,8970,105,103,104,116,4,2,65,86,2745,2752,114,114,111,119,59,1,8596,101,99,116,111,114,59,1,10574,4,2,101,114,2766,2792,101,4,3,59,65,86,2775,2777,2784,1,8867,114,114,111,119,59,1,8612,101,99,116,111,114,59,1,10586,105,97,110,103,108,101,4,3,59,66,69,2806,2808,2813,1,8882,97,114,59,1,10703,113,117,97,108,59,1,8884,112,4,3,68,84,86,2829,2841,2852,111,119,110,86,101,99,116,111,114,59,1,10577,101,101,86,101,99,116,111,114,59,1,10592,101,99,116,111,114,4,2,59,66,2863,2865,1,8639,97,114,59,1,10584,101,99,116,111,114,4,2,59,66,2881,2883,1,8636,97,114,59,1,10578,114,114,111,119,59,1,8656,105,103,104,116,97,114,114,111,119,59,1,8660,115,4,6,69,70,71,76,83,84,2922,2936,2947,2956,2962,2974,113,117,97,108,71,114,101,97,116,101,114,59,1,8922,117,108,108,69,113,117,97,108,59,1,8806,114,101,97,116,101,114,59,1,8822,101,115,115,59,1,10913,108,97,110,116,69,113,117,97,108,59,1,10877,105,108,100,101,59,1,8818,114,59,3,55349,56591,4,2,59,101,2992,2994,1,8920,102,116,97,114,114,111,119,59,1,8666,105,100,111,116,59,1,319,4,3,110,112,119,3019,3110,3115,103,4,4,76,82,108,114,3030,3058,3070,3098,101,102,116,4,2,65,82,3039,3046,114,114,111,119,59,1,10229,105,103,104,116,65,114,114,111,119,59,1,10231,105,103,104,116,65,114,114,111,119,59,1,10230,101,102,116,4,2,97,114,3079,3086,114,114,111,119,59,1,10232,105,103,104,116,97,114,114,111,119,59,1,10234,105,103,104,116,97,114,114,111,119,59,1,10233,102,59,3,55349,56643,101,114,4,2,76,82,3123,3134,101,102,116,65,114,114,111,119,59,1,8601,105,103,104,116,65,114,114,111,119,59,1,8600,4,3,99,104,116,3154,3158,3161,114,59,1,8466,59,1,8624,114,111,107,59,1,321,59,1,8810,4,8,97,99,101,102,105,111,115,117,3188,3192,3196,3222,3227,3237,3243,3248,112,59,1,10501,121,59,1,1052,4,2,100,108,3202,3213,105,117,109,83,112,97,99,101,59,1,8287,108,105,110,116,114,102,59,1,8499,114,59,3,55349,56592,110,117,115,80,108,117,115,59,1,8723,112,102,59,3,55349,56644,99,114,59,1,8499,59,1,924,4,9,74,97,99,101,102,111,115,116,117,3271,3276,3283,3306,3422,3427,4120,4126,4137,99,121,59,1,1034,99,117,116,101,59,1,323,4,3,97,101,121,3291,3297,3303,114,111,110,59,1,327,100,105,108,59,1,325,59,1,1053,4,3,103,115,119,3314,3380,3415,97,116,105,118,101,4,3,77,84,86,3327,3340,3365,101,100,105,117,109,83,112,97,99,101,59,1,8203,104,105,4,2,99,110,3348,3357,107,83,112,97,99,101,59,1,8203,83,112,97,99,101,59,1,8203,101,114,121,84,104,105,110,83,112,97,99,101,59,1,8203,116,101,100,4,2,71,76,3389,3405,114,101,97,116,101,114,71,114,101,97,116,101,114,59,1,8811,101,115,115,76,101,115,115,59,1,8810,76,105,110,101,59,1,10,114,59,3,55349,56593,4,4,66,110,112,116,3437,3444,3460,3464,114,101,97,107,59,1,8288,66,114,101,97,107,105,110,103,83,112,97,99,101,59,1,160,102,59,1,8469,4,13,59,67,68,69,71,72,76,78,80,82,83,84,86,3492,3494,3517,3536,3578,3657,3685,3784,3823,3860,3915,4066,4107,1,10988,4,2,111,117,3500,3510,110,103,114,117,101,110,116,59,1,8802,112,67,97,112,59,1,8813,111,117,98,108,101,86,101,114,116,105,99,97,108,66,97,114,59,1,8742,4,3,108,113,120,3544,3552,3571,101,109,101,110,116,59,1,8713,117,97,108,4,2,59,84,3561,3563,1,8800,105,108,100,101,59,3,8770,824,105,115,116,115,59,1,8708,114,101,97,116,101,114,4,7,59,69,70,71,76,83,84,3600,3602,3609,3621,3631,3637,3650,1,8815,113,117,97,108,59,1,8817,117,108,108,69,113,117,97,108,59,3,8807,824,114,101,97,116,101,114,59,3,8811,824,101,115,115,59,1,8825,108,97,110,116,69,113,117,97,108,59,3,10878,824,105,108,100,101,59,1,8821,117,109,112,4,2,68,69,3666,3677,111,119,110,72,117,109,112,59,3,8782,824,113,117,97,108,59,3,8783,824,101,4,2,102,115,3692,3724,116,84,114,105,97,110,103,108,101,4,3,59,66,69,3709,3711,3717,1,8938,97,114,59,3,10703,824,113,117,97,108,59,1,8940,115,4,6,59,69,71,76,83,84,3739,3741,3748,3757,3764,3777,1,8814,113,117,97,108,59,1,8816,114,101,97,116,101,114,59,1,8824,101,115,115,59,3,8810,824,108,97,110,116,69,113,117,97,108,59,3,10877,824,105,108,100,101,59,1,8820,101,115,116,101,100,4,2,71,76,3795,3812,114,101,97,116,101,114,71,114,101,97,116,101,114,59,3,10914,824,101,115,115,76,101,115,115,59,3,10913,824,114,101,99,101,100,101,115,4,3,59,69,83,3838,3840,3848,1,8832,113,117,97,108,59,3,10927,824,108,97,110,116,69,113,117,97,108,59,1,8928,4,2,101,105,3866,3881,118,101,114,115,101,69,108,101,109,101,110,116,59,1,8716,103,104,116,84,114,105,97,110,103,108,101,4,3,59,66,69,3900,3902,3908,1,8939,97,114,59,3,10704,824,113,117,97,108,59,1,8941,4,2,113,117,3921,3973,117,97,114,101,83,117,4,2,98,112,3933,3952,115,101,116,4,2,59,69,3942,3945,3,8847,824,113,117,97,108,59,1,8930,101,114,115,101,116,4,2,59,69,3963,3966,3,8848,824,113,117,97,108,59,1,8931,4,3,98,99,112,3981,4000,4045,115,101,116,4,2,59,69,3990,3993,3,8834,8402,113,117,97,108,59,1,8840,99,101,101,100,115,4,4,59,69,83,84,4015,4017,4025,4037,1,8833,113,117,97,108,59,3,10928,824,108,97,110,116,69,113,117,97,108,59,1,8929,105,108,100,101,59,3,8831,824,101,114,115,101,116,4,2,59,69,4056,4059,3,8835,8402,113,117,97,108,59,1,8841,105,108,100,101,4,4,59,69,70,84,4080,4082,4089,4100,1,8769,113,117,97,108,59,1,8772,117,108,108,69,113,117,97,108,59,1,8775,105,108,100,101,59,1,8777,101,114,116,105,99,97,108,66,97,114,59,1,8740,99,114,59,3,55349,56489,105,108,100,101,5,209,1,59,4135,1,209,59,1,925,4,14,69,97,99,100,102,103,109,111,112,114,115,116,117,118,4170,4176,4187,4205,4212,4217,4228,4253,4259,4292,4295,4316,4337,4346,108,105,103,59,1,338,99,117,116,101,5,211,1,59,4185,1,211,4,2,105,121,4193,4202,114,99,5,212,1,59,4200,1,212,59,1,1054,98,108,97,99,59,1,336,114,59,3,55349,56594,114,97,118,101,5,210,1,59,4226,1,210,4,3,97,101,105,4236,4241,4246,99,114,59,1,332,103,97,59,1,937,99,114,111,110,59,1,927,112,102,59,3,55349,56646,101,110,67,117,114,108,121,4,2,68,81,4272,4285,111,117,98,108,101,81,117,111,116,101,59,1,8220,117,111,116,101,59,1,8216,59,1,10836,4,2,99,108,4301,4306,114,59,3,55349,56490,97,115,104,5,216,1,59,4314,1,216,105,4,2,108,109,4323,4332,100,101,5,213,1,59,4330,1,213,101,115,59,1,10807,109,108,5,214,1,59,4344,1,214,101,114,4,2,66,80,4354,4380,4,2,97,114,4360,4364,114,59,1,8254,97,99,4,2,101,107,4372,4375,59,1,9182,101,116,59,1,9140,97,114,101,110,116,104,101,115,105,115,59,1,9180,4,9,97,99,102,104,105,108,111,114,115,4413,4422,4426,4431,4435,4438,4448,4471,4561,114,116,105,97,108,68,59,1,8706,121,59,1,1055,114,59,3,55349,56595,105,59,1,934,59,1,928,117,115,77,105,110,117,115,59,1,177,4,2,105,112,4454,4467,110,99,97,114,101,112,108,97,110,101,59,1,8460,102,59,1,8473,4,4,59,101,105,111,4481,4483,4526,4531,1,10939,99,101,100,101,115,4,4,59,69,83,84,4498,4500,4507,4519,1,8826,113,117,97,108,59,1,10927,108,97,110,116,69,113,117,97,108,59,1,8828,105,108,100,101,59,1,8830,109,101,59,1,8243,4,2,100,112,4537,4543,117,99,116,59,1,8719,111,114,116,105,111,110,4,2,59,97,4555,4557,1,8759,108,59,1,8733,4,2,99,105,4567,4572,114,59,3,55349,56491,59,1,936,4,4,85,102,111,115,4585,4594,4599,4604,79,84,5,34,1,59,4592,1,34,114,59,3,55349,56596,112,102,59,1,8474,99,114,59,3,55349,56492,4,12,66,69,97,99,101,102,104,105,111,114,115,117,4636,4642,4650,4681,4704,4763,4767,4771,5047,5069,5081,5094,97,114,114,59,1,10512,71,5,174,1,59,4648,1,174,4,3,99,110,114,4658,4664,4668,117,116,101,59,1,340,103,59,1,10219,114,4,2,59,116,4675,4677,1,8608,108,59,1,10518,4,3,97,101,121,4689,4695,4701,114,111,110,59,1,344,100,105,108,59,1,342,59,1,1056,4,2,59,118,4710,4712,1,8476,101,114,115,101,4,2,69,85,4722,4748,4,2,108,113,4728,4736,101,109,101,110,116,59,1,8715,117,105,108,105,98,114,105,117,109,59,1,8651,112,69,113,117,105,108,105,98,114,105,117,109,59,1,10607,114,59,1,8476,111,59,1,929,103,104,116,4,8,65,67,68,70,84,85,86,97,4792,4840,4849,4905,4912,4972,5022,5040,4,2,110,114,4798,4811,103,108,101,66,114,97,99,107,101,116,59,1,10217,114,111,119,4,3,59,66,76,4822,4824,4829,1,8594,97,114,59,1,8677,101,102,116,65,114,114,111,119,59,1,8644,101,105,108,105,110,103,59,1,8969,111,4,2,117,119,4856,4869,98,108,101,66,114,97,99,107,101,116,59,1,10215,110,4,2,84,86,4876,4887,101,101,86,101,99,116,111,114,59,1,10589,101,99,116,111,114,4,2,59,66,4898,4900,1,8642,97,114,59,1,10581,108,111,111,114,59,1,8971,4,2,101,114,4918,4944,101,4,3,59,65,86,4927,4929,4936,1,8866,114,114,111,119,59,1,8614,101,99,116,111,114,59,1,10587,105,97,110,103,108,101,4,3,59,66,69,4958,4960,4965,1,8883,97,114,59,1,10704,113,117,97,108,59,1,8885,112,4,3,68,84,86,4981,4993,5004,111,119,110,86,101,99,116,111,114,59,1,10575,101,101,86,101,99,116,111,114,59,1,10588,101,99,116,111,114,4,2,59,66,5015,5017,1,8638,97,114,59,1,10580,101,99,116,111,114,4,2,59,66,5033,5035,1,8640,97,114,59,1,10579,114,114,111,119,59,1,8658,4,2,112,117,5053,5057,102,59,1,8477,110,100,73,109,112,108,105,101,115,59,1,10608,105,103,104,116,97,114,114,111,119,59,1,8667,4,2,99,104,5087,5091,114,59,1,8475,59,1,8625,108,101,68,101,108,97,121,101,100,59,1,10740,4,13,72,79,97,99,102,104,105,109,111,113,115,116,117,5134,5150,5157,5164,5198,5203,5259,5265,5277,5283,5374,5380,5385,4,2,67,99,5140,5146,72,99,121,59,1,1065,121,59,1,1064,70,84,99,121,59,1,1068,99,117,116,101,59,1,346,4,5,59,97,101,105,121,5176,5178,5184,5190,5195,1,10940,114,111,110,59,1,352,100,105,108,59,1,350,114,99,59,1,348,59,1,1057,114,59,3,55349,56598,111,114,116,4,4,68,76,82,85,5216,5227,5238,5250,111,119,110,65,114,114,111,119,59,1,8595,101,102,116,65,114,114,111,119,59,1,8592,105,103,104,116,65,114,114,111,119,59,1,8594,112,65,114,114,111,119,59,1,8593,103,109,97,59,1,931,97,108,108,67,105,114,99,108,101,59,1,8728,112,102,59,3,55349,56650,4,2,114,117,5289,5293,116,59,1,8730,97,114,101,4,4,59,73,83,85,5306,5308,5322,5367,1,9633,110,116,101,114,115,101,99,116,105,111,110,59,1,8851,117,4,2,98,112,5329,5347,115,101,116,4,2,59,69,5338,5340,1,8847,113,117,97,108,59,1,8849,101,114,115,101,116,4,2,59,69,5358,5360,1,8848,113,117,97,108,59,1,8850,110,105,111,110,59,1,8852,99,114,59,3,55349,56494,97,114,59,1,8902,4,4,98,99,109,112,5395,5420,5475,5478,4,2,59,115,5401,5403,1,8912,101,116,4,2,59,69,5411,5413,1,8912,113,117,97,108,59,1,8838,4,2,99,104,5426,5468,101,101,100,115,4,4,59,69,83,84,5440,5442,5449,5461,1,8827,113,117,97,108,59,1,10928,108,97,110,116,69,113,117,97,108,59,1,8829,105,108,100,101,59,1,8831,84,104,97,116,59,1,8715,59,1,8721,4,3,59,101,115,5486,5488,5507,1,8913,114,115,101,116,4,2,59,69,5498,5500,1,8835,113,117,97,108,59,1,8839,101,116,59,1,8913,4,11,72,82,83,97,99,102,104,105,111,114,115,5536,5546,5552,5567,5579,5602,5607,5655,5695,5701,5711,79,82,78,5,222,1,59,5544,1,222,65,68,69,59,1,8482,4,2,72,99,5558,5563,99,121,59,1,1035,121,59,1,1062,4,2,98,117,5573,5576,59,1,9,59,1,932,4,3,97,101,121,5587,5593,5599,114,111,110,59,1,356,100,105,108,59,1,354,59,1,1058,114,59,3,55349,56599,4,2,101,105,5613,5631,4,2,114,116,5619,5627,101,102,111,114,101,59,1,8756,97,59,1,920,4,2,99,110,5637,5647,107,83,112,97,99,101,59,3,8287,8202,83,112,97,99,101,59,1,8201,108,100,101,4,4,59,69,70,84,5668,5670,5677,5688,1,8764,113,117,97,108,59,1,8771,117,108,108,69,113,117,97,108,59,1,8773,105,108,100,101,59,1,8776,112,102,59,3,55349,56651,105,112,108,101,68,111,116,59,1,8411,4,2,99,116,5717,5722,114,59,3,55349,56495,114,111,107,59,1,358,4,14,97,98,99,100,102,103,109,110,111,112,114,115,116,117,5758,5789,5805,5823,5830,5835,5846,5852,5921,5937,6089,6095,6101,6108,4,2,99,114,5764,5774,117,116,101,5,218,1,59,5772,1,218,114,4,2,59,111,5781,5783,1,8607,99,105,114,59,1,10569,114,4,2,99,101,5796,5800,121,59,1,1038,118,101,59,1,364,4,2,105,121,5811,5820,114,99,5,219,1,59,5818,1,219,59,1,1059,98,108,97,99,59,1,368,114,59,3,55349,56600,114,97,118,101,5,217,1,59,5844,1,217,97,99,114,59,1,362,4,2,100,105,5858,5905,101,114,4,2,66,80,5866,5892,4,2,97,114,5872,5876,114,59,1,95,97,99,4,2,101,107,5884,5887,59,1,9183,101,116,59,1,9141,97,114,101,110,116,104,101,115,105,115,59,1,9181,111,110,4,2,59,80,5913,5915,1,8899,108,117,115,59,1,8846,4,2,103,112,5927,5932,111,110,59,1,370,102,59,3,55349,56652,4,8,65,68,69,84,97,100,112,115,5955,5985,5996,6009,6026,6033,6044,6075,114,114,111,119,4,3,59,66,68,5967,5969,5974,1,8593,97,114,59,1,10514,111,119,110,65,114,114,111,119,59,1,8645,111,119,110,65,114,114,111,119,59,1,8597,113,117,105,108,105,98,114,105,117,109,59,1,10606,101,101,4,2,59,65,6017,6019,1,8869,114,114,111,119,59,1,8613,114,114,111,119,59,1,8657,111,119,110,97,114,114,111,119,59,1,8661,101,114,4,2,76,82,6052,6063,101,102,116,65,114,114,111,119,59,1,8598,105,103,104,116,65,114,114,111,119,59,1,8599,105,4,2,59,108,6082,6084,1,978,111,110,59,1,933,105,110,103,59,1,366,99,114,59,3,55349,56496,105,108,100,101,59,1,360,109,108,5,220,1,59,6115,1,220,4,9,68,98,99,100,101,102,111,115,118,6137,6143,6148,6152,6166,6250,6255,6261,6267,97,115,104,59,1,8875,97,114,59,1,10987,121,59,1,1042,97,115,104,4,2,59,108,6161,6163,1,8873,59,1,10982,4,2,101,114,6172,6175,59,1,8897,4,3,98,116,121,6183,6188,6238,97,114,59,1,8214,4,2,59,105,6194,6196,1,8214,99,97,108,4,4,66,76,83,84,6209,6214,6220,6231,97,114,59,1,8739,105,110,101,59,1,124,101,112,97,114,97,116,111,114,59,1,10072,105,108,100,101,59,1,8768,84,104,105,110,83,112,97,99,101,59,1,8202,114,59,3,55349,56601,112,102,59,3,55349,56653,99,114,59,3,55349,56497,100,97,115,104,59,1,8874,4,5,99,101,102,111,115,6286,6292,6298,6303,6309,105,114,99,59,1,372,100,103,101,59,1,8896,114,59,3,55349,56602,112,102,59,3,55349,56654,99,114,59,3,55349,56498,4,4,102,105,111,115,6325,6330,6333,6339,114,59,3,55349,56603,59,1,926,112,102,59,3,55349,56655,99,114,59,3,55349,56499,4,9,65,73,85,97,99,102,111,115,117,6365,6370,6375,6380,6391,6405,6410,6416,6422,99,121,59,1,1071,99,121,59,1,1031,99,121,59,1,1070,99,117,116,101,5,221,1,59,6389,1,221,4,2,105,121,6397,6402,114,99,59,1,374,59,1,1067,114,59,3,55349,56604,112,102,59,3,55349,56656,99,114,59,3,55349,56500,109,108,59,1,376,4,8,72,97,99,100,101,102,111,115,6445,6450,6457,6472,6477,6501,6505,6510,99,121,59,1,1046,99,117,116,101,59,1,377,4,2,97,121,6463,6469,114,111,110,59,1,381,59,1,1047,111,116,59,1,379,4,2,114,116,6483,6497,111,87,105,100,116,104,83,112,97,99,101,59,1,8203,97,59,1,918,114,59,1,8488,112,102,59,1,8484,99,114,59,3,55349,56501,4,16,97,98,99,101,102,103,108,109,110,111,112,114,115,116,117,119,6550,6561,6568,6612,6622,6634,6645,6672,6699,6854,6870,6923,6933,6963,6974,6983,99,117,116,101,5,225,1,59,6559,1,225,114,101,118,101,59,1,259,4,6,59,69,100,105,117,121,6582,6584,6588,6591,6600,6609,1,8766,59,3,8766,819,59,1,8767,114,99,5,226,1,59,6598,1,226,116,101,5,180,1,59,6607,1,180,59,1,1072,108,105,103,5,230,1,59,6620,1,230,4,2,59,114,6628,6630,1,8289,59,3,55349,56606,114,97,118,101,5,224,1,59,6643,1,224,4,2,101,112,6651,6667,4,2,102,112,6657,6663,115,121,109,59,1,8501,104,59,1,8501,104,97,59,1,945,4,2,97,112,6678,6692,4,2,99,108,6684,6688,114,59,1,257,103,59,1,10815,5,38,1,59,6697,1,38,4,2,100,103,6705,6737,4,5,59,97,100,115,118,6717,6719,6724,6727,6734,1,8743,110,100,59,1,10837,59,1,10844,108,111,112,101,59,1,10840,59,1,10842,4,7,59,101,108,109,114,115,122,6753,6755,6758,6762,6814,6835,6848,1,8736,59,1,10660,101,59,1,8736,115,100,4,2,59,97,6770,6772,1,8737,4,8,97,98,99,100,101,102,103,104,6790,6793,6796,6799,6802,6805,6808,6811,59,1,10664,59,1,10665,59,1,10666,59,1,10667,59,1,10668,59,1,10669,59,1,10670,59,1,10671,116,4,2,59,118,6821,6823,1,8735,98,4,2,59,100,6830,6832,1,8894,59,1,10653,4,2,112,116,6841,6845,104,59,1,8738,59,1,197,97,114,114,59,1,9084,4,2,103,112,6860,6865,111,110,59,1,261,102,59,3,55349,56658,4,7,59,69,97,101,105,111,112,6886,6888,6891,6897,6900,6904,6908,1,8776,59,1,10864,99,105,114,59,1,10863,59,1,8778,100,59,1,8779,115,59,1,39,114,111,120,4,2,59,101,6917,6919,1,8776,113,59,1,8778,105,110,103,5,229,1,59,6931,1,229,4,3,99,116,121,6941,6946,6949,114,59,3,55349,56502,59,1,42,109,112,4,2,59,101,6957,6959,1,8776,113,59,1,8781,105,108,100,101,5,227,1,59,6972,1,227,109,108,5,228,1,59,6981,1,228,4,2,99,105,6989,6997,111,110,105,110,116,59,1,8755,110,116,59,1,10769,4,16,78,97,98,99,100,101,102,105,107,108,110,111,112,114,115,117,7036,7041,7119,7135,7149,7155,7219,7224,7347,7354,7463,7489,7786,7793,7814,7866,111,116,59,1,10989,4,2,99,114,7047,7094,107,4,4,99,101,112,115,7058,7064,7073,7080,111,110,103,59,1,8780,112,115,105,108,111,110,59,1,1014,114,105,109,101,59,1,8245,105,109,4,2,59,101,7088,7090,1,8765,113,59,1,8909,4,2,118,119,7100,7105,101,101,59,1,8893,101,100,4,2,59,103,7113,7115,1,8965,101,59,1,8965,114,107,4,2,59,116,7127,7129,1,9141,98,114,107,59,1,9142,4,2,111,121,7141,7146,110,103,59,1,8780,59,1,1073,113,117,111,59,1,8222,4,5,99,109,112,114,116,7167,7181,7188,7193,7199,97,117,115,4,2,59,101,7176,7178,1,8757,59,1,8757,112,116,121,118,59,1,10672,115,105,59,1,1014,110,111,117,59,1,8492,4,3,97,104,119,7207,7210,7213,59,1,946,59,1,8502,101,101,110,59,1,8812,114,59,3,55349,56607,103,4,7,99,111,115,116,117,118,119,7241,7262,7288,7305,7328,7335,7340,4,3,97,105,117,7249,7253,7258,112,59,1,8898,114,99,59,1,9711,112,59,1,8899,4,3,100,112,116,7270,7275,7281,111,116,59,1,10752,108,117,115,59,1,10753,105,109,101,115,59,1,10754,4,2,113,116,7294,7300,99,117,112,59,1,10758,97,114,59,1,9733,114,105,97,110,103,108,101,4,2,100,117,7318,7324,111,119,110,59,1,9661,112,59,1,9651,112,108,117,115,59,1,10756,101,101,59,1,8897,101,100,103,101,59,1,8896,97,114,111,119,59,1,10509,4,3,97,107,111,7362,7436,7458,4,2,99,110,7368,7432,107,4,3,108,115,116,7377,7386,7394,111,122,101,110,103,101,59,1,10731,113,117,97,114,101,59,1,9642,114,105,97,110,103,108,101,4,4,59,100,108,114,7411,7413,7419,7425,1,9652,111,119,110,59,1,9662,101,102,116,59,1,9666,105,103,104,116,59,1,9656,107,59,1,9251,4,2,49,51,7442,7454,4,2,50,52,7448,7451,59,1,9618,59,1,9617,52,59,1,9619,99,107,59,1,9608,4,2,101,111,7469,7485,4,2,59,113,7475,7478,3,61,8421,117,105,118,59,3,8801,8421,116,59,1,8976,4,4,112,116,119,120,7499,7504,7517,7523,102,59,3,55349,56659,4,2,59,116,7510,7512,1,8869,111,109,59,1,8869,116,105,101,59,1,8904,4,12,68,72,85,86,98,100,104,109,112,116,117,118,7549,7571,7597,7619,7655,7660,7682,7708,7715,7721,7728,7750,4,4,76,82,108,114,7559,7562,7565,7568,59,1,9559,59,1,9556,59,1,9558,59,1,9555,4,5,59,68,85,100,117,7583,7585,7588,7591,7594,1,9552,59,1,9574,59,1,9577,59,1,9572,59,1,9575,4,4,76,82,108,114,7607,7610,7613,7616,59,1,9565,59,1,9562,59,1,9564,59,1,9561,4,7,59,72,76,82,104,108,114,7635,7637,7640,7643,7646,7649,7652,1,9553,59,1,9580,59,1,9571,59,1,9568,59,1,9579,59,1,9570,59,1,9567,111,120,59,1,10697,4,4,76,82,108,114,7670,7673,7676,7679,59,1,9557,59,1,9554,59,1,9488,59,1,9484,4,5,59,68,85,100,117,7694,7696,7699,7702,7705,1,9472,59,1,9573,59,1,9576,59,1,9516,59,1,9524,105,110,117,115,59,1,8863,108,117,115,59,1,8862,105,109,101,115,59,1,8864,4,4,76,82,108,114,7738,7741,7744,7747,59,1,9563,59,1,9560,59,1,9496,59,1,9492,4,7,59,72,76,82,104,108,114,7766,7768,7771,7774,7777,7780,7783,1,9474,59,1,9578,59,1,9569,59,1,9566,59,1,9532,59,1,9508,59,1,9500,114,105,109,101,59,1,8245,4,2,101,118,7799,7804,118,101,59,1,728,98,97,114,5,166,1,59,7812,1,166,4,4,99,101,105,111,7824,7829,7834,7846,114,59,3,55349,56503,109,105,59,1,8271,109,4,2,59,101,7841,7843,1,8765,59,1,8909,108,4,3,59,98,104,7855,7857,7860,1,92,59,1,10693,115,117,98,59,1,10184,4,2,108,109,7872,7885,108,4,2,59,101,7879,7881,1,8226,116,59,1,8226,112,4,3,59,69,101,7894,7896,7899,1,8782,59,1,10926,4,2,59,113,7905,7907,1,8783,59,1,8783,4,15,97,99,100,101,102,104,105,108,111,114,115,116,117,119,121,7942,8021,8075,8080,8121,8126,8157,8279,8295,8430,8446,8485,8491,8707,8726,4,3,99,112,114,7950,7956,8007,117,116,101,59,1,263,4,6,59,97,98,99,100,115,7970,7972,7977,7984,7998,8003,1,8745,110,100,59,1,10820,114,99,117,112,59,1,10825,4,2,97,117,7990,7994,112,59,1,10827,112,59,1,10823,111,116,59,1,10816,59,3,8745,65024,4,2,101,111,8013,8017,116,59,1,8257,110,59,1,711,4,4,97,101,105,117,8031,8046,8056,8061,4,2,112,114,8037,8041,115,59,1,10829,111,110,59,1,269,100,105,108,5,231,1,59,8054,1,231,114,99,59,1,265,112,115,4,2,59,115,8069,8071,1,10828,109,59,1,10832,111,116,59,1,267,4,3,100,109,110,8088,8097,8104,105,108,5,184,1,59,8095,1,184,112,116,121,118,59,1,10674,116,5,162,2,59,101,8112,8114,1,162,114,100,111,116,59,1,183,114,59,3,55349,56608,4,3,99,101,105,8134,8138,8154,121,59,1,1095,99,107,4,2,59,109,8146,8148,1,10003,97,114,107,59,1,10003,59,1,967,114,4,7,59,69,99,101,102,109,115,8174,8176,8179,8258,8261,8268,8273,1,9675,59,1,10691,4,3,59,101,108,8187,8189,8193,1,710,113,59,1,8791,101,4,2,97,100,8200,8223,114,114,111,119,4,2,108,114,8210,8216,101,102,116,59,1,8634,105,103,104,116,59,1,8635,4,5,82,83,97,99,100,8235,8238,8241,8246,8252,59,1,174,59,1,9416,115,116,59,1,8859,105,114,99,59,1,8858,97,115,104,59,1,8861,59,1,8791,110,105,110,116,59,1,10768,105,100,59,1,10991,99,105,114,59,1,10690,117,98,115,4,2,59,117,8288,8290,1,9827,105,116,59,1,9827,4,4,108,109,110,112,8305,8326,8376,8400,111,110,4,2,59,101,8313,8315,1,58,4,2,59,113,8321,8323,1,8788,59,1,8788,4,2,109,112,8332,8344,97,4,2,59,116,8339,8341,1,44,59,1,64,4,3,59,102,108,8352,8354,8358,1,8705,110,59,1,8728,101,4,2,109,120,8365,8371,101,110,116,59,1,8705,101,115,59,1,8450,4,2,103,105,8382,8395,4,2,59,100,8388,8390,1,8773,111,116,59,1,10861,110,116,59,1,8750,4,3,102,114,121,8408,8412,8417,59,3,55349,56660,111,100,59,1,8720,5,169,2,59,115,8424,8426,1,169,114,59,1,8471,4,2,97,111,8436,8441,114,114,59,1,8629,115,115,59,1,10007,4,2,99,117,8452,8457,114,59,3,55349,56504,4,2,98,112,8463,8474,4,2,59,101,8469,8471,1,10959,59,1,10961,4,2,59,101,8480,8482,1,10960,59,1,10962,100,111,116,59,1,8943,4,7,100,101,108,112,114,118,119,8507,8522,8536,8550,8600,8697,8702,97,114,114,4,2,108,114,8516,8519,59,1,10552,59,1,10549,4,2,112,115,8528,8532,114,59,1,8926,99,59,1,8927,97,114,114,4,2,59,112,8545,8547,1,8630,59,1,10557,4,6,59,98,99,100,111,115,8564,8566,8573,8587,8592,8596,1,8746,114,99,97,112,59,1,10824,4,2,97,117,8579,8583,112,59,1,10822,112,59,1,10826,111,116,59,1,8845,114,59,1,10821,59,3,8746,65024,4,4,97,108,114,118,8610,8623,8663,8672,114,114,4,2,59,109,8618,8620,1,8631,59,1,10556,121,4,3,101,118,119,8632,8651,8656,113,4,2,112,115,8639,8645,114,101,99,59,1,8926,117,99,99,59,1,8927,101,101,59,1,8910,101,100,103,101,59,1,8911,101,110,5,164,1,59,8670,1,164,101,97,114,114,111,119,4,2,108,114,8684,8690,101,102,116,59,1,8630,105,103,104,116,59,1,8631,101,101,59,1,8910,101,100,59,1,8911,4,2,99,105,8713,8721,111,110,105,110,116,59,1,8754,110,116,59,1,8753,108,99,116,121,59,1,9005,4,19,65,72,97,98,99,100,101,102,104,105,106,108,111,114,115,116,117,119,122,8773,8778,8783,8821,8839,8854,8887,8914,8930,8944,9036,9041,9058,9197,9227,9258,9281,9297,9305,114,114,59,1,8659,97,114,59,1,10597,4,4,103,108,114,115,8793,8799,8805,8809,103,101,114,59,1,8224,101,116,104,59,1,8504,114,59,1,8595,104,4,2,59,118,8816,8818,1,8208,59,1,8867,4,2,107,108,8827,8834,97,114,111,119,59,1,10511,97,99,59,1,733,4,2,97,121,8845,8851,114,111,110,59,1,271,59,1,1076,4,3,59,97,111,8862,8864,8880,1,8518,4,2,103,114,8870,8876,103,101,114,59,1,8225,114,59,1,8650,116,115,101,113,59,1,10871,4,3,103,108,109,8895,8902,8907,5,176,1,59,8900,1,176,116,97,59,1,948,112,116,121,118,59,1,10673,4,2,105,114,8920,8926,115,104,116,59,1,10623,59,3,55349,56609,97,114,4,2,108,114,8938,8941,59,1,8643,59,1,8642,4,5,97,101,103,115,118,8956,8986,8989,8996,9001,109,4,3,59,111,115,8965,8967,8983,1,8900,110,100,4,2,59,115,8975,8977,1,8900,117,105,116,59,1,9830,59,1,9830,59,1,168,97,109,109,97,59,1,989,105,110,59,1,8946,4,3,59,105,111,9009,9011,9031,1,247,100,101,5,247,2,59,111,9020,9022,1,247,110,116,105,109,101,115,59,1,8903,110,120,59,1,8903,99,121,59,1,1106,99,4,2,111,114,9048,9053,114,110,59,1,8990,111,112,59,1,8973,4,5,108,112,116,117,119,9070,9076,9081,9130,9144,108,97,114,59,1,36,102,59,3,55349,56661,4,5,59,101,109,112,115,9093,9095,9109,9116,9122,1,729,113,4,2,59,100,9102,9104,1,8784,111,116,59,1,8785,105,110,117,115,59,1,8760,108,117,115,59,1,8724,113,117,97,114,101,59,1,8865,98,108,101,98,97,114,119,101,100,103,101,59,1,8966,110,4,3,97,100,104,9153,9160,9172,114,114,111,119,59,1,8595,111,119,110,97,114,114,111,119,115,59,1,8650,97,114,112,111,111,110,4,2,108,114,9184,9190,101,102,116,59,1,8643,105,103,104,116,59,1,8642,4,2,98,99,9203,9211,107,97,114,111,119,59,1,10512,4,2,111,114,9217,9222,114,110,59,1,8991,111,112,59,1,8972,4,3,99,111,116,9235,9248,9252,4,2,114,121,9241,9245,59,3,55349,56505,59,1,1109,108,59,1,10742,114,111,107,59,1,273,4,2,100,114,9264,9269,111,116,59,1,8945,105,4,2,59,102,9276,9278,1,9663,59,1,9662,4,2,97,104,9287,9292,114,114,59,1,8693,97,114,59,1,10607,97,110,103,108,101,59,1,10662,4,2,99,105,9311,9315,121,59,1,1119,103,114,97,114,114,59,1,10239,4,18,68,97,99,100,101,102,103,108,109,110,111,112,113,114,115,116,117,120,9361,9376,9398,9439,9444,9447,9462,9495,9531,9585,9598,9614,9659,9755,9771,9792,9808,9826,4,2,68,111,9367,9372,111,116,59,1,10871,116,59,1,8785,4,2,99,115,9382,9392,117,116,101,5,233,1,59,9390,1,233,116,101,114,59,1,10862,4,4,97,105,111,121,9408,9414,9430,9436,114,111,110,59,1,283,114,4,2,59,99,9421,9423,1,8790,5,234,1,59,9428,1,234,108,111,110,59,1,8789,59,1,1101,111,116,59,1,279,59,1,8519,4,2,68,114,9453,9458,111,116,59,1,8786,59,3,55349,56610,4,3,59,114,115,9470,9472,9482,1,10906,97,118,101,5,232,1,59,9480,1,232,4,2,59,100,9488,9490,1,10902,111,116,59,1,10904,4,4,59,105,108,115,9505,9507,9515,9518,1,10905,110,116,101,114,115,59,1,9191,59,1,8467,4,2,59,100,9524,9526,1,10901,111,116,59,1,10903,4,3,97,112,115,9539,9544,9564,99,114,59,1,275,116,121,4,3,59,115,118,9554,9556,9561,1,8709,101,116,59,1,8709,59,1,8709,112,4,2,49,59,9571,9583,4,2,51,52,9577,9580,59,1,8196,59,1,8197,1,8195,4,2,103,115,9591,9594,59,1,331,112,59,1,8194,4,2,103,112,9604,9609,111,110,59,1,281,102,59,3,55349,56662,4,3,97,108,115,9622,9635,9640,114,4,2,59,115,9629,9631,1,8917,108,59,1,10723,117,115,59,1,10865,105,4,3,59,108,118,9649,9651,9656,1,949,111,110,59,1,949,59,1,1013,4,4,99,115,117,118,9669,9686,9716,9747,4,2,105,111,9675,9680,114,99,59,1,8790,108,111,110,59,1,8789,4,2,105,108,9692,9696,109,59,1,8770,97,110,116,4,2,103,108,9705,9710,116,114,59,1,10902,101,115,115,59,1,10901,4,3,97,101,105,9724,9729,9734,108,115,59,1,61,115,116,59,1,8799,118,4,2,59,68,9741,9743,1,8801,68,59,1,10872,112,97,114,115,108,59,1,10725,4,2,68,97,9761,9766,111,116,59,1,8787,114,114,59,1,10609,4,3,99,100,105,9779,9783,9788,114,59,1,8495,111,116,59,1,8784,109,59,1,8770,4,2,97,104,9798,9801,59,1,951,5,240,1,59,9806,1,240,4,2,109,114,9814,9822,108,5,235,1,59,9820,1,235,111,59,1,8364,4,3,99,105,112,9834,9838,9843,108,59,1,33,115,116,59,1,8707,4,2,101,111,9849,9859,99,116,97,116,105,111,110,59,1,8496,110,101,110,116,105,97,108,101,59,1,8519,4,12,97,99,101,102,105,106,108,110,111,112,114,115,9896,9910,9914,9921,9954,9960,9967,9989,9994,10027,10036,10164,108,108,105,110,103,100,111,116,115,101,113,59,1,8786,121,59,1,1092,109,97,108,101,59,1,9792,4,3,105,108,114,9929,9935,9950,108,105,103,59,1,64259,4,2,105,108,9941,9945,103,59,1,64256,105,103,59,1,64260,59,3,55349,56611,108,105,103,59,1,64257,108,105,103,59,3,102,106,4,3,97,108,116,9975,9979,9984,116,59,1,9837,105,103,59,1,64258,110,115,59,1,9649,111,102,59,1,402,4,2,112,114,10000,10005,102,59,3,55349,56663,4,2,97,107,10011,10016,108,108,59,1,8704,4,2,59,118,10022,10024,1,8916,59,1,10969,97,114,116,105,110,116,59,1,10765,4,2,97,111,10042,10159,4,2,99,115,10048,10155,4,6,49,50,51,52,53,55,10062,10102,10114,10135,10139,10151,4,6,50,51,52,53,54,56,10076,10083,10086,10093,10096,10099,5,189,1,59,10081,1,189,59,1,8531,5,188,1,59,10091,1,188,59,1,8533,59,1,8537,59,1,8539,4,2,51,53,10108,10111,59,1,8532,59,1,8534,4,3,52,53,56,10122,10129,10132,5,190,1,59,10127,1,190,59,1,8535,59,1,8540,53,59,1,8536,4,2,54,56,10145,10148,59,1,8538,59,1,8541,56,59,1,8542,108,59,1,8260,119,110,59,1,8994,99,114,59,3,55349,56507,4,17,69,97,98,99,100,101,102,103,105,106,108,110,111,114,115,116,118,10206,10217,10247,10254,10268,10273,10358,10363,10374,10380,10385,10406,10458,10464,10470,10497,10610,4,2,59,108,10212,10214,1,8807,59,1,10892,4,3,99,109,112,10225,10231,10244,117,116,101,59,1,501,109,97,4,2,59,100,10239,10241,1,947,59,1,989,59,1,10886,114,101,118,101,59,1,287,4,2,105,121,10260,10265,114,99,59,1,285,59,1,1075,111,116,59,1,289,4,4,59,108,113,115,10283,10285,10288,10308,1,8805,59,1,8923,4,3,59,113,115,10296,10298,10301,1,8805,59,1,8807,108,97,110,116,59,1,10878,4,4,59,99,100,108,10318,10320,10324,10345,1,10878,99,59,1,10921,111,116,4,2,59,111,10332,10334,1,10880,4,2,59,108,10340,10342,1,10882,59,1,10884,4,2,59,101,10351,10354,3,8923,65024,115,59,1,10900,114,59,3,55349,56612,4,2,59,103,10369,10371,1,8811,59,1,8921,109,101,108,59,1,8503,99,121,59,1,1107,4,4,59,69,97,106,10395,10397,10400,10403,1,8823,59,1,10898,59,1,10917,59,1,10916,4,4,69,97,101,115,10416,10419,10434,10453,59,1,8809,112,4,2,59,112,10426,10428,1,10890,114,111,120,59,1,10890,4,2,59,113,10440,10442,1,10888,4,2,59,113,10448,10450,1,10888,59,1,8809,105,109,59,1,8935,112,102,59,3,55349,56664,97,118,101,59,1,96,4,2,99,105,10476,10480,114,59,1,8458,109,4,3,59,101,108,10489,10491,10494,1,8819,59,1,10894,59,1,10896,5,62,6,59,99,100,108,113,114,10512,10514,10527,10532,10538,10545,1,62,4,2,99,105,10520,10523,59,1,10919,114,59,1,10874,111,116,59,1,8919,80,97,114,59,1,10645,117,101,115,116,59,1,10876,4,5,97,100,101,108,115,10557,10574,10579,10599,10605,4,2,112,114,10563,10570,112,114,111,120,59,1,10886,114,59,1,10616,111,116,59,1,8919,113,4,2,108,113,10586,10592,101,115,115,59,1,8923,108,101,115,115,59,1,10892,101,115,115,59,1,8823,105,109,59,1,8819,4,2,101,110,10616,10626,114,116,110,101,113,113,59,3,8809,65024,69,59,3,8809,65024,4,10,65,97,98,99,101,102,107,111,115,121,10653,10658,10713,10718,10724,10760,10765,10786,10850,10875,114,114,59,1,8660,4,4,105,108,109,114,10668,10674,10678,10684,114,115,112,59,1,8202,102,59,1,189,105,108,116,59,1,8459,4,2,100,114,10690,10695,99,121,59,1,1098,4,3,59,99,119,10703,10705,10710,1,8596,105,114,59,1,10568,59,1,8621,97,114,59,1,8463,105,114,99,59,1,293,4,3,97,108,114,10732,10748,10754,114,116,115,4,2,59,117,10741,10743,1,9829,105,116,59,1,9829,108,105,112,59,1,8230,99,111,110,59,1,8889,114,59,3,55349,56613,115,4,2,101,119,10772,10779,97,114,111,119,59,1,10533,97,114,111,119,59,1,10534,4,5,97,109,111,112,114,10798,10803,10809,10839,10844,114,114,59,1,8703,116,104,116,59,1,8763,107,4,2,108,114,10816,10827,101,102,116,97,114,114,111,119,59,1,8617,105,103,104,116,97,114,114,111,119,59,1,8618,102,59,3,55349,56665,98,97,114,59,1,8213,4,3,99,108,116,10858,10863,10869,114,59,3,55349,56509,97,115,104,59,1,8463,114,111,107,59,1,295,4,2,98,112,10881,10887,117,108,108,59,1,8259,104,101,110,59,1,8208,4,15,97,99,101,102,103,105,106,109,110,111,112,113,115,116,117,10925,10936,10958,10977,10990,11001,11039,11045,11101,11192,11220,11226,11237,11285,11299,99,117,116,101,5,237,1,59,10934,1,237,4,3,59,105,121,10944,10946,10955,1,8291,114,99,5,238,1,59,10953,1,238,59,1,1080,4,2,99,120,10964,10968,121,59,1,1077,99,108,5,161,1,59,10975,1,161,4,2,102,114,10983,10986,59,1,8660,59,3,55349,56614,114,97,118,101,5,236,1,59,10999,1,236,4,4,59,105,110,111,11011,11013,11028,11034,1,8520,4,2,105,110,11019,11024,110,116,59,1,10764,116,59,1,8749,102,105,110,59,1,10716,116,97,59,1,8489,108,105,103,59,1,307,4,3,97,111,112,11053,11092,11096,4,3,99,103,116,11061,11065,11088,114,59,1,299,4,3,101,108,112,11073,11076,11082,59,1,8465,105,110,101,59,1,8464,97,114,116,59,1,8465,104,59,1,305,102,59,1,8887,101,100,59,1,437,4,5,59,99,102,111,116,11113,11115,11121,11136,11142,1,8712,97,114,101,59,1,8453,105,110,4,2,59,116,11129,11131,1,8734,105,101,59,1,10717,100,111,116,59,1,305,4,5,59,99,101,108,112,11154,11156,11161,11179,11186,1,8747,97,108,59,1,8890,4,2,103,114,11167,11173,101,114,115,59,1,8484,99,97,108,59,1,8890,97,114,104,107,59,1,10775,114,111,100,59,1,10812,4,4,99,103,112,116,11202,11206,11211,11216,121,59,1,1105,111,110,59,1,303,102,59,3,55349,56666,97,59,1,953,114,111,100,59,1,10812,117,101,115,116,5,191,1,59,11235,1,191,4,2,99,105,11243,11248,114,59,3,55349,56510,110,4,5,59,69,100,115,118,11261,11263,11266,11271,11282,1,8712,59,1,8953,111,116,59,1,8949,4,2,59,118,11277,11279,1,8948,59,1,8947,59,1,8712,4,2,59,105,11291,11293,1,8290,108,100,101,59,1,297,4,2,107,109,11305,11310,99,121,59,1,1110,108,5,239,1,59,11316,1,239,4,6,99,102,109,111,115,117,11332,11346,11351,11357,11363,11380,4,2,105,121,11338,11343,114,99,59,1,309,59,1,1081,114,59,3,55349,56615,97,116,104,59,1,567,112,102,59,3,55349,56667,4,2,99,101,11369,11374,114,59,3,55349,56511,114,99,121,59,1,1112,107,99,121,59,1,1108,4,8,97,99,102,103,104,106,111,115,11404,11418,11433,11438,11445,11450,11455,11461,112,112,97,4,2,59,118,11413,11415,1,954,59,1,1008,4,2,101,121,11424,11430,100,105,108,59,1,311,59,1,1082,114,59,3,55349,56616,114,101,101,110,59,1,312,99,121,59,1,1093,99,121,59,1,1116,112,102,59,3,55349,56668,99,114,59,3,55349,56512,4,23,65,66,69,72,97,98,99,100,101,102,103,104,106,108,109,110,111,112,114,115,116,117,118,11515,11538,11544,11555,11560,11721,11780,11818,11868,12136,12160,12171,12203,12208,12246,12275,12327,12509,12523,12569,12641,12732,12752,4,3,97,114,116,11523,11528,11532,114,114,59,1,8666,114,59,1,8656,97,105,108,59,1,10523,97,114,114,59,1,10510,4,2,59,103,11550,11552,1,8806,59,1,10891,97,114,59,1,10594,4,9,99,101,103,109,110,112,113,114,116,11580,11586,11594,11600,11606,11624,11627,11636,11694,117,116,101,59,1,314,109,112,116,121,118,59,1,10676,114,97,110,59,1,8466,98,100,97,59,1,955,103,4,3,59,100,108,11615,11617,11620,1,10216,59,1,10641,101,59,1,10216,59,1,10885,117,111,5,171,1,59,11634,1,171,114,4,8,59,98,102,104,108,112,115,116,11655,11657,11669,11673,11677,11681,11685,11690,1,8592,4,2,59,102,11663,11665,1,8676,115,59,1,10527,115,59,1,10525,107,59,1,8617,112,59,1,8619,108,59,1,10553,105,109,59,1,10611,108,59,1,8610,4,3,59,97,101,11702,11704,11709,1,10923,105,108,59,1,10521,4,2,59,115,11715,11717,1,10925,59,3,10925,65024,4,3,97,98,114,11729,11734,11739,114,114,59,1,10508,114,107,59,1,10098,4,2,97,107,11745,11758,99,4,2,101,107,11752,11755,59,1,123,59,1,91,4,2,101,115,11764,11767,59,1,10635,108,4,2,100,117,11774,11777,59,1,10639,59,1,10637,4,4,97,101,117,121,11790,11796,11811,11815,114,111,110,59,1,318,4,2,100,105,11802,11807,105,108,59,1,316,108,59,1,8968,98,59,1,123,59,1,1083,4,4,99,113,114,115,11828,11832,11845,11864,97,59,1,10550,117,111,4,2,59,114,11840,11842,1,8220,59,1,8222,4,2,100,117,11851,11857,104,97,114,59,1,10599,115,104,97,114,59,1,10571,104,59,1,8626,4,5,59,102,103,113,115,11880,11882,12008,12011,12031,1,8804,116,4,5,97,104,108,114,116,11895,11913,11935,11947,11996,114,114,111,119,4,2,59,116,11905,11907,1,8592,97,105,108,59,1,8610,97,114,112,111,111,110,4,2,100,117,11925,11931,111,119,110,59,1,8637,112,59,1,8636,101,102,116,97,114,114,111,119,115,59,1,8647,105,103,104,116,4,3,97,104,115,11959,11974,11984,114,114,111,119,4,2,59,115,11969,11971,1,8596,59,1,8646,97,114,112,111,111,110,115,59,1,8651,113,117,105,103,97,114,114,111,119,59,1,8621,104,114,101,101,116,105,109,101,115,59,1,8907,59,1,8922,4,3,59,113,115,12019,12021,12024,1,8804,59,1,8806,108,97,110,116,59,1,10877,4,5,59,99,100,103,115,12043,12045,12049,12070,12083,1,10877,99,59,1,10920,111,116,4,2,59,111,12057,12059,1,10879,4,2,59,114,12065,12067,1,10881,59,1,10883,4,2,59,101,12076,12079,3,8922,65024,115,59,1,10899,4,5,97,100,101,103,115,12095,12103,12108,12126,12131,112,112,114,111,120,59,1,10885,111,116,59,1,8918,113,4,2,103,113,12115,12120,116,114,59,1,8922,103,116,114,59,1,10891,116,114,59,1,8822,105,109,59,1,8818,4,3,105,108,114,12144,12150,12156,115,104,116,59,1,10620,111,111,114,59,1,8970,59,3,55349,56617,4,2,59,69,12166,12168,1,8822,59,1,10897,4,2,97,98,12177,12198,114,4,2,100,117,12184,12187,59,1,8637,4,2,59,108,12193,12195,1,8636,59,1,10602,108,107,59,1,9604,99,121,59,1,1113,4,5,59,97,99,104,116,12220,12222,12227,12235,12241,1,8810,114,114,59,1,8647,111,114,110,101,114,59,1,8990,97,114,100,59,1,10603,114,105,59,1,9722,4,2,105,111,12252,12258,100,111,116,59,1,320,117,115,116,4,2,59,97,12267,12269,1,9136,99,104,101,59,1,9136,4,4,69,97,101,115,12285,12288,12303,12322,59,1,8808,112,4,2,59,112,12295,12297,1,10889,114,111,120,59,1,10889,4,2,59,113,12309,12311,1,10887,4,2,59,113,12317,12319,1,10887,59,1,8808,105,109,59,1,8934,4,8,97,98,110,111,112,116,119,122,12345,12359,12364,12421,12446,12467,12474,12490,4,2,110,114,12351,12355,103,59,1,10220,114,59,1,8701,114,107,59,1,10214,103,4,3,108,109,114,12373,12401,12409,101,102,116,4,2,97,114,12382,12389,114,114,111,119,59,1,10229,105,103,104,116,97,114,114,111,119,59,1,10231,97,112,115,116,111,59,1,10236,105,103,104,116,97,114,114,111,119,59,1,10230,112,97,114,114,111,119,4,2,108,114,12433,12439,101,102,116,59,1,8619,105,103,104,116,59,1,8620,4,3,97,102,108,12454,12458,12462,114,59,1,10629,59,3,55349,56669,117,115,59,1,10797,105,109,101,115,59,1,10804,4,2,97,98,12480,12485,115,116,59,1,8727,97,114,59,1,95,4,3,59,101,102,12498,12500,12506,1,9674,110,103,101,59,1,9674,59,1,10731,97,114,4,2,59,108,12517,12519,1,40,116,59,1,10643,4,5,97,99,104,109,116,12535,12540,12548,12561,12564,114,114,59,1,8646,111,114,110,101,114,59,1,8991,97,114,4,2,59,100,12556,12558,1,8651,59,1,10605,59,1,8206,114,105,59,1,8895,4,6,97,99,104,105,113,116,12583,12589,12594,12597,12614,12635,113,117,111,59,1,8249,114,59,3,55349,56513,59,1,8624,109,4,3,59,101,103,12606,12608,12611,1,8818,59,1,10893,59,1,10895,4,2,98,117,12620,12623,59,1,91,111,4,2,59,114,12630,12632,1,8216,59,1,8218,114,111,107,59,1,322,5,60,8,59,99,100,104,105,108,113,114,12660,12662,12675,12680,12686,12692,12698,12705,1,60,4,2,99,105,12668,12671,59,1,10918,114,59,1,10873,111,116,59,1,8918,114,101,101,59,1,8907,109,101,115,59,1,8905,97,114,114,59,1,10614,117,101,115,116,59,1,10875,4,2,80,105,12711,12716,97,114,59,1,10646,4,3,59,101,102,12724,12726,12729,1,9667,59,1,8884,59,1,9666,114,4,2,100,117,12739,12746,115,104,97,114,59,1,10570,104,97,114,59,1,10598,4,2,101,110,12758,12768,114,116,110,101,113,113,59,3,8808,65024,69,59,3,8808,65024,4,14,68,97,99,100,101,102,104,105,108,110,111,112,115,117,12803,12809,12893,12908,12914,12928,12933,12937,13011,13025,13032,13049,13052,13069,68,111,116,59,1,8762,4,4,99,108,112,114,12819,12827,12849,12887,114,5,175,1,59,12825,1,175,4,2,101,116,12833,12836,59,1,9794,4,2,59,101,12842,12844,1,10016,115,101,59,1,10016,4,2,59,115,12855,12857,1,8614,116,111,4,4,59,100,108,117,12869,12871,12877,12883,1,8614,111,119,110,59,1,8615,101,102,116,59,1,8612,112,59,1,8613,107,101,114,59,1,9646,4,2,111,121,12899,12905,109,109,97,59,1,10793,59,1,1084,97,115,104,59,1,8212,97,115,117,114,101,100,97,110,103,108,101,59,1,8737,114,59,3,55349,56618,111,59,1,8487,4,3,99,100,110,12945,12954,12985,114,111,5,181,1,59,12952,1,181,4,4,59,97,99,100,12964,12966,12971,12976,1,8739,115,116,59,1,42,105,114,59,1,10992,111,116,5,183,1,59,12983,1,183,117,115,4,3,59,98,100,12995,12997,13000,1,8722,59,1,8863,4,2,59,117,13006,13008,1,8760,59,1,10794,4,2,99,100,13017,13021,112,59,1,10971,114,59,1,8230,112,108,117,115,59,1,8723,4,2,100,112,13038,13044,101,108,115,59,1,8871,102,59,3,55349,56670,59,1,8723,4,2,99,116,13058,13063,114,59,3,55349,56514,112,111,115,59,1,8766,4,3,59,108,109,13077,13079,13087,1,956,116,105,109,97,112,59,1,8888,97,112,59,1,8888,4,24,71,76,82,86,97,98,99,100,101,102,103,104,105,106,108,109,111,112,114,115,116,117,118,119,13142,13165,13217,13229,13247,13330,13359,13414,13420,13508,13513,13579,13602,13626,13631,13762,13767,13855,13936,13995,14214,14285,14312,14432,4,2,103,116,13148,13152,59,3,8921,824,4,2,59,118,13158,13161,3,8811,8402,59,3,8811,824,4,3,101,108,116,13173,13200,13204,102,116,4,2,97,114,13181,13188,114,114,111,119,59,1,8653,105,103,104,116,97,114,114,111,119,59,1,8654,59,3,8920,824,4,2,59,118,13210,13213,3,8810,8402,59,3,8810,824,105,103,104,116,97,114,114,111,119,59,1,8655,4,2,68,100,13235,13241,97,115,104,59,1,8879,97,115,104,59,1,8878,4,5,98,99,110,112,116,13259,13264,13270,13275,13308,108,97,59,1,8711,117,116,101,59,1,324,103,59,3,8736,8402,4,5,59,69,105,111,112,13287,13289,13293,13298,13302,1,8777,59,3,10864,824,100,59,3,8779,824,115,59,1,329,114,111,120,59,1,8777,117,114,4,2,59,97,13316,13318,1,9838,108,4,2,59,115,13325,13327,1,9838,59,1,8469,4,2,115,117,13336,13344,112,5,160,1,59,13342,1,160,109,112,4,2,59,101,13352,13355,3,8782,824,59,3,8783,824,4,5,97,101,111,117,121,13371,13385,13391,13407,13411,4,2,112,114,13377,13380,59,1,10819,111,110,59,1,328,100,105,108,59,1,326,110,103,4,2,59,100,13399,13401,1,8775,111,116,59,3,10861,824,112,59,1,10818,59,1,1085,97,115,104,59,1,8211,4,7,59,65,97,100,113,115,120,13436,13438,13443,13466,13472,13478,13494,1,8800,114,114,59,1,8663,114,4,2,104,114,13450,13454,107,59,1,10532,4,2,59,111,13460,13462,1,8599,119,59,1,8599,111,116,59,3,8784,824,117,105,118,59,1,8802,4,2,101,105,13484,13489,97,114,59,1,10536,109,59,3,8770,824,105,115,116,4,2,59,115,13503,13505,1,8708,59,1,8708,114,59,3,55349,56619,4,4,69,101,115,116,13523,13527,13563,13568,59,3,8807,824,4,3,59,113,115,13535,13537,13559,1,8817,4,3,59,113,115,13545,13547,13551,1,8817,59,3,8807,824,108,97,110,116,59,3,10878,824,59,3,10878,824,105,109,59,1,8821,4,2,59,114,13574,13576,1,8815,59,1,8815,4,3,65,97,112,13587,13592,13597,114,114,59,1,8654,114,114,59,1,8622,97,114,59,1,10994,4,3,59,115,118,13610,13612,13623,1,8715,4,2,59,100,13618,13620,1,8956,59,1,8954,59,1,8715,99,121,59,1,1114,4,7,65,69,97,100,101,115,116,13647,13652,13656,13661,13665,13737,13742,114,114,59,1,8653,59,3,8806,824,114,114,59,1,8602,114,59,1,8229,4,4,59,102,113,115,13675,13677,13703,13725,1,8816,116,4,2,97,114,13684,13691,114,114,111,119,59,1,8602,105,103,104,116,97,114,114,111,119,59,1,8622,4,3,59,113,115,13711,13713,13717,1,8816,59,3,8806,824,108,97,110,116,59,3,10877,824,4,2,59,115,13731,13734,3,10877,824,59,1,8814,105,109,59,1,8820,4,2,59,114,13748,13750,1,8814,105,4,2,59,101,13757,13759,1,8938,59,1,8940,105,100,59,1,8740,4,2,112,116,13773,13778,102,59,3,55349,56671,5,172,3,59,105,110,13787,13789,13829,1,172,110,4,4,59,69,100,118,13800,13802,13806,13812,1,8713,59,3,8953,824,111,116,59,3,8949,824,4,3,97,98,99,13820,13823,13826,59,1,8713,59,1,8951,59,1,8950,105,4,2,59,118,13836,13838,1,8716,4,3,97,98,99,13846,13849,13852,59,1,8716,59,1,8958,59,1,8957,4,3,97,111,114,13863,13892,13899,114,4,4,59,97,115,116,13874,13876,13883,13888,1,8742,108,108,101,108,59,1,8742,108,59,3,11005,8421,59,3,8706,824,108,105,110,116,59,1,10772,4,3,59,99,101,13907,13909,13914,1,8832,117,101,59,1,8928,4,2,59,99,13920,13923,3,10927,824,4,2,59,101,13929,13931,1,8832,113,59,3,10927,824,4,4,65,97,105,116,13946,13951,13971,13982,114,114,59,1,8655,114,114,4,3,59,99,119,13961,13963,13967,1,8603,59,3,10547,824,59,3,8605,824,103,104,116,97,114,114,111,119,59,1,8603,114,105,4,2,59,101,13990,13992,1,8939,59,1,8941,4,7,99,104,105,109,112,113,117,14011,14036,14060,14080,14085,14090,14106,4,4,59,99,101,114,14021,14023,14028,14032,1,8833,117,101,59,1,8929,59,3,10928,824,59,3,55349,56515,111,114,116,4,2,109,112,14045,14050,105,100,59,1,8740,97,114,97,108,108,101,108,59,1,8742,109,4,2,59,101,14067,14069,1,8769,4,2,59,113,14075,14077,1,8772,59,1,8772,105,100,59,1,8740,97,114,59,1,8742,115,117,4,2,98,112,14098,14102,101,59,1,8930,101,59,1,8931,4,3,98,99,112,14114,14157,14171,4,4,59,69,101,115,14124,14126,14130,14133,1,8836,59,3,10949,824,59,1,8840,101,116,4,2,59,101,14141,14144,3,8834,8402,113,4,2,59,113,14151,14153,1,8840,59,3,10949,824,99,4,2,59,101,14164,14166,1,8833,113,59,3,10928,824,4,4,59,69,101,115,14181,14183,14187,14190,1,8837,59,3,10950,824,59,1,8841,101,116,4,2,59,101,14198,14201,3,8835,8402,113,4,2,59,113,14208,14210,1,8841,59,3,10950,824,4,4,103,105,108,114,14224,14228,14238,14242,108,59,1,8825,108,100,101,5,241,1,59,14236,1,241,103,59,1,8824,105,97,110,103,108,101,4,2,108,114,14254,14269,101,102,116,4,2,59,101,14263,14265,1,8938,113,59,1,8940,105,103,104,116,4,2,59,101,14279,14281,1,8939,113,59,1,8941,4,2,59,109,14291,14293,1,957,4,3,59,101,115,14301,14303,14308,1,35,114,111,59,1,8470,112,59,1,8199,4,9,68,72,97,100,103,105,108,114,115,14332,14338,14344,14349,14355,14369,14376,14408,14426,97,115,104,59,1,8877,97,114,114,59,1,10500,112,59,3,8781,8402,97,115,104,59,1,8876,4,2,101,116,14361,14365,59,3,8805,8402,59,3,62,8402,110,102,105,110,59,1,10718,4,3,65,101,116,14384,14389,14393,114,114,59,1,10498,59,3,8804,8402,4,2,59,114,14399,14402,3,60,8402,105,101,59,3,8884,8402,4,2,65,116,14414,14419,114,114,59,1,10499,114,105,101,59,3,8885,8402,105,109,59,3,8764,8402,4,3,65,97,110,14440,14445,14468,114,114,59,1,8662,114,4,2,104,114,14452,14456,107,59,1,10531,4,2,59,111,14462,14464,1,8598,119,59,1,8598,101,97,114,59,1,10535,4,18,83,97,99,100,101,102,103,104,105,108,109,111,112,114,115,116,117,118,14512,14515,14535,14560,14597,14603,14618,14643,14657,14662,14701,14741,14747,14769,14851,14877,14907,14916,59,1,9416,4,2,99,115,14521,14531,117,116,101,5,243,1,59,14529,1,243,116,59,1,8859,4,2,105,121,14541,14557,114,4,2,59,99,14548,14550,1,8858,5,244,1,59,14555,1,244,59,1,1086,4,5,97,98,105,111,115,14572,14577,14583,14587,14591,115,104,59,1,8861,108,97,99,59,1,337,118,59,1,10808,116,59,1,8857,111,108,100,59,1,10684,108,105,103,59,1,339,4,2,99,114,14609,14614,105,114,59,1,10687,59,3,55349,56620,4,3,111,114,116,14626,14630,14640,110,59,1,731,97,118,101,5,242,1,59,14638,1,242,59,1,10689,4,2,98,109,14649,14654,97,114,59,1,10677,59,1,937,110,116,59,1,8750,4,4,97,99,105,116,14672,14677,14693,14698,114,114,59,1,8634,4,2,105,114,14683,14687,114,59,1,10686,111,115,115,59,1,10683,110,101,59,1,8254,59,1,10688,4,3,97,101,105,14709,14714,14719,99,114,59,1,333,103,97,59,1,969,4,3,99,100,110,14727,14733,14736,114,111,110,59,1,959,59,1,10678,117,115,59,1,8854,112,102,59,3,55349,56672,4,3,97,101,108,14755,14759,14764,114,59,1,10679,114,112,59,1,10681,117,115,59,1,8853,4,7,59,97,100,105,111,115,118,14785,14787,14792,14831,14837,14841,14848,1,8744,114,114,59,1,8635,4,4,59,101,102,109,14802,14804,14817,14824,1,10845,114,4,2,59,111,14811,14813,1,8500,102,59,1,8500,5,170,1,59,14822,1,170,5,186,1,59,14829,1,186,103,111,102,59,1,8886,114,59,1,10838,108,111,112,101,59,1,10839,59,1,10843,4,3,99,108,111,14859,14863,14873,114,59,1,8500,97,115,104,5,248,1,59,14871,1,248,108,59,1,8856,105,4,2,108,109,14884,14893,100,101,5,245,1,59,14891,1,245,101,115,4,2,59,97,14901,14903,1,8855,115,59,1,10806,109,108,5,246,1,59,14914,1,246,98,97,114,59,1,9021,4,12,97,99,101,102,104,105,108,109,111,114,115,117,14948,14992,14996,15033,15038,15068,15090,15189,15192,15222,15427,15441,114,4,4,59,97,115,116,14959,14961,14976,14989,1,8741,5,182,2,59,108,14968,14970,1,182,108,101,108,59,1,8741,4,2,105,108,14982,14986,109,59,1,10995,59,1,11005,59,1,8706,121,59,1,1087,114,4,5,99,105,109,112,116,15009,15014,15019,15024,15027,110,116,59,1,37,111,100,59,1,46,105,108,59,1,8240,59,1,8869,101,110,107,59,1,8241,114,59,3,55349,56621,4,3,105,109,111,15046,15057,15063,4,2,59,118,15052,15054,1,966,59,1,981,109,97,116,59,1,8499,110,101,59,1,9742,4,3,59,116,118,15076,15078,15087,1,960,99,104,102,111,114,107,59,1,8916,59,1,982,4,2,97,117,15096,15119,110,4,2,99,107,15103,15115,107,4,2,59,104,15110,15112,1,8463,59,1,8462,118,59,1,8463,115,4,9,59,97,98,99,100,101,109,115,116,15140,15142,15148,15151,15156,15168,15171,15179,15184,1,43,99,105,114,59,1,10787,59,1,8862,105,114,59,1,10786,4,2,111,117,15162,15165,59,1,8724,59,1,10789,59,1,10866,110,5,177,1,59,15177,1,177,105,109,59,1,10790,119,111,59,1,10791,59,1,177,4,3,105,112,117,15200,15208,15213,110,116,105,110,116,59,1,10773,102,59,3,55349,56673,110,100,5,163,1,59,15220,1,163,4,10,59,69,97,99,101,105,110,111,115,117,15244,15246,15249,15253,15258,15334,15347,15367,15416,15421,1,8826,59,1,10931,112,59,1,10935,117,101,59,1,8828,4,2,59,99,15264,15266,1,10927,4,6,59,97,99,101,110,115,15280,15282,15290,15299,15303,15329,1,8826,112,112,114,111,120,59,1,10935,117,114,108,121,101,113,59,1,8828,113,59,1,10927,4,3,97,101,115,15311,15319,15324,112,112,114,111,120,59,1,10937,113,113,59,1,10933,105,109,59,1,8936,105,109,59,1,8830,109,101,4,2,59,115,15342,15344,1,8242,59,1,8473,4,3,69,97,115,15355,15358,15362,59,1,10933,112,59,1,10937,105,109,59,1,8936,4,3,100,102,112,15375,15378,15404,59,1,8719,4,3,97,108,115,15386,15392,15398,108,97,114,59,1,9006,105,110,101,59,1,8978,117,114,102,59,1,8979,4,2,59,116,15410,15412,1,8733,111,59,1,8733,105,109,59,1,8830,114,101,108,59,1,8880,4,2,99,105,15433,15438,114,59,3,55349,56517,59,1,968,110,99,115,112,59,1,8200,4,6,102,105,111,112,115,117,15462,15467,15472,15478,15485,15491,114,59,3,55349,56622,110,116,59,1,10764,112,102,59,3,55349,56674,114,105,109,101,59,1,8279,99,114,59,3,55349,56518,4,3,97,101,111,15499,15520,15534,116,4,2,101,105,15506,15515,114,110,105,111,110,115,59,1,8461,110,116,59,1,10774,115,116,4,2,59,101,15528,15530,1,63,113,59,1,8799,116,5,34,1,59,15540,1,34,4,21,65,66,72,97,98,99,100,101,102,104,105,108,109,110,111,112,114,115,116,117,120,15586,15609,15615,15620,15796,15855,15893,15931,15977,16001,16039,16183,16204,16222,16228,16285,16312,16318,16363,16408,16416,4,3,97,114,116,15594,15599,15603,114,114,59,1,8667,114,59,1,8658,97,105,108,59,1,10524,97,114,114,59,1,10511,97,114,59,1,10596,4,7,99,100,101,110,113,114,116,15636,15651,15656,15664,15687,15696,15770,4,2,101,117,15642,15646,59,3,8765,817,116,101,59,1,341,105,99,59,1,8730,109,112,116,121,118,59,1,10675,103,4,4,59,100,101,108,15675,15677,15680,15683,1,10217,59,1,10642,59,1,10661,101,59,1,10217,117,111,5,187,1,59,15694,1,187,114,4,11,59,97,98,99,102,104,108,112,115,116,119,15721,15723,15727,15739,15742,15746,15750,15754,15758,15763,15767,1,8594,112,59,1,10613,4,2,59,102,15733,15735,1,8677,115,59,1,10528,59,1,10547,115,59,1,10526,107,59,1,8618,112,59,1,8620,108,59,1,10565,105,109,59,1,10612,108,59,1,8611,59,1,8605,4,2,97,105,15776,15781,105,108,59,1,10522,111,4,2,59,110,15788,15790,1,8758,97,108,115,59,1,8474,4,3,97,98,114,15804,15809,15814,114,114,59,1,10509,114,107,59,1,10099,4,2,97,107,15820,15833,99,4,2,101,107,15827,15830,59,1,125,59,1,93,4,2,101,115,15839,15842,59,1,10636,108,4,2,100,117,15849,15852,59,1,10638,59,1,10640,4,4,97,101,117,121,15865,15871,15886,15890,114,111,110,59,1,345,4,2,100,105,15877,15882,105,108,59,1,343,108,59,1,8969,98,59,1,125,59,1,1088,4,4,99,108,113,115,15903,15907,15914,15927,97,59,1,10551,100,104,97,114,59,1,10601,117,111,4,2,59,114,15922,15924,1,8221,59,1,8221,104,59,1,8627,4,3,97,99,103,15939,15966,15970,108,4,4,59,105,112,115,15950,15952,15957,15963,1,8476,110,101,59,1,8475,97,114,116,59,1,8476,59,1,8477,116,59,1,9645,5,174,1,59,15975,1,174,4,3,105,108,114,15985,15991,15997,115,104,116,59,1,10621,111,111,114,59,1,8971,59,3,55349,56623,4,2,97,111,16007,16028,114,4,2,100,117,16014,16017,59,1,8641,4,2,59,108,16023,16025,1,8640,59,1,10604,4,2,59,118,16034,16036,1,961,59,1,1009,4,3,103,110,115,16047,16167,16171,104,116,4,6,97,104,108,114,115,116,16063,16081,16103,16130,16143,16155,114,114,111,119,4,2,59,116,16073,16075,1,8594,97,105,108,59,1,8611,97,114,112,111,111,110,4,2,100,117,16093,16099,111,119,110,59,1,8641,112,59,1,8640,101,102,116,4,2,97,104,16112,16120,114,114,111,119,115,59,1,8644,97,114,112,111,111,110,115,59,1,8652,105,103,104,116,97,114,114,111,119,115,59,1,8649,113,117,105,103,97,114,114,111,119,59,1,8605,104,114,101,101,116,105,109,101,115,59,1,8908,103,59,1,730,105,110,103,100,111,116,115,101,113,59,1,8787,4,3,97,104,109,16191,16196,16201,114,114,59,1,8644,97,114,59,1,8652,59,1,8207,111,117,115,116,4,2,59,97,16214,16216,1,9137,99,104,101,59,1,9137,109,105,100,59,1,10990,4,4,97,98,112,116,16238,16252,16257,16278,4,2,110,114,16244,16248,103,59,1,10221,114,59,1,8702,114,107,59,1,10215,4,3,97,102,108,16265,16269,16273,114,59,1,10630,59,3,55349,56675,117,115,59,1,10798,105,109,101,115,59,1,10805,4,2,97,112,16291,16304,114,4,2,59,103,16298,16300,1,41,116,59,1,10644,111,108,105,110,116,59,1,10770,97,114,114,59,1,8649,4,4,97,99,104,113,16328,16334,16339,16342,113,117,111,59,1,8250,114,59,3,55349,56519,59,1,8625,4,2,98,117,16348,16351,59,1,93,111,4,2,59,114,16358,16360,1,8217,59,1,8217,4,3,104,105,114,16371,16377,16383,114,101,101,59,1,8908,109,101,115,59,1,8906,105,4,4,59,101,102,108,16394,16396,16399,16402,1,9657,59,1,8885,59,1,9656,116,114,105,59,1,10702,108,117,104,97,114,59,1,10600,59,1,8478,4,19,97,98,99,100,101,102,104,105,108,109,111,112,113,114,115,116,117,119,122,16459,16466,16472,16572,16590,16672,16687,16746,16844,16850,16924,16963,16988,17115,17121,17154,17206,17614,17656,99,117,116,101,59,1,347,113,117,111,59,1,8218,4,10,59,69,97,99,101,105,110,112,115,121,16494,16496,16499,16513,16518,16531,16536,16556,16564,16569,1,8827,59,1,10932,4,2,112,114,16505,16508,59,1,10936,111,110,59,1,353,117,101,59,1,8829,4,2,59,100,16524,16526,1,10928,105,108,59,1,351,114,99,59,1,349,4,3,69,97,115,16544,16547,16551,59,1,10934,112,59,1,10938,105,109,59,1,8937,111,108,105,110,116,59,1,10771,105,109,59,1,8831,59,1,1089,111,116,4,3,59,98,101,16582,16584,16587,1,8901,59,1,8865,59,1,10854,4,7,65,97,99,109,115,116,120,16606,16611,16634,16642,16646,16652,16668,114,114,59,1,8664,114,4,2,104,114,16618,16622,107,59,1,10533,4,2,59,111,16628,16630,1,8600,119,59,1,8600,116,5,167,1,59,16640,1,167,105,59,1,59,119,97,114,59,1,10537,109,4,2,105,110,16659,16665,110,117,115,59,1,8726,59,1,8726,116,59,1,10038,114,4,2,59,111,16679,16682,3,55349,56624,119,110,59,1,8994,4,4,97,99,111,121,16697,16702,16716,16739,114,112,59,1,9839,4,2,104,121,16708,16713,99,121,59,1,1097,59,1,1096,114,116,4,2,109,112,16724,16729,105,100,59,1,8739,97,114,97,108,108,101,108,59,1,8741,5,173,1,59,16744,1,173,4,2,103,109,16752,16770,109,97,4,3,59,102,118,16762,16764,16767,1,963,59,1,962,59,1,962,4,8,59,100,101,103,108,110,112,114,16788,16790,16795,16806,16817,16828,16832,16838,1,8764,111,116,59,1,10858,4,2,59,113,16801,16803,1,8771,59,1,8771,4,2,59,69,16812,16814,1,10910,59,1,10912,4,2,59,69,16823,16825,1,10909,59,1,10911,101,59,1,8774,108,117,115,59,1,10788,97,114,114,59,1,10610,97,114,114,59,1,8592,4,4,97,101,105,116,16860,16883,16891,16904,4,2,108,115,16866,16878,108,115,101,116,109,105,110,117,115,59,1,8726,104,112,59,1,10803,112,97,114,115,108,59,1,10724,4,2,100,108,16897,16900,59,1,8739,101,59,1,8995,4,2,59,101,16910,16912,1,10922,4,2,59,115,16918,16920,1,10924,59,3,10924,65024,4,3,102,108,112,16932,16938,16958,116,99,121,59,1,1100,4,2,59,98,16944,16946,1,47,4,2,59,97,16952,16954,1,10692,114,59,1,9023,102,59,3,55349,56676,97,4,2,100,114,16970,16985,101,115,4,2,59,117,16978,16980,1,9824,105,116,59,1,9824,59,1,8741,4,3,99,115,117,16996,17028,17089,4,2,97,117,17002,17015,112,4,2,59,115,17009,17011,1,8851,59,3,8851,65024,112,4,2,59,115,17022,17024,1,8852,59,3,8852,65024,117,4,2,98,112,17035,17062,4,3,59,101,115,17043,17045,17048,1,8847,59,1,8849,101,116,4,2,59,101,17056,17058,1,8847,113,59,1,8849,4,3,59,101,115,17070,17072,17075,1,8848,59,1,8850,101,116,4,2,59,101,17083,17085,1,8848,113,59,1,8850,4,3,59,97,102,17097,17099,17112,1,9633,114,4,2,101,102,17106,17109,59,1,9633,59,1,9642,59,1,9642,97,114,114,59,1,8594,4,4,99,101,109,116,17131,17136,17142,17148,114,59,3,55349,56520,116,109,110,59,1,8726,105,108,101,59,1,8995,97,114,102,59,1,8902,4,2,97,114,17160,17172,114,4,2,59,102,17167,17169,1,9734,59,1,9733,4,2,97,110,17178,17202,105,103,104,116,4,2,101,112,17188,17197,112,115,105,108,111,110,59,1,1013,104,105,59,1,981,115,59,1,175,4,5,98,99,109,110,112,17218,17351,17420,17423,17427,4,9,59,69,100,101,109,110,112,114,115,17238,17240,17243,17248,17261,17267,17279,17285,17291,1,8834,59,1,10949,111,116,59,1,10941,4,2,59,100,17254,17256,1,8838,111,116,59,1,10947,117,108,116,59,1,10945,4,2,69,101,17273,17276,59,1,10955,59,1,8842,108,117,115,59,1,10943,97,114,114,59,1,10617,4,3,101,105,117,17299,17335,17339,116,4,3,59,101,110,17308,17310,17322,1,8834,113,4,2,59,113,17317,17319,1,8838,59,1,10949,101,113,4,2,59,113,17330,17332,1,8842,59,1,10955,109,59,1,10951,4,2,98,112,17345,17348,59,1,10965,59,1,10963,99,4,6,59,97,99,101,110,115,17366,17368,17376,17385,17389,17415,1,8827,112,112,114,111,120,59,1,10936,117,114,108,121,101,113,59,1,8829,113,59,1,10928,4,3,97,101,115,17397,17405,17410,112,112,114,111,120,59,1,10938,113,113,59,1,10934,105,109,59,1,8937,105,109,59,1,8831,59,1,8721,103,59,1,9834,4,13,49,50,51,59,69,100,101,104,108,109,110,112,115,17455,17462,17469,17476,17478,17481,17496,17509,17524,17530,17536,17548,17554,5,185,1,59,17460,1,185,5,178,1,59,17467,1,178,5,179,1,59,17474,1,179,1,8835,59,1,10950,4,2,111,115,17487,17491,116,59,1,10942,117,98,59,1,10968,4,2,59,100,17502,17504,1,8839,111,116,59,1,10948,115,4,2,111,117,17516,17520,108,59,1,10185,98,59,1,10967,97,114,114,59,1,10619,117,108,116,59,1,10946,4,2,69,101,17542,17545,59,1,10956,59,1,8843,108,117,115,59,1,10944,4,3,101,105,117,17562,17598,17602,116,4,3,59,101,110,17571,17573,17585,1,8835,113,4,2,59,113,17580,17582,1,8839,59,1,10950,101,113,4,2,59,113,17593,17595,1,8843,59,1,10956,109,59,1,10952,4,2,98,112,17608,17611,59,1,10964,59,1,10966,4,3,65,97,110,17622,17627,17650,114,114,59,1,8665,114,4,2,104,114,17634,17638,107,59,1,10534,4,2,59,111,17644,17646,1,8601,119,59,1,8601,119,97,114,59,1,10538,108,105,103,5,223,1,59,17664,1,223,4,13,97,98,99,100,101,102,104,105,111,112,114,115,119,17694,17709,17714,17737,17742,17749,17754,17860,17905,17957,17964,18090,18122,4,2,114,117,17700,17706,103,101,116,59,1,8982,59,1,964,114,107,59,1,9140,4,3,97,101,121,17722,17728,17734,114,111,110,59,1,357,100,105,108,59,1,355,59,1,1090,111,116,59,1,8411,108,114,101,99,59,1,8981,114,59,3,55349,56625,4,4,101,105,107,111,17764,17805,17836,17851,4,2,114,116,17770,17786,101,4,2,52,102,17777,17780,59,1,8756,111,114,101,59,1,8756,97,4,3,59,115,118,17795,17797,17802,1,952,121,109,59,1,977,59,1,977,4,2,99,110,17811,17831,107,4,2,97,115,17818,17826,112,112,114,111,120,59,1,8776,105,109,59,1,8764,115,112,59,1,8201,4,2,97,115,17842,17846,112,59,1,8776,105,109,59,1,8764,114,110,5,254,1,59,17858,1,254,4,3,108,109,110,17868,17873,17901,100,101,59,1,732,101,115,5,215,3,59,98,100,17884,17886,17898,1,215,4,2,59,97,17892,17894,1,8864,114,59,1,10801,59,1,10800,116,59,1,8749,4,3,101,112,115,17913,17917,17953,97,59,1,10536,4,4,59,98,99,102,17927,17929,17934,17939,1,8868,111,116,59,1,9014,105,114,59,1,10993,4,2,59,111,17945,17948,3,55349,56677,114,107,59,1,10970,97,59,1,10537,114,105,109,101,59,1,8244,4,3,97,105,112,17972,17977,18082,100,101,59,1,8482,4,7,97,100,101,109,112,115,116,17993,18051,18056,18059,18066,18072,18076,110,103,108,101,4,5,59,100,108,113,114,18009,18011,18017,18032,18035,1,9653,111,119,110,59,1,9663,101,102,116,4,2,59,101,18026,18028,1,9667,113,59,1,8884,59,1,8796,105,103,104,116,4,2,59,101,18045,18047,1,9657,113,59,1,8885,111,116,59,1,9708,59,1,8796,105,110,117,115,59,1,10810,108,117,115,59,1,10809,98,59,1,10701,105,109,101,59,1,10811,101,122,105,117,109,59,1,9186,4,3,99,104,116,18098,18111,18116,4,2,114,121,18104,18108,59,3,55349,56521,59,1,1094,99,121,59,1,1115,114,111,107,59,1,359,4,2,105,111,18128,18133,120,116,59,1,8812,104,101,97,100,4,2,108,114,18143,18154,101,102,116,97,114,114,111,119,59,1,8606,105,103,104,116,97,114,114,111,119,59,1,8608,4,18,65,72,97,98,99,100,102,103,104,108,109,111,112,114,115,116,117,119,18204,18209,18214,18234,18250,18268,18292,18308,18319,18343,18379,18397,18413,18504,18547,18553,18584,18603,114,114,59,1,8657,97,114,59,1,10595,4,2,99,114,18220,18230,117,116,101,5,250,1,59,18228,1,250,114,59,1,8593,114,4,2,99,101,18241,18245,121,59,1,1118,118,101,59,1,365,4,2,105,121,18256,18265,114,99,5,251,1,59,18263,1,251,59,1,1091,4,3,97,98,104,18276,18281,18287,114,114,59,1,8645,108,97,99,59,1,369,97,114,59,1,10606,4,2,105,114,18298,18304,115,104,116,59,1,10622,59,3,55349,56626,114,97,118,101,5,249,1,59,18317,1,249,4,2,97,98,18325,18338,114,4,2,108,114,18332,18335,59,1,8639,59,1,8638,108,107,59,1,9600,4,2,99,116,18349,18374,4,2,111,114,18355,18369,114,110,4,2,59,101,18363,18365,1,8988,114,59,1,8988,111,112,59,1,8975,114,105,59,1,9720,4,2,97,108,18385,18390,99,114,59,1,363,5,168,1,59,18395,1,168,4,2,103,112,18403,18408,111,110,59,1,371,102,59,3,55349,56678,4,6,97,100,104,108,115,117,18427,18434,18445,18470,18475,18494,114,114,111,119,59,1,8593,111,119,110,97,114,114,111,119,59,1,8597,97,114,112,111,111,110,4,2,108,114,18457,18463,101,102,116,59,1,8639,105,103,104,116,59,1,8638,117,115,59,1,8846,105,4,3,59,104,108,18484,18486,18489,1,965,59,1,978,111,110,59,1,965,112,97,114,114,111,119,115,59,1,8648,4,3,99,105,116,18512,18537,18542,4,2,111,114,18518,18532,114,110,4,2,59,101,18526,18528,1,8989,114,59,1,8989,111,112,59,1,8974,110,103,59,1,367,114,105,59,1,9721,99,114,59,3,55349,56522,4,3,100,105,114,18561,18566,18572,111,116,59,1,8944,108,100,101,59,1,361,105,4,2,59,102,18579,18581,1,9653,59,1,9652,4,2,97,109,18590,18595,114,114,59,1,8648,108,5,252,1,59,18601,1,252,97,110,103,108,101,59,1,10663,4,15,65,66,68,97,99,100,101,102,108,110,111,112,114,115,122,18643,18648,18661,18667,18847,18851,18857,18904,18909,18915,18931,18937,18943,18949,18996,114,114,59,1,8661,97,114,4,2,59,118,18656,18658,1,10984,59,1,10985,97,115,104,59,1,8872,4,2,110,114,18673,18679,103,114,116,59,1,10652,4,7,101,107,110,112,114,115,116,18695,18704,18711,18720,18742,18754,18810,112,115,105,108,111,110,59,1,1013,97,112,112,97,59,1,1008,111,116,104,105,110,103,59,1,8709,4,3,104,105,114,18728,18732,18735,105,59,1,981,59,1,982,111,112,116,111,59,1,8733,4,2,59,104,18748,18750,1,8597,111,59,1,1009,4,2,105,117,18760,18766,103,109,97,59,1,962,4,2,98,112,18772,18791,115,101,116,110,101,113,4,2,59,113,18784,18787,3,8842,65024,59,3,10955,65024,115,101,116,110,101,113,4,2,59,113,18803,18806,3,8843,65024,59,3,10956,65024,4,2,104,114,18816,18822,101,116,97,59,1,977,105,97,110,103,108,101,4,2,108,114,18834,18840,101,102,116,59,1,8882,105,103,104,116,59,1,8883,121,59,1,1074,97,115,104,59,1,8866,4,3,101,108,114,18865,18884,18890,4,3,59,98,101,18873,18875,18880,1,8744,97,114,59,1,8891,113,59,1,8794,108,105,112,59,1,8942,4,2,98,116,18896,18901,97,114,59,1,124,59,1,124,114,59,3,55349,56627,116,114,105,59,1,8882,115,117,4,2,98,112,18923,18927,59,3,8834,8402,59,3,8835,8402,112,102,59,3,55349,56679,114,111,112,59,1,8733,116,114,105,59,1,8883,4,2,99,117,18955,18960,114,59,3,55349,56523,4,2,98,112,18966,18981,110,4,2,69,101,18973,18977,59,3,10955,65024,59,3,8842,65024,110,4,2,69,101,18988,18992,59,3,10956,65024,59,3,8843,65024,105,103,122,97,103,59,1,10650,4,7,99,101,102,111,112,114,115,19020,19026,19061,19066,19072,19075,19089,105,114,99,59,1,373,4,2,100,105,19032,19055,4,2,98,103,19038,19043,97,114,59,1,10847,101,4,2,59,113,19050,19052,1,8743,59,1,8793,101,114,112,59,1,8472,114,59,3,55349,56628,112,102,59,3,55349,56680,59,1,8472,4,2,59,101,19081,19083,1,8768,97,116,104,59,1,8768,99,114,59,3,55349,56524,4,14,99,100,102,104,105,108,109,110,111,114,115,117,118,119,19125,19146,19152,19157,19173,19176,19192,19197,19202,19236,19252,19269,19286,19291,4,3,97,105,117,19133,19137,19142,112,59,1,8898,114,99,59,1,9711,112,59,1,8899,116,114,105,59,1,9661,114,59,3,55349,56629,4,2,65,97,19163,19168,114,114,59,1,10234,114,114,59,1,10231,59,1,958,4,2,65,97,19182,19187,114,114,59,1,10232,114,114,59,1,10229,97,112,59,1,10236,105,115,59,1,8955,4,3,100,112,116,19210,19215,19230,111,116,59,1,10752,4,2,102,108,19221,19225,59,3,55349,56681,117,115,59,1,10753,105,109,101,59,1,10754,4,2,65,97,19242,19247,114,114,59,1,10233,114,114,59,1,10230,4,2,99,113,19258,19263,114,59,3,55349,56525,99,117,112,59,1,10758,4,2,112,116,19275,19281,108,117,115,59,1,10756,114,105,59,1,9651,101,101,59,1,8897,101,100,103,101,59,1,8896,4,8,97,99,101,102,105,111,115,117,19316,19335,19349,19357,19362,19367,19373,19379,99,4,2,117,121,19323,19332,116,101,5,253,1,59,19330,1,253,59,1,1103,4,2,105,121,19341,19346,114,99,59,1,375,59,1,1099,110,5,165,1,59,19355,1,165,114,59,3,55349,56630,99,121,59,1,1111,112,102,59,3,55349,56682,99,114,59,3,55349,56526,4,2,99,109,19385,19389,121,59,1,1102,108,5,255,1,59,19395,1,255,4,10,97,99,100,101,102,104,105,111,115,119,19419,19426,19441,19446,19462,19467,19472,19480,19486,19492,99,117,116,101,59,1,378,4,2,97,121,19432,19438,114,111,110,59,1,382,59,1,1079,111,116,59,1,380,4,2,101,116,19452,19458,116,114,102,59,1,8488,97,59,1,950,114,59,3,55349,56631,99,121,59,1,1078,103,114,97,114,114,59,1,8669,112,102,59,3,55349,56683,99,114,59,3,55349,56527,4,2,106,110,19498,19501,59,1,8205,106,59,1,8204]);

  const Preprocessor$2 = preprocessor$1;
  const unicode$5 = unicode$7;
  const neTree$1 = namedEntityData$1;
  const ERR$4 = errorCodes$1;

  //Aliases
  const $$c = unicode$5.CODE_POINTS;
  const $$$1 = unicode$5.CODE_POINT_SEQUENCES;

  //C1 Unicode control character reference replacements
  const C1_CONTROLS_REFERENCE_REPLACEMENTS$1 = {
      0x80: 0x20ac,
      0x82: 0x201a,
      0x83: 0x0192,
      0x84: 0x201e,
      0x85: 0x2026,
      0x86: 0x2020,
      0x87: 0x2021,
      0x88: 0x02c6,
      0x89: 0x2030,
      0x8a: 0x0160,
      0x8b: 0x2039,
      0x8c: 0x0152,
      0x8e: 0x017d,
      0x91: 0x2018,
      0x92: 0x2019,
      0x93: 0x201c,
      0x94: 0x201d,
      0x95: 0x2022,
      0x96: 0x2013,
      0x97: 0x2014,
      0x98: 0x02dc,
      0x99: 0x2122,
      0x9a: 0x0161,
      0x9b: 0x203a,
      0x9c: 0x0153,
      0x9e: 0x017e,
      0x9f: 0x0178
  };

  // Named entity tree flags
  const HAS_DATA_FLAG$1 = 1 << 0;
  const DATA_DUPLET_FLAG$1 = 1 << 1;
  const HAS_BRANCHES_FLAG$1 = 1 << 2;
  const MAX_BRANCH_MARKER_VALUE$1 = HAS_DATA_FLAG$1 | DATA_DUPLET_FLAG$1 | HAS_BRANCHES_FLAG$1;

  //States
  const DATA_STATE$1 = 'DATA_STATE';
  const RCDATA_STATE$1 = 'RCDATA_STATE';
  const RAWTEXT_STATE$1 = 'RAWTEXT_STATE';
  const SCRIPT_DATA_STATE$1 = 'SCRIPT_DATA_STATE';
  const PLAINTEXT_STATE$1 = 'PLAINTEXT_STATE';
  const TAG_OPEN_STATE$1 = 'TAG_OPEN_STATE';
  const END_TAG_OPEN_STATE$1 = 'END_TAG_OPEN_STATE';
  const TAG_NAME_STATE$1 = 'TAG_NAME_STATE';
  const RCDATA_LESS_THAN_SIGN_STATE$1 = 'RCDATA_LESS_THAN_SIGN_STATE';
  const RCDATA_END_TAG_OPEN_STATE$1 = 'RCDATA_END_TAG_OPEN_STATE';
  const RCDATA_END_TAG_NAME_STATE$1 = 'RCDATA_END_TAG_NAME_STATE';
  const RAWTEXT_LESS_THAN_SIGN_STATE$1 = 'RAWTEXT_LESS_THAN_SIGN_STATE';
  const RAWTEXT_END_TAG_OPEN_STATE$1 = 'RAWTEXT_END_TAG_OPEN_STATE';
  const RAWTEXT_END_TAG_NAME_STATE$1 = 'RAWTEXT_END_TAG_NAME_STATE';
  const SCRIPT_DATA_LESS_THAN_SIGN_STATE$1 = 'SCRIPT_DATA_LESS_THAN_SIGN_STATE';
  const SCRIPT_DATA_END_TAG_OPEN_STATE$1 = 'SCRIPT_DATA_END_TAG_OPEN_STATE';
  const SCRIPT_DATA_END_TAG_NAME_STATE$1 = 'SCRIPT_DATA_END_TAG_NAME_STATE';
  const SCRIPT_DATA_ESCAPE_START_STATE$1 = 'SCRIPT_DATA_ESCAPE_START_STATE';
  const SCRIPT_DATA_ESCAPE_START_DASH_STATE$1 = 'SCRIPT_DATA_ESCAPE_START_DASH_STATE';
  const SCRIPT_DATA_ESCAPED_STATE$1 = 'SCRIPT_DATA_ESCAPED_STATE';
  const SCRIPT_DATA_ESCAPED_DASH_STATE$1 = 'SCRIPT_DATA_ESCAPED_DASH_STATE';
  const SCRIPT_DATA_ESCAPED_DASH_DASH_STATE$1 = 'SCRIPT_DATA_ESCAPED_DASH_DASH_STATE';
  const SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN_STATE$1 = 'SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN_STATE';
  const SCRIPT_DATA_ESCAPED_END_TAG_OPEN_STATE$1 = 'SCRIPT_DATA_ESCAPED_END_TAG_OPEN_STATE';
  const SCRIPT_DATA_ESCAPED_END_TAG_NAME_STATE$1 = 'SCRIPT_DATA_ESCAPED_END_TAG_NAME_STATE';
  const SCRIPT_DATA_DOUBLE_ESCAPE_START_STATE$1 = 'SCRIPT_DATA_DOUBLE_ESCAPE_START_STATE';
  const SCRIPT_DATA_DOUBLE_ESCAPED_STATE$1 = 'SCRIPT_DATA_DOUBLE_ESCAPED_STATE';
  const SCRIPT_DATA_DOUBLE_ESCAPED_DASH_STATE$1 = 'SCRIPT_DATA_DOUBLE_ESCAPED_DASH_STATE';
  const SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH_STATE$1 = 'SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH_STATE';
  const SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN_STATE$1 = 'SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN_STATE';
  const SCRIPT_DATA_DOUBLE_ESCAPE_END_STATE$1 = 'SCRIPT_DATA_DOUBLE_ESCAPE_END_STATE';
  const BEFORE_ATTRIBUTE_NAME_STATE$1 = 'BEFORE_ATTRIBUTE_NAME_STATE';
  const ATTRIBUTE_NAME_STATE$1 = 'ATTRIBUTE_NAME_STATE';
  const AFTER_ATTRIBUTE_NAME_STATE$1 = 'AFTER_ATTRIBUTE_NAME_STATE';
  const BEFORE_ATTRIBUTE_VALUE_STATE$1 = 'BEFORE_ATTRIBUTE_VALUE_STATE';
  const ATTRIBUTE_VALUE_DOUBLE_QUOTED_STATE$1 = 'ATTRIBUTE_VALUE_DOUBLE_QUOTED_STATE';
  const ATTRIBUTE_VALUE_SINGLE_QUOTED_STATE$1 = 'ATTRIBUTE_VALUE_SINGLE_QUOTED_STATE';
  const ATTRIBUTE_VALUE_UNQUOTED_STATE$1 = 'ATTRIBUTE_VALUE_UNQUOTED_STATE';
  const AFTER_ATTRIBUTE_VALUE_QUOTED_STATE$1 = 'AFTER_ATTRIBUTE_VALUE_QUOTED_STATE';
  const SELF_CLOSING_START_TAG_STATE$1 = 'SELF_CLOSING_START_TAG_STATE';
  const BOGUS_COMMENT_STATE$1 = 'BOGUS_COMMENT_STATE';
  const MARKUP_DECLARATION_OPEN_STATE$1 = 'MARKUP_DECLARATION_OPEN_STATE';
  const COMMENT_START_STATE$1 = 'COMMENT_START_STATE';
  const COMMENT_START_DASH_STATE$1 = 'COMMENT_START_DASH_STATE';
  const COMMENT_STATE$1 = 'COMMENT_STATE';
  const COMMENT_LESS_THAN_SIGN_STATE$1 = 'COMMENT_LESS_THAN_SIGN_STATE';
  const COMMENT_LESS_THAN_SIGN_BANG_STATE$1 = 'COMMENT_LESS_THAN_SIGN_BANG_STATE';
  const COMMENT_LESS_THAN_SIGN_BANG_DASH_STATE$1 = 'COMMENT_LESS_THAN_SIGN_BANG_DASH_STATE';
  const COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH_STATE$1 = 'COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH_STATE';
  const COMMENT_END_DASH_STATE$1 = 'COMMENT_END_DASH_STATE';
  const COMMENT_END_STATE$1 = 'COMMENT_END_STATE';
  const COMMENT_END_BANG_STATE$1 = 'COMMENT_END_BANG_STATE';
  const DOCTYPE_STATE$1 = 'DOCTYPE_STATE';
  const BEFORE_DOCTYPE_NAME_STATE$1 = 'BEFORE_DOCTYPE_NAME_STATE';
  const DOCTYPE_NAME_STATE$1 = 'DOCTYPE_NAME_STATE';
  const AFTER_DOCTYPE_NAME_STATE$1 = 'AFTER_DOCTYPE_NAME_STATE';
  const AFTER_DOCTYPE_PUBLIC_KEYWORD_STATE$1 = 'AFTER_DOCTYPE_PUBLIC_KEYWORD_STATE';
  const BEFORE_DOCTYPE_PUBLIC_IDENTIFIER_STATE$1 = 'BEFORE_DOCTYPE_PUBLIC_IDENTIFIER_STATE';
  const DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED_STATE$1 = 'DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED_STATE';
  const DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED_STATE$1 = 'DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED_STATE';
  const AFTER_DOCTYPE_PUBLIC_IDENTIFIER_STATE$1 = 'AFTER_DOCTYPE_PUBLIC_IDENTIFIER_STATE';
  const BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS_STATE$1 = 'BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS_STATE';
  const AFTER_DOCTYPE_SYSTEM_KEYWORD_STATE$1 = 'AFTER_DOCTYPE_SYSTEM_KEYWORD_STATE';
  const BEFORE_DOCTYPE_SYSTEM_IDENTIFIER_STATE$1 = 'BEFORE_DOCTYPE_SYSTEM_IDENTIFIER_STATE';
  const DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED_STATE$1 = 'DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED_STATE';
  const DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED_STATE$1 = 'DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED_STATE';
  const AFTER_DOCTYPE_SYSTEM_IDENTIFIER_STATE$1 = 'AFTER_DOCTYPE_SYSTEM_IDENTIFIER_STATE';
  const BOGUS_DOCTYPE_STATE$1 = 'BOGUS_DOCTYPE_STATE';
  const CDATA_SECTION_STATE$1 = 'CDATA_SECTION_STATE';
  const CDATA_SECTION_BRACKET_STATE$1 = 'CDATA_SECTION_BRACKET_STATE';
  const CDATA_SECTION_END_STATE$1 = 'CDATA_SECTION_END_STATE';
  const CHARACTER_REFERENCE_STATE$1 = 'CHARACTER_REFERENCE_STATE';
  const NAMED_CHARACTER_REFERENCE_STATE$1 = 'NAMED_CHARACTER_REFERENCE_STATE';
  const AMBIGUOUS_AMPERSAND_STATE$1 = 'AMBIGUOS_AMPERSAND_STATE';
  const NUMERIC_CHARACTER_REFERENCE_STATE$1 = 'NUMERIC_CHARACTER_REFERENCE_STATE';
  const HEXADEMICAL_CHARACTER_REFERENCE_START_STATE$1 = 'HEXADEMICAL_CHARACTER_REFERENCE_START_STATE';
  const DECIMAL_CHARACTER_REFERENCE_START_STATE$1 = 'DECIMAL_CHARACTER_REFERENCE_START_STATE';
  const HEXADEMICAL_CHARACTER_REFERENCE_STATE$1 = 'HEXADEMICAL_CHARACTER_REFERENCE_STATE';
  const DECIMAL_CHARACTER_REFERENCE_STATE$1 = 'DECIMAL_CHARACTER_REFERENCE_STATE';
  const NUMERIC_CHARACTER_REFERENCE_END_STATE$1 = 'NUMERIC_CHARACTER_REFERENCE_END_STATE';

  //Utils

  //OPTIMIZATION: these utility functions should not be moved out of this module. V8 Crankshaft will not inline
  //this functions if they will be situated in another module due to context switch.
  //Always perform inlining check before modifying this functions ('node --trace-inlining').
  function isWhitespace$1(cp) {
      return cp === $$c.SPACE || cp === $$c.LINE_FEED || cp === $$c.TABULATION || cp === $$c.FORM_FEED;
  }

  function isAsciiDigit$1(cp) {
      return cp >= $$c.DIGIT_0 && cp <= $$c.DIGIT_9;
  }

  function isAsciiUpper$1(cp) {
      return cp >= $$c.LATIN_CAPITAL_A && cp <= $$c.LATIN_CAPITAL_Z;
  }

  function isAsciiLower$1(cp) {
      return cp >= $$c.LATIN_SMALL_A && cp <= $$c.LATIN_SMALL_Z;
  }

  function isAsciiLetter$1(cp) {
      return isAsciiLower$1(cp) || isAsciiUpper$1(cp);
  }

  function isAsciiAlphaNumeric$1(cp) {
      return isAsciiLetter$1(cp) || isAsciiDigit$1(cp);
  }

  function isAsciiUpperHexDigit$1(cp) {
      return cp >= $$c.LATIN_CAPITAL_A && cp <= $$c.LATIN_CAPITAL_F;
  }

  function isAsciiLowerHexDigit$1(cp) {
      return cp >= $$c.LATIN_SMALL_A && cp <= $$c.LATIN_SMALL_F;
  }

  function isAsciiHexDigit$1(cp) {
      return isAsciiDigit$1(cp) || isAsciiUpperHexDigit$1(cp) || isAsciiLowerHexDigit$1(cp);
  }

  function toAsciiLowerCodePoint$1(cp) {
      return cp + 0x0020;
  }

  //NOTE: String.fromCharCode() function can handle only characters from BMP subset.
  //So, we need to workaround this manually.
  //(see: https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/String/fromCharCode#Getting_it_to_work_with_higher_values)
  function toChar$1(cp) {
      if (cp <= 0xffff) {
          return String.fromCharCode(cp);
      }

      cp -= 0x10000;
      return String.fromCharCode(((cp >>> 10) & 0x3ff) | 0xd800) + String.fromCharCode(0xdc00 | (cp & 0x3ff));
  }

  function toAsciiLowerChar$1(cp) {
      return String.fromCharCode(toAsciiLowerCodePoint$1(cp));
  }

  function findNamedEntityTreeBranch$1(nodeIx, cp) {
      const branchCount = neTree$1[++nodeIx];
      let lo = ++nodeIx;
      let hi = lo + branchCount - 1;

      while (lo <= hi) {
          const mid = (lo + hi) >>> 1;
          const midCp = neTree$1[mid];

          if (midCp < cp) {
              lo = mid + 1;
          } else if (midCp > cp) {
              hi = mid - 1;
          } else {
              return neTree$1[mid + branchCount];
          }
      }

      return -1;
  }

  //Tokenizer
  class Tokenizer$9 {
      constructor() {
          this.preprocessor = new Preprocessor$2();

          this.tokenQueue = [];

          this.allowCDATA = false;

          this.state = DATA_STATE$1;
          this.returnState = '';

          this.charRefCode = -1;
          this.tempBuff = [];
          this.lastStartTagName = '';

          this.consumedAfterSnapshot = -1;
          this.active = false;

          this.currentCharacterToken = null;
          this.currentToken = null;
          this.currentAttr = null;
      }

      //Errors
      _err() {
          // NOTE: err reporting is noop by default. Enabled by mixin.
      }

      _errOnNextCodePoint(err) {
          this._consume();
          this._err(err);
          this._unconsume();
      }

      //API
      getNextToken() {
          while (!this.tokenQueue.length && this.active) {
              this.consumedAfterSnapshot = 0;

              const cp = this._consume();

              if (!this._ensureHibernation()) {
                  this[this.state](cp);
              }
          }

          return this.tokenQueue.shift();
      }

      write(chunk, isLastChunk) {
          this.active = true;
          this.preprocessor.write(chunk, isLastChunk);
      }

      insertHtmlAtCurrentPos(chunk) {
          this.active = true;
          this.preprocessor.insertHtmlAtCurrentPos(chunk);
      }

      //Hibernation
      _ensureHibernation() {
          if (this.preprocessor.endOfChunkHit) {
              for (; this.consumedAfterSnapshot > 0; this.consumedAfterSnapshot--) {
                  this.preprocessor.retreat();
              }

              this.active = false;
              this.tokenQueue.push({ type: Tokenizer$9.HIBERNATION_TOKEN });

              return true;
          }

          return false;
      }

      //Consumption
      _consume() {
          this.consumedAfterSnapshot++;
          return this.preprocessor.advance();
      }

      _unconsume() {
          this.consumedAfterSnapshot--;
          this.preprocessor.retreat();
      }

      _reconsumeInState(state) {
          this.state = state;
          this._unconsume();
      }

      _consumeSequenceIfMatch(pattern, startCp, caseSensitive) {
          let consumedCount = 0;
          let isMatch = true;
          const patternLength = pattern.length;
          let patternPos = 0;
          let cp = startCp;
          let patternCp = void 0;

          for (; patternPos < patternLength; patternPos++) {
              if (patternPos > 0) {
                  cp = this._consume();
                  consumedCount++;
              }

              if (cp === $$c.EOF) {
                  isMatch = false;
                  break;
              }

              patternCp = pattern[patternPos];

              if (cp !== patternCp && (caseSensitive || cp !== toAsciiLowerCodePoint$1(patternCp))) {
                  isMatch = false;
                  break;
              }
          }

          if (!isMatch) {
              while (consumedCount--) {
                  this._unconsume();
              }
          }

          return isMatch;
      }

      //Temp buffer
      _isTempBufferEqualToScriptString() {
          if (this.tempBuff.length !== $$$1.SCRIPT_STRING.length) {
              return false;
          }

          for (let i = 0; i < this.tempBuff.length; i++) {
              if (this.tempBuff[i] !== $$$1.SCRIPT_STRING[i]) {
                  return false;
              }
          }

          return true;
      }

      //Token creation
      _createStartTagToken() {
          this.currentToken = {
              type: Tokenizer$9.START_TAG_TOKEN,
              tagName: '',
              selfClosing: false,
              ackSelfClosing: false,
              attrs: []
          };
      }

      _createEndTagToken() {
          this.currentToken = {
              type: Tokenizer$9.END_TAG_TOKEN,
              tagName: '',
              selfClosing: false,
              attrs: []
          };
      }

      _createCommentToken() {
          this.currentToken = {
              type: Tokenizer$9.COMMENT_TOKEN,
              data: ''
          };
      }

      _createDoctypeToken(initialName) {
          this.currentToken = {
              type: Tokenizer$9.DOCTYPE_TOKEN,
              name: initialName,
              forceQuirks: false,
              publicId: null,
              systemId: null
          };
      }

      _createCharacterToken(type, ch) {
          this.currentCharacterToken = {
              type: type,
              chars: ch
          };
      }

      _createEOFToken() {
          this.currentToken = { type: Tokenizer$9.EOF_TOKEN };
      }

      //Tag attributes
      _createAttr(attrNameFirstCh) {
          this.currentAttr = {
              name: attrNameFirstCh,
              value: ''
          };
      }

      _leaveAttrName(toState) {
          if (Tokenizer$9.getTokenAttr(this.currentToken, this.currentAttr.name) === null) {
              this.currentToken.attrs.push(this.currentAttr);
          } else {
              this._err(ERR$4.duplicateAttribute);
          }

          this.state = toState;
      }

      _leaveAttrValue(toState) {
          this.state = toState;
      }

      //Token emission
      _emitCurrentToken() {
          this._emitCurrentCharacterToken();

          const ct = this.currentToken;

          this.currentToken = null;

          //NOTE: store emited start tag's tagName to determine is the following end tag token is appropriate.
          if (ct.type === Tokenizer$9.START_TAG_TOKEN) {
              this.lastStartTagName = ct.tagName;
          } else if (ct.type === Tokenizer$9.END_TAG_TOKEN) {
              if (ct.attrs.length > 0) {
                  this._err(ERR$4.endTagWithAttributes);
              }

              if (ct.selfClosing) {
                  this._err(ERR$4.endTagWithTrailingSolidus);
              }
          }

          this.tokenQueue.push(ct);
      }

      _emitCurrentCharacterToken() {
          if (this.currentCharacterToken) {
              this.tokenQueue.push(this.currentCharacterToken);
              this.currentCharacterToken = null;
          }
      }

      _emitEOFToken() {
          this._createEOFToken();
          this._emitCurrentToken();
      }

      //Characters emission

      //OPTIMIZATION: specification uses only one type of character tokens (one token per character).
      //This causes a huge memory overhead and a lot of unnecessary parser loops. parse5 uses 3 groups of characters.
      //If we have a sequence of characters that belong to the same group, parser can process it
      //as a single solid character token.
      //So, there are 3 types of character tokens in parse5:
      //1)NULL_CHARACTER_TOKEN - \u0000-character sequences (e.g. '\u0000\u0000\u0000')
      //2)WHITESPACE_CHARACTER_TOKEN - any whitespace/new-line character sequences (e.g. '\n  \r\t   \f')
      //3)CHARACTER_TOKEN - any character sequence which don't belong to groups 1 and 2 (e.g. 'abcdef1234@@#$%^')
      _appendCharToCurrentCharacterToken(type, ch) {
          if (this.currentCharacterToken && this.currentCharacterToken.type !== type) {
              this._emitCurrentCharacterToken();
          }

          if (this.currentCharacterToken) {
              this.currentCharacterToken.chars += ch;
          } else {
              this._createCharacterToken(type, ch);
          }
      }

      _emitCodePoint(cp) {
          let type = Tokenizer$9.CHARACTER_TOKEN;

          if (isWhitespace$1(cp)) {
              type = Tokenizer$9.WHITESPACE_CHARACTER_TOKEN;
          } else if (cp === $$c.NULL) {
              type = Tokenizer$9.NULL_CHARACTER_TOKEN;
          }

          this._appendCharToCurrentCharacterToken(type, toChar$1(cp));
      }

      _emitSeveralCodePoints(codePoints) {
          for (let i = 0; i < codePoints.length; i++) {
              this._emitCodePoint(codePoints[i]);
          }
      }

      //NOTE: used then we emit character explicitly. This is always a non-whitespace and a non-null character.
      //So we can avoid additional checks here.
      _emitChars(ch) {
          this._appendCharToCurrentCharacterToken(Tokenizer$9.CHARACTER_TOKEN, ch);
      }

      // Character reference helpers
      _matchNamedCharacterReference(startCp) {
          let result = null;
          let excess = 1;
          let i = findNamedEntityTreeBranch$1(0, startCp);

          this.tempBuff.push(startCp);

          while (i > -1) {
              const current = neTree$1[i];
              const inNode = current < MAX_BRANCH_MARKER_VALUE$1;
              const nodeWithData = inNode && current & HAS_DATA_FLAG$1;

              if (nodeWithData) {
                  //NOTE: we use greedy search, so we continue lookup at this point
                  result = current & DATA_DUPLET_FLAG$1 ? [neTree$1[++i], neTree$1[++i]] : [neTree$1[++i]];
                  excess = 0;
              }

              const cp = this._consume();

              this.tempBuff.push(cp);
              excess++;

              if (cp === $$c.EOF) {
                  break;
              }

              if (inNode) {
                  i = current & HAS_BRANCHES_FLAG$1 ? findNamedEntityTreeBranch$1(i, cp) : -1;
              } else {
                  i = cp === current ? ++i : -1;
              }
          }

          while (excess--) {
              this.tempBuff.pop();
              this._unconsume();
          }

          return result;
      }

      _isCharacterReferenceInAttribute() {
          return (
              this.returnState === ATTRIBUTE_VALUE_DOUBLE_QUOTED_STATE$1 ||
              this.returnState === ATTRIBUTE_VALUE_SINGLE_QUOTED_STATE$1 ||
              this.returnState === ATTRIBUTE_VALUE_UNQUOTED_STATE$1
          );
      }

      _isCharacterReferenceAttributeQuirk(withSemicolon) {
          if (!withSemicolon && this._isCharacterReferenceInAttribute()) {
              const nextCp = this._consume();

              this._unconsume();

              return nextCp === $$c.EQUALS_SIGN || isAsciiAlphaNumeric$1(nextCp);
          }

          return false;
      }

      _flushCodePointsConsumedAsCharacterReference() {
          if (this._isCharacterReferenceInAttribute()) {
              for (let i = 0; i < this.tempBuff.length; i++) {
                  this.currentAttr.value += toChar$1(this.tempBuff[i]);
              }
          } else {
              this._emitSeveralCodePoints(this.tempBuff);
          }

          this.tempBuff = [];
      }

      // State machine

      // Data state
      //------------------------------------------------------------------
      [DATA_STATE$1](cp) {
          this.preprocessor.dropParsedChunk();

          if (cp === $$c.LESS_THAN_SIGN) {
              this.state = TAG_OPEN_STATE$1;
          } else if (cp === $$c.AMPERSAND) {
              this.returnState = DATA_STATE$1;
              this.state = CHARACTER_REFERENCE_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this._emitCodePoint(cp);
          } else if (cp === $$c.EOF) {
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      //  RCDATA state
      //------------------------------------------------------------------
      [RCDATA_STATE$1](cp) {
          this.preprocessor.dropParsedChunk();

          if (cp === $$c.AMPERSAND) {
              this.returnState = RCDATA_STATE$1;
              this.state = CHARACTER_REFERENCE_STATE$1;
          } else if (cp === $$c.LESS_THAN_SIGN) {
              this.state = RCDATA_LESS_THAN_SIGN_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this._emitChars(unicode$5.REPLACEMENT_CHARACTER);
          } else if (cp === $$c.EOF) {
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      // RAWTEXT state
      //------------------------------------------------------------------
      [RAWTEXT_STATE$1](cp) {
          this.preprocessor.dropParsedChunk();

          if (cp === $$c.LESS_THAN_SIGN) {
              this.state = RAWTEXT_LESS_THAN_SIGN_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this._emitChars(unicode$5.REPLACEMENT_CHARACTER);
          } else if (cp === $$c.EOF) {
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      // Script data state
      //------------------------------------------------------------------
      [SCRIPT_DATA_STATE$1](cp) {
          this.preprocessor.dropParsedChunk();

          if (cp === $$c.LESS_THAN_SIGN) {
              this.state = SCRIPT_DATA_LESS_THAN_SIGN_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this._emitChars(unicode$5.REPLACEMENT_CHARACTER);
          } else if (cp === $$c.EOF) {
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      // PLAINTEXT state
      //------------------------------------------------------------------
      [PLAINTEXT_STATE$1](cp) {
          this.preprocessor.dropParsedChunk();

          if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this._emitChars(unicode$5.REPLACEMENT_CHARACTER);
          } else if (cp === $$c.EOF) {
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      // Tag open state
      //------------------------------------------------------------------
      [TAG_OPEN_STATE$1](cp) {
          if (cp === $$c.EXCLAMATION_MARK) {
              this.state = MARKUP_DECLARATION_OPEN_STATE$1;
          } else if (cp === $$c.SOLIDUS) {
              this.state = END_TAG_OPEN_STATE$1;
          } else if (isAsciiLetter$1(cp)) {
              this._createStartTagToken();
              this._reconsumeInState(TAG_NAME_STATE$1);
          } else if (cp === $$c.QUESTION_MARK) {
              this._err(ERR$4.unexpectedQuestionMarkInsteadOfTagName);
              this._createCommentToken();
              this._reconsumeInState(BOGUS_COMMENT_STATE$1);
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofBeforeTagName);
              this._emitChars('<');
              this._emitEOFToken();
          } else {
              this._err(ERR$4.invalidFirstCharacterOfTagName);
              this._emitChars('<');
              this._reconsumeInState(DATA_STATE$1);
          }
      }

      // End tag open state
      //------------------------------------------------------------------
      [END_TAG_OPEN_STATE$1](cp) {
          if (isAsciiLetter$1(cp)) {
              this._createEndTagToken();
              this._reconsumeInState(TAG_NAME_STATE$1);
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._err(ERR$4.missingEndTagName);
              this.state = DATA_STATE$1;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofBeforeTagName);
              this._emitChars('</');
              this._emitEOFToken();
          } else {
              this._err(ERR$4.invalidFirstCharacterOfTagName);
              this._createCommentToken();
              this._reconsumeInState(BOGUS_COMMENT_STATE$1);
          }
      }

      // Tag name state
      //------------------------------------------------------------------
      [TAG_NAME_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              this.state = BEFORE_ATTRIBUTE_NAME_STATE$1;
          } else if (cp === $$c.SOLIDUS) {
              this.state = SELF_CLOSING_START_TAG_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else if (isAsciiUpper$1(cp)) {
              this.currentToken.tagName += toAsciiLowerChar$1(cp);
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.currentToken.tagName += unicode$5.REPLACEMENT_CHARACTER;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInTag);
              this._emitEOFToken();
          } else {
              this.currentToken.tagName += toChar$1(cp);
          }
      }

      // RCDATA less-than sign state
      //------------------------------------------------------------------
      [RCDATA_LESS_THAN_SIGN_STATE$1](cp) {
          if (cp === $$c.SOLIDUS) {
              this.tempBuff = [];
              this.state = RCDATA_END_TAG_OPEN_STATE$1;
          } else {
              this._emitChars('<');
              this._reconsumeInState(RCDATA_STATE$1);
          }
      }

      // RCDATA end tag open state
      //------------------------------------------------------------------
      [RCDATA_END_TAG_OPEN_STATE$1](cp) {
          if (isAsciiLetter$1(cp)) {
              this._createEndTagToken();
              this._reconsumeInState(RCDATA_END_TAG_NAME_STATE$1);
          } else {
              this._emitChars('</');
              this._reconsumeInState(RCDATA_STATE$1);
          }
      }

      // RCDATA end tag name state
      //------------------------------------------------------------------
      [RCDATA_END_TAG_NAME_STATE$1](cp) {
          if (isAsciiUpper$1(cp)) {
              this.currentToken.tagName += toAsciiLowerChar$1(cp);
              this.tempBuff.push(cp);
          } else if (isAsciiLower$1(cp)) {
              this.currentToken.tagName += toChar$1(cp);
              this.tempBuff.push(cp);
          } else {
              if (this.lastStartTagName === this.currentToken.tagName) {
                  if (isWhitespace$1(cp)) {
                      this.state = BEFORE_ATTRIBUTE_NAME_STATE$1;
                      return;
                  }

                  if (cp === $$c.SOLIDUS) {
                      this.state = SELF_CLOSING_START_TAG_STATE$1;
                      return;
                  }

                  if (cp === $$c.GREATER_THAN_SIGN) {
                      this.state = DATA_STATE$1;
                      this._emitCurrentToken();
                      return;
                  }
              }

              this._emitChars('</');
              this._emitSeveralCodePoints(this.tempBuff);
              this._reconsumeInState(RCDATA_STATE$1);
          }
      }

      // RAWTEXT less-than sign state
      //------------------------------------------------------------------
      [RAWTEXT_LESS_THAN_SIGN_STATE$1](cp) {
          if (cp === $$c.SOLIDUS) {
              this.tempBuff = [];
              this.state = RAWTEXT_END_TAG_OPEN_STATE$1;
          } else {
              this._emitChars('<');
              this._reconsumeInState(RAWTEXT_STATE$1);
          }
      }

      // RAWTEXT end tag open state
      //------------------------------------------------------------------
      [RAWTEXT_END_TAG_OPEN_STATE$1](cp) {
          if (isAsciiLetter$1(cp)) {
              this._createEndTagToken();
              this._reconsumeInState(RAWTEXT_END_TAG_NAME_STATE$1);
          } else {
              this._emitChars('</');
              this._reconsumeInState(RAWTEXT_STATE$1);
          }
      }

      // RAWTEXT end tag name state
      //------------------------------------------------------------------
      [RAWTEXT_END_TAG_NAME_STATE$1](cp) {
          if (isAsciiUpper$1(cp)) {
              this.currentToken.tagName += toAsciiLowerChar$1(cp);
              this.tempBuff.push(cp);
          } else if (isAsciiLower$1(cp)) {
              this.currentToken.tagName += toChar$1(cp);
              this.tempBuff.push(cp);
          } else {
              if (this.lastStartTagName === this.currentToken.tagName) {
                  if (isWhitespace$1(cp)) {
                      this.state = BEFORE_ATTRIBUTE_NAME_STATE$1;
                      return;
                  }

                  if (cp === $$c.SOLIDUS) {
                      this.state = SELF_CLOSING_START_TAG_STATE$1;
                      return;
                  }

                  if (cp === $$c.GREATER_THAN_SIGN) {
                      this._emitCurrentToken();
                      this.state = DATA_STATE$1;
                      return;
                  }
              }

              this._emitChars('</');
              this._emitSeveralCodePoints(this.tempBuff);
              this._reconsumeInState(RAWTEXT_STATE$1);
          }
      }

      // Script data less-than sign state
      //------------------------------------------------------------------
      [SCRIPT_DATA_LESS_THAN_SIGN_STATE$1](cp) {
          if (cp === $$c.SOLIDUS) {
              this.tempBuff = [];
              this.state = SCRIPT_DATA_END_TAG_OPEN_STATE$1;
          } else if (cp === $$c.EXCLAMATION_MARK) {
              this.state = SCRIPT_DATA_ESCAPE_START_STATE$1;
              this._emitChars('<!');
          } else {
              this._emitChars('<');
              this._reconsumeInState(SCRIPT_DATA_STATE$1);
          }
      }

      // Script data end tag open state
      //------------------------------------------------------------------
      [SCRIPT_DATA_END_TAG_OPEN_STATE$1](cp) {
          if (isAsciiLetter$1(cp)) {
              this._createEndTagToken();
              this._reconsumeInState(SCRIPT_DATA_END_TAG_NAME_STATE$1);
          } else {
              this._emitChars('</');
              this._reconsumeInState(SCRIPT_DATA_STATE$1);
          }
      }

      // Script data end tag name state
      //------------------------------------------------------------------
      [SCRIPT_DATA_END_TAG_NAME_STATE$1](cp) {
          if (isAsciiUpper$1(cp)) {
              this.currentToken.tagName += toAsciiLowerChar$1(cp);
              this.tempBuff.push(cp);
          } else if (isAsciiLower$1(cp)) {
              this.currentToken.tagName += toChar$1(cp);
              this.tempBuff.push(cp);
          } else {
              if (this.lastStartTagName === this.currentToken.tagName) {
                  if (isWhitespace$1(cp)) {
                      this.state = BEFORE_ATTRIBUTE_NAME_STATE$1;
                      return;
                  } else if (cp === $$c.SOLIDUS) {
                      this.state = SELF_CLOSING_START_TAG_STATE$1;
                      return;
                  } else if (cp === $$c.GREATER_THAN_SIGN) {
                      this._emitCurrentToken();
                      this.state = DATA_STATE$1;
                      return;
                  }
              }

              this._emitChars('</');
              this._emitSeveralCodePoints(this.tempBuff);
              this._reconsumeInState(SCRIPT_DATA_STATE$1);
          }
      }

      // Script data escape start state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPE_START_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this.state = SCRIPT_DATA_ESCAPE_START_DASH_STATE$1;
              this._emitChars('-');
          } else {
              this._reconsumeInState(SCRIPT_DATA_STATE$1);
          }
      }

      // Script data escape start dash state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPE_START_DASH_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this.state = SCRIPT_DATA_ESCAPED_DASH_DASH_STATE$1;
              this._emitChars('-');
          } else {
              this._reconsumeInState(SCRIPT_DATA_STATE$1);
          }
      }

      // Script data escaped state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPED_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this.state = SCRIPT_DATA_ESCAPED_DASH_STATE$1;
              this._emitChars('-');
          } else if (cp === $$c.LESS_THAN_SIGN) {
              this.state = SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this._emitChars(unicode$5.REPLACEMENT_CHARACTER);
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInScriptHtmlCommentLikeText);
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      // Script data escaped dash state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPED_DASH_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this.state = SCRIPT_DATA_ESCAPED_DASH_DASH_STATE$1;
              this._emitChars('-');
          } else if (cp === $$c.LESS_THAN_SIGN) {
              this.state = SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.state = SCRIPT_DATA_ESCAPED_STATE$1;
              this._emitChars(unicode$5.REPLACEMENT_CHARACTER);
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInScriptHtmlCommentLikeText);
              this._emitEOFToken();
          } else {
              this.state = SCRIPT_DATA_ESCAPED_STATE$1;
              this._emitCodePoint(cp);
          }
      }

      // Script data escaped dash dash state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPED_DASH_DASH_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this._emitChars('-');
          } else if (cp === $$c.LESS_THAN_SIGN) {
              this.state = SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this.state = SCRIPT_DATA_STATE$1;
              this._emitChars('>');
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.state = SCRIPT_DATA_ESCAPED_STATE$1;
              this._emitChars(unicode$5.REPLACEMENT_CHARACTER);
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInScriptHtmlCommentLikeText);
              this._emitEOFToken();
          } else {
              this.state = SCRIPT_DATA_ESCAPED_STATE$1;
              this._emitCodePoint(cp);
          }
      }

      // Script data escaped less-than sign state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN_STATE$1](cp) {
          if (cp === $$c.SOLIDUS) {
              this.tempBuff = [];
              this.state = SCRIPT_DATA_ESCAPED_END_TAG_OPEN_STATE$1;
          } else if (isAsciiLetter$1(cp)) {
              this.tempBuff = [];
              this._emitChars('<');
              this._reconsumeInState(SCRIPT_DATA_DOUBLE_ESCAPE_START_STATE$1);
          } else {
              this._emitChars('<');
              this._reconsumeInState(SCRIPT_DATA_ESCAPED_STATE$1);
          }
      }

      // Script data escaped end tag open state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPED_END_TAG_OPEN_STATE$1](cp) {
          if (isAsciiLetter$1(cp)) {
              this._createEndTagToken();
              this._reconsumeInState(SCRIPT_DATA_ESCAPED_END_TAG_NAME_STATE$1);
          } else {
              this._emitChars('</');
              this._reconsumeInState(SCRIPT_DATA_ESCAPED_STATE$1);
          }
      }

      // Script data escaped end tag name state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPED_END_TAG_NAME_STATE$1](cp) {
          if (isAsciiUpper$1(cp)) {
              this.currentToken.tagName += toAsciiLowerChar$1(cp);
              this.tempBuff.push(cp);
          } else if (isAsciiLower$1(cp)) {
              this.currentToken.tagName += toChar$1(cp);
              this.tempBuff.push(cp);
          } else {
              if (this.lastStartTagName === this.currentToken.tagName) {
                  if (isWhitespace$1(cp)) {
                      this.state = BEFORE_ATTRIBUTE_NAME_STATE$1;
                      return;
                  }

                  if (cp === $$c.SOLIDUS) {
                      this.state = SELF_CLOSING_START_TAG_STATE$1;
                      return;
                  }

                  if (cp === $$c.GREATER_THAN_SIGN) {
                      this._emitCurrentToken();
                      this.state = DATA_STATE$1;
                      return;
                  }
              }

              this._emitChars('</');
              this._emitSeveralCodePoints(this.tempBuff);
              this._reconsumeInState(SCRIPT_DATA_ESCAPED_STATE$1);
          }
      }

      // Script data double escape start state
      //------------------------------------------------------------------
      [SCRIPT_DATA_DOUBLE_ESCAPE_START_STATE$1](cp) {
          if (isWhitespace$1(cp) || cp === $$c.SOLIDUS || cp === $$c.GREATER_THAN_SIGN) {
              this.state = this._isTempBufferEqualToScriptString()
                  ? SCRIPT_DATA_DOUBLE_ESCAPED_STATE$1
                  : SCRIPT_DATA_ESCAPED_STATE$1;
              this._emitCodePoint(cp);
          } else if (isAsciiUpper$1(cp)) {
              this.tempBuff.push(toAsciiLowerCodePoint$1(cp));
              this._emitCodePoint(cp);
          } else if (isAsciiLower$1(cp)) {
              this.tempBuff.push(cp);
              this._emitCodePoint(cp);
          } else {
              this._reconsumeInState(SCRIPT_DATA_ESCAPED_STATE$1);
          }
      }

      // Script data double escaped state
      //------------------------------------------------------------------
      [SCRIPT_DATA_DOUBLE_ESCAPED_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_DASH_STATE$1;
              this._emitChars('-');
          } else if (cp === $$c.LESS_THAN_SIGN) {
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN_STATE$1;
              this._emitChars('<');
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this._emitChars(unicode$5.REPLACEMENT_CHARACTER);
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInScriptHtmlCommentLikeText);
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      // Script data double escaped dash state
      //------------------------------------------------------------------
      [SCRIPT_DATA_DOUBLE_ESCAPED_DASH_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH_STATE$1;
              this._emitChars('-');
          } else if (cp === $$c.LESS_THAN_SIGN) {
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN_STATE$1;
              this._emitChars('<');
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_STATE$1;
              this._emitChars(unicode$5.REPLACEMENT_CHARACTER);
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInScriptHtmlCommentLikeText);
              this._emitEOFToken();
          } else {
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_STATE$1;
              this._emitCodePoint(cp);
          }
      }

      // Script data double escaped dash dash state
      //------------------------------------------------------------------
      [SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this._emitChars('-');
          } else if (cp === $$c.LESS_THAN_SIGN) {
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN_STATE$1;
              this._emitChars('<');
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this.state = SCRIPT_DATA_STATE$1;
              this._emitChars('>');
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_STATE$1;
              this._emitChars(unicode$5.REPLACEMENT_CHARACTER);
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInScriptHtmlCommentLikeText);
              this._emitEOFToken();
          } else {
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_STATE$1;
              this._emitCodePoint(cp);
          }
      }

      // Script data double escaped less-than sign state
      //------------------------------------------------------------------
      [SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN_STATE$1](cp) {
          if (cp === $$c.SOLIDUS) {
              this.tempBuff = [];
              this.state = SCRIPT_DATA_DOUBLE_ESCAPE_END_STATE$1;
              this._emitChars('/');
          } else {
              this._reconsumeInState(SCRIPT_DATA_DOUBLE_ESCAPED_STATE$1);
          }
      }

      // Script data double escape end state
      //------------------------------------------------------------------
      [SCRIPT_DATA_DOUBLE_ESCAPE_END_STATE$1](cp) {
          if (isWhitespace$1(cp) || cp === $$c.SOLIDUS || cp === $$c.GREATER_THAN_SIGN) {
              this.state = this._isTempBufferEqualToScriptString()
                  ? SCRIPT_DATA_ESCAPED_STATE$1
                  : SCRIPT_DATA_DOUBLE_ESCAPED_STATE$1;

              this._emitCodePoint(cp);
          } else if (isAsciiUpper$1(cp)) {
              this.tempBuff.push(toAsciiLowerCodePoint$1(cp));
              this._emitCodePoint(cp);
          } else if (isAsciiLower$1(cp)) {
              this.tempBuff.push(cp);
              this._emitCodePoint(cp);
          } else {
              this._reconsumeInState(SCRIPT_DATA_DOUBLE_ESCAPED_STATE$1);
          }
      }

      // Before attribute name state
      //------------------------------------------------------------------
      [BEFORE_ATTRIBUTE_NAME_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              return;
          }

          if (cp === $$c.SOLIDUS || cp === $$c.GREATER_THAN_SIGN || cp === $$c.EOF) {
              this._reconsumeInState(AFTER_ATTRIBUTE_NAME_STATE$1);
          } else if (cp === $$c.EQUALS_SIGN) {
              this._err(ERR$4.unexpectedEqualsSignBeforeAttributeName);
              this._createAttr('=');
              this.state = ATTRIBUTE_NAME_STATE$1;
          } else {
              this._createAttr('');
              this._reconsumeInState(ATTRIBUTE_NAME_STATE$1);
          }
      }

      // Attribute name state
      //------------------------------------------------------------------
      [ATTRIBUTE_NAME_STATE$1](cp) {
          if (isWhitespace$1(cp) || cp === $$c.SOLIDUS || cp === $$c.GREATER_THAN_SIGN || cp === $$c.EOF) {
              this._leaveAttrName(AFTER_ATTRIBUTE_NAME_STATE$1);
              this._unconsume();
          } else if (cp === $$c.EQUALS_SIGN) {
              this._leaveAttrName(BEFORE_ATTRIBUTE_VALUE_STATE$1);
          } else if (isAsciiUpper$1(cp)) {
              this.currentAttr.name += toAsciiLowerChar$1(cp);
          } else if (cp === $$c.QUOTATION_MARK || cp === $$c.APOSTROPHE || cp === $$c.LESS_THAN_SIGN) {
              this._err(ERR$4.unexpectedCharacterInAttributeName);
              this.currentAttr.name += toChar$1(cp);
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.currentAttr.name += unicode$5.REPLACEMENT_CHARACTER;
          } else {
              this.currentAttr.name += toChar$1(cp);
          }
      }

      // After attribute name state
      //------------------------------------------------------------------
      [AFTER_ATTRIBUTE_NAME_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              return;
          }

          if (cp === $$c.SOLIDUS) {
              this.state = SELF_CLOSING_START_TAG_STATE$1;
          } else if (cp === $$c.EQUALS_SIGN) {
              this.state = BEFORE_ATTRIBUTE_VALUE_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInTag);
              this._emitEOFToken();
          } else {
              this._createAttr('');
              this._reconsumeInState(ATTRIBUTE_NAME_STATE$1);
          }
      }

      // Before attribute value state
      //------------------------------------------------------------------
      [BEFORE_ATTRIBUTE_VALUE_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              return;
          }

          if (cp === $$c.QUOTATION_MARK) {
              this.state = ATTRIBUTE_VALUE_DOUBLE_QUOTED_STATE$1;
          } else if (cp === $$c.APOSTROPHE) {
              this.state = ATTRIBUTE_VALUE_SINGLE_QUOTED_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._err(ERR$4.missingAttributeValue);
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else {
              this._reconsumeInState(ATTRIBUTE_VALUE_UNQUOTED_STATE$1);
          }
      }

      // Attribute value (double-quoted) state
      //------------------------------------------------------------------
      [ATTRIBUTE_VALUE_DOUBLE_QUOTED_STATE$1](cp) {
          if (cp === $$c.QUOTATION_MARK) {
              this.state = AFTER_ATTRIBUTE_VALUE_QUOTED_STATE$1;
          } else if (cp === $$c.AMPERSAND) {
              this.returnState = ATTRIBUTE_VALUE_DOUBLE_QUOTED_STATE$1;
              this.state = CHARACTER_REFERENCE_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.currentAttr.value += unicode$5.REPLACEMENT_CHARACTER;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInTag);
              this._emitEOFToken();
          } else {
              this.currentAttr.value += toChar$1(cp);
          }
      }

      // Attribute value (single-quoted) state
      //------------------------------------------------------------------
      [ATTRIBUTE_VALUE_SINGLE_QUOTED_STATE$1](cp) {
          if (cp === $$c.APOSTROPHE) {
              this.state = AFTER_ATTRIBUTE_VALUE_QUOTED_STATE$1;
          } else if (cp === $$c.AMPERSAND) {
              this.returnState = ATTRIBUTE_VALUE_SINGLE_QUOTED_STATE$1;
              this.state = CHARACTER_REFERENCE_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.currentAttr.value += unicode$5.REPLACEMENT_CHARACTER;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInTag);
              this._emitEOFToken();
          } else {
              this.currentAttr.value += toChar$1(cp);
          }
      }

      // Attribute value (unquoted) state
      //------------------------------------------------------------------
      [ATTRIBUTE_VALUE_UNQUOTED_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              this._leaveAttrValue(BEFORE_ATTRIBUTE_NAME_STATE$1);
          } else if (cp === $$c.AMPERSAND) {
              this.returnState = ATTRIBUTE_VALUE_UNQUOTED_STATE$1;
              this.state = CHARACTER_REFERENCE_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._leaveAttrValue(DATA_STATE$1);
              this._emitCurrentToken();
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.currentAttr.value += unicode$5.REPLACEMENT_CHARACTER;
          } else if (
              cp === $$c.QUOTATION_MARK ||
              cp === $$c.APOSTROPHE ||
              cp === $$c.LESS_THAN_SIGN ||
              cp === $$c.EQUALS_SIGN ||
              cp === $$c.GRAVE_ACCENT
          ) {
              this._err(ERR$4.unexpectedCharacterInUnquotedAttributeValue);
              this.currentAttr.value += toChar$1(cp);
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInTag);
              this._emitEOFToken();
          } else {
              this.currentAttr.value += toChar$1(cp);
          }
      }

      // After attribute value (quoted) state
      //------------------------------------------------------------------
      [AFTER_ATTRIBUTE_VALUE_QUOTED_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              this._leaveAttrValue(BEFORE_ATTRIBUTE_NAME_STATE$1);
          } else if (cp === $$c.SOLIDUS) {
              this._leaveAttrValue(SELF_CLOSING_START_TAG_STATE$1);
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._leaveAttrValue(DATA_STATE$1);
              this._emitCurrentToken();
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInTag);
              this._emitEOFToken();
          } else {
              this._err(ERR$4.missingWhitespaceBetweenAttributes);
              this._reconsumeInState(BEFORE_ATTRIBUTE_NAME_STATE$1);
          }
      }

      // Self-closing start tag state
      //------------------------------------------------------------------
      [SELF_CLOSING_START_TAG_STATE$1](cp) {
          if (cp === $$c.GREATER_THAN_SIGN) {
              this.currentToken.selfClosing = true;
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInTag);
              this._emitEOFToken();
          } else {
              this._err(ERR$4.unexpectedSolidusInTag);
              this._reconsumeInState(BEFORE_ATTRIBUTE_NAME_STATE$1);
          }
      }

      // Bogus comment state
      //------------------------------------------------------------------
      [BOGUS_COMMENT_STATE$1](cp) {
          if (cp === $$c.GREATER_THAN_SIGN) {
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else if (cp === $$c.EOF) {
              this._emitCurrentToken();
              this._emitEOFToken();
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.currentToken.data += unicode$5.REPLACEMENT_CHARACTER;
          } else {
              this.currentToken.data += toChar$1(cp);
          }
      }

      // Markup declaration open state
      //------------------------------------------------------------------
      [MARKUP_DECLARATION_OPEN_STATE$1](cp) {
          if (this._consumeSequenceIfMatch($$$1.DASH_DASH_STRING, cp, true)) {
              this._createCommentToken();
              this.state = COMMENT_START_STATE$1;
          } else if (this._consumeSequenceIfMatch($$$1.DOCTYPE_STRING, cp, false)) {
              this.state = DOCTYPE_STATE$1;
          } else if (this._consumeSequenceIfMatch($$$1.CDATA_START_STRING, cp, true)) {
              if (this.allowCDATA) {
                  this.state = CDATA_SECTION_STATE$1;
              } else {
                  this._err(ERR$4.cdataInHtmlContent);
                  this._createCommentToken();
                  this.currentToken.data = '[CDATA[';
                  this.state = BOGUS_COMMENT_STATE$1;
              }
          }

          //NOTE: sequence lookup can be abrupted by hibernation. In that case lookup
          //results are no longer valid and we will need to start over.
          else if (!this._ensureHibernation()) {
              this._err(ERR$4.incorrectlyOpenedComment);
              this._createCommentToken();
              this._reconsumeInState(BOGUS_COMMENT_STATE$1);
          }
      }

      // Comment start state
      //------------------------------------------------------------------
      [COMMENT_START_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this.state = COMMENT_START_DASH_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._err(ERR$4.abruptClosingOfEmptyComment);
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else {
              this._reconsumeInState(COMMENT_STATE$1);
          }
      }

      // Comment start dash state
      //------------------------------------------------------------------
      [COMMENT_START_DASH_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this.state = COMMENT_END_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._err(ERR$4.abruptClosingOfEmptyComment);
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInComment);
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.data += '-';
              this._reconsumeInState(COMMENT_STATE$1);
          }
      }

      // Comment state
      //------------------------------------------------------------------
      [COMMENT_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this.state = COMMENT_END_DASH_STATE$1;
          } else if (cp === $$c.LESS_THAN_SIGN) {
              this.currentToken.data += '<';
              this.state = COMMENT_LESS_THAN_SIGN_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.currentToken.data += unicode$5.REPLACEMENT_CHARACTER;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInComment);
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.data += toChar$1(cp);
          }
      }

      // Comment less-than sign state
      //------------------------------------------------------------------
      [COMMENT_LESS_THAN_SIGN_STATE$1](cp) {
          if (cp === $$c.EXCLAMATION_MARK) {
              this.currentToken.data += '!';
              this.state = COMMENT_LESS_THAN_SIGN_BANG_STATE$1;
          } else if (cp === $$c.LESS_THAN_SIGN) {
              this.currentToken.data += '!';
          } else {
              this._reconsumeInState(COMMENT_STATE$1);
          }
      }

      // Comment less-than sign bang state
      //------------------------------------------------------------------
      [COMMENT_LESS_THAN_SIGN_BANG_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this.state = COMMENT_LESS_THAN_SIGN_BANG_DASH_STATE$1;
          } else {
              this._reconsumeInState(COMMENT_STATE$1);
          }
      }

      // Comment less-than sign bang dash state
      //------------------------------------------------------------------
      [COMMENT_LESS_THAN_SIGN_BANG_DASH_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this.state = COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH_STATE$1;
          } else {
              this._reconsumeInState(COMMENT_END_DASH_STATE$1);
          }
      }

      // Comment less-than sign bang dash dash state
      //------------------------------------------------------------------
      [COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH_STATE$1](cp) {
          if (cp !== $$c.GREATER_THAN_SIGN && cp !== $$c.EOF) {
              this._err(ERR$4.nestedComment);
          }

          this._reconsumeInState(COMMENT_END_STATE$1);
      }

      // Comment end dash state
      //------------------------------------------------------------------
      [COMMENT_END_DASH_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this.state = COMMENT_END_STATE$1;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInComment);
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.data += '-';
              this._reconsumeInState(COMMENT_STATE$1);
          }
      }

      // Comment end state
      //------------------------------------------------------------------
      [COMMENT_END_STATE$1](cp) {
          if (cp === $$c.GREATER_THAN_SIGN) {
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else if (cp === $$c.EXCLAMATION_MARK) {
              this.state = COMMENT_END_BANG_STATE$1;
          } else if (cp === $$c.HYPHEN_MINUS) {
              this.currentToken.data += '-';
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInComment);
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.data += '--';
              this._reconsumeInState(COMMENT_STATE$1);
          }
      }

      // Comment end bang state
      //------------------------------------------------------------------
      [COMMENT_END_BANG_STATE$1](cp) {
          if (cp === $$c.HYPHEN_MINUS) {
              this.currentToken.data += '--!';
              this.state = COMMENT_END_DASH_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._err(ERR$4.incorrectlyClosedComment);
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInComment);
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.data += '--!';
              this._reconsumeInState(COMMENT_STATE$1);
          }
      }

      // DOCTYPE state
      //------------------------------------------------------------------
      [DOCTYPE_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              this.state = BEFORE_DOCTYPE_NAME_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._reconsumeInState(BEFORE_DOCTYPE_NAME_STATE$1);
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this._createDoctypeToken(null);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$4.missingWhitespaceBeforeDoctypeName);
              this._reconsumeInState(BEFORE_DOCTYPE_NAME_STATE$1);
          }
      }

      // Before DOCTYPE name state
      //------------------------------------------------------------------
      [BEFORE_DOCTYPE_NAME_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              return;
          }

          if (isAsciiUpper$1(cp)) {
              this._createDoctypeToken(toAsciiLowerChar$1(cp));
              this.state = DOCTYPE_NAME_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this._createDoctypeToken(unicode$5.REPLACEMENT_CHARACTER);
              this.state = DOCTYPE_NAME_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._err(ERR$4.missingDoctypeName);
              this._createDoctypeToken(null);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this.state = DATA_STATE$1;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this._createDoctypeToken(null);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._createDoctypeToken(toChar$1(cp));
              this.state = DOCTYPE_NAME_STATE$1;
          }
      }

      // DOCTYPE name state
      //------------------------------------------------------------------
      [DOCTYPE_NAME_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              this.state = AFTER_DOCTYPE_NAME_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else if (isAsciiUpper$1(cp)) {
              this.currentToken.name += toAsciiLowerChar$1(cp);
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.currentToken.name += unicode$5.REPLACEMENT_CHARACTER;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.name += toChar$1(cp);
          }
      }

      // After DOCTYPE name state
      //------------------------------------------------------------------
      [AFTER_DOCTYPE_NAME_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              return;
          }

          if (cp === $$c.GREATER_THAN_SIGN) {
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else if (this._consumeSequenceIfMatch($$$1.PUBLIC_STRING, cp, false)) {
              this.state = AFTER_DOCTYPE_PUBLIC_KEYWORD_STATE$1;
          } else if (this._consumeSequenceIfMatch($$$1.SYSTEM_STRING, cp, false)) {
              this.state = AFTER_DOCTYPE_SYSTEM_KEYWORD_STATE$1;
          }
          //NOTE: sequence lookup can be abrupted by hibernation. In that case lookup
          //results are no longer valid and we will need to start over.
          else if (!this._ensureHibernation()) {
              this._err(ERR$4.invalidCharacterSequenceAfterDoctypeName);
              this.currentToken.forceQuirks = true;
              this._reconsumeInState(BOGUS_DOCTYPE_STATE$1);
          }
      }

      // After DOCTYPE public keyword state
      //------------------------------------------------------------------
      [AFTER_DOCTYPE_PUBLIC_KEYWORD_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              this.state = BEFORE_DOCTYPE_PUBLIC_IDENTIFIER_STATE$1;
          } else if (cp === $$c.QUOTATION_MARK) {
              this._err(ERR$4.missingWhitespaceAfterDoctypePublicKeyword);
              this.currentToken.publicId = '';
              this.state = DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED_STATE$1;
          } else if (cp === $$c.APOSTROPHE) {
              this._err(ERR$4.missingWhitespaceAfterDoctypePublicKeyword);
              this.currentToken.publicId = '';
              this.state = DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._err(ERR$4.missingDoctypePublicIdentifier);
              this.currentToken.forceQuirks = true;
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$4.missingQuoteBeforeDoctypePublicIdentifier);
              this.currentToken.forceQuirks = true;
              this._reconsumeInState(BOGUS_DOCTYPE_STATE$1);
          }
      }

      // Before DOCTYPE public identifier state
      //------------------------------------------------------------------
      [BEFORE_DOCTYPE_PUBLIC_IDENTIFIER_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              return;
          }

          if (cp === $$c.QUOTATION_MARK) {
              this.currentToken.publicId = '';
              this.state = DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED_STATE$1;
          } else if (cp === $$c.APOSTROPHE) {
              this.currentToken.publicId = '';
              this.state = DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._err(ERR$4.missingDoctypePublicIdentifier);
              this.currentToken.forceQuirks = true;
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$4.missingQuoteBeforeDoctypePublicIdentifier);
              this.currentToken.forceQuirks = true;
              this._reconsumeInState(BOGUS_DOCTYPE_STATE$1);
          }
      }

      // DOCTYPE public identifier (double-quoted) state
      //------------------------------------------------------------------
      [DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED_STATE$1](cp) {
          if (cp === $$c.QUOTATION_MARK) {
              this.state = AFTER_DOCTYPE_PUBLIC_IDENTIFIER_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.currentToken.publicId += unicode$5.REPLACEMENT_CHARACTER;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._err(ERR$4.abruptDoctypePublicIdentifier);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this.state = DATA_STATE$1;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.publicId += toChar$1(cp);
          }
      }

      // DOCTYPE public identifier (single-quoted) state
      //------------------------------------------------------------------
      [DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED_STATE$1](cp) {
          if (cp === $$c.APOSTROPHE) {
              this.state = AFTER_DOCTYPE_PUBLIC_IDENTIFIER_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.currentToken.publicId += unicode$5.REPLACEMENT_CHARACTER;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._err(ERR$4.abruptDoctypePublicIdentifier);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this.state = DATA_STATE$1;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.publicId += toChar$1(cp);
          }
      }

      // After DOCTYPE public identifier state
      //------------------------------------------------------------------
      [AFTER_DOCTYPE_PUBLIC_IDENTIFIER_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              this.state = BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else if (cp === $$c.QUOTATION_MARK) {
              this._err(ERR$4.missingWhitespaceBetweenDoctypePublicAndSystemIdentifiers);
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED_STATE$1;
          } else if (cp === $$c.APOSTROPHE) {
              this._err(ERR$4.missingWhitespaceBetweenDoctypePublicAndSystemIdentifiers);
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED_STATE$1;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$4.missingQuoteBeforeDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this._reconsumeInState(BOGUS_DOCTYPE_STATE$1);
          }
      }

      // Between DOCTYPE public and system identifiers state
      //------------------------------------------------------------------
      [BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              return;
          }

          if (cp === $$c.GREATER_THAN_SIGN) {
              this._emitCurrentToken();
              this.state = DATA_STATE$1;
          } else if (cp === $$c.QUOTATION_MARK) {
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED_STATE$1;
          } else if (cp === $$c.APOSTROPHE) {
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED_STATE$1;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$4.missingQuoteBeforeDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this._reconsumeInState(BOGUS_DOCTYPE_STATE$1);
          }
      }

      // After DOCTYPE system keyword state
      //------------------------------------------------------------------
      [AFTER_DOCTYPE_SYSTEM_KEYWORD_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              this.state = BEFORE_DOCTYPE_SYSTEM_IDENTIFIER_STATE$1;
          } else if (cp === $$c.QUOTATION_MARK) {
              this._err(ERR$4.missingWhitespaceAfterDoctypeSystemKeyword);
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED_STATE$1;
          } else if (cp === $$c.APOSTROPHE) {
              this._err(ERR$4.missingWhitespaceAfterDoctypeSystemKeyword);
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._err(ERR$4.missingDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$4.missingQuoteBeforeDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this._reconsumeInState(BOGUS_DOCTYPE_STATE$1);
          }
      }

      // Before DOCTYPE system identifier state
      //------------------------------------------------------------------
      [BEFORE_DOCTYPE_SYSTEM_IDENTIFIER_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              return;
          }

          if (cp === $$c.QUOTATION_MARK) {
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED_STATE$1;
          } else if (cp === $$c.APOSTROPHE) {
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED_STATE$1;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._err(ERR$4.missingDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this.state = DATA_STATE$1;
              this._emitCurrentToken();
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$4.missingQuoteBeforeDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this._reconsumeInState(BOGUS_DOCTYPE_STATE$1);
          }
      }

      // DOCTYPE system identifier (double-quoted) state
      //------------------------------------------------------------------
      [DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED_STATE$1](cp) {
          if (cp === $$c.QUOTATION_MARK) {
              this.state = AFTER_DOCTYPE_SYSTEM_IDENTIFIER_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.currentToken.systemId += unicode$5.REPLACEMENT_CHARACTER;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._err(ERR$4.abruptDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this.state = DATA_STATE$1;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.systemId += toChar$1(cp);
          }
      }

      // DOCTYPE system identifier (single-quoted) state
      //------------------------------------------------------------------
      [DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED_STATE$1](cp) {
          if (cp === $$c.APOSTROPHE) {
              this.state = AFTER_DOCTYPE_SYSTEM_IDENTIFIER_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
              this.currentToken.systemId += unicode$5.REPLACEMENT_CHARACTER;
          } else if (cp === $$c.GREATER_THAN_SIGN) {
              this._err(ERR$4.abruptDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this.state = DATA_STATE$1;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.systemId += toChar$1(cp);
          }
      }

      // After DOCTYPE system identifier state
      //------------------------------------------------------------------
      [AFTER_DOCTYPE_SYSTEM_IDENTIFIER_STATE$1](cp) {
          if (isWhitespace$1(cp)) {
              return;
          }

          if (cp === $$c.GREATER_THAN_SIGN) {
              this._emitCurrentToken();
              this.state = DATA_STATE$1;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$4.unexpectedCharacterAfterDoctypeSystemIdentifier);
              this._reconsumeInState(BOGUS_DOCTYPE_STATE$1);
          }
      }

      // Bogus DOCTYPE state
      //------------------------------------------------------------------
      [BOGUS_DOCTYPE_STATE$1](cp) {
          if (cp === $$c.GREATER_THAN_SIGN) {
              this._emitCurrentToken();
              this.state = DATA_STATE$1;
          } else if (cp === $$c.NULL) {
              this._err(ERR$4.unexpectedNullCharacter);
          } else if (cp === $$c.EOF) {
              this._emitCurrentToken();
              this._emitEOFToken();
          }
      }

      // CDATA section state
      //------------------------------------------------------------------
      [CDATA_SECTION_STATE$1](cp) {
          if (cp === $$c.RIGHT_SQUARE_BRACKET) {
              this.state = CDATA_SECTION_BRACKET_STATE$1;
          } else if (cp === $$c.EOF) {
              this._err(ERR$4.eofInCdata);
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      // CDATA section bracket state
      //------------------------------------------------------------------
      [CDATA_SECTION_BRACKET_STATE$1](cp) {
          if (cp === $$c.RIGHT_SQUARE_BRACKET) {
              this.state = CDATA_SECTION_END_STATE$1;
          } else {
              this._emitChars(']');
              this._reconsumeInState(CDATA_SECTION_STATE$1);
          }
      }

      // CDATA section end state
      //------------------------------------------------------------------
      [CDATA_SECTION_END_STATE$1](cp) {
          if (cp === $$c.GREATER_THAN_SIGN) {
              this.state = DATA_STATE$1;
          } else if (cp === $$c.RIGHT_SQUARE_BRACKET) {
              this._emitChars(']');
          } else {
              this._emitChars(']]');
              this._reconsumeInState(CDATA_SECTION_STATE$1);
          }
      }

      // Character reference state
      //------------------------------------------------------------------
      [CHARACTER_REFERENCE_STATE$1](cp) {
          this.tempBuff = [$$c.AMPERSAND];

          if (cp === $$c.NUMBER_SIGN) {
              this.tempBuff.push(cp);
              this.state = NUMERIC_CHARACTER_REFERENCE_STATE$1;
          } else if (isAsciiAlphaNumeric$1(cp)) {
              this._reconsumeInState(NAMED_CHARACTER_REFERENCE_STATE$1);
          } else {
              this._flushCodePointsConsumedAsCharacterReference();
              this._reconsumeInState(this.returnState);
          }
      }

      // Named character reference state
      //------------------------------------------------------------------
      [NAMED_CHARACTER_REFERENCE_STATE$1](cp) {
          const matchResult = this._matchNamedCharacterReference(cp);

          //NOTE: matching can be abrupted by hibernation. In that case match
          //results are no longer valid and we will need to start over.
          if (this._ensureHibernation()) {
              this.tempBuff = [$$c.AMPERSAND];
          } else if (matchResult) {
              const withSemicolon = this.tempBuff[this.tempBuff.length - 1] === $$c.SEMICOLON;

              if (!this._isCharacterReferenceAttributeQuirk(withSemicolon)) {
                  if (!withSemicolon) {
                      this._errOnNextCodePoint(ERR$4.missingSemicolonAfterCharacterReference);
                  }

                  this.tempBuff = matchResult;
              }

              this._flushCodePointsConsumedAsCharacterReference();
              this.state = this.returnState;
          } else {
              this._flushCodePointsConsumedAsCharacterReference();
              this.state = AMBIGUOUS_AMPERSAND_STATE$1;
          }
      }

      // Ambiguos ampersand state
      //------------------------------------------------------------------
      [AMBIGUOUS_AMPERSAND_STATE$1](cp) {
          if (isAsciiAlphaNumeric$1(cp)) {
              if (this._isCharacterReferenceInAttribute()) {
                  this.currentAttr.value += toChar$1(cp);
              } else {
                  this._emitCodePoint(cp);
              }
          } else {
              if (cp === $$c.SEMICOLON) {
                  this._err(ERR$4.unknownNamedCharacterReference);
              }

              this._reconsumeInState(this.returnState);
          }
      }

      // Numeric character reference state
      //------------------------------------------------------------------
      [NUMERIC_CHARACTER_REFERENCE_STATE$1](cp) {
          this.charRefCode = 0;

          if (cp === $$c.LATIN_SMALL_X || cp === $$c.LATIN_CAPITAL_X) {
              this.tempBuff.push(cp);
              this.state = HEXADEMICAL_CHARACTER_REFERENCE_START_STATE$1;
          } else {
              this._reconsumeInState(DECIMAL_CHARACTER_REFERENCE_START_STATE$1);
          }
      }

      // Hexademical character reference start state
      //------------------------------------------------------------------
      [HEXADEMICAL_CHARACTER_REFERENCE_START_STATE$1](cp) {
          if (isAsciiHexDigit$1(cp)) {
              this._reconsumeInState(HEXADEMICAL_CHARACTER_REFERENCE_STATE$1);
          } else {
              this._err(ERR$4.absenceOfDigitsInNumericCharacterReference);
              this._flushCodePointsConsumedAsCharacterReference();
              this._reconsumeInState(this.returnState);
          }
      }

      // Decimal character reference start state
      //------------------------------------------------------------------
      [DECIMAL_CHARACTER_REFERENCE_START_STATE$1](cp) {
          if (isAsciiDigit$1(cp)) {
              this._reconsumeInState(DECIMAL_CHARACTER_REFERENCE_STATE$1);
          } else {
              this._err(ERR$4.absenceOfDigitsInNumericCharacterReference);
              this._flushCodePointsConsumedAsCharacterReference();
              this._reconsumeInState(this.returnState);
          }
      }

      // Hexademical character reference state
      //------------------------------------------------------------------
      [HEXADEMICAL_CHARACTER_REFERENCE_STATE$1](cp) {
          if (isAsciiUpperHexDigit$1(cp)) {
              this.charRefCode = this.charRefCode * 16 + cp - 0x37;
          } else if (isAsciiLowerHexDigit$1(cp)) {
              this.charRefCode = this.charRefCode * 16 + cp - 0x57;
          } else if (isAsciiDigit$1(cp)) {
              this.charRefCode = this.charRefCode * 16 + cp - 0x30;
          } else if (cp === $$c.SEMICOLON) {
              this.state = NUMERIC_CHARACTER_REFERENCE_END_STATE$1;
          } else {
              this._err(ERR$4.missingSemicolonAfterCharacterReference);
              this._reconsumeInState(NUMERIC_CHARACTER_REFERENCE_END_STATE$1);
          }
      }

      // Decimal character reference state
      //------------------------------------------------------------------
      [DECIMAL_CHARACTER_REFERENCE_STATE$1](cp) {
          if (isAsciiDigit$1(cp)) {
              this.charRefCode = this.charRefCode * 10 + cp - 0x30;
          } else if (cp === $$c.SEMICOLON) {
              this.state = NUMERIC_CHARACTER_REFERENCE_END_STATE$1;
          } else {
              this._err(ERR$4.missingSemicolonAfterCharacterReference);
              this._reconsumeInState(NUMERIC_CHARACTER_REFERENCE_END_STATE$1);
          }
      }

      // Numeric character reference end state
      //------------------------------------------------------------------
      [NUMERIC_CHARACTER_REFERENCE_END_STATE$1]() {
          if (this.charRefCode === $$c.NULL) {
              this._err(ERR$4.nullCharacterReference);
              this.charRefCode = $$c.REPLACEMENT_CHARACTER;
          } else if (this.charRefCode > 0x10ffff) {
              this._err(ERR$4.characterReferenceOutsideUnicodeRange);
              this.charRefCode = $$c.REPLACEMENT_CHARACTER;
          } else if (unicode$5.isSurrogate(this.charRefCode)) {
              this._err(ERR$4.surrogateCharacterReference);
              this.charRefCode = $$c.REPLACEMENT_CHARACTER;
          } else if (unicode$5.isUndefinedCodePoint(this.charRefCode)) {
              this._err(ERR$4.noncharacterCharacterReference);
          } else if (unicode$5.isControlCodePoint(this.charRefCode) || this.charRefCode === $$c.CARRIAGE_RETURN) {
              this._err(ERR$4.controlCharacterReference);

              const replacement = C1_CONTROLS_REFERENCE_REPLACEMENTS$1[this.charRefCode];

              if (replacement) {
                  this.charRefCode = replacement;
              }
          }

          this.tempBuff = [this.charRefCode];

          this._flushCodePointsConsumedAsCharacterReference();
          this._reconsumeInState(this.returnState);
      }
  }

  //Token types
  Tokenizer$9.CHARACTER_TOKEN = 'CHARACTER_TOKEN';
  Tokenizer$9.NULL_CHARACTER_TOKEN = 'NULL_CHARACTER_TOKEN';
  Tokenizer$9.WHITESPACE_CHARACTER_TOKEN = 'WHITESPACE_CHARACTER_TOKEN';
  Tokenizer$9.START_TAG_TOKEN = 'START_TAG_TOKEN';
  Tokenizer$9.END_TAG_TOKEN = 'END_TAG_TOKEN';
  Tokenizer$9.COMMENT_TOKEN = 'COMMENT_TOKEN';
  Tokenizer$9.DOCTYPE_TOKEN = 'DOCTYPE_TOKEN';
  Tokenizer$9.EOF_TOKEN = 'EOF_TOKEN';
  Tokenizer$9.HIBERNATION_TOKEN = 'HIBERNATION_TOKEN';

  //Tokenizer initial states for different modes
  Tokenizer$9.MODE = {
      DATA: DATA_STATE$1,
      RCDATA: RCDATA_STATE$1,
      RAWTEXT: RAWTEXT_STATE$1,
      SCRIPT_DATA: SCRIPT_DATA_STATE$1,
      PLAINTEXT: PLAINTEXT_STATE$1
  };

  //Static
  Tokenizer$9.getTokenAttr = function(token, attrName) {
      for (let i = token.attrs.length - 1; i >= 0; i--) {
          if (token.attrs[i].name === attrName) {
              return token.attrs[i].value;
          }
      }

      return null;
  };

  var tokenizer$1 = Tokenizer$9;

  var html$h = {};

  const NS$7 = (html$h.NAMESPACES = {
      HTML: 'http://www.w3.org/1999/xhtml',
      MATHML: 'http://www.w3.org/1998/Math/MathML',
      SVG: 'http://www.w3.org/2000/svg',
      XLINK: 'http://www.w3.org/1999/xlink',
      XML: 'http://www.w3.org/XML/1998/namespace',
      XMLNS: 'http://www.w3.org/2000/xmlns/'
  });

  html$h.ATTRS = {
      TYPE: 'type',
      ACTION: 'action',
      ENCODING: 'encoding',
      PROMPT: 'prompt',
      NAME: 'name',
      COLOR: 'color',
      FACE: 'face',
      SIZE: 'size'
  };

  html$h.DOCUMENT_MODE = {
      NO_QUIRKS: 'no-quirks',
      QUIRKS: 'quirks',
      LIMITED_QUIRKS: 'limited-quirks'
  };

  const $$b = (html$h.TAG_NAMES = {
      A: 'a',
      ADDRESS: 'address',
      ANNOTATION_XML: 'annotation-xml',
      APPLET: 'applet',
      AREA: 'area',
      ARTICLE: 'article',
      ASIDE: 'aside',

      B: 'b',
      BASE: 'base',
      BASEFONT: 'basefont',
      BGSOUND: 'bgsound',
      BIG: 'big',
      BLOCKQUOTE: 'blockquote',
      BODY: 'body',
      BR: 'br',
      BUTTON: 'button',

      CAPTION: 'caption',
      CENTER: 'center',
      CODE: 'code',
      COL: 'col',
      COLGROUP: 'colgroup',

      DD: 'dd',
      DESC: 'desc',
      DETAILS: 'details',
      DIALOG: 'dialog',
      DIR: 'dir',
      DIV: 'div',
      DL: 'dl',
      DT: 'dt',

      EM: 'em',
      EMBED: 'embed',

      FIELDSET: 'fieldset',
      FIGCAPTION: 'figcaption',
      FIGURE: 'figure',
      FONT: 'font',
      FOOTER: 'footer',
      FOREIGN_OBJECT: 'foreignObject',
      FORM: 'form',
      FRAME: 'frame',
      FRAMESET: 'frameset',

      H1: 'h1',
      H2: 'h2',
      H3: 'h3',
      H4: 'h4',
      H5: 'h5',
      H6: 'h6',
      HEAD: 'head',
      HEADER: 'header',
      HGROUP: 'hgroup',
      HR: 'hr',
      HTML: 'html',

      I: 'i',
      IMG: 'img',
      IMAGE: 'image',
      INPUT: 'input',
      IFRAME: 'iframe',

      KEYGEN: 'keygen',

      LABEL: 'label',
      LI: 'li',
      LINK: 'link',
      LISTING: 'listing',

      MAIN: 'main',
      MALIGNMARK: 'malignmark',
      MARQUEE: 'marquee',
      MATH: 'math',
      MENU: 'menu',
      META: 'meta',
      MGLYPH: 'mglyph',
      MI: 'mi',
      MO: 'mo',
      MN: 'mn',
      MS: 'ms',
      MTEXT: 'mtext',

      NAV: 'nav',
      NOBR: 'nobr',
      NOFRAMES: 'noframes',
      NOEMBED: 'noembed',
      NOSCRIPT: 'noscript',

      OBJECT: 'object',
      OL: 'ol',
      OPTGROUP: 'optgroup',
      OPTION: 'option',

      P: 'p',
      PARAM: 'param',
      PLAINTEXT: 'plaintext',
      PRE: 'pre',

      RB: 'rb',
      RP: 'rp',
      RT: 'rt',
      RTC: 'rtc',
      RUBY: 'ruby',

      S: 's',
      SCRIPT: 'script',
      SECTION: 'section',
      SELECT: 'select',
      SOURCE: 'source',
      SMALL: 'small',
      SPAN: 'span',
      STRIKE: 'strike',
      STRONG: 'strong',
      STYLE: 'style',
      SUB: 'sub',
      SUMMARY: 'summary',
      SUP: 'sup',

      TABLE: 'table',
      TBODY: 'tbody',
      TEMPLATE: 'template',
      TEXTAREA: 'textarea',
      TFOOT: 'tfoot',
      TD: 'td',
      TH: 'th',
      THEAD: 'thead',
      TITLE: 'title',
      TR: 'tr',
      TRACK: 'track',
      TT: 'tt',

      U: 'u',
      UL: 'ul',

      SVG: 'svg',

      VAR: 'var',

      WBR: 'wbr',

      XMP: 'xmp'
  });

  html$h.SPECIAL_ELEMENTS = {
      [NS$7.HTML]: {
          [$$b.ADDRESS]: true,
          [$$b.APPLET]: true,
          [$$b.AREA]: true,
          [$$b.ARTICLE]: true,
          [$$b.ASIDE]: true,
          [$$b.BASE]: true,
          [$$b.BASEFONT]: true,
          [$$b.BGSOUND]: true,
          [$$b.BLOCKQUOTE]: true,
          [$$b.BODY]: true,
          [$$b.BR]: true,
          [$$b.BUTTON]: true,
          [$$b.CAPTION]: true,
          [$$b.CENTER]: true,
          [$$b.COL]: true,
          [$$b.COLGROUP]: true,
          [$$b.DD]: true,
          [$$b.DETAILS]: true,
          [$$b.DIR]: true,
          [$$b.DIV]: true,
          [$$b.DL]: true,
          [$$b.DT]: true,
          [$$b.EMBED]: true,
          [$$b.FIELDSET]: true,
          [$$b.FIGCAPTION]: true,
          [$$b.FIGURE]: true,
          [$$b.FOOTER]: true,
          [$$b.FORM]: true,
          [$$b.FRAME]: true,
          [$$b.FRAMESET]: true,
          [$$b.H1]: true,
          [$$b.H2]: true,
          [$$b.H3]: true,
          [$$b.H4]: true,
          [$$b.H5]: true,
          [$$b.H6]: true,
          [$$b.HEAD]: true,
          [$$b.HEADER]: true,
          [$$b.HGROUP]: true,
          [$$b.HR]: true,
          [$$b.HTML]: true,
          [$$b.IFRAME]: true,
          [$$b.IMG]: true,
          [$$b.INPUT]: true,
          [$$b.LI]: true,
          [$$b.LINK]: true,
          [$$b.LISTING]: true,
          [$$b.MAIN]: true,
          [$$b.MARQUEE]: true,
          [$$b.MENU]: true,
          [$$b.META]: true,
          [$$b.NAV]: true,
          [$$b.NOEMBED]: true,
          [$$b.NOFRAMES]: true,
          [$$b.NOSCRIPT]: true,
          [$$b.OBJECT]: true,
          [$$b.OL]: true,
          [$$b.P]: true,
          [$$b.PARAM]: true,
          [$$b.PLAINTEXT]: true,
          [$$b.PRE]: true,
          [$$b.SCRIPT]: true,
          [$$b.SECTION]: true,
          [$$b.SELECT]: true,
          [$$b.SOURCE]: true,
          [$$b.STYLE]: true,
          [$$b.SUMMARY]: true,
          [$$b.TABLE]: true,
          [$$b.TBODY]: true,
          [$$b.TD]: true,
          [$$b.TEMPLATE]: true,
          [$$b.TEXTAREA]: true,
          [$$b.TFOOT]: true,
          [$$b.TH]: true,
          [$$b.THEAD]: true,
          [$$b.TITLE]: true,
          [$$b.TR]: true,
          [$$b.TRACK]: true,
          [$$b.UL]: true,
          [$$b.WBR]: true,
          [$$b.XMP]: true
      },
      [NS$7.MATHML]: {
          [$$b.MI]: true,
          [$$b.MO]: true,
          [$$b.MN]: true,
          [$$b.MS]: true,
          [$$b.MTEXT]: true,
          [$$b.ANNOTATION_XML]: true
      },
      [NS$7.SVG]: {
          [$$b.TITLE]: true,
          [$$b.FOREIGN_OBJECT]: true,
          [$$b.DESC]: true
      }
  };

  const HTML$7 = html$h;

  //Aliases
  const $$a = HTML$7.TAG_NAMES;
  const NS$6 = HTML$7.NAMESPACES;

  //Element utils

  //OPTIMIZATION: Integer comparisons are low-cost, so we can use very fast tag name length filters here.
  //It's faster than using dictionary.
  function isImpliedEndTagRequired$1(tn) {
      switch (tn.length) {
          case 1:
              return tn === $$a.P;

          case 2:
              return tn === $$a.RB || tn === $$a.RP || tn === $$a.RT || tn === $$a.DD || tn === $$a.DT || tn === $$a.LI;

          case 3:
              return tn === $$a.RTC;

          case 6:
              return tn === $$a.OPTION;

          case 8:
              return tn === $$a.OPTGROUP;
      }

      return false;
  }

  function isImpliedEndTagRequiredThoroughly$1(tn) {
      switch (tn.length) {
          case 1:
              return tn === $$a.P;

          case 2:
              return (
                  tn === $$a.RB ||
                  tn === $$a.RP ||
                  tn === $$a.RT ||
                  tn === $$a.DD ||
                  tn === $$a.DT ||
                  tn === $$a.LI ||
                  tn === $$a.TD ||
                  tn === $$a.TH ||
                  tn === $$a.TR
              );

          case 3:
              return tn === $$a.RTC;

          case 5:
              return tn === $$a.TBODY || tn === $$a.TFOOT || tn === $$a.THEAD;

          case 6:
              return tn === $$a.OPTION;

          case 7:
              return tn === $$a.CAPTION;

          case 8:
              return tn === $$a.OPTGROUP || tn === $$a.COLGROUP;
      }

      return false;
  }

  function isScopingElement$1(tn, ns) {
      switch (tn.length) {
          case 2:
              if (tn === $$a.TD || tn === $$a.TH) {
                  return ns === NS$6.HTML;
              } else if (tn === $$a.MI || tn === $$a.MO || tn === $$a.MN || tn === $$a.MS) {
                  return ns === NS$6.MATHML;
              }

              break;

          case 4:
              if (tn === $$a.HTML) {
                  return ns === NS$6.HTML;
              } else if (tn === $$a.DESC) {
                  return ns === NS$6.SVG;
              }

              break;

          case 5:
              if (tn === $$a.TABLE) {
                  return ns === NS$6.HTML;
              } else if (tn === $$a.MTEXT) {
                  return ns === NS$6.MATHML;
              } else if (tn === $$a.TITLE) {
                  return ns === NS$6.SVG;
              }

              break;

          case 6:
              return (tn === $$a.APPLET || tn === $$a.OBJECT) && ns === NS$6.HTML;

          case 7:
              return (tn === $$a.CAPTION || tn === $$a.MARQUEE) && ns === NS$6.HTML;

          case 8:
              return tn === $$a.TEMPLATE && ns === NS$6.HTML;

          case 13:
              return tn === $$a.FOREIGN_OBJECT && ns === NS$6.SVG;

          case 14:
              return tn === $$a.ANNOTATION_XML && ns === NS$6.MATHML;
      }

      return false;
  }

  //Stack of open elements
  class OpenElementStack$3 {
      constructor(document, treeAdapter) {
          this.stackTop = -1;
          this.items = [];
          this.current = document;
          this.currentTagName = null;
          this.currentTmplContent = null;
          this.tmplCount = 0;
          this.treeAdapter = treeAdapter;
      }

      //Index of element
      _indexOf(element) {
          let idx = -1;

          for (let i = this.stackTop; i >= 0; i--) {
              if (this.items[i] === element) {
                  idx = i;
                  break;
              }
          }
          return idx;
      }

      //Update current element
      _isInTemplate() {
          return this.currentTagName === $$a.TEMPLATE && this.treeAdapter.getNamespaceURI(this.current) === NS$6.HTML;
      }

      _updateCurrentElement() {
          this.current = this.items[this.stackTop];
          this.currentTagName = this.current && this.treeAdapter.getTagName(this.current);

          this.currentTmplContent = this._isInTemplate() ? this.treeAdapter.getTemplateContent(this.current) : null;
      }

      //Mutations
      push(element) {
          this.items[++this.stackTop] = element;
          this._updateCurrentElement();

          if (this._isInTemplate()) {
              this.tmplCount++;
          }
      }

      pop() {
          this.stackTop--;

          if (this.tmplCount > 0 && this._isInTemplate()) {
              this.tmplCount--;
          }

          this._updateCurrentElement();
      }

      replace(oldElement, newElement) {
          const idx = this._indexOf(oldElement);

          this.items[idx] = newElement;

          if (idx === this.stackTop) {
              this._updateCurrentElement();
          }
      }

      insertAfter(referenceElement, newElement) {
          const insertionIdx = this._indexOf(referenceElement) + 1;

          this.items.splice(insertionIdx, 0, newElement);

          if (insertionIdx === ++this.stackTop) {
              this._updateCurrentElement();
          }
      }

      popUntilTagNamePopped(tagName) {
          while (this.stackTop > -1) {
              const tn = this.currentTagName;
              const ns = this.treeAdapter.getNamespaceURI(this.current);

              this.pop();

              if (tn === tagName && ns === NS$6.HTML) {
                  break;
              }
          }
      }

      popUntilElementPopped(element) {
          while (this.stackTop > -1) {
              const poppedElement = this.current;

              this.pop();

              if (poppedElement === element) {
                  break;
              }
          }
      }

      popUntilNumberedHeaderPopped() {
          while (this.stackTop > -1) {
              const tn = this.currentTagName;
              const ns = this.treeAdapter.getNamespaceURI(this.current);

              this.pop();

              if (
                  tn === $$a.H1 ||
                  tn === $$a.H2 ||
                  tn === $$a.H3 ||
                  tn === $$a.H4 ||
                  tn === $$a.H5 ||
                  (tn === $$a.H6 && ns === NS$6.HTML)
              ) {
                  break;
              }
          }
      }

      popUntilTableCellPopped() {
          while (this.stackTop > -1) {
              const tn = this.currentTagName;
              const ns = this.treeAdapter.getNamespaceURI(this.current);

              this.pop();

              if (tn === $$a.TD || (tn === $$a.TH && ns === NS$6.HTML)) {
                  break;
              }
          }
      }

      popAllUpToHtmlElement() {
          //NOTE: here we assume that root <html> element is always first in the open element stack, so
          //we perform this fast stack clean up.
          this.stackTop = 0;
          this._updateCurrentElement();
      }

      clearBackToTableContext() {
          while (
              (this.currentTagName !== $$a.TABLE && this.currentTagName !== $$a.TEMPLATE && this.currentTagName !== $$a.HTML) ||
              this.treeAdapter.getNamespaceURI(this.current) !== NS$6.HTML
          ) {
              this.pop();
          }
      }

      clearBackToTableBodyContext() {
          while (
              (this.currentTagName !== $$a.TBODY &&
                  this.currentTagName !== $$a.TFOOT &&
                  this.currentTagName !== $$a.THEAD &&
                  this.currentTagName !== $$a.TEMPLATE &&
                  this.currentTagName !== $$a.HTML) ||
              this.treeAdapter.getNamespaceURI(this.current) !== NS$6.HTML
          ) {
              this.pop();
          }
      }

      clearBackToTableRowContext() {
          while (
              (this.currentTagName !== $$a.TR && this.currentTagName !== $$a.TEMPLATE && this.currentTagName !== $$a.HTML) ||
              this.treeAdapter.getNamespaceURI(this.current) !== NS$6.HTML
          ) {
              this.pop();
          }
      }

      remove(element) {
          for (let i = this.stackTop; i >= 0; i--) {
              if (this.items[i] === element) {
                  this.items.splice(i, 1);
                  this.stackTop--;
                  this._updateCurrentElement();
                  break;
              }
          }
      }

      //Search
      tryPeekProperlyNestedBodyElement() {
          //Properly nested <body> element (should be second element in stack).
          const element = this.items[1];

          return element && this.treeAdapter.getTagName(element) === $$a.BODY ? element : null;
      }

      contains(element) {
          return this._indexOf(element) > -1;
      }

      getCommonAncestor(element) {
          let elementIdx = this._indexOf(element);

          return --elementIdx >= 0 ? this.items[elementIdx] : null;
      }

      isRootHtmlElementCurrent() {
          return this.stackTop === 0 && this.currentTagName === $$a.HTML;
      }

      //Element in scope
      hasInScope(tagName) {
          for (let i = this.stackTop; i >= 0; i--) {
              const tn = this.treeAdapter.getTagName(this.items[i]);
              const ns = this.treeAdapter.getNamespaceURI(this.items[i]);

              if (tn === tagName && ns === NS$6.HTML) {
                  return true;
              }

              if (isScopingElement$1(tn, ns)) {
                  return false;
              }
          }

          return true;
      }

      hasNumberedHeaderInScope() {
          for (let i = this.stackTop; i >= 0; i--) {
              const tn = this.treeAdapter.getTagName(this.items[i]);
              const ns = this.treeAdapter.getNamespaceURI(this.items[i]);

              if (
                  (tn === $$a.H1 || tn === $$a.H2 || tn === $$a.H3 || tn === $$a.H4 || tn === $$a.H5 || tn === $$a.H6) &&
                  ns === NS$6.HTML
              ) {
                  return true;
              }

              if (isScopingElement$1(tn, ns)) {
                  return false;
              }
          }

          return true;
      }

      hasInListItemScope(tagName) {
          for (let i = this.stackTop; i >= 0; i--) {
              const tn = this.treeAdapter.getTagName(this.items[i]);
              const ns = this.treeAdapter.getNamespaceURI(this.items[i]);

              if (tn === tagName && ns === NS$6.HTML) {
                  return true;
              }

              if (((tn === $$a.UL || tn === $$a.OL) && ns === NS$6.HTML) || isScopingElement$1(tn, ns)) {
                  return false;
              }
          }

          return true;
      }

      hasInButtonScope(tagName) {
          for (let i = this.stackTop; i >= 0; i--) {
              const tn = this.treeAdapter.getTagName(this.items[i]);
              const ns = this.treeAdapter.getNamespaceURI(this.items[i]);

              if (tn === tagName && ns === NS$6.HTML) {
                  return true;
              }

              if ((tn === $$a.BUTTON && ns === NS$6.HTML) || isScopingElement$1(tn, ns)) {
                  return false;
              }
          }

          return true;
      }

      hasInTableScope(tagName) {
          for (let i = this.stackTop; i >= 0; i--) {
              const tn = this.treeAdapter.getTagName(this.items[i]);
              const ns = this.treeAdapter.getNamespaceURI(this.items[i]);

              if (ns !== NS$6.HTML) {
                  continue;
              }

              if (tn === tagName) {
                  return true;
              }

              if (tn === $$a.TABLE || tn === $$a.TEMPLATE || tn === $$a.HTML) {
                  return false;
              }
          }

          return true;
      }

      hasTableBodyContextInTableScope() {
          for (let i = this.stackTop; i >= 0; i--) {
              const tn = this.treeAdapter.getTagName(this.items[i]);
              const ns = this.treeAdapter.getNamespaceURI(this.items[i]);

              if (ns !== NS$6.HTML) {
                  continue;
              }

              if (tn === $$a.TBODY || tn === $$a.THEAD || tn === $$a.TFOOT) {
                  return true;
              }

              if (tn === $$a.TABLE || tn === $$a.HTML) {
                  return false;
              }
          }

          return true;
      }

      hasInSelectScope(tagName) {
          for (let i = this.stackTop; i >= 0; i--) {
              const tn = this.treeAdapter.getTagName(this.items[i]);
              const ns = this.treeAdapter.getNamespaceURI(this.items[i]);

              if (ns !== NS$6.HTML) {
                  continue;
              }

              if (tn === tagName) {
                  return true;
              }

              if (tn !== $$a.OPTION && tn !== $$a.OPTGROUP) {
                  return false;
              }
          }

          return true;
      }

      //Implied end tags
      generateImpliedEndTags() {
          while (isImpliedEndTagRequired$1(this.currentTagName)) {
              this.pop();
          }
      }

      generateImpliedEndTagsThoroughly() {
          while (isImpliedEndTagRequiredThoroughly$1(this.currentTagName)) {
              this.pop();
          }
      }

      generateImpliedEndTagsWithExclusion(exclusionTagName) {
          while (isImpliedEndTagRequired$1(this.currentTagName) && this.currentTagName !== exclusionTagName) {
              this.pop();
          }
      }
  }

  var openElementStack$1 = OpenElementStack$3;

  //Const
  const NOAH_ARK_CAPACITY$1 = 3;

  //List of formatting elements
  class FormattingElementList$3 {
      constructor(treeAdapter) {
          this.length = 0;
          this.entries = [];
          this.treeAdapter = treeAdapter;
          this.bookmark = null;
      }

      //Noah Ark's condition
      //OPTIMIZATION: at first we try to find possible candidates for exclusion using
      //lightweight heuristics without thorough attributes check.
      _getNoahArkConditionCandidates(newElement) {
          const candidates = [];

          if (this.length >= NOAH_ARK_CAPACITY$1) {
              const neAttrsLength = this.treeAdapter.getAttrList(newElement).length;
              const neTagName = this.treeAdapter.getTagName(newElement);
              const neNamespaceURI = this.treeAdapter.getNamespaceURI(newElement);

              for (let i = this.length - 1; i >= 0; i--) {
                  const entry = this.entries[i];

                  if (entry.type === FormattingElementList$3.MARKER_ENTRY) {
                      break;
                  }

                  const element = entry.element;
                  const elementAttrs = this.treeAdapter.getAttrList(element);

                  const isCandidate =
                      this.treeAdapter.getTagName(element) === neTagName &&
                      this.treeAdapter.getNamespaceURI(element) === neNamespaceURI &&
                      elementAttrs.length === neAttrsLength;

                  if (isCandidate) {
                      candidates.push({ idx: i, attrs: elementAttrs });
                  }
              }
          }

          return candidates.length < NOAH_ARK_CAPACITY$1 ? [] : candidates;
      }

      _ensureNoahArkCondition(newElement) {
          const candidates = this._getNoahArkConditionCandidates(newElement);
          let cLength = candidates.length;

          if (cLength) {
              const neAttrs = this.treeAdapter.getAttrList(newElement);
              const neAttrsLength = neAttrs.length;
              const neAttrsMap = Object.create(null);

              //NOTE: build attrs map for the new element so we can perform fast lookups
              for (let i = 0; i < neAttrsLength; i++) {
                  const neAttr = neAttrs[i];

                  neAttrsMap[neAttr.name] = neAttr.value;
              }

              for (let i = 0; i < neAttrsLength; i++) {
                  for (let j = 0; j < cLength; j++) {
                      const cAttr = candidates[j].attrs[i];

                      if (neAttrsMap[cAttr.name] !== cAttr.value) {
                          candidates.splice(j, 1);
                          cLength--;
                      }

                      if (candidates.length < NOAH_ARK_CAPACITY$1) {
                          return;
                      }
                  }
              }

              //NOTE: remove bottommost candidates until Noah's Ark condition will not be met
              for (let i = cLength - 1; i >= NOAH_ARK_CAPACITY$1 - 1; i--) {
                  this.entries.splice(candidates[i].idx, 1);
                  this.length--;
              }
          }
      }

      //Mutations
      insertMarker() {
          this.entries.push({ type: FormattingElementList$3.MARKER_ENTRY });
          this.length++;
      }

      pushElement(element, token) {
          this._ensureNoahArkCondition(element);

          this.entries.push({
              type: FormattingElementList$3.ELEMENT_ENTRY,
              element: element,
              token: token
          });

          this.length++;
      }

      insertElementAfterBookmark(element, token) {
          let bookmarkIdx = this.length - 1;

          for (; bookmarkIdx >= 0; bookmarkIdx--) {
              if (this.entries[bookmarkIdx] === this.bookmark) {
                  break;
              }
          }

          this.entries.splice(bookmarkIdx + 1, 0, {
              type: FormattingElementList$3.ELEMENT_ENTRY,
              element: element,
              token: token
          });

          this.length++;
      }

      removeEntry(entry) {
          for (let i = this.length - 1; i >= 0; i--) {
              if (this.entries[i] === entry) {
                  this.entries.splice(i, 1);
                  this.length--;
                  break;
              }
          }
      }

      clearToLastMarker() {
          while (this.length) {
              const entry = this.entries.pop();

              this.length--;

              if (entry.type === FormattingElementList$3.MARKER_ENTRY) {
                  break;
              }
          }
      }

      //Search
      getElementEntryInScopeWithTagName(tagName) {
          for (let i = this.length - 1; i >= 0; i--) {
              const entry = this.entries[i];

              if (entry.type === FormattingElementList$3.MARKER_ENTRY) {
                  return null;
              }

              if (this.treeAdapter.getTagName(entry.element) === tagName) {
                  return entry;
              }
          }

          return null;
      }

      getElementEntry(element) {
          for (let i = this.length - 1; i >= 0; i--) {
              const entry = this.entries[i];

              if (entry.type === FormattingElementList$3.ELEMENT_ENTRY && entry.element === element) {
                  return entry;
              }
          }

          return null;
      }
  }

  //Entry types
  FormattingElementList$3.MARKER_ENTRY = 'MARKER_ENTRY';
  FormattingElementList$3.ELEMENT_ENTRY = 'ELEMENT_ENTRY';

  var formattingElementList$1 = FormattingElementList$3;

  class Mixin$j {
      constructor(host) {
          const originalMethods = {};
          const overriddenMethods = this._getOverriddenMethods(this, originalMethods);

          for (const key of Object.keys(overriddenMethods)) {
              if (typeof overriddenMethods[key] === 'function') {
                  originalMethods[key] = host[key];
                  host[key] = overriddenMethods[key];
              }
          }
      }

      _getOverriddenMethods() {
          throw new Error('Not implemented');
      }
  }

  Mixin$j.install = function(host, Ctor, opts) {
      if (!host.__mixins) {
          host.__mixins = [];
      }

      for (let i = 0; i < host.__mixins.length; i++) {
          if (host.__mixins[i].constructor === Ctor) {
              return host.__mixins[i];
          }
      }

      const mixin = new Ctor(host, opts);

      host.__mixins.push(mixin);

      return mixin;
  };

  var mixin$1 = Mixin$j;

  const Mixin$i = mixin$1;

  class PositionTrackingPreprocessorMixin$5 extends Mixin$i {
      constructor(preprocessor) {
          super(preprocessor);

          this.preprocessor = preprocessor;
          this.isEol = false;
          this.lineStartPos = 0;
          this.droppedBufferSize = 0;

          this.offset = 0;
          this.col = 0;
          this.line = 1;
      }

      _getOverriddenMethods(mxn, orig) {
          return {
              advance() {
                  const pos = this.pos + 1;
                  const ch = this.html[pos];

                  //NOTE: LF should be in the last column of the line
                  if (mxn.isEol) {
                      mxn.isEol = false;
                      mxn.line++;
                      mxn.lineStartPos = pos;
                  }

                  if (ch === '\n' || (ch === '\r' && this.html[pos + 1] !== '\n')) {
                      mxn.isEol = true;
                  }

                  mxn.col = pos - mxn.lineStartPos + 1;
                  mxn.offset = mxn.droppedBufferSize + pos;

                  return orig.advance.call(this);
              },

              retreat() {
                  orig.retreat.call(this);

                  mxn.isEol = false;
                  mxn.col = this.pos - mxn.lineStartPos + 1;
              },

              dropParsedChunk() {
                  const prevPos = this.pos;

                  orig.dropParsedChunk.call(this);

                  const reduction = prevPos - this.pos;

                  mxn.lineStartPos -= reduction;
                  mxn.droppedBufferSize += reduction;
                  mxn.offset = mxn.droppedBufferSize + this.pos;
              }
          };
      }
  }

  var preprocessorMixin$3 = PositionTrackingPreprocessorMixin$5;

  const Mixin$h = mixin$1;
  const Tokenizer$8 = tokenizer$1;
  const PositionTrackingPreprocessorMixin$4 = preprocessorMixin$3;

  class LocationInfoTokenizerMixin$5 extends Mixin$h {
      constructor(tokenizer) {
          super(tokenizer);

          this.tokenizer = tokenizer;
          this.posTracker = Mixin$h.install(tokenizer.preprocessor, PositionTrackingPreprocessorMixin$4);
          this.currentAttrLocation = null;
          this.ctLoc = null;
      }

      _getCurrentLocation() {
          return {
              startLine: this.posTracker.line,
              startCol: this.posTracker.col,
              startOffset: this.posTracker.offset,
              endLine: -1,
              endCol: -1,
              endOffset: -1
          };
      }

      _attachCurrentAttrLocationInfo() {
          this.currentAttrLocation.endLine = this.posTracker.line;
          this.currentAttrLocation.endCol = this.posTracker.col;
          this.currentAttrLocation.endOffset = this.posTracker.offset;

          const currentToken = this.tokenizer.currentToken;
          const currentAttr = this.tokenizer.currentAttr;

          if (!currentToken.location.attrs) {
              currentToken.location.attrs = Object.create(null);
          }

          currentToken.location.attrs[currentAttr.name] = this.currentAttrLocation;
      }

      _getOverriddenMethods(mxn, orig) {
          const methods = {
              _createStartTagToken() {
                  orig._createStartTagToken.call(this);
                  this.currentToken.location = mxn.ctLoc;
              },

              _createEndTagToken() {
                  orig._createEndTagToken.call(this);
                  this.currentToken.location = mxn.ctLoc;
              },

              _createCommentToken() {
                  orig._createCommentToken.call(this);
                  this.currentToken.location = mxn.ctLoc;
              },

              _createDoctypeToken(initialName) {
                  orig._createDoctypeToken.call(this, initialName);
                  this.currentToken.location = mxn.ctLoc;
              },

              _createCharacterToken(type, ch) {
                  orig._createCharacterToken.call(this, type, ch);
                  this.currentCharacterToken.location = mxn.ctLoc;
              },

              _createEOFToken() {
                  orig._createEOFToken.call(this);
                  this.currentToken.location = mxn._getCurrentLocation();
              },

              _createAttr(attrNameFirstCh) {
                  orig._createAttr.call(this, attrNameFirstCh);
                  mxn.currentAttrLocation = mxn._getCurrentLocation();
              },

              _leaveAttrName(toState) {
                  orig._leaveAttrName.call(this, toState);
                  mxn._attachCurrentAttrLocationInfo();
              },

              _leaveAttrValue(toState) {
                  orig._leaveAttrValue.call(this, toState);
                  mxn._attachCurrentAttrLocationInfo();
              },

              _emitCurrentToken() {
                  const ctLoc = this.currentToken.location;

                  //NOTE: if we have pending character token make it's end location equal to the
                  //current token's start location.
                  if (this.currentCharacterToken) {
                      this.currentCharacterToken.location.endLine = ctLoc.startLine;
                      this.currentCharacterToken.location.endCol = ctLoc.startCol;
                      this.currentCharacterToken.location.endOffset = ctLoc.startOffset;
                  }

                  if (this.currentToken.type === Tokenizer$8.EOF_TOKEN) {
                      ctLoc.endLine = ctLoc.startLine;
                      ctLoc.endCol = ctLoc.startCol;
                      ctLoc.endOffset = ctLoc.startOffset;
                  } else {
                      ctLoc.endLine = mxn.posTracker.line;
                      ctLoc.endCol = mxn.posTracker.col + 1;
                      ctLoc.endOffset = mxn.posTracker.offset + 1;
                  }

                  orig._emitCurrentToken.call(this);
              },

              _emitCurrentCharacterToken() {
                  const ctLoc = this.currentCharacterToken && this.currentCharacterToken.location;

                  //NOTE: if we have character token and it's location wasn't set in the _emitCurrentToken(),
                  //then set it's location at the current preprocessor position.
                  //We don't need to increment preprocessor position, since character token
                  //emission is always forced by the start of the next character token here.
                  //So, we already have advanced position.
                  if (ctLoc && ctLoc.endOffset === -1) {
                      ctLoc.endLine = mxn.posTracker.line;
                      ctLoc.endCol = mxn.posTracker.col;
                      ctLoc.endOffset = mxn.posTracker.offset;
                  }

                  orig._emitCurrentCharacterToken.call(this);
              }
          };

          //NOTE: patch initial states for each mode to obtain token start position
          Object.keys(Tokenizer$8.MODE).forEach(modeName => {
              const state = Tokenizer$8.MODE[modeName];

              methods[state] = function(cp) {
                  mxn.ctLoc = mxn._getCurrentLocation();
                  orig[state].call(this, cp);
              };
          });

          return methods;
      }
  }

  var tokenizerMixin$3 = LocationInfoTokenizerMixin$5;

  const Mixin$g = mixin$1;

  class LocationInfoOpenElementStackMixin$3 extends Mixin$g {
      constructor(stack, opts) {
          super(stack);

          this.onItemPop = opts.onItemPop;
      }

      _getOverriddenMethods(mxn, orig) {
          return {
              pop() {
                  mxn.onItemPop(this.current);
                  orig.pop.call(this);
              },

              popAllUpToHtmlElement() {
                  for (let i = this.stackTop; i > 0; i--) {
                      mxn.onItemPop(this.items[i]);
                  }

                  orig.popAllUpToHtmlElement.call(this);
              },

              remove(element) {
                  mxn.onItemPop(this.current);
                  orig.remove.call(this, element);
              }
          };
      }
  }

  var openElementStackMixin$1 = LocationInfoOpenElementStackMixin$3;

  const Mixin$f = mixin$1;
  const Tokenizer$7 = tokenizer$1;
  const LocationInfoTokenizerMixin$4 = tokenizerMixin$3;
  const LocationInfoOpenElementStackMixin$2 = openElementStackMixin$1;
  const HTML$6 = html$h;

  //Aliases
  const $$9 = HTML$6.TAG_NAMES;

  class LocationInfoParserMixin$3 extends Mixin$f {
      constructor(parser) {
          super(parser);

          this.parser = parser;
          this.treeAdapter = this.parser.treeAdapter;
          this.posTracker = null;
          this.lastStartTagToken = null;
          this.lastFosterParentingLocation = null;
          this.currentToken = null;
      }

      _setStartLocation(element) {
          let loc = null;

          if (this.lastStartTagToken) {
              loc = Object.assign({}, this.lastStartTagToken.location);
              loc.startTag = this.lastStartTagToken.location;
          }

          this.treeAdapter.setNodeSourceCodeLocation(element, loc);
      }

      _setEndLocation(element, closingToken) {
          const loc = this.treeAdapter.getNodeSourceCodeLocation(element);

          if (loc) {
              if (closingToken.location) {
                  const ctLoc = closingToken.location;
                  const tn = this.treeAdapter.getTagName(element);

                  // NOTE: For cases like <p> <p> </p> - First 'p' closes without a closing
                  // tag and for cases like <td> <p> </td> - 'p' closes without a closing tag.
                  const isClosingEndTag = closingToken.type === Tokenizer$7.END_TAG_TOKEN && tn === closingToken.tagName;
                  const endLoc = {};
                  if (isClosingEndTag) {
                      endLoc.endTag = Object.assign({}, ctLoc);
                      endLoc.endLine = ctLoc.endLine;
                      endLoc.endCol = ctLoc.endCol;
                      endLoc.endOffset = ctLoc.endOffset;
                  } else {
                      endLoc.endLine = ctLoc.startLine;
                      endLoc.endCol = ctLoc.startCol;
                      endLoc.endOffset = ctLoc.startOffset;
                  }

                  this.treeAdapter.updateNodeSourceCodeLocation(element, endLoc);
              }
          }
      }

      _getOverriddenMethods(mxn, orig) {
          return {
              _bootstrap(document, fragmentContext) {
                  orig._bootstrap.call(this, document, fragmentContext);

                  mxn.lastStartTagToken = null;
                  mxn.lastFosterParentingLocation = null;
                  mxn.currentToken = null;

                  const tokenizerMixin = Mixin$f.install(this.tokenizer, LocationInfoTokenizerMixin$4);

                  mxn.posTracker = tokenizerMixin.posTracker;

                  Mixin$f.install(this.openElements, LocationInfoOpenElementStackMixin$2, {
                      onItemPop: function(element) {
                          mxn._setEndLocation(element, mxn.currentToken);
                      }
                  });
              },

              _runParsingLoop(scriptHandler) {
                  orig._runParsingLoop.call(this, scriptHandler);

                  // NOTE: generate location info for elements
                  // that remains on open element stack
                  for (let i = this.openElements.stackTop; i >= 0; i--) {
                      mxn._setEndLocation(this.openElements.items[i], mxn.currentToken);
                  }
              },

              //Token processing
              _processTokenInForeignContent(token) {
                  mxn.currentToken = token;
                  orig._processTokenInForeignContent.call(this, token);
              },

              _processToken(token) {
                  mxn.currentToken = token;
                  orig._processToken.call(this, token);

                  //NOTE: <body> and <html> are never popped from the stack, so we need to updated
                  //their end location explicitly.
                  const requireExplicitUpdate =
                      token.type === Tokenizer$7.END_TAG_TOKEN &&
                      (token.tagName === $$9.HTML || (token.tagName === $$9.BODY && this.openElements.hasInScope($$9.BODY)));

                  if (requireExplicitUpdate) {
                      for (let i = this.openElements.stackTop; i >= 0; i--) {
                          const element = this.openElements.items[i];

                          if (this.treeAdapter.getTagName(element) === token.tagName) {
                              mxn._setEndLocation(element, token);
                              break;
                          }
                      }
                  }
              },

              //Doctype
              _setDocumentType(token) {
                  orig._setDocumentType.call(this, token);

                  const documentChildren = this.treeAdapter.getChildNodes(this.document);
                  const cnLength = documentChildren.length;

                  for (let i = 0; i < cnLength; i++) {
                      const node = documentChildren[i];

                      if (this.treeAdapter.isDocumentTypeNode(node)) {
                          this.treeAdapter.setNodeSourceCodeLocation(node, token.location);
                          break;
                      }
                  }
              },

              //Elements
              _attachElementToTree(element) {
                  //NOTE: _attachElementToTree is called from _appendElement, _insertElement and _insertTemplate methods.
                  //So we will use token location stored in this methods for the element.
                  mxn._setStartLocation(element);
                  mxn.lastStartTagToken = null;
                  orig._attachElementToTree.call(this, element);
              },

              _appendElement(token, namespaceURI) {
                  mxn.lastStartTagToken = token;
                  orig._appendElement.call(this, token, namespaceURI);
              },

              _insertElement(token, namespaceURI) {
                  mxn.lastStartTagToken = token;
                  orig._insertElement.call(this, token, namespaceURI);
              },

              _insertTemplate(token) {
                  mxn.lastStartTagToken = token;
                  orig._insertTemplate.call(this, token);

                  const tmplContent = this.treeAdapter.getTemplateContent(this.openElements.current);

                  this.treeAdapter.setNodeSourceCodeLocation(tmplContent, null);
              },

              _insertFakeRootElement() {
                  orig._insertFakeRootElement.call(this);
                  this.treeAdapter.setNodeSourceCodeLocation(this.openElements.current, null);
              },

              //Comments
              _appendCommentNode(token, parent) {
                  orig._appendCommentNode.call(this, token, parent);

                  const children = this.treeAdapter.getChildNodes(parent);
                  const commentNode = children[children.length - 1];

                  this.treeAdapter.setNodeSourceCodeLocation(commentNode, token.location);
              },

              //Text
              _findFosterParentingLocation() {
                  //NOTE: store last foster parenting location, so we will be able to find inserted text
                  //in case of foster parenting
                  mxn.lastFosterParentingLocation = orig._findFosterParentingLocation.call(this);

                  return mxn.lastFosterParentingLocation;
              },

              _insertCharacters(token) {
                  orig._insertCharacters.call(this, token);

                  const hasFosterParent = this._shouldFosterParentOnInsertion();

                  const parent =
                      (hasFosterParent && mxn.lastFosterParentingLocation.parent) ||
                      this.openElements.currentTmplContent ||
                      this.openElements.current;

                  const siblings = this.treeAdapter.getChildNodes(parent);

                  const textNodeIdx =
                      hasFosterParent && mxn.lastFosterParentingLocation.beforeElement
                          ? siblings.indexOf(mxn.lastFosterParentingLocation.beforeElement) - 1
                          : siblings.length - 1;

                  const textNode = siblings[textNodeIdx];

                  //NOTE: if we have location assigned by another token, then just update end position
                  const tnLoc = this.treeAdapter.getNodeSourceCodeLocation(textNode);

                  if (tnLoc) {
                      const { endLine, endCol, endOffset } = token.location;
                      this.treeAdapter.updateNodeSourceCodeLocation(textNode, { endLine, endCol, endOffset });
                  } else {
                      this.treeAdapter.setNodeSourceCodeLocation(textNode, token.location);
                  }
              }
          };
      }
  }

  var parserMixin$3 = LocationInfoParserMixin$3;

  const Mixin$e = mixin$1;

  class ErrorReportingMixinBase$7 extends Mixin$e {
      constructor(host, opts) {
          super(host);

          this.posTracker = null;
          this.onParseError = opts.onParseError;
      }

      _setErrorLocation(err) {
          err.startLine = err.endLine = this.posTracker.line;
          err.startCol = err.endCol = this.posTracker.col;
          err.startOffset = err.endOffset = this.posTracker.offset;
      }

      _reportError(code) {
          const err = {
              code: code,
              startLine: -1,
              startCol: -1,
              startOffset: -1,
              endLine: -1,
              endCol: -1,
              endOffset: -1
          };

          this._setErrorLocation(err);
          this.onParseError(err);
      }

      _getOverriddenMethods(mxn) {
          return {
              _err(code) {
                  mxn._reportError(code);
              }
          };
      }
  }

  var mixinBase$1 = ErrorReportingMixinBase$7;

  const ErrorReportingMixinBase$6 = mixinBase$1;
  const PositionTrackingPreprocessorMixin$3 = preprocessorMixin$3;
  const Mixin$d = mixin$1;

  class ErrorReportingPreprocessorMixin$3 extends ErrorReportingMixinBase$6 {
      constructor(preprocessor, opts) {
          super(preprocessor, opts);

          this.posTracker = Mixin$d.install(preprocessor, PositionTrackingPreprocessorMixin$3);
          this.lastErrOffset = -1;
      }

      _reportError(code) {
          //NOTE: avoid reporting error twice on advance/retreat
          if (this.lastErrOffset !== this.posTracker.offset) {
              this.lastErrOffset = this.posTracker.offset;
              super._reportError(code);
          }
      }
  }

  var preprocessorMixin$2 = ErrorReportingPreprocessorMixin$3;

  const ErrorReportingMixinBase$5 = mixinBase$1;
  const ErrorReportingPreprocessorMixin$2 = preprocessorMixin$2;
  const Mixin$c = mixin$1;

  class ErrorReportingTokenizerMixin$3 extends ErrorReportingMixinBase$5 {
      constructor(tokenizer, opts) {
          super(tokenizer, opts);

          const preprocessorMixin = Mixin$c.install(tokenizer.preprocessor, ErrorReportingPreprocessorMixin$2, opts);

          this.posTracker = preprocessorMixin.posTracker;
      }
  }

  var tokenizerMixin$2 = ErrorReportingTokenizerMixin$3;

  const ErrorReportingMixinBase$4 = mixinBase$1;
  const ErrorReportingTokenizerMixin$2 = tokenizerMixin$2;
  const LocationInfoTokenizerMixin$3 = tokenizerMixin$3;
  const Mixin$b = mixin$1;

  class ErrorReportingParserMixin$3 extends ErrorReportingMixinBase$4 {
      constructor(parser, opts) {
          super(parser, opts);

          this.opts = opts;
          this.ctLoc = null;
          this.locBeforeToken = false;
      }

      _setErrorLocation(err) {
          if (this.ctLoc) {
              err.startLine = this.ctLoc.startLine;
              err.startCol = this.ctLoc.startCol;
              err.startOffset = this.ctLoc.startOffset;

              err.endLine = this.locBeforeToken ? this.ctLoc.startLine : this.ctLoc.endLine;
              err.endCol = this.locBeforeToken ? this.ctLoc.startCol : this.ctLoc.endCol;
              err.endOffset = this.locBeforeToken ? this.ctLoc.startOffset : this.ctLoc.endOffset;
          }
      }

      _getOverriddenMethods(mxn, orig) {
          return {
              _bootstrap(document, fragmentContext) {
                  orig._bootstrap.call(this, document, fragmentContext);

                  Mixin$b.install(this.tokenizer, ErrorReportingTokenizerMixin$2, mxn.opts);
                  Mixin$b.install(this.tokenizer, LocationInfoTokenizerMixin$3);
              },

              _processInputToken(token) {
                  mxn.ctLoc = token.location;

                  orig._processInputToken.call(this, token);
              },

              _err(code, options) {
                  mxn.locBeforeToken = options && options.beforeToken;
                  mxn._reportError(code);
              }
          };
      }
  }

  var parserMixin$2 = ErrorReportingParserMixin$3;

  var _default$1 = {};

  const { DOCUMENT_MODE: DOCUMENT_MODE$3 } = html$h;

  //Node construction
  _default$1.createDocument = function() {
      return {
          nodeName: '#document',
          mode: DOCUMENT_MODE$3.NO_QUIRKS,
          childNodes: []
      };
  };

  _default$1.createDocumentFragment = function() {
      return {
          nodeName: '#document-fragment',
          childNodes: []
      };
  };

  _default$1.createElement = function(tagName, namespaceURI, attrs) {
      return {
          nodeName: tagName,
          tagName: tagName,
          attrs: attrs,
          namespaceURI: namespaceURI,
          childNodes: [],
          parentNode: null
      };
  };

  _default$1.createCommentNode = function(data) {
      return {
          nodeName: '#comment',
          data: data,
          parentNode: null
      };
  };

  const createTextNode$1 = function(value) {
      return {
          nodeName: '#text',
          value: value,
          parentNode: null
      };
  };

  //Tree mutation
  const appendChild$1 = (_default$1.appendChild = function(parentNode, newNode) {
      parentNode.childNodes.push(newNode);
      newNode.parentNode = parentNode;
  });

  const insertBefore$1 = (_default$1.insertBefore = function(parentNode, newNode, referenceNode) {
      const insertionIdx = parentNode.childNodes.indexOf(referenceNode);

      parentNode.childNodes.splice(insertionIdx, 0, newNode);
      newNode.parentNode = parentNode;
  });

  _default$1.setTemplateContent = function(templateElement, contentElement) {
      templateElement.content = contentElement;
  };

  _default$1.getTemplateContent = function(templateElement) {
      return templateElement.content;
  };

  _default$1.setDocumentType = function(document, name, publicId, systemId) {
      let doctypeNode = null;

      for (let i = 0; i < document.childNodes.length; i++) {
          if (document.childNodes[i].nodeName === '#documentType') {
              doctypeNode = document.childNodes[i];
              break;
          }
      }

      if (doctypeNode) {
          doctypeNode.name = name;
          doctypeNode.publicId = publicId;
          doctypeNode.systemId = systemId;
      } else {
          appendChild$1(document, {
              nodeName: '#documentType',
              name: name,
              publicId: publicId,
              systemId: systemId
          });
      }
  };

  _default$1.setDocumentMode = function(document, mode) {
      document.mode = mode;
  };

  _default$1.getDocumentMode = function(document) {
      return document.mode;
  };

  _default$1.detachNode = function(node) {
      if (node.parentNode) {
          const idx = node.parentNode.childNodes.indexOf(node);

          node.parentNode.childNodes.splice(idx, 1);
          node.parentNode = null;
      }
  };

  _default$1.insertText = function(parentNode, text) {
      if (parentNode.childNodes.length) {
          const prevNode = parentNode.childNodes[parentNode.childNodes.length - 1];

          if (prevNode.nodeName === '#text') {
              prevNode.value += text;
              return;
          }
      }

      appendChild$1(parentNode, createTextNode$1(text));
  };

  _default$1.insertTextBefore = function(parentNode, text, referenceNode) {
      const prevNode = parentNode.childNodes[parentNode.childNodes.indexOf(referenceNode) - 1];

      if (prevNode && prevNode.nodeName === '#text') {
          prevNode.value += text;
      } else {
          insertBefore$1(parentNode, createTextNode$1(text), referenceNode);
      }
  };

  _default$1.adoptAttributes = function(recipient, attrs) {
      const recipientAttrsMap = [];

      for (let i = 0; i < recipient.attrs.length; i++) {
          recipientAttrsMap.push(recipient.attrs[i].name);
      }

      for (let j = 0; j < attrs.length; j++) {
          if (recipientAttrsMap.indexOf(attrs[j].name) === -1) {
              recipient.attrs.push(attrs[j]);
          }
      }
  };

  //Tree traversing
  _default$1.getFirstChild = function(node) {
      return node.childNodes[0];
  };

  _default$1.getChildNodes = function(node) {
      return node.childNodes;
  };

  _default$1.getParentNode = function(node) {
      return node.parentNode;
  };

  _default$1.getAttrList = function(element) {
      return element.attrs;
  };

  //Node data
  _default$1.getTagName = function(element) {
      return element.tagName;
  };

  _default$1.getNamespaceURI = function(element) {
      return element.namespaceURI;
  };

  _default$1.getTextNodeContent = function(textNode) {
      return textNode.value;
  };

  _default$1.getCommentNodeContent = function(commentNode) {
      return commentNode.data;
  };

  _default$1.getDocumentTypeNodeName = function(doctypeNode) {
      return doctypeNode.name;
  };

  _default$1.getDocumentTypeNodePublicId = function(doctypeNode) {
      return doctypeNode.publicId;
  };

  _default$1.getDocumentTypeNodeSystemId = function(doctypeNode) {
      return doctypeNode.systemId;
  };

  //Node types
  _default$1.isTextNode = function(node) {
      return node.nodeName === '#text';
  };

  _default$1.isCommentNode = function(node) {
      return node.nodeName === '#comment';
  };

  _default$1.isDocumentTypeNode = function(node) {
      return node.nodeName === '#documentType';
  };

  _default$1.isElementNode = function(node) {
      return !!node.tagName;
  };

  // Source code location
  _default$1.setNodeSourceCodeLocation = function(node, location) {
      node.sourceCodeLocation = location;
  };

  _default$1.getNodeSourceCodeLocation = function(node) {
      return node.sourceCodeLocation;
  };

  _default$1.updateNodeSourceCodeLocation = function(node, endLocation) {
      node.sourceCodeLocation = Object.assign(node.sourceCodeLocation, endLocation);
  };

  var mergeOptions$3 = function mergeOptions(defaults, options) {
      options = options || Object.create(null);

      return [defaults, options].reduce((merged, optObj) => {
          Object.keys(optObj).forEach(key => {
              merged[key] = optObj[key];
          });

          return merged;
      }, Object.create(null));
  };

  var doctype$7 = {};

  const { DOCUMENT_MODE: DOCUMENT_MODE$2 } = html$h;

  //Const
  const VALID_DOCTYPE_NAME$1 = 'html';
  const VALID_SYSTEM_ID$1 = 'about:legacy-compat';
  const QUIRKS_MODE_SYSTEM_ID$1 = 'http://www.ibm.com/data/dtd/v11/ibmxhtml1-transitional.dtd';

  const QUIRKS_MODE_PUBLIC_ID_PREFIXES$1 = [
      '+//silmaril//dtd html pro v0r11 19970101//',
      '-//as//dtd html 3.0 aswedit + extensions//',
      '-//advasoft ltd//dtd html 3.0 aswedit + extensions//',
      '-//ietf//dtd html 2.0 level 1//',
      '-//ietf//dtd html 2.0 level 2//',
      '-//ietf//dtd html 2.0 strict level 1//',
      '-//ietf//dtd html 2.0 strict level 2//',
      '-//ietf//dtd html 2.0 strict//',
      '-//ietf//dtd html 2.0//',
      '-//ietf//dtd html 2.1e//',
      '-//ietf//dtd html 3.0//',
      '-//ietf//dtd html 3.2 final//',
      '-//ietf//dtd html 3.2//',
      '-//ietf//dtd html 3//',
      '-//ietf//dtd html level 0//',
      '-//ietf//dtd html level 1//',
      '-//ietf//dtd html level 2//',
      '-//ietf//dtd html level 3//',
      '-//ietf//dtd html strict level 0//',
      '-//ietf//dtd html strict level 1//',
      '-//ietf//dtd html strict level 2//',
      '-//ietf//dtd html strict level 3//',
      '-//ietf//dtd html strict//',
      '-//ietf//dtd html//',
      '-//metrius//dtd metrius presentational//',
      '-//microsoft//dtd internet explorer 2.0 html strict//',
      '-//microsoft//dtd internet explorer 2.0 html//',
      '-//microsoft//dtd internet explorer 2.0 tables//',
      '-//microsoft//dtd internet explorer 3.0 html strict//',
      '-//microsoft//dtd internet explorer 3.0 html//',
      '-//microsoft//dtd internet explorer 3.0 tables//',
      '-//netscape comm. corp.//dtd html//',
      '-//netscape comm. corp.//dtd strict html//',
      "-//o'reilly and associates//dtd html 2.0//",
      "-//o'reilly and associates//dtd html extended 1.0//",
      "-//o'reilly and associates//dtd html extended relaxed 1.0//",
      '-//sq//dtd html 2.0 hotmetal + extensions//',
      '-//softquad software//dtd hotmetal pro 6.0::19990601::extensions to html 4.0//',
      '-//softquad//dtd hotmetal pro 4.0::19971010::extensions to html 4.0//',
      '-//spyglass//dtd html 2.0 extended//',
      '-//sun microsystems corp.//dtd hotjava html//',
      '-//sun microsystems corp.//dtd hotjava strict html//',
      '-//w3c//dtd html 3 1995-03-24//',
      '-//w3c//dtd html 3.2 draft//',
      '-//w3c//dtd html 3.2 final//',
      '-//w3c//dtd html 3.2//',
      '-//w3c//dtd html 3.2s draft//',
      '-//w3c//dtd html 4.0 frameset//',
      '-//w3c//dtd html 4.0 transitional//',
      '-//w3c//dtd html experimental 19960712//',
      '-//w3c//dtd html experimental 970421//',
      '-//w3c//dtd w3 html//',
      '-//w3o//dtd w3 html 3.0//',
      '-//webtechs//dtd mozilla html 2.0//',
      '-//webtechs//dtd mozilla html//'
  ];

  const QUIRKS_MODE_NO_SYSTEM_ID_PUBLIC_ID_PREFIXES$1 = QUIRKS_MODE_PUBLIC_ID_PREFIXES$1.concat([
      '-//w3c//dtd html 4.01 frameset//',
      '-//w3c//dtd html 4.01 transitional//'
  ]);

  const QUIRKS_MODE_PUBLIC_IDS$1 = ['-//w3o//dtd w3 html strict 3.0//en//', '-/w3c/dtd html 4.0 transitional/en', 'html'];
  const LIMITED_QUIRKS_PUBLIC_ID_PREFIXES$1 = ['-//w3c//dtd xhtml 1.0 frameset//', '-//w3c//dtd xhtml 1.0 transitional//'];

  const LIMITED_QUIRKS_WITH_SYSTEM_ID_PUBLIC_ID_PREFIXES$1 = LIMITED_QUIRKS_PUBLIC_ID_PREFIXES$1.concat([
      '-//w3c//dtd html 4.01 frameset//',
      '-//w3c//dtd html 4.01 transitional//'
  ]);

  //Utils
  function enquoteDoctypeId$1(id) {
      const quote = id.indexOf('"') !== -1 ? "'" : '"';

      return quote + id + quote;
  }

  function hasPrefix$1(publicId, prefixes) {
      for (let i = 0; i < prefixes.length; i++) {
          if (publicId.indexOf(prefixes[i]) === 0) {
              return true;
          }
      }

      return false;
  }

  //API
  doctype$7.isConforming = function(token) {
      return (
          token.name === VALID_DOCTYPE_NAME$1 &&
          token.publicId === null &&
          (token.systemId === null || token.systemId === VALID_SYSTEM_ID$1)
      );
  };

  doctype$7.getDocumentMode = function(token) {
      if (token.name !== VALID_DOCTYPE_NAME$1) {
          return DOCUMENT_MODE$2.QUIRKS;
      }

      const systemId = token.systemId;

      if (systemId && systemId.toLowerCase() === QUIRKS_MODE_SYSTEM_ID$1) {
          return DOCUMENT_MODE$2.QUIRKS;
      }

      let publicId = token.publicId;

      if (publicId !== null) {
          publicId = publicId.toLowerCase();

          if (QUIRKS_MODE_PUBLIC_IDS$1.indexOf(publicId) > -1) {
              return DOCUMENT_MODE$2.QUIRKS;
          }

          let prefixes = systemId === null ? QUIRKS_MODE_NO_SYSTEM_ID_PUBLIC_ID_PREFIXES$1 : QUIRKS_MODE_PUBLIC_ID_PREFIXES$1;

          if (hasPrefix$1(publicId, prefixes)) {
              return DOCUMENT_MODE$2.QUIRKS;
          }

          prefixes =
              systemId === null ? LIMITED_QUIRKS_PUBLIC_ID_PREFIXES$1 : LIMITED_QUIRKS_WITH_SYSTEM_ID_PUBLIC_ID_PREFIXES$1;

          if (hasPrefix$1(publicId, prefixes)) {
              return DOCUMENT_MODE$2.LIMITED_QUIRKS;
          }
      }

      return DOCUMENT_MODE$2.NO_QUIRKS;
  };

  doctype$7.serializeContent = function(name, publicId, systemId) {
      let str = '!DOCTYPE ';

      if (name) {
          str += name;
      }

      if (publicId) {
          str += ' PUBLIC ' + enquoteDoctypeId$1(publicId);
      } else if (systemId) {
          str += ' SYSTEM';
      }

      if (systemId !== null) {
          str += ' ' + enquoteDoctypeId$1(systemId);
      }

      return str;
  };

  var foreignContent$3 = {};

  const Tokenizer$6 = tokenizer$1;
  const HTML$5 = html$h;

  //Aliases
  const $$8 = HTML$5.TAG_NAMES;
  const NS$5 = HTML$5.NAMESPACES;
  const ATTRS$3 = HTML$5.ATTRS;

  //MIME types
  const MIME_TYPES$1 = {
      TEXT_HTML: 'text/html',
      APPLICATION_XML: 'application/xhtml+xml'
  };

  //Attributes
  const DEFINITION_URL_ATTR$1 = 'definitionurl';
  const ADJUSTED_DEFINITION_URL_ATTR$1 = 'definitionURL';
  const SVG_ATTRS_ADJUSTMENT_MAP$1 = {
      attributename: 'attributeName',
      attributetype: 'attributeType',
      basefrequency: 'baseFrequency',
      baseprofile: 'baseProfile',
      calcmode: 'calcMode',
      clippathunits: 'clipPathUnits',
      diffuseconstant: 'diffuseConstant',
      edgemode: 'edgeMode',
      filterunits: 'filterUnits',
      glyphref: 'glyphRef',
      gradienttransform: 'gradientTransform',
      gradientunits: 'gradientUnits',
      kernelmatrix: 'kernelMatrix',
      kernelunitlength: 'kernelUnitLength',
      keypoints: 'keyPoints',
      keysplines: 'keySplines',
      keytimes: 'keyTimes',
      lengthadjust: 'lengthAdjust',
      limitingconeangle: 'limitingConeAngle',
      markerheight: 'markerHeight',
      markerunits: 'markerUnits',
      markerwidth: 'markerWidth',
      maskcontentunits: 'maskContentUnits',
      maskunits: 'maskUnits',
      numoctaves: 'numOctaves',
      pathlength: 'pathLength',
      patterncontentunits: 'patternContentUnits',
      patterntransform: 'patternTransform',
      patternunits: 'patternUnits',
      pointsatx: 'pointsAtX',
      pointsaty: 'pointsAtY',
      pointsatz: 'pointsAtZ',
      preservealpha: 'preserveAlpha',
      preserveaspectratio: 'preserveAspectRatio',
      primitiveunits: 'primitiveUnits',
      refx: 'refX',
      refy: 'refY',
      repeatcount: 'repeatCount',
      repeatdur: 'repeatDur',
      requiredextensions: 'requiredExtensions',
      requiredfeatures: 'requiredFeatures',
      specularconstant: 'specularConstant',
      specularexponent: 'specularExponent',
      spreadmethod: 'spreadMethod',
      startoffset: 'startOffset',
      stddeviation: 'stdDeviation',
      stitchtiles: 'stitchTiles',
      surfacescale: 'surfaceScale',
      systemlanguage: 'systemLanguage',
      tablevalues: 'tableValues',
      targetx: 'targetX',
      targety: 'targetY',
      textlength: 'textLength',
      viewbox: 'viewBox',
      viewtarget: 'viewTarget',
      xchannelselector: 'xChannelSelector',
      ychannelselector: 'yChannelSelector',
      zoomandpan: 'zoomAndPan'
  };

  const XML_ATTRS_ADJUSTMENT_MAP$1 = {
      'xlink:actuate': { prefix: 'xlink', name: 'actuate', namespace: NS$5.XLINK },
      'xlink:arcrole': { prefix: 'xlink', name: 'arcrole', namespace: NS$5.XLINK },
      'xlink:href': { prefix: 'xlink', name: 'href', namespace: NS$5.XLINK },
      'xlink:role': { prefix: 'xlink', name: 'role', namespace: NS$5.XLINK },
      'xlink:show': { prefix: 'xlink', name: 'show', namespace: NS$5.XLINK },
      'xlink:title': { prefix: 'xlink', name: 'title', namespace: NS$5.XLINK },
      'xlink:type': { prefix: 'xlink', name: 'type', namespace: NS$5.XLINK },
      'xml:base': { prefix: 'xml', name: 'base', namespace: NS$5.XML },
      'xml:lang': { prefix: 'xml', name: 'lang', namespace: NS$5.XML },
      'xml:space': { prefix: 'xml', name: 'space', namespace: NS$5.XML },
      xmlns: { prefix: '', name: 'xmlns', namespace: NS$5.XMLNS },
      'xmlns:xlink': { prefix: 'xmlns', name: 'xlink', namespace: NS$5.XMLNS }
  };

  //SVG tag names adjustment map
  const SVG_TAG_NAMES_ADJUSTMENT_MAP$1 = (foreignContent$3.SVG_TAG_NAMES_ADJUSTMENT_MAP = {
      altglyph: 'altGlyph',
      altglyphdef: 'altGlyphDef',
      altglyphitem: 'altGlyphItem',
      animatecolor: 'animateColor',
      animatemotion: 'animateMotion',
      animatetransform: 'animateTransform',
      clippath: 'clipPath',
      feblend: 'feBlend',
      fecolormatrix: 'feColorMatrix',
      fecomponenttransfer: 'feComponentTransfer',
      fecomposite: 'feComposite',
      feconvolvematrix: 'feConvolveMatrix',
      fediffuselighting: 'feDiffuseLighting',
      fedisplacementmap: 'feDisplacementMap',
      fedistantlight: 'feDistantLight',
      feflood: 'feFlood',
      fefunca: 'feFuncA',
      fefuncb: 'feFuncB',
      fefuncg: 'feFuncG',
      fefuncr: 'feFuncR',
      fegaussianblur: 'feGaussianBlur',
      feimage: 'feImage',
      femerge: 'feMerge',
      femergenode: 'feMergeNode',
      femorphology: 'feMorphology',
      feoffset: 'feOffset',
      fepointlight: 'fePointLight',
      fespecularlighting: 'feSpecularLighting',
      fespotlight: 'feSpotLight',
      fetile: 'feTile',
      feturbulence: 'feTurbulence',
      foreignobject: 'foreignObject',
      glyphref: 'glyphRef',
      lineargradient: 'linearGradient',
      radialgradient: 'radialGradient',
      textpath: 'textPath'
  });

  //Tags that causes exit from foreign content
  const EXITS_FOREIGN_CONTENT$1 = {
      [$$8.B]: true,
      [$$8.BIG]: true,
      [$$8.BLOCKQUOTE]: true,
      [$$8.BODY]: true,
      [$$8.BR]: true,
      [$$8.CENTER]: true,
      [$$8.CODE]: true,
      [$$8.DD]: true,
      [$$8.DIV]: true,
      [$$8.DL]: true,
      [$$8.DT]: true,
      [$$8.EM]: true,
      [$$8.EMBED]: true,
      [$$8.H1]: true,
      [$$8.H2]: true,
      [$$8.H3]: true,
      [$$8.H4]: true,
      [$$8.H5]: true,
      [$$8.H6]: true,
      [$$8.HEAD]: true,
      [$$8.HR]: true,
      [$$8.I]: true,
      [$$8.IMG]: true,
      [$$8.LI]: true,
      [$$8.LISTING]: true,
      [$$8.MENU]: true,
      [$$8.META]: true,
      [$$8.NOBR]: true,
      [$$8.OL]: true,
      [$$8.P]: true,
      [$$8.PRE]: true,
      [$$8.RUBY]: true,
      [$$8.S]: true,
      [$$8.SMALL]: true,
      [$$8.SPAN]: true,
      [$$8.STRONG]: true,
      [$$8.STRIKE]: true,
      [$$8.SUB]: true,
      [$$8.SUP]: true,
      [$$8.TABLE]: true,
      [$$8.TT]: true,
      [$$8.U]: true,
      [$$8.UL]: true,
      [$$8.VAR]: true
  };

  //Check exit from foreign content
  foreignContent$3.causesExit = function(startTagToken) {
      const tn = startTagToken.tagName;
      const isFontWithAttrs =
          tn === $$8.FONT &&
          (Tokenizer$6.getTokenAttr(startTagToken, ATTRS$3.COLOR) !== null ||
              Tokenizer$6.getTokenAttr(startTagToken, ATTRS$3.SIZE) !== null ||
              Tokenizer$6.getTokenAttr(startTagToken, ATTRS$3.FACE) !== null);

      return isFontWithAttrs ? true : EXITS_FOREIGN_CONTENT$1[tn];
  };

  //Token adjustments
  foreignContent$3.adjustTokenMathMLAttrs = function(token) {
      for (let i = 0; i < token.attrs.length; i++) {
          if (token.attrs[i].name === DEFINITION_URL_ATTR$1) {
              token.attrs[i].name = ADJUSTED_DEFINITION_URL_ATTR$1;
              break;
          }
      }
  };

  foreignContent$3.adjustTokenSVGAttrs = function(token) {
      for (let i = 0; i < token.attrs.length; i++) {
          const adjustedAttrName = SVG_ATTRS_ADJUSTMENT_MAP$1[token.attrs[i].name];

          if (adjustedAttrName) {
              token.attrs[i].name = adjustedAttrName;
          }
      }
  };

  foreignContent$3.adjustTokenXMLAttrs = function(token) {
      for (let i = 0; i < token.attrs.length; i++) {
          const adjustedAttrEntry = XML_ATTRS_ADJUSTMENT_MAP$1[token.attrs[i].name];

          if (adjustedAttrEntry) {
              token.attrs[i].prefix = adjustedAttrEntry.prefix;
              token.attrs[i].name = adjustedAttrEntry.name;
              token.attrs[i].namespace = adjustedAttrEntry.namespace;
          }
      }
  };

  foreignContent$3.adjustTokenSVGTagName = function(token) {
      const adjustedTagName = SVG_TAG_NAMES_ADJUSTMENT_MAP$1[token.tagName];

      if (adjustedTagName) {
          token.tagName = adjustedTagName;
      }
  };

  //Integration points
  function isMathMLTextIntegrationPoint$1(tn, ns) {
      return ns === NS$5.MATHML && (tn === $$8.MI || tn === $$8.MO || tn === $$8.MN || tn === $$8.MS || tn === $$8.MTEXT);
  }

  function isHtmlIntegrationPoint$1(tn, ns, attrs) {
      if (ns === NS$5.MATHML && tn === $$8.ANNOTATION_XML) {
          for (let i = 0; i < attrs.length; i++) {
              if (attrs[i].name === ATTRS$3.ENCODING) {
                  const value = attrs[i].value.toLowerCase();

                  return value === MIME_TYPES$1.TEXT_HTML || value === MIME_TYPES$1.APPLICATION_XML;
              }
          }
      }

      return ns === NS$5.SVG && (tn === $$8.FOREIGN_OBJECT || tn === $$8.DESC || tn === $$8.TITLE);
  }

  foreignContent$3.isIntegrationPoint = function(tn, ns, attrs, foreignNS) {
      if ((!foreignNS || foreignNS === NS$5.HTML) && isHtmlIntegrationPoint$1(tn, ns, attrs)) {
          return true;
      }

      if ((!foreignNS || foreignNS === NS$5.MATHML) && isMathMLTextIntegrationPoint$1(tn, ns)) {
          return true;
      }

      return false;
  };

  const Tokenizer$5 = tokenizer$1;
  const OpenElementStack$2 = openElementStack$1;
  const FormattingElementList$2 = formattingElementList$1;
  const LocationInfoParserMixin$2 = parserMixin$3;
  const ErrorReportingParserMixin$2 = parserMixin$2;
  const Mixin$a = mixin$1;
  const defaultTreeAdapter$1 = _default$1;
  const mergeOptions$2 = mergeOptions$3;
  const doctype$6 = doctype$7;
  const foreignContent$2 = foreignContent$3;
  const ERR$3 = errorCodes$1;
  const unicode$4 = unicode$7;
  const HTML$4 = html$h;

  //Aliases
  const $$7 = HTML$4.TAG_NAMES;
  const NS$4 = HTML$4.NAMESPACES;
  const ATTRS$2 = HTML$4.ATTRS;

  const DEFAULT_OPTIONS$1 = {
      scriptingEnabled: true,
      sourceCodeLocationInfo: false,
      onParseError: null,
      treeAdapter: defaultTreeAdapter$1
  };

  //Misc constants
  const HIDDEN_INPUT_TYPE$1 = 'hidden';

  //Adoption agency loops iteration count
  const AA_OUTER_LOOP_ITER$1 = 8;
  const AA_INNER_LOOP_ITER$1 = 3;

  //Insertion modes
  const INITIAL_MODE$1 = 'INITIAL_MODE';
  const BEFORE_HTML_MODE$1 = 'BEFORE_HTML_MODE';
  const BEFORE_HEAD_MODE$1 = 'BEFORE_HEAD_MODE';
  const IN_HEAD_MODE$1 = 'IN_HEAD_MODE';
  const IN_HEAD_NO_SCRIPT_MODE$1 = 'IN_HEAD_NO_SCRIPT_MODE';
  const AFTER_HEAD_MODE$1 = 'AFTER_HEAD_MODE';
  const IN_BODY_MODE$1 = 'IN_BODY_MODE';
  const TEXT_MODE$1 = 'TEXT_MODE';
  const IN_TABLE_MODE$1 = 'IN_TABLE_MODE';
  const IN_TABLE_TEXT_MODE$1 = 'IN_TABLE_TEXT_MODE';
  const IN_CAPTION_MODE$1 = 'IN_CAPTION_MODE';
  const IN_COLUMN_GROUP_MODE$1 = 'IN_COLUMN_GROUP_MODE';
  const IN_TABLE_BODY_MODE$1 = 'IN_TABLE_BODY_MODE';
  const IN_ROW_MODE$1 = 'IN_ROW_MODE';
  const IN_CELL_MODE$1 = 'IN_CELL_MODE';
  const IN_SELECT_MODE$1 = 'IN_SELECT_MODE';
  const IN_SELECT_IN_TABLE_MODE$1 = 'IN_SELECT_IN_TABLE_MODE';
  const IN_TEMPLATE_MODE$1 = 'IN_TEMPLATE_MODE';
  const AFTER_BODY_MODE$1 = 'AFTER_BODY_MODE';
  const IN_FRAMESET_MODE$1 = 'IN_FRAMESET_MODE';
  const AFTER_FRAMESET_MODE$1 = 'AFTER_FRAMESET_MODE';
  const AFTER_AFTER_BODY_MODE$1 = 'AFTER_AFTER_BODY_MODE';
  const AFTER_AFTER_FRAMESET_MODE$1 = 'AFTER_AFTER_FRAMESET_MODE';

  //Insertion mode reset map
  const INSERTION_MODE_RESET_MAP$1 = {
      [$$7.TR]: IN_ROW_MODE$1,
      [$$7.TBODY]: IN_TABLE_BODY_MODE$1,
      [$$7.THEAD]: IN_TABLE_BODY_MODE$1,
      [$$7.TFOOT]: IN_TABLE_BODY_MODE$1,
      [$$7.CAPTION]: IN_CAPTION_MODE$1,
      [$$7.COLGROUP]: IN_COLUMN_GROUP_MODE$1,
      [$$7.TABLE]: IN_TABLE_MODE$1,
      [$$7.BODY]: IN_BODY_MODE$1,
      [$$7.FRAMESET]: IN_FRAMESET_MODE$1
  };

  //Template insertion mode switch map
  const TEMPLATE_INSERTION_MODE_SWITCH_MAP$1 = {
      [$$7.CAPTION]: IN_TABLE_MODE$1,
      [$$7.COLGROUP]: IN_TABLE_MODE$1,
      [$$7.TBODY]: IN_TABLE_MODE$1,
      [$$7.TFOOT]: IN_TABLE_MODE$1,
      [$$7.THEAD]: IN_TABLE_MODE$1,
      [$$7.COL]: IN_COLUMN_GROUP_MODE$1,
      [$$7.TR]: IN_TABLE_BODY_MODE$1,
      [$$7.TD]: IN_ROW_MODE$1,
      [$$7.TH]: IN_ROW_MODE$1
  };

  //Token handlers map for insertion modes
  const TOKEN_HANDLERS$1 = {
      [INITIAL_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: tokenInInitialMode$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: tokenInInitialMode$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: doctypeInInitialMode$1,
          [Tokenizer$5.START_TAG_TOKEN]: tokenInInitialMode$1,
          [Tokenizer$5.END_TAG_TOKEN]: tokenInInitialMode$1,
          [Tokenizer$5.EOF_TOKEN]: tokenInInitialMode$1
      },
      [BEFORE_HTML_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: tokenBeforeHtml$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: tokenBeforeHtml$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagBeforeHtml$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagBeforeHtml$1,
          [Tokenizer$5.EOF_TOKEN]: tokenBeforeHtml$1
      },
      [BEFORE_HEAD_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: tokenBeforeHead$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: tokenBeforeHead$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: misplacedDoctype$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagBeforeHead$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagBeforeHead$1,
          [Tokenizer$5.EOF_TOKEN]: tokenBeforeHead$1
      },
      [IN_HEAD_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: tokenInHead$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: tokenInHead$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: insertCharacters$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: misplacedDoctype$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagInHead$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagInHead$1,
          [Tokenizer$5.EOF_TOKEN]: tokenInHead$1
      },
      [IN_HEAD_NO_SCRIPT_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: tokenInHeadNoScript$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: tokenInHeadNoScript$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: insertCharacters$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: misplacedDoctype$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagInHeadNoScript$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagInHeadNoScript$1,
          [Tokenizer$5.EOF_TOKEN]: tokenInHeadNoScript$1
      },
      [AFTER_HEAD_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: tokenAfterHead$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: tokenAfterHead$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: insertCharacters$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: misplacedDoctype$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagAfterHead$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagAfterHead$1,
          [Tokenizer$5.EOF_TOKEN]: tokenAfterHead$1
      },
      [IN_BODY_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: characterInBody$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInBody$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagInBody$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagInBody$1,
          [Tokenizer$5.EOF_TOKEN]: eofInBody$1
      },
      [TEXT_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: insertCharacters$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: insertCharacters$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: insertCharacters$1,
          [Tokenizer$5.COMMENT_TOKEN]: ignoreToken$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: ignoreToken$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagInText$1,
          [Tokenizer$5.EOF_TOKEN]: eofInText$1
      },
      [IN_TABLE_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: characterInTable$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: characterInTable$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: characterInTable$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagInTable$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagInTable$1,
          [Tokenizer$5.EOF_TOKEN]: eofInBody$1
      },
      [IN_TABLE_TEXT_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: characterInTableText$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInTableText$1,
          [Tokenizer$5.COMMENT_TOKEN]: tokenInTableText$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: tokenInTableText$1,
          [Tokenizer$5.START_TAG_TOKEN]: tokenInTableText$1,
          [Tokenizer$5.END_TAG_TOKEN]: tokenInTableText$1,
          [Tokenizer$5.EOF_TOKEN]: tokenInTableText$1
      },
      [IN_CAPTION_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: characterInBody$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInBody$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagInCaption$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagInCaption$1,
          [Tokenizer$5.EOF_TOKEN]: eofInBody$1
      },
      [IN_COLUMN_GROUP_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: tokenInColumnGroup$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: tokenInColumnGroup$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: insertCharacters$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagInColumnGroup$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagInColumnGroup$1,
          [Tokenizer$5.EOF_TOKEN]: eofInBody$1
      },
      [IN_TABLE_BODY_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: characterInTable$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: characterInTable$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: characterInTable$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagInTableBody$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagInTableBody$1,
          [Tokenizer$5.EOF_TOKEN]: eofInBody$1
      },
      [IN_ROW_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: characterInTable$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: characterInTable$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: characterInTable$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagInRow$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagInRow$1,
          [Tokenizer$5.EOF_TOKEN]: eofInBody$1
      },
      [IN_CELL_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: characterInBody$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInBody$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagInCell$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagInCell$1,
          [Tokenizer$5.EOF_TOKEN]: eofInBody$1
      },
      [IN_SELECT_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: insertCharacters$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: insertCharacters$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagInSelect$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagInSelect$1,
          [Tokenizer$5.EOF_TOKEN]: eofInBody$1
      },
      [IN_SELECT_IN_TABLE_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: insertCharacters$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: insertCharacters$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagInSelectInTable$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagInSelectInTable$1,
          [Tokenizer$5.EOF_TOKEN]: eofInBody$1
      },
      [IN_TEMPLATE_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: characterInBody$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInBody$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagInTemplate$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagInTemplate$1,
          [Tokenizer$5.EOF_TOKEN]: eofInTemplate$1
      },
      [AFTER_BODY_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: tokenAfterBody$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: tokenAfterBody$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInBody$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendCommentToRootHtmlElement$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagAfterBody$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagAfterBody$1,
          [Tokenizer$5.EOF_TOKEN]: stopParsing$1
      },
      [IN_FRAMESET_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: insertCharacters$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagInFrameset$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagInFrameset$1,
          [Tokenizer$5.EOF_TOKEN]: stopParsing$1
      },
      [AFTER_FRAMESET_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: insertCharacters$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendComment$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagAfterFrameset$1,
          [Tokenizer$5.END_TAG_TOKEN]: endTagAfterFrameset$1,
          [Tokenizer$5.EOF_TOKEN]: stopParsing$1
      },
      [AFTER_AFTER_BODY_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: tokenAfterAfterBody$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: tokenAfterAfterBody$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInBody$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendCommentToDocument$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagAfterAfterBody$1,
          [Tokenizer$5.END_TAG_TOKEN]: tokenAfterAfterBody$1,
          [Tokenizer$5.EOF_TOKEN]: stopParsing$1
      },
      [AFTER_AFTER_FRAMESET_MODE$1]: {
          [Tokenizer$5.CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.NULL_CHARACTER_TOKEN]: ignoreToken$1,
          [Tokenizer$5.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInBody$1,
          [Tokenizer$5.COMMENT_TOKEN]: appendCommentToDocument$1,
          [Tokenizer$5.DOCTYPE_TOKEN]: ignoreToken$1,
          [Tokenizer$5.START_TAG_TOKEN]: startTagAfterAfterFrameset$1,
          [Tokenizer$5.END_TAG_TOKEN]: ignoreToken$1,
          [Tokenizer$5.EOF_TOKEN]: stopParsing$1
      }
  };

  //Parser
  class Parser$1 {
      constructor(options) {
          this.options = mergeOptions$2(DEFAULT_OPTIONS$1, options);

          this.treeAdapter = this.options.treeAdapter;
          this.pendingScript = null;

          if (this.options.sourceCodeLocationInfo) {
              Mixin$a.install(this, LocationInfoParserMixin$2);
          }

          if (this.options.onParseError) {
              Mixin$a.install(this, ErrorReportingParserMixin$2, { onParseError: this.options.onParseError });
          }
      }

      // API
      parse(html) {
          const document = this.treeAdapter.createDocument();

          this._bootstrap(document, null);
          this.tokenizer.write(html, true);
          this._runParsingLoop(null);

          return document;
      }

      parseFragment(html, fragmentContext) {
          //NOTE: use <template> element as a fragment context if context element was not provided,
          //so we will parse in "forgiving" manner
          if (!fragmentContext) {
              fragmentContext = this.treeAdapter.createElement($$7.TEMPLATE, NS$4.HTML, []);
          }

          //NOTE: create fake element which will be used as 'document' for fragment parsing.
          //This is important for jsdom there 'document' can't be recreated, therefore
          //fragment parsing causes messing of the main `document`.
          const documentMock = this.treeAdapter.createElement('documentmock', NS$4.HTML, []);

          this._bootstrap(documentMock, fragmentContext);

          if (this.treeAdapter.getTagName(fragmentContext) === $$7.TEMPLATE) {
              this._pushTmplInsertionMode(IN_TEMPLATE_MODE$1);
          }

          this._initTokenizerForFragmentParsing();
          this._insertFakeRootElement();
          this._resetInsertionMode();
          this._findFormInFragmentContext();
          this.tokenizer.write(html, true);
          this._runParsingLoop(null);

          const rootElement = this.treeAdapter.getFirstChild(documentMock);
          const fragment = this.treeAdapter.createDocumentFragment();

          this._adoptNodes(rootElement, fragment);

          return fragment;
      }

      //Bootstrap parser
      _bootstrap(document, fragmentContext) {
          this.tokenizer = new Tokenizer$5(this.options);

          this.stopped = false;

          this.insertionMode = INITIAL_MODE$1;
          this.originalInsertionMode = '';

          this.document = document;
          this.fragmentContext = fragmentContext;

          this.headElement = null;
          this.formElement = null;

          this.openElements = new OpenElementStack$2(this.document, this.treeAdapter);
          this.activeFormattingElements = new FormattingElementList$2(this.treeAdapter);

          this.tmplInsertionModeStack = [];
          this.tmplInsertionModeStackTop = -1;
          this.currentTmplInsertionMode = null;

          this.pendingCharacterTokens = [];
          this.hasNonWhitespacePendingCharacterToken = false;

          this.framesetOk = true;
          this.skipNextNewLine = false;
          this.fosterParentingEnabled = false;
      }

      //Errors
      _err() {
          // NOTE: err reporting is noop by default. Enabled by mixin.
      }

      //Parsing loop
      _runParsingLoop(scriptHandler) {
          while (!this.stopped) {
              this._setupTokenizerCDATAMode();

              const token = this.tokenizer.getNextToken();

              if (token.type === Tokenizer$5.HIBERNATION_TOKEN) {
                  break;
              }

              if (this.skipNextNewLine) {
                  this.skipNextNewLine = false;

                  if (token.type === Tokenizer$5.WHITESPACE_CHARACTER_TOKEN && token.chars[0] === '\n') {
                      if (token.chars.length === 1) {
                          continue;
                      }

                      token.chars = token.chars.substr(1);
                  }
              }

              this._processInputToken(token);

              if (scriptHandler && this.pendingScript) {
                  break;
              }
          }
      }

      runParsingLoopForCurrentChunk(writeCallback, scriptHandler) {
          this._runParsingLoop(scriptHandler);

          if (scriptHandler && this.pendingScript) {
              const script = this.pendingScript;

              this.pendingScript = null;

              scriptHandler(script);

              return;
          }

          if (writeCallback) {
              writeCallback();
          }
      }

      //Text parsing
      _setupTokenizerCDATAMode() {
          const current = this._getAdjustedCurrentElement();

          this.tokenizer.allowCDATA =
              current &&
              current !== this.document &&
              this.treeAdapter.getNamespaceURI(current) !== NS$4.HTML &&
              !this._isIntegrationPoint(current);
      }

      _switchToTextParsing(currentToken, nextTokenizerState) {
          this._insertElement(currentToken, NS$4.HTML);
          this.tokenizer.state = nextTokenizerState;
          this.originalInsertionMode = this.insertionMode;
          this.insertionMode = TEXT_MODE$1;
      }

      switchToPlaintextParsing() {
          this.insertionMode = TEXT_MODE$1;
          this.originalInsertionMode = IN_BODY_MODE$1;
          this.tokenizer.state = Tokenizer$5.MODE.PLAINTEXT;
      }

      //Fragment parsing
      _getAdjustedCurrentElement() {
          return this.openElements.stackTop === 0 && this.fragmentContext
              ? this.fragmentContext
              : this.openElements.current;
      }

      _findFormInFragmentContext() {
          let node = this.fragmentContext;

          do {
              if (this.treeAdapter.getTagName(node) === $$7.FORM) {
                  this.formElement = node;
                  break;
              }

              node = this.treeAdapter.getParentNode(node);
          } while (node);
      }

      _initTokenizerForFragmentParsing() {
          if (this.treeAdapter.getNamespaceURI(this.fragmentContext) === NS$4.HTML) {
              const tn = this.treeAdapter.getTagName(this.fragmentContext);

              if (tn === $$7.TITLE || tn === $$7.TEXTAREA) {
                  this.tokenizer.state = Tokenizer$5.MODE.RCDATA;
              } else if (
                  tn === $$7.STYLE ||
                  tn === $$7.XMP ||
                  tn === $$7.IFRAME ||
                  tn === $$7.NOEMBED ||
                  tn === $$7.NOFRAMES ||
                  tn === $$7.NOSCRIPT
              ) {
                  this.tokenizer.state = Tokenizer$5.MODE.RAWTEXT;
              } else if (tn === $$7.SCRIPT) {
                  this.tokenizer.state = Tokenizer$5.MODE.SCRIPT_DATA;
              } else if (tn === $$7.PLAINTEXT) {
                  this.tokenizer.state = Tokenizer$5.MODE.PLAINTEXT;
              }
          }
      }

      //Tree mutation
      _setDocumentType(token) {
          const name = token.name || '';
          const publicId = token.publicId || '';
          const systemId = token.systemId || '';

          this.treeAdapter.setDocumentType(this.document, name, publicId, systemId);
      }

      _attachElementToTree(element) {
          if (this._shouldFosterParentOnInsertion()) {
              this._fosterParentElement(element);
          } else {
              const parent = this.openElements.currentTmplContent || this.openElements.current;

              this.treeAdapter.appendChild(parent, element);
          }
      }

      _appendElement(token, namespaceURI) {
          const element = this.treeAdapter.createElement(token.tagName, namespaceURI, token.attrs);

          this._attachElementToTree(element);
      }

      _insertElement(token, namespaceURI) {
          const element = this.treeAdapter.createElement(token.tagName, namespaceURI, token.attrs);

          this._attachElementToTree(element);
          this.openElements.push(element);
      }

      _insertFakeElement(tagName) {
          const element = this.treeAdapter.createElement(tagName, NS$4.HTML, []);

          this._attachElementToTree(element);
          this.openElements.push(element);
      }

      _insertTemplate(token) {
          const tmpl = this.treeAdapter.createElement(token.tagName, NS$4.HTML, token.attrs);
          const content = this.treeAdapter.createDocumentFragment();

          this.treeAdapter.setTemplateContent(tmpl, content);
          this._attachElementToTree(tmpl);
          this.openElements.push(tmpl);
      }

      _insertFakeRootElement() {
          const element = this.treeAdapter.createElement($$7.HTML, NS$4.HTML, []);

          this.treeAdapter.appendChild(this.openElements.current, element);
          this.openElements.push(element);
      }

      _appendCommentNode(token, parent) {
          const commentNode = this.treeAdapter.createCommentNode(token.data);

          this.treeAdapter.appendChild(parent, commentNode);
      }

      _insertCharacters(token) {
          if (this._shouldFosterParentOnInsertion()) {
              this._fosterParentText(token.chars);
          } else {
              const parent = this.openElements.currentTmplContent || this.openElements.current;

              this.treeAdapter.insertText(parent, token.chars);
          }
      }

      _adoptNodes(donor, recipient) {
          for (let child = this.treeAdapter.getFirstChild(donor); child; child = this.treeAdapter.getFirstChild(donor)) {
              this.treeAdapter.detachNode(child);
              this.treeAdapter.appendChild(recipient, child);
          }
      }

      //Token processing
      _shouldProcessTokenInForeignContent(token) {
          const current = this._getAdjustedCurrentElement();

          if (!current || current === this.document) {
              return false;
          }

          const ns = this.treeAdapter.getNamespaceURI(current);

          if (ns === NS$4.HTML) {
              return false;
          }

          if (
              this.treeAdapter.getTagName(current) === $$7.ANNOTATION_XML &&
              ns === NS$4.MATHML &&
              token.type === Tokenizer$5.START_TAG_TOKEN &&
              token.tagName === $$7.SVG
          ) {
              return false;
          }

          const isCharacterToken =
              token.type === Tokenizer$5.CHARACTER_TOKEN ||
              token.type === Tokenizer$5.NULL_CHARACTER_TOKEN ||
              token.type === Tokenizer$5.WHITESPACE_CHARACTER_TOKEN;

          const isMathMLTextStartTag =
              token.type === Tokenizer$5.START_TAG_TOKEN && token.tagName !== $$7.MGLYPH && token.tagName !== $$7.MALIGNMARK;

          if ((isMathMLTextStartTag || isCharacterToken) && this._isIntegrationPoint(current, NS$4.MATHML)) {
              return false;
          }

          if (
              (token.type === Tokenizer$5.START_TAG_TOKEN || isCharacterToken) &&
              this._isIntegrationPoint(current, NS$4.HTML)
          ) {
              return false;
          }

          return token.type !== Tokenizer$5.EOF_TOKEN;
      }

      _processToken(token) {
          TOKEN_HANDLERS$1[this.insertionMode][token.type](this, token);
      }

      _processTokenInBodyMode(token) {
          TOKEN_HANDLERS$1[IN_BODY_MODE$1][token.type](this, token);
      }

      _processTokenInForeignContent(token) {
          if (token.type === Tokenizer$5.CHARACTER_TOKEN) {
              characterInForeignContent$1(this, token);
          } else if (token.type === Tokenizer$5.NULL_CHARACTER_TOKEN) {
              nullCharacterInForeignContent$1(this, token);
          } else if (token.type === Tokenizer$5.WHITESPACE_CHARACTER_TOKEN) {
              insertCharacters$1(this, token);
          } else if (token.type === Tokenizer$5.COMMENT_TOKEN) {
              appendComment$1(this, token);
          } else if (token.type === Tokenizer$5.START_TAG_TOKEN) {
              startTagInForeignContent$1(this, token);
          } else if (token.type === Tokenizer$5.END_TAG_TOKEN) {
              endTagInForeignContent$1(this, token);
          }
      }

      _processInputToken(token) {
          if (this._shouldProcessTokenInForeignContent(token)) {
              this._processTokenInForeignContent(token);
          } else {
              this._processToken(token);
          }

          if (token.type === Tokenizer$5.START_TAG_TOKEN && token.selfClosing && !token.ackSelfClosing) {
              this._err(ERR$3.nonVoidHtmlElementStartTagWithTrailingSolidus);
          }
      }

      //Integration points
      _isIntegrationPoint(element, foreignNS) {
          const tn = this.treeAdapter.getTagName(element);
          const ns = this.treeAdapter.getNamespaceURI(element);
          const attrs = this.treeAdapter.getAttrList(element);

          return foreignContent$2.isIntegrationPoint(tn, ns, attrs, foreignNS);
      }

      //Active formatting elements reconstruction
      _reconstructActiveFormattingElements() {
          const listLength = this.activeFormattingElements.length;

          if (listLength) {
              let unopenIdx = listLength;
              let entry = null;

              do {
                  unopenIdx--;
                  entry = this.activeFormattingElements.entries[unopenIdx];

                  if (entry.type === FormattingElementList$2.MARKER_ENTRY || this.openElements.contains(entry.element)) {
                      unopenIdx++;
                      break;
                  }
              } while (unopenIdx > 0);

              for (let i = unopenIdx; i < listLength; i++) {
                  entry = this.activeFormattingElements.entries[i];
                  this._insertElement(entry.token, this.treeAdapter.getNamespaceURI(entry.element));
                  entry.element = this.openElements.current;
              }
          }
      }

      //Close elements
      _closeTableCell() {
          this.openElements.generateImpliedEndTags();
          this.openElements.popUntilTableCellPopped();
          this.activeFormattingElements.clearToLastMarker();
          this.insertionMode = IN_ROW_MODE$1;
      }

      _closePElement() {
          this.openElements.generateImpliedEndTagsWithExclusion($$7.P);
          this.openElements.popUntilTagNamePopped($$7.P);
      }

      //Insertion modes
      _resetInsertionMode() {
          for (let i = this.openElements.stackTop, last = false; i >= 0; i--) {
              let element = this.openElements.items[i];

              if (i === 0) {
                  last = true;

                  if (this.fragmentContext) {
                      element = this.fragmentContext;
                  }
              }

              const tn = this.treeAdapter.getTagName(element);
              const newInsertionMode = INSERTION_MODE_RESET_MAP$1[tn];

              if (newInsertionMode) {
                  this.insertionMode = newInsertionMode;
                  break;
              } else if (!last && (tn === $$7.TD || tn === $$7.TH)) {
                  this.insertionMode = IN_CELL_MODE$1;
                  break;
              } else if (!last && tn === $$7.HEAD) {
                  this.insertionMode = IN_HEAD_MODE$1;
                  break;
              } else if (tn === $$7.SELECT) {
                  this._resetInsertionModeForSelect(i);
                  break;
              } else if (tn === $$7.TEMPLATE) {
                  this.insertionMode = this.currentTmplInsertionMode;
                  break;
              } else if (tn === $$7.HTML) {
                  this.insertionMode = this.headElement ? AFTER_HEAD_MODE$1 : BEFORE_HEAD_MODE$1;
                  break;
              } else if (last) {
                  this.insertionMode = IN_BODY_MODE$1;
                  break;
              }
          }
      }

      _resetInsertionModeForSelect(selectIdx) {
          if (selectIdx > 0) {
              for (let i = selectIdx - 1; i > 0; i--) {
                  const ancestor = this.openElements.items[i];
                  const tn = this.treeAdapter.getTagName(ancestor);

                  if (tn === $$7.TEMPLATE) {
                      break;
                  } else if (tn === $$7.TABLE) {
                      this.insertionMode = IN_SELECT_IN_TABLE_MODE$1;
                      return;
                  }
              }
          }

          this.insertionMode = IN_SELECT_MODE$1;
      }

      _pushTmplInsertionMode(mode) {
          this.tmplInsertionModeStack.push(mode);
          this.tmplInsertionModeStackTop++;
          this.currentTmplInsertionMode = mode;
      }

      _popTmplInsertionMode() {
          this.tmplInsertionModeStack.pop();
          this.tmplInsertionModeStackTop--;
          this.currentTmplInsertionMode = this.tmplInsertionModeStack[this.tmplInsertionModeStackTop];
      }

      //Foster parenting
      _isElementCausesFosterParenting(element) {
          const tn = this.treeAdapter.getTagName(element);

          return tn === $$7.TABLE || tn === $$7.TBODY || tn === $$7.TFOOT || tn === $$7.THEAD || tn === $$7.TR;
      }

      _shouldFosterParentOnInsertion() {
          return this.fosterParentingEnabled && this._isElementCausesFosterParenting(this.openElements.current);
      }

      _findFosterParentingLocation() {
          const location = {
              parent: null,
              beforeElement: null
          };

          for (let i = this.openElements.stackTop; i >= 0; i--) {
              const openElement = this.openElements.items[i];
              const tn = this.treeAdapter.getTagName(openElement);
              const ns = this.treeAdapter.getNamespaceURI(openElement);

              if (tn === $$7.TEMPLATE && ns === NS$4.HTML) {
                  location.parent = this.treeAdapter.getTemplateContent(openElement);
                  break;
              } else if (tn === $$7.TABLE) {
                  location.parent = this.treeAdapter.getParentNode(openElement);

                  if (location.parent) {
                      location.beforeElement = openElement;
                  } else {
                      location.parent = this.openElements.items[i - 1];
                  }

                  break;
              }
          }

          if (!location.parent) {
              location.parent = this.openElements.items[0];
          }

          return location;
      }

      _fosterParentElement(element) {
          const location = this._findFosterParentingLocation();

          if (location.beforeElement) {
              this.treeAdapter.insertBefore(location.parent, element, location.beforeElement);
          } else {
              this.treeAdapter.appendChild(location.parent, element);
          }
      }

      _fosterParentText(chars) {
          const location = this._findFosterParentingLocation();

          if (location.beforeElement) {
              this.treeAdapter.insertTextBefore(location.parent, chars, location.beforeElement);
          } else {
              this.treeAdapter.insertText(location.parent, chars);
          }
      }

      //Special elements
      _isSpecialElement(element) {
          const tn = this.treeAdapter.getTagName(element);
          const ns = this.treeAdapter.getNamespaceURI(element);

          return HTML$4.SPECIAL_ELEMENTS[ns][tn];
      }
  }

  var parser$3 = Parser$1;

  //Adoption agency algorithm
  //(see: http://www.whatwg.org/specs/web-apps/current-work/multipage/tree-construction.html#adoptionAgency)
  //------------------------------------------------------------------

  //Steps 5-8 of the algorithm
  function aaObtainFormattingElementEntry$1(p, token) {
      let formattingElementEntry = p.activeFormattingElements.getElementEntryInScopeWithTagName(token.tagName);

      if (formattingElementEntry) {
          if (!p.openElements.contains(formattingElementEntry.element)) {
              p.activeFormattingElements.removeEntry(formattingElementEntry);
              formattingElementEntry = null;
          } else if (!p.openElements.hasInScope(token.tagName)) {
              formattingElementEntry = null;
          }
      } else {
          genericEndTagInBody$1(p, token);
      }

      return formattingElementEntry;
  }

  //Steps 9 and 10 of the algorithm
  function aaObtainFurthestBlock$1(p, formattingElementEntry) {
      let furthestBlock = null;

      for (let i = p.openElements.stackTop; i >= 0; i--) {
          const element = p.openElements.items[i];

          if (element === formattingElementEntry.element) {
              break;
          }

          if (p._isSpecialElement(element)) {
              furthestBlock = element;
          }
      }

      if (!furthestBlock) {
          p.openElements.popUntilElementPopped(formattingElementEntry.element);
          p.activeFormattingElements.removeEntry(formattingElementEntry);
      }

      return furthestBlock;
  }

  //Step 13 of the algorithm
  function aaInnerLoop$1(p, furthestBlock, formattingElement) {
      let lastElement = furthestBlock;
      let nextElement = p.openElements.getCommonAncestor(furthestBlock);

      for (let i = 0, element = nextElement; element !== formattingElement; i++, element = nextElement) {
          //NOTE: store next element for the next loop iteration (it may be deleted from the stack by step 9.5)
          nextElement = p.openElements.getCommonAncestor(element);

          const elementEntry = p.activeFormattingElements.getElementEntry(element);
          const counterOverflow = elementEntry && i >= AA_INNER_LOOP_ITER$1;
          const shouldRemoveFromOpenElements = !elementEntry || counterOverflow;

          if (shouldRemoveFromOpenElements) {
              if (counterOverflow) {
                  p.activeFormattingElements.removeEntry(elementEntry);
              }

              p.openElements.remove(element);
          } else {
              element = aaRecreateElementFromEntry$1(p, elementEntry);

              if (lastElement === furthestBlock) {
                  p.activeFormattingElements.bookmark = elementEntry;
              }

              p.treeAdapter.detachNode(lastElement);
              p.treeAdapter.appendChild(element, lastElement);
              lastElement = element;
          }
      }

      return lastElement;
  }

  //Step 13.7 of the algorithm
  function aaRecreateElementFromEntry$1(p, elementEntry) {
      const ns = p.treeAdapter.getNamespaceURI(elementEntry.element);
      const newElement = p.treeAdapter.createElement(elementEntry.token.tagName, ns, elementEntry.token.attrs);

      p.openElements.replace(elementEntry.element, newElement);
      elementEntry.element = newElement;

      return newElement;
  }

  //Step 14 of the algorithm
  function aaInsertLastNodeInCommonAncestor$1(p, commonAncestor, lastElement) {
      if (p._isElementCausesFosterParenting(commonAncestor)) {
          p._fosterParentElement(lastElement);
      } else {
          const tn = p.treeAdapter.getTagName(commonAncestor);
          const ns = p.treeAdapter.getNamespaceURI(commonAncestor);

          if (tn === $$7.TEMPLATE && ns === NS$4.HTML) {
              commonAncestor = p.treeAdapter.getTemplateContent(commonAncestor);
          }

          p.treeAdapter.appendChild(commonAncestor, lastElement);
      }
  }

  //Steps 15-19 of the algorithm
  function aaReplaceFormattingElement$1(p, furthestBlock, formattingElementEntry) {
      const ns = p.treeAdapter.getNamespaceURI(formattingElementEntry.element);
      const token = formattingElementEntry.token;
      const newElement = p.treeAdapter.createElement(token.tagName, ns, token.attrs);

      p._adoptNodes(furthestBlock, newElement);
      p.treeAdapter.appendChild(furthestBlock, newElement);

      p.activeFormattingElements.insertElementAfterBookmark(newElement, formattingElementEntry.token);
      p.activeFormattingElements.removeEntry(formattingElementEntry);

      p.openElements.remove(formattingElementEntry.element);
      p.openElements.insertAfter(furthestBlock, newElement);
  }

  //Algorithm entry point
  function callAdoptionAgency$1(p, token) {
      let formattingElementEntry;

      for (let i = 0; i < AA_OUTER_LOOP_ITER$1; i++) {
          formattingElementEntry = aaObtainFormattingElementEntry$1(p, token);

          if (!formattingElementEntry) {
              break;
          }

          const furthestBlock = aaObtainFurthestBlock$1(p, formattingElementEntry);

          if (!furthestBlock) {
              break;
          }

          p.activeFormattingElements.bookmark = formattingElementEntry;

          const lastElement = aaInnerLoop$1(p, furthestBlock, formattingElementEntry.element);
          const commonAncestor = p.openElements.getCommonAncestor(formattingElementEntry.element);

          p.treeAdapter.detachNode(lastElement);
          aaInsertLastNodeInCommonAncestor$1(p, commonAncestor, lastElement);
          aaReplaceFormattingElement$1(p, furthestBlock, formattingElementEntry);
      }
  }

  //Generic token handlers
  //------------------------------------------------------------------
  function ignoreToken$1() {
      //NOTE: do nothing =)
  }

  function misplacedDoctype$1(p) {
      p._err(ERR$3.misplacedDoctype);
  }

  function appendComment$1(p, token) {
      p._appendCommentNode(token, p.openElements.currentTmplContent || p.openElements.current);
  }

  function appendCommentToRootHtmlElement$1(p, token) {
      p._appendCommentNode(token, p.openElements.items[0]);
  }

  function appendCommentToDocument$1(p, token) {
      p._appendCommentNode(token, p.document);
  }

  function insertCharacters$1(p, token) {
      p._insertCharacters(token);
  }

  function stopParsing$1(p) {
      p.stopped = true;
  }

  // The "initial" insertion mode
  //------------------------------------------------------------------
  function doctypeInInitialMode$1(p, token) {
      p._setDocumentType(token);

      const mode = token.forceQuirks ? HTML$4.DOCUMENT_MODE.QUIRKS : doctype$6.getDocumentMode(token);

      if (!doctype$6.isConforming(token)) {
          p._err(ERR$3.nonConformingDoctype);
      }

      p.treeAdapter.setDocumentMode(p.document, mode);

      p.insertionMode = BEFORE_HTML_MODE$1;
  }

  function tokenInInitialMode$1(p, token) {
      p._err(ERR$3.missingDoctype, { beforeToken: true });
      p.treeAdapter.setDocumentMode(p.document, HTML$4.DOCUMENT_MODE.QUIRKS);
      p.insertionMode = BEFORE_HTML_MODE$1;
      p._processToken(token);
  }

  // The "before html" insertion mode
  //------------------------------------------------------------------
  function startTagBeforeHtml$1(p, token) {
      if (token.tagName === $$7.HTML) {
          p._insertElement(token, NS$4.HTML);
          p.insertionMode = BEFORE_HEAD_MODE$1;
      } else {
          tokenBeforeHtml$1(p, token);
      }
  }

  function endTagBeforeHtml$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.HTML || tn === $$7.HEAD || tn === $$7.BODY || tn === $$7.BR) {
          tokenBeforeHtml$1(p, token);
      }
  }

  function tokenBeforeHtml$1(p, token) {
      p._insertFakeRootElement();
      p.insertionMode = BEFORE_HEAD_MODE$1;
      p._processToken(token);
  }

  // The "before head" insertion mode
  //------------------------------------------------------------------
  function startTagBeforeHead$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.HTML) {
          startTagInBody$1(p, token);
      } else if (tn === $$7.HEAD) {
          p._insertElement(token, NS$4.HTML);
          p.headElement = p.openElements.current;
          p.insertionMode = IN_HEAD_MODE$1;
      } else {
          tokenBeforeHead$1(p, token);
      }
  }

  function endTagBeforeHead$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.HEAD || tn === $$7.BODY || tn === $$7.HTML || tn === $$7.BR) {
          tokenBeforeHead$1(p, token);
      } else {
          p._err(ERR$3.endTagWithoutMatchingOpenElement);
      }
  }

  function tokenBeforeHead$1(p, token) {
      p._insertFakeElement($$7.HEAD);
      p.headElement = p.openElements.current;
      p.insertionMode = IN_HEAD_MODE$1;
      p._processToken(token);
  }

  // The "in head" insertion mode
  //------------------------------------------------------------------
  function startTagInHead$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.HTML) {
          startTagInBody$1(p, token);
      } else if (tn === $$7.BASE || tn === $$7.BASEFONT || tn === $$7.BGSOUND || tn === $$7.LINK || tn === $$7.META) {
          p._appendElement(token, NS$4.HTML);
          token.ackSelfClosing = true;
      } else if (tn === $$7.TITLE) {
          p._switchToTextParsing(token, Tokenizer$5.MODE.RCDATA);
      } else if (tn === $$7.NOSCRIPT) {
          if (p.options.scriptingEnabled) {
              p._switchToTextParsing(token, Tokenizer$5.MODE.RAWTEXT);
          } else {
              p._insertElement(token, NS$4.HTML);
              p.insertionMode = IN_HEAD_NO_SCRIPT_MODE$1;
          }
      } else if (tn === $$7.NOFRAMES || tn === $$7.STYLE) {
          p._switchToTextParsing(token, Tokenizer$5.MODE.RAWTEXT);
      } else if (tn === $$7.SCRIPT) {
          p._switchToTextParsing(token, Tokenizer$5.MODE.SCRIPT_DATA);
      } else if (tn === $$7.TEMPLATE) {
          p._insertTemplate(token, NS$4.HTML);
          p.activeFormattingElements.insertMarker();
          p.framesetOk = false;
          p.insertionMode = IN_TEMPLATE_MODE$1;
          p._pushTmplInsertionMode(IN_TEMPLATE_MODE$1);
      } else if (tn === $$7.HEAD) {
          p._err(ERR$3.misplacedStartTagForHeadElement);
      } else {
          tokenInHead$1(p, token);
      }
  }

  function endTagInHead$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.HEAD) {
          p.openElements.pop();
          p.insertionMode = AFTER_HEAD_MODE$1;
      } else if (tn === $$7.BODY || tn === $$7.BR || tn === $$7.HTML) {
          tokenInHead$1(p, token);
      } else if (tn === $$7.TEMPLATE) {
          if (p.openElements.tmplCount > 0) {
              p.openElements.generateImpliedEndTagsThoroughly();

              if (p.openElements.currentTagName !== $$7.TEMPLATE) {
                  p._err(ERR$3.closingOfElementWithOpenChildElements);
              }

              p.openElements.popUntilTagNamePopped($$7.TEMPLATE);
              p.activeFormattingElements.clearToLastMarker();
              p._popTmplInsertionMode();
              p._resetInsertionMode();
          } else {
              p._err(ERR$3.endTagWithoutMatchingOpenElement);
          }
      } else {
          p._err(ERR$3.endTagWithoutMatchingOpenElement);
      }
  }

  function tokenInHead$1(p, token) {
      p.openElements.pop();
      p.insertionMode = AFTER_HEAD_MODE$1;
      p._processToken(token);
  }

  // The "in head no script" insertion mode
  //------------------------------------------------------------------
  function startTagInHeadNoScript$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.HTML) {
          startTagInBody$1(p, token);
      } else if (
          tn === $$7.BASEFONT ||
          tn === $$7.BGSOUND ||
          tn === $$7.HEAD ||
          tn === $$7.LINK ||
          tn === $$7.META ||
          tn === $$7.NOFRAMES ||
          tn === $$7.STYLE
      ) {
          startTagInHead$1(p, token);
      } else if (tn === $$7.NOSCRIPT) {
          p._err(ERR$3.nestedNoscriptInHead);
      } else {
          tokenInHeadNoScript$1(p, token);
      }
  }

  function endTagInHeadNoScript$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.NOSCRIPT) {
          p.openElements.pop();
          p.insertionMode = IN_HEAD_MODE$1;
      } else if (tn === $$7.BR) {
          tokenInHeadNoScript$1(p, token);
      } else {
          p._err(ERR$3.endTagWithoutMatchingOpenElement);
      }
  }

  function tokenInHeadNoScript$1(p, token) {
      const errCode =
          token.type === Tokenizer$5.EOF_TOKEN ? ERR$3.openElementsLeftAfterEof : ERR$3.disallowedContentInNoscriptInHead;

      p._err(errCode);
      p.openElements.pop();
      p.insertionMode = IN_HEAD_MODE$1;
      p._processToken(token);
  }

  // The "after head" insertion mode
  //------------------------------------------------------------------
  function startTagAfterHead$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.HTML) {
          startTagInBody$1(p, token);
      } else if (tn === $$7.BODY) {
          p._insertElement(token, NS$4.HTML);
          p.framesetOk = false;
          p.insertionMode = IN_BODY_MODE$1;
      } else if (tn === $$7.FRAMESET) {
          p._insertElement(token, NS$4.HTML);
          p.insertionMode = IN_FRAMESET_MODE$1;
      } else if (
          tn === $$7.BASE ||
          tn === $$7.BASEFONT ||
          tn === $$7.BGSOUND ||
          tn === $$7.LINK ||
          tn === $$7.META ||
          tn === $$7.NOFRAMES ||
          tn === $$7.SCRIPT ||
          tn === $$7.STYLE ||
          tn === $$7.TEMPLATE ||
          tn === $$7.TITLE
      ) {
          p._err(ERR$3.abandonedHeadElementChild);
          p.openElements.push(p.headElement);
          startTagInHead$1(p, token);
          p.openElements.remove(p.headElement);
      } else if (tn === $$7.HEAD) {
          p._err(ERR$3.misplacedStartTagForHeadElement);
      } else {
          tokenAfterHead$1(p, token);
      }
  }

  function endTagAfterHead$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.BODY || tn === $$7.HTML || tn === $$7.BR) {
          tokenAfterHead$1(p, token);
      } else if (tn === $$7.TEMPLATE) {
          endTagInHead$1(p, token);
      } else {
          p._err(ERR$3.endTagWithoutMatchingOpenElement);
      }
  }

  function tokenAfterHead$1(p, token) {
      p._insertFakeElement($$7.BODY);
      p.insertionMode = IN_BODY_MODE$1;
      p._processToken(token);
  }

  // The "in body" insertion mode
  //------------------------------------------------------------------
  function whitespaceCharacterInBody$1(p, token) {
      p._reconstructActiveFormattingElements();
      p._insertCharacters(token);
  }

  function characterInBody$1(p, token) {
      p._reconstructActiveFormattingElements();
      p._insertCharacters(token);
      p.framesetOk = false;
  }

  function htmlStartTagInBody$1(p, token) {
      if (p.openElements.tmplCount === 0) {
          p.treeAdapter.adoptAttributes(p.openElements.items[0], token.attrs);
      }
  }

  function bodyStartTagInBody$1(p, token) {
      const bodyElement = p.openElements.tryPeekProperlyNestedBodyElement();

      if (bodyElement && p.openElements.tmplCount === 0) {
          p.framesetOk = false;
          p.treeAdapter.adoptAttributes(bodyElement, token.attrs);
      }
  }

  function framesetStartTagInBody$1(p, token) {
      const bodyElement = p.openElements.tryPeekProperlyNestedBodyElement();

      if (p.framesetOk && bodyElement) {
          p.treeAdapter.detachNode(bodyElement);
          p.openElements.popAllUpToHtmlElement();
          p._insertElement(token, NS$4.HTML);
          p.insertionMode = IN_FRAMESET_MODE$1;
      }
  }

  function addressStartTagInBody$1(p, token) {
      if (p.openElements.hasInButtonScope($$7.P)) {
          p._closePElement();
      }

      p._insertElement(token, NS$4.HTML);
  }

  function numberedHeaderStartTagInBody$1(p, token) {
      if (p.openElements.hasInButtonScope($$7.P)) {
          p._closePElement();
      }

      const tn = p.openElements.currentTagName;

      if (tn === $$7.H1 || tn === $$7.H2 || tn === $$7.H3 || tn === $$7.H4 || tn === $$7.H5 || tn === $$7.H6) {
          p.openElements.pop();
      }

      p._insertElement(token, NS$4.HTML);
  }

  function preStartTagInBody$1(p, token) {
      if (p.openElements.hasInButtonScope($$7.P)) {
          p._closePElement();
      }

      p._insertElement(token, NS$4.HTML);
      //NOTE: If the next token is a U+000A LINE FEED (LF) character token, then ignore that token and move
      //on to the next one. (Newlines at the start of pre blocks are ignored as an authoring convenience.)
      p.skipNextNewLine = true;
      p.framesetOk = false;
  }

  function formStartTagInBody$1(p, token) {
      const inTemplate = p.openElements.tmplCount > 0;

      if (!p.formElement || inTemplate) {
          if (p.openElements.hasInButtonScope($$7.P)) {
              p._closePElement();
          }

          p._insertElement(token, NS$4.HTML);

          if (!inTemplate) {
              p.formElement = p.openElements.current;
          }
      }
  }

  function listItemStartTagInBody$1(p, token) {
      p.framesetOk = false;

      const tn = token.tagName;

      for (let i = p.openElements.stackTop; i >= 0; i--) {
          const element = p.openElements.items[i];
          const elementTn = p.treeAdapter.getTagName(element);
          let closeTn = null;

          if (tn === $$7.LI && elementTn === $$7.LI) {
              closeTn = $$7.LI;
          } else if ((tn === $$7.DD || tn === $$7.DT) && (elementTn === $$7.DD || elementTn === $$7.DT)) {
              closeTn = elementTn;
          }

          if (closeTn) {
              p.openElements.generateImpliedEndTagsWithExclusion(closeTn);
              p.openElements.popUntilTagNamePopped(closeTn);
              break;
          }

          if (elementTn !== $$7.ADDRESS && elementTn !== $$7.DIV && elementTn !== $$7.P && p._isSpecialElement(element)) {
              break;
          }
      }

      if (p.openElements.hasInButtonScope($$7.P)) {
          p._closePElement();
      }

      p._insertElement(token, NS$4.HTML);
  }

  function plaintextStartTagInBody$1(p, token) {
      if (p.openElements.hasInButtonScope($$7.P)) {
          p._closePElement();
      }

      p._insertElement(token, NS$4.HTML);
      p.tokenizer.state = Tokenizer$5.MODE.PLAINTEXT;
  }

  function buttonStartTagInBody$1(p, token) {
      if (p.openElements.hasInScope($$7.BUTTON)) {
          p.openElements.generateImpliedEndTags();
          p.openElements.popUntilTagNamePopped($$7.BUTTON);
      }

      p._reconstructActiveFormattingElements();
      p._insertElement(token, NS$4.HTML);
      p.framesetOk = false;
  }

  function aStartTagInBody$1(p, token) {
      const activeElementEntry = p.activeFormattingElements.getElementEntryInScopeWithTagName($$7.A);

      if (activeElementEntry) {
          callAdoptionAgency$1(p, token);
          p.openElements.remove(activeElementEntry.element);
          p.activeFormattingElements.removeEntry(activeElementEntry);
      }

      p._reconstructActiveFormattingElements();
      p._insertElement(token, NS$4.HTML);
      p.activeFormattingElements.pushElement(p.openElements.current, token);
  }

  function bStartTagInBody$1(p, token) {
      p._reconstructActiveFormattingElements();
      p._insertElement(token, NS$4.HTML);
      p.activeFormattingElements.pushElement(p.openElements.current, token);
  }

  function nobrStartTagInBody$1(p, token) {
      p._reconstructActiveFormattingElements();

      if (p.openElements.hasInScope($$7.NOBR)) {
          callAdoptionAgency$1(p, token);
          p._reconstructActiveFormattingElements();
      }

      p._insertElement(token, NS$4.HTML);
      p.activeFormattingElements.pushElement(p.openElements.current, token);
  }

  function appletStartTagInBody$1(p, token) {
      p._reconstructActiveFormattingElements();
      p._insertElement(token, NS$4.HTML);
      p.activeFormattingElements.insertMarker();
      p.framesetOk = false;
  }

  function tableStartTagInBody$1(p, token) {
      if (
          p.treeAdapter.getDocumentMode(p.document) !== HTML$4.DOCUMENT_MODE.QUIRKS &&
          p.openElements.hasInButtonScope($$7.P)
      ) {
          p._closePElement();
      }

      p._insertElement(token, NS$4.HTML);
      p.framesetOk = false;
      p.insertionMode = IN_TABLE_MODE$1;
  }

  function areaStartTagInBody$1(p, token) {
      p._reconstructActiveFormattingElements();
      p._appendElement(token, NS$4.HTML);
      p.framesetOk = false;
      token.ackSelfClosing = true;
  }

  function inputStartTagInBody$1(p, token) {
      p._reconstructActiveFormattingElements();
      p._appendElement(token, NS$4.HTML);

      const inputType = Tokenizer$5.getTokenAttr(token, ATTRS$2.TYPE);

      if (!inputType || inputType.toLowerCase() !== HIDDEN_INPUT_TYPE$1) {
          p.framesetOk = false;
      }

      token.ackSelfClosing = true;
  }

  function paramStartTagInBody$1(p, token) {
      p._appendElement(token, NS$4.HTML);
      token.ackSelfClosing = true;
  }

  function hrStartTagInBody$1(p, token) {
      if (p.openElements.hasInButtonScope($$7.P)) {
          p._closePElement();
      }

      p._appendElement(token, NS$4.HTML);
      p.framesetOk = false;
      token.ackSelfClosing = true;
  }

  function imageStartTagInBody$1(p, token) {
      token.tagName = $$7.IMG;
      areaStartTagInBody$1(p, token);
  }

  function textareaStartTagInBody$1(p, token) {
      p._insertElement(token, NS$4.HTML);
      //NOTE: If the next token is a U+000A LINE FEED (LF) character token, then ignore that token and move
      //on to the next one. (Newlines at the start of textarea elements are ignored as an authoring convenience.)
      p.skipNextNewLine = true;
      p.tokenizer.state = Tokenizer$5.MODE.RCDATA;
      p.originalInsertionMode = p.insertionMode;
      p.framesetOk = false;
      p.insertionMode = TEXT_MODE$1;
  }

  function xmpStartTagInBody$1(p, token) {
      if (p.openElements.hasInButtonScope($$7.P)) {
          p._closePElement();
      }

      p._reconstructActiveFormattingElements();
      p.framesetOk = false;
      p._switchToTextParsing(token, Tokenizer$5.MODE.RAWTEXT);
  }

  function iframeStartTagInBody$1(p, token) {
      p.framesetOk = false;
      p._switchToTextParsing(token, Tokenizer$5.MODE.RAWTEXT);
  }

  //NOTE: here we assume that we always act as an user agent with enabled plugins, so we parse
  //<noembed> as a rawtext.
  function noembedStartTagInBody$1(p, token) {
      p._switchToTextParsing(token, Tokenizer$5.MODE.RAWTEXT);
  }

  function selectStartTagInBody$1(p, token) {
      p._reconstructActiveFormattingElements();
      p._insertElement(token, NS$4.HTML);
      p.framesetOk = false;

      if (
          p.insertionMode === IN_TABLE_MODE$1 ||
          p.insertionMode === IN_CAPTION_MODE$1 ||
          p.insertionMode === IN_TABLE_BODY_MODE$1 ||
          p.insertionMode === IN_ROW_MODE$1 ||
          p.insertionMode === IN_CELL_MODE$1
      ) {
          p.insertionMode = IN_SELECT_IN_TABLE_MODE$1;
      } else {
          p.insertionMode = IN_SELECT_MODE$1;
      }
  }

  function optgroupStartTagInBody$1(p, token) {
      if (p.openElements.currentTagName === $$7.OPTION) {
          p.openElements.pop();
      }

      p._reconstructActiveFormattingElements();
      p._insertElement(token, NS$4.HTML);
  }

  function rbStartTagInBody$1(p, token) {
      if (p.openElements.hasInScope($$7.RUBY)) {
          p.openElements.generateImpliedEndTags();
      }

      p._insertElement(token, NS$4.HTML);
  }

  function rtStartTagInBody$1(p, token) {
      if (p.openElements.hasInScope($$7.RUBY)) {
          p.openElements.generateImpliedEndTagsWithExclusion($$7.RTC);
      }

      p._insertElement(token, NS$4.HTML);
  }

  function menuStartTagInBody$1(p, token) {
      if (p.openElements.hasInButtonScope($$7.P)) {
          p._closePElement();
      }

      p._insertElement(token, NS$4.HTML);
  }

  function mathStartTagInBody$1(p, token) {
      p._reconstructActiveFormattingElements();

      foreignContent$2.adjustTokenMathMLAttrs(token);
      foreignContent$2.adjustTokenXMLAttrs(token);

      if (token.selfClosing) {
          p._appendElement(token, NS$4.MATHML);
      } else {
          p._insertElement(token, NS$4.MATHML);
      }

      token.ackSelfClosing = true;
  }

  function svgStartTagInBody$1(p, token) {
      p._reconstructActiveFormattingElements();

      foreignContent$2.adjustTokenSVGAttrs(token);
      foreignContent$2.adjustTokenXMLAttrs(token);

      if (token.selfClosing) {
          p._appendElement(token, NS$4.SVG);
      } else {
          p._insertElement(token, NS$4.SVG);
      }

      token.ackSelfClosing = true;
  }

  function genericStartTagInBody$1(p, token) {
      p._reconstructActiveFormattingElements();
      p._insertElement(token, NS$4.HTML);
  }

  //OPTIMIZATION: Integer comparisons are low-cost, so we can use very fast tag name length filters here.
  //It's faster than using dictionary.
  function startTagInBody$1(p, token) {
      const tn = token.tagName;

      switch (tn.length) {
          case 1:
              if (tn === $$7.I || tn === $$7.S || tn === $$7.B || tn === $$7.U) {
                  bStartTagInBody$1(p, token);
              } else if (tn === $$7.P) {
                  addressStartTagInBody$1(p, token);
              } else if (tn === $$7.A) {
                  aStartTagInBody$1(p, token);
              } else {
                  genericStartTagInBody$1(p, token);
              }

              break;

          case 2:
              if (tn === $$7.DL || tn === $$7.OL || tn === $$7.UL) {
                  addressStartTagInBody$1(p, token);
              } else if (tn === $$7.H1 || tn === $$7.H2 || tn === $$7.H3 || tn === $$7.H4 || tn === $$7.H5 || tn === $$7.H6) {
                  numberedHeaderStartTagInBody$1(p, token);
              } else if (tn === $$7.LI || tn === $$7.DD || tn === $$7.DT) {
                  listItemStartTagInBody$1(p, token);
              } else if (tn === $$7.EM || tn === $$7.TT) {
                  bStartTagInBody$1(p, token);
              } else if (tn === $$7.BR) {
                  areaStartTagInBody$1(p, token);
              } else if (tn === $$7.HR) {
                  hrStartTagInBody$1(p, token);
              } else if (tn === $$7.RB) {
                  rbStartTagInBody$1(p, token);
              } else if (tn === $$7.RT || tn === $$7.RP) {
                  rtStartTagInBody$1(p, token);
              } else if (tn !== $$7.TH && tn !== $$7.TD && tn !== $$7.TR) {
                  genericStartTagInBody$1(p, token);
              }

              break;

          case 3:
              if (tn === $$7.DIV || tn === $$7.DIR || tn === $$7.NAV) {
                  addressStartTagInBody$1(p, token);
              } else if (tn === $$7.PRE) {
                  preStartTagInBody$1(p, token);
              } else if (tn === $$7.BIG) {
                  bStartTagInBody$1(p, token);
              } else if (tn === $$7.IMG || tn === $$7.WBR) {
                  areaStartTagInBody$1(p, token);
              } else if (tn === $$7.XMP) {
                  xmpStartTagInBody$1(p, token);
              } else if (tn === $$7.SVG) {
                  svgStartTagInBody$1(p, token);
              } else if (tn === $$7.RTC) {
                  rbStartTagInBody$1(p, token);
              } else if (tn !== $$7.COL) {
                  genericStartTagInBody$1(p, token);
              }

              break;

          case 4:
              if (tn === $$7.HTML) {
                  htmlStartTagInBody$1(p, token);
              } else if (tn === $$7.BASE || tn === $$7.LINK || tn === $$7.META) {
                  startTagInHead$1(p, token);
              } else if (tn === $$7.BODY) {
                  bodyStartTagInBody$1(p, token);
              } else if (tn === $$7.MAIN || tn === $$7.MENU) {
                  addressStartTagInBody$1(p, token);
              } else if (tn === $$7.FORM) {
                  formStartTagInBody$1(p, token);
              } else if (tn === $$7.CODE || tn === $$7.FONT) {
                  bStartTagInBody$1(p, token);
              } else if (tn === $$7.NOBR) {
                  nobrStartTagInBody$1(p, token);
              } else if (tn === $$7.AREA) {
                  areaStartTagInBody$1(p, token);
              } else if (tn === $$7.MATH) {
                  mathStartTagInBody$1(p, token);
              } else if (tn === $$7.MENU) {
                  menuStartTagInBody$1(p, token);
              } else if (tn !== $$7.HEAD) {
                  genericStartTagInBody$1(p, token);
              }

              break;

          case 5:
              if (tn === $$7.STYLE || tn === $$7.TITLE) {
                  startTagInHead$1(p, token);
              } else if (tn === $$7.ASIDE) {
                  addressStartTagInBody$1(p, token);
              } else if (tn === $$7.SMALL) {
                  bStartTagInBody$1(p, token);
              } else if (tn === $$7.TABLE) {
                  tableStartTagInBody$1(p, token);
              } else if (tn === $$7.EMBED) {
                  areaStartTagInBody$1(p, token);
              } else if (tn === $$7.INPUT) {
                  inputStartTagInBody$1(p, token);
              } else if (tn === $$7.PARAM || tn === $$7.TRACK) {
                  paramStartTagInBody$1(p, token);
              } else if (tn === $$7.IMAGE) {
                  imageStartTagInBody$1(p, token);
              } else if (tn !== $$7.FRAME && tn !== $$7.TBODY && tn !== $$7.TFOOT && tn !== $$7.THEAD) {
                  genericStartTagInBody$1(p, token);
              }

              break;

          case 6:
              if (tn === $$7.SCRIPT) {
                  startTagInHead$1(p, token);
              } else if (
                  tn === $$7.CENTER ||
                  tn === $$7.FIGURE ||
                  tn === $$7.FOOTER ||
                  tn === $$7.HEADER ||
                  tn === $$7.HGROUP ||
                  tn === $$7.DIALOG
              ) {
                  addressStartTagInBody$1(p, token);
              } else if (tn === $$7.BUTTON) {
                  buttonStartTagInBody$1(p, token);
              } else if (tn === $$7.STRIKE || tn === $$7.STRONG) {
                  bStartTagInBody$1(p, token);
              } else if (tn === $$7.APPLET || tn === $$7.OBJECT) {
                  appletStartTagInBody$1(p, token);
              } else if (tn === $$7.KEYGEN) {
                  areaStartTagInBody$1(p, token);
              } else if (tn === $$7.SOURCE) {
                  paramStartTagInBody$1(p, token);
              } else if (tn === $$7.IFRAME) {
                  iframeStartTagInBody$1(p, token);
              } else if (tn === $$7.SELECT) {
                  selectStartTagInBody$1(p, token);
              } else if (tn === $$7.OPTION) {
                  optgroupStartTagInBody$1(p, token);
              } else {
                  genericStartTagInBody$1(p, token);
              }

              break;

          case 7:
              if (tn === $$7.BGSOUND) {
                  startTagInHead$1(p, token);
              } else if (
                  tn === $$7.DETAILS ||
                  tn === $$7.ADDRESS ||
                  tn === $$7.ARTICLE ||
                  tn === $$7.SECTION ||
                  tn === $$7.SUMMARY
              ) {
                  addressStartTagInBody$1(p, token);
              } else if (tn === $$7.LISTING) {
                  preStartTagInBody$1(p, token);
              } else if (tn === $$7.MARQUEE) {
                  appletStartTagInBody$1(p, token);
              } else if (tn === $$7.NOEMBED) {
                  noembedStartTagInBody$1(p, token);
              } else if (tn !== $$7.CAPTION) {
                  genericStartTagInBody$1(p, token);
              }

              break;

          case 8:
              if (tn === $$7.BASEFONT) {
                  startTagInHead$1(p, token);
              } else if (tn === $$7.FRAMESET) {
                  framesetStartTagInBody$1(p, token);
              } else if (tn === $$7.FIELDSET) {
                  addressStartTagInBody$1(p, token);
              } else if (tn === $$7.TEXTAREA) {
                  textareaStartTagInBody$1(p, token);
              } else if (tn === $$7.TEMPLATE) {
                  startTagInHead$1(p, token);
              } else if (tn === $$7.NOSCRIPT) {
                  if (p.options.scriptingEnabled) {
                      noembedStartTagInBody$1(p, token);
                  } else {
                      genericStartTagInBody$1(p, token);
                  }
              } else if (tn === $$7.OPTGROUP) {
                  optgroupStartTagInBody$1(p, token);
              } else if (tn !== $$7.COLGROUP) {
                  genericStartTagInBody$1(p, token);
              }

              break;

          case 9:
              if (tn === $$7.PLAINTEXT) {
                  plaintextStartTagInBody$1(p, token);
              } else {
                  genericStartTagInBody$1(p, token);
              }

              break;

          case 10:
              if (tn === $$7.BLOCKQUOTE || tn === $$7.FIGCAPTION) {
                  addressStartTagInBody$1(p, token);
              } else {
                  genericStartTagInBody$1(p, token);
              }

              break;

          default:
              genericStartTagInBody$1(p, token);
      }
  }

  function bodyEndTagInBody$1(p) {
      if (p.openElements.hasInScope($$7.BODY)) {
          p.insertionMode = AFTER_BODY_MODE$1;
      }
  }

  function htmlEndTagInBody$1(p, token) {
      if (p.openElements.hasInScope($$7.BODY)) {
          p.insertionMode = AFTER_BODY_MODE$1;
          p._processToken(token);
      }
  }

  function addressEndTagInBody$1(p, token) {
      const tn = token.tagName;

      if (p.openElements.hasInScope(tn)) {
          p.openElements.generateImpliedEndTags();
          p.openElements.popUntilTagNamePopped(tn);
      }
  }

  function formEndTagInBody$1(p) {
      const inTemplate = p.openElements.tmplCount > 0;
      const formElement = p.formElement;

      if (!inTemplate) {
          p.formElement = null;
      }

      if ((formElement || inTemplate) && p.openElements.hasInScope($$7.FORM)) {
          p.openElements.generateImpliedEndTags();

          if (inTemplate) {
              p.openElements.popUntilTagNamePopped($$7.FORM);
          } else {
              p.openElements.remove(formElement);
          }
      }
  }

  function pEndTagInBody$1(p) {
      if (!p.openElements.hasInButtonScope($$7.P)) {
          p._insertFakeElement($$7.P);
      }

      p._closePElement();
  }

  function liEndTagInBody$1(p) {
      if (p.openElements.hasInListItemScope($$7.LI)) {
          p.openElements.generateImpliedEndTagsWithExclusion($$7.LI);
          p.openElements.popUntilTagNamePopped($$7.LI);
      }
  }

  function ddEndTagInBody$1(p, token) {
      const tn = token.tagName;

      if (p.openElements.hasInScope(tn)) {
          p.openElements.generateImpliedEndTagsWithExclusion(tn);
          p.openElements.popUntilTagNamePopped(tn);
      }
  }

  function numberedHeaderEndTagInBody$1(p) {
      if (p.openElements.hasNumberedHeaderInScope()) {
          p.openElements.generateImpliedEndTags();
          p.openElements.popUntilNumberedHeaderPopped();
      }
  }

  function appletEndTagInBody$1(p, token) {
      const tn = token.tagName;

      if (p.openElements.hasInScope(tn)) {
          p.openElements.generateImpliedEndTags();
          p.openElements.popUntilTagNamePopped(tn);
          p.activeFormattingElements.clearToLastMarker();
      }
  }

  function brEndTagInBody$1(p) {
      p._reconstructActiveFormattingElements();
      p._insertFakeElement($$7.BR);
      p.openElements.pop();
      p.framesetOk = false;
  }

  function genericEndTagInBody$1(p, token) {
      const tn = token.tagName;

      for (let i = p.openElements.stackTop; i > 0; i--) {
          const element = p.openElements.items[i];

          if (p.treeAdapter.getTagName(element) === tn) {
              p.openElements.generateImpliedEndTagsWithExclusion(tn);
              p.openElements.popUntilElementPopped(element);
              break;
          }

          if (p._isSpecialElement(element)) {
              break;
          }
      }
  }

  //OPTIMIZATION: Integer comparisons are low-cost, so we can use very fast tag name length filters here.
  //It's faster than using dictionary.
  function endTagInBody$1(p, token) {
      const tn = token.tagName;

      switch (tn.length) {
          case 1:
              if (tn === $$7.A || tn === $$7.B || tn === $$7.I || tn === $$7.S || tn === $$7.U) {
                  callAdoptionAgency$1(p, token);
              } else if (tn === $$7.P) {
                  pEndTagInBody$1(p);
              } else {
                  genericEndTagInBody$1(p, token);
              }

              break;

          case 2:
              if (tn === $$7.DL || tn === $$7.UL || tn === $$7.OL) {
                  addressEndTagInBody$1(p, token);
              } else if (tn === $$7.LI) {
                  liEndTagInBody$1(p);
              } else if (tn === $$7.DD || tn === $$7.DT) {
                  ddEndTagInBody$1(p, token);
              } else if (tn === $$7.H1 || tn === $$7.H2 || tn === $$7.H3 || tn === $$7.H4 || tn === $$7.H5 || tn === $$7.H6) {
                  numberedHeaderEndTagInBody$1(p);
              } else if (tn === $$7.BR) {
                  brEndTagInBody$1(p);
              } else if (tn === $$7.EM || tn === $$7.TT) {
                  callAdoptionAgency$1(p, token);
              } else {
                  genericEndTagInBody$1(p, token);
              }

              break;

          case 3:
              if (tn === $$7.BIG) {
                  callAdoptionAgency$1(p, token);
              } else if (tn === $$7.DIR || tn === $$7.DIV || tn === $$7.NAV || tn === $$7.PRE) {
                  addressEndTagInBody$1(p, token);
              } else {
                  genericEndTagInBody$1(p, token);
              }

              break;

          case 4:
              if (tn === $$7.BODY) {
                  bodyEndTagInBody$1(p);
              } else if (tn === $$7.HTML) {
                  htmlEndTagInBody$1(p, token);
              } else if (tn === $$7.FORM) {
                  formEndTagInBody$1(p);
              } else if (tn === $$7.CODE || tn === $$7.FONT || tn === $$7.NOBR) {
                  callAdoptionAgency$1(p, token);
              } else if (tn === $$7.MAIN || tn === $$7.MENU) {
                  addressEndTagInBody$1(p, token);
              } else {
                  genericEndTagInBody$1(p, token);
              }

              break;

          case 5:
              if (tn === $$7.ASIDE) {
                  addressEndTagInBody$1(p, token);
              } else if (tn === $$7.SMALL) {
                  callAdoptionAgency$1(p, token);
              } else {
                  genericEndTagInBody$1(p, token);
              }

              break;

          case 6:
              if (
                  tn === $$7.CENTER ||
                  tn === $$7.FIGURE ||
                  tn === $$7.FOOTER ||
                  tn === $$7.HEADER ||
                  tn === $$7.HGROUP ||
                  tn === $$7.DIALOG
              ) {
                  addressEndTagInBody$1(p, token);
              } else if (tn === $$7.APPLET || tn === $$7.OBJECT) {
                  appletEndTagInBody$1(p, token);
              } else if (tn === $$7.STRIKE || tn === $$7.STRONG) {
                  callAdoptionAgency$1(p, token);
              } else {
                  genericEndTagInBody$1(p, token);
              }

              break;

          case 7:
              if (
                  tn === $$7.ADDRESS ||
                  tn === $$7.ARTICLE ||
                  tn === $$7.DETAILS ||
                  tn === $$7.SECTION ||
                  tn === $$7.SUMMARY ||
                  tn === $$7.LISTING
              ) {
                  addressEndTagInBody$1(p, token);
              } else if (tn === $$7.MARQUEE) {
                  appletEndTagInBody$1(p, token);
              } else {
                  genericEndTagInBody$1(p, token);
              }

              break;

          case 8:
              if (tn === $$7.FIELDSET) {
                  addressEndTagInBody$1(p, token);
              } else if (tn === $$7.TEMPLATE) {
                  endTagInHead$1(p, token);
              } else {
                  genericEndTagInBody$1(p, token);
              }

              break;

          case 10:
              if (tn === $$7.BLOCKQUOTE || tn === $$7.FIGCAPTION) {
                  addressEndTagInBody$1(p, token);
              } else {
                  genericEndTagInBody$1(p, token);
              }

              break;

          default:
              genericEndTagInBody$1(p, token);
      }
  }

  function eofInBody$1(p, token) {
      if (p.tmplInsertionModeStackTop > -1) {
          eofInTemplate$1(p, token);
      } else {
          p.stopped = true;
      }
  }

  // The "text" insertion mode
  //------------------------------------------------------------------
  function endTagInText$1(p, token) {
      if (token.tagName === $$7.SCRIPT) {
          p.pendingScript = p.openElements.current;
      }

      p.openElements.pop();
      p.insertionMode = p.originalInsertionMode;
  }

  function eofInText$1(p, token) {
      p._err(ERR$3.eofInElementThatCanContainOnlyText);
      p.openElements.pop();
      p.insertionMode = p.originalInsertionMode;
      p._processToken(token);
  }

  // The "in table" insertion mode
  //------------------------------------------------------------------
  function characterInTable$1(p, token) {
      const curTn = p.openElements.currentTagName;

      if (curTn === $$7.TABLE || curTn === $$7.TBODY || curTn === $$7.TFOOT || curTn === $$7.THEAD || curTn === $$7.TR) {
          p.pendingCharacterTokens = [];
          p.hasNonWhitespacePendingCharacterToken = false;
          p.originalInsertionMode = p.insertionMode;
          p.insertionMode = IN_TABLE_TEXT_MODE$1;
          p._processToken(token);
      } else {
          tokenInTable$1(p, token);
      }
  }

  function captionStartTagInTable$1(p, token) {
      p.openElements.clearBackToTableContext();
      p.activeFormattingElements.insertMarker();
      p._insertElement(token, NS$4.HTML);
      p.insertionMode = IN_CAPTION_MODE$1;
  }

  function colgroupStartTagInTable$1(p, token) {
      p.openElements.clearBackToTableContext();
      p._insertElement(token, NS$4.HTML);
      p.insertionMode = IN_COLUMN_GROUP_MODE$1;
  }

  function colStartTagInTable$1(p, token) {
      p.openElements.clearBackToTableContext();
      p._insertFakeElement($$7.COLGROUP);
      p.insertionMode = IN_COLUMN_GROUP_MODE$1;
      p._processToken(token);
  }

  function tbodyStartTagInTable$1(p, token) {
      p.openElements.clearBackToTableContext();
      p._insertElement(token, NS$4.HTML);
      p.insertionMode = IN_TABLE_BODY_MODE$1;
  }

  function tdStartTagInTable$1(p, token) {
      p.openElements.clearBackToTableContext();
      p._insertFakeElement($$7.TBODY);
      p.insertionMode = IN_TABLE_BODY_MODE$1;
      p._processToken(token);
  }

  function tableStartTagInTable$1(p, token) {
      if (p.openElements.hasInTableScope($$7.TABLE)) {
          p.openElements.popUntilTagNamePopped($$7.TABLE);
          p._resetInsertionMode();
          p._processToken(token);
      }
  }

  function inputStartTagInTable$1(p, token) {
      const inputType = Tokenizer$5.getTokenAttr(token, ATTRS$2.TYPE);

      if (inputType && inputType.toLowerCase() === HIDDEN_INPUT_TYPE$1) {
          p._appendElement(token, NS$4.HTML);
      } else {
          tokenInTable$1(p, token);
      }

      token.ackSelfClosing = true;
  }

  function formStartTagInTable$1(p, token) {
      if (!p.formElement && p.openElements.tmplCount === 0) {
          p._insertElement(token, NS$4.HTML);
          p.formElement = p.openElements.current;
          p.openElements.pop();
      }
  }

  function startTagInTable$1(p, token) {
      const tn = token.tagName;

      switch (tn.length) {
          case 2:
              if (tn === $$7.TD || tn === $$7.TH || tn === $$7.TR) {
                  tdStartTagInTable$1(p, token);
              } else {
                  tokenInTable$1(p, token);
              }

              break;

          case 3:
              if (tn === $$7.COL) {
                  colStartTagInTable$1(p, token);
              } else {
                  tokenInTable$1(p, token);
              }

              break;

          case 4:
              if (tn === $$7.FORM) {
                  formStartTagInTable$1(p, token);
              } else {
                  tokenInTable$1(p, token);
              }

              break;

          case 5:
              if (tn === $$7.TABLE) {
                  tableStartTagInTable$1(p, token);
              } else if (tn === $$7.STYLE) {
                  startTagInHead$1(p, token);
              } else if (tn === $$7.TBODY || tn === $$7.TFOOT || tn === $$7.THEAD) {
                  tbodyStartTagInTable$1(p, token);
              } else if (tn === $$7.INPUT) {
                  inputStartTagInTable$1(p, token);
              } else {
                  tokenInTable$1(p, token);
              }

              break;

          case 6:
              if (tn === $$7.SCRIPT) {
                  startTagInHead$1(p, token);
              } else {
                  tokenInTable$1(p, token);
              }

              break;

          case 7:
              if (tn === $$7.CAPTION) {
                  captionStartTagInTable$1(p, token);
              } else {
                  tokenInTable$1(p, token);
              }

              break;

          case 8:
              if (tn === $$7.COLGROUP) {
                  colgroupStartTagInTable$1(p, token);
              } else if (tn === $$7.TEMPLATE) {
                  startTagInHead$1(p, token);
              } else {
                  tokenInTable$1(p, token);
              }

              break;

          default:
              tokenInTable$1(p, token);
      }
  }

  function endTagInTable$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.TABLE) {
          if (p.openElements.hasInTableScope($$7.TABLE)) {
              p.openElements.popUntilTagNamePopped($$7.TABLE);
              p._resetInsertionMode();
          }
      } else if (tn === $$7.TEMPLATE) {
          endTagInHead$1(p, token);
      } else if (
          tn !== $$7.BODY &&
          tn !== $$7.CAPTION &&
          tn !== $$7.COL &&
          tn !== $$7.COLGROUP &&
          tn !== $$7.HTML &&
          tn !== $$7.TBODY &&
          tn !== $$7.TD &&
          tn !== $$7.TFOOT &&
          tn !== $$7.TH &&
          tn !== $$7.THEAD &&
          tn !== $$7.TR
      ) {
          tokenInTable$1(p, token);
      }
  }

  function tokenInTable$1(p, token) {
      const savedFosterParentingState = p.fosterParentingEnabled;

      p.fosterParentingEnabled = true;
      p._processTokenInBodyMode(token);
      p.fosterParentingEnabled = savedFosterParentingState;
  }

  // The "in table text" insertion mode
  //------------------------------------------------------------------
  function whitespaceCharacterInTableText$1(p, token) {
      p.pendingCharacterTokens.push(token);
  }

  function characterInTableText$1(p, token) {
      p.pendingCharacterTokens.push(token);
      p.hasNonWhitespacePendingCharacterToken = true;
  }

  function tokenInTableText$1(p, token) {
      let i = 0;

      if (p.hasNonWhitespacePendingCharacterToken) {
          for (; i < p.pendingCharacterTokens.length; i++) {
              tokenInTable$1(p, p.pendingCharacterTokens[i]);
          }
      } else {
          for (; i < p.pendingCharacterTokens.length; i++) {
              p._insertCharacters(p.pendingCharacterTokens[i]);
          }
      }

      p.insertionMode = p.originalInsertionMode;
      p._processToken(token);
  }

  // The "in caption" insertion mode
  //------------------------------------------------------------------
  function startTagInCaption$1(p, token) {
      const tn = token.tagName;

      if (
          tn === $$7.CAPTION ||
          tn === $$7.COL ||
          tn === $$7.COLGROUP ||
          tn === $$7.TBODY ||
          tn === $$7.TD ||
          tn === $$7.TFOOT ||
          tn === $$7.TH ||
          tn === $$7.THEAD ||
          tn === $$7.TR
      ) {
          if (p.openElements.hasInTableScope($$7.CAPTION)) {
              p.openElements.generateImpliedEndTags();
              p.openElements.popUntilTagNamePopped($$7.CAPTION);
              p.activeFormattingElements.clearToLastMarker();
              p.insertionMode = IN_TABLE_MODE$1;
              p._processToken(token);
          }
      } else {
          startTagInBody$1(p, token);
      }
  }

  function endTagInCaption$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.CAPTION || tn === $$7.TABLE) {
          if (p.openElements.hasInTableScope($$7.CAPTION)) {
              p.openElements.generateImpliedEndTags();
              p.openElements.popUntilTagNamePopped($$7.CAPTION);
              p.activeFormattingElements.clearToLastMarker();
              p.insertionMode = IN_TABLE_MODE$1;

              if (tn === $$7.TABLE) {
                  p._processToken(token);
              }
          }
      } else if (
          tn !== $$7.BODY &&
          tn !== $$7.COL &&
          tn !== $$7.COLGROUP &&
          tn !== $$7.HTML &&
          tn !== $$7.TBODY &&
          tn !== $$7.TD &&
          tn !== $$7.TFOOT &&
          tn !== $$7.TH &&
          tn !== $$7.THEAD &&
          tn !== $$7.TR
      ) {
          endTagInBody$1(p, token);
      }
  }

  // The "in column group" insertion mode
  //------------------------------------------------------------------
  function startTagInColumnGroup$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.HTML) {
          startTagInBody$1(p, token);
      } else if (tn === $$7.COL) {
          p._appendElement(token, NS$4.HTML);
          token.ackSelfClosing = true;
      } else if (tn === $$7.TEMPLATE) {
          startTagInHead$1(p, token);
      } else {
          tokenInColumnGroup$1(p, token);
      }
  }

  function endTagInColumnGroup$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.COLGROUP) {
          if (p.openElements.currentTagName === $$7.COLGROUP) {
              p.openElements.pop();
              p.insertionMode = IN_TABLE_MODE$1;
          }
      } else if (tn === $$7.TEMPLATE) {
          endTagInHead$1(p, token);
      } else if (tn !== $$7.COL) {
          tokenInColumnGroup$1(p, token);
      }
  }

  function tokenInColumnGroup$1(p, token) {
      if (p.openElements.currentTagName === $$7.COLGROUP) {
          p.openElements.pop();
          p.insertionMode = IN_TABLE_MODE$1;
          p._processToken(token);
      }
  }

  // The "in table body" insertion mode
  //------------------------------------------------------------------
  function startTagInTableBody$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.TR) {
          p.openElements.clearBackToTableBodyContext();
          p._insertElement(token, NS$4.HTML);
          p.insertionMode = IN_ROW_MODE$1;
      } else if (tn === $$7.TH || tn === $$7.TD) {
          p.openElements.clearBackToTableBodyContext();
          p._insertFakeElement($$7.TR);
          p.insertionMode = IN_ROW_MODE$1;
          p._processToken(token);
      } else if (
          tn === $$7.CAPTION ||
          tn === $$7.COL ||
          tn === $$7.COLGROUP ||
          tn === $$7.TBODY ||
          tn === $$7.TFOOT ||
          tn === $$7.THEAD
      ) {
          if (p.openElements.hasTableBodyContextInTableScope()) {
              p.openElements.clearBackToTableBodyContext();
              p.openElements.pop();
              p.insertionMode = IN_TABLE_MODE$1;
              p._processToken(token);
          }
      } else {
          startTagInTable$1(p, token);
      }
  }

  function endTagInTableBody$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.TBODY || tn === $$7.TFOOT || tn === $$7.THEAD) {
          if (p.openElements.hasInTableScope(tn)) {
              p.openElements.clearBackToTableBodyContext();
              p.openElements.pop();
              p.insertionMode = IN_TABLE_MODE$1;
          }
      } else if (tn === $$7.TABLE) {
          if (p.openElements.hasTableBodyContextInTableScope()) {
              p.openElements.clearBackToTableBodyContext();
              p.openElements.pop();
              p.insertionMode = IN_TABLE_MODE$1;
              p._processToken(token);
          }
      } else if (
          (tn !== $$7.BODY && tn !== $$7.CAPTION && tn !== $$7.COL && tn !== $$7.COLGROUP) ||
          (tn !== $$7.HTML && tn !== $$7.TD && tn !== $$7.TH && tn !== $$7.TR)
      ) {
          endTagInTable$1(p, token);
      }
  }

  // The "in row" insertion mode
  //------------------------------------------------------------------
  function startTagInRow$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.TH || tn === $$7.TD) {
          p.openElements.clearBackToTableRowContext();
          p._insertElement(token, NS$4.HTML);
          p.insertionMode = IN_CELL_MODE$1;
          p.activeFormattingElements.insertMarker();
      } else if (
          tn === $$7.CAPTION ||
          tn === $$7.COL ||
          tn === $$7.COLGROUP ||
          tn === $$7.TBODY ||
          tn === $$7.TFOOT ||
          tn === $$7.THEAD ||
          tn === $$7.TR
      ) {
          if (p.openElements.hasInTableScope($$7.TR)) {
              p.openElements.clearBackToTableRowContext();
              p.openElements.pop();
              p.insertionMode = IN_TABLE_BODY_MODE$1;
              p._processToken(token);
          }
      } else {
          startTagInTable$1(p, token);
      }
  }

  function endTagInRow$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.TR) {
          if (p.openElements.hasInTableScope($$7.TR)) {
              p.openElements.clearBackToTableRowContext();
              p.openElements.pop();
              p.insertionMode = IN_TABLE_BODY_MODE$1;
          }
      } else if (tn === $$7.TABLE) {
          if (p.openElements.hasInTableScope($$7.TR)) {
              p.openElements.clearBackToTableRowContext();
              p.openElements.pop();
              p.insertionMode = IN_TABLE_BODY_MODE$1;
              p._processToken(token);
          }
      } else if (tn === $$7.TBODY || tn === $$7.TFOOT || tn === $$7.THEAD) {
          if (p.openElements.hasInTableScope(tn) || p.openElements.hasInTableScope($$7.TR)) {
              p.openElements.clearBackToTableRowContext();
              p.openElements.pop();
              p.insertionMode = IN_TABLE_BODY_MODE$1;
              p._processToken(token);
          }
      } else if (
          (tn !== $$7.BODY && tn !== $$7.CAPTION && tn !== $$7.COL && tn !== $$7.COLGROUP) ||
          (tn !== $$7.HTML && tn !== $$7.TD && tn !== $$7.TH)
      ) {
          endTagInTable$1(p, token);
      }
  }

  // The "in cell" insertion mode
  //------------------------------------------------------------------
  function startTagInCell$1(p, token) {
      const tn = token.tagName;

      if (
          tn === $$7.CAPTION ||
          tn === $$7.COL ||
          tn === $$7.COLGROUP ||
          tn === $$7.TBODY ||
          tn === $$7.TD ||
          tn === $$7.TFOOT ||
          tn === $$7.TH ||
          tn === $$7.THEAD ||
          tn === $$7.TR
      ) {
          if (p.openElements.hasInTableScope($$7.TD) || p.openElements.hasInTableScope($$7.TH)) {
              p._closeTableCell();
              p._processToken(token);
          }
      } else {
          startTagInBody$1(p, token);
      }
  }

  function endTagInCell$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.TD || tn === $$7.TH) {
          if (p.openElements.hasInTableScope(tn)) {
              p.openElements.generateImpliedEndTags();
              p.openElements.popUntilTagNamePopped(tn);
              p.activeFormattingElements.clearToLastMarker();
              p.insertionMode = IN_ROW_MODE$1;
          }
      } else if (tn === $$7.TABLE || tn === $$7.TBODY || tn === $$7.TFOOT || tn === $$7.THEAD || tn === $$7.TR) {
          if (p.openElements.hasInTableScope(tn)) {
              p._closeTableCell();
              p._processToken(token);
          }
      } else if (tn !== $$7.BODY && tn !== $$7.CAPTION && tn !== $$7.COL && tn !== $$7.COLGROUP && tn !== $$7.HTML) {
          endTagInBody$1(p, token);
      }
  }

  // The "in select" insertion mode
  //------------------------------------------------------------------
  function startTagInSelect$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.HTML) {
          startTagInBody$1(p, token);
      } else if (tn === $$7.OPTION) {
          if (p.openElements.currentTagName === $$7.OPTION) {
              p.openElements.pop();
          }

          p._insertElement(token, NS$4.HTML);
      } else if (tn === $$7.OPTGROUP) {
          if (p.openElements.currentTagName === $$7.OPTION) {
              p.openElements.pop();
          }

          if (p.openElements.currentTagName === $$7.OPTGROUP) {
              p.openElements.pop();
          }

          p._insertElement(token, NS$4.HTML);
      } else if (tn === $$7.INPUT || tn === $$7.KEYGEN || tn === $$7.TEXTAREA || tn === $$7.SELECT) {
          if (p.openElements.hasInSelectScope($$7.SELECT)) {
              p.openElements.popUntilTagNamePopped($$7.SELECT);
              p._resetInsertionMode();

              if (tn !== $$7.SELECT) {
                  p._processToken(token);
              }
          }
      } else if (tn === $$7.SCRIPT || tn === $$7.TEMPLATE) {
          startTagInHead$1(p, token);
      }
  }

  function endTagInSelect$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.OPTGROUP) {
          const prevOpenElement = p.openElements.items[p.openElements.stackTop - 1];
          const prevOpenElementTn = prevOpenElement && p.treeAdapter.getTagName(prevOpenElement);

          if (p.openElements.currentTagName === $$7.OPTION && prevOpenElementTn === $$7.OPTGROUP) {
              p.openElements.pop();
          }

          if (p.openElements.currentTagName === $$7.OPTGROUP) {
              p.openElements.pop();
          }
      } else if (tn === $$7.OPTION) {
          if (p.openElements.currentTagName === $$7.OPTION) {
              p.openElements.pop();
          }
      } else if (tn === $$7.SELECT && p.openElements.hasInSelectScope($$7.SELECT)) {
          p.openElements.popUntilTagNamePopped($$7.SELECT);
          p._resetInsertionMode();
      } else if (tn === $$7.TEMPLATE) {
          endTagInHead$1(p, token);
      }
  }

  //12.2.5.4.17 The "in select in table" insertion mode
  //------------------------------------------------------------------
  function startTagInSelectInTable$1(p, token) {
      const tn = token.tagName;

      if (
          tn === $$7.CAPTION ||
          tn === $$7.TABLE ||
          tn === $$7.TBODY ||
          tn === $$7.TFOOT ||
          tn === $$7.THEAD ||
          tn === $$7.TR ||
          tn === $$7.TD ||
          tn === $$7.TH
      ) {
          p.openElements.popUntilTagNamePopped($$7.SELECT);
          p._resetInsertionMode();
          p._processToken(token);
      } else {
          startTagInSelect$1(p, token);
      }
  }

  function endTagInSelectInTable$1(p, token) {
      const tn = token.tagName;

      if (
          tn === $$7.CAPTION ||
          tn === $$7.TABLE ||
          tn === $$7.TBODY ||
          tn === $$7.TFOOT ||
          tn === $$7.THEAD ||
          tn === $$7.TR ||
          tn === $$7.TD ||
          tn === $$7.TH
      ) {
          if (p.openElements.hasInTableScope(tn)) {
              p.openElements.popUntilTagNamePopped($$7.SELECT);
              p._resetInsertionMode();
              p._processToken(token);
          }
      } else {
          endTagInSelect$1(p, token);
      }
  }

  // The "in template" insertion mode
  //------------------------------------------------------------------
  function startTagInTemplate$1(p, token) {
      const tn = token.tagName;

      if (
          tn === $$7.BASE ||
          tn === $$7.BASEFONT ||
          tn === $$7.BGSOUND ||
          tn === $$7.LINK ||
          tn === $$7.META ||
          tn === $$7.NOFRAMES ||
          tn === $$7.SCRIPT ||
          tn === $$7.STYLE ||
          tn === $$7.TEMPLATE ||
          tn === $$7.TITLE
      ) {
          startTagInHead$1(p, token);
      } else {
          const newInsertionMode = TEMPLATE_INSERTION_MODE_SWITCH_MAP$1[tn] || IN_BODY_MODE$1;

          p._popTmplInsertionMode();
          p._pushTmplInsertionMode(newInsertionMode);
          p.insertionMode = newInsertionMode;
          p._processToken(token);
      }
  }

  function endTagInTemplate$1(p, token) {
      if (token.tagName === $$7.TEMPLATE) {
          endTagInHead$1(p, token);
      }
  }

  function eofInTemplate$1(p, token) {
      if (p.openElements.tmplCount > 0) {
          p.openElements.popUntilTagNamePopped($$7.TEMPLATE);
          p.activeFormattingElements.clearToLastMarker();
          p._popTmplInsertionMode();
          p._resetInsertionMode();
          p._processToken(token);
      } else {
          p.stopped = true;
      }
  }

  // The "after body" insertion mode
  //------------------------------------------------------------------
  function startTagAfterBody$1(p, token) {
      if (token.tagName === $$7.HTML) {
          startTagInBody$1(p, token);
      } else {
          tokenAfterBody$1(p, token);
      }
  }

  function endTagAfterBody$1(p, token) {
      if (token.tagName === $$7.HTML) {
          if (!p.fragmentContext) {
              p.insertionMode = AFTER_AFTER_BODY_MODE$1;
          }
      } else {
          tokenAfterBody$1(p, token);
      }
  }

  function tokenAfterBody$1(p, token) {
      p.insertionMode = IN_BODY_MODE$1;
      p._processToken(token);
  }

  // The "in frameset" insertion mode
  //------------------------------------------------------------------
  function startTagInFrameset$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.HTML) {
          startTagInBody$1(p, token);
      } else if (tn === $$7.FRAMESET) {
          p._insertElement(token, NS$4.HTML);
      } else if (tn === $$7.FRAME) {
          p._appendElement(token, NS$4.HTML);
          token.ackSelfClosing = true;
      } else if (tn === $$7.NOFRAMES) {
          startTagInHead$1(p, token);
      }
  }

  function endTagInFrameset$1(p, token) {
      if (token.tagName === $$7.FRAMESET && !p.openElements.isRootHtmlElementCurrent()) {
          p.openElements.pop();

          if (!p.fragmentContext && p.openElements.currentTagName !== $$7.FRAMESET) {
              p.insertionMode = AFTER_FRAMESET_MODE$1;
          }
      }
  }

  // The "after frameset" insertion mode
  //------------------------------------------------------------------
  function startTagAfterFrameset$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.HTML) {
          startTagInBody$1(p, token);
      } else if (tn === $$7.NOFRAMES) {
          startTagInHead$1(p, token);
      }
  }

  function endTagAfterFrameset$1(p, token) {
      if (token.tagName === $$7.HTML) {
          p.insertionMode = AFTER_AFTER_FRAMESET_MODE$1;
      }
  }

  // The "after after body" insertion mode
  //------------------------------------------------------------------
  function startTagAfterAfterBody$1(p, token) {
      if (token.tagName === $$7.HTML) {
          startTagInBody$1(p, token);
      } else {
          tokenAfterAfterBody$1(p, token);
      }
  }

  function tokenAfterAfterBody$1(p, token) {
      p.insertionMode = IN_BODY_MODE$1;
      p._processToken(token);
  }

  // The "after after frameset" insertion mode
  //------------------------------------------------------------------
  function startTagAfterAfterFrameset$1(p, token) {
      const tn = token.tagName;

      if (tn === $$7.HTML) {
          startTagInBody$1(p, token);
      } else if (tn === $$7.NOFRAMES) {
          startTagInHead$1(p, token);
      }
  }

  // The rules for parsing tokens in foreign content
  //------------------------------------------------------------------
  function nullCharacterInForeignContent$1(p, token) {
      token.chars = unicode$4.REPLACEMENT_CHARACTER;
      p._insertCharacters(token);
  }

  function characterInForeignContent$1(p, token) {
      p._insertCharacters(token);
      p.framesetOk = false;
  }

  function startTagInForeignContent$1(p, token) {
      if (foreignContent$2.causesExit(token) && !p.fragmentContext) {
          while (
              p.treeAdapter.getNamespaceURI(p.openElements.current) !== NS$4.HTML &&
              !p._isIntegrationPoint(p.openElements.current)
          ) {
              p.openElements.pop();
          }

          p._processToken(token);
      } else {
          const current = p._getAdjustedCurrentElement();
          const currentNs = p.treeAdapter.getNamespaceURI(current);

          if (currentNs === NS$4.MATHML) {
              foreignContent$2.adjustTokenMathMLAttrs(token);
          } else if (currentNs === NS$4.SVG) {
              foreignContent$2.adjustTokenSVGTagName(token);
              foreignContent$2.adjustTokenSVGAttrs(token);
          }

          foreignContent$2.adjustTokenXMLAttrs(token);

          if (token.selfClosing) {
              p._appendElement(token, currentNs);
          } else {
              p._insertElement(token, currentNs);
          }

          token.ackSelfClosing = true;
      }
  }

  function endTagInForeignContent$1(p, token) {
      for (let i = p.openElements.stackTop; i > 0; i--) {
          const element = p.openElements.items[i];

          if (p.treeAdapter.getNamespaceURI(element) === NS$4.HTML) {
              p._processToken(token);
              break;
          }

          if (p.treeAdapter.getTagName(element).toLowerCase() === token.tagName) {
              p.openElements.popUntilElementPopped(element);
              break;
          }
      }
  }

  var Parser5 = parser$3;

  /**
   * @typedef {import('./info.js').Info} Info
   * @typedef {Record<string, Info>} Properties
   * @typedef {Record<string, string>} Normal
   */

  class Schema$6 {
    /**
     * @constructor
     * @param {Properties} property
     * @param {Normal} normal
     * @param {string} [space]
     */
    constructor(property, normal, space) {
      this.property = property;
      this.normal = normal;
      if (space) {
        this.space = space;
      }
    }
  }

  /** @type {Properties} */
  Schema$6.prototype.property = {};
  /** @type {Normal} */
  Schema$6.prototype.normal = {};
  /** @type {string|null} */
  Schema$6.prototype.space = null;

  /**
   * @typedef {import('./schema.js').Properties} Properties
   * @typedef {import('./schema.js').Normal} Normal
   */

  /**
   * @param {Schema[]} definitions
   * @param {string} [space]
   * @returns {Schema}
   */
  function merge$6(definitions, space) {
    /** @type {Properties} */
    const property = {};
    /** @type {Normal} */
    const normal = {};
    let index = -1;

    while (++index < definitions.length) {
      Object.assign(property, definitions[index].property);
      Object.assign(normal, definitions[index].normal);
    }

    return new Schema$6(property, normal, space)
  }

  /**
   * @param {string} value
   * @returns {string}
   */
  function normalize$6(value) {
    return value.toLowerCase()
  }

  class Info$6 {
    /**
     * @constructor
     * @param {string} property
     * @param {string} attribute
     */
    constructor(property, attribute) {
      /** @type {string} */
      this.property = property;
      /** @type {string} */
      this.attribute = attribute;
    }
  }

  /** @type {string|null} */
  Info$6.prototype.space = null;
  Info$6.prototype.boolean = false;
  Info$6.prototype.booleanish = false;
  Info$6.prototype.overloadedBoolean = false;
  Info$6.prototype.number = false;
  Info$6.prototype.commaSeparated = false;
  Info$6.prototype.spaceSeparated = false;
  Info$6.prototype.commaOrSpaceSeparated = false;
  Info$6.prototype.mustUseProperty = false;
  Info$6.prototype.defined = false;

  let powers$6 = 0;

  const boolean$6 = increment$6();
  const booleanish$6 = increment$6();
  const overloadedBoolean$6 = increment$6();
  const number$6 = increment$6();
  const spaceSeparated$6 = increment$6();
  const commaSeparated$6 = increment$6();
  const commaOrSpaceSeparated$6 = increment$6();

  function increment$6() {
    return 2 ** ++powers$6
  }

  var types$6 = /*#__PURE__*/Object.freeze({
      __proto__: null,
      boolean: boolean$6,
      booleanish: booleanish$6,
      overloadedBoolean: overloadedBoolean$6,
      number: number$6,
      spaceSeparated: spaceSeparated$6,
      commaSeparated: commaSeparated$6,
      commaOrSpaceSeparated: commaOrSpaceSeparated$6
  });

  /** @type {Array<keyof types>} */
  // @ts-expect-error: hush.
  const checks$6 = Object.keys(types$6);

  class DefinedInfo$6 extends Info$6 {
    /**
     * @constructor
     * @param {string} property
     * @param {string} attribute
     * @param {number|null} [mask]
     * @param {string} [space]
     */
    constructor(property, attribute, mask, space) {
      let index = -1;

      super(property, attribute);

      mark$6(this, 'space', space);

      if (typeof mask === 'number') {
        while (++index < checks$6.length) {
          const check = checks$6[index];
          mark$6(this, checks$6[index], (mask & types$6[check]) === types$6[check]);
        }
      }
    }
  }

  DefinedInfo$6.prototype.defined = true;

  /**
   * @param {DefinedInfo} values
   * @param {string} key
   * @param {unknown} value
   */
  function mark$6(values, key, value) {
    if (value) {
      // @ts-expect-error: assume `value` matches the expected value of `key`.
      values[key] = value;
    }
  }

  /**
   * @typedef {import('./schema.js').Properties} Properties
   * @typedef {import('./schema.js').Normal} Normal
   *
   * @typedef {Record<string, string>} Attributes
   *
   * @typedef {Object} Definition
   * @property {Record<string, number|null>} properties
   * @property {(attributes: Attributes, property: string) => string} transform
   * @property {string} [space]
   * @property {Attributes} [attributes]
   * @property {Array<string>} [mustUseProperty]
   */

  const own$l = {}.hasOwnProperty;

  /**
   * @param {Definition} definition
   * @returns {Schema}
   */
  function create$6(definition) {
    /** @type {Properties} */
    const property = {};
    /** @type {Normal} */
    const normal = {};
    /** @type {string} */
    let prop;

    for (prop in definition.properties) {
      if (own$l.call(definition.properties, prop)) {
        const value = definition.properties[prop];
        const info = new DefinedInfo$6(
          prop,
          definition.transform(definition.attributes || {}, prop),
          value,
          definition.space
        );

        if (
          definition.mustUseProperty &&
          definition.mustUseProperty.includes(prop)
        ) {
          info.mustUseProperty = true;
        }

        property[prop] = info;

        normal[normalize$6(prop)] = prop;
        normal[normalize$6(info.attribute)] = prop;
      }
    }

    return new Schema$6(property, normal, definition.space)
  }

  const xlink$6 = create$6({
    space: 'xlink',
    transform(_, prop) {
      return 'xlink:' + prop.slice(5).toLowerCase()
    },
    properties: {
      xLinkActuate: null,
      xLinkArcRole: null,
      xLinkHref: null,
      xLinkRole: null,
      xLinkShow: null,
      xLinkTitle: null,
      xLinkType: null
    }
  });

  const xml$6 = create$6({
    space: 'xml',
    transform(_, prop) {
      return 'xml:' + prop.slice(3).toLowerCase()
    },
    properties: {xmlLang: null, xmlBase: null, xmlSpace: null}
  });

  /**
   * @param {Record<string, string>} attributes
   * @param {string} attribute
   * @returns {string}
   */
  function caseSensitiveTransform$6(attributes, attribute) {
    return attribute in attributes ? attributes[attribute] : attribute
  }

  /**
   * @param {Record<string, string>} attributes
   * @param {string} property
   * @returns {string}
   */
  function caseInsensitiveTransform$6(attributes, property) {
    return caseSensitiveTransform$6(attributes, property.toLowerCase())
  }

  const xmlns$6 = create$6({
    space: 'xmlns',
    attributes: {xmlnsxlink: 'xmlns:xlink'},
    transform: caseInsensitiveTransform$6,
    properties: {xmlns: null, xmlnsXLink: null}
  });

  const aria$6 = create$6({
    transform(_, prop) {
      return prop === 'role' ? prop : 'aria-' + prop.slice(4).toLowerCase()
    },
    properties: {
      ariaActiveDescendant: null,
      ariaAtomic: booleanish$6,
      ariaAutoComplete: null,
      ariaBusy: booleanish$6,
      ariaChecked: booleanish$6,
      ariaColCount: number$6,
      ariaColIndex: number$6,
      ariaColSpan: number$6,
      ariaControls: spaceSeparated$6,
      ariaCurrent: null,
      ariaDescribedBy: spaceSeparated$6,
      ariaDetails: null,
      ariaDisabled: booleanish$6,
      ariaDropEffect: spaceSeparated$6,
      ariaErrorMessage: null,
      ariaExpanded: booleanish$6,
      ariaFlowTo: spaceSeparated$6,
      ariaGrabbed: booleanish$6,
      ariaHasPopup: null,
      ariaHidden: booleanish$6,
      ariaInvalid: null,
      ariaKeyShortcuts: null,
      ariaLabel: null,
      ariaLabelledBy: spaceSeparated$6,
      ariaLevel: number$6,
      ariaLive: null,
      ariaModal: booleanish$6,
      ariaMultiLine: booleanish$6,
      ariaMultiSelectable: booleanish$6,
      ariaOrientation: null,
      ariaOwns: spaceSeparated$6,
      ariaPlaceholder: null,
      ariaPosInSet: number$6,
      ariaPressed: booleanish$6,
      ariaReadOnly: booleanish$6,
      ariaRelevant: null,
      ariaRequired: booleanish$6,
      ariaRoleDescription: spaceSeparated$6,
      ariaRowCount: number$6,
      ariaRowIndex: number$6,
      ariaRowSpan: number$6,
      ariaSelected: booleanish$6,
      ariaSetSize: number$6,
      ariaSort: null,
      ariaValueMax: number$6,
      ariaValueMin: number$6,
      ariaValueNow: number$6,
      ariaValueText: null,
      role: null
    }
  });

  const html$g = create$6({
    space: 'html',
    attributes: {
      acceptcharset: 'accept-charset',
      classname: 'class',
      htmlfor: 'for',
      httpequiv: 'http-equiv'
    },
    transform: caseInsensitiveTransform$6,
    mustUseProperty: ['checked', 'multiple', 'muted', 'selected'],
    properties: {
      // Standard Properties.
      abbr: null,
      accept: commaSeparated$6,
      acceptCharset: spaceSeparated$6,
      accessKey: spaceSeparated$6,
      action: null,
      allow: null,
      allowFullScreen: boolean$6,
      allowPaymentRequest: boolean$6,
      allowUserMedia: boolean$6,
      alt: null,
      as: null,
      async: boolean$6,
      autoCapitalize: null,
      autoComplete: spaceSeparated$6,
      autoFocus: boolean$6,
      autoPlay: boolean$6,
      capture: boolean$6,
      charSet: null,
      checked: boolean$6,
      cite: null,
      className: spaceSeparated$6,
      cols: number$6,
      colSpan: null,
      content: null,
      contentEditable: booleanish$6,
      controls: boolean$6,
      controlsList: spaceSeparated$6,
      coords: number$6 | commaSeparated$6,
      crossOrigin: null,
      data: null,
      dateTime: null,
      decoding: null,
      default: boolean$6,
      defer: boolean$6,
      dir: null,
      dirName: null,
      disabled: boolean$6,
      download: overloadedBoolean$6,
      draggable: booleanish$6,
      encType: null,
      enterKeyHint: null,
      form: null,
      formAction: null,
      formEncType: null,
      formMethod: null,
      formNoValidate: boolean$6,
      formTarget: null,
      headers: spaceSeparated$6,
      height: number$6,
      hidden: boolean$6,
      high: number$6,
      href: null,
      hrefLang: null,
      htmlFor: spaceSeparated$6,
      httpEquiv: spaceSeparated$6,
      id: null,
      imageSizes: null,
      imageSrcSet: null,
      inputMode: null,
      integrity: null,
      is: null,
      isMap: boolean$6,
      itemId: null,
      itemProp: spaceSeparated$6,
      itemRef: spaceSeparated$6,
      itemScope: boolean$6,
      itemType: spaceSeparated$6,
      kind: null,
      label: null,
      lang: null,
      language: null,
      list: null,
      loading: null,
      loop: boolean$6,
      low: number$6,
      manifest: null,
      max: null,
      maxLength: number$6,
      media: null,
      method: null,
      min: null,
      minLength: number$6,
      multiple: boolean$6,
      muted: boolean$6,
      name: null,
      nonce: null,
      noModule: boolean$6,
      noValidate: boolean$6,
      onAbort: null,
      onAfterPrint: null,
      onAuxClick: null,
      onBeforePrint: null,
      onBeforeUnload: null,
      onBlur: null,
      onCancel: null,
      onCanPlay: null,
      onCanPlayThrough: null,
      onChange: null,
      onClick: null,
      onClose: null,
      onContextLost: null,
      onContextMenu: null,
      onContextRestored: null,
      onCopy: null,
      onCueChange: null,
      onCut: null,
      onDblClick: null,
      onDrag: null,
      onDragEnd: null,
      onDragEnter: null,
      onDragExit: null,
      onDragLeave: null,
      onDragOver: null,
      onDragStart: null,
      onDrop: null,
      onDurationChange: null,
      onEmptied: null,
      onEnded: null,
      onError: null,
      onFocus: null,
      onFormData: null,
      onHashChange: null,
      onInput: null,
      onInvalid: null,
      onKeyDown: null,
      onKeyPress: null,
      onKeyUp: null,
      onLanguageChange: null,
      onLoad: null,
      onLoadedData: null,
      onLoadedMetadata: null,
      onLoadEnd: null,
      onLoadStart: null,
      onMessage: null,
      onMessageError: null,
      onMouseDown: null,
      onMouseEnter: null,
      onMouseLeave: null,
      onMouseMove: null,
      onMouseOut: null,
      onMouseOver: null,
      onMouseUp: null,
      onOffline: null,
      onOnline: null,
      onPageHide: null,
      onPageShow: null,
      onPaste: null,
      onPause: null,
      onPlay: null,
      onPlaying: null,
      onPopState: null,
      onProgress: null,
      onRateChange: null,
      onRejectionHandled: null,
      onReset: null,
      onResize: null,
      onScroll: null,
      onSecurityPolicyViolation: null,
      onSeeked: null,
      onSeeking: null,
      onSelect: null,
      onSlotChange: null,
      onStalled: null,
      onStorage: null,
      onSubmit: null,
      onSuspend: null,
      onTimeUpdate: null,
      onToggle: null,
      onUnhandledRejection: null,
      onUnload: null,
      onVolumeChange: null,
      onWaiting: null,
      onWheel: null,
      open: boolean$6,
      optimum: number$6,
      pattern: null,
      ping: spaceSeparated$6,
      placeholder: null,
      playsInline: boolean$6,
      poster: null,
      preload: null,
      readOnly: boolean$6,
      referrerPolicy: null,
      rel: spaceSeparated$6,
      required: boolean$6,
      reversed: boolean$6,
      rows: number$6,
      rowSpan: number$6,
      sandbox: spaceSeparated$6,
      scope: null,
      scoped: boolean$6,
      seamless: boolean$6,
      selected: boolean$6,
      shape: null,
      size: number$6,
      sizes: null,
      slot: null,
      span: number$6,
      spellCheck: booleanish$6,
      src: null,
      srcDoc: null,
      srcLang: null,
      srcSet: null,
      start: number$6,
      step: null,
      style: null,
      tabIndex: number$6,
      target: null,
      title: null,
      translate: null,
      type: null,
      typeMustMatch: boolean$6,
      useMap: null,
      value: booleanish$6,
      width: number$6,
      wrap: null,

      // Legacy.
      // See: https://html.spec.whatwg.org/#other-elements,-attributes-and-apis
      align: null, // Several. Use CSS `text-align` instead,
      aLink: null, // `<body>`. Use CSS `a:active {color}` instead
      archive: spaceSeparated$6, // `<object>`. List of URIs to archives
      axis: null, // `<td>` and `<th>`. Use `scope` on `<th>`
      background: null, // `<body>`. Use CSS `background-image` instead
      bgColor: null, // `<body>` and table elements. Use CSS `background-color` instead
      border: number$6, // `<table>`. Use CSS `border-width` instead,
      borderColor: null, // `<table>`. Use CSS `border-color` instead,
      bottomMargin: number$6, // `<body>`
      cellPadding: null, // `<table>`
      cellSpacing: null, // `<table>`
      char: null, // Several table elements. When `align=char`, sets the character to align on
      charOff: null, // Several table elements. When `char`, offsets the alignment
      classId: null, // `<object>`
      clear: null, // `<br>`. Use CSS `clear` instead
      code: null, // `<object>`
      codeBase: null, // `<object>`
      codeType: null, // `<object>`
      color: null, // `<font>` and `<hr>`. Use CSS instead
      compact: boolean$6, // Lists. Use CSS to reduce space between items instead
      declare: boolean$6, // `<object>`
      event: null, // `<script>`
      face: null, // `<font>`. Use CSS instead
      frame: null, // `<table>`
      frameBorder: null, // `<iframe>`. Use CSS `border` instead
      hSpace: number$6, // `<img>` and `<object>`
      leftMargin: number$6, // `<body>`
      link: null, // `<body>`. Use CSS `a:link {color: *}` instead
      longDesc: null, // `<frame>`, `<iframe>`, and `<img>`. Use an `<a>`
      lowSrc: null, // `<img>`. Use a `<picture>`
      marginHeight: number$6, // `<body>`
      marginWidth: number$6, // `<body>`
      noResize: boolean$6, // `<frame>`
      noHref: boolean$6, // `<area>`. Use no href instead of an explicit `nohref`
      noShade: boolean$6, // `<hr>`. Use background-color and height instead of borders
      noWrap: boolean$6, // `<td>` and `<th>`
      object: null, // `<applet>`
      profile: null, // `<head>`
      prompt: null, // `<isindex>`
      rev: null, // `<link>`
      rightMargin: number$6, // `<body>`
      rules: null, // `<table>`
      scheme: null, // `<meta>`
      scrolling: booleanish$6, // `<frame>`. Use overflow in the child context
      standby: null, // `<object>`
      summary: null, // `<table>`
      text: null, // `<body>`. Use CSS `color` instead
      topMargin: number$6, // `<body>`
      valueType: null, // `<param>`
      version: null, // `<html>`. Use a doctype.
      vAlign: null, // Several. Use CSS `vertical-align` instead
      vLink: null, // `<body>`. Use CSS `a:visited {color}` instead
      vSpace: number$6, // `<img>` and `<object>`

      // Non-standard Properties.
      allowTransparency: null,
      autoCorrect: null,
      autoSave: null,
      disablePictureInPicture: boolean$6,
      disableRemotePlayback: boolean$6,
      prefix: null,
      property: null,
      results: number$6,
      security: null,
      unselectable: null
    }
  });

  const svg$c = create$6({
    space: 'svg',
    attributes: {
      accentHeight: 'accent-height',
      alignmentBaseline: 'alignment-baseline',
      arabicForm: 'arabic-form',
      baselineShift: 'baseline-shift',
      capHeight: 'cap-height',
      className: 'class',
      clipPath: 'clip-path',
      clipRule: 'clip-rule',
      colorInterpolation: 'color-interpolation',
      colorInterpolationFilters: 'color-interpolation-filters',
      colorProfile: 'color-profile',
      colorRendering: 'color-rendering',
      crossOrigin: 'crossorigin',
      dataType: 'datatype',
      dominantBaseline: 'dominant-baseline',
      enableBackground: 'enable-background',
      fillOpacity: 'fill-opacity',
      fillRule: 'fill-rule',
      floodColor: 'flood-color',
      floodOpacity: 'flood-opacity',
      fontFamily: 'font-family',
      fontSize: 'font-size',
      fontSizeAdjust: 'font-size-adjust',
      fontStretch: 'font-stretch',
      fontStyle: 'font-style',
      fontVariant: 'font-variant',
      fontWeight: 'font-weight',
      glyphName: 'glyph-name',
      glyphOrientationHorizontal: 'glyph-orientation-horizontal',
      glyphOrientationVertical: 'glyph-orientation-vertical',
      hrefLang: 'hreflang',
      horizAdvX: 'horiz-adv-x',
      horizOriginX: 'horiz-origin-x',
      horizOriginY: 'horiz-origin-y',
      imageRendering: 'image-rendering',
      letterSpacing: 'letter-spacing',
      lightingColor: 'lighting-color',
      markerEnd: 'marker-end',
      markerMid: 'marker-mid',
      markerStart: 'marker-start',
      navDown: 'nav-down',
      navDownLeft: 'nav-down-left',
      navDownRight: 'nav-down-right',
      navLeft: 'nav-left',
      navNext: 'nav-next',
      navPrev: 'nav-prev',
      navRight: 'nav-right',
      navUp: 'nav-up',
      navUpLeft: 'nav-up-left',
      navUpRight: 'nav-up-right',
      onAbort: 'onabort',
      onActivate: 'onactivate',
      onAfterPrint: 'onafterprint',
      onBeforePrint: 'onbeforeprint',
      onBegin: 'onbegin',
      onCancel: 'oncancel',
      onCanPlay: 'oncanplay',
      onCanPlayThrough: 'oncanplaythrough',
      onChange: 'onchange',
      onClick: 'onclick',
      onClose: 'onclose',
      onCopy: 'oncopy',
      onCueChange: 'oncuechange',
      onCut: 'oncut',
      onDblClick: 'ondblclick',
      onDrag: 'ondrag',
      onDragEnd: 'ondragend',
      onDragEnter: 'ondragenter',
      onDragExit: 'ondragexit',
      onDragLeave: 'ondragleave',
      onDragOver: 'ondragover',
      onDragStart: 'ondragstart',
      onDrop: 'ondrop',
      onDurationChange: 'ondurationchange',
      onEmptied: 'onemptied',
      onEnd: 'onend',
      onEnded: 'onended',
      onError: 'onerror',
      onFocus: 'onfocus',
      onFocusIn: 'onfocusin',
      onFocusOut: 'onfocusout',
      onHashChange: 'onhashchange',
      onInput: 'oninput',
      onInvalid: 'oninvalid',
      onKeyDown: 'onkeydown',
      onKeyPress: 'onkeypress',
      onKeyUp: 'onkeyup',
      onLoad: 'onload',
      onLoadedData: 'onloadeddata',
      onLoadedMetadata: 'onloadedmetadata',
      onLoadStart: 'onloadstart',
      onMessage: 'onmessage',
      onMouseDown: 'onmousedown',
      onMouseEnter: 'onmouseenter',
      onMouseLeave: 'onmouseleave',
      onMouseMove: 'onmousemove',
      onMouseOut: 'onmouseout',
      onMouseOver: 'onmouseover',
      onMouseUp: 'onmouseup',
      onMouseWheel: 'onmousewheel',
      onOffline: 'onoffline',
      onOnline: 'ononline',
      onPageHide: 'onpagehide',
      onPageShow: 'onpageshow',
      onPaste: 'onpaste',
      onPause: 'onpause',
      onPlay: 'onplay',
      onPlaying: 'onplaying',
      onPopState: 'onpopstate',
      onProgress: 'onprogress',
      onRateChange: 'onratechange',
      onRepeat: 'onrepeat',
      onReset: 'onreset',
      onResize: 'onresize',
      onScroll: 'onscroll',
      onSeeked: 'onseeked',
      onSeeking: 'onseeking',
      onSelect: 'onselect',
      onShow: 'onshow',
      onStalled: 'onstalled',
      onStorage: 'onstorage',
      onSubmit: 'onsubmit',
      onSuspend: 'onsuspend',
      onTimeUpdate: 'ontimeupdate',
      onToggle: 'ontoggle',
      onUnload: 'onunload',
      onVolumeChange: 'onvolumechange',
      onWaiting: 'onwaiting',
      onZoom: 'onzoom',
      overlinePosition: 'overline-position',
      overlineThickness: 'overline-thickness',
      paintOrder: 'paint-order',
      panose1: 'panose-1',
      pointerEvents: 'pointer-events',
      referrerPolicy: 'referrerpolicy',
      renderingIntent: 'rendering-intent',
      shapeRendering: 'shape-rendering',
      stopColor: 'stop-color',
      stopOpacity: 'stop-opacity',
      strikethroughPosition: 'strikethrough-position',
      strikethroughThickness: 'strikethrough-thickness',
      strokeDashArray: 'stroke-dasharray',
      strokeDashOffset: 'stroke-dashoffset',
      strokeLineCap: 'stroke-linecap',
      strokeLineJoin: 'stroke-linejoin',
      strokeMiterLimit: 'stroke-miterlimit',
      strokeOpacity: 'stroke-opacity',
      strokeWidth: 'stroke-width',
      tabIndex: 'tabindex',
      textAnchor: 'text-anchor',
      textDecoration: 'text-decoration',
      textRendering: 'text-rendering',
      typeOf: 'typeof',
      underlinePosition: 'underline-position',
      underlineThickness: 'underline-thickness',
      unicodeBidi: 'unicode-bidi',
      unicodeRange: 'unicode-range',
      unitsPerEm: 'units-per-em',
      vAlphabetic: 'v-alphabetic',
      vHanging: 'v-hanging',
      vIdeographic: 'v-ideographic',
      vMathematical: 'v-mathematical',
      vectorEffect: 'vector-effect',
      vertAdvY: 'vert-adv-y',
      vertOriginX: 'vert-origin-x',
      vertOriginY: 'vert-origin-y',
      wordSpacing: 'word-spacing',
      writingMode: 'writing-mode',
      xHeight: 'x-height',
      // These were camelcased in Tiny. Now lowercased in SVG 2
      playbackOrder: 'playbackorder',
      timelineBegin: 'timelinebegin'
    },
    transform: caseSensitiveTransform$6,
    properties: {
      about: commaOrSpaceSeparated$6,
      accentHeight: number$6,
      accumulate: null,
      additive: null,
      alignmentBaseline: null,
      alphabetic: number$6,
      amplitude: number$6,
      arabicForm: null,
      ascent: number$6,
      attributeName: null,
      attributeType: null,
      azimuth: number$6,
      bandwidth: null,
      baselineShift: null,
      baseFrequency: null,
      baseProfile: null,
      bbox: null,
      begin: null,
      bias: number$6,
      by: null,
      calcMode: null,
      capHeight: number$6,
      className: spaceSeparated$6,
      clip: null,
      clipPath: null,
      clipPathUnits: null,
      clipRule: null,
      color: null,
      colorInterpolation: null,
      colorInterpolationFilters: null,
      colorProfile: null,
      colorRendering: null,
      content: null,
      contentScriptType: null,
      contentStyleType: null,
      crossOrigin: null,
      cursor: null,
      cx: null,
      cy: null,
      d: null,
      dataType: null,
      defaultAction: null,
      descent: number$6,
      diffuseConstant: number$6,
      direction: null,
      display: null,
      dur: null,
      divisor: number$6,
      dominantBaseline: null,
      download: boolean$6,
      dx: null,
      dy: null,
      edgeMode: null,
      editable: null,
      elevation: number$6,
      enableBackground: null,
      end: null,
      event: null,
      exponent: number$6,
      externalResourcesRequired: null,
      fill: null,
      fillOpacity: number$6,
      fillRule: null,
      filter: null,
      filterRes: null,
      filterUnits: null,
      floodColor: null,
      floodOpacity: null,
      focusable: null,
      focusHighlight: null,
      fontFamily: null,
      fontSize: null,
      fontSizeAdjust: null,
      fontStretch: null,
      fontStyle: null,
      fontVariant: null,
      fontWeight: null,
      format: null,
      fr: null,
      from: null,
      fx: null,
      fy: null,
      g1: commaSeparated$6,
      g2: commaSeparated$6,
      glyphName: commaSeparated$6,
      glyphOrientationHorizontal: null,
      glyphOrientationVertical: null,
      glyphRef: null,
      gradientTransform: null,
      gradientUnits: null,
      handler: null,
      hanging: number$6,
      hatchContentUnits: null,
      hatchUnits: null,
      height: null,
      href: null,
      hrefLang: null,
      horizAdvX: number$6,
      horizOriginX: number$6,
      horizOriginY: number$6,
      id: null,
      ideographic: number$6,
      imageRendering: null,
      initialVisibility: null,
      in: null,
      in2: null,
      intercept: number$6,
      k: number$6,
      k1: number$6,
      k2: number$6,
      k3: number$6,
      k4: number$6,
      kernelMatrix: commaOrSpaceSeparated$6,
      kernelUnitLength: null,
      keyPoints: null, // SEMI_COLON_SEPARATED
      keySplines: null, // SEMI_COLON_SEPARATED
      keyTimes: null, // SEMI_COLON_SEPARATED
      kerning: null,
      lang: null,
      lengthAdjust: null,
      letterSpacing: null,
      lightingColor: null,
      limitingConeAngle: number$6,
      local: null,
      markerEnd: null,
      markerMid: null,
      markerStart: null,
      markerHeight: null,
      markerUnits: null,
      markerWidth: null,
      mask: null,
      maskContentUnits: null,
      maskUnits: null,
      mathematical: null,
      max: null,
      media: null,
      mediaCharacterEncoding: null,
      mediaContentEncodings: null,
      mediaSize: number$6,
      mediaTime: null,
      method: null,
      min: null,
      mode: null,
      name: null,
      navDown: null,
      navDownLeft: null,
      navDownRight: null,
      navLeft: null,
      navNext: null,
      navPrev: null,
      navRight: null,
      navUp: null,
      navUpLeft: null,
      navUpRight: null,
      numOctaves: null,
      observer: null,
      offset: null,
      onAbort: null,
      onActivate: null,
      onAfterPrint: null,
      onBeforePrint: null,
      onBegin: null,
      onCancel: null,
      onCanPlay: null,
      onCanPlayThrough: null,
      onChange: null,
      onClick: null,
      onClose: null,
      onCopy: null,
      onCueChange: null,
      onCut: null,
      onDblClick: null,
      onDrag: null,
      onDragEnd: null,
      onDragEnter: null,
      onDragExit: null,
      onDragLeave: null,
      onDragOver: null,
      onDragStart: null,
      onDrop: null,
      onDurationChange: null,
      onEmptied: null,
      onEnd: null,
      onEnded: null,
      onError: null,
      onFocus: null,
      onFocusIn: null,
      onFocusOut: null,
      onHashChange: null,
      onInput: null,
      onInvalid: null,
      onKeyDown: null,
      onKeyPress: null,
      onKeyUp: null,
      onLoad: null,
      onLoadedData: null,
      onLoadedMetadata: null,
      onLoadStart: null,
      onMessage: null,
      onMouseDown: null,
      onMouseEnter: null,
      onMouseLeave: null,
      onMouseMove: null,
      onMouseOut: null,
      onMouseOver: null,
      onMouseUp: null,
      onMouseWheel: null,
      onOffline: null,
      onOnline: null,
      onPageHide: null,
      onPageShow: null,
      onPaste: null,
      onPause: null,
      onPlay: null,
      onPlaying: null,
      onPopState: null,
      onProgress: null,
      onRateChange: null,
      onRepeat: null,
      onReset: null,
      onResize: null,
      onScroll: null,
      onSeeked: null,
      onSeeking: null,
      onSelect: null,
      onShow: null,
      onStalled: null,
      onStorage: null,
      onSubmit: null,
      onSuspend: null,
      onTimeUpdate: null,
      onToggle: null,
      onUnload: null,
      onVolumeChange: null,
      onWaiting: null,
      onZoom: null,
      opacity: null,
      operator: null,
      order: null,
      orient: null,
      orientation: null,
      origin: null,
      overflow: null,
      overlay: null,
      overlinePosition: number$6,
      overlineThickness: number$6,
      paintOrder: null,
      panose1: null,
      path: null,
      pathLength: number$6,
      patternContentUnits: null,
      patternTransform: null,
      patternUnits: null,
      phase: null,
      ping: spaceSeparated$6,
      pitch: null,
      playbackOrder: null,
      pointerEvents: null,
      points: null,
      pointsAtX: number$6,
      pointsAtY: number$6,
      pointsAtZ: number$6,
      preserveAlpha: null,
      preserveAspectRatio: null,
      primitiveUnits: null,
      propagate: null,
      property: commaOrSpaceSeparated$6,
      r: null,
      radius: null,
      referrerPolicy: null,
      refX: null,
      refY: null,
      rel: commaOrSpaceSeparated$6,
      rev: commaOrSpaceSeparated$6,
      renderingIntent: null,
      repeatCount: null,
      repeatDur: null,
      requiredExtensions: commaOrSpaceSeparated$6,
      requiredFeatures: commaOrSpaceSeparated$6,
      requiredFonts: commaOrSpaceSeparated$6,
      requiredFormats: commaOrSpaceSeparated$6,
      resource: null,
      restart: null,
      result: null,
      rotate: null,
      rx: null,
      ry: null,
      scale: null,
      seed: null,
      shapeRendering: null,
      side: null,
      slope: null,
      snapshotTime: null,
      specularConstant: number$6,
      specularExponent: number$6,
      spreadMethod: null,
      spacing: null,
      startOffset: null,
      stdDeviation: null,
      stemh: null,
      stemv: null,
      stitchTiles: null,
      stopColor: null,
      stopOpacity: null,
      strikethroughPosition: number$6,
      strikethroughThickness: number$6,
      string: null,
      stroke: null,
      strokeDashArray: commaOrSpaceSeparated$6,
      strokeDashOffset: null,
      strokeLineCap: null,
      strokeLineJoin: null,
      strokeMiterLimit: number$6,
      strokeOpacity: number$6,
      strokeWidth: null,
      style: null,
      surfaceScale: number$6,
      syncBehavior: null,
      syncBehaviorDefault: null,
      syncMaster: null,
      syncTolerance: null,
      syncToleranceDefault: null,
      systemLanguage: commaOrSpaceSeparated$6,
      tabIndex: number$6,
      tableValues: null,
      target: null,
      targetX: number$6,
      targetY: number$6,
      textAnchor: null,
      textDecoration: null,
      textRendering: null,
      textLength: null,
      timelineBegin: null,
      title: null,
      transformBehavior: null,
      type: null,
      typeOf: commaOrSpaceSeparated$6,
      to: null,
      transform: null,
      u1: null,
      u2: null,
      underlinePosition: number$6,
      underlineThickness: number$6,
      unicode: null,
      unicodeBidi: null,
      unicodeRange: null,
      unitsPerEm: number$6,
      values: null,
      vAlphabetic: number$6,
      vMathematical: number$6,
      vectorEffect: null,
      vHanging: number$6,
      vIdeographic: number$6,
      version: null,
      vertAdvY: number$6,
      vertOriginX: number$6,
      vertOriginY: number$6,
      viewBox: null,
      viewTarget: null,
      visibility: null,
      width: null,
      widths: null,
      wordSpacing: null,
      writingMode: null,
      x: null,
      x1: null,
      x2: null,
      xChannelSelector: null,
      xHeight: number$6,
      y: null,
      y1: null,
      y2: null,
      yChannelSelector: null,
      z: null,
      zoomAndPan: null
    }
  });

  /**
   * @typedef {import('./util/schema.js').Schema} Schema
   */

  const valid$6 = /^data[-\w.:]+$/i;
  const dash$6 = /-[a-z]/g;
  const cap$6 = /[A-Z]/g;

  /**
   * @param {Schema} schema
   * @param {string} value
   * @returns {Info}
   */
  function find$6(schema, value) {
    const normal = normalize$6(value);
    let prop = value;
    let Type = Info$6;

    if (normal in schema.normal) {
      return schema.property[schema.normal[normal]]
    }

    if (normal.length > 4 && normal.slice(0, 4) === 'data' && valid$6.test(value)) {
      // Attribute or property.
      if (value.charAt(4) === '-') {
        // Turn it into a property.
        const rest = value.slice(5).replace(dash$6, camelcase$7);
        prop = 'data' + rest.charAt(0).toUpperCase() + rest.slice(1);
      } else {
        // Turn it into an attribute.
        const rest = value.slice(4);

        if (!dash$6.test(rest)) {
          let dashes = rest.replace(cap$6, kebab$6);

          if (dashes.charAt(0) !== '-') {
            dashes = '-' + dashes;
          }

          value = 'data' + dashes;
        }
      }

      Type = DefinedInfo$6;
    }

    return new Type(prop, value)
  }

  /**
   * @param {string} $0
   * @returns {string}
   */
  function kebab$6($0) {
    return '-' + $0.toLowerCase()
  }

  /**
   * @param {string} $0
   * @returns {string}
   */
  function camelcase$7($0) {
    return $0.charAt(1).toUpperCase()
  }

  /**
   * @typedef {import('./lib/util/info.js').Info} Info
   * @typedef {import('./lib/util/schema.js').Schema} Schema
   */
  const html$f = merge$6([xml$6, xlink$6, xmlns$6, aria$6, html$g], 'html');
  const svg$b = merge$6([xml$6, xlink$6, xmlns$6, aria$6, svg$c], 'svg');

  /**
   * @typedef {import('hast').Properties} Properties
   * @typedef {import('hast').Element} Element
   */

  var search$2 = /[#.]/g;

  /**
   * Create a hast element from a simple CSS selector.
   *
   * @param selector A simple CSS selector.
   *   Can contain a tag-name (`foo`), classes (`.bar`), and an ID (`#baz`).
   *   Multiple classes are allowed.
   *   Uses the last ID if multiple IDs are found.
   * @param [defaultTagName='div'] Tag name to use if `selector` does not specify one.
   */
  const parseSelector$2 =
    /**
     * @type {(
     *  <Selector extends string, DefaultTagName extends string = 'div'>(selector?: Selector, defaultTagName?: DefaultTagName) => Element & {tagName: import('./extract.js').ExtractTagName<Selector, DefaultTagName>}
     * )}
     */
    (
      /**
       * @param {string} [selector]
       * @param {string} [defaultTagName='div']
       * @returns {Element}
       */
      function (selector, defaultTagName = 'div') {
        var value = selector || '';
        /** @type {Properties} */
        var props = {};
        var start = 0;
        /** @type {string} */
        var subvalue;
        /** @type {string} */
        var previous;
        /** @type {RegExpMatchArray} */
        var match;

        while (start < value.length) {
          search$2.lastIndex = start;
          match = search$2.exec(value);
          subvalue = value.slice(start, match ? match.index : value.length);

          if (subvalue) {
            if (!previous) {
              defaultTagName = subvalue;
            } else if (previous === '#') {
              props.id = subvalue;
            } else if (Array.isArray(props.className)) {
              props.className.push(subvalue);
            } else {
              props.className = [subvalue];
            }

            start += subvalue.length;
          }

          if (match) {
            previous = match[0];
            start++;
          }
        }

        return {
          type: 'element',
          tagName: defaultTagName,
          properties: props,
          children: []
        }
      }
    );

  /**
   * Parse space separated tokens to an array of strings.
   *
   * @param {string} value Space separated tokens
   * @returns {Array.<string>} Tokens
   */
  function parse$a(value) {
    const input = String(value || '').trim();
    return input ? input.split(/[ \t\n\r\f]+/g) : []
  }

  /**
   * @typedef {Object} StringifyOptions
   * @property {boolean} [padLeft=true] Whether to pad a space before a token (`boolean`, default: `true`).
   * @property {boolean} [padRight=false] Whether to pad a space after a token (`boolean`, default: `false`).
   */

  /**
   * Parse comma separated tokens to an array.
   *
   * @param {string} value
   * @returns {Array.<string>}
   */
  function parse$9(value) {
    /** @type {Array.<string>} */
    var tokens = [];
    var input = String(value || '');
    var index = input.indexOf(',');
    var start = 0;
    /** @type {boolean} */
    var end;
    /** @type {string} */
    var token;

    while (!end) {
      if (index === -1) {
        index = input.length;
        end = true;
      }

      token = input.slice(start, index).trim();

      if (token || !end) {
        tokens.push(token);
      }

      start = index + 1;
      index = input.indexOf(',', start);
    }

    return tokens
  }

  /**
   * @typedef {import('hast').Root} Root
   * @typedef {import('hast').Element} Element
   * @typedef {import('hast').Properties} Properties
   * @typedef {Root['children'][number]} Child
   * @typedef {Child|Root} Node
   * @typedef {import('property-information').Info} Info
   * @typedef {import('property-information').Schema} Schema
   *
   * @typedef {Root|Element} HResult
   * @typedef {string|number} HStyleValue
   * @typedef {Object.<string, HStyleValue>} HStyle
   * @typedef {string|number|boolean|null|undefined} HPrimitiveValue
   * @typedef {Array.<string|number>} HArrayValue
   * @typedef {HPrimitiveValue|HArrayValue} HPropertyValue
   * @typedef {{[property: string]: HPropertyValue|HStyle}} HProperties
   *
   * @typedef {string|number|null|undefined} HPrimitiveChild
   * @typedef {Array.<Node|HPrimitiveChild>} HArrayChild
   * @typedef {Node|HPrimitiveChild|HArrayChild} HChild
   */

  const buttonTypes$2 = new Set(['menu', 'submit', 'reset', 'button']);

  const own$k = {}.hasOwnProperty;

  /**
   * @param {Schema} schema
   * @param {string} defaultTagName
   * @param {Array.<string>} [caseSensitive]
   */
  function core$3(schema, defaultTagName, caseSensitive) {
    const adjust = caseSensitive && createAdjustMap$2(caseSensitive);

    const h =
      /**
       * @type {{
       *   (): Root
       *   (selector: null|undefined, ...children: HChild[]): Root
       *   (selector: string, properties?: HProperties, ...children: HChild[]): Element
       *   (selector: string, ...children: HChild[]): Element
       * }}
       */
      (
        /**
         * Hyperscript compatible DSL for creating virtual hast trees.
         *
         * @param {string|null} [selector]
         * @param {HProperties|HChild} [properties]
         * @param {HChild[]} children
         * @returns {HResult}
         */
        function (selector, properties, ...children) {
          let index = -1;
          /** @type {HResult} */
          let node;

          if (selector === undefined || selector === null) {
            node = {type: 'root', children: []};
            // @ts-expect-error Properties are not supported for roots.
            children.unshift(properties);
          } else {
            node = parseSelector$2(selector, defaultTagName);
            // Normalize the name.
            node.tagName = node.tagName.toLowerCase();
            if (adjust && own$k.call(adjust, node.tagName)) {
              node.tagName = adjust[node.tagName];
            }

            // Handle props.
            if (isProperties$2(properties, node.tagName)) {
              /** @type {string} */
              let key;

              for (key in properties) {
                if (own$k.call(properties, key)) {
                  // @ts-expect-error `node.properties` is set.
                  addProperty$2(schema, node.properties, key, properties[key]);
                }
              }
            } else {
              children.unshift(properties);
            }
          }

          // Handle children.
          while (++index < children.length) {
            addChild$2(node.children, children[index]);
          }

          if (node.type === 'element' && node.tagName === 'template') {
            node.content = {type: 'root', children: node.children};
            node.children = [];
          }

          return node
        }
      );

    return h
  }

  /**
   * @param {HProperties|HChild} value
   * @param {string} name
   * @returns {value is HProperties}
   */
  function isProperties$2(value, name) {
    if (
      value === null ||
      value === undefined ||
      typeof value !== 'object' ||
      Array.isArray(value)
    ) {
      return false
    }

    if (name === 'input' || !value.type || typeof value.type !== 'string') {
      return true
    }

    if ('children' in value && Array.isArray(value.children)) {
      return false
    }

    if (name === 'button') {
      return buttonTypes$2.has(value.type.toLowerCase())
    }

    return !('value' in value)
  }

  /**
   * @param {Schema} schema
   * @param {Properties} properties
   * @param {string} key
   * @param {HStyle|HPropertyValue} value
   * @returns {void}
   */
  function addProperty$2(schema, properties, key, value) {
    const info = find$6(schema, key);
    let index = -1;
    /** @type {HPropertyValue} */
    let result;

    // Ignore nullish and NaN values.
    if (value === undefined || value === null) return

    if (typeof value === 'number') {
      // Ignore NaN.
      if (Number.isNaN(value)) return

      result = value;
    }
    // Booleans.
    else if (typeof value === 'boolean') {
      result = value;
    }
    // Handle list values.
    else if (typeof value === 'string') {
      if (info.spaceSeparated) {
        result = parse$a(value);
      } else if (info.commaSeparated) {
        result = parse$9(value);
      } else if (info.commaOrSpaceSeparated) {
        result = parse$a(parse$9(value).join(' '));
      } else {
        result = parsePrimitive$2(info, info.property, value);
      }
    } else if (Array.isArray(value)) {
      result = value.concat();
    } else {
      result = info.property === 'style' ? style$2(value) : String(value);
    }

    if (Array.isArray(result)) {
      /** @type {Array.<string|number>} */
      const finalResult = [];

      while (++index < result.length) {
        // @ts-expect-error Assume no booleans in array.
        finalResult[index] = parsePrimitive$2(info, info.property, result[index]);
      }

      result = finalResult;
    }

    // Class names (which can be added both on the `selector` and here).
    if (info.property === 'className' && Array.isArray(properties.className)) {
      // @ts-expect-error Assume no booleans in `className`.
      result = properties.className.concat(result);
    }

    properties[info.property] = result;
  }

  /**
   * @param {Array.<Child>} nodes
   * @param {HChild} value
   * @returns {void}
   */
  function addChild$2(nodes, value) {
    let index = -1;

    if (value === undefined || value === null) ; else if (typeof value === 'string' || typeof value === 'number') {
      nodes.push({type: 'text', value: String(value)});
    } else if (Array.isArray(value)) {
      while (++index < value.length) {
        addChild$2(nodes, value[index]);
      }
    } else if (typeof value === 'object' && 'type' in value) {
      if (value.type === 'root') {
        addChild$2(nodes, value.children);
      } else {
        nodes.push(value);
      }
    } else {
      throw new Error('Expected node, nodes, or string, got `' + value + '`')
    }
  }

  /**
   * Parse a single primitives.
   *
   * @param {Info} info
   * @param {string} name
   * @param {HPrimitiveValue} value
   * @returns {HPrimitiveValue}
   */
  function parsePrimitive$2(info, name, value) {
    if (typeof value === 'string') {
      if (info.number && value && !Number.isNaN(Number(value))) {
        return Number(value)
      }

      if (
        (info.boolean || info.overloadedBoolean) &&
        (value === '' || normalize$6(value) === normalize$6(name))
      ) {
        return true
      }
    }

    return value
  }

  /**
   * @param {HStyle} value
   * @returns {string}
   */
  function style$2(value) {
    /** @type {Array.<string>} */
    const result = [];
    /** @type {string} */
    let key;

    for (key in value) {
      if (own$k.call(value, key)) {
        result.push([key, value[key]].join(': '));
      }
    }

    return result.join('; ')
  }

  /**
   * @param {Array.<string>} values
   * @returns {Object.<string, string>}
   */
  function createAdjustMap$2(values) {
    /** @type {Object.<string, string>} */
    const result = {};
    let index = -1;

    while (++index < values.length) {
      result[values[index].toLowerCase()] = values[index];
    }

    return result
  }

  /**
   * @typedef {import('./core.js').HChild} Child Acceptable child value
   * @typedef {import('./core.js').HProperties} Properties Acceptable properties value.
   *
   * @typedef {import('./jsx-classic').Element} h.JSX.Element
   * @typedef {import('./jsx-classic').IntrinsicAttributes} h.JSX.IntrinsicAttributes
   * @typedef {import('./jsx-classic').IntrinsicElements} h.JSX.IntrinsicElements
   * @typedef {import('./jsx-classic').ElementChildrenAttribute} h.JSX.ElementChildrenAttribute
   */

  const h$2 = core$3(html$f, 'div');

  const svgCaseSensitiveTagNames$1 = [
    'altGlyph',
    'altGlyphDef',
    'altGlyphItem',
    'animateColor',
    'animateMotion',
    'animateTransform',
    'clipPath',
    'feBlend',
    'feColorMatrix',
    'feComponentTransfer',
    'feComposite',
    'feConvolveMatrix',
    'feDiffuseLighting',
    'feDisplacementMap',
    'feDistantLight',
    'feDropShadow',
    'feFlood',
    'feFuncA',
    'feFuncB',
    'feFuncG',
    'feFuncR',
    'feGaussianBlur',
    'feImage',
    'feMerge',
    'feMergeNode',
    'feMorphology',
    'feOffset',
    'fePointLight',
    'feSpecularLighting',
    'feSpotLight',
    'feTile',
    'feTurbulence',
    'foreignObject',
    'glyphRef',
    'linearGradient',
    'radialGradient',
    'solidColor',
    'textArea',
    'textPath'
  ];

  /**
   * @typedef {import('./core.js').HChild} Child Acceptable child value
   * @typedef {import('./core.js').HProperties} Properties Acceptable properties value.
   *
   * @typedef {import('./jsx-classic').Element} s.JSX.Element
   * @typedef {import('./jsx-classic').IntrinsicAttributes} s.JSX.IntrinsicAttributes
   * @typedef {import('./jsx-classic').IntrinsicElements} s.JSX.IntrinsicElements
   * @typedef {import('./jsx-classic').ElementChildrenAttribute} s.JSX.ElementChildrenAttribute
   */

  const s$2 = core$3(svg$b, 'g', svgCaseSensitiveTagNames$1);

  /**
   * @typedef {import('unist').Point} Point
   * @typedef {import('vfile').VFile} VFile
   *
   * @typedef {Pick<Point, 'line'|'column'>} PositionalPoint
   * @typedef {Required<Point>} FullPoint
   * @typedef {NonNullable<Point['offset']>} Offset
   */

  /**
   * Get transform functions for the given `document`.
   *
   * @param {string|Uint8Array|VFile} file
   */
  function location$1(file) {
    var value = String(file);
    /** @type {Array.<number>} */
    var indices = [];
    var search = /\r?\n|\r/g;

    while (search.test(value)) {
      indices.push(search.lastIndex);
    }

    indices.push(value.length + 1);

    return {toPoint, toOffset}

    /**
     * Get the line and column-based `point` for `offset` in the bound indices.
     * Returns a point with `undefined` values when given invalid or out of bounds
     * input.
     *
     * @param {Offset} offset
     * @returns {FullPoint}
     */
    function toPoint(offset) {
      var index = -1;

      if (offset > -1 && offset < indices[indices.length - 1]) {
        while (++index < indices.length) {
          if (indices[index] > offset) {
            return {
              line: index + 1,
              column: offset - (indices[index - 1] || 0) + 1,
              offset
            }
          }
        }
      }

      return {line: undefined, column: undefined, offset: undefined}
    }

    /**
     * Get the `offset` for a line and column-based `point` in the bound indices.
     * Returns `-1` when given invalid or out of bounds input.
     *
     * @param {PositionalPoint} point
     * @returns {Offset}
     */
    function toOffset(point) {
      var line = point && point.line;
      var column = point && point.column;
      /** @type {number} */
      var offset;

      if (
        typeof line === 'number' &&
        typeof column === 'number' &&
        !Number.isNaN(line) &&
        !Number.isNaN(column) &&
        line - 1 in indices
      ) {
        offset = (indices[line - 2] || 0) + column - 1 || 0;
      }

      return offset > -1 && offset < indices[indices.length - 1] ? offset : -1
    }
  }

  /**
   * Map of web namespaces.
   *
   * @type {Record<string, string>}
   */
  const webNamespaces$3 = {
    html: 'http://www.w3.org/1999/xhtml',
    mathml: 'http://www.w3.org/1998/Math/MathML',
    svg: 'http://www.w3.org/2000/svg',
    xlink: 'http://www.w3.org/1999/xlink',
    xml: 'http://www.w3.org/XML/1998/namespace',
    xmlns: 'http://www.w3.org/2000/xmlns/'
  };

  /**
   * @typedef {import('vfile').VFile} VFile
   * @typedef {import('property-information').Schema} Schema
   * @typedef {import('unist').Position} Position
   * @typedef {import('unist').Point} Point
   * @typedef {import('hast').Parent} Parent
   * @typedef {import('hast').Element} Element
   * @typedef {import('hast').Root} Root
   * @typedef {import('hast').Text} Text
   * @typedef {import('hast').Comment} Comment
   * @typedef {import('hast').DocType} Doctype
   * @typedef {Parent['children'][number]} Child
   * @typedef {Element['children'][number]} ElementChild
   * @typedef {Child|Root} Node
   * @typedef {import('parse5').Document} P5Document
   * @typedef {import('parse5').DocumentType} P5Doctype
   * @typedef {import('parse5').CommentNode} P5Comment
   * @typedef {import('parse5').TextNode} P5Text
   * @typedef {import('parse5').Element} P5Element
   * @typedef {import('parse5').ElementLocation} P5ElementLocation
   * @typedef {import('parse5').Location} P5Location
   * @typedef {import('parse5').Attribute} P5Attribute
   * @typedef {import('parse5').Node} P5Node
   *
   * @typedef {'html'|'svg'} Space
   *
   * @callback Handler
   * @param {Context} ctx
   * @param {P5Node} node
   * @param {Array.<Child>} [children]
   * @returns {Node}
   *
   * @typedef Options
   * @property {Space} [space='html'] Whether the root of the tree is in the `'html'` or `'svg'` space. If an element in with the SVG namespace is found in `ast`, `fromParse5` automatically switches to the SVG space when entering the element, and switches back when leaving
   * @property {VFile} [file] `VFile`, used to add positional information to nodes. If given, the file should have the original HTML source as its contents
   * @property {boolean} [verbose=false] Whether to add extra positional information about starting tags, closing tags, and attributes to elements. Note: not used without `file`
   *
   * @typedef Context
   * @property {Schema} schema
   * @property {VFile|undefined} file
   * @property {boolean|undefined} verbose
   * @property {boolean} location
   */

  const own$j = {}.hasOwnProperty;

  // Handlers.
  const map$1 = {
    '#document': root$4,
    '#document-fragment': root$4,
    '#text': text$4,
    '#comment': comment$4,
    '#documentType': doctype$5
  };

  /**
   * Transform Parse5’s AST to a hast tree.
   *
   * @param {P5Node} ast
   * @param {Options|VFile} [options]
   */
  function fromParse5$1(ast, options = {}) {
    /** @type {Options} */
    let settings;
    /** @type {VFile|undefined} */
    let file;

    if (isFile$1(options)) {
      file = options;
      settings = {};
    } else {
      file = options.file;
      settings = options;
    }

    return transform$2(
      {
        schema: settings.space === 'svg' ? svg$b : html$f,
        file,
        verbose: settings.verbose,
        location: false
      },
      ast
    )
  }

  /**
   * Transform children.
   *
   * @param {Context} ctx
   * @param {P5Node} ast
   * @returns {Node}
   */
  function transform$2(ctx, ast) {
    const schema = ctx.schema;
    /** @type {Handler} */
    // @ts-expect-error: index is fine.
    const fn = own$j.call(map$1, ast.nodeName) ? map$1[ast.nodeName] : element$7;
    /** @type {Array.<Child>|undefined} */
    let children;

    // Element.
    if ('tagName' in ast) {
      ctx.schema = ast.namespaceURI === webNamespaces$3.svg ? svg$b : html$f;
    }

    if ('childNodes' in ast) {
      children = nodes$1(ctx, ast.childNodes);
    }

    const result = fn(ctx, ast, children);

    if ('sourceCodeLocation' in ast && ast.sourceCodeLocation && ctx.file) {
      // @ts-expect-error It’s fine.
      const position = createLocation$1(ctx, result, ast.sourceCodeLocation);

      if (position) {
        ctx.location = true;
        result.position = position;
      }
    }

    ctx.schema = schema;

    return result
  }

  /**
   * Transform children.
   *
   * @param {Context} ctx
   * @param {Array.<P5Node>} children
   * @returns {Array.<Child>}
   */
  function nodes$1(ctx, children) {
    let index = -1;
    /** @type {Array.<Child>} */
    const result = [];

    while (++index < children.length) {
      // @ts-expect-error Assume no roots in children.
      result[index] = transform$2(ctx, children[index]);
    }

    return result
  }

  /**
   * Transform a document.
   * Stores `ast.quirksMode` in `node.data.quirksMode`.
   *
   * @type {Handler}
   * @param {P5Document} ast
   * @param {Array.<Child>} children
   * @returns {Root}
   */
  function root$4(ctx, ast, children) {
    /** @type {Root} */
    const result = {
      type: 'root',
      children,
      data: {quirksMode: ast.mode === 'quirks' || ast.mode === 'limited-quirks'}
    };

    if (ctx.file && ctx.location) {
      const doc = String(ctx.file);
      const loc = location$1(doc);
      result.position = {
        start: loc.toPoint(0),
        end: loc.toPoint(doc.length)
      };
    }

    return result
  }

  /**
   * Transform a doctype.
   *
   * @type {Handler}
   * @returns {Doctype}
   */
  function doctype$5() {
    // @ts-expect-error Types are out of date.
    return {type: 'doctype'}
  }

  /**
   * Transform a text.
   *
   * @type {Handler}
   * @param {P5Text} ast
   * @returns {Text}
   */
  function text$4(_, ast) {
    return {type: 'text', value: ast.value}
  }

  /**
   * Transform a comment.
   *
   * @type {Handler}
   * @param {P5Comment} ast
   * @returns {Comment}
   */
  function comment$4(_, ast) {
    return {type: 'comment', value: ast.data}
  }

  /**
   * Transform an element.
   *
   * @type {Handler}
   * @param {P5Element} ast
   * @param {Array.<ElementChild>} children
   * @returns {Element}
   */
  function element$7(ctx, ast, children) {
    const fn = ctx.schema.space === 'svg' ? s$2 : h$2;
    let index = -1;
    /** @type {Object.<string, string>} */
    const props = {};

    while (++index < ast.attrs.length) {
      const attribute = ast.attrs[index];
      props[(attribute.prefix ? attribute.prefix + ':' : '') + attribute.name] =
        attribute.value;
    }

    const result = fn(ast.tagName, props, children);

    if (result.tagName === 'template' && 'content' in ast) {
      const pos = ast.sourceCodeLocation;
      const startTag = pos && pos.startTag && position$1(pos.startTag);
      const endTag = pos && pos.endTag && position$1(pos.endTag);

      /** @type {Root} */
      // @ts-expect-error Types are wrong.
      const content = transform$2(ctx, ast.content);

      if (startTag && endTag && ctx.file) {
        content.position = {start: startTag.end, end: endTag.start};
      }

      result.content = content;
    }

    return result
  }

  /**
   * Create clean positional information.
   *
   * @param {Context} ctx
   * @param {Node} node
   * @param {P5ElementLocation} location
   * @returns {Position|null}
   */
  function createLocation$1(ctx, node, location) {
    const result = position$1(location);

    if (node.type === 'element') {
      const tail = node.children[node.children.length - 1];

      // Bug for unclosed with children.
      // See: <https://github.com/inikulin/parse5/issues/109>.
      if (
        result &&
        !location.endTag &&
        tail &&
        tail.position &&
        tail.position.end
      ) {
        result.end = Object.assign({}, tail.position.end);
      }

      if (ctx.verbose) {
        /** @type {Object.<string, Position|null>} */
        const props = {};
        /** @type {string} */
        let key;

        for (key in location.attrs) {
          if (own$j.call(location.attrs, key)) {
            props[find$6(ctx.schema, key).property] = position$1(location.attrs[key]);
          }
        }

        node.data = {
          position: {
            opening: position$1(location.startTag),
            closing: location.endTag ? position$1(location.endTag) : null,
            properties: props
          }
        };
      }
    }

    return result
  }

  /**
   * @param {P5Location} loc
   * @returns {Position|null}
   */
  function position$1(loc) {
    const start = point$1({
      line: loc.startLine,
      column: loc.startCol,
      offset: loc.startOffset
    });
    const end = point$1({
      line: loc.endLine,
      column: loc.endCol,
      offset: loc.endOffset
    });
    // @ts-expect-error `null` is fine.
    return start || end ? {start, end} : null
  }

  /**
   * @param {Point} point
   * @returns {Point|null}
   */
  function point$1(point) {
    return point.line && point.column ? point : null
  }

  /**
   * @param {VFile|Options} value
   * @returns {value is VFile}
   */
  function isFile$1(value) {
    return 'messages' in value
  }

  const errors = {
    abandonedHeadElementChild: {
      reason: 'Unexpected metadata element after head',
      description:
        'Unexpected element after head. Expected the element before `</head>`',
      url: false
    },
    abruptClosingOfEmptyComment: {
      reason: 'Unexpected abruptly closed empty comment',
      description: 'Unexpected `>` or `->`. Expected `-->` to close comments'
    },
    abruptDoctypePublicIdentifier: {
      reason: 'Unexpected abruptly closed public identifier',
      description:
        'Unexpected `>`. Expected a closing `"` or `\'` after the public identifier'
    },
    abruptDoctypeSystemIdentifier: {
      reason: 'Unexpected abruptly closed system identifier',
      description:
        'Unexpected `>`. Expected a closing `"` or `\'` after the identifier identifier'
    },
    absenceOfDigitsInNumericCharacterReference: {
      reason: 'Unexpected non-digit at start of numeric character reference',
      description:
        'Unexpected `%c`. Expected `[0-9]` for decimal references or `[0-9a-fA-F]` for hexadecimal references'
    },
    cdataInHtmlContent: {
      reason: 'Unexpected CDATA section in HTML',
      description:
        'Unexpected `<![CDATA[` in HTML. Remove it, use a comment, or encode special characters instead'
    },
    characterReferenceOutsideUnicodeRange: {
      reason: 'Unexpected too big numeric character reference',
      description:
        'Unexpectedly high character reference. Expected character references to be at most hexadecimal 10ffff (or decimal 1114111)'
    },
    closingOfElementWithOpenChildElements: {
      reason: 'Unexpected closing tag with open child elements',
      description:
        'Unexpectedly closing tag. Expected other tags to be closed first',
      url: false
    },
    controlCharacterInInputStream: {
      reason: 'Unexpected control character',
      description:
        'Unexpected control character `%x`. Expected a non-control code point, 0x00, or ASCII whitespace'
    },
    controlCharacterReference: {
      reason: 'Unexpected control character reference',
      description:
        'Unexpectedly control character in reference. Expected a non-control code point, 0x00, or ASCII whitespace'
    },
    disallowedContentInNoscriptInHead: {
      reason: 'Disallowed content inside `<noscript>` in `<head>`',
      description:
        'Unexpected text character `%c`. Only use text in `<noscript>`s in `<body>`',
      url: false
    },
    duplicateAttribute: {
      reason: 'Unexpected duplicate attribute',
      description:
        'Unexpectedly double attribute. Expected attributes to occur only once'
    },
    endTagWithAttributes: {
      reason: 'Unexpected attribute on closing tag',
      description: 'Unexpected attribute. Expected `>` instead'
    },
    endTagWithTrailingSolidus: {
      reason: 'Unexpected slash at end of closing tag',
      description: 'Unexpected `%c-1`. Expected `>` instead'
    },
    endTagWithoutMatchingOpenElement: {
      reason: 'Unexpected unopened end tag',
      description: 'Unexpected end tag. Expected no end tag or another end tag',
      url: false
    },
    eofBeforeTagName: {
      reason: 'Unexpected end of file',
      description: 'Unexpected end of file. Expected tag name instead'
    },
    eofInCdata: {
      reason: 'Unexpected end of file in CDATA',
      description: 'Unexpected end of file. Expected `]]>` to close the CDATA'
    },
    eofInComment: {
      reason: 'Unexpected end of file in comment',
      description: 'Unexpected end of file. Expected `-->` to close the comment'
    },
    eofInDoctype: {
      reason: 'Unexpected end of file in doctype',
      description:
        'Unexpected end of file. Expected a valid doctype (such as `<!doctype html>`)'
    },
    eofInElementThatCanContainOnlyText: {
      reason: 'Unexpected end of file in element that can only contain text',
      description: 'Unexpected end of file. Expected text or a closing tag',
      url: false
    },
    eofInScriptHtmlCommentLikeText: {
      reason: 'Unexpected end of file in comment inside script',
      description: 'Unexpected end of file. Expected `-->` to close the comment'
    },
    eofInTag: {
      reason: 'Unexpected end of file in tag',
      description: 'Unexpected end of file. Expected `>` to close the tag'
    },
    incorrectlyClosedComment: {
      reason: 'Incorrectly closed comment',
      description: 'Unexpected `%c-1`. Expected `-->` to close the comment'
    },
    incorrectlyOpenedComment: {
      reason: 'Incorrectly opened comment',
      description: 'Unexpected `%c`. Expected `<!--` to open the comment'
    },
    invalidCharacterSequenceAfterDoctypeName: {
      reason: 'Invalid sequence after doctype name',
      description: 'Unexpected sequence at `%c`. Expected `public` or `system`'
    },
    invalidFirstCharacterOfTagName: {
      reason: 'Invalid first character in tag name',
      description: 'Unexpected `%c`. Expected an ASCII letter instead'
    },
    misplacedDoctype: {
      reason: 'Misplaced doctype',
      description: 'Unexpected doctype. Expected doctype before head',
      url: false
    },
    misplacedStartTagForHeadElement: {
      reason: 'Misplaced `<head>` start tag',
      description:
        'Unexpected start tag `<head>`. Expected `<head>` directly after doctype',
      url: false
    },
    missingAttributeValue: {
      reason: 'Missing attribute value',
      description:
        'Unexpected `%c-1`. Expected an attribute value or no `%c-1` instead'
    },
    missingDoctype: {
      reason: 'Missing doctype before other content',
      description: 'Expected a `<!doctype html>` before anything else',
      url: false
    },
    missingDoctypeName: {
      reason: 'Missing doctype name',
      description: 'Unexpected doctype end at `%c`. Expected `html` instead'
    },
    missingDoctypePublicIdentifier: {
      reason: 'Missing public identifier in doctype',
      description: 'Unexpected `%c`. Expected identifier for `public` instead'
    },
    missingDoctypeSystemIdentifier: {
      reason: 'Missing system identifier in doctype',
      description:
        'Unexpected `%c`. Expected identifier for `system` instead (suggested: `"about:legacy-compat"`)'
    },
    missingEndTagName: {
      reason: 'Missing name in end tag',
      description: 'Unexpected `%c`. Expected an ASCII letter instead'
    },
    missingQuoteBeforeDoctypePublicIdentifier: {
      reason: 'Missing quote before public identifier in doctype',
      description: 'Unexpected `%c`. Expected `"` or `\'` instead'
    },
    missingQuoteBeforeDoctypeSystemIdentifier: {
      reason: 'Missing quote before system identifier in doctype',
      description: 'Unexpected `%c`. Expected `"` or `\'` instead'
    },
    missingSemicolonAfterCharacterReference: {
      reason: 'Missing semicolon after character reference',
      description: 'Unexpected `%c`. Expected `;` instead'
    },
    missingWhitespaceAfterDoctypePublicKeyword: {
      reason: 'Missing whitespace after public identifier in doctype',
      description: 'Unexpected `%c`. Expected ASCII whitespace instead'
    },
    missingWhitespaceAfterDoctypeSystemKeyword: {
      reason: 'Missing whitespace after system identifier in doctype',
      description: 'Unexpected `%c`. Expected ASCII whitespace instead'
    },
    missingWhitespaceBeforeDoctypeName: {
      reason: 'Missing whitespace before doctype name',
      description: 'Unexpected `%c`. Expected ASCII whitespace instead'
    },
    missingWhitespaceBetweenAttributes: {
      reason: 'Missing whitespace between attributes',
      description: 'Unexpected `%c`. Expected ASCII whitespace instead'
    },
    missingWhitespaceBetweenDoctypePublicAndSystemIdentifiers: {
      reason:
        'Missing whitespace between public and system identifiers in doctype',
      description: 'Unexpected `%c`. Expected ASCII whitespace instead'
    },
    nestedComment: {
      reason: 'Unexpected nested comment',
      description: 'Unexpected `<!--`. Expected `-->`'
    },
    nestedNoscriptInHead: {
      reason: 'Unexpected nested `<noscript>` in `<head>`',
      description:
        'Unexpected `<noscript>`. Expected a closing tag or a meta element',
      url: false
    },
    nonConformingDoctype: {
      reason: 'Unexpected non-conforming doctype declaration',
      description:
        'Expected `<!doctype html>` or `<!doctype html system "about:legacy-compat">`',
      url: false
    },
    nonVoidHtmlElementStartTagWithTrailingSolidus: {
      reason: 'Unexpected trailing slash on start tag of non-void element',
      description: 'Unexpected `/`. Expected `>` instead'
    },
    noncharacterCharacterReference: {
      reason:
        'Unexpected noncharacter code point referenced by character reference',
      description: 'Unexpected code point. Do not use noncharacters in HTML'
    },
    noncharacterInInputStream: {
      reason: 'Unexpected noncharacter character',
      description: 'Unexpected code point `%x`. Do not use noncharacters in HTML'
    },
    nullCharacterReference: {
      reason: 'Unexpected NULL character referenced by character reference',
      description: 'Unexpected code point. Do not use NULL characters in HTML'
    },
    openElementsLeftAfterEof: {
      reason: 'Unexpected end of file',
      description: 'Unexpected end of file. Expected closing tag instead',
      url: false
    },
    surrogateCharacterReference: {
      reason: 'Unexpected surrogate character referenced by character reference',
      description:
        'Unexpected code point. Do not use lone surrogate characters in HTML'
    },
    surrogateInInputStream: {
      reason: 'Unexpected surrogate character',
      description:
        'Unexpected code point `%x`. Do not use lone surrogate characters in HTML'
    },
    unexpectedCharacterAfterDoctypeSystemIdentifier: {
      reason: 'Invalid character after system identifier in doctype',
      description: 'Unexpected character at `%c`. Expected `>`'
    },
    unexpectedCharacterInAttributeName: {
      reason: 'Unexpected character in attribute name',
      description:
        'Unexpected `%c`. Expected whitespace, `/`, `>`, `=`, or probably an ASCII letter'
    },
    unexpectedCharacterInUnquotedAttributeValue: {
      reason: 'Unexpected character in unquoted attribute value',
      description: 'Unexpected `%c`. Quote the attribute value to include it'
    },
    unexpectedEqualsSignBeforeAttributeName: {
      reason: 'Unexpected equals sign before attribute name',
      description: 'Unexpected `%c`. Add an attribute name before it'
    },
    unexpectedNullCharacter: {
      reason: 'Unexpected NULL character',
      description:
        'Unexpected code point `%x`. Do not use NULL characters in HTML'
    },
    unexpectedQuestionMarkInsteadOfTagName: {
      reason: 'Unexpected question mark instead of tag name',
      description: 'Unexpected `%c`. Expected an ASCII letter instead'
    },
    unexpectedSolidusInTag: {
      reason: 'Unexpected slash in tag',
      description:
        'Unexpected `%c-1`. Expected it followed by `>` or in a quoted attribute value'
    },
    unknownNamedCharacterReference: {
      reason: 'Unexpected unknown named character reference',
      description:
        'Unexpected character reference. Expected known named character references'
    }
  };

  /**
   * @typedef {import('hast').Root} Root
   * @typedef {Pick<import('hast-util-from-parse5').Options, 'space' | 'verbose'>} FromParse5Options
   *
   * @typedef {keyof errors} ErrorCode
   * @typedef {0|1|2|boolean|null|undefined} ErrorSeverity
   * @typedef {Partial<Record<ErrorCode, ErrorSeverity>>} ErrorFields
   *
   * @typedef ParseFields
   * @property {boolean|undefined} [fragment=false]
   *   Specify whether to parse a fragment, instead of a complete document.
   *   In document mode, unopened `html`, `head`, and `body` elements are opened
   *   in just the right places.
   * @property {boolean|undefined} [emitParseErrors=false]
   *   > ⚠️ Parse errors are currently being added to HTML.
   *   > Not all errors emitted by parse5 (or rehype-parse) are specced yet.
   *   > Some documentation may still be missing.
   *
   *   Emit parse errors while parsing on the vfile.
   *   Setting this to `true` starts emitting HTML parse errors.
   *
   *   Specific rules can be turned off by setting them to `false` (or `0`).
   *   The default, when `emitParseErrors: true`, is `true` (or `1`), and means
   *   that rules emit as warnings.
   *   Rules can also be configured with `2`, to turn them into fatal errors.
   *
   * @typedef {FromParse5Options & ParseFields & ErrorFields} Options
   */

  const base = 'https://html.spec.whatwg.org/multipage/parsing.html#parse-error-';

  const fatalities = {2: true, 1: false, 0: null};

  /** @type {import('unified').Plugin<[Options?] | Array<void>, string, Root>} */
  function rehypeParse(options) {
    const processorSettings = /** @type {Options} */ (this.data('settings'));
    const settings = Object.assign({}, processorSettings, options);

    Object.assign(this, {Parser: parser});

    /** @type {import('unified').ParserFunction<Root>} */
    function parser(doc, file) {
      const fn = settings.fragment ? 'parseFragment' : 'parse';
      const onParseError = settings.emitParseErrors ? onerror : null;
      const parse5 = new Parser5({
        sourceCodeLocationInfo: true,
        onParseError,
        scriptingEnabled: false
      });

      // @ts-expect-error: `parse5` returns document or fragment, which are always
      // mapped to roots.
      return fromParse5$1(parse5[fn](doc), {
        space: settings.space,
        file,
        verbose: settings.verbose
      })

      /**
       * @param {{code: string, startLine: number, startCol: number, startOffset: number, endLine: number, endCol: number, endOffset: number}} error
       */
      function onerror(error) {
        const code = error.code;
        const name = camelcase$6(code);
        const setting = settings[name];
        const config = setting === undefined || setting === null ? true : setting;
        const level = typeof config === 'number' ? config : config ? 1 : 0;
        const start = {
          line: error.startLine,
          column: error.startCol,
          offset: error.startOffset
        };
        const end = {
          line: error.endLine,
          column: error.endCol,
          offset: error.endOffset
        };
        if (level) {
          /* c8 ignore next */
          const info = errors[name] || {reason: '', description: '', url: ''};
          const message = file.message(format(info.reason), {start, end});
          message.source = 'parse-error';
          message.ruleId = code;
          message.fatal = fatalities[level];
          message.note = format(info.description);
          message.url = 'url' in info && info.url === false ? null : base + code;
        }

        /**
         * @param {string} value
         * @returns {string}
         */
        function format(value) {
          return value
            .replace(/%c(?:-(\d+))?/g, (_, /** @type {string} */ $1) => {
              const offset = $1 ? -Number.parseInt($1, 10) : 0;
              const char = doc.charAt(error.startOffset + offset);
              return char === '`' ? '` ` `' : char
            })
            .replace(
              /%x/g,
              () =>
                '0x' +
                doc.charCodeAt(error.startOffset).toString(16).toUpperCase()
            )
        }
      }
    }
  }

  /**
   * @param {string} value
   * @returns {ErrorCode}
   */
  function camelcase$6(value) {
    // @ts-expect-error: this returns a valid error code.
    return value.replace(/-[a-z]/g, ($0) => $0.charAt(1).toUpperCase())
  }

  /**
   * @typedef {import('hast').Root} Root
   * @typedef {import('katex').KatexOptions} Options
   */

  const assign = Object.assign;

  const parseHtml = unified().use(rehypeParse, {fragment: true});

  const source = 'rehype-katex';

  /**
   * Plugin to transform `<span class=math-inline>` and `<div class=math-display>`
   * with KaTeX.
   *
   * @type {import('unified').Plugin<[Options?]|void[], Root>}
   */
  function rehypeKatex(options) {
    const settings = options || {};
    const throwOnError = settings.throwOnError || false;

    return (tree, file) => {
      visit(tree, 'element', (element) => {
        const classes =
          element.properties && Array.isArray(element.properties.className)
            ? element.properties.className
            : [];
        const inline = classes.includes('math-inline');
        const displayMode = classes.includes('math-display');

        if (!inline && !displayMode) {
          return
        }

        const value = toText(element, {whitespace: 'pre'});

        /** @type {string} */
        let result;

        try {
          result = katex.renderToString(
            value,
            assign({}, settings, {displayMode, throwOnError: true})
          );
        } catch (error_) {
          const error = /** @type {Error} */ (error_);
          const fn = throwOnError ? 'fail' : 'message';
          const origin = [source, error.name.toLowerCase()].join(':');

          file[fn](error.message, element.position, origin);

          result = katex.renderToString(
            value,
            assign({}, settings, {
              displayMode,
              throwOnError: false,
              strict: 'ignore'
            })
          );
        }

        // @ts-expect-error: assume no `doctypes` in KaTeX result.
        element.children = removePosition(parseHtml.parse(result), true).children;
      });
    }
  }

  var unicode$3 = {};

  const UNDEFINED_CODE_POINTS = [
      0xfffe,
      0xffff,
      0x1fffe,
      0x1ffff,
      0x2fffe,
      0x2ffff,
      0x3fffe,
      0x3ffff,
      0x4fffe,
      0x4ffff,
      0x5fffe,
      0x5ffff,
      0x6fffe,
      0x6ffff,
      0x7fffe,
      0x7ffff,
      0x8fffe,
      0x8ffff,
      0x9fffe,
      0x9ffff,
      0xafffe,
      0xaffff,
      0xbfffe,
      0xbffff,
      0xcfffe,
      0xcffff,
      0xdfffe,
      0xdffff,
      0xefffe,
      0xeffff,
      0xffffe,
      0xfffff,
      0x10fffe,
      0x10ffff
  ];

  unicode$3.REPLACEMENT_CHARACTER = '\uFFFD';

  unicode$3.CODE_POINTS = {
      EOF: -1,
      NULL: 0x00,
      TABULATION: 0x09,
      CARRIAGE_RETURN: 0x0d,
      LINE_FEED: 0x0a,
      FORM_FEED: 0x0c,
      SPACE: 0x20,
      EXCLAMATION_MARK: 0x21,
      QUOTATION_MARK: 0x22,
      NUMBER_SIGN: 0x23,
      AMPERSAND: 0x26,
      APOSTROPHE: 0x27,
      HYPHEN_MINUS: 0x2d,
      SOLIDUS: 0x2f,
      DIGIT_0: 0x30,
      DIGIT_9: 0x39,
      SEMICOLON: 0x3b,
      LESS_THAN_SIGN: 0x3c,
      EQUALS_SIGN: 0x3d,
      GREATER_THAN_SIGN: 0x3e,
      QUESTION_MARK: 0x3f,
      LATIN_CAPITAL_A: 0x41,
      LATIN_CAPITAL_F: 0x46,
      LATIN_CAPITAL_X: 0x58,
      LATIN_CAPITAL_Z: 0x5a,
      RIGHT_SQUARE_BRACKET: 0x5d,
      GRAVE_ACCENT: 0x60,
      LATIN_SMALL_A: 0x61,
      LATIN_SMALL_F: 0x66,
      LATIN_SMALL_X: 0x78,
      LATIN_SMALL_Z: 0x7a,
      REPLACEMENT_CHARACTER: 0xfffd
  };

  unicode$3.CODE_POINT_SEQUENCES = {
      DASH_DASH_STRING: [0x2d, 0x2d], //--
      DOCTYPE_STRING: [0x44, 0x4f, 0x43, 0x54, 0x59, 0x50, 0x45], //DOCTYPE
      CDATA_START_STRING: [0x5b, 0x43, 0x44, 0x41, 0x54, 0x41, 0x5b], //[CDATA[
      SCRIPT_STRING: [0x73, 0x63, 0x72, 0x69, 0x70, 0x74], //script
      PUBLIC_STRING: [0x50, 0x55, 0x42, 0x4c, 0x49, 0x43], //PUBLIC
      SYSTEM_STRING: [0x53, 0x59, 0x53, 0x54, 0x45, 0x4d] //SYSTEM
  };

  //Surrogates
  unicode$3.isSurrogate = function(cp) {
      return cp >= 0xd800 && cp <= 0xdfff;
  };

  unicode$3.isSurrogatePair = function(cp) {
      return cp >= 0xdc00 && cp <= 0xdfff;
  };

  unicode$3.getSurrogatePairCodePoint = function(cp1, cp2) {
      return (cp1 - 0xd800) * 0x400 + 0x2400 + cp2;
  };

  //NOTE: excluding NULL and ASCII whitespace
  unicode$3.isControlCodePoint = function(cp) {
      return (
          (cp !== 0x20 && cp !== 0x0a && cp !== 0x0d && cp !== 0x09 && cp !== 0x0c && cp >= 0x01 && cp <= 0x1f) ||
          (cp >= 0x7f && cp <= 0x9f)
      );
  };

  unicode$3.isUndefinedCodePoint = function(cp) {
      return (cp >= 0xfdd0 && cp <= 0xfdef) || UNDEFINED_CODE_POINTS.indexOf(cp) > -1;
  };

  var errorCodes = {
      controlCharacterInInputStream: 'control-character-in-input-stream',
      noncharacterInInputStream: 'noncharacter-in-input-stream',
      surrogateInInputStream: 'surrogate-in-input-stream',
      nonVoidHtmlElementStartTagWithTrailingSolidus: 'non-void-html-element-start-tag-with-trailing-solidus',
      endTagWithAttributes: 'end-tag-with-attributes',
      endTagWithTrailingSolidus: 'end-tag-with-trailing-solidus',
      unexpectedSolidusInTag: 'unexpected-solidus-in-tag',
      unexpectedNullCharacter: 'unexpected-null-character',
      unexpectedQuestionMarkInsteadOfTagName: 'unexpected-question-mark-instead-of-tag-name',
      invalidFirstCharacterOfTagName: 'invalid-first-character-of-tag-name',
      unexpectedEqualsSignBeforeAttributeName: 'unexpected-equals-sign-before-attribute-name',
      missingEndTagName: 'missing-end-tag-name',
      unexpectedCharacterInAttributeName: 'unexpected-character-in-attribute-name',
      unknownNamedCharacterReference: 'unknown-named-character-reference',
      missingSemicolonAfterCharacterReference: 'missing-semicolon-after-character-reference',
      unexpectedCharacterAfterDoctypeSystemIdentifier: 'unexpected-character-after-doctype-system-identifier',
      unexpectedCharacterInUnquotedAttributeValue: 'unexpected-character-in-unquoted-attribute-value',
      eofBeforeTagName: 'eof-before-tag-name',
      eofInTag: 'eof-in-tag',
      missingAttributeValue: 'missing-attribute-value',
      missingWhitespaceBetweenAttributes: 'missing-whitespace-between-attributes',
      missingWhitespaceAfterDoctypePublicKeyword: 'missing-whitespace-after-doctype-public-keyword',
      missingWhitespaceBetweenDoctypePublicAndSystemIdentifiers:
          'missing-whitespace-between-doctype-public-and-system-identifiers',
      missingWhitespaceAfterDoctypeSystemKeyword: 'missing-whitespace-after-doctype-system-keyword',
      missingQuoteBeforeDoctypePublicIdentifier: 'missing-quote-before-doctype-public-identifier',
      missingQuoteBeforeDoctypeSystemIdentifier: 'missing-quote-before-doctype-system-identifier',
      missingDoctypePublicIdentifier: 'missing-doctype-public-identifier',
      missingDoctypeSystemIdentifier: 'missing-doctype-system-identifier',
      abruptDoctypePublicIdentifier: 'abrupt-doctype-public-identifier',
      abruptDoctypeSystemIdentifier: 'abrupt-doctype-system-identifier',
      cdataInHtmlContent: 'cdata-in-html-content',
      incorrectlyOpenedComment: 'incorrectly-opened-comment',
      eofInScriptHtmlCommentLikeText: 'eof-in-script-html-comment-like-text',
      eofInDoctype: 'eof-in-doctype',
      nestedComment: 'nested-comment',
      abruptClosingOfEmptyComment: 'abrupt-closing-of-empty-comment',
      eofInComment: 'eof-in-comment',
      incorrectlyClosedComment: 'incorrectly-closed-comment',
      eofInCdata: 'eof-in-cdata',
      absenceOfDigitsInNumericCharacterReference: 'absence-of-digits-in-numeric-character-reference',
      nullCharacterReference: 'null-character-reference',
      surrogateCharacterReference: 'surrogate-character-reference',
      characterReferenceOutsideUnicodeRange: 'character-reference-outside-unicode-range',
      controlCharacterReference: 'control-character-reference',
      noncharacterCharacterReference: 'noncharacter-character-reference',
      missingWhitespaceBeforeDoctypeName: 'missing-whitespace-before-doctype-name',
      missingDoctypeName: 'missing-doctype-name',
      invalidCharacterSequenceAfterDoctypeName: 'invalid-character-sequence-after-doctype-name',
      duplicateAttribute: 'duplicate-attribute',
      nonConformingDoctype: 'non-conforming-doctype',
      missingDoctype: 'missing-doctype',
      misplacedDoctype: 'misplaced-doctype',
      endTagWithoutMatchingOpenElement: 'end-tag-without-matching-open-element',
      closingOfElementWithOpenChildElements: 'closing-of-element-with-open-child-elements',
      disallowedContentInNoscriptInHead: 'disallowed-content-in-noscript-in-head',
      openElementsLeftAfterEof: 'open-elements-left-after-eof',
      abandonedHeadElementChild: 'abandoned-head-element-child',
      misplacedStartTagForHeadElement: 'misplaced-start-tag-for-head-element',
      nestedNoscriptInHead: 'nested-noscript-in-head',
      eofInElementThatCanContainOnlyText: 'eof-in-element-that-can-contain-only-text'
  };

  const unicode$2 = unicode$3;
  const ERR$2 = errorCodes;

  //Aliases
  const $$6 = unicode$2.CODE_POINTS;

  //Const
  const DEFAULT_BUFFER_WATERLINE = 1 << 16;

  //Preprocessor
  //NOTE: HTML input preprocessing
  //(see: http://www.whatwg.org/specs/web-apps/current-work/multipage/parsing.html#preprocessing-the-input-stream)
  class Preprocessor$1 {
      constructor() {
          this.html = null;

          this.pos = -1;
          this.lastGapPos = -1;
          this.lastCharPos = -1;

          this.gapStack = [];

          this.skipNextNewLine = false;

          this.lastChunkWritten = false;
          this.endOfChunkHit = false;
          this.bufferWaterline = DEFAULT_BUFFER_WATERLINE;
      }

      _err() {
          // NOTE: err reporting is noop by default. Enabled by mixin.
      }

      _addGap() {
          this.gapStack.push(this.lastGapPos);
          this.lastGapPos = this.pos;
      }

      _processSurrogate(cp) {
          //NOTE: try to peek a surrogate pair
          if (this.pos !== this.lastCharPos) {
              const nextCp = this.html.charCodeAt(this.pos + 1);

              if (unicode$2.isSurrogatePair(nextCp)) {
                  //NOTE: we have a surrogate pair. Peek pair character and recalculate code point.
                  this.pos++;

                  //NOTE: add gap that should be avoided during retreat
                  this._addGap();

                  return unicode$2.getSurrogatePairCodePoint(cp, nextCp);
              }
          }

          //NOTE: we are at the end of a chunk, therefore we can't infer surrogate pair yet.
          else if (!this.lastChunkWritten) {
              this.endOfChunkHit = true;
              return $$6.EOF;
          }

          //NOTE: isolated surrogate
          this._err(ERR$2.surrogateInInputStream);

          return cp;
      }

      dropParsedChunk() {
          if (this.pos > this.bufferWaterline) {
              this.lastCharPos -= this.pos;
              this.html = this.html.substring(this.pos);
              this.pos = 0;
              this.lastGapPos = -1;
              this.gapStack = [];
          }
      }

      write(chunk, isLastChunk) {
          if (this.html) {
              this.html += chunk;
          } else {
              this.html = chunk;
          }

          this.lastCharPos = this.html.length - 1;
          this.endOfChunkHit = false;
          this.lastChunkWritten = isLastChunk;
      }

      insertHtmlAtCurrentPos(chunk) {
          this.html = this.html.substring(0, this.pos + 1) + chunk + this.html.substring(this.pos + 1, this.html.length);

          this.lastCharPos = this.html.length - 1;
          this.endOfChunkHit = false;
      }

      advance() {
          this.pos++;

          if (this.pos > this.lastCharPos) {
              this.endOfChunkHit = !this.lastChunkWritten;
              return $$6.EOF;
          }

          let cp = this.html.charCodeAt(this.pos);

          //NOTE: any U+000A LINE FEED (LF) characters that immediately follow a U+000D CARRIAGE RETURN (CR) character
          //must be ignored.
          if (this.skipNextNewLine && cp === $$6.LINE_FEED) {
              this.skipNextNewLine = false;
              this._addGap();
              return this.advance();
          }

          //NOTE: all U+000D CARRIAGE RETURN (CR) characters must be converted to U+000A LINE FEED (LF) characters
          if (cp === $$6.CARRIAGE_RETURN) {
              this.skipNextNewLine = true;
              return $$6.LINE_FEED;
          }

          this.skipNextNewLine = false;

          if (unicode$2.isSurrogate(cp)) {
              cp = this._processSurrogate(cp);
          }

          //OPTIMIZATION: first check if code point is in the common allowed
          //range (ASCII alphanumeric, whitespaces, big chunk of BMP)
          //before going into detailed performance cost validation.
          const isCommonValidRange =
              (cp > 0x1f && cp < 0x7f) || cp === $$6.LINE_FEED || cp === $$6.CARRIAGE_RETURN || (cp > 0x9f && cp < 0xfdd0);

          if (!isCommonValidRange) {
              this._checkForProblematicCharacters(cp);
          }

          return cp;
      }

      _checkForProblematicCharacters(cp) {
          if (unicode$2.isControlCodePoint(cp)) {
              this._err(ERR$2.controlCharacterInInputStream);
          } else if (unicode$2.isUndefinedCodePoint(cp)) {
              this._err(ERR$2.noncharacterInInputStream);
          }
      }

      retreat() {
          if (this.pos === this.lastGapPos) {
              this.lastGapPos = this.gapStack.pop();
              this.pos--;
          }

          this.pos--;
      }
  }

  var preprocessor = Preprocessor$1;

  //NOTE: this file contains auto-generated array mapped radix tree that is used for the named entity references consumption
  //(details: https://github.com/inikulin/parse5/tree/master/scripts/generate-named-entity-data/README.md)
  var namedEntityData = new Uint16Array([4,52,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,106,303,412,810,1432,1701,1796,1987,2114,2360,2420,2484,3170,3251,4140,4393,4575,4610,5106,5512,5728,6117,6274,6315,6345,6427,6516,7002,7910,8733,9323,9870,10170,10631,10893,11318,11386,11467,12773,13092,14474,14922,15448,15542,16419,17666,18166,18611,19004,19095,19298,19397,4,16,69,77,97,98,99,102,103,108,109,110,111,112,114,115,116,117,140,150,158,169,176,194,199,210,216,222,226,242,256,266,283,294,108,105,103,5,198,1,59,148,1,198,80,5,38,1,59,156,1,38,99,117,116,101,5,193,1,59,167,1,193,114,101,118,101,59,1,258,4,2,105,121,182,191,114,99,5,194,1,59,189,1,194,59,1,1040,114,59,3,55349,56580,114,97,118,101,5,192,1,59,208,1,192,112,104,97,59,1,913,97,99,114,59,1,256,100,59,1,10835,4,2,103,112,232,237,111,110,59,1,260,102,59,3,55349,56632,112,108,121,70,117,110,99,116,105,111,110,59,1,8289,105,110,103,5,197,1,59,264,1,197,4,2,99,115,272,277,114,59,3,55349,56476,105,103,110,59,1,8788,105,108,100,101,5,195,1,59,292,1,195,109,108,5,196,1,59,301,1,196,4,8,97,99,101,102,111,114,115,117,321,350,354,383,388,394,400,405,4,2,99,114,327,336,107,115,108,97,115,104,59,1,8726,4,2,118,119,342,345,59,1,10983,101,100,59,1,8966,121,59,1,1041,4,3,99,114,116,362,369,379,97,117,115,101,59,1,8757,110,111,117,108,108,105,115,59,1,8492,97,59,1,914,114,59,3,55349,56581,112,102,59,3,55349,56633,101,118,101,59,1,728,99,114,59,1,8492,109,112,101,113,59,1,8782,4,14,72,79,97,99,100,101,102,104,105,108,111,114,115,117,442,447,456,504,542,547,569,573,577,616,678,784,790,796,99,121,59,1,1063,80,89,5,169,1,59,454,1,169,4,3,99,112,121,464,470,497,117,116,101,59,1,262,4,2,59,105,476,478,1,8914,116,97,108,68,105,102,102,101,114,101,110,116,105,97,108,68,59,1,8517,108,101,121,115,59,1,8493,4,4,97,101,105,111,514,520,530,535,114,111,110,59,1,268,100,105,108,5,199,1,59,528,1,199,114,99,59,1,264,110,105,110,116,59,1,8752,111,116,59,1,266,4,2,100,110,553,560,105,108,108,97,59,1,184,116,101,114,68,111,116,59,1,183,114,59,1,8493,105,59,1,935,114,99,108,101,4,4,68,77,80,84,591,596,603,609,111,116,59,1,8857,105,110,117,115,59,1,8854,108,117,115,59,1,8853,105,109,101,115,59,1,8855,111,4,2,99,115,623,646,107,119,105,115,101,67,111,110,116,111,117,114,73,110,116,101,103,114,97,108,59,1,8754,101,67,117,114,108,121,4,2,68,81,658,671,111,117,98,108,101,81,117,111,116,101,59,1,8221,117,111,116,101,59,1,8217,4,4,108,110,112,117,688,701,736,753,111,110,4,2,59,101,696,698,1,8759,59,1,10868,4,3,103,105,116,709,717,722,114,117,101,110,116,59,1,8801,110,116,59,1,8751,111,117,114,73,110,116,101,103,114,97,108,59,1,8750,4,2,102,114,742,745,59,1,8450,111,100,117,99,116,59,1,8720,110,116,101,114,67,108,111,99,107,119,105,115,101,67,111,110,116,111,117,114,73,110,116,101,103,114,97,108,59,1,8755,111,115,115,59,1,10799,99,114,59,3,55349,56478,112,4,2,59,67,803,805,1,8915,97,112,59,1,8781,4,11,68,74,83,90,97,99,101,102,105,111,115,834,850,855,860,865,888,903,916,921,1011,1415,4,2,59,111,840,842,1,8517,116,114,97,104,100,59,1,10513,99,121,59,1,1026,99,121,59,1,1029,99,121,59,1,1039,4,3,103,114,115,873,879,883,103,101,114,59,1,8225,114,59,1,8609,104,118,59,1,10980,4,2,97,121,894,900,114,111,110,59,1,270,59,1,1044,108,4,2,59,116,910,912,1,8711,97,59,1,916,114,59,3,55349,56583,4,2,97,102,927,998,4,2,99,109,933,992,114,105,116,105,99,97,108,4,4,65,68,71,84,950,957,978,985,99,117,116,101,59,1,180,111,4,2,116,117,964,967,59,1,729,98,108,101,65,99,117,116,101,59,1,733,114,97,118,101,59,1,96,105,108,100,101,59,1,732,111,110,100,59,1,8900,102,101,114,101,110,116,105,97,108,68,59,1,8518,4,4,112,116,117,119,1021,1026,1048,1249,102,59,3,55349,56635,4,3,59,68,69,1034,1036,1041,1,168,111,116,59,1,8412,113,117,97,108,59,1,8784,98,108,101,4,6,67,68,76,82,85,86,1065,1082,1101,1189,1211,1236,111,110,116,111,117,114,73,110,116,101,103,114,97,108,59,1,8751,111,4,2,116,119,1089,1092,59,1,168,110,65,114,114,111,119,59,1,8659,4,2,101,111,1107,1141,102,116,4,3,65,82,84,1117,1124,1136,114,114,111,119,59,1,8656,105,103,104,116,65,114,114,111,119,59,1,8660,101,101,59,1,10980,110,103,4,2,76,82,1149,1177,101,102,116,4,2,65,82,1158,1165,114,114,111,119,59,1,10232,105,103,104,116,65,114,114,111,119,59,1,10234,105,103,104,116,65,114,114,111,119,59,1,10233,105,103,104,116,4,2,65,84,1199,1206,114,114,111,119,59,1,8658,101,101,59,1,8872,112,4,2,65,68,1218,1225,114,114,111,119,59,1,8657,111,119,110,65,114,114,111,119,59,1,8661,101,114,116,105,99,97,108,66,97,114,59,1,8741,110,4,6,65,66,76,82,84,97,1264,1292,1299,1352,1391,1408,114,114,111,119,4,3,59,66,85,1276,1278,1283,1,8595,97,114,59,1,10515,112,65,114,114,111,119,59,1,8693,114,101,118,101,59,1,785,101,102,116,4,3,82,84,86,1310,1323,1334,105,103,104,116,86,101,99,116,111,114,59,1,10576,101,101,86,101,99,116,111,114,59,1,10590,101,99,116,111,114,4,2,59,66,1345,1347,1,8637,97,114,59,1,10582,105,103,104,116,4,2,84,86,1362,1373,101,101,86,101,99,116,111,114,59,1,10591,101,99,116,111,114,4,2,59,66,1384,1386,1,8641,97,114,59,1,10583,101,101,4,2,59,65,1399,1401,1,8868,114,114,111,119,59,1,8615,114,114,111,119,59,1,8659,4,2,99,116,1421,1426,114,59,3,55349,56479,114,111,107,59,1,272,4,16,78,84,97,99,100,102,103,108,109,111,112,113,115,116,117,120,1466,1470,1478,1489,1515,1520,1525,1536,1544,1593,1609,1617,1650,1664,1668,1677,71,59,1,330,72,5,208,1,59,1476,1,208,99,117,116,101,5,201,1,59,1487,1,201,4,3,97,105,121,1497,1503,1512,114,111,110,59,1,282,114,99,5,202,1,59,1510,1,202,59,1,1069,111,116,59,1,278,114,59,3,55349,56584,114,97,118,101,5,200,1,59,1534,1,200,101,109,101,110,116,59,1,8712,4,2,97,112,1550,1555,99,114,59,1,274,116,121,4,2,83,86,1563,1576,109,97,108,108,83,113,117,97,114,101,59,1,9723,101,114,121,83,109,97,108,108,83,113,117,97,114,101,59,1,9643,4,2,103,112,1599,1604,111,110,59,1,280,102,59,3,55349,56636,115,105,108,111,110,59,1,917,117,4,2,97,105,1624,1640,108,4,2,59,84,1631,1633,1,10869,105,108,100,101,59,1,8770,108,105,98,114,105,117,109,59,1,8652,4,2,99,105,1656,1660,114,59,1,8496,109,59,1,10867,97,59,1,919,109,108,5,203,1,59,1675,1,203,4,2,105,112,1683,1689,115,116,115,59,1,8707,111,110,101,110,116,105,97,108,69,59,1,8519,4,5,99,102,105,111,115,1713,1717,1722,1762,1791,121,59,1,1060,114,59,3,55349,56585,108,108,101,100,4,2,83,86,1732,1745,109,97,108,108,83,113,117,97,114,101,59,1,9724,101,114,121,83,109,97,108,108,83,113,117,97,114,101,59,1,9642,4,3,112,114,117,1770,1775,1781,102,59,3,55349,56637,65,108,108,59,1,8704,114,105,101,114,116,114,102,59,1,8497,99,114,59,1,8497,4,12,74,84,97,98,99,100,102,103,111,114,115,116,1822,1827,1834,1848,1855,1877,1882,1887,1890,1896,1978,1984,99,121,59,1,1027,5,62,1,59,1832,1,62,109,109,97,4,2,59,100,1843,1845,1,915,59,1,988,114,101,118,101,59,1,286,4,3,101,105,121,1863,1869,1874,100,105,108,59,1,290,114,99,59,1,284,59,1,1043,111,116,59,1,288,114,59,3,55349,56586,59,1,8921,112,102,59,3,55349,56638,101,97,116,101,114,4,6,69,70,71,76,83,84,1915,1933,1944,1953,1959,1971,113,117,97,108,4,2,59,76,1925,1927,1,8805,101,115,115,59,1,8923,117,108,108,69,113,117,97,108,59,1,8807,114,101,97,116,101,114,59,1,10914,101,115,115,59,1,8823,108,97,110,116,69,113,117,97,108,59,1,10878,105,108,100,101,59,1,8819,99,114,59,3,55349,56482,59,1,8811,4,8,65,97,99,102,105,111,115,117,2005,2012,2026,2032,2036,2049,2073,2089,82,68,99,121,59,1,1066,4,2,99,116,2018,2023,101,107,59,1,711,59,1,94,105,114,99,59,1,292,114,59,1,8460,108,98,101,114,116,83,112,97,99,101,59,1,8459,4,2,112,114,2055,2059,102,59,1,8461,105,122,111,110,116,97,108,76,105,110,101,59,1,9472,4,2,99,116,2079,2083,114,59,1,8459,114,111,107,59,1,294,109,112,4,2,68,69,2097,2107,111,119,110,72,117,109,112,59,1,8782,113,117,97,108,59,1,8783,4,14,69,74,79,97,99,100,102,103,109,110,111,115,116,117,2144,2149,2155,2160,2171,2189,2194,2198,2209,2245,2307,2329,2334,2341,99,121,59,1,1045,108,105,103,59,1,306,99,121,59,1,1025,99,117,116,101,5,205,1,59,2169,1,205,4,2,105,121,2177,2186,114,99,5,206,1,59,2184,1,206,59,1,1048,111,116,59,1,304,114,59,1,8465,114,97,118,101,5,204,1,59,2207,1,204,4,3,59,97,112,2217,2219,2238,1,8465,4,2,99,103,2225,2229,114,59,1,298,105,110,97,114,121,73,59,1,8520,108,105,101,115,59,1,8658,4,2,116,118,2251,2281,4,2,59,101,2257,2259,1,8748,4,2,103,114,2265,2271,114,97,108,59,1,8747,115,101,99,116,105,111,110,59,1,8898,105,115,105,98,108,101,4,2,67,84,2293,2300,111,109,109,97,59,1,8291,105,109,101,115,59,1,8290,4,3,103,112,116,2315,2320,2325,111,110,59,1,302,102,59,3,55349,56640,97,59,1,921,99,114,59,1,8464,105,108,100,101,59,1,296,4,2,107,109,2347,2352,99,121,59,1,1030,108,5,207,1,59,2358,1,207,4,5,99,102,111,115,117,2372,2386,2391,2397,2414,4,2,105,121,2378,2383,114,99,59,1,308,59,1,1049,114,59,3,55349,56589,112,102,59,3,55349,56641,4,2,99,101,2403,2408,114,59,3,55349,56485,114,99,121,59,1,1032,107,99,121,59,1,1028,4,7,72,74,97,99,102,111,115,2436,2441,2446,2452,2467,2472,2478,99,121,59,1,1061,99,121,59,1,1036,112,112,97,59,1,922,4,2,101,121,2458,2464,100,105,108,59,1,310,59,1,1050,114,59,3,55349,56590,112,102,59,3,55349,56642,99,114,59,3,55349,56486,4,11,74,84,97,99,101,102,108,109,111,115,116,2508,2513,2520,2562,2585,2981,2986,3004,3011,3146,3167,99,121,59,1,1033,5,60,1,59,2518,1,60,4,5,99,109,110,112,114,2532,2538,2544,2548,2558,117,116,101,59,1,313,98,100,97,59,1,923,103,59,1,10218,108,97,99,101,116,114,102,59,1,8466,114,59,1,8606,4,3,97,101,121,2570,2576,2582,114,111,110,59,1,317,100,105,108,59,1,315,59,1,1051,4,2,102,115,2591,2907,116,4,10,65,67,68,70,82,84,85,86,97,114,2614,2663,2672,2728,2735,2760,2820,2870,2888,2895,4,2,110,114,2620,2633,103,108,101,66,114,97,99,107,101,116,59,1,10216,114,111,119,4,3,59,66,82,2644,2646,2651,1,8592,97,114,59,1,8676,105,103,104,116,65,114,114,111,119,59,1,8646,101,105,108,105,110,103,59,1,8968,111,4,2,117,119,2679,2692,98,108,101,66,114,97,99,107,101,116,59,1,10214,110,4,2,84,86,2699,2710,101,101,86,101,99,116,111,114,59,1,10593,101,99,116,111,114,4,2,59,66,2721,2723,1,8643,97,114,59,1,10585,108,111,111,114,59,1,8970,105,103,104,116,4,2,65,86,2745,2752,114,114,111,119,59,1,8596,101,99,116,111,114,59,1,10574,4,2,101,114,2766,2792,101,4,3,59,65,86,2775,2777,2784,1,8867,114,114,111,119,59,1,8612,101,99,116,111,114,59,1,10586,105,97,110,103,108,101,4,3,59,66,69,2806,2808,2813,1,8882,97,114,59,1,10703,113,117,97,108,59,1,8884,112,4,3,68,84,86,2829,2841,2852,111,119,110,86,101,99,116,111,114,59,1,10577,101,101,86,101,99,116,111,114,59,1,10592,101,99,116,111,114,4,2,59,66,2863,2865,1,8639,97,114,59,1,10584,101,99,116,111,114,4,2,59,66,2881,2883,1,8636,97,114,59,1,10578,114,114,111,119,59,1,8656,105,103,104,116,97,114,114,111,119,59,1,8660,115,4,6,69,70,71,76,83,84,2922,2936,2947,2956,2962,2974,113,117,97,108,71,114,101,97,116,101,114,59,1,8922,117,108,108,69,113,117,97,108,59,1,8806,114,101,97,116,101,114,59,1,8822,101,115,115,59,1,10913,108,97,110,116,69,113,117,97,108,59,1,10877,105,108,100,101,59,1,8818,114,59,3,55349,56591,4,2,59,101,2992,2994,1,8920,102,116,97,114,114,111,119,59,1,8666,105,100,111,116,59,1,319,4,3,110,112,119,3019,3110,3115,103,4,4,76,82,108,114,3030,3058,3070,3098,101,102,116,4,2,65,82,3039,3046,114,114,111,119,59,1,10229,105,103,104,116,65,114,114,111,119,59,1,10231,105,103,104,116,65,114,114,111,119,59,1,10230,101,102,116,4,2,97,114,3079,3086,114,114,111,119,59,1,10232,105,103,104,116,97,114,114,111,119,59,1,10234,105,103,104,116,97,114,114,111,119,59,1,10233,102,59,3,55349,56643,101,114,4,2,76,82,3123,3134,101,102,116,65,114,114,111,119,59,1,8601,105,103,104,116,65,114,114,111,119,59,1,8600,4,3,99,104,116,3154,3158,3161,114,59,1,8466,59,1,8624,114,111,107,59,1,321,59,1,8810,4,8,97,99,101,102,105,111,115,117,3188,3192,3196,3222,3227,3237,3243,3248,112,59,1,10501,121,59,1,1052,4,2,100,108,3202,3213,105,117,109,83,112,97,99,101,59,1,8287,108,105,110,116,114,102,59,1,8499,114,59,3,55349,56592,110,117,115,80,108,117,115,59,1,8723,112,102,59,3,55349,56644,99,114,59,1,8499,59,1,924,4,9,74,97,99,101,102,111,115,116,117,3271,3276,3283,3306,3422,3427,4120,4126,4137,99,121,59,1,1034,99,117,116,101,59,1,323,4,3,97,101,121,3291,3297,3303,114,111,110,59,1,327,100,105,108,59,1,325,59,1,1053,4,3,103,115,119,3314,3380,3415,97,116,105,118,101,4,3,77,84,86,3327,3340,3365,101,100,105,117,109,83,112,97,99,101,59,1,8203,104,105,4,2,99,110,3348,3357,107,83,112,97,99,101,59,1,8203,83,112,97,99,101,59,1,8203,101,114,121,84,104,105,110,83,112,97,99,101,59,1,8203,116,101,100,4,2,71,76,3389,3405,114,101,97,116,101,114,71,114,101,97,116,101,114,59,1,8811,101,115,115,76,101,115,115,59,1,8810,76,105,110,101,59,1,10,114,59,3,55349,56593,4,4,66,110,112,116,3437,3444,3460,3464,114,101,97,107,59,1,8288,66,114,101,97,107,105,110,103,83,112,97,99,101,59,1,160,102,59,1,8469,4,13,59,67,68,69,71,72,76,78,80,82,83,84,86,3492,3494,3517,3536,3578,3657,3685,3784,3823,3860,3915,4066,4107,1,10988,4,2,111,117,3500,3510,110,103,114,117,101,110,116,59,1,8802,112,67,97,112,59,1,8813,111,117,98,108,101,86,101,114,116,105,99,97,108,66,97,114,59,1,8742,4,3,108,113,120,3544,3552,3571,101,109,101,110,116,59,1,8713,117,97,108,4,2,59,84,3561,3563,1,8800,105,108,100,101,59,3,8770,824,105,115,116,115,59,1,8708,114,101,97,116,101,114,4,7,59,69,70,71,76,83,84,3600,3602,3609,3621,3631,3637,3650,1,8815,113,117,97,108,59,1,8817,117,108,108,69,113,117,97,108,59,3,8807,824,114,101,97,116,101,114,59,3,8811,824,101,115,115,59,1,8825,108,97,110,116,69,113,117,97,108,59,3,10878,824,105,108,100,101,59,1,8821,117,109,112,4,2,68,69,3666,3677,111,119,110,72,117,109,112,59,3,8782,824,113,117,97,108,59,3,8783,824,101,4,2,102,115,3692,3724,116,84,114,105,97,110,103,108,101,4,3,59,66,69,3709,3711,3717,1,8938,97,114,59,3,10703,824,113,117,97,108,59,1,8940,115,4,6,59,69,71,76,83,84,3739,3741,3748,3757,3764,3777,1,8814,113,117,97,108,59,1,8816,114,101,97,116,101,114,59,1,8824,101,115,115,59,3,8810,824,108,97,110,116,69,113,117,97,108,59,3,10877,824,105,108,100,101,59,1,8820,101,115,116,101,100,4,2,71,76,3795,3812,114,101,97,116,101,114,71,114,101,97,116,101,114,59,3,10914,824,101,115,115,76,101,115,115,59,3,10913,824,114,101,99,101,100,101,115,4,3,59,69,83,3838,3840,3848,1,8832,113,117,97,108,59,3,10927,824,108,97,110,116,69,113,117,97,108,59,1,8928,4,2,101,105,3866,3881,118,101,114,115,101,69,108,101,109,101,110,116,59,1,8716,103,104,116,84,114,105,97,110,103,108,101,4,3,59,66,69,3900,3902,3908,1,8939,97,114,59,3,10704,824,113,117,97,108,59,1,8941,4,2,113,117,3921,3973,117,97,114,101,83,117,4,2,98,112,3933,3952,115,101,116,4,2,59,69,3942,3945,3,8847,824,113,117,97,108,59,1,8930,101,114,115,101,116,4,2,59,69,3963,3966,3,8848,824,113,117,97,108,59,1,8931,4,3,98,99,112,3981,4000,4045,115,101,116,4,2,59,69,3990,3993,3,8834,8402,113,117,97,108,59,1,8840,99,101,101,100,115,4,4,59,69,83,84,4015,4017,4025,4037,1,8833,113,117,97,108,59,3,10928,824,108,97,110,116,69,113,117,97,108,59,1,8929,105,108,100,101,59,3,8831,824,101,114,115,101,116,4,2,59,69,4056,4059,3,8835,8402,113,117,97,108,59,1,8841,105,108,100,101,4,4,59,69,70,84,4080,4082,4089,4100,1,8769,113,117,97,108,59,1,8772,117,108,108,69,113,117,97,108,59,1,8775,105,108,100,101,59,1,8777,101,114,116,105,99,97,108,66,97,114,59,1,8740,99,114,59,3,55349,56489,105,108,100,101,5,209,1,59,4135,1,209,59,1,925,4,14,69,97,99,100,102,103,109,111,112,114,115,116,117,118,4170,4176,4187,4205,4212,4217,4228,4253,4259,4292,4295,4316,4337,4346,108,105,103,59,1,338,99,117,116,101,5,211,1,59,4185,1,211,4,2,105,121,4193,4202,114,99,5,212,1,59,4200,1,212,59,1,1054,98,108,97,99,59,1,336,114,59,3,55349,56594,114,97,118,101,5,210,1,59,4226,1,210,4,3,97,101,105,4236,4241,4246,99,114,59,1,332,103,97,59,1,937,99,114,111,110,59,1,927,112,102,59,3,55349,56646,101,110,67,117,114,108,121,4,2,68,81,4272,4285,111,117,98,108,101,81,117,111,116,101,59,1,8220,117,111,116,101,59,1,8216,59,1,10836,4,2,99,108,4301,4306,114,59,3,55349,56490,97,115,104,5,216,1,59,4314,1,216,105,4,2,108,109,4323,4332,100,101,5,213,1,59,4330,1,213,101,115,59,1,10807,109,108,5,214,1,59,4344,1,214,101,114,4,2,66,80,4354,4380,4,2,97,114,4360,4364,114,59,1,8254,97,99,4,2,101,107,4372,4375,59,1,9182,101,116,59,1,9140,97,114,101,110,116,104,101,115,105,115,59,1,9180,4,9,97,99,102,104,105,108,111,114,115,4413,4422,4426,4431,4435,4438,4448,4471,4561,114,116,105,97,108,68,59,1,8706,121,59,1,1055,114,59,3,55349,56595,105,59,1,934,59,1,928,117,115,77,105,110,117,115,59,1,177,4,2,105,112,4454,4467,110,99,97,114,101,112,108,97,110,101,59,1,8460,102,59,1,8473,4,4,59,101,105,111,4481,4483,4526,4531,1,10939,99,101,100,101,115,4,4,59,69,83,84,4498,4500,4507,4519,1,8826,113,117,97,108,59,1,10927,108,97,110,116,69,113,117,97,108,59,1,8828,105,108,100,101,59,1,8830,109,101,59,1,8243,4,2,100,112,4537,4543,117,99,116,59,1,8719,111,114,116,105,111,110,4,2,59,97,4555,4557,1,8759,108,59,1,8733,4,2,99,105,4567,4572,114,59,3,55349,56491,59,1,936,4,4,85,102,111,115,4585,4594,4599,4604,79,84,5,34,1,59,4592,1,34,114,59,3,55349,56596,112,102,59,1,8474,99,114,59,3,55349,56492,4,12,66,69,97,99,101,102,104,105,111,114,115,117,4636,4642,4650,4681,4704,4763,4767,4771,5047,5069,5081,5094,97,114,114,59,1,10512,71,5,174,1,59,4648,1,174,4,3,99,110,114,4658,4664,4668,117,116,101,59,1,340,103,59,1,10219,114,4,2,59,116,4675,4677,1,8608,108,59,1,10518,4,3,97,101,121,4689,4695,4701,114,111,110,59,1,344,100,105,108,59,1,342,59,1,1056,4,2,59,118,4710,4712,1,8476,101,114,115,101,4,2,69,85,4722,4748,4,2,108,113,4728,4736,101,109,101,110,116,59,1,8715,117,105,108,105,98,114,105,117,109,59,1,8651,112,69,113,117,105,108,105,98,114,105,117,109,59,1,10607,114,59,1,8476,111,59,1,929,103,104,116,4,8,65,67,68,70,84,85,86,97,4792,4840,4849,4905,4912,4972,5022,5040,4,2,110,114,4798,4811,103,108,101,66,114,97,99,107,101,116,59,1,10217,114,111,119,4,3,59,66,76,4822,4824,4829,1,8594,97,114,59,1,8677,101,102,116,65,114,114,111,119,59,1,8644,101,105,108,105,110,103,59,1,8969,111,4,2,117,119,4856,4869,98,108,101,66,114,97,99,107,101,116,59,1,10215,110,4,2,84,86,4876,4887,101,101,86,101,99,116,111,114,59,1,10589,101,99,116,111,114,4,2,59,66,4898,4900,1,8642,97,114,59,1,10581,108,111,111,114,59,1,8971,4,2,101,114,4918,4944,101,4,3,59,65,86,4927,4929,4936,1,8866,114,114,111,119,59,1,8614,101,99,116,111,114,59,1,10587,105,97,110,103,108,101,4,3,59,66,69,4958,4960,4965,1,8883,97,114,59,1,10704,113,117,97,108,59,1,8885,112,4,3,68,84,86,4981,4993,5004,111,119,110,86,101,99,116,111,114,59,1,10575,101,101,86,101,99,116,111,114,59,1,10588,101,99,116,111,114,4,2,59,66,5015,5017,1,8638,97,114,59,1,10580,101,99,116,111,114,4,2,59,66,5033,5035,1,8640,97,114,59,1,10579,114,114,111,119,59,1,8658,4,2,112,117,5053,5057,102,59,1,8477,110,100,73,109,112,108,105,101,115,59,1,10608,105,103,104,116,97,114,114,111,119,59,1,8667,4,2,99,104,5087,5091,114,59,1,8475,59,1,8625,108,101,68,101,108,97,121,101,100,59,1,10740,4,13,72,79,97,99,102,104,105,109,111,113,115,116,117,5134,5150,5157,5164,5198,5203,5259,5265,5277,5283,5374,5380,5385,4,2,67,99,5140,5146,72,99,121,59,1,1065,121,59,1,1064,70,84,99,121,59,1,1068,99,117,116,101,59,1,346,4,5,59,97,101,105,121,5176,5178,5184,5190,5195,1,10940,114,111,110,59,1,352,100,105,108,59,1,350,114,99,59,1,348,59,1,1057,114,59,3,55349,56598,111,114,116,4,4,68,76,82,85,5216,5227,5238,5250,111,119,110,65,114,114,111,119,59,1,8595,101,102,116,65,114,114,111,119,59,1,8592,105,103,104,116,65,114,114,111,119,59,1,8594,112,65,114,114,111,119,59,1,8593,103,109,97,59,1,931,97,108,108,67,105,114,99,108,101,59,1,8728,112,102,59,3,55349,56650,4,2,114,117,5289,5293,116,59,1,8730,97,114,101,4,4,59,73,83,85,5306,5308,5322,5367,1,9633,110,116,101,114,115,101,99,116,105,111,110,59,1,8851,117,4,2,98,112,5329,5347,115,101,116,4,2,59,69,5338,5340,1,8847,113,117,97,108,59,1,8849,101,114,115,101,116,4,2,59,69,5358,5360,1,8848,113,117,97,108,59,1,8850,110,105,111,110,59,1,8852,99,114,59,3,55349,56494,97,114,59,1,8902,4,4,98,99,109,112,5395,5420,5475,5478,4,2,59,115,5401,5403,1,8912,101,116,4,2,59,69,5411,5413,1,8912,113,117,97,108,59,1,8838,4,2,99,104,5426,5468,101,101,100,115,4,4,59,69,83,84,5440,5442,5449,5461,1,8827,113,117,97,108,59,1,10928,108,97,110,116,69,113,117,97,108,59,1,8829,105,108,100,101,59,1,8831,84,104,97,116,59,1,8715,59,1,8721,4,3,59,101,115,5486,5488,5507,1,8913,114,115,101,116,4,2,59,69,5498,5500,1,8835,113,117,97,108,59,1,8839,101,116,59,1,8913,4,11,72,82,83,97,99,102,104,105,111,114,115,5536,5546,5552,5567,5579,5602,5607,5655,5695,5701,5711,79,82,78,5,222,1,59,5544,1,222,65,68,69,59,1,8482,4,2,72,99,5558,5563,99,121,59,1,1035,121,59,1,1062,4,2,98,117,5573,5576,59,1,9,59,1,932,4,3,97,101,121,5587,5593,5599,114,111,110,59,1,356,100,105,108,59,1,354,59,1,1058,114,59,3,55349,56599,4,2,101,105,5613,5631,4,2,114,116,5619,5627,101,102,111,114,101,59,1,8756,97,59,1,920,4,2,99,110,5637,5647,107,83,112,97,99,101,59,3,8287,8202,83,112,97,99,101,59,1,8201,108,100,101,4,4,59,69,70,84,5668,5670,5677,5688,1,8764,113,117,97,108,59,1,8771,117,108,108,69,113,117,97,108,59,1,8773,105,108,100,101,59,1,8776,112,102,59,3,55349,56651,105,112,108,101,68,111,116,59,1,8411,4,2,99,116,5717,5722,114,59,3,55349,56495,114,111,107,59,1,358,4,14,97,98,99,100,102,103,109,110,111,112,114,115,116,117,5758,5789,5805,5823,5830,5835,5846,5852,5921,5937,6089,6095,6101,6108,4,2,99,114,5764,5774,117,116,101,5,218,1,59,5772,1,218,114,4,2,59,111,5781,5783,1,8607,99,105,114,59,1,10569,114,4,2,99,101,5796,5800,121,59,1,1038,118,101,59,1,364,4,2,105,121,5811,5820,114,99,5,219,1,59,5818,1,219,59,1,1059,98,108,97,99,59,1,368,114,59,3,55349,56600,114,97,118,101,5,217,1,59,5844,1,217,97,99,114,59,1,362,4,2,100,105,5858,5905,101,114,4,2,66,80,5866,5892,4,2,97,114,5872,5876,114,59,1,95,97,99,4,2,101,107,5884,5887,59,1,9183,101,116,59,1,9141,97,114,101,110,116,104,101,115,105,115,59,1,9181,111,110,4,2,59,80,5913,5915,1,8899,108,117,115,59,1,8846,4,2,103,112,5927,5932,111,110,59,1,370,102,59,3,55349,56652,4,8,65,68,69,84,97,100,112,115,5955,5985,5996,6009,6026,6033,6044,6075,114,114,111,119,4,3,59,66,68,5967,5969,5974,1,8593,97,114,59,1,10514,111,119,110,65,114,114,111,119,59,1,8645,111,119,110,65,114,114,111,119,59,1,8597,113,117,105,108,105,98,114,105,117,109,59,1,10606,101,101,4,2,59,65,6017,6019,1,8869,114,114,111,119,59,1,8613,114,114,111,119,59,1,8657,111,119,110,97,114,114,111,119,59,1,8661,101,114,4,2,76,82,6052,6063,101,102,116,65,114,114,111,119,59,1,8598,105,103,104,116,65,114,114,111,119,59,1,8599,105,4,2,59,108,6082,6084,1,978,111,110,59,1,933,105,110,103,59,1,366,99,114,59,3,55349,56496,105,108,100,101,59,1,360,109,108,5,220,1,59,6115,1,220,4,9,68,98,99,100,101,102,111,115,118,6137,6143,6148,6152,6166,6250,6255,6261,6267,97,115,104,59,1,8875,97,114,59,1,10987,121,59,1,1042,97,115,104,4,2,59,108,6161,6163,1,8873,59,1,10982,4,2,101,114,6172,6175,59,1,8897,4,3,98,116,121,6183,6188,6238,97,114,59,1,8214,4,2,59,105,6194,6196,1,8214,99,97,108,4,4,66,76,83,84,6209,6214,6220,6231,97,114,59,1,8739,105,110,101,59,1,124,101,112,97,114,97,116,111,114,59,1,10072,105,108,100,101,59,1,8768,84,104,105,110,83,112,97,99,101,59,1,8202,114,59,3,55349,56601,112,102,59,3,55349,56653,99,114,59,3,55349,56497,100,97,115,104,59,1,8874,4,5,99,101,102,111,115,6286,6292,6298,6303,6309,105,114,99,59,1,372,100,103,101,59,1,8896,114,59,3,55349,56602,112,102,59,3,55349,56654,99,114,59,3,55349,56498,4,4,102,105,111,115,6325,6330,6333,6339,114,59,3,55349,56603,59,1,926,112,102,59,3,55349,56655,99,114,59,3,55349,56499,4,9,65,73,85,97,99,102,111,115,117,6365,6370,6375,6380,6391,6405,6410,6416,6422,99,121,59,1,1071,99,121,59,1,1031,99,121,59,1,1070,99,117,116,101,5,221,1,59,6389,1,221,4,2,105,121,6397,6402,114,99,59,1,374,59,1,1067,114,59,3,55349,56604,112,102,59,3,55349,56656,99,114,59,3,55349,56500,109,108,59,1,376,4,8,72,97,99,100,101,102,111,115,6445,6450,6457,6472,6477,6501,6505,6510,99,121,59,1,1046,99,117,116,101,59,1,377,4,2,97,121,6463,6469,114,111,110,59,1,381,59,1,1047,111,116,59,1,379,4,2,114,116,6483,6497,111,87,105,100,116,104,83,112,97,99,101,59,1,8203,97,59,1,918,114,59,1,8488,112,102,59,1,8484,99,114,59,3,55349,56501,4,16,97,98,99,101,102,103,108,109,110,111,112,114,115,116,117,119,6550,6561,6568,6612,6622,6634,6645,6672,6699,6854,6870,6923,6933,6963,6974,6983,99,117,116,101,5,225,1,59,6559,1,225,114,101,118,101,59,1,259,4,6,59,69,100,105,117,121,6582,6584,6588,6591,6600,6609,1,8766,59,3,8766,819,59,1,8767,114,99,5,226,1,59,6598,1,226,116,101,5,180,1,59,6607,1,180,59,1,1072,108,105,103,5,230,1,59,6620,1,230,4,2,59,114,6628,6630,1,8289,59,3,55349,56606,114,97,118,101,5,224,1,59,6643,1,224,4,2,101,112,6651,6667,4,2,102,112,6657,6663,115,121,109,59,1,8501,104,59,1,8501,104,97,59,1,945,4,2,97,112,6678,6692,4,2,99,108,6684,6688,114,59,1,257,103,59,1,10815,5,38,1,59,6697,1,38,4,2,100,103,6705,6737,4,5,59,97,100,115,118,6717,6719,6724,6727,6734,1,8743,110,100,59,1,10837,59,1,10844,108,111,112,101,59,1,10840,59,1,10842,4,7,59,101,108,109,114,115,122,6753,6755,6758,6762,6814,6835,6848,1,8736,59,1,10660,101,59,1,8736,115,100,4,2,59,97,6770,6772,1,8737,4,8,97,98,99,100,101,102,103,104,6790,6793,6796,6799,6802,6805,6808,6811,59,1,10664,59,1,10665,59,1,10666,59,1,10667,59,1,10668,59,1,10669,59,1,10670,59,1,10671,116,4,2,59,118,6821,6823,1,8735,98,4,2,59,100,6830,6832,1,8894,59,1,10653,4,2,112,116,6841,6845,104,59,1,8738,59,1,197,97,114,114,59,1,9084,4,2,103,112,6860,6865,111,110,59,1,261,102,59,3,55349,56658,4,7,59,69,97,101,105,111,112,6886,6888,6891,6897,6900,6904,6908,1,8776,59,1,10864,99,105,114,59,1,10863,59,1,8778,100,59,1,8779,115,59,1,39,114,111,120,4,2,59,101,6917,6919,1,8776,113,59,1,8778,105,110,103,5,229,1,59,6931,1,229,4,3,99,116,121,6941,6946,6949,114,59,3,55349,56502,59,1,42,109,112,4,2,59,101,6957,6959,1,8776,113,59,1,8781,105,108,100,101,5,227,1,59,6972,1,227,109,108,5,228,1,59,6981,1,228,4,2,99,105,6989,6997,111,110,105,110,116,59,1,8755,110,116,59,1,10769,4,16,78,97,98,99,100,101,102,105,107,108,110,111,112,114,115,117,7036,7041,7119,7135,7149,7155,7219,7224,7347,7354,7463,7489,7786,7793,7814,7866,111,116,59,1,10989,4,2,99,114,7047,7094,107,4,4,99,101,112,115,7058,7064,7073,7080,111,110,103,59,1,8780,112,115,105,108,111,110,59,1,1014,114,105,109,101,59,1,8245,105,109,4,2,59,101,7088,7090,1,8765,113,59,1,8909,4,2,118,119,7100,7105,101,101,59,1,8893,101,100,4,2,59,103,7113,7115,1,8965,101,59,1,8965,114,107,4,2,59,116,7127,7129,1,9141,98,114,107,59,1,9142,4,2,111,121,7141,7146,110,103,59,1,8780,59,1,1073,113,117,111,59,1,8222,4,5,99,109,112,114,116,7167,7181,7188,7193,7199,97,117,115,4,2,59,101,7176,7178,1,8757,59,1,8757,112,116,121,118,59,1,10672,115,105,59,1,1014,110,111,117,59,1,8492,4,3,97,104,119,7207,7210,7213,59,1,946,59,1,8502,101,101,110,59,1,8812,114,59,3,55349,56607,103,4,7,99,111,115,116,117,118,119,7241,7262,7288,7305,7328,7335,7340,4,3,97,105,117,7249,7253,7258,112,59,1,8898,114,99,59,1,9711,112,59,1,8899,4,3,100,112,116,7270,7275,7281,111,116,59,1,10752,108,117,115,59,1,10753,105,109,101,115,59,1,10754,4,2,113,116,7294,7300,99,117,112,59,1,10758,97,114,59,1,9733,114,105,97,110,103,108,101,4,2,100,117,7318,7324,111,119,110,59,1,9661,112,59,1,9651,112,108,117,115,59,1,10756,101,101,59,1,8897,101,100,103,101,59,1,8896,97,114,111,119,59,1,10509,4,3,97,107,111,7362,7436,7458,4,2,99,110,7368,7432,107,4,3,108,115,116,7377,7386,7394,111,122,101,110,103,101,59,1,10731,113,117,97,114,101,59,1,9642,114,105,97,110,103,108,101,4,4,59,100,108,114,7411,7413,7419,7425,1,9652,111,119,110,59,1,9662,101,102,116,59,1,9666,105,103,104,116,59,1,9656,107,59,1,9251,4,2,49,51,7442,7454,4,2,50,52,7448,7451,59,1,9618,59,1,9617,52,59,1,9619,99,107,59,1,9608,4,2,101,111,7469,7485,4,2,59,113,7475,7478,3,61,8421,117,105,118,59,3,8801,8421,116,59,1,8976,4,4,112,116,119,120,7499,7504,7517,7523,102,59,3,55349,56659,4,2,59,116,7510,7512,1,8869,111,109,59,1,8869,116,105,101,59,1,8904,4,12,68,72,85,86,98,100,104,109,112,116,117,118,7549,7571,7597,7619,7655,7660,7682,7708,7715,7721,7728,7750,4,4,76,82,108,114,7559,7562,7565,7568,59,1,9559,59,1,9556,59,1,9558,59,1,9555,4,5,59,68,85,100,117,7583,7585,7588,7591,7594,1,9552,59,1,9574,59,1,9577,59,1,9572,59,1,9575,4,4,76,82,108,114,7607,7610,7613,7616,59,1,9565,59,1,9562,59,1,9564,59,1,9561,4,7,59,72,76,82,104,108,114,7635,7637,7640,7643,7646,7649,7652,1,9553,59,1,9580,59,1,9571,59,1,9568,59,1,9579,59,1,9570,59,1,9567,111,120,59,1,10697,4,4,76,82,108,114,7670,7673,7676,7679,59,1,9557,59,1,9554,59,1,9488,59,1,9484,4,5,59,68,85,100,117,7694,7696,7699,7702,7705,1,9472,59,1,9573,59,1,9576,59,1,9516,59,1,9524,105,110,117,115,59,1,8863,108,117,115,59,1,8862,105,109,101,115,59,1,8864,4,4,76,82,108,114,7738,7741,7744,7747,59,1,9563,59,1,9560,59,1,9496,59,1,9492,4,7,59,72,76,82,104,108,114,7766,7768,7771,7774,7777,7780,7783,1,9474,59,1,9578,59,1,9569,59,1,9566,59,1,9532,59,1,9508,59,1,9500,114,105,109,101,59,1,8245,4,2,101,118,7799,7804,118,101,59,1,728,98,97,114,5,166,1,59,7812,1,166,4,4,99,101,105,111,7824,7829,7834,7846,114,59,3,55349,56503,109,105,59,1,8271,109,4,2,59,101,7841,7843,1,8765,59,1,8909,108,4,3,59,98,104,7855,7857,7860,1,92,59,1,10693,115,117,98,59,1,10184,4,2,108,109,7872,7885,108,4,2,59,101,7879,7881,1,8226,116,59,1,8226,112,4,3,59,69,101,7894,7896,7899,1,8782,59,1,10926,4,2,59,113,7905,7907,1,8783,59,1,8783,4,15,97,99,100,101,102,104,105,108,111,114,115,116,117,119,121,7942,8021,8075,8080,8121,8126,8157,8279,8295,8430,8446,8485,8491,8707,8726,4,3,99,112,114,7950,7956,8007,117,116,101,59,1,263,4,6,59,97,98,99,100,115,7970,7972,7977,7984,7998,8003,1,8745,110,100,59,1,10820,114,99,117,112,59,1,10825,4,2,97,117,7990,7994,112,59,1,10827,112,59,1,10823,111,116,59,1,10816,59,3,8745,65024,4,2,101,111,8013,8017,116,59,1,8257,110,59,1,711,4,4,97,101,105,117,8031,8046,8056,8061,4,2,112,114,8037,8041,115,59,1,10829,111,110,59,1,269,100,105,108,5,231,1,59,8054,1,231,114,99,59,1,265,112,115,4,2,59,115,8069,8071,1,10828,109,59,1,10832,111,116,59,1,267,4,3,100,109,110,8088,8097,8104,105,108,5,184,1,59,8095,1,184,112,116,121,118,59,1,10674,116,5,162,2,59,101,8112,8114,1,162,114,100,111,116,59,1,183,114,59,3,55349,56608,4,3,99,101,105,8134,8138,8154,121,59,1,1095,99,107,4,2,59,109,8146,8148,1,10003,97,114,107,59,1,10003,59,1,967,114,4,7,59,69,99,101,102,109,115,8174,8176,8179,8258,8261,8268,8273,1,9675,59,1,10691,4,3,59,101,108,8187,8189,8193,1,710,113,59,1,8791,101,4,2,97,100,8200,8223,114,114,111,119,4,2,108,114,8210,8216,101,102,116,59,1,8634,105,103,104,116,59,1,8635,4,5,82,83,97,99,100,8235,8238,8241,8246,8252,59,1,174,59,1,9416,115,116,59,1,8859,105,114,99,59,1,8858,97,115,104,59,1,8861,59,1,8791,110,105,110,116,59,1,10768,105,100,59,1,10991,99,105,114,59,1,10690,117,98,115,4,2,59,117,8288,8290,1,9827,105,116,59,1,9827,4,4,108,109,110,112,8305,8326,8376,8400,111,110,4,2,59,101,8313,8315,1,58,4,2,59,113,8321,8323,1,8788,59,1,8788,4,2,109,112,8332,8344,97,4,2,59,116,8339,8341,1,44,59,1,64,4,3,59,102,108,8352,8354,8358,1,8705,110,59,1,8728,101,4,2,109,120,8365,8371,101,110,116,59,1,8705,101,115,59,1,8450,4,2,103,105,8382,8395,4,2,59,100,8388,8390,1,8773,111,116,59,1,10861,110,116,59,1,8750,4,3,102,114,121,8408,8412,8417,59,3,55349,56660,111,100,59,1,8720,5,169,2,59,115,8424,8426,1,169,114,59,1,8471,4,2,97,111,8436,8441,114,114,59,1,8629,115,115,59,1,10007,4,2,99,117,8452,8457,114,59,3,55349,56504,4,2,98,112,8463,8474,4,2,59,101,8469,8471,1,10959,59,1,10961,4,2,59,101,8480,8482,1,10960,59,1,10962,100,111,116,59,1,8943,4,7,100,101,108,112,114,118,119,8507,8522,8536,8550,8600,8697,8702,97,114,114,4,2,108,114,8516,8519,59,1,10552,59,1,10549,4,2,112,115,8528,8532,114,59,1,8926,99,59,1,8927,97,114,114,4,2,59,112,8545,8547,1,8630,59,1,10557,4,6,59,98,99,100,111,115,8564,8566,8573,8587,8592,8596,1,8746,114,99,97,112,59,1,10824,4,2,97,117,8579,8583,112,59,1,10822,112,59,1,10826,111,116,59,1,8845,114,59,1,10821,59,3,8746,65024,4,4,97,108,114,118,8610,8623,8663,8672,114,114,4,2,59,109,8618,8620,1,8631,59,1,10556,121,4,3,101,118,119,8632,8651,8656,113,4,2,112,115,8639,8645,114,101,99,59,1,8926,117,99,99,59,1,8927,101,101,59,1,8910,101,100,103,101,59,1,8911,101,110,5,164,1,59,8670,1,164,101,97,114,114,111,119,4,2,108,114,8684,8690,101,102,116,59,1,8630,105,103,104,116,59,1,8631,101,101,59,1,8910,101,100,59,1,8911,4,2,99,105,8713,8721,111,110,105,110,116,59,1,8754,110,116,59,1,8753,108,99,116,121,59,1,9005,4,19,65,72,97,98,99,100,101,102,104,105,106,108,111,114,115,116,117,119,122,8773,8778,8783,8821,8839,8854,8887,8914,8930,8944,9036,9041,9058,9197,9227,9258,9281,9297,9305,114,114,59,1,8659,97,114,59,1,10597,4,4,103,108,114,115,8793,8799,8805,8809,103,101,114,59,1,8224,101,116,104,59,1,8504,114,59,1,8595,104,4,2,59,118,8816,8818,1,8208,59,1,8867,4,2,107,108,8827,8834,97,114,111,119,59,1,10511,97,99,59,1,733,4,2,97,121,8845,8851,114,111,110,59,1,271,59,1,1076,4,3,59,97,111,8862,8864,8880,1,8518,4,2,103,114,8870,8876,103,101,114,59,1,8225,114,59,1,8650,116,115,101,113,59,1,10871,4,3,103,108,109,8895,8902,8907,5,176,1,59,8900,1,176,116,97,59,1,948,112,116,121,118,59,1,10673,4,2,105,114,8920,8926,115,104,116,59,1,10623,59,3,55349,56609,97,114,4,2,108,114,8938,8941,59,1,8643,59,1,8642,4,5,97,101,103,115,118,8956,8986,8989,8996,9001,109,4,3,59,111,115,8965,8967,8983,1,8900,110,100,4,2,59,115,8975,8977,1,8900,117,105,116,59,1,9830,59,1,9830,59,1,168,97,109,109,97,59,1,989,105,110,59,1,8946,4,3,59,105,111,9009,9011,9031,1,247,100,101,5,247,2,59,111,9020,9022,1,247,110,116,105,109,101,115,59,1,8903,110,120,59,1,8903,99,121,59,1,1106,99,4,2,111,114,9048,9053,114,110,59,1,8990,111,112,59,1,8973,4,5,108,112,116,117,119,9070,9076,9081,9130,9144,108,97,114,59,1,36,102,59,3,55349,56661,4,5,59,101,109,112,115,9093,9095,9109,9116,9122,1,729,113,4,2,59,100,9102,9104,1,8784,111,116,59,1,8785,105,110,117,115,59,1,8760,108,117,115,59,1,8724,113,117,97,114,101,59,1,8865,98,108,101,98,97,114,119,101,100,103,101,59,1,8966,110,4,3,97,100,104,9153,9160,9172,114,114,111,119,59,1,8595,111,119,110,97,114,114,111,119,115,59,1,8650,97,114,112,111,111,110,4,2,108,114,9184,9190,101,102,116,59,1,8643,105,103,104,116,59,1,8642,4,2,98,99,9203,9211,107,97,114,111,119,59,1,10512,4,2,111,114,9217,9222,114,110,59,1,8991,111,112,59,1,8972,4,3,99,111,116,9235,9248,9252,4,2,114,121,9241,9245,59,3,55349,56505,59,1,1109,108,59,1,10742,114,111,107,59,1,273,4,2,100,114,9264,9269,111,116,59,1,8945,105,4,2,59,102,9276,9278,1,9663,59,1,9662,4,2,97,104,9287,9292,114,114,59,1,8693,97,114,59,1,10607,97,110,103,108,101,59,1,10662,4,2,99,105,9311,9315,121,59,1,1119,103,114,97,114,114,59,1,10239,4,18,68,97,99,100,101,102,103,108,109,110,111,112,113,114,115,116,117,120,9361,9376,9398,9439,9444,9447,9462,9495,9531,9585,9598,9614,9659,9755,9771,9792,9808,9826,4,2,68,111,9367,9372,111,116,59,1,10871,116,59,1,8785,4,2,99,115,9382,9392,117,116,101,5,233,1,59,9390,1,233,116,101,114,59,1,10862,4,4,97,105,111,121,9408,9414,9430,9436,114,111,110,59,1,283,114,4,2,59,99,9421,9423,1,8790,5,234,1,59,9428,1,234,108,111,110,59,1,8789,59,1,1101,111,116,59,1,279,59,1,8519,4,2,68,114,9453,9458,111,116,59,1,8786,59,3,55349,56610,4,3,59,114,115,9470,9472,9482,1,10906,97,118,101,5,232,1,59,9480,1,232,4,2,59,100,9488,9490,1,10902,111,116,59,1,10904,4,4,59,105,108,115,9505,9507,9515,9518,1,10905,110,116,101,114,115,59,1,9191,59,1,8467,4,2,59,100,9524,9526,1,10901,111,116,59,1,10903,4,3,97,112,115,9539,9544,9564,99,114,59,1,275,116,121,4,3,59,115,118,9554,9556,9561,1,8709,101,116,59,1,8709,59,1,8709,112,4,2,49,59,9571,9583,4,2,51,52,9577,9580,59,1,8196,59,1,8197,1,8195,4,2,103,115,9591,9594,59,1,331,112,59,1,8194,4,2,103,112,9604,9609,111,110,59,1,281,102,59,3,55349,56662,4,3,97,108,115,9622,9635,9640,114,4,2,59,115,9629,9631,1,8917,108,59,1,10723,117,115,59,1,10865,105,4,3,59,108,118,9649,9651,9656,1,949,111,110,59,1,949,59,1,1013,4,4,99,115,117,118,9669,9686,9716,9747,4,2,105,111,9675,9680,114,99,59,1,8790,108,111,110,59,1,8789,4,2,105,108,9692,9696,109,59,1,8770,97,110,116,4,2,103,108,9705,9710,116,114,59,1,10902,101,115,115,59,1,10901,4,3,97,101,105,9724,9729,9734,108,115,59,1,61,115,116,59,1,8799,118,4,2,59,68,9741,9743,1,8801,68,59,1,10872,112,97,114,115,108,59,1,10725,4,2,68,97,9761,9766,111,116,59,1,8787,114,114,59,1,10609,4,3,99,100,105,9779,9783,9788,114,59,1,8495,111,116,59,1,8784,109,59,1,8770,4,2,97,104,9798,9801,59,1,951,5,240,1,59,9806,1,240,4,2,109,114,9814,9822,108,5,235,1,59,9820,1,235,111,59,1,8364,4,3,99,105,112,9834,9838,9843,108,59,1,33,115,116,59,1,8707,4,2,101,111,9849,9859,99,116,97,116,105,111,110,59,1,8496,110,101,110,116,105,97,108,101,59,1,8519,4,12,97,99,101,102,105,106,108,110,111,112,114,115,9896,9910,9914,9921,9954,9960,9967,9989,9994,10027,10036,10164,108,108,105,110,103,100,111,116,115,101,113,59,1,8786,121,59,1,1092,109,97,108,101,59,1,9792,4,3,105,108,114,9929,9935,9950,108,105,103,59,1,64259,4,2,105,108,9941,9945,103,59,1,64256,105,103,59,1,64260,59,3,55349,56611,108,105,103,59,1,64257,108,105,103,59,3,102,106,4,3,97,108,116,9975,9979,9984,116,59,1,9837,105,103,59,1,64258,110,115,59,1,9649,111,102,59,1,402,4,2,112,114,10000,10005,102,59,3,55349,56663,4,2,97,107,10011,10016,108,108,59,1,8704,4,2,59,118,10022,10024,1,8916,59,1,10969,97,114,116,105,110,116,59,1,10765,4,2,97,111,10042,10159,4,2,99,115,10048,10155,4,6,49,50,51,52,53,55,10062,10102,10114,10135,10139,10151,4,6,50,51,52,53,54,56,10076,10083,10086,10093,10096,10099,5,189,1,59,10081,1,189,59,1,8531,5,188,1,59,10091,1,188,59,1,8533,59,1,8537,59,1,8539,4,2,51,53,10108,10111,59,1,8532,59,1,8534,4,3,52,53,56,10122,10129,10132,5,190,1,59,10127,1,190,59,1,8535,59,1,8540,53,59,1,8536,4,2,54,56,10145,10148,59,1,8538,59,1,8541,56,59,1,8542,108,59,1,8260,119,110,59,1,8994,99,114,59,3,55349,56507,4,17,69,97,98,99,100,101,102,103,105,106,108,110,111,114,115,116,118,10206,10217,10247,10254,10268,10273,10358,10363,10374,10380,10385,10406,10458,10464,10470,10497,10610,4,2,59,108,10212,10214,1,8807,59,1,10892,4,3,99,109,112,10225,10231,10244,117,116,101,59,1,501,109,97,4,2,59,100,10239,10241,1,947,59,1,989,59,1,10886,114,101,118,101,59,1,287,4,2,105,121,10260,10265,114,99,59,1,285,59,1,1075,111,116,59,1,289,4,4,59,108,113,115,10283,10285,10288,10308,1,8805,59,1,8923,4,3,59,113,115,10296,10298,10301,1,8805,59,1,8807,108,97,110,116,59,1,10878,4,4,59,99,100,108,10318,10320,10324,10345,1,10878,99,59,1,10921,111,116,4,2,59,111,10332,10334,1,10880,4,2,59,108,10340,10342,1,10882,59,1,10884,4,2,59,101,10351,10354,3,8923,65024,115,59,1,10900,114,59,3,55349,56612,4,2,59,103,10369,10371,1,8811,59,1,8921,109,101,108,59,1,8503,99,121,59,1,1107,4,4,59,69,97,106,10395,10397,10400,10403,1,8823,59,1,10898,59,1,10917,59,1,10916,4,4,69,97,101,115,10416,10419,10434,10453,59,1,8809,112,4,2,59,112,10426,10428,1,10890,114,111,120,59,1,10890,4,2,59,113,10440,10442,1,10888,4,2,59,113,10448,10450,1,10888,59,1,8809,105,109,59,1,8935,112,102,59,3,55349,56664,97,118,101,59,1,96,4,2,99,105,10476,10480,114,59,1,8458,109,4,3,59,101,108,10489,10491,10494,1,8819,59,1,10894,59,1,10896,5,62,6,59,99,100,108,113,114,10512,10514,10527,10532,10538,10545,1,62,4,2,99,105,10520,10523,59,1,10919,114,59,1,10874,111,116,59,1,8919,80,97,114,59,1,10645,117,101,115,116,59,1,10876,4,5,97,100,101,108,115,10557,10574,10579,10599,10605,4,2,112,114,10563,10570,112,114,111,120,59,1,10886,114,59,1,10616,111,116,59,1,8919,113,4,2,108,113,10586,10592,101,115,115,59,1,8923,108,101,115,115,59,1,10892,101,115,115,59,1,8823,105,109,59,1,8819,4,2,101,110,10616,10626,114,116,110,101,113,113,59,3,8809,65024,69,59,3,8809,65024,4,10,65,97,98,99,101,102,107,111,115,121,10653,10658,10713,10718,10724,10760,10765,10786,10850,10875,114,114,59,1,8660,4,4,105,108,109,114,10668,10674,10678,10684,114,115,112,59,1,8202,102,59,1,189,105,108,116,59,1,8459,4,2,100,114,10690,10695,99,121,59,1,1098,4,3,59,99,119,10703,10705,10710,1,8596,105,114,59,1,10568,59,1,8621,97,114,59,1,8463,105,114,99,59,1,293,4,3,97,108,114,10732,10748,10754,114,116,115,4,2,59,117,10741,10743,1,9829,105,116,59,1,9829,108,105,112,59,1,8230,99,111,110,59,1,8889,114,59,3,55349,56613,115,4,2,101,119,10772,10779,97,114,111,119,59,1,10533,97,114,111,119,59,1,10534,4,5,97,109,111,112,114,10798,10803,10809,10839,10844,114,114,59,1,8703,116,104,116,59,1,8763,107,4,2,108,114,10816,10827,101,102,116,97,114,114,111,119,59,1,8617,105,103,104,116,97,114,114,111,119,59,1,8618,102,59,3,55349,56665,98,97,114,59,1,8213,4,3,99,108,116,10858,10863,10869,114,59,3,55349,56509,97,115,104,59,1,8463,114,111,107,59,1,295,4,2,98,112,10881,10887,117,108,108,59,1,8259,104,101,110,59,1,8208,4,15,97,99,101,102,103,105,106,109,110,111,112,113,115,116,117,10925,10936,10958,10977,10990,11001,11039,11045,11101,11192,11220,11226,11237,11285,11299,99,117,116,101,5,237,1,59,10934,1,237,4,3,59,105,121,10944,10946,10955,1,8291,114,99,5,238,1,59,10953,1,238,59,1,1080,4,2,99,120,10964,10968,121,59,1,1077,99,108,5,161,1,59,10975,1,161,4,2,102,114,10983,10986,59,1,8660,59,3,55349,56614,114,97,118,101,5,236,1,59,10999,1,236,4,4,59,105,110,111,11011,11013,11028,11034,1,8520,4,2,105,110,11019,11024,110,116,59,1,10764,116,59,1,8749,102,105,110,59,1,10716,116,97,59,1,8489,108,105,103,59,1,307,4,3,97,111,112,11053,11092,11096,4,3,99,103,116,11061,11065,11088,114,59,1,299,4,3,101,108,112,11073,11076,11082,59,1,8465,105,110,101,59,1,8464,97,114,116,59,1,8465,104,59,1,305,102,59,1,8887,101,100,59,1,437,4,5,59,99,102,111,116,11113,11115,11121,11136,11142,1,8712,97,114,101,59,1,8453,105,110,4,2,59,116,11129,11131,1,8734,105,101,59,1,10717,100,111,116,59,1,305,4,5,59,99,101,108,112,11154,11156,11161,11179,11186,1,8747,97,108,59,1,8890,4,2,103,114,11167,11173,101,114,115,59,1,8484,99,97,108,59,1,8890,97,114,104,107,59,1,10775,114,111,100,59,1,10812,4,4,99,103,112,116,11202,11206,11211,11216,121,59,1,1105,111,110,59,1,303,102,59,3,55349,56666,97,59,1,953,114,111,100,59,1,10812,117,101,115,116,5,191,1,59,11235,1,191,4,2,99,105,11243,11248,114,59,3,55349,56510,110,4,5,59,69,100,115,118,11261,11263,11266,11271,11282,1,8712,59,1,8953,111,116,59,1,8949,4,2,59,118,11277,11279,1,8948,59,1,8947,59,1,8712,4,2,59,105,11291,11293,1,8290,108,100,101,59,1,297,4,2,107,109,11305,11310,99,121,59,1,1110,108,5,239,1,59,11316,1,239,4,6,99,102,109,111,115,117,11332,11346,11351,11357,11363,11380,4,2,105,121,11338,11343,114,99,59,1,309,59,1,1081,114,59,3,55349,56615,97,116,104,59,1,567,112,102,59,3,55349,56667,4,2,99,101,11369,11374,114,59,3,55349,56511,114,99,121,59,1,1112,107,99,121,59,1,1108,4,8,97,99,102,103,104,106,111,115,11404,11418,11433,11438,11445,11450,11455,11461,112,112,97,4,2,59,118,11413,11415,1,954,59,1,1008,4,2,101,121,11424,11430,100,105,108,59,1,311,59,1,1082,114,59,3,55349,56616,114,101,101,110,59,1,312,99,121,59,1,1093,99,121,59,1,1116,112,102,59,3,55349,56668,99,114,59,3,55349,56512,4,23,65,66,69,72,97,98,99,100,101,102,103,104,106,108,109,110,111,112,114,115,116,117,118,11515,11538,11544,11555,11560,11721,11780,11818,11868,12136,12160,12171,12203,12208,12246,12275,12327,12509,12523,12569,12641,12732,12752,4,3,97,114,116,11523,11528,11532,114,114,59,1,8666,114,59,1,8656,97,105,108,59,1,10523,97,114,114,59,1,10510,4,2,59,103,11550,11552,1,8806,59,1,10891,97,114,59,1,10594,4,9,99,101,103,109,110,112,113,114,116,11580,11586,11594,11600,11606,11624,11627,11636,11694,117,116,101,59,1,314,109,112,116,121,118,59,1,10676,114,97,110,59,1,8466,98,100,97,59,1,955,103,4,3,59,100,108,11615,11617,11620,1,10216,59,1,10641,101,59,1,10216,59,1,10885,117,111,5,171,1,59,11634,1,171,114,4,8,59,98,102,104,108,112,115,116,11655,11657,11669,11673,11677,11681,11685,11690,1,8592,4,2,59,102,11663,11665,1,8676,115,59,1,10527,115,59,1,10525,107,59,1,8617,112,59,1,8619,108,59,1,10553,105,109,59,1,10611,108,59,1,8610,4,3,59,97,101,11702,11704,11709,1,10923,105,108,59,1,10521,4,2,59,115,11715,11717,1,10925,59,3,10925,65024,4,3,97,98,114,11729,11734,11739,114,114,59,1,10508,114,107,59,1,10098,4,2,97,107,11745,11758,99,4,2,101,107,11752,11755,59,1,123,59,1,91,4,2,101,115,11764,11767,59,1,10635,108,4,2,100,117,11774,11777,59,1,10639,59,1,10637,4,4,97,101,117,121,11790,11796,11811,11815,114,111,110,59,1,318,4,2,100,105,11802,11807,105,108,59,1,316,108,59,1,8968,98,59,1,123,59,1,1083,4,4,99,113,114,115,11828,11832,11845,11864,97,59,1,10550,117,111,4,2,59,114,11840,11842,1,8220,59,1,8222,4,2,100,117,11851,11857,104,97,114,59,1,10599,115,104,97,114,59,1,10571,104,59,1,8626,4,5,59,102,103,113,115,11880,11882,12008,12011,12031,1,8804,116,4,5,97,104,108,114,116,11895,11913,11935,11947,11996,114,114,111,119,4,2,59,116,11905,11907,1,8592,97,105,108,59,1,8610,97,114,112,111,111,110,4,2,100,117,11925,11931,111,119,110,59,1,8637,112,59,1,8636,101,102,116,97,114,114,111,119,115,59,1,8647,105,103,104,116,4,3,97,104,115,11959,11974,11984,114,114,111,119,4,2,59,115,11969,11971,1,8596,59,1,8646,97,114,112,111,111,110,115,59,1,8651,113,117,105,103,97,114,114,111,119,59,1,8621,104,114,101,101,116,105,109,101,115,59,1,8907,59,1,8922,4,3,59,113,115,12019,12021,12024,1,8804,59,1,8806,108,97,110,116,59,1,10877,4,5,59,99,100,103,115,12043,12045,12049,12070,12083,1,10877,99,59,1,10920,111,116,4,2,59,111,12057,12059,1,10879,4,2,59,114,12065,12067,1,10881,59,1,10883,4,2,59,101,12076,12079,3,8922,65024,115,59,1,10899,4,5,97,100,101,103,115,12095,12103,12108,12126,12131,112,112,114,111,120,59,1,10885,111,116,59,1,8918,113,4,2,103,113,12115,12120,116,114,59,1,8922,103,116,114,59,1,10891,116,114,59,1,8822,105,109,59,1,8818,4,3,105,108,114,12144,12150,12156,115,104,116,59,1,10620,111,111,114,59,1,8970,59,3,55349,56617,4,2,59,69,12166,12168,1,8822,59,1,10897,4,2,97,98,12177,12198,114,4,2,100,117,12184,12187,59,1,8637,4,2,59,108,12193,12195,1,8636,59,1,10602,108,107,59,1,9604,99,121,59,1,1113,4,5,59,97,99,104,116,12220,12222,12227,12235,12241,1,8810,114,114,59,1,8647,111,114,110,101,114,59,1,8990,97,114,100,59,1,10603,114,105,59,1,9722,4,2,105,111,12252,12258,100,111,116,59,1,320,117,115,116,4,2,59,97,12267,12269,1,9136,99,104,101,59,1,9136,4,4,69,97,101,115,12285,12288,12303,12322,59,1,8808,112,4,2,59,112,12295,12297,1,10889,114,111,120,59,1,10889,4,2,59,113,12309,12311,1,10887,4,2,59,113,12317,12319,1,10887,59,1,8808,105,109,59,1,8934,4,8,97,98,110,111,112,116,119,122,12345,12359,12364,12421,12446,12467,12474,12490,4,2,110,114,12351,12355,103,59,1,10220,114,59,1,8701,114,107,59,1,10214,103,4,3,108,109,114,12373,12401,12409,101,102,116,4,2,97,114,12382,12389,114,114,111,119,59,1,10229,105,103,104,116,97,114,114,111,119,59,1,10231,97,112,115,116,111,59,1,10236,105,103,104,116,97,114,114,111,119,59,1,10230,112,97,114,114,111,119,4,2,108,114,12433,12439,101,102,116,59,1,8619,105,103,104,116,59,1,8620,4,3,97,102,108,12454,12458,12462,114,59,1,10629,59,3,55349,56669,117,115,59,1,10797,105,109,101,115,59,1,10804,4,2,97,98,12480,12485,115,116,59,1,8727,97,114,59,1,95,4,3,59,101,102,12498,12500,12506,1,9674,110,103,101,59,1,9674,59,1,10731,97,114,4,2,59,108,12517,12519,1,40,116,59,1,10643,4,5,97,99,104,109,116,12535,12540,12548,12561,12564,114,114,59,1,8646,111,114,110,101,114,59,1,8991,97,114,4,2,59,100,12556,12558,1,8651,59,1,10605,59,1,8206,114,105,59,1,8895,4,6,97,99,104,105,113,116,12583,12589,12594,12597,12614,12635,113,117,111,59,1,8249,114,59,3,55349,56513,59,1,8624,109,4,3,59,101,103,12606,12608,12611,1,8818,59,1,10893,59,1,10895,4,2,98,117,12620,12623,59,1,91,111,4,2,59,114,12630,12632,1,8216,59,1,8218,114,111,107,59,1,322,5,60,8,59,99,100,104,105,108,113,114,12660,12662,12675,12680,12686,12692,12698,12705,1,60,4,2,99,105,12668,12671,59,1,10918,114,59,1,10873,111,116,59,1,8918,114,101,101,59,1,8907,109,101,115,59,1,8905,97,114,114,59,1,10614,117,101,115,116,59,1,10875,4,2,80,105,12711,12716,97,114,59,1,10646,4,3,59,101,102,12724,12726,12729,1,9667,59,1,8884,59,1,9666,114,4,2,100,117,12739,12746,115,104,97,114,59,1,10570,104,97,114,59,1,10598,4,2,101,110,12758,12768,114,116,110,101,113,113,59,3,8808,65024,69,59,3,8808,65024,4,14,68,97,99,100,101,102,104,105,108,110,111,112,115,117,12803,12809,12893,12908,12914,12928,12933,12937,13011,13025,13032,13049,13052,13069,68,111,116,59,1,8762,4,4,99,108,112,114,12819,12827,12849,12887,114,5,175,1,59,12825,1,175,4,2,101,116,12833,12836,59,1,9794,4,2,59,101,12842,12844,1,10016,115,101,59,1,10016,4,2,59,115,12855,12857,1,8614,116,111,4,4,59,100,108,117,12869,12871,12877,12883,1,8614,111,119,110,59,1,8615,101,102,116,59,1,8612,112,59,1,8613,107,101,114,59,1,9646,4,2,111,121,12899,12905,109,109,97,59,1,10793,59,1,1084,97,115,104,59,1,8212,97,115,117,114,101,100,97,110,103,108,101,59,1,8737,114,59,3,55349,56618,111,59,1,8487,4,3,99,100,110,12945,12954,12985,114,111,5,181,1,59,12952,1,181,4,4,59,97,99,100,12964,12966,12971,12976,1,8739,115,116,59,1,42,105,114,59,1,10992,111,116,5,183,1,59,12983,1,183,117,115,4,3,59,98,100,12995,12997,13000,1,8722,59,1,8863,4,2,59,117,13006,13008,1,8760,59,1,10794,4,2,99,100,13017,13021,112,59,1,10971,114,59,1,8230,112,108,117,115,59,1,8723,4,2,100,112,13038,13044,101,108,115,59,1,8871,102,59,3,55349,56670,59,1,8723,4,2,99,116,13058,13063,114,59,3,55349,56514,112,111,115,59,1,8766,4,3,59,108,109,13077,13079,13087,1,956,116,105,109,97,112,59,1,8888,97,112,59,1,8888,4,24,71,76,82,86,97,98,99,100,101,102,103,104,105,106,108,109,111,112,114,115,116,117,118,119,13142,13165,13217,13229,13247,13330,13359,13414,13420,13508,13513,13579,13602,13626,13631,13762,13767,13855,13936,13995,14214,14285,14312,14432,4,2,103,116,13148,13152,59,3,8921,824,4,2,59,118,13158,13161,3,8811,8402,59,3,8811,824,4,3,101,108,116,13173,13200,13204,102,116,4,2,97,114,13181,13188,114,114,111,119,59,1,8653,105,103,104,116,97,114,114,111,119,59,1,8654,59,3,8920,824,4,2,59,118,13210,13213,3,8810,8402,59,3,8810,824,105,103,104,116,97,114,114,111,119,59,1,8655,4,2,68,100,13235,13241,97,115,104,59,1,8879,97,115,104,59,1,8878,4,5,98,99,110,112,116,13259,13264,13270,13275,13308,108,97,59,1,8711,117,116,101,59,1,324,103,59,3,8736,8402,4,5,59,69,105,111,112,13287,13289,13293,13298,13302,1,8777,59,3,10864,824,100,59,3,8779,824,115,59,1,329,114,111,120,59,1,8777,117,114,4,2,59,97,13316,13318,1,9838,108,4,2,59,115,13325,13327,1,9838,59,1,8469,4,2,115,117,13336,13344,112,5,160,1,59,13342,1,160,109,112,4,2,59,101,13352,13355,3,8782,824,59,3,8783,824,4,5,97,101,111,117,121,13371,13385,13391,13407,13411,4,2,112,114,13377,13380,59,1,10819,111,110,59,1,328,100,105,108,59,1,326,110,103,4,2,59,100,13399,13401,1,8775,111,116,59,3,10861,824,112,59,1,10818,59,1,1085,97,115,104,59,1,8211,4,7,59,65,97,100,113,115,120,13436,13438,13443,13466,13472,13478,13494,1,8800,114,114,59,1,8663,114,4,2,104,114,13450,13454,107,59,1,10532,4,2,59,111,13460,13462,1,8599,119,59,1,8599,111,116,59,3,8784,824,117,105,118,59,1,8802,4,2,101,105,13484,13489,97,114,59,1,10536,109,59,3,8770,824,105,115,116,4,2,59,115,13503,13505,1,8708,59,1,8708,114,59,3,55349,56619,4,4,69,101,115,116,13523,13527,13563,13568,59,3,8807,824,4,3,59,113,115,13535,13537,13559,1,8817,4,3,59,113,115,13545,13547,13551,1,8817,59,3,8807,824,108,97,110,116,59,3,10878,824,59,3,10878,824,105,109,59,1,8821,4,2,59,114,13574,13576,1,8815,59,1,8815,4,3,65,97,112,13587,13592,13597,114,114,59,1,8654,114,114,59,1,8622,97,114,59,1,10994,4,3,59,115,118,13610,13612,13623,1,8715,4,2,59,100,13618,13620,1,8956,59,1,8954,59,1,8715,99,121,59,1,1114,4,7,65,69,97,100,101,115,116,13647,13652,13656,13661,13665,13737,13742,114,114,59,1,8653,59,3,8806,824,114,114,59,1,8602,114,59,1,8229,4,4,59,102,113,115,13675,13677,13703,13725,1,8816,116,4,2,97,114,13684,13691,114,114,111,119,59,1,8602,105,103,104,116,97,114,114,111,119,59,1,8622,4,3,59,113,115,13711,13713,13717,1,8816,59,3,8806,824,108,97,110,116,59,3,10877,824,4,2,59,115,13731,13734,3,10877,824,59,1,8814,105,109,59,1,8820,4,2,59,114,13748,13750,1,8814,105,4,2,59,101,13757,13759,1,8938,59,1,8940,105,100,59,1,8740,4,2,112,116,13773,13778,102,59,3,55349,56671,5,172,3,59,105,110,13787,13789,13829,1,172,110,4,4,59,69,100,118,13800,13802,13806,13812,1,8713,59,3,8953,824,111,116,59,3,8949,824,4,3,97,98,99,13820,13823,13826,59,1,8713,59,1,8951,59,1,8950,105,4,2,59,118,13836,13838,1,8716,4,3,97,98,99,13846,13849,13852,59,1,8716,59,1,8958,59,1,8957,4,3,97,111,114,13863,13892,13899,114,4,4,59,97,115,116,13874,13876,13883,13888,1,8742,108,108,101,108,59,1,8742,108,59,3,11005,8421,59,3,8706,824,108,105,110,116,59,1,10772,4,3,59,99,101,13907,13909,13914,1,8832,117,101,59,1,8928,4,2,59,99,13920,13923,3,10927,824,4,2,59,101,13929,13931,1,8832,113,59,3,10927,824,4,4,65,97,105,116,13946,13951,13971,13982,114,114,59,1,8655,114,114,4,3,59,99,119,13961,13963,13967,1,8603,59,3,10547,824,59,3,8605,824,103,104,116,97,114,114,111,119,59,1,8603,114,105,4,2,59,101,13990,13992,1,8939,59,1,8941,4,7,99,104,105,109,112,113,117,14011,14036,14060,14080,14085,14090,14106,4,4,59,99,101,114,14021,14023,14028,14032,1,8833,117,101,59,1,8929,59,3,10928,824,59,3,55349,56515,111,114,116,4,2,109,112,14045,14050,105,100,59,1,8740,97,114,97,108,108,101,108,59,1,8742,109,4,2,59,101,14067,14069,1,8769,4,2,59,113,14075,14077,1,8772,59,1,8772,105,100,59,1,8740,97,114,59,1,8742,115,117,4,2,98,112,14098,14102,101,59,1,8930,101,59,1,8931,4,3,98,99,112,14114,14157,14171,4,4,59,69,101,115,14124,14126,14130,14133,1,8836,59,3,10949,824,59,1,8840,101,116,4,2,59,101,14141,14144,3,8834,8402,113,4,2,59,113,14151,14153,1,8840,59,3,10949,824,99,4,2,59,101,14164,14166,1,8833,113,59,3,10928,824,4,4,59,69,101,115,14181,14183,14187,14190,1,8837,59,3,10950,824,59,1,8841,101,116,4,2,59,101,14198,14201,3,8835,8402,113,4,2,59,113,14208,14210,1,8841,59,3,10950,824,4,4,103,105,108,114,14224,14228,14238,14242,108,59,1,8825,108,100,101,5,241,1,59,14236,1,241,103,59,1,8824,105,97,110,103,108,101,4,2,108,114,14254,14269,101,102,116,4,2,59,101,14263,14265,1,8938,113,59,1,8940,105,103,104,116,4,2,59,101,14279,14281,1,8939,113,59,1,8941,4,2,59,109,14291,14293,1,957,4,3,59,101,115,14301,14303,14308,1,35,114,111,59,1,8470,112,59,1,8199,4,9,68,72,97,100,103,105,108,114,115,14332,14338,14344,14349,14355,14369,14376,14408,14426,97,115,104,59,1,8877,97,114,114,59,1,10500,112,59,3,8781,8402,97,115,104,59,1,8876,4,2,101,116,14361,14365,59,3,8805,8402,59,3,62,8402,110,102,105,110,59,1,10718,4,3,65,101,116,14384,14389,14393,114,114,59,1,10498,59,3,8804,8402,4,2,59,114,14399,14402,3,60,8402,105,101,59,3,8884,8402,4,2,65,116,14414,14419,114,114,59,1,10499,114,105,101,59,3,8885,8402,105,109,59,3,8764,8402,4,3,65,97,110,14440,14445,14468,114,114,59,1,8662,114,4,2,104,114,14452,14456,107,59,1,10531,4,2,59,111,14462,14464,1,8598,119,59,1,8598,101,97,114,59,1,10535,4,18,83,97,99,100,101,102,103,104,105,108,109,111,112,114,115,116,117,118,14512,14515,14535,14560,14597,14603,14618,14643,14657,14662,14701,14741,14747,14769,14851,14877,14907,14916,59,1,9416,4,2,99,115,14521,14531,117,116,101,5,243,1,59,14529,1,243,116,59,1,8859,4,2,105,121,14541,14557,114,4,2,59,99,14548,14550,1,8858,5,244,1,59,14555,1,244,59,1,1086,4,5,97,98,105,111,115,14572,14577,14583,14587,14591,115,104,59,1,8861,108,97,99,59,1,337,118,59,1,10808,116,59,1,8857,111,108,100,59,1,10684,108,105,103,59,1,339,4,2,99,114,14609,14614,105,114,59,1,10687,59,3,55349,56620,4,3,111,114,116,14626,14630,14640,110,59,1,731,97,118,101,5,242,1,59,14638,1,242,59,1,10689,4,2,98,109,14649,14654,97,114,59,1,10677,59,1,937,110,116,59,1,8750,4,4,97,99,105,116,14672,14677,14693,14698,114,114,59,1,8634,4,2,105,114,14683,14687,114,59,1,10686,111,115,115,59,1,10683,110,101,59,1,8254,59,1,10688,4,3,97,101,105,14709,14714,14719,99,114,59,1,333,103,97,59,1,969,4,3,99,100,110,14727,14733,14736,114,111,110,59,1,959,59,1,10678,117,115,59,1,8854,112,102,59,3,55349,56672,4,3,97,101,108,14755,14759,14764,114,59,1,10679,114,112,59,1,10681,117,115,59,1,8853,4,7,59,97,100,105,111,115,118,14785,14787,14792,14831,14837,14841,14848,1,8744,114,114,59,1,8635,4,4,59,101,102,109,14802,14804,14817,14824,1,10845,114,4,2,59,111,14811,14813,1,8500,102,59,1,8500,5,170,1,59,14822,1,170,5,186,1,59,14829,1,186,103,111,102,59,1,8886,114,59,1,10838,108,111,112,101,59,1,10839,59,1,10843,4,3,99,108,111,14859,14863,14873,114,59,1,8500,97,115,104,5,248,1,59,14871,1,248,108,59,1,8856,105,4,2,108,109,14884,14893,100,101,5,245,1,59,14891,1,245,101,115,4,2,59,97,14901,14903,1,8855,115,59,1,10806,109,108,5,246,1,59,14914,1,246,98,97,114,59,1,9021,4,12,97,99,101,102,104,105,108,109,111,114,115,117,14948,14992,14996,15033,15038,15068,15090,15189,15192,15222,15427,15441,114,4,4,59,97,115,116,14959,14961,14976,14989,1,8741,5,182,2,59,108,14968,14970,1,182,108,101,108,59,1,8741,4,2,105,108,14982,14986,109,59,1,10995,59,1,11005,59,1,8706,121,59,1,1087,114,4,5,99,105,109,112,116,15009,15014,15019,15024,15027,110,116,59,1,37,111,100,59,1,46,105,108,59,1,8240,59,1,8869,101,110,107,59,1,8241,114,59,3,55349,56621,4,3,105,109,111,15046,15057,15063,4,2,59,118,15052,15054,1,966,59,1,981,109,97,116,59,1,8499,110,101,59,1,9742,4,3,59,116,118,15076,15078,15087,1,960,99,104,102,111,114,107,59,1,8916,59,1,982,4,2,97,117,15096,15119,110,4,2,99,107,15103,15115,107,4,2,59,104,15110,15112,1,8463,59,1,8462,118,59,1,8463,115,4,9,59,97,98,99,100,101,109,115,116,15140,15142,15148,15151,15156,15168,15171,15179,15184,1,43,99,105,114,59,1,10787,59,1,8862,105,114,59,1,10786,4,2,111,117,15162,15165,59,1,8724,59,1,10789,59,1,10866,110,5,177,1,59,15177,1,177,105,109,59,1,10790,119,111,59,1,10791,59,1,177,4,3,105,112,117,15200,15208,15213,110,116,105,110,116,59,1,10773,102,59,3,55349,56673,110,100,5,163,1,59,15220,1,163,4,10,59,69,97,99,101,105,110,111,115,117,15244,15246,15249,15253,15258,15334,15347,15367,15416,15421,1,8826,59,1,10931,112,59,1,10935,117,101,59,1,8828,4,2,59,99,15264,15266,1,10927,4,6,59,97,99,101,110,115,15280,15282,15290,15299,15303,15329,1,8826,112,112,114,111,120,59,1,10935,117,114,108,121,101,113,59,1,8828,113,59,1,10927,4,3,97,101,115,15311,15319,15324,112,112,114,111,120,59,1,10937,113,113,59,1,10933,105,109,59,1,8936,105,109,59,1,8830,109,101,4,2,59,115,15342,15344,1,8242,59,1,8473,4,3,69,97,115,15355,15358,15362,59,1,10933,112,59,1,10937,105,109,59,1,8936,4,3,100,102,112,15375,15378,15404,59,1,8719,4,3,97,108,115,15386,15392,15398,108,97,114,59,1,9006,105,110,101,59,1,8978,117,114,102,59,1,8979,4,2,59,116,15410,15412,1,8733,111,59,1,8733,105,109,59,1,8830,114,101,108,59,1,8880,4,2,99,105,15433,15438,114,59,3,55349,56517,59,1,968,110,99,115,112,59,1,8200,4,6,102,105,111,112,115,117,15462,15467,15472,15478,15485,15491,114,59,3,55349,56622,110,116,59,1,10764,112,102,59,3,55349,56674,114,105,109,101,59,1,8279,99,114,59,3,55349,56518,4,3,97,101,111,15499,15520,15534,116,4,2,101,105,15506,15515,114,110,105,111,110,115,59,1,8461,110,116,59,1,10774,115,116,4,2,59,101,15528,15530,1,63,113,59,1,8799,116,5,34,1,59,15540,1,34,4,21,65,66,72,97,98,99,100,101,102,104,105,108,109,110,111,112,114,115,116,117,120,15586,15609,15615,15620,15796,15855,15893,15931,15977,16001,16039,16183,16204,16222,16228,16285,16312,16318,16363,16408,16416,4,3,97,114,116,15594,15599,15603,114,114,59,1,8667,114,59,1,8658,97,105,108,59,1,10524,97,114,114,59,1,10511,97,114,59,1,10596,4,7,99,100,101,110,113,114,116,15636,15651,15656,15664,15687,15696,15770,4,2,101,117,15642,15646,59,3,8765,817,116,101,59,1,341,105,99,59,1,8730,109,112,116,121,118,59,1,10675,103,4,4,59,100,101,108,15675,15677,15680,15683,1,10217,59,1,10642,59,1,10661,101,59,1,10217,117,111,5,187,1,59,15694,1,187,114,4,11,59,97,98,99,102,104,108,112,115,116,119,15721,15723,15727,15739,15742,15746,15750,15754,15758,15763,15767,1,8594,112,59,1,10613,4,2,59,102,15733,15735,1,8677,115,59,1,10528,59,1,10547,115,59,1,10526,107,59,1,8618,112,59,1,8620,108,59,1,10565,105,109,59,1,10612,108,59,1,8611,59,1,8605,4,2,97,105,15776,15781,105,108,59,1,10522,111,4,2,59,110,15788,15790,1,8758,97,108,115,59,1,8474,4,3,97,98,114,15804,15809,15814,114,114,59,1,10509,114,107,59,1,10099,4,2,97,107,15820,15833,99,4,2,101,107,15827,15830,59,1,125,59,1,93,4,2,101,115,15839,15842,59,1,10636,108,4,2,100,117,15849,15852,59,1,10638,59,1,10640,4,4,97,101,117,121,15865,15871,15886,15890,114,111,110,59,1,345,4,2,100,105,15877,15882,105,108,59,1,343,108,59,1,8969,98,59,1,125,59,1,1088,4,4,99,108,113,115,15903,15907,15914,15927,97,59,1,10551,100,104,97,114,59,1,10601,117,111,4,2,59,114,15922,15924,1,8221,59,1,8221,104,59,1,8627,4,3,97,99,103,15939,15966,15970,108,4,4,59,105,112,115,15950,15952,15957,15963,1,8476,110,101,59,1,8475,97,114,116,59,1,8476,59,1,8477,116,59,1,9645,5,174,1,59,15975,1,174,4,3,105,108,114,15985,15991,15997,115,104,116,59,1,10621,111,111,114,59,1,8971,59,3,55349,56623,4,2,97,111,16007,16028,114,4,2,100,117,16014,16017,59,1,8641,4,2,59,108,16023,16025,1,8640,59,1,10604,4,2,59,118,16034,16036,1,961,59,1,1009,4,3,103,110,115,16047,16167,16171,104,116,4,6,97,104,108,114,115,116,16063,16081,16103,16130,16143,16155,114,114,111,119,4,2,59,116,16073,16075,1,8594,97,105,108,59,1,8611,97,114,112,111,111,110,4,2,100,117,16093,16099,111,119,110,59,1,8641,112,59,1,8640,101,102,116,4,2,97,104,16112,16120,114,114,111,119,115,59,1,8644,97,114,112,111,111,110,115,59,1,8652,105,103,104,116,97,114,114,111,119,115,59,1,8649,113,117,105,103,97,114,114,111,119,59,1,8605,104,114,101,101,116,105,109,101,115,59,1,8908,103,59,1,730,105,110,103,100,111,116,115,101,113,59,1,8787,4,3,97,104,109,16191,16196,16201,114,114,59,1,8644,97,114,59,1,8652,59,1,8207,111,117,115,116,4,2,59,97,16214,16216,1,9137,99,104,101,59,1,9137,109,105,100,59,1,10990,4,4,97,98,112,116,16238,16252,16257,16278,4,2,110,114,16244,16248,103,59,1,10221,114,59,1,8702,114,107,59,1,10215,4,3,97,102,108,16265,16269,16273,114,59,1,10630,59,3,55349,56675,117,115,59,1,10798,105,109,101,115,59,1,10805,4,2,97,112,16291,16304,114,4,2,59,103,16298,16300,1,41,116,59,1,10644,111,108,105,110,116,59,1,10770,97,114,114,59,1,8649,4,4,97,99,104,113,16328,16334,16339,16342,113,117,111,59,1,8250,114,59,3,55349,56519,59,1,8625,4,2,98,117,16348,16351,59,1,93,111,4,2,59,114,16358,16360,1,8217,59,1,8217,4,3,104,105,114,16371,16377,16383,114,101,101,59,1,8908,109,101,115,59,1,8906,105,4,4,59,101,102,108,16394,16396,16399,16402,1,9657,59,1,8885,59,1,9656,116,114,105,59,1,10702,108,117,104,97,114,59,1,10600,59,1,8478,4,19,97,98,99,100,101,102,104,105,108,109,111,112,113,114,115,116,117,119,122,16459,16466,16472,16572,16590,16672,16687,16746,16844,16850,16924,16963,16988,17115,17121,17154,17206,17614,17656,99,117,116,101,59,1,347,113,117,111,59,1,8218,4,10,59,69,97,99,101,105,110,112,115,121,16494,16496,16499,16513,16518,16531,16536,16556,16564,16569,1,8827,59,1,10932,4,2,112,114,16505,16508,59,1,10936,111,110,59,1,353,117,101,59,1,8829,4,2,59,100,16524,16526,1,10928,105,108,59,1,351,114,99,59,1,349,4,3,69,97,115,16544,16547,16551,59,1,10934,112,59,1,10938,105,109,59,1,8937,111,108,105,110,116,59,1,10771,105,109,59,1,8831,59,1,1089,111,116,4,3,59,98,101,16582,16584,16587,1,8901,59,1,8865,59,1,10854,4,7,65,97,99,109,115,116,120,16606,16611,16634,16642,16646,16652,16668,114,114,59,1,8664,114,4,2,104,114,16618,16622,107,59,1,10533,4,2,59,111,16628,16630,1,8600,119,59,1,8600,116,5,167,1,59,16640,1,167,105,59,1,59,119,97,114,59,1,10537,109,4,2,105,110,16659,16665,110,117,115,59,1,8726,59,1,8726,116,59,1,10038,114,4,2,59,111,16679,16682,3,55349,56624,119,110,59,1,8994,4,4,97,99,111,121,16697,16702,16716,16739,114,112,59,1,9839,4,2,104,121,16708,16713,99,121,59,1,1097,59,1,1096,114,116,4,2,109,112,16724,16729,105,100,59,1,8739,97,114,97,108,108,101,108,59,1,8741,5,173,1,59,16744,1,173,4,2,103,109,16752,16770,109,97,4,3,59,102,118,16762,16764,16767,1,963,59,1,962,59,1,962,4,8,59,100,101,103,108,110,112,114,16788,16790,16795,16806,16817,16828,16832,16838,1,8764,111,116,59,1,10858,4,2,59,113,16801,16803,1,8771,59,1,8771,4,2,59,69,16812,16814,1,10910,59,1,10912,4,2,59,69,16823,16825,1,10909,59,1,10911,101,59,1,8774,108,117,115,59,1,10788,97,114,114,59,1,10610,97,114,114,59,1,8592,4,4,97,101,105,116,16860,16883,16891,16904,4,2,108,115,16866,16878,108,115,101,116,109,105,110,117,115,59,1,8726,104,112,59,1,10803,112,97,114,115,108,59,1,10724,4,2,100,108,16897,16900,59,1,8739,101,59,1,8995,4,2,59,101,16910,16912,1,10922,4,2,59,115,16918,16920,1,10924,59,3,10924,65024,4,3,102,108,112,16932,16938,16958,116,99,121,59,1,1100,4,2,59,98,16944,16946,1,47,4,2,59,97,16952,16954,1,10692,114,59,1,9023,102,59,3,55349,56676,97,4,2,100,114,16970,16985,101,115,4,2,59,117,16978,16980,1,9824,105,116,59,1,9824,59,1,8741,4,3,99,115,117,16996,17028,17089,4,2,97,117,17002,17015,112,4,2,59,115,17009,17011,1,8851,59,3,8851,65024,112,4,2,59,115,17022,17024,1,8852,59,3,8852,65024,117,4,2,98,112,17035,17062,4,3,59,101,115,17043,17045,17048,1,8847,59,1,8849,101,116,4,2,59,101,17056,17058,1,8847,113,59,1,8849,4,3,59,101,115,17070,17072,17075,1,8848,59,1,8850,101,116,4,2,59,101,17083,17085,1,8848,113,59,1,8850,4,3,59,97,102,17097,17099,17112,1,9633,114,4,2,101,102,17106,17109,59,1,9633,59,1,9642,59,1,9642,97,114,114,59,1,8594,4,4,99,101,109,116,17131,17136,17142,17148,114,59,3,55349,56520,116,109,110,59,1,8726,105,108,101,59,1,8995,97,114,102,59,1,8902,4,2,97,114,17160,17172,114,4,2,59,102,17167,17169,1,9734,59,1,9733,4,2,97,110,17178,17202,105,103,104,116,4,2,101,112,17188,17197,112,115,105,108,111,110,59,1,1013,104,105,59,1,981,115,59,1,175,4,5,98,99,109,110,112,17218,17351,17420,17423,17427,4,9,59,69,100,101,109,110,112,114,115,17238,17240,17243,17248,17261,17267,17279,17285,17291,1,8834,59,1,10949,111,116,59,1,10941,4,2,59,100,17254,17256,1,8838,111,116,59,1,10947,117,108,116,59,1,10945,4,2,69,101,17273,17276,59,1,10955,59,1,8842,108,117,115,59,1,10943,97,114,114,59,1,10617,4,3,101,105,117,17299,17335,17339,116,4,3,59,101,110,17308,17310,17322,1,8834,113,4,2,59,113,17317,17319,1,8838,59,1,10949,101,113,4,2,59,113,17330,17332,1,8842,59,1,10955,109,59,1,10951,4,2,98,112,17345,17348,59,1,10965,59,1,10963,99,4,6,59,97,99,101,110,115,17366,17368,17376,17385,17389,17415,1,8827,112,112,114,111,120,59,1,10936,117,114,108,121,101,113,59,1,8829,113,59,1,10928,4,3,97,101,115,17397,17405,17410,112,112,114,111,120,59,1,10938,113,113,59,1,10934,105,109,59,1,8937,105,109,59,1,8831,59,1,8721,103,59,1,9834,4,13,49,50,51,59,69,100,101,104,108,109,110,112,115,17455,17462,17469,17476,17478,17481,17496,17509,17524,17530,17536,17548,17554,5,185,1,59,17460,1,185,5,178,1,59,17467,1,178,5,179,1,59,17474,1,179,1,8835,59,1,10950,4,2,111,115,17487,17491,116,59,1,10942,117,98,59,1,10968,4,2,59,100,17502,17504,1,8839,111,116,59,1,10948,115,4,2,111,117,17516,17520,108,59,1,10185,98,59,1,10967,97,114,114,59,1,10619,117,108,116,59,1,10946,4,2,69,101,17542,17545,59,1,10956,59,1,8843,108,117,115,59,1,10944,4,3,101,105,117,17562,17598,17602,116,4,3,59,101,110,17571,17573,17585,1,8835,113,4,2,59,113,17580,17582,1,8839,59,1,10950,101,113,4,2,59,113,17593,17595,1,8843,59,1,10956,109,59,1,10952,4,2,98,112,17608,17611,59,1,10964,59,1,10966,4,3,65,97,110,17622,17627,17650,114,114,59,1,8665,114,4,2,104,114,17634,17638,107,59,1,10534,4,2,59,111,17644,17646,1,8601,119,59,1,8601,119,97,114,59,1,10538,108,105,103,5,223,1,59,17664,1,223,4,13,97,98,99,100,101,102,104,105,111,112,114,115,119,17694,17709,17714,17737,17742,17749,17754,17860,17905,17957,17964,18090,18122,4,2,114,117,17700,17706,103,101,116,59,1,8982,59,1,964,114,107,59,1,9140,4,3,97,101,121,17722,17728,17734,114,111,110,59,1,357,100,105,108,59,1,355,59,1,1090,111,116,59,1,8411,108,114,101,99,59,1,8981,114,59,3,55349,56625,4,4,101,105,107,111,17764,17805,17836,17851,4,2,114,116,17770,17786,101,4,2,52,102,17777,17780,59,1,8756,111,114,101,59,1,8756,97,4,3,59,115,118,17795,17797,17802,1,952,121,109,59,1,977,59,1,977,4,2,99,110,17811,17831,107,4,2,97,115,17818,17826,112,112,114,111,120,59,1,8776,105,109,59,1,8764,115,112,59,1,8201,4,2,97,115,17842,17846,112,59,1,8776,105,109,59,1,8764,114,110,5,254,1,59,17858,1,254,4,3,108,109,110,17868,17873,17901,100,101,59,1,732,101,115,5,215,3,59,98,100,17884,17886,17898,1,215,4,2,59,97,17892,17894,1,8864,114,59,1,10801,59,1,10800,116,59,1,8749,4,3,101,112,115,17913,17917,17953,97,59,1,10536,4,4,59,98,99,102,17927,17929,17934,17939,1,8868,111,116,59,1,9014,105,114,59,1,10993,4,2,59,111,17945,17948,3,55349,56677,114,107,59,1,10970,97,59,1,10537,114,105,109,101,59,1,8244,4,3,97,105,112,17972,17977,18082,100,101,59,1,8482,4,7,97,100,101,109,112,115,116,17993,18051,18056,18059,18066,18072,18076,110,103,108,101,4,5,59,100,108,113,114,18009,18011,18017,18032,18035,1,9653,111,119,110,59,1,9663,101,102,116,4,2,59,101,18026,18028,1,9667,113,59,1,8884,59,1,8796,105,103,104,116,4,2,59,101,18045,18047,1,9657,113,59,1,8885,111,116,59,1,9708,59,1,8796,105,110,117,115,59,1,10810,108,117,115,59,1,10809,98,59,1,10701,105,109,101,59,1,10811,101,122,105,117,109,59,1,9186,4,3,99,104,116,18098,18111,18116,4,2,114,121,18104,18108,59,3,55349,56521,59,1,1094,99,121,59,1,1115,114,111,107,59,1,359,4,2,105,111,18128,18133,120,116,59,1,8812,104,101,97,100,4,2,108,114,18143,18154,101,102,116,97,114,114,111,119,59,1,8606,105,103,104,116,97,114,114,111,119,59,1,8608,4,18,65,72,97,98,99,100,102,103,104,108,109,111,112,114,115,116,117,119,18204,18209,18214,18234,18250,18268,18292,18308,18319,18343,18379,18397,18413,18504,18547,18553,18584,18603,114,114,59,1,8657,97,114,59,1,10595,4,2,99,114,18220,18230,117,116,101,5,250,1,59,18228,1,250,114,59,1,8593,114,4,2,99,101,18241,18245,121,59,1,1118,118,101,59,1,365,4,2,105,121,18256,18265,114,99,5,251,1,59,18263,1,251,59,1,1091,4,3,97,98,104,18276,18281,18287,114,114,59,1,8645,108,97,99,59,1,369,97,114,59,1,10606,4,2,105,114,18298,18304,115,104,116,59,1,10622,59,3,55349,56626,114,97,118,101,5,249,1,59,18317,1,249,4,2,97,98,18325,18338,114,4,2,108,114,18332,18335,59,1,8639,59,1,8638,108,107,59,1,9600,4,2,99,116,18349,18374,4,2,111,114,18355,18369,114,110,4,2,59,101,18363,18365,1,8988,114,59,1,8988,111,112,59,1,8975,114,105,59,1,9720,4,2,97,108,18385,18390,99,114,59,1,363,5,168,1,59,18395,1,168,4,2,103,112,18403,18408,111,110,59,1,371,102,59,3,55349,56678,4,6,97,100,104,108,115,117,18427,18434,18445,18470,18475,18494,114,114,111,119,59,1,8593,111,119,110,97,114,114,111,119,59,1,8597,97,114,112,111,111,110,4,2,108,114,18457,18463,101,102,116,59,1,8639,105,103,104,116,59,1,8638,117,115,59,1,8846,105,4,3,59,104,108,18484,18486,18489,1,965,59,1,978,111,110,59,1,965,112,97,114,114,111,119,115,59,1,8648,4,3,99,105,116,18512,18537,18542,4,2,111,114,18518,18532,114,110,4,2,59,101,18526,18528,1,8989,114,59,1,8989,111,112,59,1,8974,110,103,59,1,367,114,105,59,1,9721,99,114,59,3,55349,56522,4,3,100,105,114,18561,18566,18572,111,116,59,1,8944,108,100,101,59,1,361,105,4,2,59,102,18579,18581,1,9653,59,1,9652,4,2,97,109,18590,18595,114,114,59,1,8648,108,5,252,1,59,18601,1,252,97,110,103,108,101,59,1,10663,4,15,65,66,68,97,99,100,101,102,108,110,111,112,114,115,122,18643,18648,18661,18667,18847,18851,18857,18904,18909,18915,18931,18937,18943,18949,18996,114,114,59,1,8661,97,114,4,2,59,118,18656,18658,1,10984,59,1,10985,97,115,104,59,1,8872,4,2,110,114,18673,18679,103,114,116,59,1,10652,4,7,101,107,110,112,114,115,116,18695,18704,18711,18720,18742,18754,18810,112,115,105,108,111,110,59,1,1013,97,112,112,97,59,1,1008,111,116,104,105,110,103,59,1,8709,4,3,104,105,114,18728,18732,18735,105,59,1,981,59,1,982,111,112,116,111,59,1,8733,4,2,59,104,18748,18750,1,8597,111,59,1,1009,4,2,105,117,18760,18766,103,109,97,59,1,962,4,2,98,112,18772,18791,115,101,116,110,101,113,4,2,59,113,18784,18787,3,8842,65024,59,3,10955,65024,115,101,116,110,101,113,4,2,59,113,18803,18806,3,8843,65024,59,3,10956,65024,4,2,104,114,18816,18822,101,116,97,59,1,977,105,97,110,103,108,101,4,2,108,114,18834,18840,101,102,116,59,1,8882,105,103,104,116,59,1,8883,121,59,1,1074,97,115,104,59,1,8866,4,3,101,108,114,18865,18884,18890,4,3,59,98,101,18873,18875,18880,1,8744,97,114,59,1,8891,113,59,1,8794,108,105,112,59,1,8942,4,2,98,116,18896,18901,97,114,59,1,124,59,1,124,114,59,3,55349,56627,116,114,105,59,1,8882,115,117,4,2,98,112,18923,18927,59,3,8834,8402,59,3,8835,8402,112,102,59,3,55349,56679,114,111,112,59,1,8733,116,114,105,59,1,8883,4,2,99,117,18955,18960,114,59,3,55349,56523,4,2,98,112,18966,18981,110,4,2,69,101,18973,18977,59,3,10955,65024,59,3,8842,65024,110,4,2,69,101,18988,18992,59,3,10956,65024,59,3,8843,65024,105,103,122,97,103,59,1,10650,4,7,99,101,102,111,112,114,115,19020,19026,19061,19066,19072,19075,19089,105,114,99,59,1,373,4,2,100,105,19032,19055,4,2,98,103,19038,19043,97,114,59,1,10847,101,4,2,59,113,19050,19052,1,8743,59,1,8793,101,114,112,59,1,8472,114,59,3,55349,56628,112,102,59,3,55349,56680,59,1,8472,4,2,59,101,19081,19083,1,8768,97,116,104,59,1,8768,99,114,59,3,55349,56524,4,14,99,100,102,104,105,108,109,110,111,114,115,117,118,119,19125,19146,19152,19157,19173,19176,19192,19197,19202,19236,19252,19269,19286,19291,4,3,97,105,117,19133,19137,19142,112,59,1,8898,114,99,59,1,9711,112,59,1,8899,116,114,105,59,1,9661,114,59,3,55349,56629,4,2,65,97,19163,19168,114,114,59,1,10234,114,114,59,1,10231,59,1,958,4,2,65,97,19182,19187,114,114,59,1,10232,114,114,59,1,10229,97,112,59,1,10236,105,115,59,1,8955,4,3,100,112,116,19210,19215,19230,111,116,59,1,10752,4,2,102,108,19221,19225,59,3,55349,56681,117,115,59,1,10753,105,109,101,59,1,10754,4,2,65,97,19242,19247,114,114,59,1,10233,114,114,59,1,10230,4,2,99,113,19258,19263,114,59,3,55349,56525,99,117,112,59,1,10758,4,2,112,116,19275,19281,108,117,115,59,1,10756,114,105,59,1,9651,101,101,59,1,8897,101,100,103,101,59,1,8896,4,8,97,99,101,102,105,111,115,117,19316,19335,19349,19357,19362,19367,19373,19379,99,4,2,117,121,19323,19332,116,101,5,253,1,59,19330,1,253,59,1,1103,4,2,105,121,19341,19346,114,99,59,1,375,59,1,1099,110,5,165,1,59,19355,1,165,114,59,3,55349,56630,99,121,59,1,1111,112,102,59,3,55349,56682,99,114,59,3,55349,56526,4,2,99,109,19385,19389,121,59,1,1102,108,5,255,1,59,19395,1,255,4,10,97,99,100,101,102,104,105,111,115,119,19419,19426,19441,19446,19462,19467,19472,19480,19486,19492,99,117,116,101,59,1,378,4,2,97,121,19432,19438,114,111,110,59,1,382,59,1,1079,111,116,59,1,380,4,2,101,116,19452,19458,116,114,102,59,1,8488,97,59,1,950,114,59,3,55349,56631,99,121,59,1,1078,103,114,97,114,114,59,1,8669,112,102,59,3,55349,56683,99,114,59,3,55349,56527,4,2,106,110,19498,19501,59,1,8205,106,59,1,8204]);

  const Preprocessor = preprocessor;
  const unicode$1 = unicode$3;
  const neTree = namedEntityData;
  const ERR$1 = errorCodes;

  //Aliases
  const $$5 = unicode$1.CODE_POINTS;
  const $$ = unicode$1.CODE_POINT_SEQUENCES;

  //C1 Unicode control character reference replacements
  const C1_CONTROLS_REFERENCE_REPLACEMENTS = {
      0x80: 0x20ac,
      0x82: 0x201a,
      0x83: 0x0192,
      0x84: 0x201e,
      0x85: 0x2026,
      0x86: 0x2020,
      0x87: 0x2021,
      0x88: 0x02c6,
      0x89: 0x2030,
      0x8a: 0x0160,
      0x8b: 0x2039,
      0x8c: 0x0152,
      0x8e: 0x017d,
      0x91: 0x2018,
      0x92: 0x2019,
      0x93: 0x201c,
      0x94: 0x201d,
      0x95: 0x2022,
      0x96: 0x2013,
      0x97: 0x2014,
      0x98: 0x02dc,
      0x99: 0x2122,
      0x9a: 0x0161,
      0x9b: 0x203a,
      0x9c: 0x0153,
      0x9e: 0x017e,
      0x9f: 0x0178
  };

  // Named entity tree flags
  const HAS_DATA_FLAG = 1 << 0;
  const DATA_DUPLET_FLAG = 1 << 1;
  const HAS_BRANCHES_FLAG = 1 << 2;
  const MAX_BRANCH_MARKER_VALUE = HAS_DATA_FLAG | DATA_DUPLET_FLAG | HAS_BRANCHES_FLAG;

  //States
  const DATA_STATE = 'DATA_STATE';
  const RCDATA_STATE = 'RCDATA_STATE';
  const RAWTEXT_STATE = 'RAWTEXT_STATE';
  const SCRIPT_DATA_STATE = 'SCRIPT_DATA_STATE';
  const PLAINTEXT_STATE = 'PLAINTEXT_STATE';
  const TAG_OPEN_STATE = 'TAG_OPEN_STATE';
  const END_TAG_OPEN_STATE = 'END_TAG_OPEN_STATE';
  const TAG_NAME_STATE = 'TAG_NAME_STATE';
  const RCDATA_LESS_THAN_SIGN_STATE = 'RCDATA_LESS_THAN_SIGN_STATE';
  const RCDATA_END_TAG_OPEN_STATE = 'RCDATA_END_TAG_OPEN_STATE';
  const RCDATA_END_TAG_NAME_STATE = 'RCDATA_END_TAG_NAME_STATE';
  const RAWTEXT_LESS_THAN_SIGN_STATE = 'RAWTEXT_LESS_THAN_SIGN_STATE';
  const RAWTEXT_END_TAG_OPEN_STATE = 'RAWTEXT_END_TAG_OPEN_STATE';
  const RAWTEXT_END_TAG_NAME_STATE = 'RAWTEXT_END_TAG_NAME_STATE';
  const SCRIPT_DATA_LESS_THAN_SIGN_STATE = 'SCRIPT_DATA_LESS_THAN_SIGN_STATE';
  const SCRIPT_DATA_END_TAG_OPEN_STATE = 'SCRIPT_DATA_END_TAG_OPEN_STATE';
  const SCRIPT_DATA_END_TAG_NAME_STATE = 'SCRIPT_DATA_END_TAG_NAME_STATE';
  const SCRIPT_DATA_ESCAPE_START_STATE = 'SCRIPT_DATA_ESCAPE_START_STATE';
  const SCRIPT_DATA_ESCAPE_START_DASH_STATE = 'SCRIPT_DATA_ESCAPE_START_DASH_STATE';
  const SCRIPT_DATA_ESCAPED_STATE = 'SCRIPT_DATA_ESCAPED_STATE';
  const SCRIPT_DATA_ESCAPED_DASH_STATE = 'SCRIPT_DATA_ESCAPED_DASH_STATE';
  const SCRIPT_DATA_ESCAPED_DASH_DASH_STATE = 'SCRIPT_DATA_ESCAPED_DASH_DASH_STATE';
  const SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN_STATE = 'SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN_STATE';
  const SCRIPT_DATA_ESCAPED_END_TAG_OPEN_STATE = 'SCRIPT_DATA_ESCAPED_END_TAG_OPEN_STATE';
  const SCRIPT_DATA_ESCAPED_END_TAG_NAME_STATE = 'SCRIPT_DATA_ESCAPED_END_TAG_NAME_STATE';
  const SCRIPT_DATA_DOUBLE_ESCAPE_START_STATE = 'SCRIPT_DATA_DOUBLE_ESCAPE_START_STATE';
  const SCRIPT_DATA_DOUBLE_ESCAPED_STATE = 'SCRIPT_DATA_DOUBLE_ESCAPED_STATE';
  const SCRIPT_DATA_DOUBLE_ESCAPED_DASH_STATE = 'SCRIPT_DATA_DOUBLE_ESCAPED_DASH_STATE';
  const SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH_STATE = 'SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH_STATE';
  const SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN_STATE = 'SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN_STATE';
  const SCRIPT_DATA_DOUBLE_ESCAPE_END_STATE = 'SCRIPT_DATA_DOUBLE_ESCAPE_END_STATE';
  const BEFORE_ATTRIBUTE_NAME_STATE = 'BEFORE_ATTRIBUTE_NAME_STATE';
  const ATTRIBUTE_NAME_STATE = 'ATTRIBUTE_NAME_STATE';
  const AFTER_ATTRIBUTE_NAME_STATE = 'AFTER_ATTRIBUTE_NAME_STATE';
  const BEFORE_ATTRIBUTE_VALUE_STATE = 'BEFORE_ATTRIBUTE_VALUE_STATE';
  const ATTRIBUTE_VALUE_DOUBLE_QUOTED_STATE = 'ATTRIBUTE_VALUE_DOUBLE_QUOTED_STATE';
  const ATTRIBUTE_VALUE_SINGLE_QUOTED_STATE = 'ATTRIBUTE_VALUE_SINGLE_QUOTED_STATE';
  const ATTRIBUTE_VALUE_UNQUOTED_STATE = 'ATTRIBUTE_VALUE_UNQUOTED_STATE';
  const AFTER_ATTRIBUTE_VALUE_QUOTED_STATE = 'AFTER_ATTRIBUTE_VALUE_QUOTED_STATE';
  const SELF_CLOSING_START_TAG_STATE = 'SELF_CLOSING_START_TAG_STATE';
  const BOGUS_COMMENT_STATE = 'BOGUS_COMMENT_STATE';
  const MARKUP_DECLARATION_OPEN_STATE = 'MARKUP_DECLARATION_OPEN_STATE';
  const COMMENT_START_STATE = 'COMMENT_START_STATE';
  const COMMENT_START_DASH_STATE = 'COMMENT_START_DASH_STATE';
  const COMMENT_STATE = 'COMMENT_STATE';
  const COMMENT_LESS_THAN_SIGN_STATE = 'COMMENT_LESS_THAN_SIGN_STATE';
  const COMMENT_LESS_THAN_SIGN_BANG_STATE = 'COMMENT_LESS_THAN_SIGN_BANG_STATE';
  const COMMENT_LESS_THAN_SIGN_BANG_DASH_STATE = 'COMMENT_LESS_THAN_SIGN_BANG_DASH_STATE';
  const COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH_STATE = 'COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH_STATE';
  const COMMENT_END_DASH_STATE = 'COMMENT_END_DASH_STATE';
  const COMMENT_END_STATE = 'COMMENT_END_STATE';
  const COMMENT_END_BANG_STATE = 'COMMENT_END_BANG_STATE';
  const DOCTYPE_STATE = 'DOCTYPE_STATE';
  const BEFORE_DOCTYPE_NAME_STATE = 'BEFORE_DOCTYPE_NAME_STATE';
  const DOCTYPE_NAME_STATE = 'DOCTYPE_NAME_STATE';
  const AFTER_DOCTYPE_NAME_STATE = 'AFTER_DOCTYPE_NAME_STATE';
  const AFTER_DOCTYPE_PUBLIC_KEYWORD_STATE = 'AFTER_DOCTYPE_PUBLIC_KEYWORD_STATE';
  const BEFORE_DOCTYPE_PUBLIC_IDENTIFIER_STATE = 'BEFORE_DOCTYPE_PUBLIC_IDENTIFIER_STATE';
  const DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED_STATE = 'DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED_STATE';
  const DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED_STATE = 'DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED_STATE';
  const AFTER_DOCTYPE_PUBLIC_IDENTIFIER_STATE = 'AFTER_DOCTYPE_PUBLIC_IDENTIFIER_STATE';
  const BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS_STATE = 'BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS_STATE';
  const AFTER_DOCTYPE_SYSTEM_KEYWORD_STATE = 'AFTER_DOCTYPE_SYSTEM_KEYWORD_STATE';
  const BEFORE_DOCTYPE_SYSTEM_IDENTIFIER_STATE = 'BEFORE_DOCTYPE_SYSTEM_IDENTIFIER_STATE';
  const DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED_STATE = 'DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED_STATE';
  const DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED_STATE = 'DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED_STATE';
  const AFTER_DOCTYPE_SYSTEM_IDENTIFIER_STATE = 'AFTER_DOCTYPE_SYSTEM_IDENTIFIER_STATE';
  const BOGUS_DOCTYPE_STATE = 'BOGUS_DOCTYPE_STATE';
  const CDATA_SECTION_STATE = 'CDATA_SECTION_STATE';
  const CDATA_SECTION_BRACKET_STATE = 'CDATA_SECTION_BRACKET_STATE';
  const CDATA_SECTION_END_STATE = 'CDATA_SECTION_END_STATE';
  const CHARACTER_REFERENCE_STATE = 'CHARACTER_REFERENCE_STATE';
  const NAMED_CHARACTER_REFERENCE_STATE = 'NAMED_CHARACTER_REFERENCE_STATE';
  const AMBIGUOUS_AMPERSAND_STATE = 'AMBIGUOS_AMPERSAND_STATE';
  const NUMERIC_CHARACTER_REFERENCE_STATE = 'NUMERIC_CHARACTER_REFERENCE_STATE';
  const HEXADEMICAL_CHARACTER_REFERENCE_START_STATE = 'HEXADEMICAL_CHARACTER_REFERENCE_START_STATE';
  const DECIMAL_CHARACTER_REFERENCE_START_STATE = 'DECIMAL_CHARACTER_REFERENCE_START_STATE';
  const HEXADEMICAL_CHARACTER_REFERENCE_STATE = 'HEXADEMICAL_CHARACTER_REFERENCE_STATE';
  const DECIMAL_CHARACTER_REFERENCE_STATE = 'DECIMAL_CHARACTER_REFERENCE_STATE';
  const NUMERIC_CHARACTER_REFERENCE_END_STATE = 'NUMERIC_CHARACTER_REFERENCE_END_STATE';

  //Utils

  //OPTIMIZATION: these utility functions should not be moved out of this module. V8 Crankshaft will not inline
  //this functions if they will be situated in another module due to context switch.
  //Always perform inlining check before modifying this functions ('node --trace-inlining').
  function isWhitespace(cp) {
      return cp === $$5.SPACE || cp === $$5.LINE_FEED || cp === $$5.TABULATION || cp === $$5.FORM_FEED;
  }

  function isAsciiDigit(cp) {
      return cp >= $$5.DIGIT_0 && cp <= $$5.DIGIT_9;
  }

  function isAsciiUpper(cp) {
      return cp >= $$5.LATIN_CAPITAL_A && cp <= $$5.LATIN_CAPITAL_Z;
  }

  function isAsciiLower(cp) {
      return cp >= $$5.LATIN_SMALL_A && cp <= $$5.LATIN_SMALL_Z;
  }

  function isAsciiLetter(cp) {
      return isAsciiLower(cp) || isAsciiUpper(cp);
  }

  function isAsciiAlphaNumeric(cp) {
      return isAsciiLetter(cp) || isAsciiDigit(cp);
  }

  function isAsciiUpperHexDigit(cp) {
      return cp >= $$5.LATIN_CAPITAL_A && cp <= $$5.LATIN_CAPITAL_F;
  }

  function isAsciiLowerHexDigit(cp) {
      return cp >= $$5.LATIN_SMALL_A && cp <= $$5.LATIN_SMALL_F;
  }

  function isAsciiHexDigit(cp) {
      return isAsciiDigit(cp) || isAsciiUpperHexDigit(cp) || isAsciiLowerHexDigit(cp);
  }

  function toAsciiLowerCodePoint(cp) {
      return cp + 0x0020;
  }

  //NOTE: String.fromCharCode() function can handle only characters from BMP subset.
  //So, we need to workaround this manually.
  //(see: https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/String/fromCharCode#Getting_it_to_work_with_higher_values)
  function toChar(cp) {
      if (cp <= 0xffff) {
          return String.fromCharCode(cp);
      }

      cp -= 0x10000;
      return String.fromCharCode(((cp >>> 10) & 0x3ff) | 0xd800) + String.fromCharCode(0xdc00 | (cp & 0x3ff));
  }

  function toAsciiLowerChar(cp) {
      return String.fromCharCode(toAsciiLowerCodePoint(cp));
  }

  function findNamedEntityTreeBranch(nodeIx, cp) {
      const branchCount = neTree[++nodeIx];
      let lo = ++nodeIx;
      let hi = lo + branchCount - 1;

      while (lo <= hi) {
          const mid = (lo + hi) >>> 1;
          const midCp = neTree[mid];

          if (midCp < cp) {
              lo = mid + 1;
          } else if (midCp > cp) {
              hi = mid - 1;
          } else {
              return neTree[mid + branchCount];
          }
      }

      return -1;
  }

  //Tokenizer
  class Tokenizer$4 {
      constructor() {
          this.preprocessor = new Preprocessor();

          this.tokenQueue = [];

          this.allowCDATA = false;

          this.state = DATA_STATE;
          this.returnState = '';

          this.charRefCode = -1;
          this.tempBuff = [];
          this.lastStartTagName = '';

          this.consumedAfterSnapshot = -1;
          this.active = false;

          this.currentCharacterToken = null;
          this.currentToken = null;
          this.currentAttr = null;
      }

      //Errors
      _err() {
          // NOTE: err reporting is noop by default. Enabled by mixin.
      }

      _errOnNextCodePoint(err) {
          this._consume();
          this._err(err);
          this._unconsume();
      }

      //API
      getNextToken() {
          while (!this.tokenQueue.length && this.active) {
              this.consumedAfterSnapshot = 0;

              const cp = this._consume();

              if (!this._ensureHibernation()) {
                  this[this.state](cp);
              }
          }

          return this.tokenQueue.shift();
      }

      write(chunk, isLastChunk) {
          this.active = true;
          this.preprocessor.write(chunk, isLastChunk);
      }

      insertHtmlAtCurrentPos(chunk) {
          this.active = true;
          this.preprocessor.insertHtmlAtCurrentPos(chunk);
      }

      //Hibernation
      _ensureHibernation() {
          if (this.preprocessor.endOfChunkHit) {
              for (; this.consumedAfterSnapshot > 0; this.consumedAfterSnapshot--) {
                  this.preprocessor.retreat();
              }

              this.active = false;
              this.tokenQueue.push({ type: Tokenizer$4.HIBERNATION_TOKEN });

              return true;
          }

          return false;
      }

      //Consumption
      _consume() {
          this.consumedAfterSnapshot++;
          return this.preprocessor.advance();
      }

      _unconsume() {
          this.consumedAfterSnapshot--;
          this.preprocessor.retreat();
      }

      _reconsumeInState(state) {
          this.state = state;
          this._unconsume();
      }

      _consumeSequenceIfMatch(pattern, startCp, caseSensitive) {
          let consumedCount = 0;
          let isMatch = true;
          const patternLength = pattern.length;
          let patternPos = 0;
          let cp = startCp;
          let patternCp = void 0;

          for (; patternPos < patternLength; patternPos++) {
              if (patternPos > 0) {
                  cp = this._consume();
                  consumedCount++;
              }

              if (cp === $$5.EOF) {
                  isMatch = false;
                  break;
              }

              patternCp = pattern[patternPos];

              if (cp !== patternCp && (caseSensitive || cp !== toAsciiLowerCodePoint(patternCp))) {
                  isMatch = false;
                  break;
              }
          }

          if (!isMatch) {
              while (consumedCount--) {
                  this._unconsume();
              }
          }

          return isMatch;
      }

      //Temp buffer
      _isTempBufferEqualToScriptString() {
          if (this.tempBuff.length !== $$.SCRIPT_STRING.length) {
              return false;
          }

          for (let i = 0; i < this.tempBuff.length; i++) {
              if (this.tempBuff[i] !== $$.SCRIPT_STRING[i]) {
                  return false;
              }
          }

          return true;
      }

      //Token creation
      _createStartTagToken() {
          this.currentToken = {
              type: Tokenizer$4.START_TAG_TOKEN,
              tagName: '',
              selfClosing: false,
              ackSelfClosing: false,
              attrs: []
          };
      }

      _createEndTagToken() {
          this.currentToken = {
              type: Tokenizer$4.END_TAG_TOKEN,
              tagName: '',
              selfClosing: false,
              attrs: []
          };
      }

      _createCommentToken() {
          this.currentToken = {
              type: Tokenizer$4.COMMENT_TOKEN,
              data: ''
          };
      }

      _createDoctypeToken(initialName) {
          this.currentToken = {
              type: Tokenizer$4.DOCTYPE_TOKEN,
              name: initialName,
              forceQuirks: false,
              publicId: null,
              systemId: null
          };
      }

      _createCharacterToken(type, ch) {
          this.currentCharacterToken = {
              type: type,
              chars: ch
          };
      }

      _createEOFToken() {
          this.currentToken = { type: Tokenizer$4.EOF_TOKEN };
      }

      //Tag attributes
      _createAttr(attrNameFirstCh) {
          this.currentAttr = {
              name: attrNameFirstCh,
              value: ''
          };
      }

      _leaveAttrName(toState) {
          if (Tokenizer$4.getTokenAttr(this.currentToken, this.currentAttr.name) === null) {
              this.currentToken.attrs.push(this.currentAttr);
          } else {
              this._err(ERR$1.duplicateAttribute);
          }

          this.state = toState;
      }

      _leaveAttrValue(toState) {
          this.state = toState;
      }

      //Token emission
      _emitCurrentToken() {
          this._emitCurrentCharacterToken();

          const ct = this.currentToken;

          this.currentToken = null;

          //NOTE: store emited start tag's tagName to determine is the following end tag token is appropriate.
          if (ct.type === Tokenizer$4.START_TAG_TOKEN) {
              this.lastStartTagName = ct.tagName;
          } else if (ct.type === Tokenizer$4.END_TAG_TOKEN) {
              if (ct.attrs.length > 0) {
                  this._err(ERR$1.endTagWithAttributes);
              }

              if (ct.selfClosing) {
                  this._err(ERR$1.endTagWithTrailingSolidus);
              }
          }

          this.tokenQueue.push(ct);
      }

      _emitCurrentCharacterToken() {
          if (this.currentCharacterToken) {
              this.tokenQueue.push(this.currentCharacterToken);
              this.currentCharacterToken = null;
          }
      }

      _emitEOFToken() {
          this._createEOFToken();
          this._emitCurrentToken();
      }

      //Characters emission

      //OPTIMIZATION: specification uses only one type of character tokens (one token per character).
      //This causes a huge memory overhead and a lot of unnecessary parser loops. parse5 uses 3 groups of characters.
      //If we have a sequence of characters that belong to the same group, parser can process it
      //as a single solid character token.
      //So, there are 3 types of character tokens in parse5:
      //1)NULL_CHARACTER_TOKEN - \u0000-character sequences (e.g. '\u0000\u0000\u0000')
      //2)WHITESPACE_CHARACTER_TOKEN - any whitespace/new-line character sequences (e.g. '\n  \r\t   \f')
      //3)CHARACTER_TOKEN - any character sequence which don't belong to groups 1 and 2 (e.g. 'abcdef1234@@#$%^')
      _appendCharToCurrentCharacterToken(type, ch) {
          if (this.currentCharacterToken && this.currentCharacterToken.type !== type) {
              this._emitCurrentCharacterToken();
          }

          if (this.currentCharacterToken) {
              this.currentCharacterToken.chars += ch;
          } else {
              this._createCharacterToken(type, ch);
          }
      }

      _emitCodePoint(cp) {
          let type = Tokenizer$4.CHARACTER_TOKEN;

          if (isWhitespace(cp)) {
              type = Tokenizer$4.WHITESPACE_CHARACTER_TOKEN;
          } else if (cp === $$5.NULL) {
              type = Tokenizer$4.NULL_CHARACTER_TOKEN;
          }

          this._appendCharToCurrentCharacterToken(type, toChar(cp));
      }

      _emitSeveralCodePoints(codePoints) {
          for (let i = 0; i < codePoints.length; i++) {
              this._emitCodePoint(codePoints[i]);
          }
      }

      //NOTE: used then we emit character explicitly. This is always a non-whitespace and a non-null character.
      //So we can avoid additional checks here.
      _emitChars(ch) {
          this._appendCharToCurrentCharacterToken(Tokenizer$4.CHARACTER_TOKEN, ch);
      }

      // Character reference helpers
      _matchNamedCharacterReference(startCp) {
          let result = null;
          let excess = 1;
          let i = findNamedEntityTreeBranch(0, startCp);

          this.tempBuff.push(startCp);

          while (i > -1) {
              const current = neTree[i];
              const inNode = current < MAX_BRANCH_MARKER_VALUE;
              const nodeWithData = inNode && current & HAS_DATA_FLAG;

              if (nodeWithData) {
                  //NOTE: we use greedy search, so we continue lookup at this point
                  result = current & DATA_DUPLET_FLAG ? [neTree[++i], neTree[++i]] : [neTree[++i]];
                  excess = 0;
              }

              const cp = this._consume();

              this.tempBuff.push(cp);
              excess++;

              if (cp === $$5.EOF) {
                  break;
              }

              if (inNode) {
                  i = current & HAS_BRANCHES_FLAG ? findNamedEntityTreeBranch(i, cp) : -1;
              } else {
                  i = cp === current ? ++i : -1;
              }
          }

          while (excess--) {
              this.tempBuff.pop();
              this._unconsume();
          }

          return result;
      }

      _isCharacterReferenceInAttribute() {
          return (
              this.returnState === ATTRIBUTE_VALUE_DOUBLE_QUOTED_STATE ||
              this.returnState === ATTRIBUTE_VALUE_SINGLE_QUOTED_STATE ||
              this.returnState === ATTRIBUTE_VALUE_UNQUOTED_STATE
          );
      }

      _isCharacterReferenceAttributeQuirk(withSemicolon) {
          if (!withSemicolon && this._isCharacterReferenceInAttribute()) {
              const nextCp = this._consume();

              this._unconsume();

              return nextCp === $$5.EQUALS_SIGN || isAsciiAlphaNumeric(nextCp);
          }

          return false;
      }

      _flushCodePointsConsumedAsCharacterReference() {
          if (this._isCharacterReferenceInAttribute()) {
              for (let i = 0; i < this.tempBuff.length; i++) {
                  this.currentAttr.value += toChar(this.tempBuff[i]);
              }
          } else {
              this._emitSeveralCodePoints(this.tempBuff);
          }

          this.tempBuff = [];
      }

      // State machine

      // Data state
      //------------------------------------------------------------------
      [DATA_STATE](cp) {
          this.preprocessor.dropParsedChunk();

          if (cp === $$5.LESS_THAN_SIGN) {
              this.state = TAG_OPEN_STATE;
          } else if (cp === $$5.AMPERSAND) {
              this.returnState = DATA_STATE;
              this.state = CHARACTER_REFERENCE_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this._emitCodePoint(cp);
          } else if (cp === $$5.EOF) {
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      //  RCDATA state
      //------------------------------------------------------------------
      [RCDATA_STATE](cp) {
          this.preprocessor.dropParsedChunk();

          if (cp === $$5.AMPERSAND) {
              this.returnState = RCDATA_STATE;
              this.state = CHARACTER_REFERENCE_STATE;
          } else if (cp === $$5.LESS_THAN_SIGN) {
              this.state = RCDATA_LESS_THAN_SIGN_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this._emitChars(unicode$1.REPLACEMENT_CHARACTER);
          } else if (cp === $$5.EOF) {
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      // RAWTEXT state
      //------------------------------------------------------------------
      [RAWTEXT_STATE](cp) {
          this.preprocessor.dropParsedChunk();

          if (cp === $$5.LESS_THAN_SIGN) {
              this.state = RAWTEXT_LESS_THAN_SIGN_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this._emitChars(unicode$1.REPLACEMENT_CHARACTER);
          } else if (cp === $$5.EOF) {
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      // Script data state
      //------------------------------------------------------------------
      [SCRIPT_DATA_STATE](cp) {
          this.preprocessor.dropParsedChunk();

          if (cp === $$5.LESS_THAN_SIGN) {
              this.state = SCRIPT_DATA_LESS_THAN_SIGN_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this._emitChars(unicode$1.REPLACEMENT_CHARACTER);
          } else if (cp === $$5.EOF) {
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      // PLAINTEXT state
      //------------------------------------------------------------------
      [PLAINTEXT_STATE](cp) {
          this.preprocessor.dropParsedChunk();

          if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this._emitChars(unicode$1.REPLACEMENT_CHARACTER);
          } else if (cp === $$5.EOF) {
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      // Tag open state
      //------------------------------------------------------------------
      [TAG_OPEN_STATE](cp) {
          if (cp === $$5.EXCLAMATION_MARK) {
              this.state = MARKUP_DECLARATION_OPEN_STATE;
          } else if (cp === $$5.SOLIDUS) {
              this.state = END_TAG_OPEN_STATE;
          } else if (isAsciiLetter(cp)) {
              this._createStartTagToken();
              this._reconsumeInState(TAG_NAME_STATE);
          } else if (cp === $$5.QUESTION_MARK) {
              this._err(ERR$1.unexpectedQuestionMarkInsteadOfTagName);
              this._createCommentToken();
              this._reconsumeInState(BOGUS_COMMENT_STATE);
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofBeforeTagName);
              this._emitChars('<');
              this._emitEOFToken();
          } else {
              this._err(ERR$1.invalidFirstCharacterOfTagName);
              this._emitChars('<');
              this._reconsumeInState(DATA_STATE);
          }
      }

      // End tag open state
      //------------------------------------------------------------------
      [END_TAG_OPEN_STATE](cp) {
          if (isAsciiLetter(cp)) {
              this._createEndTagToken();
              this._reconsumeInState(TAG_NAME_STATE);
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._err(ERR$1.missingEndTagName);
              this.state = DATA_STATE;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofBeforeTagName);
              this._emitChars('</');
              this._emitEOFToken();
          } else {
              this._err(ERR$1.invalidFirstCharacterOfTagName);
              this._createCommentToken();
              this._reconsumeInState(BOGUS_COMMENT_STATE);
          }
      }

      // Tag name state
      //------------------------------------------------------------------
      [TAG_NAME_STATE](cp) {
          if (isWhitespace(cp)) {
              this.state = BEFORE_ATTRIBUTE_NAME_STATE;
          } else if (cp === $$5.SOLIDUS) {
              this.state = SELF_CLOSING_START_TAG_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else if (isAsciiUpper(cp)) {
              this.currentToken.tagName += toAsciiLowerChar(cp);
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.currentToken.tagName += unicode$1.REPLACEMENT_CHARACTER;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInTag);
              this._emitEOFToken();
          } else {
              this.currentToken.tagName += toChar(cp);
          }
      }

      // RCDATA less-than sign state
      //------------------------------------------------------------------
      [RCDATA_LESS_THAN_SIGN_STATE](cp) {
          if (cp === $$5.SOLIDUS) {
              this.tempBuff = [];
              this.state = RCDATA_END_TAG_OPEN_STATE;
          } else {
              this._emitChars('<');
              this._reconsumeInState(RCDATA_STATE);
          }
      }

      // RCDATA end tag open state
      //------------------------------------------------------------------
      [RCDATA_END_TAG_OPEN_STATE](cp) {
          if (isAsciiLetter(cp)) {
              this._createEndTagToken();
              this._reconsumeInState(RCDATA_END_TAG_NAME_STATE);
          } else {
              this._emitChars('</');
              this._reconsumeInState(RCDATA_STATE);
          }
      }

      // RCDATA end tag name state
      //------------------------------------------------------------------
      [RCDATA_END_TAG_NAME_STATE](cp) {
          if (isAsciiUpper(cp)) {
              this.currentToken.tagName += toAsciiLowerChar(cp);
              this.tempBuff.push(cp);
          } else if (isAsciiLower(cp)) {
              this.currentToken.tagName += toChar(cp);
              this.tempBuff.push(cp);
          } else {
              if (this.lastStartTagName === this.currentToken.tagName) {
                  if (isWhitespace(cp)) {
                      this.state = BEFORE_ATTRIBUTE_NAME_STATE;
                      return;
                  }

                  if (cp === $$5.SOLIDUS) {
                      this.state = SELF_CLOSING_START_TAG_STATE;
                      return;
                  }

                  if (cp === $$5.GREATER_THAN_SIGN) {
                      this.state = DATA_STATE;
                      this._emitCurrentToken();
                      return;
                  }
              }

              this._emitChars('</');
              this._emitSeveralCodePoints(this.tempBuff);
              this._reconsumeInState(RCDATA_STATE);
          }
      }

      // RAWTEXT less-than sign state
      //------------------------------------------------------------------
      [RAWTEXT_LESS_THAN_SIGN_STATE](cp) {
          if (cp === $$5.SOLIDUS) {
              this.tempBuff = [];
              this.state = RAWTEXT_END_TAG_OPEN_STATE;
          } else {
              this._emitChars('<');
              this._reconsumeInState(RAWTEXT_STATE);
          }
      }

      // RAWTEXT end tag open state
      //------------------------------------------------------------------
      [RAWTEXT_END_TAG_OPEN_STATE](cp) {
          if (isAsciiLetter(cp)) {
              this._createEndTagToken();
              this._reconsumeInState(RAWTEXT_END_TAG_NAME_STATE);
          } else {
              this._emitChars('</');
              this._reconsumeInState(RAWTEXT_STATE);
          }
      }

      // RAWTEXT end tag name state
      //------------------------------------------------------------------
      [RAWTEXT_END_TAG_NAME_STATE](cp) {
          if (isAsciiUpper(cp)) {
              this.currentToken.tagName += toAsciiLowerChar(cp);
              this.tempBuff.push(cp);
          } else if (isAsciiLower(cp)) {
              this.currentToken.tagName += toChar(cp);
              this.tempBuff.push(cp);
          } else {
              if (this.lastStartTagName === this.currentToken.tagName) {
                  if (isWhitespace(cp)) {
                      this.state = BEFORE_ATTRIBUTE_NAME_STATE;
                      return;
                  }

                  if (cp === $$5.SOLIDUS) {
                      this.state = SELF_CLOSING_START_TAG_STATE;
                      return;
                  }

                  if (cp === $$5.GREATER_THAN_SIGN) {
                      this._emitCurrentToken();
                      this.state = DATA_STATE;
                      return;
                  }
              }

              this._emitChars('</');
              this._emitSeveralCodePoints(this.tempBuff);
              this._reconsumeInState(RAWTEXT_STATE);
          }
      }

      // Script data less-than sign state
      //------------------------------------------------------------------
      [SCRIPT_DATA_LESS_THAN_SIGN_STATE](cp) {
          if (cp === $$5.SOLIDUS) {
              this.tempBuff = [];
              this.state = SCRIPT_DATA_END_TAG_OPEN_STATE;
          } else if (cp === $$5.EXCLAMATION_MARK) {
              this.state = SCRIPT_DATA_ESCAPE_START_STATE;
              this._emitChars('<!');
          } else {
              this._emitChars('<');
              this._reconsumeInState(SCRIPT_DATA_STATE);
          }
      }

      // Script data end tag open state
      //------------------------------------------------------------------
      [SCRIPT_DATA_END_TAG_OPEN_STATE](cp) {
          if (isAsciiLetter(cp)) {
              this._createEndTagToken();
              this._reconsumeInState(SCRIPT_DATA_END_TAG_NAME_STATE);
          } else {
              this._emitChars('</');
              this._reconsumeInState(SCRIPT_DATA_STATE);
          }
      }

      // Script data end tag name state
      //------------------------------------------------------------------
      [SCRIPT_DATA_END_TAG_NAME_STATE](cp) {
          if (isAsciiUpper(cp)) {
              this.currentToken.tagName += toAsciiLowerChar(cp);
              this.tempBuff.push(cp);
          } else if (isAsciiLower(cp)) {
              this.currentToken.tagName += toChar(cp);
              this.tempBuff.push(cp);
          } else {
              if (this.lastStartTagName === this.currentToken.tagName) {
                  if (isWhitespace(cp)) {
                      this.state = BEFORE_ATTRIBUTE_NAME_STATE;
                      return;
                  } else if (cp === $$5.SOLIDUS) {
                      this.state = SELF_CLOSING_START_TAG_STATE;
                      return;
                  } else if (cp === $$5.GREATER_THAN_SIGN) {
                      this._emitCurrentToken();
                      this.state = DATA_STATE;
                      return;
                  }
              }

              this._emitChars('</');
              this._emitSeveralCodePoints(this.tempBuff);
              this._reconsumeInState(SCRIPT_DATA_STATE);
          }
      }

      // Script data escape start state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPE_START_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this.state = SCRIPT_DATA_ESCAPE_START_DASH_STATE;
              this._emitChars('-');
          } else {
              this._reconsumeInState(SCRIPT_DATA_STATE);
          }
      }

      // Script data escape start dash state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPE_START_DASH_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this.state = SCRIPT_DATA_ESCAPED_DASH_DASH_STATE;
              this._emitChars('-');
          } else {
              this._reconsumeInState(SCRIPT_DATA_STATE);
          }
      }

      // Script data escaped state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPED_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this.state = SCRIPT_DATA_ESCAPED_DASH_STATE;
              this._emitChars('-');
          } else if (cp === $$5.LESS_THAN_SIGN) {
              this.state = SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this._emitChars(unicode$1.REPLACEMENT_CHARACTER);
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInScriptHtmlCommentLikeText);
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      // Script data escaped dash state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPED_DASH_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this.state = SCRIPT_DATA_ESCAPED_DASH_DASH_STATE;
              this._emitChars('-');
          } else if (cp === $$5.LESS_THAN_SIGN) {
              this.state = SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.state = SCRIPT_DATA_ESCAPED_STATE;
              this._emitChars(unicode$1.REPLACEMENT_CHARACTER);
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInScriptHtmlCommentLikeText);
              this._emitEOFToken();
          } else {
              this.state = SCRIPT_DATA_ESCAPED_STATE;
              this._emitCodePoint(cp);
          }
      }

      // Script data escaped dash dash state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPED_DASH_DASH_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this._emitChars('-');
          } else if (cp === $$5.LESS_THAN_SIGN) {
              this.state = SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this.state = SCRIPT_DATA_STATE;
              this._emitChars('>');
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.state = SCRIPT_DATA_ESCAPED_STATE;
              this._emitChars(unicode$1.REPLACEMENT_CHARACTER);
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInScriptHtmlCommentLikeText);
              this._emitEOFToken();
          } else {
              this.state = SCRIPT_DATA_ESCAPED_STATE;
              this._emitCodePoint(cp);
          }
      }

      // Script data escaped less-than sign state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN_STATE](cp) {
          if (cp === $$5.SOLIDUS) {
              this.tempBuff = [];
              this.state = SCRIPT_DATA_ESCAPED_END_TAG_OPEN_STATE;
          } else if (isAsciiLetter(cp)) {
              this.tempBuff = [];
              this._emitChars('<');
              this._reconsumeInState(SCRIPT_DATA_DOUBLE_ESCAPE_START_STATE);
          } else {
              this._emitChars('<');
              this._reconsumeInState(SCRIPT_DATA_ESCAPED_STATE);
          }
      }

      // Script data escaped end tag open state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPED_END_TAG_OPEN_STATE](cp) {
          if (isAsciiLetter(cp)) {
              this._createEndTagToken();
              this._reconsumeInState(SCRIPT_DATA_ESCAPED_END_TAG_NAME_STATE);
          } else {
              this._emitChars('</');
              this._reconsumeInState(SCRIPT_DATA_ESCAPED_STATE);
          }
      }

      // Script data escaped end tag name state
      //------------------------------------------------------------------
      [SCRIPT_DATA_ESCAPED_END_TAG_NAME_STATE](cp) {
          if (isAsciiUpper(cp)) {
              this.currentToken.tagName += toAsciiLowerChar(cp);
              this.tempBuff.push(cp);
          } else if (isAsciiLower(cp)) {
              this.currentToken.tagName += toChar(cp);
              this.tempBuff.push(cp);
          } else {
              if (this.lastStartTagName === this.currentToken.tagName) {
                  if (isWhitespace(cp)) {
                      this.state = BEFORE_ATTRIBUTE_NAME_STATE;
                      return;
                  }

                  if (cp === $$5.SOLIDUS) {
                      this.state = SELF_CLOSING_START_TAG_STATE;
                      return;
                  }

                  if (cp === $$5.GREATER_THAN_SIGN) {
                      this._emitCurrentToken();
                      this.state = DATA_STATE;
                      return;
                  }
              }

              this._emitChars('</');
              this._emitSeveralCodePoints(this.tempBuff);
              this._reconsumeInState(SCRIPT_DATA_ESCAPED_STATE);
          }
      }

      // Script data double escape start state
      //------------------------------------------------------------------
      [SCRIPT_DATA_DOUBLE_ESCAPE_START_STATE](cp) {
          if (isWhitespace(cp) || cp === $$5.SOLIDUS || cp === $$5.GREATER_THAN_SIGN) {
              this.state = this._isTempBufferEqualToScriptString()
                  ? SCRIPT_DATA_DOUBLE_ESCAPED_STATE
                  : SCRIPT_DATA_ESCAPED_STATE;
              this._emitCodePoint(cp);
          } else if (isAsciiUpper(cp)) {
              this.tempBuff.push(toAsciiLowerCodePoint(cp));
              this._emitCodePoint(cp);
          } else if (isAsciiLower(cp)) {
              this.tempBuff.push(cp);
              this._emitCodePoint(cp);
          } else {
              this._reconsumeInState(SCRIPT_DATA_ESCAPED_STATE);
          }
      }

      // Script data double escaped state
      //------------------------------------------------------------------
      [SCRIPT_DATA_DOUBLE_ESCAPED_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_DASH_STATE;
              this._emitChars('-');
          } else if (cp === $$5.LESS_THAN_SIGN) {
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN_STATE;
              this._emitChars('<');
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this._emitChars(unicode$1.REPLACEMENT_CHARACTER);
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInScriptHtmlCommentLikeText);
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      // Script data double escaped dash state
      //------------------------------------------------------------------
      [SCRIPT_DATA_DOUBLE_ESCAPED_DASH_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH_STATE;
              this._emitChars('-');
          } else if (cp === $$5.LESS_THAN_SIGN) {
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN_STATE;
              this._emitChars('<');
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_STATE;
              this._emitChars(unicode$1.REPLACEMENT_CHARACTER);
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInScriptHtmlCommentLikeText);
              this._emitEOFToken();
          } else {
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_STATE;
              this._emitCodePoint(cp);
          }
      }

      // Script data double escaped dash dash state
      //------------------------------------------------------------------
      [SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this._emitChars('-');
          } else if (cp === $$5.LESS_THAN_SIGN) {
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN_STATE;
              this._emitChars('<');
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this.state = SCRIPT_DATA_STATE;
              this._emitChars('>');
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_STATE;
              this._emitChars(unicode$1.REPLACEMENT_CHARACTER);
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInScriptHtmlCommentLikeText);
              this._emitEOFToken();
          } else {
              this.state = SCRIPT_DATA_DOUBLE_ESCAPED_STATE;
              this._emitCodePoint(cp);
          }
      }

      // Script data double escaped less-than sign state
      //------------------------------------------------------------------
      [SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN_STATE](cp) {
          if (cp === $$5.SOLIDUS) {
              this.tempBuff = [];
              this.state = SCRIPT_DATA_DOUBLE_ESCAPE_END_STATE;
              this._emitChars('/');
          } else {
              this._reconsumeInState(SCRIPT_DATA_DOUBLE_ESCAPED_STATE);
          }
      }

      // Script data double escape end state
      //------------------------------------------------------------------
      [SCRIPT_DATA_DOUBLE_ESCAPE_END_STATE](cp) {
          if (isWhitespace(cp) || cp === $$5.SOLIDUS || cp === $$5.GREATER_THAN_SIGN) {
              this.state = this._isTempBufferEqualToScriptString()
                  ? SCRIPT_DATA_ESCAPED_STATE
                  : SCRIPT_DATA_DOUBLE_ESCAPED_STATE;

              this._emitCodePoint(cp);
          } else if (isAsciiUpper(cp)) {
              this.tempBuff.push(toAsciiLowerCodePoint(cp));
              this._emitCodePoint(cp);
          } else if (isAsciiLower(cp)) {
              this.tempBuff.push(cp);
              this._emitCodePoint(cp);
          } else {
              this._reconsumeInState(SCRIPT_DATA_DOUBLE_ESCAPED_STATE);
          }
      }

      // Before attribute name state
      //------------------------------------------------------------------
      [BEFORE_ATTRIBUTE_NAME_STATE](cp) {
          if (isWhitespace(cp)) {
              return;
          }

          if (cp === $$5.SOLIDUS || cp === $$5.GREATER_THAN_SIGN || cp === $$5.EOF) {
              this._reconsumeInState(AFTER_ATTRIBUTE_NAME_STATE);
          } else if (cp === $$5.EQUALS_SIGN) {
              this._err(ERR$1.unexpectedEqualsSignBeforeAttributeName);
              this._createAttr('=');
              this.state = ATTRIBUTE_NAME_STATE;
          } else {
              this._createAttr('');
              this._reconsumeInState(ATTRIBUTE_NAME_STATE);
          }
      }

      // Attribute name state
      //------------------------------------------------------------------
      [ATTRIBUTE_NAME_STATE](cp) {
          if (isWhitespace(cp) || cp === $$5.SOLIDUS || cp === $$5.GREATER_THAN_SIGN || cp === $$5.EOF) {
              this._leaveAttrName(AFTER_ATTRIBUTE_NAME_STATE);
              this._unconsume();
          } else if (cp === $$5.EQUALS_SIGN) {
              this._leaveAttrName(BEFORE_ATTRIBUTE_VALUE_STATE);
          } else if (isAsciiUpper(cp)) {
              this.currentAttr.name += toAsciiLowerChar(cp);
          } else if (cp === $$5.QUOTATION_MARK || cp === $$5.APOSTROPHE || cp === $$5.LESS_THAN_SIGN) {
              this._err(ERR$1.unexpectedCharacterInAttributeName);
              this.currentAttr.name += toChar(cp);
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.currentAttr.name += unicode$1.REPLACEMENT_CHARACTER;
          } else {
              this.currentAttr.name += toChar(cp);
          }
      }

      // After attribute name state
      //------------------------------------------------------------------
      [AFTER_ATTRIBUTE_NAME_STATE](cp) {
          if (isWhitespace(cp)) {
              return;
          }

          if (cp === $$5.SOLIDUS) {
              this.state = SELF_CLOSING_START_TAG_STATE;
          } else if (cp === $$5.EQUALS_SIGN) {
              this.state = BEFORE_ATTRIBUTE_VALUE_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInTag);
              this._emitEOFToken();
          } else {
              this._createAttr('');
              this._reconsumeInState(ATTRIBUTE_NAME_STATE);
          }
      }

      // Before attribute value state
      //------------------------------------------------------------------
      [BEFORE_ATTRIBUTE_VALUE_STATE](cp) {
          if (isWhitespace(cp)) {
              return;
          }

          if (cp === $$5.QUOTATION_MARK) {
              this.state = ATTRIBUTE_VALUE_DOUBLE_QUOTED_STATE;
          } else if (cp === $$5.APOSTROPHE) {
              this.state = ATTRIBUTE_VALUE_SINGLE_QUOTED_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._err(ERR$1.missingAttributeValue);
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else {
              this._reconsumeInState(ATTRIBUTE_VALUE_UNQUOTED_STATE);
          }
      }

      // Attribute value (double-quoted) state
      //------------------------------------------------------------------
      [ATTRIBUTE_VALUE_DOUBLE_QUOTED_STATE](cp) {
          if (cp === $$5.QUOTATION_MARK) {
              this.state = AFTER_ATTRIBUTE_VALUE_QUOTED_STATE;
          } else if (cp === $$5.AMPERSAND) {
              this.returnState = ATTRIBUTE_VALUE_DOUBLE_QUOTED_STATE;
              this.state = CHARACTER_REFERENCE_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.currentAttr.value += unicode$1.REPLACEMENT_CHARACTER;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInTag);
              this._emitEOFToken();
          } else {
              this.currentAttr.value += toChar(cp);
          }
      }

      // Attribute value (single-quoted) state
      //------------------------------------------------------------------
      [ATTRIBUTE_VALUE_SINGLE_QUOTED_STATE](cp) {
          if (cp === $$5.APOSTROPHE) {
              this.state = AFTER_ATTRIBUTE_VALUE_QUOTED_STATE;
          } else if (cp === $$5.AMPERSAND) {
              this.returnState = ATTRIBUTE_VALUE_SINGLE_QUOTED_STATE;
              this.state = CHARACTER_REFERENCE_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.currentAttr.value += unicode$1.REPLACEMENT_CHARACTER;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInTag);
              this._emitEOFToken();
          } else {
              this.currentAttr.value += toChar(cp);
          }
      }

      // Attribute value (unquoted) state
      //------------------------------------------------------------------
      [ATTRIBUTE_VALUE_UNQUOTED_STATE](cp) {
          if (isWhitespace(cp)) {
              this._leaveAttrValue(BEFORE_ATTRIBUTE_NAME_STATE);
          } else if (cp === $$5.AMPERSAND) {
              this.returnState = ATTRIBUTE_VALUE_UNQUOTED_STATE;
              this.state = CHARACTER_REFERENCE_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._leaveAttrValue(DATA_STATE);
              this._emitCurrentToken();
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.currentAttr.value += unicode$1.REPLACEMENT_CHARACTER;
          } else if (
              cp === $$5.QUOTATION_MARK ||
              cp === $$5.APOSTROPHE ||
              cp === $$5.LESS_THAN_SIGN ||
              cp === $$5.EQUALS_SIGN ||
              cp === $$5.GRAVE_ACCENT
          ) {
              this._err(ERR$1.unexpectedCharacterInUnquotedAttributeValue);
              this.currentAttr.value += toChar(cp);
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInTag);
              this._emitEOFToken();
          } else {
              this.currentAttr.value += toChar(cp);
          }
      }

      // After attribute value (quoted) state
      //------------------------------------------------------------------
      [AFTER_ATTRIBUTE_VALUE_QUOTED_STATE](cp) {
          if (isWhitespace(cp)) {
              this._leaveAttrValue(BEFORE_ATTRIBUTE_NAME_STATE);
          } else if (cp === $$5.SOLIDUS) {
              this._leaveAttrValue(SELF_CLOSING_START_TAG_STATE);
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._leaveAttrValue(DATA_STATE);
              this._emitCurrentToken();
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInTag);
              this._emitEOFToken();
          } else {
              this._err(ERR$1.missingWhitespaceBetweenAttributes);
              this._reconsumeInState(BEFORE_ATTRIBUTE_NAME_STATE);
          }
      }

      // Self-closing start tag state
      //------------------------------------------------------------------
      [SELF_CLOSING_START_TAG_STATE](cp) {
          if (cp === $$5.GREATER_THAN_SIGN) {
              this.currentToken.selfClosing = true;
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInTag);
              this._emitEOFToken();
          } else {
              this._err(ERR$1.unexpectedSolidusInTag);
              this._reconsumeInState(BEFORE_ATTRIBUTE_NAME_STATE);
          }
      }

      // Bogus comment state
      //------------------------------------------------------------------
      [BOGUS_COMMENT_STATE](cp) {
          if (cp === $$5.GREATER_THAN_SIGN) {
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else if (cp === $$5.EOF) {
              this._emitCurrentToken();
              this._emitEOFToken();
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.currentToken.data += unicode$1.REPLACEMENT_CHARACTER;
          } else {
              this.currentToken.data += toChar(cp);
          }
      }

      // Markup declaration open state
      //------------------------------------------------------------------
      [MARKUP_DECLARATION_OPEN_STATE](cp) {
          if (this._consumeSequenceIfMatch($$.DASH_DASH_STRING, cp, true)) {
              this._createCommentToken();
              this.state = COMMENT_START_STATE;
          } else if (this._consumeSequenceIfMatch($$.DOCTYPE_STRING, cp, false)) {
              this.state = DOCTYPE_STATE;
          } else if (this._consumeSequenceIfMatch($$.CDATA_START_STRING, cp, true)) {
              if (this.allowCDATA) {
                  this.state = CDATA_SECTION_STATE;
              } else {
                  this._err(ERR$1.cdataInHtmlContent);
                  this._createCommentToken();
                  this.currentToken.data = '[CDATA[';
                  this.state = BOGUS_COMMENT_STATE;
              }
          }

          //NOTE: sequence lookup can be abrupted by hibernation. In that case lookup
          //results are no longer valid and we will need to start over.
          else if (!this._ensureHibernation()) {
              this._err(ERR$1.incorrectlyOpenedComment);
              this._createCommentToken();
              this._reconsumeInState(BOGUS_COMMENT_STATE);
          }
      }

      // Comment start state
      //------------------------------------------------------------------
      [COMMENT_START_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this.state = COMMENT_START_DASH_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._err(ERR$1.abruptClosingOfEmptyComment);
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else {
              this._reconsumeInState(COMMENT_STATE);
          }
      }

      // Comment start dash state
      //------------------------------------------------------------------
      [COMMENT_START_DASH_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this.state = COMMENT_END_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._err(ERR$1.abruptClosingOfEmptyComment);
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInComment);
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.data += '-';
              this._reconsumeInState(COMMENT_STATE);
          }
      }

      // Comment state
      //------------------------------------------------------------------
      [COMMENT_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this.state = COMMENT_END_DASH_STATE;
          } else if (cp === $$5.LESS_THAN_SIGN) {
              this.currentToken.data += '<';
              this.state = COMMENT_LESS_THAN_SIGN_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.currentToken.data += unicode$1.REPLACEMENT_CHARACTER;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInComment);
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.data += toChar(cp);
          }
      }

      // Comment less-than sign state
      //------------------------------------------------------------------
      [COMMENT_LESS_THAN_SIGN_STATE](cp) {
          if (cp === $$5.EXCLAMATION_MARK) {
              this.currentToken.data += '!';
              this.state = COMMENT_LESS_THAN_SIGN_BANG_STATE;
          } else if (cp === $$5.LESS_THAN_SIGN) {
              this.currentToken.data += '!';
          } else {
              this._reconsumeInState(COMMENT_STATE);
          }
      }

      // Comment less-than sign bang state
      //------------------------------------------------------------------
      [COMMENT_LESS_THAN_SIGN_BANG_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this.state = COMMENT_LESS_THAN_SIGN_BANG_DASH_STATE;
          } else {
              this._reconsumeInState(COMMENT_STATE);
          }
      }

      // Comment less-than sign bang dash state
      //------------------------------------------------------------------
      [COMMENT_LESS_THAN_SIGN_BANG_DASH_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this.state = COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH_STATE;
          } else {
              this._reconsumeInState(COMMENT_END_DASH_STATE);
          }
      }

      // Comment less-than sign bang dash dash state
      //------------------------------------------------------------------
      [COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH_STATE](cp) {
          if (cp !== $$5.GREATER_THAN_SIGN && cp !== $$5.EOF) {
              this._err(ERR$1.nestedComment);
          }

          this._reconsumeInState(COMMENT_END_STATE);
      }

      // Comment end dash state
      //------------------------------------------------------------------
      [COMMENT_END_DASH_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this.state = COMMENT_END_STATE;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInComment);
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.data += '-';
              this._reconsumeInState(COMMENT_STATE);
          }
      }

      // Comment end state
      //------------------------------------------------------------------
      [COMMENT_END_STATE](cp) {
          if (cp === $$5.GREATER_THAN_SIGN) {
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else if (cp === $$5.EXCLAMATION_MARK) {
              this.state = COMMENT_END_BANG_STATE;
          } else if (cp === $$5.HYPHEN_MINUS) {
              this.currentToken.data += '-';
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInComment);
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.data += '--';
              this._reconsumeInState(COMMENT_STATE);
          }
      }

      // Comment end bang state
      //------------------------------------------------------------------
      [COMMENT_END_BANG_STATE](cp) {
          if (cp === $$5.HYPHEN_MINUS) {
              this.currentToken.data += '--!';
              this.state = COMMENT_END_DASH_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._err(ERR$1.incorrectlyClosedComment);
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInComment);
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.data += '--!';
              this._reconsumeInState(COMMENT_STATE);
          }
      }

      // DOCTYPE state
      //------------------------------------------------------------------
      [DOCTYPE_STATE](cp) {
          if (isWhitespace(cp)) {
              this.state = BEFORE_DOCTYPE_NAME_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._reconsumeInState(BEFORE_DOCTYPE_NAME_STATE);
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this._createDoctypeToken(null);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$1.missingWhitespaceBeforeDoctypeName);
              this._reconsumeInState(BEFORE_DOCTYPE_NAME_STATE);
          }
      }

      // Before DOCTYPE name state
      //------------------------------------------------------------------
      [BEFORE_DOCTYPE_NAME_STATE](cp) {
          if (isWhitespace(cp)) {
              return;
          }

          if (isAsciiUpper(cp)) {
              this._createDoctypeToken(toAsciiLowerChar(cp));
              this.state = DOCTYPE_NAME_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this._createDoctypeToken(unicode$1.REPLACEMENT_CHARACTER);
              this.state = DOCTYPE_NAME_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._err(ERR$1.missingDoctypeName);
              this._createDoctypeToken(null);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this.state = DATA_STATE;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this._createDoctypeToken(null);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._createDoctypeToken(toChar(cp));
              this.state = DOCTYPE_NAME_STATE;
          }
      }

      // DOCTYPE name state
      //------------------------------------------------------------------
      [DOCTYPE_NAME_STATE](cp) {
          if (isWhitespace(cp)) {
              this.state = AFTER_DOCTYPE_NAME_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else if (isAsciiUpper(cp)) {
              this.currentToken.name += toAsciiLowerChar(cp);
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.currentToken.name += unicode$1.REPLACEMENT_CHARACTER;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.name += toChar(cp);
          }
      }

      // After DOCTYPE name state
      //------------------------------------------------------------------
      [AFTER_DOCTYPE_NAME_STATE](cp) {
          if (isWhitespace(cp)) {
              return;
          }

          if (cp === $$5.GREATER_THAN_SIGN) {
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else if (this._consumeSequenceIfMatch($$.PUBLIC_STRING, cp, false)) {
              this.state = AFTER_DOCTYPE_PUBLIC_KEYWORD_STATE;
          } else if (this._consumeSequenceIfMatch($$.SYSTEM_STRING, cp, false)) {
              this.state = AFTER_DOCTYPE_SYSTEM_KEYWORD_STATE;
          }
          //NOTE: sequence lookup can be abrupted by hibernation. In that case lookup
          //results are no longer valid and we will need to start over.
          else if (!this._ensureHibernation()) {
              this._err(ERR$1.invalidCharacterSequenceAfterDoctypeName);
              this.currentToken.forceQuirks = true;
              this._reconsumeInState(BOGUS_DOCTYPE_STATE);
          }
      }

      // After DOCTYPE public keyword state
      //------------------------------------------------------------------
      [AFTER_DOCTYPE_PUBLIC_KEYWORD_STATE](cp) {
          if (isWhitespace(cp)) {
              this.state = BEFORE_DOCTYPE_PUBLIC_IDENTIFIER_STATE;
          } else if (cp === $$5.QUOTATION_MARK) {
              this._err(ERR$1.missingWhitespaceAfterDoctypePublicKeyword);
              this.currentToken.publicId = '';
              this.state = DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED_STATE;
          } else if (cp === $$5.APOSTROPHE) {
              this._err(ERR$1.missingWhitespaceAfterDoctypePublicKeyword);
              this.currentToken.publicId = '';
              this.state = DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._err(ERR$1.missingDoctypePublicIdentifier);
              this.currentToken.forceQuirks = true;
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$1.missingQuoteBeforeDoctypePublicIdentifier);
              this.currentToken.forceQuirks = true;
              this._reconsumeInState(BOGUS_DOCTYPE_STATE);
          }
      }

      // Before DOCTYPE public identifier state
      //------------------------------------------------------------------
      [BEFORE_DOCTYPE_PUBLIC_IDENTIFIER_STATE](cp) {
          if (isWhitespace(cp)) {
              return;
          }

          if (cp === $$5.QUOTATION_MARK) {
              this.currentToken.publicId = '';
              this.state = DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED_STATE;
          } else if (cp === $$5.APOSTROPHE) {
              this.currentToken.publicId = '';
              this.state = DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._err(ERR$1.missingDoctypePublicIdentifier);
              this.currentToken.forceQuirks = true;
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$1.missingQuoteBeforeDoctypePublicIdentifier);
              this.currentToken.forceQuirks = true;
              this._reconsumeInState(BOGUS_DOCTYPE_STATE);
          }
      }

      // DOCTYPE public identifier (double-quoted) state
      //------------------------------------------------------------------
      [DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED_STATE](cp) {
          if (cp === $$5.QUOTATION_MARK) {
              this.state = AFTER_DOCTYPE_PUBLIC_IDENTIFIER_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.currentToken.publicId += unicode$1.REPLACEMENT_CHARACTER;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._err(ERR$1.abruptDoctypePublicIdentifier);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this.state = DATA_STATE;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.publicId += toChar(cp);
          }
      }

      // DOCTYPE public identifier (single-quoted) state
      //------------------------------------------------------------------
      [DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED_STATE](cp) {
          if (cp === $$5.APOSTROPHE) {
              this.state = AFTER_DOCTYPE_PUBLIC_IDENTIFIER_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.currentToken.publicId += unicode$1.REPLACEMENT_CHARACTER;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._err(ERR$1.abruptDoctypePublicIdentifier);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this.state = DATA_STATE;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.publicId += toChar(cp);
          }
      }

      // After DOCTYPE public identifier state
      //------------------------------------------------------------------
      [AFTER_DOCTYPE_PUBLIC_IDENTIFIER_STATE](cp) {
          if (isWhitespace(cp)) {
              this.state = BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else if (cp === $$5.QUOTATION_MARK) {
              this._err(ERR$1.missingWhitespaceBetweenDoctypePublicAndSystemIdentifiers);
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED_STATE;
          } else if (cp === $$5.APOSTROPHE) {
              this._err(ERR$1.missingWhitespaceBetweenDoctypePublicAndSystemIdentifiers);
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED_STATE;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$1.missingQuoteBeforeDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this._reconsumeInState(BOGUS_DOCTYPE_STATE);
          }
      }

      // Between DOCTYPE public and system identifiers state
      //------------------------------------------------------------------
      [BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS_STATE](cp) {
          if (isWhitespace(cp)) {
              return;
          }

          if (cp === $$5.GREATER_THAN_SIGN) {
              this._emitCurrentToken();
              this.state = DATA_STATE;
          } else if (cp === $$5.QUOTATION_MARK) {
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED_STATE;
          } else if (cp === $$5.APOSTROPHE) {
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED_STATE;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$1.missingQuoteBeforeDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this._reconsumeInState(BOGUS_DOCTYPE_STATE);
          }
      }

      // After DOCTYPE system keyword state
      //------------------------------------------------------------------
      [AFTER_DOCTYPE_SYSTEM_KEYWORD_STATE](cp) {
          if (isWhitespace(cp)) {
              this.state = BEFORE_DOCTYPE_SYSTEM_IDENTIFIER_STATE;
          } else if (cp === $$5.QUOTATION_MARK) {
              this._err(ERR$1.missingWhitespaceAfterDoctypeSystemKeyword);
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED_STATE;
          } else if (cp === $$5.APOSTROPHE) {
              this._err(ERR$1.missingWhitespaceAfterDoctypeSystemKeyword);
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._err(ERR$1.missingDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$1.missingQuoteBeforeDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this._reconsumeInState(BOGUS_DOCTYPE_STATE);
          }
      }

      // Before DOCTYPE system identifier state
      //------------------------------------------------------------------
      [BEFORE_DOCTYPE_SYSTEM_IDENTIFIER_STATE](cp) {
          if (isWhitespace(cp)) {
              return;
          }

          if (cp === $$5.QUOTATION_MARK) {
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED_STATE;
          } else if (cp === $$5.APOSTROPHE) {
              this.currentToken.systemId = '';
              this.state = DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED_STATE;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._err(ERR$1.missingDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this.state = DATA_STATE;
              this._emitCurrentToken();
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$1.missingQuoteBeforeDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this._reconsumeInState(BOGUS_DOCTYPE_STATE);
          }
      }

      // DOCTYPE system identifier (double-quoted) state
      //------------------------------------------------------------------
      [DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED_STATE](cp) {
          if (cp === $$5.QUOTATION_MARK) {
              this.state = AFTER_DOCTYPE_SYSTEM_IDENTIFIER_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.currentToken.systemId += unicode$1.REPLACEMENT_CHARACTER;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._err(ERR$1.abruptDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this.state = DATA_STATE;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.systemId += toChar(cp);
          }
      }

      // DOCTYPE system identifier (single-quoted) state
      //------------------------------------------------------------------
      [DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED_STATE](cp) {
          if (cp === $$5.APOSTROPHE) {
              this.state = AFTER_DOCTYPE_SYSTEM_IDENTIFIER_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
              this.currentToken.systemId += unicode$1.REPLACEMENT_CHARACTER;
          } else if (cp === $$5.GREATER_THAN_SIGN) {
              this._err(ERR$1.abruptDoctypeSystemIdentifier);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this.state = DATA_STATE;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this.currentToken.systemId += toChar(cp);
          }
      }

      // After DOCTYPE system identifier state
      //------------------------------------------------------------------
      [AFTER_DOCTYPE_SYSTEM_IDENTIFIER_STATE](cp) {
          if (isWhitespace(cp)) {
              return;
          }

          if (cp === $$5.GREATER_THAN_SIGN) {
              this._emitCurrentToken();
              this.state = DATA_STATE;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInDoctype);
              this.currentToken.forceQuirks = true;
              this._emitCurrentToken();
              this._emitEOFToken();
          } else {
              this._err(ERR$1.unexpectedCharacterAfterDoctypeSystemIdentifier);
              this._reconsumeInState(BOGUS_DOCTYPE_STATE);
          }
      }

      // Bogus DOCTYPE state
      //------------------------------------------------------------------
      [BOGUS_DOCTYPE_STATE](cp) {
          if (cp === $$5.GREATER_THAN_SIGN) {
              this._emitCurrentToken();
              this.state = DATA_STATE;
          } else if (cp === $$5.NULL) {
              this._err(ERR$1.unexpectedNullCharacter);
          } else if (cp === $$5.EOF) {
              this._emitCurrentToken();
              this._emitEOFToken();
          }
      }

      // CDATA section state
      //------------------------------------------------------------------
      [CDATA_SECTION_STATE](cp) {
          if (cp === $$5.RIGHT_SQUARE_BRACKET) {
              this.state = CDATA_SECTION_BRACKET_STATE;
          } else if (cp === $$5.EOF) {
              this._err(ERR$1.eofInCdata);
              this._emitEOFToken();
          } else {
              this._emitCodePoint(cp);
          }
      }

      // CDATA section bracket state
      //------------------------------------------------------------------
      [CDATA_SECTION_BRACKET_STATE](cp) {
          if (cp === $$5.RIGHT_SQUARE_BRACKET) {
              this.state = CDATA_SECTION_END_STATE;
          } else {
              this._emitChars(']');
              this._reconsumeInState(CDATA_SECTION_STATE);
          }
      }

      // CDATA section end state
      //------------------------------------------------------------------
      [CDATA_SECTION_END_STATE](cp) {
          if (cp === $$5.GREATER_THAN_SIGN) {
              this.state = DATA_STATE;
          } else if (cp === $$5.RIGHT_SQUARE_BRACKET) {
              this._emitChars(']');
          } else {
              this._emitChars(']]');
              this._reconsumeInState(CDATA_SECTION_STATE);
          }
      }

      // Character reference state
      //------------------------------------------------------------------
      [CHARACTER_REFERENCE_STATE](cp) {
          this.tempBuff = [$$5.AMPERSAND];

          if (cp === $$5.NUMBER_SIGN) {
              this.tempBuff.push(cp);
              this.state = NUMERIC_CHARACTER_REFERENCE_STATE;
          } else if (isAsciiAlphaNumeric(cp)) {
              this._reconsumeInState(NAMED_CHARACTER_REFERENCE_STATE);
          } else {
              this._flushCodePointsConsumedAsCharacterReference();
              this._reconsumeInState(this.returnState);
          }
      }

      // Named character reference state
      //------------------------------------------------------------------
      [NAMED_CHARACTER_REFERENCE_STATE](cp) {
          const matchResult = this._matchNamedCharacterReference(cp);

          //NOTE: matching can be abrupted by hibernation. In that case match
          //results are no longer valid and we will need to start over.
          if (this._ensureHibernation()) {
              this.tempBuff = [$$5.AMPERSAND];
          } else if (matchResult) {
              const withSemicolon = this.tempBuff[this.tempBuff.length - 1] === $$5.SEMICOLON;

              if (!this._isCharacterReferenceAttributeQuirk(withSemicolon)) {
                  if (!withSemicolon) {
                      this._errOnNextCodePoint(ERR$1.missingSemicolonAfterCharacterReference);
                  }

                  this.tempBuff = matchResult;
              }

              this._flushCodePointsConsumedAsCharacterReference();
              this.state = this.returnState;
          } else {
              this._flushCodePointsConsumedAsCharacterReference();
              this.state = AMBIGUOUS_AMPERSAND_STATE;
          }
      }

      // Ambiguos ampersand state
      //------------------------------------------------------------------
      [AMBIGUOUS_AMPERSAND_STATE](cp) {
          if (isAsciiAlphaNumeric(cp)) {
              if (this._isCharacterReferenceInAttribute()) {
                  this.currentAttr.value += toChar(cp);
              } else {
                  this._emitCodePoint(cp);
              }
          } else {
              if (cp === $$5.SEMICOLON) {
                  this._err(ERR$1.unknownNamedCharacterReference);
              }

              this._reconsumeInState(this.returnState);
          }
      }

      // Numeric character reference state
      //------------------------------------------------------------------
      [NUMERIC_CHARACTER_REFERENCE_STATE](cp) {
          this.charRefCode = 0;

          if (cp === $$5.LATIN_SMALL_X || cp === $$5.LATIN_CAPITAL_X) {
              this.tempBuff.push(cp);
              this.state = HEXADEMICAL_CHARACTER_REFERENCE_START_STATE;
          } else {
              this._reconsumeInState(DECIMAL_CHARACTER_REFERENCE_START_STATE);
          }
      }

      // Hexademical character reference start state
      //------------------------------------------------------------------
      [HEXADEMICAL_CHARACTER_REFERENCE_START_STATE](cp) {
          if (isAsciiHexDigit(cp)) {
              this._reconsumeInState(HEXADEMICAL_CHARACTER_REFERENCE_STATE);
          } else {
              this._err(ERR$1.absenceOfDigitsInNumericCharacterReference);
              this._flushCodePointsConsumedAsCharacterReference();
              this._reconsumeInState(this.returnState);
          }
      }

      // Decimal character reference start state
      //------------------------------------------------------------------
      [DECIMAL_CHARACTER_REFERENCE_START_STATE](cp) {
          if (isAsciiDigit(cp)) {
              this._reconsumeInState(DECIMAL_CHARACTER_REFERENCE_STATE);
          } else {
              this._err(ERR$1.absenceOfDigitsInNumericCharacterReference);
              this._flushCodePointsConsumedAsCharacterReference();
              this._reconsumeInState(this.returnState);
          }
      }

      // Hexademical character reference state
      //------------------------------------------------------------------
      [HEXADEMICAL_CHARACTER_REFERENCE_STATE](cp) {
          if (isAsciiUpperHexDigit(cp)) {
              this.charRefCode = this.charRefCode * 16 + cp - 0x37;
          } else if (isAsciiLowerHexDigit(cp)) {
              this.charRefCode = this.charRefCode * 16 + cp - 0x57;
          } else if (isAsciiDigit(cp)) {
              this.charRefCode = this.charRefCode * 16 + cp - 0x30;
          } else if (cp === $$5.SEMICOLON) {
              this.state = NUMERIC_CHARACTER_REFERENCE_END_STATE;
          } else {
              this._err(ERR$1.missingSemicolonAfterCharacterReference);
              this._reconsumeInState(NUMERIC_CHARACTER_REFERENCE_END_STATE);
          }
      }

      // Decimal character reference state
      //------------------------------------------------------------------
      [DECIMAL_CHARACTER_REFERENCE_STATE](cp) {
          if (isAsciiDigit(cp)) {
              this.charRefCode = this.charRefCode * 10 + cp - 0x30;
          } else if (cp === $$5.SEMICOLON) {
              this.state = NUMERIC_CHARACTER_REFERENCE_END_STATE;
          } else {
              this._err(ERR$1.missingSemicolonAfterCharacterReference);
              this._reconsumeInState(NUMERIC_CHARACTER_REFERENCE_END_STATE);
          }
      }

      // Numeric character reference end state
      //------------------------------------------------------------------
      [NUMERIC_CHARACTER_REFERENCE_END_STATE]() {
          if (this.charRefCode === $$5.NULL) {
              this._err(ERR$1.nullCharacterReference);
              this.charRefCode = $$5.REPLACEMENT_CHARACTER;
          } else if (this.charRefCode > 0x10ffff) {
              this._err(ERR$1.characterReferenceOutsideUnicodeRange);
              this.charRefCode = $$5.REPLACEMENT_CHARACTER;
          } else if (unicode$1.isSurrogate(this.charRefCode)) {
              this._err(ERR$1.surrogateCharacterReference);
              this.charRefCode = $$5.REPLACEMENT_CHARACTER;
          } else if (unicode$1.isUndefinedCodePoint(this.charRefCode)) {
              this._err(ERR$1.noncharacterCharacterReference);
          } else if (unicode$1.isControlCodePoint(this.charRefCode) || this.charRefCode === $$5.CARRIAGE_RETURN) {
              this._err(ERR$1.controlCharacterReference);

              const replacement = C1_CONTROLS_REFERENCE_REPLACEMENTS[this.charRefCode];

              if (replacement) {
                  this.charRefCode = replacement;
              }
          }

          this.tempBuff = [this.charRefCode];

          this._flushCodePointsConsumedAsCharacterReference();
          this._reconsumeInState(this.returnState);
      }
  }

  //Token types
  Tokenizer$4.CHARACTER_TOKEN = 'CHARACTER_TOKEN';
  Tokenizer$4.NULL_CHARACTER_TOKEN = 'NULL_CHARACTER_TOKEN';
  Tokenizer$4.WHITESPACE_CHARACTER_TOKEN = 'WHITESPACE_CHARACTER_TOKEN';
  Tokenizer$4.START_TAG_TOKEN = 'START_TAG_TOKEN';
  Tokenizer$4.END_TAG_TOKEN = 'END_TAG_TOKEN';
  Tokenizer$4.COMMENT_TOKEN = 'COMMENT_TOKEN';
  Tokenizer$4.DOCTYPE_TOKEN = 'DOCTYPE_TOKEN';
  Tokenizer$4.EOF_TOKEN = 'EOF_TOKEN';
  Tokenizer$4.HIBERNATION_TOKEN = 'HIBERNATION_TOKEN';

  //Tokenizer initial states for different modes
  Tokenizer$4.MODE = {
      DATA: DATA_STATE,
      RCDATA: RCDATA_STATE,
      RAWTEXT: RAWTEXT_STATE,
      SCRIPT_DATA: SCRIPT_DATA_STATE,
      PLAINTEXT: PLAINTEXT_STATE
  };

  //Static
  Tokenizer$4.getTokenAttr = function(token, attrName) {
      for (let i = token.attrs.length - 1; i >= 0; i--) {
          if (token.attrs[i].name === attrName) {
              return token.attrs[i].value;
          }
      }

      return null;
  };

  var tokenizer = Tokenizer$4;

  var html$e = {};

  const NS$3 = (html$e.NAMESPACES = {
      HTML: 'http://www.w3.org/1999/xhtml',
      MATHML: 'http://www.w3.org/1998/Math/MathML',
      SVG: 'http://www.w3.org/2000/svg',
      XLINK: 'http://www.w3.org/1999/xlink',
      XML: 'http://www.w3.org/XML/1998/namespace',
      XMLNS: 'http://www.w3.org/2000/xmlns/'
  });

  html$e.ATTRS = {
      TYPE: 'type',
      ACTION: 'action',
      ENCODING: 'encoding',
      PROMPT: 'prompt',
      NAME: 'name',
      COLOR: 'color',
      FACE: 'face',
      SIZE: 'size'
  };

  html$e.DOCUMENT_MODE = {
      NO_QUIRKS: 'no-quirks',
      QUIRKS: 'quirks',
      LIMITED_QUIRKS: 'limited-quirks'
  };

  const $$4 = (html$e.TAG_NAMES = {
      A: 'a',
      ADDRESS: 'address',
      ANNOTATION_XML: 'annotation-xml',
      APPLET: 'applet',
      AREA: 'area',
      ARTICLE: 'article',
      ASIDE: 'aside',

      B: 'b',
      BASE: 'base',
      BASEFONT: 'basefont',
      BGSOUND: 'bgsound',
      BIG: 'big',
      BLOCKQUOTE: 'blockquote',
      BODY: 'body',
      BR: 'br',
      BUTTON: 'button',

      CAPTION: 'caption',
      CENTER: 'center',
      CODE: 'code',
      COL: 'col',
      COLGROUP: 'colgroup',

      DD: 'dd',
      DESC: 'desc',
      DETAILS: 'details',
      DIALOG: 'dialog',
      DIR: 'dir',
      DIV: 'div',
      DL: 'dl',
      DT: 'dt',

      EM: 'em',
      EMBED: 'embed',

      FIELDSET: 'fieldset',
      FIGCAPTION: 'figcaption',
      FIGURE: 'figure',
      FONT: 'font',
      FOOTER: 'footer',
      FOREIGN_OBJECT: 'foreignObject',
      FORM: 'form',
      FRAME: 'frame',
      FRAMESET: 'frameset',

      H1: 'h1',
      H2: 'h2',
      H3: 'h3',
      H4: 'h4',
      H5: 'h5',
      H6: 'h6',
      HEAD: 'head',
      HEADER: 'header',
      HGROUP: 'hgroup',
      HR: 'hr',
      HTML: 'html',

      I: 'i',
      IMG: 'img',
      IMAGE: 'image',
      INPUT: 'input',
      IFRAME: 'iframe',

      KEYGEN: 'keygen',

      LABEL: 'label',
      LI: 'li',
      LINK: 'link',
      LISTING: 'listing',

      MAIN: 'main',
      MALIGNMARK: 'malignmark',
      MARQUEE: 'marquee',
      MATH: 'math',
      MENU: 'menu',
      META: 'meta',
      MGLYPH: 'mglyph',
      MI: 'mi',
      MO: 'mo',
      MN: 'mn',
      MS: 'ms',
      MTEXT: 'mtext',

      NAV: 'nav',
      NOBR: 'nobr',
      NOFRAMES: 'noframes',
      NOEMBED: 'noembed',
      NOSCRIPT: 'noscript',

      OBJECT: 'object',
      OL: 'ol',
      OPTGROUP: 'optgroup',
      OPTION: 'option',

      P: 'p',
      PARAM: 'param',
      PLAINTEXT: 'plaintext',
      PRE: 'pre',

      RB: 'rb',
      RP: 'rp',
      RT: 'rt',
      RTC: 'rtc',
      RUBY: 'ruby',

      S: 's',
      SCRIPT: 'script',
      SECTION: 'section',
      SELECT: 'select',
      SOURCE: 'source',
      SMALL: 'small',
      SPAN: 'span',
      STRIKE: 'strike',
      STRONG: 'strong',
      STYLE: 'style',
      SUB: 'sub',
      SUMMARY: 'summary',
      SUP: 'sup',

      TABLE: 'table',
      TBODY: 'tbody',
      TEMPLATE: 'template',
      TEXTAREA: 'textarea',
      TFOOT: 'tfoot',
      TD: 'td',
      TH: 'th',
      THEAD: 'thead',
      TITLE: 'title',
      TR: 'tr',
      TRACK: 'track',
      TT: 'tt',

      U: 'u',
      UL: 'ul',

      SVG: 'svg',

      VAR: 'var',

      WBR: 'wbr',

      XMP: 'xmp'
  });

  html$e.SPECIAL_ELEMENTS = {
      [NS$3.HTML]: {
          [$$4.ADDRESS]: true,
          [$$4.APPLET]: true,
          [$$4.AREA]: true,
          [$$4.ARTICLE]: true,
          [$$4.ASIDE]: true,
          [$$4.BASE]: true,
          [$$4.BASEFONT]: true,
          [$$4.BGSOUND]: true,
          [$$4.BLOCKQUOTE]: true,
          [$$4.BODY]: true,
          [$$4.BR]: true,
          [$$4.BUTTON]: true,
          [$$4.CAPTION]: true,
          [$$4.CENTER]: true,
          [$$4.COL]: true,
          [$$4.COLGROUP]: true,
          [$$4.DD]: true,
          [$$4.DETAILS]: true,
          [$$4.DIR]: true,
          [$$4.DIV]: true,
          [$$4.DL]: true,
          [$$4.DT]: true,
          [$$4.EMBED]: true,
          [$$4.FIELDSET]: true,
          [$$4.FIGCAPTION]: true,
          [$$4.FIGURE]: true,
          [$$4.FOOTER]: true,
          [$$4.FORM]: true,
          [$$4.FRAME]: true,
          [$$4.FRAMESET]: true,
          [$$4.H1]: true,
          [$$4.H2]: true,
          [$$4.H3]: true,
          [$$4.H4]: true,
          [$$4.H5]: true,
          [$$4.H6]: true,
          [$$4.HEAD]: true,
          [$$4.HEADER]: true,
          [$$4.HGROUP]: true,
          [$$4.HR]: true,
          [$$4.HTML]: true,
          [$$4.IFRAME]: true,
          [$$4.IMG]: true,
          [$$4.INPUT]: true,
          [$$4.LI]: true,
          [$$4.LINK]: true,
          [$$4.LISTING]: true,
          [$$4.MAIN]: true,
          [$$4.MARQUEE]: true,
          [$$4.MENU]: true,
          [$$4.META]: true,
          [$$4.NAV]: true,
          [$$4.NOEMBED]: true,
          [$$4.NOFRAMES]: true,
          [$$4.NOSCRIPT]: true,
          [$$4.OBJECT]: true,
          [$$4.OL]: true,
          [$$4.P]: true,
          [$$4.PARAM]: true,
          [$$4.PLAINTEXT]: true,
          [$$4.PRE]: true,
          [$$4.SCRIPT]: true,
          [$$4.SECTION]: true,
          [$$4.SELECT]: true,
          [$$4.SOURCE]: true,
          [$$4.STYLE]: true,
          [$$4.SUMMARY]: true,
          [$$4.TABLE]: true,
          [$$4.TBODY]: true,
          [$$4.TD]: true,
          [$$4.TEMPLATE]: true,
          [$$4.TEXTAREA]: true,
          [$$4.TFOOT]: true,
          [$$4.TH]: true,
          [$$4.THEAD]: true,
          [$$4.TITLE]: true,
          [$$4.TR]: true,
          [$$4.TRACK]: true,
          [$$4.UL]: true,
          [$$4.WBR]: true,
          [$$4.XMP]: true
      },
      [NS$3.MATHML]: {
          [$$4.MI]: true,
          [$$4.MO]: true,
          [$$4.MN]: true,
          [$$4.MS]: true,
          [$$4.MTEXT]: true,
          [$$4.ANNOTATION_XML]: true
      },
      [NS$3.SVG]: {
          [$$4.TITLE]: true,
          [$$4.FOREIGN_OBJECT]: true,
          [$$4.DESC]: true
      }
  };

  const HTML$3 = html$e;

  //Aliases
  const $$3 = HTML$3.TAG_NAMES;
  const NS$2 = HTML$3.NAMESPACES;

  //Element utils

  //OPTIMIZATION: Integer comparisons are low-cost, so we can use very fast tag name length filters here.
  //It's faster than using dictionary.
  function isImpliedEndTagRequired(tn) {
      switch (tn.length) {
          case 1:
              return tn === $$3.P;

          case 2:
              return tn === $$3.RB || tn === $$3.RP || tn === $$3.RT || tn === $$3.DD || tn === $$3.DT || tn === $$3.LI;

          case 3:
              return tn === $$3.RTC;

          case 6:
              return tn === $$3.OPTION;

          case 8:
              return tn === $$3.OPTGROUP;
      }

      return false;
  }

  function isImpliedEndTagRequiredThoroughly(tn) {
      switch (tn.length) {
          case 1:
              return tn === $$3.P;

          case 2:
              return (
                  tn === $$3.RB ||
                  tn === $$3.RP ||
                  tn === $$3.RT ||
                  tn === $$3.DD ||
                  tn === $$3.DT ||
                  tn === $$3.LI ||
                  tn === $$3.TD ||
                  tn === $$3.TH ||
                  tn === $$3.TR
              );

          case 3:
              return tn === $$3.RTC;

          case 5:
              return tn === $$3.TBODY || tn === $$3.TFOOT || tn === $$3.THEAD;

          case 6:
              return tn === $$3.OPTION;

          case 7:
              return tn === $$3.CAPTION;

          case 8:
              return tn === $$3.OPTGROUP || tn === $$3.COLGROUP;
      }

      return false;
  }

  function isScopingElement(tn, ns) {
      switch (tn.length) {
          case 2:
              if (tn === $$3.TD || tn === $$3.TH) {
                  return ns === NS$2.HTML;
              } else if (tn === $$3.MI || tn === $$3.MO || tn === $$3.MN || tn === $$3.MS) {
                  return ns === NS$2.MATHML;
              }

              break;

          case 4:
              if (tn === $$3.HTML) {
                  return ns === NS$2.HTML;
              } else if (tn === $$3.DESC) {
                  return ns === NS$2.SVG;
              }

              break;

          case 5:
              if (tn === $$3.TABLE) {
                  return ns === NS$2.HTML;
              } else if (tn === $$3.MTEXT) {
                  return ns === NS$2.MATHML;
              } else if (tn === $$3.TITLE) {
                  return ns === NS$2.SVG;
              }

              break;

          case 6:
              return (tn === $$3.APPLET || tn === $$3.OBJECT) && ns === NS$2.HTML;

          case 7:
              return (tn === $$3.CAPTION || tn === $$3.MARQUEE) && ns === NS$2.HTML;

          case 8:
              return tn === $$3.TEMPLATE && ns === NS$2.HTML;

          case 13:
              return tn === $$3.FOREIGN_OBJECT && ns === NS$2.SVG;

          case 14:
              return tn === $$3.ANNOTATION_XML && ns === NS$2.MATHML;
      }

      return false;
  }

  //Stack of open elements
  class OpenElementStack$1 {
      constructor(document, treeAdapter) {
          this.stackTop = -1;
          this.items = [];
          this.current = document;
          this.currentTagName = null;
          this.currentTmplContent = null;
          this.tmplCount = 0;
          this.treeAdapter = treeAdapter;
      }

      //Index of element
      _indexOf(element) {
          let idx = -1;

          for (let i = this.stackTop; i >= 0; i--) {
              if (this.items[i] === element) {
                  idx = i;
                  break;
              }
          }
          return idx;
      }

      //Update current element
      _isInTemplate() {
          return this.currentTagName === $$3.TEMPLATE && this.treeAdapter.getNamespaceURI(this.current) === NS$2.HTML;
      }

      _updateCurrentElement() {
          this.current = this.items[this.stackTop];
          this.currentTagName = this.current && this.treeAdapter.getTagName(this.current);

          this.currentTmplContent = this._isInTemplate() ? this.treeAdapter.getTemplateContent(this.current) : null;
      }

      //Mutations
      push(element) {
          this.items[++this.stackTop] = element;
          this._updateCurrentElement();

          if (this._isInTemplate()) {
              this.tmplCount++;
          }
      }

      pop() {
          this.stackTop--;

          if (this.tmplCount > 0 && this._isInTemplate()) {
              this.tmplCount--;
          }

          this._updateCurrentElement();
      }

      replace(oldElement, newElement) {
          const idx = this._indexOf(oldElement);

          this.items[idx] = newElement;

          if (idx === this.stackTop) {
              this._updateCurrentElement();
          }
      }

      insertAfter(referenceElement, newElement) {
          const insertionIdx = this._indexOf(referenceElement) + 1;

          this.items.splice(insertionIdx, 0, newElement);

          if (insertionIdx === ++this.stackTop) {
              this._updateCurrentElement();
          }
      }

      popUntilTagNamePopped(tagName) {
          while (this.stackTop > -1) {
              const tn = this.currentTagName;
              const ns = this.treeAdapter.getNamespaceURI(this.current);

              this.pop();

              if (tn === tagName && ns === NS$2.HTML) {
                  break;
              }
          }
      }

      popUntilElementPopped(element) {
          while (this.stackTop > -1) {
              const poppedElement = this.current;

              this.pop();

              if (poppedElement === element) {
                  break;
              }
          }
      }

      popUntilNumberedHeaderPopped() {
          while (this.stackTop > -1) {
              const tn = this.currentTagName;
              const ns = this.treeAdapter.getNamespaceURI(this.current);

              this.pop();

              if (
                  tn === $$3.H1 ||
                  tn === $$3.H2 ||
                  tn === $$3.H3 ||
                  tn === $$3.H4 ||
                  tn === $$3.H5 ||
                  (tn === $$3.H6 && ns === NS$2.HTML)
              ) {
                  break;
              }
          }
      }

      popUntilTableCellPopped() {
          while (this.stackTop > -1) {
              const tn = this.currentTagName;
              const ns = this.treeAdapter.getNamespaceURI(this.current);

              this.pop();

              if (tn === $$3.TD || (tn === $$3.TH && ns === NS$2.HTML)) {
                  break;
              }
          }
      }

      popAllUpToHtmlElement() {
          //NOTE: here we assume that root <html> element is always first in the open element stack, so
          //we perform this fast stack clean up.
          this.stackTop = 0;
          this._updateCurrentElement();
      }

      clearBackToTableContext() {
          while (
              (this.currentTagName !== $$3.TABLE && this.currentTagName !== $$3.TEMPLATE && this.currentTagName !== $$3.HTML) ||
              this.treeAdapter.getNamespaceURI(this.current) !== NS$2.HTML
          ) {
              this.pop();
          }
      }

      clearBackToTableBodyContext() {
          while (
              (this.currentTagName !== $$3.TBODY &&
                  this.currentTagName !== $$3.TFOOT &&
                  this.currentTagName !== $$3.THEAD &&
                  this.currentTagName !== $$3.TEMPLATE &&
                  this.currentTagName !== $$3.HTML) ||
              this.treeAdapter.getNamespaceURI(this.current) !== NS$2.HTML
          ) {
              this.pop();
          }
      }

      clearBackToTableRowContext() {
          while (
              (this.currentTagName !== $$3.TR && this.currentTagName !== $$3.TEMPLATE && this.currentTagName !== $$3.HTML) ||
              this.treeAdapter.getNamespaceURI(this.current) !== NS$2.HTML
          ) {
              this.pop();
          }
      }

      remove(element) {
          for (let i = this.stackTop; i >= 0; i--) {
              if (this.items[i] === element) {
                  this.items.splice(i, 1);
                  this.stackTop--;
                  this._updateCurrentElement();
                  break;
              }
          }
      }

      //Search
      tryPeekProperlyNestedBodyElement() {
          //Properly nested <body> element (should be second element in stack).
          const element = this.items[1];

          return element && this.treeAdapter.getTagName(element) === $$3.BODY ? element : null;
      }

      contains(element) {
          return this._indexOf(element) > -1;
      }

      getCommonAncestor(element) {
          let elementIdx = this._indexOf(element);

          return --elementIdx >= 0 ? this.items[elementIdx] : null;
      }

      isRootHtmlElementCurrent() {
          return this.stackTop === 0 && this.currentTagName === $$3.HTML;
      }

      //Element in scope
      hasInScope(tagName) {
          for (let i = this.stackTop; i >= 0; i--) {
              const tn = this.treeAdapter.getTagName(this.items[i]);
              const ns = this.treeAdapter.getNamespaceURI(this.items[i]);

              if (tn === tagName && ns === NS$2.HTML) {
                  return true;
              }

              if (isScopingElement(tn, ns)) {
                  return false;
              }
          }

          return true;
      }

      hasNumberedHeaderInScope() {
          for (let i = this.stackTop; i >= 0; i--) {
              const tn = this.treeAdapter.getTagName(this.items[i]);
              const ns = this.treeAdapter.getNamespaceURI(this.items[i]);

              if (
                  (tn === $$3.H1 || tn === $$3.H2 || tn === $$3.H3 || tn === $$3.H4 || tn === $$3.H5 || tn === $$3.H6) &&
                  ns === NS$2.HTML
              ) {
                  return true;
              }

              if (isScopingElement(tn, ns)) {
                  return false;
              }
          }

          return true;
      }

      hasInListItemScope(tagName) {
          for (let i = this.stackTop; i >= 0; i--) {
              const tn = this.treeAdapter.getTagName(this.items[i]);
              const ns = this.treeAdapter.getNamespaceURI(this.items[i]);

              if (tn === tagName && ns === NS$2.HTML) {
                  return true;
              }

              if (((tn === $$3.UL || tn === $$3.OL) && ns === NS$2.HTML) || isScopingElement(tn, ns)) {
                  return false;
              }
          }

          return true;
      }

      hasInButtonScope(tagName) {
          for (let i = this.stackTop; i >= 0; i--) {
              const tn = this.treeAdapter.getTagName(this.items[i]);
              const ns = this.treeAdapter.getNamespaceURI(this.items[i]);

              if (tn === tagName && ns === NS$2.HTML) {
                  return true;
              }

              if ((tn === $$3.BUTTON && ns === NS$2.HTML) || isScopingElement(tn, ns)) {
                  return false;
              }
          }

          return true;
      }

      hasInTableScope(tagName) {
          for (let i = this.stackTop; i >= 0; i--) {
              const tn = this.treeAdapter.getTagName(this.items[i]);
              const ns = this.treeAdapter.getNamespaceURI(this.items[i]);

              if (ns !== NS$2.HTML) {
                  continue;
              }

              if (tn === tagName) {
                  return true;
              }

              if (tn === $$3.TABLE || tn === $$3.TEMPLATE || tn === $$3.HTML) {
                  return false;
              }
          }

          return true;
      }

      hasTableBodyContextInTableScope() {
          for (let i = this.stackTop; i >= 0; i--) {
              const tn = this.treeAdapter.getTagName(this.items[i]);
              const ns = this.treeAdapter.getNamespaceURI(this.items[i]);

              if (ns !== NS$2.HTML) {
                  continue;
              }

              if (tn === $$3.TBODY || tn === $$3.THEAD || tn === $$3.TFOOT) {
                  return true;
              }

              if (tn === $$3.TABLE || tn === $$3.HTML) {
                  return false;
              }
          }

          return true;
      }

      hasInSelectScope(tagName) {
          for (let i = this.stackTop; i >= 0; i--) {
              const tn = this.treeAdapter.getTagName(this.items[i]);
              const ns = this.treeAdapter.getNamespaceURI(this.items[i]);

              if (ns !== NS$2.HTML) {
                  continue;
              }

              if (tn === tagName) {
                  return true;
              }

              if (tn !== $$3.OPTION && tn !== $$3.OPTGROUP) {
                  return false;
              }
          }

          return true;
      }

      //Implied end tags
      generateImpliedEndTags() {
          while (isImpliedEndTagRequired(this.currentTagName)) {
              this.pop();
          }
      }

      generateImpliedEndTagsThoroughly() {
          while (isImpliedEndTagRequiredThoroughly(this.currentTagName)) {
              this.pop();
          }
      }

      generateImpliedEndTagsWithExclusion(exclusionTagName) {
          while (isImpliedEndTagRequired(this.currentTagName) && this.currentTagName !== exclusionTagName) {
              this.pop();
          }
      }
  }

  var openElementStack = OpenElementStack$1;

  //Const
  const NOAH_ARK_CAPACITY = 3;

  //List of formatting elements
  class FormattingElementList$1 {
      constructor(treeAdapter) {
          this.length = 0;
          this.entries = [];
          this.treeAdapter = treeAdapter;
          this.bookmark = null;
      }

      //Noah Ark's condition
      //OPTIMIZATION: at first we try to find possible candidates for exclusion using
      //lightweight heuristics without thorough attributes check.
      _getNoahArkConditionCandidates(newElement) {
          const candidates = [];

          if (this.length >= NOAH_ARK_CAPACITY) {
              const neAttrsLength = this.treeAdapter.getAttrList(newElement).length;
              const neTagName = this.treeAdapter.getTagName(newElement);
              const neNamespaceURI = this.treeAdapter.getNamespaceURI(newElement);

              for (let i = this.length - 1; i >= 0; i--) {
                  const entry = this.entries[i];

                  if (entry.type === FormattingElementList$1.MARKER_ENTRY) {
                      break;
                  }

                  const element = entry.element;
                  const elementAttrs = this.treeAdapter.getAttrList(element);

                  const isCandidate =
                      this.treeAdapter.getTagName(element) === neTagName &&
                      this.treeAdapter.getNamespaceURI(element) === neNamespaceURI &&
                      elementAttrs.length === neAttrsLength;

                  if (isCandidate) {
                      candidates.push({ idx: i, attrs: elementAttrs });
                  }
              }
          }

          return candidates.length < NOAH_ARK_CAPACITY ? [] : candidates;
      }

      _ensureNoahArkCondition(newElement) {
          const candidates = this._getNoahArkConditionCandidates(newElement);
          let cLength = candidates.length;

          if (cLength) {
              const neAttrs = this.treeAdapter.getAttrList(newElement);
              const neAttrsLength = neAttrs.length;
              const neAttrsMap = Object.create(null);

              //NOTE: build attrs map for the new element so we can perform fast lookups
              for (let i = 0; i < neAttrsLength; i++) {
                  const neAttr = neAttrs[i];

                  neAttrsMap[neAttr.name] = neAttr.value;
              }

              for (let i = 0; i < neAttrsLength; i++) {
                  for (let j = 0; j < cLength; j++) {
                      const cAttr = candidates[j].attrs[i];

                      if (neAttrsMap[cAttr.name] !== cAttr.value) {
                          candidates.splice(j, 1);
                          cLength--;
                      }

                      if (candidates.length < NOAH_ARK_CAPACITY) {
                          return;
                      }
                  }
              }

              //NOTE: remove bottommost candidates until Noah's Ark condition will not be met
              for (let i = cLength - 1; i >= NOAH_ARK_CAPACITY - 1; i--) {
                  this.entries.splice(candidates[i].idx, 1);
                  this.length--;
              }
          }
      }

      //Mutations
      insertMarker() {
          this.entries.push({ type: FormattingElementList$1.MARKER_ENTRY });
          this.length++;
      }

      pushElement(element, token) {
          this._ensureNoahArkCondition(element);

          this.entries.push({
              type: FormattingElementList$1.ELEMENT_ENTRY,
              element: element,
              token: token
          });

          this.length++;
      }

      insertElementAfterBookmark(element, token) {
          let bookmarkIdx = this.length - 1;

          for (; bookmarkIdx >= 0; bookmarkIdx--) {
              if (this.entries[bookmarkIdx] === this.bookmark) {
                  break;
              }
          }

          this.entries.splice(bookmarkIdx + 1, 0, {
              type: FormattingElementList$1.ELEMENT_ENTRY,
              element: element,
              token: token
          });

          this.length++;
      }

      removeEntry(entry) {
          for (let i = this.length - 1; i >= 0; i--) {
              if (this.entries[i] === entry) {
                  this.entries.splice(i, 1);
                  this.length--;
                  break;
              }
          }
      }

      clearToLastMarker() {
          while (this.length) {
              const entry = this.entries.pop();

              this.length--;

              if (entry.type === FormattingElementList$1.MARKER_ENTRY) {
                  break;
              }
          }
      }

      //Search
      getElementEntryInScopeWithTagName(tagName) {
          for (let i = this.length - 1; i >= 0; i--) {
              const entry = this.entries[i];

              if (entry.type === FormattingElementList$1.MARKER_ENTRY) {
                  return null;
              }

              if (this.treeAdapter.getTagName(entry.element) === tagName) {
                  return entry;
              }
          }

          return null;
      }

      getElementEntry(element) {
          for (let i = this.length - 1; i >= 0; i--) {
              const entry = this.entries[i];

              if (entry.type === FormattingElementList$1.ELEMENT_ENTRY && entry.element === element) {
                  return entry;
              }
          }

          return null;
      }
  }

  //Entry types
  FormattingElementList$1.MARKER_ENTRY = 'MARKER_ENTRY';
  FormattingElementList$1.ELEMENT_ENTRY = 'ELEMENT_ENTRY';

  var formattingElementList = FormattingElementList$1;

  class Mixin$9 {
      constructor(host) {
          const originalMethods = {};
          const overriddenMethods = this._getOverriddenMethods(this, originalMethods);

          for (const key of Object.keys(overriddenMethods)) {
              if (typeof overriddenMethods[key] === 'function') {
                  originalMethods[key] = host[key];
                  host[key] = overriddenMethods[key];
              }
          }
      }

      _getOverriddenMethods() {
          throw new Error('Not implemented');
      }
  }

  Mixin$9.install = function(host, Ctor, opts) {
      if (!host.__mixins) {
          host.__mixins = [];
      }

      for (let i = 0; i < host.__mixins.length; i++) {
          if (host.__mixins[i].constructor === Ctor) {
              return host.__mixins[i];
          }
      }

      const mixin = new Ctor(host, opts);

      host.__mixins.push(mixin);

      return mixin;
  };

  var mixin = Mixin$9;

  const Mixin$8 = mixin;

  class PositionTrackingPreprocessorMixin$2 extends Mixin$8 {
      constructor(preprocessor) {
          super(preprocessor);

          this.preprocessor = preprocessor;
          this.isEol = false;
          this.lineStartPos = 0;
          this.droppedBufferSize = 0;

          this.offset = 0;
          this.col = 0;
          this.line = 1;
      }

      _getOverriddenMethods(mxn, orig) {
          return {
              advance() {
                  const pos = this.pos + 1;
                  const ch = this.html[pos];

                  //NOTE: LF should be in the last column of the line
                  if (mxn.isEol) {
                      mxn.isEol = false;
                      mxn.line++;
                      mxn.lineStartPos = pos;
                  }

                  if (ch === '\n' || (ch === '\r' && this.html[pos + 1] !== '\n')) {
                      mxn.isEol = true;
                  }

                  mxn.col = pos - mxn.lineStartPos + 1;
                  mxn.offset = mxn.droppedBufferSize + pos;

                  return orig.advance.call(this);
              },

              retreat() {
                  orig.retreat.call(this);

                  mxn.isEol = false;
                  mxn.col = this.pos - mxn.lineStartPos + 1;
              },

              dropParsedChunk() {
                  const prevPos = this.pos;

                  orig.dropParsedChunk.call(this);

                  const reduction = prevPos - this.pos;

                  mxn.lineStartPos -= reduction;
                  mxn.droppedBufferSize += reduction;
                  mxn.offset = mxn.droppedBufferSize + this.pos;
              }
          };
      }
  }

  var preprocessorMixin$1 = PositionTrackingPreprocessorMixin$2;

  const Mixin$7 = mixin;
  const Tokenizer$3 = tokenizer;
  const PositionTrackingPreprocessorMixin$1 = preprocessorMixin$1;

  class LocationInfoTokenizerMixin$2 extends Mixin$7 {
      constructor(tokenizer) {
          super(tokenizer);

          this.tokenizer = tokenizer;
          this.posTracker = Mixin$7.install(tokenizer.preprocessor, PositionTrackingPreprocessorMixin$1);
          this.currentAttrLocation = null;
          this.ctLoc = null;
      }

      _getCurrentLocation() {
          return {
              startLine: this.posTracker.line,
              startCol: this.posTracker.col,
              startOffset: this.posTracker.offset,
              endLine: -1,
              endCol: -1,
              endOffset: -1
          };
      }

      _attachCurrentAttrLocationInfo() {
          this.currentAttrLocation.endLine = this.posTracker.line;
          this.currentAttrLocation.endCol = this.posTracker.col;
          this.currentAttrLocation.endOffset = this.posTracker.offset;

          const currentToken = this.tokenizer.currentToken;
          const currentAttr = this.tokenizer.currentAttr;

          if (!currentToken.location.attrs) {
              currentToken.location.attrs = Object.create(null);
          }

          currentToken.location.attrs[currentAttr.name] = this.currentAttrLocation;
      }

      _getOverriddenMethods(mxn, orig) {
          const methods = {
              _createStartTagToken() {
                  orig._createStartTagToken.call(this);
                  this.currentToken.location = mxn.ctLoc;
              },

              _createEndTagToken() {
                  orig._createEndTagToken.call(this);
                  this.currentToken.location = mxn.ctLoc;
              },

              _createCommentToken() {
                  orig._createCommentToken.call(this);
                  this.currentToken.location = mxn.ctLoc;
              },

              _createDoctypeToken(initialName) {
                  orig._createDoctypeToken.call(this, initialName);
                  this.currentToken.location = mxn.ctLoc;
              },

              _createCharacterToken(type, ch) {
                  orig._createCharacterToken.call(this, type, ch);
                  this.currentCharacterToken.location = mxn.ctLoc;
              },

              _createEOFToken() {
                  orig._createEOFToken.call(this);
                  this.currentToken.location = mxn._getCurrentLocation();
              },

              _createAttr(attrNameFirstCh) {
                  orig._createAttr.call(this, attrNameFirstCh);
                  mxn.currentAttrLocation = mxn._getCurrentLocation();
              },

              _leaveAttrName(toState) {
                  orig._leaveAttrName.call(this, toState);
                  mxn._attachCurrentAttrLocationInfo();
              },

              _leaveAttrValue(toState) {
                  orig._leaveAttrValue.call(this, toState);
                  mxn._attachCurrentAttrLocationInfo();
              },

              _emitCurrentToken() {
                  const ctLoc = this.currentToken.location;

                  //NOTE: if we have pending character token make it's end location equal to the
                  //current token's start location.
                  if (this.currentCharacterToken) {
                      this.currentCharacterToken.location.endLine = ctLoc.startLine;
                      this.currentCharacterToken.location.endCol = ctLoc.startCol;
                      this.currentCharacterToken.location.endOffset = ctLoc.startOffset;
                  }

                  if (this.currentToken.type === Tokenizer$3.EOF_TOKEN) {
                      ctLoc.endLine = ctLoc.startLine;
                      ctLoc.endCol = ctLoc.startCol;
                      ctLoc.endOffset = ctLoc.startOffset;
                  } else {
                      ctLoc.endLine = mxn.posTracker.line;
                      ctLoc.endCol = mxn.posTracker.col + 1;
                      ctLoc.endOffset = mxn.posTracker.offset + 1;
                  }

                  orig._emitCurrentToken.call(this);
              },

              _emitCurrentCharacterToken() {
                  const ctLoc = this.currentCharacterToken && this.currentCharacterToken.location;

                  //NOTE: if we have character token and it's location wasn't set in the _emitCurrentToken(),
                  //then set it's location at the current preprocessor position.
                  //We don't need to increment preprocessor position, since character token
                  //emission is always forced by the start of the next character token here.
                  //So, we already have advanced position.
                  if (ctLoc && ctLoc.endOffset === -1) {
                      ctLoc.endLine = mxn.posTracker.line;
                      ctLoc.endCol = mxn.posTracker.col;
                      ctLoc.endOffset = mxn.posTracker.offset;
                  }

                  orig._emitCurrentCharacterToken.call(this);
              }
          };

          //NOTE: patch initial states for each mode to obtain token start position
          Object.keys(Tokenizer$3.MODE).forEach(modeName => {
              const state = Tokenizer$3.MODE[modeName];

              methods[state] = function(cp) {
                  mxn.ctLoc = mxn._getCurrentLocation();
                  orig[state].call(this, cp);
              };
          });

          return methods;
      }
  }

  var tokenizerMixin$1 = LocationInfoTokenizerMixin$2;

  const Mixin$6 = mixin;

  class LocationInfoOpenElementStackMixin$1 extends Mixin$6 {
      constructor(stack, opts) {
          super(stack);

          this.onItemPop = opts.onItemPop;
      }

      _getOverriddenMethods(mxn, orig) {
          return {
              pop() {
                  mxn.onItemPop(this.current);
                  orig.pop.call(this);
              },

              popAllUpToHtmlElement() {
                  for (let i = this.stackTop; i > 0; i--) {
                      mxn.onItemPop(this.items[i]);
                  }

                  orig.popAllUpToHtmlElement.call(this);
              },

              remove(element) {
                  mxn.onItemPop(this.current);
                  orig.remove.call(this, element);
              }
          };
      }
  }

  var openElementStackMixin = LocationInfoOpenElementStackMixin$1;

  const Mixin$5 = mixin;
  const Tokenizer$2 = tokenizer;
  const LocationInfoTokenizerMixin$1 = tokenizerMixin$1;
  const LocationInfoOpenElementStackMixin = openElementStackMixin;
  const HTML$2 = html$e;

  //Aliases
  const $$2 = HTML$2.TAG_NAMES;

  class LocationInfoParserMixin$1 extends Mixin$5 {
      constructor(parser) {
          super(parser);

          this.parser = parser;
          this.treeAdapter = this.parser.treeAdapter;
          this.posTracker = null;
          this.lastStartTagToken = null;
          this.lastFosterParentingLocation = null;
          this.currentToken = null;
      }

      _setStartLocation(element) {
          let loc = null;

          if (this.lastStartTagToken) {
              loc = Object.assign({}, this.lastStartTagToken.location);
              loc.startTag = this.lastStartTagToken.location;
          }

          this.treeAdapter.setNodeSourceCodeLocation(element, loc);
      }

      _setEndLocation(element, closingToken) {
          const loc = this.treeAdapter.getNodeSourceCodeLocation(element);

          if (loc) {
              if (closingToken.location) {
                  const ctLoc = closingToken.location;
                  const tn = this.treeAdapter.getTagName(element);

                  // NOTE: For cases like <p> <p> </p> - First 'p' closes without a closing
                  // tag and for cases like <td> <p> </td> - 'p' closes without a closing tag.
                  const isClosingEndTag = closingToken.type === Tokenizer$2.END_TAG_TOKEN && tn === closingToken.tagName;
                  const endLoc = {};
                  if (isClosingEndTag) {
                      endLoc.endTag = Object.assign({}, ctLoc);
                      endLoc.endLine = ctLoc.endLine;
                      endLoc.endCol = ctLoc.endCol;
                      endLoc.endOffset = ctLoc.endOffset;
                  } else {
                      endLoc.endLine = ctLoc.startLine;
                      endLoc.endCol = ctLoc.startCol;
                      endLoc.endOffset = ctLoc.startOffset;
                  }

                  this.treeAdapter.updateNodeSourceCodeLocation(element, endLoc);
              }
          }
      }

      _getOverriddenMethods(mxn, orig) {
          return {
              _bootstrap(document, fragmentContext) {
                  orig._bootstrap.call(this, document, fragmentContext);

                  mxn.lastStartTagToken = null;
                  mxn.lastFosterParentingLocation = null;
                  mxn.currentToken = null;

                  const tokenizerMixin = Mixin$5.install(this.tokenizer, LocationInfoTokenizerMixin$1);

                  mxn.posTracker = tokenizerMixin.posTracker;

                  Mixin$5.install(this.openElements, LocationInfoOpenElementStackMixin, {
                      onItemPop: function(element) {
                          mxn._setEndLocation(element, mxn.currentToken);
                      }
                  });
              },

              _runParsingLoop(scriptHandler) {
                  orig._runParsingLoop.call(this, scriptHandler);

                  // NOTE: generate location info for elements
                  // that remains on open element stack
                  for (let i = this.openElements.stackTop; i >= 0; i--) {
                      mxn._setEndLocation(this.openElements.items[i], mxn.currentToken);
                  }
              },

              //Token processing
              _processTokenInForeignContent(token) {
                  mxn.currentToken = token;
                  orig._processTokenInForeignContent.call(this, token);
              },

              _processToken(token) {
                  mxn.currentToken = token;
                  orig._processToken.call(this, token);

                  //NOTE: <body> and <html> are never popped from the stack, so we need to updated
                  //their end location explicitly.
                  const requireExplicitUpdate =
                      token.type === Tokenizer$2.END_TAG_TOKEN &&
                      (token.tagName === $$2.HTML || (token.tagName === $$2.BODY && this.openElements.hasInScope($$2.BODY)));

                  if (requireExplicitUpdate) {
                      for (let i = this.openElements.stackTop; i >= 0; i--) {
                          const element = this.openElements.items[i];

                          if (this.treeAdapter.getTagName(element) === token.tagName) {
                              mxn._setEndLocation(element, token);
                              break;
                          }
                      }
                  }
              },

              //Doctype
              _setDocumentType(token) {
                  orig._setDocumentType.call(this, token);

                  const documentChildren = this.treeAdapter.getChildNodes(this.document);
                  const cnLength = documentChildren.length;

                  for (let i = 0; i < cnLength; i++) {
                      const node = documentChildren[i];

                      if (this.treeAdapter.isDocumentTypeNode(node)) {
                          this.treeAdapter.setNodeSourceCodeLocation(node, token.location);
                          break;
                      }
                  }
              },

              //Elements
              _attachElementToTree(element) {
                  //NOTE: _attachElementToTree is called from _appendElement, _insertElement and _insertTemplate methods.
                  //So we will use token location stored in this methods for the element.
                  mxn._setStartLocation(element);
                  mxn.lastStartTagToken = null;
                  orig._attachElementToTree.call(this, element);
              },

              _appendElement(token, namespaceURI) {
                  mxn.lastStartTagToken = token;
                  orig._appendElement.call(this, token, namespaceURI);
              },

              _insertElement(token, namespaceURI) {
                  mxn.lastStartTagToken = token;
                  orig._insertElement.call(this, token, namespaceURI);
              },

              _insertTemplate(token) {
                  mxn.lastStartTagToken = token;
                  orig._insertTemplate.call(this, token);

                  const tmplContent = this.treeAdapter.getTemplateContent(this.openElements.current);

                  this.treeAdapter.setNodeSourceCodeLocation(tmplContent, null);
              },

              _insertFakeRootElement() {
                  orig._insertFakeRootElement.call(this);
                  this.treeAdapter.setNodeSourceCodeLocation(this.openElements.current, null);
              },

              //Comments
              _appendCommentNode(token, parent) {
                  orig._appendCommentNode.call(this, token, parent);

                  const children = this.treeAdapter.getChildNodes(parent);
                  const commentNode = children[children.length - 1];

                  this.treeAdapter.setNodeSourceCodeLocation(commentNode, token.location);
              },

              //Text
              _findFosterParentingLocation() {
                  //NOTE: store last foster parenting location, so we will be able to find inserted text
                  //in case of foster parenting
                  mxn.lastFosterParentingLocation = orig._findFosterParentingLocation.call(this);

                  return mxn.lastFosterParentingLocation;
              },

              _insertCharacters(token) {
                  orig._insertCharacters.call(this, token);

                  const hasFosterParent = this._shouldFosterParentOnInsertion();

                  const parent =
                      (hasFosterParent && mxn.lastFosterParentingLocation.parent) ||
                      this.openElements.currentTmplContent ||
                      this.openElements.current;

                  const siblings = this.treeAdapter.getChildNodes(parent);

                  const textNodeIdx =
                      hasFosterParent && mxn.lastFosterParentingLocation.beforeElement
                          ? siblings.indexOf(mxn.lastFosterParentingLocation.beforeElement) - 1
                          : siblings.length - 1;

                  const textNode = siblings[textNodeIdx];

                  //NOTE: if we have location assigned by another token, then just update end position
                  const tnLoc = this.treeAdapter.getNodeSourceCodeLocation(textNode);

                  if (tnLoc) {
                      const { endLine, endCol, endOffset } = token.location;
                      this.treeAdapter.updateNodeSourceCodeLocation(textNode, { endLine, endCol, endOffset });
                  } else {
                      this.treeAdapter.setNodeSourceCodeLocation(textNode, token.location);
                  }
              }
          };
      }
  }

  var parserMixin$1 = LocationInfoParserMixin$1;

  const Mixin$4 = mixin;

  class ErrorReportingMixinBase$3 extends Mixin$4 {
      constructor(host, opts) {
          super(host);

          this.posTracker = null;
          this.onParseError = opts.onParseError;
      }

      _setErrorLocation(err) {
          err.startLine = err.endLine = this.posTracker.line;
          err.startCol = err.endCol = this.posTracker.col;
          err.startOffset = err.endOffset = this.posTracker.offset;
      }

      _reportError(code) {
          const err = {
              code: code,
              startLine: -1,
              startCol: -1,
              startOffset: -1,
              endLine: -1,
              endCol: -1,
              endOffset: -1
          };

          this._setErrorLocation(err);
          this.onParseError(err);
      }

      _getOverriddenMethods(mxn) {
          return {
              _err(code) {
                  mxn._reportError(code);
              }
          };
      }
  }

  var mixinBase = ErrorReportingMixinBase$3;

  const ErrorReportingMixinBase$2 = mixinBase;
  const PositionTrackingPreprocessorMixin = preprocessorMixin$1;
  const Mixin$3 = mixin;

  class ErrorReportingPreprocessorMixin$1 extends ErrorReportingMixinBase$2 {
      constructor(preprocessor, opts) {
          super(preprocessor, opts);

          this.posTracker = Mixin$3.install(preprocessor, PositionTrackingPreprocessorMixin);
          this.lastErrOffset = -1;
      }

      _reportError(code) {
          //NOTE: avoid reporting error twice on advance/retreat
          if (this.lastErrOffset !== this.posTracker.offset) {
              this.lastErrOffset = this.posTracker.offset;
              super._reportError(code);
          }
      }
  }

  var preprocessorMixin = ErrorReportingPreprocessorMixin$1;

  const ErrorReportingMixinBase$1 = mixinBase;
  const ErrorReportingPreprocessorMixin = preprocessorMixin;
  const Mixin$2 = mixin;

  class ErrorReportingTokenizerMixin$1 extends ErrorReportingMixinBase$1 {
      constructor(tokenizer, opts) {
          super(tokenizer, opts);

          const preprocessorMixin = Mixin$2.install(tokenizer.preprocessor, ErrorReportingPreprocessorMixin, opts);

          this.posTracker = preprocessorMixin.posTracker;
      }
  }

  var tokenizerMixin = ErrorReportingTokenizerMixin$1;

  const ErrorReportingMixinBase = mixinBase;
  const ErrorReportingTokenizerMixin = tokenizerMixin;
  const LocationInfoTokenizerMixin = tokenizerMixin$1;
  const Mixin$1 = mixin;

  class ErrorReportingParserMixin$1 extends ErrorReportingMixinBase {
      constructor(parser, opts) {
          super(parser, opts);

          this.opts = opts;
          this.ctLoc = null;
          this.locBeforeToken = false;
      }

      _setErrorLocation(err) {
          if (this.ctLoc) {
              err.startLine = this.ctLoc.startLine;
              err.startCol = this.ctLoc.startCol;
              err.startOffset = this.ctLoc.startOffset;

              err.endLine = this.locBeforeToken ? this.ctLoc.startLine : this.ctLoc.endLine;
              err.endCol = this.locBeforeToken ? this.ctLoc.startCol : this.ctLoc.endCol;
              err.endOffset = this.locBeforeToken ? this.ctLoc.startOffset : this.ctLoc.endOffset;
          }
      }

      _getOverriddenMethods(mxn, orig) {
          return {
              _bootstrap(document, fragmentContext) {
                  orig._bootstrap.call(this, document, fragmentContext);

                  Mixin$1.install(this.tokenizer, ErrorReportingTokenizerMixin, mxn.opts);
                  Mixin$1.install(this.tokenizer, LocationInfoTokenizerMixin);
              },

              _processInputToken(token) {
                  mxn.ctLoc = token.location;

                  orig._processInputToken.call(this, token);
              },

              _err(code, options) {
                  mxn.locBeforeToken = options && options.beforeToken;
                  mxn._reportError(code);
              }
          };
      }
  }

  var parserMixin = ErrorReportingParserMixin$1;

  var _default = {};

  const { DOCUMENT_MODE: DOCUMENT_MODE$1 } = html$e;

  //Node construction
  _default.createDocument = function() {
      return {
          nodeName: '#document',
          mode: DOCUMENT_MODE$1.NO_QUIRKS,
          childNodes: []
      };
  };

  _default.createDocumentFragment = function() {
      return {
          nodeName: '#document-fragment',
          childNodes: []
      };
  };

  _default.createElement = function(tagName, namespaceURI, attrs) {
      return {
          nodeName: tagName,
          tagName: tagName,
          attrs: attrs,
          namespaceURI: namespaceURI,
          childNodes: [],
          parentNode: null
      };
  };

  _default.createCommentNode = function(data) {
      return {
          nodeName: '#comment',
          data: data,
          parentNode: null
      };
  };

  const createTextNode = function(value) {
      return {
          nodeName: '#text',
          value: value,
          parentNode: null
      };
  };

  //Tree mutation
  const appendChild = (_default.appendChild = function(parentNode, newNode) {
      parentNode.childNodes.push(newNode);
      newNode.parentNode = parentNode;
  });

  const insertBefore = (_default.insertBefore = function(parentNode, newNode, referenceNode) {
      const insertionIdx = parentNode.childNodes.indexOf(referenceNode);

      parentNode.childNodes.splice(insertionIdx, 0, newNode);
      newNode.parentNode = parentNode;
  });

  _default.setTemplateContent = function(templateElement, contentElement) {
      templateElement.content = contentElement;
  };

  _default.getTemplateContent = function(templateElement) {
      return templateElement.content;
  };

  _default.setDocumentType = function(document, name, publicId, systemId) {
      let doctypeNode = null;

      for (let i = 0; i < document.childNodes.length; i++) {
          if (document.childNodes[i].nodeName === '#documentType') {
              doctypeNode = document.childNodes[i];
              break;
          }
      }

      if (doctypeNode) {
          doctypeNode.name = name;
          doctypeNode.publicId = publicId;
          doctypeNode.systemId = systemId;
      } else {
          appendChild(document, {
              nodeName: '#documentType',
              name: name,
              publicId: publicId,
              systemId: systemId
          });
      }
  };

  _default.setDocumentMode = function(document, mode) {
      document.mode = mode;
  };

  _default.getDocumentMode = function(document) {
      return document.mode;
  };

  _default.detachNode = function(node) {
      if (node.parentNode) {
          const idx = node.parentNode.childNodes.indexOf(node);

          node.parentNode.childNodes.splice(idx, 1);
          node.parentNode = null;
      }
  };

  _default.insertText = function(parentNode, text) {
      if (parentNode.childNodes.length) {
          const prevNode = parentNode.childNodes[parentNode.childNodes.length - 1];

          if (prevNode.nodeName === '#text') {
              prevNode.value += text;
              return;
          }
      }

      appendChild(parentNode, createTextNode(text));
  };

  _default.insertTextBefore = function(parentNode, text, referenceNode) {
      const prevNode = parentNode.childNodes[parentNode.childNodes.indexOf(referenceNode) - 1];

      if (prevNode && prevNode.nodeName === '#text') {
          prevNode.value += text;
      } else {
          insertBefore(parentNode, createTextNode(text), referenceNode);
      }
  };

  _default.adoptAttributes = function(recipient, attrs) {
      const recipientAttrsMap = [];

      for (let i = 0; i < recipient.attrs.length; i++) {
          recipientAttrsMap.push(recipient.attrs[i].name);
      }

      for (let j = 0; j < attrs.length; j++) {
          if (recipientAttrsMap.indexOf(attrs[j].name) === -1) {
              recipient.attrs.push(attrs[j]);
          }
      }
  };

  //Tree traversing
  _default.getFirstChild = function(node) {
      return node.childNodes[0];
  };

  _default.getChildNodes = function(node) {
      return node.childNodes;
  };

  _default.getParentNode = function(node) {
      return node.parentNode;
  };

  _default.getAttrList = function(element) {
      return element.attrs;
  };

  //Node data
  _default.getTagName = function(element) {
      return element.tagName;
  };

  _default.getNamespaceURI = function(element) {
      return element.namespaceURI;
  };

  _default.getTextNodeContent = function(textNode) {
      return textNode.value;
  };

  _default.getCommentNodeContent = function(commentNode) {
      return commentNode.data;
  };

  _default.getDocumentTypeNodeName = function(doctypeNode) {
      return doctypeNode.name;
  };

  _default.getDocumentTypeNodePublicId = function(doctypeNode) {
      return doctypeNode.publicId;
  };

  _default.getDocumentTypeNodeSystemId = function(doctypeNode) {
      return doctypeNode.systemId;
  };

  //Node types
  _default.isTextNode = function(node) {
      return node.nodeName === '#text';
  };

  _default.isCommentNode = function(node) {
      return node.nodeName === '#comment';
  };

  _default.isDocumentTypeNode = function(node) {
      return node.nodeName === '#documentType';
  };

  _default.isElementNode = function(node) {
      return !!node.tagName;
  };

  // Source code location
  _default.setNodeSourceCodeLocation = function(node, location) {
      node.sourceCodeLocation = location;
  };

  _default.getNodeSourceCodeLocation = function(node) {
      return node.sourceCodeLocation;
  };

  _default.updateNodeSourceCodeLocation = function(node, endLocation) {
      node.sourceCodeLocation = Object.assign(node.sourceCodeLocation, endLocation);
  };

  var mergeOptions$1 = function mergeOptions(defaults, options) {
      options = options || Object.create(null);

      return [defaults, options].reduce((merged, optObj) => {
          Object.keys(optObj).forEach(key => {
              merged[key] = optObj[key];
          });

          return merged;
      }, Object.create(null));
  };

  var doctype$4 = {};

  const { DOCUMENT_MODE } = html$e;

  //Const
  const VALID_DOCTYPE_NAME = 'html';
  const VALID_SYSTEM_ID = 'about:legacy-compat';
  const QUIRKS_MODE_SYSTEM_ID = 'http://www.ibm.com/data/dtd/v11/ibmxhtml1-transitional.dtd';

  const QUIRKS_MODE_PUBLIC_ID_PREFIXES = [
      '+//silmaril//dtd html pro v0r11 19970101//',
      '-//as//dtd html 3.0 aswedit + extensions//',
      '-//advasoft ltd//dtd html 3.0 aswedit + extensions//',
      '-//ietf//dtd html 2.0 level 1//',
      '-//ietf//dtd html 2.0 level 2//',
      '-//ietf//dtd html 2.0 strict level 1//',
      '-//ietf//dtd html 2.0 strict level 2//',
      '-//ietf//dtd html 2.0 strict//',
      '-//ietf//dtd html 2.0//',
      '-//ietf//dtd html 2.1e//',
      '-//ietf//dtd html 3.0//',
      '-//ietf//dtd html 3.2 final//',
      '-//ietf//dtd html 3.2//',
      '-//ietf//dtd html 3//',
      '-//ietf//dtd html level 0//',
      '-//ietf//dtd html level 1//',
      '-//ietf//dtd html level 2//',
      '-//ietf//dtd html level 3//',
      '-//ietf//dtd html strict level 0//',
      '-//ietf//dtd html strict level 1//',
      '-//ietf//dtd html strict level 2//',
      '-//ietf//dtd html strict level 3//',
      '-//ietf//dtd html strict//',
      '-//ietf//dtd html//',
      '-//metrius//dtd metrius presentational//',
      '-//microsoft//dtd internet explorer 2.0 html strict//',
      '-//microsoft//dtd internet explorer 2.0 html//',
      '-//microsoft//dtd internet explorer 2.0 tables//',
      '-//microsoft//dtd internet explorer 3.0 html strict//',
      '-//microsoft//dtd internet explorer 3.0 html//',
      '-//microsoft//dtd internet explorer 3.0 tables//',
      '-//netscape comm. corp.//dtd html//',
      '-//netscape comm. corp.//dtd strict html//',
      "-//o'reilly and associates//dtd html 2.0//",
      "-//o'reilly and associates//dtd html extended 1.0//",
      "-//o'reilly and associates//dtd html extended relaxed 1.0//",
      '-//sq//dtd html 2.0 hotmetal + extensions//',
      '-//softquad software//dtd hotmetal pro 6.0::19990601::extensions to html 4.0//',
      '-//softquad//dtd hotmetal pro 4.0::19971010::extensions to html 4.0//',
      '-//spyglass//dtd html 2.0 extended//',
      '-//sun microsystems corp.//dtd hotjava html//',
      '-//sun microsystems corp.//dtd hotjava strict html//',
      '-//w3c//dtd html 3 1995-03-24//',
      '-//w3c//dtd html 3.2 draft//',
      '-//w3c//dtd html 3.2 final//',
      '-//w3c//dtd html 3.2//',
      '-//w3c//dtd html 3.2s draft//',
      '-//w3c//dtd html 4.0 frameset//',
      '-//w3c//dtd html 4.0 transitional//',
      '-//w3c//dtd html experimental 19960712//',
      '-//w3c//dtd html experimental 970421//',
      '-//w3c//dtd w3 html//',
      '-//w3o//dtd w3 html 3.0//',
      '-//webtechs//dtd mozilla html 2.0//',
      '-//webtechs//dtd mozilla html//'
  ];

  const QUIRKS_MODE_NO_SYSTEM_ID_PUBLIC_ID_PREFIXES = QUIRKS_MODE_PUBLIC_ID_PREFIXES.concat([
      '-//w3c//dtd html 4.01 frameset//',
      '-//w3c//dtd html 4.01 transitional//'
  ]);

  const QUIRKS_MODE_PUBLIC_IDS = ['-//w3o//dtd w3 html strict 3.0//en//', '-/w3c/dtd html 4.0 transitional/en', 'html'];
  const LIMITED_QUIRKS_PUBLIC_ID_PREFIXES = ['-//w3c//dtd xhtml 1.0 frameset//', '-//w3c//dtd xhtml 1.0 transitional//'];

  const LIMITED_QUIRKS_WITH_SYSTEM_ID_PUBLIC_ID_PREFIXES = LIMITED_QUIRKS_PUBLIC_ID_PREFIXES.concat([
      '-//w3c//dtd html 4.01 frameset//',
      '-//w3c//dtd html 4.01 transitional//'
  ]);

  //Utils
  function enquoteDoctypeId(id) {
      const quote = id.indexOf('"') !== -1 ? "'" : '"';

      return quote + id + quote;
  }

  function hasPrefix(publicId, prefixes) {
      for (let i = 0; i < prefixes.length; i++) {
          if (publicId.indexOf(prefixes[i]) === 0) {
              return true;
          }
      }

      return false;
  }

  //API
  doctype$4.isConforming = function(token) {
      return (
          token.name === VALID_DOCTYPE_NAME &&
          token.publicId === null &&
          (token.systemId === null || token.systemId === VALID_SYSTEM_ID)
      );
  };

  doctype$4.getDocumentMode = function(token) {
      if (token.name !== VALID_DOCTYPE_NAME) {
          return DOCUMENT_MODE.QUIRKS;
      }

      const systemId = token.systemId;

      if (systemId && systemId.toLowerCase() === QUIRKS_MODE_SYSTEM_ID) {
          return DOCUMENT_MODE.QUIRKS;
      }

      let publicId = token.publicId;

      if (publicId !== null) {
          publicId = publicId.toLowerCase();

          if (QUIRKS_MODE_PUBLIC_IDS.indexOf(publicId) > -1) {
              return DOCUMENT_MODE.QUIRKS;
          }

          let prefixes = systemId === null ? QUIRKS_MODE_NO_SYSTEM_ID_PUBLIC_ID_PREFIXES : QUIRKS_MODE_PUBLIC_ID_PREFIXES;

          if (hasPrefix(publicId, prefixes)) {
              return DOCUMENT_MODE.QUIRKS;
          }

          prefixes =
              systemId === null ? LIMITED_QUIRKS_PUBLIC_ID_PREFIXES : LIMITED_QUIRKS_WITH_SYSTEM_ID_PUBLIC_ID_PREFIXES;

          if (hasPrefix(publicId, prefixes)) {
              return DOCUMENT_MODE.LIMITED_QUIRKS;
          }
      }

      return DOCUMENT_MODE.NO_QUIRKS;
  };

  doctype$4.serializeContent = function(name, publicId, systemId) {
      let str = '!DOCTYPE ';

      if (name) {
          str += name;
      }

      if (publicId) {
          str += ' PUBLIC ' + enquoteDoctypeId(publicId);
      } else if (systemId) {
          str += ' SYSTEM';
      }

      if (systemId !== null) {
          str += ' ' + enquoteDoctypeId(systemId);
      }

      return str;
  };

  var foreignContent$1 = {};

  const Tokenizer$1 = tokenizer;
  const HTML$1 = html$e;

  //Aliases
  const $$1 = HTML$1.TAG_NAMES;
  const NS$1 = HTML$1.NAMESPACES;
  const ATTRS$1 = HTML$1.ATTRS;

  //MIME types
  const MIME_TYPES = {
      TEXT_HTML: 'text/html',
      APPLICATION_XML: 'application/xhtml+xml'
  };

  //Attributes
  const DEFINITION_URL_ATTR = 'definitionurl';
  const ADJUSTED_DEFINITION_URL_ATTR = 'definitionURL';
  const SVG_ATTRS_ADJUSTMENT_MAP = {
      attributename: 'attributeName',
      attributetype: 'attributeType',
      basefrequency: 'baseFrequency',
      baseprofile: 'baseProfile',
      calcmode: 'calcMode',
      clippathunits: 'clipPathUnits',
      diffuseconstant: 'diffuseConstant',
      edgemode: 'edgeMode',
      filterunits: 'filterUnits',
      glyphref: 'glyphRef',
      gradienttransform: 'gradientTransform',
      gradientunits: 'gradientUnits',
      kernelmatrix: 'kernelMatrix',
      kernelunitlength: 'kernelUnitLength',
      keypoints: 'keyPoints',
      keysplines: 'keySplines',
      keytimes: 'keyTimes',
      lengthadjust: 'lengthAdjust',
      limitingconeangle: 'limitingConeAngle',
      markerheight: 'markerHeight',
      markerunits: 'markerUnits',
      markerwidth: 'markerWidth',
      maskcontentunits: 'maskContentUnits',
      maskunits: 'maskUnits',
      numoctaves: 'numOctaves',
      pathlength: 'pathLength',
      patterncontentunits: 'patternContentUnits',
      patterntransform: 'patternTransform',
      patternunits: 'patternUnits',
      pointsatx: 'pointsAtX',
      pointsaty: 'pointsAtY',
      pointsatz: 'pointsAtZ',
      preservealpha: 'preserveAlpha',
      preserveaspectratio: 'preserveAspectRatio',
      primitiveunits: 'primitiveUnits',
      refx: 'refX',
      refy: 'refY',
      repeatcount: 'repeatCount',
      repeatdur: 'repeatDur',
      requiredextensions: 'requiredExtensions',
      requiredfeatures: 'requiredFeatures',
      specularconstant: 'specularConstant',
      specularexponent: 'specularExponent',
      spreadmethod: 'spreadMethod',
      startoffset: 'startOffset',
      stddeviation: 'stdDeviation',
      stitchtiles: 'stitchTiles',
      surfacescale: 'surfaceScale',
      systemlanguage: 'systemLanguage',
      tablevalues: 'tableValues',
      targetx: 'targetX',
      targety: 'targetY',
      textlength: 'textLength',
      viewbox: 'viewBox',
      viewtarget: 'viewTarget',
      xchannelselector: 'xChannelSelector',
      ychannelselector: 'yChannelSelector',
      zoomandpan: 'zoomAndPan'
  };

  const XML_ATTRS_ADJUSTMENT_MAP = {
      'xlink:actuate': { prefix: 'xlink', name: 'actuate', namespace: NS$1.XLINK },
      'xlink:arcrole': { prefix: 'xlink', name: 'arcrole', namespace: NS$1.XLINK },
      'xlink:href': { prefix: 'xlink', name: 'href', namespace: NS$1.XLINK },
      'xlink:role': { prefix: 'xlink', name: 'role', namespace: NS$1.XLINK },
      'xlink:show': { prefix: 'xlink', name: 'show', namespace: NS$1.XLINK },
      'xlink:title': { prefix: 'xlink', name: 'title', namespace: NS$1.XLINK },
      'xlink:type': { prefix: 'xlink', name: 'type', namespace: NS$1.XLINK },
      'xml:base': { prefix: 'xml', name: 'base', namespace: NS$1.XML },
      'xml:lang': { prefix: 'xml', name: 'lang', namespace: NS$1.XML },
      'xml:space': { prefix: 'xml', name: 'space', namespace: NS$1.XML },
      xmlns: { prefix: '', name: 'xmlns', namespace: NS$1.XMLNS },
      'xmlns:xlink': { prefix: 'xmlns', name: 'xlink', namespace: NS$1.XMLNS }
  };

  //SVG tag names adjustment map
  const SVG_TAG_NAMES_ADJUSTMENT_MAP = (foreignContent$1.SVG_TAG_NAMES_ADJUSTMENT_MAP = {
      altglyph: 'altGlyph',
      altglyphdef: 'altGlyphDef',
      altglyphitem: 'altGlyphItem',
      animatecolor: 'animateColor',
      animatemotion: 'animateMotion',
      animatetransform: 'animateTransform',
      clippath: 'clipPath',
      feblend: 'feBlend',
      fecolormatrix: 'feColorMatrix',
      fecomponenttransfer: 'feComponentTransfer',
      fecomposite: 'feComposite',
      feconvolvematrix: 'feConvolveMatrix',
      fediffuselighting: 'feDiffuseLighting',
      fedisplacementmap: 'feDisplacementMap',
      fedistantlight: 'feDistantLight',
      feflood: 'feFlood',
      fefunca: 'feFuncA',
      fefuncb: 'feFuncB',
      fefuncg: 'feFuncG',
      fefuncr: 'feFuncR',
      fegaussianblur: 'feGaussianBlur',
      feimage: 'feImage',
      femerge: 'feMerge',
      femergenode: 'feMergeNode',
      femorphology: 'feMorphology',
      feoffset: 'feOffset',
      fepointlight: 'fePointLight',
      fespecularlighting: 'feSpecularLighting',
      fespotlight: 'feSpotLight',
      fetile: 'feTile',
      feturbulence: 'feTurbulence',
      foreignobject: 'foreignObject',
      glyphref: 'glyphRef',
      lineargradient: 'linearGradient',
      radialgradient: 'radialGradient',
      textpath: 'textPath'
  });

  //Tags that causes exit from foreign content
  const EXITS_FOREIGN_CONTENT = {
      [$$1.B]: true,
      [$$1.BIG]: true,
      [$$1.BLOCKQUOTE]: true,
      [$$1.BODY]: true,
      [$$1.BR]: true,
      [$$1.CENTER]: true,
      [$$1.CODE]: true,
      [$$1.DD]: true,
      [$$1.DIV]: true,
      [$$1.DL]: true,
      [$$1.DT]: true,
      [$$1.EM]: true,
      [$$1.EMBED]: true,
      [$$1.H1]: true,
      [$$1.H2]: true,
      [$$1.H3]: true,
      [$$1.H4]: true,
      [$$1.H5]: true,
      [$$1.H6]: true,
      [$$1.HEAD]: true,
      [$$1.HR]: true,
      [$$1.I]: true,
      [$$1.IMG]: true,
      [$$1.LI]: true,
      [$$1.LISTING]: true,
      [$$1.MENU]: true,
      [$$1.META]: true,
      [$$1.NOBR]: true,
      [$$1.OL]: true,
      [$$1.P]: true,
      [$$1.PRE]: true,
      [$$1.RUBY]: true,
      [$$1.S]: true,
      [$$1.SMALL]: true,
      [$$1.SPAN]: true,
      [$$1.STRONG]: true,
      [$$1.STRIKE]: true,
      [$$1.SUB]: true,
      [$$1.SUP]: true,
      [$$1.TABLE]: true,
      [$$1.TT]: true,
      [$$1.U]: true,
      [$$1.UL]: true,
      [$$1.VAR]: true
  };

  //Check exit from foreign content
  foreignContent$1.causesExit = function(startTagToken) {
      const tn = startTagToken.tagName;
      const isFontWithAttrs =
          tn === $$1.FONT &&
          (Tokenizer$1.getTokenAttr(startTagToken, ATTRS$1.COLOR) !== null ||
              Tokenizer$1.getTokenAttr(startTagToken, ATTRS$1.SIZE) !== null ||
              Tokenizer$1.getTokenAttr(startTagToken, ATTRS$1.FACE) !== null);

      return isFontWithAttrs ? true : EXITS_FOREIGN_CONTENT[tn];
  };

  //Token adjustments
  foreignContent$1.adjustTokenMathMLAttrs = function(token) {
      for (let i = 0; i < token.attrs.length; i++) {
          if (token.attrs[i].name === DEFINITION_URL_ATTR) {
              token.attrs[i].name = ADJUSTED_DEFINITION_URL_ATTR;
              break;
          }
      }
  };

  foreignContent$1.adjustTokenSVGAttrs = function(token) {
      for (let i = 0; i < token.attrs.length; i++) {
          const adjustedAttrName = SVG_ATTRS_ADJUSTMENT_MAP[token.attrs[i].name];

          if (adjustedAttrName) {
              token.attrs[i].name = adjustedAttrName;
          }
      }
  };

  foreignContent$1.adjustTokenXMLAttrs = function(token) {
      for (let i = 0; i < token.attrs.length; i++) {
          const adjustedAttrEntry = XML_ATTRS_ADJUSTMENT_MAP[token.attrs[i].name];

          if (adjustedAttrEntry) {
              token.attrs[i].prefix = adjustedAttrEntry.prefix;
              token.attrs[i].name = adjustedAttrEntry.name;
              token.attrs[i].namespace = adjustedAttrEntry.namespace;
          }
      }
  };

  foreignContent$1.adjustTokenSVGTagName = function(token) {
      const adjustedTagName = SVG_TAG_NAMES_ADJUSTMENT_MAP[token.tagName];

      if (adjustedTagName) {
          token.tagName = adjustedTagName;
      }
  };

  //Integration points
  function isMathMLTextIntegrationPoint(tn, ns) {
      return ns === NS$1.MATHML && (tn === $$1.MI || tn === $$1.MO || tn === $$1.MN || tn === $$1.MS || tn === $$1.MTEXT);
  }

  function isHtmlIntegrationPoint(tn, ns, attrs) {
      if (ns === NS$1.MATHML && tn === $$1.ANNOTATION_XML) {
          for (let i = 0; i < attrs.length; i++) {
              if (attrs[i].name === ATTRS$1.ENCODING) {
                  const value = attrs[i].value.toLowerCase();

                  return value === MIME_TYPES.TEXT_HTML || value === MIME_TYPES.APPLICATION_XML;
              }
          }
      }

      return ns === NS$1.SVG && (tn === $$1.FOREIGN_OBJECT || tn === $$1.DESC || tn === $$1.TITLE);
  }

  foreignContent$1.isIntegrationPoint = function(tn, ns, attrs, foreignNS) {
      if ((!foreignNS || foreignNS === NS$1.HTML) && isHtmlIntegrationPoint(tn, ns, attrs)) {
          return true;
      }

      if ((!foreignNS || foreignNS === NS$1.MATHML) && isMathMLTextIntegrationPoint(tn, ns)) {
          return true;
      }

      return false;
  };

  const Tokenizer = tokenizer;
  const OpenElementStack = openElementStack;
  const FormattingElementList = formattingElementList;
  const LocationInfoParserMixin = parserMixin$1;
  const ErrorReportingParserMixin = parserMixin;
  const Mixin = mixin;
  const defaultTreeAdapter = _default;
  const mergeOptions = mergeOptions$1;
  const doctype$3 = doctype$4;
  const foreignContent = foreignContent$1;
  const ERR = errorCodes;
  const unicode = unicode$3;
  const HTML = html$e;

  //Aliases
  const $ = HTML.TAG_NAMES;
  const NS = HTML.NAMESPACES;
  const ATTRS = HTML.ATTRS;

  const DEFAULT_OPTIONS = {
      scriptingEnabled: true,
      sourceCodeLocationInfo: false,
      onParseError: null,
      treeAdapter: defaultTreeAdapter
  };

  //Misc constants
  const HIDDEN_INPUT_TYPE = 'hidden';

  //Adoption agency loops iteration count
  const AA_OUTER_LOOP_ITER = 8;
  const AA_INNER_LOOP_ITER = 3;

  //Insertion modes
  const INITIAL_MODE = 'INITIAL_MODE';
  const BEFORE_HTML_MODE = 'BEFORE_HTML_MODE';
  const BEFORE_HEAD_MODE = 'BEFORE_HEAD_MODE';
  const IN_HEAD_MODE = 'IN_HEAD_MODE';
  const IN_HEAD_NO_SCRIPT_MODE = 'IN_HEAD_NO_SCRIPT_MODE';
  const AFTER_HEAD_MODE = 'AFTER_HEAD_MODE';
  const IN_BODY_MODE = 'IN_BODY_MODE';
  const TEXT_MODE = 'TEXT_MODE';
  const IN_TABLE_MODE = 'IN_TABLE_MODE';
  const IN_TABLE_TEXT_MODE = 'IN_TABLE_TEXT_MODE';
  const IN_CAPTION_MODE = 'IN_CAPTION_MODE';
  const IN_COLUMN_GROUP_MODE = 'IN_COLUMN_GROUP_MODE';
  const IN_TABLE_BODY_MODE = 'IN_TABLE_BODY_MODE';
  const IN_ROW_MODE = 'IN_ROW_MODE';
  const IN_CELL_MODE = 'IN_CELL_MODE';
  const IN_SELECT_MODE = 'IN_SELECT_MODE';
  const IN_SELECT_IN_TABLE_MODE = 'IN_SELECT_IN_TABLE_MODE';
  const IN_TEMPLATE_MODE = 'IN_TEMPLATE_MODE';
  const AFTER_BODY_MODE = 'AFTER_BODY_MODE';
  const IN_FRAMESET_MODE = 'IN_FRAMESET_MODE';
  const AFTER_FRAMESET_MODE = 'AFTER_FRAMESET_MODE';
  const AFTER_AFTER_BODY_MODE = 'AFTER_AFTER_BODY_MODE';
  const AFTER_AFTER_FRAMESET_MODE = 'AFTER_AFTER_FRAMESET_MODE';

  //Insertion mode reset map
  const INSERTION_MODE_RESET_MAP = {
      [$.TR]: IN_ROW_MODE,
      [$.TBODY]: IN_TABLE_BODY_MODE,
      [$.THEAD]: IN_TABLE_BODY_MODE,
      [$.TFOOT]: IN_TABLE_BODY_MODE,
      [$.CAPTION]: IN_CAPTION_MODE,
      [$.COLGROUP]: IN_COLUMN_GROUP_MODE,
      [$.TABLE]: IN_TABLE_MODE,
      [$.BODY]: IN_BODY_MODE,
      [$.FRAMESET]: IN_FRAMESET_MODE
  };

  //Template insertion mode switch map
  const TEMPLATE_INSERTION_MODE_SWITCH_MAP = {
      [$.CAPTION]: IN_TABLE_MODE,
      [$.COLGROUP]: IN_TABLE_MODE,
      [$.TBODY]: IN_TABLE_MODE,
      [$.TFOOT]: IN_TABLE_MODE,
      [$.THEAD]: IN_TABLE_MODE,
      [$.COL]: IN_COLUMN_GROUP_MODE,
      [$.TR]: IN_TABLE_BODY_MODE,
      [$.TD]: IN_ROW_MODE,
      [$.TH]: IN_ROW_MODE
  };

  //Token handlers map for insertion modes
  const TOKEN_HANDLERS = {
      [INITIAL_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: tokenInInitialMode,
          [Tokenizer.NULL_CHARACTER_TOKEN]: tokenInInitialMode,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: doctypeInInitialMode,
          [Tokenizer.START_TAG_TOKEN]: tokenInInitialMode,
          [Tokenizer.END_TAG_TOKEN]: tokenInInitialMode,
          [Tokenizer.EOF_TOKEN]: tokenInInitialMode
      },
      [BEFORE_HTML_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: tokenBeforeHtml,
          [Tokenizer.NULL_CHARACTER_TOKEN]: tokenBeforeHtml,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagBeforeHtml,
          [Tokenizer.END_TAG_TOKEN]: endTagBeforeHtml,
          [Tokenizer.EOF_TOKEN]: tokenBeforeHtml
      },
      [BEFORE_HEAD_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: tokenBeforeHead,
          [Tokenizer.NULL_CHARACTER_TOKEN]: tokenBeforeHead,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: misplacedDoctype,
          [Tokenizer.START_TAG_TOKEN]: startTagBeforeHead,
          [Tokenizer.END_TAG_TOKEN]: endTagBeforeHead,
          [Tokenizer.EOF_TOKEN]: tokenBeforeHead
      },
      [IN_HEAD_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: tokenInHead,
          [Tokenizer.NULL_CHARACTER_TOKEN]: tokenInHead,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: insertCharacters,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: misplacedDoctype,
          [Tokenizer.START_TAG_TOKEN]: startTagInHead,
          [Tokenizer.END_TAG_TOKEN]: endTagInHead,
          [Tokenizer.EOF_TOKEN]: tokenInHead
      },
      [IN_HEAD_NO_SCRIPT_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: tokenInHeadNoScript,
          [Tokenizer.NULL_CHARACTER_TOKEN]: tokenInHeadNoScript,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: insertCharacters,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: misplacedDoctype,
          [Tokenizer.START_TAG_TOKEN]: startTagInHeadNoScript,
          [Tokenizer.END_TAG_TOKEN]: endTagInHeadNoScript,
          [Tokenizer.EOF_TOKEN]: tokenInHeadNoScript
      },
      [AFTER_HEAD_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: tokenAfterHead,
          [Tokenizer.NULL_CHARACTER_TOKEN]: tokenAfterHead,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: insertCharacters,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: misplacedDoctype,
          [Tokenizer.START_TAG_TOKEN]: startTagAfterHead,
          [Tokenizer.END_TAG_TOKEN]: endTagAfterHead,
          [Tokenizer.EOF_TOKEN]: tokenAfterHead
      },
      [IN_BODY_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: characterInBody,
          [Tokenizer.NULL_CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInBody,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagInBody,
          [Tokenizer.END_TAG_TOKEN]: endTagInBody,
          [Tokenizer.EOF_TOKEN]: eofInBody
      },
      [TEXT_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: insertCharacters,
          [Tokenizer.NULL_CHARACTER_TOKEN]: insertCharacters,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: insertCharacters,
          [Tokenizer.COMMENT_TOKEN]: ignoreToken,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: ignoreToken,
          [Tokenizer.END_TAG_TOKEN]: endTagInText,
          [Tokenizer.EOF_TOKEN]: eofInText
      },
      [IN_TABLE_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: characterInTable,
          [Tokenizer.NULL_CHARACTER_TOKEN]: characterInTable,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: characterInTable,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagInTable,
          [Tokenizer.END_TAG_TOKEN]: endTagInTable,
          [Tokenizer.EOF_TOKEN]: eofInBody
      },
      [IN_TABLE_TEXT_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: characterInTableText,
          [Tokenizer.NULL_CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInTableText,
          [Tokenizer.COMMENT_TOKEN]: tokenInTableText,
          [Tokenizer.DOCTYPE_TOKEN]: tokenInTableText,
          [Tokenizer.START_TAG_TOKEN]: tokenInTableText,
          [Tokenizer.END_TAG_TOKEN]: tokenInTableText,
          [Tokenizer.EOF_TOKEN]: tokenInTableText
      },
      [IN_CAPTION_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: characterInBody,
          [Tokenizer.NULL_CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInBody,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagInCaption,
          [Tokenizer.END_TAG_TOKEN]: endTagInCaption,
          [Tokenizer.EOF_TOKEN]: eofInBody
      },
      [IN_COLUMN_GROUP_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: tokenInColumnGroup,
          [Tokenizer.NULL_CHARACTER_TOKEN]: tokenInColumnGroup,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: insertCharacters,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagInColumnGroup,
          [Tokenizer.END_TAG_TOKEN]: endTagInColumnGroup,
          [Tokenizer.EOF_TOKEN]: eofInBody
      },
      [IN_TABLE_BODY_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: characterInTable,
          [Tokenizer.NULL_CHARACTER_TOKEN]: characterInTable,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: characterInTable,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagInTableBody,
          [Tokenizer.END_TAG_TOKEN]: endTagInTableBody,
          [Tokenizer.EOF_TOKEN]: eofInBody
      },
      [IN_ROW_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: characterInTable,
          [Tokenizer.NULL_CHARACTER_TOKEN]: characterInTable,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: characterInTable,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagInRow,
          [Tokenizer.END_TAG_TOKEN]: endTagInRow,
          [Tokenizer.EOF_TOKEN]: eofInBody
      },
      [IN_CELL_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: characterInBody,
          [Tokenizer.NULL_CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInBody,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagInCell,
          [Tokenizer.END_TAG_TOKEN]: endTagInCell,
          [Tokenizer.EOF_TOKEN]: eofInBody
      },
      [IN_SELECT_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: insertCharacters,
          [Tokenizer.NULL_CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: insertCharacters,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagInSelect,
          [Tokenizer.END_TAG_TOKEN]: endTagInSelect,
          [Tokenizer.EOF_TOKEN]: eofInBody
      },
      [IN_SELECT_IN_TABLE_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: insertCharacters,
          [Tokenizer.NULL_CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: insertCharacters,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagInSelectInTable,
          [Tokenizer.END_TAG_TOKEN]: endTagInSelectInTable,
          [Tokenizer.EOF_TOKEN]: eofInBody
      },
      [IN_TEMPLATE_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: characterInBody,
          [Tokenizer.NULL_CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInBody,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagInTemplate,
          [Tokenizer.END_TAG_TOKEN]: endTagInTemplate,
          [Tokenizer.EOF_TOKEN]: eofInTemplate
      },
      [AFTER_BODY_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: tokenAfterBody,
          [Tokenizer.NULL_CHARACTER_TOKEN]: tokenAfterBody,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInBody,
          [Tokenizer.COMMENT_TOKEN]: appendCommentToRootHtmlElement,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagAfterBody,
          [Tokenizer.END_TAG_TOKEN]: endTagAfterBody,
          [Tokenizer.EOF_TOKEN]: stopParsing
      },
      [IN_FRAMESET_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.NULL_CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: insertCharacters,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagInFrameset,
          [Tokenizer.END_TAG_TOKEN]: endTagInFrameset,
          [Tokenizer.EOF_TOKEN]: stopParsing
      },
      [AFTER_FRAMESET_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.NULL_CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: insertCharacters,
          [Tokenizer.COMMENT_TOKEN]: appendComment,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagAfterFrameset,
          [Tokenizer.END_TAG_TOKEN]: endTagAfterFrameset,
          [Tokenizer.EOF_TOKEN]: stopParsing
      },
      [AFTER_AFTER_BODY_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: tokenAfterAfterBody,
          [Tokenizer.NULL_CHARACTER_TOKEN]: tokenAfterAfterBody,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInBody,
          [Tokenizer.COMMENT_TOKEN]: appendCommentToDocument,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagAfterAfterBody,
          [Tokenizer.END_TAG_TOKEN]: tokenAfterAfterBody,
          [Tokenizer.EOF_TOKEN]: stopParsing
      },
      [AFTER_AFTER_FRAMESET_MODE]: {
          [Tokenizer.CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.NULL_CHARACTER_TOKEN]: ignoreToken,
          [Tokenizer.WHITESPACE_CHARACTER_TOKEN]: whitespaceCharacterInBody,
          [Tokenizer.COMMENT_TOKEN]: appendCommentToDocument,
          [Tokenizer.DOCTYPE_TOKEN]: ignoreToken,
          [Tokenizer.START_TAG_TOKEN]: startTagAfterAfterFrameset,
          [Tokenizer.END_TAG_TOKEN]: ignoreToken,
          [Tokenizer.EOF_TOKEN]: stopParsing
      }
  };

  //Parser
  class Parser {
      constructor(options) {
          this.options = mergeOptions(DEFAULT_OPTIONS, options);

          this.treeAdapter = this.options.treeAdapter;
          this.pendingScript = null;

          if (this.options.sourceCodeLocationInfo) {
              Mixin.install(this, LocationInfoParserMixin);
          }

          if (this.options.onParseError) {
              Mixin.install(this, ErrorReportingParserMixin, { onParseError: this.options.onParseError });
          }
      }

      // API
      parse(html) {
          const document = this.treeAdapter.createDocument();

          this._bootstrap(document, null);
          this.tokenizer.write(html, true);
          this._runParsingLoop(null);

          return document;
      }

      parseFragment(html, fragmentContext) {
          //NOTE: use <template> element as a fragment context if context element was not provided,
          //so we will parse in "forgiving" manner
          if (!fragmentContext) {
              fragmentContext = this.treeAdapter.createElement($.TEMPLATE, NS.HTML, []);
          }

          //NOTE: create fake element which will be used as 'document' for fragment parsing.
          //This is important for jsdom there 'document' can't be recreated, therefore
          //fragment parsing causes messing of the main `document`.
          const documentMock = this.treeAdapter.createElement('documentmock', NS.HTML, []);

          this._bootstrap(documentMock, fragmentContext);

          if (this.treeAdapter.getTagName(fragmentContext) === $.TEMPLATE) {
              this._pushTmplInsertionMode(IN_TEMPLATE_MODE);
          }

          this._initTokenizerForFragmentParsing();
          this._insertFakeRootElement();
          this._resetInsertionMode();
          this._findFormInFragmentContext();
          this.tokenizer.write(html, true);
          this._runParsingLoop(null);

          const rootElement = this.treeAdapter.getFirstChild(documentMock);
          const fragment = this.treeAdapter.createDocumentFragment();

          this._adoptNodes(rootElement, fragment);

          return fragment;
      }

      //Bootstrap parser
      _bootstrap(document, fragmentContext) {
          this.tokenizer = new Tokenizer(this.options);

          this.stopped = false;

          this.insertionMode = INITIAL_MODE;
          this.originalInsertionMode = '';

          this.document = document;
          this.fragmentContext = fragmentContext;

          this.headElement = null;
          this.formElement = null;

          this.openElements = new OpenElementStack(this.document, this.treeAdapter);
          this.activeFormattingElements = new FormattingElementList(this.treeAdapter);

          this.tmplInsertionModeStack = [];
          this.tmplInsertionModeStackTop = -1;
          this.currentTmplInsertionMode = null;

          this.pendingCharacterTokens = [];
          this.hasNonWhitespacePendingCharacterToken = false;

          this.framesetOk = true;
          this.skipNextNewLine = false;
          this.fosterParentingEnabled = false;
      }

      //Errors
      _err() {
          // NOTE: err reporting is noop by default. Enabled by mixin.
      }

      //Parsing loop
      _runParsingLoop(scriptHandler) {
          while (!this.stopped) {
              this._setupTokenizerCDATAMode();

              const token = this.tokenizer.getNextToken();

              if (token.type === Tokenizer.HIBERNATION_TOKEN) {
                  break;
              }

              if (this.skipNextNewLine) {
                  this.skipNextNewLine = false;

                  if (token.type === Tokenizer.WHITESPACE_CHARACTER_TOKEN && token.chars[0] === '\n') {
                      if (token.chars.length === 1) {
                          continue;
                      }

                      token.chars = token.chars.substr(1);
                  }
              }

              this._processInputToken(token);

              if (scriptHandler && this.pendingScript) {
                  break;
              }
          }
      }

      runParsingLoopForCurrentChunk(writeCallback, scriptHandler) {
          this._runParsingLoop(scriptHandler);

          if (scriptHandler && this.pendingScript) {
              const script = this.pendingScript;

              this.pendingScript = null;

              scriptHandler(script);

              return;
          }

          if (writeCallback) {
              writeCallback();
          }
      }

      //Text parsing
      _setupTokenizerCDATAMode() {
          const current = this._getAdjustedCurrentElement();

          this.tokenizer.allowCDATA =
              current &&
              current !== this.document &&
              this.treeAdapter.getNamespaceURI(current) !== NS.HTML &&
              !this._isIntegrationPoint(current);
      }

      _switchToTextParsing(currentToken, nextTokenizerState) {
          this._insertElement(currentToken, NS.HTML);
          this.tokenizer.state = nextTokenizerState;
          this.originalInsertionMode = this.insertionMode;
          this.insertionMode = TEXT_MODE;
      }

      switchToPlaintextParsing() {
          this.insertionMode = TEXT_MODE;
          this.originalInsertionMode = IN_BODY_MODE;
          this.tokenizer.state = Tokenizer.MODE.PLAINTEXT;
      }

      //Fragment parsing
      _getAdjustedCurrentElement() {
          return this.openElements.stackTop === 0 && this.fragmentContext
              ? this.fragmentContext
              : this.openElements.current;
      }

      _findFormInFragmentContext() {
          let node = this.fragmentContext;

          do {
              if (this.treeAdapter.getTagName(node) === $.FORM) {
                  this.formElement = node;
                  break;
              }

              node = this.treeAdapter.getParentNode(node);
          } while (node);
      }

      _initTokenizerForFragmentParsing() {
          if (this.treeAdapter.getNamespaceURI(this.fragmentContext) === NS.HTML) {
              const tn = this.treeAdapter.getTagName(this.fragmentContext);

              if (tn === $.TITLE || tn === $.TEXTAREA) {
                  this.tokenizer.state = Tokenizer.MODE.RCDATA;
              } else if (
                  tn === $.STYLE ||
                  tn === $.XMP ||
                  tn === $.IFRAME ||
                  tn === $.NOEMBED ||
                  tn === $.NOFRAMES ||
                  tn === $.NOSCRIPT
              ) {
                  this.tokenizer.state = Tokenizer.MODE.RAWTEXT;
              } else if (tn === $.SCRIPT) {
                  this.tokenizer.state = Tokenizer.MODE.SCRIPT_DATA;
              } else if (tn === $.PLAINTEXT) {
                  this.tokenizer.state = Tokenizer.MODE.PLAINTEXT;
              }
          }
      }

      //Tree mutation
      _setDocumentType(token) {
          const name = token.name || '';
          const publicId = token.publicId || '';
          const systemId = token.systemId || '';

          this.treeAdapter.setDocumentType(this.document, name, publicId, systemId);
      }

      _attachElementToTree(element) {
          if (this._shouldFosterParentOnInsertion()) {
              this._fosterParentElement(element);
          } else {
              const parent = this.openElements.currentTmplContent || this.openElements.current;

              this.treeAdapter.appendChild(parent, element);
          }
      }

      _appendElement(token, namespaceURI) {
          const element = this.treeAdapter.createElement(token.tagName, namespaceURI, token.attrs);

          this._attachElementToTree(element);
      }

      _insertElement(token, namespaceURI) {
          const element = this.treeAdapter.createElement(token.tagName, namespaceURI, token.attrs);

          this._attachElementToTree(element);
          this.openElements.push(element);
      }

      _insertFakeElement(tagName) {
          const element = this.treeAdapter.createElement(tagName, NS.HTML, []);

          this._attachElementToTree(element);
          this.openElements.push(element);
      }

      _insertTemplate(token) {
          const tmpl = this.treeAdapter.createElement(token.tagName, NS.HTML, token.attrs);
          const content = this.treeAdapter.createDocumentFragment();

          this.treeAdapter.setTemplateContent(tmpl, content);
          this._attachElementToTree(tmpl);
          this.openElements.push(tmpl);
      }

      _insertFakeRootElement() {
          const element = this.treeAdapter.createElement($.HTML, NS.HTML, []);

          this.treeAdapter.appendChild(this.openElements.current, element);
          this.openElements.push(element);
      }

      _appendCommentNode(token, parent) {
          const commentNode = this.treeAdapter.createCommentNode(token.data);

          this.treeAdapter.appendChild(parent, commentNode);
      }

      _insertCharacters(token) {
          if (this._shouldFosterParentOnInsertion()) {
              this._fosterParentText(token.chars);
          } else {
              const parent = this.openElements.currentTmplContent || this.openElements.current;

              this.treeAdapter.insertText(parent, token.chars);
          }
      }

      _adoptNodes(donor, recipient) {
          for (let child = this.treeAdapter.getFirstChild(donor); child; child = this.treeAdapter.getFirstChild(donor)) {
              this.treeAdapter.detachNode(child);
              this.treeAdapter.appendChild(recipient, child);
          }
      }

      //Token processing
      _shouldProcessTokenInForeignContent(token) {
          const current = this._getAdjustedCurrentElement();

          if (!current || current === this.document) {
              return false;
          }

          const ns = this.treeAdapter.getNamespaceURI(current);

          if (ns === NS.HTML) {
              return false;
          }

          if (
              this.treeAdapter.getTagName(current) === $.ANNOTATION_XML &&
              ns === NS.MATHML &&
              token.type === Tokenizer.START_TAG_TOKEN &&
              token.tagName === $.SVG
          ) {
              return false;
          }

          const isCharacterToken =
              token.type === Tokenizer.CHARACTER_TOKEN ||
              token.type === Tokenizer.NULL_CHARACTER_TOKEN ||
              token.type === Tokenizer.WHITESPACE_CHARACTER_TOKEN;

          const isMathMLTextStartTag =
              token.type === Tokenizer.START_TAG_TOKEN && token.tagName !== $.MGLYPH && token.tagName !== $.MALIGNMARK;

          if ((isMathMLTextStartTag || isCharacterToken) && this._isIntegrationPoint(current, NS.MATHML)) {
              return false;
          }

          if (
              (token.type === Tokenizer.START_TAG_TOKEN || isCharacterToken) &&
              this._isIntegrationPoint(current, NS.HTML)
          ) {
              return false;
          }

          return token.type !== Tokenizer.EOF_TOKEN;
      }

      _processToken(token) {
          TOKEN_HANDLERS[this.insertionMode][token.type](this, token);
      }

      _processTokenInBodyMode(token) {
          TOKEN_HANDLERS[IN_BODY_MODE][token.type](this, token);
      }

      _processTokenInForeignContent(token) {
          if (token.type === Tokenizer.CHARACTER_TOKEN) {
              characterInForeignContent(this, token);
          } else if (token.type === Tokenizer.NULL_CHARACTER_TOKEN) {
              nullCharacterInForeignContent(this, token);
          } else if (token.type === Tokenizer.WHITESPACE_CHARACTER_TOKEN) {
              insertCharacters(this, token);
          } else if (token.type === Tokenizer.COMMENT_TOKEN) {
              appendComment(this, token);
          } else if (token.type === Tokenizer.START_TAG_TOKEN) {
              startTagInForeignContent(this, token);
          } else if (token.type === Tokenizer.END_TAG_TOKEN) {
              endTagInForeignContent(this, token);
          }
      }

      _processInputToken(token) {
          if (this._shouldProcessTokenInForeignContent(token)) {
              this._processTokenInForeignContent(token);
          } else {
              this._processToken(token);
          }

          if (token.type === Tokenizer.START_TAG_TOKEN && token.selfClosing && !token.ackSelfClosing) {
              this._err(ERR.nonVoidHtmlElementStartTagWithTrailingSolidus);
          }
      }

      //Integration points
      _isIntegrationPoint(element, foreignNS) {
          const tn = this.treeAdapter.getTagName(element);
          const ns = this.treeAdapter.getNamespaceURI(element);
          const attrs = this.treeAdapter.getAttrList(element);

          return foreignContent.isIntegrationPoint(tn, ns, attrs, foreignNS);
      }

      //Active formatting elements reconstruction
      _reconstructActiveFormattingElements() {
          const listLength = this.activeFormattingElements.length;

          if (listLength) {
              let unopenIdx = listLength;
              let entry = null;

              do {
                  unopenIdx--;
                  entry = this.activeFormattingElements.entries[unopenIdx];

                  if (entry.type === FormattingElementList.MARKER_ENTRY || this.openElements.contains(entry.element)) {
                      unopenIdx++;
                      break;
                  }
              } while (unopenIdx > 0);

              for (let i = unopenIdx; i < listLength; i++) {
                  entry = this.activeFormattingElements.entries[i];
                  this._insertElement(entry.token, this.treeAdapter.getNamespaceURI(entry.element));
                  entry.element = this.openElements.current;
              }
          }
      }

      //Close elements
      _closeTableCell() {
          this.openElements.generateImpliedEndTags();
          this.openElements.popUntilTableCellPopped();
          this.activeFormattingElements.clearToLastMarker();
          this.insertionMode = IN_ROW_MODE;
      }

      _closePElement() {
          this.openElements.generateImpliedEndTagsWithExclusion($.P);
          this.openElements.popUntilTagNamePopped($.P);
      }

      //Insertion modes
      _resetInsertionMode() {
          for (let i = this.openElements.stackTop, last = false; i >= 0; i--) {
              let element = this.openElements.items[i];

              if (i === 0) {
                  last = true;

                  if (this.fragmentContext) {
                      element = this.fragmentContext;
                  }
              }

              const tn = this.treeAdapter.getTagName(element);
              const newInsertionMode = INSERTION_MODE_RESET_MAP[tn];

              if (newInsertionMode) {
                  this.insertionMode = newInsertionMode;
                  break;
              } else if (!last && (tn === $.TD || tn === $.TH)) {
                  this.insertionMode = IN_CELL_MODE;
                  break;
              } else if (!last && tn === $.HEAD) {
                  this.insertionMode = IN_HEAD_MODE;
                  break;
              } else if (tn === $.SELECT) {
                  this._resetInsertionModeForSelect(i);
                  break;
              } else if (tn === $.TEMPLATE) {
                  this.insertionMode = this.currentTmplInsertionMode;
                  break;
              } else if (tn === $.HTML) {
                  this.insertionMode = this.headElement ? AFTER_HEAD_MODE : BEFORE_HEAD_MODE;
                  break;
              } else if (last) {
                  this.insertionMode = IN_BODY_MODE;
                  break;
              }
          }
      }

      _resetInsertionModeForSelect(selectIdx) {
          if (selectIdx > 0) {
              for (let i = selectIdx - 1; i > 0; i--) {
                  const ancestor = this.openElements.items[i];
                  const tn = this.treeAdapter.getTagName(ancestor);

                  if (tn === $.TEMPLATE) {
                      break;
                  } else if (tn === $.TABLE) {
                      this.insertionMode = IN_SELECT_IN_TABLE_MODE;
                      return;
                  }
              }
          }

          this.insertionMode = IN_SELECT_MODE;
      }

      _pushTmplInsertionMode(mode) {
          this.tmplInsertionModeStack.push(mode);
          this.tmplInsertionModeStackTop++;
          this.currentTmplInsertionMode = mode;
      }

      _popTmplInsertionMode() {
          this.tmplInsertionModeStack.pop();
          this.tmplInsertionModeStackTop--;
          this.currentTmplInsertionMode = this.tmplInsertionModeStack[this.tmplInsertionModeStackTop];
      }

      //Foster parenting
      _isElementCausesFosterParenting(element) {
          const tn = this.treeAdapter.getTagName(element);

          return tn === $.TABLE || tn === $.TBODY || tn === $.TFOOT || tn === $.THEAD || tn === $.TR;
      }

      _shouldFosterParentOnInsertion() {
          return this.fosterParentingEnabled && this._isElementCausesFosterParenting(this.openElements.current);
      }

      _findFosterParentingLocation() {
          const location = {
              parent: null,
              beforeElement: null
          };

          for (let i = this.openElements.stackTop; i >= 0; i--) {
              const openElement = this.openElements.items[i];
              const tn = this.treeAdapter.getTagName(openElement);
              const ns = this.treeAdapter.getNamespaceURI(openElement);

              if (tn === $.TEMPLATE && ns === NS.HTML) {
                  location.parent = this.treeAdapter.getTemplateContent(openElement);
                  break;
              } else if (tn === $.TABLE) {
                  location.parent = this.treeAdapter.getParentNode(openElement);

                  if (location.parent) {
                      location.beforeElement = openElement;
                  } else {
                      location.parent = this.openElements.items[i - 1];
                  }

                  break;
              }
          }

          if (!location.parent) {
              location.parent = this.openElements.items[0];
          }

          return location;
      }

      _fosterParentElement(element) {
          const location = this._findFosterParentingLocation();

          if (location.beforeElement) {
              this.treeAdapter.insertBefore(location.parent, element, location.beforeElement);
          } else {
              this.treeAdapter.appendChild(location.parent, element);
          }
      }

      _fosterParentText(chars) {
          const location = this._findFosterParentingLocation();

          if (location.beforeElement) {
              this.treeAdapter.insertTextBefore(location.parent, chars, location.beforeElement);
          } else {
              this.treeAdapter.insertText(location.parent, chars);
          }
      }

      //Special elements
      _isSpecialElement(element) {
          const tn = this.treeAdapter.getTagName(element);
          const ns = this.treeAdapter.getNamespaceURI(element);

          return HTML.SPECIAL_ELEMENTS[ns][tn];
      }
  }

  var parser$2 = Parser;

  //Adoption agency algorithm
  //(see: http://www.whatwg.org/specs/web-apps/current-work/multipage/tree-construction.html#adoptionAgency)
  //------------------------------------------------------------------

  //Steps 5-8 of the algorithm
  function aaObtainFormattingElementEntry(p, token) {
      let formattingElementEntry = p.activeFormattingElements.getElementEntryInScopeWithTagName(token.tagName);

      if (formattingElementEntry) {
          if (!p.openElements.contains(formattingElementEntry.element)) {
              p.activeFormattingElements.removeEntry(formattingElementEntry);
              formattingElementEntry = null;
          } else if (!p.openElements.hasInScope(token.tagName)) {
              formattingElementEntry = null;
          }
      } else {
          genericEndTagInBody(p, token);
      }

      return formattingElementEntry;
  }

  //Steps 9 and 10 of the algorithm
  function aaObtainFurthestBlock(p, formattingElementEntry) {
      let furthestBlock = null;

      for (let i = p.openElements.stackTop; i >= 0; i--) {
          const element = p.openElements.items[i];

          if (element === formattingElementEntry.element) {
              break;
          }

          if (p._isSpecialElement(element)) {
              furthestBlock = element;
          }
      }

      if (!furthestBlock) {
          p.openElements.popUntilElementPopped(formattingElementEntry.element);
          p.activeFormattingElements.removeEntry(formattingElementEntry);
      }

      return furthestBlock;
  }

  //Step 13 of the algorithm
  function aaInnerLoop(p, furthestBlock, formattingElement) {
      let lastElement = furthestBlock;
      let nextElement = p.openElements.getCommonAncestor(furthestBlock);

      for (let i = 0, element = nextElement; element !== formattingElement; i++, element = nextElement) {
          //NOTE: store next element for the next loop iteration (it may be deleted from the stack by step 9.5)
          nextElement = p.openElements.getCommonAncestor(element);

          const elementEntry = p.activeFormattingElements.getElementEntry(element);
          const counterOverflow = elementEntry && i >= AA_INNER_LOOP_ITER;
          const shouldRemoveFromOpenElements = !elementEntry || counterOverflow;

          if (shouldRemoveFromOpenElements) {
              if (counterOverflow) {
                  p.activeFormattingElements.removeEntry(elementEntry);
              }

              p.openElements.remove(element);
          } else {
              element = aaRecreateElementFromEntry(p, elementEntry);

              if (lastElement === furthestBlock) {
                  p.activeFormattingElements.bookmark = elementEntry;
              }

              p.treeAdapter.detachNode(lastElement);
              p.treeAdapter.appendChild(element, lastElement);
              lastElement = element;
          }
      }

      return lastElement;
  }

  //Step 13.7 of the algorithm
  function aaRecreateElementFromEntry(p, elementEntry) {
      const ns = p.treeAdapter.getNamespaceURI(elementEntry.element);
      const newElement = p.treeAdapter.createElement(elementEntry.token.tagName, ns, elementEntry.token.attrs);

      p.openElements.replace(elementEntry.element, newElement);
      elementEntry.element = newElement;

      return newElement;
  }

  //Step 14 of the algorithm
  function aaInsertLastNodeInCommonAncestor(p, commonAncestor, lastElement) {
      if (p._isElementCausesFosterParenting(commonAncestor)) {
          p._fosterParentElement(lastElement);
      } else {
          const tn = p.treeAdapter.getTagName(commonAncestor);
          const ns = p.treeAdapter.getNamespaceURI(commonAncestor);

          if (tn === $.TEMPLATE && ns === NS.HTML) {
              commonAncestor = p.treeAdapter.getTemplateContent(commonAncestor);
          }

          p.treeAdapter.appendChild(commonAncestor, lastElement);
      }
  }

  //Steps 15-19 of the algorithm
  function aaReplaceFormattingElement(p, furthestBlock, formattingElementEntry) {
      const ns = p.treeAdapter.getNamespaceURI(formattingElementEntry.element);
      const token = formattingElementEntry.token;
      const newElement = p.treeAdapter.createElement(token.tagName, ns, token.attrs);

      p._adoptNodes(furthestBlock, newElement);
      p.treeAdapter.appendChild(furthestBlock, newElement);

      p.activeFormattingElements.insertElementAfterBookmark(newElement, formattingElementEntry.token);
      p.activeFormattingElements.removeEntry(formattingElementEntry);

      p.openElements.remove(formattingElementEntry.element);
      p.openElements.insertAfter(furthestBlock, newElement);
  }

  //Algorithm entry point
  function callAdoptionAgency(p, token) {
      let formattingElementEntry;

      for (let i = 0; i < AA_OUTER_LOOP_ITER; i++) {
          formattingElementEntry = aaObtainFormattingElementEntry(p, token);

          if (!formattingElementEntry) {
              break;
          }

          const furthestBlock = aaObtainFurthestBlock(p, formattingElementEntry);

          if (!furthestBlock) {
              break;
          }

          p.activeFormattingElements.bookmark = formattingElementEntry;

          const lastElement = aaInnerLoop(p, furthestBlock, formattingElementEntry.element);
          const commonAncestor = p.openElements.getCommonAncestor(formattingElementEntry.element);

          p.treeAdapter.detachNode(lastElement);
          aaInsertLastNodeInCommonAncestor(p, commonAncestor, lastElement);
          aaReplaceFormattingElement(p, furthestBlock, formattingElementEntry);
      }
  }

  //Generic token handlers
  //------------------------------------------------------------------
  function ignoreToken() {
      //NOTE: do nothing =)
  }

  function misplacedDoctype(p) {
      p._err(ERR.misplacedDoctype);
  }

  function appendComment(p, token) {
      p._appendCommentNode(token, p.openElements.currentTmplContent || p.openElements.current);
  }

  function appendCommentToRootHtmlElement(p, token) {
      p._appendCommentNode(token, p.openElements.items[0]);
  }

  function appendCommentToDocument(p, token) {
      p._appendCommentNode(token, p.document);
  }

  function insertCharacters(p, token) {
      p._insertCharacters(token);
  }

  function stopParsing(p) {
      p.stopped = true;
  }

  // The "initial" insertion mode
  //------------------------------------------------------------------
  function doctypeInInitialMode(p, token) {
      p._setDocumentType(token);

      const mode = token.forceQuirks ? HTML.DOCUMENT_MODE.QUIRKS : doctype$3.getDocumentMode(token);

      if (!doctype$3.isConforming(token)) {
          p._err(ERR.nonConformingDoctype);
      }

      p.treeAdapter.setDocumentMode(p.document, mode);

      p.insertionMode = BEFORE_HTML_MODE;
  }

  function tokenInInitialMode(p, token) {
      p._err(ERR.missingDoctype, { beforeToken: true });
      p.treeAdapter.setDocumentMode(p.document, HTML.DOCUMENT_MODE.QUIRKS);
      p.insertionMode = BEFORE_HTML_MODE;
      p._processToken(token);
  }

  // The "before html" insertion mode
  //------------------------------------------------------------------
  function startTagBeforeHtml(p, token) {
      if (token.tagName === $.HTML) {
          p._insertElement(token, NS.HTML);
          p.insertionMode = BEFORE_HEAD_MODE;
      } else {
          tokenBeforeHtml(p, token);
      }
  }

  function endTagBeforeHtml(p, token) {
      const tn = token.tagName;

      if (tn === $.HTML || tn === $.HEAD || tn === $.BODY || tn === $.BR) {
          tokenBeforeHtml(p, token);
      }
  }

  function tokenBeforeHtml(p, token) {
      p._insertFakeRootElement();
      p.insertionMode = BEFORE_HEAD_MODE;
      p._processToken(token);
  }

  // The "before head" insertion mode
  //------------------------------------------------------------------
  function startTagBeforeHead(p, token) {
      const tn = token.tagName;

      if (tn === $.HTML) {
          startTagInBody(p, token);
      } else if (tn === $.HEAD) {
          p._insertElement(token, NS.HTML);
          p.headElement = p.openElements.current;
          p.insertionMode = IN_HEAD_MODE;
      } else {
          tokenBeforeHead(p, token);
      }
  }

  function endTagBeforeHead(p, token) {
      const tn = token.tagName;

      if (tn === $.HEAD || tn === $.BODY || tn === $.HTML || tn === $.BR) {
          tokenBeforeHead(p, token);
      } else {
          p._err(ERR.endTagWithoutMatchingOpenElement);
      }
  }

  function tokenBeforeHead(p, token) {
      p._insertFakeElement($.HEAD);
      p.headElement = p.openElements.current;
      p.insertionMode = IN_HEAD_MODE;
      p._processToken(token);
  }

  // The "in head" insertion mode
  //------------------------------------------------------------------
  function startTagInHead(p, token) {
      const tn = token.tagName;

      if (tn === $.HTML) {
          startTagInBody(p, token);
      } else if (tn === $.BASE || tn === $.BASEFONT || tn === $.BGSOUND || tn === $.LINK || tn === $.META) {
          p._appendElement(token, NS.HTML);
          token.ackSelfClosing = true;
      } else if (tn === $.TITLE) {
          p._switchToTextParsing(token, Tokenizer.MODE.RCDATA);
      } else if (tn === $.NOSCRIPT) {
          if (p.options.scriptingEnabled) {
              p._switchToTextParsing(token, Tokenizer.MODE.RAWTEXT);
          } else {
              p._insertElement(token, NS.HTML);
              p.insertionMode = IN_HEAD_NO_SCRIPT_MODE;
          }
      } else if (tn === $.NOFRAMES || tn === $.STYLE) {
          p._switchToTextParsing(token, Tokenizer.MODE.RAWTEXT);
      } else if (tn === $.SCRIPT) {
          p._switchToTextParsing(token, Tokenizer.MODE.SCRIPT_DATA);
      } else if (tn === $.TEMPLATE) {
          p._insertTemplate(token, NS.HTML);
          p.activeFormattingElements.insertMarker();
          p.framesetOk = false;
          p.insertionMode = IN_TEMPLATE_MODE;
          p._pushTmplInsertionMode(IN_TEMPLATE_MODE);
      } else if (tn === $.HEAD) {
          p._err(ERR.misplacedStartTagForHeadElement);
      } else {
          tokenInHead(p, token);
      }
  }

  function endTagInHead(p, token) {
      const tn = token.tagName;

      if (tn === $.HEAD) {
          p.openElements.pop();
          p.insertionMode = AFTER_HEAD_MODE;
      } else if (tn === $.BODY || tn === $.BR || tn === $.HTML) {
          tokenInHead(p, token);
      } else if (tn === $.TEMPLATE) {
          if (p.openElements.tmplCount > 0) {
              p.openElements.generateImpliedEndTagsThoroughly();

              if (p.openElements.currentTagName !== $.TEMPLATE) {
                  p._err(ERR.closingOfElementWithOpenChildElements);
              }

              p.openElements.popUntilTagNamePopped($.TEMPLATE);
              p.activeFormattingElements.clearToLastMarker();
              p._popTmplInsertionMode();
              p._resetInsertionMode();
          } else {
              p._err(ERR.endTagWithoutMatchingOpenElement);
          }
      } else {
          p._err(ERR.endTagWithoutMatchingOpenElement);
      }
  }

  function tokenInHead(p, token) {
      p.openElements.pop();
      p.insertionMode = AFTER_HEAD_MODE;
      p._processToken(token);
  }

  // The "in head no script" insertion mode
  //------------------------------------------------------------------
  function startTagInHeadNoScript(p, token) {
      const tn = token.tagName;

      if (tn === $.HTML) {
          startTagInBody(p, token);
      } else if (
          tn === $.BASEFONT ||
          tn === $.BGSOUND ||
          tn === $.HEAD ||
          tn === $.LINK ||
          tn === $.META ||
          tn === $.NOFRAMES ||
          tn === $.STYLE
      ) {
          startTagInHead(p, token);
      } else if (tn === $.NOSCRIPT) {
          p._err(ERR.nestedNoscriptInHead);
      } else {
          tokenInHeadNoScript(p, token);
      }
  }

  function endTagInHeadNoScript(p, token) {
      const tn = token.tagName;

      if (tn === $.NOSCRIPT) {
          p.openElements.pop();
          p.insertionMode = IN_HEAD_MODE;
      } else if (tn === $.BR) {
          tokenInHeadNoScript(p, token);
      } else {
          p._err(ERR.endTagWithoutMatchingOpenElement);
      }
  }

  function tokenInHeadNoScript(p, token) {
      const errCode =
          token.type === Tokenizer.EOF_TOKEN ? ERR.openElementsLeftAfterEof : ERR.disallowedContentInNoscriptInHead;

      p._err(errCode);
      p.openElements.pop();
      p.insertionMode = IN_HEAD_MODE;
      p._processToken(token);
  }

  // The "after head" insertion mode
  //------------------------------------------------------------------
  function startTagAfterHead(p, token) {
      const tn = token.tagName;

      if (tn === $.HTML) {
          startTagInBody(p, token);
      } else if (tn === $.BODY) {
          p._insertElement(token, NS.HTML);
          p.framesetOk = false;
          p.insertionMode = IN_BODY_MODE;
      } else if (tn === $.FRAMESET) {
          p._insertElement(token, NS.HTML);
          p.insertionMode = IN_FRAMESET_MODE;
      } else if (
          tn === $.BASE ||
          tn === $.BASEFONT ||
          tn === $.BGSOUND ||
          tn === $.LINK ||
          tn === $.META ||
          tn === $.NOFRAMES ||
          tn === $.SCRIPT ||
          tn === $.STYLE ||
          tn === $.TEMPLATE ||
          tn === $.TITLE
      ) {
          p._err(ERR.abandonedHeadElementChild);
          p.openElements.push(p.headElement);
          startTagInHead(p, token);
          p.openElements.remove(p.headElement);
      } else if (tn === $.HEAD) {
          p._err(ERR.misplacedStartTagForHeadElement);
      } else {
          tokenAfterHead(p, token);
      }
  }

  function endTagAfterHead(p, token) {
      const tn = token.tagName;

      if (tn === $.BODY || tn === $.HTML || tn === $.BR) {
          tokenAfterHead(p, token);
      } else if (tn === $.TEMPLATE) {
          endTagInHead(p, token);
      } else {
          p._err(ERR.endTagWithoutMatchingOpenElement);
      }
  }

  function tokenAfterHead(p, token) {
      p._insertFakeElement($.BODY);
      p.insertionMode = IN_BODY_MODE;
      p._processToken(token);
  }

  // The "in body" insertion mode
  //------------------------------------------------------------------
  function whitespaceCharacterInBody(p, token) {
      p._reconstructActiveFormattingElements();
      p._insertCharacters(token);
  }

  function characterInBody(p, token) {
      p._reconstructActiveFormattingElements();
      p._insertCharacters(token);
      p.framesetOk = false;
  }

  function htmlStartTagInBody(p, token) {
      if (p.openElements.tmplCount === 0) {
          p.treeAdapter.adoptAttributes(p.openElements.items[0], token.attrs);
      }
  }

  function bodyStartTagInBody(p, token) {
      const bodyElement = p.openElements.tryPeekProperlyNestedBodyElement();

      if (bodyElement && p.openElements.tmplCount === 0) {
          p.framesetOk = false;
          p.treeAdapter.adoptAttributes(bodyElement, token.attrs);
      }
  }

  function framesetStartTagInBody(p, token) {
      const bodyElement = p.openElements.tryPeekProperlyNestedBodyElement();

      if (p.framesetOk && bodyElement) {
          p.treeAdapter.detachNode(bodyElement);
          p.openElements.popAllUpToHtmlElement();
          p._insertElement(token, NS.HTML);
          p.insertionMode = IN_FRAMESET_MODE;
      }
  }

  function addressStartTagInBody(p, token) {
      if (p.openElements.hasInButtonScope($.P)) {
          p._closePElement();
      }

      p._insertElement(token, NS.HTML);
  }

  function numberedHeaderStartTagInBody(p, token) {
      if (p.openElements.hasInButtonScope($.P)) {
          p._closePElement();
      }

      const tn = p.openElements.currentTagName;

      if (tn === $.H1 || tn === $.H2 || tn === $.H3 || tn === $.H4 || tn === $.H5 || tn === $.H6) {
          p.openElements.pop();
      }

      p._insertElement(token, NS.HTML);
  }

  function preStartTagInBody(p, token) {
      if (p.openElements.hasInButtonScope($.P)) {
          p._closePElement();
      }

      p._insertElement(token, NS.HTML);
      //NOTE: If the next token is a U+000A LINE FEED (LF) character token, then ignore that token and move
      //on to the next one. (Newlines at the start of pre blocks are ignored as an authoring convenience.)
      p.skipNextNewLine = true;
      p.framesetOk = false;
  }

  function formStartTagInBody(p, token) {
      const inTemplate = p.openElements.tmplCount > 0;

      if (!p.formElement || inTemplate) {
          if (p.openElements.hasInButtonScope($.P)) {
              p._closePElement();
          }

          p._insertElement(token, NS.HTML);

          if (!inTemplate) {
              p.formElement = p.openElements.current;
          }
      }
  }

  function listItemStartTagInBody(p, token) {
      p.framesetOk = false;

      const tn = token.tagName;

      for (let i = p.openElements.stackTop; i >= 0; i--) {
          const element = p.openElements.items[i];
          const elementTn = p.treeAdapter.getTagName(element);
          let closeTn = null;

          if (tn === $.LI && elementTn === $.LI) {
              closeTn = $.LI;
          } else if ((tn === $.DD || tn === $.DT) && (elementTn === $.DD || elementTn === $.DT)) {
              closeTn = elementTn;
          }

          if (closeTn) {
              p.openElements.generateImpliedEndTagsWithExclusion(closeTn);
              p.openElements.popUntilTagNamePopped(closeTn);
              break;
          }

          if (elementTn !== $.ADDRESS && elementTn !== $.DIV && elementTn !== $.P && p._isSpecialElement(element)) {
              break;
          }
      }

      if (p.openElements.hasInButtonScope($.P)) {
          p._closePElement();
      }

      p._insertElement(token, NS.HTML);
  }

  function plaintextStartTagInBody(p, token) {
      if (p.openElements.hasInButtonScope($.P)) {
          p._closePElement();
      }

      p._insertElement(token, NS.HTML);
      p.tokenizer.state = Tokenizer.MODE.PLAINTEXT;
  }

  function buttonStartTagInBody(p, token) {
      if (p.openElements.hasInScope($.BUTTON)) {
          p.openElements.generateImpliedEndTags();
          p.openElements.popUntilTagNamePopped($.BUTTON);
      }

      p._reconstructActiveFormattingElements();
      p._insertElement(token, NS.HTML);
      p.framesetOk = false;
  }

  function aStartTagInBody(p, token) {
      const activeElementEntry = p.activeFormattingElements.getElementEntryInScopeWithTagName($.A);

      if (activeElementEntry) {
          callAdoptionAgency(p, token);
          p.openElements.remove(activeElementEntry.element);
          p.activeFormattingElements.removeEntry(activeElementEntry);
      }

      p._reconstructActiveFormattingElements();
      p._insertElement(token, NS.HTML);
      p.activeFormattingElements.pushElement(p.openElements.current, token);
  }

  function bStartTagInBody(p, token) {
      p._reconstructActiveFormattingElements();
      p._insertElement(token, NS.HTML);
      p.activeFormattingElements.pushElement(p.openElements.current, token);
  }

  function nobrStartTagInBody(p, token) {
      p._reconstructActiveFormattingElements();

      if (p.openElements.hasInScope($.NOBR)) {
          callAdoptionAgency(p, token);
          p._reconstructActiveFormattingElements();
      }

      p._insertElement(token, NS.HTML);
      p.activeFormattingElements.pushElement(p.openElements.current, token);
  }

  function appletStartTagInBody(p, token) {
      p._reconstructActiveFormattingElements();
      p._insertElement(token, NS.HTML);
      p.activeFormattingElements.insertMarker();
      p.framesetOk = false;
  }

  function tableStartTagInBody(p, token) {
      if (
          p.treeAdapter.getDocumentMode(p.document) !== HTML.DOCUMENT_MODE.QUIRKS &&
          p.openElements.hasInButtonScope($.P)
      ) {
          p._closePElement();
      }

      p._insertElement(token, NS.HTML);
      p.framesetOk = false;
      p.insertionMode = IN_TABLE_MODE;
  }

  function areaStartTagInBody(p, token) {
      p._reconstructActiveFormattingElements();
      p._appendElement(token, NS.HTML);
      p.framesetOk = false;
      token.ackSelfClosing = true;
  }

  function inputStartTagInBody(p, token) {
      p._reconstructActiveFormattingElements();
      p._appendElement(token, NS.HTML);

      const inputType = Tokenizer.getTokenAttr(token, ATTRS.TYPE);

      if (!inputType || inputType.toLowerCase() !== HIDDEN_INPUT_TYPE) {
          p.framesetOk = false;
      }

      token.ackSelfClosing = true;
  }

  function paramStartTagInBody(p, token) {
      p._appendElement(token, NS.HTML);
      token.ackSelfClosing = true;
  }

  function hrStartTagInBody(p, token) {
      if (p.openElements.hasInButtonScope($.P)) {
          p._closePElement();
      }

      p._appendElement(token, NS.HTML);
      p.framesetOk = false;
      token.ackSelfClosing = true;
  }

  function imageStartTagInBody(p, token) {
      token.tagName = $.IMG;
      areaStartTagInBody(p, token);
  }

  function textareaStartTagInBody(p, token) {
      p._insertElement(token, NS.HTML);
      //NOTE: If the next token is a U+000A LINE FEED (LF) character token, then ignore that token and move
      //on to the next one. (Newlines at the start of textarea elements are ignored as an authoring convenience.)
      p.skipNextNewLine = true;
      p.tokenizer.state = Tokenizer.MODE.RCDATA;
      p.originalInsertionMode = p.insertionMode;
      p.framesetOk = false;
      p.insertionMode = TEXT_MODE;
  }

  function xmpStartTagInBody(p, token) {
      if (p.openElements.hasInButtonScope($.P)) {
          p._closePElement();
      }

      p._reconstructActiveFormattingElements();
      p.framesetOk = false;
      p._switchToTextParsing(token, Tokenizer.MODE.RAWTEXT);
  }

  function iframeStartTagInBody(p, token) {
      p.framesetOk = false;
      p._switchToTextParsing(token, Tokenizer.MODE.RAWTEXT);
  }

  //NOTE: here we assume that we always act as an user agent with enabled plugins, so we parse
  //<noembed> as a rawtext.
  function noembedStartTagInBody(p, token) {
      p._switchToTextParsing(token, Tokenizer.MODE.RAWTEXT);
  }

  function selectStartTagInBody(p, token) {
      p._reconstructActiveFormattingElements();
      p._insertElement(token, NS.HTML);
      p.framesetOk = false;

      if (
          p.insertionMode === IN_TABLE_MODE ||
          p.insertionMode === IN_CAPTION_MODE ||
          p.insertionMode === IN_TABLE_BODY_MODE ||
          p.insertionMode === IN_ROW_MODE ||
          p.insertionMode === IN_CELL_MODE
      ) {
          p.insertionMode = IN_SELECT_IN_TABLE_MODE;
      } else {
          p.insertionMode = IN_SELECT_MODE;
      }
  }

  function optgroupStartTagInBody(p, token) {
      if (p.openElements.currentTagName === $.OPTION) {
          p.openElements.pop();
      }

      p._reconstructActiveFormattingElements();
      p._insertElement(token, NS.HTML);
  }

  function rbStartTagInBody(p, token) {
      if (p.openElements.hasInScope($.RUBY)) {
          p.openElements.generateImpliedEndTags();
      }

      p._insertElement(token, NS.HTML);
  }

  function rtStartTagInBody(p, token) {
      if (p.openElements.hasInScope($.RUBY)) {
          p.openElements.generateImpliedEndTagsWithExclusion($.RTC);
      }

      p._insertElement(token, NS.HTML);
  }

  function menuStartTagInBody(p, token) {
      if (p.openElements.hasInButtonScope($.P)) {
          p._closePElement();
      }

      p._insertElement(token, NS.HTML);
  }

  function mathStartTagInBody(p, token) {
      p._reconstructActiveFormattingElements();

      foreignContent.adjustTokenMathMLAttrs(token);
      foreignContent.adjustTokenXMLAttrs(token);

      if (token.selfClosing) {
          p._appendElement(token, NS.MATHML);
      } else {
          p._insertElement(token, NS.MATHML);
      }

      token.ackSelfClosing = true;
  }

  function svgStartTagInBody(p, token) {
      p._reconstructActiveFormattingElements();

      foreignContent.adjustTokenSVGAttrs(token);
      foreignContent.adjustTokenXMLAttrs(token);

      if (token.selfClosing) {
          p._appendElement(token, NS.SVG);
      } else {
          p._insertElement(token, NS.SVG);
      }

      token.ackSelfClosing = true;
  }

  function genericStartTagInBody(p, token) {
      p._reconstructActiveFormattingElements();
      p._insertElement(token, NS.HTML);
  }

  //OPTIMIZATION: Integer comparisons are low-cost, so we can use very fast tag name length filters here.
  //It's faster than using dictionary.
  function startTagInBody(p, token) {
      const tn = token.tagName;

      switch (tn.length) {
          case 1:
              if (tn === $.I || tn === $.S || tn === $.B || tn === $.U) {
                  bStartTagInBody(p, token);
              } else if (tn === $.P) {
                  addressStartTagInBody(p, token);
              } else if (tn === $.A) {
                  aStartTagInBody(p, token);
              } else {
                  genericStartTagInBody(p, token);
              }

              break;

          case 2:
              if (tn === $.DL || tn === $.OL || tn === $.UL) {
                  addressStartTagInBody(p, token);
              } else if (tn === $.H1 || tn === $.H2 || tn === $.H3 || tn === $.H4 || tn === $.H5 || tn === $.H6) {
                  numberedHeaderStartTagInBody(p, token);
              } else if (tn === $.LI || tn === $.DD || tn === $.DT) {
                  listItemStartTagInBody(p, token);
              } else if (tn === $.EM || tn === $.TT) {
                  bStartTagInBody(p, token);
              } else if (tn === $.BR) {
                  areaStartTagInBody(p, token);
              } else if (tn === $.HR) {
                  hrStartTagInBody(p, token);
              } else if (tn === $.RB) {
                  rbStartTagInBody(p, token);
              } else if (tn === $.RT || tn === $.RP) {
                  rtStartTagInBody(p, token);
              } else if (tn !== $.TH && tn !== $.TD && tn !== $.TR) {
                  genericStartTagInBody(p, token);
              }

              break;

          case 3:
              if (tn === $.DIV || tn === $.DIR || tn === $.NAV) {
                  addressStartTagInBody(p, token);
              } else if (tn === $.PRE) {
                  preStartTagInBody(p, token);
              } else if (tn === $.BIG) {
                  bStartTagInBody(p, token);
              } else if (tn === $.IMG || tn === $.WBR) {
                  areaStartTagInBody(p, token);
              } else if (tn === $.XMP) {
                  xmpStartTagInBody(p, token);
              } else if (tn === $.SVG) {
                  svgStartTagInBody(p, token);
              } else if (tn === $.RTC) {
                  rbStartTagInBody(p, token);
              } else if (tn !== $.COL) {
                  genericStartTagInBody(p, token);
              }

              break;

          case 4:
              if (tn === $.HTML) {
                  htmlStartTagInBody(p, token);
              } else if (tn === $.BASE || tn === $.LINK || tn === $.META) {
                  startTagInHead(p, token);
              } else if (tn === $.BODY) {
                  bodyStartTagInBody(p, token);
              } else if (tn === $.MAIN || tn === $.MENU) {
                  addressStartTagInBody(p, token);
              } else if (tn === $.FORM) {
                  formStartTagInBody(p, token);
              } else if (tn === $.CODE || tn === $.FONT) {
                  bStartTagInBody(p, token);
              } else if (tn === $.NOBR) {
                  nobrStartTagInBody(p, token);
              } else if (tn === $.AREA) {
                  areaStartTagInBody(p, token);
              } else if (tn === $.MATH) {
                  mathStartTagInBody(p, token);
              } else if (tn === $.MENU) {
                  menuStartTagInBody(p, token);
              } else if (tn !== $.HEAD) {
                  genericStartTagInBody(p, token);
              }

              break;

          case 5:
              if (tn === $.STYLE || tn === $.TITLE) {
                  startTagInHead(p, token);
              } else if (tn === $.ASIDE) {
                  addressStartTagInBody(p, token);
              } else if (tn === $.SMALL) {
                  bStartTagInBody(p, token);
              } else if (tn === $.TABLE) {
                  tableStartTagInBody(p, token);
              } else if (tn === $.EMBED) {
                  areaStartTagInBody(p, token);
              } else if (tn === $.INPUT) {
                  inputStartTagInBody(p, token);
              } else if (tn === $.PARAM || tn === $.TRACK) {
                  paramStartTagInBody(p, token);
              } else if (tn === $.IMAGE) {
                  imageStartTagInBody(p, token);
              } else if (tn !== $.FRAME && tn !== $.TBODY && tn !== $.TFOOT && tn !== $.THEAD) {
                  genericStartTagInBody(p, token);
              }

              break;

          case 6:
              if (tn === $.SCRIPT) {
                  startTagInHead(p, token);
              } else if (
                  tn === $.CENTER ||
                  tn === $.FIGURE ||
                  tn === $.FOOTER ||
                  tn === $.HEADER ||
                  tn === $.HGROUP ||
                  tn === $.DIALOG
              ) {
                  addressStartTagInBody(p, token);
              } else if (tn === $.BUTTON) {
                  buttonStartTagInBody(p, token);
              } else if (tn === $.STRIKE || tn === $.STRONG) {
                  bStartTagInBody(p, token);
              } else if (tn === $.APPLET || tn === $.OBJECT) {
                  appletStartTagInBody(p, token);
              } else if (tn === $.KEYGEN) {
                  areaStartTagInBody(p, token);
              } else if (tn === $.SOURCE) {
                  paramStartTagInBody(p, token);
              } else if (tn === $.IFRAME) {
                  iframeStartTagInBody(p, token);
              } else if (tn === $.SELECT) {
                  selectStartTagInBody(p, token);
              } else if (tn === $.OPTION) {
                  optgroupStartTagInBody(p, token);
              } else {
                  genericStartTagInBody(p, token);
              }

              break;

          case 7:
              if (tn === $.BGSOUND) {
                  startTagInHead(p, token);
              } else if (
                  tn === $.DETAILS ||
                  tn === $.ADDRESS ||
                  tn === $.ARTICLE ||
                  tn === $.SECTION ||
                  tn === $.SUMMARY
              ) {
                  addressStartTagInBody(p, token);
              } else if (tn === $.LISTING) {
                  preStartTagInBody(p, token);
              } else if (tn === $.MARQUEE) {
                  appletStartTagInBody(p, token);
              } else if (tn === $.NOEMBED) {
                  noembedStartTagInBody(p, token);
              } else if (tn !== $.CAPTION) {
                  genericStartTagInBody(p, token);
              }

              break;

          case 8:
              if (tn === $.BASEFONT) {
                  startTagInHead(p, token);
              } else if (tn === $.FRAMESET) {
                  framesetStartTagInBody(p, token);
              } else if (tn === $.FIELDSET) {
                  addressStartTagInBody(p, token);
              } else if (tn === $.TEXTAREA) {
                  textareaStartTagInBody(p, token);
              } else if (tn === $.TEMPLATE) {
                  startTagInHead(p, token);
              } else if (tn === $.NOSCRIPT) {
                  if (p.options.scriptingEnabled) {
                      noembedStartTagInBody(p, token);
                  } else {
                      genericStartTagInBody(p, token);
                  }
              } else if (tn === $.OPTGROUP) {
                  optgroupStartTagInBody(p, token);
              } else if (tn !== $.COLGROUP) {
                  genericStartTagInBody(p, token);
              }

              break;

          case 9:
              if (tn === $.PLAINTEXT) {
                  plaintextStartTagInBody(p, token);
              } else {
                  genericStartTagInBody(p, token);
              }

              break;

          case 10:
              if (tn === $.BLOCKQUOTE || tn === $.FIGCAPTION) {
                  addressStartTagInBody(p, token);
              } else {
                  genericStartTagInBody(p, token);
              }

              break;

          default:
              genericStartTagInBody(p, token);
      }
  }

  function bodyEndTagInBody(p) {
      if (p.openElements.hasInScope($.BODY)) {
          p.insertionMode = AFTER_BODY_MODE;
      }
  }

  function htmlEndTagInBody(p, token) {
      if (p.openElements.hasInScope($.BODY)) {
          p.insertionMode = AFTER_BODY_MODE;
          p._processToken(token);
      }
  }

  function addressEndTagInBody(p, token) {
      const tn = token.tagName;

      if (p.openElements.hasInScope(tn)) {
          p.openElements.generateImpliedEndTags();
          p.openElements.popUntilTagNamePopped(tn);
      }
  }

  function formEndTagInBody(p) {
      const inTemplate = p.openElements.tmplCount > 0;
      const formElement = p.formElement;

      if (!inTemplate) {
          p.formElement = null;
      }

      if ((formElement || inTemplate) && p.openElements.hasInScope($.FORM)) {
          p.openElements.generateImpliedEndTags();

          if (inTemplate) {
              p.openElements.popUntilTagNamePopped($.FORM);
          } else {
              p.openElements.remove(formElement);
          }
      }
  }

  function pEndTagInBody(p) {
      if (!p.openElements.hasInButtonScope($.P)) {
          p._insertFakeElement($.P);
      }

      p._closePElement();
  }

  function liEndTagInBody(p) {
      if (p.openElements.hasInListItemScope($.LI)) {
          p.openElements.generateImpliedEndTagsWithExclusion($.LI);
          p.openElements.popUntilTagNamePopped($.LI);
      }
  }

  function ddEndTagInBody(p, token) {
      const tn = token.tagName;

      if (p.openElements.hasInScope(tn)) {
          p.openElements.generateImpliedEndTagsWithExclusion(tn);
          p.openElements.popUntilTagNamePopped(tn);
      }
  }

  function numberedHeaderEndTagInBody(p) {
      if (p.openElements.hasNumberedHeaderInScope()) {
          p.openElements.generateImpliedEndTags();
          p.openElements.popUntilNumberedHeaderPopped();
      }
  }

  function appletEndTagInBody(p, token) {
      const tn = token.tagName;

      if (p.openElements.hasInScope(tn)) {
          p.openElements.generateImpliedEndTags();
          p.openElements.popUntilTagNamePopped(tn);
          p.activeFormattingElements.clearToLastMarker();
      }
  }

  function brEndTagInBody(p) {
      p._reconstructActiveFormattingElements();
      p._insertFakeElement($.BR);
      p.openElements.pop();
      p.framesetOk = false;
  }

  function genericEndTagInBody(p, token) {
      const tn = token.tagName;

      for (let i = p.openElements.stackTop; i > 0; i--) {
          const element = p.openElements.items[i];

          if (p.treeAdapter.getTagName(element) === tn) {
              p.openElements.generateImpliedEndTagsWithExclusion(tn);
              p.openElements.popUntilElementPopped(element);
              break;
          }

          if (p._isSpecialElement(element)) {
              break;
          }
      }
  }

  //OPTIMIZATION: Integer comparisons are low-cost, so we can use very fast tag name length filters here.
  //It's faster than using dictionary.
  function endTagInBody(p, token) {
      const tn = token.tagName;

      switch (tn.length) {
          case 1:
              if (tn === $.A || tn === $.B || tn === $.I || tn === $.S || tn === $.U) {
                  callAdoptionAgency(p, token);
              } else if (tn === $.P) {
                  pEndTagInBody(p);
              } else {
                  genericEndTagInBody(p, token);
              }

              break;

          case 2:
              if (tn === $.DL || tn === $.UL || tn === $.OL) {
                  addressEndTagInBody(p, token);
              } else if (tn === $.LI) {
                  liEndTagInBody(p);
              } else if (tn === $.DD || tn === $.DT) {
                  ddEndTagInBody(p, token);
              } else if (tn === $.H1 || tn === $.H2 || tn === $.H3 || tn === $.H4 || tn === $.H5 || tn === $.H6) {
                  numberedHeaderEndTagInBody(p);
              } else if (tn === $.BR) {
                  brEndTagInBody(p);
              } else if (tn === $.EM || tn === $.TT) {
                  callAdoptionAgency(p, token);
              } else {
                  genericEndTagInBody(p, token);
              }

              break;

          case 3:
              if (tn === $.BIG) {
                  callAdoptionAgency(p, token);
              } else if (tn === $.DIR || tn === $.DIV || tn === $.NAV || tn === $.PRE) {
                  addressEndTagInBody(p, token);
              } else {
                  genericEndTagInBody(p, token);
              }

              break;

          case 4:
              if (tn === $.BODY) {
                  bodyEndTagInBody(p);
              } else if (tn === $.HTML) {
                  htmlEndTagInBody(p, token);
              } else if (tn === $.FORM) {
                  formEndTagInBody(p);
              } else if (tn === $.CODE || tn === $.FONT || tn === $.NOBR) {
                  callAdoptionAgency(p, token);
              } else if (tn === $.MAIN || tn === $.MENU) {
                  addressEndTagInBody(p, token);
              } else {
                  genericEndTagInBody(p, token);
              }

              break;

          case 5:
              if (tn === $.ASIDE) {
                  addressEndTagInBody(p, token);
              } else if (tn === $.SMALL) {
                  callAdoptionAgency(p, token);
              } else {
                  genericEndTagInBody(p, token);
              }

              break;

          case 6:
              if (
                  tn === $.CENTER ||
                  tn === $.FIGURE ||
                  tn === $.FOOTER ||
                  tn === $.HEADER ||
                  tn === $.HGROUP ||
                  tn === $.DIALOG
              ) {
                  addressEndTagInBody(p, token);
              } else if (tn === $.APPLET || tn === $.OBJECT) {
                  appletEndTagInBody(p, token);
              } else if (tn === $.STRIKE || tn === $.STRONG) {
                  callAdoptionAgency(p, token);
              } else {
                  genericEndTagInBody(p, token);
              }

              break;

          case 7:
              if (
                  tn === $.ADDRESS ||
                  tn === $.ARTICLE ||
                  tn === $.DETAILS ||
                  tn === $.SECTION ||
                  tn === $.SUMMARY ||
                  tn === $.LISTING
              ) {
                  addressEndTagInBody(p, token);
              } else if (tn === $.MARQUEE) {
                  appletEndTagInBody(p, token);
              } else {
                  genericEndTagInBody(p, token);
              }

              break;

          case 8:
              if (tn === $.FIELDSET) {
                  addressEndTagInBody(p, token);
              } else if (tn === $.TEMPLATE) {
                  endTagInHead(p, token);
              } else {
                  genericEndTagInBody(p, token);
              }

              break;

          case 10:
              if (tn === $.BLOCKQUOTE || tn === $.FIGCAPTION) {
                  addressEndTagInBody(p, token);
              } else {
                  genericEndTagInBody(p, token);
              }

              break;

          default:
              genericEndTagInBody(p, token);
      }
  }

  function eofInBody(p, token) {
      if (p.tmplInsertionModeStackTop > -1) {
          eofInTemplate(p, token);
      } else {
          p.stopped = true;
      }
  }

  // The "text" insertion mode
  //------------------------------------------------------------------
  function endTagInText(p, token) {
      if (token.tagName === $.SCRIPT) {
          p.pendingScript = p.openElements.current;
      }

      p.openElements.pop();
      p.insertionMode = p.originalInsertionMode;
  }

  function eofInText(p, token) {
      p._err(ERR.eofInElementThatCanContainOnlyText);
      p.openElements.pop();
      p.insertionMode = p.originalInsertionMode;
      p._processToken(token);
  }

  // The "in table" insertion mode
  //------------------------------------------------------------------
  function characterInTable(p, token) {
      const curTn = p.openElements.currentTagName;

      if (curTn === $.TABLE || curTn === $.TBODY || curTn === $.TFOOT || curTn === $.THEAD || curTn === $.TR) {
          p.pendingCharacterTokens = [];
          p.hasNonWhitespacePendingCharacterToken = false;
          p.originalInsertionMode = p.insertionMode;
          p.insertionMode = IN_TABLE_TEXT_MODE;
          p._processToken(token);
      } else {
          tokenInTable(p, token);
      }
  }

  function captionStartTagInTable(p, token) {
      p.openElements.clearBackToTableContext();
      p.activeFormattingElements.insertMarker();
      p._insertElement(token, NS.HTML);
      p.insertionMode = IN_CAPTION_MODE;
  }

  function colgroupStartTagInTable(p, token) {
      p.openElements.clearBackToTableContext();
      p._insertElement(token, NS.HTML);
      p.insertionMode = IN_COLUMN_GROUP_MODE;
  }

  function colStartTagInTable(p, token) {
      p.openElements.clearBackToTableContext();
      p._insertFakeElement($.COLGROUP);
      p.insertionMode = IN_COLUMN_GROUP_MODE;
      p._processToken(token);
  }

  function tbodyStartTagInTable(p, token) {
      p.openElements.clearBackToTableContext();
      p._insertElement(token, NS.HTML);
      p.insertionMode = IN_TABLE_BODY_MODE;
  }

  function tdStartTagInTable(p, token) {
      p.openElements.clearBackToTableContext();
      p._insertFakeElement($.TBODY);
      p.insertionMode = IN_TABLE_BODY_MODE;
      p._processToken(token);
  }

  function tableStartTagInTable(p, token) {
      if (p.openElements.hasInTableScope($.TABLE)) {
          p.openElements.popUntilTagNamePopped($.TABLE);
          p._resetInsertionMode();
          p._processToken(token);
      }
  }

  function inputStartTagInTable(p, token) {
      const inputType = Tokenizer.getTokenAttr(token, ATTRS.TYPE);

      if (inputType && inputType.toLowerCase() === HIDDEN_INPUT_TYPE) {
          p._appendElement(token, NS.HTML);
      } else {
          tokenInTable(p, token);
      }

      token.ackSelfClosing = true;
  }

  function formStartTagInTable(p, token) {
      if (!p.formElement && p.openElements.tmplCount === 0) {
          p._insertElement(token, NS.HTML);
          p.formElement = p.openElements.current;
          p.openElements.pop();
      }
  }

  function startTagInTable(p, token) {
      const tn = token.tagName;

      switch (tn.length) {
          case 2:
              if (tn === $.TD || tn === $.TH || tn === $.TR) {
                  tdStartTagInTable(p, token);
              } else {
                  tokenInTable(p, token);
              }

              break;

          case 3:
              if (tn === $.COL) {
                  colStartTagInTable(p, token);
              } else {
                  tokenInTable(p, token);
              }

              break;

          case 4:
              if (tn === $.FORM) {
                  formStartTagInTable(p, token);
              } else {
                  tokenInTable(p, token);
              }

              break;

          case 5:
              if (tn === $.TABLE) {
                  tableStartTagInTable(p, token);
              } else if (tn === $.STYLE) {
                  startTagInHead(p, token);
              } else if (tn === $.TBODY || tn === $.TFOOT || tn === $.THEAD) {
                  tbodyStartTagInTable(p, token);
              } else if (tn === $.INPUT) {
                  inputStartTagInTable(p, token);
              } else {
                  tokenInTable(p, token);
              }

              break;

          case 6:
              if (tn === $.SCRIPT) {
                  startTagInHead(p, token);
              } else {
                  tokenInTable(p, token);
              }

              break;

          case 7:
              if (tn === $.CAPTION) {
                  captionStartTagInTable(p, token);
              } else {
                  tokenInTable(p, token);
              }

              break;

          case 8:
              if (tn === $.COLGROUP) {
                  colgroupStartTagInTable(p, token);
              } else if (tn === $.TEMPLATE) {
                  startTagInHead(p, token);
              } else {
                  tokenInTable(p, token);
              }

              break;

          default:
              tokenInTable(p, token);
      }
  }

  function endTagInTable(p, token) {
      const tn = token.tagName;

      if (tn === $.TABLE) {
          if (p.openElements.hasInTableScope($.TABLE)) {
              p.openElements.popUntilTagNamePopped($.TABLE);
              p._resetInsertionMode();
          }
      } else if (tn === $.TEMPLATE) {
          endTagInHead(p, token);
      } else if (
          tn !== $.BODY &&
          tn !== $.CAPTION &&
          tn !== $.COL &&
          tn !== $.COLGROUP &&
          tn !== $.HTML &&
          tn !== $.TBODY &&
          tn !== $.TD &&
          tn !== $.TFOOT &&
          tn !== $.TH &&
          tn !== $.THEAD &&
          tn !== $.TR
      ) {
          tokenInTable(p, token);
      }
  }

  function tokenInTable(p, token) {
      const savedFosterParentingState = p.fosterParentingEnabled;

      p.fosterParentingEnabled = true;
      p._processTokenInBodyMode(token);
      p.fosterParentingEnabled = savedFosterParentingState;
  }

  // The "in table text" insertion mode
  //------------------------------------------------------------------
  function whitespaceCharacterInTableText(p, token) {
      p.pendingCharacterTokens.push(token);
  }

  function characterInTableText(p, token) {
      p.pendingCharacterTokens.push(token);
      p.hasNonWhitespacePendingCharacterToken = true;
  }

  function tokenInTableText(p, token) {
      let i = 0;

      if (p.hasNonWhitespacePendingCharacterToken) {
          for (; i < p.pendingCharacterTokens.length; i++) {
              tokenInTable(p, p.pendingCharacterTokens[i]);
          }
      } else {
          for (; i < p.pendingCharacterTokens.length; i++) {
              p._insertCharacters(p.pendingCharacterTokens[i]);
          }
      }

      p.insertionMode = p.originalInsertionMode;
      p._processToken(token);
  }

  // The "in caption" insertion mode
  //------------------------------------------------------------------
  function startTagInCaption(p, token) {
      const tn = token.tagName;

      if (
          tn === $.CAPTION ||
          tn === $.COL ||
          tn === $.COLGROUP ||
          tn === $.TBODY ||
          tn === $.TD ||
          tn === $.TFOOT ||
          tn === $.TH ||
          tn === $.THEAD ||
          tn === $.TR
      ) {
          if (p.openElements.hasInTableScope($.CAPTION)) {
              p.openElements.generateImpliedEndTags();
              p.openElements.popUntilTagNamePopped($.CAPTION);
              p.activeFormattingElements.clearToLastMarker();
              p.insertionMode = IN_TABLE_MODE;
              p._processToken(token);
          }
      } else {
          startTagInBody(p, token);
      }
  }

  function endTagInCaption(p, token) {
      const tn = token.tagName;

      if (tn === $.CAPTION || tn === $.TABLE) {
          if (p.openElements.hasInTableScope($.CAPTION)) {
              p.openElements.generateImpliedEndTags();
              p.openElements.popUntilTagNamePopped($.CAPTION);
              p.activeFormattingElements.clearToLastMarker();
              p.insertionMode = IN_TABLE_MODE;

              if (tn === $.TABLE) {
                  p._processToken(token);
              }
          }
      } else if (
          tn !== $.BODY &&
          tn !== $.COL &&
          tn !== $.COLGROUP &&
          tn !== $.HTML &&
          tn !== $.TBODY &&
          tn !== $.TD &&
          tn !== $.TFOOT &&
          tn !== $.TH &&
          tn !== $.THEAD &&
          tn !== $.TR
      ) {
          endTagInBody(p, token);
      }
  }

  // The "in column group" insertion mode
  //------------------------------------------------------------------
  function startTagInColumnGroup(p, token) {
      const tn = token.tagName;

      if (tn === $.HTML) {
          startTagInBody(p, token);
      } else if (tn === $.COL) {
          p._appendElement(token, NS.HTML);
          token.ackSelfClosing = true;
      } else if (tn === $.TEMPLATE) {
          startTagInHead(p, token);
      } else {
          tokenInColumnGroup(p, token);
      }
  }

  function endTagInColumnGroup(p, token) {
      const tn = token.tagName;

      if (tn === $.COLGROUP) {
          if (p.openElements.currentTagName === $.COLGROUP) {
              p.openElements.pop();
              p.insertionMode = IN_TABLE_MODE;
          }
      } else if (tn === $.TEMPLATE) {
          endTagInHead(p, token);
      } else if (tn !== $.COL) {
          tokenInColumnGroup(p, token);
      }
  }

  function tokenInColumnGroup(p, token) {
      if (p.openElements.currentTagName === $.COLGROUP) {
          p.openElements.pop();
          p.insertionMode = IN_TABLE_MODE;
          p._processToken(token);
      }
  }

  // The "in table body" insertion mode
  //------------------------------------------------------------------
  function startTagInTableBody(p, token) {
      const tn = token.tagName;

      if (tn === $.TR) {
          p.openElements.clearBackToTableBodyContext();
          p._insertElement(token, NS.HTML);
          p.insertionMode = IN_ROW_MODE;
      } else if (tn === $.TH || tn === $.TD) {
          p.openElements.clearBackToTableBodyContext();
          p._insertFakeElement($.TR);
          p.insertionMode = IN_ROW_MODE;
          p._processToken(token);
      } else if (
          tn === $.CAPTION ||
          tn === $.COL ||
          tn === $.COLGROUP ||
          tn === $.TBODY ||
          tn === $.TFOOT ||
          tn === $.THEAD
      ) {
          if (p.openElements.hasTableBodyContextInTableScope()) {
              p.openElements.clearBackToTableBodyContext();
              p.openElements.pop();
              p.insertionMode = IN_TABLE_MODE;
              p._processToken(token);
          }
      } else {
          startTagInTable(p, token);
      }
  }

  function endTagInTableBody(p, token) {
      const tn = token.tagName;

      if (tn === $.TBODY || tn === $.TFOOT || tn === $.THEAD) {
          if (p.openElements.hasInTableScope(tn)) {
              p.openElements.clearBackToTableBodyContext();
              p.openElements.pop();
              p.insertionMode = IN_TABLE_MODE;
          }
      } else if (tn === $.TABLE) {
          if (p.openElements.hasTableBodyContextInTableScope()) {
              p.openElements.clearBackToTableBodyContext();
              p.openElements.pop();
              p.insertionMode = IN_TABLE_MODE;
              p._processToken(token);
          }
      } else if (
          (tn !== $.BODY && tn !== $.CAPTION && tn !== $.COL && tn !== $.COLGROUP) ||
          (tn !== $.HTML && tn !== $.TD && tn !== $.TH && tn !== $.TR)
      ) {
          endTagInTable(p, token);
      }
  }

  // The "in row" insertion mode
  //------------------------------------------------------------------
  function startTagInRow(p, token) {
      const tn = token.tagName;

      if (tn === $.TH || tn === $.TD) {
          p.openElements.clearBackToTableRowContext();
          p._insertElement(token, NS.HTML);
          p.insertionMode = IN_CELL_MODE;
          p.activeFormattingElements.insertMarker();
      } else if (
          tn === $.CAPTION ||
          tn === $.COL ||
          tn === $.COLGROUP ||
          tn === $.TBODY ||
          tn === $.TFOOT ||
          tn === $.THEAD ||
          tn === $.TR
      ) {
          if (p.openElements.hasInTableScope($.TR)) {
              p.openElements.clearBackToTableRowContext();
              p.openElements.pop();
              p.insertionMode = IN_TABLE_BODY_MODE;
              p._processToken(token);
          }
      } else {
          startTagInTable(p, token);
      }
  }

  function endTagInRow(p, token) {
      const tn = token.tagName;

      if (tn === $.TR) {
          if (p.openElements.hasInTableScope($.TR)) {
              p.openElements.clearBackToTableRowContext();
              p.openElements.pop();
              p.insertionMode = IN_TABLE_BODY_MODE;
          }
      } else if (tn === $.TABLE) {
          if (p.openElements.hasInTableScope($.TR)) {
              p.openElements.clearBackToTableRowContext();
              p.openElements.pop();
              p.insertionMode = IN_TABLE_BODY_MODE;
              p._processToken(token);
          }
      } else if (tn === $.TBODY || tn === $.TFOOT || tn === $.THEAD) {
          if (p.openElements.hasInTableScope(tn) || p.openElements.hasInTableScope($.TR)) {
              p.openElements.clearBackToTableRowContext();
              p.openElements.pop();
              p.insertionMode = IN_TABLE_BODY_MODE;
              p._processToken(token);
          }
      } else if (
          (tn !== $.BODY && tn !== $.CAPTION && tn !== $.COL && tn !== $.COLGROUP) ||
          (tn !== $.HTML && tn !== $.TD && tn !== $.TH)
      ) {
          endTagInTable(p, token);
      }
  }

  // The "in cell" insertion mode
  //------------------------------------------------------------------
  function startTagInCell(p, token) {
      const tn = token.tagName;

      if (
          tn === $.CAPTION ||
          tn === $.COL ||
          tn === $.COLGROUP ||
          tn === $.TBODY ||
          tn === $.TD ||
          tn === $.TFOOT ||
          tn === $.TH ||
          tn === $.THEAD ||
          tn === $.TR
      ) {
          if (p.openElements.hasInTableScope($.TD) || p.openElements.hasInTableScope($.TH)) {
              p._closeTableCell();
              p._processToken(token);
          }
      } else {
          startTagInBody(p, token);
      }
  }

  function endTagInCell(p, token) {
      const tn = token.tagName;

      if (tn === $.TD || tn === $.TH) {
          if (p.openElements.hasInTableScope(tn)) {
              p.openElements.generateImpliedEndTags();
              p.openElements.popUntilTagNamePopped(tn);
              p.activeFormattingElements.clearToLastMarker();
              p.insertionMode = IN_ROW_MODE;
          }
      } else if (tn === $.TABLE || tn === $.TBODY || tn === $.TFOOT || tn === $.THEAD || tn === $.TR) {
          if (p.openElements.hasInTableScope(tn)) {
              p._closeTableCell();
              p._processToken(token);
          }
      } else if (tn !== $.BODY && tn !== $.CAPTION && tn !== $.COL && tn !== $.COLGROUP && tn !== $.HTML) {
          endTagInBody(p, token);
      }
  }

  // The "in select" insertion mode
  //------------------------------------------------------------------
  function startTagInSelect(p, token) {
      const tn = token.tagName;

      if (tn === $.HTML) {
          startTagInBody(p, token);
      } else if (tn === $.OPTION) {
          if (p.openElements.currentTagName === $.OPTION) {
              p.openElements.pop();
          }

          p._insertElement(token, NS.HTML);
      } else if (tn === $.OPTGROUP) {
          if (p.openElements.currentTagName === $.OPTION) {
              p.openElements.pop();
          }

          if (p.openElements.currentTagName === $.OPTGROUP) {
              p.openElements.pop();
          }

          p._insertElement(token, NS.HTML);
      } else if (tn === $.INPUT || tn === $.KEYGEN || tn === $.TEXTAREA || tn === $.SELECT) {
          if (p.openElements.hasInSelectScope($.SELECT)) {
              p.openElements.popUntilTagNamePopped($.SELECT);
              p._resetInsertionMode();

              if (tn !== $.SELECT) {
                  p._processToken(token);
              }
          }
      } else if (tn === $.SCRIPT || tn === $.TEMPLATE) {
          startTagInHead(p, token);
      }
  }

  function endTagInSelect(p, token) {
      const tn = token.tagName;

      if (tn === $.OPTGROUP) {
          const prevOpenElement = p.openElements.items[p.openElements.stackTop - 1];
          const prevOpenElementTn = prevOpenElement && p.treeAdapter.getTagName(prevOpenElement);

          if (p.openElements.currentTagName === $.OPTION && prevOpenElementTn === $.OPTGROUP) {
              p.openElements.pop();
          }

          if (p.openElements.currentTagName === $.OPTGROUP) {
              p.openElements.pop();
          }
      } else if (tn === $.OPTION) {
          if (p.openElements.currentTagName === $.OPTION) {
              p.openElements.pop();
          }
      } else if (tn === $.SELECT && p.openElements.hasInSelectScope($.SELECT)) {
          p.openElements.popUntilTagNamePopped($.SELECT);
          p._resetInsertionMode();
      } else if (tn === $.TEMPLATE) {
          endTagInHead(p, token);
      }
  }

  //12.2.5.4.17 The "in select in table" insertion mode
  //------------------------------------------------------------------
  function startTagInSelectInTable(p, token) {
      const tn = token.tagName;

      if (
          tn === $.CAPTION ||
          tn === $.TABLE ||
          tn === $.TBODY ||
          tn === $.TFOOT ||
          tn === $.THEAD ||
          tn === $.TR ||
          tn === $.TD ||
          tn === $.TH
      ) {
          p.openElements.popUntilTagNamePopped($.SELECT);
          p._resetInsertionMode();
          p._processToken(token);
      } else {
          startTagInSelect(p, token);
      }
  }

  function endTagInSelectInTable(p, token) {
      const tn = token.tagName;

      if (
          tn === $.CAPTION ||
          tn === $.TABLE ||
          tn === $.TBODY ||
          tn === $.TFOOT ||
          tn === $.THEAD ||
          tn === $.TR ||
          tn === $.TD ||
          tn === $.TH
      ) {
          if (p.openElements.hasInTableScope(tn)) {
              p.openElements.popUntilTagNamePopped($.SELECT);
              p._resetInsertionMode();
              p._processToken(token);
          }
      } else {
          endTagInSelect(p, token);
      }
  }

  // The "in template" insertion mode
  //------------------------------------------------------------------
  function startTagInTemplate(p, token) {
      const tn = token.tagName;

      if (
          tn === $.BASE ||
          tn === $.BASEFONT ||
          tn === $.BGSOUND ||
          tn === $.LINK ||
          tn === $.META ||
          tn === $.NOFRAMES ||
          tn === $.SCRIPT ||
          tn === $.STYLE ||
          tn === $.TEMPLATE ||
          tn === $.TITLE
      ) {
          startTagInHead(p, token);
      } else {
          const newInsertionMode = TEMPLATE_INSERTION_MODE_SWITCH_MAP[tn] || IN_BODY_MODE;

          p._popTmplInsertionMode();
          p._pushTmplInsertionMode(newInsertionMode);
          p.insertionMode = newInsertionMode;
          p._processToken(token);
      }
  }

  function endTagInTemplate(p, token) {
      if (token.tagName === $.TEMPLATE) {
          endTagInHead(p, token);
      }
  }

  function eofInTemplate(p, token) {
      if (p.openElements.tmplCount > 0) {
          p.openElements.popUntilTagNamePopped($.TEMPLATE);
          p.activeFormattingElements.clearToLastMarker();
          p._popTmplInsertionMode();
          p._resetInsertionMode();
          p._processToken(token);
      } else {
          p.stopped = true;
      }
  }

  // The "after body" insertion mode
  //------------------------------------------------------------------
  function startTagAfterBody(p, token) {
      if (token.tagName === $.HTML) {
          startTagInBody(p, token);
      } else {
          tokenAfterBody(p, token);
      }
  }

  function endTagAfterBody(p, token) {
      if (token.tagName === $.HTML) {
          if (!p.fragmentContext) {
              p.insertionMode = AFTER_AFTER_BODY_MODE;
          }
      } else {
          tokenAfterBody(p, token);
      }
  }

  function tokenAfterBody(p, token) {
      p.insertionMode = IN_BODY_MODE;
      p._processToken(token);
  }

  // The "in frameset" insertion mode
  //------------------------------------------------------------------
  function startTagInFrameset(p, token) {
      const tn = token.tagName;

      if (tn === $.HTML) {
          startTagInBody(p, token);
      } else if (tn === $.FRAMESET) {
          p._insertElement(token, NS.HTML);
      } else if (tn === $.FRAME) {
          p._appendElement(token, NS.HTML);
          token.ackSelfClosing = true;
      } else if (tn === $.NOFRAMES) {
          startTagInHead(p, token);
      }
  }

  function endTagInFrameset(p, token) {
      if (token.tagName === $.FRAMESET && !p.openElements.isRootHtmlElementCurrent()) {
          p.openElements.pop();

          if (!p.fragmentContext && p.openElements.currentTagName !== $.FRAMESET) {
              p.insertionMode = AFTER_FRAMESET_MODE;
          }
      }
  }

  // The "after frameset" insertion mode
  //------------------------------------------------------------------
  function startTagAfterFrameset(p, token) {
      const tn = token.tagName;

      if (tn === $.HTML) {
          startTagInBody(p, token);
      } else if (tn === $.NOFRAMES) {
          startTagInHead(p, token);
      }
  }

  function endTagAfterFrameset(p, token) {
      if (token.tagName === $.HTML) {
          p.insertionMode = AFTER_AFTER_FRAMESET_MODE;
      }
  }

  // The "after after body" insertion mode
  //------------------------------------------------------------------
  function startTagAfterAfterBody(p, token) {
      if (token.tagName === $.HTML) {
          startTagInBody(p, token);
      } else {
          tokenAfterAfterBody(p, token);
      }
  }

  function tokenAfterAfterBody(p, token) {
      p.insertionMode = IN_BODY_MODE;
      p._processToken(token);
  }

  // The "after after frameset" insertion mode
  //------------------------------------------------------------------
  function startTagAfterAfterFrameset(p, token) {
      const tn = token.tagName;

      if (tn === $.HTML) {
          startTagInBody(p, token);
      } else if (tn === $.NOFRAMES) {
          startTagInHead(p, token);
      }
  }

  // The rules for parsing tokens in foreign content
  //------------------------------------------------------------------
  function nullCharacterInForeignContent(p, token) {
      token.chars = unicode.REPLACEMENT_CHARACTER;
      p._insertCharacters(token);
  }

  function characterInForeignContent(p, token) {
      p._insertCharacters(token);
      p.framesetOk = false;
  }

  function startTagInForeignContent(p, token) {
      if (foreignContent.causesExit(token) && !p.fragmentContext) {
          while (
              p.treeAdapter.getNamespaceURI(p.openElements.current) !== NS.HTML &&
              !p._isIntegrationPoint(p.openElements.current)
          ) {
              p.openElements.pop();
          }

          p._processToken(token);
      } else {
          const current = p._getAdjustedCurrentElement();
          const currentNs = p.treeAdapter.getNamespaceURI(current);

          if (currentNs === NS.MATHML) {
              foreignContent.adjustTokenMathMLAttrs(token);
          } else if (currentNs === NS.SVG) {
              foreignContent.adjustTokenSVGTagName(token);
              foreignContent.adjustTokenSVGAttrs(token);
          }

          foreignContent.adjustTokenXMLAttrs(token);

          if (token.selfClosing) {
              p._appendElement(token, currentNs);
          } else {
              p._insertElement(token, currentNs);
          }

          token.ackSelfClosing = true;
      }
  }

  function endTagInForeignContent(p, token) {
      for (let i = p.openElements.stackTop; i > 0; i--) {
          const element = p.openElements.items[i];

          if (p.treeAdapter.getNamespaceURI(element) === NS.HTML) {
              p._processToken(token);
              break;
          }

          if (p.treeAdapter.getTagName(element).toLowerCase() === token.tagName) {
              p.openElements.popUntilElementPopped(element);
              break;
          }
      }
  }

  /**
   * @typedef {import('./info.js').Info} Info
   * @typedef {Record<string, Info>} Properties
   * @typedef {Record<string, string>} Normal
   */

  class Schema$5 {
    /**
     * @constructor
     * @param {Properties} property
     * @param {Normal} normal
     * @param {string} [space]
     */
    constructor(property, normal, space) {
      this.property = property;
      this.normal = normal;
      if (space) {
        this.space = space;
      }
    }
  }

  /** @type {Properties} */
  Schema$5.prototype.property = {};
  /** @type {Normal} */
  Schema$5.prototype.normal = {};
  /** @type {string|null} */
  Schema$5.prototype.space = null;

  /**
   * @typedef {import('./schema.js').Properties} Properties
   * @typedef {import('./schema.js').Normal} Normal
   */

  /**
   * @param {Schema[]} definitions
   * @param {string} [space]
   * @returns {Schema}
   */
  function merge$5(definitions, space) {
    /** @type {Properties} */
    const property = {};
    /** @type {Normal} */
    const normal = {};
    let index = -1;

    while (++index < definitions.length) {
      Object.assign(property, definitions[index].property);
      Object.assign(normal, definitions[index].normal);
    }

    return new Schema$5(property, normal, space)
  }

  /**
   * @param {string} value
   * @returns {string}
   */
  function normalize$5(value) {
    return value.toLowerCase()
  }

  class Info$5 {
    /**
     * @constructor
     * @param {string} property
     * @param {string} attribute
     */
    constructor(property, attribute) {
      /** @type {string} */
      this.property = property;
      /** @type {string} */
      this.attribute = attribute;
    }
  }

  /** @type {string|null} */
  Info$5.prototype.space = null;
  Info$5.prototype.boolean = false;
  Info$5.prototype.booleanish = false;
  Info$5.prototype.overloadedBoolean = false;
  Info$5.prototype.number = false;
  Info$5.prototype.commaSeparated = false;
  Info$5.prototype.spaceSeparated = false;
  Info$5.prototype.commaOrSpaceSeparated = false;
  Info$5.prototype.mustUseProperty = false;
  Info$5.prototype.defined = false;

  let powers$5 = 0;

  const boolean$5 = increment$5();
  const booleanish$5 = increment$5();
  const overloadedBoolean$5 = increment$5();
  const number$5 = increment$5();
  const spaceSeparated$5 = increment$5();
  const commaSeparated$5 = increment$5();
  const commaOrSpaceSeparated$5 = increment$5();

  function increment$5() {
    return 2 ** ++powers$5
  }

  var types$5 = /*#__PURE__*/Object.freeze({
      __proto__: null,
      boolean: boolean$5,
      booleanish: booleanish$5,
      overloadedBoolean: overloadedBoolean$5,
      number: number$5,
      spaceSeparated: spaceSeparated$5,
      commaSeparated: commaSeparated$5,
      commaOrSpaceSeparated: commaOrSpaceSeparated$5
  });

  /** @type {Array<keyof types>} */
  // @ts-expect-error: hush.
  const checks$5 = Object.keys(types$5);

  class DefinedInfo$5 extends Info$5 {
    /**
     * @constructor
     * @param {string} property
     * @param {string} attribute
     * @param {number|null} [mask]
     * @param {string} [space]
     */
    constructor(property, attribute, mask, space) {
      let index = -1;

      super(property, attribute);

      mark$5(this, 'space', space);

      if (typeof mask === 'number') {
        while (++index < checks$5.length) {
          const check = checks$5[index];
          mark$5(this, checks$5[index], (mask & types$5[check]) === types$5[check]);
        }
      }
    }
  }

  DefinedInfo$5.prototype.defined = true;

  /**
   * @param {DefinedInfo} values
   * @param {string} key
   * @param {unknown} value
   */
  function mark$5(values, key, value) {
    if (value) {
      // @ts-expect-error: assume `value` matches the expected value of `key`.
      values[key] = value;
    }
  }

  /**
   * @typedef {import('./schema.js').Properties} Properties
   * @typedef {import('./schema.js').Normal} Normal
   *
   * @typedef {Record<string, string>} Attributes
   *
   * @typedef {Object} Definition
   * @property {Record<string, number|null>} properties
   * @property {(attributes: Attributes, property: string) => string} transform
   * @property {string} [space]
   * @property {Attributes} [attributes]
   * @property {Array<string>} [mustUseProperty]
   */

  const own$i = {}.hasOwnProperty;

  /**
   * @param {Definition} definition
   * @returns {Schema}
   */
  function create$5(definition) {
    /** @type {Properties} */
    const property = {};
    /** @type {Normal} */
    const normal = {};
    /** @type {string} */
    let prop;

    for (prop in definition.properties) {
      if (own$i.call(definition.properties, prop)) {
        const value = definition.properties[prop];
        const info = new DefinedInfo$5(
          prop,
          definition.transform(definition.attributes || {}, prop),
          value,
          definition.space
        );

        if (
          definition.mustUseProperty &&
          definition.mustUseProperty.includes(prop)
        ) {
          info.mustUseProperty = true;
        }

        property[prop] = info;

        normal[normalize$5(prop)] = prop;
        normal[normalize$5(info.attribute)] = prop;
      }
    }

    return new Schema$5(property, normal, definition.space)
  }

  const xlink$5 = create$5({
    space: 'xlink',
    transform(_, prop) {
      return 'xlink:' + prop.slice(5).toLowerCase()
    },
    properties: {
      xLinkActuate: null,
      xLinkArcRole: null,
      xLinkHref: null,
      xLinkRole: null,
      xLinkShow: null,
      xLinkTitle: null,
      xLinkType: null
    }
  });

  const xml$5 = create$5({
    space: 'xml',
    transform(_, prop) {
      return 'xml:' + prop.slice(3).toLowerCase()
    },
    properties: {xmlLang: null, xmlBase: null, xmlSpace: null}
  });

  /**
   * @param {Record<string, string>} attributes
   * @param {string} attribute
   * @returns {string}
   */
  function caseSensitiveTransform$5(attributes, attribute) {
    return attribute in attributes ? attributes[attribute] : attribute
  }

  /**
   * @param {Record<string, string>} attributes
   * @param {string} property
   * @returns {string}
   */
  function caseInsensitiveTransform$5(attributes, property) {
    return caseSensitiveTransform$5(attributes, property.toLowerCase())
  }

  const xmlns$5 = create$5({
    space: 'xmlns',
    attributes: {xmlnsxlink: 'xmlns:xlink'},
    transform: caseInsensitiveTransform$5,
    properties: {xmlns: null, xmlnsXLink: null}
  });

  const aria$5 = create$5({
    transform(_, prop) {
      return prop === 'role' ? prop : 'aria-' + prop.slice(4).toLowerCase()
    },
    properties: {
      ariaActiveDescendant: null,
      ariaAtomic: booleanish$5,
      ariaAutoComplete: null,
      ariaBusy: booleanish$5,
      ariaChecked: booleanish$5,
      ariaColCount: number$5,
      ariaColIndex: number$5,
      ariaColSpan: number$5,
      ariaControls: spaceSeparated$5,
      ariaCurrent: null,
      ariaDescribedBy: spaceSeparated$5,
      ariaDetails: null,
      ariaDisabled: booleanish$5,
      ariaDropEffect: spaceSeparated$5,
      ariaErrorMessage: null,
      ariaExpanded: booleanish$5,
      ariaFlowTo: spaceSeparated$5,
      ariaGrabbed: booleanish$5,
      ariaHasPopup: null,
      ariaHidden: booleanish$5,
      ariaInvalid: null,
      ariaKeyShortcuts: null,
      ariaLabel: null,
      ariaLabelledBy: spaceSeparated$5,
      ariaLevel: number$5,
      ariaLive: null,
      ariaModal: booleanish$5,
      ariaMultiLine: booleanish$5,
      ariaMultiSelectable: booleanish$5,
      ariaOrientation: null,
      ariaOwns: spaceSeparated$5,
      ariaPlaceholder: null,
      ariaPosInSet: number$5,
      ariaPressed: booleanish$5,
      ariaReadOnly: booleanish$5,
      ariaRelevant: null,
      ariaRequired: booleanish$5,
      ariaRoleDescription: spaceSeparated$5,
      ariaRowCount: number$5,
      ariaRowIndex: number$5,
      ariaRowSpan: number$5,
      ariaSelected: booleanish$5,
      ariaSetSize: number$5,
      ariaSort: null,
      ariaValueMax: number$5,
      ariaValueMin: number$5,
      ariaValueNow: number$5,
      ariaValueText: null,
      role: null
    }
  });

  const html$d = create$5({
    space: 'html',
    attributes: {
      acceptcharset: 'accept-charset',
      classname: 'class',
      htmlfor: 'for',
      httpequiv: 'http-equiv'
    },
    transform: caseInsensitiveTransform$5,
    mustUseProperty: ['checked', 'multiple', 'muted', 'selected'],
    properties: {
      // Standard Properties.
      abbr: null,
      accept: commaSeparated$5,
      acceptCharset: spaceSeparated$5,
      accessKey: spaceSeparated$5,
      action: null,
      allow: null,
      allowFullScreen: boolean$5,
      allowPaymentRequest: boolean$5,
      allowUserMedia: boolean$5,
      alt: null,
      as: null,
      async: boolean$5,
      autoCapitalize: null,
      autoComplete: spaceSeparated$5,
      autoFocus: boolean$5,
      autoPlay: boolean$5,
      capture: boolean$5,
      charSet: null,
      checked: boolean$5,
      cite: null,
      className: spaceSeparated$5,
      cols: number$5,
      colSpan: null,
      content: null,
      contentEditable: booleanish$5,
      controls: boolean$5,
      controlsList: spaceSeparated$5,
      coords: number$5 | commaSeparated$5,
      crossOrigin: null,
      data: null,
      dateTime: null,
      decoding: null,
      default: boolean$5,
      defer: boolean$5,
      dir: null,
      dirName: null,
      disabled: boolean$5,
      download: overloadedBoolean$5,
      draggable: booleanish$5,
      encType: null,
      enterKeyHint: null,
      form: null,
      formAction: null,
      formEncType: null,
      formMethod: null,
      formNoValidate: boolean$5,
      formTarget: null,
      headers: spaceSeparated$5,
      height: number$5,
      hidden: boolean$5,
      high: number$5,
      href: null,
      hrefLang: null,
      htmlFor: spaceSeparated$5,
      httpEquiv: spaceSeparated$5,
      id: null,
      imageSizes: null,
      imageSrcSet: null,
      inputMode: null,
      integrity: null,
      is: null,
      isMap: boolean$5,
      itemId: null,
      itemProp: spaceSeparated$5,
      itemRef: spaceSeparated$5,
      itemScope: boolean$5,
      itemType: spaceSeparated$5,
      kind: null,
      label: null,
      lang: null,
      language: null,
      list: null,
      loading: null,
      loop: boolean$5,
      low: number$5,
      manifest: null,
      max: null,
      maxLength: number$5,
      media: null,
      method: null,
      min: null,
      minLength: number$5,
      multiple: boolean$5,
      muted: boolean$5,
      name: null,
      nonce: null,
      noModule: boolean$5,
      noValidate: boolean$5,
      onAbort: null,
      onAfterPrint: null,
      onAuxClick: null,
      onBeforePrint: null,
      onBeforeUnload: null,
      onBlur: null,
      onCancel: null,
      onCanPlay: null,
      onCanPlayThrough: null,
      onChange: null,
      onClick: null,
      onClose: null,
      onContextLost: null,
      onContextMenu: null,
      onContextRestored: null,
      onCopy: null,
      onCueChange: null,
      onCut: null,
      onDblClick: null,
      onDrag: null,
      onDragEnd: null,
      onDragEnter: null,
      onDragExit: null,
      onDragLeave: null,
      onDragOver: null,
      onDragStart: null,
      onDrop: null,
      onDurationChange: null,
      onEmptied: null,
      onEnded: null,
      onError: null,
      onFocus: null,
      onFormData: null,
      onHashChange: null,
      onInput: null,
      onInvalid: null,
      onKeyDown: null,
      onKeyPress: null,
      onKeyUp: null,
      onLanguageChange: null,
      onLoad: null,
      onLoadedData: null,
      onLoadedMetadata: null,
      onLoadEnd: null,
      onLoadStart: null,
      onMessage: null,
      onMessageError: null,
      onMouseDown: null,
      onMouseEnter: null,
      onMouseLeave: null,
      onMouseMove: null,
      onMouseOut: null,
      onMouseOver: null,
      onMouseUp: null,
      onOffline: null,
      onOnline: null,
      onPageHide: null,
      onPageShow: null,
      onPaste: null,
      onPause: null,
      onPlay: null,
      onPlaying: null,
      onPopState: null,
      onProgress: null,
      onRateChange: null,
      onRejectionHandled: null,
      onReset: null,
      onResize: null,
      onScroll: null,
      onSecurityPolicyViolation: null,
      onSeeked: null,
      onSeeking: null,
      onSelect: null,
      onSlotChange: null,
      onStalled: null,
      onStorage: null,
      onSubmit: null,
      onSuspend: null,
      onTimeUpdate: null,
      onToggle: null,
      onUnhandledRejection: null,
      onUnload: null,
      onVolumeChange: null,
      onWaiting: null,
      onWheel: null,
      open: boolean$5,
      optimum: number$5,
      pattern: null,
      ping: spaceSeparated$5,
      placeholder: null,
      playsInline: boolean$5,
      poster: null,
      preload: null,
      readOnly: boolean$5,
      referrerPolicy: null,
      rel: spaceSeparated$5,
      required: boolean$5,
      reversed: boolean$5,
      rows: number$5,
      rowSpan: number$5,
      sandbox: spaceSeparated$5,
      scope: null,
      scoped: boolean$5,
      seamless: boolean$5,
      selected: boolean$5,
      shape: null,
      size: number$5,
      sizes: null,
      slot: null,
      span: number$5,
      spellCheck: booleanish$5,
      src: null,
      srcDoc: null,
      srcLang: null,
      srcSet: null,
      start: number$5,
      step: null,
      style: null,
      tabIndex: number$5,
      target: null,
      title: null,
      translate: null,
      type: null,
      typeMustMatch: boolean$5,
      useMap: null,
      value: booleanish$5,
      width: number$5,
      wrap: null,

      // Legacy.
      // See: https://html.spec.whatwg.org/#other-elements,-attributes-and-apis
      align: null, // Several. Use CSS `text-align` instead,
      aLink: null, // `<body>`. Use CSS `a:active {color}` instead
      archive: spaceSeparated$5, // `<object>`. List of URIs to archives
      axis: null, // `<td>` and `<th>`. Use `scope` on `<th>`
      background: null, // `<body>`. Use CSS `background-image` instead
      bgColor: null, // `<body>` and table elements. Use CSS `background-color` instead
      border: number$5, // `<table>`. Use CSS `border-width` instead,
      borderColor: null, // `<table>`. Use CSS `border-color` instead,
      bottomMargin: number$5, // `<body>`
      cellPadding: null, // `<table>`
      cellSpacing: null, // `<table>`
      char: null, // Several table elements. When `align=char`, sets the character to align on
      charOff: null, // Several table elements. When `char`, offsets the alignment
      classId: null, // `<object>`
      clear: null, // `<br>`. Use CSS `clear` instead
      code: null, // `<object>`
      codeBase: null, // `<object>`
      codeType: null, // `<object>`
      color: null, // `<font>` and `<hr>`. Use CSS instead
      compact: boolean$5, // Lists. Use CSS to reduce space between items instead
      declare: boolean$5, // `<object>`
      event: null, // `<script>`
      face: null, // `<font>`. Use CSS instead
      frame: null, // `<table>`
      frameBorder: null, // `<iframe>`. Use CSS `border` instead
      hSpace: number$5, // `<img>` and `<object>`
      leftMargin: number$5, // `<body>`
      link: null, // `<body>`. Use CSS `a:link {color: *}` instead
      longDesc: null, // `<frame>`, `<iframe>`, and `<img>`. Use an `<a>`
      lowSrc: null, // `<img>`. Use a `<picture>`
      marginHeight: number$5, // `<body>`
      marginWidth: number$5, // `<body>`
      noResize: boolean$5, // `<frame>`
      noHref: boolean$5, // `<area>`. Use no href instead of an explicit `nohref`
      noShade: boolean$5, // `<hr>`. Use background-color and height instead of borders
      noWrap: boolean$5, // `<td>` and `<th>`
      object: null, // `<applet>`
      profile: null, // `<head>`
      prompt: null, // `<isindex>`
      rev: null, // `<link>`
      rightMargin: number$5, // `<body>`
      rules: null, // `<table>`
      scheme: null, // `<meta>`
      scrolling: booleanish$5, // `<frame>`. Use overflow in the child context
      standby: null, // `<object>`
      summary: null, // `<table>`
      text: null, // `<body>`. Use CSS `color` instead
      topMargin: number$5, // `<body>`
      valueType: null, // `<param>`
      version: null, // `<html>`. Use a doctype.
      vAlign: null, // Several. Use CSS `vertical-align` instead
      vLink: null, // `<body>`. Use CSS `a:visited {color}` instead
      vSpace: number$5, // `<img>` and `<object>`

      // Non-standard Properties.
      allowTransparency: null,
      autoCorrect: null,
      autoSave: null,
      disablePictureInPicture: boolean$5,
      disableRemotePlayback: boolean$5,
      prefix: null,
      property: null,
      results: number$5,
      security: null,
      unselectable: null
    }
  });

  const svg$a = create$5({
    space: 'svg',
    attributes: {
      accentHeight: 'accent-height',
      alignmentBaseline: 'alignment-baseline',
      arabicForm: 'arabic-form',
      baselineShift: 'baseline-shift',
      capHeight: 'cap-height',
      className: 'class',
      clipPath: 'clip-path',
      clipRule: 'clip-rule',
      colorInterpolation: 'color-interpolation',
      colorInterpolationFilters: 'color-interpolation-filters',
      colorProfile: 'color-profile',
      colorRendering: 'color-rendering',
      crossOrigin: 'crossorigin',
      dataType: 'datatype',
      dominantBaseline: 'dominant-baseline',
      enableBackground: 'enable-background',
      fillOpacity: 'fill-opacity',
      fillRule: 'fill-rule',
      floodColor: 'flood-color',
      floodOpacity: 'flood-opacity',
      fontFamily: 'font-family',
      fontSize: 'font-size',
      fontSizeAdjust: 'font-size-adjust',
      fontStretch: 'font-stretch',
      fontStyle: 'font-style',
      fontVariant: 'font-variant',
      fontWeight: 'font-weight',
      glyphName: 'glyph-name',
      glyphOrientationHorizontal: 'glyph-orientation-horizontal',
      glyphOrientationVertical: 'glyph-orientation-vertical',
      hrefLang: 'hreflang',
      horizAdvX: 'horiz-adv-x',
      horizOriginX: 'horiz-origin-x',
      horizOriginY: 'horiz-origin-y',
      imageRendering: 'image-rendering',
      letterSpacing: 'letter-spacing',
      lightingColor: 'lighting-color',
      markerEnd: 'marker-end',
      markerMid: 'marker-mid',
      markerStart: 'marker-start',
      navDown: 'nav-down',
      navDownLeft: 'nav-down-left',
      navDownRight: 'nav-down-right',
      navLeft: 'nav-left',
      navNext: 'nav-next',
      navPrev: 'nav-prev',
      navRight: 'nav-right',
      navUp: 'nav-up',
      navUpLeft: 'nav-up-left',
      navUpRight: 'nav-up-right',
      onAbort: 'onabort',
      onActivate: 'onactivate',
      onAfterPrint: 'onafterprint',
      onBeforePrint: 'onbeforeprint',
      onBegin: 'onbegin',
      onCancel: 'oncancel',
      onCanPlay: 'oncanplay',
      onCanPlayThrough: 'oncanplaythrough',
      onChange: 'onchange',
      onClick: 'onclick',
      onClose: 'onclose',
      onCopy: 'oncopy',
      onCueChange: 'oncuechange',
      onCut: 'oncut',
      onDblClick: 'ondblclick',
      onDrag: 'ondrag',
      onDragEnd: 'ondragend',
      onDragEnter: 'ondragenter',
      onDragExit: 'ondragexit',
      onDragLeave: 'ondragleave',
      onDragOver: 'ondragover',
      onDragStart: 'ondragstart',
      onDrop: 'ondrop',
      onDurationChange: 'ondurationchange',
      onEmptied: 'onemptied',
      onEnd: 'onend',
      onEnded: 'onended',
      onError: 'onerror',
      onFocus: 'onfocus',
      onFocusIn: 'onfocusin',
      onFocusOut: 'onfocusout',
      onHashChange: 'onhashchange',
      onInput: 'oninput',
      onInvalid: 'oninvalid',
      onKeyDown: 'onkeydown',
      onKeyPress: 'onkeypress',
      onKeyUp: 'onkeyup',
      onLoad: 'onload',
      onLoadedData: 'onloadeddata',
      onLoadedMetadata: 'onloadedmetadata',
      onLoadStart: 'onloadstart',
      onMessage: 'onmessage',
      onMouseDown: 'onmousedown',
      onMouseEnter: 'onmouseenter',
      onMouseLeave: 'onmouseleave',
      onMouseMove: 'onmousemove',
      onMouseOut: 'onmouseout',
      onMouseOver: 'onmouseover',
      onMouseUp: 'onmouseup',
      onMouseWheel: 'onmousewheel',
      onOffline: 'onoffline',
      onOnline: 'ononline',
      onPageHide: 'onpagehide',
      onPageShow: 'onpageshow',
      onPaste: 'onpaste',
      onPause: 'onpause',
      onPlay: 'onplay',
      onPlaying: 'onplaying',
      onPopState: 'onpopstate',
      onProgress: 'onprogress',
      onRateChange: 'onratechange',
      onRepeat: 'onrepeat',
      onReset: 'onreset',
      onResize: 'onresize',
      onScroll: 'onscroll',
      onSeeked: 'onseeked',
      onSeeking: 'onseeking',
      onSelect: 'onselect',
      onShow: 'onshow',
      onStalled: 'onstalled',
      onStorage: 'onstorage',
      onSubmit: 'onsubmit',
      onSuspend: 'onsuspend',
      onTimeUpdate: 'ontimeupdate',
      onToggle: 'ontoggle',
      onUnload: 'onunload',
      onVolumeChange: 'onvolumechange',
      onWaiting: 'onwaiting',
      onZoom: 'onzoom',
      overlinePosition: 'overline-position',
      overlineThickness: 'overline-thickness',
      paintOrder: 'paint-order',
      panose1: 'panose-1',
      pointerEvents: 'pointer-events',
      referrerPolicy: 'referrerpolicy',
      renderingIntent: 'rendering-intent',
      shapeRendering: 'shape-rendering',
      stopColor: 'stop-color',
      stopOpacity: 'stop-opacity',
      strikethroughPosition: 'strikethrough-position',
      strikethroughThickness: 'strikethrough-thickness',
      strokeDashArray: 'stroke-dasharray',
      strokeDashOffset: 'stroke-dashoffset',
      strokeLineCap: 'stroke-linecap',
      strokeLineJoin: 'stroke-linejoin',
      strokeMiterLimit: 'stroke-miterlimit',
      strokeOpacity: 'stroke-opacity',
      strokeWidth: 'stroke-width',
      tabIndex: 'tabindex',
      textAnchor: 'text-anchor',
      textDecoration: 'text-decoration',
      textRendering: 'text-rendering',
      typeOf: 'typeof',
      underlinePosition: 'underline-position',
      underlineThickness: 'underline-thickness',
      unicodeBidi: 'unicode-bidi',
      unicodeRange: 'unicode-range',
      unitsPerEm: 'units-per-em',
      vAlphabetic: 'v-alphabetic',
      vHanging: 'v-hanging',
      vIdeographic: 'v-ideographic',
      vMathematical: 'v-mathematical',
      vectorEffect: 'vector-effect',
      vertAdvY: 'vert-adv-y',
      vertOriginX: 'vert-origin-x',
      vertOriginY: 'vert-origin-y',
      wordSpacing: 'word-spacing',
      writingMode: 'writing-mode',
      xHeight: 'x-height',
      // These were camelcased in Tiny. Now lowercased in SVG 2
      playbackOrder: 'playbackorder',
      timelineBegin: 'timelinebegin'
    },
    transform: caseSensitiveTransform$5,
    properties: {
      about: commaOrSpaceSeparated$5,
      accentHeight: number$5,
      accumulate: null,
      additive: null,
      alignmentBaseline: null,
      alphabetic: number$5,
      amplitude: number$5,
      arabicForm: null,
      ascent: number$5,
      attributeName: null,
      attributeType: null,
      azimuth: number$5,
      bandwidth: null,
      baselineShift: null,
      baseFrequency: null,
      baseProfile: null,
      bbox: null,
      begin: null,
      bias: number$5,
      by: null,
      calcMode: null,
      capHeight: number$5,
      className: spaceSeparated$5,
      clip: null,
      clipPath: null,
      clipPathUnits: null,
      clipRule: null,
      color: null,
      colorInterpolation: null,
      colorInterpolationFilters: null,
      colorProfile: null,
      colorRendering: null,
      content: null,
      contentScriptType: null,
      contentStyleType: null,
      crossOrigin: null,
      cursor: null,
      cx: null,
      cy: null,
      d: null,
      dataType: null,
      defaultAction: null,
      descent: number$5,
      diffuseConstant: number$5,
      direction: null,
      display: null,
      dur: null,
      divisor: number$5,
      dominantBaseline: null,
      download: boolean$5,
      dx: null,
      dy: null,
      edgeMode: null,
      editable: null,
      elevation: number$5,
      enableBackground: null,
      end: null,
      event: null,
      exponent: number$5,
      externalResourcesRequired: null,
      fill: null,
      fillOpacity: number$5,
      fillRule: null,
      filter: null,
      filterRes: null,
      filterUnits: null,
      floodColor: null,
      floodOpacity: null,
      focusable: null,
      focusHighlight: null,
      fontFamily: null,
      fontSize: null,
      fontSizeAdjust: null,
      fontStretch: null,
      fontStyle: null,
      fontVariant: null,
      fontWeight: null,
      format: null,
      fr: null,
      from: null,
      fx: null,
      fy: null,
      g1: commaSeparated$5,
      g2: commaSeparated$5,
      glyphName: commaSeparated$5,
      glyphOrientationHorizontal: null,
      glyphOrientationVertical: null,
      glyphRef: null,
      gradientTransform: null,
      gradientUnits: null,
      handler: null,
      hanging: number$5,
      hatchContentUnits: null,
      hatchUnits: null,
      height: null,
      href: null,
      hrefLang: null,
      horizAdvX: number$5,
      horizOriginX: number$5,
      horizOriginY: number$5,
      id: null,
      ideographic: number$5,
      imageRendering: null,
      initialVisibility: null,
      in: null,
      in2: null,
      intercept: number$5,
      k: number$5,
      k1: number$5,
      k2: number$5,
      k3: number$5,
      k4: number$5,
      kernelMatrix: commaOrSpaceSeparated$5,
      kernelUnitLength: null,
      keyPoints: null, // SEMI_COLON_SEPARATED
      keySplines: null, // SEMI_COLON_SEPARATED
      keyTimes: null, // SEMI_COLON_SEPARATED
      kerning: null,
      lang: null,
      lengthAdjust: null,
      letterSpacing: null,
      lightingColor: null,
      limitingConeAngle: number$5,
      local: null,
      markerEnd: null,
      markerMid: null,
      markerStart: null,
      markerHeight: null,
      markerUnits: null,
      markerWidth: null,
      mask: null,
      maskContentUnits: null,
      maskUnits: null,
      mathematical: null,
      max: null,
      media: null,
      mediaCharacterEncoding: null,
      mediaContentEncodings: null,
      mediaSize: number$5,
      mediaTime: null,
      method: null,
      min: null,
      mode: null,
      name: null,
      navDown: null,
      navDownLeft: null,
      navDownRight: null,
      navLeft: null,
      navNext: null,
      navPrev: null,
      navRight: null,
      navUp: null,
      navUpLeft: null,
      navUpRight: null,
      numOctaves: null,
      observer: null,
      offset: null,
      onAbort: null,
      onActivate: null,
      onAfterPrint: null,
      onBeforePrint: null,
      onBegin: null,
      onCancel: null,
      onCanPlay: null,
      onCanPlayThrough: null,
      onChange: null,
      onClick: null,
      onClose: null,
      onCopy: null,
      onCueChange: null,
      onCut: null,
      onDblClick: null,
      onDrag: null,
      onDragEnd: null,
      onDragEnter: null,
      onDragExit: null,
      onDragLeave: null,
      onDragOver: null,
      onDragStart: null,
      onDrop: null,
      onDurationChange: null,
      onEmptied: null,
      onEnd: null,
      onEnded: null,
      onError: null,
      onFocus: null,
      onFocusIn: null,
      onFocusOut: null,
      onHashChange: null,
      onInput: null,
      onInvalid: null,
      onKeyDown: null,
      onKeyPress: null,
      onKeyUp: null,
      onLoad: null,
      onLoadedData: null,
      onLoadedMetadata: null,
      onLoadStart: null,
      onMessage: null,
      onMouseDown: null,
      onMouseEnter: null,
      onMouseLeave: null,
      onMouseMove: null,
      onMouseOut: null,
      onMouseOver: null,
      onMouseUp: null,
      onMouseWheel: null,
      onOffline: null,
      onOnline: null,
      onPageHide: null,
      onPageShow: null,
      onPaste: null,
      onPause: null,
      onPlay: null,
      onPlaying: null,
      onPopState: null,
      onProgress: null,
      onRateChange: null,
      onRepeat: null,
      onReset: null,
      onResize: null,
      onScroll: null,
      onSeeked: null,
      onSeeking: null,
      onSelect: null,
      onShow: null,
      onStalled: null,
      onStorage: null,
      onSubmit: null,
      onSuspend: null,
      onTimeUpdate: null,
      onToggle: null,
      onUnload: null,
      onVolumeChange: null,
      onWaiting: null,
      onZoom: null,
      opacity: null,
      operator: null,
      order: null,
      orient: null,
      orientation: null,
      origin: null,
      overflow: null,
      overlay: null,
      overlinePosition: number$5,
      overlineThickness: number$5,
      paintOrder: null,
      panose1: null,
      path: null,
      pathLength: number$5,
      patternContentUnits: null,
      patternTransform: null,
      patternUnits: null,
      phase: null,
      ping: spaceSeparated$5,
      pitch: null,
      playbackOrder: null,
      pointerEvents: null,
      points: null,
      pointsAtX: number$5,
      pointsAtY: number$5,
      pointsAtZ: number$5,
      preserveAlpha: null,
      preserveAspectRatio: null,
      primitiveUnits: null,
      propagate: null,
      property: commaOrSpaceSeparated$5,
      r: null,
      radius: null,
      referrerPolicy: null,
      refX: null,
      refY: null,
      rel: commaOrSpaceSeparated$5,
      rev: commaOrSpaceSeparated$5,
      renderingIntent: null,
      repeatCount: null,
      repeatDur: null,
      requiredExtensions: commaOrSpaceSeparated$5,
      requiredFeatures: commaOrSpaceSeparated$5,
      requiredFonts: commaOrSpaceSeparated$5,
      requiredFormats: commaOrSpaceSeparated$5,
      resource: null,
      restart: null,
      result: null,
      rotate: null,
      rx: null,
      ry: null,
      scale: null,
      seed: null,
      shapeRendering: null,
      side: null,
      slope: null,
      snapshotTime: null,
      specularConstant: number$5,
      specularExponent: number$5,
      spreadMethod: null,
      spacing: null,
      startOffset: null,
      stdDeviation: null,
      stemh: null,
      stemv: null,
      stitchTiles: null,
      stopColor: null,
      stopOpacity: null,
      strikethroughPosition: number$5,
      strikethroughThickness: number$5,
      string: null,
      stroke: null,
      strokeDashArray: commaOrSpaceSeparated$5,
      strokeDashOffset: null,
      strokeLineCap: null,
      strokeLineJoin: null,
      strokeMiterLimit: number$5,
      strokeOpacity: number$5,
      strokeWidth: null,
      style: null,
      surfaceScale: number$5,
      syncBehavior: null,
      syncBehaviorDefault: null,
      syncMaster: null,
      syncTolerance: null,
      syncToleranceDefault: null,
      systemLanguage: commaOrSpaceSeparated$5,
      tabIndex: number$5,
      tableValues: null,
      target: null,
      targetX: number$5,
      targetY: number$5,
      textAnchor: null,
      textDecoration: null,
      textRendering: null,
      textLength: null,
      timelineBegin: null,
      title: null,
      transformBehavior: null,
      type: null,
      typeOf: commaOrSpaceSeparated$5,
      to: null,
      transform: null,
      u1: null,
      u2: null,
      underlinePosition: number$5,
      underlineThickness: number$5,
      unicode: null,
      unicodeBidi: null,
      unicodeRange: null,
      unitsPerEm: number$5,
      values: null,
      vAlphabetic: number$5,
      vMathematical: number$5,
      vectorEffect: null,
      vHanging: number$5,
      vIdeographic: number$5,
      version: null,
      vertAdvY: number$5,
      vertOriginX: number$5,
      vertOriginY: number$5,
      viewBox: null,
      viewTarget: null,
      visibility: null,
      width: null,
      widths: null,
      wordSpacing: null,
      writingMode: null,
      x: null,
      x1: null,
      x2: null,
      xChannelSelector: null,
      xHeight: number$5,
      y: null,
      y1: null,
      y2: null,
      yChannelSelector: null,
      z: null,
      zoomAndPan: null
    }
  });

  /**
   * @typedef {import('./util/schema.js').Schema} Schema
   */

  const valid$5 = /^data[-\w.:]+$/i;
  const dash$5 = /-[a-z]/g;
  const cap$5 = /[A-Z]/g;

  /**
   * @param {Schema} schema
   * @param {string} value
   * @returns {Info}
   */
  function find$5(schema, value) {
    const normal = normalize$5(value);
    let prop = value;
    let Type = Info$5;

    if (normal in schema.normal) {
      return schema.property[schema.normal[normal]]
    }

    if (normal.length > 4 && normal.slice(0, 4) === 'data' && valid$5.test(value)) {
      // Attribute or property.
      if (value.charAt(4) === '-') {
        // Turn it into a property.
        const rest = value.slice(5).replace(dash$5, camelcase$5);
        prop = 'data' + rest.charAt(0).toUpperCase() + rest.slice(1);
      } else {
        // Turn it into an attribute.
        const rest = value.slice(4);

        if (!dash$5.test(rest)) {
          let dashes = rest.replace(cap$5, kebab$5);

          if (dashes.charAt(0) !== '-') {
            dashes = '-' + dashes;
          }

          value = 'data' + dashes;
        }
      }

      Type = DefinedInfo$5;
    }

    return new Type(prop, value)
  }

  /**
   * @param {string} $0
   * @returns {string}
   */
  function kebab$5($0) {
    return '-' + $0.toLowerCase()
  }

  /**
   * @param {string} $0
   * @returns {string}
   */
  function camelcase$5($0) {
    return $0.charAt(1).toUpperCase()
  }

  /**
   * @typedef {import('./lib/util/info.js').Info} Info
   * @typedef {import('./lib/util/schema.js').Schema} Schema
   */
  const html$c = merge$5([xml$5, xlink$5, xmlns$5, aria$5, html$d], 'html');
  const svg$9 = merge$5([xml$5, xlink$5, xmlns$5, aria$5, svg$a], 'svg');

  /**
   * @typedef {import('hast').Properties} Properties
   * @typedef {import('hast').Element} Element
   */

  var search$1 = /[#.]/g;

  /**
   * Create a hast element from a simple CSS selector.
   *
   * @param selector A simple CSS selector.
   *   Can contain a tag-name (`foo`), classes (`.bar`), and an ID (`#baz`).
   *   Multiple classes are allowed.
   *   Uses the last ID if multiple IDs are found.
   * @param [defaultTagName='div'] Tag name to use if `selector` does not specify one.
   */
  const parseSelector$1 =
    /**
     * @type {(
     *  <Selector extends string, DefaultTagName extends string = 'div'>(selector?: Selector, defaultTagName?: DefaultTagName) => Element & {tagName: import('./extract.js').ExtractTagName<Selector, DefaultTagName>}
     * )}
     */
    (
      /**
       * @param {string} [selector]
       * @param {string} [defaultTagName='div']
       * @returns {Element}
       */
      function (selector, defaultTagName = 'div') {
        var value = selector || '';
        /** @type {Properties} */
        var props = {};
        var start = 0;
        /** @type {string} */
        var subvalue;
        /** @type {string} */
        var previous;
        /** @type {RegExpMatchArray} */
        var match;

        while (start < value.length) {
          search$1.lastIndex = start;
          match = search$1.exec(value);
          subvalue = value.slice(start, match ? match.index : value.length);

          if (subvalue) {
            if (!previous) {
              defaultTagName = subvalue;
            } else if (previous === '#') {
              props.id = subvalue;
            } else if (Array.isArray(props.className)) {
              props.className.push(subvalue);
            } else {
              props.className = [subvalue];
            }

            start += subvalue.length;
          }

          if (match) {
            previous = match[0];
            start++;
          }
        }

        return {
          type: 'element',
          tagName: defaultTagName,
          properties: props,
          children: []
        }
      }
    );

  /**
   * Parse space separated tokens to an array of strings.
   *
   * @param {string} value Space separated tokens
   * @returns {Array.<string>} Tokens
   */
  function parse$8(value) {
    const input = String(value || '').trim();
    return input ? input.split(/[ \t\n\r\f]+/g) : []
  }

  /**
   * @typedef {Object} StringifyOptions
   * @property {boolean} [padLeft=true] Whether to pad a space before a token (`boolean`, default: `true`).
   * @property {boolean} [padRight=false] Whether to pad a space after a token (`boolean`, default: `false`).
   */

  /**
   * Parse comma separated tokens to an array.
   *
   * @param {string} value
   * @returns {Array.<string>}
   */
  function parse$7(value) {
    /** @type {Array.<string>} */
    var tokens = [];
    var input = String(value || '');
    var index = input.indexOf(',');
    var start = 0;
    /** @type {boolean} */
    var end;
    /** @type {string} */
    var token;

    while (!end) {
      if (index === -1) {
        index = input.length;
        end = true;
      }

      token = input.slice(start, index).trim();

      if (token || !end) {
        tokens.push(token);
      }

      start = index + 1;
      index = input.indexOf(',', start);
    }

    return tokens
  }

  /**
   * @typedef {import('hast').Root} Root
   * @typedef {import('hast').Element} Element
   * @typedef {import('hast').Properties} Properties
   * @typedef {Root['children'][number]} Child
   * @typedef {Child|Root} Node
   * @typedef {import('property-information').Info} Info
   * @typedef {import('property-information').Schema} Schema
   *
   * @typedef {Root|Element} HResult
   * @typedef {string|number} HStyleValue
   * @typedef {Object.<string, HStyleValue>} HStyle
   * @typedef {string|number|boolean|null|undefined} HPrimitiveValue
   * @typedef {Array.<string|number>} HArrayValue
   * @typedef {HPrimitiveValue|HArrayValue} HPropertyValue
   * @typedef {{[property: string]: HPropertyValue|HStyle}} HProperties
   *
   * @typedef {string|number|null|undefined} HPrimitiveChild
   * @typedef {Array.<Node|HPrimitiveChild>} HArrayChild
   * @typedef {Node|HPrimitiveChild|HArrayChild} HChild
   */

  const buttonTypes$1 = new Set(['menu', 'submit', 'reset', 'button']);

  const own$h = {}.hasOwnProperty;

  /**
   * @param {Schema} schema
   * @param {string} defaultTagName
   * @param {Array.<string>} [caseSensitive]
   */
  function core$2(schema, defaultTagName, caseSensitive) {
    const adjust = caseSensitive && createAdjustMap$1(caseSensitive);

    const h =
      /**
       * @type {{
       *   (): Root
       *   (selector: null|undefined, ...children: HChild[]): Root
       *   (selector: string, properties?: HProperties, ...children: HChild[]): Element
       *   (selector: string, ...children: HChild[]): Element
       * }}
       */
      (
        /**
         * Hyperscript compatible DSL for creating virtual hast trees.
         *
         * @param {string|null} [selector]
         * @param {HProperties|HChild} [properties]
         * @param {HChild[]} children
         * @returns {HResult}
         */
        function (selector, properties, ...children) {
          let index = -1;
          /** @type {HResult} */
          let node;

          if (selector === undefined || selector === null) {
            node = {type: 'root', children: []};
            // @ts-expect-error Properties are not supported for roots.
            children.unshift(properties);
          } else {
            node = parseSelector$1(selector, defaultTagName);
            // Normalize the name.
            node.tagName = node.tagName.toLowerCase();
            if (adjust && own$h.call(adjust, node.tagName)) {
              node.tagName = adjust[node.tagName];
            }

            // Handle props.
            if (isProperties$1(properties, node.tagName)) {
              /** @type {string} */
              let key;

              for (key in properties) {
                if (own$h.call(properties, key)) {
                  // @ts-expect-error `node.properties` is set.
                  addProperty$1(schema, node.properties, key, properties[key]);
                }
              }
            } else {
              children.unshift(properties);
            }
          }

          // Handle children.
          while (++index < children.length) {
            addChild$1(node.children, children[index]);
          }

          if (node.type === 'element' && node.tagName === 'template') {
            node.content = {type: 'root', children: node.children};
            node.children = [];
          }

          return node
        }
      );

    return h
  }

  /**
   * @param {HProperties|HChild} value
   * @param {string} name
   * @returns {value is HProperties}
   */
  function isProperties$1(value, name) {
    if (
      value === null ||
      value === undefined ||
      typeof value !== 'object' ||
      Array.isArray(value)
    ) {
      return false
    }

    if (name === 'input' || !value.type || typeof value.type !== 'string') {
      return true
    }

    if ('children' in value && Array.isArray(value.children)) {
      return false
    }

    if (name === 'button') {
      return buttonTypes$1.has(value.type.toLowerCase())
    }

    return !('value' in value)
  }

  /**
   * @param {Schema} schema
   * @param {Properties} properties
   * @param {string} key
   * @param {HStyle|HPropertyValue} value
   * @returns {void}
   */
  function addProperty$1(schema, properties, key, value) {
    const info = find$5(schema, key);
    let index = -1;
    /** @type {HPropertyValue} */
    let result;

    // Ignore nullish and NaN values.
    if (value === undefined || value === null) return

    if (typeof value === 'number') {
      // Ignore NaN.
      if (Number.isNaN(value)) return

      result = value;
    }
    // Booleans.
    else if (typeof value === 'boolean') {
      result = value;
    }
    // Handle list values.
    else if (typeof value === 'string') {
      if (info.spaceSeparated) {
        result = parse$8(value);
      } else if (info.commaSeparated) {
        result = parse$7(value);
      } else if (info.commaOrSpaceSeparated) {
        result = parse$8(parse$7(value).join(' '));
      } else {
        result = parsePrimitive$1(info, info.property, value);
      }
    } else if (Array.isArray(value)) {
      result = value.concat();
    } else {
      result = info.property === 'style' ? style$1(value) : String(value);
    }

    if (Array.isArray(result)) {
      /** @type {Array.<string|number>} */
      const finalResult = [];

      while (++index < result.length) {
        // @ts-expect-error Assume no booleans in array.
        finalResult[index] = parsePrimitive$1(info, info.property, result[index]);
      }

      result = finalResult;
    }

    // Class names (which can be added both on the `selector` and here).
    if (info.property === 'className' && Array.isArray(properties.className)) {
      // @ts-expect-error Assume no booleans in `className`.
      result = properties.className.concat(result);
    }

    properties[info.property] = result;
  }

  /**
   * @param {Array.<Child>} nodes
   * @param {HChild} value
   * @returns {void}
   */
  function addChild$1(nodes, value) {
    let index = -1;

    if (value === undefined || value === null) ; else if (typeof value === 'string' || typeof value === 'number') {
      nodes.push({type: 'text', value: String(value)});
    } else if (Array.isArray(value)) {
      while (++index < value.length) {
        addChild$1(nodes, value[index]);
      }
    } else if (typeof value === 'object' && 'type' in value) {
      if (value.type === 'root') {
        addChild$1(nodes, value.children);
      } else {
        nodes.push(value);
      }
    } else {
      throw new Error('Expected node, nodes, or string, got `' + value + '`')
    }
  }

  /**
   * Parse a single primitives.
   *
   * @param {Info} info
   * @param {string} name
   * @param {HPrimitiveValue} value
   * @returns {HPrimitiveValue}
   */
  function parsePrimitive$1(info, name, value) {
    if (typeof value === 'string') {
      if (info.number && value && !Number.isNaN(Number(value))) {
        return Number(value)
      }

      if (
        (info.boolean || info.overloadedBoolean) &&
        (value === '' || normalize$5(value) === normalize$5(name))
      ) {
        return true
      }
    }

    return value
  }

  /**
   * @param {HStyle} value
   * @returns {string}
   */
  function style$1(value) {
    /** @type {Array.<string>} */
    const result = [];
    /** @type {string} */
    let key;

    for (key in value) {
      if (own$h.call(value, key)) {
        result.push([key, value[key]].join(': '));
      }
    }

    return result.join('; ')
  }

  /**
   * @param {Array.<string>} values
   * @returns {Object.<string, string>}
   */
  function createAdjustMap$1(values) {
    /** @type {Object.<string, string>} */
    const result = {};
    let index = -1;

    while (++index < values.length) {
      result[values[index].toLowerCase()] = values[index];
    }

    return result
  }

  /**
   * @typedef {import('./core.js').HChild} Child Acceptable child value
   * @typedef {import('./core.js').HProperties} Properties Acceptable properties value.
   *
   * @typedef {import('./jsx-classic').Element} h.JSX.Element
   * @typedef {import('./jsx-classic').IntrinsicAttributes} h.JSX.IntrinsicAttributes
   * @typedef {import('./jsx-classic').IntrinsicElements} h.JSX.IntrinsicElements
   * @typedef {import('./jsx-classic').ElementChildrenAttribute} h.JSX.ElementChildrenAttribute
   */

  const h$1 = core$2(html$c, 'div');

  const svgCaseSensitiveTagNames = [
    'altGlyph',
    'altGlyphDef',
    'altGlyphItem',
    'animateColor',
    'animateMotion',
    'animateTransform',
    'clipPath',
    'feBlend',
    'feColorMatrix',
    'feComponentTransfer',
    'feComposite',
    'feConvolveMatrix',
    'feDiffuseLighting',
    'feDisplacementMap',
    'feDistantLight',
    'feDropShadow',
    'feFlood',
    'feFuncA',
    'feFuncB',
    'feFuncG',
    'feFuncR',
    'feGaussianBlur',
    'feImage',
    'feMerge',
    'feMergeNode',
    'feMorphology',
    'feOffset',
    'fePointLight',
    'feSpecularLighting',
    'feSpotLight',
    'feTile',
    'feTurbulence',
    'foreignObject',
    'glyphRef',
    'linearGradient',
    'radialGradient',
    'solidColor',
    'textArea',
    'textPath'
  ];

  /**
   * @typedef {import('./core.js').HChild} Child Acceptable child value
   * @typedef {import('./core.js').HProperties} Properties Acceptable properties value.
   *
   * @typedef {import('./jsx-classic').Element} s.JSX.Element
   * @typedef {import('./jsx-classic').IntrinsicAttributes} s.JSX.IntrinsicAttributes
   * @typedef {import('./jsx-classic').IntrinsicElements} s.JSX.IntrinsicElements
   * @typedef {import('./jsx-classic').ElementChildrenAttribute} s.JSX.ElementChildrenAttribute
   */

  const s$1 = core$2(svg$9, 'g', svgCaseSensitiveTagNames);

  /**
   * @typedef {import('unist').Point} Point
   * @typedef {import('vfile').VFile} VFile
   *
   * @typedef {Pick<Point, 'line'|'column'>} PositionalPoint
   * @typedef {Required<Point>} FullPoint
   * @typedef {NonNullable<Point['offset']>} Offset
   */

  /**
   * Get transform functions for the given `document`.
   *
   * @param {string|Uint8Array|VFile} file
   */
  function location(file) {
    var value = String(file);
    /** @type {Array.<number>} */
    var indices = [];
    var search = /\r?\n|\r/g;

    while (search.test(value)) {
      indices.push(search.lastIndex);
    }

    indices.push(value.length + 1);

    return {toPoint, toOffset}

    /**
     * Get the line and column-based `point` for `offset` in the bound indices.
     * Returns a point with `undefined` values when given invalid or out of bounds
     * input.
     *
     * @param {Offset} offset
     * @returns {FullPoint}
     */
    function toPoint(offset) {
      var index = -1;

      if (offset > -1 && offset < indices[indices.length - 1]) {
        while (++index < indices.length) {
          if (indices[index] > offset) {
            return {
              line: index + 1,
              column: offset - (indices[index - 1] || 0) + 1,
              offset
            }
          }
        }
      }

      return {line: undefined, column: undefined, offset: undefined}
    }

    /**
     * Get the `offset` for a line and column-based `point` in the bound indices.
     * Returns `-1` when given invalid or out of bounds input.
     *
     * @param {PositionalPoint} point
     * @returns {Offset}
     */
    function toOffset(point) {
      var line = point && point.line;
      var column = point && point.column;
      /** @type {number} */
      var offset;

      if (
        typeof line === 'number' &&
        typeof column === 'number' &&
        !Number.isNaN(line) &&
        !Number.isNaN(column) &&
        line - 1 in indices
      ) {
        offset = (indices[line - 2] || 0) + column - 1 || 0;
      }

      return offset > -1 && offset < indices[indices.length - 1] ? offset : -1
    }
  }

  /**
   * Map of web namespaces.
   *
   * @type {Record<string, string>}
   */
  const webNamespaces$2 = {
    html: 'http://www.w3.org/1999/xhtml',
    mathml: 'http://www.w3.org/1998/Math/MathML',
    svg: 'http://www.w3.org/2000/svg',
    xlink: 'http://www.w3.org/1999/xlink',
    xml: 'http://www.w3.org/XML/1998/namespace',
    xmlns: 'http://www.w3.org/2000/xmlns/'
  };

  /**
   * @typedef {import('vfile').VFile} VFile
   * @typedef {import('property-information').Schema} Schema
   * @typedef {import('unist').Position} Position
   * @typedef {import('unist').Point} Point
   * @typedef {import('hast').Parent} Parent
   * @typedef {import('hast').Element} Element
   * @typedef {import('hast').Root} Root
   * @typedef {import('hast').Text} Text
   * @typedef {import('hast').Comment} Comment
   * @typedef {import('hast').DocType} Doctype
   * @typedef {Parent['children'][number]} Child
   * @typedef {Element['children'][number]} ElementChild
   * @typedef {Child|Root} Node
   * @typedef {import('parse5').Document} P5Document
   * @typedef {import('parse5').DocumentType} P5Doctype
   * @typedef {import('parse5').CommentNode} P5Comment
   * @typedef {import('parse5').TextNode} P5Text
   * @typedef {import('parse5').Element} P5Element
   * @typedef {import('parse5').ElementLocation} P5ElementLocation
   * @typedef {import('parse5').Location} P5Location
   * @typedef {import('parse5').Attribute} P5Attribute
   * @typedef {import('parse5').Node} P5Node
   *
   * @typedef {'html'|'svg'} Space
   *
   * @callback Handler
   * @param {Context} ctx
   * @param {P5Node} node
   * @param {Array.<Child>} [children]
   * @returns {Node}
   *
   * @typedef Options
   * @property {Space} [space='html'] Whether the root of the tree is in the `'html'` or `'svg'` space. If an element in with the SVG namespace is found in `ast`, `fromParse5` automatically switches to the SVG space when entering the element, and switches back when leaving
   * @property {VFile} [file] `VFile`, used to add positional information to nodes. If given, the file should have the original HTML source as its contents
   * @property {boolean} [verbose=false] Whether to add extra positional information about starting tags, closing tags, and attributes to elements. Note: not used without `file`
   *
   * @typedef Context
   * @property {Schema} schema
   * @property {VFile|undefined} file
   * @property {boolean|undefined} verbose
   * @property {boolean} location
   */

  const own$g = {}.hasOwnProperty;

  // Handlers.
  const map = {
    '#document': root$3,
    '#document-fragment': root$3,
    '#text': text$3,
    '#comment': comment$3,
    '#documentType': doctype$2
  };

  /**
   * Transform Parse5’s AST to a hast tree.
   *
   * @param {P5Node} ast
   * @param {Options|VFile} [options]
   */
  function fromParse5(ast, options = {}) {
    /** @type {Options} */
    let settings;
    /** @type {VFile|undefined} */
    let file;

    if (isFile(options)) {
      file = options;
      settings = {};
    } else {
      file = options.file;
      settings = options;
    }

    return transform$1(
      {
        schema: settings.space === 'svg' ? svg$9 : html$c,
        file,
        verbose: settings.verbose,
        location: false
      },
      ast
    )
  }

  /**
   * Transform children.
   *
   * @param {Context} ctx
   * @param {P5Node} ast
   * @returns {Node}
   */
  function transform$1(ctx, ast) {
    const schema = ctx.schema;
    /** @type {Handler} */
    // @ts-expect-error: index is fine.
    const fn = own$g.call(map, ast.nodeName) ? map[ast.nodeName] : element$6;
    /** @type {Array.<Child>|undefined} */
    let children;

    // Element.
    if ('tagName' in ast) {
      ctx.schema = ast.namespaceURI === webNamespaces$2.svg ? svg$9 : html$c;
    }

    if ('childNodes' in ast) {
      children = nodes(ctx, ast.childNodes);
    }

    const result = fn(ctx, ast, children);

    if ('sourceCodeLocation' in ast && ast.sourceCodeLocation && ctx.file) {
      // @ts-expect-error It’s fine.
      const position = createLocation(ctx, result, ast.sourceCodeLocation);

      if (position) {
        ctx.location = true;
        result.position = position;
      }
    }

    ctx.schema = schema;

    return result
  }

  /**
   * Transform children.
   *
   * @param {Context} ctx
   * @param {Array.<P5Node>} children
   * @returns {Array.<Child>}
   */
  function nodes(ctx, children) {
    let index = -1;
    /** @type {Array.<Child>} */
    const result = [];

    while (++index < children.length) {
      // @ts-expect-error Assume no roots in children.
      result[index] = transform$1(ctx, children[index]);
    }

    return result
  }

  /**
   * Transform a document.
   * Stores `ast.quirksMode` in `node.data.quirksMode`.
   *
   * @type {Handler}
   * @param {P5Document} ast
   * @param {Array.<Child>} children
   * @returns {Root}
   */
  function root$3(ctx, ast, children) {
    /** @type {Root} */
    const result = {
      type: 'root',
      children,
      data: {quirksMode: ast.mode === 'quirks' || ast.mode === 'limited-quirks'}
    };

    if (ctx.file && ctx.location) {
      const doc = String(ctx.file);
      const loc = location(doc);
      result.position = {
        start: loc.toPoint(0),
        end: loc.toPoint(doc.length)
      };
    }

    return result
  }

  /**
   * Transform a doctype.
   *
   * @type {Handler}
   * @returns {Doctype}
   */
  function doctype$2() {
    // @ts-expect-error Types are out of date.
    return {type: 'doctype'}
  }

  /**
   * Transform a text.
   *
   * @type {Handler}
   * @param {P5Text} ast
   * @returns {Text}
   */
  function text$3(_, ast) {
    return {type: 'text', value: ast.value}
  }

  /**
   * Transform a comment.
   *
   * @type {Handler}
   * @param {P5Comment} ast
   * @returns {Comment}
   */
  function comment$3(_, ast) {
    return {type: 'comment', value: ast.data}
  }

  /**
   * Transform an element.
   *
   * @type {Handler}
   * @param {P5Element} ast
   * @param {Array.<ElementChild>} children
   * @returns {Element}
   */
  function element$6(ctx, ast, children) {
    const fn = ctx.schema.space === 'svg' ? s$1 : h$1;
    let index = -1;
    /** @type {Object.<string, string>} */
    const props = {};

    while (++index < ast.attrs.length) {
      const attribute = ast.attrs[index];
      props[(attribute.prefix ? attribute.prefix + ':' : '') + attribute.name] =
        attribute.value;
    }

    const result = fn(ast.tagName, props, children);

    if (result.tagName === 'template' && 'content' in ast) {
      const pos = ast.sourceCodeLocation;
      const startTag = pos && pos.startTag && position(pos.startTag);
      const endTag = pos && pos.endTag && position(pos.endTag);

      /** @type {Root} */
      // @ts-expect-error Types are wrong.
      const content = transform$1(ctx, ast.content);

      if (startTag && endTag && ctx.file) {
        content.position = {start: startTag.end, end: endTag.start};
      }

      result.content = content;
    }

    return result
  }

  /**
   * Create clean positional information.
   *
   * @param {Context} ctx
   * @param {Node} node
   * @param {P5ElementLocation} location
   * @returns {Position|null}
   */
  function createLocation(ctx, node, location) {
    const result = position(location);

    if (node.type === 'element') {
      const tail = node.children[node.children.length - 1];

      // Bug for unclosed with children.
      // See: <https://github.com/inikulin/parse5/issues/109>.
      if (
        result &&
        !location.endTag &&
        tail &&
        tail.position &&
        tail.position.end
      ) {
        result.end = Object.assign({}, tail.position.end);
      }

      if (ctx.verbose) {
        /** @type {Object.<string, Position|null>} */
        const props = {};
        /** @type {string} */
        let key;

        for (key in location.attrs) {
          if (own$g.call(location.attrs, key)) {
            props[find$5(ctx.schema, key).property] = position(location.attrs[key]);
          }
        }

        node.data = {
          position: {
            opening: position(location.startTag),
            closing: location.endTag ? position(location.endTag) : null,
            properties: props
          }
        };
      }
    }

    return result
  }

  /**
   * @param {P5Location} loc
   * @returns {Position|null}
   */
  function position(loc) {
    const start = point({
      line: loc.startLine,
      column: loc.startCol,
      offset: loc.startOffset
    });
    const end = point({
      line: loc.endLine,
      column: loc.endCol,
      offset: loc.endOffset
    });
    // @ts-expect-error `null` is fine.
    return start || end ? {start, end} : null
  }

  /**
   * @param {Point} point
   * @returns {Point|null}
   */
  function point(point) {
    return point.line && point.column ? point : null
  }

  /**
   * @param {VFile|Options} value
   * @returns {value is VFile}
   */
  function isFile(value) {
    return 'messages' in value
  }

  /**
   * @typedef {import('./info.js').Info} Info
   * @typedef {Record<string, Info>} Properties
   * @typedef {Record<string, string>} Normal
   */

  class Schema$4 {
    /**
     * @constructor
     * @param {Properties} property
     * @param {Normal} normal
     * @param {string} [space]
     */
    constructor(property, normal, space) {
      this.property = property;
      this.normal = normal;
      if (space) {
        this.space = space;
      }
    }
  }

  /** @type {Properties} */
  Schema$4.prototype.property = {};
  /** @type {Normal} */
  Schema$4.prototype.normal = {};
  /** @type {string|null} */
  Schema$4.prototype.space = null;

  /**
   * @typedef {import('./schema.js').Properties} Properties
   * @typedef {import('./schema.js').Normal} Normal
   */

  /**
   * @param {Schema[]} definitions
   * @param {string} [space]
   * @returns {Schema}
   */
  function merge$4(definitions, space) {
    /** @type {Properties} */
    const property = {};
    /** @type {Normal} */
    const normal = {};
    let index = -1;

    while (++index < definitions.length) {
      Object.assign(property, definitions[index].property);
      Object.assign(normal, definitions[index].normal);
    }

    return new Schema$4(property, normal, space)
  }

  /**
   * @param {string} value
   * @returns {string}
   */
  function normalize$4(value) {
    return value.toLowerCase()
  }

  class Info$4 {
    /**
     * @constructor
     * @param {string} property
     * @param {string} attribute
     */
    constructor(property, attribute) {
      /** @type {string} */
      this.property = property;
      /** @type {string} */
      this.attribute = attribute;
    }
  }

  /** @type {string|null} */
  Info$4.prototype.space = null;
  Info$4.prototype.boolean = false;
  Info$4.prototype.booleanish = false;
  Info$4.prototype.overloadedBoolean = false;
  Info$4.prototype.number = false;
  Info$4.prototype.commaSeparated = false;
  Info$4.prototype.spaceSeparated = false;
  Info$4.prototype.commaOrSpaceSeparated = false;
  Info$4.prototype.mustUseProperty = false;
  Info$4.prototype.defined = false;

  let powers$4 = 0;

  const boolean$4 = increment$4();
  const booleanish$4 = increment$4();
  const overloadedBoolean$4 = increment$4();
  const number$4 = increment$4();
  const spaceSeparated$4 = increment$4();
  const commaSeparated$4 = increment$4();
  const commaOrSpaceSeparated$4 = increment$4();

  function increment$4() {
    return 2 ** ++powers$4
  }

  var types$4 = /*#__PURE__*/Object.freeze({
      __proto__: null,
      boolean: boolean$4,
      booleanish: booleanish$4,
      overloadedBoolean: overloadedBoolean$4,
      number: number$4,
      spaceSeparated: spaceSeparated$4,
      commaSeparated: commaSeparated$4,
      commaOrSpaceSeparated: commaOrSpaceSeparated$4
  });

  /** @type {Array<keyof types>} */
  // @ts-expect-error: hush.
  const checks$4 = Object.keys(types$4);

  class DefinedInfo$4 extends Info$4 {
    /**
     * @constructor
     * @param {string} property
     * @param {string} attribute
     * @param {number|null} [mask]
     * @param {string} [space]
     */
    constructor(property, attribute, mask, space) {
      let index = -1;

      super(property, attribute);

      mark$4(this, 'space', space);

      if (typeof mask === 'number') {
        while (++index < checks$4.length) {
          const check = checks$4[index];
          mark$4(this, checks$4[index], (mask & types$4[check]) === types$4[check]);
        }
      }
    }
  }

  DefinedInfo$4.prototype.defined = true;

  /**
   * @param {DefinedInfo} values
   * @param {string} key
   * @param {unknown} value
   */
  function mark$4(values, key, value) {
    if (value) {
      // @ts-expect-error: assume `value` matches the expected value of `key`.
      values[key] = value;
    }
  }

  /**
   * @typedef {import('./schema.js').Properties} Properties
   * @typedef {import('./schema.js').Normal} Normal
   *
   * @typedef {Record<string, string>} Attributes
   *
   * @typedef {Object} Definition
   * @property {Record<string, number|null>} properties
   * @property {(attributes: Attributes, property: string) => string} transform
   * @property {string} [space]
   * @property {Attributes} [attributes]
   * @property {Array<string>} [mustUseProperty]
   */

  const own$f = {}.hasOwnProperty;

  /**
   * @param {Definition} definition
   * @returns {Schema}
   */
  function create$4(definition) {
    /** @type {Properties} */
    const property = {};
    /** @type {Normal} */
    const normal = {};
    /** @type {string} */
    let prop;

    for (prop in definition.properties) {
      if (own$f.call(definition.properties, prop)) {
        const value = definition.properties[prop];
        const info = new DefinedInfo$4(
          prop,
          definition.transform(definition.attributes || {}, prop),
          value,
          definition.space
        );

        if (
          definition.mustUseProperty &&
          definition.mustUseProperty.includes(prop)
        ) {
          info.mustUseProperty = true;
        }

        property[prop] = info;

        normal[normalize$4(prop)] = prop;
        normal[normalize$4(info.attribute)] = prop;
      }
    }

    return new Schema$4(property, normal, definition.space)
  }

  const xlink$4 = create$4({
    space: 'xlink',
    transform(_, prop) {
      return 'xlink:' + prop.slice(5).toLowerCase()
    },
    properties: {
      xLinkActuate: null,
      xLinkArcRole: null,
      xLinkHref: null,
      xLinkRole: null,
      xLinkShow: null,
      xLinkTitle: null,
      xLinkType: null
    }
  });

  const xml$4 = create$4({
    space: 'xml',
    transform(_, prop) {
      return 'xml:' + prop.slice(3).toLowerCase()
    },
    properties: {xmlLang: null, xmlBase: null, xmlSpace: null}
  });

  /**
   * @param {Record<string, string>} attributes
   * @param {string} attribute
   * @returns {string}
   */
  function caseSensitiveTransform$4(attributes, attribute) {
    return attribute in attributes ? attributes[attribute] : attribute
  }

  /**
   * @param {Record<string, string>} attributes
   * @param {string} property
   * @returns {string}
   */
  function caseInsensitiveTransform$4(attributes, property) {
    return caseSensitiveTransform$4(attributes, property.toLowerCase())
  }

  const xmlns$4 = create$4({
    space: 'xmlns',
    attributes: {xmlnsxlink: 'xmlns:xlink'},
    transform: caseInsensitiveTransform$4,
    properties: {xmlns: null, xmlnsXLink: null}
  });

  const aria$4 = create$4({
    transform(_, prop) {
      return prop === 'role' ? prop : 'aria-' + prop.slice(4).toLowerCase()
    },
    properties: {
      ariaActiveDescendant: null,
      ariaAtomic: booleanish$4,
      ariaAutoComplete: null,
      ariaBusy: booleanish$4,
      ariaChecked: booleanish$4,
      ariaColCount: number$4,
      ariaColIndex: number$4,
      ariaColSpan: number$4,
      ariaControls: spaceSeparated$4,
      ariaCurrent: null,
      ariaDescribedBy: spaceSeparated$4,
      ariaDetails: null,
      ariaDisabled: booleanish$4,
      ariaDropEffect: spaceSeparated$4,
      ariaErrorMessage: null,
      ariaExpanded: booleanish$4,
      ariaFlowTo: spaceSeparated$4,
      ariaGrabbed: booleanish$4,
      ariaHasPopup: null,
      ariaHidden: booleanish$4,
      ariaInvalid: null,
      ariaKeyShortcuts: null,
      ariaLabel: null,
      ariaLabelledBy: spaceSeparated$4,
      ariaLevel: number$4,
      ariaLive: null,
      ariaModal: booleanish$4,
      ariaMultiLine: booleanish$4,
      ariaMultiSelectable: booleanish$4,
      ariaOrientation: null,
      ariaOwns: spaceSeparated$4,
      ariaPlaceholder: null,
      ariaPosInSet: number$4,
      ariaPressed: booleanish$4,
      ariaReadOnly: booleanish$4,
      ariaRelevant: null,
      ariaRequired: booleanish$4,
      ariaRoleDescription: spaceSeparated$4,
      ariaRowCount: number$4,
      ariaRowIndex: number$4,
      ariaRowSpan: number$4,
      ariaSelected: booleanish$4,
      ariaSetSize: number$4,
      ariaSort: null,
      ariaValueMax: number$4,
      ariaValueMin: number$4,
      ariaValueNow: number$4,
      ariaValueText: null,
      role: null
    }
  });

  const html$b = create$4({
    space: 'html',
    attributes: {
      acceptcharset: 'accept-charset',
      classname: 'class',
      htmlfor: 'for',
      httpequiv: 'http-equiv'
    },
    transform: caseInsensitiveTransform$4,
    mustUseProperty: ['checked', 'multiple', 'muted', 'selected'],
    properties: {
      // Standard Properties.
      abbr: null,
      accept: commaSeparated$4,
      acceptCharset: spaceSeparated$4,
      accessKey: spaceSeparated$4,
      action: null,
      allow: null,
      allowFullScreen: boolean$4,
      allowPaymentRequest: boolean$4,
      allowUserMedia: boolean$4,
      alt: null,
      as: null,
      async: boolean$4,
      autoCapitalize: null,
      autoComplete: spaceSeparated$4,
      autoFocus: boolean$4,
      autoPlay: boolean$4,
      capture: boolean$4,
      charSet: null,
      checked: boolean$4,
      cite: null,
      className: spaceSeparated$4,
      cols: number$4,
      colSpan: null,
      content: null,
      contentEditable: booleanish$4,
      controls: boolean$4,
      controlsList: spaceSeparated$4,
      coords: number$4 | commaSeparated$4,
      crossOrigin: null,
      data: null,
      dateTime: null,
      decoding: null,
      default: boolean$4,
      defer: boolean$4,
      dir: null,
      dirName: null,
      disabled: boolean$4,
      download: overloadedBoolean$4,
      draggable: booleanish$4,
      encType: null,
      enterKeyHint: null,
      form: null,
      formAction: null,
      formEncType: null,
      formMethod: null,
      formNoValidate: boolean$4,
      formTarget: null,
      headers: spaceSeparated$4,
      height: number$4,
      hidden: boolean$4,
      high: number$4,
      href: null,
      hrefLang: null,
      htmlFor: spaceSeparated$4,
      httpEquiv: spaceSeparated$4,
      id: null,
      imageSizes: null,
      imageSrcSet: null,
      inputMode: null,
      integrity: null,
      is: null,
      isMap: boolean$4,
      itemId: null,
      itemProp: spaceSeparated$4,
      itemRef: spaceSeparated$4,
      itemScope: boolean$4,
      itemType: spaceSeparated$4,
      kind: null,
      label: null,
      lang: null,
      language: null,
      list: null,
      loading: null,
      loop: boolean$4,
      low: number$4,
      manifest: null,
      max: null,
      maxLength: number$4,
      media: null,
      method: null,
      min: null,
      minLength: number$4,
      multiple: boolean$4,
      muted: boolean$4,
      name: null,
      nonce: null,
      noModule: boolean$4,
      noValidate: boolean$4,
      onAbort: null,
      onAfterPrint: null,
      onAuxClick: null,
      onBeforePrint: null,
      onBeforeUnload: null,
      onBlur: null,
      onCancel: null,
      onCanPlay: null,
      onCanPlayThrough: null,
      onChange: null,
      onClick: null,
      onClose: null,
      onContextLost: null,
      onContextMenu: null,
      onContextRestored: null,
      onCopy: null,
      onCueChange: null,
      onCut: null,
      onDblClick: null,
      onDrag: null,
      onDragEnd: null,
      onDragEnter: null,
      onDragExit: null,
      onDragLeave: null,
      onDragOver: null,
      onDragStart: null,
      onDrop: null,
      onDurationChange: null,
      onEmptied: null,
      onEnded: null,
      onError: null,
      onFocus: null,
      onFormData: null,
      onHashChange: null,
      onInput: null,
      onInvalid: null,
      onKeyDown: null,
      onKeyPress: null,
      onKeyUp: null,
      onLanguageChange: null,
      onLoad: null,
      onLoadedData: null,
      onLoadedMetadata: null,
      onLoadEnd: null,
      onLoadStart: null,
      onMessage: null,
      onMessageError: null,
      onMouseDown: null,
      onMouseEnter: null,
      onMouseLeave: null,
      onMouseMove: null,
      onMouseOut: null,
      onMouseOver: null,
      onMouseUp: null,
      onOffline: null,
      onOnline: null,
      onPageHide: null,
      onPageShow: null,
      onPaste: null,
      onPause: null,
      onPlay: null,
      onPlaying: null,
      onPopState: null,
      onProgress: null,
      onRateChange: null,
      onRejectionHandled: null,
      onReset: null,
      onResize: null,
      onScroll: null,
      onSecurityPolicyViolation: null,
      onSeeked: null,
      onSeeking: null,
      onSelect: null,
      onSlotChange: null,
      onStalled: null,
      onStorage: null,
      onSubmit: null,
      onSuspend: null,
      onTimeUpdate: null,
      onToggle: null,
      onUnhandledRejection: null,
      onUnload: null,
      onVolumeChange: null,
      onWaiting: null,
      onWheel: null,
      open: boolean$4,
      optimum: number$4,
      pattern: null,
      ping: spaceSeparated$4,
      placeholder: null,
      playsInline: boolean$4,
      poster: null,
      preload: null,
      readOnly: boolean$4,
      referrerPolicy: null,
      rel: spaceSeparated$4,
      required: boolean$4,
      reversed: boolean$4,
      rows: number$4,
      rowSpan: number$4,
      sandbox: spaceSeparated$4,
      scope: null,
      scoped: boolean$4,
      seamless: boolean$4,
      selected: boolean$4,
      shape: null,
      size: number$4,
      sizes: null,
      slot: null,
      span: number$4,
      spellCheck: booleanish$4,
      src: null,
      srcDoc: null,
      srcLang: null,
      srcSet: null,
      start: number$4,
      step: null,
      style: null,
      tabIndex: number$4,
      target: null,
      title: null,
      translate: null,
      type: null,
      typeMustMatch: boolean$4,
      useMap: null,
      value: booleanish$4,
      width: number$4,
      wrap: null,

      // Legacy.
      // See: https://html.spec.whatwg.org/#other-elements,-attributes-and-apis
      align: null, // Several. Use CSS `text-align` instead,
      aLink: null, // `<body>`. Use CSS `a:active {color}` instead
      archive: spaceSeparated$4, // `<object>`. List of URIs to archives
      axis: null, // `<td>` and `<th>`. Use `scope` on `<th>`
      background: null, // `<body>`. Use CSS `background-image` instead
      bgColor: null, // `<body>` and table elements. Use CSS `background-color` instead
      border: number$4, // `<table>`. Use CSS `border-width` instead,
      borderColor: null, // `<table>`. Use CSS `border-color` instead,
      bottomMargin: number$4, // `<body>`
      cellPadding: null, // `<table>`
      cellSpacing: null, // `<table>`
      char: null, // Several table elements. When `align=char`, sets the character to align on
      charOff: null, // Several table elements. When `char`, offsets the alignment
      classId: null, // `<object>`
      clear: null, // `<br>`. Use CSS `clear` instead
      code: null, // `<object>`
      codeBase: null, // `<object>`
      codeType: null, // `<object>`
      color: null, // `<font>` and `<hr>`. Use CSS instead
      compact: boolean$4, // Lists. Use CSS to reduce space between items instead
      declare: boolean$4, // `<object>`
      event: null, // `<script>`
      face: null, // `<font>`. Use CSS instead
      frame: null, // `<table>`
      frameBorder: null, // `<iframe>`. Use CSS `border` instead
      hSpace: number$4, // `<img>` and `<object>`
      leftMargin: number$4, // `<body>`
      link: null, // `<body>`. Use CSS `a:link {color: *}` instead
      longDesc: null, // `<frame>`, `<iframe>`, and `<img>`. Use an `<a>`
      lowSrc: null, // `<img>`. Use a `<picture>`
      marginHeight: number$4, // `<body>`
      marginWidth: number$4, // `<body>`
      noResize: boolean$4, // `<frame>`
      noHref: boolean$4, // `<area>`. Use no href instead of an explicit `nohref`
      noShade: boolean$4, // `<hr>`. Use background-color and height instead of borders
      noWrap: boolean$4, // `<td>` and `<th>`
      object: null, // `<applet>`
      profile: null, // `<head>`
      prompt: null, // `<isindex>`
      rev: null, // `<link>`
      rightMargin: number$4, // `<body>`
      rules: null, // `<table>`
      scheme: null, // `<meta>`
      scrolling: booleanish$4, // `<frame>`. Use overflow in the child context
      standby: null, // `<object>`
      summary: null, // `<table>`
      text: null, // `<body>`. Use CSS `color` instead
      topMargin: number$4, // `<body>`
      valueType: null, // `<param>`
      version: null, // `<html>`. Use a doctype.
      vAlign: null, // Several. Use CSS `vertical-align` instead
      vLink: null, // `<body>`. Use CSS `a:visited {color}` instead
      vSpace: number$4, // `<img>` and `<object>`

      // Non-standard Properties.
      allowTransparency: null,
      autoCorrect: null,
      autoSave: null,
      disablePictureInPicture: boolean$4,
      disableRemotePlayback: boolean$4,
      prefix: null,
      property: null,
      results: number$4,
      security: null,
      unselectable: null
    }
  });

  const svg$8 = create$4({
    space: 'svg',
    attributes: {
      accentHeight: 'accent-height',
      alignmentBaseline: 'alignment-baseline',
      arabicForm: 'arabic-form',
      baselineShift: 'baseline-shift',
      capHeight: 'cap-height',
      className: 'class',
      clipPath: 'clip-path',
      clipRule: 'clip-rule',
      colorInterpolation: 'color-interpolation',
      colorInterpolationFilters: 'color-interpolation-filters',
      colorProfile: 'color-profile',
      colorRendering: 'color-rendering',
      crossOrigin: 'crossorigin',
      dataType: 'datatype',
      dominantBaseline: 'dominant-baseline',
      enableBackground: 'enable-background',
      fillOpacity: 'fill-opacity',
      fillRule: 'fill-rule',
      floodColor: 'flood-color',
      floodOpacity: 'flood-opacity',
      fontFamily: 'font-family',
      fontSize: 'font-size',
      fontSizeAdjust: 'font-size-adjust',
      fontStretch: 'font-stretch',
      fontStyle: 'font-style',
      fontVariant: 'font-variant',
      fontWeight: 'font-weight',
      glyphName: 'glyph-name',
      glyphOrientationHorizontal: 'glyph-orientation-horizontal',
      glyphOrientationVertical: 'glyph-orientation-vertical',
      hrefLang: 'hreflang',
      horizAdvX: 'horiz-adv-x',
      horizOriginX: 'horiz-origin-x',
      horizOriginY: 'horiz-origin-y',
      imageRendering: 'image-rendering',
      letterSpacing: 'letter-spacing',
      lightingColor: 'lighting-color',
      markerEnd: 'marker-end',
      markerMid: 'marker-mid',
      markerStart: 'marker-start',
      navDown: 'nav-down',
      navDownLeft: 'nav-down-left',
      navDownRight: 'nav-down-right',
      navLeft: 'nav-left',
      navNext: 'nav-next',
      navPrev: 'nav-prev',
      navRight: 'nav-right',
      navUp: 'nav-up',
      navUpLeft: 'nav-up-left',
      navUpRight: 'nav-up-right',
      onAbort: 'onabort',
      onActivate: 'onactivate',
      onAfterPrint: 'onafterprint',
      onBeforePrint: 'onbeforeprint',
      onBegin: 'onbegin',
      onCancel: 'oncancel',
      onCanPlay: 'oncanplay',
      onCanPlayThrough: 'oncanplaythrough',
      onChange: 'onchange',
      onClick: 'onclick',
      onClose: 'onclose',
      onCopy: 'oncopy',
      onCueChange: 'oncuechange',
      onCut: 'oncut',
      onDblClick: 'ondblclick',
      onDrag: 'ondrag',
      onDragEnd: 'ondragend',
      onDragEnter: 'ondragenter',
      onDragExit: 'ondragexit',
      onDragLeave: 'ondragleave',
      onDragOver: 'ondragover',
      onDragStart: 'ondragstart',
      onDrop: 'ondrop',
      onDurationChange: 'ondurationchange',
      onEmptied: 'onemptied',
      onEnd: 'onend',
      onEnded: 'onended',
      onError: 'onerror',
      onFocus: 'onfocus',
      onFocusIn: 'onfocusin',
      onFocusOut: 'onfocusout',
      onHashChange: 'onhashchange',
      onInput: 'oninput',
      onInvalid: 'oninvalid',
      onKeyDown: 'onkeydown',
      onKeyPress: 'onkeypress',
      onKeyUp: 'onkeyup',
      onLoad: 'onload',
      onLoadedData: 'onloadeddata',
      onLoadedMetadata: 'onloadedmetadata',
      onLoadStart: 'onloadstart',
      onMessage: 'onmessage',
      onMouseDown: 'onmousedown',
      onMouseEnter: 'onmouseenter',
      onMouseLeave: 'onmouseleave',
      onMouseMove: 'onmousemove',
      onMouseOut: 'onmouseout',
      onMouseOver: 'onmouseover',
      onMouseUp: 'onmouseup',
      onMouseWheel: 'onmousewheel',
      onOffline: 'onoffline',
      onOnline: 'ononline',
      onPageHide: 'onpagehide',
      onPageShow: 'onpageshow',
      onPaste: 'onpaste',
      onPause: 'onpause',
      onPlay: 'onplay',
      onPlaying: 'onplaying',
      onPopState: 'onpopstate',
      onProgress: 'onprogress',
      onRateChange: 'onratechange',
      onRepeat: 'onrepeat',
      onReset: 'onreset',
      onResize: 'onresize',
      onScroll: 'onscroll',
      onSeeked: 'onseeked',
      onSeeking: 'onseeking',
      onSelect: 'onselect',
      onShow: 'onshow',
      onStalled: 'onstalled',
      onStorage: 'onstorage',
      onSubmit: 'onsubmit',
      onSuspend: 'onsuspend',
      onTimeUpdate: 'ontimeupdate',
      onToggle: 'ontoggle',
      onUnload: 'onunload',
      onVolumeChange: 'onvolumechange',
      onWaiting: 'onwaiting',
      onZoom: 'onzoom',
      overlinePosition: 'overline-position',
      overlineThickness: 'overline-thickness',
      paintOrder: 'paint-order',
      panose1: 'panose-1',
      pointerEvents: 'pointer-events',
      referrerPolicy: 'referrerpolicy',
      renderingIntent: 'rendering-intent',
      shapeRendering: 'shape-rendering',
      stopColor: 'stop-color',
      stopOpacity: 'stop-opacity',
      strikethroughPosition: 'strikethrough-position',
      strikethroughThickness: 'strikethrough-thickness',
      strokeDashArray: 'stroke-dasharray',
      strokeDashOffset: 'stroke-dashoffset',
      strokeLineCap: 'stroke-linecap',
      strokeLineJoin: 'stroke-linejoin',
      strokeMiterLimit: 'stroke-miterlimit',
      strokeOpacity: 'stroke-opacity',
      strokeWidth: 'stroke-width',
      tabIndex: 'tabindex',
      textAnchor: 'text-anchor',
      textDecoration: 'text-decoration',
      textRendering: 'text-rendering',
      typeOf: 'typeof',
      underlinePosition: 'underline-position',
      underlineThickness: 'underline-thickness',
      unicodeBidi: 'unicode-bidi',
      unicodeRange: 'unicode-range',
      unitsPerEm: 'units-per-em',
      vAlphabetic: 'v-alphabetic',
      vHanging: 'v-hanging',
      vIdeographic: 'v-ideographic',
      vMathematical: 'v-mathematical',
      vectorEffect: 'vector-effect',
      vertAdvY: 'vert-adv-y',
      vertOriginX: 'vert-origin-x',
      vertOriginY: 'vert-origin-y',
      wordSpacing: 'word-spacing',
      writingMode: 'writing-mode',
      xHeight: 'x-height',
      // These were camelcased in Tiny. Now lowercased in SVG 2
      playbackOrder: 'playbackorder',
      timelineBegin: 'timelinebegin'
    },
    transform: caseSensitiveTransform$4,
    properties: {
      about: commaOrSpaceSeparated$4,
      accentHeight: number$4,
      accumulate: null,
      additive: null,
      alignmentBaseline: null,
      alphabetic: number$4,
      amplitude: number$4,
      arabicForm: null,
      ascent: number$4,
      attributeName: null,
      attributeType: null,
      azimuth: number$4,
      bandwidth: null,
      baselineShift: null,
      baseFrequency: null,
      baseProfile: null,
      bbox: null,
      begin: null,
      bias: number$4,
      by: null,
      calcMode: null,
      capHeight: number$4,
      className: spaceSeparated$4,
      clip: null,
      clipPath: null,
      clipPathUnits: null,
      clipRule: null,
      color: null,
      colorInterpolation: null,
      colorInterpolationFilters: null,
      colorProfile: null,
      colorRendering: null,
      content: null,
      contentScriptType: null,
      contentStyleType: null,
      crossOrigin: null,
      cursor: null,
      cx: null,
      cy: null,
      d: null,
      dataType: null,
      defaultAction: null,
      descent: number$4,
      diffuseConstant: number$4,
      direction: null,
      display: null,
      dur: null,
      divisor: number$4,
      dominantBaseline: null,
      download: boolean$4,
      dx: null,
      dy: null,
      edgeMode: null,
      editable: null,
      elevation: number$4,
      enableBackground: null,
      end: null,
      event: null,
      exponent: number$4,
      externalResourcesRequired: null,
      fill: null,
      fillOpacity: number$4,
      fillRule: null,
      filter: null,
      filterRes: null,
      filterUnits: null,
      floodColor: null,
      floodOpacity: null,
      focusable: null,
      focusHighlight: null,
      fontFamily: null,
      fontSize: null,
      fontSizeAdjust: null,
      fontStretch: null,
      fontStyle: null,
      fontVariant: null,
      fontWeight: null,
      format: null,
      fr: null,
      from: null,
      fx: null,
      fy: null,
      g1: commaSeparated$4,
      g2: commaSeparated$4,
      glyphName: commaSeparated$4,
      glyphOrientationHorizontal: null,
      glyphOrientationVertical: null,
      glyphRef: null,
      gradientTransform: null,
      gradientUnits: null,
      handler: null,
      hanging: number$4,
      hatchContentUnits: null,
      hatchUnits: null,
      height: null,
      href: null,
      hrefLang: null,
      horizAdvX: number$4,
      horizOriginX: number$4,
      horizOriginY: number$4,
      id: null,
      ideographic: number$4,
      imageRendering: null,
      initialVisibility: null,
      in: null,
      in2: null,
      intercept: number$4,
      k: number$4,
      k1: number$4,
      k2: number$4,
      k3: number$4,
      k4: number$4,
      kernelMatrix: commaOrSpaceSeparated$4,
      kernelUnitLength: null,
      keyPoints: null, // SEMI_COLON_SEPARATED
      keySplines: null, // SEMI_COLON_SEPARATED
      keyTimes: null, // SEMI_COLON_SEPARATED
      kerning: null,
      lang: null,
      lengthAdjust: null,
      letterSpacing: null,
      lightingColor: null,
      limitingConeAngle: number$4,
      local: null,
      markerEnd: null,
      markerMid: null,
      markerStart: null,
      markerHeight: null,
      markerUnits: null,
      markerWidth: null,
      mask: null,
      maskContentUnits: null,
      maskUnits: null,
      mathematical: null,
      max: null,
      media: null,
      mediaCharacterEncoding: null,
      mediaContentEncodings: null,
      mediaSize: number$4,
      mediaTime: null,
      method: null,
      min: null,
      mode: null,
      name: null,
      navDown: null,
      navDownLeft: null,
      navDownRight: null,
      navLeft: null,
      navNext: null,
      navPrev: null,
      navRight: null,
      navUp: null,
      navUpLeft: null,
      navUpRight: null,
      numOctaves: null,
      observer: null,
      offset: null,
      onAbort: null,
      onActivate: null,
      onAfterPrint: null,
      onBeforePrint: null,
      onBegin: null,
      onCancel: null,
      onCanPlay: null,
      onCanPlayThrough: null,
      onChange: null,
      onClick: null,
      onClose: null,
      onCopy: null,
      onCueChange: null,
      onCut: null,
      onDblClick: null,
      onDrag: null,
      onDragEnd: null,
      onDragEnter: null,
      onDragExit: null,
      onDragLeave: null,
      onDragOver: null,
      onDragStart: null,
      onDrop: null,
      onDurationChange: null,
      onEmptied: null,
      onEnd: null,
      onEnded: null,
      onError: null,
      onFocus: null,
      onFocusIn: null,
      onFocusOut: null,
      onHashChange: null,
      onInput: null,
      onInvalid: null,
      onKeyDown: null,
      onKeyPress: null,
      onKeyUp: null,
      onLoad: null,
      onLoadedData: null,
      onLoadedMetadata: null,
      onLoadStart: null,
      onMessage: null,
      onMouseDown: null,
      onMouseEnter: null,
      onMouseLeave: null,
      onMouseMove: null,
      onMouseOut: null,
      onMouseOver: null,
      onMouseUp: null,
      onMouseWheel: null,
      onOffline: null,
      onOnline: null,
      onPageHide: null,
      onPageShow: null,
      onPaste: null,
      onPause: null,
      onPlay: null,
      onPlaying: null,
      onPopState: null,
      onProgress: null,
      onRateChange: null,
      onRepeat: null,
      onReset: null,
      onResize: null,
      onScroll: null,
      onSeeked: null,
      onSeeking: null,
      onSelect: null,
      onShow: null,
      onStalled: null,
      onStorage: null,
      onSubmit: null,
      onSuspend: null,
      onTimeUpdate: null,
      onToggle: null,
      onUnload: null,
      onVolumeChange: null,
      onWaiting: null,
      onZoom: null,
      opacity: null,
      operator: null,
      order: null,
      orient: null,
      orientation: null,
      origin: null,
      overflow: null,
      overlay: null,
      overlinePosition: number$4,
      overlineThickness: number$4,
      paintOrder: null,
      panose1: null,
      path: null,
      pathLength: number$4,
      patternContentUnits: null,
      patternTransform: null,
      patternUnits: null,
      phase: null,
      ping: spaceSeparated$4,
      pitch: null,
      playbackOrder: null,
      pointerEvents: null,
      points: null,
      pointsAtX: number$4,
      pointsAtY: number$4,
      pointsAtZ: number$4,
      preserveAlpha: null,
      preserveAspectRatio: null,
      primitiveUnits: null,
      propagate: null,
      property: commaOrSpaceSeparated$4,
      r: null,
      radius: null,
      referrerPolicy: null,
      refX: null,
      refY: null,
      rel: commaOrSpaceSeparated$4,
      rev: commaOrSpaceSeparated$4,
      renderingIntent: null,
      repeatCount: null,
      repeatDur: null,
      requiredExtensions: commaOrSpaceSeparated$4,
      requiredFeatures: commaOrSpaceSeparated$4,
      requiredFonts: commaOrSpaceSeparated$4,
      requiredFormats: commaOrSpaceSeparated$4,
      resource: null,
      restart: null,
      result: null,
      rotate: null,
      rx: null,
      ry: null,
      scale: null,
      seed: null,
      shapeRendering: null,
      side: null,
      slope: null,
      snapshotTime: null,
      specularConstant: number$4,
      specularExponent: number$4,
      spreadMethod: null,
      spacing: null,
      startOffset: null,
      stdDeviation: null,
      stemh: null,
      stemv: null,
      stitchTiles: null,
      stopColor: null,
      stopOpacity: null,
      strikethroughPosition: number$4,
      strikethroughThickness: number$4,
      string: null,
      stroke: null,
      strokeDashArray: commaOrSpaceSeparated$4,
      strokeDashOffset: null,
      strokeLineCap: null,
      strokeLineJoin: null,
      strokeMiterLimit: number$4,
      strokeOpacity: number$4,
      strokeWidth: null,
      style: null,
      surfaceScale: number$4,
      syncBehavior: null,
      syncBehaviorDefault: null,
      syncMaster: null,
      syncTolerance: null,
      syncToleranceDefault: null,
      systemLanguage: commaOrSpaceSeparated$4,
      tabIndex: number$4,
      tableValues: null,
      target: null,
      targetX: number$4,
      targetY: number$4,
      textAnchor: null,
      textDecoration: null,
      textRendering: null,
      textLength: null,
      timelineBegin: null,
      title: null,
      transformBehavior: null,
      type: null,
      typeOf: commaOrSpaceSeparated$4,
      to: null,
      transform: null,
      u1: null,
      u2: null,
      underlinePosition: number$4,
      underlineThickness: number$4,
      unicode: null,
      unicodeBidi: null,
      unicodeRange: null,
      unitsPerEm: number$4,
      values: null,
      vAlphabetic: number$4,
      vMathematical: number$4,
      vectorEffect: null,
      vHanging: number$4,
      vIdeographic: number$4,
      version: null,
      vertAdvY: number$4,
      vertOriginX: number$4,
      vertOriginY: number$4,
      viewBox: null,
      viewTarget: null,
      visibility: null,
      width: null,
      widths: null,
      wordSpacing: null,
      writingMode: null,
      x: null,
      x1: null,
      x2: null,
      xChannelSelector: null,
      xHeight: number$4,
      y: null,
      y1: null,
      y2: null,
      yChannelSelector: null,
      z: null,
      zoomAndPan: null
    }
  });

  /**
   * @typedef {import('./util/schema.js').Schema} Schema
   */

  const valid$4 = /^data[-\w.:]+$/i;
  const dash$4 = /-[a-z]/g;
  const cap$4 = /[A-Z]/g;

  /**
   * @param {Schema} schema
   * @param {string} value
   * @returns {Info}
   */
  function find$4(schema, value) {
    const normal = normalize$4(value);
    let prop = value;
    let Type = Info$4;

    if (normal in schema.normal) {
      return schema.property[schema.normal[normal]]
    }

    if (normal.length > 4 && normal.slice(0, 4) === 'data' && valid$4.test(value)) {
      // Attribute or property.
      if (value.charAt(4) === '-') {
        // Turn it into a property.
        const rest = value.slice(5).replace(dash$4, camelcase$4);
        prop = 'data' + rest.charAt(0).toUpperCase() + rest.slice(1);
      } else {
        // Turn it into an attribute.
        const rest = value.slice(4);

        if (!dash$4.test(rest)) {
          let dashes = rest.replace(cap$4, kebab$4);

          if (dashes.charAt(0) !== '-') {
            dashes = '-' + dashes;
          }

          value = 'data' + dashes;
        }
      }

      Type = DefinedInfo$4;
    }

    return new Type(prop, value)
  }

  /**
   * @param {string} $0
   * @returns {string}
   */
  function kebab$4($0) {
    return '-' + $0.toLowerCase()
  }

  /**
   * @param {string} $0
   * @returns {string}
   */
  function camelcase$4($0) {
    return $0.charAt(1).toUpperCase()
  }

  /**
   * @typedef {import('./lib/util/info.js').Info} Info
   * @typedef {import('./lib/util/schema.js').Schema} Schema
   */
  const html$a = merge$4([xml$4, xlink$4, xmlns$4, aria$4, html$b], 'html');
  const svg$7 = merge$4([xml$4, xlink$4, xmlns$4, aria$4, svg$8], 'svg');

  /**
   * @typedef {import('./info.js').Info} Info
   * @typedef {Record<string, Info>} Properties
   * @typedef {Record<string, string>} Normal
   */

  class Schema$3 {
    /**
     * @constructor
     * @param {Properties} property
     * @param {Normal} normal
     * @param {string} [space]
     */
    constructor(property, normal, space) {
      this.property = property;
      this.normal = normal;
      if (space) {
        this.space = space;
      }
    }
  }

  /** @type {Properties} */
  Schema$3.prototype.property = {};
  /** @type {Normal} */
  Schema$3.prototype.normal = {};
  /** @type {string|null} */
  Schema$3.prototype.space = null;

  /**
   * @typedef {import('./schema.js').Properties} Properties
   * @typedef {import('./schema.js').Normal} Normal
   */

  /**
   * @param {Schema[]} definitions
   * @param {string} [space]
   * @returns {Schema}
   */
  function merge$3(definitions, space) {
    /** @type {Properties} */
    const property = {};
    /** @type {Normal} */
    const normal = {};
    let index = -1;

    while (++index < definitions.length) {
      Object.assign(property, definitions[index].property);
      Object.assign(normal, definitions[index].normal);
    }

    return new Schema$3(property, normal, space)
  }

  /**
   * @param {string} value
   * @returns {string}
   */
  function normalize$3(value) {
    return value.toLowerCase()
  }

  class Info$3 {
    /**
     * @constructor
     * @param {string} property
     * @param {string} attribute
     */
    constructor(property, attribute) {
      /** @type {string} */
      this.property = property;
      /** @type {string} */
      this.attribute = attribute;
    }
  }

  /** @type {string|null} */
  Info$3.prototype.space = null;
  Info$3.prototype.boolean = false;
  Info$3.prototype.booleanish = false;
  Info$3.prototype.overloadedBoolean = false;
  Info$3.prototype.number = false;
  Info$3.prototype.commaSeparated = false;
  Info$3.prototype.spaceSeparated = false;
  Info$3.prototype.commaOrSpaceSeparated = false;
  Info$3.prototype.mustUseProperty = false;
  Info$3.prototype.defined = false;

  let powers$3 = 0;

  const boolean$3 = increment$3();
  const booleanish$3 = increment$3();
  const overloadedBoolean$3 = increment$3();
  const number$3 = increment$3();
  const spaceSeparated$3 = increment$3();
  const commaSeparated$3 = increment$3();
  const commaOrSpaceSeparated$3 = increment$3();

  function increment$3() {
    return 2 ** ++powers$3
  }

  var types$3 = /*#__PURE__*/Object.freeze({
      __proto__: null,
      boolean: boolean$3,
      booleanish: booleanish$3,
      overloadedBoolean: overloadedBoolean$3,
      number: number$3,
      spaceSeparated: spaceSeparated$3,
      commaSeparated: commaSeparated$3,
      commaOrSpaceSeparated: commaOrSpaceSeparated$3
  });

  /** @type {Array<keyof types>} */
  // @ts-expect-error: hush.
  const checks$3 = Object.keys(types$3);

  class DefinedInfo$3 extends Info$3 {
    /**
     * @constructor
     * @param {string} property
     * @param {string} attribute
     * @param {number|null} [mask]
     * @param {string} [space]
     */
    constructor(property, attribute, mask, space) {
      let index = -1;

      super(property, attribute);

      mark$3(this, 'space', space);

      if (typeof mask === 'number') {
        while (++index < checks$3.length) {
          const check = checks$3[index];
          mark$3(this, checks$3[index], (mask & types$3[check]) === types$3[check]);
        }
      }
    }
  }

  DefinedInfo$3.prototype.defined = true;

  /**
   * @param {DefinedInfo} values
   * @param {string} key
   * @param {unknown} value
   */
  function mark$3(values, key, value) {
    if (value) {
      // @ts-expect-error: assume `value` matches the expected value of `key`.
      values[key] = value;
    }
  }

  /**
   * @typedef {import('./schema.js').Properties} Properties
   * @typedef {import('./schema.js').Normal} Normal
   *
   * @typedef {Record<string, string>} Attributes
   *
   * @typedef {Object} Definition
   * @property {Record<string, number|null>} properties
   * @property {(attributes: Attributes, property: string) => string} transform
   * @property {string} [space]
   * @property {Attributes} [attributes]
   * @property {Array<string>} [mustUseProperty]
   */

  const own$e = {}.hasOwnProperty;

  /**
   * @param {Definition} definition
   * @returns {Schema}
   */
  function create$3(definition) {
    /** @type {Properties} */
    const property = {};
    /** @type {Normal} */
    const normal = {};
    /** @type {string} */
    let prop;

    for (prop in definition.properties) {
      if (own$e.call(definition.properties, prop)) {
        const value = definition.properties[prop];
        const info = new DefinedInfo$3(
          prop,
          definition.transform(definition.attributes || {}, prop),
          value,
          definition.space
        );

        if (
          definition.mustUseProperty &&
          definition.mustUseProperty.includes(prop)
        ) {
          info.mustUseProperty = true;
        }

        property[prop] = info;

        normal[normalize$3(prop)] = prop;
        normal[normalize$3(info.attribute)] = prop;
      }
    }

    return new Schema$3(property, normal, definition.space)
  }

  const xlink$3 = create$3({
    space: 'xlink',
    transform(_, prop) {
      return 'xlink:' + prop.slice(5).toLowerCase()
    },
    properties: {
      xLinkActuate: null,
      xLinkArcRole: null,
      xLinkHref: null,
      xLinkRole: null,
      xLinkShow: null,
      xLinkTitle: null,
      xLinkType: null
    }
  });

  const xml$3 = create$3({
    space: 'xml',
    transform(_, prop) {
      return 'xml:' + prop.slice(3).toLowerCase()
    },
    properties: {xmlLang: null, xmlBase: null, xmlSpace: null}
  });

  /**
   * @param {Record<string, string>} attributes
   * @param {string} attribute
   * @returns {string}
   */
  function caseSensitiveTransform$3(attributes, attribute) {
    return attribute in attributes ? attributes[attribute] : attribute
  }

  /**
   * @param {Record<string, string>} attributes
   * @param {string} property
   * @returns {string}
   */
  function caseInsensitiveTransform$3(attributes, property) {
    return caseSensitiveTransform$3(attributes, property.toLowerCase())
  }

  const xmlns$3 = create$3({
    space: 'xmlns',
    attributes: {xmlnsxlink: 'xmlns:xlink'},
    transform: caseInsensitiveTransform$3,
    properties: {xmlns: null, xmlnsXLink: null}
  });

  const aria$3 = create$3({
    transform(_, prop) {
      return prop === 'role' ? prop : 'aria-' + prop.slice(4).toLowerCase()
    },
    properties: {
      ariaActiveDescendant: null,
      ariaAtomic: booleanish$3,
      ariaAutoComplete: null,
      ariaBusy: booleanish$3,
      ariaChecked: booleanish$3,
      ariaColCount: number$3,
      ariaColIndex: number$3,
      ariaColSpan: number$3,
      ariaControls: spaceSeparated$3,
      ariaCurrent: null,
      ariaDescribedBy: spaceSeparated$3,
      ariaDetails: null,
      ariaDisabled: booleanish$3,
      ariaDropEffect: spaceSeparated$3,
      ariaErrorMessage: null,
      ariaExpanded: booleanish$3,
      ariaFlowTo: spaceSeparated$3,
      ariaGrabbed: booleanish$3,
      ariaHasPopup: null,
      ariaHidden: booleanish$3,
      ariaInvalid: null,
      ariaKeyShortcuts: null,
      ariaLabel: null,
      ariaLabelledBy: spaceSeparated$3,
      ariaLevel: number$3,
      ariaLive: null,
      ariaModal: booleanish$3,
      ariaMultiLine: booleanish$3,
      ariaMultiSelectable: booleanish$3,
      ariaOrientation: null,
      ariaOwns: spaceSeparated$3,
      ariaPlaceholder: null,
      ariaPosInSet: number$3,
      ariaPressed: booleanish$3,
      ariaReadOnly: booleanish$3,
      ariaRelevant: null,
      ariaRequired: booleanish$3,
      ariaRoleDescription: spaceSeparated$3,
      ariaRowCount: number$3,
      ariaRowIndex: number$3,
      ariaRowSpan: number$3,
      ariaSelected: booleanish$3,
      ariaSetSize: number$3,
      ariaSort: null,
      ariaValueMax: number$3,
      ariaValueMin: number$3,
      ariaValueNow: number$3,
      ariaValueText: null,
      role: null
    }
  });

  const html$9 = create$3({
    space: 'html',
    attributes: {
      acceptcharset: 'accept-charset',
      classname: 'class',
      htmlfor: 'for',
      httpequiv: 'http-equiv'
    },
    transform: caseInsensitiveTransform$3,
    mustUseProperty: ['checked', 'multiple', 'muted', 'selected'],
    properties: {
      // Standard Properties.
      abbr: null,
      accept: commaSeparated$3,
      acceptCharset: spaceSeparated$3,
      accessKey: spaceSeparated$3,
      action: null,
      allow: null,
      allowFullScreen: boolean$3,
      allowPaymentRequest: boolean$3,
      allowUserMedia: boolean$3,
      alt: null,
      as: null,
      async: boolean$3,
      autoCapitalize: null,
      autoComplete: spaceSeparated$3,
      autoFocus: boolean$3,
      autoPlay: boolean$3,
      capture: boolean$3,
      charSet: null,
      checked: boolean$3,
      cite: null,
      className: spaceSeparated$3,
      cols: number$3,
      colSpan: null,
      content: null,
      contentEditable: booleanish$3,
      controls: boolean$3,
      controlsList: spaceSeparated$3,
      coords: number$3 | commaSeparated$3,
      crossOrigin: null,
      data: null,
      dateTime: null,
      decoding: null,
      default: boolean$3,
      defer: boolean$3,
      dir: null,
      dirName: null,
      disabled: boolean$3,
      download: overloadedBoolean$3,
      draggable: booleanish$3,
      encType: null,
      enterKeyHint: null,
      form: null,
      formAction: null,
      formEncType: null,
      formMethod: null,
      formNoValidate: boolean$3,
      formTarget: null,
      headers: spaceSeparated$3,
      height: number$3,
      hidden: boolean$3,
      high: number$3,
      href: null,
      hrefLang: null,
      htmlFor: spaceSeparated$3,
      httpEquiv: spaceSeparated$3,
      id: null,
      imageSizes: null,
      imageSrcSet: null,
      inputMode: null,
      integrity: null,
      is: null,
      isMap: boolean$3,
      itemId: null,
      itemProp: spaceSeparated$3,
      itemRef: spaceSeparated$3,
      itemScope: boolean$3,
      itemType: spaceSeparated$3,
      kind: null,
      label: null,
      lang: null,
      language: null,
      list: null,
      loading: null,
      loop: boolean$3,
      low: number$3,
      manifest: null,
      max: null,
      maxLength: number$3,
      media: null,
      method: null,
      min: null,
      minLength: number$3,
      multiple: boolean$3,
      muted: boolean$3,
      name: null,
      nonce: null,
      noModule: boolean$3,
      noValidate: boolean$3,
      onAbort: null,
      onAfterPrint: null,
      onAuxClick: null,
      onBeforePrint: null,
      onBeforeUnload: null,
      onBlur: null,
      onCancel: null,
      onCanPlay: null,
      onCanPlayThrough: null,
      onChange: null,
      onClick: null,
      onClose: null,
      onContextLost: null,
      onContextMenu: null,
      onContextRestored: null,
      onCopy: null,
      onCueChange: null,
      onCut: null,
      onDblClick: null,
      onDrag: null,
      onDragEnd: null,
      onDragEnter: null,
      onDragExit: null,
      onDragLeave: null,
      onDragOver: null,
      onDragStart: null,
      onDrop: null,
      onDurationChange: null,
      onEmptied: null,
      onEnded: null,
      onError: null,
      onFocus: null,
      onFormData: null,
      onHashChange: null,
      onInput: null,
      onInvalid: null,
      onKeyDown: null,
      onKeyPress: null,
      onKeyUp: null,
      onLanguageChange: null,
      onLoad: null,
      onLoadedData: null,
      onLoadedMetadata: null,
      onLoadEnd: null,
      onLoadStart: null,
      onMessage: null,
      onMessageError: null,
      onMouseDown: null,
      onMouseEnter: null,
      onMouseLeave: null,
      onMouseMove: null,
      onMouseOut: null,
      onMouseOver: null,
      onMouseUp: null,
      onOffline: null,
      onOnline: null,
      onPageHide: null,
      onPageShow: null,
      onPaste: null,
      onPause: null,
      onPlay: null,
      onPlaying: null,
      onPopState: null,
      onProgress: null,
      onRateChange: null,
      onRejectionHandled: null,
      onReset: null,
      onResize: null,
      onScroll: null,
      onSecurityPolicyViolation: null,
      onSeeked: null,
      onSeeking: null,
      onSelect: null,
      onSlotChange: null,
      onStalled: null,
      onStorage: null,
      onSubmit: null,
      onSuspend: null,
      onTimeUpdate: null,
      onToggle: null,
      onUnhandledRejection: null,
      onUnload: null,
      onVolumeChange: null,
      onWaiting: null,
      onWheel: null,
      open: boolean$3,
      optimum: number$3,
      pattern: null,
      ping: spaceSeparated$3,
      placeholder: null,
      playsInline: boolean$3,
      poster: null,
      preload: null,
      readOnly: boolean$3,
      referrerPolicy: null,
      rel: spaceSeparated$3,
      required: boolean$3,
      reversed: boolean$3,
      rows: number$3,
      rowSpan: number$3,
      sandbox: spaceSeparated$3,
      scope: null,
      scoped: boolean$3,
      seamless: boolean$3,
      selected: boolean$3,
      shape: null,
      size: number$3,
      sizes: null,
      slot: null,
      span: number$3,
      spellCheck: booleanish$3,
      src: null,
      srcDoc: null,
      srcLang: null,
      srcSet: null,
      start: number$3,
      step: null,
      style: null,
      tabIndex: number$3,
      target: null,
      title: null,
      translate: null,
      type: null,
      typeMustMatch: boolean$3,
      useMap: null,
      value: booleanish$3,
      width: number$3,
      wrap: null,

      // Legacy.
      // See: https://html.spec.whatwg.org/#other-elements,-attributes-and-apis
      align: null, // Several. Use CSS `text-align` instead,
      aLink: null, // `<body>`. Use CSS `a:active {color}` instead
      archive: spaceSeparated$3, // `<object>`. List of URIs to archives
      axis: null, // `<td>` and `<th>`. Use `scope` on `<th>`
      background: null, // `<body>`. Use CSS `background-image` instead
      bgColor: null, // `<body>` and table elements. Use CSS `background-color` instead
      border: number$3, // `<table>`. Use CSS `border-width` instead,
      borderColor: null, // `<table>`. Use CSS `border-color` instead,
      bottomMargin: number$3, // `<body>`
      cellPadding: null, // `<table>`
      cellSpacing: null, // `<table>`
      char: null, // Several table elements. When `align=char`, sets the character to align on
      charOff: null, // Several table elements. When `char`, offsets the alignment
      classId: null, // `<object>`
      clear: null, // `<br>`. Use CSS `clear` instead
      code: null, // `<object>`
      codeBase: null, // `<object>`
      codeType: null, // `<object>`
      color: null, // `<font>` and `<hr>`. Use CSS instead
      compact: boolean$3, // Lists. Use CSS to reduce space between items instead
      declare: boolean$3, // `<object>`
      event: null, // `<script>`
      face: null, // `<font>`. Use CSS instead
      frame: null, // `<table>`
      frameBorder: null, // `<iframe>`. Use CSS `border` instead
      hSpace: number$3, // `<img>` and `<object>`
      leftMargin: number$3, // `<body>`
      link: null, // `<body>`. Use CSS `a:link {color: *}` instead
      longDesc: null, // `<frame>`, `<iframe>`, and `<img>`. Use an `<a>`
      lowSrc: null, // `<img>`. Use a `<picture>`
      marginHeight: number$3, // `<body>`
      marginWidth: number$3, // `<body>`
      noResize: boolean$3, // `<frame>`
      noHref: boolean$3, // `<area>`. Use no href instead of an explicit `nohref`
      noShade: boolean$3, // `<hr>`. Use background-color and height instead of borders
      noWrap: boolean$3, // `<td>` and `<th>`
      object: null, // `<applet>`
      profile: null, // `<head>`
      prompt: null, // `<isindex>`
      rev: null, // `<link>`
      rightMargin: number$3, // `<body>`
      rules: null, // `<table>`
      scheme: null, // `<meta>`
      scrolling: booleanish$3, // `<frame>`. Use overflow in the child context
      standby: null, // `<object>`
      summary: null, // `<table>`
      text: null, // `<body>`. Use CSS `color` instead
      topMargin: number$3, // `<body>`
      valueType: null, // `<param>`
      version: null, // `<html>`. Use a doctype.
      vAlign: null, // Several. Use CSS `vertical-align` instead
      vLink: null, // `<body>`. Use CSS `a:visited {color}` instead
      vSpace: number$3, // `<img>` and `<object>`

      // Non-standard Properties.
      allowTransparency: null,
      autoCorrect: null,
      autoSave: null,
      disablePictureInPicture: boolean$3,
      disableRemotePlayback: boolean$3,
      prefix: null,
      property: null,
      results: number$3,
      security: null,
      unselectable: null
    }
  });

  const svg$6 = create$3({
    space: 'svg',
    attributes: {
      accentHeight: 'accent-height',
      alignmentBaseline: 'alignment-baseline',
      arabicForm: 'arabic-form',
      baselineShift: 'baseline-shift',
      capHeight: 'cap-height',
      className: 'class',
      clipPath: 'clip-path',
      clipRule: 'clip-rule',
      colorInterpolation: 'color-interpolation',
      colorInterpolationFilters: 'color-interpolation-filters',
      colorProfile: 'color-profile',
      colorRendering: 'color-rendering',
      crossOrigin: 'crossorigin',
      dataType: 'datatype',
      dominantBaseline: 'dominant-baseline',
      enableBackground: 'enable-background',
      fillOpacity: 'fill-opacity',
      fillRule: 'fill-rule',
      floodColor: 'flood-color',
      floodOpacity: 'flood-opacity',
      fontFamily: 'font-family',
      fontSize: 'font-size',
      fontSizeAdjust: 'font-size-adjust',
      fontStretch: 'font-stretch',
      fontStyle: 'font-style',
      fontVariant: 'font-variant',
      fontWeight: 'font-weight',
      glyphName: 'glyph-name',
      glyphOrientationHorizontal: 'glyph-orientation-horizontal',
      glyphOrientationVertical: 'glyph-orientation-vertical',
      hrefLang: 'hreflang',
      horizAdvX: 'horiz-adv-x',
      horizOriginX: 'horiz-origin-x',
      horizOriginY: 'horiz-origin-y',
      imageRendering: 'image-rendering',
      letterSpacing: 'letter-spacing',
      lightingColor: 'lighting-color',
      markerEnd: 'marker-end',
      markerMid: 'marker-mid',
      markerStart: 'marker-start',
      navDown: 'nav-down',
      navDownLeft: 'nav-down-left',
      navDownRight: 'nav-down-right',
      navLeft: 'nav-left',
      navNext: 'nav-next',
      navPrev: 'nav-prev',
      navRight: 'nav-right',
      navUp: 'nav-up',
      navUpLeft: 'nav-up-left',
      navUpRight: 'nav-up-right',
      onAbort: 'onabort',
      onActivate: 'onactivate',
      onAfterPrint: 'onafterprint',
      onBeforePrint: 'onbeforeprint',
      onBegin: 'onbegin',
      onCancel: 'oncancel',
      onCanPlay: 'oncanplay',
      onCanPlayThrough: 'oncanplaythrough',
      onChange: 'onchange',
      onClick: 'onclick',
      onClose: 'onclose',
      onCopy: 'oncopy',
      onCueChange: 'oncuechange',
      onCut: 'oncut',
      onDblClick: 'ondblclick',
      onDrag: 'ondrag',
      onDragEnd: 'ondragend',
      onDragEnter: 'ondragenter',
      onDragExit: 'ondragexit',
      onDragLeave: 'ondragleave',
      onDragOver: 'ondragover',
      onDragStart: 'ondragstart',
      onDrop: 'ondrop',
      onDurationChange: 'ondurationchange',
      onEmptied: 'onemptied',
      onEnd: 'onend',
      onEnded: 'onended',
      onError: 'onerror',
      onFocus: 'onfocus',
      onFocusIn: 'onfocusin',
      onFocusOut: 'onfocusout',
      onHashChange: 'onhashchange',
      onInput: 'oninput',
      onInvalid: 'oninvalid',
      onKeyDown: 'onkeydown',
      onKeyPress: 'onkeypress',
      onKeyUp: 'onkeyup',
      onLoad: 'onload',
      onLoadedData: 'onloadeddata',
      onLoadedMetadata: 'onloadedmetadata',
      onLoadStart: 'onloadstart',
      onMessage: 'onmessage',
      onMouseDown: 'onmousedown',
      onMouseEnter: 'onmouseenter',
      onMouseLeave: 'onmouseleave',
      onMouseMove: 'onmousemove',
      onMouseOut: 'onmouseout',
      onMouseOver: 'onmouseover',
      onMouseUp: 'onmouseup',
      onMouseWheel: 'onmousewheel',
      onOffline: 'onoffline',
      onOnline: 'ononline',
      onPageHide: 'onpagehide',
      onPageShow: 'onpageshow',
      onPaste: 'onpaste',
      onPause: 'onpause',
      onPlay: 'onplay',
      onPlaying: 'onplaying',
      onPopState: 'onpopstate',
      onProgress: 'onprogress',
      onRateChange: 'onratechange',
      onRepeat: 'onrepeat',
      onReset: 'onreset',
      onResize: 'onresize',
      onScroll: 'onscroll',
      onSeeked: 'onseeked',
      onSeeking: 'onseeking',
      onSelect: 'onselect',
      onShow: 'onshow',
      onStalled: 'onstalled',
      onStorage: 'onstorage',
      onSubmit: 'onsubmit',
      onSuspend: 'onsuspend',
      onTimeUpdate: 'ontimeupdate',
      onToggle: 'ontoggle',
      onUnload: 'onunload',
      onVolumeChange: 'onvolumechange',
      onWaiting: 'onwaiting',
      onZoom: 'onzoom',
      overlinePosition: 'overline-position',
      overlineThickness: 'overline-thickness',
      paintOrder: 'paint-order',
      panose1: 'panose-1',
      pointerEvents: 'pointer-events',
      referrerPolicy: 'referrerpolicy',
      renderingIntent: 'rendering-intent',
      shapeRendering: 'shape-rendering',
      stopColor: 'stop-color',
      stopOpacity: 'stop-opacity',
      strikethroughPosition: 'strikethrough-position',
      strikethroughThickness: 'strikethrough-thickness',
      strokeDashArray: 'stroke-dasharray',
      strokeDashOffset: 'stroke-dashoffset',
      strokeLineCap: 'stroke-linecap',
      strokeLineJoin: 'stroke-linejoin',
      strokeMiterLimit: 'stroke-miterlimit',
      strokeOpacity: 'stroke-opacity',
      strokeWidth: 'stroke-width',
      tabIndex: 'tabindex',
      textAnchor: 'text-anchor',
      textDecoration: 'text-decoration',
      textRendering: 'text-rendering',
      typeOf: 'typeof',
      underlinePosition: 'underline-position',
      underlineThickness: 'underline-thickness',
      unicodeBidi: 'unicode-bidi',
      unicodeRange: 'unicode-range',
      unitsPerEm: 'units-per-em',
      vAlphabetic: 'v-alphabetic',
      vHanging: 'v-hanging',
      vIdeographic: 'v-ideographic',
      vMathematical: 'v-mathematical',
      vectorEffect: 'vector-effect',
      vertAdvY: 'vert-adv-y',
      vertOriginX: 'vert-origin-x',
      vertOriginY: 'vert-origin-y',
      wordSpacing: 'word-spacing',
      writingMode: 'writing-mode',
      xHeight: 'x-height',
      // These were camelcased in Tiny. Now lowercased in SVG 2
      playbackOrder: 'playbackorder',
      timelineBegin: 'timelinebegin'
    },
    transform: caseSensitiveTransform$3,
    properties: {
      about: commaOrSpaceSeparated$3,
      accentHeight: number$3,
      accumulate: null,
      additive: null,
      alignmentBaseline: null,
      alphabetic: number$3,
      amplitude: number$3,
      arabicForm: null,
      ascent: number$3,
      attributeName: null,
      attributeType: null,
      azimuth: number$3,
      bandwidth: null,
      baselineShift: null,
      baseFrequency: null,
      baseProfile: null,
      bbox: null,
      begin: null,
      bias: number$3,
      by: null,
      calcMode: null,
      capHeight: number$3,
      className: spaceSeparated$3,
      clip: null,
      clipPath: null,
      clipPathUnits: null,
      clipRule: null,
      color: null,
      colorInterpolation: null,
      colorInterpolationFilters: null,
      colorProfile: null,
      colorRendering: null,
      content: null,
      contentScriptType: null,
      contentStyleType: null,
      crossOrigin: null,
      cursor: null,
      cx: null,
      cy: null,
      d: null,
      dataType: null,
      defaultAction: null,
      descent: number$3,
      diffuseConstant: number$3,
      direction: null,
      display: null,
      dur: null,
      divisor: number$3,
      dominantBaseline: null,
      download: boolean$3,
      dx: null,
      dy: null,
      edgeMode: null,
      editable: null,
      elevation: number$3,
      enableBackground: null,
      end: null,
      event: null,
      exponent: number$3,
      externalResourcesRequired: null,
      fill: null,
      fillOpacity: number$3,
      fillRule: null,
      filter: null,
      filterRes: null,
      filterUnits: null,
      floodColor: null,
      floodOpacity: null,
      focusable: null,
      focusHighlight: null,
      fontFamily: null,
      fontSize: null,
      fontSizeAdjust: null,
      fontStretch: null,
      fontStyle: null,
      fontVariant: null,
      fontWeight: null,
      format: null,
      fr: null,
      from: null,
      fx: null,
      fy: null,
      g1: commaSeparated$3,
      g2: commaSeparated$3,
      glyphName: commaSeparated$3,
      glyphOrientationHorizontal: null,
      glyphOrientationVertical: null,
      glyphRef: null,
      gradientTransform: null,
      gradientUnits: null,
      handler: null,
      hanging: number$3,
      hatchContentUnits: null,
      hatchUnits: null,
      height: null,
      href: null,
      hrefLang: null,
      horizAdvX: number$3,
      horizOriginX: number$3,
      horizOriginY: number$3,
      id: null,
      ideographic: number$3,
      imageRendering: null,
      initialVisibility: null,
      in: null,
      in2: null,
      intercept: number$3,
      k: number$3,
      k1: number$3,
      k2: number$3,
      k3: number$3,
      k4: number$3,
      kernelMatrix: commaOrSpaceSeparated$3,
      kernelUnitLength: null,
      keyPoints: null, // SEMI_COLON_SEPARATED
      keySplines: null, // SEMI_COLON_SEPARATED
      keyTimes: null, // SEMI_COLON_SEPARATED
      kerning: null,
      lang: null,
      lengthAdjust: null,
      letterSpacing: null,
      lightingColor: null,
      limitingConeAngle: number$3,
      local: null,
      markerEnd: null,
      markerMid: null,
      markerStart: null,
      markerHeight: null,
      markerUnits: null,
      markerWidth: null,
      mask: null,
      maskContentUnits: null,
      maskUnits: null,
      mathematical: null,
      max: null,
      media: null,
      mediaCharacterEncoding: null,
      mediaContentEncodings: null,
      mediaSize: number$3,
      mediaTime: null,
      method: null,
      min: null,
      mode: null,
      name: null,
      navDown: null,
      navDownLeft: null,
      navDownRight: null,
      navLeft: null,
      navNext: null,
      navPrev: null,
      navRight: null,
      navUp: null,
      navUpLeft: null,
      navUpRight: null,
      numOctaves: null,
      observer: null,
      offset: null,
      onAbort: null,
      onActivate: null,
      onAfterPrint: null,
      onBeforePrint: null,
      onBegin: null,
      onCancel: null,
      onCanPlay: null,
      onCanPlayThrough: null,
      onChange: null,
      onClick: null,
      onClose: null,
      onCopy: null,
      onCueChange: null,
      onCut: null,
      onDblClick: null,
      onDrag: null,
      onDragEnd: null,
      onDragEnter: null,
      onDragExit: null,
      onDragLeave: null,
      onDragOver: null,
      onDragStart: null,
      onDrop: null,
      onDurationChange: null,
      onEmptied: null,
      onEnd: null,
      onEnded: null,
      onError: null,
      onFocus: null,
      onFocusIn: null,
      onFocusOut: null,
      onHashChange: null,
      onInput: null,
      onInvalid: null,
      onKeyDown: null,
      onKeyPress: null,
      onKeyUp: null,
      onLoad: null,
      onLoadedData: null,
      onLoadedMetadata: null,
      onLoadStart: null,
      onMessage: null,
      onMouseDown: null,
      onMouseEnter: null,
      onMouseLeave: null,
      onMouseMove: null,
      onMouseOut: null,
      onMouseOver: null,
      onMouseUp: null,
      onMouseWheel: null,
      onOffline: null,
      onOnline: null,
      onPageHide: null,
      onPageShow: null,
      onPaste: null,
      onPause: null,
      onPlay: null,
      onPlaying: null,
      onPopState: null,
      onProgress: null,
      onRateChange: null,
      onRepeat: null,
      onReset: null,
      onResize: null,
      onScroll: null,
      onSeeked: null,
      onSeeking: null,
      onSelect: null,
      onShow: null,
      onStalled: null,
      onStorage: null,
      onSubmit: null,
      onSuspend: null,
      onTimeUpdate: null,
      onToggle: null,
      onUnload: null,
      onVolumeChange: null,
      onWaiting: null,
      onZoom: null,
      opacity: null,
      operator: null,
      order: null,
      orient: null,
      orientation: null,
      origin: null,
      overflow: null,
      overlay: null,
      overlinePosition: number$3,
      overlineThickness: number$3,
      paintOrder: null,
      panose1: null,
      path: null,
      pathLength: number$3,
      patternContentUnits: null,
      patternTransform: null,
      patternUnits: null,
      phase: null,
      ping: spaceSeparated$3,
      pitch: null,
      playbackOrder: null,
      pointerEvents: null,
      points: null,
      pointsAtX: number$3,
      pointsAtY: number$3,
      pointsAtZ: number$3,
      preserveAlpha: null,
      preserveAspectRatio: null,
      primitiveUnits: null,
      propagate: null,
      property: commaOrSpaceSeparated$3,
      r: null,
      radius: null,
      referrerPolicy: null,
      refX: null,
      refY: null,
      rel: commaOrSpaceSeparated$3,
      rev: commaOrSpaceSeparated$3,
      renderingIntent: null,
      repeatCount: null,
      repeatDur: null,
      requiredExtensions: commaOrSpaceSeparated$3,
      requiredFeatures: commaOrSpaceSeparated$3,
      requiredFonts: commaOrSpaceSeparated$3,
      requiredFormats: commaOrSpaceSeparated$3,
      resource: null,
      restart: null,
      result: null,
      rotate: null,
      rx: null,
      ry: null,
      scale: null,
      seed: null,
      shapeRendering: null,
      side: null,
      slope: null,
      snapshotTime: null,
      specularConstant: number$3,
      specularExponent: number$3,
      spreadMethod: null,
      spacing: null,
      startOffset: null,
      stdDeviation: null,
      stemh: null,
      stemv: null,
      stitchTiles: null,
      stopColor: null,
      stopOpacity: null,
      strikethroughPosition: number$3,
      strikethroughThickness: number$3,
      string: null,
      stroke: null,
      strokeDashArray: commaOrSpaceSeparated$3,
      strokeDashOffset: null,
      strokeLineCap: null,
      strokeLineJoin: null,
      strokeMiterLimit: number$3,
      strokeOpacity: number$3,
      strokeWidth: null,
      style: null,
      surfaceScale: number$3,
      syncBehavior: null,
      syncBehaviorDefault: null,
      syncMaster: null,
      syncTolerance: null,
      syncToleranceDefault: null,
      systemLanguage: commaOrSpaceSeparated$3,
      tabIndex: number$3,
      tableValues: null,
      target: null,
      targetX: number$3,
      targetY: number$3,
      textAnchor: null,
      textDecoration: null,
      textRendering: null,
      textLength: null,
      timelineBegin: null,
      title: null,
      transformBehavior: null,
      type: null,
      typeOf: commaOrSpaceSeparated$3,
      to: null,
      transform: null,
      u1: null,
      u2: null,
      underlinePosition: number$3,
      underlineThickness: number$3,
      unicode: null,
      unicodeBidi: null,
      unicodeRange: null,
      unitsPerEm: number$3,
      values: null,
      vAlphabetic: number$3,
      vMathematical: number$3,
      vectorEffect: null,
      vHanging: number$3,
      vIdeographic: number$3,
      version: null,
      vertAdvY: number$3,
      vertOriginX: number$3,
      vertOriginY: number$3,
      viewBox: null,
      viewTarget: null,
      visibility: null,
      width: null,
      widths: null,
      wordSpacing: null,
      writingMode: null,
      x: null,
      x1: null,
      x2: null,
      xChannelSelector: null,
      xHeight: number$3,
      y: null,
      y1: null,
      y2: null,
      yChannelSelector: null,
      z: null,
      zoomAndPan: null
    }
  });

  /**
   * @typedef {import('./util/schema.js').Schema} Schema
   */

  const valid$3 = /^data[-\w.:]+$/i;
  const dash$3 = /-[a-z]/g;
  const cap$3 = /[A-Z]/g;

  /**
   * @param {Schema} schema
   * @param {string} value
   * @returns {Info}
   */
  function find$3(schema, value) {
    const normal = normalize$3(value);
    let prop = value;
    let Type = Info$3;

    if (normal in schema.normal) {
      return schema.property[schema.normal[normal]]
    }

    if (normal.length > 4 && normal.slice(0, 4) === 'data' && valid$3.test(value)) {
      // Attribute or property.
      if (value.charAt(4) === '-') {
        // Turn it into a property.
        const rest = value.slice(5).replace(dash$3, camelcase$3);
        prop = 'data' + rest.charAt(0).toUpperCase() + rest.slice(1);
      } else {
        // Turn it into an attribute.
        const rest = value.slice(4);

        if (!dash$3.test(rest)) {
          let dashes = rest.replace(cap$3, kebab$3);

          if (dashes.charAt(0) !== '-') {
            dashes = '-' + dashes;
          }

          value = 'data' + dashes;
        }
      }

      Type = DefinedInfo$3;
    }

    return new Type(prop, value)
  }

  /**
   * @param {string} $0
   * @returns {string}
   */
  function kebab$3($0) {
    return '-' + $0.toLowerCase()
  }

  /**
   * @param {string} $0
   * @returns {string}
   */
  function camelcase$3($0) {
    return $0.charAt(1).toUpperCase()
  }

  const hastToReact = {
    classId: 'classID',
    dataType: 'datatype',
    itemId: 'itemID',
    strokeDashArray: 'strokeDasharray',
    strokeDashOffset: 'strokeDashoffset',
    strokeLineCap: 'strokeLinecap',
    strokeLineJoin: 'strokeLinejoin',
    strokeMiterLimit: 'strokeMiterlimit',
    typeOf: 'typeof',
    xLinkActuate: 'xlinkActuate',
    xLinkArcRole: 'xlinkArcrole',
    xLinkHref: 'xlinkHref',
    xLinkRole: 'xlinkRole',
    xLinkShow: 'xlinkShow',
    xLinkTitle: 'xlinkTitle',
    xLinkType: 'xlinkType',
    xmlnsXLink: 'xmlnsXlink'
  };

  /**
   * @typedef {import('./lib/util/info.js').Info} Info
   * @typedef {import('./lib/util/schema.js').Schema} Schema
   */
  const html$8 = merge$3([xml$3, xlink$3, xmlns$3, aria$3, html$9], 'html');
  const svg$5 = merge$3([xml$3, xlink$3, xmlns$3, aria$3, svg$6], 'svg');

  /**
   * Parse space separated tokens to an array of strings.
   *
   * @param {string} value Space separated tokens
   * @returns {Array.<string>} Tokens
   */

  /**
   * Serialize an array of strings as space separated tokens.
   *
   * @param {Array.<string|number>} values Tokens
   * @returns {string} Space separated tokens
   */
  function stringify$6(values) {
    return values.join(' ').trim()
  }

  /**
   * @typedef {Object} StringifyOptions
   * @property {boolean} [padLeft=true] Whether to pad a space before a token (`boolean`, default: `true`).
   * @property {boolean} [padRight=false] Whether to pad a space after a token (`boolean`, default: `false`).
   */

  /**
   * Serialize an array of strings to comma separated tokens.
   *
   * @param {Array.<string|number>} values
   * @param {StringifyOptions} [options]
   * @returns {string}
   */
  function stringify$5(values, options) {
    var settings = options || {};

    // Ensure the last empty entry is seen.
    if (values[values.length - 1] === '') {
      values = values.concat('');
    }

    return values
      .join(
        (settings.padRight ? ' ' : '') +
          ',' +
          (settings.padLeft === false ? '' : ' ')
      )
      .trim()
  }

  // http://www.w3.org/TR/CSS21/grammar.html
  // https://github.com/visionmedia/css-parse/pull/49#issuecomment-30088027
  var COMMENT_REGEX = /\/\*[^*]*\*+([^/*][^*]*\*+)*\//g;

  var NEWLINE_REGEX = /\n/g;
  var WHITESPACE_REGEX = /^\s*/;

  // declaration
  var PROPERTY_REGEX = /^(\*?[-#/*\\\w]+(\[[0-9a-z_-]+\])?)\s*/;
  var COLON_REGEX = /^:\s*/;
  var VALUE_REGEX = /^((?:'(?:\\'|.)*?'|"(?:\\"|.)*?"|\([^)]*?\)|[^};])+)/;
  var SEMICOLON_REGEX = /^[;\s]*/;

  // https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String/Trim#Polyfill
  var TRIM_REGEX = /^\s+|\s+$/g;

  // strings
  var NEWLINE = '\n';
  var FORWARD_SLASH = '/';
  var ASTERISK = '*';
  var EMPTY_STRING = '';

  // types
  var TYPE_COMMENT = 'comment';
  var TYPE_DECLARATION = 'declaration';

  /**
   * @param {String} style
   * @param {Object} [options]
   * @return {Object[]}
   * @throws {TypeError}
   * @throws {Error}
   */
  var inlineStyleParser = function(style, options) {
    if (typeof style !== 'string') {
      throw new TypeError('First argument must be a string');
    }

    if (!style) return [];

    options = options || {};

    /**
     * Positional.
     */
    var lineno = 1;
    var column = 1;

    /**
     * Update lineno and column based on `str`.
     *
     * @param {String} str
     */
    function updatePosition(str) {
      var lines = str.match(NEWLINE_REGEX);
      if (lines) lineno += lines.length;
      var i = str.lastIndexOf(NEWLINE);
      column = ~i ? str.length - i : column + str.length;
    }

    /**
     * Mark position and patch `node.position`.
     *
     * @return {Function}
     */
    function position() {
      var start = { line: lineno, column: column };
      return function(node) {
        node.position = new Position(start);
        whitespace();
        return node;
      };
    }

    /**
     * Store position information for a node.
     *
     * @constructor
     * @property {Object} start
     * @property {Object} end
     * @property {undefined|String} source
     */
    function Position(start) {
      this.start = start;
      this.end = { line: lineno, column: column };
      this.source = options.source;
    }

    /**
     * Non-enumerable source string.
     */
    Position.prototype.content = style;

    /**
     * Error `msg`.
     *
     * @param {String} msg
     * @throws {Error}
     */
    function error(msg) {
      var err = new Error(
        options.source + ':' + lineno + ':' + column + ': ' + msg
      );
      err.reason = msg;
      err.filename = options.source;
      err.line = lineno;
      err.column = column;
      err.source = style;

      if (options.silent) ; else {
        throw err;
      }
    }

    /**
     * Match `re` and return captures.
     *
     * @param {RegExp} re
     * @return {undefined|Array}
     */
    function match(re) {
      var m = re.exec(style);
      if (!m) return;
      var str = m[0];
      updatePosition(str);
      style = style.slice(str.length);
      return m;
    }

    /**
     * Parse whitespace.
     */
    function whitespace() {
      match(WHITESPACE_REGEX);
    }

    /**
     * Parse comments.
     *
     * @param {Object[]} [rules]
     * @return {Object[]}
     */
    function comments(rules) {
      var c;
      rules = rules || [];
      while ((c = comment())) {
        if (c !== false) {
          rules.push(c);
        }
      }
      return rules;
    }

    /**
     * Parse comment.
     *
     * @return {Object}
     * @throws {Error}
     */
    function comment() {
      var pos = position();
      if (FORWARD_SLASH != style.charAt(0) || ASTERISK != style.charAt(1)) return;

      var i = 2;
      while (
        EMPTY_STRING != style.charAt(i) &&
        (ASTERISK != style.charAt(i) || FORWARD_SLASH != style.charAt(i + 1))
      ) {
        ++i;
      }
      i += 2;

      if (EMPTY_STRING === style.charAt(i - 1)) {
        return error('End of comment missing');
      }

      var str = style.slice(2, i - 2);
      column += 2;
      updatePosition(str);
      style = style.slice(i);
      column += 2;

      return pos({
        type: TYPE_COMMENT,
        comment: str
      });
    }

    /**
     * Parse declaration.
     *
     * @return {Object}
     * @throws {Error}
     */
    function declaration() {
      var pos = position();

      // prop
      var prop = match(PROPERTY_REGEX);
      if (!prop) return;
      comment();

      // :
      if (!match(COLON_REGEX)) return error("property missing ':'");

      // val
      var val = match(VALUE_REGEX);

      var ret = pos({
        type: TYPE_DECLARATION,
        property: trim(prop[0].replace(COMMENT_REGEX, EMPTY_STRING)),
        value: val
          ? trim(val[0].replace(COMMENT_REGEX, EMPTY_STRING))
          : EMPTY_STRING
      });

      // ;
      match(SEMICOLON_REGEX);

      return ret;
    }

    /**
     * Parse declarations.
     *
     * @return {Object[]}
     */
    function declarations() {
      var decls = [];

      comments(decls);

      // declarations
      var decl;
      while ((decl = declaration())) {
        if (decl !== false) {
          decls.push(decl);
          comments(decls);
        }
      }

      return decls;
    }

    whitespace();
    return declarations();
  };

  /**
   * Trim `str`.
   *
   * @param {String} str
   * @return {String}
   */
  function trim(str) {
    return str ? str.replace(TRIM_REGEX, EMPTY_STRING) : EMPTY_STRING;
  }

  var parse$6 = inlineStyleParser;

  /**
   * Parses inline style to object.
   *
   * @example
   * // returns { 'line-height': '42' }
   * StyleToObject('line-height: 42;');
   *
   * @param  {String}      style      - The inline style.
   * @param  {Function}    [iterator] - The iterator function.
   * @return {null|Object}
   */
  function StyleToObject(style, iterator) {
    var output = null;
    if (!style || typeof style !== 'string') {
      return output;
    }

    var declaration;
    var declarations = parse$6(style);
    var hasIterator = typeof iterator === 'function';
    var property;
    var value;

    for (var i = 0, len = declarations.length; i < len; i++) {
      declaration = declarations[i];
      property = declaration.property;
      value = declaration.value;

      if (hasIterator) {
        iterator(property, value, declaration);
      } else if (value) {
        output || (output = {});
        output[property] = value;
      }
    }

    return output;
  }

  var styleToObject = StyleToObject;

  /**
   * Map of web namespaces.
   *
   * @type {Record<string, string>}
   */
  const webNamespaces$1 = {
    html: 'http://www.w3.org/1999/xhtml',
    mathml: 'http://www.w3.org/1998/Math/MathML',
    svg: 'http://www.w3.org/2000/svg',
    xlink: 'http://www.w3.org/1999/xlink',
    xml: 'http://www.w3.org/XML/1998/namespace',
    xmlns: 'http://www.w3.org/2000/xmlns/'
  };

  /**
   * @typedef {import('hast').Element} Element
   * @typedef {import('hast').Root} Root
   * @typedef {import('hast').Text} Text
   *
   * @typedef {import('unist-util-is').AssertPredicate<Element>} AssertElement
   * @typedef {import('unist-util-is').AssertPredicate<Text>} AssertText
   * @typedef {import('unist-util-is').AssertPredicate<Root>} AssertRoot
   *
   * @callback CreateElementLike
   * @param {string} name
   * @param {any} attributes
   * @param {Array.<string|any>} [children]
   * @returns {any}
   *
   * @typedef Context
   * @property {html|svg} schema
   * @property {string|null} prefix
   * @property {number} key
   * @property {boolean} react
   * @property {boolean} vue
   * @property {boolean} vdom
   * @property {boolean} hyperscript
   *
   * @typedef Options
   * @property {string|null} [prefix]
   * @property {'html'|'svg'} [space]
   */

  const ns = /** @type {Record<string, string>} */ (webNamespaces$1);
  const toReact = /** @type {Record<string, string>} */ (hastToReact);

  const own$d = {}.hasOwnProperty;

  /** @type {AssertRoot} */
  // @ts-expect-error it’s correct.
  const root$2 = convert$1('root');
  /** @type {AssertElement} */
  // @ts-expect-error it’s correct.
  const element$5 = convert$1('element');
  /** @type {AssertText} */
  // @ts-expect-error it’s correct.
  const text$2 = convert$1('text');

  /**
   * @template {CreateElementLike} H
   * @param {H} h
   * @param {Element|Root} tree
   * @param {string|boolean|Options} [options]
   * @returns {ReturnType<H>}
   */
  function toH(h, tree, options) {
    if (typeof h !== 'function') {
      throw new TypeError('h is not a function')
    }

    const r = react(h);
    const v = vue(h);
    const vd = vdom(h);
    /** @type {string|boolean|null|undefined} */
    let prefix;
    /** @type {Element} */
    let node;

    if (typeof options === 'string' || typeof options === 'boolean') {
      prefix = options;
      options = {};
    } else {
      if (!options) options = {};
      prefix = options.prefix;
    }

    if (root$2(tree)) {
      // @ts-expect-error Allow `doctypes` in there, we’ll filter them out later.
      node =
        tree.children.length === 1 && element$5(tree.children[0])
          ? tree.children[0]
          : {
              type: 'element',
              tagName: 'div',
              properties: {},
              children: tree.children
            };
    } else if (element$5(tree)) {
      node = tree;
    } else {
      throw new Error(
        // @ts-expect-error runtime.
        'Expected root or element, not `' + ((tree && tree.type) || tree) + '`'
      )
    }

    return transform(h, node, {
      schema: options.space === 'svg' ? svg$5 : html$8,
      prefix:
        prefix === undefined || prefix === null
          ? r || v || vd
            ? 'h-'
            : null
          : typeof prefix === 'string'
          ? prefix
          : prefix
          ? 'h-'
          : null,
      key: 0,
      react: r,
      vue: v,
      vdom: vd,
      hyperscript: hyperscript(h)
    })
  }

  /**
   * Transform a hast node through a hyperscript interface to *anything*!
   *
   * @template {CreateElementLike} H
   * @param {H} h
   * @param {Element} node
   * @param {Context} ctx
   */
  function transform(h, node, ctx) {
    const parentSchema = ctx.schema;
    let schema = parentSchema;
    let name = node.tagName;
    /** @type {Record<string, unknown>} */
    const attributes = {};
    /** @type {Array.<ReturnType<H>|string>} */
    const nodes = [];
    let index = -1;
    /** @type {string} */
    let key;

    if (parentSchema.space === 'html' && name.toLowerCase() === 'svg') {
      schema = svg$5;
      ctx.schema = schema;
    }

    for (key in node.properties) {
      if (node.properties && own$d.call(node.properties, key)) {
        addAttribute(attributes, key, node.properties[key], ctx, name);
      }
    }

    if (ctx.vdom) {
      if (schema.space === 'html') {
        name = name.toUpperCase();
      } else if (schema.space) {
        attributes.namespace = ns[schema.space];
      }
    }

    if (ctx.prefix) {
      ctx.key++;
      attributes.key = ctx.prefix + ctx.key;
    }

    if (node.children) {
      while (++index < node.children.length) {
        const value = node.children[index];

        if (element$5(value)) {
          nodes.push(transform(h, value, ctx));
        } else if (text$2(value)) {
          nodes.push(value.value);
        }
      }
    }

    // Restore parent schema.
    ctx.schema = parentSchema;

    // Ensure no React warnings are triggered for void elements having children
    // passed in.
    return nodes.length > 0
      ? h.call(node, name, attributes, nodes)
      : h.call(node, name, attributes)
  }

  /**
   * @param {Record<string, unknown>} props
   * @param {string} prop
   * @param {unknown} value
   * @param {Context} ctx
   * @param {string} name
   */
  // eslint-disable-next-line complexity, max-params
  function addAttribute(props, prop, value, ctx, name) {
    const info = find$3(ctx.schema, prop);
    /** @type {string|undefined} */
    let subprop;

    // Ignore nullish and `NaN` values.
    // Ignore `false` and falsey known booleans for hyperlike DSLs.
    if (
      value === undefined ||
      value === null ||
      (typeof value === 'number' && Number.isNaN(value)) ||
      (value === false && (ctx.vue || ctx.vdom || ctx.hyperscript)) ||
      (!value && info.boolean && (ctx.vue || ctx.vdom || ctx.hyperscript))
    ) {
      return
    }

    if (Array.isArray(value)) {
      // Accept `array`.
      // Most props are space-separated.
      value = info.commaSeparated ? stringify$5(value) : stringify$6(value);
    }

    // Treat `true` and truthy known booleans.
    if (info.boolean && ctx.hyperscript) {
      value = '';
    }

    // VDOM, Vue, and React accept `style` as object.
    if (
      info.property === 'style' &&
      typeof value === 'string' &&
      (ctx.react || ctx.vue || ctx.vdom)
    ) {
      value = parseStyle(value, name);
    }

    if (ctx.vue) {
      if (info.property !== 'style') subprop = 'attrs';
    } else if (!info.mustUseProperty) {
      if (ctx.vdom) {
        if (info.property !== 'style') subprop = 'attributes';
      } else if (ctx.hyperscript) {
        subprop = 'attrs';
      }
    }

    if (subprop) {
      props[subprop] = Object.assign(props[subprop] || {}, {
        [info.attribute]: value
      });
    } else if (info.space && ctx.react) {
      props[toReact[info.property] || info.property] = value;
    } else {
      props[info.attribute] = value;
    }
  }

  /**
   * Check if `h` is `react.createElement`.
   *
   * @param {CreateElementLike} h
   * @returns {boolean}
   */
  function react(h) {
    /** @type {unknown} */
    const node = h('div', {});
    return Boolean(
      node &&
        // @ts-expect-error Looks like a React node.
        ('_owner' in node || '_store' in node) &&
        // @ts-expect-error Looks like a React node.
        (node.key === undefined || node.key === null)
    )
  }

  /**
   * Check if `h` is `hyperscript`.
   *
   * @param {CreateElementLike} h
   * @returns {boolean}
   */
  function hyperscript(h) {
    return 'context' in h && 'cleanup' in h
  }

  /**
   * Check if `h` is `virtual-dom/h`.
   *
   * @param {CreateElementLike} h
   * @returns {boolean}
   */
  function vdom(h) {
    /** @type {unknown} */
    const node = h('div', {});
    // @ts-expect-error Looks like a vnode.
    return node.type === 'VirtualNode'
  }

  /**
   * Check if `h` is Vue.
   *
   * @param {CreateElementLike} h
   * @returns {boolean}
   */
  function vue(h) {
    /** @type {unknown} */
    const node = h('div', {});
    // @ts-expect-error Looks like a Vue node.
    return Boolean(node && node.context && node.context._isVue)
  }

  /**
   * @param {string} value
   * @param {string} tagName
   * @returns {Record<string, string>}
   */
  function parseStyle(value, tagName) {
    /** @type {Record<string, string>} */
    const result = {};

    try {
      styleToObject(value, (name, value) => {
        if (name.slice(0, 4) === '-ms-') name = 'ms-' + name.slice(4);

        result[
          name.replace(
            /-([a-z])/g,
            /**
             * @param {string} _
             * @param {string} $1
             * @returns {string}
             */ (_, $1) => $1.toUpperCase()
          )
        ] = value;
      });
    } catch (error) {
      error.message =
        tagName + '[style]' + error.message.slice('undefined'.length);
      throw error
    }

    return result
  }

  /**
   * Map of web namespaces.
   *
   * @type {Record<string, string>}
   */
  const webNamespaces = {
    html: 'http://www.w3.org/1999/xhtml',
    mathml: 'http://www.w3.org/1998/Math/MathML',
    svg: 'http://www.w3.org/2000/svg',
    xlink: 'http://www.w3.org/1999/xlink',
    xml: 'http://www.w3.org/XML/1998/namespace',
    xmlns: 'http://www.w3.org/2000/xmlns/'
  };

  var own$c = {}.hasOwnProperty;

  /**
   * @callback Handler
   * @param {...unknown} value
   * @return {unknown}
   *
   * @typedef {Record<string, Handler>} Handlers
   *
   * @typedef {Object} Options
   * @property {Handler} [unknown]
   * @property {Handler} [invalid]
   * @property {Handlers} [handlers]
   */

  /**
   * Handle values based on a property.
   *
   * @param {string} key
   * @param {Options} [options]
   */
  function zwitch(key, options) {
    var settings = options || {};

    /**
     * Handle one value.
     * Based on the bound `key`, a respective handler will be called.
     * If `value` is not an object, or doesn’t have a `key` property, the special
     * “invalid” handler will be called.
     * If `value` has an unknown `key`, the special “unknown” handler will be
     * called.
     *
     * All arguments, and the context object, are passed through to the handler,
     * and it’s result is returned.
     *
     * @param {...unknown} [value]
     * @this {unknown}
     * @returns {unknown}
     * @property {Handler} invalid
     * @property {Handler} unknown
     * @property {Handlers} handlers
     */
    function one(value) {
      var fn = one.invalid;
      var handlers = one.handlers;

      if (value && own$c.call(value, key)) {
        fn = own$c.call(handlers, value[key]) ? handlers[value[key]] : one.unknown;
      }

      if (fn) {
        return fn.apply(this, arguments)
      }
    }

    one.handlers = settings.handlers || {};
    one.invalid = settings.invalid;
    one.unknown = settings.unknown;

    return one
  }

  /**
   * @typedef {import('parse5').Node} P5Node
   * @typedef {import('parse5').Document} P5Document
   * @typedef {import('parse5').DocumentFragment} P5Fragment
   * @typedef {import('parse5').DocumentType} P5Doctype
   * @typedef {import('parse5').CommentNode} P5Comment
   * @typedef {import('parse5').TextNode} P5Text
   * @typedef {import('parse5').Element} P5Element
   * @typedef {import('parse5').Attribute} P5Attribute
   * @typedef {import('parse5').ParentNode} P5Parent
   * @typedef {Exclude<P5Node, P5Document|P5Fragment>} P5Child
   * @typedef {import('property-information').Schema} Schema
   * @typedef {import('property-information').Info} Info
   * @typedef {'html'|'svg'} Space
   * @typedef {import('hast').Parent} Parent
   * @typedef {import('hast').Root} Root
   * @typedef {import('hast').DocType} Doctype
   * @typedef {import('hast').Element} Element
   * @typedef {import('hast').Text} Text
   * @typedef {import('hast').Comment} Comment
   * @typedef {Parent['children'][number]} Child
   * @typedef {Child|Root} Node
   *
   * @callback Handle
   * @param {Node} node
   * @param {Schema} schema
   * @returns {P5Node}
   */

  var own$b = {}.hasOwnProperty;

  var one$2 = zwitch('type', {handlers: {root: root$1, element: element$4, text: text$1, comment: comment$2, doctype: doctype$1}});

  /**
   * Transform a tree from hast to Parse5’s AST.
   *
   * @param {Node} tree
   * @param {Space} [space='html']
   * @returns {P5Node}
   */
  function toParse5(tree, space) {
    // @ts-ignore Types are wrong.
    return one$2(tree, space === 'svg' ? svg$7 : html$a)
  }

  /**
   * @type {Handle}
   * @param {Root} node
   * @returns {P5Document}
   */
  function root$1(node, schema) {
    /** @type {P5Document} */
    var p5 = {
      nodeName: '#document',
      mode: (node.data || {}).quirksMode ? 'quirks' : 'no-quirks',
      childNodes: []
    };
    // @ts-ignore Assume correct children.
    p5.childNodes = all$2(node.children, p5, schema);
    return patch(node, p5)
  }

  /**
   * @type {Handle}
   * @param {Root} node
   * @returns {P5Fragment}
   */
  function fragment(node, schema) {
    /** @type {P5Fragment} */
    var p5 = {nodeName: '#document-fragment', childNodes: []};
    // @ts-ignore Assume correct children.
    p5.childNodes = all$2(node.children, p5, schema);
    return patch(node, p5)
  }

  /**
   * @type {Handle}
   * @param {Doctype} node
   * @returns {P5Doctype}
   */
  function doctype$1(node) {
    return patch(node, {
      nodeName: '#documentType',
      name: 'html',
      publicId: '',
      systemId: '',
      parentNode: undefined
    })
  }

  /**
   * @type {Handle}
   * @param {Text} node
   * @returns {P5Text}
   */
  function text$1(node) {
    return patch(node, {
      nodeName: '#text',
      value: node.value,
      parentNode: undefined
    })
  }

  /**
   * @type {Handle}
   * @param {Comment} node
   * @returns {P5Comment}
   */
  function comment$2(node) {
    return patch(node, {
      nodeName: '#comment',
      data: node.value,
      parentNode: undefined
    })
  }

  /**
   * @type {Handle}
   * @param {Element} node
   * @returns {P5Element}
   */
  function element$4(node, schema) {
    /** @type {Space} */
    // @ts-ignore Assume space.
    var space = schema.space;
    return toH(h, Object.assign({}, node, {children: []}), {space})

    /**
     * @param {string} name
     * @param {Object.<string, string|boolean|number>} attrs
     */
    function h(name, attrs) {
      /** @type {Array.<P5Attribute>} */
      var values = [];
      /** @type {Info} */
      var info;
      /** @type {P5Attribute} */
      var value;
      /** @type {string} */
      var key;
      /** @type {number} */
      var index;
      /** @type {P5Element} */
      var p5;

      for (key in attrs) {
        if (!own$b.call(attrs, key) || attrs[key] === false) {
          continue
        }

        info = find$4(schema, key);

        if (info.boolean && !attrs[key]) {
          continue
        }

        value = {name: key, value: attrs[key] === true ? '' : String(attrs[key])};

        if (info.space && info.space !== 'html' && info.space !== 'svg') {
          index = key.indexOf(':');

          if (index < 0) {
            value.prefix = '';
          } else {
            value.name = key.slice(index + 1);
            value.prefix = key.slice(0, index);
          }

          value.namespace = webNamespaces[info.space];
        }

        values.push(value);
      }

      if (schema.space === 'html' && node.tagName === 'svg') schema = svg$7;

      p5 = patch(node, {
        nodeName: name,
        tagName: name,
        attrs: values,
        namespaceURI: webNamespaces[schema.space],
        childNodes: [],
        parentNode: undefined
      });

      // @ts-ignore Assume correct children.
      p5.childNodes = all$2(node.children, p5, schema);

      // @ts-ignore Types are wrong.
      if (name === 'template') p5.content = fragment(node.content, schema);

      return p5
    }
  }

  /**
   * @param {Array.<Child>} children
   * @param {P5Parent} p5
   * @param {Schema} schema
   * @returns {Array.<P5Child>}
   */
  function all$2(children, p5, schema) {
    var index = -1;
    /** @type {Array.<P5Child>} */
    var result = [];
    /** @type {P5Child} */
    var child;

    if (children) {
      while (++index < children.length) {
        // @ts-ignore Assume child.
        child = one$2(children[index], schema);

        // @ts-ignore types are wrong.
        child.parentNode = p5;

        result.push(child);
      }
    }

    return result
  }

  /**
   * Patch specific properties.
   *
   * @template {P5Node} T
   * @param {Node} node
   * @param {T} p5
   * @returns {T}
   */
  function patch(node, p5) {
    var position = node.position;

    if (position && position.start && position.end) {
      // @ts-ignore Types are wrong.
      p5.sourceCodeLocation = {
        startLine: position.start.line,
        startCol: position.start.column,
        startOffset: position.start.offset,
        endLine: position.end.line,
        endCol: position.end.column,
        endOffset: position.end.offset
      };
    }

    return p5
  }

  /**
   * List of HTML void tag names.
   *
   * @type {Array<string>}
   */
  const htmlVoidElements = [
    'area',
    'base',
    'basefont',
    'bgsound',
    'br',
    'col',
    'command',
    'embed',
    'frame',
    'hr',
    'image',
    'img',
    'input',
    'isindex',
    'keygen',
    'link',
    'menuitem',
    'meta',
    'nextid',
    'param',
    'source',
    'track',
    'wbr'
  ];

  /**
   * @typedef {import('vfile').VFile} VFile
   * @typedef {import('parse5').Document} P5Document
   * @typedef {import('parse5').DocumentFragment} P5Fragment
   * @typedef {Omit<import('parse5').Element, 'parentNode'>} P5Element
   * @typedef {import('parse5').Attribute} P5Attribute
   * @typedef {Omit<import('parse5').Location, 'startOffset' | 'endOffset'> & {startOffset: number|undefined, endOffset: number|undefined}} P5Location
   * @typedef {import('parse5').ParserOptions} P5ParserOptions
   * @typedef {import('hast').Root} Root
   * @typedef {import('hast').DocType} Doctype
   * @typedef {import('hast').Element} Element
   * @typedef {import('hast').Text} Text
   * @typedef {import('hast').Comment} Comment
   * @typedef {import('hast').Content} Content
   * @typedef {Root|Content} Node
   * @typedef {import('../complex-types').Raw} Raw
   *
   * @typedef {Omit<Comment, 'value'> & {value: {stitch: Node}}} Stitch
   *
   * @typedef Options
   * @property {Array.<string>} [passThrough]
   *   List of custom hast node types to pass through (keep) in hast.
   *   If the passed through nodes have children, those children are expected to
   *   be hast and will be handled.
   *
   * @typedef HiddenTokenizer
   * @property {Array.<HiddenLocationTracker>} __mixins
   *   Way too simple, but works for us.
   * @property {HiddenPreprocessor} preprocessor
   * @property {(value: string) => void} write
   * @property {() => number} _consume
   * @property {Array.<HiddenToken>} tokenQueue
   * @property {string} state
   * @property {string} returnState
   * @property {number} charRefCode
   * @property {Array.<number>} tempBuff
   * @property {Function} _flushCodePointsConsumedAsCharacterReference
   * @property {string} lastStartTagName
   * @property {number} consumedAfterSnapshot
   * @property {boolean} active
   * @property {HiddenToken|undefined} currentCharacterToken
   * @property {HiddenToken|undefined} currentToken
   * @property {unknown} currentAttr
   * @property {Function} NAMED_CHARACTER_REFERENCE_STATE
   * @property {Function} NUMERIC_CHARACTER_REFERENCE_END_STATE
   *
   * @typedef {Object.<string, unknown> & {location: P5Location}} HiddenToken
   *
   * @typedef HiddenPreprocessor
   * @property {string|undefined} html
   * @property {number} pos
   * @property {number} lastGapPos
   * @property {number} lastCharPos
   * @property {Array.<number>} gapStack
   * @property {boolean} skipNextNewLine
   * @property {boolean} lastChunkWritten
   * @property {boolean} endOfChunkHit
   *
   * @typedef HiddenLocationTracker
   * @property {P5Location|undefined} currentAttrLocation
   * @property {P5Location} ctLoc
   * @property {HiddenPosTracker} posTracker
   *
   * @typedef HiddenPosTracker
   * @property {boolean} isEol
   * @property {number} lineStartPos
   * @property {number} droppedBufferSize
   * @property {number} offset
   * @property {number} col
   * @property {number} line
   */

  const inTemplateMode = 'IN_TEMPLATE_MODE';
  const dataState = 'DATA_STATE';
  const characterToken = 'CHARACTER_TOKEN';
  const startTagToken = 'START_TAG_TOKEN';
  const endTagToken = 'END_TAG_TOKEN';
  const commentToken = 'COMMENT_TOKEN';
  const doctypeToken = 'DOCTYPE_TOKEN';

  /** @type {P5ParserOptions} */
  const parseOptions = {sourceCodeLocationInfo: true, scriptingEnabled: false};

  /**
   * Given a hast tree and an optional vfile (for positional info), return a new
   * parsed-again hast tree.
   *
   * @param tree
   *   Original hast tree.
   * @param file
   *   Virtual file for positional info, optional.
   * @param options
   *   Configuration.
   */
  const raw$1 =
    /**
     * @type {(
     *   ((tree: Node, file: VFile|undefined, options?: Options) => Node) &
     *   ((tree: Node, options?: Options) => Node)
     * )}
     */
    (
      /**
       * @param {Node} tree
       * @param {VFile} [file]
       * @param {Options} [options]
       */
      function (tree, file, options) {
        let index = -1;
        const parser = new parser$2(parseOptions);
        const one = zwitch('type', {
          // @ts-expect-error: hush.
          handlers: {root, element, text, comment, doctype, raw: handleRaw},
          // @ts-expect-error: hush.
          unknown
        });
        /** @type {boolean|undefined} */
        let stitches;
        /** @type {HiddenTokenizer|undefined} */
        let tokenizer;
        /** @type {HiddenPreprocessor|undefined} */
        let preprocessor;
        /** @type {HiddenPosTracker|undefined} */
        let posTracker;
        /** @type {HiddenLocationTracker|undefined} */
        let locationTracker;

        if (isOptions(file)) {
          options = file;
          file = undefined;
        }

        if (options && options.passThrough) {
          while (++index < options.passThrough.length) {
            // @ts-expect-error: hush.
            one.handlers[options.passThrough[index]] = stitch;
          }
        }

        const result = fromParse5(
          documentMode(tree) ? document() : fragment(),
          file
        );

        if (stitches) {
          visit$2(result, 'comment', (node, index, parent) => {
            const stitch = /** @type {Stitch} */ (/** @type {unknown} */ (node));
            if (stitch.value.stitch && parent !== null && index !== null) {
              // @ts-expect-error: assume the stitch is allowed.
              parent.children[index] = stitch.value.stitch;
              return index
            }
          });
        }

        // Unpack if possible and when not given a `root`.
        if (
          tree.type !== 'root' &&
          result.type === 'root' &&
          result.children.length === 1
        ) {
          return result.children[0]
        }

        return result

        /**
         * @returns {P5Fragment}
         */
        function fragment() {
          /** @type {P5Element} */
          const context = {
            nodeName: 'template',
            tagName: 'template',
            attrs: [],
            namespaceURI: webNamespaces$2.html,
            childNodes: []
          };
          /** @type {P5Element} */
          const mock = {
            nodeName: 'documentmock',
            tagName: 'documentmock',
            attrs: [],
            namespaceURI: webNamespaces$2.html,
            childNodes: []
          };
          /** @type {P5Fragment} */
          const doc = {nodeName: '#document-fragment', childNodes: []};

          parser._bootstrap(mock, context);
          parser._pushTmplInsertionMode(inTemplateMode);
          parser._initTokenizerForFragmentParsing();
          parser._insertFakeRootElement();
          parser._resetInsertionMode();
          parser._findFormInFragmentContext();

          tokenizer = parser.tokenizer;
          /* c8 ignore next */
          if (!tokenizer) throw new Error('Expected `tokenizer`')
          preprocessor = tokenizer.preprocessor;
          locationTracker = tokenizer.__mixins[0];
          posTracker = locationTracker.posTracker;

          one(tree);

          parser._adoptNodes(mock.childNodes[0], doc);

          return doc
        }

        /**
         * @returns {P5Document}
         */
        function document() {
          /** @type {P5Document} */
          const doc = parser.treeAdapter.createDocument();

          parser._bootstrap(doc, undefined);
          tokenizer = parser.tokenizer;
          /* c8 ignore next */
          if (!tokenizer) throw new Error('Expected `tokenizer`')
          preprocessor = tokenizer.preprocessor;
          locationTracker = tokenizer.__mixins[0];
          posTracker = locationTracker.posTracker;

          one(tree);

          return doc
        }

        /**
         * @param {Content[]} nodes
         * @returns {void}
         */
        function all(nodes) {
          let index = -1;

          /* istanbul ignore else - invalid nodes, see rehypejs/rehype-raw#7. */
          if (nodes) {
            while (++index < nodes.length) {
              one(nodes[index]);
            }
          }
        }

        /**
         * @param {Root} node
         * @returns {void}
         */
        function root(node) {
          all(node.children);
        }

        /**
         * @param {Element} node
         * @returns {void}
         */
        function element(node) {
          resetTokenizer();
          parser._processToken(startTag(node), webNamespaces$2.html);

          all(node.children);

          if (!htmlVoidElements.includes(node.tagName)) {
            resetTokenizer();
            parser._processToken(endTag(node));
          }
        }

        /**
         * @param {Text} node
         * @returns {void}
         */
        function text(node) {
          resetTokenizer();
          parser._processToken({
            type: characterToken,
            chars: node.value,
            location: createParse5Location(node)
          });
        }

        /**
         * @param {Doctype} node
         * @returns {void}
         */
        function doctype(node) {
          resetTokenizer();
          parser._processToken({
            type: doctypeToken,
            name: 'html',
            forceQuirks: false,
            publicId: '',
            systemId: '',
            location: createParse5Location(node)
          });
        }

        /**
         * @param {Comment|Stitch} node
         * @returns {void}
         */
        function comment(node) {
          resetTokenizer();
          parser._processToken({
            type: commentToken,
            data: node.value,
            location: createParse5Location(node)
          });
        }

        /**
         * @param {Raw} node
         * @returns {void}
         */
        function handleRaw(node) {
          const start = pointStart(node);
          const line = start.line || 1;
          const column = start.column || 1;
          const offset = start.offset || 0;

          /* c8 ignore next 4 */
          if (!preprocessor) throw new Error('Expected `preprocessor`')
          if (!tokenizer) throw new Error('Expected `tokenizer`')
          if (!posTracker) throw new Error('Expected `posTracker`')
          if (!locationTracker) throw new Error('Expected `locationTracker`')

          // Reset preprocessor:
          // See: <https://github.com/inikulin/parse5/blob/9c683e1/packages/parse5/lib/tokenizer/preprocessor.js>.
          preprocessor.html = undefined;
          preprocessor.pos = -1;
          preprocessor.lastGapPos = -1;
          preprocessor.lastCharPos = -1;
          preprocessor.gapStack = [];
          preprocessor.skipNextNewLine = false;
          preprocessor.lastChunkWritten = false;
          preprocessor.endOfChunkHit = false;

          // Reset preprocessor mixin:
          // See: <https://github.com/inikulin/parse5/blob/9c683e1/packages/parse5/lib/extensions/position-tracking/preprocessor-mixin.js>.
          posTracker.isEol = false;
          posTracker.lineStartPos = -column + 1; // Looks weird, but ensures we get correct positional info.
          posTracker.droppedBufferSize = offset;
          posTracker.offset = 0;
          posTracker.col = 1;
          posTracker.line = line;

          // Reset location tracker:
          // See: <https://github.com/inikulin/parse5/blob/9c683e1/packages/parse5/lib/extensions/location-info/tokenizer-mixin.js>.
          locationTracker.currentAttrLocation = undefined;
          locationTracker.ctLoc = createParse5Location(node);

          // See the code for `parse` and `parseFragment`:
          // See: <https://github.com/inikulin/parse5/blob/9c683e1/packages/parse5/lib/parser/index.js#L371>.
          tokenizer.write(node.value);
          parser._runParsingLoop(null);

          // Character references hang, so if we ended there, we need to flush
          // those too.
          // We reset the preprocessor as if the document ends here.
          // Then one single call to the relevant state does the trick, parse5
          // consumes the whole token.
          if (
            tokenizer.state === 'NAMED_CHARACTER_REFERENCE_STATE' ||
            tokenizer.state === 'NUMERIC_CHARACTER_REFERENCE_END_STATE'
          ) {
            preprocessor.lastChunkWritten = true;
            tokenizer[tokenizer.state](tokenizer._consume());
          }

          // Process final characters if they’re still there after hibernating.
          // Similar to:
          // See: <https://github.com/inikulin/parse5/blob/9c683e1/packages/parse5/lib/extensions/location-info/tokenizer-mixin.js#L95>.
          const token = tokenizer.currentCharacterToken;

          if (token) {
            token.location.endLine = posTracker.line;
            token.location.endCol = posTracker.col + 1;
            token.location.endOffset = posTracker.offset + 1;
            parser._processToken(token);
          }
        }

        /**
         * @param {Node} node
         */
        function stitch(node) {
          stitches = true;

          /** @type {Node} */
          let clone;

          // Recurse, because to somewhat handle `[<x>]</x>` (where `[]` denotes the
          // passed through node).
          if ('children' in node) {
            clone = {
              ...node,
              children: raw$1(
                {type: 'root', children: node.children},
                file,
                options
                // @ts-expect-error Assume a given parent yields a parent.
              ).children
            };
          } else {
            clone = {...node};
          }

          // Hack: `value` is supposed to be a string, but as none of the tools
          // (`parse5` or `hast-util-from-parse5`) looks at it, we can pass nodes
          // through.
          comment({type: 'comment', value: {stitch: clone}});
        }

        function resetTokenizer() {
          /* c8 ignore next */
          if (!tokenizer) throw new Error('Expected `tokenizer`')

          // Reset tokenizer:
          // See: <https://github.com/inikulin/parse5/blob/9c683e1/packages/parse5/lib/tokenizer/index.js#L218-L234>.
          // Especially putting it back in the `data` state is useful: some elements,
          // like textareas and iframes, change the state.
          // See GH-7.
          // But also if broken HTML is in `raw`, and then a correct element is given.
          // See GH-11.
          tokenizer.tokenQueue = [];
          tokenizer.state = dataState;
          tokenizer.returnState = '';
          tokenizer.charRefCode = -1;
          tokenizer.tempBuff = [];
          tokenizer.lastStartTagName = '';
          tokenizer.consumedAfterSnapshot = -1;
          tokenizer.active = false;
          tokenizer.currentCharacterToken = undefined;
          tokenizer.currentToken = undefined;
          tokenizer.currentAttr = undefined;
        }
      }
    );
  /**
   * @param {Element} node
   * @returns {HiddenToken}
   */
  function startTag(node) {
    /** @type {P5Location} */
    const location = Object.assign(createParse5Location(node));
    // @ts-expect-error extra positional info.
    location.startTag = Object.assign({}, location);

    // Untyped token.
    return {
      type: startTagToken,
      tagName: node.tagName,
      selfClosing: false,
      attrs: attributes$1(node),
      location
    }
  }

  /**
   * @param {Element} node
   * @returns {Array.<P5Attribute>}
   */
  function attributes$1(node) {
    return toParse5({
      tagName: node.tagName,
      type: 'element',
      properties: node.properties,
      children: []
      // @ts-expect-error Assume element.
    }).attrs
  }

  /**
   * @param {Element} node
   * @returns {HiddenToken}
   */
  function endTag(node) {
    /** @type {P5Location} */
    const location = Object.assign(createParse5Location(node));
    // @ts-expect-error extra positional info.
    location.startTag = Object.assign({}, location);

    // Untyped token.
    return {
      type: endTagToken,
      tagName: node.tagName,
      attrs: [],
      location
    }
  }

  /**
   * @param {Node} node
   */
  function unknown(node) {
    throw new Error('Cannot compile `' + node.type + '` node')
  }

  /**
   * @param {Node} node
   * @returns {boolean}
   */
  function documentMode(node) {
    const head = node.type === 'root' ? node.children[0] : node;
    return Boolean(
      head &&
        (head.type === 'doctype' ||
          (head.type === 'element' && head.tagName === 'html'))
    )
  }

  /**
   * @param {Node|Stitch} node
   * @returns {P5Location}
   */
  function createParse5Location(node) {
    const start = pointStart(node);
    const end = pointEnd(node);

    return {
      startLine: start.line,
      startCol: start.column,
      startOffset: start.offset,
      endLine: end.line,
      endCol: end.column,
      endOffset: end.offset
    }
  }

  /**
   * @param {VFile|Options|undefined} value
   * @return {value is Options}
   */
  function isOptions(value) {
    return Boolean(value && !('message' in value && 'messages' in value))
  }

  /**
   * @typedef {import('hast').Root} Root
   * @typedef {import('hast-util-raw').Options} Options
   * @typedef {import('hast-util-raw')} DoNotTouchAsThisImportIncludesRawInTree
   */

  /**
   * Plugin to parse the tree again (and raw nodes).
   * Keeping positional info OK.  🙌
   *
   * @type {import('unified').Plugin<[Options?] | Array<void>, Root>}
   */
  function rehypeRaw(options = {}) {
    return (tree, file) => {
      // Assume that when a root was given, it’s also returned.
      const result = /** @type {Root} */ (raw$1(tree, file, options));
      return result
    }
  }

  /**
   * @typedef {import('./info.js').Info} Info
   * @typedef {Record<string, Info>} Properties
   * @typedef {Record<string, string>} Normal
   */

  class Schema$2 {
    /**
     * @constructor
     * @param {Properties} property
     * @param {Normal} normal
     * @param {string} [space]
     */
    constructor(property, normal, space) {
      this.property = property;
      this.normal = normal;
      if (space) {
        this.space = space;
      }
    }
  }

  /** @type {Properties} */
  Schema$2.prototype.property = {};
  /** @type {Normal} */
  Schema$2.prototype.normal = {};
  /** @type {string|null} */
  Schema$2.prototype.space = null;

  /**
   * @typedef {import('./schema.js').Properties} Properties
   * @typedef {import('./schema.js').Normal} Normal
   */

  /**
   * @param {Schema[]} definitions
   * @param {string} [space]
   * @returns {Schema}
   */
  function merge$2(definitions, space) {
    /** @type {Properties} */
    const property = {};
    /** @type {Normal} */
    const normal = {};
    let index = -1;

    while (++index < definitions.length) {
      Object.assign(property, definitions[index].property);
      Object.assign(normal, definitions[index].normal);
    }

    return new Schema$2(property, normal, space)
  }

  /**
   * @param {string} value
   * @returns {string}
   */
  function normalize$2(value) {
    return value.toLowerCase()
  }

  class Info$2 {
    /**
     * @constructor
     * @param {string} property
     * @param {string} attribute
     */
    constructor(property, attribute) {
      /** @type {string} */
      this.property = property;
      /** @type {string} */
      this.attribute = attribute;
    }
  }

  /** @type {string|null} */
  Info$2.prototype.space = null;
  Info$2.prototype.boolean = false;
  Info$2.prototype.booleanish = false;
  Info$2.prototype.overloadedBoolean = false;
  Info$2.prototype.number = false;
  Info$2.prototype.commaSeparated = false;
  Info$2.prototype.spaceSeparated = false;
  Info$2.prototype.commaOrSpaceSeparated = false;
  Info$2.prototype.mustUseProperty = false;
  Info$2.prototype.defined = false;

  let powers$2 = 0;

  const boolean$2 = increment$2();
  const booleanish$2 = increment$2();
  const overloadedBoolean$2 = increment$2();
  const number$2 = increment$2();
  const spaceSeparated$2 = increment$2();
  const commaSeparated$2 = increment$2();
  const commaOrSpaceSeparated$2 = increment$2();

  function increment$2() {
    return 2 ** ++powers$2
  }

  var types$2 = /*#__PURE__*/Object.freeze({
      __proto__: null,
      boolean: boolean$2,
      booleanish: booleanish$2,
      overloadedBoolean: overloadedBoolean$2,
      number: number$2,
      spaceSeparated: spaceSeparated$2,
      commaSeparated: commaSeparated$2,
      commaOrSpaceSeparated: commaOrSpaceSeparated$2
  });

  /** @type {Array<keyof types>} */
  // @ts-expect-error: hush.
  const checks$2 = Object.keys(types$2);

  class DefinedInfo$2 extends Info$2 {
    /**
     * @constructor
     * @param {string} property
     * @param {string} attribute
     * @param {number|null} [mask]
     * @param {string} [space]
     */
    constructor(property, attribute, mask, space) {
      let index = -1;

      super(property, attribute);

      mark$2(this, 'space', space);

      if (typeof mask === 'number') {
        while (++index < checks$2.length) {
          const check = checks$2[index];
          mark$2(this, checks$2[index], (mask & types$2[check]) === types$2[check]);
        }
      }
    }
  }

  DefinedInfo$2.prototype.defined = true;

  /**
   * @param {DefinedInfo} values
   * @param {string} key
   * @param {unknown} value
   */
  function mark$2(values, key, value) {
    if (value) {
      // @ts-expect-error: assume `value` matches the expected value of `key`.
      values[key] = value;
    }
  }

  /**
   * @typedef {import('./schema.js').Properties} Properties
   * @typedef {import('./schema.js').Normal} Normal
   *
   * @typedef {Record<string, string>} Attributes
   *
   * @typedef {Object} Definition
   * @property {Record<string, number|null>} properties
   * @property {(attributes: Attributes, property: string) => string} transform
   * @property {string} [space]
   * @property {Attributes} [attributes]
   * @property {Array<string>} [mustUseProperty]
   */

  const own$a = {}.hasOwnProperty;

  /**
   * @param {Definition} definition
   * @returns {Schema}
   */
  function create$2(definition) {
    /** @type {Properties} */
    const property = {};
    /** @type {Normal} */
    const normal = {};
    /** @type {string} */
    let prop;

    for (prop in definition.properties) {
      if (own$a.call(definition.properties, prop)) {
        const value = definition.properties[prop];
        const info = new DefinedInfo$2(
          prop,
          definition.transform(definition.attributes || {}, prop),
          value,
          definition.space
        );

        if (
          definition.mustUseProperty &&
          definition.mustUseProperty.includes(prop)
        ) {
          info.mustUseProperty = true;
        }

        property[prop] = info;

        normal[normalize$2(prop)] = prop;
        normal[normalize$2(info.attribute)] = prop;
      }
    }

    return new Schema$2(property, normal, definition.space)
  }

  const xlink$2 = create$2({
    space: 'xlink',
    transform(_, prop) {
      return 'xlink:' + prop.slice(5).toLowerCase()
    },
    properties: {
      xLinkActuate: null,
      xLinkArcRole: null,
      xLinkHref: null,
      xLinkRole: null,
      xLinkShow: null,
      xLinkTitle: null,
      xLinkType: null
    }
  });

  const xml$2 = create$2({
    space: 'xml',
    transform(_, prop) {
      return 'xml:' + prop.slice(3).toLowerCase()
    },
    properties: {xmlLang: null, xmlBase: null, xmlSpace: null}
  });

  /**
   * @param {Record<string, string>} attributes
   * @param {string} attribute
   * @returns {string}
   */
  function caseSensitiveTransform$2(attributes, attribute) {
    return attribute in attributes ? attributes[attribute] : attribute
  }

  /**
   * @param {Record<string, string>} attributes
   * @param {string} property
   * @returns {string}
   */
  function caseInsensitiveTransform$2(attributes, property) {
    return caseSensitiveTransform$2(attributes, property.toLowerCase())
  }

  const xmlns$2 = create$2({
    space: 'xmlns',
    attributes: {xmlnsxlink: 'xmlns:xlink'},
    transform: caseInsensitiveTransform$2,
    properties: {xmlns: null, xmlnsXLink: null}
  });

  const aria$2 = create$2({
    transform(_, prop) {
      return prop === 'role' ? prop : 'aria-' + prop.slice(4).toLowerCase()
    },
    properties: {
      ariaActiveDescendant: null,
      ariaAtomic: booleanish$2,
      ariaAutoComplete: null,
      ariaBusy: booleanish$2,
      ariaChecked: booleanish$2,
      ariaColCount: number$2,
      ariaColIndex: number$2,
      ariaColSpan: number$2,
      ariaControls: spaceSeparated$2,
      ariaCurrent: null,
      ariaDescribedBy: spaceSeparated$2,
      ariaDetails: null,
      ariaDisabled: booleanish$2,
      ariaDropEffect: spaceSeparated$2,
      ariaErrorMessage: null,
      ariaExpanded: booleanish$2,
      ariaFlowTo: spaceSeparated$2,
      ariaGrabbed: booleanish$2,
      ariaHasPopup: null,
      ariaHidden: booleanish$2,
      ariaInvalid: null,
      ariaKeyShortcuts: null,
      ariaLabel: null,
      ariaLabelledBy: spaceSeparated$2,
      ariaLevel: number$2,
      ariaLive: null,
      ariaModal: booleanish$2,
      ariaMultiLine: booleanish$2,
      ariaMultiSelectable: booleanish$2,
      ariaOrientation: null,
      ariaOwns: spaceSeparated$2,
      ariaPlaceholder: null,
      ariaPosInSet: number$2,
      ariaPressed: booleanish$2,
      ariaReadOnly: booleanish$2,
      ariaRelevant: null,
      ariaRequired: booleanish$2,
      ariaRoleDescription: spaceSeparated$2,
      ariaRowCount: number$2,
      ariaRowIndex: number$2,
      ariaRowSpan: number$2,
      ariaSelected: booleanish$2,
      ariaSetSize: number$2,
      ariaSort: null,
      ariaValueMax: number$2,
      ariaValueMin: number$2,
      ariaValueNow: number$2,
      ariaValueText: null,
      role: null
    }
  });

  const html$7 = create$2({
    space: 'html',
    attributes: {
      acceptcharset: 'accept-charset',
      classname: 'class',
      htmlfor: 'for',
      httpequiv: 'http-equiv'
    },
    transform: caseInsensitiveTransform$2,
    mustUseProperty: ['checked', 'multiple', 'muted', 'selected'],
    properties: {
      // Standard Properties.
      abbr: null,
      accept: commaSeparated$2,
      acceptCharset: spaceSeparated$2,
      accessKey: spaceSeparated$2,
      action: null,
      allow: null,
      allowFullScreen: boolean$2,
      allowPaymentRequest: boolean$2,
      allowUserMedia: boolean$2,
      alt: null,
      as: null,
      async: boolean$2,
      autoCapitalize: null,
      autoComplete: spaceSeparated$2,
      autoFocus: boolean$2,
      autoPlay: boolean$2,
      capture: boolean$2,
      charSet: null,
      checked: boolean$2,
      cite: null,
      className: spaceSeparated$2,
      cols: number$2,
      colSpan: null,
      content: null,
      contentEditable: booleanish$2,
      controls: boolean$2,
      controlsList: spaceSeparated$2,
      coords: number$2 | commaSeparated$2,
      crossOrigin: null,
      data: null,
      dateTime: null,
      decoding: null,
      default: boolean$2,
      defer: boolean$2,
      dir: null,
      dirName: null,
      disabled: boolean$2,
      download: overloadedBoolean$2,
      draggable: booleanish$2,
      encType: null,
      enterKeyHint: null,
      form: null,
      formAction: null,
      formEncType: null,
      formMethod: null,
      formNoValidate: boolean$2,
      formTarget: null,
      headers: spaceSeparated$2,
      height: number$2,
      hidden: boolean$2,
      high: number$2,
      href: null,
      hrefLang: null,
      htmlFor: spaceSeparated$2,
      httpEquiv: spaceSeparated$2,
      id: null,
      imageSizes: null,
      imageSrcSet: null,
      inputMode: null,
      integrity: null,
      is: null,
      isMap: boolean$2,
      itemId: null,
      itemProp: spaceSeparated$2,
      itemRef: spaceSeparated$2,
      itemScope: boolean$2,
      itemType: spaceSeparated$2,
      kind: null,
      label: null,
      lang: null,
      language: null,
      list: null,
      loading: null,
      loop: boolean$2,
      low: number$2,
      manifest: null,
      max: null,
      maxLength: number$2,
      media: null,
      method: null,
      min: null,
      minLength: number$2,
      multiple: boolean$2,
      muted: boolean$2,
      name: null,
      nonce: null,
      noModule: boolean$2,
      noValidate: boolean$2,
      onAbort: null,
      onAfterPrint: null,
      onAuxClick: null,
      onBeforePrint: null,
      onBeforeUnload: null,
      onBlur: null,
      onCancel: null,
      onCanPlay: null,
      onCanPlayThrough: null,
      onChange: null,
      onClick: null,
      onClose: null,
      onContextLost: null,
      onContextMenu: null,
      onContextRestored: null,
      onCopy: null,
      onCueChange: null,
      onCut: null,
      onDblClick: null,
      onDrag: null,
      onDragEnd: null,
      onDragEnter: null,
      onDragExit: null,
      onDragLeave: null,
      onDragOver: null,
      onDragStart: null,
      onDrop: null,
      onDurationChange: null,
      onEmptied: null,
      onEnded: null,
      onError: null,
      onFocus: null,
      onFormData: null,
      onHashChange: null,
      onInput: null,
      onInvalid: null,
      onKeyDown: null,
      onKeyPress: null,
      onKeyUp: null,
      onLanguageChange: null,
      onLoad: null,
      onLoadedData: null,
      onLoadedMetadata: null,
      onLoadEnd: null,
      onLoadStart: null,
      onMessage: null,
      onMessageError: null,
      onMouseDown: null,
      onMouseEnter: null,
      onMouseLeave: null,
      onMouseMove: null,
      onMouseOut: null,
      onMouseOver: null,
      onMouseUp: null,
      onOffline: null,
      onOnline: null,
      onPageHide: null,
      onPageShow: null,
      onPaste: null,
      onPause: null,
      onPlay: null,
      onPlaying: null,
      onPopState: null,
      onProgress: null,
      onRateChange: null,
      onRejectionHandled: null,
      onReset: null,
      onResize: null,
      onScroll: null,
      onSecurityPolicyViolation: null,
      onSeeked: null,
      onSeeking: null,
      onSelect: null,
      onSlotChange: null,
      onStalled: null,
      onStorage: null,
      onSubmit: null,
      onSuspend: null,
      onTimeUpdate: null,
      onToggle: null,
      onUnhandledRejection: null,
      onUnload: null,
      onVolumeChange: null,
      onWaiting: null,
      onWheel: null,
      open: boolean$2,
      optimum: number$2,
      pattern: null,
      ping: spaceSeparated$2,
      placeholder: null,
      playsInline: boolean$2,
      poster: null,
      preload: null,
      readOnly: boolean$2,
      referrerPolicy: null,
      rel: spaceSeparated$2,
      required: boolean$2,
      reversed: boolean$2,
      rows: number$2,
      rowSpan: number$2,
      sandbox: spaceSeparated$2,
      scope: null,
      scoped: boolean$2,
      seamless: boolean$2,
      selected: boolean$2,
      shape: null,
      size: number$2,
      sizes: null,
      slot: null,
      span: number$2,
      spellCheck: booleanish$2,
      src: null,
      srcDoc: null,
      srcLang: null,
      srcSet: null,
      start: number$2,
      step: null,
      style: null,
      tabIndex: number$2,
      target: null,
      title: null,
      translate: null,
      type: null,
      typeMustMatch: boolean$2,
      useMap: null,
      value: booleanish$2,
      width: number$2,
      wrap: null,

      // Legacy.
      // See: https://html.spec.whatwg.org/#other-elements,-attributes-and-apis
      align: null, // Several. Use CSS `text-align` instead,
      aLink: null, // `<body>`. Use CSS `a:active {color}` instead
      archive: spaceSeparated$2, // `<object>`. List of URIs to archives
      axis: null, // `<td>` and `<th>`. Use `scope` on `<th>`
      background: null, // `<body>`. Use CSS `background-image` instead
      bgColor: null, // `<body>` and table elements. Use CSS `background-color` instead
      border: number$2, // `<table>`. Use CSS `border-width` instead,
      borderColor: null, // `<table>`. Use CSS `border-color` instead,
      bottomMargin: number$2, // `<body>`
      cellPadding: null, // `<table>`
      cellSpacing: null, // `<table>`
      char: null, // Several table elements. When `align=char`, sets the character to align on
      charOff: null, // Several table elements. When `char`, offsets the alignment
      classId: null, // `<object>`
      clear: null, // `<br>`. Use CSS `clear` instead
      code: null, // `<object>`
      codeBase: null, // `<object>`
      codeType: null, // `<object>`
      color: null, // `<font>` and `<hr>`. Use CSS instead
      compact: boolean$2, // Lists. Use CSS to reduce space between items instead
      declare: boolean$2, // `<object>`
      event: null, // `<script>`
      face: null, // `<font>`. Use CSS instead
      frame: null, // `<table>`
      frameBorder: null, // `<iframe>`. Use CSS `border` instead
      hSpace: number$2, // `<img>` and `<object>`
      leftMargin: number$2, // `<body>`
      link: null, // `<body>`. Use CSS `a:link {color: *}` instead
      longDesc: null, // `<frame>`, `<iframe>`, and `<img>`. Use an `<a>`
      lowSrc: null, // `<img>`. Use a `<picture>`
      marginHeight: number$2, // `<body>`
      marginWidth: number$2, // `<body>`
      noResize: boolean$2, // `<frame>`
      noHref: boolean$2, // `<area>`. Use no href instead of an explicit `nohref`
      noShade: boolean$2, // `<hr>`. Use background-color and height instead of borders
      noWrap: boolean$2, // `<td>` and `<th>`
      object: null, // `<applet>`
      profile: null, // `<head>`
      prompt: null, // `<isindex>`
      rev: null, // `<link>`
      rightMargin: number$2, // `<body>`
      rules: null, // `<table>`
      scheme: null, // `<meta>`
      scrolling: booleanish$2, // `<frame>`. Use overflow in the child context
      standby: null, // `<object>`
      summary: null, // `<table>`
      text: null, // `<body>`. Use CSS `color` instead
      topMargin: number$2, // `<body>`
      valueType: null, // `<param>`
      version: null, // `<html>`. Use a doctype.
      vAlign: null, // Several. Use CSS `vertical-align` instead
      vLink: null, // `<body>`. Use CSS `a:visited {color}` instead
      vSpace: number$2, // `<img>` and `<object>`

      // Non-standard Properties.
      allowTransparency: null,
      autoCorrect: null,
      autoSave: null,
      disablePictureInPicture: boolean$2,
      disableRemotePlayback: boolean$2,
      prefix: null,
      property: null,
      results: number$2,
      security: null,
      unselectable: null
    }
  });

  const svg$4 = create$2({
    space: 'svg',
    attributes: {
      accentHeight: 'accent-height',
      alignmentBaseline: 'alignment-baseline',
      arabicForm: 'arabic-form',
      baselineShift: 'baseline-shift',
      capHeight: 'cap-height',
      className: 'class',
      clipPath: 'clip-path',
      clipRule: 'clip-rule',
      colorInterpolation: 'color-interpolation',
      colorInterpolationFilters: 'color-interpolation-filters',
      colorProfile: 'color-profile',
      colorRendering: 'color-rendering',
      crossOrigin: 'crossorigin',
      dataType: 'datatype',
      dominantBaseline: 'dominant-baseline',
      enableBackground: 'enable-background',
      fillOpacity: 'fill-opacity',
      fillRule: 'fill-rule',
      floodColor: 'flood-color',
      floodOpacity: 'flood-opacity',
      fontFamily: 'font-family',
      fontSize: 'font-size',
      fontSizeAdjust: 'font-size-adjust',
      fontStretch: 'font-stretch',
      fontStyle: 'font-style',
      fontVariant: 'font-variant',
      fontWeight: 'font-weight',
      glyphName: 'glyph-name',
      glyphOrientationHorizontal: 'glyph-orientation-horizontal',
      glyphOrientationVertical: 'glyph-orientation-vertical',
      hrefLang: 'hreflang',
      horizAdvX: 'horiz-adv-x',
      horizOriginX: 'horiz-origin-x',
      horizOriginY: 'horiz-origin-y',
      imageRendering: 'image-rendering',
      letterSpacing: 'letter-spacing',
      lightingColor: 'lighting-color',
      markerEnd: 'marker-end',
      markerMid: 'marker-mid',
      markerStart: 'marker-start',
      navDown: 'nav-down',
      navDownLeft: 'nav-down-left',
      navDownRight: 'nav-down-right',
      navLeft: 'nav-left',
      navNext: 'nav-next',
      navPrev: 'nav-prev',
      navRight: 'nav-right',
      navUp: 'nav-up',
      navUpLeft: 'nav-up-left',
      navUpRight: 'nav-up-right',
      onAbort: 'onabort',
      onActivate: 'onactivate',
      onAfterPrint: 'onafterprint',
      onBeforePrint: 'onbeforeprint',
      onBegin: 'onbegin',
      onCancel: 'oncancel',
      onCanPlay: 'oncanplay',
      onCanPlayThrough: 'oncanplaythrough',
      onChange: 'onchange',
      onClick: 'onclick',
      onClose: 'onclose',
      onCopy: 'oncopy',
      onCueChange: 'oncuechange',
      onCut: 'oncut',
      onDblClick: 'ondblclick',
      onDrag: 'ondrag',
      onDragEnd: 'ondragend',
      onDragEnter: 'ondragenter',
      onDragExit: 'ondragexit',
      onDragLeave: 'ondragleave',
      onDragOver: 'ondragover',
      onDragStart: 'ondragstart',
      onDrop: 'ondrop',
      onDurationChange: 'ondurationchange',
      onEmptied: 'onemptied',
      onEnd: 'onend',
      onEnded: 'onended',
      onError: 'onerror',
      onFocus: 'onfocus',
      onFocusIn: 'onfocusin',
      onFocusOut: 'onfocusout',
      onHashChange: 'onhashchange',
      onInput: 'oninput',
      onInvalid: 'oninvalid',
      onKeyDown: 'onkeydown',
      onKeyPress: 'onkeypress',
      onKeyUp: 'onkeyup',
      onLoad: 'onload',
      onLoadedData: 'onloadeddata',
      onLoadedMetadata: 'onloadedmetadata',
      onLoadStart: 'onloadstart',
      onMessage: 'onmessage',
      onMouseDown: 'onmousedown',
      onMouseEnter: 'onmouseenter',
      onMouseLeave: 'onmouseleave',
      onMouseMove: 'onmousemove',
      onMouseOut: 'onmouseout',
      onMouseOver: 'onmouseover',
      onMouseUp: 'onmouseup',
      onMouseWheel: 'onmousewheel',
      onOffline: 'onoffline',
      onOnline: 'ononline',
      onPageHide: 'onpagehide',
      onPageShow: 'onpageshow',
      onPaste: 'onpaste',
      onPause: 'onpause',
      onPlay: 'onplay',
      onPlaying: 'onplaying',
      onPopState: 'onpopstate',
      onProgress: 'onprogress',
      onRateChange: 'onratechange',
      onRepeat: 'onrepeat',
      onReset: 'onreset',
      onResize: 'onresize',
      onScroll: 'onscroll',
      onSeeked: 'onseeked',
      onSeeking: 'onseeking',
      onSelect: 'onselect',
      onShow: 'onshow',
      onStalled: 'onstalled',
      onStorage: 'onstorage',
      onSubmit: 'onsubmit',
      onSuspend: 'onsuspend',
      onTimeUpdate: 'ontimeupdate',
      onToggle: 'ontoggle',
      onUnload: 'onunload',
      onVolumeChange: 'onvolumechange',
      onWaiting: 'onwaiting',
      onZoom: 'onzoom',
      overlinePosition: 'overline-position',
      overlineThickness: 'overline-thickness',
      paintOrder: 'paint-order',
      panose1: 'panose-1',
      pointerEvents: 'pointer-events',
      referrerPolicy: 'referrerpolicy',
      renderingIntent: 'rendering-intent',
      shapeRendering: 'shape-rendering',
      stopColor: 'stop-color',
      stopOpacity: 'stop-opacity',
      strikethroughPosition: 'strikethrough-position',
      strikethroughThickness: 'strikethrough-thickness',
      strokeDashArray: 'stroke-dasharray',
      strokeDashOffset: 'stroke-dashoffset',
      strokeLineCap: 'stroke-linecap',
      strokeLineJoin: 'stroke-linejoin',
      strokeMiterLimit: 'stroke-miterlimit',
      strokeOpacity: 'stroke-opacity',
      strokeWidth: 'stroke-width',
      tabIndex: 'tabindex',
      textAnchor: 'text-anchor',
      textDecoration: 'text-decoration',
      textRendering: 'text-rendering',
      typeOf: 'typeof',
      underlinePosition: 'underline-position',
      underlineThickness: 'underline-thickness',
      unicodeBidi: 'unicode-bidi',
      unicodeRange: 'unicode-range',
      unitsPerEm: 'units-per-em',
      vAlphabetic: 'v-alphabetic',
      vHanging: 'v-hanging',
      vIdeographic: 'v-ideographic',
      vMathematical: 'v-mathematical',
      vectorEffect: 'vector-effect',
      vertAdvY: 'vert-adv-y',
      vertOriginX: 'vert-origin-x',
      vertOriginY: 'vert-origin-y',
      wordSpacing: 'word-spacing',
      writingMode: 'writing-mode',
      xHeight: 'x-height',
      // These were camelcased in Tiny. Now lowercased in SVG 2
      playbackOrder: 'playbackorder',
      timelineBegin: 'timelinebegin'
    },
    transform: caseSensitiveTransform$2,
    properties: {
      about: commaOrSpaceSeparated$2,
      accentHeight: number$2,
      accumulate: null,
      additive: null,
      alignmentBaseline: null,
      alphabetic: number$2,
      amplitude: number$2,
      arabicForm: null,
      ascent: number$2,
      attributeName: null,
      attributeType: null,
      azimuth: number$2,
      bandwidth: null,
      baselineShift: null,
      baseFrequency: null,
      baseProfile: null,
      bbox: null,
      begin: null,
      bias: number$2,
      by: null,
      calcMode: null,
      capHeight: number$2,
      className: spaceSeparated$2,
      clip: null,
      clipPath: null,
      clipPathUnits: null,
      clipRule: null,
      color: null,
      colorInterpolation: null,
      colorInterpolationFilters: null,
      colorProfile: null,
      colorRendering: null,
      content: null,
      contentScriptType: null,
      contentStyleType: null,
      crossOrigin: null,
      cursor: null,
      cx: null,
      cy: null,
      d: null,
      dataType: null,
      defaultAction: null,
      descent: number$2,
      diffuseConstant: number$2,
      direction: null,
      display: null,
      dur: null,
      divisor: number$2,
      dominantBaseline: null,
      download: boolean$2,
      dx: null,
      dy: null,
      edgeMode: null,
      editable: null,
      elevation: number$2,
      enableBackground: null,
      end: null,
      event: null,
      exponent: number$2,
      externalResourcesRequired: null,
      fill: null,
      fillOpacity: number$2,
      fillRule: null,
      filter: null,
      filterRes: null,
      filterUnits: null,
      floodColor: null,
      floodOpacity: null,
      focusable: null,
      focusHighlight: null,
      fontFamily: null,
      fontSize: null,
      fontSizeAdjust: null,
      fontStretch: null,
      fontStyle: null,
      fontVariant: null,
      fontWeight: null,
      format: null,
      fr: null,
      from: null,
      fx: null,
      fy: null,
      g1: commaSeparated$2,
      g2: commaSeparated$2,
      glyphName: commaSeparated$2,
      glyphOrientationHorizontal: null,
      glyphOrientationVertical: null,
      glyphRef: null,
      gradientTransform: null,
      gradientUnits: null,
      handler: null,
      hanging: number$2,
      hatchContentUnits: null,
      hatchUnits: null,
      height: null,
      href: null,
      hrefLang: null,
      horizAdvX: number$2,
      horizOriginX: number$2,
      horizOriginY: number$2,
      id: null,
      ideographic: number$2,
      imageRendering: null,
      initialVisibility: null,
      in: null,
      in2: null,
      intercept: number$2,
      k: number$2,
      k1: number$2,
      k2: number$2,
      k3: number$2,
      k4: number$2,
      kernelMatrix: commaOrSpaceSeparated$2,
      kernelUnitLength: null,
      keyPoints: null, // SEMI_COLON_SEPARATED
      keySplines: null, // SEMI_COLON_SEPARATED
      keyTimes: null, // SEMI_COLON_SEPARATED
      kerning: null,
      lang: null,
      lengthAdjust: null,
      letterSpacing: null,
      lightingColor: null,
      limitingConeAngle: number$2,
      local: null,
      markerEnd: null,
      markerMid: null,
      markerStart: null,
      markerHeight: null,
      markerUnits: null,
      markerWidth: null,
      mask: null,
      maskContentUnits: null,
      maskUnits: null,
      mathematical: null,
      max: null,
      media: null,
      mediaCharacterEncoding: null,
      mediaContentEncodings: null,
      mediaSize: number$2,
      mediaTime: null,
      method: null,
      min: null,
      mode: null,
      name: null,
      navDown: null,
      navDownLeft: null,
      navDownRight: null,
      navLeft: null,
      navNext: null,
      navPrev: null,
      navRight: null,
      navUp: null,
      navUpLeft: null,
      navUpRight: null,
      numOctaves: null,
      observer: null,
      offset: null,
      onAbort: null,
      onActivate: null,
      onAfterPrint: null,
      onBeforePrint: null,
      onBegin: null,
      onCancel: null,
      onCanPlay: null,
      onCanPlayThrough: null,
      onChange: null,
      onClick: null,
      onClose: null,
      onCopy: null,
      onCueChange: null,
      onCut: null,
      onDblClick: null,
      onDrag: null,
      onDragEnd: null,
      onDragEnter: null,
      onDragExit: null,
      onDragLeave: null,
      onDragOver: null,
      onDragStart: null,
      onDrop: null,
      onDurationChange: null,
      onEmptied: null,
      onEnd: null,
      onEnded: null,
      onError: null,
      onFocus: null,
      onFocusIn: null,
      onFocusOut: null,
      onHashChange: null,
      onInput: null,
      onInvalid: null,
      onKeyDown: null,
      onKeyPress: null,
      onKeyUp: null,
      onLoad: null,
      onLoadedData: null,
      onLoadedMetadata: null,
      onLoadStart: null,
      onMessage: null,
      onMouseDown: null,
      onMouseEnter: null,
      onMouseLeave: null,
      onMouseMove: null,
      onMouseOut: null,
      onMouseOver: null,
      onMouseUp: null,
      onMouseWheel: null,
      onOffline: null,
      onOnline: null,
      onPageHide: null,
      onPageShow: null,
      onPaste: null,
      onPause: null,
      onPlay: null,
      onPlaying: null,
      onPopState: null,
      onProgress: null,
      onRateChange: null,
      onRepeat: null,
      onReset: null,
      onResize: null,
      onScroll: null,
      onSeeked: null,
      onSeeking: null,
      onSelect: null,
      onShow: null,
      onStalled: null,
      onStorage: null,
      onSubmit: null,
      onSuspend: null,
      onTimeUpdate: null,
      onToggle: null,
      onUnload: null,
      onVolumeChange: null,
      onWaiting: null,
      onZoom: null,
      opacity: null,
      operator: null,
      order: null,
      orient: null,
      orientation: null,
      origin: null,
      overflow: null,
      overlay: null,
      overlinePosition: number$2,
      overlineThickness: number$2,
      paintOrder: null,
      panose1: null,
      path: null,
      pathLength: number$2,
      patternContentUnits: null,
      patternTransform: null,
      patternUnits: null,
      phase: null,
      ping: spaceSeparated$2,
      pitch: null,
      playbackOrder: null,
      pointerEvents: null,
      points: null,
      pointsAtX: number$2,
      pointsAtY: number$2,
      pointsAtZ: number$2,
      preserveAlpha: null,
      preserveAspectRatio: null,
      primitiveUnits: null,
      propagate: null,
      property: commaOrSpaceSeparated$2,
      r: null,
      radius: null,
      referrerPolicy: null,
      refX: null,
      refY: null,
      rel: commaOrSpaceSeparated$2,
      rev: commaOrSpaceSeparated$2,
      renderingIntent: null,
      repeatCount: null,
      repeatDur: null,
      requiredExtensions: commaOrSpaceSeparated$2,
      requiredFeatures: commaOrSpaceSeparated$2,
      requiredFonts: commaOrSpaceSeparated$2,
      requiredFormats: commaOrSpaceSeparated$2,
      resource: null,
      restart: null,
      result: null,
      rotate: null,
      rx: null,
      ry: null,
      scale: null,
      seed: null,
      shapeRendering: null,
      side: null,
      slope: null,
      snapshotTime: null,
      specularConstant: number$2,
      specularExponent: number$2,
      spreadMethod: null,
      spacing: null,
      startOffset: null,
      stdDeviation: null,
      stemh: null,
      stemv: null,
      stitchTiles: null,
      stopColor: null,
      stopOpacity: null,
      strikethroughPosition: number$2,
      strikethroughThickness: number$2,
      string: null,
      stroke: null,
      strokeDashArray: commaOrSpaceSeparated$2,
      strokeDashOffset: null,
      strokeLineCap: null,
      strokeLineJoin: null,
      strokeMiterLimit: number$2,
      strokeOpacity: number$2,
      strokeWidth: null,
      style: null,
      surfaceScale: number$2,
      syncBehavior: null,
      syncBehaviorDefault: null,
      syncMaster: null,
      syncTolerance: null,
      syncToleranceDefault: null,
      systemLanguage: commaOrSpaceSeparated$2,
      tabIndex: number$2,
      tableValues: null,
      target: null,
      targetX: number$2,
      targetY: number$2,
      textAnchor: null,
      textDecoration: null,
      textRendering: null,
      textLength: null,
      timelineBegin: null,
      title: null,
      transformBehavior: null,
      type: null,
      typeOf: commaOrSpaceSeparated$2,
      to: null,
      transform: null,
      u1: null,
      u2: null,
      underlinePosition: number$2,
      underlineThickness: number$2,
      unicode: null,
      unicodeBidi: null,
      unicodeRange: null,
      unitsPerEm: number$2,
      values: null,
      vAlphabetic: number$2,
      vMathematical: number$2,
      vectorEffect: null,
      vHanging: number$2,
      vIdeographic: number$2,
      version: null,
      vertAdvY: number$2,
      vertOriginX: number$2,
      vertOriginY: number$2,
      viewBox: null,
      viewTarget: null,
      visibility: null,
      width: null,
      widths: null,
      wordSpacing: null,
      writingMode: null,
      x: null,
      x1: null,
      x2: null,
      xChannelSelector: null,
      xHeight: number$2,
      y: null,
      y1: null,
      y2: null,
      yChannelSelector: null,
      z: null,
      zoomAndPan: null
    }
  });

  /**
   * @typedef {import('./util/schema.js').Schema} Schema
   */

  const valid$2 = /^data[-\w.:]+$/i;
  const dash$2 = /-[a-z]/g;
  const cap$2 = /[A-Z]/g;

  /**
   * @param {Schema} schema
   * @param {string} value
   * @returns {Info}
   */
  function find$2(schema, value) {
    const normal = normalize$2(value);
    let prop = value;
    let Type = Info$2;

    if (normal in schema.normal) {
      return schema.property[schema.normal[normal]]
    }

    if (normal.length > 4 && normal.slice(0, 4) === 'data' && valid$2.test(value)) {
      // Attribute or property.
      if (value.charAt(4) === '-') {
        // Turn it into a property.
        const rest = value.slice(5).replace(dash$2, camelcase$2);
        prop = 'data' + rest.charAt(0).toUpperCase() + rest.slice(1);
      } else {
        // Turn it into an attribute.
        const rest = value.slice(4);

        if (!dash$2.test(rest)) {
          let dashes = rest.replace(cap$2, kebab$2);

          if (dashes.charAt(0) !== '-') {
            dashes = '-' + dashes;
          }

          value = 'data' + dashes;
        }
      }

      Type = DefinedInfo$2;
    }

    return new Type(prop, value)
  }

  /**
   * @param {string} $0
   * @returns {string}
   */
  function kebab$2($0) {
    return '-' + $0.toLowerCase()
  }

  /**
   * @param {string} $0
   * @returns {string}
   */
  function camelcase$2($0) {
    return $0.charAt(1).toUpperCase()
  }

  /**
   * @typedef {import('./lib/util/info.js').Info} Info
   * @typedef {import('./lib/util/schema.js').Schema} Schema
   */
  const html$6 = merge$2([xml$2, xlink$2, xmlns$2, aria$2, html$7], 'html');
  const svg$3 = merge$2([xml$2, xlink$2, xmlns$2, aria$2, svg$4], 'svg');

  const rtlRange = '\u0591-\u07FF\uFB1D-\uFDFD\uFE70-\uFEFC';
  const ltrRange =
    'A-Za-z\u00C0-\u00D6\u00D8-\u00F6' +
    '\u00F8-\u02B8\u0300-\u0590\u0800-\u1FFF\u200E\u2C00-\uFB1C' +
    '\uFE00-\uFE6F\uFEFD-\uFFFF';

  /* eslint-disable no-misleading-character-class */
  const rtl = new RegExp('^[^' + ltrRange + ']*[' + rtlRange + ']');
  const ltr = new RegExp('^[^' + rtlRange + ']*[' + ltrRange + ']');
  /* eslint-enable no-misleading-character-class */

  /**
   * Detect the direction of text: left-to-right, right-to-left, or neutral
   *
   * @param {string} value
   * @returns {'rtl'|'ltr'|'neutral'}
   */
  function direction(value) {
    const source = String(value || '');
    return rtl.test(source) ? 'rtl' : ltr.test(source) ? 'ltr' : 'neutral'
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   * @typedef {import('hast').Element} Element
   *
   * @typedef {string} TagName
   * @typedef {null|undefined|TagName|TestFunctionAnything|Array.<TagName|TestFunctionAnything>} Test
   */

  /**
   * @template {Element} T
   * @typedef {null|undefined|T['tagName']|TestFunctionPredicate<T>|Array.<T['tagName']|TestFunctionPredicate<T>>} PredicateTest
   */

  /**
   * Check if an element passes a test
   *
   * @callback TestFunctionAnything
   * @param {Element} element
   * @param {number|null|undefined} [index]
   * @param {Parent|null|undefined} [parent]
   * @returns {boolean|void}
   */

  /**
   * Check if an element passes a certain node test
   *
   * @template {Element} X
   * @callback TestFunctionPredicate
   * @param {Element} element
   * @param {number|null|undefined} [index]
   * @param {Parent|null|undefined} [parent]
   * @returns {element is X}
   */

  /**
   * Check if a node is an element and passes a certain node test
   *
   * @callback AssertAnything
   * @param {unknown} [node]
   * @param {number|null|undefined} [index]
   * @param {Parent|null|undefined} [parent]
   * @returns {boolean}
   */

  /**
   * Check if a node is an element and passes a certain node test
   *
   * @template {Element} Y
   * @callback AssertPredicate
   * @param {unknown} [node]
   * @param {number|null|undefined} [index]
   * @param {Parent|null|undefined} [parent]
   * @returns {node is Y}
   */

  // Check if `node` is an `element` and whether it passes the given test.
  const isElement$1 =
    /**
     * Check if a node is an element and passes a test.
     * When a `parent` node is known the `index` of node should also be given.
     *
     * @type {(
     *   (() => false) &
     *   (<T extends Element = Element>(node: unknown, test?: PredicateTest<T>, index?: number, parent?: Parent, context?: unknown) => node is T) &
     *   ((node: unknown, test: Test, index?: number, parent?: Parent, context?: unknown) => boolean)
     * )}
     */
    (
      /**
       * Check if a node passes a test.
       * When a `parent` node is known the `index` of node should also be given.
       *
       * @param {unknown} [node] Node to check
       * @param {Test} [test] When nullish, checks if `node` is a `Node`.
       * When `string`, works like passing `function (node) {return node.type === test}`.
       * When `function` checks if function passed the node is true.
       * When `array`, checks any one of the subtests pass.
       * @param {number} [index] Position of `node` in `parent`
       * @param {Parent} [parent] Parent of `node`
       * @param {unknown} [context] Context object to invoke `test` with
       * @returns {boolean} Whether test passed and `node` is an `Element` (object with `type` set to `element` and `tagName` set to a non-empty string).
       */
      // eslint-disable-next-line max-params
      function (node, test, index, parent, context) {
        const check = convertElement$1(test);

        if (
          index !== undefined &&
          index !== null &&
          (typeof index !== 'number' ||
            index < 0 ||
            index === Number.POSITIVE_INFINITY)
        ) {
          throw new Error('Expected positive finite index for child node')
        }

        if (
          parent !== undefined &&
          parent !== null &&
          (!parent.type || !parent.children)
        ) {
          throw new Error('Expected parent node')
        }

        // @ts-expect-error Looks like a node.
        if (!node || !node.type || typeof node.type !== 'string') {
          return false
        }

        if (
          (parent === undefined || parent === null) !==
          (index === undefined || index === null)
        ) {
          throw new Error('Expected both parent and index')
        }

        return check.call(context, node, index, parent)
      }
    );

  const convertElement$1 =
    /**
     * @type {(
     *   (<T extends Element>(test: T['tagName']|TestFunctionPredicate<T>) => AssertPredicate<T>) &
     *   ((test?: Test) => AssertAnything)
     * )}
     */
    (
      /**
       * Generate an assertion from a check.
       * @param {Test} [test]
       * When nullish, checks if `node` is a `Node`.
       * When `string`, works like passing `function (node) {return node.type === test}`.
       * When `function` checks if function passed the node is true.
       * When `object`, checks that all keys in test are in node, and that they have (strictly) equal values.
       * When `array`, checks any one of the subtests pass.
       * @returns {AssertAnything}
       */
      function (test) {
        if (test === undefined || test === null) {
          return element$3
        }

        if (typeof test === 'string') {
          return tagNameFactory$1(test)
        }

        if (typeof test === 'object') {
          return anyFactory$1(test)
        }

        if (typeof test === 'function') {
          return castFactory$1(test)
        }

        throw new Error('Expected function, string, or array as test')
      }
    );

  /**
   * @param {Array.<TagName|TestFunctionAnything>} tests
   * @returns {AssertAnything}
   */
  function anyFactory$1(tests) {
    /** @type {Array.<AssertAnything>} */
    const checks = [];
    let index = -1;

    while (++index < tests.length) {
      checks[index] = convertElement$1(tests[index]);
    }

    return castFactory$1(any)

    /**
     * @this {unknown}
     * @param {unknown[]} parameters
     * @returns {boolean}
     */
    function any(...parameters) {
      let index = -1;

      while (++index < checks.length) {
        if (checks[index].call(this, ...parameters)) {
          return true
        }
      }

      return false
    }
  }

  /**
   * Utility to convert a string into a function which checks a given node’s tag
   * name for said string.
   *
   * @param {TagName} check
   * @returns {AssertAnything}
   */
  function tagNameFactory$1(check) {
    return tagName

    /**
     * @param {unknown} node
     * @returns {boolean}
     */
    function tagName(node) {
      return element$3(node) && node.tagName === check
    }
  }

  /**
   * @param {TestFunctionAnything} check
   * @returns {AssertAnything}
   */
  function castFactory$1(check) {
    return assertion

    /**
     * @this {unknown}
     * @param {unknown} node
     * @param {Array.<unknown>} parameters
     * @returns {boolean}
     */
    function assertion(node, ...parameters) {
      // @ts-expect-error: fine.
      return element$3(node) && Boolean(check.call(this, node, ...parameters))
    }
  }

  /**
   * Utility to return true if this is an element.
   * @param {unknown} node
   * @returns {node is Element}
   */
  function element$3(node) {
    return Boolean(
      node &&
        typeof node === 'object' &&
        // @ts-expect-error Looks like a node.
        node.type === 'element' &&
        // @ts-expect-error Looks like an element.
        typeof node.tagName === 'string'
    )
  }

  /**
   * @fileoverview
   *   Get the plain-text value of a hast node.
   * @longdescription
   *   ## Use
   *
   *   ```js
   *   import {h} from 'hastscript'
   *   import {toString} from 'hast-util-to-string'
   *
   *   toString(h('p', 'Alpha'))
   *   //=> 'Alpha'
   *   toString(h('div', [h('b', 'Bold'), ' and ', h('i', 'italic'), '.']))
   *   //=> 'Bold and italic.'
   *   ```
   *
   *   ## API
   *
   *   ### `toString(node)`
   *
   *   Transform a node to a string.
   */

  /**
   * @typedef {import('hast').Root} Root
   * @typedef {import('hast').Element} Element
   * @typedef {Root|Root['children'][number]} Node
   */

  /**
   * Get the plain-text value of a hast node.
   *
   * @param {Node} node
   * @returns {string}
   */
  function toString(node) {
    // “The concatenation of data of all the Text node descendants of the context
    // object, in tree order.”
    if ('children' in node) {
      return all$1(node)
    }

    // “Context object’s data.”
    return 'value' in node ? node.value : ''
  }

  /**
   * @param {Node} node
   * @returns {string}
   */
  function one$1(node) {
    if (node.type === 'text') {
      return node.value
    }

    return 'children' in node ? all$1(node) : ''
  }

  /**
   * @param {Root|Element} node
   * @returns {string}
   */
  function all$1(node) {
    let index = -1;
    /** @type {string[]} */
    const result = [];

    while (++index < node.children.length) {
      result[index] = one$1(node.children[index]);
    }

    return result.join('')
  }

  /**
   * @typedef {import('./types.js').Node} Node
   * @typedef {import('./types.js').Element} Element
   * @typedef {import('./types.js').Parent} Parent
   * @typedef {import('hast-util-is-element').AssertPredicate<Element>} IsElement
   */

  /**
   * @param {Node} node
   * @returns {node is Parent}
   */
  function parent(node) {
    // @ts-expect-error: hush.
    return Array.isArray(node.children)
  }

  /** @type {IsElement} */
  // @ts-expect-error it works.
  const element$2 = convertElement$1();

  /**
   * @typedef {import('./types.js').SelectState} SelectState
   * @typedef {import('./types.js').HastNode} HastNode
   * @typedef {import('./types.js').ElementChild} ElementChild
   * @typedef {import('./types.js').Direction} Direction
   * @typedef {import('unist-util-visit').Visitor<ElementChild>} Visitor
   */

  /**
   * @param {SelectState} state
   * @param {HastNode} node
   * @returns {() => void}
   */
  // eslint-disable-next-line complexity
  function enterState(state, node) {
    const schema = state.schema;
    const language = state.language;
    const currentDirection = state.direction;
    const editableOrEditingHost = state.editableOrEditingHost;
    /** @type {Direction|undefined} */
    let dirInferred;
    /** @type {boolean|undefined} */
    let found;

    if (element$2(node) && node.properties) {
      const lang = node.properties.xmlLang || node.properties.lang;
      const type = node.properties.type || 'text';
      const dir = dirProperty(node);

      if (lang !== undefined && lang !== null) {
        state.language = String(lang);
        found = true;
      }

      if (schema && schema.space === 'html') {
        if (node.properties.contentEditable === 'true') {
          state.editableOrEditingHost = true;
          found = true;
        }

        if (isElement$1(node, 'svg')) {
          state.schema = svg$3;
          found = true;
        }

        // See: <https://html.spec.whatwg.org/#the-directionality>.
        // Explicit `[dir=rtl]`.
        if (dir === 'rtl') {
          dirInferred = dir;
        } else if (
          // Explicit `[dir=ltr]`.
          dir === 'ltr' ||
          // HTML with an invalid or no `[dir]`.
          (dir !== 'auto' && isElement$1(node, 'html')) ||
          // `input[type=tel]` with an invalid or no `[dir]`.
          (dir !== 'auto' && isElement$1(node, 'input') && type === 'tel')
        ) {
          dirInferred = 'ltr';
          // `[dir=auto]` or `bdi` with an invalid or no `[dir]`.
        } else if (dir === 'auto' || isElement$1(node, 'bdi')) {
          if (isElement$1(node, 'textarea')) {
            // Check contents of `<textarea>`.
            dirInferred = dirBidi(toString(node));
          } else if (
            isElement$1(node, 'input') &&
            (type === 'email' ||
              type === 'search' ||
              type === 'tel' ||
              type === 'text')
          ) {
            // Check value of `<input>`.
            // @ts-expect-error something is `never` in types but this is needed.
            dirInferred = node.properties.value
              ? // @ts-expect-error Assume string
                dirBidi(node.properties.value)
              : 'ltr';
          } else {
            // Check text nodes in `node`.
            visit$2(node, inferDirectionality);
          }
        }

        if (dirInferred) {
          state.direction = dirInferred;
          found = true;
        }
      }
      // Turn off editing mode in non-HTML spaces.
      else if (state.editableOrEditingHost) {
        state.editableOrEditingHost = false;
        found = true;
      }
    }

    return found ? reset : noop

    function reset() {
      state.schema = schema;
      state.language = language;
      state.direction = currentDirection;
      state.editableOrEditingHost = editableOrEditingHost;
    }

    /** @type {Visitor} */
    function inferDirectionality(child) {
      if (child.type === 'text') {
        dirInferred = dirBidi(child.value);
        return dirInferred ? EXIT$2 : null
      }

      if (
        child !== node &&
        (isElement$1(child, ['bdi', 'script', 'style', 'textare']) ||
          dirProperty(child))
      ) {
        return SKIP$2
      }
    }
  }

  /**
   * @param {string} value
   * @returns {Direction|undefined}
   */
  function dirBidi(value) {
    const result = direction(value);
    return result === 'neutral' ? undefined : result
  }

  /**
   * @param {ElementChild} node
   * @returns {Direction|undefined}
   */
  function dirProperty(node) {
    const value =
      element$2(node) && node.properties && typeof node.properties.dir === 'string'
        ? node.properties.dir.toLowerCase()
        : undefined;

    return value === 'auto' || value === 'ltr' || value === 'rtl'
      ? value
      : undefined
  }

  function noop() {}

  /**
   * @typedef {import('./types.js').Rule} Rule
   * @typedef {import('./types.js').Node} Node
   * @typedef {import('./types.js').Element} Element
   * @typedef {import('./types.js').Parent} Parent
   * @typedef {import('./types.js').SelectState} SelectState
   * @typedef {import('./types.js').SelectIterator} SelectIterator
   * @typedef {import('./types.js').Handler} Handler
   */

  const own$9 = {}.hasOwnProperty;

  const handle$2 = zwitch('nestingOperator', {
    // @ts-expect-error: hush.
    unknown: unknownNesting,
    // @ts-expect-error: hush.
    invalid: topScan, // `undefined` is the top query selector.
    handlers: {
      // @ts-expect-error: hush.
      null: descendant, // `null` is the descendant combinator.
      // @ts-expect-error: hush.
      '>': child,
      // @ts-expect-error: hush.
      '+': adjacentSibling,
      // @ts-expect-error: hush.
      '~': generalSibling
    }
  });

  /** @type {Handler} */
  function nest(query, node, index, parent, state) {
    handle$2(query, node, index, parent, state);
  }

  // Shouldn’t be called, parser gives correct data.
  /* c8 ignore next 6 */
  /**
   * @param {{[x: string]: unknown, type: string}} query
   */
  function unknownNesting(query) {
    throw new Error('Unexpected nesting `' + query.nestingOperator + '`')
  }

  /** @type {Handler} */
  function topScan(query, node, index, parent, state) {
    // Shouldn’t happen.
    /* c8 ignore next 3 */
    if (parent || index === null) {
      throw new Error('topScan is supposed to be called from the root node')
    }

    // Shouldn’t happen.
    /* c8 ignore next 3 */
    if (!state.iterator) {
      throw new Error('Expected `iterator`')
    }

    state.iterator(query, node, index, parent, state);
    if (!state.shallow) descendant(query, node, index, parent, state);
  }

  /** @type {Handler} */
  function descendant(query, node, index, parent, state) {
    const previous = state.iterator;

    state.iterator = iterator;
    child(query, node, index, parent, state);

    /** @type {SelectIterator} */
    function iterator(query, node, index, parent, state) {
      // Shouldn’t happen.
      /* c8 ignore next 3 */
      if (!previous) {
        throw new Error('Expected `iterator`')
      }

      state.iterator = previous;
      previous(query, node, index, parent, state);
      state.iterator = iterator;

      if (state.one && state.found) return

      child(query, node, index, parent, state);
    }
  }

  /** @type {Handler} */
  function child(query, node, _1, _2, state) {
    if (!parent(node)) return
    if (node.children.length === 0) return
    indexedSearch(query, node, state);
  }

  /** @type {Handler} */
  function adjacentSibling(query, _, index, parent, state) {
    // Shouldn’t happen.
    /* c8 ignore next */
    if (!parent || index === null) return
    indexedSearch(query, parent, state, index + 1, true);
  }

  /** @type {Handler} */
  function generalSibling(query, _, index, parent, state) {
    // Shouldn’t happen.
    /* c8 ignore next */
    if (!parent || index === null) return
    indexedSearch(query, parent, state, index + 1);
  }

  /**
   * Handles `typeIndex` and `typeCount` properties for every walker.
   *
   * @param {Rule} query
   * @param {Parent} parent
   * @param {SelectState} state
   * @param {number} [from=0]
   * @param {boolean} [firstElementOnly=false]
   */
  function indexedSearch(query, parent, state, from, firstElementOnly) {
    const handle = state.index ? delay : add;
    const children = parent.children;
    let elements = 0;
    let index = -1;
    /** @type {Object.<string, number>} */
    const types = {};
    /** @type {Array.<Function>} */
    const delayed = [];

    // Start looking at `from`
    if (from === undefined || from === null) from = 0;

    // Exit if there are no further nodes.
    if (from >= children.length) return

    // If we need to index for types, do so for all elements before `from`.
    if (state.index) {
      while (++index < from) {
        const child = children[index];
        if (element$2(child)) count(child.tagName);
      }
    }

    index = from - 1;

    while (++index < children.length) {
      const child = children[index];
      // Only check elements.
      // Check either all elements, or only check the first sibling
      if (element$2(child)) {
        handle(child, index);

        // Stop if we’re looking for one node and it’s already found.
        if (state.one && state.found) return
        if (firstElementOnly) break
      }
    }

    if (state.index) {
      index = -1;

      while (++index < delayed.length) {
        delayed[index]();
        if (state.one && state.found) return
      }
    }

    /**
     * @param {Element} node
     * @param {number} childIndex
     */
    function delay(node, childIndex) {
      const elementsBefore = elements;
      const elementsByTypeBefore = own$9.call(types, node.tagName)
        ? types[node.tagName]
        : 0;

      count(node.tagName);

      delayed.push(fn);

      function fn() {
        // Before counting further elements:
        state.elementIndex = elementsBefore;
        state.typeIndex = elementsByTypeBefore;

        // After counting all elements.
        state.elementCount = elements;
        state.typeCount = types[node.tagName];

        add(node, childIndex);
      }
    }

    /**
     * @param {Element} node
     * @param {number} childIndex
     */
    function add(node, childIndex) {
      const exit = enterState(state, node);

      // Shouldn’t happen.
      /* c8 ignore next 3 */
      if (!state.iterator) {
        throw new Error('Expected `iterator`')
      }

      state.iterator(query, node, childIndex, parent, state);
      exit();
    }

    /**
     * @param {string} name
     */
    function count(name) {
      if (!own$9.call(types, name)) types[name] = 0;
      elements++;
      types[name]++;
    }
  }

  /**
   * See <https://tools.ietf.org/html/rfc4647#section-3.1>
   * for more information on the algorithms.
   */

  /**
   * @typedef {string} Tag
   * @typedef {Array<Tag>} Tags
   * @typedef {string} Range
   * @typedef {Array<Range>} Ranges
   */

  /**
   * @callback Check
   * @param {Tag} tag
   * @param {Range} range
   * @returns {boolean}
   */

  /**
   * @callback Filter
   * @param {Tag|Tags} tag
   * @param {Range|Ranges} [ranges]
   * @returns {Tag}
   */

  /**
   * @callback Lookup
   * @param {Tag|Tags} tag
   * @param {Range|Ranges} [ranges]
   * @returns {Tag}
   */

  /**
   * Factory to perform a filter or a lookup.
   * This factory creates a function that accepts a list of tags and a list of
   * ranges, and contains logic to exit early for lookups.
   * `check` just has to deal with one tag and one range.
   * This match function iterates over ranges, and for each range,
   * iterates over tags.  That way, earlier ranges matching any tag have
   * precedence over later ranges.
   *
   * @type {{
   *   (check: Check, filter: true): Filter
   *   (check: Check, filter?: false): Lookup
   * }}
   */
  // prettier-ignore
  const factory = (
    /**
     * @param {Check} check
     * @param {boolean} [filter=false]
     */
    function (check, filter) {
      return match

      /**
       * @param {Tag|Tags} tags
       * @param {Range|Ranges} [ranges='*']
       * @returns {Tag|Tags|undefined}
       */
      function match(tags, ranges) {
        let left = cast(tags, 'tag');
        const right = cast(
          ranges === null || ranges === undefined ? '*' : ranges,
          'range'
        );
        /** @type {Tags} */
        const matches = [];
        let rightIndex = -1;

        while (++rightIndex < right.length) {
          const range = right[rightIndex].toLowerCase();

          // Ignore wildcards in lookup mode.
          if (!filter && range === '*') continue

          let leftIndex = -1;
          /** @type {Tags} */
          const next = [];

          while (++leftIndex < left.length) {
            if (check(left[leftIndex].toLowerCase(), range)) {
              // Exit if this is a lookup and we have a match.
              if (!filter) return left[leftIndex]
              matches.push(left[leftIndex]);
            } else {
              next.push(left[leftIndex]);
            }
          }

          left = next;
        }

        // If this is a filter, return the list.  If it’s a lookup, we didn’t find
        // a match, so return `undefined`.
        return filter ? matches : undefined
      }
    }
  );

  /**
   * Extended Filtering (Section 3.3.2) matches a language priority list
   * consisting of extended language ranges (Section 2.2) to sets of language
   * tags.
   * @param {Tag|Tags} tags
   * @param {Range|Ranges} [ranges]
   * @returns {Tags}
   */
  const extendedFilter = factory(
    /** @type {Check} */
    function (tag, range) {
      // 3.3.2.1
      const left = tag.split('-');
      const right = range.split('-');
      let leftIndex = 0;
      let rightIndex = 0;

      // 3.3.2.2
      if (right[rightIndex] !== '*' && left[leftIndex] !== right[rightIndex]) {
        return false
      }

      leftIndex++;
      rightIndex++;

      // 3.3.2.3
      while (rightIndex < right.length) {
        // 3.3.2.3.A
        if (right[rightIndex] === '*') {
          rightIndex++;
          continue
        }

        // 3.3.2.3.B
        if (!left[leftIndex]) return false

        // 3.3.2.3.C
        if (left[leftIndex] === right[rightIndex]) {
          leftIndex++;
          rightIndex++;
          continue
        }

        // 3.3.2.3.D
        if (left[leftIndex].length === 1) return false

        // 3.3.2.3.E
        leftIndex++;
      }

      // 3.3.2.4
      return true
    },
    true
  );

  /**
   * Validate tags or ranges, and cast them to arrays.
   *
   * @param {string|Array<string>} values
   * @param {string} name
   * @returns {Array<string>}
   */
  function cast(values, name) {
    const value = values && typeof values === 'string' ? [values] : values;

    if (!value || typeof value !== 'object' || !('length' in value)) {
      throw new Error(
        'Invalid ' + name + ' `' + value + '`, expected non-empty string'
      )
    }

    return value
  }

  /**
   * @typedef {Object} StringifyOptions
   * @property {boolean} [padLeft=true] Whether to pad a space before a token (`boolean`, default: `true`).
   * @property {boolean} [padRight=false] Whether to pad a space after a token (`boolean`, default: `false`).
   */

  /**
   * Parse comma separated tokens to an array.
   *
   * @param {string} value
   * @returns {Array.<string>}
   */
  function parse$5(value) {
    /** @type {Array.<string>} */
    var tokens = [];
    var input = String(value || '');
    var index = input.indexOf(',');
    var start = 0;
    /** @type {boolean} */
    var end;
    /** @type {string} */
    var token;

    while (!end) {
      if (index === -1) {
        index = input.length;
        end = true;
      }

      token = input.slice(start, index).trim();

      if (token || !end) {
        tokens.push(token);
      }

      start = index + 1;
      index = input.indexOf(',', start);
    }

    return tokens
  }

  /**
   * Serialize an array of strings to comma separated tokens.
   *
   * @param {Array.<string|number>} values
   * @param {StringifyOptions} [options]
   * @returns {string}
   */
  function stringify$4(values, options) {
    var settings = options || {};

    // Ensure the last empty entry is seen.
    if (values[values.length - 1] === '') {
      values = values.concat('');
    }

    return values
      .join(
        (settings.padRight ? ' ' : '') +
          ',' +
          (settings.padLeft === false ? '' : ' ')
      )
      .trim()
  }

  var own$8 = {}.hasOwnProperty;

  /**
   * Check if `node` has a set `name` property.
   *
   * @param {unknown} node
   * @param {string} name
   * @returns {boolean}
   */
  function hasProperty(node, name) {
    /** @type {unknown} */
    var value =
      name &&
      node &&
      typeof node === 'object' &&
      // @ts-ignore Looks like a node.
      node.type === 'element' &&
      // @ts-ignore Looks like an element.
      node.properties &&
      // @ts-ignore Looks like an element.
      own$8.call(node.properties, name) &&
      // @ts-ignore Looks like an element.
      node.properties[name];

    return value !== null && value !== undefined && value !== false
  }

  /**
   * @param {unknown} thing
   * @returns {boolean}
   */
  function whitespace$1(thing) {
    /** @type {string} */
    var value =
      // @ts-ignore looks like a node.
      thing && typeof thing === 'object' && thing.type === 'text'
        ? // @ts-ignore looks like a text.
          thing.value || ''
        : thing;

    // HTML whitespace expression.
    // See <https://html.spec.whatwg.org/#space-character>.
    return typeof value === 'string' && value.replace(/[ \t\n\f\r]/g, '') === ''
  }

  /**
   * @typedef {import('./types.js').Rule} Rule
   * @typedef {import('./types.js').RulePseudo} RulePseudo
   * @typedef {import('./types.js').RulePseudoNth} RulePseudoNth
   * @typedef {import('./types.js').RulePseudoSelector} RulePseudoSelector
   * @typedef {import('./types.js').Parent} Parent
   * @typedef {import('./types.js').Selector} Selector
   * @typedef {import('./types.js').Selectors} Selectors
   * @typedef {import('./types.js').SelectState} SelectState
   * @typedef {import('./types.js').Element} Element
   * @typedef {import('./types.js').ElementChild} ElementChild
   */

  const handle$1 = zwitch('name', {
    // @ts-expect-error: hush.
    unknown: unknownPseudo,
    invalid: invalidPseudo,
    handlers: {
      // @ts-expect-error: hush.
      any: matches,
      // @ts-expect-error: hush.
      'any-link': anyLink,
      // @ts-expect-error: hush.
      blank,
      // @ts-expect-error: hush.
      checked,
      // @ts-expect-error: hush.
      dir,
      // @ts-expect-error: hush.
      disabled,
      // @ts-expect-error: hush.
      empty,
      // @ts-expect-error: hush.
      enabled,
      // @ts-expect-error: hush.
      'first-child': firstChild,
      // @ts-expect-error: hush.
      'first-of-type': firstOfType,
      // @ts-expect-error: hush.
      has,
      // @ts-expect-error: hush.
      lang,
      // @ts-expect-error: hush.
      'last-child': lastChild,
      // @ts-expect-error: hush.
      'last-of-type': lastOfType,
      // @ts-expect-error: hush.
      matches,
      // @ts-expect-error: hush.
      not,
      // @ts-expect-error: hush.
      'nth-child': nthChild,
      // @ts-expect-error: hush.
      'nth-last-child': nthLastChild,
      // @ts-expect-error: hush.
      'nth-of-type': nthOfType,
      // @ts-expect-error: hush.
      'nth-last-of-type': nthLastOfType,
      // @ts-expect-error: hush.
      'only-child': onlyChild,
      // @ts-expect-error: hush.
      'only-of-type': onlyOfType,
      // @ts-expect-error: hush.
      optional,
      // @ts-expect-error: hush.
      'read-only': readOnly,
      // @ts-expect-error: hush.
      'read-write': readWrite,
      // @ts-expect-error: hush.
      required,
      // @ts-expect-error: hush.
      root,
      // @ts-expect-error: hush.
      scope
    }
  });

  pseudo.needsIndex = [
    'first-child',
    'first-of-type',
    'last-child',
    'last-of-type',
    'nth-child',
    'nth-last-child',
    'nth-of-type',
    'nth-last-of-type',
    'only-child',
    'only-of-type'
  ];

  /**
   * @param {Rule} query
   * @param {Element} element
   * @param {number|null} index
   * @param {Parent|null} parent
   * @param {SelectState} state
   * @returns {boolean}
   */
  function pseudo(query, element, index, parent, state) {
    const pseudos = query.pseudos;
    let offset = -1;

    while (++offset < pseudos.length) {
      if (!handle$1(pseudos[offset], element, index, parent, state)) return false
    }

    return true
  }

  /**
   * @param {RulePseudoSelector} query
   * @param {Element} element
   * @param {number|null} _1
   * @param {Parent|null} _2
   * @param {SelectState} state
   * @returns {boolean}
   */
  function matches(query, element, _1, _2, state) {
    const shallow = state.shallow;
    const one = state.one;

    state.shallow = true;
    state.one = true;

    const result = any(query.value, element, state)[0] === element;

    state.shallow = shallow;
    state.one = one;

    return result
  }

  /**
   * @param {RulePseudoSelector} query
   * @param {Element} element
   * @param {number|null} index
   * @param {Parent|null} parent
   * @param {SelectState} state
   * @returns {boolean}
   */
  function not(query, element, index, parent, state) {
    return !matches(query, element, index, parent, state)
  }

  /**
   * @param {RulePseudo} _
   * @param {Element} element
   * @returns {boolean}
   */
  function anyLink(_, element) {
    return (
      isElement$1(element, ['a', 'area', 'link']) && hasProperty(element, 'href')
    )
  }

  /**
   * @param {RulePseudo} _
   * @param {Element} element
   * @returns {boolean}
   */
  function checked(_, element) {
    if (isElement$1(element, ['input', 'menuitem'])) {
      return Boolean(
        element.properties &&
          (element.properties.type === 'checkbox' ||
            element.properties.type === 'radio') &&
          hasProperty(element, 'checked')
      )
    }

    if (isElement$1(element, 'option')) {
      return hasProperty(element, 'selected')
    }

    return false
  }

  /**
   * @param {RulePseudo} query
   * @param {Element} _1
   * @param {number|null} _2
   * @param {Parent|null} _3
   * @param {SelectState} state
   * @returns {boolean}
   */
  function dir(query, _1, _2, _3, state) {
    return state.direction === query.value
  }

  /**
   * @param {RulePseudo} _
   * @param {Element} element
   * @returns {boolean}
   */
  function disabled(_, element) {
    return (
      isElement$1(element, [
        'button',
        'input',
        'select',
        'textarea',
        'optgroup',
        'option',
        'menuitem',
        'fieldset'
      ]) && hasProperty(element, 'disabled')
    )
  }

  /**
   * @param {RulePseudo} query
   * @param {Element} element
   * @returns {boolean}
   */
  function enabled(query, element) {
    return !disabled(query, element)
  }

  /**
   * @param {RulePseudo} _
   * @param {Element} element
   * @returns {boolean}
   */
  function required(_, element) {
    return (
      isElement$1(element, ['input', 'textarea', 'select']) &&
      hasProperty(element, 'required')
    )
  }

  /**
   * @param {RulePseudo} query
   * @param {Element} element
   * @returns {boolean}
   */
  function optional(query, element) {
    return !required(query, element)
  }

  /**
   * @param {RulePseudo} _
   * @param {Element} element
   * @param {number|null} _1
   * @param {Parent|null} _2
   * @param {SelectState} state
   * @returns {boolean}
   */
  function readWrite(_, element, _1, _2, state) {
    return isElement$1(element, ['input', 'textarea'])
      ? !hasProperty(element, 'readOnly') && !hasProperty(element, 'disabled')
      : Boolean(state.editableOrEditingHost)
  }

  /**
   * @param {RulePseudo} query
   * @param {Element} element
   * @param {number|null} index
   * @param {Parent|null} parent
   * @param {SelectState} state
   * @returns {boolean}
   */
  function readOnly(query, element, index, parent, state) {
    return !readWrite(query, element, index, parent, state)
  }

  /**
   * @param {RulePseudo} _
   * @param {Element} element
   * @param {number|null} _1
   * @param {Parent|null} parent
   * @param {SelectState} state
   * @returns {boolean}
   */
  function root(_, element, _1, parent, state) {
    return Boolean(
      (!parent || parent.type === 'root') &&
        state.schema &&
        (state.schema.space === 'html' || state.schema.space === 'svg') &&
        isElement$1(element, ['html', 'svg'])
    )
  }

  /**
   * @param {RulePseudo} _
   * @param {Element} element
   * @param {number|null} _1
   * @param {Parent|null} _2
   * @param {SelectState} state
   * @returns {boolean}
   */
  function scope(_, element, _1, _2, state) {
    return Boolean(
      isElement$1(element) &&
        state.scopeElements &&
        state.scopeElements.includes(element)
    )
  }

  /**
   * @param {RulePseudo} _
   * @param {Element} element
   * @returns {boolean}
   */
  function empty(_, element) {
    return !someChildren(element, check)

    /**
     * @param {ElementChild} child
     * @returns {boolean}
     */
    function check(child) {
      return child.type === 'element' || child.type === 'text'
    }
  }

  /**
   * @param {RulePseudo} _
   * @param {Element} element
   * @returns {boolean}
   */
  function blank(_, element) {
    return !someChildren(element, check)

    /**
     * @param {ElementChild} child
     * @returns {boolean}
     */
    function check(child) {
      return (
        child.type === 'element' || (child.type === 'text' && !whitespace$1(child))
      )
    }
  }

  /**
   * @param {RulePseudo} query
   * @param {Element} _1
   * @param {number|null} _2
   * @param {Parent|null} _3
   * @param {SelectState} state
   * @returns {boolean}
   */
  function firstChild(query, _1, _2, _3, state) {
    assertDeep(state, query);
    return state.elementIndex === 0
  }

  /**
   * @param {RulePseudo} query
   * @param {Element} _1
   * @param {number|null} _2
   * @param {Parent|null} _3
   * @param {SelectState} state
   * @returns {boolean}
   */
  function lang(query, _1, _2, _3, state) {
    return (
      state.language !== '' &&
      state.language !== undefined &&
      state.language !== null &&
      // @ts-expect-error never `selectors`.
      extendedFilter(state.language, parse$5(query.value)).length > 0
    )
  }

  /**
   * @param {RulePseudo} query
   * @param {Element} _1
   * @param {number|null} _2
   * @param {Parent|null} _3
   * @param {SelectState} state
   * @returns {boolean}
   */
  function lastChild(query, _1, _2, _3, state) {
    assertDeep(state, query);
    return Boolean(
      state.elementCount && state.elementIndex === state.elementCount - 1
    )
  }

  /**
   * @param {RulePseudo} query
   * @param {Element} _1
   * @param {number|null} _2
   * @param {Parent|null} _3
   * @param {SelectState} state
   * @returns {boolean}
   */
  function onlyChild(query, _1, _2, _3, state) {
    assertDeep(state, query);
    return state.elementCount === 1
  }

  /**
   * @param {RulePseudoNth} query
   * @param {Element} _1
   * @param {number|null} _2
   * @param {Parent|null} _3
   * @param {SelectState} state
   * @returns {boolean}
   */
  function nthChild(query, _1, _2, _3, state) {
    assertDeep(state, query);
    return (
      typeof state.elementIndex === 'number' && query.value(state.elementIndex)
    )
  }

  /**
   * @param {RulePseudoNth} query
   * @param {Element} _1
   * @param {number|null} _2
   * @param {Parent|null} _3
   * @param {SelectState} state
   * @returns {boolean}
   */
  function nthLastChild(query, _1, _2, _3, state) {
    assertDeep(state, query);
    return Boolean(
      typeof state.elementCount === 'number' &&
        typeof state.elementIndex === 'number' &&
        query.value(state.elementCount - state.elementIndex - 1)
    )
  }

  /**
   * @param {RulePseudoNth} query
   * @param {Element} _1
   * @param {number|null} _2
   * @param {Parent|null} _3
   * @param {SelectState} state
   * @returns {boolean}
   */
  function nthOfType(query, _1, _2, _3, state) {
    assertDeep(state, query);
    return typeof state.typeIndex === 'number' && query.value(state.typeIndex)
  }

  /**
   * @param {RulePseudoNth} query
   * @param {Element} _1
   * @param {number|null} _2
   * @param {Parent|null} _3
   * @param {SelectState} state
   * @returns {boolean}
   */
  function nthLastOfType(query, _1, _2, _3, state) {
    assertDeep(state, query);
    return (
      typeof state.typeCount === 'number' &&
      typeof state.typeIndex === 'number' &&
      query.value(state.typeCount - 1 - state.typeIndex)
    )
  }

  /**
   * @param {RulePseudo} query
   * @param {Element} _1
   * @param {number|null} _2
   * @param {Parent|null} _3
   * @param {SelectState} state
   * @returns {boolean}
   */
  function firstOfType(query, _1, _2, _3, state) {
    assertDeep(state, query);
    return state.typeIndex === 0
  }

  /**
   * @param {RulePseudo} query
   * @param {Element} _1
   * @param {number|null} _2
   * @param {Parent|null} _3
   * @param {SelectState} state
   * @returns {boolean}
   */
  function lastOfType(query, _1, _2, _3, state) {
    assertDeep(state, query);
    return (
      typeof state.typeIndex === 'number' &&
      typeof state.typeCount === 'number' &&
      state.typeIndex === state.typeCount - 1
    )
  }

  /**
   * @param {RulePseudo} query
   * @param {Element} _1
   * @param {number|null} _2
   * @param {Parent|null} _3
   * @param {SelectState} state
   * @returns {boolean}
   */
  function onlyOfType(query, _1, _2, _3, state) {
    assertDeep(state, query);
    return state.typeCount === 1
  }

  /**
   * @param {Element} element
   * @param {(child: ElementChild) => boolean} check
   * @returns {boolean}
   */
  function someChildren(element, check) {
    const children = element.children;
    let index = -1;

    while (++index < children.length) {
      if (check(children[index])) return true
    }

    return false
  }

  // Shouldn’t be called, parser gives correct data.
  /* c8 ignore next 3 */
  function invalidPseudo() {
    throw new Error('Invalid pseudo-selector')
  }

  /**
   * @param {RulePseudo} query
   */
  function unknownPseudo(query) {
    if (query.name) {
      throw new Error('Unknown pseudo-selector `' + query.name + '`')
    }

    throw new Error('Unexpected pseudo-element or empty pseudo-class')
  }

  /**
   * @param {SelectState} state
   * @param {RulePseudo|RulePseudoNth} query
   */
  function assertDeep(state, query) {
    if (state.shallow) {
      throw new Error('Cannot use `:' + query.name + '` without parent')
    }
  }

  /**
   * @param {RulePseudoSelector} query
   * @param {Element} element
   * @param {number|null} _2
   * @param {Parent|null} _3
   * @param {SelectState} state
   * @returns {boolean}
   */
  function has(query, element, _2, _3, state) {
    const shallow = state.shallow;
    const one = state.one;
    const scopeElements = state.scopeElements;
    const value = appendScope(query.value);

    state.shallow = false;
    state.one = true;
    state.scopeElements = [element];

    const result = any(value, element, state).length > 0;

    state.shallow = shallow;
    state.one = one;
    state.scopeElements = scopeElements;

    return result
  }

  /**
   * @param {Selector} value
   * @returns {Selectors}
   */
  function appendScope(value) {
    /** @type {Selectors} */
    const selector =
      value.type === 'ruleSet' ? {type: 'selectors', selectors: [value]} : value;
    let index = -1;

    while (++index < selector.selectors.length) {
      const rule = selector.selectors[index].rule;
      rule.nestingOperator = null;

      if (
        !rule.pseudos ||
        rule.pseudos.length !== 1 ||
        rule.pseudos[0].name !== 'scope'
      ) {
        selector.selectors[index] = {
          type: 'ruleSet',
          // @ts-expect-error pseudos are fine w/ just a name!
          rule: {type: 'rule', rule, pseudos: [{name: 'scope'}]}
        };
      }
    }

    return selector
  }

  /**
   * Parse space separated tokens to an array of strings.
   *
   * @param {string} value Space separated tokens
   * @returns {Array.<string>} Tokens
   */

  /**
   * Serialize an array of strings as space separated tokens.
   *
   * @param {Array.<string|number>} values Tokens
   * @returns {string} Space separated tokens
   */
  function stringify$3(values) {
    return values.join(' ').trim()
  }

  /**
   * @typedef {import('./types.js').Rule} Rule
   * @typedef {import('./types.js').RuleAttr} RuleAttr
   * @typedef {import('./types.js').Element} Element
   * @typedef {import('./types.js').Schema} Schema
   * @typedef {import('./types.js').Info} Info
   * @typedef {import('./types.js').PropertyValue} PropertyValue
   */

  const handle = zwitch('operator', {
    // @ts-expect-error: hush.
    unknown: unknownOperator,
    // @ts-expect-error: hush.
    invalid: exists,
    handlers: {
      // @ts-expect-error: hush.
      '=': exact,
      // @ts-expect-error: hush.
      '~=': spaceSeparatedList,
      // @ts-expect-error: hush.
      '|=': exactOrPrefix,
      // @ts-expect-error: hush.
      '^=': begins,
      // @ts-expect-error: hush.
      '$=': ends,
      // @ts-expect-error: hush.
      '*=': contains
    }
  });

  /**
   * @param {Rule} query
   * @param {Element} element
   * @param {Schema} schema
   * @returns {boolean}
   */
  function attribute(query, element, schema) {
    const attrs = query.attrs;
    let index = -1;

    while (++index < attrs.length) {
      if (!handle(attrs[index], element, find$2(schema, attrs[index].name))) {
        return false
      }
    }

    return true
  }

  /**
   * `[attr]`
   *
   * @param {RuleAttr} _
   * @param {Element} element
   * @param {Info} info
   * @returns {boolean}
   */
  function exists(_, element, info) {
    return hasProperty(element, info.property)
  }

  /**
   * `[attr=value]`
   *
   * @param {RuleAttr} query
   * @param {Element} element
   * @param {Info} info
   * @returns {boolean}
   */
  function exact(query, element, info) {
    return Boolean(
      hasProperty(element, info.property) &&
        element.properties &&
        normalizeValue(element.properties[info.property], info) === query.value
    )
  }

  /**
   * `[attr~=value]`
   *
   * @param {RuleAttr} query
   * @param {Element} element
   * @param {Info} info
   * @returns {boolean}
   */
  function spaceSeparatedList(query, element, info) {
    const value = element.properties && element.properties[info.property];

    return (
      // If this is a comma-separated list, and the query is contained in it, return
      // true.
      (!info.commaSeparated &&
        value &&
        typeof value === 'object' &&
        query.value &&
        value.includes(query.value)) ||
      // For all other values (including comma-separated lists), return whether this
      // is an exact match.
      (hasProperty(element, info.property) &&
        normalizeValue(value, info) === query.value)
    )
  }

  /**
   * `[attr|=value]`
   *
   * @param {RuleAttr} query
   * @param {Element} element
   * @param {Info} info
   * @returns {boolean}
   */
  function exactOrPrefix(query, element, info) {
    const value = normalizeValue(
      element.properties && element.properties[info.property],
      info
    );

    return Boolean(
      hasProperty(element, info.property) &&
        query.value &&
        (value === query.value ||
          (value.slice(0, query.value.length) === query.value &&
            value.charAt(query.value.length) === '-'))
    )
  }

  /**
   * `[attr^=value]`
   *
   * @param {RuleAttr} query
   * @param {Element} element
   * @param {Info} info
   * @returns {boolean}
   */
  function begins(query, element, info) {
    return Boolean(
      hasProperty(element, info.property) &&
        element.properties &&
        query.value &&
        normalizeValue(element.properties[info.property], info).slice(
          0,
          query.value.length
        ) === query.value
    )
  }

  /**
   * `[attr$=value]`
   *
   * @param {RuleAttr} query
   * @param {Element} element
   * @param {Info} info
   * @returns {boolean}
   */
  function ends(query, element, info) {
    return Boolean(
      hasProperty(element, info.property) &&
        element.properties &&
        query.value &&
        normalizeValue(element.properties[info.property], info).slice(
          -query.value.length
        ) === query.value
    )
  }

  /**
   * `[attr*=value]`
   *
   * @param {RuleAttr} query
   * @param {Element} element
   * @param {Info} info
   * @returns {boolean}
   */
  function contains(query, element, info) {
    return Boolean(
      hasProperty(element, info.property) &&
        element.properties &&
        query.value &&
        normalizeValue(element.properties[info.property], info).includes(
          query.value
        )
    )
  }

  // Shouldn’t be called, Parser throws an error instead.
  /**
   * @param {RuleAttr} query
   * @returns {boolean}
   */
  /* c8 ignore next 3 */
  function unknownOperator(query) {
    throw new Error('Unknown operator `' + query.operator + '`')
  }

  /**
   * Stringify a hast value back to its HTML form.
   *
   * @param {PropertyValue} value
   * @param {Info} info
   * @returns {string}
   */
  function normalizeValue(value, info) {
    if (typeof value === 'boolean') {
      return info.attribute
    }

    if (Array.isArray(value)) {
      return (info.commaSeparated ? stringify$4 : stringify$3)(value)
    }

    return String(value)
  }

  /**
   * @typedef {import('./types.js').Rule} Rule
   * @typedef {import('./types.js').Element} Element
   */

  /**
   * @param {Rule} query
   * @param {Element} element
   * @returns {boolean}
   */
  function className(query, element) {
    /** @type {Array.<string>} */
    // @ts-expect-error Assume array.
    const value = element.properties.className || [];
    let index = -1;

    if (query.classNames) {
      while (++index < query.classNames.length) {
        if (!value.includes(query.classNames[index])) return false
      }
    }

    return true
  }

  /**
   * @typedef {import('./types.js').Rule} Rule
   * @typedef {import('./types.js').Element} Element
   */

  /**
   * @param {Rule} query
   * @param {Element} element
   * @returns {boolean}
   */
  function id(query, element) {
    return Boolean(element.properties && element.properties.id === query.id)
  }

  /**
   * @typedef {import('./types.js').Rule} Rule
   * @typedef {import('./types.js').Element} Element
   */

  /**
   * @param {Rule} query
   * @param {Element} element
   * @returns {boolean}
   */
  function name(query, element) {
    return query.tagName === '*' || query.tagName === element.tagName
  }

  /**
   * @typedef {import('./types.js').Rule} Rule
   * @typedef {import('./types.js').HastNode} HastNode
   * @typedef {import('./types.js').Element} Element
   * @typedef {import('./types.js').Parent} Parent
   * @typedef {import('./types.js').SelectState} SelectState
   * @typedef {import('hast-util-is-element').AssertPredicate<Element>} IsElement
   */

  /**
   * @param {Rule} query
   * @param {HastNode} node
   * @param {number|null} index
   * @param {Parent|null} parent
   * @param {SelectState} state
   * @returns {boolean}
   */
  function test(query, node, index, parent, state) {
    return Boolean(
      element$2(node) &&
        state.schema &&
        (!query.tagName || name(query, node)) &&
        (!query.classNames || className(query, node)) &&
        (!query.id || id(query, node)) &&
        (!query.attrs || attribute(query, node, state.schema)) &&
        (!query.pseudos || pseudo(query, node, index, parent, state))
    )
  }

  /**
   * @typedef {import('hast').Element} Element
   * @typedef {import('./types.js').Selectors} Selectors
   * @typedef {import('./types.js').Rule} Rule
   * @typedef {import('./types.js').RuleSet} RuleSet
   * @typedef {import('./types.js').HastNode} HastNode
   * @typedef {import('./types.js').SelectIterator} SelectIterator
   * @typedef {import('./types.js').SelectState} SelectState
   */

  const type = zwitch('type', {
    // @ts-expect-error: hush.
    unknown: unknownType,
    invalid: invalidType,
    // @ts-expect-error: hush.
    handlers: {selectors: selectors$1, ruleSet: ruleSet$1, rule: rule$1}
  });

  /**
   * @param {Selectors|RuleSet|Rule} query
   * @param {HastNode|undefined} node
   * @param {SelectState} state
   * @returns {Array.<Element>}
   */
  function any(query, node, state) {
    // @ts-expect-error zwitch types are off.
    return query && node ? type(query, node, state) : []
  }

  /**
   * @param {Selectors} query
   * @param {HastNode} node
   * @param {SelectState} state
   * @returns {Array.<Element>}
   */
  function selectors$1(query, node, state) {
    const collector = new Collector(state.one);
    let index = -1;

    while (++index < query.selectors.length) {
      collector.collectAll(ruleSet$1(query.selectors[index], node, state));
    }

    return collector.result
  }

  /**
   * @param {RuleSet} query
   * @param {HastNode} node
   * @param {SelectState} state
   * @returns {Array.<Element>}
   */
  function ruleSet$1(query, node, state) {
    return rule$1(query.rule, node, state)
  }

  /**
   * @param {Rule} query
   * @param {HastNode} tree
   * @param {SelectState} state
   * @returns {Array.<Element>}
   */
  function rule$1(query, tree, state) {
    const collector = new Collector(state.one);

    if (state.shallow && query.rule) {
      throw new Error('Expected selector without nesting')
    }

    nest(
      query,
      tree,
      0,
      null,
      configure(query, {
        schema: state.space === 'svg' ? svg$3 : html$6,
        language: undefined,
        direction: 'ltr',
        editableOrEditingHost: false,
        // @ts-expect-error assume elements.
        scopeElements: tree.type === 'root' ? tree.children : [tree],
        iterator,
        one: state.one,
        shallow: state.shallow
      })
    );

    return collector.result

    /** @type {SelectIterator} */
    function iterator(query, node, index, parent, state) {
      const exit = enterState(state, node);

      if (test(query, node, index, parent, state)) {
        if (query.rule) {
          nest(query.rule, node, index, parent, configure(query.rule, state));
        } else {
          // @ts-expect-error `test` also asserts `node is Element`
          collector.collect(node);
          state.found = true;
        }
      }

      exit();
    }

    /**
     * @template {SelectState} S
     * @param {Rule} query
     * @param {S} state
     * @returns {S}
     */
    function configure(query, state) {
      const pseudos = query.pseudos || [];
      let index = -1;

      while (++index < pseudos.length) {
        if (pseudo.needsIndex.includes(pseudos[index].name)) {
          state.index = true;
          break
        }
      }

      return state
    }
  }

  // Shouldn’t be called, all data is handled.
  /* c8 ignore next 6 */
  /**
   * @param {{[x: string]: unknown, type: string}} query
   */
  function unknownType(query) {
    throw new Error('Unknown type `' + query.type + '`')
  }

  // Shouldn’t be called, parser gives correct data.
  /* c8 ignore next 3 */
  function invalidType() {
    throw new Error('Invalid type')
  }

  class Collector {
    /**
     * @param {boolean|undefined} [one]
     */
    constructor(one) {
      /** @type {Array.<Element>} */
      this.result = [];
      /** @type {boolean|undefined} */
      this.one = one;
      /** @type {boolean} */
      this.found = false;
    }

    /**
     * Append nodes to array, filtering out duplicates.
     *
     * @param {Array.<Element>} elements
     */
    collectAll(elements) {
      let index = -1;

      while (++index < elements.length) {
        this.collect(elements[index]);
      }
    }

    /**
     * Append one node.
     *
     * @param {Element} element
     */
    collect(element) {
      if (this.one) {
        // Shouldn’t happen, safeguards performance problems.
        /* c8 ignore next */
        if (this.found) return
        this.found = true;
      }

      if (!this.result.includes(element)) this.result.push(element);
    }
  }

  var lib$1 = {};

  var parserContext = {};

  var utils = {};

  (function (exports) {
  Object.defineProperty(exports, "__esModule", { value: true });
  function isIdentStart(c) {
      return (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') || (c === '-') || (c === '_');
  }
  exports.isIdentStart = isIdentStart;
  function isIdent(c) {
      return (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') || (c >= '0' && c <= '9') || c === '-' || c === '_';
  }
  exports.isIdent = isIdent;
  function isHex(c) {
      return (c >= 'a' && c <= 'f') || (c >= 'A' && c <= 'F') || (c >= '0' && c <= '9');
  }
  exports.isHex = isHex;
  function escapeIdentifier(s) {
      var len = s.length;
      var result = '';
      var i = 0;
      while (i < len) {
          var chr = s.charAt(i);
          if (exports.identSpecialChars[chr]) {
              result += '\\' + chr;
          }
          else {
              if (!(chr === '_' || chr === '-' ||
                  (chr >= 'A' && chr <= 'Z') ||
                  (chr >= 'a' && chr <= 'z') ||
                  (i !== 0 && chr >= '0' && chr <= '9'))) {
                  var charCode = chr.charCodeAt(0);
                  if ((charCode & 0xF800) === 0xD800) {
                      var extraCharCode = s.charCodeAt(i++);
                      if ((charCode & 0xFC00) !== 0xD800 || (extraCharCode & 0xFC00) !== 0xDC00) {
                          throw Error('UCS-2(decode): illegal sequence');
                      }
                      charCode = ((charCode & 0x3FF) << 10) + (extraCharCode & 0x3FF) + 0x10000;
                  }
                  result += '\\' + charCode.toString(16) + ' ';
              }
              else {
                  result += chr;
              }
          }
          i++;
      }
      return result;
  }
  exports.escapeIdentifier = escapeIdentifier;
  function escapeStr(s) {
      var len = s.length;
      var result = '';
      var i = 0;
      var replacement;
      while (i < len) {
          var chr = s.charAt(i);
          if (chr === '"') {
              chr = '\\"';
          }
          else if (chr === '\\') {
              chr = '\\\\';
          }
          else if ((replacement = exports.strReplacementsRev[chr]) !== undefined) {
              chr = replacement;
          }
          result += chr;
          i++;
      }
      return "\"" + result + "\"";
  }
  exports.escapeStr = escapeStr;
  exports.identSpecialChars = {
      '!': true,
      '"': true,
      '#': true,
      '$': true,
      '%': true,
      '&': true,
      '\'': true,
      '(': true,
      ')': true,
      '*': true,
      '+': true,
      ',': true,
      '.': true,
      '/': true,
      ';': true,
      '<': true,
      '=': true,
      '>': true,
      '?': true,
      '@': true,
      '[': true,
      '\\': true,
      ']': true,
      '^': true,
      '`': true,
      '{': true,
      '|': true,
      '}': true,
      '~': true
  };
  exports.strReplacementsRev = {
      '\n': '\\n',
      '\r': '\\r',
      '\t': '\\t',
      '\f': '\\f',
      '\v': '\\v'
  };
  exports.singleQuoteEscapeChars = {
      n: '\n',
      r: '\r',
      t: '\t',
      f: '\f',
      '\\': '\\',
      '\'': '\''
  };
  exports.doubleQuotesEscapeChars = {
      n: '\n',
      r: '\r',
      t: '\t',
      f: '\f',
      '\\': '\\',
      '"': '"'
  };
  }(utils));

  Object.defineProperty(parserContext, "__esModule", { value: true });
  var utils_1$1 = utils;
  function parseCssSelector(str, pos, pseudos, attrEqualityMods, ruleNestingOperators, substitutesEnabled) {
      var l = str.length;
      var chr = '';
      function getStr(quote, escapeTable) {
          var result = '';
          pos++;
          chr = str.charAt(pos);
          while (pos < l) {
              if (chr === quote) {
                  pos++;
                  return result;
              }
              else if (chr === '\\') {
                  pos++;
                  chr = str.charAt(pos);
                  var esc = void 0;
                  if (chr === quote) {
                      result += quote;
                  }
                  else if ((esc = escapeTable[chr]) !== undefined) {
                      result += esc;
                  }
                  else if (utils_1$1.isHex(chr)) {
                      var hex = chr;
                      pos++;
                      chr = str.charAt(pos);
                      while (utils_1$1.isHex(chr)) {
                          hex += chr;
                          pos++;
                          chr = str.charAt(pos);
                      }
                      if (chr === ' ') {
                          pos++;
                          chr = str.charAt(pos);
                      }
                      result += String.fromCharCode(parseInt(hex, 16));
                      continue;
                  }
                  else {
                      result += chr;
                  }
              }
              else {
                  result += chr;
              }
              pos++;
              chr = str.charAt(pos);
          }
          return result;
      }
      function getIdent() {
          var result = '';
          chr = str.charAt(pos);
          while (pos < l) {
              if (utils_1$1.isIdent(chr)) {
                  result += chr;
              }
              else if (chr === '\\') {
                  pos++;
                  if (pos >= l) {
                      throw Error('Expected symbol but end of file reached.');
                  }
                  chr = str.charAt(pos);
                  if (utils_1$1.identSpecialChars[chr]) {
                      result += chr;
                  }
                  else if (utils_1$1.isHex(chr)) {
                      var hex = chr;
                      pos++;
                      chr = str.charAt(pos);
                      while (utils_1$1.isHex(chr)) {
                          hex += chr;
                          pos++;
                          chr = str.charAt(pos);
                      }
                      if (chr === ' ') {
                          pos++;
                          chr = str.charAt(pos);
                      }
                      result += String.fromCharCode(parseInt(hex, 16));
                      continue;
                  }
                  else {
                      result += chr;
                  }
              }
              else {
                  return result;
              }
              pos++;
              chr = str.charAt(pos);
          }
          return result;
      }
      function skipWhitespace() {
          chr = str.charAt(pos);
          var result = false;
          while (chr === ' ' || chr === "\t" || chr === "\n" || chr === "\r" || chr === "\f") {
              result = true;
              pos++;
              chr = str.charAt(pos);
          }
          return result;
      }
      function parse() {
          var res = parseSelector();
          if (pos < l) {
              throw Error('Rule expected but "' + str.charAt(pos) + '" found.');
          }
          return res;
      }
      function parseSelector() {
          var selector = parseSingleSelector();
          if (!selector) {
              return null;
          }
          var res = selector;
          chr = str.charAt(pos);
          while (chr === ',') {
              pos++;
              skipWhitespace();
              if (res.type !== 'selectors') {
                  res = {
                      type: 'selectors',
                      selectors: [selector]
                  };
              }
              selector = parseSingleSelector();
              if (!selector) {
                  throw Error('Rule expected after ",".');
              }
              res.selectors.push(selector);
          }
          return res;
      }
      function parseSingleSelector() {
          skipWhitespace();
          var selector = {
              type: 'ruleSet'
          };
          var rule = parseRule();
          if (!rule) {
              return null;
          }
          var currentRule = selector;
          while (rule) {
              rule.type = 'rule';
              currentRule.rule = rule;
              currentRule = rule;
              skipWhitespace();
              chr = str.charAt(pos);
              if (pos >= l || chr === ',' || chr === ')') {
                  break;
              }
              if (ruleNestingOperators[chr]) {
                  var op = chr;
                  pos++;
                  skipWhitespace();
                  rule = parseRule();
                  if (!rule) {
                      throw Error('Rule expected after "' + op + '".');
                  }
                  rule.nestingOperator = op;
              }
              else {
                  rule = parseRule();
                  if (rule) {
                      rule.nestingOperator = null;
                  }
              }
          }
          return selector;
      }
      // @ts-ignore no-overlap
      function parseRule() {
          var rule = null;
          while (pos < l) {
              chr = str.charAt(pos);
              if (chr === '*') {
                  pos++;
                  (rule = rule || {}).tagName = '*';
              }
              else if (utils_1$1.isIdentStart(chr) || chr === '\\') {
                  (rule = rule || {}).tagName = getIdent();
              }
              else if (chr === '.') {
                  pos++;
                  rule = rule || {};
                  (rule.classNames = rule.classNames || []).push(getIdent());
              }
              else if (chr === '#') {
                  pos++;
                  (rule = rule || {}).id = getIdent();
              }
              else if (chr === '[') {
                  pos++;
                  skipWhitespace();
                  var attr = {
                      name: getIdent()
                  };
                  skipWhitespace();
                  // @ts-ignore
                  if (chr === ']') {
                      pos++;
                  }
                  else {
                      var operator = '';
                      if (attrEqualityMods[chr]) {
                          operator = chr;
                          pos++;
                          chr = str.charAt(pos);
                      }
                      if (pos >= l) {
                          throw Error('Expected "=" but end of file reached.');
                      }
                      if (chr !== '=') {
                          throw Error('Expected "=" but "' + chr + '" found.');
                      }
                      attr.operator = operator + '=';
                      pos++;
                      skipWhitespace();
                      var attrValue = '';
                      attr.valueType = 'string';
                      // @ts-ignore
                      if (chr === '"') {
                          attrValue = getStr('"', utils_1$1.doubleQuotesEscapeChars);
                          // @ts-ignore
                      }
                      else if (chr === '\'') {
                          attrValue = getStr('\'', utils_1$1.singleQuoteEscapeChars);
                          // @ts-ignore
                      }
                      else if (substitutesEnabled && chr === '$') {
                          pos++;
                          attrValue = getIdent();
                          attr.valueType = 'substitute';
                      }
                      else {
                          while (pos < l) {
                              if (chr === ']') {
                                  break;
                              }
                              attrValue += chr;
                              pos++;
                              chr = str.charAt(pos);
                          }
                          attrValue = attrValue.trim();
                      }
                      skipWhitespace();
                      if (pos >= l) {
                          throw Error('Expected "]" but end of file reached.');
                      }
                      if (chr !== ']') {
                          throw Error('Expected "]" but "' + chr + '" found.');
                      }
                      pos++;
                      attr.value = attrValue;
                  }
                  rule = rule || {};
                  (rule.attrs = rule.attrs || []).push(attr);
              }
              else if (chr === ':') {
                  pos++;
                  var pseudoName = getIdent();
                  var pseudo = {
                      name: pseudoName
                  };
                  // @ts-ignore
                  if (chr === '(') {
                      pos++;
                      var value = '';
                      skipWhitespace();
                      if (pseudos[pseudoName] === 'selector') {
                          pseudo.valueType = 'selector';
                          value = parseSelector();
                      }
                      else {
                          pseudo.valueType = pseudos[pseudoName] || 'string';
                          // @ts-ignore
                          if (chr === '"') {
                              value = getStr('"', utils_1$1.doubleQuotesEscapeChars);
                              // @ts-ignore
                          }
                          else if (chr === '\'') {
                              value = getStr('\'', utils_1$1.singleQuoteEscapeChars);
                              // @ts-ignore
                          }
                          else if (substitutesEnabled && chr === '$') {
                              pos++;
                              value = getIdent();
                              pseudo.valueType = 'substitute';
                          }
                          else {
                              while (pos < l) {
                                  if (chr === ')') {
                                      break;
                                  }
                                  value += chr;
                                  pos++;
                                  chr = str.charAt(pos);
                              }
                              value = value.trim();
                          }
                          skipWhitespace();
                      }
                      if (pos >= l) {
                          throw Error('Expected ")" but end of file reached.');
                      }
                      if (chr !== ')') {
                          throw Error('Expected ")" but "' + chr + '" found.');
                      }
                      pos++;
                      pseudo.value = value;
                  }
                  rule = rule || {};
                  (rule.pseudos = rule.pseudos || []).push(pseudo);
              }
              else {
                  break;
              }
          }
          return rule;
      }
      return parse();
  }
  parserContext.parseCssSelector = parseCssSelector;

  var render = {};

  Object.defineProperty(render, "__esModule", { value: true });
  var utils_1 = utils;
  function renderEntity(entity) {
      var res = '';
      switch (entity.type) {
          case 'ruleSet':
              var currentEntity = entity.rule;
              var parts = [];
              while (currentEntity) {
                  if (currentEntity.nestingOperator) {
                      parts.push(currentEntity.nestingOperator);
                  }
                  parts.push(renderEntity(currentEntity));
                  currentEntity = currentEntity.rule;
              }
              res = parts.join(' ');
              break;
          case 'selectors':
              res = entity.selectors.map(renderEntity).join(', ');
              break;
          case 'rule':
              if (entity.tagName) {
                  if (entity.tagName === '*') {
                      res = '*';
                  }
                  else {
                      res = utils_1.escapeIdentifier(entity.tagName);
                  }
              }
              if (entity.id) {
                  res += "#" + utils_1.escapeIdentifier(entity.id);
              }
              if (entity.classNames) {
                  res += entity.classNames.map(function (cn) {
                      return "." + (utils_1.escapeIdentifier(cn));
                  }).join('');
              }
              if (entity.attrs) {
                  res += entity.attrs.map(function (attr) {
                      if ('operator' in attr) {
                          if (attr.valueType === 'substitute') {
                              return "[" + utils_1.escapeIdentifier(attr.name) + attr.operator + "$" + attr.value + "]";
                          }
                          else {
                              return "[" + utils_1.escapeIdentifier(attr.name) + attr.operator + utils_1.escapeStr(attr.value) + "]";
                          }
                      }
                      else {
                          return "[" + utils_1.escapeIdentifier(attr.name) + "]";
                      }
                  }).join('');
              }
              if (entity.pseudos) {
                  res += entity.pseudos.map(function (pseudo) {
                      if (pseudo.valueType) {
                          if (pseudo.valueType === 'selector') {
                              return ":" + utils_1.escapeIdentifier(pseudo.name) + "(" + renderEntity(pseudo.value) + ")";
                          }
                          else if (pseudo.valueType === 'substitute') {
                              return ":" + utils_1.escapeIdentifier(pseudo.name) + "($" + pseudo.value + ")";
                          }
                          else if (pseudo.valueType === 'numeric') {
                              return ":" + utils_1.escapeIdentifier(pseudo.name) + "(" + pseudo.value + ")";
                          }
                          else {
                              return (":" + utils_1.escapeIdentifier(pseudo.name) +
                                  "(" + utils_1.escapeIdentifier(pseudo.value) + ")");
                          }
                      }
                      else {
                          return ":" + utils_1.escapeIdentifier(pseudo.name);
                      }
                  }).join('');
              }
              break;
          default:
              throw Error('Unknown entity type: "' + entity.type + '".');
      }
      return res;
  }
  render.renderEntity = renderEntity;

  Object.defineProperty(lib$1, "__esModule", { value: true });
  var parser_context_1 = parserContext;
  var render_1 = render;
  var CssSelectorParser = /** @class */ (function () {
      function CssSelectorParser() {
          this.pseudos = {};
          this.attrEqualityMods = {};
          this.ruleNestingOperators = {};
          this.substitutesEnabled = false;
      }
      CssSelectorParser.prototype.registerSelectorPseudos = function () {
          var pseudos = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              pseudos[_i] = arguments[_i];
          }
          for (var _a = 0, pseudos_1 = pseudos; _a < pseudos_1.length; _a++) {
              var pseudo = pseudos_1[_a];
              this.pseudos[pseudo] = 'selector';
          }
          return this;
      };
      CssSelectorParser.prototype.unregisterSelectorPseudos = function () {
          var pseudos = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              pseudos[_i] = arguments[_i];
          }
          for (var _a = 0, pseudos_2 = pseudos; _a < pseudos_2.length; _a++) {
              var pseudo = pseudos_2[_a];
              delete this.pseudos[pseudo];
          }
          return this;
      };
      CssSelectorParser.prototype.registerNumericPseudos = function () {
          var pseudos = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              pseudos[_i] = arguments[_i];
          }
          for (var _a = 0, pseudos_3 = pseudos; _a < pseudos_3.length; _a++) {
              var pseudo = pseudos_3[_a];
              this.pseudos[pseudo] = 'numeric';
          }
          return this;
      };
      CssSelectorParser.prototype.unregisterNumericPseudos = function () {
          var pseudos = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              pseudos[_i] = arguments[_i];
          }
          for (var _a = 0, pseudos_4 = pseudos; _a < pseudos_4.length; _a++) {
              var pseudo = pseudos_4[_a];
              delete this.pseudos[pseudo];
          }
          return this;
      };
      CssSelectorParser.prototype.registerNestingOperators = function () {
          var operators = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              operators[_i] = arguments[_i];
          }
          for (var _a = 0, operators_1 = operators; _a < operators_1.length; _a++) {
              var operator = operators_1[_a];
              this.ruleNestingOperators[operator] = true;
          }
          return this;
      };
      CssSelectorParser.prototype.unregisterNestingOperators = function () {
          var operators = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              operators[_i] = arguments[_i];
          }
          for (var _a = 0, operators_2 = operators; _a < operators_2.length; _a++) {
              var operator = operators_2[_a];
              delete this.ruleNestingOperators[operator];
          }
          return this;
      };
      CssSelectorParser.prototype.registerAttrEqualityMods = function () {
          var mods = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              mods[_i] = arguments[_i];
          }
          for (var _a = 0, mods_1 = mods; _a < mods_1.length; _a++) {
              var mod = mods_1[_a];
              this.attrEqualityMods[mod] = true;
          }
          return this;
      };
      CssSelectorParser.prototype.unregisterAttrEqualityMods = function () {
          var mods = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              mods[_i] = arguments[_i];
          }
          for (var _a = 0, mods_2 = mods; _a < mods_2.length; _a++) {
              var mod = mods_2[_a];
              delete this.attrEqualityMods[mod];
          }
          return this;
      };
      CssSelectorParser.prototype.enableSubstitutes = function () {
          this.substitutesEnabled = true;
          return this;
      };
      CssSelectorParser.prototype.disableSubstitutes = function () {
          this.substitutesEnabled = false;
          return this;
      };
      CssSelectorParser.prototype.parse = function (str) {
          return parser_context_1.parseCssSelector(str, 0, this.pseudos, this.attrEqualityMods, this.ruleNestingOperators, this.substitutesEnabled);
      };
      CssSelectorParser.prototype.render = function (path) {
          return render_1.renderEntity(path).trim();
      };
      return CssSelectorParser;
  }());
  var CssSelectorParser_1 = lib$1.CssSelectorParser = CssSelectorParser;

  var lib = {};

  var parse$4 = {};

  // Following http://www.w3.org/TR/css3-selectors/#nth-child-pseudo
  Object.defineProperty(parse$4, "__esModule", { value: true });
  parse$4.parse = void 0;
  // Whitespace as per https://www.w3.org/TR/selectors-3/#lex is " \t\r\n\f"
  var whitespace = new Set([9, 10, 12, 13, 32]);
  var ZERO = "0".charCodeAt(0);
  var NINE = "9".charCodeAt(0);
  /**
   * Parses an expression.
   *
   * @throws An `Error` if parsing fails.
   * @returns An array containing the integer step size and the integer offset of the nth rule.
   * @example nthCheck.parse("2n+3"); // returns [2, 3]
   */
  function parse$3(formula) {
      formula = formula.trim().toLowerCase();
      if (formula === "even") {
          return [2, 0];
      }
      else if (formula === "odd") {
          return [2, 1];
      }
      // Parse [ ['-'|'+']? INTEGER? {N} [ S* ['-'|'+'] S* INTEGER ]?
      var idx = 0;
      var a = 0;
      var sign = readSign();
      var number = readNumber();
      if (idx < formula.length && formula.charAt(idx) === "n") {
          idx++;
          a = sign * (number !== null && number !== void 0 ? number : 1);
          skipWhitespace();
          if (idx < formula.length) {
              sign = readSign();
              skipWhitespace();
              number = readNumber();
          }
          else {
              sign = number = 0;
          }
      }
      // Throw if there is anything else
      if (number === null || idx < formula.length) {
          throw new Error("n-th rule couldn't be parsed ('" + formula + "')");
      }
      return [a, sign * number];
      function readSign() {
          if (formula.charAt(idx) === "-") {
              idx++;
              return -1;
          }
          if (formula.charAt(idx) === "+") {
              idx++;
          }
          return 1;
      }
      function readNumber() {
          var start = idx;
          var value = 0;
          while (idx < formula.length &&
              formula.charCodeAt(idx) >= ZERO &&
              formula.charCodeAt(idx) <= NINE) {
              value = value * 10 + (formula.charCodeAt(idx) - ZERO);
              idx++;
          }
          // Return `null` if we didn't read anything.
          return idx === start ? null : value;
      }
      function skipWhitespace() {
          while (idx < formula.length &&
              whitespace.has(formula.charCodeAt(idx))) {
              idx++;
          }
      }
  }
  parse$4.parse = parse$3;

  var compile$2 = {};

  var boolbase = {
    trueFunc: function trueFunc(){
      return true;
    },
    falseFunc: function falseFunc(){
      return false;
    }
  };

  Object.defineProperty(compile$2, "__esModule", { value: true });
  compile$2.compile = void 0;
  var boolbase_1 = boolbase;
  /**
   * Returns a function that checks if an elements index matches the given rule
   * highly optimized to return the fastest solution.
   *
   * @param parsed A tuple [a, b], as returned by `parse`.
   * @returns A highly optimized function that returns whether an index matches the nth-check.
   * @example
   * const check = nthCheck.compile([2, 3]);
   *
   * check(0); // `false`
   * check(1); // `false`
   * check(2); // `true`
   * check(3); // `false`
   * check(4); // `true`
   * check(5); // `false`
   * check(6); // `true`
   */
  function compile$1(parsed) {
      var a = parsed[0];
      // Subtract 1 from `b`, to convert from one- to zero-indexed.
      var b = parsed[1] - 1;
      /*
       * When `b <= 0`, `a * n` won't be lead to any matches for `a < 0`.
       * Besides, the specification states that no elements are
       * matched when `a` and `b` are 0.
       *
       * `b < 0` here as we subtracted 1 from `b` above.
       */
      if (b < 0 && a <= 0)
          return boolbase_1.falseFunc;
      // When `a` is in the range -1..1, it matches any element (so only `b` is checked).
      if (a === -1)
          return function (index) { return index <= b; };
      if (a === 0)
          return function (index) { return index === b; };
      // When `b <= 0` and `a === 1`, they match any element.
      if (a === 1)
          return b < 0 ? boolbase_1.trueFunc : function (index) { return index >= b; };
      /*
       * Otherwise, modulo can be used to check if there is a match.
       *
       * Modulo doesn't care about the sign, so let's use `a`s absolute value.
       */
      var absA = Math.abs(a);
      // Get `b mod a`, + a if this is negative.
      var bMod = ((b % absA) + absA) % absA;
      return a > 1
          ? function (index) { return index >= b && index % absA === bMod; }
          : function (index) { return index <= b && index % absA === bMod; };
  }
  compile$2.compile = compile$1;

  (function (exports) {
  Object.defineProperty(exports, "__esModule", { value: true });
  exports.compile = exports.parse = void 0;
  var parse_1 = parse$4;
  Object.defineProperty(exports, "parse", { enumerable: true, get: function () { return parse_1.parse; } });
  var compile_1 = compile$2;
  Object.defineProperty(exports, "compile", { enumerable: true, get: function () { return compile_1.compile; } });
  /**
   * Parses and compiles a formula to a highly optimized function.
   * Combination of `parse` and `compile`.
   *
   * If the formula doesn't match any elements,
   * it returns [`boolbase`](https://github.com/fb55/boolbase)'s `falseFunc`.
   * Otherwise, a function accepting an _index_ is returned, which returns
   * whether or not the passed _index_ matches the formula.
   *
   * Note: The nth-rule starts counting at `1`, the returned function at `0`.
   *
   * @param formula The formula to compile.
   * @example
   * const check = nthCheck("2n+3");
   *
   * check(0); // `false`
   * check(1); // `false`
   * check(2); // `true`
   * check(3); // `false`
   * check(4); // `true`
   * check(5); // `false`
   * check(6); // `true`
   */
  function nthCheck(formula) {
      return (0, compile_1.compile)((0, parse_1.parse)(formula));
  }
  exports.default = nthCheck;
  }(lib));

  var fauxEsmNthCheck = /*@__PURE__*/getDefaultExportFromCjs(lib);

  /**
   * @typedef {import('./types.js').Selector} Selector
   * @typedef {import('./types.js').Selectors} Selectors
   * @typedef {import('./types.js').RuleSet} RuleSet
   * @typedef {import('./types.js').Rule} Rule
   * @typedef {import('./types.js').RulePseudo} RulePseudo
   * @typedef {import('./types.js').RulePseudoNth} RulePseudoNth
   */

  /** @type {import('nth-check').default} */
  // @ts-expect-error
  const nthCheck = fauxEsmNthCheck.default;

  const nth = new Set([
    'nth-child',
    'nth-last-child',
    'nth-of-type',
    'nth-last-of-type'
  ]);

  const parser$1 = new CssSelectorParser_1();

  // @ts-expect-error: hush.
  const compile = zwitch('type', {handlers: {selectors, ruleSet, rule}});

  parser$1.registerAttrEqualityMods('~', '|', '^', '$', '*');
  parser$1.registerSelectorPseudos('any', 'matches', 'not', 'has');
  parser$1.registerNestingOperators('>', '+', '~');

  /**
   * @param {string} selector
   * @returns {Selector}
   */
  function parse$2(selector) {
    if (typeof selector !== 'string') {
      throw new TypeError('Expected `string` as selector, not `' + selector + '`')
    }

    // @ts-expect-error types are wrong.
    return compile(parser$1.parse(selector))
  }

  /**
   * @param {Selectors} query
   * @returns {Selectors}
   */
  function selectors(query) {
    let index = -1;

    while (++index < query.selectors.length) {
      compile(query.selectors[index]);
    }

    return query
  }

  /**
   * @param {RuleSet} query
   * @returns {Rule}
   */
  function ruleSet(query) {
    return rule(query.rule)
  }

  /**
   * @param {Rule} query
   * @returns {Rule}
   */
  function rule(query) {
    const pseudos = query.pseudos || [];
    let index = -1;

    while (++index < pseudos.length) {
      const pseudo = pseudos[index];

      if (nth.has(pseudo.name)) {
        // @ts-expect-error Patch a non-primitive type.
        pseudo.value = nthCheck(pseudo.value);
        // @ts-expect-error Patch a non-primitive type.
        pseudo.valueType = 'function';
      }
    }

    compile(query.rule);

    return query
  }

  /**
   * @typedef {import('./lib/types.js').Element} Element
   * @typedef {import('./lib/types.js').HastNode} HastNode
   * @typedef {import('./lib/types.js').Space} Space
   */

  /**
   * @param {string} selector
   * @param {HastNode} [node]
   * @param {Space} [space]
   * @returns {Array.<Element>}
   */
  function selectAll(selector, node, space) {
    return any(parse$2(selector), node, {space})
  }

  const remarkRewrite = (options) => {
      const { selector, rewrite } = options || {};
      return (tree) => {
          if (!rewrite || typeof rewrite !== 'function')
              return;
          if (selector && typeof selector === 'string') {
              const selected = selectAll(selector, tree);
              if (selected && selected.length > 0) {
                  visit$2(tree, selected, (node, index, parent) => {
                      rewrite(node, index, parent);
                  });
              }
              return;
          }
          visit$2(tree, (node, index, parent) => {
              rewrite(node, index, parent);
          });
      };
  };
  var rehypeRewrite = remarkRewrite;

  /**
   * @typedef {import('./info.js').Info} Info
   * @typedef {Record<string, Info>} Properties
   * @typedef {Record<string, string>} Normal
   */

  class Schema$1 {
    /**
     * @constructor
     * @param {Properties} property
     * @param {Normal} normal
     * @param {string} [space]
     */
    constructor(property, normal, space) {
      this.property = property;
      this.normal = normal;
      if (space) {
        this.space = space;
      }
    }
  }

  /** @type {Properties} */
  Schema$1.prototype.property = {};
  /** @type {Normal} */
  Schema$1.prototype.normal = {};
  /** @type {string|null} */
  Schema$1.prototype.space = null;

  /**
   * @typedef {import('./schema.js').Properties} Properties
   * @typedef {import('./schema.js').Normal} Normal
   */

  /**
   * @param {Schema[]} definitions
   * @param {string} [space]
   * @returns {Schema}
   */
  function merge$1(definitions, space) {
    /** @type {Properties} */
    const property = {};
    /** @type {Normal} */
    const normal = {};
    let index = -1;

    while (++index < definitions.length) {
      Object.assign(property, definitions[index].property);
      Object.assign(normal, definitions[index].normal);
    }

    return new Schema$1(property, normal, space)
  }

  /**
   * @param {string} value
   * @returns {string}
   */
  function normalize$1(value) {
    return value.toLowerCase()
  }

  class Info$1 {
    /**
     * @constructor
     * @param {string} property
     * @param {string} attribute
     */
    constructor(property, attribute) {
      /** @type {string} */
      this.property = property;
      /** @type {string} */
      this.attribute = attribute;
    }
  }

  /** @type {string|null} */
  Info$1.prototype.space = null;
  Info$1.prototype.boolean = false;
  Info$1.prototype.booleanish = false;
  Info$1.prototype.overloadedBoolean = false;
  Info$1.prototype.number = false;
  Info$1.prototype.commaSeparated = false;
  Info$1.prototype.spaceSeparated = false;
  Info$1.prototype.commaOrSpaceSeparated = false;
  Info$1.prototype.mustUseProperty = false;
  Info$1.prototype.defined = false;

  let powers$1 = 0;

  const boolean$1 = increment$1();
  const booleanish$1 = increment$1();
  const overloadedBoolean$1 = increment$1();
  const number$1 = increment$1();
  const spaceSeparated$1 = increment$1();
  const commaSeparated$1 = increment$1();
  const commaOrSpaceSeparated$1 = increment$1();

  function increment$1() {
    return 2 ** ++powers$1
  }

  var types$1 = /*#__PURE__*/Object.freeze({
      __proto__: null,
      boolean: boolean$1,
      booleanish: booleanish$1,
      overloadedBoolean: overloadedBoolean$1,
      number: number$1,
      spaceSeparated: spaceSeparated$1,
      commaSeparated: commaSeparated$1,
      commaOrSpaceSeparated: commaOrSpaceSeparated$1
  });

  /** @type {Array<keyof types>} */
  // @ts-expect-error: hush.
  const checks$1 = Object.keys(types$1);

  class DefinedInfo$1 extends Info$1 {
    /**
     * @constructor
     * @param {string} property
     * @param {string} attribute
     * @param {number|null} [mask]
     * @param {string} [space]
     */
    constructor(property, attribute, mask, space) {
      let index = -1;

      super(property, attribute);

      mark$1(this, 'space', space);

      if (typeof mask === 'number') {
        while (++index < checks$1.length) {
          const check = checks$1[index];
          mark$1(this, checks$1[index], (mask & types$1[check]) === types$1[check]);
        }
      }
    }
  }

  DefinedInfo$1.prototype.defined = true;

  /**
   * @param {DefinedInfo} values
   * @param {string} key
   * @param {unknown} value
   */
  function mark$1(values, key, value) {
    if (value) {
      // @ts-expect-error: assume `value` matches the expected value of `key`.
      values[key] = value;
    }
  }

  /**
   * @typedef {import('./schema.js').Properties} Properties
   * @typedef {import('./schema.js').Normal} Normal
   *
   * @typedef {Record<string, string>} Attributes
   *
   * @typedef {Object} Definition
   * @property {Record<string, number|null>} properties
   * @property {(attributes: Attributes, property: string) => string} transform
   * @property {string} [space]
   * @property {Attributes} [attributes]
   * @property {Array<string>} [mustUseProperty]
   */

  const own$7 = {}.hasOwnProperty;

  /**
   * @param {Definition} definition
   * @returns {Schema}
   */
  function create$1(definition) {
    /** @type {Properties} */
    const property = {};
    /** @type {Normal} */
    const normal = {};
    /** @type {string} */
    let prop;

    for (prop in definition.properties) {
      if (own$7.call(definition.properties, prop)) {
        const value = definition.properties[prop];
        const info = new DefinedInfo$1(
          prop,
          definition.transform(definition.attributes || {}, prop),
          value,
          definition.space
        );

        if (
          definition.mustUseProperty &&
          definition.mustUseProperty.includes(prop)
        ) {
          info.mustUseProperty = true;
        }

        property[prop] = info;

        normal[normalize$1(prop)] = prop;
        normal[normalize$1(info.attribute)] = prop;
      }
    }

    return new Schema$1(property, normal, definition.space)
  }

  const xlink$1 = create$1({
    space: 'xlink',
    transform(_, prop) {
      return 'xlink:' + prop.slice(5).toLowerCase()
    },
    properties: {
      xLinkActuate: null,
      xLinkArcRole: null,
      xLinkHref: null,
      xLinkRole: null,
      xLinkShow: null,
      xLinkTitle: null,
      xLinkType: null
    }
  });

  const xml$1 = create$1({
    space: 'xml',
    transform(_, prop) {
      return 'xml:' + prop.slice(3).toLowerCase()
    },
    properties: {xmlLang: null, xmlBase: null, xmlSpace: null}
  });

  /**
   * @param {Record<string, string>} attributes
   * @param {string} attribute
   * @returns {string}
   */
  function caseSensitiveTransform$1(attributes, attribute) {
    return attribute in attributes ? attributes[attribute] : attribute
  }

  /**
   * @param {Record<string, string>} attributes
   * @param {string} property
   * @returns {string}
   */
  function caseInsensitiveTransform$1(attributes, property) {
    return caseSensitiveTransform$1(attributes, property.toLowerCase())
  }

  const xmlns$1 = create$1({
    space: 'xmlns',
    attributes: {xmlnsxlink: 'xmlns:xlink'},
    transform: caseInsensitiveTransform$1,
    properties: {xmlns: null, xmlnsXLink: null}
  });

  const aria$1 = create$1({
    transform(_, prop) {
      return prop === 'role' ? prop : 'aria-' + prop.slice(4).toLowerCase()
    },
    properties: {
      ariaActiveDescendant: null,
      ariaAtomic: booleanish$1,
      ariaAutoComplete: null,
      ariaBusy: booleanish$1,
      ariaChecked: booleanish$1,
      ariaColCount: number$1,
      ariaColIndex: number$1,
      ariaColSpan: number$1,
      ariaControls: spaceSeparated$1,
      ariaCurrent: null,
      ariaDescribedBy: spaceSeparated$1,
      ariaDetails: null,
      ariaDisabled: booleanish$1,
      ariaDropEffect: spaceSeparated$1,
      ariaErrorMessage: null,
      ariaExpanded: booleanish$1,
      ariaFlowTo: spaceSeparated$1,
      ariaGrabbed: booleanish$1,
      ariaHasPopup: null,
      ariaHidden: booleanish$1,
      ariaInvalid: null,
      ariaKeyShortcuts: null,
      ariaLabel: null,
      ariaLabelledBy: spaceSeparated$1,
      ariaLevel: number$1,
      ariaLive: null,
      ariaModal: booleanish$1,
      ariaMultiLine: booleanish$1,
      ariaMultiSelectable: booleanish$1,
      ariaOrientation: null,
      ariaOwns: spaceSeparated$1,
      ariaPlaceholder: null,
      ariaPosInSet: number$1,
      ariaPressed: booleanish$1,
      ariaReadOnly: booleanish$1,
      ariaRelevant: null,
      ariaRequired: booleanish$1,
      ariaRoleDescription: spaceSeparated$1,
      ariaRowCount: number$1,
      ariaRowIndex: number$1,
      ariaRowSpan: number$1,
      ariaSelected: booleanish$1,
      ariaSetSize: number$1,
      ariaSort: null,
      ariaValueMax: number$1,
      ariaValueMin: number$1,
      ariaValueNow: number$1,
      ariaValueText: null,
      role: null
    }
  });

  const html$5 = create$1({
    space: 'html',
    attributes: {
      acceptcharset: 'accept-charset',
      classname: 'class',
      htmlfor: 'for',
      httpequiv: 'http-equiv'
    },
    transform: caseInsensitiveTransform$1,
    mustUseProperty: ['checked', 'multiple', 'muted', 'selected'],
    properties: {
      // Standard Properties.
      abbr: null,
      accept: commaSeparated$1,
      acceptCharset: spaceSeparated$1,
      accessKey: spaceSeparated$1,
      action: null,
      allow: null,
      allowFullScreen: boolean$1,
      allowPaymentRequest: boolean$1,
      allowUserMedia: boolean$1,
      alt: null,
      as: null,
      async: boolean$1,
      autoCapitalize: null,
      autoComplete: spaceSeparated$1,
      autoFocus: boolean$1,
      autoPlay: boolean$1,
      capture: boolean$1,
      charSet: null,
      checked: boolean$1,
      cite: null,
      className: spaceSeparated$1,
      cols: number$1,
      colSpan: null,
      content: null,
      contentEditable: booleanish$1,
      controls: boolean$1,
      controlsList: spaceSeparated$1,
      coords: number$1 | commaSeparated$1,
      crossOrigin: null,
      data: null,
      dateTime: null,
      decoding: null,
      default: boolean$1,
      defer: boolean$1,
      dir: null,
      dirName: null,
      disabled: boolean$1,
      download: overloadedBoolean$1,
      draggable: booleanish$1,
      encType: null,
      enterKeyHint: null,
      form: null,
      formAction: null,
      formEncType: null,
      formMethod: null,
      formNoValidate: boolean$1,
      formTarget: null,
      headers: spaceSeparated$1,
      height: number$1,
      hidden: boolean$1,
      high: number$1,
      href: null,
      hrefLang: null,
      htmlFor: spaceSeparated$1,
      httpEquiv: spaceSeparated$1,
      id: null,
      imageSizes: null,
      imageSrcSet: null,
      inputMode: null,
      integrity: null,
      is: null,
      isMap: boolean$1,
      itemId: null,
      itemProp: spaceSeparated$1,
      itemRef: spaceSeparated$1,
      itemScope: boolean$1,
      itemType: spaceSeparated$1,
      kind: null,
      label: null,
      lang: null,
      language: null,
      list: null,
      loading: null,
      loop: boolean$1,
      low: number$1,
      manifest: null,
      max: null,
      maxLength: number$1,
      media: null,
      method: null,
      min: null,
      minLength: number$1,
      multiple: boolean$1,
      muted: boolean$1,
      name: null,
      nonce: null,
      noModule: boolean$1,
      noValidate: boolean$1,
      onAbort: null,
      onAfterPrint: null,
      onAuxClick: null,
      onBeforePrint: null,
      onBeforeUnload: null,
      onBlur: null,
      onCancel: null,
      onCanPlay: null,
      onCanPlayThrough: null,
      onChange: null,
      onClick: null,
      onClose: null,
      onContextLost: null,
      onContextMenu: null,
      onContextRestored: null,
      onCopy: null,
      onCueChange: null,
      onCut: null,
      onDblClick: null,
      onDrag: null,
      onDragEnd: null,
      onDragEnter: null,
      onDragExit: null,
      onDragLeave: null,
      onDragOver: null,
      onDragStart: null,
      onDrop: null,
      onDurationChange: null,
      onEmptied: null,
      onEnded: null,
      onError: null,
      onFocus: null,
      onFormData: null,
      onHashChange: null,
      onInput: null,
      onInvalid: null,
      onKeyDown: null,
      onKeyPress: null,
      onKeyUp: null,
      onLanguageChange: null,
      onLoad: null,
      onLoadedData: null,
      onLoadedMetadata: null,
      onLoadEnd: null,
      onLoadStart: null,
      onMessage: null,
      onMessageError: null,
      onMouseDown: null,
      onMouseEnter: null,
      onMouseLeave: null,
      onMouseMove: null,
      onMouseOut: null,
      onMouseOver: null,
      onMouseUp: null,
      onOffline: null,
      onOnline: null,
      onPageHide: null,
      onPageShow: null,
      onPaste: null,
      onPause: null,
      onPlay: null,
      onPlaying: null,
      onPopState: null,
      onProgress: null,
      onRateChange: null,
      onRejectionHandled: null,
      onReset: null,
      onResize: null,
      onScroll: null,
      onSecurityPolicyViolation: null,
      onSeeked: null,
      onSeeking: null,
      onSelect: null,
      onSlotChange: null,
      onStalled: null,
      onStorage: null,
      onSubmit: null,
      onSuspend: null,
      onTimeUpdate: null,
      onToggle: null,
      onUnhandledRejection: null,
      onUnload: null,
      onVolumeChange: null,
      onWaiting: null,
      onWheel: null,
      open: boolean$1,
      optimum: number$1,
      pattern: null,
      ping: spaceSeparated$1,
      placeholder: null,
      playsInline: boolean$1,
      poster: null,
      preload: null,
      readOnly: boolean$1,
      referrerPolicy: null,
      rel: spaceSeparated$1,
      required: boolean$1,
      reversed: boolean$1,
      rows: number$1,
      rowSpan: number$1,
      sandbox: spaceSeparated$1,
      scope: null,
      scoped: boolean$1,
      seamless: boolean$1,
      selected: boolean$1,
      shape: null,
      size: number$1,
      sizes: null,
      slot: null,
      span: number$1,
      spellCheck: booleanish$1,
      src: null,
      srcDoc: null,
      srcLang: null,
      srcSet: null,
      start: number$1,
      step: null,
      style: null,
      tabIndex: number$1,
      target: null,
      title: null,
      translate: null,
      type: null,
      typeMustMatch: boolean$1,
      useMap: null,
      value: booleanish$1,
      width: number$1,
      wrap: null,

      // Legacy.
      // See: https://html.spec.whatwg.org/#other-elements,-attributes-and-apis
      align: null, // Several. Use CSS `text-align` instead,
      aLink: null, // `<body>`. Use CSS `a:active {color}` instead
      archive: spaceSeparated$1, // `<object>`. List of URIs to archives
      axis: null, // `<td>` and `<th>`. Use `scope` on `<th>`
      background: null, // `<body>`. Use CSS `background-image` instead
      bgColor: null, // `<body>` and table elements. Use CSS `background-color` instead
      border: number$1, // `<table>`. Use CSS `border-width` instead,
      borderColor: null, // `<table>`. Use CSS `border-color` instead,
      bottomMargin: number$1, // `<body>`
      cellPadding: null, // `<table>`
      cellSpacing: null, // `<table>`
      char: null, // Several table elements. When `align=char`, sets the character to align on
      charOff: null, // Several table elements. When `char`, offsets the alignment
      classId: null, // `<object>`
      clear: null, // `<br>`. Use CSS `clear` instead
      code: null, // `<object>`
      codeBase: null, // `<object>`
      codeType: null, // `<object>`
      color: null, // `<font>` and `<hr>`. Use CSS instead
      compact: boolean$1, // Lists. Use CSS to reduce space between items instead
      declare: boolean$1, // `<object>`
      event: null, // `<script>`
      face: null, // `<font>`. Use CSS instead
      frame: null, // `<table>`
      frameBorder: null, // `<iframe>`. Use CSS `border` instead
      hSpace: number$1, // `<img>` and `<object>`
      leftMargin: number$1, // `<body>`
      link: null, // `<body>`. Use CSS `a:link {color: *}` instead
      longDesc: null, // `<frame>`, `<iframe>`, and `<img>`. Use an `<a>`
      lowSrc: null, // `<img>`. Use a `<picture>`
      marginHeight: number$1, // `<body>`
      marginWidth: number$1, // `<body>`
      noResize: boolean$1, // `<frame>`
      noHref: boolean$1, // `<area>`. Use no href instead of an explicit `nohref`
      noShade: boolean$1, // `<hr>`. Use background-color and height instead of borders
      noWrap: boolean$1, // `<td>` and `<th>`
      object: null, // `<applet>`
      profile: null, // `<head>`
      prompt: null, // `<isindex>`
      rev: null, // `<link>`
      rightMargin: number$1, // `<body>`
      rules: null, // `<table>`
      scheme: null, // `<meta>`
      scrolling: booleanish$1, // `<frame>`. Use overflow in the child context
      standby: null, // `<object>`
      summary: null, // `<table>`
      text: null, // `<body>`. Use CSS `color` instead
      topMargin: number$1, // `<body>`
      valueType: null, // `<param>`
      version: null, // `<html>`. Use a doctype.
      vAlign: null, // Several. Use CSS `vertical-align` instead
      vLink: null, // `<body>`. Use CSS `a:visited {color}` instead
      vSpace: number$1, // `<img>` and `<object>`

      // Non-standard Properties.
      allowTransparency: null,
      autoCorrect: null,
      autoSave: null,
      disablePictureInPicture: boolean$1,
      disableRemotePlayback: boolean$1,
      prefix: null,
      property: null,
      results: number$1,
      security: null,
      unselectable: null
    }
  });

  const svg$2 = create$1({
    space: 'svg',
    attributes: {
      accentHeight: 'accent-height',
      alignmentBaseline: 'alignment-baseline',
      arabicForm: 'arabic-form',
      baselineShift: 'baseline-shift',
      capHeight: 'cap-height',
      className: 'class',
      clipPath: 'clip-path',
      clipRule: 'clip-rule',
      colorInterpolation: 'color-interpolation',
      colorInterpolationFilters: 'color-interpolation-filters',
      colorProfile: 'color-profile',
      colorRendering: 'color-rendering',
      crossOrigin: 'crossorigin',
      dataType: 'datatype',
      dominantBaseline: 'dominant-baseline',
      enableBackground: 'enable-background',
      fillOpacity: 'fill-opacity',
      fillRule: 'fill-rule',
      floodColor: 'flood-color',
      floodOpacity: 'flood-opacity',
      fontFamily: 'font-family',
      fontSize: 'font-size',
      fontSizeAdjust: 'font-size-adjust',
      fontStretch: 'font-stretch',
      fontStyle: 'font-style',
      fontVariant: 'font-variant',
      fontWeight: 'font-weight',
      glyphName: 'glyph-name',
      glyphOrientationHorizontal: 'glyph-orientation-horizontal',
      glyphOrientationVertical: 'glyph-orientation-vertical',
      hrefLang: 'hreflang',
      horizAdvX: 'horiz-adv-x',
      horizOriginX: 'horiz-origin-x',
      horizOriginY: 'horiz-origin-y',
      imageRendering: 'image-rendering',
      letterSpacing: 'letter-spacing',
      lightingColor: 'lighting-color',
      markerEnd: 'marker-end',
      markerMid: 'marker-mid',
      markerStart: 'marker-start',
      navDown: 'nav-down',
      navDownLeft: 'nav-down-left',
      navDownRight: 'nav-down-right',
      navLeft: 'nav-left',
      navNext: 'nav-next',
      navPrev: 'nav-prev',
      navRight: 'nav-right',
      navUp: 'nav-up',
      navUpLeft: 'nav-up-left',
      navUpRight: 'nav-up-right',
      onAbort: 'onabort',
      onActivate: 'onactivate',
      onAfterPrint: 'onafterprint',
      onBeforePrint: 'onbeforeprint',
      onBegin: 'onbegin',
      onCancel: 'oncancel',
      onCanPlay: 'oncanplay',
      onCanPlayThrough: 'oncanplaythrough',
      onChange: 'onchange',
      onClick: 'onclick',
      onClose: 'onclose',
      onCopy: 'oncopy',
      onCueChange: 'oncuechange',
      onCut: 'oncut',
      onDblClick: 'ondblclick',
      onDrag: 'ondrag',
      onDragEnd: 'ondragend',
      onDragEnter: 'ondragenter',
      onDragExit: 'ondragexit',
      onDragLeave: 'ondragleave',
      onDragOver: 'ondragover',
      onDragStart: 'ondragstart',
      onDrop: 'ondrop',
      onDurationChange: 'ondurationchange',
      onEmptied: 'onemptied',
      onEnd: 'onend',
      onEnded: 'onended',
      onError: 'onerror',
      onFocus: 'onfocus',
      onFocusIn: 'onfocusin',
      onFocusOut: 'onfocusout',
      onHashChange: 'onhashchange',
      onInput: 'oninput',
      onInvalid: 'oninvalid',
      onKeyDown: 'onkeydown',
      onKeyPress: 'onkeypress',
      onKeyUp: 'onkeyup',
      onLoad: 'onload',
      onLoadedData: 'onloadeddata',
      onLoadedMetadata: 'onloadedmetadata',
      onLoadStart: 'onloadstart',
      onMessage: 'onmessage',
      onMouseDown: 'onmousedown',
      onMouseEnter: 'onmouseenter',
      onMouseLeave: 'onmouseleave',
      onMouseMove: 'onmousemove',
      onMouseOut: 'onmouseout',
      onMouseOver: 'onmouseover',
      onMouseUp: 'onmouseup',
      onMouseWheel: 'onmousewheel',
      onOffline: 'onoffline',
      onOnline: 'ononline',
      onPageHide: 'onpagehide',
      onPageShow: 'onpageshow',
      onPaste: 'onpaste',
      onPause: 'onpause',
      onPlay: 'onplay',
      onPlaying: 'onplaying',
      onPopState: 'onpopstate',
      onProgress: 'onprogress',
      onRateChange: 'onratechange',
      onRepeat: 'onrepeat',
      onReset: 'onreset',
      onResize: 'onresize',
      onScroll: 'onscroll',
      onSeeked: 'onseeked',
      onSeeking: 'onseeking',
      onSelect: 'onselect',
      onShow: 'onshow',
      onStalled: 'onstalled',
      onStorage: 'onstorage',
      onSubmit: 'onsubmit',
      onSuspend: 'onsuspend',
      onTimeUpdate: 'ontimeupdate',
      onToggle: 'ontoggle',
      onUnload: 'onunload',
      onVolumeChange: 'onvolumechange',
      onWaiting: 'onwaiting',
      onZoom: 'onzoom',
      overlinePosition: 'overline-position',
      overlineThickness: 'overline-thickness',
      paintOrder: 'paint-order',
      panose1: 'panose-1',
      pointerEvents: 'pointer-events',
      referrerPolicy: 'referrerpolicy',
      renderingIntent: 'rendering-intent',
      shapeRendering: 'shape-rendering',
      stopColor: 'stop-color',
      stopOpacity: 'stop-opacity',
      strikethroughPosition: 'strikethrough-position',
      strikethroughThickness: 'strikethrough-thickness',
      strokeDashArray: 'stroke-dasharray',
      strokeDashOffset: 'stroke-dashoffset',
      strokeLineCap: 'stroke-linecap',
      strokeLineJoin: 'stroke-linejoin',
      strokeMiterLimit: 'stroke-miterlimit',
      strokeOpacity: 'stroke-opacity',
      strokeWidth: 'stroke-width',
      tabIndex: 'tabindex',
      textAnchor: 'text-anchor',
      textDecoration: 'text-decoration',
      textRendering: 'text-rendering',
      typeOf: 'typeof',
      underlinePosition: 'underline-position',
      underlineThickness: 'underline-thickness',
      unicodeBidi: 'unicode-bidi',
      unicodeRange: 'unicode-range',
      unitsPerEm: 'units-per-em',
      vAlphabetic: 'v-alphabetic',
      vHanging: 'v-hanging',
      vIdeographic: 'v-ideographic',
      vMathematical: 'v-mathematical',
      vectorEffect: 'vector-effect',
      vertAdvY: 'vert-adv-y',
      vertOriginX: 'vert-origin-x',
      vertOriginY: 'vert-origin-y',
      wordSpacing: 'word-spacing',
      writingMode: 'writing-mode',
      xHeight: 'x-height',
      // These were camelcased in Tiny. Now lowercased in SVG 2
      playbackOrder: 'playbackorder',
      timelineBegin: 'timelinebegin'
    },
    transform: caseSensitiveTransform$1,
    properties: {
      about: commaOrSpaceSeparated$1,
      accentHeight: number$1,
      accumulate: null,
      additive: null,
      alignmentBaseline: null,
      alphabetic: number$1,
      amplitude: number$1,
      arabicForm: null,
      ascent: number$1,
      attributeName: null,
      attributeType: null,
      azimuth: number$1,
      bandwidth: null,
      baselineShift: null,
      baseFrequency: null,
      baseProfile: null,
      bbox: null,
      begin: null,
      bias: number$1,
      by: null,
      calcMode: null,
      capHeight: number$1,
      className: spaceSeparated$1,
      clip: null,
      clipPath: null,
      clipPathUnits: null,
      clipRule: null,
      color: null,
      colorInterpolation: null,
      colorInterpolationFilters: null,
      colorProfile: null,
      colorRendering: null,
      content: null,
      contentScriptType: null,
      contentStyleType: null,
      crossOrigin: null,
      cursor: null,
      cx: null,
      cy: null,
      d: null,
      dataType: null,
      defaultAction: null,
      descent: number$1,
      diffuseConstant: number$1,
      direction: null,
      display: null,
      dur: null,
      divisor: number$1,
      dominantBaseline: null,
      download: boolean$1,
      dx: null,
      dy: null,
      edgeMode: null,
      editable: null,
      elevation: number$1,
      enableBackground: null,
      end: null,
      event: null,
      exponent: number$1,
      externalResourcesRequired: null,
      fill: null,
      fillOpacity: number$1,
      fillRule: null,
      filter: null,
      filterRes: null,
      filterUnits: null,
      floodColor: null,
      floodOpacity: null,
      focusable: null,
      focusHighlight: null,
      fontFamily: null,
      fontSize: null,
      fontSizeAdjust: null,
      fontStretch: null,
      fontStyle: null,
      fontVariant: null,
      fontWeight: null,
      format: null,
      fr: null,
      from: null,
      fx: null,
      fy: null,
      g1: commaSeparated$1,
      g2: commaSeparated$1,
      glyphName: commaSeparated$1,
      glyphOrientationHorizontal: null,
      glyphOrientationVertical: null,
      glyphRef: null,
      gradientTransform: null,
      gradientUnits: null,
      handler: null,
      hanging: number$1,
      hatchContentUnits: null,
      hatchUnits: null,
      height: null,
      href: null,
      hrefLang: null,
      horizAdvX: number$1,
      horizOriginX: number$1,
      horizOriginY: number$1,
      id: null,
      ideographic: number$1,
      imageRendering: null,
      initialVisibility: null,
      in: null,
      in2: null,
      intercept: number$1,
      k: number$1,
      k1: number$1,
      k2: number$1,
      k3: number$1,
      k4: number$1,
      kernelMatrix: commaOrSpaceSeparated$1,
      kernelUnitLength: null,
      keyPoints: null, // SEMI_COLON_SEPARATED
      keySplines: null, // SEMI_COLON_SEPARATED
      keyTimes: null, // SEMI_COLON_SEPARATED
      kerning: null,
      lang: null,
      lengthAdjust: null,
      letterSpacing: null,
      lightingColor: null,
      limitingConeAngle: number$1,
      local: null,
      markerEnd: null,
      markerMid: null,
      markerStart: null,
      markerHeight: null,
      markerUnits: null,
      markerWidth: null,
      mask: null,
      maskContentUnits: null,
      maskUnits: null,
      mathematical: null,
      max: null,
      media: null,
      mediaCharacterEncoding: null,
      mediaContentEncodings: null,
      mediaSize: number$1,
      mediaTime: null,
      method: null,
      min: null,
      mode: null,
      name: null,
      navDown: null,
      navDownLeft: null,
      navDownRight: null,
      navLeft: null,
      navNext: null,
      navPrev: null,
      navRight: null,
      navUp: null,
      navUpLeft: null,
      navUpRight: null,
      numOctaves: null,
      observer: null,
      offset: null,
      onAbort: null,
      onActivate: null,
      onAfterPrint: null,
      onBeforePrint: null,
      onBegin: null,
      onCancel: null,
      onCanPlay: null,
      onCanPlayThrough: null,
      onChange: null,
      onClick: null,
      onClose: null,
      onCopy: null,
      onCueChange: null,
      onCut: null,
      onDblClick: null,
      onDrag: null,
      onDragEnd: null,
      onDragEnter: null,
      onDragExit: null,
      onDragLeave: null,
      onDragOver: null,
      onDragStart: null,
      onDrop: null,
      onDurationChange: null,
      onEmptied: null,
      onEnd: null,
      onEnded: null,
      onError: null,
      onFocus: null,
      onFocusIn: null,
      onFocusOut: null,
      onHashChange: null,
      onInput: null,
      onInvalid: null,
      onKeyDown: null,
      onKeyPress: null,
      onKeyUp: null,
      onLoad: null,
      onLoadedData: null,
      onLoadedMetadata: null,
      onLoadStart: null,
      onMessage: null,
      onMouseDown: null,
      onMouseEnter: null,
      onMouseLeave: null,
      onMouseMove: null,
      onMouseOut: null,
      onMouseOver: null,
      onMouseUp: null,
      onMouseWheel: null,
      onOffline: null,
      onOnline: null,
      onPageHide: null,
      onPageShow: null,
      onPaste: null,
      onPause: null,
      onPlay: null,
      onPlaying: null,
      onPopState: null,
      onProgress: null,
      onRateChange: null,
      onRepeat: null,
      onReset: null,
      onResize: null,
      onScroll: null,
      onSeeked: null,
      onSeeking: null,
      onSelect: null,
      onShow: null,
      onStalled: null,
      onStorage: null,
      onSubmit: null,
      onSuspend: null,
      onTimeUpdate: null,
      onToggle: null,
      onUnload: null,
      onVolumeChange: null,
      onWaiting: null,
      onZoom: null,
      opacity: null,
      operator: null,
      order: null,
      orient: null,
      orientation: null,
      origin: null,
      overflow: null,
      overlay: null,
      overlinePosition: number$1,
      overlineThickness: number$1,
      paintOrder: null,
      panose1: null,
      path: null,
      pathLength: number$1,
      patternContentUnits: null,
      patternTransform: null,
      patternUnits: null,
      phase: null,
      ping: spaceSeparated$1,
      pitch: null,
      playbackOrder: null,
      pointerEvents: null,
      points: null,
      pointsAtX: number$1,
      pointsAtY: number$1,
      pointsAtZ: number$1,
      preserveAlpha: null,
      preserveAspectRatio: null,
      primitiveUnits: null,
      propagate: null,
      property: commaOrSpaceSeparated$1,
      r: null,
      radius: null,
      referrerPolicy: null,
      refX: null,
      refY: null,
      rel: commaOrSpaceSeparated$1,
      rev: commaOrSpaceSeparated$1,
      renderingIntent: null,
      repeatCount: null,
      repeatDur: null,
      requiredExtensions: commaOrSpaceSeparated$1,
      requiredFeatures: commaOrSpaceSeparated$1,
      requiredFonts: commaOrSpaceSeparated$1,
      requiredFormats: commaOrSpaceSeparated$1,
      resource: null,
      restart: null,
      result: null,
      rotate: null,
      rx: null,
      ry: null,
      scale: null,
      seed: null,
      shapeRendering: null,
      side: null,
      slope: null,
      snapshotTime: null,
      specularConstant: number$1,
      specularExponent: number$1,
      spreadMethod: null,
      spacing: null,
      startOffset: null,
      stdDeviation: null,
      stemh: null,
      stemv: null,
      stitchTiles: null,
      stopColor: null,
      stopOpacity: null,
      strikethroughPosition: number$1,
      strikethroughThickness: number$1,
      string: null,
      stroke: null,
      strokeDashArray: commaOrSpaceSeparated$1,
      strokeDashOffset: null,
      strokeLineCap: null,
      strokeLineJoin: null,
      strokeMiterLimit: number$1,
      strokeOpacity: number$1,
      strokeWidth: null,
      style: null,
      surfaceScale: number$1,
      syncBehavior: null,
      syncBehaviorDefault: null,
      syncMaster: null,
      syncTolerance: null,
      syncToleranceDefault: null,
      systemLanguage: commaOrSpaceSeparated$1,
      tabIndex: number$1,
      tableValues: null,
      target: null,
      targetX: number$1,
      targetY: number$1,
      textAnchor: null,
      textDecoration: null,
      textRendering: null,
      textLength: null,
      timelineBegin: null,
      title: null,
      transformBehavior: null,
      type: null,
      typeOf: commaOrSpaceSeparated$1,
      to: null,
      transform: null,
      u1: null,
      u2: null,
      underlinePosition: number$1,
      underlineThickness: number$1,
      unicode: null,
      unicodeBidi: null,
      unicodeRange: null,
      unitsPerEm: number$1,
      values: null,
      vAlphabetic: number$1,
      vMathematical: number$1,
      vectorEffect: null,
      vHanging: number$1,
      vIdeographic: number$1,
      version: null,
      vertAdvY: number$1,
      vertOriginX: number$1,
      vertOriginY: number$1,
      viewBox: null,
      viewTarget: null,
      visibility: null,
      width: null,
      widths: null,
      wordSpacing: null,
      writingMode: null,
      x: null,
      x1: null,
      x2: null,
      xChannelSelector: null,
      xHeight: number$1,
      y: null,
      y1: null,
      y2: null,
      yChannelSelector: null,
      z: null,
      zoomAndPan: null
    }
  });

  /**
   * @typedef {import('./util/schema.js').Schema} Schema
   */

  const valid$1 = /^data[-\w.:]+$/i;
  const dash$1 = /-[a-z]/g;
  const cap$1 = /[A-Z]/g;

  /**
   * @param {Schema} schema
   * @param {string} value
   * @returns {Info}
   */
  function find$1(schema, value) {
    const normal = normalize$1(value);
    let prop = value;
    let Type = Info$1;

    if (normal in schema.normal) {
      return schema.property[schema.normal[normal]]
    }

    if (normal.length > 4 && normal.slice(0, 4) === 'data' && valid$1.test(value)) {
      // Attribute or property.
      if (value.charAt(4) === '-') {
        // Turn it into a property.
        const rest = value.slice(5).replace(dash$1, camelcase$1);
        prop = 'data' + rest.charAt(0).toUpperCase() + rest.slice(1);
      } else {
        // Turn it into an attribute.
        const rest = value.slice(4);

        if (!dash$1.test(rest)) {
          let dashes = rest.replace(cap$1, kebab$1);

          if (dashes.charAt(0) !== '-') {
            dashes = '-' + dashes;
          }

          value = 'data' + dashes;
        }
      }

      Type = DefinedInfo$1;
    }

    return new Type(prop, value)
  }

  /**
   * @param {string} $0
   * @returns {string}
   */
  function kebab$1($0) {
    return '-' + $0.toLowerCase()
  }

  /**
   * @param {string} $0
   * @returns {string}
   */
  function camelcase$1($0) {
    return $0.charAt(1).toUpperCase()
  }

  /**
   * @typedef {import('./lib/util/info.js').Info} Info
   * @typedef {import('./lib/util/schema.js').Schema} Schema
   */
  const html$4 = merge$1([xml$1, xlink$1, xmlns$1, aria$1, html$5], 'html');
  const svg$1 = merge$1([xml$1, xlink$1, xmlns$1, aria$1, svg$2], 'svg');

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   * @typedef {import('hast').Element} Element
   *
   * @typedef {string} TagName
   * @typedef {null|undefined|TagName|TestFunctionAnything|Array.<TagName|TestFunctionAnything>} Test
   */

  /**
   * @template {Element} T
   * @typedef {null|undefined|T['tagName']|TestFunctionPredicate<T>|Array.<T['tagName']|TestFunctionPredicate<T>>} PredicateTest
   */

  /**
   * Check if an element passes a test
   *
   * @callback TestFunctionAnything
   * @param {Element} element
   * @param {number|null|undefined} [index]
   * @param {Parent|null|undefined} [parent]
   * @returns {boolean|void}
   */

  /**
   * Check if an element passes a certain node test
   *
   * @template {Element} X
   * @callback TestFunctionPredicate
   * @param {Element} element
   * @param {number|null|undefined} [index]
   * @param {Parent|null|undefined} [parent]
   * @returns {element is X}
   */

  /**
   * Check if a node is an element and passes a certain node test
   *
   * @callback AssertAnything
   * @param {unknown} [node]
   * @param {number|null|undefined} [index]
   * @param {Parent|null|undefined} [parent]
   * @returns {boolean}
   */

  /**
   * Check if a node is an element and passes a certain node test
   *
   * @template {Element} Y
   * @callback AssertPredicate
   * @param {unknown} [node]
   * @param {number|null|undefined} [index]
   * @param {Parent|null|undefined} [parent]
   * @returns {node is Y}
   */

  // Check if `node` is an `element` and whether it passes the given test.
  const isElement =
    /**
     * Check if a node is an element and passes a test.
     * When a `parent` node is known the `index` of node should also be given.
     *
     * @type {(
     *   (() => false) &
     *   (<T extends Element = Element>(node: unknown, test?: PredicateTest<T>, index?: number, parent?: Parent, context?: unknown) => node is T) &
     *   ((node: unknown, test: Test, index?: number, parent?: Parent, context?: unknown) => boolean)
     * )}
     */
    (
      /**
       * Check if a node passes a test.
       * When a `parent` node is known the `index` of node should also be given.
       *
       * @param {unknown} [node] Node to check
       * @param {Test} [test] When nullish, checks if `node` is a `Node`.
       * When `string`, works like passing `function (node) {return node.type === test}`.
       * When `function` checks if function passed the node is true.
       * When `array`, checks any one of the subtests pass.
       * @param {number} [index] Position of `node` in `parent`
       * @param {Parent} [parent] Parent of `node`
       * @param {unknown} [context] Context object to invoke `test` with
       * @returns {boolean} Whether test passed and `node` is an `Element` (object with `type` set to `element` and `tagName` set to a non-empty string).
       */
      // eslint-disable-next-line max-params
      function (node, test, index, parent, context) {
        const check = convertElement(test);

        if (
          index !== undefined &&
          index !== null &&
          (typeof index !== 'number' ||
            index < 0 ||
            index === Number.POSITIVE_INFINITY)
        ) {
          throw new Error('Expected positive finite index for child node')
        }

        if (
          parent !== undefined &&
          parent !== null &&
          (!parent.type || !parent.children)
        ) {
          throw new Error('Expected parent node')
        }

        // @ts-expect-error Looks like a node.
        if (!node || !node.type || typeof node.type !== 'string') {
          return false
        }

        if (
          (parent === undefined || parent === null) !==
          (index === undefined || index === null)
        ) {
          throw new Error('Expected both parent and index')
        }

        return check.call(context, node, index, parent)
      }
    );

  const convertElement =
    /**
     * @type {(
     *   (<T extends Element>(test: T['tagName']|TestFunctionPredicate<T>) => AssertPredicate<T>) &
     *   ((test?: Test) => AssertAnything)
     * )}
     */
    (
      /**
       * Generate an assertion from a check.
       * @param {Test} [test]
       * When nullish, checks if `node` is a `Node`.
       * When `string`, works like passing `function (node) {return node.type === test}`.
       * When `function` checks if function passed the node is true.
       * When `object`, checks that all keys in test are in node, and that they have (strictly) equal values.
       * When `array`, checks any one of the subtests pass.
       * @returns {AssertAnything}
       */
      function (test) {
        if (test === undefined || test === null) {
          return element$1
        }

        if (typeof test === 'string') {
          return tagNameFactory(test)
        }

        if (typeof test === 'object') {
          return anyFactory(test)
        }

        if (typeof test === 'function') {
          return castFactory(test)
        }

        throw new Error('Expected function, string, or array as test')
      }
    );

  /**
   * @param {Array.<TagName|TestFunctionAnything>} tests
   * @returns {AssertAnything}
   */
  function anyFactory(tests) {
    /** @type {Array.<AssertAnything>} */
    const checks = [];
    let index = -1;

    while (++index < tests.length) {
      checks[index] = convertElement(tests[index]);
    }

    return castFactory(any)

    /**
     * @this {unknown}
     * @param {unknown[]} parameters
     * @returns {boolean}
     */
    function any(...parameters) {
      let index = -1;

      while (++index < checks.length) {
        if (checks[index].call(this, ...parameters)) {
          return true
        }
      }

      return false
    }
  }

  /**
   * Utility to convert a string into a function which checks a given node’s tag
   * name for said string.
   *
   * @param {TagName} check
   * @returns {AssertAnything}
   */
  function tagNameFactory(check) {
    return tagName

    /**
     * @param {unknown} node
     * @returns {boolean}
     */
    function tagName(node) {
      return element$1(node) && node.tagName === check
    }
  }

  /**
   * @param {TestFunctionAnything} check
   * @returns {AssertAnything}
   */
  function castFactory(check) {
    return assertion

    /**
     * @this {unknown}
     * @param {unknown} node
     * @param {Array.<unknown>} parameters
     * @returns {boolean}
     */
    function assertion(node, ...parameters) {
      // @ts-expect-error: fine.
      return element$1(node) && Boolean(check.call(this, node, ...parameters))
    }
  }

  /**
   * Utility to return true if this is an element.
   * @param {unknown} node
   * @returns {node is Element}
   */
  function element$1(node) {
    return Boolean(
      node &&
        typeof node === 'object' &&
        // @ts-expect-error Looks like a node.
        node.type === 'element' &&
        // @ts-expect-error Looks like an element.
        typeof node.tagName === 'string'
    )
  }

  /**
   * @typedef {import('../../types.js').Comment} Comment
   */

  /** @type {import('unist-util-is').AssertPredicate<Comment>} */
  // @ts-ignore
  const comment$1 = convert$1('comment');

  /**
   * @typedef {import('../../types.js').Parent} Parent
   * @typedef {import('../../types.js').Child} Child
   */

  const siblingAfter = siblings(1);
  const siblingBefore = siblings(-1);

  /**
   * Factory to check siblings in a direction.
   *
   * @param {number} increment
   */
  function siblings(increment) {
    return sibling

    /**
     * Find applicable siblings in a direction.
     *
     * @param {Parent} parent
     * @param {number} index
     * @param {boolean} [includeWhitespace=false]
     * @returns {Child}
     */
    function sibling(parent, index, includeWhitespace) {
      const siblings = parent && parent.children;
      let offset = index + increment;
      let next = siblings && siblings[offset];

      if (!includeWhitespace) {
        while (next && whitespace$1(next)) {
          offset += increment;
          next = siblings[offset];
        }
      }

      return next
    }
  }

  /**
   * @typedef {import('../../types.js').Node} Node
   * @typedef {import('../../types.js').Text} Text
   */

  /** @type {import('unist-util-is').AssertPredicate<Text>} */
  // @ts-ignore
  const isText = convert$1('text');

  /**
   * Check if `node` starts with whitespace.
   *
   * @param {Node} node
   * @returns {boolean}
   */
  function whitespaceStart(node) {
    return isText(node) && whitespace$1(node.value.charAt(0))
  }

  /**
   * @typedef {import('../types.js').OmitHandle} OmitHandle
   */

  const own$6 = {}.hasOwnProperty;

  /**
   * Factory to check if a given node can have a tag omitted.
   *
   * @param {Object.<string, OmitHandle>} handlers
   * @returns {OmitHandle}
   */
  function omission$1(handlers) {
    return omit

    /**
     * Check if a given node can have a tag omitted.
     *
     * @type {OmitHandle}
     */
    function omit(node, index, parent) {
      return (
        own$6.call(handlers, node.tagName) &&
        handlers[node.tagName](node, index, parent)
      )
    }
  }

  /**
   * @typedef {import('../types.js').OmitHandle} OmitHandle
   */

  const closing = omission$1({
    html: html$3,
    head: headOrColgroupOrCaption,
    body: body$1,
    p: p$1,
    li,
    dt,
    dd,
    rt: rubyElement,
    rp: rubyElement,
    optgroup,
    option,
    menuitem,
    colgroup: headOrColgroupOrCaption,
    caption: headOrColgroupOrCaption,
    thead,
    tbody: tbody$1,
    tfoot,
    tr,
    td: cells,
    th: cells
  });

  /**
   * Macro for `</head>`, `</colgroup>`, and `</caption>`.
   *
   * @type {OmitHandle}
   */
  function headOrColgroupOrCaption(_, index, parent) {
    const next = siblingAfter(parent, index, true);
    return !next || (!comment$1(next) && !whitespaceStart(next))
  }

  /**
   * Whether to omit `</html>`.
   *
   * @type {OmitHandle}
   */
  function html$3(_, index, parent) {
    const next = siblingAfter(parent, index);
    return !next || !comment$1(next)
  }

  /**
   * Whether to omit `</body>`.
   *
   * @type {OmitHandle}
   */
  function body$1(_, index, parent) {
    const next = siblingAfter(parent, index);
    return !next || !comment$1(next)
  }

  /**
   * Whether to omit `</p>`.
   *
   * @type {OmitHandle}
   */
  function p$1(_, index, parent) {
    const next = siblingAfter(parent, index);
    return next
      ? isElement(next, [
          'address',
          'article',
          'aside',
          'blockquote',
          'details',
          'div',
          'dl',
          'fieldset',
          'figcaption',
          'figure',
          'footer',
          'form',
          'h1',
          'h2',
          'h3',
          'h4',
          'h5',
          'h6',
          'header',
          'hgroup',
          'hr',
          'main',
          'menu',
          'nav',
          'ol',
          'p',
          'pre',
          'section',
          'table',
          'ul'
        ])
      : !parent ||
          // Confusing parent.
          !isElement(parent, [
            'a',
            'audio',
            'del',
            'ins',
            'map',
            'noscript',
            'video'
          ])
  }

  /**
   * Whether to omit `</li>`.
   *
   * @type {OmitHandle}
   */
  function li(_, index, parent) {
    const next = siblingAfter(parent, index);
    return !next || isElement(next, 'li')
  }

  /**
   * Whether to omit `</dt>`.
   *
   * @type {OmitHandle}
   */
  function dt(_, index, parent) {
    const next = siblingAfter(parent, index);
    return next && isElement(next, ['dt', 'dd'])
  }

  /**
   * Whether to omit `</dd>`.
   *
   * @type {OmitHandle}
   */
  function dd(_, index, parent) {
    const next = siblingAfter(parent, index);
    return !next || isElement(next, ['dt', 'dd'])
  }

  /**
   * Whether to omit `</rt>` or `</rp>`.
   *
   * @type {OmitHandle}
   */
  function rubyElement(_, index, parent) {
    const next = siblingAfter(parent, index);
    return !next || isElement(next, ['rp', 'rt'])
  }

  /**
   * Whether to omit `</optgroup>`.
   *
   * @type {OmitHandle}
   */
  function optgroup(_, index, parent) {
    const next = siblingAfter(parent, index);
    return !next || isElement(next, 'optgroup')
  }

  /**
   * Whether to omit `</option>`.
   *
   * @type {OmitHandle}
   */
  function option(_, index, parent) {
    const next = siblingAfter(parent, index);
    return !next || isElement(next, ['option', 'optgroup'])
  }

  /**
   * Whether to omit `</menuitem>`.
   *
   * @type {OmitHandle}
   */
  function menuitem(_, index, parent) {
    const next = siblingAfter(parent, index);
    return !next || isElement(next, ['menuitem', 'hr', 'menu'])
  }

  /**
   * Whether to omit `</thead>`.
   *
   * @type {OmitHandle}
   */
  function thead(_, index, parent) {
    const next = siblingAfter(parent, index);
    return next && isElement(next, ['tbody', 'tfoot'])
  }

  /**
   * Whether to omit `</tbody>`.
   *
   * @type {OmitHandle}
   */
  function tbody$1(_, index, parent) {
    const next = siblingAfter(parent, index);
    return !next || isElement(next, ['tbody', 'tfoot'])
  }

  /**
   * Whether to omit `</tfoot>`.
   *
   * @type {OmitHandle}
   */
  function tfoot(_, index, parent) {
    return !siblingAfter(parent, index)
  }

  /**
   * Whether to omit `</tr>`.
   *
   * @type {OmitHandle}
   */
  function tr(_, index, parent) {
    const next = siblingAfter(parent, index);
    return !next || isElement(next, 'tr')
  }

  /**
   * Whether to omit `</td>` or `</th>`.
   *
   * @type {OmitHandle}
   */
  function cells(_, index, parent) {
    const next = siblingAfter(parent, index);
    return !next || isElement(next, ['td', 'th'])
  }

  /**
   * @typedef {import('../types.js').OmitHandle} OmitHandle
   * @typedef {import('../types.js').Child} Child
   */

  const opening = omission$1({
    html: html$2,
    head,
    body,
    colgroup,
    tbody
  });

  /**
   * Whether to omit `<html>`.
   *
   * @type {OmitHandle}
   */
  function html$2(node) {
    const head = siblingAfter(node, -1);
    return !head || !comment$1(head)
  }

  /**
   * Whether to omit `<head>`.
   *
   * @type {OmitHandle}
   */
  function head(node) {
    const children = node.children;
    /** @type {Array.<string>} */
    const seen = [];
    let index = -1;
    /** @type {Child} */
    let child;

    while (++index < children.length) {
      child = children[index];
      if (isElement(child, ['title', 'base'])) {
        if (seen.includes(child.tagName)) return false
        seen.push(child.tagName);
      }
    }

    return children.length > 0
  }

  /**
   * Whether to omit `<body>`.
   *
   * @type {OmitHandle}
   */
  function body(node) {
    const head = siblingAfter(node, -1, true);

    return (
      !head ||
      (!comment$1(head) &&
        !whitespaceStart(head) &&
        !isElement(head, ['meta', 'link', 'script', 'style', 'template']))
    )
  }

  /**
   * Whether to omit `<colgroup>`.
   * The spec describes some logic for the opening tag, but it’s easier to
   * implement in the closing tag, to the same effect, so we handle it there
   * instead.
   *
   * @type {OmitHandle}
   */
  function colgroup(node, index, parent) {
    const previous = siblingBefore(parent, index);
    const head = siblingAfter(node, -1, true);

    // Previous colgroup was already omitted.
    if (
      isElement(previous, 'colgroup') &&
      closing(previous, parent.children.indexOf(previous), parent)
    ) {
      return false
    }

    return head && isElement(head, 'col')
  }

  /**
   * Whether to omit `<tbody>`.
   *
   * @type {OmitHandle}
   */
  function tbody(node, index, parent) {
    const previous = siblingBefore(parent, index);
    const head = siblingAfter(node, -1);

    // Previous table section was already omitted.
    if (
      isElement(previous, ['thead', 'tbody']) &&
      closing(previous, parent.children.indexOf(previous), parent)
    ) {
      return false
    }

    return head && isElement(head, 'tr')
  }

  /**
   * @typedef {import('../types.js').Omission} Omission
   */

  /** @type {Omission} */
  const omission = {opening, closing};

  /**
   * Parse space separated tokens to an array of strings.
   *
   * @param {string} value Space separated tokens
   * @returns {Array.<string>} Tokens
   */

  /**
   * Serialize an array of strings as space separated tokens.
   *
   * @param {Array.<string|number>} values Tokens
   * @returns {string} Space separated tokens
   */
  function stringify$2(values) {
    return values.join(' ').trim()
  }

  /**
   * @typedef {Object} StringifyOptions
   * @property {boolean} [padLeft=true] Whether to pad a space before a token (`boolean`, default: `true`).
   * @property {boolean} [padRight=false] Whether to pad a space after a token (`boolean`, default: `false`).
   */

  /**
   * Serialize an array of strings to comma separated tokens.
   *
   * @param {Array.<string|number>} values
   * @param {StringifyOptions} [options]
   * @returns {string}
   */
  function stringify$1(values, options) {
    var settings = options || {};

    // Ensure the last empty entry is seen.
    if (values[values.length - 1] === '') {
      values = values.concat('');
    }

    return values
      .join(
        (settings.padRight ? ' ' : '') +
          ',' +
          (settings.padLeft === false ? '' : ' ')
      )
      .trim()
  }

  /**
   * @typedef {Object} CoreOptions
   * @property {string[]} [subset=[]]
   *   Whether to only escape the given subset of characters.
   * @property {boolean} [escapeOnly=false]
   *   Whether to only escape possibly dangerous characters.
   *   Those characters are `"`, `&`, `'`, `<`, `>`, and `` ` ``.
   *
   * @typedef {Object} FormatOptions
   * @property {(code: number, next: number, options: CoreWithFormatOptions) => string} format
   *   Format strategy.
   *
   * @typedef {CoreOptions & FormatOptions & import('./util/format-smart.js').FormatSmartOptions} CoreWithFormatOptions
   */

  /**
   * Encode certain characters in `value`.
   *
   * @param {string} value
   * @param {CoreWithFormatOptions} options
   * @returns {string}
   */
  function core$1(value, options) {
    value = value.replace(
      options.subset ? charactersToExpression(options.subset) : /["&'<>`]/g,
      basic
    );

    if (options.subset || options.escapeOnly) {
      return value
    }

    return (
      value
        // Surrogate pairs.
        .replace(/[\uD800-\uDBFF][\uDC00-\uDFFF]/g, surrogate)
        // BMP control characters (C0 except for LF, CR, SP; DEL; and some more
        // non-ASCII ones).
        .replace(
          // eslint-disable-next-line no-control-regex, unicorn/no-hex-escape
          /[\x01-\t\v\f\x0E-\x1F\x7F\x81\x8D\x8F\x90\x9D\xA0-\uFFFF]/g,
          basic
        )
    )

    /**
     * @param {string} pair
     * @param {number} index
     * @param {string} all
     */
    function surrogate(pair, index, all) {
      return options.format(
        (pair.charCodeAt(0) - 0xd800) * 0x400 +
          pair.charCodeAt(1) -
          0xdc00 +
          0x10000,
        all.charCodeAt(index + 2),
        options
      )
    }

    /**
     * @param {string} character
     * @param {number} index
     * @param {string} all
     */
    function basic(character, index, all) {
      return options.format(
        character.charCodeAt(0),
        all.charCodeAt(index + 1),
        options
      )
    }
  }

  /**
   * @param {string[]} subset
   * @returns {RegExp}
   */
  function charactersToExpression(subset) {
    /** @type {string[]} */
    const groups = [];
    let index = -1;

    while (++index < subset.length) {
      groups.push(subset[index].replace(/[|\\{}()[\]^$+*?.]/g, '\\$&'));
    }

    return new RegExp('(?:' + groups.join('|') + ')', 'g')
  }

  /**
   * Configurable ways to encode characters as hexadecimal references.
   *
   * @param {number} code
   * @param {number} next
   * @param {boolean|undefined} omit
   * @returns {string}
   */
  function toHexadecimal(code, next, omit) {
    const value = '&#x' + code.toString(16).toUpperCase();
    return omit && next && !/[\dA-Fa-f]/.test(String.fromCharCode(next))
      ? value
      : value + ';'
  }

  /**
   * Configurable ways to encode characters as decimal references.
   *
   * @param {number} code
   * @param {number} next
   * @param {boolean|undefined} omit
   * @returns {string}
   */
  function toDecimal(code, next, omit) {
    const value = '&#' + String(code);
    return omit && next && !/\d/.test(String.fromCharCode(next))
      ? value
      : value + ';'
  }

  /**
   * List of legacy HTML named character references that don’t need a trailing semicolon.
   *
   * @type {Array<string>}
   */
  const characterEntitiesLegacy = [
    'AElig',
    'AMP',
    'Aacute',
    'Acirc',
    'Agrave',
    'Aring',
    'Atilde',
    'Auml',
    'COPY',
    'Ccedil',
    'ETH',
    'Eacute',
    'Ecirc',
    'Egrave',
    'Euml',
    'GT',
    'Iacute',
    'Icirc',
    'Igrave',
    'Iuml',
    'LT',
    'Ntilde',
    'Oacute',
    'Ocirc',
    'Ograve',
    'Oslash',
    'Otilde',
    'Ouml',
    'QUOT',
    'REG',
    'THORN',
    'Uacute',
    'Ucirc',
    'Ugrave',
    'Uuml',
    'Yacute',
    'aacute',
    'acirc',
    'acute',
    'aelig',
    'agrave',
    'amp',
    'aring',
    'atilde',
    'auml',
    'brvbar',
    'ccedil',
    'cedil',
    'cent',
    'copy',
    'curren',
    'deg',
    'divide',
    'eacute',
    'ecirc',
    'egrave',
    'eth',
    'euml',
    'frac12',
    'frac14',
    'frac34',
    'gt',
    'iacute',
    'icirc',
    'iexcl',
    'igrave',
    'iquest',
    'iuml',
    'laquo',
    'lt',
    'macr',
    'micro',
    'middot',
    'nbsp',
    'not',
    'ntilde',
    'oacute',
    'ocirc',
    'ograve',
    'ordf',
    'ordm',
    'oslash',
    'otilde',
    'ouml',
    'para',
    'plusmn',
    'pound',
    'quot',
    'raquo',
    'reg',
    'sect',
    'shy',
    'sup1',
    'sup2',
    'sup3',
    'szlig',
    'thorn',
    'times',
    'uacute',
    'ucirc',
    'ugrave',
    'uml',
    'uuml',
    'yacute',
    'yen',
    'yuml'
  ];

  /**
   * Map of named character references from HTML 4.
   *
   * @type {Record<string, string>}
   */
  const characterEntitiesHtml4 = {
    nbsp: ' ',
    iexcl: '¡',
    cent: '¢',
    pound: '£',
    curren: '¤',
    yen: '¥',
    brvbar: '¦',
    sect: '§',
    uml: '¨',
    copy: '©',
    ordf: 'ª',
    laquo: '«',
    not: '¬',
    shy: '­',
    reg: '®',
    macr: '¯',
    deg: '°',
    plusmn: '±',
    sup2: '²',
    sup3: '³',
    acute: '´',
    micro: 'µ',
    para: '¶',
    middot: '·',
    cedil: '¸',
    sup1: '¹',
    ordm: 'º',
    raquo: '»',
    frac14: '¼',
    frac12: '½',
    frac34: '¾',
    iquest: '¿',
    Agrave: 'À',
    Aacute: 'Á',
    Acirc: 'Â',
    Atilde: 'Ã',
    Auml: 'Ä',
    Aring: 'Å',
    AElig: 'Æ',
    Ccedil: 'Ç',
    Egrave: 'È',
    Eacute: 'É',
    Ecirc: 'Ê',
    Euml: 'Ë',
    Igrave: 'Ì',
    Iacute: 'Í',
    Icirc: 'Î',
    Iuml: 'Ï',
    ETH: 'Ð',
    Ntilde: 'Ñ',
    Ograve: 'Ò',
    Oacute: 'Ó',
    Ocirc: 'Ô',
    Otilde: 'Õ',
    Ouml: 'Ö',
    times: '×',
    Oslash: 'Ø',
    Ugrave: 'Ù',
    Uacute: 'Ú',
    Ucirc: 'Û',
    Uuml: 'Ü',
    Yacute: 'Ý',
    THORN: 'Þ',
    szlig: 'ß',
    agrave: 'à',
    aacute: 'á',
    acirc: 'â',
    atilde: 'ã',
    auml: 'ä',
    aring: 'å',
    aelig: 'æ',
    ccedil: 'ç',
    egrave: 'è',
    eacute: 'é',
    ecirc: 'ê',
    euml: 'ë',
    igrave: 'ì',
    iacute: 'í',
    icirc: 'î',
    iuml: 'ï',
    eth: 'ð',
    ntilde: 'ñ',
    ograve: 'ò',
    oacute: 'ó',
    ocirc: 'ô',
    otilde: 'õ',
    ouml: 'ö',
    divide: '÷',
    oslash: 'ø',
    ugrave: 'ù',
    uacute: 'ú',
    ucirc: 'û',
    uuml: 'ü',
    yacute: 'ý',
    thorn: 'þ',
    yuml: 'ÿ',
    fnof: 'ƒ',
    Alpha: 'Α',
    Beta: 'Β',
    Gamma: 'Γ',
    Delta: 'Δ',
    Epsilon: 'Ε',
    Zeta: 'Ζ',
    Eta: 'Η',
    Theta: 'Θ',
    Iota: 'Ι',
    Kappa: 'Κ',
    Lambda: 'Λ',
    Mu: 'Μ',
    Nu: 'Ν',
    Xi: 'Ξ',
    Omicron: 'Ο',
    Pi: 'Π',
    Rho: 'Ρ',
    Sigma: 'Σ',
    Tau: 'Τ',
    Upsilon: 'Υ',
    Phi: 'Φ',
    Chi: 'Χ',
    Psi: 'Ψ',
    Omega: 'Ω',
    alpha: 'α',
    beta: 'β',
    gamma: 'γ',
    delta: 'δ',
    epsilon: 'ε',
    zeta: 'ζ',
    eta: 'η',
    theta: 'θ',
    iota: 'ι',
    kappa: 'κ',
    lambda: 'λ',
    mu: 'μ',
    nu: 'ν',
    xi: 'ξ',
    omicron: 'ο',
    pi: 'π',
    rho: 'ρ',
    sigmaf: 'ς',
    sigma: 'σ',
    tau: 'τ',
    upsilon: 'υ',
    phi: 'φ',
    chi: 'χ',
    psi: 'ψ',
    omega: 'ω',
    thetasym: 'ϑ',
    upsih: 'ϒ',
    piv: 'ϖ',
    bull: '•',
    hellip: '…',
    prime: '′',
    Prime: '″',
    oline: '‾',
    frasl: '⁄',
    weierp: '℘',
    image: 'ℑ',
    real: 'ℜ',
    trade: '™',
    alefsym: 'ℵ',
    larr: '←',
    uarr: '↑',
    rarr: '→',
    darr: '↓',
    harr: '↔',
    crarr: '↵',
    lArr: '⇐',
    uArr: '⇑',
    rArr: '⇒',
    dArr: '⇓',
    hArr: '⇔',
    forall: '∀',
    part: '∂',
    exist: '∃',
    empty: '∅',
    nabla: '∇',
    isin: '∈',
    notin: '∉',
    ni: '∋',
    prod: '∏',
    sum: '∑',
    minus: '−',
    lowast: '∗',
    radic: '√',
    prop: '∝',
    infin: '∞',
    ang: '∠',
    and: '∧',
    or: '∨',
    cap: '∩',
    cup: '∪',
    int: '∫',
    there4: '∴',
    sim: '∼',
    cong: '≅',
    asymp: '≈',
    ne: '≠',
    equiv: '≡',
    le: '≤',
    ge: '≥',
    sub: '⊂',
    sup: '⊃',
    nsub: '⊄',
    sube: '⊆',
    supe: '⊇',
    oplus: '⊕',
    otimes: '⊗',
    perp: '⊥',
    sdot: '⋅',
    lceil: '⌈',
    rceil: '⌉',
    lfloor: '⌊',
    rfloor: '⌋',
    lang: '〈',
    rang: '〉',
    loz: '◊',
    spades: '♠',
    clubs: '♣',
    hearts: '♥',
    diams: '♦',
    quot: '"',
    amp: '&',
    lt: '<',
    gt: '>',
    OElig: 'Œ',
    oelig: 'œ',
    Scaron: 'Š',
    scaron: 'š',
    Yuml: 'Ÿ',
    circ: 'ˆ',
    tilde: '˜',
    ensp: ' ',
    emsp: ' ',
    thinsp: ' ',
    zwnj: '‌',
    zwj: '‍',
    lrm: '‎',
    rlm: '‏',
    ndash: '–',
    mdash: '—',
    lsquo: '‘',
    rsquo: '’',
    sbquo: '‚',
    ldquo: '“',
    rdquo: '”',
    bdquo: '„',
    dagger: '†',
    Dagger: '‡',
    permil: '‰',
    lsaquo: '‹',
    rsaquo: '›',
    euro: '€'
  };

  /**
   * List of legacy (that don’t need a trailing `;`) named references which could,
   * depending on what follows them, turn into a different meaning
   *
   * @type {Array.<string>}
   */
  const dangerous = [
    'cent',
    'copy',
    'divide',
    'gt',
    'lt',
    'not',
    'para',
    'times'
  ];

  const own$5 = {}.hasOwnProperty;

  /**
   * `characterEntitiesHtml4` but inverted.
   *
   * @type {Object.<string, string>}
   */
  const characters = {};

  /** @type {string} */
  let key;

  for (key in characterEntitiesHtml4) {
    if (own$5.call(characterEntitiesHtml4, key)) {
      characters[characterEntitiesHtml4[key]] = key;
    }
  }

  /**
   * Configurable ways to encode characters as named references.
   *
   * @param {number} code
   * @param {number} next
   * @param {boolean|undefined} omit
   * @param {boolean|undefined} attribute
   * @returns {string}
   */
  function toNamed(code, next, omit, attribute) {
    const character = String.fromCharCode(code);

    if (own$5.call(characters, character)) {
      const name = characters[character];
      const value = '&' + name;

      if (
        omit &&
        characterEntitiesLegacy.includes(name) &&
        !dangerous.includes(name) &&
        (!attribute ||
          (next &&
            next !== 61 /* `=` */ &&
            /[^\da-z]/i.test(String.fromCharCode(next))))
      ) {
        return value
      }

      return value + ';'
    }

    return ''
  }

  /**
   * @typedef {Object} FormatSmartOptions
   * @property {boolean} [useNamedReferences=false]
   *   Prefer named character references (`&amp;`) where possible.
   * @property {boolean} [useShortestReferences=false]
   *   Prefer the shortest possible reference, if that results in less bytes.
   *   **Note**: `useNamedReferences` can be omitted when using `useShortestReferences`.
   * @property {boolean} [omitOptionalSemicolons=false]
   *   Whether to omit semicolons when possible.
   *   **Note**: This creates what HTML calls “parse errors” but is otherwise still valid HTML — don’t use this except when building a minifier.
   *   Omitting semicolons is possible for certain named and numeric references in some cases.
   * @property {boolean} [attribute=false]
   *   Create character references which don’t fail in attributes.
   *   **Note**: `attribute` only applies when operating dangerously with
   *   `omitOptionalSemicolons: true`.
   */

  /**
   * Configurable ways to encode a character yielding pretty or small results.
   *
   * @param {number} code
   * @param {number} next
   * @param {FormatSmartOptions} options
   * @returns {string}
   */
  function formatSmart(code, next, options) {
    let numeric = toHexadecimal(code, next, options.omitOptionalSemicolons);
    /** @type {string|undefined} */
    let named;

    if (options.useNamedReferences || options.useShortestReferences) {
      named = toNamed(
        code,
        next,
        options.omitOptionalSemicolons,
        options.attribute
      );
    }

    // Use the shortest numeric reference when requested.
    // A simple algorithm would use decimal for all code points under 100, as
    // those are shorter than hexadecimal:
    //
    // * `&#99;` vs `&#x63;` (decimal shorter)
    // * `&#100;` vs `&#x64;` (equal)
    //
    // However, because we take `next` into consideration when `omit` is used,
    // And it would be possible that decimals are shorter on bigger values as
    // well if `next` is hexadecimal but not decimal, we instead compare both.
    if (
      (options.useShortestReferences || !named) &&
      options.useShortestReferences
    ) {
      const decimal = toDecimal(code, next, options.omitOptionalSemicolons);

      if (decimal.length < numeric.length) {
        numeric = decimal;
      }
    }

    return named &&
      (!options.useShortestReferences || named.length < numeric.length)
      ? named
      : numeric
  }

  /**
   * @typedef {import('./core.js').CoreOptions & import('./util/format-smart.js').FormatSmartOptions} Options
   * @typedef {import('./core.js').CoreOptions} LightOptions
   */

  /**
   * Encode special characters in `value`.
   *
   * @param {string} value
   *   Value to encode.
   * @param {Options} [options]
   *   Configuration.
   * @returns {string}
   *   Encoded value.
   */
  function stringifyEntities(value, options) {
    return core$1(value, Object.assign({format: formatSmart}, options))
  }

  /**
   * Count how often a character (or substring) is used in a string.
   *
   * @param {string} value
   *   Value to search in.
   * @param {string} character
   *   Character (or substring) to look for.
   * @return {number}
   *   Number of times `character` occurred in `value`.
   */
  function ccount(value, character) {
    const source = String(value);

    if (typeof character !== 'string') {
      throw new TypeError('Expected character')
    }

    let count = 0;
    let index = source.indexOf(character);

    while (index !== -1) {
      count++;
      index = source.indexOf(character, index + character.length);
    }

    return count
  }

  // Maps of subsets.
  // Each value is a matrix of tuples.
  // The first value causes parse errors, the second is valid.
  // Of both values, the first value is unsafe, and the second is safe.
  const constants = {
    // See: <https://html.spec.whatwg.org/#attribute-name-state>.
    name: [
      ['\t\n\f\r &/=>'.split(''), '\t\n\f\r "&\'/=>`'.split('')],
      ['\0\t\n\f\r "&\'/<=>'.split(''), '\0\t\n\f\r "&\'/<=>`'.split('')]
    ],
    // See: <https://html.spec.whatwg.org/#attribute-value-(unquoted)-state>.
    unquoted: [
      ['\t\n\f\r &>'.split(''), '\0\t\n\f\r "&\'<=>`'.split('')],
      ['\0\t\n\f\r "&\'<=>`'.split(''), '\0\t\n\f\r "&\'<=>`'.split('')]
    ],
    // See: <https://html.spec.whatwg.org/#attribute-value-(single-quoted)-state>.
    single: [
      ["&'".split(''), '"&\'`'.split('')],
      ["\0&'".split(''), '\0"&\'`'.split('')]
    ],
    // See: <https://html.spec.whatwg.org/#attribute-value-(double-quoted)-state>.
    double: [
      ['"&'.split(''), '"&\'`'.split('')],
      ['\0"&'.split(''), '\0"&\'`'.split('')]
    ]
  };

  /**
   * @typedef {import('./types.js').Handle} Handle
   * @typedef {import('./types.js').Comment} Comment
   */

  /**
   * @type {Handle}
   * @param {Comment} node
   */
  function comment(ctx, node) {
    // See: <https://html.spec.whatwg.org/multipage/syntax.html#comments>
    return ctx.bogusComments
      ? '<?' +
          stringifyEntities(
            node.value,
            Object.assign({}, ctx.entities, {subset: ['>']})
          ) +
          '>'
      : '<!--' + node.value.replace(/^>|^->|<!--|-->|--!>|<!-$/g, encode) + '-->'

    /**
     * @param {string} $0
     */
    function encode($0) {
      return stringifyEntities(
        $0,
        Object.assign({}, ctx.entities, {subset: ['<', '>']})
      )
    }
  }

  /**
   * @typedef {import('./types.js').Handle} Handle
   */

  /**
   * @type {Handle}
   */
  function doctype(ctx) {
    return (
      '<!' +
      (ctx.upperDoctype ? 'DOCTYPE' : 'doctype') +
      (ctx.tightDoctype ? '' : ' ') +
      'html>'
    )
  }

  /**
   * @typedef {import('./types.js').Handle} Handle
   * @typedef {import('./types.js').Text} Text
   */

  /**
   * @type {Handle}
   * @param {Text} node
   */
  function text(ctx, node, _, parent) {
    // Check if content of `node` should be escaped.
    return parent &&
      parent.type === 'element' &&
      // @ts-expect-error: hush.
      (parent.tagName === 'script' || parent.tagName === 'style')
      ? node.value
      : stringifyEntities(
          node.value,
          Object.assign({}, ctx.entities, {subset: ['<', '&']})
        )
  }

  /**
   * @typedef {import('./types.js').Handle} Handle
   * @typedef {import('./types.js').Raw} Raw
   */

  /**
   * @type {Handle}
   * @param {Raw} node
   */
  function raw(ctx, node, index, parent) {
    // @ts-ignore Hush.
    return ctx.dangerous ? node.value : text(ctx, node, index, parent)
  }

  /**
   * @typedef {import('./types.js').Handle} Handle
   * @typedef {import('./types.js').Element} Element
   * @typedef {import('./types.js').Context} Context
   * @typedef {import('./types.js').Properties} Properties
   * @typedef {import('./types.js').PropertyValue} PropertyValue
   * @typedef {import('./types.js').Parent} Parent
   */

  /**
   * @type {Object.<string, Handle>}
   */
  const handlers = {
    comment,
    doctype,
    element,
    // @ts-ignore `raw` is nonstandard
    raw,
    // @ts-ignore `root` is a parent.
    root: all,
    text
  };

  const own$4 = {}.hasOwnProperty;

  /**
   * @type {Handle}
   */
  function one(ctx, node, index, parent) {
    if (!node || !node.type) {
      throw new Error('Expected node, not `' + node + '`')
    }

    if (!own$4.call(handlers, node.type)) {
      throw new Error('Cannot compile unknown node `' + node.type + '`')
    }

    return handlers[node.type](ctx, node, index, parent)
  }

  /**
   * Serialize all children of `parent`.
   *
   * @type {Handle}
   * @param {Parent} parent
   */
  function all(ctx, parent) {
    /** @type {Array.<string>} */
    const results = [];
    const children = (parent && parent.children) || [];
    let index = -1;

    while (++index < children.length) {
      results[index] = one(ctx, children[index], index, parent);
    }

    return results.join('')
  }

  /**
   * @type {Handle}
   * @param {Element} node
   */
  // eslint-disable-next-line complexity
  function element(ctx, node, index, parent) {
    const schema = ctx.schema;
    const omit = schema.space === 'svg' ? undefined : ctx.omit;
    let selfClosing =
      schema.space === 'svg'
        ? ctx.closeEmpty
        : ctx.voids.includes(node.tagName.toLowerCase());
    /** @type {Array.<string>} */
    const parts = [];
    /** @type {string} */
    let last;

    if (schema.space === 'html' && node.tagName === 'svg') {
      ctx.schema = svg$1;
    }

    const attrs = serializeAttributes(ctx, node.properties);

    const content = all(
      ctx,
      schema.space === 'html' && node.tagName === 'template' ? node.content : node
    );

    ctx.schema = schema;

    // If the node is categorised as void, but it has children, remove the
    // categorisation.
    // This enables for example `menuitem`s, which are void in W3C HTML but not
    // void in WHATWG HTML, to be stringified properly.
    if (content) selfClosing = false;

    if (attrs || !omit || !omit.opening(node, index, parent)) {
      parts.push('<', node.tagName, attrs ? ' ' + attrs : '');

      if (selfClosing && (schema.space === 'svg' || ctx.close)) {
        last = attrs.charAt(attrs.length - 1);
        if (
          !ctx.tightClose ||
          last === '/' ||
          (last && last !== '"' && last !== "'")
        ) {
          parts.push(' ');
        }

        parts.push('/');
      }

      parts.push('>');
    }

    parts.push(content);

    if (!selfClosing && (!omit || !omit.closing(node, index, parent))) {
      parts.push('</' + node.tagName + '>');
    }

    return parts.join('')
  }

  /**
   * @param {Context} ctx
   * @param {Properties} props
   * @returns {string}
   */
  function serializeAttributes(ctx, props) {
    /** @type {Array.<string>} */
    const values = [];
    let index = -1;
    /** @type {string} */
    let key;
    /** @type {string} */
    let value;
    /** @type {string} */
    let last;

    for (key in props) {
      if (props[key] !== undefined && props[key] !== null) {
        value = serializeAttribute(ctx, key, props[key]);
        if (value) values.push(value);
      }
    }

    while (++index < values.length) {
      last = ctx.tight ? values[index].charAt(values[index].length - 1) : null;

      // In tight mode, don’t add a space after quoted attributes.
      if (index !== values.length - 1 && last !== '"' && last !== "'") {
        values[index] += ' ';
      }
    }

    return values.join('')
  }

  /**
   * @param {Context} ctx
   * @param {string} key
   * @param {PropertyValue} value
   * @returns {string}
   */
  // eslint-disable-next-line complexity
  function serializeAttribute(ctx, key, value) {
    const info = find$1(ctx.schema, key);
    let quote = ctx.quote;
    /** @type {string} */
    let result;

    if (info.overloadedBoolean && (value === info.attribute || value === '')) {
      value = true;
    } else if (
      info.boolean ||
      (info.overloadedBoolean && typeof value !== 'string')
    ) {
      value = Boolean(value);
    }

    if (
      value === undefined ||
      value === null ||
      value === false ||
      (typeof value === 'number' && Number.isNaN(value))
    ) {
      return ''
    }

    const name = stringifyEntities(
      info.attribute,
      Object.assign({}, ctx.entities, {
        // Always encode without parse errors in non-HTML.
        subset:
          constants.name[ctx.schema.space === 'html' ? ctx.valid : 1][ctx.safe]
      })
    );

    // No value.
    // There is currently only one boolean property in SVG: `[download]` on
    // `<a>`.
    // This property does not seem to work in browsers (FF, Sa, Ch), so I can’t
    // test if dropping the value works.
    // But I assume that it should:
    //
    // ```html
    // <!doctype html>
    // <svg viewBox="0 0 100 100">
    //   <a href=https://example.com download>
    //     <circle cx=50 cy=40 r=35 />
    //   </a>
    // </svg>
    // ```
    //
    // See: <https://github.com/wooorm/property-information/blob/main/lib/svg.js>
    if (value === true) return name

    value =
      typeof value === 'object' && 'length' in value
        ? // `spaces` doesn’t accept a second argument, but it’s given here just to
          // keep the code cleaner.
          (info.commaSeparated ? stringify$1 : stringify$2)(value, {
            padLeft: !ctx.tightLists
          })
        : String(value);

    if (ctx.collapseEmpty && !value) return name

    // Check unquoted value.
    if (ctx.unquoted) {
      result = stringifyEntities(
        value,
        Object.assign({}, ctx.entities, {
          subset: constants.unquoted[ctx.valid][ctx.safe],
          attribute: true
        })
      );
    }

    // If we don’t want unquoted, or if `value` contains character references when
    // unquoted…
    if (result !== value) {
      // If the alternative is less common than `quote`, switch.
      if (ctx.smart && ccount(value, quote) > ccount(value, ctx.alternative)) {
        quote = ctx.alternative;
      }

      result =
        quote +
        stringifyEntities(
          value,
          Object.assign({}, ctx.entities, {
            // Always encode without parse errors in non-HTML.
            subset: (quote === "'" ? constants.single : constants.double)[
              ctx.schema.space === 'html' ? ctx.valid : 1
            ][ctx.safe],
            attribute: true
          })
        ) +
        quote;
    }

    // Don’t add a `=` for unquoted empties.
    return name + (result ? '=' + result : result)
  }

  /**
   * @typedef {import('./types.js').Node} Node
   * @typedef {import('./types.js').Options} Options
   * @typedef {import('./types.js').Context} Context
   * @typedef {import('./types.js').Quote} Quote
   */

  /**
   * @param {Node|Array.<Node>} node
   * @param {Options} [options]
   * @returns {string}
   */
  function toHtml(node, options = {}) {
    const quote = options.quote || '"';
    /** @type {Quote} */
    const alternative = quote === '"' ? "'" : '"';

    if (quote !== '"' && quote !== "'") {
      throw new Error('Invalid quote `' + quote + '`, expected `\'` or `"`')
    }

    /** @type {Context} */
    const context = {
      valid: options.allowParseErrors ? 0 : 1,
      safe: options.allowDangerousCharacters ? 0 : 1,
      schema: options.space === 'svg' ? svg$1 : html$4,
      omit: options.omitOptionalTags ? omission : undefined,
      quote,
      alternative,
      smart: options.quoteSmart,
      unquoted: options.preferUnquoted,
      tight: options.tightAttributes,
      upperDoctype: options.upperDoctype,
      tightDoctype: options.tightDoctype,
      bogusComments: options.bogusComments,
      tightLists: options.tightCommaSeparatedLists,
      tightClose: options.tightSelfClosing,
      collapseEmpty: options.collapseEmptyAttributes,
      dangerous: options.allowDangerousHtml,
      voids: options.voids || htmlVoidElements.concat(),
      entities: options.entities || {},
      close: options.closeSelfClosing,
      closeEmpty: options.closeEmptyElements
    };

    return one(
      context,
      // @ts-ignore Assume `node` does not contain a root.
      Array.isArray(node) ? {type: 'root', children: node} : node,
      null,
      null
    )
  }

  /**
   * @typedef {import('hast').Root} Root
   * @typedef {Root|Root['children'][number]} Node
   * @typedef {import('hast-util-to-html').Options} Options
   */

  /** @type {import('unified').Plugin<[Options?]|Array<void>, Node, string>} */
  function rehypeStringify(config) {
    const processorSettings = /** @type {Options} */ (this.data('settings'));
    const settings = Object.assign({}, processorSettings, config);

    Object.assign(this, {Compiler: compiler});

    /**
     * @type {import('unified').CompilerFunction<Node, string>}
     */
    function compiler(tree) {
      return toHtml(tree, settings)
    }
  }

  /**
   * @typedef {import('unist').Node} Node
   * @typedef {import('unist').Parent} Parent
   * @typedef {import('unist-util-is').Test} Test
   */

  const own$3 = {}.hasOwnProperty;

  /**
   * Create a new tree consisting of copies of all nodes that pass test.
   * The tree is walked in preorder (NLR), visiting the node itself, then its head, etc.
   *
   * @param tree Tree to filter.
   * @param options Configuration (optional).
   * @param test is-compatible test (such as a type).
   * @returns Given `tree` or `null` if it didn’t pass `test`.
   */
  const filter =
    /**
     * @type {(
     *  (<Tree extends Node, Check extends Test>(node: Tree, options: FilterOptions, test: Check) => import('./complex-types').Matches<Tree, Check>) &
     *  (<Tree extends Node, Check extends Test>(node: Tree, test: Check) => import('./complex-types').Matches<Tree, Check>) &
     *  (<Tree extends Node>(node: Tree, options?: FilterOptions) => Tree)
     * )}
     */
    (
      /**
       * @param {Node} tree
       * @param {FilterOptions} options
       * @param {Test} test
       * @returns {Node|null}
       */
      function (tree, options, test) {
        const is = convert$1(test || options);
        const cascade =
          options.cascade === undefined || options.cascade === null
            ? true
            : options.cascade;

        return preorder(tree)

        /**
         * @param {Node} node
         * @param {number|undefined} [index]
         * @param {Parent|undefined} [parent]
         * @returns {Node|null}
         */
        function preorder(node, index, parent) {
          /** @type {Array.<Node>} */
          const children = [];
          /** @type {number} */
          let childIndex;
          /** @type {Node} */
          let result;
          /** @type {string} */
          let key;

          if (!is(node, index, parent)) return null

          // @ts-expect-error: Looks like a parent.
          if (node.children) {
            childIndex = -1;

            // @ts-expect-error Looks like a parent.
            while (++childIndex < node.children.length) {
              // @ts-expect-error Looks like a parent.
              result = preorder(node.children[childIndex], childIndex, node);

              if (result) {
                children.push(result);
              }
            }

            // @ts-expect-error Looks like a parent.
            if (cascade && node.children.length > 0 && children.length === 0)
              return null
          }

          // Create a shallow clone, using the new children.
          /** @type {typeof node} */
          // @ts-expect-error all the fields will be copied over.
          const next = {};

          for (key in node) {
            if (own$3.call(node, key)) {
              // @ts-expect-error: Looks like a record.
              next[key] = key === 'children' ? children : node[key];
            }
          }

          return next
        }
      }
    );

  var parseNumericRange = {exports: {}};

  /**
   * @param {string} string    The string to parse
   * @returns {Array<number>}  Returns an energetic array.
   */

  (function (module, exports) {
  function parsePart(string) {
    let res = [];
    let m;

    for (let str of string.split(",").map((str) => str.trim())) {
      // just a number
      if (/^-?\d+$/.test(str)) {
        res.push(parseInt(str, 10));
      } else if (
        (m = str.match(/^(-?\d+)(-|\.\.\.?|\u2025|\u2026|\u22EF)(-?\d+)$/))
      ) {
        // 1-5 or 1..5 (equivalent) or 1...5 (doesn't include 5)
        let [_, lhs, sep, rhs] = m;

        if (lhs && rhs) {
          lhs = parseInt(lhs);
          rhs = parseInt(rhs);
          const incr = lhs < rhs ? 1 : -1;

          // Make it inclusive by moving the right 'stop-point' away by one.
          if (sep === "-" || sep === ".." || sep === "\u2025") rhs += incr;

          for (let i = lhs; i !== rhs; i += incr) res.push(i);
        }
      }
    }

    return res;
  }

  exports.default = parsePart;
  module.exports = parsePart;
  }(parseNumericRange, parseNumericRange.exports));

  var n = parseNumericRange.exports;

  // @ts-nocheck
  clike.displayName = 'clike';
  clike.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function clike(Prism) {
    Prism.languages.clike = {
      comment: [
        {
          pattern: /(^|[^\\])\/\*[\s\S]*?(?:\*\/|$)/,
          lookbehind: true,
          greedy: true
        },
        {
          pattern: /(^|[^\\:])\/\/.*/,
          lookbehind: true,
          greedy: true
        }
      ],
      string: {
        pattern: /(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,
        greedy: true
      },
      'class-name': {
        pattern:
          /(\b(?:class|extends|implements|instanceof|interface|new|trait)\s+|\bcatch\s+\()[\w.\\]+/i,
        lookbehind: true,
        inside: {
          punctuation: /[.\\]/
        }
      },
      keyword:
        /\b(?:break|catch|continue|do|else|finally|for|function|if|in|instanceof|new|null|return|throw|try|while)\b/,
      boolean: /\b(?:false|true)\b/,
      function: /\b\w+(?=\()/,
      number: /\b0x[\da-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?/i,
      operator: /[<>]=?|[!=]=?=?|--?|\+\+?|&&?|\|\|?|[?*/~^%]/,
      punctuation: /[{}[\];(),.:]/
    };
  }

  // @ts-nocheck
  c$1.displayName = 'c';
  c$1.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function c$1(Prism) {
    Prism.register(clike);
    Prism.languages.c = Prism.languages.extend('clike', {
      comment: {
        pattern:
          /\/\/(?:[^\r\n\\]|\\(?:\r\n?|\n|(?![\r\n])))*|\/\*[\s\S]*?(?:\*\/|$)/,
        greedy: true
      },
      string: {
        // https://en.cppreference.com/w/c/language/string_literal
        pattern: /"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"/,
        greedy: true
      },
      'class-name': {
        pattern:
          /(\b(?:enum|struct)\s+(?:__attribute__\s*\(\([\s\S]*?\)\)\s*)?)\w+|\b[a-z]\w*_t\b/,
        lookbehind: true
      },
      keyword:
        /\b(?:_Alignas|_Alignof|_Atomic|_Bool|_Complex|_Generic|_Imaginary|_Noreturn|_Static_assert|_Thread_local|__attribute__|asm|auto|break|case|char|const|continue|default|do|double|else|enum|extern|float|for|goto|if|inline|int|long|register|return|short|signed|sizeof|static|struct|switch|typedef|typeof|union|unsigned|void|volatile|while)\b/,
      function: /\b[a-z_]\w*(?=\s*\()/i,
      number:
        /(?:\b0x(?:[\da-f]+(?:\.[\da-f]*)?|\.[\da-f]+)(?:p[+-]?\d+)?|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?)[ful]{0,4}/i,
      operator: />>=?|<<=?|->|([-+&|:])\1|[?:~]|[-+*/%&|^!=<>]=?/
    });
    Prism.languages.insertBefore('c', 'string', {
      char: {
        // https://en.cppreference.com/w/c/language/character_constant
        pattern: /'(?:\\(?:\r\n|[\s\S])|[^'\\\r\n]){0,32}'/,
        greedy: true
      }
    });
    Prism.languages.insertBefore('c', 'string', {
      macro: {
        // allow for multiline macro definitions
        // spaces after the # character compile fine with gcc
        pattern:
          /(^[\t ]*)#\s*[a-z](?:[^\r\n\\/]|\/(?!\*)|\/\*(?:[^*]|\*(?!\/))*\*\/|\\(?:\r\n|[\s\S]))*/im,
        lookbehind: true,
        greedy: true,
        alias: 'property',
        inside: {
          string: [
            {
              // highlight the path of the include statement as a string
              pattern: /^(#\s*include\s*)<[^>]+>/,
              lookbehind: true
            },
            Prism.languages.c['string']
          ],
          char: Prism.languages.c['char'],
          comment: Prism.languages.c['comment'],
          'macro-name': [
            {
              pattern: /(^#\s*define\s+)\w+\b(?!\()/i,
              lookbehind: true
            },
            {
              pattern: /(^#\s*define\s+)\w+\b(?=\()/i,
              lookbehind: true,
              alias: 'function'
            }
          ],
          // highlight macro directives as keywords
          directive: {
            pattern: /^(#\s*)[a-z]+/,
            lookbehind: true,
            alias: 'keyword'
          },
          'directive-hash': /^#/,
          punctuation: /##|\\(?=[\r\n])/,
          expression: {
            pattern: /\S[\s\S]*/,
            inside: Prism.languages.c
          }
        }
      }
    });
    Prism.languages.insertBefore('c', 'function', {
      // highlight predefined macros as constants
      constant:
        /\b(?:EOF|NULL|SEEK_CUR|SEEK_END|SEEK_SET|__DATE__|__FILE__|__LINE__|__TIMESTAMP__|__TIME__|__func__|stderr|stdin|stdout)\b/
    });
    delete Prism.languages.c['boolean'];
  }

  // @ts-nocheck
  cpp.displayName = 'cpp';
  cpp.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function cpp(Prism) {
    Prism.register(c$1)
    ;(function (Prism) {
      var keyword =
        /\b(?:alignas|alignof|asm|auto|bool|break|case|catch|char|char16_t|char32_t|char8_t|class|co_await|co_return|co_yield|compl|concept|const|const_cast|consteval|constexpr|constinit|continue|decltype|default|delete|do|double|dynamic_cast|else|enum|explicit|export|extern|final|float|for|friend|goto|if|import|inline|int|int16_t|int32_t|int64_t|int8_t|long|module|mutable|namespace|new|noexcept|nullptr|operator|override|private|protected|public|register|reinterpret_cast|requires|return|short|signed|sizeof|static|static_assert|static_cast|struct|switch|template|this|thread_local|throw|try|typedef|typeid|typename|uint16_t|uint32_t|uint64_t|uint8_t|union|unsigned|using|virtual|void|volatile|wchar_t|while)\b/;
      var modName = /\b(?!<keyword>)\w+(?:\s*\.\s*\w+)*\b/.source.replace(
        /<keyword>/g,
        function () {
          return keyword.source
        }
      );
      Prism.languages.cpp = Prism.languages.extend('c', {
        'class-name': [
          {
            pattern: RegExp(
              /(\b(?:class|concept|enum|struct|typename)\s+)(?!<keyword>)\w+/.source.replace(
                /<keyword>/g,
                function () {
                  return keyword.source
                }
              )
            ),
            lookbehind: true
          }, // This is intended to capture the class name of method implementations like:
          //   void foo::bar() const {}
          // However! The `foo` in the above example could also be a namespace, so we only capture the class name if
          // it starts with an uppercase letter. This approximation should give decent results.
          /\b[A-Z]\w*(?=\s*::\s*\w+\s*\()/, // This will capture the class name before destructors like:
          //   Foo::~Foo() {}
          /\b[A-Z_]\w*(?=\s*::\s*~\w+\s*\()/i, // This also intends to capture the class name of method implementations but here the class has template
          // parameters, so it can't be a namespace (until C++ adds generic namespaces).
          /\b\w+(?=\s*<(?:[^<>]|<(?:[^<>]|<[^<>]*>)*>)*>\s*::\s*\w+\s*\()/
        ],
        keyword: keyword,
        number: {
          pattern:
            /(?:\b0b[01']+|\b0x(?:[\da-f']+(?:\.[\da-f']*)?|\.[\da-f']+)(?:p[+-]?[\d']+)?|(?:\b[\d']+(?:\.[\d']*)?|\B\.[\d']+)(?:e[+-]?[\d']+)?)[ful]{0,4}/i,
          greedy: true
        },
        operator:
          />>=?|<<=?|->|--|\+\+|&&|\|\||[?:~]|<=>|[-+*/%&|^!=<>]=?|\b(?:and|and_eq|bitand|bitor|not|not_eq|or|or_eq|xor|xor_eq)\b/,
        boolean: /\b(?:false|true)\b/
      });
      Prism.languages.insertBefore('cpp', 'string', {
        module: {
          // https://en.cppreference.com/w/cpp/language/modules
          pattern: RegExp(
            /(\b(?:import|module)\s+)/.source +
              '(?:' + // header-name
              /"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"|<[^<>\r\n]*>/.source +
              '|' + // module name or partition or both
              /<mod-name>(?:\s*:\s*<mod-name>)?|:\s*<mod-name>/.source.replace(
                /<mod-name>/g,
                function () {
                  return modName
                }
              ) +
              ')'
          ),
          lookbehind: true,
          greedy: true,
          inside: {
            string: /^[<"][\s\S]+/,
            operator: /:/,
            punctuation: /\./
          }
        },
        'raw-string': {
          pattern: /R"([^()\\ ]{0,16})\([\s\S]*?\)\1"/,
          alias: 'string',
          greedy: true
        }
      });
      Prism.languages.insertBefore('cpp', 'keyword', {
        'generic-function': {
          pattern: /\b(?!operator\b)[a-z_]\w*\s*<(?:[^<>]|<[^<>]*>)*>(?=\s*\()/i,
          inside: {
            function: /^\w+/,
            generic: {
              pattern: /<[\s\S]+/,
              alias: 'class-name',
              inside: Prism.languages.cpp
            }
          }
        }
      });
      Prism.languages.insertBefore('cpp', 'operator', {
        'double-colon': {
          pattern: /::/,
          alias: 'punctuation'
        }
      });
      Prism.languages.insertBefore('cpp', 'class-name', {
        // the base clause is an optional list of parent classes
        // https://en.cppreference.com/w/cpp/language/class
        'base-clause': {
          pattern:
            /(\b(?:class|struct)\s+\w+\s*:\s*)[^;{}"'\s]+(?:\s+[^;{}"'\s]+)*(?=\s*[;{])/,
          lookbehind: true,
          greedy: true,
          inside: Prism.languages.extend('cpp', {})
        }
      });
      Prism.languages.insertBefore(
        'inside',
        'double-colon',
        {
          // All untokenized words that are not namespaces should be class names
          'class-name': /\b[a-z_]\w*\b(?!\s*::)/i
        },
        Prism.languages.cpp['base-clause']
      );
    })(Prism);
  }

  // @ts-nocheck
  arduino.displayName = 'arduino';
  arduino.aliases = ['ino'];

  /** @type {import('../core.js').Syntax} */
  function arduino(Prism) {
    Prism.register(cpp);
    Prism.languages.arduino = Prism.languages.extend('cpp', {
      keyword:
        /\b(?:String|array|bool|boolean|break|byte|case|catch|continue|default|do|double|else|finally|for|function|goto|if|in|instanceof|int|integer|long|loop|new|null|return|setup|string|switch|throw|try|void|while|word)\b/,
      constant:
        /\b(?:ANALOG_MESSAGE|DEFAULT|DIGITAL_MESSAGE|EXTERNAL|FIRMATA_STRING|HIGH|INPUT|INPUT_PULLUP|INTERNAL|INTERNAL1V1|INTERNAL2V56|LED_BUILTIN|LOW|OUTPUT|REPORT_ANALOG|REPORT_DIGITAL|SET_PIN_MODE|SYSEX_START|SYSTEM_RESET)\b/,
      builtin:
        /\b(?:Audio|BSSID|Bridge|Client|Console|EEPROM|Esplora|EsploraTFT|Ethernet|EthernetClient|EthernetServer|EthernetUDP|File|FileIO|FileSystem|Firmata|GPRS|GSM|GSMBand|GSMClient|GSMModem|GSMPIN|GSMScanner|GSMServer|GSMVoiceCall|GSM_SMS|HttpClient|IPAddress|IRread|Keyboard|KeyboardController|LiquidCrystal|LiquidCrystal_I2C|Mailbox|Mouse|MouseController|PImage|Process|RSSI|RobotControl|RobotMotor|SD|SPI|SSID|Scheduler|Serial|Server|Servo|SoftwareSerial|Stepper|Stream|TFT|Task|USBHost|WiFi|WiFiClient|WiFiServer|WiFiUDP|Wire|YunClient|YunServer|abs|addParameter|analogRead|analogReadResolution|analogReference|analogWrite|analogWriteResolution|answerCall|attach|attachGPRS|attachInterrupt|attached|autoscroll|available|background|beep|begin|beginPacket|beginSD|beginSMS|beginSpeaker|beginTFT|beginTransmission|beginWrite|bit|bitClear|bitRead|bitSet|bitWrite|blink|blinkVersion|buffer|changePIN|checkPIN|checkPUK|checkReg|circle|cityNameRead|cityNameWrite|clear|clearScreen|click|close|compassRead|config|connect|connected|constrain|cos|countryNameRead|countryNameWrite|createChar|cursor|debugPrint|delay|delayMicroseconds|detach|detachInterrupt|digitalRead|digitalWrite|disconnect|display|displayLogos|drawBMP|drawCompass|encryptionType|end|endPacket|endSMS|endTransmission|endWrite|exists|exitValue|fill|find|findUntil|flush|gatewayIP|get|getAsynchronously|getBand|getButton|getCurrentCarrier|getIMEI|getKey|getModifiers|getOemKey|getPINUsed|getResult|getSignalStrength|getSocket|getVoiceCallStatus|getXChange|getYChange|hangCall|height|highByte|home|image|interrupts|isActionDone|isDirectory|isListening|isPIN|isPressed|isValid|keyPressed|keyReleased|keyboardRead|knobRead|leftToRight|line|lineFollowConfig|listen|listenOnLocalhost|loadImage|localIP|lowByte|macAddress|maintain|map|max|messageAvailable|micros|millis|min|mkdir|motorsStop|motorsWrite|mouseDragged|mouseMoved|mousePressed|mouseReleased|move|noAutoscroll|noBlink|noBuffer|noCursor|noDisplay|noFill|noInterrupts|noListenOnLocalhost|noStroke|noTone|onReceive|onRequest|open|openNextFile|overflow|parseCommand|parseFloat|parseInt|parsePacket|pauseMode|peek|pinMode|playFile|playMelody|point|pointTo|position|pow|prepare|press|print|printFirmwareVersion|printVersion|println|process|processInput|pulseIn|put|random|randomSeed|read|readAccelerometer|readBlue|readButton|readBytes|readBytesUntil|readGreen|readJoystickButton|readJoystickSwitch|readJoystickX|readJoystickY|readLightSensor|readMessage|readMicrophone|readNetworks|readRed|readSlider|readString|readStringUntil|readTemperature|ready|rect|release|releaseAll|remoteIP|remoteNumber|remotePort|remove|requestFrom|retrieveCallingNumber|rewindDirectory|rightToLeft|rmdir|robotNameRead|robotNameWrite|run|runAsynchronously|runShellCommand|runShellCommandAsynchronously|running|scanNetworks|scrollDisplayLeft|scrollDisplayRight|seek|sendAnalog|sendDigitalPortPair|sendDigitalPorts|sendString|sendSysex|serialEvent|setBand|setBitOrder|setClockDivider|setCursor|setDNS|setDataMode|setFirmwareVersion|setMode|setPINUsed|setSpeed|setTextSize|setTimeout|shiftIn|shiftOut|shutdown|sin|size|sqrt|startLoop|step|stop|stroke|subnetMask|switchPIN|tan|tempoWrite|text|tone|transfer|tuneWrite|turn|updateIR|userNameRead|userNameWrite|voiceCall|waitContinue|width|write|writeBlue|writeGreen|writeJSON|writeMessage|writeMicroseconds|writeRGB|writeRed|yield)\b/
    });
    Prism.languages.ino = Prism.languages.arduino;
  }

  // @ts-nocheck
  bash.displayName = 'bash';
  bash.aliases = ['shell'];

  /** @type {import('../core.js').Syntax} */
  function bash(Prism) {
  (function (Prism) {
      // $ set | grep '^[A-Z][^[:space:]]*=' | cut -d= -f1 | tr '\n' '|'
      // + LC_ALL, RANDOM, REPLY, SECONDS.
      // + make sure PS1..4 are here as they are not always set,
      // - some useless things.
      var envVars =
        '\\b(?:BASH|BASHOPTS|BASH_ALIASES|BASH_ARGC|BASH_ARGV|BASH_CMDS|BASH_COMPLETION_COMPAT_DIR|BASH_LINENO|BASH_REMATCH|BASH_SOURCE|BASH_VERSINFO|BASH_VERSION|COLORTERM|COLUMNS|COMP_WORDBREAKS|DBUS_SESSION_BUS_ADDRESS|DEFAULTS_PATH|DESKTOP_SESSION|DIRSTACK|DISPLAY|EUID|GDMSESSION|GDM_LANG|GNOME_KEYRING_CONTROL|GNOME_KEYRING_PID|GPG_AGENT_INFO|GROUPS|HISTCONTROL|HISTFILE|HISTFILESIZE|HISTSIZE|HOME|HOSTNAME|HOSTTYPE|IFS|INSTANCE|JOB|LANG|LANGUAGE|LC_ADDRESS|LC_ALL|LC_IDENTIFICATION|LC_MEASUREMENT|LC_MONETARY|LC_NAME|LC_NUMERIC|LC_PAPER|LC_TELEPHONE|LC_TIME|LESSCLOSE|LESSOPEN|LINES|LOGNAME|LS_COLORS|MACHTYPE|MAILCHECK|MANDATORY_PATH|NO_AT_BRIDGE|OLDPWD|OPTERR|OPTIND|ORBIT_SOCKETDIR|OSTYPE|PAPERSIZE|PATH|PIPESTATUS|PPID|PS1|PS2|PS3|PS4|PWD|RANDOM|REPLY|SECONDS|SELINUX_INIT|SESSION|SESSIONTYPE|SESSION_MANAGER|SHELL|SHELLOPTS|SHLVL|SSH_AUTH_SOCK|TERM|UID|UPSTART_EVENTS|UPSTART_INSTANCE|UPSTART_JOB|UPSTART_SESSION|USER|WINDOWID|XAUTHORITY|XDG_CONFIG_DIRS|XDG_CURRENT_DESKTOP|XDG_DATA_DIRS|XDG_GREETER_DATA_DIR|XDG_MENU_PREFIX|XDG_RUNTIME_DIR|XDG_SEAT|XDG_SEAT_PATH|XDG_SESSION_DESKTOP|XDG_SESSION_ID|XDG_SESSION_PATH|XDG_SESSION_TYPE|XDG_VTNR|XMODIFIERS)\\b';
      var commandAfterHeredoc = {
        pattern: /(^(["']?)\w+\2)[ \t]+\S.*/,
        lookbehind: true,
        alias: 'punctuation',
        // this looks reasonably well in all themes
        inside: null // see below
      };
      var insideString = {
        bash: commandAfterHeredoc,
        environment: {
          pattern: RegExp('\\$' + envVars),
          alias: 'constant'
        },
        variable: [
          // [0]: Arithmetic Environment
          {
            pattern: /\$?\(\([\s\S]+?\)\)/,
            greedy: true,
            inside: {
              // If there is a $ sign at the beginning highlight $(( and )) as variable
              variable: [
                {
                  pattern: /(^\$\(\([\s\S]+)\)\)/,
                  lookbehind: true
                },
                /^\$\(\(/
              ],
              number:
                /\b0x[\dA-Fa-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee]-?\d+)?/,
              // Operators according to https://www.gnu.org/software/bash/manual/bashref.html#Shell-Arithmetic
              operator:
                /--|\+\+|\*\*=?|<<=?|>>=?|&&|\|\||[=!+\-*/%<>^&|]=?|[?~:]/,
              // If there is no $ sign at the beginning highlight (( and )) as punctuation
              punctuation: /\(\(?|\)\)?|,|;/
            }
          }, // [1]: Command Substitution
          {
            pattern: /\$\((?:\([^)]+\)|[^()])+\)|`[^`]+`/,
            greedy: true,
            inside: {
              variable: /^\$\(|^`|\)$|`$/
            }
          }, // [2]: Brace expansion
          {
            pattern: /\$\{[^}]+\}/,
            greedy: true,
            inside: {
              operator: /:[-=?+]?|[!\/]|##?|%%?|\^\^?|,,?/,
              punctuation: /[\[\]]/,
              environment: {
                pattern: RegExp('(\\{)' + envVars),
                lookbehind: true,
                alias: 'constant'
              }
            }
          },
          /\$(?:\w+|[#?*!@$])/
        ],
        // Escape sequences from echo and printf's manuals, and escaped quotes.
        entity:
          /\\(?:[abceEfnrtv\\"]|O?[0-7]{1,3}|U[0-9a-fA-F]{8}|u[0-9a-fA-F]{4}|x[0-9a-fA-F]{1,2})/
      };
      Prism.languages.bash = {
        shebang: {
          pattern: /^#!\s*\/.*/,
          alias: 'important'
        },
        comment: {
          pattern: /(^|[^"{\\$])#.*/,
          lookbehind: true
        },
        'function-name': [
          // a) function foo {
          // b) foo() {
          // c) function foo() {
          // but not “foo {”
          {
            // a) and c)
            pattern: /(\bfunction\s+)[\w-]+(?=(?:\s*\(?:\s*\))?\s*\{)/,
            lookbehind: true,
            alias: 'function'
          },
          {
            // b)
            pattern: /\b[\w-]+(?=\s*\(\s*\)\s*\{)/,
            alias: 'function'
          }
        ],
        // Highlight variable names as variables in for and select beginnings.
        'for-or-select': {
          pattern: /(\b(?:for|select)\s+)\w+(?=\s+in\s)/,
          alias: 'variable',
          lookbehind: true
        },
        // Highlight variable names as variables in the left-hand part
        // of assignments (“=” and “+=”).
        'assign-left': {
          pattern: /(^|[\s;|&]|[<>]\()\w+(?=\+?=)/,
          inside: {
            environment: {
              pattern: RegExp('(^|[\\s;|&]|[<>]\\()' + envVars),
              lookbehind: true,
              alias: 'constant'
            }
          },
          alias: 'variable',
          lookbehind: true
        },
        string: [
          // Support for Here-documents https://en.wikipedia.org/wiki/Here_document
          {
            pattern: /((?:^|[^<])<<-?\s*)(\w+)\s[\s\S]*?(?:\r?\n|\r)\2/,
            lookbehind: true,
            greedy: true,
            inside: insideString
          }, // Here-document with quotes around the tag
          // → No expansion (so no “inside”).
          {
            pattern: /((?:^|[^<])<<-?\s*)(["'])(\w+)\2\s[\s\S]*?(?:\r?\n|\r)\3/,
            lookbehind: true,
            greedy: true,
            inside: {
              bash: commandAfterHeredoc
            }
          }, // “Normal” string
          {
            // https://www.gnu.org/software/bash/manual/html_node/Double-Quotes.html
            pattern:
              /(^|[^\\](?:\\\\)*)"(?:\\[\s\S]|\$\([^)]+\)|\$(?!\()|`[^`]+`|[^"\\`$])*"/,
            lookbehind: true,
            greedy: true,
            inside: insideString
          },
          {
            // https://www.gnu.org/software/bash/manual/html_node/Single-Quotes.html
            pattern: /(^|[^$\\])'[^']*'/,
            lookbehind: true,
            greedy: true
          },
          {
            // https://www.gnu.org/software/bash/manual/html_node/ANSI_002dC-Quoting.html
            pattern: /\$'(?:[^'\\]|\\[\s\S])*'/,
            greedy: true,
            inside: {
              entity: insideString.entity
            }
          }
        ],
        environment: {
          pattern: RegExp('\\$?' + envVars),
          alias: 'constant'
        },
        variable: insideString.variable,
        function: {
          pattern:
            /(^|[\s;|&]|[<>]\()(?:add|apropos|apt|apt-cache|apt-get|aptitude|aspell|automysqlbackup|awk|basename|bash|bc|bconsole|bg|bzip2|cal|cat|cfdisk|chgrp|chkconfig|chmod|chown|chroot|cksum|clear|cmp|column|comm|composer|cp|cron|crontab|csplit|curl|cut|date|dc|dd|ddrescue|debootstrap|df|diff|diff3|dig|dir|dircolors|dirname|dirs|dmesg|docker|docker-compose|du|egrep|eject|env|ethtool|expand|expect|expr|fdformat|fdisk|fg|fgrep|file|find|fmt|fold|format|free|fsck|ftp|fuser|gawk|git|gparted|grep|groupadd|groupdel|groupmod|groups|grub-mkconfig|gzip|halt|head|hg|history|host|hostname|htop|iconv|id|ifconfig|ifdown|ifup|import|install|ip|jobs|join|kill|killall|less|link|ln|locate|logname|logrotate|look|lpc|lpr|lprint|lprintd|lprintq|lprm|ls|lsof|lynx|make|man|mc|mdadm|mkconfig|mkdir|mke2fs|mkfifo|mkfs|mkisofs|mknod|mkswap|mmv|more|most|mount|mtools|mtr|mutt|mv|nano|nc|netstat|nice|nl|node|nohup|notify-send|npm|nslookup|op|open|parted|passwd|paste|pathchk|ping|pkill|pnpm|podman|podman-compose|popd|pr|printcap|printenv|ps|pushd|pv|quota|quotacheck|quotactl|ram|rar|rcp|reboot|remsync|rename|renice|rev|rm|rmdir|rpm|rsync|scp|screen|sdiff|sed|sendmail|seq|service|sftp|sh|shellcheck|shuf|shutdown|sleep|slocate|sort|split|ssh|stat|strace|su|sudo|sum|suspend|swapon|sync|tac|tail|tar|tee|time|timeout|top|touch|tr|traceroute|tsort|tty|umount|uname|unexpand|uniq|units|unrar|unshar|unzip|update-grub|uptime|useradd|userdel|usermod|users|uudecode|uuencode|v|vcpkg|vdir|vi|vim|virsh|vmstat|wait|watch|wc|wget|whereis|which|who|whoami|write|xargs|xdg-open|yarn|yes|zenity|zip|zsh|zypper)(?=$|[)\s;|&])/,
          lookbehind: true
        },
        keyword: {
          pattern:
            /(^|[\s;|&]|[<>]\()(?:case|do|done|elif|else|esac|fi|for|function|if|in|select|then|until|while)(?=$|[)\s;|&])/,
          lookbehind: true
        },
        // https://www.gnu.org/software/bash/manual/html_node/Shell-Builtin-Commands.html
        builtin: {
          pattern:
            /(^|[\s;|&]|[<>]\()(?:\.|:|alias|bind|break|builtin|caller|cd|command|continue|declare|echo|enable|eval|exec|exit|export|getopts|hash|help|let|local|logout|mapfile|printf|pwd|read|readarray|readonly|return|set|shift|shopt|source|test|times|trap|type|typeset|ulimit|umask|unalias|unset)(?=$|[)\s;|&])/,
          lookbehind: true,
          // Alias added to make those easier to distinguish from strings.
          alias: 'class-name'
        },
        boolean: {
          pattern: /(^|[\s;|&]|[<>]\()(?:false|true)(?=$|[)\s;|&])/,
          lookbehind: true
        },
        'file-descriptor': {
          pattern: /\B&\d\b/,
          alias: 'important'
        },
        operator: {
          // Lots of redirections here, but not just that.
          pattern:
            /\d?<>|>\||\+=|=[=~]?|!=?|<<[<-]?|[&\d]?>>|\d[<>]&?|[<>][&=]?|&[>&]?|\|[&|]?/,
          inside: {
            'file-descriptor': {
              pattern: /^\d/,
              alias: 'important'
            }
          }
        },
        punctuation: /\$?\(\(?|\)\)?|\.\.|[{}[\];\\]/,
        number: {
          pattern: /(^|\s)(?:[1-9]\d*|0)(?:[.,]\d+)?\b/,
          lookbehind: true
        }
      };
      commandAfterHeredoc.inside = Prism.languages.bash;
      /* Patterns in command substitution. */
      var toBeCopied = [
        'comment',
        'function-name',
        'for-or-select',
        'assign-left',
        'string',
        'environment',
        'function',
        'keyword',
        'builtin',
        'boolean',
        'file-descriptor',
        'operator',
        'punctuation',
        'number'
      ];
      var inside = insideString.variable[1].inside;
      for (var i = 0; i < toBeCopied.length; i++) {
        inside[toBeCopied[i]] = Prism.languages.bash[toBeCopied[i]];
      }
      Prism.languages.shell = Prism.languages.bash;
    })(Prism);
  }

  // @ts-nocheck
  csharp.displayName = 'csharp';
  csharp.aliases = ['cs', 'dotnet'];

  /** @type {import('../core.js').Syntax} */
  function csharp(Prism) {
    Prism.register(clike)
    ;(function (Prism) {
      /**
       * Replaces all placeholders "<<n>>" of given pattern with the n-th replacement (zero based).
       *
       * Note: This is a simple text based replacement. Be careful when using backreferences!
       *
       * @param {string} pattern the given pattern.
       * @param {string[]} replacements a list of replacement which can be inserted into the given pattern.
       * @returns {string} the pattern with all placeholders replaced with their corresponding replacements.
       * @example replace(/a<<0>>a/.source, [/b+/.source]) === /a(?:b+)a/.source
       */
      function replace(pattern, replacements) {
        return pattern.replace(/<<(\d+)>>/g, function (m, index) {
          return '(?:' + replacements[+index] + ')'
        })
      }
      /**
       * @param {string} pattern
       * @param {string[]} replacements
       * @param {string} [flags]
       * @returns {RegExp}
       */
      function re(pattern, replacements, flags) {
        return RegExp(replace(pattern, replacements), flags || '')
      }
      /**
       * Creates a nested pattern where all occurrences of the string `<<self>>` are replaced with the pattern itself.
       *
       * @param {string} pattern
       * @param {number} depthLog2
       * @returns {string}
       */
      function nested(pattern, depthLog2) {
        for (var i = 0; i < depthLog2; i++) {
          pattern = pattern.replace(/<<self>>/g, function () {
            return '(?:' + pattern + ')'
          });
        }
        return pattern.replace(/<<self>>/g, '[^\\s\\S]')
      } // https://docs.microsoft.com/en-us/dotnet/csharp/language-reference/keywords/
      var keywordKinds = {
        // keywords which represent a return or variable type
        type: 'bool byte char decimal double dynamic float int long object sbyte short string uint ulong ushort var void',
        // keywords which are used to declare a type
        typeDeclaration: 'class enum interface record struct',
        // contextual keywords
        // ("var" and "dynamic" are missing because they are used like types)
        contextual:
          'add alias and ascending async await by descending from(?=\\s*(?:\\w|$)) get global group into init(?=\\s*;) join let nameof not notnull on or orderby partial remove select set unmanaged value when where with(?=\\s*{)',
        // all other keywords
        other:
          'abstract as base break case catch checked const continue default delegate do else event explicit extern finally fixed for foreach goto if implicit in internal is lock namespace new null operator out override params private protected public readonly ref return sealed sizeof stackalloc static switch this throw try typeof unchecked unsafe using virtual volatile while yield'
      }; // keywords
      function keywordsToPattern(words) {
        return '\\b(?:' + words.trim().replace(/ /g, '|') + ')\\b'
      }
      var typeDeclarationKeywords = keywordsToPattern(
        keywordKinds.typeDeclaration
      );
      var keywords = RegExp(
        keywordsToPattern(
          keywordKinds.type +
            ' ' +
            keywordKinds.typeDeclaration +
            ' ' +
            keywordKinds.contextual +
            ' ' +
            keywordKinds.other
        )
      );
      var nonTypeKeywords = keywordsToPattern(
        keywordKinds.typeDeclaration +
          ' ' +
          keywordKinds.contextual +
          ' ' +
          keywordKinds.other
      );
      var nonContextualKeywords = keywordsToPattern(
        keywordKinds.type +
          ' ' +
          keywordKinds.typeDeclaration +
          ' ' +
          keywordKinds.other
      ); // types
      var generic = nested(/<(?:[^<>;=+\-*/%&|^]|<<self>>)*>/.source, 2); // the idea behind the other forbidden characters is to prevent false positives. Same for tupleElement.
      var nestedRound = nested(/\((?:[^()]|<<self>>)*\)/.source, 2);
      var name = /@?\b[A-Za-z_]\w*\b/.source;
      var genericName = replace(/<<0>>(?:\s*<<1>>)?/.source, [name, generic]);
      var identifier = replace(/(?!<<0>>)<<1>>(?:\s*\.\s*<<1>>)*/.source, [
        nonTypeKeywords,
        genericName
      ]);
      var array = /\[\s*(?:,\s*)*\]/.source;
      var typeExpressionWithoutTuple = replace(
        /<<0>>(?:\s*(?:\?\s*)?<<1>>)*(?:\s*\?)?/.source,
        [identifier, array]
      );
      var tupleElement = replace(
        /[^,()<>[\];=+\-*/%&|^]|<<0>>|<<1>>|<<2>>/.source,
        [generic, nestedRound, array]
      );
      var tuple = replace(/\(<<0>>+(?:,<<0>>+)+\)/.source, [tupleElement]);
      var typeExpression = replace(
        /(?:<<0>>|<<1>>)(?:\s*(?:\?\s*)?<<2>>)*(?:\s*\?)?/.source,
        [tuple, identifier, array]
      );
      var typeInside = {
        keyword: keywords,
        punctuation: /[<>()?,.:[\]]/
      }; // strings & characters
      // https://docs.microsoft.com/en-us/dotnet/csharp/language-reference/language-specification/lexical-structure#character-literals
      // https://docs.microsoft.com/en-us/dotnet/csharp/language-reference/language-specification/lexical-structure#string-literals
      var character = /'(?:[^\r\n'\\]|\\.|\\[Uux][\da-fA-F]{1,8})'/.source; // simplified pattern
      var regularString = /"(?:\\.|[^\\"\r\n])*"/.source;
      var verbatimString = /@"(?:""|\\[\s\S]|[^\\"])*"(?!")/.source;
      Prism.languages.csharp = Prism.languages.extend('clike', {
        string: [
          {
            pattern: re(/(^|[^$\\])<<0>>/.source, [verbatimString]),
            lookbehind: true,
            greedy: true
          },
          {
            pattern: re(/(^|[^@$\\])<<0>>/.source, [regularString]),
            lookbehind: true,
            greedy: true
          }
        ],
        'class-name': [
          {
            // Using static
            // using static System.Math;
            pattern: re(/(\busing\s+static\s+)<<0>>(?=\s*;)/.source, [
              identifier
            ]),
            lookbehind: true,
            inside: typeInside
          },
          {
            // Using alias (type)
            // using Project = PC.MyCompany.Project;
            pattern: re(/(\busing\s+<<0>>\s*=\s*)<<1>>(?=\s*;)/.source, [
              name,
              typeExpression
            ]),
            lookbehind: true,
            inside: typeInside
          },
          {
            // Using alias (alias)
            // using Project = PC.MyCompany.Project;
            pattern: re(/(\busing\s+)<<0>>(?=\s*=)/.source, [name]),
            lookbehind: true
          },
          {
            // Type declarations
            // class Foo<A, B>
            // interface Foo<out A, B>
            pattern: re(/(\b<<0>>\s+)<<1>>/.source, [
              typeDeclarationKeywords,
              genericName
            ]),
            lookbehind: true,
            inside: typeInside
          },
          {
            // Single catch exception declaration
            // catch(Foo)
            // (things like catch(Foo e) is covered by variable declaration)
            pattern: re(/(\bcatch\s*\(\s*)<<0>>/.source, [identifier]),
            lookbehind: true,
            inside: typeInside
          },
          {
            // Name of the type parameter of generic constraints
            // where Foo : class
            pattern: re(/(\bwhere\s+)<<0>>/.source, [name]),
            lookbehind: true
          },
          {
            // Casts and checks via as and is.
            // as Foo<A>, is Bar<B>
            // (things like if(a is Foo b) is covered by variable declaration)
            pattern: re(/(\b(?:is(?:\s+not)?|as)\s+)<<0>>/.source, [
              typeExpressionWithoutTuple
            ]),
            lookbehind: true,
            inside: typeInside
          },
          {
            // Variable, field and parameter declaration
            // (Foo bar, Bar baz, Foo[,,] bay, Foo<Bar, FooBar<Bar>> bax)
            pattern: re(
              /\b<<0>>(?=\s+(?!<<1>>|with\s*\{)<<2>>(?:\s*[=,;:{)\]]|\s+(?:in|when)\b))/
                .source,
              [typeExpression, nonContextualKeywords, name]
            ),
            inside: typeInside
          }
        ],
        keyword: keywords,
        // https://docs.microsoft.com/en-us/dotnet/csharp/language-reference/language-specification/lexical-structure#literals
        number:
          /(?:\b0(?:x[\da-f_]*[\da-f]|b[01_]*[01])|(?:\B\.\d+(?:_+\d+)*|\b\d+(?:_+\d+)*(?:\.\d+(?:_+\d+)*)?)(?:e[-+]?\d+(?:_+\d+)*)?)(?:[dflmu]|lu|ul)?\b/i,
        operator: />>=?|<<=?|[-=]>|([-+&|])\1|~|\?\?=?|[-+*/%&|^!=<>]=?/,
        punctuation: /\?\.?|::|[{}[\];(),.:]/
      });
      Prism.languages.insertBefore('csharp', 'number', {
        range: {
          pattern: /\.\./,
          alias: 'operator'
        }
      });
      Prism.languages.insertBefore('csharp', 'punctuation', {
        'named-parameter': {
          pattern: re(/([(,]\s*)<<0>>(?=\s*:)/.source, [name]),
          lookbehind: true,
          alias: 'punctuation'
        }
      });
      Prism.languages.insertBefore('csharp', 'class-name', {
        namespace: {
          // namespace Foo.Bar {}
          // using Foo.Bar;
          pattern: re(
            /(\b(?:namespace|using)\s+)<<0>>(?:\s*\.\s*<<0>>)*(?=\s*[;{])/.source,
            [name]
          ),
          lookbehind: true,
          inside: {
            punctuation: /\./
          }
        },
        'type-expression': {
          // default(Foo), typeof(Foo<Bar>), sizeof(int)
          pattern: re(
            /(\b(?:default|sizeof|typeof)\s*\(\s*(?!\s))(?:[^()\s]|\s(?!\s)|<<0>>)*(?=\s*\))/
              .source,
            [nestedRound]
          ),
          lookbehind: true,
          alias: 'class-name',
          inside: typeInside
        },
        'return-type': {
          // Foo<Bar> ForBar(); Foo IFoo.Bar() => 0
          // int this[int index] => 0; T IReadOnlyList<T>.this[int index] => this[index];
          // int Foo => 0; int Foo { get; set } = 0;
          pattern: re(
            /<<0>>(?=\s+(?:<<1>>\s*(?:=>|[({]|\.\s*this\s*\[)|this\s*\[))/.source,
            [typeExpression, identifier]
          ),
          inside: typeInside,
          alias: 'class-name'
        },
        'constructor-invocation': {
          // new List<Foo<Bar[]>> { }
          pattern: re(/(\bnew\s+)<<0>>(?=\s*[[({])/.source, [typeExpression]),
          lookbehind: true,
          inside: typeInside,
          alias: 'class-name'
        },
        /*'explicit-implementation': {
  // int IFoo<Foo>.Bar => 0; void IFoo<Foo<Foo>>.Foo<T>();
  pattern: replace(/\b<<0>>(?=\.<<1>>)/, className, methodOrPropertyDeclaration),
  inside: classNameInside,
  alias: 'class-name'
  },*/
        'generic-method': {
          // foo<Bar>()
          pattern: re(/<<0>>\s*<<1>>(?=\s*\()/.source, [name, generic]),
          inside: {
            function: re(/^<<0>>/.source, [name]),
            generic: {
              pattern: RegExp(generic),
              alias: 'class-name',
              inside: typeInside
            }
          }
        },
        'type-list': {
          // The list of types inherited or of generic constraints
          // class Foo<F> : Bar, IList<FooBar>
          // where F : Bar, IList<int>
          pattern: re(
            /\b((?:<<0>>\s+<<1>>|record\s+<<1>>\s*<<5>>|where\s+<<2>>)\s*:\s*)(?:<<3>>|<<4>>|<<1>>\s*<<5>>|<<6>>)(?:\s*,\s*(?:<<3>>|<<4>>|<<6>>))*(?=\s*(?:where|[{;]|=>|$))/
              .source,
            [
              typeDeclarationKeywords,
              genericName,
              name,
              typeExpression,
              keywords.source,
              nestedRound,
              /\bnew\s*\(\s*\)/.source
            ]
          ),
          lookbehind: true,
          inside: {
            'record-arguments': {
              pattern: re(/(^(?!new\s*\()<<0>>\s*)<<1>>/.source, [
                genericName,
                nestedRound
              ]),
              lookbehind: true,
              greedy: true,
              inside: Prism.languages.csharp
            },
            keyword: keywords,
            'class-name': {
              pattern: RegExp(typeExpression),
              greedy: true,
              inside: typeInside
            },
            punctuation: /[,()]/
          }
        },
        preprocessor: {
          pattern: /(^[\t ]*)#.*/m,
          lookbehind: true,
          alias: 'property',
          inside: {
            // highlight preprocessor directives as keywords
            directive: {
              pattern:
                /(#)\b(?:define|elif|else|endif|endregion|error|if|line|nullable|pragma|region|undef|warning)\b/,
              lookbehind: true,
              alias: 'keyword'
            }
          }
        }
      }); // attributes
      var regularStringOrCharacter = regularString + '|' + character;
      var regularStringCharacterOrComment = replace(
        /\/(?![*/])|\/\/[^\r\n]*[\r\n]|\/\*(?:[^*]|\*(?!\/))*\*\/|<<0>>/.source,
        [regularStringOrCharacter]
      );
      var roundExpression = nested(
        replace(/[^"'/()]|<<0>>|\(<<self>>*\)/.source, [
          regularStringCharacterOrComment
        ]),
        2
      ); // https://docs.microsoft.com/en-us/dotnet/csharp/programming-guide/concepts/attributes/#attribute-targets
      var attrTarget =
        /\b(?:assembly|event|field|method|module|param|property|return|type)\b/
          .source;
      var attr = replace(/<<0>>(?:\s*\(<<1>>*\))?/.source, [
        identifier,
        roundExpression
      ]);
      Prism.languages.insertBefore('csharp', 'class-name', {
        attribute: {
          // Attributes
          // [Foo], [Foo(1), Bar(2, Prop = "foo")], [return: Foo(1), Bar(2)], [assembly: Foo(Bar)]
          pattern: re(
            /((?:^|[^\s\w>)?])\s*\[\s*)(?:<<0>>\s*:\s*)?<<1>>(?:\s*,\s*<<1>>)*(?=\s*\])/
              .source,
            [attrTarget, attr]
          ),
          lookbehind: true,
          greedy: true,
          inside: {
            target: {
              pattern: re(/^<<0>>(?=\s*:)/.source, [attrTarget]),
              alias: 'keyword'
            },
            'attribute-arguments': {
              pattern: re(/\(<<0>>*\)/.source, [roundExpression]),
              inside: Prism.languages.csharp
            },
            'class-name': {
              pattern: RegExp(identifier),
              inside: {
                punctuation: /\./
              }
            },
            punctuation: /[:,]/
          }
        }
      }); // string interpolation
      var formatString = /:[^}\r\n]+/.source; // multi line
      var mInterpolationRound = nested(
        replace(/[^"'/()]|<<0>>|\(<<self>>*\)/.source, [
          regularStringCharacterOrComment
        ]),
        2
      );
      var mInterpolation = replace(/\{(?!\{)(?:(?![}:])<<0>>)*<<1>>?\}/.source, [
        mInterpolationRound,
        formatString
      ]); // single line
      var sInterpolationRound = nested(
        replace(
          /[^"'/()]|\/(?!\*)|\/\*(?:[^*]|\*(?!\/))*\*\/|<<0>>|\(<<self>>*\)/
            .source,
          [regularStringOrCharacter]
        ),
        2
      );
      var sInterpolation = replace(/\{(?!\{)(?:(?![}:])<<0>>)*<<1>>?\}/.source, [
        sInterpolationRound,
        formatString
      ]);
      function createInterpolationInside(interpolation, interpolationRound) {
        return {
          interpolation: {
            pattern: re(/((?:^|[^{])(?:\{\{)*)<<0>>/.source, [interpolation]),
            lookbehind: true,
            inside: {
              'format-string': {
                pattern: re(/(^\{(?:(?![}:])<<0>>)*)<<1>>(?=\}$)/.source, [
                  interpolationRound,
                  formatString
                ]),
                lookbehind: true,
                inside: {
                  punctuation: /^:/
                }
              },
              punctuation: /^\{|\}$/,
              expression: {
                pattern: /[\s\S]+/,
                alias: 'language-csharp',
                inside: Prism.languages.csharp
              }
            }
          },
          string: /[\s\S]+/
        }
      }
      Prism.languages.insertBefore('csharp', 'string', {
        'interpolation-string': [
          {
            pattern: re(
              /(^|[^\\])(?:\$@|@\$)"(?:""|\\[\s\S]|\{\{|<<0>>|[^\\{"])*"/.source,
              [mInterpolation]
            ),
            lookbehind: true,
            greedy: true,
            inside: createInterpolationInside(mInterpolation, mInterpolationRound)
          },
          {
            pattern: re(/(^|[^@\\])\$"(?:\\.|\{\{|<<0>>|[^\\"{])*"/.source, [
              sInterpolation
            ]),
            lookbehind: true,
            greedy: true,
            inside: createInterpolationInside(sInterpolation, sInterpolationRound)
          }
        ],
        char: {
          pattern: RegExp(character),
          greedy: true
        }
      });
      Prism.languages.dotnet = Prism.languages.cs = Prism.languages.csharp;
    })(Prism);
  }

  // @ts-nocheck
  markup.displayName = 'markup';
  markup.aliases = ['atom', 'html', 'mathml', 'rss', 'ssml', 'svg', 'xml'];

  /** @type {import('../core.js').Syntax} */
  function markup(Prism) {
    Prism.languages.markup = {
      comment: {
        pattern: /<!--(?:(?!<!--)[\s\S])*?-->/,
        greedy: true
      },
      prolog: {
        pattern: /<\?[\s\S]+?\?>/,
        greedy: true
      },
      doctype: {
        // https://www.w3.org/TR/xml/#NT-doctypedecl
        pattern:
          /<!DOCTYPE(?:[^>"'[\]]|"[^"]*"|'[^']*')+(?:\[(?:[^<"'\]]|"[^"]*"|'[^']*'|<(?!!--)|<!--(?:[^-]|-(?!->))*-->)*\]\s*)?>/i,
        greedy: true,
        inside: {
          'internal-subset': {
            pattern: /(^[^\[]*\[)[\s\S]+(?=\]>$)/,
            lookbehind: true,
            greedy: true,
            inside: null // see below
          },
          string: {
            pattern: /"[^"]*"|'[^']*'/,
            greedy: true
          },
          punctuation: /^<!|>$|[[\]]/,
          'doctype-tag': /^DOCTYPE/i,
          name: /[^\s<>'"]+/
        }
      },
      cdata: {
        pattern: /<!\[CDATA\[[\s\S]*?\]\]>/i,
        greedy: true
      },
      tag: {
        pattern:
          /<\/?(?!\d)[^\s>\/=$<%]+(?:\s(?:\s*[^\s>\/=]+(?:\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+(?=[\s>]))|(?=[\s/>])))+)?\s*\/?>/,
        greedy: true,
        inside: {
          tag: {
            pattern: /^<\/?[^\s>\/]+/,
            inside: {
              punctuation: /^<\/?/,
              namespace: /^[^\s>\/:]+:/
            }
          },
          'special-attr': [],
          'attr-value': {
            pattern: /=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+)/,
            inside: {
              punctuation: [
                {
                  pattern: /^=/,
                  alias: 'attr-equals'
                },
                /"|'/
              ]
            }
          },
          punctuation: /\/?>/,
          'attr-name': {
            pattern: /[^\s>\/]+/,
            inside: {
              namespace: /^[^\s>\/:]+:/
            }
          }
        }
      },
      entity: [
        {
          pattern: /&[\da-z]{1,8};/i,
          alias: 'named-entity'
        },
        /&#x?[\da-f]{1,8};/i
      ]
    };
    Prism.languages.markup['tag'].inside['attr-value'].inside['entity'] =
      Prism.languages.markup['entity'];
    Prism.languages.markup['doctype'].inside['internal-subset'].inside =
      Prism.languages.markup; // Plugin to make entity title show the real entity, idea by Roman Komarov
    Prism.hooks.add('wrap', function (env) {
      if (env.type === 'entity') {
        env.attributes['title'] = env.content.value.replace(/&amp;/, '&');
      }
    });
    Object.defineProperty(Prism.languages.markup.tag, 'addInlined', {
      /**
       * Adds an inlined language to markup.
       *
       * An example of an inlined language is CSS with `<style>` tags.
       *
       * @param {string} tagName The name of the tag that contains the inlined language. This name will be treated as
       * case insensitive.
       * @param {string} lang The language key.
       * @example
       * addInlined('style', 'css');
       */
      value: function addInlined(tagName, lang) {
        var includedCdataInside = {};
        includedCdataInside['language-' + lang] = {
          pattern: /(^<!\[CDATA\[)[\s\S]+?(?=\]\]>$)/i,
          lookbehind: true,
          inside: Prism.languages[lang]
        };
        includedCdataInside['cdata'] = /^<!\[CDATA\[|\]\]>$/i;
        var inside = {
          'included-cdata': {
            pattern: /<!\[CDATA\[[\s\S]*?\]\]>/i,
            inside: includedCdataInside
          }
        };
        inside['language-' + lang] = {
          pattern: /[\s\S]+/,
          inside: Prism.languages[lang]
        };
        var def = {};
        def[tagName] = {
          pattern: RegExp(
            /(<__[^>]*>)(?:<!\[CDATA\[(?:[^\]]|\](?!\]>))*\]\]>|(?!<!\[CDATA\[)[\s\S])*?(?=<\/__>)/.source.replace(
              /__/g,
              function () {
                return tagName
              }
            ),
            'i'
          ),
          lookbehind: true,
          greedy: true,
          inside: inside
        };
        Prism.languages.insertBefore('markup', 'cdata', def);
      }
    });
    Object.defineProperty(Prism.languages.markup.tag, 'addAttribute', {
      /**
       * Adds an pattern to highlight languages embedded in HTML attributes.
       *
       * An example of an inlined language is CSS with `style` attributes.
       *
       * @param {string} attrName The name of the tag that contains the inlined language. This name will be treated as
       * case insensitive.
       * @param {string} lang The language key.
       * @example
       * addAttribute('style', 'css');
       */
      value: function (attrName, lang) {
        Prism.languages.markup.tag.inside['special-attr'].push({
          pattern: RegExp(
            /(^|["'\s])/.source +
              '(?:' +
              attrName +
              ')' +
              /\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+(?=[\s>]))/.source,
            'i'
          ),
          lookbehind: true,
          inside: {
            'attr-name': /^[^\s=]+/,
            'attr-value': {
              pattern: /=[\s\S]+/,
              inside: {
                value: {
                  pattern: /(^=\s*(["']|(?!["'])))\S[\s\S]*(?=\2$)/,
                  lookbehind: true,
                  alias: [lang, 'language-' + lang],
                  inside: Prism.languages[lang]
                },
                punctuation: [
                  {
                    pattern: /^=/,
                    alias: 'attr-equals'
                  },
                  /"|'/
                ]
              }
            }
          }
        });
      }
    });
    Prism.languages.html = Prism.languages.markup;
    Prism.languages.mathml = Prism.languages.markup;
    Prism.languages.svg = Prism.languages.markup;
    Prism.languages.xml = Prism.languages.extend('markup', {});
    Prism.languages.ssml = Prism.languages.xml;
    Prism.languages.atom = Prism.languages.xml;
    Prism.languages.rss = Prism.languages.xml;
  }

  // @ts-nocheck
  css.displayName = 'css';
  css.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function css(Prism) {
  (function (Prism) {
      var string =
        /(?:"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"|'(?:\\(?:\r\n|[\s\S])|[^'\\\r\n])*')/;
      Prism.languages.css = {
        comment: /\/\*[\s\S]*?\*\//,
        atrule: {
          pattern: /@[\w-](?:[^;{\s]|\s+(?![\s{]))*(?:;|(?=\s*\{))/,
          inside: {
            rule: /^@[\w-]+/,
            'selector-function-argument': {
              pattern:
                /(\bselector\s*\(\s*(?![\s)]))(?:[^()\s]|\s+(?![\s)])|\((?:[^()]|\([^()]*\))*\))+(?=\s*\))/,
              lookbehind: true,
              alias: 'selector'
            },
            keyword: {
              pattern: /(^|[^\w-])(?:and|not|only|or)(?![\w-])/,
              lookbehind: true
            } // See rest below
          }
        },
        url: {
          // https://drafts.csswg.org/css-values-3/#urls
          pattern: RegExp(
            '\\burl\\((?:' +
              string.source +
              '|' +
              /(?:[^\\\r\n()"']|\\[\s\S])*/.source +
              ')\\)',
            'i'
          ),
          greedy: true,
          inside: {
            function: /^url/i,
            punctuation: /^\(|\)$/,
            string: {
              pattern: RegExp('^' + string.source + '$'),
              alias: 'url'
            }
          }
        },
        selector: {
          pattern: RegExp(
            '(^|[{}\\s])[^{}\\s](?:[^{};"\'\\s]|\\s+(?![\\s{])|' +
              string.source +
              ')*(?=\\s*\\{)'
          ),
          lookbehind: true
        },
        string: {
          pattern: string,
          greedy: true
        },
        property: {
          pattern:
            /(^|[^-\w\xA0-\uFFFF])(?!\s)[-_a-z\xA0-\uFFFF](?:(?!\s)[-\w\xA0-\uFFFF])*(?=\s*:)/i,
          lookbehind: true
        },
        important: /!important\b/i,
        function: {
          pattern: /(^|[^-a-z0-9])[-a-z0-9]+(?=\()/i,
          lookbehind: true
        },
        punctuation: /[(){};:,]/
      };
      Prism.languages.css['atrule'].inside.rest = Prism.languages.css;
      var markup = Prism.languages.markup;
      if (markup) {
        markup.tag.addInlined('style', 'css');
        markup.tag.addAttribute('style', 'css');
      }
    })(Prism);
  }

  // @ts-nocheck
  diff.displayName = 'diff';
  diff.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function diff(Prism) {
  (function (Prism) {
      Prism.languages.diff = {
        coord: [
          // Match all kinds of coord lines (prefixed by "+++", "---" or "***").
          /^(?:\*{3}|-{3}|\+{3}).*$/m, // Match "@@ ... @@" coord lines in unified diff.
          /^@@.*@@$/m, // Match coord lines in normal diff (starts with a number).
          /^\d.*$/m
        ] // deleted, inserted, unchanged, diff
      };
      /**
       * A map from the name of a block to its line prefix.
       *
       * @type {Object<string, string>}
       */
      var PREFIXES = {
        'deleted-sign': '-',
        'deleted-arrow': '<',
        'inserted-sign': '+',
        'inserted-arrow': '>',
        unchanged: ' ',
        diff: '!'
      }; // add a token for each prefix
      Object.keys(PREFIXES).forEach(function (name) {
        var prefix = PREFIXES[name];
        var alias = [];
        if (!/^\w+$/.test(name)) {
          // "deleted-sign" -> "deleted"
          alias.push(/\w+/.exec(name)[0]);
        }
        if (name === 'diff') {
          alias.push('bold');
        }
        Prism.languages.diff[name] = {
          pattern: RegExp(
            '^(?:[' + prefix + '].*(?:\r\n?|\n|(?![\\s\\S])))+',
            'm'
          ),
          alias: alias,
          inside: {
            line: {
              pattern: /(.)(?=[\s\S]).*(?:\r\n?|\n)?/,
              lookbehind: true
            },
            prefix: {
              pattern: /[\s\S]/,
              alias: /\w+/.exec(name)[0]
            }
          }
        };
      }); // make prefixes available to Diff plugin
      Object.defineProperty(Prism.languages.diff, 'PREFIXES', {
        value: PREFIXES
      });
    })(Prism);
  }

  // @ts-nocheck
  go.displayName = 'go';
  go.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function go(Prism) {
    Prism.register(clike);
    Prism.languages.go = Prism.languages.extend('clike', {
      string: {
        pattern: /(^|[^\\])"(?:\\.|[^"\\\r\n])*"|`[^`]*`/,
        lookbehind: true,
        greedy: true
      },
      keyword:
        /\b(?:break|case|chan|const|continue|default|defer|else|fallthrough|for|func|go(?:to)?|if|import|interface|map|package|range|return|select|struct|switch|type|var)\b/,
      boolean: /\b(?:_|false|iota|nil|true)\b/,
      number: [
        // binary and octal integers
        /\b0(?:b[01_]+|o[0-7_]+)i?\b/i, // hexadecimal integers and floats
        /\b0x(?:[a-f\d_]+(?:\.[a-f\d_]*)?|\.[a-f\d_]+)(?:p[+-]?\d+(?:_\d+)*)?i?(?!\w)/i, // decimal integers and floats
        /(?:\b\d[\d_]*(?:\.[\d_]*)?|\B\.\d[\d_]*)(?:e[+-]?[\d_]+)?i?(?!\w)/i
      ],
      operator:
        /[*\/%^!=]=?|\+[=+]?|-[=-]?|\|[=|]?|&(?:=|&|\^=?)?|>(?:>=?|=)?|<(?:<=?|=|-)?|:=|\.\.\./,
      builtin:
        /\b(?:append|bool|byte|cap|close|complex|complex(?:64|128)|copy|delete|error|float(?:32|64)|u?int(?:8|16|32|64)?|imag|len|make|new|panic|print(?:ln)?|real|recover|rune|string|uintptr)\b/
    });
    Prism.languages.insertBefore('go', 'string', {
      char: {
        pattern: /'(?:\\.|[^'\\\r\n]){0,10}'/,
        greedy: true
      }
    });
    delete Prism.languages.go['class-name'];
  }

  // @ts-nocheck
  ini.displayName = 'ini';
  ini.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function ini(Prism) {
    Prism.languages.ini = {
      /**
       * The component mimics the behavior of the Win32 API parser.
       *
       * @see {@link https://github.com/PrismJS/prism/issues/2775#issuecomment-787477723}
       */
      comment: {
        pattern: /(^[ \f\t\v]*)[#;][^\n\r]*/m,
        lookbehind: true
      },
      section: {
        pattern: /(^[ \f\t\v]*)\[[^\n\r\]]*\]?/m,
        lookbehind: true,
        inside: {
          'section-name': {
            pattern: /(^\[[ \f\t\v]*)[^ \f\t\v\]]+(?:[ \f\t\v]+[^ \f\t\v\]]+)*/,
            lookbehind: true,
            alias: 'selector'
          },
          punctuation: /\[|\]/
        }
      },
      key: {
        pattern:
          /(^[ \f\t\v]*)[^ \f\n\r\t\v=]+(?:[ \f\t\v]+[^ \f\n\r\t\v=]+)*(?=[ \f\t\v]*=)/m,
        lookbehind: true,
        alias: 'attr-name'
      },
      value: {
        pattern: /(=[ \f\t\v]*)[^ \f\n\r\t\v]+(?:[ \f\t\v]+[^ \f\n\r\t\v]+)*/,
        lookbehind: true,
        alias: 'attr-value',
        inside: {
          'inner-value': {
            pattern: /^("|').+(?=\1$)/,
            lookbehind: true
          }
        }
      },
      punctuation: /=/
    };
  }

  // @ts-nocheck
  java.displayName = 'java';
  java.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function java(Prism) {
    Prism.register(clike)
    ;(function (Prism) {
      var keywords =
        /\b(?:abstract|assert|boolean|break|byte|case|catch|char|class|const|continue|default|do|double|else|enum|exports|extends|final|finally|float|for|goto|if|implements|import|instanceof|int|interface|long|module|native|new|non-sealed|null|open|opens|package|permits|private|protected|provides|public|record|requires|return|sealed|short|static|strictfp|super|switch|synchronized|this|throw|throws|to|transient|transitive|try|uses|var|void|volatile|while|with|yield)\b/; // full package (optional) + parent classes (optional)
      var classNamePrefix = /(^|[^\w.])(?:[a-z]\w*\s*\.\s*)*(?:[A-Z]\w*\s*\.\s*)*/
        .source; // based on the java naming conventions
      var className = {
        pattern: RegExp(classNamePrefix + /[A-Z](?:[\d_A-Z]*[a-z]\w*)?\b/.source),
        lookbehind: true,
        inside: {
          namespace: {
            pattern: /^[a-z]\w*(?:\s*\.\s*[a-z]\w*)*(?:\s*\.)?/,
            inside: {
              punctuation: /\./
            }
          },
          punctuation: /\./
        }
      };
      Prism.languages.java = Prism.languages.extend('clike', {
        string: {
          pattern: /(^|[^\\])"(?:\\.|[^"\\\r\n])*"/,
          lookbehind: true,
          greedy: true
        },
        'class-name': [
          className,
          {
            // variables and parameters
            // this to support class names (or generic parameters) which do not contain a lower case letter (also works for methods)
            pattern: RegExp(
              classNamePrefix + /[A-Z]\w*(?=\s+\w+\s*[;,=()])/.source
            ),
            lookbehind: true,
            inside: className.inside
          }
        ],
        keyword: keywords,
        function: [
          Prism.languages.clike.function,
          {
            pattern: /(::\s*)[a-z_]\w*/,
            lookbehind: true
          }
        ],
        number:
          /\b0b[01][01_]*L?\b|\b0x(?:\.[\da-f_p+-]+|[\da-f_]+(?:\.[\da-f_p+-]+)?)\b|(?:\b\d[\d_]*(?:\.[\d_]*)?|\B\.\d[\d_]*)(?:e[+-]?\d[\d_]*)?[dfl]?/i,
        operator: {
          pattern:
            /(^|[^.])(?:<<=?|>>>?=?|->|--|\+\+|&&|\|\||::|[?:~]|[-+*/%&|^!=<>]=?)/m,
          lookbehind: true
        }
      });
      Prism.languages.insertBefore('java', 'string', {
        'triple-quoted-string': {
          // http://openjdk.java.net/jeps/355#Description
          pattern: /"""[ \t]*[\r\n](?:(?:"|"")?(?:\\.|[^"\\]))*"""/,
          greedy: true,
          alias: 'string'
        },
        char: {
          pattern: /'(?:\\.|[^'\\\r\n]){1,6}'/,
          greedy: true
        }
      });
      Prism.languages.insertBefore('java', 'class-name', {
        annotation: {
          pattern: /(^|[^.])@\w+(?:\s*\.\s*\w+)*/,
          lookbehind: true,
          alias: 'punctuation'
        },
        generics: {
          pattern:
            /<(?:[\w\s,.?]|&(?!&)|<(?:[\w\s,.?]|&(?!&)|<(?:[\w\s,.?]|&(?!&)|<(?:[\w\s,.?]|&(?!&))*>)*>)*>)*>/,
          inside: {
            'class-name': className,
            keyword: keywords,
            punctuation: /[<>(),.:]/,
            operator: /[?&|]/
          }
        },
        namespace: {
          pattern: RegExp(
            /(\b(?:exports|import(?:\s+static)?|module|open|opens|package|provides|requires|to|transitive|uses|with)\s+)(?!<keyword>)[a-z]\w*(?:\.[a-z]\w*)*\.?/.source.replace(
              /<keyword>/g,
              function () {
                return keywords.source
              }
            )
          ),
          lookbehind: true,
          inside: {
            punctuation: /\./
          }
        }
      });
    })(Prism);
  }

  // @ts-nocheck
  regex.displayName = 'regex';
  regex.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function regex(Prism) {
  (function (Prism) {
      var specialEscape = {
        pattern: /\\[\\(){}[\]^$+*?|.]/,
        alias: 'escape'
      };
      var escape =
        /\\(?:x[\da-fA-F]{2}|u[\da-fA-F]{4}|u\{[\da-fA-F]+\}|0[0-7]{0,2}|[123][0-7]{2}|c[a-zA-Z]|.)/;
      var charSet = {
        pattern: /\.|\\[wsd]|\\p\{[^{}]+\}/i,
        alias: 'class-name'
      };
      var charSetWithoutDot = {
        pattern: /\\[wsd]|\\p\{[^{}]+\}/i,
        alias: 'class-name'
      };
      var rangeChar = '(?:[^\\\\-]|' + escape.source + ')';
      var range = RegExp(rangeChar + '-' + rangeChar); // the name of a capturing group
      var groupName = {
        pattern: /(<|')[^<>']+(?=[>']$)/,
        lookbehind: true,
        alias: 'variable'
      };
      Prism.languages.regex = {
        'char-class': {
          pattern: /((?:^|[^\\])(?:\\\\)*)\[(?:[^\\\]]|\\[\s\S])*\]/,
          lookbehind: true,
          inside: {
            'char-class-negation': {
              pattern: /(^\[)\^/,
              lookbehind: true,
              alias: 'operator'
            },
            'char-class-punctuation': {
              pattern: /^\[|\]$/,
              alias: 'punctuation'
            },
            range: {
              pattern: range,
              inside: {
                escape: escape,
                'range-punctuation': {
                  pattern: /-/,
                  alias: 'operator'
                }
              }
            },
            'special-escape': specialEscape,
            'char-set': charSetWithoutDot,
            escape: escape
          }
        },
        'special-escape': specialEscape,
        'char-set': charSet,
        backreference: [
          {
            // a backreference which is not an octal escape
            pattern: /\\(?![123][0-7]{2})[1-9]/,
            alias: 'keyword'
          },
          {
            pattern: /\\k<[^<>']+>/,
            alias: 'keyword',
            inside: {
              'group-name': groupName
            }
          }
        ],
        anchor: {
          pattern: /[$^]|\\[ABbGZz]/,
          alias: 'function'
        },
        escape: escape,
        group: [
          {
            // https://docs.oracle.com/javase/10/docs/api/java/util/regex/Pattern.html
            // https://docs.microsoft.com/en-us/dotnet/standard/base-types/regular-expression-language-quick-reference?view=netframework-4.7.2#grouping-constructs
            // (), (?<name>), (?'name'), (?>), (?:), (?=), (?!), (?<=), (?<!), (?is-m), (?i-m:)
            pattern:
              /\((?:\?(?:<[^<>']+>|'[^<>']+'|[>:]|<?[=!]|[idmnsuxU]+(?:-[idmnsuxU]+)?:?))?/,
            alias: 'punctuation',
            inside: {
              'group-name': groupName
            }
          },
          {
            pattern: /\)/,
            alias: 'punctuation'
          }
        ],
        quantifier: {
          pattern: /(?:[+*?]|\{\d+(?:,\d*)?\})[?+]?/,
          alias: 'number'
        },
        alternation: {
          pattern: /\|/,
          alias: 'keyword'
        }
      };
    })(Prism);
  }

  // @ts-nocheck
  javascript.displayName = 'javascript';
  javascript.aliases = ['js'];

  /** @type {import('../core.js').Syntax} */
  function javascript(Prism) {
    Prism.register(clike);
    Prism.languages.javascript = Prism.languages.extend('clike', {
      'class-name': [
        Prism.languages.clike['class-name'],
        {
          pattern:
            /(^|[^$\w\xA0-\uFFFF])(?!\s)[_$A-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\.(?:constructor|prototype))/,
          lookbehind: true
        }
      ],
      keyword: [
        {
          pattern: /((?:^|\})\s*)catch\b/,
          lookbehind: true
        },
        {
          pattern:
            /(^|[^.]|\.\.\.\s*)\b(?:as|assert(?=\s*\{)|async(?=\s*(?:function\b|\(|[$\w\xA0-\uFFFF]|$))|await|break|case|class|const|continue|debugger|default|delete|do|else|enum|export|extends|finally(?=\s*(?:\{|$))|for|from(?=\s*(?:['"]|$))|function|(?:get|set)(?=\s*(?:[#\[$\w\xA0-\uFFFF]|$))|if|implements|import|in|instanceof|interface|let|new|null|of|package|private|protected|public|return|static|super|switch|this|throw|try|typeof|undefined|var|void|while|with|yield)\b/,
          lookbehind: true
        }
      ],
      // Allow for all non-ASCII characters (See http://stackoverflow.com/a/2008444)
      function:
        /#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*(?:\.\s*(?:apply|bind|call)\s*)?\()/,
      number: {
        pattern: RegExp(
          /(^|[^\w$])/.source +
            '(?:' + // constant
            (/NaN|Infinity/.source +
              '|' + // binary integer
              /0[bB][01]+(?:_[01]+)*n?/.source +
              '|' + // octal integer
              /0[oO][0-7]+(?:_[0-7]+)*n?/.source +
              '|' + // hexadecimal integer
              /0[xX][\dA-Fa-f]+(?:_[\dA-Fa-f]+)*n?/.source +
              '|' + // decimal bigint
              /\d+(?:_\d+)*n/.source +
              '|' + // decimal number (integer or float) but no bigint
              /(?:\d+(?:_\d+)*(?:\.(?:\d+(?:_\d+)*)?)?|\.\d+(?:_\d+)*)(?:[Ee][+-]?\d+(?:_\d+)*)?/
                .source) +
            ')' +
            /(?![\w$])/.source
        ),
        lookbehind: true
      },
      operator:
        /--|\+\+|\*\*=?|=>|&&=?|\|\|=?|[!=]==|<<=?|>>>?=?|[-+*/%&|^!=<>]=?|\.{3}|\?\?=?|\?\.?|[~:]/
    });
    Prism.languages.javascript['class-name'][0].pattern =
      /(\b(?:class|extends|implements|instanceof|interface|new)\s+)[\w.\\]+/;
    Prism.languages.insertBefore('javascript', 'keyword', {
      regex: {
        // eslint-disable-next-line regexp/no-dupe-characters-character-class
        pattern:
          /((?:^|[^$\w\xA0-\uFFFF."'\])\s]|\b(?:return|yield))\s*)\/(?:\[(?:[^\]\\\r\n]|\\.)*\]|\\.|[^/\\\[\r\n])+\/[dgimyus]{0,7}(?=(?:\s|\/\*(?:[^*]|\*(?!\/))*\*\/)*(?:$|[\r\n,.;:})\]]|\/\/))/,
        lookbehind: true,
        greedy: true,
        inside: {
          'regex-source': {
            pattern: /^(\/)[\s\S]+(?=\/[a-z]*$)/,
            lookbehind: true,
            alias: 'language-regex',
            inside: Prism.languages.regex
          },
          'regex-delimiter': /^\/|\/$/,
          'regex-flags': /^[a-z]+$/
        }
      },
      // This must be declared before keyword because we use "function" inside the look-forward
      'function-variable': {
        pattern:
          /#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*[=:]\s*(?:async\s*)?(?:\bfunction\b|(?:\((?:[^()]|\([^()]*\))*\)|(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*)\s*=>))/,
        alias: 'function'
      },
      parameter: [
        {
          pattern:
            /(function(?:\s+(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*)?\s*\(\s*)(?!\s)(?:[^()\s]|\s+(?![\s)])|\([^()]*\))+(?=\s*\))/,
          lookbehind: true,
          inside: Prism.languages.javascript
        },
        {
          pattern:
            /(^|[^$\w\xA0-\uFFFF])(?!\s)[_$a-z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*=>)/i,
          lookbehind: true,
          inside: Prism.languages.javascript
        },
        {
          pattern:
            /(\(\s*)(?!\s)(?:[^()\s]|\s+(?![\s)])|\([^()]*\))+(?=\s*\)\s*=>)/,
          lookbehind: true,
          inside: Prism.languages.javascript
        },
        {
          pattern:
            /((?:\b|\s|^)(?!(?:as|async|await|break|case|catch|class|const|continue|debugger|default|delete|do|else|enum|export|extends|finally|for|from|function|get|if|implements|import|in|instanceof|interface|let|new|null|of|package|private|protected|public|return|set|static|super|switch|this|throw|try|typeof|undefined|var|void|while|with|yield)(?![$\w\xA0-\uFFFF]))(?:(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*\s*)\(\s*|\]\s*\(\s*)(?!\s)(?:[^()\s]|\s+(?![\s)])|\([^()]*\))+(?=\s*\)\s*\{)/,
          lookbehind: true,
          inside: Prism.languages.javascript
        }
      ],
      constant: /\b[A-Z](?:[A-Z_]|\dx?)*\b/
    });
    Prism.languages.insertBefore('javascript', 'string', {
      hashbang: {
        pattern: /^#!.*/,
        greedy: true,
        alias: 'comment'
      },
      'template-string': {
        pattern:
          /`(?:\\[\s\S]|\$\{(?:[^{}]|\{(?:[^{}]|\{[^}]*\})*\})+\}|(?!\$\{)[^\\`])*`/,
        greedy: true,
        inside: {
          'template-punctuation': {
            pattern: /^`|`$/,
            alias: 'string'
          },
          interpolation: {
            pattern:
              /((?:^|[^\\])(?:\\{2})*)\$\{(?:[^{}]|\{(?:[^{}]|\{[^}]*\})*\})+\}/,
            lookbehind: true,
            inside: {
              'interpolation-punctuation': {
                pattern: /^\$\{|\}$/,
                alias: 'punctuation'
              },
              rest: Prism.languages.javascript
            }
          },
          string: /[\s\S]+/
        }
      },
      'string-property': {
        pattern:
          /((?:^|[,{])[ \t]*)(["'])(?:\\(?:\r\n|[\s\S])|(?!\2)[^\\\r\n])*\2(?=\s*:)/m,
        lookbehind: true,
        greedy: true,
        alias: 'property'
      }
    });
    Prism.languages.insertBefore('javascript', 'operator', {
      'literal-property': {
        pattern:
          /((?:^|[,{])[ \t]*)(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*:)/m,
        lookbehind: true,
        alias: 'property'
      }
    });
    if (Prism.languages.markup) {
      Prism.languages.markup.tag.addInlined('script', 'javascript'); // add attribute support for all DOM events.
      // https://developer.mozilla.org/en-US/docs/Web/Events#Standard_events
      Prism.languages.markup.tag.addAttribute(
        /on(?:abort|blur|change|click|composition(?:end|start|update)|dblclick|error|focus(?:in|out)?|key(?:down|up)|load|mouse(?:down|enter|leave|move|out|over|up)|reset|resize|scroll|select|slotchange|submit|unload|wheel)/
          .source,
        'javascript'
      );
    }
    Prism.languages.js = Prism.languages.javascript;
  }

  // @ts-nocheck
  json.displayName = 'json';
  json.aliases = ['webmanifest'];

  /** @type {import('../core.js').Syntax} */
  function json(Prism) {
    // https://www.json.org/json-en.html
    Prism.languages.json = {
      property: {
        pattern: /(^|[^\\])"(?:\\.|[^\\"\r\n])*"(?=\s*:)/,
        lookbehind: true,
        greedy: true
      },
      string: {
        pattern: /(^|[^\\])"(?:\\.|[^\\"\r\n])*"(?!\s*:)/,
        lookbehind: true,
        greedy: true
      },
      comment: {
        pattern: /\/\/.*|\/\*[\s\S]*?(?:\*\/|$)/,
        greedy: true
      },
      number: /-?\b\d+(?:\.\d+)?(?:e[+-]?\d+)?\b/i,
      punctuation: /[{}[\],]/,
      operator: /:/,
      boolean: /\b(?:false|true)\b/,
      null: {
        pattern: /\bnull\b/,
        alias: 'keyword'
      }
    };
    Prism.languages.webmanifest = Prism.languages.json;
  }

  // @ts-nocheck
  kotlin.displayName = 'kotlin';
  kotlin.aliases = ['kt', 'kts'];

  /** @type {import('../core.js').Syntax} */
  function kotlin(Prism) {
    Prism.register(clike)
    ;(function (Prism) {
      Prism.languages.kotlin = Prism.languages.extend('clike', {
        keyword: {
          // The lookbehind prevents wrong highlighting of e.g. kotlin.properties.get
          pattern:
            /(^|[^.])\b(?:abstract|actual|annotation|as|break|by|catch|class|companion|const|constructor|continue|crossinline|data|do|dynamic|else|enum|expect|external|final|finally|for|fun|get|if|import|in|infix|init|inline|inner|interface|internal|is|lateinit|noinline|null|object|open|operator|out|override|package|private|protected|public|reified|return|sealed|set|super|suspend|tailrec|this|throw|to|try|typealias|val|var|vararg|when|where|while)\b/,
          lookbehind: true
        },
        function: [
          {
            pattern: /(?:`[^\r\n`]+`|\b\w+)(?=\s*\()/,
            greedy: true
          },
          {
            pattern: /(\.)(?:`[^\r\n`]+`|\w+)(?=\s*\{)/,
            lookbehind: true,
            greedy: true
          }
        ],
        number:
          /\b(?:0[xX][\da-fA-F]+(?:_[\da-fA-F]+)*|0[bB][01]+(?:_[01]+)*|\d+(?:_\d+)*(?:\.\d+(?:_\d+)*)?(?:[eE][+-]?\d+(?:_\d+)*)?[fFL]?)\b/,
        operator:
          /\+[+=]?|-[-=>]?|==?=?|!(?:!|==?)?|[\/*%<>]=?|[?:]:?|\.\.|&&|\|\||\b(?:and|inv|or|shl|shr|ushr|xor)\b/
      });
      delete Prism.languages.kotlin['class-name'];
      var interpolationInside = {
        'interpolation-punctuation': {
          pattern: /^\$\{?|\}$/,
          alias: 'punctuation'
        },
        expression: {
          pattern: /[\s\S]+/,
          inside: Prism.languages.kotlin
        }
      };
      Prism.languages.insertBefore('kotlin', 'string', {
        // https://kotlinlang.org/spec/expressions.html#string-interpolation-expressions
        'string-literal': [
          {
            pattern: /"""(?:[^$]|\$(?:(?!\{)|\{[^{}]*\}))*?"""/,
            alias: 'multiline',
            inside: {
              interpolation: {
                pattern: /\$(?:[a-z_]\w*|\{[^{}]*\})/i,
                inside: interpolationInside
              },
              string: /[\s\S]+/
            }
          },
          {
            pattern: /"(?:[^"\\\r\n$]|\\.|\$(?:(?!\{)|\{[^{}]*\}))*"/,
            alias: 'singleline',
            inside: {
              interpolation: {
                pattern: /((?:^|[^\\])(?:\\{2})*)\$(?:[a-z_]\w*|\{[^{}]*\})/i,
                lookbehind: true,
                inside: interpolationInside
              },
              string: /[\s\S]+/
            }
          }
        ],
        char: {
          // https://kotlinlang.org/spec/expressions.html#character-literals
          pattern: /'(?:[^'\\\r\n]|\\(?:.|u[a-fA-F0-9]{0,4}))'/,
          greedy: true
        }
      });
      delete Prism.languages.kotlin['string'];
      Prism.languages.insertBefore('kotlin', 'keyword', {
        annotation: {
          pattern: /\B@(?:\w+:)?(?:[A-Z]\w*|\[[^\]]+\])/,
          alias: 'builtin'
        }
      });
      Prism.languages.insertBefore('kotlin', 'function', {
        label: {
          pattern: /\b\w+@|@\w+\b/,
          alias: 'symbol'
        }
      });
      Prism.languages.kt = Prism.languages.kotlin;
      Prism.languages.kts = Prism.languages.kotlin;
    })(Prism);
  }

  // @ts-nocheck
  less.displayName = 'less';
  less.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function less(Prism) {
    Prism.register(css);
    /* FIXME :
  :extend() is not handled specifically : its highlighting is buggy.
  Mixin usage must be inside a ruleset to be highlighted.
  At-rules (e.g. import) containing interpolations are buggy.
  Detached rulesets are highlighted as at-rules.
  A comment before a mixin usage prevents the latter to be properly highlighted.
  */
    Prism.languages.less = Prism.languages.extend('css', {
      comment: [
        /\/\*[\s\S]*?\*\//,
        {
          pattern: /(^|[^\\])\/\/.*/,
          lookbehind: true
        }
      ],
      atrule: {
        pattern:
          /@[\w-](?:\((?:[^(){}]|\([^(){}]*\))*\)|[^(){};\s]|\s+(?!\s))*?(?=\s*\{)/,
        inside: {
          punctuation: /[:()]/
        }
      },
      // selectors and mixins are considered the same
      selector: {
        pattern:
          /(?:@\{[\w-]+\}|[^{};\s@])(?:@\{[\w-]+\}|\((?:[^(){}]|\([^(){}]*\))*\)|[^(){};@\s]|\s+(?!\s))*?(?=\s*\{)/,
        inside: {
          // mixin parameters
          variable: /@+[\w-]+/
        }
      },
      property: /(?:@\{[\w-]+\}|[\w-])+(?:\+_?)?(?=\s*:)/,
      operator: /[+\-*\/]/
    });
    Prism.languages.insertBefore('less', 'property', {
      variable: [
        // Variable declaration (the colon must be consumed!)
        {
          pattern: /@[\w-]+\s*:/,
          inside: {
            punctuation: /:/
          }
        }, // Variable usage
        /@@?[\w-]+/
      ],
      'mixin-usage': {
        pattern: /([{;]\s*)[.#](?!\d)[\w-].*?(?=[(;])/,
        lookbehind: true,
        alias: 'function'
      }
    });
  }

  // @ts-nocheck
  lua.displayName = 'lua';
  lua.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function lua(Prism) {
    Prism.languages.lua = {
      comment: /^#!.+|--(?:\[(=*)\[[\s\S]*?\]\1\]|.*)/m,
      // \z may be used to skip the following space
      string: {
        pattern:
          /(["'])(?:(?!\1)[^\\\r\n]|\\z(?:\r\n|\s)|\\(?:\r\n|[^z]))*\1|\[(=*)\[[\s\S]*?\]\2\]/,
        greedy: true
      },
      number:
        /\b0x[a-f\d]+(?:\.[a-f\d]*)?(?:p[+-]?\d+)?\b|\b\d+(?:\.\B|(?:\.\d*)?(?:e[+-]?\d+)?\b)|\B\.\d+(?:e[+-]?\d+)?\b/i,
      keyword:
        /\b(?:and|break|do|else|elseif|end|false|for|function|goto|if|in|local|nil|not|or|repeat|return|then|true|until|while)\b/,
      function: /(?!\d)\w+(?=\s*(?:[({]))/,
      operator: [
        /[-+*%^&|#]|\/\/?|<[<=]?|>[>=]?|[=~]=?/,
        {
          // Match ".." but don't break "..."
          pattern: /(^|[^.])\.\.(?!\.)/,
          lookbehind: true
        }
      ],
      punctuation: /[\[\](){},;]|\.+|:+/
    };
  }

  // @ts-nocheck
  makefile.displayName = 'makefile';
  makefile.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function makefile(Prism) {
    Prism.languages.makefile = {
      comment: {
        pattern: /(^|[^\\])#(?:\\(?:\r\n|[\s\S])|[^\\\r\n])*/,
        lookbehind: true
      },
      string: {
        pattern: /(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,
        greedy: true
      },
      'builtin-target': {
        pattern: /\.[A-Z][^:#=\s]+(?=\s*:(?!=))/,
        alias: 'builtin'
      },
      target: {
        pattern: /^(?:[^:=\s]|[ \t]+(?![\s:]))+(?=\s*:(?!=))/m,
        alias: 'symbol',
        inside: {
          variable: /\$+(?:(?!\$)[^(){}:#=\s]+|(?=[({]))/
        }
      },
      variable: /\$+(?:(?!\$)[^(){}:#=\s]+|\([@*%<^+?][DF]\)|(?=[({]))/,
      // Directives
      keyword:
        /-include\b|\b(?:define|else|endef|endif|export|ifn?def|ifn?eq|include|override|private|sinclude|undefine|unexport|vpath)\b/,
      function: {
        pattern:
          /(\()(?:abspath|addsuffix|and|basename|call|dir|error|eval|file|filter(?:-out)?|findstring|firstword|flavor|foreach|guile|if|info|join|lastword|load|notdir|or|origin|patsubst|realpath|shell|sort|strip|subst|suffix|value|warning|wildcard|word(?:list|s)?)(?=[ \t])/,
        lookbehind: true
      },
      operator: /(?:::|[?:+!])?=|[|@]/,
      punctuation: /[:;(){}]/
    };
  }

  // @ts-nocheck
  yaml.displayName = 'yaml';
  yaml.aliases = ['yml'];

  /** @type {import('../core.js').Syntax} */
  function yaml(Prism) {
  (function (Prism) {
      // https://yaml.org/spec/1.2/spec.html#c-ns-anchor-property
      // https://yaml.org/spec/1.2/spec.html#c-ns-alias-node
      var anchorOrAlias = /[*&][^\s[\]{},]+/; // https://yaml.org/spec/1.2/spec.html#c-ns-tag-property
      var tag =
        /!(?:<[\w\-%#;/?:@&=+$,.!~*'()[\]]+>|(?:[a-zA-Z\d-]*!)?[\w\-%#;/?:@&=+$.~*'()]+)?/; // https://yaml.org/spec/1.2/spec.html#c-ns-properties(n,c)
      var properties =
        '(?:' +
        tag.source +
        '(?:[ \t]+' +
        anchorOrAlias.source +
        ')?|' +
        anchorOrAlias.source +
        '(?:[ \t]+' +
        tag.source +
        ')?)'; // https://yaml.org/spec/1.2/spec.html#ns-plain(n,c)
      // This is a simplified version that doesn't support "#" and multiline keys
      // All these long scarry character classes are simplified versions of YAML's characters
      var plainKey =
        /(?:[^\s\x00-\x08\x0e-\x1f!"#%&'*,\-:>?@[\]`{|}\x7f-\x84\x86-\x9f\ud800-\udfff\ufffe\uffff]|[?:-]<PLAIN>)(?:[ \t]*(?:(?![#:])<PLAIN>|:<PLAIN>))*/.source.replace(
          /<PLAIN>/g,
          function () {
            return /[^\s\x00-\x08\x0e-\x1f,[\]{}\x7f-\x84\x86-\x9f\ud800-\udfff\ufffe\uffff]/
              .source
          }
        );
      var string = /"(?:[^"\\\r\n]|\\.)*"|'(?:[^'\\\r\n]|\\.)*'/.source;
      /**
       *
       * @param {string} value
       * @param {string} [flags]
       * @returns {RegExp}
       */
      function createValuePattern(value, flags) {
        flags = (flags || '').replace(/m/g, '') + 'm'; // add m flag
        var pattern =
          /([:\-,[{]\s*(?:\s<<prop>>[ \t]+)?)(?:<<value>>)(?=[ \t]*(?:$|,|\]|\}|(?:[\r\n]\s*)?#))/.source
            .replace(/<<prop>>/g, function () {
              return properties
            })
            .replace(/<<value>>/g, function () {
              return value
            });
        return RegExp(pattern, flags)
      }
      Prism.languages.yaml = {
        scalar: {
          pattern: RegExp(
            /([\-:]\s*(?:\s<<prop>>[ \t]+)?[|>])[ \t]*(?:((?:\r?\n|\r)[ \t]+)\S[^\r\n]*(?:\2[^\r\n]+)*)/.source.replace(
              /<<prop>>/g,
              function () {
                return properties
              }
            )
          ),
          lookbehind: true,
          alias: 'string'
        },
        comment: /#.*/,
        key: {
          pattern: RegExp(
            /((?:^|[:\-,[{\r\n?])[ \t]*(?:<<prop>>[ \t]+)?)<<key>>(?=\s*:\s)/.source
              .replace(/<<prop>>/g, function () {
                return properties
              })
              .replace(/<<key>>/g, function () {
                return '(?:' + plainKey + '|' + string + ')'
              })
          ),
          lookbehind: true,
          greedy: true,
          alias: 'atrule'
        },
        directive: {
          pattern: /(^[ \t]*)%.+/m,
          lookbehind: true,
          alias: 'important'
        },
        datetime: {
          pattern: createValuePattern(
            /\d{4}-\d\d?-\d\d?(?:[tT]|[ \t]+)\d\d?:\d{2}:\d{2}(?:\.\d*)?(?:[ \t]*(?:Z|[-+]\d\d?(?::\d{2})?))?|\d{4}-\d{2}-\d{2}|\d\d?:\d{2}(?::\d{2}(?:\.\d*)?)?/
              .source
          ),
          lookbehind: true,
          alias: 'number'
        },
        boolean: {
          pattern: createValuePattern(/false|true/.source, 'i'),
          lookbehind: true,
          alias: 'important'
        },
        null: {
          pattern: createValuePattern(/null|~/.source, 'i'),
          lookbehind: true,
          alias: 'important'
        },
        string: {
          pattern: createValuePattern(string),
          lookbehind: true,
          greedy: true
        },
        number: {
          pattern: createValuePattern(
            /[+-]?(?:0x[\da-f]+|0o[0-7]+|(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?|\.inf|\.nan)/
              .source,
            'i'
          ),
          lookbehind: true
        },
        tag: tag,
        important: anchorOrAlias,
        punctuation: /---|[:[\]{}\-,|>?]|\.\.\./
      };
      Prism.languages.yml = Prism.languages.yaml;
    })(Prism);
  }

  // @ts-nocheck
  markdown$1.displayName = 'markdown';
  markdown$1.aliases = ['md'];

  /** @type {import('../core.js').Syntax} */
  function markdown$1(Prism) {
    Prism.register(markup)
    ;(function (Prism) {
      // Allow only one line break
      var inner = /(?:\\.|[^\\\n\r]|(?:\n|\r\n?)(?![\r\n]))/.source;
      /**
       * This function is intended for the creation of the bold or italic pattern.
       *
       * This also adds a lookbehind group to the given pattern to ensure that the pattern is not backslash-escaped.
       *
       * _Note:_ Keep in mind that this adds a capturing group.
       *
       * @param {string} pattern
       * @returns {RegExp}
       */
      function createInline(pattern) {
        pattern = pattern.replace(/<inner>/g, function () {
          return inner
        });
        return RegExp(/((?:^|[^\\])(?:\\{2})*)/.source + '(?:' + pattern + ')')
      }
      var tableCell = /(?:\\.|``(?:[^`\r\n]|`(?!`))+``|`[^`\r\n]+`|[^\\|\r\n`])+/
        .source;
      var tableRow =
        /\|?__(?:\|__)+\|?(?:(?:\n|\r\n?)|(?![\s\S]))/.source.replace(
          /__/g,
          function () {
            return tableCell
          }
        );
      var tableLine =
        /\|?[ \t]*:?-{3,}:?[ \t]*(?:\|[ \t]*:?-{3,}:?[ \t]*)+\|?(?:\n|\r\n?)/
          .source;
      Prism.languages.markdown = Prism.languages.extend('markup', {});
      Prism.languages.insertBefore('markdown', 'prolog', {
        'front-matter-block': {
          pattern: /(^(?:\s*[\r\n])?)---(?!.)[\s\S]*?[\r\n]---(?!.)/,
          lookbehind: true,
          greedy: true,
          inside: {
            punctuation: /^---|---$/,
            'front-matter': {
              pattern: /\S+(?:\s+\S+)*/,
              alias: ['yaml', 'language-yaml'],
              inside: Prism.languages.yaml
            }
          }
        },
        blockquote: {
          // > ...
          pattern: /^>(?:[\t ]*>)*/m,
          alias: 'punctuation'
        },
        table: {
          pattern: RegExp(
            '^' + tableRow + tableLine + '(?:' + tableRow + ')*',
            'm'
          ),
          inside: {
            'table-data-rows': {
              pattern: RegExp(
                '^(' + tableRow + tableLine + ')(?:' + tableRow + ')*$'
              ),
              lookbehind: true,
              inside: {
                'table-data': {
                  pattern: RegExp(tableCell),
                  inside: Prism.languages.markdown
                },
                punctuation: /\|/
              }
            },
            'table-line': {
              pattern: RegExp('^(' + tableRow + ')' + tableLine + '$'),
              lookbehind: true,
              inside: {
                punctuation: /\||:?-{3,}:?/
              }
            },
            'table-header-row': {
              pattern: RegExp('^' + tableRow + '$'),
              inside: {
                'table-header': {
                  pattern: RegExp(tableCell),
                  alias: 'important',
                  inside: Prism.languages.markdown
                },
                punctuation: /\|/
              }
            }
          }
        },
        code: [
          {
            // Prefixed by 4 spaces or 1 tab and preceded by an empty line
            pattern:
              /((?:^|\n)[ \t]*\n|(?:^|\r\n?)[ \t]*\r\n?)(?: {4}|\t).+(?:(?:\n|\r\n?)(?: {4}|\t).+)*/,
            lookbehind: true,
            alias: 'keyword'
          },
          {
            // ```optional language
            // code block
            // ```
            pattern: /^```[\s\S]*?^```$/m,
            greedy: true,
            inside: {
              'code-block': {
                pattern: /^(```.*(?:\n|\r\n?))[\s\S]+?(?=(?:\n|\r\n?)^```$)/m,
                lookbehind: true
              },
              'code-language': {
                pattern: /^(```).+/,
                lookbehind: true
              },
              punctuation: /```/
            }
          }
        ],
        title: [
          {
            // title 1
            // =======
            // title 2
            // -------
            pattern: /\S.*(?:\n|\r\n?)(?:==+|--+)(?=[ \t]*$)/m,
            alias: 'important',
            inside: {
              punctuation: /==+$|--+$/
            }
          },
          {
            // # title 1
            // ###### title 6
            pattern: /(^\s*)#.+/m,
            lookbehind: true,
            alias: 'important',
            inside: {
              punctuation: /^#+|#+$/
            }
          }
        ],
        hr: {
          // ***
          // ---
          // * * *
          // -----------
          pattern: /(^\s*)([*-])(?:[\t ]*\2){2,}(?=\s*$)/m,
          lookbehind: true,
          alias: 'punctuation'
        },
        list: {
          // * item
          // + item
          // - item
          // 1. item
          pattern: /(^\s*)(?:[*+-]|\d+\.)(?=[\t ].)/m,
          lookbehind: true,
          alias: 'punctuation'
        },
        'url-reference': {
          // [id]: http://example.com "Optional title"
          // [id]: http://example.com 'Optional title'
          // [id]: http://example.com (Optional title)
          // [id]: <http://example.com> "Optional title"
          pattern:
            /!?\[[^\]]+\]:[\t ]+(?:\S+|<(?:\\.|[^>\\])+>)(?:[\t ]+(?:"(?:\\.|[^"\\])*"|'(?:\\.|[^'\\])*'|\((?:\\.|[^)\\])*\)))?/,
          inside: {
            variable: {
              pattern: /^(!?\[)[^\]]+/,
              lookbehind: true
            },
            string:
              /(?:"(?:\\.|[^"\\])*"|'(?:\\.|[^'\\])*'|\((?:\\.|[^)\\])*\))$/,
            punctuation: /^[\[\]!:]|[<>]/
          },
          alias: 'url'
        },
        bold: {
          // **strong**
          // __strong__
          // allow one nested instance of italic text using the same delimiter
          pattern: createInline(
            /\b__(?:(?!_)<inner>|_(?:(?!_)<inner>)+_)+__\b|\*\*(?:(?!\*)<inner>|\*(?:(?!\*)<inner>)+\*)+\*\*/
              .source
          ),
          lookbehind: true,
          greedy: true,
          inside: {
            content: {
              pattern: /(^..)[\s\S]+(?=..$)/,
              lookbehind: true,
              inside: {} // see below
            },
            punctuation: /\*\*|__/
          }
        },
        italic: {
          // *em*
          // _em_
          // allow one nested instance of bold text using the same delimiter
          pattern: createInline(
            /\b_(?:(?!_)<inner>|__(?:(?!_)<inner>)+__)+_\b|\*(?:(?!\*)<inner>|\*\*(?:(?!\*)<inner>)+\*\*)+\*/
              .source
          ),
          lookbehind: true,
          greedy: true,
          inside: {
            content: {
              pattern: /(^.)[\s\S]+(?=.$)/,
              lookbehind: true,
              inside: {} // see below
            },
            punctuation: /[*_]/
          }
        },
        strike: {
          // ~~strike through~~
          // ~strike~
          // eslint-disable-next-line regexp/strict
          pattern: createInline(/(~~?)(?:(?!~)<inner>)+\2/.source),
          lookbehind: true,
          greedy: true,
          inside: {
            content: {
              pattern: /(^~~?)[\s\S]+(?=\1$)/,
              lookbehind: true,
              inside: {} // see below
            },
            punctuation: /~~?/
          }
        },
        'code-snippet': {
          // `code`
          // ``code``
          pattern:
            /(^|[^\\`])(?:``[^`\r\n]+(?:`[^`\r\n]+)*``(?!`)|`[^`\r\n]+`(?!`))/,
          lookbehind: true,
          greedy: true,
          alias: ['code', 'keyword']
        },
        url: {
          // [example](http://example.com "Optional title")
          // [example][id]
          // [example] [id]
          pattern: createInline(
            /!?\[(?:(?!\])<inner>)+\](?:\([^\s)]+(?:[\t ]+"(?:\\.|[^"\\])*")?\)|[ \t]?\[(?:(?!\])<inner>)+\])/
              .source
          ),
          lookbehind: true,
          greedy: true,
          inside: {
            operator: /^!/,
            content: {
              pattern: /(^\[)[^\]]+(?=\])/,
              lookbehind: true,
              inside: {} // see below
            },
            variable: {
              pattern: /(^\][ \t]?\[)[^\]]+(?=\]$)/,
              lookbehind: true
            },
            url: {
              pattern: /(^\]\()[^\s)]+/,
              lookbehind: true
            },
            string: {
              pattern: /(^[ \t]+)"(?:\\.|[^"\\])*"(?=\)$)/,
              lookbehind: true
            }
          }
        }
      })
      ;['url', 'bold', 'italic', 'strike'].forEach(function (token) {
  ['url', 'bold', 'italic', 'strike', 'code-snippet'].forEach(function (
          inside
        ) {
          if (token !== inside) {
            Prism.languages.markdown[token].inside.content.inside[inside] =
              Prism.languages.markdown[inside];
          }
        });
      });
      Prism.hooks.add('after-tokenize', function (env) {
        if (env.language !== 'markdown' && env.language !== 'md') {
          return
        }
        function walkTokens(tokens) {
          if (!tokens || typeof tokens === 'string') {
            return
          }
          for (var i = 0, l = tokens.length; i < l; i++) {
            var token = tokens[i];
            if (token.type !== 'code') {
              walkTokens(token.content);
              continue
            }
            /*
             * Add the correct `language-xxxx` class to this code block. Keep in mind that the `code-language` token
             * is optional. But the grammar is defined so that there is only one case we have to handle:
             *
             * token.content = [
             *     <span class="punctuation">```</span>,
             *     <span class="code-language">xxxx</span>,
             *     '\n', // exactly one new lines (\r or \n or \r\n)
             *     <span class="code-block">...</span>,
             *     '\n', // exactly one new lines again
             *     <span class="punctuation">```</span>
             * ];
             */
            var codeLang = token.content[1];
            var codeBlock = token.content[3];
            if (
              codeLang &&
              codeBlock &&
              codeLang.type === 'code-language' &&
              codeBlock.type === 'code-block' &&
              typeof codeLang.content === 'string'
            ) {
              // this might be a language that Prism does not support
              // do some replacements to support C++, C#, and F#
              var lang = codeLang.content
                .replace(/\b#/g, 'sharp')
                .replace(/\b\+\+/g, 'pp'); // only use the first word
              lang = (/[a-z][\w-]*/i.exec(lang) || [''])[0].toLowerCase();
              var alias = 'language-' + lang; // add alias
              if (!codeBlock.alias) {
                codeBlock.alias = [alias];
              } else if (typeof codeBlock.alias === 'string') {
                codeBlock.alias = [codeBlock.alias, alias];
              } else {
                codeBlock.alias.push(alias);
              }
            }
          }
        }
        walkTokens(env.tokens);
      });
      Prism.hooks.add('wrap', function (env) {
        if (env.type !== 'code-block') {
          return
        }
        var codeLang = '';
        for (var i = 0, l = env.classes.length; i < l; i++) {
          var cls = env.classes[i];
          var match = /language-(.+)/.exec(cls);
          if (match) {
            codeLang = match[1];
            break
          }
        }
        var grammar = Prism.languages[codeLang];
        if (!grammar) {
          if (codeLang && codeLang !== 'none' && Prism.plugins.autoloader) {
            var id =
              'md-' +
              new Date().valueOf() +
              '-' +
              Math.floor(Math.random() * 1e16);
            env.attributes['id'] = id;
            Prism.plugins.autoloader.loadLanguages(codeLang, function () {
              var ele = document.getElementById(id);
              if (ele) {
                ele.innerHTML = Prism.highlight(
                  ele.textContent,
                  Prism.languages[codeLang],
                  codeLang
                );
              }
            });
          }
        } else {
          env.content = Prism.highlight(
            textContent(env.content.value),
            grammar,
            codeLang
          );
        }
      });
      var tagPattern = RegExp(Prism.languages.markup.tag.pattern.source, 'gi');
      /**
       * A list of known entity names.
       *
       * This will always be incomplete to save space. The current list is the one used by lowdash's unescape function.
       *
       * @see {@link https://github.com/lodash/lodash/blob/2da024c3b4f9947a48517639de7560457cd4ec6c/unescape.js#L2}
       */
      var KNOWN_ENTITY_NAMES = {
        amp: '&',
        lt: '<',
        gt: '>',
        quot: '"'
      }; // IE 11 doesn't support `String.fromCodePoint`
      var fromCodePoint = String.fromCodePoint || String.fromCharCode;
      /**
       * Returns the text content of a given HTML source code string.
       *
       * @param {string} html
       * @returns {string}
       */
      function textContent(html) {
        // remove all tags
        var text = html.replace(tagPattern, ''); // decode known entities
        text = text.replace(/&(\w{1,8}|#x?[\da-f]{1,8});/gi, function (m, code) {
          code = code.toLowerCase();
          if (code[0] === '#') {
            var value;
            if (code[1] === 'x') {
              value = parseInt(code.slice(2), 16);
            } else {
              value = Number(code.slice(1));
            }
            return fromCodePoint(value)
          } else {
            var known = KNOWN_ENTITY_NAMES[code];
            if (known) {
              return known
            } // unable to decode
            return m
          }
        });
        return text
      }
      Prism.languages.md = Prism.languages.markdown;
    })(Prism);
  }

  // @ts-nocheck
  objectivec.displayName = 'objectivec';
  objectivec.aliases = ['objc'];

  /** @type {import('../core.js').Syntax} */
  function objectivec(Prism) {
    Prism.register(c$1);
    Prism.languages.objectivec = Prism.languages.extend('c', {
      string: {
        pattern: /@?"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"/,
        greedy: true
      },
      keyword:
        /\b(?:asm|auto|break|case|char|const|continue|default|do|double|else|enum|extern|float|for|goto|if|in|inline|int|long|register|return|self|short|signed|sizeof|static|struct|super|switch|typedef|typeof|union|unsigned|void|volatile|while)\b|(?:@interface|@end|@implementation|@protocol|@class|@public|@protected|@private|@property|@try|@catch|@finally|@throw|@synthesize|@dynamic|@selector)\b/,
      operator: /-[->]?|\+\+?|!=?|<<?=?|>>?=?|==?|&&?|\|\|?|[~^%?*\/@]/
    });
    delete Prism.languages.objectivec['class-name'];
    Prism.languages.objc = Prism.languages.objectivec;
  }

  // @ts-nocheck
  perl.displayName = 'perl';
  perl.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function perl(Prism) {
  (function (Prism) {
      var brackets =
        /(?:\((?:[^()\\]|\\[\s\S])*\)|\{(?:[^{}\\]|\\[\s\S])*\}|\[(?:[^[\]\\]|\\[\s\S])*\]|<(?:[^<>\\]|\\[\s\S])*>)/
          .source;
      Prism.languages.perl = {
        comment: [
          {
            // POD
            pattern: /(^\s*)=\w[\s\S]*?=cut.*/m,
            lookbehind: true,
            greedy: true
          },
          {
            pattern: /(^|[^\\$])#.*/,
            lookbehind: true,
            greedy: true
          }
        ],
        // TODO Could be nice to handle Heredoc too.
        string: [
          {
            pattern: RegExp(
              /\b(?:q|qq|qw|qx)(?![a-zA-Z0-9])\s*/.source +
                '(?:' +
                [
                  // q/.../
                  /([^a-zA-Z0-9\s{(\[<])(?:(?!\1)[^\\]|\\[\s\S])*\1/.source, // q a...a
                  // eslint-disable-next-line regexp/strict
                  /([a-zA-Z0-9])(?:(?!\2)[^\\]|\\[\s\S])*\2/.source, // q(...)
                  // q{...}
                  // q[...]
                  // q<...>
                  brackets
                ].join('|') +
                ')'
            ),
            greedy: true
          }, // "...", `...`
          {
            pattern: /("|`)(?:(?!\1)[^\\]|\\[\s\S])*\1/,
            greedy: true
          }, // '...'
          // FIXME Multi-line single-quoted strings are not supported as they would break variables containing '
          {
            pattern: /'(?:[^'\\\r\n]|\\.)*'/,
            greedy: true
          }
        ],
        regex: [
          {
            pattern: RegExp(
              /\b(?:m|qr)(?![a-zA-Z0-9])\s*/.source +
                '(?:' +
                [
                  // m/.../
                  /([^a-zA-Z0-9\s{(\[<])(?:(?!\1)[^\\]|\\[\s\S])*\1/.source, // m a...a
                  // eslint-disable-next-line regexp/strict
                  /([a-zA-Z0-9])(?:(?!\2)[^\\]|\\[\s\S])*\2/.source, // m(...)
                  // m{...}
                  // m[...]
                  // m<...>
                  brackets
                ].join('|') +
                ')' +
                /[msixpodualngc]*/.source
            ),
            greedy: true
          }, // The lookbehinds prevent -s from breaking
          {
            pattern: RegExp(
              /(^|[^-])\b(?:s|tr|y)(?![a-zA-Z0-9])\s*/.source +
                '(?:' +
                [
                  // s/.../.../
                  // eslint-disable-next-line regexp/strict
                  /([^a-zA-Z0-9\s{(\[<])(?:(?!\2)[^\\]|\\[\s\S])*\2(?:(?!\2)[^\\]|\\[\s\S])*\2/
                    .source, // s a...a...a
                  // eslint-disable-next-line regexp/strict
                  /([a-zA-Z0-9])(?:(?!\3)[^\\]|\\[\s\S])*\3(?:(?!\3)[^\\]|\\[\s\S])*\3/
                    .source, // s(...)(...)
                  // s{...}{...}
                  // s[...][...]
                  // s<...><...>
                  // s(...)[...]
                  brackets + /\s*/.source + brackets
                ].join('|') +
                ')' +
                /[msixpodualngcer]*/.source
            ),
            lookbehind: true,
            greedy: true
          }, // /.../
          // The look-ahead tries to prevent two divisions on
          // the same line from being highlighted as regex.
          // This does not support multi-line regex.
          {
            pattern:
              /\/(?:[^\/\\\r\n]|\\.)*\/[msixpodualngc]*(?=\s*(?:$|[\r\n,.;})&|\-+*~<>!?^]|(?:and|cmp|eq|ge|gt|le|lt|ne|not|or|x|xor)\b))/,
            greedy: true
          }
        ],
        // FIXME Not sure about the handling of ::, ', and #
        variable: [
          // ${^POSTMATCH}
          /[&*$@%]\{\^[A-Z]+\}/, // $^V
          /[&*$@%]\^[A-Z_]/, // ${...}
          /[&*$@%]#?(?=\{)/, // $foo
          /[&*$@%]#?(?:(?:::)*'?(?!\d)[\w$]+(?![\w$]))+(?:::)*/, // $1
          /[&*$@%]\d+/, // $_, @_, %!
          // The negative lookahead prevents from breaking the %= operator
          /(?!%=)[$@%][!"#$%&'()*+,\-.\/:;<=>?@[\\\]^_`{|}~]/
        ],
        filehandle: {
          // <>, <FOO>, _
          pattern: /<(?![<=])\S*?>|\b_\b/,
          alias: 'symbol'
        },
        'v-string': {
          // v1.2, 1.2.3
          pattern: /v\d+(?:\.\d+)*|\d+(?:\.\d+){2,}/,
          alias: 'string'
        },
        function: {
          pattern: /(\bsub[ \t]+)\w+/,
          lookbehind: true
        },
        keyword:
          /\b(?:any|break|continue|default|delete|die|do|else|elsif|eval|for|foreach|given|goto|if|last|local|my|next|our|package|print|redo|require|return|say|state|sub|switch|undef|unless|until|use|when|while)\b/,
        number:
          /\b(?:0x[\dA-Fa-f](?:_?[\dA-Fa-f])*|0b[01](?:_?[01])*|(?:(?:\d(?:_?\d)*)?\.)?\d(?:_?\d)*(?:[Ee][+-]?\d+)?)\b/,
        operator:
          /-[rwxoRWXOezsfdlpSbctugkTBMAC]\b|\+[+=]?|-[-=>]?|\*\*?=?|\/\/?=?|=[=~>]?|~[~=]?|\|\|?=?|&&?=?|<(?:=>?|<=?)?|>>?=?|![~=]?|[%^]=?|\.(?:=|\.\.?)?|[\\?]|\bx(?:=|\b)|\b(?:and|cmp|eq|ge|gt|le|lt|ne|not|or|xor)\b/,
        punctuation: /[{}[\];(),:]/
      };
    })(Prism);
  }

  // @ts-nocheck
  markupTemplating.displayName = 'markup-templating';
  markupTemplating.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function markupTemplating(Prism) {
    Prism.register(markup)
    ;(function (Prism) {
      /**
       * Returns the placeholder for the given language id and index.
       *
       * @param {string} language
       * @param {string|number} index
       * @returns {string}
       */
      function getPlaceholder(language, index) {
        return '___' + language.toUpperCase() + index + '___'
      }
      Object.defineProperties((Prism.languages['markup-templating'] = {}), {
        buildPlaceholders: {
          /**
           * Tokenize all inline templating expressions matching `placeholderPattern`.
           *
           * If `replaceFilter` is provided, only matches of `placeholderPattern` for which `replaceFilter` returns
           * `true` will be replaced.
           *
           * @param {object} env The environment of the `before-tokenize` hook.
           * @param {string} language The language id.
           * @param {RegExp} placeholderPattern The matches of this pattern will be replaced by placeholders.
           * @param {(match: string) => boolean} [replaceFilter]
           */
          value: function (env, language, placeholderPattern, replaceFilter) {
            if (env.language !== language) {
              return
            }
            var tokenStack = (env.tokenStack = []);
            env.code = env.code.replace(placeholderPattern, function (match) {
              if (typeof replaceFilter === 'function' && !replaceFilter(match)) {
                return match
              }
              var i = tokenStack.length;
              var placeholder; // Check for existing strings
              while (
                env.code.indexOf((placeholder = getPlaceholder(language, i))) !==
                -1
              ) {
                ++i;
              } // Create a sparse array
              tokenStack[i] = match;
              return placeholder
            }); // Switch the grammar to markup
            env.grammar = Prism.languages.markup;
          }
        },
        tokenizePlaceholders: {
          /**
           * Replace placeholders with proper tokens after tokenizing.
           *
           * @param {object} env The environment of the `after-tokenize` hook.
           * @param {string} language The language id.
           */
          value: function (env, language) {
            if (env.language !== language || !env.tokenStack) {
              return
            } // Switch the grammar back
            env.grammar = Prism.languages[language];
            var j = 0;
            var keys = Object.keys(env.tokenStack);
            function walkTokens(tokens) {
              for (var i = 0; i < tokens.length; i++) {
                // all placeholders are replaced already
                if (j >= keys.length) {
                  break
                }
                var token = tokens[i];
                if (
                  typeof token === 'string' ||
                  (token.content && typeof token.content === 'string')
                ) {
                  var k = keys[j];
                  var t = env.tokenStack[k];
                  var s = typeof token === 'string' ? token : token.content;
                  var placeholder = getPlaceholder(language, k);
                  var index = s.indexOf(placeholder);
                  if (index > -1) {
                    ++j;
                    var before = s.substring(0, index);
                    var middle = new Prism.Token(
                      language,
                      Prism.tokenize(t, env.grammar),
                      'language-' + language,
                      t
                    );
                    var after = s.substring(index + placeholder.length);
                    var replacement = [];
                    if (before) {
                      replacement.push.apply(replacement, walkTokens([before]));
                    }
                    replacement.push(middle);
                    if (after) {
                      replacement.push.apply(replacement, walkTokens([after]));
                    }
                    if (typeof token === 'string') {
                      tokens.splice.apply(tokens, [i, 1].concat(replacement));
                    } else {
                      token.content = replacement;
                    }
                  }
                } else if (
                  token.content
                  /* && typeof token.content !== 'string' */
                ) {
                  walkTokens(token.content);
                }
              }
              return tokens
            }
            walkTokens(env.tokens);
          }
        }
      });
    })(Prism);
  }

  // @ts-nocheck
  php.displayName = 'php';
  php.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function php(Prism) {
    Prism.register(markupTemplating)
    /**
     * Original by Aaron Harun: http://aahacreative.com/2012/07/31/php-syntax-highlighting-prism/
     * Modified by Miles Johnson: http://milesj.me
     * Rewritten by Tom Pavelec
     *
     * Supports PHP 5.3 - 8.0
     */
    ;(function (Prism) {
      var comment = /\/\*[\s\S]*?\*\/|\/\/.*|#(?!\[).*/;
      var constant = [
        {
          pattern: /\b(?:false|true)\b/i,
          alias: 'boolean'
        },
        {
          pattern: /(::\s*)\b[a-z_]\w*\b(?!\s*\()/i,
          greedy: true,
          lookbehind: true
        },
        {
          pattern: /(\b(?:case|const)\s+)\b[a-z_]\w*(?=\s*[;=])/i,
          greedy: true,
          lookbehind: true
        },
        /\b(?:null)\b/i,
        /\b[A-Z_][A-Z0-9_]*\b(?!\s*\()/
      ];
      var number =
        /\b0b[01]+(?:_[01]+)*\b|\b0o[0-7]+(?:_[0-7]+)*\b|\b0x[\da-f]+(?:_[\da-f]+)*\b|(?:\b\d+(?:_\d+)*\.?(?:\d+(?:_\d+)*)?|\B\.\d+)(?:e[+-]?\d+)?/i;
      var operator =
        /<?=>|\?\?=?|\.{3}|\??->|[!=]=?=?|::|\*\*=?|--|\+\+|&&|\|\||<<|>>|[?~]|[/^|%*&<>.+-]=?/;
      var punctuation = /[{}\[\](),:;]/;
      Prism.languages.php = {
        delimiter: {
          pattern: /\?>$|^<\?(?:php(?=\s)|=)?/i,
          alias: 'important'
        },
        comment: comment,
        variable: /\$+(?:\w+\b|(?=\{))/,
        package: {
          pattern:
            /(namespace\s+|use\s+(?:function\s+)?)(?:\\?\b[a-z_]\w*)+\b(?!\\)/i,
          lookbehind: true,
          inside: {
            punctuation: /\\/
          }
        },
        'class-name-definition': {
          pattern: /(\b(?:class|enum|interface|trait)\s+)\b[a-z_]\w*(?!\\)\b/i,
          lookbehind: true,
          alias: 'class-name'
        },
        'function-definition': {
          pattern: /(\bfunction\s+)[a-z_]\w*(?=\s*\()/i,
          lookbehind: true,
          alias: 'function'
        },
        keyword: [
          {
            pattern:
              /(\(\s*)\b(?:array|bool|boolean|float|int|integer|object|string)\b(?=\s*\))/i,
            alias: 'type-casting',
            greedy: true,
            lookbehind: true
          },
          {
            pattern:
              /([(,?]\s*)\b(?:array(?!\s*\()|bool|callable|(?:false|null)(?=\s*\|)|float|int|iterable|mixed|object|self|static|string)\b(?=\s*\$)/i,
            alias: 'type-hint',
            greedy: true,
            lookbehind: true
          },
          {
            pattern:
              /(\)\s*:\s*(?:\?\s*)?)\b(?:array(?!\s*\()|bool|callable|(?:false|null)(?=\s*\|)|float|int|iterable|mixed|object|self|static|string|void)\b/i,
            alias: 'return-type',
            greedy: true,
            lookbehind: true
          },
          {
            pattern:
              /\b(?:array(?!\s*\()|bool|float|int|iterable|mixed|object|string|void)\b/i,
            alias: 'type-declaration',
            greedy: true
          },
          {
            pattern: /(\|\s*)(?:false|null)\b|\b(?:false|null)(?=\s*\|)/i,
            alias: 'type-declaration',
            greedy: true,
            lookbehind: true
          },
          {
            pattern: /\b(?:parent|self|static)(?=\s*::)/i,
            alias: 'static-context',
            greedy: true
          },
          {
            // yield from
            pattern: /(\byield\s+)from\b/i,
            lookbehind: true
          }, // `class` is always a keyword unlike other keywords
          /\bclass\b/i,
          {
            // https://www.php.net/manual/en/reserved.keywords.php
            //
            // keywords cannot be preceded by "->"
            // the complex lookbehind means `(?<!(?:->|::)\s*)`
            pattern:
              /((?:^|[^\s>:]|(?:^|[^-])>|(?:^|[^:]):)\s*)\b(?:abstract|and|array|as|break|callable|case|catch|clone|const|continue|declare|default|die|do|echo|else|elseif|empty|enddeclare|endfor|endforeach|endif|endswitch|endwhile|enum|eval|exit|extends|final|finally|fn|for|foreach|function|global|goto|if|implements|include|include_once|instanceof|insteadof|interface|isset|list|match|namespace|new|or|parent|print|private|protected|public|require|require_once|return|self|static|switch|throw|trait|try|unset|use|var|while|xor|yield|__halt_compiler)\b/i,
            lookbehind: true
          }
        ],
        'argument-name': {
          pattern: /([(,]\s+)\b[a-z_]\w*(?=\s*:(?!:))/i,
          lookbehind: true
        },
        'class-name': [
          {
            pattern:
              /(\b(?:extends|implements|instanceof|new(?!\s+self|\s+static))\s+|\bcatch\s*\()\b[a-z_]\w*(?!\\)\b/i,
            greedy: true,
            lookbehind: true
          },
          {
            pattern: /(\|\s*)\b[a-z_]\w*(?!\\)\b/i,
            greedy: true,
            lookbehind: true
          },
          {
            pattern: /\b[a-z_]\w*(?!\\)\b(?=\s*\|)/i,
            greedy: true
          },
          {
            pattern: /(\|\s*)(?:\\?\b[a-z_]\w*)+\b/i,
            alias: 'class-name-fully-qualified',
            greedy: true,
            lookbehind: true,
            inside: {
              punctuation: /\\/
            }
          },
          {
            pattern: /(?:\\?\b[a-z_]\w*)+\b(?=\s*\|)/i,
            alias: 'class-name-fully-qualified',
            greedy: true,
            inside: {
              punctuation: /\\/
            }
          },
          {
            pattern:
              /(\b(?:extends|implements|instanceof|new(?!\s+self\b|\s+static\b))\s+|\bcatch\s*\()(?:\\?\b[a-z_]\w*)+\b(?!\\)/i,
            alias: 'class-name-fully-qualified',
            greedy: true,
            lookbehind: true,
            inside: {
              punctuation: /\\/
            }
          },
          {
            pattern: /\b[a-z_]\w*(?=\s*\$)/i,
            alias: 'type-declaration',
            greedy: true
          },
          {
            pattern: /(?:\\?\b[a-z_]\w*)+(?=\s*\$)/i,
            alias: ['class-name-fully-qualified', 'type-declaration'],
            greedy: true,
            inside: {
              punctuation: /\\/
            }
          },
          {
            pattern: /\b[a-z_]\w*(?=\s*::)/i,
            alias: 'static-context',
            greedy: true
          },
          {
            pattern: /(?:\\?\b[a-z_]\w*)+(?=\s*::)/i,
            alias: ['class-name-fully-qualified', 'static-context'],
            greedy: true,
            inside: {
              punctuation: /\\/
            }
          },
          {
            pattern: /([(,?]\s*)[a-z_]\w*(?=\s*\$)/i,
            alias: 'type-hint',
            greedy: true,
            lookbehind: true
          },
          {
            pattern: /([(,?]\s*)(?:\\?\b[a-z_]\w*)+(?=\s*\$)/i,
            alias: ['class-name-fully-qualified', 'type-hint'],
            greedy: true,
            lookbehind: true,
            inside: {
              punctuation: /\\/
            }
          },
          {
            pattern: /(\)\s*:\s*(?:\?\s*)?)\b[a-z_]\w*(?!\\)\b/i,
            alias: 'return-type',
            greedy: true,
            lookbehind: true
          },
          {
            pattern: /(\)\s*:\s*(?:\?\s*)?)(?:\\?\b[a-z_]\w*)+\b(?!\\)/i,
            alias: ['class-name-fully-qualified', 'return-type'],
            greedy: true,
            lookbehind: true,
            inside: {
              punctuation: /\\/
            }
          }
        ],
        constant: constant,
        function: {
          pattern: /(^|[^\\\w])\\?[a-z_](?:[\w\\]*\w)?(?=\s*\()/i,
          lookbehind: true,
          inside: {
            punctuation: /\\/
          }
        },
        property: {
          pattern: /(->\s*)\w+/,
          lookbehind: true
        },
        number: number,
        operator: operator,
        punctuation: punctuation
      };
      var string_interpolation = {
        pattern:
          /\{\$(?:\{(?:\{[^{}]+\}|[^{}]+)\}|[^{}])+\}|(^|[^\\{])\$+(?:\w+(?:\[[^\r\n\[\]]+\]|->\w+)?)/,
        lookbehind: true,
        inside: Prism.languages.php
      };
      var string = [
        {
          pattern: /<<<'([^']+)'[\r\n](?:.*[\r\n])*?\1;/,
          alias: 'nowdoc-string',
          greedy: true,
          inside: {
            delimiter: {
              pattern: /^<<<'[^']+'|[a-z_]\w*;$/i,
              alias: 'symbol',
              inside: {
                punctuation: /^<<<'?|[';]$/
              }
            }
          }
        },
        {
          pattern:
            /<<<(?:"([^"]+)"[\r\n](?:.*[\r\n])*?\1;|([a-z_]\w*)[\r\n](?:.*[\r\n])*?\2;)/i,
          alias: 'heredoc-string',
          greedy: true,
          inside: {
            delimiter: {
              pattern: /^<<<(?:"[^"]+"|[a-z_]\w*)|[a-z_]\w*;$/i,
              alias: 'symbol',
              inside: {
                punctuation: /^<<<"?|[";]$/
              }
            },
            interpolation: string_interpolation
          }
        },
        {
          pattern: /`(?:\\[\s\S]|[^\\`])*`/,
          alias: 'backtick-quoted-string',
          greedy: true
        },
        {
          pattern: /'(?:\\[\s\S]|[^\\'])*'/,
          alias: 'single-quoted-string',
          greedy: true
        },
        {
          pattern: /"(?:\\[\s\S]|[^\\"])*"/,
          alias: 'double-quoted-string',
          greedy: true,
          inside: {
            interpolation: string_interpolation
          }
        }
      ];
      Prism.languages.insertBefore('php', 'variable', {
        string: string,
        attribute: {
          pattern:
            /#\[(?:[^"'\/#]|\/(?![*/])|\/\/.*$|#(?!\[).*$|\/\*(?:[^*]|\*(?!\/))*\*\/|"(?:\\[\s\S]|[^\\"])*"|'(?:\\[\s\S]|[^\\'])*')+\](?=\s*[a-z$#])/im,
          greedy: true,
          inside: {
            'attribute-content': {
              pattern: /^(#\[)[\s\S]+(?=\]$)/,
              lookbehind: true,
              // inside can appear subset of php
              inside: {
                comment: comment,
                string: string,
                'attribute-class-name': [
                  {
                    pattern: /([^:]|^)\b[a-z_]\w*(?!\\)\b/i,
                    alias: 'class-name',
                    greedy: true,
                    lookbehind: true
                  },
                  {
                    pattern: /([^:]|^)(?:\\?\b[a-z_]\w*)+/i,
                    alias: ['class-name', 'class-name-fully-qualified'],
                    greedy: true,
                    lookbehind: true,
                    inside: {
                      punctuation: /\\/
                    }
                  }
                ],
                constant: constant,
                number: number,
                operator: operator,
                punctuation: punctuation
              }
            },
            delimiter: {
              pattern: /^#\[|\]$/,
              alias: 'punctuation'
            }
          }
        }
      });
      Prism.hooks.add('before-tokenize', function (env) {
        if (!/<\?/.test(env.code)) {
          return
        }
        var phpPattern =
          /<\?(?:[^"'/#]|\/(?![*/])|("|')(?:\\[\s\S]|(?!\1)[^\\])*\1|(?:\/\/|#(?!\[))(?:[^?\n\r]|\?(?!>))*(?=$|\?>|[\r\n])|#\[|\/\*(?:[^*]|\*(?!\/))*(?:\*\/|$))*?(?:\?>|$)/g;
        Prism.languages['markup-templating'].buildPlaceholders(
          env,
          'php',
          phpPattern
        );
      });
      Prism.hooks.add('after-tokenize', function (env) {
        Prism.languages['markup-templating'].tokenizePlaceholders(env, 'php');
      });
    })(Prism);
  }

  // @ts-nocheck
  python.displayName = 'python';
  python.aliases = ['py'];

  /** @type {import('../core.js').Syntax} */
  function python(Prism) {
    Prism.languages.python = {
      comment: {
        pattern: /(^|[^\\])#.*/,
        lookbehind: true,
        greedy: true
      },
      'string-interpolation': {
        pattern:
          /(?:f|fr|rf)(?:("""|''')[\s\S]*?\1|("|')(?:\\.|(?!\2)[^\\\r\n])*\2)/i,
        greedy: true,
        inside: {
          interpolation: {
            // "{" <expression> <optional "!s", "!r", or "!a"> <optional ":" format specifier> "}"
            pattern:
              /((?:^|[^{])(?:\{\{)*)\{(?!\{)(?:[^{}]|\{(?!\{)(?:[^{}]|\{(?!\{)(?:[^{}])+\})+\})+\}/,
            lookbehind: true,
            inside: {
              'format-spec': {
                pattern: /(:)[^:(){}]+(?=\}$)/,
                lookbehind: true
              },
              'conversion-option': {
                pattern: /![sra](?=[:}]$)/,
                alias: 'punctuation'
              },
              rest: null
            }
          },
          string: /[\s\S]+/
        }
      },
      'triple-quoted-string': {
        pattern: /(?:[rub]|br|rb)?("""|''')[\s\S]*?\1/i,
        greedy: true,
        alias: 'string'
      },
      string: {
        pattern: /(?:[rub]|br|rb)?("|')(?:\\.|(?!\1)[^\\\r\n])*\1/i,
        greedy: true
      },
      function: {
        pattern: /((?:^|\s)def[ \t]+)[a-zA-Z_]\w*(?=\s*\()/g,
        lookbehind: true
      },
      'class-name': {
        pattern: /(\bclass\s+)\w+/i,
        lookbehind: true
      },
      decorator: {
        pattern: /(^[\t ]*)@\w+(?:\.\w+)*/m,
        lookbehind: true,
        alias: ['annotation', 'punctuation'],
        inside: {
          punctuation: /\./
        }
      },
      keyword:
        /\b(?:_(?=\s*:)|and|as|assert|async|await|break|case|class|continue|def|del|elif|else|except|exec|finally|for|from|global|if|import|in|is|lambda|match|nonlocal|not|or|pass|print|raise|return|try|while|with|yield)\b/,
      builtin:
        /\b(?:__import__|abs|all|any|apply|ascii|basestring|bin|bool|buffer|bytearray|bytes|callable|chr|classmethod|cmp|coerce|compile|complex|delattr|dict|dir|divmod|enumerate|eval|execfile|file|filter|float|format|frozenset|getattr|globals|hasattr|hash|help|hex|id|input|int|intern|isinstance|issubclass|iter|len|list|locals|long|map|max|memoryview|min|next|object|oct|open|ord|pow|property|range|raw_input|reduce|reload|repr|reversed|round|set|setattr|slice|sorted|staticmethod|str|sum|super|tuple|type|unichr|unicode|vars|xrange|zip)\b/,
      boolean: /\b(?:False|None|True)\b/,
      number:
        /\b0(?:b(?:_?[01])+|o(?:_?[0-7])+|x(?:_?[a-f0-9])+)\b|(?:\b\d+(?:_\d+)*(?:\.(?:\d+(?:_\d+)*)?)?|\B\.\d+(?:_\d+)*)(?:e[+-]?\d+(?:_\d+)*)?j?(?!\w)/i,
      operator: /[-+%=]=?|!=|:=|\*\*?=?|\/\/?=?|<[<=>]?|>[=>]?|[&|^~]/,
      punctuation: /[{}[\];(),.:]/
    };
    Prism.languages.python['string-interpolation'].inside[
      'interpolation'
    ].inside.rest = Prism.languages.python;
    Prism.languages.py = Prism.languages.python;
  }

  // @ts-nocheck
  r.displayName = 'r';
  r.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function r(Prism) {
    Prism.languages.r = {
      comment: /#.*/,
      string: {
        pattern: /(['"])(?:\\.|(?!\1)[^\\\r\n])*\1/,
        greedy: true
      },
      'percent-operator': {
        // Includes user-defined operators
        // and %%, %*%, %/%, %in%, %o%, %x%
        pattern: /%[^%\s]*%/,
        alias: 'operator'
      },
      boolean: /\b(?:FALSE|TRUE)\b/,
      ellipsis: /\.\.(?:\.|\d+)/,
      number: [
        /\b(?:Inf|NaN)\b/,
        /(?:\b0x[\dA-Fa-f]+(?:\.\d*)?|\b\d+(?:\.\d*)?|\B\.\d+)(?:[EePp][+-]?\d+)?[iL]?/
      ],
      keyword:
        /\b(?:NA|NA_character_|NA_complex_|NA_integer_|NA_real_|NULL|break|else|for|function|if|in|next|repeat|while)\b/,
      operator: /->?>?|<(?:=|<?-)?|[>=!]=?|::?|&&?|\|\|?|[+*\/^$@~]/,
      punctuation: /[(){}\[\],;]/
    };
  }

  // @ts-nocheck
  ruby.displayName = 'ruby';
  ruby.aliases = ['rb'];

  /** @type {import('../core.js').Syntax} */
  function ruby(Prism) {
    Prism.register(clike)
    /**
     * Original by Samuel Flores
     *
     * Adds the following new token classes:
     *     constant, builtin, variable, symbol, regex
     */
    ;(function (Prism) {
      Prism.languages.ruby = Prism.languages.extend('clike', {
        comment: {
          pattern: /#.*|^=begin\s[\s\S]*?^=end/m,
          greedy: true
        },
        'class-name': {
          pattern:
            /(\b(?:class|module)\s+|\bcatch\s+\()[\w.\\]+|\b[A-Z_]\w*(?=\s*\.\s*new\b)/,
          lookbehind: true,
          inside: {
            punctuation: /[.\\]/
          }
        },
        keyword:
          /\b(?:BEGIN|END|alias|and|begin|break|case|class|def|define_method|defined|do|each|else|elsif|end|ensure|extend|for|if|in|include|module|new|next|nil|not|or|prepend|private|protected|public|raise|redo|require|rescue|retry|return|self|super|then|throw|undef|unless|until|when|while|yield)\b/,
        operator:
          /\.{2,3}|&\.|===|<?=>|[!=]?~|(?:&&|\|\||<<|>>|\*\*|[+\-*/%<>!^&|=])=?|[?:]/,
        punctuation: /[(){}[\].,;]/
      });
      Prism.languages.insertBefore('ruby', 'operator', {
        'double-colon': {
          pattern: /::/,
          alias: 'punctuation'
        }
      });
      var interpolation = {
        pattern: /((?:^|[^\\])(?:\\{2})*)#\{(?:[^{}]|\{[^{}]*\})*\}/,
        lookbehind: true,
        inside: {
          content: {
            pattern: /^(#\{)[\s\S]+(?=\}$)/,
            lookbehind: true,
            inside: Prism.languages.ruby
          },
          delimiter: {
            pattern: /^#\{|\}$/,
            alias: 'punctuation'
          }
        }
      };
      delete Prism.languages.ruby.function;
      var percentExpression =
        '(?:' +
        [
          /([^a-zA-Z0-9\s{(\[<=])(?:(?!\1)[^\\]|\\[\s\S])*\1/.source,
          /\((?:[^()\\]|\\[\s\S]|\((?:[^()\\]|\\[\s\S])*\))*\)/.source,
          /\{(?:[^{}\\]|\\[\s\S]|\{(?:[^{}\\]|\\[\s\S])*\})*\}/.source,
          /\[(?:[^\[\]\\]|\\[\s\S]|\[(?:[^\[\]\\]|\\[\s\S])*\])*\]/.source,
          /<(?:[^<>\\]|\\[\s\S]|<(?:[^<>\\]|\\[\s\S])*>)*>/.source
        ].join('|') +
        ')';
      var symbolName =
        /(?:"(?:\\.|[^"\\\r\n])*"|(?:\b[a-zA-Z_]\w*|[^\s\0-\x7F]+)[?!]?|\$.)/
          .source;
      Prism.languages.insertBefore('ruby', 'keyword', {
        'regex-literal': [
          {
            pattern: RegExp(
              /%r/.source + percentExpression + /[egimnosux]{0,6}/.source
            ),
            greedy: true,
            inside: {
              interpolation: interpolation,
              regex: /[\s\S]+/
            }
          },
          {
            pattern:
              /(^|[^/])\/(?!\/)(?:\[[^\r\n\]]+\]|\\.|[^[/\\\r\n])+\/[egimnosux]{0,6}(?=\s*(?:$|[\r\n,.;})#]))/,
            lookbehind: true,
            greedy: true,
            inside: {
              interpolation: interpolation,
              regex: /[\s\S]+/
            }
          }
        ],
        variable: /[@$]+[a-zA-Z_]\w*(?:[?!]|\b)/,
        symbol: [
          {
            pattern: RegExp(/(^|[^:]):/.source + symbolName),
            lookbehind: true,
            greedy: true
          },
          {
            pattern: RegExp(
              /([\r\n{(,][ \t]*)/.source + symbolName + /(?=:(?!:))/.source
            ),
            lookbehind: true,
            greedy: true
          }
        ],
        'method-definition': {
          pattern: /(\bdef\s+)\w+(?:\s*\.\s*\w+)?/,
          lookbehind: true,
          inside: {
            function: /\b\w+$/,
            keyword: /^self\b/,
            'class-name': /^\w+/,
            punctuation: /\./
          }
        }
      });
      Prism.languages.insertBefore('ruby', 'string', {
        'string-literal': [
          {
            pattern: RegExp(/%[qQiIwWs]?/.source + percentExpression),
            greedy: true,
            inside: {
              interpolation: interpolation,
              string: /[\s\S]+/
            }
          },
          {
            pattern:
              /("|')(?:#\{[^}]+\}|#(?!\{)|\\(?:\r\n|[\s\S])|(?!\1)[^\\#\r\n])*\1/,
            greedy: true,
            inside: {
              interpolation: interpolation,
              string: /[\s\S]+/
            }
          },
          {
            pattern: /<<[-~]?([a-z_]\w*)[\r\n](?:.*[\r\n])*?[\t ]*\1/i,
            alias: 'heredoc-string',
            greedy: true,
            inside: {
              delimiter: {
                pattern: /^<<[-~]?[a-z_]\w*|\b[a-z_]\w*$/i,
                inside: {
                  symbol: /\b\w+/,
                  punctuation: /^<<[-~]?/
                }
              },
              interpolation: interpolation,
              string: /[\s\S]+/
            }
          },
          {
            pattern: /<<[-~]?'([a-z_]\w*)'[\r\n](?:.*[\r\n])*?[\t ]*\1/i,
            alias: 'heredoc-string',
            greedy: true,
            inside: {
              delimiter: {
                pattern: /^<<[-~]?'[a-z_]\w*'|\b[a-z_]\w*$/i,
                inside: {
                  symbol: /\b\w+/,
                  punctuation: /^<<[-~]?'|'$/
                }
              },
              string: /[\s\S]+/
            }
          }
        ],
        'command-literal': [
          {
            pattern: RegExp(/%x/.source + percentExpression),
            greedy: true,
            inside: {
              interpolation: interpolation,
              command: {
                pattern: /[\s\S]+/,
                alias: 'string'
              }
            }
          },
          {
            pattern: /`(?:#\{[^}]+\}|#(?!\{)|\\(?:\r\n|[\s\S])|[^\\`#\r\n])*`/,
            greedy: true,
            inside: {
              interpolation: interpolation,
              command: {
                pattern: /[\s\S]+/,
                alias: 'string'
              }
            }
          }
        ]
      });
      delete Prism.languages.ruby.string;
      Prism.languages.insertBefore('ruby', 'number', {
        builtin:
          /\b(?:Array|Bignum|Binding|Class|Continuation|Dir|Exception|FalseClass|File|Fixnum|Float|Hash|IO|Integer|MatchData|Method|Module|NilClass|Numeric|Object|Proc|Range|Regexp|Stat|String|Struct|Symbol|TMS|Thread|ThreadGroup|Time|TrueClass)\b/,
        constant: /\b[A-Z][A-Z0-9_]*(?:[?!]|\b)/
      });
      Prism.languages.rb = Prism.languages.ruby;
    })(Prism);
  }

  // @ts-nocheck
  rust.displayName = 'rust';
  rust.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function rust(Prism) {
  (function (Prism) {
      var multilineComment = /\/\*(?:[^*/]|\*(?!\/)|\/(?!\*)|<self>)*\*\//.source;
      for (var i = 0; i < 2; i++) {
        // support 4 levels of nested comments
        multilineComment = multilineComment.replace(/<self>/g, function () {
          return multilineComment
        });
      }
      multilineComment = multilineComment.replace(/<self>/g, function () {
        return /[^\s\S]/.source
      });
      Prism.languages.rust = {
        comment: [
          {
            pattern: RegExp(/(^|[^\\])/.source + multilineComment),
            lookbehind: true,
            greedy: true
          },
          {
            pattern: /(^|[^\\:])\/\/.*/,
            lookbehind: true,
            greedy: true
          }
        ],
        string: {
          pattern: /b?"(?:\\[\s\S]|[^\\"])*"|b?r(#*)"(?:[^"]|"(?!\1))*"\1/,
          greedy: true
        },
        char: {
          pattern:
            /b?'(?:\\(?:x[0-7][\da-fA-F]|u\{(?:[\da-fA-F]_*){1,6}\}|.)|[^\\\r\n\t'])'/,
          greedy: true
        },
        attribute: {
          pattern: /#!?\[(?:[^\[\]"]|"(?:\\[\s\S]|[^\\"])*")*\]/,
          greedy: true,
          alias: 'attr-name',
          inside: {
            string: null // see below
          }
        },
        // Closure params should not be confused with bitwise OR |
        'closure-params': {
          pattern: /([=(,:]\s*|\bmove\s*)\|[^|]*\||\|[^|]*\|(?=\s*(?:\{|->))/,
          lookbehind: true,
          greedy: true,
          inside: {
            'closure-punctuation': {
              pattern: /^\||\|$/,
              alias: 'punctuation'
            },
            rest: null // see below
          }
        },
        'lifetime-annotation': {
          pattern: /'\w+/,
          alias: 'symbol'
        },
        'fragment-specifier': {
          pattern: /(\$\w+:)[a-z]+/,
          lookbehind: true,
          alias: 'punctuation'
        },
        variable: /\$\w+/,
        'function-definition': {
          pattern: /(\bfn\s+)\w+/,
          lookbehind: true,
          alias: 'function'
        },
        'type-definition': {
          pattern: /(\b(?:enum|struct|trait|type|union)\s+)\w+/,
          lookbehind: true,
          alias: 'class-name'
        },
        'module-declaration': [
          {
            pattern: /(\b(?:crate|mod)\s+)[a-z][a-z_\d]*/,
            lookbehind: true,
            alias: 'namespace'
          },
          {
            pattern:
              /(\b(?:crate|self|super)\s*)::\s*[a-z][a-z_\d]*\b(?:\s*::(?:\s*[a-z][a-z_\d]*\s*::)*)?/,
            lookbehind: true,
            alias: 'namespace',
            inside: {
              punctuation: /::/
            }
          }
        ],
        keyword: [
          // https://github.com/rust-lang/reference/blob/master/src/keywords.md
          /\b(?:Self|abstract|as|async|await|become|box|break|const|continue|crate|do|dyn|else|enum|extern|final|fn|for|if|impl|in|let|loop|macro|match|mod|move|mut|override|priv|pub|ref|return|self|static|struct|super|trait|try|type|typeof|union|unsafe|unsized|use|virtual|where|while|yield)\b/, // primitives and str
          // https://doc.rust-lang.org/stable/rust-by-example/primitives.html
          /\b(?:bool|char|f(?:32|64)|[ui](?:8|16|32|64|128|size)|str)\b/
        ],
        // functions can technically start with an upper-case letter, but this will introduce a lot of false positives
        // and Rust's naming conventions recommend snake_case anyway.
        // https://doc.rust-lang.org/1.0.0/style/style/naming/README.html
        function: /\b[a-z_]\w*(?=\s*(?:::\s*<|\())/,
        macro: {
          pattern: /\b\w+!/,
          alias: 'property'
        },
        constant: /\b[A-Z_][A-Z_\d]+\b/,
        'class-name': /\b[A-Z]\w*\b/,
        namespace: {
          pattern: /(?:\b[a-z][a-z_\d]*\s*::\s*)*\b[a-z][a-z_\d]*\s*::(?!\s*<)/,
          inside: {
            punctuation: /::/
          }
        },
        // Hex, oct, bin, dec numbers with visual separators and type suffix
        number:
          /\b(?:0x[\dA-Fa-f](?:_?[\dA-Fa-f])*|0o[0-7](?:_?[0-7])*|0b[01](?:_?[01])*|(?:(?:\d(?:_?\d)*)?\.)?\d(?:_?\d)*(?:[Ee][+-]?\d+)?)(?:_?(?:f32|f64|[iu](?:8|16|32|64|size)?))?\b/,
        boolean: /\b(?:false|true)\b/,
        punctuation: /->|\.\.=|\.{1,3}|::|[{}[\];(),:]/,
        operator: /[-+*\/%!^]=?|=[=>]?|&[&=]?|\|[|=]?|<<?=?|>>?=?|[@?]/
      };
      Prism.languages.rust['closure-params'].inside.rest = Prism.languages.rust;
      Prism.languages.rust['attribute'].inside['string'] =
        Prism.languages.rust['string'];
    })(Prism);
  }

  // @ts-nocheck
  sass.displayName = 'sass';
  sass.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function sass(Prism) {
    Prism.register(css)
    ;(function (Prism) {
      Prism.languages.sass = Prism.languages.extend('css', {
        // Sass comments don't need to be closed, only indented
        comment: {
          pattern: /^([ \t]*)\/[\/*].*(?:(?:\r?\n|\r)\1[ \t].+)*/m,
          lookbehind: true,
          greedy: true
        }
      });
      Prism.languages.insertBefore('sass', 'atrule', {
        // We want to consume the whole line
        'atrule-line': {
          // Includes support for = and + shortcuts
          pattern: /^(?:[ \t]*)[@+=].+/m,
          greedy: true,
          inside: {
            atrule: /(?:@[\w-]+|[+=])/
          }
        }
      });
      delete Prism.languages.sass.atrule;
      var variable = /\$[-\w]+|#\{\$[-\w]+\}/;
      var operator = [
        /[+*\/%]|[=!]=|<=?|>=?|\b(?:and|not|or)\b/,
        {
          pattern: /(\s)-(?=\s)/,
          lookbehind: true
        }
      ];
      Prism.languages.insertBefore('sass', 'property', {
        // We want to consume the whole line
        'variable-line': {
          pattern: /^[ \t]*\$.+/m,
          greedy: true,
          inside: {
            punctuation: /:/,
            variable: variable,
            operator: operator
          }
        },
        // We want to consume the whole line
        'property-line': {
          pattern: /^[ \t]*(?:[^:\s]+ *:.*|:[^:\s].*)/m,
          greedy: true,
          inside: {
            property: [
              /[^:\s]+(?=\s*:)/,
              {
                pattern: /(:)[^:\s]+/,
                lookbehind: true
              }
            ],
            punctuation: /:/,
            variable: variable,
            operator: operator,
            important: Prism.languages.sass.important
          }
        }
      });
      delete Prism.languages.sass.property;
      delete Prism.languages.sass.important; // Now that whole lines for other patterns are consumed,
      // what's left should be selectors
      Prism.languages.insertBefore('sass', 'punctuation', {
        selector: {
          pattern:
            /^([ \t]*)\S(?:,[^,\r\n]+|[^,\r\n]*)(?:,[^,\r\n]+)*(?:,(?:\r?\n|\r)\1[ \t]+\S(?:,[^,\r\n]+|[^,\r\n]*)(?:,[^,\r\n]+)*)*/m,
          lookbehind: true,
          greedy: true
        }
      });
    })(Prism);
  }

  // @ts-nocheck
  scss.displayName = 'scss';
  scss.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function scss(Prism) {
    Prism.register(css);
    Prism.languages.scss = Prism.languages.extend('css', {
      comment: {
        pattern: /(^|[^\\])(?:\/\*[\s\S]*?\*\/|\/\/.*)/,
        lookbehind: true
      },
      atrule: {
        pattern: /@[\w-](?:\([^()]+\)|[^()\s]|\s+(?!\s))*?(?=\s+[{;])/,
        inside: {
          rule: /@[\w-]+/ // See rest below
        }
      },
      // url, compassified
      url: /(?:[-a-z]+-)?url(?=\()/i,
      // CSS selector regex is not appropriate for Sass
      // since there can be lot more things (var, @ directive, nesting..)
      // a selector must start at the end of a property or after a brace (end of other rules or nesting)
      // it can contain some characters that aren't used for defining rules or end of selector, & (parent selector), or interpolated variable
      // the end of a selector is found when there is no rules in it ( {} or {\s}) or if there is a property (because an interpolated var
      // can "pass" as a selector- e.g: proper#{$erty})
      // this one was hard to do, so please be careful if you edit this one :)
      selector: {
        // Initial look-ahead is used to prevent matching of blank selectors
        pattern:
          /(?=\S)[^@;{}()]?(?:[^@;{}()\s]|\s+(?!\s)|#\{\$[-\w]+\})+(?=\s*\{(?:\}|\s|[^}][^:{}]*[:{][^}]))/,
        inside: {
          parent: {
            pattern: /&/,
            alias: 'important'
          },
          placeholder: /%[-\w]+/,
          variable: /\$[-\w]+|#\{\$[-\w]+\}/
        }
      },
      property: {
        pattern: /(?:[-\w]|\$[-\w]|#\{\$[-\w]+\})+(?=\s*:)/,
        inside: {
          variable: /\$[-\w]+|#\{\$[-\w]+\}/
        }
      }
    });
    Prism.languages.insertBefore('scss', 'atrule', {
      keyword: [
        /@(?:content|debug|each|else(?: if)?|extend|for|forward|function|if|import|include|mixin|return|use|warn|while)\b/i,
        {
          pattern: /( )(?:from|through)(?= )/,
          lookbehind: true
        }
      ]
    });
    Prism.languages.insertBefore('scss', 'important', {
      // var and interpolated vars
      variable: /\$[-\w]+|#\{\$[-\w]+\}/
    });
    Prism.languages.insertBefore('scss', 'function', {
      'module-modifier': {
        pattern: /\b(?:as|hide|show|with)\b/i,
        alias: 'keyword'
      },
      placeholder: {
        pattern: /%[-\w]+/,
        alias: 'selector'
      },
      statement: {
        pattern: /\B!(?:default|optional)\b/i,
        alias: 'keyword'
      },
      boolean: /\b(?:false|true)\b/,
      null: {
        pattern: /\bnull\b/,
        alias: 'keyword'
      },
      operator: {
        pattern: /(\s)(?:[-+*\/%]|[=!]=|<=?|>=?|and|not|or)(?=\s)/,
        lookbehind: true
      }
    });
    Prism.languages.scss['atrule'].inside.rest = Prism.languages.scss;
  }

  // @ts-nocheck
  sql.displayName = 'sql';
  sql.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function sql(Prism) {
    Prism.languages.sql = {
      comment: {
        pattern: /(^|[^\\])(?:\/\*[\s\S]*?\*\/|(?:--|\/\/|#).*)/,
        lookbehind: true
      },
      variable: [
        {
          pattern: /@(["'`])(?:\\[\s\S]|(?!\1)[^\\])+\1/,
          greedy: true
        },
        /@[\w.$]+/
      ],
      string: {
        pattern: /(^|[^@\\])("|')(?:\\[\s\S]|(?!\2)[^\\]|\2\2)*\2/,
        greedy: true,
        lookbehind: true
      },
      identifier: {
        pattern: /(^|[^@\\])`(?:\\[\s\S]|[^`\\]|``)*`/,
        greedy: true,
        lookbehind: true,
        inside: {
          punctuation: /^`|`$/
        }
      },
      function:
        /\b(?:AVG|COUNT|FIRST|FORMAT|LAST|LCASE|LEN|MAX|MID|MIN|MOD|NOW|ROUND|SUM|UCASE)(?=\s*\()/i,
      // Should we highlight user defined functions too?
      keyword:
        /\b(?:ACTION|ADD|AFTER|ALGORITHM|ALL|ALTER|ANALYZE|ANY|APPLY|AS|ASC|AUTHORIZATION|AUTO_INCREMENT|BACKUP|BDB|BEGIN|BERKELEYDB|BIGINT|BINARY|BIT|BLOB|BOOL|BOOLEAN|BREAK|BROWSE|BTREE|BULK|BY|CALL|CASCADED?|CASE|CHAIN|CHAR(?:ACTER|SET)?|CHECK(?:POINT)?|CLOSE|CLUSTERED|COALESCE|COLLATE|COLUMNS?|COMMENT|COMMIT(?:TED)?|COMPUTE|CONNECT|CONSISTENT|CONSTRAINT|CONTAINS(?:TABLE)?|CONTINUE|CONVERT|CREATE|CROSS|CURRENT(?:_DATE|_TIME|_TIMESTAMP|_USER)?|CURSOR|CYCLE|DATA(?:BASES?)?|DATE(?:TIME)?|DAY|DBCC|DEALLOCATE|DEC|DECIMAL|DECLARE|DEFAULT|DEFINER|DELAYED|DELETE|DELIMITERS?|DENY|DESC|DESCRIBE|DETERMINISTIC|DISABLE|DISCARD|DISK|DISTINCT|DISTINCTROW|DISTRIBUTED|DO|DOUBLE|DROP|DUMMY|DUMP(?:FILE)?|DUPLICATE|ELSE(?:IF)?|ENABLE|ENCLOSED|END|ENGINE|ENUM|ERRLVL|ERRORS|ESCAPED?|EXCEPT|EXEC(?:UTE)?|EXISTS|EXIT|EXPLAIN|EXTENDED|FETCH|FIELDS|FILE|FILLFACTOR|FIRST|FIXED|FLOAT|FOLLOWING|FOR(?: EACH ROW)?|FORCE|FOREIGN|FREETEXT(?:TABLE)?|FROM|FULL|FUNCTION|GEOMETRY(?:COLLECTION)?|GLOBAL|GOTO|GRANT|GROUP|HANDLER|HASH|HAVING|HOLDLOCK|HOUR|IDENTITY(?:COL|_INSERT)?|IF|IGNORE|IMPORT|INDEX|INFILE|INNER|INNODB|INOUT|INSERT|INT|INTEGER|INTERSECT|INTERVAL|INTO|INVOKER|ISOLATION|ITERATE|JOIN|KEYS?|KILL|LANGUAGE|LAST|LEAVE|LEFT|LEVEL|LIMIT|LINENO|LINES|LINESTRING|LOAD|LOCAL|LOCK|LONG(?:BLOB|TEXT)|LOOP|MATCH(?:ED)?|MEDIUM(?:BLOB|INT|TEXT)|MERGE|MIDDLEINT|MINUTE|MODE|MODIFIES|MODIFY|MONTH|MULTI(?:LINESTRING|POINT|POLYGON)|NATIONAL|NATURAL|NCHAR|NEXT|NO|NONCLUSTERED|NULLIF|NUMERIC|OFF?|OFFSETS?|ON|OPEN(?:DATASOURCE|QUERY|ROWSET)?|OPTIMIZE|OPTION(?:ALLY)?|ORDER|OUT(?:ER|FILE)?|OVER|PARTIAL|PARTITION|PERCENT|PIVOT|PLAN|POINT|POLYGON|PRECEDING|PRECISION|PREPARE|PREV|PRIMARY|PRINT|PRIVILEGES|PROC(?:EDURE)?|PUBLIC|PURGE|QUICK|RAISERROR|READS?|REAL|RECONFIGURE|REFERENCES|RELEASE|RENAME|REPEAT(?:ABLE)?|REPLACE|REPLICATION|REQUIRE|RESIGNAL|RESTORE|RESTRICT|RETURN(?:ING|S)?|REVOKE|RIGHT|ROLLBACK|ROUTINE|ROW(?:COUNT|GUIDCOL|S)?|RTREE|RULE|SAVE(?:POINT)?|SCHEMA|SECOND|SELECT|SERIAL(?:IZABLE)?|SESSION(?:_USER)?|SET(?:USER)?|SHARE|SHOW|SHUTDOWN|SIMPLE|SMALLINT|SNAPSHOT|SOME|SONAME|SQL|START(?:ING)?|STATISTICS|STATUS|STRIPED|SYSTEM_USER|TABLES?|TABLESPACE|TEMP(?:ORARY|TABLE)?|TERMINATED|TEXT(?:SIZE)?|THEN|TIME(?:STAMP)?|TINY(?:BLOB|INT|TEXT)|TOP?|TRAN(?:SACTIONS?)?|TRIGGER|TRUNCATE|TSEQUAL|TYPES?|UNBOUNDED|UNCOMMITTED|UNDEFINED|UNION|UNIQUE|UNLOCK|UNPIVOT|UNSIGNED|UPDATE(?:TEXT)?|USAGE|USE|USER|USING|VALUES?|VAR(?:BINARY|CHAR|CHARACTER|YING)|VIEW|WAITFOR|WARNINGS|WHEN|WHERE|WHILE|WITH(?: ROLLUP|IN)?|WORK|WRITE(?:TEXT)?|YEAR)\b/i,
      boolean: /\b(?:FALSE|NULL|TRUE)\b/i,
      number: /\b0x[\da-f]+\b|\b\d+(?:\.\d*)?|\B\.\d+\b/i,
      operator:
        /[-+*\/=%^~]|&&?|\|\|?|!=?|<(?:=>?|<|>)?|>[>=]?|\b(?:AND|BETWEEN|DIV|ILIKE|IN|IS|LIKE|NOT|OR|REGEXP|RLIKE|SOUNDS LIKE|XOR)\b/i,
      punctuation: /[;[\]()`,.]/
    };
  }

  // @ts-nocheck
  swift.displayName = 'swift';
  swift.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function swift(Prism) {
    Prism.languages.swift = {
      comment: {
        // Nested comments are supported up to 2 levels
        pattern:
          /(^|[^\\:])(?:\/\/.*|\/\*(?:[^/*]|\/(?!\*)|\*(?!\/)|\/\*(?:[^*]|\*(?!\/))*\*\/)*\*\/)/,
        lookbehind: true,
        greedy: true
      },
      'string-literal': [
        // https://docs.swift.org/swift-book/LanguageGuide/StringsAndCharacters.html
        {
          pattern: RegExp(
            /(^|[^"#])/.source +
              '(?:' + // single-line string
              /"(?:\\(?:\((?:[^()]|\([^()]*\))*\)|\r\n|[^(])|[^\\\r\n"])*"/
                .source +
              '|' + // multi-line string
              /"""(?:\\(?:\((?:[^()]|\([^()]*\))*\)|[^(])|[^\\"]|"(?!""))*"""/
                .source +
              ')' +
              /(?!["#])/.source
          ),
          lookbehind: true,
          greedy: true,
          inside: {
            interpolation: {
              pattern: /(\\\()(?:[^()]|\([^()]*\))*(?=\))/,
              lookbehind: true,
              inside: null // see below
            },
            'interpolation-punctuation': {
              pattern: /^\)|\\\($/,
              alias: 'punctuation'
            },
            punctuation: /\\(?=[\r\n])/,
            string: /[\s\S]+/
          }
        },
        {
          pattern: RegExp(
            /(^|[^"#])(#+)/.source +
              '(?:' + // single-line string
              /"(?:\\(?:#+\((?:[^()]|\([^()]*\))*\)|\r\n|[^#])|[^\\\r\n])*?"/
                .source +
              '|' + // multi-line string
              /"""(?:\\(?:#+\((?:[^()]|\([^()]*\))*\)|[^#])|[^\\])*?"""/.source +
              ')' +
              '\\2'
          ),
          lookbehind: true,
          greedy: true,
          inside: {
            interpolation: {
              pattern: /(\\#+\()(?:[^()]|\([^()]*\))*(?=\))/,
              lookbehind: true,
              inside: null // see below
            },
            'interpolation-punctuation': {
              pattern: /^\)|\\#+\($/,
              alias: 'punctuation'
            },
            string: /[\s\S]+/
          }
        }
      ],
      directive: {
        // directives with conditions
        pattern: RegExp(
          /#/.source +
            '(?:' +
            (/(?:elseif|if)\b/.source +
              '(?:[ \t]*' + // This regex is a little complex. It's equivalent to this:
              //   (?:![ \t]*)?(?:\b\w+\b(?:[ \t]*<round>)?|<round>)(?:[ \t]*(?:&&|\|\|))?
              // where <round> is a general parentheses expression.
              /(?:![ \t]*)?(?:\b\w+\b(?:[ \t]*\((?:[^()]|\([^()]*\))*\))?|\((?:[^()]|\([^()]*\))*\))(?:[ \t]*(?:&&|\|\|))?/
                .source +
              ')+') +
            '|' +
            /(?:else|endif)\b/.source +
            ')'
        ),
        alias: 'property',
        inside: {
          'directive-name': /^#\w+/,
          boolean: /\b(?:false|true)\b/,
          number: /\b\d+(?:\.\d+)*\b/,
          operator: /!|&&|\|\||[<>]=?/,
          punctuation: /[(),]/
        }
      },
      literal: {
        pattern:
          /#(?:colorLiteral|column|dsohandle|file(?:ID|Literal|Path)?|function|imageLiteral|line)\b/,
        alias: 'constant'
      },
      'other-directive': {
        pattern: /#\w+\b/,
        alias: 'property'
      },
      attribute: {
        pattern: /@\w+/,
        alias: 'atrule'
      },
      'function-definition': {
        pattern: /(\bfunc\s+)\w+/,
        lookbehind: true,
        alias: 'function'
      },
      label: {
        // https://docs.swift.org/swift-book/LanguageGuide/ControlFlow.html#ID141
        pattern:
          /\b(break|continue)\s+\w+|\b[a-zA-Z_]\w*(?=\s*:\s*(?:for|repeat|while)\b)/,
        lookbehind: true,
        alias: 'important'
      },
      keyword:
        /\b(?:Any|Protocol|Self|Type|actor|as|assignment|associatedtype|associativity|async|await|break|case|catch|class|continue|convenience|default|defer|deinit|didSet|do|dynamic|else|enum|extension|fallthrough|fileprivate|final|for|func|get|guard|higherThan|if|import|in|indirect|infix|init|inout|internal|is|isolated|lazy|left|let|lowerThan|mutating|none|nonisolated|nonmutating|open|operator|optional|override|postfix|precedencegroup|prefix|private|protocol|public|repeat|required|rethrows|return|right|safe|self|set|some|static|struct|subscript|super|switch|throw|throws|try|typealias|unowned|unsafe|var|weak|where|while|willSet)\b/,
      boolean: /\b(?:false|true)\b/,
      nil: {
        pattern: /\bnil\b/,
        alias: 'constant'
      },
      'short-argument': /\$\d+\b/,
      omit: {
        pattern: /\b_\b/,
        alias: 'keyword'
      },
      number:
        /\b(?:[\d_]+(?:\.[\de_]+)?|0x[a-f0-9_]+(?:\.[a-f0-9p_]+)?|0b[01_]+|0o[0-7_]+)\b/i,
      // A class name must start with an upper-case letter and be either 1 letter long or contain a lower-case letter.
      'class-name': /\b[A-Z](?:[A-Z_\d]*[a-z]\w*)?\b/,
      function: /\b[a-z_]\w*(?=\s*\()/i,
      constant: /\b(?:[A-Z_]{2,}|k[A-Z][A-Za-z_]+)\b/,
      // Operators are generic in Swift. Developers can even create new operators (e.g. +++).
      // https://docs.swift.org/swift-book/ReferenceManual/zzSummaryOfTheGrammar.html#ID481
      // This regex only supports ASCII operators.
      operator: /[-+*/%=!<>&|^~?]+|\.[.\-+*/%=!<>&|^~?]+/,
      punctuation: /[{}[\]();,.:\\]/
    };
    Prism.languages.swift['string-literal'].forEach(function (rule) {
      rule.inside['interpolation'].inside = Prism.languages.swift;
    });
  }

  // @ts-nocheck
  typescript.displayName = 'typescript';
  typescript.aliases = ['ts'];

  /** @type {import('../core.js').Syntax} */
  function typescript(Prism) {
    Prism.register(javascript)
    ;(function (Prism) {
      Prism.languages.typescript = Prism.languages.extend('javascript', {
        'class-name': {
          pattern:
            /(\b(?:class|extends|implements|instanceof|interface|new|type)\s+)(?!keyof\b)(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?:\s*<(?:[^<>]|<(?:[^<>]|<[^<>]*>)*>)*>)?/,
          lookbehind: true,
          greedy: true,
          inside: null // see below
        },
        builtin:
          /\b(?:Array|Function|Promise|any|boolean|console|never|number|string|symbol|unknown)\b/
      }); // The keywords TypeScript adds to JavaScript
      Prism.languages.typescript.keyword.push(
        /\b(?:abstract|declare|is|keyof|readonly|require)\b/, // keywords that have to be followed by an identifier
        /\b(?:asserts|infer|interface|module|namespace|type)\b(?=\s*(?:[{_$a-zA-Z\xA0-\uFFFF]|$))/, // This is for `import type *, {}`
        /\btype\b(?=\s*(?:[\{*]|$))/
      ); // doesn't work with TS because TS is too complex
      delete Prism.languages.typescript['parameter'];
      delete Prism.languages.typescript['literal-property']; // a version of typescript specifically for highlighting types
      var typeInside = Prism.languages.extend('typescript', {});
      delete typeInside['class-name'];
      Prism.languages.typescript['class-name'].inside = typeInside;
      Prism.languages.insertBefore('typescript', 'function', {
        decorator: {
          pattern: /@[$\w\xA0-\uFFFF]+/,
          inside: {
            at: {
              pattern: /^@/,
              alias: 'operator'
            },
            function: /^[\s\S]+/
          }
        },
        'generic-function': {
          // e.g. foo<T extends "bar" | "baz">( ...
          pattern:
            /#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*\s*<(?:[^<>]|<(?:[^<>]|<[^<>]*>)*>)*>(?=\s*\()/,
          greedy: true,
          inside: {
            function: /^#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*/,
            generic: {
              pattern: /<[\s\S]+/,
              // everything after the first <
              alias: 'class-name',
              inside: typeInside
            }
          }
        }
      });
      Prism.languages.ts = Prism.languages.typescript;
    })(Prism);
  }

  // @ts-nocheck
  basic.displayName = 'basic';
  basic.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function basic(Prism) {
    Prism.languages.basic = {
      comment: {
        pattern: /(?:!|REM\b).+/i,
        inside: {
          keyword: /^REM/i
        }
      },
      string: {
        pattern: /"(?:""|[!#$%&'()*,\/:;<=>?^\w +\-.])*"/,
        greedy: true
      },
      number: /(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:E[+-]?\d+)?/i,
      keyword:
        /\b(?:AS|BEEP|BLOAD|BSAVE|CALL(?: ABSOLUTE)?|CASE|CHAIN|CHDIR|CLEAR|CLOSE|CLS|COM|COMMON|CONST|DATA|DECLARE|DEF(?: FN| SEG|DBL|INT|LNG|SNG|STR)|DIM|DO|DOUBLE|ELSE|ELSEIF|END|ENVIRON|ERASE|ERROR|EXIT|FIELD|FILES|FOR|FUNCTION|GET|GOSUB|GOTO|IF|INPUT|INTEGER|IOCTL|KEY|KILL|LINE INPUT|LOCATE|LOCK|LONG|LOOP|LSET|MKDIR|NAME|NEXT|OFF|ON(?: COM| ERROR| KEY| TIMER)?|OPEN|OPTION BASE|OUT|POKE|PUT|READ|REDIM|REM|RESTORE|RESUME|RETURN|RMDIR|RSET|RUN|SELECT CASE|SHARED|SHELL|SINGLE|SLEEP|STATIC|STEP|STOP|STRING|SUB|SWAP|SYSTEM|THEN|TIMER|TO|TROFF|TRON|TYPE|UNLOCK|UNTIL|USING|VIEW PRINT|WAIT|WEND|WHILE|WRITE)(?:\$|\b)/i,
      function:
        /\b(?:ABS|ACCESS|ACOS|ANGLE|AREA|ARITHMETIC|ARRAY|ASIN|ASK|AT|ATN|BASE|BEGIN|BREAK|CAUSE|CEIL|CHR|CLIP|COLLATE|COLOR|CON|COS|COSH|COT|CSC|DATE|DATUM|DEBUG|DECIMAL|DEF|DEG|DEGREES|DELETE|DET|DEVICE|DISPLAY|DOT|ELAPSED|EPS|ERASABLE|EXLINE|EXP|EXTERNAL|EXTYPE|FILETYPE|FIXED|FP|GO|GRAPH|HANDLER|IDN|IMAGE|IN|INT|INTERNAL|IP|IS|KEYED|LBOUND|LCASE|LEFT|LEN|LENGTH|LET|LINE|LINES|LOG|LOG10|LOG2|LTRIM|MARGIN|MAT|MAX|MAXNUM|MID|MIN|MISSING|MOD|NATIVE|NUL|NUMERIC|OF|OPTION|ORD|ORGANIZATION|OUTIN|OUTPUT|PI|POINT|POINTER|POINTS|POS|PRINT|PROGRAM|PROMPT|RAD|RADIANS|RANDOMIZE|RECORD|RECSIZE|RECTYPE|RELATIVE|REMAINDER|REPEAT|REST|RETRY|REWRITE|RIGHT|RND|ROUND|RTRIM|SAME|SEC|SELECT|SEQUENTIAL|SET|SETTER|SGN|SIN|SINH|SIZE|SKIP|SQR|STANDARD|STATUS|STR|STREAM|STYLE|TAB|TAN|TANH|TEMPLATE|TEXT|THERE|TIME|TIMEOUT|TRACE|TRANSFORM|TRUNCATE|UBOUND|UCASE|USE|VAL|VARIABLE|VIEWPORT|WHEN|WINDOW|WITH|ZER|ZONEWIDTH)(?:\$|\b)/i,
      operator: /<[=>]?|>=?|[+\-*\/^=&]|\b(?:AND|EQV|IMP|NOT|OR|XOR)\b/i,
      punctuation: /[,;:()]/
    };
  }

  // @ts-nocheck
  vbnet.displayName = 'vbnet';
  vbnet.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function vbnet(Prism) {
    Prism.register(basic);
    Prism.languages.vbnet = Prism.languages.extend('basic', {
      comment: [
        {
          pattern: /(?:!|REM\b).+/i,
          inside: {
            keyword: /^REM/i
          }
        },
        {
          pattern: /(^|[^\\:])'.*/,
          lookbehind: true,
          greedy: true
        }
      ],
      string: {
        pattern: /(^|[^"])"(?:""|[^"])*"(?!")/,
        lookbehind: true,
        greedy: true
      },
      keyword:
        /(?:\b(?:ADDHANDLER|ADDRESSOF|ALIAS|AND|ANDALSO|AS|BEEP|BLOAD|BOOLEAN|BSAVE|BYREF|BYTE|BYVAL|CALL(?: ABSOLUTE)?|CASE|CATCH|CBOOL|CBYTE|CCHAR|CDATE|CDBL|CDEC|CHAIN|CHAR|CHDIR|CINT|CLASS|CLEAR|CLNG|CLOSE|CLS|COBJ|COM|COMMON|CONST|CONTINUE|CSBYTE|CSHORT|CSNG|CSTR|CTYPE|CUINT|CULNG|CUSHORT|DATA|DATE|DECIMAL|DECLARE|DEF(?: FN| SEG|DBL|INT|LNG|SNG|STR)|DEFAULT|DELEGATE|DIM|DIRECTCAST|DO|DOUBLE|ELSE|ELSEIF|END|ENUM|ENVIRON|ERASE|ERROR|EVENT|EXIT|FALSE|FIELD|FILES|FINALLY|FOR(?: EACH)?|FRIEND|FUNCTION|GET|GETTYPE|GETXMLNAMESPACE|GLOBAL|GOSUB|GOTO|HANDLES|IF|IMPLEMENTS|IMPORTS|IN|INHERITS|INPUT|INTEGER|INTERFACE|IOCTL|IS|ISNOT|KEY|KILL|LET|LIB|LIKE|LINE INPUT|LOCATE|LOCK|LONG|LOOP|LSET|ME|MKDIR|MOD|MODULE|MUSTINHERIT|MUSTOVERRIDE|MYBASE|MYCLASS|NAME|NAMESPACE|NARROWING|NEW|NEXT|NOT|NOTHING|NOTINHERITABLE|NOTOVERRIDABLE|OBJECT|OF|OFF|ON(?: COM| ERROR| KEY| TIMER)?|OPEN|OPERATOR|OPTION(?: BASE)?|OPTIONAL|OR|ORELSE|OUT|OVERLOADS|OVERRIDABLE|OVERRIDES|PARAMARRAY|PARTIAL|POKE|PRIVATE|PROPERTY|PROTECTED|PUBLIC|PUT|RAISEEVENT|READ|READONLY|REDIM|REM|REMOVEHANDLER|RESTORE|RESUME|RETURN|RMDIR|RSET|RUN|SBYTE|SELECT(?: CASE)?|SET|SHADOWS|SHARED|SHELL|SHORT|SINGLE|SLEEP|STATIC|STEP|STOP|STRING|STRUCTURE|SUB|SWAP|SYNCLOCK|SYSTEM|THEN|THROW|TIMER|TO|TROFF|TRON|TRUE|TRY|TRYCAST|TYPE|TYPEOF|UINTEGER|ULONG|UNLOCK|UNTIL|USHORT|USING|VIEW PRINT|WAIT|WEND|WHEN|WHILE|WIDENING|WITH|WITHEVENTS|WRITE|WRITEONLY|XOR)|\B(?:#CONST|#ELSE|#ELSEIF|#END|#IF))(?:\$|\b)/i,
      punctuation: /[,;:(){}]/
    });
  }

  /**
   * @typedef {import('./info.js').Info} Info
   * @typedef {Record<string, Info>} Properties
   * @typedef {Record<string, string>} Normal
   */

  class Schema {
    /**
     * @constructor
     * @param {Properties} property
     * @param {Normal} normal
     * @param {string} [space]
     */
    constructor(property, normal, space) {
      this.property = property;
      this.normal = normal;
      if (space) {
        this.space = space;
      }
    }
  }

  /** @type {Properties} */
  Schema.prototype.property = {};
  /** @type {Normal} */
  Schema.prototype.normal = {};
  /** @type {string|null} */
  Schema.prototype.space = null;

  /**
   * @typedef {import('./schema.js').Properties} Properties
   * @typedef {import('./schema.js').Normal} Normal
   */

  /**
   * @param {Schema[]} definitions
   * @param {string} [space]
   * @returns {Schema}
   */
  function merge(definitions, space) {
    /** @type {Properties} */
    const property = {};
    /** @type {Normal} */
    const normal = {};
    let index = -1;

    while (++index < definitions.length) {
      Object.assign(property, definitions[index].property);
      Object.assign(normal, definitions[index].normal);
    }

    return new Schema(property, normal, space)
  }

  /**
   * @param {string} value
   * @returns {string}
   */
  function normalize(value) {
    return value.toLowerCase()
  }

  class Info {
    /**
     * @constructor
     * @param {string} property
     * @param {string} attribute
     */
    constructor(property, attribute) {
      /** @type {string} */
      this.property = property;
      /** @type {string} */
      this.attribute = attribute;
    }
  }

  /** @type {string|null} */
  Info.prototype.space = null;
  Info.prototype.boolean = false;
  Info.prototype.booleanish = false;
  Info.prototype.overloadedBoolean = false;
  Info.prototype.number = false;
  Info.prototype.commaSeparated = false;
  Info.prototype.spaceSeparated = false;
  Info.prototype.commaOrSpaceSeparated = false;
  Info.prototype.mustUseProperty = false;
  Info.prototype.defined = false;

  let powers = 0;

  const boolean = increment();
  const booleanish = increment();
  const overloadedBoolean = increment();
  const number = increment();
  const spaceSeparated = increment();
  const commaSeparated = increment();
  const commaOrSpaceSeparated = increment();

  function increment() {
    return 2 ** ++powers
  }

  var types = /*#__PURE__*/Object.freeze({
      __proto__: null,
      boolean: boolean,
      booleanish: booleanish,
      overloadedBoolean: overloadedBoolean,
      number: number,
      spaceSeparated: spaceSeparated,
      commaSeparated: commaSeparated,
      commaOrSpaceSeparated: commaOrSpaceSeparated
  });

  /** @type {Array<keyof types>} */
  // @ts-expect-error: hush.
  const checks = Object.keys(types);

  class DefinedInfo extends Info {
    /**
     * @constructor
     * @param {string} property
     * @param {string} attribute
     * @param {number|null} [mask]
     * @param {string} [space]
     */
    constructor(property, attribute, mask, space) {
      let index = -1;

      super(property, attribute);

      mark(this, 'space', space);

      if (typeof mask === 'number') {
        while (++index < checks.length) {
          const check = checks[index];
          mark(this, checks[index], (mask & types[check]) === types[check]);
        }
      }
    }
  }

  DefinedInfo.prototype.defined = true;

  /**
   * @param {DefinedInfo} values
   * @param {string} key
   * @param {unknown} value
   */
  function mark(values, key, value) {
    if (value) {
      // @ts-expect-error: assume `value` matches the expected value of `key`.
      values[key] = value;
    }
  }

  /**
   * @typedef {import('./schema.js').Properties} Properties
   * @typedef {import('./schema.js').Normal} Normal
   *
   * @typedef {Record<string, string>} Attributes
   *
   * @typedef {Object} Definition
   * @property {Record<string, number|null>} properties
   * @property {(attributes: Attributes, property: string) => string} transform
   * @property {string} [space]
   * @property {Attributes} [attributes]
   * @property {Array<string>} [mustUseProperty]
   */

  const own$2 = {}.hasOwnProperty;

  /**
   * @param {Definition} definition
   * @returns {Schema}
   */
  function create(definition) {
    /** @type {Properties} */
    const property = {};
    /** @type {Normal} */
    const normal = {};
    /** @type {string} */
    let prop;

    for (prop in definition.properties) {
      if (own$2.call(definition.properties, prop)) {
        const value = definition.properties[prop];
        const info = new DefinedInfo(
          prop,
          definition.transform(definition.attributes || {}, prop),
          value,
          definition.space
        );

        if (
          definition.mustUseProperty &&
          definition.mustUseProperty.includes(prop)
        ) {
          info.mustUseProperty = true;
        }

        property[prop] = info;

        normal[normalize(prop)] = prop;
        normal[normalize(info.attribute)] = prop;
      }
    }

    return new Schema(property, normal, definition.space)
  }

  const xlink = create({
    space: 'xlink',
    transform(_, prop) {
      return 'xlink:' + prop.slice(5).toLowerCase()
    },
    properties: {
      xLinkActuate: null,
      xLinkArcRole: null,
      xLinkHref: null,
      xLinkRole: null,
      xLinkShow: null,
      xLinkTitle: null,
      xLinkType: null
    }
  });

  const xml = create({
    space: 'xml',
    transform(_, prop) {
      return 'xml:' + prop.slice(3).toLowerCase()
    },
    properties: {xmlLang: null, xmlBase: null, xmlSpace: null}
  });

  /**
   * @param {Record<string, string>} attributes
   * @param {string} attribute
   * @returns {string}
   */
  function caseSensitiveTransform(attributes, attribute) {
    return attribute in attributes ? attributes[attribute] : attribute
  }

  /**
   * @param {Record<string, string>} attributes
   * @param {string} property
   * @returns {string}
   */
  function caseInsensitiveTransform(attributes, property) {
    return caseSensitiveTransform(attributes, property.toLowerCase())
  }

  const xmlns = create({
    space: 'xmlns',
    attributes: {xmlnsxlink: 'xmlns:xlink'},
    transform: caseInsensitiveTransform,
    properties: {xmlns: null, xmlnsXLink: null}
  });

  const aria = create({
    transform(_, prop) {
      return prop === 'role' ? prop : 'aria-' + prop.slice(4).toLowerCase()
    },
    properties: {
      ariaActiveDescendant: null,
      ariaAtomic: booleanish,
      ariaAutoComplete: null,
      ariaBusy: booleanish,
      ariaChecked: booleanish,
      ariaColCount: number,
      ariaColIndex: number,
      ariaColSpan: number,
      ariaControls: spaceSeparated,
      ariaCurrent: null,
      ariaDescribedBy: spaceSeparated,
      ariaDetails: null,
      ariaDisabled: booleanish,
      ariaDropEffect: spaceSeparated,
      ariaErrorMessage: null,
      ariaExpanded: booleanish,
      ariaFlowTo: spaceSeparated,
      ariaGrabbed: booleanish,
      ariaHasPopup: null,
      ariaHidden: booleanish,
      ariaInvalid: null,
      ariaKeyShortcuts: null,
      ariaLabel: null,
      ariaLabelledBy: spaceSeparated,
      ariaLevel: number,
      ariaLive: null,
      ariaModal: booleanish,
      ariaMultiLine: booleanish,
      ariaMultiSelectable: booleanish,
      ariaOrientation: null,
      ariaOwns: spaceSeparated,
      ariaPlaceholder: null,
      ariaPosInSet: number,
      ariaPressed: booleanish,
      ariaReadOnly: booleanish,
      ariaRelevant: null,
      ariaRequired: booleanish,
      ariaRoleDescription: spaceSeparated,
      ariaRowCount: number,
      ariaRowIndex: number,
      ariaRowSpan: number,
      ariaSelected: booleanish,
      ariaSetSize: number,
      ariaSort: null,
      ariaValueMax: number,
      ariaValueMin: number,
      ariaValueNow: number,
      ariaValueText: null,
      role: null
    }
  });

  const html$1 = create({
    space: 'html',
    attributes: {
      acceptcharset: 'accept-charset',
      classname: 'class',
      htmlfor: 'for',
      httpequiv: 'http-equiv'
    },
    transform: caseInsensitiveTransform,
    mustUseProperty: ['checked', 'multiple', 'muted', 'selected'],
    properties: {
      // Standard Properties.
      abbr: null,
      accept: commaSeparated,
      acceptCharset: spaceSeparated,
      accessKey: spaceSeparated,
      action: null,
      allow: null,
      allowFullScreen: boolean,
      allowPaymentRequest: boolean,
      allowUserMedia: boolean,
      alt: null,
      as: null,
      async: boolean,
      autoCapitalize: null,
      autoComplete: spaceSeparated,
      autoFocus: boolean,
      autoPlay: boolean,
      capture: boolean,
      charSet: null,
      checked: boolean,
      cite: null,
      className: spaceSeparated,
      cols: number,
      colSpan: null,
      content: null,
      contentEditable: booleanish,
      controls: boolean,
      controlsList: spaceSeparated,
      coords: number | commaSeparated,
      crossOrigin: null,
      data: null,
      dateTime: null,
      decoding: null,
      default: boolean,
      defer: boolean,
      dir: null,
      dirName: null,
      disabled: boolean,
      download: overloadedBoolean,
      draggable: booleanish,
      encType: null,
      enterKeyHint: null,
      form: null,
      formAction: null,
      formEncType: null,
      formMethod: null,
      formNoValidate: boolean,
      formTarget: null,
      headers: spaceSeparated,
      height: number,
      hidden: boolean,
      high: number,
      href: null,
      hrefLang: null,
      htmlFor: spaceSeparated,
      httpEquiv: spaceSeparated,
      id: null,
      imageSizes: null,
      imageSrcSet: null,
      inputMode: null,
      integrity: null,
      is: null,
      isMap: boolean,
      itemId: null,
      itemProp: spaceSeparated,
      itemRef: spaceSeparated,
      itemScope: boolean,
      itemType: spaceSeparated,
      kind: null,
      label: null,
      lang: null,
      language: null,
      list: null,
      loading: null,
      loop: boolean,
      low: number,
      manifest: null,
      max: null,
      maxLength: number,
      media: null,
      method: null,
      min: null,
      minLength: number,
      multiple: boolean,
      muted: boolean,
      name: null,
      nonce: null,
      noModule: boolean,
      noValidate: boolean,
      onAbort: null,
      onAfterPrint: null,
      onAuxClick: null,
      onBeforePrint: null,
      onBeforeUnload: null,
      onBlur: null,
      onCancel: null,
      onCanPlay: null,
      onCanPlayThrough: null,
      onChange: null,
      onClick: null,
      onClose: null,
      onContextLost: null,
      onContextMenu: null,
      onContextRestored: null,
      onCopy: null,
      onCueChange: null,
      onCut: null,
      onDblClick: null,
      onDrag: null,
      onDragEnd: null,
      onDragEnter: null,
      onDragExit: null,
      onDragLeave: null,
      onDragOver: null,
      onDragStart: null,
      onDrop: null,
      onDurationChange: null,
      onEmptied: null,
      onEnded: null,
      onError: null,
      onFocus: null,
      onFormData: null,
      onHashChange: null,
      onInput: null,
      onInvalid: null,
      onKeyDown: null,
      onKeyPress: null,
      onKeyUp: null,
      onLanguageChange: null,
      onLoad: null,
      onLoadedData: null,
      onLoadedMetadata: null,
      onLoadEnd: null,
      onLoadStart: null,
      onMessage: null,
      onMessageError: null,
      onMouseDown: null,
      onMouseEnter: null,
      onMouseLeave: null,
      onMouseMove: null,
      onMouseOut: null,
      onMouseOver: null,
      onMouseUp: null,
      onOffline: null,
      onOnline: null,
      onPageHide: null,
      onPageShow: null,
      onPaste: null,
      onPause: null,
      onPlay: null,
      onPlaying: null,
      onPopState: null,
      onProgress: null,
      onRateChange: null,
      onRejectionHandled: null,
      onReset: null,
      onResize: null,
      onScroll: null,
      onSecurityPolicyViolation: null,
      onSeeked: null,
      onSeeking: null,
      onSelect: null,
      onSlotChange: null,
      onStalled: null,
      onStorage: null,
      onSubmit: null,
      onSuspend: null,
      onTimeUpdate: null,
      onToggle: null,
      onUnhandledRejection: null,
      onUnload: null,
      onVolumeChange: null,
      onWaiting: null,
      onWheel: null,
      open: boolean,
      optimum: number,
      pattern: null,
      ping: spaceSeparated,
      placeholder: null,
      playsInline: boolean,
      poster: null,
      preload: null,
      readOnly: boolean,
      referrerPolicy: null,
      rel: spaceSeparated,
      required: boolean,
      reversed: boolean,
      rows: number,
      rowSpan: number,
      sandbox: spaceSeparated,
      scope: null,
      scoped: boolean,
      seamless: boolean,
      selected: boolean,
      shape: null,
      size: number,
      sizes: null,
      slot: null,
      span: number,
      spellCheck: booleanish,
      src: null,
      srcDoc: null,
      srcLang: null,
      srcSet: null,
      start: number,
      step: null,
      style: null,
      tabIndex: number,
      target: null,
      title: null,
      translate: null,
      type: null,
      typeMustMatch: boolean,
      useMap: null,
      value: booleanish,
      width: number,
      wrap: null,

      // Legacy.
      // See: https://html.spec.whatwg.org/#other-elements,-attributes-and-apis
      align: null, // Several. Use CSS `text-align` instead,
      aLink: null, // `<body>`. Use CSS `a:active {color}` instead
      archive: spaceSeparated, // `<object>`. List of URIs to archives
      axis: null, // `<td>` and `<th>`. Use `scope` on `<th>`
      background: null, // `<body>`. Use CSS `background-image` instead
      bgColor: null, // `<body>` and table elements. Use CSS `background-color` instead
      border: number, // `<table>`. Use CSS `border-width` instead,
      borderColor: null, // `<table>`. Use CSS `border-color` instead,
      bottomMargin: number, // `<body>`
      cellPadding: null, // `<table>`
      cellSpacing: null, // `<table>`
      char: null, // Several table elements. When `align=char`, sets the character to align on
      charOff: null, // Several table elements. When `char`, offsets the alignment
      classId: null, // `<object>`
      clear: null, // `<br>`. Use CSS `clear` instead
      code: null, // `<object>`
      codeBase: null, // `<object>`
      codeType: null, // `<object>`
      color: null, // `<font>` and `<hr>`. Use CSS instead
      compact: boolean, // Lists. Use CSS to reduce space between items instead
      declare: boolean, // `<object>`
      event: null, // `<script>`
      face: null, // `<font>`. Use CSS instead
      frame: null, // `<table>`
      frameBorder: null, // `<iframe>`. Use CSS `border` instead
      hSpace: number, // `<img>` and `<object>`
      leftMargin: number, // `<body>`
      link: null, // `<body>`. Use CSS `a:link {color: *}` instead
      longDesc: null, // `<frame>`, `<iframe>`, and `<img>`. Use an `<a>`
      lowSrc: null, // `<img>`. Use a `<picture>`
      marginHeight: number, // `<body>`
      marginWidth: number, // `<body>`
      noResize: boolean, // `<frame>`
      noHref: boolean, // `<area>`. Use no href instead of an explicit `nohref`
      noShade: boolean, // `<hr>`. Use background-color and height instead of borders
      noWrap: boolean, // `<td>` and `<th>`
      object: null, // `<applet>`
      profile: null, // `<head>`
      prompt: null, // `<isindex>`
      rev: null, // `<link>`
      rightMargin: number, // `<body>`
      rules: null, // `<table>`
      scheme: null, // `<meta>`
      scrolling: booleanish, // `<frame>`. Use overflow in the child context
      standby: null, // `<object>`
      summary: null, // `<table>`
      text: null, // `<body>`. Use CSS `color` instead
      topMargin: number, // `<body>`
      valueType: null, // `<param>`
      version: null, // `<html>`. Use a doctype.
      vAlign: null, // Several. Use CSS `vertical-align` instead
      vLink: null, // `<body>`. Use CSS `a:visited {color}` instead
      vSpace: number, // `<img>` and `<object>`

      // Non-standard Properties.
      allowTransparency: null,
      autoCorrect: null,
      autoSave: null,
      disablePictureInPicture: boolean,
      disableRemotePlayback: boolean,
      prefix: null,
      property: null,
      results: number,
      security: null,
      unselectable: null
    }
  });

  const svg = create({
    space: 'svg',
    attributes: {
      accentHeight: 'accent-height',
      alignmentBaseline: 'alignment-baseline',
      arabicForm: 'arabic-form',
      baselineShift: 'baseline-shift',
      capHeight: 'cap-height',
      className: 'class',
      clipPath: 'clip-path',
      clipRule: 'clip-rule',
      colorInterpolation: 'color-interpolation',
      colorInterpolationFilters: 'color-interpolation-filters',
      colorProfile: 'color-profile',
      colorRendering: 'color-rendering',
      crossOrigin: 'crossorigin',
      dataType: 'datatype',
      dominantBaseline: 'dominant-baseline',
      enableBackground: 'enable-background',
      fillOpacity: 'fill-opacity',
      fillRule: 'fill-rule',
      floodColor: 'flood-color',
      floodOpacity: 'flood-opacity',
      fontFamily: 'font-family',
      fontSize: 'font-size',
      fontSizeAdjust: 'font-size-adjust',
      fontStretch: 'font-stretch',
      fontStyle: 'font-style',
      fontVariant: 'font-variant',
      fontWeight: 'font-weight',
      glyphName: 'glyph-name',
      glyphOrientationHorizontal: 'glyph-orientation-horizontal',
      glyphOrientationVertical: 'glyph-orientation-vertical',
      hrefLang: 'hreflang',
      horizAdvX: 'horiz-adv-x',
      horizOriginX: 'horiz-origin-x',
      horizOriginY: 'horiz-origin-y',
      imageRendering: 'image-rendering',
      letterSpacing: 'letter-spacing',
      lightingColor: 'lighting-color',
      markerEnd: 'marker-end',
      markerMid: 'marker-mid',
      markerStart: 'marker-start',
      navDown: 'nav-down',
      navDownLeft: 'nav-down-left',
      navDownRight: 'nav-down-right',
      navLeft: 'nav-left',
      navNext: 'nav-next',
      navPrev: 'nav-prev',
      navRight: 'nav-right',
      navUp: 'nav-up',
      navUpLeft: 'nav-up-left',
      navUpRight: 'nav-up-right',
      onAbort: 'onabort',
      onActivate: 'onactivate',
      onAfterPrint: 'onafterprint',
      onBeforePrint: 'onbeforeprint',
      onBegin: 'onbegin',
      onCancel: 'oncancel',
      onCanPlay: 'oncanplay',
      onCanPlayThrough: 'oncanplaythrough',
      onChange: 'onchange',
      onClick: 'onclick',
      onClose: 'onclose',
      onCopy: 'oncopy',
      onCueChange: 'oncuechange',
      onCut: 'oncut',
      onDblClick: 'ondblclick',
      onDrag: 'ondrag',
      onDragEnd: 'ondragend',
      onDragEnter: 'ondragenter',
      onDragExit: 'ondragexit',
      onDragLeave: 'ondragleave',
      onDragOver: 'ondragover',
      onDragStart: 'ondragstart',
      onDrop: 'ondrop',
      onDurationChange: 'ondurationchange',
      onEmptied: 'onemptied',
      onEnd: 'onend',
      onEnded: 'onended',
      onError: 'onerror',
      onFocus: 'onfocus',
      onFocusIn: 'onfocusin',
      onFocusOut: 'onfocusout',
      onHashChange: 'onhashchange',
      onInput: 'oninput',
      onInvalid: 'oninvalid',
      onKeyDown: 'onkeydown',
      onKeyPress: 'onkeypress',
      onKeyUp: 'onkeyup',
      onLoad: 'onload',
      onLoadedData: 'onloadeddata',
      onLoadedMetadata: 'onloadedmetadata',
      onLoadStart: 'onloadstart',
      onMessage: 'onmessage',
      onMouseDown: 'onmousedown',
      onMouseEnter: 'onmouseenter',
      onMouseLeave: 'onmouseleave',
      onMouseMove: 'onmousemove',
      onMouseOut: 'onmouseout',
      onMouseOver: 'onmouseover',
      onMouseUp: 'onmouseup',
      onMouseWheel: 'onmousewheel',
      onOffline: 'onoffline',
      onOnline: 'ononline',
      onPageHide: 'onpagehide',
      onPageShow: 'onpageshow',
      onPaste: 'onpaste',
      onPause: 'onpause',
      onPlay: 'onplay',
      onPlaying: 'onplaying',
      onPopState: 'onpopstate',
      onProgress: 'onprogress',
      onRateChange: 'onratechange',
      onRepeat: 'onrepeat',
      onReset: 'onreset',
      onResize: 'onresize',
      onScroll: 'onscroll',
      onSeeked: 'onseeked',
      onSeeking: 'onseeking',
      onSelect: 'onselect',
      onShow: 'onshow',
      onStalled: 'onstalled',
      onStorage: 'onstorage',
      onSubmit: 'onsubmit',
      onSuspend: 'onsuspend',
      onTimeUpdate: 'ontimeupdate',
      onToggle: 'ontoggle',
      onUnload: 'onunload',
      onVolumeChange: 'onvolumechange',
      onWaiting: 'onwaiting',
      onZoom: 'onzoom',
      overlinePosition: 'overline-position',
      overlineThickness: 'overline-thickness',
      paintOrder: 'paint-order',
      panose1: 'panose-1',
      pointerEvents: 'pointer-events',
      referrerPolicy: 'referrerpolicy',
      renderingIntent: 'rendering-intent',
      shapeRendering: 'shape-rendering',
      stopColor: 'stop-color',
      stopOpacity: 'stop-opacity',
      strikethroughPosition: 'strikethrough-position',
      strikethroughThickness: 'strikethrough-thickness',
      strokeDashArray: 'stroke-dasharray',
      strokeDashOffset: 'stroke-dashoffset',
      strokeLineCap: 'stroke-linecap',
      strokeLineJoin: 'stroke-linejoin',
      strokeMiterLimit: 'stroke-miterlimit',
      strokeOpacity: 'stroke-opacity',
      strokeWidth: 'stroke-width',
      tabIndex: 'tabindex',
      textAnchor: 'text-anchor',
      textDecoration: 'text-decoration',
      textRendering: 'text-rendering',
      typeOf: 'typeof',
      underlinePosition: 'underline-position',
      underlineThickness: 'underline-thickness',
      unicodeBidi: 'unicode-bidi',
      unicodeRange: 'unicode-range',
      unitsPerEm: 'units-per-em',
      vAlphabetic: 'v-alphabetic',
      vHanging: 'v-hanging',
      vIdeographic: 'v-ideographic',
      vMathematical: 'v-mathematical',
      vectorEffect: 'vector-effect',
      vertAdvY: 'vert-adv-y',
      vertOriginX: 'vert-origin-x',
      vertOriginY: 'vert-origin-y',
      wordSpacing: 'word-spacing',
      writingMode: 'writing-mode',
      xHeight: 'x-height',
      // These were camelcased in Tiny. Now lowercased in SVG 2
      playbackOrder: 'playbackorder',
      timelineBegin: 'timelinebegin'
    },
    transform: caseSensitiveTransform,
    properties: {
      about: commaOrSpaceSeparated,
      accentHeight: number,
      accumulate: null,
      additive: null,
      alignmentBaseline: null,
      alphabetic: number,
      amplitude: number,
      arabicForm: null,
      ascent: number,
      attributeName: null,
      attributeType: null,
      azimuth: number,
      bandwidth: null,
      baselineShift: null,
      baseFrequency: null,
      baseProfile: null,
      bbox: null,
      begin: null,
      bias: number,
      by: null,
      calcMode: null,
      capHeight: number,
      className: spaceSeparated,
      clip: null,
      clipPath: null,
      clipPathUnits: null,
      clipRule: null,
      color: null,
      colorInterpolation: null,
      colorInterpolationFilters: null,
      colorProfile: null,
      colorRendering: null,
      content: null,
      contentScriptType: null,
      contentStyleType: null,
      crossOrigin: null,
      cursor: null,
      cx: null,
      cy: null,
      d: null,
      dataType: null,
      defaultAction: null,
      descent: number,
      diffuseConstant: number,
      direction: null,
      display: null,
      dur: null,
      divisor: number,
      dominantBaseline: null,
      download: boolean,
      dx: null,
      dy: null,
      edgeMode: null,
      editable: null,
      elevation: number,
      enableBackground: null,
      end: null,
      event: null,
      exponent: number,
      externalResourcesRequired: null,
      fill: null,
      fillOpacity: number,
      fillRule: null,
      filter: null,
      filterRes: null,
      filterUnits: null,
      floodColor: null,
      floodOpacity: null,
      focusable: null,
      focusHighlight: null,
      fontFamily: null,
      fontSize: null,
      fontSizeAdjust: null,
      fontStretch: null,
      fontStyle: null,
      fontVariant: null,
      fontWeight: null,
      format: null,
      fr: null,
      from: null,
      fx: null,
      fy: null,
      g1: commaSeparated,
      g2: commaSeparated,
      glyphName: commaSeparated,
      glyphOrientationHorizontal: null,
      glyphOrientationVertical: null,
      glyphRef: null,
      gradientTransform: null,
      gradientUnits: null,
      handler: null,
      hanging: number,
      hatchContentUnits: null,
      hatchUnits: null,
      height: null,
      href: null,
      hrefLang: null,
      horizAdvX: number,
      horizOriginX: number,
      horizOriginY: number,
      id: null,
      ideographic: number,
      imageRendering: null,
      initialVisibility: null,
      in: null,
      in2: null,
      intercept: number,
      k: number,
      k1: number,
      k2: number,
      k3: number,
      k4: number,
      kernelMatrix: commaOrSpaceSeparated,
      kernelUnitLength: null,
      keyPoints: null, // SEMI_COLON_SEPARATED
      keySplines: null, // SEMI_COLON_SEPARATED
      keyTimes: null, // SEMI_COLON_SEPARATED
      kerning: null,
      lang: null,
      lengthAdjust: null,
      letterSpacing: null,
      lightingColor: null,
      limitingConeAngle: number,
      local: null,
      markerEnd: null,
      markerMid: null,
      markerStart: null,
      markerHeight: null,
      markerUnits: null,
      markerWidth: null,
      mask: null,
      maskContentUnits: null,
      maskUnits: null,
      mathematical: null,
      max: null,
      media: null,
      mediaCharacterEncoding: null,
      mediaContentEncodings: null,
      mediaSize: number,
      mediaTime: null,
      method: null,
      min: null,
      mode: null,
      name: null,
      navDown: null,
      navDownLeft: null,
      navDownRight: null,
      navLeft: null,
      navNext: null,
      navPrev: null,
      navRight: null,
      navUp: null,
      navUpLeft: null,
      navUpRight: null,
      numOctaves: null,
      observer: null,
      offset: null,
      onAbort: null,
      onActivate: null,
      onAfterPrint: null,
      onBeforePrint: null,
      onBegin: null,
      onCancel: null,
      onCanPlay: null,
      onCanPlayThrough: null,
      onChange: null,
      onClick: null,
      onClose: null,
      onCopy: null,
      onCueChange: null,
      onCut: null,
      onDblClick: null,
      onDrag: null,
      onDragEnd: null,
      onDragEnter: null,
      onDragExit: null,
      onDragLeave: null,
      onDragOver: null,
      onDragStart: null,
      onDrop: null,
      onDurationChange: null,
      onEmptied: null,
      onEnd: null,
      onEnded: null,
      onError: null,
      onFocus: null,
      onFocusIn: null,
      onFocusOut: null,
      onHashChange: null,
      onInput: null,
      onInvalid: null,
      onKeyDown: null,
      onKeyPress: null,
      onKeyUp: null,
      onLoad: null,
      onLoadedData: null,
      onLoadedMetadata: null,
      onLoadStart: null,
      onMessage: null,
      onMouseDown: null,
      onMouseEnter: null,
      onMouseLeave: null,
      onMouseMove: null,
      onMouseOut: null,
      onMouseOver: null,
      onMouseUp: null,
      onMouseWheel: null,
      onOffline: null,
      onOnline: null,
      onPageHide: null,
      onPageShow: null,
      onPaste: null,
      onPause: null,
      onPlay: null,
      onPlaying: null,
      onPopState: null,
      onProgress: null,
      onRateChange: null,
      onRepeat: null,
      onReset: null,
      onResize: null,
      onScroll: null,
      onSeeked: null,
      onSeeking: null,
      onSelect: null,
      onShow: null,
      onStalled: null,
      onStorage: null,
      onSubmit: null,
      onSuspend: null,
      onTimeUpdate: null,
      onToggle: null,
      onUnload: null,
      onVolumeChange: null,
      onWaiting: null,
      onZoom: null,
      opacity: null,
      operator: null,
      order: null,
      orient: null,
      orientation: null,
      origin: null,
      overflow: null,
      overlay: null,
      overlinePosition: number,
      overlineThickness: number,
      paintOrder: null,
      panose1: null,
      path: null,
      pathLength: number,
      patternContentUnits: null,
      patternTransform: null,
      patternUnits: null,
      phase: null,
      ping: spaceSeparated,
      pitch: null,
      playbackOrder: null,
      pointerEvents: null,
      points: null,
      pointsAtX: number,
      pointsAtY: number,
      pointsAtZ: number,
      preserveAlpha: null,
      preserveAspectRatio: null,
      primitiveUnits: null,
      propagate: null,
      property: commaOrSpaceSeparated,
      r: null,
      radius: null,
      referrerPolicy: null,
      refX: null,
      refY: null,
      rel: commaOrSpaceSeparated,
      rev: commaOrSpaceSeparated,
      renderingIntent: null,
      repeatCount: null,
      repeatDur: null,
      requiredExtensions: commaOrSpaceSeparated,
      requiredFeatures: commaOrSpaceSeparated,
      requiredFonts: commaOrSpaceSeparated,
      requiredFormats: commaOrSpaceSeparated,
      resource: null,
      restart: null,
      result: null,
      rotate: null,
      rx: null,
      ry: null,
      scale: null,
      seed: null,
      shapeRendering: null,
      side: null,
      slope: null,
      snapshotTime: null,
      specularConstant: number,
      specularExponent: number,
      spreadMethod: null,
      spacing: null,
      startOffset: null,
      stdDeviation: null,
      stemh: null,
      stemv: null,
      stitchTiles: null,
      stopColor: null,
      stopOpacity: null,
      strikethroughPosition: number,
      strikethroughThickness: number,
      string: null,
      stroke: null,
      strokeDashArray: commaOrSpaceSeparated,
      strokeDashOffset: null,
      strokeLineCap: null,
      strokeLineJoin: null,
      strokeMiterLimit: number,
      strokeOpacity: number,
      strokeWidth: null,
      style: null,
      surfaceScale: number,
      syncBehavior: null,
      syncBehaviorDefault: null,
      syncMaster: null,
      syncTolerance: null,
      syncToleranceDefault: null,
      systemLanguage: commaOrSpaceSeparated,
      tabIndex: number,
      tableValues: null,
      target: null,
      targetX: number,
      targetY: number,
      textAnchor: null,
      textDecoration: null,
      textRendering: null,
      textLength: null,
      timelineBegin: null,
      title: null,
      transformBehavior: null,
      type: null,
      typeOf: commaOrSpaceSeparated,
      to: null,
      transform: null,
      u1: null,
      u2: null,
      underlinePosition: number,
      underlineThickness: number,
      unicode: null,
      unicodeBidi: null,
      unicodeRange: null,
      unitsPerEm: number,
      values: null,
      vAlphabetic: number,
      vMathematical: number,
      vectorEffect: null,
      vHanging: number,
      vIdeographic: number,
      version: null,
      vertAdvY: number,
      vertOriginX: number,
      vertOriginY: number,
      viewBox: null,
      viewTarget: null,
      visibility: null,
      width: null,
      widths: null,
      wordSpacing: null,
      writingMode: null,
      x: null,
      x1: null,
      x2: null,
      xChannelSelector: null,
      xHeight: number,
      y: null,
      y1: null,
      y2: null,
      yChannelSelector: null,
      z: null,
      zoomAndPan: null
    }
  });

  /**
   * @typedef {import('./util/schema.js').Schema} Schema
   */

  const valid = /^data[-\w.:]+$/i;
  const dash = /-[a-z]/g;
  const cap = /[A-Z]/g;

  /**
   * @param {Schema} schema
   * @param {string} value
   * @returns {Info}
   */
  function find(schema, value) {
    const normal = normalize(value);
    let prop = value;
    let Type = Info;

    if (normal in schema.normal) {
      return schema.property[schema.normal[normal]]
    }

    if (normal.length > 4 && normal.slice(0, 4) === 'data' && valid.test(value)) {
      // Attribute or property.
      if (value.charAt(4) === '-') {
        // Turn it into a property.
        const rest = value.slice(5).replace(dash, camelcase);
        prop = 'data' + rest.charAt(0).toUpperCase() + rest.slice(1);
      } else {
        // Turn it into an attribute.
        const rest = value.slice(4);

        if (!dash.test(rest)) {
          let dashes = rest.replace(cap, kebab);

          if (dashes.charAt(0) !== '-') {
            dashes = '-' + dashes;
          }

          value = 'data' + dashes;
        }
      }

      Type = DefinedInfo;
    }

    return new Type(prop, value)
  }

  /**
   * @param {string} $0
   * @returns {string}
   */
  function kebab($0) {
    return '-' + $0.toLowerCase()
  }

  /**
   * @param {string} $0
   * @returns {string}
   */
  function camelcase($0) {
    return $0.charAt(1).toUpperCase()
  }

  /**
   * @typedef {import('./lib/util/info.js').Info} Info
   * @typedef {import('./lib/util/schema.js').Schema} Schema
   */
  const html = merge([xml, xlink, xmlns, aria, html$1], 'html');
  merge([xml, xlink, xmlns, aria, svg], 'svg');

  /**
   * @typedef {import('hast').Properties} Properties
   * @typedef {import('hast').Element} Element
   */

  var search = /[#.]/g;

  /**
   * Create a hast element from a simple CSS selector.
   *
   * @param selector A simple CSS selector.
   *   Can contain a tag-name (`foo`), classes (`.bar`), and an ID (`#baz`).
   *   Multiple classes are allowed.
   *   Uses the last ID if multiple IDs are found.
   * @param [defaultTagName='div'] Tag name to use if `selector` does not specify one.
   */
  const parseSelector =
    /**
     * @type {(
     *  <Selector extends string, DefaultTagName extends string = 'div'>(selector?: Selector, defaultTagName?: DefaultTagName) => Element & {tagName: import('./extract.js').ExtractTagName<Selector, DefaultTagName>}
     * )}
     */
    (
      /**
       * @param {string} [selector]
       * @param {string} [defaultTagName='div']
       * @returns {Element}
       */
      function (selector, defaultTagName = 'div') {
        var value = selector || '';
        /** @type {Properties} */
        var props = {};
        var start = 0;
        /** @type {string} */
        var subvalue;
        /** @type {string} */
        var previous;
        /** @type {RegExpMatchArray} */
        var match;

        while (start < value.length) {
          search.lastIndex = start;
          match = search.exec(value);
          subvalue = value.slice(start, match ? match.index : value.length);

          if (subvalue) {
            if (!previous) {
              defaultTagName = subvalue;
            } else if (previous === '#') {
              props.id = subvalue;
            } else if (Array.isArray(props.className)) {
              props.className.push(subvalue);
            } else {
              props.className = [subvalue];
            }

            start += subvalue.length;
          }

          if (match) {
            previous = match[0];
            start++;
          }
        }

        return {
          type: 'element',
          tagName: defaultTagName,
          properties: props,
          children: []
        }
      }
    );

  /**
   * Parse space separated tokens to an array of strings.
   *
   * @param {string} value Space separated tokens
   * @returns {Array.<string>} Tokens
   */
  function parse$1(value) {
    const input = String(value || '').trim();
    return input ? input.split(/[ \t\n\r\f]+/g) : []
  }

  /**
   * @typedef {Object} StringifyOptions
   * @property {boolean} [padLeft=true] Whether to pad a space before a token (`boolean`, default: `true`).
   * @property {boolean} [padRight=false] Whether to pad a space after a token (`boolean`, default: `false`).
   */

  /**
   * Parse comma separated tokens to an array.
   *
   * @param {string} value
   * @returns {Array.<string>}
   */
  function parse(value) {
    /** @type {Array.<string>} */
    var tokens = [];
    var input = String(value || '');
    var index = input.indexOf(',');
    var start = 0;
    /** @type {boolean} */
    var end;
    /** @type {string} */
    var token;

    while (!end) {
      if (index === -1) {
        index = input.length;
        end = true;
      }

      token = input.slice(start, index).trim();

      if (token || !end) {
        tokens.push(token);
      }

      start = index + 1;
      index = input.indexOf(',', start);
    }

    return tokens
  }

  /**
   * @typedef {import('hast').Root} Root
   * @typedef {import('hast').Element} Element
   * @typedef {import('hast').Properties} Properties
   * @typedef {Root['children'][number]} Child
   * @typedef {Child|Root} Node
   * @typedef {import('property-information').Info} Info
   * @typedef {import('property-information').Schema} Schema
   *
   * @typedef {Root|Element} HResult
   * @typedef {string|number} HStyleValue
   * @typedef {Object.<string, HStyleValue>} HStyle
   * @typedef {string|number|boolean|null|undefined} HPrimitiveValue
   * @typedef {Array.<string|number>} HArrayValue
   * @typedef {HPrimitiveValue|HArrayValue} HPropertyValue
   * @typedef {{[property: string]: HPropertyValue|HStyle}} HProperties
   *
   * @typedef {string|number|null|undefined} HPrimitiveChild
   * @typedef {Array.<Node|HPrimitiveChild>} HArrayChild
   * @typedef {Node|HPrimitiveChild|HArrayChild} HChild
   */

  const buttonTypes = new Set(['menu', 'submit', 'reset', 'button']);

  const own$1 = {}.hasOwnProperty;

  /**
   * @param {Schema} schema
   * @param {string} defaultTagName
   * @param {Array.<string>} [caseSensitive]
   */
  function core(schema, defaultTagName, caseSensitive) {
    const adjust = caseSensitive && createAdjustMap(caseSensitive);

    const h =
      /**
       * @type {{
       *   (): Root
       *   (selector: null|undefined, ...children: HChild[]): Root
       *   (selector: string, properties?: HProperties, ...children: HChild[]): Element
       *   (selector: string, ...children: HChild[]): Element
       * }}
       */
      (
        /**
         * Hyperscript compatible DSL for creating virtual hast trees.
         *
         * @param {string|null} [selector]
         * @param {HProperties|HChild} [properties]
         * @param {HChild[]} children
         * @returns {HResult}
         */
        function (selector, properties, ...children) {
          let index = -1;
          /** @type {HResult} */
          let node;

          if (selector === undefined || selector === null) {
            node = {type: 'root', children: []};
            // @ts-expect-error Properties are not supported for roots.
            children.unshift(properties);
          } else {
            node = parseSelector(selector, defaultTagName);
            // Normalize the name.
            node.tagName = node.tagName.toLowerCase();
            if (adjust && own$1.call(adjust, node.tagName)) {
              node.tagName = adjust[node.tagName];
            }

            // Handle props.
            if (isProperties(properties, node.tagName)) {
              /** @type {string} */
              let key;

              for (key in properties) {
                if (own$1.call(properties, key)) {
                  // @ts-expect-error `node.properties` is set.
                  addProperty(schema, node.properties, key, properties[key]);
                }
              }
            } else {
              children.unshift(properties);
            }
          }

          // Handle children.
          while (++index < children.length) {
            addChild(node.children, children[index]);
          }

          if (node.type === 'element' && node.tagName === 'template') {
            node.content = {type: 'root', children: node.children};
            node.children = [];
          }

          return node
        }
      );

    return h
  }

  /**
   * @param {HProperties|HChild} value
   * @param {string} name
   * @returns {value is HProperties}
   */
  function isProperties(value, name) {
    if (
      value === null ||
      value === undefined ||
      typeof value !== 'object' ||
      Array.isArray(value)
    ) {
      return false
    }

    if (name === 'input' || !value.type || typeof value.type !== 'string') {
      return true
    }

    if ('children' in value && Array.isArray(value.children)) {
      return false
    }

    if (name === 'button') {
      return buttonTypes.has(value.type.toLowerCase())
    }

    return !('value' in value)
  }

  /**
   * @param {Schema} schema
   * @param {Properties} properties
   * @param {string} key
   * @param {HStyle|HPropertyValue} value
   * @returns {void}
   */
  function addProperty(schema, properties, key, value) {
    const info = find(schema, key);
    let index = -1;
    /** @type {HPropertyValue} */
    let result;

    // Ignore nullish and NaN values.
    if (value === undefined || value === null) return

    if (typeof value === 'number') {
      // Ignore NaN.
      if (Number.isNaN(value)) return

      result = value;
    }
    // Booleans.
    else if (typeof value === 'boolean') {
      result = value;
    }
    // Handle list values.
    else if (typeof value === 'string') {
      if (info.spaceSeparated) {
        result = parse$1(value);
      } else if (info.commaSeparated) {
        result = parse(value);
      } else if (info.commaOrSpaceSeparated) {
        result = parse$1(parse(value).join(' '));
      } else {
        result = parsePrimitive(info, info.property, value);
      }
    } else if (Array.isArray(value)) {
      result = value.concat();
    } else {
      result = info.property === 'style' ? style(value) : String(value);
    }

    if (Array.isArray(result)) {
      /** @type {Array.<string|number>} */
      const finalResult = [];

      while (++index < result.length) {
        // @ts-expect-error Assume no booleans in array.
        finalResult[index] = parsePrimitive(info, info.property, result[index]);
      }

      result = finalResult;
    }

    // Class names (which can be added both on the `selector` and here).
    if (info.property === 'className' && Array.isArray(properties.className)) {
      // @ts-expect-error Assume no booleans in `className`.
      result = properties.className.concat(result);
    }

    properties[info.property] = result;
  }

  /**
   * @param {Array.<Child>} nodes
   * @param {HChild} value
   * @returns {void}
   */
  function addChild(nodes, value) {
    let index = -1;

    if (value === undefined || value === null) ; else if (typeof value === 'string' || typeof value === 'number') {
      nodes.push({type: 'text', value: String(value)});
    } else if (Array.isArray(value)) {
      while (++index < value.length) {
        addChild(nodes, value[index]);
      }
    } else if (typeof value === 'object' && 'type' in value) {
      if (value.type === 'root') {
        addChild(nodes, value.children);
      } else {
        nodes.push(value);
      }
    } else {
      throw new Error('Expected node, nodes, or string, got `' + value + '`')
    }
  }

  /**
   * Parse a single primitives.
   *
   * @param {Info} info
   * @param {string} name
   * @param {HPrimitiveValue} value
   * @returns {HPrimitiveValue}
   */
  function parsePrimitive(info, name, value) {
    if (typeof value === 'string') {
      if (info.number && value && !Number.isNaN(Number(value))) {
        return Number(value)
      }

      if (
        (info.boolean || info.overloadedBoolean) &&
        (value === '' || normalize(value) === normalize(name))
      ) {
        return true
      }
    }

    return value
  }

  /**
   * @param {HStyle} value
   * @returns {string}
   */
  function style(value) {
    /** @type {Array.<string>} */
    const result = [];
    /** @type {string} */
    let key;

    for (key in value) {
      if (own$1.call(value, key)) {
        result.push([key, value[key]].join(': '));
      }
    }

    return result.join('; ')
  }

  /**
   * @param {Array.<string>} values
   * @returns {Object.<string, string>}
   */
  function createAdjustMap(values) {
    /** @type {Object.<string, string>} */
    const result = {};
    let index = -1;

    while (++index < values.length) {
      result[values[index].toLowerCase()] = values[index];
    }

    return result
  }

  /**
   * @typedef {import('./core.js').HChild} Child Acceptable child value
   * @typedef {import('./core.js').HProperties} Properties Acceptable properties value.
   *
   * @typedef {import('./jsx-classic').Element} h.JSX.Element
   * @typedef {import('./jsx-classic').IntrinsicAttributes} h.JSX.IntrinsicAttributes
   * @typedef {import('./jsx-classic').IntrinsicElements} h.JSX.IntrinsicElements
   * @typedef {import('./jsx-classic').ElementChildrenAttribute} h.JSX.ElementChildrenAttribute
   */

  const h = core(html, 'div');

  /**
   * Map of invalid numeric character references to their replacements, according to HTML.
   *
   * @type {Record<number, string>}
   */
  const characterReferenceInvalid = {
    0: '�',
    128: '€',
    130: '‚',
    131: 'ƒ',
    132: '„',
    133: '…',
    134: '†',
    135: '‡',
    136: 'ˆ',
    137: '‰',
    138: 'Š',
    139: '‹',
    140: 'Œ',
    142: 'Ž',
    145: '‘',
    146: '’',
    147: '“',
    148: '”',
    149: '•',
    150: '–',
    151: '—',
    152: '˜',
    153: '™',
    154: 'š',
    155: '›',
    156: 'œ',
    158: 'ž',
    159: 'Ÿ'
  };

  /**
   * Check if the given character code, or the character code at the first
   * character, is decimal.
   *
   * @param {string|number} character
   * @returns {boolean} Whether `character` is a decimal
   */
  function isDecimal(character) {
    const code =
      typeof character === 'string' ? character.charCodeAt(0) : character;

    return code >= 48 && code <= 57 /* 0-9 */
  }

  /**
   * Check if the given character code, or the character code at the first
   * character, is hexadecimal.
   *
   * @param {string|number} character
   * @returns {boolean} Whether `character` is hexadecimal
   */
  function isHexadecimal(character) {
    const code =
      typeof character === 'string' ? character.charCodeAt(0) : character;

    return (
      (code >= 97 /* a */ && code <= 102) /* z */ ||
      (code >= 65 /* A */ && code <= 70) /* Z */ ||
      (code >= 48 /* A */ && code <= 57) /* Z */
    )
  }

  /**
   * Check if the given character code, or the character code at the first
   * character, is alphabetical.
   *
   * @param {string|number} character
   * @returns {boolean} Whether `character` is alphabetical.
   */
  function isAlphabetical(character) {
    const code =
      typeof character === 'string' ? character.charCodeAt(0) : character;

    return (
      (code >= 97 && code <= 122) /* a-z */ ||
      (code >= 65 && code <= 90) /* A-Z */
    )
  }

  /**
   * Check if the given character code, or the character code at the first
   * character, is alphanumerical.
   *
   * @param {string|number} character
   * @returns {boolean} Whether `character` is alphanumerical.
   */
  function isAlphanumerical(character) {
    return isAlphabetical(character) || isDecimal(character)
  }

  /**
   * @typedef {import('unist').Point} Point
   * @typedef {import('unist').Position} Position
   */

  const fromCharCode = String.fromCharCode;

  // Warning messages.
  const messages = [
    '',
    /* 1: Non terminated (named) */
    'Named character references must be terminated by a semicolon',
    /* 2: Non terminated (numeric) */
    'Numeric character references must be terminated by a semicolon',
    /* 3: Empty (named) */
    'Named character references cannot be empty',
    /* 4: Empty (numeric) */
    'Numeric character references cannot be empty',
    /* 5: Unknown (named) */
    'Named character references must be known',
    /* 6: Disallowed (numeric) */
    'Numeric character references cannot be disallowed',
    /* 7: Prohibited (numeric) */
    'Numeric character references cannot be outside the permissible Unicode range'
  ];

  /**
   * Parse HTML character references.
   *
   * @param {string} value
   * @param {import('../index.js').Options} [options={}]
   */
  function parseEntities(value, options = {}) {
    const additional =
      typeof options.additional === 'string'
        ? options.additional.charCodeAt(0)
        : options.additional;
    /** @type {string[]} */
    const result = [];
    let index = 0;
    let lines = -1;
    let queue = '';
    /** @type {Point|undefined} */
    let point;
    /** @type {number[]|undefined} */
    let indent;

    if (options.position) {
      if ('start' in options.position || 'indent' in options.position) {
        indent = options.position.indent;
        point = options.position.start;
      } else {
        point = options.position;
      }
    }

    let line = (point ? point.line : 0) || 1;
    let column = (point ? point.column : 0) || 1;

    // Cache the current point.
    let previous = now();
    /** @type {number|undefined} */
    let character;

    // Ensure the algorithm walks over the first character (inclusive).
    index--;

    while (++index <= value.length) {
      // If the previous character was a newline.
      if (character === 10 /* `\n` */) {
        column = (indent ? indent[lines] : 0) || 1;
      }

      character = value.charCodeAt(index);

      if (character === 38 /* `&` */) {
        const following = value.charCodeAt(index + 1);

        // The behavior depends on the identity of the next character.
        if (
          following === 9 /* `\t` */ ||
          following === 10 /* `\n` */ ||
          following === 12 /* `\f` */ ||
          following === 32 /* ` ` */ ||
          following === 38 /* `&` */ ||
          following === 60 /* `<` */ ||
          Number.isNaN(following) ||
          (additional && following === additional)
        ) {
          // Not a character reference.
          // No characters are consumed, and nothing is returned.
          // This is not an error, either.
          queue += fromCharCode(character);
          column++;
          continue
        }

        const start = index + 1;
        let begin = start;
        let end = start;
        /** @type {string} */
        let type;

        if (following === 35 /* `#` */) {
          // Numerical reference.
          end = ++begin;

          // The behavior further depends on the next character.
          const following = value.charCodeAt(end);

          if (following === 88 /* `X` */ || following === 120 /* `x` */) {
            // ASCII hexadecimal digits.
            type = 'hexadecimal';
            end = ++begin;
          } else {
            // ASCII decimal digits.
            type = 'decimal';
          }
        } else {
          // Named reference.
          type = 'named';
        }

        let characterReferenceCharacters = '';
        let characterReference = '';
        let characters = '';
        // Each type of character reference accepts different characters.
        // This test is used to detect whether a reference has ended (as the semicolon
        // is not strictly needed).
        const test =
          type === 'named'
            ? isAlphanumerical
            : type === 'decimal'
            ? isDecimal
            : isHexadecimal;

        end--;

        while (++end <= value.length) {
          const following = value.charCodeAt(end);

          if (!test(following)) {
            break
          }

          characters += fromCharCode(following);

          // Check if we can match a legacy named reference.
          // If so, we cache that as the last viable named reference.
          // This ensures we do not need to walk backwards later.
          if (type === 'named' && characterEntitiesLegacy.includes(characters)) {
            characterReferenceCharacters = characters;
            // @ts-expect-error: always able to decode.
            characterReference = decodeNamedCharacterReference(characters);
          }
        }

        let terminated = value.charCodeAt(end) === 59; /* `;` */

        if (terminated) {
          end++;

          const namedReference =
            type === 'named' ? decodeNamedCharacterReference(characters) : false;

          if (namedReference) {
            characterReferenceCharacters = characters;
            characterReference = namedReference;
          }
        }

        let diff = 1 + end - start;
        let reference = '';

        if (!terminated && options.nonTerminated === false) ; else if (!characters) {
          // An empty (possible) reference is valid, unless it’s numeric (thus an
          // ampersand followed by an octothorp).
          if (type !== 'named') {
            warning(4 /* Empty (numeric) */, diff);
          }
        } else if (type === 'named') {
          // An ampersand followed by anything unknown, and not terminated, is
          // invalid.
          if (terminated && !characterReference) {
            warning(5 /* Unknown (named) */, 1);
          } else {
            // If there’s something after an named reference which is not known,
            // cap the reference.
            if (characterReferenceCharacters !== characters) {
              end = begin + characterReferenceCharacters.length;
              diff = 1 + end - begin;
              terminated = false;
            }

            // If the reference is not terminated, warn.
            if (!terminated) {
              const reason = characterReferenceCharacters
                ? 1 /* Non terminated (named) */
                : 3; /* Empty (named) */

              if (options.attribute) {
                const following = value.charCodeAt(end);

                if (following === 61 /* `=` */) {
                  warning(reason, diff);
                  characterReference = '';
                } else if (isAlphanumerical(following)) {
                  characterReference = '';
                } else {
                  warning(reason, diff);
                }
              } else {
                warning(reason, diff);
              }
            }
          }

          reference = characterReference;
        } else {
          if (!terminated) {
            // All nonterminated numeric references are not rendered, and emit a
            // warning.
            warning(2 /* Non terminated (numeric) */, diff);
          }

          // When terminated and numerical, parse as either hexadecimal or
          // decimal.
          let referenceCode = Number.parseInt(
            characters,
            type === 'hexadecimal' ? 16 : 10
          );

          // Emit a warning when the parsed number is prohibited, and replace with
          // replacement character.
          if (prohibited(referenceCode)) {
            warning(7 /* Prohibited (numeric) */, diff);
            reference = fromCharCode(65533 /* `�` */);
          } else if (referenceCode in characterReferenceInvalid) {
            // Emit a warning when the parsed number is disallowed, and replace by
            // an alternative.
            warning(6 /* Disallowed (numeric) */, diff);
            reference = characterReferenceInvalid[referenceCode];
          } else {
            // Parse the number.
            let output = '';

            // Emit a warning when the parsed number should not be used.
            if (disallowed(referenceCode)) {
              warning(6 /* Disallowed (numeric) */, diff);
            }

            // Serialize the number.
            if (referenceCode > 0xffff) {
              referenceCode -= 0x10000;
              output += fromCharCode((referenceCode >>> (10 & 0x3ff)) | 0xd800);
              referenceCode = 0xdc00 | (referenceCode & 0x3ff);
            }

            reference = output + fromCharCode(referenceCode);
          }
        }

        // Found it!
        // First eat the queued characters as normal text, then eat a reference.
        if (reference) {
          flush();

          previous = now();
          index = end - 1;
          column += end - start + 1;
          result.push(reference);
          const next = now();
          next.offset++;

          if (options.reference) {
            options.reference.call(
              options.referenceContext,
              reference,
              {start: previous, end: next},
              value.slice(start - 1, end)
            );
          }

          previous = next;
        } else {
          // If we could not find a reference, queue the checked characters (as
          // normal characters), and move the pointer to their end.
          // This is possible because we can be certain neither newlines nor
          // ampersands are included.
          characters = value.slice(start - 1, end);
          queue += characters;
          column += characters.length;
          index = end - 1;
        }
      } else {
        // Handle anything other than an ampersand, including newlines and EOF.
        if (character === 10 /* `\n` */) {
          line++;
          lines++;
          column = 0;
        }

        if (Number.isNaN(character)) {
          flush();
        } else {
          queue += fromCharCode(character);
          column++;
        }
      }
    }

    // Return the reduced nodes.
    return result.join('')

    // Get current position.
    function now() {
      return {
        line,
        column,
        offset: index + ((point ? point.offset : 0) || 0)
      }
    }

    /**
     * Handle the warning.
     *
     * @param {1|2|3|4|5|6|7} code
     * @param {number} offset
     */
    function warning(code, offset) {
      /** @type {ReturnType<now>} */
      let position;

      if (options.warning) {
        position = now();
        position.column += offset;
        position.offset += offset;

        options.warning.call(
          options.warningContext,
          messages[code],
          position,
          code
        );
      }
    }

    /**
     * Flush `queue` (normal text).
     * Macro invoked before each reference and at the end of `value`.
     * Does nothing when `queue` is empty.
     */
    function flush() {
      if (queue) {
        result.push(queue);

        if (options.text) {
          options.text.call(options.textContext, queue, {
            start: previous,
            end: now()
          });
        }

        queue = '';
      }
    }
  }

  /**
   * Check if `character` is outside the permissible unicode range.
   *
   * @param {number} code
   * @returns {boolean}
   */
  function prohibited(code) {
    return (code >= 0xd800 && code <= 0xdfff) || code > 0x10ffff
  }

  /**
   * Check if `character` is disallowed.
   *
   * @param {number} code
   * @returns {boolean}
   */
  function disallowed(code) {
    return (
      (code >= 0x0001 && code <= 0x0008) ||
      code === 0x000b ||
      (code >= 0x000d && code <= 0x001f) ||
      (code >= 0x007f && code <= 0x009f) ||
      (code >= 0xfdd0 && code <= 0xfdef) ||
      (code & 0xffff) === 0xffff ||
      (code & 0xffff) === 0xfffe
    )
  }

  var prismCore = {exports: {}};

  (function (module) {
  /// <reference lib="WebWorker"/>

  var _self = (typeof window !== 'undefined')
    ? window   // if in browser
    : (
      (typeof WorkerGlobalScope !== 'undefined' && self instanceof WorkerGlobalScope)
        ? self // if in worker
        : {}   // if in node js
    );

  /**
   * Prism: Lightweight, robust, elegant syntax highlighting
   *
   * @license MIT <https://opensource.org/licenses/MIT>
   * @author Lea Verou <https://lea.verou.me>
   * @namespace
   * @public
   */
  var Prism = (function (_self) {

    // Private helper vars
    var lang = /(?:^|\s)lang(?:uage)?-([\w-]+)(?=\s|$)/i;
    var uniqueId = 0;

    // The grammar object for plaintext
    var plainTextGrammar = {};


    var _ = {
      /**
       * By default, Prism will attempt to highlight all code elements (by calling {@link Prism.highlightAll}) on the
       * current page after the page finished loading. This might be a problem if e.g. you wanted to asynchronously load
       * additional languages or plugins yourself.
       *
       * By setting this value to `true`, Prism will not automatically highlight all code elements on the page.
       *
       * You obviously have to change this value before the automatic highlighting started. To do this, you can add an
       * empty Prism object into the global scope before loading the Prism script like this:
       *
       * ```js
       * window.Prism = window.Prism || {};
       * Prism.manual = true;
       * // add a new <script> to load Prism's script
       * ```
       *
       * @default false
       * @type {boolean}
       * @memberof Prism
       * @public
       */
      manual: _self.Prism && _self.Prism.manual,
      /**
       * By default, if Prism is in a web worker, it assumes that it is in a worker it created itself, so it uses
       * `addEventListener` to communicate with its parent instance. However, if you're using Prism manually in your
       * own worker, you don't want it to do this.
       *
       * By setting this value to `true`, Prism will not add its own listeners to the worker.
       *
       * You obviously have to change this value before Prism executes. To do this, you can add an
       * empty Prism object into the global scope before loading the Prism script like this:
       *
       * ```js
       * window.Prism = window.Prism || {};
       * Prism.disableWorkerMessageHandler = true;
       * // Load Prism's script
       * ```
       *
       * @default false
       * @type {boolean}
       * @memberof Prism
       * @public
       */
      disableWorkerMessageHandler: _self.Prism && _self.Prism.disableWorkerMessageHandler,

      /**
       * A namespace for utility methods.
       *
       * All function in this namespace that are not explicitly marked as _public_ are for __internal use only__ and may
       * change or disappear at any time.
       *
       * @namespace
       * @memberof Prism
       */
      util: {
        encode: function encode(tokens) {
          if (tokens instanceof Token) {
            return new Token(tokens.type, encode(tokens.content), tokens.alias);
          } else if (Array.isArray(tokens)) {
            return tokens.map(encode);
          } else {
            return tokens.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/\u00a0/g, ' ');
          }
        },

        /**
         * Returns the name of the type of the given value.
         *
         * @param {any} o
         * @returns {string}
         * @example
         * type(null)      === 'Null'
         * type(undefined) === 'Undefined'
         * type(123)       === 'Number'
         * type('foo')     === 'String'
         * type(true)      === 'Boolean'
         * type([1, 2])    === 'Array'
         * type({})        === 'Object'
         * type(String)    === 'Function'
         * type(/abc+/)    === 'RegExp'
         */
        type: function (o) {
          return Object.prototype.toString.call(o).slice(8, -1);
        },

        /**
         * Returns a unique number for the given object. Later calls will still return the same number.
         *
         * @param {Object} obj
         * @returns {number}
         */
        objId: function (obj) {
          if (!obj['__id']) {
            Object.defineProperty(obj, '__id', { value: ++uniqueId });
          }
          return obj['__id'];
        },

        /**
         * Creates a deep clone of the given object.
         *
         * The main intended use of this function is to clone language definitions.
         *
         * @param {T} o
         * @param {Record<number, any>} [visited]
         * @returns {T}
         * @template T
         */
        clone: function deepClone(o, visited) {
          visited = visited || {};

          var clone; var id;
          switch (_.util.type(o)) {
            case 'Object':
              id = _.util.objId(o);
              if (visited[id]) {
                return visited[id];
              }
              clone = /** @type {Record<string, any>} */ ({});
              visited[id] = clone;

              for (var key in o) {
                if (o.hasOwnProperty(key)) {
                  clone[key] = deepClone(o[key], visited);
                }
              }

              return /** @type {any} */ (clone);

            case 'Array':
              id = _.util.objId(o);
              if (visited[id]) {
                return visited[id];
              }
              clone = [];
              visited[id] = clone;

              (/** @type {Array} */(/** @type {any} */(o))).forEach(function (v, i) {
                clone[i] = deepClone(v, visited);
              });

              return /** @type {any} */ (clone);

            default:
              return o;
          }
        },

        /**
         * Returns the Prism language of the given element set by a `language-xxxx` or `lang-xxxx` class.
         *
         * If no language is set for the element or the element is `null` or `undefined`, `none` will be returned.
         *
         * @param {Element} element
         * @returns {string}
         */
        getLanguage: function (element) {
          while (element) {
            var m = lang.exec(element.className);
            if (m) {
              return m[1].toLowerCase();
            }
            element = element.parentElement;
          }
          return 'none';
        },

        /**
         * Sets the Prism `language-xxxx` class of the given element.
         *
         * @param {Element} element
         * @param {string} language
         * @returns {void}
         */
        setLanguage: function (element, language) {
          // remove all `language-xxxx` classes
          // (this might leave behind a leading space)
          element.className = element.className.replace(RegExp(lang, 'gi'), '');

          // add the new `language-xxxx` class
          // (using `classList` will automatically clean up spaces for us)
          element.classList.add('language-' + language);
        },

        /**
         * Returns the script element that is currently executing.
         *
         * This does __not__ work for line script element.
         *
         * @returns {HTMLScriptElement | null}
         */
        currentScript: function () {
          if (typeof document === 'undefined') {
            return null;
          }
          if ('currentScript' in document && 1 < 2 /* hack to trip TS' flow analysis */) {
            return /** @type {any} */ (document.currentScript);
          }

          // IE11 workaround
          // we'll get the src of the current script by parsing IE11's error stack trace
          // this will not work for inline scripts

          try {
            throw new Error();
          } catch (err) {
            // Get file src url from stack. Specifically works with the format of stack traces in IE.
            // A stack will look like this:
            //
            // Error
            //    at _.util.currentScript (http://localhost/components/prism-core.js:119:5)
            //    at Global code (http://localhost/components/prism-core.js:606:1)

            var src = (/at [^(\r\n]*\((.*):[^:]+:[^:]+\)$/i.exec(err.stack) || [])[1];
            if (src) {
              var scripts = document.getElementsByTagName('script');
              for (var i in scripts) {
                if (scripts[i].src == src) {
                  return scripts[i];
                }
              }
            }
            return null;
          }
        },

        /**
         * Returns whether a given class is active for `element`.
         *
         * The class can be activated if `element` or one of its ancestors has the given class and it can be deactivated
         * if `element` or one of its ancestors has the negated version of the given class. The _negated version_ of the
         * given class is just the given class with a `no-` prefix.
         *
         * Whether the class is active is determined by the closest ancestor of `element` (where `element` itself is
         * closest ancestor) that has the given class or the negated version of it. If neither `element` nor any of its
         * ancestors have the given class or the negated version of it, then the default activation will be returned.
         *
         * In the paradoxical situation where the closest ancestor contains __both__ the given class and the negated
         * version of it, the class is considered active.
         *
         * @param {Element} element
         * @param {string} className
         * @param {boolean} [defaultActivation=false]
         * @returns {boolean}
         */
        isActive: function (element, className, defaultActivation) {
          var no = 'no-' + className;

          while (element) {
            var classList = element.classList;
            if (classList.contains(className)) {
              return true;
            }
            if (classList.contains(no)) {
              return false;
            }
            element = element.parentElement;
          }
          return !!defaultActivation;
        }
      },

      /**
       * This namespace contains all currently loaded languages and the some helper functions to create and modify languages.
       *
       * @namespace
       * @memberof Prism
       * @public
       */
      languages: {
        /**
         * The grammar for plain, unformatted text.
         */
        plain: plainTextGrammar,
        plaintext: plainTextGrammar,
        text: plainTextGrammar,
        txt: plainTextGrammar,

        /**
         * Creates a deep copy of the language with the given id and appends the given tokens.
         *
         * If a token in `redef` also appears in the copied language, then the existing token in the copied language
         * will be overwritten at its original position.
         *
         * ## Best practices
         *
         * Since the position of overwriting tokens (token in `redef` that overwrite tokens in the copied language)
         * doesn't matter, they can technically be in any order. However, this can be confusing to others that trying to
         * understand the language definition because, normally, the order of tokens matters in Prism grammars.
         *
         * Therefore, it is encouraged to order overwriting tokens according to the positions of the overwritten tokens.
         * Furthermore, all non-overwriting tokens should be placed after the overwriting ones.
         *
         * @param {string} id The id of the language to extend. This has to be a key in `Prism.languages`.
         * @param {Grammar} redef The new tokens to append.
         * @returns {Grammar} The new language created.
         * @public
         * @example
         * Prism.languages['css-with-colors'] = Prism.languages.extend('css', {
         *     // Prism.languages.css already has a 'comment' token, so this token will overwrite CSS' 'comment' token
         *     // at its original position
         *     'comment': { ... },
         *     // CSS doesn't have a 'color' token, so this token will be appended
         *     'color': /\b(?:red|green|blue)\b/
         * });
         */
        extend: function (id, redef) {
          var lang = _.util.clone(_.languages[id]);

          for (var key in redef) {
            lang[key] = redef[key];
          }

          return lang;
        },

        /**
         * Inserts tokens _before_ another token in a language definition or any other grammar.
         *
         * ## Usage
         *
         * This helper method makes it easy to modify existing languages. For example, the CSS language definition
         * not only defines CSS highlighting for CSS documents, but also needs to define highlighting for CSS embedded
         * in HTML through `<style>` elements. To do this, it needs to modify `Prism.languages.markup` and add the
         * appropriate tokens. However, `Prism.languages.markup` is a regular JavaScript object literal, so if you do
         * this:
         *
         * ```js
         * Prism.languages.markup.style = {
         *     // token
         * };
         * ```
         *
         * then the `style` token will be added (and processed) at the end. `insertBefore` allows you to insert tokens
         * before existing tokens. For the CSS example above, you would use it like this:
         *
         * ```js
         * Prism.languages.insertBefore('markup', 'cdata', {
         *     'style': {
         *         // token
         *     }
         * });
         * ```
         *
         * ## Special cases
         *
         * If the grammars of `inside` and `insert` have tokens with the same name, the tokens in `inside`'s grammar
         * will be ignored.
         *
         * This behavior can be used to insert tokens after `before`:
         *
         * ```js
         * Prism.languages.insertBefore('markup', 'comment', {
         *     'comment': Prism.languages.markup.comment,
         *     // tokens after 'comment'
         * });
         * ```
         *
         * ## Limitations
         *
         * The main problem `insertBefore` has to solve is iteration order. Since ES2015, the iteration order for object
         * properties is guaranteed to be the insertion order (except for integer keys) but some browsers behave
         * differently when keys are deleted and re-inserted. So `insertBefore` can't be implemented by temporarily
         * deleting properties which is necessary to insert at arbitrary positions.
         *
         * To solve this problem, `insertBefore` doesn't actually insert the given tokens into the target object.
         * Instead, it will create a new object and replace all references to the target object with the new one. This
         * can be done without temporarily deleting properties, so the iteration order is well-defined.
         *
         * However, only references that can be reached from `Prism.languages` or `insert` will be replaced. I.e. if
         * you hold the target object in a variable, then the value of the variable will not change.
         *
         * ```js
         * var oldMarkup = Prism.languages.markup;
         * var newMarkup = Prism.languages.insertBefore('markup', 'comment', { ... });
         *
         * assert(oldMarkup !== Prism.languages.markup);
         * assert(newMarkup === Prism.languages.markup);
         * ```
         *
         * @param {string} inside The property of `root` (e.g. a language id in `Prism.languages`) that contains the
         * object to be modified.
         * @param {string} before The key to insert before.
         * @param {Grammar} insert An object containing the key-value pairs to be inserted.
         * @param {Object<string, any>} [root] The object containing `inside`, i.e. the object that contains the
         * object to be modified.
         *
         * Defaults to `Prism.languages`.
         * @returns {Grammar} The new grammar object.
         * @public
         */
        insertBefore: function (inside, before, insert, root) {
          root = root || /** @type {any} */ (_.languages);
          var grammar = root[inside];
          /** @type {Grammar} */
          var ret = {};

          for (var token in grammar) {
            if (grammar.hasOwnProperty(token)) {

              if (token == before) {
                for (var newToken in insert) {
                  if (insert.hasOwnProperty(newToken)) {
                    ret[newToken] = insert[newToken];
                  }
                }
              }

              // Do not insert token which also occur in insert. See #1525
              if (!insert.hasOwnProperty(token)) {
                ret[token] = grammar[token];
              }
            }
          }

          var old = root[inside];
          root[inside] = ret;

          // Update references in other language definitions
          _.languages.DFS(_.languages, function (key, value) {
            if (value === old && key != inside) {
              this[key] = ret;
            }
          });

          return ret;
        },

        // Traverse a language definition with Depth First Search
        DFS: function DFS(o, callback, type, visited) {
          visited = visited || {};

          var objId = _.util.objId;

          for (var i in o) {
            if (o.hasOwnProperty(i)) {
              callback.call(o, i, o[i], type || i);

              var property = o[i];
              var propertyType = _.util.type(property);

              if (propertyType === 'Object' && !visited[objId(property)]) {
                visited[objId(property)] = true;
                DFS(property, callback, null, visited);
              } else if (propertyType === 'Array' && !visited[objId(property)]) {
                visited[objId(property)] = true;
                DFS(property, callback, i, visited);
              }
            }
          }
        }
      },

      plugins: {},

      /**
       * This is the most high-level function in Prism’s API.
       * It fetches all the elements that have a `.language-xxxx` class and then calls {@link Prism.highlightElement} on
       * each one of them.
       *
       * This is equivalent to `Prism.highlightAllUnder(document, async, callback)`.
       *
       * @param {boolean} [async=false] Same as in {@link Prism.highlightAllUnder}.
       * @param {HighlightCallback} [callback] Same as in {@link Prism.highlightAllUnder}.
       * @memberof Prism
       * @public
       */
      highlightAll: function (async, callback) {
        _.highlightAllUnder(document, async, callback);
      },

      /**
       * Fetches all the descendants of `container` that have a `.language-xxxx` class and then calls
       * {@link Prism.highlightElement} on each one of them.
       *
       * The following hooks will be run:
       * 1. `before-highlightall`
       * 2. `before-all-elements-highlight`
       * 3. All hooks of {@link Prism.highlightElement} for each element.
       *
       * @param {ParentNode} container The root element, whose descendants that have a `.language-xxxx` class will be highlighted.
       * @param {boolean} [async=false] Whether each element is to be highlighted asynchronously using Web Workers.
       * @param {HighlightCallback} [callback] An optional callback to be invoked on each element after its highlighting is done.
       * @memberof Prism
       * @public
       */
      highlightAllUnder: function (container, async, callback) {
        var env = {
          callback: callback,
          container: container,
          selector: 'code[class*="language-"], [class*="language-"] code, code[class*="lang-"], [class*="lang-"] code'
        };

        _.hooks.run('before-highlightall', env);

        env.elements = Array.prototype.slice.apply(env.container.querySelectorAll(env.selector));

        _.hooks.run('before-all-elements-highlight', env);

        for (var i = 0, element; (element = env.elements[i++]);) {
          _.highlightElement(element, async === true, env.callback);
        }
      },

      /**
       * Highlights the code inside a single element.
       *
       * The following hooks will be run:
       * 1. `before-sanity-check`
       * 2. `before-highlight`
       * 3. All hooks of {@link Prism.highlight}. These hooks will be run by an asynchronous worker if `async` is `true`.
       * 4. `before-insert`
       * 5. `after-highlight`
       * 6. `complete`
       *
       * Some the above hooks will be skipped if the element doesn't contain any text or there is no grammar loaded for
       * the element's language.
       *
       * @param {Element} element The element containing the code.
       * It must have a class of `language-xxxx` to be processed, where `xxxx` is a valid language identifier.
       * @param {boolean} [async=false] Whether the element is to be highlighted asynchronously using Web Workers
       * to improve performance and avoid blocking the UI when highlighting very large chunks of code. This option is
       * [disabled by default](https://prismjs.com/faq.html#why-is-asynchronous-highlighting-disabled-by-default).
       *
       * Note: All language definitions required to highlight the code must be included in the main `prism.js` file for
       * asynchronous highlighting to work. You can build your own bundle on the
       * [Download page](https://prismjs.com/download.html).
       * @param {HighlightCallback} [callback] An optional callback to be invoked after the highlighting is done.
       * Mostly useful when `async` is `true`, since in that case, the highlighting is done asynchronously.
       * @memberof Prism
       * @public
       */
      highlightElement: function (element, async, callback) {
        // Find language
        var language = _.util.getLanguage(element);
        var grammar = _.languages[language];

        // Set language on the element, if not present
        _.util.setLanguage(element, language);

        // Set language on the parent, for styling
        var parent = element.parentElement;
        if (parent && parent.nodeName.toLowerCase() === 'pre') {
          _.util.setLanguage(parent, language);
        }

        var code = element.textContent;

        var env = {
          element: element,
          language: language,
          grammar: grammar,
          code: code
        };

        function insertHighlightedCode(highlightedCode) {
          env.highlightedCode = highlightedCode;

          _.hooks.run('before-insert', env);

          env.element.innerHTML = env.highlightedCode;

          _.hooks.run('after-highlight', env);
          _.hooks.run('complete', env);
          callback && callback.call(env.element);
        }

        _.hooks.run('before-sanity-check', env);

        // plugins may change/add the parent/element
        parent = env.element.parentElement;
        if (parent && parent.nodeName.toLowerCase() === 'pre' && !parent.hasAttribute('tabindex')) {
          parent.setAttribute('tabindex', '0');
        }

        if (!env.code) {
          _.hooks.run('complete', env);
          callback && callback.call(env.element);
          return;
        }

        _.hooks.run('before-highlight', env);

        if (!env.grammar) {
          insertHighlightedCode(_.util.encode(env.code));
          return;
        }

        if (async && _self.Worker) {
          var worker = new Worker(_.filename);

          worker.onmessage = function (evt) {
            insertHighlightedCode(evt.data);
          };

          worker.postMessage(JSON.stringify({
            language: env.language,
            code: env.code,
            immediateClose: true
          }));
        } else {
          insertHighlightedCode(_.highlight(env.code, env.grammar, env.language));
        }
      },

      /**
       * Low-level function, only use if you know what you’re doing. It accepts a string of text as input
       * and the language definitions to use, and returns a string with the HTML produced.
       *
       * The following hooks will be run:
       * 1. `before-tokenize`
       * 2. `after-tokenize`
       * 3. `wrap`: On each {@link Token}.
       *
       * @param {string} text A string with the code to be highlighted.
       * @param {Grammar} grammar An object containing the tokens to use.
       *
       * Usually a language definition like `Prism.languages.markup`.
       * @param {string} language The name of the language definition passed to `grammar`.
       * @returns {string} The highlighted HTML.
       * @memberof Prism
       * @public
       * @example
       * Prism.highlight('var foo = true;', Prism.languages.javascript, 'javascript');
       */
      highlight: function (text, grammar, language) {
        var env = {
          code: text,
          grammar: grammar,
          language: language
        };
        _.hooks.run('before-tokenize', env);
        if (!env.grammar) {
          throw new Error('The language "' + env.language + '" has no grammar.');
        }
        env.tokens = _.tokenize(env.code, env.grammar);
        _.hooks.run('after-tokenize', env);
        return Token.stringify(_.util.encode(env.tokens), env.language);
      },

      /**
       * This is the heart of Prism, and the most low-level function you can use. It accepts a string of text as input
       * and the language definitions to use, and returns an array with the tokenized code.
       *
       * When the language definition includes nested tokens, the function is called recursively on each of these tokens.
       *
       * This method could be useful in other contexts as well, as a very crude parser.
       *
       * @param {string} text A string with the code to be highlighted.
       * @param {Grammar} grammar An object containing the tokens to use.
       *
       * Usually a language definition like `Prism.languages.markup`.
       * @returns {TokenStream} An array of strings and tokens, a token stream.
       * @memberof Prism
       * @public
       * @example
       * let code = `var foo = 0;`;
       * let tokens = Prism.tokenize(code, Prism.languages.javascript);
       * tokens.forEach(token => {
       *     if (token instanceof Prism.Token && token.type === 'number') {
       *         console.log(`Found numeric literal: ${token.content}`);
       *     }
       * });
       */
      tokenize: function (text, grammar) {
        var rest = grammar.rest;
        if (rest) {
          for (var token in rest) {
            grammar[token] = rest[token];
          }

          delete grammar.rest;
        }

        var tokenList = new LinkedList();
        addAfter(tokenList, tokenList.head, text);

        matchGrammar(text, tokenList, grammar, tokenList.head, 0);

        return toArray(tokenList);
      },

      /**
       * @namespace
       * @memberof Prism
       * @public
       */
      hooks: {
        all: {},

        /**
         * Adds the given callback to the list of callbacks for the given hook.
         *
         * The callback will be invoked when the hook it is registered for is run.
         * Hooks are usually directly run by a highlight function but you can also run hooks yourself.
         *
         * One callback function can be registered to multiple hooks and the same hook multiple times.
         *
         * @param {string} name The name of the hook.
         * @param {HookCallback} callback The callback function which is given environment variables.
         * @public
         */
        add: function (name, callback) {
          var hooks = _.hooks.all;

          hooks[name] = hooks[name] || [];

          hooks[name].push(callback);
        },

        /**
         * Runs a hook invoking all registered callbacks with the given environment variables.
         *
         * Callbacks will be invoked synchronously and in the order in which they were registered.
         *
         * @param {string} name The name of the hook.
         * @param {Object<string, any>} env The environment variables of the hook passed to all callbacks registered.
         * @public
         */
        run: function (name, env) {
          var callbacks = _.hooks.all[name];

          if (!callbacks || !callbacks.length) {
            return;
          }

          for (var i = 0, callback; (callback = callbacks[i++]);) {
            callback(env);
          }
        }
      },

      Token: Token
    };
    _self.Prism = _;


    // Typescript note:
    // The following can be used to import the Token type in JSDoc:
    //
    //   @typedef {InstanceType<import("./prism-core")["Token"]>} Token

    /**
     * Creates a new token.
     *
     * @param {string} type See {@link Token#type type}
     * @param {string | TokenStream} content See {@link Token#content content}
     * @param {string|string[]} [alias] The alias(es) of the token.
     * @param {string} [matchedStr=""] A copy of the full string this token was created from.
     * @class
     * @global
     * @public
     */
    function Token(type, content, alias, matchedStr) {
      /**
       * The type of the token.
       *
       * This is usually the key of a pattern in a {@link Grammar}.
       *
       * @type {string}
       * @see GrammarToken
       * @public
       */
      this.type = type;
      /**
       * The strings or tokens contained by this token.
       *
       * This will be a token stream if the pattern matched also defined an `inside` grammar.
       *
       * @type {string | TokenStream}
       * @public
       */
      this.content = content;
      /**
       * The alias(es) of the token.
       *
       * @type {string|string[]}
       * @see GrammarToken
       * @public
       */
      this.alias = alias;
      // Copy of the full string this token was created from
      this.length = (matchedStr || '').length | 0;
    }

    /**
     * A token stream is an array of strings and {@link Token Token} objects.
     *
     * Token streams have to fulfill a few properties that are assumed by most functions (mostly internal ones) that process
     * them.
     *
     * 1. No adjacent strings.
     * 2. No empty strings.
     *
     *    The only exception here is the token stream that only contains the empty string and nothing else.
     *
     * @typedef {Array<string | Token>} TokenStream
     * @global
     * @public
     */

    /**
     * Converts the given token or token stream to an HTML representation.
     *
     * The following hooks will be run:
     * 1. `wrap`: On each {@link Token}.
     *
     * @param {string | Token | TokenStream} o The token or token stream to be converted.
     * @param {string} language The name of current language.
     * @returns {string} The HTML representation of the token or token stream.
     * @memberof Token
     * @static
     */
    Token.stringify = function stringify(o, language) {
      if (typeof o == 'string') {
        return o;
      }
      if (Array.isArray(o)) {
        var s = '';
        o.forEach(function (e) {
          s += stringify(e, language);
        });
        return s;
      }

      var env = {
        type: o.type,
        content: stringify(o.content, language),
        tag: 'span',
        classes: ['token', o.type],
        attributes: {},
        language: language
      };

      var aliases = o.alias;
      if (aliases) {
        if (Array.isArray(aliases)) {
          Array.prototype.push.apply(env.classes, aliases);
        } else {
          env.classes.push(aliases);
        }
      }

      _.hooks.run('wrap', env);

      var attributes = '';
      for (var name in env.attributes) {
        attributes += ' ' + name + '="' + (env.attributes[name] || '').replace(/"/g, '&quot;') + '"';
      }

      return '<' + env.tag + ' class="' + env.classes.join(' ') + '"' + attributes + '>' + env.content + '</' + env.tag + '>';
    };

    /**
     * @param {RegExp} pattern
     * @param {number} pos
     * @param {string} text
     * @param {boolean} lookbehind
     * @returns {RegExpExecArray | null}
     */
    function matchPattern(pattern, pos, text, lookbehind) {
      pattern.lastIndex = pos;
      var match = pattern.exec(text);
      if (match && lookbehind && match[1]) {
        // change the match to remove the text matched by the Prism lookbehind group
        var lookbehindLength = match[1].length;
        match.index += lookbehindLength;
        match[0] = match[0].slice(lookbehindLength);
      }
      return match;
    }

    /**
     * @param {string} text
     * @param {LinkedList<string | Token>} tokenList
     * @param {any} grammar
     * @param {LinkedListNode<string | Token>} startNode
     * @param {number} startPos
     * @param {RematchOptions} [rematch]
     * @returns {void}
     * @private
     *
     * @typedef RematchOptions
     * @property {string} cause
     * @property {number} reach
     */
    function matchGrammar(text, tokenList, grammar, startNode, startPos, rematch) {
      for (var token in grammar) {
        if (!grammar.hasOwnProperty(token) || !grammar[token]) {
          continue;
        }

        var patterns = grammar[token];
        patterns = Array.isArray(patterns) ? patterns : [patterns];

        for (var j = 0; j < patterns.length; ++j) {
          if (rematch && rematch.cause == token + ',' + j) {
            return;
          }

          var patternObj = patterns[j];
          var inside = patternObj.inside;
          var lookbehind = !!patternObj.lookbehind;
          var greedy = !!patternObj.greedy;
          var alias = patternObj.alias;

          if (greedy && !patternObj.pattern.global) {
            // Without the global flag, lastIndex won't work
            var flags = patternObj.pattern.toString().match(/[imsuy]*$/)[0];
            patternObj.pattern = RegExp(patternObj.pattern.source, flags + 'g');
          }

          /** @type {RegExp} */
          var pattern = patternObj.pattern || patternObj;

          for ( // iterate the token list and keep track of the current token/string position
            var currentNode = startNode.next, pos = startPos;
            currentNode !== tokenList.tail;
            pos += currentNode.value.length, currentNode = currentNode.next
          ) {

            if (rematch && pos >= rematch.reach) {
              break;
            }

            var str = currentNode.value;

            if (tokenList.length > text.length) {
              // Something went terribly wrong, ABORT, ABORT!
              return;
            }

            if (str instanceof Token) {
              continue;
            }

            var removeCount = 1; // this is the to parameter of removeBetween
            var match;

            if (greedy) {
              match = matchPattern(pattern, pos, text, lookbehind);
              if (!match || match.index >= text.length) {
                break;
              }

              var from = match.index;
              var to = match.index + match[0].length;
              var p = pos;

              // find the node that contains the match
              p += currentNode.value.length;
              while (from >= p) {
                currentNode = currentNode.next;
                p += currentNode.value.length;
              }
              // adjust pos (and p)
              p -= currentNode.value.length;
              pos = p;

              // the current node is a Token, then the match starts inside another Token, which is invalid
              if (currentNode.value instanceof Token) {
                continue;
              }

              // find the last node which is affected by this match
              for (
                var k = currentNode;
                k !== tokenList.tail && (p < to || typeof k.value === 'string');
                k = k.next
              ) {
                removeCount++;
                p += k.value.length;
              }
              removeCount--;

              // replace with the new match
              str = text.slice(pos, p);
              match.index -= pos;
            } else {
              match = matchPattern(pattern, 0, str, lookbehind);
              if (!match) {
                continue;
              }
            }

            // eslint-disable-next-line no-redeclare
            var from = match.index;
            var matchStr = match[0];
            var before = str.slice(0, from);
            var after = str.slice(from + matchStr.length);

            var reach = pos + str.length;
            if (rematch && reach > rematch.reach) {
              rematch.reach = reach;
            }

            var removeFrom = currentNode.prev;

            if (before) {
              removeFrom = addAfter(tokenList, removeFrom, before);
              pos += before.length;
            }

            removeRange(tokenList, removeFrom, removeCount);

            var wrapped = new Token(token, inside ? _.tokenize(matchStr, inside) : matchStr, alias, matchStr);
            currentNode = addAfter(tokenList, removeFrom, wrapped);

            if (after) {
              addAfter(tokenList, currentNode, after);
            }

            if (removeCount > 1) {
              // at least one Token object was removed, so we have to do some rematching
              // this can only happen if the current pattern is greedy

              /** @type {RematchOptions} */
              var nestedRematch = {
                cause: token + ',' + j,
                reach: reach
              };
              matchGrammar(text, tokenList, grammar, currentNode.prev, pos, nestedRematch);

              // the reach might have been extended because of the rematching
              if (rematch && nestedRematch.reach > rematch.reach) {
                rematch.reach = nestedRematch.reach;
              }
            }
          }
        }
      }
    }

    /**
     * @typedef LinkedListNode
     * @property {T} value
     * @property {LinkedListNode<T> | null} prev The previous node.
     * @property {LinkedListNode<T> | null} next The next node.
     * @template T
     * @private
     */

    /**
     * @template T
     * @private
     */
    function LinkedList() {
      /** @type {LinkedListNode<T>} */
      var head = { value: null, prev: null, next: null };
      /** @type {LinkedListNode<T>} */
      var tail = { value: null, prev: head, next: null };
      head.next = tail;

      /** @type {LinkedListNode<T>} */
      this.head = head;
      /** @type {LinkedListNode<T>} */
      this.tail = tail;
      this.length = 0;
    }

    /**
     * Adds a new node with the given value to the list.
     *
     * @param {LinkedList<T>} list
     * @param {LinkedListNode<T>} node
     * @param {T} value
     * @returns {LinkedListNode<T>} The added node.
     * @template T
     */
    function addAfter(list, node, value) {
      // assumes that node != list.tail && values.length >= 0
      var next = node.next;

      var newNode = { value: value, prev: node, next: next };
      node.next = newNode;
      next.prev = newNode;
      list.length++;

      return newNode;
    }
    /**
     * Removes `count` nodes after the given node. The given node will not be removed.
     *
     * @param {LinkedList<T>} list
     * @param {LinkedListNode<T>} node
     * @param {number} count
     * @template T
     */
    function removeRange(list, node, count) {
      var next = node.next;
      for (var i = 0; i < count && next !== list.tail; i++) {
        next = next.next;
      }
      node.next = next;
      next.prev = node;
      list.length -= i;
    }
    /**
     * @param {LinkedList<T>} list
     * @returns {T[]}
     * @template T
     */
    function toArray(list) {
      var array = [];
      var node = list.head.next;
      while (node !== list.tail) {
        array.push(node.value);
        node = node.next;
      }
      return array;
    }


    if (!_self.document) {
      if (!_self.addEventListener) {
        // in Node.js
        return _;
      }

      if (!_.disableWorkerMessageHandler) {
        // In worker
        _self.addEventListener('message', function (evt) {
          var message = JSON.parse(evt.data);
          var lang = message.language;
          var code = message.code;
          var immediateClose = message.immediateClose;

          _self.postMessage(_.highlight(code, _.languages[lang], lang));
          if (immediateClose) {
            _self.close();
          }
        }, false);
      }

      return _;
    }

    // Get current script and highlight
    var script = _.util.currentScript();

    if (script) {
      _.filename = script.src;

      if (script.hasAttribute('data-manual')) {
        _.manual = true;
      }
    }

    function highlightAutomaticallyCallback() {
      if (!_.manual) {
        _.highlightAll();
      }
    }

    if (!_.manual) {
      // If the document state is "loading", then we'll use DOMContentLoaded.
      // If the document state is "interactive" and the prism.js script is deferred, then we'll also use the
      // DOMContentLoaded event because there might be some plugins or languages which have also been deferred and they
      // might take longer one animation frame to execute which can create a race condition where only some plugins have
      // been loaded when Prism.highlightAll() is executed, depending on how fast resources are loaded.
      // See https://github.com/PrismJS/prism/issues/2102
      var readyState = document.readyState;
      if (readyState === 'loading' || readyState === 'interactive' && script && script.defer) {
        document.addEventListener('DOMContentLoaded', highlightAutomaticallyCallback);
      } else {
        if (window.requestAnimationFrame) {
          window.requestAnimationFrame(highlightAutomaticallyCallback);
        } else {
          window.setTimeout(highlightAutomaticallyCallback, 16);
        }
      }
    }

    return _;

  }(_self));

  if (module.exports) {
    module.exports = Prism;
  }

  // hack for components to work correctly in node.js
  if (typeof commonjsGlobal !== 'undefined') {
    commonjsGlobal.Prism = Prism;
  }

  // some additional documentation/types

  /**
   * The expansion of a simple `RegExp` literal to support additional properties.
   *
   * @typedef GrammarToken
   * @property {RegExp} pattern The regular expression of the token.
   * @property {boolean} [lookbehind=false] If `true`, then the first capturing group of `pattern` will (effectively)
   * behave as a lookbehind group meaning that the captured text will not be part of the matched text of the new token.
   * @property {boolean} [greedy=false] Whether the token is greedy.
   * @property {string|string[]} [alias] An optional alias or list of aliases.
   * @property {Grammar} [inside] The nested grammar of this token.
   *
   * The `inside` grammar will be used to tokenize the text value of each token of this kind.
   *
   * This can be used to make nested and even recursive language definitions.
   *
   * Note: This can cause infinite recursion. Be careful when you embed different languages or even the same language into
   * each another.
   * @global
   * @public
   */

  /**
   * @typedef Grammar
   * @type {Object<string, RegExp | GrammarToken | Array<RegExp | GrammarToken>>}
   * @property {Grammar} [rest] An optional grammar object that will be appended to this grammar.
   * @global
   * @public
   */

  /**
   * A function which will invoked after an element was successfully highlighted.
   *
   * @callback HighlightCallback
   * @param {Element} element The element successfully highlighted.
   * @returns {void}
   * @global
   * @public
   */

  /**
   * @callback HookCallback
   * @param {Object<string, any>} env The environment variables of the hook.
   * @returns {void}
   * @global
   * @public
   */
  }(prismCore));

  var Prism = prismCore.exports;

  /**
   * @typedef _Token A hidden Prism token
   * @property {string} type
   * @property {string} content
   * @property {string} alias
   * @property {number} length
   *
   * @typedef _Env A hidden Prism environment
   * @property {string} type
   * @property {string} tag
   * @property {Text|RefractorElement|Array<Text|RefractorElement>} content
   * @property {Array<string>} classes
   * @property {Record<string, string>} attributes
   * @property {string} language
   *
   * @typedef {import('hast').Root} Root
   * @typedef {import('hast').Element} Element
   * @typedef {import('hast').Text} Text
   * @typedef {Omit<Element, 'children'> & {children: Array<RefractorElement|Text>}} RefractorElement
   * @typedef {Omit<Root, 'children'> & {children: Array<RefractorElement|Text>}} RefractorRoot
   *
   * @typedef {import('prismjs').Languages} Languages
   * @typedef {import('prismjs').Grammar} Grammar Whatever this is, Prism handles it.
   * @typedef {((prism: unknown) => void) & {displayName: string, aliases?: string[]}} Syntax A refractor syntax function
   *
   * @typedef Refractor Virtual syntax highlighting
   * @property {highlight} highlight
   * @property {alias} alias
   * @property {register} register
   * @property {registered} registered
   * @property {listLanguages} listLanguages
   * @property {Languages} languages
   */

  /* eslint-disable no-undef */
  // Don’t allow Prism to run on page load in browser or to start messaging from
  // workers.
  /* c8 ignore next 15 */
  /** @type {typeof globalThis} */
  const ctx =
    typeof globalThis === 'object'
      ? globalThis
      : // @ts-expect-error
      typeof self === 'object'
      ? // @ts-expect-error
        self
      : // @ts-expect-error
      typeof window === 'object'
      ? // @ts-expect-error
        window
      : typeof global === 'object'
      ? global
      : {};
  /* eslint-enable no-undef */

  const restore = capture();

  /* c8 ignore next 5 */
  ctx.Prism = ctx.Prism || {};
  ctx.Prism.manual = true;
  ctx.Prism.disableWorkerMessageHandler = true;

  /* eslint-enable import/first */

  restore();

  const own = {}.hasOwnProperty;

  // Inherit.
  function Refractor() {}

  Refractor.prototype = Prism;

  /** @type {Refractor} */
  // @ts-expect-error: TS is wrong.
  const refractor = new Refractor();

  // Create.
  refractor.highlight = highlight;
  refractor.register = register;
  refractor.alias = alias;
  refractor.registered = registered;
  refractor.listLanguages = listLanguages;

  // @ts-expect-error Overwrite Prism.
  refractor.util.encode = encode;
  // @ts-expect-error Overwrite Prism.
  refractor.Token.stringify = stringify;

  /**
   * Highlight `value` (code) as `language` (programming language).
   *
   * @param {string} value
   *   Code to highlight.
   * @param {string|Grammar} language
   *   Programming language name, alias, or grammar.
   * @returns {RefractorRoot}
   *   Node representing highlighted code.
   */
  function highlight(value, language) {
    if (typeof value !== 'string') {
      throw new TypeError('Expected `string` for `value`, got `' + value + '`')
    }

    /** @type {Grammar} */
    let grammar;
    /** @type {string|undefined} */
    let name;

    // `name` is a grammar object.
    if (language && typeof language === 'object') {
      grammar = language;
    } else {
      name = language;

      if (typeof name !== 'string') {
        throw new TypeError('Expected `string` for `name`, got `' + name + '`')
      }

      if (own.call(refractor.languages, name)) {
        grammar = refractor.languages[name];
      } else {
        throw new Error('Unknown language: `' + name + '` is not registered')
      }
    }

    return {
      type: 'root',
      children: Prism.highlight.call(refractor, value, grammar, name)
    }
  }

  /**
   * Register a syntax.
   *
   * @param {Syntax} syntax
   *   Language function made for refractor, as in, the files in
   *   `refractor/lang/*.js`.
   * @returns {void}
   */
  function register(syntax) {
    // @ts-expect-error: runtime.
    if (typeof syntax !== 'function' || !syntax.displayName) {
      throw new Error('Expected `function` for `syntax`, got `' + syntax + '`')
    }

    // Do not duplicate registrations.
    // @ts-expect-error: TypeScript is wrong.
    if (!own.call(refractor.languages, syntax.displayName)) {
      // @ts-expect-error: TypeScript is wrong.
      syntax(refractor);
    }
  }

  /**
   * Register aliases for already registered languages.
   *
   * @param {Record<string, string|Array<string>>|string} language
   * @param {string|Array<string>} [alias]
   * @returns {void}
   */
  function alias(language, alias) {
    const languages = refractor.languages;
    /** @type {Record<string, string|Array<string>>} */
    let map = {};

    if (typeof language === 'string') {
      if (alias) {
        map[language] = alias;
      }
    } else {
      map = language;
    }

    /** @type {string} */
    let key;

    for (key in map) {
      if (own.call(map, key)) {
        const value = map[key];
        const list = typeof value === 'string' ? [value] : value;
        let index = -1;

        while (++index < list.length) {
          languages[list[index]] = languages[key];
        }
      }
    }
  }

  /**
   * Check whether an `alias` or `language` is registered.
   *
   * @param {string} aliasOrLanguage
   * @returns {boolean}
   */
  function registered(aliasOrLanguage) {
    if (typeof aliasOrLanguage !== 'string') {
      throw new TypeError(
        'Expected `string` for `aliasOrLanguage`, got `' + aliasOrLanguage + '`'
      )
    }

    return own.call(refractor.languages, aliasOrLanguage)
  }

  /**
   * List all registered languages (names and aliases).
   *
   * @returns {Array<string>}
   */
  function listLanguages() {
    const languages = refractor.languages;
    /** @type {Array<string>} */
    const list = [];
    /** @type {string} */
    let language;

    for (language in languages) {
      if (
        own.call(languages, language) &&
        typeof languages[language] === 'object'
      ) {
        list.push(language);
      }
    }

    return list
  }

  /**
   * @param {string|_Token|Array<string|_Token>} value
   * @param {string} language
   * @returns {Text|RefractorElement|Array<Text|RefractorElement>}
   */
  function stringify(value, language) {
    if (typeof value === 'string') {
      return {type: 'text', value}
    }

    if (Array.isArray(value)) {
      /** @type {Array<Text|RefractorElement>} */
      const result = [];
      let index = -1;

      while (++index < value.length) {
        if (
          value[index] !== '' &&
          value[index] !== null &&
          value[index] !== undefined
        ) {
          // @ts-expect-error Assume no sub-arrays.
          result.push(stringify(value[index], language));
        }
      }

      return result
    }

    /** @type {_Env} */
    const env = {
      type: value.type,
      content: stringify(value.content, language),
      tag: 'span',
      classes: ['token', value.type],
      attributes: {},
      language
    };

    if (value.alias) {
      env.classes.push(
        ...(typeof value.alias === 'string' ? [value.alias] : value.alias)
      );
    }

    // @ts-expect-error Prism.
    refractor.hooks.run('wrap', env);

    // @ts-expect-error Hush, it’s fine.
    return h(
      env.tag + '.' + env.classes.join('.'),
      attributes(env.attributes),
      env.content
    )
  }

  /**
   * @template {unknown} T
   * @param {T} tokens
   * @returns {T}
   */
  function encode(tokens) {
    return tokens
  }

  /**
   * @param {Record<string, string>} attrs
   * @returns {Record<string, string>}
   */
  function attributes(attrs) {
    /** @type {string} */
    let key;

    for (key in attrs) {
      if (own.call(attrs, key)) {
        attrs[key] = parseEntities(attrs[key]);
      }
    }

    return attrs
  }

  /**
   * @returns {() => void}
   */
  function capture() {
    /** @type {boolean|undefined} */
    let defined = 'Prism' in ctx;
    /* c8 ignore next */
    let current = defined ? ctx.Prism : undefined;

    return restore

    /**
     * @returns {void}
     */
    function restore() {
      /* istanbul ignore else - Clean leaks after Prism. */
      if (defined) {
        // @ts-expect-error: hush.
        ctx.Prism = current;
        /* c8 ignore next 4 */
      } else {
        // @ts-expect-error: hush.
        delete ctx.Prism;
      }

      defined = undefined;
      current = undefined;
    }
  }

  /**
   * @typedef {import('./core.js').RefractorRoot} RefractorRoot
   * @typedef {import('./core.js').RefractorElement} RefractorElement
   * @typedef {import('./core.js').Text} Text
   * @typedef {import('./core.js').Grammar} Grammar
   * @typedef {import('./core.js').Syntax} Syntax
   */

  refractor.register(clike);
  refractor.register(c$1);
  refractor.register(cpp);
  refractor.register(arduino);
  refractor.register(bash);
  refractor.register(csharp);
  refractor.register(markup);
  refractor.register(css);
  refractor.register(diff);
  refractor.register(go);
  refractor.register(ini);
  refractor.register(java);
  refractor.register(regex);
  refractor.register(javascript);
  refractor.register(json);
  refractor.register(kotlin);
  refractor.register(less);
  refractor.register(lua);
  refractor.register(makefile);
  refractor.register(yaml);
  refractor.register(markdown$1);
  refractor.register(objectivec);
  refractor.register(perl);
  refractor.register(markupTemplating);
  refractor.register(php);
  refractor.register(python);
  refractor.register(r);
  refractor.register(ruby);
  refractor.register(rust);
  refractor.register(sass);
  refractor.register(scss);
  refractor.register(sql);
  refractor.register(swift);
  refractor.register(typescript);
  refractor.register(basic);
  refractor.register(vbnet);

  // @ts-nocheck
  abap.displayName = 'abap';
  abap.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function abap(Prism) {
    Prism.languages.abap = {
      comment: /^\*.*/m,
      string: /(`|')(?:\\.|(?!\1)[^\\\r\n])*\1/,
      'string-template': {
        pattern: /([|}])(?:\\.|[^\\|{\r\n])*(?=[|{])/,
        lookbehind: true,
        alias: 'string'
      },
      /* End Of Line comments should not interfere with strings when the
  quote character occurs within them. We assume a string being highlighted
  inside an EOL comment is more acceptable than the opposite.
  */
      'eol-comment': {
        pattern: /(^|\s)".*/m,
        lookbehind: true,
        alias: 'comment'
      },
      keyword: {
        pattern:
          /(\s|\.|^)(?:SCIENTIFIC_WITH_LEADING_ZERO|SCALE_PRESERVING_SCIENTIFIC|RMC_COMMUNICATION_FAILURE|END-ENHANCEMENT-SECTION|MULTIPLY-CORRESPONDING|SUBTRACT-CORRESPONDING|VERIFICATION-MESSAGE|DIVIDE-CORRESPONDING|ENHANCEMENT-SECTION|CURRENCY_CONVERSION|RMC_SYSTEM_FAILURE|START-OF-SELECTION|MOVE-CORRESPONDING|RMC_INVALID_STATUS|CUSTOMER-FUNCTION|END-OF-DEFINITION|ENHANCEMENT-POINT|SYSTEM-EXCEPTIONS|ADD-CORRESPONDING|SCALE_PRESERVING|SELECTION-SCREEN|CURSOR-SELECTION|END-OF-SELECTION|LOAD-OF-PROGRAM|SCROLL-BOUNDARY|SELECTION-TABLE|EXCEPTION-TABLE|IMPLEMENTATIONS|PARAMETER-TABLE|RIGHT-JUSTIFIED|UNIT_CONVERSION|AUTHORITY-CHECK|LIST-PROCESSING|SIGN_AS_POSTFIX|COL_BACKGROUND|IMPLEMENTATION|INTERFACE-POOL|TRANSFORMATION|IDENTIFICATION|ENDENHANCEMENT|LINE-SELECTION|INITIALIZATION|LEFT-JUSTIFIED|SELECT-OPTIONS|SELECTION-SETS|COMMUNICATION|CORRESPONDING|DECIMAL_SHIFT|PRINT-CONTROL|VALUE-REQUEST|CHAIN-REQUEST|FUNCTION-POOL|FIELD-SYMBOLS|FUNCTIONALITY|INVERTED-DATE|SELECTION-SET|CLASS-METHODS|OUTPUT-LENGTH|CLASS-CODING|COL_NEGATIVE|ERRORMESSAGE|FIELD-GROUPS|HELP-REQUEST|NO-EXTENSION|NO-TOPOFPAGE|REDEFINITION|DISPLAY-MODE|ENDINTERFACE|EXIT-COMMAND|FIELD-SYMBOL|NO-SCROLLING|SHORTDUMP-ID|ACCESSPOLICY|CLASS-EVENTS|COL_POSITIVE|DECLARATIONS|ENHANCEMENTS|FILTER-TABLE|SWITCHSTATES|SYNTAX-CHECK|TRANSPORTING|ASYNCHRONOUS|SYNTAX-TRACE|TOKENIZATION|USER-COMMAND|WITH-HEADING|ABAP-SOURCE|BREAK-POINT|CHAIN-INPUT|COMPRESSION|FIXED-POINT|NEW-SECTION|NON-UNICODE|OCCURRENCES|RESPONSIBLE|SYSTEM-CALL|TRACE-TABLE|ABBREVIATED|CHAR-TO-HEX|END-OF-FILE|ENDFUNCTION|ENVIRONMENT|ASSOCIATION|COL_HEADING|EDITOR-CALL|END-OF-PAGE|ENGINEERING|IMPLEMENTED|INTENSIFIED|RADIOBUTTON|SYSTEM-EXIT|TOP-OF-PAGE|TRANSACTION|APPLICATION|CONCATENATE|DESTINATION|ENHANCEMENT|IMMEDIATELY|NO-GROUPING|PRECOMPILED|REPLACEMENT|TITLE-LINES|ACTIVATION|BYTE-ORDER|CLASS-POOL|CONNECTION|CONVERSION|DEFINITION|DEPARTMENT|EXPIRATION|INHERITING|MESSAGE-ID|NO-HEADING|PERFORMING|QUEUE-ONLY|RIGHTSPACE|SCIENTIFIC|STATUSINFO|STRUCTURES|SYNCPOINTS|WITH-TITLE|ATTRIBUTES|BOUNDARIES|CLASS-DATA|COL_NORMAL|DD\/MM\/YYYY|DESCENDING|INTERFACES|LINE-COUNT|MM\/DD\/YYYY|NON-UNIQUE|PRESERVING|SELECTIONS|STATEMENTS|SUBROUTINE|TRUNCATION|TYPE-POOLS|ARITHMETIC|BACKGROUND|ENDPROVIDE|EXCEPTIONS|IDENTIFIER|INDEX-LINE|OBLIGATORY|PARAMETERS|PERCENTAGE|PUSHBUTTON|RESOLUTION|COMPONENTS|DEALLOCATE|DISCONNECT|DUPLICATES|FIRST-LINE|HEAD-LINES|NO-DISPLAY|OCCURRENCE|RESPECTING|RETURNCODE|SUBMATCHES|TRACE-FILE|ASCENDING|BYPASSING|ENDMODULE|EXCEPTION|EXCLUDING|EXPORTING|INCREMENT|MATCHCODE|PARAMETER|PARTIALLY|PREFERRED|REFERENCE|REPLACING|RETURNING|SELECTION|SEPARATED|SPECIFIED|STATEMENT|TIMESTAMP|TYPE-POOL|ACCEPTING|APPENDAGE|ASSIGNING|COL_GROUP|COMPARING|CONSTANTS|DANGEROUS|IMPORTING|INSTANCES|LEFTSPACE|LOG-POINT|QUICKINFO|READ-ONLY|SCROLLING|SQLSCRIPT|STEP-LOOP|TOP-LINES|TRANSLATE|APPENDING|AUTHORITY|CHARACTER|COMPONENT|CONDITION|DIRECTORY|DUPLICATE|MESSAGING|RECEIVING|SUBSCREEN|ACCORDING|COL_TOTAL|END-LINES|ENDMETHOD|ENDSELECT|EXPANDING|EXTENSION|INCLUDING|INFOTYPES|INTERFACE|INTERVALS|LINE-SIZE|PF-STATUS|PROCEDURE|PROTECTED|REQUESTED|RESUMABLE|RIGHTPLUS|SAP-SPOOL|SECONDARY|STRUCTURE|SUBSTRING|TABLEVIEW|NUMOFCHAR|ADJACENT|ANALYSIS|ASSIGNED|BACKWARD|CHANNELS|CHECKBOX|CONTINUE|CRITICAL|DATAINFO|DD\/MM\/YY|DURATION|ENCODING|ENDCLASS|FUNCTION|LEFTPLUS|LINEFEED|MM\/DD\/YY|OVERFLOW|RECEIVED|SKIPPING|SORTABLE|STANDARD|SUBTRACT|SUPPRESS|TABSTRIP|TITLEBAR|TRUNCATE|UNASSIGN|WHENEVER|ANALYZER|COALESCE|COMMENTS|CONDENSE|DECIMALS|DEFERRED|ENDWHILE|EXPLICIT|KEYWORDS|MESSAGES|POSITION|PRIORITY|RECEIVER|RENAMING|TIMEZONE|TRAILING|ALLOCATE|CENTERED|CIRCULAR|CONTROLS|CURRENCY|DELETING|DESCRIBE|DISTANCE|ENDCATCH|EXPONENT|EXTENDED|GENERATE|IGNORING|INCLUDES|INTERNAL|MAJOR-ID|MODIFIER|NEW-LINE|OPTIONAL|PROPERTY|ROLLBACK|STARTING|SUPPLIED|ABSTRACT|CHANGING|CONTEXTS|CREATING|CUSTOMER|DATABASE|DAYLIGHT|DEFINING|DISTINCT|DIVISION|ENABLING|ENDCHAIN|ESCAPING|HARMLESS|IMPLICIT|INACTIVE|LANGUAGE|MINOR-ID|MULTIPLY|NEW-PAGE|NO-TITLE|POS_HIGH|SEPARATE|TEXTPOOL|TRANSFER|SELECTOR|DBMAXLEN|ITERATOR|ARCHIVE|BIT-XOR|BYTE-CO|COLLECT|COMMENT|CURRENT|DEFAULT|DISPLAY|ENDFORM|EXTRACT|LEADING|LISTBOX|LOCATOR|MEMBERS|METHODS|NESTING|POS_LOW|PROCESS|PROVIDE|RAISING|RESERVE|SECONDS|SUMMARY|VISIBLE|BETWEEN|BIT-AND|BYTE-CS|CLEANUP|COMPUTE|CONTROL|CONVERT|DATASET|ENDCASE|FORWARD|HEADERS|HOTSPOT|INCLUDE|INVERSE|KEEPING|NO-ZERO|OBJECTS|OVERLAY|PADDING|PATTERN|PROGRAM|REFRESH|SECTION|SUMMING|TESTING|VERSION|WINDOWS|WITHOUT|BIT-NOT|BYTE-CA|BYTE-NA|CASTING|CONTEXT|COUNTRY|DYNAMIC|ENABLED|ENDLOOP|EXECUTE|FRIENDS|HANDLER|HEADING|INITIAL|\*-INPUT|LOGFILE|MAXIMUM|MINIMUM|NO-GAPS|NO-SIGN|PRAGMAS|PRIMARY|PRIVATE|REDUCED|REPLACE|REQUEST|RESULTS|UNICODE|WARNING|ALIASES|BYTE-CN|BYTE-NS|CALLING|COL_KEY|COLUMNS|CONNECT|ENDEXEC|ENTRIES|EXCLUDE|FILTERS|FURTHER|HELP-ID|LOGICAL|MAPPING|MESSAGE|NAMETAB|OPTIONS|PACKAGE|PERFORM|RECEIVE|STATICS|VARYING|BINDING|CHARLEN|GREATER|XSTRLEN|ACCEPT|APPEND|DETAIL|ELSEIF|ENDING|ENDTRY|FORMAT|FRAMES|GIVING|HASHED|HEADER|IMPORT|INSERT|MARGIN|MODULE|NATIVE|OBJECT|OFFSET|REMOTE|RESUME|SAVING|SIMPLE|SUBMIT|TABBED|TOKENS|UNIQUE|UNPACK|UPDATE|WINDOW|YELLOW|ACTUAL|ASPECT|CENTER|CURSOR|DELETE|DIALOG|DIVIDE|DURING|ERRORS|EVENTS|EXTEND|FILTER|HANDLE|HAVING|IGNORE|LITTLE|MEMORY|NO-GAP|OCCURS|OPTION|PERSON|PLACES|PUBLIC|REDUCE|REPORT|RESULT|SINGLE|SORTED|SWITCH|SYNTAX|TARGET|VALUES|WRITER|ASSERT|BLOCKS|BOUNDS|BUFFER|CHANGE|COLUMN|COMMIT|CONCAT|COPIES|CREATE|DDMMYY|DEFINE|ENDIAN|ESCAPE|EXPAND|KERNEL|LAYOUT|LEGACY|LEVELS|MMDDYY|NUMBER|OUTPUT|RANGES|READER|RETURN|SCREEN|SEARCH|SELECT|SHARED|SOURCE|STABLE|STATIC|SUBKEY|SUFFIX|TABLES|UNWIND|YYMMDD|ASSIGN|BACKUP|BEFORE|BINARY|BIT-OR|BLANKS|CLIENT|CODING|COMMON|DEMAND|DYNPRO|EXCEPT|EXISTS|EXPORT|FIELDS|GLOBAL|GROUPS|LENGTH|LOCALE|MEDIUM|METHOD|MODIFY|NESTED|OTHERS|REJECT|SCROLL|SUPPLY|SYMBOL|ENDFOR|STRLEN|ALIGN|BEGIN|BOUND|ENDAT|ENTRY|EVENT|FINAL|FLUSH|GRANT|INNER|SHORT|USING|WRITE|AFTER|BLACK|BLOCK|CLOCK|COLOR|COUNT|DUMMY|EMPTY|ENDDO|ENDON|GREEN|INDEX|INOUT|LEAVE|LEVEL|LINES|MODIF|ORDER|OUTER|RANGE|RESET|RETRY|RIGHT|SMART|SPLIT|STYLE|TABLE|THROW|UNDER|UNTIL|UPPER|UTF-8|WHERE|ALIAS|BLANK|CLEAR|CLOSE|EXACT|FETCH|FIRST|FOUND|GROUP|LLANG|LOCAL|OTHER|REGEX|SPOOL|TITLE|TYPES|VALID|WHILE|ALPHA|BOXED|CATCH|CHAIN|CHECK|CLASS|COVER|ENDIF|EQUIV|FIELD|FLOOR|FRAME|INPUT|LOWER|MATCH|NODES|PAGES|PRINT|RAISE|ROUND|SHIFT|SPACE|SPOTS|STAMP|STATE|TASKS|TIMES|TRMAC|ULINE|UNION|VALUE|WIDTH|EQUAL|LOG10|TRUNC|BLOB|CASE|CEIL|CLOB|COND|EXIT|FILE|GAPS|HOLD|INCL|INTO|KEEP|KEYS|LAST|LINE|LONG|LPAD|MAIL|MODE|OPEN|PINK|READ|ROWS|TEST|THEN|ZERO|AREA|BACK|BADI|BYTE|CAST|EDIT|EXEC|FAIL|FIND|FKEQ|FONT|FREE|GKEQ|HIDE|INIT|ITNO|LATE|LOOP|MAIN|MARK|MOVE|NEXT|NULL|RISK|ROLE|UNIT|WAIT|ZONE|BASE|CALL|CODE|DATA|DATE|FKGE|GKGE|HIGH|KIND|LEFT|LIST|MASK|MESH|NAME|NODE|PACK|PAGE|POOL|SEND|SIGN|SIZE|SOME|STOP|TASK|TEXT|TIME|USER|VARY|WITH|WORD|BLUE|CONV|COPY|DEEP|ELSE|FORM|FROM|HINT|ICON|JOIN|LIKE|LOAD|ONLY|PART|SCAN|SKIP|SORT|TYPE|UNIX|VIEW|WHEN|WORK|ACOS|ASIN|ATAN|COSH|EACH|FRAC|LESS|RTTI|SINH|SQRT|TANH|AVG|BIT|DIV|ISO|LET|OUT|PAD|SQL|ALL|CI_|CPI|END|LOB|LPI|MAX|MIN|NEW|OLE|RUN|SET|\?TO|YES|ABS|ADD|AND|BIG|FOR|HDB|JOB|LOW|NOT|SAP|TRY|VIA|XML|ANY|GET|IDS|KEY|MOD|OFF|PUT|RAW|RED|REF|SUM|TAB|XSD|CNT|COS|EXP|LOG|SIN|TAN|XOR|AT|CO|CP|DO|GT|ID|IF|NS|OR|BT|CA|CS|GE|NA|NB|EQ|IN|LT|NE|NO|OF|ON|PF|TO|AS|BY|CN|IS|LE|NP|UP|E|I|M|O|Z|C|X)\b/i,
        lookbehind: true
      },
      /* Numbers can be only integers. Decimal or Hex appear only as strings */
      number: /\b\d+\b/,
      /* Operators must always be surrounded by whitespace, they cannot be put
  adjacent to operands.
  */
      operator: {
        pattern: /(\s)(?:\*\*?|<[=>]?|>=?|\?=|[-+\/=])(?=\s)/,
        lookbehind: true
      },
      'string-operator': {
        pattern: /(\s)&&?(?=\s)/,
        lookbehind: true,
        /* The official editor highlights */
        alias: 'keyword'
      },
      'token-operator': [
        {
          /* Special operators used to access structure components, class methods/attributes, etc. */
          pattern: /(\w)(?:->?|=>|[~|{}])(?=\w)/,
          lookbehind: true,
          alias: 'punctuation'
        },
        {
          /* Special tokens used do delimit string templates */
          pattern: /[|{}]/,
          alias: 'punctuation'
        }
      ],
      punctuation: /[,.:()]/
    };
  }

  // @ts-nocheck
  abnf.displayName = 'abnf';
  abnf.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function abnf(Prism) {
  (function (Prism) {
      var coreRules =
        '(?:ALPHA|BIT|CHAR|CR|CRLF|CTL|DIGIT|DQUOTE|HEXDIG|HTAB|LF|LWSP|OCTET|SP|VCHAR|WSP)';
      Prism.languages.abnf = {
        comment: /;.*/,
        string: {
          pattern: /(?:%[is])?"[^"\n\r]*"/,
          greedy: true,
          inside: {
            punctuation: /^%[is]/
          }
        },
        range: {
          pattern: /%(?:b[01]+-[01]+|d\d+-\d+|x[A-F\d]+-[A-F\d]+)/i,
          alias: 'number'
        },
        terminal: {
          pattern:
            /%(?:b[01]+(?:\.[01]+)*|d\d+(?:\.\d+)*|x[A-F\d]+(?:\.[A-F\d]+)*)/i,
          alias: 'number'
        },
        repetition: {
          pattern: /(^|[^\w-])(?:\d*\*\d*|\d+)/,
          lookbehind: true,
          alias: 'operator'
        },
        definition: {
          pattern: /(^[ \t]*)(?:[a-z][\w-]*|<[^<>\r\n]*>)(?=\s*=)/m,
          lookbehind: true,
          alias: 'keyword',
          inside: {
            punctuation: /<|>/
          }
        },
        'core-rule': {
          pattern: RegExp(
            '(?:(^|[^<\\w-])' + coreRules + '|<' + coreRules + '>)(?![\\w-])',
            'i'
          ),
          lookbehind: true,
          alias: ['rule', 'constant'],
          inside: {
            punctuation: /<|>/
          }
        },
        rule: {
          pattern: /(^|[^<\w-])[a-z][\w-]*|<[^<>\r\n]*>/i,
          lookbehind: true,
          inside: {
            punctuation: /<|>/
          }
        },
        operator: /=\/?|\//,
        punctuation: /[()\[\]]/
      };
    })(Prism);
  }

  // @ts-nocheck
  actionscript.displayName = 'actionscript';
  actionscript.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function actionscript(Prism) {
    Prism.register(javascript);
    Prism.languages.actionscript = Prism.languages.extend('javascript', {
      keyword:
        /\b(?:as|break|case|catch|class|const|default|delete|do|dynamic|each|else|extends|final|finally|for|function|get|if|implements|import|in|include|instanceof|interface|internal|is|namespace|native|new|null|override|package|private|protected|public|return|set|static|super|switch|this|throw|try|typeof|use|var|void|while|with)\b/,
      operator: /\+\+|--|(?:[+\-*\/%^]|&&?|\|\|?|<<?|>>?>?|[!=]=?)=?|[~?@]/
    });
    Prism.languages.actionscript['class-name'].alias = 'function'; // doesn't work with AS because AS is too complex
    delete Prism.languages.actionscript['parameter'];
    delete Prism.languages.actionscript['literal-property'];
    if (Prism.languages.markup) {
      Prism.languages.insertBefore('actionscript', 'string', {
        xml: {
          pattern:
            /(^|[^.])<\/?\w+(?:\s+[^\s>\/=]+=("|')(?:\\[\s\S]|(?!\2)[^\\])*\2)*\s*\/?>/,
          lookbehind: true,
          inside: Prism.languages.markup
        }
      });
    }
  }

  // @ts-nocheck
  ada.displayName = 'ada';
  ada.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function ada(Prism) {
    Prism.languages.ada = {
      comment: /--.*/,
      string: /"(?:""|[^"\r\f\n])*"/,
      number: [
        {
          pattern:
            /\b\d(?:_?\d)*#[\dA-F](?:_?[\dA-F])*(?:\.[\dA-F](?:_?[\dA-F])*)?#(?:E[+-]?\d(?:_?\d)*)?/i
        },
        {
          pattern: /\b\d(?:_?\d)*(?:\.\d(?:_?\d)*)?(?:E[+-]?\d(?:_?\d)*)?\b/i
        }
      ],
      'attr-name': /\b'\w+/,
      keyword:
        /\b(?:abort|abs|abstract|accept|access|aliased|all|and|array|at|begin|body|case|constant|declare|delay|delta|digits|do|else|elsif|end|entry|exception|exit|for|function|generic|goto|if|in|interface|is|limited|loop|mod|new|not|null|of|others|out|overriding|package|pragma|private|procedure|protected|raise|range|record|rem|renames|requeue|return|reverse|select|separate|some|subtype|synchronized|tagged|task|terminate|then|type|until|use|when|while|with|xor)\b/i,
      boolean: /\b(?:false|true)\b/i,
      operator: /<[=>]?|>=?|=>?|:=|\/=?|\*\*?|[&+-]/,
      punctuation: /\.\.?|[,;():]/,
      char: /'.'/,
      variable: /\b[a-z](?:\w)*\b/i
    };
  }

  // @ts-nocheck
  agda.displayName = 'agda';
  agda.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function agda(Prism) {
  (function (Prism) {
      Prism.languages.agda = {
        comment: /\{-[\s\S]*?(?:-\}|$)|--.*/,
        string: {
          pattern: /"(?:\\(?:\r\n|[\s\S])|[^\\\r\n"])*"/,
          greedy: true
        },
        punctuation: /[(){}⦃⦄.;@]/,
        'class-name': {
          pattern: /((?:data|record) +)\S+/,
          lookbehind: true
        },
        function: {
          pattern: /(^[ \t]*)(?!\s)[^:\r\n]+(?=:)/m,
          lookbehind: true
        },
        operator: {
          pattern: /(^\s*|\s)(?:[=|:∀→λ\\?_]|->)(?=\s)/,
          lookbehind: true
        },
        keyword:
          /\b(?:Set|abstract|constructor|data|eta-equality|field|forall|hiding|import|in|inductive|infix|infixl|infixr|instance|let|macro|module|mutual|no-eta-equality|open|overlap|pattern|postulate|primitive|private|public|quote|quoteContext|quoteGoal|quoteTerm|record|renaming|rewrite|syntax|tactic|unquote|unquoteDecl|unquoteDef|using|variable|where|with)\b/
      };
    })(Prism);
  }

  // @ts-nocheck
  al.displayName = 'al';
  al.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function al(Prism) {
    // based on https://github.com/microsoft/AL/blob/master/grammar/alsyntax.tmlanguage
    Prism.languages.al = {
      comment: /\/\/.*|\/\*[\s\S]*?\*\//,
      string: {
        pattern: /'(?:''|[^'\r\n])*'(?!')|"(?:""|[^"\r\n])*"(?!")/,
        greedy: true
      },
      function: {
        pattern:
          /(\b(?:event|procedure|trigger)\s+|(?:^|[^.])\.\s*)[a-z_]\w*(?=\s*\()/i,
        lookbehind: true
      },
      keyword: [
        // keywords
        /\b(?:array|asserterror|begin|break|case|do|downto|else|end|event|exit|for|foreach|function|if|implements|in|indataset|interface|internal|local|of|procedure|program|protected|repeat|runonclient|securityfiltering|suppressdispose|temporary|then|to|trigger|until|var|while|with|withevents)\b/i, // objects and metadata that are used like keywords
        /\b(?:action|actions|addafter|addbefore|addfirst|addlast|area|assembly|chartpart|codeunit|column|controladdin|cuegroup|customizes|dataitem|dataset|dotnet|elements|enum|enumextension|extends|field|fieldattribute|fieldelement|fieldgroup|fieldgroups|fields|filter|fixed|grid|group|key|keys|label|labels|layout|modify|moveafter|movebefore|movefirst|movelast|page|pagecustomization|pageextension|part|profile|query|repeater|report|requestpage|schema|separator|systempart|table|tableelement|tableextension|textattribute|textelement|type|usercontrol|value|xmlport)\b/i
      ],
      number:
        /\b(?:0x[\da-f]+|(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?)(?:F|LL?|U(?:LL?)?)?\b/i,
      boolean: /\b(?:false|true)\b/i,
      variable: /\b(?:Curr(?:FieldNo|Page|Report)|x?Rec|RequestOptionsPage)\b/,
      'class-name':
        /\b(?:automation|biginteger|bigtext|blob|boolean|byte|char|clienttype|code|completiontriggererrorlevel|connectiontype|database|dataclassification|datascope|date|dateformula|datetime|decimal|defaultlayout|dialog|dictionary|dotnetassembly|dotnettypedeclaration|duration|errorinfo|errortype|executioncontext|executionmode|fieldclass|fieldref|fieldtype|file|filterpagebuilder|guid|httpclient|httpcontent|httpheaders|httprequestmessage|httpresponsemessage|instream|integer|joker|jsonarray|jsonobject|jsontoken|jsonvalue|keyref|list|moduledependencyinfo|moduleinfo|none|notification|notificationscope|objecttype|option|outstream|pageresult|record|recordid|recordref|reportformat|securityfilter|sessionsettings|tableconnectiontype|tablefilter|testaction|testfield|testfilterfield|testpage|testpermissions|testrequestpage|text|textbuilder|textconst|textencoding|time|transactionmodel|transactiontype|variant|verbosity|version|view|views|webserviceactioncontext|webserviceactionresultcode|xmlattribute|xmlattributecollection|xmlcdata|xmlcomment|xmldeclaration|xmldocument|xmldocumenttype|xmlelement|xmlnamespacemanager|xmlnametable|xmlnode|xmlnodelist|xmlprocessinginstruction|xmlreadoptions|xmltext|xmlwriteoptions)\b/i,
      operator: /\.\.|:[=:]|[-+*/]=?|<>|[<>]=?|=|\b(?:and|div|mod|not|or|xor)\b/i,
      punctuation: /[()\[\]{}:.;,]/
    };
  }

  // @ts-nocheck
  antlr4.displayName = 'antlr4';
  antlr4.aliases = ['g4'];

  /** @type {import('../core.js').Syntax} */
  function antlr4(Prism) {
    Prism.languages.antlr4 = {
      comment: /\/\/.*|\/\*[\s\S]*?(?:\*\/|$)/,
      string: {
        pattern: /'(?:\\.|[^\\'\r\n])*'/,
        greedy: true
      },
      'character-class': {
        pattern: /\[(?:\\.|[^\\\]\r\n])*\]/,
        greedy: true,
        alias: 'regex',
        inside: {
          range: {
            pattern: /([^[]|(?:^|[^\\])(?:\\\\)*\\\[)-(?!\])/,
            lookbehind: true,
            alias: 'punctuation'
          },
          escape:
            /\\(?:u(?:[a-fA-F\d]{4}|\{[a-fA-F\d]+\})|[pP]\{[=\w-]+\}|[^\r\nupP])/,
          punctuation: /[\[\]]/
        }
      },
      action: {
        pattern: /\{(?:[^{}]|\{(?:[^{}]|\{(?:[^{}]|\{[^{}]*\})*\})*\})*\}/,
        greedy: true,
        inside: {
          content: {
            // this might be C, C++, Python, Java, C#, or any other language ANTLR4 compiles to
            pattern: /(\{)[\s\S]+(?=\})/,
            lookbehind: true
          },
          punctuation: /[{}]/
        }
      },
      command: {
        pattern:
          /(->\s*(?!\s))(?:\s*(?:,\s*)?\b[a-z]\w*(?:\s*\([^()\r\n]*\))?)+(?=\s*;)/i,
        lookbehind: true,
        inside: {
          function: /\b\w+(?=\s*(?:[,(]|$))/,
          punctuation: /[,()]/
        }
      },
      annotation: {
        pattern: /@\w+(?:::\w+)*/,
        alias: 'keyword'
      },
      label: {
        pattern: /#[ \t]*\w+/,
        alias: 'punctuation'
      },
      keyword:
        /\b(?:catch|channels|finally|fragment|grammar|import|lexer|locals|mode|options|parser|returns|throws|tokens)\b/,
      definition: [
        {
          pattern: /\b[a-z]\w*(?=\s*:)/,
          alias: ['rule', 'class-name']
        },
        {
          pattern: /\b[A-Z]\w*(?=\s*:)/,
          alias: ['token', 'constant']
        }
      ],
      constant: /\b[A-Z][A-Z_]*\b/,
      operator: /\.\.|->|[|~]|[*+?]\??/,
      punctuation: /[;:()=]/
    };
    Prism.languages.g4 = Prism.languages.antlr4;
  }

  // @ts-nocheck
  apacheconf.displayName = 'apacheconf';
  apacheconf.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function apacheconf(Prism) {
    Prism.languages.apacheconf = {
      comment: /#.*/,
      'directive-inline': {
        pattern:
          /(^[\t ]*)\b(?:AcceptFilter|AcceptPathInfo|AccessFileName|Action|Add(?:Alt|AltByEncoding|AltByType|Charset|DefaultCharset|Description|Encoding|Handler|Icon|IconByEncoding|IconByType|InputFilter|Language|ModuleInfo|OutputFilter|OutputFilterByType|Type)|Alias|AliasMatch|Allow(?:CONNECT|EncodedSlashes|Methods|Override|OverrideList)?|Anonymous(?:_LogEmail|_MustGiveEmail|_NoUserID|_VerifyEmail)?|AsyncRequestWorkerFactor|Auth(?:BasicAuthoritative|BasicFake|BasicProvider|BasicUseDigestAlgorithm|DBDUserPWQuery|DBDUserRealmQuery|DBMGroupFile|DBMType|DBMUserFile|Digest(?:Algorithm|Domain|NonceLifetime|Provider|Qop|ShmemSize)|Form(?:Authoritative|Body|DisableNoStore|FakeBasicAuth|Location|LoginRequiredLocation|LoginSuccessLocation|LogoutLocation|Method|Mimetype|Password|Provider|SitePassphrase|Size|Username)|GroupFile|LDAP(?:AuthorizePrefix|BindAuthoritative|BindDN|BindPassword|CharsetConfig|CompareAsUser|CompareDNOnServer|DereferenceAliases|GroupAttribute|GroupAttributeIsDN|InitialBindAsUser|InitialBindPattern|MaxSubGroupDepth|RemoteUserAttribute|RemoteUserIsDN|SearchAsUser|SubGroupAttribute|SubGroupClass|Url)|Merging|Name|nCache(?:Context|Enable|ProvideFor|SOCache|Timeout)|nzFcgiCheckAuthnProvider|nzFcgiDefineProvider|Type|UserFile|zDBDLoginToReferer|zDBDQuery|zDBDRedirectQuery|zDBMType|zSendForbiddenOnFailure)|BalancerGrowth|BalancerInherit|BalancerMember|BalancerPersist|BrowserMatch|BrowserMatchNoCase|BufferedLogs|BufferSize|Cache(?:DefaultExpire|DetailHeader|DirLength|DirLevels|Disable|Enable|File|Header|IgnoreCacheControl|IgnoreHeaders|IgnoreNoLastMod|IgnoreQueryString|IgnoreURLSessionIdentifiers|KeyBaseURL|LastModifiedFactor|Lock|LockMaxAge|LockPath|MaxExpire|MaxFileSize|MinExpire|MinFileSize|NegotiatedDocs|QuickHandler|ReadSize|ReadTime|Root|Socache(?:MaxSize|MaxTime|MinTime|ReadSize|ReadTime)?|StaleOnError|StoreExpired|StoreNoStore|StorePrivate)|CGIDScriptTimeout|CGIMapExtension|CharsetDefault|CharsetOptions|CharsetSourceEnc|CheckCaseOnly|CheckSpelling|ChrootDir|ContentDigest|CookieDomain|CookieExpires|CookieName|CookieStyle|CookieTracking|CoreDumpDirectory|CustomLog|Dav|DavDepthInfinity|DavGenericLockDB|DavLockDB|DavMinTimeout|DBDExptime|DBDInitSQL|DBDKeep|DBDMax|DBDMin|DBDParams|DBDPersist|DBDPrepareSQL|DBDriver|DefaultIcon|DefaultLanguage|DefaultRuntimeDir|DefaultType|Define|Deflate(?:BufferSize|CompressionLevel|FilterNote|InflateLimitRequestBody|InflateRatio(?:Burst|Limit)|MemLevel|WindowSize)|Deny|DirectoryCheckHandler|DirectoryIndex|DirectoryIndexRedirect|DirectorySlash|DocumentRoot|DTracePrivileges|DumpIOInput|DumpIOOutput|EnableExceptionHook|EnableMMAP|EnableSendfile|Error|ErrorDocument|ErrorLog|ErrorLogFormat|Example|ExpiresActive|ExpiresByType|ExpiresDefault|ExtendedStatus|ExtFilterDefine|ExtFilterOptions|FallbackResource|FileETag|FilterChain|FilterDeclare|FilterProtocol|FilterProvider|FilterTrace|ForceLanguagePriority|ForceType|ForensicLog|GprofDir|GracefulShutdownTimeout|Group|Header|HeaderName|Heartbeat(?:Address|Listen|MaxServers|Storage)|HostnameLookups|IdentityCheck|IdentityCheckTimeout|ImapBase|ImapDefault|ImapMenu|Include|IncludeOptional|Index(?:HeadInsert|Ignore|IgnoreReset|Options|OrderDefault|StyleSheet)|InputSed|ISAPI(?:AppendLogToErrors|AppendLogToQuery|CacheFile|FakeAsync|LogNotSupported|ReadAheadBuffer)|KeepAlive|KeepAliveTimeout|KeptBodySize|LanguagePriority|LDAP(?:CacheEntries|CacheTTL|ConnectionPoolTTL|ConnectionTimeout|LibraryDebug|OpCacheEntries|OpCacheTTL|ReferralHopLimit|Referrals|Retries|RetryDelay|SharedCacheFile|SharedCacheSize|Timeout|TrustedClientCert|TrustedGlobalCert|TrustedMode|VerifyServerCert)|Limit(?:InternalRecursion|Request(?:Body|Fields|FieldSize|Line)|XMLRequestBody)|Listen|ListenBackLog|LoadFile|LoadModule|LogFormat|LogLevel|LogMessage|LuaAuthzProvider|LuaCodeCache|Lua(?:Hook(?:AccessChecker|AuthChecker|CheckUserID|Fixups|InsertFilter|Log|MapToStorage|TranslateName|TypeChecker)|Inherit|InputFilter|MapHandler|OutputFilter|PackageCPath|PackagePath|QuickHandler|Root|Scope)|Max(?:ConnectionsPerChild|KeepAliveRequests|MemFree|RangeOverlaps|RangeReversals|Ranges|RequestWorkers|SpareServers|SpareThreads|Threads)|MergeTrailers|MetaDir|MetaFiles|MetaSuffix|MimeMagicFile|MinSpareServers|MinSpareThreads|MMapFile|ModemStandard|ModMimeUsePathInfo|MultiviewsMatch|Mutex|NameVirtualHost|NoProxy|NWSSLTrustedCerts|NWSSLUpgradeable|Options|Order|OutputSed|PassEnv|PidFile|PrivilegesMode|Protocol|ProtocolEcho|Proxy(?:AddHeaders|BadHeader|Block|Domain|ErrorOverride|ExpressDBMFile|ExpressDBMType|ExpressEnable|FtpDirCharset|FtpEscapeWildcards|FtpListOnWildcard|HTML(?:BufSize|CharsetOut|DocType|Enable|Events|Extended|Fixups|Interp|Links|Meta|StripComments|URLMap)|IOBufferSize|MaxForwards|Pass(?:Inherit|InterpolateEnv|Match|Reverse|ReverseCookieDomain|ReverseCookiePath)?|PreserveHost|ReceiveBufferSize|Remote|RemoteMatch|Requests|SCGIInternalRedirect|SCGISendfile|Set|SourceAddress|Status|Timeout|Via)|ReadmeName|ReceiveBufferSize|Redirect|RedirectMatch|RedirectPermanent|RedirectTemp|ReflectorHeader|RemoteIP(?:Header|InternalProxy|InternalProxyList|ProxiesHeader|TrustedProxy|TrustedProxyList)|RemoveCharset|RemoveEncoding|RemoveHandler|RemoveInputFilter|RemoveLanguage|RemoveOutputFilter|RemoveType|RequestHeader|RequestReadTimeout|Require|Rewrite(?:Base|Cond|Engine|Map|Options|Rule)|RLimitCPU|RLimitMEM|RLimitNPROC|Satisfy|ScoreBoardFile|Script(?:Alias|AliasMatch|InterpreterSource|Log|LogBuffer|LogLength|Sock)?|SecureListen|SeeRequestTail|SendBufferSize|Server(?:Admin|Alias|Limit|Name|Path|Root|Signature|Tokens)|Session(?:Cookie(?:Name|Name2|Remove)|Crypto(?:Cipher|Driver|Passphrase|PassphraseFile)|DBD(?:CookieName|CookieName2|CookieRemove|DeleteLabel|InsertLabel|PerUser|SelectLabel|UpdateLabel)|Env|Exclude|Header|Include|MaxAge)?|SetEnv|SetEnvIf|SetEnvIfExpr|SetEnvIfNoCase|SetHandler|SetInputFilter|SetOutputFilter|SSIEndTag|SSIErrorMsg|SSIETag|SSILastModified|SSILegacyExprParser|SSIStartTag|SSITimeFormat|SSIUndefinedEcho|SSL(?:CACertificateFile|CACertificatePath|CADNRequestFile|CADNRequestPath|CARevocationCheck|CARevocationFile|CARevocationPath|CertificateChainFile|CertificateFile|CertificateKeyFile|CipherSuite|Compression|CryptoDevice|Engine|FIPS|HonorCipherOrder|InsecureRenegotiation|OCSP(?:DefaultResponder|Enable|OverrideResponder|ResponderTimeout|ResponseMaxAge|ResponseTimeSkew|UseRequestNonce)|OpenSSLConfCmd|Options|PassPhraseDialog|Protocol|Proxy(?:CACertificateFile|CACertificatePath|CARevocation(?:Check|File|Path)|CheckPeer(?:CN|Expire|Name)|CipherSuite|Engine|MachineCertificate(?:ChainFile|File|Path)|Protocol|Verify|VerifyDepth)|RandomSeed|RenegBufferSize|Require|RequireSSL|Session(?:Cache|CacheTimeout|TicketKeyFile|Tickets)|SRPUnknownUserSeed|SRPVerifierFile|Stapling(?:Cache|ErrorCacheTimeout|FakeTryLater|ForceURL|ResponderTimeout|ResponseMaxAge|ResponseTimeSkew|ReturnResponderErrors|StandardCacheTimeout)|StrictSNIVHostCheck|UserName|UseStapling|VerifyClient|VerifyDepth)|StartServers|StartThreads|Substitute|Suexec|SuexecUserGroup|ThreadLimit|ThreadsPerChild|ThreadStackSize|TimeOut|TraceEnable|TransferLog|TypesConfig|UnDefine|UndefMacro|UnsetEnv|Use|UseCanonicalName|UseCanonicalPhysicalPort|User|UserDir|VHostCGIMode|VHostCGIPrivs|VHostGroup|VHostPrivs|VHostSecure|VHostUser|Virtual(?:DocumentRoot|ScriptAlias)(?:IP)?|WatchdogInterval|XBitHack|xml2EncAlias|xml2EncDefault|xml2StartParse)\b/im,
        lookbehind: true,
        alias: 'property'
      },
      'directive-block': {
        pattern:
          /<\/?\b(?:Auth[nz]ProviderAlias|Directory|DirectoryMatch|Else|ElseIf|Files|FilesMatch|If|IfDefine|IfModule|IfVersion|Limit|LimitExcept|Location|LocationMatch|Macro|Proxy|Require(?:All|Any|None)|VirtualHost)\b.*>/i,
        inside: {
          'directive-block': {
            pattern: /^<\/?\w+/,
            inside: {
              punctuation: /^<\/?/
            },
            alias: 'tag'
          },
          'directive-block-parameter': {
            pattern: /.*[^>]/,
            inside: {
              punctuation: /:/,
              string: {
                pattern: /("|').*\1/,
                inside: {
                  variable: /[$%]\{?(?:\w\.?[-+:]?)+\}?/
                }
              }
            },
            alias: 'attr-value'
          },
          punctuation: />/
        },
        alias: 'tag'
      },
      'directive-flags': {
        pattern: /\[(?:[\w=],?)+\]/,
        alias: 'keyword'
      },
      string: {
        pattern: /("|').*\1/,
        inside: {
          variable: /[$%]\{?(?:\w\.?[-+:]?)+\}?/
        }
      },
      variable: /[$%]\{?(?:\w\.?[-+:]?)+\}?/,
      regex: /\^?.*\$|\^.*\$?/
    };
  }

  // @ts-nocheck
  apex.displayName = 'apex';
  apex.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function apex(Prism) {
    Prism.register(clike);
    Prism.register(sql)
    ;(function (Prism) {
      var keywords =
        /\b(?:(?:after|before)(?=\s+[a-z])|abstract|activate|and|any|array|as|asc|autonomous|begin|bigdecimal|blob|boolean|break|bulk|by|byte|case|cast|catch|char|class|collect|commit|const|continue|currency|date|datetime|decimal|default|delete|desc|do|double|else|end|enum|exception|exit|export|extends|final|finally|float|for|from|get(?=\s*[{};])|global|goto|group|having|hint|if|implements|import|in|inner|insert|instanceof|int|integer|interface|into|join|like|limit|list|long|loop|map|merge|new|not|null|nulls|number|object|of|on|or|outer|override|package|parallel|pragma|private|protected|public|retrieve|return|rollback|select|set|short|sObject|sort|static|string|super|switch|synchronized|system|testmethod|then|this|throw|time|transaction|transient|trigger|try|undelete|update|upsert|using|virtual|void|webservice|when|where|while|(?:inherited|with|without)\s+sharing)\b/i;
      var className =
        /\b(?:(?=[a-z_]\w*\s*[<\[])|(?!<keyword>))[A-Z_]\w*(?:\s*\.\s*[A-Z_]\w*)*\b(?:\s*(?:\[\s*\]|<(?:[^<>]|<(?:[^<>]|<[^<>]*>)*>)*>))*/.source.replace(
          /<keyword>/g,
          function () {
            return keywords.source
          }
        );
      /** @param {string} pattern */
      function insertClassName(pattern) {
        return RegExp(
          pattern.replace(/<CLASS-NAME>/g, function () {
            return className
          }),
          'i'
        )
      }
      var classNameInside = {
        keyword: keywords,
        punctuation: /[()\[\]{};,:.<>]/
      };
      Prism.languages.apex = {
        comment: Prism.languages.clike.comment,
        string: Prism.languages.clike.string,
        sql: {
          pattern: /((?:[=,({:]|\breturn)\s*)\[[^\[\]]*\]/i,
          lookbehind: true,
          greedy: true,
          alias: 'language-sql',
          inside: Prism.languages.sql
        },
        annotation: {
          pattern: /@\w+\b/,
          alias: 'punctuation'
        },
        'class-name': [
          {
            pattern: insertClassName(
              /(\b(?:class|enum|extends|implements|instanceof|interface|new|trigger\s+\w+\s+on)\s+)<CLASS-NAME>/
                .source
            ),
            lookbehind: true,
            inside: classNameInside
          },
          {
            // cast
            pattern: insertClassName(
              /(\(\s*)<CLASS-NAME>(?=\s*\)\s*[\w(])/.source
            ),
            lookbehind: true,
            inside: classNameInside
          },
          {
            // variable/parameter declaration and return types
            pattern: insertClassName(/<CLASS-NAME>(?=\s*\w+\s*[;=,(){:])/.source),
            inside: classNameInside
          }
        ],
        trigger: {
          pattern: /(\btrigger\s+)\w+\b/i,
          lookbehind: true,
          alias: 'class-name'
        },
        keyword: keywords,
        function: /\b[a-z_]\w*(?=\s*\()/i,
        boolean: /\b(?:false|true)\b/i,
        number: /(?:\B\.\d+|\b\d+(?:\.\d+|L)?)\b/i,
        operator:
          /[!=](?:==?)?|\?\.?|&&|\|\||--|\+\+|[-+*/^&|]=?|:|<<?=?|>{1,3}=?/,
        punctuation: /[()\[\]{};,.]/
      };
    })(Prism);
  }

  // @ts-nocheck
  apl.displayName = 'apl';
  apl.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function apl(Prism) {
    Prism.languages.apl = {
      comment: /(?:⍝|#[! ]).*$/m,
      string: {
        pattern: /'(?:[^'\r\n]|'')*'/,
        greedy: true
      },
      number:
        /¯?(?:\d*\.?\b\d+(?:e[+¯]?\d+)?|¯|∞)(?:j¯?(?:(?:\d+(?:\.\d+)?|\.\d+)(?:e[+¯]?\d+)?|¯|∞))?/i,
      statement: /:[A-Z][a-z][A-Za-z]*\b/,
      'system-function': {
        pattern: /⎕[A-Z]+/i,
        alias: 'function'
      },
      constant: /[⍬⌾#⎕⍞]/,
      function: /[-+×÷⌈⌊∣|⍳⍸?*⍟○!⌹<≤=>≥≠≡≢∊⍷∪∩~∨∧⍱⍲⍴,⍪⌽⊖⍉↑↓⊂⊃⊆⊇⌷⍋⍒⊤⊥⍕⍎⊣⊢⍁⍂≈⍯↗¤→]/,
      'monadic-operator': {
        pattern: /[\\\/⌿⍀¨⍨⌶&∥]/,
        alias: 'operator'
      },
      'dyadic-operator': {
        pattern: /[.⍣⍠⍤∘⌸@⌺⍥]/,
        alias: 'operator'
      },
      assignment: {
        pattern: /←/,
        alias: 'keyword'
      },
      punctuation: /[\[;\]()◇⋄]/,
      dfn: {
        pattern: /[{}⍺⍵⍶⍹∇⍫:]/,
        alias: 'builtin'
      }
    };
  }

  // @ts-nocheck
  applescript.displayName = 'applescript';
  applescript.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function applescript(Prism) {
    Prism.languages.applescript = {
      comment: [
        // Allow one level of nesting
        /\(\*(?:\(\*(?:[^*]|\*(?!\)))*\*\)|(?!\(\*)[\s\S])*?\*\)/,
        /--.+/,
        /#.+/
      ],
      string: /"(?:\\.|[^"\\\r\n])*"/,
      number: /(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e-?\d+)?\b/i,
      operator: [
        /[&=≠≤≥*+\-\/÷^]|[<>]=?/,
        /\b(?:(?:begin|end|start)s? with|(?:contains?|(?:does not|doesn't) contain)|(?:is|isn't|is not) (?:contained by|in)|(?:(?:is|isn't|is not) )?(?:greater|less) than(?: or equal)?(?: to)?|(?:comes|(?:does not|doesn't) come) (?:after|before)|(?:is|isn't|is not) equal(?: to)?|(?:(?:does not|doesn't) equal|equal to|equals|is not|isn't)|(?:a )?(?:ref(?: to)?|reference to)|(?:and|as|div|mod|not|or))\b/
      ],
      keyword:
        /\b(?:about|above|after|against|apart from|around|aside from|at|back|before|beginning|behind|below|beneath|beside|between|but|by|considering|continue|copy|does|eighth|else|end|equal|error|every|exit|false|fifth|first|for|fourth|from|front|get|given|global|if|ignoring|in|instead of|into|is|it|its|last|local|me|middle|my|ninth|of|on|onto|out of|over|prop|property|put|repeat|return|returning|second|set|seventh|since|sixth|some|tell|tenth|that|the|then|third|through|thru|timeout|times|to|transaction|true|try|until|where|while|whose|with|without)\b/,
      'class-name':
        /\b(?:POSIX file|RGB color|alias|application|boolean|centimeters|centimetres|class|constant|cubic centimeters|cubic centimetres|cubic feet|cubic inches|cubic meters|cubic metres|cubic yards|date|degrees Celsius|degrees Fahrenheit|degrees Kelvin|feet|file|gallons|grams|inches|integer|kilograms|kilometers|kilometres|list|liters|litres|meters|metres|miles|number|ounces|pounds|quarts|real|record|reference|script|square feet|square kilometers|square kilometres|square meters|square metres|square miles|square yards|text|yards)\b/,
      punctuation: /[{}():,¬«»《》]/
    };
  }

  // @ts-nocheck
  aql.displayName = 'aql';
  aql.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function aql(Prism) {
    Prism.languages.aql = {
      comment: /\/\/.*|\/\*[\s\S]*?\*\//,
      property: {
        pattern:
          /([{,]\s*)(?:(?!\d)\w+|(["'´`])(?:(?!\2)[^\\\r\n]|\\.)*\2)(?=\s*:)/,
        lookbehind: true,
        greedy: true
      },
      string: {
        pattern: /(["'])(?:(?!\1)[^\\\r\n]|\\.)*\1/,
        greedy: true
      },
      identifier: {
        pattern: /([´`])(?:(?!\1)[^\\\r\n]|\\.)*\1/,
        greedy: true
      },
      variable: /@@?\w+/,
      keyword: [
        {
          pattern: /(\bWITH\s+)COUNT(?=\s+INTO\b)/i,
          lookbehind: true
        },
        /\b(?:AGGREGATE|ALL|AND|ANY|ASC|COLLECT|DESC|DISTINCT|FILTER|FOR|GRAPH|IN|INBOUND|INSERT|INTO|K_PATHS|K_SHORTEST_PATHS|LET|LIKE|LIMIT|NONE|NOT|NULL|OR|OUTBOUND|REMOVE|REPLACE|RETURN|SHORTEST_PATH|SORT|UPDATE|UPSERT|WINDOW|WITH)\b/i, // pseudo keywords get a lookbehind to avoid false positives
        {
          pattern: /(^|[^\w.[])(?:KEEP|PRUNE|SEARCH|TO)\b/i,
          lookbehind: true
        },
        {
          pattern: /(^|[^\w.[])(?:CURRENT|NEW|OLD)\b/,
          lookbehind: true
        },
        {
          pattern: /\bOPTIONS(?=\s*\{)/i
        }
      ],
      function: /\b(?!\d)\w+(?=\s*\()/,
      boolean: /\b(?:false|true)\b/i,
      range: {
        pattern: /\.\./,
        alias: 'operator'
      },
      number: [
        /\b0b[01]+/i,
        /\b0x[0-9a-f]+/i,
        /(?:\B\.\d+|\b(?:0|[1-9]\d*)(?:\.\d+)?)(?:e[+-]?\d+)?/i
      ],
      operator: /\*{2,}|[=!]~|[!=<>]=?|&&|\|\||[-+*/%]/,
      punctuation: /::|[?.:,;()[\]{}]/
    };
  }

  // @ts-nocheck
  arff.displayName = 'arff';
  arff.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function arff(Prism) {
    Prism.languages.arff = {
      comment: /%.*/,
      string: {
        pattern: /(["'])(?:\\.|(?!\1)[^\\\r\n])*\1/,
        greedy: true
      },
      keyword: /@(?:attribute|data|end|relation)\b/i,
      number: /\b\d+(?:\.\d+)?\b/,
      punctuation: /[{},]/
    };
  }

  // @ts-nocheck
  asciidoc.displayName = 'asciidoc';
  asciidoc.aliases = ['adoc'];

  /** @type {import('../core.js').Syntax} */
  function asciidoc(Prism) {
  (function (Prism) {
      var attributes = {
        pattern:
          /(^[ \t]*)\[(?!\[)(?:(["'$`])(?:(?!\2)[^\\]|\\.)*\2|\[(?:[^\[\]\\]|\\.)*\]|[^\[\]\\"'$`]|\\.)*\]/m,
        lookbehind: true,
        inside: {
          quoted: {
            pattern: /([$`])(?:(?!\1)[^\\]|\\.)*\1/,
            inside: {
              punctuation: /^[$`]|[$`]$/
            }
          },
          interpreted: {
            pattern: /'(?:[^'\\]|\\.)*'/,
            inside: {
              punctuation: /^'|'$/ // See rest below
            }
          },
          string: /"(?:[^"\\]|\\.)*"/,
          variable: /\w+(?==)/,
          punctuation: /^\[|\]$|,/,
          operator: /=/,
          // The negative look-ahead prevents blank matches
          'attr-value': /(?!^\s+$).+/
        }
      };
      var asciidoc = (Prism.languages.asciidoc = {
        'comment-block': {
          pattern: /^(\/{4,})(?:\r?\n|\r)(?:[\s\S]*(?:\r?\n|\r))??\1/m,
          alias: 'comment'
        },
        table: {
          pattern: /^\|={3,}(?:(?:\r?\n|\r(?!\n)).*)*?(?:\r?\n|\r)\|={3,}$/m,
          inside: {
            specifiers: {
              pattern:
                /(?:(?:(?:\d+(?:\.\d+)?|\.\d+)[+*](?:[<^>](?:\.[<^>])?|\.[<^>])?|[<^>](?:\.[<^>])?|\.[<^>])[a-z]*|[a-z]+)(?=\|)/,
              alias: 'attr-value'
            },
            punctuation: {
              pattern: /(^|[^\\])[|!]=*/,
              lookbehind: true
            } // See rest below
          }
        },
        'passthrough-block': {
          pattern: /^(\+{4,})(?:\r?\n|\r)(?:[\s\S]*(?:\r?\n|\r))??\1$/m,
          inside: {
            punctuation: /^\++|\++$/ // See rest below
          }
        },
        // Literal blocks and listing blocks
        'literal-block': {
          pattern: /^(-{4,}|\.{4,})(?:\r?\n|\r)(?:[\s\S]*(?:\r?\n|\r))??\1$/m,
          inside: {
            punctuation: /^(?:-+|\.+)|(?:-+|\.+)$/ // See rest below
          }
        },
        // Sidebar blocks, quote blocks, example blocks and open blocks
        'other-block': {
          pattern:
            /^(--|\*{4,}|_{4,}|={4,})(?:\r?\n|\r)(?:[\s\S]*(?:\r?\n|\r))??\1$/m,
          inside: {
            punctuation: /^(?:-+|\*+|_+|=+)|(?:-+|\*+|_+|=+)$/ // See rest below
          }
        },
        // list-punctuation and list-label must appear before indented-block
        'list-punctuation': {
          pattern:
            /(^[ \t]*)(?:-|\*{1,5}|\.{1,5}|(?:[a-z]|\d+)\.|[xvi]+\))(?= )/im,
          lookbehind: true,
          alias: 'punctuation'
        },
        'list-label': {
          pattern: /(^[ \t]*)[a-z\d].+(?::{2,4}|;;)(?=\s)/im,
          lookbehind: true,
          alias: 'symbol'
        },
        'indented-block': {
          pattern: /((\r?\n|\r)\2)([ \t]+)\S.*(?:(?:\r?\n|\r)\3.+)*(?=\2{2}|$)/,
          lookbehind: true
        },
        comment: /^\/\/.*/m,
        title: {
          pattern:
            /^.+(?:\r?\n|\r)(?:={3,}|-{3,}|~{3,}|\^{3,}|\+{3,})$|^={1,5} .+|^\.(?![\s.]).*/m,
          alias: 'important',
          inside: {
            punctuation: /^(?:\.|=+)|(?:=+|-+|~+|\^+|\++)$/ // See rest below
          }
        },
        'attribute-entry': {
          pattern: /^:[^:\r\n]+:(?: .*?(?: \+(?:\r?\n|\r).*?)*)?$/m,
          alias: 'tag'
        },
        attributes: attributes,
        hr: {
          pattern: /^'{3,}$/m,
          alias: 'punctuation'
        },
        'page-break': {
          pattern: /^<{3,}$/m,
          alias: 'punctuation'
        },
        admonition: {
          pattern: /^(?:CAUTION|IMPORTANT|NOTE|TIP|WARNING):/m,
          alias: 'keyword'
        },
        callout: [
          {
            pattern: /(^[ \t]*)<?\d*>/m,
            lookbehind: true,
            alias: 'symbol'
          },
          {
            pattern: /<\d+>/,
            alias: 'symbol'
          }
        ],
        macro: {
          pattern:
            /\b[a-z\d][a-z\d-]*::?(?:[^\s\[\]]*\[(?:[^\]\\"']|(["'])(?:(?!\1)[^\\]|\\.)*\1|\\.)*\])/,
          inside: {
            function: /^[a-z\d-]+(?=:)/,
            punctuation: /^::?/,
            attributes: {
              pattern: /(?:\[(?:[^\]\\"']|(["'])(?:(?!\1)[^\\]|\\.)*\1|\\.)*\])/,
              inside: attributes.inside
            }
          }
        },
        inline: {
          /*
  The initial look-behind prevents the highlighting of escaped quoted text.
  Quoted text can be multi-line but cannot span an empty line.
  All quoted text can have attributes before [foobar, 'foobar', baz="bar"].
  First, we handle the constrained quotes.
  Those must be bounded by non-word chars and cannot have spaces between the delimiter and the first char.
  They are, in order: _emphasis_, ``double quotes'', `single quotes', `monospace`, 'emphasis', *strong*, +monospace+ and #unquoted#
  Then we handle the unconstrained quotes.
  Those do not have the restrictions of the constrained quotes.
  They are, in order: __emphasis__, **strong**, ++monospace++, +++passthrough+++, ##unquoted##, $$passthrough$$, ~subscript~, ^superscript^, {attribute-reference}, [[anchor]], [[[bibliography anchor]]], <<xref>>, (((indexes))) and ((indexes))
  */
          pattern:
            /(^|[^\\])(?:(?:\B\[(?:[^\]\\"']|(["'])(?:(?!\2)[^\\]|\\.)*\2|\\.)*\])?(?:\b_(?!\s)(?: _|[^_\\\r\n]|\\.)+(?:(?:\r?\n|\r)(?: _|[^_\\\r\n]|\\.)+)*_\b|\B``(?!\s).+?(?:(?:\r?\n|\r).+?)*''\B|\B`(?!\s)(?:[^`'\s]|\s+\S)+['`]\B|\B(['*+#])(?!\s)(?: \3|(?!\3)[^\\\r\n]|\\.)+(?:(?:\r?\n|\r)(?: \3|(?!\3)[^\\\r\n]|\\.)+)*\3\B)|(?:\[(?:[^\]\\"']|(["'])(?:(?!\4)[^\\]|\\.)*\4|\\.)*\])?(?:(__|\*\*|\+\+\+?|##|\$\$|[~^]).+?(?:(?:\r?\n|\r).+?)*\5|\{[^}\r\n]+\}|\[\[\[?.+?(?:(?:\r?\n|\r).+?)*\]?\]\]|<<.+?(?:(?:\r?\n|\r).+?)*>>|\(\(\(?.+?(?:(?:\r?\n|\r).+?)*\)?\)\)))/m,
          lookbehind: true,
          inside: {
            attributes: attributes,
            url: {
              pattern: /^(?:\[\[\[?.+?\]?\]\]|<<.+?>>)$/,
              inside: {
                punctuation: /^(?:\[\[\[?|<<)|(?:\]\]\]?|>>)$/
              }
            },
            'attribute-ref': {
              pattern: /^\{.+\}$/,
              inside: {
                variable: {
                  pattern: /(^\{)[a-z\d,+_-]+/,
                  lookbehind: true
                },
                operator: /^[=?!#%@$]|!(?=[:}])/,
                punctuation: /^\{|\}$|::?/
              }
            },
            italic: {
              pattern: /^(['_])[\s\S]+\1$/,
              inside: {
                punctuation: /^(?:''?|__?)|(?:''?|__?)$/
              }
            },
            bold: {
              pattern: /^\*[\s\S]+\*$/,
              inside: {
                punctuation: /^\*\*?|\*\*?$/
              }
            },
            punctuation:
              /^(?:``?|\+{1,3}|##?|\$\$|[~^]|\(\(\(?)|(?:''?|\+{1,3}|##?|\$\$|[~^`]|\)?\)\))$/
          }
        },
        replacement: {
          pattern: /\((?:C|R|TM)\)/,
          alias: 'builtin'
        },
        entity: /&#?[\da-z]{1,8};/i,
        'line-continuation': {
          pattern: /(^| )\+$/m,
          lookbehind: true,
          alias: 'punctuation'
        }
      }); // Allow some nesting. There is no recursion though, so cloning should not be needed.
      function copyFromAsciiDoc(keys) {
        keys = keys.split(' ');
        var o = {};
        for (var i = 0, l = keys.length; i < l; i++) {
          o[keys[i]] = asciidoc[keys[i]];
        }
        return o
      }
      attributes.inside['interpreted'].inside.rest = copyFromAsciiDoc(
        'macro inline replacement entity'
      );
      asciidoc['passthrough-block'].inside.rest = copyFromAsciiDoc('macro');
      asciidoc['literal-block'].inside.rest = copyFromAsciiDoc('callout');
      asciidoc['table'].inside.rest = copyFromAsciiDoc(
        'comment-block passthrough-block literal-block other-block list-punctuation indented-block comment title attribute-entry attributes hr page-break admonition list-label callout macro inline replacement entity line-continuation'
      );
      asciidoc['other-block'].inside.rest = copyFromAsciiDoc(
        'table list-punctuation indented-block comment attribute-entry attributes hr page-break admonition list-label macro inline replacement entity line-continuation'
      );
      asciidoc['title'].inside.rest = copyFromAsciiDoc(
        'macro inline replacement entity'
      ); // Plugin to make entity title show the real entity, idea by Roman Komarov
      Prism.hooks.add('wrap', function (env) {
        if (env.type === 'entity') {
          env.attributes['title'] = env.content.value.replace(/&amp;/, '&');
        }
      });
      Prism.languages.adoc = Prism.languages.asciidoc;
    })(Prism);
  }

  // @ts-nocheck
  aspnet.displayName = 'aspnet';
  aspnet.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function aspnet(Prism) {
    Prism.register(csharp);
    Prism.register(markup);
    Prism.languages.aspnet = Prism.languages.extend('markup', {
      'page-directive': {
        pattern: /<%\s*@.*%>/,
        alias: 'tag',
        inside: {
          'page-directive': {
            pattern:
              /<%\s*@\s*(?:Assembly|Control|Implements|Import|Master(?:Type)?|OutputCache|Page|PreviousPageType|Reference|Register)?|%>/i,
            alias: 'tag'
          },
          rest: Prism.languages.markup.tag.inside
        }
      },
      directive: {
        pattern: /<%.*%>/,
        alias: 'tag',
        inside: {
          directive: {
            pattern: /<%\s*?[$=%#:]{0,2}|%>/,
            alias: 'tag'
          },
          rest: Prism.languages.csharp
        }
      }
    }); // Regexp copied from prism-markup, with a negative look-ahead added
    Prism.languages.aspnet.tag.pattern =
      /<(?!%)\/?[^\s>\/]+(?:\s+[^\s>\/=]+(?:=(?:("|')(?:\\[\s\S]|(?!\1)[^\\])*\1|[^\s'">=]+))?)*\s*\/?>/; // match directives of attribute value foo="<% Bar %>"
    Prism.languages.insertBefore(
      'inside',
      'punctuation',
      {
        directive: Prism.languages.aspnet['directive']
      },
      Prism.languages.aspnet.tag.inside['attr-value']
    );
    Prism.languages.insertBefore('aspnet', 'comment', {
      'asp-comment': {
        pattern: /<%--[\s\S]*?--%>/,
        alias: ['asp', 'comment']
      }
    }); // script runat="server" contains csharp, not javascript
    Prism.languages.insertBefore(
      'aspnet',
      Prism.languages.javascript ? 'script' : 'tag',
      {
        'asp-script': {
          pattern:
            /(<script(?=.*runat=['"]?server\b)[^>]*>)[\s\S]*?(?=<\/script>)/i,
          lookbehind: true,
          alias: ['asp', 'script'],
          inside: Prism.languages.csharp || {}
        }
      }
    );
  }

  // @ts-nocheck
  asm6502.displayName = 'asm6502';
  asm6502.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function asm6502(Prism) {
    Prism.languages.asm6502 = {
      comment: /;.*/,
      directive: {
        pattern: /\.\w+(?= )/,
        alias: 'property'
      },
      string: /(["'`])(?:\\.|(?!\1)[^\\\r\n])*\1/,
      'op-code': {
        pattern:
          /\b(?:ADC|AND|ASL|BCC|BCS|BEQ|BIT|BMI|BNE|BPL|BRK|BVC|BVS|CLC|CLD|CLI|CLV|CMP|CPX|CPY|DEC|DEX|DEY|EOR|INC|INX|INY|JMP|JSR|LDA|LDX|LDY|LSR|NOP|ORA|PHA|PHP|PLA|PLP|ROL|ROR|RTI|RTS|SBC|SEC|SED|SEI|STA|STX|STY|TAX|TAY|TSX|TXA|TXS|TYA|adc|and|asl|bcc|bcs|beq|bit|bmi|bne|bpl|brk|bvc|bvs|clc|cld|cli|clv|cmp|cpx|cpy|dec|dex|dey|eor|inc|inx|iny|jmp|jsr|lda|ldx|ldy|lsr|nop|ora|pha|php|pla|plp|rol|ror|rti|rts|sbc|sec|sed|sei|sta|stx|sty|tax|tay|tsx|txa|txs|tya)\b/,
        alias: 'keyword'
      },
      'hex-number': {
        pattern: /#?\$[\da-f]{1,4}\b/i,
        alias: 'number'
      },
      'binary-number': {
        pattern: /#?%[01]+\b/,
        alias: 'number'
      },
      'decimal-number': {
        pattern: /#?\b\d+\b/,
        alias: 'number'
      },
      register: {
        pattern: /\b[xya]\b/i,
        alias: 'variable'
      },
      punctuation: /[(),:]/
    };
  }

  // @ts-nocheck
  asmatmel.displayName = 'asmatmel';
  asmatmel.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function asmatmel(Prism) {
    Prism.languages.asmatmel = {
      comment: {
        pattern: /;.*/,
        greedy: true
      },
      string: {
        pattern: /(["'`])(?:\\.|(?!\1)[^\\\r\n])*\1/,
        greedy: true
      },
      constant: /\b(?:PORT[A-Z]|DDR[A-Z]|(?:DD|P)[A-Z](?:\d|[0-2]\d|3[01]))\b/,
      directive: {
        pattern: /\.\w+(?= )/,
        alias: 'property'
      },
      'r-register': {
        pattern: /\br(?:\d|[12]\d|3[01])\b/,
        alias: 'variable'
      },
      'op-code': {
        pattern:
          /\b(?:ADC|ADD|ADIW|AND|ANDI|ASR|BCLR|BLD|BRBC|BRBS|BRCC|BRCS|BREAK|BREQ|BRGE|BRHC|BRHS|BRID|BRIE|BRLO|BRLT|BRMI|BRNE|BRPL|BRSH|BRTC|BRTS|BRVC|BRVS|BSET|BST|CALL|CBI|CBR|CLC|CLH|CLI|CLN|CLR|CLS|CLT|CLV|CLZ|COM|CP|CPC|CPI|CPSE|DEC|DES|EICALL|EIJMP|ELPM|EOR|FMUL|FMULS|FMULSU|ICALL|IJMP|IN|INC|JMP|LAC|LAS|LAT|LD|LD[A-Za-z0-9]|LPM|LSL|LSR|MOV|MOVW|MUL|MULS|MULSU|NEG|NOP|OR|ORI|OUT|POP|PUSH|RCALL|RET|RETI|RJMP|ROL|ROR|SBC|SBCI|SBI|SBIC|SBIS|SBIW|SBR|SBRC|SBRS|SEC|SEH|SEI|SEN|SER|SES|SET|SEV|SEZ|SLEEP|SPM|ST|ST[A-Z0-9]|SUB|SUBI|SWAP|TST|WDR|XCH|adc|add|adiw|and|andi|asr|bclr|bld|brbc|brbs|brcc|brcs|break|breq|brge|brhc|brhs|brid|brie|brlo|brlt|brmi|brne|brpl|brsh|brtc|brts|brvc|brvs|bset|bst|call|cbi|cbr|clc|clh|cli|cln|clr|cls|clt|clv|clz|com|cp|cpc|cpi|cpse|dec|des|eicall|eijmp|elpm|eor|fmul|fmuls|fmulsu|icall|ijmp|in|inc|jmp|lac|las|lat|ld|ld[a-z0-9]|lpm|lsl|lsr|mov|movw|mul|muls|mulsu|neg|nop|or|ori|out|pop|push|rcall|ret|reti|rjmp|rol|ror|sbc|sbci|sbi|sbic|sbis|sbiw|sbr|sbrc|sbrs|sec|seh|sei|sen|ser|ses|set|sev|sez|sleep|spm|st|st[a-zA-Z0-9]|sub|subi|swap|tst|wdr|xch)\b/,
        alias: 'keyword'
      },
      'hex-number': {
        pattern: /#?\$[\da-f]{2,4}\b/i,
        alias: 'number'
      },
      'binary-number': {
        pattern: /#?%[01]+\b/,
        alias: 'number'
      },
      'decimal-number': {
        pattern: /#?\b\d+\b/,
        alias: 'number'
      },
      register: {
        pattern: /\b[acznvshtixy]\b/i,
        alias: 'variable'
      },
      operator: />>=?|<<=?|&&?|\|\|?|[-+*/%&|^!=<>?]=?/,
      punctuation: /[(),:]/
    };
  }

  // @ts-nocheck
  autohotkey.displayName = 'autohotkey';
  autohotkey.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function autohotkey(Prism) {
    // NOTES - follows first-first highlight method, block is locked after highlight, different from SyntaxHl
    Prism.languages.autohotkey = {
      comment: [
        {
          pattern: /(^|\s);.*/,
          lookbehind: true
        },
        {
          pattern:
            /(^[\t ]*)\/\*(?:[\r\n](?![ \t]*\*\/)|[^\r\n])*(?:[\r\n][ \t]*\*\/)?/m,
          lookbehind: true,
          greedy: true
        }
      ],
      tag: {
        // labels
        pattern: /^([ \t]*)[^\s,`":]+(?=:[ \t]*$)/m,
        lookbehind: true
      },
      string: /"(?:[^"\n\r]|"")*"/,
      variable: /%\w+%/,
      number: /\b0x[\dA-Fa-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee]-?\d+)?/,
      operator:
        /\?|\/\/?=?|:=|\|[=|]?|&[=&]?|\+[=+]?|-[=-]?|\*[=*]?|<(?:<=?|>|=)?|>>?=?|[.^!=~]=?|\b(?:AND|NOT|OR)\b/,
      boolean: /\b(?:false|true)\b/,
      selector:
        /\b(?:AutoTrim|BlockInput|Break|Click|ClipWait|Continue|Control|ControlClick|ControlFocus|ControlGet|ControlGetFocus|ControlGetPos|ControlGetText|ControlMove|ControlSend|ControlSendRaw|ControlSetText|CoordMode|Critical|DetectHiddenText|DetectHiddenWindows|Drive|DriveGet|DriveSpaceFree|EnvAdd|EnvDiv|EnvGet|EnvMult|EnvSet|EnvSub|EnvUpdate|Exit|ExitApp|FileAppend|FileCopy|FileCopyDir|FileCreateDir|FileCreateShortcut|FileDelete|FileEncoding|FileGetAttrib|FileGetShortcut|FileGetSize|FileGetTime|FileGetVersion|FileInstall|FileMove|FileMoveDir|FileRead|FileReadLine|FileRecycle|FileRecycleEmpty|FileRemoveDir|FileSelectFile|FileSelectFolder|FileSetAttrib|FileSetTime|FormatTime|GetKeyState|Gosub|Goto|GroupActivate|GroupAdd|GroupClose|GroupDeactivate|Gui|GuiControl|GuiControlGet|Hotkey|ImageSearch|IniDelete|IniRead|IniWrite|Input|InputBox|KeyWait|ListHotkeys|ListLines|ListVars|Loop|Menu|MouseClick|MouseClickDrag|MouseGetPos|MouseMove|MsgBox|OnExit|OutputDebug|Pause|PixelGetColor|PixelSearch|PostMessage|Process|Progress|Random|RegDelete|RegRead|RegWrite|Reload|Repeat|Return|Run|RunAs|RunWait|Send|SendEvent|SendInput|SendMessage|SendMode|SendPlay|SendRaw|SetBatchLines|SetCapslockState|SetControlDelay|SetDefaultMouseSpeed|SetEnv|SetFormat|SetKeyDelay|SetMouseDelay|SetNumlockState|SetRegView|SetScrollLockState|SetStoreCapslockMode|SetTimer|SetTitleMatchMode|SetWinDelay|SetWorkingDir|Shutdown|Sleep|Sort|SoundBeep|SoundGet|SoundGetWaveVolume|SoundPlay|SoundSet|SoundSetWaveVolume|SplashImage|SplashTextOff|SplashTextOn|SplitPath|StatusBarGetText|StatusBarWait|StringCaseSense|StringGetPos|StringLeft|StringLen|StringLower|StringMid|StringReplace|StringRight|StringSplit|StringTrimLeft|StringTrimRight|StringUpper|Suspend|SysGet|Thread|ToolTip|Transform|TrayTip|URLDownloadToFile|WinActivate|WinActivateBottom|WinClose|WinGet|WinGetActiveStats|WinGetActiveTitle|WinGetClass|WinGetPos|WinGetText|WinGetTitle|WinHide|WinKill|WinMaximize|WinMenuSelectItem|WinMinimize|WinMinimizeAll|WinMinimizeAllUndo|WinMove|WinRestore|WinSet|WinSetTitle|WinShow|WinWait|WinWaitActive|WinWaitClose|WinWaitNotActive)\b/i,
      constant:
        /\b(?:a_ahkpath|a_ahkversion|a_appdata|a_appdatacommon|a_autotrim|a_batchlines|a_caretx|a_carety|a_computername|a_controldelay|a_cursor|a_dd|a_ddd|a_dddd|a_defaultmousespeed|a_desktop|a_desktopcommon|a_detecthiddentext|a_detecthiddenwindows|a_endchar|a_eventinfo|a_exitreason|a_fileencoding|a_formatfloat|a_formatinteger|a_gui|a_guicontrol|a_guicontrolevent|a_guievent|a_guiheight|a_guiwidth|a_guix|a_guiy|a_hour|a_iconfile|a_iconhidden|a_iconnumber|a_icontip|a_index|a_ipaddress1|a_ipaddress2|a_ipaddress3|a_ipaddress4|a_is64bitos|a_isadmin|a_iscompiled|a_iscritical|a_ispaused|a_issuspended|a_isunicode|a_keydelay|a_language|a_lasterror|a_linefile|a_linenumber|a_loopfield|a_loopfileattrib|a_loopfiledir|a_loopfileext|a_loopfilefullpath|a_loopfilelongpath|a_loopfilename|a_loopfileshortname|a_loopfileshortpath|a_loopfilesize|a_loopfilesizekb|a_loopfilesizemb|a_loopfiletimeaccessed|a_loopfiletimecreated|a_loopfiletimemodified|a_loopreadline|a_loopregkey|a_loopregname|a_loopregsubkey|a_loopregtimemodified|a_loopregtype|a_mday|a_min|a_mm|a_mmm|a_mmmm|a_mon|a_mousedelay|a_msec|a_mydocuments|a_now|a_nowutc|a_numbatchlines|a_ostype|a_osversion|a_priorhotkey|a_priorkey|a_programfiles|a_programs|a_programscommon|a_ptrsize|a_regview|a_screendpi|a_screenheight|a_screenwidth|a_scriptdir|a_scriptfullpath|a_scripthwnd|a_scriptname|a_sec|a_space|a_startmenu|a_startmenucommon|a_startup|a_startupcommon|a_stringcasesense|a_tab|a_temp|a_thisfunc|a_thishotkey|a_thislabel|a_thismenu|a_thismenuitem|a_thismenuitempos|a_tickcount|a_timeidle|a_timeidlephysical|a_timesincepriorhotkey|a_timesincethishotkey|a_titlematchmode|a_titlematchmodespeed|a_username|a_wday|a_windelay|a_windir|a_workingdir|a_yday|a_year|a_yweek|a_yyyy|clipboard|clipboardall|comspec|errorlevel|programfiles)\b/i,
      builtin:
        /\b(?:abs|acos|asc|asin|atan|ceil|chr|class|comobjactive|comobjarray|comobjconnect|comobjcreate|comobjerror|comobjflags|comobjget|comobjquery|comobjtype|comobjvalue|cos|dllcall|exp|fileexist|Fileopen|floor|format|il_add|il_create|il_destroy|instr|isfunc|islabel|IsObject|ln|log|ltrim|lv_add|lv_delete|lv_deletecol|lv_getcount|lv_getnext|lv_gettext|lv_insert|lv_insertcol|lv_modify|lv_modifycol|lv_setimagelist|mod|numget|numput|onmessage|regexmatch|regexreplace|registercallback|round|rtrim|sb_seticon|sb_setparts|sb_settext|sin|sqrt|strlen|strreplace|strsplit|substr|tan|tv_add|tv_delete|tv_get|tv_getchild|tv_getcount|tv_getnext|tv_getparent|tv_getprev|tv_getselection|tv_gettext|tv_modify|varsetcapacity|winactive|winexist|__Call|__Get|__New|__Set)\b/i,
      symbol:
        /\b(?:alt|altdown|altup|appskey|backspace|browser_back|browser_favorites|browser_forward|browser_home|browser_refresh|browser_search|browser_stop|bs|capslock|ctrl|ctrlbreak|ctrldown|ctrlup|del|delete|down|end|enter|esc|escape|f1|f10|f11|f12|f13|f14|f15|f16|f17|f18|f19|f2|f20|f21|f22|f23|f24|f3|f4|f5|f6|f7|f8|f9|home|ins|insert|joy1|joy10|joy11|joy12|joy13|joy14|joy15|joy16|joy17|joy18|joy19|joy2|joy20|joy21|joy22|joy23|joy24|joy25|joy26|joy27|joy28|joy29|joy3|joy30|joy31|joy32|joy4|joy5|joy6|joy7|joy8|joy9|joyaxes|joybuttons|joyinfo|joyname|joypov|joyr|joyu|joyv|joyx|joyy|joyz|lalt|launch_app1|launch_app2|launch_mail|launch_media|lbutton|lcontrol|lctrl|left|lshift|lwin|lwindown|lwinup|mbutton|media_next|media_play_pause|media_prev|media_stop|numlock|numpad0|numpad1|numpad2|numpad3|numpad4|numpad5|numpad6|numpad7|numpad8|numpad9|numpadadd|numpadclear|numpaddel|numpaddiv|numpaddot|numpaddown|numpadend|numpadenter|numpadhome|numpadins|numpadleft|numpadmult|numpadpgdn|numpadpgup|numpadright|numpadsub|numpadup|pgdn|pgup|printscreen|ralt|rbutton|rcontrol|rctrl|right|rshift|rwin|rwindown|rwinup|scrolllock|shift|shiftdown|shiftup|space|tab|up|volume_down|volume_mute|volume_up|wheeldown|wheelleft|wheelright|wheelup|xbutton1|xbutton2)\b/i,
      important:
        /#\b(?:AllowSameLineComments|ClipboardTimeout|CommentFlag|DerefChar|ErrorStdOut|EscapeChar|HotkeyInterval|HotkeyModifierTimeout|Hotstring|If|IfTimeout|IfWinActive|IfWinExist|IfWinNotActive|IfWinNotExist|Include|IncludeAgain|InputLevel|InstallKeybdHook|InstallMouseHook|KeyHistory|MaxHotkeysPerInterval|MaxMem|MaxThreads|MaxThreadsBuffer|MaxThreadsPerHotkey|MenuMaskKey|NoEnv|NoTrayIcon|Persistent|SingleInstance|UseHook|Warn|WinActivateForce)\b/i,
      keyword:
        /\b(?:Abort|AboveNormal|Add|ahk_class|ahk_exe|ahk_group|ahk_id|ahk_pid|All|Alnum|Alpha|AltSubmit|AltTab|AltTabAndMenu|AltTabMenu|AltTabMenuDismiss|AlwaysOnTop|AutoSize|Background|BackgroundTrans|BelowNormal|between|BitAnd|BitNot|BitOr|BitShiftLeft|BitShiftRight|BitXOr|Bold|Border|Button|ByRef|Catch|Checkbox|Checked|CheckedGray|Choose|ChooseString|Close|Color|ComboBox|Contains|ControlList|Count|Date|DateTime|Days|DDL|Default|DeleteAll|Delimiter|Deref|Destroy|Digit|Disable|Disabled|DropDownList|Edit|Eject|Else|Enable|Enabled|Error|Exist|Expand|ExStyle|FileSystem|Finally|First|Flash|Float|FloatFast|Focus|Font|for|global|Grid|Group|GroupBox|GuiClose|GuiContextMenu|GuiDropFiles|GuiEscape|GuiSize|Hdr|Hidden|Hide|High|HKCC|HKCR|HKCU|HKEY_CLASSES_ROOT|HKEY_CURRENT_CONFIG|HKEY_CURRENT_USER|HKEY_LOCAL_MACHINE|HKEY_USERS|HKLM|HKU|Hours|HScroll|Icon|IconSmall|ID|IDLast|If|IfEqual|IfExist|IfGreater|IfGreaterOrEqual|IfInString|IfLess|IfLessOrEqual|IfMsgBox|IfNotEqual|IfNotExist|IfNotInString|IfWinActive|IfWinExist|IfWinNotActive|IfWinNotExist|Ignore|ImageList|in|Integer|IntegerFast|Interrupt|is|italic|Join|Label|LastFound|LastFoundExist|Limit|Lines|List|ListBox|ListView|local|Lock|Logoff|Low|Lower|Lowercase|MainWindow|Margin|Maximize|MaximizeBox|MaxSize|Minimize|MinimizeBox|MinMax|MinSize|Minutes|MonthCal|Mouse|Move|Multi|NA|No|NoActivate|NoDefault|NoHide|NoIcon|NoMainWindow|norm|Normal|NoSort|NoSortHdr|NoStandard|Not|NoTab|NoTimers|Number|Off|Ok|On|OwnDialogs|Owner|Parse|Password|Picture|Pixel|Pos|Pow|Priority|ProcessName|Radio|Range|Read|ReadOnly|Realtime|Redraw|Region|REG_BINARY|REG_DWORD|REG_EXPAND_SZ|REG_MULTI_SZ|REG_SZ|Relative|Rename|Report|Resize|Restore|Retry|RGB|Screen|Seconds|Section|Serial|SetLabel|ShiftAltTab|Show|Single|Slider|SortDesc|Standard|static|Status|StatusBar|StatusCD|strike|Style|Submit|SysMenu|Tab2|TabStop|Text|Theme|Throw|Tile|ToggleCheck|ToggleEnable|ToolWindow|Top|Topmost|TransColor|Transparent|Tray|TreeView|Try|TryAgain|Type|UnCheck|underline|Unicode|Unlock|Until|UpDown|Upper|Uppercase|UseErrorLevel|Vis|VisFirst|Visible|VScroll|Wait|WaitClose|WantCtrlA|WantF2|WantReturn|While|Wrap|Xdigit|xm|xp|xs|Yes|ym|yp|ys)\b/i,
      function: /[^(); \t,\n+*\-=?>:\\\/<&%\[\]]+(?=\()/,
      punctuation: /[{}[\]():,]/
    };
  }

  // @ts-nocheck
  autoit.displayName = 'autoit';
  autoit.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function autoit(Prism) {
    Prism.languages.autoit = {
      comment: [
        /;.*/,
        {
          // The multi-line comments delimiters can actually be commented out with ";"
          pattern:
            /(^[\t ]*)#(?:comments-start|cs)[\s\S]*?^[ \t]*#(?:ce|comments-end)/m,
          lookbehind: true
        }
      ],
      url: {
        pattern: /(^[\t ]*#include\s+)(?:<[^\r\n>]+>|"[^\r\n"]+")/m,
        lookbehind: true
      },
      string: {
        pattern: /(["'])(?:\1\1|(?!\1)[^\r\n])*\1/,
        greedy: true,
        inside: {
          variable: /([%$@])\w+\1/
        }
      },
      directive: {
        pattern: /(^[\t ]*)#[\w-]+/m,
        lookbehind: true,
        alias: 'keyword'
      },
      function: /\b\w+(?=\()/,
      // Variables and macros
      variable: /[$@]\w+/,
      keyword:
        /\b(?:Case|Const|Continue(?:Case|Loop)|Default|Dim|Do|Else(?:If)?|End(?:Func|If|Select|Switch|With)|Enum|Exit(?:Loop)?|For|Func|Global|If|In|Local|Next|Null|ReDim|Select|Static|Step|Switch|Then|To|Until|Volatile|WEnd|While|With)\b/i,
      number: /\b(?:0x[\da-f]+|\d+(?:\.\d+)?(?:e[+-]?\d+)?)\b/i,
      boolean: /\b(?:False|True)\b/i,
      operator: /<[=>]?|[-+*\/=&>]=?|[?^]|\b(?:And|Not|Or)\b/i,
      punctuation: /[\[\]().,:]/
    };
  }

  // @ts-nocheck
  avisynth.displayName = 'avisynth';
  avisynth.aliases = ['avs'];

  /** @type {import('../core.js').Syntax} */
  function avisynth(Prism) {
  (function (Prism) {
      function replace(pattern, replacements) {
        return pattern.replace(/<<(\d+)>>/g, function (m, index) {
          return replacements[+index]
        })
      }
      function re(pattern, replacements, flags) {
        return RegExp(replace(pattern, replacements), flags || '')
      }
      var types = /bool|clip|float|int|string|val/.source;
      var internals = [
        // bools
        /is(?:bool|clip|float|int|string)|defined|(?:(?:internal)?function|var)?exists?/
          .source, // control
        /apply|assert|default|eval|import|nop|select|undefined/.source, // global
        /opt_(?:allowfloataudio|avipadscanlines|dwchannelmask|enable_(?:b64a|planartopackedrgb|v210|y3_10_10|y3_10_16)|usewaveextensible|vdubplanarhack)|set(?:cachemode|maxcpu|memorymax|planarlegacyalignment|workingdir)/
          .source, // conv
        /hex(?:value)?|value/.source, // numeric
        /abs|ceil|continued(?:denominator|numerator)?|exp|floor|fmod|frac|log(?:10)?|max|min|muldiv|pi|pow|rand|round|sign|spline|sqrt/
          .source, // trig
        /a?sinh?|a?cosh?|a?tan[2h]?/.source, // bit
        /(?:bit(?:and|not|x?or|[lr]?shift[aslu]?|sh[lr]|sa[lr]|[lr]rotatel?|ro[rl]|te?st|set(?:count)?|cl(?:ea)?r|ch(?:an)?ge?))/
          .source, // runtime
        /average(?:[bgr]|chroma[uv]|luma)|(?:[rgb]|chroma[uv]|luma|rgb|[yuv](?=difference(?:fromprevious|tonext)))difference(?:fromprevious|tonext)?|[yuvrgb]plane(?:median|min|max|minmaxdifference)/
          .source, // script
        /getprocessinfo|logmsg|script(?:dir(?:utf8)?|file(?:utf8)?|name(?:utf8)?)|setlogparams/
          .source, // string
        /chr|(?:fill|find|left|mid|replace|rev|right)str|format|[lu]case|ord|str(?:cmpi?|fromutf8|len|toutf8)|time|trim(?:all|left|right)/
          .source, // version
        /isversionorgreater|version(?:number|string)/.source, // helper
        /buildpixeltype|colorspacenametopixeltype/.source, // avsplus
        /addautoloaddir|on(?:cpu|cuda)|prefetch|setfiltermtmode/.source
      ].join('|');
      var properties = [
        // content
        /has(?:audio|video)/.source, // resolution
        /height|width/.source, // framerate
        /frame(?:count|rate)|framerate(?:denominator|numerator)/.source, // interlacing
        /getparity|is(?:field|frame)based/.source, // color format
        /bitspercomponent|componentsize|hasalpha|is(?:planar(?:rgba?)?|interleaved|rgb(?:24|32|48|64)?|y(?:8|u(?:va?|y2))?|yv(?:12|16|24|411)|420|422|444|packedrgb)|numcomponents|pixeltype/
          .source, // audio
        /audio(?:bits|channels|duration|length(?:[fs]|hi|lo)?|rate)|isaudio(?:float|int)/
          .source
      ].join('|');
      var filters = [
        // source
        /avi(?:file)?source|directshowsource|image(?:reader|source|sourceanim)|opendmlsource|segmented(?:avisource|directshowsource)|wavsource/
          .source, // color
        /coloryuv|convertbacktoyuy2|convertto(?:RGB(?:24|32|48|64)|(?:planar)?RGBA?|Y8?|YV(?:12|16|24|411)|YUVA?(?:411|420|422|444)|YUY2)|fixluminance|gr[ae]yscale|invert|levels|limiter|mergea?rgb|merge(?:chroma|luma)|rgbadjust|show(?:alpha|blue|green|red)|swapuv|tweak|[uv]toy8?|ytouv/
          .source, // overlay
        /(?:colorkey|reset)mask|layer|mask(?:hs)?|merge|overlay|subtract/.source, // geometry
        /addborders|(?:bicubic|bilinear|blackman|gauss|lanczos4|lanczos|point|sinc|spline(?:16|36|64))resize|crop(?:bottom)?|flip(?:horizontal|vertical)|(?:horizontal|vertical)?reduceby2|letterbox|skewrows|turn(?:180|left|right)/
          .source, // pixel
        /blur|fixbrokenchromaupsampling|generalconvolution|(?:spatial|temporal)soften|sharpen/
          .source, // timeline
        /trim|(?:un)?alignedsplice|(?:assume|assumescaled|change|convert)FPS|(?:delete|duplicate)frame|dissolve|fade(?:in|io|out)[02]?|freezeframe|interleave|loop|reverse|select(?:even|odd|(?:range)?every)/
          .source, // interlace
        /assume[bt]ff|assume(?:field|frame)based|bob|complementparity|doubleweave|peculiarblend|pulldown|separate(?:columns|fields|rows)|swapfields|weave(?:columns|rows)?/
          .source, // audio
        /amplify(?:db)?|assumesamplerate|audiodub(?:ex)?|audiotrim|convertaudioto(?:(?:8|16|24|32)bit|float)|converttomono|delayaudio|ensurevbrmp3sync|get(?:left|right)?channel|kill(?:audio|video)|mergechannels|mixaudio|monotostereo|normalize|resampleaudio|ssrc|supereq|timestretch/
          .source, // conditional
        /animate|applyrange|conditional(?:filter|reader|select)|frameevaluate|scriptclip|tcp(?:server|source)|writefile(?:end|if|start)?/
          .source, // export
        /imagewriter/.source, // debug
        /blackness|blankclip|colorbars(?:hd)?|compare|dumpfiltergraph|echo|histogram|info|messageclip|preroll|setgraphanalysis|show(?:framenumber|smpte|time)|showfiveversions|stack(?:horizontal|vertical)|subtitle|tone|version/
          .source
      ].join('|');
      var allinternals = [internals, properties, filters].join('|');
      Prism.languages.avisynth = {
        comment: [
          {
            // Matches [* *] nestable block comments, but only supports 1 level of nested comments
            // /\[\*(?:[^\[*]|\[(?!\*)|\*(?!\])|<self>)*\*\]/
            pattern:
              /(^|[^\\])\[\*(?:[^\[*]|\[(?!\*)|\*(?!\])|\[\*(?:[^\[*]|\[(?!\*)|\*(?!\]))*\*\])*\*\]/,
            lookbehind: true,
            greedy: true
          },
          {
            // Matches /* */ block comments
            pattern: /(^|[^\\])\/\*[\s\S]*?(?:\*\/|$)/,
            lookbehind: true,
            greedy: true
          },
          {
            // Matches # comments
            pattern: /(^|[^\\$])#.*/,
            lookbehind: true,
            greedy: true
          }
        ],
        // Handle before strings because optional arguments are surrounded by double quotes
        argument: {
          pattern: re(/\b(?:<<0>>)\s+("?)\w+\1/.source, [types], 'i'),
          inside: {
            keyword: /^\w+/
          }
        },
        // Optional argument assignment
        'argument-label': {
          pattern: /([,(][\s\\]*)\w+\s*=(?!=)/,
          lookbehind: true,
          inside: {
            'argument-name': {
              pattern: /^\w+/,
              alias: 'punctuation'
            },
            punctuation: /=$/
          }
        },
        string: [
          {
            // triple double-quoted
            pattern: /"""[\s\S]*?"""/,
            greedy: true
          },
          {
            // single double-quoted
            pattern: /"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"/,
            greedy: true,
            inside: {
              constant: {
                // These *are* case-sensitive!
                pattern:
                  /\b(?:DEFAULT_MT_MODE|(?:MAINSCRIPT|PROGRAM|SCRIPT)DIR|(?:MACHINE|USER)_(?:CLASSIC|PLUS)_PLUGINS)\b/
              }
            }
          }
        ],
        // The special "last" variable that takes the value of the last implicitly returned clip
        variable: /\b(?:last)\b/i,
        boolean: /\b(?:false|no|true|yes)\b/i,
        keyword:
          /\b(?:catch|else|for|function|global|if|return|try|while|__END__)\b/i,
        constant: /\bMT_(?:MULTI_INSTANCE|NICE_FILTER|SERIALIZED|SPECIAL_MT)\b/,
        // AviSynth's internal functions, filters, and properties
        'builtin-function': {
          pattern: re(/\b(?:<<0>>)\b/.source, [allinternals], 'i'),
          alias: 'function'
        },
        'type-cast': {
          pattern: re(/\b(?:<<0>>)(?=\s*\()/.source, [types], 'i'),
          alias: 'keyword'
        },
        // External/user-defined filters
        function: {
          pattern: /\b[a-z_]\w*(?=\s*\()|(\.)[a-z_]\w*\b/i,
          lookbehind: true
        },
        // Matches a \ as the first or last character on a line
        'line-continuation': {
          pattern: /(^[ \t]*)\\|\\(?=[ \t]*$)/m,
          lookbehind: true,
          alias: 'punctuation'
        },
        number:
          /\B\$(?:[\da-f]{6}|[\da-f]{8})\b|(?:(?:\b|\B-)\d+(?:\.\d*)?\b|\B\.\d+\b)/i,
        operator: /\+\+?|[!=<>]=?|&&|\|\||[?:*/%-]/,
        punctuation: /[{}\[\]();,.]/
      };
      Prism.languages.avs = Prism.languages.avisynth;
    })(Prism);
  }

  // @ts-nocheck
  avroIdl.displayName = 'avro-idl';
  avroIdl.aliases = ['avdl'];

  /** @type {import('../core.js').Syntax} */
  function avroIdl(Prism) {
    // GitHub: https://github.com/apache/avro
    // Docs: https://avro.apache.org/docs/current/idl.html
    Prism.languages['avro-idl'] = {
      comment: {
        pattern: /\/\/.*|\/\*[\s\S]*?\*\//,
        greedy: true
      },
      string: {
        pattern: /(^|[^\\])"(?:[^\r\n"\\]|\\.)*"/,
        lookbehind: true,
        greedy: true
      },
      annotation: {
        pattern: /@(?:[$\w.-]|`[^\r\n`]+`)+/,
        greedy: true,
        alias: 'function'
      },
      'function-identifier': {
        pattern: /`[^\r\n`]+`(?=\s*\()/,
        greedy: true,
        alias: 'function'
      },
      identifier: {
        pattern: /`[^\r\n`]+`/,
        greedy: true
      },
      'class-name': {
        pattern: /(\b(?:enum|error|protocol|record|throws)\b\s+)[$\w]+/,
        lookbehind: true,
        greedy: true
      },
      keyword:
        /\b(?:array|boolean|bytes|date|decimal|double|enum|error|false|fixed|float|idl|import|int|local_timestamp_ms|long|map|null|oneway|protocol|record|schema|string|throws|time_ms|timestamp_ms|true|union|uuid|void)\b/,
      function: /\b[a-z_]\w*(?=\s*\()/i,
      number: [
        {
          pattern:
            /(^|[^\w.])-?(?:(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?|0x(?:[a-f0-9]+(?:\.[a-f0-9]*)?|\.[a-f0-9]+)(?:p[+-]?\d+)?)[dfl]?(?![\w.])/i,
          lookbehind: true
        },
        /-?\b(?:Infinity|NaN)\b/
      ],
      operator: /=/,
      punctuation: /[()\[\]{}<>.:,;-]/
    };
    Prism.languages.avdl = Prism.languages['avro-idl'];
  }

  // @ts-nocheck
  batch.displayName = 'batch';
  batch.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function batch(Prism) {
  (function (Prism) {
      var variable = /%%?[~:\w]+%?|!\S+!/;
      var parameter = {
        pattern: /\/[a-z?]+(?=[ :]|$):?|-[a-z]\b|--[a-z-]+\b/im,
        alias: 'attr-name',
        inside: {
          punctuation: /:/
        }
      };
      var string = /"(?:[\\"]"|[^"])*"(?!")/;
      var number = /(?:\b|-)\d+\b/;
      Prism.languages.batch = {
        comment: [
          /^::.*/m,
          {
            pattern: /((?:^|[&(])[ \t]*)rem\b(?:[^^&)\r\n]|\^(?:\r\n|[\s\S]))*/im,
            lookbehind: true
          }
        ],
        label: {
          pattern: /^:.*/m,
          alias: 'property'
        },
        command: [
          {
            // FOR command
            pattern:
              /((?:^|[&(])[ \t]*)for(?: \/[a-z?](?:[ :](?:"[^"]*"|[^\s"/]\S*))?)* \S+ in \([^)]+\) do/im,
            lookbehind: true,
            inside: {
              keyword: /\b(?:do|in)\b|^for\b/i,
              string: string,
              parameter: parameter,
              variable: variable,
              number: number,
              punctuation: /[()',]/
            }
          },
          {
            // IF command
            pattern:
              /((?:^|[&(])[ \t]*)if(?: \/[a-z?](?:[ :](?:"[^"]*"|[^\s"/]\S*))?)* (?:not )?(?:cmdextversion \d+|defined \w+|errorlevel \d+|exist \S+|(?:"[^"]*"|(?!")(?:(?!==)\S)+)?(?:==| (?:equ|geq|gtr|leq|lss|neq) )(?:"[^"]*"|[^\s"]\S*))/im,
            lookbehind: true,
            inside: {
              keyword:
                /\b(?:cmdextversion|defined|errorlevel|exist|not)\b|^if\b/i,
              string: string,
              parameter: parameter,
              variable: variable,
              number: number,
              operator: /\^|==|\b(?:equ|geq|gtr|leq|lss|neq)\b/i
            }
          },
          {
            // ELSE command
            pattern: /((?:^|[&()])[ \t]*)else\b/im,
            lookbehind: true,
            inside: {
              keyword: /^else\b/i
            }
          },
          {
            // SET command
            pattern:
              /((?:^|[&(])[ \t]*)set(?: \/[a-z](?:[ :](?:"[^"]*"|[^\s"/]\S*))?)* (?:[^^&)\r\n]|\^(?:\r\n|[\s\S]))*/im,
            lookbehind: true,
            inside: {
              keyword: /^set\b/i,
              string: string,
              parameter: parameter,
              variable: [variable, /\w+(?=(?:[*\/%+\-&^|]|<<|>>)?=)/],
              number: number,
              operator: /[*\/%+\-&^|]=?|<<=?|>>=?|[!~_=]/,
              punctuation: /[()',]/
            }
          },
          {
            // Other commands
            pattern:
              /((?:^|[&(])[ \t]*@?)\w+\b(?:"(?:[\\"]"|[^"])*"(?!")|[^"^&)\r\n]|\^(?:\r\n|[\s\S]))*/m,
            lookbehind: true,
            inside: {
              keyword: /^\w+\b/,
              string: string,
              parameter: parameter,
              label: {
                pattern: /(^\s*):\S+/m,
                lookbehind: true,
                alias: 'property'
              },
              variable: variable,
              number: number,
              operator: /\^/
            }
          }
        ],
        operator: /[&@]/,
        punctuation: /[()']/
      };
    })(Prism);
  }

  // @ts-nocheck
  bbcode.displayName = 'bbcode';
  bbcode.aliases = ['shortcode'];

  /** @type {import('../core.js').Syntax} */
  function bbcode(Prism) {
    Prism.languages.bbcode = {
      tag: {
        pattern:
          /\[\/?[^\s=\]]+(?:\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'"\]=]+))?(?:\s+[^\s=\]]+\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'"\]=]+))*\s*\]/,
        inside: {
          tag: {
            pattern: /^\[\/?[^\s=\]]+/,
            inside: {
              punctuation: /^\[\/?/
            }
          },
          'attr-value': {
            pattern: /=\s*(?:"[^"]*"|'[^']*'|[^\s'"\]=]+)/,
            inside: {
              punctuation: [
                /^=/,
                {
                  pattern: /^(\s*)["']|["']$/,
                  lookbehind: true
                }
              ]
            }
          },
          punctuation: /\]/,
          'attr-name': /[^\s=\]]+/
        }
      }
    };
    Prism.languages.shortcode = Prism.languages.bbcode;
  }

  // @ts-nocheck
  bicep.displayName = 'bicep';
  bicep.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function bicep(Prism) {
    // based loosely upon: https://github.com/Azure/bicep/blob/main/src/textmate/bicep.tmlanguage
    Prism.languages.bicep = {
      comment: [
        {
          // multiline comments eg /* ASDF */
          pattern: /(^|[^\\])\/\*[\s\S]*?(?:\*\/|$)/,
          lookbehind: true,
          greedy: true
        },
        {
          // singleline comments eg // ASDF
          pattern: /(^|[^\\:])\/\/.*/,
          lookbehind: true,
          greedy: true
        }
      ],
      property: [
        {
          pattern: /([\r\n][ \t]*)[a-z_]\w*(?=[ \t]*:)/i,
          lookbehind: true
        },
        {
          pattern: /([\r\n][ \t]*)'(?:\\.|\$(?!\{)|[^'\\\r\n$])*'(?=[ \t]*:)/,
          lookbehind: true,
          greedy: true
        }
      ],
      string: [
        {
          pattern: /'''[^'][\s\S]*?'''/,
          greedy: true
        },
        {
          pattern: /(^|[^\\'])'(?:\\.|\$(?!\{)|[^'\\\r\n$])*'/,
          lookbehind: true,
          greedy: true
        }
      ],
      'interpolated-string': {
        pattern: /(^|[^\\'])'(?:\\.|\$(?:(?!\{)|\{[^{}\r\n]*\})|[^'\\\r\n$])*'/,
        lookbehind: true,
        greedy: true,
        inside: {
          interpolation: {
            pattern: /\$\{[^{}\r\n]*\}/,
            inside: {
              expression: {
                pattern: /(^\$\{)[\s\S]+(?=\}$)/,
                lookbehind: true
              },
              punctuation: /^\$\{|\}$/
            }
          },
          string: /[\s\S]+/
        }
      },
      datatype: {
        pattern: /(\b(?:output|param)\b[ \t]+\w+[ \t]+)\w+\b/,
        lookbehind: true,
        alias: 'class-name'
      },
      boolean: /\b(?:false|true)\b/,
      // https://github.com/Azure/bicep/blob/114a3251b4e6e30082a58729f19a8cc4e374ffa6/src/textmate/bicep.tmlanguage#L184
      keyword:
        /\b(?:existing|for|if|in|module|null|output|param|resource|targetScope|var)\b/,
      decorator: /@\w+\b/,
      function: /\b[a-z_]\w*(?=[ \t]*\()/i,
      number: /(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:E[+-]?\d+)?/i,
      operator:
        /--|\+\+|\*\*=?|=>|&&=?|\|\|=?|[!=]==|<<=?|>>>?=?|[-+*/%&|^!=<>]=?|\.{3}|\?\?=?|\?\.?|[~:]/,
      punctuation: /[{}[\];(),.:]/
    };
    Prism.languages.bicep['interpolated-string'].inside['interpolation'].inside[
      'expression'
    ].inside = Prism.languages.bicep;
  }

  // @ts-nocheck
  birb.displayName = 'birb';
  birb.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function birb(Prism) {
    Prism.register(clike);
    Prism.languages.birb = Prism.languages.extend('clike', {
      string: {
        pattern: /r?("|')(?:\\.|(?!\1)[^\\])*\1/,
        greedy: true
      },
      'class-name': [
        /\b[A-Z](?:[\d_]*[a-zA-Z]\w*)?\b/, // matches variable and function return types (parameters as well).
        /\b(?:[A-Z]\w*|(?!(?:var|void)\b)[a-z]\w*)(?=\s+\w+\s*[;,=()])/
      ],
      keyword:
        /\b(?:assert|break|case|class|const|default|else|enum|final|follows|for|grab|if|nest|new|next|noSeeb|return|static|switch|throw|var|void|while)\b/,
      operator: /\+\+|--|&&|\|\||<<=?|>>=?|~(?:\/=?)?|[+\-*\/%&^|=!<>]=?|\?|:/,
      variable: /\b[a-z_]\w*\b/
    });
    Prism.languages.insertBefore('birb', 'function', {
      metadata: {
        pattern: /<\w+>/,
        greedy: true,
        alias: 'symbol'
      }
    });
  }

  // @ts-nocheck
  bison.displayName = 'bison';
  bison.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function bison(Prism) {
    Prism.register(c$1);
    Prism.languages.bison = Prism.languages.extend('c', {});
    Prism.languages.insertBefore('bison', 'comment', {
      bison: {
        // This should match all the beginning of the file
        // including the prologue(s), the bison declarations and
        // the grammar rules.
        pattern: /^(?:[^%]|%(?!%))*%%[\s\S]*?%%/,
        inside: {
          c: {
            // Allow for one level of nested braces
            pattern: /%\{[\s\S]*?%\}|\{(?:\{[^}]*\}|[^{}])*\}/,
            inside: {
              delimiter: {
                pattern: /^%?\{|%?\}$/,
                alias: 'punctuation'
              },
              'bison-variable': {
                pattern: /[$@](?:<[^\s>]+>)?[\w$]+/,
                alias: 'variable',
                inside: {
                  punctuation: /<|>/
                }
              },
              rest: Prism.languages.c
            }
          },
          comment: Prism.languages.c.comment,
          string: Prism.languages.c.string,
          property: /\S+(?=:)/,
          keyword: /%\w+/,
          number: {
            pattern: /(^|[^@])\b(?:0x[\da-f]+|\d+)/i,
            lookbehind: true
          },
          punctuation: /%[%?]|[|:;\[\]<>]/
        }
      }
    });
  }

  // @ts-nocheck
  bnf.displayName = 'bnf';
  bnf.aliases = ['rbnf'];

  /** @type {import('../core.js').Syntax} */
  function bnf(Prism) {
    Prism.languages.bnf = {
      string: {
        pattern: /"[^\r\n"]*"|'[^\r\n']*'/
      },
      definition: {
        pattern: /<[^<>\r\n\t]+>(?=\s*::=)/,
        alias: ['rule', 'keyword'],
        inside: {
          punctuation: /^<|>$/
        }
      },
      rule: {
        pattern: /<[^<>\r\n\t]+>/,
        inside: {
          punctuation: /^<|>$/
        }
      },
      operator: /::=|[|()[\]{}*+?]|\.{3}/
    };
    Prism.languages.rbnf = Prism.languages.bnf;
  }

  // @ts-nocheck
  brainfuck.displayName = 'brainfuck';
  brainfuck.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function brainfuck(Prism) {
    Prism.languages.brainfuck = {
      pointer: {
        pattern: /<|>/,
        alias: 'keyword'
      },
      increment: {
        pattern: /\+/,
        alias: 'inserted'
      },
      decrement: {
        pattern: /-/,
        alias: 'deleted'
      },
      branching: {
        pattern: /\[|\]/,
        alias: 'important'
      },
      operator: /[.,]/,
      comment: /\S+/
    };
  }

  // @ts-nocheck
  brightscript.displayName = 'brightscript';
  brightscript.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function brightscript(Prism) {
    Prism.languages.brightscript = {
      comment: /(?:\brem|').*/i,
      'directive-statement': {
        pattern: /(^[\t ]*)#(?:const|else(?:[\t ]+if)?|end[\t ]+if|error|if).*/im,
        lookbehind: true,
        alias: 'property',
        inside: {
          'error-message': {
            pattern: /(^#error).+/,
            lookbehind: true
          },
          directive: {
            pattern: /^#(?:const|else(?:[\t ]+if)?|end[\t ]+if|error|if)/,
            alias: 'keyword'
          },
          expression: {
            pattern: /[\s\S]+/,
            inside: null // see below
          }
        }
      },
      property: {
        pattern:
          /([\r\n{,][\t ]*)(?:(?!\d)\w+|"(?:[^"\r\n]|"")*"(?!"))(?=[ \t]*:)/,
        lookbehind: true,
        greedy: true
      },
      string: {
        pattern: /"(?:[^"\r\n]|"")*"(?!")/,
        greedy: true
      },
      'class-name': {
        pattern: /(\bAs[\t ]+)\w+/i,
        lookbehind: true
      },
      keyword:
        /\b(?:As|Dim|Each|Else|Elseif|End|Exit|For|Function|Goto|If|In|Print|Return|Step|Stop|Sub|Then|To|While)\b/i,
      boolean: /\b(?:false|true)\b/i,
      function: /\b(?!\d)\w+(?=[\t ]*\()/,
      number: /(?:\b\d+(?:\.\d+)?(?:[ed][+-]\d+)?|&h[a-f\d]+)\b[%&!#]?/i,
      operator:
        /--|\+\+|>>=?|<<=?|<>|[-+*/\\<>]=?|[:^=?]|\b(?:and|mod|not|or)\b/i,
      punctuation: /[.,;()[\]{}]/,
      constant: /\b(?:LINE_NUM)\b/i
    };
    Prism.languages.brightscript['directive-statement'].inside.expression.inside =
      Prism.languages.brightscript;
  }

  // @ts-nocheck
  bro.displayName = 'bro';
  bro.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function bro(Prism) {
    Prism.languages.bro = {
      comment: {
        pattern: /(^|[^\\$])#.*/,
        lookbehind: true,
        inside: {
          italic: /\b(?:FIXME|TODO|XXX)\b/
        }
      },
      string: {
        pattern: /(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,
        greedy: true
      },
      boolean: /\b[TF]\b/,
      function: {
        pattern: /(\b(?:event|function|hook)[ \t]+)\w+(?:::\w+)?/,
        lookbehind: true
      },
      builtin:
        /(?:@(?:load(?:-(?:plugin|sigs))?|unload|prefixes|ifn?def|else|(?:end)?if|DIR|FILENAME))|(?:&?(?:add_func|create_expire|default|delete_func|encrypt|error_handler|expire_func|group|log|mergeable|optional|persistent|priority|raw_output|read_expire|redef|rotate_interval|rotate_size|synchronized|type_column|write_expire))/,
      constant: {
        pattern: /(\bconst[ \t]+)\w+/i,
        lookbehind: true
      },
      keyword:
        /\b(?:add|addr|alarm|any|bool|break|const|continue|count|delete|double|else|enum|event|export|file|for|function|global|hook|if|in|int|interval|local|module|next|of|opaque|pattern|port|print|record|return|schedule|set|string|subnet|table|time|timeout|using|vector|when)\b/,
      operator: /--?|\+\+?|!=?=?|<=?|>=?|==?=?|&&|\|\|?|\?|\*|\/|~|\^|%/,
      number: /\b0x[\da-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?/i,
      punctuation: /[{}[\];(),.:]/
    };
  }

  // @ts-nocheck
  bsl.displayName = 'bsl';
  bsl.aliases = ['oscript'];

  /** @type {import('../core.js').Syntax} */
  function bsl(Prism) {
    /* eslint-disable no-misleading-character-class */
    // 1C:Enterprise
    // https://github.com/Diversus23/
    //
    Prism.languages.bsl = {
      comment: /\/\/.*/,
      string: [
        // Строки
        // Strings
        {
          pattern: /"(?:[^"]|"")*"(?!")/,
          greedy: true
        }, // Дата и время
        // Date & time
        {
          pattern: /'(?:[^'\r\n\\]|\\.)*'/
        }
      ],
      keyword: [
        {
          // RU
          pattern:
            /(^|[^\w\u0400-\u0484\u0487-\u052f\u1d2b\u1d78\u2de0-\u2dff\ua640-\ua69f\ufe2e\ufe2f])(?:пока|для|новый|прервать|попытка|исключение|вызватьисключение|иначе|конецпопытки|неопределено|функция|перем|возврат|конецфункции|если|иначеесли|процедура|конецпроцедуры|тогда|знач|экспорт|конецесли|из|каждого|истина|ложь|по|цикл|конеццикла|выполнить)(?![\w\u0400-\u0484\u0487-\u052f\u1d2b\u1d78\u2de0-\u2dff\ua640-\ua69f\ufe2e\ufe2f])/i,
          lookbehind: true
        },
        {
          // EN
          pattern:
            /\b(?:break|do|each|else|elseif|enddo|endfunction|endif|endprocedure|endtry|except|execute|export|false|for|function|if|in|new|null|procedure|raise|return|then|to|true|try|undefined|val|var|while)\b/i
        }
      ],
      number: {
        pattern:
          /(^(?=\d)|[^\w\u0400-\u0484\u0487-\u052f\u1d2b\u1d78\u2de0-\u2dff\ua640-\ua69f\ufe2e\ufe2f])(?:\d+(?:\.\d*)?|\.\d+)(?:E[+-]?\d+)?/i,
        lookbehind: true
      },
      operator: [
        /[<>+\-*/]=?|[%=]/, // RU
        {
          pattern:
            /(^|[^\w\u0400-\u0484\u0487-\u052f\u1d2b\u1d78\u2de0-\u2dff\ua640-\ua69f\ufe2e\ufe2f])(?:и|или|не)(?![\w\u0400-\u0484\u0487-\u052f\u1d2b\u1d78\u2de0-\u2dff\ua640-\ua69f\ufe2e\ufe2f])/i,
          lookbehind: true
        }, // EN
        {
          pattern: /\b(?:and|not|or)\b/i
        }
      ],
      punctuation: /\(\.|\.\)|[()\[\]:;,.]/,
      directive: [
        // Теги препроцессора вида &Клиент, &Сервер, ...
        // Preprocessor tags of the type &Client, &Server, ...
        {
          pattern: /^([ \t]*)&.*/m,
          lookbehind: true,
          greedy: true,
          alias: 'important'
        }, // Инструкции препроцессора вида:
        // #Если Сервер Тогда
        // ...
        // #КонецЕсли
        // Preprocessor instructions of the form:
        // #If Server Then
        // ...
        // #EndIf
        {
          pattern: /^([ \t]*)#.*/gm,
          lookbehind: true,
          greedy: true,
          alias: 'important'
        }
      ]
    };
    Prism.languages.oscript = Prism.languages['bsl'];
  }

  // @ts-nocheck
  cfscript.displayName = 'cfscript';
  cfscript.aliases = ['cfc'];

  /** @type {import('../core.js').Syntax} */
  function cfscript(Prism) {
    Prism.register(clike);
    // https://cfdocs.org/script
    Prism.languages.cfscript = Prism.languages.extend('clike', {
      comment: [
        {
          pattern: /(^|[^\\])\/\*[\s\S]*?(?:\*\/|$)/,
          lookbehind: true,
          inside: {
            annotation: {
              pattern: /(?:^|[^.])@[\w\.]+/,
              alias: 'punctuation'
            }
          }
        },
        {
          pattern: /(^|[^\\:])\/\/.*/,
          lookbehind: true,
          greedy: true
        }
      ],
      keyword:
        /\b(?:abstract|break|catch|component|continue|default|do|else|extends|final|finally|for|function|if|in|include|package|private|property|public|remote|required|rethrow|return|static|switch|throw|try|var|while|xml)\b(?!\s*=)/,
      operator: [
        /\+\+|--|&&|\|\||::|=>|[!=]==|<=?|>=?|[-+*/%&|^!=<>]=?|\?(?:\.|:)?|[?:]/,
        /\b(?:and|contains|eq|equal|eqv|gt|gte|imp|is|lt|lte|mod|not|or|xor)\b/
      ],
      scope: {
        pattern:
          /\b(?:application|arguments|cgi|client|cookie|local|session|super|this|variables)\b/,
        alias: 'global'
      },
      type: {
        pattern:
          /\b(?:any|array|binary|boolean|date|guid|numeric|query|string|struct|uuid|void|xml)\b/,
        alias: 'builtin'
      }
    });
    Prism.languages.insertBefore('cfscript', 'keyword', {
      // This must be declared before keyword because we use "function" inside the lookahead
      'function-variable': {
        pattern:
          /[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*[=:]\s*(?:\bfunction\b|(?:\((?:[^()]|\([^()]*\))*\)|(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*)\s*=>))/,
        alias: 'function'
      }
    });
    delete Prism.languages.cfscript['class-name'];
    Prism.languages.cfc = Prism.languages['cfscript'];
  }

  // @ts-nocheck
  chaiscript.displayName = 'chaiscript';
  chaiscript.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function chaiscript(Prism) {
    Prism.register(clike);
    Prism.register(cpp);
    Prism.languages.chaiscript = Prism.languages.extend('clike', {
      string: {
        pattern: /(^|[^\\])'(?:[^'\\]|\\[\s\S])*'/,
        lookbehind: true,
        greedy: true
      },
      'class-name': [
        {
          // e.g. class Rectangle { ... }
          pattern: /(\bclass\s+)\w+/,
          lookbehind: true
        },
        {
          // e.g. attr Rectangle::height, def Rectangle::area() { ... }
          pattern: /(\b(?:attr|def)\s+)\w+(?=\s*::)/,
          lookbehind: true
        }
      ],
      keyword:
        /\b(?:attr|auto|break|case|catch|class|continue|def|default|else|finally|for|fun|global|if|return|switch|this|try|var|while)\b/,
      number: [Prism.languages.cpp.number, /\b(?:Infinity|NaN)\b/],
      operator:
        />>=?|<<=?|\|\||&&|:[:=]?|--|\+\+|[=!<>+\-*/%|&^]=?|[?~]|`[^`\r\n]{1,4}`/
    });
    Prism.languages.insertBefore('chaiscript', 'operator', {
      'parameter-type': {
        // e.g. def foo(int x, Vector y) {...}
        pattern: /([,(]\s*)\w+(?=\s+\w)/,
        lookbehind: true,
        alias: 'class-name'
      }
    });
    Prism.languages.insertBefore('chaiscript', 'string', {
      'string-interpolation': {
        pattern:
          /(^|[^\\])"(?:[^"$\\]|\\[\s\S]|\$(?!\{)|\$\{(?:[^{}]|\{(?:[^{}]|\{[^{}]*\})*\})*\})*"/,
        lookbehind: true,
        greedy: true,
        inside: {
          interpolation: {
            pattern:
              /((?:^|[^\\])(?:\\{2})*)\$\{(?:[^{}]|\{(?:[^{}]|\{[^{}]*\})*\})*\}/,
            lookbehind: true,
            inside: {
              'interpolation-expression': {
                pattern: /(^\$\{)[\s\S]+(?=\}$)/,
                lookbehind: true,
                inside: Prism.languages.chaiscript
              },
              'interpolation-punctuation': {
                pattern: /^\$\{|\}$/,
                alias: 'punctuation'
              }
            }
          },
          string: /[\s\S]+/
        }
      }
    });
  }

  // @ts-nocheck
  cil.displayName = 'cil';
  cil.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function cil(Prism) {
    Prism.languages.cil = {
      comment: /\/\/.*/,
      string: {
        pattern: /(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,
        greedy: true
      },
      directive: {
        pattern: /(^|\W)\.[a-z]+(?=\s)/,
        lookbehind: true,
        alias: 'class-name'
      },
      // Actually an assembly reference
      variable: /\[[\w\.]+\]/,
      keyword:
        /\b(?:abstract|ansi|assembly|auto|autochar|beforefieldinit|bool|bstr|byvalstr|catch|char|cil|class|currency|date|decimal|default|enum|error|explicit|extends|extern|famandassem|family|famorassem|final(?:ly)?|float32|float64|hidebysig|u?int(?:8|16|32|64)?|iant|idispatch|implements|import|initonly|instance|interface|iunknown|literal|lpstr|lpstruct|lptstr|lpwstr|managed|method|native(?:Type)?|nested|newslot|object(?:ref)?|pinvokeimpl|private|privatescope|public|reqsecobj|rtspecialname|runtime|sealed|sequential|serializable|specialname|static|string|struct|syschar|tbstr|unicode|unmanagedexp|unsigned|value(?:type)?|variant|virtual|void)\b/,
      function:
        /\b(?:(?:constrained|no|readonly|tail|unaligned|volatile)\.)?(?:conv\.(?:[iu][1248]?|ovf\.[iu][1248]?(?:\.un)?|r\.un|r4|r8)|ldc\.(?:i4(?:\.\d+|\.[mM]1|\.s)?|i8|r4|r8)|ldelem(?:\.[iu][1248]?|\.r[48]|\.ref|a)?|ldind\.(?:[iu][1248]?|r[48]|ref)|stelem\.?(?:i[1248]?|r[48]|ref)?|stind\.(?:i[1248]?|r[48]|ref)?|end(?:fault|filter|finally)|ldarg(?:\.[0-3s]|a(?:\.s)?)?|ldloc(?:\.\d+|\.s)?|sub(?:\.ovf(?:\.un)?)?|mul(?:\.ovf(?:\.un)?)?|add(?:\.ovf(?:\.un)?)?|stloc(?:\.[0-3s])?|refany(?:type|val)|blt(?:\.un)?(?:\.s)?|ble(?:\.un)?(?:\.s)?|bgt(?:\.un)?(?:\.s)?|bge(?:\.un)?(?:\.s)?|unbox(?:\.any)?|init(?:blk|obj)|call(?:i|virt)?|brfalse(?:\.s)?|bne\.un(?:\.s)?|ldloca(?:\.s)?|brzero(?:\.s)?|brtrue(?:\.s)?|brnull(?:\.s)?|brinst(?:\.s)?|starg(?:\.s)?|leave(?:\.s)?|shr(?:\.un)?|rem(?:\.un)?|div(?:\.un)?|clt(?:\.un)?|alignment|castclass|ldvirtftn|beq(?:\.s)?|ckfinite|ldsflda|ldtoken|localloc|mkrefany|rethrow|cgt\.un|arglist|switch|stsfld|sizeof|newobj|newarr|ldsfld|ldnull|ldflda|isinst|throw|stobj|stfld|ldstr|ldobj|ldlen|ldftn|ldfld|cpobj|cpblk|break|br\.s|xor|shl|ret|pop|not|nop|neg|jmp|dup|cgt|ceq|box|and|or|br)\b/,
      boolean: /\b(?:false|true)\b/,
      number: /\b-?(?:0x[0-9a-f]+|\d+)(?:\.[0-9a-f]+)?\b/i,
      punctuation: /[{}[\];(),:=]|IL_[0-9A-Za-z]+/
    };
  }

  // @ts-nocheck
  clojure.displayName = 'clojure';
  clojure.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function clojure(Prism) {
    // Copied from https://github.com/jeluard/prism-clojure
    Prism.languages.clojure = {
      comment: {
        pattern: /;.*/,
        greedy: true
      },
      string: {
        pattern: /"(?:[^"\\]|\\.)*"/,
        greedy: true
      },
      char: /\\\w+/,
      symbol: {
        pattern: /(^|[\s()\[\]{},])::?[\w*+!?'<>=/.-]+/,
        lookbehind: true
      },
      keyword: {
        pattern:
          /(\()(?:-|->|->>|\.|\.\.|\*|\/|\+|<|<=|=|==|>|>=|accessor|agent|agent-errors|aget|alength|all-ns|alter|and|append-child|apply|array-map|aset|aset-boolean|aset-byte|aset-char|aset-double|aset-float|aset-int|aset-long|aset-short|assert|assoc|await|await-for|bean|binding|bit-and|bit-not|bit-or|bit-shift-left|bit-shift-right|bit-xor|boolean|branch\?|butlast|byte|cast|char|children|class|clear-agent-errors|comment|commute|comp|comparator|complement|concat|cond|conj|cons|constantly|construct-proxy|contains\?|count|create-ns|create-struct|cycle|dec|declare|def|def-|definline|definterface|defmacro|defmethod|defmulti|defn|defn-|defonce|defproject|defprotocol|defrecord|defstruct|deftype|deref|difference|disj|dissoc|distinct|do|doall|doc|dorun|doseq|dosync|dotimes|doto|double|down|drop|drop-while|edit|end\?|ensure|eval|every\?|false\?|ffirst|file-seq|filter|find|find-doc|find-ns|find-var|first|float|flush|fn|fnseq|for|frest|gensym|get|get-proxy-class|hash-map|hash-set|identical\?|identity|if|if-let|if-not|import|in-ns|inc|index|insert-child|insert-left|insert-right|inspect-table|inspect-tree|instance\?|int|interleave|intersection|into|into-array|iterate|join|key|keys|keyword|keyword\?|last|lazy-cat|lazy-cons|left|lefts|let|line-seq|list|list\*|load|load-file|locking|long|loop|macroexpand|macroexpand-1|make-array|make-node|map|map-invert|map\?|mapcat|max|max-key|memfn|merge|merge-with|meta|min|min-key|monitor-enter|name|namespace|neg\?|new|newline|next|nil\?|node|not|not-any\?|not-every\?|not=|ns|ns-imports|ns-interns|ns-map|ns-name|ns-publics|ns-refers|ns-resolve|ns-unmap|nth|nthrest|or|parse|partial|path|peek|pop|pos\?|pr|pr-str|print|print-str|println|println-str|prn|prn-str|project|proxy|proxy-mappings|quot|quote|rand|rand-int|range|re-find|re-groups|re-matcher|re-matches|re-pattern|re-seq|read|read-line|recur|reduce|ref|ref-set|refer|rem|remove|remove-method|remove-ns|rename|rename-keys|repeat|replace|replicate|resolve|rest|resultset-seq|reverse|rfirst|right|rights|root|rrest|rseq|second|select|select-keys|send|send-off|seq|seq-zip|seq\?|set|set!|short|slurp|some|sort|sort-by|sorted-map|sorted-map-by|sorted-set|special-symbol\?|split-at|split-with|str|string\?|struct|struct-map|subs|subvec|symbol|symbol\?|sync|take|take-nth|take-while|test|throw|time|to-array|to-array-2d|tree-seq|true\?|try|union|up|update-proxy|val|vals|var|var-get|var-set|var\?|vector|vector-zip|vector\?|when|when-first|when-let|when-not|with-local-vars|with-meta|with-open|with-out-str|xml-seq|xml-zip|zero\?|zipmap|zipper)(?=[\s)]|$)/,
        lookbehind: true
      },
      boolean: /\b(?:false|nil|true)\b/,
      number: {
        pattern:
          /(^|[^\w$@])(?:\d+(?:[/.]\d+)?(?:e[+-]?\d+)?|0x[a-f0-9]+|[1-9]\d?r[a-z0-9]+)[lmn]?(?![\w$@])/i,
        lookbehind: true
      },
      function: {
        pattern: /((?:^|[^'])\()[\w*+!?'<>=/.-]+(?=[\s)]|$)/,
        lookbehind: true
      },
      operator: /[#@^`~]/,
      punctuation: /[{}\[\](),]/
    };
  }

  // @ts-nocheck
  cmake.displayName = 'cmake';
  cmake.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function cmake(Prism) {
    Prism.languages.cmake = {
      comment: /#.*/,
      string: {
        pattern: /"(?:[^\\"]|\\.)*"/,
        greedy: true,
        inside: {
          interpolation: {
            pattern: /\$\{(?:[^{}$]|\$\{[^{}$]*\})*\}/,
            inside: {
              punctuation: /\$\{|\}/,
              variable: /\w+/
            }
          }
        }
      },
      variable:
        /\b(?:CMAKE_\w+|\w+_(?:(?:BINARY|SOURCE)_DIR|DESCRIPTION|HOMEPAGE_URL|ROOT|VERSION(?:_MAJOR|_MINOR|_PATCH|_TWEAK)?)|(?:ANDROID|APPLE|BORLAND|BUILD_SHARED_LIBS|CACHE|CPACK_(?:ABSOLUTE_DESTINATION_FILES|COMPONENT_INCLUDE_TOPLEVEL_DIRECTORY|ERROR_ON_ABSOLUTE_INSTALL_DESTINATION|INCLUDE_TOPLEVEL_DIRECTORY|INSTALL_DEFAULT_DIRECTORY_PERMISSIONS|INSTALL_SCRIPT|PACKAGING_INSTALL_PREFIX|SET_DESTDIR|WARN_ON_ABSOLUTE_INSTALL_DESTINATION)|CTEST_(?:BINARY_DIRECTORY|BUILD_COMMAND|BUILD_NAME|BZR_COMMAND|BZR_UPDATE_OPTIONS|CHANGE_ID|CHECKOUT_COMMAND|CONFIGURATION_TYPE|CONFIGURE_COMMAND|COVERAGE_COMMAND|COVERAGE_EXTRA_FLAGS|CURL_OPTIONS|CUSTOM_(?:COVERAGE_EXCLUDE|ERROR_EXCEPTION|ERROR_MATCH|ERROR_POST_CONTEXT|ERROR_PRE_CONTEXT|MAXIMUM_FAILED_TEST_OUTPUT_SIZE|MAXIMUM_NUMBER_OF_(?:ERRORS|WARNINGS)|MAXIMUM_PASSED_TEST_OUTPUT_SIZE|MEMCHECK_IGNORE|POST_MEMCHECK|POST_TEST|PRE_MEMCHECK|PRE_TEST|TESTS_IGNORE|WARNING_EXCEPTION|WARNING_MATCH)|CVS_CHECKOUT|CVS_COMMAND|CVS_UPDATE_OPTIONS|DROP_LOCATION|DROP_METHOD|DROP_SITE|DROP_SITE_CDASH|DROP_SITE_PASSWORD|DROP_SITE_USER|EXTRA_COVERAGE_GLOB|GIT_COMMAND|GIT_INIT_SUBMODULES|GIT_UPDATE_CUSTOM|GIT_UPDATE_OPTIONS|HG_COMMAND|HG_UPDATE_OPTIONS|LABELS_FOR_SUBPROJECTS|MEMORYCHECK_(?:COMMAND|COMMAND_OPTIONS|SANITIZER_OPTIONS|SUPPRESSIONS_FILE|TYPE)|NIGHTLY_START_TIME|P4_CLIENT|P4_COMMAND|P4_OPTIONS|P4_UPDATE_OPTIONS|RUN_CURRENT_SCRIPT|SCP_COMMAND|SITE|SOURCE_DIRECTORY|SUBMIT_URL|SVN_COMMAND|SVN_OPTIONS|SVN_UPDATE_OPTIONS|TEST_LOAD|TEST_TIMEOUT|TRIGGER_SITE|UPDATE_COMMAND|UPDATE_OPTIONS|UPDATE_VERSION_ONLY|USE_LAUNCHERS)|CYGWIN|ENV|EXECUTABLE_OUTPUT_PATH|GHS-MULTI|IOS|LIBRARY_OUTPUT_PATH|MINGW|MSVC(?:10|11|12|14|60|70|71|80|90|_IDE|_TOOLSET_VERSION|_VERSION)?|MSYS|PROJECT_(?:BINARY_DIR|DESCRIPTION|HOMEPAGE_URL|NAME|SOURCE_DIR|VERSION|VERSION_(?:MAJOR|MINOR|PATCH|TWEAK))|UNIX|WIN32|WINCE|WINDOWS_PHONE|WINDOWS_STORE|XCODE|XCODE_VERSION))\b/,
      property:
        /\b(?:cxx_\w+|(?:ARCHIVE_OUTPUT_(?:DIRECTORY|NAME)|COMPILE_DEFINITIONS|COMPILE_PDB_NAME|COMPILE_PDB_OUTPUT_DIRECTORY|EXCLUDE_FROM_DEFAULT_BUILD|IMPORTED_(?:IMPLIB|LIBNAME|LINK_DEPENDENT_LIBRARIES|LINK_INTERFACE_LANGUAGES|LINK_INTERFACE_LIBRARIES|LINK_INTERFACE_MULTIPLICITY|LOCATION|NO_SONAME|OBJECTS|SONAME)|INTERPROCEDURAL_OPTIMIZATION|LIBRARY_OUTPUT_DIRECTORY|LIBRARY_OUTPUT_NAME|LINK_FLAGS|LINK_INTERFACE_LIBRARIES|LINK_INTERFACE_MULTIPLICITY|LOCATION|MAP_IMPORTED_CONFIG|OSX_ARCHITECTURES|OUTPUT_NAME|PDB_NAME|PDB_OUTPUT_DIRECTORY|RUNTIME_OUTPUT_DIRECTORY|RUNTIME_OUTPUT_NAME|STATIC_LIBRARY_FLAGS|VS_CSHARP|VS_DOTNET_REFERENCEPROP|VS_DOTNET_REFERENCE|VS_GLOBAL_SECTION_POST|VS_GLOBAL_SECTION_PRE|VS_GLOBAL|XCODE_ATTRIBUTE)_\w+|\w+_(?:CLANG_TIDY|COMPILER_LAUNCHER|CPPCHECK|CPPLINT|INCLUDE_WHAT_YOU_USE|OUTPUT_NAME|POSTFIX|VISIBILITY_PRESET)|ABSTRACT|ADDITIONAL_MAKE_CLEAN_FILES|ADVANCED|ALIASED_TARGET|ALLOW_DUPLICATE_CUSTOM_TARGETS|ANDROID_(?:ANT_ADDITIONAL_OPTIONS|API|API_MIN|ARCH|ASSETS_DIRECTORIES|GUI|JAR_DEPENDENCIES|NATIVE_LIB_DEPENDENCIES|NATIVE_LIB_DIRECTORIES|PROCESS_MAX|PROGUARD|PROGUARD_CONFIG_PATH|SECURE_PROPS_PATH|SKIP_ANT_STEP|STL_TYPE)|ARCHIVE_OUTPUT_DIRECTORY|ATTACHED_FILES|ATTACHED_FILES_ON_FAIL|AUTOGEN_(?:BUILD_DIR|ORIGIN_DEPENDS|PARALLEL|SOURCE_GROUP|TARGETS_FOLDER|TARGET_DEPENDS)|AUTOMOC|AUTOMOC_(?:COMPILER_PREDEFINES|DEPEND_FILTERS|EXECUTABLE|MACRO_NAMES|MOC_OPTIONS|SOURCE_GROUP|TARGETS_FOLDER)|AUTORCC|AUTORCC_EXECUTABLE|AUTORCC_OPTIONS|AUTORCC_SOURCE_GROUP|AUTOUIC|AUTOUIC_EXECUTABLE|AUTOUIC_OPTIONS|AUTOUIC_SEARCH_PATHS|BINARY_DIR|BUILDSYSTEM_TARGETS|BUILD_RPATH|BUILD_RPATH_USE_ORIGIN|BUILD_WITH_INSTALL_NAME_DIR|BUILD_WITH_INSTALL_RPATH|BUNDLE|BUNDLE_EXTENSION|CACHE_VARIABLES|CLEAN_NO_CUSTOM|COMMON_LANGUAGE_RUNTIME|COMPATIBLE_INTERFACE_(?:BOOL|NUMBER_MAX|NUMBER_MIN|STRING)|COMPILE_(?:DEFINITIONS|FEATURES|FLAGS|OPTIONS|PDB_NAME|PDB_OUTPUT_DIRECTORY)|COST|CPACK_DESKTOP_SHORTCUTS|CPACK_NEVER_OVERWRITE|CPACK_PERMANENT|CPACK_STARTUP_SHORTCUTS|CPACK_START_MENU_SHORTCUTS|CPACK_WIX_ACL|CROSSCOMPILING_EMULATOR|CUDA_EXTENSIONS|CUDA_PTX_COMPILATION|CUDA_RESOLVE_DEVICE_SYMBOLS|CUDA_SEPARABLE_COMPILATION|CUDA_STANDARD|CUDA_STANDARD_REQUIRED|CXX_EXTENSIONS|CXX_STANDARD|CXX_STANDARD_REQUIRED|C_EXTENSIONS|C_STANDARD|C_STANDARD_REQUIRED|DEBUG_CONFIGURATIONS|DEFINE_SYMBOL|DEFINITIONS|DEPENDS|DEPLOYMENT_ADDITIONAL_FILES|DEPLOYMENT_REMOTE_DIRECTORY|DISABLED|DISABLED_FEATURES|ECLIPSE_EXTRA_CPROJECT_CONTENTS|ECLIPSE_EXTRA_NATURES|ENABLED_FEATURES|ENABLED_LANGUAGES|ENABLE_EXPORTS|ENVIRONMENT|EXCLUDE_FROM_ALL|EXCLUDE_FROM_DEFAULT_BUILD|EXPORT_NAME|EXPORT_PROPERTIES|EXTERNAL_OBJECT|EchoString|FAIL_REGULAR_EXPRESSION|FIND_LIBRARY_USE_LIB32_PATHS|FIND_LIBRARY_USE_LIB64_PATHS|FIND_LIBRARY_USE_LIBX32_PATHS|FIND_LIBRARY_USE_OPENBSD_VERSIONING|FIXTURES_CLEANUP|FIXTURES_REQUIRED|FIXTURES_SETUP|FOLDER|FRAMEWORK|Fortran_FORMAT|Fortran_MODULE_DIRECTORY|GENERATED|GENERATOR_FILE_NAME|GENERATOR_IS_MULTI_CONFIG|GHS_INTEGRITY_APP|GHS_NO_SOURCE_GROUP_FILE|GLOBAL_DEPENDS_DEBUG_MODE|GLOBAL_DEPENDS_NO_CYCLES|GNUtoMS|HAS_CXX|HEADER_FILE_ONLY|HELPSTRING|IMPLICIT_DEPENDS_INCLUDE_TRANSFORM|IMPORTED|IMPORTED_(?:COMMON_LANGUAGE_RUNTIME|CONFIGURATIONS|GLOBAL|IMPLIB|LIBNAME|LINK_DEPENDENT_LIBRARIES|LINK_INTERFACE_(?:LANGUAGES|LIBRARIES|MULTIPLICITY)|LOCATION|NO_SONAME|OBJECTS|SONAME)|IMPORT_PREFIX|IMPORT_SUFFIX|INCLUDE_DIRECTORIES|INCLUDE_REGULAR_EXPRESSION|INSTALL_NAME_DIR|INSTALL_RPATH|INSTALL_RPATH_USE_LINK_PATH|INTERFACE_(?:AUTOUIC_OPTIONS|COMPILE_DEFINITIONS|COMPILE_FEATURES|COMPILE_OPTIONS|INCLUDE_DIRECTORIES|LINK_DEPENDS|LINK_DIRECTORIES|LINK_LIBRARIES|LINK_OPTIONS|POSITION_INDEPENDENT_CODE|SOURCES|SYSTEM_INCLUDE_DIRECTORIES)|INTERPROCEDURAL_OPTIMIZATION|IN_TRY_COMPILE|IOS_INSTALL_COMBINED|JOB_POOLS|JOB_POOL_COMPILE|JOB_POOL_LINK|KEEP_EXTENSION|LABELS|LANGUAGE|LIBRARY_OUTPUT_DIRECTORY|LINKER_LANGUAGE|LINK_(?:DEPENDS|DEPENDS_NO_SHARED|DIRECTORIES|FLAGS|INTERFACE_LIBRARIES|INTERFACE_MULTIPLICITY|LIBRARIES|OPTIONS|SEARCH_END_STATIC|SEARCH_START_STATIC|WHAT_YOU_USE)|LISTFILE_STACK|LOCATION|MACOSX_BUNDLE|MACOSX_BUNDLE_INFO_PLIST|MACOSX_FRAMEWORK_INFO_PLIST|MACOSX_PACKAGE_LOCATION|MACOSX_RPATH|MACROS|MANUALLY_ADDED_DEPENDENCIES|MEASUREMENT|MODIFIED|NAME|NO_SONAME|NO_SYSTEM_FROM_IMPORTED|OBJECT_DEPENDS|OBJECT_OUTPUTS|OSX_ARCHITECTURES|OUTPUT_NAME|PACKAGES_FOUND|PACKAGES_NOT_FOUND|PARENT_DIRECTORY|PASS_REGULAR_EXPRESSION|PDB_NAME|PDB_OUTPUT_DIRECTORY|POSITION_INDEPENDENT_CODE|POST_INSTALL_SCRIPT|PREDEFINED_TARGETS_FOLDER|PREFIX|PRE_INSTALL_SCRIPT|PRIVATE_HEADER|PROCESSORS|PROCESSOR_AFFINITY|PROJECT_LABEL|PUBLIC_HEADER|REPORT_UNDEFINED_PROPERTIES|REQUIRED_FILES|RESOURCE|RESOURCE_LOCK|RULE_LAUNCH_COMPILE|RULE_LAUNCH_CUSTOM|RULE_LAUNCH_LINK|RULE_MESSAGES|RUNTIME_OUTPUT_DIRECTORY|RUN_SERIAL|SKIP_AUTOGEN|SKIP_AUTOMOC|SKIP_AUTORCC|SKIP_AUTOUIC|SKIP_BUILD_RPATH|SKIP_RETURN_CODE|SOURCES|SOURCE_DIR|SOVERSION|STATIC_LIBRARY_FLAGS|STATIC_LIBRARY_OPTIONS|STRINGS|SUBDIRECTORIES|SUFFIX|SYMBOLIC|TARGET_ARCHIVES_MAY_BE_SHARED_LIBS|TARGET_MESSAGES|TARGET_SUPPORTS_SHARED_LIBS|TESTS|TEST_INCLUDE_FILE|TEST_INCLUDE_FILES|TIMEOUT|TIMEOUT_AFTER_MATCH|TYPE|USE_FOLDERS|VALUE|VARIABLES|VERSION|VISIBILITY_INLINES_HIDDEN|VS_(?:CONFIGURATION_TYPE|COPY_TO_OUT_DIR|DEBUGGER_(?:COMMAND|COMMAND_ARGUMENTS|ENVIRONMENT|WORKING_DIRECTORY)|DEPLOYMENT_CONTENT|DEPLOYMENT_LOCATION|DOTNET_REFERENCES|DOTNET_REFERENCES_COPY_LOCAL|GLOBAL_KEYWORD|GLOBAL_PROJECT_TYPES|GLOBAL_ROOTNAMESPACE|INCLUDE_IN_VSIX|IOT_STARTUP_TASK|KEYWORD|RESOURCE_GENERATOR|SCC_AUXPATH|SCC_LOCALPATH|SCC_PROJECTNAME|SCC_PROVIDER|SDK_REFERENCES|SHADER_(?:DISABLE_OPTIMIZATIONS|ENABLE_DEBUG|ENTRYPOINT|FLAGS|MODEL|OBJECT_FILE_NAME|OUTPUT_HEADER_FILE|TYPE|VARIABLE_NAME)|STARTUP_PROJECT|TOOL_OVERRIDE|USER_PROPS|WINRT_COMPONENT|WINRT_EXTENSIONS|WINRT_REFERENCES|XAML_TYPE)|WILL_FAIL|WIN32_EXECUTABLE|WINDOWS_EXPORT_ALL_SYMBOLS|WORKING_DIRECTORY|WRAP_EXCLUDE|XCODE_(?:EMIT_EFFECTIVE_PLATFORM_NAME|EXPLICIT_FILE_TYPE|FILE_ATTRIBUTES|LAST_KNOWN_FILE_TYPE|PRODUCT_TYPE|SCHEME_(?:ADDRESS_SANITIZER|ADDRESS_SANITIZER_USE_AFTER_RETURN|ARGUMENTS|DISABLE_MAIN_THREAD_CHECKER|DYNAMIC_LIBRARY_LOADS|DYNAMIC_LINKER_API_USAGE|ENVIRONMENT|EXECUTABLE|GUARD_MALLOC|MAIN_THREAD_CHECKER_STOP|MALLOC_GUARD_EDGES|MALLOC_SCRIBBLE|MALLOC_STACK|THREAD_SANITIZER(?:_STOP)?|UNDEFINED_BEHAVIOUR_SANITIZER(?:_STOP)?|ZOMBIE_OBJECTS))|XCTEST)\b/,
      keyword:
        /\b(?:add_compile_definitions|add_compile_options|add_custom_command|add_custom_target|add_definitions|add_dependencies|add_executable|add_library|add_link_options|add_subdirectory|add_test|aux_source_directory|break|build_command|build_name|cmake_host_system_information|cmake_minimum_required|cmake_parse_arguments|cmake_policy|configure_file|continue|create_test_sourcelist|ctest_build|ctest_configure|ctest_coverage|ctest_empty_binary_directory|ctest_memcheck|ctest_read_custom_files|ctest_run_script|ctest_sleep|ctest_start|ctest_submit|ctest_test|ctest_update|ctest_upload|define_property|else|elseif|enable_language|enable_testing|endforeach|endfunction|endif|endmacro|endwhile|exec_program|execute_process|export|export_library_dependencies|file|find_file|find_library|find_package|find_path|find_program|fltk_wrap_ui|foreach|function|get_cmake_property|get_directory_property|get_filename_component|get_property|get_source_file_property|get_target_property|get_test_property|if|include|include_directories|include_external_msproject|include_guard|include_regular_expression|install|install_files|install_programs|install_targets|link_directories|link_libraries|list|load_cache|load_command|macro|make_directory|mark_as_advanced|math|message|option|output_required_files|project|qt_wrap_cpp|qt_wrap_ui|remove|remove_definitions|return|separate_arguments|set|set_directory_properties|set_property|set_source_files_properties|set_target_properties|set_tests_properties|site_name|source_group|string|subdir_depends|subdirs|target_compile_definitions|target_compile_features|target_compile_options|target_include_directories|target_link_directories|target_link_libraries|target_link_options|target_sources|try_compile|try_run|unset|use_mangled_mesa|utility_source|variable_requires|variable_watch|while|write_file)(?=\s*\()\b/,
      boolean: /\b(?:FALSE|OFF|ON|TRUE)\b/,
      namespace:
        /\b(?:INTERFACE|PRIVATE|PROPERTIES|PUBLIC|SHARED|STATIC|TARGET_OBJECTS)\b/,
      operator:
        /\b(?:AND|DEFINED|EQUAL|GREATER|LESS|MATCHES|NOT|OR|STREQUAL|STRGREATER|STRLESS|VERSION_EQUAL|VERSION_GREATER|VERSION_LESS)\b/,
      inserted: {
        pattern: /\b\w+::\w+\b/,
        alias: 'class-name'
      },
      number: /\b\d+(?:\.\d+)*\b/,
      function: /\b[a-z_]\w*(?=\s*\()\b/i,
      punctuation: /[()>}]|\$[<{]/
    };
  }

  // @ts-nocheck
  cobol.displayName = 'cobol';
  cobol.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function cobol(Prism) {
    Prism.languages.cobol = {
      comment: {
        pattern: /\*>.*|(^[ \t]*)\*.*/m,
        lookbehind: true,
        greedy: true
      },
      string: {
        pattern: /[xzgn]?(?:"(?:[^\r\n"]|"")*"(?!")|'(?:[^\r\n']|'')*'(?!'))/i,
        greedy: true
      },
      level: {
        pattern: /(^[ \t]*)\d+\b/m,
        lookbehind: true,
        greedy: true,
        alias: 'number'
      },
      'class-name': {
        // https://github.com/antlr/grammars-v4/blob/42edd5b687d183b5fa679e858a82297bd27141e7/cobol85/Cobol85.g4#L1015
        pattern:
          /(\bpic(?:ture)?\s+)(?:(?:[-\w$/,:*+<>]|\.(?!\s|$))(?:\(\d+\))?)+/i,
        lookbehind: true,
        inside: {
          number: {
            pattern: /(\()\d+/,
            lookbehind: true
          },
          punctuation: /[()]/
        }
      },
      keyword: {
        pattern:
          /(^|[^\w-])(?:ABORT|ACCEPT|ACCESS|ADD|ADDRESS|ADVANCING|AFTER|ALIGNED|ALL|ALPHABET|ALPHABETIC|ALPHABETIC-LOWER|ALPHABETIC-UPPER|ALPHANUMERIC|ALPHANUMERIC-EDITED|ALSO|ALTER|ALTERNATE|ANY|ARE|AREA|AREAS|AS|ASCENDING|ASCII|ASSIGN|ASSOCIATED-DATA|ASSOCIATED-DATA-LENGTH|AT|ATTRIBUTE|AUTHOR|AUTO|AUTO-SKIP|BACKGROUND-COLOR|BACKGROUND-COLOUR|BASIS|BEEP|BEFORE|BEGINNING|BELL|BINARY|BIT|BLANK|BLINK|BLOCK|BOTTOM|BOUNDS|BY|BYFUNCTION|BYTITLE|CALL|CANCEL|CAPABLE|CCSVERSION|CD|CF|CH|CHAINING|CHANGED|CHANNEL|CHARACTER|CHARACTERS|CLASS|CLASS-ID|CLOCK-UNITS|CLOSE|CLOSE-DISPOSITION|COBOL|CODE|CODE-SET|COL|COLLATING|COLUMN|COM-REG|COMMA|COMMITMENT|COMMON|COMMUNICATION|COMP|COMP-1|COMP-2|COMP-3|COMP-4|COMP-5|COMPUTATIONAL|COMPUTATIONAL-1|COMPUTATIONAL-2|COMPUTATIONAL-3|COMPUTATIONAL-4|COMPUTATIONAL-5|COMPUTE|CONFIGURATION|CONTAINS|CONTENT|CONTINUE|CONTROL|CONTROL-POINT|CONTROLS|CONVENTION|CONVERTING|COPY|CORR|CORRESPONDING|COUNT|CRUNCH|CURRENCY|CURSOR|DATA|DATA-BASE|DATE|DATE-COMPILED|DATE-WRITTEN|DAY|DAY-OF-WEEK|DBCS|DE|DEBUG-CONTENTS|DEBUG-ITEM|DEBUG-LINE|DEBUG-NAME|DEBUG-SUB-1|DEBUG-SUB-2|DEBUG-SUB-3|DEBUGGING|DECIMAL-POINT|DECLARATIVES|DEFAULT|DEFAULT-DISPLAY|DEFINITION|DELETE|DELIMITED|DELIMITER|DEPENDING|DESCENDING|DESTINATION|DETAIL|DFHRESP|DFHVALUE|DISABLE|DISK|DISPLAY|DISPLAY-1|DIVIDE|DIVISION|DONTCARE|DOUBLE|DOWN|DUPLICATES|DYNAMIC|EBCDIC|EGCS|EGI|ELSE|EMI|EMPTY-CHECK|ENABLE|END|END-ACCEPT|END-ADD|END-CALL|END-COMPUTE|END-DELETE|END-DIVIDE|END-EVALUATE|END-IF|END-MULTIPLY|END-OF-PAGE|END-PERFORM|END-READ|END-RECEIVE|END-RETURN|END-REWRITE|END-SEARCH|END-START|END-STRING|END-SUBTRACT|END-UNSTRING|END-WRITE|ENDING|ENTER|ENTRY|ENTRY-PROCEDURE|ENVIRONMENT|EOL|EOP|EOS|ERASE|ERROR|ESCAPE|ESI|EVALUATE|EVENT|EVERY|EXCEPTION|EXCLUSIVE|EXHIBIT|EXIT|EXPORT|EXTEND|EXTENDED|EXTERNAL|FD|FILE|FILE-CONTROL|FILLER|FINAL|FIRST|FOOTING|FOR|FOREGROUND-COLOR|FOREGROUND-COLOUR|FROM|FULL|FUNCTION|FUNCTION-POINTER|FUNCTIONNAME|GENERATE|GIVING|GLOBAL|GO|GOBACK|GRID|GROUP|HEADING|HIGH-VALUE|HIGH-VALUES|HIGHLIGHT|I-O|I-O-CONTROL|ID|IDENTIFICATION|IF|IMPLICIT|IMPORT|IN|INDEX|INDEXED|INDICATE|INITIAL|INITIALIZE|INITIATE|INPUT|INPUT-OUTPUT|INSPECT|INSTALLATION|INTEGER|INTO|INVALID|INVOKE|IS|JUST|JUSTIFIED|KANJI|KEPT|KEY|KEYBOARD|LABEL|LANGUAGE|LAST|LB|LD|LEADING|LEFT|LEFTLINE|LENGTH|LENGTH-CHECK|LIBACCESS|LIBPARAMETER|LIBRARY|LIMIT|LIMITS|LINAGE|LINAGE-COUNTER|LINE|LINE-COUNTER|LINES|LINKAGE|LIST|LOCAL|LOCAL-STORAGE|LOCK|LONG-DATE|LONG-TIME|LOW-VALUE|LOW-VALUES|LOWER|LOWLIGHT|MEMORY|MERGE|MESSAGE|MMDDYYYY|MODE|MODULES|MORE-LABELS|MOVE|MULTIPLE|MULTIPLY|NAMED|NATIONAL|NATIONAL-EDITED|NATIVE|NEGATIVE|NETWORK|NEXT|NO|NO-ECHO|NULL|NULLS|NUMBER|NUMERIC|NUMERIC-DATE|NUMERIC-EDITED|NUMERIC-TIME|OBJECT-COMPUTER|OCCURS|ODT|OF|OFF|OMITTED|ON|OPEN|OPTIONAL|ORDER|ORDERLY|ORGANIZATION|OTHER|OUTPUT|OVERFLOW|OVERLINE|OWN|PACKED-DECIMAL|PADDING|PAGE|PAGE-COUNTER|PASSWORD|PERFORM|PF|PH|PIC|PICTURE|PLUS|POINTER|PORT|POSITION|POSITIVE|PRINTER|PRINTING|PRIVATE|PROCEDURE|PROCEDURE-POINTER|PROCEDURES|PROCEED|PROCESS|PROGRAM|PROGRAM-ID|PROGRAM-LIBRARY|PROMPT|PURGE|QUEUE|QUOTE|QUOTES|RANDOM|RD|READ|READER|REAL|RECEIVE|RECEIVED|RECORD|RECORDING|RECORDS|RECURSIVE|REDEFINES|REEL|REF|REFERENCE|REFERENCES|RELATIVE|RELEASE|REMAINDER|REMARKS|REMOTE|REMOVAL|REMOVE|RENAMES|REPLACE|REPLACING|REPORT|REPORTING|REPORTS|REQUIRED|RERUN|RESERVE|RESET|RETURN|RETURN-CODE|RETURNING|REVERSE-VIDEO|REVERSED|REWIND|REWRITE|RF|RH|RIGHT|ROUNDED|RUN|SAME|SAVE|SCREEN|SD|SEARCH|SECTION|SECURE|SECURITY|SEGMENT|SEGMENT-LIMIT|SELECT|SEND|SENTENCE|SEPARATE|SEQUENCE|SEQUENTIAL|SET|SHARED|SHAREDBYALL|SHAREDBYRUNUNIT|SHARING|SHIFT-IN|SHIFT-OUT|SHORT-DATE|SIGN|SIZE|SORT|SORT-CONTROL|SORT-CORE-SIZE|SORT-FILE-SIZE|SORT-MERGE|SORT-MESSAGE|SORT-MODE-SIZE|SORT-RETURN|SOURCE|SOURCE-COMPUTER|SPACE|SPACES|SPECIAL-NAMES|STANDARD|STANDARD-1|STANDARD-2|START|STATUS|STOP|STRING|SUB-QUEUE-1|SUB-QUEUE-2|SUB-QUEUE-3|SUBTRACT|SUM|SUPPRESS|SYMBOL|SYMBOLIC|SYNC|SYNCHRONIZED|TABLE|TALLY|TALLYING|TAPE|TASK|TERMINAL|TERMINATE|TEST|TEXT|THEN|THREAD|THREAD-LOCAL|THROUGH|THRU|TIME|TIMER|TIMES|TITLE|TO|TODAYS-DATE|TODAYS-NAME|TOP|TRAILING|TRUNCATED|TYPE|TYPEDEF|UNDERLINE|UNIT|UNSTRING|UNTIL|UP|UPON|USAGE|USE|USING|VALUE|VALUES|VARYING|VIRTUAL|WAIT|WHEN|WHEN-COMPILED|WITH|WORDS|WORKING-STORAGE|WRITE|YEAR|YYYYDDD|YYYYMMDD|ZERO-FILL|ZEROES|ZEROS)(?![\w-])/i,
        lookbehind: true
      },
      boolean: {
        pattern: /(^|[^\w-])(?:false|true)(?![\w-])/i,
        lookbehind: true
      },
      number: {
        pattern:
          /(^|[^\w-])(?:[+-]?(?:(?:\d+(?:[.,]\d+)?|[.,]\d+)(?:e[+-]?\d+)?|zero))(?![\w-])/i,
        lookbehind: true
      },
      operator: [
        /<>|[<>]=?|[=+*/&]/,
        {
          pattern: /(^|[^\w-])(?:-|and|equal|greater|less|not|or|than)(?![\w-])/i,
          lookbehind: true
        }
      ],
      punctuation: /[.:,()]/
    };
  }

  // @ts-nocheck
  coffeescript.displayName = 'coffeescript';
  coffeescript.aliases = ['coffee'];

  /** @type {import('../core.js').Syntax} */
  function coffeescript(Prism) {
    Prism.register(javascript)
    ;(function (Prism) {
      // Ignore comments starting with { to privilege string interpolation highlighting
      var comment = /#(?!\{).+/;
      var interpolation = {
        pattern: /#\{[^}]+\}/,
        alias: 'variable'
      };
      Prism.languages.coffeescript = Prism.languages.extend('javascript', {
        comment: comment,
        string: [
          // Strings are multiline
          {
            pattern: /'(?:\\[\s\S]|[^\\'])*'/,
            greedy: true
          },
          {
            // Strings are multiline
            pattern: /"(?:\\[\s\S]|[^\\"])*"/,
            greedy: true,
            inside: {
              interpolation: interpolation
            }
          }
        ],
        keyword:
          /\b(?:and|break|by|catch|class|continue|debugger|delete|do|each|else|extend|extends|false|finally|for|if|in|instanceof|is|isnt|let|loop|namespace|new|no|not|null|of|off|on|or|own|return|super|switch|then|this|throw|true|try|typeof|undefined|unless|until|when|while|window|with|yes|yield)\b/,
        'class-member': {
          pattern: /@(?!\d)\w+/,
          alias: 'variable'
        }
      });
      Prism.languages.insertBefore('coffeescript', 'comment', {
        'multiline-comment': {
          pattern: /###[\s\S]+?###/,
          alias: 'comment'
        },
        // Block regexp can contain comments and interpolation
        'block-regex': {
          pattern: /\/{3}[\s\S]*?\/{3}/,
          alias: 'regex',
          inside: {
            comment: comment,
            interpolation: interpolation
          }
        }
      });
      Prism.languages.insertBefore('coffeescript', 'string', {
        'inline-javascript': {
          pattern: /`(?:\\[\s\S]|[^\\`])*`/,
          inside: {
            delimiter: {
              pattern: /^`|`$/,
              alias: 'punctuation'
            },
            script: {
              pattern: /[\s\S]+/,
              alias: 'language-javascript',
              inside: Prism.languages.javascript
            }
          }
        },
        // Block strings
        'multiline-string': [
          {
            pattern: /'''[\s\S]*?'''/,
            greedy: true,
            alias: 'string'
          },
          {
            pattern: /"""[\s\S]*?"""/,
            greedy: true,
            alias: 'string',
            inside: {
              interpolation: interpolation
            }
          }
        ]
      });
      Prism.languages.insertBefore('coffeescript', 'keyword', {
        // Object property
        property: /(?!\d)\w+(?=\s*:(?!:))/
      });
      delete Prism.languages.coffeescript['template-string'];
      Prism.languages.coffee = Prism.languages.coffeescript;
    })(Prism);
  }

  // @ts-nocheck
  concurnas.displayName = 'concurnas';
  concurnas.aliases = ['conc'];

  /** @type {import('../core.js').Syntax} */
  function concurnas(Prism) {
    Prism.languages.concurnas = {
      comment: {
        pattern: /(^|[^\\])(?:\/\*[\s\S]*?(?:\*\/|$)|\/\/.*)/,
        lookbehind: true,
        greedy: true
      },
      langext: {
        pattern: /\b\w+\s*\|\|[\s\S]+?\|\|/,
        greedy: true,
        inside: {
          'class-name': /^\w+/,
          string: {
            pattern: /(^\s*\|\|)[\s\S]+(?=\|\|$)/,
            lookbehind: true
          },
          punctuation: /\|\|/
        }
      },
      function: {
        pattern: /((?:^|\s)def[ \t]+)[a-zA-Z_]\w*(?=\s*\()/,
        lookbehind: true
      },
      keyword:
        /\b(?:abstract|actor|also|annotation|assert|async|await|bool|boolean|break|byte|case|catch|changed|char|class|closed|constant|continue|def|default|del|double|elif|else|enum|every|extends|false|finally|float|for|from|global|gpudef|gpukernel|if|import|in|init|inject|int|lambda|local|long|loop|match|new|nodefault|null|of|onchange|open|out|override|package|parfor|parforsync|post|pre|private|protected|provide|provider|public|return|shared|short|single|size_t|sizeof|super|sync|this|throw|trait|trans|transient|true|try|typedef|unchecked|using|val|var|void|while|with)\b/,
      boolean: /\b(?:false|true)\b/,
      number:
        /\b0b[01][01_]*L?\b|\b0x(?:[\da-f_]*\.)?[\da-f_p+-]+\b|(?:\b\d[\d_]*(?:\.[\d_]*)?|\B\.\d[\d_]*)(?:e[+-]?\d[\d_]*)?[dfls]?/i,
      punctuation: /[{}[\];(),.:]/,
      operator:
        /<==|>==|=>|->|<-|<>|&==|&<>|\?:?|\.\?|\+\+|--|[-+*/=<>]=?|[!^~]|\b(?:and|as|band|bor|bxor|comp|is|isnot|mod|or)\b=?/,
      annotation: {
        pattern: /@(?:\w+:)?(?:\w+|\[[^\]]+\])?/,
        alias: 'builtin'
      }
    };
    Prism.languages.insertBefore('concurnas', 'langext', {
      'regex-literal': {
        pattern: /\br("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,
        greedy: true,
        inside: {
          interpolation: {
            pattern:
              /((?:^|[^\\])(?:\\{2})*)\{(?:[^{}]|\{(?:[^{}]|\{[^}]*\})*\})+\}/,
            lookbehind: true,
            inside: Prism.languages.concurnas
          },
          regex: /[\s\S]+/
        }
      },
      'string-literal': {
        pattern: /(?:\B|\bs)("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,
        greedy: true,
        inside: {
          interpolation: {
            pattern:
              /((?:^|[^\\])(?:\\{2})*)\{(?:[^{}]|\{(?:[^{}]|\{[^}]*\})*\})+\}/,
            lookbehind: true,
            inside: Prism.languages.concurnas
          },
          string: /[\s\S]+/
        }
      }
    });
    Prism.languages.conc = Prism.languages.concurnas;
  }

  // @ts-nocheck
  csp.displayName = 'csp';
  csp.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function csp(Prism) {
  (function (Prism) {
      /**
       * @param {string} source
       * @returns {RegExp}
       */
      function value(source) {
        return RegExp(
          /([ \t])/.source + '(?:' + source + ')' + /(?=[\s;]|$)/.source,
          'i'
        )
      }
      Prism.languages.csp = {
        directive: {
          pattern:
            /(^|[\s;])(?:base-uri|block-all-mixed-content|(?:child|connect|default|font|frame|img|manifest|media|object|prefetch|script|style|worker)-src|disown-opener|form-action|frame-(?:ancestors|options)|input-protection(?:-(?:clip|selectors))?|navigate-to|plugin-types|policy-uri|referrer|reflected-xss|report-(?:to|uri)|require-sri-for|sandbox|(?:script|style)-src-(?:attr|elem)|upgrade-insecure-requests)(?=[\s;]|$)/i,
          lookbehind: true,
          alias: 'property'
        },
        scheme: {
          pattern: value(/[a-z][a-z0-9.+-]*:/.source),
          lookbehind: true
        },
        none: {
          pattern: value(/'none'/.source),
          lookbehind: true,
          alias: 'keyword'
        },
        nonce: {
          pattern: value(/'nonce-[-+/\w=]+'/.source),
          lookbehind: true,
          alias: 'number'
        },
        hash: {
          pattern: value(/'sha(?:256|384|512)-[-+/\w=]+'/.source),
          lookbehind: true,
          alias: 'number'
        },
        host: {
          pattern: value(
            /[a-z][a-z0-9.+-]*:\/\/[^\s;,']*/.source +
              '|' +
              /\*[^\s;,']*/.source +
              '|' +
              /[a-z0-9-]+(?:\.[a-z0-9-]+)+(?::[\d*]+)?(?:\/[^\s;,']*)?/.source
          ),
          lookbehind: true,
          alias: 'url',
          inside: {
            important: /\*/
          }
        },
        keyword: [
          {
            pattern: value(/'unsafe-[a-z-]+'/.source),
            lookbehind: true,
            alias: 'unsafe'
          },
          {
            pattern: value(/'[a-z-]+'/.source),
            lookbehind: true,
            alias: 'safe'
          }
        ],
        punctuation: /;/
      };
    })(Prism);
  }

  // @ts-nocheck
  coq.displayName = 'coq';
  coq.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function coq(Prism) {
  (function (Prism) {
      // https://github.com/coq/coq
      var commentSource = /\(\*(?:[^(*]|\((?!\*)|\*(?!\))|<self>)*\*\)/.source;
      for (var i = 0; i < 2; i++) {
        commentSource = commentSource.replace(/<self>/g, function () {
          return commentSource
        });
      }
      commentSource = commentSource.replace(/<self>/g, '[]');
      Prism.languages.coq = {
        comment: RegExp(commentSource),
        string: {
          pattern: /"(?:[^"]|"")*"(?!")/,
          greedy: true
        },
        attribute: [
          {
            pattern: RegExp(
              /#\[(?:[^\[\]("]|"(?:[^"]|"")*"(?!")|\((?!\*)|<comment>)*\]/.source.replace(
                /<comment>/g,
                function () {
                  return commentSource
                }
              )
            ),
            greedy: true,
            alias: 'attr-name',
            inside: {
              comment: RegExp(commentSource),
              string: {
                pattern: /"(?:[^"]|"")*"(?!")/,
                greedy: true
              },
              operator: /=/,
              punctuation: /^#\[|\]$|[,()]/
            }
          },
          {
            pattern:
              /\b(?:Cumulative|Global|Local|Monomorphic|NonCumulative|Polymorphic|Private|Program)\b/,
            alias: 'attr-name'
          }
        ],
        keyword:
          /\b(?:Abort|About|Add|Admit|Admitted|All|Arguments|As|Assumptions|Axiom|Axioms|Back|BackTo|Backtrace|BinOp|BinOpSpec|BinRel|Bind|Blacklist|Canonical|Case|Cd|Check|Class|Classes|Close|CoFixpoint|CoInductive|Coercion|Coercions|Collection|Combined|Compute|Conjecture|Conjectures|Constant|Constants|Constraint|Constructors|Context|Corollary|Create|CstOp|Custom|Cut|Debug|Declare|Defined|Definition|Delimit|Dependencies|Dependent|Derive|Diffs|Drop|Elimination|End|Entry|Equality|Eval|Example|Existential|Existentials|Existing|Export|Extern|Extraction|Fact|Fail|Field|File|Firstorder|Fixpoint|Flags|Focus|From|Funclass|Function|Functional|GC|Generalizable|Goal|Grab|Grammar|Graph|Guarded|Haskell|Heap|Hide|Hint|HintDb|Hints|Hypotheses|Hypothesis|IF|Identity|Immediate|Implicit|Implicits|Import|Include|Induction|Inductive|Infix|Info|Initial|InjTyp|Inline|Inspect|Instance|Instances|Intro|Intros|Inversion|Inversion_clear|JSON|Language|Left|Lemma|Let|Lia|Libraries|Library|Load|LoadPath|Locate|Ltac|Ltac2|ML|Match|Method|Minimality|Module|Modules|Morphism|Next|NoInline|Notation|Number|OCaml|Obligation|Obligations|Opaque|Open|Optimize|Parameter|Parameters|Parametric|Path|Paths|Prenex|Preterm|Primitive|Print|Profile|Projections|Proof|Prop|PropBinOp|PropOp|PropUOp|Property|Proposition|Pwd|Qed|Quit|Rec|Record|Recursive|Redirect|Reduction|Register|Relation|Remark|Remove|Require|Reserved|Reset|Resolve|Restart|Rewrite|Right|Ring|Rings|SProp|Saturate|Save|Scheme|Scope|Scopes|Search|SearchHead|SearchPattern|SearchRewrite|Section|Separate|Set|Setoid|Show|Signatures|Solve|Solver|Sort|Sortclass|Sorted|Spec|Step|Strategies|Strategy|String|Structure|SubClass|Subgraph|SuchThat|Tactic|Term|TestCompile|Theorem|Time|Timeout|To|Transparent|Type|Typeclasses|Types|Typing|UnOp|UnOpSpec|Undelimit|Undo|Unfocus|Unfocused|Unfold|Universe|Universes|Unshelve|Variable|Variables|Variant|Verbose|View|Visibility|Zify|_|apply|as|at|by|cofix|else|end|exists|exists2|fix|for|forall|fun|if|in|let|match|measure|move|removed|return|struct|then|using|wf|where|with)\b/,
        number:
          /\b(?:0x[a-f0-9][a-f0-9_]*(?:\.[a-f0-9_]+)?(?:p[+-]?\d[\d_]*)?|\d[\d_]*(?:\.[\d_]+)?(?:e[+-]?\d[\d_]*)?)\b/i,
        punct: {
          pattern: /@\{|\{\||\[=|:>/,
          alias: 'punctuation'
        },
        operator:
          /\/\\|\\\/|\.{2,3}|:{1,2}=|\*\*|[-=]>|<(?:->?|[+:=>]|<:)|>(?:=|->)|\|[-|]?|[-!%&*+/<=>?@^~']/,
        punctuation: /\.\(|`\(|@\{|`\{|\{\||\[=|:>|[:.,;(){}\[\]]/
      };
    })(Prism);
  }

  // @ts-nocheck
  crystal.displayName = 'crystal';
  crystal.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function crystal(Prism) {
    Prism.register(ruby)
    ;(function (Prism) {
      Prism.languages.crystal = Prism.languages.extend('ruby', {
        keyword: [
          /\b(?:__DIR__|__END_LINE__|__FILE__|__LINE__|abstract|alias|annotation|as|asm|begin|break|case|class|def|do|else|elsif|end|ensure|enum|extend|for|fun|if|ifdef|include|instance_sizeof|lib|macro|module|next|of|out|pointerof|private|protected|ptr|require|rescue|return|select|self|sizeof|struct|super|then|type|typeof|undef|uninitialized|union|unless|until|when|while|with|yield)\b/,
          {
            pattern: /(\.\s*)(?:is_a|responds_to)\?/,
            lookbehind: true
          }
        ],
        number:
          /\b(?:0b[01_]*[01]|0o[0-7_]*[0-7]|0x[\da-fA-F_]*[\da-fA-F]|(?:\d(?:[\d_]*\d)?)(?:\.[\d_]*\d)?(?:[eE][+-]?[\d_]*\d)?)(?:_(?:[uif](?:8|16|32|64))?)?\b/,
        operator: [/->/, Prism.languages.ruby.operator],
        punctuation: /[(){}[\].,;\\]/
      });
      Prism.languages.insertBefore('crystal', 'string-literal', {
        attribute: {
          pattern: /@\[.*?\]/,
          inside: {
            delimiter: {
              pattern: /^@\[|\]$/,
              alias: 'punctuation'
            },
            attribute: {
              pattern: /^(\s*)\w+/,
              lookbehind: true,
              alias: 'class-name'
            },
            args: {
              pattern: /\S(?:[\s\S]*\S)?/,
              inside: Prism.languages.crystal
            }
          }
        },
        expansion: {
          pattern: /\{(?:\{.*?\}|%.*?%)\}/,
          inside: {
            content: {
              pattern: /^(\{.)[\s\S]+(?=.\}$)/,
              lookbehind: true,
              inside: Prism.languages.crystal
            },
            delimiter: {
              pattern: /^\{[\{%]|[\}%]\}$/,
              alias: 'operator'
            }
          }
        },
        char: {
          pattern:
            /'(?:[^\\\r\n]{1,2}|\\(?:.|u(?:[A-Fa-f0-9]{1,4}|\{[A-Fa-f0-9]{1,6}\})))'/,
          greedy: true
        }
      });
    })(Prism);
  }

  // @ts-nocheck
  cssExtras.displayName = 'css-extras';
  cssExtras.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function cssExtras(Prism) {
    Prism.register(css)
    ;(function (Prism) {
      var string = /("|')(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/;
      var selectorInside;
      Prism.languages.css.selector = {
        pattern: Prism.languages.css.selector.pattern,
        lookbehind: true,
        inside: (selectorInside = {
          'pseudo-element':
            /:(?:after|before|first-letter|first-line|selection)|::[-\w]+/,
          'pseudo-class': /:[-\w]+/,
          class: /\.[-\w]+/,
          id: /#[-\w]+/,
          attribute: {
            pattern: RegExp('\\[(?:[^[\\]"\']|' + string.source + ')*\\]'),
            greedy: true,
            inside: {
              punctuation: /^\[|\]$/,
              'case-sensitivity': {
                pattern: /(\s)[si]$/i,
                lookbehind: true,
                alias: 'keyword'
              },
              namespace: {
                pattern: /^(\s*)(?:(?!\s)[-*\w\xA0-\uFFFF])*\|(?!=)/,
                lookbehind: true,
                inside: {
                  punctuation: /\|$/
                }
              },
              'attr-name': {
                pattern: /^(\s*)(?:(?!\s)[-\w\xA0-\uFFFF])+/,
                lookbehind: true
              },
              'attr-value': [
                string,
                {
                  pattern: /(=\s*)(?:(?!\s)[-\w\xA0-\uFFFF])+(?=\s*$)/,
                  lookbehind: true
                }
              ],
              operator: /[|~*^$]?=/
            }
          },
          'n-th': [
            {
              pattern: /(\(\s*)[+-]?\d*[\dn](?:\s*[+-]\s*\d+)?(?=\s*\))/,
              lookbehind: true,
              inside: {
                number: /[\dn]+/,
                operator: /[+-]/
              }
            },
            {
              pattern: /(\(\s*)(?:even|odd)(?=\s*\))/i,
              lookbehind: true
            }
          ],
          combinator: />|\+|~|\|\|/,
          // the `tag` token has been existed and removed.
          // because we can't find a perfect tokenize to match it.
          // if you want to add it, please read https://github.com/PrismJS/prism/pull/2373 first.
          punctuation: /[(),]/
        })
      };
      Prism.languages.css['atrule'].inside['selector-function-argument'].inside =
        selectorInside;
      Prism.languages.insertBefore('css', 'property', {
        variable: {
          pattern:
            /(^|[^-\w\xA0-\uFFFF])--(?!\s)[-_a-z\xA0-\uFFFF](?:(?!\s)[-\w\xA0-\uFFFF])*/i,
          lookbehind: true
        }
      });
      var unit = {
        pattern: /(\b\d+)(?:%|[a-z]+(?![\w-]))/,
        lookbehind: true
      }; // 123 -123 .123 -.123 12.3 -12.3
      var number = {
        pattern: /(^|[^\w.-])-?(?:\d+(?:\.\d+)?|\.\d+)/,
        lookbehind: true
      };
      Prism.languages.insertBefore('css', 'function', {
        operator: {
          pattern: /(\s)[+\-*\/](?=\s)/,
          lookbehind: true
        },
        // CAREFUL!
        // Previewers and Inline color use hexcode and color.
        hexcode: {
          pattern: /\B#[\da-f]{3,8}\b/i,
          alias: 'color'
        },
        color: [
          {
            pattern:
              /(^|[^\w-])(?:AliceBlue|AntiqueWhite|Aqua|Aquamarine|Azure|Beige|Bisque|Black|BlanchedAlmond|Blue|BlueViolet|Brown|BurlyWood|CadetBlue|Chartreuse|Chocolate|Coral|CornflowerBlue|Cornsilk|Crimson|Cyan|DarkBlue|DarkCyan|DarkGoldenRod|DarkGr[ae]y|DarkGreen|DarkKhaki|DarkMagenta|DarkOliveGreen|DarkOrange|DarkOrchid|DarkRed|DarkSalmon|DarkSeaGreen|DarkSlateBlue|DarkSlateGr[ae]y|DarkTurquoise|DarkViolet|DeepPink|DeepSkyBlue|DimGr[ae]y|DodgerBlue|FireBrick|FloralWhite|ForestGreen|Fuchsia|Gainsboro|GhostWhite|Gold|GoldenRod|Gr[ae]y|Green|GreenYellow|HoneyDew|HotPink|IndianRed|Indigo|Ivory|Khaki|Lavender|LavenderBlush|LawnGreen|LemonChiffon|LightBlue|LightCoral|LightCyan|LightGoldenRodYellow|LightGr[ae]y|LightGreen|LightPink|LightSalmon|LightSeaGreen|LightSkyBlue|LightSlateGr[ae]y|LightSteelBlue|LightYellow|Lime|LimeGreen|Linen|Magenta|Maroon|MediumAquaMarine|MediumBlue|MediumOrchid|MediumPurple|MediumSeaGreen|MediumSlateBlue|MediumSpringGreen|MediumTurquoise|MediumVioletRed|MidnightBlue|MintCream|MistyRose|Moccasin|NavajoWhite|Navy|OldLace|Olive|OliveDrab|Orange|OrangeRed|Orchid|PaleGoldenRod|PaleGreen|PaleTurquoise|PaleVioletRed|PapayaWhip|PeachPuff|Peru|Pink|Plum|PowderBlue|Purple|Red|RosyBrown|RoyalBlue|SaddleBrown|Salmon|SandyBrown|SeaGreen|SeaShell|Sienna|Silver|SkyBlue|SlateBlue|SlateGr[ae]y|Snow|SpringGreen|SteelBlue|Tan|Teal|Thistle|Tomato|Transparent|Turquoise|Violet|Wheat|White|WhiteSmoke|Yellow|YellowGreen)(?![\w-])/i,
            lookbehind: true
          },
          {
            pattern:
              /\b(?:hsl|rgb)\(\s*\d{1,3}\s*,\s*\d{1,3}%?\s*,\s*\d{1,3}%?\s*\)\B|\b(?:hsl|rgb)a\(\s*\d{1,3}\s*,\s*\d{1,3}%?\s*,\s*\d{1,3}%?\s*,\s*(?:0|0?\.\d+|1)\s*\)\B/i,
            inside: {
              unit: unit,
              number: number,
              function: /[\w-]+(?=\()/,
              punctuation: /[(),]/
            }
          }
        ],
        // it's important that there is no boundary assertion after the hex digits
        entity: /\\[\da-f]{1,8}/i,
        unit: unit,
        number: number
      });
    })(Prism);
  }

  // @ts-nocheck
  csv.displayName = 'csv';
  csv.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function csv(Prism) {
    // https://tools.ietf.org/html/rfc4180
    Prism.languages.csv = {
      value: /[^\r\n,"]+|"(?:[^"]|"")*"(?!")/,
      punctuation: /,/
    };
  }

  // @ts-nocheck
  cypher.displayName = 'cypher';
  cypher.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function cypher(Prism) {
    Prism.languages.cypher = {
      // https://neo4j.com/docs/cypher-manual/current/syntax/comments/
      comment: /\/\/.*/,
      string: {
        pattern: /"(?:[^"\\\r\n]|\\.)*"|'(?:[^'\\\r\n]|\\.)*'/,
        greedy: true
      },
      'class-name': {
        pattern: /(:\s*)(?:\w+|`(?:[^`\\\r\n])*`)(?=\s*[{):])/,
        lookbehind: true,
        greedy: true
      },
      relationship: {
        pattern:
          /(-\[\s*(?:\w+\s*|`(?:[^`\\\r\n])*`\s*)?:\s*|\|\s*:\s*)(?:\w+|`(?:[^`\\\r\n])*`)/,
        lookbehind: true,
        greedy: true,
        alias: 'property'
      },
      identifier: {
        pattern: /`(?:[^`\\\r\n])*`/,
        greedy: true
      },
      variable: /\$\w+/,
      // https://neo4j.com/docs/cypher-manual/current/syntax/reserved/
      keyword:
        /\b(?:ADD|ALL|AND|AS|ASC|ASCENDING|ASSERT|BY|CALL|CASE|COMMIT|CONSTRAINT|CONTAINS|CREATE|CSV|DELETE|DESC|DESCENDING|DETACH|DISTINCT|DO|DROP|ELSE|END|ENDS|EXISTS|FOR|FOREACH|IN|INDEX|IS|JOIN|KEY|LIMIT|LOAD|MANDATORY|MATCH|MERGE|NODE|NOT|OF|ON|OPTIONAL|OR|ORDER(?=\s+BY)|PERIODIC|REMOVE|REQUIRE|RETURN|SCALAR|SCAN|SET|SKIP|START|STARTS|THEN|UNION|UNIQUE|UNWIND|USING|WHEN|WHERE|WITH|XOR|YIELD)\b/i,
      function: /\b\w+\b(?=\s*\()/,
      boolean: /\b(?:false|null|true)\b/i,
      number: /\b(?:0x[\da-fA-F]+|\d+(?:\.\d+)?(?:[eE][+-]?\d+)?)\b/,
      // https://neo4j.com/docs/cypher-manual/current/syntax/operators/
      operator: /:|<--?|--?>?|<>|=~?|[<>]=?|[+*/%^|]|\.\.\.?/,
      punctuation: /[()[\]{},;.]/
    };
  }

  // @ts-nocheck
  d.displayName = 'd';
  d.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function d(Prism) {
    Prism.register(clike);
    Prism.languages.d = Prism.languages.extend('clike', {
      comment: [
        {
          // Shebang
          pattern: /^\s*#!.+/,
          greedy: true
        },
        {
          pattern: RegExp(
            /(^|[^\\])/.source +
              '(?:' +
              [
                // /+ comment +/
                // Allow one level of nesting
                /\/\+(?:\/\+(?:[^+]|\+(?!\/))*\+\/|(?!\/\+)[\s\S])*?\+\//.source, // // comment
                /\/\/.*/.source, // /* comment */
                /\/\*[\s\S]*?\*\//.source
              ].join('|') +
              ')'
          ),
          lookbehind: true,
          greedy: true
        }
      ],
      string: [
        {
          pattern: RegExp(
            [
              // r"", x""
              /\b[rx]"(?:\\[\s\S]|[^\\"])*"[cwd]?/.source, // q"[]", q"()", q"<>", q"{}"
              /\bq"(?:\[[\s\S]*?\]|\([\s\S]*?\)|<[\s\S]*?>|\{[\s\S]*?\})"/.source, // q"IDENT
              // ...
              // IDENT"
              /\bq"((?!\d)\w+)$[\s\S]*?^\1"/.source, // q"//", q"||", etc.
              // eslint-disable-next-line regexp/strict
              /\bq"(.)[\s\S]*?\2"/.source, // eslint-disable-next-line regexp/strict
              /(["`])(?:\\[\s\S]|(?!\3)[^\\])*\3[cwd]?/.source
            ].join('|'),
            'm'
          ),
          greedy: true
        },
        {
          pattern: /\bq\{(?:\{[^{}]*\}|[^{}])*\}/,
          greedy: true,
          alias: 'token-string'
        }
      ],
      // In order: $, keywords and special tokens, globally defined symbols
      keyword:
        /\$|\b(?:__(?:(?:DATE|EOF|FILE|FUNCTION|LINE|MODULE|PRETTY_FUNCTION|TIMESTAMP|TIME|VENDOR|VERSION)__|gshared|parameters|traits|vector)|abstract|alias|align|asm|assert|auto|body|bool|break|byte|case|cast|catch|cdouble|cent|cfloat|char|class|const|continue|creal|dchar|debug|default|delegate|delete|deprecated|do|double|dstring|else|enum|export|extern|false|final|finally|float|for|foreach|foreach_reverse|function|goto|idouble|if|ifloat|immutable|import|inout|int|interface|invariant|ireal|lazy|long|macro|mixin|module|new|nothrow|null|out|override|package|pragma|private|protected|ptrdiff_t|public|pure|real|ref|return|scope|shared|short|size_t|static|string|struct|super|switch|synchronized|template|this|throw|true|try|typedef|typeid|typeof|ubyte|ucent|uint|ulong|union|unittest|ushort|version|void|volatile|wchar|while|with|wstring)\b/,
      number: [
        // The lookbehind and the negative look-ahead try to prevent bad highlighting of the .. operator
        // Hexadecimal numbers must be handled separately to avoid problems with exponent "e"
        /\b0x\.?[a-f\d_]+(?:(?!\.\.)\.[a-f\d_]*)?(?:p[+-]?[a-f\d_]+)?[ulfi]{0,4}/i,
        {
          pattern:
            /((?:\.\.)?)(?:\b0b\.?|\b|\.)\d[\d_]*(?:(?!\.\.)\.[\d_]*)?(?:e[+-]?\d[\d_]*)?[ulfi]{0,4}/i,
          lookbehind: true
        }
      ],
      operator:
        /\|[|=]?|&[&=]?|\+[+=]?|-[-=]?|\.?\.\.|=[>=]?|!(?:i[ns]\b|<>?=?|>=?|=)?|\bi[ns]\b|(?:<[<>]?|>>?>?|\^\^|[*\/%^~])=?/
    });
    Prism.languages.insertBefore('d', 'string', {
      // Characters
      // 'a', '\\', '\n', '\xFF', '\377', '\uFFFF', '\U0010FFFF', '\quot'
      char: /'(?:\\(?:\W|\w+)|[^\\])'/
    });
    Prism.languages.insertBefore('d', 'keyword', {
      property: /\B@\w*/
    });
    Prism.languages.insertBefore('d', 'function', {
      register: {
        // Iasm registers
        pattern:
          /\b(?:[ABCD][LHX]|E?(?:BP|DI|SI|SP)|[BS]PL|[ECSDGF]S|CR[0234]|[DS]IL|DR[012367]|E[ABCD]X|X?MM[0-7]|R(?:1[0-5]|[89])[BWD]?|R[ABCD]X|R[BS]P|R[DS]I|TR[3-7]|XMM(?:1[0-5]|[89])|YMM(?:1[0-5]|\d))\b|\bST(?:\([0-7]\)|\b)/,
        alias: 'variable'
      }
    });
  }

  // @ts-nocheck
  dart.displayName = 'dart';
  dart.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function dart(Prism) {
    Prism.register(clike)
    ;(function (Prism) {
      var keywords = [
        /\b(?:async|sync|yield)\*/,
        /\b(?:abstract|assert|async|await|break|case|catch|class|const|continue|covariant|default|deferred|do|dynamic|else|enum|export|extends|extension|external|factory|final|finally|for|get|hide|if|implements|import|in|interface|library|mixin|new|null|on|operator|part|rethrow|return|set|show|static|super|switch|sync|this|throw|try|typedef|var|void|while|with|yield)\b/
      ]; // Handles named imports, such as http.Client
      var packagePrefix = /(^|[^\w.])(?:[a-z]\w*\s*\.\s*)*(?:[A-Z]\w*\s*\.\s*)*/
        .source; // based on the dart naming conventions
      var className = {
        pattern: RegExp(packagePrefix + /[A-Z](?:[\d_A-Z]*[a-z]\w*)?\b/.source),
        lookbehind: true,
        inside: {
          namespace: {
            pattern: /^[a-z]\w*(?:\s*\.\s*[a-z]\w*)*(?:\s*\.)?/,
            inside: {
              punctuation: /\./
            }
          }
        }
      };
      Prism.languages.dart = Prism.languages.extend('clike', {
        'class-name': [
          className,
          {
            // variables and parameters
            // this to support class names (or generic parameters) which do not contain a lower case letter (also works for methods)
            pattern: RegExp(
              packagePrefix + /[A-Z]\w*(?=\s+\w+\s*[;,=()])/.source
            ),
            lookbehind: true,
            inside: className.inside
          }
        ],
        keyword: keywords,
        operator:
          /\bis!|\b(?:as|is)\b|\+\+|--|&&|\|\||<<=?|>>=?|~(?:\/=?)?|[+\-*\/%&^|=!<>]=?|\?/
      });
      Prism.languages.insertBefore('dart', 'string', {
        'string-literal': {
          pattern:
            /r?(?:("""|''')[\s\S]*?\1|(["'])(?:\\.|(?!\2)[^\\\r\n])*\2(?!\2))/,
          greedy: true,
          inside: {
            interpolation: {
              pattern:
                /((?:^|[^\\])(?:\\{2})*)\$(?:\w+|\{(?:[^{}]|\{[^{}]*\})*\})/,
              lookbehind: true,
              inside: {
                punctuation: /^\$\{?|\}$/,
                expression: {
                  pattern: /[\s\S]+/,
                  inside: Prism.languages.dart
                }
              }
            },
            string: /[\s\S]+/
          }
        },
        string: undefined
      });
      Prism.languages.insertBefore('dart', 'class-name', {
        metadata: {
          pattern: /@\w+/,
          alias: 'function'
        }
      });
      Prism.languages.insertBefore('dart', 'class-name', {
        generics: {
          pattern:
            /<(?:[\w\s,.&?]|<(?:[\w\s,.&?]|<(?:[\w\s,.&?]|<[\w\s,.&?]*>)*>)*>)*>/,
          inside: {
            'class-name': className,
            keyword: keywords,
            punctuation: /[<>(),.:]/,
            operator: /[?&|]/
          }
        }
      });
    })(Prism);
  }

  // @ts-nocheck
  dataweave.displayName = 'dataweave';
  dataweave.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function dataweave(Prism) {
  (function (Prism) {
      Prism.languages.dataweave = {
        url: /\b[A-Za-z]+:\/\/[\w/:.?=&-]+|\burn:[\w:.?=&-]+/,
        property: {
          pattern: /(?:\b\w+#)?(?:"(?:\\.|[^\\"\r\n])*"|\b\w+)(?=\s*[:@])/,
          greedy: true
        },
        string: {
          pattern: /(["'`])(?:\\[\s\S]|(?!\1)[^\\])*\1/,
          greedy: true
        },
        'mime-type':
          /\b(?:application|audio|image|multipart|text|video)\/[\w+-]+/,
        date: {
          pattern: /\|[\w:+-]+\|/,
          greedy: true
        },
        comment: [
          {
            pattern: /(^|[^\\])\/\*[\s\S]*?(?:\*\/|$)/,
            lookbehind: true,
            greedy: true
          },
          {
            pattern: /(^|[^\\:])\/\/.*/,
            lookbehind: true,
            greedy: true
          }
        ],
        regex: {
          pattern: /\/(?:[^\\\/\r\n]|\\[^\r\n])+\//,
          greedy: true
        },
        keyword:
          /\b(?:and|as|at|case|do|else|fun|if|input|is|match|not|ns|null|or|output|type|unless|update|using|var)\b/,
        function: /\b[A-Z_]\w*(?=\s*\()/i,
        number: /-?\b\d+(?:\.\d+)?(?:e[+-]?\d+)?\b/i,
        punctuation: /[{}[\];(),.:@]/,
        operator: /<<|>>|->|[<>~=]=?|!=|--?-?|\+\+?|!|\?/,
        boolean: /\b(?:false|true)\b/
      };
    })(Prism);
  }

  // @ts-nocheck
  dax.displayName = 'dax';
  dax.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function dax(Prism) {
    Prism.languages.dax = {
      comment: {
        pattern: /(^|[^\\])(?:\/\*[\s\S]*?\*\/|(?:--|\/\/).*)/,
        lookbehind: true
      },
      'data-field': {
        pattern:
          /'(?:[^']|'')*'(?!')(?:\[[ \w\xA0-\uFFFF]+\])?|\w+\[[ \w\xA0-\uFFFF]+\]/,
        alias: 'symbol'
      },
      measure: {
        pattern: /\[[ \w\xA0-\uFFFF]+\]/,
        alias: 'constant'
      },
      string: {
        pattern: /"(?:[^"]|"")*"(?!")/,
        greedy: true
      },
      function:
        /\b(?:ABS|ACOS|ACOSH|ACOT|ACOTH|ADDCOLUMNS|ADDMISSINGITEMS|ALL|ALLCROSSFILTERED|ALLEXCEPT|ALLNOBLANKROW|ALLSELECTED|AND|APPROXIMATEDISTINCTCOUNT|ASIN|ASINH|ATAN|ATANH|AVERAGE|AVERAGEA|AVERAGEX|BETA\.DIST|BETA\.INV|BLANK|CALCULATE|CALCULATETABLE|CALENDAR|CALENDARAUTO|CEILING|CHISQ\.DIST|CHISQ\.DIST\.RT|CHISQ\.INV|CHISQ\.INV\.RT|CLOSINGBALANCEMONTH|CLOSINGBALANCEQUARTER|CLOSINGBALANCEYEAR|COALESCE|COMBIN|COMBINA|COMBINEVALUES|CONCATENATE|CONCATENATEX|CONFIDENCE\.NORM|CONFIDENCE\.T|CONTAINS|CONTAINSROW|CONTAINSSTRING|CONTAINSSTRINGEXACT|CONVERT|COS|COSH|COT|COTH|COUNT|COUNTA|COUNTAX|COUNTBLANK|COUNTROWS|COUNTX|CROSSFILTER|CROSSJOIN|CURRENCY|CURRENTGROUP|CUSTOMDATA|DATATABLE|DATE|DATEADD|DATEDIFF|DATESBETWEEN|DATESINPERIOD|DATESMTD|DATESQTD|DATESYTD|DATEVALUE|DAY|DEGREES|DETAILROWS|DISTINCT|DISTINCTCOUNT|DISTINCTCOUNTNOBLANK|DIVIDE|EARLIER|EARLIEST|EDATE|ENDOFMONTH|ENDOFQUARTER|ENDOFYEAR|EOMONTH|ERROR|EVEN|EXACT|EXCEPT|EXP|EXPON\.DIST|FACT|FALSE|FILTER|FILTERS|FIND|FIRSTDATE|FIRSTNONBLANK|FIRSTNONBLANKVALUE|FIXED|FLOOR|FORMAT|GCD|GENERATE|GENERATEALL|GENERATESERIES|GEOMEAN|GEOMEANX|GROUPBY|HASONEFILTER|HASONEVALUE|HOUR|IF|IF\.EAGER|IFERROR|IGNORE|INT|INTERSECT|ISBLANK|ISCROSSFILTERED|ISEMPTY|ISERROR|ISEVEN|ISFILTERED|ISINSCOPE|ISLOGICAL|ISNONTEXT|ISNUMBER|ISO\.CEILING|ISODD|ISONORAFTER|ISSELECTEDMEASURE|ISSUBTOTAL|ISTEXT|KEEPFILTERS|KEYWORDMATCH|LASTDATE|LASTNONBLANK|LASTNONBLANKVALUE|LCM|LEFT|LEN|LN|LOG|LOG10|LOOKUPVALUE|LOWER|MAX|MAXA|MAXX|MEDIAN|MEDIANX|MID|MIN|MINA|MINUTE|MINX|MOD|MONTH|MROUND|NATURALINNERJOIN|NATURALLEFTOUTERJOIN|NEXTDAY|NEXTMONTH|NEXTQUARTER|NEXTYEAR|NONVISUAL|NORM\.DIST|NORM\.INV|NORM\.S\.DIST|NORM\.S\.INV|NOT|NOW|ODD|OPENINGBALANCEMONTH|OPENINGBALANCEQUARTER|OPENINGBALANCEYEAR|OR|PARALLELPERIOD|PATH|PATHCONTAINS|PATHITEM|PATHITEMREVERSE|PATHLENGTH|PERCENTILE\.EXC|PERCENTILE\.INC|PERCENTILEX\.EXC|PERCENTILEX\.INC|PERMUT|PI|POISSON\.DIST|POWER|PREVIOUSDAY|PREVIOUSMONTH|PREVIOUSQUARTER|PREVIOUSYEAR|PRODUCT|PRODUCTX|QUARTER|QUOTIENT|RADIANS|RAND|RANDBETWEEN|RANK\.EQ|RANKX|RELATED|RELATEDTABLE|REMOVEFILTERS|REPLACE|REPT|RIGHT|ROLLUP|ROLLUPADDISSUBTOTAL|ROLLUPGROUP|ROLLUPISSUBTOTAL|ROUND|ROUNDDOWN|ROUNDUP|ROW|SAMEPERIODLASTYEAR|SAMPLE|SEARCH|SECOND|SELECTCOLUMNS|SELECTEDMEASURE|SELECTEDMEASUREFORMATSTRING|SELECTEDMEASURENAME|SELECTEDVALUE|SIGN|SIN|SINH|SQRT|SQRTPI|STARTOFMONTH|STARTOFQUARTER|STARTOFYEAR|STDEV\.P|STDEV\.S|STDEVX\.P|STDEVX\.S|SUBSTITUTE|SUBSTITUTEWITHINDEX|SUM|SUMMARIZE|SUMMARIZECOLUMNS|SUMX|SWITCH|T\.DIST|T\.DIST\.2T|T\.DIST\.RT|T\.INV|T\.INV\.2T|TAN|TANH|TIME|TIMEVALUE|TODAY|TOPN|TOPNPERLEVEL|TOPNSKIP|TOTALMTD|TOTALQTD|TOTALYTD|TREATAS|TRIM|TRUE|TRUNC|UNICHAR|UNICODE|UNION|UPPER|USERELATIONSHIP|USERNAME|USEROBJECTID|USERPRINCIPALNAME|UTCNOW|UTCTODAY|VALUE|VALUES|VAR\.P|VAR\.S|VARX\.P|VARX\.S|WEEKDAY|WEEKNUM|XIRR|XNPV|YEAR|YEARFRAC)(?=\s*\()/i,
      keyword:
        /\b(?:DEFINE|EVALUATE|MEASURE|ORDER\s+BY|RETURN|VAR|START\s+AT|ASC|DESC)\b/i,
      boolean: {
        pattern: /\b(?:FALSE|NULL|TRUE)\b/i,
        alias: 'constant'
      },
      number: /\b\d+(?:\.\d*)?|\B\.\d+\b/,
      operator: /:=|[-+*\/=^]|&&?|\|\||<(?:=>?|<|>)?|>[>=]?|\b(?:IN|NOT)\b/i,
      punctuation: /[;\[\](){}`,.]/
    };
  }

  // @ts-nocheck
  dhall.displayName = 'dhall';
  dhall.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function dhall(Prism) {
    // ABNF grammar:
    // https://github.com/dhall-lang/dhall-lang/blob/master/standard/dhall.abnf
    Prism.languages.dhall = {
      // Multi-line comments can be nested. E.g. {- foo {- bar -} -}
      // The multi-line pattern is essentially this:
      //   \{-(?:[^-{]|-(?!\})|\{(?!-)|<SELF>)*-\}
      comment:
        /--.*|\{-(?:[^-{]|-(?!\})|\{(?!-)|\{-(?:[^-{]|-(?!\})|\{(?!-))*-\})*-\}/,
      string: {
        pattern: /"(?:[^"\\]|\\.)*"|''(?:[^']|'(?!')|'''|''\$\{)*''(?!'|\$)/,
        greedy: true,
        inside: {
          interpolation: {
            pattern: /\$\{[^{}]*\}/,
            inside: {
              expression: {
                pattern: /(^\$\{)[\s\S]+(?=\}$)/,
                lookbehind: true,
                alias: 'language-dhall',
                inside: null // see blow
              },
              punctuation: /\$\{|\}/
            }
          }
        }
      },
      label: {
        pattern: /`[^`]*`/,
        greedy: true
      },
      url: {
        // https://github.com/dhall-lang/dhall-lang/blob/5fde8ef1bead6fb4e999d3c1ffe7044cd019d63a/standard/dhall.abnf#L596
        pattern:
          /\bhttps?:\/\/[\w.:%!$&'*+;=@~-]+(?:\/[\w.:%!$&'*+;=@~-]*)*(?:\?[/?\w.:%!$&'*+;=@~-]*)?/,
        greedy: true
      },
      env: {
        // https://github.com/dhall-lang/dhall-lang/blob/5fde8ef1bead6fb4e999d3c1ffe7044cd019d63a/standard/dhall.abnf#L661
        pattern: /\benv:(?:(?!\d)\w+|"(?:[^"\\=]|\\.)*")/,
        greedy: true,
        inside: {
          function: /^env/,
          operator: /^:/,
          variable: /[\s\S]+/
        }
      },
      hash: {
        // https://github.com/dhall-lang/dhall-lang/blob/5fde8ef1bead6fb4e999d3c1ffe7044cd019d63a/standard/dhall.abnf#L725
        pattern: /\bsha256:[\da-fA-F]{64}\b/,
        inside: {
          function: /sha256/,
          operator: /:/,
          number: /[\da-fA-F]{64}/
        }
      },
      // https://github.com/dhall-lang/dhall-lang/blob/5fde8ef1bead6fb4e999d3c1ffe7044cd019d63a/standard/dhall.abnf#L359
      keyword:
        /\b(?:as|assert|else|forall|if|in|let|merge|missing|then|toMap|using|with)\b|\u2200/,
      builtin: /\b(?:None|Some)\b/,
      boolean: /\b(?:False|True)\b/,
      number:
        /\bNaN\b|-?\bInfinity\b|[+-]?\b(?:0x[\da-fA-F]+|\d+(?:\.\d+)?(?:e[+-]?\d+)?)\b/,
      operator:
        /\/\\|\/\/\\\\|&&|\|\||===|[!=]=|\/\/|->|\+\+|::|[+*#@=:?<>|\\\u2227\u2a53\u2261\u2afd\u03bb\u2192]/,
      punctuation: /\.\.|[{}\[\](),./]/,
      // we'll just assume that every capital word left is a type name
      'class-name': /\b[A-Z]\w*\b/
    };
    Prism.languages.dhall.string.inside.interpolation.inside.expression.inside =
      Prism.languages.dhall;
  }

  // @ts-nocheck
  django.displayName = 'django';
  django.aliases = ['jinja2'];

  /** @type {import('../core.js').Syntax} */
  function django(Prism) {
    Prism.register(markupTemplating)
    // Django/Jinja2 syntax definition for Prism.js <http://prismjs.com> syntax highlighter.
    // Mostly it works OK but can paint code incorrectly on complex html/template tag combinations.
    ;(function (Prism) {
      Prism.languages.django = {
        comment: /^\{#[\s\S]*?#\}$/,
        tag: {
          pattern: /(^\{%[+-]?\s*)\w+/,
          lookbehind: true,
          alias: 'keyword'
        },
        delimiter: {
          pattern: /^\{[{%][+-]?|[+-]?[}%]\}$/,
          alias: 'punctuation'
        },
        string: {
          pattern: /("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,
          greedy: true
        },
        filter: {
          pattern: /(\|)\w+/,
          lookbehind: true,
          alias: 'function'
        },
        test: {
          pattern: /(\bis\s+(?:not\s+)?)(?!not\b)\w+/,
          lookbehind: true,
          alias: 'function'
        },
        function: /\b[a-z_]\w+(?=\s*\()/i,
        keyword:
          /\b(?:and|as|by|else|for|if|import|in|is|loop|not|or|recursive|with|without)\b/,
        operator: /[-+%=]=?|!=|\*\*?=?|\/\/?=?|<[<=>]?|>[=>]?|[&|^~]/,
        number: /\b\d+(?:\.\d+)?\b/,
        boolean: /[Ff]alse|[Nn]one|[Tt]rue/,
        variable: /\b\w+\b/,
        punctuation: /[{}[\](),.:;]/
      };
      var pattern = /\{\{[\s\S]*?\}\}|\{%[\s\S]*?%\}|\{#[\s\S]*?#\}/g;
      var markupTemplating = Prism.languages['markup-templating'];
      Prism.hooks.add('before-tokenize', function (env) {
        markupTemplating.buildPlaceholders(env, 'django', pattern);
      });
      Prism.hooks.add('after-tokenize', function (env) {
        markupTemplating.tokenizePlaceholders(env, 'django');
      }); // Add an Jinja2 alias
      Prism.languages.jinja2 = Prism.languages.django;
      Prism.hooks.add('before-tokenize', function (env) {
        markupTemplating.buildPlaceholders(env, 'jinja2', pattern);
      });
      Prism.hooks.add('after-tokenize', function (env) {
        markupTemplating.tokenizePlaceholders(env, 'jinja2');
      });
    })(Prism);
  }

  // @ts-nocheck
  dnsZoneFile.displayName = 'dns-zone-file';
  dnsZoneFile.aliases = ['dns-zone'];

  /** @type {import('../core.js').Syntax} */
  function dnsZoneFile(Prism) {
    Prism.languages['dns-zone-file'] = {
      comment: /;.*/,
      string: {
        pattern: /"(?:\\.|[^"\\\r\n])*"/,
        greedy: true
      },
      variable: [
        {
          pattern: /(^\$ORIGIN[ \t]+)\S+/m,
          lookbehind: true
        },
        {
          pattern: /(^|\s)@(?=\s|$)/,
          lookbehind: true
        }
      ],
      keyword: /^\$(?:INCLUDE|ORIGIN|TTL)(?=\s|$)/m,
      class: {
        // https://tools.ietf.org/html/rfc1035#page-13
        pattern: /(^|\s)(?:CH|CS|HS|IN)(?=\s|$)/,
        lookbehind: true,
        alias: 'keyword'
      },
      type: {
        // https://en.wikipedia.org/wiki/List_of_DNS_record_types
        pattern:
          /(^|\s)(?:A|A6|AAAA|AFSDB|APL|ATMA|CAA|CDNSKEY|CDS|CERT|CNAME|DHCID|DLV|DNAME|DNSKEY|DS|EID|GID|GPOS|HINFO|HIP|IPSECKEY|ISDN|KEY|KX|LOC|MAILA|MAILB|MB|MD|MF|MG|MINFO|MR|MX|NAPTR|NB|NBSTAT|NIMLOC|NINFO|NS|NSAP|NSAP-PTR|NSEC|NSEC3|NSEC3PARAM|NULL|NXT|OPENPGPKEY|PTR|PX|RKEY|RP|RRSIG|RT|SIG|SINK|SMIMEA|SOA|SPF|SRV|SSHFP|TA|TKEY|TLSA|TSIG|TXT|UID|UINFO|UNSPEC|URI|WKS|X25)(?=\s|$)/,
        lookbehind: true,
        alias: 'keyword'
      },
      punctuation: /[()]/
    };
    Prism.languages['dns-zone'] = Prism.languages['dns-zone-file'];
  }

  // @ts-nocheck
  docker.displayName = 'docker';
  docker.aliases = ['dockerfile'];

  /** @type {import('../core.js').Syntax} */
  function docker(Prism) {
  (function (Prism) {
      // Many of the following regexes will contain negated lookaheads like `[ \t]+(?![ \t])`. This is a trick to ensure
      // that quantifiers behave *atomically*. Atomic quantifiers are necessary to prevent exponential backtracking.
      var spaceAfterBackSlash =
        /\\[\r\n](?:\s|\\[\r\n]|#.*(?!.))*(?![\s#]|\\[\r\n])/.source; // At least one space, comment, or line break
      var space = /(?:[ \t]+(?![ \t])(?:<SP_BS>)?|<SP_BS>)/.source.replace(
        /<SP_BS>/g,
        function () {
          return spaceAfterBackSlash
        }
      );
      var string =
        /"(?:[^"\\\r\n]|\\(?:\r\n|[\s\S]))*"|'(?:[^'\\\r\n]|\\(?:\r\n|[\s\S]))*'/
          .source;
      var option = /--[\w-]+=(?:<STR>|(?!["'])(?:[^\s\\]|\\.)+)/.source.replace(
        /<STR>/g,
        function () {
          return string
        }
      );
      var stringRule = {
        pattern: RegExp(string),
        greedy: true
      };
      var commentRule = {
        pattern: /(^[ \t]*)#.*/m,
        lookbehind: true,
        greedy: true
      };
      /**
       * @param {string} source
       * @param {string} flags
       * @returns {RegExp}
       */
      function re(source, flags) {
        source = source
          .replace(/<OPT>/g, function () {
            return option
          })
          .replace(/<SP>/g, function () {
            return space
          });
        return RegExp(source, flags)
      }
      Prism.languages.docker = {
        instruction: {
          pattern:
            /(^[ \t]*)(?:ADD|ARG|CMD|COPY|ENTRYPOINT|ENV|EXPOSE|FROM|HEALTHCHECK|LABEL|MAINTAINER|ONBUILD|RUN|SHELL|STOPSIGNAL|USER|VOLUME|WORKDIR)(?=\s)(?:\\.|[^\r\n\\])*(?:\\$(?:\s|#.*$)*(?![\s#])(?:\\.|[^\r\n\\])*)*/im,
          lookbehind: true,
          greedy: true,
          inside: {
            options: {
              pattern: re(
                /(^(?:ONBUILD<SP>)?\w+<SP>)<OPT>(?:<SP><OPT>)*/.source,
                'i'
              ),
              lookbehind: true,
              greedy: true,
              inside: {
                property: {
                  pattern: /(^|\s)--[\w-]+/,
                  lookbehind: true
                },
                string: [
                  stringRule,
                  {
                    pattern: /(=)(?!["'])(?:[^\s\\]|\\.)+/,
                    lookbehind: true
                  }
                ],
                operator: /\\$/m,
                punctuation: /=/
              }
            },
            keyword: [
              {
                // https://docs.docker.com/engine/reference/builder/#healthcheck
                pattern: re(
                  /(^(?:ONBUILD<SP>)?HEALTHCHECK<SP>(?:<OPT><SP>)*)(?:CMD|NONE)\b/
                    .source,
                  'i'
                ),
                lookbehind: true,
                greedy: true
              },
              {
                // https://docs.docker.com/engine/reference/builder/#from
                pattern: re(
                  /(^(?:ONBUILD<SP>)?FROM<SP>(?:<OPT><SP>)*(?!--)[^ \t\\]+<SP>)AS/
                    .source,
                  'i'
                ),
                lookbehind: true,
                greedy: true
              },
              {
                // https://docs.docker.com/engine/reference/builder/#onbuild
                pattern: re(/(^ONBUILD<SP>)\w+/.source, 'i'),
                lookbehind: true,
                greedy: true
              },
              {
                pattern: /^\w+/,
                greedy: true
              }
            ],
            comment: commentRule,
            string: stringRule,
            variable: /\$(?:\w+|\{[^{}"'\\]*\})/,
            operator: /\\$/m
          }
        },
        comment: commentRule
      };
      Prism.languages.dockerfile = Prism.languages.docker;
    })(Prism);
  }

  // @ts-nocheck
  dot.displayName = 'dot';
  dot.aliases = ['gv'];

  /** @type {import('../core.js').Syntax} */
  function dot(Prism) {
  (function (Prism) {
      var ID =
        '(?:' +
        [
          // an identifier
          /[a-zA-Z_\x80-\uFFFF][\w\x80-\uFFFF]*/.source, // a number
          /-?(?:\.\d+|\d+(?:\.\d*)?)/.source, // a double-quoted string
          /"[^"\\]*(?:\\[\s\S][^"\\]*)*"/.source, // HTML-like string
          /<(?:[^<>]|(?!<!--)<(?:[^<>"']|"[^"]*"|'[^']*')+>|<!--(?:[^-]|-(?!->))*-->)*>/
            .source
        ].join('|') +
        ')';
      var IDInside = {
        markup: {
          pattern: /(^<)[\s\S]+(?=>$)/,
          lookbehind: true,
          alias: ['language-markup', 'language-html', 'language-xml'],
          inside: Prism.languages.markup
        }
      };
      /**
       * @param {string} source
       * @param {string} flags
       * @returns {RegExp}
       */
      function withID(source, flags) {
        return RegExp(
          source.replace(/<ID>/g, function () {
            return ID
          }),
          flags
        )
      }
      Prism.languages.dot = {
        comment: {
          pattern: /\/\/.*|\/\*[\s\S]*?\*\/|^#.*/m,
          greedy: true
        },
        'graph-name': {
          pattern: withID(
            /(\b(?:digraph|graph|subgraph)[ \t\r\n]+)<ID>/.source,
            'i'
          ),
          lookbehind: true,
          greedy: true,
          alias: 'class-name',
          inside: IDInside
        },
        'attr-value': {
          pattern: withID(/(=[ \t\r\n]*)<ID>/.source),
          lookbehind: true,
          greedy: true,
          inside: IDInside
        },
        'attr-name': {
          pattern: withID(/([\[;, \t\r\n])<ID>(?=[ \t\r\n]*=)/.source),
          lookbehind: true,
          greedy: true,
          inside: IDInside
        },
        keyword: /\b(?:digraph|edge|graph|node|strict|subgraph)\b/i,
        'compass-point': {
          pattern: /(:[ \t\r\n]*)(?:[ewc_]|[ns][ew]?)(?![\w\x80-\uFFFF])/,
          lookbehind: true,
          alias: 'builtin'
        },
        node: {
          pattern: withID(/(^|[^-.\w\x80-\uFFFF\\])<ID>/.source),
          lookbehind: true,
          greedy: true,
          inside: IDInside
        },
        operator: /[=:]|-[->]/,
        punctuation: /[\[\]{};,]/
      };
      Prism.languages.gv = Prism.languages.dot;
    })(Prism);
  }

  // @ts-nocheck
  ebnf.displayName = 'ebnf';
  ebnf.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function ebnf(Prism) {
    Prism.languages.ebnf = {
      comment: /\(\*[\s\S]*?\*\)/,
      string: {
        pattern: /"[^"\r\n]*"|'[^'\r\n]*'/,
        greedy: true
      },
      special: {
        pattern: /\?[^?\r\n]*\?/,
        greedy: true,
        alias: 'class-name'
      },
      definition: {
        pattern: /^([\t ]*)[a-z]\w*(?:[ \t]+[a-z]\w*)*(?=\s*=)/im,
        lookbehind: true,
        alias: ['rule', 'keyword']
      },
      rule: /\b[a-z]\w*(?:[ \t]+[a-z]\w*)*\b/i,
      punctuation: /\([:/]|[:/]\)|[.,;()[\]{}]/,
      operator: /[-=|*/!]/
    };
  }

  // @ts-nocheck
  editorconfig.displayName = 'editorconfig';
  editorconfig.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function editorconfig(Prism) {
    Prism.languages.editorconfig = {
      // https://editorconfig-specification.readthedocs.io
      comment: /[;#].*/,
      section: {
        pattern: /(^[ \t]*)\[.+\]/m,
        lookbehind: true,
        alias: 'selector',
        inside: {
          regex: /\\\\[\[\]{},!?.*]/,
          // Escape special characters with '\\'
          operator: /[!?]|\.\.|\*{1,2}/,
          punctuation: /[\[\]{},]/
        }
      },
      key: {
        pattern: /(^[ \t]*)[^\s=]+(?=[ \t]*=)/m,
        lookbehind: true,
        alias: 'attr-name'
      },
      value: {
        pattern: /=.*/,
        alias: 'attr-value',
        inside: {
          punctuation: /^=/
        }
      }
    };
  }

  // @ts-nocheck
  eiffel.displayName = 'eiffel';
  eiffel.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function eiffel(Prism) {
    Prism.languages.eiffel = {
      comment: /--.*/,
      string: [
        // Aligned-verbatim-strings
        {
          pattern: /"([^[]*)\[[\s\S]*?\]\1"/,
          greedy: true
        }, // Non-aligned-verbatim-strings
        {
          pattern: /"([^{]*)\{[\s\S]*?\}\1"/,
          greedy: true
        }, // Single-line string
        {
          pattern: /"(?:%(?:(?!\n)\s)*\n\s*%|%\S|[^%"\r\n])*"/,
          greedy: true
        }
      ],
      // normal char | special char | char code
      char: /'(?:%.|[^%'\r\n])+'/,
      keyword:
        /\b(?:across|agent|alias|all|and|as|assign|attached|attribute|check|class|convert|create|Current|debug|deferred|detachable|do|else|elseif|end|ensure|expanded|export|external|feature|from|frozen|if|implies|inherit|inspect|invariant|like|local|loop|not|note|obsolete|old|once|or|Precursor|redefine|rename|require|rescue|Result|retry|select|separate|some|then|undefine|until|variant|Void|when|xor)\b/i,
      boolean: /\b(?:False|True)\b/i,
      // Convention: class-names are always all upper-case characters
      'class-name': /\b[A-Z][\dA-Z_]*\b/,
      number: [
        // hexa | octal | bin
        /\b0[xcb][\da-f](?:_*[\da-f])*\b/i, // Decimal
        /(?:\b\d(?:_*\d)*)?\.(?:(?:\d(?:_*\d)*)?e[+-]?)?\d(?:_*\d)*\b|\b\d(?:_*\d)*\b\.?/i
      ],
      punctuation: /:=|<<|>>|\(\||\|\)|->|\.(?=\w)|[{}[\];(),:?]/,
      operator: /\\\\|\|\.\.\||\.\.|\/[~\/=]?|[><]=?|[-+*^=~]/
    };
  }

  // @ts-nocheck
  ejs.displayName = 'ejs';
  ejs.aliases = ['eta'];

  /** @type {import('../core.js').Syntax} */
  function ejs(Prism) {
    Prism.register(javascript);
    Prism.register(markupTemplating)
    ;(function (Prism) {
      Prism.languages.ejs = {
        delimiter: {
          pattern: /^<%[-_=]?|[-_]?%>$/,
          alias: 'punctuation'
        },
        comment: /^#[\s\S]*/,
        'language-javascript': {
          pattern: /[\s\S]+/,
          inside: Prism.languages.javascript
        }
      };
      Prism.hooks.add('before-tokenize', function (env) {
        var ejsPattern = /<%(?!%)[\s\S]+?%>/g;
        Prism.languages['markup-templating'].buildPlaceholders(
          env,
          'ejs',
          ejsPattern
        );
      });
      Prism.hooks.add('after-tokenize', function (env) {
        Prism.languages['markup-templating'].tokenizePlaceholders(env, 'ejs');
      });
      Prism.languages.eta = Prism.languages.ejs;
    })(Prism);
  }

  // @ts-nocheck
  elixir.displayName = 'elixir';
  elixir.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function elixir(Prism) {
    Prism.languages.elixir = {
      doc: {
        pattern:
          /@(?:doc|moduledoc)\s+(?:("""|''')[\s\S]*?\1|("|')(?:\\(?:\r\n|[\s\S])|(?!\2)[^\\\r\n])*\2)/,
        inside: {
          attribute: /^@\w+/,
          string: /['"][\s\S]+/
        }
      },
      comment: {
        pattern: /#.*/,
        greedy: true
      },
      // ~r"""foo""" (multi-line), ~r'''foo''' (multi-line), ~r/foo/, ~r|foo|, ~r"foo", ~r'foo', ~r(foo), ~r[foo], ~r{foo}, ~r<foo>
      regex: {
        pattern:
          /~[rR](?:("""|''')(?:\\[\s\S]|(?!\1)[^\\])+\1|([\/|"'])(?:\\.|(?!\2)[^\\\r\n])+\2|\((?:\\.|[^\\)\r\n])+\)|\[(?:\\.|[^\\\]\r\n])+\]|\{(?:\\.|[^\\}\r\n])+\}|<(?:\\.|[^\\>\r\n])+>)[uismxfr]*/,
        greedy: true
      },
      string: [
        {
          // ~s"""foo""" (multi-line), ~s'''foo''' (multi-line), ~s/foo/, ~s|foo|, ~s"foo", ~s'foo', ~s(foo), ~s[foo], ~s{foo} (with interpolation care), ~s<foo>
          pattern:
            /~[cCsSwW](?:("""|''')(?:\\[\s\S]|(?!\1)[^\\])+\1|([\/|"'])(?:\\.|(?!\2)[^\\\r\n])+\2|\((?:\\.|[^\\)\r\n])+\)|\[(?:\\.|[^\\\]\r\n])+\]|\{(?:\\.|#\{[^}]+\}|#(?!\{)|[^#\\}\r\n])+\}|<(?:\\.|[^\\>\r\n])+>)[csa]?/,
          greedy: true,
          inside: {
            // See interpolation below
          }
        },
        {
          pattern: /("""|''')[\s\S]*?\1/,
          greedy: true,
          inside: {
            // See interpolation below
          }
        },
        {
          // Multi-line strings are allowed
          pattern: /("|')(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,
          greedy: true,
          inside: {
            // See interpolation below
          }
        }
      ],
      atom: {
        // Look-behind prevents bad highlighting of the :: operator
        pattern: /(^|[^:]):\w+/,
        lookbehind: true,
        alias: 'symbol'
      },
      module: {
        pattern: /\b[A-Z]\w*\b/,
        alias: 'class-name'
      },
      // Look-ahead prevents bad highlighting of the :: operator
      'attr-name': /\b\w+\??:(?!:)/,
      argument: {
        // Look-behind prevents bad highlighting of the && operator
        pattern: /(^|[^&])&\d+/,
        lookbehind: true,
        alias: 'variable'
      },
      attribute: {
        pattern: /@\w+/,
        alias: 'variable'
      },
      function: /\b[_a-zA-Z]\w*[?!]?(?:(?=\s*(?:\.\s*)?\()|(?=\/\d))/,
      number: /\b(?:0[box][a-f\d_]+|\d[\d_]*)(?:\.[\d_]+)?(?:e[+-]?[\d_]+)?\b/i,
      keyword:
        /\b(?:after|alias|and|case|catch|cond|def(?:callback|delegate|exception|impl|macro|module|n|np|p|protocol|struct)?|do|else|end|fn|for|if|import|not|or|quote|raise|require|rescue|try|unless|unquote|use|when)\b/,
      boolean: /\b(?:false|nil|true)\b/,
      operator: [
        /\bin\b|&&?|\|[|>]?|\\\\|::|\.\.\.?|\+\+?|-[->]?|<[-=>]|>=|!==?|\B!|=(?:==?|[>~])?|[*\/^]/,
        {
          // We don't want to match <<
          pattern: /([^<])<(?!<)/,
          lookbehind: true
        },
        {
          // We don't want to match >>
          pattern: /([^>])>(?!>)/,
          lookbehind: true
        }
      ],
      punctuation: /<<|>>|[.,%\[\]{}()]/
    };
    Prism.languages.elixir.string.forEach(function (o) {
      o.inside = {
        interpolation: {
          pattern: /#\{[^}]+\}/,
          inside: {
            delimiter: {
              pattern: /^#\{|\}$/,
              alias: 'punctuation'
            },
            rest: Prism.languages.elixir
          }
        }
      };
    });
  }

  // @ts-nocheck
  elm.displayName = 'elm';
  elm.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function elm(Prism) {
    Prism.languages.elm = {
      comment: /--.*|\{-[\s\S]*?-\}/,
      char: {
        pattern:
          /'(?:[^\\'\r\n]|\\(?:[abfnrtv\\']|\d+|x[0-9a-fA-F]+|u\{[0-9a-fA-F]+\}))'/,
        greedy: true
      },
      string: [
        {
          // Multiline strings are wrapped in triple ". Quotes may appear unescaped.
          pattern: /"""[\s\S]*?"""/,
          greedy: true
        },
        {
          pattern: /"(?:[^\\"\r\n]|\\.)*"/,
          greedy: true
        }
      ],
      'import-statement': {
        // The imported or hidden names are not included in this import
        // statement. This is because we want to highlight those exactly like
        // we do for the names in the program.
        pattern:
          /(^[\t ]*)import\s+[A-Z]\w*(?:\.[A-Z]\w*)*(?:\s+as\s+(?:[A-Z]\w*)(?:\.[A-Z]\w*)*)?(?:\s+exposing\s+)?/m,
        lookbehind: true,
        inside: {
          keyword: /\b(?:as|exposing|import)\b/
        }
      },
      keyword:
        /\b(?:alias|as|case|else|exposing|if|in|infixl|infixr|let|module|of|then|type)\b/,
      // These are builtin variables only. Constructors are highlighted later as a constant.
      builtin:
        /\b(?:abs|acos|always|asin|atan|atan2|ceiling|clamp|compare|cos|curry|degrees|e|flip|floor|fromPolar|identity|isInfinite|isNaN|logBase|max|min|negate|never|not|pi|radians|rem|round|sin|sqrt|tan|toFloat|toPolar|toString|truncate|turns|uncurry|xor)\b/,
      // decimal integers and floating point numbers | hexadecimal integers
      number: /\b(?:\d+(?:\.\d+)?(?:e[+-]?\d+)?|0x[0-9a-f]+)\b/i,
      // Most of this is needed because of the meaning of a single '.'.
      // If it stands alone freely, it is the function composition.
      // It may also be a separator between a module name and an identifier => no
      // operator. If it comes together with other special characters it is an
      // operator too.
      // Valid operator characters in 0.18: +-/*=.$<>:&|^?%#@~!
      // Ref: https://groups.google.com/forum/#!msg/elm-dev/0AHSnDdkSkQ/E0SVU70JEQAJ
      operator: /\s\.\s|[+\-/*=.$<>:&|^?%#@~!]{2,}|[+\-/*=$<>:&|^?%#@~!]/,
      // In Elm, nearly everything is a variable, do not highlight these.
      hvariable: /\b(?:[A-Z]\w*\.)*[a-z]\w*\b/,
      constant: /\b(?:[A-Z]\w*\.)*[A-Z]\w*\b/,
      punctuation: /[{}[\]|(),.:]/
    };
  }

  // @ts-nocheck
  etlua.displayName = 'etlua';
  etlua.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function etlua(Prism) {
    Prism.register(lua);
    Prism.register(markupTemplating)
    ;(function (Prism) {
      Prism.languages.etlua = {
        delimiter: {
          pattern: /^<%[-=]?|-?%>$/,
          alias: 'punctuation'
        },
        'language-lua': {
          pattern: /[\s\S]+/,
          inside: Prism.languages.lua
        }
      };
      Prism.hooks.add('before-tokenize', function (env) {
        var pattern = /<%[\s\S]+?%>/g;
        Prism.languages['markup-templating'].buildPlaceholders(
          env,
          'etlua',
          pattern
        );
      });
      Prism.hooks.add('after-tokenize', function (env) {
        Prism.languages['markup-templating'].tokenizePlaceholders(env, 'etlua');
      });
    })(Prism);
  }

  // @ts-nocheck
  erb.displayName = 'erb';
  erb.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function erb(Prism) {
    Prism.register(markupTemplating);
    Prism.register(ruby)
    ;(function (Prism) {
      Prism.languages.erb = {
        delimiter: {
          pattern: /^(\s*)<%=?|%>(?=\s*$)/,
          lookbehind: true,
          alias: 'punctuation'
        },
        ruby: {
          pattern: /\s*\S[\s\S]*/,
          alias: 'language-ruby',
          inside: Prism.languages.ruby
        }
      };
      Prism.hooks.add('before-tokenize', function (env) {
        var erbPattern =
          /<%=?(?:[^\r\n]|[\r\n](?!=begin)|[\r\n]=begin\s(?:[^\r\n]|[\r\n](?!=end))*[\r\n]=end)+?%>/g;
        Prism.languages['markup-templating'].buildPlaceholders(
          env,
          'erb',
          erbPattern
        );
      });
      Prism.hooks.add('after-tokenize', function (env) {
        Prism.languages['markup-templating'].tokenizePlaceholders(env, 'erb');
      });
    })(Prism);
  }

  // @ts-nocheck
  erlang.displayName = 'erlang';
  erlang.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function erlang(Prism) {
    Prism.languages.erlang = {
      comment: /%.+/,
      string: {
        pattern: /"(?:\\.|[^\\"\r\n])*"/,
        greedy: true
      },
      'quoted-function': {
        pattern: /'(?:\\.|[^\\'\r\n])+'(?=\()/,
        alias: 'function'
      },
      'quoted-atom': {
        pattern: /'(?:\\.|[^\\'\r\n])+'/,
        alias: 'atom'
      },
      boolean: /\b(?:false|true)\b/,
      keyword: /\b(?:after|case|catch|end|fun|if|of|receive|try|when)\b/,
      number: [
        /\$\\?./,
        /\b\d+#[a-z0-9]+/i,
        /(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?/i
      ],
      function: /\b[a-z][\w@]*(?=\()/,
      variable: {
        // Look-behind is used to prevent wrong highlighting of atoms containing "@"
        pattern: /(^|[^@])(?:\b|\?)[A-Z_][\w@]*/,
        lookbehind: true
      },
      operator: [
        /[=\/<>:]=|=[:\/]=|\+\+?|--?|[=*\/!]|\b(?:and|andalso|band|bnot|bor|bsl|bsr|bxor|div|not|or|orelse|rem|xor)\b/,
        {
          // We don't want to match <<
          pattern: /(^|[^<])<(?!<)/,
          lookbehind: true
        },
        {
          // We don't want to match >>
          pattern: /(^|[^>])>(?!>)/,
          lookbehind: true
        }
      ],
      atom: /\b[a-z][\w@]*/,
      punctuation: /[()[\]{}:;,.#|]|<<|>>/
    };
  }

  // @ts-nocheck
  excelFormula.displayName = 'excel-formula';
  excelFormula.aliases = ['xls', 'xlsx'];

  /** @type {import('../core.js').Syntax} */
  function excelFormula(Prism) {
    Prism.languages['excel-formula'] = {
      comment: {
        pattern: /(\bN\(\s*)"(?:[^"]|"")*"(?=\s*\))/i,
        lookbehind: true,
        greedy: true
      },
      string: {
        pattern: /"(?:[^"]|"")*"(?!")/,
        greedy: true
      },
      reference: {
        // https://www.ablebits.com/office-addins-blog/2015/12/08/excel-reference-another-sheet-workbook/
        // Sales!B2
        // 'Winter sales'!B2
        // [Sales.xlsx]Jan!B2:B5
        // D:\Reports\[Sales.xlsx]Jan!B2:B5
        // '[Sales.xlsx]Jan sales'!B2:B5
        // 'D:\Reports\[Sales.xlsx]Jan sales'!B2:B5
        pattern:
          /(?:'[^']*'|(?:[^\s()[\]{}<>*?"';,$&]*\[[^^\s()[\]{}<>*?"']+\])?\w+)!/,
        greedy: true,
        alias: 'string',
        inside: {
          operator: /!$/,
          punctuation: /'/,
          sheet: {
            pattern: /[^[\]]+$/,
            alias: 'function'
          },
          file: {
            pattern: /\[[^[\]]+\]$/,
            inside: {
              punctuation: /[[\]]/
            }
          },
          path: /[\s\S]+/
        }
      },
      'function-name': {
        pattern: /\b[A-Z]\w*(?=\()/i,
        alias: 'keyword'
      },
      range: {
        pattern:
          /\$?\b(?:[A-Z]+\$?\d+:\$?[A-Z]+\$?\d+|[A-Z]+:\$?[A-Z]+|\d+:\$?\d+)\b/i,
        alias: 'property',
        inside: {
          operator: /:/,
          cell: /\$?[A-Z]+\$?\d+/i,
          column: /\$?[A-Z]+/i,
          row: /\$?\d+/
        }
      },
      cell: {
        // Excel is case insensitive, so the string "foo1" could be either a variable or a cell.
        // To combat this, we match cells case insensitive, if the contain at least one "$", and case sensitive otherwise.
        pattern: /\b[A-Z]+\d+\b|\$[A-Za-z]+\$?\d+\b|\b[A-Za-z]+\$\d+\b/,
        alias: 'property'
      },
      number: /(?:\b\d+(?:\.\d+)?|\B\.\d+)(?:e[+-]?\d+)?\b/i,
      boolean: /\b(?:FALSE|TRUE)\b/i,
      operator: /[-+*/^%=&,]|<[=>]?|>=?/,
      punctuation: /[[\]();{}|]/
    };
    Prism.languages['xlsx'] = Prism.languages['xls'] =
      Prism.languages['excel-formula'];
  }

  // @ts-nocheck
  fsharp.displayName = 'fsharp';
  fsharp.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function fsharp(Prism) {
    Prism.register(clike);
    Prism.languages.fsharp = Prism.languages.extend('clike', {
      comment: [
        {
          pattern: /(^|[^\\])\(\*(?!\))[\s\S]*?\*\)/,
          lookbehind: true,
          greedy: true
        },
        {
          pattern: /(^|[^\\:])\/\/.*/,
          lookbehind: true,
          greedy: true
        }
      ],
      string: {
        pattern: /(?:"""[\s\S]*?"""|@"(?:""|[^"])*"|"(?:\\[\s\S]|[^\\"])*")B?/,
        greedy: true
      },
      'class-name': {
        pattern:
          /(\b(?:exception|inherit|interface|new|of|type)\s+|\w\s*:\s*|\s:\??>\s*)[.\w]+\b(?:\s*(?:->|\*)\s*[.\w]+\b)*(?!\s*[:.])/,
        lookbehind: true,
        inside: {
          operator: /->|\*/,
          punctuation: /\./
        }
      },
      keyword:
        /\b(?:let|return|use|yield)(?:!\B|\b)|\b(?:abstract|and|as|asr|assert|atomic|base|begin|break|checked|class|component|const|constraint|constructor|continue|default|delegate|do|done|downcast|downto|eager|elif|else|end|event|exception|extern|external|false|finally|fixed|for|fun|function|functor|global|if|in|include|inherit|inline|interface|internal|land|lazy|lor|lsl|lsr|lxor|match|member|method|mixin|mod|module|mutable|namespace|new|not|null|object|of|open|or|override|parallel|private|process|protected|public|pure|rec|sealed|select|sig|static|struct|tailcall|then|to|trait|true|try|type|upcast|val|virtual|void|volatile|when|while|with)\b/,
      number: [
        /\b0x[\da-fA-F]+(?:LF|lf|un)?\b/,
        /\b0b[01]+(?:uy|y)?\b/,
        /(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[fm]|e[+-]?\d+)?\b/i,
        /\b\d+(?:[IlLsy]|UL|u[lsy]?)?\b/
      ],
      operator:
        /([<>~&^])\1\1|([*.:<>&])\2|<-|->|[!=:]=|<?\|{1,3}>?|\??(?:<=|>=|<>|[-+*/%=<>])\??|[!?^&]|~[+~-]|:>|:\?>?/
    });
    Prism.languages.insertBefore('fsharp', 'keyword', {
      preprocessor: {
        pattern: /(^[\t ]*)#.*/m,
        lookbehind: true,
        alias: 'property',
        inside: {
          directive: {
            pattern: /(^#)\b(?:else|endif|if|light|line|nowarn)\b/,
            lookbehind: true,
            alias: 'keyword'
          }
        }
      }
    });
    Prism.languages.insertBefore('fsharp', 'punctuation', {
      'computation-expression': {
        pattern: /\b[_a-z]\w*(?=\s*\{)/i,
        alias: 'keyword'
      }
    });
    Prism.languages.insertBefore('fsharp', 'string', {
      annotation: {
        pattern: /\[<.+?>\]/,
        greedy: true,
        inside: {
          punctuation: /^\[<|>\]$/,
          'class-name': {
            pattern: /^\w+$|(^|;\s*)[A-Z]\w*(?=\()/,
            lookbehind: true
          },
          'annotation-content': {
            pattern: /[\s\S]+/,
            inside: Prism.languages.fsharp
          }
        }
      },
      char: {
        pattern:
          /'(?:[^\\']|\\(?:.|\d{3}|x[a-fA-F\d]{2}|u[a-fA-F\d]{4}|U[a-fA-F\d]{8}))'B?/,
        greedy: true
      }
    });
  }

  // @ts-nocheck
  factor.displayName = 'factor';
  factor.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function factor(Prism) {
  (function (Prism) {
      var comment_inside = {
        function:
          /\b(?:BUGS?|FIX(?:MES?)?|NOTES?|TODOS?|XX+|HACKS?|WARN(?:ING)?|\?{2,}|!{2,})\b/
      };
      var string_inside = {
        number: /\\[^\s']|%\w/
      };
      var factor = {
        comment: [
          {
            // ! single-line exclamation point comments with whitespace after/around the !
            pattern: /(^|\s)(?:! .*|!$)/,
            lookbehind: true,
            inside: comment_inside
          },
          /* from basis/multiline: */
          {
            // /* comment */, /* comment*/
            pattern: /(^|\s)\/\*\s[\s\S]*?\*\/(?=\s|$)/,
            lookbehind: true,
            greedy: true,
            inside: comment_inside
          },
          {
            // ![[ comment ]] , ![===[ comment]===]
            pattern: /(^|\s)!\[(={0,6})\[\s[\s\S]*?\]\2\](?=\s|$)/,
            lookbehind: true,
            greedy: true,
            inside: comment_inside
          }
        ],
        number: [
          {
            // basic base 10 integers 9, -9
            pattern: /(^|\s)[+-]?\d+(?=\s|$)/,
            lookbehind: true
          },
          {
            // base prefix integers 0b010 0o70 0xad 0d10 0XAD -0xa9
            pattern: /(^|\s)[+-]?0(?:b[01]+|o[0-7]+|d\d+|x[\dA-F]+)(?=\s|$)/i,
            lookbehind: true
          },
          {
            // fractional ratios 1/5 -1/5 and the literal float approximations 1/5. -1/5.
            pattern: /(^|\s)[+-]?\d+\/\d+\.?(?=\s|$)/,
            lookbehind: true
          },
          {
            // positive mixed numbers 23+1/5 +23+1/5
            pattern: /(^|\s)\+?\d+\+\d+\/\d+(?=\s|$)/,
            lookbehind: true
          },
          {
            // negative mixed numbers -23-1/5
            pattern: /(^|\s)-\d+-\d+\/\d+(?=\s|$)/,
            lookbehind: true
          },
          {
            // basic decimal floats -0.01 0. .0 .1 -.1 -1. -12.13 +12.13
            // and scientific notation with base 10 exponents 3e4 3e-4 .3e-4
            pattern:
              /(^|\s)[+-]?(?:\d*\.\d+|\d+\.\d*|\d+)(?:e[+-]?\d+)?(?=\s|$)/i,
            lookbehind: true
          },
          {
            // NAN literal syntax NAN: 80000deadbeef, NAN: a
            pattern: /(^|\s)NAN:\s+[\da-fA-F]+(?=\s|$)/,
            lookbehind: true
          },
          {
            /*
  base prefix floats 0x1.0p3 (8.0) 0b1.010p2 (5.0) 0x1.p1 0b1.11111111p11111...
  "The normalized hex form ±0x1.MMMMMMMMMMMMM[pP]±EEEE allows any floating-point number to be specified precisely.
  The values of MMMMMMMMMMMMM and EEEE map directly to the mantissa and exponent fields of the binary IEEE 754 representation."
  <https://docs.factorcode.org/content/article-syntax-floats.html>
  */
            pattern:
              /(^|\s)[+-]?0(?:b1\.[01]*|o1\.[0-7]*|d1\.\d*|x1\.[\dA-F]*)p\d+(?=\s|$)/i,
            lookbehind: true
          }
        ],
        // R/ regexp?\/\\/
        regexp: {
          pattern:
            /(^|\s)R\/\s(?:\\\S|[^\\/])*\/(?:[idmsr]*|[idmsr]+-[idmsr]+)(?=\s|$)/,
          lookbehind: true,
          alias: 'number',
          inside: {
            variable: /\\\S/,
            keyword: /[+?*\[\]^$(){}.|]/,
            operator: {
              pattern: /(\/)[idmsr]+(?:-[idmsr]+)?/,
              lookbehind: true
            }
          }
        },
        boolean: {
          pattern: /(^|\s)[tf](?=\s|$)/,
          lookbehind: true
        },
        // SBUF" asd", URL" ://...", P" /etc/"
        'custom-string': {
          pattern: /(^|\s)[A-Z0-9\-]+"\s(?:\\\S|[^"\\])*"/,
          lookbehind: true,
          greedy: true,
          alias: 'string',
          inside: {
            number: /\\\S|%\w|\//
          }
        },
        'multiline-string': [
          {
            // STRING: name \n content \n ; -> CONSTANT: name "content" (symbol)
            pattern: /(^|\s)STRING:\s+\S+(?:\n|\r\n).*(?:\n|\r\n)\s*;(?=\s|$)/,
            lookbehind: true,
            greedy: true,
            alias: 'string',
            inside: {
              number: string_inside.number,
              // trailing semicolon on its own line
              'semicolon-or-setlocal': {
                pattern: /([\r\n][ \t]*);(?=\s|$)/,
                lookbehind: true,
                alias: 'function'
              }
            }
          },
          {
            // HEREDOC: marker \n content \n marker ; -> "content" (immediate)
            pattern: /(^|\s)HEREDOC:\s+\S+(?:\n|\r\n).*(?:\n|\r\n)\s*\S+(?=\s|$)/,
            lookbehind: true,
            greedy: true,
            alias: 'string',
            inside: string_inside
          },
          {
            // [[ string ]], [==[ string]==]
            pattern: /(^|\s)\[(={0,6})\[\s[\s\S]*?\]\2\](?=\s|$)/,
            lookbehind: true,
            greedy: true,
            alias: 'string',
            inside: string_inside
          }
        ],
        'special-using': {
          pattern: /(^|\s)USING:(?:\s\S+)*(?=\s+;(?:\s|$))/,
          lookbehind: true,
          alias: 'function',
          inside: {
            // this is essentially a regex for vocab names, which i don't want to specify
            // but the USING: gets picked up as a vocab name
            string: {
              pattern: /(\s)[^:\s]+/,
              lookbehind: true
            }
          }
        },
        /* this description of stack effect literal syntax is not complete and not as specific as theoretically possible
  trying to do better is more work and regex-computation-time than it's worth though.
  - we'd like to have the "delimiter" parts of the stack effect [ (, --, and ) ] be a different (less-important or comment-like) colour to the stack effect contents
  - we'd like if nested stack effects were treated as such rather than just appearing flat (with `inside`)
  - we'd like if the following variable name conventions were recognised specifically:
  special row variables = ..a b..
  type and stack effect annotations end with a colon = ( quot: ( a: ( -- ) -- b ) -- x ), ( x: number -- )
  word throws unconditional error = *
  any other word-like variable name = a ? q' etc
  https://docs.factorcode.org/content/article-effects.html
  these are pretty complicated to highlight properly without a real parser, and therefore out of scope
  the old pattern, which may be later useful, was: (^|\s)(?:call|execute|eval)?\((?:\s+[^"\r\n\t ]\S*)*?\s+--(?:\s+[^"\n\t ]\S*)*?\s+\)(?=\s|$)
  */
        // current solution is not great
        'stack-effect-delimiter': [
          {
            // opening parenthesis
            pattern: /(^|\s)(?:call|eval|execute)?\((?=\s)/,
            lookbehind: true,
            alias: 'operator'
          },
          {
            // middle --
            pattern: /(\s)--(?=\s)/,
            lookbehind: true,
            alias: 'operator'
          },
          {
            // closing parenthesis
            pattern: /(\s)\)(?=\s|$)/,
            lookbehind: true,
            alias: 'operator'
          }
        ],
        combinators: {
          pattern: null,
          lookbehind: true,
          alias: 'keyword'
        },
        'kernel-builtin': {
          pattern: null,
          lookbehind: true,
          alias: 'variable'
        },
        'sequences-builtin': {
          pattern: null,
          lookbehind: true,
          alias: 'variable'
        },
        'math-builtin': {
          pattern: null,
          lookbehind: true,
          alias: 'variable'
        },
        'constructor-word': {
          // <array> but not <=>
          pattern: /(^|\s)<(?!=+>|-+>)\S+>(?=\s|$)/,
          lookbehind: true,
          alias: 'keyword'
        },
        'other-builtin-syntax': {
          pattern: null,
          lookbehind: true,
          alias: 'operator'
        },
        /*
  full list of supported word naming conventions: (the convention appears outside of the [brackets])
  set-[x]
  change-[x]
  with-[x]
  new-[x]
  >[string]
  [base]>
  [string]>[number]
  +[symbol]+
  [boolean-word]?
  ?[of]
  [slot-reader]>>
  >>[slot-setter]
  [slot-writer]<<
  ([implementation-detail])
  [mutater]!
  [variant]*
  [prettyprint].
  $[help-markup]
  <constructors>, SYNTAX:, etc are supported by their own patterns.
  `with` and `new` from `kernel` are their own builtins.
  see <https://docs.factorcode.org/content/article-conventions.html>
  */
        'conventionally-named-word': {
          pattern:
            /(^|\s)(?!")(?:(?:change|new|set|with)-\S+|\$\S+|>[^>\s]+|[^:>\s]+>|[^>\s]+>[^>\s]+|\+[^+\s]+\+|[^?\s]+\?|\?[^?\s]+|[^>\s]+>>|>>[^>\s]+|[^<\s]+<<|\([^()\s]+\)|[^!\s]+!|[^*\s]\S*\*|[^.\s]\S*\.)(?=\s|$)/,
          lookbehind: true,
          alias: 'keyword'
        },
        'colon-syntax': {
          pattern: /(^|\s)(?:[A-Z0-9\-]+#?)?:{1,2}\s+(?:;\S+|(?!;)\S+)(?=\s|$)/,
          lookbehind: true,
          greedy: true,
          alias: 'function'
        },
        'semicolon-or-setlocal': {
          pattern: /(\s)(?:;|:>)(?=\s|$)/,
          lookbehind: true,
          alias: 'function'
        },
        // do not highlight leading } or trailing X{ at the begin/end of the file as it's invalid syntax
        'curly-brace-literal-delimiter': [
          {
            // opening
            pattern: /(^|\s)[a-z]*\{(?=\s)/i,
            lookbehind: true,
            alias: 'operator'
          },
          {
            // closing
            pattern: /(\s)\}(?=\s|$)/,
            lookbehind: true,
            alias: 'operator'
          }
        ],
        // do not highlight leading ] or trailing [ at the begin/end of the file as it's invalid syntax
        'quotation-delimiter': [
          {
            // opening
            pattern: /(^|\s)\[(?=\s)/,
            lookbehind: true,
            alias: 'operator'
          },
          {
            // closing
            pattern: /(\s)\](?=\s|$)/,
            lookbehind: true,
            alias: 'operator'
          }
        ],
        'normal-word': {
          pattern: /(^|\s)[^"\s]\S*(?=\s|$)/,
          lookbehind: true
        },
        /*
  basic first-class string "a"
  with escaped double-quote "a\""
  escaped backslash "\\"
  and general escapes since Factor has so many "\N"
  syntax that works in the reference implementation that isn't fully
  supported because it's an implementation detail:
  "string 1""string 2" -> 2 strings (works anyway)
  "string"5 -> string, 5
  "string"[ ] -> string, quotation
  { "a"} -> array<string>
  the rest of those examples all properly recognise the string, but not
  the other object (number, quotation, etc)
  this is fine for a regex-only implementation.
  */
        string: {
          pattern: /"(?:\\\S|[^"\\])*"/,
          greedy: true,
          inside: string_inside
        }
      };
      var escape = function (str) {
        return (str + '').replace(/([.?*+\^$\[\]\\(){}|\-])/g, '\\$1')
      };
      var arrToWordsRegExp = function (arr) {
        return new RegExp('(^|\\s)(?:' + arr.map(escape).join('|') + ')(?=\\s|$)')
      };
      var builtins = {
        'kernel-builtin': [
          'or',
          '2nipd',
          '4drop',
          'tuck',
          'wrapper',
          'nip',
          'wrapper?',
          'callstack>array',
          'die',
          'dupd',
          'callstack',
          'callstack?',
          '3dup',
          'hashcode',
          'pick',
          '4nip',
          'build',
          '>boolean',
          'nipd',
          'clone',
          '5nip',
          'eq?',
          '?',
          '=',
          'swapd',
          '2over',
          'clear',
          '2dup',
          'get-retainstack',
          'not',
          'tuple?',
          'dup',
          '3nipd',
          'call',
          '-rotd',
          'object',
          'drop',
          'assert=',
          'assert?',
          '-rot',
          'execute',
          'boa',
          'get-callstack',
          'curried?',
          '3drop',
          'pickd',
          'overd',
          'over',
          'roll',
          '3nip',
          'swap',
          'and',
          '2nip',
          'rotd',
          'throw',
          '(clone)',
          'hashcode*',
          'spin',
          'reach',
          '4dup',
          'equal?',
          'get-datastack',
          'assert',
          '2drop',
          '<wrapper>',
          'boolean?',
          'identity-hashcode',
          'identity-tuple?',
          'null',
          'composed?',
          'new',
          '5drop',
          'rot',
          '-roll',
          'xor',
          'identity-tuple',
          'boolean'
        ],
        'other-builtin-syntax': [
          // syntax
          '=======',
          'recursive',
          'flushable',
          '>>',
          '<<<<<<',
          'M\\',
          'B',
          'PRIVATE>',
          '\\',
          '======',
          'final',
          'inline',
          'delimiter',
          'deprecated',
          '<PRIVATE',
          '>>>>>>',
          '<<<<<<<',
          'parse-complex',
          'malformed-complex',
          'read-only',
          '>>>>>>>',
          'call-next-method',
          '<<',
          'foldable', // literals
          '$',
          '$[',
          '${'
        ],
        'sequences-builtin': [
          'member-eq?',
          'mismatch',
          'append',
          'assert-sequence=',
          'longer',
          'repetition',
          'clone-like',
          '3sequence',
          'assert-sequence?',
          'last-index-from',
          'reversed',
          'index-from',
          'cut*',
          'pad-tail',
          'join-as',
          'remove-eq!',
          'concat-as',
          'but-last',
          'snip',
          'nths',
          'nth',
          'sequence',
          'longest',
          'slice?',
          '<slice>',
          'remove-nth',
          'tail-slice',
          'empty?',
          'tail*',
          'member?',
          'virtual-sequence?',
          'set-length',
          'drop-prefix',
          'iota',
          'unclip',
          'bounds-error?',
          'unclip-last-slice',
          'non-negative-integer-expected',
          'non-negative-integer-expected?',
          'midpoint@',
          'longer?',
          '?set-nth',
          '?first',
          'rest-slice',
          'prepend-as',
          'prepend',
          'fourth',
          'sift',
          'subseq-start',
          'new-sequence',
          '?last',
          'like',
          'first4',
          '1sequence',
          'reverse',
          'slice',
          'virtual@',
          'repetition?',
          'set-last',
          'index',
          '4sequence',
          'max-length',
          'set-second',
          'immutable-sequence',
          'first2',
          'first3',
          'supremum',
          'unclip-slice',
          'suffix!',
          'insert-nth',
          'tail',
          '3append',
          'short',
          'suffix',
          'concat',
          'flip',
          'immutable?',
          'reverse!',
          '2sequence',
          'sum',
          'delete-all',
          'indices',
          'snip-slice',
          '<iota>',
          'check-slice',
          'sequence?',
          'head',
          'append-as',
          'halves',
          'sequence=',
          'collapse-slice',
          '?second',
          'slice-error?',
          'product',
          'bounds-check?',
          'bounds-check',
          'immutable',
          'virtual-exemplar',
          'harvest',
          'remove',
          'pad-head',
          'last',
          'set-fourth',
          'cartesian-product',
          'remove-eq',
          'shorten',
          'shorter',
          'reversed?',
          'shorter?',
          'shortest',
          'head-slice',
          'pop*',
          'tail-slice*',
          'but-last-slice',
          'iota?',
          'append!',
          'cut-slice',
          'new-resizable',
          'head-slice*',
          'sequence-hashcode',
          'pop',
          'set-nth',
          '?nth',
          'second',
          'join',
          'immutable-sequence?',
          '<reversed>',
          '3append-as',
          'virtual-sequence',
          'subseq?',
          'remove-nth!',
          'length',
          'last-index',
          'lengthen',
          'assert-sequence',
          'copy',
          'move',
          'third',
          'first',
          'tail?',
          'set-first',
          'prefix',
          'bounds-error',
          '<repetition>',
          'exchange',
          'surround',
          'cut',
          'min-length',
          'set-third',
          'push-all',
          'head?',
          'subseq-start-from',
          'delete-slice',
          'rest',
          'sum-lengths',
          'head*',
          'infimum',
          'remove!',
          'glue',
          'slice-error',
          'subseq',
          'push',
          'replace-slice',
          'subseq-as',
          'unclip-last'
        ],
        'math-builtin': [
          'number=',
          'next-power-of-2',
          '?1+',
          'fp-special?',
          'imaginary-part',
          'float>bits',
          'number?',
          'fp-infinity?',
          'bignum?',
          'fp-snan?',
          'denominator',
          'gcd',
          '*',
          '+',
          'fp-bitwise=',
          '-',
          'u>=',
          '/',
          '>=',
          'bitand',
          'power-of-2?',
          'log2-expects-positive',
          'neg?',
          '<',
          'log2',
          '>',
          'integer?',
          'number',
          'bits>double',
          '2/',
          'zero?',
          'bits>float',
          'float?',
          'shift',
          'ratio?',
          'rect>',
          'even?',
          'ratio',
          'fp-sign',
          'bitnot',
          '>fixnum',
          'complex?',
          '/i',
          'integer>fixnum',
          '/f',
          'sgn',
          '>bignum',
          'next-float',
          'u<',
          'u>',
          'mod',
          'recip',
          'rational',
          '>float',
          '2^',
          'integer',
          'fixnum?',
          'neg',
          'fixnum',
          'sq',
          'bignum',
          '>rect',
          'bit?',
          'fp-qnan?',
          'simple-gcd',
          'complex',
          '<fp-nan>',
          'real',
          '>fraction',
          'double>bits',
          'bitor',
          'rem',
          'fp-nan-payload',
          'real-part',
          'log2-expects-positive?',
          'prev-float',
          'align',
          'unordered?',
          'float',
          'fp-nan?',
          'abs',
          'bitxor',
          'integer>fixnum-strict',
          'u<=',
          'odd?',
          '<=',
          '/mod',
          '>integer',
          'real?',
          'rational?',
          'numerator'
        ] // that's all for now
      };
      Object.keys(builtins).forEach(function (k) {
        factor[k].pattern = arrToWordsRegExp(builtins[k]);
      });
      var combinators = [
        // kernel
        '2bi',
        'while',
        '2tri',
        'bi*',
        '4dip',
        'both?',
        'same?',
        'tri@',
        'curry',
        'prepose',
        '3bi',
        '?if',
        'tri*',
        '2keep',
        '3keep',
        'curried',
        '2keepd',
        'when',
        '2bi*',
        '2tri*',
        '4keep',
        'bi@',
        'keepdd',
        'do',
        'unless*',
        'tri-curry',
        'if*',
        'loop',
        'bi-curry*',
        'when*',
        '2bi@',
        '2tri@',
        'with',
        '2with',
        'either?',
        'bi',
        'until',
        '3dip',
        '3curry',
        'tri-curry*',
        'tri-curry@',
        'bi-curry',
        'keepd',
        'compose',
        '2dip',
        'if',
        '3tri',
        'unless',
        'tuple',
        'keep',
        '2curry',
        'tri',
        'most',
        'while*',
        'dip',
        'composed',
        'bi-curry@', // sequences
        'find-last-from',
        'trim-head-slice',
        'map-as',
        'each-from',
        'none?',
        'trim-tail',
        'partition',
        'if-empty',
        'accumulate*',
        'reject!',
        'find-from',
        'accumulate-as',
        'collector-for-as',
        'reject',
        'map',
        'map-sum',
        'accumulate!',
        '2each-from',
        'follow',
        'supremum-by',
        'map!',
        'unless-empty',
        'collector',
        'padding',
        'reduce-index',
        'replicate-as',
        'infimum-by',
        'trim-tail-slice',
        'count',
        'find-index',
        'filter',
        'accumulate*!',
        'reject-as',
        'map-integers',
        'map-find',
        'reduce',
        'selector',
        'interleave',
        '2map',
        'filter-as',
        'binary-reduce',
        'map-index-as',
        'find',
        'produce',
        'filter!',
        'replicate',
        'cartesian-map',
        'cartesian-each',
        'find-index-from',
        'map-find-last',
        '3map-as',
        '3map',
        'find-last',
        'selector-as',
        '2map-as',
        '2map-reduce',
        'accumulate',
        'each',
        'each-index',
        'accumulate*-as',
        'when-empty',
        'all?',
        'collector-as',
        'push-either',
        'new-like',
        'collector-for',
        '2selector',
        'push-if',
        '2all?',
        'map-reduce',
        '3each',
        'any?',
        'trim-slice',
        '2reduce',
        'change-nth',
        'produce-as',
        '2each',
        'trim',
        'trim-head',
        'cartesian-find',
        'map-index', // math
        'if-zero',
        'each-integer',
        'unless-zero',
        '(find-integer)',
        'when-zero',
        'find-last-integer',
        '(all-integers?)',
        'times',
        '(each-integer)',
        'find-integer',
        'all-integers?', // math.combinators
        'unless-negative',
        'if-positive',
        'when-positive',
        'when-negative',
        'unless-positive',
        'if-negative', // combinators
        'case',
        '2cleave',
        'cond>quot',
        'case>quot',
        '3cleave',
        'wrong-values',
        'to-fixed-point',
        'alist>quot',
        'cond',
        'cleave',
        'call-effect',
        'recursive-hashcode',
        'spread',
        'deep-spread>quot', // combinators.short-circuit
        '2||',
        '0||',
        'n||',
        '0&&',
        '2&&',
        '3||',
        '1||',
        '1&&',
        'n&&',
        '3&&', // combinators.smart
        'smart-unless*',
        'keep-inputs',
        'reduce-outputs',
        'smart-when*',
        'cleave>array',
        'smart-with',
        'smart-apply',
        'smart-if',
        'inputs/outputs',
        'output>sequence-n',
        'map-outputs',
        'map-reduce-outputs',
        'dropping',
        'output>array',
        'smart-map-reduce',
        'smart-2map-reduce',
        'output>array-n',
        'nullary',
        'input<sequence',
        'append-outputs',
        'drop-inputs',
        'inputs',
        'smart-2reduce',
        'drop-outputs',
        'smart-reduce',
        'preserving',
        'smart-when',
        'outputs',
        'append-outputs-as',
        'smart-unless',
        'smart-if*',
        'sum-outputs',
        'input<sequence-unsafe',
        'output>sequence' // tafn
      ];
      factor.combinators.pattern = arrToWordsRegExp(combinators);
      Prism.languages.factor = factor;
    })(Prism);
  }

  // @ts-nocheck
  $false.displayName = 'false';
  $false.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function $false(Prism) {
  (function (Prism) {
      /**
       * Based on the manual by Wouter van Oortmerssen.
       *
       * @see {@link https://github.com/PrismJS/prism/issues/2801#issue-829717504}
       */
      Prism.languages['false'] = {
        comment: {
          pattern: /\{[^}]*\}/
        },
        string: {
          pattern: /"[^"]*"/,
          greedy: true
        },
        'character-code': {
          pattern: /'(?:[^\r]|\r\n?)/,
          alias: 'number'
        },
        'assembler-code': {
          pattern: /\d+`/,
          alias: 'important'
        },
        number: /\d+/,
        operator: /[-!#$%&'*+,./:;=>?@\\^_`|~ßø]/,
        punctuation: /\[|\]/,
        variable: /[a-z]/,
        'non-standard': {
          pattern: /[()<BDO®]/,
          alias: 'bold'
        }
      };
    })(Prism);
  }

  // @ts-nocheck
  firestoreSecurityRules.displayName = 'firestore-security-rules';
  firestoreSecurityRules.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function firestoreSecurityRules(Prism) {
    Prism.register(clike);
    Prism.languages['firestore-security-rules'] = Prism.languages.extend(
      'clike',
      {
        comment: /\/\/.*/,
        keyword:
          /\b(?:allow|function|if|match|null|return|rules_version|service)\b/,
        operator: /&&|\|\||[<>!=]=?|[-+*/%]|\b(?:in|is)\b/
      }
    );
    delete Prism.languages['firestore-security-rules']['class-name'];
    Prism.languages.insertBefore('firestore-security-rules', 'keyword', {
      path: {
        pattern:
          /(^|[\s(),])(?:\/(?:[\w\xA0-\uFFFF]+|\{[\w\xA0-\uFFFF]+(?:=\*\*)?\}|\$\([\w\xA0-\uFFFF.]+\)))+/,
        lookbehind: true,
        greedy: true,
        inside: {
          variable: {
            pattern: /\{[\w\xA0-\uFFFF]+(?:=\*\*)?\}|\$\([\w\xA0-\uFFFF.]+\)/,
            inside: {
              operator: /=/,
              keyword: /\*\*/,
              punctuation: /[.$(){}]/
            }
          },
          punctuation: /\//
        }
      },
      method: {
        // to make the pattern shorter, the actual method names are omitted
        pattern: /(\ballow\s+)[a-z]+(?:\s*,\s*[a-z]+)*(?=\s*[:;])/,
        lookbehind: true,
        alias: 'builtin',
        inside: {
          punctuation: /,/
        }
      }
    });
  }

  // @ts-nocheck
  flow.displayName = 'flow';
  flow.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function flow(Prism) {
    Prism.register(javascript)
    ;(function (Prism) {
      Prism.languages.flow = Prism.languages.extend('javascript', {});
      Prism.languages.insertBefore('flow', 'keyword', {
        type: [
          {
            pattern:
              /\b(?:[Bb]oolean|Function|[Nn]umber|[Ss]tring|any|mixed|null|void)\b/,
            alias: 'tag'
          }
        ]
      });
      Prism.languages.flow['function-variable'].pattern =
        /(?!\s)[_$a-z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*=\s*(?:function\b|(?:\([^()]*\)(?:\s*:\s*\w+)?|(?!\s)[_$a-z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*)\s*=>))/i;
      delete Prism.languages.flow['parameter'];
      Prism.languages.insertBefore('flow', 'operator', {
        'flow-punctuation': {
          pattern: /\{\||\|\}/,
          alias: 'punctuation'
        }
      });
      if (!Array.isArray(Prism.languages.flow.keyword)) {
        Prism.languages.flow.keyword = [Prism.languages.flow.keyword];
      }
      Prism.languages.flow.keyword.unshift(
        {
          pattern: /(^|[^$]\b)(?:Class|declare|opaque|type)\b(?!\$)/,
          lookbehind: true
        },
        {
          pattern:
            /(^|[^$]\B)\$(?:Diff|Enum|Exact|Keys|ObjMap|PropertyType|Record|Shape|Subtype|Supertype|await)\b(?!\$)/,
          lookbehind: true
        }
      );
    })(Prism);
  }

  // @ts-nocheck
  fortran.displayName = 'fortran';
  fortran.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function fortran(Prism) {
    Prism.languages.fortran = {
      'quoted-number': {
        pattern: /[BOZ](['"])[A-F0-9]+\1/i,
        alias: 'number'
      },
      string: {
        pattern:
          /(?:\b\w+_)?(['"])(?:\1\1|&(?:\r\n?|\n)(?:[ \t]*!.*(?:\r\n?|\n)|(?![ \t]*!))|(?!\1).)*(?:\1|&)/,
        inside: {
          comment: {
            pattern: /(&(?:\r\n?|\n)\s*)!.*/,
            lookbehind: true
          }
        }
      },
      comment: {
        pattern: /!.*/,
        greedy: true
      },
      boolean: /\.(?:FALSE|TRUE)\.(?:_\w+)?/i,
      number: /(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[ED][+-]?\d+)?(?:_\w+)?/i,
      keyword: [
        // Types
        /\b(?:CHARACTER|COMPLEX|DOUBLE ?PRECISION|INTEGER|LOGICAL|REAL)\b/i, // END statements
        /\b(?:END ?)?(?:BLOCK ?DATA|DO|FILE|FORALL|FUNCTION|IF|INTERFACE|MODULE(?! PROCEDURE)|PROGRAM|SELECT|SUBROUTINE|TYPE|WHERE)\b/i, // Statements
        /\b(?:ALLOCATABLE|ALLOCATE|BACKSPACE|CALL|CASE|CLOSE|COMMON|CONTAINS|CONTINUE|CYCLE|DATA|DEALLOCATE|DIMENSION|DO|END|EQUIVALENCE|EXIT|EXTERNAL|FORMAT|GO ?TO|IMPLICIT(?: NONE)?|INQUIRE|INTENT|INTRINSIC|MODULE PROCEDURE|NAMELIST|NULLIFY|OPEN|OPTIONAL|PARAMETER|POINTER|PRINT|PRIVATE|PUBLIC|READ|RETURN|REWIND|SAVE|SELECT|STOP|TARGET|WHILE|WRITE)\b/i, // Others
        /\b(?:ASSIGNMENT|DEFAULT|ELEMENTAL|ELSE|ELSEIF|ELSEWHERE|ENTRY|IN|INCLUDE|INOUT|KIND|NULL|ONLY|OPERATOR|OUT|PURE|RECURSIVE|RESULT|SEQUENCE|STAT|THEN|USE)\b/i
      ],
      operator: [
        /\*\*|\/\/|=>|[=\/]=|[<>]=?|::|[+\-*=%]|\.[A-Z]+\./i,
        {
          // Use lookbehind to prevent confusion with (/ /)
          pattern: /(^|(?!\().)\/(?!\))/,
          lookbehind: true
        }
      ],
      punctuation: /\(\/|\/\)|[(),;:&]/
    };
  }

  // @ts-nocheck
  ftl.displayName = 'ftl';
  ftl.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function ftl(Prism) {
    Prism.register(markupTemplating)
    ;(function (Prism) {
      // https://freemarker.apache.org/docs/dgui_template_exp.html
      // FTL expression with 4 levels of nesting supported
      var FTL_EXPR =
        /[^<()"']|\((?:<expr>)*\)|<(?!#--)|<#--(?:[^-]|-(?!->))*-->|"(?:[^\\"]|\\.)*"|'(?:[^\\']|\\.)*'/
          .source;
      for (var i = 0; i < 2; i++) {
        FTL_EXPR = FTL_EXPR.replace(/<expr>/g, function () {
          return FTL_EXPR
        });
      }
      FTL_EXPR = FTL_EXPR.replace(/<expr>/g, /[^\s\S]/.source);
      var ftl = {
        comment: /<#--[\s\S]*?-->/,
        string: [
          {
            // raw string
            pattern: /\br("|')(?:(?!\1)[^\\]|\\.)*\1/,
            greedy: true
          },
          {
            pattern: RegExp(
              /("|')(?:(?!\1|\$\{)[^\\]|\\.|\$\{(?:(?!\})(?:<expr>))*\})*\1/.source.replace(
                /<expr>/g,
                function () {
                  return FTL_EXPR
                }
              )
            ),
            greedy: true,
            inside: {
              interpolation: {
                pattern: RegExp(
                  /((?:^|[^\\])(?:\\\\)*)\$\{(?:(?!\})(?:<expr>))*\}/.source.replace(
                    /<expr>/g,
                    function () {
                      return FTL_EXPR
                    }
                  )
                ),
                lookbehind: true,
                inside: {
                  'interpolation-punctuation': {
                    pattern: /^\$\{|\}$/,
                    alias: 'punctuation'
                  },
                  rest: null
                }
              }
            }
          }
        ],
        keyword: /\b(?:as)\b/,
        boolean: /\b(?:false|true)\b/,
        'builtin-function': {
          pattern: /((?:^|[^?])\?\s*)\w+/,
          lookbehind: true,
          alias: 'function'
        },
        function: /\b\w+(?=\s*\()/,
        number: /\b\d+(?:\.\d+)?\b/,
        operator:
          /\.\.[<*!]?|->|--|\+\+|&&|\|\||\?{1,2}|[-+*/%!=<>]=?|\b(?:gt|gte|lt|lte)\b/,
        punctuation: /[,;.:()[\]{}]/
      };
      ftl.string[1].inside.interpolation.inside.rest = ftl;
      Prism.languages.ftl = {
        'ftl-comment': {
          // the pattern is shortened to be more efficient
          pattern: /^<#--[\s\S]*/,
          alias: 'comment'
        },
        'ftl-directive': {
          pattern: /^<[\s\S]+>$/,
          inside: {
            directive: {
              pattern: /(^<\/?)[#@][a-z]\w*/i,
              lookbehind: true,
              alias: 'keyword'
            },
            punctuation: /^<\/?|\/?>$/,
            content: {
              pattern: /\s*\S[\s\S]*/,
              alias: 'ftl',
              inside: ftl
            }
          }
        },
        'ftl-interpolation': {
          pattern: /^\$\{[\s\S]*\}$/,
          inside: {
            punctuation: /^\$\{|\}$/,
            content: {
              pattern: /\s*\S[\s\S]*/,
              alias: 'ftl',
              inside: ftl
            }
          }
        }
      };
      Prism.hooks.add('before-tokenize', function (env) {
        // eslint-disable-next-line regexp/no-useless-lazy
        var pattern = RegExp(
          /<#--[\s\S]*?-->|<\/?[#@][a-zA-Z](?:<expr>)*?>|\$\{(?:<expr>)*?\}/.source.replace(
            /<expr>/g,
            function () {
              return FTL_EXPR
            }
          ),
          'gi'
        );
        Prism.languages['markup-templating'].buildPlaceholders(
          env,
          'ftl',
          pattern
        );
      });
      Prism.hooks.add('after-tokenize', function (env) {
        Prism.languages['markup-templating'].tokenizePlaceholders(env, 'ftl');
      });
    })(Prism);
  }

  // @ts-nocheck
  gml.displayName = 'gml';
  gml.aliases = ['gamemakerlanguage'];

  /** @type {import('../core.js').Syntax} */
  function gml(Prism) {
    Prism.register(clike);
    Prism.languages.gamemakerlanguage = Prism.languages.gml =
      Prism.languages.extend('clike', {
        keyword:
          /\b(?:break|case|continue|default|do|else|enum|exit|for|globalvar|if|repeat|return|switch|until|var|while)\b/,
        number:
          /(?:\b0x[\da-f]+|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?)[ulf]{0,4}/i,
        operator:
          /--|\+\+|[-+%/=]=?|!=|\*\*?=?|<[<=>]?|>[=>]?|&&?|\^\^?|\|\|?|~|\b(?:and|at|not|or|with|xor)\b/,
        constant:
          /\b(?:GM_build_date|GM_version|action_(?:continue|restart|reverse|stop)|all|gamespeed_(?:fps|microseconds)|global|local|noone|other|pi|pointer_(?:invalid|null)|self|timezone_(?:local|utc)|undefined|ev_(?:create|destroy|step|alarm|keyboard|mouse|collision|other|draw|draw_(?:begin|end|post|pre)|keypress|keyrelease|trigger|(?:left|middle|no|right)_button|(?:left|middle|right)_press|(?:left|middle|right)_release|mouse_(?:enter|leave|wheel_down|wheel_up)|global_(?:left|middle|right)_button|global_(?:left|middle|right)_press|global_(?:left|middle|right)_release|joystick(?:1|2)_(?:button1|button2|button3|button4|button5|button6|button7|button8|down|left|right|up)|outside|boundary|game_start|game_end|room_start|room_end|no_more_lives|animation_end|end_of_path|no_more_health|user\d|gui|gui_begin|gui_end|step_(?:begin|end|normal))|vk_(?:alt|anykey|backspace|control|delete|down|end|enter|escape|home|insert|left|nokey|pagedown|pageup|pause|printscreen|return|right|shift|space|tab|up|f\d|numpad\d|add|decimal|divide|lalt|lcontrol|lshift|multiply|ralt|rcontrol|rshift|subtract)|achievement_(?:filter_(?:all_players|favorites_only|friends_only)|friends_info|info|leaderboard_info|our_info|pic_loaded|show_(?:achievement|bank|friend_picker|leaderboard|profile|purchase_prompt|ui)|type_challenge|type_score_challenge)|asset_(?:font|object|path|room|script|shader|sound|sprite|tiles|timeline|unknown)|audio_(?:3d|falloff_(?:exponent_distance|exponent_distance_clamped|inverse_distance|inverse_distance_clamped|linear_distance|linear_distance_clamped|none)|mono|new_system|old_system|stereo)|bm_(?:add|complex|dest_alpha|dest_color|dest_colour|inv_dest_alpha|inv_dest_color|inv_dest_colour|inv_src_alpha|inv_src_color|inv_src_colour|max|normal|one|src_alpha|src_alpha_sat|src_color|src_colour|subtract|zero)|browser_(?:chrome|firefox|ie|ie_mobile|not_a_browser|opera|safari|safari_mobile|tizen|unknown|windows_store)|buffer_(?:bool|f16|f32|f64|fast|fixed|generalerror|grow|invalidtype|network|outofbounds|outofspace|s16|s32|s8|seek_end|seek_relative|seek_start|string|text|u16|u32|u64|u8|vbuffer|wrap)|c_(?:aqua|black|blue|dkgray|fuchsia|gray|green|lime|ltgray|maroon|navy|olive|orange|purple|red|silver|teal|white|yellow)|cmpfunc_(?:always|equal|greater|greaterequal|less|lessequal|never|notequal)|cr_(?:appstart|arrow|beam|cross|default|drag|handpoint|hourglass|none|size_all|size_nesw|size_ns|size_nwse|size_we|uparrow)|cull_(?:clockwise|counterclockwise|noculling)|device_(?:emulator|tablet)|device_ios_(?:ipad|ipad_retina|iphone|iphone5|iphone6|iphone6plus|iphone_retina|unknown)|display_(?:landscape|landscape_flipped|portrait|portrait_flipped)|dll_(?:cdecl|cdel|stdcall)|ds_type_(?:grid|list|map|priority|queue|stack)|ef_(?:cloud|ellipse|explosion|firework|flare|rain|ring|smoke|smokeup|snow|spark|star)|fa_(?:archive|bottom|center|directory|hidden|left|middle|readonly|right|sysfile|top|volumeid)|fb_login_(?:default|fallback_to_webview|forcing_safari|forcing_webview|no_fallback_to_webview|use_system_account)|iap_(?:available|canceled|ev_consume|ev_product|ev_purchase|ev_restore|ev_storeload|failed|purchased|refunded|status_available|status_loading|status_processing|status_restoring|status_unavailable|status_uninitialised|storeload_failed|storeload_ok|unavailable)|leaderboard_type_(?:number|time_mins_secs)|lighttype_(?:dir|point)|matrix_(?:projection|view|world)|mb_(?:any|left|middle|none|right)|network_(?:config_(?:connect_timeout|disable_reliable_udp|enable_reliable_udp|use_non_blocking_socket)|socket_(?:bluetooth|tcp|udp)|type_(?:connect|data|disconnect|non_blocking_connect))|of_challenge_(?:lose|tie|win)|os_(?:android|ios|linux|macosx|ps3|ps4|psvita|unknown|uwp|win32|win8native|windows|winphone|xboxone)|phy_debug_render_(?:aabb|collision_pairs|coms|core_shapes|joints|obb|shapes)|phy_joint_(?:anchor_1_x|anchor_1_y|anchor_2_x|anchor_2_y|angle|angle_limits|damping_ratio|frequency|length_1|length_2|lower_angle_limit|max_force|max_length|max_motor_force|max_motor_torque|max_torque|motor_force|motor_speed|motor_torque|reaction_force_x|reaction_force_y|reaction_torque|speed|translation|upper_angle_limit)|phy_particle_data_flag_(?:category|color|colour|position|typeflags|velocity)|phy_particle_flag_(?:colormixing|colourmixing|elastic|powder|spring|tensile|viscous|wall|water|zombie)|phy_particle_group_flag_(?:rigid|solid)|pr_(?:linelist|linestrip|pointlist|trianglefan|trianglelist|trianglestrip)|ps_(?:distr|shape)_(?:diamond|ellipse|gaussian|invgaussian|line|linear|rectangle)|pt_shape_(?:circle|cloud|disk|explosion|flare|line|pixel|ring|smoke|snow|spark|sphere|square|star)|ty_(?:real|string)|gp_(?:face\d|axislh|axislv|axisrh|axisrv|padd|padl|padr|padu|select|shoulderl|shoulderlb|shoulderr|shoulderrb|start|stickl|stickr)|lb_disp_(?:none|numeric|time_ms|time_sec)|lb_sort_(?:ascending|descending|none)|ov_(?:achievements|community|friends|gamegroup|players|settings)|ugc_(?:filetype_(?:community|microtrans)|list_(?:Favorited|Followed|Published|Subscribed|UsedOrPlayed|VotedDown|VotedOn|VotedUp|WillVoteLater)|match_(?:AllGuides|Artwork|Collections|ControllerBindings|IntegratedGuides|Items|Items_Mtx|Items_ReadyToUse|Screenshots|UsableInGame|Videos|WebGuides)|query_(?:AcceptedForGameRankedByAcceptanceDate|CreatedByFriendsRankedByPublicationDate|FavoritedByFriendsRankedByPublicationDate|NotYetRated)|query_RankedBy(?:NumTimesReported|PublicationDate|TextSearch|TotalVotesAsc|Trend|Vote|VotesUp)|result_success|sortorder_CreationOrder(?:Asc|Desc)|sortorder_(?:ForModeration|LastUpdatedDesc|SubscriptionDateDesc|TitleAsc|VoteScoreDesc)|visibility_(?:friends_only|private|public))|vertex_usage_(?:binormal|blendindices|blendweight|color|colour|depth|fog|normal|position|psize|sample|tangent|texcoord|textcoord)|vertex_type_(?:float\d|color|colour|ubyte4)|input_type|layerelementtype_(?:background|instance|oldtilemap|particlesystem|sprite|tile|tilemap|undefined)|se_(?:chorus|compressor|echo|equalizer|flanger|gargle|none|reverb)|text_type|tile_(?:flip|index_mask|mirror|rotate)|(?:obj|rm|scr|spr)\w+)\b/,
        variable:
          /\b(?:alarm|application_surface|async_load|background_(?:alpha|blend|color|colour|foreground|height|hspeed|htiled|index|showcolor|showcolour|visible|vspeed|vtiled|width|x|xscale|y|yscale)|bbox_(?:bottom|left|right|top)|browser_(?:height|width)|caption_(?:health|lives|score)|current_(?:day|hour|minute|month|second|time|weekday|year)|cursor_sprite|debug_mode|delta_time|direction|display_aa|error_(?:last|occurred)|event_(?:action|number|object|type)|fps|fps_real|friction|game_(?:display|project|save)_(?:id|name)|gamemaker_(?:pro|registered|version)|gravity|gravity_direction|(?:h|v)speed|health|iap_data|id|image_(?:alpha|angle|blend|depth|index|number|speed|xscale|yscale)|instance_(?:count|id)|keyboard_(?:key|lastchar|lastkey|string)|layer|lives|mask_index|mouse_(?:button|lastbutton|x|y)|object_index|os_(?:browser|device|type|version)|path_(?:endaction|index|orientation|position|positionprevious|scale|speed)|persistent|phy_(?:rotation|(?:col_normal|collision|com|linear_velocity|position|speed)_(?:x|y)|angular_(?:damping|velocity)|position_(?:x|y)previous|speed|linear_damping|bullet|fixed_rotation|active|mass|inertia|dynamic|kinematic|sleeping|collision_points)|pointer_(?:invalid|null)|room|room_(?:caption|first|height|last|persistent|speed|width)|score|secure_mode|show_(?:health|lives|score)|solid|speed|sprite_(?:height|index|width|xoffset|yoffset)|temp_directory|timeline_(?:index|loop|position|running|speed)|transition_(?:color|kind|steps)|undefined|view_(?:angle|current|enabled|(?:h|v)(?:border|speed)|(?:h|w|x|y)port|(?:h|w|x|y)view|object|surface_id|visible)|visible|webgl_enabled|working_directory|(?:x|y)(?:previous|start)|x|y|argument(?:_relitive|_count|\d)|argument|global|local|other|self)\b/
      });
  }

  // @ts-nocheck
  gap.displayName = 'gap';
  gap.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function gap(Prism) {
    // https://www.gap-system.org/Manuals/doc/ref/chap4.html
    // https://www.gap-system.org/Manuals/doc/ref/chap27.html
    Prism.languages.gap = {
      shell: {
        pattern: /^gap>[\s\S]*?(?=^gap>|$(?![\s\S]))/m,
        greedy: true,
        inside: {
          gap: {
            pattern: /^(gap>).+(?:(?:\r(?:\n|(?!\n))|\n)>.*)*/,
            lookbehind: true,
            inside: null // see below
          },
          punctuation: /^gap>/
        }
      },
      comment: {
        pattern: /#.*/,
        greedy: true
      },
      string: {
        pattern:
          /(^|[^\\'"])(?:'(?:[^\r\n\\']|\\.){1,10}'|"(?:[^\r\n\\"]|\\.)*"(?!")|"""[\s\S]*?""")/,
        lookbehind: true,
        greedy: true,
        inside: {
          continuation: {
            pattern: /([\r\n])>/,
            lookbehind: true,
            alias: 'punctuation'
          }
        }
      },
      keyword:
        /\b(?:Assert|Info|IsBound|QUIT|TryNextMethod|Unbind|and|atomic|break|continue|do|elif|else|end|fi|for|function|if|in|local|mod|not|od|or|quit|readonly|readwrite|rec|repeat|return|then|until|while)\b/,
      boolean: /\b(?:false|true)\b/,
      function: /\b[a-z_]\w*(?=\s*\()/i,
      number: {
        pattern:
          /(^|[^\w.]|\.\.)(?:\d+(?:\.\d*)?|\.\d+)(?:[eE][+-]?\d+)?(?:_[a-z]?)?(?=$|[^\w.]|\.\.)/,
        lookbehind: true
      },
      continuation: {
        pattern: /([\r\n])>/,
        lookbehind: true,
        alias: 'punctuation'
      },
      operator: /->|[-+*/^~=!]|<>|[<>]=?|:=|\.\./,
      punctuation: /[()[\]{},;.:]/
    };
    Prism.languages.gap.shell.inside.gap.inside = Prism.languages.gap;
  }

  // @ts-nocheck
  gcode.displayName = 'gcode';
  gcode.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function gcode(Prism) {
    Prism.languages.gcode = {
      comment: /;.*|\B\(.*?\)\B/,
      string: {
        pattern: /"(?:""|[^"])*"/,
        greedy: true
      },
      keyword: /\b[GM]\d+(?:\.\d+)?\b/,
      property: /\b[A-Z]/,
      checksum: {
        pattern: /(\*)\d+/,
        lookbehind: true,
        alias: 'number'
      },
      // T0:0:0
      punctuation: /[:*]/
    };
  }

  // @ts-nocheck
  gdscript.displayName = 'gdscript';
  gdscript.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function gdscript(Prism) {
    Prism.languages.gdscript = {
      comment: /#.*/,
      string: {
        pattern:
          /@?(?:("|')(?:(?!\1)[^\n\\]|\\[\s\S])*\1(?!"|')|"""(?:[^\\]|\\[\s\S])*?""")/,
        greedy: true
      },
      'class-name': {
        // class_name Foo, extends Bar, class InnerClass
        // export(int) var baz, export(int, 0) var i
        // as Node
        // const FOO: int = 9, var bar: bool = true
        // func add(reference: Item, amount: int) -> Item:
        pattern:
          /(^(?:class|class_name|extends)[ \t]+|^export\([ \t]*|\bas[ \t]+|(?:\b(?:const|var)[ \t]|[,(])[ \t]*\w+[ \t]*:[ \t]*|->[ \t]*)[a-zA-Z_]\w*/m,
        lookbehind: true
      },
      keyword:
        /\b(?:and|as|assert|break|breakpoint|class|class_name|const|continue|elif|else|enum|export|extends|for|func|if|in|is|master|mastersync|match|not|null|onready|or|pass|preload|puppet|puppetsync|remote|remotesync|return|self|setget|signal|static|tool|var|while|yield)\b/,
      function: /\b[a-z_]\w*(?=[ \t]*\()/i,
      variable: /\$\w+/,
      number: [
        /\b0b[01_]+\b|\b0x[\da-fA-F_]+\b|(?:\b\d[\d_]*(?:\.[\d_]*)?|\B\.[\d_]+)(?:e[+-]?[\d_]+)?\b/,
        /\b(?:INF|NAN|PI|TAU)\b/
      ],
      constant: /\b[A-Z][A-Z_\d]*\b/,
      boolean: /\b(?:false|true)\b/,
      operator: /->|:=|&&|\|\||<<|>>|[-+*/%&|!<>=]=?|[~^]/,
      punctuation: /[.:,;()[\]{}]/
    };
  }

  // @ts-nocheck
  gedcom.displayName = 'gedcom';
  gedcom.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function gedcom(Prism) {
    Prism.languages.gedcom = {
      'line-value': {
        // Preceded by level, optional pointer, and tag
        pattern:
          /(^[\t ]*\d+ +(?:@\w[\w!"$%&'()*+,\-./:;<=>?[\\\]^`{|}~\x80-\xfe #]*@ +)?\w+ ).+/m,
        lookbehind: true,
        inside: {
          pointer: {
            pattern: /^@\w[\w!"$%&'()*+,\-./:;<=>?[\\\]^`{|}~\x80-\xfe #]*@$/,
            alias: 'variable'
          }
        }
      },
      tag: {
        // Preceded by level and optional pointer
        pattern:
          /(^[\t ]*\d+ +(?:@\w[\w!"$%&'()*+,\-./:;<=>?[\\\]^`{|}~\x80-\xfe #]*@ +)?)\w+/m,
        lookbehind: true,
        alias: 'string'
      },
      level: {
        pattern: /(^[\t ]*)\d+/m,
        lookbehind: true,
        alias: 'number'
      },
      pointer: {
        pattern: /@\w[\w!"$%&'()*+,\-./:;<=>?[\\\]^`{|}~\x80-\xfe #]*@/,
        alias: 'variable'
      }
    };
  }

  // @ts-nocheck
  gherkin.displayName = 'gherkin';
  gherkin.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function gherkin(Prism) {
  (function (Prism) {
      var tableRow = /(?:\r?\n|\r)[ \t]*\|.+\|(?:(?!\|).)*/.source;
      Prism.languages.gherkin = {
        pystring: {
          pattern: /("""|''')[\s\S]+?\1/,
          alias: 'string'
        },
        comment: {
          pattern: /(^[ \t]*)#.*/m,
          lookbehind: true
        },
        tag: {
          pattern: /(^[ \t]*)@\S*/m,
          lookbehind: true
        },
        feature: {
          pattern:
            /((?:^|\r?\n|\r)[ \t]*)(?:Ability|Ahoy matey!|Arwedd|Aspekt|Besigheid Behoefte|Business Need|Caracteristica|Característica|Egenskab|Egenskap|Eiginleiki|Feature|Fīča|Fitur|Fonctionnalité|Fonksyonalite|Funcionalidade|Funcionalitat|Functionalitate|Funcţionalitate|Funcționalitate|Functionaliteit|Fungsi|Funkcia|Funkcija|Funkcionalitāte|Funkcionalnost|Funkcja|Funksie|Funktionalität|Funktionalitéit|Funzionalità|Hwaet|Hwæt|Jellemző|Karakteristik|Lastnost|Mak|Mogucnost|laH|Mogućnost|Moznosti|Možnosti|OH HAI|Omadus|Ominaisuus|Osobina|Özellik|Potrzeba biznesowa|perbogh|poQbogh malja'|Požadavek|Požiadavka|Pretty much|Qap|Qu'meH 'ut|Savybė|Tính năng|Trajto|Vermoë|Vlastnosť|Właściwość|Značilnost|Δυνατότητα|Λειτουργία|Могућност|Мөмкинлек|Особина|Свойство|Үзенчәлеклелек|Функционал|Функционалност|Функция|Функціонал|תכונה|خاصية|خصوصیت|صلاحیت|کاروبار کی ضرورت|وِیژگی|रूप लेख|ਖਾਸੀਅਤ|ਨਕਸ਼ ਨੁਹਾਰ|ਮੁਹਾਂਦਰਾ|గుణము|ಹೆಚ್ಚಳ|ความต้องการทางธุรกิจ|ความสามารถ|โครงหลัก|기능|フィーチャ|功能|機能):(?:[^:\r\n]+(?:\r?\n|\r|$))*/,
          lookbehind: true,
          inside: {
            important: {
              pattern: /(:)[^\r\n]+/,
              lookbehind: true
            },
            keyword: /[^:\r\n]+:/
          }
        },
        scenario: {
          pattern:
            /(^[ \t]*)(?:Abstract Scenario|Abstrakt Scenario|Achtergrond|Aer|Ær|Agtergrond|All y'all|Antecedentes|Antecedents|Atburðarás|Atburðarásir|Awww, look mate|B4|Background|Baggrund|Bakgrund|Bakgrunn|Bakgrunnur|Beispiele|Beispiller|Bối cảnh|Cefndir|Cenario|Cenário|Cenario de Fundo|Cenário de Fundo|Cenarios|Cenários|Contesto|Context|Contexte|Contexto|Conto|Contoh|Contone|Dæmi|Dasar|Dead men tell no tales|Delineacao do Cenario|Delineação do Cenário|Dis is what went down|Dữ liệu|Dyagram Senaryo|Dyagram senaryo|Egzanp|Ejemplos|Eksempler|Ekzemploj|Enghreifftiau|Esbozo do escenario|Escenari|Escenario|Esempi|Esquema de l'escenari|Esquema del escenario|Esquema do Cenario|Esquema do Cenário|EXAMPLZ|Examples|Exempel|Exemple|Exemples|Exemplos|First off|Fono|Forgatókönyv|Forgatókönyv vázlat|Fundo|Geçmiş|Grundlage|Hannergrond|ghantoH|Háttér|Heave to|Istorik|Juhtumid|Keadaan|Khung kịch bản|Khung tình huống|Kịch bản|Koncept|Konsep skenario|Kontèks|Kontekst|Kontekstas|Konteksts|Kontext|Konturo de la scenaro|Latar Belakang|lut chovnatlh|lut|lutmey|Lýsing Atburðarásar|Lýsing Dæma|MISHUN SRSLY|MISHUN|Menggariskan Senario|mo'|Náčrt Scenára|Náčrt Scénáře|Náčrt Scenáru|Oris scenarija|Örnekler|Osnova|Osnova Scenára|Osnova scénáře|Osnutek|Ozadje|Paraugs|Pavyzdžiai|Példák|Piemēri|Plan du scénario|Plan du Scénario|Plan Senaryo|Plan senaryo|Plang vum Szenario|Pozadí|Pozadie|Pozadina|Príklady|Příklady|Primer|Primeri|Primjeri|Przykłady|Raamstsenaarium|Reckon it's like|Rerefons|Scenár|Scénář|Scenarie|Scenarij|Scenarijai|Scenarijaus šablonas|Scenariji|Scenārijs|Scenārijs pēc parauga|Scenarijus|Scenario|Scénario|Scenario Amlinellol|Scenario Outline|Scenario Template|Scenariomal|Scenariomall|Scenarios|Scenariu|Scenariusz|Scenaro|Schema dello scenario|Se ðe|Se the|Se þe|Senario|Senaryo Deskripsyon|Senaryo deskripsyon|Senaryo|Senaryo taslağı|Shiver me timbers|Situācija|Situai|Situasie Uiteensetting|Situasie|Skenario konsep|Skenario|Skica|Structura scenariu|Structură scenariu|Struktura scenarija|Stsenaarium|Swa hwaer swa|Swa|Swa hwær swa|Szablon scenariusza|Szenario|Szenariogrundriss|Tapaukset|Tapaus|Tapausaihio|Taust|Tausta|Template Keadaan|Template Senario|Template Situai|The thing of it is|Tình huống|Variantai|Voorbeelde|Voorbeelden|Wharrimean is|Yo-ho-ho|You'll wanna|Założenia|Παραδείγματα|Περιγραφή Σεναρίου|Σενάρια|Σενάριο|Υπόβαθρο|Кереш|Контекст|Концепт|Мисаллар|Мисоллар|Основа|Передумова|Позадина|Предистория|Предыстория|Приклади|Пример|Примери|Примеры|Рамка на сценарий|Скица|Структура сценарија|Структура сценария|Структура сценарію|Сценарий|Сценарий структураси|Сценарийның төзелеше|Сценарији|Сценарио|Сценарій|Тарих|Үрнәкләр|דוגמאות|רקע|תבנית תרחיש|תרחיש|الخلفية|الگوی سناریو|امثلة|پس منظر|زمینه|سناریو|سيناريو|سيناريو مخطط|مثالیں|منظر نامے کا خاکہ|منظرنامہ|نمونه ها|उदाहरण|परिदृश्य|परिदृश्य रूपरेखा|पृष्ठभूमि|ਉਦਾਹਰਨਾਂ|ਪਟਕਥਾ|ਪਟਕਥਾ ਢਾਂਚਾ|ਪਟਕਥਾ ਰੂਪ ਰੇਖਾ|ਪਿਛੋਕੜ|ఉదాహరణలు|కథనం|నేపథ్యం|సన్నివేశం|ಉದಾಹರಣೆಗಳು|ಕಥಾಸಾರಾಂಶ|ವಿವರಣೆ|ಹಿನ್ನೆಲೆ|โครงสร้างของเหตุการณ์|ชุดของตัวอย่าง|ชุดของเหตุการณ์|แนวคิด|สรุปเหตุการณ์|เหตุการณ์|배경|시나리오|시나리오 개요|예|サンプル|シナリオ|シナリオアウトライン|シナリオテンプレ|シナリオテンプレート|テンプレ|例|例子|剧本|剧本大纲|劇本|劇本大綱|场景|场景大纲|場景|場景大綱|背景):[^:\r\n]*/m,
          lookbehind: true,
          inside: {
            important: {
              pattern: /(:)[^\r\n]*/,
              lookbehind: true
            },
            keyword: /[^:\r\n]+:/
          }
        },
        'table-body': {
          // Look-behind is used to skip the table head, which has the same format as any table row
          pattern: RegExp('(' + tableRow + ')(?:' + tableRow + ')+'),
          lookbehind: true,
          inside: {
            outline: {
              pattern: /<[^>]+>/,
              alias: 'variable'
            },
            td: {
              pattern: /\s*[^\s|][^|]*/,
              alias: 'string'
            },
            punctuation: /\|/
          }
        },
        'table-head': {
          pattern: RegExp(tableRow),
          inside: {
            th: {
              pattern: /\s*[^\s|][^|]*/,
              alias: 'variable'
            },
            punctuation: /\|/
          }
        },
        atrule: {
          pattern:
            /(^[ \t]+)(?:'a|'ach|'ej|7|a|A také|A taktiež|A tiež|A zároveň|Aber|Ac|Adott|Akkor|Ak|Aleshores|Ale|Ali|Allora|Alors|Als|Ama|Amennyiben|Amikor|Ampak|an|AN|Ananging|And y'all|And|Angenommen|Anrhegedig a|An|Apabila|Atès|Atesa|Atunci|Avast!|Aye|A|awer|Bagi|Banjur|Bet|Biết|Blimey!|Buh|But at the end of the day I reckon|But y'all|But|BUT|Cal|Când|Cand|Cando|Ce|Cuando|Če|Ða ðe|Ða|Dadas|Dada|Dados|Dado|DaH ghu' bejlu'|dann|Dann|Dano|Dan|Dar|Dat fiind|Data|Date fiind|Date|Dati fiind|Dati|Daţi fiind|Dați fiind|DEN|Dato|De|Den youse gotta|Dengan|Diberi|Diyelim ki|Donada|Donat|Donitaĵo|Do|Dun|Duota|Ðurh|Eeldades|Ef|Eğer ki|Entao|Então|Entón|E|En|Entonces|Epi|És|Etant donnée|Etant donné|Et|Étant données|Étant donnée|Étant donné|Etant données|Etant donnés|Étant donnés|Fakat|Gangway!|Gdy|Gegeben seien|Gegeben sei|Gegeven|Gegewe|ghu' noblu'|Gitt|Given y'all|Given|Givet|Givun|Ha|Cho|I CAN HAZ|In|Ir|It's just unbelievable|I|Ja|Jeśli|Jeżeli|Kad|Kada|Kadar|Kai|Kaj|Když|Keď|Kemudian|Ketika|Khi|Kiedy|Ko|Kuid|Kui|Kun|Lan|latlh|Le sa a|Let go and haul|Le|Lè sa a|Lè|Logo|Lorsqu'<|Lorsque|mä|Maar|Mais|Mając|Ma|Majd|Maka|Manawa|Mas|Men|Menawa|Mutta|Nalika|Nalikaning|Nanging|Når|När|Nato|Nhưng|Niin|Njuk|O zaman|Och|Og|Oletetaan|Ond|Onda|Oraz|Pak|Pero|Però|Podano|Pokiaľ|Pokud|Potem|Potom|Privzeto|Pryd|Quan|Quand|Quando|qaSDI'|Så|Sed|Se|Siis|Sipoze ke|Sipoze Ke|Sipoze|Si|Şi|Și|Soit|Stel|Tada|Tad|Takrat|Tak|Tapi|Ter|Tetapi|Tha the|Tha|Then y'all|Then|Thì|Thurh|Toda|Too right|Un|Und|ugeholl|Và|vaj|Vendar|Ve|wann|Wanneer|WEN|Wenn|When y'all|When|Wtedy|Wun|Y'know|Yeah nah|Yna|Youse know like when|Youse know when youse got|Y|Za predpokladu|Za předpokladu|Zadan|Zadani|Zadano|Zadate|Zadato|Zakładając|Zaradi|Zatati|Þa þe|Þa|Þá|Þegar|Þurh|Αλλά|Δεδομένου|Και|Όταν|Τότε|А також|Агар|Але|Али|Аммо|А|Әгәр|Әйтик|Әмма|Бирок|Ва|Вә|Дадено|Дано|Допустим|Если|Задате|Задати|Задато|И|І|К тому же|Када|Кад|Когато|Когда|Коли|Ләкин|Лекин|Нәтиҗәдә|Нехай|Но|Онда|Припустимо, що|Припустимо|Пусть|Также|Та|Тогда|Тоді|То|Унда|Һәм|Якщо|אבל|אזי|אז|בהינתן|וגם|כאשר|آنگاه|اذاً|اگر|اما|اور|با فرض|بالفرض|بفرض|پھر|تب|ثم|جب|عندما|فرض کیا|لكن|لیکن|متى|هنگامی|و|अगर|और|कदा|किन्तु|चूंकि|जब|तथा|तदा|तब|परन्तु|पर|यदि|ਅਤੇ|ਜਦੋਂ|ਜਿਵੇਂ ਕਿ|ਜੇਕਰ|ਤਦ|ਪਰ|అప్పుడు|ఈ పరిస్థితిలో|కాని|చెప్పబడినది|మరియు|ಆದರೆ|ನಂತರ|ನೀಡಿದ|ಮತ್ತು|ಸ್ಥಿತಿಯನ್ನು|กำหนดให้|ดังนั้น|แต่|เมื่อ|และ|그러면<|그리고<|단<|만약<|만일<|먼저<|조건<|하지만<|かつ<|しかし<|ただし<|ならば<|もし<|並且<|但し<|但是<|假如<|假定<|假設<|假设<|前提<|同时<|同時<|并且<|当<|當<|而且<|那么<|那麼<)(?=[ \t])/m,
          lookbehind: true
        },
        string: {
          pattern: /"(?:\\.|[^"\\\r\n])*"|'(?:\\.|[^'\\\r\n])*'/,
          inside: {
            outline: {
              pattern: /<[^>]+>/,
              alias: 'variable'
            }
          }
        },
        outline: {
          pattern: /<[^>]+>/,
          alias: 'variable'
        }
      };
    })(Prism);
  }

  // @ts-nocheck
  git.displayName = 'git';
  git.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function git(Prism) {
    Prism.languages.git = {
      /*
       * A simple one line comment like in a git status command
       * For instance:
       * $ git status
       * # On branch infinite-scroll
       * # Your branch and 'origin/sharedBranches/frontendTeam/infinite-scroll' have diverged,
       * # and have 1 and 2 different commits each, respectively.
       * nothing to commit (working directory clean)
       */
      comment: /^#.*/m,
      /*
       * Regexp to match the changed lines in a git diff output. Check the example below.
       */
      deleted: /^[-–].*/m,
      inserted: /^\+.*/m,
      /*
       * a string (double and simple quote)
       */
      string: /("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,
      /*
       * a git command. It starts with a random prompt finishing by a $, then "git" then some other parameters
       * For instance:
       * $ git add file.txt
       */
      command: {
        pattern: /^.*\$ git .*$/m,
        inside: {
          /*
           * A git command can contain a parameter starting by a single or a double dash followed by a string
           * For instance:
           * $ git diff --cached
           * $ git log -p
           */
          parameter: /\s--?\w+/
        }
      },
      /*
       * Coordinates displayed in a git diff command
       * For instance:
       * $ git diff
       * diff --git file.txt file.txt
       * index 6214953..1d54a52 100644
       * --- file.txt
       * +++ file.txt
       * @@ -1 +1,2 @@
       * -Here's my tetx file
       * +Here's my text file
       * +And this is the second line
       */
      coord: /^@@.*@@$/m,
      /*
       * Match a "commit [SHA1]" line in a git log output.
       * For instance:
       * $ git log
       * commit a11a14ef7e26f2ca62d4b35eac455ce636d0dc09
       * Author: lgiraudel
       * Date:   Mon Feb 17 11:18:34 2014 +0100
       *
       *     Add of a new line
       */
      'commit-sha1': /^commit \w{40}$/m
    };
  }

  // @ts-nocheck
  glsl.displayName = 'glsl';
  glsl.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function glsl(Prism) {
    Prism.register(c$1);
    Prism.languages.glsl = Prism.languages.extend('c', {
      keyword:
        /\b(?:active|asm|atomic_uint|attribute|[ibdu]?vec[234]|bool|break|buffer|case|cast|centroid|class|coherent|common|const|continue|d?mat[234](?:x[234])?|default|discard|do|double|else|enum|extern|external|false|filter|fixed|flat|float|for|fvec[234]|goto|half|highp|hvec[234]|[iu]?sampler2DMS(?:Array)?|[iu]?sampler2DRect|[iu]?samplerBuffer|[iu]?samplerCube|[iu]?samplerCubeArray|[iu]?sampler[123]D|[iu]?sampler[12]DArray|[iu]?image2DMS(?:Array)?|[iu]?image2DRect|[iu]?imageBuffer|[iu]?imageCube|[iu]?imageCubeArray|[iu]?image[123]D|[iu]?image[12]DArray|if|in|inline|inout|input|int|interface|invariant|layout|long|lowp|mediump|namespace|noinline|noperspective|out|output|partition|patch|precise|precision|public|readonly|resource|restrict|return|sample|sampler[12]DArrayShadow|sampler[12]DShadow|sampler2DRectShadow|sampler3DRect|samplerCubeArrayShadow|samplerCubeShadow|shared|short|sizeof|smooth|static|struct|subroutine|superp|switch|template|this|true|typedef|uint|uniform|union|unsigned|using|varying|void|volatile|while|writeonly)\b/
    });
  }

  // @ts-nocheck
  gn.displayName = 'gn';
  gn.aliases = ['gni'];

  /** @type {import('../core.js').Syntax} */
  function gn(Prism) {
    // https://gn.googlesource.com/gn/+/refs/heads/main/docs/reference.md#grammar
    Prism.languages.gn = {
      comment: {
        pattern: /#.*/,
        greedy: true
      },
      'string-literal': {
        pattern: /(^|[^\\"])"(?:[^\r\n"\\]|\\.)*"/,
        lookbehind: true,
        greedy: true,
        inside: {
          interpolation: {
            pattern:
              /((?:^|[^\\])(?:\\{2})*)\$(?:\{[\s\S]*?\}|[a-zA-Z_]\w*|0x[a-fA-F0-9]{2})/,
            lookbehind: true,
            inside: {
              number: /^\$0x[\s\S]{2}$/,
              variable: /^\$\w+$/,
              'interpolation-punctuation': {
                pattern: /^\$\{|\}$/,
                alias: 'punctuation'
              },
              expression: {
                pattern: /[\s\S]+/,
                inside: null // see below
              }
            }
          },
          string: /[\s\S]+/
        }
      },
      keyword: /\b(?:else|if)\b/,
      boolean: /\b(?:false|true)\b/,
      'builtin-function': {
        // a few functions get special highlighting to improve readability
        pattern:
          /\b(?:assert|defined|foreach|import|pool|print|template|tool|toolchain)(?=\s*\()/i,
        alias: 'keyword'
      },
      function: /\b[a-z_]\w*(?=\s*\()/i,
      constant:
        /\b(?:current_cpu|current_os|current_toolchain|default_toolchain|host_cpu|host_os|root_build_dir|root_gen_dir|root_out_dir|target_cpu|target_gen_dir|target_os|target_out_dir)\b/,
      number: /-?\b\d+\b/,
      operator: /[-+!=<>]=?|&&|\|\|/,
      punctuation: /[(){}[\],.]/
    };
    Prism.languages.gn['string-literal'].inside['interpolation'].inside[
      'expression'
    ].inside = Prism.languages.gn;
    Prism.languages.gni = Prism.languages.gn;
  }

  // @ts-nocheck
  goModule.displayName = 'go-module';
  goModule.aliases = ['go-mod'];

  /** @type {import('../core.js').Syntax} */
  function goModule(Prism) {
    // https://go.dev/ref/mod#go-mod-file-module
    Prism.languages['go-mod'] = Prism.languages['go-module'] = {
      comment: {
        pattern: /\/\/.*/,
        greedy: true
      },
      version: {
        pattern: /(^|[\s()[\],])v\d+\.\d+\.\d+(?:[+-][-+.\w]*)?(?![^\s()[\],])/,
        lookbehind: true,
        alias: 'number'
      },
      'go-version': {
        pattern: /((?:^|\s)go\s+)\d+(?:\.\d+){1,2}/,
        lookbehind: true,
        alias: 'number'
      },
      keyword: {
        pattern: /^([ \t]*)(?:exclude|go|module|replace|require|retract)\b/m,
        lookbehind: true
      },
      operator: /=>/,
      punctuation: /[()[\],]/
    };
  }

  // @ts-nocheck
  graphql.displayName = 'graphql';
  graphql.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function graphql(Prism) {
    Prism.languages.graphql = {
      comment: /#.*/,
      description: {
        pattern:
          /(?:"""(?:[^"]|(?!""")")*"""|"(?:\\.|[^\\"\r\n])*")(?=\s*[a-z_])/i,
        greedy: true,
        alias: 'string',
        inside: {
          'language-markdown': {
            pattern: /(^"(?:"")?)(?!\1)[\s\S]+(?=\1$)/,
            lookbehind: true,
            inside: Prism.languages.markdown
          }
        }
      },
      string: {
        pattern: /"""(?:[^"]|(?!""")")*"""|"(?:\\.|[^\\"\r\n])*"/,
        greedy: true
      },
      number: /(?:\B-|\b)\d+(?:\.\d+)?(?:e[+-]?\d+)?\b/i,
      boolean: /\b(?:false|true)\b/,
      variable: /\$[a-z_]\w*/i,
      directive: {
        pattern: /@[a-z_]\w*/i,
        alias: 'function'
      },
      'attr-name': {
        pattern: /\b[a-z_]\w*(?=\s*(?:\((?:[^()"]|"(?:\\.|[^\\"\r\n])*")*\))?:)/i,
        greedy: true
      },
      'atom-input': {
        pattern: /\b[A-Z]\w*Input\b/,
        alias: 'class-name'
      },
      scalar: /\b(?:Boolean|Float|ID|Int|String)\b/,
      constant: /\b[A-Z][A-Z_\d]*\b/,
      'class-name': {
        pattern:
          /(\b(?:enum|implements|interface|on|scalar|type|union)\s+|&\s*|:\s*|\[)[A-Z_]\w*/,
        lookbehind: true
      },
      fragment: {
        pattern: /(\bfragment\s+|\.{3}\s*(?!on\b))[a-zA-Z_]\w*/,
        lookbehind: true,
        alias: 'function'
      },
      'definition-mutation': {
        pattern: /(\bmutation\s+)[a-zA-Z_]\w*/,
        lookbehind: true,
        alias: 'function'
      },
      'definition-query': {
        pattern: /(\bquery\s+)[a-zA-Z_]\w*/,
        lookbehind: true,
        alias: 'function'
      },
      keyword:
        /\b(?:directive|enum|extend|fragment|implements|input|interface|mutation|on|query|repeatable|scalar|schema|subscription|type|union)\b/,
      operator: /[!=|&]|\.{3}/,
      'property-query': /\w+(?=\s*\()/,
      object: /\w+(?=\s*\{)/,
      punctuation: /[!(){}\[\]:=,]/,
      property: /\w+/
    };
    Prism.hooks.add('after-tokenize', function afterTokenizeGraphql(env) {
      if (env.language !== 'graphql') {
        return
      }
      /**
       * get the graphql token stream that we want to customize
       *
       * @typedef {InstanceType<import("./prism-core")["Token"]>} Token
       * @type {Token[]}
       */
      var validTokens = env.tokens.filter(function (token) {
        return (
          typeof token !== 'string' &&
          token.type !== 'comment' &&
          token.type !== 'scalar'
        )
      });
      var currentIndex = 0;
      /**
       * Returns whether the token relative to the current index has the given type.
       *
       * @param {number} offset
       * @returns {Token | undefined}
       */
      function getToken(offset) {
        return validTokens[currentIndex + offset]
      }
      /**
       * Returns whether the token relative to the current index has the given type.
       *
       * @param {readonly string[]} types
       * @param {number} [offset=0]
       * @returns {boolean}
       */
      function isTokenType(types, offset) {
        offset = offset || 0;
        for (var i = 0; i < types.length; i++) {
          var token = getToken(i + offset);
          if (!token || token.type !== types[i]) {
            return false
          }
        }
        return true
      }
      /**
       * Returns the index of the closing bracket to an opening bracket.
       *
       * It is assumed that `token[currentIndex - 1]` is an opening bracket.
       *
       * If no closing bracket could be found, `-1` will be returned.
       *
       * @param {RegExp} open
       * @param {RegExp} close
       * @returns {number}
       */
      function findClosingBracket(open, close) {
        var stackHeight = 1;
        for (var i = currentIndex; i < validTokens.length; i++) {
          var token = validTokens[i];
          var content = token.content;
          if (token.type === 'punctuation' && typeof content === 'string') {
            if (open.test(content)) {
              stackHeight++;
            } else if (close.test(content)) {
              stackHeight--;
              if (stackHeight === 0) {
                return i
              }
            }
          }
        }
        return -1
      }
      /**
       * Adds an alias to the given token.
       *
       * @param {Token} token
       * @param {string} alias
       * @returns {void}
       */
      function addAlias(token, alias) {
        var aliases = token.alias;
        if (!aliases) {
          token.alias = aliases = [];
        } else if (!Array.isArray(aliases)) {
          token.alias = aliases = [aliases];
        }
        aliases.push(alias);
      }
      for (; currentIndex < validTokens.length; ) {
        var startToken = validTokens[currentIndex++]; // add special aliases for mutation tokens
        if (startToken.type === 'keyword' && startToken.content === 'mutation') {
          // any array of the names of all input variables (if any)
          var inputVariables = [];
          if (
            isTokenType(['definition-mutation', 'punctuation']) &&
            getToken(1).content === '('
          ) {
            // definition
            currentIndex += 2; // skip 'definition-mutation' and 'punctuation'
            var definitionEnd = findClosingBracket(/^\($/, /^\)$/);
            if (definitionEnd === -1) {
              continue
            } // find all input variables
            for (; currentIndex < definitionEnd; currentIndex++) {
              var t = getToken(0);
              if (t.type === 'variable') {
                addAlias(t, 'variable-input');
                inputVariables.push(t.content);
              }
            }
            currentIndex = definitionEnd + 1;
          }
          if (
            isTokenType(['punctuation', 'property-query']) &&
            getToken(0).content === '{'
          ) {
            currentIndex++; // skip opening bracket
            addAlias(getToken(0), 'property-mutation');
            if (inputVariables.length > 0) {
              var mutationEnd = findClosingBracket(/^\{$/, /^\}$/);
              if (mutationEnd === -1) {
                continue
              } // give references to input variables a special alias
              for (var i = currentIndex; i < mutationEnd; i++) {
                var varToken = validTokens[i];
                if (
                  varToken.type === 'variable' &&
                  inputVariables.indexOf(varToken.content) >= 0
                ) {
                  addAlias(varToken, 'variable-input');
                }
              }
            }
          }
        }
      }
    });
  }

  // @ts-nocheck
  groovy.displayName = 'groovy';
  groovy.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function groovy(Prism) {
    Prism.register(clike);
    Prism.languages.groovy = Prism.languages.extend('clike', {
      string: [
        {
          // https://groovy-lang.org/syntax.html#_dollar_slashy_string
          pattern:
            /("""|''')(?:[^\\]|\\[\s\S])*?\1|\$\/(?:[^/$]|\$(?:[/$]|(?![/$]))|\/(?!\$))*\/\$/,
          greedy: true
        },
        {
          // TODO: Slash strings (e.g. /foo/) can contain line breaks but this will cause a lot of trouble with
          // simple division (see JS regex), so find a fix maybe?
          pattern: /(["'/])(?:\\.|(?!\1)[^\\\r\n])*\1/,
          greedy: true
        }
      ],
      keyword:
        /\b(?:abstract|as|assert|boolean|break|byte|case|catch|char|class|const|continue|def|default|do|double|else|enum|extends|final|finally|float|for|goto|if|implements|import|in|instanceof|int|interface|long|native|new|package|private|protected|public|return|short|static|strictfp|super|switch|synchronized|this|throw|throws|trait|transient|try|void|volatile|while)\b/,
      number:
        /\b(?:0b[01_]+|0x[\da-f_]+(?:\.[\da-f_p\-]+)?|[\d_]+(?:\.[\d_]+)?(?:e[+-]?\d+)?)[glidf]?\b/i,
      operator: {
        pattern:
          /(^|[^.])(?:~|==?~?|\?[.:]?|\*(?:[.=]|\*=?)?|\.[@&]|\.\.<|\.\.(?!\.)|-[-=>]?|\+[+=]?|!=?|<(?:<=?|=>?)?|>(?:>>?=?|=)?|&[&=]?|\|[|=]?|\/=?|\^=?|%=?)/,
        lookbehind: true
      },
      punctuation: /\.+|[{}[\];(),:$]/
    });
    Prism.languages.insertBefore('groovy', 'string', {
      shebang: {
        pattern: /#!.+/,
        alias: 'comment'
      }
    });
    Prism.languages.insertBefore('groovy', 'punctuation', {
      'spock-block': /\b(?:and|cleanup|expect|given|setup|then|when|where):/
    });
    Prism.languages.insertBefore('groovy', 'function', {
      annotation: {
        pattern: /(^|[^.])@\w+/,
        lookbehind: true,
        alias: 'punctuation'
      }
    }); // Handle string interpolation
    Prism.hooks.add('wrap', function (env) {
      if (env.language === 'groovy' && env.type === 'string') {
        var delimiter = env.content.value[0];
        if (delimiter != "'") {
          var pattern = /([^\\])(?:\$(?:\{.*?\}|[\w.]+))/;
          if (delimiter === '$') {
            pattern = /([^\$])(?:\$(?:\{.*?\}|[\w.]+))/;
          } // To prevent double HTML-encoding we have to decode env.content first
          env.content.value = env.content.value
            .replace(/&lt;/g, '<')
            .replace(/&amp;/g, '&');
          env.content = Prism.highlight(env.content.value, {
            expression: {
              pattern: pattern,
              lookbehind: true,
              inside: Prism.languages.groovy
            }
          });
          env.classes.push(delimiter === '/' ? 'regex' : 'gstring');
        }
      }
    });
  }

  // @ts-nocheck
  textile.displayName = 'textile';
  textile.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function textile(Prism) {
    Prism.register(markup)
    ;(function (Prism) {
      // We don't allow for pipes inside parentheses
      // to not break table pattern |(. foo |). bar |
      var modifierRegex = /\([^|()\n]+\)|\[[^\]\n]+\]|\{[^}\n]+\}/.source; // Opening and closing parentheses which are not a modifier
      // This pattern is necessary to prevent exponential backtracking
      var parenthesesRegex = /\)|\((?![^|()\n]+\))/.source;
      /**
       * @param {string} source
       * @param {string} [flags]
       */
      function withModifier(source, flags) {
        return RegExp(
          source
            .replace(/<MOD>/g, function () {
              return '(?:' + modifierRegex + ')'
            })
            .replace(/<PAR>/g, function () {
              return '(?:' + parenthesesRegex + ')'
            }),
          flags || ''
        )
      }
      var modifierTokens = {
        css: {
          pattern: /\{[^{}]+\}/,
          inside: {
            rest: Prism.languages.css
          }
        },
        'class-id': {
          pattern: /(\()[^()]+(?=\))/,
          lookbehind: true,
          alias: 'attr-value'
        },
        lang: {
          pattern: /(\[)[^\[\]]+(?=\])/,
          lookbehind: true,
          alias: 'attr-value'
        },
        // Anything else is punctuation (the first pattern is for row/col spans inside tables)
        punctuation: /[\\\/]\d+|\S/
      };
      var textile = (Prism.languages.textile = Prism.languages.extend('markup', {
        phrase: {
          pattern: /(^|\r|\n)\S[\s\S]*?(?=$|\r?\n\r?\n|\r\r)/,
          lookbehind: true,
          inside: {
            // h1. Header 1
            'block-tag': {
              pattern: withModifier(/^[a-z]\w*(?:<MOD>|<PAR>|[<>=])*\./.source),
              inside: {
                modifier: {
                  pattern: withModifier(
                    /(^[a-z]\w*)(?:<MOD>|<PAR>|[<>=])+(?=\.)/.source
                  ),
                  lookbehind: true,
                  inside: modifierTokens
                },
                tag: /^[a-z]\w*/,
                punctuation: /\.$/
              }
            },
            // # List item
            // * List item
            list: {
              pattern: withModifier(/^[*#]+<MOD>*\s+\S.*/.source, 'm'),
              inside: {
                modifier: {
                  pattern: withModifier(/(^[*#]+)<MOD>+/.source),
                  lookbehind: true,
                  inside: modifierTokens
                },
                punctuation: /^[*#]+/
              }
            },
            // | cell | cell | cell |
            table: {
              // Modifiers can be applied to the row: {color:red}.|1|2|3|
              // or the cell: |{color:red}.1|2|3|
              pattern: withModifier(
                /^(?:(?:<MOD>|<PAR>|[<>=^~])+\.\s*)?(?:\|(?:(?:<MOD>|<PAR>|[<>=^~_]|[\\/]\d+)+\.|(?!(?:<MOD>|<PAR>|[<>=^~_]|[\\/]\d+)+\.))[^|]*)+\|/
                  .source,
                'm'
              ),
              inside: {
                modifier: {
                  // Modifiers for rows after the first one are
                  // preceded by a pipe and a line feed
                  pattern: withModifier(
                    /(^|\|(?:\r?\n|\r)?)(?:<MOD>|<PAR>|[<>=^~_]|[\\/]\d+)+(?=\.)/
                      .source
                  ),
                  lookbehind: true,
                  inside: modifierTokens
                },
                punctuation: /\||^\./
              }
            },
            inline: {
              // eslint-disable-next-line regexp/no-super-linear-backtracking
              pattern: withModifier(
                /(^|[^a-zA-Z\d])(\*\*|__|\?\?|[*_%@+\-^~])<MOD>*.+?\2(?![a-zA-Z\d])/
                  .source
              ),
              lookbehind: true,
              inside: {
                // Note: superscripts and subscripts are not handled specifically
                // *bold*, **bold**
                bold: {
                  // eslint-disable-next-line regexp/no-super-linear-backtracking
                  pattern: withModifier(/(^(\*\*?)<MOD>*).+?(?=\2)/.source),
                  lookbehind: true
                },
                // _italic_, __italic__
                italic: {
                  // eslint-disable-next-line regexp/no-super-linear-backtracking
                  pattern: withModifier(/(^(__?)<MOD>*).+?(?=\2)/.source),
                  lookbehind: true
                },
                // ??cite??
                cite: {
                  // eslint-disable-next-line regexp/no-super-linear-backtracking
                  pattern: withModifier(/(^\?\?<MOD>*).+?(?=\?\?)/.source),
                  lookbehind: true,
                  alias: 'string'
                },
                // @code@
                code: {
                  // eslint-disable-next-line regexp/no-super-linear-backtracking
                  pattern: withModifier(/(^@<MOD>*).+?(?=@)/.source),
                  lookbehind: true,
                  alias: 'keyword'
                },
                // +inserted+
                inserted: {
                  // eslint-disable-next-line regexp/no-super-linear-backtracking
                  pattern: withModifier(/(^\+<MOD>*).+?(?=\+)/.source),
                  lookbehind: true
                },
                // -deleted-
                deleted: {
                  // eslint-disable-next-line regexp/no-super-linear-backtracking
                  pattern: withModifier(/(^-<MOD>*).+?(?=-)/.source),
                  lookbehind: true
                },
                // %span%
                span: {
                  // eslint-disable-next-line regexp/no-super-linear-backtracking
                  pattern: withModifier(/(^%<MOD>*).+?(?=%)/.source),
                  lookbehind: true
                },
                modifier: {
                  pattern: withModifier(
                    /(^\*\*|__|\?\?|[*_%@+\-^~])<MOD>+/.source
                  ),
                  lookbehind: true,
                  inside: modifierTokens
                },
                punctuation: /[*_%?@+\-^~]+/
              }
            },
            // [alias]http://example.com
            'link-ref': {
              pattern: /^\[[^\]]+\]\S+$/m,
              inside: {
                string: {
                  pattern: /(^\[)[^\]]+(?=\])/,
                  lookbehind: true
                },
                url: {
                  pattern: /(^\])\S+$/,
                  lookbehind: true
                },
                punctuation: /[\[\]]/
              }
            },
            // "text":http://example.com
            // "text":link-ref
            link: {
              // eslint-disable-next-line regexp/no-super-linear-backtracking
              pattern: withModifier(
                /"<MOD>*[^"]+":.+?(?=[^\w/]?(?:\s|$))/.source
              ),
              inside: {
                text: {
                  // eslint-disable-next-line regexp/no-super-linear-backtracking
                  pattern: withModifier(/(^"<MOD>*)[^"]+(?=")/.source),
                  lookbehind: true
                },
                modifier: {
                  pattern: withModifier(/(^")<MOD>+/.source),
                  lookbehind: true,
                  inside: modifierTokens
                },
                url: {
                  pattern: /(:).+/,
                  lookbehind: true
                },
                punctuation: /[":]/
              }
            },
            // !image.jpg!
            // !image.jpg(Title)!:http://example.com
            image: {
              pattern: withModifier(
                /!(?:<MOD>|<PAR>|[<>=])*(?![<>=])[^!\s()]+(?:\([^)]+\))?!(?::.+?(?=[^\w/]?(?:\s|$)))?/
                  .source
              ),
              inside: {
                source: {
                  pattern: withModifier(
                    /(^!(?:<MOD>|<PAR>|[<>=])*)(?![<>=])[^!\s()]+(?:\([^)]+\))?(?=!)/
                      .source
                  ),
                  lookbehind: true,
                  alias: 'url'
                },
                modifier: {
                  pattern: withModifier(/(^!)(?:<MOD>|<PAR>|[<>=])+/.source),
                  lookbehind: true,
                  inside: modifierTokens
                },
                url: {
                  pattern: /(:).+/,
                  lookbehind: true
                },
                punctuation: /[!:]/
              }
            },
            // Footnote[1]
            footnote: {
              pattern: /\b\[\d+\]/,
              alias: 'comment',
              inside: {
                punctuation: /\[|\]/
              }
            },
            // CSS(Cascading Style Sheet)
            acronym: {
              pattern: /\b[A-Z\d]+\([^)]+\)/,
              inside: {
                comment: {
                  pattern: /(\()[^()]+(?=\))/,
                  lookbehind: true
                },
                punctuation: /[()]/
              }
            },
            // Prism(C)
            mark: {
              pattern: /\b\((?:C|R|TM)\)/,
              alias: 'comment',
              inside: {
                punctuation: /[()]/
              }
            }
          }
        }
      }));
      var phraseInside = textile['phrase'].inside;
      var nestedPatterns = {
        inline: phraseInside['inline'],
        link: phraseInside['link'],
        image: phraseInside['image'],
        footnote: phraseInside['footnote'],
        acronym: phraseInside['acronym'],
        mark: phraseInside['mark']
      }; // Only allow alpha-numeric HTML tags, not XML tags
      textile.tag.pattern =
        /<\/?(?!\d)[a-z0-9]+(?:\s+[^\s>\/=]+(?:=(?:("|')(?:\\[\s\S]|(?!\1)[^\\])*\1|[^\s'">=]+))?)*\s*\/?>/i; // Allow some nesting
      var phraseInlineInside = phraseInside['inline'].inside;
      phraseInlineInside['bold'].inside = nestedPatterns;
      phraseInlineInside['italic'].inside = nestedPatterns;
      phraseInlineInside['inserted'].inside = nestedPatterns;
      phraseInlineInside['deleted'].inside = nestedPatterns;
      phraseInlineInside['span'].inside = nestedPatterns; // Allow some styles inside table cells
      var phraseTableInside = phraseInside['table'].inside;
      phraseTableInside['inline'] = nestedPatterns['inline'];
      phraseTableInside['link'] = nestedPatterns['link'];
      phraseTableInside['image'] = nestedPatterns['image'];
      phraseTableInside['footnote'] = nestedPatterns['footnote'];
      phraseTableInside['acronym'] = nestedPatterns['acronym'];
      phraseTableInside['mark'] = nestedPatterns['mark'];
    })(Prism);
  }

  // @ts-nocheck
  haml.displayName = 'haml';
  haml.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function haml(Prism) {
    Prism.register(ruby)
    /* TODO
  Handle multiline code after tag
  %foo= some |
  multiline |
  code |
  */
    ;(function (Prism) {
      Prism.languages.haml = {
        // Multiline stuff should appear before the rest
        'multiline-comment': {
          pattern:
            /((?:^|\r?\n|\r)([\t ]*))(?:\/|-#).*(?:(?:\r?\n|\r)\2[\t ].+)*/,
          lookbehind: true,
          alias: 'comment'
        },
        'multiline-code': [
          {
            pattern:
              /((?:^|\r?\n|\r)([\t ]*)(?:[~-]|[&!]?=)).*,[\t ]*(?:(?:\r?\n|\r)\2[\t ].*,[\t ]*)*(?:(?:\r?\n|\r)\2[\t ].+)/,
            lookbehind: true,
            inside: Prism.languages.ruby
          },
          {
            pattern:
              /((?:^|\r?\n|\r)([\t ]*)(?:[~-]|[&!]?=)).*\|[\t ]*(?:(?:\r?\n|\r)\2[\t ].*\|[\t ]*)*/,
            lookbehind: true,
            inside: Prism.languages.ruby
          }
        ],
        // See at the end of the file for known filters
        filter: {
          pattern:
            /((?:^|\r?\n|\r)([\t ]*)):[\w-]+(?:(?:\r?\n|\r)(?:\2[\t ].+|\s*?(?=\r?\n|\r)))+/,
          lookbehind: true,
          inside: {
            'filter-name': {
              pattern: /^:[\w-]+/,
              alias: 'symbol'
            }
          }
        },
        markup: {
          pattern: /((?:^|\r?\n|\r)[\t ]*)<.+/,
          lookbehind: true,
          inside: Prism.languages.markup
        },
        doctype: {
          pattern: /((?:^|\r?\n|\r)[\t ]*)!!!(?: .+)?/,
          lookbehind: true
        },
        tag: {
          // Allows for one nested group of braces
          pattern:
            /((?:^|\r?\n|\r)[\t ]*)[%.#][\w\-#.]*[\w\-](?:\([^)]+\)|\{(?:\{[^}]+\}|[^{}])+\}|\[[^\]]+\])*[\/<>]*/,
          lookbehind: true,
          inside: {
            attributes: [
              {
                // Lookbehind tries to prevent interpolations from breaking it all
                // Allows for one nested group of braces
                pattern: /(^|[^#])\{(?:\{[^}]+\}|[^{}])+\}/,
                lookbehind: true,
                inside: Prism.languages.ruby
              },
              {
                pattern: /\([^)]+\)/,
                inside: {
                  'attr-value': {
                    pattern: /(=\s*)(?:"(?:\\.|[^\\"\r\n])*"|[^)\s]+)/,
                    lookbehind: true
                  },
                  'attr-name': /[\w:-]+(?=\s*!?=|\s*[,)])/,
                  punctuation: /[=(),]/
                }
              },
              {
                pattern: /\[[^\]]+\]/,
                inside: Prism.languages.ruby
              }
            ],
            punctuation: /[<>]/
          }
        },
        code: {
          pattern: /((?:^|\r?\n|\r)[\t ]*(?:[~-]|[&!]?=)).+/,
          lookbehind: true,
          inside: Prism.languages.ruby
        },
        // Interpolations in plain text
        interpolation: {
          pattern: /#\{[^}]+\}/,
          inside: {
            delimiter: {
              pattern: /^#\{|\}$/,
              alias: 'punctuation'
            },
            ruby: {
              pattern: /[\s\S]+/,
              inside: Prism.languages.ruby
            }
          }
        },
        punctuation: {
          pattern: /((?:^|\r?\n|\r)[\t ]*)[~=\-&!]+/,
          lookbehind: true
        }
      };
      var filter_pattern =
        '((?:^|\\r?\\n|\\r)([\\t ]*)):{{filter_name}}(?:(?:\\r?\\n|\\r)(?:\\2[\\t ].+|\\s*?(?=\\r?\\n|\\r)))+'; // Non exhaustive list of available filters and associated languages
      var filters = [
        'css',
        {
          filter: 'coffee',
          language: 'coffeescript'
        },
        'erb',
        'javascript',
        'less',
        'markdown',
        'ruby',
        'scss',
        'textile'
      ];
      var all_filters = {};
      for (var i = 0, l = filters.length; i < l; i++) {
        var filter = filters[i];
        filter =
          typeof filter === 'string'
            ? {
                filter: filter,
                language: filter
              }
            : filter;
        if (Prism.languages[filter.language]) {
          all_filters['filter-' + filter.filter] = {
            pattern: RegExp(
              filter_pattern.replace('{{filter_name}}', function () {
                return filter.filter
              })
            ),
            lookbehind: true,
            inside: {
              'filter-name': {
                pattern: /^:[\w-]+/,
                alias: 'symbol'
              },
              text: {
                pattern: /[\s\S]+/,
                alias: [filter.language, 'language-' + filter.language],
                inside: Prism.languages[filter.language]
              }
            }
          };
        }
      }
      Prism.languages.insertBefore('haml', 'filter', all_filters);
    })(Prism);
  }

  // @ts-nocheck
  handlebars.displayName = 'handlebars';
  handlebars.aliases = ['hbs'];

  /** @type {import('../core.js').Syntax} */
  function handlebars(Prism) {
    Prism.register(markupTemplating)
    ;(function (Prism) {
      Prism.languages.handlebars = {
        comment: /\{\{![\s\S]*?\}\}/,
        delimiter: {
          pattern: /^\{\{\{?|\}\}\}?$/,
          alias: 'punctuation'
        },
        string: /(["'])(?:\\.|(?!\1)[^\\\r\n])*\1/,
        number: /\b0x[\dA-Fa-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee][+-]?\d+)?/,
        boolean: /\b(?:false|true)\b/,
        block: {
          pattern: /^(\s*(?:~\s*)?)[#\/]\S+?(?=\s*(?:~\s*)?$|\s)/,
          lookbehind: true,
          alias: 'keyword'
        },
        brackets: {
          pattern: /\[[^\]]+\]/,
          inside: {
            punctuation: /\[|\]/,
            variable: /[\s\S]+/
          }
        },
        punctuation: /[!"#%&':()*+,.\/;<=>@\[\\\]^`{|}~]/,
        variable: /[^!"#%&'()*+,\/;<=>@\[\\\]^`{|}~\s]+/
      };
      Prism.hooks.add('before-tokenize', function (env) {
        var handlebarsPattern = /\{\{\{[\s\S]+?\}\}\}|\{\{[\s\S]+?\}\}/g;
        Prism.languages['markup-templating'].buildPlaceholders(
          env,
          'handlebars',
          handlebarsPattern
        );
      });
      Prism.hooks.add('after-tokenize', function (env) {
        Prism.languages['markup-templating'].tokenizePlaceholders(
          env,
          'handlebars'
        );
      });
      Prism.languages.hbs = Prism.languages.handlebars;
    })(Prism);
  }

  // @ts-nocheck
  haskell.displayName = 'haskell';
  haskell.aliases = ['hs'];

  /** @type {import('../core.js').Syntax} */
  function haskell(Prism) {
    Prism.languages.haskell = {
      comment: {
        pattern:
          /(^|[^-!#$%*+=?&@|~.:<>^\\\/])(?:--(?:(?=.)[^-!#$%*+=?&@|~.:<>^\\\/].*|$)|\{-[\s\S]*?-\})/m,
        lookbehind: true
      },
      char: {
        pattern:
          /'(?:[^\\']|\\(?:[abfnrtv\\"'&]|\^[A-Z@[\]^_]|ACK|BEL|BS|CAN|CR|DC1|DC2|DC3|DC4|DEL|DLE|EM|ENQ|EOT|ESC|ETB|ETX|FF|FS|GS|HT|LF|NAK|NUL|RS|SI|SO|SOH|SP|STX|SUB|SYN|US|VT|\d+|o[0-7]+|x[0-9a-fA-F]+))'/,
        alias: 'string'
      },
      string: {
        pattern: /"(?:[^\\"]|\\(?:\S|\s+\\))*"/,
        greedy: true
      },
      keyword:
        /\b(?:case|class|data|deriving|do|else|if|in|infixl|infixr|instance|let|module|newtype|of|primitive|then|type|where)\b/,
      'import-statement': {
        // The imported or hidden names are not included in this import
        // statement. This is because we want to highlight those exactly like
        // we do for the names in the program.
        pattern:
          /(^[\t ]*)import\s+(?:qualified\s+)?(?:[A-Z][\w']*)(?:\.[A-Z][\w']*)*(?:\s+as\s+(?:[A-Z][\w']*)(?:\.[A-Z][\w']*)*)?(?:\s+hiding\b)?/m,
        lookbehind: true,
        inside: {
          keyword: /\b(?:as|hiding|import|qualified)\b/,
          punctuation: /\./
        }
      },
      // These are builtin variables only. Constructors are highlighted later as a constant.
      builtin:
        /\b(?:abs|acos|acosh|all|and|any|appendFile|approxRational|asTypeOf|asin|asinh|atan|atan2|atanh|basicIORun|break|catch|ceiling|chr|compare|concat|concatMap|const|cos|cosh|curry|cycle|decodeFloat|denominator|digitToInt|div|divMod|drop|dropWhile|either|elem|encodeFloat|enumFrom|enumFromThen|enumFromThenTo|enumFromTo|error|even|exp|exponent|fail|filter|flip|floatDigits|floatRadix|floatRange|floor|fmap|foldl|foldl1|foldr|foldr1|fromDouble|fromEnum|fromInt|fromInteger|fromIntegral|fromRational|fst|gcd|getChar|getContents|getLine|group|head|id|inRange|index|init|intToDigit|interact|ioError|isAlpha|isAlphaNum|isAscii|isControl|isDenormalized|isDigit|isHexDigit|isIEEE|isInfinite|isLower|isNaN|isNegativeZero|isOctDigit|isPrint|isSpace|isUpper|iterate|last|lcm|length|lex|lexDigits|lexLitChar|lines|log|logBase|lookup|map|mapM|mapM_|max|maxBound|maximum|maybe|min|minBound|minimum|mod|negate|not|notElem|null|numerator|odd|or|ord|otherwise|pack|pi|pred|primExitWith|print|product|properFraction|putChar|putStr|putStrLn|quot|quotRem|range|rangeSize|read|readDec|readFile|readFloat|readHex|readIO|readInt|readList|readLitChar|readLn|readOct|readParen|readSigned|reads|readsPrec|realToFrac|recip|rem|repeat|replicate|return|reverse|round|scaleFloat|scanl|scanl1|scanr|scanr1|seq|sequence|sequence_|show|showChar|showInt|showList|showLitChar|showParen|showSigned|showString|shows|showsPrec|significand|signum|sin|sinh|snd|sort|span|splitAt|sqrt|subtract|succ|sum|tail|take|takeWhile|tan|tanh|threadToIOResult|toEnum|toInt|toInteger|toLower|toRational|toUpper|truncate|uncurry|undefined|unlines|until|unwords|unzip|unzip3|userError|words|writeFile|zip|zip3|zipWith|zipWith3)\b/,
      // decimal integers and floating point numbers | octal integers | hexadecimal integers
      number: /\b(?:\d+(?:\.\d+)?(?:e[+-]?\d+)?|0o[0-7]+|0x[0-9a-f]+)\b/i,
      operator: [
        {
          // infix operator
          pattern: /`(?:[A-Z][\w']*\.)*[_a-z][\w']*`/,
          greedy: true
        },
        {
          // function composition
          pattern: /(\s)\.(?=\s)/,
          lookbehind: true
        }, // Most of this is needed because of the meaning of a single '.'.
        // If it stands alone freely, it is the function composition.
        // It may also be a separator between a module name and an identifier => no
        // operator. If it comes together with other special characters it is an
        // operator too.
        //
        // This regex means: /[-!#$%*+=?&@|~.:<>^\\\/]+/ without /\./.
        /[-!#$%*+=?&@|~:<>^\\\/][-!#$%*+=?&@|~.:<>^\\\/]*|\.[-!#$%*+=?&@|~.:<>^\\\/]+/
      ],
      // In Haskell, nearly everything is a variable, do not highlight these.
      hvariable: {
        pattern: /\b(?:[A-Z][\w']*\.)*[_a-z][\w']*/,
        inside: {
          punctuation: /\./
        }
      },
      constant: {
        pattern: /\b(?:[A-Z][\w']*\.)*[A-Z][\w']*/,
        inside: {
          punctuation: /\./
        }
      },
      punctuation: /[{}[\];(),.:]/
    };
    Prism.languages.hs = Prism.languages.haskell;
  }

  // @ts-nocheck
  haxe.displayName = 'haxe';
  haxe.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function haxe(Prism) {
    Prism.register(clike);
    Prism.languages.haxe = Prism.languages.extend('clike', {
      string: {
        // Strings can be multi-line
        pattern: /"(?:[^"\\]|\\[\s\S])*"/,
        greedy: true
      },
      'class-name': [
        {
          pattern:
            /(\b(?:abstract|class|enum|extends|implements|interface|new|typedef)\s+)[A-Z_]\w*/,
          lookbehind: true
        }, // based on naming convention
        /\b[A-Z]\w*/
      ],
      // The final look-ahead prevents highlighting of keywords if expressions such as "haxe.macro.Expr"
      keyword:
        /\bthis\b|\b(?:abstract|as|break|case|cast|catch|class|continue|default|do|dynamic|else|enum|extends|extern|final|for|from|function|if|implements|import|in|inline|interface|macro|new|null|operator|overload|override|package|private|public|return|static|super|switch|throw|to|try|typedef|untyped|using|var|while)(?!\.)\b/,
      function: {
        pattern: /\b[a-z_]\w*(?=\s*(?:<[^<>]*>\s*)?\()/i,
        greedy: true
      },
      operator: /\.{3}|\+\+|--|&&|\|\||->|=>|(?:<<?|>{1,3}|[-+*/%!=&|^])=?|[?:~]/
    });
    Prism.languages.insertBefore('haxe', 'string', {
      'string-interpolation': {
        pattern: /'(?:[^'\\]|\\[\s\S])*'/,
        greedy: true,
        inside: {
          interpolation: {
            pattern: /(^|[^\\])\$(?:\w+|\{[^{}]+\})/,
            lookbehind: true,
            inside: {
              'interpolation-punctuation': {
                pattern: /^\$\{?|\}$/,
                alias: 'punctuation'
              },
              expression: {
                pattern: /[\s\S]+/,
                inside: Prism.languages.haxe
              }
            }
          },
          string: /[\s\S]+/
        }
      }
    });
    Prism.languages.insertBefore('haxe', 'class-name', {
      regex: {
        pattern: /~\/(?:[^\/\\\r\n]|\\.)+\/[a-z]*/,
        greedy: true,
        inside: {
          'regex-flags': /\b[a-z]+$/,
          'regex-source': {
            pattern: /^(~\/)[\s\S]+(?=\/$)/,
            lookbehind: true,
            alias: 'language-regex',
            inside: Prism.languages.regex
          },
          'regex-delimiter': /^~\/|\/$/
        }
      }
    });
    Prism.languages.insertBefore('haxe', 'keyword', {
      preprocessor: {
        pattern: /#(?:else|elseif|end|if)\b.*/,
        alias: 'property'
      },
      metadata: {
        pattern: /@:?[\w.]+/,
        alias: 'symbol'
      },
      reification: {
        pattern: /\$(?:\w+|(?=\{))/,
        alias: 'important'
      }
    });
  }

  // @ts-nocheck
  hcl.displayName = 'hcl';
  hcl.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function hcl(Prism) {
    Prism.languages.hcl = {
      comment: /(?:\/\/|#).*|\/\*[\s\S]*?(?:\*\/|$)/,
      heredoc: {
        pattern: /<<-?(\w+\b)[\s\S]*?^[ \t]*\1/m,
        greedy: true,
        alias: 'string'
      },
      keyword: [
        {
          pattern:
            /(?:data|resource)\s+(?:"(?:\\[\s\S]|[^\\"])*")(?=\s+"[\w-]+"\s+\{)/i,
          inside: {
            type: {
              pattern: /(resource|data|\s+)(?:"(?:\\[\s\S]|[^\\"])*")/i,
              lookbehind: true,
              alias: 'variable'
            }
          }
        },
        {
          pattern:
            /(?:backend|module|output|provider|provisioner|variable)\s+(?:[\w-]+|"(?:\\[\s\S]|[^\\"])*")\s+(?=\{)/i,
          inside: {
            type: {
              pattern:
                /(backend|module|output|provider|provisioner|variable)\s+(?:[\w-]+|"(?:\\[\s\S]|[^\\"])*")\s+/i,
              lookbehind: true,
              alias: 'variable'
            }
          }
        },
        /[\w-]+(?=\s+\{)/
      ],
      property: [/[-\w\.]+(?=\s*=(?!=))/, /"(?:\\[\s\S]|[^\\"])+"(?=\s*[:=])/],
      string: {
        pattern:
          /"(?:[^\\$"]|\\[\s\S]|\$(?:(?=")|\$+(?!\$)|[^"${])|\$\{(?:[^{}"]|"(?:[^\\"]|\\[\s\S])*")*\})*"/,
        greedy: true,
        inside: {
          interpolation: {
            pattern: /(^|[^$])\$\{(?:[^{}"]|"(?:[^\\"]|\\[\s\S])*")*\}/,
            lookbehind: true,
            inside: {
              type: {
                pattern:
                  /(\b(?:count|data|local|module|path|self|terraform|var)\b\.)[\w\*]+/i,
                lookbehind: true,
                alias: 'variable'
              },
              keyword: /\b(?:count|data|local|module|path|self|terraform|var)\b/i,
              function: /\w+(?=\()/,
              string: {
                pattern: /"(?:\\[\s\S]|[^\\"])*"/,
                greedy: true
              },
              number: /\b0x[\da-f]+\b|\b\d+(?:\.\d*)?(?:e[+-]?\d+)?/i,
              punctuation: /[!\$#%&'()*+,.\/;<=>@\[\\\]^`{|}~?:]/
            }
          }
        }
      },
      number: /\b0x[\da-f]+\b|\b\d+(?:\.\d*)?(?:e[+-]?\d+)?/i,
      boolean: /\b(?:false|true)\b/i,
      punctuation: /[=\[\]{}]/
    };
  }

  // @ts-nocheck
  hlsl.displayName = 'hlsl';
  hlsl.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function hlsl(Prism) {
    Prism.register(c$1);
    Prism.languages.hlsl = Prism.languages.extend('c', {
      // Regarding keywords and class names:
      // The list of all keywords was split into 'keyword' and 'class-name' tokens based on whether they are capitalized.
      // https://docs.microsoft.com/en-us/windows/win32/direct3dhlsl/dx-graphics-hlsl-appendix-keywords
      // https://docs.microsoft.com/en-us/windows/win32/direct3dhlsl/dx-graphics-hlsl-appendix-reserved-words
      'class-name': [
        Prism.languages.c['class-name'],
        /\b(?:AppendStructuredBuffer|BlendState|Buffer|ByteAddressBuffer|CompileShader|ComputeShader|ConsumeStructuredBuffer|DepthStencilState|DepthStencilView|DomainShader|GeometryShader|Hullshader|InputPatch|LineStream|OutputPatch|PixelShader|PointStream|RWBuffer|RWByteAddressBuffer|RWStructuredBuffer|RWTexture(?:1D|1DArray|2D|2DArray|3D)|RasterizerState|RenderTargetView|SamplerComparisonState|SamplerState|StructuredBuffer|Texture(?:1D|1DArray|2D|2DArray|2DMS|2DMSArray|3D|Cube|CubeArray)|TriangleStream|VertexShader)\b/
      ],
      keyword: [
        // HLSL keyword
        /\b(?:asm|asm_fragment|auto|break|case|catch|cbuffer|centroid|char|class|column_major|compile|compile_fragment|const|const_cast|continue|default|delete|discard|do|dynamic_cast|else|enum|explicit|export|extern|for|friend|fxgroup|goto|groupshared|if|in|inline|inout|interface|line|lineadj|linear|long|matrix|mutable|namespace|new|nointerpolation|noperspective|operator|out|packoffset|pass|pixelfragment|point|precise|private|protected|public|register|reinterpret_cast|return|row_major|sample|sampler|shared|short|signed|sizeof|snorm|stateblock|stateblock_state|static|static_cast|string|struct|switch|tbuffer|technique|technique10|technique11|template|texture|this|throw|triangle|triangleadj|try|typedef|typename|uniform|union|unorm|unsigned|using|vector|vertexfragment|virtual|void|volatile|while)\b/, // scalar, vector, and matrix types
        /\b(?:bool|double|dword|float|half|int|min(?:10float|12int|16(?:float|int|uint))|uint)(?:[1-4](?:x[1-4])?)?\b/
      ],
      // https://docs.microsoft.com/en-us/windows/win32/direct3dhlsl/dx-graphics-hlsl-appendix-grammar#floating-point-numbers
      number:
        /(?:(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[eE][+-]?\d+)?|\b0x[\da-fA-F]+)[fFhHlLuU]?\b/,
      boolean: /\b(?:false|true)\b/
    });
  }

  // @ts-nocheck
  hoon.displayName = 'hoon';
  hoon.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function hoon(Prism) {
    Prism.languages.hoon = {
      comment: {
        pattern: /::.*/,
        greedy: true
      },
      string: {
        pattern: /"[^"]*"|'[^']*'/,
        greedy: true
      },
      constant: /%(?:\.[ny]|[\w-]+)/,
      'class-name': /@(?:[a-z0-9-]*[a-z0-9])?|\*/i,
      function: /(?:\+[-+] {2})?(?:[a-z](?:[a-z0-9-]*[a-z0-9])?)/,
      keyword:
        /\.[\^\+\*=\?]|![><:\.=\?!]|=[>|:,\.\-\^<+;/~\*\?]|\?[>|:\.\-\^<\+&~=@!]|\|[\$_%:\.\-\^~\*=@\?]|\+[|\$\+\*]|:[_\-\^\+~\*]|%[_:\.\-\^\+~\*=]|\^[|:\.\-\+&~\*=\?]|\$[|_%:<>\-\^&~@=\?]|;[:<\+;\/~\*=]|~[>|\$_%<\+\/&=\?!]|--|==/
    };
  }

  // @ts-nocheck
  hpkp.displayName = 'hpkp';
  hpkp.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function hpkp(Prism) {
    /**
     * Original by Scott Helme.
     *
     * Reference: https://scotthelme.co.uk/hpkp-cheat-sheet/
     */
    Prism.languages.hpkp = {
      directive: {
        pattern:
          /\b(?:includeSubDomains|max-age|pin-sha256|preload|report-to|report-uri|strict)(?=[\s;=]|$)/i,
        alias: 'property'
      },
      operator: /=/,
      punctuation: /;/
    };
  }

  // @ts-nocheck
  hsts.displayName = 'hsts';
  hsts.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function hsts(Prism) {
    /**
     * Original by Scott Helme.
     *
     * Reference: https://scotthelme.co.uk/hsts-cheat-sheet/
     */
    Prism.languages.hsts = {
      directive: {
        pattern: /\b(?:includeSubDomains|max-age|preload)(?=[\s;=]|$)/i,
        alias: 'property'
      },
      operator: /=/,
      punctuation: /;/
    };
  }

  // @ts-nocheck
  uri.displayName = 'uri';
  uri.aliases = ['url'];

  /** @type {import('../core.js').Syntax} */
  function uri(Prism) {
    // https://tools.ietf.org/html/rfc3986#appendix-A
    Prism.languages.uri = {
      scheme: {
        pattern: /^[a-z][a-z0-9+.-]*:/im,
        greedy: true,
        inside: {
          'scheme-delimiter': /:$/
        }
      },
      fragment: {
        pattern: /#[\w\-.~!$&'()*+,;=%:@/?]*/,
        inside: {
          'fragment-delimiter': /^#/
        }
      },
      query: {
        pattern: /\?[\w\-.~!$&'()*+,;=%:@/?]*/,
        inside: {
          'query-delimiter': {
            pattern: /^\?/,
            greedy: true
          },
          'pair-delimiter': /[&;]/,
          pair: {
            pattern: /^[^=][\s\S]*/,
            inside: {
              key: /^[^=]+/,
              value: {
                pattern: /(^=)[\s\S]+/,
                lookbehind: true
              }
            }
          }
        }
      },
      authority: {
        pattern: RegExp(
          /^\/\//.source + // [ userinfo "@" ]
            /(?:[\w\-.~!$&'()*+,;=%:]*@)?/.source + // host
            ('(?:' + // IP-literal
              /\[(?:[0-9a-fA-F:.]{2,48}|v[0-9a-fA-F]+\.[\w\-.~!$&'()*+,;=]+)\]/
                .source +
              '|' + // IPv4address or registered name
              /[\w\-.~!$&'()*+,;=%]*/.source +
              ')') + // [ ":" port ]
            /(?::\d*)?/.source,
          'm'
        ),
        inside: {
          'authority-delimiter': /^\/\//,
          'user-info-segment': {
            pattern: /^[\w\-.~!$&'()*+,;=%:]*@/,
            inside: {
              'user-info-delimiter': /@$/,
              'user-info': /^[\w\-.~!$&'()*+,;=%:]+/
            }
          },
          'port-segment': {
            pattern: /:\d*$/,
            inside: {
              'port-delimiter': /^:/,
              port: /^\d+/
            }
          },
          host: {
            pattern: /[\s\S]+/,
            inside: {
              'ip-literal': {
                pattern: /^\[[\s\S]+\]$/,
                inside: {
                  'ip-literal-delimiter': /^\[|\]$/,
                  'ipv-future': /^v[\s\S]+/,
                  'ipv6-address': /^[\s\S]+/
                }
              },
              'ipv4-address':
                /^(?:(?:[03-9]\d?|[12]\d{0,2})\.){3}(?:[03-9]\d?|[12]\d{0,2})$/
            }
          }
        }
      },
      path: {
        pattern: /^[\w\-.~!$&'()*+,;=%:@/]+/m,
        inside: {
          'path-separator': /\//
        }
      }
    };
    Prism.languages.url = Prism.languages.uri;
  }

  // @ts-nocheck
  http.displayName = 'http';
  http.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function http(Prism) {
  (function (Prism) {
      /**
       * @param {string} name
       * @returns {RegExp}
       */
      function headerValueOf(name) {
        return RegExp('(^(?:' + name + '):[ \t]*(?![ \t]))[^]+', 'i')
      }
      Prism.languages.http = {
        'request-line': {
          pattern:
            /^(?:CONNECT|DELETE|GET|HEAD|OPTIONS|PATCH|POST|PRI|PUT|SEARCH|TRACE)\s(?:https?:\/\/|\/)\S*\sHTTP\/[\d.]+/m,
          inside: {
            // HTTP Method
            method: {
              pattern: /^[A-Z]+\b/,
              alias: 'property'
            },
            // Request Target e.g. http://example.com, /path/to/file
            'request-target': {
              pattern: /^(\s)(?:https?:\/\/|\/)\S*(?=\s)/,
              lookbehind: true,
              alias: 'url',
              inside: Prism.languages.uri
            },
            // HTTP Version
            'http-version': {
              pattern: /^(\s)HTTP\/[\d.]+/,
              lookbehind: true,
              alias: 'property'
            }
          }
        },
        'response-status': {
          pattern: /^HTTP\/[\d.]+ \d+ .+/m,
          inside: {
            // HTTP Version
            'http-version': {
              pattern: /^HTTP\/[\d.]+/,
              alias: 'property'
            },
            // Status Code
            'status-code': {
              pattern: /^(\s)\d+(?=\s)/,
              lookbehind: true,
              alias: 'number'
            },
            // Reason Phrase
            'reason-phrase': {
              pattern: /^(\s).+/,
              lookbehind: true,
              alias: 'string'
            }
          }
        },
        header: {
          pattern: /^[\w-]+:.+(?:(?:\r\n?|\n)[ \t].+)*/m,
          inside: {
            'header-value': [
              {
                pattern: headerValueOf(/Content-Security-Policy/.source),
                lookbehind: true,
                alias: ['csp', 'languages-csp'],
                inside: Prism.languages.csp
              },
              {
                pattern: headerValueOf(/Public-Key-Pins(?:-Report-Only)?/.source),
                lookbehind: true,
                alias: ['hpkp', 'languages-hpkp'],
                inside: Prism.languages.hpkp
              },
              {
                pattern: headerValueOf(/Strict-Transport-Security/.source),
                lookbehind: true,
                alias: ['hsts', 'languages-hsts'],
                inside: Prism.languages.hsts
              },
              {
                pattern: headerValueOf(/[^:]+/.source),
                lookbehind: true
              }
            ],
            'header-name': {
              pattern: /^[^:]+/,
              alias: 'keyword'
            },
            punctuation: /^:/
          }
        }
      }; // Create a mapping of Content-Type headers to language definitions
      var langs = Prism.languages;
      var httpLanguages = {
        'application/javascript': langs.javascript,
        'application/json': langs.json || langs.javascript,
        'application/xml': langs.xml,
        'text/xml': langs.xml,
        'text/html': langs.html,
        'text/css': langs.css,
        'text/plain': langs.plain
      }; // Declare which types can also be suffixes
      var suffixTypes = {
        'application/json': true,
        'application/xml': true
      };
      /**
       * Returns a pattern for the given content type which matches it and any type which has it as a suffix.
       *
       * @param {string} contentType
       * @returns {string}
       */
      function getSuffixPattern(contentType) {
        var suffix = contentType.replace(/^[a-z]+\//, '');
        var suffixPattern = '\\w+/(?:[\\w.-]+\\+)+' + suffix + '(?![+\\w.-])';
        return '(?:' + contentType + '|' + suffixPattern + ')'
      } // Insert each content type parser that has its associated language
      // currently loaded.
      var options;
      for (var contentType in httpLanguages) {
        if (httpLanguages[contentType]) {
          options = options || {};
          var pattern = suffixTypes[contentType]
            ? getSuffixPattern(contentType)
            : contentType;
          options[contentType.replace(/\//g, '-')] = {
            pattern: RegExp(
              '(' +
                /content-type:\s*/.source +
                pattern +
                /(?:(?:\r\n?|\n)[\w-].*)*(?:\r(?:\n|(?!\n))|\n)/.source +
                ')' + // This is a little interesting:
                // The HTTP format spec required 1 empty line before the body to make everything unambiguous.
                // However, when writing code by hand (e.g. to display on a website) people can forget about this,
                // so we want to be liberal here. We will allow the empty line to be omitted if the first line of
                // the body does not start with a [\w-] character (as headers do).
                /[^ \t\w-][\s\S]*/.source,
              'i'
            ),
            lookbehind: true,
            inside: httpLanguages[contentType]
          };
        }
      }
      if (options) {
        Prism.languages.insertBefore('http', 'header', options);
      }
    })(Prism);
  }

  // @ts-nocheck
  ichigojam.displayName = 'ichigojam';
  ichigojam.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function ichigojam(Prism) {
    // according to the offical reference (EN)
    // https://ichigojam.net/IchigoJam-en.html
    Prism.languages.ichigojam = {
      comment: /(?:\B'|REM)(?:[^\n\r]*)/i,
      string: {
        pattern: /"(?:""|[!#$%&'()*,\/:;<=>?^\w +\-.])*"/,
        greedy: true
      },
      number: /\B#[0-9A-F]+|\B`[01]+|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:E[+-]?\d+)?/i,
      keyword:
        /\b(?:BEEP|BPS|CASE|CLEAR|CLK|CLO|CLP|CLS|CLT|CLV|CONT|COPY|ELSE|END|FILE|FILES|FOR|GOSUB|GOTO|GSB|IF|INPUT|KBD|LED|LET|LIST|LOAD|LOCATE|LRUN|NEW|NEXT|OUT|PLAY|POKE|PRINT|PWM|REM|RENUM|RESET|RETURN|RIGHT|RTN|RUN|SAVE|SCROLL|SLEEP|SRND|STEP|STOP|SUB|TEMPO|THEN|TO|UART|VIDEO|WAIT)(?:\$|\b)/i,
      function:
        /\b(?:ABS|ANA|ASC|BIN|BTN|DEC|END|FREE|HELP|HEX|I2CR|I2CW|IN|INKEY|LEN|LINE|PEEK|RND|SCR|SOUND|STR|TICK|USR|VER|VPEEK|ZER)(?:\$|\b)/i,
      label: /(?:\B@\S+)/,
      operator: /<[=>]?|>=?|\|\||&&|[+\-*\/=|&^~!]|\b(?:AND|NOT|OR)\b/i,
      punctuation: /[\[,;:()\]]/
    };
  }

  // @ts-nocheck
  icon.displayName = 'icon';
  icon.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function icon(Prism) {
    Prism.languages.icon = {
      comment: /#.*/,
      string: {
        pattern: /(["'])(?:(?!\1)[^\\\r\n_]|\\.|_(?!\1)(?:\r\n|[\s\S]))*\1/,
        greedy: true
      },
      number: /\b(?:\d+r[a-z\d]+|\d+(?:\.\d+)?(?:e[+-]?\d+)?)\b|\.\d+\b/i,
      'builtin-keyword': {
        pattern:
          /&(?:allocated|ascii|clock|collections|cset|current|date|dateline|digits|dump|e|error(?:number|text|value)?|errout|fail|features|file|host|input|lcase|letters|level|line|main|null|output|phi|pi|pos|progname|random|regions|source|storage|subject|time|trace|ucase|version)\b/,
        alias: 'variable'
      },
      directive: {
        pattern: /\$\w+/,
        alias: 'builtin'
      },
      keyword:
        /\b(?:break|by|case|create|default|do|else|end|every|fail|global|if|initial|invocable|link|local|next|not|of|procedure|record|repeat|return|static|suspend|then|to|until|while)\b/,
      function: /\b(?!\d)\w+(?=\s*[({]|\s*!\s*\[)/,
      operator:
        /[+-]:(?!=)|(?:[\/?@^%&]|\+\+?|--?|==?=?|~==?=?|\*\*?|\|\|\|?|<(?:->?|<?=?)|>>?=?)(?::=)?|:(?:=:?)?|[!.\\|~]/,
      punctuation: /[\[\](){},;]/
    };
  }

  // @ts-nocheck
  icuMessageFormat.displayName = 'icu-message-format';
  icuMessageFormat.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function icuMessageFormat(Prism) {
  (function (Prism) {
      /**
       * @param {string} source
       * @param {number} level
       * @returns {string}
       */
      function nested(source, level) {
        if (level <= 0) {
          return /[]/.source
        } else {
          return source.replace(/<SELF>/g, function () {
            return nested(source, level - 1)
          })
        }
      }
      var stringPattern = /'[{}:=,](?:[^']|'')*'(?!')/;
      var escape = {
        pattern: /''/,
        greedy: true,
        alias: 'operator'
      };
      var string = {
        pattern: stringPattern,
        greedy: true,
        inside: {
          escape: escape
        }
      };
      var argumentSource = nested(
        /\{(?:[^{}']|'(?![{},'])|''|<STR>|<SELF>)*\}/.source.replace(
          /<STR>/g,
          function () {
            return stringPattern.source
          }
        ),
        8
      );
      var nestedMessage = {
        pattern: RegExp(argumentSource),
        inside: {
          message: {
            pattern: /^(\{)[\s\S]+(?=\}$)/,
            lookbehind: true,
            inside: null // see below
          },
          'message-delimiter': {
            pattern: /./,
            alias: 'punctuation'
          }
        }
      };
      Prism.languages['icu-message-format'] = {
        argument: {
          pattern: RegExp(argumentSource),
          greedy: true,
          inside: {
            content: {
              pattern: /^(\{)[\s\S]+(?=\}$)/,
              lookbehind: true,
              inside: {
                'argument-name': {
                  pattern: /^(\s*)[^{}:=,\s]+/,
                  lookbehind: true
                },
                'choice-style': {
                  // https://unicode-org.github.io/icu-docs/apidoc/released/icu4c/classicu_1_1ChoiceFormat.html#details
                  pattern: /^(\s*,\s*choice\s*,\s*)\S(?:[\s\S]*\S)?/,
                  lookbehind: true,
                  inside: {
                    punctuation: /\|/,
                    range: {
                      pattern: /^(\s*)[+-]?(?:\d+(?:\.\d*)?|\u221e)\s*[<#\u2264]/,
                      lookbehind: true,
                      inside: {
                        operator: /[<#\u2264]/,
                        number: /\S+/
                      }
                    },
                    rest: null // see below
                  }
                },
                'plural-style': {
                  // https://unicode-org.github.io/icu-docs/apidoc/released/icu4j/com/ibm/icu/text/PluralFormat.html#:~:text=Patterns%20and%20Their%20Interpretation
                  pattern:
                    /^(\s*,\s*(?:plural|selectordinal)\s*,\s*)\S(?:[\s\S]*\S)?/,
                  lookbehind: true,
                  inside: {
                    offset: /^offset:\s*\d+/,
                    'nested-message': nestedMessage,
                    selector: {
                      pattern: /=\d+|[^{}:=,\s]+/,
                      inside: {
                        keyword: /^(?:few|many|one|other|two|zero)$/
                      }
                    }
                  }
                },
                'select-style': {
                  // https://unicode-org.github.io/icu-docs/apidoc/released/icu4j/com/ibm/icu/text/SelectFormat.html#:~:text=Patterns%20and%20Their%20Interpretation
                  pattern: /^(\s*,\s*select\s*,\s*)\S(?:[\s\S]*\S)?/,
                  lookbehind: true,
                  inside: {
                    'nested-message': nestedMessage,
                    selector: {
                      pattern: /[^{}:=,\s]+/,
                      inside: {
                        keyword: /^other$/
                      }
                    }
                  }
                },
                keyword: /\b(?:choice|plural|select|selectordinal)\b/,
                'arg-type': {
                  pattern: /\b(?:date|duration|number|ordinal|spellout|time)\b/,
                  alias: 'keyword'
                },
                'arg-skeleton': {
                  pattern: /(,\s*)::[^{}:=,\s]+/,
                  lookbehind: true
                },
                'arg-style': {
                  pattern:
                    /(,\s*)(?:currency|full|integer|long|medium|percent|short)(?=\s*$)/,
                  lookbehind: true
                },
                'arg-style-text': {
                  pattern: RegExp(
                    /(^\s*,\s*(?=\S))/.source +
                      nested(/(?:[^{}']|'[^']*'|\{(?:<SELF>)?\})+/.source, 8) +
                      '$'
                  ),
                  lookbehind: true,
                  alias: 'string'
                },
                punctuation: /,/
              }
            },
            'argument-delimiter': {
              pattern: /./,
              alias: 'operator'
            }
          }
        },
        escape: escape,
        string: string
      };
      nestedMessage.inside.message.inside = Prism.languages['icu-message-format'];
      Prism.languages['icu-message-format'].argument.inside.content.inside[
        'choice-style'
      ].inside.rest = Prism.languages['icu-message-format'];
    })(Prism);
  }

  // @ts-nocheck
  idris.displayName = 'idris';
  idris.aliases = ['idr'];

  /** @type {import('../core.js').Syntax} */
  function idris(Prism) {
    Prism.register(haskell);
    Prism.languages.idris = Prism.languages.extend('haskell', {
      comment: {
        pattern: /(?:(?:--|\|\|\|).*$|\{-[\s\S]*?-\})/m
      },
      keyword:
        /\b(?:Type|case|class|codata|constructor|corecord|data|do|dsl|else|export|if|implementation|implicit|import|impossible|in|infix|infixl|infixr|instance|interface|let|module|mutual|namespace|of|parameters|partial|postulate|private|proof|public|quoteGoal|record|rewrite|syntax|then|total|using|where|with)\b/,
      builtin: undefined
    });
    Prism.languages.insertBefore('idris', 'keyword', {
      'import-statement': {
        pattern: /(^\s*import\s+)(?:[A-Z][\w']*)(?:\.[A-Z][\w']*)*/m,
        lookbehind: true,
        inside: {
          punctuation: /\./
        }
      }
    });
    Prism.languages.idr = Prism.languages.idris;
  }

  // @ts-nocheck
  ignore.displayName = 'ignore';
  ignore.aliases = ['gitignore', 'hgignore', 'npmignore'];

  /** @type {import('../core.js').Syntax} */
  function ignore(Prism) {
  (function (Prism) {
      Prism.languages.ignore = {
        // https://git-scm.com/docs/gitignore
        comment: /^#.*/m,
        entry: {
          pattern: /\S(?:.*(?:(?:\\ )|\S))?/,
          alias: 'string',
          inside: {
            operator: /^!|\*\*?|\?/,
            regex: {
              pattern: /(^|[^\\])\[[^\[\]]*\]/,
              lookbehind: true
            },
            punctuation: /\//
          }
        }
      };
      Prism.languages.gitignore = Prism.languages.ignore;
      Prism.languages.hgignore = Prism.languages.ignore;
      Prism.languages.npmignore = Prism.languages.ignore;
    })(Prism);
  }

  // @ts-nocheck
  inform7.displayName = 'inform7';
  inform7.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function inform7(Prism) {
    Prism.languages.inform7 = {
      string: {
        pattern: /"[^"]*"/,
        inside: {
          substitution: {
            pattern: /\[[^\[\]]+\]/,
            inside: {
              delimiter: {
                pattern: /\[|\]/,
                alias: 'punctuation'
              } // See rest below
            }
          }
        }
      },
      comment: {
        pattern: /\[[^\[\]]+\]/,
        greedy: true
      },
      title: {
        pattern: /^[ \t]*(?:book|chapter|part(?! of)|section|table|volume)\b.+/im,
        alias: 'important'
      },
      number: {
        pattern:
          /(^|[^-])(?:\b\d+(?:\.\d+)?(?:\^\d+)?(?:(?!\d)\w+)?|\b(?:eight|eleven|five|four|nine|one|seven|six|ten|three|twelve|two))\b(?!-)/i,
        lookbehind: true
      },
      verb: {
        pattern:
          /(^|[^-])\b(?:answering|applying to|are|asking|attacking|be(?:ing)?|burning|buying|called|carries|carry(?! out)|carrying|climbing|closing|conceal(?:ing|s)?|consulting|contain(?:ing|s)?|cutting|drinking|dropping|eating|enclos(?:es?|ing)|entering|examining|exiting|getting|giving|going|ha(?:s|ve|ving)|hold(?:ing|s)?|impl(?:ies|y)|incorporat(?:es?|ing)|inserting|is|jumping|kissing|listening|locking|looking|mean(?:ing|s)?|opening|provid(?:es?|ing)|pulling|pushing|putting|relat(?:es?|ing)|removing|searching|see(?:ing|s)?|setting|showing|singing|sleeping|smelling|squeezing|support(?:ing|s)?|swearing|switching|taking|tasting|telling|thinking|throwing|touching|turning|tying|unlock(?:ing|s)?|var(?:ies|y|ying)|waiting|waking|waving|wear(?:ing|s)?)\b(?!-)/i,
        lookbehind: true,
        alias: 'operator'
      },
      keyword: {
        pattern:
          /(^|[^-])\b(?:after|before|carry out|check|continue the action|definition(?= *:)|do nothing|else|end (?:if|the story|unless)|every turn|if|include|instead(?: of)?|let|move|no|now|otherwise|repeat|report|resume the story|rule for|running through|say(?:ing)?|stop the action|test|try(?:ing)?|understand|unless|use|when|while|yes)\b(?!-)/i,
        lookbehind: true
      },
      property: {
        pattern:
          /(^|[^-])\b(?:adjacent(?! to)|carried|closed|concealed|contained|dark|described|edible|empty|enclosed|enterable|even|female|fixed in place|full|handled|held|improper-named|incorporated|inedible|invisible|lighted|lit|lock(?:able|ed)|male|marked for listing|mentioned|negative|neuter|non-(?:empty|full|recurring)|odd|opaque|open(?:able)?|plural-named|portable|positive|privately-named|proper-named|provided|publically-named|pushable between rooms|recurring|related|rubbing|scenery|seen|singular-named|supported|swinging|switch(?:able|ed(?: off| on)?)|touch(?:able|ed)|transparent|unconcealed|undescribed|unlit|unlocked|unmarked for listing|unmentioned|unopenable|untouchable|unvisited|variable|visible|visited|wearable|worn)\b(?!-)/i,
        lookbehind: true,
        alias: 'symbol'
      },
      position: {
        pattern:
          /(^|[^-])\b(?:above|adjacent to|back side of|below|between|down|east|everywhere|front side|here|in|inside(?: from)?|north(?:east|west)?|nowhere|on(?: top of)?|other side|outside(?: from)?|parts? of|regionally in|south(?:east|west)?|through|up|west|within)\b(?!-)/i,
        lookbehind: true,
        alias: 'keyword'
      },
      type: {
        pattern:
          /(^|[^-])\b(?:actions?|activit(?:ies|y)|actors?|animals?|backdrops?|containers?|devices?|directions?|doors?|holders?|kinds?|lists?|m[ae]n|nobody|nothing|nouns?|numbers?|objects?|people|persons?|player(?:'s holdall)?|regions?|relations?|rooms?|rule(?:book)?s?|scenes?|someone|something|supporters?|tables?|texts?|things?|time|vehicles?|wom[ae]n)\b(?!-)/i,
        lookbehind: true,
        alias: 'variable'
      },
      punctuation: /[.,:;(){}]/
    };
    Prism.languages.inform7['string'].inside['substitution'].inside.rest =
      Prism.languages.inform7; // We don't want the remaining text in the substitution to be highlighted as the string.
    Prism.languages.inform7['string'].inside['substitution'].inside.rest.text = {
      pattern: /\S(?:\s*\S)*/,
      alias: 'comment'
    };
  }

  // @ts-nocheck
  io.displayName = 'io';
  io.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function io(Prism) {
    Prism.languages.io = {
      comment: {
        pattern: /(^|[^\\])(?:\/\*[\s\S]*?(?:\*\/|$)|\/\/.*|#.*)/,
        lookbehind: true,
        greedy: true
      },
      'triple-quoted-string': {
        pattern: /"""(?:\\[\s\S]|(?!""")[^\\])*"""/,
        greedy: true,
        alias: 'string'
      },
      string: {
        pattern: /"(?:\\.|[^\\\r\n"])*"/,
        greedy: true
      },
      keyword:
        /\b(?:activate|activeCoroCount|asString|block|break|call|catch|clone|collectGarbage|compileString|continue|do|doFile|doMessage|doString|else|elseif|exit|for|foreach|forward|getEnvironmentVariable|getSlot|hasSlot|if|ifFalse|ifNil|ifNilEval|ifTrue|isActive|isNil|isResumable|list|message|method|parent|pass|pause|perform|performWithArgList|print|println|proto|raise|raiseResumable|removeSlot|resend|resume|schedulerSleepSeconds|self|sender|setSchedulerSleepSeconds|setSlot|shallowCopy|slotNames|super|system|then|thisBlock|thisContext|try|type|uniqueId|updateSlot|wait|while|write|yield)\b/,
      builtin:
        /\b(?:Array|AudioDevice|AudioMixer|BigNum|Block|Box|Buffer|CFunction|CGI|Color|Curses|DBM|DNSResolver|DOConnection|DOProxy|DOServer|Date|Directory|Duration|DynLib|Error|Exception|FFT|File|Fnmatch|Font|Future|GL|GLE|GLScissor|GLU|GLUCylinder|GLUQuadric|GLUSphere|GLUT|Host|Image|Importer|LinkList|List|Lobby|Locals|MD5|MP3Decoder|MP3Encoder|Map|Message|Movie|Notification|Number|Object|OpenGL|Point|Protos|Random|Regex|SGML|SGMLElement|SGMLParser|SQLite|Sequence|Server|ShowMessage|SleepyCat|SleepyCatCursor|Socket|SocketManager|Sound|Soup|Store|String|Tree|UDPSender|UPDReceiver|URL|User|Warning|WeakLink)\b/,
      boolean: /\b(?:false|nil|true)\b/,
      number: /\b0x[\da-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e-?\d+)?/i,
      operator:
        /[=!*/%+\-^&|]=|>>?=?|<<?=?|:?:?=|\+\+?|--?|\*\*?|\/\/?|%|\|\|?|&&?|\b(?:and|not|or|return)\b|@@?|\?\??|\.\./,
      punctuation: /[{}[\];(),.:]/
    };
  }

  // @ts-nocheck
  j.displayName = 'j';
  j.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function j(Prism) {
    Prism.languages.j = {
      comment: {
        pattern: /\bNB\..*/,
        greedy: true
      },
      string: {
        pattern: /'(?:''|[^'\r\n])*'/,
        greedy: true
      },
      keyword:
        /\b(?:(?:CR|LF|adverb|conjunction|def|define|dyad|monad|noun|verb)\b|(?:assert|break|case|catch[dt]?|continue|do|else|elseif|end|fcase|for|for_\w+|goto_\w+|if|label_\w+|return|select|throw|try|while|whilst)\.)/,
      verb: {
        // Negative look-ahead prevents bad highlighting
        // of ^: ;. =. =: !. !:
        pattern:
          /(?!\^:|;\.|[=!][.:])(?:\{(?:\.|::?)?|p(?:\.\.?|:)|[=!\]]|[<>+*\-%$|,#][.:]?|[?^]\.?|[;\[]:?|[~}"i][.:]|[ACeEIjLor]\.|(?:[_\/\\qsux]|_?\d):)/,
        alias: 'keyword'
      },
      number:
        /\b_?(?:(?!\d:)\d+(?:\.\d+)?(?:(?:ad|ar|[ejpx])_?\d+(?:\.\d+)?)*(?:b_?[\da-z]+(?:\.[\da-z]+)?)?|_\b(?!\.))/,
      adverb: {
        pattern: /[~}]|[\/\\]\.?|[bfM]\.|t[.:]/,
        alias: 'builtin'
      },
      operator: /[=a][.:]|_\./,
      conjunction: {
        pattern: /&(?:\.:?|:)?|[.:@][.:]?|[!D][.:]|[;dHT]\.|`:?|[\^LS]:|"/,
        alias: 'variable'
      },
      punctuation: /[()]/
    };
  }

  // @ts-nocheck
  javadoclike.displayName = 'javadoclike';
  javadoclike.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function javadoclike(Prism) {
  (function (Prism) {
      var javaDocLike = (Prism.languages.javadoclike = {
        parameter: {
          pattern:
            /(^[\t ]*(?:\/{3}|\*|\/\*\*)\s*@(?:arg|arguments|param)\s+)\w+/m,
          lookbehind: true
        },
        keyword: {
          // keywords are the first word in a line preceded be an `@` or surrounded by curly braces.
          // @word, {@word}
          pattern: /(^[\t ]*(?:\/{3}|\*|\/\*\*)\s*|\{)@[a-z][a-zA-Z-]+\b/m,
          lookbehind: true
        },
        punctuation: /[{}]/
      });
      /**
       * Adds doc comment support to the given language and calls a given callback on each doc comment pattern.
       *
       * @param {string} lang the language add doc comment support to.
       * @param {(pattern: {inside: {rest: undefined}}) => void} callback the function called with each doc comment pattern as argument.
       */
      function docCommentSupport(lang, callback) {
        var tokenName = 'doc-comment';
        var grammar = Prism.languages[lang];
        if (!grammar) {
          return
        }
        var token = grammar[tokenName];
        if (!token) {
          // add doc comment: /** */
          var definition = {};
          definition[tokenName] = {
            pattern: /(^|[^\\])\/\*\*[^/][\s\S]*?(?:\*\/|$)/,
            lookbehind: true,
            alias: 'comment'
          };
          grammar = Prism.languages.insertBefore(lang, 'comment', definition);
          token = grammar[tokenName];
        }
        if (token instanceof RegExp) {
          // convert regex to object
          token = grammar[tokenName] = {
            pattern: token
          };
        }
        if (Array.isArray(token)) {
          for (var i = 0, l = token.length; i < l; i++) {
            if (token[i] instanceof RegExp) {
              token[i] = {
                pattern: token[i]
              };
            }
            callback(token[i]);
          }
        } else {
          callback(token);
        }
      }
      /**
       * Adds doc-comment support to the given languages for the given documentation language.
       *
       * @param {string[]|string} languages
       * @param {Object} docLanguage
       */
      function addSupport(languages, docLanguage) {
        if (typeof languages === 'string') {
          languages = [languages];
        }
        languages.forEach(function (lang) {
          docCommentSupport(lang, function (pattern) {
            if (!pattern.inside) {
              pattern.inside = {};
            }
            pattern.inside.rest = docLanguage;
          });
        });
      }
      Object.defineProperty(javaDocLike, 'addSupport', {
        value: addSupport
      });
      javaDocLike.addSupport(['java', 'javascript', 'php'], javaDocLike);
    })(Prism);
  }

  // @ts-nocheck
  scala.displayName = 'scala';
  scala.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function scala(Prism) {
    Prism.register(java);
    Prism.languages.scala = Prism.languages.extend('java', {
      'triple-quoted-string': {
        pattern: /"""[\s\S]*?"""/,
        greedy: true,
        alias: 'string'
      },
      string: {
        pattern: /("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,
        greedy: true
      },
      keyword:
        /<-|=>|\b(?:abstract|case|catch|class|def|do|else|extends|final|finally|for|forSome|if|implicit|import|lazy|match|new|null|object|override|package|private|protected|return|sealed|self|super|this|throw|trait|try|type|val|var|while|with|yield)\b/,
      number:
        /\b0x(?:[\da-f]*\.)?[\da-f]+|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e\d+)?[dfl]?/i,
      builtin:
        /\b(?:Any|AnyRef|AnyVal|Boolean|Byte|Char|Double|Float|Int|Long|Nothing|Short|String|Unit)\b/,
      symbol: /'[^\d\s\\]\w*/
    });
    Prism.languages.insertBefore('scala', 'triple-quoted-string', {
      'string-interpolation': {
        pattern:
          /\b[a-z]\w*(?:"""(?:[^$]|\$(?:[^{]|\{(?:[^{}]|\{[^{}]*\})*\}))*?"""|"(?:[^$"\r\n]|\$(?:[^{]|\{(?:[^{}]|\{[^{}]*\})*\}))*")/i,
        greedy: true,
        inside: {
          id: {
            pattern: /^\w+/,
            greedy: true,
            alias: 'function'
          },
          escape: {
            pattern: /\\\$"|\$[$"]/,
            greedy: true,
            alias: 'symbol'
          },
          interpolation: {
            pattern: /\$(?:\w+|\{(?:[^{}]|\{[^{}]*\})*\})/,
            greedy: true,
            inside: {
              punctuation: /^\$\{?|\}$/,
              expression: {
                pattern: /[\s\S]+/,
                inside: Prism.languages.scala
              }
            }
          },
          string: /[\s\S]+/
        }
      }
    });
    delete Prism.languages.scala['class-name'];
    delete Prism.languages.scala['function'];
  }

  // @ts-nocheck
  javadoc.displayName = 'javadoc';
  javadoc.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function javadoc(Prism) {
    Prism.register(java);
    Prism.register(javadoclike);
    Prism.register(markup)
    ;(function (Prism) {
      var codeLinePattern = /(^(?:[\t ]*(?:\*\s*)*))[^*\s].*$/m;
      var memberReference = /#\s*\w+(?:\s*\([^()]*\))?/.source;
      var reference =
        /(?:\b[a-zA-Z]\w+\s*\.\s*)*\b[A-Z]\w*(?:\s*<mem>)?|<mem>/.source.replace(
          /<mem>/g,
          function () {
            return memberReference
          }
        );
      Prism.languages.javadoc = Prism.languages.extend('javadoclike', {});
      Prism.languages.insertBefore('javadoc', 'keyword', {
        reference: {
          pattern: RegExp(
            /(@(?:exception|link|linkplain|see|throws|value)\s+(?:\*\s*)?)/
              .source +
              '(?:' +
              reference +
              ')'
          ),
          lookbehind: true,
          inside: {
            function: {
              pattern: /(#\s*)\w+(?=\s*\()/,
              lookbehind: true
            },
            field: {
              pattern: /(#\s*)\w+/,
              lookbehind: true
            },
            namespace: {
              pattern: /\b(?:[a-z]\w*\s*\.\s*)+/,
              inside: {
                punctuation: /\./
              }
            },
            'class-name': /\b[A-Z]\w*/,
            keyword: Prism.languages.java.keyword,
            punctuation: /[#()[\],.]/
          }
        },
        'class-name': {
          // @param <T> the first generic type parameter
          pattern: /(@param\s+)<[A-Z]\w*>/,
          lookbehind: true,
          inside: {
            punctuation: /[.<>]/
          }
        },
        'code-section': [
          {
            pattern:
              /(\{@code\s+(?!\s))(?:[^\s{}]|\s+(?![\s}])|\{(?:[^{}]|\{(?:[^{}]|\{(?:[^{}]|\{[^{}]*\})*\})*\})*\})+(?=\s*\})/,
            lookbehind: true,
            inside: {
              code: {
                // there can't be any HTML inside of {@code} tags
                pattern: codeLinePattern,
                lookbehind: true,
                inside: Prism.languages.java,
                alias: 'language-java'
              }
            }
          },
          {
            pattern:
              /(<(code|pre|tt)>(?!<code>)\s*)\S(?:\S|\s+\S)*?(?=\s*<\/\2>)/,
            lookbehind: true,
            inside: {
              line: {
                pattern: codeLinePattern,
                lookbehind: true,
                inside: {
                  // highlight HTML tags and entities
                  tag: Prism.languages.markup.tag,
                  entity: Prism.languages.markup.entity,
                  code: {
                    // everything else is Java code
                    pattern: /.+/,
                    inside: Prism.languages.java,
                    alias: 'language-java'
                  }
                }
              }
            }
          }
        ],
        tag: Prism.languages.markup.tag,
        entity: Prism.languages.markup.entity
      });
      Prism.languages.javadoclike.addSupport('java', Prism.languages.javadoc);
    })(Prism);
  }

  // @ts-nocheck
  javastacktrace.displayName = 'javastacktrace';
  javastacktrace.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function javastacktrace(Prism) {
    // Specification:
    // https://docs.oracle.com/en/java/javase/13/docs/api/java.base/java/lang/Throwable.html#printStackTrace()
    Prism.languages.javastacktrace = {
      // java.sql.SQLException: Violation of unique constraint MY_ENTITY_UK_1: duplicate value(s) for column(s) MY_COLUMN in statement [...]
      // Caused by: java.sql.SQLException: Violation of unique constraint MY_ENTITY_UK_1: duplicate value(s) for column(s) MY_COLUMN in statement [...]
      // Caused by: com.example.myproject.MyProjectServletException
      // Caused by: MidLevelException: LowLevelException
      // Suppressed: Resource$CloseFailException: Resource ID = 0
      summary: {
        pattern:
          /^([\t ]*)(?:(?:Caused by:|Suppressed:|Exception in thread "[^"]*")[\t ]+)?[\w$.]+(?::.*)?$/m,
        lookbehind: true,
        inside: {
          keyword: {
            pattern:
              /^([\t ]*)(?:(?:Caused by|Suppressed)(?=:)|Exception in thread)/m,
            lookbehind: true
          },
          // the current thread if the summary starts with 'Exception in thread'
          string: {
            pattern: /^(\s*)"[^"]*"/,
            lookbehind: true
          },
          exceptions: {
            pattern: /^(:?\s*)[\w$.]+(?=:|$)/,
            lookbehind: true,
            inside: {
              'class-name': /[\w$]+$/,
              namespace: /\b[a-z]\w*\b/,
              punctuation: /\./
            }
          },
          message: {
            pattern: /(:\s*)\S.*/,
            lookbehind: true,
            alias: 'string'
          },
          punctuation: /:/
        }
      },
      // at org.mortbay.jetty.servlet.ServletHandler$CachedChain.doFilter(ServletHandler.java:1166)
      // at org.hsqldb.jdbc.Util.throwError(Unknown Source) here could be some notes
      // at java.base/java.lang.Class.forName0(Native Method)
      // at Util.<init>(Unknown Source)
      // at com.foo.loader/foo@9.0/com.foo.Main.run(Main.java:101)
      // at com.foo.loader//com.foo.bar.App.run(App.java:12)
      // at acme@2.1/org.acme.Lib.test(Lib.java:80)
      // at MyClass.mash(MyClass.java:9)
      //
      // More information:
      // https://docs.oracle.com/en/java/javase/13/docs/api/java.base/java/lang/StackTraceElement.html#toString()
      //
      // A valid Java module name is defined as:
      //   "A module name consists of one or more Java identifiers (§3.8) separated by "." tokens."
      // https://docs.oracle.com/javase/specs/jls/se9/html/jls-6.html#jls-ModuleName
      //
      // A Java module version is defined by this class:
      // https://docs.oracle.com/javase/9/docs/api/java/lang/module/ModuleDescriptor.Version.html
      // This is the implementation of the `parse` method in JDK13:
      // https://github.com/matcdac/jdk/blob/2305df71d1b7710266ae0956d73927a225132c0f/src/java.base/share/classes/java/lang/module/ModuleDescriptor.java#L1108
      // However, to keep this simple, a version will be matched by the pattern /@[\w$.+-]*/.
      'stack-frame': {
        pattern: /^([\t ]*)at (?:[\w$./]|@[\w$.+-]*\/)+(?:<init>)?\([^()]*\)/m,
        lookbehind: true,
        inside: {
          keyword: {
            pattern: /^(\s*)at(?= )/,
            lookbehind: true
          },
          source: [
            // (Main.java:15)
            // (Main.scala:15)
            {
              pattern: /(\()\w+\.\w+:\d+(?=\))/,
              lookbehind: true,
              inside: {
                file: /^\w+\.\w+/,
                punctuation: /:/,
                'line-number': {
                  pattern: /\b\d+\b/,
                  alias: 'number'
                }
              }
            }, // (Unknown Source)
            // (Native Method)
            // (...something...)
            {
              pattern: /(\()[^()]*(?=\))/,
              lookbehind: true,
              inside: {
                keyword: /^(?:Native Method|Unknown Source)$/
              }
            }
          ],
          'class-name': /[\w$]+(?=\.(?:<init>|[\w$]+)\()/,
          function: /(?:<init>|[\w$]+)(?=\()/,
          'class-loader': {
            pattern: /(\s)[a-z]\w*(?:\.[a-z]\w*)*(?=\/[\w@$.]*\/)/,
            lookbehind: true,
            alias: 'namespace',
            inside: {
              punctuation: /\./
            }
          },
          module: {
            pattern: /([\s/])[a-z]\w*(?:\.[a-z]\w*)*(?:@[\w$.+-]*)?(?=\/)/,
            lookbehind: true,
            inside: {
              version: {
                pattern: /(@)[\s\S]+/,
                lookbehind: true,
                alias: 'number'
              },
              punctuation: /[@.]/
            }
          },
          namespace: {
            pattern: /(?:\b[a-z]\w*\.)+/,
            inside: {
              punctuation: /\./
            }
          },
          punctuation: /[()/.]/
        }
      },
      // ... 32 more
      // ... 32 common frames omitted
      more: {
        pattern: /^([\t ]*)\.{3} \d+ [a-z]+(?: [a-z]+)*/m,
        lookbehind: true,
        inside: {
          punctuation: /\.{3}/,
          number: /\d+/,
          keyword: /\b[a-z]+(?: [a-z]+)*\b/
        }
      }
    };
  }

  // @ts-nocheck
  jexl.displayName = 'jexl';
  jexl.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function jexl(Prism) {
    Prism.languages.jexl = {
      string: /(["'])(?:\\[\s\S]|(?!\1)[^\\])*\1/,
      transform: {
        pattern:
          /(\|\s*)[a-zA-Zа-яА-Я_\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u00FF$][\wа-яА-Я\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u00FF$]*/,
        alias: 'function',
        lookbehind: true
      },
      function:
        /[a-zA-Zа-яА-Я_\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u00FF$][\wа-яА-Я\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u00FF$]*\s*(?=\()/,
      number: /\b\d+(?:\.\d+)?\b|\B\.\d+\b/,
      operator: /[<>!]=?|-|\+|&&|==|\|\|?|\/\/?|[?:*^%]/,
      boolean: /\b(?:false|true)\b/,
      keyword: /\bin\b/,
      punctuation: /[{}[\](),.]/
    };
  }

  // @ts-nocheck
  jolie.displayName = 'jolie';
  jolie.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function jolie(Prism) {
    Prism.register(clike);
    Prism.languages.jolie = Prism.languages.extend('clike', {
      string: {
        pattern: /(^|[^\\])"(?:\\[\s\S]|[^"\\])*"/,
        lookbehind: true,
        greedy: true
      },
      'class-name': {
        pattern:
          /((?:\b(?:as|courier|embed|in|inputPort|outputPort|service)\b|@)[ \t]*)\w+/,
        lookbehind: true
      },
      keyword:
        /\b(?:as|cH|comp|concurrent|constants|courier|cset|csets|default|define|else|embed|embedded|execution|exit|extender|for|foreach|forward|from|global|if|import|in|include|init|inputPort|install|instanceof|interface|is_defined|linkIn|linkOut|main|new|nullProcess|outputPort|over|private|provide|public|scope|sequential|service|single|spawn|synchronized|this|throw|throws|type|undef|until|while|with)\b/,
      function: /\b[a-z_]\w*(?=[ \t]*[@(])/i,
      number: /(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?l?/i,
      operator: /-[-=>]?|\+[+=]?|<[<=]?|[>=*!]=?|&&|\|\||[?\/%^@|]/,
      punctuation: /[()[\]{},;.:]/,
      builtin:
        /\b(?:Byte|any|bool|char|double|enum|float|int|length|long|ranges|regex|string|undefined|void)\b/
    });
    Prism.languages.insertBefore('jolie', 'keyword', {
      aggregates: {
        pattern:
          /(\bAggregates\s*:\s*)(?:\w+(?:\s+with\s+\w+)?\s*,\s*)*\w+(?:\s+with\s+\w+)?/,
        lookbehind: true,
        inside: {
          keyword: /\bwith\b/,
          'class-name': /\w+/,
          punctuation: /,/
        }
      },
      redirects: {
        pattern:
          /(\bRedirects\s*:\s*)(?:\w+\s*=>\s*\w+\s*,\s*)*(?:\w+\s*=>\s*\w+)/,
        lookbehind: true,
        inside: {
          punctuation: /,/,
          'class-name': /\w+/,
          operator: /=>/
        }
      },
      property: {
        pattern:
          /\b(?:Aggregates|[Ii]nterfaces|Java|Javascript|Jolie|[Ll]ocation|OneWay|[Pp]rotocol|Redirects|RequestResponse)\b(?=[ \t]*:)/
      }
    });
  }

  // @ts-nocheck
  jq.displayName = 'jq';
  jq.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function jq(Prism) {
  (function (Prism) {
      var interpolation = /\\\((?:[^()]|\([^()]*\))*\)/.source;
      var string = RegExp(
        /(^|[^\\])"(?:[^"\r\n\\]|\\[^\r\n(]|__)*"/.source.replace(
          /__/g,
          function () {
            return interpolation
          }
        )
      );
      var stringInterpolation = {
        interpolation: {
          pattern: RegExp(/((?:^|[^\\])(?:\\{2})*)/.source + interpolation),
          lookbehind: true,
          inside: {
            content: {
              pattern: /^(\\\()[\s\S]+(?=\)$)/,
              lookbehind: true,
              inside: null // see below
            },
            punctuation: /^\\\(|\)$/
          }
        }
      };
      var jq = (Prism.languages.jq = {
        comment: /#.*/,
        property: {
          pattern: RegExp(string.source + /(?=\s*:(?!:))/.source),
          lookbehind: true,
          greedy: true,
          inside: stringInterpolation
        },
        string: {
          pattern: string,
          lookbehind: true,
          greedy: true,
          inside: stringInterpolation
        },
        function: {
          pattern: /(\bdef\s+)[a-z_]\w+/i,
          lookbehind: true
        },
        variable: /\B\$\w+/,
        'property-literal': {
          pattern: /\b[a-z_]\w*(?=\s*:(?!:))/i,
          alias: 'property'
        },
        keyword:
          /\b(?:as|break|catch|def|elif|else|end|foreach|if|import|include|label|module|modulemeta|null|reduce|then|try|while)\b/,
        boolean: /\b(?:false|true)\b/,
        number: /(?:\b\d+\.|\B\.)?\b\d+(?:[eE][+-]?\d+)?\b/,
        operator: [
          {
            pattern: /\|=?/,
            alias: 'pipe'
          },
          /\.\.|[!=<>]?=|\?\/\/|\/\/=?|[-+*/%]=?|[<>?]|\b(?:and|not|or)\b/
        ],
        'c-style-function': {
          pattern: /\b[a-z_]\w*(?=\s*\()/i,
          alias: 'function'
        },
        punctuation: /::|[()\[\]{},:;]|\.(?=\s*[\[\w$])/,
        dot: {
          pattern: /\./,
          alias: 'important'
        }
      });
      stringInterpolation.interpolation.inside.content.inside = jq;
    })(Prism);
  }

  // @ts-nocheck
  jsTemplates.displayName = 'js-templates';
  jsTemplates.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function jsTemplates(Prism) {
    Prism.register(javascript)
    ;(function (Prism) {
      var templateString = Prism.languages.javascript['template-string']; // see the pattern in prism-javascript.js
      var templateLiteralPattern = templateString.pattern.source;
      var interpolationObject = templateString.inside['interpolation'];
      var interpolationPunctuationObject =
        interpolationObject.inside['interpolation-punctuation'];
      var interpolationPattern = interpolationObject.pattern.source;
      /**
       * Creates a new pattern to match a template string with a special tag.
       *
       * This will return `undefined` if there is no grammar with the given language id.
       *
       * @param {string} language The language id of the embedded language. E.g. `markdown`.
       * @param {string} tag The regex pattern to match the tag.
       * @returns {object | undefined}
       * @example
       * createTemplate('css', /\bcss/.source);
       */
      function createTemplate(language, tag) {
        if (!Prism.languages[language]) {
          return undefined
        }
        return {
          pattern: RegExp('((?:' + tag + ')\\s*)' + templateLiteralPattern),
          lookbehind: true,
          greedy: true,
          inside: {
            'template-punctuation': {
              pattern: /^`|`$/,
              alias: 'string'
            },
            'embedded-code': {
              pattern: /[\s\S]+/,
              alias: language
            }
          }
        }
      }
      Prism.languages.javascript['template-string'] = [
        // styled-jsx:
        //   css`a { color: #25F; }`
        // styled-components:
        //   styled.h1`color: red;`
        createTemplate(
          'css',
          /\b(?:styled(?:\([^)]*\))?(?:\s*\.\s*\w+(?:\([^)]*\))*)*|css(?:\s*\.\s*(?:global|resolve))?|createGlobalStyle|keyframes)/
            .source
        ), // html`<p></p>`
        // div.innerHTML = `<p></p>`
        createTemplate('html', /\bhtml|\.\s*(?:inner|outer)HTML\s*\+?=/.source), // svg`<path fill="#fff" d="M55.37 ..."/>`
        createTemplate('svg', /\bsvg/.source), // md`# h1`, markdown`## h2`
        createTemplate('markdown', /\b(?:markdown|md)/.source), // gql`...`, graphql`...`, graphql.experimental`...`
        createTemplate(
          'graphql',
          /\b(?:gql|graphql(?:\s*\.\s*experimental)?)/.source
        ), // sql`...`
        createTemplate('sql', /\bsql/.source), // vanilla template string
        templateString
      ].filter(Boolean);
      /**
       * Returns a specific placeholder literal for the given language.
       *
       * @param {number} counter
       * @param {string} language
       * @returns {string}
       */
      function getPlaceholder(counter, language) {
        return '___' + language.toUpperCase() + '_' + counter + '___'
      }
      /**
       * Returns the tokens of `Prism.tokenize` but also runs the `before-tokenize` and `after-tokenize` hooks.
       *
       * @param {string} code
       * @param {any} grammar
       * @param {string} language
       * @returns {(string|Token)[]}
       */
      function tokenizeWithHooks(code, grammar, language) {
        var env = {
          code: code,
          grammar: grammar,
          language: language
        };
        Prism.hooks.run('before-tokenize', env);
        env.tokens = Prism.tokenize(env.code, env.grammar);
        Prism.hooks.run('after-tokenize', env);
        return env.tokens
      }
      /**
       * Returns the token of the given JavaScript interpolation expression.
       *
       * @param {string} expression The code of the expression. E.g. `"${42}"`
       * @returns {Token}
       */
      function tokenizeInterpolationExpression(expression) {
        var tempGrammar = {};
        tempGrammar['interpolation-punctuation'] = interpolationPunctuationObject;
        /** @type {Array} */
        var tokens = Prism.tokenize(expression, tempGrammar);
        if (tokens.length === 3) {
          /**
           * The token array will look like this
           * [
           *     ["interpolation-punctuation", "${"]
           *     "..." // JavaScript expression of the interpolation
           *     ["interpolation-punctuation", "}"]
           * ]
           */
          var args = [1, 1];
          args.push.apply(
            args,
            tokenizeWithHooks(tokens[1], Prism.languages.javascript, 'javascript')
          );
          tokens.splice.apply(tokens, args);
        }
        return new Prism.Token(
          'interpolation',
          tokens,
          interpolationObject.alias,
          expression
        )
      }
      /**
       * Tokenizes the given code with support for JavaScript interpolation expressions mixed in.
       *
       * This function has 3 phases:
       *
       * 1. Replace all JavaScript interpolation expression with a placeholder.
       *    The placeholder will have the syntax of a identify of the target language.
       * 2. Tokenize the code with placeholders.
       * 3. Tokenize the interpolation expressions and re-insert them into the tokenize code.
       *    The insertion only works if a placeholder hasn't been "ripped apart" meaning that the placeholder has been
       *    tokenized as two tokens by the grammar of the embedded language.
       *
       * @param {string} code
       * @param {object} grammar
       * @param {string} language
       * @returns {Token}
       */
      function tokenizeEmbedded(code, grammar, language) {
        // 1. First filter out all interpolations
        // because they might be escaped, we need a lookbehind, so we use Prism
        /** @type {(Token|string)[]} */
        var _tokens = Prism.tokenize(code, {
          interpolation: {
            pattern: RegExp(interpolationPattern),
            lookbehind: true
          }
        }); // replace all interpolations with a placeholder which is not in the code already
        var placeholderCounter = 0;
        /** @type {Object<string, string>} */
        var placeholderMap = {};
        var embeddedCode = _tokens
          .map(function (token) {
            if (typeof token === 'string') {
              return token
            } else {
              var interpolationExpression = token.content;
              var placeholder;
              while (
                code.indexOf(
                  (placeholder = getPlaceholder(placeholderCounter++, language))
                ) !== -1
              ) {
                /* noop */
              }
              placeholderMap[placeholder] = interpolationExpression;
              return placeholder
            }
          })
          .join(''); // 2. Tokenize the embedded code
        var embeddedTokens = tokenizeWithHooks(embeddedCode, grammar, language); // 3. Re-insert the interpolation
        var placeholders = Object.keys(placeholderMap);
        placeholderCounter = 0;
        /**
         *
         * @param {(Token|string)[]} tokens
         * @returns {void}
         */
        function walkTokens(tokens) {
          for (var i = 0; i < tokens.length; i++) {
            if (placeholderCounter >= placeholders.length) {
              return
            }
            var token = tokens[i];
            if (typeof token === 'string' || typeof token.content === 'string') {
              var placeholder = placeholders[placeholderCounter];
              var s =
                typeof token === 'string'
                  ? token
                  : /** @type {string} */
                    token.content;
              var index = s.indexOf(placeholder);
              if (index !== -1) {
                ++placeholderCounter;
                var before = s.substring(0, index);
                var middle = tokenizeInterpolationExpression(
                  placeholderMap[placeholder]
                );
                var after = s.substring(index + placeholder.length);
                var replacement = [];
                if (before) {
                  replacement.push(before);
                }
                replacement.push(middle);
                if (after) {
                  var afterTokens = [after];
                  walkTokens(afterTokens);
                  replacement.push.apply(replacement, afterTokens);
                }
                if (typeof token === 'string') {
                  tokens.splice.apply(tokens, [i, 1].concat(replacement));
                  i += replacement.length - 1;
                } else {
                  token.content = replacement;
                }
              }
            } else {
              var content = token.content;
              if (Array.isArray(content)) {
                walkTokens(content);
              } else {
                walkTokens([content]);
              }
            }
          }
        }
        walkTokens(embeddedTokens);
        return new Prism.Token(
          language,
          embeddedTokens,
          'language-' + language,
          code
        )
      }
      /**
       * The languages for which JS templating will handle tagged template literals.
       *
       * JS templating isn't active for only JavaScript but also related languages like TypeScript, JSX, and TSX.
       */
      var supportedLanguages = {
        javascript: true,
        js: true,
        typescript: true,
        ts: true,
        jsx: true,
        tsx: true
      };
      Prism.hooks.add('after-tokenize', function (env) {
        if (!(env.language in supportedLanguages)) {
          return
        }
        /**
         * Finds and tokenizes all template strings with an embedded languages.
         *
         * @param {(Token | string)[]} tokens
         * @returns {void}
         */
        function findTemplateStrings(tokens) {
          for (var i = 0, l = tokens.length; i < l; i++) {
            var token = tokens[i];
            if (typeof token === 'string') {
              continue
            }
            var content = token.content;
            if (!Array.isArray(content)) {
              if (typeof content !== 'string') {
                findTemplateStrings([content]);
              }
              continue
            }
            if (token.type === 'template-string') {
              /**
               * A JavaScript template-string token will look like this:
               *
               * ["template-string", [
               *     ["template-punctuation", "`"],
               *     (
               *         An array of "string" and "interpolation" tokens. This is the simple string case.
               *         or
               *         ["embedded-code", "..."] This is the token containing the embedded code.
               *                                  It also has an alias which is the language of the embedded code.
               *     ),
               *     ["template-punctuation", "`"]
               * ]]
               */
              var embedded = content[1];
              if (
                content.length === 3 &&
                typeof embedded !== 'string' &&
                embedded.type === 'embedded-code'
              ) {
                // get string content
                var code = stringContent(embedded);
                var alias = embedded.alias;
                var language = Array.isArray(alias) ? alias[0] : alias;
                var grammar = Prism.languages[language];
                if (!grammar) {
                  // the embedded language isn't registered.
                  continue
                }
                content[1] = tokenizeEmbedded(code, grammar, language);
              }
            } else {
              findTemplateStrings(content);
            }
          }
        }
        findTemplateStrings(env.tokens);
      });
      /**
       * Returns the string content of a token or token stream.
       *
       * @param {string | Token | (string | Token)[]} value
       * @returns {string}
       */
      function stringContent(value) {
        if (typeof value === 'string') {
          return value
        } else if (Array.isArray(value)) {
          return value.map(stringContent).join('')
        } else {
          return stringContent(value.content)
        }
      }
    })(Prism);
  }

  // @ts-nocheck
  jsdoc.displayName = 'jsdoc';
  jsdoc.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function jsdoc(Prism) {
    Prism.register(javadoclike);
    Prism.register(javascript);
    Prism.register(typescript)
    ;(function (Prism) {
      var javascript = Prism.languages.javascript;
      var type = /\{(?:[^{}]|\{(?:[^{}]|\{[^{}]*\})*\})+\}/.source;
      var parameterPrefix =
        '(@(?:arg|argument|param|property)\\s+(?:' + type + '\\s+)?)';
      Prism.languages.jsdoc = Prism.languages.extend('javadoclike', {
        parameter: {
          // @param {string} foo - foo bar
          pattern: RegExp(
            parameterPrefix + /(?:(?!\s)[$\w\xA0-\uFFFF.])+(?=\s|$)/.source
          ),
          lookbehind: true,
          inside: {
            punctuation: /\./
          }
        }
      });
      Prism.languages.insertBefore('jsdoc', 'keyword', {
        'optional-parameter': {
          // @param {string} [baz.foo="bar"] foo bar
          pattern: RegExp(
            parameterPrefix +
              /\[(?:(?!\s)[$\w\xA0-\uFFFF.])+(?:=[^[\]]+)?\](?=\s|$)/.source
          ),
          lookbehind: true,
          inside: {
            parameter: {
              pattern: /(^\[)[$\w\xA0-\uFFFF\.]+/,
              lookbehind: true,
              inside: {
                punctuation: /\./
              }
            },
            code: {
              pattern: /(=)[\s\S]*(?=\]$)/,
              lookbehind: true,
              inside: javascript,
              alias: 'language-javascript'
            },
            punctuation: /[=[\]]/
          }
        },
        'class-name': [
          {
            pattern: RegExp(
              /(@(?:augments|class|extends|interface|memberof!?|template|this|typedef)\s+(?:<TYPE>\s+)?)[A-Z]\w*(?:\.[A-Z]\w*)*/.source.replace(
                /<TYPE>/g,
                function () {
                  return type
                }
              )
            ),
            lookbehind: true,
            inside: {
              punctuation: /\./
            }
          },
          {
            pattern: RegExp('(@[a-z]+\\s+)' + type),
            lookbehind: true,
            inside: {
              string: javascript.string,
              number: javascript.number,
              boolean: javascript.boolean,
              keyword: Prism.languages.typescript.keyword,
              operator: /=>|\.\.\.|[&|?:*]/,
              punctuation: /[.,;=<>{}()[\]]/
            }
          }
        ],
        example: {
          pattern:
            /(@example\s+(?!\s))(?:[^@\s]|\s+(?!\s))+?(?=\s*(?:\*\s*)?(?:@\w|\*\/))/,
          lookbehind: true,
          inside: {
            code: {
              pattern: /^([\t ]*(?:\*\s*)?)\S.*$/m,
              lookbehind: true,
              inside: javascript,
              alias: 'language-javascript'
            }
          }
        }
      });
      Prism.languages.javadoclike.addSupport('javascript', Prism.languages.jsdoc);
    })(Prism);
  }

  // @ts-nocheck
  n4js.displayName = 'n4js';
  n4js.aliases = ['n4jsd'];

  /** @type {import('../core.js').Syntax} */
  function n4js(Prism) {
    Prism.register(javascript);
    Prism.languages.n4js = Prism.languages.extend('javascript', {
      // Keywords from N4JS language spec: https://numberfour.github.io/n4js/spec/N4JSSpec.html
      keyword:
        /\b(?:Array|any|boolean|break|case|catch|class|const|constructor|continue|debugger|declare|default|delete|do|else|enum|export|extends|false|finally|for|from|function|get|if|implements|import|in|instanceof|interface|let|module|new|null|number|package|private|protected|public|return|set|static|string|super|switch|this|throw|true|try|typeof|var|void|while|with|yield)\b/
    });
    Prism.languages.insertBefore('n4js', 'constant', {
      // Annotations in N4JS spec: https://numberfour.github.io/n4js/spec/N4JSSpec.html#_annotations
      annotation: {
        pattern: /@+\w+/,
        alias: 'operator'
      }
    });
    Prism.languages.n4jsd = Prism.languages.n4js;
  }

  // @ts-nocheck
  jsExtras.displayName = 'js-extras';
  jsExtras.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function jsExtras(Prism) {
    Prism.register(javascript)
    ;(function (Prism) {
      Prism.languages.insertBefore('javascript', 'function-variable', {
        'method-variable': {
          pattern: RegExp(
            '(\\.\\s*)' +
              Prism.languages.javascript['function-variable'].pattern.source
          ),
          lookbehind: true,
          alias: ['function-variable', 'method', 'function', 'property-access']
        }
      });
      Prism.languages.insertBefore('javascript', 'function', {
        method: {
          pattern: RegExp(
            '(\\.\\s*)' + Prism.languages.javascript['function'].source
          ),
          lookbehind: true,
          alias: ['function', 'property-access']
        }
      });
      Prism.languages.insertBefore('javascript', 'constant', {
        'known-class-name': [
          {
            // standard built-ins
            // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects
            pattern:
              /\b(?:(?:Float(?:32|64)|(?:Int|Uint)(?:8|16|32)|Uint8Clamped)?Array|ArrayBuffer|BigInt|Boolean|DataView|Date|Error|Function|Intl|JSON|(?:Weak)?(?:Map|Set)|Math|Number|Object|Promise|Proxy|Reflect|RegExp|String|Symbol|WebAssembly)\b/,
            alias: 'class-name'
          },
          {
            // errors
            pattern: /\b(?:[A-Z]\w*)Error\b/,
            alias: 'class-name'
          }
        ]
      });
      /**
       * Replaces the `<ID>` placeholder in the given pattern with a pattern for general JS identifiers.
       *
       * @param {string} source
       * @param {string} [flags]
       * @returns {RegExp}
       */
      function withId(source, flags) {
        return RegExp(
          source.replace(/<ID>/g, function () {
            return /(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*/.source
          }),
          flags
        )
      }
      Prism.languages.insertBefore('javascript', 'keyword', {
        imports: {
          // https://tc39.es/ecma262/#sec-imports
          pattern: withId(
            /(\bimport\b\s*)(?:<ID>(?:\s*,\s*(?:\*\s*as\s+<ID>|\{[^{}]*\}))?|\*\s*as\s+<ID>|\{[^{}]*\})(?=\s*\bfrom\b)/
              .source
          ),
          lookbehind: true,
          inside: Prism.languages.javascript
        },
        exports: {
          // https://tc39.es/ecma262/#sec-exports
          pattern: withId(
            /(\bexport\b\s*)(?:\*(?:\s*as\s+<ID>)?(?=\s*\bfrom\b)|\{[^{}]*\})/
              .source
          ),
          lookbehind: true,
          inside: Prism.languages.javascript
        }
      });
      Prism.languages.javascript['keyword'].unshift(
        {
          pattern: /\b(?:as|default|export|from|import)\b/,
          alias: 'module'
        },
        {
          pattern:
            /\b(?:await|break|catch|continue|do|else|finally|for|if|return|switch|throw|try|while|yield)\b/,
          alias: 'control-flow'
        },
        {
          pattern: /\bnull\b/,
          alias: ['null', 'nil']
        },
        {
          pattern: /\bundefined\b/,
          alias: 'nil'
        }
      );
      Prism.languages.insertBefore('javascript', 'operator', {
        spread: {
          pattern: /\.{3}/,
          alias: 'operator'
        },
        arrow: {
          pattern: /=>/,
          alias: 'operator'
        }
      });
      Prism.languages.insertBefore('javascript', 'punctuation', {
        'property-access': {
          pattern: withId(/(\.\s*)#?<ID>/.source),
          lookbehind: true
        },
        'maybe-class-name': {
          pattern: /(^|[^$\w\xA0-\uFFFF])[A-Z][$\w\xA0-\uFFFF]+/,
          lookbehind: true
        },
        dom: {
          // this contains only a few commonly used DOM variables
          pattern:
            /\b(?:document|(?:local|session)Storage|location|navigator|performance|window)\b/,
          alias: 'variable'
        },
        console: {
          pattern: /\bconsole(?=\s*\.)/,
          alias: 'class-name'
        }
      }); // add 'maybe-class-name' to tokens which might be a class name
      var maybeClassNameTokens = [
        'function',
        'function-variable',
        'method',
        'method-variable',
        'property-access'
      ];
      for (var i = 0; i < maybeClassNameTokens.length; i++) {
        var token = maybeClassNameTokens[i];
        var value = Prism.languages.javascript[token]; // convert regex to object
        if (Prism.util.type(value) === 'RegExp') {
          value = Prism.languages.javascript[token] = {
            pattern: value
          };
        } // keep in mind that we don't support arrays
        var inside = value.inside || {};
        value.inside = inside;
        inside['maybe-class-name'] = /^[A-Z][\s\S]*/;
      }
    })(Prism);
  }

  // @ts-nocheck
  json5.displayName = 'json5';
  json5.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function json5(Prism) {
    Prism.register(json)
    ;(function (Prism) {
      var string = /("|')(?:\\(?:\r\n?|\n|.)|(?!\1)[^\\\r\n])*\1/;
      Prism.languages.json5 = Prism.languages.extend('json', {
        property: [
          {
            pattern: RegExp(string.source + '(?=\\s*:)'),
            greedy: true
          },
          {
            pattern:
              /(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*:)/,
            alias: 'unquoted'
          }
        ],
        string: {
          pattern: string,
          greedy: true
        },
        number:
          /[+-]?\b(?:NaN|Infinity|0x[a-fA-F\d]+)\b|[+-]?(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[eE][+-]?\d+\b)?/
      });
    })(Prism);
  }

  // @ts-nocheck
  jsonp.displayName = 'jsonp';
  jsonp.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function jsonp(Prism) {
    Prism.register(json);
    Prism.languages.jsonp = Prism.languages.extend('json', {
      punctuation: /[{}[\]();,.]/
    });
    Prism.languages.insertBefore('jsonp', 'punctuation', {
      function: /(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*\()/
    });
  }

  // @ts-nocheck
  jsstacktrace.displayName = 'jsstacktrace';
  jsstacktrace.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function jsstacktrace(Prism) {
    Prism.languages.jsstacktrace = {
      'error-message': {
        pattern: /^\S.*/m,
        alias: 'string'
      },
      'stack-frame': {
        pattern: /(^[ \t]+)at[ \t].*/m,
        lookbehind: true,
        inside: {
          'not-my-code': {
            pattern:
              /^at[ \t]+(?!\s)(?:node\.js|<unknown>|.*(?:node_modules|\(<anonymous>\)|\(<unknown>|<anonymous>$|\(internal\/|\(node\.js)).*/m,
            alias: 'comment'
          },
          filename: {
            pattern: /(\bat\s+(?!\s)|\()(?:[a-zA-Z]:)?[^():]+(?=:)/,
            lookbehind: true,
            alias: 'url'
          },
          function: {
            pattern:
              /(\bat\s+(?:new\s+)?)(?!\s)[_$a-zA-Z\xA0-\uFFFF<][.$\w\xA0-\uFFFF<>]*/,
            lookbehind: true,
            inside: {
              punctuation: /\./
            }
          },
          punctuation: /[()]/,
          keyword: /\b(?:at|new)\b/,
          alias: {
            pattern: /\[(?:as\s+)?(?!\s)[_$a-zA-Z\xA0-\uFFFF][$\w\xA0-\uFFFF]*\]/,
            alias: 'variable'
          },
          'line-number': {
            pattern: /:\d+(?::\d+)?\b/,
            alias: 'number',
            inside: {
              punctuation: /:/
            }
          }
        }
      }
    };
  }

  // @ts-nocheck
  julia.displayName = 'julia';
  julia.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function julia(Prism) {
    Prism.languages.julia = {
      comment: {
        // support one level of nested comments
        // https://github.com/JuliaLang/julia/pull/6128
        pattern:
          /(^|[^\\])(?:#=(?:[^#=]|=(?!#)|#(?!=)|#=(?:[^#=]|=(?!#)|#(?!=))*=#)*=#|#.*)/,
        lookbehind: true
      },
      regex: {
        // https://docs.julialang.org/en/v1/manual/strings/#Regular-Expressions-1
        pattern: /r"(?:\\.|[^"\\\r\n])*"[imsx]{0,4}/,
        greedy: true
      },
      string: {
        // https://docs.julialang.org/en/v1/manual/strings/#String-Basics-1
        // https://docs.julialang.org/en/v1/manual/strings/#non-standard-string-literals-1
        // https://docs.julialang.org/en/v1/manual/running-external-programs/#Running-External-Programs-1
        pattern:
          /"""[\s\S]+?"""|(?:\b\w+)?"(?:\\.|[^"\\\r\n])*"|`(?:[^\\`\r\n]|\\.)*`/,
        greedy: true
      },
      char: {
        // https://docs.julialang.org/en/v1/manual/strings/#man-characters-1
        pattern: /(^|[^\w'])'(?:\\[^\r\n][^'\r\n]*|[^\\\r\n])'/,
        lookbehind: true,
        greedy: true
      },
      keyword:
        /\b(?:abstract|baremodule|begin|bitstype|break|catch|ccall|const|continue|do|else|elseif|end|export|finally|for|function|global|if|immutable|import|importall|in|let|local|macro|module|print|println|quote|return|struct|try|type|typealias|using|while)\b/,
      boolean: /\b(?:false|true)\b/,
      number:
        /(?:\b(?=\d)|\B(?=\.))(?:0[box])?(?:[\da-f]+(?:_[\da-f]+)*(?:\.(?:\d+(?:_\d+)*)?)?|\.\d+(?:_\d+)*)(?:[efp][+-]?\d+(?:_\d+)*)?j?/i,
      // https://docs.julialang.org/en/v1/manual/mathematical-operations/
      // https://docs.julialang.org/en/v1/manual/mathematical-operations/#Operator-Precedence-and-Associativity-1
      operator:
        /&&|\|\||[-+*^%÷⊻&$\\]=?|\/[\/=]?|!=?=?|\|[=>]?|<(?:<=?|[=:|])?|>(?:=|>>?=?)?|==?=?|[~≠≤≥'√∛]/,
      punctuation: /::?|[{}[\]();,.?]/,
      // https://docs.julialang.org/en/v1/base/numbers/#Base.im
      constant: /\b(?:(?:Inf|NaN)(?:16|32|64)?|im|pi)\b|[πℯ]/
    };
  }

  // @ts-nocheck
  keepalived.displayName = 'keepalived';
  keepalived.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function keepalived(Prism) {
    Prism.languages.keepalived = {
      comment: {
        pattern: /[#!].*/,
        greedy: true
      },
      string: {
        pattern:
          /(^|[^\\])(?:"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"|'(?:\\(?:\r\n|[\s\S])|[^'\\\r\n])*')/,
        lookbehind: true,
        greedy: true
      },
      // support IPv4, IPv6, subnet mask
      ip: {
        pattern: RegExp(
          /\b(?:(?:(?:[\da-f]{1,4}:){7}[\da-f]{1,4}|(?:[\da-f]{1,4}:){6}:[\da-f]{1,4}|(?:[\da-f]{1,4}:){5}:(?:[\da-f]{1,4}:)?[\da-f]{1,4}|(?:[\da-f]{1,4}:){4}:(?:[\da-f]{1,4}:){0,2}[\da-f]{1,4}|(?:[\da-f]{1,4}:){3}:(?:[\da-f]{1,4}:){0,3}[\da-f]{1,4}|(?:[\da-f]{1,4}:){2}:(?:[\da-f]{1,4}:){0,4}[\da-f]{1,4}|(?:[\da-f]{1,4}:){6}<ipv4>|(?:[\da-f]{1,4}:){0,5}:<ipv4>|::(?:[\da-f]{1,4}:){0,5}<ipv4>|[\da-f]{1,4}::(?:[\da-f]{1,4}:){0,5}[\da-f]{1,4}|::(?:[\da-f]{1,4}:){0,6}[\da-f]{1,4}|(?:[\da-f]{1,4}:){1,7}:)(?:\/\d{1,3})?|<ipv4>(?:\/\d{1,2})?)\b/.source.replace(
            /<ipv4>/g,
            function () {
              return /(?:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)\.){3}(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d))/
                .source
            }
          ),
          'i'
        ),
        alias: 'number'
      },
      // support *nix / Windows, directory / file
      path: {
        pattern:
          /(\s)\/(?:[^\/\s]+\/)*[^\/\s]*|\b[a-zA-Z]:\\(?:[^\\\s]+\\)*[^\\\s]*/,
        lookbehind: true,
        alias: 'string'
      },
      variable: /\$\{?\w+\}?/,
      email: {
        pattern: /[\w-]+@[\w-]+(?:\.[\w-]{2,3}){1,2}/,
        alias: 'string'
      },
      'conditional-configuration': {
        pattern: /@\^?[\w-]+/,
        alias: 'variable'
      },
      operator: /=/,
      property:
        /\b(?:BFD_CHECK|DNS_CHECK|FILE_CHECK|HTTP_GET|MISC_CHECK|NAME|PING_CHECK|SCRIPTS|SMTP_CHECK|SSL|SSL_GET|TCP_CHECK|UDP_CHECK|accept|advert_int|alpha|auth_pass|auth_type|authentication|bfd_cpu_affinity|bfd_instance|bfd_no_swap|bfd_priority|bfd_process_name|bfd_rlimit_rttime|bfd_rt_priority|bind_if|bind_port|bindto|ca|certificate|check_unicast_src|checker|checker_cpu_affinity|checker_log_all_failures|checker_no_swap|checker_priority|checker_rlimit_rttime|checker_rt_priority|child_wait_time|connect_ip|connect_port|connect_timeout|dbus_service_name|debug|default_interface|delay|delay_before_retry|delay_loop|digest|dont_track_primary|dynamic|dynamic_interfaces|enable_(?:dbus|script_security|sni|snmp_checker|snmp_rfc|snmp_rfcv2|snmp_rfcv3|snmp_vrrp|traps)|end|fall|fast_recovery|file|flag-[123]|fork_delay|full_command|fwmark|garp_group|garp_interval|garp_lower_prio_delay|garp_lower_prio_repeat|garp_master_delay|garp_master_refresh|garp_master_refresh_repeat|garp_master_repeat|global_defs|global_tracking|gna_interval|group|ha_suspend|hashed|helo_name|higher_prio_send_advert|hoplimit|http_protocol|hysteresis|idle_tx|include|inhibit_on_failure|init_fail|init_file|instance|interface|interfaces|interval|ip_family|ipvs_process_name|keepalived.conf|kernel_rx_buf_size|key|linkbeat_interfaces|linkbeat_use_polling|log_all_failures|log_unknown_vrids|lower_prio_no_advert|lthreshold|lvs_flush|lvs_flush_onstop|lvs_method|lvs_netlink_cmd_rcv_bufs|lvs_netlink_cmd_rcv_bufs_force|lvs_netlink_monitor_rcv_bufs|lvs_netlink_monitor_rcv_bufs_force|lvs_notify_fifo|lvs_notify_fifo_script|lvs_sched|lvs_sync_daemon|max_auto_priority|max_hops|mcast_src_ip|mh-fallback|mh-port|min_auto_priority_delay|min_rx|min_tx|misc_dynamic|misc_path|misc_timeout|multiplier|name|namespace_with_ipsets|native_ipv6|neighbor_ip|net_namespace|net_namespace_ipvs|nftables|nftables_counters|nftables_ifindex|nftables_priority|no_accept|no_checker_emails|no_email_faults|nopreempt|notification_email|notification_email_from|notify|notify_backup|notify_deleted|notify_down|notify_fault|notify_fifo|notify_fifo_script|notify_master|notify_master_rx_lower_pri|notify_priority_changes|notify_stop|notify_up|old_unicast_checksum|omega|ops|param_match|passive|password|path|persistence_engine|persistence_granularity|persistence_timeout|preempt|preempt_delay|priority|process|process_monitor_rcv_bufs|process_monitor_rcv_bufs_force|process_name|process_names|promote_secondaries|protocol|proxy_arp|proxy_arp_pvlan|quorum|quorum_down|quorum_max|quorum_up|random_seed|real_server|regex|regex_max_offset|regex_min_offset|regex_no_match|regex_options|regex_stack|reload_repeat|reload_time_file|require_reply|retry|rise|router_id|rs_init_notifies|script|script_user|sh-fallback|sh-port|shutdown_script|shutdown_script_timeout|skip_check_adv_addr|smtp_alert|smtp_alert_checker|smtp_alert_vrrp|smtp_connect_timeout|smtp_helo_name|smtp_server|snmp_socket|sorry_server|sorry_server_inhibit|sorry_server_lvs_method|source_ip|start|startup_script|startup_script_timeout|state|static_ipaddress|static_routes|static_rules|status_code|step|strict_mode|sync_group_tracking_weight|terminate_delay|timeout|track_bfd|track_file|track_group|track_interface|track_process|track_script|track_src_ip|ttl|type|umask|unicast_peer|unicast_src_ip|unicast_ttl|url|use_ipvlan|use_pid_dir|use_vmac|user|uthreshold|val[123]|version|virtual_ipaddress|virtual_ipaddress_excluded|virtual_router_id|virtual_routes|virtual_rules|virtual_server|virtual_server_group|virtualhost|vmac_xmit_base|vrrp|vrrp_(?:check_unicast_src|cpu_affinity|garp_interval|garp_lower_prio_delay|garp_lower_prio_repeat|garp_master_delay|garp_master_refresh|garp_master_refresh_repeat|garp_master_repeat|gna_interval|higher_prio_send_advert|instance|ipsets|iptables|lower_prio_no_advert|mcast_group4|mcast_group6|min_garp|netlink_cmd_rcv_bufs|netlink_cmd_rcv_bufs_force|netlink_monitor_rcv_bufs|netlink_monitor_rcv_bufs_force|no_swap|notify_fifo|notify_fifo_script|notify_priority_changes|priority|process_name|rlimit_rttime|rt_priority|rx_bufs_multiplier|rx_bufs_policy|script|skip_check_adv_addr|startup_delay|strict|sync_group|track_process|version)|warmup|weight)\b/,
      constant:
        /\b(?:A|AAAA|AH|BACKUP|CNAME|DR|MASTER|MX|NAT|NS|PASS|SCTP|SOA|TCP|TUN|TXT|UDP|dh|fo|lblc|lblcr|lc|mh|nq|ovf|rr|sed|sh|wlc|wrr)\b/,
      number: {
        pattern: /(^|[^\w.-])-?\d+(?:\.\d+)?/,
        lookbehind: true
      },
      boolean: /\b(?:false|no|off|on|true|yes)\b/,
      punctuation: /[\{\}]/
    };
  }

  // @ts-nocheck
  keyman.displayName = 'keyman';
  keyman.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function keyman(Prism) {
    Prism.languages.keyman = {
      comment: {
        pattern: /\bc .*/i,
        greedy: true
      },
      string: {
        pattern: /"[^"\r\n]*"|'[^'\r\n]*'/,
        greedy: true
      },
      'virtual-key': {
        pattern:
          /\[\s*(?:(?:ALT|CAPS|CTRL|LALT|LCTRL|NCAPS|RALT|RCTRL|SHIFT)\s+)*(?:[TKU]_[\w?]+|[A-E]\d\d?|"[^"\r\n]*"|'[^'\r\n]*')\s*\]/i,
        greedy: true,
        alias: 'function' // alias for styles
      },
      // https://help.keyman.com/developer/language/guide/headers
      'header-keyword': {
        pattern: /&\w+/,
        alias: 'bold' // alias for styles
      },
      'header-statement': {
        pattern:
          /\b(?:bitmap|bitmaps|caps always off|caps on only|copyright|hotkey|language|layout|message|name|shift frees caps|version)\b/i,
        alias: 'bold' // alias for styles
      },
      'rule-keyword': {
        pattern:
          /\b(?:any|baselayout|beep|call|context|deadkey|dk|if|index|layer|notany|nul|outs|platform|reset|return|save|set|store|use)\b/i,
        alias: 'keyword'
      },
      'structural-keyword': {
        pattern: /\b(?:ansi|begin|group|match|nomatch|unicode|using keys)\b/i,
        alias: 'keyword'
      },
      'compile-target': {
        pattern: /\$(?:keyman|keymanonly|keymanweb|kmfl|weaver):/i,
        alias: 'property'
      },
      // U+####, x###, d### characters and numbers
      number: /\b(?:U\+[\dA-F]+|d\d+|x[\da-f]+|\d+)\b/i,
      operator: /[+>\\$]|\.\./,
      punctuation: /[()=,]/
    };
  }

  // @ts-nocheck
  kumir.displayName = 'kumir';
  kumir.aliases = ['kum'];

  /** @type {import('../core.js').Syntax} */
  function kumir(Prism) {
  (function (Prism) {
      /**
       * Regular expression for characters that are not allowed in identifiers.
       *
       * @type {string}
       */
      var nonId = /\s\x00-\x1f\x22-\x2f\x3a-\x3f\x5b-\x5e\x60\x7b-\x7e/.source;
      /**
       * Surround a regular expression for IDs with patterns for non-ID sequences.
       *
       * @param {string} pattern A regular expression for identifiers.
       * @param {string} [flags] The regular expression flags.
       * @returns {RegExp} A wrapped regular expression for identifiers.
       */
      function wrapId(pattern, flags) {
        return RegExp(pattern.replace(/<nonId>/g, nonId), flags)
      }
      Prism.languages.kumir = {
        comment: {
          pattern: /\|.*/
        },
        prolog: {
          pattern: /#.*/,
          greedy: true
        },
        string: {
          pattern: /"[^\n\r"]*"|'[^\n\r']*'/,
          greedy: true
        },
        boolean: {
          pattern: wrapId(/(^|[<nonId>])(?:да|нет)(?=[<nonId>]|$)/.source),
          lookbehind: true
        },
        'operator-word': {
          pattern: wrapId(/(^|[<nonId>])(?:и|или|не)(?=[<nonId>]|$)/.source),
          lookbehind: true,
          alias: 'keyword'
        },
        'system-variable': {
          pattern: wrapId(/(^|[<nonId>])знач(?=[<nonId>]|$)/.source),
          lookbehind: true,
          alias: 'keyword'
        },
        type: [
          {
            pattern: wrapId(
              /(^|[<nonId>])(?:вещ|лит|лог|сим|цел)(?:\x20*таб)?(?=[<nonId>]|$)/
                .source
            ),
            lookbehind: true,
            alias: 'builtin'
          },
          {
            pattern: wrapId(
              /(^|[<nonId>])(?:компл|сканкод|файл|цвет)(?=[<nonId>]|$)/.source
            ),
            lookbehind: true,
            alias: 'important'
          }
        ],
        /**
         * Should be performed after searching for type names because of "таб".
         * "таб" is a reserved word, but never used without a preceding type name.
         * "НАЗНАЧИТЬ", "Фввод", and "Фвывод" are not reserved words.
         */
        keyword: {
          pattern: wrapId(
            /(^|[<nonId>])(?:алг|арг(?:\x20*рез)?|ввод|ВКЛЮЧИТЬ|вс[её]|выбор|вывод|выход|дано|для|до|дс|если|иначе|исп|использовать|кон(?:(?:\x20+|_)исп)?|кц(?:(?:\x20+|_)при)?|надо|нач|нс|нц|от|пауза|пока|при|раза?|рез|стоп|таб|то|утв|шаг)(?=[<nonId>]|$)/
              .source
          ),
          lookbehind: true
        },
        /** Should be performed after searching for reserved words. */
        name: {
          // eslint-disable-next-line regexp/no-super-linear-backtracking
          pattern: wrapId(
            /(^|[<nonId>])[^\d<nonId>][^<nonId>]*(?:\x20+[^<nonId>]+)*(?=[<nonId>]|$)/
              .source
          ),
          lookbehind: true
        },
        /** Should be performed after searching for names. */
        number: {
          pattern: wrapId(
            /(^|[<nonId>])(?:\B\$[\da-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?)(?=[<nonId>]|$)/
              .source,
            'i'
          ),
          lookbehind: true
        },
        /** Should be performed after searching for words. */
        punctuation: /:=|[(),:;\[\]]/,
        /**
         * Should be performed after searching for
         * - numeric constants (because of "+" and "-");
         * - punctuation marks (because of ":=" and "=").
         */
        'operator-char': {
          pattern: /\*\*?|<[=>]?|>=?|[-+/=]/,
          alias: 'operator'
        }
      };
      Prism.languages.kum = Prism.languages.kumir;
    })(Prism);
  }

  // @ts-nocheck
  kusto.displayName = 'kusto';
  kusto.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function kusto(Prism) {
    Prism.languages.kusto = {
      comment: {
        pattern: /\/\/.*/,
        greedy: true
      },
      string: {
        pattern:
          /```[\s\S]*?```|[hH]?(?:"(?:[^\r\n\\"]|\\.)*"|'(?:[^\r\n\\']|\\.)*'|@(?:"[^\r\n"]*"|'[^\r\n']*'))/,
        greedy: true
      },
      verb: {
        pattern: /(\|\s*)[a-z][\w-]*/i,
        lookbehind: true,
        alias: 'keyword'
      },
      command: {
        pattern: /\.[a-z][a-z\d-]*\b/,
        alias: 'keyword'
      },
      'class-name':
        /\b(?:bool|datetime|decimal|dynamic|guid|int|long|real|string|timespan)\b/,
      keyword:
        /\b(?:access|alias|and|anti|as|asc|auto|between|by|(?:contains|(?:ends|starts)with|has(?:perfix|suffix)?)(?:_cs)?|database|declare|desc|external|from|fullouter|has_all|in|ingestion|inline|inner|innerunique|into|(?:left|right)(?:anti(?:semi)?|inner|outer|semi)?|let|like|local|not|of|on|or|pattern|print|query_parameters|range|restrict|schema|set|step|table|tables|to|view|where|with|matches\s+regex|nulls\s+(?:first|last))(?![\w-])/,
      boolean: /\b(?:false|null|true)\b/,
      function: /\b[a-z_]\w*(?=\s*\()/,
      datetime: [
        {
          // RFC 822 + RFC 850
          pattern:
            /\b(?:(?:Fri|Friday|Mon|Monday|Sat|Saturday|Sun|Sunday|Thu|Thursday|Tue|Tuesday|Wed|Wednesday)\s*,\s*)?\d{1,2}(?:\s+|-)(?:Apr|Aug|Dec|Feb|Jan|Jul|Jun|Mar|May|Nov|Oct|Sep)(?:\s+|-)\d{2}\s+\d{2}:\d{2}(?::\d{2})?(?:\s*(?:\b(?:[A-Z]|(?:[ECMT][DS]|GM|U)T)|[+-]\d{4}))?\b/,
          alias: 'number'
        },
        {
          // ISO 8601
          pattern:
            /[+-]?\b(?:\d{4}-\d{2}-\d{2}(?:[ T]\d{2}:\d{2}(?::\d{2}(?:\.\d+)?)?)?|\d{2}:\d{2}(?::\d{2}(?:\.\d+)?)?)Z?/,
          alias: 'number'
        }
      ],
      number:
        /\b(?:0x[0-9A-Fa-f]+|\d+(?:\.\d+)?(?:[Ee][+-]?\d+)?)(?:(?:min|sec|[mnµ]s|[dhms]|microsecond|tick)\b)?|[+-]?\binf\b/,
      operator: /=>|[!=]~|[!=<>]=?|[-+*/%|]|\.\./,
      punctuation: /[()\[\]{},;.:]/
    };
  }

  // @ts-nocheck
  latex.displayName = 'latex';
  latex.aliases = ['context', 'tex'];

  /** @type {import('../core.js').Syntax} */
  function latex(Prism) {
  (function (Prism) {
      var funcPattern = /\\(?:[^a-z()[\]]|[a-z*]+)/i;
      var insideEqu = {
        'equation-command': {
          pattern: funcPattern,
          alias: 'regex'
        }
      };
      Prism.languages.latex = {
        comment: /%.*/,
        // the verbatim environment prints whitespace to the document
        cdata: {
          pattern:
            /(\\begin\{((?:lstlisting|verbatim)\*?)\})[\s\S]*?(?=\\end\{\2\})/,
          lookbehind: true
        },
        /*
         * equations can be between $$ $$ or $ $ or \( \) or \[ \]
         * (all are multiline)
         */
        equation: [
          {
            pattern:
              /\$\$(?:\\[\s\S]|[^\\$])+\$\$|\$(?:\\[\s\S]|[^\\$])+\$|\\\([\s\S]*?\\\)|\\\[[\s\S]*?\\\]/,
            inside: insideEqu,
            alias: 'string'
          },
          {
            pattern:
              /(\\begin\{((?:align|eqnarray|equation|gather|math|multline)\*?)\})[\s\S]*?(?=\\end\{\2\})/,
            lookbehind: true,
            inside: insideEqu,
            alias: 'string'
          }
        ],
        /*
         * arguments which are keywords or references are highlighted
         * as keywords
         */
        keyword: {
          pattern:
            /(\\(?:begin|cite|documentclass|end|label|ref|usepackage)(?:\[[^\]]+\])?\{)[^}]+(?=\})/,
          lookbehind: true
        },
        url: {
          pattern: /(\\url\{)[^}]+(?=\})/,
          lookbehind: true
        },
        /*
         * section or chapter headlines are highlighted as bold so that
         * they stand out more
         */
        headline: {
          pattern:
            /(\\(?:chapter|frametitle|paragraph|part|section|subparagraph|subsection|subsubparagraph|subsubsection|subsubsubparagraph)\*?(?:\[[^\]]+\])?\{)[^}]+(?=\})/,
          lookbehind: true,
          alias: 'class-name'
        },
        function: {
          pattern: funcPattern,
          alias: 'selector'
        },
        punctuation: /[[\]{}&]/
      };
      Prism.languages.tex = Prism.languages.latex;
      Prism.languages.context = Prism.languages.latex;
    })(Prism);
  }

  // @ts-nocheck
  latte.displayName = 'latte';
  latte.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function latte(Prism) {
    Prism.register(clike);
    Prism.register(markupTemplating);
    Prism.register(php)
    ;(function (Prism) {
      Prism.languages.latte = {
        comment: /^\{\*[\s\S]*/,
        'latte-tag': {
          // https://latte.nette.org/en/tags
          pattern: /(^\{(?:\/(?=[a-z]))?)(?:[=_]|[a-z]\w*\b(?!\())/i,
          lookbehind: true,
          alias: 'important'
        },
        delimiter: {
          pattern: /^\{\/?|\}$/,
          alias: 'punctuation'
        },
        php: {
          pattern: /\S(?:[\s\S]*\S)?/,
          alias: 'language-php',
          inside: Prism.languages.php
        }
      };
      var markupLatte = Prism.languages.extend('markup', {});
      Prism.languages.insertBefore(
        'inside',
        'attr-value',
        {
          'n-attr': {
            pattern: /n:[\w-]+(?:\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+))?/,
            inside: {
              'attr-name': {
                pattern: /^[^\s=]+/,
                alias: 'important'
              },
              'attr-value': {
                pattern: /=[\s\S]+/,
                inside: {
                  punctuation: [
                    /^=/,
                    {
                      pattern: /^(\s*)["']|["']$/,
                      lookbehind: true
                    }
                  ],
                  php: {
                    pattern: /\S(?:[\s\S]*\S)?/,
                    inside: Prism.languages.php
                  }
                }
              }
            }
          }
        },
        markupLatte.tag
      );
      Prism.hooks.add('before-tokenize', function (env) {
        if (env.language !== 'latte') {
          return
        }
        var lattePattern =
          /\{\*[\s\S]*?\*\}|\{[^'"\s{}*](?:[^"'/{}]|\/(?![*/])|("|')(?:\\[\s\S]|(?!\1)[^\\])*\1|\/\*(?:[^*]|\*(?!\/))*\*\/)*\}/g;
        Prism.languages['markup-templating'].buildPlaceholders(
          env,
          'latte',
          lattePattern
        );
        env.grammar = markupLatte;
      });
      Prism.hooks.add('after-tokenize', function (env) {
        Prism.languages['markup-templating'].tokenizePlaceholders(env, 'latte');
      });
    })(Prism);
  }

  // @ts-nocheck
  scheme.displayName = 'scheme';
  scheme.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function scheme(Prism) {
  (function (Prism) {
      Prism.languages.scheme = {
        // this supports "normal" single-line comments:
        //   ; comment
        // and (potentially nested) multiline comments:
        //   #| comment #| nested |# still comment |#
        // (only 1 level of nesting is supported)
        comment:
          /;.*|#;\s*(?:\((?:[^()]|\([^()]*\))*\)|\[(?:[^\[\]]|\[[^\[\]]*\])*\])|#\|(?:[^#|]|#(?!\|)|\|(?!#)|#\|(?:[^#|]|#(?!\|)|\|(?!#))*\|#)*\|#/,
        string: {
          pattern: /"(?:[^"\\]|\\.)*"/,
          greedy: true
        },
        symbol: {
          pattern: /'[^()\[\]#'\s]+/,
          greedy: true
        },
        char: {
          pattern:
            /#\\(?:[ux][a-fA-F\d]+\b|[-a-zA-Z]+\b|[\uD800-\uDBFF][\uDC00-\uDFFF]|\S)/,
          greedy: true
        },
        'lambda-parameter': [
          // https://www.cs.cmu.edu/Groups/AI/html/r4rs/r4rs_6.html#SEC30
          {
            pattern:
              /((?:^|[^'`#])[(\[]lambda\s+)(?:[^|()\[\]'\s]+|\|(?:[^\\|]|\\.)*\|)/,
            lookbehind: true
          },
          {
            pattern: /((?:^|[^'`#])[(\[]lambda\s+[(\[])[^()\[\]']+/,
            lookbehind: true
          }
        ],
        keyword: {
          pattern:
            /((?:^|[^'`#])[(\[])(?:begin|case(?:-lambda)?|cond(?:-expand)?|define(?:-library|-macro|-record-type|-syntax|-values)?|defmacro|delay(?:-force)?|do|else|except|export|guard|if|import|include(?:-ci|-library-declarations)?|lambda|let(?:rec)?(?:-syntax|-values|\*)?|let\*-values|only|parameterize|prefix|(?:quasi-?)?quote|rename|set!|syntax-(?:case|rules)|unless|unquote(?:-splicing)?|when)(?=[()\[\]\s]|$)/,
          lookbehind: true
        },
        builtin: {
          // all functions of the base library of R7RS plus some of built-ins of R5Rs
          pattern:
            /((?:^|[^'`#])[(\[])(?:abs|and|append|apply|assoc|ass[qv]|binary-port\?|boolean=?\?|bytevector(?:-append|-copy|-copy!|-length|-u8-ref|-u8-set!|\?)?|caar|cadr|call-with-(?:current-continuation|port|values)|call\/cc|car|cdar|cddr|cdr|ceiling|char(?:->integer|-ready\?|\?|<\?|<=\?|=\?|>\?|>=\?)|close-(?:input-port|output-port|port)|complex\?|cons|current-(?:error|input|output)-port|denominator|dynamic-wind|eof-object\??|eq\?|equal\?|eqv\?|error|error-object(?:-irritants|-message|\?)|eval|even\?|exact(?:-integer-sqrt|-integer\?|\?)?|expt|features|file-error\?|floor(?:-quotient|-remainder|\/)?|flush-output-port|for-each|gcd|get-output-(?:bytevector|string)|inexact\??|input-port(?:-open\?|\?)|integer(?:->char|\?)|lcm|length|list(?:->string|->vector|-copy|-ref|-set!|-tail|\?)?|make-(?:bytevector|list|parameter|string|vector)|map|max|member|memq|memv|min|modulo|negative\?|newline|not|null\?|number(?:->string|\?)|numerator|odd\?|open-(?:input|output)-(?:bytevector|string)|or|output-port(?:-open\?|\?)|pair\?|peek-char|peek-u8|port\?|positive\?|procedure\?|quotient|raise|raise-continuable|rational\?|rationalize|read-(?:bytevector|bytevector!|char|error\?|line|string|u8)|real\?|remainder|reverse|round|set-c[ad]r!|square|string(?:->list|->number|->symbol|->utf8|->vector|-append|-copy|-copy!|-fill!|-for-each|-length|-map|-ref|-set!|\?|<\?|<=\?|=\?|>\?|>=\?)?|substring|symbol(?:->string|\?|=\?)|syntax-error|textual-port\?|truncate(?:-quotient|-remainder|\/)?|u8-ready\?|utf8->string|values|vector(?:->list|->string|-append|-copy|-copy!|-fill!|-for-each|-length|-map|-ref|-set!|\?)?|with-exception-handler|write-(?:bytevector|char|string|u8)|zero\?)(?=[()\[\]\s]|$)/,
          lookbehind: true
        },
        operator: {
          pattern: /((?:^|[^'`#])[(\[])(?:[-+*%/]|[<>]=?|=>?)(?=[()\[\]\s]|$)/,
          lookbehind: true
        },
        number: {
          // The number pattern from [the R7RS spec](https://small.r7rs.org/attachment/r7rs.pdf).
          //
          // <number>      := <num 2>|<num 8>|<num 10>|<num 16>
          // <num R>       := <prefix R><complex R>
          // <complex R>   := <real R>(?:@<real R>|<imaginary R>)?|<imaginary R>
          // <imaginary R> := [+-](?:<ureal R>|(?:inf|nan)\.0)?i
          // <real R>      := [+-]?<ureal R>|[+-](?:inf|nan)\.0
          // <ureal R>     := <uint R>(?:\/<uint R>)?
          //                | <decimal R>
          //
          // <decimal 10>  := (?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?
          // <uint R>      := <digit R>+
          // <prefix R>    := <radix R>(?:#[ei])?|(?:#[ei])?<radix R>
          // <radix 2>     := #b
          // <radix 8>     := #o
          // <radix 10>    := (?:#d)?
          // <radix 16>    := #x
          // <digit 2>     := [01]
          // <digit 8>     := [0-7]
          // <digit 10>    := \d
          // <digit 16>    := [0-9a-f]
          //
          // The problem with this grammar is that the resulting regex is way to complex, so we simplify by grouping all
          // non-decimal bases together. This results in a decimal (dec) and combined binary, octal, and hexadecimal (box)
          // pattern:
          pattern: RegExp(
            SortedBNF({
              '<ureal dec>':
                /\d+(?:\/\d+)|(?:\d+(?:\.\d*)?|\.\d+)(?:[esfdl][+-]?\d+)?/.source,
              '<real dec>': /[+-]?<ureal dec>|[+-](?:inf|nan)\.0/.source,
              '<imaginary dec>': /[+-](?:<ureal dec>|(?:inf|nan)\.0)?i/.source,
              '<complex dec>':
                /<real dec>(?:@<real dec>|<imaginary dec>)?|<imaginary dec>/
                  .source,
              '<num dec>': /(?:#d(?:#[ei])?|#[ei](?:#d)?)?<complex dec>/.source,
              '<ureal box>': /[0-9a-f]+(?:\/[0-9a-f]+)?/.source,
              '<real box>': /[+-]?<ureal box>|[+-](?:inf|nan)\.0/.source,
              '<imaginary box>': /[+-](?:<ureal box>|(?:inf|nan)\.0)?i/.source,
              '<complex box>':
                /<real box>(?:@<real box>|<imaginary box>)?|<imaginary box>/
                  .source,
              '<num box>': /#[box](?:#[ei])?|(?:#[ei])?#[box]<complex box>/
                .source,
              '<number>': /(^|[()\[\]\s])(?:<num dec>|<num box>)(?=[()\[\]\s]|$)/
                .source
            }),
            'i'
          ),
          lookbehind: true
        },
        boolean: {
          pattern: /(^|[()\[\]\s])#(?:[ft]|false|true)(?=[()\[\]\s]|$)/,
          lookbehind: true
        },
        function: {
          pattern:
            /((?:^|[^'`#])[(\[])(?:[^|()\[\]'\s]+|\|(?:[^\\|]|\\.)*\|)(?=[()\[\]\s]|$)/,
          lookbehind: true
        },
        identifier: {
          pattern: /(^|[()\[\]\s])\|(?:[^\\|]|\\.)*\|(?=[()\[\]\s]|$)/,
          lookbehind: true,
          greedy: true
        },
        punctuation: /[()\[\]']/
      };
      /**
       * Given a topologically sorted BNF grammar, this will return the RegExp source of last rule of the grammar.
       *
       * @param {Record<string, string>} grammar
       * @returns {string}
       */
      function SortedBNF(grammar) {
        for (var key in grammar) {
          grammar[key] = grammar[key].replace(/<[\w\s]+>/g, function (key) {
            return '(?:' + grammar[key].trim() + ')'
          });
        } // return the last item
        return grammar[key]
      }
    })(Prism);
  }

  // @ts-nocheck
  lilypond.displayName = 'lilypond';
  lilypond.aliases = ['ly'];

  /** @type {import('../core.js').Syntax} */
  function lilypond(Prism) {
    Prism.register(scheme)
    ;(function (Prism) {
      var schemeExpression =
        /\((?:[^();"#\\]|\\[\s\S]|;.*(?!.)|"(?:[^"\\]|\\.)*"|#(?:\{(?:(?!#\})[\s\S])*#\}|[^{])|<expr>)*\)/
          .source; // allow for up to pow(2, recursivenessLog2) many levels of recursive brace expressions
      // For some reason, this can't be 4
      var recursivenessLog2 = 5;
      for (var i = 0; i < recursivenessLog2; i++) {
        schemeExpression = schemeExpression.replace(/<expr>/g, function () {
          return schemeExpression
        });
      }
      schemeExpression = schemeExpression.replace(/<expr>/g, /[^\s\S]/.source);
      var lilypond = (Prism.languages.lilypond = {
        comment: /%(?:(?!\{).*|\{[\s\S]*?%\})/,
        'embedded-scheme': {
          pattern: RegExp(
            /(^|[=\s])#(?:"(?:[^"\\]|\\.)*"|[^\s()"]*(?:[^\s()]|<expr>))/.source.replace(
              /<expr>/g,
              function () {
                return schemeExpression
              }
            ),
            'm'
          ),
          lookbehind: true,
          greedy: true,
          inside: {
            scheme: {
              pattern: /^(#)[\s\S]+$/,
              lookbehind: true,
              alias: 'language-scheme',
              inside: {
                'embedded-lilypond': {
                  pattern: /#\{[\s\S]*?#\}/,
                  greedy: true,
                  inside: {
                    punctuation: /^#\{|#\}$/,
                    lilypond: {
                      pattern: /[\s\S]+/,
                      alias: 'language-lilypond',
                      inside: null // see below
                    }
                  }
                },
                rest: Prism.languages.scheme
              }
            },
            punctuation: /#/
          }
        },
        string: {
          pattern: /"(?:[^"\\]|\\.)*"/,
          greedy: true
        },
        'class-name': {
          pattern: /(\\new\s+)[\w-]+/,
          lookbehind: true
        },
        keyword: {
          pattern: /\\[a-z][-\w]*/i,
          inside: {
            punctuation: /^\\/
          }
        },
        operator: /[=|]|<<|>>/,
        punctuation: {
          pattern:
            /(^|[a-z\d])(?:'+|,+|[_^]?-[_^]?(?:[-+^!>._]|(?=\d))|[_^]\.?|[.!])|[{}()[\]<>^~]|\\[()[\]<>\\!]|--|__/,
          lookbehind: true
        },
        number: /\b\d+(?:\/\d+)?\b/
      });
      lilypond['embedded-scheme'].inside['scheme'].inside[
        'embedded-lilypond'
      ].inside['lilypond'].inside = lilypond;
      Prism.languages.ly = lilypond;
    })(Prism);
  }

  // @ts-nocheck
  liquid.displayName = 'liquid';
  liquid.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function liquid(Prism) {
    Prism.register(markupTemplating);
    Prism.languages.liquid = {
      comment: {
        pattern: /(^\{%\s*comment\s*%\})[\s\S]+(?=\{%\s*endcomment\s*%\}$)/,
        lookbehind: true
      },
      delimiter: {
        pattern: /^\{(?:\{\{|[%\{])-?|-?(?:\}\}|[%\}])\}$/,
        alias: 'punctuation'
      },
      string: {
        pattern: /"[^"]*"|'[^']*'/,
        greedy: true
      },
      keyword:
        /\b(?:as|assign|break|(?:end)?(?:capture|case|comment|for|form|if|paginate|raw|style|tablerow|unless)|continue|cycle|decrement|echo|else|elsif|in|include|increment|limit|liquid|offset|range|render|reversed|section|when|with)\b/,
      object:
        /\b(?:address|all_country_option_tags|article|block|blog|cart|checkout|collection|color|country|country_option_tags|currency|current_page|current_tags|customer|customer_address|date|discount_allocation|discount_application|external_video|filter|filter_value|font|forloop|fulfillment|generic_file|gift_card|group|handle|image|line_item|link|linklist|localization|location|measurement|media|metafield|model|model_source|order|page|page_description|page_image|page_title|part|policy|product|product_option|recommendations|request|robots|routes|rule|script|search|selling_plan|selling_plan_allocation|selling_plan_group|shipping_method|shop|shop_locale|sitemap|store_availability|tax_line|template|theme|transaction|unit_price_measurement|user_agent|variant|video|video_source)\b/,
      function: [
        {
          pattern: /(\|\s*)\w+/,
          lookbehind: true,
          alias: 'filter'
        },
        {
          // array functions
          pattern: /(\.\s*)(?:first|last|size)/,
          lookbehind: true
        }
      ],
      boolean: /\b(?:false|nil|true)\b/,
      range: {
        pattern: /\.\./,
        alias: 'operator'
      },
      // https://github.com/Shopify/liquid/blob/698f5e0d967423e013f6169d9111bd969bd78337/lib/liquid/lexer.rb#L21
      number: /\b\d+(?:\.\d+)?\b/,
      operator: /[!=]=|<>|[<>]=?|[|?:=-]|\b(?:and|contains(?=\s)|or)\b/,
      punctuation: /[.,\[\]()]/,
      empty: {
        pattern: /\bempty\b/,
        alias: 'keyword'
      }
    };
    Prism.hooks.add('before-tokenize', function (env) {
      var liquidPattern =
        /\{%\s*comment\s*%\}[\s\S]*?\{%\s*endcomment\s*%\}|\{(?:%[\s\S]*?%|\{\{[\s\S]*?\}\}|\{[\s\S]*?\})\}/g;
      var insideRaw = false;
      Prism.languages['markup-templating'].buildPlaceholders(
        env,
        'liquid',
        liquidPattern,
        function (match) {
          var tagMatch = /^\{%-?\s*(\w+)/.exec(match);
          if (tagMatch) {
            var tag = tagMatch[1];
            if (tag === 'raw' && !insideRaw) {
              insideRaw = true;
              return true
            } else if (tag === 'endraw') {
              insideRaw = false;
              return true
            }
          }
          return !insideRaw
        }
      );
    });
    Prism.hooks.add('after-tokenize', function (env) {
      Prism.languages['markup-templating'].tokenizePlaceholders(env, 'liquid');
    });
  }

  // @ts-nocheck
  lisp.displayName = 'lisp';
  lisp.aliases = ['elisp', 'emacs', 'emacs-lisp'];

  /** @type {import('../core.js').Syntax} */
  function lisp(Prism) {
  (function (Prism) {
      /**
       * Functions to construct regular expressions
       * e.g. (interactive ... or (interactive)
       *
       * @param {string} name
       * @returns {RegExp}
       */
      function simple_form(name) {
        return RegExp(/(\()/.source + '(?:' + name + ')' + /(?=[\s\)])/.source)
      }
      /**
       * booleans and numbers
       *
       * @param {string} pattern
       * @returns {RegExp}
       */
      function primitive(pattern) {
        return RegExp(
          /([\s([])/.source + '(?:' + pattern + ')' + /(?=[\s)])/.source
        )
      } // Patterns in regular expressions
      // Symbol name. See https://www.gnu.org/software/emacs/manual/html_node/elisp/Symbol-Type.html
      // & and : are excluded as they are usually used for special purposes
      var symbol = /(?!\d)[-+*/~!@$%^=<>{}\w]+/.source; // symbol starting with & used in function arguments
      var marker = '&' + symbol; // Open parenthesis for look-behind
      var par = '(\\()';
      var endpar = '(?=\\))'; // End the pattern with look-ahead space
      var space = '(?=\\s)';
      var nestedPar =
        /(?:[^()]|\((?:[^()]|\((?:[^()]|\((?:[^()]|\((?:[^()]|\([^()]*\))*\))*\))*\))*\))*/
          .source;
      var language = {
        // Three or four semicolons are considered a heading.
        // See https://www.gnu.org/software/emacs/manual/html_node/elisp/Comment-Tips.html
        heading: {
          pattern: /;;;.*/,
          alias: ['comment', 'title']
        },
        comment: /;.*/,
        string: {
          pattern: /"(?:[^"\\]|\\.)*"/,
          greedy: true,
          inside: {
            argument: /[-A-Z]+(?=[.,\s])/,
            symbol: RegExp('`' + symbol + "'")
          }
        },
        'quoted-symbol': {
          pattern: RegExp("#?'" + symbol),
          alias: ['variable', 'symbol']
        },
        'lisp-property': {
          pattern: RegExp(':' + symbol),
          alias: 'property'
        },
        splice: {
          pattern: RegExp(',@?' + symbol),
          alias: ['symbol', 'variable']
        },
        keyword: [
          {
            pattern: RegExp(
              par +
                '(?:and|(?:cl-)?letf|cl-loop|cond|cons|error|if|(?:lexical-)?let\\*?|message|not|null|or|provide|require|setq|unless|use-package|when|while)' +
                space
            ),
            lookbehind: true
          },
          {
            pattern: RegExp(
              par +
                '(?:append|by|collect|concat|do|finally|for|in|return)' +
                space
            ),
            lookbehind: true
          }
        ],
        declare: {
          pattern: simple_form(/declare/.source),
          lookbehind: true,
          alias: 'keyword'
        },
        interactive: {
          pattern: simple_form(/interactive/.source),
          lookbehind: true,
          alias: 'keyword'
        },
        boolean: {
          pattern: primitive(/nil|t/.source),
          lookbehind: true
        },
        number: {
          pattern: primitive(/[-+]?\d+(?:\.\d*)?/.source),
          lookbehind: true
        },
        defvar: {
          pattern: RegExp(par + 'def(?:const|custom|group|var)\\s+' + symbol),
          lookbehind: true,
          inside: {
            keyword: /^def[a-z]+/,
            variable: RegExp(symbol)
          }
        },
        defun: {
          pattern: RegExp(
            par +
              /(?:cl-)?(?:defmacro|defun\*?)\s+/.source +
              symbol +
              /\s+\(/.source +
              nestedPar +
              /\)/.source
          ),
          lookbehind: true,
          greedy: true,
          inside: {
            keyword: /^(?:cl-)?def\S+/,
            // See below, this property needs to be defined later so that it can
            // reference the language object.
            arguments: null,
            function: {
              pattern: RegExp('(^\\s)' + symbol),
              lookbehind: true
            },
            punctuation: /[()]/
          }
        },
        lambda: {
          pattern: RegExp(
            par +
              'lambda\\s+\\(\\s*(?:&?' +
              symbol +
              '(?:\\s+&?' +
              symbol +
              ')*\\s*)?\\)'
          ),
          lookbehind: true,
          greedy: true,
          inside: {
            keyword: /^lambda/,
            // See below, this property needs to be defined later so that it can
            // reference the language object.
            arguments: null,
            punctuation: /[()]/
          }
        },
        car: {
          pattern: RegExp(par + symbol),
          lookbehind: true
        },
        punctuation: [
          // open paren, brackets, and close paren
          /(?:['`,]?\(|[)\[\]])/, // cons
          {
            pattern: /(\s)\.(?=\s)/,
            lookbehind: true
          }
        ]
      };
      var arg = {
        'lisp-marker': RegExp(marker),
        varform: {
          pattern: RegExp(
            /\(/.source + symbol + /\s+(?=\S)/.source + nestedPar + /\)/.source
          ),
          inside: language
        },
        argument: {
          pattern: RegExp(/(^|[\s(])/.source + symbol),
          lookbehind: true,
          alias: 'variable'
        },
        rest: language
      };
      var forms = '\\S+(?:\\s+\\S+)*';
      var arglist = {
        pattern: RegExp(par + nestedPar + endpar),
        lookbehind: true,
        inside: {
          'rest-vars': {
            pattern: RegExp('&(?:body|rest)\\s+' + forms),
            inside: arg
          },
          'other-marker-vars': {
            pattern: RegExp('&(?:aux|optional)\\s+' + forms),
            inside: arg
          },
          keys: {
            pattern: RegExp('&key\\s+' + forms + '(?:\\s+&allow-other-keys)?'),
            inside: arg
          },
          argument: {
            pattern: RegExp(symbol),
            alias: 'variable'
          },
          punctuation: /[()]/
        }
      };
      language['lambda'].inside.arguments = arglist;
      language['defun'].inside.arguments = Prism.util.clone(arglist);
      language['defun'].inside.arguments.inside.sublist = arglist;
      Prism.languages.lisp = language;
      Prism.languages.elisp = language;
      Prism.languages.emacs = language;
      Prism.languages['emacs-lisp'] = language;
    })(Prism);
  }

  // @ts-nocheck
  livescript.displayName = 'livescript';
  livescript.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function livescript(Prism) {
    Prism.languages.livescript = {
      comment: [
        {
          pattern: /(^|[^\\])\/\*[\s\S]*?\*\//,
          lookbehind: true
        },
        {
          pattern: /(^|[^\\])#.*/,
          lookbehind: true
        }
      ],
      'interpolated-string': {
        /* Look-behind and look-ahead prevents wrong behavior of the greedy pattern
         * forcing it to match """-quoted string when it would otherwise match "-quoted first. */
        pattern: /(^|[^"])("""|")(?:\\[\s\S]|(?!\2)[^\\])*\2(?!")/,
        lookbehind: true,
        greedy: true,
        inside: {
          variable: {
            pattern: /(^|[^\\])#[a-z_](?:-?[a-z]|[\d_])*/m,
            lookbehind: true
          },
          interpolation: {
            pattern: /(^|[^\\])#\{[^}]+\}/m,
            lookbehind: true,
            inside: {
              'interpolation-punctuation': {
                pattern: /^#\{|\}$/,
                alias: 'variable'
              } // See rest below
            }
          },
          string: /[\s\S]+/
        }
      },
      string: [
        {
          pattern: /('''|')(?:\\[\s\S]|(?!\1)[^\\])*\1/,
          greedy: true
        },
        {
          pattern: /<\[[\s\S]*?\]>/,
          greedy: true
        },
        /\\[^\s,;\])}]+/
      ],
      regex: [
        {
          pattern: /\/\/(?:\[[^\r\n\]]*\]|\\.|(?!\/\/)[^\\\[])+\/\/[gimyu]{0,5}/,
          greedy: true,
          inside: {
            comment: {
              pattern: /(^|[^\\])#.*/,
              lookbehind: true
            }
          }
        },
        {
          pattern: /\/(?:\[[^\r\n\]]*\]|\\.|[^/\\\r\n\[])+\/[gimyu]{0,5}/,
          greedy: true
        }
      ],
      keyword: {
        pattern:
          /(^|(?!-).)\b(?:break|case|catch|class|const|continue|default|do|else|extends|fallthrough|finally|for(?: ever)?|function|if|implements|it|let|loop|new|null|otherwise|own|return|super|switch|that|then|this|throw|try|unless|until|var|void|when|while|yield)(?!-)\b/m,
        lookbehind: true
      },
      'keyword-operator': {
        pattern:
          /(^|[^-])\b(?:(?:delete|require|typeof)!|(?:and|by|delete|export|from|import(?: all)?|in|instanceof|is(?: not|nt)?|not|of|or|til|to|typeof|with|xor)(?!-)\b)/m,
        lookbehind: true,
        alias: 'operator'
      },
      boolean: {
        pattern: /(^|[^-])\b(?:false|no|off|on|true|yes)(?!-)\b/m,
        lookbehind: true
      },
      argument: {
        // Don't match .&. nor &&
        pattern: /(^|(?!\.&\.)[^&])&(?!&)\d*/m,
        lookbehind: true,
        alias: 'variable'
      },
      number: /\b(?:\d+~[\da-z]+|\d[\d_]*(?:\.\d[\d_]*)?(?:[a-z]\w*)?)/i,
      identifier: /[a-z_](?:-?[a-z]|[\d_])*/i,
      operator: [
        // Spaced .
        {
          pattern: /( )\.(?= )/,
          lookbehind: true
        }, // Full list, in order:
        // .= .~ .. ...
        // .&. .^. .<<. .>>. .>>>.
        // := :: ::=
        // &&
        // || |>
        // < << <<< <<<<
        // <- <-- <-! <--!
        // <~ <~~ <~! <~~!
        // <| <= <?
        // > >> >= >?
        // - -- -> -->
        // + ++
        // @ @@
        // % %%
        // * **
        // ! != !~=
        // !~> !~~>
        // !-> !-->
        // ~ ~> ~~> ~=
        // = ==
        // ^ ^^
        // / ?
        /\.(?:[=~]|\.\.?)|\.(?:[&|^]|<<|>>>?)\.|:(?:=|:=?)|&&|\|[|>]|<(?:<<?<?|--?!?|~~?!?|[|=?])?|>[>=?]?|-(?:->?|>)?|\+\+?|@@?|%%?|\*\*?|!(?:~?=|--?>|~?~>)?|~(?:~?>|=)?|==?|\^\^?|[\/?]/
      ],
      punctuation: /[(){}\[\]|.,:;`]/
    };
    Prism.languages.livescript['interpolated-string'].inside[
      'interpolation'
    ].inside.rest = Prism.languages.livescript;
  }

  // @ts-nocheck
  llvm.displayName = 'llvm';
  llvm.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function llvm(Prism) {
  (function (Prism) {
      Prism.languages.llvm = {
        comment: /;.*/,
        string: {
          pattern: /"[^"]*"/,
          greedy: true
        },
        boolean: /\b(?:false|true)\b/,
        variable: /[%@!#](?:(?!\d)(?:[-$.\w]|\\[a-f\d]{2})+|\d+)/i,
        label: /(?!\d)(?:[-$.\w]|\\[a-f\d]{2})+:/i,
        type: {
          pattern:
            /\b(?:double|float|fp128|half|i[1-9]\d*|label|metadata|ppc_fp128|token|void|x86_fp80|x86_mmx)\b/,
          alias: 'class-name'
        },
        keyword: /\b[a-z_][a-z_0-9]*\b/,
        number:
          /[+-]?\b\d+(?:\.\d+)?(?:[eE][+-]?\d+)?\b|\b0x[\dA-Fa-f]+\b|\b0xK[\dA-Fa-f]{20}\b|\b0x[ML][\dA-Fa-f]{32}\b|\b0xH[\dA-Fa-f]{4}\b/,
        punctuation: /[{}[\];(),.!*=<>]/
      };
    })(Prism);
  }

  // @ts-nocheck
  log.displayName = 'log';
  log.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function log(Prism) {
    // This is a language definition for generic log files.
    // Since there is no one log format, this language definition has to support all formats to some degree.
    //
    // Based on https://github.com/MTDL9/vim-log-highlighting
    Prism.languages.log = {
      string: {
        // Single-quoted strings must not be confused with plain text. E.g. Can't isn't Susan's Chris' toy
        pattern: /"(?:[^"\\\r\n]|\\.)*"|'(?![st] | \w)(?:[^'\\\r\n]|\\.)*'/,
        greedy: true
      },
      exception: {
        pattern:
          /(^|[^\w.])[a-z][\w.]*(?:Error|Exception):.*(?:(?:\r\n?|\n)[ \t]*(?:at[ \t].+|\.{3}.*|Caused by:.*))+(?:(?:\r\n?|\n)[ \t]*\.\.\. .*)?/,
        lookbehind: true,
        greedy: true,
        alias: ['javastacktrace', 'language-javastacktrace'],
        inside: Prism.languages['javastacktrace'] || {
          keyword: /\bat\b/,
          function: /[a-z_][\w$]*(?=\()/,
          punctuation: /[.:()]/
        }
      },
      level: [
        {
          pattern:
            /\b(?:ALERT|CRIT|CRITICAL|EMERG|EMERGENCY|ERR|ERROR|FAILURE|FATAL|SEVERE)\b/,
          alias: ['error', 'important']
        },
        {
          pattern: /\b(?:WARN|WARNING|WRN)\b/,
          alias: ['warning', 'important']
        },
        {
          pattern: /\b(?:DISPLAY|INF|INFO|NOTICE|STATUS)\b/,
          alias: ['info', 'keyword']
        },
        {
          pattern: /\b(?:DBG|DEBUG|FINE)\b/,
          alias: ['debug', 'keyword']
        },
        {
          pattern: /\b(?:FINER|FINEST|TRACE|TRC|VERBOSE|VRB)\b/,
          alias: ['trace', 'comment']
        }
      ],
      property: {
        pattern:
          /((?:^|[\]|])[ \t]*)[a-z_](?:[\w-]|\b\/\b)*(?:[. ]\(?\w(?:[\w-]|\b\/\b)*\)?)*:(?=\s)/im,
        lookbehind: true
      },
      separator: {
        pattern: /(^|[^-+])-{3,}|={3,}|\*{3,}|- - /m,
        lookbehind: true,
        alias: 'comment'
      },
      url: /\b(?:file|ftp|https?):\/\/[^\s|,;'"]*[^\s|,;'">.]/,
      email: {
        pattern: /(^|\s)[-\w+.]+@[a-z][a-z0-9-]*(?:\.[a-z][a-z0-9-]*)+(?=\s)/,
        lookbehind: true,
        alias: 'url'
      },
      'ip-address': {
        pattern: /\b(?:\d{1,3}(?:\.\d{1,3}){3})\b/,
        alias: 'constant'
      },
      'mac-address': {
        pattern: /\b[a-f0-9]{2}(?::[a-f0-9]{2}){5}\b/i,
        alias: 'constant'
      },
      domain: {
        pattern:
          /(^|\s)[a-z][a-z0-9-]*(?:\.[a-z][a-z0-9-]*)*\.[a-z][a-z0-9-]+(?=\s)/,
        lookbehind: true,
        alias: 'constant'
      },
      uuid: {
        pattern:
          /\b[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}\b/i,
        alias: 'constant'
      },
      hash: {
        pattern: /\b(?:[a-f0-9]{32}){1,2}\b/i,
        alias: 'constant'
      },
      'file-path': {
        pattern:
          /\b[a-z]:[\\/][^\s|,;:(){}\[\]"']+|(^|[\s:\[\](>|])\.{0,2}\/\w[^\s|,;:(){}\[\]"']*/i,
        lookbehind: true,
        greedy: true,
        alias: 'string'
      },
      date: {
        pattern: RegExp(
          /\b\d{4}[-/]\d{2}[-/]\d{2}(?:T(?=\d{1,2}:)|(?=\s\d{1,2}:))/.source +
            '|' +
            /\b\d{1,4}[-/ ](?:\d{1,2}|Apr|Aug|Dec|Feb|Jan|Jul|Jun|Mar|May|Nov|Oct|Sep)[-/ ]\d{2,4}T?\b/
              .source +
            '|' +
            /\b(?:(?:Fri|Mon|Sat|Sun|Thu|Tue|Wed)(?:\s{1,2}(?:Apr|Aug|Dec|Feb|Jan|Jul|Jun|Mar|May|Nov|Oct|Sep))?|Apr|Aug|Dec|Feb|Jan|Jul|Jun|Mar|May|Nov|Oct|Sep)\s{1,2}\d{1,2}\b/
              .source,
          'i'
        ),
        alias: 'number'
      },
      time: {
        pattern:
          /\b\d{1,2}:\d{1,2}:\d{1,2}(?:[.,:]\d+)?(?:\s?[+-]\d{2}:?\d{2}|Z)?\b/,
        alias: 'number'
      },
      boolean: /\b(?:false|null|true)\b/i,
      number: {
        pattern:
          /(^|[^.\w])(?:0x[a-f0-9]+|0o[0-7]+|0b[01]+|v?\d[\da-f]*(?:\.\d+)*(?:e[+-]?\d+)?[a-z]{0,3}\b)\b(?!\.\w)/i,
        lookbehind: true
      },
      operator: /[;:?<=>~/@!$%&+\-|^(){}*#]/,
      punctuation: /[\[\].,]/
    };
  }

  // @ts-nocheck
  lolcode.displayName = 'lolcode';
  lolcode.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function lolcode(Prism) {
    Prism.languages.lolcode = {
      comment: [/\bOBTW\s[\s\S]*?\sTLDR\b/, /\bBTW.+/],
      string: {
        pattern: /"(?::.|[^":])*"/,
        inside: {
          variable: /:\{[^}]+\}/,
          symbol: [/:\([a-f\d]+\)/i, /:\[[^\]]+\]/, /:[)>o":]/]
        },
        greedy: true
      },
      number: /(?:\B-)?(?:\b\d+(?:\.\d*)?|\B\.\d+)/,
      symbol: {
        pattern: /(^|\s)(?:A )?(?:BUKKIT|NOOB|NUMBAR|NUMBR|TROOF|YARN)(?=\s|,|$)/,
        lookbehind: true,
        inside: {
          keyword: /A(?=\s)/
        }
      },
      label: {
        pattern: /((?:^|\s)(?:IM IN YR|IM OUTTA YR) )[a-zA-Z]\w*/,
        lookbehind: true,
        alias: 'string'
      },
      function: {
        pattern: /((?:^|\s)(?:HOW IZ I|I IZ|IZ) )[a-zA-Z]\w*/,
        lookbehind: true
      },
      keyword: [
        {
          pattern:
            /(^|\s)(?:AN|FOUND YR|GIMMEH|GTFO|HAI|HAS A|HOW IZ I|I HAS A|I IZ|IF U SAY SO|IM IN YR|IM OUTTA YR|IS NOW(?: A)?|ITZ(?: A)?|IZ|KTHX|KTHXBYE|LIEK(?: A)?|MAEK|MEBBE|MKAY|NERFIN|NO WAI|O HAI IM|O RLY\?|OIC|OMG|OMGWTF|R|SMOOSH|SRS|TIL|UPPIN|VISIBLE|WILE|WTF\?|YA RLY|YR)(?=\s|,|$)/,
          lookbehind: true
        },
        /'Z(?=\s|,|$)/
      ],
      boolean: {
        pattern: /(^|\s)(?:FAIL|WIN)(?=\s|,|$)/,
        lookbehind: true
      },
      variable: {
        pattern: /(^|\s)IT(?=\s|,|$)/,
        lookbehind: true
      },
      operator: {
        pattern:
          /(^|\s)(?:NOT|BOTH SAEM|DIFFRINT|(?:ALL|ANY|BIGGR|BOTH|DIFF|EITHER|MOD|PRODUKT|QUOSHUNT|SMALLR|SUM|WON) OF)(?=\s|,|$)/,
        lookbehind: true
      },
      punctuation: /\.{3}|…|,|!/
    };
  }

  // @ts-nocheck
  magma.displayName = 'magma';
  magma.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function magma(Prism) {
    Prism.languages.magma = {
      output: {
        pattern:
          /^(>.*(?:\r(?:\n|(?!\n))|\n))(?!>)(?:.+|(?:\r(?:\n|(?!\n))|\n)(?!>).*)(?:(?:\r(?:\n|(?!\n))|\n)(?!>).*)*/m,
        lookbehind: true,
        greedy: true
      },
      comment: {
        pattern: /\/\/.*|\/\*[\s\S]*?\*\//,
        greedy: true
      },
      string: {
        pattern: /(^|[^\\"])"(?:[^\r\n\\"]|\\.)*"/,
        lookbehind: true,
        greedy: true
      },
      // http://magma.maths.usyd.edu.au/magma/handbook/text/82
      keyword:
        /\b(?:_|adj|and|assert|assert2|assert3|assigned|break|by|case|cat|catch|clear|cmpeq|cmpne|continue|declare|default|delete|diff|div|do|elif|else|end|eq|error|eval|exists|exit|for|forall|forward|fprintf|freeze|function|ge|gt|if|iload|import|in|intrinsic|is|join|le|load|local|lt|meet|mod|ne|not|notadj|notin|notsubset|or|print|printf|procedure|quit|random|read|readi|repeat|require|requirege|requirerange|restore|return|save|sdiff|select|subset|then|time|to|try|until|vprint|vprintf|vtime|when|where|while|xor)\b/,
      boolean: /\b(?:false|true)\b/,
      generator: {
        pattern: /\b[a-z_]\w*(?=\s*<)/i,
        alias: 'class-name'
      },
      function: /\b[a-z_]\w*(?=\s*\()/i,
      number: {
        pattern:
          /(^|[^\w.]|\.\.)(?:\d+(?:\.\d*)?|\.\d+)(?:[eE][+-]?\d+)?(?:_[a-z]?)?(?=$|[^\w.]|\.\.)/,
        lookbehind: true
      },
      operator: /->|[-+*/^~!|#=]|:=|\.\./,
      punctuation: /[()[\]{}<>,;.:]/
    };
  }

  // @ts-nocheck
  matlab.displayName = 'matlab';
  matlab.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function matlab(Prism) {
    Prism.languages.matlab = {
      comment: [/%\{[\s\S]*?\}%/, /%.+/],
      string: {
        pattern: /\B'(?:''|[^'\r\n])*'/,
        greedy: true
      },
      // FIXME We could handle imaginary numbers as a whole
      number: /(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[eE][+-]?\d+)?(?:[ij])?|\b[ij]\b/,
      keyword:
        /\b(?:NaN|break|case|catch|continue|else|elseif|end|for|function|if|inf|otherwise|parfor|pause|pi|return|switch|try|while)\b/,
      function: /\b(?!\d)\w+(?=\s*\()/,
      operator: /\.?[*^\/\\']|[+\-:@]|[<>=~]=?|&&?|\|\|?/,
      punctuation: /\.{3}|[.,;\[\](){}!]/
    };
  }

  // @ts-nocheck
  maxscript.displayName = 'maxscript';
  maxscript.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function maxscript(Prism) {
  (function (Prism) {
      var keywords =
        /\b(?:about|and|animate|as|at|attributes|by|case|catch|collect|continue|coordsys|do|else|exit|fn|for|from|function|global|if|in|local|macroscript|mapped|max|not|of|off|on|or|parameters|persistent|plugin|rcmenu|return|rollout|set|struct|then|throw|to|tool|try|undo|utility|when|where|while|with)\b/i;
      Prism.languages.maxscript = {
        comment: {
          pattern: /\/\*[\s\S]*?(?:\*\/|$)|--.*/,
          greedy: true
        },
        string: {
          pattern: /(^|[^"\\@])(?:"(?:[^"\\]|\\[\s\S])*"|@"[^"]*")/,
          lookbehind: true,
          greedy: true
        },
        path: {
          pattern: /\$(?:[\w/\\.*?]|'[^']*')*/,
          greedy: true,
          alias: 'string'
        },
        'function-call': {
          pattern: RegExp(
            '((?:' + // start of line
              (/^/.source +
                '|' + // operators and other language constructs
                /[;=<>+\-*/^({\[]/.source +
                '|' + // keywords as part of statements
                /\b(?:and|by|case|catch|collect|do|else|if|in|not|or|return|then|to|try|where|while|with)\b/
                  .source) +
              ')[ \t]*)' +
              '(?!' +
              keywords.source +
              ')' +
              /[a-z_]\w*\b/.source +
              '(?=[ \t]*(?:' + // variable
              ('(?!' +
                keywords.source +
                ')' +
                /[a-z_]/.source +
                '|' + // number
                /\d|-\.?\d/.source +
                '|' + // other expressions or literals
                /[({'"$@#?]/.source) +
              '))',
            'im'
          ),
          lookbehind: true,
          greedy: true,
          alias: 'function'
        },
        'function-definition': {
          pattern: /(\b(?:fn|function)\s+)\w+\b/i,
          lookbehind: true,
          alias: 'function'
        },
        argument: {
          pattern: /\b[a-z_]\w*(?=:)/i,
          alias: 'attr-name'
        },
        keyword: keywords,
        boolean: /\b(?:false|true)\b/,
        time: {
          pattern:
            /(^|[^\w.])(?:(?:(?:\d+(?:\.\d*)?|\.\d+)(?:[eEdD][+-]\d+|[LP])?[msft])+|\d+:\d+(?:\.\d*)?)(?![\w.:])/,
          lookbehind: true,
          alias: 'number'
        },
        number: [
          {
            pattern:
              /(^|[^\w.])(?:(?:\d+(?:\.\d*)?|\.\d+)(?:[eEdD][+-]\d+|[LP])?|0x[a-fA-F0-9]+)(?![\w.:])/,
            lookbehind: true
          },
          /\b(?:e|pi)\b/
        ],
        constant: /\b(?:dontcollect|ok|silentValue|undefined|unsupplied)\b/,
        color: {
          pattern: /\b(?:black|blue|brown|gray|green|orange|red|white|yellow)\b/i,
          alias: 'constant'
        },
        operator: /[-+*/<>=!]=?|[&^?]|#(?!\()/,
        punctuation: /[()\[\]{}.:,;]|#(?=\()|\\$/m
      };
    })(Prism);
  }

  // @ts-nocheck
  mel.displayName = 'mel';
  mel.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function mel(Prism) {
    Prism.languages.mel = {
      comment: /\/\/.*/,
      code: {
        pattern: /`(?:\\.|[^\\`\r\n])*`/,
        greedy: true,
        alias: 'italic',
        inside: {
          delimiter: {
            pattern: /^`|`$/,
            alias: 'punctuation'
          } // See rest below
        }
      },
      string: {
        pattern: /"(?:\\.|[^\\"\r\n])*"/,
        greedy: true
      },
      variable: /\$\w+/,
      number: /\b0x[\da-fA-F]+\b|\b\d+(?:\.\d*)?|\B\.\d+/,
      flag: {
        pattern: /-[^\d\W]\w*/,
        alias: 'operator'
      },
      keyword:
        /\b(?:break|case|continue|default|do|else|float|for|global|if|in|int|matrix|proc|return|string|switch|vector|while)\b/,
      function:
        /\b\w+(?=\()|\b(?:CBG|HfAddAttractorToAS|HfAssignAS|HfBuildEqualMap|HfBuildFurFiles|HfBuildFurImages|HfCancelAFR|HfConnectASToHF|HfCreateAttractor|HfDeleteAS|HfEditAS|HfPerformCreateAS|HfRemoveAttractorFromAS|HfSelectAttached|HfSelectAttractors|HfUnAssignAS|Mayatomr|about|abs|addAttr|addAttributeEditorNodeHelp|addDynamic|addNewShelfTab|addPP|addPanelCategory|addPrefixToName|advanceToNextDrivenKey|affectedNet|affects|aimConstraint|air|alias|aliasAttr|align|alignCtx|alignCurve|alignSurface|allViewFit|ambientLight|angle|angleBetween|animCone|animCurveEditor|animDisplay|animView|annotate|appendStringArray|applicationName|applyAttrPreset|applyTake|arcLenDimContext|arcLengthDimension|arclen|arrayMapper|art3dPaintCtx|artAttrCtx|artAttrPaintVertexCtx|artAttrSkinPaintCtx|artAttrTool|artBuildPaintMenu|artFluidAttrCtx|artPuttyCtx|artSelectCtx|artSetPaintCtx|artUserPaintCtx|assignCommand|assignInputDevice|assignViewportFactories|attachCurve|attachDeviceAttr|attachSurface|attrColorSliderGrp|attrCompatibility|attrControlGrp|attrEnumOptionMenu|attrEnumOptionMenuGrp|attrFieldGrp|attrFieldSliderGrp|attrNavigationControlGrp|attrPresetEditWin|attributeExists|attributeInfo|attributeMenu|attributeQuery|autoKeyframe|autoPlace|bakeClip|bakeFluidShading|bakePartialHistory|bakeResults|bakeSimulation|basename|basenameEx|batchRender|bessel|bevel|bevelPlus|binMembership|bindSkin|blend2|blendShape|blendShapeEditor|blendShapePanel|blendTwoAttr|blindDataType|boneLattice|boundary|boxDollyCtx|boxZoomCtx|bufferCurve|buildBookmarkMenu|buildKeyframeMenu|button|buttonManip|cacheFile|cacheFileCombine|cacheFileMerge|cacheFileTrack|camera|cameraView|canCreateManip|canvas|capitalizeString|catch|catchQuiet|ceil|changeSubdivComponentDisplayLevel|changeSubdivRegion|channelBox|character|characterMap|characterOutlineEditor|characterize|chdir|checkBox|checkBoxGrp|checkDefaultRenderGlobals|choice|circle|circularFillet|clamp|clear|clearCache|clip|clipEditor|clipEditorCurrentTimeCtx|clipSchedule|clipSchedulerOutliner|clipTrimBefore|closeCurve|closeSurface|cluster|cmdFileOutput|cmdScrollFieldExecuter|cmdScrollFieldReporter|cmdShell|coarsenSubdivSelectionList|collision|color|colorAtPoint|colorEditor|colorIndex|colorIndexSliderGrp|colorSliderButtonGrp|colorSliderGrp|columnLayout|commandEcho|commandLine|commandPort|compactHairSystem|componentEditor|compositingInterop|computePolysetVolume|condition|cone|confirmDialog|connectAttr|connectControl|connectDynamic|connectJoint|connectionInfo|constrain|constrainValue|constructionHistory|container|containsMultibyte|contextInfo|control|convertFromOldLayers|convertIffToPsd|convertLightmap|convertSolidTx|convertTessellation|convertUnit|copyArray|copyFlexor|copyKey|copySkinWeights|cos|cpButton|cpCache|cpClothSet|cpCollision|cpConstraint|cpConvClothToMesh|cpForces|cpGetSolverAttr|cpPanel|cpProperty|cpRigidCollisionFilter|cpSeam|cpSetEdit|cpSetSolverAttr|cpSolver|cpSolverTypes|cpTool|cpUpdateClothUVs|createDisplayLayer|createDrawCtx|createEditor|createLayeredPsdFile|createMotionField|createNewShelf|createNode|createRenderLayer|createSubdivRegion|cross|crossProduct|ctxAbort|ctxCompletion|ctxEditMode|ctxTraverse|currentCtx|currentTime|currentTimeCtx|currentUnit|curve|curveAddPtCtx|curveCVCtx|curveEPCtx|curveEditorCtx|curveIntersect|curveMoveEPCtx|curveOnSurface|curveSketchCtx|cutKey|cycleCheck|cylinder|dagPose|date|defaultLightListCheckBox|defaultNavigation|defineDataServer|defineVirtualDevice|deformer|deg_to_rad|delete|deleteAttr|deleteShadingGroupsAndMaterials|deleteShelfTab|deleteUI|deleteUnusedBrushes|delrandstr|detachCurve|detachDeviceAttr|detachSurface|deviceEditor|devicePanel|dgInfo|dgdirty|dgeval|dgtimer|dimWhen|directKeyCtx|directionalLight|dirmap|dirname|disable|disconnectAttr|disconnectJoint|diskCache|displacementToPoly|displayAffected|displayColor|displayCull|displayLevelOfDetail|displayPref|displayRGBColor|displaySmoothness|displayStats|displayString|displaySurface|distanceDimContext|distanceDimension|doBlur|dolly|dollyCtx|dopeSheetEditor|dot|dotProduct|doubleProfileBirailSurface|drag|dragAttrContext|draggerContext|dropoffLocator|duplicate|duplicateCurve|duplicateSurface|dynCache|dynControl|dynExport|dynExpression|dynGlobals|dynPaintEditor|dynParticleCtx|dynPref|dynRelEdPanel|dynRelEditor|dynamicLoad|editAttrLimits|editDisplayLayerGlobals|editDisplayLayerMembers|editRenderLayerAdjustment|editRenderLayerGlobals|editRenderLayerMembers|editor|editorTemplate|effector|emit|emitter|enableDevice|encodeString|endString|endsWith|env|equivalent|equivalentTol|erf|error|eval|evalDeferred|evalEcho|event|exactWorldBoundingBox|exclusiveLightCheckBox|exec|executeForEachObject|exists|exp|expression|expressionEditorListen|extendCurve|extendSurface|extrude|fcheck|fclose|feof|fflush|fgetline|fgetword|file|fileBrowserDialog|fileDialog|fileExtension|fileInfo|filetest|filletCurve|filter|filterCurve|filterExpand|filterStudioImport|findAllIntersections|findAnimCurves|findKeyframe|findMenuItem|findRelatedSkinCluster|finder|firstParentOf|fitBspline|flexor|floatEq|floatField|floatFieldGrp|floatScrollBar|floatSlider|floatSlider2|floatSliderButtonGrp|floatSliderGrp|floor|flow|fluidCacheInfo|fluidEmitter|fluidVoxelInfo|flushUndo|fmod|fontDialog|fopen|formLayout|format|fprint|frameLayout|fread|freeFormFillet|frewind|fromNativePath|fwrite|gamma|gauss|geometryConstraint|getApplicationVersionAsFloat|getAttr|getClassification|getDefaultBrush|getFileList|getFluidAttr|getInputDeviceRange|getMayaPanelTypes|getModifiers|getPanel|getParticleAttr|getPluginResource|getenv|getpid|glRender|glRenderEditor|globalStitch|gmatch|goal|gotoBindPose|grabColor|gradientControl|gradientControlNoAttr|graphDollyCtx|graphSelectContext|graphTrackCtx|gravity|grid|gridLayout|group|groupObjectsByName|hardenPointCurve|hardware|hardwareRenderPanel|headsUpDisplay|headsUpMessage|help|helpLine|hermite|hide|hilite|hitTest|hotBox|hotkey|hotkeyCheck|hsv_to_rgb|hudButton|hudSlider|hudSliderButton|hwReflectionMap|hwRender|hwRenderLoad|hyperGraph|hyperPanel|hyperShade|hypot|iconTextButton|iconTextCheckBox|iconTextRadioButton|iconTextRadioCollection|iconTextScrollList|iconTextStaticLabel|ikHandle|ikHandleCtx|ikHandleDisplayScale|ikSolver|ikSplineHandleCtx|ikSystem|ikSystemInfo|ikfkDisplayMethod|illustratorCurves|image|imfPlugins|inheritTransform|insertJoint|insertJointCtx|insertKeyCtx|insertKnotCurve|insertKnotSurface|instance|instanceable|instancer|intField|intFieldGrp|intScrollBar|intSlider|intSliderGrp|interToUI|internalVar|intersect|iprEngine|isAnimCurve|isConnected|isDirty|isParentOf|isSameObject|isTrue|isValidObjectName|isValidString|isValidUiName|isolateSelect|itemFilter|itemFilterAttr|itemFilterRender|itemFilterType|joint|jointCluster|jointCtx|jointDisplayScale|jointLattice|keyTangent|keyframe|keyframeOutliner|keyframeRegionCurrentTimeCtx|keyframeRegionDirectKeyCtx|keyframeRegionDollyCtx|keyframeRegionInsertKeyCtx|keyframeRegionMoveKeyCtx|keyframeRegionScaleKeyCtx|keyframeRegionSelectKeyCtx|keyframeRegionSetKeyCtx|keyframeRegionTrackCtx|keyframeStats|lassoContext|lattice|latticeDeformKeyCtx|launch|launchImageEditor|layerButton|layeredShaderPort|layeredTexturePort|layout|layoutDialog|lightList|lightListEditor|lightListPanel|lightlink|lineIntersection|linearPrecision|linstep|listAnimatable|listAttr|listCameras|listConnections|listDeviceAttachments|listHistory|listInputDeviceAxes|listInputDeviceButtons|listInputDevices|listMenuAnnotation|listNodeTypes|listPanelCategories|listRelatives|listSets|listTransforms|listUnselected|listerEditor|loadFluid|loadNewShelf|loadPlugin|loadPluginLanguageResources|loadPrefObjects|localizedPanelLabel|lockNode|loft|log|longNameOf|lookThru|ls|lsThroughFilter|lsType|lsUI|mag|makeIdentity|makeLive|makePaintable|makeRoll|makeSingleSurface|makeTubeOn|makebot|manipMoveContext|manipMoveLimitsCtx|manipOptions|manipRotateContext|manipRotateLimitsCtx|manipScaleContext|manipScaleLimitsCtx|marker|match|max|memory|menu|menuBarLayout|menuEditor|menuItem|menuItemToShelf|menuSet|menuSetPref|messageLine|min|minimizeApp|mirrorJoint|modelCurrentTimeCtx|modelEditor|modelPanel|mouse|movIn|movOut|move|moveIKtoFK|moveKeyCtx|moveVertexAlongDirection|multiProfileBirailSurface|mute|nParticle|nameCommand|nameField|namespace|namespaceInfo|newPanelItems|newton|nodeCast|nodeIconButton|nodeOutliner|nodePreset|nodeType|noise|nonLinear|normalConstraint|normalize|nurbsBoolean|nurbsCopyUVSet|nurbsCube|nurbsEditUV|nurbsPlane|nurbsSelect|nurbsSquare|nurbsToPoly|nurbsToPolygonsPref|nurbsToSubdiv|nurbsToSubdivPref|nurbsUVSet|nurbsViewDirectionVector|objExists|objectCenter|objectLayer|objectType|objectTypeUI|obsoleteProc|oceanNurbsPreviewPlane|offsetCurve|offsetCurveOnSurface|offsetSurface|openGLExtension|openMayaPref|optionMenu|optionMenuGrp|optionVar|orbit|orbitCtx|orientConstraint|outlinerEditor|outlinerPanel|overrideModifier|paintEffectsDisplay|pairBlend|palettePort|paneLayout|panel|panelConfiguration|panelHistory|paramDimContext|paramDimension|paramLocator|parent|parentConstraint|particle|particleExists|particleInstancer|particleRenderInfo|partition|pasteKey|pathAnimation|pause|pclose|percent|performanceOptions|pfxstrokes|pickWalk|picture|pixelMove|planarSrf|plane|play|playbackOptions|playblast|plugAttr|plugNode|pluginInfo|pluginResourceUtil|pointConstraint|pointCurveConstraint|pointLight|pointMatrixMult|pointOnCurve|pointOnSurface|pointPosition|poleVectorConstraint|polyAppend|polyAppendFacetCtx|polyAppendVertex|polyAutoProjection|polyAverageNormal|polyAverageVertex|polyBevel|polyBlendColor|polyBlindData|polyBoolOp|polyBridgeEdge|polyCacheMonitor|polyCheck|polyChipOff|polyClipboard|polyCloseBorder|polyCollapseEdge|polyCollapseFacet|polyColorBlindData|polyColorDel|polyColorPerVertex|polyColorSet|polyCompare|polyCone|polyCopyUV|polyCrease|polyCreaseCtx|polyCreateFacet|polyCreateFacetCtx|polyCube|polyCut|polyCutCtx|polyCylinder|polyCylindricalProjection|polyDelEdge|polyDelFacet|polyDelVertex|polyDuplicateAndConnect|polyDuplicateEdge|polyEditUV|polyEditUVShell|polyEvaluate|polyExtrudeEdge|polyExtrudeFacet|polyExtrudeVertex|polyFlipEdge|polyFlipUV|polyForceUV|polyGeoSampler|polyHelix|polyInfo|polyInstallAction|polyLayoutUV|polyListComponentConversion|polyMapCut|polyMapDel|polyMapSew|polyMapSewMove|polyMergeEdge|polyMergeEdgeCtx|polyMergeFacet|polyMergeFacetCtx|polyMergeUV|polyMergeVertex|polyMirrorFace|polyMoveEdge|polyMoveFacet|polyMoveFacetUV|polyMoveUV|polyMoveVertex|polyNormal|polyNormalPerVertex|polyNormalizeUV|polyOptUvs|polyOptions|polyOutput|polyPipe|polyPlanarProjection|polyPlane|polyPlatonicSolid|polyPoke|polyPrimitive|polyPrism|polyProjection|polyPyramid|polyQuad|polyQueryBlindData|polyReduce|polySelect|polySelectConstraint|polySelectConstraintMonitor|polySelectCtx|polySelectEditCtx|polySeparate|polySetToFaceNormal|polySewEdge|polyShortestPathCtx|polySmooth|polySoftEdge|polySphere|polySphericalProjection|polySplit|polySplitCtx|polySplitEdge|polySplitRing|polySplitVertex|polyStraightenUVBorder|polySubdivideEdge|polySubdivideFacet|polyToSubdiv|polyTorus|polyTransfer|polyTriangulate|polyUVSet|polyUnite|polyWedgeFace|popen|popupMenu|pose|pow|preloadRefEd|print|progressBar|progressWindow|projFileViewer|projectCurve|projectTangent|projectionContext|projectionManip|promptDialog|propModCtx|propMove|psdChannelOutliner|psdEditTextureFile|psdExport|psdTextureFile|putenv|pwd|python|querySubdiv|quit|rad_to_deg|radial|radioButton|radioButtonGrp|radioCollection|radioMenuItemCollection|rampColorPort|rand|randomizeFollicles|randstate|rangeControl|readTake|rebuildCurve|rebuildSurface|recordAttr|recordDevice|redo|reference|referenceEdit|referenceQuery|refineSubdivSelectionList|refresh|refreshAE|registerPluginResource|rehash|reloadImage|removeJoint|removeMultiInstance|removePanelCategory|rename|renameAttr|renameSelectionList|renameUI|render|renderGlobalsNode|renderInfo|renderLayerButton|renderLayerParent|renderLayerPostProcess|renderLayerUnparent|renderManip|renderPartition|renderQualityNode|renderSettings|renderThumbnailUpdate|renderWindowEditor|renderWindowSelectContext|renderer|reorder|reorderDeformers|requires|reroot|resampleFluid|resetAE|resetPfxToPolyCamera|resetTool|resolutionNode|retarget|reverseCurve|reverseSurface|revolve|rgb_to_hsv|rigidBody|rigidSolver|roll|rollCtx|rootOf|rot|rotate|rotationInterpolation|roundConstantRadius|rowColumnLayout|rowLayout|runTimeCommand|runup|sampleImage|saveAllShelves|saveAttrPreset|saveFluid|saveImage|saveInitialState|saveMenu|savePrefObjects|savePrefs|saveShelf|saveToolSettings|scale|scaleBrushBrightness|scaleComponents|scaleConstraint|scaleKey|scaleKeyCtx|sceneEditor|sceneUIReplacement|scmh|scriptCtx|scriptEditorInfo|scriptJob|scriptNode|scriptTable|scriptToShelf|scriptedPanel|scriptedPanelType|scrollField|scrollLayout|sculpt|searchPathArray|seed|selLoadSettings|select|selectContext|selectCurveCV|selectKey|selectKeyCtx|selectKeyframeRegionCtx|selectMode|selectPref|selectPriority|selectType|selectedNodes|selectionConnection|separator|setAttr|setAttrEnumResource|setAttrMapping|setAttrNiceNameResource|setConstraintRestPosition|setDefaultShadingGroup|setDrivenKeyframe|setDynamic|setEditCtx|setEditor|setFluidAttr|setFocus|setInfinity|setInputDeviceMapping|setKeyCtx|setKeyPath|setKeyframe|setKeyframeBlendshapeTargetWts|setMenuMode|setNodeNiceNameResource|setNodeTypeFlag|setParent|setParticleAttr|setPfxToPolyCamera|setPluginResource|setProject|setStampDensity|setStartupMessage|setState|setToolTo|setUITemplate|setXformManip|sets|shadingConnection|shadingGeometryRelCtx|shadingLightRelCtx|shadingNetworkCompare|shadingNode|shapeCompare|shelfButton|shelfLayout|shelfTabLayout|shellField|shortNameOf|showHelp|showHidden|showManipCtx|showSelectionInTitle|showShadingGroupAttrEditor|showWindow|sign|simplify|sin|singleProfileBirailSurface|size|sizeBytes|skinCluster|skinPercent|smoothCurve|smoothTangentSurface|smoothstep|snap2to2|snapKey|snapMode|snapTogetherCtx|snapshot|soft|softMod|softModCtx|sort|sound|soundControl|source|spaceLocator|sphere|sphrand|spotLight|spotLightPreviewPort|spreadSheetEditor|spring|sqrt|squareSurface|srtContext|stackTrace|startString|startsWith|stitchAndExplodeShell|stitchSurface|stitchSurfacePoints|strcmp|stringArrayCatenate|stringArrayContains|stringArrayCount|stringArrayInsertAtIndex|stringArrayIntersector|stringArrayRemove|stringArrayRemoveAtIndex|stringArrayRemoveDuplicates|stringArrayRemoveExact|stringArrayToString|stringToStringArray|strip|stripPrefixFromName|stroke|subdAutoProjection|subdCleanTopology|subdCollapse|subdDuplicateAndConnect|subdEditUV|subdListComponentConversion|subdMapCut|subdMapSewMove|subdMatchTopology|subdMirror|subdToBlind|subdToPoly|subdTransferUVsToCache|subdiv|subdivCrease|subdivDisplaySmoothness|substitute|substituteAllString|substituteGeometry|substring|surface|surfaceSampler|surfaceShaderList|swatchDisplayPort|switchTable|symbolButton|symbolCheckBox|sysFile|system|tabLayout|tan|tangentConstraint|texLatticeDeformContext|texManipContext|texMoveContext|texMoveUVShellContext|texRotateContext|texScaleContext|texSelectContext|texSelectShortestPathCtx|texSmudgeUVContext|texWinToolCtx|text|textCurves|textField|textFieldButtonGrp|textFieldGrp|textManip|textScrollList|textToShelf|textureDisplacePlane|textureHairColor|texturePlacementContext|textureWindow|threadCount|threePointArcCtx|timeControl|timePort|timerX|toNativePath|toggle|toggleAxis|toggleWindowVisibility|tokenize|tokenizeList|tolerance|tolower|toolButton|toolCollection|toolDropped|toolHasOptions|toolPropertyWindow|torus|toupper|trace|track|trackCtx|transferAttributes|transformCompare|transformLimits|translator|trim|trunc|truncateFluidCache|truncateHairCache|tumble|tumbleCtx|turbulence|twoPointArcCtx|uiRes|uiTemplate|unassignInputDevice|undo|undoInfo|ungroup|uniform|unit|unloadPlugin|untangleUV|untitledFileName|untrim|upAxis|updateAE|userCtx|uvLink|uvSnapshot|validateShelfName|vectorize|view2dToolCtx|viewCamera|viewClipPlane|viewFit|viewHeadOn|viewLookAt|viewManip|viewPlace|viewSet|visor|volumeAxis|vortex|waitCursor|warning|webBrowser|webBrowserPrefs|whatIs|window|windowPref|wire|wireContext|workspace|wrinkle|wrinkleContext|writeTake|xbmLangPathList|xform)\b/,
      operator: [
        /\+[+=]?|-[-=]?|&&|\|\||[<>]=|[*\/!=]=?|[%^]/,
        {
          // We don't want to match <<
          pattern: /(^|[^<])<(?!<)/,
          lookbehind: true
        },
        {
          // We don't want to match >>
          pattern: /(^|[^>])>(?!>)/,
          lookbehind: true
        }
      ],
      punctuation: /<<|>>|[.,:;?\[\](){}]/
    };
    Prism.languages.mel['code'].inside.rest = Prism.languages.mel;
  }

  // @ts-nocheck
  mermaid.displayName = 'mermaid';
  mermaid.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function mermaid(Prism) {
    Prism.languages.mermaid = {
      comment: {
        pattern: /%%.*/,
        greedy: true
      },
      style: {
        pattern:
          /^([ \t]*(?:classDef|linkStyle|style)[ \t]+[\w$-]+[ \t]+)\w.*[^\s;]/m,
        lookbehind: true,
        inside: {
          property: /\b\w[\w-]*(?=[ \t]*:)/,
          operator: /:/,
          punctuation: /,/
        }
      },
      'inter-arrow-label': {
        pattern:
          /([^<>ox.=-])(?:-[-.]|==)(?![<>ox.=-])[ \t]*(?:"[^"\r\n]*"|[^\s".=-](?:[^\r\n.=-]*[^\s.=-])?)[ \t]*(?:\.+->?|--+[->]|==+[=>])(?![<>ox.=-])/,
        lookbehind: true,
        greedy: true,
        inside: {
          arrow: {
            pattern: /(?:\.+->?|--+[->]|==+[=>])$/,
            alias: 'operator'
          },
          label: {
            pattern: /^([\s\S]{2}[ \t]*)\S(?:[\s\S]*\S)?/,
            lookbehind: true,
            alias: 'property'
          },
          'arrow-head': {
            pattern: /^\S+/,
            alias: ['arrow', 'operator']
          }
        }
      },
      arrow: [
        // This might look complex but it really isn't.
        // There are many possible arrows (see tests) and it's impossible to fit all of them into one pattern. The
        // problem is that we only have one lookbehind per pattern. However, we cannot disallow too many arrow
        // characters in the one lookbehind because that would create too many false negatives. So we have to split the
        // arrows into different patterns.
        {
          // ER diagram
          pattern: /(^|[^{}|o.-])[|}][|o](?:--|\.\.)[|o][|{](?![{}|o.-])/,
          lookbehind: true,
          alias: 'operator'
        },
        {
          // flow chart
          // (?:==+|--+|-\.*-)
          pattern:
            /(^|[^<>ox.=-])(?:[<ox](?:==+|--+|-\.*-)[>ox]?|(?:==+|--+|-\.*-)[>ox]|===+|---+|-\.+-)(?![<>ox.=-])/,
          lookbehind: true,
          alias: 'operator'
        },
        {
          // sequence diagram
          pattern:
            /(^|[^<>()x-])(?:--?(?:>>|[x>)])(?![<>()x])|(?:<<|[x<(])--?(?!-))/,
          lookbehind: true,
          alias: 'operator'
        },
        {
          // class diagram
          pattern:
            /(^|[^<>|*o.-])(?:[*o]--|--[*o]|<\|?(?:--|\.\.)|(?:--|\.\.)\|?>|--|\.\.)(?![<>|*o.-])/,
          lookbehind: true,
          alias: 'operator'
        }
      ],
      label: {
        pattern: /(^|[^|<])\|(?:[^\r\n"|]|"[^"\r\n]*")+\|/,
        lookbehind: true,
        greedy: true,
        alias: 'property'
      },
      text: {
        pattern: /(?:[(\[{]+|\b>)(?:[^\r\n"()\[\]{}]|"[^"\r\n]*")+(?:[)\]}]+|>)/,
        alias: 'string'
      },
      string: {
        pattern: /"[^"\r\n]*"/,
        greedy: true
      },
      annotation: {
        pattern:
          /<<(?:abstract|choice|enumeration|fork|interface|join|service)>>|\[\[(?:choice|fork|join)\]\]/i,
        alias: 'important'
      },
      keyword: [
        // This language has both case-sensitive and case-insensitive keywords
        {
          pattern:
            /(^[ \t]*)(?:action|callback|class|classDef|classDiagram|click|direction|erDiagram|flowchart|gantt|gitGraph|graph|journey|link|linkStyle|pie|requirementDiagram|sequenceDiagram|stateDiagram|stateDiagram-v2|style|subgraph)(?![\w$-])/m,
          lookbehind: true,
          greedy: true
        },
        {
          pattern:
            /(^[ \t]*)(?:activate|alt|and|as|autonumber|deactivate|else|end(?:[ \t]+note)?|loop|opt|par|participant|rect|state|note[ \t]+(?:over|(?:left|right)[ \t]+of))(?![\w$-])/im,
          lookbehind: true,
          greedy: true
        }
      ],
      entity: /#[a-z0-9]+;/,
      operator: {
        pattern: /(\w[ \t]*)&(?=[ \t]*\w)|:::|:/,
        lookbehind: true
      },
      punctuation: /[(){};]/
    };
  }

  // @ts-nocheck
  mizar.displayName = 'mizar';
  mizar.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function mizar(Prism) {
    Prism.languages.mizar = {
      comment: /::.+/,
      keyword:
        /@proof\b|\b(?:according|aggregate|all|and|antonym|are|as|associativity|assume|asymmetry|attr|be|begin|being|by|canceled|case|cases|clusters?|coherence|commutativity|compatibility|connectedness|consider|consistency|constructors|contradiction|correctness|def|deffunc|define|definitions?|defpred|do|does|end|environ|equals|ex|exactly|existence|for|from|func|given|hence|hereby|holds|idempotence|identity|iff?|implies|involutiveness|irreflexivity|is|it|let|means|mode|non|not|notations?|now|of|or|otherwise|over|per|pred|prefix|projectivity|proof|provided|qua|reconsider|redefine|reduce|reducibility|reflexivity|registrations?|requirements|reserve|sch|schemes?|section|selector|set|sethood|st|struct|such|suppose|symmetry|synonym|take|that|the|then|theorems?|thesis|thus|to|transitivity|uniqueness|vocabular(?:ies|y)|when|where|with|wrt)\b/,
      parameter: {
        pattern: /\$(?:10|\d)/,
        alias: 'variable'
      },
      variable: /\b\w+(?=:)/,
      number: /(?:\b|-)\d+\b/,
      operator: /\.\.\.|->|&|\.?=/,
      punctuation: /\(#|#\)|[,:;\[\](){}]/
    };
  }

  // @ts-nocheck
  mongodb.displayName = 'mongodb';
  mongodb.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function mongodb(Prism) {
    Prism.register(javascript)
    ;(function (Prism) {
      var operators = [
        // query and projection
        '$eq',
        '$gt',
        '$gte',
        '$in',
        '$lt',
        '$lte',
        '$ne',
        '$nin',
        '$and',
        '$not',
        '$nor',
        '$or',
        '$exists',
        '$type',
        '$expr',
        '$jsonSchema',
        '$mod',
        '$regex',
        '$text',
        '$where',
        '$geoIntersects',
        '$geoWithin',
        '$near',
        '$nearSphere',
        '$all',
        '$elemMatch',
        '$size',
        '$bitsAllClear',
        '$bitsAllSet',
        '$bitsAnyClear',
        '$bitsAnySet',
        '$comment',
        '$elemMatch',
        '$meta',
        '$slice', // update
        '$currentDate',
        '$inc',
        '$min',
        '$max',
        '$mul',
        '$rename',
        '$set',
        '$setOnInsert',
        '$unset',
        '$addToSet',
        '$pop',
        '$pull',
        '$push',
        '$pullAll',
        '$each',
        '$position',
        '$slice',
        '$sort',
        '$bit', // aggregation pipeline stages
        '$addFields',
        '$bucket',
        '$bucketAuto',
        '$collStats',
        '$count',
        '$currentOp',
        '$facet',
        '$geoNear',
        '$graphLookup',
        '$group',
        '$indexStats',
        '$limit',
        '$listLocalSessions',
        '$listSessions',
        '$lookup',
        '$match',
        '$merge',
        '$out',
        '$planCacheStats',
        '$project',
        '$redact',
        '$replaceRoot',
        '$replaceWith',
        '$sample',
        '$set',
        '$skip',
        '$sort',
        '$sortByCount',
        '$unionWith',
        '$unset',
        '$unwind',
        '$setWindowFields', // aggregation pipeline operators
        '$abs',
        '$accumulator',
        '$acos',
        '$acosh',
        '$add',
        '$addToSet',
        '$allElementsTrue',
        '$and',
        '$anyElementTrue',
        '$arrayElemAt',
        '$arrayToObject',
        '$asin',
        '$asinh',
        '$atan',
        '$atan2',
        '$atanh',
        '$avg',
        '$binarySize',
        '$bsonSize',
        '$ceil',
        '$cmp',
        '$concat',
        '$concatArrays',
        '$cond',
        '$convert',
        '$cos',
        '$dateFromParts',
        '$dateToParts',
        '$dateFromString',
        '$dateToString',
        '$dayOfMonth',
        '$dayOfWeek',
        '$dayOfYear',
        '$degreesToRadians',
        '$divide',
        '$eq',
        '$exp',
        '$filter',
        '$first',
        '$floor',
        '$function',
        '$gt',
        '$gte',
        '$hour',
        '$ifNull',
        '$in',
        '$indexOfArray',
        '$indexOfBytes',
        '$indexOfCP',
        '$isArray',
        '$isNumber',
        '$isoDayOfWeek',
        '$isoWeek',
        '$isoWeekYear',
        '$last',
        '$last',
        '$let',
        '$literal',
        '$ln',
        '$log',
        '$log10',
        '$lt',
        '$lte',
        '$ltrim',
        '$map',
        '$max',
        '$mergeObjects',
        '$meta',
        '$min',
        '$millisecond',
        '$minute',
        '$mod',
        '$month',
        '$multiply',
        '$ne',
        '$not',
        '$objectToArray',
        '$or',
        '$pow',
        '$push',
        '$radiansToDegrees',
        '$range',
        '$reduce',
        '$regexFind',
        '$regexFindAll',
        '$regexMatch',
        '$replaceOne',
        '$replaceAll',
        '$reverseArray',
        '$round',
        '$rtrim',
        '$second',
        '$setDifference',
        '$setEquals',
        '$setIntersection',
        '$setIsSubset',
        '$setUnion',
        '$size',
        '$sin',
        '$slice',
        '$split',
        '$sqrt',
        '$stdDevPop',
        '$stdDevSamp',
        '$strcasecmp',
        '$strLenBytes',
        '$strLenCP',
        '$substr',
        '$substrBytes',
        '$substrCP',
        '$subtract',
        '$sum',
        '$switch',
        '$tan',
        '$toBool',
        '$toDate',
        '$toDecimal',
        '$toDouble',
        '$toInt',
        '$toLong',
        '$toObjectId',
        '$toString',
        '$toLower',
        '$toUpper',
        '$trim',
        '$trunc',
        '$type',
        '$week',
        '$year',
        '$zip',
        '$count',
        '$dateAdd',
        '$dateDiff',
        '$dateSubtract',
        '$dateTrunc',
        '$getField',
        '$rand',
        '$sampleRate',
        '$setField',
        '$unsetField', // aggregation pipeline query modifiers
        '$comment',
        '$explain',
        '$hint',
        '$max',
        '$maxTimeMS',
        '$min',
        '$orderby',
        '$query',
        '$returnKey',
        '$showDiskLoc',
        '$natural'
      ];
      var builtinFunctions = [
        'ObjectId',
        'Code',
        'BinData',
        'DBRef',
        'Timestamp',
        'NumberLong',
        'NumberDecimal',
        'MaxKey',
        'MinKey',
        'RegExp',
        'ISODate',
        'UUID'
      ];
      operators = operators.map(function (operator) {
        return operator.replace('$', '\\$')
      });
      var operatorsSource = '(?:' + operators.join('|') + ')\\b';
      Prism.languages.mongodb = Prism.languages.extend('javascript', {});
      Prism.languages.insertBefore('mongodb', 'string', {
        property: {
          pattern:
            /(?:(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1|(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*)(?=\s*:)/,
          greedy: true,
          inside: {
            keyword: RegExp('^([\'"])?' + operatorsSource + '(?:\\1)?$')
          }
        }
      });
      Prism.languages.mongodb.string.inside = {
        url: {
          // url pattern
          pattern:
            /https?:\/\/[-\w@:%.+~#=]{1,256}\.[a-z0-9()]{1,6}\b[-\w()@:%+.~#?&/=]*/i,
          greedy: true
        },
        entity: {
          // ipv4
          pattern:
            /\b(?:(?:[01]?\d\d?|2[0-4]\d|25[0-5])\.){3}(?:[01]?\d\d?|2[0-4]\d|25[0-5])\b/,
          greedy: true
        }
      };
      Prism.languages.insertBefore('mongodb', 'constant', {
        builtin: {
          pattern: RegExp('\\b(?:' + builtinFunctions.join('|') + ')\\b'),
          alias: 'keyword'
        }
      });
    })(Prism);
  }

  // @ts-nocheck
  monkey.displayName = 'monkey';
  monkey.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function monkey(Prism) {
    Prism.languages.monkey = {
      comment: {
        pattern: /^#Rem\s[\s\S]*?^#End|'.+/im,
        greedy: true
      },
      string: {
        pattern: /"[^"\r\n]*"/,
        greedy: true
      },
      preprocessor: {
        pattern: /(^[ \t]*)#.+/m,
        lookbehind: true,
        greedy: true,
        alias: 'property'
      },
      function: /\b\w+(?=\()/,
      'type-char': {
        pattern: /\b[?%#$]/,
        alias: 'class-name'
      },
      number: {
        pattern:
          /((?:\.\.)?)(?:(?:\b|\B-\.?|\B\.)\d+(?:(?!\.\.)\.\d*)?|\$[\da-f]+)/i,
        lookbehind: true
      },
      keyword:
        /\b(?:Abstract|Array|Bool|Case|Catch|Class|Const|Continue|Default|Eachin|Else|ElseIf|End|EndIf|Exit|Extends|Extern|False|Field|Final|Float|For|Forever|Function|Global|If|Implements|Import|Inline|Int|Interface|Local|Method|Module|New|Next|Null|Object|Private|Property|Public|Repeat|Return|Select|Self|Step|Strict|String|Super|Then|Throw|To|True|Try|Until|Void|Wend|While)\b/i,
      operator:
        /\.\.|<[=>]?|>=?|:?=|(?:[+\-*\/&~|]|\b(?:Mod|Shl|Shr)\b)=?|\b(?:And|Not|Or)\b/i,
      punctuation: /[.,:;()\[\]]/
    };
  }

  // @ts-nocheck
  moonscript.displayName = 'moonscript';
  moonscript.aliases = ['moon'];

  /** @type {import('../core.js').Syntax} */
  function moonscript(Prism) {
    Prism.languages.moonscript = {
      comment: /--.*/,
      string: [
        {
          pattern: /'[^']*'|\[(=*)\[[\s\S]*?\]\1\]/,
          greedy: true
        },
        {
          pattern: /"[^"]*"/,
          greedy: true,
          inside: {
            interpolation: {
              pattern: /#\{[^{}]*\}/,
              inside: {
                moonscript: {
                  pattern: /(^#\{)[\s\S]+(?=\})/,
                  lookbehind: true,
                  inside: null // see beow
                },
                'interpolation-punctuation': {
                  pattern: /#\{|\}/,
                  alias: 'punctuation'
                }
              }
            }
          }
        }
      ],
      'class-name': [
        {
          pattern: /(\b(?:class|extends)[ \t]+)\w+/,
          lookbehind: true
        }, // class-like names start with a capital letter
        /\b[A-Z]\w*/
      ],
      keyword:
        /\b(?:class|continue|do|else|elseif|export|extends|for|from|if|import|in|local|nil|return|self|super|switch|then|unless|using|when|while|with)\b/,
      variable: /@@?\w*/,
      property: {
        pattern: /\b(?!\d)\w+(?=:)|(:)(?!\d)\w+/,
        lookbehind: true
      },
      function: {
        pattern:
          /\b(?:_G|_VERSION|assert|collectgarbage|coroutine\.(?:create|resume|running|status|wrap|yield)|debug\.(?:debug|getfenv|gethook|getinfo|getlocal|getmetatable|getregistry|getupvalue|setfenv|sethook|setlocal|setmetatable|setupvalue|traceback)|dofile|error|getfenv|getmetatable|io\.(?:close|flush|input|lines|open|output|popen|read|stderr|stdin|stdout|tmpfile|type|write)|ipairs|load|loadfile|loadstring|math\.(?:abs|acos|asin|atan|atan2|ceil|cos|cosh|deg|exp|floor|fmod|frexp|ldexp|log|log10|max|min|modf|pi|pow|rad|random|randomseed|sin|sinh|sqrt|tan|tanh)|module|next|os\.(?:clock|date|difftime|execute|exit|getenv|remove|rename|setlocale|time|tmpname)|package\.(?:cpath|loaded|loadlib|path|preload|seeall)|pairs|pcall|print|rawequal|rawget|rawset|require|select|setfenv|setmetatable|string\.(?:byte|char|dump|find|format|gmatch|gsub|len|lower|match|rep|reverse|sub|upper)|table\.(?:concat|insert|maxn|remove|sort)|tonumber|tostring|type|unpack|xpcall)\b/,
        inside: {
          punctuation: /\./
        }
      },
      boolean: /\b(?:false|true)\b/,
      number:
        /(?:\B\.\d+|\b\d+\.\d+|\b\d+(?=[eE]))(?:[eE][-+]?\d+)?\b|\b(?:0x[a-fA-F\d]+|\d+)(?:U?LL)?\b/,
      operator:
        /\.{3}|[-=]>|~=|(?:[-+*/%<>!=]|\.\.)=?|[:#^]|\b(?:and|or)\b=?|\b(?:not)\b/,
      punctuation: /[.,()[\]{}\\]/
    };
    Prism.languages.moonscript.string[1].inside.interpolation.inside.moonscript.inside =
      Prism.languages.moonscript;
    Prism.languages.moon = Prism.languages.moonscript;
  }

  // @ts-nocheck
  n1ql.displayName = 'n1ql';
  n1ql.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function n1ql(Prism) {
    // https://docs.couchbase.com/server/current/n1ql/n1ql-language-reference/index.html
    Prism.languages.n1ql = {
      comment: {
        pattern: /\/\*[\s\S]*?(?:$|\*\/)|--.*/,
        greedy: true
      },
      string: {
        pattern: /(["'])(?:\\[\s\S]|(?!\1)[^\\]|\1\1)*\1/,
        greedy: true
      },
      identifier: {
        pattern: /`(?:\\[\s\S]|[^\\`]|``)*`/,
        greedy: true
      },
      parameter: /\$[\w.]+/,
      // https://docs.couchbase.com/server/current/n1ql/n1ql-language-reference/reservedwords.html#n1ql-reserved-words
      keyword:
        /\b(?:ADVISE|ALL|ALTER|ANALYZE|AS|ASC|AT|BEGIN|BINARY|BOOLEAN|BREAK|BUCKET|BUILD|BY|CALL|CAST|CLUSTER|COLLATE|COLLECTION|COMMIT|COMMITTED|CONNECT|CONTINUE|CORRELATE|CORRELATED|COVER|CREATE|CURRENT|DATABASE|DATASET|DATASTORE|DECLARE|DECREMENT|DELETE|DERIVED|DESC|DESCRIBE|DISTINCT|DO|DROP|EACH|ELEMENT|EXCEPT|EXCLUDE|EXECUTE|EXPLAIN|FETCH|FILTER|FLATTEN|FLUSH|FOLLOWING|FOR|FORCE|FROM|FTS|FUNCTION|GOLANG|GRANT|GROUP|GROUPS|GSI|HASH|HAVING|IF|IGNORE|ILIKE|INCLUDE|INCREMENT|INDEX|INFER|INLINE|INNER|INSERT|INTERSECT|INTO|IS|ISOLATION|JAVASCRIPT|JOIN|KEY|KEYS|KEYSPACE|KNOWN|LANGUAGE|LAST|LEFT|LET|LETTING|LEVEL|LIMIT|LSM|MAP|MAPPING|MATCHED|MATERIALIZED|MERGE|MINUS|MISSING|NAMESPACE|NEST|NL|NO|NTH_VALUE|NULL|NULLS|NUMBER|OBJECT|OFFSET|ON|OPTION|OPTIONS|ORDER|OTHERS|OUTER|OVER|PARSE|PARTITION|PASSWORD|PATH|POOL|PRECEDING|PREPARE|PRIMARY|PRIVATE|PRIVILEGE|PROBE|PROCEDURE|PUBLIC|RANGE|RAW|REALM|REDUCE|RENAME|RESPECT|RETURN|RETURNING|REVOKE|RIGHT|ROLE|ROLLBACK|ROW|ROWS|SATISFIES|SAVEPOINT|SCHEMA|SCOPE|SELECT|SELF|SEMI|SET|SHOW|SOME|START|STATISTICS|STRING|SYSTEM|TIES|TO|TRAN|TRANSACTION|TRIGGER|TRUNCATE|UNBOUNDED|UNDER|UNION|UNIQUE|UNKNOWN|UNNEST|UNSET|UPDATE|UPSERT|USE|USER|USING|VALIDATE|VALUE|VALUES|VIA|VIEW|WHERE|WHILE|WINDOW|WITH|WORK|XOR)\b/i,
      function: /\b[a-z_]\w*(?=\s*\()/i,
      boolean: /\b(?:FALSE|TRUE)\b/i,
      number: /(?:\b\d+\.|\B\.)\d+e[+\-]?\d+\b|\b\d+(?:\.\d*)?|\B\.\d+\b/i,
      operator:
        /[-+*\/%]|!=|==?|\|\||<[>=]?|>=?|\b(?:AND|ANY|ARRAY|BETWEEN|CASE|ELSE|END|EVERY|EXISTS|FIRST|IN|LIKE|NOT|OR|THEN|VALUED|WHEN|WITHIN)\b/i,
      punctuation: /[;[\](),.{}:]/
    };
  }

  // @ts-nocheck
  nand2tetrisHdl.displayName = 'nand2tetris-hdl';
  nand2tetrisHdl.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function nand2tetrisHdl(Prism) {
    Prism.languages['nand2tetris-hdl'] = {
      comment: /\/\/.*|\/\*[\s\S]*?(?:\*\/|$)/,
      keyword: /\b(?:BUILTIN|CHIP|CLOCKED|IN|OUT|PARTS)\b/,
      boolean: /\b(?:false|true)\b/,
      function: /\b[A-Za-z][A-Za-z0-9]*(?=\()/,
      number: /\b\d+\b/,
      operator: /=|\.\./,
      punctuation: /[{}[\];(),:]/
    };
  }

  // @ts-nocheck
  naniscript.displayName = 'naniscript';
  naniscript.aliases = ['nani'];

  /** @type {import('../core.js').Syntax} */
  function naniscript(Prism) {
  (function (Prism) {
      var expressionDef = /\{[^\r\n\[\]{}]*\}/;
      var params = {
        'quoted-string': {
          pattern: /"(?:[^"\\]|\\.)*"/,
          alias: 'operator'
        },
        'command-param-id': {
          pattern: /(\s)\w+:/,
          lookbehind: true,
          alias: 'property'
        },
        'command-param-value': [
          {
            pattern: expressionDef,
            alias: 'selector'
          },
          {
            pattern: /([\t ])\S+/,
            lookbehind: true,
            greedy: true,
            alias: 'operator'
          },
          {
            pattern: /\S(?:.*\S)?/,
            alias: 'operator'
          }
        ]
      };
      Prism.languages.naniscript = {
        // ; ...
        comment: {
          pattern: /^([\t ]*);.*/m,
          lookbehind: true
        },
        // > ...
        // Define is a control line starting with '>' followed by a word, a space and a text.
        define: {
          pattern: /^>.+/m,
          alias: 'tag',
          inside: {
            value: {
              pattern: /(^>\w+[\t ]+)(?!\s)[^{}\r\n]+/,
              lookbehind: true,
              alias: 'operator'
            },
            key: {
              pattern: /(^>)\w+/,
              lookbehind: true
            }
          }
        },
        // # ...
        label: {
          pattern: /^([\t ]*)#[\t ]*\w+[\t ]*$/m,
          lookbehind: true,
          alias: 'regex'
        },
        command: {
          pattern: /^([\t ]*)@\w+(?=[\t ]|$).*/m,
          lookbehind: true,
          alias: 'function',
          inside: {
            'command-name': /^@\w+/,
            expression: {
              pattern: expressionDef,
              greedy: true,
              alias: 'selector'
            },
            'command-params': {
              pattern: /\s*\S[\s\S]*/,
              inside: params
            }
          }
        },
        // Generic is any line that doesn't start with operators: ;>#@
        'generic-text': {
          pattern: /(^[ \t]*)[^#@>;\s].*/m,
          lookbehind: true,
          alias: 'punctuation',
          inside: {
            // \{ ... \} ... \[ ... \] ... \"
            'escaped-char': /\\[{}\[\]"]/,
            expression: {
              pattern: expressionDef,
              greedy: true,
              alias: 'selector'
            },
            'inline-command': {
              pattern: /\[[\t ]*\w[^\r\n\[\]]*\]/,
              greedy: true,
              alias: 'function',
              inside: {
                'command-params': {
                  pattern: /(^\[[\t ]*\w+\b)[\s\S]+(?=\]$)/,
                  lookbehind: true,
                  inside: params
                },
                'command-param-name': {
                  pattern: /^(\[[\t ]*)\w+/,
                  lookbehind: true,
                  alias: 'name'
                },
                'start-stop-char': /[\[\]]/
              }
            }
          }
        }
      };
      Prism.languages.nani = Prism.languages['naniscript'];
      /** @typedef {InstanceType<import("./prism-core")["Token"]>} Token */
      /**
       * This hook is used to validate generic-text tokens for balanced brackets.
       * Mark token as bad-line when contains not balanced brackets: {},[]
       */
      Prism.hooks.add('after-tokenize', function (env) {
        /** @type {(Token | string)[]} */
        var tokens = env.tokens;
        tokens.forEach(function (token) {
          if (typeof token !== 'string' && token.type === 'generic-text') {
            var content = getTextContent(token);
            if (!isBracketsBalanced(content)) {
              token.type = 'bad-line';
              token.content = content;
            }
          }
        });
      });
      /**
       * @param {string} input
       * @returns {boolean}
       */
      function isBracketsBalanced(input) {
        var brackets = '[]{}';
        var stack = [];
        for (var i = 0; i < input.length; i++) {
          var bracket = input[i];
          var bracketsIndex = brackets.indexOf(bracket);
          if (bracketsIndex !== -1) {
            if (bracketsIndex % 2 === 0) {
              stack.push(bracketsIndex + 1);
            } else if (stack.pop() !== bracketsIndex) {
              return false
            }
          }
        }
        return stack.length === 0
      }
      /**
       * @param {string | Token | (string | Token)[]} token
       * @returns {string}
       */
      function getTextContent(token) {
        if (typeof token === 'string') {
          return token
        } else if (Array.isArray(token)) {
          return token.map(getTextContent).join('')
        } else {
          return getTextContent(token.content)
        }
      }
    })(Prism);
  }

  // @ts-nocheck
  nasm.displayName = 'nasm';
  nasm.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function nasm(Prism) {
    Prism.languages.nasm = {
      comment: /;.*$/m,
      string: /(["'`])(?:\\.|(?!\1)[^\\\r\n])*\1/,
      label: {
        pattern: /(^\s*)[A-Za-z._?$][\w.?$@~#]*:/m,
        lookbehind: true,
        alias: 'function'
      },
      keyword: [
        /\[?BITS (?:16|32|64)\]?/,
        {
          pattern: /(^\s*)section\s*[a-z.]+:?/im,
          lookbehind: true
        },
        /(?:extern|global)[^;\r\n]*/i,
        /(?:CPU|DEFAULT|FLOAT).*$/m
      ],
      register: {
        pattern:
          /\b(?:st\d|[xyz]mm\d\d?|[cdt]r\d|r\d\d?[bwd]?|[er]?[abcd]x|[abcd][hl]|[er]?(?:bp|di|si|sp)|[cdefgs]s)\b/i,
        alias: 'variable'
      },
      number:
        /(?:\b|(?=\$))(?:0[hx](?:\.[\da-f]+|[\da-f]+(?:\.[\da-f]+)?)(?:p[+-]?\d+)?|\d[\da-f]+[hx]|\$\d[\da-f]*|0[oq][0-7]+|[0-7]+[oq]|0[by][01]+|[01]+[by]|0[dt]\d+|(?:\d+(?:\.\d+)?|\.\d+)(?:\.?e[+-]?\d+)?[dt]?)\b/i,
      operator: /[\[\]*+\-\/%<>=&|$!]/
    };
  }

  // @ts-nocheck
  neon.displayName = 'neon';
  neon.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function neon(Prism) {
    Prism.languages.neon = {
      comment: {
        pattern: /#.*/,
        greedy: true
      },
      datetime: {
        pattern:
          /(^|[[{(=:,\s])\d\d\d\d-\d\d?-\d\d?(?:(?:[Tt]| +)\d\d?:\d\d:\d\d(?:\.\d*)? *(?:Z|[-+]\d\d?(?::?\d\d)?)?)?(?=$|[\]}),\s])/,
        lookbehind: true,
        alias: 'number'
      },
      key: {
        pattern: /(^|[[{(,\s])[^,:=[\]{}()'"\s]+(?=\s*:(?:$|[\]}),\s])|\s*=)/,
        lookbehind: true,
        alias: 'atrule'
      },
      number: {
        pattern:
          /(^|[[{(=:,\s])[+-]?(?:0x[\da-fA-F]+|0o[0-7]+|0b[01]+|(?:\d+(?:\.\d*)?|\.?\d+)(?:[eE][+-]?\d+)?)(?=$|[\]}),:=\s])/,
        lookbehind: true
      },
      boolean: {
        pattern: /(^|[[{(=:,\s])(?:false|no|true|yes)(?=$|[\]}),:=\s])/i,
        lookbehind: true
      },
      null: {
        pattern: /(^|[[{(=:,\s])(?:null)(?=$|[\]}),:=\s])/i,
        lookbehind: true,
        alias: 'keyword'
      },
      string: {
        pattern:
          /(^|[[{(=:,\s])(?:('''|""")\r?\n(?:(?:[^\r\n]|\r?\n(?![\t ]*\2))*\r?\n)?[\t ]*\2|'[^'\r\n]*'|"(?:\\.|[^\\"\r\n])*")/,
        lookbehind: true,
        greedy: true
      },
      literal: {
        pattern:
          /(^|[[{(=:,\s])(?:[^#"',:=[\]{}()\s`-]|[:-][^"',=[\]{}()\s])(?:[^,:=\]})(\s]|:(?![\s,\]})]|$)|[ \t]+[^#,:=\]})(\s])*/,
        lookbehind: true,
        alias: 'string'
      },
      punctuation: /[,:=[\]{}()-]/
    };
  }

  // @ts-nocheck
  nevod.displayName = 'nevod';
  nevod.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function nevod(Prism) {
    Prism.languages.nevod = {
      comment: /\/\/.*|(?:\/\*[\s\S]*?(?:\*\/|$))/,
      string: {
        pattern: /(?:"(?:""|[^"])*"(?!")|'(?:''|[^'])*'(?!'))!?\*?/,
        greedy: true,
        inside: {
          'string-attrs': /!$|!\*$|\*$/
        }
      },
      namespace: {
        pattern: /(@namespace\s+)[a-zA-Z0-9\-.]+(?=\s*\{)/,
        lookbehind: true
      },
      pattern: {
        pattern:
          /(@pattern\s+)?#?[a-zA-Z0-9\-.]+(?:\s*\(\s*(?:~\s*)?[a-zA-Z0-9\-.]+\s*(?:,\s*(?:~\s*)?[a-zA-Z0-9\-.]*)*\))?(?=\s*=)/,
        lookbehind: true,
        inside: {
          'pattern-name': {
            pattern: /^#?[a-zA-Z0-9\-.]+/,
            alias: 'class-name'
          },
          fields: {
            pattern: /\(.*\)/,
            inside: {
              'field-name': {
                pattern: /[a-zA-Z0-9\-.]+/,
                alias: 'variable'
              },
              punctuation: /[,()]/,
              operator: {
                pattern: /~/,
                alias: 'field-hidden-mark'
              }
            }
          }
        }
      },
      search: {
        pattern: /(@search\s+|#)[a-zA-Z0-9\-.]+(?:\.\*)?(?=\s*;)/,
        alias: 'function',
        lookbehind: true
      },
      keyword:
        /@(?:having|inside|namespace|outside|pattern|require|search|where)\b/,
      'standard-pattern': {
        pattern:
          /\b(?:Alpha|AlphaNum|Any|Blank|End|LineBreak|Num|NumAlpha|Punct|Space|Start|Symbol|Word|WordBreak)\b(?:\([a-zA-Z0-9\-.,\s+]*\))?/,
        inside: {
          'standard-pattern-name': {
            pattern: /^[a-zA-Z0-9\-.]+/,
            alias: 'builtin'
          },
          quantifier: {
            pattern: /\b\d+(?:\s*\+|\s*-\s*\d+)?(?!\w)/,
            alias: 'number'
          },
          'standard-pattern-attr': {
            pattern: /[a-zA-Z0-9\-.]+/,
            alias: 'builtin'
          },
          punctuation: /[,()]/
        }
      },
      quantifier: {
        pattern: /\b\d+(?:\s*\+|\s*-\s*\d+)?(?!\w)/,
        alias: 'number'
      },
      operator: [
        {
          pattern: /=/,
          alias: 'pattern-def'
        },
        {
          pattern: /&/,
          alias: 'conjunction'
        },
        {
          pattern: /~/,
          alias: 'exception'
        },
        {
          pattern: /\?/,
          alias: 'optionality'
        },
        {
          pattern: /[[\]]/,
          alias: 'repetition'
        },
        {
          pattern: /[{}]/,
          alias: 'variation'
        },
        {
          pattern: /[+_]/,
          alias: 'sequence'
        },
        {
          pattern: /\.{2,3}/,
          alias: 'span'
        }
      ],
      'field-capture': [
        {
          pattern:
            /([a-zA-Z0-9\-.]+\s*\()\s*[a-zA-Z0-9\-.]+\s*:\s*[a-zA-Z0-9\-.]+(?:\s*,\s*[a-zA-Z0-9\-.]+\s*:\s*[a-zA-Z0-9\-.]+)*(?=\s*\))/,
          lookbehind: true,
          inside: {
            'field-name': {
              pattern: /[a-zA-Z0-9\-.]+/,
              alias: 'variable'
            },
            colon: /:/
          }
        },
        {
          pattern: /[a-zA-Z0-9\-.]+\s*:/,
          inside: {
            'field-name': {
              pattern: /[a-zA-Z0-9\-.]+/,
              alias: 'variable'
            },
            colon: /:/
          }
        }
      ],
      punctuation: /[:;,()]/,
      name: /[a-zA-Z0-9\-.]+/
    };
  }

  // @ts-nocheck
  nginx.displayName = 'nginx';
  nginx.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function nginx(Prism) {
  (function (Prism) {
      var variable =
        /\$(?:\w[a-z\d]*(?:_[^\x00-\x1F\s"'\\()$]*)?|\{[^}\s"'\\]+\})/i;
      Prism.languages.nginx = {
        comment: {
          pattern: /(^|[\s{};])#.*/,
          lookbehind: true,
          greedy: true
        },
        directive: {
          pattern:
            /(^|\s)\w(?:[^;{}"'\\\s]|\\.|"(?:[^"\\]|\\.)*"|'(?:[^'\\]|\\.)*'|\s+(?:#.*(?!.)|(?![#\s])))*?(?=\s*[;{])/,
          lookbehind: true,
          greedy: true,
          inside: {
            string: {
              pattern:
                /((?:^|[^\\])(?:\\\\)*)(?:"(?:[^"\\]|\\.)*"|'(?:[^'\\]|\\.)*')/,
              lookbehind: true,
              greedy: true,
              inside: {
                escape: {
                  pattern: /\\["'\\nrt]/,
                  alias: 'entity'
                },
                variable: variable
              }
            },
            comment: {
              pattern: /(\s)#.*/,
              lookbehind: true,
              greedy: true
            },
            keyword: {
              pattern: /^\S+/,
              greedy: true
            },
            // other patterns
            boolean: {
              pattern: /(\s)(?:off|on)(?!\S)/,
              lookbehind: true
            },
            number: {
              pattern: /(\s)\d+[a-z]*(?!\S)/i,
              lookbehind: true
            },
            variable: variable
          }
        },
        punctuation: /[{};]/
      };
    })(Prism);
  }

  // @ts-nocheck
  nim.displayName = 'nim';
  nim.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function nim(Prism) {
    Prism.languages.nim = {
      comment: {
        pattern: /#.*/,
        greedy: true
      },
      string: {
        // Double-quoted strings can be prefixed by an identifier (Generalized raw string literals)
        pattern:
          /(?:\b(?!\d)(?:\w|\\x[89a-fA-F][0-9a-fA-F])+)?(?:"""[\s\S]*?"""(?!")|"(?:\\[\s\S]|""|[^"\\])*")/,
        greedy: true
      },
      char: {
        // Character literals are handled specifically to prevent issues with numeric type suffixes
        pattern: /'(?:\\(?:\d+|x[\da-fA-F]{0,2}|.)|[^'])'/,
        greedy: true
      },
      function: {
        pattern:
          /(?:(?!\d)(?:\w|\\x[89a-fA-F][0-9a-fA-F])+|`[^`\r\n]+`)\*?(?:\[[^\]]+\])?(?=\s*\()/,
        greedy: true,
        inside: {
          operator: /\*$/
        }
      },
      // We don't want to highlight operators (and anything really) inside backticks
      identifier: {
        pattern: /`[^`\r\n]+`/,
        greedy: true,
        inside: {
          punctuation: /`/
        }
      },
      // The negative look ahead prevents wrong highlighting of the .. operator
      number:
        /\b(?:0[xXoObB][\da-fA-F_]+|\d[\d_]*(?:(?!\.\.)\.[\d_]*)?(?:[eE][+-]?\d[\d_]*)?)(?:'?[iuf]\d*)?/,
      keyword:
        /\b(?:addr|as|asm|atomic|bind|block|break|case|cast|concept|const|continue|converter|defer|discard|distinct|do|elif|else|end|enum|except|export|finally|for|from|func|generic|if|import|include|interface|iterator|let|macro|method|mixin|nil|object|out|proc|ptr|raise|ref|return|static|template|try|tuple|type|using|var|when|while|with|without|yield)\b/,
      operator: {
        // Look behind and look ahead prevent wrong highlighting of punctuations [. .] {. .} (. .)
        // but allow the slice operator .. to take precedence over them
        // One can define his own operators in Nim so all combination of operators might be an operator.
        pattern:
          /(^|[({\[](?=\.\.)|(?![({\[]\.).)(?:(?:[=+\-*\/<>@$~&%|!?^:\\]|\.\.|\.(?![)}\]]))+|\b(?:and|div|in|is|isnot|mod|not|notin|of|or|shl|shr|xor)\b)/m,
        lookbehind: true
      },
      punctuation: /[({\[]\.|\.[)}\]]|[`(){}\[\],:]/
    };
  }

  // @ts-nocheck
  nix.displayName = 'nix';
  nix.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function nix(Prism) {
    Prism.languages.nix = {
      comment: {
        pattern: /\/\*[\s\S]*?\*\/|#.*/,
        greedy: true
      },
      string: {
        pattern: /"(?:[^"\\]|\\[\s\S])*"|''(?:(?!'')[\s\S]|''(?:'|\\|\$\{))*''/,
        greedy: true,
        inside: {
          interpolation: {
            // The lookbehind ensures the ${} is not preceded by \ or ''
            pattern: /(^|(?:^|(?!'').)[^\\])\$\{(?:[^{}]|\{[^}]*\})*\}/,
            lookbehind: true,
            inside: null // see below
          }
        }
      },
      url: [
        /\b(?:[a-z]{3,7}:\/\/)[\w\-+%~\/.:#=?&]+/,
        {
          pattern:
            /([^\/])(?:[\w\-+%~.:#=?&]*(?!\/\/)[\w\-+%~\/.:#=?&])?(?!\/\/)\/[\w\-+%~\/.:#=?&]*/,
          lookbehind: true
        }
      ],
      antiquotation: {
        pattern: /\$(?=\{)/,
        alias: 'important'
      },
      number: /\b\d+\b/,
      keyword: /\b(?:assert|builtins|else|if|in|inherit|let|null|or|then|with)\b/,
      function:
        /\b(?:abort|add|all|any|attrNames|attrValues|baseNameOf|compareVersions|concatLists|currentSystem|deepSeq|derivation|dirOf|div|elem(?:At)?|fetch(?:Tarball|url)|filter(?:Source)?|fromJSON|genList|getAttr|getEnv|hasAttr|hashString|head|import|intersectAttrs|is(?:Attrs|Bool|Function|Int|List|Null|String)|length|lessThan|listToAttrs|map|mul|parseDrvName|pathExists|read(?:Dir|File)|removeAttrs|replaceStrings|seq|sort|stringLength|sub(?:string)?|tail|throw|to(?:File|JSON|Path|String|XML)|trace|typeOf)\b|\bfoldl'\B/,
      boolean: /\b(?:false|true)\b/,
      operator: /[=!<>]=?|\+\+?|\|\||&&|\/\/|->?|[?@]/,
      punctuation: /[{}()[\].,:;]/
    };
    Prism.languages.nix.string.inside.interpolation.inside = Prism.languages.nix;
  }

  // @ts-nocheck
  nsis.displayName = 'nsis';
  nsis.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function nsis(Prism) {
    /**
     * Original by Jan T. Sott (http://github.com/idleberg)
     *
     * Includes all commands and plug-ins shipped with NSIS 3.08
     */
    Prism.languages.nsis = {
      comment: {
        pattern: /(^|[^\\])(?:\/\*[\s\S]*?\*\/|[#;].*)/,
        lookbehind: true,
        greedy: true
      },
      string: {
        pattern: /("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,
        greedy: true
      },
      keyword: {
        pattern:
          /(^[\t ]*)(?:Abort|Add(?:BrandingImage|Size)|AdvSplash|Allow(?:RootDirInstall|SkipFiles)|AutoCloseWindow|BG(?:Font|Gradient|Image)|Banner|BrandingText|BringToFront|CRCCheck|Call(?:InstDLL)?|Caption|ChangeUI|CheckBitmap|ClearErrors|CompletedText|ComponentText|CopyFiles|Create(?:Directory|Font|ShortCut)|Delete(?:INISec|INIStr|RegKey|RegValue)?|Detail(?:Print|sButtonText)|Dialer|Dir(?:Text|Var|Verify)|EnableWindow|Enum(?:RegKey|RegValue)|Exch|Exec(?:Shell(?:Wait)?|Wait)?|ExpandEnvStrings|File(?:BufSize|Close|ErrorText|Open|Read|ReadByte|ReadUTF16LE|ReadWord|Seek|Write|WriteByte|WriteUTF16LE|WriteWord)?|Find(?:Close|First|Next|Window)|FlushINI|Get(?:CurInstType|CurrentAddress|DLLVersion(?:Local)?|DlgItem|ErrorLevel|FileTime(?:Local)?|FullPathName|Function(?:Address|End)?|InstDirError|LabelAddress|TempFileName)|Goto|HideWindow|Icon|If(?:Abort|Errors|FileExists|RebootFlag|Silent)|InitPluginsDir|InstProgressFlags|Inst(?:Type(?:GetText|SetText)?)|Install(?:ButtonText|Colors|Dir(?:RegKey)?)|Int(?:64|Ptr)?CmpU?|Int(?:64)?Fmt|Int(?:Ptr)?Op|IsWindow|Lang(?:DLL|String)|License(?:BkColor|Data|ForceSelection|LangString|Text)|LoadLanguageFile|LockWindow|Log(?:Set|Text)|Manifest(?:DPIAware|SupportedOS)|Math|MessageBox|MiscButtonText|NSISdl|Name|Nop|OutFile|PE(?:DllCharacteristics|SubsysVer)|Page(?:Callbacks)?|Pop|Push|Quit|RMDir|Read(?:EnvStr|INIStr|RegDWORD|RegStr)|Reboot|RegDLL|Rename|RequestExecutionLevel|ReserveFile|Return|SearchPath|Section(?:End|GetFlags|GetInstTypes|GetSize|GetText|Group|In|SetFlags|SetInstTypes|SetSize|SetText)?|SendMessage|Set(?:AutoClose|BrandingImage|Compress|Compressor(?:DictSize)?|CtlColors|CurInstType|DatablockOptimize|DateSave|Details(?:Print|View)|ErrorLevel|Errors|FileAttributes|Font|OutPath|Overwrite|PluginUnload|RebootFlag|RegView|ShellVarContext|Silent)|Show(?:InstDetails|UninstDetails|Window)|Silent(?:Install|UnInstall)|Sleep|SpaceTexts|Splash|StartMenu|Str(?:CmpS?|Cpy|Len)|SubCaption|System|UnRegDLL|Unicode|UninstPage|Uninstall(?:ButtonText|Caption|Icon|SubCaption|Text)|UserInfo|VI(?:AddVersionKey|FileVersion|ProductVersion)|VPatch|Var|WindowIcon|Write(?:INIStr|Reg(?:Bin|DWORD|ExpandStr|MultiStr|None|Str)|Uninstaller)|XPStyle|ns(?:Dialogs|Exec))\b/m,
        lookbehind: true
      },
      property:
        /\b(?:ARCHIVE|FILE_(?:ATTRIBUTE_ARCHIVE|ATTRIBUTE_NORMAL|ATTRIBUTE_OFFLINE|ATTRIBUTE_READONLY|ATTRIBUTE_SYSTEM|ATTRIBUTE_TEMPORARY)|HK(?:(?:CR|CU|LM)(?:32|64)?|DD|PD|U)|HKEY_(?:CLASSES_ROOT|CURRENT_CONFIG|CURRENT_USER|DYN_DATA|LOCAL_MACHINE|PERFORMANCE_DATA|USERS)|ID(?:ABORT|CANCEL|IGNORE|NO|OK|RETRY|YES)|MB_(?:ABORTRETRYIGNORE|DEFBUTTON1|DEFBUTTON2|DEFBUTTON3|DEFBUTTON4|ICONEXCLAMATION|ICONINFORMATION|ICONQUESTION|ICONSTOP|OK|OKCANCEL|RETRYCANCEL|RIGHT|RTLREADING|SETFOREGROUND|TOPMOST|USERICON|YESNO)|NORMAL|OFFLINE|READONLY|SHCTX|SHELL_CONTEXT|SYSTEM|TEMPORARY|admin|all|auto|both|colored|false|force|hide|highest|lastused|leave|listonly|none|normal|notset|off|on|open|print|show|silent|silentlog|smooth|textonly|true|user)\b/,
      constant: /\$\{[!\w\.:\^-]+\}|\$\([!\w\.:\^-]+\)/,
      variable: /\$\w[\w\.]*/,
      number: /\b0x[\dA-Fa-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee]-?\d+)?/,
      operator: /--?|\+\+?|<=?|>=?|==?=?|&&?|\|\|?|[?*\/~^%]/,
      punctuation: /[{}[\];(),.:]/,
      important: {
        pattern:
          /(^[\t ]*)!(?:addincludedir|addplugindir|appendfile|cd|define|delfile|echo|else|endif|error|execute|finalize|getdllversion|gettlbversion|if|ifdef|ifmacrodef|ifmacrondef|ifndef|include|insertmacro|macro|macroend|makensis|packhdr|pragma|searchparse|searchreplace|system|tempfile|undef|verbose|warning)\b/im,
        lookbehind: true
      }
    };
  }

  // @ts-nocheck
  ocaml.displayName = 'ocaml';
  ocaml.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function ocaml(Prism) {
    // https://ocaml.org/manual/lex.html
    Prism.languages.ocaml = {
      comment: {
        pattern: /\(\*[\s\S]*?\*\)/,
        greedy: true
      },
      char: {
        pattern: /'(?:[^\\\r\n']|\\(?:.|[ox]?[0-9a-f]{1,3}))'/i,
        greedy: true
      },
      string: [
        {
          pattern: /"(?:\\(?:[\s\S]|\r\n)|[^\\\r\n"])*"/,
          greedy: true
        },
        {
          pattern: /\{([a-z_]*)\|[\s\S]*?\|\1\}/,
          greedy: true
        }
      ],
      number: [
        // binary and octal
        /\b(?:0b[01][01_]*|0o[0-7][0-7_]*)\b/i, // hexadecimal
        /\b0x[a-f0-9][a-f0-9_]*(?:\.[a-f0-9_]*)?(?:p[+-]?\d[\d_]*)?(?!\w)/i, // decimal
        /\b\d[\d_]*(?:\.[\d_]*)?(?:e[+-]?\d[\d_]*)?(?!\w)/i
      ],
      directive: {
        pattern: /\B#\w+/,
        alias: 'property'
      },
      label: {
        pattern: /\B~\w+/,
        alias: 'property'
      },
      'type-variable': {
        pattern: /\B'\w+/,
        alias: 'function'
      },
      variant: {
        pattern: /`\w+/,
        alias: 'symbol'
      },
      // For the list of keywords and operators,
      // see: http://caml.inria.fr/pub/docs/manual-ocaml/lex.html#sec84
      keyword:
        /\b(?:as|assert|begin|class|constraint|do|done|downto|else|end|exception|external|for|fun|function|functor|if|in|include|inherit|initializer|lazy|let|match|method|module|mutable|new|nonrec|object|of|open|private|rec|sig|struct|then|to|try|type|val|value|virtual|when|where|while|with)\b/,
      boolean: /\b(?:false|true)\b/,
      'operator-like-punctuation': {
        pattern: /\[[<>|]|[>|]\]|\{<|>\}/,
        alias: 'punctuation'
      },
      // Custom operators are allowed
      operator:
        /\.[.~]|:[=>]|[=<>@^|&+\-*\/$%!?~][!$%&*+\-.\/:<=>?@^|~]*|\b(?:and|asr|land|lor|lsl|lsr|lxor|mod|or)\b/,
      punctuation: /;;|::|[(){}\[\].,:;#]|\b_\b/
    };
  }

  // @ts-nocheck
  opencl.displayName = 'opencl';
  opencl.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function opencl(Prism) {
    Prism.register(c$1)
    ;(function (Prism) {
      /* OpenCL kernel language */
      Prism.languages.opencl = Prism.languages.extend('c', {
        // Extracted from the official specs (2.0) and http://streamcomputing.eu/downloads/?opencl.lang (opencl-keywords, opencl-types) and http://sourceforge.net/tracker/?func=detail&aid=2957794&group_id=95717&atid=612384 (Words2, partly Words3)
        keyword:
          /\b(?:(?:__)?(?:constant|global|kernel|local|private|read_only|read_write|write_only)|__attribute__|auto|(?:bool|u?(?:char|int|long|short)|half|quad)(?:2|3|4|8|16)?|break|case|complex|const|continue|(?:double|float)(?:16(?:x(?:1|2|4|8|16))?|1x(?:1|2|4|8|16)|2(?:x(?:1|2|4|8|16))?|3|4(?:x(?:1|2|4|8|16))?|8(?:x(?:1|2|4|8|16))?)?|default|do|else|enum|extern|for|goto|if|imaginary|inline|packed|pipe|register|restrict|return|signed|sizeof|static|struct|switch|typedef|uniform|union|unsigned|void|volatile|while)\b/,
        // Extracted from http://streamcomputing.eu/downloads/?opencl.lang (opencl-const)
        // Math Constants: https://www.khronos.org/registry/OpenCL/sdk/2.1/docs/man/xhtml/mathConstants.html
        // Macros and Limits: https://www.khronos.org/registry/OpenCL/sdk/2.1/docs/man/xhtml/macroLimits.html
        number:
          /(?:\b0x(?:[\da-f]+(?:\.[\da-f]*)?|\.[\da-f]+)(?:p[+-]?\d+)?|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?)[fuhl]{0,4}/i,
        boolean: /\b(?:false|true)\b/,
        'constant-opencl-kernel': {
          pattern:
            /\b(?:CHAR_(?:BIT|MAX|MIN)|CLK_(?:ADDRESS_(?:CLAMP(?:_TO_EDGE)?|NONE|REPEAT)|FILTER_(?:LINEAR|NEAREST)|(?:GLOBAL|LOCAL)_MEM_FENCE|NORMALIZED_COORDS_(?:FALSE|TRUE))|CL_(?:BGRA|(?:HALF_)?FLOAT|INTENSITY|LUMINANCE|A?R?G?B?[Ax]?|(?:(?:UN)?SIGNED|[US]NORM)_(?:INT(?:8|16|32))|UNORM_(?:INT_101010|SHORT_(?:555|565)))|(?:DBL|FLT|HALF)_(?:DIG|EPSILON|(?:MAX|MIN)(?:(?:_10)?_EXP)?|MANT_DIG)|FLT_RADIX|HUGE_VALF?|(?:INT|LONG|SCHAR|SHRT)_(?:MAX|MIN)|INFINITY|MAXFLOAT|M_(?:[12]_PI|2_SQRTPI|E|LN(?:2|10)|LOG(?:2|10)E?|PI(?:_[24])?|SQRT(?:1_2|2))(?:_F|_H)?|NAN|(?:UCHAR|UINT|ULONG|USHRT)_MAX)\b/,
          alias: 'constant'
        }
      });
      Prism.languages.insertBefore('opencl', 'class-name', {
        // https://www.khronos.org/registry/OpenCL/sdk/2.1/docs/man/xhtml/scalarDataTypes.html
        // https://www.khronos.org/registry/OpenCL/sdk/2.1/docs/man/xhtml/otherDataTypes.html
        'builtin-type': {
          pattern:
            /\b(?:_cl_(?:command_queue|context|device_id|event|kernel|mem|platform_id|program|sampler)|cl_(?:image_format|mem_fence_flags)|clk_event_t|event_t|image(?:1d_(?:array_|buffer_)?t|2d_(?:array_(?:depth_|msaa_depth_|msaa_)?|depth_|msaa_depth_|msaa_)?t|3d_t)|intptr_t|ndrange_t|ptrdiff_t|queue_t|reserve_id_t|sampler_t|size_t|uintptr_t)\b/,
          alias: 'keyword'
        }
      });
      var attributes = {
        // Extracted from http://streamcomputing.eu/downloads/?opencl_host.lang (opencl-types and opencl-host)
        'type-opencl-host': {
          pattern:
            /\b(?:cl_(?:GLenum|GLint|GLuin|addressing_mode|bitfield|bool|buffer_create_type|build_status|channel_(?:order|type)|(?:u?(?:char|int|long|short)|double|float)(?:2|3|4|8|16)?|command_(?:queue(?:_info|_properties)?|type)|context(?:_info|_properties)?|device_(?:exec_capabilities|fp_config|id|info|local_mem_type|mem_cache_type|type)|(?:event|sampler)(?:_info)?|filter_mode|half|image_info|kernel(?:_info|_work_group_info)?|map_flags|mem(?:_flags|_info|_object_type)?|platform_(?:id|info)|profiling_info|program(?:_build_info|_info)?))\b/,
          alias: 'keyword'
        },
        'boolean-opencl-host': {
          pattern: /\bCL_(?:FALSE|TRUE)\b/,
          alias: 'boolean'
        },
        // Extracted from cl.h (2.0) and http://streamcomputing.eu/downloads/?opencl_host.lang (opencl-const)
        'constant-opencl-host': {
          pattern:
            /\bCL_(?:A|ABGR|ADDRESS_(?:CLAMP(?:_TO_EDGE)?|MIRRORED_REPEAT|NONE|REPEAT)|ARGB|BGRA|BLOCKING|BUFFER_CREATE_TYPE_REGION|BUILD_(?:ERROR|IN_PROGRESS|NONE|PROGRAM_FAILURE|SUCCESS)|COMMAND_(?:ACQUIRE_GL_OBJECTS|BARRIER|COPY_(?:BUFFER(?:_RECT|_TO_IMAGE)?|IMAGE(?:_TO_BUFFER)?)|FILL_(?:BUFFER|IMAGE)|MAP(?:_BUFFER|_IMAGE)|MARKER|MIGRATE(?:_SVM)?_MEM_OBJECTS|NATIVE_KERNEL|NDRANGE_KERNEL|READ_(?:BUFFER(?:_RECT)?|IMAGE)|RELEASE_GL_OBJECTS|SVM_(?:FREE|MAP|MEMCPY|MEMFILL|UNMAP)|TASK|UNMAP_MEM_OBJECT|USER|WRITE_(?:BUFFER(?:_RECT)?|IMAGE))|COMPILER_NOT_AVAILABLE|COMPILE_PROGRAM_FAILURE|COMPLETE|CONTEXT_(?:DEVICES|INTEROP_USER_SYNC|NUM_DEVICES|PLATFORM|PROPERTIES|REFERENCE_COUNT)|DEPTH(?:_STENCIL)?|DEVICE_(?:ADDRESS_BITS|AFFINITY_DOMAIN_(?:L[1-4]_CACHE|NEXT_PARTITIONABLE|NUMA)|AVAILABLE|BUILT_IN_KERNELS|COMPILER_AVAILABLE|DOUBLE_FP_CONFIG|ENDIAN_LITTLE|ERROR_CORRECTION_SUPPORT|EXECUTION_CAPABILITIES|EXTENSIONS|GLOBAL_(?:MEM_(?:CACHELINE_SIZE|CACHE_SIZE|CACHE_TYPE|SIZE)|VARIABLE_PREFERRED_TOTAL_SIZE)|HOST_UNIFIED_MEMORY|IL_VERSION|IMAGE(?:2D_MAX_(?:HEIGHT|WIDTH)|3D_MAX_(?:DEPTH|HEIGHT|WIDTH)|_BASE_ADDRESS_ALIGNMENT|_MAX_ARRAY_SIZE|_MAX_BUFFER_SIZE|_PITCH_ALIGNMENT|_SUPPORT)|LINKER_AVAILABLE|LOCAL_MEM_SIZE|LOCAL_MEM_TYPE|MAX_(?:CLOCK_FREQUENCY|COMPUTE_UNITS|CONSTANT_ARGS|CONSTANT_BUFFER_SIZE|GLOBAL_VARIABLE_SIZE|MEM_ALLOC_SIZE|NUM_SUB_GROUPS|ON_DEVICE_(?:EVENTS|QUEUES)|PARAMETER_SIZE|PIPE_ARGS|READ_IMAGE_ARGS|READ_WRITE_IMAGE_ARGS|SAMPLERS|WORK_GROUP_SIZE|WORK_ITEM_DIMENSIONS|WORK_ITEM_SIZES|WRITE_IMAGE_ARGS)|MEM_BASE_ADDR_ALIGN|MIN_DATA_TYPE_ALIGN_SIZE|NAME|NATIVE_VECTOR_WIDTH_(?:CHAR|DOUBLE|FLOAT|HALF|INT|LONG|SHORT)|NOT_(?:AVAILABLE|FOUND)|OPENCL_C_VERSION|PARENT_DEVICE|PARTITION_(?:AFFINITY_DOMAIN|BY_AFFINITY_DOMAIN|BY_COUNTS|BY_COUNTS_LIST_END|EQUALLY|FAILED|MAX_SUB_DEVICES|PROPERTIES|TYPE)|PIPE_MAX_(?:ACTIVE_RESERVATIONS|PACKET_SIZE)|PLATFORM|PREFERRED_(?:GLOBAL_ATOMIC_ALIGNMENT|INTEROP_USER_SYNC|LOCAL_ATOMIC_ALIGNMENT|PLATFORM_ATOMIC_ALIGNMENT|VECTOR_WIDTH_(?:CHAR|DOUBLE|FLOAT|HALF|INT|LONG|SHORT))|PRINTF_BUFFER_SIZE|PROFILE|PROFILING_TIMER_RESOLUTION|QUEUE_(?:ON_(?:DEVICE_(?:MAX_SIZE|PREFERRED_SIZE|PROPERTIES)|HOST_PROPERTIES)|PROPERTIES)|REFERENCE_COUNT|SINGLE_FP_CONFIG|SUB_GROUP_INDEPENDENT_FORWARD_PROGRESS|SVM_(?:ATOMICS|CAPABILITIES|COARSE_GRAIN_BUFFER|FINE_GRAIN_BUFFER|FINE_GRAIN_SYSTEM)|TYPE(?:_ACCELERATOR|_ALL|_CPU|_CUSTOM|_DEFAULT|_GPU)?|VENDOR(?:_ID)?|VERSION)|DRIVER_VERSION|EVENT_(?:COMMAND_(?:EXECUTION_STATUS|QUEUE|TYPE)|CONTEXT|REFERENCE_COUNT)|EXEC_(?:KERNEL|NATIVE_KERNEL|STATUS_ERROR_FOR_EVENTS_IN_WAIT_LIST)|FILTER_(?:LINEAR|NEAREST)|FLOAT|FP_(?:CORRECTLY_ROUNDED_DIVIDE_SQRT|DENORM|FMA|INF_NAN|ROUND_TO_INF|ROUND_TO_NEAREST|ROUND_TO_ZERO|SOFT_FLOAT)|GLOBAL|HALF_FLOAT|IMAGE_(?:ARRAY_SIZE|BUFFER|DEPTH|ELEMENT_SIZE|FORMAT|FORMAT_MISMATCH|FORMAT_NOT_SUPPORTED|HEIGHT|NUM_MIP_LEVELS|NUM_SAMPLES|ROW_PITCH|SLICE_PITCH|WIDTH)|INTENSITY|INVALID_(?:ARG_INDEX|ARG_SIZE|ARG_VALUE|BINARY|BUFFER_SIZE|BUILD_OPTIONS|COMMAND_QUEUE|COMPILER_OPTIONS|CONTEXT|DEVICE|DEVICE_PARTITION_COUNT|DEVICE_QUEUE|DEVICE_TYPE|EVENT|EVENT_WAIT_LIST|GLOBAL_OFFSET|GLOBAL_WORK_SIZE|GL_OBJECT|HOST_PTR|IMAGE_DESCRIPTOR|IMAGE_FORMAT_DESCRIPTOR|IMAGE_SIZE|KERNEL|KERNEL_ARGS|KERNEL_DEFINITION|KERNEL_NAME|LINKER_OPTIONS|MEM_OBJECT|MIP_LEVEL|OPERATION|PIPE_SIZE|PLATFORM|PROGRAM|PROGRAM_EXECUTABLE|PROPERTY|QUEUE_PROPERTIES|SAMPLER|VALUE|WORK_DIMENSION|WORK_GROUP_SIZE|WORK_ITEM_SIZE)|KERNEL_(?:ARG_(?:ACCESS_(?:NONE|QUALIFIER|READ_ONLY|READ_WRITE|WRITE_ONLY)|ADDRESS_(?:CONSTANT|GLOBAL|LOCAL|PRIVATE|QUALIFIER)|INFO_NOT_AVAILABLE|NAME|TYPE_(?:CONST|NAME|NONE|PIPE|QUALIFIER|RESTRICT|VOLATILE))|ATTRIBUTES|COMPILE_NUM_SUB_GROUPS|COMPILE_WORK_GROUP_SIZE|CONTEXT|EXEC_INFO_SVM_FINE_GRAIN_SYSTEM|EXEC_INFO_SVM_PTRS|FUNCTION_NAME|GLOBAL_WORK_SIZE|LOCAL_MEM_SIZE|LOCAL_SIZE_FOR_SUB_GROUP_COUNT|MAX_NUM_SUB_GROUPS|MAX_SUB_GROUP_SIZE_FOR_NDRANGE|NUM_ARGS|PREFERRED_WORK_GROUP_SIZE_MULTIPLE|PRIVATE_MEM_SIZE|PROGRAM|REFERENCE_COUNT|SUB_GROUP_COUNT_FOR_NDRANGE|WORK_GROUP_SIZE)|LINKER_NOT_AVAILABLE|LINK_PROGRAM_FAILURE|LOCAL|LUMINANCE|MAP_(?:FAILURE|READ|WRITE|WRITE_INVALIDATE_REGION)|MEM_(?:ALLOC_HOST_PTR|ASSOCIATED_MEMOBJECT|CONTEXT|COPY_HOST_PTR|COPY_OVERLAP|FLAGS|HOST_NO_ACCESS|HOST_PTR|HOST_READ_ONLY|HOST_WRITE_ONLY|KERNEL_READ_AND_WRITE|MAP_COUNT|OBJECT_(?:ALLOCATION_FAILURE|BUFFER|IMAGE1D|IMAGE1D_ARRAY|IMAGE1D_BUFFER|IMAGE2D|IMAGE2D_ARRAY|IMAGE3D|PIPE)|OFFSET|READ_ONLY|READ_WRITE|REFERENCE_COUNT|SIZE|SVM_ATOMICS|SVM_FINE_GRAIN_BUFFER|TYPE|USES_SVM_POINTER|USE_HOST_PTR|WRITE_ONLY)|MIGRATE_MEM_OBJECT_(?:CONTENT_UNDEFINED|HOST)|MISALIGNED_SUB_BUFFER_OFFSET|NONE|NON_BLOCKING|OUT_OF_(?:HOST_MEMORY|RESOURCES)|PIPE_(?:MAX_PACKETS|PACKET_SIZE)|PLATFORM_(?:EXTENSIONS|HOST_TIMER_RESOLUTION|NAME|PROFILE|VENDOR|VERSION)|PROFILING_(?:COMMAND_(?:COMPLETE|END|QUEUED|START|SUBMIT)|INFO_NOT_AVAILABLE)|PROGRAM_(?:BINARIES|BINARY_SIZES|BINARY_TYPE(?:_COMPILED_OBJECT|_EXECUTABLE|_LIBRARY|_NONE)?|BUILD_(?:GLOBAL_VARIABLE_TOTAL_SIZE|LOG|OPTIONS|STATUS)|CONTEXT|DEVICES|IL|KERNEL_NAMES|NUM_DEVICES|NUM_KERNELS|REFERENCE_COUNT|SOURCE)|QUEUED|QUEUE_(?:CONTEXT|DEVICE|DEVICE_DEFAULT|ON_DEVICE|ON_DEVICE_DEFAULT|OUT_OF_ORDER_EXEC_MODE_ENABLE|PROFILING_ENABLE|PROPERTIES|REFERENCE_COUNT|SIZE)|R|RA|READ_(?:ONLY|WRITE)_CACHE|RG|RGB|RGBA|RGBx|RGx|RUNNING|Rx|SAMPLER_(?:ADDRESSING_MODE|CONTEXT|FILTER_MODE|LOD_MAX|LOD_MIN|MIP_FILTER_MODE|NORMALIZED_COORDS|REFERENCE_COUNT)|(?:UN)?SIGNED_INT(?:8|16|32)|SNORM_INT(?:8|16)|SUBMITTED|SUCCESS|UNORM_INT(?:8|16|24|_101010|_101010_2)|UNORM_SHORT_(?:555|565)|VERSION_(?:1_0|1_1|1_2|2_0|2_1)|sBGRA|sRGB|sRGBA|sRGBx)\b/,
          alias: 'constant'
        },
        // Extracted from cl.h (2.0) and http://streamcomputing.eu/downloads/?opencl_host.lang (opencl-host)
        'function-opencl-host': {
          pattern:
            /\bcl(?:BuildProgram|CloneKernel|CompileProgram|Create(?:Buffer|CommandQueue(?:WithProperties)?|Context|ContextFromType|Image|Image2D|Image3D|Kernel|KernelsInProgram|Pipe|ProgramWith(?:Binary|BuiltInKernels|IL|Source)|Sampler|SamplerWithProperties|SubBuffer|SubDevices|UserEvent)|Enqueue(?:(?:Barrier|Marker)(?:WithWaitList)?|Copy(?:Buffer(?:Rect|ToImage)?|Image(?:ToBuffer)?)|(?:Fill|Map)(?:Buffer|Image)|MigrateMemObjects|NDRangeKernel|NativeKernel|(?:Read|Write)(?:Buffer(?:Rect)?|Image)|SVM(?:Free|Map|MemFill|Memcpy|MigrateMem|Unmap)|Task|UnmapMemObject|WaitForEvents)|Finish|Flush|Get(?:CommandQueueInfo|ContextInfo|Device(?:AndHostTimer|IDs|Info)|Event(?:Profiling)?Info|ExtensionFunctionAddress(?:ForPlatform)?|HostTimer|ImageInfo|Kernel(?:ArgInfo|Info|SubGroupInfo|WorkGroupInfo)|MemObjectInfo|PipeInfo|Platform(?:IDs|Info)|Program(?:Build)?Info|SamplerInfo|SupportedImageFormats)|LinkProgram|(?:Release|Retain)(?:CommandQueue|Context|Device|Event|Kernel|MemObject|Program|Sampler)|SVM(?:Alloc|Free)|Set(?:CommandQueueProperty|DefaultDeviceCommandQueue|EventCallback|Kernel|Kernel(?:Arg(?:SVMPointer)?|ExecInfo)|MemObjectDestructorCallback|UserEventStatus)|Unload(?:Platform)?Compiler|WaitForEvents)\b/,
          alias: 'function'
        }
      };
      /* OpenCL host API */
      Prism.languages.insertBefore('c', 'keyword', attributes); // C++ includes everything from the OpenCL C host API plus the classes defined in cl2.h
      if (Prism.languages.cpp) {
        // Extracted from doxygen class list http://github.khronos.org/OpenCL-CLHPP/annotated.html
        attributes['type-opencl-host-cpp'] = {
          pattern:
            /\b(?:Buffer|BufferGL|BufferRenderGL|CommandQueue|Context|Device|DeviceCommandQueue|EnqueueArgs|Event|Image|Image1D|Image1DArray|Image1DBuffer|Image2D|Image2DArray|Image2DGL|Image3D|Image3DGL|ImageFormat|ImageGL|Kernel|KernelFunctor|LocalSpaceArg|Memory|NDRange|Pipe|Platform|Program|SVMAllocator|SVMTraitAtomic|SVMTraitCoarse|SVMTraitFine|SVMTraitReadOnly|SVMTraitReadWrite|SVMTraitWriteOnly|Sampler|UserEvent)\b/,
          alias: 'keyword'
        };
        Prism.languages.insertBefore('cpp', 'keyword', attributes);
      }
    })(Prism);
  }

  // @ts-nocheck
  openqasm.displayName = 'openqasm';
  openqasm.aliases = ['qasm'];

  /** @type {import('../core.js').Syntax} */
  function openqasm(Prism) {
    // https://qiskit.github.io/openqasm/grammar/index.html
    Prism.languages.openqasm = {
      comment: /\/\*[\s\S]*?\*\/|\/\/.*/,
      string: {
        pattern: /"[^"\r\n\t]*"|'[^'\r\n\t]*'/,
        greedy: true
      },
      keyword:
        /\b(?:CX|OPENQASM|U|barrier|boxas|boxto|break|const|continue|ctrl|def|defcal|defcalgrammar|delay|else|end|for|gate|gphase|if|in|include|inv|kernel|lengthof|let|measure|pow|reset|return|rotary|stretchinf|while)\b|#pragma\b/,
      'class-name':
        /\b(?:angle|bit|bool|creg|fixed|float|int|length|qreg|qubit|stretch|uint)\b/,
      function: /\b(?:cos|exp|ln|popcount|rotl|rotr|sin|sqrt|tan)\b(?=\s*\()/,
      constant: /\b(?:euler|pi|tau)\b|π|𝜏|ℇ/,
      number: {
        pattern:
          /(^|[^.\w$])(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?(?:dt|ns|us|µs|ms|s)?/i,
        lookbehind: true
      },
      operator: /->|>>=?|<<=?|&&|\|\||\+\+|--|[!=<>&|~^+\-*/%]=?|@/,
      punctuation: /[(){}\[\];,:.]/
    };
    Prism.languages.qasm = Prism.languages.openqasm;
  }

  // @ts-nocheck
  oz.displayName = 'oz';
  oz.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function oz(Prism) {
    Prism.languages.oz = {
      comment: {
        pattern: /\/\*[\s\S]*?\*\/|%.*/,
        greedy: true
      },
      string: {
        pattern: /"(?:[^"\\]|\\[\s\S])*"/,
        greedy: true
      },
      atom: {
        pattern: /'(?:[^'\\]|\\[\s\S])*'/,
        greedy: true,
        alias: 'builtin'
      },
      keyword:
        /\$|\[\]|\b(?:_|at|attr|case|catch|choice|class|cond|declare|define|dis|else(?:case|if)?|end|export|fail|false|feat|finally|from|fun|functor|if|import|in|local|lock|meth|nil|not|of|or|prepare|proc|prop|raise|require|self|skip|then|thread|true|try|unit)\b/,
      function: [
        /\b[a-z][A-Za-z\d]*(?=\()/,
        {
          pattern: /(\{)[A-Z][A-Za-z\d]*\b/,
          lookbehind: true
        }
      ],
      number:
        /\b(?:0[bx][\da-f]+|\d+(?:\.\d*)?(?:e~?\d+)?)\b|&(?:[^\\]|\\(?:\d{3}|.))/i,
      variable: /`(?:[^`\\]|\\.)+`/,
      'attr-name': /\b\w+(?=[ \t]*:(?![:=]))/,
      operator:
        /:(?:=|::?)|<[-:=]?|=(?:=|<?:?)|>=?:?|\\=:?|!!?|[|#+\-*\/,~^@]|\b(?:andthen|div|mod|orelse)\b/,
      punctuation: /[\[\](){}.:;?]/
    };
  }

  // @ts-nocheck
  parigp.displayName = 'parigp';
  parigp.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function parigp(Prism) {
    Prism.languages.parigp = {
      comment: /\/\*[\s\S]*?\*\/|\\\\.*/,
      string: {
        pattern: /"(?:[^"\\\r\n]|\\.)*"/,
        greedy: true
      },
      // PARI/GP does not care about white spaces at all
      // so let's process the keywords to build an appropriate regexp
      // (e.g. "b *r *e *a *k", etc.)
      keyword: (function () {
        var keywords = [
          'breakpoint',
          'break',
          'dbg_down',
          'dbg_err',
          'dbg_up',
          'dbg_x',
          'forcomposite',
          'fordiv',
          'forell',
          'forpart',
          'forprime',
          'forstep',
          'forsubgroup',
          'forvec',
          'for',
          'iferr',
          'if',
          'local',
          'my',
          'next',
          'return',
          'until',
          'while'
        ];
        keywords = keywords
          .map(function (keyword) {
            return keyword.split('').join(' *')
          })
          .join('|');
        return RegExp('\\b(?:' + keywords + ')\\b')
      })(),
      function: /\b\w(?:[\w ]*\w)?(?= *\()/,
      number: {
        // The lookbehind and the negative lookahead prevent from breaking the .. operator
        pattern:
          /((?:\. *\. *)?)(?:\b\d(?: *\d)*(?: *(?!\. *\.)\.(?: *\d)*)?|\. *\d(?: *\d)*)(?: *e *(?:[+-] *)?\d(?: *\d)*)?/i,
        lookbehind: true
      },
      operator:
        /\. *\.|[*\/!](?: *=)?|%(?: *=|(?: *#)?(?: *')*)?|\+(?: *[+=])?|-(?: *[-=>])?|<(?: *>|(?: *<)?(?: *=)?)?|>(?: *>)?(?: *=)?|=(?: *=){0,2}|\\(?: *\/)?(?: *=)?|&(?: *&)?|\| *\||['#~^]/,
      punctuation: /[\[\]{}().,:;|]/
    };
  }

  // @ts-nocheck
  parser.displayName = 'parser';
  parser.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function parser(Prism) {
    Prism.register(markup)
    ;(function (Prism) {
      var parser = (Prism.languages.parser = Prism.languages.extend('markup', {
        keyword: {
          pattern:
            /(^|[^^])(?:\^(?:case|eval|for|if|switch|throw)\b|@(?:BASE|CLASS|GET(?:_DEFAULT)?|OPTIONS|SET_DEFAULT|USE)\b)/,
          lookbehind: true
        },
        variable: {
          pattern: /(^|[^^])\B\$(?:\w+|(?=[.{]))(?:(?:\.|::?)\w+)*(?:\.|::?)?/,
          lookbehind: true,
          inside: {
            punctuation: /\.|:+/
          }
        },
        function: {
          pattern: /(^|[^^])\B[@^]\w+(?:(?:\.|::?)\w+)*(?:\.|::?)?/,
          lookbehind: true,
          inside: {
            keyword: {
              pattern: /(^@)(?:GET_|SET_)/,
              lookbehind: true
            },
            punctuation: /\.|:+/
          }
        },
        escape: {
          pattern: /\^(?:[$^;@()\[\]{}"':]|#[a-f\d]*)/i,
          alias: 'builtin'
        },
        punctuation: /[\[\](){};]/
      }));
      parser = Prism.languages.insertBefore('parser', 'keyword', {
        'parser-comment': {
          pattern: /(\s)#.*/,
          lookbehind: true,
          alias: 'comment'
        },
        expression: {
          // Allow for 3 levels of depth
          pattern: /(^|[^^])\((?:[^()]|\((?:[^()]|\((?:[^()])*\))*\))*\)/,
          greedy: true,
          lookbehind: true,
          inside: {
            string: {
              pattern: /(^|[^^])(["'])(?:(?!\2)[^^]|\^[\s\S])*\2/,
              lookbehind: true
            },
            keyword: parser.keyword,
            variable: parser.variable,
            function: parser.function,
            boolean: /\b(?:false|true)\b/,
            number: /\b(?:0x[a-f\d]+|\d+(?:\.\d*)?(?:e[+-]?\d+)?)\b/i,
            escape: parser.escape,
            operator:
              /[~+*\/\\%]|!(?:\|\|?|=)?|&&?|\|\|?|==|<[<=]?|>[>=]?|-[fd]?|\b(?:def|eq|ge|gt|in|is|le|lt|ne)\b/,
            punctuation: parser.punctuation
          }
        }
      });
      Prism.languages.insertBefore(
        'inside',
        'punctuation',
        {
          expression: parser.expression,
          keyword: parser.keyword,
          variable: parser.variable,
          function: parser.function,
          escape: parser.escape,
          'parser-punctuation': {
            pattern: parser.punctuation,
            alias: 'punctuation'
          }
        },
        parser['tag'].inside['attr-value']
      );
    })(Prism);
  }

  // @ts-nocheck
  pascal.displayName = 'pascal';
  pascal.aliases = ['objectpascal'];

  /** @type {import('../core.js').Syntax} */
  function pascal(Prism) {
    // Based on Free Pascal
    /* TODO
  Support inline asm ?
  */
    Prism.languages.pascal = {
      directive: {
        pattern: /\{\$[\s\S]*?\}/,
        greedy: true,
        alias: ['marco', 'property']
      },
      comment: {
        pattern: /\(\*[\s\S]*?\*\)|\{[\s\S]*?\}|\/\/.*/,
        greedy: true
      },
      string: {
        pattern: /(?:'(?:''|[^'\r\n])*'(?!')|#[&$%]?[a-f\d]+)+|\^[a-z]/i,
        greedy: true
      },
      asm: {
        pattern: /(\basm\b)[\s\S]+?(?=\bend\s*[;[])/i,
        lookbehind: true,
        greedy: true,
        inside: null // see below
      },
      keyword: [
        {
          // Turbo Pascal
          pattern:
            /(^|[^&])\b(?:absolute|array|asm|begin|case|const|constructor|destructor|do|downto|else|end|file|for|function|goto|if|implementation|inherited|inline|interface|label|nil|object|of|operator|packed|procedure|program|record|reintroduce|repeat|self|set|string|then|to|type|unit|until|uses|var|while|with)\b/i,
          lookbehind: true
        },
        {
          // Free Pascal
          pattern: /(^|[^&])\b(?:dispose|exit|false|new|true)\b/i,
          lookbehind: true
        },
        {
          // Object Pascal
          pattern:
            /(^|[^&])\b(?:class|dispinterface|except|exports|finalization|finally|initialization|inline|library|on|out|packed|property|raise|resourcestring|threadvar|try)\b/i,
          lookbehind: true
        },
        {
          // Modifiers
          pattern:
            /(^|[^&])\b(?:absolute|abstract|alias|assembler|bitpacked|break|cdecl|continue|cppdecl|cvar|default|deprecated|dynamic|enumerator|experimental|export|external|far|far16|forward|generic|helper|implements|index|interrupt|iochecks|local|message|name|near|nodefault|noreturn|nostackframe|oldfpccall|otherwise|overload|override|pascal|platform|private|protected|public|published|read|register|reintroduce|result|safecall|saveregisters|softfloat|specialize|static|stdcall|stored|strict|unaligned|unimplemented|varargs|virtual|write)\b/i,
          lookbehind: true
        }
      ],
      number: [
        // Hexadecimal, octal and binary
        /(?:[&%]\d+|\$[a-f\d]+)/i, // Decimal
        /\b\d+(?:\.\d+)?(?:e[+-]?\d+)?/i
      ],
      operator: [
        /\.\.|\*\*|:=|<[<=>]?|>[>=]?|[+\-*\/]=?|[@^=]/,
        {
          pattern:
            /(^|[^&])\b(?:and|as|div|exclude|in|include|is|mod|not|or|shl|shr|xor)\b/,
          lookbehind: true
        }
      ],
      punctuation: /\(\.|\.\)|[()\[\]:;,.]/
    };
    Prism.languages.pascal.asm.inside = Prism.languages.extend('pascal', {
      asm: undefined,
      keyword: undefined,
      operator: undefined
    });
    Prism.languages.objectpascal = Prism.languages.pascal;
  }

  // @ts-nocheck
  pascaligo.displayName = 'pascaligo';
  pascaligo.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function pascaligo(Prism) {
  (function (Prism) {
      // Pascaligo is a layer 2 smart contract language for the tezos blockchain
      var braces = /\((?:[^()]|\((?:[^()]|\([^()]*\))*\))*\)/.source;
      var type = /(?:\b\w+(?:<braces>)?|<braces>)/.source.replace(
        /<braces>/g,
        function () {
          return braces
        }
      );
      var pascaligo = (Prism.languages.pascaligo = {
        comment: /\(\*[\s\S]+?\*\)|\/\/.*/,
        string: {
          pattern: /(["'`])(?:\\[\s\S]|(?!\1)[^\\])*\1|\^[a-z]/i,
          greedy: true
        },
        'class-name': [
          {
            pattern: RegExp(
              /(\btype\s+\w+\s+is\s+)<type>/.source.replace(
                /<type>/g,
                function () {
                  return type
                }
              ),
              'i'
            ),
            lookbehind: true,
            inside: null // see below
          },
          {
            pattern: RegExp(
              /<type>(?=\s+is\b)/.source.replace(/<type>/g, function () {
                return type
              }),
              'i'
            ),
            inside: null // see below
          },
          {
            pattern: RegExp(
              /(:\s*)<type>/.source.replace(/<type>/g, function () {
                return type
              })
            ),
            lookbehind: true,
            inside: null // see below
          }
        ],
        keyword: {
          pattern:
            /(^|[^&])\b(?:begin|block|case|const|else|end|fail|for|from|function|if|is|nil|of|remove|return|skip|then|type|var|while|with)\b/i,
          lookbehind: true
        },
        boolean: {
          pattern: /(^|[^&])\b(?:False|True)\b/i,
          lookbehind: true
        },
        builtin: {
          pattern: /(^|[^&])\b(?:bool|int|list|map|nat|record|string|unit)\b/i,
          lookbehind: true
        },
        function: /\b\w+(?=\s*\()/,
        number: [
          // Hexadecimal, octal and binary
          /%[01]+|&[0-7]+|\$[a-f\d]+/i, // Decimal
          /\b\d+(?:\.\d+)?(?:e[+-]?\d+)?(?:mtz|n)?/i
        ],
        operator:
          /->|=\/=|\.\.|\*\*|:=|<[<=>]?|>[>=]?|[+\-*\/]=?|[@^=|]|\b(?:and|mod|or)\b/,
        punctuation: /\(\.|\.\)|[()\[\]:;,.{}]/
      });
      var classNameInside = [
        'comment',
        'keyword',
        'builtin',
        'operator',
        'punctuation'
      ].reduce(function (accum, key) {
        accum[key] = pascaligo[key];
        return accum
      }, {});
      pascaligo['class-name'].forEach(function (p) {
        p.inside = classNameInside;
      });
    })(Prism);
  }

  // @ts-nocheck
  psl.displayName = 'psl';
  psl.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function psl(Prism) {
    Prism.languages.psl = {
      comment: {
        pattern: /#.*/,
        greedy: true
      },
      string: {
        pattern: /"(?:\\.|[^\\"])*"/,
        greedy: true,
        inside: {
          symbol: /\\[ntrbA-Z"\\]/
        }
      },
      'heredoc-string': {
        pattern: /<<<([a-zA-Z_]\w*)[\r\n](?:.*[\r\n])*?\1\b/,
        alias: 'string',
        greedy: true
      },
      keyword:
        /\b(?:__multi|__single|case|default|do|else|elsif|exit|export|for|foreach|function|if|last|line|local|next|requires|return|switch|until|while|word)\b/,
      constant:
        /\b(?:ALARM|CHART_ADD_GRAPH|CHART_DELETE_GRAPH|CHART_DESTROY|CHART_LOAD|CHART_PRINT|EOF|OFFLINE|OK|PSL_PROF_LOG|R_CHECK_HORIZ|R_CHECK_VERT|R_CLICKER|R_COLUMN|R_FRAME|R_ICON|R_LABEL|R_LABEL_CENTER|R_LIST_MULTIPLE|R_LIST_MULTIPLE_ND|R_LIST_SINGLE|R_LIST_SINGLE_ND|R_MENU|R_POPUP|R_POPUP_SCROLLED|R_RADIO_HORIZ|R_RADIO_VERT|R_ROW|R_SCALE_HORIZ|R_SCALE_VERT|R_SEP_HORIZ|R_SEP_VERT|R_SPINNER|R_TEXT_FIELD|R_TEXT_FIELD_LABEL|R_TOGGLE|TRIM_LEADING|TRIM_LEADING_AND_TRAILING|TRIM_REDUNDANT|TRIM_TRAILING|VOID|WARN)\b/,
      boolean: /\b(?:FALSE|False|NO|No|TRUE|True|YES|Yes|false|no|true|yes)\b/,
      variable: /\b(?:PslDebug|errno|exit_status)\b/,
      builtin: {
        pattern:
          /\b(?:PslExecute|PslFunctionCall|PslFunctionExists|PslSetOptions|_snmp_debug|acos|add_diary|annotate|annotate_get|ascii_to_ebcdic|asctime|asin|atan|atexit|batch_set|blackout|cat|ceil|chan_exists|change_state|close|code_cvt|cond_signal|cond_wait|console_type|convert_base|convert_date|convert_locale_date|cos|cosh|create|date|dcget_text|destroy|destroy_lock|dget_text|difference|dump_hist|ebcdic_to_ascii|encrypt|event_archive|event_catalog_get|event_check|event_query|event_range_manage|event_range_query|event_report|event_schedule|event_trigger|event_trigger2|execute|exists|exp|fabs|file|floor|fmod|fopen|fseek|ftell|full_discovery|get|get_chan_info|get_ranges|get_text|get_vars|getenv|gethostinfo|getpid|getpname|grep|history|history_get_retention|in_transition|index|int|internal|intersection|is_var|isnumber|join|kill|length|lines|lock|lock_info|log|log10|loge|matchline|msg_check|msg_get_format|msg_get_severity|msg_printf|msg_sprintf|ntharg|nthargf|nthline|nthlinef|num_bytes|num_consoles|pconfig|popen|poplines|pow|print|printf|proc_exists|process|random|read|readln|refresh_parameters|remote_check|remote_close|remote_event_query|remote_event_trigger|remote_file_send|remote_open|remove|replace|rindex|sec_check_priv|sec_store_get|sec_store_set|set|set_alarm_ranges|set_locale|share|sin|sinh|sleep|snmp_agent_config|snmp_agent_start|snmp_agent_stop|snmp_close|snmp_config|snmp_get|snmp_get_next|snmp_h_get|snmp_h_get_next|snmp_h_set|snmp_open|snmp_set|snmp_trap_ignore|snmp_trap_listen|snmp_trap_raise_std_trap|snmp_trap_receive|snmp_trap_register_im|snmp_trap_send|snmp_walk|sopen|sort|splitline|sprintf|sqrt|srandom|str_repeat|strcasecmp|subset|substr|system|tail|tan|tanh|text_domain|time|tmpnam|tolower|toupper|trace_psl_process|trim|union|unique|unlock|unset|va_arg|va_start|write)\b/,
        alias: 'builtin-function'
      },
      'foreach-variable': {
        pattern:
          /(\bforeach\s+(?:(?:\w+\b|"(?:\\.|[^\\"])*")\s+){0,2})[_a-zA-Z]\w*(?=\s*\()/,
        lookbehind: true,
        greedy: true
      },
      function: /\b[_a-z]\w*\b(?=\s*\()/i,
      number: /\b(?:0x[0-9a-f]+|\d+(?:\.\d+)?)\b/i,
      operator: /--|\+\+|&&=?|\|\|=?|<<=?|>>=?|[=!]~|[-+*/%&|^!=<>]=?|\.|[:?]/,
      punctuation: /[(){}\[\];,]/
    };
  }

  // @ts-nocheck
  pcaxis.displayName = 'pcaxis';
  pcaxis.aliases = ['px'];

  /** @type {import('../core.js').Syntax} */
  function pcaxis(Prism) {
    Prism.languages.pcaxis = {
      string: /"[^"]*"/,
      keyword: {
        pattern:
          /((?:^|;)\s*)[-A-Z\d]+(?:\s*\[[-\w]+\])?(?:\s*\("[^"]*"(?:,\s*"[^"]*")*\))?(?=\s*=)/,
        lookbehind: true,
        greedy: true,
        inside: {
          keyword: /^[-A-Z\d]+/,
          language: {
            pattern: /^(\s*)\[[-\w]+\]/,
            lookbehind: true,
            inside: {
              punctuation: /^\[|\]$/,
              property: /[-\w]+/
            }
          },
          'sub-key': {
            pattern: /^(\s*)\S[\s\S]*/,
            lookbehind: true,
            inside: {
              parameter: {
                pattern: /"[^"]*"/,
                alias: 'property'
              },
              punctuation: /^\(|\)$|,/
            }
          }
        }
      },
      operator: /=/,
      tlist: {
        pattern:
          /TLIST\s*\(\s*\w+(?:(?:\s*,\s*"[^"]*")+|\s*,\s*"[^"]*"-"[^"]*")?\s*\)/,
        greedy: true,
        inside: {
          function: /^TLIST/,
          property: {
            pattern: /^(\s*\(\s*)\w+/,
            lookbehind: true
          },
          string: /"[^"]*"/,
          punctuation: /[(),]/,
          operator: /-/
        }
      },
      punctuation: /[;,]/,
      number: {
        pattern: /(^|\s)\d+(?:\.\d+)?(?!\S)/,
        lookbehind: true
      },
      boolean: /NO|YES/
    };
    Prism.languages.px = Prism.languages.pcaxis;
  }

  // @ts-nocheck
  peoplecode.displayName = 'peoplecode';
  peoplecode.aliases = ['pcode'];

  /** @type {import('../core.js').Syntax} */
  function peoplecode(Prism) {
    Prism.languages.peoplecode = {
      comment: RegExp(
        [
          // C-style multiline comments
          /\/\*[\s\S]*?\*\//.source, // REM comments
          /\bREM[^;]*;/.source, // Nested <* *> comments
          /<\*(?:[^<*]|\*(?!>)|<(?!\*)|<\*(?:(?!\*>)[\s\S])*\*>)*\*>/.source, // /+ +/ comments
          /\/\+[\s\S]*?\+\//.source
        ].join('|')
      ),
      string: {
        pattern: /'(?:''|[^'\r\n])*'(?!')|"(?:""|[^"\r\n])*"(?!")/,
        greedy: true
      },
      variable: /%\w+/,
      'function-definition': {
        pattern: /((?:^|[^\w-])(?:function|method)\s+)\w+/i,
        lookbehind: true,
        alias: 'function'
      },
      'class-name': {
        pattern:
          /((?:^|[^-\w])(?:as|catch|class|component|create|extends|global|implements|instance|local|of|property|returns)\s+)\w+(?::\w+)*/i,
        lookbehind: true,
        inside: {
          punctuation: /:/
        }
      },
      keyword:
        /\b(?:abstract|alias|as|catch|class|component|constant|create|declare|else|end-(?:class|evaluate|for|function|get|if|method|set|try|while)|evaluate|extends|for|function|get|global|if|implements|import|instance|library|local|method|null|of|out|peopleCode|private|program|property|protected|readonly|ref|repeat|returns?|set|step|then|throw|to|try|until|value|when(?:-other)?|while)\b/i,
      'operator-keyword': {
        pattern: /\b(?:and|not|or)\b/i,
        alias: 'operator'
      },
      function: /[_a-z]\w*(?=\s*\()/i,
      boolean: /\b(?:false|true)\b/i,
      number: /\b\d+(?:\.\d+)?\b/,
      operator: /<>|[<>]=?|!=|\*\*|[-+*/|=@]/,
      punctuation: /[:.;,()[\]]/
    };
    Prism.languages.pcode = Prism.languages.peoplecode;
  }

  // @ts-nocheck
  phpdoc.displayName = 'phpdoc';
  phpdoc.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function phpdoc(Prism) {
    Prism.register(javadoclike);
    Prism.register(php)
    ;(function (Prism) {
      var typeExpression = /(?:\b[a-zA-Z]\w*|[|\\[\]])+/.source;
      Prism.languages.phpdoc = Prism.languages.extend('javadoclike', {
        parameter: {
          pattern: RegExp(
            '(@(?:global|param|property(?:-read|-write)?|var)\\s+(?:' +
              typeExpression +
              '\\s+)?)\\$\\w+'
          ),
          lookbehind: true
        }
      });
      Prism.languages.insertBefore('phpdoc', 'keyword', {
        'class-name': [
          {
            pattern: RegExp(
              '(@(?:global|package|param|property(?:-read|-write)?|return|subpackage|throws|var)\\s+)' +
                typeExpression
            ),
            lookbehind: true,
            inside: {
              keyword:
                /\b(?:array|bool|boolean|callback|double|false|float|int|integer|mixed|null|object|resource|self|string|true|void)\b/,
              punctuation: /[|\\[\]()]/
            }
          }
        ]
      });
      Prism.languages.javadoclike.addSupport('php', Prism.languages.phpdoc);
    })(Prism);
  }

  // @ts-nocheck
  phpExtras.displayName = 'php-extras';
  phpExtras.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function phpExtras(Prism) {
    Prism.register(php);
    Prism.languages.insertBefore('php', 'variable', {
      this: {
        pattern: /\$this\b/,
        alias: 'keyword'
      },
      global:
        /\$(?:GLOBALS|HTTP_RAW_POST_DATA|_(?:COOKIE|ENV|FILES|GET|POST|REQUEST|SERVER|SESSION)|argc|argv|http_response_header|php_errormsg)\b/,
      scope: {
        pattern: /\b[\w\\]+::/,
        inside: {
          keyword: /\b(?:parent|self|static)\b/,
          punctuation: /::|\\/
        }
      }
    });
  }

  // @ts-nocheck
  plsql.displayName = 'plsql';
  plsql.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function plsql(Prism) {
    Prism.register(sql);
    Prism.languages.plsql = Prism.languages.extend('sql', {
      comment: {
        pattern: /\/\*[\s\S]*?\*\/|--.*/,
        greedy: true
      },
      // https://docs.oracle.com/en/database/oracle/oracle-database/21/lnpls/plsql-reserved-words-keywords.html
      keyword:
        /\b(?:A|ACCESSIBLE|ADD|AGENT|AGGREGATE|ALL|ALTER|AND|ANY|ARRAY|AS|ASC|AT|ATTRIBUTE|AUTHID|AVG|BEGIN|BETWEEN|BFILE_BASE|BINARY|BLOB_BASE|BLOCK|BODY|BOTH|BOUND|BULK|BY|BYTE|C|CALL|CALLING|CASCADE|CASE|CHAR|CHARACTER|CHARSET|CHARSETFORM|CHARSETID|CHAR_BASE|CHECK|CLOB_BASE|CLONE|CLOSE|CLUSTER|CLUSTERS|COLAUTH|COLLECT|COLUMNS|COMMENT|COMMIT|COMMITTED|COMPILED|COMPRESS|CONNECT|CONSTANT|CONSTRUCTOR|CONTEXT|CONTINUE|CONVERT|COUNT|CRASH|CREATE|CREDENTIAL|CURRENT|CURSOR|CUSTOMDATUM|DANGLING|DATA|DATE|DATE_BASE|DAY|DECLARE|DEFAULT|DEFINE|DELETE|DESC|DETERMINISTIC|DIRECTORY|DISTINCT|DOUBLE|DROP|DURATION|ELEMENT|ELSE|ELSIF|EMPTY|END|ESCAPE|EXCEPT|EXCEPTION|EXCEPTIONS|EXCLUSIVE|EXECUTE|EXISTS|EXIT|EXTERNAL|FETCH|FINAL|FIRST|FIXED|FLOAT|FOR|FORALL|FORCE|FROM|FUNCTION|GENERAL|GOTO|GRANT|GROUP|HASH|HAVING|HEAP|HIDDEN|HOUR|IDENTIFIED|IF|IMMEDIATE|IMMUTABLE|IN|INCLUDING|INDEX|INDEXES|INDICATOR|INDICES|INFINITE|INSERT|INSTANTIABLE|INT|INTERFACE|INTERSECT|INTERVAL|INTO|INVALIDATE|IS|ISOLATION|JAVA|LANGUAGE|LARGE|LEADING|LENGTH|LEVEL|LIBRARY|LIKE|LIKE2|LIKE4|LIKEC|LIMIT|LIMITED|LOCAL|LOCK|LONG|LOOP|MAP|MAX|MAXLEN|MEMBER|MERGE|MIN|MINUS|MINUTE|MOD|MODE|MODIFY|MONTH|MULTISET|MUTABLE|NAME|NAN|NATIONAL|NATIVE|NCHAR|NEW|NOCOMPRESS|NOCOPY|NOT|NOWAIT|NULL|NUMBER_BASE|OBJECT|OCICOLL|OCIDATE|OCIDATETIME|OCIDURATION|OCIINTERVAL|OCILOBLOCATOR|OCINUMBER|OCIRAW|OCIREF|OCIREFCURSOR|OCIROWID|OCISTRING|OCITYPE|OF|OLD|ON|ONLY|OPAQUE|OPEN|OPERATOR|OPTION|OR|ORACLE|ORADATA|ORDER|ORGANIZATION|ORLANY|ORLVARY|OTHERS|OUT|OVERLAPS|OVERRIDING|PACKAGE|PARALLEL_ENABLE|PARAMETER|PARAMETERS|PARENT|PARTITION|PASCAL|PERSISTABLE|PIPE|PIPELINED|PLUGGABLE|POLYMORPHIC|PRAGMA|PRECISION|PRIOR|PRIVATE|PROCEDURE|PUBLIC|RAISE|RANGE|RAW|READ|RECORD|REF|REFERENCE|RELIES_ON|REM|REMAINDER|RENAME|RESOURCE|RESULT|RESULT_CACHE|RETURN|RETURNING|REVERSE|REVOKE|ROLLBACK|ROW|SAMPLE|SAVE|SAVEPOINT|SB1|SB2|SB4|SECOND|SEGMENT|SELECT|SELF|SEPARATE|SEQUENCE|SERIALIZABLE|SET|SHARE|SHORT|SIZE|SIZE_T|SOME|SPARSE|SQL|SQLCODE|SQLDATA|SQLNAME|SQLSTATE|STANDARD|START|STATIC|STDDEV|STORED|STRING|STRUCT|STYLE|SUBMULTISET|SUBPARTITION|SUBSTITUTABLE|SUBTYPE|SUM|SYNONYM|TABAUTH|TABLE|TDO|THE|THEN|TIME|TIMESTAMP|TIMEZONE_ABBR|TIMEZONE_HOUR|TIMEZONE_MINUTE|TIMEZONE_REGION|TO|TRAILING|TRANSACTION|TRANSACTIONAL|TRUSTED|TYPE|UB1|UB2|UB4|UNDER|UNION|UNIQUE|UNPLUG|UNSIGNED|UNTRUSTED|UPDATE|USE|USING|VALIST|VALUE|VALUES|VARIABLE|VARIANCE|VARRAY|VARYING|VIEW|VIEWS|VOID|WHEN|WHERE|WHILE|WITH|WORK|WRAPPED|WRITE|YEAR|ZONE)\b/i,
      // https://docs.oracle.com/en/database/oracle/oracle-database/21/lnpls/plsql-language-fundamentals.html#GUID-96A42F7C-7A71-4B90-8255-CA9C8BD9722E
      operator: /:=?|=>|[<>^~!]=|\.\.|\|\||\*\*|[-+*/%<>=@]/
    });
    Prism.languages.insertBefore('plsql', 'operator', {
      label: {
        pattern: /<<\s*\w+\s*>>/,
        alias: 'symbol'
      }
    });
  }

  // @ts-nocheck
  powerquery.displayName = 'powerquery';
  powerquery.aliases = ['mscript', 'pq'];

  /** @type {import('../core.js').Syntax} */
  function powerquery(Prism) {
    // https://docs.microsoft.com/en-us/powerquery-m/power-query-m-language-specification
    Prism.languages.powerquery = {
      comment: {
        pattern: /(^|[^\\])(?:\/\*[\s\S]*?\*\/|\/\/.*)/,
        lookbehind: true,
        greedy: true
      },
      'quoted-identifier': {
        pattern: /#"(?:[^"\r\n]|"")*"(?!")/,
        greedy: true
      },
      string: {
        pattern: /(?:#!)?"(?:[^"\r\n]|"")*"(?!")/,
        greedy: true
      },
      constant: [
        /\bDay\.(?:Friday|Monday|Saturday|Sunday|Thursday|Tuesday|Wednesday)\b/,
        /\bTraceLevel\.(?:Critical|Error|Information|Verbose|Warning)\b/,
        /\bOccurrence\.(?:All|First|Last)\b/,
        /\bOrder\.(?:Ascending|Descending)\b/,
        /\bRoundingMode\.(?:AwayFromZero|Down|ToEven|TowardZero|Up)\b/,
        /\bMissingField\.(?:Error|Ignore|UseNull)\b/,
        /\bQuoteStyle\.(?:Csv|None)\b/,
        /\bJoinKind\.(?:FullOuter|Inner|LeftAnti|LeftOuter|RightAnti|RightOuter)\b/,
        /\bGroupKind\.(?:Global|Local)\b/,
        /\bExtraValues\.(?:Error|Ignore|List)\b/,
        /\bJoinAlgorithm\.(?:Dynamic|LeftHash|LeftIndex|PairwiseHash|RightHash|RightIndex|SortMerge)\b/,
        /\bJoinSide\.(?:Left|Right)\b/,
        /\bPrecision\.(?:Decimal|Double)\b/,
        /\bRelativePosition\.From(?:End|Start)\b/,
        /\bTextEncoding\.(?:Ascii|BigEndianUnicode|Unicode|Utf16|Utf8|Windows)\b/,
        /\b(?:Any|Binary|Date|DateTime|DateTimeZone|Duration|Function|Int16|Int32|Int64|Int8|List|Logical|None|Number|Record|Table|Text|Time)\.Type\b/,
        /\bnull\b/
      ],
      boolean: /\b(?:false|true)\b/,
      keyword:
        /\b(?:and|as|each|else|error|if|in|is|let|meta|not|nullable|optional|or|otherwise|section|shared|then|try|type)\b|#(?:binary|date|datetime|datetimezone|duration|infinity|nan|sections|shared|table|time)\b/,
      function: {
        pattern: /(^|[^#\w.])[a-z_][\w.]*(?=\s*\()/i,
        lookbehind: true
      },
      'data-type': {
        pattern:
          /\b(?:any|anynonnull|binary|date|datetime|datetimezone|duration|function|list|logical|none|number|record|table|text|time)\b/,
        alias: 'class-name'
      },
      number: {
        pattern:
          /\b0x[\da-f]+\b|(?:[+-]?(?:\b\d+\.)?\b\d+|[+-]\.\d+|(^|[^.])\B\.\d+)(?:e[+-]?\d+)?\b/i,
        lookbehind: true
      },
      operator: /[-+*\/&?@^]|<(?:=>?|>)?|>=?|=>?|\.\.\.?/,
      punctuation: /[,;\[\](){}]/
    };
    Prism.languages.pq = Prism.languages['powerquery'];
    Prism.languages.mscript = Prism.languages['powerquery'];
  }

  // @ts-nocheck
  powershell.displayName = 'powershell';
  powershell.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function powershell(Prism) {
  (function (Prism) {
      var powershell = (Prism.languages.powershell = {
        comment: [
          {
            pattern: /(^|[^`])<#[\s\S]*?#>/,
            lookbehind: true
          },
          {
            pattern: /(^|[^`])#.*/,
            lookbehind: true
          }
        ],
        string: [
          {
            pattern: /"(?:`[\s\S]|[^`"])*"/,
            greedy: true,
            inside: null // see below
          },
          {
            pattern: /'(?:[^']|'')*'/,
            greedy: true
          }
        ],
        // Matches name spaces as well as casts, attribute decorators. Force starting with letter to avoid matching array indices
        // Supports two levels of nested brackets (e.g. `[OutputType([System.Collections.Generic.List[int]])]`)
        namespace: /\[[a-z](?:\[(?:\[[^\]]*\]|[^\[\]])*\]|[^\[\]])*\]/i,
        boolean: /\$(?:false|true)\b/i,
        variable: /\$\w+\b/,
        // Cmdlets and aliases. Aliases should come last, otherwise "write" gets preferred over "write-host" for example
        // Get-Command | ?{ $_.ModuleName -match "Microsoft.PowerShell.(Util|Core|Management)" }
        // Get-Alias | ?{ $_.ReferencedCommand.Module.Name -match "Microsoft.PowerShell.(Util|Core|Management)" }
        function: [
          /\b(?:Add|Approve|Assert|Backup|Block|Checkpoint|Clear|Close|Compare|Complete|Compress|Confirm|Connect|Convert|ConvertFrom|ConvertTo|Copy|Debug|Deny|Disable|Disconnect|Dismount|Edit|Enable|Enter|Exit|Expand|Export|Find|ForEach|Format|Get|Grant|Group|Hide|Import|Initialize|Install|Invoke|Join|Limit|Lock|Measure|Merge|Move|New|Open|Optimize|Out|Ping|Pop|Protect|Publish|Push|Read|Receive|Redo|Register|Remove|Rename|Repair|Request|Reset|Resize|Resolve|Restart|Restore|Resume|Revoke|Save|Search|Select|Send|Set|Show|Skip|Sort|Split|Start|Step|Stop|Submit|Suspend|Switch|Sync|Tee|Test|Trace|Unblock|Undo|Uninstall|Unlock|Unprotect|Unpublish|Unregister|Update|Use|Wait|Watch|Where|Write)-[a-z]+\b/i,
          /\b(?:ac|cat|chdir|clc|cli|clp|clv|compare|copy|cp|cpi|cpp|cvpa|dbp|del|diff|dir|ebp|echo|epal|epcsv|epsn|erase|fc|fl|ft|fw|gal|gbp|gc|gci|gcs|gdr|gi|gl|gm|gp|gps|group|gsv|gu|gv|gwmi|iex|ii|ipal|ipcsv|ipsn|irm|iwmi|iwr|kill|lp|ls|measure|mi|mount|move|mp|mv|nal|ndr|ni|nv|ogv|popd|ps|pushd|pwd|rbp|rd|rdr|ren|ri|rm|rmdir|rni|rnp|rp|rv|rvpa|rwmi|sal|saps|sasv|sbp|sc|select|set|shcm|si|sl|sleep|sls|sort|sp|spps|spsv|start|sv|swmi|tee|trcm|type|write)\b/i
        ],
        // per http://technet.microsoft.com/en-us/library/hh847744.aspx
        keyword:
          /\b(?:Begin|Break|Catch|Class|Continue|Data|Define|Do|DynamicParam|Else|ElseIf|End|Exit|Filter|Finally|For|ForEach|From|Function|If|InlineScript|Parallel|Param|Process|Return|Sequence|Switch|Throw|Trap|Try|Until|Using|Var|While|Workflow)\b/i,
        operator: {
          pattern:
            /(^|\W)(?:!|-(?:b?(?:and|x?or)|as|(?:Not)?(?:Contains|In|Like|Match)|eq|ge|gt|is(?:Not)?|Join|le|lt|ne|not|Replace|sh[lr])\b|-[-=]?|\+[+=]?|[*\/%]=?)/i,
          lookbehind: true
        },
        punctuation: /[|{}[\];(),.]/
      }); // Variable interpolation inside strings, and nested expressions
      powershell.string[0].inside = {
        function: {
          // Allow for one level of nesting
          pattern: /(^|[^`])\$\((?:\$\([^\r\n()]*\)|(?!\$\()[^\r\n)])*\)/,
          lookbehind: true,
          inside: powershell
        },
        boolean: powershell.boolean,
        variable: powershell.variable
      };
    })(Prism);
  }

  // @ts-nocheck
  processing.displayName = 'processing';
  processing.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function processing(Prism) {
    Prism.register(clike);
    Prism.languages.processing = Prism.languages.extend('clike', {
      keyword:
        /\b(?:break|case|catch|class|continue|default|else|extends|final|for|if|implements|import|new|null|private|public|return|static|super|switch|this|try|void|while)\b/,
      // Spaces are allowed between function name and parenthesis
      function: /\b\w+(?=\s*\()/,
      operator: /<[<=]?|>[>=]?|&&?|\|\|?|[%?]|[!=+\-*\/]=?/
    });
    Prism.languages.insertBefore('processing', 'number', {
      // Special case: XML is a type
      constant: /\b(?!XML\b)[A-Z][A-Z\d_]+\b/,
      type: {
        pattern: /\b(?:boolean|byte|char|color|double|float|int|[A-Z]\w*)\b/,
        alias: 'class-name'
      }
    });
  }

  // @ts-nocheck
  prolog.displayName = 'prolog';
  prolog.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function prolog(Prism) {
    Prism.languages.prolog = {
      // Syntax depends on the implementation
      comment: {
        pattern: /\/\*[\s\S]*?\*\/|%.*/,
        greedy: true
      },
      // Depending on the implementation, strings may allow escaped newlines and quote-escape
      string: {
        pattern: /(["'])(?:\1\1|\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1(?!\1)/,
        greedy: true
      },
      builtin: /\b(?:fx|fy|xf[xy]?|yfx?)\b/,
      // FIXME: Should we list all null-ary predicates (not followed by a parenthesis) like halt, trace, etc.?
      function: /\b[a-z]\w*(?:(?=\()|\/\d+)/,
      number: /\b\d+(?:\.\d*)?/,
      // Custom operators are allowed
      operator: /[:\\=><\-?*@\/;+^|!$.]+|\b(?:is|mod|not|xor)\b/,
      punctuation: /[(){}\[\],]/
    };
  }

  // @ts-nocheck
  promql.displayName = 'promql';
  promql.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function promql(Prism) {
  (function (Prism) {
      // PromQL Aggregation Operators
      // (https://prometheus.io/docs/prometheus/latest/querying/operators/#aggregation-operators)
      var aggregations = [
        'sum',
        'min',
        'max',
        'avg',
        'group',
        'stddev',
        'stdvar',
        'count',
        'count_values',
        'bottomk',
        'topk',
        'quantile'
      ]; // PromQL vector matching + the by and without clauses
      // (https://prometheus.io/docs/prometheus/latest/querying/operators/#vector-matching)
      var vectorMatching = [
        'on',
        'ignoring',
        'group_right',
        'group_left',
        'by',
        'without'
      ]; // PromQL offset modifier
      // (https://prometheus.io/docs/prometheus/latest/querying/basics/#offset-modifier)
      var offsetModifier = ['offset'];
      var keywords = aggregations.concat(vectorMatching, offsetModifier);
      Prism.languages.promql = {
        comment: {
          pattern: /(^[ \t]*)#.*/m,
          lookbehind: true
        },
        'vector-match': {
          // Match the comma-separated label lists inside vector matching:
          pattern: new RegExp(
            '((?:' + vectorMatching.join('|') + ')\\s*)\\([^)]*\\)'
          ),
          lookbehind: true,
          inside: {
            'label-key': {
              pattern: /\b[^,]+\b/,
              alias: 'attr-name'
            },
            punctuation: /[(),]/
          }
        },
        'context-labels': {
          pattern: /\{[^{}]*\}/,
          inside: {
            'label-key': {
              pattern: /\b[a-z_]\w*(?=\s*(?:=|![=~]))/,
              alias: 'attr-name'
            },
            'label-value': {
              pattern: /(["'`])(?:\\[\s\S]|(?!\1)[^\\])*\1/,
              greedy: true,
              alias: 'attr-value'
            },
            punctuation: /\{|\}|=~?|![=~]|,/
          }
        },
        'context-range': [
          {
            pattern: /\[[\w\s:]+\]/,
            // [1m]
            inside: {
              punctuation: /\[|\]|:/,
              'range-duration': {
                pattern: /\b(?:\d+(?:[smhdwy]|ms))+\b/i,
                alias: 'number'
              }
            }
          },
          {
            pattern: /(\boffset\s+)\w+/,
            // offset 1m
            lookbehind: true,
            inside: {
              'range-duration': {
                pattern: /\b(?:\d+(?:[smhdwy]|ms))+\b/i,
                alias: 'number'
              }
            }
          }
        ],
        keyword: new RegExp('\\b(?:' + keywords.join('|') + ')\\b', 'i'),
        function: /\b[a-z_]\w*(?=\s*\()/i,
        number:
          /[-+]?(?:(?:\b\d+(?:\.\d+)?|\B\.\d+)(?:e[-+]?\d+)?\b|\b(?:0x[0-9a-f]+|nan|inf)\b)/i,
        operator: /[\^*/%+-]|==|!=|<=|<|>=|>|\b(?:and|or|unless)\b/i,
        punctuation: /[{};()`,.[\]]/
      };
    })(Prism);
  }

  // @ts-nocheck
  properties.displayName = 'properties';
  properties.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function properties(Prism) {
    Prism.languages.properties = {
      comment: /^[ \t]*[#!].*$/m,
      'attr-value': {
        pattern:
          /(^[ \t]*(?:\\(?:\r\n|[\s\S])|[^\\\s:=])+(?: *[=:] *(?! )| ))(?:\\(?:\r\n|[\s\S])|[^\\\r\n])+/m,
        lookbehind: true
      },
      'attr-name': /^[ \t]*(?:\\(?:\r\n|[\s\S])|[^\\\s:=])+(?= *[=:]| )/m,
      punctuation: /[=:]/
    };
  }

  // @ts-nocheck
  protobuf.displayName = 'protobuf';
  protobuf.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function protobuf(Prism) {
    Prism.register(clike)
    ;(function (Prism) {
      var builtinTypes =
        /\b(?:bool|bytes|double|s?fixed(?:32|64)|float|[su]?int(?:32|64)|string)\b/;
      Prism.languages.protobuf = Prism.languages.extend('clike', {
        'class-name': [
          {
            pattern:
              /(\b(?:enum|extend|message|service)\s+)[A-Za-z_]\w*(?=\s*\{)/,
            lookbehind: true
          },
          {
            pattern:
              /(\b(?:rpc\s+\w+|returns)\s*\(\s*(?:stream\s+)?)\.?[A-Za-z_]\w*(?:\.[A-Za-z_]\w*)*(?=\s*\))/,
            lookbehind: true
          }
        ],
        keyword:
          /\b(?:enum|extend|extensions|import|message|oneof|option|optional|package|public|repeated|required|reserved|returns|rpc(?=\s+\w)|service|stream|syntax|to)\b(?!\s*=\s*\d)/,
        function: /\b[a-z_]\w*(?=\s*\()/i
      });
      Prism.languages.insertBefore('protobuf', 'operator', {
        map: {
          pattern: /\bmap<\s*[\w.]+\s*,\s*[\w.]+\s*>(?=\s+[a-z_]\w*\s*[=;])/i,
          alias: 'class-name',
          inside: {
            punctuation: /[<>.,]/,
            builtin: builtinTypes
          }
        },
        builtin: builtinTypes,
        'positional-class-name': {
          pattern: /(?:\b|\B\.)[a-z_]\w*(?:\.[a-z_]\w*)*(?=\s+[a-z_]\w*\s*[=;])/i,
          alias: 'class-name',
          inside: {
            punctuation: /\./
          }
        },
        annotation: {
          pattern: /(\[\s*)[a-z_]\w*(?=\s*=)/i,
          lookbehind: true
        }
      });
    })(Prism);
  }

  // @ts-nocheck
  stylus.displayName = 'stylus';
  stylus.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function stylus(Prism) {
  (function (Prism) {
      var unit = {
        pattern: /(\b\d+)(?:%|[a-z]+)/,
        lookbehind: true
      }; // 123 -123 .123 -.123 12.3 -12.3
      var number = {
        pattern: /(^|[^\w.-])-?(?:\d+(?:\.\d+)?|\.\d+)/,
        lookbehind: true
      };
      var inside = {
        comment: {
          pattern: /(^|[^\\])(?:\/\*[\s\S]*?\*\/|\/\/.*)/,
          lookbehind: true
        },
        url: {
          pattern: /\burl\((["']?).*?\1\)/i,
          greedy: true
        },
        string: {
          pattern: /("|')(?:(?!\1)[^\\\r\n]|\\(?:\r\n|[\s\S]))*\1/,
          greedy: true
        },
        interpolation: null,
        // See below
        func: null,
        // See below
        important: /\B!(?:important|optional)\b/i,
        keyword: {
          pattern: /(^|\s+)(?:(?:else|for|if|return|unless)(?=\s|$)|@[\w-]+)/,
          lookbehind: true
        },
        hexcode: /#[\da-f]{3,6}/i,
        color: [
          /\b(?:AliceBlue|AntiqueWhite|Aqua|Aquamarine|Azure|Beige|Bisque|Black|BlanchedAlmond|Blue|BlueViolet|Brown|BurlyWood|CadetBlue|Chartreuse|Chocolate|Coral|CornflowerBlue|Cornsilk|Crimson|Cyan|DarkBlue|DarkCyan|DarkGoldenRod|DarkGr[ae]y|DarkGreen|DarkKhaki|DarkMagenta|DarkOliveGreen|DarkOrange|DarkOrchid|DarkRed|DarkSalmon|DarkSeaGreen|DarkSlateBlue|DarkSlateGr[ae]y|DarkTurquoise|DarkViolet|DeepPink|DeepSkyBlue|DimGr[ae]y|DodgerBlue|FireBrick|FloralWhite|ForestGreen|Fuchsia|Gainsboro|GhostWhite|Gold|GoldenRod|Gr[ae]y|Green|GreenYellow|HoneyDew|HotPink|IndianRed|Indigo|Ivory|Khaki|Lavender|LavenderBlush|LawnGreen|LemonChiffon|LightBlue|LightCoral|LightCyan|LightGoldenRodYellow|LightGr[ae]y|LightGreen|LightPink|LightSalmon|LightSeaGreen|LightSkyBlue|LightSlateGr[ae]y|LightSteelBlue|LightYellow|Lime|LimeGreen|Linen|Magenta|Maroon|MediumAquaMarine|MediumBlue|MediumOrchid|MediumPurple|MediumSeaGreen|MediumSlateBlue|MediumSpringGreen|MediumTurquoise|MediumVioletRed|MidnightBlue|MintCream|MistyRose|Moccasin|NavajoWhite|Navy|OldLace|Olive|OliveDrab|Orange|OrangeRed|Orchid|PaleGoldenRod|PaleGreen|PaleTurquoise|PaleVioletRed|PapayaWhip|PeachPuff|Peru|Pink|Plum|PowderBlue|Purple|Red|RosyBrown|RoyalBlue|SaddleBrown|Salmon|SandyBrown|SeaGreen|SeaShell|Sienna|Silver|SkyBlue|SlateBlue|SlateGr[ae]y|Snow|SpringGreen|SteelBlue|Tan|Teal|Thistle|Tomato|Transparent|Turquoise|Violet|Wheat|White|WhiteSmoke|Yellow|YellowGreen)\b/i,
          {
            pattern:
              /\b(?:hsl|rgb)\(\s*\d{1,3}\s*,\s*\d{1,3}%?\s*,\s*\d{1,3}%?\s*\)\B|\b(?:hsl|rgb)a\(\s*\d{1,3}\s*,\s*\d{1,3}%?\s*,\s*\d{1,3}%?\s*,\s*(?:0|0?\.\d+|1)\s*\)\B/i,
            inside: {
              unit: unit,
              number: number,
              function: /[\w-]+(?=\()/,
              punctuation: /[(),]/
            }
          }
        ],
        entity: /\\[\da-f]{1,8}/i,
        unit: unit,
        boolean: /\b(?:false|true)\b/,
        operator: [
          // We want non-word chars around "-" because it is
          // accepted in property names.
          /~|[+!\/%<>?=]=?|[-:]=|\*[*=]?|\.{2,3}|&&|\|\||\B-\B|\b(?:and|in|is(?: a| defined| not|nt)?|not|or)\b/
        ],
        number: number,
        punctuation: /[{}()\[\];:,]/
      };
      inside['interpolation'] = {
        pattern: /\{[^\r\n}:]+\}/,
        alias: 'variable',
        inside: {
          delimiter: {
            pattern: /^\{|\}$/,
            alias: 'punctuation'
          },
          rest: inside
        }
      };
      inside['func'] = {
        pattern: /[\w-]+\([^)]*\).*/,
        inside: {
          function: /^[^(]+/,
          rest: inside
        }
      };
      Prism.languages.stylus = {
        'atrule-declaration': {
          pattern: /(^[ \t]*)@.+/m,
          lookbehind: true,
          inside: {
            atrule: /^@[\w-]+/,
            rest: inside
          }
        },
        'variable-declaration': {
          pattern: /(^[ \t]*)[\w$-]+\s*.?=[ \t]*(?:\{[^{}]*\}|\S.*|$)/m,
          lookbehind: true,
          inside: {
            variable: /^\S+/,
            rest: inside
          }
        },
        statement: {
          pattern: /(^[ \t]*)(?:else|for|if|return|unless)[ \t].+/m,
          lookbehind: true,
          inside: {
            keyword: /^\S+/,
            rest: inside
          }
        },
        // A property/value pair cannot end with a comma or a brace
        // It cannot have indented content unless it ended with a semicolon
        'property-declaration': {
          pattern:
            /((?:^|\{)([ \t]*))(?:[\w-]|\{[^}\r\n]+\})+(?:\s*:\s*|[ \t]+)(?!\s)[^{\r\n]*(?:;|[^{\r\n,]$(?!(?:\r?\n|\r)(?:\{|\2[ \t])))/m,
          lookbehind: true,
          inside: {
            property: {
              pattern: /^[^\s:]+/,
              inside: {
                interpolation: inside.interpolation
              }
            },
            rest: inside
          }
        },
        // A selector can contain parentheses only as part of a pseudo-element
        // It can span multiple lines.
        // It must end with a comma or an accolade or have indented content.
        selector: {
          pattern:
            /(^[ \t]*)(?:(?=\S)(?:[^{}\r\n:()]|::?[\w-]+(?:\([^)\r\n]*\)|(?![\w-]))|\{[^}\r\n]+\})+)(?:(?:\r?\n|\r)(?:\1(?:(?=\S)(?:[^{}\r\n:()]|::?[\w-]+(?:\([^)\r\n]*\)|(?![\w-]))|\{[^}\r\n]+\})+)))*(?:,$|\{|(?=(?:\r?\n|\r)(?:\{|\1[ \t])))/m,
          lookbehind: true,
          inside: {
            interpolation: inside.interpolation,
            comment: inside.comment,
            punctuation: /[{},]/
          }
        },
        func: inside.func,
        string: inside.string,
        comment: {
          pattern: /(^|[^\\])(?:\/\*[\s\S]*?\*\/|\/\/.*)/,
          lookbehind: true,
          greedy: true
        },
        interpolation: inside.interpolation,
        punctuation: /[{}()\[\];:.]/
      };
    })(Prism);
  }

  // @ts-nocheck
  twig.displayName = 'twig';
  twig.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function twig(Prism) {
    Prism.register(markupTemplating);
    Prism.languages.twig = {
      comment: /^\{#[\s\S]*?#\}$/,
      'tag-name': {
        pattern: /(^\{%-?\s*)\w+/,
        lookbehind: true,
        alias: 'keyword'
      },
      delimiter: {
        pattern: /^\{[{%]-?|-?[%}]\}$/,
        alias: 'punctuation'
      },
      string: {
        pattern: /("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,
        inside: {
          punctuation: /^['"]|['"]$/
        }
      },
      keyword: /\b(?:even|if|odd)\b/,
      boolean: /\b(?:false|null|true)\b/,
      number: /\b0x[\dA-Fa-f]+|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee][-+]?\d+)?/,
      operator: [
        {
          pattern:
            /(\s)(?:and|b-and|b-or|b-xor|ends with|in|is|matches|not|or|same as|starts with)(?=\s)/,
          lookbehind: true
        },
        /[=<>]=?|!=|\*\*?|\/\/?|\?:?|[-+~%|]/
      ],
      punctuation: /[()\[\]{}:.,]/
    };
    Prism.hooks.add('before-tokenize', function (env) {
      if (env.language !== 'twig') {
        return
      }
      var pattern = /\{(?:#[\s\S]*?#|%[\s\S]*?%|\{[\s\S]*?\})\}/g;
      Prism.languages['markup-templating'].buildPlaceholders(env, 'twig', pattern);
    });
    Prism.hooks.add('after-tokenize', function (env) {
      Prism.languages['markup-templating'].tokenizePlaceholders(env, 'twig');
    });
  }

  // @ts-nocheck
  pug.displayName = 'pug';
  pug.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function pug(Prism) {
    Prism.register(javascript);
    Prism.register(markup)
    ;(function (Prism) {
      // TODO:
      // - Add CSS highlighting inside <style> tags
      // - Add support for multi-line code blocks
      // - Add support for interpolation #{} and !{}
      // - Add support for tag interpolation #[]
      // - Add explicit support for plain text using |
      // - Add support for markup embedded in plain text
      Prism.languages.pug = {
        // Multiline stuff should appear before the rest
        // This handles both single-line and multi-line comments
        comment: {
          pattern: /(^([\t ]*))\/\/.*(?:(?:\r?\n|\r)\2[\t ].+)*/m,
          lookbehind: true
        },
        // All the tag-related part is in lookbehind
        // so that it can be highlighted by the "tag" pattern
        'multiline-script': {
          pattern:
            /(^([\t ]*)script\b.*\.[\t ]*)(?:(?:\r?\n|\r(?!\n))(?:\2[\t ].+|\s*?(?=\r?\n|\r)))+/m,
          lookbehind: true,
          inside: Prism.languages.javascript
        },
        // See at the end of the file for known filters
        filter: {
          pattern:
            /(^([\t ]*)):.+(?:(?:\r?\n|\r(?!\n))(?:\2[\t ].+|\s*?(?=\r?\n|\r)))+/m,
          lookbehind: true,
          inside: {
            'filter-name': {
              pattern: /^:[\w-]+/,
              alias: 'variable'
            },
            text: /\S[\s\S]*/
          }
        },
        'multiline-plain-text': {
          pattern:
            /(^([\t ]*)[\w\-#.]+\.[\t ]*)(?:(?:\r?\n|\r(?!\n))(?:\2[\t ].+|\s*?(?=\r?\n|\r)))+/m,
          lookbehind: true
        },
        markup: {
          pattern: /(^[\t ]*)<.+/m,
          lookbehind: true,
          inside: Prism.languages.markup
        },
        doctype: {
          pattern: /((?:^|\n)[\t ]*)doctype(?: .+)?/,
          lookbehind: true
        },
        // This handle all conditional and loop keywords
        'flow-control': {
          pattern:
            /(^[\t ]*)(?:case|default|each|else|if|unless|when|while)\b(?: .+)?/m,
          lookbehind: true,
          inside: {
            each: {
              pattern: /^each .+? in\b/,
              inside: {
                keyword: /\b(?:each|in)\b/,
                punctuation: /,/
              }
            },
            branch: {
              pattern: /^(?:case|default|else|if|unless|when|while)\b/,
              alias: 'keyword'
            },
            rest: Prism.languages.javascript
          }
        },
        keyword: {
          pattern: /(^[\t ]*)(?:append|block|extends|include|prepend)\b.+/m,
          lookbehind: true
        },
        mixin: [
          // Declaration
          {
            pattern: /(^[\t ]*)mixin .+/m,
            lookbehind: true,
            inside: {
              keyword: /^mixin/,
              function: /\w+(?=\s*\(|\s*$)/,
              punctuation: /[(),.]/
            }
          }, // Usage
          {
            pattern: /(^[\t ]*)\+.+/m,
            lookbehind: true,
            inside: {
              name: {
                pattern: /^\+\w+/,
                alias: 'function'
              },
              rest: Prism.languages.javascript
            }
          }
        ],
        script: {
          pattern: /(^[\t ]*script(?:(?:&[^(]+)?\([^)]+\))*[\t ]).+/m,
          lookbehind: true,
          inside: Prism.languages.javascript
        },
        'plain-text': {
          pattern:
            /(^[\t ]*(?!-)[\w\-#.]*[\w\-](?:(?:&[^(]+)?\([^)]+\))*\/?[\t ]).+/m,
          lookbehind: true
        },
        tag: {
          pattern: /(^[\t ]*)(?!-)[\w\-#.]*[\w\-](?:(?:&[^(]+)?\([^)]+\))*\/?:?/m,
          lookbehind: true,
          inside: {
            attributes: [
              {
                pattern: /&[^(]+\([^)]+\)/,
                inside: Prism.languages.javascript
              },
              {
                pattern: /\([^)]+\)/,
                inside: {
                  'attr-value': {
                    pattern: /(=\s*(?!\s))(?:\{[^}]*\}|[^,)\r\n]+)/,
                    lookbehind: true,
                    inside: Prism.languages.javascript
                  },
                  'attr-name': /[\w-]+(?=\s*!?=|\s*[,)])/,
                  punctuation: /[!=(),]+/
                }
              }
            ],
            punctuation: /:/,
            'attr-id': /#[\w\-]+/,
            'attr-class': /\.[\w\-]+/
          }
        },
        code: [
          {
            pattern: /(^[\t ]*(?:-|!?=)).+/m,
            lookbehind: true,
            inside: Prism.languages.javascript
          }
        ],
        punctuation: /[.\-!=|]+/
      };
      var filter_pattern =
        /(^([\t ]*)):<filter_name>(?:(?:\r?\n|\r(?!\n))(?:\2[\t ].+|\s*?(?=\r?\n|\r)))+/
          .source; // Non exhaustive list of available filters and associated languages
      var filters = [
        {
          filter: 'atpl',
          language: 'twig'
        },
        {
          filter: 'coffee',
          language: 'coffeescript'
        },
        'ejs',
        'handlebars',
        'less',
        'livescript',
        'markdown',
        {
          filter: 'sass',
          language: 'scss'
        },
        'stylus'
      ];
      var all_filters = {};
      for (var i = 0, l = filters.length; i < l; i++) {
        var filter = filters[i];
        filter =
          typeof filter === 'string'
            ? {
                filter: filter,
                language: filter
              }
            : filter;
        if (Prism.languages[filter.language]) {
          all_filters['filter-' + filter.filter] = {
            pattern: RegExp(
              filter_pattern.replace('<filter_name>', function () {
                return filter.filter
              }),
              'm'
            ),
            lookbehind: true,
            inside: {
              'filter-name': {
                pattern: /^:[\w-]+/,
                alias: 'variable'
              },
              text: {
                pattern: /\S[\s\S]*/,
                alias: [filter.language, 'language-' + filter.language],
                inside: Prism.languages[filter.language]
              }
            }
          };
        }
      }
      Prism.languages.insertBefore('pug', 'filter', all_filters);
    })(Prism);
  }

  // @ts-nocheck
  puppet.displayName = 'puppet';
  puppet.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function puppet(Prism) {
  (function (Prism) {
      Prism.languages.puppet = {
        heredoc: [
          // Matches the content of a quoted heredoc string (subject to interpolation)
          {
            pattern:
              /(@\("([^"\r\n\/):]+)"(?:\/[nrts$uL]*)?\).*(?:\r?\n|\r))(?:.*(?:\r?\n|\r(?!\n)))*?[ \t]*(?:\|[ \t]*)?(?:-[ \t]*)?\2/,
            lookbehind: true,
            alias: 'string',
            inside: {
              // Matches the end tag
              punctuation: /(?=\S).*\S(?= *$)/ // See interpolation below
            }
          }, // Matches the content of an unquoted heredoc string (no interpolation)
          {
            pattern:
              /(@\(([^"\r\n\/):]+)(?:\/[nrts$uL]*)?\).*(?:\r?\n|\r))(?:.*(?:\r?\n|\r(?!\n)))*?[ \t]*(?:\|[ \t]*)?(?:-[ \t]*)?\2/,
            lookbehind: true,
            greedy: true,
            alias: 'string',
            inside: {
              // Matches the end tag
              punctuation: /(?=\S).*\S(?= *$)/
            }
          }, // Matches the start tag of heredoc strings
          {
            pattern: /@\("?(?:[^"\r\n\/):]+)"?(?:\/[nrts$uL]*)?\)/,
            alias: 'string',
            inside: {
              punctuation: {
                pattern: /(\().+?(?=\))/,
                lookbehind: true
              }
            }
          }
        ],
        'multiline-comment': {
          pattern: /(^|[^\\])\/\*[\s\S]*?\*\//,
          lookbehind: true,
          greedy: true,
          alias: 'comment'
        },
        regex: {
          // Must be prefixed with the keyword "node" or a non-word char
          pattern:
            /((?:\bnode\s+|[~=\(\[\{,]\s*|[=+]>\s*|^\s*))\/(?:[^\/\\]|\\[\s\S])+\/(?:[imx]+\b|\B)/,
          lookbehind: true,
          greedy: true,
          inside: {
            // Extended regexes must have the x flag. They can contain single-line comments.
            'extended-regex': {
              pattern: /^\/(?:[^\/\\]|\\[\s\S])+\/[im]*x[im]*$/,
              inside: {
                comment: /#.*/
              }
            }
          }
        },
        comment: {
          pattern: /(^|[^\\])#.*/,
          lookbehind: true,
          greedy: true
        },
        string: {
          // Allow for one nested level of double quotes inside interpolation
          pattern:
            /(["'])(?:\$\{(?:[^'"}]|(["'])(?:(?!\2)[^\\]|\\[\s\S])*\2)+\}|\$(?!\{)|(?!\1)[^\\$]|\\[\s\S])*\1/,
          greedy: true,
          inside: {
            'double-quoted': {
              pattern: /^"[\s\S]*"$/,
              inside: {
                // See interpolation below
              }
            }
          }
        },
        variable: {
          pattern: /\$(?:::)?\w+(?:::\w+)*/,
          inside: {
            punctuation: /::/
          }
        },
        'attr-name': /(?:\b\w+|\*)(?=\s*=>)/,
        function: [
          {
            pattern: /(\.)(?!\d)\w+/,
            lookbehind: true
          },
          /\b(?:contain|debug|err|fail|include|info|notice|realize|require|tag|warning)\b|\b(?!\d)\w+(?=\()/
        ],
        number: /\b(?:0x[a-f\d]+|\d+(?:\.\d+)?(?:e-?\d+)?)\b/i,
        boolean: /\b(?:false|true)\b/,
        // Includes words reserved for future use
        keyword:
          /\b(?:application|attr|case|class|consumes|default|define|else|elsif|function|if|import|inherits|node|private|produces|type|undef|unless)\b/,
        datatype: {
          pattern:
            /\b(?:Any|Array|Boolean|Callable|Catalogentry|Class|Collection|Data|Default|Enum|Float|Hash|Integer|NotUndef|Numeric|Optional|Pattern|Regexp|Resource|Runtime|Scalar|String|Struct|Tuple|Type|Undef|Variant)\b/,
          alias: 'symbol'
        },
        operator:
          /=[=~>]?|![=~]?|<(?:<\|?|[=~|-])?|>[>=]?|->?|~>|\|>?>?|[*\/%+?]|\b(?:and|in|or)\b/,
        punctuation: /[\[\]{}().,;]|:+/
      };
      var interpolation = [
        {
          // Allow for one nested level of braces inside interpolation
          pattern:
            /(^|[^\\])\$\{(?:[^'"{}]|\{[^}]*\}|(["'])(?:(?!\2)[^\\]|\\[\s\S])*\2)+\}/,
          lookbehind: true,
          inside: {
            'short-variable': {
              // Negative look-ahead prevent wrong highlighting of functions
              pattern: /(^\$\{)(?!\w+\()(?:::)?\w+(?:::\w+)*/,
              lookbehind: true,
              alias: 'variable',
              inside: {
                punctuation: /::/
              }
            },
            delimiter: {
              pattern: /^\$/,
              alias: 'variable'
            },
            rest: Prism.languages.puppet
          }
        },
        {
          pattern: /(^|[^\\])\$(?:::)?\w+(?:::\w+)*/,
          lookbehind: true,
          alias: 'variable',
          inside: {
            punctuation: /::/
          }
        }
      ];
      Prism.languages.puppet['heredoc'][0].inside.interpolation = interpolation;
      Prism.languages.puppet['string'].inside[
        'double-quoted'
      ].inside.interpolation = interpolation;
    })(Prism);
  }

  // @ts-nocheck
  pure.displayName = 'pure';
  pure.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function pure(Prism) {
  (function (Prism) {
      // https://agraef.github.io/pure-docs/pure.html#lexical-matters
      Prism.languages.pure = {
        comment: [
          {
            pattern: /(^|[^\\])\/\*[\s\S]*?\*\//,
            lookbehind: true
          },
          {
            pattern: /(^|[^\\:])\/\/.*/,
            lookbehind: true
          },
          /#!.+/
        ],
        'inline-lang': {
          pattern: /%<[\s\S]+?%>/,
          greedy: true,
          inside: {
            lang: {
              pattern: /(^%< *)-\*-.+?-\*-/,
              lookbehind: true,
              alias: 'comment'
            },
            delimiter: {
              pattern: /^%<.*|%>$/,
              alias: 'punctuation'
            }
          }
        },
        string: {
          pattern: /"(?:\\.|[^"\\\r\n])*"/,
          greedy: true
        },
        number: {
          // The look-behind prevents wrong highlighting of the .. operator
          pattern:
            /((?:\.\.)?)(?:\b(?:inf|nan)\b|\b0x[\da-f]+|(?:\b(?:0b)?\d+(?:\.\d+)?|\B\.\d+)(?:e[+-]?\d+)?L?)/i,
          lookbehind: true
        },
        keyword:
          /\b(?:NULL|ans|break|bt|case|catch|cd|clear|const|def|del|dump|else|end|exit|extern|false|force|help|if|infix[lr]?|interface|let|ls|mem|namespace|nonfix|of|otherwise|outfix|override|postfix|prefix|private|public|pwd|quit|run|save|show|stats|then|throw|trace|true|type|underride|using|when|with)\b/,
        function:
          /\b(?:abs|add_(?:addr|constdef|(?:fundef|interface|macdef|typedef)(?:_at)?|vardef)|all|any|applp?|arity|bigintp?|blob(?:_crc|_size|p)?|boolp?|byte_c?string(?:_pointer)?|byte_(?:matrix|pointer)|calloc|cat|catmap|ceil|char[ps]?|check_ptrtag|chr|clear_sentry|clearsym|closurep?|cmatrixp?|cols?|colcat(?:map)?|colmap|colrev|colvector(?:p|seq)?|complex(?:_float_(?:matrix|pointer)|_matrix(?:_view)?|_pointer|p)?|conj|cookedp?|cst|cstring(?:_(?:dup|list|vector))?|curry3?|cyclen?|del_(?:constdef|fundef|interface|macdef|typedef|vardef)|delete|diag(?:mat)?|dim|dmatrixp?|do|double(?:_matrix(?:_view)?|_pointer|p)?|dowith3?|drop|dropwhile|eval(?:cmd)?|exactp|filter|fix|fixity|flip|float(?:_matrix|_pointer)|floor|fold[lr]1?|frac|free|funp?|functionp?|gcd|get(?:_(?:byte|constdef|double|float|fundef|int(?:64)?|interface(?:_typedef)?|long|macdef|pointer|ptrtag|sentry|short|string|typedef|vardef))?|globsym|hash|head|id|im|imatrixp?|index|inexactp|infp|init|insert|int(?:_matrix(?:_view)?|_pointer|p)?|int64_(?:matrix|pointer)|integerp?|iteraten?|iterwhile|join|keys?|lambdap?|last(?:err(?:pos)?)?|lcd|list[2p]?|listmap|make_ptrtag|malloc|map|matcat|matrixp?|max|member|min|nanp|nargs|nmatrixp?|null|numberp?|ord|pack(?:ed)?|pointer(?:_cast|_tag|_type|p)?|pow|pred|ptrtag|put(?:_(?:byte|double|float|int(?:64)?|long|pointer|short|string))?|rationalp?|re|realp?|realloc|recordp?|redim|reduce(?:_with)?|refp?|repeatn?|reverse|rlistp?|round|rows?|rowcat(?:map)?|rowmap|rowrev|rowvector(?:p|seq)?|same|scan[lr]1?|sentry|sgn|short_(?:matrix|pointer)|slice|smatrixp?|sort|split|str|strcat|stream|stride|string(?:_(?:dup|list|vector)|p)?|subdiag(?:mat)?|submat|subseq2?|substr|succ|supdiag(?:mat)?|symbolp?|tail|take|takewhile|thunkp?|transpose|trunc|tuplep?|typep|ubyte|uint(?:64)?|ulong|uncurry3?|unref|unzip3?|update|ushort|vals?|varp?|vector(?:p|seq)?|void|zip3?|zipwith3?)\b/,
        special: {
          pattern: /\b__[a-z]+__\b/i,
          alias: 'builtin'
        },
        // Any combination of operator chars can be an operator
        // eslint-disable-next-line no-misleading-character-class
        operator:
          /(?:[!"#$%&'*+,\-.\/:<=>?@\\^`|~\u00a1-\u00bf\u00d7-\u00f7\u20d0-\u2bff]|\b_+\b)+|\b(?:and|div|mod|not|or)\b/,
        // FIXME: How can we prevent | and , to be highlighted as operator when they are used alone?
        punctuation: /[(){}\[\];,|]/
      };
      var inlineLanguages = [
        'c',
        {
          lang: 'c++',
          alias: 'cpp'
        },
        'fortran'
      ];
      var inlineLanguageRe = /%< *-\*- *<lang>\d* *-\*-[\s\S]+?%>/.source;
      inlineLanguages.forEach(function (lang) {
        var alias = lang;
        if (typeof lang !== 'string') {
          alias = lang.alias;
          lang = lang.lang;
        }
        if (Prism.languages[alias]) {
          var o = {};
          o['inline-lang-' + alias] = {
            pattern: RegExp(
              inlineLanguageRe.replace(
                '<lang>',
                lang.replace(/([.+*?\/\\(){}\[\]])/g, '\\$1')
              ),
              'i'
            ),
            inside: Prism.util.clone(Prism.languages.pure['inline-lang'].inside)
          };
          o['inline-lang-' + alias].inside.rest = Prism.util.clone(
            Prism.languages[alias]
          );
          Prism.languages.insertBefore('pure', 'inline-lang', o);
        }
      }); // C is the default inline language
      if (Prism.languages.c) {
        Prism.languages.pure['inline-lang'].inside.rest = Prism.util.clone(
          Prism.languages.c
        );
      }
    })(Prism);
  }

  // @ts-nocheck
  purebasic.displayName = 'purebasic';
  purebasic.aliases = ['pbfasm'];

  /** @type {import('../core.js').Syntax} */
  function purebasic(Prism) {
    Prism.register(clike);
    /*
  Original Code by Bas Groothedde
  !!MANY THANKS!! I never would have made this, regex and me will never be best friends ;)
  ==> https://codepen.io/ImagineProgramming/details/JYydBy/
  slightly changed to pass all tests
  */
    // PureBasic support, steal stuff from ansi-c
    Prism.languages.purebasic = Prism.languages.extend('clike', {
      comment: /;.*/,
      keyword:
        /\b(?:align|and|as|break|calldebugger|case|compilercase|compilerdefault|compilerelse|compilerelseif|compilerendif|compilerendselect|compilererror|compilerif|compilerselect|continue|data|datasection|debug|debuglevel|declare|declarec|declarecdll|declaredll|declaremodule|default|define|dim|disableasm|disabledebugger|disableexplicit|else|elseif|enableasm|enabledebugger|enableexplicit|end|enddatasection|enddeclaremodule|endenumeration|endif|endimport|endinterface|endmacro|endmodule|endprocedure|endselect|endstructure|endstructureunion|endwith|enumeration|extends|fakereturn|for|foreach|forever|global|gosub|goto|if|import|importc|includebinary|includefile|includepath|interface|macro|module|newlist|newmap|next|not|or|procedure|procedurec|procedurecdll|proceduredll|procedurereturn|protected|prototype|prototypec|read|redim|repeat|restore|return|runtime|select|shared|static|step|structure|structureunion|swap|threaded|to|until|wend|while|with|xincludefile|xor)\b/i,
      function: /\b\w+(?:\.\w+)?\s*(?=\()/,
      number: /(?:\$[\da-f]+|\b-?(?:\d+(?:\.\d+)?|\.\d+)(?:e[+-]?\d+)?)\b/i,
      operator:
        /(?:@\*?|\?|\*)\w+|-[>-]?|\+\+?|!=?|<<?=?|>>?=?|==?|&&?|\|?\||[~^%?*/@]/
    });
    Prism.languages.insertBefore('purebasic', 'keyword', {
      tag: /#\w+\$?/,
      asm: {
        pattern: /(^[\t ]*)!.*/m,
        lookbehind: true,
        alias: 'tag',
        inside: {
          comment: /;.*/,
          string: {
            pattern: /(["'`])(?:\\.|(?!\1)[^\\\r\n])*\1/,
            greedy: true
          },
          // Anonymous label references, i.e.: jmp @b
          'label-reference-anonymous': {
            pattern: /(!\s*j[a-z]+\s+)@[fb]/i,
            lookbehind: true,
            alias: 'fasm-label'
          },
          // Named label reference, i.e.: jne label1
          'label-reference-addressed': {
            pattern: /(!\s*j[a-z]+\s+)[A-Z._?$@][\w.?$@~#]*/i,
            lookbehind: true,
            alias: 'fasm-label'
          },
          keyword: [
            /\b(?:extern|global)\b[^;\r\n]*/i,
            /\b(?:CPU|DEFAULT|FLOAT)\b.*/
          ],
          function: {
            pattern: /^([\t ]*!\s*)[\da-z]+(?=\s|$)/im,
            lookbehind: true
          },
          'function-inline': {
            pattern: /(:\s*)[\da-z]+(?=\s)/i,
            lookbehind: true,
            alias: 'function'
          },
          label: {
            pattern: /^([\t ]*!\s*)[A-Za-z._?$@][\w.?$@~#]*(?=:)/m,
            lookbehind: true,
            alias: 'fasm-label'
          },
          register:
            /\b(?:st\d|[xyz]mm\d\d?|[cdt]r\d|r\d\d?[bwd]?|[er]?[abcd]x|[abcd][hl]|[er]?(?:bp|di|si|sp)|[cdefgs]s|mm\d+)\b/i,
          number:
            /(?:\b|-|(?=\$))(?:0[hx](?:[\da-f]*\.)?[\da-f]+(?:p[+-]?\d+)?|\d[\da-f]+[hx]|\$\d[\da-f]*|0[oq][0-7]+|[0-7]+[oq]|0[by][01]+|[01]+[by]|0[dt]\d+|(?:\d+(?:\.\d+)?|\.\d+)(?:\.?e[+-]?\d+)?[dt]?)\b/i,
          operator: /[\[\]*+\-/%<>=&|$!,.:]/
        }
      }
    });
    delete Prism.languages.purebasic['class-name'];
    delete Prism.languages.purebasic['boolean'];
    Prism.languages.pbfasm = Prism.languages['purebasic'];
  }

  // @ts-nocheck
  purescript.displayName = 'purescript';
  purescript.aliases = ['purs'];

  /** @type {import('../core.js').Syntax} */
  function purescript(Prism) {
    Prism.register(haskell);
    Prism.languages.purescript = Prism.languages.extend('haskell', {
      keyword:
        /\b(?:ado|case|class|data|derive|do|else|forall|if|in|infixl|infixr|instance|let|module|newtype|of|primitive|then|type|where)\b|∀/,
      'import-statement': {
        // The imported or hidden names are not included in this import
        // statement. This is because we want to highlight those exactly like
        // we do for the names in the program.
        pattern:
          /(^[\t ]*)import\s+[A-Z][\w']*(?:\.[A-Z][\w']*)*(?:\s+as\s+[A-Z][\w']*(?:\.[A-Z][\w']*)*)?(?:\s+hiding\b)?/m,
        lookbehind: true,
        inside: {
          keyword: /\b(?:as|hiding|import)\b/,
          punctuation: /\./
        }
      },
      // These are builtin functions only. Constructors are highlighted later as a constant.
      builtin:
        /\b(?:absurd|add|ap|append|apply|between|bind|bottom|clamp|compare|comparing|compose|conj|const|degree|discard|disj|div|eq|flap|flip|gcd|identity|ifM|join|lcm|liftA1|liftM1|map|max|mempty|min|mod|mul|negate|not|notEq|one|otherwise|recip|show|sub|top|unit|unless|unlessM|void|when|whenM|zero)\b/,
      operator: [
        // Infix operators
        Prism.languages.haskell.operator[0], // ASCII operators
        Prism.languages.haskell.operator[2], // All UTF16 Unicode operator symbols
        // This regex is equivalent to /(?=[\x80-\uFFFF])[\p{gc=Math_Symbol}\p{gc=Currency_Symbol}\p{Modifier_Symbol}\p{Other_Symbol}]/u
        // See https://github.com/PrismJS/prism/issues/3006 for more details.
        /[\xa2-\xa6\xa8\xa9\xac\xae-\xb1\xb4\xb8\xd7\xf7\u02c2-\u02c5\u02d2-\u02df\u02e5-\u02eb\u02ed\u02ef-\u02ff\u0375\u0384\u0385\u03f6\u0482\u058d-\u058f\u0606-\u0608\u060b\u060e\u060f\u06de\u06e9\u06fd\u06fe\u07f6\u07fe\u07ff\u09f2\u09f3\u09fa\u09fb\u0af1\u0b70\u0bf3-\u0bfa\u0c7f\u0d4f\u0d79\u0e3f\u0f01-\u0f03\u0f13\u0f15-\u0f17\u0f1a-\u0f1f\u0f34\u0f36\u0f38\u0fbe-\u0fc5\u0fc7-\u0fcc\u0fce\u0fcf\u0fd5-\u0fd8\u109e\u109f\u1390-\u1399\u166d\u17db\u1940\u19de-\u19ff\u1b61-\u1b6a\u1b74-\u1b7c\u1fbd\u1fbf-\u1fc1\u1fcd-\u1fcf\u1fdd-\u1fdf\u1fed-\u1fef\u1ffd\u1ffe\u2044\u2052\u207a-\u207c\u208a-\u208c\u20a0-\u20bf\u2100\u2101\u2103-\u2106\u2108\u2109\u2114\u2116-\u2118\u211e-\u2123\u2125\u2127\u2129\u212e\u213a\u213b\u2140-\u2144\u214a-\u214d\u214f\u218a\u218b\u2190-\u2307\u230c-\u2328\u232b-\u2426\u2440-\u244a\u249c-\u24e9\u2500-\u2767\u2794-\u27c4\u27c7-\u27e5\u27f0-\u2982\u2999-\u29d7\u29dc-\u29fb\u29fe-\u2b73\u2b76-\u2b95\u2b97-\u2bff\u2ce5-\u2cea\u2e50\u2e51\u2e80-\u2e99\u2e9b-\u2ef3\u2f00-\u2fd5\u2ff0-\u2ffb\u3004\u3012\u3013\u3020\u3036\u3037\u303e\u303f\u309b\u309c\u3190\u3191\u3196-\u319f\u31c0-\u31e3\u3200-\u321e\u322a-\u3247\u3250\u3260-\u327f\u328a-\u32b0\u32c0-\u33ff\u4dc0-\u4dff\ua490-\ua4c6\ua700-\ua716\ua720\ua721\ua789\ua78a\ua828-\ua82b\ua836-\ua839\uaa77-\uaa79\uab5b\uab6a\uab6b\ufb29\ufbb2-\ufbc1\ufdfc\ufdfd\ufe62\ufe64-\ufe66\ufe69\uff04\uff0b\uff1c-\uff1e\uff3e\uff40\uff5c\uff5e\uffe0-\uffe6\uffe8-\uffee\ufffc\ufffd]/
      ]
    });
    Prism.languages.purs = Prism.languages.purescript;
  }

  // @ts-nocheck
  qsharp.displayName = 'qsharp';
  qsharp.aliases = ['qs'];

  /** @type {import('../core.js').Syntax} */
  function qsharp(Prism) {
    Prism.register(clike)
    ;(function (Prism) {
      /**
       * Replaces all placeholders "<<n>>" of given pattern with the n-th replacement (zero based).
       *
       * Note: This is a simple text based replacement. Be careful when using backreferences!
       *
       * @param {string} pattern the given pattern.
       * @param {string[]} replacements a list of replacement which can be inserted into the given pattern.
       * @returns {string} the pattern with all placeholders replaced with their corresponding replacements.
       * @example replace(/a<<0>>a/.source, [/b+/.source]) === /a(?:b+)a/.source
       */
      function replace(pattern, replacements) {
        return pattern.replace(/<<(\d+)>>/g, function (m, index) {
          return '(?:' + replacements[+index] + ')'
        })
      }
      /**
       * @param {string} pattern
       * @param {string[]} replacements
       * @param {string} [flags]
       * @returns {RegExp}
       */
      function re(pattern, replacements, flags) {
        return RegExp(replace(pattern, replacements), flags || '')
      }
      /**
       * Creates a nested pattern where all occurrences of the string `<<self>>` are replaced with the pattern itself.
       *
       * @param {string} pattern
       * @param {number} depthLog2
       * @returns {string}
       */
      function nested(pattern, depthLog2) {
        for (var i = 0; i < depthLog2; i++) {
          pattern = pattern.replace(/<<self>>/g, function () {
            return '(?:' + pattern + ')'
          });
        }
        return pattern.replace(/<<self>>/g, '[^\\s\\S]')
      } // https://docs.microsoft.com/en-us/azure/quantum/user-guide/language/typesystem/
      // https://github.com/microsoft/qsharp-language/tree/main/Specifications/Language/5_Grammar
      var keywordKinds = {
        // keywords which represent a return or variable type
        type: 'Adj BigInt Bool Ctl Double false Int One Pauli PauliI PauliX PauliY PauliZ Qubit Range Result String true Unit Zero',
        // all other keywords
        other:
          'Adjoint adjoint apply as auto body borrow borrowing Controlled controlled distribute elif else fail fixup for function if in internal intrinsic invert is let mutable namespace new newtype open operation repeat return self set until use using while within'
      }; // keywords
      function keywordsToPattern(words) {
        return '\\b(?:' + words.trim().replace(/ /g, '|') + ')\\b'
      }
      var keywords = RegExp(
        keywordsToPattern(keywordKinds.type + ' ' + keywordKinds.other)
      ); // types
      var identifier = /\b[A-Za-z_]\w*\b/.source;
      var qualifiedName = replace(/<<0>>(?:\s*\.\s*<<0>>)*/.source, [identifier]);
      var typeInside = {
        keyword: keywords,
        punctuation: /[<>()?,.:[\]]/
      }; // strings
      var regularString = /"(?:\\.|[^\\"])*"/.source;
      Prism.languages.qsharp = Prism.languages.extend('clike', {
        comment: /\/\/.*/,
        string: [
          {
            pattern: re(/(^|[^$\\])<<0>>/.source, [regularString]),
            lookbehind: true,
            greedy: true
          }
        ],
        'class-name': [
          {
            // open Microsoft.Quantum.Canon;
            // open Microsoft.Quantum.Canon as CN;
            pattern: re(/(\b(?:as|open)\s+)<<0>>(?=\s*(?:;|as\b))/.source, [
              qualifiedName
            ]),
            lookbehind: true,
            inside: typeInside
          },
          {
            // namespace Quantum.App1;
            pattern: re(/(\bnamespace\s+)<<0>>(?=\s*\{)/.source, [qualifiedName]),
            lookbehind: true,
            inside: typeInside
          }
        ],
        keyword: keywords,
        number:
          /(?:\b0(?:x[\da-f]+|b[01]+|o[0-7]+)|(?:\B\.\d+|\b\d+(?:\.\d*)?)(?:e[-+]?\d+)?)l?\b/i,
        operator:
          /\band=|\bor=|\band\b|\bnot\b|\bor\b|<[-=]|[-=]>|>>>=?|<<<=?|\^\^\^=?|\|\|\|=?|&&&=?|w\/=?|~~~|[*\/+\-^=!%]=?/,
        punctuation: /::|[{}[\];(),.:]/
      });
      Prism.languages.insertBefore('qsharp', 'number', {
        range: {
          pattern: /\.\./,
          alias: 'operator'
        }
      }); // single line
      var interpolationExpr = nested(
        replace(/\{(?:[^"{}]|<<0>>|<<self>>)*\}/.source, [regularString]),
        2
      );
      Prism.languages.insertBefore('qsharp', 'string', {
        'interpolation-string': {
          pattern: re(/\$"(?:\\.|<<0>>|[^\\"{])*"/.source, [interpolationExpr]),
          greedy: true,
          inside: {
            interpolation: {
              pattern: re(/((?:^|[^\\])(?:\\\\)*)<<0>>/.source, [
                interpolationExpr
              ]),
              lookbehind: true,
              inside: {
                punctuation: /^\{|\}$/,
                expression: {
                  pattern: /[\s\S]+/,
                  alias: 'language-qsharp',
                  inside: Prism.languages.qsharp
                }
              }
            },
            string: /[\s\S]+/
          }
        }
      });
    })(Prism);
    Prism.languages.qs = Prism.languages.qsharp;
  }

  // @ts-nocheck
  q.displayName = 'q';
  q.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function q(Prism) {
    Prism.languages.q = {
      string: /"(?:\\.|[^"\\\r\n])*"/,
      comment: [
        // From http://code.kx.com/wiki/Reference/Slash:
        // When / is following a space (or a right parenthesis, bracket, or brace), it is ignored with the rest of the line.
        {
          pattern: /([\t )\]}])\/.*/,
          lookbehind: true,
          greedy: true
        }, // From http://code.kx.com/wiki/Reference/Slash:
        // A line which has / as its first character and contains at least one other non-whitespace character is a whole-line comment and is ignored entirely.
        // A / on a line by itself begins a multiline comment which is terminated by the next \ on a line by itself.
        // If a / is not matched by a \, the multiline comment is unterminated and continues to end of file.
        // The / and \ must be the first char on the line, but may be followed by any amount of whitespace.
        {
          pattern:
            /(^|\r?\n|\r)\/[\t ]*(?:(?:\r?\n|\r)(?:.*(?:\r?\n|\r(?!\n)))*?(?:\\(?=[\t ]*(?:\r?\n|\r))|$)|\S.*)/,
          lookbehind: true,
          greedy: true
        }, // From http://code.kx.com/wiki/Reference/Slash:
        // A \ on a line by itself with no preceding matching / will comment to end of file.
        {
          pattern: /^\\[\t ]*(?:\r?\n|\r)[\s\S]+/m,
          greedy: true
        },
        {
          pattern: /^#!.+/m,
          greedy: true
        }
      ],
      symbol: /`(?::\S+|[\w.]*)/,
      datetime: {
        pattern:
          /0N[mdzuvt]|0W[dtz]|\d{4}\.\d\d(?:m|\.\d\d(?:T(?:\d\d(?::\d\d(?::\d\d(?:[.:]\d\d\d)?)?)?)?)?[dz]?)|\d\d:\d\d(?::\d\d(?:[.:]\d\d\d)?)?[uvt]?/,
        alias: 'number'
      },
      // The negative look-ahead prevents bad highlighting
      // of verbs 0: and 1:
      number:
        /\b(?![01]:)(?:0N[hje]?|0W[hj]?|0[wn]|0x[\da-fA-F]+|\d+(?:\.\d*)?(?:e[+-]?\d+)?[hjfeb]?)/,
      keyword:
        /\\\w+\b|\b(?:abs|acos|aj0?|all|and|any|asc|asin|asof|atan|attr|avgs?|binr?|by|ceiling|cols|cor|cos|count|cov|cross|csv|cut|delete|deltas|desc|dev|differ|distinct|div|do|dsave|ej|enlist|eval|except|exec|exit|exp|fby|fills|first|fkeys|flip|floor|from|get|getenv|group|gtime|hclose|hcount|hdel|hopen|hsym|iasc|identity|idesc|if|ij|in|insert|inter|inv|keys?|last|like|list|ljf?|load|log|lower|lsq|ltime|ltrim|mavg|maxs?|mcount|md5|mdev|med|meta|mins?|mmax|mmin|mmu|mod|msum|neg|next|not|null|or|over|parse|peach|pj|plist|prds?|prev|prior|rand|rank|ratios|raze|read0|read1|reciprocal|reval|reverse|rload|rotate|rsave|rtrim|save|scan|scov|sdev|select|set|setenv|show|signum|sin|sqrt|ssr?|string|sublist|sums?|sv|svar|system|tables|tan|til|trim|txf|type|uj|ungroup|union|update|upper|upsert|value|var|views?|vs|wavg|where|while|within|wj1?|wsum|ww|xasc|xbar|xcols?|xdesc|xexp|xgroup|xkey|xlog|xprev|xrank)\b/,
      adverb: {
        pattern: /['\/\\]:?|\beach\b/,
        alias: 'function'
      },
      verb: {
        pattern: /(?:\B\.\B|\b[01]:|<[=>]?|>=?|[:+\-*%,!?~=|$&#@^]):?|\b_\b:?/,
        alias: 'operator'
      },
      punctuation: /[(){}\[\];.]/
    };
  }

  // @ts-nocheck
  qml.displayName = 'qml';
  qml.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function qml(Prism) {
    Prism.register(javascript)
    ;(function (Prism) {
      var jsString = /"(?:\\.|[^\\"\r\n])*"|'(?:\\.|[^\\'\r\n])*'/.source;
      var jsComment = /\/\/.*(?!.)|\/\*(?:[^*]|\*(?!\/))*\*\//.source;
      var jsExpr =
        /(?:[^\\()[\]{}"'/]|<string>|\/(?![*/])|<comment>|\(<expr>*\)|\[<expr>*\]|\{<expr>*\}|\\[\s\S])/.source
          .replace(/<string>/g, function () {
            return jsString
          })
          .replace(/<comment>/g, function () {
            return jsComment
          }); // the pattern will blow up, so only a few iterations
      for (var i = 0; i < 2; i++) {
        jsExpr = jsExpr.replace(/<expr>/g, function () {
          return jsExpr
        });
      }
      jsExpr = jsExpr.replace(/<expr>/g, '[^\\s\\S]');
      Prism.languages.qml = {
        comment: {
          pattern: /\/\/.*|\/\*[\s\S]*?\*\//,
          greedy: true
        },
        'javascript-function': {
          pattern: RegExp(
            /((?:^|;)[ \t]*)function\s+(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*\s*\(<js>*\)\s*\{<js>*\}/.source.replace(
              /<js>/g,
              function () {
                return jsExpr
              }
            ),
            'm'
          ),
          lookbehind: true,
          greedy: true,
          alias: 'language-javascript',
          inside: Prism.languages.javascript
        },
        'class-name': {
          pattern: /((?:^|[:;])[ \t]*)(?!\d)\w+(?=[ \t]*\{|[ \t]+on\b)/m,
          lookbehind: true
        },
        property: [
          {
            pattern: /((?:^|[;{])[ \t]*)(?!\d)\w+(?:\.\w+)*(?=[ \t]*:)/m,
            lookbehind: true
          },
          {
            pattern:
              /((?:^|[;{])[ \t]*)property[ \t]+(?!\d)\w+(?:\.\w+)*[ \t]+(?!\d)\w+(?:\.\w+)*(?=[ \t]*:)/m,
            lookbehind: true,
            inside: {
              keyword: /^property/,
              property: /\w+(?:\.\w+)*/
            }
          }
        ],
        'javascript-expression': {
          pattern: RegExp(
            /(:[ \t]*)(?![\s;}[])(?:(?!$|[;}])<js>)+/.source.replace(
              /<js>/g,
              function () {
                return jsExpr
              }
            ),
            'm'
          ),
          lookbehind: true,
          greedy: true,
          alias: 'language-javascript',
          inside: Prism.languages.javascript
        },
        string: {
          pattern: /"(?:\\.|[^\\"\r\n])*"/,
          greedy: true
        },
        keyword: /\b(?:as|import|on)\b/,
        punctuation: /[{}[\]:;,]/
      };
    })(Prism);
  }

  // @ts-nocheck
  qore.displayName = 'qore';
  qore.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function qore(Prism) {
    Prism.register(clike);
    Prism.languages.qore = Prism.languages.extend('clike', {
      comment: {
        pattern: /(^|[^\\])(?:\/\*[\s\S]*?\*\/|(?:\/\/|#).*)/,
        lookbehind: true
      },
      // Overridden to allow unescaped multi-line strings
      string: {
        pattern: /("|')(?:\\[\s\S]|(?!\1)[^\\])*\1/,
        greedy: true
      },
      keyword:
        /\b(?:abstract|any|assert|binary|bool|boolean|break|byte|case|catch|char|class|code|const|continue|data|default|do|double|else|enum|extends|final|finally|float|for|goto|hash|if|implements|import|inherits|instanceof|int|interface|long|my|native|new|nothing|null|object|our|own|private|reference|rethrow|return|short|soft(?:bool|date|float|int|list|number|string)|static|strictfp|string|sub|super|switch|synchronized|this|throw|throws|transient|try|void|volatile|while)\b/,
      boolean: /\b(?:false|true)\b/i,
      function: /\$?\b(?!\d)\w+(?=\()/,
      number:
        /\b(?:0b[01]+|0x(?:[\da-f]*\.)?[\da-fp\-]+|(?:\d+(?:\.\d+)?|\.\d+)(?:e\d+)?[df]|(?:\d+(?:\.\d+)?|\.\d+))\b/i,
      operator: {
        pattern:
          /(^|[^.])(?:\+[+=]?|-[-=]?|[!=](?:==?|~)?|>>?=?|<(?:=>?|<=?)?|&[&=]?|\|[|=]?|[*\/%^]=?|[~?])/,
        lookbehind: true
      },
      variable: /\$(?!\d)\w+\b/
    });
  }

  // @ts-nocheck
  racket.displayName = 'racket';
  racket.aliases = ['rkt'];

  /** @type {import('../core.js').Syntax} */
  function racket(Prism) {
    Prism.register(scheme);
    Prism.languages.racket = Prism.languages.extend('scheme', {
      'lambda-parameter': {
        // the racket lambda syntax is a lot more complex, so we won't even attempt to capture it.
        // this will just prevent false positives of the `function` pattern
        pattern: /([(\[]lambda\s+[(\[])[^()\[\]'\s]+/,
        lookbehind: true
      }
    });
    Prism.languages.insertBefore('racket', 'string', {
      lang: {
        pattern: /^#lang.+/m,
        greedy: true,
        alias: 'keyword'
      }
    });
    Prism.languages.rkt = Prism.languages.racket;
  }

  // @ts-nocheck
  cshtml.displayName = 'cshtml';
  cshtml.aliases = ['razor'];

  /** @type {import('../core.js').Syntax} */
  function cshtml(Prism) {
    Prism.register(csharp);
    Prism.register(markup)
    // Docs:
    // https://docs.microsoft.com/en-us/aspnet/core/razor-pages/?view=aspnetcore-5.0&tabs=visual-studio
    // https://docs.microsoft.com/en-us/aspnet/core/mvc/views/razor?view=aspnetcore-5.0
    ;(function (Prism) {
      var commentLike = /\/(?![/*])|\/\/.*[\r\n]|\/\*[^*]*(?:\*(?!\/)[^*]*)*\*\//
        .source;
      var stringLike =
        /@(?!")|"(?:[^\r\n\\"]|\\.)*"|@"(?:[^\\"]|""|\\[\s\S])*"(?!")/.source +
        '|' +
        /'(?:(?:[^\r\n'\\]|\\.|\\[Uux][\da-fA-F]{1,8})'|(?=[^\\](?!')))/.source;
      /**
       * Creates a nested pattern where all occurrences of the string `<<self>>` are replaced with the pattern itself.
       *
       * @param {string} pattern
       * @param {number} depthLog2
       * @returns {string}
       */
      function nested(pattern, depthLog2) {
        for (var i = 0; i < depthLog2; i++) {
          pattern = pattern.replace(/<self>/g, function () {
            return '(?:' + pattern + ')'
          });
        }
        return pattern
          .replace(/<self>/g, '[^\\s\\S]')
          .replace(/<str>/g, '(?:' + stringLike + ')')
          .replace(/<comment>/g, '(?:' + commentLike + ')')
      }
      var round = nested(/\((?:[^()'"@/]|<str>|<comment>|<self>)*\)/.source, 2);
      var square = nested(/\[(?:[^\[\]'"@/]|<str>|<comment>|<self>)*\]/.source, 2);
      var curly = nested(/\{(?:[^{}'"@/]|<str>|<comment>|<self>)*\}/.source, 2);
      var angle = nested(/<(?:[^<>'"@/]|<str>|<comment>|<self>)*>/.source, 2); // Note about the above bracket patterns:
      // They all ignore HTML expressions that might be in the C# code. This is a problem because HTML (like strings and
      // comments) is parsed differently. This is a huge problem because HTML might contain brackets and quotes which
      // messes up the bracket and string counting implemented by the above patterns.
      //
      // This problem is not fixable because 1) HTML expression are highly context sensitive and very difficult to detect
      // and 2) they require one capturing group at every nested level. See the `tagRegion` pattern to admire the
      // complexity of an HTML expression.
      //
      // To somewhat alleviate the problem a bit, the patterns for characters (e.g. 'a') is very permissive, it also
      // allows invalid characters to support HTML expressions like this: <p>That's it!</p>.
      var tagAttrs =
        /(?:\s(?:\s*[^\s>\/=]+(?:\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+(?=[\s>]))|(?=[\s/>])))+)?/
          .source;
      var tagContent = /(?!\d)[^\s>\/=$<%]+/.source + tagAttrs + /\s*\/?>/.source;
      var tagRegion =
        /\B@?/.source +
        '(?:' +
        /<([a-zA-Z][\w:]*)/.source +
        tagAttrs +
        /\s*>/.source +
        '(?:' +
        (/[^<]/.source +
          '|' + // all tags that are not the start tag
          // eslint-disable-next-line regexp/strict
          /<\/?(?!\1\b)/.source +
          tagContent +
          '|' + // nested start tag
          nested(
            // eslint-disable-next-line regexp/strict
            /<\1/.source +
              tagAttrs +
              /\s*>/.source +
              '(?:' +
              (/[^<]/.source +
                '|' + // all tags that are not the start tag
                // eslint-disable-next-line regexp/strict
                /<\/?(?!\1\b)/.source +
                tagContent +
                '|' +
                '<self>') +
              ')*' + // eslint-disable-next-line regexp/strict
              /<\/\1\s*>/.source,
            2
          )) +
        ')*' + // eslint-disable-next-line regexp/strict
        /<\/\1\s*>/.source +
        '|' +
        /</.source +
        tagContent +
        ')'; // Now for the actual language definition(s):
      //
      // Razor as a language has 2 parts:
      //  1) CSHTML: A markup-like language that has been extended with inline C# code expressions and blocks.
      //  2) C#+HTML: A variant of C# that can contain CSHTML tags as expressions.
      //
      // In the below code, both CSHTML and C#+HTML will be create as separate language definitions that reference each
      // other. However, only CSHTML will be exported via `Prism.languages`.
      Prism.languages.cshtml = Prism.languages.extend('markup', {});
      var csharpWithHtml = Prism.languages.insertBefore(
        'csharp',
        'string',
        {
          html: {
            pattern: RegExp(tagRegion),
            greedy: true,
            inside: Prism.languages.cshtml
          }
        },
        {
          csharp: Prism.languages.extend('csharp', {})
        }
      );
      var cs = {
        pattern: /\S[\s\S]*/,
        alias: 'language-csharp',
        inside: csharpWithHtml
      };
      Prism.languages.insertBefore('cshtml', 'prolog', {
        'razor-comment': {
          pattern: /@\*[\s\S]*?\*@/,
          greedy: true,
          alias: 'comment'
        },
        block: {
          pattern: RegExp(
            /(^|[^@])@/.source +
              '(?:' +
              [
                // @{ ... }
                curly, // @code{ ... }
                /(?:code|functions)\s*/.source + curly, // @for (...) { ... }
                /(?:for|foreach|lock|switch|using|while)\s*/.source +
                  round +
                  /\s*/.source +
                  curly, // @do { ... } while (...);
                /do\s*/.source +
                  curly +
                  /\s*while\s*/.source +
                  round +
                  /(?:\s*;)?/.source, // @try { ... } catch (...) { ... } finally { ... }
                /try\s*/.source +
                  curly +
                  /\s*catch\s*/.source +
                  round +
                  /\s*/.source +
                  curly +
                  /\s*finally\s*/.source +
                  curly, // @if (...) {...} else if (...) {...} else {...}
                /if\s*/.source +
                  round +
                  /\s*/.source +
                  curly +
                  '(?:' +
                  /\s*else/.source +
                  '(?:' +
                  /\s+if\s*/.source +
                  round +
                  ')?' +
                  /\s*/.source +
                  curly +
                  ')*'
              ].join('|') +
              ')'
          ),
          lookbehind: true,
          greedy: true,
          inside: {
            keyword: /^@\w*/,
            csharp: cs
          }
        },
        directive: {
          pattern:
            /^([ \t]*)@(?:addTagHelper|attribute|implements|inherits|inject|layout|model|namespace|page|preservewhitespace|removeTagHelper|section|tagHelperPrefix|using)(?=\s).*/m,
          lookbehind: true,
          greedy: true,
          inside: {
            keyword: /^@\w+/,
            csharp: cs
          }
        },
        value: {
          pattern: RegExp(
            /(^|[^@])@/.source +
              /(?:await\b\s*)?/.source +
              '(?:' +
              /\w+\b/.source +
              '|' +
              round +
              ')' +
              '(?:' +
              /[?!]?\.\w+\b/.source +
              '|' +
              round +
              '|' +
              square +
              '|' +
              angle +
              round +
              ')*'
          ),
          lookbehind: true,
          greedy: true,
          alias: 'variable',
          inside: {
            keyword: /^@/,
            csharp: cs
          }
        },
        'delegate-operator': {
          pattern: /(^|[^@])@(?=<)/,
          lookbehind: true,
          alias: 'operator'
        }
      });
      Prism.languages.razor = Prism.languages.cshtml;
    })(Prism);
  }

  // @ts-nocheck
  jsx.displayName = 'jsx';
  jsx.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function jsx(Prism) {
    Prism.register(javascript);
    Prism.register(markup)
    ;(function (Prism) {
      var javascript = Prism.util.clone(Prism.languages.javascript);
      var space = /(?:\s|\/\/.*(?!.)|\/\*(?:[^*]|\*(?!\/))\*\/)/.source;
      var braces = /(?:\{(?:\{(?:\{[^{}]*\}|[^{}])*\}|[^{}])*\})/.source;
      var spread = /(?:\{<S>*\.{3}(?:[^{}]|<BRACES>)*\})/.source;
      /**
       * @param {string} source
       * @param {string} [flags]
       */
      function re(source, flags) {
        source = source
          .replace(/<S>/g, function () {
            return space
          })
          .replace(/<BRACES>/g, function () {
            return braces
          })
          .replace(/<SPREAD>/g, function () {
            return spread
          });
        return RegExp(source, flags)
      }
      spread = re(spread).source;
      Prism.languages.jsx = Prism.languages.extend('markup', javascript);
      Prism.languages.jsx.tag.pattern = re(
        /<\/?(?:[\w.:-]+(?:<S>+(?:[\w.:$-]+(?:=(?:"(?:\\[\s\S]|[^\\"])*"|'(?:\\[\s\S]|[^\\'])*'|[^\s{'"/>=]+|<BRACES>))?|<SPREAD>))*<S>*\/?)?>/
          .source
      );
      Prism.languages.jsx.tag.inside['tag'].pattern = /^<\/?[^\s>\/]*/;
      Prism.languages.jsx.tag.inside['attr-value'].pattern =
        /=(?!\{)(?:"(?:\\[\s\S]|[^\\"])*"|'(?:\\[\s\S]|[^\\'])*'|[^\s'">]+)/;
      Prism.languages.jsx.tag.inside['tag'].inside['class-name'] =
        /^[A-Z]\w*(?:\.[A-Z]\w*)*$/;
      Prism.languages.jsx.tag.inside['comment'] = javascript['comment'];
      Prism.languages.insertBefore(
        'inside',
        'attr-name',
        {
          spread: {
            pattern: re(/<SPREAD>/.source),
            inside: Prism.languages.jsx
          }
        },
        Prism.languages.jsx.tag
      );
      Prism.languages.insertBefore(
        'inside',
        'special-attr',
        {
          script: {
            // Allow for two levels of nesting
            pattern: re(/=<BRACES>/.source),
            alias: 'language-javascript',
            inside: {
              'script-punctuation': {
                pattern: /^=(?=\{)/,
                alias: 'punctuation'
              },
              rest: Prism.languages.jsx
            }
          }
        },
        Prism.languages.jsx.tag
      ); // The following will handle plain text inside tags
      var stringifyToken = function (token) {
        if (!token) {
          return ''
        }
        if (typeof token === 'string') {
          return token
        }
        if (typeof token.content === 'string') {
          return token.content
        }
        return token.content.map(stringifyToken).join('')
      };
      var walkTokens = function (tokens) {
        var openedTags = [];
        for (var i = 0; i < tokens.length; i++) {
          var token = tokens[i];
          var notTagNorBrace = false;
          if (typeof token !== 'string') {
            if (
              token.type === 'tag' &&
              token.content[0] &&
              token.content[0].type === 'tag'
            ) {
              // We found a tag, now find its kind
              if (token.content[0].content[0].content === '</') {
                // Closing tag
                if (
                  openedTags.length > 0 &&
                  openedTags[openedTags.length - 1].tagName ===
                    stringifyToken(token.content[0].content[1])
                ) {
                  // Pop matching opening tag
                  openedTags.pop();
                }
              } else {
                if (token.content[token.content.length - 1].content === '/>') ; else {
                  // Opening tag
                  openedTags.push({
                    tagName: stringifyToken(token.content[0].content[1]),
                    openedBraces: 0
                  });
                }
              }
            } else if (
              openedTags.length > 0 &&
              token.type === 'punctuation' &&
              token.content === '{'
            ) {
              // Here we might have entered a JSX context inside a tag
              openedTags[openedTags.length - 1].openedBraces++;
            } else if (
              openedTags.length > 0 &&
              openedTags[openedTags.length - 1].openedBraces > 0 &&
              token.type === 'punctuation' &&
              token.content === '}'
            ) {
              // Here we might have left a JSX context inside a tag
              openedTags[openedTags.length - 1].openedBraces--;
            } else {
              notTagNorBrace = true;
            }
          }
          if (notTagNorBrace || typeof token === 'string') {
            if (
              openedTags.length > 0 &&
              openedTags[openedTags.length - 1].openedBraces === 0
            ) {
              // Here we are inside a tag, and not inside a JSX context.
              // That's plain text: drop any tokens matched.
              var plainText = stringifyToken(token); // And merge text with adjacent text
              if (
                i < tokens.length - 1 &&
                (typeof tokens[i + 1] === 'string' ||
                  tokens[i + 1].type === 'plain-text')
              ) {
                plainText += stringifyToken(tokens[i + 1]);
                tokens.splice(i + 1, 1);
              }
              if (
                i > 0 &&
                (typeof tokens[i - 1] === 'string' ||
                  tokens[i - 1].type === 'plain-text')
              ) {
                plainText = stringifyToken(tokens[i - 1]) + plainText;
                tokens.splice(i - 1, 1);
                i--;
              }
              tokens[i] = new Prism.Token(
                'plain-text',
                plainText,
                null,
                plainText
              );
            }
          }
          if (token.content && typeof token.content !== 'string') {
            walkTokens(token.content);
          }
        }
      };
      Prism.hooks.add('after-tokenize', function (env) {
        if (env.language !== 'jsx' && env.language !== 'tsx') {
          return
        }
        walkTokens(env.tokens);
      });
    })(Prism);
  }

  // @ts-nocheck
  tsx.displayName = 'tsx';
  tsx.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function tsx(Prism) {
    Prism.register(jsx);
    Prism.register(typescript)
    ;(function (Prism) {
      var typescript = Prism.util.clone(Prism.languages.typescript);
      Prism.languages.tsx = Prism.languages.extend('jsx', typescript); // doesn't work with TS because TS is too complex
      delete Prism.languages.tsx['parameter'];
      delete Prism.languages.tsx['literal-property']; // This will prevent collisions between TSX tags and TS generic types.
      // Idea by https://github.com/karlhorky
      // Discussion: https://github.com/PrismJS/prism/issues/2594#issuecomment-710666928
      var tag = Prism.languages.tsx.tag;
      tag.pattern = RegExp(
        /(^|[^\w$]|(?=<\/))/.source + '(?:' + tag.pattern.source + ')',
        tag.pattern.flags
      );
      tag.lookbehind = true;
    })(Prism);
  }

  // @ts-nocheck
  reason.displayName = 'reason';
  reason.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function reason(Prism) {
    Prism.register(clike);
    Prism.languages.reason = Prism.languages.extend('clike', {
      string: {
        pattern: /"(?:\\(?:\r\n|[\s\S])|[^\\\r\n"])*"/,
        greedy: true
      },
      // 'class-name' must be matched *after* 'constructor' defined below
      'class-name': /\b[A-Z]\w*/,
      keyword:
        /\b(?:and|as|assert|begin|class|constraint|do|done|downto|else|end|exception|external|for|fun|function|functor|if|in|include|inherit|initializer|lazy|let|method|module|mutable|new|nonrec|object|of|open|or|private|rec|sig|struct|switch|then|to|try|type|val|virtual|when|while|with)\b/,
      operator:
        /\.{3}|:[:=]|\|>|->|=(?:==?|>)?|<=?|>=?|[|^?'#!~`]|[+\-*\/]\.?|\b(?:asr|land|lor|lsl|lsr|lxor|mod)\b/
    });
    Prism.languages.insertBefore('reason', 'class-name', {
      char: {
        pattern: /'(?:\\x[\da-f]{2}|\\o[0-3][0-7][0-7]|\\\d{3}|\\.|[^'\\\r\n])'/,
        greedy: true
      },
      // Negative look-ahead prevents from matching things like String.capitalize
      constructor: /\b[A-Z]\w*\b(?!\s*\.)/,
      label: {
        pattern: /\b[a-z]\w*(?=::)/,
        alias: 'symbol'
      }
    }); // We can't match functions property, so let's not even try.
    delete Prism.languages.reason.function;
  }

  // @ts-nocheck
  rego.displayName = 'rego';
  rego.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function rego(Prism) {
    // https://www.openpolicyagent.org/docs/latest/policy-reference/
    Prism.languages.rego = {
      comment: /#.*/,
      property: {
        pattern:
          /(^|[^\\.])(?:"(?:\\.|[^\\"\r\n])*"|`[^`]*`|\b[a-z_]\w*\b)(?=\s*:(?!=))/i,
        lookbehind: true,
        greedy: true
      },
      string: {
        pattern: /(^|[^\\])"(?:\\.|[^\\"\r\n])*"|`[^`]*`/,
        lookbehind: true,
        greedy: true
      },
      keyword:
        /\b(?:as|default|else|import|not|null|package|set(?=\s*\()|some|with)\b/,
      boolean: /\b(?:false|true)\b/,
      function: {
        pattern: /\b[a-z_]\w*\b(?:\s*\.\s*\b[a-z_]\w*\b)*(?=\s*\()/i,
        inside: {
          namespace: /\b\w+\b(?=\s*\.)/,
          punctuation: /\./
        }
      },
      number: /-?\b\d+(?:\.\d+)?(?:e[+-]?\d+)?\b/i,
      operator: /[-+*/%|&]|[<>:=]=?|!=|\b_\b/,
      punctuation: /[,;.\[\]{}()]/
    };
  }

  // @ts-nocheck
  renpy.displayName = 'renpy';
  renpy.aliases = ['rpy'];

  /** @type {import('../core.js').Syntax} */
  function renpy(Prism) {
    Prism.languages.renpy = {
      comment: {
        pattern: /(^|[^\\])#.+/,
        lookbehind: true
      },
      string: {
        pattern:
          /("""|''')[\s\S]+?\1|("|')(?:\\.|(?!\2)[^\\])*\2|(?:^#?(?:(?:[0-9a-fA-F]){3}|[0-9a-fA-F]{6})$)/m,
        greedy: true
      },
      function: /\b[a-z_]\w*(?=\()/i,
      property:
        /\b(?:Update|UpdateVersion|action|activate_sound|adv_nvl_transition|after_load_transition|align|alpha|alt|anchor|antialias|area|auto|background|bar_invert|bar_resizing|bar_vertical|black_color|bold|bottom_bar|bottom_gutter|bottom_margin|bottom_padding|box_reverse|box_wrap|can_update|caret|child|color|crop|default_afm_enable|default_afm_time|default_fullscreen|default_text_cps|developer|directory_name|drag_handle|drag_joined|drag_name|drag_raise|draggable|dragged|drop_shadow|drop_shadow_color|droppable|dropped|easein|easeout|edgescroll|end_game_transition|end_splash_transition|enter_replay_transition|enter_sound|enter_transition|enter_yesno_transition|executable_name|exit_replay_transition|exit_sound|exit_transition|exit_yesno_transition|fadein|fadeout|first_indent|first_spacing|fit_first|focus|focus_mask|font|foreground|game_main_transition|get_installed_packages|google_play_key|google_play_salt|ground|has_music|has_sound|has_voice|height|help|hinting|hover|hover_background|hover_color|hover_sound|hovered|hyperlink_functions|idle|idle_color|image_style|include_update|insensitive|insensitive_background|insensitive_color|inside|intra_transition|italic|justify|kerning|keyboard_focus|language|layer_clipping|layers|layout|left_bar|left_gutter|left_margin|left_padding|length|line_leading|line_overlap_split|line_spacing|linear|main_game_transition|main_menu_music|maximum|min_width|minimum|minwidth|modal|mouse|mousewheel|name|narrator_menu|newline_indent|nvl_adv_transition|offset|order_reverse|outlines|overlay_functions|pos|position|prefix|radius|range|rest_indent|right_bar|right_gutter|right_margin|right_padding|rotate|rotate_pad|ruby_style|sample_sound|save_directory|say_attribute_transition|screen_height|screen_width|scrollbars|selected_hover|selected_hover_color|selected_idle|selected_idle_color|selected_insensitive|show_side_image|show_two_window|side_spacing|side_xpos|side_ypos|size|size_group|slow_cps|slow_cps_multiplier|spacing|strikethrough|subpixel|text_align|text_style|text_xpos|text_y_fudge|text_ypos|thumb|thumb_offset|thumb_shadow|thumbnail_height|thumbnail_width|time|top_bar|top_gutter|top_margin|top_padding|translations|underline|unscrollable|update|value|version|version_name|version_tuple|vertical|width|window_hide_transition|window_icon|window_left_padding|window_show_transition|window_title|windows_icon|xadjustment|xalign|xanchor|xanchoraround|xaround|xcenter|xfill|xinitial|xmargin|xmaximum|xminimum|xoffset|xofsset|xpadding|xpos|xsize|xzoom|yadjustment|yalign|yanchor|yanchoraround|yaround|ycenter|yfill|yinitial|ymargin|ymaximum|yminimum|yoffset|ypadding|ypos|ysize|ysizexysize|yzoom|zoom|zorder)\b/,
      tag: /\b(?:bar|block|button|buttoscreenn|drag|draggroup|fixed|frame|grid|[hv]box|hotbar|hotspot|image|imagebutton|imagemap|input|key|label|menu|mm_menu_frame|mousearea|nvl|parallel|screen|self|side|tag|text|textbutton|timer|vbar|viewport|window)\b|\$/,
      keyword:
        /\b(?:None|add|adjustment|alignaround|allow|angle|animation|around|as|assert|behind|box_layout|break|build|cache|call|center|changed|child_size|choice|circles|class|clear|clicked|clipping|clockwise|config|contains|continue|corner1|corner2|counterclockwise|def|default|define|del|delay|disabled|disabled_text|dissolve|elif|else|event|except|exclude|exec|expression|fade|finally|for|from|function|global|gm_root|has|hide|id|if|import|in|init|is|jump|knot|lambda|left|less_rounded|mm_root|movie|music|null|on|onlayer|pass|pause|persistent|play|print|python|queue|raise|random|renpy|repeat|return|right|rounded_window|scene|scope|set|show|slow|slow_abortable|slow_done|sound|stop|store|style|style_group|substitute|suffix|theme|transform|transform_anchor|transpose|try|ui|unhovered|updater|use|voice|while|widget|widget_hover|widget_selected|widget_text|yield)\b/,
      boolean: /\b(?:[Ff]alse|[Tt]rue)\b/,
      number:
        /(?:\b(?:0[bo])?(?:(?:\d|0x[\da-f])[\da-f]*(?:\.\d*)?)|\B\.\d+)(?:e[+-]?\d+)?j?/i,
      operator:
        /[-+%=]=?|!=|\*\*?=?|\/\/?=?|<[<=>]?|>[=>]?|[&|^~]|\b(?:and|at|not|or|with)\b/,
      punctuation: /[{}[\];(),.:]/
    };
    Prism.languages.rpy = Prism.languages.renpy;
  }

  // @ts-nocheck
  rest.displayName = 'rest';
  rest.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function rest(Prism) {
    Prism.languages.rest = {
      table: [
        {
          pattern:
            /(^[\t ]*)(?:\+[=-]+)+\+(?:\r?\n|\r)(?:\1[+|].+[+|](?:\r?\n|\r))+\1(?:\+[=-]+)+\+/m,
          lookbehind: true,
          inside: {
            punctuation: /\||(?:\+[=-]+)+\+/
          }
        },
        {
          pattern:
            /(^[\t ]*)=+ [ =]*=(?:(?:\r?\n|\r)\1.+)+(?:\r?\n|\r)\1=+ [ =]*=(?=(?:\r?\n|\r){2}|\s*$)/m,
          lookbehind: true,
          inside: {
            punctuation: /[=-]+/
          }
        }
      ],
      // Directive-like patterns
      'substitution-def': {
        pattern: /(^[\t ]*\.\. )\|(?:[^|\s](?:[^|]*[^|\s])?)\| [^:]+::/m,
        lookbehind: true,
        inside: {
          substitution: {
            pattern: /^\|(?:[^|\s]|[^|\s][^|]*[^|\s])\|/,
            alias: 'attr-value',
            inside: {
              punctuation: /^\||\|$/
            }
          },
          directive: {
            pattern: /( )(?! )[^:]+::/,
            lookbehind: true,
            alias: 'function',
            inside: {
              punctuation: /::$/
            }
          }
        }
      },
      'link-target': [
        {
          pattern: /(^[\t ]*\.\. )\[[^\]]+\]/m,
          lookbehind: true,
          alias: 'string',
          inside: {
            punctuation: /^\[|\]$/
          }
        },
        {
          pattern: /(^[\t ]*\.\. )_(?:`[^`]+`|(?:[^:\\]|\\.)+):/m,
          lookbehind: true,
          alias: 'string',
          inside: {
            punctuation: /^_|:$/
          }
        }
      ],
      directive: {
        pattern: /(^[\t ]*\.\. )[^:]+::/m,
        lookbehind: true,
        alias: 'function',
        inside: {
          punctuation: /::$/
        }
      },
      comment: {
        // The two alternatives try to prevent highlighting of blank comments
        pattern:
          /(^[\t ]*\.\.)(?:(?: .+)?(?:(?:\r?\n|\r).+)+| .+)(?=(?:\r?\n|\r){2}|$)/m,
        lookbehind: true
      },
      title: [
        // Overlined and underlined
        {
          pattern:
            /^(([!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~])\2+)(?:\r?\n|\r).+(?:\r?\n|\r)\1$/m,
          inside: {
            punctuation:
              /^[!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~]+|[!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~]+$/,
            important: /.+/
          }
        }, // Underlined only
        {
          pattern:
            /(^|(?:\r?\n|\r){2}).+(?:\r?\n|\r)([!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~])\2+(?=\r?\n|\r|$)/,
          lookbehind: true,
          inside: {
            punctuation: /[!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~]+$/,
            important: /.+/
          }
        }
      ],
      hr: {
        pattern:
          /((?:\r?\n|\r){2})([!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~])\2{3,}(?=(?:\r?\n|\r){2})/,
        lookbehind: true,
        alias: 'punctuation'
      },
      field: {
        pattern: /(^[\t ]*):[^:\r\n]+:(?= )/m,
        lookbehind: true,
        alias: 'attr-name'
      },
      'command-line-option': {
        pattern:
          /(^[\t ]*)(?:[+-][a-z\d]|(?:--|\/)[a-z\d-]+)(?:[ =](?:[a-z][\w-]*|<[^<>]+>))?(?:, (?:[+-][a-z\d]|(?:--|\/)[a-z\d-]+)(?:[ =](?:[a-z][\w-]*|<[^<>]+>))?)*(?=(?:\r?\n|\r)? {2,}\S)/im,
        lookbehind: true,
        alias: 'symbol'
      },
      'literal-block': {
        pattern: /::(?:\r?\n|\r){2}([ \t]+)(?![ \t]).+(?:(?:\r?\n|\r)\1.+)*/,
        inside: {
          'literal-block-punctuation': {
            pattern: /^::/,
            alias: 'punctuation'
          }
        }
      },
      'quoted-literal-block': {
        pattern:
          /::(?:\r?\n|\r){2}([!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~]).*(?:(?:\r?\n|\r)\1.*)*/,
        inside: {
          'literal-block-punctuation': {
            pattern: /^(?:::|([!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~])\1*)/m,
            alias: 'punctuation'
          }
        }
      },
      'list-bullet': {
        pattern:
          /(^[\t ]*)(?:[*+\-•‣⁃]|\(?(?:\d+|[a-z]|[ivxdclm]+)\)|(?:\d+|[a-z]|[ivxdclm]+)\.)(?= )/im,
        lookbehind: true,
        alias: 'punctuation'
      },
      'doctest-block': {
        pattern: /(^[\t ]*)>>> .+(?:(?:\r?\n|\r).+)*/m,
        lookbehind: true,
        inside: {
          punctuation: /^>>>/
        }
      },
      inline: [
        {
          pattern:
            /(^|[\s\-:\/'"<(\[{])(?::[^:]+:`.*?`|`.*?`:[^:]+:|(\*\*?|``?|\|)(?!\s)(?:(?!\2).)*\S\2(?=[\s\-.,:;!?\\\/'")\]}]|$))/m,
          lookbehind: true,
          inside: {
            bold: {
              pattern: /(^\*\*).+(?=\*\*$)/,
              lookbehind: true
            },
            italic: {
              pattern: /(^\*).+(?=\*$)/,
              lookbehind: true
            },
            'inline-literal': {
              pattern: /(^``).+(?=``$)/,
              lookbehind: true,
              alias: 'symbol'
            },
            role: {
              pattern: /^:[^:]+:|:[^:]+:$/,
              alias: 'function',
              inside: {
                punctuation: /^:|:$/
              }
            },
            'interpreted-text': {
              pattern: /(^`).+(?=`$)/,
              lookbehind: true,
              alias: 'attr-value'
            },
            substitution: {
              pattern: /(^\|).+(?=\|$)/,
              lookbehind: true,
              alias: 'attr-value'
            },
            punctuation: /\*\*?|``?|\|/
          }
        }
      ],
      link: [
        {
          pattern: /\[[^\[\]]+\]_(?=[\s\-.,:;!?\\\/'")\]}]|$)/,
          alias: 'string',
          inside: {
            punctuation: /^\[|\]_$/
          }
        },
        {
          pattern:
            /(?:\b[a-z\d]+(?:[_.:+][a-z\d]+)*_?_|`[^`]+`_?_|_`[^`]+`)(?=[\s\-.,:;!?\\\/'")\]}]|$)/i,
          alias: 'string',
          inside: {
            punctuation: /^_?`|`$|`?_?_$/
          }
        }
      ],
      // Line block start,
      // quote attribution,
      // explicit markup start,
      // and anonymous hyperlink target shortcut (__)
      punctuation: {
        pattern: /(^[\t ]*)(?:\|(?= |$)|(?:---?|—|\.\.|__)(?= )|\.\.$)/m,
        lookbehind: true
      }
    };
  }

  // @ts-nocheck
  rip.displayName = 'rip';
  rip.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function rip(Prism) {
    Prism.languages.rip = {
      comment: {
        pattern: /#.*/,
        greedy: true
      },
      char: {
        pattern: /\B`[^\s`'",.:;#\/\\()<>\[\]{}]\b/,
        greedy: true
      },
      string: {
        pattern: /("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,
        greedy: true
      },
      regex: {
        pattern:
          /(^|[^/])\/(?!\/)(?:\[[^\n\r\]]*\]|\\.|[^/\\\r\n\[])+\/(?=\s*(?:$|[\r\n,.;})]))/,
        lookbehind: true,
        greedy: true
      },
      keyword:
        /(?:=>|->)|\b(?:case|catch|class|else|exit|finally|if|raise|return|switch|try)\b/,
      builtin: /@|\bSystem\b/,
      boolean: /\b(?:false|true)\b/,
      date: /\b\d{4}-\d{2}-\d{2}\b/,
      time: /\b\d{2}:\d{2}:\d{2}\b/,
      datetime: /\b\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\b/,
      symbol: /:[^\d\s`'",.:;#\/\\()<>\[\]{}][^\s`'",.:;#\/\\()<>\[\]{}]*/,
      number: /[+-]?\b(?:\d+\.\d+|\d+)\b/,
      punctuation: /(?:\.{2,3})|[`,.:;=\/\\()<>\[\]{}]/,
      reference: /[^\d\s`'",.:;#\/\\()<>\[\]{}][^\s`'",.:;#\/\\()<>\[\]{}]*/
    };
  }

  // @ts-nocheck
  roboconf.displayName = 'roboconf';
  roboconf.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function roboconf(Prism) {
    Prism.languages.roboconf = {
      comment: /#.*/,
      keyword: {
        pattern:
          /(^|\s)(?:(?:external|import)\b|(?:facet|instance of)(?=[ \t]+[\w-]+[ \t]*\{))/,
        lookbehind: true
      },
      component: {
        pattern: /[\w-]+(?=[ \t]*\{)/,
        alias: 'variable'
      },
      property: /[\w.-]+(?=[ \t]*:)/,
      value: {
        pattern: /(=[ \t]*(?![ \t]))[^,;]+/,
        lookbehind: true,
        alias: 'attr-value'
      },
      optional: {
        pattern: /\(optional\)/,
        alias: 'builtin'
      },
      wildcard: {
        pattern: /(\.)\*/,
        lookbehind: true,
        alias: 'operator'
      },
      punctuation: /[{},.;:=]/
    };
  }

  // @ts-nocheck
  robotframework.displayName = 'robotframework';
  robotframework.aliases = ['robot'];

  /** @type {import('../core.js').Syntax} */
  function robotframework(Prism) {
  (function (Prism) {
      var comment = {
        pattern: /(^[ \t]*| {2}|\t)#.*/m,
        lookbehind: true,
        greedy: true
      };
      var variable = {
        pattern: /((?:^|[^\\])(?:\\{2})*)[$@&%]\{(?:[^{}\r\n]|\{[^{}\r\n]*\})*\}/,
        lookbehind: true,
        inside: {
          punctuation: /^[$@&%]\{|\}$/
        }
      };
      function createSection(name, inside) {
        var extendecInside = {};
        extendecInside['section-header'] = {
          pattern: /^ ?\*{3}.+?\*{3}/,
          alias: 'keyword'
        }; // copy inside tokens
        for (var token in inside) {
          extendecInside[token] = inside[token];
        }
        extendecInside['tag'] = {
          pattern: /([\r\n](?: {2}|\t)[ \t]*)\[[-\w]+\]/,
          lookbehind: true,
          inside: {
            punctuation: /\[|\]/
          }
        };
        extendecInside['variable'] = variable;
        extendecInside['comment'] = comment;
        return {
          pattern: RegExp(
            /^ ?\*{3}[ \t]*<name>[ \t]*\*{3}(?:.|[\r\n](?!\*{3}))*/.source.replace(
              /<name>/g,
              function () {
                return name
              }
            ),
            'im'
          ),
          alias: 'section',
          inside: extendecInside
        }
      }
      var docTag = {
        pattern:
          /(\[Documentation\](?: {2}|\t)[ \t]*)(?![ \t]|#)(?:.|(?:\r\n?|\n)[ \t]*\.{3})+/,
        lookbehind: true,
        alias: 'string'
      };
      var testNameLike = {
        pattern: /([\r\n] ?)(?!#)(?:\S(?:[ \t]\S)*)+/,
        lookbehind: true,
        alias: 'function',
        inside: {
          variable: variable
        }
      };
      var testPropertyLike = {
        pattern: /([\r\n](?: {2}|\t)[ \t]*)(?!\[|\.{3}|#)(?:\S(?:[ \t]\S)*)+/,
        lookbehind: true,
        inside: {
          variable: variable
        }
      };
      Prism.languages['robotframework'] = {
        settings: createSection('Settings', {
          documentation: {
            pattern:
              /([\r\n] ?Documentation(?: {2}|\t)[ \t]*)(?![ \t]|#)(?:.|(?:\r\n?|\n)[ \t]*\.{3})+/,
            lookbehind: true,
            alias: 'string'
          },
          property: {
            pattern: /([\r\n] ?)(?!\.{3}|#)(?:\S(?:[ \t]\S)*)+/,
            lookbehind: true
          }
        }),
        variables: createSection('Variables'),
        'test-cases': createSection('Test Cases', {
          'test-name': testNameLike,
          documentation: docTag,
          property: testPropertyLike
        }),
        keywords: createSection('Keywords', {
          'keyword-name': testNameLike,
          documentation: docTag,
          property: testPropertyLike
        }),
        tasks: createSection('Tasks', {
          'task-name': testNameLike,
          documentation: docTag,
          property: testPropertyLike
        }),
        comment: comment
      };
      Prism.languages.robot = Prism.languages['robotframework'];
    })(Prism);
  }

  // @ts-nocheck
  sas.displayName = 'sas';
  sas.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function sas(Prism) {
  (function (Prism) {
      var stringPattern = /(?:"(?:""|[^"])*"(?!")|'(?:''|[^'])*'(?!'))/.source;
      var number = /\b(?:\d[\da-f]*x|\d+(?:\.\d+)?(?:e[+-]?\d+)?)\b/i;
      var numericConstant = {
        pattern: RegExp(stringPattern + '[bx]'),
        alias: 'number'
      };
      var macroVariable = {
        pattern: /&[a-z_]\w*/i
      };
      var macroKeyword = {
        pattern:
          /((?:^|\s|=|\())%(?:ABORT|BY|CMS|COPY|DISPLAY|DO|ELSE|END|EVAL|GLOBAL|GO|GOTO|IF|INC|INCLUDE|INDEX|INPUT|KTRIM|LENGTH|LET|LIST|LOCAL|PUT|QKTRIM|QSCAN|QSUBSTR|QSYSFUNC|QUPCASE|RETURN|RUN|SCAN|SUBSTR|SUPERQ|SYMDEL|SYMEXIST|SYMGLOBL|SYMLOCAL|SYSCALL|SYSEVALF|SYSEXEC|SYSFUNC|SYSGET|SYSRPUT|THEN|TO|TSO|UNQUOTE|UNTIL|UPCASE|WHILE|WINDOW)\b/i,
        lookbehind: true,
        alias: 'keyword'
      };
      var step = {
        pattern: /(^|\s)(?:proc\s+\w+|data(?!=)|quit|run)\b/i,
        alias: 'keyword',
        lookbehind: true
      };
      var comment = [
        /\/\*[\s\S]*?\*\//,
        {
          pattern: /(^[ \t]*|;\s*)\*[^;]*;/m,
          lookbehind: true
        }
      ];
      var string = {
        pattern: RegExp(stringPattern),
        greedy: true
      };
      var punctuation = /[$%@.(){}\[\];,\\]/;
      var func = {
        pattern: /%?\b\w+(?=\()/,
        alias: 'keyword'
      };
      var args = {
        function: func,
        'arg-value': {
          pattern: /(=\s*)[A-Z\.]+/i,
          lookbehind: true
        },
        operator: /=/,
        'macro-variable': macroVariable,
        arg: {
          pattern: /[A-Z]+/i,
          alias: 'keyword'
        },
        number: number,
        'numeric-constant': numericConstant,
        punctuation: punctuation,
        string: string
      };
      var format = {
        pattern: /\b(?:format|put)\b=?[\w'$.]+/i,
        inside: {
          keyword: /^(?:format|put)(?==)/i,
          equals: /=/,
          format: {
            pattern: /(?:\w|\$\d)+\.\d?/,
            alias: 'number'
          }
        }
      };
      var altformat = {
        pattern: /\b(?:format|put)\s+[\w']+(?:\s+[$.\w]+)+(?=;)/i,
        inside: {
          keyword: /^(?:format|put)/i,
          format: {
            pattern: /[\w$]+\.\d?/,
            alias: 'number'
          }
        }
      };
      var globalStatements = {
        pattern:
          /((?:^|\s)=?)(?:catname|checkpoint execute_always|dm|endsas|filename|footnote|%include|libname|%list|lock|missing|options|page|resetline|%run|sasfile|skip|sysecho|title\d?)\b/i,
        lookbehind: true,
        alias: 'keyword'
      };
      var submitStatement = {
        pattern: /(^|\s)(?:submit(?:\s+(?:load|norun|parseonly))?|endsubmit)\b/i,
        lookbehind: true,
        alias: 'keyword'
      };
      var actionSets =
        /aStore|accessControl|aggregation|audio|autotune|bayesianNetClassifier|bioMedImage|boolRule|builtins|cardinality|cdm|clustering|conditionalRandomFields|configuration|copula|countreg|dataDiscovery|dataPreprocess|dataSciencePilot|dataStep|decisionTree|deduplication|deepLearn|deepNeural|deepRnn|ds2|ecm|entityRes|espCluster|explainModel|factmac|fastKnn|fcmpact|fedSql|freqTab|gVarCluster|gam|gleam|graphSemiSupLearn|hiddenMarkovModel|hyperGroup|ica|image|iml|kernalPca|langModel|ldaTopic|loadStreams|mbc|mixed|mlTools|modelPublishing|network|neuralNet|nmf|nonParametricBayes|nonlinear|optNetwork|optimization|panel|pca|percentile|phreg|pls|qkb|qlim|quantreg|recommend|regression|reinforcementLearn|robustPca|ruleMining|sampling|sandwich|sccasl|search(?:Analytics)?|sentimentAnalysis|sequence|session(?:Prop)?|severity|simSystem|simple|smartData|sparkEmbeddedProcess|sparseML|spatialreg|spc|stabilityMonitoring|svDataDescription|svm|table|text(?:Filters|Frequency|Mining|Parse|Rule(?:Develop|Score)|Topic|Util)|timeData|transpose|tsInfo|tsReconcile|uniTimeSeries|varReduce/
          .source;
      var casActions = {
        pattern: RegExp(
          /(^|\s)(?:action\s+)?(?:<act>)\.[a-z]+\b[^;]+/.source.replace(
            /<act>/g,
            function () {
              return actionSets
            }
          ),
          'i'
        ),
        lookbehind: true,
        inside: {
          keyword: RegExp(
            /(?:<act>)\.[a-z]+\b/.source.replace(/<act>/g, function () {
              return actionSets
            }),
            'i'
          ),
          action: {
            pattern: /(?:action)/i,
            alias: 'keyword'
          },
          comment: comment,
          function: func,
          'arg-value': args['arg-value'],
          operator: args.operator,
          argument: args.arg,
          number: number,
          'numeric-constant': numericConstant,
          punctuation: punctuation,
          string: string
        }
      };
      var keywords = {
        pattern:
          /((?:^|\s)=?)(?:after|analysis|and|array|barchart|barwidth|begingraph|by|call|cas|cbarline|cfill|class(?:lev)?|close|column|computed?|contains|continue|data(?==)|define|delete|describe|document|do\s+over|do|dol|drop|dul|else|end(?:comp|source)?|entryTitle|eval(?:uate)?|exec(?:ute)?|exit|file(?:name)?|fill(?:attrs)?|flist|fnc|function(?:list)?|global|goto|group(?:by)?|headline|headskip|histogram|if|infile|keep|keylabel|keyword|label|layout|leave|legendlabel|length|libname|loadactionset|merge|midpoints|_?null_|name|noobs|nowd|ods|options|or|otherwise|out(?:put)?|over(?:lay)?|plot|print|put|raise|ranexp|rannor|rbreak|retain|return|select|session|sessref|set|source|statgraph|sum|summarize|table|temp|terminate|then\s+do|then|title\d?|to|var|when|where|xaxisopts|y2axisopts|yaxisopts)\b/i,
        lookbehind: true
      };
      Prism.languages.sas = {
        datalines: {
          pattern: /^([ \t]*)(?:cards|(?:data)?lines);[\s\S]+?^[ \t]*;/im,
          lookbehind: true,
          alias: 'string',
          inside: {
            keyword: {
              pattern: /^(?:cards|(?:data)?lines)/i
            },
            punctuation: /;/
          }
        },
        'proc-sql': {
          pattern:
            /(^proc\s+(?:fed)?sql(?:\s+[\w|=]+)?;)[\s\S]+?(?=^(?:proc\s+\w+|data|quit|run);|(?![\s\S]))/im,
          lookbehind: true,
          inside: {
            sql: {
              pattern: RegExp(
                /^[ \t]*(?:select|alter\s+table|(?:create|describe|drop)\s+(?:index|table(?:\s+constraints)?|view)|create\s+unique\s+index|insert\s+into|update)(?:<str>|[^;"'])+;/.source.replace(
                  /<str>/g,
                  function () {
                    return stringPattern
                  }
                ),
                'im'
              ),
              alias: 'language-sql',
              inside: Prism.languages.sql
            },
            'global-statements': globalStatements,
            'sql-statements': {
              pattern:
                /(^|\s)(?:disconnect\s+from|begin|commit|exec(?:ute)?|reset|rollback|validate)\b/i,
              lookbehind: true,
              alias: 'keyword'
            },
            number: number,
            'numeric-constant': numericConstant,
            punctuation: punctuation,
            string: string
          }
        },
        'proc-groovy': {
          pattern:
            /(^proc\s+groovy(?:\s+[\w|=]+)?;)[\s\S]+?(?=^(?:proc\s+\w+|data|quit|run);|(?![\s\S]))/im,
          lookbehind: true,
          inside: {
            comment: comment,
            groovy: {
              pattern: RegExp(
                /(^[ \t]*submit(?:\s+(?:load|norun|parseonly))?)(?:<str>|[^"'])+?(?=endsubmit;)/.source.replace(
                  /<str>/g,
                  function () {
                    return stringPattern
                  }
                ),
                'im'
              ),
              lookbehind: true,
              alias: 'language-groovy',
              inside: Prism.languages.groovy
            },
            keyword: keywords,
            'submit-statement': submitStatement,
            'global-statements': globalStatements,
            number: number,
            'numeric-constant': numericConstant,
            punctuation: punctuation,
            string: string
          }
        },
        'proc-lua': {
          pattern:
            /(^proc\s+lua(?:\s+[\w|=]+)?;)[\s\S]+?(?=^(?:proc\s+\w+|data|quit|run);|(?![\s\S]))/im,
          lookbehind: true,
          inside: {
            comment: comment,
            lua: {
              pattern: RegExp(
                /(^[ \t]*submit(?:\s+(?:load|norun|parseonly))?)(?:<str>|[^"'])+?(?=endsubmit;)/.source.replace(
                  /<str>/g,
                  function () {
                    return stringPattern
                  }
                ),
                'im'
              ),
              lookbehind: true,
              alias: 'language-lua',
              inside: Prism.languages.lua
            },
            keyword: keywords,
            'submit-statement': submitStatement,
            'global-statements': globalStatements,
            number: number,
            'numeric-constant': numericConstant,
            punctuation: punctuation,
            string: string
          }
        },
        'proc-cas': {
          pattern:
            /(^proc\s+cas(?:\s+[\w|=]+)?;)[\s\S]+?(?=^(?:proc\s+\w+|quit|data);|(?![\s\S]))/im,
          lookbehind: true,
          inside: {
            comment: comment,
            'statement-var': {
              pattern: /((?:^|\s)=?)saveresult\s[^;]+/im,
              lookbehind: true,
              inside: {
                statement: {
                  pattern: /^saveresult\s+\S+/i,
                  inside: {
                    keyword: /^(?:saveresult)/i
                  }
                },
                rest: args
              }
            },
            'cas-actions': casActions,
            statement: {
              pattern:
                /((?:^|\s)=?)(?:default|(?:un)?set|on|output|upload)[^;]+/im,
              lookbehind: true,
              inside: args
            },
            step: step,
            keyword: keywords,
            function: func,
            format: format,
            altformat: altformat,
            'global-statements': globalStatements,
            number: number,
            'numeric-constant': numericConstant,
            punctuation: punctuation,
            string: string
          }
        },
        'proc-args': {
          pattern: RegExp(
            /(^proc\s+\w+\s+)(?!\s)(?:[^;"']|<str>)+;/.source.replace(
              /<str>/g,
              function () {
                return stringPattern
              }
            ),
            'im'
          ),
          lookbehind: true,
          inside: args
        },
        /*Special keywords within macros*/
        'macro-keyword': macroKeyword,
        'macro-variable': macroVariable,
        'macro-string-functions': {
          pattern:
            /((?:^|\s|=))%(?:BQUOTE|NRBQUOTE|NRQUOTE|NRSTR|QUOTE|STR)\(.*?(?:[^%]\))/i,
          lookbehind: true,
          inside: {
            function: {
              pattern: /%(?:BQUOTE|NRBQUOTE|NRQUOTE|NRSTR|QUOTE|STR)/i,
              alias: 'keyword'
            },
            'macro-keyword': macroKeyword,
            'macro-variable': macroVariable,
            'escaped-char': {
              pattern: /%['"()<>=¬^~;,#]/
            },
            punctuation: punctuation
          }
        },
        'macro-declaration': {
          pattern: /^%macro[^;]+(?=;)/im,
          inside: {
            keyword: /%macro/i
          }
        },
        'macro-end': {
          pattern: /^%mend[^;]+(?=;)/im,
          inside: {
            keyword: /%mend/i
          }
        },
        /*%_zscore(headcir, _lhc, _mhc, _shc, headcz, headcpct, _Fheadcz); */
        macro: {
          pattern: /%_\w+(?=\()/,
          alias: 'keyword'
        },
        input: {
          pattern: /\binput\s[-\w\s/*.$&]+;/i,
          inside: {
            input: {
              alias: 'keyword',
              pattern: /^input/i
            },
            comment: comment,
            number: number,
            'numeric-constant': numericConstant
          }
        },
        'options-args': {
          pattern: /(^options)[-'"|/\\<>*+=:()\w\s]*(?=;)/im,
          lookbehind: true,
          inside: args
        },
        'cas-actions': casActions,
        comment: comment,
        function: func,
        format: format,
        altformat: altformat,
        'numeric-constant': numericConstant,
        datetime: {
          // '1jan2013'd, '9:25:19pm't, '18jan2003:9:27:05am'dt
          pattern: RegExp(stringPattern + '(?:dt?|t)'),
          alias: 'number'
        },
        string: string,
        step: step,
        keyword: keywords,
        // In SAS Studio syntax highlighting, these operators are styled like keywords
        'operator-keyword': {
          pattern: /\b(?:eq|ge|gt|in|le|lt|ne|not)\b/i,
          alias: 'operator'
        },
        // Decimal (1.2e23), hexadecimal (0c1x)
        number: number,
        operator: /\*\*?|\|\|?|!!?|¦¦?|<[>=]?|>[<=]?|[-+\/=&]|[~¬^]=?/,
        punctuation: punctuation
      };
    })(Prism);
  }

  // @ts-nocheck
  shellSession.displayName = 'shell-session';
  shellSession.aliases = ['sh-session', 'shellsession'];

  /** @type {import('../core.js').Syntax} */
  function shellSession(Prism) {
    Prism.register(bash)
    ;(function (Prism) {
      // CAREFUL!
      // The following patterns are concatenated, so the group referenced by a back reference is non-obvious!
      var strings = [
        // normal string
        /"(?:\\[\s\S]|\$\([^)]+\)|\$(?!\()|`[^`]+`|[^"\\`$])*"/.source,
        /'[^']*'/.source,
        /\$'(?:[^'\\]|\\[\s\S])*'/.source, // here doc
        // 2 capturing groups
        /<<-?\s*(["']?)(\w+)\1\s[\s\S]*?[\r\n]\2/.source
      ].join('|');
      Prism.languages['shell-session'] = {
        command: {
          pattern: RegExp(
            // user info
            /^/.source +
              '(?:' + // <user> ":" ( <path> )?
              (/[^\s@:$#%*!/\\]+@[^\r\n@:$#%*!/\\]+(?::[^\0-\x1F$#%*?"<>:;|]+)?/
                .source +
                '|' + // <path>
                // Since the path pattern is quite general, we will require it to start with a special character to
                // prevent false positives.
                /[/~.][^\0-\x1F$#%*?"<>@:;|]*/.source) +
              ')?' + // shell symbol
              /[$#%](?=\s)/.source + // bash command
              /(?:[^\\\r\n \t'"<$]|[ \t](?:(?!#)|#.*$)|\\(?:[^\r]|\r\n?)|\$(?!')|<(?!<)|<<str>>)+/.source.replace(
                /<<str>>/g,
                function () {
                  return strings
                }
              ),
            'm'
          ),
          greedy: true,
          inside: {
            info: {
              // foo@bar:~/files$ exit
              // foo@bar$ exit
              // ~/files$ exit
              pattern: /^[^#$%]+/,
              alias: 'punctuation',
              inside: {
                user: /^[^\s@:$#%*!/\\]+@[^\r\n@:$#%*!/\\]+/,
                punctuation: /:/,
                path: /[\s\S]+/
              }
            },
            bash: {
              pattern: /(^[$#%]\s*)\S[\s\S]*/,
              lookbehind: true,
              alias: 'language-bash',
              inside: Prism.languages.bash
            },
            'shell-symbol': {
              pattern: /^[$#%]/,
              alias: 'important'
            }
          }
        },
        output: /.(?:.*(?:[\r\n]|.$))*/
      };
      Prism.languages['sh-session'] = Prism.languages['shellsession'] =
        Prism.languages['shell-session'];
    })(Prism);
  }

  // @ts-nocheck
  smali.displayName = 'smali';
  smali.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function smali(Prism) {
    // Test files for the parser itself:
    // https://github.com/JesusFreke/smali/tree/master/smali/src/test/resources/LexerTest
    Prism.languages.smali = {
      comment: /#.*/,
      string: {
        pattern: /"(?:[^\r\n\\"]|\\.)*"|'(?:[^\r\n\\']|\\(?:.|u[\da-fA-F]{4}))'/,
        greedy: true
      },
      'class-name': {
        pattern:
          /(^|[^L])L(?:(?:\w+|`[^`\r\n]*`)\/)*(?:[\w$]+|`[^`\r\n]*`)(?=\s*;)/,
        lookbehind: true,
        inside: {
          'class-name': {
            pattern: /(^L|\/)(?:[\w$]+|`[^`\r\n]*`)$/,
            lookbehind: true
          },
          namespace: {
            pattern: /^(L)(?:(?:\w+|`[^`\r\n]*`)\/)+/,
            lookbehind: true,
            inside: {
              punctuation: /\//
            }
          },
          builtin: /^L/
        }
      },
      builtin: [
        {
          // Reference: https://github.com/JesusFreke/smali/wiki/TypesMethodsAndFields#types
          pattern: /([();\[])[BCDFIJSVZ]+/,
          lookbehind: true
        },
        {
          // e.g. .field mWifiOnUid:I
          pattern: /([\w$>]:)[BCDFIJSVZ]/,
          lookbehind: true
        }
      ],
      keyword: [
        {
          pattern: /(\.end\s+)[\w-]+/,
          lookbehind: true
        },
        {
          pattern: /(^|[^\w.-])\.(?!\d)[\w-]+/,
          lookbehind: true
        },
        {
          pattern:
            /(^|[^\w.-])(?:abstract|annotation|bridge|constructor|enum|final|interface|private|protected|public|runtime|static|synthetic|system|transient)(?![\w.-])/,
          lookbehind: true
        }
      ],
      function: {
        pattern: /(^|[^\w.-])(?:\w+|<[\w$-]+>)(?=\()/,
        lookbehind: true
      },
      field: {
        pattern: /[\w$]+(?=:)/,
        alias: 'variable'
      },
      register: {
        pattern: /(^|[^\w.-])[vp]\d(?![\w.-])/,
        lookbehind: true,
        alias: 'variable'
      },
      boolean: {
        pattern: /(^|[^\w.-])(?:false|true)(?![\w.-])/,
        lookbehind: true
      },
      number: {
        pattern:
          /(^|[^/\w.-])-?(?:NAN|INFINITY|0x(?:[\dA-F]+(?:\.[\dA-F]*)?|\.[\dA-F]+)(?:p[+-]?[\dA-F]+)?|(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?)[dflst]?(?![\w.-])/i,
        lookbehind: true
      },
      label: {
        pattern: /(:)\w+/,
        lookbehind: true,
        alias: 'property'
      },
      operator: /->|\.\.|[\[=]/,
      punctuation: /[{}(),;:]/
    };
  }

  // @ts-nocheck
  smalltalk.displayName = 'smalltalk';
  smalltalk.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function smalltalk(Prism) {
    Prism.languages.smalltalk = {
      comment: {
        pattern: /"(?:""|[^"])*"/,
        greedy: true
      },
      char: {
        pattern: /\$./,
        greedy: true
      },
      string: {
        pattern: /'(?:''|[^'])*'/,
        greedy: true
      },
      symbol: /#[\da-z]+|#(?:-|([+\/\\*~<>=@%|&?!])\1?)|#(?=\()/i,
      'block-arguments': {
        pattern: /(\[\s*):[^\[|]*\|/,
        lookbehind: true,
        inside: {
          variable: /:[\da-z]+/i,
          punctuation: /\|/
        }
      },
      'temporary-variables': {
        pattern: /\|[^|]+\|/,
        inside: {
          variable: /[\da-z]+/i,
          punctuation: /\|/
        }
      },
      keyword: /\b(?:new|nil|self|super)\b/,
      boolean: /\b(?:false|true)\b/,
      number: [
        /\d+r-?[\dA-Z]+(?:\.[\dA-Z]+)?(?:e-?\d+)?/,
        /\b\d+(?:\.\d+)?(?:e-?\d+)?/
      ],
      operator: /[<=]=?|:=|~[~=]|\/\/?|\\\\|>[>=]?|[!^+\-*&|,@]/,
      punctuation: /[.;:?\[\](){}]/
    };
  }

  // @ts-nocheck
  smarty.displayName = 'smarty';
  smarty.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function smarty(Prism) {
    Prism.register(markupTemplating)
    ;(function (Prism) {
      Prism.languages.smarty = {
        comment: {
          pattern: /^\{\*[\s\S]*?\*\}/,
          greedy: true
        },
        'embedded-php': {
          pattern: /^\{php\}[\s\S]*?\{\/php\}/,
          greedy: true,
          inside: {
            smarty: {
              pattern: /^\{php\}|\{\/php\}$/,
              inside: null // see below
            },
            php: {
              pattern: /[\s\S]+/,
              alias: 'language-php',
              inside: Prism.languages.php
            }
          }
        },
        string: [
          {
            pattern: /"(?:\\.|[^"\\\r\n])*"/,
            greedy: true,
            inside: {
              interpolation: {
                pattern: /\{[^{}]*\}|`[^`]*`/,
                inside: {
                  'interpolation-punctuation': {
                    pattern: /^[{`]|[`}]$/,
                    alias: 'punctuation'
                  },
                  expression: {
                    pattern: /[\s\S]+/,
                    inside: null // see below
                  }
                }
              },
              variable: /\$\w+/
            }
          },
          {
            pattern: /'(?:\\.|[^'\\\r\n])*'/,
            greedy: true
          }
        ],
        keyword: {
          pattern: /(^\{\/?)[a-z_]\w*\b(?!\()/i,
          lookbehind: true,
          greedy: true
        },
        delimiter: {
          pattern: /^\{\/?|\}$/,
          greedy: true,
          alias: 'punctuation'
        },
        number: /\b0x[\dA-Fa-f]+|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee][-+]?\d+)?/,
        variable: [
          /\$(?!\d)\w+/,
          /#(?!\d)\w+#/,
          {
            pattern: /(\.|->|\w\s*=)(?!\d)\w+\b(?!\()/,
            lookbehind: true
          },
          {
            pattern: /(\[)(?!\d)\w+(?=\])/,
            lookbehind: true
          }
        ],
        function: {
          pattern: /(\|\s*)@?[a-z_]\w*|\b[a-z_]\w*(?=\()/i,
          lookbehind: true
        },
        'attr-name': /\b[a-z_]\w*(?=\s*=)/i,
        boolean: /\b(?:false|no|off|on|true|yes)\b/,
        punctuation: /[\[\](){}.,:`]|->/,
        operator: [
          /[+\-*\/%]|==?=?|[!<>]=?|&&|\|\|?/,
          /\bis\s+(?:not\s+)?(?:div|even|odd)(?:\s+by)?\b/,
          /\b(?:and|eq|gt?e|gt|lt?e|lt|mod|neq?|not|or)\b/
        ]
      };
      Prism.languages.smarty['embedded-php'].inside.smarty.inside =
        Prism.languages.smarty;
      Prism.languages.smarty.string[0].inside.interpolation.inside.expression.inside =
        Prism.languages.smarty;
      var string = /"(?:\\.|[^"\\\r\n])*"|'(?:\\.|[^'\\\r\n])*'/;
      var smartyPattern = RegExp(
        // comments
        /\{\*[\s\S]*?\*\}/.source +
          '|' + // php tags
          /\{php\}[\s\S]*?\{\/php\}/.source +
          '|' + // smarty blocks
          /\{(?:[^{}"']|<str>|\{(?:[^{}"']|<str>|\{(?:[^{}"']|<str>)*\})*\})*\}/.source.replace(
            /<str>/g,
            function () {
              return string.source
            }
          ),
        'g'
      ); // Tokenize all inline Smarty expressions
      Prism.hooks.add('before-tokenize', function (env) {
        var smartyLiteralStart = '{literal}';
        var smartyLiteralEnd = '{/literal}';
        var smartyLiteralMode = false;
        Prism.languages['markup-templating'].buildPlaceholders(
          env,
          'smarty',
          smartyPattern,
          function (match) {
            // Smarty tags inside {literal} block are ignored
            if (match === smartyLiteralEnd) {
              smartyLiteralMode = false;
            }
            if (!smartyLiteralMode) {
              if (match === smartyLiteralStart) {
                smartyLiteralMode = true;
              }
              return true
            }
            return false
          }
        );
      }); // Re-insert the tokens after tokenizing
      Prism.hooks.add('after-tokenize', function (env) {
        Prism.languages['markup-templating'].tokenizePlaceholders(env, 'smarty');
      });
    })(Prism);
  }

  // @ts-nocheck
  sml.displayName = 'sml';
  sml.aliases = ['smlnj'];

  /** @type {import('../core.js').Syntax} */
  function sml(Prism) {
  (function (Prism) {
      var keywords =
        /\b(?:abstype|and|andalso|as|case|datatype|do|else|end|eqtype|exception|fn|fun|functor|handle|if|in|include|infix|infixr|let|local|nonfix|of|op|open|orelse|raise|rec|sharing|sig|signature|struct|structure|then|type|val|where|while|with|withtype)\b/i;
      Prism.languages.sml = {
        // allow one level of nesting
        comment:
          /\(\*(?:[^*(]|\*(?!\))|\((?!\*)|\(\*(?:[^*(]|\*(?!\))|\((?!\*))*\*\))*\*\)/,
        string: {
          pattern: /#?"(?:[^"\\]|\\.)*"/,
          greedy: true
        },
        'class-name': [
          {
            // This is only an approximation since the real grammar is context-free
            //
            // Why the main loop so complex?
            // The main loop is approximately the same as /(?:\s*(?:[*,]|->)\s*<TERMINAL>)*/ which is, obviously, a lot
            // simpler. The difference is that if a comma is the last iteration of the loop, then the terminal must be
            // followed by a long identifier.
            pattern: RegExp(
              /((?:^|[^:]):\s*)<TERMINAL>(?:\s*(?:(?:\*|->)\s*<TERMINAL>|,\s*<TERMINAL>(?:(?=<NOT-LAST>)|(?!<NOT-LAST>)\s+<LONG-ID>)))*/.source
                .replace(/<NOT-LAST>/g, function () {
                  return /\s*(?:[*,]|->)/.source
                })
                .replace(/<TERMINAL>/g, function () {
                  return /(?:'[\w']*|<LONG-ID>|\((?:[^()]|\([^()]*\))*\)|\{(?:[^{}]|\{[^{}]*\})*\})(?:\s+<LONG-ID>)*/
                    .source
                })
                .replace(/<LONG-ID>/g, function () {
                  return /(?!<KEYWORD>)[a-z\d_][\w'.]*/.source
                })
                .replace(/<KEYWORD>/g, function () {
                  return keywords.source
                }),
              'i'
            ),
            lookbehind: true,
            greedy: true,
            inside: null // see below
          },
          {
            pattern:
              /((?:^|[^\w'])(?:datatype|exception|functor|signature|structure|type)\s+)[a-z_][\w'.]*/i,
            lookbehind: true
          }
        ],
        function: {
          pattern: /((?:^|[^\w'])fun\s+)[a-z_][\w'.]*/i,
          lookbehind: true
        },
        keyword: keywords,
        variable: {
          pattern: /(^|[^\w'])'[\w']*/,
          lookbehind: true
        },
        number: /~?\b(?:\d+(?:\.\d+)?(?:e~?\d+)?|0x[\da-f]+)\b/i,
        word: {
          pattern: /\b0w(?:\d+|x[\da-f]+)\b/i,
          alias: 'constant'
        },
        boolean: /\b(?:false|true)\b/i,
        operator: /\.\.\.|:[>=:]|=>?|->|[<>]=?|[!+\-*/^#|@~]/,
        punctuation: /[(){}\[\].:,;]/
      };
      Prism.languages.sml['class-name'][0].inside = Prism.languages.sml;
      Prism.languages.smlnj = Prism.languages.sml;
    })(Prism);
  }

  // @ts-nocheck
  solidity.displayName = 'solidity';
  solidity.aliases = ['sol'];

  /** @type {import('../core.js').Syntax} */
  function solidity(Prism) {
    Prism.register(clike);
    Prism.languages.solidity = Prism.languages.extend('clike', {
      'class-name': {
        pattern:
          /(\b(?:contract|enum|interface|library|new|struct|using)\s+)(?!\d)[\w$]+/,
        lookbehind: true
      },
      keyword:
        /\b(?:_|anonymous|as|assembly|assert|break|calldata|case|constant|constructor|continue|contract|default|delete|do|else|emit|enum|event|external|for|from|function|if|import|indexed|inherited|interface|internal|is|let|library|mapping|memory|modifier|new|payable|pragma|private|public|pure|require|returns?|revert|selfdestruct|solidity|storage|struct|suicide|switch|this|throw|using|var|view|while)\b/,
      operator: /=>|->|:=|=:|\*\*|\+\+|--|\|\||&&|<<=?|>>=?|[-+*/%^&|<>!=]=?|[~?]/
    });
    Prism.languages.insertBefore('solidity', 'keyword', {
      builtin:
        /\b(?:address|bool|byte|u?int(?:8|16|24|32|40|48|56|64|72|80|88|96|104|112|120|128|136|144|152|160|168|176|184|192|200|208|216|224|232|240|248|256)?|string|bytes(?:[1-9]|[12]\d|3[0-2])?)\b/
    });
    Prism.languages.insertBefore('solidity', 'number', {
      version: {
        pattern: /([<>]=?|\^)\d+\.\d+\.\d+\b/,
        lookbehind: true,
        alias: 'number'
      }
    });
    Prism.languages.sol = Prism.languages.solidity;
  }

  // @ts-nocheck
  solutionFile.displayName = 'solution-file';
  solutionFile.aliases = ['sln'];

  /** @type {import('../core.js').Syntax} */
  function solutionFile(Prism) {
  (function (Prism) {
      var guid = {
        // https://en.wikipedia.org/wiki/Universally_unique_identifier#Format
        pattern: /\{[\da-f]{8}-[\da-f]{4}-[\da-f]{4}-[\da-f]{4}-[\da-f]{12}\}/i,
        alias: 'constant',
        inside: {
          punctuation: /[{}]/
        }
      };
      Prism.languages['solution-file'] = {
        comment: {
          pattern: /#.*/,
          greedy: true
        },
        string: {
          pattern: /"[^"\r\n]*"|'[^'\r\n]*'/,
          greedy: true,
          inside: {
            guid: guid
          }
        },
        object: {
          // Foo
          //   Bar("abs") = 9
          //   EndBar
          //   Prop = TRUE
          // EndFoo
          pattern:
            /^([ \t]*)(?:([A-Z]\w*)\b(?=.*(?:\r\n?|\n)(?:\1[ \t].*(?:\r\n?|\n))*\1End\2(?=[ \t]*$))|End[A-Z]\w*(?=[ \t]*$))/m,
          lookbehind: true,
          greedy: true,
          alias: 'keyword'
        },
        property: {
          pattern: /^([ \t]*)(?!\s)[^\r\n"#=()]*[^\s"#=()](?=\s*=)/m,
          lookbehind: true,
          inside: {
            guid: guid
          }
        },
        guid: guid,
        number: /\b\d+(?:\.\d+)*\b/,
        boolean: /\b(?:FALSE|TRUE)\b/,
        operator: /=/,
        punctuation: /[(),]/
      };
      Prism.languages['sln'] = Prism.languages['solution-file'];
    })(Prism);
  }

  // @ts-nocheck
  soy.displayName = 'soy';
  soy.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function soy(Prism) {
    Prism.register(markupTemplating)
    ;(function (Prism) {
      var stringPattern = /(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/;
      var numberPattern = /\b\d+(?:\.\d+)?(?:[eE][+-]?\d+)?\b|\b0x[\dA-F]+\b/;
      Prism.languages.soy = {
        comment: [
          /\/\*[\s\S]*?\*\//,
          {
            pattern: /(\s)\/\/.*/,
            lookbehind: true,
            greedy: true
          }
        ],
        'command-arg': {
          pattern:
            /(\{+\/?\s*(?:alias|call|delcall|delpackage|deltemplate|namespace|template)\s+)\.?[\w.]+/,
          lookbehind: true,
          alias: 'string',
          inside: {
            punctuation: /\./
          }
        },
        parameter: {
          pattern: /(\{+\/?\s*@?param\??\s+)\.?[\w.]+/,
          lookbehind: true,
          alias: 'variable'
        },
        keyword: [
          {
            pattern:
              /(\{+\/?[^\S\r\n]*)(?:\\[nrt]|alias|call|case|css|default|delcall|delpackage|deltemplate|else(?:if)?|fallbackmsg|for(?:each)?|if(?:empty)?|lb|let|literal|msg|namespace|nil|@?param\??|rb|sp|switch|template|xid)/,
            lookbehind: true
          },
          /\b(?:any|as|attributes|bool|css|float|html|in|int|js|list|map|null|number|string|uri)\b/
        ],
        delimiter: {
          pattern: /^\{+\/?|\/?\}+$/,
          alias: 'punctuation'
        },
        property: /\w+(?==)/,
        variable: {
          pattern: /\$[^\W\d]\w*(?:\??(?:\.\w+|\[[^\]]+\]))*/,
          inside: {
            string: {
              pattern: stringPattern,
              greedy: true
            },
            number: numberPattern,
            punctuation: /[\[\].?]/
          }
        },
        string: {
          pattern: stringPattern,
          greedy: true
        },
        function: [
          /\w+(?=\()/,
          {
            pattern: /(\|[^\S\r\n]*)\w+/,
            lookbehind: true
          }
        ],
        boolean: /\b(?:false|true)\b/,
        number: numberPattern,
        operator: /\?:?|<=?|>=?|==?|!=|[+*/%-]|\b(?:and|not|or)\b/,
        punctuation: /[{}()\[\]|.,:]/
      }; // Tokenize all inline Soy expressions
      Prism.hooks.add('before-tokenize', function (env) {
        var soyPattern = /\{\{.+?\}\}|\{.+?\}|\s\/\/.*|\/\*[\s\S]*?\*\//g;
        var soyLitteralStart = '{literal}';
        var soyLitteralEnd = '{/literal}';
        var soyLitteralMode = false;
        Prism.languages['markup-templating'].buildPlaceholders(
          env,
          'soy',
          soyPattern,
          function (match) {
            // Soy tags inside {literal} block are ignored
            if (match === soyLitteralEnd) {
              soyLitteralMode = false;
            }
            if (!soyLitteralMode) {
              if (match === soyLitteralStart) {
                soyLitteralMode = true;
              }
              return true
            }
            return false
          }
        );
      }); // Re-insert the tokens after tokenizing
      Prism.hooks.add('after-tokenize', function (env) {
        Prism.languages['markup-templating'].tokenizePlaceholders(env, 'soy');
      });
    })(Prism);
  }

  // @ts-nocheck
  turtle.displayName = 'turtle';
  turtle.aliases = ['trig'];

  /** @type {import('../core.js').Syntax} */
  function turtle(Prism) {
    Prism.languages.turtle = {
      comment: {
        pattern: /#.*/,
        greedy: true
      },
      'multiline-string': {
        pattern:
          /"""(?:(?:""?)?(?:[^"\\]|\\.))*"""|'''(?:(?:''?)?(?:[^'\\]|\\.))*'''/,
        greedy: true,
        alias: 'string',
        inside: {
          comment: /#.*/
        }
      },
      string: {
        pattern: /"(?:[^\\"\r\n]|\\.)*"|'(?:[^\\'\r\n]|\\.)*'/,
        greedy: true
      },
      url: {
        pattern:
          /<(?:[^\x00-\x20<>"{}|^`\\]|\\(?:u[\da-fA-F]{4}|U[\da-fA-F]{8}))*>/,
        greedy: true,
        inside: {
          punctuation: /[<>]/
        }
      },
      function: {
        pattern:
          /(?:(?![-.\d\xB7])[-.\w\xB7\xC0-\uFFFD]+)?:(?:(?![-.])(?:[-.:\w\xC0-\uFFFD]|%[\da-f]{2}|\\.)+)?/i,
        inside: {
          'local-name': {
            pattern: /([^:]*:)[\s\S]+/,
            lookbehind: true
          },
          prefix: {
            pattern: /[\s\S]+/,
            inside: {
              punctuation: /:/
            }
          }
        }
      },
      number: /[+-]?\b\d+(?:\.\d*)?(?:e[+-]?\d+)?/i,
      punctuation: /[{}.,;()[\]]|\^\^/,
      boolean: /\b(?:false|true)\b/,
      keyword: [/(?:\ba|@prefix|@base)\b|=/, /\b(?:base|graph|prefix)\b/i],
      tag: {
        pattern: /@[a-z]+(?:-[a-z\d]+)*/i,
        inside: {
          punctuation: /@/
        }
      }
    };
    Prism.languages.trig = Prism.languages['turtle'];
  }

  // @ts-nocheck
  sparql.displayName = 'sparql';
  sparql.aliases = ['rq'];

  /** @type {import('../core.js').Syntax} */
  function sparql(Prism) {
    Prism.register(turtle);
    Prism.languages.sparql = Prism.languages.extend('turtle', {
      boolean: /\b(?:false|true)\b/i,
      variable: {
        pattern: /[?$]\w+/,
        greedy: true
      }
    });
    Prism.languages.insertBefore('sparql', 'punctuation', {
      keyword: [
        /\b(?:A|ADD|ALL|AS|ASC|ASK|BNODE|BY|CLEAR|CONSTRUCT|COPY|CREATE|DATA|DEFAULT|DELETE|DESC|DESCRIBE|DISTINCT|DROP|EXISTS|FILTER|FROM|GROUP|HAVING|INSERT|INTO|LIMIT|LOAD|MINUS|MOVE|NAMED|NOT|NOW|OFFSET|OPTIONAL|ORDER|RAND|REDUCED|SELECT|SEPARATOR|SERVICE|SILENT|STRUUID|UNION|USING|UUID|VALUES|WHERE)\b/i,
        /\b(?:ABS|AVG|BIND|BOUND|CEIL|COALESCE|CONCAT|CONTAINS|COUNT|DATATYPE|DAY|ENCODE_FOR_URI|FLOOR|GROUP_CONCAT|HOURS|IF|IRI|isBLANK|isIRI|isLITERAL|isNUMERIC|isURI|LANG|LANGMATCHES|LCASE|MAX|MD5|MIN|MINUTES|MONTH|REGEX|REPLACE|ROUND|sameTerm|SAMPLE|SECONDS|SHA1|SHA256|SHA384|SHA512|STR|STRAFTER|STRBEFORE|STRDT|STRENDS|STRLANG|STRLEN|STRSTARTS|SUBSTR|SUM|TIMEZONE|TZ|UCASE|URI|YEAR)\b(?=\s*\()/i,
        /\b(?:BASE|GRAPH|PREFIX)\b/i
      ]
    });
    Prism.languages.rq = Prism.languages.sparql;
  }

  // @ts-nocheck
  splunkSpl.displayName = 'splunk-spl';
  splunkSpl.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function splunkSpl(Prism) {
    Prism.languages['splunk-spl'] = {
      comment: /`comment\("(?:\\.|[^\\"])*"\)`/,
      string: {
        pattern: /"(?:\\.|[^\\"])*"/,
        greedy: true
      },
      // https://docs.splunk.com/Documentation/Splunk/7.3.0/SearchReference/ListOfSearchCommands
      keyword:
        /\b(?:abstract|accum|addcoltotals|addinfo|addtotals|analyzefields|anomalies|anomalousvalue|anomalydetection|append|appendcols|appendcsv|appendlookup|appendpipe|arules|associate|audit|autoregress|bin|bucket|bucketdir|chart|cluster|cofilter|collect|concurrency|contingency|convert|correlate|datamodel|dbinspect|dedup|delete|delta|diff|erex|eval|eventcount|eventstats|extract|fieldformat|fields|fieldsummary|filldown|fillnull|findtypes|folderize|foreach|format|from|gauge|gentimes|geom|geomfilter|geostats|head|highlight|history|iconify|input|inputcsv|inputlookup|iplocation|join|kmeans|kv|kvform|loadjob|localize|localop|lookup|makecontinuous|makemv|makeresults|map|mcollect|metadata|metasearch|meventcollect|mstats|multikv|multisearch|mvcombine|mvexpand|nomv|outlier|outputcsv|outputlookup|outputtext|overlap|pivot|predict|rangemap|rare|regex|relevancy|reltime|rename|replace|rest|return|reverse|rex|rtorder|run|savedsearch|script|scrub|search|searchtxn|selfjoin|sendemail|set|setfields|sichart|sirare|sistats|sitimechart|sitop|sort|spath|stats|strcat|streamstats|table|tags|tail|timechart|timewrap|top|transaction|transpose|trendline|tscollect|tstats|typeahead|typelearner|typer|union|uniq|untable|where|x11|xmlkv|xmlunescape|xpath|xyseries)\b/i,
      'operator-word': {
        pattern: /\b(?:and|as|by|not|or|xor)\b/i,
        alias: 'operator'
      },
      function: /\b\w+(?=\s*\()/,
      property: /\b\w+(?=\s*=(?!=))/,
      date: {
        // MM/DD/YYYY(:HH:MM:SS)?
        pattern: /\b\d{1,2}\/\d{1,2}\/\d{1,4}(?:(?::\d{1,2}){3})?\b/,
        alias: 'number'
      },
      number: /\b\d+(?:\.\d+)?\b/,
      boolean: /\b(?:f|false|t|true)\b/i,
      operator: /[<>=]=?|[-+*/%|]/,
      punctuation: /[()[\],]/
    };
  }

  // @ts-nocheck
  sqf.displayName = 'sqf';
  sqf.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function sqf(Prism) {
    Prism.register(clike);
    Prism.languages.sqf = Prism.languages.extend('clike', {
      string: {
        pattern: /"(?:(?:"")?[^"])*"(?!")|'(?:[^'])*'/,
        greedy: true
      },
      keyword:
        /\b(?:breakOut|breakTo|call|case|catch|default|do|echo|else|execFSM|execVM|exitWith|for|forEach|forEachMember|forEachMemberAgent|forEachMemberTeam|from|goto|if|nil|preprocessFile|preprocessFileLineNumbers|private|scopeName|spawn|step|switch|then|throw|to|try|while|with)\b/i,
      boolean: /\b(?:false|true)\b/i,
      function:
        /\b(?:abs|accTime|acos|action|actionIDs|actionKeys|actionKeysImages|actionKeysNames|actionKeysNamesArray|actionName|actionParams|activateAddons|activatedAddons|activateKey|add3DENConnection|add3DENEventHandler|add3DENLayer|addAction|addBackpack|addBackpackCargo|addBackpackCargoGlobal|addBackpackGlobal|addCamShake|addCuratorAddons|addCuratorCameraArea|addCuratorEditableObjects|addCuratorEditingArea|addCuratorPoints|addEditorObject|addEventHandler|addForce|addForceGeneratorRTD|addGoggles|addGroupIcon|addHandgunItem|addHeadgear|addItem|addItemCargo|addItemCargoGlobal|addItemPool|addItemToBackpack|addItemToUniform|addItemToVest|addLiveStats|addMagazine|addMagazineAmmoCargo|addMagazineCargo|addMagazineCargoGlobal|addMagazineGlobal|addMagazinePool|addMagazines|addMagazineTurret|addMenu|addMenuItem|addMissionEventHandler|addMPEventHandler|addMusicEventHandler|addOwnedMine|addPlayerScores|addPrimaryWeaponItem|addPublicVariableEventHandler|addRating|addResources|addScore|addScoreSide|addSecondaryWeaponItem|addSwitchableUnit|addTeamMember|addToRemainsCollector|addTorque|addUniform|addVehicle|addVest|addWaypoint|addWeapon|addWeaponCargo|addWeaponCargoGlobal|addWeaponGlobal|addWeaponItem|addWeaponPool|addWeaponTurret|admin|agent|agents|AGLToASL|aimedAtTarget|aimPos|airDensityCurveRTD|airDensityRTD|airplaneThrottle|airportSide|AISFinishHeal|alive|all3DENEntities|allAirports|allControls|allCurators|allCutLayers|allDead|allDeadMen|allDisplays|allGroups|allMapMarkers|allMines|allMissionObjects|allow3DMode|allowCrewInImmobile|allowCuratorLogicIgnoreAreas|allowDamage|allowDammage|allowFileOperations|allowFleeing|allowGetIn|allowSprint|allPlayers|allSimpleObjects|allSites|allTurrets|allUnits|allUnitsUAV|allVariables|ammo|ammoOnPylon|animate|animateBay|animateDoor|animatePylon|animateSource|animationNames|animationPhase|animationSourcePhase|animationState|append|apply|armoryPoints|arrayIntersect|asin|ASLToAGL|ASLToATL|assert|assignAsCargo|assignAsCargoIndex|assignAsCommander|assignAsDriver|assignAsGunner|assignAsTurret|assignCurator|assignedCargo|assignedCommander|assignedDriver|assignedGunner|assignedItems|assignedTarget|assignedTeam|assignedVehicle|assignedVehicleRole|assignItem|assignTeam|assignToAirport|atan|atan2|atg|ATLToASL|attachedObject|attachedObjects|attachedTo|attachObject|attachTo|attackEnabled|backpack|backpackCargo|backpackContainer|backpackItems|backpackMagazines|backpackSpaceFor|behaviour|benchmark|binocular|blufor|boundingBox|boundingBoxReal|boundingCenter|briefingName|buildingExit|buildingPos|buldozer_EnableRoadDiag|buldozer_IsEnabledRoadDiag|buldozer_LoadNewRoads|buldozer_reloadOperMap|buttonAction|buttonSetAction|cadetMode|callExtension|camCommand|camCommit|camCommitPrepared|camCommitted|camConstuctionSetParams|camCreate|camDestroy|cameraEffect|cameraEffectEnableHUD|cameraInterest|cameraOn|cameraView|campaignConfigFile|camPreload|camPreloaded|camPrepareBank|camPrepareDir|camPrepareDive|camPrepareFocus|camPrepareFov|camPrepareFovRange|camPreparePos|camPrepareRelPos|camPrepareTarget|camSetBank|camSetDir|camSetDive|camSetFocus|camSetFov|camSetFovRange|camSetPos|camSetRelPos|camSetTarget|camTarget|camUseNVG|canAdd|canAddItemToBackpack|canAddItemToUniform|canAddItemToVest|cancelSimpleTaskDestination|canFire|canMove|canSlingLoad|canStand|canSuspend|canTriggerDynamicSimulation|canUnloadInCombat|canVehicleCargo|captive|captiveNum|cbChecked|cbSetChecked|ceil|channelEnabled|cheatsEnabled|checkAIFeature|checkVisibility|civilian|className|clear3DENAttribute|clear3DENInventory|clearAllItemsFromBackpack|clearBackpackCargo|clearBackpackCargoGlobal|clearForcesRTD|clearGroupIcons|clearItemCargo|clearItemCargoGlobal|clearItemPool|clearMagazineCargo|clearMagazineCargoGlobal|clearMagazinePool|clearOverlay|clearRadio|clearVehicleInit|clearWeaponCargo|clearWeaponCargoGlobal|clearWeaponPool|clientOwner|closeDialog|closeDisplay|closeOverlay|collapseObjectTree|collect3DENHistory|collectiveRTD|combatMode|commandArtilleryFire|commandChat|commander|commandFire|commandFollow|commandFSM|commandGetOut|commandingMenu|commandMove|commandRadio|commandStop|commandSuppressiveFire|commandTarget|commandWatch|comment|commitOverlay|compile|compileFinal|completedFSM|composeText|configClasses|configFile|configHierarchy|configName|configNull|configProperties|configSourceAddonList|configSourceMod|configSourceModList|confirmSensorTarget|connectTerminalToUAV|controlNull|controlsGroupCtrl|copyFromClipboard|copyToClipboard|copyWaypoints|cos|count|countEnemy|countFriendly|countSide|countType|countUnknown|create3DENComposition|create3DENEntity|createAgent|createCenter|createDialog|createDiaryLink|createDiaryRecord|createDiarySubject|createDisplay|createGearDialog|createGroup|createGuardedPoint|createLocation|createMarker|createMarkerLocal|createMenu|createMine|createMissionDisplay|createMPCampaignDisplay|createSimpleObject|createSimpleTask|createSite|createSoundSource|createTask|createTeam|createTrigger|createUnit|createVehicle|createVehicleCrew|createVehicleLocal|crew|ctAddHeader|ctAddRow|ctClear|ctCurSel|ctData|ctFindHeaderRows|ctFindRowHeader|ctHeaderControls|ctHeaderCount|ctRemoveHeaders|ctRemoveRows|ctrlActivate|ctrlAddEventHandler|ctrlAngle|ctrlAutoScrollDelay|ctrlAutoScrollRewind|ctrlAutoScrollSpeed|ctrlChecked|ctrlClassName|ctrlCommit|ctrlCommitted|ctrlCreate|ctrlDelete|ctrlEnable|ctrlEnabled|ctrlFade|ctrlHTMLLoaded|ctrlIDC|ctrlIDD|ctrlMapAnimAdd|ctrlMapAnimClear|ctrlMapAnimCommit|ctrlMapAnimDone|ctrlMapCursor|ctrlMapMouseOver|ctrlMapScale|ctrlMapScreenToWorld|ctrlMapWorldToScreen|ctrlModel|ctrlModelDirAndUp|ctrlModelScale|ctrlParent|ctrlParentControlsGroup|ctrlPosition|ctrlRemoveAllEventHandlers|ctrlRemoveEventHandler|ctrlScale|ctrlSetActiveColor|ctrlSetAngle|ctrlSetAutoScrollDelay|ctrlSetAutoScrollRewind|ctrlSetAutoScrollSpeed|ctrlSetBackgroundColor|ctrlSetChecked|ctrlSetDisabledColor|ctrlSetEventHandler|ctrlSetFade|ctrlSetFocus|ctrlSetFont|ctrlSetFontH1|ctrlSetFontH1B|ctrlSetFontH2|ctrlSetFontH2B|ctrlSetFontH3|ctrlSetFontH3B|ctrlSetFontH4|ctrlSetFontH4B|ctrlSetFontH5|ctrlSetFontH5B|ctrlSetFontH6|ctrlSetFontH6B|ctrlSetFontHeight|ctrlSetFontHeightH1|ctrlSetFontHeightH2|ctrlSetFontHeightH3|ctrlSetFontHeightH4|ctrlSetFontHeightH5|ctrlSetFontHeightH6|ctrlSetFontHeightSecondary|ctrlSetFontP|ctrlSetFontPB|ctrlSetFontSecondary|ctrlSetForegroundColor|ctrlSetModel|ctrlSetModelDirAndUp|ctrlSetModelScale|ctrlSetPixelPrecision|ctrlSetPosition|ctrlSetScale|ctrlSetStructuredText|ctrlSetText|ctrlSetTextColor|ctrlSetTextColorSecondary|ctrlSetTextSecondary|ctrlSetTooltip|ctrlSetTooltipColorBox|ctrlSetTooltipColorShade|ctrlSetTooltipColorText|ctrlShow|ctrlShown|ctrlText|ctrlTextHeight|ctrlTextSecondary|ctrlTextWidth|ctrlType|ctrlVisible|ctRowControls|ctRowCount|ctSetCurSel|ctSetData|ctSetHeaderTemplate|ctSetRowTemplate|ctSetValue|ctValue|curatorAddons|curatorCamera|curatorCameraArea|curatorCameraAreaCeiling|curatorCoef|curatorEditableObjects|curatorEditingArea|curatorEditingAreaType|curatorMouseOver|curatorPoints|curatorRegisteredObjects|curatorSelected|curatorWaypointCost|current3DENOperation|currentChannel|currentCommand|currentMagazine|currentMagazineDetail|currentMagazineDetailTurret|currentMagazineTurret|currentMuzzle|currentNamespace|currentTask|currentTasks|currentThrowable|currentVisionMode|currentWaypoint|currentWeapon|currentWeaponMode|currentWeaponTurret|currentZeroing|cursorObject|cursorTarget|customChat|customRadio|cutFadeOut|cutObj|cutRsc|cutText|damage|date|dateToNumber|daytime|deActivateKey|debriefingText|debugFSM|debugLog|deg|delete3DENEntities|deleteAt|deleteCenter|deleteCollection|deleteEditorObject|deleteGroup|deleteGroupWhenEmpty|deleteIdentity|deleteLocation|deleteMarker|deleteMarkerLocal|deleteRange|deleteResources|deleteSite|deleteStatus|deleteTeam|deleteVehicle|deleteVehicleCrew|deleteWaypoint|detach|detectedMines|diag_activeMissionFSMs|diag_activeScripts|diag_activeSQFScripts|diag_activeSQSScripts|diag_captureFrame|diag_captureFrameToFile|diag_captureSlowFrame|diag_codePerformance|diag_drawMode|diag_dynamicSimulationEnd|diag_enable|diag_enabled|diag_fps|diag_fpsMin|diag_frameNo|diag_lightNewLoad|diag_list|diag_log|diag_logSlowFrame|diag_mergeConfigFile|diag_recordTurretLimits|diag_setLightNew|diag_tickTime|diag_toggle|dialog|diarySubjectExists|didJIP|didJIPOwner|difficulty|difficultyEnabled|difficultyEnabledRTD|difficultyOption|direction|directSay|disableAI|disableCollisionWith|disableConversation|disableDebriefingStats|disableMapIndicators|disableNVGEquipment|disableRemoteSensors|disableSerialization|disableTIEquipment|disableUAVConnectability|disableUserInput|displayAddEventHandler|displayCtrl|displayNull|displayParent|displayRemoveAllEventHandlers|displayRemoveEventHandler|displaySetEventHandler|dissolveTeam|distance|distance2D|distanceSqr|distributionRegion|do3DENAction|doArtilleryFire|doFire|doFollow|doFSM|doGetOut|doMove|doorPhase|doStop|doSuppressiveFire|doTarget|doWatch|drawArrow|drawEllipse|drawIcon|drawIcon3D|drawLine|drawLine3D|drawLink|drawLocation|drawPolygon|drawRectangle|drawTriangle|driver|drop|dynamicSimulationDistance|dynamicSimulationDistanceCoef|dynamicSimulationEnabled|dynamicSimulationSystemEnabled|east|edit3DENMissionAttributes|editObject|editorSetEventHandler|effectiveCommander|emptyPositions|enableAI|enableAIFeature|enableAimPrecision|enableAttack|enableAudioFeature|enableAutoStartUpRTD|enableAutoTrimRTD|enableCamShake|enableCaustics|enableChannel|enableCollisionWith|enableCopilot|enableDebriefingStats|enableDiagLegend|enableDynamicSimulation|enableDynamicSimulationSystem|enableEndDialog|enableEngineArtillery|enableEnvironment|enableFatigue|enableGunLights|enableInfoPanelComponent|enableIRLasers|enableMimics|enablePersonTurret|enableRadio|enableReload|enableRopeAttach|enableSatNormalOnDetail|enableSaving|enableSentences|enableSimulation|enableSimulationGlobal|enableStamina|enableStressDamage|enableTeamSwitch|enableTraffic|enableUAVConnectability|enableUAVWaypoints|enableVehicleCargo|enableVehicleSensor|enableWeaponDisassembly|endl|endLoadingScreen|endMission|engineOn|enginesIsOnRTD|enginesPowerRTD|enginesRpmRTD|enginesTorqueRTD|entities|environmentEnabled|estimatedEndServerTime|estimatedTimeLeft|evalObjectArgument|everyBackpack|everyContainer|exec|execEditorScript|exp|expectedDestination|exportJIPMessages|eyeDirection|eyePos|face|faction|fadeMusic|fadeRadio|fadeSound|fadeSpeech|failMission|fillWeaponsFromPool|find|findCover|findDisplay|findEditorObject|findEmptyPosition|findEmptyPositionReady|findIf|findNearestEnemy|finishMissionInit|finite|fire|fireAtTarget|firstBackpack|flag|flagAnimationPhase|flagOwner|flagSide|flagTexture|fleeing|floor|flyInHeight|flyInHeightASL|fog|fogForecast|fogParams|forceAddUniform|forceAtPositionRTD|forcedMap|forceEnd|forceFlagTexture|forceFollowRoad|forceGeneratorRTD|forceMap|forceRespawn|forceSpeed|forceWalk|forceWeaponFire|forceWeatherChange|forgetTarget|format|formation|formationDirection|formationLeader|formationMembers|formationPosition|formationTask|formatText|formLeader|freeLook|fromEditor|fuel|fullCrew|gearIDCAmmoCount|gearSlotAmmoCount|gearSlotData|get3DENActionState|get3DENAttribute|get3DENCamera|get3DENConnections|get3DENEntity|get3DENEntityID|get3DENGrid|get3DENIconsVisible|get3DENLayerEntities|get3DENLinesVisible|get3DENMissionAttribute|get3DENMouseOver|get3DENSelected|getAimingCoef|getAllEnvSoundControllers|getAllHitPointsDamage|getAllOwnedMines|getAllSoundControllers|getAmmoCargo|getAnimAimPrecision|getAnimSpeedCoef|getArray|getArtilleryAmmo|getArtilleryComputerSettings|getArtilleryETA|getAssignedCuratorLogic|getAssignedCuratorUnit|getBackpackCargo|getBleedingRemaining|getBurningValue|getCameraViewDirection|getCargoIndex|getCenterOfMass|getClientState|getClientStateNumber|getCompatiblePylonMagazines|getConnectedUAV|getContainerMaxLoad|getCursorObjectParams|getCustomAimCoef|getDammage|getDescription|getDir|getDirVisual|getDLCAssetsUsage|getDLCAssetsUsageByName|getDLCs|getDLCUsageTime|getEditorCamera|getEditorMode|getEditorObjectScope|getElevationOffset|getEngineTargetRpmRTD|getEnvSoundController|getFatigue|getFieldManualStartPage|getForcedFlagTexture|getFriend|getFSMVariable|getFuelCargo|getGroupIcon|getGroupIconParams|getGroupIcons|getHideFrom|getHit|getHitIndex|getHitPointDamage|getItemCargo|getMagazineCargo|getMarkerColor|getMarkerPos|getMarkerSize|getMarkerType|getMass|getMissionConfig|getMissionConfigValue|getMissionDLCs|getMissionLayerEntities|getMissionLayers|getModelInfo|getMousePosition|getMusicPlayedTime|getNumber|getObjectArgument|getObjectChildren|getObjectDLC|getObjectMaterials|getObjectProxy|getObjectTextures|getObjectType|getObjectViewDistance|getOxygenRemaining|getPersonUsedDLCs|getPilotCameraDirection|getPilotCameraPosition|getPilotCameraRotation|getPilotCameraTarget|getPlateNumber|getPlayerChannel|getPlayerScores|getPlayerUID|getPlayerUIDOld|getPos|getPosASL|getPosASLVisual|getPosASLW|getPosATL|getPosATLVisual|getPosVisual|getPosWorld|getPylonMagazines|getRelDir|getRelPos|getRemoteSensorsDisabled|getRepairCargo|getResolution|getRotorBrakeRTD|getShadowDistance|getShotParents|getSlingLoad|getSoundController|getSoundControllerResult|getSpeed|getStamina|getStatValue|getSuppression|getTerrainGrid|getTerrainHeightASL|getText|getTotalDLCUsageTime|getTrimOffsetRTD|getUnitLoadout|getUnitTrait|getUserMFDText|getUserMFDValue|getVariable|getVehicleCargo|getWeaponCargo|getWeaponSway|getWingsOrientationRTD|getWingsPositionRTD|getWPPos|glanceAt|globalChat|globalRadio|goggles|group|groupChat|groupFromNetId|groupIconSelectable|groupIconsVisible|groupId|groupOwner|groupRadio|groupSelectedUnits|groupSelectUnit|grpNull|gunner|gusts|halt|handgunItems|handgunMagazine|handgunWeapon|handsHit|hasInterface|hasPilotCamera|hasWeapon|hcAllGroups|hcGroupParams|hcLeader|hcRemoveAllGroups|hcRemoveGroup|hcSelected|hcSelectGroup|hcSetGroup|hcShowBar|hcShownBar|headgear|hideBody|hideObject|hideObjectGlobal|hideSelection|hint|hintC|hintCadet|hintSilent|hmd|hostMission|htmlLoad|HUDMovementLevels|humidity|image|importAllGroups|importance|in|inArea|inAreaArray|incapacitatedState|independent|inflame|inflamed|infoPanel|infoPanelComponentEnabled|infoPanelComponents|infoPanels|inGameUISetEventHandler|inheritsFrom|initAmbientLife|inPolygon|inputAction|inRangeOfArtillery|insertEditorObject|intersect|is3DEN|is3DENMultiplayer|isAbleToBreathe|isAgent|isAimPrecisionEnabled|isArray|isAutoHoverOn|isAutonomous|isAutoStartUpEnabledRTD|isAutotest|isAutoTrimOnRTD|isBleeding|isBurning|isClass|isCollisionLightOn|isCopilotEnabled|isDamageAllowed|isDedicated|isDLCAvailable|isEngineOn|isEqualTo|isEqualType|isEqualTypeAll|isEqualTypeAny|isEqualTypeArray|isEqualTypeParams|isFilePatchingEnabled|isFlashlightOn|isFlatEmpty|isForcedWalk|isFormationLeader|isGroupDeletedWhenEmpty|isHidden|isInRemainsCollector|isInstructorFigureEnabled|isIRLaserOn|isKeyActive|isKindOf|isLaserOn|isLightOn|isLocalized|isManualFire|isMarkedForCollection|isMultiplayer|isMultiplayerSolo|isNil|isNull|isNumber|isObjectHidden|isObjectRTD|isOnRoad|isPipEnabled|isPlayer|isRealTime|isRemoteExecuted|isRemoteExecutedJIP|isServer|isShowing3DIcons|isSimpleObject|isSprintAllowed|isStaminaEnabled|isSteamMission|isStreamFriendlyUIEnabled|isStressDamageEnabled|isText|isTouchingGround|isTurnedOut|isTutHintsEnabled|isUAVConnectable|isUAVConnected|isUIContext|isUniformAllowed|isVehicleCargo|isVehicleRadarOn|isVehicleSensorEnabled|isWalking|isWeaponDeployed|isWeaponRested|itemCargo|items|itemsWithMagazines|join|joinAs|joinAsSilent|joinSilent|joinString|kbAddDatabase|kbAddDatabaseTargets|kbAddTopic|kbHasTopic|kbReact|kbRemoveTopic|kbTell|kbWasSaid|keyImage|keyName|knowsAbout|land|landAt|landResult|language|laserTarget|lbAdd|lbClear|lbColor|lbColorRight|lbCurSel|lbData|lbDelete|lbIsSelected|lbPicture|lbPictureRight|lbSelection|lbSetColor|lbSetColorRight|lbSetCurSel|lbSetData|lbSetPicture|lbSetPictureColor|lbSetPictureColorDisabled|lbSetPictureColorSelected|lbSetPictureRight|lbSetPictureRightColor|lbSetPictureRightColorDisabled|lbSetPictureRightColorSelected|lbSetSelectColor|lbSetSelectColorRight|lbSetSelected|lbSetText|lbSetTextRight|lbSetTooltip|lbSetValue|lbSize|lbSort|lbSortByValue|lbText|lbTextRight|lbValue|leader|leaderboardDeInit|leaderboardGetRows|leaderboardInit|leaderboardRequestRowsFriends|leaderboardRequestRowsGlobal|leaderboardRequestRowsGlobalAroundUser|leaderboardsRequestUploadScore|leaderboardsRequestUploadScoreKeepBest|leaderboardState|leaveVehicle|libraryCredits|libraryDisclaimers|lifeState|lightAttachObject|lightDetachObject|lightIsOn|lightnings|limitSpeed|linearConversion|lineBreak|lineIntersects|lineIntersectsObjs|lineIntersectsSurfaces|lineIntersectsWith|linkItem|list|listObjects|listRemoteTargets|listVehicleSensors|ln|lnbAddArray|lnbAddColumn|lnbAddRow|lnbClear|lnbColor|lnbColorRight|lnbCurSelRow|lnbData|lnbDeleteColumn|lnbDeleteRow|lnbGetColumnsPosition|lnbPicture|lnbPictureRight|lnbSetColor|lnbSetColorRight|lnbSetColumnsPos|lnbSetCurSelRow|lnbSetData|lnbSetPicture|lnbSetPictureColor|lnbSetPictureColorRight|lnbSetPictureColorSelected|lnbSetPictureColorSelectedRight|lnbSetPictureRight|lnbSetText|lnbSetTextRight|lnbSetValue|lnbSize|lnbSort|lnbSortByValue|lnbText|lnbTextRight|lnbValue|load|loadAbs|loadBackpack|loadFile|loadGame|loadIdentity|loadMagazine|loadOverlay|loadStatus|loadUniform|loadVest|local|localize|locationNull|locationPosition|lock|lockCameraTo|lockCargo|lockDriver|locked|lockedCargo|lockedDriver|lockedTurret|lockIdentity|lockTurret|lockWP|log|logEntities|logNetwork|logNetworkTerminate|lookAt|lookAtPos|magazineCargo|magazines|magazinesAllTurrets|magazinesAmmo|magazinesAmmoCargo|magazinesAmmoFull|magazinesDetail|magazinesDetailBackpack|magazinesDetailUniform|magazinesDetailVest|magazinesTurret|magazineTurretAmmo|mapAnimAdd|mapAnimClear|mapAnimCommit|mapAnimDone|mapCenterOnCamera|mapGridPosition|markAsFinishedOnSteam|markerAlpha|markerBrush|markerColor|markerDir|markerPos|markerShape|markerSize|markerText|markerType|max|members|menuAction|menuAdd|menuChecked|menuClear|menuCollapse|menuData|menuDelete|menuEnable|menuEnabled|menuExpand|menuHover|menuPicture|menuSetAction|menuSetCheck|menuSetData|menuSetPicture|menuSetValue|menuShortcut|menuShortcutText|menuSize|menuSort|menuText|menuURL|menuValue|min|mineActive|mineDetectedBy|missionConfigFile|missionDifficulty|missionName|missionNamespace|missionStart|missionVersion|modelToWorld|modelToWorldVisual|modelToWorldVisualWorld|modelToWorldWorld|modParams|moonIntensity|moonPhase|morale|move|move3DENCamera|moveInAny|moveInCargo|moveInCommander|moveInDriver|moveInGunner|moveInTurret|moveObjectToEnd|moveOut|moveTime|moveTo|moveToCompleted|moveToFailed|musicVolume|name|nameSound|nearEntities|nearestBuilding|nearestLocation|nearestLocations|nearestLocationWithDubbing|nearestObject|nearestObjects|nearestTerrainObjects|nearObjects|nearObjectsReady|nearRoads|nearSupplies|nearTargets|needReload|netId|netObjNull|newOverlay|nextMenuItemIndex|nextWeatherChange|nMenuItems|numberOfEnginesRTD|numberToDate|objectCurators|objectFromNetId|objectParent|objNull|objStatus|onBriefingGear|onBriefingGroup|onBriefingNotes|onBriefingPlan|onBriefingTeamSwitch|onCommandModeChanged|onDoubleClick|onEachFrame|onGroupIconClick|onGroupIconOverEnter|onGroupIconOverLeave|onHCGroupSelectionChanged|onMapSingleClick|onPlayerConnected|onPlayerDisconnected|onPreloadFinished|onPreloadStarted|onShowNewObject|onTeamSwitch|openCuratorInterface|openDLCPage|openDSInterface|openMap|openSteamApp|openYoutubeVideo|opfor|orderGetIn|overcast|overcastForecast|owner|param|params|parseNumber|parseSimpleArray|parseText|parsingNamespace|particlesQuality|pi|pickWeaponPool|pitch|pixelGrid|pixelGridBase|pixelGridNoUIScale|pixelH|pixelW|playableSlotsNumber|playableUnits|playAction|playActionNow|player|playerRespawnTime|playerSide|playersNumber|playGesture|playMission|playMove|playMoveNow|playMusic|playScriptedMission|playSound|playSound3D|position|positionCameraToWorld|posScreenToWorld|posWorldToScreen|ppEffectAdjust|ppEffectCommit|ppEffectCommitted|ppEffectCreate|ppEffectDestroy|ppEffectEnable|ppEffectEnabled|ppEffectForceInNVG|precision|preloadCamera|preloadObject|preloadSound|preloadTitleObj|preloadTitleRsc|primaryWeapon|primaryWeaponItems|primaryWeaponMagazine|priority|processDiaryLink|processInitCommands|productVersion|profileName|profileNamespace|profileNameSteam|progressLoadingScreen|progressPosition|progressSetPosition|publicVariable|publicVariableClient|publicVariableServer|pushBack|pushBackUnique|putWeaponPool|queryItemsPool|queryMagazinePool|queryWeaponPool|rad|radioChannelAdd|radioChannelCreate|radioChannelRemove|radioChannelSetCallSign|radioChannelSetLabel|radioVolume|rain|rainbow|random|rank|rankId|rating|rectangular|registeredTasks|registerTask|reload|reloadEnabled|remoteControl|remoteExec|remoteExecCall|remoteExecutedOwner|remove3DENConnection|remove3DENEventHandler|remove3DENLayer|removeAction|removeAll3DENEventHandlers|removeAllActions|removeAllAssignedItems|removeAllContainers|removeAllCuratorAddons|removeAllCuratorCameraAreas|removeAllCuratorEditingAreas|removeAllEventHandlers|removeAllHandgunItems|removeAllItems|removeAllItemsWithMagazines|removeAllMissionEventHandlers|removeAllMPEventHandlers|removeAllMusicEventHandlers|removeAllOwnedMines|removeAllPrimaryWeaponItems|removeAllWeapons|removeBackpack|removeBackpackGlobal|removeCuratorAddons|removeCuratorCameraArea|removeCuratorEditableObjects|removeCuratorEditingArea|removeDrawIcon|removeDrawLinks|removeEventHandler|removeFromRemainsCollector|removeGoggles|removeGroupIcon|removeHandgunItem|removeHeadgear|removeItem|removeItemFromBackpack|removeItemFromUniform|removeItemFromVest|removeItems|removeMagazine|removeMagazineGlobal|removeMagazines|removeMagazinesTurret|removeMagazineTurret|removeMenuItem|removeMissionEventHandler|removeMPEventHandler|removeMusicEventHandler|removeOwnedMine|removePrimaryWeaponItem|removeSecondaryWeaponItem|removeSimpleTask|removeSwitchableUnit|removeTeamMember|removeUniform|removeVest|removeWeapon|removeWeaponAttachmentCargo|removeWeaponCargo|removeWeaponGlobal|removeWeaponTurret|reportRemoteTarget|requiredVersion|resetCamShake|resetSubgroupDirection|resistance|resize|resources|respawnVehicle|restartEditorCamera|reveal|revealMine|reverse|reversedMouseY|roadAt|roadsConnectedTo|roleDescription|ropeAttachedObjects|ropeAttachedTo|ropeAttachEnabled|ropeAttachTo|ropeCreate|ropeCut|ropeDestroy|ropeDetach|ropeEndPosition|ropeLength|ropes|ropeUnwind|ropeUnwound|rotorsForcesRTD|rotorsRpmRTD|round|runInitScript|safeZoneH|safeZoneW|safeZoneWAbs|safeZoneX|safeZoneXAbs|safeZoneY|save3DENInventory|saveGame|saveIdentity|saveJoysticks|saveOverlay|saveProfileNamespace|saveStatus|saveVar|savingEnabled|say|say2D|say3D|score|scoreSide|screenshot|screenToWorld|scriptDone|scriptName|scriptNull|scudState|secondaryWeapon|secondaryWeaponItems|secondaryWeaponMagazine|select|selectBestPlaces|selectDiarySubject|selectedEditorObjects|selectEditorObject|selectionNames|selectionPosition|selectLeader|selectMax|selectMin|selectNoPlayer|selectPlayer|selectRandom|selectRandomWeighted|selectWeapon|selectWeaponTurret|sendAUMessage|sendSimpleCommand|sendTask|sendTaskResult|sendUDPMessage|serverCommand|serverCommandAvailable|serverCommandExecutable|serverName|serverTime|set|set3DENAttribute|set3DENAttributes|set3DENGrid|set3DENIconsVisible|set3DENLayer|set3DENLinesVisible|set3DENLogicType|set3DENMissionAttribute|set3DENMissionAttributes|set3DENModelsVisible|set3DENObjectType|set3DENSelected|setAccTime|setActualCollectiveRTD|setAirplaneThrottle|setAirportSide|setAmmo|setAmmoCargo|setAmmoOnPylon|setAnimSpeedCoef|setAperture|setApertureNew|setArmoryPoints|setAttributes|setAutonomous|setBehaviour|setBleedingRemaining|setBrakesRTD|setCameraInterest|setCamShakeDefParams|setCamShakeParams|setCamUseTI|setCaptive|setCenterOfMass|setCollisionLight|setCombatMode|setCompassOscillation|setConvoySeparation|setCuratorCameraAreaCeiling|setCuratorCoef|setCuratorEditingAreaType|setCuratorWaypointCost|setCurrentChannel|setCurrentTask|setCurrentWaypoint|setCustomAimCoef|setCustomWeightRTD|setDamage|setDammage|setDate|setDebriefingText|setDefaultCamera|setDestination|setDetailMapBlendPars|setDir|setDirection|setDrawIcon|setDriveOnPath|setDropInterval|setDynamicSimulationDistance|setDynamicSimulationDistanceCoef|setEditorMode|setEditorObjectScope|setEffectCondition|setEngineRpmRTD|setFace|setFaceAnimation|setFatigue|setFeatureType|setFlagAnimationPhase|setFlagOwner|setFlagSide|setFlagTexture|setFog|setForceGeneratorRTD|setFormation|setFormationTask|setFormDir|setFriend|setFromEditor|setFSMVariable|setFuel|setFuelCargo|setGroupIcon|setGroupIconParams|setGroupIconsSelectable|setGroupIconsVisible|setGroupId|setGroupIdGlobal|setGroupOwner|setGusts|setHideBehind|setHit|setHitIndex|setHitPointDamage|setHorizonParallaxCoef|setHUDMovementLevels|setIdentity|setImportance|setInfoPanel|setLeader|setLightAmbient|setLightAttenuation|setLightBrightness|setLightColor|setLightDayLight|setLightFlareMaxDistance|setLightFlareSize|setLightIntensity|setLightnings|setLightUseFlare|setLocalWindParams|setMagazineTurretAmmo|setMarkerAlpha|setMarkerAlphaLocal|setMarkerBrush|setMarkerBrushLocal|setMarkerColor|setMarkerColorLocal|setMarkerDir|setMarkerDirLocal|setMarkerPos|setMarkerPosLocal|setMarkerShape|setMarkerShapeLocal|setMarkerSize|setMarkerSizeLocal|setMarkerText|setMarkerTextLocal|setMarkerType|setMarkerTypeLocal|setMass|setMimic|setMousePosition|setMusicEffect|setMusicEventHandler|setName|setNameSound|setObjectArguments|setObjectMaterial|setObjectMaterialGlobal|setObjectProxy|setObjectTexture|setObjectTextureGlobal|setObjectViewDistance|setOvercast|setOwner|setOxygenRemaining|setParticleCircle|setParticleClass|setParticleFire|setParticleParams|setParticleRandom|setPilotCameraDirection|setPilotCameraRotation|setPilotCameraTarget|setPilotLight|setPiPEffect|setPitch|setPlateNumber|setPlayable|setPlayerRespawnTime|setPos|setPosASL|setPosASL2|setPosASLW|setPosATL|setPosition|setPosWorld|setPylonLoadOut|setPylonsPriority|setRadioMsg|setRain|setRainbow|setRandomLip|setRank|setRectangular|setRepairCargo|setRotorBrakeRTD|setShadowDistance|setShotParents|setSide|setSimpleTaskAlwaysVisible|setSimpleTaskCustomData|setSimpleTaskDescription|setSimpleTaskDestination|setSimpleTaskTarget|setSimpleTaskType|setSimulWeatherLayers|setSize|setSkill|setSlingLoad|setSoundEffect|setSpeaker|setSpeech|setSpeedMode|setStamina|setStaminaScheme|setStatValue|setSuppression|setSystemOfUnits|setTargetAge|setTaskMarkerOffset|setTaskResult|setTaskState|setTerrainGrid|setText|setTimeMultiplier|setTitleEffect|setToneMapping|setToneMappingParams|setTrafficDensity|setTrafficDistance|setTrafficGap|setTrafficSpeed|setTriggerActivation|setTriggerArea|setTriggerStatements|setTriggerText|setTriggerTimeout|setTriggerType|setType|setUnconscious|setUnitAbility|setUnitLoadout|setUnitPos|setUnitPosWeak|setUnitRank|setUnitRecoilCoefficient|setUnitTrait|setUnloadInCombat|setUserActionText|setUserMFDText|setUserMFDValue|setVariable|setVectorDir|setVectorDirAndUp|setVectorUp|setVehicleAmmo|setVehicleAmmoDef|setVehicleArmor|setVehicleCargo|setVehicleId|setVehicleInit|setVehicleLock|setVehiclePosition|setVehicleRadar|setVehicleReceiveRemoteTargets|setVehicleReportOwnPosition|setVehicleReportRemoteTargets|setVehicleTIPars|setVehicleVarName|setVelocity|setVelocityModelSpace|setVelocityTransformation|setViewDistance|setVisibleIfTreeCollapsed|setWantedRpmRTD|setWaves|setWaypointBehaviour|setWaypointCombatMode|setWaypointCompletionRadius|setWaypointDescription|setWaypointForceBehaviour|setWaypointFormation|setWaypointHousePosition|setWaypointLoiterRadius|setWaypointLoiterType|setWaypointName|setWaypointPosition|setWaypointScript|setWaypointSpeed|setWaypointStatements|setWaypointTimeout|setWaypointType|setWaypointVisible|setWeaponReloadingTime|setWind|setWindDir|setWindForce|setWindStr|setWingForceScaleRTD|setWPPos|show3DIcons|showChat|showCinemaBorder|showCommandingMenu|showCompass|showCuratorCompass|showGPS|showHUD|showLegend|showMap|shownArtilleryComputer|shownChat|shownCompass|shownCuratorCompass|showNewEditorObject|shownGPS|shownHUD|shownMap|shownPad|shownRadio|shownScoretable|shownUAVFeed|shownWarrant|shownWatch|showPad|showRadio|showScoretable|showSubtitles|showUAVFeed|showWarrant|showWatch|showWaypoint|showWaypoints|side|sideAmbientLife|sideChat|sideEmpty|sideEnemy|sideFriendly|sideLogic|sideRadio|sideUnknown|simpleTasks|simulationEnabled|simulCloudDensity|simulCloudOcclusion|simulInClouds|simulWeatherSync|sin|size|sizeOf|skill|skillFinal|skipTime|sleep|sliderPosition|sliderRange|sliderSetPosition|sliderSetRange|sliderSetSpeed|sliderSpeed|slingLoadAssistantShown|soldierMagazines|someAmmo|sort|soundVolume|speaker|speed|speedMode|splitString|sqrt|squadParams|stance|startLoadingScreen|stop|stopEngineRTD|stopped|str|sunOrMoon|supportInfo|suppressFor|surfaceIsWater|surfaceNormal|surfaceType|swimInDepth|switchableUnits|switchAction|switchCamera|switchGesture|switchLight|switchMove|synchronizedObjects|synchronizedTriggers|synchronizedWaypoints|synchronizeObjectsAdd|synchronizeObjectsRemove|synchronizeTrigger|synchronizeWaypoint|systemChat|systemOfUnits|tan|targetKnowledge|targets|targetsAggregate|targetsQuery|taskAlwaysVisible|taskChildren|taskCompleted|taskCustomData|taskDescription|taskDestination|taskHint|taskMarkerOffset|taskNull|taskParent|taskResult|taskState|taskType|teamMember|teamMemberNull|teamName|teams|teamSwitch|teamSwitchEnabled|teamType|terminate|terrainIntersect|terrainIntersectASL|terrainIntersectAtASL|text|textLog|textLogFormat|tg|time|timeMultiplier|titleCut|titleFadeOut|titleObj|titleRsc|titleText|toArray|toFixed|toLower|toString|toUpper|triggerActivated|triggerActivation|triggerArea|triggerAttachedVehicle|triggerAttachObject|triggerAttachVehicle|triggerDynamicSimulation|triggerStatements|triggerText|triggerTimeout|triggerTimeoutCurrent|triggerType|turretLocal|turretOwner|turretUnit|tvAdd|tvClear|tvCollapse|tvCollapseAll|tvCount|tvCurSel|tvData|tvDelete|tvExpand|tvExpandAll|tvPicture|tvPictureRight|tvSetColor|tvSetCurSel|tvSetData|tvSetPicture|tvSetPictureColor|tvSetPictureColorDisabled|tvSetPictureColorSelected|tvSetPictureRight|tvSetPictureRightColor|tvSetPictureRightColorDisabled|tvSetPictureRightColorSelected|tvSetSelectColor|tvSetText|tvSetTooltip|tvSetValue|tvSort|tvSortByValue|tvText|tvTooltip|tvValue|type|typeName|typeOf|UAVControl|uiNamespace|uiSleep|unassignCurator|unassignItem|unassignTeam|unassignVehicle|underwater|uniform|uniformContainer|uniformItems|uniformMagazines|unitAddons|unitAimPosition|unitAimPositionVisual|unitBackpack|unitIsUAV|unitPos|unitReady|unitRecoilCoefficient|units|unitsBelowHeight|unlinkItem|unlockAchievement|unregisterTask|updateDrawIcon|updateMenuItem|updateObjectTree|useAIOperMapObstructionTest|useAISteeringComponent|useAudioTimeForMoves|userInputDisabled|vectorAdd|vectorCos|vectorCrossProduct|vectorDiff|vectorDir|vectorDirVisual|vectorDistance|vectorDistanceSqr|vectorDotProduct|vectorFromTo|vectorMagnitude|vectorMagnitudeSqr|vectorModelToWorld|vectorModelToWorldVisual|vectorMultiply|vectorNormalized|vectorUp|vectorUpVisual|vectorWorldToModel|vectorWorldToModelVisual|vehicle|vehicleCargoEnabled|vehicleChat|vehicleRadio|vehicleReceiveRemoteTargets|vehicleReportOwnPosition|vehicleReportRemoteTargets|vehicles|vehicleVarName|velocity|velocityModelSpace|verifySignature|vest|vestContainer|vestItems|vestMagazines|viewDistance|visibleCompass|visibleGPS|visibleMap|visiblePosition|visiblePositionASL|visibleScoretable|visibleWatch|waitUntil|waves|waypointAttachedObject|waypointAttachedVehicle|waypointAttachObject|waypointAttachVehicle|waypointBehaviour|waypointCombatMode|waypointCompletionRadius|waypointDescription|waypointForceBehaviour|waypointFormation|waypointHousePosition|waypointLoiterRadius|waypointLoiterType|waypointName|waypointPosition|waypoints|waypointScript|waypointsEnabledUAV|waypointShow|waypointSpeed|waypointStatements|waypointTimeout|waypointTimeoutCurrent|waypointType|waypointVisible|weaponAccessories|weaponAccessoriesCargo|weaponCargo|weaponDirection|weaponInertia|weaponLowered|weapons|weaponsItems|weaponsItemsCargo|weaponState|weaponsTurret|weightRTD|west|WFSideText|wind|windDir|windRTD|windStr|wingsForcesRTD|worldName|worldSize|worldToModel|worldToModelVisual|worldToScreen)\b/i,
      number:
        /(?:\$|\b0x)[\da-f]+\b|(?:\B\.\d+|\b\d+(?:\.\d+)?)(?:e[+-]?\d+)?\b/i,
      operator: /##|>>|&&|\|\||[!=<>]=?|[-+*/%#^]|\b(?:and|mod|not|or)\b/i,
      'magic-variable': {
        pattern:
          /\b(?:this|thisList|thisTrigger|_exception|_fnc_scriptName|_fnc_scriptNameParent|_forEachIndex|_this|_thisEventHandler|_thisFSM|_thisScript|_x)\b/i,
        alias: 'keyword'
      },
      constant: /\bDIK(?:_[a-z\d]+)+\b/i
    });
    Prism.languages.insertBefore('sqf', 'string', {
      macro: {
        pattern: /(^[ \t]*)#[a-z](?:[^\r\n\\]|\\(?:\r\n|[\s\S]))*/im,
        lookbehind: true,
        greedy: true,
        alias: 'property',
        inside: {
          directive: {
            pattern: /#[a-z]+\b/i,
            alias: 'keyword'
          },
          comment: Prism.languages.sqf.comment
        }
      }
    });
    delete Prism.languages.sqf['class-name'];
  }

  // @ts-nocheck
  squirrel.displayName = 'squirrel';
  squirrel.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function squirrel(Prism) {
    Prism.register(clike);
    Prism.languages.squirrel = Prism.languages.extend('clike', {
      comment: [
        Prism.languages.clike['comment'][0],
        {
          pattern: /(^|[^\\:])(?:\/\/|#).*/,
          lookbehind: true,
          greedy: true
        }
      ],
      string: {
        pattern: /(^|[^\\"'@])(?:@"(?:[^"]|"")*"(?!")|"(?:[^\\\r\n"]|\\.)*")/,
        lookbehind: true,
        greedy: true
      },
      'class-name': {
        pattern: /(\b(?:class|enum|extends|instanceof)\s+)\w+(?:\.\w+)*/,
        lookbehind: true,
        inside: {
          punctuation: /\./
        }
      },
      keyword:
        /\b(?:__FILE__|__LINE__|base|break|case|catch|class|clone|const|constructor|continue|default|delete|else|enum|extends|for|foreach|function|if|in|instanceof|local|null|resume|return|static|switch|this|throw|try|typeof|while|yield)\b/,
      number: /\b(?:0x[0-9a-fA-F]+|\d+(?:\.(?:\d+|[eE][+-]?\d+))?)\b/,
      operator: /\+\+|--|<=>|<[-<]|>>>?|&&?|\|\|?|[-+*/%!=<>]=?|[~^]|::?/,
      punctuation: /[(){}\[\],;.]/
    });
    Prism.languages.insertBefore('squirrel', 'string', {
      char: {
        pattern: /(^|[^\\"'])'(?:[^\\']|\\(?:[xuU][0-9a-fA-F]{0,8}|[\s\S]))'/,
        lookbehind: true,
        greedy: true
      }
    });
    Prism.languages.insertBefore('squirrel', 'operator', {
      'attribute-punctuation': {
        pattern: /<\/|\/>/,
        alias: 'important'
      },
      lambda: {
        pattern: /@(?=\()/,
        alias: 'operator'
      }
    });
  }

  // @ts-nocheck
  stan.displayName = 'stan';
  stan.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function stan(Prism) {
  (function (Prism) {
      // https://mc-stan.org/docs/2_28/reference-manual/bnf-grammars.html
      var higherOrderFunctions =
        /\b(?:algebra_solver|algebra_solver_newton|integrate_1d|integrate_ode|integrate_ode_bdf|integrate_ode_rk45|map_rect|ode_(?:adams|bdf|ckrk|rk45)(?:_tol)?|ode_adjoint_tol_ctl|reduce_sum|reduce_sum_static)\b/;
      Prism.languages.stan = {
        comment: /\/\/.*|\/\*[\s\S]*?\*\/|#(?!include).*/,
        string: {
          // String literals can contain spaces and any printable ASCII characters except for " and \
          // https://mc-stan.org/docs/2_24/reference-manual/print-statements-section.html#string-literals
          pattern: /"[\x20\x21\x23-\x5B\x5D-\x7E]*"/,
          greedy: true
        },
        directive: {
          pattern: /^([ \t]*)#include\b.*/m,
          lookbehind: true,
          alias: 'property'
        },
        'function-arg': {
          pattern: RegExp(
            '(' +
              higherOrderFunctions.source +
              /\s*\(\s*/.source +
              ')' +
              /[a-zA-Z]\w*/.source
          ),
          lookbehind: true,
          alias: 'function'
        },
        constraint: {
          pattern: /(\b(?:int|matrix|real|row_vector|vector)\s*)<[^<>]*>/,
          lookbehind: true,
          inside: {
            expression: {
              pattern: /(=\s*)\S(?:\S|\s+(?!\s))*?(?=\s*(?:>$|,\s*\w+\s*=))/,
              lookbehind: true,
              inside: null // see below
            },
            property: /\b[a-z]\w*(?=\s*=)/i,
            operator: /=/,
            punctuation: /^<|>$|,/
          }
        },
        keyword: [
          {
            pattern:
              /\bdata(?=\s*\{)|\b(?:functions|generated|model|parameters|quantities|transformed)\b/,
            alias: 'program-block'
          },
          /\b(?:array|break|cholesky_factor_corr|cholesky_factor_cov|complex|continue|corr_matrix|cov_matrix|data|else|for|if|in|increment_log_prob|int|matrix|ordered|positive_ordered|print|real|reject|return|row_vector|simplex|target|unit_vector|vector|void|while)\b/, // these are functions that are known to take another function as their first argument.
          higherOrderFunctions
        ],
        function: /\b[a-z]\w*(?=\s*\()/i,
        number:
          /(?:\b\d+(?:_\d+)*(?:\.(?:\d+(?:_\d+)*)?)?|\B\.\d+(?:_\d+)*)(?:E[+-]?\d+(?:_\d+)*)?i?(?!\w)/i,
        boolean: /\b(?:false|true)\b/,
        operator: /<-|\.[*/]=?|\|\|?|&&|[!=<>+\-*/]=?|['^%~?:]/,
        punctuation: /[()\[\]{},;]/
      };
      Prism.languages.stan.constraint.inside.expression.inside =
        Prism.languages.stan;
    })(Prism);
  }

  // @ts-nocheck
  iecst.displayName = 'iecst';
  iecst.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function iecst(Prism) {
    Prism.languages.iecst = {
      comment: [
        {
          pattern:
            /(^|[^\\])(?:\/\*[\s\S]*?(?:\*\/|$)|\(\*[\s\S]*?(?:\*\)|$)|\{[\s\S]*?(?:\}|$))/,
          lookbehind: true,
          greedy: true
        },
        {
          pattern: /(^|[^\\:])\/\/.*/,
          lookbehind: true,
          greedy: true
        }
      ],
      string: {
        pattern: /(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,
        greedy: true
      },
      keyword: [
        /\b(?:END_)?(?:PROGRAM|CONFIGURATION|INTERFACE|FUNCTION_BLOCK|FUNCTION|ACTION|TRANSITION|TYPE|STRUCT|(?:INITIAL_)?STEP|NAMESPACE|LIBRARY|CHANNEL|FOLDER|RESOURCE|VAR_(?:ACCESS|CONFIG|EXTERNAL|GLOBAL|INPUT|IN_OUT|OUTPUT|TEMP)|VAR|METHOD|PROPERTY)\b/i,
        /\b(?:AT|BY|(?:END_)?(?:CASE|FOR|IF|REPEAT|WHILE)|CONSTANT|CONTINUE|DO|ELSE|ELSIF|EXIT|EXTENDS|FROM|GET|GOTO|IMPLEMENTS|JMP|NON_RETAIN|OF|PRIVATE|PROTECTED|PUBLIC|RETAIN|RETURN|SET|TASK|THEN|TO|UNTIL|USING|WITH|__CATCH|__ENDTRY|__FINALLY|__TRY)\b/
      ],
      'class-name':
        /\b(?:ANY|ARRAY|BOOL|BYTE|U?(?:D|L|S)?INT|(?:D|L)?WORD|DATE(?:_AND_TIME)?|DT|L?REAL|POINTER|STRING|TIME(?:_OF_DAY)?|TOD)\b/,
      address: {
        pattern: /%[IQM][XBWDL][\d.]*|%[IQ][\d.]*/,
        alias: 'symbol'
      },
      number:
        /\b(?:16#[\da-f]+|2#[01_]+|0x[\da-f]+)\b|\b(?:D|DT|T|TOD)#[\d_shmd:]*|\b[A-Z]*#[\d.,_]*|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?/i,
      boolean: /\b(?:FALSE|NULL|TRUE)\b/,
      operator:
        /S?R?:?=>?|&&?|\*\*?|<[=>]?|>=?|[-:^/+#]|\b(?:AND|EQ|EXPT|GE|GT|LE|LT|MOD|NE|NOT|OR|XOR)\b/,
      function: /\b[a-z_]\w*(?=\s*\()/i,
      punctuation: /[()[\].,;]/
    };
  }

  // @ts-nocheck
  systemd.displayName = 'systemd';
  systemd.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function systemd(Prism) {
  (function (Prism) {
      var comment = {
        pattern: /^[;#].*/m,
        greedy: true
      };
      var quotesSource = /"(?:[^\r\n"\\]|\\(?:[^\r]|\r\n?))*"(?!\S)/.source;
      Prism.languages.systemd = {
        comment: comment,
        section: {
          pattern: /^\[[^\n\r\[\]]*\](?=[ \t]*$)/m,
          greedy: true,
          inside: {
            punctuation: /^\[|\]$/,
            'section-name': {
              pattern: /[\s\S]+/,
              alias: 'selector'
            }
          }
        },
        key: {
          pattern: /^[^\s=]+(?=[ \t]*=)/m,
          greedy: true,
          alias: 'attr-name'
        },
        value: {
          // This pattern is quite complex because of two properties:
          //  1) Quotes (strings) must be preceded by a space. Since we can't use lookbehinds, we have to "resolve"
          //     the lookbehind. You will see this in the main loop where spaces are handled separately.
          //  2) Line continuations.
          //     After line continuations, empty lines and comments are ignored so we have to consume them.
          pattern: RegExp(
            /(=[ \t]*(?!\s))/.source + // the value either starts with quotes or not
              '(?:' +
              quotesSource +
              '|(?=[^"\r\n]))' + // main loop
              '(?:' +
              (/[^\s\\]/.source + // handle spaces separately because of quotes
                '|' +
                '[ \t]+(?:(?![ \t"])|' +
                quotesSource +
                ')' + // line continuation
                '|' +
                /\\[\r\n]+(?:[#;].*[\r\n]+)*(?![#;])/.source) +
              ')*'
          ),
          lookbehind: true,
          greedy: true,
          alias: 'attr-value',
          inside: {
            comment: comment,
            quoted: {
              pattern: RegExp(/(^|\s)/.source + quotesSource),
              lookbehind: true,
              greedy: true
            },
            punctuation: /\\$/m,
            boolean: {
              pattern: /^(?:false|no|off|on|true|yes)$/,
              greedy: true
            }
          }
        },
        punctuation: /=/
      };
    })(Prism);
  }

  // @ts-nocheck
  t4Templating.displayName = 't4-templating';
  t4Templating.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function t4Templating(Prism) {
  (function (Prism) {
      function createBlock(prefix, inside, contentAlias) {
        return {
          pattern: RegExp('<#' + prefix + '[\\s\\S]*?#>'),
          alias: 'block',
          inside: {
            delimiter: {
              pattern: RegExp('^<#' + prefix + '|#>$'),
              alias: 'important'
            },
            content: {
              pattern: /[\s\S]+/,
              inside: inside,
              alias: contentAlias
            }
          }
        }
      }
      function createT4(insideLang) {
        var grammar = Prism.languages[insideLang];
        var className = 'language-' + insideLang;
        return {
          block: {
            pattern: /<#[\s\S]+?#>/,
            inside: {
              directive: createBlock('@', {
                'attr-value': {
                  pattern: /=(?:("|')(?:\\[\s\S]|(?!\1)[^\\])*\1|[^\s'">=]+)/,
                  inside: {
                    punctuation: /^=|^["']|["']$/
                  }
                },
                keyword: /\b\w+(?=\s)/,
                'attr-name': /\b\w+/
              }),
              expression: createBlock('=', grammar, className),
              'class-feature': createBlock('\\+', grammar, className),
              standard: createBlock('', grammar, className)
            }
          }
        }
      }
      Prism.languages['t4-templating'] = Object.defineProperty({}, 'createT4', {
        value: createT4
      });
    })(Prism);
  }

  // @ts-nocheck
  t4Cs.displayName = 't4-cs';
  t4Cs.aliases = ['t4'];

  /** @type {import('../core.js').Syntax} */
  function t4Cs(Prism) {
    Prism.register(csharp);
    Prism.register(t4Templating);
    Prism.languages.t4 = Prism.languages['t4-cs'] =
      Prism.languages['t4-templating'].createT4('csharp');
  }

  // @ts-nocheck
  t4Vb.displayName = 't4-vb';
  t4Vb.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function t4Vb(Prism) {
    Prism.register(t4Templating);
    Prism.register(vbnet);
    Prism.languages['t4-vb'] = Prism.languages['t4-templating'].createT4('vbnet');
  }

  // @ts-nocheck
  tap.displayName = 'tap';
  tap.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function tap(Prism) {
    Prism.register(yaml);
    // https://en.wikipedia.org/wiki/Test_Anything_Protocol
    Prism.languages.tap = {
      fail: /not ok[^#{\n\r]*/,
      pass: /ok[^#{\n\r]*/,
      pragma: /pragma [+-][a-z]+/,
      bailout: /bail out!.*/i,
      version: /TAP version \d+/i,
      plan: /\b\d+\.\.\d+(?: +#.*)?/,
      subtest: {
        pattern: /# Subtest(?:: .*)?/,
        greedy: true
      },
      punctuation: /[{}]/,
      directive: /#.*/,
      yamlish: {
        pattern: /(^[ \t]*)---[\s\S]*?[\r\n][ \t]*\.\.\.$/m,
        lookbehind: true,
        inside: Prism.languages.yaml,
        alias: 'language-yaml'
      }
    };
  }

  // @ts-nocheck
  tcl.displayName = 'tcl';
  tcl.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function tcl(Prism) {
    Prism.languages.tcl = {
      comment: {
        pattern: /(^|[^\\])#.*/,
        lookbehind: true
      },
      string: {
        pattern: /"(?:[^"\\\r\n]|\\(?:\r\n|[\s\S]))*"/,
        greedy: true
      },
      variable: [
        {
          pattern: /(\$)(?:::)?(?:[a-zA-Z0-9]+::)*\w+/,
          lookbehind: true
        },
        {
          pattern: /(\$)\{[^}]+\}/,
          lookbehind: true
        },
        {
          pattern: /(^[\t ]*set[ \t]+)(?:::)?(?:[a-zA-Z0-9]+::)*\w+/m,
          lookbehind: true
        }
      ],
      function: {
        pattern: /(^[\t ]*proc[ \t]+)\S+/m,
        lookbehind: true
      },
      builtin: [
        {
          pattern:
            /(^[\t ]*)(?:break|class|continue|error|eval|exit|for|foreach|if|proc|return|switch|while)\b/m,
          lookbehind: true
        },
        /\b(?:else|elseif)\b/
      ],
      scope: {
        pattern: /(^[\t ]*)(?:global|upvar|variable)\b/m,
        lookbehind: true,
        alias: 'constant'
      },
      keyword: {
        pattern:
          /(^[\t ]*|\[)(?:Safe_Base|Tcl|after|append|apply|array|auto_(?:execok|import|load|mkindex|qualify|reset)|automkindex_old|bgerror|binary|catch|cd|chan|clock|close|concat|dde|dict|encoding|eof|exec|expr|fblocked|fconfigure|fcopy|file(?:event|name)?|flush|gets|glob|history|http|incr|info|interp|join|lappend|lassign|lindex|linsert|list|llength|load|lrange|lrepeat|lreplace|lreverse|lsearch|lset|lsort|math(?:func|op)|memory|msgcat|namespace|open|package|parray|pid|pkg_mkIndex|platform|puts|pwd|re_syntax|read|refchan|regexp|registry|regsub|rename|scan|seek|set|socket|source|split|string|subst|tcl(?:_endOfWord|_findLibrary|startOf(?:Next|Previous)Word|test|vars|wordBreak(?:After|Before))|tell|time|tm|trace|unknown|unload|unset|update|uplevel|vwait)\b/m,
        lookbehind: true
      },
      operator:
        /!=?|\*\*?|==|&&?|\|\|?|<[=<]?|>[=>]?|[-+~\/%?^]|\b(?:eq|in|ne|ni)\b/,
      punctuation: /[{}()\[\]]/
    };
  }

  // @ts-nocheck
  tt2.displayName = 'tt2';
  tt2.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function tt2(Prism) {
    Prism.register(clike);
    Prism.register(markupTemplating)
    ;(function (Prism) {
      Prism.languages.tt2 = Prism.languages.extend('clike', {
        comment: /#.*|\[%#[\s\S]*?%\]/,
        keyword:
          /\b(?:BLOCK|CALL|CASE|CATCH|CLEAR|DEBUG|DEFAULT|ELSE|ELSIF|END|FILTER|FINAL|FOREACH|GET|IF|IN|INCLUDE|INSERT|LAST|MACRO|META|NEXT|PERL|PROCESS|RAWPERL|RETURN|SET|STOP|SWITCH|TAGS|THROW|TRY|UNLESS|USE|WHILE|WRAPPER)\b/,
        punctuation: /[[\]{},()]/
      });
      Prism.languages.insertBefore('tt2', 'number', {
        operator: /=[>=]?|!=?|<=?|>=?|&&|\|\|?|\b(?:and|not|or)\b/,
        variable: {
          pattern: /\b[a-z]\w*(?:\s*\.\s*(?:\d+|\$?[a-z]\w*))*\b/i
        }
      });
      Prism.languages.insertBefore('tt2', 'keyword', {
        delimiter: {
          pattern: /^(?:\[%|%%)-?|-?%\]$/,
          alias: 'punctuation'
        }
      });
      Prism.languages.insertBefore('tt2', 'string', {
        'single-quoted-string': {
          pattern: /'[^\\']*(?:\\[\s\S][^\\']*)*'/,
          greedy: true,
          alias: 'string'
        },
        'double-quoted-string': {
          pattern: /"[^\\"]*(?:\\[\s\S][^\\"]*)*"/,
          greedy: true,
          alias: 'string',
          inside: {
            variable: {
              pattern: /\$(?:[a-z]\w*(?:\.(?:\d+|\$?[a-z]\w*))*)/i
            }
          }
        }
      }); // The different types of TT2 strings "replace" the C-like standard string
      delete Prism.languages.tt2.string;
      Prism.hooks.add('before-tokenize', function (env) {
        var tt2Pattern = /\[%[\s\S]+?%\]/g;
        Prism.languages['markup-templating'].buildPlaceholders(
          env,
          'tt2',
          tt2Pattern
        );
      });
      Prism.hooks.add('after-tokenize', function (env) {
        Prism.languages['markup-templating'].tokenizePlaceholders(env, 'tt2');
      });
    })(Prism);
  }

  // @ts-nocheck
  toml.displayName = 'toml';
  toml.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function toml(Prism) {
  (function (Prism) {
      var key = /(?:[\w-]+|'[^'\n\r]*'|"(?:\\.|[^\\"\r\n])*")/.source;
      /**
       * @param {string} pattern
       */
      function insertKey(pattern) {
        return pattern.replace(/__/g, function () {
          return key
        })
      }
      Prism.languages.toml = {
        comment: {
          pattern: /#.*/,
          greedy: true
        },
        table: {
          pattern: RegExp(
            insertKey(
              /(^[\t ]*\[\s*(?:\[\s*)?)__(?:\s*\.\s*__)*(?=\s*\])/.source
            ),
            'm'
          ),
          lookbehind: true,
          greedy: true,
          alias: 'class-name'
        },
        key: {
          pattern: RegExp(
            insertKey(/(^[\t ]*|[{,]\s*)__(?:\s*\.\s*__)*(?=\s*=)/.source),
            'm'
          ),
          lookbehind: true,
          greedy: true,
          alias: 'property'
        },
        string: {
          pattern:
            /"""(?:\\[\s\S]|[^\\])*?"""|'''[\s\S]*?'''|'[^'\n\r]*'|"(?:\\.|[^\\"\r\n])*"/,
          greedy: true
        },
        date: [
          {
            // Offset Date-Time, Local Date-Time, Local Date
            pattern:
              /\b\d{4}-\d{2}-\d{2}(?:[T\s]\d{2}:\d{2}:\d{2}(?:\.\d+)?(?:Z|[+-]\d{2}:\d{2})?)?\b/i,
            alias: 'number'
          },
          {
            // Local Time
            pattern: /\b\d{2}:\d{2}:\d{2}(?:\.\d+)?\b/,
            alias: 'number'
          }
        ],
        number:
          /(?:\b0(?:x[\da-zA-Z]+(?:_[\da-zA-Z]+)*|o[0-7]+(?:_[0-7]+)*|b[10]+(?:_[10]+)*))\b|[-+]?\b\d+(?:_\d+)*(?:\.\d+(?:_\d+)*)?(?:[eE][+-]?\d+(?:_\d+)*)?\b|[-+]?\b(?:inf|nan)\b/,
        boolean: /\b(?:false|true)\b/,
        punctuation: /[.,=[\]{}]/
      };
    })(Prism);
  }

  // @ts-nocheck
  tremor.displayName = 'tremor';
  tremor.aliases = ['trickle', 'troy'];

  /** @type {import('../core.js').Syntax} */
  function tremor(Prism) {
  (function (Prism) {
      Prism.languages.tremor = {
        comment: {
          pattern: /(^|[^\\])(?:\/\*[\s\S]*?\*\/|(?:--|\/\/|#).*)/,
          lookbehind: true
        },
        'interpolated-string': null,
        // see below
        extractor: {
          pattern: /\b[a-z_]\w*\|(?:[^\r\n\\|]|\\(?:\r\n|[\s\S]))*\|/i,
          greedy: true,
          inside: {
            regex: {
              pattern: /(^re)\|[\s\S]+/,
              lookbehind: true
            },
            function: /^\w+/,
            value: /\|[\s\S]+/
          }
        },
        identifier: {
          pattern: /`[^`]*`/,
          greedy: true
        },
        function: /\b[a-z_]\w*(?=\s*(?:::\s*<|\())\b/,
        keyword:
          /\b(?:args|as|by|case|config|connect|connector|const|copy|create|default|define|deploy|drop|each|emit|end|erase|event|flow|fn|for|from|group|having|insert|into|intrinsic|let|links|match|merge|mod|move|of|operator|patch|pipeline|recur|script|select|set|sliding|state|stream|to|tumbling|update|use|when|where|window|with)\b/,
        boolean: /\b(?:false|null|true)\b/i,
        number:
          /\b(?:0b[01_]*|0x[0-9a-fA-F_]*|\d[\d_]*(?:\.\d[\d_]*)?(?:[Ee][+-]?[\d_]+)?)\b/,
        'pattern-punctuation': {
          pattern: /%(?=[({[])/,
          alias: 'punctuation'
        },
        operator:
          /[-+*\/%~!^]=?|=[=>]?|&[&=]?|\|[|=]?|<<?=?|>>?>?=?|(?:absent|and|not|or|present|xor)\b/,
        punctuation: /::|[;\[\]()\{\},.:]/
      };
      var interpolationPattern =
        /#\{(?:[^"{}]|\{[^{}]*\}|"(?:[^"\\\r\n]|\\(?:\r\n|[\s\S]))*")*\}/.source;
      Prism.languages.tremor['interpolated-string'] = {
        pattern: RegExp(
          /(^|[^\\])/.source +
            '(?:' +
            '"""(?:' +
            /[^"\\#]|\\[\s\S]|"(?!"")|#(?!\{)/.source +
            '|' +
            interpolationPattern +
            ')*"""' +
            '|' +
            '"(?:' +
            /[^"\\\r\n#]|\\(?:\r\n|[\s\S])|#(?!\{)/.source +
            '|' +
            interpolationPattern +
            ')*"' +
            ')'
        ),
        lookbehind: true,
        greedy: true,
        inside: {
          interpolation: {
            pattern: RegExp(interpolationPattern),
            inside: {
              punctuation: /^#\{|\}$/,
              expression: {
                pattern: /[\s\S]+/,
                inside: Prism.languages.tremor
              }
            }
          },
          string: /[\s\S]+/
        }
      };
      Prism.languages.troy = Prism.languages['tremor'];
      Prism.languages.trickle = Prism.languages['tremor'];
    })(Prism);
  }

  // @ts-nocheck
  typoscript.displayName = 'typoscript';
  typoscript.aliases = ['tsconfig'];

  /** @type {import('../core.js').Syntax} */
  function typoscript(Prism) {
  (function (Prism) {
      var keywords =
        /\b(?:ACT|ACTIFSUB|CARRAY|CASE|CLEARGIF|COA|COA_INT|CONSTANTS|CONTENT|CUR|EDITPANEL|EFFECT|EXT|FILE|FLUIDTEMPLATE|FORM|FRAME|FRAMESET|GIFBUILDER|GMENU|GMENU_FOLDOUT|GMENU_LAYERS|GP|HMENU|HRULER|HTML|IENV|IFSUB|IMAGE|IMGMENU|IMGMENUITEM|IMGTEXT|IMG_RESOURCE|INCLUDE_TYPOSCRIPT|JSMENU|JSMENUITEM|LLL|LOAD_REGISTER|NO|PAGE|RECORDS|RESTORE_REGISTER|TEMPLATE|TEXT|TMENU|TMENUITEM|TMENU_LAYERS|USER|USER_INT|_GIFBUILDER|global|globalString|globalVar)\b/;
      Prism.languages.typoscript = {
        comment: [
          {
            // multiline comments /* */
            pattern: /(^|[^\\])\/\*[\s\S]*?(?:\*\/|$)/,
            lookbehind: true
          },
          {
            // double-slash comments - ignored when backslashes or colon is found in front
            // also ignored whenever directly after an equal-sign, because it would probably be an url without protocol
            pattern: /(^|[^\\:= \t]|(?:^|[^= \t])[ \t]+)\/\/.*/,
            lookbehind: true,
            greedy: true
          },
          {
            // hash comments - ignored when leading quote is found for hex colors in strings
            pattern: /(^|[^"'])#.*/,
            lookbehind: true,
            greedy: true
          }
        ],
        function: [
          {
            // old include style
            pattern:
              /<INCLUDE_TYPOSCRIPT:\s*source\s*=\s*(?:"[^"\r\n]*"|'[^'\r\n]*')\s*>/,
            inside: {
              string: {
                pattern: /"[^"\r\n]*"|'[^'\r\n]*'/,
                inside: {
                  keyword: keywords
                }
              },
              keyword: {
                pattern: /INCLUDE_TYPOSCRIPT/
              }
            }
          },
          {
            // new include style
            pattern: /@import\s*(?:"[^"\r\n]*"|'[^'\r\n]*')/,
            inside: {
              string: /"[^"\r\n]*"|'[^'\r\n]*'/
            }
          }
        ],
        string: {
          pattern: /^([^=]*=[< ]?)(?:(?!\]\n).)*/,
          lookbehind: true,
          inside: {
            function: /\{\$.*\}/,
            // constants include
            keyword: keywords,
            number: /^\d+$/,
            punctuation: /[,|:]/
          }
        },
        keyword: keywords,
        number: {
          // special highlighting for indexes of arrays in tags
          pattern: /\b\d+\s*[.{=]/,
          inside: {
            operator: /[.{=]/
          }
        },
        tag: {
          pattern: /\.?[-\w\\]+\.?/,
          inside: {
            punctuation: /\./
          }
        },
        punctuation: /[{}[\];(),.:|]/,
        operator: /[<>]=?|[!=]=?=?|--?|\+\+?|&&?|\|\|?|[?*/~^%]/
      };
      Prism.languages.tsconfig = Prism.languages.typoscript;
    })(Prism);
  }

  // @ts-nocheck
  unrealscript.displayName = 'unrealscript';
  unrealscript.aliases = ['uc', 'uscript'];

  /** @type {import('../core.js').Syntax} */
  function unrealscript(Prism) {
    Prism.languages.unrealscript = {
      comment: /\/\/.*|\/\*[\s\S]*?\*\//,
      string: {
        pattern: /(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,
        greedy: true
      },
      category: {
        pattern:
          /(\b(?:(?:autoexpand|hide|show)categories|var)\s*\()[^()]+(?=\))/,
        lookbehind: true,
        greedy: true,
        alias: 'property'
      },
      metadata: {
        pattern: /(\w\s*)<\s*\w+\s*=[^<>|=\r\n]+(?:\|\s*\w+\s*=[^<>|=\r\n]+)*>/,
        lookbehind: true,
        greedy: true,
        inside: {
          property: /\b\w+(?=\s*=)/,
          operator: /=/,
          punctuation: /[<>|]/
        }
      },
      macro: {
        pattern: /`\w+/,
        alias: 'property'
      },
      'class-name': {
        pattern:
          /(\b(?:class|enum|extends|interface|state(?:\(\))?|struct|within)\s+)\w+/,
        lookbehind: true
      },
      keyword:
        /\b(?:abstract|actor|array|auto|autoexpandcategories|bool|break|byte|case|class|classgroup|client|coerce|collapsecategories|config|const|continue|default|defaultproperties|delegate|dependson|deprecated|do|dontcollapsecategories|editconst|editinlinenew|else|enum|event|exec|export|extends|final|float|for|forcescriptorder|foreach|function|goto|guid|hidecategories|hidedropdown|if|ignores|implements|inherits|input|int|interface|iterator|latent|local|material|name|native|nativereplication|noexport|nontransient|noteditinlinenew|notplaceable|operator|optional|out|pawn|perobjectconfig|perobjectlocalized|placeable|postoperator|preoperator|private|protected|reliable|replication|return|server|showcategories|simulated|singular|state|static|string|struct|structdefault|structdefaultproperties|switch|texture|transient|travel|unreliable|until|var|vector|while|within)\b/,
      function: /\b[a-z_]\w*(?=\s*\()/i,
      boolean: /\b(?:false|true)\b/,
      number: /\b0x[\da-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?/i,
      // https://docs.unrealengine.com/udk/Three/UnrealScriptExpressions.html
      operator:
        />>|<<|--|\+\+|\*\*|[-+*/~!=<>$@]=?|&&?|\|\|?|\^\^?|[?:%]|\b(?:ClockwiseFrom|Cross|Dot)\b/,
      punctuation: /[()[\]{};,.]/
    };
    Prism.languages.uc = Prism.languages.uscript = Prism.languages.unrealscript;
  }

  // @ts-nocheck
  uorazor.displayName = 'uorazor';
  uorazor.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function uorazor(Prism) {
    Prism.languages.uorazor = {
      'comment-hash': {
        pattern: /#.*/,
        alias: 'comment',
        greedy: true
      },
      'comment-slash': {
        pattern: /\/\/.*/,
        alias: 'comment',
        greedy: true
      },
      string: {
        pattern: /("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,
        inside: {
          punctuation: /^['"]|['"]$/
        },
        greedy: true
      },
      'source-layers': {
        pattern:
          /\b(?:arms|backpack|blue|bracelet|cancel|clear|cloak|criminal|earrings|enemy|facialhair|friend|friendly|gloves|gray|grey|ground|hair|head|innerlegs|innertorso|innocent|lefthand|middletorso|murderer|neck|nonfriendly|onehandedsecondary|outerlegs|outertorso|pants|red|righthand|ring|self|shirt|shoes|talisman|waist)\b/i,
        alias: 'function'
      },
      'source-commands': {
        pattern:
          /\b(?:alliance|attack|cast|clearall|clearignore|clearjournal|clearlist|clearsysmsg|createlist|createtimer|dclick|dclicktype|dclickvar|dress|dressconfig|drop|droprelloc|emote|getlabel|guild|gumpclose|gumpresponse|hotkey|ignore|lasttarget|lift|lifttype|menu|menuresponse|msg|org|organize|organizer|overhead|pause|poplist|potion|promptresponse|pushlist|removelist|removetimer|rename|restock|say|scav|scavenger|script|setability|setlasttarget|setskill|settimer|setvar|sysmsg|target|targetloc|targetrelloc|targettype|undress|unignore|unsetvar|useobject|useonce|useskill|usetype|virtue|wait|waitforgump|waitformenu|waitforprompt|waitforstat|waitforsysmsg|waitfortarget|walk|wfsysmsg|wft|whisper|yell)\b/,
        alias: 'function'
      },
      'tag-name': {
        pattern: /(^\{%-?\s*)\w+/,
        lookbehind: true,
        alias: 'keyword'
      },
      delimiter: {
        pattern: /^\{[{%]-?|-?[%}]\}$/,
        alias: 'punctuation'
      },
      function:
        /\b(?:atlist|close|closest|count|counter|counttype|dead|dex|diffhits|diffmana|diffstam|diffweight|find|findbuff|finddebuff|findlayer|findtype|findtypelist|followers|gumpexists|hidden|hits|hp|hue|human|humanoid|ingump|inlist|insysmessage|insysmsg|int|invul|lhandempty|list|listexists|mana|maxhits|maxhp|maxmana|maxstam|maxweight|monster|mounted|name|next|noto|paralyzed|poisoned|position|prev|previous|queued|rand|random|rhandempty|skill|stam|str|targetexists|timer|timerexists|varexist|warmode|weight)\b/,
      keyword:
        /\b(?:and|as|break|continue|else|elseif|endfor|endif|endwhile|for|if|loop|not|or|replay|stop|while)\b/,
      boolean: /\b(?:false|null|true)\b/,
      number: /\b0x[\dA-Fa-f]+|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee][-+]?\d+)?/,
      operator: [
        {
          pattern:
            /(\s)(?:and|b-and|b-or|b-xor|ends with|in|is|matches|not|or|same as|starts with)(?=\s)/,
          lookbehind: true
        },
        /[=<>]=?|!=|\*\*?|\/\/?|\?:?|[-+~%|]/
      ],
      punctuation: /[()\[\]{}:.,]/
    };
  }

  // @ts-nocheck
  v.displayName = 'v';
  v.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function v(Prism) {
    Prism.register(clike)
    ;(function (Prism) {
      var interpolationExpr = {
        pattern: /[\s\S]+/,
        inside: null
      };
      Prism.languages.v = Prism.languages.extend('clike', {
        string: {
          pattern: /r?(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,
          alias: 'quoted-string',
          greedy: true,
          inside: {
            interpolation: {
              pattern:
                /((?:^|[^\\])(?:\\{2})*)\$(?:\{[^{}]*\}|\w+(?:\.\w+(?:\([^\(\)]*\))?|\[[^\[\]]+\])*)/,
              lookbehind: true,
              inside: {
                'interpolation-variable': {
                  pattern: /^\$\w[\s\S]*$/,
                  alias: 'variable'
                },
                'interpolation-punctuation': {
                  pattern: /^\$\{|\}$/,
                  alias: 'punctuation'
                },
                'interpolation-expression': interpolationExpr
              }
            }
          }
        },
        'class-name': {
          pattern: /(\b(?:enum|interface|struct|type)\s+)(?:C\.)?\w+/,
          lookbehind: true
        },
        keyword:
          /(?:\b(?:__global|as|asm|assert|atomic|break|chan|const|continue|defer|else|embed|enum|fn|for|go(?:to)?|if|import|in|interface|is|lock|match|module|mut|none|or|pub|return|rlock|select|shared|sizeof|static|struct|type(?:of)?|union|unsafe)|\$(?:else|for|if)|#(?:flag|include))\b/,
        number:
          /\b(?:0x[a-f\d]+(?:_[a-f\d]+)*|0b[01]+(?:_[01]+)*|0o[0-7]+(?:_[0-7]+)*|\d+(?:_\d+)*(?:\.\d+(?:_\d+)*)?)\b/i,
        operator:
          /~|\?|[*\/%^!=]=?|\+[=+]?|-[=-]?|\|[=|]?|&(?:=|&|\^=?)?|>(?:>=?|=)?|<(?:<=?|=|-)?|:=|\.\.\.?/,
        builtin:
          /\b(?:any(?:_float|_int)?|bool|byte(?:ptr)?|charptr|f(?:32|64)|i(?:8|16|64|128|nt)|rune|size_t|string|u(?:16|32|64|128)|voidptr)\b/
      });
      interpolationExpr.inside = Prism.languages.v;
      Prism.languages.insertBefore('v', 'string', {
        char: {
          pattern: /`(?:\\`|\\?[^`]{1,2})`/,
          // using {1,2} instead of `u` flag for compatibility
          alias: 'rune'
        }
      });
      Prism.languages.insertBefore('v', 'operator', {
        attribute: {
          pattern:
            /(^[\t ]*)\[(?:deprecated|direct_array_access|flag|inline|live|ref_only|typedef|unsafe_fn|windows_stdcall)\]/m,
          lookbehind: true,
          alias: 'annotation',
          inside: {
            punctuation: /[\[\]]/,
            keyword: /\w+/
          }
        },
        generic: {
          pattern: /<\w+>(?=\s*[\)\{])/,
          inside: {
            punctuation: /[<>]/,
            'class-name': /\w+/
          }
        }
      });
      Prism.languages.insertBefore('v', 'function', {
        'generic-function': {
          // e.g. foo<T>( ...
          pattern: /\b\w+\s*<\w+>(?=\()/,
          inside: {
            function: /^\w+/,
            generic: {
              pattern: /<\w+>/,
              inside: Prism.languages.v.generic.inside
            }
          }
        }
      });
    })(Prism);
  }

  // @ts-nocheck
  vala.displayName = 'vala';
  vala.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function vala(Prism) {
    Prism.register(clike);
    Prism.languages.vala = Prism.languages.extend('clike', {
      // Classes copied from prism-csharp
      'class-name': [
        {
          // (Foo bar, Bar baz)
          pattern: /\b[A-Z]\w*(?:\.\w+)*\b(?=(?:\?\s+|\*?\s+\*?)\w)/,
          inside: {
            punctuation: /\./
          }
        },
        {
          // [Foo]
          pattern: /(\[)[A-Z]\w*(?:\.\w+)*\b/,
          lookbehind: true,
          inside: {
            punctuation: /\./
          }
        },
        {
          // class Foo : Bar
          pattern:
            /(\b(?:class|interface)\s+[A-Z]\w*(?:\.\w+)*\s*:\s*)[A-Z]\w*(?:\.\w+)*\b/,
          lookbehind: true,
          inside: {
            punctuation: /\./
          }
        },
        {
          // class Foo
          pattern:
            /((?:\b(?:class|enum|interface|new|struct)\s+)|(?:catch\s+\())[A-Z]\w*(?:\.\w+)*\b/,
          lookbehind: true,
          inside: {
            punctuation: /\./
          }
        }
      ],
      keyword:
        /\b(?:abstract|as|assert|async|base|bool|break|case|catch|char|class|const|construct|continue|default|delegate|delete|do|double|dynamic|else|ensures|enum|errordomain|extern|finally|float|for|foreach|get|if|in|inline|int|int16|int32|int64|int8|interface|internal|is|lock|long|namespace|new|null|out|override|owned|params|private|protected|public|ref|requires|return|set|short|signal|sizeof|size_t|ssize_t|static|string|struct|switch|this|throw|throws|try|typeof|uchar|uint|uint16|uint32|uint64|uint8|ulong|unichar|unowned|ushort|using|value|var|virtual|void|volatile|weak|while|yield)\b/i,
      function: /\b\w+(?=\s*\()/,
      number:
        /(?:\b0x[\da-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?)(?:f|u?l?)?/i,
      operator:
        /\+\+|--|&&|\|\||<<=?|>>=?|=>|->|~|[+\-*\/%&^|=!<>]=?|\?\??|\.\.\./,
      punctuation: /[{}[\];(),.:]/,
      constant: /\b[A-Z0-9_]+\b/
    });
    Prism.languages.insertBefore('vala', 'string', {
      'raw-string': {
        pattern: /"""[\s\S]*?"""/,
        greedy: true,
        alias: 'string'
      },
      'template-string': {
        pattern: /@"[\s\S]*?"/,
        greedy: true,
        inside: {
          interpolation: {
            pattern: /\$(?:\([^)]*\)|[a-zA-Z]\w*)/,
            inside: {
              delimiter: {
                pattern: /^\$\(?|\)$/,
                alias: 'punctuation'
              },
              rest: Prism.languages.vala
            }
          },
          string: /[\s\S]+/
        }
      }
    });
    Prism.languages.insertBefore('vala', 'keyword', {
      regex: {
        pattern:
          /\/(?:\[(?:[^\]\\\r\n]|\\.)*\]|\\.|[^/\\\[\r\n])+\/[imsx]{0,4}(?=\s*(?:$|[\r\n,.;})\]]))/,
        greedy: true,
        inside: {
          'regex-source': {
            pattern: /^(\/)[\s\S]+(?=\/[a-z]*$)/,
            lookbehind: true,
            alias: 'language-regex',
            inside: Prism.languages.regex
          },
          'regex-delimiter': /^\//,
          'regex-flags': /^[a-z]+$/
        }
      }
    });
  }

  // @ts-nocheck
  velocity.displayName = 'velocity';
  velocity.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function velocity(Prism) {
    Prism.register(markup)
    ;(function (Prism) {
      Prism.languages.velocity = Prism.languages.extend('markup', {});
      var velocity = {
        variable: {
          pattern:
            /(^|[^\\](?:\\\\)*)\$!?(?:[a-z][\w-]*(?:\([^)]*\))?(?:\.[a-z][\w-]*(?:\([^)]*\))?|\[[^\]]+\])*|\{[^}]+\})/i,
          lookbehind: true,
          inside: {} // See below
        },
        string: {
          pattern: /"[^"]*"|'[^']*'/,
          greedy: true
        },
        number: /\b\d+\b/,
        boolean: /\b(?:false|true)\b/,
        operator:
          /[=!<>]=?|[+*/%-]|&&|\|\||\.\.|\b(?:eq|g[et]|l[et]|n(?:e|ot))\b/,
        punctuation: /[(){}[\]:,.]/
      };
      velocity.variable.inside = {
        string: velocity['string'],
        function: {
          pattern: /([^\w-])[a-z][\w-]*(?=\()/,
          lookbehind: true
        },
        number: velocity['number'],
        boolean: velocity['boolean'],
        punctuation: velocity['punctuation']
      };
      Prism.languages.insertBefore('velocity', 'comment', {
        unparsed: {
          pattern: /(^|[^\\])#\[\[[\s\S]*?\]\]#/,
          lookbehind: true,
          greedy: true,
          inside: {
            punctuation: /^#\[\[|\]\]#$/
          }
        },
        'velocity-comment': [
          {
            pattern: /(^|[^\\])#\*[\s\S]*?\*#/,
            lookbehind: true,
            greedy: true,
            alias: 'comment'
          },
          {
            pattern: /(^|[^\\])##.*/,
            lookbehind: true,
            greedy: true,
            alias: 'comment'
          }
        ],
        directive: {
          pattern:
            /(^|[^\\](?:\\\\)*)#@?(?:[a-z][\w-]*|\{[a-z][\w-]*\})(?:\s*\((?:[^()]|\([^()]*\))*\))?/i,
          lookbehind: true,
          inside: {
            keyword: {
              pattern: /^#@?(?:[a-z][\w-]*|\{[a-z][\w-]*\})|\bin\b/,
              inside: {
                punctuation: /[{}]/
              }
            },
            rest: velocity
          }
        },
        variable: velocity['variable']
      });
      Prism.languages.velocity['tag'].inside['attr-value'].inside.rest =
        Prism.languages.velocity;
    })(Prism);
  }

  // @ts-nocheck
  verilog.displayName = 'verilog';
  verilog.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function verilog(Prism) {
    Prism.languages.verilog = {
      comment: {
        pattern: /\/\/.*|\/\*[\s\S]*?\*\//,
        greedy: true
      },
      string: {
        pattern: /"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"/,
        greedy: true
      },
      'kernel-function': {
        // support for any kernel function (ex: $display())
        pattern: /\B\$\w+\b/,
        alias: 'property'
      },
      // support for user defined constants (ex: `define)
      constant: /\B`\w+\b/,
      function: /\b\w+(?=\()/,
      // support for verilog and system verilog keywords
      keyword:
        /\b(?:alias|and|assert|assign|assume|automatic|before|begin|bind|bins|binsof|bit|break|buf|bufif0|bufif1|byte|case|casex|casez|cell|chandle|class|clocking|cmos|config|const|constraint|context|continue|cover|covergroup|coverpoint|cross|deassign|default|defparam|design|disable|dist|do|edge|else|end|endcase|endclass|endclocking|endconfig|endfunction|endgenerate|endgroup|endinterface|endmodule|endpackage|endprimitive|endprogram|endproperty|endsequence|endspecify|endtable|endtask|enum|event|expect|export|extends|extern|final|first_match|for|force|foreach|forever|fork|forkjoin|function|generate|genvar|highz0|highz1|if|iff|ifnone|ignore_bins|illegal_bins|import|incdir|include|initial|inout|input|inside|instance|int|integer|interface|intersect|join|join_any|join_none|large|liblist|library|local|localparam|logic|longint|macromodule|matches|medium|modport|module|nand|negedge|new|nmos|nor|noshowcancelled|not|notif0|notif1|null|or|output|package|packed|parameter|pmos|posedge|primitive|priority|program|property|protected|pull0|pull1|pulldown|pullup|pulsestyle_ondetect|pulsestyle_onevent|pure|rand|randc|randcase|randsequence|rcmos|real|realtime|ref|reg|release|repeat|return|rnmos|rpmos|rtran|rtranif0|rtranif1|scalared|sequence|shortint|shortreal|showcancelled|signed|small|solve|specify|specparam|static|string|strong0|strong1|struct|super|supply0|supply1|table|tagged|task|this|throughout|time|timeprecision|timeunit|tran|tranif0|tranif1|tri|tri0|tri1|triand|trior|trireg|type|typedef|union|unique|unsigned|use|uwire|var|vectored|virtual|void|wait|wait_order|wand|weak0|weak1|while|wildcard|wire|with|within|wor|xnor|xor)\b/,
      // bold highlighting for all verilog and system verilog logic blocks
      important: /\b(?:always|always_comb|always_ff|always_latch)\b(?: *@)?/,
      // support for time ticks, vectors, and real numbers
      number:
        /\B##?\d+|(?:\b\d+)?'[odbh] ?[\da-fzx_?]+|\b(?:\d*[._])?\d+(?:e[-+]?\d+)?/i,
      operator: /[-+{}^~%*\/?=!<>&|]+/,
      punctuation: /[[\];(),.:]/
    };
  }

  // @ts-nocheck
  vhdl.displayName = 'vhdl';
  vhdl.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function vhdl(Prism) {
    Prism.languages.vhdl = {
      comment: /--.+/,
      // support for all logic vectors
      'vhdl-vectors': {
        pattern: /\b[oxb]"[\da-f_]+"|"[01uxzwlh-]+"/i,
        alias: 'number'
      },
      // support for operator overloading included
      'quoted-function': {
        pattern: /"\S+?"(?=\()/,
        alias: 'function'
      },
      string: /"(?:[^\\"\r\n]|\\(?:\r\n|[\s\S]))*"/,
      constant: /\b(?:library|use)\b/i,
      // support for predefined attributes included
      keyword:
        /\b(?:'active|'ascending|'base|'delayed|'driving|'driving_value|'event|'high|'image|'instance_name|'last_active|'last_event|'last_value|'left|'leftof|'length|'low|'path_name|'pos|'pred|'quiet|'range|'reverse_range|'right|'rightof|'simple_name|'stable|'succ|'transaction|'val|'value|access|after|alias|all|architecture|array|assert|attribute|begin|block|body|buffer|bus|case|component|configuration|constant|disconnect|downto|else|elsif|end|entity|exit|file|for|function|generate|generic|group|guarded|if|impure|in|inertial|inout|is|label|library|linkage|literal|loop|map|new|next|null|of|on|open|others|out|package|port|postponed|procedure|process|pure|range|record|register|reject|report|return|select|severity|shared|signal|subtype|then|to|transport|type|unaffected|units|until|use|variable|wait|when|while|with)\b/i,
      boolean: /\b(?:false|true)\b/i,
      function: /\w+(?=\()/,
      // decimal, based, physical, and exponential numbers supported
      number: /'[01uxzwlh-]'|\b(?:\d+#[\da-f_.]+#|\d[\d_.]*)(?:e[-+]?\d+)?/i,
      operator:
        /[<>]=?|:=|[-+*/&=]|\b(?:abs|and|mod|nand|nor|not|or|rem|rol|ror|sla|sll|sra|srl|xnor|xor)\b/i,
      punctuation: /[{}[\];(),.:]/
    };
  }

  // @ts-nocheck
  vim.displayName = 'vim';
  vim.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function vim(Prism) {
    Prism.languages.vim = {
      string: /"(?:[^"\\\r\n]|\\.)*"|'(?:[^'\r\n]|'')*'/,
      comment: /".*/,
      function: /\b\w+(?=\()/,
      keyword:
        /\b(?:N|Next|P|Print|X|XMLent|XMLns|ab|abbreviate|abc|abclear|abo|aboveleft|al|all|ar|arga|argadd|argd|argdelete|argdo|arge|argedit|argg|argglobal|argl|arglocal|args|argu|argument|as|ascii|b|bN|bNext|ba|bad|badd|ball|bd|bdelete|be|bel|belowright|bf|bfirst|bl|blast|bm|bmodified|bn|bnext|bo|botright|bp|bprevious|br|brea|break|breaka|breakadd|breakd|breakdel|breakl|breaklist|brewind|bro|browse|bufdo|buffer|buffers|bun|bunload|bw|bwipeout|c|cN|cNext|cNfcNfile|ca|cabbrev|cabc|cabclear|cad|caddb|caddbuffer|caddexpr|caddf|caddfile|cal|call|cat|catch|cb|cbuffer|cc|ccl|cclose|cd|ce|center|cex|cexpr|cf|cfile|cfir|cfirst|cg|cgetb|cgetbuffer|cgete|cgetexpr|cgetfile|change|changes|chd|chdir|che|checkpath|checkt|checktime|cl|cla|clast|clist|clo|close|cmapc|cmapclear|cn|cnew|cnewer|cnext|cnf|cnfile|cnorea|cnoreabbrev|co|col|colder|colo|colorscheme|comc|comclear|comp|compiler|con|conf|confirm|continue|cope|copen|copy|cp|cpf|cpfile|cprevious|cq|cquit|cr|crewind|cu|cuna|cunabbrev|cunmap|cw|cwindow|d|debugg|debuggreedy|delc|delcommand|delete|delf|delfunction|delm|delmarks|di|diffg|diffget|diffoff|diffpatch|diffpu|diffput|diffsplit|diffthis|diffu|diffupdate|dig|digraphs|display|dj|djump|dl|dlist|dr|drop|ds|dsearch|dsp|dsplit|e|earlier|echoe|echoerr|echom|echomsg|echon|edit|el|else|elsei|elseif|em|emenu|en|endf|endfo|endfor|endfun|endfunction|endif|endt|endtry|endw|endwhile|ene|enew|ex|exi|exit|exu|exusage|f|file|files|filetype|fin|fina|finally|find|fini|finish|fir|first|fix|fixdel|fo|fold|foldc|foldclose|foldd|folddoc|folddoclosed|folddoopen|foldo|foldopen|for|fu|fun|function|go|goto|gr|grep|grepa|grepadd|h|ha|hardcopy|help|helpf|helpfind|helpg|helpgrep|helpt|helptags|hid|hide|his|history|ia|iabbrev|iabc|iabclear|if|ij|ijump|il|ilist|imapc|imapclear|in|inorea|inoreabbrev|isearch|isp|isplit|iu|iuna|iunabbrev|iunmap|j|join|ju|jumps|k|kee|keepalt|keepj|keepjumps|keepmarks|l|lN|lNext|lNf|lNfile|la|lad|laddb|laddbuffer|laddexpr|laddf|laddfile|lan|language|last|later|lb|lbuffer|lc|lcd|lch|lchdir|lcl|lclose|left|lefta|leftabove|let|lex|lexpr|lf|lfile|lfir|lfirst|lg|lgetb|lgetbuffer|lgete|lgetexpr|lgetfile|lgr|lgrep|lgrepa|lgrepadd|lh|lhelpgrep|list|ll|lla|llast|lli|llist|lm|lmak|lmake|lmap|lmapc|lmapclear|ln|lne|lnew|lnewer|lnext|lnf|lnfile|lnoremap|lo|loadview|loc|lockmarks|lockv|lockvar|lol|lolder|lop|lopen|lp|lpf|lpfile|lprevious|lr|lrewind|ls|lt|ltag|lu|lunmap|lv|lvimgrep|lvimgrepa|lvimgrepadd|lw|lwindow|m|ma|mak|make|mark|marks|mat|match|menut|menutranslate|mk|mkexrc|mks|mksession|mksp|mkspell|mkv|mkvie|mkview|mkvimrc|mod|mode|move|mz|mzf|mzfile|mzscheme|n|nbkey|new|next|nmapc|nmapclear|noh|nohlsearch|norea|noreabbrev|nu|number|nun|nunmap|o|omapc|omapclear|on|only|open|opt|options|ou|ounmap|p|pc|pclose|pe|ped|pedit|perl|perld|perldo|po|pop|popu|popup|pp|ppop|pre|preserve|prev|previous|print|prof|profd|profdel|profile|promptf|promptfind|promptr|promptrepl|ps|psearch|ptN|ptNext|pta|ptag|ptf|ptfirst|ptj|ptjump|ptl|ptlast|ptn|ptnext|ptp|ptprevious|ptr|ptrewind|pts|ptselect|pu|put|pw|pwd|py|pyf|pyfile|python|q|qa|qall|quit|quita|quitall|r|read|rec|recover|red|redi|redir|redo|redr|redraw|redraws|redrawstatus|reg|registers|res|resize|ret|retab|retu|return|rew|rewind|ri|right|rightb|rightbelow|ru|rub|ruby|rubyd|rubydo|rubyf|rubyfile|runtime|rv|rviminfo|sN|sNext|sa|sal|sall|san|sandbox|sargument|sav|saveas|sb|sbN|sbNext|sba|sball|sbf|sbfirst|sbl|sblast|sbm|sbmodified|sbn|sbnext|sbp|sbprevious|sbr|sbrewind|sbuffer|scrip|scripte|scriptencoding|scriptnames|se|set|setf|setfiletype|setg|setglobal|setl|setlocal|sf|sfind|sfir|sfirst|sh|shell|sign|sil|silent|sim|simalt|sl|sla|slast|sleep|sm|smagic|smap|smapc|smapclear|sme|smenu|sn|snext|sni|sniff|sno|snomagic|snor|snoremap|snoreme|snoremenu|so|sor|sort|source|sp|spe|spelld|spelldump|spellgood|spelli|spellinfo|spellr|spellrepall|spellu|spellundo|spellw|spellwrong|split|spr|sprevious|sre|srewind|st|sta|stag|star|startg|startgreplace|startinsert|startr|startreplace|stj|stjump|stop|stopi|stopinsert|sts|stselect|sun|sunhide|sunm|sunmap|sus|suspend|sv|sview|syncbind|t|tN|tNext|ta|tab|tabN|tabNext|tabc|tabclose|tabd|tabdo|tabe|tabedit|tabf|tabfind|tabfir|tabfirst|tabl|tablast|tabm|tabmove|tabn|tabnew|tabnext|tabo|tabonly|tabp|tabprevious|tabr|tabrewind|tabs|tag|tags|tc|tcl|tcld|tcldo|tclf|tclfile|te|tearoff|tf|tfirst|th|throw|tj|tjump|tl|tlast|tm|tmenu|tn|tnext|to|topleft|tp|tprevious|tr|trewind|try|ts|tselect|tu|tunmenu|u|una|unabbreviate|undo|undoj|undojoin|undol|undolist|unh|unhide|unlet|unlo|unlockvar|unm|unmap|up|update|ve|verb|verbose|version|vert|vertical|vi|vie|view|vim|vimgrep|vimgrepa|vimgrepadd|visual|viu|viusage|vmapc|vmapclear|vne|vnew|vs|vsplit|vu|vunmap|w|wN|wNext|wa|wall|wh|while|win|winc|wincmd|windo|winp|winpos|winsize|wn|wnext|wp|wprevious|wq|wqa|wqall|write|ws|wsverb|wv|wviminfo|x|xa|xall|xit|xm|xmap|xmapc|xmapclear|xme|xmenu|xn|xnoremap|xnoreme|xnoremenu|xu|xunmap|y|yank)\b/,
      builtin:
        /\b(?:acd|ai|akm|aleph|allowrevins|altkeymap|ambiwidth|ambw|anti|antialias|arab|arabic|arabicshape|ari|arshape|autochdir|autocmd|autoindent|autoread|autowrite|autowriteall|aw|awa|background|backspace|backup|backupcopy|backupdir|backupext|backupskip|balloondelay|ballooneval|balloonexpr|bdir|bdlay|beval|bex|bexpr|bg|bh|bin|binary|biosk|bioskey|bk|bkc|bomb|breakat|brk|browsedir|bs|bsdir|bsk|bt|bufhidden|buflisted|buftype|casemap|ccv|cdpath|cedit|cfu|ch|charconvert|ci|cin|cindent|cink|cinkeys|cino|cinoptions|cinw|cinwords|clipboard|cmdheight|cmdwinheight|cmp|cms|columns|com|comments|commentstring|compatible|complete|completefunc|completeopt|consk|conskey|copyindent|cot|cpo|cpoptions|cpt|cscopepathcomp|cscopeprg|cscopequickfix|cscopetag|cscopetagorder|cscopeverbose|cspc|csprg|csqf|cst|csto|csverb|cuc|cul|cursorcolumn|cursorline|cwh|debug|deco|def|define|delcombine|dex|dg|dict|dictionary|diff|diffexpr|diffopt|digraph|dip|dir|directory|dy|ea|ead|eadirection|eb|ed|edcompatible|ef|efm|ei|ek|enc|encoding|endofline|eol|ep|equalalways|equalprg|errorbells|errorfile|errorformat|esckeys|et|eventignore|expandtab|exrc|fcl|fcs|fdc|fde|fdi|fdl|fdls|fdm|fdn|fdo|fdt|fen|fenc|fencs|fex|ff|ffs|fileencoding|fileencodings|fileformat|fileformats|fillchars|fk|fkmap|flp|fml|fmr|foldcolumn|foldenable|foldexpr|foldignore|foldlevel|foldlevelstart|foldmarker|foldmethod|foldminlines|foldnestmax|foldtext|formatexpr|formatlistpat|formatoptions|formatprg|fp|fs|fsync|ft|gcr|gd|gdefault|gfm|gfn|gfs|gfw|ghr|gp|grepformat|grepprg|gtl|gtt|guicursor|guifont|guifontset|guifontwide|guiheadroom|guioptions|guipty|guitablabel|guitabtooltip|helpfile|helpheight|helplang|hf|hh|hi|hidden|highlight|hk|hkmap|hkmapp|hkp|hl|hlg|hls|hlsearch|ic|icon|iconstring|ignorecase|im|imactivatekey|imak|imc|imcmdline|imd|imdisable|imi|iminsert|ims|imsearch|inc|include|includeexpr|incsearch|inde|indentexpr|indentkeys|indk|inex|inf|infercase|insertmode|invacd|invai|invakm|invallowrevins|invaltkeymap|invanti|invantialias|invar|invarab|invarabic|invarabicshape|invari|invarshape|invautochdir|invautoindent|invautoread|invautowrite|invautowriteall|invaw|invawa|invbackup|invballooneval|invbeval|invbin|invbinary|invbiosk|invbioskey|invbk|invbl|invbomb|invbuflisted|invcf|invci|invcin|invcindent|invcompatible|invconfirm|invconsk|invconskey|invcopyindent|invcp|invcscopetag|invcscopeverbose|invcst|invcsverb|invcuc|invcul|invcursorcolumn|invcursorline|invdeco|invdelcombine|invdg|invdiff|invdigraph|invdisable|invea|inveb|inved|invedcompatible|invek|invendofline|inveol|invequalalways|inverrorbells|invesckeys|invet|invex|invexpandtab|invexrc|invfen|invfk|invfkmap|invfoldenable|invgd|invgdefault|invguipty|invhid|invhidden|invhk|invhkmap|invhkmapp|invhkp|invhls|invhlsearch|invic|invicon|invignorecase|invim|invimc|invimcmdline|invimd|invincsearch|invinf|invinfercase|invinsertmode|invis|invjoinspaces|invjs|invlazyredraw|invlbr|invlinebreak|invlisp|invlist|invloadplugins|invlpl|invlz|invma|invmacatsui|invmagic|invmh|invml|invmod|invmodeline|invmodifiable|invmodified|invmore|invmousef|invmousefocus|invmousehide|invnu|invnumber|invodev|invopendevice|invpaste|invpi|invpreserveindent|invpreviewwindow|invprompt|invpvw|invreadonly|invremap|invrestorescreen|invrevins|invri|invrightleft|invrightleftcmd|invrl|invrlc|invro|invrs|invru|invruler|invsb|invsc|invscb|invscrollbind|invscs|invsecure|invsft|invshellslash|invshelltemp|invshiftround|invshortname|invshowcmd|invshowfulltag|invshowmatch|invshowmode|invsi|invsm|invsmartcase|invsmartindent|invsmarttab|invsmd|invsn|invsol|invspell|invsplitbelow|invsplitright|invspr|invsr|invssl|invsta|invstartofline|invstmp|invswapfile|invswf|invta|invtagbsearch|invtagrelative|invtagstack|invtbi|invtbidi|invtbs|invtermbidi|invterse|invtextauto|invtextmode|invtf|invtgst|invtildeop|invtimeout|invtitle|invto|invtop|invtr|invttimeout|invttybuiltin|invttyfast|invtx|invvb|invvisualbell|invwa|invwarn|invwb|invweirdinvert|invwfh|invwfw|invwildmenu|invwinfixheight|invwinfixwidth|invwiv|invwmnu|invwrap|invwrapscan|invwrite|invwriteany|invwritebackup|invws|isf|isfname|isi|isident|isk|iskeyword|isprint|joinspaces|js|key|keymap|keymodel|keywordprg|km|kmp|kp|langmap|langmenu|laststatus|lazyredraw|lbr|lcs|linebreak|lines|linespace|lisp|lispwords|listchars|loadplugins|lpl|lsp|lz|macatsui|magic|makeef|makeprg|matchpairs|matchtime|maxcombine|maxfuncdepth|maxmapdepth|maxmem|maxmempattern|maxmemtot|mco|mef|menuitems|mfd|mh|mis|mkspellmem|ml|mls|mm|mmd|mmp|mmt|modeline|modelines|modifiable|modified|more|mouse|mousef|mousefocus|mousehide|mousem|mousemodel|mouses|mouseshape|mouset|mousetime|mp|mps|msm|mzq|mzquantum|nf|noacd|noai|noakm|noallowrevins|noaltkeymap|noanti|noantialias|noar|noarab|noarabic|noarabicshape|noari|noarshape|noautochdir|noautoindent|noautoread|noautowrite|noautowriteall|noaw|noawa|nobackup|noballooneval|nobeval|nobin|nobinary|nobiosk|nobioskey|nobk|nobl|nobomb|nobuflisted|nocf|noci|nocin|nocindent|nocompatible|noconfirm|noconsk|noconskey|nocopyindent|nocp|nocscopetag|nocscopeverbose|nocst|nocsverb|nocuc|nocul|nocursorcolumn|nocursorline|nodeco|nodelcombine|nodg|nodiff|nodigraph|nodisable|noea|noeb|noed|noedcompatible|noek|noendofline|noeol|noequalalways|noerrorbells|noesckeys|noet|noex|noexpandtab|noexrc|nofen|nofk|nofkmap|nofoldenable|nogd|nogdefault|noguipty|nohid|nohidden|nohk|nohkmap|nohkmapp|nohkp|nohls|noic|noicon|noignorecase|noim|noimc|noimcmdline|noimd|noincsearch|noinf|noinfercase|noinsertmode|nois|nojoinspaces|nojs|nolazyredraw|nolbr|nolinebreak|nolisp|nolist|noloadplugins|nolpl|nolz|noma|nomacatsui|nomagic|nomh|noml|nomod|nomodeline|nomodifiable|nomodified|nomore|nomousef|nomousefocus|nomousehide|nonu|nonumber|noodev|noopendevice|nopaste|nopi|nopreserveindent|nopreviewwindow|noprompt|nopvw|noreadonly|noremap|norestorescreen|norevins|nori|norightleft|norightleftcmd|norl|norlc|noro|nors|noru|noruler|nosb|nosc|noscb|noscrollbind|noscs|nosecure|nosft|noshellslash|noshelltemp|noshiftround|noshortname|noshowcmd|noshowfulltag|noshowmatch|noshowmode|nosi|nosm|nosmartcase|nosmartindent|nosmarttab|nosmd|nosn|nosol|nospell|nosplitbelow|nosplitright|nospr|nosr|nossl|nosta|nostartofline|nostmp|noswapfile|noswf|nota|notagbsearch|notagrelative|notagstack|notbi|notbidi|notbs|notermbidi|noterse|notextauto|notextmode|notf|notgst|notildeop|notimeout|notitle|noto|notop|notr|nottimeout|nottybuiltin|nottyfast|notx|novb|novisualbell|nowa|nowarn|nowb|noweirdinvert|nowfh|nowfw|nowildmenu|nowinfixheight|nowinfixwidth|nowiv|nowmnu|nowrap|nowrapscan|nowrite|nowriteany|nowritebackup|nows|nrformats|numberwidth|nuw|odev|oft|ofu|omnifunc|opendevice|operatorfunc|opfunc|osfiletype|pa|para|paragraphs|paste|pastetoggle|patchexpr|patchmode|path|pdev|penc|pex|pexpr|pfn|ph|pheader|pi|pm|pmbcs|pmbfn|popt|preserveindent|previewheight|previewwindow|printdevice|printencoding|printexpr|printfont|printheader|printmbcharset|printmbfont|printoptions|prompt|pt|pumheight|pvh|pvw|qe|quoteescape|readonly|remap|report|restorescreen|revins|rightleft|rightleftcmd|rl|rlc|ro|rs|rtp|ruf|ruler|rulerformat|runtimepath|sbo|sc|scb|scr|scroll|scrollbind|scrolljump|scrolloff|scrollopt|scs|sect|sections|secure|sel|selection|selectmode|sessionoptions|sft|shcf|shellcmdflag|shellpipe|shellquote|shellredir|shellslash|shelltemp|shelltype|shellxquote|shiftround|shiftwidth|shm|shortmess|shortname|showbreak|showcmd|showfulltag|showmatch|showmode|showtabline|shq|si|sidescroll|sidescrolloff|siso|sj|slm|smartcase|smartindent|smarttab|smc|smd|softtabstop|sol|spc|spell|spellcapcheck|spellfile|spelllang|spellsuggest|spf|spl|splitbelow|splitright|sps|sr|srr|ss|ssl|ssop|stal|startofline|statusline|stl|stmp|su|sua|suffixes|suffixesadd|sw|swapfile|swapsync|swb|swf|switchbuf|sws|sxq|syn|synmaxcol|syntax|t_AB|t_AF|t_AL|t_CS|t_CV|t_Ce|t_Co|t_Cs|t_DL|t_EI|t_F1|t_F2|t_F3|t_F4|t_F5|t_F6|t_F7|t_F8|t_F9|t_IE|t_IS|t_K1|t_K3|t_K4|t_K5|t_K6|t_K7|t_K8|t_K9|t_KA|t_KB|t_KC|t_KD|t_KE|t_KF|t_KG|t_KH|t_KI|t_KJ|t_KK|t_KL|t_RI|t_RV|t_SI|t_Sb|t_Sf|t_WP|t_WS|t_ZH|t_ZR|t_al|t_bc|t_cd|t_ce|t_cl|t_cm|t_cs|t_da|t_db|t_dl|t_fs|t_k1|t_k2|t_k3|t_k4|t_k5|t_k6|t_k7|t_k8|t_k9|t_kB|t_kD|t_kI|t_kN|t_kP|t_kb|t_kd|t_ke|t_kh|t_kl|t_kr|t_ks|t_ku|t_le|t_mb|t_md|t_me|t_mr|t_ms|t_nd|t_op|t_se|t_so|t_sr|t_te|t_ti|t_ts|t_ue|t_us|t_ut|t_vb|t_ve|t_vi|t_vs|t_xs|tabline|tabpagemax|tabstop|tagbsearch|taglength|tagrelative|tagstack|tal|tb|tbi|tbidi|tbis|tbs|tenc|term|termbidi|termencoding|terse|textauto|textmode|textwidth|tgst|thesaurus|tildeop|timeout|timeoutlen|title|titlelen|titleold|titlestring|toolbar|toolbariconsize|top|tpm|tsl|tsr|ttimeout|ttimeoutlen|ttm|tty|ttybuiltin|ttyfast|ttym|ttymouse|ttyscroll|ttytype|tw|tx|uc|ul|undolevels|updatecount|updatetime|ut|vb|vbs|vdir|verbosefile|vfile|viewdir|viewoptions|viminfo|virtualedit|visualbell|vop|wak|warn|wb|wc|wcm|wd|weirdinvert|wfh|wfw|whichwrap|wi|wig|wildchar|wildcharm|wildignore|wildmenu|wildmode|wildoptions|wim|winaltkeys|window|winfixheight|winfixwidth|winheight|winminheight|winminwidth|winwidth|wiv|wiw|wm|wmh|wmnu|wmw|wop|wrap|wrapmargin|wrapscan|writeany|writebackup|writedelay|ww)\b/,
      number: /\b(?:0x[\da-f]+|\d+(?:\.\d+)?)\b/i,
      operator:
        /\|\||&&|[-+.]=?|[=!](?:[=~][#?]?)?|[<>]=?[#?]?|[*\/%?]|\b(?:is(?:not)?)\b/,
      punctuation: /[{}[\](),;:]/
    };
  }

  // @ts-nocheck
  visualBasic.displayName = 'visual-basic';
  visualBasic.aliases = ['vb', 'vba'];

  /** @type {import('../core.js').Syntax} */
  function visualBasic(Prism) {
    Prism.languages['visual-basic'] = {
      comment: {
        pattern: /(?:['‘’]|REM\b)(?:[^\r\n_]|_(?:\r\n?|\n)?)*/i,
        inside: {
          keyword: /^REM/i
        }
      },
      directive: {
        pattern:
          /#(?:Const|Else|ElseIf|End|ExternalChecksum|ExternalSource|If|Region)(?:\b_[ \t]*(?:\r\n?|\n)|.)+/i,
        alias: 'property',
        greedy: true
      },
      string: {
        pattern: /\$?["“”](?:["“”]{2}|[^"“”])*["“”]C?/i,
        greedy: true
      },
      date: {
        pattern:
          /#[ \t]*(?:\d+([/-])\d+\1\d+(?:[ \t]+(?:\d+[ \t]*(?:AM|PM)|\d+:\d+(?::\d+)?(?:[ \t]*(?:AM|PM))?))?|\d+[ \t]*(?:AM|PM)|\d+:\d+(?::\d+)?(?:[ \t]*(?:AM|PM))?)[ \t]*#/i,
        alias: 'number'
      },
      number:
        /(?:(?:\b\d+(?:\.\d+)?|\.\d+)(?:E[+-]?\d+)?|&[HO][\dA-F]+)(?:[FRD]|U?[ILS])?/i,
      boolean: /\b(?:False|Nothing|True)\b/i,
      keyword:
        /\b(?:AddHandler|AddressOf|Alias|And(?:Also)?|As|Boolean|ByRef|Byte|ByVal|Call|Case|Catch|C(?:Bool|Byte|Char|Date|Dbl|Dec|Int|Lng|Obj|SByte|Short|Sng|Str|Type|UInt|ULng|UShort)|Char|Class|Const|Continue|Currency|Date|Decimal|Declare|Default|Delegate|Dim|DirectCast|Do|Double|Each|Else(?:If)?|End(?:If)?|Enum|Erase|Error|Event|Exit|Finally|For|Friend|Function|Get(?:Type|XMLNamespace)?|Global|GoSub|GoTo|Handles|If|Implements|Imports|In|Inherits|Integer|Interface|Is|IsNot|Let|Lib|Like|Long|Loop|Me|Mod|Module|Must(?:Inherit|Override)|My(?:Base|Class)|Namespace|Narrowing|New|Next|Not(?:Inheritable|Overridable)?|Object|Of|On|Operator|Option(?:al)?|Or(?:Else)?|Out|Overloads|Overridable|Overrides|ParamArray|Partial|Private|Property|Protected|Public|RaiseEvent|ReadOnly|ReDim|RemoveHandler|Resume|Return|SByte|Select|Set|Shadows|Shared|short|Single|Static|Step|Stop|String|Structure|Sub|SyncLock|Then|Throw|To|Try|TryCast|Type|TypeOf|U(?:Integer|Long|Short)|Until|Using|Variant|Wend|When|While|Widening|With(?:Events)?|WriteOnly|Xor)\b/i,
      operator: /[+\-*/\\^<=>&#@$%!]|\b_(?=[ \t]*[\r\n])/,
      punctuation: /[{}().,:?]/
    };
    Prism.languages.vb = Prism.languages['visual-basic'];
    Prism.languages.vba = Prism.languages['visual-basic'];
  }

  // @ts-nocheck
  warpscript.displayName = 'warpscript';
  warpscript.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function warpscript(Prism) {
    Prism.languages.warpscript = {
      comment: /#.*|\/\/.*|\/\*[\s\S]*?\*\//,
      string: {
        pattern:
          /"(?:[^"\\\r\n]|\\.)*"|'(?:[^'\\\r\n]|\\.)*'|<'(?:[^\\']|'(?!>)|\\.)*'>/,
        greedy: true
      },
      variable: /\$\S+/,
      macro: {
        pattern: /@\S+/,
        alias: 'property'
      },
      // WarpScript doesn't have any keywords, these are all functions under the control category
      // https://www.warp10.io/tags/control
      keyword:
        /\b(?:BREAK|CHECKMACRO|CONTINUE|CUDF|DEFINED|DEFINEDMACRO|EVAL|FAIL|FOR|FOREACH|FORSTEP|IFT|IFTE|MSGFAIL|NRETURN|RETHROW|RETURN|SWITCH|TRY|UDF|UNTIL|WHILE)\b/,
      number:
        /[+-]?\b(?:NaN|Infinity|\d+(?:\.\d*)?(?:[Ee][+-]?\d+)?|0x[\da-fA-F]+|0b[01]+)\b/,
      boolean: /\b(?:F|T|false|true)\b/,
      punctuation: /<%|%>|[{}[\]()]/,
      // Some operators from the "operators" category
      // https://www.warp10.io/tags/operators
      operator:
        /==|&&?|\|\|?|\*\*?|>>>?|<<|[<>!~]=?|[-/%^]|\+!?|\b(?:AND|NOT|OR)\b/
    };
  }

  // @ts-nocheck
  wasm.displayName = 'wasm';
  wasm.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function wasm(Prism) {
    Prism.languages.wasm = {
      comment: [
        /\(;[\s\S]*?;\)/,
        {
          pattern: /;;.*/,
          greedy: true
        }
      ],
      string: {
        pattern: /"(?:\\[\s\S]|[^"\\])*"/,
        greedy: true
      },
      keyword: [
        {
          pattern: /\b(?:align|offset)=/,
          inside: {
            operator: /=/
          }
        },
        {
          pattern:
            /\b(?:(?:f32|f64|i32|i64)(?:\.(?:abs|add|and|ceil|clz|const|convert_[su]\/i(?:32|64)|copysign|ctz|demote\/f64|div(?:_[su])?|eqz?|extend_[su]\/i32|floor|ge(?:_[su])?|gt(?:_[su])?|le(?:_[su])?|load(?:(?:8|16|32)_[su])?|lt(?:_[su])?|max|min|mul|neg?|nearest|or|popcnt|promote\/f32|reinterpret\/[fi](?:32|64)|rem_[su]|rot[lr]|shl|shr_[su]|sqrt|store(?:8|16|32)?|sub|trunc(?:_[su]\/f(?:32|64))?|wrap\/i64|xor))?|memory\.(?:grow|size))\b/,
          inside: {
            punctuation: /\./
          }
        },
        /\b(?:anyfunc|block|br(?:_if|_table)?|call(?:_indirect)?|data|drop|elem|else|end|export|func|get_(?:global|local)|global|if|import|local|loop|memory|module|mut|nop|offset|param|result|return|select|set_(?:global|local)|start|table|tee_local|then|type|unreachable)\b/
      ],
      variable: /\$[\w!#$%&'*+\-./:<=>?@\\^`|~]+/,
      number:
        /[+-]?\b(?:\d(?:_?\d)*(?:\.\d(?:_?\d)*)?(?:[eE][+-]?\d(?:_?\d)*)?|0x[\da-fA-F](?:_?[\da-fA-F])*(?:\.[\da-fA-F](?:_?[\da-fA-D])*)?(?:[pP][+-]?\d(?:_?\d)*)?)\b|\binf\b|\bnan(?::0x[\da-fA-F](?:_?[\da-fA-D])*)?\b/,
      punctuation: /[()]/
    };
  }

  // @ts-nocheck
  webIdl.displayName = 'web-idl';
  webIdl.aliases = ['webidl'];

  /** @type {import('../core.js').Syntax} */
  function webIdl(Prism) {
  (function (Prism) {
      var id = /(?:\B-|\b_|\b)[A-Za-z][\w-]*(?![\w-])/.source;
      var type =
        '(?:' +
        /\b(?:unsigned\s+)?long\s+long(?![\w-])/.source +
        '|' +
        /\b(?:unrestricted|unsigned)\s+[a-z]+(?![\w-])/.source +
        '|' +
        /(?!(?:unrestricted|unsigned)\b)/.source +
        id +
        /(?:\s*<(?:[^<>]|<[^<>]*>)*>)?/.source +
        ')' +
        /(?:\s*\?)?/.source;
      var typeInside = {};
      Prism.languages['web-idl'] = {
        comment: {
          pattern: /\/\/.*|\/\*[\s\S]*?\*\//,
          greedy: true
        },
        string: {
          pattern: /"[^"]*"/,
          greedy: true
        },
        namespace: {
          pattern: RegExp(/(\bnamespace\s+)/.source + id),
          lookbehind: true
        },
        'class-name': [
          {
            pattern:
              /(^|[^\w-])(?:iterable|maplike|setlike)\s*<(?:[^<>]|<[^<>]*>)*>/,
            lookbehind: true,
            inside: typeInside
          },
          {
            pattern: RegExp(
              /(\b(?:attribute|const|deleter|getter|optional|setter)\s+)/.source +
                type
            ),
            lookbehind: true,
            inside: typeInside
          },
          {
            // callback return type
            pattern: RegExp(
              '(' + /\bcallback\s+/.source + id + /\s*=\s*/.source + ')' + type
            ),
            lookbehind: true,
            inside: typeInside
          },
          {
            // typedef
            pattern: RegExp(/(\btypedef\b\s*)/.source + type),
            lookbehind: true,
            inside: typeInside
          },
          {
            pattern: RegExp(
              /(\b(?:callback|dictionary|enum|interface(?:\s+mixin)?)\s+)(?!(?:interface|mixin)\b)/
                .source + id
            ),
            lookbehind: true
          },
          {
            // inheritance
            pattern: RegExp(/(:\s*)/.source + id),
            lookbehind: true
          }, // includes and implements
          RegExp(id + /(?=\s+(?:implements|includes)\b)/.source),
          {
            pattern: RegExp(/(\b(?:implements|includes)\s+)/.source + id),
            lookbehind: true
          },
          {
            // function return type, parameter types, and dictionary members
            pattern: RegExp(
              type +
                '(?=' +
                /\s*(?:\.{3}\s*)?/.source +
                id +
                /\s*[(),;=]/.source +
                ')'
            ),
            inside: typeInside
          }
        ],
        builtin:
          /\b(?:ArrayBuffer|BigInt64Array|BigUint64Array|ByteString|DOMString|DataView|Float32Array|Float64Array|FrozenArray|Int16Array|Int32Array|Int8Array|ObservableArray|Promise|USVString|Uint16Array|Uint32Array|Uint8Array|Uint8ClampedArray)\b/,
        keyword: [
          /\b(?:async|attribute|callback|const|constructor|deleter|dictionary|enum|getter|implements|includes|inherit|interface|mixin|namespace|null|optional|or|partial|readonly|required|setter|static|stringifier|typedef|unrestricted)\b/, // type keywords
          /\b(?:any|bigint|boolean|byte|double|float|iterable|long|maplike|object|octet|record|sequence|setlike|short|symbol|undefined|unsigned|void)\b/
        ],
        boolean: /\b(?:false|true)\b/,
        number: {
          pattern:
            /(^|[^\w-])-?(?:0x[0-9a-f]+|(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?|NaN|Infinity)(?![\w-])/i,
          lookbehind: true
        },
        operator: /\.{3}|[=:?<>-]/,
        punctuation: /[(){}[\].,;]/
      };
      for (var key in Prism.languages['web-idl']) {
        if (key !== 'class-name') {
          typeInside[key] = Prism.languages['web-idl'][key];
        }
      }
      Prism.languages['webidl'] = Prism.languages['web-idl'];
    })(Prism);
  }

  // @ts-nocheck
  wiki.displayName = 'wiki';
  wiki.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function wiki(Prism) {
    Prism.register(markup);
    Prism.languages.wiki = Prism.languages.extend('markup', {
      'block-comment': {
        pattern: /(^|[^\\])\/\*[\s\S]*?\*\//,
        lookbehind: true,
        alias: 'comment'
      },
      heading: {
        pattern: /^(=+)[^=\r\n].*?\1/m,
        inside: {
          punctuation: /^=+|=+$/,
          important: /.+/
        }
      },
      emphasis: {
        // TODO Multi-line
        pattern: /('{2,5}).+?\1/,
        inside: {
          'bold-italic': {
            pattern: /(''''').+?(?=\1)/,
            lookbehind: true,
            alias: ['bold', 'italic']
          },
          bold: {
            pattern: /(''')[^'](?:.*?[^'])?(?=\1)/,
            lookbehind: true
          },
          italic: {
            pattern: /('')[^'](?:.*?[^'])?(?=\1)/,
            lookbehind: true
          },
          punctuation: /^''+|''+$/
        }
      },
      hr: {
        pattern: /^-{4,}/m,
        alias: 'punctuation'
      },
      url: [
        /ISBN +(?:97[89][ -]?)?(?:\d[ -]?){9}[\dx]\b|(?:PMID|RFC) +\d+/i,
        /\[\[.+?\]\]|\[.+?\]/
      ],
      variable: [
        /__[A-Z]+__/, // FIXME Nested structures should be handled
        // {{formatnum:{{#expr:{{{3}}}}}}}
        /\{{3}.+?\}{3}/,
        /\{\{.+?\}\}/
      ],
      symbol: [/^#redirect/im, /~{3,5}/],
      // Handle table attrs:
      // {|
      // ! style="text-align:left;"| Item
      // |}
      'table-tag': {
        pattern: /((?:^|[|!])[|!])[^|\r\n]+\|(?!\|)/m,
        lookbehind: true,
        inside: {
          'table-bar': {
            pattern: /\|$/,
            alias: 'punctuation'
          },
          rest: Prism.languages.markup['tag'].inside
        }
      },
      punctuation: /^(?:\{\||\|\}|\|-|[*#:;!|])|\|\||!!/m
    });
    Prism.languages.insertBefore('wiki', 'tag', {
      // Prevent highlighting inside <nowiki>, <source> and <pre> tags
      nowiki: {
        pattern: /<(nowiki|pre|source)\b[^>]*>[\s\S]*?<\/\1>/i,
        inside: {
          tag: {
            pattern: /<(?:nowiki|pre|source)\b[^>]*>|<\/(?:nowiki|pre|source)>/i,
            inside: Prism.languages.markup['tag'].inside
          }
        }
      }
    });
  }

  // @ts-nocheck
  wolfram.displayName = 'wolfram';
  wolfram.aliases = ['mathematica', 'nb', 'wl'];

  /** @type {import('../core.js').Syntax} */
  function wolfram(Prism) {
    Prism.languages.wolfram = {
      // Allow one level of nesting - note: regex taken from applescipt
      comment: /\(\*(?:\(\*(?:[^*]|\*(?!\)))*\*\)|(?!\(\*)[\s\S])*?\*\)/,
      string: {
        pattern: /"(?:\\.|[^"\\\r\n])*"/,
        greedy: true
      },
      keyword:
        /\b(?:Abs|AbsArg|Accuracy|Block|Do|For|Function|If|Manipulate|Module|Nest|NestList|None|Return|Switch|Table|Which|While)\b/,
      context: {
        pattern: /\b\w+`+\w*/,
        alias: 'class-name'
      },
      blank: {
        pattern: /\b\w+_\b/,
        alias: 'regex'
      },
      'global-variable': {
        pattern: /\$\w+/,
        alias: 'variable'
      },
      boolean: /\b(?:False|True)\b/,
      number:
        /(?:\b(?=\d)|\B(?=\.))(?:0[bo])?(?:(?:\d|0x[\da-f])[\da-f]*(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?j?\b/i,
      operator:
        /\/\.|;|=\.|\^=|\^:=|:=|<<|>>|<\||\|>|:>|\|->|->|<-|@@@|@@|@|\/@|=!=|===|==|=|\+|-|\^|\[\/-+%=\]=?|!=|\*\*?=?|\/\/?=?|<[<=>]?|>[=>]?|[&|^~]/,
      punctuation: /[{}[\];(),.:]/
    };
    Prism.languages.mathematica = Prism.languages.wolfram;
    Prism.languages.wl = Prism.languages.wolfram;
    Prism.languages.nb = Prism.languages.wolfram;
  }

  // @ts-nocheck
  wren.displayName = 'wren';
  wren.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function wren(Prism) {
    // https://wren.io/
    Prism.languages.wren = {
      // Multiline comments in Wren can have nested multiline comments
      // Comments: // and /* */
      comment: [
        {
          // support 3 levels of nesting
          // regex: \/\*(?:[^*/]|\*(?!\/)|\/(?!\*)|<self>)*\*\/
          pattern:
            /\/\*(?:[^*/]|\*(?!\/)|\/(?!\*)|\/\*(?:[^*/]|\*(?!\/)|\/(?!\*)|\/\*(?:[^*/]|\*(?!\/)|\/(?!\*))*\*\/)*\*\/)*\*\//,
          greedy: true
        },
        {
          pattern: /(^|[^\\:])\/\/.*/,
          lookbehind: true,
          greedy: true
        }
      ],
      // Triple quoted strings are multiline but cannot have interpolation (raw strings)
      // Based on prism-python.js
      'triple-quoted-string': {
        pattern: /"""[\s\S]*?"""/,
        greedy: true,
        alias: 'string'
      },
      // see below
      'string-literal': null,
      // #!/usr/bin/env wren on the first line
      hashbang: {
        pattern: /^#!\/.+/,
        greedy: true,
        alias: 'comment'
      },
      // Attributes are special keywords to add meta data to classes
      attribute: {
        // #! attributes are stored in class properties
        // #!myvar = true
        // #attributes are not stored and dismissed at compilation
        pattern: /#!?[ \t\u3000]*\w+/,
        alias: 'keyword'
      },
      'class-name': [
        {
          // class definition
          // class Meta {}
          pattern: /(\bclass\s+)\w+/,
          lookbehind: true
        }, // A class must always start with an uppercase.
        // File.read
        /\b[A-Z][a-z\d_]*\b/
      ],
      // A constant can be a variable, class, property or method. Just named in all uppercase letters
      constant: /\b[A-Z][A-Z\d_]*\b/,
      null: {
        pattern: /\bnull\b/,
        alias: 'keyword'
      },
      keyword:
        /\b(?:as|break|class|construct|continue|else|for|foreign|if|import|in|is|return|static|super|this|var|while)\b/,
      boolean: /\b(?:false|true)\b/,
      number: /\b(?:0x[\da-f]+|\d+(?:\.\d+)?(?:e[+-]?\d+)?)\b/i,
      // Functions can be Class.method()
      function: /\b[a-z_]\w*(?=\s*[({])/i,
      operator: /<<|>>|[=!<>]=?|&&|\|\||[-+*/%~^&|?:]|\.{2,3}/,
      punctuation: /[\[\](){}.,;]/
    };
    Prism.languages.wren['string-literal'] = {
      // A single quote string is multiline and can have interpolation (similar to JS backticks ``)
      pattern:
        /(^|[^\\"])"(?:[^\\"%]|\\[\s\S]|%(?!\()|%\((?:[^()]|\((?:[^()]|\([^)]*\))*\))*\))*"/,
      lookbehind: true,
      greedy: true,
      inside: {
        interpolation: {
          // "%(interpolation)"
          pattern:
            /((?:^|[^\\])(?:\\{2})*)%\((?:[^()]|\((?:[^()]|\([^)]*\))*\))*\)/,
          lookbehind: true,
          inside: {
            expression: {
              pattern: /^(%\()[\s\S]+(?=\)$)/,
              lookbehind: true,
              inside: Prism.languages.wren
            },
            'interpolation-punctuation': {
              pattern: /^%\(|\)$/,
              alias: 'punctuation'
            }
          }
        },
        string: /[\s\S]+/
      }
    };
  }

  // @ts-nocheck
  xeora.displayName = 'xeora';
  xeora.aliases = ['xeoracube'];

  /** @type {import('../core.js').Syntax} */
  function xeora(Prism) {
    Prism.register(markup)
    ;(function (Prism) {
      Prism.languages.xeora = Prism.languages.extend('markup', {
        constant: {
          pattern: /\$(?:DomainContents|PageRenderDuration)\$/,
          inside: {
            punctuation: {
              pattern: /\$/
            }
          }
        },
        variable: {
          pattern: /\$@?(?:#+|[-+*~=^])?[\w.]+\$/,
          inside: {
            punctuation: {
              pattern: /[$.]/
            },
            operator: {
              pattern: /#+|[-+*~=^@]/
            }
          }
        },
        'function-inline': {
          pattern:
            /\$F:[-\w.]+\?[-\w.]+(?:,(?:(?:@[-#]*\w+\.[\w+.]\.*)*\|)*(?:(?:[\w+]|[-#*.~^]+[\w+]|=\S)(?:[^$=]|=+[^=])*=*|(?:@[-#]*\w+\.[\w+.]\.*)+(?:(?:[\w+]|[-#*~^][-#*.~^]*[\w+]|=\S)(?:[^$=]|=+[^=])*=*)?)?)?\$/,
          inside: {
            variable: {
              pattern: /(?:[,|])@?(?:#+|[-+*~=^])?[\w.]+/,
              inside: {
                punctuation: {
                  pattern: /[,.|]/
                },
                operator: {
                  pattern: /#+|[-+*~=^@]/
                }
              }
            },
            punctuation: {
              pattern: /\$\w:|[$:?.,|]/
            }
          },
          alias: 'function'
        },
        'function-block': {
          pattern:
            /\$XF:\{[-\w.]+\?[-\w.]+(?:,(?:(?:@[-#]*\w+\.[\w+.]\.*)*\|)*(?:(?:[\w+]|[-#*.~^]+[\w+]|=\S)(?:[^$=]|=+[^=])*=*|(?:@[-#]*\w+\.[\w+.]\.*)+(?:(?:[\w+]|[-#*~^][-#*.~^]*[\w+]|=\S)(?:[^$=]|=+[^=])*=*)?)?)?\}:XF\$/,
          inside: {
            punctuation: {
              pattern: /[$:{}?.,|]/
            }
          },
          alias: 'function'
        },
        'directive-inline': {
          pattern: /\$\w(?:#\d+\+?)?(?:\[[-\w.]+\])?:[-\/\w.]+\$/,
          inside: {
            punctuation: {
              pattern: /\$(?:\w:|C(?:\[|#\d))?|[:{[\]]/,
              inside: {
                tag: {
                  pattern: /#\d/
                }
              }
            }
          },
          alias: 'function'
        },
        'directive-block-open': {
          pattern:
            /\$\w+:\{|\$\w(?:#\d+\+?)?(?:\[[-\w.]+\])?:[-\w.]+:\{(?:![A-Z]+)?/,
          inside: {
            punctuation: {
              pattern: /\$(?:\w:|C(?:\[|#\d))?|[:{[\]]/,
              inside: {
                tag: {
                  pattern: /#\d/
                }
              }
            },
            attribute: {
              pattern: /![A-Z]+$/,
              inside: {
                punctuation: {
                  pattern: /!/
                }
              },
              alias: 'keyword'
            }
          },
          alias: 'function'
        },
        'directive-block-separator': {
          pattern: /\}:[-\w.]+:\{/,
          inside: {
            punctuation: {
              pattern: /[:{}]/
            }
          },
          alias: 'function'
        },
        'directive-block-close': {
          pattern: /\}:[-\w.]+\$/,
          inside: {
            punctuation: {
              pattern: /[:{}$]/
            }
          },
          alias: 'function'
        }
      });
      Prism.languages.insertBefore(
        'inside',
        'punctuation',
        {
          variable: Prism.languages.xeora['function-inline'].inside['variable']
        },
        Prism.languages.xeora['function-block']
      );
      Prism.languages.xeoracube = Prism.languages.xeora;
    })(Prism);
  }

  // @ts-nocheck
  xmlDoc.displayName = 'xml-doc';
  xmlDoc.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function xmlDoc(Prism) {
    Prism.register(markup)
    ;(function (Prism) {
      /**
       * If the given language is present, it will insert the given doc comment grammar token into it.
       *
       * @param {string} lang
       * @param {any} docComment
       */
      function insertDocComment(lang, docComment) {
        if (Prism.languages[lang]) {
          Prism.languages.insertBefore(lang, 'comment', {
            'doc-comment': docComment
          });
        }
      }
      var tag = Prism.languages.markup.tag;
      var slashDocComment = {
        pattern: /\/\/\/.*/,
        greedy: true,
        alias: 'comment',
        inside: {
          tag: tag
        }
      };
      var tickDocComment = {
        pattern: /'''.*/,
        greedy: true,
        alias: 'comment',
        inside: {
          tag: tag
        }
      };
      insertDocComment('csharp', slashDocComment);
      insertDocComment('fsharp', slashDocComment);
      insertDocComment('vbnet', tickDocComment);
    })(Prism);
  }

  // @ts-nocheck
  xojo.displayName = 'xojo';
  xojo.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function xojo(Prism) {
    Prism.languages.xojo = {
      comment: {
        pattern: /(?:'|\/\/|Rem\b).+/i,
        greedy: true
      },
      string: {
        pattern: /"(?:""|[^"])*"/,
        greedy: true
      },
      number: [/(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:E[+-]?\d+)?/i, /&[bchou][a-z\d]+/i],
      directive: {
        pattern: /#(?:Else|ElseIf|Endif|If|Pragma)\b/i,
        alias: 'property'
      },
      keyword:
        /\b(?:AddHandler|App|Array|As(?:signs)?|Auto|Boolean|Break|By(?:Ref|Val)|Byte|Call|Case|Catch|CFStringRef|CGFloat|Class|Color|Const|Continue|CString|Currency|CurrentMethodName|Declare|Delegate|Dim|Do(?:uble|wnTo)?|Each|Else(?:If)?|End|Enumeration|Event|Exception|Exit|Extends|False|Finally|For|Function|Get|GetTypeInfo|Global|GOTO|If|Implements|In|Inherits|Int(?:8|16|32|64|eger|erface)?|Lib|Loop|Me|Module|Next|Nil|Object|Optional|OSType|ParamArray|Private|Property|Protected|PString|Ptr|Raise(?:Event)?|ReDim|RemoveHandler|Return|Select(?:or)?|Self|Set|Shared|Short|Single|Soft|Static|Step|String|Sub|Super|Text|Then|To|True|Try|Ubound|UInt(?:8|16|32|64|eger)?|Until|Using|Var(?:iant)?|Wend|While|WindowPtr|WString)\b/i,
      operator:
        /<[=>]?|>=?|[+\-*\/\\^=]|\b(?:AddressOf|And|Ctype|IsA?|Mod|New|Not|Or|WeakAddressOf|Xor)\b/i,
      punctuation: /[.,;:()]/
    };
  }

  // @ts-nocheck
  xquery.displayName = 'xquery';
  xquery.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function xquery(Prism) {
    Prism.register(markup)
    ;(function (Prism) {
      Prism.languages.xquery = Prism.languages.extend('markup', {
        'xquery-comment': {
          pattern: /\(:[\s\S]*?:\)/,
          greedy: true,
          alias: 'comment'
        },
        string: {
          pattern: /(["'])(?:\1\1|(?!\1)[\s\S])*\1/,
          greedy: true
        },
        extension: {
          pattern: /\(#.+?#\)/,
          alias: 'symbol'
        },
        variable: /\$[-\w:]+/,
        axis: {
          pattern:
            /(^|[^-])(?:ancestor(?:-or-self)?|attribute|child|descendant(?:-or-self)?|following(?:-sibling)?|parent|preceding(?:-sibling)?|self)(?=::)/,
          lookbehind: true,
          alias: 'operator'
        },
        'keyword-operator': {
          pattern:
            /(^|[^:-])\b(?:and|castable as|div|eq|except|ge|gt|idiv|instance of|intersect|is|le|lt|mod|ne|or|union)\b(?=$|[^:-])/,
          lookbehind: true,
          alias: 'operator'
        },
        keyword: {
          pattern:
            /(^|[^:-])\b(?:as|ascending|at|base-uri|boundary-space|case|cast as|collation|construction|copy-namespaces|declare|default|descending|else|empty (?:greatest|least)|encoding|every|external|for|function|if|import|in|inherit|lax|let|map|module|namespace|no-inherit|no-preserve|option|order(?: by|ed|ing)?|preserve|return|satisfies|schema|some|stable|strict|strip|then|to|treat as|typeswitch|unordered|validate|variable|version|where|xquery)\b(?=$|[^:-])/,
          lookbehind: true
        },
        function: /[\w-]+(?::[\w-]+)*(?=\s*\()/,
        'xquery-element': {
          pattern: /(element\s+)[\w-]+(?::[\w-]+)*/,
          lookbehind: true,
          alias: 'tag'
        },
        'xquery-attribute': {
          pattern: /(attribute\s+)[\w-]+(?::[\w-]+)*/,
          lookbehind: true,
          alias: 'attr-name'
        },
        builtin: {
          pattern:
            /(^|[^:-])\b(?:attribute|comment|document|element|processing-instruction|text|xs:(?:ENTITIES|ENTITY|ID|IDREFS?|NCName|NMTOKENS?|NOTATION|Name|QName|anyAtomicType|anyType|anyURI|base64Binary|boolean|byte|date|dateTime|dayTimeDuration|decimal|double|duration|float|gDay|gMonth|gMonthDay|gYear|gYearMonth|hexBinary|int|integer|language|long|negativeInteger|nonNegativeInteger|nonPositiveInteger|normalizedString|positiveInteger|short|string|time|token|unsigned(?:Byte|Int|Long|Short)|untyped(?:Atomic)?|yearMonthDuration))\b(?=$|[^:-])/,
          lookbehind: true
        },
        number: /\b\d+(?:\.\d+)?(?:E[+-]?\d+)?/,
        operator: [
          /[+*=?|@]|\.\.?|:=|!=|<[=<]?|>[=>]?/,
          {
            pattern: /(\s)-(?=\s)/,
            lookbehind: true
          }
        ],
        punctuation: /[[\](){},;:/]/
      });
      Prism.languages.xquery.tag.pattern =
        /<\/?(?!\d)[^\s>\/=$<%]+(?:\s+[^\s>\/=]+(?:=(?:("|')(?:\\[\s\S]|\{(?!\{)(?:\{(?:\{[^{}]*\}|[^{}])*\}|[^{}])+\}|(?!\1)[^\\])*\1|[^\s'">=]+))?)*\s*\/?>/;
      Prism.languages.xquery['tag'].inside['attr-value'].pattern =
        /=(?:("|')(?:\\[\s\S]|\{(?!\{)(?:\{(?:\{[^{}]*\}|[^{}])*\}|[^{}])+\}|(?!\1)[^\\])*\1|[^\s'">=]+)/;
      Prism.languages.xquery['tag'].inside['attr-value'].inside['punctuation'] =
        /^="|"$/;
      Prism.languages.xquery['tag'].inside['attr-value'].inside['expression'] = {
        // Allow for two levels of nesting
        pattern: /\{(?!\{)(?:\{(?:\{[^{}]*\}|[^{}])*\}|[^{}])+\}/,
        inside: Prism.languages.xquery,
        alias: 'language-xquery'
      }; // The following will handle plain text inside tags
      var stringifyToken = function (token) {
        if (typeof token === 'string') {
          return token
        }
        if (typeof token.content === 'string') {
          return token.content
        }
        return token.content.map(stringifyToken).join('')
      };
      var walkTokens = function (tokens) {
        var openedTags = [];
        for (var i = 0; i < tokens.length; i++) {
          var token = tokens[i];
          var notTagNorBrace = false;
          if (typeof token !== 'string') {
            if (
              token.type === 'tag' &&
              token.content[0] &&
              token.content[0].type === 'tag'
            ) {
              // We found a tag, now find its kind
              if (token.content[0].content[0].content === '</') {
                // Closing tag
                if (
                  openedTags.length > 0 &&
                  openedTags[openedTags.length - 1].tagName ===
                    stringifyToken(token.content[0].content[1])
                ) {
                  // Pop matching opening tag
                  openedTags.pop();
                }
              } else {
                if (token.content[token.content.length - 1].content === '/>') ; else {
                  // Opening tag
                  openedTags.push({
                    tagName: stringifyToken(token.content[0].content[1]),
                    openedBraces: 0
                  });
                }
              }
            } else if (
              openedTags.length > 0 &&
              token.type === 'punctuation' &&
              token.content === '{' && // Ignore `{{`
              (!tokens[i + 1] ||
                tokens[i + 1].type !== 'punctuation' ||
                tokens[i + 1].content !== '{') &&
              (!tokens[i - 1] ||
                tokens[i - 1].type !== 'plain-text' ||
                tokens[i - 1].content !== '{')
            ) {
              // Here we might have entered an XQuery expression inside a tag
              openedTags[openedTags.length - 1].openedBraces++;
            } else if (
              openedTags.length > 0 &&
              openedTags[openedTags.length - 1].openedBraces > 0 &&
              token.type === 'punctuation' &&
              token.content === '}'
            ) {
              // Here we might have left an XQuery expression inside a tag
              openedTags[openedTags.length - 1].openedBraces--;
            } else if (token.type !== 'comment') {
              notTagNorBrace = true;
            }
          }
          if (notTagNorBrace || typeof token === 'string') {
            if (
              openedTags.length > 0 &&
              openedTags[openedTags.length - 1].openedBraces === 0
            ) {
              // Here we are inside a tag, and not inside an XQuery expression.
              // That's plain text: drop any tokens matched.
              var plainText = stringifyToken(token); // And merge text with adjacent text
              if (
                i < tokens.length - 1 &&
                (typeof tokens[i + 1] === 'string' ||
                  tokens[i + 1].type === 'plain-text')
              ) {
                plainText += stringifyToken(tokens[i + 1]);
                tokens.splice(i + 1, 1);
              }
              if (
                i > 0 &&
                (typeof tokens[i - 1] === 'string' ||
                  tokens[i - 1].type === 'plain-text')
              ) {
                plainText = stringifyToken(tokens[i - 1]) + plainText;
                tokens.splice(i - 1, 1);
                i--;
              }
              if (/^\s+$/.test(plainText)) {
                tokens[i] = plainText;
              } else {
                tokens[i] = new Prism.Token(
                  'plain-text',
                  plainText,
                  null,
                  plainText
                );
              }
            }
          }
          if (token.content && typeof token.content !== 'string') {
            walkTokens(token.content);
          }
        }
      };
      Prism.hooks.add('after-tokenize', function (env) {
        if (env.language !== 'xquery') {
          return
        }
        walkTokens(env.tokens);
      });
    })(Prism);
  }

  // @ts-nocheck
  yang.displayName = 'yang';
  yang.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function yang(Prism) {
    Prism.languages.yang = {
      // https://tools.ietf.org/html/rfc6020#page-34
      // http://www.yang-central.org/twiki/bin/view/Main/YangExamples
      comment: /\/\*[\s\S]*?\*\/|\/\/.*/,
      string: {
        pattern: /"(?:[^\\"]|\\.)*"|'[^']*'/,
        greedy: true
      },
      keyword: {
        pattern: /(^|[{};\r\n][ \t]*)[a-z_][\w.-]*/i,
        lookbehind: true
      },
      namespace: {
        pattern: /(\s)[a-z_][\w.-]*(?=:)/i,
        lookbehind: true
      },
      boolean: /\b(?:false|true)\b/,
      operator: /\+/,
      punctuation: /[{};:]/
    };
  }

  // @ts-nocheck
  zig.displayName = 'zig';
  zig.aliases = [];

  /** @type {import('../core.js').Syntax} */
  function zig(Prism) {
  (function (Prism) {
      function literal(str) {
        return function () {
          return str
        }
      }
      var keyword =
        /\b(?:align|allowzero|and|anyframe|anytype|asm|async|await|break|cancel|catch|comptime|const|continue|defer|else|enum|errdefer|error|export|extern|fn|for|if|inline|linksection|nakedcc|noalias|nosuspend|null|or|orelse|packed|promise|pub|resume|return|stdcallcc|struct|suspend|switch|test|threadlocal|try|undefined|union|unreachable|usingnamespace|var|volatile|while)\b/;
      var IDENTIFIER = '\\b(?!' + keyword.source + ')(?!\\d)\\w+\\b';
      var ALIGN = /align\s*\((?:[^()]|\([^()]*\))*\)/.source;
      var PREFIX_TYPE_OP =
        /(?:\?|\bpromise->|(?:\[[^[\]]*\]|\*(?!\*)|\*\*)(?:\s*<ALIGN>|\s*const\b|\s*volatile\b|\s*allowzero\b)*)/.source.replace(
          /<ALIGN>/g,
          literal(ALIGN)
        );
      var SUFFIX_EXPR =
        /(?:\bpromise\b|(?:\berror\.)?<ID>(?:\.<ID>)*(?!\s+<ID>))/.source.replace(
          /<ID>/g,
          literal(IDENTIFIER)
        );
      var TYPE =
        '(?!\\s)(?:!?\\s*(?:' + PREFIX_TYPE_OP + '\\s*)*' + SUFFIX_EXPR + ')+';
      /*
       * A simplified grammar for Zig compile time type literals:
       *
       * TypeExpr = ( "!"? PREFIX_TYPE_OP* SUFFIX_EXPR )+
       *
       * SUFFIX_EXPR = ( \b "promise" \b | ( \b "error" "." )? IDENTIFIER ( "." IDENTIFIER )* (?! \s+ IDENTIFIER ) )
       *
       * PREFIX_TYPE_OP = "?"
       *                | \b "promise" "->"
       *                | ( "[" [^\[\]]* "]" | "*" | "**" ) ( ALIGN | "const" \b | "volatile" \b | "allowzero" \b )*
       *
       * ALIGN = "align" "(" ( [^()] | "(" [^()]* ")" )* ")"
       *
       * IDENTIFIER = \b (?! KEYWORD ) [a-zA-Z_] \w* \b
       *
       */
      Prism.languages.zig = {
        comment: [
          {
            pattern: /\/\/[/!].*/,
            alias: 'doc-comment'
          },
          /\/{2}.*/
        ],
        string: [
          {
            // "string" and c"string"
            pattern: /(^|[^\\@])c?"(?:[^"\\\r\n]|\\.)*"/,
            lookbehind: true,
            greedy: true
          },
          {
            // multiline strings and c-strings
            pattern: /([\r\n])([ \t]+c?\\{2}).*(?:(?:\r\n?|\n)\2.*)*/,
            lookbehind: true,
            greedy: true
          }
        ],
        char: {
          // characters 'a', '\n', '\xFF', '\u{10FFFF}'
          pattern:
            /(^|[^\\])'(?:[^'\\\r\n]|[\uD800-\uDFFF]{2}|\\(?:.|x[a-fA-F\d]{2}|u\{[a-fA-F\d]{1,6}\}))'/,
          lookbehind: true,
          greedy: true
        },
        builtin: /\B@(?!\d)\w+(?=\s*\()/,
        label: {
          pattern:
            /(\b(?:break|continue)\s*:\s*)\w+\b|\b(?!\d)\w+\b(?=\s*:\s*(?:\{|while\b))/,
          lookbehind: true
        },
        'class-name': [
          // const Foo = struct {};
          /\b(?!\d)\w+(?=\s*=\s*(?:(?:extern|packed)\s+)?(?:enum|struct|union)\s*[({])/,
          {
            // const x: i32 = 9;
            // var x: Bar;
            // fn foo(x: bool, y: f32) void {}
            pattern: RegExp(
              /(:\s*)<TYPE>(?=\s*(?:<ALIGN>\s*)?[=;,)])|<TYPE>(?=\s*(?:<ALIGN>\s*)?\{)/.source
                .replace(/<TYPE>/g, literal(TYPE))
                .replace(/<ALIGN>/g, literal(ALIGN))
            ),
            lookbehind: true,
            inside: null // see below
          },
          {
            // extern fn foo(x: f64) f64; (optional alignment)
            pattern: RegExp(
              /(\)\s*)<TYPE>(?=\s*(?:<ALIGN>\s*)?;)/.source
                .replace(/<TYPE>/g, literal(TYPE))
                .replace(/<ALIGN>/g, literal(ALIGN))
            ),
            lookbehind: true,
            inside: null // see below
          }
        ],
        'builtin-type': {
          pattern:
            /\b(?:anyerror|bool|c_u?(?:int|long|longlong|short)|c_longdouble|c_void|comptime_(?:float|int)|f(?:16|32|64|128)|[iu](?:8|16|32|64|128|size)|noreturn|type|void)\b/,
          alias: 'keyword'
        },
        keyword: keyword,
        function: /\b(?!\d)\w+(?=\s*\()/,
        number:
          /\b(?:0b[01]+|0o[0-7]+|0x[a-fA-F\d]+(?:\.[a-fA-F\d]*)?(?:[pP][+-]?[a-fA-F\d]+)?|\d+(?:\.\d*)?(?:[eE][+-]?\d+)?)\b/,
        boolean: /\b(?:false|true)\b/,
        operator:
          /\.[*?]|\.{2,3}|[-=]>|\*\*|\+\+|\|\||(?:<<|>>|[-+*]%|[-+*/%^&|<>!=])=?|[?~]/,
        punctuation: /[.:,;(){}[\]]/
      };
      Prism.languages.zig['class-name'].forEach(function (obj) {
        if (obj.inside === null) {
          obj.inside = Prism.languages.zig;
        }
      });
    })(Prism);
  }

  /**
   * @typedef {import('./core.js').RefractorRoot} RefractorRoot
   * @typedef {import('./core.js').RefractorElement} RefractorElement
   * @typedef {import('./core.js').Text} Text
   * @typedef {import('./core.js').Grammar} Grammar
   * @typedef {import('./core.js').Syntax} Syntax
   */

  refractor.register(markup);
  refractor.register(css);
  refractor.register(clike);
  refractor.register(regex);
  refractor.register(javascript);
  refractor.register(abap);
  refractor.register(abnf);
  refractor.register(actionscript);
  refractor.register(ada);
  refractor.register(agda);
  refractor.register(al);
  refractor.register(antlr4);
  refractor.register(apacheconf);
  refractor.register(sql);
  refractor.register(apex);
  refractor.register(apl);
  refractor.register(applescript);
  refractor.register(aql);
  refractor.register(c$1);
  refractor.register(cpp);
  refractor.register(arduino);
  refractor.register(arff);
  refractor.register(asciidoc);
  refractor.register(csharp);
  refractor.register(aspnet);
  refractor.register(asm6502);
  refractor.register(asmatmel);
  refractor.register(autohotkey);
  refractor.register(autoit);
  refractor.register(avisynth);
  refractor.register(avroIdl);
  refractor.register(bash);
  refractor.register(basic);
  refractor.register(batch);
  refractor.register(bbcode);
  refractor.register(bicep);
  refractor.register(birb);
  refractor.register(bison);
  refractor.register(bnf);
  refractor.register(brainfuck);
  refractor.register(brightscript);
  refractor.register(bro);
  refractor.register(bsl);
  refractor.register(cfscript);
  refractor.register(chaiscript);
  refractor.register(cil);
  refractor.register(clojure);
  refractor.register(cmake);
  refractor.register(cobol);
  refractor.register(coffeescript);
  refractor.register(concurnas);
  refractor.register(csp);
  refractor.register(coq);
  refractor.register(ruby);
  refractor.register(crystal);
  refractor.register(cssExtras);
  refractor.register(csv);
  refractor.register(cypher);
  refractor.register(d);
  refractor.register(dart);
  refractor.register(dataweave);
  refractor.register(dax);
  refractor.register(dhall);
  refractor.register(diff);
  refractor.register(markupTemplating);
  refractor.register(django);
  refractor.register(dnsZoneFile);
  refractor.register(docker);
  refractor.register(dot);
  refractor.register(ebnf);
  refractor.register(editorconfig);
  refractor.register(eiffel);
  refractor.register(ejs);
  refractor.register(elixir);
  refractor.register(elm);
  refractor.register(lua);
  refractor.register(etlua);
  refractor.register(erb);
  refractor.register(erlang);
  refractor.register(excelFormula);
  refractor.register(fsharp);
  refractor.register(factor);
  refractor.register($false);
  refractor.register(firestoreSecurityRules);
  refractor.register(flow);
  refractor.register(fortran);
  refractor.register(ftl);
  refractor.register(gml);
  refractor.register(gap);
  refractor.register(gcode);
  refractor.register(gdscript);
  refractor.register(gedcom);
  refractor.register(gherkin);
  refractor.register(git);
  refractor.register(glsl);
  refractor.register(gn);
  refractor.register(go);
  refractor.register(goModule);
  refractor.register(yaml);
  refractor.register(markdown$1);
  refractor.register(graphql);
  refractor.register(groovy);
  refractor.register(less);
  refractor.register(scss);
  refractor.register(textile);
  refractor.register(haml);
  refractor.register(handlebars);
  refractor.register(haskell);
  refractor.register(haxe);
  refractor.register(hcl);
  refractor.register(hlsl);
  refractor.register(hoon);
  refractor.register(hpkp);
  refractor.register(hsts);
  refractor.register(json);
  refractor.register(uri);
  refractor.register(http);
  refractor.register(ichigojam);
  refractor.register(icon);
  refractor.register(icuMessageFormat);
  refractor.register(idris);
  refractor.register(ignore);
  refractor.register(inform7);
  refractor.register(ini);
  refractor.register(io);
  refractor.register(j);
  refractor.register(java);
  refractor.register(php);
  refractor.register(javadoclike);
  refractor.register(scala);
  refractor.register(javadoc);
  refractor.register(javastacktrace);
  refractor.register(jexl);
  refractor.register(jolie);
  refractor.register(jq);
  refractor.register(jsTemplates);
  refractor.register(typescript);
  refractor.register(jsdoc);
  refractor.register(n4js);
  refractor.register(jsExtras);
  refractor.register(json5);
  refractor.register(jsonp);
  refractor.register(jsstacktrace);
  refractor.register(julia);
  refractor.register(keepalived);
  refractor.register(keyman);
  refractor.register(kotlin);
  refractor.register(kumir);
  refractor.register(kusto);
  refractor.register(latex);
  refractor.register(latte);
  refractor.register(scheme);
  refractor.register(lilypond);
  refractor.register(liquid);
  refractor.register(lisp);
  refractor.register(livescript);
  refractor.register(llvm);
  refractor.register(log);
  refractor.register(lolcode);
  refractor.register(magma);
  refractor.register(makefile);
  refractor.register(matlab);
  refractor.register(maxscript);
  refractor.register(mel);
  refractor.register(mermaid);
  refractor.register(mizar);
  refractor.register(mongodb);
  refractor.register(monkey);
  refractor.register(moonscript);
  refractor.register(n1ql);
  refractor.register(nand2tetrisHdl);
  refractor.register(naniscript);
  refractor.register(nasm);
  refractor.register(neon);
  refractor.register(nevod);
  refractor.register(nginx);
  refractor.register(nim);
  refractor.register(nix);
  refractor.register(nsis);
  refractor.register(objectivec);
  refractor.register(ocaml);
  refractor.register(opencl);
  refractor.register(openqasm);
  refractor.register(oz);
  refractor.register(parigp);
  refractor.register(parser);
  refractor.register(pascal);
  refractor.register(pascaligo);
  refractor.register(psl);
  refractor.register(pcaxis);
  refractor.register(peoplecode);
  refractor.register(perl);
  refractor.register(phpdoc);
  refractor.register(phpExtras);
  refractor.register(plsql);
  refractor.register(powerquery);
  refractor.register(powershell);
  refractor.register(processing);
  refractor.register(prolog);
  refractor.register(promql);
  refractor.register(properties);
  refractor.register(protobuf);
  refractor.register(stylus);
  refractor.register(twig);
  refractor.register(pug);
  refractor.register(puppet);
  refractor.register(pure);
  refractor.register(purebasic);
  refractor.register(purescript);
  refractor.register(python);
  refractor.register(qsharp);
  refractor.register(q);
  refractor.register(qml);
  refractor.register(qore);
  refractor.register(r);
  refractor.register(racket);
  refractor.register(cshtml);
  refractor.register(jsx);
  refractor.register(tsx);
  refractor.register(reason);
  refractor.register(rego);
  refractor.register(renpy);
  refractor.register(rest);
  refractor.register(rip);
  refractor.register(roboconf);
  refractor.register(robotframework);
  refractor.register(rust);
  refractor.register(sas);
  refractor.register(sass);
  refractor.register(shellSession);
  refractor.register(smali);
  refractor.register(smalltalk);
  refractor.register(smarty);
  refractor.register(sml);
  refractor.register(solidity);
  refractor.register(solutionFile);
  refractor.register(soy);
  refractor.register(turtle);
  refractor.register(sparql);
  refractor.register(splunkSpl);
  refractor.register(sqf);
  refractor.register(squirrel);
  refractor.register(stan);
  refractor.register(iecst);
  refractor.register(swift);
  refractor.register(systemd);
  refractor.register(t4Templating);
  refractor.register(t4Cs);
  refractor.register(vbnet);
  refractor.register(t4Vb);
  refractor.register(tap);
  refractor.register(tcl);
  refractor.register(tt2);
  refractor.register(toml);
  refractor.register(tremor);
  refractor.register(typoscript);
  refractor.register(unrealscript);
  refractor.register(uorazor);
  refractor.register(v);
  refractor.register(vala);
  refractor.register(velocity);
  refractor.register(verilog);
  refractor.register(vhdl);
  refractor.register(vim);
  refractor.register(visualBasic);
  refractor.register(warpscript);
  refractor.register(wasm);
  refractor.register(webIdl);
  refractor.register(wiki);
  refractor.register(wolfram);
  refractor.register(wren);
  refractor.register(xeora);
  refractor.register(xmlDoc);
  refractor.register(xojo);
  refractor.register(xquery);
  refractor.register(yang);
  refractor.register(zig);

  function a(){a=function(e,t){return new r(e,void 0,t)};var e=RegExp.prototype,t=new WeakMap;function r(e,n,i){var o=new RegExp(e,n);return t.set(o,i||t.get(e)),l(o,r.prototype)}function n(e,r){var n=t.get(r);return Object.keys(n).reduce(function(t,r){return t[r]=e[n[r]],t},Object.create(null))}return s(r,RegExp),r.prototype.exec=function(t){var r=e.exec.call(this,t);return r&&(r.groups=n(r,this)),r},r.prototype[Symbol.replace]=function(r,i){if("string"==typeof i){var o=t.get(this);return e[Symbol.replace].call(this,r,i.replace(/\$<([^>]+)>/g,function(e,t){return "$"+o[t]}))}if("function"==typeof i){var a=this;return e[Symbol.replace].call(this,r,function(){var e=arguments;return "object"!=typeof e[e.length-1]&&(e=[].slice.call(e)).push(n(e,a)),i.apply(this,e)})}return e[Symbol.replace].call(this,r,i)},a.apply(this,arguments)}function s(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&l(e,t);}function l(e,t){return l=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e},l(e,t)}function u(e,t){(null==t||t>e.length)&&(t=e.length);for(var r=0,n=new Array(t);r<t;r++)n[r]=e[r];return n}function c(e,t){var r="undefined"!=typeof Symbol&&e[Symbol.iterator]||e["@@iterator"];if(r)return (r=r.call(e)).next.bind(r);if(Array.isArray(e)||(r=function(e,t){if(e){if("string"==typeof e)return u(e,t);var r=Object.prototype.toString.call(e).slice(8,-1);return "Object"===r&&e.constructor&&(r=e.constructor.name),"Map"===r||"Set"===r?Array.from(e):"Arguments"===r||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(r)?u(e,t):void 0}}(e))||t&&e&&"number"==typeof e.length){r&&(e=r);var n=0;return function(){return n>=e.length?{done:!0}:{done:!1,value:e[n++]}}}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}var p=function e(t){return t.reduce(function(t,r){if("text"===r.type){if(-1===r.value.indexOf("\n"))return t.push(r),t;for(var n,i=r.value.split("\n"),o=c(i.entries());!(n=o()).done;){var a=n.value,s=a[0],l=a[1];t.push({type:"text",value:s===i.length-1?l:l+"\n",position:{start:{line:r.position.start.line+s},end:{line:r.position.start.line+s}}});}return t}return Object.prototype.hasOwnProperty.call(r,"children")?(r.children=e(r.children),t.push(r),t):(t.push(r),t)},[])},f=function(i){return function(o){return void 0===o&&(o={}),function(t){visit$2(t,"element",s);};function s(e,s,l){if(l&&"pre"===l.tagName&&"code"===e.tagName){var u=e.data&&e.data.meta?e.data.meta:"";e.properties.className?"boolean"==typeof e.properties.className?e.properties.className=[]:Array.isArray(e.properties.className)||(e.properties.className=[e.properties.className]):e.properties.className=[],e.properties.className.push("code-highlight");var f,h=function(e){for(var t,r=c(e.properties.className);!(t=r()).done;){var n=t.value;if("language-"===n.slice(0,9))return n.slice(9).toLowerCase()}return null}(e);if(h)try{f=i.highlight(toString(e),h),l.properties.className=(l.properties.className||[]).concat("language-"+h);}catch(t){if(!o.ignoreMissing||!/Unknown language/.test(t.message))throw t;f=e;}else f=e;var m,d=(m=1,function e(t){return t.reduce(function(t,r){if("text"===r.type){var n=(r.value.match(/\n/g)||"").length;return r.position={start:{line:m,column:0},end:{line:m+n,column:0}},m+=n,t.push(r),t}if(Object.prototype.hasOwnProperty.call(r,"children")){var i=m;return r.children=e(r.children),t.push(r),r.position={start:{line:i,column:0},end:{line:m,column:0}},t}return t.push(r),t},[])})(f.children);f.children=p(d),f.children.length>0&&(f.position={start:{line:f.children[0].position.start.line,column:0},end:{line:f.children[f.children.length-1].position.end.line,column:0}});for(var g,y,v=function(e){var t=/{([\d,-]+)}/,r=e.split(",").map(function(e){return e.trim()}).join();if(t.test(r)){var i=t.exec(r)[1],o=n(i);return function(e){return o.includes(e+1)}}return function(){return !1}}(u),b=function(e){var t=/*#__PURE__*/a(/showLineNumbers=([0-9]+)/i,{lines:1});if(t.test(e)){var r=t.exec(e);return Number(r.groups.lines)}return 1}(u),N=(""===(g=toString(e).split(/\n/))[g.length-1].trim()&&g.pop(),g.map(function(e){return {type:"element",tagName:"span",properties:{className:["code-line"]},children:[{type:"text",value:e}]}})),w=function(){var e=y.value,n=e[0],i=e[1];(u.toLowerCase().includes("showLineNumbers".toLowerCase())||o.showLineNumbers)&&(i.properties.line=[(n+b).toString()],i.properties.className.push("line-number")),v(n)&&i.properties.className.push("highlight-line"),"diff"===h&&"-"===toString(i).substring(0,1)?i.properties.className.push("deleted"):"diff"===h&&"+"===toString(i).substring(0,1)&&i.properties.className.push("inserted");var a=filter(f,function(e){return e.position.start.line<=n+1&&e.position.end.line>=n+1});i.children=a.children;},x=c(N.entries());!(y=x()).done;)w();e.children=N;}}}};f(refractor);var m=f(refractor);

  var getCodeString = function (data, code) {
      if (data === void 0) { data = []; }
      if (code === void 0) { code = ''; }
      data.forEach(function (node) {
          if (node.type === 'text') {
              code += node.value;
          }
          else if (node.type === 'element' && node.children && Array.isArray(node.children)) {
              code += getCodeString(node.children);
          }
      });
      return code;
  };
  function markdown(markdownStr, options) {
      if (markdownStr === void 0) { markdownStr = ''; }
      if (options === void 0) { options = {}; }
      var processor = unified$1()
          .use(remarkParse)
          .use(remarkGfm)
          .use(options.remarkPlugins || [])
          .use(remarkRehype$1, __assign(__assign({}, options.remarkRehypeOptions), { allowDangerousHtml: true }))
          .use(rehypeVideo)
          .use(rehypeRaw)
          .use(m, { ignoreMissing: true })
          .use(rehypeAttrs$1, { properties: 'attr' })
          .use(rehypeRewrite, {
          rewrite: function (node, index, parent) {
              if (node.type == 'element' && node.tagName === 'code') {
                  var _a = (node.properties || {}).className, className = _a === void 0 ? [] : _a;
                  var found = (Array.isArray(className) ? className : [className]).find(function (str) { return String(str).toLocaleLowerCase().indexOf('language-katex') > -1; });
                  var code = getCodeString(node.children);
                  if (found && node.properties) {
                      if (Array.isArray(node.properties.className)) {
                          console.log(parent);
                          if (parent && parent.type === 'element' && parent.properties) {
                              parent.properties.className = ['language-katex'];
                          }
                          node.properties.className.push('math');
                          node.properties.className.push('math-display');
                          node.children = [
                              {
                                  type: 'text',
                                  value: code,
                              },
                          ];
                      }
                  }
                  if (/^katex/.test(code.toLocaleLowerCase())) {
                      node.properties.className = ['math', 'math-inline'];
                      node.children = [
                          {
                              type: 'text',
                              value: code.replace(/^KaTeX:(\s.)?/i, ''),
                          },
                      ];
                  }
              }
              if (options.rewrite && typeof options.rewrite === 'function') {
                  options.rewrite(node, index, parent);
              }
          },
      })
          .use(rehypeKatex)
          .use(options.rehypePlugins || [])
          .use(rehypeStringify);
      var file = new VFile$1();
      file.value = markdownStr;
      var hastNode = processor.runSync(processor.parse(file), file);
      if (options.hastNode) {
          return hastNode;
      }
      return processor.stringify(hastNode, file);
  }

  exports["default"] = markdown;
  exports.getCodeString = getCodeString;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=markdown.js.map
